!+ HIRS Data Variables for processing within ATOVPP
!
! Name:  pphstore.h
!
! Purpose:
!   'INCLUDE' file for COMMON / pphstore /
!   containing HIRS data for use within ATOVPP code.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0        12/04/96  Original code. (R.J.Renshaw)
!  1.1        12/12/96  Calibration flagword deleted for 1D format vn 1.8
!  1.2        10/03/98  instrument temperature included
!  1.3        20/05/98  extra pre-processing variables included
!  1.4        15/06/98  HSatId added
!  1.5        02/07/98  Hangles,Hsurfcost,Hscatindx,Hcrosby REAL not INTEGER
!  1.6        12/08/05  Add Hyear1, Hday1, Htime1, Hline1 (NCA)
!
!  Size of common block is 3,205 4-byte words / HIRS scanline    + 6*4 bytes
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
      INTEGER       hrs_stvnnb     ! format version number of this file
         PARAMETER (hrs_stvnnb = 131) !  - this code is set up for vn no 1.31
!
      INTEGER       HSatId         ! e.g. 15 for NOAA-15
!
      INTEGER       Hrecdno(Hnlines)               ! record number (1C & 1D)
!            note that Hrecdno() = MDI indicates that the scanline is missing
      INTEGER       Hlineno(Hnlines)               ! scanline number
      INTEGER       Hyear(Hnlines)                 ! scanline year
      INTEGER       Hday(Hnlines)                  ! scanline day of year
      INTEGER       Htime(Hnlines)                 ! scanline time of day (ms)
      INTEGER       Hqualind(Hnlines)              ! quality indicator bit field
                                                   !  (hrs1d_qualind in 1D)
!                 bit 31: do not use scan for product generation
!                 bit 30: time sequence error detected with this scan
!                 bit 29: data gap precedes this scan
!                 bit 28: no HIRS calibration
!                 bit 27: no earth location
!                 bit 26: first good time following a clock update
!                 bit 25: instrument status changed with this scan
!                 bit 24-0: spare  <zero fill>
!
      INTEGER       Hscanqual(Hnlines)             ! scan line quality flags
                                                   !  (hrs1d_scanqual in 1D)
!                 time problem code
!                 (all bits off implies the scan time is as expected)
!                 bit 31-24: spare   <zero fill>
!                 bit 23: time field is bad but can probably be inferred
!                         from the previous good time
!                 bit 22: time field is bad and can't be inferred
!                         from the previous good time
!                 bit 21: this record starts a sequence that is inconsistent
!                         with previous times (i.e., there is a time
!                         discontinuity). This may or may not be associated
!                         with a spacecraft clock update. (see bit 26 above)
!                 bit 20: start of a sequence that apparently repeats scan
!                         times that have been previously accepted.
!                 bits 19-16: spare
!
!                 calibration problem code
!
!                 (all bits set to 0 indicates normal calibration)
!                  bit 15: scan line was not calibrated because of bad time
!                  bit 14: scan line was calibrated using fewer than the
!                          preferred number of scan lines because of
!                          proximity to start or end of data set or
!                          to a data gap.
!                  bit 13: scan line was not calibrated because of bad or
!                          insufficient PRT data
!                  bit 12: scan line was calibrated but with marginal PRT data
!                  bit 11: some uncalibrated channels of this scan.
!                          See channel indicators.
!                  bit 10: uncalibrated due to instrument mode
!                  bits 9 and 8: spare
!
!                  Earth location problem code
!
!                  (all bits set to 0 implies the Earth location was normal)
!                  bit 7: not Earth located because of bad time
!                  bit 6: Earth location questionable because of questionable
!                         time code (see time problem flags above)
!                  bit 5: Earth location questionable. Only marginal agreement
!                         with reasonableness check.
!                  bit 4: Earth location questionable. Fails reasonableness
!                         check
!                  bit 3-0: spare
!
      INTEGER       Hinstrtemp(Hnlines)            ! baseline temperature
      INTEGER       Hinstrtempa1(Hnlines)          ! AMSU-A1 temperature
      INTEGER       Hinstrtempa2(Hnlines)          ! AMSU-A2 temperature
      INTEGER       Hinstrtempb(Hnlines)           ! AMSU-B temperature
      REAL          Hlats(Hnfovs,Hnlines)          ! latitudes (deg, N +ve)
      REAL          Hlons(Hnfovs,Hnlines)          ! longitudes (deg, E +ve)
      INTEGER       Helev(Hnfovs,Hnlines)          ! surface height (m)
      INTEGER       Hsurftype(Hnfovs,Hnlines)!surface type(0=sea,1=mixed,2=land)
      REAL          Hangles(4,Hnfovs,Hnlines)      !scan angles for Hnfovs
!                                          (1,*,*) local zenith angle (deg)
!                                          (2,*,*) local azimuth angle (deg)
!                                          (3,*,*) solar zenith angle (deg)
!                                          (4,*,*) solar azimuth angle (deg)
      INTEGER       Haltitude(Hnlines)             ! satellite altitude (km*10)
                                                   ! above reference ellipsoid
      REAL          Hbtemps(Hnchan,Hnfovs,Hnlines) ! HIRS brightness temps (K)
      REAL          Hmapbt(Anchan+Bnchan,Hnfovs,Hnlines) ! AMSU-A&B BTs (K)
      INTEGER       Hqc(Hnfovs,Hnlines)            ! ATOVPP QC word for HIRS
!                                                (becomes hrs1d_hirsqual in 1D)
!                   (all bits off implies acceptable data)
!                   bit 31: spare   <zero fill>
!                   bit 30: set if secondary calibration used
!                   bit 29-22: spare   <zero fill>
!                   bit 21: HIRS cloud test (TBD)
!                   bit 20-1: bit n set to 1 if brightness temperature in
!                             HIRS channel n is physically unreasonable or has
!                             not been calculated due to calibration  
!                             problems.
!                   bit 0:    bad or missing data (in all channels)

      INTEGER       Hmapqc(Hnfovs,Hnlines)         ! ATOVPP QC word for AMSU
                                                   !  (word 1 in hrs1d_prepro)
!                            (all bits off implies acceptable data)
!                  bit 31: set if AMSU-A surface types not all the same
!                  bit 30: set if AMSU-A used secondary calibration
!                  bit 29: set if AMSU-B used secondary calibration
!                  bit 28: set if AMSU-B data missing
!                  bit 27: flag for cloud cost set for any mapped AMSU-A
!                  bit 26: scattering flag set for any mapped AMSU-A
!                          (only set over the sea)
!                  bit 25: logistic precipitation probability test
!                          calculated from AMSU-A data mapped to HIRS grid
!                  bit 24: Grody light rainfall test calculated on HIRS grid
!                  bit 23: mismatch between AMSU-A/B 89GHz values (any AMSU-A)
!                  bit 22: mismatch between surface type from topography
!                          dataset and from pre-processing (any AMSU-A)
!                  bit 21-4: spare   <zero fill>
!                  bit 3: set when AVHRR channel 3 is albedo not brightness temp
!                  bit 2: flag for cloud cost (recalculated on HIRS grid)
!                  bit 1: flag for scattering index (recalculated on HIRS grid)
!                  bit 0: set if AMSU-A & B data missing
!
!       or alternatively, for TOVS :
!                           taken from nearest MSU FOV:
!                  bit 31: spare   <zero fill>
!                  bit 30: set if secondary calibration used for nearest MSU
!                  bit 29-4: spare   <zero fill>
!                  bit 3: set if MSU brightness temperatures are suspect
!                      due to data having been used in the mapping
!                      from a surface type different to that of the HIRS fov
!                  bit 2: set if MSU brightness temperatures derived from
!                      secondary calibration coeff's were mapped
!                  bit 1: set if less than 4 MSU fov's used in mapping
!                  bit 0: set if MSU data missing due to insufficient
!                      good data available for the mapping.
!
      INTEGER       Hmapsurf(Hnfovs,Hnlines)       ! estimated surface type
                                                   !  (word 2 in hrs1d_prepro)
      REAL          Hsurfcost(Hnfovs,Hnlines)      ! AMSU-A max prob cloud test
                                                   !  (word 3 in hrs1d_prepro)
      REAL          Hscatindx(Hnfovs,Hnlines)      ! AMSU-A scattering index
                                                   !  (word 4 in hrs1d_prepro)
      REAL          Hcrosby(Hnfovs,Hnlines)        ! AMSU-A logistic precip test
                                                   !  (word 5 in hrs1d_prepro)
      REAL          Hsatht                         ! nominal sat height
      REAL          Hsatperiod                     ! nominal orbit period (mins)
      INTEGER       Hhdrcnt              !count of header records in 1C/1D file
      INTEGER       Hscnlin              !count of scanlines in 1C/1D file
      INTEGER       Hinstruments                   ! instruments present
!                   (bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR)
      INTEGER       Hyear1        ! year of first "good" HIRS scanline
      INTEGER       Hday1         ! day
      INTEGER       Htime1        ! time
      INTEGER       Hline1        ! scanline no.
!

      COMMON / pphstore / HSatId, Hrecdno, Hlineno, Hyear, Hday, Htime,
     & Hqualind, Hscanqual, Hinstrtemp, Hinstrtempa1, Hinstrtempa2,
     & Hinstrtempb, Hlats, Hlons, Helev, Hsurftype, Hangles, Haltitude,
     & Hbtemps, Hmapbt, Hqc, Hmapqc, Hmapsurf, Hsurfcost, Hscatindx,
     & Hcrosby, Hsatht, Hsatperiod, Hhdrcnt, Hscnlin, Hinstruments,
     & Hyear1, Hday1, Htime1, Hline1
