!       timang.h
!
! constants for the scanning geometry and timing of NOAA instruments
!
!
!       scsttim  scanning step time  (jdays)
!       scdtim   scanning delta time (jdays)
!       scstang  scanning step angle (rad)
!       scmpix   pixel number at nadir
!       steptl   step time between two lines (sec)
!
!       the medium pixel is the real pixel value at nadir(number of steps +1)/2.
!       scanning step angle > 0 <==> scanning from right to left according
!                                    to the satellite motion
!
!
! NOAA16 specific values:
!========================
! Version modified on January 12th, 2001   P. Brunel
! AVHRR scanning step angle is modified according to the value
! of maximum scanning angle given by E. Harrod (NOAA/NESDIS, 30 November 2000
! personal communication): 55.25deg   step is 0.0539814deg
!
! HIRS scanning is shifted of around one pixel. Estimations made by mapping
! AVHRR into HIRS field of view show that the best shift is 2 degrees which
! correspond to 1.11 pixel.
!
! NOAA17 specific values:
!========================
! Version modified on August 12th, 2002   P. Brunel
! HIRS scanning is shifted of around one pixel. Estimations made by mapping
! AVHRR into HIRS field of view show that the best shift is 1.7 degree
!
! Even if the abosulte mounting mistake is one HIRS pixel, we introduce
! the value of the alignment difference of AVHRR and HIRS, because the
! calculation of the attitude is based on AVHRR measurements.
! 
!-
!
! 1 AVHRR - LAC
! line is instantaneous, 6 lines/sec, 2048 pixels/line
! digitizing frequency 39936 pix/s
! delta at nadir 34 ms
        scsttim(1) = 0.000d0    
        scdtim(1) = 0.034d0 /86400.d0
        scstang(1) = 2 * pi * 6 / 39936.d0
        scmpix(1)  = 1024.5d0
        steptl(1) = 1.d0/6.d0
        ! special value of step angle for N16
        if(cp_cus(sp_satnam(1:6),'noaa16')) then
           scstang(1)=0.053955078d0 * pi/180.d0
        endif
!
! 2 GAC
        scsttim(2) = 0.000d0    
        scdtim(2) = 0.0d0
        scstang(2) = 0.d0
        scmpix(2)  = 0.d0
        steptl(2) = 0.d0
!
! 3 AVHRR - HRPT
! line is instantaneous, 6 lines/sec, 2048 pixels/line
! digitizing frequency 39936 pix/s
! delta at nadir 34 ms
        scsttim(3) = 0.000d0    
        scdtim(3) = 0.034d0 /86400.d0
        scstang(3) = 2 * pi * 6 / 39936.d0
        scmpix(3)  = 1024.5d0
        steptl(3) = 1.d0/6.d0
        ! special value of step angle for N16
        if(cp_cus(sp_satnam(1:6),'noaa16')) then
           scstang(3)=0.053955078d0 * pi/180.d0
        endif
!
! 4 TIP
        scsttim(4) = 0.000d0    
        scdtim(4) = 0.0d0
        scstang(4) = 0.d0
        scmpix(4)  = 0.d0
        steptl(4) = 0.d0
!
! 5 HIRS
! 6.4 s between lines, 56 pixels/line
! 0.1 s between pixels, -1.8 deg step angle
! delta 0.05 s
        scsttim(5) = 0.100d0 / 86400d0
        scdtim(5) = 0.050d0 / 86400d0
        scstang(5) = -1.800d0 * pi / 180.d0
        scmpix(5)  = 28.5d0
        steptl(5) = 6.4d0
        ! shift of 2 degrees for N16
        if(cp_cus(sp_satnam(1:6),'noaa16')) then
           scmpix(5)= 28.5d0 -2.0d0/1.8d0
        endif
        ! shift of 1.7 degrees for N17
        if(cp_cus(sp_satnam(1:6),'noaa17')) then
           scmpix(5)= 28.5d0 -1.7d0/1.8d0
        endif
!
! 6 MSU
! 25.6 s between lines, 11 pixels/line
! 1.81 s between pixels, -38 pixels for 360 deg
! delta 0.90 s
        scsttim(6) = 1.810d0 / 86400d0
        scdtim(6) = 0.900d0 / 86400d0
        scstang(6) = -2d0 * pi /38d0
        scmpix(6)  = 6.d0
        steptl(6) = 25.6d0
!       
! 7 SSU
! 32.0 s between lines, 9 pixels/line
! 4.0 s between pixels, -10 deg step angle
! delta 2.0 s
        scsttim(7) = 4.00d0 / 86400d0
        scdtim(7) = 2.00d0 / 86400d0
        scstang(7) = -10.00d0 * pi / 180.d0
        scmpix(7)  = 4.5d0
        steptl(7) = 32.0d0
!
! 8 DCS
        scsttim(8) = 0.000d0    
        scdtim(8) = 0.0d0
        scstang(8) = 0.d0
        scmpix(8)  = 0.d0
        steptl(8) = 0.d0
!
! 9 SEM
        scsttim(9) = 0.000d0    
        scdtim(9) = 0.0d0
        scstang(9) = 0.d0
        scmpix(9)  = 0.d0
        steptl(9) = 0.d0
!
! 10 AMSU-A
! 8.0 s between lines, 30 pixels/line
! 0.2025 s between pixels, -10/3 deg step angle
! delta 0.02 s
        scsttim(10) = 0.2025d0 / 86400d0 
        scdtim(10) = 0.020d0  / 86400d0
        scstang(10) = (-10.d0 /3.d0 )* pi / 180.d0
        scmpix(10)  = 15.5d0
        steptl(10) = 8.d0 
!
! 11 AMSU-B
! 8/3 s between lines, 90 pixels/line
! 0.019 s between pixels, -1.1 deg step angle
! delta 0.001 s
        scsttim(11) = 0.019d0  / 86400d0
        scdtim(11) = 0.001d0  / 86400d0
        scstang(11) = -1.1d0 * pi / 180.d0
        scmpix(11)  = 45.5d0
        steptl(11) = (8.d0/3.d0) 
!
! 12 MHS
! 8/3 s between lines, 90 pixels/line
! 0.019 s between pixels, -1.11111 deg step angle
! delta 0.001 s
        scsttim(12) = 0.019d0  / 86400d0
        scdtim(12) = 0.001d0  / 86400d0
        scstang(12) = -1.11111d0 * pi / 180.d0
        scmpix(12)  = 45.5d0
        steptl(12) = (8.d0/3.d0) 

