C LIBS = hirscl_algoV4 hirscl navnoaa tbus satid navtool f7nl1b f7ml f7tp f7cp f7gp
      Program hcalcb1_algoV4

!------------------------------------------------------------------------------
!NAME $Id: hcalcb1_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE 
!   Calculates the B0/B1 and average slope for HIRS instrument
!   The initial data are stored in the historical file 
!         (filename hirs_historic.txt)
!   results are written in the file nammed hirs_b1aslope.txt 
!
!DESCRIPTION   
! In standard input:
! Print option (IPRINT)
! Reference date (YYYY, MM, DD)
! Reference hour (HH, MN)
! Number of hours (NBHOUR)
!
! In output file
! Reference date (YYYY, MM, DD)
! Reference hour (HH, MN)
! Number of hours (NBHOUR)
! one line per channel containing:
!   avgslope, b1, b1n, b1d
!          average slope all conditions 
!          b1     all conditions
!          b1     night conditions
!          b1     daylight conditions
!
!
! Reference
!   HIRS calibration Algorithm Version 4.0 (Nov 3, 2003)
!   C. Cao NESDIS
!
!HISTORY (CHANGES) 
!  (version    date         comment           <author>)
!    1.0     January 05    Creation          P. Brunel (MF)
!    1.1     14/06/05      Add a format to write 
!        "c, avgslope(ic), stdevavgslope(ic), b1(ic), b1n(ic), b1d(ic)"
!                                             T.Labrot and P.Brunel (MF)  
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!ARGUMENTS 
!     inputs :
!     outputs :
!
!UPDATED ELEMENTS OF INCLUDES 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! 
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

       Implicit None
       
! includes/commons
#include "hrsparam_algoV4.h"

! Local variables read into the standard input

      Integer iprint        ! Print option
      Integer yyyy, mm, dd  ! reference date
      Integer hh, mn        ! reference hour
      Integer nbhour        ! number of considered hours preceeding data date

! Others local variables

      Integer hrs_logu      ! Logical unit of log file
      Integer hrs_histou    ! Logical unit of historical file
      Integer hrs_b1aslopu  ! Logical unit of output file      
      Integer ig(7)         ! reference julian instant
      Double Precision obs_date ! Julian day of current observation

      Real b0(hrs_mxirchn)  ! BO values for all conditions
      Real b1(hrs_mxirchn)  ! B1 values for all conditions
      Real b0n(hrs_mxirchn) ! BO values for night conditions
      Real b1n(hrs_mxirchn) ! B1 values for night conditions
      Real b0d(hrs_mxirchn) ! B0 values for day light conditions
      Real b1d(hrs_mxirchn) ! B1 values for day light conditions
      Real avgslope(hrs_mxirchn) ! average slope
      Real stdevavgslope(hrs_mxirchn) ! standard deviation of the average slope
      
      Integer ic            ! channel loop indice
      Integer ios           ! I/O return code
      
!--END OF DECLARATIONS----------------------------------------------------------
      
      hrs_logu=10
      hrs_histou=11
      hrs_b1aslopu=12
      
      Read(*,*,iostat=ios) iprint, yyyy, mm, dd, 
     &                     hh, mn, nbhour 

      If( ios .Ne. 0 ) Then
         Call ml_wti('error','reading input ios ',ios)
         Stop
      End If

      Open (hrs_histou, file='hirs_historic.txt',
     &       status='old', form='formatted',
     &       access='sequential', iostat=ios)
      If( ios .Ne. 0 ) Then
         Call ml_wti('error',
     &    'opening hirs calibration historical file ios ',ios)
         Stop
      End If
      
      Open (hrs_b1aslopu, file='hirs_b1aslope.txt',
     &       status='new', form='formatted',
     &       iostat=ios)
      If( ios .Ne. 0 ) Then
         Call ml_wti('error',
     &    'opening output file ios ',ios)
         Stop
      End If
      
      Open (hrs_logu, file='hcalcb1.log',
     &       status='new', form='formatted',
     &       iostat=ios)
      If( ios .Ne. 0 ) Then
         Call ml_wti('error',
     &    'opening hcalcb1 log file ios ',ios)
         Stop
      End If

      ig(1) = yyyy
      ig(2) = mm    
      ig(3) = dd
      ig(4) = hh
      ig(5) = mn
      ig(6) = 0
      ig(7) = 0

      Call xigj(ig, obs_date)
      
      Call  h_calcb1_algoV4(
     &     iprint,
     &     hrs_histou,
     &     hrs_logu, 
     &     nbhour,
     &     obs_date,
     &     b0n,
     &     b1n,
     &     b0d,
     &     b1d,
     &     b0,
     &     b1,
     &     avgslope,
     &     stdevavgslope)
     
      Write(hrs_b1aslopu,
     & '(1x,i4,1x,i2.2,1x,i2.2)',iostat=ios) yyyy, mm, dd
      If( ios .Ne. 0 ) Then
         Call ml_wti('error','writing reference date ios ',ios)
         Stop
      End If

      Write(hrs_b1aslopu,
     & '(1x,i2.2,1x,i2.2)',iostat=ios) hh, mn
      If( ios .Ne. 0 ) Then
         Call ml_wti('error','writing reference hour ios ',ios)
         Stop
      End If

      Write(hrs_b1aslopu,
     &  '(1x,i2.2)',iostat=ios) nbhour
      If( ios .Ne. 0 ) Then
         Call ml_wti('error','writing number of hours ios ',ios)
         Stop
      End If

      Do ic = 1, hrs_mxirchn
         Write(hrs_b1aslopu,'(1x,i2,1x,1p,5e15.6)') 
     &   ic, avgslope(ic), stdevavgslope(ic), b1(ic), b1n(ic), b1d(ic)
         If( ios .Ne. 0 ) Then
            Call ml_wti('error','writing b1  ios ',ios)
            Stop
         End If
      End Do

      Close (hrs_histou, iostat=ios)
      If( ios .Ne. 0 ) Then
         Call ml_wti('error',
     &    'closing hirs calibration historical file ios ',ios)
         Stop
      End If
      
      Close (hrs_b1aslopu, iostat=ios)
      If( ios .Ne. 0 ) Then
         Call ml_wti('error',
     &    'closing output file ios ',ios)
         Stop
      End If
      
      Close (hrs_logu, iostat=ios)
      If( ios .Ne. 0 ) Then
         Call ml_wti('error',
     &    'closing hcalcb1 log file ios ',ios)
         Stop
      End If

      Stop
      End
