#!/bin/ksh
#
# Korn Shell to manage the HIST file (=hirs_historic.txt) 
# 
# PURPOSE
#    When the ${HIST} file has a number of lines superior to ${HIST_SIZE_HIGH},
#    it is copied to ${HIST}.0 file.
#    If ${HIST}.0 file alreday exists, it is moved to ${HIST}.1 .
#    ${HIST}.${HIST_NMAX} can be stored.
#    The final part of ${HIST} is remained in ${HIST}. The final part is 
#    defined from the first line with the chain 'HIRS CALIB INFO' after 
#    having remained the last ${HIST_SIZE_LOW} lines of ${HIST} 
#    to the last line of ${HIST}.
#
#    See usage
#
#    No default values.
#     
#HISTORY (CHANGES) 
#    (version     date         comment           <author>)
#       1.0     January 05     creation      P.Brunel, T. Labrot (MF)
#
#CURRENT CODE OWNER: SAF NWP
#
#COPYRIGHT
#    This software was developed within the context of
#    the EUMETSAT Satellite Application Facility on
#    Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 25 November 1998, between
#    EUMETSAT and the Met Office, UK, by one or more partners
#    within the NWP SAF. The partners in the NWP SAF are
#    the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2005, EUMETSAT, All Rights Reserved.
#
#--------------------------------------------------------------------------                      


# usage
#-------- 
usage(){
	echo
	echo ' hirs_historic_file_manage: manage the historic file (HIRS calibration algorithm 4)'
	echo
	echo 'usage : hirs_historic_file_manage -m number -r number -n number file';
	echo
	echo ' -m Maximum size allowed for the historical file in number of lines '
	echo ' -r Maximum size of the new file in number of lines'
	echo ' -n Maximum number of historical files stored'
	echo ' file Historic file to manage'
	echo
	}

#  get the arguments
#-------------------
	while getopts m:r:n: c
	do
               case $c in
               m)	HIST_SIZE_HIGH=$OPTARG;;
               r)	HIST_SIZE_LOW=$OPTARG;;
               n)	HIST_NMAX=$OPTARG;;               
               \?)      usage
                        exit 2;;
               esac
	done
	shift `expr $OPTIND - 1`

	if [ "$#" -le 0 ] ; then
	  echo 'the name of the file is missing'
	  usage
	  exit 1
	fi

HIST=$1
echo HIST,${HIST}

if [ -z "${HIST_SIZE_HIGH}" ]   ; then usage ; exit 1 ;fi
if [ -z "${HIST_SIZE_LOW}" ] ; then usage ; exit 1 ;fi
if [ -z "${HIST_NMAX}" ]     ; then usage ; exit 1 ;fi


# Process
#--------
if [ -s ${HIST} ] ; then
  NBLIN=` cat ${HIST} | wc -l `
  if [ ${NBLIN} -gt ${HIST_SIZE_HIGH}  ] ; then
    echo "Start roll of HIRS historical file  ${HIST} "
    echo "File contains ${NBLIN} lines"
    \rm -f  ${HIST}.tmp
    ed -s ${HIST} <<EOF
-${HIST_SIZE_LOW}
/HIRS CALIB INFO
.,\$w  ${HIST}.tmp
q
EOF

    i=${HIST_NMAX}
    while [ ${i} -gt 0 ]; do
      j=`expr ${i} - 1`
      test -f ${HIST}.${j} && mv ${HIST}.${j} ${HIST}.${i}
      i=${j}
    done
    mv ${HIST} ${HIST}.0

    mv ${HIST}.tmp ${HIST}
    ls -l ${HIST}*
    echo "File is renamed as  ${HIST}.0"
    echo "New HIRS historical file now contains around ${HIST_SIZE_LOW} lines"
    echo "End of roll of HIRS historical file  ${HIST} "
  
  else
  
    echo "Number of lines of the HIST file is inferior to HIST_SIZE_HIGH" 
  
  fi
  
else

  echo "HIST file does not exist "
       
fi

exit

