C LIBS = hirscl_algoV4 hirscl navnoaa tbus satid navtool f7nl1b f7ml f7tp f7cp f7gp auxaapp
      PROGRAM hirscl_algoV4

!------------------------------------------------------------------------------
!NAME $Id: hirscl_algoV4.F 55 2006-09-21 08:35:34Z frna $
!
!PURPOSE 
!     This main program pilots the navigation, the localisation and the
!     calibration of the HIRS/3 or 4.
!
!     This module is very similar to the module hirscl.
!     But it doesn't work for data of pre-NOAAK satellies.
!     It was created to manage the HIRS calibration algorithm version 4.0
!
!     The algorithm version 4.0 is described in the paper "HIRS calibration 
!     Algorithm Version 4.0" (Nov 3, 2003) C. Cao NESDIS . It is also 
!     useful to refer to the NOAA KLM User's guide Section 7.2 
!
!     This algoritm was developped in AAPP to have a very near 
!     calibration process of the NOAA one in the aim to have homogeneous
!     data between local data and global data.
!     Nevertheless the main difference between the AAPP algorithm and the
!     NOAA algorithm is for the average data (slope and B1, coefficient of 
!     correlation between the telescope temperature and calibration 
!     intercept). NOAA uses 24 hours averages. With local data it is not
!     available. So the average slope and B1 are computed from past data.
!     The past data are taken from the reference date - x hours to the 
!     reference date. By default, the reference date is the date of 
!     the orbit. X is to define in ATOVS_ENV.
!     The second main difference is that the AAPP algorithm does not
!     control the moon contamination on the space views. It will be 
!     done in the next version.
!     The choise of flagging some bits in the level 1b could also
!     be different. This will improve in the next versions. 
!
!     Note that the new algorithm has been validated using limited 
!     amount of NOAA KLM/HIRS data. 
!
!     Note that with only one good calibration cycle, Earth view 
!     scan lines are not calibrated.   
!
!DESCRIPTION 
!     For the HIRS instrument, a super-swath is defined as 40 scan-lines of 
!     HIRS data that start with a calibration cycle, followed by 38 Earth
!     view scan lines. 
!     A calibration cycle is made of 1 space view line, plus 1 IWT
!     (Internal Black Target) or BB (Black Body) view line.
!     From data of the 2 lines, calibration coefficients are computed.         
!
!         
!HISTORY (CHANGES) 
!    (version   date         comment           <author>)
!    1.0     January 05     Creation          T. Labrot (MF)
!    1.1     14/06/05       Move the place of the call to the routine
!                           hgetl1belement_algoV4 after the localisation 
!                           task to have values in localisation variables.
!                                           T.Labrot and P.Brunel (MF) 
!    1.2     16/02/06       Use STOP instead of exit(0) - more portable. NCA
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!                      
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     

!includes/commons
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hb1averageslope_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
#include "hrs1bhd.h"

! variables
      LOGICAL partialsw(hrs_mxirchn, hrs_mxcalib+1) 
                     !true if partial superswath for cycle for the slopes
                       
      INTEGER noanum !order satellite number 
      INTEGER nselectedslopes(hrs_mxirchn,hrs_mxcalib+1)  
                     !number of selected slopes (0 to 3) for each cycle
      INTEGER selectedslopes(3,hrs_mxirchn,hrs_mxcalib+1) 
                     !cycle number (1to3) selected slopes
                        
      REAL b11(hrs_mxchn,hrs_mxlin)
                     !correlation coefficient between the telescope temperature
                     !and calibration intercept used in the equations
      REAL sttp(hrs_mxlin) 
                     !Second Telescope Temperature Prime for all lines (K)                                      
      DOUBLE PRECISION iwttmp(hrs_mxcalib)
                     !IWT temperatures (one for each calib cycle)
      DOUBLE PRECISION iwtrad(hrs_mxirchn,hrs_mxcalib)
                     !IR channel radiances of the IWT
      DOUBLE PRECISION calibcoef(hrs_mxchn,3,hrs_mxlin)
                     !third auto coefs,intercepts,slopes, array 
                     !for each scan line, for each channel
      DOUBLE PRECISION b1mn(hrs_mxchn),b1std(hrs_mxchn)
                     !mean of the b1 coefficients, standard deviation of 
                     !the b1 coefficients
   
!--END OF DECLARATIONS----------------------------------------------------------

!------------------------------
! inits and opens global files 
!------------------------------

       CALL hclin_algoV4
       CALL hclsetu_algoV4

!----------------------
! Earth location part
!----------------------

       IF (hrs_lflg.EQ.1) THEN
         CALL ml_wt('info','hirs navigation start')

! Earth locate 1B scans
         CALL h_loc

! Updates navigation header variables
           CALL hd1bnav(hrs_h_modelid,hrs_h_nadloctol,hrs_h_locbit,
     & hrs_h_yawerr,
     & hrs_h_rollerr,hrs_h_pitcherr,hrs_h_epoyr,hrs_h_epody,
     & hrs_h_epotime,hrs_h_smaxis,hrs_h_eccen,hrs_h_incli,
     & hrs_h_argper,hrs_h_rascnod,hrs_h_manom,hrs_h_xpos,
     & hrs_h_ypos,hrs_h_zpos,hrs_h_xvel,hrs_h_yvel,hrs_h_zvel,
     & hrs_h_earthsun)
         CALL ml_wt('info','navigation end')

       ENDIF
       
! if no calibration, write and close the level 1b file
       IF (hrs_cflg.EQ.0) THEN       
        CALL hl1bwrt_algoV4
        CALL hclexit_algoV4
        CALL ml_wt('info','hirs navigation normal end')
        STOP
      ENDIF 
       
!----------------------
! calibration part
!----------------------

       IF (hrs_cflg.EQ.1) THEN
        
         CALL ml_wt('info','HIRS calibration start')
         
! Gets the elements of the HIRS level 1b commons that are useful 
! for the calibration task         
         
         CALL hgetl1belement_algoV4
! Inits and opens calibration specific files

         CALL h_cinit_algoV4(noanum)
         
! Reads the 'b1/average slope' file to get B1 and the historic average 
! slope that will be used in this process 
     
         CALL h_readb1slope_algoV4
          
! Checks the instrument status to define which scan lines are usable         

         CALL h_instrtest_algoV4

! Looks for the calibration lines and cycles in the orbit

         CALL h_scanpos_algoV4(noanum)

! If enough good calibration cycles, the process continues. 

         IF (goodcalib.GE.1) THEN 

!   Computes the IWT temperature for each IWT scan line

            CALL h_iwttmp_algoV4( 
     &     iwttmp)             

!   Computes the radiances of the IWT, for each channel and 
!   for each IWT scan line

            CALL h_iwtrad_algoV4(
     &     iwttmp,
     &     iwtrad)

!   Computes the output counts mean for each channel and
!   for each Space line and each IWT line
            
            CALL h_cntmn_algoV4

!   Computes the BB coefficients (or raw coefficients)
!   for each channel and for each calib cycle 
            
            CALL h_BBinterslop_algoV4(
     &     iwtrad)
            
!   Controls the quality of the BBslopes for each calib cycle

            CALL h_BBslopcontrol_algoV4           
            
!   Determines the calib cycles which will be involved in
!   the calculation of average slope for each superswath or
!   partial superswath 

            CALL h_partial_superswath_algoV4(
     &     partialsw,
     &     nselectedslopes,
     &     selectedslopes)
 
!   Computes the calibration slopes for each channel and for each 
!   Earth view scan line.            
 
            CALL  h_slope_algoV4 (
     &     partialsw,
     &     nselectedslopes,
     &     selectedslopes, 
     &     calibcoef) 
 
!   Computes the Second Telescope Temperature for all lines

           CALL h_sttemp_algoV4(
     &      sttp)
                        
!   Computes the intercept for each channel and for each Earth 
!   view line            
  
            CALL h_intercept_algoV4(
     &      sttp,
     &      iwtrad,
     &      calibcoef,
     &      b11) 
 
!   Stores calibration information of all calibration
!   cycles in a historical ASCII file             
             
            CALL   h_write_histo_algoV4(
     &     noanum)
              
!   Computes the means and the standard deviations 
!   of the slopes on all the lines
!   of the b1 coefficients of all the lines.
!   b1 is the correlation coefficient between the telescope 
!   temperature and calibration intercept for a given channel

            CALL h_gtb1mean_algoV4(
     &     b11,
     &     b1mn,
     &     b1std)
     
!   Finishes to compute all the outputs of the HIRS 
!   calibration task before updating the HIRS lev1b file

            CALL h_uphrs1bcommon1_algoV4(
     &     b1mn,
     &     b1std, 
     &     calibcoef)
               
           
!   Writes into the level 1b file and exits           
           CALL hl1bwrt_algoV4
           CALL hclexit_algoV4
           CALL ml_wt('info','HIRS calibration NORMAL END')
           STOP
           
         ELSE 

! If no enough calibration cycle, only updates the commons l1b

            CALL h_uphrs1bcommon2_algoV4
            
! Writes into the level 1b file and exits error          
            CALL hl1bwrt_algoV4
            CALL hclexit_algoV4
            CALL ml_wt('error','No enough calib cycles found')
            CALL ml_wt('error','No calibration possible')
            CALL ml_wt('error','hirscl_algoV4 aborted')
            CALL aappexit(10)           
   
         ENDIF

       ENDIF
       
       STOP
       END


