      subroutine ama_cal(iscan,ichan,calok,targ_temp,inst_temp,
     &    itavg,spavg,ispvw,pllo)
!+
!NOM/NAME $Id: ama_cal.F 191 2010-01-27 13:42:50Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module computes the AMSU-A calibration coefficients given
!     the mean space view and internal target counts and the internal target
!     temperature. it computes the gain (in units of counts/mW/sqm.ster.cm-1).
!     It then computes the primary calibration coeffs a0, a1 & a2 in units
!     of mW/(sqm.ster.cm-1), mW/(sqm.ster.cm-1)/count &
!     mW/(sqm.ster.cm-1)/count^2 and then scaled by 1E9, 1E13 & 1E19 to
!     allow them to be represented as integers in the 1B file.
!     The parameters determined from the thermal vac data supplied
!     in calibration parameter file: uo, u1 and u2 are used to compute
!     these coefficients as defined in the noaa/ukmo calibration spec.
!     see UK Met Office RSI working paper 66 (1994) for overall philosophy.
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date      comment           <author>)
! 0.0                 AMSU-B          R. Saunders
! 1.0      June 96                    N. Atkinson
! 1.1      March 03   f90 test        T.Labrot
!                     I.OR.J not correct and modified by IOR(I,J)
! 1.2      March 07   Avoid divide by zero error if r21 and r32 are zero,
!                     i.e. if an instrument is powered off. NCA.
! 1.3      Jan 2010   Set c2=0 if the quadratic coefficient overflows. NCA.
!
!ARGUMENTS 
!     iscan - in scan line number
!     ichan - in AMSU-A channel number
!     calok(15,mxlin) - in/out status of scan line for calibration
!     targ_temp(3,mxlin) - in mean target temp for scan line from PRT data
!     inst_temp(3,mxlin) - in mean instrument temp for scan from PRT data
!     itavg(15) - in mean internal target counts for each channel
!     spavg(15) - in mean space view counts for each channel
!     ispvw(3,mxlin) - in space viewing direction
!     pllo(mxlin) - in PLLO for chan 9-14
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!            plank
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################
!
!--DECLARATIONS----------------------------------------------------------------
      implicit none
!
!
! includes/commons
#include "ama_cinit.h"
#include "amaclin.h"
#include "ama1b.h"
#include "amascn.h"
!
! local variables
      integer*4 iscan,ichan,ichan1,inst,inst1,i,ispvw(3,mxlin)
      integer*4 pllo(mxlin),instarr(15),instarr1(15),chanoff1(15)
      data instarr/3,3,2,2,2,1,1,2,1,1,1,1,1,1,1/  !1=A1-1,2=A1-2,3=A2
! instarr1 is used to index reftemp(4,3):
      data instarr1/3,3,2,2,2,1,1,2,4,4,4,4,4,4,1/ !1=A1-1,2=A1-2,3=A2,4=A1PLLO2
! chanoff1 is used to index intcorr,intcntmin,intcntmax,spcntmin,
! spcntmax and nonlin:
      data chanoff1/0,0,0,0,0,0,0,0,7,7,7,7,7,7,0/ !Chan offset for PLLO2
      real itavg(15),spavg(15)       ! internal target & space view counts
      real targ_temp(3,mxlin),inst_temp(3,mxlin) 
                                            ! int target temp & inst temp
      real c2,cc,cw,cwg,recg,sdc,recg2
      logical*1 calok(15,mxlin)   ! Cal status
      real btemp,tsp,radsp,radbb,gain,rdiff,r32,r21,n32,n21,i32,i21
      real u021,u121,u221,u032,u132,u232,delta_t1,delta_t2,intc,tmp
! 
      gain = 0.0   !Initialize
      inst = instarr(ichan)  !Instrument 1,2 or 3 for A1-1,A1-2,A2
      if (pllo(iscan).eq.1) then      !For parameters sensitive to PLLO2
        inst1=instarr(ichan)  !Primary PLLO
        ichan1 = ichan
      else
        inst1=instarr1(ichan)  !Redundant PLLO
        ichan1 = ichan + chanoff1(ichan)
      endif
! Do not calibrate if target or instrument temperature is bad
      if (targ_temp(inst,iscan).eq.0.0.or.inst_temp(inst,iscan).eq.0.0) 
     &  calok(ichan,iscan)=.false.

! apply antenna correction to temperature of space
      tsp = tspace + spcorr(ichan,ispvw(inst,iscan)+1)  
! Set nonlinearity correction and warm target bias correction variables
      r21 = reftemp(inst1,2)-reftemp(inst1,1)
      r32 = reftemp(inst1,3)-reftemp(inst1,2)
      if (r21 .ne. 0.0) then 
        n21 = (nonlin(ichan1,2)-nonlin(ichan1,1))/r21 
        i21 = (intcorr(ichan1,2)-intcorr(ichan1,1))/r21
        u021 = (u0(ichan,2)-u0(ichan,1))/r21
        u121 = (u1(ichan,2)-u1(ichan,1))/r21
        u221 = (u2(ichan,2)-u2(ichan,1))/r21
      else
        n21 = 0.0
        i21 = 0.0
        u021 = 0.0
        u121 = 0.0
        u221 = 0.0
      endif

      if (r32 .ne. 0.0) then
        n32 = (nonlin(ichan1,3)-nonlin(ichan1,2))/r32
        i32 = (intcorr(ichan1,3)-intcorr(ichan1,2))/r32
        u032 = (u0(ichan,3)-u0(ichan,2))/r32
        u132 = (u1(ichan,3)-u1(ichan,2))/r32
        u232 = (u2(ichan,3)-u2(ichan,2))/r32
      else
        n32 = 0.0
        i32 = 0.0
        u032 = 0.0
        u132 = 0.0
        u232 = 0.0
      endif
      
      delta_t1 = inst_temp(inst,iscan)-reftemp(inst1,1)
      delta_t2 = inst_temp(inst,iscan)-reftemp(inst1,2)
!
C.. Correct target temp to reduce warm biases for channel (ignore inst temp)
      if(calok(ichan,iscan))then
       if (delta_t2 .lt. 0)then
        intc = intcorr(ichan1,1) + delta_t1*i21
       else
        intc = intcorr(ichan1,2) + delta_t2*i32
       endif
       btemp = targ_temp(inst,iscan) + intc
C.. convert physical temp to effective temp for AMSU-A channel
       btemp = A(ichan)*btemp + B(ichan)
!.. convert  temps to radiance in mW/cm-1/ster/sqm
       call plank(freq(ichan),btemp,radbb)
       call plank(freq(ichan),tsp,radsp)
!...Compute gain
       rdiff = radbb-radsp
       if(rdiff.gt.0.00002)then
        gain = (itavg(ichan)-spavg(ichan))/rdiff
       else
        gain = 0.0
       endif
!...compute non-linear coefficient for instrument temp by interpolation
       if (delta_t2 .lt. 0)then
        c2 = nonlin(ichan1,1) + delta_t1*n21
       else
        c2 = nonlin(ichan1,2) + delta_t2*n32
       endif
      endif
!
!...Compute calibration coefficients
!
      i = 1 + (ichan-1)*3
      if (gain.gt.0.0.and.calok(ichan,iscan))then
        cw = itavg(ichan)
        cc = spavg(ichan)
        cwg = cw/gain
        recg = 1./gain
        recg2 = recg*recg

        tmp = c2*recg2*1E19
        if (abs(tmp).lt.2.147483647E9) then 
          pmcal(i,iscan) = nint(tmp)
        else
          pmcal(i,iscan) = 0
          c2 = 0.0       !Try a linear calibration
          if (iprint.ge.1) 
     &      write(ama_cllgu,*)'Chan=',ichan,' scan=',iscan,
     &      ' a2 overflow:',tmp
        endif

        tmp = (recg - c2*(cc+cw)*recg2)*1E13
        if (abs(tmp).lt.2.147483647E9) then 
          pmcal(i+1,iscan) = nint(tmp)
        else
          pmcal(i+1,iscan) = 0
          if (iprint.ge.1)
     &      write(ama_cllgu,*)'Chan=',ichan,' scan=',iscan,
     &      ' a1 overflow:',tmp
        endif

        tmp = (radbb-cwg + c2*cw*cc*recg2)*1E9
        if (abs(tmp).lt.2.147483647E9) then 
          pmcal(i+2,iscan) = nint(tmp)
        else
          pmcal(i+2,iscan) = 0
          if (iprint.ge.1)
     &      write(ama_cllgu,*)'Chan=',ichan,' scan=',iscan,
     &      ' a0 overflow:',tmp
        endif
      else                   !Gain<0 or calok false
        pmcal(i,iscan) = 0
        pmcal(i+1,iscan) = 0
        pmcal(i+2,iscan) = 0
        linqualflgs(iscan) = IOR(linqualflgs(iscan),2048) !Set bit 11 to 
                                        !indicate some uncalibrated channels
      endif
!
!...Put secondary coefficients in 1B common also interpolate for inst temp
!
      if (delta_t2 .lt. 0)then
        sdc = u2(ichan,1) + delta_t1*u221
        sdcal(i,iscan) = nint(sdc)
        sdc = u1(ichan,1) + delta_t1*u121
        sdcal(i+1,iscan) = nint(sdc)
        sdc = u0(ichan,1) + delta_t1*u021
        sdcal(i+2,iscan) = nint(sdc)
      else
        sdc = u2(ichan,2) + delta_t2*u232
        sdcal(i,iscan) = nint(sdc)
        sdc = u1(ichan,2) + delta_t2*u132
        sdcal(i+1,iscan) = nint(sdc)
        sdc = u0(ichan,2) + delta_t2*u032
        sdcal(i+2,iscan) = nint(sdc)
      endif
!
      RETURN
      END
