      subroutine amb_smpmn(lstat,ispvw,itmean,spmean,sampok)
!+
!NOM/NAME $Id: amb_smpmn.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module computes the average AMSU-B internal target and 
!     space view counts for a single scan line. 
!     Also performs quality control on the space view and internal
!     target view counts and updates the scan line flags if necessary.
!
!METHOD
! For v1.3 it has been heavily modified to conform more closely with the
! method used by NESDIS for AMSU-B, and to add the MHS moon detection code:
! 1) Check antenna pointing. Flag and reject if outside limits.
! 2) Reject any space samples that are within a specified angle to the moon. If
!    all are within this angle then retain the sample with lowest counts.
! 3) Reject individual samples that are outside gross limits (but use the rest)
! 4) Compute mean space and target counts from remaining good samples
! 5) Reject all samples if any one of the good samples differs from the mean
!    by more than the specified tolerance
! 6) Set channel quality flags
! 7) Check line-to-line consistency of mean counts
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                         <author>)
! 1.0      Nov 95      no                            R.Saunders
! 1.1      Jan 97     check on cal counts            R.Saunders
! 1.2      March 03   f90 test                       T.Labrot
! 1.3      April 04   Revision based on mhs_smpmn.F  N C Atkinson
! 1.4      Sept 05    Insert 'do ichan' in loop 10   N C Atkinson
!         
!ARGUMENTS  lstat(mxlin) - in status of scan
!           ispvw(mxlin) - in space view selected (0-3)
!           itmean(5,mxlin) - out scan mean internal target counts
!           spmean(5,mxlin) - out scan mean space view counts
!           sampok(5,mxlin)- in/out cal view counts flag
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!           none
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
!
!
! includes/commons
#include "amb_cinit.h"
#include "ambclin.h"
#include "amb1b.h"
#include "ambscn.h"
!
! local variables
      integer ichan,is,iscan,iv,iv2   !index of loops
      integer ispvw(mxlin),iold,nwarm,ncold,coldcts(4),warmcts(4)
      integer coldmin, ismin, nmoon
      real scmeanw,scmeanc,p,diffcounts
      real itmean(5,mxlin),spmean(5,mxlin),s1,s2,i1,i2,d
      logical*1 sampok(5,mxlin)
      logical*1 coldok(4),warmok(4),lstat(mxlin),validfound
      logical*1 warnwarm,warnspace
      data warnwarm/.true./
      data warnspace/.true./
      logical*1 errorwarm,errorspace,firstchan
      
      integer*2 i2word2,i2word4,i2word16,i2word32,i2word48
      parameter (i2word2=2)
      parameter (i2word4=4)
      parameter (i2word16=16)
      parameter (i2word32=32)
      parameter (i2word48=48)
!
!--END OF DECLARATIONS---------------------------------------------------------
!
      if (iprint.ge.1) write(amb_cllgu,*)
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'
      if (iprint.ge.1) write(amb_cllgu,*) '**   SUBROUTINE AMB_SMPMN *'
      if (iprint.ge.1) write(amb_cllgu,*) '***************************'

!----start scan line loop----------------------------------------------------
      do 200 iscan = 1 , amb_h_scnlin
      if (lstat(iscan)) then             !scan line ok

!...Do antenna pointing check on space views. As well as computing the position
!   from the counts, look at the flag supplied with the instrument telemetry.
!   Zero counts at pixel 1, and 1.1 deg per FOV. 

       do 10 is=1,nview    !Loop round each view
         iv2 = (is-1)*6+1
         p = 41.05 + (0.5+abs(coldcal(iv2,iscan)))*ant_coef    ! posn decode
         if (p.gt.360) p = p-360.0
         if ( p.lt. (scanpos(is,ispvw(iscan)+1)-pos_errc) .or. 
     &        p.gt. (scanpos(is,ispvw(iscan)+1)+pos_errc)) then
           do ichan=1,5
             sampok(ichan,iscan)=.false.
           enddo
           if ( iprint.ge.1 .and. warnspace) then
             write (amb_cllgu,'(A,I4,A,F6.1,A)')
     &        '***warning space view position error, line',iscan,
     &        ',',p-scanpos(is,ispvw(iscan)+1),' deg'
             warnspace = .false.
           endif
!Flag questionable calibration
           linqualflgs(iscan) = IOR(linqualflgs(iscan),512)  !linqualflgs bit 9
         else
           if (iprint.ge.1 .and. .not.warnspace) then
             write(amb_cllgu,*)'Space view position now OK, line',iscan
             warnspace = .true.
           endif
         endif

!...do antenna pointing check on internal target views
!   Assume zero counts at pixel 1, and 1.1 deg per FOV. 

         p = 41.05 + (0.5+abs(warmcal(iv2,iscan)))*ant_coef    ! posn decode
         if (p.gt.360) p = p-360.0
         if ( p.lt. (scanpos(is+4,ispvw(iscan)+1)-pos_errc) .or. 
     &        p.gt. (scanpos(is+4,ispvw(iscan)+1)+pos_errc)) then
           do ichan=1,5
             sampok(ichan,iscan)=.false.
           enddo
           if ( iprint.ge.1 .and. warnwarm) then
             write (amb_cllgu,'(A,I4,A,F6.1,A)')
     &        '***warning target view position error, line',iscan,
     &        ',',p-scanpos(is+4,ispvw(iscan)+1),' deg'
             warnwarm = .false.
           endif
!Flag questionable calibration
           linqualflgs(iscan) = IOR(linqualflgs(iscan),256)  !linqualflgs bit 8
         else
           if (iprint.ge.1 .and. .not.warnwarm) then
             write(amb_cllgu,*)'Target view position now OK, line',iscan
             warnwarm = .true.
           endif
         endif
   10  continue

!----start channel loop------------------------------------------------------

       firstchan = .true.

       do 100 ichan = 1 , 5
       spmean(ichan,iscan) = 0.0    !Initialise arrays
       itmean(ichan,iscan) = 0.0
       
       if (sampok(ichan,iscan)) then   !Omit any channels that are off
        scmeanw = 0.0
        scmeanc = 0.0
        errorwarm=.true.
        errorspace=.true.

!...perform quality checks on counts for each view and compute mean counts

        nwarm = 0     !Number of good warm samples
        ncold = 0     !Number of good cold samples
        scmeanw = 0.0
        scmeanc = 0.0
        coldmin = 65535
        ismin = 1     !Sample with lowest space counts
        nmoon = 0     !Number of samples close to moon

        do 15 is = 1 , nview
         iv = (is-1)*6+ichan+1
         warmok(is) = .true.
         coldok(is) = .true.
         warmcts(is) = warmcal(iv,iscan) 
         coldcts(is) = coldcal(iv,iscan) 

!...find the sample with the lowest space counts (in case all fail the moon
!   angle check), and check the moon angles.

         if (coldcts(is) .lt. coldmin) then
           coldmin = coldcts(is)
           ismin = is
         endif

         if (moonang(is,iscan)*0.01 .lt. moonthresh) then
           nmoon = nmoon + 1
           coldok(is) = .false.
           if (iprint.ge.1.and.firstchan)
     &       write(amb_cllgu,'(A,I4,A,I2,A,F7.2)')
     &       'Scan ',iscan,' sample',is,
     &       ' Moon angle=',moonang(is,iscan)*0.01
         endif
  15    continue

!...If all samples are close to the moon, use the sample with lowest counts
!   For AMSU-B there must be 1 sample that is OK given the 1.1 deg beam width
!   and 1 deg sample separation. Note that we need to set the value of
!   moonthresh rather conservatively to allow for errors in the navigation.

        if (nmoon.eq.nview) then
          coldok(ismin) = .true.
        endif

        do 16 is = 1 , nview

!...do gross limit check. Reject sample if outside gross limits
!   Warnings are printed out once per scan line

         if (warmcts(is).gt.intcntmax(ichan) .or. 
     &   warmcts(is).lt.intcntmin(ichan)) then
           warmok(is) = .false.
           if (iprint.ge.1.and.errorwarm)
     &       write(amb_cllgu,*)'Warm view gross limit err: scan',iscan,
     &        ',chan',ichan,',counts',warmcts(is)
           errorwarm=.false.
         endif

         if (coldcts(is).gt.spcntmax(ichan) .or. 
     &   coldcts(is).lt.spcntmin(ichan)) then
           coldok(is) = .false.
           if (iprint.ge.1.and.errorspace)
     &       write(amb_cllgu,*)'Cold view gross limit err: scan',iscan,
     &        ',chan',ichan,',counts',coldcts(is)
           errorspace=.false.
         endif

!...Sum the counts
         if (warmok(is)) then
           scmeanw = scmeanw + warmcts(is)
           nwarm = nwarm + 1
         endif
         if (coldok(is)) then
           scmeanc = scmeanc + coldcts(is)
           ncold = ncold + 1
         endif

  16    continue

!Print out moon angles, cold counts & warm counts (remove later)
!        if (iprint.ge.2) then
!          if (firstchan) write(amb_cllgu,'(A,I5,A,4F7.2)')
!     &        ' Scan ',iscan,' moonang=',(moonang(is,iscan)*0.01,is=1,4)
!         write(amb_cllgu,'(I5,4I6,2X,4I6)')
!     &        ichan+15,(coldcts(is),is=1,4),(warmcts(is),is=1,4)
!        endif

!
!...Look at differences from mean and reject if too large
!
        if (sampok(ichan,iscan)) then

         if (nwarm.gt.0) then
           itmean(ichan,iscan) = scmeanw/float(nwarm)
           do is=1,nview
             diffcounts = warmcts(is)-itmean(ichan,iscan)
             if (warmok(is).and.abs(diffcounts).gt.dcount(ichan)) then
               if (iprint.ge.1) write(amb_cllgu,'(A,I4,A,I1,A,F6.0)')
     &               'Scan ',iscan,' Warm diff',is,'=',diffcounts
               sampok(ichan,iscan)=.false.
               itmean(ichan,iscan) = 0.0
               nwarm = 0
             endif
           enddo
         endif
              
         if (ncold.gt.0) then
           spmean(ichan,iscan) = scmeanc/float(ncold)
           do is=1,nview
             diffcounts = coldcts(is)-spmean(ichan,iscan)
             if (coldok(is).and.abs(diffcounts).gt.dcount(ichan)) then
               if (iprint.ge.1) write(amb_cllgu,'(A,I4,A,I1,A,F6.0)')
     &               'Scan ',iscan,' Cold diff',is,'=',diffcounts
               if (iprint.ge.1) write(amb_cllgu,*) spmean(ichan,iscan) 
               sampok(ichan,iscan)=.false.
               spmean(ichan,iscan) = 0.0
               ncold = 0
             endif
           enddo
         endif

        endif   !sampok(ichan,iscan)
!
!...Set channel quality flags
!
        if (ncold.eq.0) then          
         chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word16)
        else if (ncold.lt.nview) then
         chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word2)
        endif
             
        if (nwarm.eq.0) then          
         chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word32)
        else if (nwarm.lt.nview) then
         chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word4)
        endif

       endif      !sampok(ichan,iscan)     
       firstchan = .false.

 100   continue   !-------end of channel loop------------

      endif      !lstat(iscan) OK

 200  continue   !-------end of scan line loop----------
!
! Now look at line-to-line consistency of the Space and Internal target counts.
! The difference between successive 'good' values must be less than the 
! 'dcount' values for each channel from the calibration data file. 
! A pair of consistent readings is necessary to define the start of a 'good'
! sequence, after which inconsistent readings are flagged as bad. If
! more than 'ngap' scans elapses since the last good point, the sequence is
! restarted. Isolated valid readings (i.e. more than 'ngap' scans between
! each one) are considered unreliable and are flagged as bad.
!
! Note that this code gives good resilience against occasional bad readings but
! will not cope with a calibration jump. So best to keep 'ngap' fairly small.
!
      do 500 ichan = 1 , 5
        validfound=.false.
        iold = -999
        d = dcount(ichan)

        do 400 iscan = 1 , amb_h_scnlin
          if (.not.validfound) then   !Start of sequence - no good data yet
            if(lstat(iscan).and.sampok(ichan,iscan))then
              if ((iscan-iold).gt.ngap) then
                if (iold.gt.0) then
                  sampok(ichan,iold)=.false. !Data gap: old channel is invalid
                  linqualflgs(iold)=IOR(linqualflgs(iold),2048) !Flag bad channel
                      !This point could be OK, but there are no other good 
                      !points in the vicinity, so flag both internal and space 
                      !counts as bad.
                  chqualflg(ichan,iold)=
     &                         IOR(chqualflg(ichan,iold),i2word48)
                endif
              else
                s1=spmean(ichan,iold)   !Test space and internal counts for
                s2=spmean(ichan,iscan)  !consistency.
                i1=itmean(ichan,iold)
                i2=itmean(ichan,iscan)
                if(abs(s1-s2).le.d.and.abs(i1-i2).le.d)then
                  validfound=.true.    !Diff good - both points accepted
                else
                  sampok(ichan,iold)=.false.  !Diff bad - reject old point
                  linqualflgs(iold)=IOR(linqualflgs(iold),2048) !Flag bad channel
                  if (abs(s1-s2).gt.d)
     &             chqualflg(ichan,iold)=
     &                   IOR(chqualflg(ichan,iold),i2word16)
                  if (abs(i1-i2).gt.d)
     &              chqualflg(ichan,iold)=
     &                   IOR(chqualflg(ichan,iold),i2word32)
                endif
              endif
              iold=iscan
            endif
          else          !Valid data already present

!Start again if gap is too long 
            if ((iscan-iold).gt.ngap) then 
              validfound=.false.
              if (lstat(iscan).and.sampok(ichan,iscan))then
                iold=iscan  
              else
                iold=-999
              endif
            else
              if (lstat(iscan).and.sampok(ichan,iscan))then
                s1=spmean(ichan,iold)
                s2=spmean(ichan,iscan)
                i1=itmean(ichan,iold)
                i2=itmean(ichan,iscan)
                if(abs(s1-s2).le.d.and.abs(i1-i2).le.d)then
                  iold=iscan             !New point valid - update iold
                else
                  sampok(ichan,iscan)=.false.  !New point is inconsistent
                  linqualflgs(iscan)=IOR(linqualflgs(iscan),2048) !Bad channel
                  if (abs(s1-s2).gt.d)
     &              chqualflg(ichan,iscan)=
     &                     IOR(chqualflg(ichan,iscan),i2word16)
                  if (abs(i1-i2).gt.d)
     &              chqualflg(ichan,iscan)=
     &                     IOR(chqualflg(ichan,iscan),i2word32)
                endif
              endif  
            endif
          endif
 400    continue  ! end of scan line loop---------------------
        if (.not.validfound.and.iold.gt.0) then
          sampok(ichan,iold)=.false.  !Set last point flags
          linqualflgs(iold)=IOR(linqualflgs(iold),2048)    !Bad channel
          chqualflg(ichan,iold)=IOR(chqualflg(ichan,iold),i2word48)
        endif
 500  continue    ! end of channel loop------------------------
      return
      end





