! Name: $Id: avh_lbc.F 596 2018-11-06 15:03:21Z frna $
      subroutine avh_lbc(line, avh_rad, avh_bt, avh_qc, bit_qc,
     &                     missing, good_lines, bad_lines)
!-----------------------------------------------------------------------
!
! BUT/PURPOSE: 
!       AVHRR level 1b to level 1c
!	takes one level1b avhrr line and 
!       converts numerical counts onto radiances and BTs    
!       needs common "avh1b.h"
!       will initialize common "cavh_cpi.h"
!  
! DESCRIPTION: FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!	see documentations     
!
! PROPRIETAIRE/OWNER:
!
! HISTORIQUE/HISTORY  (MODIFICATIONS/CHANGES)
!
! version	date		comment      		author
!   1.0         28/12/98         no                    P. Brunel
!   1.1         26/01/99     change scale3             M. Derrien 
!   1.2         March 03                               T.Labrot
!        f90 test : Changed  type XXX/yyy/
!                        to  type XXX
!                            data XXX/yyy/
!                   ncount is defined as a real but it is used as subscript
!                   of tabalb ==> created a new variable integer i4ncount
!   1.3         May 05     Allow for different definitions of radtempcnv. NCA.
!   1.7      17/01/2006 : Philippe Marguinaud : change float( k )
!                         to float( int( k ) ), where k has a kind=2
!   1.8      13/10/2011 : Nigel Atkinson : use operational calvis if available.
!   1.9      12/02/2014 : Increase size of tabalb, to accommodate scaled
!                         radiances from EPS 1B (PFS) data.
!   1.10    09/03/2017 : Pascale Roquet: change the tabalb size 
!                        and computation of i4max 
!                        (case with huges values in visible chanel)
!   1.11    24/09/2018 ; Pascale Roquet : correct radiance units in comment
!                        
!  
! ARGUMENTS:
!
! FUNCTIONS AND SUBROUTINES CALLS (RETURN CODES FOR FUNCTIONS):
!
!   gp_gb16   : get bit from an integer*2
!   gp_gb32   : get bit from an integer
!
! COPYRIGHT(c) Meteo-France ( EVOLUNIX - ATOVS - EUMETSAT project 1994 )
!
!-----------------------------------------------------------------------
 
      implicit none

!---- DECLARATIONS -----------------------------------------------------

!---- ARGUMENTS --------------------------------------------------------
      integer line		! scan line number
      real avh_rad(5,2048) 	! radiances mW/(m2.sr.cm-1)
      real avh_bt(5,2048)	! brigthness temperature (K) and albedo
      integer avh_qc(2048)	! QC word for each avhrr fov
                                !    (bit 'bit_QC' is 0=good, 1=bad fov)
      integer bit_qc		! bit number for quality
      integer missing		! line quality  0=present   1=missing
      integer good_lines(6)
      integer bad_lines(6)
! for visible channels radiances and Bts are the same

!---- COMMONS ----------------------------------------------------------
#include "avh1b.h"
#include "cavh_cpi.h"
!
!---- PARAMETERS -------------------------------------------------------
!---- LOCAL VARIABLES --------------------------------------------------

      integer p,c		! pixel, channel
      real ncount		! earth viewed numerical count (real)
      integer i4ncount          ! earth viewed numerical count (integer)
      integer i4nmax            ! maximum value of i4ncount (integer)

      real slope		! coef for visible calibration
      real intercept		! ....
      real tabalb(0:65535,3)	! table for count to albedo

      real k1,k2,k3		! coefficient conversion count ->radiance
      double precision nu	! central wavenumber
      real radiance		! radiance  mW/(m2.sr.cm-1)

      integer expon3 ! exponent of scaling factor for 1st IR cal. coeff
      integer expon4 ! exponent of scaling factor for 2nd IR cal. coeff
      integer expon5 ! exponent of scaling factor for 3rd IR cal. coeff
      real scale3 ! scaling factors for k1
      real scale4 ! scaling factor for k2
      real scale5 ! scaling factor for k3
!!$      data scale3 /1.e-06/
!!$      data scale4 /1.e-06/
!!$      data scale5 /1.e-06/

      integer good_qc		! good quality word
      integer bad_qc		! bad quality word
      integer*4 status_channel ! avh_h_inststat2 or avh_h_inststat1
!f90  logical first /.true./
      logical first 
      data first/.true./
      integer vcoefset		! visible coef set

      save       ! save local tables and logical first
!---- EQUIVALENCES -----------------------------------------------------
!---- FUNCTIONS --------------------------------------------------------
      integer*2  gp_gb16 ! get bit from an integer*2
      integer  gp_gb32   ! get bit from an integer
      real avh_cir_r     ! radiance to brightness temperature conversion real
!---- CODE ------------------------------------------------------------- 

      if (first) then
!
!       modification to correct lack of accuracy of scaling factors 
!       proposed by NOAA   ( M. Derrien 26/01/1999)
!       must be considered only for a transition, because we
!       do not change the l1bversnb number due to CMS consideration
!       and we don't have the new NESDIS scaling factors 
!       this will change scaling factors only for 1b files 
!       whose data are newer than 27 January 1999,12 UTC
!       date where changed at CMS
!                  
!       Now avh_filler2(3) stores the 3 exponents of the scaling
!       coefficients. Subroutine avh_lbc will be able to read the
!       new exponents and will suppose exponent 6 for all archived
!       files. Beware of reading these new files without taking into
!       account the filler2 array
        expon3=0
        expon4=0
        expon5=0
        ! test values of filler2 between 6 and 12 in case of
        ! bad values in level1b file
        if(avh_filler2(1).ge.6 .and. avh_filler2(1).le.12) then
           expon3 = avh_filler2(1)
        endif
        if(avh_filler2(2).ge.6 .and. avh_filler2(2).le.12) then
           expon4 = avh_filler2(2)
        endif
        if(avh_filler2(3).ge.6 .and. avh_filler2(3).le.12) then
           expon5 = avh_filler2(3)
        endif

        if(expon3*expon4*expon5 .eq.0) then
           !one of the 3 exponents is 0, take the default ones
           if (avh_h_siteid.eq.'CMS' .and.
     &         avh_h_startdatajd.ge.17923.5)then
              !new values at CMS starting on Jan 27th, 1999
              expon3=9
              expon4=6
              expon5=6
           else
              expon3=6
              expon4=6
              expon5=6
           endif
        endif

        call ml_wti('info',
     &    'IR calibration exponent of scaling coefficient 1', expon3)
        call ml_wti('info',
     &    'IR calibration exponent of scaling coefficient 2', expon4)
        call ml_wti('info',
     &    'IR calibration exponent of scaling coefficient 3', expon5)

        scale3 = 10. ** (-1 *expon3)
        scale4 = 10. ** (-1 *expon4)
        scale5 = 10. ** (-1 *expon5)


!!$        if (avh_h_startdatajd.ge.17923.5) then
!!$          scale3 = 1.e-09
!!$        endif      
         !fabrication des tables de conversion pour les canaux visibles
         !-------------------------------------------------------------
         
         !We use the operational coefficients if available, otherwise prelaunch 
         ! operational set is 1 
         ! test set is 2
         ! prelaunch set is 3
         do c = 1, avh_mxvischn
            if (avh_calvis(1,1,c) .NE. 0) then     !test the operational slope
              vcoefset=1
            else
              vcoefset=3
            endif
            slope     = avh_calvis(1,vcoefset,c)*1.e-10
            intercept = avh_calvis(2,vcoefset,c)*1.e-7
            do i4ncount = 0 , avh_calvis(5,vcoefset,c)
               tabalb(i4ncount,c) = (slope * i4ncount + intercept)
               if(tabalb(i4ncount,c).lt.0.) tabalb(i4ncount,c)=0. 
               if(tabalb(i4ncount,c).gt.160.) tabalb(i4ncount,c) = 160.
            enddo
            slope     = avh_calvis(3,vcoefset,c)*1.e-10
            if (slope .lt. 0) slope = slope + 0.4294967296  !Ensure positive
            intercept = avh_calvis(4,vcoefset,c)*1.e-7
            i4nmax =  65535
            do i4ncount = avh_calvis(5,vcoefset,c), i4nmax
               tabalb(i4ncount,c) = (slope * i4ncount + intercept)
               if(tabalb(i4ncount,c).lt.0.) tabalb(i4ncount,c) = 0.
               if(tabalb(i4ncount,c).gt.160.) tabalb(i4ncount,c) = 160.
            enddo

         enddo
       
         !Infrared channels
         !-----------------
         ! C1 and C2 are defined here because the common
         ! cavh_cpi has not been initialized (call avh_cpar)
         ! planck1 is C1 * nu3
         ! planck2 is C2 * nu
         ! avh_bandcor(1,c) is nu (cm-1)
         ! avh_bandcor(2,c) is intercept (K), for T* = A+BT
         ! avh_bandcor(3,c) is slope (K/K)

         C1=0.000011910659d0
         C2=1.438833d0
	
         !Planck coefficients channel 3B
         c=1  
         nu=avh_h_radtempcnv(1,c)*1.e-02
         planck1(c)=C1*nu*nu*nu
         planck2(c)=C2*nu
         avh_bandcor(1,c)=nu
         !Planck coefficients channel 4
         c=2
         nu=avh_h_radtempcnv(1,c)*1.e-03
         planck1(c)=C1*nu*nu*nu
         planck2(c)=C2*nu
         avh_bandcor(1,c)=nu
         !Planck coefficients channel 5
         c=3
         nu=avh_h_radtempcnv(1,c)*1.e-03
         planck1(c)=C1*nu*nu*nu
         planck2(c)=C2*nu
         avh_bandcor(1,c)=nu

         ! For the band correction coefs there are different 1b definitions:
         !  case 1: avh_h_radtempcnv=[v,A,B], where T* = A+BT, A and B both +ve
         !  case 2: avh_h_radtempcnv=[v,a,b], where T = a+bT*, i.e. A=-a/b,B=1/b
         !  NESDIS and AAPP v5 use definition 2 (as implied by the name
         !  "radtempcnv") but AAPP v1-4 use definition 1. Use sign of 
         !  avh_h_radtempcnv(2,chan) to identify the correct one.
         do c=1,3
          if (avh_h_radtempcnv(2,c).ge.0) then                 !case 1 
           avh_bandcor(2,c)=avh_h_radtempcnv(2,c)*1.e-05
           avh_bandcor(3,c)=avh_h_radtempcnv(3,c)*1.e-06
          else
           avh_bandcor(2,c)=                                   !case 2
     &     -avh_h_radtempcnv(2,c)*1.e-05/(avh_h_radtempcnv(3,c)*1.e-06)
           avh_bandcor(3,c)=1./(avh_h_radtempcnv(3,c)*1.e-06)
          endif
         enddo

         !Radiance to temperature tables, all IR channels (common temp_conv)
         call avh_icon

         first = .false.
      end if

      !channel satatus for the line
      !---------------------------- 
      status_channel = avh_h_inststat1
      if (avh_h_statchrecnb.eq.0) then
        status_channel = avh_h_inststat1
      else if(line.ge.avh_h_statchrecnb) then
        status_channel = avh_h_inststat2
      endif

      !missing line test
      !-----------------
      if(gp_gb32(avh_qualind,31).eq.1)then          

         !missing line, 0 ==> 5 channels 
         do p=1,2048
            do c=1,5
               avh_bt(c,p)=0.
            enddo
         enddo
         do p=1,2048
            do c=1,5
               avh_rad(c,p)=0.
            enddo
         enddo
         missing = 1
         bad_lines(1) = bad_lines(1) + 1
         bad_lines(2) = bad_lines(2) + 1
         bad_lines(3) = bad_lines(3) + 1
         bad_lines(4) = bad_lines(4) + 1
         bad_lines(5) = bad_lines(5) + 1
         bad_lines(6) = bad_lines(6) + 1
            

      else
         !good line
         missing = 0

         !channel VIS 1
         !=============
         c = 1
         !testing if the channel is enable (1)
         if(gp_gb32(status_channel,13).eq.1)then

            good_lines(1) = good_lines(1)+1
            !loop on 2048 pixels
            do p=1,2048                 
               avh_bt(c,p) = tabalb(avh_hrpt(c,p),c)
               avh_rad(c,p)=0.
            enddo
         else
            !channel is disable
            bad_lines(1) = bad_lines(1) + 1
            do p=1,2048
               avh_bt(c,p)=0.
               avh_rad(c,p)=0.
            enddo
         endif

         !channel VIS 2
         !=============
         c = 2

         !testing if the channel is enable (1)
         if(gp_gb32(status_channel,12).eq.1)then

            good_lines(2) = good_lines(2)+1
            !loop on 2048 pixels
            do p=1,2048                  
               avh_bt(c,p) = tabalb(avh_hrpt(c,p),c)
               avh_rad(c,p)=0.
            enddo

         else
            !channel is disable
            bad_lines(2) = bad_lines(2) + 1
            do p=1,2048
               avh_bt(c,p)=0.
               avh_rad(c,p)=0.
            enddo
         endif


         !canal IR 3B or canal Vis 3A
         !===========================
         c = 3

         !testing the selection of the channel 3 (bit 0 = 1 for 3B)
         if(gp_gb16(avh_scnlinbit,0).eq.1)then

            !testing if the channel 3Bis enable (1)
            if(gp_gb32(status_channel,10).eq.1)then

               !testing the calibration quality flag
               if(gp_gb16(avh_calqual(1),7).eq.0)then  
                  k1=avh_calir(1,1,1)*scale3
                  k2=avh_calir(2,1,1)*scale4
                  k3=avh_calir(3,1,1)*scale5

                  good_lines(3) = good_lines(3)+1
                  !loop on 2048 pixels
                  do p=1,2048
                     ncount = float(int(avh_hrpt(c,p)))
                     radiance = k1*ncount*ncount + k2*ncount + k3
                     avh_rad(c,p) = radiance
                     avh_bt(c,p)=avh_cir_r(radiance,1)

         !     add a test on TB 
         !     In general, AVHRR/3 channel 3B measurements do not provide reasonable 
         !     radiance values for target temperatures of 180 K, since the signal 
         !     is too weak for detection. This effect is not a pecularity of 
         !     AVHRR/3 onboard Metop-A or B, but can also occur 
         !     for measurements of AVHRR/3 instruments onboard the NOAA-satellites.
                     if (avh_bt(c,p) .le. 180.) then
                          avh_bt(c,p)  = 180. 
                     end if
                  enddo

               else
                  !calibration not usable 
                  bad_lines(3) = bad_lines(3) + 1
                  do p=1,2048
                     avh_rad(c,p)=0.
                     avh_bt(c,p)=0.
                  enddo
               endif
            else
               !channel is disable
               bad_lines(3) = bad_lines(3) + 1
               do p=1,2048
                  avh_rad(c,p)=0.
                  avh_bt(c,p)=0.
               enddo
            endif

         else
            !canal 3A

            !testing if the channel 3A is enable (1)
            if(gp_gb32(status_channel,11).eq.1)then

               good_lines(6) = good_lines(6)+1
               !loop on 2048 pixels
               do p=1,2048 
                  avh_bt(c,p) = tabalb(avh_hrpt(c,p),3)
                  avh_rad(c,p)=0.
               enddo

            else 
               !channel is disable
               bad_lines(6) = bad_lines(6) + 1
               do p=1,2048
                  avh_bt(c,p)=0.
                  avh_rad(c,p)=0.
               enddo
            endif

         endif!channel 3A or 3B


         !channel IR 4
         !============
         c = 4

         !testing if the channel is enable (1)
         if(gp_gb32(status_channel,9).eq.1)then

            !testing the calibration quality flag
            if(gp_gb16(avh_calqual(2),7).eq.0)then
               k1=avh_calir(1,1,2)*scale3
               k2=avh_calir(2,1,2)*scale4
               k3=avh_calir(3,1,2)*scale5

               good_lines(4) = good_lines(4)+1
               !loop on 2048 pixels
               do p=1,2048
                  ncount = float(int(avh_hrpt(c,p)))
                  radiance = k1*ncount*ncount + k2*ncount + k3
                  avh_rad(c,p) = radiance
                  avh_bt(c,p)=avh_cir_r(radiance,2)
               enddo
            else
               !calibration not usable 
               bad_lines(4) = bad_lines(4) + 1
               do p=1,2048
                  avh_rad(c,p)=0.
                  avh_bt(c,p)=0.
               enddo
            endif
         else
            !channel is disable
            bad_lines(4) = bad_lines(4) + 1
            do p=1,2048
               avh_rad(c,p)=0.
               avh_bt(c,p)=0.
            enddo
         endif


         !canal IR 5
         c = 5

         !testing if the channel is enable (1)
         if(gp_gb32(status_channel,8).eq.1)then

            !testing the calibration quality flag
            if(gp_gb16(avh_calqual(3),7).eq.0)then
               k1=avh_calir(1,1,3)*scale3
               k2=avh_calir(2,1,3)*scale4
               k3=avh_calir(3,1,3)*scale5

               good_lines(5) = good_lines(5)+1
               !loop on 2048 pixels
               do p=1,2048
                  ncount = float(int(avh_hrpt(c,p)))
                  radiance = k1*ncount*ncount + k2*ncount + k3
                  avh_rad(c,p) = radiance
                  avh_bt(c,p)=avh_cir_r(radiance,3)
               enddo
            else
               !calibration not usable 
               bad_lines(5) = bad_lines(5) + 1
               do p=1,2048
                  avh_rad(c,p)=0.
                  avh_bt(c,p)=0.
               enddo
            endif
         else
            !channel is disable
            bad_lines(5) = bad_lines(5) + 1
            do p=1,2048
               avh_rad(c,p)=0.
               avh_bt(c,p)=0.
            enddo
         endif

      endif!missing line


      good_qc = 0
      if(missing.eq.0) then
        do p=1,2048
          avh_qc(p) = good_qc		! good line
        end do
      else

        bad_qc = ibset(good_qc,bit_qc)
        call ml_wti('info','missing AVHRR line',line)
        do p=1,2048
          avh_qc(p) = bad_qc		! missing line
        end do
      end if

      return
      end
