!+ Calculate angle between the moon and the space view of a cross-track
!  scanner such as AMSU
!
      SUBROUTINE moon_amsu(latarray,lonarray,azcorr,jday,hour,ra,
     &   sindec,cosdec,N,sinamsual,cosamsual,angle_diff)
!
! Method:
! 1) Use lat/lon of 2 samples straddling mid-swath to give the azimuth of the
!    scan axis. 
! 2) Convert moon position to local azimuth and altitude
! 3) Calculate the angular separation between moon and space view
!
! Intended for use with AMSU-A, but could be used for any cross-track scanner
!
! References:
! 1) http://www.stargazing.net/kepler/moon.html
! 2) Astronomy with your personal computer, P Duffet-Smith, Cambridge
!    University Press
!
!  Current code owner: N.C.Atkinson
!
!  History:
!  Version  Date       Comment
!  1.0    06/06/01  Original code.
!  1.1    22/05/03  Web reference updated.
!  1.2    22/04/04  Add implicit none
!  1.3    07/06/06  Check cosazimuth < 1.0
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################

      IMPLICIT NONE                                                      

! Arguments:
      INTEGER N           !Number of space views to consider
      REAL*8 jday         !Time in days from 12Z, 1 Jan 2000
      REAL   latarray(2)  !Latitude of 2 samples straddling mid-swath (deg)
      REAL   lonarray(2)  !Longitude of 2 samples straddling mid-swath (deg)
      REAL   azcorr       !Azimuth correction (radians) to allow for satellite 
                          !forward motion. Pre-calculated.
      REAL   hour         !Hour of day
      REAL   ra           !Right ascension of moon
      REAL   sindec,cosdec !Sine and cosine of moon declination
      REAL   sinamsual(N)  !Sine of instrument space view altitude
      REAL   cosamsual(N)  !Cosine of instrument space view altitude
      REAL   angle_diff(N) !Angular separation in degrees

! Local scalars:
      REAL*4 rads,lat,lon,londeg,amsuaz,lst,ha,sinlat,coslat,
     &    sinaltitude,cosaltitude,sinazimuth,cosazimuth,azimuth,
     &    cosamsuaz,sinamsuaz
      INTEGER I

      rads = 3.1415927/180.0

!Calculate AMSU space view azimuth. Use lat/lon information for nadir samples
      IF ((lonarray(1)-lonarray(2)).GT.300.0) THEN
        lonarray(2)=lonarray(2)+360.0                    !Allow for wrap around
      ELSE IF ((lonarray(2)-lonarray(1)).GT.300.0) THEN
        lonarray(1)=lonarray(1)+360.0
      ENDIF
      lat = (latarray(1)+latarray(2))*rads/2.0
      londeg = (lonarray(1)+lonarray(2))/2.0
      lon = londeg*rads
      amsuaz = atan2((lonarray(2)-lonarray(1))*cos(lat), 
     &   latarray(2)-latarray(1)) + azcorr
      sinamsuaz = sin(amsuaz)
      cosamsuaz = cos(amsuaz)

!Local sidereal time in degrees (Ref 1)
      lst = 100.46 + 0.985647352*jday + hour*15 + londeg       !Approximation

!Hour angle (radians)
      ha = lst*rads - ra

!Moon azimuth and altitude (Ref 2)
      sinlat = sin(lat)
      coslat = cos(lat)
      sinaltitude = sindec*sinlat + cosdec*coslat*cos(ha)
      cosaltitude = cos(asin(sinaltitude))
      cosazimuth = (sindec - sinlat*sinaltitude)/(coslat*cosaltitude)
      if( cosazimuth .gt. +1.0 ) cosazimuth = +1.0
      if( cosazimuth .lt. -1.0 ) cosazimuth = -1.0
      azimuth = acos(cosazimuth)
      if (sin(ha) .gt. 0) azimuth=6.283185307-azimuth   !Remove ambiguity
      sinazimuth = sin(azimuth)

!Angular difference in degrees, using scalar product of the two vectors
      DO I=1,N
        angle_diff(I) = acos( cosamsual(I)*cosaltitude*
     &     (cosamsuaz*cosazimuth+sinamsuaz*sinazimuth)
     &    + sinamsual(I)*sinaltitude )/rads
      ENDDO

      RETURN
      END  

