        subroutine h_calibcoeffile(lu,satid, versnb,versyr,versday,
     &                             ch20year,ch20day,f2,lambda)
!+
!NOM/NAME $Id: h_calibcoeffile.F 114 2008-02-04 15:46:22Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module opens/closes and reads the calibration coefficients file. 
!     Data are stored in common hrs_clcf (include cinit.h).
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!  
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      June 95      no             T. LABROT
! 1.1      Mar 04   Modify for linux   N Atkinson / A Korpela
! 2.0      Febr 05  Because NOAA18 has 5 PRTs and 6 coefs for each,
!                   need to update calcoef.dat file, and so its read program
!                   Remove "Reasonable radiance limits by channel" T.Labrot (MF)
! 2.1      May 07   Change open for g95   P. Marguinaud
! 2.2      July 07  Add test on ios       T. Labrot (MF)
!         
!ARGUMENTS 
!     input:
!      integer lu        !logical unit for the calib.coef. file
!      integer satid     !order satellite number   
!     output:
!      integer versnb    !number of the version of the calib. coef. file
!      integer versyr    !year of the version of the calib. coef. file
!      integer versday   !day of the year of the calib. coef. file 
!      integer ch20year  !year of most recent solar channel calibration
!      integer ch20day   !day of year of most recent solar channel calibration
!      dble prec f2      !albedo-radiance ch20 solar filtered irradiance
!      dble prec lambda  !albedo-radiance ch20 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! 
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none 
                                                     

! includes/commons
#include "cinit.h"
#include "hclin.h"
#include "hrsparam.h"

! argument variables 
      integer lu,satid
      integer versnb,ch20year,ch20day 
      integer versyr,versday 
      double precision f2,lambda  
          
! local variables
      character*80 text           !variable text
      character*80 charact        !buffer storing one line of the file
      character*6 satname
      integer ios                 !variable that receives the error status
      integer i,j                 !index for loops
      integer tab2(2)             !array for the reading of the file
      double precision tab0(6)    !array for the reading of the file
      double precision tab1(2)    !array for the reading of the file

! functions
      character*6 noascnam

!--END OF DECLARATIONS---------------------------------------------------------

! Open and read calibration coefficients file
#ifdef FORTRAN_OPEN_RW
       open(lu,access='sequential',status='old',iostat=ios,
     >  action = 'read')
#else
       open(lu,access='sequential',status='old',iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       if(ios.ne.0)
     &   open(lu,err=900,access='sequential',
     &      status='unknown',iostat=ios)
#endif

       if(ios.eq.0) then
         text='The calib. coef. file has been opened'
         call ml_wt('info',text)
       endif

! Skip the header lines
       read(lu,'(a80)',err=901) charact
       do while (charact(1:1) .eq. '#')
         read(lu,'(a80)',err=901) charact
       enddo
       backspace lu

! Parameters of the calibration coefficients file 
       read(lu,*,err=901)versnb
       read(lu,*,err=901)versyr
       read(lu,*,err=901)versday

       if (iprint.ge.1) then
         write(hrs_cllgu,*)
         write(hrs_cllgu,*) 'calibration coefficients file' 
         write(hrs_cllgu,500)versnb,versyr,versday
       endif

       write(text,'(2X,"Version nb, year, day of year :",
     & I2,1X,I4,1X,I3)')versnb,versyr,versday
       call ml_wt('info',text)
       
! First & second radiation constants
       read(lu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. 'First ')
         read(lu,'(a15)',err=901) charact
       enddo
       read(lu,*,err=901) plck(1),plck(2)

! To get the data of the good satellite
       satname = noascnam(satid) 
       read(lu,'(a80)',err=901) charact
       do while (charact(4:9) .ne. satname)
         read(lu,'(a15)',err=901) charact
       enddo

! IWT PRT count to temperature conversion coefficients (5 PRTs,6 coefs)
       read(lu,*,err=901)
       read(lu,*,err=901)
       do i=1,5
         read(lu,*,err=901)tab0
         do j=1,6
           prtcnttmp(i,j) = tab0(j)
         enddo
       enddo

! Weight coefficients for each PRT
       read(lu,*,err=901)
       read(lu,*,err=901)prtnb
       read(lu,*,err=901)prtwght

! IR central waves
       read(lu,*,err=901)
       read(lu,*,err=901)
       do i=1,hrs_mxirchn
         read(lu,*,err=901) cwave(i)
       enddo

! Band-correction coefficients
       read(lu,*,err=901)
       read(lu,*,err=901)
       read(lu,*,err=901)
       do i=1,hrs_mxirchn
         read(lu,*,err=901)tab1
         bdcorr(i,1) = tab1(1)
         bdcorr(i,2) = tab1(2)
       enddo

! Upper and lower limits for the radiant signal output counts from the 
! space view scan (IR channels only).
       read(lu,*,err=901)
       read(lu,*,err=901)
       do i=1,hrs_mxirchn
         read(lu,*,err=901) tab2
         spcntlm(i,1) = tab2(1)
         spcntlm(i,2) = tab2(2)
       enddo

! Upper and lower limits for the radiant signal output counts from the 
! IWT scan (IR channels only).
       read(lu,*,err=901)
       read(lu,*,err=901)
       do i=1,hrs_mxirchn
         read(lu,*,err=901)tab2
         iwtcntlm(i,1) = tab2(1)
         iwtcntlm(i,2) = tab2(2)
       enddo


! Maximum value of the difference between the maximum and minimum values 
! of PRT output counts.
       read(lu,*,err=901)
       read(lu,*,err=901)
       read(lu,*,err=901) prtminmax

! Calibration coefficients for the visible channel
!    year and day of year of the most recent solar channel calibration
!    % albedo/count , % albedo
       read(lu,*,err=901)
       read(lu,*,err=901)
       read(lu,*,err=901)
       read(lu,*,err=901)
       read(lu,*,err=901) ch20year,ch20day
       read(lu,*,err=901) visibcoef

! Albedo-radiance visible channel solar filtered irradiance and
! equivalent filter width 
       read(lu,*,err=901)
       read(lu,*,err=901)
       read(lu,*,err=901) f2,lambda


       close(lu,err=902)
       go to 200

 500  format(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)

 900  write(hrs_cllgu,*)'Error opening calib. coef. file'
      text='Calib coefficients file could not be open'
      call ml_wt('error',text)
      call aappexit(2)

 901  write(hrs_cllgu,*)'Error reading calib coefficients file'
      text='Calib coefficients file could not be read'
      call ml_wt('error',text)
      call aappexit(2)

 902  write(hrs_cllgu,*)'Error closing calib coefficients file'
      text='Calib coefficients file could not be closed' 
      call ml_wt('error',text)
      call aappexit(2)

 200  return
      end
