       subroutine h_interslop(calib,spcntmn,iwtcntmn,iwtrad,
     &                       calibcoef0)
!+
!NOM/NAME $Id: h_interslop.F 42 2006-06-27 13:38:07Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     For each superswath and each channel, this module computes the
!     calibration coefficients: slope (auto coefficient 1), intercept
!     (auto coefficient 0) and third coefficient (auto coefficient 2) 
!     that is equal at zero for the moment. 
!     The coefficients of the visible channel are not measured in flight. 
!     (pre-launch values).    
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0     June 1995       no              T. LABROT
! 1.1     Febr 2005   Modify format of writing to write number of calib
!                     cycle > 10          T. Labrot
! 1.2     June 2006   Check iwtrad not missing     P Marguinaud
!         
!ARGUMENTS 
!  input :
!    integer calib           !total number of full calib cycle for the orbit
!    dble prec iwtrad(hrs_mxirchn,hrs_mxcalib)  
!        IR channel radiances from the IWT scan (derived from the PRT readings)
!    dble prec spcntmn(hrs_mxirchn,hrs_mxcalib) 
!        mean radiant signal output counts in space view scan for each channel
!    dble prec iwtcntmn(hrs_mxirchn,hrs_mxcalib)
!        mean radiant signal output counts in IWT scan for each IR channel
!
!  output:
!    dble prec calibcoef0(hrs_mxchn,3,hrs_mxcalib) 
!                !third auto coefficients, slopes, intercepts for each channel
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "hrsparam.h"
#include "missingval.h"

! argument variables
      integer calib
      double precision iwtrad(hrs_mxirchn,*)
      double precision spcntmn(hrs_mxirchn,*)
      double precision iwtcntmn(hrs_mxirchn,*)
      double precision calibcoef0(hrs_mxchn,3,*)

! local variables
      character*80 text   !variable text
      integer icalib,ich  !index of loops
      double precision slope

!--END OF DECLARATIONS----------------------------------------------------------

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '* SUBROUTINE H_INTERSLOP  *'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'

! Processing of each superswath in the orbit. 
      do icalib=1,calib

       if (iprint.ge.1) write(hrs_cllgu,fmt=500) icalib
       write(text,'("Compute the 3 coefs, calib cycle nb ",I2)') icalib
       call ml_wt('info',text)
      
! Processing for each IR channel
       do ich=1,hrs_mxirchn

        if ((spcntmn(ich,icalib).lt.missdp) .and.
     &            (iwtcntmn(ich,icalib).lt.missdp)) then

         if ( (spcntmn(ich,icalib) - iwtcntmn(ich,icalib)) .eq. 0.) then

          if (iprint.ge.1) write(hrs_cllgu,fmt=510) ich
          write(text,'("Difference in counts is 0 for channel ",I2)')ich
          call ml_wt('warning',text)

          calibcoef0(ich,1,icalib) = missdp
          calibcoef0(ich,2,icalib) = missdp
          calibcoef0(ich,3,icalib) = missdp

         else

          if( iwtrad(ich,icalib) .lt. missdp ) then
           calibcoef0(ich,2,icalib)=iwtrad(ich,icalib) /
     &                        (iwtcntmn(ich,icalib)-spcntmn(ich,icalib))
           calibcoef0(ich,3,icalib)=
     &                   -1*calibcoef0(ich,2,icalib)*spcntmn(ich,icalib)
           calibcoef0(ich,1,icalib)=0.d+0

          else
          
           calibcoef0(ich,2,icalib) = missdp
           calibcoef0(ich,3,icalib) = missdp
           calibcoef0(ich,1,icalib) = missdp

          endif

         endif

        else

          if (iprint.ge.1) write(hrs_cllgu,fmt=520) ich
          write(text,'("At least one outpout counts mean is missing",
     &          " ch ",I2)')ich
          call ml_wt('warning',text)

          calibcoef0(ich,1,icalib) = missdp
          calibcoef0(ich,2,icalib) = missdp
          calibcoef0(ich,3,icalib) = missdp

        endif          

       enddo


! Processing for the visible channel
       calibcoef0(20,2,icalib) = visibcoef(1)
       calibcoef0(20,3,icalib) = visibcoef(2)
       calibcoef0(20,1,icalib) = 0.d+0

       do ich=1,hrs_mxchn
        if (iprint.ge.1) then
          write(hrs_cllgu,fmt=530) ich
          write(hrs_cllgu,fmt=540) calibcoef0(ich,2,icalib)
          write(hrs_cllgu,fmt=550) calibcoef0(ich,3,icalib)
          write(hrs_cllgu,fmt=560) calibcoef0(ich,1,icalib)
        endif  
          
        if(calibcoef0(ich,2,icalib).ge.missdp) then
         slope=9.999999
        write(text,'("  ch:",I2," slope:",F10.6,"   int:",F10.4)')
     &  ich,slope,calibcoef0(ich,3,icalib)
          call ml_wt('info',text)
        else
        write(text,'("  ch:",I2," slope:",F10.6,"   int:",F10.4)')
     &  ich,calibcoef0(ich,2,icalib),calibcoef0(ich,3,icalib)
          call ml_wt('info',text)
        endif        
       enddo 

      enddo

 500  format(/3X,'FULL CALIBRATION CYCLE NUMBER: ',I2)
 510  format(6X,'channel number:',I2,3X,'difference in counts is 0')
 520  format(6X,'channel number:',I2,3X,'at least one radiant ',
     & 'signal outpout counts mean is missing')
 530  format(6X,'channel number:',I2)
 540  format(10X,'slope = ',F20.10)
 550  format(10X,'intercept = ',F20.10)
 560  format(10X,'third coef = ',F20.10)

      return
      end
          
