	subroutine h_iwttmp(calib,iwtlintab,quality, iwttmp)
!+
!NOM/NAME $Id: h_iwttmp.F 46 2006-08-18 14:39:27Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!    For each superswath, this module computes the IWT temperature. 
!
!DESCRIPTION 
!    FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!    
!    For each of the active PRTs, this module computes the mean of a sample of  
!    PRT readings from the internal warm target scan line and from a
!    specified number of scan lines before the IWT scan line and an other    
!    specified number of scan lines after the IWT scan line. 
!    Tests: -the difference between the maximum PRT readings value and the
!            minimum one must be included in a limit prtminmax (cinit.h)
!           -minimum number of valid PRT readings to compute a mean 
!            (prtminnb,cinit.h)
!    After,PRT readings means are converted to temperatures, using the 
!    conversion coefficients(prtcnttmp(4,5), cinit.h).
!    The final IWT temperature is computed by averaging the temperatures from 
!    the individual active PRTs.      
!             
!PROPRIETAIRE/OWNER  
!    Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      June 95      no              T. LABROT
! 1.1     March 03			T. LABROT 
!  f90 test: warning message for line:if (gp_btn(hrs_qualind(ilin),31).eq..true.) then 
!            Correction: if (gp_btn(hrs_qualind(ilin),31)) then    
! 1.2     Feb 05     remove hard-coding parameters
!                    add prtnb, number of PRTs read in calcoef.dat file
!                    taken into account that coefs in calcoef.dat file are
!                    given in dg C and not in deg K         T. LABROT (MF) 
! 1.3   10/06/05     modify the loop (k=3,7 --> 13,17) to get the counts of
!                    PRT nb 5                               T. LABROT (MF)
! 1.4   27/06/06     Bug fix - set iwttmp missing if goodtmp.lt.2  P.Marguinaud
! 1.5   18/08/06     check prtread not zero  NCA (reported by H Murata)
!         
!ARGUMENTS 
!   input :
!    integer calib                  !total number of calib cycle for the orbit
!    integer iwtlintab(hrs_mxcalib) !numbers of IWT scan lines 
!
!   input/output 
!    integer quality(7,hrs_mxlin)   !array of quality information 
!
!   output:
!    dble prec iwttmp(hrs_mxcalib)  !IWT temperatures (one for each calib cycle)
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! logical function gp_btn !give true or false according to the value of the bit
! subroutine h_prtstat    !compute mean,max,min of the PRT data
! subroutine h_prtsum     !compute cumulative sum of valid PRT readings
!    
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------
      implicit none
                                                      
 
!includes/commons
#include "hclin.h"
#include "cinit.h"
#include "hrsparam.h"
#include "hrs1bdts.h"
#include "usablelines.h"
#include "missingval.h"

! argument variables
      integer calib,iwtlintab(*),quality(7,*)
      double precision iwttmp(*)

! local variables
      character*80 text   !variable text
      logical gp_btn      !function
      integer iprt        !index of the PRTs
      integer i,ii        !index of loops
      integer icalib      !index of loop over good calibration cycle
      integer activprt    !at the end of the loop, number of active PRTs 
      integer lin         !number of one IWT scan line in the orbit
      integer ilin        !number of the current scan line (loop)
      integer ilin0,ilin1 !numbers of the first and the last scan lines in the
                          !orbit where we read PRT data
      integer k,k0        !index to read the data    
      integer datanb(prtnb)   !at the end of the loop and for each PRT,total
                          !number of readings,max value=(prtbfore+prtafter+1)*5
      integer goodprtnb   !number of readings used to compute the mean
      integer ejecprtnb   !number of rejected PRT readings for one PRT 
      integer goodprt(prtnb)  !numbers of readings used to compute the mean
      integer ejecprt(prtnb)  !numbers of rejected PRT readings for one PRT 
      integer imindt,imaxdt !index of the min and the max values of PRT readings
      integer goodtmp     !at the end of the loop, number of good temp.
      integer badiwttmp   !number of calib cycle where the IWT temperature
                          !has not been computed

      double precision tmp(prtnb)      !temperature of the 4 PRTs
      double precision dtmean      !mean of readings from one PRT
      double precision dtstd       !standard deviation of readings from one PRT
      double precision prtmean(prtnb)  !means of readings of the 4 PRTs
      double precision prtstd(prtnb)   !standard deviations of readings 
!                                   of the 4 PRTs

      integer*2 prtread(prtnb,5*(prtbfore+prtafter+1))     
                                   !array containing PRT readings 
      integer*2  mindt             !minimum value of PRT readings
      integer*2  maxdt             !maximum value of PRT readings

!--END OF DECLARATIONS----------------------------------------------------------

      do i=1,hrs_mxcalib
        iwttmp(i)=0.d+0
      enddo

      if (iprint.ge.1) write(hrs_cllgu,*)
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'
      if (iprint.ge.1) write(hrs_cllgu,*) '**  SUBROUTINE H_IWTTMP  **'
      if (iprint.ge.2) write(hrs_cllgu,*) '***************************'

! Test about the activity of each PRT.
      activprt = 0
      do iprt=1,prtnb
        if (prtwght(iprt) .eq. 1) then
          activprt = activprt + 1
        else
          text='Not working PRT number'
          call ml_wti('notice',text,iprt)
          if(iprint.ge.1) write(hrs_cllgu,fmt=500) iprt
        endif
      enddo

      if(iprint.ge.1) write(hrs_cllgu,fmt=510) activprt

! For each IWT scan line in a full calibration cycle

      do icalib=1,calib

        lin=iwtlintab(icalib)

        if(iprint.ge.1) write(hrs_cllgu,fmt=520) lin
        text='Compute IWT temp., calib. cycle'
        call ml_wti('info',text,icalib)

! Initialize or reinitialize arrays

        do iprt=1,prtnb
          do ii=1,5*(prtbfore+prtafter+1)
            prtread(iprt,ii)=missint2
          enddo
          datanb(iprt)=0
          prtmean(iprt)=missdp
          prtstd(iprt) =missdp
          ejecprt(iprt)=0
          goodprt(iprt) =0
          tmp(iprt)=0.d+0
        enddo

        dtmean=missdp
        mindt=missint2
        maxdt=missint2
        imindt=0
        imaxdt=0

! Scan lines where we take PRT readings.
        ilin0=max(lin0,(lin-prtbfore))
        ilin1=min((lin+prtafter),lin1)

        if(iprint.ge.2) write(hrs_cllgu,fmt=530) ilin0,ilin1

        do ilin=ilin0,ilin1

! Test: data of this scan line are usable or not by checking bit 31 of the 
!       quality indicator bit field. If not, skip to the next line

          if (gp_btn(hrs_qualind(ilin),31)) then 

            if(iprint.ge.2) write(hrs_cllgu,fmt=540)
            if(iprint.ge.2) write(hrs_cllgu,fmt=550)

          else

! For each PRT
            do iprt=1,prtnb

! Test: control switch indicates that a PRT has been turned off or not.
              if (prtwght(iprt).eq.1) then

! Get the 5 readings by scan line, without test. Count the readings (datanb)
               if (iprt.le.4) then 
                if (iprt.eq.1) k0=3
                if (iprt.eq.2) k0=8
                if (iprt.eq.3) k0=13
                if (iprt.eq.4) k0=18

                do k=k0,(k0+4)
                 datanb(iprt) = datanb(iprt) + 1
                 prtread(iprt,datanb(iprt)) = hrs_elem(k,58,ilin)
                 if (prtread(iprt,datanb(iprt)) .EQ. 0) then
                   prtread(iprt,datanb(iprt)) = missint2
                 endif
                enddo
               endif
               
               if (iprt.eq.5) THEN
                do k=13,17
                 datanb(iprt) = datanb(iprt) + 1
                 prtread(iprt,datanb(iprt)) =hrs_elem(k,59,ilin)
                 if (prtread(iprt,datanb(iprt)) .EQ. 0) then
                   prtread(iprt,datanb(iprt)) = missint2
                 endif
                enddo 
               endif  

              endif

            enddo

          endif

        enddo 
 
        if(iprint.ge.2) then
          write(hrs_cllgu,fmt=560) datanb
          if (prtnb.eq.4) then
            do i=1,5*(ilin1-ilin0+1)
              write(hrs_cllgu,*)'   ',prtread(1,i),'   ',
     &        prtread(2,i),'   ',prtread(3,i),'   ',prtread(4,i)
            enddo
          endif
          if (prtnb.eq.5) then
            do i=1,5*(ilin1-ilin0+1)
              write(hrs_cllgu,*)'   ',prtread(1,i),'   ',
     &        prtread(2,i),'   ',prtread(3,i),'   ',prtread(4,i),
     &        prtread(5,i)
            enddo
          endif
        endif  


! If enough data, compute the mean of the PRT readings, the maximum and minimum 
! values of the readings.
        do iprt=1,prtnb
 
          goodprtnb = 0
          ejecprtnb = 0

          if(iprint.ge.1) write(hrs_cllgu,fmt=570) iprt

          if (datanb(iprt).ge.prtminnb) then

            call h_prtstat(iprt,prtread,datanb, dtmean,dtstd,
     &                     goodprtnb,mindt,maxdt,imindt,imaxdt)
         

            if(iprint.ge.2) then
              write(hrs_cllgu,fmt=580) dtmean,dtstd
              write(hrs_cllgu,fmt=590) maxdt
              write(hrs_cllgu,fmt=600) mindt
            endif

! Test with prtminmax (max difference between max value and min one)
            do while ((maxdt-mindt).gt.prtminmax(iprt).and.
     &                 dtmean.ne.missdp)

              if (iprint.ge.2) then
                write(hrs_cllgu,*)
                write(hrs_cllgu,*) '    "do while loop" '
              endif
 
              ejecprtnb = ejecprtnb + 1 

! Delete the value that is furthest from the mean.
             if ((maxdt-dtmean).gt.(dtmean-mindt)) then
               prtread(iprt,imaxdt)=missint2
             endif
             if ((maxdt-dtmean).lt.(dtmean-mindt)) then
               prtread(iprt,imindt)=missint2
             endif
             if ((maxdt-dtmean).eq.(dtmean-mindt)) then
               prtread(iprt,imaxdt)=missint2
               prtread(iprt,imindt)=missint2
             endif

! Enough filtered PRT readings to compute an average
             if ((goodprtnb-1).ge.prtminnb) then

               if (iprint.ge.2) then
                write(hrs_cllgu,fmt=560) datanb
                if (prtnb.eq.4) then
                  do i=1,5*(ilin1-ilin0+1)
                   write(hrs_cllgu,*)'   ',prtread(1,i),'   ',
     &             prtread(2,i),'   ',prtread(3,i),'   ',prtread(4,i)
                  enddo
                endif
                if (prtnb.eq.5) then
                  do i=1,5*(ilin1-ilin0+1)
                   write(hrs_cllgu,*)'   ',prtread(1,i),'   ',
     &             prtread(2,i),'   ',prtread(3,i),'   ',prtread(4,i),
     &             prtread(5,i)
                  enddo
                endif
               endif

               call h_prtstat(iprt,prtread,datanb,dtmean,dtstd,     
     &                        goodprtnb,mindt,maxdt,imindt,imaxdt)


               if(iprint.ge.2) then
                 write(hrs_cllgu,*)
                 write(hrs_cllgu,fmt=580) dtmean,dtstd
                 write(hrs_cllgu,fmt=590) maxdt
                 write(hrs_cllgu,fmt=600) mindt
               endif


             else
! Not enough filtered PRT readings to compute the mean

               if(iprint.ge.2) write(hrs_cllgu,fmt=610) 
     
               dtmean = missdp

             endif

           enddo

! The final mean and the final standard deviation for PRT number iprt
           prtmean(iprt) = dtmean
           prtstd(iprt)  = dtstd
           goodprt(iprt) = goodprtnb
           ejecprt(iprt) = ejecprtnb
           
      
         else
! This PRT has been turned off or no enough data for this prt 
           if(iprint.ge.2) write(hrs_cllgu,fmt=620) iprt

           prtmean(iprt) = missdp
           prtstd(iprt)  = missdp
           goodprt(iprt) = 0
           ejecprt(iprt) = 0

         endif

         if (iprint.ge.1) then        
           write(hrs_cllgu,fmt=630) prtmean(iprt)
           write(hrs_cllgu,fmt=640) datanb(iprt)
           write(hrs_cllgu,fmt=650) goodprt(iprt)
           write(hrs_cllgu,fmt=660) ejecprt(iprt) 
         endif

! Converte the mean PRT reading counts to temperature.

         if (prtmean(iprt).ne.missdp) then
           do ii=1,6
             tmp(iprt)=tmp(iprt)+
     &              (prtcnttmp(iprt,ii)*(prtmean(iprt)**(ii-1)))
           enddo 
           tmp(iprt)=tmp(iprt)+273.15
         else
           tmp(iprt) = missdp
         endif


         if(iprint.ge.1) write(hrs_cllgu,fmt=670) iprt,tmp(iprt)

       enddo

! Accumulate means and standard deviations to compute final statistical values
!(if requested)

       if (hrs_stflg.eq.1) then
         call h_prtsum(icalib,prtmean,prtstd,4,goodprt,ejecprt)
       endif


! Average the temperatures of active PRTs if at least 2 PRTs have survived
! the different test.
       goodtmp=0
       do iprt=1,prtnb
         if (tmp(iprt) .lt. missdp) then
           iwttmp(icalib) = iwttmp(icalib) + tmp(iprt)
           goodtmp = goodtmp + 1
         endif
       enddo

       if (goodtmp.ge.2) iwttmp(icalib) = iwttmp(icalib)/goodtmp
       if (goodtmp.lt.2) then
         if ( iprint.ge.1 ) write(hrs_cllgu,fmt=680) 
         iwttmp(icalib) = missdp
       endif


       if(iprint.ge.1) write(hrs_cllgu,fmt=690) iwttmp(icalib)
       text= '  IWT temperature:'
       call ml_wtdp('info',text,iwttmp(icalib))
 
      enddo

! Update the  quality parameter
      do icalib=1,calib
        if (iwttmp(icalib).eq.missdp) badiwttmp=badiwttmp+1
      enddo
      if (badiwttmp.eq.calib) then 
        do ilin=lin0,lin1
          quality(4,ilin) = 1
        enddo
      endif

 500  format(/2X,'PRT nb ',I1,' has been turned off')
 510  format(/2X,'total number of active PRTs: ',I1)
 520  format(/2X,'IWT SCAN LINE NUMBER: ',I4)
 530  format(4X,'PRT readings come from line',I4,' to line ',I4)
 540  format(8X,'bit31=1 in the quality indicator bit field')
 550  format(8X,'-- do not use scan for product generation')
 560  format(/4X,'PRT readings total number: ',5(I3))
 570  format(/4X,'PRT NUMBER: ',I1)
 580  format(/9X,'PRT readings mean and standard dev.:',F12.5,3x,D14.7)
 590  format(9X,'PRT readings max value:',I6)
 600  format(9X,'PRT readings min value:',I6)
 610  format(/5X,'not enough filtered PRT readings to',
     & ' compute an average')
 620  format(/5X,'PRT nb ',I1,' not working or no readings')
 630  format(9X,'filtered PRT readings mean: ',F12.5)
 640  format(9X,'initial number of PRT readings: ',I4)
 650  format(9X,'number of filtered PRT readings: ',I4)
 660  format(9X,'number of rejected filtered PRT readings: ',I4)
 670  format(/9X,'temperature of the PRT nb',I2,': ',F12.5)
 680  format(9X,'no enough PRT temperature to compute a mean')
 690  format(//3X,' IWT TEMPERATURE: ',F12.5)

      return
      end
         
