      subroutine h_limit(ich,sp1fv,counts,cntlm,infcnt,supcnt,misscnt)
!+
!NOM/NAME $Id: h_limit.F 25 2006-02-03 15:43:53Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module checks the output counts values that must be 
!     inside limits.  
!     HIRS instruments have contamination in the first xx FOVs of the space
!     scan (8 for the HIRS/2, unknown for the moment for the HIRS/3). The 
!     variable sp1fov (include cinit.h) indicates the first field of view
!     in the space scan that is not contaminated. The same FOVS will be
!     eliminated from the scan of the warm target.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!             
!PROPRIETAIRE/OWNER  
!     Tiphaine LABROT
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment           <author>)
! 1.0      dec. 95       no             T. LABROT
! 1.1      june 96    "warnings" when values are inf, sup or near limits (TL)
! 1.2      jan. 97    correction of errors in the variable "text"  (TL)
! 2.0      august00   Add the calculation of the mean/standard deviation of
!                     output counts out of range to make easier the upgrade of 
!                     calcoef.dat file   (TL)
! 2.1     March 2003  f90 test: Cut lines exceeding 72 characters.   (TL)
! 2.2     Feb 2004    use hrs_mxirchn instead of hard coded 19 (TL)
!         
!ARGUMENTS 
!    input :
!     integer ich          !number of channel
!     integer sp1fv        !the first field of view in the space scan that
!                           is not contaminated.
!     integer cntlm(hrs_mxirchn,2)  !up and low output count limits
!
!    input/output:
!     integer counts(hrs_mxpix)    !for one channel and 56 FOVs output counts
!
!    output :
!     integer infcnt       !counter of counts lower than min limit
!     integer supcnt       !counter of counts greater than max limit
!     integer misscnt      !counter of missing count data
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!      
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      implicit none                                                     
 
 
!includes/commons
#include "hclin.h"
#include "hrsparam.h"
#include "missingval.h"

! argument variables
      integer ich,sp1fv,counts(*),cntlm(hrs_mxirchn,*)
      integer infcnt,supcnt,misscnt

! local variables
      integer i               !index of loop
      character*80 text       !variable text
      integer diffmin,diffmax !counter when CN values are near limits
      integer countsinf(hrs_mxpix)  !array of output counts inferior to the min limit
      integer countssup(hrs_mxpix)  !array of output counts superior to the max limit
      integer countsinfmin          !min value of countsinf(hrs_mxpix)
      integer countssupmax          !max value of countssup(hrs_mxpix)
      double precision countsinfmn  !mean of output counts inferior to the min limit
      double precision countsinfstd !standard deviation of output counts inferior to the min limit
      double precision countssupmn  !mean of output counts superior to the max limit
      double precision countssupstd !standard deviation of output counts superior to the max limit
!--END OF DECLARATIONS---------------------------------------------------------

      infcnt = 0
      supcnt = 0
      misscnt= 0
      diffmin=0
      diffmax=0
      countsinfmin=missint
      countssupmax=-missint
      countsinfmn =0.d+0
      countsinfstd=0.d+0
      countssupmn =0.d+0
      countssupstd=0.d+0
      do i=1,hrs_mxpix
       countsinf(i)=0
       countssup(i)=0
      enddo 
      
      
      if (iprint.ge.2) write(hrs_cllgu,*)
     & '      *subroutine H_LIMIT*'

      do i=sp1fv,hrs_mxpix

        if (counts(i).lt.missint) then

          if (counts(i).lt.cntlm(ich,1)) then
            if (iprint.ge.1) write(hrs_cllgu,fmt=500) counts(i),i
            infcnt = infcnt + 1
            countsinf(infcnt)=counts(i)
            counts(i) = missint
          else
            
            if(counts(i).lt.cntlm(ich,1)+5) diffmin=diffmin+1
           
            if (counts(i).gt.cntlm(ich,2)) then
             if (iprint.ge.1) write(hrs_cllgu,fmt=510) counts(i),i
             supcnt = supcnt + 1
             countssup(supcnt)=counts(i)
             counts(i) = missint
            else
             if(counts(i).gt.(cntlm(ich,2)-5)) diffmax=diffmax+1
            endif
            
          endif
           
        else

          misscnt =  misscnt + 1

        endif

      enddo
      
      if(infcnt.gt.0) then
      
        do i=1,infcnt
         countsinfmn  = countsinfmn  + countsinf(i)
         countsinfstd = countsinfstd + countsinf(i)*countsinf(i)
         if (countsinf(i).lt.countsinfmin) countsinfmin=countsinf(i)
        enddo
        countsinfmn  = countsinfmn/infcnt
        countsinfstd = (countsinfstd/infcnt)-(countsinfmn**2)
        if (countsinfstd .ge. 0.d+0) countsinfstd=dsqrt(countsinfstd)
        
        write(text,'("ch",I2,": ",I2," CN values are inf to limit")')
     &   ich,infcnt
        call ml_wt('warning',text)
        write(text,'("  min:",I7,"  counts mean/stdev:",2F11.2)')
     &   countsinfmin,countsinfmn,countsinfstd
        call ml_wt('warning',text)
        write(text,'("  Perhaps min value of CN limits ",I6,
     &  " must be changed")') cntlm(ich,1)   
        call ml_wt('warning',text)
        
      endif
      
      if(supcnt.gt.0) then 
      
        do i=1,supcnt
         countssupmn  = countssupmn  + countssup(i)
         countssupstd = countssupstd + countssup(i)*countssup(i)
         if (countssup(i).gt.countssupmax) countssupmax=countssup(i)
        enddo
        countssupmn  = countssupmn/supcnt
        countssupstd = (countssupstd/supcnt)-(countssupmn**2)
        if ( countssupstd .ge. 0.d+0) countssupstd=dsqrt(countssupstd)
        
        write(text,'("ch",I2,": ",I2," CN values are sup to limit")')
     &   ich,supcnt
        call ml_wt('warning',text)
        write(text,'("  max:",I7,"  counts mean/stdev:",2F11.2)')
     &   countssupmax,countssupmn,countssupstd
        call ml_wt('warning',text)
        write(text,'("  Perhaps max value of CN limits ",I6,
     &  " must be changed")') cntlm(ich,2)   
        call ml_wt('warning',text)        
  
      endif
      
      if(misscnt.gt.0) then 
        write(text,'("ch",I2,": ",I2," CN values are missing")')
     &   ich,misscnt
        call ml_wt('warning',text)
      endif
      
      if(diffmin.gt.0) then
        write(text,'("ch",I2," difference CNTvalue - minlimit",
     &  " < 5 : lim",I6,"   nb ",I2)') ich,cntlm(ich,1),diffmin
        call ml_wt('warning',text)
        text='Perhaps min value of CN limits must be changed'
        call ml_wt('warning',text)
      endif
      
      if(diffmax.gt.0) then
        write(text,'("ch",I2," difference maxlimit - CNTvalue",
     &  " < 5 : lim",I6,"   nb ",I2)') ich,cntlm(ich,2),diffmax
        call ml_wt('warning',text)
        text='Perhaps max value of CN limits must be changed'
        call ml_wt('warning',text)
      endif
                  

      if (iprint.ge.2) write(hrs_cllgu,*)
     &  '      *end subroutine H_LIMIT*'


 500   format(12X,'count lower than min limit:',I6,4X,'ifov:',I2)
 510   format(12X,'count greater than max limit:',I6,4X,'ifov:',I2)

       return
       end
