       SUBROUTINE h_BBinterslop_algoV4(iwtrad)

!------------------------------------------------------------------------------
!NAME $Id: h_BBinterslop_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE 
!     For each calib cycle and each channel, this module computes the
!     Black Body (BB) (or raw) slope (auto coefficient 1), the BB (or raw)
!     intercept (auto coefficient 0) and third coefficient (auto coefficient 2)
!     that is equal at zero for the moment. 
!     The coefficients of the visible channel are not measured in flight. 
!     (pre-launch values).
!
!DESCRIPTION
!     This module is very similar to the module h_interslop.
!     It was created to manage the HIRS calibration algorithm version 4.0 
!
!     Assuming that radiance can be related to counts through the 
!     quadratic equation:
!        Rad = a0 + a1*Count_mean + a2*Count_mean*Count_mean
!        with a0,a1,a2 the 3 auto calibration coefficients
!     Assuming  a2=0
!     For the space: Rad_space= a0 + a1*Count_mean_space
!     For the space: Rad_bb   = a0 + a1*Count_mean_bb
!     Assuming Rad_space=0
!     Then
!     a1=BBslope = (Rad_bb)/(Count_mean_bb - Count_mean_space)
!     a0=BBintercept = - BBslope*Count_mean_space
!
!     See also NOAA-KLM User'Guide Section 7.2
!
!     If one element of the equation is missing, then BBslope and BBintercept
!     are not computed. The calibration cycle for the given channel is not 
!     qualified for further processing.     
!     
!HISTORY (CHANGES) 
!     (version   date         comment           <author>)
!      1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.              
!         
!ARGUMENTS 
!  inputs :
!    DOUBLE PRECISION iwtrad(hrs_mxirchn,hrs_mxcalib)  
!        IR channel radiances from the IWT scan (derived from the PRT readings)
!
!  outputs:
!
!UPDATED ELEMENTS OF INCLUDES
!
!  in hcalibcycleinfo_algoV4.h
!      output DBLE PREC calibcoef0(hrs_mxchn,3,hrs_mxcalib)
!              ! Black Body(BB) third coefficient,BB slope, BB intercept
!              ! = raw third coefficient,raw slope, raw intercept
!              ! for each of the 20 channels
!
!  in include hcalibcycleinfo_algoV4.h
!      input/ouput: LOGICAL qualified(hrs_mxchn,hrs_mxcalib) 
!                !=true if the calib cycle is usable
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"

! argument variables
      DOUBLE PRECISION  iwtrad(hrs_mxirchn,*)

! local variables
      CHARACTER*80 text     !variable text
      INTEGER icalib,ich    !index of loops

!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '*SUB H_BBINTERSLOP_ALGOV4 *'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      
      text='Computes the BB coefs for each calib cycle'
      CALL ml_wt('info',text)

! Processing of each calib cycle in the orbit. 
      DO icalib=1,calib
      
! Processing for each IR channel
       DO ich=1,hrs_mxirchn
       
       IF (qualified(ich,icalib)) THEN

        IF ((spcntmn(ich,icalib).NE.missdp) .AND.
     &            (iwtcntmn(ich,icalib).NE.missdp)) THEN

         IF ( (spcntmn(ich,icalib) - iwtcntmn(ich,icalib)) .NE. 0.) THEN

          
          calibcoef0(ich,2,icalib)=iwtrad(ich,icalib) /
     &                        (iwtcntmn(ich,icalib)-spcntmn(ich,icalib))
          calibcoef0(ich,3,icalib)=
     &                   -1*calibcoef0(ich,2,icalib)*spcntmn(ich,icalib)
          calibcoef0(ich,1,icalib)=0.d+0
          
         ELSE

          calibcoef0(ich,1,icalib) = missdp
          calibcoef0(ich,2,icalib) = missdp
          calibcoef0(ich,3,icalib) = missdp
          
          qualified(ich,icalib) = .FALSE.
          
          IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=500)icalib,ich
          IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=510)
          
          WRITE(text,'("Calib cycle ",I2," channel ",I2,
     &     ": Difference in counts means is 0")')
     &     icalib,ich
          CALL ml_wt('warning',text)
          WRITE(text,'("==>No qualified calib cycle  ",
     &     "==>No BBslope computed")')
          CALL ml_wt('warning',text)        

         ENDIF  ! IF ( (spcntmn(ich,icalib) - iwtcntmn(ich,icalib)) .NE. 0.) THEN 

        ELSE    ! IF ((spcntmn(ich,icalib).NE.missdp) .AND.
!               ! (iwtcntmn(ich,icalib).NE.missdp)) THEN
        
          calibcoef0(ich,1,icalib) = missdp
          calibcoef0(ich,2,icalib) = missdp
          calibcoef0(ich,3,icalib) = missdp
          
          qualified(ich,icalib) = .FALSE.
        
          IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=520)icalib,ich
          IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=510)
          
          WRITE(text,'("Calib cycle ",I2," channel ",I2,
     &    ": At least one outpout counts mean is missing")')
     &     icalib,ich
          CALL ml_wt('warning',text)
          WRITE(text,'("==>No qualified calib cycle  ",
     &       "==>No BBslope computed")')
          CALL ml_wt('warning',text)

        ENDIF   ! IF ((spcntmn(ich,icalib).NE.missdp) .AND.
!               ! (iwtcntmn(ich,icalib).NE.missdp)) 
        
        ELSE    ! IF (qualified(ich,icalib)) THEN
        
           calibcoef0(ich,1,icalib) = missdp
           calibcoef0(ich,2,icalib) = missdp
           calibcoef0(ich,3,icalib) = missdp
        
           IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=530)icalib,ich
           
           WRITE(text,'("No qualified calib cycle ",I2,
     &       " for channel ",I2," no BBslope computed")')icalib,ich
           CALL ml_wt('warning',text)        
           
        ENDIF   ! IF (qualified(ich,icalib)) THEN  

       ENDDO    ! DO ich=1,hrs_mxirchn


! Processing for the visible channel
       calibcoef0(20,2,icalib) = visibcoef(1)
       calibcoef0(20,3,icalib) = visibcoef(2)
       calibcoef0(20,1,icalib) = 0.d+0
       
! Printing for control        
       IF (iprint.GE.2) THEN
       
         DO ich=1,hrs_mxchn
       
          WRITE(hrs_cllgu,fmt=540) icalib,ich
          WRITE(hrs_cllgu,fmt=550) calibcoef0(ich,2,icalib),
     &          calibcoef0(ich,3,icalib),calibcoef0(ich,1,icalib)
        
         ENDDO
         
       ENDIF  
                         

      ENDDO     ! DO icalib=1,calib

 500  FORMAT(2X,'calib: ',I2,'  channel ',I2,3X,
     & 'difference in counts means is 0')
 510  FORMAT(6X,'==>No qualified calib cycle  ',
     &          '==>No BBslope computed')   
 520  FORMAT(2X,'calib: ',I2,'channel ',I2,3X,'at least one outpout ',
     & 'counts mean is missing')
 530  FORMAT(2X,'No qualified calib cycle ',I2,
     &           ' for channel ',I2,' no BBslope computed')     
 540  FORMAT(6X,'CALIB: ',I2,'  CHANNEL: ',I2)
 550  FORMAT(3X,'slope intercept third coef : ',F16.9,3X,
     & F14.7,3X,F4.2 )

 
      RETURN
      END
          
