       SUBROUTINE h_BBslopcontrol_algoV4

!------------------------------------------------------------------------------
!NAME $Id: h_BBslopcontrol_algoV4.F 72 2007-04-27 08:21:59Z frna $
!
!PURPOSE 
!    This module controls the quality of the BBslopes (or raw slopes).
!
!DESCRIPTION
!    First, the BBslopes (or raw slopes) are checked to ensure that their values are 
!    within a threshold from their mean value. The threshold is defined in
!    the h_testcoef_algoV4.dat file and after in the include hcinit_algoV4.h.
!    If not, the one furthest from the mean value is removed and the mean is
!    recomputed. This process continues till the condition is satisfied.
!
!    Then, the slopes are checked against the average slope (read in the 
!    hirs_b1aslope.txt). If it deviates from the average slope by more than 
!    a threshold for a channel, it is considered an anomaly. 
!
!    If the slope for a channel is anomalous, the intercept and the other
!    coefficient (quadratic one) are also suspect.
!    The calib cycle is not qualified.
!
!    Reference:
!    HIRS calibration Algorithm Version 4.0 (Nov 3, 2003)
!    C. Cao NESDIS     
!     
!HISTORY (CHANGES) 
!    (version   date         comment           <author>)
!    1.0     January 05     Creation          T.Labrot (MF)
!    1.1     13 June 05     Bug on one threshold computation (ABS)
!                           Add test on imaxdiff to avoid a bad dimension
!                           of the array.     T.Labrot and P.Brunel (MF)
!    1.2     25 April 07    Added and modified 'write' commands into the log 
!                           file T.Labrot (MF)  
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.              
!         
!ARGUMENTS 
!    inputs :
!
!    outputs:
!
!UPDATED ELEMENTS OF INCLUDES
!
!    in hcalibcycleinfo_algoV4.h
!      input/output DBLE PREC calibcoef0(hrs_mxchn,3,hrs_mxcalib)
!              ! Black Body(BB) third coefficient,BB slope, BB intercept
!              ! = raw third coefficient,raw slope, raw intercept
!              ! for each of the 20 channels
!
!    in include hcalibcycleinfo_algoV4.h
!      input/ouput: LOGICAL qualified(hrs_mxchn,hrs_mxcalib) 
!              !=true if the calib cycle is usable
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hb1averageslope_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"

! argument variables

! local variables
      CHARACTER*80 text     !variable text
      INTEGER icalib,ich    !index of loops
      INTEGER goodBBslope,badBBslope  !number of good/bad calib cycles
      INTEGER imaxdiff      !index of the calib cycle where the difference 
                            !between the BBslope and the mean is max                     
      DOUBLE PRECISION maxdiff    !max difference between the BBslope and the mean
      DOUBLE PRECISION BBslopemn  !mean of the BB slopes
      DOUBLE PRECISION threshold  !threshold

!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '*S. H_BBSLOPCONTROL_ALGOV4*'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      
      text='Controls the BBslopes of the calib cycles'
      CALL ml_wt('info',text)
      
      
! For each channel, computes the mean of the BBslopes on the total
! number of calibration cycles
      
      DO ich=1,hrs_mxirchn
      
        IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=500) ich
        
        BBslopemn  = 0.d+0
        goodBBslope=0
         
        DO icalib=1,calib
          IF (calibcoef0(ich,2,icalib).NE.missdp) THEN
           BBslopemn = BBslopemn+calibcoef0(ich,2,icalib)
           goodBBslope=goodBBslope+1
          ENDIF
        ENDDO

        IF (goodBBslope.NE.0) THEN
          BBslopemn=BBslopemn/goodBBslope
        ELSE
          BBslopemn=missdp
        ENDIF 
        
        IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=510) BBslopemn,goodBBslope

! Checks each BBslope to ensure that its value is within a threshold
! from their mean value 

         badBBslope=0
         IF (BBslopemn.NE.missdp) THEN
           threshold=ABS(BBslopemn*BBslopethreshold/100.)
         ELSE
           threshold=0.
         ENDIF
         
         IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=520)
     &   BBslopethreshold,threshold
            
         DO icalib=1,calib
         
           IF (calibcoef0(ich,2,icalib).NE.missdp .AND.
     &         threshold.NE.0.) THEN
         
             IF (ABS(calibcoef0(ich,2,icalib)-BBslopemn)
     &           .GT.threshold) THEN
               badBBslope = badBBslope + 1
               IF (iprint.GE.2)
     &         WRITE(hrs_cllgu,530) icalib,
     &         calibcoef0(ich,2,icalib), badBBslope 
             
             ENDIF
             
           ENDIF
             
         ENDDO
         
         IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=540) badBBslope         
         
! If all the BBslopes are not within a threshold from their mean value, 
! the one furthest from the mean value is removed
         
         DO WHILE (badBBslope.NE.0 .AND. BBslopemn.NE.missdp)
         
         IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=550) 
 
           maxdiff =0.d+0
           imaxdiff=0
           
           DO icalib=1,calib
             
             IF ( calibcoef0(ich,2,icalib).NE.missdp .AND.
     &           BBslopemn.NE.missdp ) THEN
              
               IF ( (ABS(calibcoef0(ich,2,icalib)-BBslopemn))
     &             .GT. maxdiff ) THEN
                            
                maxdiff =ABS(calibcoef0(ich,2,icalib)-BBslopemn)
                imaxdiff= icalib
  
               ENDIF
             
             ENDIF 
             
           ENDDO 
           
           WRITE(hrs_cllgu,560) imaxdiff,maxdiff          
           
           IF (imaxdiff.GT.0) THEN
             calibcoef0(ich,2,imaxdiff) = missdp 
             calibcoef0(ich,1,imaxdiff) = missdp
             calibcoef0(ich,3,imaxdiff) = missdp
             qualified(ich,imaxdiff)=.FALSE. 
           ENDIF  
             
! Recomputes the mean.               

           IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=570)
           
           BBslopemn = 0.d+0                 
           goodBBslope=0

           DO icalib=1,calib           
             IF (calibcoef0(ich,2,icalib).NE.missdp) THEN
               BBslopemn = BBslopemn+calibcoef0(ich,2,icalib)
               goodBBslope=goodBBslope+1
             ENDIF             
           ENDDO

           IF (goodBBslope.NE.0) THEN
             BBslopemn=BBslopemn/goodBBslope
           ELSE
             BBslopemn =missdp
           ENDIF
           
           IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=510) 
     &           BBslopemn,goodBBslope
             
! Checks each BBslope to ensure that its value is within a threshold
! from their mean value 

           badBBslope=0
           IF (BBslopemn.NE.missdp) THEN
             threshold=ABS(BBslopemn*BBslopethreshold/100.)
           ELSE
             threshold=0.
           ENDIF
           
           IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=520)
     &   BBslopethreshold,threshold
     
           DO icalib=1,calib
         
            IF (calibcoef0(ich,2,icalib).NE.missdp .AND.
     &          threshold.NE.0.) THEN
         
             IF (ABS(calibcoef0(ich,2,icalib)-BBslopemn)
     &           .GT.threshold) THEN
               badBBslope = badBBslope + 1
               IF (iprint.GE.2)
     &         WRITE(hrs_cllgu,530) icalib,
     &         calibcoef0(ich,2,icalib), badBBslope 
             ENDIF
             
            ENDIF
             
           ENDDO
           
           IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=540) badBBslope
           
         ENDDO          !DO WHILE (badBBslope.NE.0 .AND. BBslopemn.NE.missdp)      

        
! Checks the BBslopes against the average slope(historic file).
! If it deviates from the average slope value by more than
! a threshold, then the mean is considered an anomaly.
! The average slope will be used and the BBintercept is also suspect. 
         IF (avgslope(ich).NE.0. .AND. avgslope(ich).NE.missreal ) THEN
         
          threshold=ABS(avgslope(ich)*avgslopethreshold/100.)
         
          IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=580)
     &    avgslopethreshold,threshold,avgslope(ich)
     
          DO icalib=1,calib
         
           IF (calibcoef0(ich,2,icalib).NE.missdp) THEN
           
             IF (ABS(calibcoef0(ich,2,icalib)-avgslope(ich))
     &               .GT.threshold) THEN 
! From what it is witten in the C. Cao's paper, 
! calibcoef0(ich,2,icalib)=avgslope(ich) in this case.
! I prefer to have  calibcoef0(ich,2,icalib)=missdp to avoid to test
! if avgslope is equal to 0 or is missing.
! This has no effect for the processing because we have 
! qualified(ich,icalib)=.FALSE., so this calib cycle will not be used.    
     
               calibcoef0(ich,2,icalib)=missdp
               calibcoef0(ich,1,icalib)=missdp
               calibcoef0(ich,3,icalib)=missdp
               qualified(ich,icalib)=.FALSE.
               
             ENDIF 
             
           ENDIF
           
          ENDDO
          
         ELSE
           
           IF (iprint.GE.1) WRITE(hrs_cllgu,
     &           '("    no control with avgslope")') 
           IF (iprint.GE.1) WRITE(hrs_cllgu,'("    avgslope missing")') 
         
         ENDIF  
                           
      ENDDO             !DO ich=1,hrs_mxirchn

! Printing for control        
      IF (iprint.GE.2) THEN
        WRITE(hrs_cllgu,*)
        DO icalib=1,calib       
          DO ich=1,hrs_mxchn       
           WRITE(hrs_cllgu,fmt=590) icalib,ich
           WRITE(hrs_cllgu,fmt=600) calibcoef0(ich,2,icalib),
     &           calibcoef0(ich,3,icalib),calibcoef0(ich,1,icalib)        
          ENDDO          
        ENDDO          
      ENDIF 

 500  FORMAT(2X,'CHANNEL : ',I2)
 510  FORMAT(4X,'value of the mean of the BBslopes:',F12.6,
     &'  nb_goodcalib_used:',I4)
 520  FORMAT(4X,'values of the thresholds : ',I2,F15.10)
 530  FORMAT(4X,'bad slope: icalib,slope,badBBslope:',I4,F15.10,I4)
 540  FORMAT(4X,'value of badBBslope ',I2)
 550  FORMAT(4X,'in DO WHILE loop')
 560  FORMAT(4X,'removed slope: icalib,diff:',I4,F15.10)
 570  FORMAT(4X,'recompute the mean')
 580  FORMAT(4X,'values of the thresholds and avgslope : ',I2,2F20.10) 
 590  FORMAT(2X,'CALIB: ',I2,'  CHANNEL: ',I2)
 600  FORMAT(2X,'slope intercept third coef : ',F16.9,3X,
     & F14.7,3X,F4.2 )
 
     
      RETURN
      END
          
