      SUBROUTINE h_calcb1_algoV4(
     &     iprint, 
     &     file_lu,
     &     log_lu, 
     &     past_hours,
     &     obs_date,
     &     b0n,
     &     b1n,
     &     b0d,
     &     b1d,
     &     b0,
     &     b1,
     &     avgslope,
     &     stdevavgslope)
     
!------------------------------------------------------------------------------
!NAME $Id: h_calcb1_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE      
!   Calculates the B0/B1 coef taking into account the "pasthours"
!   data stored in the historical file opened on unit file_lu
!   Calculates the average slope and its standard deviation
!   over the same period.
!
!   Output is separated by Night and Day (b1n, b1d) and
!   all conditions (b1)
!   If any problem the B1 are set to 0. which will make
!   no use of these values
!
!DESCRIPTION
! References
!   HIRS calibration Algorithm Version 4.0 (Nov 3, 2003)
!   C. Cao NESDIS
!   NOAA-KLM User'Guide Section 7.2
!
!HISTORY (CHANGES) 
!(version   date         comment                                 <author>)
! 1.0     January 05     creation                             P. Brunel (MF)
! 2.0     Mar 05         Add the standard deviation           T. Labrot (MF)
!                        of the average slope 
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.              
!         
!ARGUMENTS 
!  inputs :
!     Integer iprint     ! Print option
!     Integer file_lu    ! Logical unit of historical file
!     Integer log_lu     ! Logical unit of log file
!     Integer past_hours ! number of considered hours preceeding data date
!     Double Precision obs_date ! Julian day of current observation
!
!  outputs:
!     Real b0(hrs_mxirchn) ! BO values for all conditions
!     Real b1(hrs_mxirchn) ! B1 values for all conditions
!     Real b0n(hrs_mxirchn) ! BO values for night conditions
!     Real b1n(hrs_mxirchn) ! B1 values for night conditions
!     Real b0d(hrs_mxirchn) ! B0 values for day light conditions
!     Real b1d(hrs_mxirchn) ! B1 values for day light conditions
!     Real avgslope(hrs_mxirchn) ! average slope
!     Real stdevavgslope(hrs_mxirchn) ! standard deviation of the average slope
! 
!UPDATED ELEMENTS OF INCLUDES
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!   
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------
  
      Implicit None
      
!includes/commons      
#include "hrsparam_algoV4.h"
#include "hcalibhisto_algoV4.h"

! argument variables
      Integer iprint     ! Print option
      Integer file_lu    ! Logical unit of historical file
      Integer log_lu     ! Logical unit of log file
      Integer past_hours ! number of considered hours preceeding data date
      Double Precision obs_date ! Julian day of current observation

      Real b0(hrs_mxirchn)  ! BO values for all conditions
      Real b1(hrs_mxirchn)  ! B1 values for all conditions
      Real b0n(hrs_mxirchn) ! BO values for night conditions
      Real b1n(hrs_mxirchn) ! B1 values for night conditions
      Real b0d(hrs_mxirchn) ! B0 values for day light conditions
      Real b1d(hrs_mxirchn) ! B1 values for day light conditions
      Real avgslope(hrs_mxirchn) ! average slope
      Real stdevavgslope(hrs_mxirchn) ! standard deviation of the average slope
      
! local variables 
      Integer k        ! current cycle number
      Integer ic       ! channel loop indice
      Integer ncycles  ! total number of cycles stored
      Integer n        ! total valid cycles
      Integer nn       ! total night valid cycles
      Integer nd       ! total day valid cycles

      Integer errStatus   ! I/O return code

      Real ra, rn, rd

      Double Precision a000(hrs_mxcalibhisto)
      Double Precision a000d(hrs_mxcalibhisto)
      Double Precision a000n(hrs_mxcalibhisto)
      Double Precision temp(hrs_mxcalibhisto)
      Double Precision tempd(hrs_mxcalibhisto)
      Double Precision tempn(hrs_mxcalibhisto)
      Double Precision slope(hrs_mxcalibhisto)

      Double Precision a,b,r
      Double Precision moy, sig, rms, min, max

      Double Precision past_days, dij
      Character*22 cht

      Logical found
      
!--END OF DECLARATIONS----------------------------------------------------------      
 
      If (iprint.Ge.1) Then
         Write(log_lu,*) 
         Write(log_lu,*) '**  h_calculate_b1   **'
         Write(log_lu,*) 'number of past hours ',past_hours
      End If
      
      past_days = past_hours / 24.

      !Rewind the input file
      Rewind(file_lu)

      ! store data in index k
      k = 1
      Call h_read_histo_algoV4(iprint, file_lu, log_lu, k, errStatus)

      found = .False.
      Do While( errStatus .Eq. 0 ) 
         dij =  obs_date - ijsp(k)
         Call xijc(ijsp(k),cht)
    
         ! reject if ijdate outside [obs_date - past_hours , obs_date]
         If ( dij .Le. past_days .And. dij .Gt. 0.) Then
            ! OK keep data
            found = .True.
            If (iprint.Ge.1) Then
               Write(log_lu,*) 'store ',k,' ', cht, orbnum(k),
     &                          kcal(k), totcal(k), solzen(k)
            End If
            ! next index
            k = k+1
         Else
            If (iprint.Ge.1) Then
               Write(log_lu,*) 'outside date range ',k, cht
            End If
         End If
         Call h_read_histo_algoV4(iprint, file_lu, log_lu, k, errStatus)
      End Do


      ! All data stored, number of good data is k-1
      If( found ) Then
         ncycles = k - 1

         If (iprint.Ge.1) Then
            Write(log_lu,*) 'number of stored cycles ',ncycles
         End If

         Do ic = 1, hrs_mxirchn
            n  = 0
            nd = 0
            nn = 0

            b0(ic)  = 0.
            b1(ic)  = 0.
            b0n(ic) = 0.
            b1n(ic) = 0.
            b0d(ic) = 0.
            b1d(ic) = 0.
            avgslope(ic) = 0.

            Do k = 1, ncycles  
               If( qualified(ic,k) ) Then
                  If( solzen(k) .Lt. 90 ) Then
                    ! day light condition at nadir
                     nd = nd +1
                     a000d(nd) = calibcoef0(ic,3,k)
                     tempd(nd) = stt(k)
                  Else
                    ! night condition at nadir
                     nn = nn + 1
                     a000n(nn) = calibcoef0(ic,3,k)
                     tempn(nn) = stt(k)
                  End If

                  n = n +1
                  a000(n)  = calibcoef0(ic,3,k)
                  temp(n)  = stt(k)
                  slope(n) = calibcoef0(ic,2,k)
               End If
            End Do              ! k

            If( n .Gt.2) Then
               Call moy_rms(temp, n, moy, sig, rms, min, max)
               If( sig .Gt. 1.e-06) Then
                  ! calculate only if enough variability in STT
                  Call reglin(temp, a000, n, a, b, r)
                  b1(ic) = a
                  b0(ic) = b
                  ra     = r
               End If
            End If

            If( nn .Gt.2) Then
               Call moy_rms(tempn, nn, moy, sig, rms, min, max)
               If( sig .Gt. 1.e-06) Then
                  ! calculate only if enough variability in STT
                  Call reglin(tempn, a000n, nn,  a, b, r)
                  b1n(ic) = a
                  b0n(ic) = b
                  rn      = r
               End If
            End If

            If( nd .Gt.2) Then
               Call moy_rms(tempd, nd, moy, sig, rms, min, max)
               If( sig .Gt. 1.e-06) Then
                  ! calculate only if enough variability in STT
                  Call reglin(tempd, a000d, nd,  a, b, r)
                  b1d(ic) = a
                  b0d(ic) = b
                  rd      = r
               End If
            End If

            Call moy_rms(slope, n, moy, sig, rms, min, max)
            avgslope(ic) = moy
            stdevavgslope(ic) = sig

            If ( iprint .Ge. 1 ) Then
               Write(log_lu,*) ic,'  b0 b1 r all      ',
     &              b0(ic), b1(ic), ra
               Write(log_lu,*) ic,'  b0 b1 r night    ', 
     &              b0n(ic), b1n(ic), rn
               Write(log_lu,*) ic,'  b0 b1 r day      ', 
     &              b0d(ic), b1d(ic), rd
               Write(log_lu,*) ic,'  avg slope  stdev ', 
     &              avgslope(ic),stdevavgslope(ic)
            End If

         End Do ! ic
      Else ! not found 
         Do ic = 1, hrs_mxirchn
            b0(ic)  = 0.
            b1(ic)  = 0.
            b0n(ic) = 0.
            b1n(ic) = 0.
            b0d(ic) = 0.
            b1d(ic) = 0.
            avgslope(ic) = 0.
            stdevavgslope(ic) = 0.
         End Do
      Endif ! if found

      Return
      End
      
      
          Subroutine REGLIN(X,Y,n,a,b,r)
!
!=============================================[ documentation       ]===
!---------------------------------------------[ identification      ]---
!     auteur            : Gerard LEGENDRE
!     date de creation  : 19/02/88
!---------------------------------------------[ role                ]---
!     calcul la regression lineaire  Y=a*X+b
!     r=coefficient de correlation
!---------------------------------------------[ domaine de validite ]---
!     Aucune restriction
!---------------------------------------------[ modifications       ]---
!=============================================[ declarations        ]===
      Implicit None
!---------------------------------------------[ arguments           ]---
       Integer n
       Double Precision X(n)
       Double Precision Y(n)
       Double Precision a
       Double Precision b
       Double Precision r
!---------------------------------------------[ variables locales   ]---
       Double Precision SigmaX,MoyenneX,SigmaY,MoyenneY,Produit
       Integer i
!=============================================[ instructions        ]===
       If(n.Gt.1) Then

          SigmaX  =0.0D0
          MoyenneX=0.0D0
          SigmaY  =0.0D0
          MoyenneY=0.0D0
          Produit =0.0D0

          Do 10 i=1,n
             SigmaX=SigmaX+((X(i)-MoyenneX)**2)/i
             SigmaX=(i-1)*SigmaX/i
             MoyenneX=(i-1)*MoyenneX+X(i)
             MoyenneX=MoyenneX/i

             SigmaY=SigmaY+((Y(i)-MoyenneY)**2)/i
             SigmaY=(i-1)*SigmaY/i
             MoyenneY=(i-1)*MoyenneY+Y(i)
             MoyenneY=MoyenneY/i

             Produit=Produit+X(i)*Y(i)
10        Continue

          a=(Produit/n-MoyenneX*MoyenneY)/SigmaX
          b=MoyenneY-a*MoyenneX
          r=(a**2)*SigmaX/SigmaY
          r=dsqrt(r)

       Else
          a=0.D0
          b=Y(1)
          r=1.0D0
       Endif
       Return
!=============================================[ formats             ]===
      End

           
      Subroutine moy_rms(x, n, moy, sig, rms, min, max)
!+
!
!BUT/PURPOSE 
!       computation od mean and standard deviations on the x's.
!DESCRIPTION 
!
!
!PROPRIETAIRE/OWNER
!       P. BRUNEL
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   June 2001     no               P. BRUNEL
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France
      Implicit None     
!
!ARGUMENTS 
        Integer n               ! number of input values
        Double Precision x(n)           ! values on which statistics are done
        Double Precision moy            ! mean
        Double Precision sig            ! standard deviation
        Double Precision rms            ! root mean square
        Double Precision min            ! minimum value
        Double Precision max            ! maximum value
!
!DECLARATIONS
!
! Common declarations:
!
! Local parameters:
!
! Commons
!
! Functions
!
! Local variables:
        Integer i
! 
!-
!
        sig = 0.d0
        moy = 0.d0
        rms = 0.d0
        min  = 999.e10
        max  = -999.e10
        If(n.Le.0) Return

        Do i=1,n
           moy = moy+x(i)
           If(x(i) .Gt. max) Then
              max = x(i)
           Endif
           If(x(i) .Lt. min) Then
              min = x(i)
           Endif
        End Do
        moy = moy/n

        Do i=1,n
           sig = sig+(x(i)-moy)**2
        End Do
        sig = dsqrt(sig/n)

        Do i=1,n
           rms = rms + (x(i))**2
        End Do
        rms = dsqrt(rms/n)

        Return

        End
