        SUBROUTINE h_calibcoeffile_algoV4(satid)

!------------------------------------------------------------------------------
!NAME $Id: h_calibcoeffile_algoV4.F 120 2008-06-27 09:32:19Z frna $
!
!PURPOSE 
!    This module opens/closes and reads the calibration coefficients file. 
!    Data are stored in common hrs_clcf (include hcinit_algoV4.h).
!    The number of coefficients can vary from one satellite to another.
!    Coefficients are always ranged into arrays with the same size and
!    when coefficients don't exist, the values are put to zero.
!
!DESCRIPTION
!    This module is very similar to the module h_calibcoeffile.
!    It was created to manage the HIRS calibration algorithm version4.0
!
!HISTORY (CHANGES) 
!    (version    date          comment           <author>)
!       1.0     January 05     creation          T. Labrot (MF)
!       1.1     May 07         Change open for g95
!       1.2     July 07        Add test on ios   T. Labrot (MF)  
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!    
!ARGUMENTS 
!     inputs :
!      INTEGER satid     !order satellite number   
!     outputs :
!
!UPDATED ELEMENTS OF INCLUDES
!     In hcinit_algoV4.h :
!         versnb,versyr,versday,ch20year,ch20day
!         prtwght,spcntlm,iwtcntlm,nedn,
!         iwtprtcnttmp,ictprtcnttmp,telescopcnttmp,
!         fwcnttmp,bpcnttmp,patchexpcnttmp,radcnttmp,
!         scmircnttmp,electcnttmp,patchfcnttmp,
!         scmotcnttmp,fwmcnttmp,chsgcnttmp,
!         cwave,bdcorr,nedn,plck,visibcoef 
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!     CHARACTER*6 noascnam  !delivers the NOAA satellite name for a given
!                            !order number.
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE 
                                                     
! includes/commons
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"

! argument variables 
      INTEGER satid
      
! functions
      CHARACTER*6 noascnam       
          
! local variables
      CHARACTER*80 text           !variable text
      CHARACTER*80 charact        !buffer storing one line of the file
      CHARACTER*6 satname         !name of the satellite (noaa16 for example)
      INTEGER ios                 !variable that receives the error status
      INTEGER il,ic               !index for loops
      INTEGER tab2(2)             !array for the reading of the file
      DOUBLE PRECISION tab0(6)    !array for the reading of the file
      DOUBLE PRECISION tab1(2)    !array for the reading of the file

!--END OF DECLARATIONS---------------------------------------------------------

! Opens and reads the calibration coefficients file
#ifdef FORTRAN_OPEN_RW
       OPEN(hrs_coefu,access='sequential',status='old',
     &      iostat=ios, action = 'READ' )
#else
       OPEN(hrs_coefu,access='sequential',status='old',
     &      iostat=ios)

!      If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       IF(ios.NE.0)
     &   OPEN(hrs_coefu,err=900,access='sequential',
     &      status='unknown',iostat=ios)
#endif

       if(ios.ne.0) then
         text='h_calibcoeffile_algoV4: coef file could not be opened'
         call ml_wti('error',text,ios)
         call aappexit(2)
       endif

! Skips the header lines
       READ(hrs_coefu,'(a80)',err=901) charact

       DO WHILE (charact(1:1) .EQ. '#')
         READ(hrs_coefu,'(a80)',err=901) charact
       ENDDO
       BACKSPACE hrs_coefu

! Parameters of the calibration coefficients file 
       READ(hrs_coefu,*,err=901)versnb
       READ(hrs_coefu,*,err=901)versyr
       READ(hrs_coefu,*,err=901)versday
       IF (iprint.GE.1) THEN
         WRITE(hrs_cllgu,*)
         WRITE(hrs_cllgu,*) 'calibration coefficients file' 
         WRITE(hrs_cllgu,500)versnb,versyr,versday
       ENDIF

       WRITE(text,'("Coef file: Version nb, year, day of year :",
     & I2,1X,I4,1X,I3)')versnb,versyr,versday
       CALL ml_wt('info',text)
       
! First & second radiation constants
       READ(hrs_coefu,'(a80)',err=901) charact
       DO WHILE (charact(4:9) .ne. 'First ')
         READ(hrs_coefu,'(a15)',err=901) charact
       ENDDO
       READ(hrs_coefu,*,err=901) plck(1),plck(2)

! To get the data of the good satellite
       satname = noascnam(satid) 
       READ(hrs_coefu,'(a80)',err=901) charact
       DO WHILE (charact(4:9) .NE. satname)
         READ(hrs_coefu,'(a15)',err=901) charact
       ENDDO
       
! IWT PRT count to temperature conversion coefficients (4 PRTs for HIRS3,
! 5 PRTs for HIRS4, 5 coefs for one PRT for NOAA15/16/17, 6 coefs for one 
! PRT for NOAA18)
      READ(hrs_coefu,*,err=901)
      READ(hrs_coefu,*,err=901)
      
      DO il=1,5
        READ(hrs_coefu,*,err=901)tab0
        DO ic=1,6
          iwtprtcnttmp(il,ic) = tab0(ic)
        ENDDO
      ENDDO
      
! Number of prt and weight coefficients for each PRT
      READ(hrs_coefu,*,err=901)
      READ(hrs_coefu,*,err=901)prtnb
      READ(hrs_coefu,*,err=901)prtwght
              
! ICT PRT count to temperature conversion coefficients (4 PRTs for HIRS3,
! 1 PRT for HIRS4, 6 coefs for one PRT for NOAA15/16/17/18)
      READ(hrs_coefu,*,err=901)
      READ(hrs_coefu,*,err=901)
      
      DO il=1,4
        read(hrs_coefu,*,err=901)tab0
        DO ic=1,6
          ictprtcnttmp(il,ic) = tab0(ic)
        ENDDO
      ENDDO

! Telescope count to temperature conversion coefficients (2 telescope for HIRS3,
! 1 for HIRS4, 6 coefs)
      READ(hrs_coefu,*,err=901)
      READ(hrs_coefu,*,err=901)
      READ(hrs_coefu,*,err=901)
      DO il=1,3
        read(hrs_coefu,*,err=901)tab0
        DO ic=1,6
          telescopcnttmp(il,ic) = tab0(ic)
        ENDDO
      ENDDO

! IR central waves
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       DO il=1,hrs_mxirchn
         READ(hrs_coefu,*,err=901) cwave(il)
       ENDDO
     
! Band-correction coefficients
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       DO il=1,hrs_mxirchn
         READ(hrs_coefu,*,err=901)tab1
         bdcorr(il,1) = tab1(1)
         bdcorr(il,2) = tab1(2)
       ENDDO

! Noise Equivalent Delta Radiance (NEDN) 
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       DO il=1,hrs_mxirchn
         READ(hrs_coefu,*,err=901) nedn(il)
       ENDDO     

! Upper and lower limits for the radiant signal output counts from the 
! space view scan (IR channels only).
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       DO il=1,hrs_mxirchn
         READ(hrs_coefu,*,err=901) tab2
         spcntlm(il,1) = tab2(1)
         spcntlm(il,2) = tab2(2)
       ENDDO

! Upper and lower limits for the radiant signal output counts from the 
! IWT scan (IR channels only).
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       DO il=1,hrs_mxirchn
         READ(hrs_coefu,*,err=901)tab2
         iwtcntlm(il,1) = tab2(1)
         iwtcntlm(il,2) = tab2(2)
       ENDDO

! Calibration coefficients for the visible channel
!    year and day of year of the most recent solar channel calibration
!    % albedo/count , % albedo
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901) ch20year,ch20day
       READ(hrs_coefu,*,err=901) visibcoef

! Filter wheel count to temperature conversion coefficients
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       DO il=1,4
         READ(hrs_coefu,*,err=901)tab0
         DO ic=1,6  
           fwcnttmp(il,ic) = tab0(ic)
         ENDDO  
       ENDDO
 
! Various count to temperature conversion coefficients
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)
       READ(hrs_coefu,*,err=901)

!   Base plate 
       READ(hrs_coefu,*,err=901)bpcnttmp

!   Patch (expd) 
       READ(hrs_coefu,*,err=901)patchexpcnttmp

!   Radiator
       READ(hrs_coefu,*,err=901)radcnttmp
       
!   Scan Mirror
       READ(hrs_coefu,*,err=901)scmircnttmp

!   Electronics 
       READ(hrs_coefu,*,err=901)electcnttmp

!   Patch (full)
       READ(hrs_coefu,*,err=901)patchfcnttmp

!   Scan Motor 
       READ(hrs_coefu,*,err=901)scmotcnttmp
 
!   Filter Wheel Motor
       READ(hrs_coefu,*,err=901)fwmcnttmp

!   Coller HSG
       READ(hrs_coefu,*,err=901)chsgcnttmp    
  
! Close the calibration coefficients file
      CLOSE(hrs_coefu,err=902)
      
      GOTO 200

 500  FORMAT(2X,' version number: ',I3/'   version year: ',I4/
     &       '   version day of year: ',I3)

 900  WRITE(hrs_cllgu,*)'Error opening calib. coef. file'
      text='Calib coefficients file could not be open'
      CALL ml_wt('error',text)
      text='h_calibcoeffile_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 901  WRITE(hrs_cllgu,*)'Error reading calib coefficients file'
      text='Calib coefficients file could not be read'
      CALL ml_wt('error',text)
      text='h_calibcoeffile_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 902  WRITE(hrs_cllgu,*)'Error closing calib coefficients file'
      text='Calib coefficients file could not be closed' 
      CALL ml_wt('error',text)
      text='h_calibcoeffile_algoV4 aborted'
      CALL ml_wt('error',text)
      CALL aappexit(2)

 200  RETURN
      END
