        SUBROUTINE h_cntmn_algoV4

!------------------------------------------------------------------------------
!NAME $Id: h_cntmn_algoV4.F 72 2007-04-27 08:21:59Z frna $
!
!PURPOSE 
!    For each calibration cycle, computes the mean of filtered radiant signal 
!    outpout counts for the views of space and IWT (internal warm target) lines.
!
!DESCRIPTION
!     For each calibration cycle:
!
!     HIRS instruments have contamination in the first xx FOVs of the space
!     scan. The variable sp1fv (h_testcoef_algoV4.dat file and after in the
!     include hcinit_algoV4.h) indicates the first field of view in the 
!     space scan that is not contaminated. The same FOVS are eliminated
!     from the scan line of the warm target.
!
!     When screening the space and IWT view samples, different tests are 
!     applied.
!     Step 0: Control the quality of each FOV
!             If for one FOV, we got X same values of chcounts, 
!             then the FOV is declared bad.      
!     Step 1: Samples with counts that are out of the gross limits are removed.
!             The gross limits are defined in the h_calcoef_algoV4.dat
!             file and after in the include hcinit_algoV4.h.
!             This test is done in the subroutine h_limit_algoV4
!             If all samples are out of the gross limits, the slope
!             can not be computed later in the process.
!     Step 2: The mean and the standard deviation of the remaining
!             samples from step 1 are computed if the samples are 
!             enough numerous. This minimum number is defined in the
!             h_testcoef_algoV4.dat file and after in the include
!             hcinit_algoV4.h.
!             This step is done by the subroutine h_cntstat_algoV4
!     Step 3: The standard deviation from step 2 is compared 
!             to the NEDC specification with
!                   NEDC=absolu(NEDN/average_slope). 
!             NEDN is defined in the h_calcoef_algoV4.dat file and after
!             in the include hcinit_algoV4.h.
!             The average slope is read in the *_hirs_b1aslope.txt file
!             and after in the include hb1averageslope_algoV4.h. It is 
!             computed off-line. It corresponds to the average slope 
!             computed with previous data (see the program hcalcb1_algoV4).
!
!             If STD < NEDC, all samples pass the test. Then the samples 
!             that are out of 3 sigma are removed. Then mean and standard
!             deviation are recomputed following step 2. The recomputed
!             mean will be used for further processing.
!
!             If STD > NEDC, the level 1B will be flagged. The median value 
!             of the samples is computed (subroutine h_median_algoV4) and 
!             will be use for futher processing.
!    
!             Note: If NEDC can not be computed,  after the launch period 
!             for example , we suppose that we are in the case 
!             (STD < NEDC)
!        
!    Reference:
!        HIRS calibration Algorithm Version 4.0 (Nov 3, 2003)
!         C. Cao NESDIS
!        NOAA-KLM User'Guide Section 7.2                     
!
!HISTORY (CHANGES) 
!    (version     date         comment           <author>)
!       1.0     January 05     Creation          T.Labrot (MF)
!       1.1     13 June 05     The median, the observed NeDn are always computed
!                              to help for the monitoring  T.Labrot and P.Brunel (MF)
!       1.2     25 April 07    Added the step 0 (control on each FOV)  T.Labrot (MF)
!
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.          
!         
!ARGUMENTS 
!
!UPDATED ELEMENTS OF INCLUDES
!    in include hcalibcycleinfo_algoV4.h: 
!      qualified(hrs_mxchn,hrs_mxcalib) !=true if the calib cycle is usable
!      spcntmn(hrs_mxirchn,hrs_mxcalib)
!              !Mean of the space counts for the 19 IR channels
!      spcntstd(hrs_mxirchn,hrs_mxcalib)
!              !Standard deviation of the space counts for the 19 IR channels
!      spgoodcnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of space counts used to compute the mean
!      spmisscnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of missing space counts 
!      spinfcnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of space counts inferiors to the limit 
!      spsupcnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of space counts superiors to the limit 
!      spejeccnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of rejected space counts
!              !=missing value if median value is used for the mean
!      iwtcntmn(hrs_mxirchn,hrs_mxcalib)
!              !Mean of the IWT counts for the 19 IR channels
!      iwtcntstd(hrs_mxirchn,hrs_mxcalib)
!              !Standard deviation of the IWT counts for the 19 IR channels
!      iwtgoodcnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of IWT counts used to compute the mean
!      iwtmisscnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of missing IWT counts 
!      iwtinfcnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of IWT counts inferiors to the limit 
!      iwtsupcnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of IWT counts superiors to the limit 
!      iwtejeccnt(hrs_mxirchn,hrs_mxcalib)
!              !Number of rejected IWT counts
!              !=missing value if median value is used for the mean
!
!  in include  hcaliblineinfo_algoV4.h:
!     linechquality(6,hrs_mxchn,hrs_mxlin)                       
!                           !1: calibration failed
!                           !2: calibration marginal
!                           !3: calibration slope came from the HCF
!                           !4: BB views failed NEDC test
!                           !5: space views failed NEDC test
!                           !6: some QC (quality control) tests could not be 
!                           !   applied 
! 
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!     SUBROUTINE h_orderch_algoV4(inptab,outtab,dim1,dim2,xx) :
!                It moves the order of the HIRS channels.
!
!     SUBROUTINE h_limit_algoV4(sp1fv,counts,cntlm,infcnt,supcnt,misscnt)
!                It checks if counts are outside limits or not.
!
!     SUBROUTINE h_cntstat_algoV4(sp1fv,minvw,counts,dtmn,dtstd,goodcnt) :
!                For one scan line (space or IWT one), it computes the mean and
!                the standard deviation of radiant signal outpout counts.
!    
!     SUBROUTINE h_median_algoV4(sp1fv,dim,tabs,medcount) :
!               For one scan line (space or IWT one), this module computes the 
!               median value of output counts.
!
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE
                                                      
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hb1averageslope_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
#include "hcaliblineinfo_algoV4.h"


! argument variables
      
! local variables
      CHARACTER*80 text         !variable text
      INTEGER icalib,ich,ifv    !index of loops
      INTEGER ilin              !scan line number in the orbit
      INTEGER ifv_flag(hrs_mxpix) !quality flag for each FOV of a space or IWT line
      INTEGER value_cnt         !total number of channels with the same outpout count
!                                for one FOV
      INTEGER ich_check         !index of a channel that is compared with others channels
!                                for one FOV
      INTEGER value_check       !value of the output count for the ich_check channel
      INTEGER infcnt            !counter of output count values
!                                lower than min limit
      INTEGER supcnt            !counter of output count values
!                               !greater than max limit
      INTEGER misscnt           !counter of missing output count values
      INTEGER goodcnt           !counter of valid outputcount values
      INTEGER badcount          !output counts values exceeding 
!                                stdevcf*standard deviation.
      INTEGER inputch(hrs_mxchn,hrs_mxpix)   
!                     !output counts for all channels and hrs_mxpix FOVs
      INTEGER chcounts(hrs_mxchn,hrs_mxpix)   
!                     !output counts for all channels and hrs_mxpix FOVs
                                !but the channels has been ordered
      INTEGER counts(hrs_mxpix) !output counts for one channel and 56 FOVs
      INTEGER medcount          !median value of output counts
      
      DOUBLE PRECISION dtmn,dtstd   
                                !mean and standard deviation for one scan line
                                
      DOUBLE PRECISION nedc(hrs_mxirchn) !noise equivalent delta count
      DOUBLE PRECISION nedn_obs          !observed noise computed with the 
                                         !standard deviation 
                                
!--END OF DECLARATIONS----------------------------------------------------------


      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '**  SUB. H_CNTMN_ALGOV4  **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)

      text='Computes means of outpout counts for space/IWT lines'
      CALL ml_wt('info',text)

! Initializes
!------------      
      DO ich=1,hrs_mxirchn
        nedc(ich)=missdp
      ENDDO  
      DO ich=1,hrs_mxchn
        DO ifv=1,hrs_mxpix
          inputch(ich,ifv)=missint
          chcounts(ich,ifv)=missint
        ENDDO
      ENDDO    
      DO ifv=1,hrs_mxpix
        ifv_flag(ifv)=0
        counts(ifv)=missint
      ENDDO 
      value_check=0
      value_cnt=0
      infcnt=missint
      supcnt=missint
      misscnt=missint
      goodcnt=missint 
      badcount=missint   
      dtmn  = missdp
      dtstd = missdp 
      medcount=missint
      
! Computes the NEDC value
!------------------------

      DO ich=1,hrs_mxirchn
          
        IF (avgslope(ich).NE.0.) THEN
          nedc(ich) = abs( nedn(ich)/avgslope(ich) )
        ELSE
          nedc(ich) = missdp
          IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=500) ich
          IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=510)
          WRITE(text,'("AverageSlope = 0 for channel ",I2)') ich
          CALL ml_wt('warning',text)
          WRITE(text,'("nedc = missdp")')
          CALL ml_wt('warning',text)
        ENDIF 
          
      ENDDO
          

! For the different calibration cycles                
      DO icalib=1,calib

! ------------------------------------------------
! Processing of each space scan line in the orbit. 
! ------------------------------------------------
        ilin=splintab(icalib)

        IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=520) icalib,ilin
      
! Puts HIRS channels output counts in order 1,2,3,...
        DO ifv=0,hrs_mxpix-1
          DO ich=3,3+hrs_mxchn-1
            inputch(ich-2,ifv+1)=helem(ich,ifv,ilin)
          ENDDO
        ENDDO

        CALL h_orderch_algoV4(inputch,chcounts,hrs_mxchn,hrs_mxpix,0)

!        IF (iprint.GE.2) THEN
!          DO ifv=1,hrs_mxpix
!            WRITE(hrs_cllgu,fmt=530) 
!            DO ich=1,hrs_mxchn
!              WRITE(hrs_cllgu,fmt=540) ifv,ich,chcounts(ich,ifv)
!            ENDDO
!          ENDDO
!        ENDIF
        
! Control the quality of each FOV  (step 0)
! If for one FOV, we got X same values of chcounts, then the FOV is declared bad.

        DO ifv=1,hrs_mxpix
        
          DO ich_check=1,hrs_mxchn
          
           IF (ifv_flag(ifv).EQ.0) THEN
                   
            value_check=chcounts(ich_check,ifv)
            value_cnt=0
            
            DO ich=1,hrs_mxchn
              IF (ich_check.NE.ich) THEN
                IF (value_check.EQ.chcounts(ich,ifv)) THEN
                  value_cnt=value_cnt+1
                ENDIF
              ENDIF
            ENDDO
            
            IF (value_cnt.GE.cntmax) THEN
! This IFOV is declared bad
               ifv_flag(ifv)=1
               IF (iprint.GE.1) 
     &          WRITE(hrs_cllgu,fmt=545) ifv,value_cnt,ifv_flag(ifv)                               
            ENDIF
       
          ENDIF
          
         ENDDO
          
        ENDDO                      
                  

! For each IR channel, computes the mean of filtered radiant signal 
!------------------------------------------------------------------
!outpout counts
!--------------
 
        DO ich=1,hrs_mxirchn
       
          IF (qualified(ich,icalib)) THEN
         
            IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=550) ich 

! Picks up the appropriate counts for the channel ich

            DO ifv=1,hrs_mxpix
            
              IF (ifv_flag(ifv).EQ.1) THEN
                counts(ifv)= missint
              ELSE  
                counts(ifv)=chcounts(ich,ifv)
              ENDIF
                
            ENDDO
            
!         IF (iprint.GE.2) THEN
!          DO ifv=1,hrs_mxpix
!           WRITE(hrs_cllgu,fmt=530) 
!           WRITE(hrs_cllgu,fmt=540) ifv,ich,counts(ifv)
!          ENDDO
!         ENDIF 
      
! Test: outpout counts included in limits
                  
            infcnt=0
            supcnt=0
            misscnt=0
            CALL h_limit_algoV4
     &          (ich,sp1fv,counts,spcntlm,infcnt,supcnt,misscnt)
     
! Updates the variable to flag the level 1B if all or some views are outside limits 

            IF ( infcnt.NE.0 .OR. supcnt.NE.0) THEN
              linechquality(2,ich,ilin)=1
            ELSE
              linechquality(2,ich,ilin)=0   
            ENDIF
             

! Computes the mean and  the standard deviation of output counts.
! (test:minimum of filtered data to compute the mean)

            dtmn=0.d+0
            dtstd=0.d+0
            goodcnt=0
            CALL h_cntstat_algoV4(sp1fv,minvw,counts,dtmn,dtstd,goodcnt)
            
! Computes the median value of the samples 
! The median value, although not always used, is computed for the monitoring
            medcount=0    
            CALL h_median_algoV4(sp1fv,hrs_mxpix,counts,medcount)
            
! Computes the observed NeDn for monitoring
            nedn_obs=0.
            nedn_obs = ABS(dtstd * avgslope(ich))           
          
            IF(linechquality(2,ich,ilin).EQ.0 .AND. dtmn.EQ.missdp) 
     &         linechquality(2,ich,ilin)=1
     
                   
! Compares the standard deviation of the remaining samples to the NEDC value
! IF (nedc(ich).EQ.missdp), we suppose that we are in the case
! (dtstd.le.nedc(ich)) (after the launch period for example)
            
            IF (dtmn.NE.missdp .AND. dtstd.NE.missdp) THEN
            
              IF (nedc(ich).NE.missdp.AND.dtstd.GT.nedc(ich)) THEN
             
                IF(iprint.GE.1) THEN
                  WRITE(hrs_cllgu,fmt=560) 
                  WRITE(hrs_cllgu,fmt=565) nedn_obs, nedn(ich) 
                ENDIF    
 
                linechquality(5,ich,ilin)=1
             
                spcntmn(ich,icalib)=medcount
                spcntstd(ich,icalib) = dtstd
                spcntmed(ich,icalib) = medcount
                spnedn(ich,icalib)   = nedn_obs
                spgoodcnt(ich,icalib)= 1
                spinfcnt(ich,icalib) = infcnt
                spsupcnt(ich,icalib) = supcnt
                spmisscnt(ich,icalib)= misscnt
                spejeccnt(ich,icalib)= goodcnt-1


              ELSE    !IF(nedc(ich).NE.missdp.AND.dtstd.GT.nedc(ich)) THEN 

                IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=570)          
            
                linechquality(5,ich,ilin)=0
            
!--------------Removes outpout counts that are out of stdevcf*dtstd
            
                badcount=0
    
                DO ifv=sp1fv,hrs_mxpix

                  IF (counts(ifv).NE.missint) THEN
                    IF(abs(counts(ifv)-dtmn).GT.(stdevcf*dtstd))THEN

                      IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=580) 
     &                                  ich,ifv,counts(ifv)
 
                      counts(ifv)=missint
                      badcount   = badcount+1
       
                    ENDIF
                  ENDIF
          
                ENDDO

                IF (badcount.EQ.0.AND.iprint.GE.1) 
     &                            WRITE(hrs_cllgu,fmt=590)

                IF (badcount.NE.0) THEN
!--------------Recomputes the statistics 

                  IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=600)
              
                  dtmn = 0.d+0
                  dtstd= 0.d+0
                  goodcnt=0
              
                  CALL h_cntstat_algoV4
     &                  (sp1fv,minvw,counts,dtmn,dtstd,goodcnt)

  
                  IF (linechquality(2,ich,ilin).EQ.0)
     &                      linechquality(2,ich,ilin)=1
             

                ENDIF           !IF (badcount.NE.0) THEN
          

                spcntmn(ich,icalib)  = dtmn
                spcntstd(ich,icalib) = dtstd
                spcntmed(ich,icalib) = medcount
                spnedn(ich,icalib)   = nedn_obs                
                spgoodcnt(ich,icalib)= goodcnt
                spinfcnt(ich,icalib) = infcnt
                spsupcnt(ich,icalib) = supcnt
                spmisscnt(ich,icalib)= misscnt
                spejeccnt(ich,icalib)= badcount
          
              ENDIF             !IF(nedc(ich).NE.missdp.AND.dtstd.GT.nedc(ich)) THEN
              
            ELSE                !IF (dtmn.NE.missdp .AND. dtstd.NE.missdp) THEN
            
              spcntmn(ich,icalib)  = dtmn
              spcntstd(ich,icalib) = dtstd
              spcntmed(ich,icalib) = medcount
              spnedn(ich,icalib)   = nedn_obs  
              spgoodcnt(ich,icalib)= goodcnt
              spinfcnt(ich,icalib) = infcnt
              spsupcnt(ich,icalib) = supcnt
              spmisscnt(ich,icalib)= misscnt
              spejeccnt(ich,icalib)= missint
              
            ENDIF               !IF (dtmn.NE.missdp .AND. dtstd.NE.missdp) THEN
            
! Updates qualified area

            IF (spcntmn(ich,icalib).EQ.missdp) THEN
              
              qualified(ich,icalib)=.FALSE.
              
              IF(iprint.GE.1) THEN
                WRITE(hrs_cllgu,fmt=610)icalib,ich
                WRITE(hrs_cllgu,fmt=620)
              ENDIF
              
              WRITE(text,'("Calib cycle ",I2," Channel ",I2,
     &         ":No mean or standard deviation for space")')icalib,ich
              CALL ml_wt('warning',text)
              WRITE(text,'("==>No qualified calib cycle")')
              CALL ml_wt('warning',text)

           ENDIF 
         
         ELSE
         
           IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=630)icalib,ich
           
           WRITE(text,'("No qualified calib cycle ",I2,
     &           " for channel ",I2)')icalib,ich
           CALL ml_wt('warning',text)        
           
         ENDIF          !IF (qualified(ich,icalib)) THEN          

       
! End of the loop "for each channel"
        ENDDO
        
        
!--------------------------------------------------------------------------
! Reinitializes arrays and parameters for the processing of a IWT scan line
!---------------------------------------------------------------------------
        DO ifv=1,hrs_mxpix
          counts(ifv)=0
          ifv_flag(ifv)=0
          DO ich=1,hrs_mxchn
            chcounts(ich,ifv)=0
            inputch(ich,ifv) =0
          ENDDO
        ENDDO

        value_check=0
        value_cnt=0
        dtmn = 0.d+0
        dtstd = 0.d+0
 
!-------------------------------------------------
! Processing of each IWT scan line in the orbit. 
!-------------------------------------------------
        ilin=iwtlintab(icalib)

        IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=700) icalib,ilin
        
! Puts HIRS channels output counts in order 1,2,3,...
        DO ifv=0,hrs_mxpix-1
          DO ich=3,3+hrs_mxchn-1
            inputch(ich-2,ifv+1)=helem(ich,ifv,ilin)
          ENDDO
        ENDDO

        CALL h_orderch_algoV4(inputch,chcounts,hrs_mxchn,hrs_mxpix,0)
        
!        IF (iprint.GE.2) THEN
!          DO ifv=1,hrs_mxpix
!            WRITE(hrs_cllgu,fmt=530) 
!            DO ich=1,hrs_mxchn
!              WRITE(hrs_cllgu,fmt=540) ifv,ich,chcounts(ich,ifv)
!            ENDDO
!          ENDDO
!        ENDIF        

! Control the quality of each FOV   (step 0)
! If for one FOV, we got X same values of chcounts, then the FOV is declared bad.

        DO ifv=1,hrs_mxpix
        
          DO ich_check=1,hrs_mxchn
          
           IF (ifv_flag(ifv).EQ.0) THEN
                   
            value_check=chcounts(ich_check,ifv)
            value_cnt=0
            
            DO ich=1,hrs_mxchn
              IF (ich_check.NE.ich) THEN
                IF (value_check.EQ.chcounts(ich,ifv)) THEN
                  value_cnt=value_cnt+1
                ENDIF
              ENDIF
            ENDDO
                       
            IF (value_cnt.GE.cntmax) THEN
! This IFOV is declared bad
               ifv_flag(ifv)=1
               IF (iprint.GE.1)
     &          WRITE(hrs_cllgu,fmt=545) ifv,value_cnt ,ifv_flag(ifv)               
            ENDIF

          ENDIF
          
         ENDDO
          
        ENDDO                      
                  
                  

! For each IR channel,computes the mean of filtered radiant signal
!-----------------------------------------------------------------
!outpout counts
!--------------

        DO ich=1,hrs_mxirchn

          IF (qualified(ich,icalib)) THEN
        
            IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=550) ich

! Picks up the appropriate counts for the channel ich
             DO ifv=1,hrs_mxpix
            
              IF (ifv_flag(ifv).EQ.1) THEN
                counts(ifv)= missint
              ELSE  
                counts(ifv)=chcounts(ich,ifv)
              ENDIF
                
            ENDDO 
                       
!         IF (iprint.GE.2) THEN
!          DO ifv=1,hrs_mxpix
!            WRITE(hrs_cllgu,fmt=530) 
!            WRITE(hrs_cllgu,fmt=540) ifv,ich,counts(ifv)
!          ENDDO
!        ENDIF 

! Test: outpout counts included in limits
          
            infcnt=0
            supcnt=0
            misscnt=0          
            CALL h_limit_algoV4
     &       (ich,sp1fv,counts,iwtcntlm,infcnt,supcnt,misscnt)

! Updates the variable to flag the level 1B if all or some views are outside limits 

            IF ( infcnt.NE.0 .OR. supcnt.NE.0) THEN
              linechquality(2,ich,ilin)=1
            ELSE
              linechquality(2,ich,ilin)=0   
            ENDIF

! Computes the mean and  the standard deviation of output counts.
! (test:minimum of filtered data to compute the mean)

            dtmn=0.d+0
            dtstd=0.d+0
            goodcnt=0
            CALL h_cntstat_algoV4(sp1fv,minvw,counts,dtmn,dtstd,goodcnt)
            
! Computes the median value of the samples 
! The median value, although not always used, is computed for the monitoring
            medcount=0      
            CALL h_median_algoV4(sp1fv,hrs_mxpix,counts,medcount)
            
! Computes the observed NeDn for monitoring
            nedn_obs=0.
            nedn_obs = ABS(dtstd * avgslope(ich))
                
            IF(linechquality(2,ich,ilin).EQ.0 .AND. dtmn.EQ.missdp) 
     &         linechquality(2,ich,ilin)=1
          
! Compares the standard deviation of the remaining samples to the NEDC value
! IF (nedc(ich).EQ.missdp), we suppose that we are in the case
! (dtstd.le.nedc(ich)) (after the launch period for example)         
        
            IF (dtmn.NE.missdp .AND. dtstd.NE.missdp) THEN
                
             IF (nedc(ich).NE.missdp.AND.dtstd.GT.nedc(ich)) THEN
       
               IF(iprint.GE.1) THEN
                 WRITE(hrs_cllgu,fmt=560) 
                 WRITE(hrs_cllgu,fmt=565) nedn_obs, nedn(ich) 
               ENDIF 
                   
               linechquality(4,ich,ilin)=1
               
               iwtcntmn(ich,icalib) = medcount
               iwtcntstd(ich,icalib) = dtstd
               iwtcntmed(ich,icalib) = medcount
               iwtnedn(ich,icalib)   = nedn_obs
               iwtgoodcnt(ich,icalib)= 1
               iwtinfcnt(ich,icalib) = infcnt
               iwtsupcnt(ich,icalib) = supcnt
               iwtmisscnt(ich,icalib)= misscnt
               iwtejeccnt(ich,icalib)= goodcnt-1
                
             ELSE    !IF(nedc(ich).NE.missdp.AND.dtstd.GT.nedc(ich)) THEN
                        
               IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=570)
            
               linechquality(4,ich,ilin)=0
                   
!--------------Removes outpout counts that are out of stdevcf*dtstd

               badcount=0

               DO ifv=sp1fv,hrs_mxpix

                 IF (counts(ifv).LT.missint) THEN
                   IF(abs(counts(ifv)-dtmn).GT.(stdevcf*dtstd)) THEN

                     IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=580) 
     &                                  ich,ifv,counts(ifv) 

                     counts(ifv)=missint
                     badcount   = badcount+1

                   ENDIF
                 ENDIF

               ENDDO

               IF(badcount.EQ.0.AND.iprint.GE.1) 
     &               WRITE(hrs_cllgu,fmt=590)

               IF (badcount.NE.0) THEN
!--------------Recomputes the statistics

                 IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=600)
                 
                 dtmn = 0.d+0
                 dtstd= 0.d+0
                 goodcnt=0
                 CALL h_cntstat_algoV4
     &                 (sp1fv,minvw,counts,dtmn,dtstd,goodcnt)
     
                 IF (linechquality(2,ich,ilin).EQ.0)
     &                      linechquality(2,ich,ilin)=1   
     

               ENDIF            !IF (badcount.NE.0) THEN

               iwtcntmn(ich,icalib)  = dtmn
               iwtcntstd(ich,icalib) = dtstd
               iwtcntmed(ich,icalib) = medcount
               iwtnedn(ich,icalib)   = nedn_obs
               iwtgoodcnt(ich,icalib)= goodcnt
               iwtinfcnt(ich,icalib) = infcnt
               iwtsupcnt(ich,icalib) = supcnt
               iwtmisscnt(ich,icalib)= misscnt
               iwtejeccnt(ich,icalib)= badcount
               
                   
 
 
             ENDIF              !IF(nedc(ich).NE.missdp.AND.dtstd.GT.nedc(ich)) THEN  
            
           ELSE                 !IF (dtmn.NE.missdp .AND. dtstd.NE.missdp) THEN  
           
             iwtcntmn(ich,icalib)  = dtmn
             iwtcntstd(ich,icalib) = dtstd
             iwtcntmed(ich,icalib) = medcount
             iwtnedn(ich,icalib)   = nedn_obs
             iwtgoodcnt(ich,icalib)= goodcnt
             iwtinfcnt(ich,icalib) = infcnt
             iwtsupcnt(ich,icalib) = supcnt
             iwtmisscnt(ich,icalib)= misscnt
             iwtejeccnt(ich,icalib)= missint
             
             
              
           ENDIF                !IF (dtmn.NE.missdp .AND. dtstd.NE.missdp) THEN  

! Updates qualified area         
         
           IF (iwtcntmn(ich,icalib).EQ.missdp) THEN
           
             qualified(ich,icalib)=.FALSE.
             
             IF(iprint.GE.1) THEN
               WRITE(hrs_cllgu,fmt=710)icalib,ich
               WRITE(hrs_cllgu,fmt=620)
             ENDIF
             
             WRITE(text,'("Calib cycle ",I2," Channel ",I2,
     &         ":No mean or standard deviation for IWT")')icalib,ich
             CALL ml_wt('warning',text)
             WRITE(text,'("==>No qualified calib cycle")')
             CALL ml_wt('warning',text)
                                            
           ENDIF 
         
         ELSE
         
           IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=630)icalib,ich
           
           WRITE(text,'("No qualified calib cycle ",I2,
     &         " for channel ",I2)')icalib,ich
           CALL ml_wt('warning',text)        
         
         ENDIF          !IF (qualified(ich,icalib)) THEN 

! End of the loop "for each channel"
        ENDDO
         
! End of the loop "for each calibration cycle"
       ENDDO 

      
 500   FORMAT(4X,'averageslope = 0 for channel ',I2)
 510   FORMAT(4X,'nedc = missdp')
 520   FORMAT(/2X,'CALIB CYCLE :',I2,4X,'SPACE LINE NUMBER: ',I4)
! 530   FORMAT(/2X,'ordered channels: ifov   ich    chcounts')
! 540   FORMAT(20X,I2,5X,I2,7X,I5)
 545   FORMAT(2X,'FOV: ',I2,3X,'got ',I2,
     &      ' same value of CNs. FOV flag:',I2)
 550   FORMAT(2X,'CHANNEL ',I2,': ')
 560   FORMAT(8X,'nedc(ich).ne.missdp and standard deviation > nedc')
 565   FORMAT(8X,'nedc observed and specified: ',2f8.4) 
 570   FORMAT(8X,'nedc(ich).eq.missdp or standard deviation <= nedc')
 580   FORMAT(8X,'eliminated values: ch:',I2,'  ifov:',I2,
     &        '  count:',I6)
 590   FORMAT(8X,'eliminated values = none')
 600   FORMAT(8X,'for filtered values:')
 610   FORMAT(2X,'Calib cycle ',I2,' Channel ',I2,
     &     ':No mean or standard deviation for space')
 620   FORMAT(2X,'==>No qualified calib cycle')
 630   FORMAT(2X,'No qualified calib cycle ',I2,
     &           ' for channel ',I2)
  

 700   FORMAT(/2X,'CALIB CYCLE :',I2,4X,'IWT LINE NUMBER: ',I4)
 710   FORMAT(2X,'Calib cycle ',I2,'  Channel ',I2,
     &     ':No mean or standard deviation for IWT')


       RETURN
       END




 
               








