       SUBROUTINE h_intercept_algoV4(
     & sttp,
     & iwtrad,
     & calibcoef,
     & b11)

!------------------------------------------------------------------------------
!NAME $Id: h_intercept_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE
!   This modules computes the intercept for each channel and each Earth 
!   view line   
!
!DESCRIPTION
!   For each IR channel (ich):
!
!   First step: 
!     Computes the intercept at the calib cycle k, Ic'(k), using the average 
!     space view count in the equation:   Ic'(k)= -S'(k-1:k)*Csp(k)
!     If  Csp(k) doesn't exist, compute I'(k) using the average IWT view count 
!     and the IWT radiance in the equation:
!                 Ic'(k)= Riwt(k)-S'(k-1:k)*Ciwt(k)
!     If Riwt(k) or Ciwt(k) don't exist, Ic'(k) is declared bad.
!
!   Second step:
!     Computes the linear intercept Il'(n) for each Earth view scan line ilin. 
!     Il'(n) is the linear interpolation of the intercept between the 2 
!     calib cycles (k-1) and (k).
!                 Il'(ilin)=Ic'(k-1)+n/40 *(Ic'(k)-Ic'(k-1))
!                  with n=scan line number in a superswath 
!                       Ic'(k-1) and Ic'(k-1)=intercept computed at the first step   
!     For the Earth lines before the first calib line, Il'(ilin)=I'(1)
!     with  I'(1) the intercept computed at the first step for the first calib cycle 
!     For the Earth line after the last calib line, Il'(ilin)=I'(calib)
!     with  I'(calib) the intercept computed at the first step for the last calib cycle
!
!   Third step:
!     Computes the correction Tt'(n) to the intercept based on the telescope 
!     temperature for each Earth view scan line ilin. 
!                 It'(ilin)=b1*[(t(n)-t(k-1)) - n/40 * (t(k)-t(k-1))]
!                  with n=scan line number in a superswath
!                       b1=correlation coefficient between the telescope
!                          temperature and calibration intercept for a given
!                          channel. b1 is computed off line by hcalcb1_algoV4.F.
!                       t(n)=telescope temperature at scan line n
!                       t(k)=telescope temperature at calib cycle k
!
!   Fourth step:
!     Computes the final intercept:
!      with no correction computed from the telescope temperature 
!                   I'(ilin)=Il'(ilin)
!      with correction computed from the telescope temperature
!                   I'(ilin)=Il'(ilin)+It'(ilin)
!
!   For the visible channel, the coefficients are furnished by the fabricator 
!   of the instrument and will not change during the lifetime of the satellite.   
!
!   At the end, the module updates the elements hcalcof/hscalcof 
!   to write later in the HIRS level 1B file.
!
!   For each 38 Earth view scan line of a super-swath, 
!      Final intercepts  in the primary coefficients words
!      Linear intercepts in the secondary coefficients words
!   For the space view scan line,
!      Recomputed intercepts in the primary and secondary coefficients words
!   For the blackbody scan line,
!      BB intercepts or raw intercepts in the primary and secondary coefficients words
!                      
!  REFERENCE
!   HIRS calibration Algorithm Version 4.0 (Nov 3, 2003)
!   C. Cao NESDIS
!   NOAA KLM User's Guide Section 7.2
! 
!HISTORY (CHANGES) 
!  (version    date          comment           <author>)
!     1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP 
!   
!COPYRIGHT
!   This software was developed within the context of
!   the EUMETSAT Satellite Application Facility on
!   Numerical Weather Prediction (NWP SAF), under the
!   Cooperation Agreement dated 25 November 1998, between
!   EUMETSAT and the Met Office, UK, by one or more partners
!   within the NWP SAF. The partners in the NWP SAF are
!   the Met Office, ECMWF, KNMI and MeteoFrance.    
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
! 
!ARGUMENTS 
!   inputs :
!    REAL sttp(hrs_mxlin)  ! STT for all lines
!    DBLE PREC iwtrad(hrs_mxirchn,hrs_mxcalib)
!                  !IR channel radiances of the IWT 
!  
!   inputs/outputs :
!    DBLE PREC calibcoef(hrs_mxchn,3,hrs_mxlin)
!                  !Third auto coefs,slopes,intercepts array for each scan line
!
!   outputs :
!    REAL b11(hrs_mxchn,hrs_mxlin) 
!                  !Correlation coefficient between the telescope 
!                  !temperature and calibration intercept used in
!                  !the equations
!
!
!UPDATED ELEMENTS OF INCLUDES
!  in include  hcaliblineinfo_algoV4.h 
!          INTEGER linechquality(6,hrs_mxchn,hrs_mxlin)                       
!                              !1: calibration failed
!                              !2: calibration marginal
!                              !3: calibration slope came from the HCF
!                              !4: BB views failed NEDC test
!                              !5: space views failed NEDC test
!                              !6: some QC (quality control) tests could not be 
!                              !   applied 
! 
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hb1averageslope_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
      REAL sttp(hrs_mxlin)  ! STT for all lines 
      DOUBLE PRECISION iwtrad(hrs_mxirchn,hrs_mxcalib)
                     !IR channel radiances of the IWT 
      DOUBLE PRECISION calibcoef(hrs_mxchn,3,hrs_mxlin)
                     !third auto coefs,slopes,intercepts array for each scan line
      REAL b11(hrs_mxchn,hrs_mxlin) 
                     !Correlation coefficient between the telescope 
                     !temperature and calibration intercept used in
                     !the equations
! local variables
      CHARACTER*80 text        !variable text
      INTEGER icalib,ilin,ich  !index of loops
      INTEGER nlin       !number of lines between the current scan line
                         !and a space scan line
      INTEGER k          !index to find the good calib cycle for the 
                         !current scan line
      INTEGER l1border(hrs_mxchn) !channels in the l1b order                   
      INTEGER splin               !current space line
      INTEGER coef0(hrs_mxchn)    !intercept array to update the level 1b                   
      REAL solzen        !Solar zenith angle at the nadir in degrees
      
      DOUBLE PRECISION Ic(hrs_mxirchn,hrs_mxcalib)
                         !the intercept of IR channels for each calib cycle
      DOUBLE PRECISION Il(hrs_mxirchn,hrs_mxlin)
                         !the linear intercept of IR channels for each scan line
      DOUBLE PRECISION It(hrs_mxirchn,hrs_mxlin)
                         !the "telescope" intercept of IR channels for each scan line
      DOUBLE PRECISION Sratio
                         !slope ratio

!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '* SUB H_INTERCEPT_ALGOV4  *'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      
      
! Initializes the outputs and the local variables
!------------------------------------------------
 
      DATA l1border/ 1,17,2,3,13,4,18,11,19,7,8,20,
     &          10,14,6,5,15,12,16,9 /
     
      DO ich=1,hrs_mxirchn
        DO icalib=1,hrs_mxcalib
          Ic(ich,icalib) = missdp
        ENDDO  
        DO ilin=1,hrs_mxlin
          Il(ich,ilin) = missdp
          It(ich,ilin) = missdp
          b11(ich,ilin) = missreal
        ENDDO
      ENDDO
      
      DO ich=1,hrs_mxchn
        DO ilin=1,hrs_mxlin
          calibcoef(ich,3,ilin) = missdp
        ENDDO
      ENDDO 
       
      solzen = missreal


! Computes the intercept for each channel and each calibration cycle
!-------------------------------------------------------------------
! using the average slopes derived before and average space counts.
!------------------------------------------------------------------
! If the average space count doesn't exist, try with IWT data.
!------------------------------------------------------------- 

      IF (iprint.GE.1)WRITE(hrs_cllgu,
     &        '("Computes the calib intercept ")')  
      text='Computes the calib intercept'
      CALL ml_wt('info',text)

      DO icalib=1,calib
       DO ich=1,hrs_mxirchn
        
         IF (calibcoef(ich,2,splintab(icalib)) .NE. missdp) THEN
        
           IF (spcntmn(ich,icalib).NE.missdp) THEN
          
             Ic(ich,icalib) = 
     &       -1*calibcoef(ich,2,splintab(icalib))*spcntmn(ich,icalib)
     
           ELSE IF (iwtcntmn(ich,icalib).NE.missdp .AND. 
     &              iwtrad(ich,icalib).NE.missdp) THEN
               
             Ic(ich,icalib) = iwtrad(ich,icalib)
     &       -1*calibcoef(ich,2,splintab(icalib))*iwtcntmn(ich,icalib) 
     
             IF (iprint.GE.1) 
     &       WRITE(hrs_cllgu,
     &       '("intercept Ic computed with IWT data for calib/channel: "
     &       ,i2,2x,i2)') icalib,ich
 
             WRITE(text,
     &       '("intercept Ic computed with IWT data for calib/channel: "
     &       ,i2,2x,i2)') icalib,ich
             CALL ml_wt('info',text)
     
           ELSE
            
             Ic(ich,icalib) = missdp
            
             IF (iprint.GE.1) 
     &       WRITE(hrs_cllgu,
     &       '("intercept Ic not computed with IWT data for ", 
     &       "calib/channel: ",i2,2x,i2)') icalib,ich
 
             WRITE(text,
     &       '("intercept Ic not computed with IWT data for ",
     &       "calib/channel: ",i2,2x,i2)') icalib,ich
             CALL ml_wt('warning',text)
            
           ENDIF
           
         ELSE   !IF (calibcoef(ich,2,splintab(icalib)) .NE. missdp) THEN
         
           Ic(ich,icalib) = missdp 
              
           IF (iprint.GE.1) 
     &     WRITE(hrs_cllgu,
     &     '("slope missing. intercept Ic not computed  for ", 
     &     "calib/channel: ",i2,2x,i2)') icalib,ich
 
           WRITE(text,
     &     '("slope missing. intercept Ic not computed for ",
     &     "calib/channel: ",i2,2x,i2)') icalib,ich
           CALL ml_wt('warning',text)
           
         ENDIF                         
        
       ENDDO            !DO ich=1,hrs_mxirchn
        
      ENDDO             !DO icalib=1,calib


! Computes the linear intercept for each channel and for all the Earth
!--------------------------------------------------------------------- 
! view scan lines by the linear interpolation of the intercepts 
!--------------------------------------------------------------
! between the two calibration cycles (k-1) and (k) .
!---------------------------------------------------
   
      IF (iprint.GE.1)WRITE(hrs_cllgu,*)
      IF (iprint.GE.1)WRITE(hrs_cllgu,
     &        '("Computes the linear intercept ")') 
      text='Computes the linear intercept'
      CALL ml_wt('info',text)
      
!  For scan lines located behind the first calib cycle and before the
!  last calib cycle
 
      DO ilin=splintab(1),splintab(calib)-1
      
!  In this scheme the space scan marks the start of the next superswath and
!  contains the recomputed slope of the next superswath.      
        
        k=1
        DO WHILE (ilin.GE.splintab(k).AND.k.LE.calib)
          k=k+1
        ENDDO
        
        nlin=ilin-splintab(k-1)
        
        DO ich=1,hrs_mxirchn
          
          IF ( Ic(ich,k).NE.missdp .AND.
     &         Ic(ich,k-1).NE.missdp ) THEN
         
!  To interpolate the intercepts we must ensure that we use the same slope at
!  both ends of the interpolation. Note that Ic(ich,k-1) contains the intercept
!  that has been recomputed using S'(k-1:k), and calibcoef(ich,2,splintab(k-1))
!  contains S'(k-1:k). So Ic(ich,k) needs to be corrected before using it.

             Sratio = calibcoef(ich,2,splintab(k-1)) / 
     &           calibcoef(ich,2,splintab(k))
             Il(ich,ilin) = Ic(ich,k-1)
     &       + nlin*( Sratio*Ic(ich,k)-Ic(ich,k-1) )/40
     
             IF (iprint.GE.2) 
     &       WRITE(hrs_cllgu,'("ilin ich k spk nlin Ick Ick-1 Il",
     &       I4,2(1x,I2),1x,I4,1x,I2,1x,3(2X,F9.4))') 
     &       ilin,ich,k,splintab(k),nlin,
     &       Sratio*Ic(ich,k),Ic(ich,k-1),Il(ich,ilin)
     
          ELSE 
             
            Il( ich,ilin) = missdp
             
            IF (iprint.GE.1) 
     &      WRITE(hrs_cllgu,
     &      '("intercept Il not computed for line/channel: ", 
     &      i4,2x,i2)') ilin,ich
 
            WRITE(text,
     &      '("intercept Il not computed for line/channel: ",
     &      i4,2x,i2)') ilin,ich
            CALL ml_wt('warning',text)
              
          ENDIF 
          
        ENDDO           !DO ich=1,hrs_mxirchn
        
      ENDDO             !DO ilin=splintab(1),splintab(calib)-1     
               

!  For scan lines located before the first calib cycle 
      WRITE(hrs_cllgu,*)
      
      DO ilin=lin0,splintab(1)-1
        DO ich=1,hrs_mxirchn
          
          Il(ich,ilin) = Ic(ich,1)
          
          IF (iprint.GE.2) 
     &    WRITE(hrs_cllgu,'("ilin ich spk Ick Il",
     &    I4,1x,I2,1x,I4,1x,2(2X,F9.4))') 
     &    ilin,ich,splintab(1),
     &    Ic(ich,1),Il(ich,ilin)
             
          IF (Il(ich,ilin).EQ.missdp) THEN
            IF (iprint.GE.1) 
     &      WRITE(hrs_cllgu,
     &      '("intercept Il not computed for line/channel: ", 
     &      i4,2x,i2)') ilin,ich
            
            WRITE(text,
     &      '("intercept Il not computed for line/channel: ",
     &      i4,2x,i2)') ilin,ich
            CALL ml_wt('warning',text)
          ENDIF
          
        ENDDO           !DO ich=1,hrs_mxirchn
      ENDDO             !DO ilin=lin0,splintab(1)-1
        
! For scan lines located behind the last calib cycle
      WRITE(hrs_cllgu,*)
      
      DO ilin=splintab(calib),lin1
        DO ich=1,hrs_mxirchn
        
          Il( ich,ilin) = Ic(ich,calib)
                   
          IF (iprint.GE.2) 
     &    WRITE(hrs_cllgu,'("ilin ich spk Ick Il",
     &    I4,1x,I2,1x,I4,2(2X,F9.4))') 
     &    ilin,ich,splintab(calib),
     &    Ic(ich,calib),Il(ich,ilin)
          
          IF (Il(ich,ilin).EQ.missdp) THEN
            IF (iprint.GE.1) 
     &      WRITE(hrs_cllgu,
     &      '("intercept Il not computed for line/channel: ", 
     &      i4,2x,i2)') ilin,ich

            WRITE(text,
     &      '("intercept Il not computed for line/channel: ",
     &      i4,2x,i2)') ilin,ich
            CALL ml_wt('warning',text)
          ENDIF
          
        ENDDO           !DO ich=1,hrs_mxirchn  
      ENDDO             !DO ilin=splintab(calib),lin1
      
     
! If asked (hrs_ttflg), 
!----------------------
! first, computes the correction It'(n) to the intercept based  
!-------------------------------------------------------------
!       on the telescope temperature for each Earth view scan line ilin.
!-----------------------------------------------------------------------
! then, computes the final intercept 
!-----------------------------------
      IF (hrs_ttflg.GE.1) THEN
      
      IF (iprint.GE.1)WRITE(hrs_cllgu,*)
      IF (iprint.GE.1)WRITE(hrs_cllgu,
     &'("Computes the intercept based on the telescope temp.")') 
      text='Computes the intercept based on the telescope temp.'
      CALL ml_wt('info',text)

! Value of b1: in all light conditions, in day light conditions or 
! in night conditions.
! Checks the flag
      IF (hrs_ttflg.EQ.1) THEN
      
        DO ilin=lin0,lin1
          DO ich=1,hrs_mxirchn 
            b11(ich,ilin)=b1(ich)
          ENDDO  
        ENDDO

      ELSE IF (hrs_ttflg.EQ.2) THEN

        DO ilin=lin0,lin1
          solzen=(hang(82,ilin)+hang(85,ilin))/200.
          IF (solzen.LT.90.) THEN
!         Day light conditions at the nadir
            b11(ich,ilin)=b1d(ich)
          ELSE
!         Night conditions at the nadir                   
            b11(ich,ilin)=b1n(ich)
          ENDIF 
        ENDDO
        
      ENDIF 
       
!  For scan lines located behind the first calib cycle and before the
!  last calib cycle
       
      DO ilin=splintab(1),splintab(calib)-1
        
        k=1
        DO WHILE (ilin.GE.splintab(k).AND.k.LE.calib)
          k=k+1
        ENDDO

        nlin=ilin-splintab(k-1)
        
        DO ich=1,hrs_mxirchn
        
         IF( b11(ich,ilin).NE.0. .AND. b11(ich,ilin).NE.missreal
     &    .AND. sttp(ilin).NE.missreal 
     &    .AND. sttp(splintab(k-1)).NE.missreal .AND.
     &    sttp(splintab(k)).NE.missreal ) THEN          

          It(ich,ilin)=b11(ich,ilin)*(
     &          ( sttp(ilin)-sttp(splintab(k-1)) ) -
     &          nlin/40.0 * ( sttp(splintab(k))-sttp(splintab(k-1)) )
     &          )

         ELSE

          It(ich,ilin)=missdp
           
          IF (iprint.GE.1) 
     &    WRITE(hrs_cllgu,
     &    '("intercept It not computed for line/channel: ", 
     &    i4,2x,i2)') ilin,ich

          WRITE(text,
     &    '("intercept It not computed for line/channel: ",
     &    i4,2x,i2)') ilin,ich
          CALL ml_wt('warning',text)
          
         ENDIF 
          
        ENDDO           !DO ich=1,hrs_mxirchn
     
      ENDDO             !DO ilin=splintab(1),splintab(calib)-1
      
!  For scan lines located before the first calib cycle

      DO ilin=lin0,splintab(1)-1
        DO ich=1,hrs_mxirchn
        
          IF( b11(ich,ilin).NE.0. .AND. b11(ich,ilin).NE.missreal 
     &    .AND. sttp(ilin).NE.missreal 
     &    .AND. sttp(splintab(1)).NE.missreal ) THEN          
          
           It(ich,ilin) =b11(ich,ilin)*( sttp(ilin)-sttp(splintab(1)) )

          ELSE
 
           It(ich,ilin) = missdp
           
           IF (iprint.GE.1)
     &     WRITE(hrs_cllgu,
     &     '("intercept It not computed for line/channel: ", 
     &     i4,2x,i2)') ilin,ich
 
           WRITE(text,
     &     '("intercept It not computed for line/channel: ",
     &     i4,2x,i2)') ilin,ich
           CALL ml_wt('warning',text)
           
          ENDIF 
          
        ENDDO           !DO ich=1,hrs_mxirchn
      ENDDO             !DO ilin=lin0,splintab(1)-1
  
! For scan lines located behind the last calib cycle

      DO ilin=splintab(calib),lin1
        DO ich=1,hrs_mxirchn
        
          IF( b11(ich,ilin).NE.0. .AND. b11(ich,ilin).NE.missreal 
     &    .AND. sttp(ilin).NE.missreal 
     &    .AND. sttp(splintab(calib)).NE.missreal ) THEN   
         
            It(ich,ilin) =b11(ich,ilin)
     &                    *( sttp(ilin)-sttp(splintab(calib)) )

          ELSE
 
            It(ich,ilin) =missdp
         
            IF (iprint.GE.1)
     &      WRITE(hrs_cllgu,
     &      '("intercept It not computed for line/channel: ", 
     &      i4,2x,i2)') ilin,ich
          
            WRITE(text,
     &      '("intercept It not computed for line/channel: ",
     &      i4,2x,i2)') ilin,ich
            CALL ml_wt('warning',text)
          
          ENDIF 
           
        ENDDO           !DO ich=1,hrs_mxirch   
      ENDDO             !DO ilin=splintab(calib),lin1
      
! Computes the final intercept with a correction computed from the 
! telescope temperature   I(ilin)=Il(ilin)+It(ilin) 
      IF (iprint.GE.1)WRITE(hrs_cllgu,*)
      IF (iprint.GE.1)WRITE(hrs_cllgu,
     &'("Computes the final intercept ")')
      
      DO ilin=lin0,lin1
        DO ich=1,hrs_mxirchn

          IF (Il(ich,ilin).NE.missdp  .AND.  
     &          It(ich,ilin).NE.missdp) THEN

             calibcoef(ich,3,ilin)=Il(ich,ilin)+It(ich,ilin)
            
          ELSE IF (Il(ich,ilin).NE.missdp) THEN
            
             calibcoef(ich,3,ilin)=Il(ich,ilin)
             
             IF (iprint.GE.1) 
     &       WRITE(hrs_cllgu,
     &       '("final intercept computed with ",
     &         "linear term for line/channel: ", 
     &        i4,2x,i2)') ilin,ich
          
             WRITE(text,
     &       '("final intercept computed with ",
     &         "linear term for line/channel: ",
     &       i4,2x,i2)') ilin,ich
             CALL ml_wt('warning',text)
   
          ELSE
             calibcoef(ich,3,ilin)=missdp
               
             IF (iprint.GE.1) 
     &       WRITE(hrs_cllgu,
     &       '("final intercept not computed for line/channel: ", 
     &        i4,2x,i2)') ilin,ich
          
             WRITE(text,
     &       '("final intercept not computed for line/channel: ",
     &       i4,2x,i2)') ilin,ich
             CALL ml_wt('warning',text)
             
          ENDIF
         
        ENDDO    
      ENDDO    

      ELSE      !IF (hrs_ttflg.GE.1) THEN
      
! Computes the final intercept with no correction computed from the 
! telescope temperature   I(ilin)=Il(ilin)
      IF (iprint.GE.1)WRITE(hrs_cllgu,*)
      IF (iprint.GE.1)WRITE(hrs_cllgu,
     &'("Computes the final intercept ")')
     
      DO ilin=lin0,lin1
        DO ich=1,hrs_mxirchn
        
          IF (Il(ich,ilin).NE.missdp) THEN
     
            calibcoef(ich,3,ilin)=Il(ich,ilin)
 
          ELSE

            calibcoef(ich,3,ilin)=missdp
               
            IF (iprint.GE.1)
     &      WRITE(hrs_cllgu,
     &      '("final intercept not computed for line/channel: ", 
     &       i4,2x,i2)') ilin,ich
          
            WRITE(text,
     &      '("final intercept not computed for line/channel: ",
     &      i4,2x,i2)') ilin,ich
            CALL ml_wt('warning',text)
             
          ENDIF
         
        ENDDO    
      ENDDO      
      
      IF (iprint.GE.1)
     &  WRITE(hrs_cllgu,
     &  '("intercept no fonction of the telescope temp.")') 
    
       text='Intercept no fonction of the telescope temp.'
       CALL ml_wt('info',text)
      
      ENDIF     !IF (hrs_ttflg.GE.1) THEN
           
! For the visible channel
      ich=20
      DO ilin=lin0,lin1
        calibcoef(20,3,ilin)=calibcoef0(20,3,1)
        b11(20,ilin)=0.
      ENDDO
        
      
! Updates the quality area
      DO ilin=lin0,lin1
        DO ich=1,hrs_mxchn
          IF (calibcoef(ich,3,ilin).EQ.missdp) THEN
            linechquality(1,ich,ilin)=1
          ELSE
            linechquality(1,ich,ilin)=0
          ENDIF
        ENDDO
      ENDDO       
 
      IF (iprint.GE.2) THEN
        DO ilin=lin0,lin1
          DO ich=1,hrs_mxirchn
            WRITE(hrs_cllgu,fmt=500) 
     &      ilin,ich,Il(ich,ilin),It(ich,ilin),calibcoef(ich,3,ilin)
          ENDDO
        ENDDO             
      ENDIF
      
! Stores the final (interpolated) intercepts as primary calibration 
!------------------------------------------------------------------
! coefficients
!-------------

      DO ilin=lin0,lin1
        
        DO ich=1,hrs_mxchn
          
          IF (calibcoef(ich,3,ilin).NE.missdp) THEN
            coef0(ich)=IDINT(calibcoef(ich,3,ilin)*1.d+06)
          ELSE
            coef0(ich)=missint
          ENDIF
        
        ENDDO           !DO ich=1,hrs_mxchn
                
        DO ich=1,hrs_mxchn
          hcalcof(3,ich,ilin)=coef0(l1border(ich))
        ENDDO
        
      ENDDO             !DO ilin=lin0,lin1 
      
! Stores the linear intercepts as secondary calibration 
!------------------------------------------------------
! coefficients
!-------------
      
      DO ilin=lin0,lin1
        
        DO ich=1,hrs_mxirchn
          
          IF (Il(ich,ilin).NE.missdp) THEN
            coef0(ich)=IDINT(Il(ich,ilin)*1.d+06)
          ELSE
            coef0(ich)=missint
          ENDIF
        
        ENDDO           !DO ich=1,hrs_mxirchn
        
        coef0(20)=IDINT(calibcoef0(20,3,1)*1.d+06)
        
        DO ich=1,hrs_mxchn
          hscalcof(3,ich,ilin)=coef0(l1border(ich))
        ENDDO
        
      ENDDO             !DO ilin=lin0,lin1
      
! Stores the recomputed intercepts as primary calibration 
!--------------------------------------------------------
! coefficients and as secondary calibration coefficients
!-------------------------------------------------------
! along with the spaceview scan-lines
!------------------------------------

      DO icalib=1,calib
      
        splin=splintab(icalib)
        
        DO ich=1,hrs_mxirchn
        
          IF (Ic(ich,icalib).NE.missdp) THEN
            coef0(ich)=IDINT(Ic(ich,icalib)*1.d+06)
          ELSE
            coef0(ich)=missint
          ENDIF
        
        ENDDO           !DO ich=1,hrs_mxirchn
        
        coef0(20)=IDINT(calibcoef0(20,3,icalib)*1.d+06)
        
        DO ich=1,hrs_mxchn
          hcalcof(3,ich,splin)=coef0(l1border(ich))
          hscalcof(3,ich,splin)=coef0(l1border(ich))
        ENDDO
        

! Stores the BB (raw) intercepts as primary calibration 
!------------------------------------------------------
! coefficients and as secondary calibration coefficients
!-------------------------------------------------------
! along with the blackbody scan-lines
!------------------------------------
      
        ilin=iwtlintab(icalib)
        
        DO ich=1,hrs_mxchn
        
          IF (calibcoef0(ich,3,icalib).NE.missdp) THEN
            coef0(ich)=IDINT(calibcoef0(ich,3,icalib)*1.d+06)
          ELSE
            coef0(ich)=missint 
          ENDIF
        
        ENDDO           !DO ich=1,hrs_mxchn 
                  
        DO ich=1,hrs_mxchn
          hcalcof(3,ich,splin)=coef0(l1border(ich))
          hscalcof(3,ich,splin)=coef0(l1border(ich))
        ENDDO
      
      ENDDO             !DO icalib=1,calib 
      
       
 500  FORMAT('ilin ich  Il  It  Ifinal: ',I4,2X,I2,3(2X,F9.4))                

      RETURN
      END    
          
