	SUBROUTINE h_iwtrad_algoV4(iwttmp,iwtrad)

!------------------------------------------------------------------------------
!NAME $Id: h_iwtrad_algoV4.F 24 2006-02-03 11:51:40Z frna $
!
!PURPOSE 
!    The IWT temperatures are converted into radiance for each HIRS channel. 
!
!DESCRIPTION
!    The IWT radiances are computed from the Planck relation.
!
!    First the "effective" temperature is computed:
!         T* = b + cT
!    where b and c are channel dependent coefficients (the so-called 
!    band-correction coefficients). b and c are computed before the launch 
!    of the satellite.
!    In AAPP, b and c are read in the h_calcoef_algoV4.dat file and ranged
!    in the include hcinit_algoV4.h
!    T is the average temperature of the internal target.
!
!    Then, the Planck relation is applied.
!       Rad = c1*(Central_Wave)^3/[exp(c2*Central_Wave'T*) - 1]
!    where c1 and c2 are the 2 constant terms.
!    In AAPP, c1 and c2 are read in the h_calcoef_algoV4.dat file and ranged
!    in the include hcinit_algoV4.h, the same for the central waves.
!     
!    This module is very similar to the module h_iwtrad.
!    It was created to manage the HIRS calibration algorithm version4.0
!
!    Reference: NOAA KLM User's Guide Section 7.2
!
!HISTORY (CHANGES) 
!    (version   date          comment           <author>)
!      1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!              
!         
!ARGUMENTS 
!     inputs :
!      DBLE PREC iwttmp(hrs_mxcalib)  !IWT temperatures  
!
!     outputs :
!      DBLE PREC iwtrad(hrs_mxirchn,hrs_mxcalib)   
!                                     !IR channel radiances of the IWT
!
!UPDATED ELEMENTS OF INCLUDES
!    in include hcalibcycleinfo_algoV4.h: 
!     as input/output: logical qualified(hrs_mxchn,hrs_mxcalib)
!                                 !=true if the calib cycle is usable
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                    
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"

! argument variables
      DOUBLE PRECISION iwttmp(hrs_mxcalib)
      DOUBLE PRECISION iwtrad(hrs_mxirchn,hrs_mxcalib)

! local variables
      CHARACTER*80 text        !variable text
      INTEGER icalib,ich       !index of loops
      INTEGER lin              !number of the current IWT scan line in the orbit
      DOUBLE PRECISION iwtefftmp(hrs_mxirchn,hrs_mxcalib) 
                               !"effective" IWT temperature for the IR channels
      DOUBLE PRECISION planck1,planck2 
                               !intermediary variables for the Planck function

!--END OF DECLARATIONS----------------------------------------------------------

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '**  SUB. H_IWTRAD_ALGOV4 **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      
! Initializes the output
      DO ich=1,hrs_mxirchn
        DO icalib=1,hrs_mxcalib
          iwtrad(ich,icalib)=missdp
        ENDDO
      ENDDO       

! For each calib cycle
      DO icalib=1,calib
      
! For each qualified calib cycle        
      IF (qualified(1,icalib)) THEN
      
       lin=iwtlintab(icalib)

       IF (iprint.GE.2) write(hrs_cllgu,fmt=500) lin
       IF (iprint.GE.2) write(hrs_cllgu,fmt=510) iwttmp(icalib)


! For each IR channel in a qualified calib cycle
       DO ich=1,hrs_mxirchn

         IF (qualified(ich,icalib) .AND. 
     &               iwttmp(icalib) .NE. missdp) THEN

! The IWT temperature is replaced by an "effective" temperature
           iwtefftmp(ich,icalib)=bdcorr(ich,1)+bdcorr(ich,2)
     &                           *iwttmp(icalib)


! Applies Planck function
           planck1 = plck(1)*(cwave(ich)**3)
           planck2 = dexp( plck(2)*cwave(ich)/iwtefftmp(ich,icalib))-1
           iwtrad(ich,icalib) = planck1 / planck2
                  
         ELSE
          
!          !iwtrad(ich,icalib) already initialized at the value missdp
           
           qualified(ich,icalib) =.FALSE.
           
           IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=520)icalib,ich
           IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=530)
           
           WRITE(text,'("Calib cycle ",I2," channel ",I2,
     &     ": No IWT radiance")')icalib,ich 
           CALL ml_wt('warning',text)
           WRITE(text,'("==> No qualified calib cycle")')
           CALL ml_wt('warning',text)     

         ENDIF

         IF(iprint.GE.2) WRITE(hrs_cllgu,fmt=540) ich,iwtrad(ich,icalib)


       ENDDO	!DO ich=1,hrs_mxirchn
       
      ELSE	!IF (qualified(1,icalib)) THEN
        
!          !iwtrad(ich,icalib) already initialized at the value missdp
         
         IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=550)icalib
         IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=560)
         
         WRITE(text,'("No qualified calib cycle ",I2)')icalib
         CALL ml_wt('warning',text) 
         WRITE(text,'("==>No IWT radiance for all the channels")')
         CALL ml_wt('warning',text)       
       
      ENDIF	!IF (qualified(1,icalib)) THEN
      
      ENDDO	!DO icalib=1,calib

 500  FORMAT(2X,'IWT scan line number: ',I4)
 510  FORMAT(2X,'IWT temperature: ',F12.5) 
 520  FORMAT(2X,'Calib cycle ',I2,' channel ',I2,': No IWT radiance')
 530  FORMAT(2X,'==>No qualified calib cycle')    
 540  FORMAT(5X,'IR channel ',I2,3X,'IWT rad: ',F17.7)
 550  FORMAT(2X,'No qualified calib cycle ',I2)
 560  FORMAT(2X,'==>No IWT radiance for all the channels')
  
      RETURN
      END

       
         
