	SUBROUTINE h_iwttmp_algoV4(iwttmp) 

!------------------------------------------------------------------------------
!NAME $Id: h_iwttmp_algoV4.F 24 2006-02-03 11:51:40Z frna $
!
!PURPOSE 
!    For each IWT scan line, this module computes the IWT temperatures. 
!
!DESCRIPTION     
!    For each of the active PRTs, this module computes the mean of a sample of  
!    PRT readings from the internal warm target (IWT) scan line and from a
!    specified number of scan lines before the IWT scan line and an other    
!    specified number of scan lines after the IWT scan line. 
!    The specified number is read in the h_testcoef_algoV4.dat and ranges
!    in the include hcinit_algoV4.h.
!    Tests: -the difference between the maximum PRT readings value and the
!            minimum one must be included in a limit prtminmax (hcinit_algoV4.h)
!           -minimum number of valid PRT readings to compute a mean 
!            (prtminnb,hcinit_algoV4.h)
!    After,PRT readings means are converted to temperatures, using the 
!    conversion coefficients(prtcnttmp(5,6), hcinit_algoV4.h).
!    The final IWT temperature is computed by averaging the temperatures from 
!    the individual active PRTs.
!
!    Note that we have 4 PRTs for HIRS/3 and 5 PRTs for HIRS/4
!
!    This module is very similar to the module h_iwttmp.
!    It was created to manage the HIRS calibration algorithm version4.0     
!             
!HISTORY (CHANGES) 
!    (version    date          comment           <author>)
!       1.0     January 05     creation          T. Labrot (MF)
!       1.1     10/06/05     modify the loop (k=3,7 --> 13,17) to get the 
!                            counts of PRT nb 5  T. LABROT (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.              
!         
!ARGUMENTS 
!   inputs :
!
!   outputs :
!    DBLE PREC iwttmp(hrs_mxcalib)  !IWT temperatures 
!
!UPDATED ELEMENTS OF INCLUDES
!    in include hcalibcycleinfo_algoV4.h: 
!      LOGICAL qualified(hrs_mxchn,hrs_mxcalib)  !=true if the calib cycle is usable
!      DBLE PREC tmp(hrs_mxprtnb,hrs_mxcalib)    !Temperature of mxprtnb PRTs
!      DBLE PRECprtmean(hrs_mxprtnb,hrs_mxcalib) !Mean of the PRT readings
!      DBLE PREC prtstd(hrs_mxprtnb,hrs_mxcalib) !Standard deviation of the PRT readings
!      INTEGER datanb(hrs_mxprtnb,hrs_mxcalib)   !Initial number of PRT readings
!      INTEGER goodprt(hrs_mxprtnb,hrs_mxcalib)  !Number of good PRT readings
!      INTEGER ejecprt(hrs_mxprtnb,hrs_mxcalib)  !Number of rejected PRT readings
!        
!    in include hcaliblineinfo_algoV4.h: 
!      INTEGER linequality(10,hrs_mxlin) 
!                            !quality array for each scan line to update
!                            !quality flags in the HIRS level 1b
!                            !1: scan line no usable because the instrument
!                            !   status is OFF
!                            !2: scan line no usable because the bit 31 (do not
!                            !   use scan for product generation) is ON
!                            !3: insufficient data for calibration
!                            !4: scan was not calibrated
!                            !5: scan contains marginal calibration in some 
!                            !   of the IR channels
!                            !6: PRT quality test failed
!                            !7: PRT data marginal, some readings were rejected
!                            !8: scan contains some uncalibrated channels
!                            !9: scan indicates that the normal HIRS calibration 
!                            !   sequence is disabled 
!                            !10: space view scan is lunar contaminated
!       INTEGER linechquality(6,hrs_mxchn,hrs_mxlin)                       
!                            !1: calibration failed
!                            !2: calibration marginal
!                            !3: calibration slope came from the HCF
!                            !4: BB views failed NEDC test
!                            !5: space views failed NEDC test
!                            !6: some QC (quality control) tests could not be 
!                            !   applied                
!
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! LOGICAL FUNCTION gp_btn      !give true or false according to the value of the bit
! SUBROUTINE h_prtstat_algoV4  !compute mean,max,min of the PRT data
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------
      
      IMPLICIT NONE
                                                      
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
      DOUBLE PRECISION iwttmp(hrs_mxcalib)
      
! fumctions
      LOGICAL gp_btn
      
! local variables
      CHARACTER*80 text   !variable text

      INTEGER iprt        !index of the PRTs
      INTEGER i,ii,ich    !index of loops
      INTEGER icalib      !index of loop over good calibration cycle
  
      INTEGER lin         !number of one IWT scan line in the orbit
      INTEGER ilin        !number of the current scan line (loop)
      INTEGER ilin0,ilin1 !numbers of the first and the last scan lines in the
                          !orbit where we read PRT data
      INTEGER k,k0        !index to read the data    
      INTEGER prtreadnb(prtnb)   !at the end of the loop and for each PRT,total
                          !number of readings,max value=(prtbfore+prtafter+1)*5
      INTEGER goodprtnb   !number of readings used to compute the mean
      INTEGER ejecprtnb   !number of rejected PRT readings for one PRT 

      INTEGER imindt,imaxdt !index of the min and the max values of PRT readings
      INTEGER goodtmp     !at the end of the loop, number of good temp.

      DOUBLE PRECISION temp        !temperature of 1 PRT
      DOUBLE PRECISION dtmean      !mean of readings from one PRT
      DOUBLE PRECISION dtstd       !standard deviation of readings from one PRT
 

      INTEGER*2 prtread(prtnb,5*(prtbfore+prtafter+1)) 
                                   !array containing PRT readings 
      INTEGER*2  mindt             !minimum value of PRT readings
      INTEGER*2  maxdt             !maximum value of PRT readings

!--END OF DECLARATIONS----------------------------------------------------------
      
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '*************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '*  SUB H_IWTTMP_ALGOV4  *'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '*************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      
      text='Computes IWT temp. for each IWT scan line'
      CALL ml_wt('info',text)
      
!-----------------------
! Initializes the output
!-----------------------
      DO icalib=1,hrs_mxcalib
        iwttmp(icalib)=missdp
      ENDDO

!------------------------------------------------------------------------
! For each IWT scan line (or each calib cycle), computes the temperatures
!------------------------------------------------------------------------ 
      
      DO icalib=1,calib
      
      IF (qualified(1,icalib)) THEN

        lin=iwtlintab(icalib)

        IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=500) lin
        

! Scan lines where we take PRT readings.
!---------------------------------------
        ilin0=max(lin0,(lin-prtbfore))
        ilin1=min((lin+prtafter),lin1)

        IF(iprint.GE.2) WRITE(hrs_cllgu,fmt=510) ilin0,ilin1

! Gets the PRT readings for each PRT.
!-----------------------------------
        
! Initializes or reinitializes arrays

        DO iprt=1,prtnb
          DO ii=1,(prtbfore+prtafter+1)*5
            prtread(iprt,ii)=missint2
            prtreadnb(iprt) =0
          ENDDO
        ENDDO
        
        
        DO ilin=ilin0,ilin1
                
! Test: data of this scan line are usable or not by checking bit 31 of the 
!       quality indicator bit field. If not, skip to the next line

          IF (gp_btn(hqualind(ilin),31)) THEN
          
            linequality(2,ilin)=1 
	    
	    IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=520)
            IF(iprint.GE.1) WRITE(hrs_cllgu,fmt=530)
	    
	    WRITE(text,'("line ",I4," bit31=1 in the quality '//
     &	    'indicator bit field")')ilin
            CALL ml_wt('notice',text)
            WRITE(text,'("-- do not use scan for product generation")')
            CALL ml_wt('notice',text)
            
          ELSE
          
            linequality(2,ilin)=0

! For each PRT
            DO iprt=1,prtnb

! Test: controls switch indicates that a PRT has been turned off or not.
              IF (prtwght(iprt).EQ.1) THEN              

! Gets the 5 readings by scan line, without test. Count the readings (prtreadnb)
               IF (iprt.LE.4) THEN 
                 IF (iprt.EQ.1) k0=3
                 IF (iprt.EQ.2) k0=8
                 IF (iprt.EQ.3) k0=13
                 IF (iprt.EQ.4) k0=18

                 DO k=k0,(k0+4)
                  prtreadnb(iprt) = prtreadnb(iprt) + 1
                  prtread(iprt,prtreadnb(iprt)) = helem(k,58,ilin)
                 ENDDO
               ENDIF
               
               IF (iprt.EQ.5) THEN
                DO k=13,17
                 prtreadnb(iprt) = prtreadnb(iprt) + 1
                 prtread(iprt,prtreadnb(iprt)) =helem(k,59,ilin)
                ENDDO 
               ENDIF 
                                 
              ELSE	!IF (prtwght(iprt).EQ.1) THEN 
              
               IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=540) iprt
               WRITE(text,'("Not working PRT number ")')
               CALL ml_wti('notice',text,iprt)
               
              ENDIF     !IF (prtwght(iprt).EQ.1) THEN

            ENDDO	!DO iprt=1,prtnb

          ENDIF		!IF (gp_btn(hqualind(ilin),31)) THEN 

        ENDDO		!DO ilin=ilin0,ilin1
 
        IF (iprint.GE.2) THEN
          WRITE(hrs_cllgu,fmt=550) prtreadnb
          IF (prtnb.EQ.4) THEN
           DO i=1,5*(ilin1-ilin0+1)
            WRITE(hrs_cllgu,*)'   ',prtread(1,i),'   ',
     &      prtread(2,i),'   ',prtread(3,i),'   ',prtread(4,i)
           ENDDO
          ENDIF
          IF (prtnb.EQ.5) THEN
           DO i=1,5*(ilin1-ilin0+1)
            WRITE(hrs_cllgu,*)'   ',prtread(1,i),'   ',
     &      prtread(2,i),'   ',prtread(3,i),'   ',prtread(4,i),
     &      prtread(5,i)
           ENDDO
          ENDIF
        ENDIF  


! If enough data, computes the mean of the PRT readings, the maximum and minimum
!------------------------------------------------------------------------------- 
! values of the readings.
!------------------------
        DO iprt=1,prtnb
 
          dtmean=missdp
          dtstd =missdp
          goodprtnb = missint
          ejecprtnb = missint
          mindt=missint2
          maxdt=missint2
          imindt=0
          imaxdt=0
 
          
          IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=560) iprt

          IF (prtreadnb(iprt).GE.prtminnb) THEN

! Every thing is OK

            CALL h_prtstat_algoV4(iprt,prtnb,prtread,prtreadnb, dtmean,
     &                  dtstd,goodprtnb,mindt,maxdt,imindt,imaxdt)
     
            IF (iprint.GE.2) THEN
              WRITE(hrs_cllgu,fmt=570) dtmean,dtstd
              WRITE(hrs_cllgu,fmt=580) maxdt
              WRITE(hrs_cllgu,fmt=590) mindt
            ENDIF

! Tests with prtminmax (max difference between max value and min one)
            ejecprtnb = 0

            DO WHILE ((maxdt-mindt).GT.prtminmax .AND.
     &                 dtmean.NE.missdp)

              IF (iprint.GE.2)
     &            WRITE(hrs_cllgu,*) '    "do while loop" '
 
              ejecprtnb = ejecprtnb + 1 

! Deletes the value that is furthest from the mean.
             IF ((maxdt-dtmean).GT.(dtmean-mindt)) THEN
               prtread(iprt,imaxdt)=missint2
             ENDIF
             IF ((maxdt-dtmean).LT.(dtmean-mindt)) THEN
               prtread(iprt,imindt)=missint2
             ENDIF
             IF ((maxdt-dtmean).EQ.(dtmean-mindt)) THEN
               prtread(iprt,imaxdt)=missint2
               prtread(iprt,imindt)=missint2
             ENDIF

! Enough filtered PRT readings to compute an average

             IF ((goodprtnb-1).GE.prtminnb) THEN

              IF (iprint.GE.2) THEN
               WRITE(hrs_cllgu,fmt=550) prtreadnb
               IF (prtnb.EQ.4) THEN
                DO i=1,5*(ilin1-ilin0+1)
                 WRITE(hrs_cllgu,*)'   ',prtread(1,i),'   ',
     &           prtread(2,i),'   ',prtread(3,i),'   ',prtread(4,i)
                ENDDO
               ENDIF
               IF (prtnb.EQ.5) THEN
                DO i=1,5*(ilin1-ilin0+1)
                 WRITE(hrs_cllgu,*)'   ',prtread(1,i),'   ',
     &           prtread(2,i),'   ',prtread(3,i),'   ',prtread(4,i),
     &           prtread(5,i)
                ENDDO
               ENDIF
              ENDIF
               
               CALL h_prtstat_algoV4(iprt,prtnb,prtread,prtreadnb,    
     &             dtmean,dtstd,goodprtnb,mindt,maxdt,imindt,imaxdt)


               IF (iprint.GE.2) THEN
                 WRITE(hrs_cllgu,fmt=570) dtmean,dtstd
                 WRITE(hrs_cllgu,fmt=580) maxdt
                 WRITE(hrs_cllgu,fmt=590) mindt
               ENDIF


             ELSE	!IF ((goodprtnb-1).GE.prtminnb) THEN
! Not enough filtered PRT readings to compute the mean

               IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=600) 
     
               dtmean = missdp
               dtstd  = missdp
               goodprtnb = missint
               ejecprtnb = missint

             ENDIF	!IF ((goodprtnb-1).GE.prtminnb) THEN 

           ENDDO	!end of the loop "do while"

      
         ELSE
! This PRT has been turned off or no enough data for this PRT 
           IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=610) iprt

           dtmean = missdp
           dtstd  = missdp
           goodprtnb = missint
           ejecprtnb = missint

         ENDIF

         IF (iprint.GE.2) THEN        
           WRITE(hrs_cllgu,fmt=620) dtmean
           WRITE(hrs_cllgu,fmt=630) prtreadnb(iprt)
           WRITE(hrs_cllgu,fmt=640) goodprtnb
           WRITE(hrs_cllgu,fmt=650) ejecprtnb
         ENDIF

! Convertes the mean PRT reading counts to temperature
!-----------------------------------------------------

         IF (dtmean.NE.missdp) THEN
           temp=0.d+0
           DO ii=1,6
             temp=temp+
     &         (iwtprtcnttmp(iprt,ii)*(dtmean**(ii-1)))
           ENDDO 
           temp=temp+273.15
         ELSE
           temp = missdp
         endif


         IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=660) iprt,temp
         
! Puts the values in final arrays
!--------------------------------         
          tmp(iprt,icalib)=temp
          prtmean(iprt,icalib)=dtmean
          prtstd(iprt,icalib)=dtstd
          datanb(iprt,icalib)=prtreadnb(iprt)
          goodprt(iprt,icalib)=goodprtnb
          ejecprt(iprt,icalib)=ejecprtnb
          
       ENDDO	!DO iprt=1,prtnb
       
! Updates linequality: If for only 1 PRT (on the prtnb of existing PRTs), 
!--------------------  the number of rejected data is >0, then the flag is on  
       
       linequality(7,iwtlintab(icalib))=0
       DO iprt=1,prtnb
         IF (ejecprt(iprt,icalib).NE.0) 
     &           linequality(7,iwtlintab(icalib))=1 
       ENDDO 
            

! Averages the temperatures of active PRTs if at least numprtlm PRTs have 
!------------------------------------------------------------------------
! survived the different test
!----------------------------
       goodtmp=0
       iwttmp(icalib) = 0.d+0
       DO iprt=1,prtnb
         IF (tmp(iprt,icalib) .NE. missdp) THEN
           iwttmp(icalib) = iwttmp(icalib) + tmp(iprt,icalib)
           goodtmp = goodtmp + 1
         ENDIF
       ENDDO

       IF (goodtmp.GE.numprtlm) THEN
       
          iwttmp(icalib) = iwttmp(icalib)/goodtmp
          
          linequality(6,iwtlintab(icalib))=0 
          
          IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=670) iwttmp(icalib)

       
       ELSE
          
          iwttmp(icalib) = missdp
          
          DO ich=1,hrs_mxchn
            qualified(ich,icalib)=.FALSE.
          ENDDO
          
          linequality(6,iwtlintab(icalib))=1  

          IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=680)icalib
          WRITE(text,'("Calib cycle ",I2," no IWT temperature ")')icalib
          CALL ml_wt('warning',text) 
          WRITE(text,'("==>No qualified calib cycle")')
          CALL ml_wt('warning',text)        

       ENDIF


      ELSE		!IF (qualified(1,icalib)) THEN
       
!        !iwttmp(icalib) already initialized to missdp

         IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=690)icalib
         WRITE(text,'("No qualified calib cycle ",I2)')icalib
         CALL ml_wt('info',text)
         WRITE(text,'("==>No IWT temperature")')
         CALL ml_wt('info',text)        
         
      ENDIF		!IF (qualified(1,icalib)) THEN
 
      ENDDO		!DO icalib=1,calib
      
 500  FORMAT(/2X,'IWT SCAN LINE NUMBER: ',I4)
 510  FORMAT(4X,'PRT readings come from line ',I4,' to line ',I4)
 520  FORMAT(8X,'bit31=1 in the quality indicator bit field')
 530  FORMAT(8X,'-- do not use scan for product generation')
 540  FORMAT(/2X,'PRT nb ',I1,' has been turned off')
 550  FORMAT(/4X,'PRT readings total number: ',5(I3))
 560  FORMAT(/4X,'PRT NUMBER: ',I1)
 570  FORMAT(/9X,'PRT readings mean and standard dev.:',F12.5,3x,D14.7)
 580  FORMAT(9X,'PRT readings max value:',I6)
 590  FORMAT(9X,'PRT readings min value:',I6)
 600  FORMAT(/5X,'not enough filtered PRT readings to',
     & ' compute an average')
 610  FORMAT(/5X,'PRT nb ',I1,' not working or no enough readings')
 620  FORMAT(9X,'filtered PRT readings mean: ',F12.5)
 630  FORMAT(9X,'initial number of PRT readings: ',I4)
 640  FORMAT(9X,'number of filtered PRT readings: ',I4)
 650  FORMAT(9X,'number of rejected filtered PRT readings: ',I4)
 660  FORMAT(/9X,'temperature of the PRT nb',I2,': ',F12.5)
 670  FORMAT(//3X,' IWT TEMPERATURE: ',F12.5)
 680  FORMAT(2X,'Calib cycle ',I2,' no IWT temperature ',
     &          '==>No qualified calib cycle')
 690  FORMAT(2X,'No qualified calib cycle ',I2,
     &       '==>NO IWT temperature')    

      RETURN
      END

    
        

     


 

         
