      SUBROUTINE h_limit_algoV4(ich,sp1fv,counts,cntlm,infcnt,supcnt,
     &misscnt)

!------------------------------------------------------------------------------
!NAME $Id: h_limit_algoV4.F 25 2006-02-03 15:43:53Z frna $
!
!PURPOSE 
!    This module checks the output counts values that must be 
!    inside limits.
!
!DESCRIPTION  
!    HIRS instruments have contamination in the first xx FOVs of the space
!    scan. The variable sp1fv (h_testcoef_algoV4.dat file and after in the
!    include hcinit_algoV4.h) indicates the first field of view in the
!    space scan that is not contaminated. The same FOVS are eliminated 
!    from the scan of the warm target.
!    The gross limits are defined in the h_calcoef_algoV4.dat
!    file and after in the include hcinit_algoV4.h.
!
!    This module is very similar to the module h_limit.
!    It was created to manage the HIRS calibration algorithm version4.0
!
!HISTORY (CHANGES) 
!    (version    date          comment           <author>)
!       1.0     January 05     creation          T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.          
!         
!ARGUMENTS 
!    inputs :
!     INTEGER ich          !number of the current channel
!     INTEGER sp1fv        !the first field of view in the space scan that
!                           is not contaminated.
!     INTEGER cntlm(hrs_mxirchn,2) !up and low output count limits
!
!    inputs/outputs:
!     INTEGER counts(hrs_mxpix)    !countd for one channel and 56 FOVs output counts
!
!    outputs :
!     INTEGER infcnt       !counter of counts lower than min limit
!     INTEGER supcnt       !counter of counts greater than max limit
!     INTEGER misscnt      !counter of missing count data
!
!!UPDATED ELEMENTS OF INCLUDES
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!      
!------------------------------------------------------------------------------


!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"

! argument variables
      INTEGER ich,sp1fv,counts(hrs_mxpix),cntlm(hrs_mxirchn,2)
      INTEGER infcnt,supcnt,misscnt

! local variables
      CHARACTER*80 text       !variable text 
      INTEGER i               !index of loop
      INTEGER diffmin,diffmax !counter when CN values are near limits
      INTEGER countsinf(hrs_mxpix)  !array of output counts inferior to the min limit
      INTEGER countssup(hrs_mxpix)  !array of output counts superior to the max limit
      INTEGER countsinfmin          !min value of countsinf(hrs_mxpix)
      INTEGER countssupmax          !max value of countssup(hrs_mxpix)
      DOUBLE PRECISION countsinfmn  !mean of output counts inferior to the min limit
      DOUBLE PRECISION countsinfstd !standard deviation of output counts inferior to the min limit
      DOUBLE PRECISION countssupmn  !mean of output counts superior to the max limit
      DOUBLE PRECISION countssupstd !standard deviation of output counts superior to the max limit

!--END OF DECLARATIONS---------------------------------------------------------

      infcnt = 0
      supcnt = 0
      misscnt= 0
      
      diffmin=0
      diffmax=0
      countsinfmin=missint
      countssupmax=-missint
      countsinfmn =0.d+0
      countsinfstd=0.d+0
      countssupmn =0.d+0
      countssupstd=0.d+0
      DO i=1,hrs_mxpix
       countsinf(i)=0
       countssup(i)=0
      ENDDO 
      
      
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     & '   *sub H_LIMIT_ALGOV4*'

      DO i=sp1fv,hrs_mxpix

        IF (counts(i).NE.missint) THEN

          IF (counts(i).LT.cntlm(ich,1)) THEN
          
            IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=500) counts(i),i
            infcnt = infcnt + 1
            countsinf(infcnt)=counts(i)
            counts(i) = missint
            
          ELSE
            
            IF(counts(i).LT.cntlm(ich,1)+5) diffmin=diffmin+1
           
            IF (counts(i).GT.cntlm(ich,2)) THEN
            
             IF (iprint.GE.1) WRITE(hrs_cllgu,fmt=510) counts(i),i
             supcnt = supcnt + 1
             countssup(supcnt)=counts(i)
             counts(i) = missint
             
            ELSE
             
             IF(counts(i).GT.(cntlm(ich,2)-5)) diffmax=diffmax+1
             
            ENDIF
            
          ENDIF
           
        ELSE    !IF (counts(i).NE.missint) THEN

          misscnt =  misscnt + 1

        ENDIF   !IF (counts(i).NE.missint) THEN

      ENDDO     !DO i=sp1fv,hrs_mxpix
      

! Various printed messages to follow the evolution of the output counts

      IF (infcnt.GT.0) THEN
      
        DO i=1,infcnt
         countsinfmn  = countsinfmn  + countsinf(i)
         countsinfstd = countsinfstd + countsinf(i)*countsinf(i)
         IF (countsinf(i).lt.countsinfmin) countsinfmin=countsinf(i)
        ENDDO
        
        countsinfmn  = countsinfmn/infcnt
        countsinfstd = (countsinfstd/infcnt)-(countsinfmn**2)
        IF (countsinfstd .GE. 0.d+0) countsinfstd=dsqrt(countsinfstd)
        
        WRITE(text,'("ch",I2,": ",I2," CN values are inf to limit")')
     &   ich,infcnt
        CALL ml_wt('warning',text)
        WRITE(text,'("  min:",I7,"  counts mean/stdev:",2F11.2)')
     &   countsinfmin,countsinfmn,countsinfstd
        CALL ml_wt('warning',text)
        WRITE(text,'("  Perhaps min value of CN limits ",I6,
     &  " must be changed")') cntlm(ich,1)   
        CALL ml_wt('warning',text)
        
      ENDIF
      
      IF(supcnt.GT.0) THEN 
      
        DO i=1,supcnt
         countssupmn  = countssupmn  + countssup(i)
         countssupstd = countssupstd + countssup(i)*countssup(i)
         IF (countssup(i).GT.countssupmax) countssupmax=countssup(i)
        ENDDO
        countssupmn  = countssupmn/supcnt
        countssupstd = (countssupstd/supcnt)-(countssupmn**2)
        IF ( countssupstd .GE. 0.d+0) countssupstd=dsqrt(countssupstd)
        
        WRITE(text,'("ch",I2,": ",I2," CN values are sup to limit")')
     &   ich,supcnt
        CALL ml_wt('warning',text)
        WRITE(text,'("  max:",I7,"  counts mean/stdev:",2F11.2)')
     &   countssupmax,countssupmn,countssupstd
        CALL ml_wt('warning',text)
        WRITE(text,'("  Perhaps max value of CN limits ",I6,
     &  " must be changed")') cntlm(ich,2)   
        CALL ml_wt('warning',text)        
  
      ENDIF
      
      IF(misscnt.GT.0) THEN 
        WRITE(text,'("ch",I2,": ",I2," CN values are missing")')
     &   ich,misscnt
        CALL ml_wt('warning',text)
      ENDIF
      
      IF(diffmin.GT.0) THEN
        WRITE(text,'("ch",I2," difference CNTvalue - minlimit",
     &  " < 5 : lim",I6,"   nb ",I2)') ich,cntlm(ich,1),diffmin
        CALL ml_wt('warning',text)
        text='Perhaps min value of CN limits must be changed'
        CALL ml_wt('warning',text)
      ENDIF
      
      IF(diffmax.GT.0) THEN
        WRITE(text,'("ch",I2," difference maxlimit - CNTvalue",
     &  " < 5 : lim",I6,"   nb ",I2)') ich,cntlm(ich,2),diffmax
        CALL ml_wt('warning',text)
        text='Perhaps max value of CN limits must be changed'
        CALL ml_wt('warning',text)
      ENDIF
                  

      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &  '   *end sub H_LIMIT_ALGOV4*'


 500   FORMAT(12X,'count lower than min limit:',I6,4X,'ifov:',I2)
 510   FORMAT(12X,'count greater than max limit:',I6,4X,'ifov:',I2)

       RETURN
       END

