      SUBROUTINE h_slope_algoV4(
     &     partialsw,
     &     nselectedslopes,
     &     selectedslopes, 
     &     calibcoef)

!------------------------------------------------------------------------------
!NAME $Id: h_slope_algoV4.F 72 2007-04-27 08:21:59Z frna $
!
!PURPOSE
!   This module computes the calibration slopes for each channel and  
!   for all the Earth view scan lines.
!
!DESCRIPTION
!   For a normal super-swath k (in the middle of an orbit for example),
!   the calibration slopes for the 38 Earth view scan lines of the 
!   current super-swath k is the running average of 3 values: 
!    the BB slopes of calib cycle(k),the BB slopes of calib cycle(k-1)
!    the BB slopes of calib cycle(k-2).
!
!   For a partial super-swath k (when an orbit does not begin or end
!   with a complete calibration cycle or when the data stream is broken
!   in the middle of an orbit for examples), the calibration slopes 
!   for the 38 Earth view scan lines of the current super-swath k
!   is the average value of the qualified slopes from the nearest 2
!   (or at least one) calibration cycles.
!   If case neither value is qualified, the calibration slopes 
!   for the 38 Earth view scan lines of the current super-swath k
!   is the average slope computed with historic data. If the average 
!   slope is equal to zero or is missing, the calibration slopes 
!   for the 38 Earth view scan lines of the current super-swath k
!   are also missing.
!
!   At the end, the module updates the elements hcalcof/hscalcof to write 
!   later in the HIRS level 1b file.
!   Into the level 1b file:
!   For each 38 Earth view scan line of a super-swath, 
!      Recomputed slopes in the primary and secondary coefficients words
!   For the space view scan line,
!      Recomputed slopes in the primary and secondary coefficients words
!   For the blackbody scan line,
!      BB slope or raw slope in the primary and secondary coefficients words
!
!   REFERENCE
!   HIRS calibration Algorithm Version 4.0 (Nov 3, 2003)
!   C. Cao NESDIS
!
!HISTORY (CHANGES) 
!(version   date         comment              <author>)
! 1.0     January 05     creation            T. Labrot (MF)
! 1.1     25 April 07    Modified the 'write' command into the log file
!                                            T. Labrot (MF)
!
!CURRENT CODE OWNER: SAF NWP
!     
!COPYRIGHT
!   This software was developed within the context of
!   the EUMETSAT Satellite Application Facility on
!   Numerical Weather Prediction (NWP SAF), under the
!   Cooperation Agreement dated 25 November 1998, between
!   EUMETSAT and the Met Office, UK, by one or more partners
!   within the NWP SAF. The partners in the NWP SAF are
!   the Met Office, ECMWF, KNMI and MeteoFrance.    
!   
!   Copyright 2005, EUMETSAT, All Rights Reserved.
!      
!ARGUMENTS 
!   inputs :
!    LOGICAL partialsw(hrs_mxirchn, hrs_mxcalib+1) 
!                     ! true if partial superswath for cycle
!    INTEGER nselectedslopes(hrs_mxirchn, hrs_mxcalib+1)
!                     ! number of selected slopes (1 to 3) for each cycle
!                     ! 0 is not possible
!    INTEGER selectedslopes(3,hrs_mxirchn,hrs_mxcalib+1)
!                     ! cycle number (1to3) selected slopes
!
!   outputs :
!    DBLE PREC calibcoef(hrs_mxchn,3,hrs_mxlin)
!                   !third auto coefs,slopes,intercepts array for each scan line
!
!UPDATED ELEMENTS OF INCLUDES
!  in include  hcaliblineinfo_algoV4.h:
!     INTEGER linechquality(6,hrs_mxchn,hrs_mxlin)                       
!                         !1: calibration failed
!                         !2: calibration marginal
!                         !3: calibration slope came from the HCF
!                         !4: BB views failed NEDC test
!                         !5: space views failed NEDC test
!                         !6: some QC (quality control) tests could not be 
!                         !   applied  
!  
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     
 
!includes/commons
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hb1averageslope_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
! Add artificially 1 to hrs_mxcalib to compute the calibration coefficient 
! after the last calibration cycle.
      LOGICAL partialsw(hrs_mxirchn, hrs_mxcalib+1) 
      INTEGER nselectedslopes(hrs_mxirchn, hrs_mxcalib+1)
      INTEGER selectedslopes(3,hrs_mxirchn,hrs_mxcalib+1)
      DOUBLE PRECISION slope(hrs_mxirchn, hrs_mxcalib+1)
      DOUBLE PRECISION calibcoef(hrs_mxchn,3,hrs_mxlin)

! local variables
      CHARACTER*80 text          !variable text
      INTEGER i,icalib,ich,ilin  !index of loops
      INTEGER k                  !calib cycle index to be used
      INTEGER l1border(hrs_mxchn)  !channels in the l1b order
      INTEGER coef0(hrs_mxchn)   !slopes array       
      
!--END OF DECLARATIONS----------------------------------------------------------
     
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)
      IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '** SUB. H_SLOPE_ALGOV4   **'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
      IF (iprint.GE.1) WRITE(hrs_cllgu,*)

      DATA l1border/ 1,17,2,3,13,4,18,11,19,7,8,20,
     &          10,14,6,5,15,12,16,9 /

! Initializes the outputs
!------------------------
!  and fills the quadratic coefficient with zero
!------------------------------------------    
      DO ilin=1,hrs_mxlin
        DO ich=1,hrs_mxchn
          calibcoef(ich,2,ilin) = missdp
          calibcoef(ich,1,ilin) = 0.d+0
        ENDDO
      ENDDO
      

! Initializes some variables
!---------------------------      
      DO icalib=1,hrs_mxcalib+1
        DO ich=1,hrs_mxirchn 
           slope(ich,icalib)=0.d+0
        ENDDO
      ENDDO     


!Computes slope for each IR channel and for each super_swath
!-----------------------------------------------------------
 
      text='Computes slope for each channel and for each calib cycle'
      CALL ml_wt('info',text)
      
! Adds artificially 1 to calib to compute the calibration coefficient 
! after the last calibration cycle.

      DO icalib=1,calib+1
      
      IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=500)
      
        DO ich=1,hrs_mxirchn
          
          IF (nselectedslopes(ich,icalib).GE.1) THEN
          
            DO i=1,nselectedslopes(ich,icalib)     
              slope(ich,icalib)=slope(ich,icalib) +
     &            calibcoef0(ich,2,selectedslopes(i,ich,icalib))
            ENDDO
            slope(ich,icalib)=slope(ich,icalib)/
     &                       nselectedslopes(ich,icalib)

! Updates the quality array for the Earth scan lines
             IF (icalib.eq.1) THEN
               DO ilin=lin0,splintab(icalib)-1
                 linechquality(3,ich,ilin)=0
               ENDDO
             ENDIF  
             IF (icalib.EQ.calib+1) THEN
               DO ilin=iwtlintab(icalib-1)+1,lin1
                 linechquality(3,ich,ilin)=0
               ENDDO
             ENDIF 
             IF (icalib.GE.2 .AND. icalib.LE.calib) THEN
               DO ilin=iwtlintab(icalib-1)+1,splintab(icalib)-1
                 linechquality(3,ich,ilin)=0
               ENDDO
             ENDIF     
          
          ELSE IF (nselectedslopes(ich,icalib).EQ.0) THEN
            
            IF (avgslope(ich).NE.0..AND.avgslope(ich).NE.missreal) THEN

              slope(ich,icalib)=avgslope(ich)
! Updates the quality array for the Earth scan lines
              IF (icalib.EQ.1) THEN
                DO ilin=lin0,splintab(icalib)-1
                  linechquality(3,ich,ilin)=1
                ENDDO
              ENDIF
              IF (icalib.EQ.calib+1) THEN
                DO ilin=iwtlintab(icalib-1)+1,lin1
                  linechquality(3,ich,ilin)=1
                ENDDO
              ENDIF 
              IF (icalib.GE.2 .AND. icalib.LE.calib) THEN
                DO ilin=iwtlintab(icalib-1)+1,splintab(icalib)-1
                  linechquality(3,ich,ilin)=1
                ENDDO
              ENDIF
              WRITE(text,'(2X,"nselectedslopes=0 for ",
     &        "icalib ich ",I3,1X,I2)') icalib,ich
              CALL ml_wt('notice',text)
              text='       the average slope is used' 
              CALL ml_wt('notice',text)
              
            ELSE   !IF (avgslope(ich).NE.0..AND.avgslope(ich).NE.missreal) THEN
            
              slope(ich,icalib)=missdp  
! Updates the quality array for the Earth scan lines
              IF (icalib.EQ.1) THEN
                DO ilin=lin0,splintab(icalib)-1
                  linechquality(1,ich,ilin)=1
                ENDDO
              ENDIF
              IF (icalib.EQ.calib+1) THEN
                DO ilin=iwtlintab(icalib-1)+1,lin1
                  linechquality(1,ich,ilin)=1
                ENDDO
              ENDIF 
              IF (icalib.GE.2 .AND. icalib.LE.calib) THEN
                DO ilin=iwtlintab(icalib-1)+1,splintab(icalib)-1
                  linechquality(1,ich,ilin)=1
                ENDDO
              ENDIF
              WRITE(text,'(2X,"nselectedslopes=0 for ",
     &        "icalib ich ",I3,1X,I2)') icalib,ich
              CALL ml_wt('notice',text)
              text='       the average slope=0 or is missing' 
              CALL ml_wt('notice',text)
              text='       slope is missing'
              CALL ml_wt('notice',text)
              
            ENDIF   !IF (avgslope(ich).NE.0..AND.avgslope(ich).NE.missreal) THEN                                    
          
          ELSE      !IF (nselectedslopes(ich,icalib).EQ.0) THEN

            IF (avgslope(ich).NE.0..AND.avgslope(ich).NE.missreal) THEN
            
              slope(ich,icalib)=avgslope(ich)
! Updates the quality array for Earth scan lines
              IF (icalib.EQ.1) THEN
                DO ilin=lin0,splintab(icalib)-1
                  linechquality(3,ich,ilin)=1
                ENDDO
              ENDIF   
              IF (icalib.EQ.calib+1) THEN
                DO ilin=iwtlintab(icalib-1)+1,lin1
                  linechquality(3,ich,ilin)=1
                ENDDO
              ENDIF 
              IF (icalib.GE.2 .AND. icalib.LE.calib) THEN
                DO ilin=iwtlintab(icalib-1)+1,splintab(icalib)-1
                  linechquality(3,ich,ilin)=1
                ENDDO
              ENDIF                 
              WRITE(text,'(2X,"bad value for nselectedslopes for ",
     &        "icalib ich ",I3,1X,I2)') icalib,ich
              CALL ml_wt('notice',text)
              text='       the average slope is used' 
              CALL ml_wt('notice',text)
          
          ELSE   !IF (avgslope(ich).NE.0..AND.avgslope(ich).NE.missreal) THEN
            
              slope(ich,icalib)=missdp  
! Updates the quality array for the Earth scan lines
              IF (icalib.EQ.1) THEN
                DO ilin=lin0,splintab(icalib)-1
                  linechquality(1,ich,ilin)=1
                ENDDO
              ENDIF
              IF (icalib.EQ.calib+1) THEN
                DO ilin=iwtlintab(icalib-1)+1,lin1
                  linechquality(1,ich,ilin)=1
                ENDDO
              ENDIF 
              IF (icalib.GE.2 .AND. icalib.LE.calib) THEN
                DO ilin=iwtlintab(icalib-1)+1,splintab(icalib)-1
                  linechquality(1,ich,ilin)=1
                ENDDO
              ENDIF
              WRITE(text,'(2X,"bad value for nselectedslopes for ",
     &        "icalib ich ",I3,1X,I2)') icalib,ich
              CALL ml_wt('notice',text)
              text='       the average slope=0 or is missing' 
              CALL ml_wt('notice',text)
              text='       slope is missing'
              CALL ml_wt('notice',text)
              
            ENDIF   !IF (avgslope(ich).NE.0..AND.avgslope(ich).NE.missreal) THEN
          
          ENDIF
          
   
          IF (iprint.GE.2) THEN
          
           IF(nselectedslopes(ich,icalib).EQ.0)
     &     WRITE(hrs_cllgu,fmt=510)
     &     icalib,ich,nselectedslopes(ich,icalib)
     
           IF(nselectedslopes(ich,icalib).EQ.1)
     &     WRITE(hrs_cllgu,fmt=520)
     &     icalib,ich,nselectedslopes(ich,icalib),
     &     selectedslopes(1,ich,icalib),
     &     slope(ich,icalib),
     &     calibcoef0(ich,2,selectedslopes(1,ich,icalib))
     
           IF(nselectedslopes(ich,icalib).EQ.2)
     &     WRITE(hrs_cllgu,fmt=530)
     &     icalib,ich,nselectedslopes(ich,icalib),
     &     selectedslopes(1,ich,icalib),
     &     selectedslopes(2,ich,icalib),
     &     slope(ich,icalib),
     &     calibcoef0(ich,2,selectedslopes(1,ich,icalib)),
     &     calibcoef0(ich,2,selectedslopes(2,ich,icalib))
     
           IF(nselectedslopes(ich,icalib).EQ.3)
     &     WRITE(hrs_cllgu,fmt=540)
     &     icalib,ich,nselectedslopes(ich,icalib),
     &     selectedslopes(1,ich,icalib),
     &     selectedslopes(2,ich,icalib),
     &     selectedslopes(3,ich,icalib),
     &     slope(ich,icalib),
     &     calibcoef0(ich,2,selectedslopes(1,ich,icalib)),
     &     calibcoef0(ich,2,selectedslopes(2,ich,icalib)),
     &     calibcoef0(ich,2,selectedslopes(3,ich,icalib))
     
          ENDIF
     
        ENDDO   !DO ich=1,hrs_mxirchn
         
      ENDDO     !DO icalib=1,calib+1
      
      
      
!Computes slope for each channel and for all the Earth view scan lines
!--------------------------------------------------------------------- 

      WRITE(text,'("Computes slope for each channel and for",
     & " each Earth view scan line")')
      CALL ml_wt('info',text)
      
      DO ilin=lin0,lin1

!  In this scheme the space scan marks the start of the next superswath and
!  contains the recomputed slope of the next superswath.
      
        k=1
        DO WHILE (ilin.GE.splintab(k) .AND. k.LE.calib)
          k=k+1
        ENDDO

        IF (iprint.GE.2) WRITE(hrs_cllgu,*)
        DO ich=1,hrs_mxirchn 
          calibcoef(ich,2,ilin)=slope(ich,k)                 
          IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=550)
     &    ilin,ich,k,slope(ich,k)                      
        ENDDO
        
        
        
! For the visible channel
        ich=20
       
        calibcoef(20,2,ilin)=calibcoef0(20,2,1)
        IF (iprint.GE.2) WRITE(hrs_cllgu,fmt=550)
     &    ilin,ich,k,calibcoef(20,2,ilin)
          
      ENDDO     !DO ilin=lin0,lin1


! Stores the recomputed slopes as primary calibration coefficients   
!-------------------------------------------------------------------
! and as secondary (spare) coefficients.
!---------------------------------------
       DO ilin=lin0,lin1
      
         DO ich=1,hrs_mxchn
          
          IF (calibcoef(ich,2,ilin).NE.missdp) THEN
            coef0(ich)=IDINT(calibcoef(ich,2,ilin)*1.d+09)
          ELSE
            coef0(ich)=missint
          ENDIF
        
        ENDDO           !DO ich=1,hrs_mxchn

        DO ich=1,hrs_mxchn
          hcalcof(2,ich,ilin) =coef0(l1border(ich))
          hscalcof(2,ich,ilin)=coef0(l1border(ich))
        ENDDO
        
      ENDDO             !DO ilin=lin0,lin1 
          
! Stores the recomputed slopes as primary calibration coefficients   
!-----------------------------------------------------------------
! and as secondary (spare) coefficients along with the space view
!----------------------------------------------------------------
! scan lines
!-----------

!      It was done in the previous lines 
 
! Stores the BBslopes as primary calibration coefficients   
!--------------------------------------------------------
! and as secondary (spare) coefficients along with the blackbody 
!---------------------------------------------------------------
! scan lines
!----------- 
  
      DO icalib=1,calib
      
        ilin=iwtlintab(icalib)
        
        DO ich=1,hrs_mxchn

         IF (calibcoef0(ich,2,icalib).NE.missdp) THEN
           coef0(ich)=IDINT(calibcoef0(ich,2,icalib)*1.d+09)
         ELSE
           coef0(ich)=missint 
         ENDIF
         
        ENDDO 
        

        DO ich=1,hrs_mxchn
          hcalcof(2,ich,ilin) =coef0(l1border(ich))
          hscalcof(2,ich,ilin)=coef0(l1border(ich))
        ENDDO
      
      ENDDO     ! DO icalib=1,calib
                    
           
 500  FORMAT(/2X,'calib   ich  nb   s1   s2   s3       slope' //
     & '           slopes1         slopes2          slopes3') 
 510  FORMAT(2X,I3,5X,I2,3X,I2)
 520  FORMAT(2X,I3,5X,I2,3X,I2,3X,I2,10X,3X,F14.10,2x,F14.10)
 530  FORMAT(2X,I3,5X,I2,3X,I2,3X,I2,3X,I2,5X,3X,F14.10,2(2x,F14.10))
 540  FORMAT(2X,I3,5X,I2,3X,I2,3X,I2,3X,I2,3X,I2,3X,F14.10,3(2x,F14.10))
 550  FORMAT(2X,'ilin ich  k slope ',I3,3X,I2,2X,I2,2X,F16.10)      
              
      RETURN
      END
