      Subroutine h_sttemp_algoV4(STTp)

!------------------------------------------------------------------------------
!NAME $Id: h_sttemp_algoV4.F 40 2006-06-20 08:32:40Z frna $
!
!PURPOSE   
!  Computes the Second Telescope Temperature for all lines
!  (STT prime, see below)
!  Computes the Primary Telescope Temperature, the Tertiary Telescope Temperature,
!  the Baseplate temperature, the Filter Wheel housing temperature.
! 
!DESCRIPTION
!  Reference
!   NOAA KLM User's Guide Section 7.2 
!
!  The HIRS instrument measures all radiation falling on the detector. 
!  That is, it is a total radiometer. To minimize false fluctuations in
!  the signals, the instrument temperature is carefully controlled, so 
!  that most optical components experience temperature changes very 
!  slowly with respect to the times between calibration cycles 
!  (256 seconds). However, the baffle (identified as the secondary 
!  mirror) is a blackened light material which is subject to short-term
!  temperature changes from emission or blackbody radiation and 
!  absorption of incident radiation from the internal blackbody source
!  (about 280 K), space (2.73 K), and the variable earth views
!  (about 200-275 K plus reflected sunlight). The result is that the 
!  contribution by the baffle affects all measurements on a time scale
!  of seconds and this must be accounted for.
!
!  At every line, the secondary mirror temperature is linearly 
!  interpolated to the midpoint of the radiometric data 
!  (beam position 28.5) according to:
!            Tsnnp = Tsnn-01 + 0.4609 * (Tsnn - Tsnn-01)
!  where nn is the line number in the superswath (00 to 39), nn-01 
!  is the previous line, and Ts is the measurement of the secondary 
!  mirror temperature. In the first line of an orbit the value
!            Tsnnp = Tsnn - 0.5391 * (Tsnn+1 - Tsnn)
!  is used. 
!  The interpolated values are preserved during the processing
!  of a superswath (40 values plus the value from the last line of
!  the previous superswath).
!
!
!  Coefficients to calculate Tsnn are obtained from 
!  NOAA KLM User's Guide Appendix D
!  Example for Noaa17:  Values Table D.3-12 for the NOAA-17 HIRS/303 
! 
!  a0 = 260.29119 	
!  a1 =   0.01693469 	
!  a2 =  -2.41317E-06 	
!  a3 =   4.01919E-10 	
!  a4 =   1.17566E-14
!
!  There are 6 coefficients for HIRS/4 instruments (A to F)
!  Table D.4-12. NOAA-18 HIRS/H305 primary, secondary and tertiary 
!  telescope temperature coefficients.     
!
!  Measurement of STT is given in HIRS data element 62 word number 5
!  This is the same for HIRS/3 and HIRS/4
!    1&2   Header for Element 62  (4 bytes)
!    3     Scan Mirror Temperature (1 word, 2 bytes)
!    4     Primary Telescope Temperature (1 word, 2 bytes)
!    5     Secondary Telescope Temperature (1 word 2 bytes)
!
!HISTORY (CHANGES) 
!  (version    date         comment           <author>)
!    1.0     January 05    Creation          P. Brunel (MF)
!    1.1     14/06/05      Correct bug in tsnn computation:
!                           Term a2*csm*2 --> a2*csm**2
!                          Add the computations of PTT, TTT, BBT, FWT
!                                            T.Labrot and P.Brunel (MF)
!    1.2     20/06/06      Make csm real (was integer)  N.Atkinson  
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!
!ARGUMENTS 
!   outputs :
!     Real sttp(hrs_mxlin) ! STT prime for all lines (K)
!
!UPDATED ELEMENTS OF INCLUDES 
!   in hcalibcycleinfo_algoV4.h:      
!         Real stt(hrs_mxlin)      ! Secondary Telescope Temperature
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      Implicit None
      
!includes/commons      
#include "hmissingval_algoV4.h"
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hcinit_algoV4.h"
#include "hrs1belem_algoV4.h"
#include "hcalibcycleinfo_algoV4.h"
#include "hcaliblineinfo_algoV4.h"

! argument variables
      Real sttp(hrs_mxlin) ! STT prime for all lines (K)
      
! local variables      
      Integer ilin, ml ! line  number
      integer fw, is

      Integer last_good  !
      Integer new_good   ! variables for filling of bad data 
      Logical missing_on !

      Double Precision a0, a1, a2, a3, a4, a5 ! double precision coefs
      Double Precision tsnn
      Double Precision temp
      Double Precision csm      ! measured count

!--END OF DECLARATIONS----------------------------------------------------------


      If (iprint.Ge.1) Write(hrs_cllgu,*)
      If (iprint.Ge.2) Write(hrs_cllgu,*) '***************************'
      If (iprint.Ge.1) Write(hrs_cllgu,*) '**  SUB H_SSTEMP_ALGOV4  **'
      If (iprint.Ge.1) Write(hrs_cllgu,*) '***************************'
      If (iprint.Ge.1) Write(hrs_cllgu,*)

!------------------------------------------------------------------
! Secondary telescope temperature
!------------------------------------------------------------------

! Initializes the ai coefficients with data got from h_calcoef_algoV4.dat file
!-----------------------------------------------------------------------------
      a0=telescopcnttmp(2,1) + 273.15
      a1=telescopcnttmp(2,2)
      a2=telescopcnttmp(2,3)
      a3=telescopcnttmp(2,4)
      a4=telescopcnttmp(2,5)
      a5=telescopcnttmp(2,6)
               
! Calculates the Second Telescope Temperature for every scan line
!----------------------------------------------------------------
      Do ilin = lin0, lin1

        csm = helem(5,62,ilin)

        tsnn = a0 + a1*csm + a2*csm**2 + a3*csm**3 +
     &                           a4*csm**4 + a5*csm**5

         If( tsnn .GT. 200. .And. tsnn .LT. 300. ) Then
            stt(ilin) = tsnn
         Else
            stt(ilin) = missreal
         End if
         
      End do


! Calculation of Second Telescope Temperature "Prime" for every scan line
!------------------------------------------------------------------------
      ! At every line, the secondary mirror temperature is linearly 
      ! interpolated to the midpoint of the radiometric data 
      ! (beam position 28.5) according to:
      !            Tsnnp = Tsnn-01 + 0.4609 * (Tsnn - Tsnn-01)
      ! where nn is the line number in the superswath (00 to 39), nn-01 
      ! is the previous line, and Ts is the measurement of the secondary 
      ! mirror temperature. In the first line of an orbit the value
      !            Tsnnp = Tsnn - 0.5391 * (Tsnn+1 - Tsnn)
      ! is used. The interpolated values are preserved during the processing
      ! of a superswath (40 values plus the value from the last line of
      ! the previous superswath).

      If( stt(lin0) .Ne. missreal .And. stt(lin0+1) .Ne. missreal ) Then
         sttp(lin0) =  stt(lin0) - 0.5391 * (stt(lin0+1) - stt(lin0))
      Else
         sttp(lin0) = missreal
      End if

      Do ilin = lin0+1, lin1
        If(stt(ilin).Ne.missreal .And. stt(ilin-1).Ne.missreal) Then
            sttp(ilin) =  stt(ilin-1) + 
     &                     0.4609 * (stt(ilin) - stt(ilin-1))
        Else
            sttp(ilin) = missreal
        End if
      End do

! Interpolation of STTp for missing values
!-----------------------------------------
      ! If all lines are bad then the output STTp will 
      ! only contain 0.
      missing_on =.FALSE.
      last_good = 0
      new_good  = 0
      Do ilin = lin0, lin1
         If( sttp(ilin) .Ne. missreal ) Then
            If( missing_on ) Then
               new_good = ilin
               ! if last = 0 then use same value from 1 to new
               If ( last_good .Eq. 0 ) Then
                  Do ml=lin0, new_good-1
                     sttp(ml) = sttp(new_good)
                  End do
               Else
               ! linearly interpolate from last+1 to new-1
                  Do ml = last_good+1, new_good-1
                     sttp(ml) = sttp(last_good) + (ml-last_good) *
     &                  (sttp(new_good) - sttp(last_good)) / 
     &                        (new_good - last_good) 
                  End do
               End if
               missing_on =.FALSE.
            Else
               last_good = ilin
            End if
         Else
            missing_on = .TRUE.
         End if
      End do
      
      If (iprint.Ge.2) Then
         Do ilin = lin0, lin1
           Write(hrs_cllgu,fmt=500) ilin,stt(ilin),sttp(ilin)
         End do
      End if
      
!------------------------------------------------------------------
! Primary telescope temperature
!------------------------------------------------------------------

! Initializes the ai coefficients with data got from h_calcoef_algoV4.dat file
!-----------------------------------------------------------------------------
      a0=telescopcnttmp(1,1) + 273.15
      a1=telescopcnttmp(1,2)
      a2=telescopcnttmp(1,3)
      a3=telescopcnttmp(1,4)
      a4=telescopcnttmp(1,5)
      a5=telescopcnttmp(1,6)
      
! Calculates the Primary Telescope Temperature for every scan line
!----------------------------------------------------------------
      Do ilin = lin0, lin1  
        csm = helem(4,62,ilin)

        temp = a0 + a1*csm + a2*csm**2 + a3*csm**3 +
     &                           a4*csm**4 + a5*csm**5

         If( temp .GT. 200. .And. temp .LT. 300. ) Then
            ptt(ilin) = temp
         Else
            ptt(ilin) = missreal
         End if
         
      End do
      
!------------------------------------------------------------------
! Tertiary telescope temperature
!------------------------------------------------------------------

! Initializes the ai coefficients with data got from h_calcoef_algoV4.dat file
!-----------------------------------------------------------------------------
      a0=telescopcnttmp(3,1) + 273.15
      a1=telescopcnttmp(3,2)
      a2=telescopcnttmp(3,3)
      a3=telescopcnttmp(3,4)
      a4=telescopcnttmp(3,5)
      a5=telescopcnttmp(3,6)
      
! Calculates the Tertiary Telescope Temperature for every scan line
!----------------------------------------------------------------
      Do ilin = lin0, lin1 
       
        csm = ( helem(18,59,ilin) + 
     &          helem(19,59,ilin) +
     &          helem(20,59,ilin) +
     &          helem(21,59,ilin) +
     &          helem(22,59,ilin)) /5.

        temp = a0 + a1*csm + a2*csm**2 + a3*csm**3 +
     &                           a4*csm**4 + a5*csm**5

         If( temp .GT. 200. .And. temp .LT. 300. ) Then
            ttt(ilin) = temp
         Else
            ttt(ilin) = missreal
         End if
         
      End do
      
!------------------------------------------------------------------
! Baseplate temperature
!------------------------------------------------------------------

! Initializes the ai coefficients with data got from h_calcoef_algoV4.dat file
!-----------------------------------------------------------------------------
      a0=bpcnttmp(1) + 273.15
      a1=bpcnttmp(2)
      a2=bpcnttmp(3)
      a3=bpcnttmp(4)
      a4=bpcnttmp(5)
      a5=bpcnttmp(6)
      
! Calculates the Baseplate Temperature for every scan line
!----------------------------------------------------------------
      Do ilin = lin0, lin1 
       
        csm = helem(6,62,ilin)

        temp = a0 + a1*csm + a2*csm**2 + a3*csm**3 +
     &                           a4*csm**4 + a5*csm**5

         If( temp .GT. 200. .And. temp .LT. 300. ) Then
            bbt(ilin) = temp
         Else
            bbt(ilin) = missreal
         End if
         
      End do 
                           
!------------------------------------------------------------------
! Filter Wheel housing temperature
!------------------------------------------------------------------

      do fw = 1, 4
      
! Initializes the ai coefficients with data got from h_calcoef_algoV4.dat file
!-----------------------------------------------------------------------------
      a0=fwcnttmp(fw,1) + 273.15
      a1=fwcnttmp(fw,2)
      a2=fwcnttmp(fw,3)
      a3=fwcnttmp(fw,4)
      a4=fwcnttmp(fw,5)
      a5=fwcnttmp(fw,6)
      
! Calculates the Filter Wheel housing Temperature for every scan line
!--------------------------------------------------------------------
      Do ilin = lin0, lin1 
       
        is = 3 + (fw - 1) *5

        csm = ( helem(is  ,60,ilin) + 
     &          helem(is+1,60,ilin) +
     &          helem(is+2,60,ilin) +
     &          helem(is+3,60,ilin) +
     &          helem(is+4,60,ilin)) /5.


        temp = a0 + a1*csm + a2*csm**2 + a3*csm**3 +
     &                           a4*csm**4 + a5*csm**5

         If( temp .GT. 200. .And. temp .LT. 300. ) Then
            fwt(fw,ilin) = temp
         Else
            fwt(fw,ilin) = missreal
         End if
         
      End do
      
      End do
                             
 500  Format('line STT STTprime : ',1x,i4,2f9.3)
 
      Return
      End
