        SUBROUTINE hclsetu_algoV4

!------------------------------------------------------------------------------
!NAME $Id: hclsetu_algoV4.F 55 2006-09-21 08:35:34Z frna $
!
!PURPOSE 
!    This module opens the log/debug file (if requested. see iprint value).
!    It opens the HIRS.level1b, reads the header, all the records.
!    The data are ranged in commons hrs1bhd and hrs1bdts
!    (includes hrs1bhd.h, hrs1bdts.h).
!
!    It prepares the navigation/localisation task:     
!     Opens/closes the SATPOS file.
!     Opens/closes the Clock error file to do clock corrections if requested
!
!DESCRIPTION
!    This module is very similar to the module hclsetu.
!    It was created to manage the HIRS calibration algorithm version4.0
!     
!HISTORY (CHANGES) 
!    (version   date          comment           <author>)
! 1.0     January 05     creation          T. Labrot (MF)
! 1.1     Mar 05   Allow for NESDIS data: subtract 4096 from 1b counts. NCA
!
!CURRENT CODE OWNER: SAF NWP
!
!COPYRIGHT
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!              
!         
!ARGUMENTS 
!
!UPDATED ELEMENTS OF INCLUDES
!    All the elements in the includes hrs1bhd.h,hrs1bdt.h,hrs1bdts.h
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    INTEGER FUNCTION   ord1bid :
!                  delivers the order satellite number for a given 
!                  level 1b id number (PACS number) and a julian date
!    CHARACTER*6 FUNCTION  noascnam :  
!                  delivers the NOAA satellite name for a given order number
!    byteswap1b
!    sp_read, def_att, calatt, clkerr_get
!
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

      IMPLICIT NONE                                                     

! includes/commons
#include "hrsparam_algoV4.h"
#include "hclin_algoV4.h"
#include "hrs1bhd.h"
#include "hrs1bdt.h"
#include "hrs1bdts.h"
#include "c_1bnav.h"
#include "reclopt.h"

! argument variables

! functions
      CHARACTER*6 noascnam 
      INTEGER     ord1bid 
      
! local variables
      CHARACTER*80 text  !variable text 
      CHARACTER*6 scname !NOAA satellite name
      INTEGER i,j        !index of loops
      INTEGER ios        !variable that receives the error status
      INTEGER loc_rl     !record length
      INTEGER recn       !record number in the HIRS 1B file
      INTEGER recn0      !first record number for data in the HIRS 1B file
      INTEGER recn1      !last record number in the HIRS 1B file

      
      INTEGER ierr
      INTEGER satid
      INTEGER att_mode   ! attitude mode
                         ! 0 = NOAA POES operational mode
                         ! 1 = METOP Yaw steering mode
                         ! 2 = Geocentric mode
      DOUBLE PRECISION ijstrt,ijend
                
!--END OF DECLARATIONS---------------------------------------------------------


! Opens the log/debug file (if requested)
!----------------------------------------
      IF (iprint.GE.1) THEN

        OPEN(hrs_cllgu,form='formatted',access='sequential',
     &       status='new',iostat=ios)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF (ios.NE.0)
     &  OPEN(hrs_cllgu,form='formatted',access='sequential',
     &       status='unknown',iostat=ios)

        IF (ios.NE.0) THEN
          text='The log/debug file could not be opened, ios:'
          CALL ml_wti('error',text,ios)
          CALL ml_wt('error','hclsetu_algoV4 aborted')
          CALL aappexit(2)
        ELSE
          text='The log/debug file has been opened'
          CALL ml_wt('info',text)

       IF (iprint.GE.1) WRITE(hrs_cllgu,*)
       IF (iprint.GE.2) WRITE(hrs_cllgu,*) '***************************'
       IF (iprint.GE.1) WRITE(hrs_cllgu,*) '**  SUB. HCLSETU_ALGOV4  **'
       IF (iprint.GE.1) WRITE(hrs_cllgu,*) '***************************'
       IF (iprint.GE.1) WRITE(hrs_cllgu,*)

         IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &   '  the log/debug file has been opened'
        ENDIF

      ENDIF     !IF (iprint.GE.1) THEN

! Opens the HIRS-level1b file
!----------------------------
      loc_rl=reclhrs/rldiv
      OPEN(hrs_l1bu,recl=loc_rl,form='unformatted',access='direct',
     &     status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF (ios.NE.0)
     & OPEN(hrs_l1bu,recl=loc_rl,form='unformatted',access='direct',
     &      status='unknown',iostat=ios)

      IF (ios.NE.0) THEN
         IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &   '  HIRS1B file could not be opened,ios:',ios
         text='HIRS1B file could not be opened,  ios:'
         CALL ml_wti('error',text,ios)
         CALL ml_wt('error','hclsetu_algoV4 aborted')
         CALL aappexit(2)
      ELSE
         IF (iprint.GE.1) write(hrs_cllgu,*)
     &   '  HIRS1B file has been opened'
         text='HIRS1B file has been opened'
         CALL ml_wt('info',text)
      ENDIF

! Reads the header of the HIRS-level1b file
!------------------------------------------

      READ(hrs_l1bu,rec=1,iostat=ios) hrs_hd
      
      IF (ios.NE.0) THEN
      
        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &   '  HIRS1B header could not be read,  ios:',ios
        text='HIRS1B header could not be read ,  ios:'
        CALL ml_wti('error',text,ios)
        CALL ml_wt('error','hclsetu_algoV4 aborted')
        CALL aappexit(2)

      ELSE

        IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &   '  HIRS1B header has been read'
        text='HIRS1B header has been read'
        CALL ml_wt('info',text)
     
        CALL byteswap1b("HRS1BHD")

      ENDIF     !IF (ios.NE.0) THEN
       
! Reads the data of the HIRS1B file (all the records)
!----------------------------------------------------

      recn0=hrs_h_hdrcnt+1
      recn1=hrs_h_hdrcnt+hrs_h_scnlin
    
      DO recn=recn0,recn1
         
        READ(hrs_l1bu,rec=recn,iostat=ios) hrs_dt
         
        IF (ios.NE.0) THEN
         
          IF (iprint.GE.1) WRITE(hrs_cllgu,*)
     &    '  HIRS1B record could not be read, ios:',ios
          text='HIRS1B record could not be read,  ios:'
          CALL ml_wti('error',text,ios)
          CALL ml_wt('error','hclsetu_algoV4 aborted')
          CALL aappexit(2)

        ELSE
           
          IF (iprint.GE.2) 
     &    WRITE(hrs_cllgu,*) '  record number:',recn,
     &    '  has been read'       

          CALL byteswap1b("HRS1BDT")
          
          hrs_scnlin(recn - hrs_h_hdrcnt)     = hrs_scnlin0             
          hrs_scnlinyr(recn - hrs_h_hdrcnt)   = hrs_scnlinyr0
          hrs_scnlindy(recn - hrs_h_hdrcnt)   = hrs_scnlindy0
          hrs_clockdrift(recn - hrs_h_hdrcnt) = hrs_clockdrift0
          hrs_scnlintime(recn - hrs_h_hdrcnt) = hrs_scnlintime0
          hrs_scnlinf(recn - hrs_h_hdrcnt)    = hrs_scnlinf0
          hrs_mjfrcnt(recn - hrs_h_hdrcnt)    = hrs_mjfrcnt0
          hrs_scnpos(recn - hrs_h_hdrcnt)     = hrs_scnpos0
          hrs_scntyp(recn - hrs_h_hdrcnt)     = hrs_scntyp0
          hrs_qualind(recn - hrs_h_hdrcnt)    = hrs_qualind0
          hrs_linqualflgs(recn - hrs_h_hdrcnt)= hrs_linqualflgs0
          hrs_navstat(recn - hrs_h_hdrcnt)    = hrs_navstat0
          hrs_attangtime(recn - hrs_h_hdrcnt) = hrs_attangtime0
          hrs_rollang(recn - hrs_h_hdrcnt)    = hrs_rollang0
          hrs_pitchang(recn - hrs_h_hdrcnt)   = hrs_pitchang0
          hrs_yawang(recn - hrs_h_hdrcnt)     = hrs_yawang0
          hrs_scalti(recn - hrs_h_hdrcnt)     = hrs_scalti0
          hrs_moonang(recn - hrs_h_hdrcnt)    = hrs_moonang0
          hrs_digitbupdatefg(recn - hrs_h_hdrcnt)
     &                                    =hrs_digitbupdatefg0
          hrs_digitbwrd(recn - hrs_h_hdrcnt)      = hrs_digitbwrd0
          hrs_analogupdatefg(recn-hrs_h_hdrcnt)=hrs_analogupdatefg0

           DO i=1,2
             hrs_filler1(i,recn - hrs_h_hdrcnt)= hrs_filler10(i)
           ENDDO
           DO i=1,4
             hrs_filler2(i,recn - hrs_h_hdrcnt)= hrs_filler20(i)
           ENDDO
           DO i=1,3
             hrs_filler3(i,recn - hrs_h_hdrcnt)= hrs_filler30(i)
           ENDDO
           DO i=1,6
             hrs_filler4(i,recn - hrs_h_hdrcnt)= hrs_filler40(i)
           ENDDO
           DO i=1,11
             hrs_filler5(i,recn - hrs_h_hdrcnt)= hrs_filler50(i)
           ENDDO

           Do i=1,3
           hrs_yawsteering(i,recn - hrs_h_hdrcnt)=hrs_yawsteering0(i)
           hrs_totattcorr(i,recn - hrs_h_hdrcnt) =hrs_totattcorr0(i)
           ENDDO
             
           DO i=1,168
             hrs_ang(i,recn - hrs_h_hdrcnt)    = hrs_ang0(i)
           ENDDO
           
           DO i=1,112
             hrs_pos(i,recn - hrs_h_hdrcnt)    = hrs_pos0(i)
           ENDDO

           DO i=1,hrs_mxchn
             hrs_chqualflg(i,recn-hrs_h_hdrcnt) = hrs_chqualflg0(i)
           ENDDO  

           DO j=1,3
             DO i=1,hrs_mxchn
               hrs_calcof(j,i,recn-hrs_h_hdrcnt)=hrs_calcof0(j,i)
               hrs_scalcof(j,i,recn-hrs_h_hdrcnt)=hrs_scalcof0(j,i)
             ENDDO
           ENDDO

           DO j=1,24
             DO i=0,63 
              hrs_elem(j,i,recn - hrs_h_hdrcnt) = hrs_elem0(j,i)
             ENDDO
           ENDDO

!     If this is NESDIS, subtract 4096 from radiometric counts
           IF (hrs_h_siteid .EQ. "NSS") THEN
             do j=3,22
               do i=0,57 
                 hrs_elem(j,i,recn - hrs_h_hdrcnt) = 
     &             hrs_elem(j,i,recn - hrs_h_hdrcnt) - 4096
               enddo
             enddo
           ENDIF

           DO i=0,63
            hrs_mnfrqual(i,recn-hrs_h_hdrcnt)  = hrs_mnfrqual0(i)
           ENDDO
         
           DO i=1,16
             hrs_anwrd(i,recn-hrs_h_hdrcnt)  = hrs_anwrd0(i)
           ENDDO 

        ENDIF   !IF (ios.NE.0)  THEN

      ENDDO     !DO recn=recn0,recn1
       
  

! Opens localisation files if requested
!--------------------------------------
      IF (hrs_lflg.EQ.1) THEN
      
       satid = hrs_h_satid
       satid = ord1bid (satid,hrs_h_startdatajd)
       scname=noascnam(satid)
 
!   Opens the SATPOS file
!------------------------
       OPEN (unit=satposu,status='old',iostat=ios)

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
       IF ( ios.NE.0 )
     & OPEN(unit=satposu,status='unknown',iostat=ios)

       IF ( ios.NE.0 ) THEN
         CALL ml_wti('error',
     &   'while opening the satpos file I/O status',ios)
         CALL ml_wt('error','hclsetu_algoV4 aborted')
         CALL aappexit(2)
       ENDIF
        
!  Reads and closes the satpos file
!----------------------------------

       ijstrt=hrs_h_startdatajd + 
     &      (hrs_h_startdatatime-errclock*1000.)/86400.d3
       ijend=hrs_h_enddatajd + 
     &       (hrs_h_enddatatime-errclock*1000.)/86400.d3

       CALL sp_read(ijstrt,ijend,satposu,scname,ierr)
       CLOSE(unit=satposu,iostat=ios)

       IF (ierr.NE.0) THEN
         CALL ml_wti('error','return code of sp_read differ from 0',
     &               ierr)
         CALL ml_wt('error','hclsetu_algoV4 aborted')
         CALL ml_wt('error','navigation aborted')
         CALL ml_wt('error','but calibration can work')
         CALL ml_wt('error','try calibration alone')
         CALL ml_wt('error','hclsetu_algoV4 aborted')
         CALL aappexit(2)
       ENDIF

       IF (yaw.LT.-99.d-03) THEN
         CALL def_att(scname,att_mode,yaw,roll,pitch)
         CALL ml_wt('info','take default attitude values')
         CALL ml_wti('info','default attitude mode ',att_mode)
         CALL ml_wtdp('info','default attitude yaw ',yaw)
         CALL ml_wtdp('info','default attitude roll ',roll)
         CALL ml_wtdp('info','default attitude pitch ',pitch)
       ELSE
         CALL ml_wtdp('info','attitude yaw ',yaw)
         CALL ml_wtdp('info','attitude roll ',roll)
         CALL ml_wtdp('info','attitude pitch ',pitch)
       ENDIF
        
        CALL calatt(yaw,roll,pitch,att)
        
! If the file has already been located we do not correct
! again from the clock error
! --------------------------

        IF (hrs_h_epoyr.LE.1950) THEN
        
          OPEN (unit=ckdriftu,status='old',iostat=ios)
          IF (ios.NE.0)
     &     OPEN (unit=ckdriftu,status='unknown',iostat=ios)
          IF (ios.NE.0) THEN
            CALL ml_wti('error',
     &       'opening clock error file I/O status',ios)
            CALL ml_wt('error','hclsetu_algoV4 aborted')
            CALL aappexit(2)
          ENDIF
          
          ijstrt=hrs_h_startdatajd + 
     &      (hrs_h_startdatatime)/86400.d3
          CALL clkerr_get(ckdriftu,ijstrt,errclock,ierr)          
          CLOSE(unit=ckdriftu,iostat=ios)

          IF (ierr.NE.0 .OR. ABS(errclock).GT.2.) THEN
            CALL ml_wti('warning',
     &       'reading clock error value from clkerr_get ',ierr)
            CALL ml_wtr('warning','clock error from file',
     &                 errclock)
            CALL ml_wt('warning','clock error forced to 0')
            errclock=0. 
          ELSE
            CALL ml_wtr('info','clock error value (sec)',errclock)
          ENDIF
          
        ELSE
        
          errclock=0.
          CALL ml_wt('info',
     &     '1B file already corrected from clock error')
     
        ENDIF
        
! Inits Earth location parameters
!--------------------------------
        first_pnav = 1
        first_lnav = 1
        step_pnav = 1
        step_lnav = 1
        nb_pnav = 56
      ENDIF      
 
      RETURN
      END


