      subroutine mhs_cal(iscan,ichan,prtstat,calok,targ_temp,inst_temp,
     &    itavg,spavg,ispvw)
!+
!NOM/NAME $Id: mhs_cal.F 401 2014-07-08 13:47:31Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module computes the MHS calibration coefficients given
!     the mean space view and internal target counts and the internal target
!     temperature. it computes the gain (in units of counts/mW/sqm.ster.cm-1).
!     It then computes the primary calibration coeffs a0, a1 & a2 in units
!     of mW/(sqm.ster.cm-1), mW/(sqm.ster.cm-1)/count & 
!     mW/(sqm.ster.cm-1)/count^2 and then scaled by 1E6, 1E10 & 1E16 to
!     allow them to be represented as integers in the 1B file. 
!     The parameters determined from the thermal vac data supplied
!     in calibration parameter file: uo, u1 and u2 are used to compute
!     these coefficients as defined in the noaa/ukmo calibration spec.
!     see UK Met Office RSI working paper 66 (1994) for overall philosophy.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                              <author>)
! 1.0      June 2003  Based on amb_cal.F                   NCAtkinson
! 1.1      Feb 2007   Bug fix - was always using channel 4 space corr. NCA
! 1.2      Jan 2010   Set c2=0 if the quadratic coefficient overflows. NCA.
! 1.3      July 2014  Separate the ICHAR and IAND functions for portability. NCA
!         
!ARGUMENTS iscan - in scan line number
!          ichan - in MHS channel number
!          prtstat(mxlin) - in status of PRT temp data
!          calok(5,mxlin) - in status of scan line for calibration
!          targ_temp(mxlin) - in mean target temp for scan line from PRT data
!          inst_temp(mxlin) - in mean instrument temp for scan from PRT data
!          itavg(5) - in mean internal target counts for each channel
!          spavg(5) - in mean space view counts for each channel
!          ispvw(mxlin) - in space viewing direction
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!           
!            plank
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
!
!
! includes/commons
#include "mhs_cinit.h"
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
!
! Arguments
      integer*4 iscan,ichan,ispvw(mxlin)
      logical*1 prtstat(mxlin),calok(5,mxlin)
      real      targ_temp(mxlin),inst_temp(mxlin),itavg(5),spavg(5)
      
! local variables
      integer*4 i,lo_offset,ioffset
      real c2,cc,cw,cwg,recg,sdc,recg2,tmp,nonlin_use(3),tref(3)
      real btemp,tsp,radsp,radbb,gain,rdiff,f32,f21,g32,g21,intc

!--END OF DECLARATIONS---------------------------------------------------------

      gain = 0.0
      if(targ_temp(iscan).eq.0. .or. inst_temp(iscan).eq.0.) then
        if (iprint.ge.1)
     &       write(mhs_cllgu,*) 'scan',iscan,' chan',ichan,
     &      ' no cal: Targ & inst temp:',targ_temp(iscan),
     &      inst_temp(iscan)
        calok(ichan,iscan)=.false.
      endif


!...Check which LO is being used for this channel. Examine the switch status.
      lo_offset = 0
      i = ICHAR(switchstat(1,iscan))
      if (ichan.eq.1) then
        if (IAND(i,2) .NE. 0) lo_offset=5  !H1
      else if (ichan.eq.2) then
        if (IAND(i,8) .NE. 0) lo_offset=5  !H2
      else if (ichan.eq.3.or.ichan.eq.4) then
        if (IAND(i,32) .NE. 0) lo_offset=5 !H3/H4
      else
        i = ICHAR(switchstat(2,iscan))
        if (IAND(i,2) .NE. 0) lo_offset=5  !H5
      endif
      
!...Select the correct nonlinearity coefficients, depending on LO
      do i=1,3
        nonlin_use(i) = nonlin(ichan+lo_offset, i)
      enddo

!...Select the correct reference temperatures
      ioffset = mhs_h_tsensid*3 
      do i=1,3
        tref(i) = reftemp(i+ioffset)
      enddo

!...Apply antenna correction to temperature of space
      tsp = tspace + spcorr(ichan,ispvw(iscan)+1)      ! apply antenna corrn

!...Fractional departures from nominal temperatures
      f21 = (inst_temp(iscan)-tref(1))/(tref(2)-tref(1))
      f32 = (inst_temp(iscan)-tref(2))/(tref(3)-tref(2))
      g21 = 1.0 - f21
      g32 = 1.0 - f32

!...Interpolate internal target correction  
      if(calok(ichan,iscan))then
       if (f32 .lt. 0) then
        intc = g21*intcorr(ichan,1) + f21*intcorr(ichan,2)
       else
        intc = g32*intcorr(ichan,2) + f32*intcorr(ichan,3)
       endif
       btemp = targ_temp(iscan) + intc

!...Convert physical temp to effective temp for MHS channel
       btemp = A(ichan)*btemp + B(ichan)

!...Convert  temps to radiance in mW/cm-1/ster/sqm
       call plank(freq(ichan),btemp,radbb)
       call plank(freq(ichan),tsp,radsp)

!...Compute gain                         
       rdiff = radbb-radsp
       if(rdiff.gt.0.00002)then
        gain = (itavg(ichan)-spavg(ichan))/rdiff
       else
        gain = 0.0
       endif

!...Interpolate non-linear coefficient
       if ( f32 .lt. 0) then
        c2 = g21*nonlin_use(1) + f21*nonlin_use(2)
       else
        c2 = g32*nonlin_use(2) + f32*nonlin_use(3)
       endif
      endif

!...Compute calibration coefficients and scale for 1B
      i = 1 + (ichan-1)*3
      if (gain.gt.0.0.and.calok(ichan,iscan))then
        cw = itavg(ichan)
        cc = spavg(ichan)
        cwg = cw/gain
        recg = 1./gain
        recg2 = recg*recg
        tmp = c2*recg2*1E16
        if(abs(tmp).le.2.147483647E9)then
          pmcal(i,iscan) = nint(tmp)
        else
          pmcal(i,iscan) = 0
          c2 = 0.0       !Try a linear calibration
        endif
        tmp = (recg - c2*(cc+cw)*recg2)*1E10
        if(abs(tmp).le.2.147483647E9)then
          pmcal(i+1,iscan) = nint(tmp)
        else
          pmcal(i+1,iscan) = 0
        endif
        tmp = (radbb-cwg + c2*cw*cc*recg2)*1E6
        if(abs(tmp).le.2.147483647E9)then
          pmcal(i+2,iscan) = nint(tmp)
        else
          pmcal(i+2,iscan) = 0
        endif

!remove diagnostic
!        if (iprint.eq.2)then
!          write(mhs_cllgu,*)iscan,ichan,gain,pmcal(i,iscan),
!     &      pmcal(i+1,iscan),pmcal(i+2,iscan) 
!        endif

      else
        if (gain.lt.0.and.iprint.ge.1)
     &       write(mhs_cllgu,*) 'scan',iscan,' chan',ichan,
     &         'no cal: gain=',gain
        pmcal(i,iscan) = 0
        pmcal(i+1,iscan) = 0
        pmcal(i+2,iscan) = 0
        linqualflgs(iscan) = IOR(linqualflgs(iscan),2048) !Set bit 11: some
                                                          !uncalibrated channels
      endif

!
!...Put secondary coefficients in 1B common also interpolate for inst temp
!
      if ( f32 .lt. 0)then
        sdc = g21*u2(ichan,1) + f21*u2(ichan,2)
        sdcal(i,iscan) = nint(sdc)
        sdc = g21*u1(ichan,1) + f21*u1(ichan,2)
        sdcal(i+1,iscan) = nint(sdc)
        sdc = g21*u0(ichan,1) + f21*u0(ichan,2)
        sdcal(i+2,iscan) = nint(sdc)
      else
        sdc = g32*u2(ichan,2) + f32*u2(ichan,3)
        sdcal(i,iscan) = nint(sdc)
        sdc = g32*u1(ichan,2) + f32*u1(ichan,3)
        sdcal(i+1,iscan) = nint(sdc)
        sdc = g32*u0(ichan,2) + f32*u0(ichan,3)
        sdcal(i+2,iscan) = nint(sdc)
      endif
!
      RETURN
      END

