      subroutine mhs_iwttmp(lstat,prtstat,targ_temp,inst_temp)
!+
!NOM/NAME $Id: mhs_iwttmp.F 450 2015-07-10 10:04:52Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE
!     This module extracts the MHS target and instrument PRT data
!     necessary for the calibration and performs quality control checks
!     and sets flags for bad PRT data. The target temperature data
!     extracted from good lines is the averaged internal target PRT
!     temperature (there are 5 PRTs in the MHS target).
!     Instrument temperature for MHS is the LO for channel H5 or H1.
!     If there is no good PRT data prtstat is set to false for that scan line.
!
!DESCRIPTION
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version   date     comment                                 <author>)
! 1.0     June 2003  Based on amb_iwttmp                    NCAtkinson
! 1.1     July 2004  Correct bug                            NCAtkinson
! 1.2     Oct 2005   Fix divide by zero error when computing alpha and beta. NCA
! 1.3     17/01/2006 Philippe Marguinaud : change hex constants from
!                       '1234'X to Z'1234', and float( k ) to
!                       float( int( k ) ), where k has a kind=2
! 1.4     23/01/2006 Allow for systems that have ICHAR as signed. NCA
! 1.5     17/08/2006 use i4word00002000 and i4word00001000. NCA
! 1.6     28/02/2012 double tolerance on instrument temp. NCA
! 1.7     03/07/2014 Separate the ICHAR and IAND functions for portability. NCA
! 1.8     03/07/2015 Change t=float(ICHAR()) to t=ICHAR() for portability.
!
!ARGUMENTS
!      lstat(mxlin)   L*1 in  If false don't use scan line
!      prtstat(mxlin) L*1 out  If .TRUE. then PRT data ok
!      targ_temp(mxlin) r*4 out avg internal target temp deg K
!      inst_temp(mxlin) r*4 out instrument temperature deg K
!
! EXTERNALS
! prtchk
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS---------------------------------------------------------------
      implicit none
!
! includes/commons
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
#include "mhs_cinit.h"
!
! Arguments
      logical*1 lstat(mxlin),prtstat(mxlin)
      real*4    targ_temp(mxlin),inst_temp(mxlin)

! Local variables
      real*4 prtavg,t,d,difft,diffi
      real*4 prtwghtuse(5),obcttemp1(5),sr,sc,scc,scr,alpha,beta,r
      integer*4 iscan,iprt,ires,nprt,ichan,ngoodweight,iold,i
      integer*4 resoffset,prtoffset
      logical*1 validfound
      
      integer*2 i2word1,i2word8
      parameter (i2word1=1)
      parameter (i2word8=8)
      
      integer*4 i4word00002000, i4word00001000
      parameter (i4word00002000=Z'00002000')
      parameter (i4word00001000=Z'00001000')
!
!--END OF DECLARATIONS--------------------------------------------------------

      if (iprint.ge.1) write(mhs_cllgu,*)
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
      if (iprint.ge.1) write(mhs_cllgu,*) '**   SUBROUTINE MHS_IWTTMP*'
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
!
!..Loop in scan line................................................
      do 100 iscan = 1 , mhs_h_scnlin
        if(lstat(iscan))then

! ...get instrument temperature....
          if (mhs_h_tsensid .eq. 0) then 
            t = ICHAR(temps(4,iscan))!primary: LO H5 temperature [QBS5]
          else
            t = ICHAR(temps(1,iscan))!secondary: LO H1 temperature [QBS1]
          endif
          IF (t.LT.0) t = t + 256.0     !Ensure unsigned
          inst_temp(iscan) = thermcnttmp(1) + t*(thermcnttmp(2) + 
     &      t*(thermcnttmp(3) + t*(thermcnttmp(4) + t*thermcnttmp(5))))

! ...get reference resistances & counts and compute linear fit
          i = ICHAR(mode(iscan))
          if (IAND(i,8) .EQ. 0) THEN   !check Mode bit 3   
            resoffset = 0         !PIE-A
            prtoffset = 0
          else
            resoffset = 3         !PIE-B
            prtoffset = 5
          endif
          sr = 0.0
          sc = 0.0
          scc = 0.0
          scr = 0.0
          do i=1,3                !Form sums
            ires = i+resoffset
            sc = sc + refrescts(i,iscan)
            sr = sr + refres(ires)
            scc = scc + refrescts(i,iscan)*
     &            float(int(refrescts(i,iscan)))
            scr = scr + refres(ires)*refrescts(i,iscan)
          enddo
          if ( (3*scc - sc*sc) .EQ. 0 ) then
            alpha = 0
            beta = 0
          else      
            alpha = (sr*scc - sc*scr)/(3*scc - sc*sc)   !intercept
            beta = (3*scr - sr*sc)/(3*scc - sc*sc)      !slope
          endif

! ...convert counts to temperature....
          do i = 1,5
            r = alpha + beta*obctprt(i,iscan)   !Resistance
            iprt = i + prtoffset       !point to correct set of coefficients
            obcttemp1(i) = prtrestmp(1,iprt) + r*(prtrestmp(2,iprt)
     &        +  r*(prtrestmp(3,iprt) + r*prtrestmp(4,iprt)))
            prtwghtuse(i) = prtwght(iprt)  !Select correct set of weights
            obcttemp(i,iscan) = obcttemp1(i)*1000 + 0.5  !Integer
          enddo

! ...carry out a check of PRT consistency on scan line and compute mean
          call prtchk(5,obcttemp1,prtwghtuse,prtmax,prtmin,prtdt,prtavg,
     &      nprt,ngoodweight)

! ...compute avg prt temp for target
          if ( nprt .ge. nprtmin)then
            targ_temp(iscan) = prtavg
            prtstat(iscan) = .true.
            if (nprt .lt. ngoodweight)then
              do ichan=1,5
                chqualflg(ichan,iscan) = 
     &                   IOR(chqualflg(ichan,iscan),i2word1)
              enddo               !Some bad PRTs
            endif
          else
            targ_temp(iscan) = 0.0
            prtstat(iscan) = .false.
!...set flags after interline test
          endif
        else              !if lstat false (i.e. problem with instrument status)
          targ_temp(iscan) = 0.0
          inst_temp(iscan) = 0.0
          prtstat(iscan) = .false.
        endif
!
 100  Continue   ! ......end of scan line loop
!
! Now rescan through PRT temps to do interline test and fill in gaps
! by allowing up to nprtgap scans to have the last good PRT value or the next
! good PRT value. The difference between successive 'good' values must be
! less than the 'prtdt' value from the calibration data file.
! Logic is similar to the interline test on Space/Internal target counts,
! except that here we correct any bad values.
!
      d=prtdt
      validfound=.false.
      iold = -999
      do 400 iscan = 1 , mhs_h_scnlin
        if (.not.validfound) then   !Start of sequence - no good data yet
          if (prtstat(iscan)) then
            if ((iscan-iold).gt.nprtgap) then
              if (iold.gt.0) then
!                write(mhs_cllgu,*)'scan ',iold,': Data gap'
                prtstat(iold)=.false.        !Data gap: old point is invalid
                targ_temp(iold)=0.0
              endif
            else
              if(abs(targ_temp(iold)-targ_temp(iscan)).le.d.and.
     &           abs(inst_temp(iold)-inst_temp(iscan)).le.d*2)then
                validfound=.true.    !Diff good - both points accepted

! See whether there are any prior points which can be assigned this temperature
                do 200 i=iold-1,iold-nprtgap-1,-1
                  if (i.lt.1) goto 210
                  if (prtstat(i)) goto 210   !Quit loop at first good point
! Replace faulty target or instrument temps with old value
                  difft = abs(targ_temp(i)-targ_temp(iold))
                  diffi = abs(inst_temp(i)-inst_temp(iold))
                  if(difft.gt.d) targ_temp(i)=targ_temp(iold)
                  if(diffi.gt.d*2) inst_temp(i)=inst_temp(iold)
                  if (iprint.ge.1)
     &             write(mhs_cllgu,*)'Scan ',i,
     &              ': PRT substituted by later value'
  200           continue
  210           continue

              else
                prtstat(iold)=.false.  !Diff bad - reject old point
                targ_temp(iold)=0.0
              endif
            endif
            iold=iscan
          endif
        else                         !Valid data already present
          if ((iscan-iold).gt.nprtgap) then
            validfound=.false.   !Too long ago - start again
!            if (prtstat(iscan).eq..true.) then
            if (prtstat(iscan)) then
              iold=iscan
            else
              iold=-999
            endif
          else                         !Test for consistency
            difft = abs(targ_temp(iold)-targ_temp(iscan))
            diffi = abs(inst_temp(iold)-inst_temp(iscan))
            if (difft.le.d.and.diffi.le.d*2) then
              iold=iscan              !New point valid - update iold
            else
! Replace faulty target or instrument temps with old value, but set prtstat 
! false so that final statistics show the bad data
              if(difft.gt.d) targ_temp(iscan)=targ_temp(iold)
              if(diffi.gt.d*2) inst_temp(iscan)=inst_temp(iold)
              prtstat(iscan)=.false.
              if (iprint.ge.1)
     &           write(mhs_cllgu,*)'Scan ',iscan,
     &              ': PRT substituted by prior value'
            endif
          endif
        endif
 400  continue  ! end of scan line loop---------------------
      if (.not.validfound.and.iold.gt.0) then 
        prtstat(iold)=.false.      !Set last point
        targ_temp(iold)=0.0
      endif
!
! Set flags
!
      do iscan=1,mhs_h_scnlin
        if (targ_temp(iscan).eq.0.0) then
          linqualflgs(iscan)=IOR(linqualflgs(iscan),i4word00002000)  !Bad PRT data
          do ichan=1,5
            chqualflg(ichan,iscan)=IOR(chqualflg(ichan,iscan),i2word8)
          enddo
        else
          if (.not.prtstat(iscan)) then
            linqualflgs(iscan)=IOR(linqualflgs(iscan),i4word00001000)  !Marginal PRT
          do ichan=1,5
            chqualflg(ichan,iscan)=IOR(chqualflg(ichan,iscan),i2word8)
          enddo
          endif
        endif
      enddo

      if(iprint.gt.1)then
        do iscan = 1 , mhs_h_scnlin
           write(mhs_cllgu,*)'Scan',iscan,
     &   ' Target prt temp=',targ_temp(iscan)
        enddo
      endif
!
      return
      end

