!+ Moon angle calculations for MHS
!
      SUBROUTINE MHS_MOON
      IMPLICIT NONE
!
! Purpose:
!  Calculate angle between moon and each MHS space view.
!
! Method:
!  1) Calculate position of moon at start and end of an MHS orbit 
!  2) Calculate space view azimuth and elevation
!  3) Calculate moon local azimuth and elevation for each scan
!  4) Compute angular differences and store in common area
!
!  Current code owner: N.C.Atkinson
!
!  History:
!
!  Version  Date       Comment
!  1.0    Sept 2003  Original code.        N C Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
#include "mhsclin.h"
#include "mhsscn.h"
#include "mhs1b.h"
#include "mhs_cinit.h"
!
! Local variables:
      INTEGER i,iscan,iv2       !Array index
      REAL    dec               !Declination
      REAL    amsual            !Altitude of space view
      REAL    sinamsual(4)      !sin altitude 
      REAL    cosamsual(4)      !cos altitude
      REAL    angle_diff(4)     !Angle between moon and each space view
      REAL    lat(2)            !Latitude of samples 45 and 46
      REAL    lon(2)            !Longitude of samples 45 and 46
      REAL    azcorr            !Azimuth correction (radians)
      REAL    hour              !Time of day in hours
      REAL    f1,f2             !Used for interpolation
      REAL*8  scandays          !Time of scan in days from 12Z, 1 Jan 2000
      REAL*8  tdays(2)          !Start/end time in days from 12Z, 1 Jan 2000
      REAL    ra(2)             !Start/end right ascension of moon (radians)
      REAL    sindec(2)         !Start/end sine of moon declination
      REAL    cosdec(2)         !Start/end cosine of moon declination
      REAL    ra1,sindec1,cosdec1
      REAL    d2r               !Degrees to radians
      REAL    minangle, maxangle
      REAL    p                 !Antenna position
!
!
! Function and subroutine calls:
      EXTERNAL MOON_POSITION    !Astronomical calculation
      EXTERNAL MOON_AMSU        !This subroutine does the geometry
!
!- End of header

      if (iprint.ge.1) write(mhs_cllgu,*)
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
      if (iprint.ge.1) write(mhs_cllgu,*) '**   SUBROUTINE MHS_MOON  *'
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'

      minangle = 180.0
      maxangle = 0.0
      d2r = 3.1415927/180.0     !Degrees to radians

!Calculate start and end times, in days since 12h 1 Jan 2000 (t=0)
!Note: ama_h_startdatajd is day number starting 00h 1 Jan 1950 (day=1)
!50 years = 18262 days.

      tdays(1) = mhs_h_startdatajd - 18262.5
     &    + mhs_h_startdatatime/(24*3600000.0)
      tdays(2) = mhs_h_enddatajd - 18262.5 
     &    + mhs_h_enddatatime/(24*3600000.0)

!Calculate moon position at start and end of orbit. This could be done for each 
!scan line, but it saves computation if we interpolate.
!Result is geocentric right ascension and declination

      CALL MOON_POSITION(tdays(1), ra(1), dec)
      IF (iprint.ge.2) write(mhs_cllgu,*)
     & 'Moon start ra,dec (radian):',ra(1),dec
      sindec(1) = sin(dec)
      cosdec(1) = cos(dec)
      CALL MOON_POSITION(tdays(2), ra(2), dec)
      IF (iprint.ge.2) write(mhs_cllgu,*)
     & 'Moon end ra,dec (radian)  :',ra(2),dec
      sindec(2) = sin(dec)
      cosdec(2) = cos(dec)
      IF ((ra(1)-ra(2)).gt.3.14159) THEN     !Allow for wrap-around in RA
        ra(2)=ra(2)+6.283185
      ENDIF

!....Loop over all scan lines

C      IF (iprint.ge.2) write(mhs_cllgu,*) 'Moon angles:'
      do iscan=1,mhs_h_scnlin

!Select latitude and longitude in centre of swath, samples 45 and 46
        lat(1) = earthl(89,iscan)*1E-4  !Pix 45
        lat(2) = earthl(91,iscan)*1E-4
        lon(1) = earthl(90,iscan)*1E-4  !Pix 46
        lon(2) = earthl(92,iscan)*1E-4
        IF ((lat(1).EQ.lat(2)).AND.(lon(1).EQ.lon(2))) THEN
C          write(mhs_cllgu,*) lat(1),lat(2),lon(1),lon(2)
          do i=1,4
            angle_diff(i)=180.0       !Cannot use navigation
            moonang(i,iscan) = 18000
          enddo
        ELSE

!Correct for finite time delay between samples. For MHS the delay is
!0.019 sec (TBC). In this time the satellite will have travelled 
!2*pi*6371*0.019/(101*60) = 0.1255km. The cross-track sampling dist is
!tan(1.11111 deg)*850km = 16.49km. So the azimuth correction is
!0.1255/16.49 = 0.00761 radians. Add empirical correction from data file.

          azcorr = 0.00761 + moon_azcorr*d2r   !radians

!Time of day in hours and days

          hour = scnlintime(iscan)/3600000.0
          scandays = hour/24.0
          scandays = scandays + NINT(tdays(1)) - 0.5
          IF (scandays.LT.tdays(1)) THEN
            scandays = tdays(1)         !This should not happen
          ELSE IF (scandays.GT.tdays(2)) THEN
            scandays = tdays(2)
          ENDIF
          f1 = (tdays(2)-scandays)/(tdays(2)-tdays(1))  !Start/end weighting
          f2 = 1.0-f1
      
!Calculate the space view altitude using space view info in status word.
!Add 1.04 degrees to allow for parallax and any supplied correction

          do i=1,4
            iv2 = (i-1)*6+1
            p = -90.0 + coldcal(iv2,iscan)*ant_coef    ! posn decode
            amsual = (p - 180.0 + 1.04 + moon_elevcorr)*d2r
            sinamsual(i) = sin(amsual)
            cosamsual(i) = cos(amsual)
          enddo

!Calculate angular separation between space view and moon, for 4 space views.
!Linearly interpolate the moon parameters.

          ra1 = f1*ra(1)+f2*ra(2)                !Interpolate
          sindec1 = f1*sindec(1)+f2*sindec(2)
          cosdec1 = f1*cosdec(1)+f2*cosdec(2)

          CALL moon_amsu(lat,lon, azcorr, scandays, hour, 
     &     ra1, sindec1, cosdec1, 4,sinamsual,cosamsual,angle_diff)
     
!Store in common area
          DO i=1,4
            moonang(i,iscan) = NINT(angle_diff(i) * 100.0)
            IF (angle_diff(i).lt.minangle) minangle=angle_diff(i)
            IF (angle_diff(i).gt.maxangle.and.angle_diff(i).lt.180.0)
     &          maxangle=angle_diff(i)
          ENDDO
        ENDIF
C       IF (iprint.ge.2) then
C         write(mhs_cllgu,'(I4,4F6.1)') iscan,(angle_diff(i),i=1,4)
C       ENDIF

      ENDDO     !Scan line loop
      IF (iprint.ge.1) then
        write(mhs_cllgu,'(A,2F7.2)') 'Min and max moon angles:',
     &      minangle,maxangle
      ENDIF

      RETURN
      END  
