      subroutine mhs_smpmn(lstat,ispvw,itmean,spmean,sampok)
!+
!NOM/NAME $Id: mhs_smpmn.F 401 2014-07-08 13:47:31Z frna $
!
!------------------------------------------------------------------------------
!BUT/PURPOSE 
!     This module computes the average MHS internal target and 
!     space view counts for a single scan line. 
!     Also performs quality control on the space view and internal
!     target view counts and updates the scan line flags if necessary.
!
!METHOD
! Although based on amb_smpmn.F it has been heavily modified to conform more
! closely with the method used by NESDIS for AMSU-B:
! 1) Check antenna pointing. Flag and reject if outside limits.
! 2) Check all samples are within ADC dynamic range (new for MHS)
! 3) Reject any space samples that are within a specified angle to the moon. If
!    all are within this angle then retain the sample with lowest counts.
! 4) Reject individual samples that are outside gross limits (but use the rest)
! 5) Compute mean space and target counts from remaining good samples
! 6) Reject all samples if any one of the good samples differs from the mean
!    by more than the specified tolerance
! 7) Set channel quality flags
! 8) Check line-to-line consistency of mean counts
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!  
!PROPRIETAIRE/OWNER  
!     Roger SAUNDERS
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date      comment                         <author>)
! 1.0      April 2004 Based on amb_smpmn.F            N C Atkinson
! 1.1      April 2005 Bug fix in sampok indexing. NCA
! 1.2      10/06/05   Fix the test on validity bits. NCA
! 1.3      11/01/06   Bug fix: reset coldmin, ismin and nmoon every scan. NCA
! 1.4      11/09/07   Allow calibration when chvalflgs indicates ADC range
!                     warning is set. NCA
! 1.5      03/07/14   Separate the ICHAR and IAND functions for portability. NCA
!         
!ARGUMENTS  lstat(mxlin) - in status of scan
!           ispvw(mxlin) - in space view selected (0-3)
!           itmean(5,mxlin) - out scan mean internal target counts
!           spmean(5,mxlin) - out scan mean space view counts
!           sampok(5,mxlin)- in/out cal view counts flag
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!                  AT LONDON RD., BRACKNELL, BERKS, RG12 2SZ, U.K.
!#########################################################################

!--DECLARATIONS----------------------------------------------------------------
      implicit none                                                     
!
!
! includes/commons
#include "mhs_cinit.h"
#include "mhsclin.h"
#include "mhs1b.h"
#include "mhsscn.h"
!
! local variables
      integer i,ichan,is,iscan,iv,iv1,iv2   !index of loops
      integer ispvw(mxlin),iold,chvalnum,nwarm,ncold
      integer coldcts(4),warmcts(4)
      integer coldmin, ismin, nmoon
      integer errval(4)
      data errval/1,2,4,8/   !Values for position error flag
      real scmeanw,scmeanc,p,diffcounts
      real itmean(5,mxlin),spmean(5,mxlin),s1,s2,i1,i2,d,tmean
      logical*1 sampok(5,mxlin)
      logical*1 coldok(4),warmok(4),lstat(mxlin),validfound
      logical*1 warnwarm,warnspace,warnvalid(5)
      data warnwarm/.true./
      data warnspace/.true./
      data warnvalid/.true.,.true.,.true.,.true.,.true./
      logical*1 errorwarm,errorspace,firstchan,firstsample
      
      integer*2 i2word2,i2word4,i2word16,i2word32,i2word48
      parameter (i2word2=2)
      parameter (i2word4=4)
      parameter (i2word16=16)
      parameter (i2word32=32)
      parameter (i2word48=48)
!
!--END OF DECLARATIONS---------------------------------------------------------
!
      if (iprint.ge.1) write(mhs_cllgu,*)
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'
      if (iprint.ge.1) write(mhs_cllgu,*) '**   SUBROUTINE MHS_SMPMN *'
      if (iprint.ge.1) write(mhs_cllgu,*) '***************************'

!----start scan line loop----------------------------------------------------
      do 200 iscan = 1 , mhs_h_scnlin
      spaceval(iscan) = char(0)
      targval(iscan) = char(0)
      if (lstat(iscan)) then             !scan line ok

!...Do antenna pointing check on space views. As well as computing the position
!   from the counts, look at the flag supplied with the instrument telemetry.
!   Assume zero counts at internal target view. Define angle = 90 deg at nadir 

       do 10 is=1,nview    !Loop round each view
         iv2 = (is-1)*6+1
         p = -90.0 + coldcal(iv2,iscan)*ant_coef    ! posn decode
         if ( p.lt. (scanpos(is,ispvw(iscan)+1)-pos_errc) .or. 
     &        p.gt. (scanpos(is,ispvw(iscan)+1)+pos_errc) .or.
     &        ichar(spaceval(iscan)) .ne. 0) then
           do ichan=1,5
             sampok(ichan,iscan)=.false.
           enddo
           spaceval(iscan) = CHAR(ICHAR(spaceval(iscan)) + errval(is))
           if ( iprint.ge.1 .and. warnspace) then
             write (mhs_cllgu,'(A,I4,A,F6.1,A)')
     &        '***warning space view position error, line',iscan,
     &        ',',p-scanpos(is,ispvw(iscan)+1),' deg'
             warnspace = .false.
           endif
!Flag questionable calibration
           linqualflgs(iscan) = IOR(linqualflgs(iscan),512)  !linqualflgs bit 9
         else
           if (iprint.ge.1 .and. .not.warnspace) then
             write(mhs_cllgu,*)'Space view position now OK, line',iscan
             warnspace = .true.
           endif
         endif

!...do antenna pointing check on internal target views

         p = -90.0 + warmcal(iv2,iscan)*ant_coef    ! posn decode
         if (p.lt.0) p = p+360.0
         if ( p.lt. (scanpos(is+4,ispvw(iscan)+1)-pos_errc) .or. 
     &        p.gt. (scanpos(is+4,ispvw(iscan)+1)+pos_errc) .or.
     &        ichar(targval(iscan)) .ne. 0) then
           do ichan=1,5
             sampok(ichan,iscan)=.false.
           enddo
           targval(iscan) = CHAR(ICHAR(targval(iscan)) + errval(is))
           if ( iprint.ge.1 .and. warnwarm) then
             write (mhs_cllgu,'(A,I4,A,F6.1,A)')
     &        '***warning target view position error, line',iscan,
     &        ',',p-scanpos(is+4,ispvw(iscan)+1),' deg'
             warnwarm = .false.
           endif
!Flag questionable calibration
           linqualflgs(iscan) = IOR(linqualflgs(iscan),256)  !linqualflgs bit 8
         else
           if (iprint.ge.1 .and. .not.warnwarm) then
             write(mhs_cllgu,*)'Target view position now OK, line',iscan
             warnwarm = .true.
           endif
         endif
   10  continue

!----start channel loop------------------------------------------------------

       chvalnum = 256               !Channel validity word bit 8
       firstchan = .true.

       do 100 ichan = 1 , 5
       spmean(ichan,iscan) = 0.0    !Initialise arrays
       itmean(ichan,iscan) = 0.0
       chvalnum = chvalnum/2        !Next channel validity bit
       
       if (sampok(ichan,iscan)) then   !Omit any channels that are off
        scmeanw = 0.0
        scmeanc = 0.0
        errorwarm=.true.
        errorspace=.true.

!...Check channel valid flags, i.e. all samples are within ADC dynamic range.
!   Warning is only printed out the first time it occurs in a sequence.
!   (Note that for NOAA-N thermal vac data the validity bits are always zero,
!   so change to ".NE.0". The bits are correct for flight data.)
!   Sept 2007: from discussions with EUMETSAT it seems the flag is set if the 
!   dynamic range is less than 50% filled. This is not an error, so do not
!   reject the data. 

        i = ICHAR(chvalflgs(iscan))
        if (IAND(i,chvalnum).EQ.0) then
C         sampok(ichan,iscan) = .false.
         if (iprint.ge.1 .and. warnvalid(ichan)) then
           warnvalid(ichan) = .false.
           write(mhs_cllgu,'(A,I2,A,I4,A,Z2.2)')
     &        'Warning: ADC range, channel',ichan,' line ',iscan,
     &        ' chvalflgs=',ICHAR(chvalflgs(iscan))
         endif
        else
         if (iprint.ge.1 .and. .not.warnvalid(ichan)) then
           warnvalid(ichan) = .true.
           write(mhs_cllgu,'(A,I2,A,I4)')
     &        'ADC range now OK, channel',ichan,' line ',iscan
         endif
        endif

!...perform quality checks on counts for each view and compute mean counts

        nwarm = 0     !Number of good warm samples
        ncold = 0     !Number of good cold samples
        scmeanw = 0.0
        scmeanc = 0.0
        coldmin = 65535
        ismin = 1     !Sample with lowest space counts
        nmoon = 0     !Number of samples close to moon

        do 15 is = 1 , nview
         firstsample = .true.
         iv = (is-1)*6+ichan+1
         warmok(is) = .true.
         coldok(is) = .true. 
         warmcts(is) = warmcal(iv,iscan) 
         coldcts(is) = coldcal(iv,iscan) 

!...find the sample with the lowest space counts (in case all fail the moon
!   angle check), and check the moon angles.

         if (coldcts(is) .lt. coldmin) then
           coldmin = coldcts(is)
           ismin = is
         endif

         if (moonang(is,iscan)*0.01 .lt. moonthresh) then
           nmoon = nmoon + 1
           coldok(is) = .false.
           if (iprint.ge.1.and.firstchan)
     &       write(mhs_cllgu,'(A,I4,A,I2,A,F7.2)')
     &       'Scan ',iscan,' sample',is,
     &       ' Moon angle=',moonang(is,iscan)*0.01
           if (firstsample) then
             firstsample = .false.
             mhs_h_moonscnlin = mhs_h_moonscnlin + 1
           endif
         endif
  15    continue

!...If all samples are close to the moon, use the sample with lowest counts
!   For MHS there must be 1 sample that is OK given the 1.1 deg beam width
!   and 1.1 deg sample separation. Note that we need to set the value of
!   moonthresh rather conservatively to allow for errors in the navigation.

        if (nmoon.eq.nview) then
          coldok(ismin) = .true.
        endif

        do 16 is = 1 , nview

!...do gross limit check. Reject sample if outside gross limits
!   Warnings are printed out once per scan line

         if (warmcts(is).gt.intcntmax(ichan) .or. 
     &   warmcts(is).lt.intcntmin(ichan)) then
           warmok(is) = .false.
           if (iprint.ge.1.and.errorwarm)
     &       write(mhs_cllgu,*)'Warm view gross limit err: scan',iscan,
     &        ',chan',ichan,',counts',warmcal(iv,iscan)
           errorwarm=.false.
         endif

         if (coldcts(is).gt.spcntmax(ichan) .or. 
     &   coldcts(is).lt.spcntmin(ichan)) then
           coldok(is) = .false.
           if (iprint.ge.1.and.errorspace)
     &       write(mhs_cllgu,*)'Cold view gross limit err: scan',iscan,
     &        ',chan',ichan,',counts',coldcts(is)
           errorspace=.false.
         endif

!...Sum the counts
         if (warmok(is)) then
           scmeanw = scmeanw + warmcts(is)
           nwarm = nwarm + 1
         endif
         if (coldok(is)) then
           scmeanc = scmeanc + coldcts(is)
           ncold = ncold + 1
         endif

  16    continue

!Print out moon angles, cold counts & warm counts (remove later)
!        if (iprint.ge.2) then
!          if (firstchan) write(mhs_cllgu,'(A,I5,A,4F7.2)')
!     &        ' Scan ',iscan,' moonang=',(moonang(is,iscan)*0.01,is=1,4)
!          write(mhs_cllgu,'(I5,4I6,2X,4I6)')
!     &        ichan+15,(coldcts(is),is=1,4),(warmcts(is),is=1,4)
!        endif

!
!...Look at differences from mean and reject if too large
!
        if (sampok(ichan,iscan)) then

         if (nwarm.gt.0) then
           itmean(ichan,iscan) = scmeanw/float(nwarm)
           tmean = itmean(ichan,iscan)
           do is=1,nview
             diffcounts = warmcts(is)-tmean
             if (warmok(is).and.abs(diffcounts).gt.dcount(ichan)) then
               if (iprint.ge.1) write(mhs_cllgu,'(A,I4,A,I1,A,F6.0)')
     &               'Scan ',iscan,' Warm diff',is,'=',diffcounts
               sampok(ichan,iscan)=.false.
               itmean(ichan,iscan) = 0.0
               nwarm = 0
             endif
           enddo
         endif
              
         if (ncold.gt.0) then
           spmean(ichan,iscan) = scmeanc/float(ncold)
           tmean = spmean(ichan,iscan)
           do is=1,nview
             diffcounts = coldcts(is)-tmean
             if (coldok(is).and.abs(diffcounts).gt.dcount(ichan)) then
               if (iprint.ge.1) 
     &           write(mhs_cllgu,'(A,I4,A,I2,A,I1,A,F6.0)')
     &               'Scan ',iscan,' chan',ichan,
     &               ' Cold diff',is,'=',diffcounts
               sampok(ichan,iscan)=.false.
               spmean(ichan,iscan) = 0.0
               ncold = 0
             endif
           enddo
         endif

        endif   !sampok(ichan,iscan)
!
!...Set channel quality flags
!
        if (ncold.eq.0) then          
         chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word16)
        else if (ncold.lt.nview) then
         chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word2)
        endif
             
        if (nwarm.eq.0) then          
         chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word32)
        else if (nwarm.lt.nview) then
         chqualflg(ichan,iscan) = IOR(chqualflg(ichan,iscan),i2word4)
        endif

       endif      !sampok(ichan,iscan)     
       firstchan = .false.

 100   continue   !-------end of channel loop------------

      endif      !lstat(iscan) OK

 200  continue   !-------end of scan line loop----------
!
! Now look at line-to-line consistency of the Space and Internal target counts.
! The difference between successive 'good' values must be less than the 
! 'dcount' values for each channel from the calibration data file. 
! A pair of consistent readings is necessary to define the start of a 'good'
! sequence, after which inconsistent readings are flagged as bad. If
! more than 'ngap' scans elapses since the last good point, the sequence is
! restarted. Isolated valid readings (i.e. more than 'ngap' scans between
! each one) are considered unreliable and are flagged as bad.
!
! Note that this code gives good resilience against occasional bad readings but
! will not cope with a calibration jump. So best to keep 'ngap' fairly small.
!
      do 500 ichan = 1 , 5
        validfound=.false.
        iold = -999
        d = dcount(ichan)

        do 400 iscan = 1 , mhs_h_scnlin
          if (.not.validfound) then   !Start of sequence - no good data yet
            if(lstat(iscan).and.sampok(ichan,iscan))then
              if ((iscan-iold).gt.ngap) then
                if (iold.gt.0) then
                  sampok(ichan,iold)=.false. !Data gap: old channel is invalid
                  linqualflgs(iold)=IOR(linqualflgs(iold),2048) !Flag bad channel
                      !This point could be OK, but there are no other good 
                      !points in the vicinity, so flag both internal and space 
                      !counts as bad.
                  chqualflg(ichan,iold)=
     &                         IOR(chqualflg(ichan,iold),i2word48)
                endif
              else
                s1=spmean(ichan,iold)   !Test space and internal counts for
                s2=spmean(ichan,iscan)  !consistency.
                i1=itmean(ichan,iold)
                i2=itmean(ichan,iscan)
                if(abs(s1-s2).le.d.and.abs(i1-i2).le.d)then
                  validfound=.true.    !Diff good - both points accepted
                else
                  sampok(ichan,iold)=.false.  !Diff bad - reject old point
                  linqualflgs(iold)=IOR(linqualflgs(iold),2048) !Flag bad channel
                  if (abs(s1-s2).gt.d) then
                    write(mhs_cllgu,'(A,I4,A,F10.1)')
     &               'Scan ',iscan,
     &               ' space counts diff from last good scan=',s1-s2
                   chqualflg(ichan,iold)=
     &                   IOR(chqualflg(ichan,iold),i2word16)
                  endif
                  if (abs(i1-i2).gt.d) then
                    write(mhs_cllgu,'(A,I4,A,F10.1)')
     &               'Scan ',iscan,
     &               '  warm counts diff from last good scan=',i1-i2
                    chqualflg(ichan,iold)=
     &                   IOR(chqualflg(ichan,iold),i2word32)
                  endif
                endif
              endif
              iold=iscan
            endif
          else          !Valid data already present

!Start again if gap is too long or if hardware gain or DC offset has changed
            if (((iscan-iold).gt.ngap).or.
     &          (dcoffset(ichan,iold).ne.dcoffset(ichan,iscan)).or.
     &          (chgains(ichan,iold).ne.chgains(ichan,iscan))) then 
              validfound=.false.
              if (lstat(iscan).and.sampok(ichan,iscan))then
                write(mhs_cllgu,*)'Re-set sequence at scan',iscan
                iold=iscan  
              else
                write(mhs_cllgu,*)
     &              'Re-set sequence to invalid at scan',iscan
                iold=-999
              endif
            else
              if (lstat(iscan).and.sampok(ichan,iscan))then
                s1=spmean(ichan,iold)
                s2=spmean(ichan,iscan)
                i1=itmean(ichan,iold)
                i2=itmean(ichan,iscan)
                if(abs(s1-s2).le.d.and.abs(i1-i2).le.d)then
                  iold=iscan             !New point valid - update iold
                else
                  sampok(ichan,iscan)=.false.  !New point is inconsistent
                  linqualflgs(iscan)=IOR(linqualflgs(iscan),2048) !Bad channel
                  if (abs(s1-s2).gt.d) then
                    write(mhs_cllgu,'(A,I4,A,I4,A,F10.1)')
     &               'Scan ',iscan,' Chan ',ichan,
     &               ' space counts diff from last good scan=',s1-s2
                    chqualflg(ichan,iscan)=
     &                     IOR(chqualflg(ichan,iscan),i2word16)
                  endif
                  if (abs(i1-i2).gt.d) then
                    write(mhs_cllgu,'(A,I4,A,I4,A,F10.1)')
     &               'Scan ',iscan,' Chan ',ichan,
     &               '  warm counts diff from last good scan=',i1-i2
                    chqualflg(ichan,iscan)=
     &                     IOR(chqualflg(ichan,iscan),i2word32)
                  endif
                endif
              endif  
            endif
          endif
 400    continue  ! end of scan line loop---------------------
        if (.not.validfound.and.iold.gt.0) then
          sampok(ichan,iold)=.false.  !Set last point flags
          linqualflgs(iold)=IOR(linqualflgs(iold),2048)    !Bad channel
          chqualflg(ichan,iold)=IOR(chqualflg(ichan,iold),i2word48)
        endif
 500  continue    ! end of channel loop------------------------
      return
      end





