!C LIBS = 
      PROGRAM convert_chrpt
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id:
!
! Description:
! Convert CHRPT data from FY1 satellite to a form that is compatible with NOAA
! HRPT, taking just the AVHRR-like channels. Output can be fed into the AAPP
! decommutation routine. We create some dummy TIP data in order to satisfy the
! AAPP error checks. Also some of the variables (target temps and warm cal
! counts) are stored in non-standard locations in the output file.
!
! The input frame length is 27740 bytes at the Met Office. May be different for
! other receiver manufacturers. This represents 22180 10-bit words when unpacked.
! Alternatively, can accept an input file that has already been unpacked into
! 16-bit words. 
!
! Inputs (read from standard input): 
!    name of input file, name of output file, day of year, unpacked (1=true)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2007, EUMETSAT, All Rights Reserved.
!
! History:
! 1.0  06/08/2007  Nigel Atkinson
! 1.1  27/08/2008  Accept input data in unpacked 16-bit format
! 1.2  20/02/2009  Correction: initialise "unpacked" to false.
!                  Change satid to 1 to avoid conflict with NOAA-19.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! References:
! NOAA:  http://www2.ncdc.noaa.gov/docs/klm/html/c4/sec4-1.htm
! CHRPT: http://www.sat.dundee.ac.uk/hrptformat.html
!
!-----------------------------------------------------------------------------------------------------------------|
! CHRPT Frame Format (detailed)  
!
!-----------------------------------------------------------------------------------------------------------------|
!   Function   | No of words | Word position | Bit number and meaning                                             |
!-----------------------------------------------------------------------------------------------------------------|
! Frame Sync   |     6       |       1       | 1010000100 First 60 bits are a 63 bit PN generator started in the  |
!              |             |       2       | 0101101111 all ones state. The generator polynominal is            |
!              |             |       3       | 1101011100 X^6 + X^5 + X^2 + X + 1                                 |
!              |             |       4       | 0110011101                                                         |
!              |             |       5       | 1000001111                                                         |
!              |             |       6       | 0010010101                                                         |
!-----------------------------------------------------------------------------------------------------------------|
! ID           |    2        |       7       | 1-4 bit    Satellite ID, 1100=FY1C, 1101=FY1D                      |
!              |             |       7       | 5-10 bit   Spare word                                              |
!              |             |       8       |            Spare word                                              |
!-----------------------------------------------------------------------------------------------------------------|
! Time Code    |    4        |       9       | 1-9 bit    binary day count (always zero)                          |
!              |             |       9       | 10 bit	   0                                                      |
!              |             |      10       | 1-3 bit	   101                                                    |
!              |             |      10       | 4-10 bit   part of binary millisecond of day count                 |
!              |             |    11-12      |            part of binary millisecond of day count                 |
!-----------------------------------------------------------------------------------------------------------------|
! Telemetry    |    10       |    13-22      | Ramp calibration of 10 channels of MVISR                           |
! (Ramp)       |             |               |                                                                    |
!-----------------------------------------------------------------------------------------------------------------|
! Telemtery    |    10       |      23       | Temperature of first stage of radiant cooler A                     |
! (Temp)       |             |      24       | Temperature of first stage of radiant cooler B                     |
!              |             |      25       | Temperature of second stage of radiant cooler A                    |
!              |             |      26       | Temperature of second stage of radiant cooler B                    |
!              |             |      27       | Temp controlling voltage of 2nd stage radiant cooler A             |
!              |             |      28       | Temp controlling voltage of 2nd stage radiant cooler B             |
!              |             |    29-32      | Temp of sheath of radiometer, 1 word per platinium resistance bulb |
!-----------------------------------------------------------------------------------------------------------------|
! Reference    |     60      |    33-92      | 6 sampling words for every channel                                 |
! Black Body   |             |               |                                                                    |
!-----------------------------------------------------------------------------------------------------------------|
! Space        |  100        |    93-192     | 10 sampling words for every channel                                |
!-----------------------------------------------------------------------------------------------------------------|
! Spare words  |   1408      |   193-1600    | 00000 11101 Derived by inverting the output of a 1023 PN sequence  |
!              |             |               | 00000 01100 prodived by a feedback shift register generating the   |
!              |             |               | 10010 01010 polynominal X^ + X^ + X^ + X^ + 1                      |
!              |             |               | ..... .....                                                        |
!              |             |               | 00100 11110                                                        |
!              |             |               | 11111 11000                                                        |
!              |             |               | 11000 01101                                                        |
!              |             |               | 11011 00101                                                        |
!-----------------------------------------------------------------------------------------------------------------|
! Earth Data   |    20480    |   1601-22080  | 2048 words for every channels from channel 1 to channel 10.        |
!              |             |               | Each frame contains the data obtained during one Earth scan of the |
!              |             |               | MVISR sensor. The data from the 10 sensor channels of the MVIRS    |
!              |             |               | are time multiplexed.                                              |
!-----------------------------------------------------------------------------------------------------------------|
! Aux. Sync    |      100    |  22081-22180  | 11111 00010 Derived from the non inverting output of a 1023 bit PN |
!              |             |               | 11111 10011 sequence provided by a feedback shift register         |
!              |             |               | 01101 10101 generating the polynominal X^10 + X^5 + X^2 + X + 1    |
!              |             |               | ..... .....                                                        |
!              |             |               | 01111 10000                                                        |
!              |             |               | 11110 01100                                                        |
!-----------------------------------------------------------------------------------------------------------------| 

!Parameters

      integer bytes_in
      parameter (bytes_in=27740)  !packed bytes input (per frame)
      integer bytes_in_unpacked
      parameter (bytes_in_unpacked=44360)  !for use with unpacked input
      integer words_in
      parameter (words_in=22180)  !unpacked words input
      integer bytes_out
      parameter (bytes_out=22180) !bytes output
      integer words_out
      parameter (words_out=11090) !words output
      integer nchan
      parameter (nchan=10)        !number of channels in input data
      integer nspot
      parameter (nspot=2048)      !number of spots per scan
      integer nfours
      parameter (nfours=words_in/4) !number of groups of 4 input words
      integer satid
      parameter (satid=1) !make this consistent with satid.txt (AVHRR & TIP I/Ds)
                          !Can't use the original ID because 13 and 14 are taken.
      integer tipstart
      parameter (tipstart=104)   !Position of start of TIP frames in NOAA format
      integer*2 i2word127
      parameter (i2word127=127)
      
      integer rldiv       !Record length dividor, compiler-dependent
#ifdef AAPP_RLDIV
      parameter ( rldiv = AAPP_RLDIV )
#else
      parameter ( rldiv = 1 )
#endif

!Functions

      integer tipword

!Variables

      character*100 name_in
      character*100 name_out

      character*1 b(bytes_in)     !input bytes
      integer*2 y(words_in)       !input 10-bit words
      integer*2 noaa(words_out)   !output 10-bit words

      integer ios,irec,i,j,k,daynum,ib(5),millisecs
      integer jj(nfours),kk(nfours)
      
      integer bb_index(30)  !6 BB words per channel
      integer sp_index(50)  !10 space words per channel
      data bb_index /1,2,3,4,5, 11,12,13,14,15, 21,22,23,24,25,
     &  31,32,33,34,35, 41,42,43,44,45, 51,52,53,54,55/
      data sp_index /1,2,3,4,5, 11,12,13,14,15, 21,22,23,24,25,
     &  31,32,33,34,35, 41,42,43,44,45, 51,52,53,54,55,  
     &  61,62,63,64,65, 71,72,73,74,75, 81,82,83,84,85, 
     &  91,92,93,94,95/
     
      integer hrpmf,tipmf
      
! The following variables are used if the data are in unpacked format

      logical unpacked      !If input file is already in unpacked format
      data unpacked/.false./

      integer*2 sync1swap
      data sync1swap/Z'8402'/     !normal sync is Z'0284'
      integer*2 tenbitswap
      data tenbitswap/Z'FF03'/    !byte swapped bit mask

      character*1 hrptabc(bytes_in_unpacked),ctmp
      equivalence (hrptabc(1),y(1))

!-----------------End of declarations-------------------------------------------

!Initialise

      hrpmf = 1       !1 to 3
      tipmf = 0       !0 to 319

      do i=1,nfours
        jj(i) = (i-1)*4 + 1      !Index to unpacked words
        kk(i) = (i-1)*5 + 1      !Index to packed words
      enddo

!Read the file names and open them

      read(*,'(A)') name_in
      read(*,'(A)') name_out
      read(*,*) daynum
      read(*,*,err=100) i
      if (i.ne.0) unpacked = .true.

  100 if (unpacked) then
        open(10,file=name_in,status='old',access='direct',
     &    recl=bytes_in_unpacked/rldiv)
      else
        open(10,file=name_in,status='old',access='direct',
     &    recl=bytes_in/rldiv)
      endif

      open(11,file=name_out,status='new',access='direct',
     &  recl=bytes_out/rldiv,iostat=ios)
      if (ios.ne.0) then
        open(11,file=name_out,status='replace',access='direct',
     &    recl=bytes_out/rldiv)
      endif
      
!Read each frame

      ios=0
      irec=1
      do while (ios .eq. 0)
        if (unpacked) then
          read(10,rec=irec,iostat=ios) y
          if (IAND(y(1),tenbitswap).eq.sync1swap) then  !byte swap if needed
            do k=1,bytes_in_unpacked,2
              ctmp=hrptabc(k)
              hrptabc(k)=hrptabc(k+1)                
              hrptabc(k+1)=ctmp                
            enddo
          endif
        else
          read(10,rec=irec,iostat=ios) b
        endif
        
        if (ios.eq.0) then

!Unpack the 10-bit words into 16 bit integers (if not already unpacked)

          if (.not. unpacked) then
            do i=1,nfours
              do j=1,5
                ib(j) = ICHAR(b(kk(i)+j-1))     !convert byte to integer
              enddo
              y(jj(i)) = ib(1)*4 + ib(2)/64
              y(jj(i)+1) = mod(ib(2),64)*16 + ib(3)/16
              y(jj(i)+2) = mod(ib(3),16)*64 + ib(4)/4
              y(jj(i)+3) = mod(ib(4),4)*256 + ib(5)
            enddo
          endif

!Fill the NOAA HRPT frame

          do i=1,6
            noaa(i) = y(i)    !Frame sync
          enddo
          
  !bit numbering is 1 to 10 where 1 is MS
  !Word 7:
  !  Bit 2-3: hrpmf
  !  Bit 4-7: satid
  !  Bit 10: 1 = AVHRR 3b

          noaa(7) = 128*hrpmf + satid*8 + 1  !ID
          noaa(8) = 0                        !spare (undefined)
          
          noaa(9) = daynum*2     !bits 1-9 day number (missing from CHRPT)
          do i=10,12
            noaa(i) = y(i)       !Date/time
          enddo
          
          millisecs = IAND(noaa(10),i2word127)*1024*1024 + 
     &         noaa(11)*1024 + noaa(12)

          do i=1,4
            noaa(12+i) = y(28+i)  !put target temperatures in ramp cal slots
          enddo
          do i=1,30
            noaa(22+i) = y(32+bb_index(i))  !ref black body in back scan slots
          enddo
          do i=1,50
            noaa(52+i) = y(92+sp_index(i))  !space data
          enddo

          noaa(103) = 0               !Sync Delta not used
          noaa(104) = tipword(0)      !Dummy TIP data
          do i=105,623
            noaa(i) = noaa(104)
          enddo
          
!Insert necessary parts of the TIP frames

          if (hrpmf .eq. 1) then
            do i=0,4         !5 TIP minor frames
              noaa(tipstart + i*104) = tipword(237)              !sync word
              noaa(tipstart+1 + i*104) = tipword(226)
              noaa(tipstart+2 + i*104) = tipword(satid)          !satid
              noaa(tipstart+4 + i*104) = tipword(tipmf/256)      !minor frame counter MS bit
              noaa(tipstart+5 + i*104) = tipword(mod(tipmf,256)) !minor frame counter 8 bits

              if (tipmf .eq. 0) then
                noaa(tipstart+8 + i*104) = tipword(noaa(9)/4)      !date and time in frame 0
                noaa(tipstart+9 + i*104) =tipword(mod((noaa(9)/2),2)*128
     &             + 5*8 + millisecs/(65536*256))
                noaa(tipstart+10 + i*104) = 
     &             tipword(IAND(millisecs,(255*65536))/65536)
                noaa(tipstart+11 + i*104) = 
     &             tipword(IAND(millisecs,(255*256))/256)
                noaa(tipstart+12 + i*104) = tipword(IAND(millisecs,255))
              endif
              tipmf = MOD((tipmf + 1),320)       !0 to 319
            enddo
          endif           !hrpmf .eq. 1

          do i=0,2047
            j = 751 + i*5
            k = 1601 + i*10
            noaa(j) = y(k)      !5 channels per spot
            noaa(j+1) = y(k+1)
            noaa(j+2) = y(k+2)
            noaa(j+3) = y(k+3)
            noaa(j+4) = y(k+4)
          enddo

          do i=1,100
            noaa(10990+i) = y(22080+i)     !Auxiliary sync
          enddo
          
          hrpmf = mod(hrpmf,3) + 1     !1 to 3

!Write out the data

          write(11,rec=irec) noaa
      
          irec = irec + 1
        endif           !ios .eq. 0
      enddo
      
      close(10)
      close(11)
      END
      
!--------------------------------------------------------------

      FUNCTION get_parity(I)
      INTEGER get_parity,I
      INTEGER J, bitval, bitsum
      bitval = 1
      bitsum = 0
      DO J=1,8
        bitsum = bitsum + IAND(I,bitval)/bitval
        bitval = bitval*2
      ENDDO
      get_parity = MOD(bitsum,2)
      RETURN
      END
      
!--------------------------------------------------------------

      FUNCTION tipword(I)
      INTEGER tipword,I
      INTEGER get_parity
      tipword = I*4 + get_parity(I)*2 + 1 - IAND(I,128)/128
      RETURN
      END
