!C LIBS = 
      PROGRAM unpack_noaa_hrpt
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: unpack_noaa_hrpt.F 211 2010-04-26 12:34:57Z frna $
!
! Description:
! Unpack NOAA HRPT data into 16-bit words. Assumes the reception station has
! already placed the sync sequence at the start of each record.
!
! Inputs (read from standard input): 
!    name of input file, name of output file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2009, EUMETSAT, All Rights Reserved.
!
! History:
! 1.0  11/09/2009  Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! References:
! NOAA:  http://www2.ncdc.noaa.gov/docs/klm/html/c4/sec4-1.htm
!
!-----------------------------------------------------------------------------------------------------------------| 

!Parameters

!Note that bytes_in and words_out may need to be customised to suit the
!reception station. HRPT minor frame length is 11090 10-bit words = 13862.5
!bytes. At the Met Office the minor frames are output in pairs and padded at
!the end to give records of 27740 bytes in, 22180 words out.

      integer bytes_in
      parameter (bytes_in=27740)  !packed bytes input per record
      integer words_out
      parameter (words_out=22180)  !unpacked words output per record
      integer nfours
      parameter (nfours=words_out/4) !number of groups of 4 input words
      
      integer rldiv       !Record length dividor, compiler-dependent
#ifdef AAPP_RLDIV
      parameter ( rldiv = AAPP_RLDIV )
#else
      parameter ( rldiv = 1 )
#endif

!Variables

      character*100 name_in
      character*100 name_out

      character*1 b(bytes_in)     !input bytes
      integer*2 y(words_out)      !input 10-bit words

      integer ios,irec,i,j,ib(5)
      integer jj(nfours),kk(nfours)
      
!-----------------End of declarations-------------------------------------------

!Initialise

      do i=1,nfours
        jj(i) = (i-1)*4 + 1      !Index to unpacked words
        kk(i) = (i-1)*5 + 1      !Index to packed words
      enddo

!Read the file names and open them

      read(*,'(A)') name_in
      read(*,'(A)') name_out

      open(10,file=name_in,status='old',access='direct',
     &    recl=bytes_in/rldiv)

      open(11,file=name_out,status='new',access='direct',
     &  recl=(words_out*2)/rldiv,iostat=ios)
      if (ios.ne.0) then
        open(11,file=name_out,status='replace',access='direct',
     &    recl=(words_out*2)/rldiv)
      endif
      
!Read each frame

      ios=0
      irec=1
      do while (ios .eq. 0)
  
        read(10,rec=irec,iostat=ios) b
        
        if (ios.eq.0) then

!Unpack the 10-bit words into 16 bit integers

          do i=1,nfours
            do j=1,5
              ib(j) = ICHAR(b(kk(i)+j-1))     !convert byte to integer
            enddo
            y(jj(i)) = ib(1)*4 + ib(2)/64
            y(jj(i)+1) = mod(ib(2),64)*16 + ib(3)/16
            y(jj(i)+2) = mod(ib(3),16)*64 + ib(4)/4
            y(jj(i)+3) = mod(ib(4),4)*256 + ib(5)
          enddo

!Write out the data

          IF (y(1) .NE. 644 .AND. y(1) .NE. 33794) THEN
            write(*,*)'Sync error at record',irec,':',y(1)
          ENDIF
          write(11,rec=irec) y
      
          irec = irec + 1
        endif           !ios .eq. 0
      enddo
      
      close(10)
      close(11)
      END
