C!
C! ROUTINE          AMSUA_LINECHECK   SUBROUTINE  FORT77
C!
C! PURPOSE          To recover data on AMSU-A scanline when gaps or parity errors
C!                  have been recorded during reading the line.
C!
C! DESCRIPTION      AMSU-A data in TIP-frames has a curious feature:
C!                   "...shall contain an undetermined number of 'fill words'
C!                    These fill words shall be 0001H and will be intermingled
C!                    with valid data."
C!                  This means that data words must be picked from the stream rejecting 
C!                  all words with value WORD=1.  But the consequence is that when there 
C!                  is just one parity error word, we don't know if it was data word or
C!                  fill word, and then for the rest of the line we don't know for sure
C!                  what the data is (which ifov, which channel) !!!  Great...
C!                  The same is true ofcourse if there is a missing hrpt-line during
C!                  AMSU-A scanline data stream.
C!
C!                  Easy solution to the problem is to discard the whole line, if
C!                  there has been any of the two problems.  The user can do this
C!                  when reading the result files by taking notice of the quality 
C!                  indicator  word for each line.  The first bit of the word will
C!                  be set in each line which had parity errors or gaps.
C!                  However, this means throwing away a good part of the data,
C!                  especially having the transmission problems with NOAA-15.
C!                  
C!                  The other solution is to recover the data in the line with this
C!                  subroutine.
C!                  This routine is called by AMSGET.F after collecting all data
C!                  (AMSU-A1 or AMSU-A2) for one scan line into array amsuin.  
C!                  Flag PROBLEMS tells if there are any problem words in this line,
C!                  and those words are  marked as follows
C!                      amsuin(i) = 0    Parity error word was here 
C!                      amsuin(i) = 1    Gap (i.e. several lost words) was here 
C!                  The task is now to make sense of the AMSU-A data in this line.
C!                  Method is based on the fact that the first word in each 30 
C!                  IFOV groups is the antenna position, which is known with the
C!                  accuracy of about 0.2 degrees.  It is then in most cases possible 
C!                  to decide where the data words belong to, and especially if that is
C!                  not possible to know.  In that case those words are set to zero, so
C!                  that affected scan line may have just a few "holes" where decision
C!                  was not possible, instead of having to reject the whole line.
C!                  This makes essentially all lines usable, regardless of parity errors
C!                  or gaps.  The user may still be somewhat more cautious with these
C!                  recovered lines, as there can always be some "pathological cases".
C!                  But this is true also for problem-free scan lines. There may
C!                  have been two bit errors in single word, so that parity error
C!                  is not set, for example. Recovered scan lines (should) be now
C!                  almost as reliable as problem-free scan lines.
C!
C! VERSION          1.00   061299  A.V.KORPELA
C!                  1.01   June 02 T.Labrot and P.Brunel
C!  f90 test : I.and.J not correct and modified by IAND(I,J)
C!             Cut lines exceeding 72 characters.
C!             Add working variable for IAND tests
C!             Modif. type of variables. 
C!               Exple: integer*4 Async1 --> integer*2
C!             Changed type XXX/yyy/
C!                  to type XXX
C!                     parameter(XXX=yyy)   or data XXX/yyy/
C!                  1.02   09/03/04 Modify for linux   N Atkinson / A Korpela
C!                  1.03   17/01/2006 : Philippe Marguinaud : change hex constants from
C!                                      '1234'X to Z'1234'
C!
C! CHANGES          00 None
C!
C!====================================================================================

      subroutine amsua_linecheck(amsuinst,Problems,amsuin,amsuout,Liw,
     & Dbg)

      IMPLICIT none

      CHARACTER*(*) amsuinst            ! AMSU-A instrument 'AMSUA1' or 'AMSUA2'
      LOGICAL*1     Problems            ! Flag for parity errors or gaps during scanline
      INTEGER*2     amsuin(*)           ! AMSU-A  input data for one scan line
      INTEGER*2     amsuout(*)          ! AMSU-A output data for one scan line
      INTEGER*4     Liw                 ! Last input word - number of input AMSU data words
      LOGICAL*1     Dbg                 ! Flag for printing details of the line analysis

      INTEGER*4    Maxiw
      PARAMETER    (Maxiw=1040)

      LOGICAL*4    First                ! Logical flag for first subroutine call
      DATA First/.TRUE./


      REAL*4       Ant_coef_a11,Ant_off_a11 ! Coefficients for antenna positions
      REAL*4       Ant_coef_a12,Ant_off_a12
      REAL*4       Ant_coef_a2,Ant_off_a2
      REAL*4       chsum(15),chsamples(15)
      DATA         Ant_coef_a11/0/
      DATA         Ant_off_a11/0/
      DATA         Ant_coef_a12/0/
      DATA         Ant_off_a12/0/
      DATA         Ant_coef_a2/0/
      DATA         Ant_off_a2/0/
      DATA         chsum/15*0.0/
      DATA         chsamples/15*0.0/
      
      
      
      INTEGER*2    Gap
      PARAMETER(Gap=1)
      
      INTEGER*2    Sync1,Sync2,Sync3,Sync4
      INTEGER*4    Ama_coefu
      PARAMETER (Ama_coefu=50)
     
      INTEGER*2    Async1,Async2    !Sync words at start of frame
      PARAMETER (Async1=Z'FFFF')
      PARAMETER (Async2=Z'FF00')
      INTEGER*4    Eoaf
      INTEGER*4    Eoal
      INTEGER*4    Nwords
      INTEGER*4    Gsize
      INTEGER*4    Gants
      INTEGER*4    Gcnts
      INTEGER*4    Gcals
      INTEGER*4    Ach1,Ach2
      INTEGER*4    Cntmin,Cntmax
      INTEGER*4    Calmin,Calmax
      REAL*4       Ant_off,Ant_coef

      LOGICAL*1    fovfound(31)
      INTEGER*4    fovstart(31),fovend(31)
      REAL*4       nompos(30)
      INTEGER*4    startsfrom(Maxiw)
      LOGICAL*1    amsudel(Maxiw)
      LOGICAL*1    looksa(Maxiw),looksc(Maxiw)
      REAL*4       apos(Maxiw),vapos(Maxiw)
      INTEGER*2    vamsu(Maxiw)

      INTEGER*4    iw,ow,word,afov,afov30,ncal,ii,nant,nant2,ncnt,ic,ia
      INTEGER*4    istart,astart1,astart2,iend,cstart1,cstart2,ach,
     &               nxtcount
      INTEGER*4    sw1,sw2,sw3,lastsync
      LOGICAL*4    issync,isantenna,iscount,iscalib
      LOGICAL*4    looks_count,looks_antenna,looks_calib
      LOGICAL*4    skipall,leaveall,delete
      LOGICAL*4    found,continue,ok
      INTEGER*4    dels
      REAL*4       pos,avg,rat
      CHARACTER*80 txt
      INTEGER*2 I2WORD  ! working variable for IAND/IOR tests
      INTEGER      errStatus
!
! Read in the coefficients for antenna positions
!
      IF (First) THEN
         First=.FALSE.
         OPEN(Ama_coefu,ACCESS='SEQUENTIAL',STATUS='OLD',
     &      IOSTAT=errStatus)

!        If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN(Ama_coefu,ERR=900,ACCESS='SEQUENTIAL',STATUS='UNKNOWN')

         DO WHILE (txt(1:69) .NE. 
     & '# Slope and offset for counts to antenna posn '//
     & 'in deg, A1-1 and A1-2 #')
            READ(Ama_coefu,'(A80)',ERR=901) txt
            ENDDO
         READ(Ama_coefu,*,ERR=901) Ant_coef_a11,Ant_off_a11,
     &        Ant_coef_a12,Ant_off_a12
         DO WHILE (txT(1:54) .NE. 
     & '# Slope and offset for counts to antenna posn in deg #')
            READ(Ama_coefu,'(A80)',ERR=901) txt
            ENDDO
         READ(Ama_coefu,*,ERR=901) Ant_coef_a2,Ant_off_a2
         CLOSE(Ama_coefu)
         ENDIF
!
! Initial settings
!
      IF (amsuinst.EQ.'AMSUA1') THEN
         Nwords = 13
         Gants = 4
         Gcnts = 13
         Ach1 = 3
         Ach2 = 15
         Eoaf = 514
         Eoal = 620
         Ant_off = Ant_off_a11
         Ant_coef = Ant_coef_a11
         Sync1 = -1
         Sync2 = -247
         Sync3 = 526
         Sync4 = 0
         ENDIF
      IF (amsuinst.EQ.'AMSUA2') THEN
         Nwords = 7
         Gants = 2
         Gcnts = 2
         Ach1 = 1
         Ach2 = 2
         Eoaf = 124
         Eoal = 156
         Ant_off = Ant_off_a2
         Ant_coef = Ant_coef_a2
         Sync1 = -1
         Sync2 = -250
         Sync3 = 518
         Sync4 = 0
         ENDIF
      Gsize=Gants+Gcnts
      Gcals=Eoal-Eoaf
      Cntmin = 10000
      Cntmax = 20000
      Calmin = 10000
      Calmax = 33000

      IF (.NOT.Problems) THEN   ! Simply copy input to output and return
         DO ow=1,Eoal
            amsuout(ow)=amsuin(ow)
            ENDDO
         RETURN
         ENDIF

      DO ow=1,Eoal              ! Make AMSU return array empty - before filling
         amsuout(ow)=0
         ENDDO

      IF (Liw.EQ.0) RETURN      ! Nothing to do

      DO iw=1,Maxiw     ! Compute apparent antenna position for each input word
         word=MOD(amsuin(iw)+65536,65536)/4
         pos = Ant_off + Ant_coef*word
         pos = AMOD(pos+360+135, 360.0)-135.0
         apos(iw)=pos
         IF (word.EQ.0 .OR. iw.GT.Liw) apos(iw)=9999.   ! Gaps and parity errors and unused buffer
         amsudel(iw)=.FALSE.
         ENDDO

      DO iw=1,Maxiw     ! FOV starting points
         startsfrom(iw)=0
         ENDDO

      DO afov=1,30      ! Compute nominal antenna positions 
         NOMPOS(afov) = 48.333 - (afov-1)*3.333333
         ENDDO
!
!  Find Sync 
!
      sw1=0
      sw2=0
      sw3=0
      lastsync=0
      DO WHILE (lastsync.LT.Maxiw-1 .AND. amsuin(lastsync+1).EQ.Gap) 
         lastsync=lastsync+1
         ENDDO
      DO iw=1,Liw   
         IF (sw1*sw2.NE.0 .AND. sw3.EQ.0 .AND. amsuin(iw).EQ.Sync3) 
     &         sw3=iw
         I2WORD = IAND(amsuin(iw),Async2)
         IF (sw1.NE.0 .AND. sw2.EQ.0 .AND. I2WORD.EQ.Async2) sw2=iw
         I2WORD = IAND(amsuin(iw),Async1)
         IF (sw1.EQ.0 .AND. I2WORD.EQ.Async1) sw1=iw
         ENDDO
      IF (sw1*sw2.NE.0 .AND. sw2.GT.sw1) THEN
         found=.TRUE.
         DO iw=sw1+1,sw2-1
            IF (amsuin(iw).NE.0) found=.FALSE.
            ENDDO
         IF (found) lastsync=sw2
         DO iw=sw2+1,sw3-1
            IF (amsuin(iw).NE.0) found=.FALSE.
            ENDDO
         IF (found) lastsync=sw3
         IF (lastsync.EQ.sw3 .AND. amsuin(lastsync+1).EQ.0)
     &        lastsync=sw3+1
         ENDIF
!
! We simply set the sync, however
!
      amsuout(1)=Sync1
      amsuout(2)=Sync2
      amsuout(3)=Sync3
      amsuout(4)=Sync4
!
!  Find FOV starting points
!
      afov30=0
      iend=Liw
      ncal=0
      DO WHILE (iend.GT.1 .AND. ncal.LT.Gcals)
         IF (amsuin(iend).EQ.Gap) THEN
            ncal=ncal+Nwords
         ELSE
            ncal=ncal+1
            ENDIF
         iend=iend-1
         ENDDO
      DO afov=30,1,-1
         nant=0
         iw=iend
         nxtcount=iw+1
         found=.FALSE.
         DO WHILE (.NOT.found .AND. iw.GT.0)
            word=MOD(amsuin(iw)+65536,65536)/2
            looks_antenna=(ABS(apos(iw)-nompos(afov)).LT.7)
     &    .AND.amsuin(iw).NE.0
            looks_count=.NOT.looks_antenna
     &   .AND.(Cntmin.LT.word.AND.word.LT.Cntmax)
            IF (looks_count) nxtcount=iw
            IF (looks_antenna) nant=nant+1
            IF (looks_count .OR. amsuin(iw).EQ.Gap) nant=0
            IF (amsuin(iw).EQ.Gap) nxtcount=Liw
            found=(ABS(apos(iw)-nompos(afov)).LT.0.5)
     &   .AND.(iend-iw.GE.Gcnts.OR.nxtcount-iw.GE.Gcnts)
            IF (found) THEN
               found=.FALSE.            ! Verification required
               istart=iw
               ii=iw
               nant2=nant
               continue=.TRUE.
               DO WHILE (continue .AND. ii.GT.1) 
                  ii=ii-1
                  continue=(ABS(apos(ii)-nompos(afov)).LT.0.5)
                  IF (continue) istart=ii
                  IF (continue) nant2=nant2+1
                  continue=continue.OR.amsuin(ii).EQ.0
                  ENDDO
               IF (istart.GT.1) THEN
                  word=MOD(amsuin(istart-1)+65536,65536)/2
                  looks_count=(Cntmin.LT.word.AND.word.LT.Cntmax)
                  IF (afov.GT.1) found=looks_count
                  IF (afov.EQ.1) found=(istart-1.EQ.lastsync
     &    .AND. amsuin(istart-1).EQ.0)
                  ENDIF
               found=(found).AND.(nxtcount-istart.GE.Gants)
               IF (.NOT.found .AND. nant2.EQ.Gants) found=.TRUE.
               IF (found) THEN
                  if (Dbg) write(6,*) 'STARTFOUND',afov
                  IF (afov.EQ.30) afov30=istart
                  startsfrom(istart)=afov
                  iend=istart-1
                  ENDIF
               ENDIF
            iw=iw-1
            ENDDO
         ENDDO
!
!  Verify calibration
!
      found=.FALSE.
      IF (afov30.GT.0) THEN
         istart=afov30+Gants
         iw=afov30
         nant=0
         continue=.TRUE.
         DO WHILE(continue)
            word=MOD(amsuin(iw)+65536,65536)/2
            looks_antenna=(ABS(apos(iw)-nompos(30)).LT.7)
            looks_count=(.NOT.looks_antenna 
     & .AND. (Cntmin.LT.word.AND.word.LT.Cntmax))
            IF (looks_antenna) THEN
               nant=nant+1
               IF (iw+1.GT.istart) istart=iw+1
               ENDIF
            IF (looks_count .OR. iw.EQ.Liw .OR. amsuin(iw).EQ.Gap) THEN
               continue=.FALSE.
               found=(iw.EQ.istart)
            ELSE
               iw=iw+1
               continue=(nant.LT.Gants)
               found=.NOT.continue
               ENDIF
            ENDDO
         ENDIF
      IF (found) THEN
         dels=0
         ok=.TRUE.
         DO iw=istart,Liw
            word=MOD(amsuin(iw)+65536,65536)/2
            looks_calib=(Calmin.LT.word.AND.word.LT.Calmax)
            IF (.NOT.looks_calib) dels=dels+1
            ok=(ok.AND.(amsuin(iw).NE.Gap))
            ENDDO
         leaveall=(ok.AND.(1+Liw-istart.EQ.Gcals+Gcnts))        ! All dels are parity errors in actual values
         skipall =(ok.AND.(1+Liw-istart-dels.EQ.Gcals+Gcnts))   ! All dels are additional noise
         IF (leaveall) THEN                             ! Copy all calibrations
            ow=Eoaf-Gcnts
            DO iw=istart,Liw
               ow=ow+1
               IF (ow.GT.Eoaf) THEN
                  amsuout(ow)=amsuin(iw)
                  IF (ow.EQ.Eoaf+1) startsfrom(iw)=31
                  ENDIF
               ENDDO
            ENDIF
         IF (skipall) THEN                              ! Copy all calibrations, skip wrong values
            ow=Eoaf-Gcnts
            DO iw=istart,Liw
               word=MOD(amsuin(iw)+65536,65536)/2 
               looks_calib=(Calmin.LT.word.AND.word.LT.Calmax)
               IF (looks_calib) ow=ow+1
               IF (looks_calib.AND.ow.GT.Eoaf) THEN
                  amsuout(ow)=amsuin(iw)
                  IF (ow.EQ.Eoaf+1) startsfrom(iw)=31
                  ENDIF
               ENDDO
            ENDIF
         IF (.NOT.(leaveall.OR.skipall)) THEN           ! Copy known calibrations
            ow=Eoaf-Gcnts
            looks_calib=.TRUE.
            DO iw=istart,Liw
               word=MOD(amsuin(iw)+65536,65536)/2 
               looks_calib = looks_calib
     &   .AND.(Calmin.LT.word.AND.word.LT.Calmax)
               IF (looks_calib) ow=ow+1
               IF (looks_calib.AND.ow.EQ.Eoaf) startsfrom(iw+1)=31
               IF (looks_calib.AND.ow.GT.Eoaf) amsuout(ow)=amsuin(iw)
               ENDDO
            ow=Eoal+1
            looks_calib=.TRUE.
            DO iw=Liw,istart,-1
               word=MOD(amsuin(iw)+65536,65536)/2
               looks_calib = looks_calib
     &   .AND.(Calmin.LT.word.AND.word.LT.Calmax)
               IF (looks_calib) ow=ow-1
               IF (looks_calib.AND.ow.GT.Eoaf) THEN
                  amsuout(ow)=amsuin(iw)
                  ENDIF
               ENDDO
            ENDIF
         ENDIF
!
! Find FOV groups
!
      DO afov=1,31
         fovfound(afov)=.FALSE.
         fovstart(afov)=0
         fovend(afov)=0
         ENDDO

      DO afov=30,1,-1

         IF (.NOT.fovfound(afov)) THEN          ! Can we find current and next starting points ?
            istart=0
            iend=0
            DO ii=Liw,1,-1
               IF (startsfrom(ii).EQ.afov) istart=ii
               IF (startsfrom(ii).EQ.afov+1) iend=ii-1
               IF (istart.EQ.0 .AND. iend.NE.0 .AND. amsuin(ii).EQ.Gap)
     &               iend=ii
               ENDDO
            IF (istart*iend.NE.0) THEN
               fovfound(afov)=.TRUE.
               if (Dbg) write(6,*) 'FOUND start-end',afov,istart,iend
               fovstart(afov)=istart
               fovend(afov)=iend
               ENDIF
            ENDIF

         iend=0                                 ! Can we go backwards from the next starting point ?
         DO ii=Liw,1,-1
            IF (startsfrom(ii).EQ.afov+1) iend=ii-1
            ENDDO
         IF (iend.GT.0 .AND. .NOT.fovfound(afov)) THEN
            if (Dbg) write(6,*) 'FINDSTART',afov
            astart1=lastsync+1          ! Earliest possible start position
            DO ii=1,iend
               IF (startsfrom(ii).NE.0) astart1=ii+1
               ENDDO
            iw=astart1
            DO ii=iw,iend
               IF (amsuin(ii).EQ.Gap) astart1=ii+1
               ENDDO
            IF (Dbg) THEN
               DO ii=astart1,iend
                  word=MOD(amsuin(ii)+65536,65536)/2
                  IF (startsfrom(ii).NE.0) 
     &                 write(6,*) ii,word,startsfrom(ii)
                  IF (startsfrom(ii).EQ.0) write(6,*) ii,word
                  ENDDO
               ENDIF
            ncnt=0
            nant=0
            iw=iend+1
            astart2=iw-Gsize            ! Latest possible start position
            ia=iw-Gsize                 ! Antenna values at IA and later belong to this group for sure
            ic=iw-Gcnts                 ! Counts  values at IC and later belong to this group for sure
            cstart1=astart1+Gants
            cstart2=astart2+Gants
            continue=(astart1.LE.astart2)
            IF (continue) THEN
               DO WHILE(continue)
                  iw=iw-1
                  word=MOD(amsuin(iw)+65536,65536)/2
                  looks_antenna=(ABS(apos(iw)-nompos(afov)).LT.7)
                  looks_count=(.NOT.looks_antenna) 
     &               .AND. (Cntmin.LT.word.AND.word.LT.Cntmax)
                  IF (looks_count) THEN
                     ncnt=ncnt+1
                     IF (nant.GT.0.OR.ncnt.GT.Gcnts) THEN       ! Impossible to have count
                        iw=iw+1
                        ncnt=ncnt-1
                        astart1=iw
                        IF (astart1+Gants.GT.cstart1) 
     &                cstart1=astart1+Gants
                        continue=.FALSE.
                     ELSEIF (iw.LT.ic) THEN     ! Possible count but no decision possible
                        astart1=0
                        continue=.FALSE.
                     ELSE
                        IF (iw.LT.cstart2) cstart2=iw
                        IF (ncnt.EQ.Gcnts) cstart1=iw
                        IF (iw-Gants.LT.astart2) astart2=iw-Gants
                        IF (iw-Gants.LT.ic) ic=iw-Gants
                        IF (iw-Gsize.LT.ia) ia=iw-Gsize
                        ENDIF
                     ENDIF
                  IF (looks_antenna) THEN
                     nant=nant+1
                     IF (iw+1.GT.cstart1) cstart1=iw+1
                     IF (nant.EQ.Gants .AND. iw.LE.astart2) THEN        ! Found the start
                        astart1=iw
                        astart2=iw
                        continue=.FALSE.
                     ELSEIF (iw.LT.ia) THEN                             ! No decision possible
                        astart1=0
                        continue=.FALSE.
                     ELSE
                        IF (iw.LT.astart2) astart2=iw
                        ENDIF
                     ENDIF
                  IF (iw.LE.astart1) continue=.FALSE.
                  ENDDO         ! continue
               IF (Dbg) write(6,*) 'GROUP  start',astart1,astart2
               IF (Dbg) write(6,*) 'COUNTS start',cstart1,cstart2
               IF (astart1.NE.0 .AND. astart1.LT.astart2) THEN
                  IF (cstart1.LT.cstart2) THEN
                     looks_count=.TRUE.
                     DO ii=iend,cstart1,-1
                        word=MOD(amsuin(ii)+65536,65536)/2
                        looks_count=(looks_count 
     &                    .AND. (Cntmin.LT.word.AND.word.LT.Cntmax))
                        IF (.NOT.looks_count) THEN
                           amsuin(ii)=0
                           apos(ii)=9999.
                           amsudel(ii)=.TRUE.
                           ENDIF
                        ENDDO
                     ENDIF
                  DO ii=cstart2-1,astart1,-1
                     amsuin(ii)=0
                     apos(ii)=9999.
                     amsudel(ii)=.TRUE.
                     ENDDO
                  astart2=cstart2-Gants
                  IF (Dbg) write(6,*) 
     &                  'FORCED start from',astart1,' to',astart2
                  astart1=astart2
                  ENDIF
               IF (Dbg .AND. astart1.NE.astart2) 
     &                write(6,*) 'GIVEUP',afov
               IF (astart1.EQ.astart2) THEN
                  startsfrom(astart1)=afov
                  fovfound(afov)=.TRUE.
                  fovstart(afov)=astart1
                  fovend(afov)=iend
                  IF (Dbg) write(6,*) 'SUCCESS bwds',afov,astart1,iend
                  ENDIF
               ENDIF    ! continue
            ENDIF

         ENDDO  ! afov
!
!  Verify and copy the found ifov groups
!
      DO afov=1,30
         IF (Dbg .AND. .NOT.fovfound(afov)) 
     &             write(6,*) 'SEARCH FAILED',afov
         IF (fovfound(afov)) THEN
            istart=fovstart(afov)
            iend=fovend(afov)
            iw=0
            IF (amsuin(istart).EQ.Gap) THEN
               DO ii=1,Gsize
                  iw=iw+1
                  vamsu(iw)=0
                  vapos(iw)=9999.
                  ENDDO
               istart=istart+1
               ENDIF
            DO ii=istart,iend
               iw=iw+1
               vamsu(iw)=amsuin(ii)
               vapos(iw)=apos(ii)
               ENDDO
            IF (amsuin(iend).EQ.Gap) THEN
               iw=iw-1
               DO ii=1,Gsize
                  iw=iw+1
                  vamsu(iw)=0
                  vapos(iw)=9999.
                  ENDDO
               ENDIF
            istart=1
            iend=iw
            nant=0
            ncnt=0
            DO iw=istart,iend
               word=MOD(vamsu(iw)+65536,65536)/2
               looksa(iw)=(ABS(vapos(iw)-nompos(afov)).LT.7)
               looksc(iw)=.NOT.looksa(iw)
     &                .AND. (Cntmin.LT.word.AND.word.LT.Cntmax)
               IF (looksa(iw)) nant=nant+1
               IF (looksc(iw)) ncnt=ncnt+1
               ENDDO
            IF (1+iend-istart.EQ.Gsize) THEN
               ow=5+(afov-1)*Gsize
               DO iw=istart,iend
                  amsuout(ow+(iw-istart))=vamsu(iw)
                  ENDDO
            ELSE
               ia=istart
               ic=iend
               DO ii=istart+1,iend
                  IF (looksa(ii)) ia=ii
                  ENDDO
               DO ii=iend-1,istart,-1
                  IF (looksc(ii)) ic=ii
                  ENDDO
               IF (1+iend-istart.GT.Gsize .AND. ic.GT.ia 
     & .AND. ic.GT.istart+1 .AND. ia.LT.iend-1 .AND. 
     &             nant.LE.Gants .AND. ncnt.LE.Gcnts) THEN
                  continue=.TRUE.
                  ow=4+(afov-1)*Gsize
                  DO iw=istart,ia
                     continue = continue .AND. 
     &                  (looksa(iw) .OR. nant.EQ.Gants)
                     IF (continue .AND. looksa(iw)) THEN
                        ow=ow+1
                        amsuout(ow)=vamsu(iw)
                        ENDIF
                     ENDDO
                  continue=.TRUE.
                  ow=5+afov*Gsize
                  DO iw=iend,ic,-1
                     continue = continue .AND. 
     &                 (looksc(iw) .OR. ncnt.EQ.Gcnts)
                     IF (continue .AND. looksc(iw)) THEN
                        ow=ow-1
                        amsuout(ow)=vamsu(iw)
                        ENDIF
                     ENDDO
                  IF (ic.EQ.ia+1) THEN
                     continue=.TRUE.
                     ow=5+Gants+(afov-1)*Gsize
                     DO iw=ia,istart,-1
                        continue = continue .AND. 
     &                   (looksa(iw) .OR. nant.EQ.Gants)
                        IF (continue .AND. looksa(iw)) THEN
                           ow=ow-1
                           amsuout(ow)=vamsu(iw)
                           ENDIF
                        ENDDO
                     continue=.TRUE.
                     ow=4+Gants+(afov-1)*Gsize
                     DO iw=ic,iend
                        continue = continue .AND. 
     &                   (looksc(iw) .OR. ncnt.EQ.Gcnts)
                        IF (continue .AND. looksc(iw)) THEN
                           ow=ow+1
                           amsuout(ow)=vamsu(iw)
                           ENDIF
                        ENDDO
                     ENDIF
               ELSE
                  IF (Dbg) write(6,*) 'INVALID group',afov,istart,iend
                  fovfound(afov)=.FALSE.
                  fovstart(afov)=0
                  fovend(afov)=0
                  ENDIF
               ENDIF    ! interval istart-iend alternatives
            ENDIF       ! IF (fovfound ...
         ENDDO          ! DO afov
!
! Finally, the average test
!
      DO afov=1,30      
      IF (fovfound(afov)) THEN
         ow=4+Gants+(afov-1)*Gsize
         DO ach=Ach1,Ach2
            ow=ow+1
            word=MOD(amsuout(ow)+65536,65536)/2
            IF (word.NE.0) THEN
               chsum(ach)=chsum(ach)+word
               chsamples(ach)=chsamples(ach)+1
               ENDIF
            ENDDO
         ENDIF
         ENDDO

      DO ach=Ach1,Ach2
      IF (chsamples(ach).LT.15) THEN    ! Let's require at least half a scanline to begin with each channel
         ow=5+Gants+ach-Ach1-Gsize
         DO afov=1,30
            ow=ow+Gsize
            word=MOD(amsuout(ow)+65536,65536)/2
            IF (word.NE.0) THEN
               amsuout(ow)=0
               chsum(ach)=chsum(ach)-word
               chsamples(ach)=chsamples(ach)-1
               ENDIF
            ENDDO
         ENDIF
         ENDDO

      DO afov=1,30
      IF (fovfound(afov)) THEN
         delete=.FALSE.
         ow=4+Gants+(afov-1)*Gsize
         DO ach=Ach1,Ach2
            ow=ow+1
            word=MOD(amsuout(ow)+65536,65536)/2
            IF (word.NE.0 .AND. chsamples(ach).GT.0) THEN
               avg=chsum(ach)/chsamples(ach)
               rat=word/avg
               IF (rat.LT.0.9 .OR. rat.GT.1.1) delete=.TRUE.
               ENDIF
            ENDDO
         IF (delete) THEN
            IF (Dbg) write(6,*) 'DELETING suspicious afov',afov
            ow=4+Gants+(afov-1)*Gsize
            DO ach=Ach1,Ach2
               ow=ow+1
               word=MOD(amsuout(ow)+65536,65536)/2
               IF (word.NE.0) THEN
                  avg=chsum(ach)/chsamples(ach)
                  IF (Dbg) write(6,*) ach,word,avg
                  amsuout(ow)=0
                  chsum(ach)=chsum(ach)-word
                  chsamples(ach)=chsamples(ach)-1
                  ENDIF
               ENDDO
            fovfound(afov)=.FALSE.
            fovstart(afov)=0
            fovend(afov)=0
            ENDIF
         ENDIF
         ENDDO
!
! Debug printing
!
      IF (Dbg) THEN     !  Print input with ifov group starting points
         write(6,*)
         write(6,*) 'AMSU input analysis'       
         DO iw=1,lastsync
            word=MOD(amsuin(iw)+65536,65536)/2
            txt='                              '
            IF (amsuin(iw).EQ.Gap) THEN
               write(txt(2:2),'(A1)') 'G'
            ELSE
               write(txt(6:13),'(I8)') word
               ENDIF
            write(txt(15:15),'(A1)') 's'
            write(6,*) iw,txt(1:20)
            ENDDO
         DO iw=lastsync+1,Liw
            word=MOD(amsuin(iw)+65536,65536)/2
            txt='                              '
            IF (amsuin(iw).EQ.Gap) THEN
               write(txt(2:2),'(A1)') 'G'
            ELSE
               IF (amsudel(iw)) write(txt(2:2),'(A1)') '?'
               write(txt(6:13),'(I8)') word
               ENDIF
            IF (startsfrom(iw).NE.0) THEN
               write(txt(16:17),'(I2)') startsfrom(iw)
               IF (fovfound(startsfrom(iw))) 
     &             write(txt(19:19),'(A1)') '*'
               ENDIF
            write(6,*) iw,txt(1:20)
            ENDDO
         write(6,*)
         write(6,*) 'AMSU output line'
         DO ow=1,Eoal           !  Print final output line
            word=MOD(amsuout(ow)+65536,65536)/2
            afov=1+(ow-5)/Gsize
            issync=(ow.LE.4)
            isantenna=.FALSE.
            iscount=.FALSE.
            iscalib=(ow.GT.Eoaf)
            IF (5.LE.ow.AND.ow.LE.Eoaf) THEN
               isantenna=(MOD(ow-5,Gsize).LT.Gants)
               iscount=(.NOT.isantenna)
               ENDIF
            if (issync) write(6,*) ow,word,' s'
            if (isantenna) write(6,*) ow,word,' A',afov
            if (iscount) write(6,*) ow,word
            if (iscalib) write(6,*) ow,word,' c'
            ENDDO
         ENDIF

      RETURN    

 900  WRITE(6,*)'Error opening cal coefficients file'
      STOP
 901  WRITE(6,*)'Error reading cal coefficients file'
      STOP

      END
