!routine          atovdc   subroutine  fort77
!
!purpose          to decommutate atovs data from tip/amsu minor frames
!
!description      atovdc performs the decommutation task. the tip/amsu minor
!                 frames as received from the noaa-klm spacecraft are
!                 embedded in three consecutive hrpt minor frames. the first
!                 hrpt minor frame contains 5 tip minor frames, the second
!                 hrpt minor frame contains 'backfill' (dummy data?) and
!                 the third hrpt minor frame contains 5 amsu minor frames.
!                 the location of the tip words are defined in the klm
!                 telemetry list document. the words as received from hrptdc
!                 still have 2 parity bits as the lsb which have to be
!                 removed. The module atovdc will extract the hirs/3,
!                 amsu-a, amsu-b and dcs words from the tip/amsu minor
!                 frames and place the extracted words into separate
!                 hirs/3, amsu-a, amsu-b & dcs level 1a files.
!                 atovdc extracts key parameters (e.g. tip time,
!                 tip s/c id, sync bit check, tip status) from the
!                 raw tip/amsu data and performs some quality control.
!                 the extracted hirs/amsu/dcs words are written out to
!                 1a files so that each record contains one scan line of
!                 data/telemetry. Options also exist to read pre-noaa-k tip
!                 data and output hirs-2/msu files.
!
!version          1.02   270996  r.w.saunders
!                 1.03   150196
!                 1.04   150197  added analogue telemetry (N Atkinson)
!                 2.0    030397  m. derrien robustness updates
!                 2.1            Initial release of AAPP
!                 2.2    181298  Increase size of AMSU-B analogs (N Atkinson)
!                 2.3    March 03    T.Labrot
!    f90 test: I.and.J not correct and modified by iand(I,J)
!              I.or.J not correct and modified by ior(I,J)
!              Cut lines exceeding 72 characters.
!              Changed  type XXX/yyy/
!                   to  type XXX
!                       parameter(XXX=yyy)   or data XXX/yyy/
!                 2.4    Feb 04  adjust mseca if call to amsout occurs before
!                                TIP time update. N.C.Atkinson
!                 2.5    Mar 04  Initialise berr. A Korpela
!                 2.6    24-May-2004 More strict AMSU minor frame test. A.Korpela
!                 2.7    June 04  Add MHS. N Atkinson
!                 2.8    July 04  Change statement for test on noaa klm serie
!                                         P. Brunel
!                 2.9    Feb 05  Do not update TIP time until AMSU is OK. NCA
!                 2.10   Oct 06  Introduce mseca_offset, to get correct AMSU
!                                times when data are filled. NCA
!                 2.11   Apr 09  Comment out diagnostic to unit 6 - causes
!                                problems on some systems. NCA
!
!arguments
!      integer yr   -- year of data (e.g. 1997). if year<1978 then yr
!                                                taken from systim.
!	integer optn(12)
!                optn(1) -- 0 or 1 for level of error logging
!                optn(2) -- 0 or 1 for hirs/3 (1 indicates extract data)
!                optn(3) -- 0 or 1 for amsu-a1
!                optn(4) -- 0 or 1 for amsu-a2
!                optn(5) -- 0 or 1 for amsu-b / MHS
!                optn(6) -- 0 or 1 for hirs/2
!                optn(7) -- 0 or 1 for msu
!                optn(8) -- 0 or 1 for dcs
!                optn(9) -- 0 or 1 for sem
!                optn(10)-- 0 or 1 for sbuv
!                optn(11)-- 0 or 1 for sar
!		  optn(12)-- 0 or 1 for avhrr
!     (e.g. for noaa-k configuration array might be: 1,1,1,1,1,0,0,0,0,0,0,1)
!     option numbers are overridden by sat id check (e.g. if sat id < 12
!     then only hirs/2 and msu extracted)
!
!  integer ninst     -- number of tip/amsu instruments to process (without
!                           avhrr)
!  integer opmode    --  0 = off-line, 1= real time
!  integer satid     -- satellite id defined as: tiros-n=  1
!                           (for complete list see     |   |
!                           memo from derrien)       noaa-j = 11
!                                                    noaa-k = 12
!  integer idat      -- extracted and checked hrpt day number
!  integer orbmf     -- orbit mf number i.e. avhrr scan line number
!                           (1=first)
!  integer itime     -- extracted and checked hrpt msecs from midnight
!  integer systim(3) -- current year daynumber, msecs from midnight
!  integer*4 startorb,endorb --  orbit number (start and end)
!  integer hrpmf     -- hrpt minor frame number (1-3)
!  integer*2 hrpaip(104,5) -- current tip/backfill/amsu array
!                                 note this assumes that hrptdc unpacks
!                                 the 10bit hrpt words to 16bits.
!  integer nbmishrp  -- no. of missing HRPT frames between current and last
!
! common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
!               ludcs,lusem,lusbuv,lusar,luavh
! common /scan/ msech,msecm,mseca,hirs,msu,amsua1,amsua2,amsub,mhs
! common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
!
!subprograms
!
!     atovdc ---|---getmf
!               |---tiptim
!               |---tipqc --- xdqj
!               |            
!               |---amsuqc
!               |---hirget 
!            or |---amsget--- amsua_linecheck
!            or |---otiget
!            or |---dcsget--- dcsin
!               |
!               |---anaget
!               |---hirout---|---hirhdu---|---xdqj
!               |            |            |---xiqg
!               |            |            |---xigc
!               |            |            |---gp_sb32
!               |            |            |---cp_ups
!               |    
!               |
!               |            |---hirsdc---|---gp_x16
!               |            |            |---gp_sb16
!               |
!               |
!               |            |---hirdtr
!               |            |---hirdtw
!               |            |---gp_sb32
!               |
!               !                         |---cp_ups
!               |---amsout---|---amshdu---|---xdqj
!               |            |            |---xiqg
!               |            |            |---xigc   
!               |            |            |---amast
!               |            |            |---ambst
!               |            |            |---mhsst
!               |            |    
!               |            |---amsadc---|---gp_sb32
!               |            |---amsbdc---|---gp_sb32 
!               |            |---mhsdc----|---gp_sb32 
!               |            |---amanag_out
!               |            |---amadtr
!               |            |---amadtw
!               |            |---ambdtr
!               |            |---ambdtw
!               |            |---mhsdtr
!               |            |---mhsdtw
!               |
!               |---msuout---|---msuhdu---|---xdqj
!               |            |            |---xiqg
!               |            |            |---xigc
!               |            |            |---cp_ups
!               |            |---msudc
!               |            |---msudtr
!               |            |---msudtw
!               |            |---gp_sb32
!               |
!               |---dcsout---|---dcshdu---|---xdqj
!               |            |            |---xiqg
!               |            |            |---xigc
!               |            |            |---cp_ups
!               |            |---dcsdtr
!               |            |---dcsdtw
!               |            |---xiqj
!               |            |---xijq
!
!
!
!references       atovdc design document version 1.9 Nov 95
!
!changes          01 none
!                 02 Changes from Marcel and Nigel to make code more robust
!                 03 Include analogue telemetry. N C Atkinson, 15/1/96
!
!#########################################################################
! copyright        the contents of this file are crown copyright. the
!                  software, data and other proprietary information
!                  contained therein is supplied for bonafide research
!                  purposes. commercial or business use is not permitted
!                  without the written authority of the u.k. met. office.
!                  at london rd., bracknell, berks, rg12 2sz, u.k.
!#########################################################################
      subroutine atovdc(yr,optn,ninst,opmode,satid,idat,orbmf,
     &itime,systim,startorb,endorb,hrpmf,hrpaip,nbmishrp)
!
      implicit none
!
      integer*4 hrpmf                   !hrpt minor frame number
      integer*4 tipmf                   !tip minor frame number
      integer*2 hrpaip(104,5)           !5 tip/amsu/backfill minor frames
      integer*4 yr,optn(12)             !year of data, options
      integer*4 systim(3)               !system year, dayno, msecs from midnt
      integer*4 startorb,endorb         ! orbit number (start and end)
      integer*4 day,msecs               ! tip day no and msecs from midnight

      integer*4 nbmishrp                ! number of missing hrpt frames
!                                        between current and last one
!
      integer*4 msech,mseca,msecm       ! hirs/amsu/msu times
      integer msecd                     ! dcs time
      integer lastjmf                   ! last major frame number
      integer iqdcs(3)                    ! current instant for dcs
      integer*4 ninst                   ! no. of instruments to extract
      integer*4 opmode                  ! real time (1) or off-line (0)
      integer*4 satid                   ! hrpt s/c address
      integer*4 err,msuerr              ! error code (0=ok) see tipqc for more
      integer*4 aerr,berr(3)            ! amsu-a & b error codes
      integer*4 orbmf                   ! orbit main frame no.
      integer*4 itime                   ! hrpt time msecs from midnight
      integer*4 idat                    ! hrpt day number
      integer*4 imf,amf                 ! tip and amsu minor frame counters
      data amf/-1/
      integer*4 tipjmf                  ! major frame in tip data
      integer*4 hirs3                   ! hirs 3 flag (set to 1 if hirs-3)
      integer*4 i,j                     ! counters
      integer*2 hirs(1152)              ! one hirs scan line
      integer*2 msu(112)                ! one msu scan line
      integer*2 amsua1(620),amsua2(156) ! amsu-a data for 1 scan line
      integer*2 amsub(1950)             ! amsu-b data for 3 scan lines
      character*1 mhs(3900)             ! MHS data for 3 scan lines
      integer*2 hianag(1)               ! hirs analog telemetry more info
      integer*2 amanag(71)              ! amsu analog telemetry
      integer*2 hivalid(1)              ! hirs analog telemetry validity
      integer*2 amvalid(71)             ! amsu analog telemetry validity
      integer*4 nmf                     ! no. of tip maj/minor frames read
      integer*4 nhirs,nmsu,namsua,namsub,ndcs !no scan lines
      integer*4 lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu
      integer*4 ludcs,lusem,lusbuv,lusar,luavh
      integer*2 dcs(10240)              ! dcs data
      integer dcswrd4                   ! temporary word for dcs
      integer*2 dcsan(14)               ! dcs analog data
      integer lastmf                    ! mast tip minor frame
      integer*4 word1,word2             ! temporary words
!f90  integer*4 z1/1/
      integer*4 z1
      parameter (z1=1)
      integer*2 blankaip(104,5)         ! tip/aip array filled with 0
!     integer*2 sem(1)                  ! sem data modify to use
!     integer*2 sbuv(1)                 ! sbuv data modify to use
!     integer*2 sar(1)                  ! sar data modify to use
!f90  logical*1 first/.true./,oldtip/.false./,fullhi,fullam,fullms
      logical*1 first,oldtip,fullhi,fullam,fullms,mhspresent
      data first/.true./
      data oldtip/.false./
      data mhspresent/.false./
      
!f90  logical*1 gotime/.false./
      logical*1 gotime
      data gotime/.false./
      
!f90  logical*1 start_hirs /.false./ 
      logical*1 start_hirs        ! becomes true when a start of a line has been found
      data  start_hirs/.false./
      
!f90  logical*1 first_amf/.true./
      logical*1 first_amf
      data  first_amf/.true./
      
      logical*1 ok1,ok2,ok3,ok4,found
      integer   mfmod
      integer orbmf_amf_start,amf_start,exp_amf
      integer*4 exp_mis_amf,orbmf_amjf_start,amjf_start,exp_mis_amjf
      logical*1	filler
      
!      logical*1 fullsa,fullse,fullsb
!
      integer id_in    ! noaa id corresponding to satid variable
      integer id_noaak ! noaa id corresponding to Noaa-K == 15
      integer noascid  ! lisatid function

      integer  exp_tipmnf, exp_hrpmnf
      integer  tipmnf_start, k
      integer tipmf1,hrpmf1
      integer last_hrpscn , last_hrpmf ,last_tipmf
      integer mseca_offset
      common/iocom/ lulog,luhirs3,luamsa1,luamsa2,luamsb,luhirs2,lumsu,
     &          ludcs,lusem,lusbuv,lusar,luavh
      common /scan/ msech,msecm,mseca,hirs,msu,amsua1,amsua2,amsub,mhs
      common /log/msecs,day,nmf,nhirs,nmsu,namsua,namsub,ndcs
      data amvalid/71*1/   !Initialize invalid analogue data flag for AMSU
      data hivalid/1/      !Initialize invalid analogue data flag for HIRS
      data id_noaak/15/
      save
!      --------------------------------------------------
!      section 1    for initialisation at the first call
!      --------------------------------------------------
      berr(1) = 0
      berr(2) = 0
      berr(3) = 0
      if (first)then
        if (optn(6).eq.1 .or. optn(7).eq.1)oldtip=.true. !pre noaa-k data
        ! test the noaa sc id (12 for noaa12) to
        ! determine if it is a klm serie or not
        id_in = noascid(satid)
        if(id_in .lt. id_noaak ) oldtip=.true.
        !if ( satid .lt. 12 )oldtip=.true.
        if (id_in .gt. id_noaak+2) mhspresent = .true. !NOAA-N,N'
        !if (satid .ge. 15) mhspresent = .true. !NOAA-N,N'
        if (yr .lt. 1978) yr = systim(1) !force invalid year to be current
        hirs3 = 1
        if (oldtip) hirs3=0
        first=.false.
        nmf = 0
        err = 0         ! initialise error flag
        msuerr = 0      ! initialise error flag
        day = idat      ! in case tip time problem set hrpt day number
        msecs = itime   ! in case tip time problem set hrpt time
        mseca = msecs
        msecd = msecs
        mseca_offset = 0
        nhirs = 0
        nmsu = 0
        namsua = 0
        namsub = 0
        ndcs = 0
        iqdcs(1)=yr
        iqdcs(2)=day
!        write(lulog,101) systim(3)/1000,systim(2),systim(1)
        call ml_wt('info','atovdc started')
        word1 = hrpaip(5,1)/4
!f90    word1 = word1.and.z1
        word1 = IAND(word1,z1)
        word1 = word1*256                 ! shift left 8 bits
        word2 = hrpaip(6,1)/4
        tipmf = word1 + word2
        tipmnf_start = tipmf
        tipjmf=hrpaip(4,1)/16
!f90    tipjmf=tipjmf.and.7
        tipjmf=IAND(tipjmf,7)
        iqdcs(3)=msecd-tipmf*100
        lastjmf=tipjmf
        lastmf=tipmf-1
        if(lastmf.lt.0) lastmf=0
        do i=1,5
          do j=1,104
            blankaip(j,i)=0
          enddo
        enddo
      endif
!
!   expected tip minor frame counter for the first minor frame among 5
! according to the fact that the first hrpt scan line is an hrpt minor frame 1
!
      exp_tipmnf=mod(tipmnf_start+((orbmf-1)/3)*5,320)
      if(.not.first_amf) then
        exp_amf=mod(amf_start+(orbmf-orbmf_amf_start)/3*5,80)
      endif
!
!     expected hrpt minor frame counter
!
      exp_hrpmnf=mod(orbmf-1,3)+1
!----------------------------------------------------------------------
!      word1 = hrpaip(5,1)/4
!      word1 = word1.and.z1
!      word1 = word1*256                 ! shift left 8 bits
!      word2 = hrpaip(6,1)/4
!      read_tipmnf= word1 + word2
!      if(read_tipmnf.ne.exp_tipmnf) then
!        write(lulog,103)orbmf,exp_tipmnf,read_tipmnf,exp_hrpmnf,
!     &  hrpmf
!103     format('hrpt line ',i5,' expected tipmf',i4,' read',i4,
!     &  ' expected hrpmf ',i2,' read',i2)
!      endif
!-----------------------------------------------------------------------
!
!     section 2 to fill arrays when there are missing hrpt scan lines
!
!-----------------------------------------------------------------------
      if (nbmishrp.ne.0) then
        last_hrpscn = orbmf - nbmishrp -1 ! last good scan number
        last_hrpmf = mod(last_hrpscn-1,3) + 1 ! its minor frame
        last_tipmf = mod(tipmnf_start+((last_hrpscn-1)/3)*5 + 4,320)!its tipmf        
!
!        write(lulog,*) last_hrpscn,last_hrpmf,last_tipmf
!       ------------------------------------
!       loop on missing lines to fill arrays
!       ------------------------------------
        do k = last_hrpscn +1 , orbmf - 1
          exp_mis_amf = mod(amf_start+(k-orbmf_amf_start)/3*5,80)
          exp_mis_amjf = mod(amjf_start+(k-orbmf_amjf_start)/48,4)
          hrpmf1= mod( k -1 , 3 ) + 1
          tipmf1 =mod (tipmnf_start+((k-1)/3)*5,320)
!         ------------------------
!         2.1 oldtip case  filling
!         ------------------------
          if ( hrpmf1 . eq .1 .and. oldtip) then
            do imf = 1 , 5                 ! loop in 5 minor frames
              tipmf = mod(tipmf1 + imf -1,320)
              if(mod(tipmf,64).eq.1) start_hirs=.true.

!              write(lulog,*) 'filling missing tip mnf ',tipmf,k,hrpmf1
              msecs=msecs+100
              if(msecs.gt.86400000)then
                msecs = msecs - 86400000
                day = day+1
              endif
              if(tipmf.eq.0) then
                tipjmf = tipjmf+1
                if(tipjmf.gt.7) tipjmf = 0
!
!     in case of an already started pass the hirs scan line
!     time is the one of the minor frame 0 minus 6300 msecs
!     else it will be the time of the following TIP minor frame
! 
                if( start_hirs ) then
                  msech = msecs - 6300
                  if (msech.lt.0) msech = msech + 86400000
                else
                  msech = msecs + 100
                  if(msech.gt.86400000) msech = msech - 86400000
                endif
                mseca = msecs
                iqdcs(1)=yr
                iqdcs(2)=day
                msecd=msecs
                iqdcs(3)=msecd
                nmf = nmf + 1
              endif
!f90          msuerr= err.or.msuerr
              msuerr= IOR(err,msuerr)
              call otiget(optn,imf,tipmf,blankaip,hirs,msu,
     &             fullhi,fullms,msecs,msecm,msuerr)
! ..write out hirs/msu scan lines when full
              if (fullhi)call hirout(hirs,hianag,yr,msech,systim,
     &          satid,err,fullhi,gotime,startorb,endorb,hirs3)     ! write 1a
              if (fullms)then
                call msuout(msu,yr,msecm,systim,satid,msuerr,
     &                 fullms,gotime,startorb,endorb) ! write 1a
              endif
! process dcs data
              if(optn(8).eq.1) then
                call dcsget
     &          (satid,imf,tipmf,blankaip,dcswrd4,dcsan,dcs)
                if(tipmf.eq.319) then
! ..write out dcs scan lines when full
                msecd=msecd+320000
                call dcsout(dcswrd4,dcsan,dcs,iqdcs,lastjmf,
     &              satid,err,gotime,startorb,endorb)
              endif
            endif
          enddo  ! end loop on 5 minor frames

!         ------------------------
!         2.2 new tip case filling
!         ------------------------

          else if (hrpmf1.eq.1.and..not.oldtip) then
!
            do imf = 1 , 5                 ! loop in 5 minor frames
              tipmf = mod(tipmf1 + imf -1,320)
              if(mod(tipmf,64).eq.1) start_hirs=.true.
!             write(lulog,*) 'filling missing tip mnf ',tipmf
              msecs=msecs+100
              if(msecs.gt.86400000)then
                msecs = msecs - 86400000
                day = day+1
              endif
              if(tipmf.eq.0) then
                tipjmf = tipjmf+1
                if(tipjmf.gt.7) tipjmf = 0
!
!     in case of an already started pass the hirs scan line
!      time is the one of the minor frame 0 minus 6300 msecs
!      else it will be the time of the following TIP minor frame
! 
                if( start_hirs ) then
                  msech = msecs - 6300
                  if (msech.lt.0) msech = msech + 86400000
                else
                  msech = msecs + 100
                  if(msech.gt.86400000) msech = msech - 86400000
                endif
                mseca = msecs + mseca_offset
                iqdcs(1)=yr
                iqdcs(2)=day
                msecd=msecs
                iqdcs(3)=msecd
                nmf = nmf + 1
              endif
!
              if(optn(2).eq.1)call hirget
     &           (imf,tipmf,blankaip,hirs,fullhi) ! hirs-3
! process dcs data
              if(optn(8).eq.1) then
                call dcsget
     &           (satid,imf,tipmf,blankaip,dcswrd4,dcsan,dcs)
                if(tipmf.eq.319) then
! normallly full array
                  msecd=msecd+320000
                  call dcsout(dcswrd4,dcsan,dcs,iqdcs,lastjmf,
     &               satid,err,gotime,startorb,endorb)
                endif
              endif
              lastmf=tipmf
              lastjmf=tipjmf
!             if(optn(9).eq.1)call semget(imf,tipmf,blankaip,sem,fullse)  ! sem
!             if(optn(10).eq.1)call sbuget(imf,tipmf,blankaip,sbuv,fullsb)!sbuv
!             if(optn(11).eq.1)call sarget(imf,tipmf,blankaip,sar,fullsa) ! sar
              call anaget(imf,tipmf,blankaip,hianag,amanag,
     &          hivalid,amvalid)   !extract analogue telemetry
!
!             ..write out hirs-3/dcs etc scan lines when full
              if (fullhi)call hirout(hirs,hianag,yr,msech,systim,
     &            satid,err,fullhi,gotime,startorb,endorb,hirs3)     ! write 1a
!             if (fullse)call seout(sem,yr,day,msecs,err,startorb,endorb)
!             if (fullsb)call sbout(sbuv,yr,day,msecs,err,startorb,endorb)
!             if (fullsa)call saout(sar,yr,day,msecs,err,startorb,endorb)
!
            enddo
          else if(hrpmf1.eq.3 .and. .not. oldtip)then
!
!---- section fill data from 5 amsu minor frames ----------
!---- has to be revised by UKMO
!----------------------------------------------------------
            if (optn(3).eq.1.or.optn(4).eq.1.or.optn(5).eq.1)then
            if (first_amf) return
            
!
              do  23 imf = 1 , 5                 ! loop in 5 minor frames
!
                amf=mod(exp_mis_amf+imf-1,80)
	        if (amf.eq.0) exp_mis_amjf=mod(exp_mis_amjf+1,4)
	        filler=.TRUE.
                call ml_wti('info','amsu is filled at mf',amf)
                if(amf.lt.0) amf=80+amf 
                blankaip(5,imf)=amf
                blankaip(6,imf)=exp_mis_amjf
!.. dummy MIU minor frame count, 2 frames behind AIP minor frame
                blankaip(8,imf)=mod(amf+78,80)
!.. do checks on data
                call amsuqc(imf,amf,aerr,berr,blankaip,filler) ! do checks on data
                if (amf .eq. -1) go to 23
!.. get amsu
                call amsget(optn,imf,amf,blankaip,amsua1,
     &           amsua2,amsub,mhs,filler,fullam,mhspresent)

!.. write out amsu scan lines when full
c               write(6,*) fullam,' INSERTED amsu minfri in',tipmf,imf
                if(fullam) call amsout(optn,amsua1,amsua2,amsub,mhs,
     &            amanag,amvalid,yr,mseca,systim,satid,aerr,berr,fullam,
     &            gotime,startorb,endorb,mhspresent)
!
 23           continue
            endif
          endif
        enddo
      endif
!----------------------------------------
!     end of section filling missing data
!----------------------------------------

!     ---------------------------------------
!     section 3 extraction of real data
!     ----------------------------------------
!     3.1 hrpt minor frame 2 backfill data
!     ----------------------------------------
      if (exp_hrpmnf.eq.2) return  ! backfill data so return
!
!------- remove 2 parity bits ------------------------------------------
!
      do 5 j = 1 , 5
      do 5 i = 1 , 104
        hrpaip(i,j) = hrpaip(i,j)/4    ! shift right 2 bits
 5    continue
!     ---------------------------------------------------------------------
!     3.2 section to read and extract data from 5 noaa-klm tip minor frames
!     ---------------------------------------------------------------------
      if (exp_hrpmnf.eq.1 .and. .not. oldtip)then
!
        do 10 imf = 1 , 5                 ! loop in 5 minor frames
!
!         get current tip minor & major frame number
!
          call getmf(imf,tipmf,tipjmf,err,hrpaip)
          if (tipmf.ne.mod(exp_tipmnf+imf-1,320)) then
! check tip minor frame counter against expected
! and correct if necessary
            k=mod(exp_tipmnf+imf-1,320)
            tipmf=k
          endif
          if(mod(tipmf,64).eq.1) start_hirs=.true.
!
!         get tip date & time if minor frame zero
!         v2.9: also require that we know the AIP minor frame number
!
          if (tipmf .eq. 0 .and. amf .ne. -1)then
            call tiptim(imf,opmode,systim,idat,itime,err,hrpaip,gotime)
            if (msecs.gt.86400000)then
             msecs = msecs - 86400000
             day = day+1
            endif
!
!     in case of an already started pass the hirs scan line
!      time is the one of the minor frame 0 minus 6300 msecs
!      else it will be the time of the following TIP minor frame
! 
            if( start_hirs ) then
               msech = msecs - 6300
               if (msech.lt.0) msech = msech + 86400000
            else
              msech = msecs + 100
              if(msech.gt.86400000) msech = msech - 86400000
            endif

! amsout is called at AMSU minor frame 79 and assumes that the TIP time has just
! been updated. But sometimes the TIP time update occurs afterwards - allow for 
! this in v2.4.
!
            if (amf.ge.40.and.amf.lt.79) then
              mseca_offset = 0
	    else
	      mseca_offset = 8000
	    endif
            mseca = msecs + mseca_offset
	    
            iqdcs(1)=yr
            iqdcs(2)=day
            msecd=msecs
            iqdcs(3)=msecd
!
!           if (nmf.eq.0)write(lulog,102)msecs,day,yr
!
            if(nmf.eq.0) call ml_wti('info','tip data starts at ',msecs)
            if(nmf.eq.0) call ml_wti('info',
     &           'tip time update at amf ',amf)
            nmf = nmf + 1
          else
            msecs = msecs+100
            if(msecs.gt.86400000)then
              msecs = msecs - 86400000
              day = day+1
            endif
          endif
!
          call tipqc(imf,satid,idat,yr,err,hrpaip) ! do checks on data
!
          if(optn(2).eq.1)call hirget(imf,tipmf,hrpaip,hirs,fullhi) ! hirs-3
! process dcs data
          if(optn(8).eq.1) then
! we need a valid minor frame number
            if(tipmf.eq.319) then
! normallly full array
              call dcsget
     &             (satid,imf,tipmf,hrpaip,dcswrd4,dcsan,dcs)
              call dcsout(dcswrd4,dcsan,dcs,iqdcs,lastjmf,satid,err,
     &                 gotime,startorb,endorb)
            else
              call dcsget
     &       (satid,imf,tipmf,hrpaip,dcswrd4,dcsan,dcs)
          endif
        endif
        lastmf=tipmf
        lastjmf=tipjmf
!       if(optn(9).eq.1)call semget(imf,tipmf,hrpaip,sem,fullse)  ! sem
!       if(optn(10).eq.1)call sbuget(imf,tipmf,hrpaip,sbuv,fullsb)! sbuv
!       if(optn(11).eq.1)call sarget(imf,tipmf,hrpaip,sar,fullsa) ! sar
        call anaget(imf,tipmf,hrpaip,hianag,amanag,hivalid,amvalid) 
                                                !extract analogue telemetry
!
! ..write out hirs-3/dcs etc scan lines when full
         if (fullhi)call hirout(hirs,hianag,yr,msech,systim,
     &     satid,err,fullhi,gotime,startorb,endorb,hirs3)     ! write 1a
!       if (fullse)call seout(sem,yr,day,msecs,err,startorb,endorb)
!       if (fullsb)call sbout(sbuv,yr,day,msecs,err,startorb,endorb)
!       if (fullsa)call saout(sar,yr,day,msecs,err,startorb,endorb)
!
 10    continue
      endif
!     -------------------------------------------------------------
!     3.3 section to read and extract data from 5 amsu minor frames
!     -------------------------------------------------------------
!
      if (exp_hrpmnf.eq.3 .and. .not. oldtip)then
        filler=.FALSE.
        if (optn(3).eq.1.or.optn(4).eq.1.or.optn(5).eq.1)then
          if(first_amf) then
            found=.false.
            imf = 1
C2.6 100        continue
C2.6            if(hrpaip(5,imf+1)-hrpaip(5,imf).eq.1) then
C2.6              found = .true.
C2.6            else
C2.6              imf=imf+1
C2.6              if(imf.le.4) go to 100
C2.6            endif
!MOD/NIWA begin
!Require that all 5 minor frames have consistent frame numbers
!(consecutive/wrapping in 0-79)
            mfmod = 0
            if (hrpaip(5,imf+1).eq.0) mfmod = 80
            ok1 = (mfmod+hrpaip(5,imf+1)-hrpaip(5,imf).eq.1)
            if (hrpaip(5,imf+2).eq.0) mfmod = 80
            ok2 = (mfmod+hrpaip(5,imf+2)-hrpaip(5,imf).eq.2)
            if (hrpaip(5,imf+3).eq.0) mfmod = 80
            ok3 = (mfmod+hrpaip(5,imf+3)-hrpaip(5,imf).eq.3)
            if (hrpaip(5,imf+4).eq.0) mfmod = 80
            ok4 = (mfmod+hrpaip(5,imf+4)-hrpaip(5,imf).eq.4)
            found = (ok1.AND.ok2.AND.ok3.AND.ok4)
!MOD/NIWA end

            if(found) then
              amf_start=hrpaip(5,imf)-imf+1    
              orbmf_amjf_start=orbmf-3*((hrpaip(5,imf)-1)/5)
              amjf_start=hrpaip(6,imf)
              if(amf_start.lt.0) amf_start=80+amf_start
              first_amf=.false.
              orbmf_amf_start=orbmf
              call ml_wti('info','good amf start at amf number ',
     &                   amf_start)
              call ml_wti('info','for line number ',
     &                   orbmf_amf_start)
              exp_amf=mod(amf_start+(orbmf-orbmf_amf_start)/3*5,80)
            else 
              return
            endif
          endif
          amf=hrpaip(5,1)
!          type *,'exp_amf ',exp_amf,' amf ',amf         
!
! loop in 5 minor frames
c              do j = 1,10
c              write(6,*) 'VER ATOVDC',j,hrpaip(j,1)
c              enddo

          do 30 imf = 1 , 5
!
            amf=mod(exp_amf+imf-1,80)
            if(amf.lt.0) amf=80+amf
            if(hrpaip(5,imf).ne.amf) then
               hrpaip(5,imf)=amf
               call ml_wti('info','correction of amsu mf',amf)
            endif  
            call amsuqc(imf,amf,aerr,berr,hrpaip,filler) ! do checks on data
!
! to be revised by ukmo here
!
            if (amf .eq. -1) go to 30
!           get amsu
            call amsget(optn,imf,amf,hrpaip,amsua1,amsua2,
     &          amsub,mhs,filler,fullam,mhspresent)

! ..write out amsu scan lines when full
            if (fullam) call amsout(optn,amsua1,amsua2,amsub,mhs,amanag,
     &         amvalid,yr,mseca,systim,satid,aerr,berr,fullam,gotime,
     &         startorb,endorb,mhspresent)
!
 30       continue
        else
          amf = 0      !Dummy value for amf if AMSU not required
        endif        !(optn(3).eq.1.or.optn(4).eq.1.or.optn(5).eq.1)
      endif        !(exp_hrpmnf.eq.3 .and. .not. oldtip)
!     ----------------------------------------------------
!     3.3 section to read and extract pre-noaa-k tip data
!          from 5 tip minor frames
!     ----------------------------------------------------
      if (exp_hrpmnf.eq.1 .and. oldtip) then
!
        do imf = 1 , 5  ! loop in 5 minor frames
!
!         get current tip minor and major frame number
!
          call getmf(imf,tipmf,tipjmf,err,hrpaip)
!
          if (tipmf.ne.mod(exp_tipmnf+imf-1,320)) then
!         check tip minor frame counter against expected
!         and correct if necessary
            k=mod(exp_tipmnf+imf-1,320)
            tipmf=k
          endif
          if(mod(tipmf,64).eq.1) start_hirs=.true.
!
!        get tip date & time if minor frame zero
!
          if (tipmf .eq. 0) then
            call tiptim(imf,opmode,systim,idat,itime,err,hrpaip,gotime)
            if (msecs.gt.86400000)then
              msecs = msecs - 86400000
              day = day+1
            endif
            msech = msecs-6300
!
!     in case of an already started pass the hirs scan line
!      time is the one of the minor frame 0 minus 6300 msecs
!      else it will be the time of the following TIP minor frame
! 
            if( start_hirs ) then
              msech = msecs - 6300
              if (msech.lt.0) msech = msech + 86400000
            else
              msech = msecs + 100
              if(msech.gt.86400000) msech = msech - 86400000
            endif
            iqdcs(1)=yr
            iqdcs(2)=day
            msecd=msecs
            iqdcs(3)=msecd
!          if (nmf.eq.0)write(lulog,102)msecs,day,yr
            if(nmf.eq.0) call ml_wti('info','tip data starts at ',msecs)
            nmf = nmf + 1
          else
            msecs = msecs+100
            if(msecs.gt.86400000) then
              msecs = msecs - 86400000
              day = day+1
            endif
          endif
!          write(lulog,*)' tip mf=',tipmf

!
          call tipqc(imf,satid,idat,yr,err,hrpaip) ! do checks on data
!
!        extract hirs & msu words
!
!f90      msuerr = err.or.msuerr
          msuerr = IOR(err,msuerr)
          call otiget(optn,imf,tipmf,hrpaip,hirs,msu,fullhi,fullms,
     &     msecs,msecm,msuerr)
!        ..write out hirs/msu scan lines when full
          if (fullhi) call hirout(hirs,hianag,yr,msech,systim,
     &      satid,err,fullhi,gotime,startorb,endorb,hirs3)     ! write 1a
          if (fullms) call msuout(msu,yr,msecm,systim,satid,msuerr,
     &                 fullms,gotime,startorb,endorb) ! write 1a

! process dcs data
          if (optn(8).eq.1) then
            call dcsget
     &       (satid,imf,tipmf,hrpaip,dcswrd4,dcsan,dcs)
            if(tipmf.eq.319) then
! normallly full array with tipmf 319
              call dcsout(dcswrd4,dcsan,dcs,iqdcs,lastjmf,satid,err,
     &                 gotime,startorb,endorb)
            endif
          endif
          lastmf=tipmf
          lastjmf=tipjmf
        enddo ! end loop on 5 minor frames
!
      endif ! on hrpt mnf and oltip test
!
 99   return
 101  format(1x,'#####################################################',
     &        /,'    atovdc started at ',i6,' secs past midnight on '/,
     &          '    day ',i3,1x,i4,/)
 102  format(1x,'tip data starts at ',i9,'ms past 0z on day',i3,1x,i4,/)
      end
