#!/bin/ksh
# $Id: VIIRS_CRIS_MAIA4_RUN.ksh 617 2019-02-06 09:39:42Z frna $
#
# Korn Shell script to process CrIS from SDR to BUFR, with VIIRS cloud mask and
# cluster analysis.
#
# Assumes that input VIIRS files are individual granules, i.e. not aggregated
# by CSPP. Granule size is ~85s so expect ~9 granules in a typical 12-minute
# pass.
#
# The script runs multiple sub-processes (one for each VIIRS granule), so a
# multi-core machine is recommended.
#
# AAPP must have been built with the hdf5 and ecCodes libraries linked in.
#
# Additional tools required:
#   nagg v1.6.2 or later, see https://support.hdfgroup.org/projects/jpss/jpss_downloads.html
#   python (for AAPP tool naggmaia4.py)
#
# Input via environment variables:
#   CRIS_SDR_DIR  - directory containing CrIS SDR files. Default: current
#   VIIRS_SDR_DIR - directory containing VIIRS SDR files. Default: current
#   VIIRS_BANDS   - bands to include in the cluster analysis. Default: "M5 M7 M12 M15 M16"
#   DIR_FORECAST  - directory containing GFS forecast files. You could download these in
#                   advance if you want to save time.
#
# Products are generated in the current directory.
#
# PROPRIETAIRE/OWNER
#       NWP SAF
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2019, EUMETSAT, All Rights Reserved.
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date    comment/author)
#  1.0    29/01/2019   Initial, N Atkinson
#
################################################################################

[ "$AAPP_PREFIX" ] || { echo "Error: AAPP_PREFIX not defined"; exit 2; }
[ -d "$AAPP_PREFIX" ] || { echo "Error: AAPP_PREFIX is not a directory: $AAPP_PREFIX"; exit 2; }

. "$AAPP_PREFIX/ATOVS_CONF"

#-------------------------------
# Usage
#-------------------------------

usage(){
  echo 'usage : VIIRS_CRIS_MAIA4_RUN [--help | -h]'
  echo 'optional environment variables:'
  echo '    CRIS_SDR_DIR, VIIRS_SDR_DIR, VIIRS_BANDS, DIR_FORECAST'
  echo 'default VIIRS_BANDS="M5 M7 M12 M15 M16" (AVHRR-like channels)'
}

if [[ "$1" == "-h" || "$1" == "--help" ]]; then
  usage
  exit 0
fi

v=$(nagg --version | grep Version) || { echo "Error: nagg is not in PATH"; exit 1; }
echo $v
d=${v#*"released on"}
if [ $(date +%s -d $d) -lt $(date +%s -d 2016-07-22) ]; then 
  echo "Please use a newer version of nagg, see https://support.hdfgroup.org/projects/jpss/jpss_downloads.html"
  exit 1
fi

topdir=$PWD
export DIR_FORECAST=${DIR_FORECAST:-$topdir/forecast}
export MAIA4_USE_GFS="yes"
mkdir -p $DIR_FORECAST

CRIS_SDR_DIR=${CRIS_SDR_DIR:-$topdir}
VIIRS_SDR_DIR=${VIIRS_SDR_DIR:-$topdir}
VIIRS_BANDS=${VIIRS_BANDS:-"M5 M7 M12 M15 M16"}

rm -f *XXXX_XXX*            #remove any aggregate files left over
rm -rf d????????_t???????   #remove any directories left over
rm -f clusters.h5 maia4_agg.h5

#--------------------------------------------------------------------------
#Create links to the VIIRS SDRs in a separate subdirectory for each granule
#--------------------------------------------------------------------------

timestamps=$(cd $VIIRS_SDR_DIR; ls GMTCO*.h5 | sed y/"_"/" "/ | awk '{print $3"_"$4}')
[ "$timestamps" ] || { echo "No GMTCO files found"; exit 1; }

for granule in $timestamps; do
  echo "Create directory: $granule"
  cd $topdir
  firstgran=${firstgran:-$granule}
  lastgran=$granule
  mkdir -p $granule
  cd $granule
  for file in $(ls $VIIRS_SDR_DIR/GM*$granule*.h5); do ln -sf $file; done
  for file in $(ls $VIIRS_SDR_DIR/SVM*$granule*.h5); do ln -sf $file; done
  for file in $(ls $VIIRS_SDR_DIR/SVI*$granule*.h5); do ln -sf $file; done
done

#--------------------------------------
#Get the forecast files that are needed
#--------------------------------------

cd $topdir/$firstgran
file=$(ls GMTCO*.h5)
set $(IFS="_"; echo $file)
YYYYMMDD_Deb=${3:1:8}
HHMN_Deb=${4:1:4}
NNNNN=${6:1:5}
SATIMG=$2

cd $topdir/$lastgran
file=$(ls GMTCO*.h5)
set $(IFS="_"; echo $file)
YYYYMMDD_End=${3:1:8}
HHMN_End=${4:1:4}

echo "get forecast files for ${YYYYMMDD_Deb} ${HHMN_Deb} ${YYYYMMDD_End} ${HHMN_End}"
. maia_env
. maia_date
. liblog.sh
get_forecast_file_gfs_cspp ${YYYYMMDD_Deb} ${HHMN_Deb} ${YYYYMMDD_End} ${HHMN_End}

#----------------------------
#Process granules in parallel
#----------------------------

echo "Call MAIA4_RUN for each granule"
for granule in $timestamps; do
  cd $topdir/$granule
  MAIA4_RUN $PWD >MAIA4_RUN.out 2>&1 &
  pids="${pids} $!"
done
echo "pids=$pids"
 
finished=N
while [ $finished = N ]; do
  finished=Y
  for pid in $pids; do
    [ "$(ps $pid | grep -v PID)" ] && finished=N
  done
  [ $finished = N ] && sleep 5
done
cd $topdir

#--------------------------------------------
#Merge the VIIRS files for the required bands
#--------------------------------------------

spec=""
viirsfiles="$(ls d*/GMTCO*.h5)"
for band in $VIIRS_BANDS; do
  bandnum=${band#M*}
  nn=$(printf "%2.2d" $bandnum)
  spec="${spec:+$spec,}SVM${nn}"
  viirsfiles="${viirsfiles} $(ls d*/SVM${nn}*.h5)"
done 

echo "Merge VIIRS files using nagg"
nagg --onefile -n 99 -t $spec -g GMTCO $viirsfiles    #note that --onefile not available in 1.5.2
viirs_agg_file=$(ls GMTCO-SVM*XXXX_XXX.h5 | head -1)  #should only be one file
[ -f "$viirs_agg_file" ] || { echo "problem running nagg for VIIRS"; exit 1; }

#--------------------
#Merge the MAIA files
#--------------------

echo "Merge MAIA files using naggmaia4.py"
maiafiles=$(ls d*/viiCT*.h5)
naggmaia4.py $maiafiles
[ -f maia4_agg.h5 ] || { echo "problem running naggmaia4.py"; exit 1; }

#---------------------------
#Aggregate the CrIS granules
#---------------------------

echo "Merge CrIS files using nagg"
filetype=SCRIS
ls ${CRIS_SDR_DIR}/SCRIF*h5 >/dev/null 2>&1
[ $? -eq 0 ] && filetype=SCRIF

nagg -n 99 -t ${filetype} ${CRIS_SDR_DIR}/SCRI*.h5 ${CRIS_SDR_DIR}/GCRSO*.h5
cris_h5=$(ls GCRSO-SCRI*.h5 | head -1)
[ -f $cris_h5 ] || { echo "problem running nagg for CrIS"; exit 1; }

#-------------------
#Convert CrIS to l1c
#-------------------

echo "Convert CrIS to l1c"
cris_sdr -g $cris_h5 $cris_h5
cris_l1c=$(ls GCRSO-SCRI*.l1c)
[ -f $cris_l1c ] || { echo "problem running cris_sdr"; exit 1; }

#----------------------
#CrIS channel selection
#----------------------

echo "Run cris_channels"
cris_l1c_channels=${cris_l1c%.l1c}_channels.l1c
cris_channels $cris_l1c $cris_l1c_channels
[ -f $cris_l1c_channels ] || { echo "problem running cris_channels"; exit 1; }

#-----------------
#Run viirs_to_cris
#-----------------

echo "Run viirs_to_cris"
viirs_to_cris -m maia4_agg.h5 -g $viirs_agg_file -C clusters.h5 $cris_l1c_channels $viirs_agg_file
[ -f clusters.h5 ] || { echo "problem running viirs_to_cris"; exit 1; }

#-----------
#BUFR encode
#-----------

echo "BUFR encode"
eccodes_encodebufr_1c -i $cris_l1c_channels CRISFSR
cris_bufr=${cris_l1c_channels%.l1c}.bufr
[ -f $cris_bufr ] || { echo "problem running eccodes_encodebufr_1c"; exit 1; }

#-------------------
#Append the clusters
#-------------------

echo "Run cris_bufr_add_clusters"
outfile=${cris_bufr%.bufr}_clusters.bufr
cris_bufr_add_clusters $cris_bufr clusters.h5 $outfile

[ -f $outfile ] || { echo "problem running cris_bufr_add_clusters"; exit 1; }
echo "created $outfile"
echo "End of VIIRS_CRIS_MAIA4_RUN"


