C LIBS =  avhrrin_tools atovin atov avhrcl navnoaa tbus satid navtool f7nl1b f7ml f7tp f7cp f7gp auxaapp
      Program avhrrin

!------------------------------------------------------------------------------
!  $Id: avhrrin.F 557 2017-10-03 13:12:34Z frna $
!  (C) NWP SAF
!------------------------------------------------------------------------------
!------------------------------------------------------------------------------
!NAME 
!   avhrrin
!  
!PURPOSE 
!  Process HRPT/AVHRR level 1b  to AVHRR level 1c (AAPP formats)
!  Apply calibration coefficients to AVHRR numeric counts to get reflectances and
!  brightness temperatures.   
!
!DESCRIPTION 
!
!CODE OWNER
!   NWP SAF  
!
!HISTORY (CHANGES) 
!  (version    date         comment           <author>)
!    1.0      May 09       Creation          T. Labrot (MF)
!    1.1      11/06/2010   Allow avh_h_dataname to be empty. NCA
!    1.2      20/11/2015   Correction for scnlinbit (PR)
!    1.3      10/02/2016   Add NEdT
!    1.4      01/11/2016   Correct avh1c_spare3
!
!
!INPUT DATA 
!      FDFunit          ! logical units  
!      AVH1Bunit
!      AVH1Cunit
!      DBGunit                          
!      idbg             ! debugging option
!      line1            ! First and last AVHRR lines to write
!      line2            ! (idbg must be equal to 2)
!      pix1             ! First and last pixels of lines numbered    
!      pix2             ! line1 to line2 to write (idbg must be equal to 2)
!
!UPDATED ELEMENTS OF INCLUDES
!  avhrrin.h
!    FDFunit, AVH1Bunit, AVH1Cunit, DBGunit
!    idbg, 
!    line1, line2 
!    pix1, pix2
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!    infdf     (libatovin)
!    avh_lbc   (libavhrcl)
!    ioavh1b   (libmaia3_tools)
!    ioavh1c1d (libmaia3_tools)
!    avh1bhdp  (libmaia3_tools)
!    avh1bdtp  (libmaia3_tools)
!    avh1chdp  (libmaia3_tools)
!    avh1cdtp  (libmaia3_tools)
!    aappexit  (libauxaapp)
!    ml_xxx    (libf7ml)
!
!COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!------------------------------------------------------------------------------

!--DECLARATIONS----------------------------------------------------------------

       Implicit None
       
!Parameters for ATOVIN Processing  (infdf)
#include "inparms.h" 
!Unit numbers and options
#include "avhrrin.h" 
!Bit numbers for intrinsic bit routines (eg BTEST)          
#include "bits.h" 
!Data
#include "avh1b.h"
#include "avh1c.h"

     
! Local scalars:

       INTEGER avh1c_vers_code          ! Version number of 1B format compatible
       PARAMETER (avh1c_vers_code=1)    ! with the code in this subroutine
       INTEGER avh1c_header_nb          ! Number of header record
       PARAMETER (avh1c_header_nb=1)
       
       INTEGER p,c                      ! Loop index for pixels and channels
       INTEGER i,j                      ! Loop index and used in setting bits
       INTEGER isat                     ! Loop index for satellite number
       INTEGER ios                      ! Status
       
       INTEGER AVH1Brecord              ! Record number
       INTEGER AVH1Crecord              ! Record number 
       
       INTEGER line_read                ! Number of the AVHRR line read into the file
       INTEGER line_write               ! Number of the AVHRR line wriiten into the file
 
       INTEGER sat_index                ! satellite index (between 1 & MaxSats)
       INTEGER sttorbit                 ! last 2 digits, orbit number at start
       INTEGER endorbit                 ! last 2 digits, orbit number at end
       
       REAL avh_rad(avh1c_mxhrpchn,avh1c_mxpix)    ! Radiances mW/cm2.st.cm-1
       REAL avh_bt(avh1c_mxhrpchn,avh1c_mxpix)     ! Brigthness temperature (K) and albedo
       INTEGER avh_qc(avh1c_mxpix)                 ! QC word for each avhrr fov
                                                   !   (bit 'bit_QC' is 0=good, 1=bad fov)
       INTEGER avh_bitqc                           ! Bit number for quality
       INTEGER missing                             ! Line quality  0=present   1=missing
       INTEGER good_lines(avh1c_mxvischn+avh1c_mxirchn) ! Number of good lines for each channels 
       INTEGER bad_lines(avh1c_mxvischn+avh1c_mxirchn)  ! Number of bad lines for each channels
       INTEGER ib 
       
       
       
!--END OF DECLARATIONS----------------------------------------------------------------       

       CALL ml_wt('info',' ')
       CALL ml_wt('info','START of AVHRRIN.EXE')

!------------------------------------------------------------------------------
! INITIALISATION
!------------------------------------------------------------------------------       
       line_read =0
       line_write=0
       
       DO  p=1, avh1c_mxpix
         
         DO c=1, avh1c_mxhrpchn
            avh_rad(c,p)=0.
            avh_bt(c,p)=0.
         ENDDO
         
         avh_qc(p) = 0
         
       ENDDO
           
       missing=0
              
       DO i = 1 , (avh1c_mxvischn+avh1c_mxirchn)
         good_lines(i) = 0
         bad_lines(i)  = 0
       ENDDO
       
       DO i = 1 , 31
         bits(i) = 0
       ENDDO  
       
!------------------------------------------------------------------------------
! READ INPUT DATA and THE OPTIONS
!------------------------------------------------------------------------------
      READ(*,*) FDFunit   
      READ(*,*) AVH1Bunit
      READ(*,*) AVH1Cunit
      READ(*,*) DBGunit
      READ(*,*) idbg
      READ(*,*) line1
      READ(*,*) line2 
      READ(*,*) pix1    
      READ(*,*) pix2
      
      CALL ml_wti('info', 'FDFunit         :',FDFunit)  
      CALL ml_wti('info', 'AVH1Bunit       :',AVH1Bunit)
      CALL ml_wti('info', 'AVH1Cunit       :',AVH1Cunit)
      CALL ml_wti('info', 'AVHRRINDBGunit  :',DBGunit)
      CALL ml_wti('info', 'AVHRRINidbg     :',idbg)
      CALL ml_wti('info', 'LINE1idbg       :',line1)
      CALL ml_wti('info', 'LINE2idbg       :',line2) 
      CALL ml_wti('info', 'PIX1idbg        :',pix1)   
      CALL ml_wti('info', 'PIX2idbg        :',pix2 )       
      
!------------------------------------------------------------------------------
! DEFINE BIT NUMBERS (in "bits.h")
!------------------------------------------------------------------------------

! Convention used in 1B & 1C files is that an INTEGER*4 word has bits
! numbered 0-31, with bit 0 being the least significant bit.
! Some platforms take bit 31 as the LSB. Here we explicitly define
! the order of bits that we use, to keep the code portable.

      i = 1

      IF ( BTEST(i,0) ) THEN      !  bit 0 is LSB

         DO j = 0, 31
            bits(j)  =  j
         ENDDO  

      ELSEIF( BTEST(i,31) ) THEN      !  bit 31 is LSB

         DO j = 0, 31
            bits(j)  =  31 - j
         ENDDO  

      ELSE                            !  something wrong

         CALL ml_wt('error',
     &   'Bit problem: BTEST(1,0) and BTEST(1,31) are both FALSE')
         CALL aappexit(4)

      ENDIF
 
!------------------------------------------------------------------------------
! VARIOUS TESTS
!------------------------------------------------------------------------------     
      
      IF (avh_mxpix .NE. avh1c_mxpix) THEN
        CALL ml_wt('error',
     &     'Not same value for avh_mxpix and avh1c_mxpix')
        CALL aappexit(3)
      ENDIF
      
      IF (avh_mxhrpchn .NE. avh1c_mxhrpchn) THEN
        CALL ml_wt('error',
     &     'Not same value for avh_mxhrpchn and avh1c_mxhrpchn')
        CALL aappexit(3)
      ENDIF
      
      IF (avh_mxirchn .NE. avh1c_mxirchn) THEN
        CALL ml_wt('error',
     &     'Not same value for avh_mxirchn and avh1c_mxirchn')
        CALL aappexit(3)
      ENDIF
      
      IF (avh_mxvischn .NE. avh1c_mxvischn) THEN
        CALL ml_wt('error',
     &     'Not same value for avh_mxvischn and avh1c_mxvischn')
        CALL aappexit(3)
      ENDIF  
      
!------------------------------------------------------------------------------
! READ THE FIXED DATA FILE
!------------------------------------------------------------------------------
     
      CALL ml_wti('info','READ the fixed data file: UNIT=',FDFunit)
      CALL infdf(FDFunit, 0)     ! Don't read antenna efficiencies until later
       
!------------------------------------------------------------------------------
! OPEN THE INPUT AND OUTPUT FILES
!------------------------------------------------------------------------------
      AVH1Brecord=0
      AVH1Crecord=0
      CALL ioavh1b("OPEN/READ",AVH1Bunit,AVH1Brecord)
      CALL ioavh1c1d("OPEN/WRITE",AVH1Cunit,AVH1Crecord)
                 
!------------------------------------------------------------------------------
! READ THE HEADER of THE INPUT FILE
!------------------------------------------------------------------------------        
      AVH1Brecord = 1            ! Headers start on record 1.
      CALL ioavh1b("READ HEADER",AVH1Bunit,AVH1Brecord)

      IF (idbg.GE.2) THEN
        CALL ml_wti('info', 'PRINT hrpt/avhrr header l1b, idbg =',idbg)
        CALL avh1bhdp(DBGunit)
      ENDIF  
      

      !Is the 1B data AVHRR ?
      !"avh_h_datatyp" from 1B header should be 1,2,3 for AVHRR
      ! (1 = LAC; 2 = GAC; 3 = HRPT)
      IF ( avh_h_datatyp .LT. 0 .AND. avh_h_datatyp .GT. 3 ) THEN
         CALL ml_wt('error','Data type in input file not AVHRR')
         CALL ml_wti('error','      avh_h_datatyp:', INT(avh_h_datatyp))       
         CALL aappexit(3)
      ENDIF
      

!------------------------------------------------------------------------------
! SET UP THE HEADER of THE OUTPUT FILE
!------------------------------------------------------------------------------
      
      !Is the 1C include file compatible with this source code ?      
      IF ( avh1c_vers_code .NE. avh1c_vers ) THEN
        CALL ml_wti('error','Code expects 1C format :',avh1c_vers_code)
        CALL ml_wti('error','But avh1c.h :',avh1c_vers)
        CALL aappexit(3) 
      ENDIF        

      !Convert satellite id from NESDIS 1B code to e.g. 14 for NOAA-14
      !MaxSats defined in inparms.h
      !NESDISid(.) =   4 for NOAA-15 for example
      !NOAAid(.)   =  15 for NOAA-15 for example 
      !WMOid(.)    = 206 for NOAA-15 for example
      sat_index = 0
      DO isat = 1, MaxSats
         IF ( avh_h_satid .EQ. NESDISid(isat) ) THEN
            avh1c_h_satid = NOAAid(isat)
            avh1c_h_wmosatid = WMOid(isat)
            sat_index = isat
         ENDIF
      ENDDO  ! isat
      
      IF ( sat_index .EQ. 0 ) THEN
        CALL ml_wti('error','1B satid not recognised:',INT(avh_h_satid))
        CALL aappexit(3) 
      ENDIF

      avh1c_h_site='XXX'
      READ(avh_h_dataname,"(a3)",iostat=ios) avh1c_h_site    !cms for example
      avh1c_h_cfill1      = " "
      avh1c_h_1bsite      = avh_h_siteid                   
      avh1c_h_cfill2      = " "         
      avh1c_h_versnb      = avh1c_vers
      avh1c_h_versyr      = avh1c_versyr                        
      avh1c_h_versdy      = avh1c_versdy
      avh1c_h_hdrcnt      = avh1c_header_nb      
      avh1c_h_instrument  = avh_h_instid
      
      !SatHt and OrbitPd defined in inparms.h     
      avh1c_h_satht  = NINT(SatHt(sat_index)*10.) !nominal sat height, km*10     
      avh1c_h_period = NINT(OrbitPd(sat_index))   !nominal orbit period (seconds)
     
!     !dataname = cms.HIRX.NP.D09063.S0038.E0046.B0036161.WE
!     !32 1ers caracteres :cms.HIRX.NP.D09063.S0038.E0046.B
!     !35 1ers caracteres :cms.HIRX.NP.D09063.S0038.E0046.B003
!     !35 1ers caracteres :cms.HIRX.NP.D09063.S0038.E0046.B00361
      avh1c_h_startorbit = 0
      sttorbit = 0
      endorbit = 0
      READ(avh_h_dataname,"(32X,I5)",iostat=ios) avh1c_h_startorbit !orbit number at start
      READ(avh_h_dataname,"(35X,I2)",iostat=ios) sttorbit ! last 2 digits, start orbit no. 
      READ(avh_h_dataname,"(37X,I2)",iostat=ios) endorbit ! last 2 digits, final orbit no.
      
      avh1c_h_endorbit = endorbit + 100*(avh1c_h_startorbit/100)
      IF ( sttorbit.GT.endorbit )
     &     avh1c_h_endorbit = avh1c_h_endorbit + 100     
      
         
      avh1c_h_startdatayr = avh_h_startdatayr
      avh1c_h_startdatady = avh_h_startdatady
      avh1c_h_startdatatime = avh_h_startdatatime
   
      avh1c_h_enddatayr  = avh_h_enddatayr
      avh1c_h_enddatady  = avh_h_enddatady 
      avh1c_h_enddatatime  = avh_h_enddatatime
      avh1c_h_scnlin       = avh_h_scnlin 
      avh1c_h_misscnlin    = avh_h_misscnlin
      avh1c_h_callocscnlin = avh_h_callocscnlin 
                                  
      avh1c_h_datagaps = avh_h_datagaps   
      avh1c_h_datatyp  = avh_h_datatyp
      
      avh1c_h_infilename = avh_h_dataname
       
      avh1c_h_cpidsyr = avh_h_cpidsyr
      avh1c_h_cpidsdy = avh_h_cpidsdy
      
      avh1c_h_rollerr  = avh_h_rollerr
      avh1c_h_pitcherr = avh_h_pitcherr
      avh1c_h_yawerr   = avh_h_yawerr   

      DO j=1,avh_mxvischn
        DO i=1,2
         avh1c_h_albcnv(i,j) = avh_h_albcnv(i,j)
        ENDDO
      ENDDO
 
      DO j=1,avh_mxirchn
        DO i=1,3
         avh1c_h_radtempcnv(i,j) = avh_h_radtempcnv(i,j)
        ENDDO
      ENDDO     
 
      DO i=1,7388
        avh1c_h_filler(i)=0
      ENDDO  
      
!----------------------------------------------------------------------------
! WRITE OUT THE HEADER OF THE AVHRR L1C FILE 
!----------------------------------------------------------------------------  
      
      AVH1Crecord = 1
      CALL ioavh1c1d("WRITE HEADER",AVH1Cunit,AVH1Crecord)
      
      !Print header level1c
      
      IF (idbg.GE.2) THEN
        CALL ml_wti('info', 'PRINT hrpt/avhrr header l1c, idbg =',idbg)
        CALL avh1chdp(DBGunit)
      ENDIF    
            
!----------------------------------------------------------------------------
! LOOP THROUGH SCAN LINES, READING, APPLIING CALIBRATION COEFFIIENTS, 
! WRITING INTO THE OUTPUT FILE
!----------------------------------------------------------------------------

      DO AVH1Brecord = 1+avh_h_hdrcnt,avh_h_scnlin+avh_h_hdrcnt
                  
        !Read 1 record
        CALL ioavh1b("READ DATA",AVH1Bunit,AVH1Brecord)
                
        line_read = AVH1Brecord - avh_h_hdrcnt
                        
        !Print  level1b data record if requested
        IF (idbg.GE.2) THEN
          IF (line_read.GE.line1 .and. line_read.LE.line2) THEN
            CALL ml_wti('info',
     &      'PRINT AVHRR l1b line,line_read=',line_read)
            CALL avh1bdtp(DBGunit,line_read)
          ENDIF
        ENDIF 
        
        ! line-read input,  others output  
        
        CALL avh_lbc(line_read, 
     &               avh_rad,
     &               avh_bt,
     &               avh_qc,
     &               avh_bitqc,
     &               missing,
     &               good_lines,
     &               bad_lines)
                             
                                         
        !For problems when converting numerical counts into radiances and BTs
        IF (avh_qc(1) .NE. 0) THEN
          CALL ml_wti('info','avh_lbc bad line: ',line_read)
          CALL MVBITS(1,0,1,avh_scnlinqual,31)
        ENDIF
         
        !Set values for avhr1c record
        avh1c_scnlin     =      line_read
        avh1c_scnlinyr   =      avh_scnlinyr
        avh1c_scnlindy   =      avh_scnlindy
        avh1c_scnlintime =      avh_scnlintime  
        avh1c_clockdrift =      avh_clockdrift
        
        
        ! convert int16 to int32  (problem with values for descending passes (bit15=1)
        avh1c_scnlinbit=0 
        do ib=0,15    
          avh1c_scnlinbit = avh1c_scnlinbit + 
     &               2**(ib) * ibits(avh_scnlinbit,ib,1)

        enddo
        
        avh1c_qualind    =      avh_qualind
        avh1c_scnlinqual =      avh_scnlinqual
        
        DO i=1,avh1c_mxirchn
          avh1c_nedt(i)  =      avh_filler3(i)/256
        ENDDO

        DO i=1,avh1c_mxirchn
          avh1c_calqual(i)  =   avh_calqual(i)
        ENDDO
          
        avh1c_scalti=   avh_scalti
        
        DO j=1,51
          DO i=1,3
           avh1c_ang(i,j) =     avh_ang(i,j)
          ENDDO
          DO i=1,2 
           avh1c_pos(i,j) =     avh_pos(i,j)
          ENDDO
        ENDDO   
    
        !In K*100 
        DO j=1,avh1c_mxpix
          DO i=1,avh1c_mxhrpchn
           avh1c_btemps(i,j)=   nint(avh_bt(i,j)*100.)
          ENDDO
          avh1c_maskind = 0
          avh1c_mask(j) = 0
          avh1c_tsurf(j)= 0
        ENDDO 
        
        DO i=1,2
          avh1c_spare0(i) = 0
          avh1c_spare1(i) = 0
          avh1c_spare2(i) = 0
        ENDDO
        
        DO i=1,7
          avh1c_spare3(i) = 0
        ENDDO     
        
        !Write out avh1c data record
            
        AVH1Crecord= avh1c_h_hdrcnt + (AVH1Brecord-avh_h_hdrcnt)
        CALL ioavh1c1d("WRITE DATA",AVH1Cunit,AVH1Crecord)
        
        !Print  level1c data record if requested
        IF (idbg.GE.2) THEN
          line_write=AVH1Crecord - avh1c_h_hdrcnt
          IF (line_write.GE.line1 .and. line_write.LE.line2) THEN
            CALL ml_wti('info', 'PRINT AVHRR l1c line, line_write=',
     &                  line_write)
            CALL avh1cdtp(DBGunit,line_write,pix1,pix2)
          ENDIF
        ENDIF    
      
      ENDDO             !AVH1Brecord = 1+avh_h_hdrcnt,avh_h_scnlin
     

!-----------------
! CLOSE THE FILES 
!-----------------
      AVH1Brecord=1
      AVH1Crecord=1
      CALL ioavh1b("CLOSE",AVH1Bunit,AVH1Brecord)
      CALL ioavh1c1d("CLOSE",AVH1Cunit,AVH1Crecord)

      CALL ml_wt('info','END of AVHRRIN.EXE')


      STOP
      END         
      
