#! /bin/ksh
###########################################################################
#   $Id: avhrrin.ksh 530 2017-06-27 11:07:32Z frna $
#   (C) NWP SAF
###########################################################################
#
# NOM/NAME  
#  avhrrin.ksh 
# 
# DESCRIPTION 
#  Korn Shell to run the main program avhrrin.exe
#  Process HRPT/AVHRR level 1b  to AVHRR level 1c (AAPP formats)
#  Apply calibration coefficients to AVHRR numeric counts to get reflectances and
#  brightness temperatures.
#
# USAGE
#  avhrrin [options]
#       Options:   -i file_name1 : full pathname of the input hrpt/avhrr 1b file
#                                  (default $WRK/hrpt.l1b) 
#	           -o file_name2 : full pathname of the output hrpt/avhrr 1c file
#                                  (default $WRK/avhrr.l1c) 
#
# ENVIRONMENT VARIABLES must be defined outside this script (no default value)
#   (See ATOVS_ENV6)
#   AAPP_PREFIX
#   WRK
#   FORT
#   DIR_PREPROC for fdf.dat
#   FDFunit
#   AVH1Bunit
#   AVH1Cunit
#   AVHRRINDBGunit
#   AVHRRINidbg
#   LINE1idbg
#   LINE2idbg
#   PIX1idbg
#   PIX2idbg
#
# EXIT CODE
#   0  normal end
#   1  bad input parameters, input data, usage,...
#   2  bad output code for avhrrin.exe
#  
# PROPRIETAIRE/OWNER  
#  NWP SAF
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0      May 09     initial         Tiphaine Labrot (MF)
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2005, EUMETSAT, All Rights Reserved.
#
############################################################################

#set -x -o xtrace

#-------------------------------
# Parameters
#-------------------------------
if [ "$AAPP_PREFIX" = "" ]; then
  echo 'Error: AAPP_PREFIX not defined'
  exit 1
fi
if [ ! -d "$AAPP_PREFIX" ]; then
  echo 'Error: AAPP_PREFIX is not a directory:' $AAPP_PREFIX
  exit 1
fi
if [ ! -s "$AAPP_PREFIX/ATOVS_CONF" ]; then
  echo 'Error: $AAPP_PREFIX/ATOVS_CONF not existed or empty' 
  exit 1
fi

. "$AAPP_PREFIX/ATOVS_CONF"

#-------------------------------
# External functions
#-------------------------------

. liblog.sh

log_info "AVHRRIN"

#-------------------------------
# Usage
#-------------------------------

usage(){
	echo 'usage : avhrrin [options]'
	echo '   -i file_name1 : full pathname of the input hrpt/avhrr 1b file'
	echo '                   (default $WRK/hrpt.l1b)' 
	echo '   -o file_name2 : full pathname of the output hrpt/avhrr 1c file'
        echo '                   (default $WRK/avhrr.l1c)' 
	}
	
#-------------------------------	
#  Get the arguments
#-------------------------------

while getopts i:o: c
do
  case $c in
     i)	IFILE=$OPTARG;;
     o)	OFILE=$OPTARG;;
     \?)  usage
          exit 1;;
  esac
done
	
# Default values	
IFILE=${IFILE:-$WRK/hrpt.l1b}
OFILE=${OFILE:-$WRK/avhrr.l1c}

log_info "Input  file: ${IFILE}"
log_info "Output file: ${OFILE}"

#------------------------
# Verify the input file 
#------------------------
ROOT_IFILE=`echo ${IFILE} | cut -c 1-1`
if [ ! "${ROOT_IFILE}" = '/' ] ; then
  log_error "File should be given with a full path name"
  exit 1
fi
if [ ! -s ${IFILE} ]; then
  log_error "Nonexistent or empty file ${IFILE}"
  exit 1
fi	

#------------------------------
# Processing
#------------------------------

log_info "Start of processing ${IFILE}"

cd ${WRK}
log_info "Working directory: ${WRK}"

rm -f ${FORT}*
rm -f avhrrin.log
rm -f avhrrin.dbg
rm -f ${OFILE}

#------------------------------
# Assign files to unit numbers
#------------------------------

# Point to fixed data files
ln -s ${DIR_PREPROC}/fdf.dat ${FORT}${FDFunit}

# Point to HRPT/AVHRR level 1b input file
ln -s ${IFILE} ${FORT}${AVH1Bunit}

# Point to AVHRR level 1c output file
touch ${OFILE}
ln -s ${OFILE} ${FORT}${AVH1Cunit}

# if dbg on, create the file where the AVHHRR data will be written. 

if [ ${AVHRRINidbg} -ge 1 ] ; then
 touch avhrrin.dbg
 ln -s avhrrin.dbg ${FORT}${AVHRRINDBGunit}
fi

#-------------------------------------
# Convert HRPT/AVHRR l1b to l1c file
# Run avhrrin.exe
#-------------------------------------

log_info FDFunit:${FDFunit}
log_info AVH1Bunit:${AVH1Bunit}
log_info AVH1Cunit:${AVH1Cunit}
log_info AVHRRINDBGunit:${AVHRRINDBGunit}
log_info AVHRRINidbg:${AVHRRINidbg}
log_info LINE1idbg:${LINE1idbg}
log_info LINE2idbg:${LINE2idbg}
log_info PIX1idbg:${PIX1idbg}
log_info PIX2idbg:${PIX2idbg} 

log_info "Call to avhrrin.exe"

avhrrin.exe << EOF > avhrrin.log 2>&1
${FDFunit}
${AVH1Bunit}
${AVH1Cunit}
${AVHRRINDBGunit}
${AVHRRINidbg}
${LINE1idbg}
${LINE2idbg}
${PIX1idbg}
${PIX2idbg}
EOF
crdu=$?

# Check error code
if [ ${crdu} -ne 0 ] ; then
 log_error "Return code of avhrrin.exe: ${crdu}"
 exit 2
fi

log_info "After call to avhrrin.exe"


#-------------------------
# Move/remove/clean up 
#-------------------------
rm -f ${FORT}*

#-------------------------------------------------------------------------------
# conclusion
#-------------------------------------------------------------------------------
log_info "End of AVHRRIN processing ${IFILE}"
exit 0
