#!/bin/ksh
###########################################################################
#  (C) NWP SAF
###########################################################################
#
# NOM/NAME  
#  maia4.ksh 
# 
# DESCRIPTION 
#  Korn Shell to run the main program maia4_Virrs.exe
#  Process the  MAIA v4 cloud mask for VIIRS data.
#  Input files : VIIRS SDR files
#  Output file: viiCT file
#  
#
# USAGE
#      maia4 GMODO/GMTCO_file
#
#     maia4 expects to find the other SDR files (SVI, SVM, GITCO or GIMCO)
#     in the same directories than the GMODO/GMTCO file
#
# ENVIRONMENT VARIABLES must be defined outside this script (no default value)
#
# DIR_MAIA4_THRESHOLDS << directory of the MAIA4 thresholds >>
# DIR_MAIA4_ATLAS <<directory of the MAIA4 atlas + topography files >>
# PAR_MAIA4_COMPRESS (0 no compression 1 compression)
# 
# MAP_BOX_PSIZE                # box size for environment in pixel
# MAP_BOX_LSIZE                # box size for environment in line 
# LOCAL_BOX_NPB                 # local box size for variance in pixel
# LOCAL_BOX_NLB                 # local box size for variance in line
#
#
# EXIT CODE
#   0  normal end
#      when the cloud mask cannot be done (somme channel are missing) exit 0 with a warning
#   2  error
#  
# PROPRIETAIRE/OWNER  
#  NWP SAF
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     2012       initial          L. Lavanant (MF)
#  1.1    Dec 2012    process exit cases         P. Roquet (MF)
#  1.2    Dec 2017    update for NOAA20          P. Roquet (MF)
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2005, EUMETSAT, All Rights Reserved.
#
############################################################################			
#
# *****************************************************************************
#=====================================================================================
function maia_Viirs {
#=====================================================================================

log_info "start maia_Viirs"

rm -f viirs_* virrsI_* virrsloc virrsloI VIS*txt maiaCT_out.h5
rm -f fort*

log_info "Start processing:"
log_info "`basename ${FILE_1B}`"



# initialisation
#------------------------------------------------------
liste_can='01 04 05 07 09 10 11 12 13 14 15 16'
liste_Ican='1 2 3 4 5'
FILES_ID='FILES_ID.txt'
rm -f ${FILES_ID}


# satellite and datation of the sdr file
#------------------------------------------------------
dir_sdr=`dirname ${FILE_1B}`
file=`basename ${FILE_1B}`
file_deb=`echo ${file} | awk '{print substr($0,11,27)}'`;

set `basename ${FILE_1B} | tr '_' ' ' `
SATNAME=`echo $2`
echo "SATNAME=$SATNAME"
case ${SATNAME} in
npp) SATIMG="npp";;
j01) SATIMG="noaa20";;
j02) SATIMG="noaa21";;
esac
YYYYMMDD=`echo $3 | awk '{print substr($0,2,8)}'`;
YYYYMMDD_Deb=${YYYYMMDD}
YYYYMMDD_End=${YYYYMMDD}

HHMNSS_start=`echo $4 | awk '{print substr($0,2,6)}'`;
HHMNSS_end=`echo $5 | awk '{print substr($0,2,6)}'`;
HHMN_Deb=`echo ${HHMNSS_start} | awk '{print substr($0,1,4)}'`;
HHMN_End=`echo ${HHMNSS_end} | awk '{print substr($0,1,4)}'`;
Name_Deb=`echo $1 | awk '{print substr($0,1,5)}'`;

if [ ${HHMN_End} -lt ${HHMN_Deb} ]
then
#  we have to change the date
 YYYYMMDD_End=`date_pn  ${YYYYMMDD_End} 1`
 log_info "YYYYMMDD_End ${YYYYMMDD_End}"
fi
case "${Name_Deb}" in
  'GMODO' ) Name_Deb_I='GIMGO' ;;
  'GMTCO' ) Name_Deb_I='GITCO' ;;
   *      ) log_error 'UNKNOWN Name_Deb: ' ${Name_Deb};
            exit ;;
esac

# links the sdr files
#------------------------------------------------------
for FIC_geo in `ls ${dir_sdr}/${Name_Deb}_${SATNAME}_d${YYYYMMDD}*_t${HHMNSS_start}*_e${HHMNSS_end}*_*h5 2>/dev/null`
do
      if [ ! -f virrsloc  ] ; then
         ln -s ${FIC_geo} 	virrsloc
      fi
      echo `basename ${FIC_geo}` 	> ${FILES_ID}
done

for ican in $liste_can
do
   for FIC in `ls ${dir_sdr}/SVM${ican}_${SATNAME}_d${YYYYMMDD}*_t${HHMNSS_start}*_e${HHMNSS_end}*_*h5 2>/dev/null`
   do
      if [ ! -f virrs_${ican}  ] ; then
         ln -s ${FIC} 		virrs_${ican}
      fi
   done
done

for ican in $liste_Ican
do
   for FIC in `ls ${dir_sdr}/SVI0${ican}_${SATNAME}_d${YYYYMMDD}*_t${HHMNSS_start}*_e${HHMNSS_end}*_*h5 2>/dev/null`
   do
      if [ ! -f virrsI_${ican}  ] ; then
         ln -s ${FIC} 		virrsI_${ican}
      fi
   done
done

for FIC in `ls ${dir_sdr}/${Name_Deb_I}_${SATNAME}_d${YYYYMMDD}*_t${HHMNSS_start}*_e${HHMNSS_end}*_*h5 2>/dev/null`
do
      if [ ! -f virrsloI  ] ; then
         ln -s ${FIC} 		virrsloI
      fi
done


# Runs MAIA
#------------------------------------------------------
rm -f file_CT_name.txt
IDBG_MAIA=${PAR_MAIA4_IDBG:-0}
COMPRESS_MAIA=${PAR_MAIA4_COMPRESS:-0}

ERR=-99
if [ -f virrs_16  ] ; then

maia4_env 

if [ ${MAIA4_ENV} != 'OK' ] ; then
   log_error in maia4_env. Mask cannot be done
   remove_maia4_env
   exit 1
fi

if [ ${MAIA4_ENV} == 'OK' ] ; then
   log_info "OK MAIA4_ENV ${file_hdf_name}"
   ln -s ${file_hdf_name}	fort.16
   maia_Viirs.exe <<EOF
${IDBG_MAIA}
${COMPRESS_MAIA}
EOF

   ERR=$?
   if [ "${ERR}" != 0 ] ; then
      log_error maia_Viirs.exe : output in error ${ERR}
      remove_maia4_env
      exit ${ERR}
   fi
fi

remove_maia4_env
	

fi

rm -f virrs_* virrsI_* virrsloc virrsloI VIS*txt
rm -f ${FILES_ID}


}


usage()
{
 echo "usage"
}

. liblog.sh


#------------------------------------------------------
# get the arguments
#------------------------------------------------------
while getopts S:s:d:h:n: c
do
    case $c in
      S) SOURCE=${OPTARG};;
      s) SATIMG=$OPTARG ;;
      d) YYYYMMDD=$OPTARG ;;
      h) HHMN=$OPTARG ;;
      n) NNNNN=$OPTARG ;;
      *) usage
         exit 2;;
    esac
done

shift `expr $OPTIND - 1`



file_source=$1
#--------------------------------------------------------
# Cloud Mask maia
#--------------------------------------------------------
file_CT=maiaCT_out.h5
file_hdf_name=file_CT_name.txt


rm -f ${file_hdf_name}
rm -f ${file_CT}

. maia_env
. maia_date


FILE_1B=${file_source}
maia_Viirs
 
if [ -f "$file_hdf_name" ]
then
 viiCT_file_name=`cat $file_hdf_name`
 mv maiaCT_out.h5 ${viiCT_file_name}
 rm fort.16 ${file_hdf_name}
 log_info "Cloud Mask file : ${viiCT_file_name}"
 ls -l ${viiCT_file_name}
else
 log_warning "Cloud Mask has not been made" 
 rm fort.16
fi 
exit 0
