#!/bin/ksh
###########################################################################
#  (C) NWP SAF
###########################################################################
#
# NOM/NAME  
#  maia4.ksh 
# 
# DESCRIPTION 
#  Korn Shell to run the main program maia4_Avhrr.exe
#  Process the  MAIA v4 cloud mask for VIIRS data.
#  Input files : avhl1c hdf5 files
#  Output file: avhCT file
#  
#
# USAGE
#      maia4 avhl1c_hdf5_file
#
#     
#
# ENVIRONMENT VARIABLES must be defined outside this script (no default value)
#
# DIR_MAIA4_THRESHOLDS << directory of the MAIA4 thresholds >>
# DIR_MAIA4_ATLAS <<directory of the MAIA4 atlas + topography files >>
# PAR_MAIA4_COMPRESS (0 no compression 1 compression)
# 
# MAP_BOX_PSIZE                # box size for environment in pixel
# MAP_BOX_LSIZE                # box size for environment in line 
# LOCAL_BOX_NPB                 # local box size for variance in pixel
# LOCAL_BOX_NLB                 # local box size for variance in line
#
#
# EXIT CODE
#   0  normal end
#      when the cloud mask cannot be done (somme channel are missing) exit 0 with a warning
#   2  error
#  
# PROPRIETAIRE/OWNER  
#  NWP SAF
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment              <author>)
#  1.0     2015        initial              L. Lavanant (MF)
#  1.1     10/03/2017  adaptation for AAPP  P. Roquet (MF)
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2005, EUMETSAT, All Rights Reserved.
#
############################################################################                    
#
# *****************************************************************************

#=====================================================================================
function maia_Avhrr {
#=====================================================================================

# to run MAIAv4 cloud mask for NOAA/METOP AVHRR data in H5

log_info "Start maia_Avhrr"

rm -f fort.*
rm -f maia*.h5

log_info "`basename ${FILE_1C}`"

# datation of the AVHRR level 1c file
#------------------------------------
YYYY=`get_attribute_h5 startdatayr ${FILE_1C}`
DAY=`get_attribute_h5 startdatady ${FILE_1C}`
TIME=`get_attribute_h5 startdatatime_ms ${FILE_1C}`
MMDD=`date_MMDD ${YYYY} ${DAY}`
HHMN=`date_HHMN ${TIME}`
YYYYMMDD_Deb=${YYYY}${MMDD}
HHMN_Deb=${HHMN}

echo ${YYYYMMDD_Deb}_${HHMN_Deb}

YYYY=`get_attribute_h5 enddatayr ${FILE_1C}`
DAY=`get_attribute_h5 enddatady ${FILE_1C}`
TIME=`get_attribute_h5 enddatatime_ms ${FILE_1C}`
MMDD=`date_MMDD ${YYYY} ${DAY}`
HHMN=`date_HHMN ${TIME}`
YYYYMMDD_End=${YYYY}${MMDD}
HHMN_End=${HHMN}

echo ${YYYYMMDD_End}_${HHMN_End}


#Verify the full pathname of the file
#-----------------------------------------
ROOT_FILE=`echo ${FILE_1C} | cut -c 1-1`
if [ ! "${ROOT_FILE}" = '/' ] ; then
  log_error "file should be given with a full path name"
  exit 1
fi

log_info ${SATIMG} ${NNNNN}
case ${SATIMG} in
metop01|M01) SAT=M01;;
metop02|M02) SAT=M02;;
metop03|M03) SAT=M03;;
noaa16|N16) SAT=N16;;
noaa17|N17) SAT=N17;;
noaa18|N18) SAT=N18;;
noaa19|N19) SAT=N19;;
esac
rm -f fort.16

# Maia parametrization
#-----------------------
echo
echo MAIA_AVHRR

rm -f fort.*

maia4_env

if [ ! -s Bg_BeforObs -o ! -s Bg_AfterObs ]
then
  log_error "forecast Before or After not available"
  log_error "exiting maia4_avhrr"
  remove_maia4_env
  exit 1
fi
ls -l  Bg_BeforObs
ls -l  Bg_AfterObs
#------------------
# Runs MAIA
#------------------
log_info "run maia"

ln -s ${file_hdf_name}  fort.16
ln -s ${FILE_1C}        avh1c.h5

IDBG_MAIA=${PAR_MAIA4_IDBG:-0}
COMPRESS_MAIA=${PAR_MAIA4_COMPRESS:-1}

/usr/bin/time --verbose maia_Avhrr.exe  << EOF
${IDBG_MAIA}
${COMPRESS_MAIA}
EOF

crdu=$?
if [ ${crdu} -ne 0 ] ; then
 log_error "output in error of maia_Avhr.exe ${crdu}"
 remove_maia4_env
 exit  ${crdu}
fi
	
# remove files
#-------------
remove_maia4_env

rm -f fort.*
rm -f  avh1c.h5

log_info "end of maia_Avhrr"

}


usage()
{
 echo "usage"
}

. liblog.sh


#------------------------------------------------------
# get the arguments
#------------------------------------------------------
OUTPUT=""
while getopts s:d:h:n:o: c
do
    case $c in
      s) SATIMG=$OPTARG ;;
      d) YYYYMMDD=$OPTARG ;;
      h) HHMN=$OPTARG ;;
      n) NNNNN=$OPTARG ;;
      o) OUTPUT=$OPTARG ;;
      *) usage
         exit 2;;
    esac
done

shift `expr $OPTIND - 1`

file_source=$1
log_info "source file: ${file_source}"
log_info "satimg: ${SATIMG}"
log_info "date: ${YYYYMMDD} ${HHMN} ${NNNNN}"
#--------------------------------------------------------
# Cloud Mask maia
#--------------------------------------------------------
file_CT=maiaCT_out.h5
file_hdf_name=file_CT_name.txt


rm -f ${file_hdf_name}
rm -f ${file_CT}

. ${AAPP_PREFIX}/AAPP/bin/maia_env
. ${AAPP_PREFIX}/AAPP/bin/maia_date


FILE_1C=${file_source}
log_info "satimg: ${SATIMG}"
maia_Avhrr

if [ ! -f "$file_hdf_name" ]
then
 log_error "Cloud Mask has not been made" 
fi 
read hdf_name < ${file_hdf_name}
mv  ${file_CT}  ${hdf_name}

rm ${file_hdf_name}

rm -f fort.*
if [ ! -z "${OUTPUT}" ]
then 
  mv ${hdf_name}  ${OUTPUT}
fi

exit 0
