! Name: $Id: maia_Viirs.F90 548 2017-08-30 09:28:19Z frna $
PROGRAM maia_Viirs
!
! LIBS =  maia4IO maia4 aapp_viirs f7ml f7tp f7cp f7gp atov auxaapp

!------------------------------------------------------------------------------      
!------------------------------------------------------------------------------
! Name:
!   maia_Viirs.F90
!
! Purpose:
!   Process the  MAIA v4 cloud mask for VIIRS data.
!
! Method:
!
!
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!  call maia_read_Viirs  (libmaia4IO)
!  maia_Analyse_field (libmaia4)
!  maia_VerifMissing_fields  (libmaia4)
!  mk_voisinage  (libmaia4)
!  maia_Box_reset  (libmaia4)
!  maia_Pixel_reset  (libmaia4)
!  maia_Fill_Input_Virrs (libmaia4)
!  maia maia_Write_ViiCT_hdf5 (libmaia4IO)
!
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
!  Version     Date      Comment                          Author
!  V 1.O 2012         created                           L. Lavanant     
!  V 1.1 Feb 2014 add control for missing channels 
!                 change format CloudMask
!                 use 0.01 degres landsea atlas      L. Lavanant
!  V 1.2 May 2015 modify control for pixel processing P. Roquet
!  V 1.3 May 2016 maia version 4.3 Lydie Lavanant
!  V 1.4 January 2016 maia version 4.4 Lydie Lavanant
!  V 1.5 mai 2017 maia_par size 30->31 (add T2M)  
!  V 1.6 august 2017 memory optimisation : maia_par is an array of maia_param type
!
! Code Description:
!   FORTRAN 90
!
!--DECLARATIONS----------------------------------------------------------------
#ifdef HAS_LIBHDF5

 USE MOD_MAIA_CONST
 USE mod_maia_types
IMPLICIT NONE

! maia routine arguments
  type( debug )         :: idbg         ! 
  type( box_id )        :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( pix_info)       :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data)       :: pixel        ! viirs observations (albedo in %, Tb in K)
  LOGICAL               :: new_box
  Type (field)          :: field_M
  Type (field)          :: field_I
  Type (field_info)     :: field_id
  type( topo_field)     :: landsea, elev        ! surface topography landsea and elev

! logical units, file name
  INTEGER, PARAMETER :: iuID = 12       ! unit for file
                           
  INTEGER :: ll, pp, ip, il, n, ic,pixI,ligI
  INTEGER :: ios=0, ios_ID=0
                      
  INTEGER :: idbg_maia                  ! debug level (0= nodebug, 2 is maximum)
  LOGICAL :: idbg_main
  LOGICAL :: missing_field= .true., missing_data= .true.
  LOGICAL :: compress
  INTEGER :: icompress
  INTEGER :: allocstatus  
  type(maia_param), allocatable    :: maia_par(:,:)    ! output arguments for MAIA subroutine
   
  real :: t10, t12                      ! pour test duree execution
!----------------------------------------------------------------- 
  call cpu_time(t10)
  compress=.true.
  ! Launch options
  read(*,*) idbg_maia
  read(*,*) icompress
  if ( icompress .eq. 0 ) compress = .false.

  idbg_main=.false.
  idbg % info=.false.
  idbg % debug=.false.
  if(idbg_maia == 1) idbg_main=.true.
  if(idbg_maia >= 1) idbg % info=.true.
  if(idbg_maia == 2) idbg % debug=.true.
  idbg %nbw     =0
  idbg %nbw_bg  =0

  write(*,*) '  --maia_Viirs maia V 4.5 -------------------------------------------'
  call ml_wti('INFO','idbg_maia',idbg_maia)
  call ml_wti('INFO','compress',icompress)
  open(iuID,file='FILES_ID.txt', status='old', iostat=ios_ID)
  if(ios /= 0) then
     call ml_wti('error','open FILES_ID ios',ios_ID)
     stop 2
  endif
  do while(ios_ID==0)
     read(iuID, *,iostat=ios_ID) field_M % dataname_gmodo
     if(ios_ID /=0) then
        exit
     endif
  end do
  call ml_wt('INFO',field_M % dataname_gmodo(1:44))

!-----------------------------
  ! lecture des champs data Viirs
  write(*,*) '  -----------------------------------------------------------------------'
  call ml_wt('INFO','Read Viirs fields')

  call maia_read_Viirs ( idbg, field_M,  field_I )
  if(idbg%debug) write(*,*) 'maia_read_Viirs out'
   
!-----------------------------
  ! analyse des champs data
  field_M% ImgName = 'viirs'
  call maia_Analyse_field (idbg, field_M, field_id )
  
  call maia_VerifMissing_fields (idbg, field_M, field_I, missing_field )
  if ( missing_field ) then
     write(*,*) '  -----------------------------------------------------------------------'
     call ml_wt('WARNING', 'some necessary input fields are missing. Cannot run Maia. EXIT 0')
     write(*,*) '  -----------------------------------------------------------------------'
     call exit
  endif
  
  write(*,*) '  ViirsField: '
  write(*,*) '  Nb_Lines M effective / I   :', field_M % NLines_filled, field_I % Nb_Lines
  write(*,*) '  LatMin LatMax LonMin LonMax:', field_id % LatM_min, field_id % LatM_max, &
                                               field_id % LonM_min, field_id % LonM_max
  write(*,*) '  nbLat_box, nbLon_box       :', field_id % nbLat_box, field_id % nbLon_box
  write(*,*) '  nb_pixels, nb_line         :', field_M % Nb_pixels, field_M % Nb_Lines
  ! reflectance channels 06 08 16mm
  do ic=1,3
  do ligI=1, field_I % Nb_Lines
  do pixI=1, field_I % Nb_pixels
     if(field_I % sunzen (pixI,ligI) > vmanq .and. field_I % sunzen(pixI,ligI)<90. .and. &
        field_I % Obs(ic,pixI,ligI)>0) then
        field_I % Obs(ic,pixI,ligI) = field_I % Obs(ic,pixI,ligI) * cos(cdr* field_I % sunzen(pixI,ligI) )
     endif
  end do
  end do
  end do
!-----------------------------
  ! Voisinage
!  if (field_I % Nb_Lines == imanq) then
  call ml_wt('INFO','Select Medium pixels NEIGHBORS')
  call mk_voisinage (idbg, field_M)

!-----------------------------
  ! read landsea + elevation atlases  
  write(*,*) '  -----------------------------------------------------------------------'
  call ml_wt('INFO','Read landsea & elevation atlases')
  call maia_Read_Topo (idbg, field_id, landsea, elev) 
     
! init output arguments for MAIA subroutine
  call maia_param_array_init( maia_par, field_M % Nb_pixels, field_M % Nb_Lines,allocstatus)
  if ( allocstatus /= 0 ) then
    call ml_wt('ERROR','not enough memory')
    STOP 10
  endif 
 
!-----------------------------
  ! boucle sur les boites et les pixels dans les boites
  write(*,*) '  -----------------------------------------------------------------------'
  call ml_wt('INFO','First Call to MAIA routine')
  write(*,*) '  -----------------------------------------------------------------------'
 
  do ll=1, field_id % nbLat_box
  if ( MOD(ll,50) == 0 ) call ml_wti('INFO','box_lat number',ll)
  do pp=1, field_id % nbLon_box

     if(field_id % nb_fov(pp,ll)>0) then
        new_box=.true.

        call maia_Box_reset ( idbg, box ) 
        call maia_Box_GetTopo ( idbg, pp, ll, field_id, field_M, landsea, elev, box)

        do n=1, field_id % nb_fov(pp,ll)

           ip=field_id % pix_box(n, pp, ll)
           il=field_id % lig_box(n, pp, ll)          

           call maia_Pixel_reset (idbg, pix_id, pixel ) 
           call maia_Fill_Input_Virrs (idbg, ip, il, field_M, field_I, pix_id, pixel, box)    

           if(pixel % usable) then

              call maia (idbg, new_box, landsea, elev, field_id, box, pix_id, pixel, maia_par(ip,il))
              new_box=.false. 
              missing_data= .false.
           else
              if (idbg % debug) then
                write(*,*) 'Pixel is not usable'
                call maia_Pr_InfoPix(pix_id, pixel)
              endif
           endif        !if(field_M % Obs(9,ip,il)> vmanq)
        end do          !n=1, nb_fov(pp,ll)
     endif              !if(nb_fov(pp,ll)>0)
  end do
  end do

!-----------------------------
  write(*,*) '  -----------------------------------------------------------------------'
  if ( missing_data ) then
     write(*,*) '  -----------------------------------------------------------------------'
     call ml_wt('WARNING', 'some data are missing  : cannot make viiCT file')
     write(*,*) '  -----------------------------------------------------------------------'
     stop
  else 
     call ml_wt('INFO','Write Maia HDF5 file')
     call maia_Write_ViiCT_hdf5 (idbg, field_id, field_M, field_I, maia_par,compress)
  endif 

!-----------------------------
  write(*,*) '  -----------------------------------------------------------------------'
  call cpu_time(t12)
  call ml_wtr('INFO','TOT_TIME',t12-t10)
            
#else

  write(0,'("viirs_maia4.exe has not been built with HDF-5 Fortran interface")')
  write(0,'("Please run configure with --external-libs=-Lxxx/lib -lhdf5hl_fortran")')
  write(0,'(" and --external-includes=-Lxxx/include")')
  write(0,'(" where xxx is the location of the HDF-5 library")')
  
#endif

END PROGRAM maia_Viirs
