#! /bin/ksh
#
# MAIA 4 environment 
#
# gets the forecast, climatology and threshold files
# creates the appropriate links with fortran files.
#
# Must be inserted just before the execution of main fortran code
# The user just needs to call function "maia4_env" with the input global variables. 
#
# The "remove_maia4_env" function removes temporary files. 
#
# owner 	: MF/DP/CMS/R&D
# Authors       : lydie lavanant
# date      	: 09/01/2012
#
# inputs global variables (not localy modified) :
# -----------------------------------------------
# SATIMG     satellite name (noaaxx, metop02, npp)
# YYYYMMDD   date of the orbit
# HHMN       time of the orbit
# 
# DIR_MAIA4_ATLAS : 	directory for ATLAS and topography files 
# DIR_MAIA4_THRESHOLDS: directory for MAIA4 IR and VIS thresholds
# DIR_FORECAST:		directory from which we search the forecast files
#                       filename is YYYYMM/YYYYMMDDHHMN.HHH
#       	        YYYYMMDDHHMN: time and date of start of forecast 
#       	        HHH: the time period
#       	        surface file: YYYYMM/YYYYMMDDHHMN.CST
# NFORPERDAY : 		number of forecasts per day (2 or 4) default is 4
#=====================================================================================
function get_attribute_h5 {
#=====================================================================================
   attribute=$1
   file=$2
   h5dump -a ${attribute} ${file} | grep ':' > toto.log ; read a value  < toto.log ; rm -f toto.log
   echo ${value}
}
#=====================================================================================
function get_forecast_file {
#=====================================================================================
  # input  : YYYYMMDD_Deb HHMN_Deb
  # input  : YYYYMMDD_End HHMN_End
  # output : FORECAST_DEB and FORECAST_END including date of orbit 
  #
  # First compute the date and time of the nearest end of forecast cycle 
  # Forecast and surface files are cat in ONE file
  # Grib Forecast name is of pattern YYYYMM/YYYYMMDDHHMN.HHH
  # try to look if const fields (h and lsm) are in a file named .CST or in the .ANA
  
  log_info "get_forecast_file ${YYYYMMDD_Deb} ${HHMN_Deb} ${YYYYMMDD_End} ${HHMN_End}"
  log_info "DIR_Bg: ${DIR_FORECAST}/${FORC_YYYYMM} "

  NFORPERDAY=${NFORPERDAY:=4}
  log_info "nb forecast per day = ${NFORPERDAY}" 
  rm -f forecast_deb forecast_end
  FORECAST_DEB='Bg_BeforObs'
  FORECAST_END='Bg_AfterObs'
  touch ${FORECAST_DEB} ; chmod u+w ${FORECAST_DEB}
  touch ${FORECAST_END} ; chmod u+w ${FORECAST_END}
  
  if [ "${NFORPERDAY}" = 2 ] ; then
    DATE_DEB=${YYYYMMDD_Deb}
    DATE_END=${YYYYMMDD_End}
    if [ "${HHMN_Deb}" -gt 1200 ] ; then
      HH_DEB=1200
    else
      HH_DEB=0000
    fi
    if [ "${HHMN_End}" -gt 1200 ] ; then
      HH_END=0000
      DATE_END=`date_pn ${YYYYMMDD_End} 1`
    else
      HH_END=1200
    fi
  else
    DATE_DEB=${YYYYMMDD_Deb}
    if [ "${HHMN_Deb}" -gt 1800 ] ; then
      HH_DEB=1800
    elif [ "${HHMN_Deb}" -gt 1200 ] ; then
      HH_DEB=1200
    elif [ "${HHMN_Deb}" -gt 0600 ] ; then
      HH_DEB=0600
    elif [ "${HHMN_Deb}" -gt 0000 ] ; then
      HH_DEB=0000
    else
      HH_DEB=0000
    fi
    DATE_END=${YYYYMMDD_End}
    if [ "${HHMN_End}" -gt 1800 ] ; then
      HH_END=0000
      DATE_END=`date_pn ${YYYYMMDD_End} 1`
    elif [ "${HHMN_End}" -gt 1200 ] ; then
      HH_END=1800
    elif [ "${HHMN_End}" -gt 0600 ] ; then
      HH_END=1200
    elif [ "${HHMN_End}" -gt 0000 ] ; then
      HH_END=0600
    else
      HH_END=0000
    fi
  fi
  log_info "DEB: look for GRIB valid for ${DATE_DEB} ${HH_DEB}"
  log_info "END: look for GRIB valid for ${DATE_END} ${HH_END}"
  #search the nearest forecast for the input date
  # add 6 by 6 hours duration starting with 006 
  FORC_DEB='hhhhh'
  ECH=000
  while [ ! -f ${FORC_DEB} ] && [ "${ECH}" -le 72 ]
  do
    ECH=`echo "${ECH}" | awk '{printf("%.3d", $1 + 6)}'`
    log_info "date_hour_phours ${DATE_DEB} ${HH_DEB} -${ECH}00"
    set `date_hour_phours ${DATE_DEB} ${HH_DEB} -${ECH}00`
    FORC_DATE=$1
    FORC_HH=$2
    log_info "FORC_DATE=${FORC_DATE} FORC_HH=${FORC_HH}"
    FORC_YYYYMM=`echo ${FORC_DATE} | cut -c1-6 `
    FORC_DEB=${DIR_FORECAST}/${FORC_YYYYMM}/${FORC_DATE}${FORC_HH}.${ECH}
    SURF_DEB=${DIR_FORECAST}/${FORC_YYYYMM}/${FORC_DATE}0000.CST
    ANA_DEB=${DIR_FORECAST}/${FORC_YYYYMM}/${FORC_DATE}0000.ANA
    log_info "seek ${FORC_DEB}"
  done

  FORC_END='hhhhh'
  ECH=000
  while [ ! -f ${FORC_END} ] && [ "${ECH}" -le 72 ]
  do
    ECH=`echo "${ECH}" | awk '{printf("%.3d", $1 + 6)}'`
    set `date_hour_phours ${DATE_END} ${HH_END} -${ECH}00`
    FORC_DATE=$1
    FORC_HH=$2
    FORC_YYYYMM=`echo ${FORC_DATE} | cut -c1-6 `
    FORC_END=${DIR_FORECAST}/${FORC_YYYYMM}/${FORC_DATE}${FORC_HH}.${ECH}
    SURF_END=${DIR_FORECAST}/${FORC_YYYYMM}/${FORC_DATE}0000.CST
  done
  
  log_info "found:"
  val_grib=`grib_valid_date ${FORC_DEB}`
  log_info "FORC_DEB:${FORC_DEB}  (${val_grib})"
  log_info "SURF_DEB:${SURF_DEB}"
  
  if [ ! -f ${FORC_DEB}  ] ; then
    log_warning " Forecast deb is empty or not available"
    FORECAST_DEB=/dev/null
  else
    log_info "Bg_BeforObs: `basename ${FORC_DEB}` "
    cp ${FORC_DEB} ${FORECAST_DEB}
  fi
  val_grib=`grib_valid_date ${FORC_END}`
  log_info "FORC_END:${FORC_END}  (${val_grib})"
  log_info "SURF_END:${SURF_END}"
  if [ ! -f ${FORC_END} ] ; then
    log_warning " Forecast end is empty or not available"
    FORECAST_END=/dev/null
  else
    log_info "Bg_AfterObs: `basename ${FORC_END}` "
    cp ${FORC_END} ${FORECAST_END}
  fi

# append constant fields
if [ -f  ${SURF_DEB} ] ; then
  # append the CST grib to forecast
  cat ${SURF_DEB} >> ${FORECAST_DEB}
else 
  # extract h and lsm from ANA file if exist
   if [ -f  ${ANA_DEB} ] ; then
     rm -f h_lsm
     log_info "extract h/lsm fields from ANA"
     grib_copy -w shortName=h/lsm ${ANA_DEB} h_lsm
     cat h_lsm >>  ${FORECAST_DEB}
   fi
fi

if [ -f  ${SURF_END} ] ; then
  # append the CST grib to forecast
  cat ${SURF_END} >> ${FORECAST_END}
else 
  # take h_lsm if exist
  if [ -f h_lsm ] ; then
    cat h_lsm >>  ${FORECAST_END}
    rm h_lsm
  fi
fi
}
# end of function get_forecast_file
#=====================================================================================
function get_forecast_file_gfs_cspp {
#=====================================================================================
  # input  : YYYYMMDD_Deb HHMN_Deb
  # input  : YYYYMMDD_End HHMN_End
  # output : FORECAST_DEB and FORECAST_END including date of orbit 
  #
  # First compute the date and time of the nearest end of forecast cycle 
  # Forecast and surface files are cat in ONE file
  # surface file is assumed to be constant 
  # Grib Forecast name is of pattern YYYY_MM_DD_CCC/gfs.press_gr.0p5deg_pt.YYYYMMDD_HH_hhh/npoess.grib2
  
  log_info "get_forecast_file_gfs_cspp ${YYYYMMDD_Deb} ${HHMN_Deb} ${YYYYMMDD_End} ${HHMN_End}"
  log_info "DIR_Bg: ${DIR_FORECAST}/${FORC_YYYYMM} "

  NFORPERDAY=4
    
  rm -f forecast_deb forecast_end
  FORECAST_DEB='Bg_BeforObs'
  FORECAST_END='Bg_AfterObs'
  touch ${FORECAST_DEB} ; chmod u+w ${FORECAST_DEB}
  touch ${FORECAST_END} ; chmod u+w ${FORECAST_END}
  
    DATE_DEB=${YYYYMMDD_Deb}
    if [ "${HHMN_Deb}" -gt 1800 ] ; then
      HH_DEB=1800
    elif [ "${HHMN_Deb}" -gt 1200 ] ; then
      HH_DEB=1200
    elif [ "${HHMN_Deb}" -gt 0600 ] ; then
      HH_DEB=0600
    elif [ "${HHMN_Deb}" -gt 0000 ] ; then
      HH_DEB=0000
    else
      HH_DEB=0000
    fi
    DATE_END=${YYYYMMDD_End}
    if [ "${HHMN_End}" -gt 1800 ] ; then
      HH_END=0000
      DATE_END=`date_pn ${YYYYMMDD_End} 1`
    elif [ "${HHMN_End}" -gt 1200 ] ; then
      HH_END=1800
    elif [ "${HHMN_End}" -gt 0600 ] ; then
      HH_END=1200 
    elif [ "${HHMN_End}" -gt 0000 ] ; then
      HH_END=0600
    else
      HH_END=0000
    fi


  #search the nearest forecast for the input date
  # add 6 by 6 hours duration starting with 006 
  FORC_DEB='hhhhh'
  ECH=000
  while [ ! -f ${FORC_DEB} ] && [ "${ECH}" -le 72 ]
  do
    ECH=`echo "${ECH}" | awk '{printf("%.3d", $1 + 6)}'`
    set `date_hour_phours ${DATE_DEB} ${HH_DEB} -${ECH}00`
    FORC_DATE=$1
    FORC_HH=$2
    FORC_YYYYMM=`echo ${FORC_DATE} | cut -c1-6 `
    FORC_HH_GFS=`echo ${FORC_HH} | cut -c1-2 `
    DIR_FORC_GFS=`date_YYYY_MM_DD_CCC ${FORC_DATE}`
    FORC_DEB_FILE_NAME=gfs.press_gr.0p5deg_pt.${FORC_DATE}_${FORC_HH_GFS}_${ECH}.npoess.grib2
    FORC_DEB=${DIR_FORECAST}/${DIR_FORC_GFS}/${FORC_DEB_FILE_NAME}
    log_info "looking for ${FORC_DEB}"
    if [ ! -f ${FORC_DEB} ] ; then
    # try to download it    
      if [ ! -d ${DIR_FORECAST}/${DIR_FORC_GFS} ] ; then
	mkdir -p  ${DIR_FORECAST}/${DIR_FORC_GFS}
      fi
      log_info "try to download  ${MAIA4_REMOTE_GFS_DIR}/${DIR_FORC_GFS}/${FORC_DEB_FILE_NAME}"
      curl --fail ${MAIA4_REMOTE_GFS_DIR}/${DIR_FORC_GFS}/${FORC_DEB_FILE_NAME} > ${FORC_DEB}
      ret_code=$?
      if [ "$ret_code" -ne "0" ] ; then
         log_info "curl failed exit code $ret_code"
         rm -f ${FORC_DEB}
      fi
      if  [ -f ${FORC_DEB} ] ; then
        if [[ "$(stat --version 2>/dev/null)" = *GNU* ]]; then
          siz=`stat -c %s ${FORC_DEB}`
        else
          siz=`stat -f %z ${FORC_DEB}`
        fi
        if [ "${siz}" -le "5000000" ] ; then
           log_info "wrong size for ${FORC_DEB} remove it"
           rm -f  ${FORC_DEB}
        fi
      fi
    fi

  done
  SURF_DEB=${DIR_MAIA4_ATLAS}/arpege_grib_surface.grb
  log_info "FORC_DEB:${FORC_DEB}"
  ls -l ${FORC_DEB}
  log_info "SURF_DEB:${SURF_DEB}"
  ls -l ${SURF_DEB}

  if [ ! -f ${FORC_DEB} ] || [ ! -f ${SURF_DEB} ] ;  then
    log_warning " Forecast deb is empty or not available"
    FORECAST_DEB=/dev/null
  else
    log_info "Bg_BeforObs: `basename ${FORC_DEB}` "
    cp ${FORC_DEB} ${FORECAST_DEB}
    cat ${SURF_DEB} >> ${FORECAST_DEB}
  fi
  FORC_END='hhhhh'
  ECH=000

  while [ ! -f ${FORC_END} ] && [ "${ECH}" -le 72 ]
  do
    ECH=`echo "${ECH}" | awk '{printf("%.3d", $1 + 6)}'`
    set `date_hour_phours ${DATE_END} ${HH_END} -${ECH}00`
    FORC_DATE=$1
    FORC_HH=$2
    FORC_YYYYMM=`echo ${FORC_DATE} | cut -c1-6 `
    FORC_HH_GFS=`echo ${FORC_HH} | cut -c1-2 `
    DIR_FORC_GFS=`date_YYYY_MM_DD_CCC ${FORC_DATE}` 
    FORC_END_FILE_NAME=gfs.press_gr.0p5deg_pt.${FORC_DATE}_${FORC_HH_GFS}_${ECH}.npoess.grib2
    FORC_END=${DIR_FORECAST}/${DIR_FORC_GFS}/${FORC_END_FILE_NAME}
    log_info "looking for ${FORC_END} ech=${ECH}"
    if [ ! -f ${FORC_END} ] ; then
      if [ ! -d ${DIR_FORECAST}/${DIR_FORC_GFS} ] ; then
        mkdir -p  ${DIR_FORECAST}/${DIR_FORC_GFS}
      fi
       # try to download it
      log_info "try to download  ${MAIA4_REMOTE_GFS_DIR}/${DIR_FORC_GFS}/${FORC_END_FILE_NAME}"
      curl --fail ${MAIA4_REMOTE_GFS_DIR}/${DIR_FORC_GFS}/${FORC_END_FILE_NAME} > ${FORC_END}
      ret_code=$?
      if [ "$ret_code" -ne "0" ] ; then
         log_info "curl failed exit code $ret_code"
         rm -f ${FORC_END}
      fi
      if [ -f ${FORC_END} ] ; then
        if [[ "$(stat --version 2>/dev/null)" = *GNU* ]]; then
          siz=`stat -c %s ${FORC_END}`
        else
          siz=`stat -f %z ${FORC_END}`
        fi
      	if [ "${siz}" -le "5000000" ] ; then
          log_info "wrong size for ${FORC_END} remove it"
          rm -f  ${FORC_END}
      	fi
     fi
    fi
  done
  SURF_END=${DIR_MAIA4_ATLAS}/arpege_grib_surface.grb
  log_info "FORC_END:${FORC_END}"
  ls -l ${FORC_END}
  log_info "SURF_END:${SURF_END}"
  ls -l ${SURF_END}
  if [ ! -f ${FORC_END} ] || [ ! -f ${SURF_END} ] ; then
    log_warning " Forecast end is empty or not available"
    FORECAST_END=/dev/null
  else
    log_info "Bg_AfterObs: `basename ${FORC_END}` "
    cp ${FORC_END} ${FORECAST_END}
    cat ${SURF_END} >> ${FORECAST_END}
  fi
}
# end of function get_forecast_file
#=====================================================================================
function get_climatology_file {
#=====================================================================================
  # get the atlas files (ABEDO SSTMIN SSTMOY)
  MMC=`echo ${YYYYMMDD} | awk '{print substr($0,5,2)}'`
  NBJY=`date_nbjY  ${YYYYMMDD}`
  PartM=`date_numPartM  ${YYYYMMDD}`

  if   [ "${NBJY}" -ge 353 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_353.h5" 
  elif [ "${NBJY}" -ge 337 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_337.h5" 
  elif [ "${NBJY}" -ge 321 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_321.h5" 
  elif [ "${NBJY}" -ge 305 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_305.h5" 
  elif [ "${NBJY}" -ge 289 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_289.h5" 
  elif [ "${NBJY}" -ge 273 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_273.h5" 
  elif [ "${NBJY}" -ge 257 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_257.h5" 
  elif [ "${NBJY}" -ge 241 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_241.h5" 
  elif [ "${NBJY}" -ge 225 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_225.h5" 
  elif [ "${NBJY}" -ge 209 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_209.h5" 
  elif [ "${NBJY}" -ge 193 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_193.h5" 
  elif [ "${NBJY}" -ge 177 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_177.h5" 
  elif [ "${NBJY}" -ge 161 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_161.h5" 
  elif [ "${NBJY}" -ge 145 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_145.h5" 
  elif [ "${NBJY}" -ge 129 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_129.h5" 
  elif [ "${NBJY}" -ge 113 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_113.h5" 
  elif [ "${NBJY}" -ge 097 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_097.h5" 
  elif [ "${NBJY}" -ge 081 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_081.h5" 
  elif [ "${NBJY}" -ge 065 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_065.h5" 
  elif [ "${NBJY}" -ge 049 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_049.h5" 
  elif [ "${NBJY}" -ge 033 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_033.h5" 
  elif [ "${NBJY}" -ge 017 ] ; then ALBEDO="${DIR_MAIA4_ATLAS}/albmax_017.h5" 
  else                              ALBEDO="${DIR_MAIA4_ATLAS}/albmax_001.h5" 
  fi

  SSTMIN="${DIR_MAIA4_ATLAS}/sstmin_${MMC}_${PartM}.h5"
  SSTMOY="${DIR_MAIA4_ATLAS}/sstmoy_${MMC}_${PartM}.h5"

  log_info "DIR_ATLAS: ${DIR_MAIA4_ATLAS} "
  log_info "   ALBEDO: `basename ${ALBEDO}` "
  log_info "   SSTMIN: `basename ${SSTMIN}` "
  log_info "   SSTMOY: `basename ${SSTMOY}` "

  # unzip when necessary
  if [ ! -f ${ALBEDO} ]; then
    if [ ! -f ${ALBEDO}* ]; then
      log_error " No ${ALBEDO} available"
      exit 1
    else
      atlasf=`ls ${ALBEDO}.*`
      EXT=`echo ${atlasf} | awk '{n=split($0,noms,"."); print noms[n]}'`
      if [ "${EXT}" = 'gz' ] ; then
	log_info "gunzip ${atlasf} first"
	gzip -d ${atlasf}
      else 
	log_error "${atlasf} in unknown format"
	exit 1
      fi
    fi
  fi

  if [ ! -f ${SSTMIN} ]; then
    if [ ! -f ${SSTMIN}* ]; then
      log_error " No ${SSTMIN} available"
      exit 1
    else
      atlasf=`ls ${SSTMIN}.*`
      EXT=`echo ${atlasf} | awk '{n=split($0,noms,"."); print noms[n]}'`
      if [ "${EXT}" = 'gz' ] ; then
	log_info "gunzip ${atlasf} first"
	gzip -d ${atlasf}
      else 
	log_error "${atlasf} in unknown format"
	exit 1
      fi
    fi
  fi

}
# end of function get_climatology_file


#---------------------------------------------------------------------
# USER level functions
#---------------------------------------------------------------------
function maia4_env {
  # defines the environment files for MAIA v4
  log_info "entering maia4_env"
  ls -l  ${DIR_MAIA4_THRESHOLDS}/Thresholds_Sea_${SATIMG}.txt
  if [ ! -f  ${DIR_MAIA4_THRESHOLDS}/Thresholds_Sea_${SATIMG}.txt ] ; then
    log_error "Maia4 thresholds files are missing"
    log_error "directory is ${DIR_MAIA4_THRESHOLDS}"
    exit 1
  fi

  remove_maia4_env

  #Thresholds:
  ln -s ${DIR_MAIA4_THRESHOLDS}/Thresholds_Sea_${SATIMG}.txt	THfile_Sea
  ln -s ${DIR_MAIA4_THRESHOLDS}/Thresholds_Land_${SATIMG}.txt	THfile_Land
  ln -s ${DIR_MAIA4_THRESHOLDS}/Thresholds_Opaq_${SATIMG}.txt	THfile_Opaq
  ln -s ${DIR_MAIA4_THRESHOLDS}/Thresholds_Phase.txt		THfile_Phase
  
  # Albedo, SST files from climatology files
  get_climatology_file
  
  ln -s ${ALBEDO} albmax.h5
  ln -s ${SSTMIN} sstmin.h5	
  ln -s ${SSTMOY} sstmoy.h5	

  #Forecast files
  if [ "${MAIA4_USE_GFS}" = "yes" ]
  then
    if [ -z "${MAIA4_REMOTE_GFS_DIR}" ] ; then
      log_info "use GFS from default location : http://jpssdb.ssec.wisc.edu/cspp_v_2_0/ancillary/"
      MAIA4_REMOTE_GFS_DIR="http://jpssdb.ssec.wisc.edu/cspp_v_2_0/ancillary"
    else 
     log_info "use GFS from ${MAIA4_REMOTE_GFS_DIR}"
    fi
    get_forecast_file_gfs_cspp
  else
    get_forecast_file
  fi
  MAIA4_ENV='OK'
  if [ "${FORECAST_DEB}" == '/dev/null' ] || [ "${FORECAST_END}" == '/dev/null' ]; then
    MAIA4_ENV='ERROR'
  fi

  # coefficients for skin surface temperature
  ln -s ${DIR_MAIA4_THRESHOLDS}/Coefs_SST.txt SSTfile.txt
  
  # topography:
  ln -s  ${DIR_MAIA4_ATLAS}/landsea.h5   landsea.h5
  ln -s  ${DIR_MAIA4_ATLAS}/elevation.h5 elevation.h5
  
  # coefficients for visible thresholds
  ln -s  ${DIR_MAIA4_THRESHOLDS}/VIS_land_${SATIMG}.txt VIS_land.txt
  ln -s  ${DIR_MAIA4_THRESHOLDS}/VIS_sea_${SATIMG}.txt  VIS_sea.txt
  ln -s  ${DIR_MAIA4_THRESHOLDS}/VIS_reflect.txt        VIS_reflect.txt

  export MAIA4_ENV
  
  log_info "exiting maia4_env"
}

#end of function maia4_environment

function remove_maia4_env {
  CDIR=`pwd`
  
  rm -f THfile_Sea
  rm -f THfile_Land
  rm -f THfile_Opaq
  rm -f THfile_Phase
  rm -f albmax.h5
  rm -f sstmin.h5
  rm -f sstmoy.h5
  rm -f Bg_BeforObs
  rm -f Bg_AfterObs
  rm -f landsea.h5
  rm -f elevation.h5
  rm -f VIS_land.txt
  rm -f VIS_sea.txt
  rm -f VIS_reflect.txt
  rm -f SSTfile.txt
}
# end of function remove_maia4_environment
