! Name: $Id: read_maiaCT.F90 530 2017-06-27 11:07:32Z frna $
Program read_maiaCT
! name:
!
! Purpose: Read a VIIRS MAIA v4 file 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    26/10/2012  Original code. (P. Roquet)
!  1.1    26/03/2013  Bug fix (P.Roquet)
!  1.2    06/01/2017  Clean code (P.Roquet)
!  1.3    11/04/2017  Remove unused variables
!
! Code Description: F90
!
!
! LIBS = aapp_viirs atov auxaapp
!-------------------------------------------------------------------------------


#ifdef HAS_LIBHDF5
use hdf5
USE H5LT
USE aapp_hdf_mod
USE viirs_maia4_data_def
USE viirs_maia4_data_mem
USE viirs_maia4_data_io
USE viirs_maia4_header_def
USE viirs_maia4_header_mem
USE viirs_maia4_header_io

#include "aapp_throw.h"
implicit none

  character(len=256) :: filename, gname

  type(viirs_maia4_header)  :: x
  type(viirs_maia4_data)    :: y

  INTEGER(HID_T) :: file_id, group_id

  INTEGER :: HDFERR
  INTEGER :: ERR
  
  integer :: ios
  integer :: classif, Mask_Confidence, Clair

  integer :: ech=1
  INTEGER, PARAMETER :: imanq = -9999
  REAL, PARAMETER ::    vmanq = -9999.
  REAL, ALLOCATABLE :: Latitude(:,:), Longitude(:,:)
  REAL, ALLOCATABLE :: CloudTopTemperature(:,:), CloudTopPressure(:,:)
  real  :: latmin, lonmin, latmax, lonmax
  integer ::il,ip,nbpix,nbline 
  integer ::l,p

TRY

  Read(*,'(a)',iostat=ios) filename
  read(*,*) ech
  read(*,*) lonmin
  read(*,*) latmin
  read(*,*) lonmax
  read(*,*) latmax

  CALL OPEN_HDF( HDFERR )
  THROWM(HDFERR.ne.0,"Cannot open HDF library")


  CALL H5FOPEN_F( TRIM(filename), H5F_ACC_RDONLY_F, File_id, err, H5P_DEFAULT_F)
  THROWM(err.ne.0,"Cannot open File"//TRIM(Filename))


  gname = "HEADER"
  CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
  THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

  CALL viirs_maia4_header_rh(x,group_id,err)
  THROWM(err.ne.0,"Cannot read MAIAv4 Header")

  CALL h5gclose_f( group_id, err)
  THROWM(err.ne.0,"Cannot close group "//TRIM(gname))
      

  gname = "DATA"
  CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
  THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

  CALL viirs_maia4_data_rh(y,group_id,err)
  THROWM(err.ne.0,"Cannot read MAIAv4 Data")

  CALL h5gclose_f( group_id, err)
  THROWM(err.ne.0,"Cannot close group "//TRIM(gname))

  CALL h5fclose_f(file_id, err) 
  THROWM(err.ne.0,"Cannot close file "//TRIM(Filename))



  CALL CLOSE_HDF( HDFERR )
  THROWM(HDFERR.ne.0,"Cannot close HDF library")


 
 nbline=x%Nb_Lines
 nbpix=x%Nb_pixels 
 allocate ( Latitude(nbpix,nbline), Longitude(nbpix,nbline))
 allocate ( CloudTopTemperature(nbpix,nbline), CloudTopPressure(nbpix,nbline))
 do il=1, nbline
  do ip=1, nbpix
     if(y%Latitude(ip,il) /= imanq*10000)       Latitude(ip,il)=y%Latitude(ip,il)/10000.
     if(y%Longitude(ip,il) /= imanq*10000)      Longitude(ip,il)=y%Longitude(ip,il)/10000.
! for example if we need CloudTopPressure or CloudTopTemperature (not used in this program)
! Convert CloudTopPressure in hPa
     if(y%CloudTopPres(ip,il) /= imanq)         CloudTopPressure(ip,il)= y%CloudTopPres(ip,il)/10.
! Convert Temperature in degres celcius
     if(y%CloudTopTemp(ip,il) /= imanq)         CloudTopTemperature(ip,il)= y%CloudTopTemp(ip,il)/100.
     
  end do
  end do

  do l=1,nbline, ech
     do p=1, nbpix, ech
        if (Latitude(p,l)>=latmin .and. Latitude(p,l)<=latmax .and. &
            Longitude(p,l)>=lonmin .and. Longitude(p,l)<=lonmax) then
        if ( IBITS(y%Mask_ind(p,l),1,1) ==1 .and. Latitude(p,l)> vmanq) then
           Clair = 1
           Mask_Confidence = IBITS(y%CloudMask(p,l),5,2)
           if(Mask_Confidence==0 .or. Mask_Confidence==1) Clair=0
           Clair= IBITS(y%CloudMask(p,l),13,2)
           classif=IBITS(y%CloudType(p,l),4,5)  ! classif sur 5 bits
           if(Longitude(p,l)>-361 .and. Latitude(p,l)>-91.) then
           write(20,'(3f10.3)') Longitude(p,l), Latitude(p,l), Clair*1.
           write(21,'(3f10.3)') Longitude(p,l), Latitude(p,l), Mask_Confidence*1.
           write(23,'(3f10.3, 2i5)') Longitude(p,l), Latitude(p,l), classif*1., l,p
           endif
        endif
        endif
     end do
  end do

PCATCH

#else

  write(0,'("viirs_maia4.exe has not been built with HDF-5 Fortran interface")')
  write(0,'("Please run configure with --external-libs=-Lxxx/lib -lhdf5hl_fortran")')
  write(0,'(" and --external-includes=-Lxxx/include")')
  write(0,'(" where xxx is the location of the HDF-5 library")')
  
#endif

End Program read_maiaCT

