Program viirs_paste_sdrpp
! name:
!
! Purpose: Paste all VIIRS SDR files for a single Band
! Files are supposed to be one file per granule, one file per channel
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    15/04/2011  Original code. (P. Brunel)
!  1.1    11/04/2017  Remove unused variables (P. Roquet)
!
! Code Description: F90
!
!
! LIBS = aapp_viirs atov auxaapp
!-------------------------------------------------------------------------------


#ifdef HAS_LIBHDF5
use hdf5
USE H5LT
USE aapp_hdf_mod
USE viirs_sdr_def
USE viirs_sdr_mem
USE viirs_sdr_io

USE viirs_sdr_mod
#include "aapp_throw.h"
implicit none
  integer,parameter :: fmax = 64
  
  character(len=256) :: tmpfilename
  character(len=256) :: tmpbasename
  character(len=256) :: outfilename
  character(len=256),allocatable :: filenames(:)
  character(len=256) :: filename(fmax,16)
  character(len=256) :: geolocfile(fmax)

  character(len=3) :: band_name
  integer :: ichan, igran, ios
  integer :: mchan, nchan
  integer, allocatable :: channels(:)
  integer :: nchannels, npixels, nlines, nscans, ngranules
  integer :: ngran(20), ngranref, g

  type(viirs_sdr)  :: x, y, z

  INTEGER :: HDFERR
  INTEGER :: ERR
  LOGICAL :: compress
  
  integer :: i

TRY

  CALL OPEN_HDF( HDFERR )
  THROWM(HDFERR.ne.0,"Cannot open HDF library")

  
  g = 0
  mchan = 0
  ngran = 0
  ngranref = 0
  compress = .false.
! read output filename
  Read(*,'(a)',iostat=ios) outfilename
  
! Read all input filenames and enentually compress is wanted
  Do
    Read(*,'(a)',iostat=ios) tmpfilename
    tmpbasename =  aapp_basename( tmpfilename )
    if ( ios .NE. 0 ) exit
    Select Case ( tmpbasename(1:3)  )
      case ('GIM','GMO','GDN','GIT','GMT')
        g = g+1
        geolocfile(g) = tmpfilename
      case ('SVI','SVM')
        read(tmpbasename(4:5),'(i2)') ichan
        ngran(ichan) = ngran(ichan)+1
        ngranref = ngran(ichan)
        filename(ngran(ichan),ichan) = tmpfilename
        if( mchan .lt. ichan ) mchan = ichan
        band_name = tmpbasename(3:3)
      case ('SVD')
        ichan = 1
        ngran(ichan) = ngran(ichan)+1
        ngranref = ngran(ichan)
        filename(ngran(ichan),ichan) = tmpfilename
        mchan = 1
        band_name = tmpbasename(3:3)
      case('com','COM')
        compress=.true.
    End Select
  EndDo
  write(*,*) "compress : ",compress    

! number of channels
! verify same number of granules for all channels
  nchan = 0
  Do ichan = 1, mchan
    if( ngran(ichan) .GT. 0 ) then
      if ( ngran(ichan) .ne. ngranref ) then
        err = 1
        THROWM(ERR.ne.0,"Not same number of granules per channel")
      end if
      nchan = nchan +1
    end if
  End Do

  allocate ( filenames(nchan), channels(nchan), stat=err )
  THROWM(err.ne.0,"Cannot allocate filenames and channels")

! loop on granules
  Do igran = 1, ngranref
    channels = 0
    i = 0
    Do ichan = 1, mchan
      if( ngran(ichan) .GT. 0 ) then
        i = i +1
        filenames(i) = filename(igran,ichan) 
        channels(i) = ichan
      end if
    End Do
    write(0,*) "start granule ", igran

    Do i = 1, size(filenames)
      write(0,*) "filenames ", TRIM(filenames(i))
#ifdef DEBUG
      write(0,*) "channels ", channels
#endif
    End Do
    write(0,*) "geo  ", TRIM(geolocfile(igran))

    call viirs_sdr_load(band_name, z, filenames, err, channels=channels, clean=.true., geolocfile=geolocfile(igran))
    THROWM(HDFERR.ne.0,"Cannot load VIIRS SDR structure")

    CALL viirs_sdr_info( z, nchannels, npixels, nlines, nscans, ngranules, err )
    write(*,*) "nchannels ", nchannels
    write(*,*) "npixels   ", npixels
    write(*,*) "nlines    ", nlines
    write(*,*) "nscans    ", nscans
    write(*,*) "ngranules ", ngranules

! paste granules
    if ( igran .eq. 1 ) THEN
      call viirs_sdr_copy(x,z,err)
      THROWM(HDFERR.ne.0,"Cannot copy VIIRS SDR structure")
      call viirs_sdr_copy(y,z,err)
      THROWM(HDFERR.ne.0,"Cannot copy VIIRS SDR structure")
    else
      call viirs_sdr_free(x,err)
      call viirs_sdr_paste(x,y,z,err)
      THROWM(HDFERR.ne.0,"Cannot paste VIIRS SDR structure")
      call viirs_sdr_free(y,err)
      call viirs_sdr_copy(y,x,err)
      THROWM(HDFERR.ne.0,"Cannot copy VIIRS SDR structure")
    endif
    
    CALL viirs_sdr_free(z,err) 
    THROWM(HDFERR.ne.0,"Cannot free VIIRS SDR structure")

  End do
    
  call viirs_sdr_free(y,err)
  THROWM(HDFERR.ne.0,"Cannot free VIIRS SDR structure")

  CALL viirs_sdr_info( x, nchannels, npixels, nlines, nscans, ngranules, err )
  write(*,*) "output result "
  write(*,*) "nchannels ", nchannels
  write(*,*) "npixels   ", npixels
  write(*,*) "nlines    ", nlines
  write(*,*) "nscans    ", nscans
  write(*,*) "ngranules ", ngranules
  
  call  viirs_sdr_save(x, outfilename, err,compress)
  THROWM(ERR.ne.0,"Cannot save file")

  CALL viirs_sdr_free(x,err) 
  THROWM(HDFERR.ne.0,"Cannot free VIIRS SDR structure")

  CALL CLOSE_HDF( HDFERR )
  THROWM(HDFERR.ne.0,"Cannot close HDF library")

PCATCH

#else

  write(0,'("viirs_paste_sdr.exe has not been built with HDF-5 Fortran interface")')
  write(0,'("Please run configure with --external-libs=-Lxxx/lib -lhdf5hl_fortran")')
  write(0,'(" and --external-includes=-Lxxx/include")')
  write(0,'(" where xxx is the location of the HDF-5 library")')
  
#endif

End Program viirs_paste_sdrpp
