! Name: $Id: viirs_to_cris.F90 598 2018-11-09 11:48:06Z frna $
! LIBS = mapviirscris aapp_viirs atovpp atov cal f7nl1b navtool f7ml f7tp f7cp f7gp auxaapp clusters lapack-lite
!-----------------------------------------
!> This program maps the CrIS field of view onto the VIIRS imager data
!!
!! Purpose:
!!   - Read CrIS level 1C or 1D satellite data
!!   - Read VIIRS SDR and/or MAIA4 file
!!   - Get geolocation from HDF5 geolocation file
!!   - Map VIIRS data (Radiances and Cloud Mask) to CrIS FOV
!!   - Write out CrIS level 1C or 1D
!!   - if CrIS input file is 1C and env MAP_DO_CLUSTERS="Y" compute clusters
!!     on each CrIS FOR, and compute statistics on each CrIS FOV (output: cluster_output.h5)
!!
!! input files in current directory:
!!   - VIIRS SDR: viirssdr.H5
!!   - VIIRS GEO: viirsgeo.H5 (could be a link to SDR file)
!!   - CrIS 1D  : fort.28
!!   - CrIS 1C  : fort 18  (only used if no level 1D)
!!   - MAIA4    : maia4.H5 (optional)
!!   - Mapping FDF : fort.55
!!
!! Environment variables
!! VIIRS_BAND the VIIRS band name (I or M), mandatory
!! MAP_VIIRS_CRIS_DEBUG the debug level optional, 0=no debug, 1=ouput of mapping details, 2=1+logfile
!! SHIFT_PIX the pixel shift for radiances, optional (number of pixels)
!! SHIFT_LIN the line shift for radiances, optional (number of lines)
!!
!! output file
!!   - CrIS 1D : fort.28 (updated with VIIRS values)
!!   - CrIS 1C : fort.18 (if no level 1D)
!!   - clusters_output.h5 (if input file is 1C and MAP_DO_CLUSTERS="Y")
!!
!! See also adjacency algorithm described in : 
!!      Joint Polar Satellite System (JPSS)
!!      Operational Algorithm Description (OAD)
!!      Document for Common Adjacency Software
!!   available at (as of Jan 2014):
!!     http://npp.gsfc.nasa.gov/sciencedocuments/2013-06/474-00097_OAD_Cmn-Adj_B.pdf
!!
!! Current Code Owner: NWP SAF
!!
!! COPYRIGHT
!!    This software was developed within the context of the EUMETSAT Satellite
!!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!!
!!    Copyright 2013, EUMETSAT, All Rights Reserved.
!!
!! @author
!! 16/12/13  P. Brunel
!! 09/05/2016: Bug fix 
!! 11/04/2017  Remove unused variables (P. Roquet)
!! 08/09/2017  Set cris1d_cloudflag to -9999 after GET_CRIS1D_VIIRS,
!!             i.e. discard values set in atovpp (NA)
!! 10/2018     Bugs fixes and Add clusters (P. Roquet)
!! 09/11/2018  Bug fix if not enough valid pixels for clusters (P.Roquet)
!-------------------------------------------------------------------------------
Program viirs_to_cris
!> debut programme.
!!

#ifdef HAS_LIBHDF5
use hdf5
USE H5LT
USE AAPP_HDF_MOD
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MEM
USE VIIRS_SDR_MOD
USE MAP_VIIRS_ADJACENCY_DEF
USE MAP_VIIRS_ADJACENCY_MOD

USE MAP_VIIRS_CRIS_DEF
USE MAP_VIIRS_CRIS_MOD

#include "aapp_throw.h"
implicit none

  TYPE(VIIRS_SDR)      :: SDR
  TYPE(VIIRS_MAIA4)    :: MAIA4
  TYPE(CFOVVIIRS)      :: MAPPING
  TYPE(FOV_TYPE)       :: MYFOV 
  TYPE(CFORVIIRS)      :: FORMAP
  TYPE(ADJTABLE)       :: TABLE
  TYPE(VIIRS_LOCATION) :: VIIRSLOC
  TYPE(MAP_VIIRS_OPTIONS) :: OPTIONS
  TYPE(CLUSTERVIIRS) :: CLUSTER
  
  ! Variables for loop on all CrIS 1C/1D records and FOVs
  INTEGER :: IFOR, IFOV, NFOR, NFOV, NSCAN
  INTEGER :: HDRCNT, RECPERSCAN
  INTEGER :: FOV, FOVSPERFOR
  INTEGER :: CRISRECORD
  LOGICAL :: LCRIS1C ! True if we run a CrIS 1C file

  ! DATA for 1D output format
  ! TAKE CARE this should be excatly the same as the cris1d.h f77 file
  INTEGER, PARAMETER ::    cris_1dnchanviirs = 22  ! Number of VIIRS channels in format
  INTEGER, PARAMETER ::    cris_1dnfor = 30         ! Number of FORs present in 1d
  INTEGER ::    cris1d_h_nviirschan       !number of VIIRS channels in format
  INTEGER ::    cris1d_h_nviirschanused   !number of channels used
  INTEGER ::    cris1d_h_viirschans(cris_1dnchanviirs)  !Channel numbers
  INTEGER ::    cris1d_h_viirsbands(cris_1dnchanviirs)  !Band numbers
  INTEGER ::    cris1d_viirsradiance(cris_1dnchanviirs,cris_1dnfor)! W/(m2 sr micron) *1E7
  INTEGER ::    cris1d_viirsreflectance(cris_1dnchanviirs,cris_1dnfor)
  INTEGER ::    cris1d_cloudflag(10,cris_1dnfor)=0!CrIS cloud flag (user defined)
  
  ! DATA for 1C output format
  ! TAKE CARE this should be excatly the same as the cris1c.h f77 file
  INTEGER, PARAMETER ::    cris_1cnfov =  9  ! Number of fields of view
  INTEGER, PARAMETER ::    cris_1cnfor = 30  ! Number of FORs present in 1d
  INTEGER ::  cris1c_landfrac(cris_1cnfov,cris_1cnfor)     ! land fraction
  INTEGER ::  cris1c_landsea(cris_1cnfov,cris_1cnfor)      ! land/sea qualifier
  INTEGER ::  cris1c_cloudcover(cris_1cnfov,cris_1cnfor)   ! cloud cover (percent)
  INTEGER ::  cris1c_cloudtopht(cris_1cnfov,cris_1cnfor)   ! height of top of cloud (m)

  INTEGER :: HDFERR
  INTEGER :: ERR
  INTEGER IFORSTART, IFOREND
  LOGICAL :: FirstCluster
  CHARACTER(LEN=256) :: ClusterFile
  TYPE(clusterparameter) :: CLUSTERPARAM
  
TRY
  NFOR = 30
  FirstCluster = .True.
  ClusterFile = "cluster_output.h5"

  ! Open HDF Library
  CALL OPEN_HDF( HDFERR )
  THROWM(HDFERR.ne.0,"Cannot open HDF library")

  CALL MAP_VIIRS_OPTIONS_SETUP(OPTIONS, ERR)
  THROWM(ERR.ne.0,"OPTIONS SETUP error")
  IF(OPTIONS%DEBUG .GE. 2) CALL MAP_VIIRS_PRINT_OPTIONS(OPTIONS)
  
  IF (OPTIONS%DO_CLUSTERS) cluster%first_time = .True.

  ! Prepare CrIS data (call f77 interface routines)
  ! Returns logical LCRIS1C
  CALL OPEN_CRIS(cris1Cunit, cris1Dunit, LCRIS1C)
  IF(LCRIS1C) THEN
    CALL GET_CRIS1C_HEADER(cris1Cunit, hdrcnt, recperscan, nfor, nfov, nscan)
    fovsperfor = 9
  ELSE
    OPTIONS%DO_CLUSTERS = .FALSE.
    CALL GET_CRIS1D_HEADER(cris1Dunit, hdrcnt, recperscan, nfor, nfov, nscan, &
       & cris1d_h_nviirschan, cris1d_h_nviirschanused, &
       & cris1d_h_viirschans, cris1d_h_viirsbands)
    fovsperfor = 1
  ENDIF

  ! VIIRS and MAIA data
  ! Fills any other OPTIONS
  CALL MAP_VIIRS_SETUP(OPTIONS, SDR, MAIA4, VIIRSLOC, ERR)

  IF( OPTIONS%DEBUG .GE. 1) THEN
    CALL VIIRSLOC_SAVE( OPTIONS, VIIRSLOC, ERR)
    THROWM(ERR.ne.0,"viirsloc save error")
  ENDIF

  THROWM(ERR.ne.0,"SETUP error")

  CALL MAP_VIIRS_CRIS_FDF(crisfunit, SDR%ROOT_ATT%Platform_Short_Name, err)
  THROWM(ERR.ne.0,"Cannot load VIIRS-CRIS FDF")
  
  IF (OPTIONS%DO_CLUSTERS) THEN
     write(*,*) "update cluster parameters"
     CALL MAP_UPDATE_CLUSTER_PARAM(OPTIONS,SDR,CLUSTERPARAM, ERR)
     IF (err .ne. 0) THEN
        write(*,*) "WARNING : Cannot do cluster"
        OPTIONS%DO_CLUSTERS = .False.
     ENDIF
  ENDIF
  ! Create the adjacency table
  IF( OPTIONS%DEBUG .GE. 1) WRITE(0,*) "call MAP_VIIRS_ADJACENCY_NEW"
  CALL MAP_VIIRS_ADJACENCY_NEW(TABLE, VIIRSLOC%LATITUDE, VIIRSLOC%LONGITUDE, err)
  THROWM(ERR.ne.0,"Adjacency SETUP error")
  IF( OPTIONS%DEBUG .GE. 1) WRITE(0,*) "out MAP_VIIRS_ADJACENCY_NEW"

  IF(OPTIONS%DEBUG .GE. 2) THEN
    CALL MAP_VIIRS_ADJACENCY_SAVE( TABLE, "adjacency-table.H5", "/", ERR)
    THROWM(ERR.ne.0,"Adjacency save error")
  ENDIF

  IF( .NOT. LCRIS1C ) THEN
    ! If CrIS data is level 1d
    ! manage CrIS 1D header variables concerning VIIRS
    ! Read, Initialise, Write
    CALL MAP_CONV_SDR_1D_H(OPTIONS, SDR, &
       & cris_1dnchanviirs, cris1d_h_nviirschan, cris1d_h_nviirschanused, &
       & cris1d_h_viirschans, cris1d_h_viirsbands )

    CALL PUT_CRIS1D_HEADER(cris1Dunit, &
       & cris1d_h_nviirschan, cris1d_h_nviirschanused, &
       & cris1d_h_viirschans, cris1d_h_viirsbands)
  ENDIF

! Loop on all CrIS records
  CRISRECORD = hdrcnt + 1
  RECORDL: DO WHILE ( CRISRECORD.LE.(hdrcnt+nscan*recperscan) )
  
!   Read a scan line (or part of scan line for 1d)
    CALL READ_CRIS (cris1Cunit, cris1Dunit, LCRIS1C, CRISRECORD)
    IF( LCRIS1C ) THEN
      CALL GET_CRIS1C_VIIRS(cris1c_landfrac, cris1c_landsea, cris1c_cloudcover, cris1c_cloudtopht)
    ELSE
      CALL GET_CRIS1D_VIIRS(cris1d_viirsradiance, cris1d_viirsreflectance, cris1d_cloudflag )
      cris1d_cloudflag = -9999    !discard the values set in atovpp
    ENDIF
    write(*,*) "Read cris line"
    IF(OPTIONS%DEBUG .GE. 1)  WRITE(0,*) CRISRECORD
     
    ! Loop on Field Of Regards
    IFOV = 0 ! IFOV only needed if work with 1C format
    ! it will be modified after

      ! 1D format: only one Field Of View (FOV) per record
      !            all Field Of Regard (FOR) per record

     IFORSTART = 1
     IFOREND = 30
      FORL: DO IFOR= IFORSTART, IFOREND
        FOVL: DO FOV=1, FOVSPERFOR
          IF(LCRIS1C) THEN
             IFOV = FOV
          ENDIF

        ! New mapping structure for new FOR/FOV
        CALL MAP_NEW(OPTIONS, MAPPING, LCRIS1C, IFOR, IFOV)
        !if( MAPPING%fov.eq.1 .and. MAPPING%for.eq.4 .and. CRISRECORD .eq. hdrcnt + 1)then
        ifov=mapping%fov

        ! Find CrIS location in VIIRS data
        CALL MAP_NEAREST_VIIRS(OPTIONS, MAPPING, VIIRSLOC, ERR)
        ! TEST ERR=99 => CRIS FOV outside VIIRS data; no update of 1D variables 
        ! TEST ERR=1  => ERROR
        IF( ERR.GE.90 ) THEN
          CALL MAP_FREE( MAPPING, ERR )
          CYCLE FORL
        ENDIF
        IF (ERR.NE.0) THEN
           write(*,*) "ERROR NEAREST VIIRS"
        ENDIF

        ! Compute Mapping Box pixels/lines and find neighbors
        CALL MAP_CRIS_FOV_BOX(OPTIONS, MAPPING, TABLE, ERR)
         IF (ERR.NE.0) THEN
           write(*,*) "ERROR MAP CRIS FOV BOX"
        ENDIF

        ! Compute the FOV Mask and Weights
        CALL MAP_CRIS_FOV_MASK(OPTIONS, MAPPING, VIIRSLOC, ERR)
        ! Returns ERR=98 for FOV outside, or partly outside BOX
        IF( ERR.EQ.98 ) THEN
          CALL MAP_FREE( MAPPING, ERR )
          CYCLE FORL
        ENDIF
         IF (ERR.NE.0) THEN
           write(*,*) "ERROR MAP FREE"
        ENDIF

        IF( OPTIONS%DEBUG .GE. 3 ) THEN
          CALL MAP_SAVE_DEBUG(OPTIONS, MAPPING, TABLE, ERR)  
           IF (ERR .ne. 0)   write(*,*) "ERROR ON MAP_SAVE_DEBUG"
        ENDIF

        IF (OPTIONS%DO_CLUSTERS) THEN
           CALL MAP_PUT_MAPPING_IN_FOV( OPTIONS, MAPPING, MYFOV)
           FORMAP%fovs(FOV) = MYFOV
        ENDIF

       IF ( FOV .eq. 9 .and. OPTIONS%DO_CLUSTERS) THEN

          CALL MAP_CRIS_FOR_BOX(OPTIONS, FORMAP, TABLE, ERR)
         IF (ERR.NE.0) THEN
           write(*,*) "ERROR MAP CRIS FOR BOX"
         ENDIF

         CALL MAP_CLUSTER_SDR (OPTIONS, FORMAP, SDR,  VIIRSLOC, CLUSTERPARAM, CLUSTER, ERR )
         THROWM(ERR .ne. 0 .and. ERR .ne. 1,"ERROR ON MAP_CLUSTER_SDR")
     
        IF( ERR .eq. 0) THEN  
          CALL CLUSTER_SAVE_OUTPUT( OPTIONS, ClusterFile,FirstCluster,CLUSTER, &
          FORMAP, ERR)
          THROWM(ERR .ne. 0,"ERROR ON CLUSTER_SAVE_OUTPUT")
          IF (FirstCluster) FirstCluster = .False. 
            IF( OPTIONS%DEBUG .GE. 1 ) THEN  
             CALL CLUSTER_SAVE(OPTIONS,CLUSTER, ERR)  
             THROWM(ERR .ne. 0,"ERROR ON CLUSTER_SAVE")
            ENDIF
        ENDIF

       ENDIF ! DO_CLUSTER

        ! Do VIIRS SDR processing in CrIS FOV only for 1D
        IF (.NOT. LCRIS1C) THEN
         CALL MAP_CONV_SDR_1D_D(OPTIONS, MAPPING, SDR,     &
              & cris_1dnchanviirs,    cris_1dnfor,         &
              & cris1d_h_viirschans,                       &
              & cris1d_viirsradiance, cris1d_viirsreflectance )
        ENDIF

        ! Do VIIRS MAIA4 processing in CrIS FOV
        ! ADD THE 1C DATA FOR CLOUD MASK
        IF( MAIA4%LMAIA4 ) THEN
          CALL MAP_CONV_MAIA4(OPTIONS, MAPPING, MAIA4,      &
                            & cris1c_landfrac(IFOV,IFOR),   &
                            & cris1c_landsea(IFOV,IFOR),    &
                            & cris1c_cloudcover(IFOV,IFOR), &
                            & cris1c_cloudtopht(IFOV,IFOR), &
                            & cris1d_cloudflag(:,IFOR), ERR)
           IF (ERR .ne. 0) THEN
             write(*,*) "ERROR ON MAP_CONv MAIA4"
            ENDIF
        ENDIF
        !endif

       IF ( FOV .eq. 9 .and. OPTIONS%DO_CLUSTERS) THEN
         CALL MAPFOR_FREE(FORMAP,ERR)
         THROWM(ERR.ne.0,"Cannot free formap")
       ENDIF ! DO_CLUSTER

      END DO FOVL
    END DO FORL

    !  Update a scan line
    IF( LCRIS1C ) THEN
      ! Here we update the CrIS 1C record with all calculated values
      ! Any CrIS FOV outside VIIRS data (above return codes 99 and 98) are not updated
      !    because they did not pass through the convolution subroutine
      CALL PUT_CRIS1C_VIIRS(cris1c_landfrac, cris1c_landsea, cris1c_cloudcover, cris1c_cloudtopht)
      CALL WRITE_CRIS1C(cris1Cunit, CRISRECORD)
    ELSE
      ! Here we update the CrIS 1D record with all calculated values
      ! Any CrIS FOV outside VIIRS data (above return codes 99 and 98) are not updated
      !    because they did not pass through the convolution subroutine
      CALL PUT_CRIS1D_VIIRS(cris1d_viirsradiance, cris1d_viirsreflectance, cris1d_cloudflag )
      CALL WRITE_CRIS1D(cris1Dunit, CRISRECORD)
    END IF

    ! Increment record number for next read
    CRISRECORD = CRISRECORD + 1
  END DO RECORDL

  ! Close CrIS file
  CALL CLOSE_CRIS(cris1Cunit, cris1Dunit, LCRIS1C)

  ! Free memory
  CALL MAP_VIIRS_ADJACENCY_FREE(TABLE, err)
  THROWM(ERR.ne.0,"Cannot free adjacency table")
  CALL MAP_VIIRS_FREE(SDR, MAIA4, VIIRSLOC, ERR)
  THROWM(ERR.ne.0,"Cannot free mapping")

  ! Close HDF library
  CALL CLOSE_HDF( HDFERR )
  THROWM(HDFERR.ne.0,"Cannot close HDF library")

PCATCH

#else

  write(0,'("viirs_to_cris.exe has not been built with HDF-5 Fortran interface")')
  write(0,'("Please run configure with --external-libs=-Lxxx/lib -lhdf5hl_fortran")')
  write(0,'(" and --external-includes=-Lxxx/include")')
  write(0,'(" where xxx is the location of the HDF-5 library")')
  
#endif

End Program viirs_to_cris
