#!/bin/ksh
#
#
#Mapping of CrIS FOVs in VIIRS SDR
#
#History:
# 1.0   19/02/2014  initial version. P Brunel
# 1.1   07/10/2014  introduce threshold  P.B.
# 1.2   24/10/2018  add do clusters option
#==============================================================================
#set -x

. "$AAPP_PREFIX/ATOVS_CONF"

usage(){
  echo
  echo 'usage : viirs_to_cris [-d|-D] [-t threshold] [-b band] [-m Maia4file] [-g Geofile] [ -C clustersfile ] [-s sorting_channel] CrISfile VIIRSfile '
  echo
  echo ' where band      is a VIIRS band name I or M'
  echo '       Maia4file is a VIIRS MAIA 4 HDF5 file'
  echo '       Geofile   is a VIIRS geolocation HDF5 file'
  echo '       Crisfile  is a CrIS AAPP level 1c/1d file'
  echo '       VIIRSfile is a VIIRS SDR  HDF5 file with geo localisation'
  echo '       -d debug level 1'
  echo '       -D debug level 2'  
  echo '       threshold is the minimum percentage of valid VIIRS pixels for mapping'
  echo '       clustersfile is the output of the clusters analysis (hdf5 format) (only used if the CrIS file is 1c)'
  echo '       sorting_channel is the VIIRS channel number used for the clusters sorting (1-16 for M band, ou 1-5 for I band)'
  echo ' if the -C option is not present, the cluster analysis is not done'    
}

#Look for the exe file, first in script directory then in $PATH

exe=$(dirname $0)/viirs_to_cris.exe

[ -x $exe ] || exe=$(IFS=":"; for p in $PATH; do q=$p/viirs_to_cris.exe; \
  [ -x $q ] && echo $q && return; done)

[ -x $exe ] || { echo "Cannot find viirs_to_cris.exe"; exit 1; }

missing=$(ldd $exe | grep "not found")

if [ "$missing" != "" ]; then
  echo $missing
  echo "Library not found, please locate the shared library listed above" 
  echo " and add it to your LD_LIBRARY_PATH environment variable"
  exit 1
fi

band="M"
dbg=0
threshold=50.0
export MAP_CLUSTER_SORT_CHANNEL=16
export MAP_DO_CLUSTERS="n"
while getopts m:g:b:t:dDC:s: c
do
  case $c in
     m)	maiafile=$OPTARG;;
     g)	geofile=$OPTARG;;
     b)	band=$OPTARG;;
     t) threshold=$OPTARG;;
     d)	dbg=1;;
     D)	dbg=2;;
     C) MAP_DO_CLUSTERS="y"
        CLUSTER_FILE=$OPTARG;;
     s) MAP_CLUSTER_SORT_CHANNEL=$OPTARG;;
     \?)      usage
              exit 2;;
  esac
done
shift `expr $OPTIND - 1`

[ $# -lt 2 ] && { usage; exit 1; }
[ -f $1 ] || { echo "CrIS 1C/1D file $1 not found"; exit 1; }
[ -f $2 ] || { echo "VIIRS SDR file $1 not found"; exit 1; }

geofile=${geofile:-$2}

rm -f ${FORT}18 ${FORT}28 ${FORT}55 cluster_output.h5
ln -sf $1       ${FORT}18
ln -sf $1       ${FORT}28
ln -sf $2       viirssdr.H5 
ln -sf $geofile viirsgeo.H5 
ln -sf $DIR_PREPROC/mapping_viirs_cris.fdf ${FORT}55
if [ "$band" = "M" ] && [ -f $maiafile ]; then
  ln -sf $maiafile maia4.H5
fi


# Environment variables for Mapping
# upcase VIIRS Band name
export VIIRS_BAND="$(echo $band | tr '[a-z]' '[A-Z]')"
export MAP_VIIRS_CRIS_DEBUG=${dbg}
export MAP_VIIRS_CRIS_THRESHOLD=${threshold}
echo "CLUSTER SORT CHANNEL=${MAP_CLUSTER_SORT_CHANNEL}"
$exe > viirs_to_cris-${band}.log

status=$?
if [[ $status != 0 ]]; then
   echo "*** Error $status in $exe"
   echo "*** Band:${BAND}"
   echo "*** Input files:"
   echo "***   CrIS     : $1"
   echo "***   VIIRS SDR: $2"
   echo "***   geofile  : ${geofile}"
   echo "***   maiafile : ${maiafile}"
   echo "***   log      : viirs_to_cris-${band}.log"
   echo "*** Threshold of good VIIRS pixels for mapping : ${MAP_VIIRS_CRIS_THRESHOLD}"
   echo "*** Do cluster if crisl1c is present : $MAP_DO_CLUSTERS"
fi

rm -f ${FORT}18 ${FORT}28 ${FORT}55
rm -f viirsgeo.H5 viirssdr.H5 
rm -f maia4.H5
rm -f adjacency-table.H5
if [ -f  cluster_output.h5 ]; then
   mv  cluster_output.h5 ${CLUSTER_FILE}
   echo "*** create cluster file : ${CLUSTER_FILE}"
fi
exit $status


