!> In this module, we process VIIRS SDR files in HDF5 format
!
! 2011/05 Pascal Brunel
!
MODULE viirs_edr_img_MOD

#ifdef HAS_LIBHDF5

#include "aapp_throw.h"
      USE jpss_meta_root_def
      USE jpss_meta_root_io
      USE jpss_meta_root_mem
      USE jpss_meta_aggregate_def
      USE jpss_meta_aggregate_io
      USE jpss_meta_aggregate_mem
      USE jpss_meta_granule_def
      USE jpss_meta_granule_io
      USE jpss_meta_granule_mem

      USE viirs_edr_img_data_def
      USE viirs_edr_img_data_io
      USE viirs_edr_img_data_mem

      USE viirs_edr_img_geo_def
     
      USE viirs_edr_img_def
      USE viirs_edr_img_mem


      

      IMPLICIT NONE

      !PUBLIC :: viirs_edr_img_data_proc
      PUBLIC :: viirs_edr_img_load
      PUBLIC :: viirs_edr_img_info
      !PUBLIC :: viirs_edr_img_geo_load
      PRIVATE

      CONTAINS
      subroutine viirs_edr_img_load( bandname, x, filenames, err, channels , geolocfile, no_geo, clean)
      ! Loads and pre-process all VIIRS data for a given Band according to options, returns x:
      ! If only one filename, it will be used for all channels
      ! If channels is present then corresponding channels will be read
      !   if not, all channels of given Band are read from one file or from the list of files
      ! If no_geo is present and false, or not present
      !   geolocation file is loaded from geolocfile (if present) or from
      !   the root attribute N_GEO_Ref, but same directory as filenames(1). Then geolocation is processed.
      !
      use hdf5
      USE H5LT
      USE aapp_hdf_mod


      implicit none

      character(len=*), intent(in) :: bandname ! VIIRS Band Name: I M or DNB
      type(viirs_edr_img),  intent(inout) :: x
      character(len=*), intent(in) :: filenames(:) ! Name of file (one for all or one per channel)

      integer,          intent(out) ::err
      integer, optional, intent(in) :: channels(:) ! if present, the list of channels
      character(len=*), optional, intent(in) ::  geolocfile ! file name for geolocation
      logical, optional, intent(in) :: no_geo ! if TRUE geolocation is not loaded 
      logical, optional, intent(in) :: clean  ! if TRUE remove unsed arrays 

      type(jpss_meta_aggregate) :: att
            
            
      character(len=256) :: filename
      character(len=256) :: geoname
      character(len=256) :: gname, dname, aname
      character(len=3)   :: channel_name

      integer :: nfiles
      integer :: ifov
      integer :: ichan, nchans, maxchans, channel
      integer :: lchans(16)
      character(len=1) :: nband

      INTEGER(HID_T) :: file_id, group_id, dataset_id, attr_id

      LOGICAL :: attr_exists       ! .TRUE. if exists, .FALSE. otherwise
      LOGICAL :: lget_geo
      
TRY
      lget_geo = .TRUE.
      IF( present(no_geo) ) THEN
        lget_geo = .NOT. no_geo
      END IF

      CALL viirs_edr_img_init(x, bandname, err)
      THROWM(ERR.ne.0,"Cannot init EDR IMG")

      nband = bandname
      Select Case ( nband )
        case ('i','I')
          maxchans = 5
        case ('m','M')
          maxchans = 6
        case default
          maxchans = 1
      End Select
      write(*,*) 'size(channels) ',size(channels)
      write(*,*) 'maxchans ',maxchans
      write(*,*) 'bandname ',bandname
      write(*,*) 'nband ',nband
      
      nfiles = size(filenames)
      write(*,*) 'size(filenames) ',nfiles
      write(*,*) filenames(1)

      nchans = 0
      lchans = 0
      If ( present(channels) ) Then
        DO ichan = 1, size(channels)
          IF( channels(ichan) .GT. 0 .AND. channels(ichan) .LE. maxchans ) THEN
            nchans = nchans +1
            lchans(nchans) = channels(ichan)
          END IF
        END DO
      ELSE
        nchans = maxchans
        lchans = (/ (ichan, ichan = 1, nchans) /)
      END IF
      
      write(*,*) 'nchans ',nchans
      write(*,*) 'lchans ',lchans

      IF( nchans .LE. 0 .OR. nchans .GT. maxchans ) err = 1
      THROWM(err.ne.0,"bad list of channels")

      IF( nfiles .GT. 1 .AND. nchans .GT. 1 .AND. nfiles .LT. nchans ) THEN
        err = 1
        THROWM(err.ne.0,"different number of files and channels")
      END IF
      
      DO ichan = 1, nchans
        channel = lchans(ichan) 
        IF( nfiles .EQ. 1 ) THEN
          filename = filenames(1)
        ELSE
          filename = filenames(ichan)
        END IF
        write(0,*) "viirs_edr_img_load channel ", channel
        write(0,*) "viirs_edr_img_load file ", filename

        CALL viirs_edr_img_load_channel( filename, bandname, channel, x, err )
        THROWM(ERR.ne.0,"Cannot read SDR for file"//TRIM(filename))

        call viirs_edr_img_data_proc( x%band(channel), err, clean = clean)
        THROWM(ERR.ne.0,"Cannot process SDR for channel"//x%band(channel)%channel_name)

      END DO

      !get Geolocation data
      IF( lget_geo ) THEN
        IF ( present(geolocfile) ) THEN
           geoname = geolocfile
        ELSE 
          IF ( LEN(TRIM(x%root_att%N_GEO_Ref)) .GT. 0 ) THEN
      
            geoname = TRIM(aapp_dirname(filenames(1)))//TRIM(x%root_att%N_GEO_Ref)
        
          ELSE
      
            geoname = filename
          END IF
        END IF
        write(0,*) "viirs_edr_img_geo_load ", geoname
            
        CALL viirs_edr_img_geo_load( geoname, bandname, x%geo, err )
        THROWM(err.ne.0,"Cannot load GEO from"//TRIM(geoname))
        write(0,*) "viirs_edr_img_geo_load fin"
        
        CALL viirs_edr_img_geo_proc( x%geo, x%agg_att, err )
        THROWM(err.ne.0,"Cannot Proc GEO")

      END IF
      
CATCH
      end subroutine viirs_edr_img_load
   
      subroutine viirs_edr_img_load_channel( filename, bandname, channel, x, err )
      ! loads "All_Data" for a given channel/band from HDF5 file filename
      ! loads root attributes
      ! loads aggregate attributes
      use hdf5
      USE H5LT
      USE aapp_hdf_mod


      implicit none

      character(len=*), intent(in) :: filename ! Name of file
      character(len=*), intent(in) :: bandname ! VIIRS Band Name: I M or DNB
      integer, intent(in) :: channel
      type(viirs_edr_img), intent(inout) :: x
      integer,intent(out)::err

      type(jpss_meta_aggregate) :: att
      type(jpss_meta_granule)   :: att_gra
      
      character(len=256) :: gname, dname, aname, sname
      character(len=16)   :: channel_name
      character(len=3)   :: band_name

      integer :: ifov, i
      character(len=1) :: nband
      !character(len=4) :: Band_ID

      INTEGER(HID_T) :: file_id, group_id, dataset_id, attr_id

      LOGICAL :: attr_exists       ! .TRUE. if exists, .FALSE. otherwise
      
TRY

      nband = ADJUSTL(bandname)
      channel_name = ''
      Select Case ( nband )
        case ('i','I')
          write(channel_name,'("I",i1,"-IMG")') channel
        case ('m','M')
          Select Case ( channel ) 
            case (1)
              channel_name="M1ST"
            case (2)
              channel_name="M2ND"
            case (3)
              channel_name="M3RD"
            case (4)
              channel_name="M4TH"
            case (5)
              channel_name="M5TH"
            case (6)
              channel_name="M6TH"
            case default
              err = 1
              THROWM(err.ne.0,"invalid channel number")
          End Select
        case default
          channel_name='NCC'
      End Select

      write(0,*) 'deb ',channel_name
   
      CALL H5FOPEN_F( TRIM(filename), H5F_ACC_RDONLY_F, File_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open File"//TRIM(Filename))
     
      gname = "All_Data/VIIRS-"//TRIM(channel_name)//"-EDR_All"

      CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

      ! Take care the xxx_rh routines are doing an init, so it
      ! resets all variables
      Call viirs_edr_img_data_rh(x%band(channel),group_id,err)
      THROWM(ERR.ne.0,"BAD RETURN")
      
      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))
      

      !/****** Read the top-level attributes ******/  
      Call jpss_meta_root_rh(x%root_att,file_id,"/",err)
      THROWM(ERR.ne.0,"BAD RETURN")
      
    
      write(*,*) "Mission_Name ", x%root_att%Mission_Name
      write(*,*) "N_GEO_Ref ", x%root_att%N_GEO_Ref


      gname = "Data_Products/VIIRS-"//TRIM(channel_name)//"-EDR"
      CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

      dname = "VIIRS-"//TRIM(channel_name)//"-EDR_Aggr" 

      CALL jpss_meta_aggregate_rh(att,group_id,dname,err)
      THROWM(err.ne.0,"Cannot read aggegate attributes for "//TRIM(dname))
     

      dname = "VIIRS-"//TRIM(channel_name)//"-EDR_Gran_0" 

      CALL jpss_meta_granule_rh(att_gra,group_id,dname,err)
      THROWM(err.ne.0,"Cannot read granule_0 attributes for "//TRIM(dname))

      IF( att_gra%Band_ID .EQ. "" ) THEN
        att_gra%Band_ID(1:1) = bandname(1:1)
        Write(att_gra%Band_Id(2:2),"(i1)") channel
      ENDIF



      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))
      
      CALL viirs_edr_img_checkaggregate( x, att, err )
      THROWM(err.ne.0,"Cannot check Aggregate Attributes")

      CALL jpss_meta_aggregate_copy(x%agg_att, att, err)
      THROWM(err.ne.0,"Cannot Copy Aggregate Attributes")


      IF( att_gra%Band_ID(1:1) .EQ. "I" ) THEN
        x%band(channel)%channel_name = TRIM(att_gra%Band_ID)
        Read( x%band(channel)%channel_name, '(1x,I1)') x%band(channel)%channel
      ELSEIF ( att_gra%Band_ID(1:1) .EQ. "M" ) THEN
        x%band(channel)%channel_name = TRIM(att_gra%Band_ID)
        IF ( LEN_TRIM(att_gra%Band_ID) .EQ. 2 ) THEN
          READ( x%band(channel)%channel_name, '(1x,I1)') x%band(channel)%channel
        ELSE IF( LEN_TRIM(att_gra%Band_ID) .EQ. 3 ) THEN
          READ( x%band(channel)%channel_name, '(1x,I2)') x%band(channel)%channel
        END IF
      ELSE
        x%band(channel)%channel_name = "NCC"
        x%band(channel)%channel      = 1
      END IF

      write(0,*) "x%band(channel)%channel_name ",x%band(channel)%channel_name
      write(0,*) "x%band(channel)%channel ",x%band(channel)%channel

      CALL h5fclose_f(file_id, err) 
      THROWM(err.ne.0,"Cannot close file "//TRIM(Filename))

CATCH
      end subroutine viirs_edr_img_load_channel
  
      subroutine viirs_edr_img_geo_load( filename, bandname, x, err )
      ! loads Geolocation for a given band from HDF5 file filename
      use hdf5
      USE H5LT
      USE aapp_hdf_mod
      USE viirs_edr_img_geo_def
      USE viirs_edr_img_geo_io
      USE viirs_edr_img_geo_mem


      implicit none

      character(len=*), intent(in) :: filename
      character(len=*), intent(in) :: bandname ! VIIRS Band Name: I M or DNB
      type(viirs_edr_img_geo), intent(inout) :: x
      integer,intent(out)::err

      character(len=256) :: gname
      character(len=1)   :: nband

      INTEGER(HID_T) :: file_id, group_id      
TRY
      nband = ADJUSTL(bandname)
      Select Case ( nband )
        case ('i','I')
          gname = "All_Data/VIIRS-IMG-GTM-EDR-GEO_All"
        case ('m','M')
          gname = "All_Data/VIIRS-MOD-GTM-EDR-GEO_All"
        case default
          gname = "All_Data/VIIRS-NCC-EDR-GEO_All"
      End Select

      CALL H5FOPEN_F( TRIM(filename), H5F_ACC_RDONLY_F, File_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open File"//TRIM(Filename))


      CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

      Call viirs_edr_img_geo_rh(x,group_id,err)
      THROWM(err.ne.0,"Cannot Read Geolocation for "//TRIM(gname))
      
      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))

      CALL h5fclose_f(file_id, err) 
      THROWM(err.ne.0,"Cannot close file "//TRIM(Filename))

CATCH
      end subroutine viirs_edr_img_geo_load
      
      subroutine viirs_edr_img_info( x, nchannels, npixels, nlines, nscans, ngranules, err )
      ! returns number of pixels/lines/scans/granules
      ! returns the real number of channels loaded
      implicit none

      type(viirs_edr_img), intent(in) :: x
      integer,intent(out)::err
      integer, intent(out) :: nchannels, npixels, nlines, nscans, ngranules
      integer :: channel, mchannels
TRY
        
      nchannels = 0
      ngranules = 0
      nscans    = 0
      nlines    = 0
      npixels   = 0
      
      mchannels = size(x%band)
      DO channel = 1, mchannels
        IF(x%band(channel)%channel .NE. 0 ) THEN
          nchannels = nchannels + 1
        END IF
      END DO

 write(0,*) "mchannels ", mchannels
 write(0,*) "nchannels ", nchannels
 write(0,*) "channels ", x%band(:)%channel

      DO channel = 1, mchannels
        IF(x%band(channel)%channel .NE. 0 ) THEN
          ngranules   = x%agg_att%AggregateNumberGranules
        !  nscans      = size(x%band(channel)%ModeScan)          
          IF(associated(x%band(channel)%Reflectances)) THEN
            nlines    = size(x%band(channel)%Reflectances,2)
            npixels   = size(x%band(channel)%Reflectances,1)
          ELSE IF(associated(x%band(channel)%Radiance)) THEN
            nlines    = size(x%band(channel)%Radiance,2)
            npixels   = size(x%band(channel)%Radiance,1)
          ELSE IF(associated(x%band(channel)%BrightnessTemperature)) THEN
            nlines    = size(x%band(channel)%BrightnessTemperature,2)
            npixels   = size(x%band(channel)%BrightnessTemperature,1)
          ELSE IF(associated(x%band(channel)%BrightnessTemperatureOrReflectance)) THEN
            nlines    = size(x%band(channel)%BrightnessTemperatureOrReflectance,2)
            npixels   = size(x%band(channel)%BrightnessTemperatureOrReflectance,1)
          ELSE IF(associated(x%band(channel)%Albedos)) THEN
            nlines    = size(x%band(channel)%Albedos,2)
            npixels   = size(x%band(channel)%Albedos,1)
          ELSE
            err = 1 
            THROWM(ERR.ne.0,"Cannot determine number of lines")
          END IF
        
          IF(associated(x%band(channel)%ReflectanceFactors)) THEN
            ngranules = size(x%band(channel)%ReflectanceFactors,1) / 2
          ELSE IF(associated(x%band(channel)%RadianceFactors)) THEN
            ngranules = size(x%band(channel)%RadianceFactors,1) / 2
          ELSE IF(associated(x%band(channel)%BrightnessTemperatureFactors)) THEN
            ngranules = size(x%band(channel)%BrightnessTemperatureFactors,1) / 2
          ELSE IF(associated(x%band(channel)%BrightnessTemperatureOrReflectanceFactors)) THEN
            ngranules = size(x%band(channel)%BrightnessTemperatureOrReflectanceFactors,1) / 2
          ELSE IF(associated(x%band(channel)%AlbedoFactors)) THEN
            ngranules = size(x%band(channel)%AlbedoFactors,1) / 2
          ELSE
            err = 1 
            THROWM(ERR.ne.0,"Cannot determine number of granules")
          END IF

          EXIT
        ENDIF
      END DO
      

CATCH
      end subroutine viirs_edr_img_info
      
      
      
      subroutine viirs_edr_img_data_proc( x, err, clean)
      ! processing of the data part of the structure viirs_edr_img_data
      ! applies scaling factors according to channels

      implicit none

      type(viirs_edr_img_data), intent(inout) :: x
      integer,intent(out)::err
      logical, optional, intent(in) :: clean  ! if TRUE remove unsed arrays 

      logical :: scale_rad
      logical :: lclean
      integer :: nlines, npixels, ngranules
      integer :: line, pixel, granule
      integer :: l1, l2, goff
      integer :: lines_per_granule
TRY
        lclean = .false.
        IF ( Present( clean )) THEN
          lclean = clean
        END IF
        
        IF(associated(x%Reflectance)) THEN
          nlines    = size(x%Reflectance,2)
          npixels   = size(x%Reflectance,1)
          ngranules = size(x%ReflectanceFactors,1) / 2
        ELSE IF(associated(x%Radiance)) THEN
          nlines    = size(x%Radiance,2)
          npixels   = size(x%Radiance,1)
          ngranules = size(x%RadianceFactors,1) / 2
        ELSE IF(associated(x%BrightnessTemperature)) THEN
          nlines    = size(x%BrightnessTemperature,2)
          npixels   = size(x%BrightnessTemperature,1)
          ngranules = size(x%BrightnessTemperatureFactors,1) / 2
        ELSE IF(associated(x%BrightnessTemperatureOrReflectance)) THEN
          nlines    = size(x%BrightnessTemperatureOrReflectance,2)
          npixels   = size(x%BrightnessTemperatureOrReflectance,1)
          ngranules = size(x%BrightnessTemperatureOrReflectanceFactors,1) / 2
        ELSE IF(associated(x%Albedo)) THEN
          nlines    = size(x%Albedo,2)
          npixels   = size(x%Albedo,1)
          ngranules = size(x%AlbedoFactors,1) / 2
        ELSE
          err = 1 
          THROWM(ERR.ne.0,"Cannot determine number of lines")
        END IF
        lines_per_granule = nlines/ngranules

      write(*,*) 'number of granules', ngranules
      write(*,*) 'number of lines', nlines
      write(*,*) 'number of pixels', npixels
      write(*,*) 'lines_per_granule', lines_per_granule

        
      write(0,*) x%Channel_name

      IF( associated(x%Reflectance) ) THEN

        ! apply scaling factors to Relectances

        IF(.NOT. associated(x%Reflectances)) then
          ALLOCATE( x%reflectances(size(x%reflectance,1),size(x%reflectance,2)), STAT=ERR)
          THROWM(ERR.ne.0,"Cannot allocate Relectances (real values) ")
        ENDIF
        write(0,*) "reflec , chan ",x%channel
        write(0,*) "reflec fact ",x%ReflectanceFactors
        write(*,*) size(x%reflectance,1) ! pixels
        write(*,*) size(x%reflectance,2) ! lines
        
        DO granule = 1, ngranules
          l1 = (granule - 1) * lines_per_granule +1
          l2 = min( l1+lines_per_granule-1, nlines)
          goff = (granule-1)*2
          DO line = l1,l2
            DO pixel = 1,npixels

              IF( x%reflectance(pixel,line) .LT. 65500 ) THEN
                 x%reflectances(pixel,line) = &
                   & x%reflectance(pixel,line) * x%ReflectanceFactors(1+goff) + &
                   & x%ReflectanceFactors(2+goff) 
              ELSE
                x%reflectances(pixel,line) = -999.9
              END IF
            END DO
          END DO
        END DO
        IF ( lclean ) THEN
          deallocate(x%reflectance,stat=err)
          THROWM(err.ne.0,"CANNOT DEALLOCATE reflectance")
          nullify(x%reflectance)
          x%ReflectanceFactors(1) = 1.
          x%ReflectanceFactors(2) = 0.
        ENDIF
      ENDIF
      
           
      IF(associated(x%Radiance)) THEN

          write(0,*) "Rad , chan ",x%channel
          write(0,*) "Rad fact ",x%RadianceFactors

        DO granule = 1, ngranules
          l1 = (granule - 1) * lines_per_granule +1
          l2 = min( l1+lines_per_granule-1, nlines)
          goff = (granule-1)*2
          DO line = l1,l2
            DO pixel = 1,npixels

              IF( x%Radiance(pixel,line) .LT. 65500 ) THEN
                 x%Radiance(pixel,line) = &
                   & x%Radiance(pixel,line) * x%RadianceFactors(1+goff) + &
                   & x%RadianceFactors(2+goff) 
              ELSE
                x%Radiance(pixel,line) = -999.9
              END IF
            END DO
          END DO
          ! clear scaling factors when applied
          x%RadianceFactors(1+goff) = 1.
          x%RadianceFactors(2+goff) = 0.
        END DO
      END IF
      
      
      IF(associated(x%BrightnessTemperature)) THEN

        write(0,*) "BT , chan ",x%channel
        write(0,*) "BT fact ",x%BrightnessTemperatureFactors
          ! apply scaling factors to Brightness Temperatures

        DO granule = 1, ngranules
          l1 = (granule - 1) * lines_per_granule +1
          l2 = min( l1+lines_per_granule-1, nlines)
          goff = (granule-1)*2
          DO line = l1,l2
            DO pixel = 1,npixels

              IF( x%BrightnessTemperature(pixel,line) .LT. 65500 ) THEN
                 x%BrightnessTemperature(pixel,line) = &
                   & x%BrightnessTemperature(pixel,line) * x%BrightnessTemperatureFactors(1+goff) + &
                   & x%BrightnessTemperatureFactors(2+goff) 
              ELSE
                x%BrightnessTemperature(pixel,line) = -999.9
              END IF
            END DO
          END DO
          ! clear scaling factors when applied
          x%BrightnessTemperatureFactors(1+goff) = 1.
          x%BrightnessTemperatureFactors(2+goff) = 0.
        END DO

      END IF


      IF( associated(x%Albedo) ) THEN

        ! apply scaling factors to Relectances

        IF(.NOT. associated(x%Albedos)) then
          ALLOCATE( x%Albedos(size(x%Albedo,1),size(x%Albedo,2)), STAT=ERR)
          THROWM(ERR.ne.0,"Cannot allocate Albedos (real values) ")
        ENDIF
        write(0,*) "Albedo , chan ",x%channel
        write(0,*) "Albedo fact ",x%AlbedoFactors
        write(*,*) size(x%Albedo,1) ! pixels
        write(*,*) size(x%Albedo,2) ! lines
        
        DO granule = 1, ngranules
          l1 = (granule - 1) * lines_per_granule +1
          l2 = min( l1+lines_per_granule-1, nlines)
          goff = (granule-1)*2
          DO line = l1,l2
            DO pixel = 1,npixels

              IF( x%Albedo(pixel,line) .LT. 65500 ) THEN
                 x%Albedos(pixel,line) = &
                   & x%Albedo(pixel,line) * x%AlbedoFactors(1+goff) + &
                   & x%AlbedoFactors(2+goff) 
              ELSE
                x%Albedos(pixel,line) = -999.9
              END IF
            END DO
          END DO
        END DO
        IF ( lclean ) THEN
          deallocate(x%Albedo,stat=err)
          THROWM(err.ne.0,"CANNOT DEALLOCATE Albedo")
          nullify(x%Albedo)
          x%AlbedoFactors(1) = 1.
          x%AlbedoFactors(2) = 0.
        ENDIF
      ENDIF
      
CATCH
      end subroutine viirs_edr_img_data_proc
      
      subroutine viirs_edr_img_geo_proc( x, att, err )
      ! processing of the geelocation structure viirs_edr_img_geo
      ! calculates the TAI offset and calculates the UTC time for each scan.    
      implicit none

      type(viirs_edr_img_geo), intent(inout) :: x
      type(jpss_meta_aggregate), intent(in) :: att

      integer,intent(out)::err

      integer :: nscans, ngranules
      integer :: iscan
      integer :: taioffset ! offset UTC TAI in seconds
      integer :: taioffset_ms
      integer :: hour, minute
      real*8  :: second
      real*8  :: secs
      !integer*8 :: 

      
TRY
      ! Estimate difference TAI UTC in seconds
      ! AggregateBeginningTime is UTC
      ! Starting Time is IET (1/1/1958) microsecond
      write(0,*) 'x%Time(1) ',x%Time(1)
      write(0,*) 'x%Time(1) secs of day',MOD(x%Time(1),86400000000_8)/1000000.0

      read(att%AggregateBeginningTime,"(i2,i2,f9.6)") hour, minute, second
      secs = hour*3600.0 + minute*60.0 + second
      write(0,*) 'att%AggregateBeginningTime ',att%AggregateBeginningTime
      write(0,*) '    hour, minute, second ',hour, minute, second
      write(0,*) '    secs of day ',secs
      
      read (att%AggregateBeginningDate,"(I4,I2,I2)") &
           & x%startyear, x%startmonth, x%startday
      
      write(0,*) "x%startyear, x%startmonth, x%startday ",&
           & x%startyear, x%startmonth, x%startday

      IF( x%Time(1) .GT. 0 ) THEN
        taioffset = floor((MOD(x%Time(1),86400000000_8))/1000000.0 - secs + 0.5)
        write(0,*) 'taioffset ',taioffset
        if (taioffset < 0) taioffset = taioffset + 86400
        write(0,*) 'taioffset ',taioffset
      ELSE
        ERR = 1
        THROWM(ERR.NE.0,"Unable to determine TAI offset ")
      END IF
      taioffset_ms = taioffset * 1000


      IF(.NOT. associated(x%UTC)) then
        ALLOCATE( x%UTC(size(x%Time)), STAT=ERR)
        THROWM(ERR.ne.0,"Cannot allocate UTC  ")
      ENDIF

      WHERE ( x%Time > 0 )
        x%UTC = MOD(x%Time,86400000000_8)/1000_8 - taioffset_ms
      ELSEWHERE
        x%UTC = -999
      ENDWHERE

      write(*,*) 'latitude', size(x%latitude,1),size(x%latitude,2)
      write(*,*) 'SatelliteAzimuthAngle', size(x%SatelliteAzimuthAngle,1),size(x%SatelliteAzimuthAngle,2)
      write(*,*) 'Height', size(x%Height,1), size(x%Height,2)
                

CATCH
      end subroutine viirs_edr_img_geo_proc
      
      subroutine viirs_edr_img_checkaggregate( x, y, err )
    ! verifies agg_att Y is the same as the one contained in X, if any
    
      implicit none

      type(viirs_edr_img), intent(inout) :: x
      type(jpss_meta_aggregate), intent(inout) :: y
      INTEGER, intent(out) :: ERR         ! Error code 


TRY
      
      IF( x%agg_att%AggregateBeginningOrbitNumber .EQ. 0 ) RETURN
      
      IF ( y%AggregateBeginningDate .NE. x%agg_att%AggregateBeginningDate ) ERR=1
      THROWM(ERR.NE.0,"different AggregateBeginningDate")

      IF ( y%AggregateBeginningGranuleID .NE. x%agg_att%AggregateBeginningGranuleID ) ERR=1
      THROWM(ERR.NE.0,"different AggregateBeginningGranuleID")
        
      IF ( y%AggregateBeginningOrbitNumber .NE. x%agg_att%AggregateBeginningOrbitNumber ) ERR=1
      THROWM(ERR.NE.0,"different AggregateBeginningOrbitNumber")
        
      IF ( y%AggregateBeginningTime .NE. x%agg_att%AggregateBeginningTime ) ERR=1
      THROWM(ERR.NE.0,"different AggregateBeginningTime")
        
      IF ( y%AggregateEndingDate .NE. x%agg_att%AggregateEndingDate ) ERR=1
      THROWM(ERR.NE.0,"different AggregateEndingDate")

      IF ( y%AggregateEndingGranuleID .NE. x%agg_att%AggregateEndingGranuleID ) ERR=1
      THROWM(ERR.NE.0,"different AggregateEndingGranuleID")
        
      IF ( y%AggregateEndingOrbitNumber .NE. x%agg_att%AggregateEndingOrbitNumber ) ERR=1
      THROWM(ERR.NE.0,"different AggregateEndingOrbitNumber")
        
      IF ( y% AggregateEndingTime .NE. x%agg_att%AggregateEndingTime ) ERR=1
      THROWM(ERR.NE.0,"different AggregateEndingTime")
        
      IF ( y% AggregateNumberGranules .NE. x%agg_att%AggregateNumberGranules ) ERR=1
      THROWM(ERR.NE.0,"different AggregateNumberGranules")
            
CATCH
      end subroutine viirs_edr_img_checkaggregate
      
Character*256 Function aapp_dirname( path )
  Character(len=*), Intent(in) :: path
  
  Integer :: i
  aapp_dirname = ""
  i = Len( Trim( path ) ) - 1
  Do
    If( i .le. 0 ) Return
    If( path(i:i) .eq. '/' ) Exit
    i = i - 1
  EndDo
  aapp_dirname = path(1:i)
End Function

Character*256 Function aapp_basename( path )
  Character(len=*), Intent(in) :: path
  
  Integer :: i
  aapp_basename = ""
  i = Len( Trim( path ) ) - 1
  Do
    If( i .le. 0 ) Then
      i = 0
      Exit
    EndIf
    If( path(i:i) .eq. '/' ) Exit
    i = i - 1
  EndDo
  aapp_basename = path(i+1:)
End Function

#endif

END MODULE viirs_edr_img_MOD
