! Name: $Id: viirs_sdr_mod.F90
!
! Purpose: memory routines for multi chanel/multi granule VIIRS SDR files in HDF5 format
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!   1.0   05/2012       Initial version (Pascal Brunel)
!   1.1   10/12/2012    Release for AAPP Pascale Roquet)

module viirs_sdr_mem
#include "aapp_throw.h"
Use viirs_sdr_def

Use jpss_meta_root_mem
Use jpss_meta_aggregate_mem
Use jpss_meta_granule_mem
Use jpss_meta_dataproducts_mem
Use viirs_sdr_data_mem
Use viirs_sdr_geo_mem

implicit none
private
public :: viirs_sdr_free
public :: viirs_sdr_init
public :: viirs_sdr_copy
public :: viirs_sdr_paste


contains

subroutine viirs_sdr_free(x,err)
type(viirs_sdr),intent(inout)::x
integer,intent(out)::err
integer::ichan, nchan
TRY
err = 0

Call jpss_meta_root_free(x%root_att,err)
THROWM(err.ne.0,"CANNOT FREE Rot Attributes")


Call jpss_meta_aggregate_free(x%agg_att,err)
THROWM(err.ne.0,"CANNOT Free Aggregate Attributes")

If(associated(x%band))then
  nchan = size( x%band )
  Do ichan=1,nchan
  
    Call jpss_meta_granule_free(x%gran0_att(ichan),err)
    THROWM(err.ne.0,"CANNOT Free Gran_0 Attributes")

    Call jpss_meta_dataproducts_free(x%datap_att(ichan),err)
    THROWM(err.ne.0,"CANNOT Free Data Products Band data")

    Call viirs_sdr_data_free(x%band(ichan),err)
    THROWM(err.ne.0,"CANNOT Free Band data")
  End Do
  deallocate(x%band,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE Band data")
  nullify(x%band)
End If

Call viirs_sdr_geo_free(x%geo,err)
THROWM(err.ne.0,"CANNOT Free Geolocation")

CATCH
end subroutine

subroutine viirs_sdr_copy(x,y,err,data)
type(viirs_sdr),intent(inout)::x
type(viirs_sdr),intent(in)::y
integer,intent(out)::err
logical, optional :: data
integer::lb(7),ub(7)
logical :: data1
integer :: ichan, nchan

TRY

  data1=.true.
  if(present(data))data1 = data
  
  Call jpss_meta_root_copy(x%root_att,y%root_att,err,data)
  THROWM(err.ne.0,"CANNOT Copy Rot Attributes")

  Call jpss_meta_aggregate_copy(x%agg_att,y%agg_att,err,data)
  THROWM(err.ne.0,"CANNOT Copy Aggregate Attributes")
  

  If(associated(y%band))then
    nchan = size( y%band )
    allocate(x%datap_att(nchan),stat=err)
    THROWM(err.ne.0,"CANNOT ALLOCATE SDR DataP bands")

    allocate(x%band(nchan),stat=err)
    THROWM(err.ne.0,"CANNOT ALLOCATE SDR bands")

    allocate(x%gran0_att(nchan),stat=err)
    THROWM(err.ne.0,"CANNOT ALLOCATE SDR Gran_0 Att bands")

    Do ichan=1,nchan
    
      Call jpss_meta_granule_copy(x%gran0_att(ichan),y%gran0_att(ichan),err,data)
      THROWM(err.ne.0,"CANNOT Copy Gran_0 Attributes")

      Call jpss_meta_dataproducts_copy(x%datap_att(ichan),y%datap_att(ichan),err,data)
      THROWM(err.ne.0,"CANNOT Copy Aggregate Attributes")

      Call viirs_sdr_data_copy(x%band(ichan),y%band(ichan),err,data)
      THROWM(err.ne.0,"CANNOT Copy Band data")
    End Do
  End If

  Call viirs_sdr_geo_copy(x%geo,y%geo,err,data)
  THROWM(err.ne.0,"CANNOT Copy Geolocation")

CATCH
end subroutine

subroutine viirs_sdr_paste(x,y,z,err,data)
type(viirs_sdr),intent(inout)::x
type(viirs_sdr),intent(in)::y,z
integer,intent(out)::err
logical, optional :: data
integer::lb(7),ub(7)
logical :: data1
integer :: ichan, nchan

TRY

  data1=.true.
  if(present(data))data1 = data
  
  ! For ROOT attributes just copy Y or Z
  Call jpss_meta_root_copy(x%root_att,y%root_att,err,data)
  THROWM(err.ne.0,"CANNOT Paste Rot Attributes")

  Call jpss_meta_aggregate_paste(x%agg_att,y%agg_att,z%agg_att,err,data)
  THROWM(err.ne.0,"CANNOT Paste Aggregate Attributes")  
  x%agg_att%AggregateEndingDate        = z%agg_att%AggregateEndingDate
  x%agg_att%AggregateEndingGranuleID   = z%agg_att%AggregateEndingGranuleID
  x%agg_att%AggregateEndingOrbitNumber = z%agg_att%AggregateEndingOrbitNumber 
  x%agg_att%AggregateEndingTime        = z%agg_att%AggregateEndingTime 
  x%agg_att%AggregateNumberGranules    = x%agg_att%AggregateNumberGranules + &
                                         z%agg_att%AggregateNumberGranules


  If(associated(y%band))then
    nchan = size( y%band )
    
    allocate(x%datap_att(nchan),stat=err)
    THROWM(err.ne.0,"CANNOT ALLOCATE SDR DataP bands")
    allocate(x%band(nchan),stat=err)
    THROWM(err.ne.0,"CANNOT ALLOCATE SDR bands")
    allocate(x%gran0_att(nchan),stat=err)
    THROWM(err.ne.0,"CANNOT ALLOCATE SDR Gran_0 Att bands")

    Do ichan=1,nchan
    
      Call jpss_meta_dataproducts_paste(x%datap_att(ichan), &
                          & y%datap_att(ichan),z%datap_att(ichan),err,data)
      THROWM(err.ne.0,"CANNOT Copy Aggregate Attributes")

      Call jpss_meta_granule_paste(x%gran0_att(ichan),&
                          & y%gran0_att(ichan),z%gran0_att(ichan),err,data)
      THROWM(err.ne.0,"CANNOT Paste Gran_0 Attributes")

      Call viirs_sdr_data_paste(x%band(ichan),y%band(ichan),z%band(ichan),err,data)
      THROWM(err.ne.0,"CANNOT Paste Band data")
    End Do
  End If

  Call viirs_sdr_geo_paste(x%geo,y%geo,z%geo,err,data)
  THROWM(err.ne.0,"CANNOT Paste Geolocation")

CATCH
end subroutine

subroutine viirs_sdr_init(x, band, err)
type(viirs_sdr),intent(inout)::x
character(len=*) :: band   ! Band name I;M;DN
integer,intent(out)::err

character(len=1)::nband
integer::ichan, nchan

TRY

  Call jpss_meta_root_init(x%root_att)

  Call jpss_meta_aggregate_init(x%agg_att)

  nband = ADJUSTL(band)
  Select Case ( nband )
    case ('i','I')
      nchan = 5
    case ('m','M')
      nchan = 16
    case default
      nchan = 1
  End Select
  
  allocate(x%datap_att(nchan),stat=err)
  THROWM(err.ne.0,"CANNOT ALLOCATE SDR DataP bands")

  allocate(x%band(nchan),stat=err)
  THROWM(err.ne.0,"CANNOT ALLOCATE SDR bands")

  allocate(x%gran0_att(nchan),stat=err)
  THROWM(err.ne.0,"CANNOT ALLOCATE SDR Gran_0 Att bands")

  Do ichan=1,nchan
    Call jpss_meta_dataproducts_init(x%datap_att(ichan))

    Call jpss_meta_granule_init(x%gran0_att(ichan))

    Call viirs_sdr_data_init(x%band(ichan))
  End Do

  Call viirs_sdr_geo_init(x%geo)

CATCH
end subroutine

end module
