! Name: $Id: viirs_sdr_mod.F90
!
! Purpose: In this module, we process VIIRS SDR files in HDF5 format
!
! Method:
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!   1.0   05/2012       Initial version (Pascal Brunel)
!   1.1   10/12/2012    Release for AAPP Pascale Roquet)
!
!
!
MODULE VIIRS_SDR_MOD

#ifdef HAS_LIBHDF5

#include "aapp_throw.h"
      USE jpss_meta_root_def
      USE jpss_meta_root_io
      USE jpss_meta_root_mem
      USE jpss_meta_aggregate_def
      USE jpss_meta_aggregate_io
      USE jpss_meta_aggregate_mem
      USE jpss_meta_granule_def
      USE jpss_meta_granule_io
      USE jpss_meta_granule_mem
      USE jpss_meta_dataproducts_def
      USE jpss_meta_dataproducts_io
      USE jpss_meta_dataproducts_mem
     
      USE viirs_sdr_data_def
      USE viirs_sdr_data_io
      USE viirs_sdr_data_mem

      USE viirs_sdr_geo_def
     
      USE viirs_sdr_def
      USE viirs_sdr_mem

      IMPLICIT NONE

      !PUBLIC :: viirs_sdr_data_proc
      PUBLIC :: viirs_sdr_load
      PUBLIC :: viirs_sdr_info
      !PUBLIC :: viirs_sdr_geo_load
      PUBLIC :: viirs_sdr_save
      PUBLIC :: aapp_basename
      PRIVATE

      CONTAINS
      subroutine viirs_sdr_load( bandname, x, filenames, err, channels , geolocfile, no_geo, clean)
      ! Loads and pre-process all VIIRS data for a given Band according to options, returns x:
      ! If only one filename, it will be used for all channels
      ! If channels is present then corresponding channels will be read
      !   if not, all channels of given Band are read from one file or from the list of files
      ! If no_geo is present and false, or not present
      !   geolocation file is loaded from geolocfile (if present) or from
      !   the root attribute N_GEO_Ref, but same directory as filenames(1). Then geolocation is processed.
      !
      use hdf5
      USE H5LT
      USE aapp_hdf_mod


      implicit none

      character(len=*), intent(in) :: bandname ! VIIRS Band Name: I M or DNB
      type(viirs_sdr),  intent(inout) :: x
      character(len=*), intent(in) :: filenames(:) ! Name of file (one for all or one per channel)

      integer,          intent(out) ::err
      integer, optional, intent(in) :: channels(:) ! if present, the list of channels
      character(len=*), optional, intent(in) ::  geolocfile ! file name for geolocation
      logical, optional, intent(in) :: no_geo ! if TRUE geolocation is not loaded 
      logical, optional, intent(in) :: clean  ! if TRUE remove unsed arrays 

      type(jpss_meta_aggregate) :: att
            
            
      character(len=256) :: filename
      character(len=256) :: geoname
      character(len=256) :: gname, dname, aname
      character(len=3)   :: channel_name

      integer :: nfiles
      integer :: ifov
      integer :: ichan, nchans, maxchans, channel
      integer :: lchans(16)
      character(len=1) :: nband

      INTEGER(HID_T) :: file_id, group_id, dataset_id, attr_id

      LOGICAL :: attr_exists       ! .TRUE. if exists, .FALSE. otherwise
      LOGICAL :: lget_geo
      
TRY
      lget_geo = .TRUE.
      IF( present(no_geo) ) THEN
        lget_geo = .NOT. no_geo
      END IF

      CALL viirs_sdr_init(x, bandname, err)
      THROWM(ERR.ne.0,"Cannot init SDR ")

      nband = bandname
      Select Case ( nband )
        case ('i','I')
          maxchans = 5
        case ('m','M')
          maxchans = 16
        case default
          maxchans = 1
      End Select
      
      nfiles = size(filenames)
      nchans = 0
      lchans = 0
      If ( present(channels) ) Then
        DO ichan = 1, size(channels)
          IF( channels(ichan) .GT. 0 .AND. channels(ichan) .LE. maxchans ) THEN
            nchans = nchans +1
            lchans(nchans) = channels(ichan)
          END IF
        END DO
      ELSE
        nchans = maxchans
        lchans (1:nchans) = (/ (ichan, ichan = 1, nchans) /)

      END IF
      
      IF( nchans .LE. 0 .OR. nchans .GT. maxchans ) err = 1
      THROWM(err.ne.0,"bad list of channels")

      IF( nfiles .GT. 1 .AND. nchans .GT. 1 .AND. nfiles .LT. nchans ) THEN
        err = 1
        THROWM(err.ne.0,"different number of files and channels")
      END IF
      
      DO ichan = 1, nchans
        channel = lchans(ichan) 
        IF( nfiles .EQ. 1 ) THEN
          filename = filenames(1)
        ELSE
          filename = filenames(ichan)
        END IF
#ifdef DEBUG
        write(0,*) "viirs_sdr_load channel ", channel
        write(0,*) "viirs_sdr_load from file ", TRIM(filename)
#endif
        CALL viirs_sdr_load_channel( filename, bandname, channel, x, err )
        THROWM(ERR.ne.0,"Cannot read SDR for file"//TRIM(filename))
#ifdef DEBUG
        write(0,*) "viirs_sdr_load_channel fin ", channel
#endif
        call viirs_sdr_data_proc( x%band(channel), err, clean = clean)
        THROWM(ERR.ne.0,"Cannot process SDR for channel"//x%band(channel)%channel_name)

      END DO
      
      !get Geolocation data
      IF( lget_geo ) THEN
        IF ( present(geolocfile) ) THEN
           geoname = geolocfile
        ELSE 
          IF ( LEN(TRIM(x%root_att%N_GEO_Ref)) .GT. 1 ) THEN
      
            geoname = TRIM(aapp_dirname(filenames(1)))//TRIM(x%root_att%N_GEO_Ref)
        
          ELSE
      
            geoname = filename
          END IF
        END IF
#ifdef DEBUG
        write(0,*) "viirs_sdr_geo_load file ", TRIM(geoname)
#endif           
        CALL viirs_sdr_geo_load( geoname, bandname, x%geo, err )
        THROWM(err.ne.0,"Cannot load GEO from"//TRIM(geoname))
#ifdef DEBUG
        write(0,*) "viirs_sdr_geo_load fin"
#endif
        CALL viirs_sdr_geo_proc( x%geo, x%agg_att, err )
        THROWM(err.ne.0,"Cannot Proc GEO")

      END IF
      
CATCH
      end subroutine viirs_sdr_load
   
   
      subroutine viirs_sdr_save( x, filename, err, compress)
      !
      use hdf5
      USE H5LT
      USE aapp_hdf_mod


      implicit none

      type(viirs_sdr),  intent(in) :: x
      character(len=*), intent(in) :: filename ! Name of file (one for all or one per channel)

      integer,          intent(out) :: err            
      logical, optional, intent(in) :: compress 

      character(len=256) :: gname, dname, aname
      character(len=3)   :: channel_name

      integer :: nfiles
      integer :: ifov
      integer :: ichan, nchans, maxchans, channel
      integer :: lchans(16)

      INTEGER(HID_T) :: file_id, group_id, dataset_id, attr_id
      INTEGER :: SAERR
      Logical :: savegeo = .true.
      
      type(jpss_meta_root) :: root_att

TRY
      Call jpss_meta_root_copy(root_att,x%root_att,err)
      THROWM(err.ne.0,"Cannot copy root attributes")


      If ( savegeo ) Then
         root_att%N_GEO_Ref=""
      End If
      
!Do not complain, the file may exist
!
      CALL H5ESET_AUTO_F( 0, SAERR )
      CALL H5FCREATE_F( TRIM(filename), H5F_ACC_TRUNC_F, FILE_ID, ERR )

      call jpss_meta_root_wh(root_att,file_id,"/",err)
      THROWM(err.ne.0,"Cannot write root attributes")

      CALL H5GCLOSE_F( FILE_ID, ERR )
      
      CALL H5ESET_AUTO_F( 1, SAERR )
      
      nchans = size( x%band )
      Do ichan=1,nchans
        If ( x%band(ichan)%channel .LE. 0 ) Cycle
       
        write(0,*) "viirs_sdr_save channel ", x%band(ichan)%channel_name
        write(0,*) "viirs_sdr_save file ", TRIM(filename)
        
        call viirs_sdr_save_channel( filename, x%band(ichan)%channel_name(1:1), ichan, x, err, compress=compress)
        THROWM(ERR.ne.0,"Cannot save SDR for file"//TRIM(filename))
#ifdef DEBUG
        write(0,*) "viirs_sdr_save_channel fin ", ichan
#endif
        IF( savegeo ) THEN
        CALL viirs_sdr_geo_save( filename, x%band(ichan)%channel_name(1:1), x%geo, err, compress=compress )
        THROWM(err.ne.0,"Cannot save GEO from"//TRIM(filename))
#ifdef DEBUG
        write(0,*) "viirs_sdr_geo_save fin"
#endif
        savegeo = .false.
        ENDIF

      END DO
      
CATCH
      end subroutine viirs_sdr_save 
   
      subroutine viirs_sdr_load_channel( filename, bandname, channel, x, err )
      ! loads "All_Data" for a given channel/band from HDF5 file filename
      ! loads root attributes
      ! loads aggregate attributes
      use hdf5
      USE H5LT
      USE aapp_hdf_mod


      implicit none

      character(len=*), intent(in) :: filename ! Name of file
      character(len=*), intent(in) :: bandname ! VIIRS Band Name: I M or DNB
      integer, intent(in) :: channel
      type(viirs_sdr), intent(inout) :: x
      integer,intent(out)::err

      type(jpss_meta_aggregate) :: att

      character(len=256) :: gname, dname, aname
      character(len=3)   :: channel_name
      character(len=3)   :: band_name

      integer :: ifov
      character(len=1) :: nband

      INTEGER(HID_T) :: file_id, group_id, dataset_id, attr_id

      LOGICAL :: attr_exists       ! .TRUE. if exists, .FALSE. otherwise
      
TRY

      nband = ADJUSTL(bandname)
      channel_name = ''
      Select Case ( nband )
        case ('i','I')
          write(channel_name,'("I",i1)') channel
        case ('m','M')
          if ( channel .LT. 10 ) Then
            write(channel_name,'("M",i1)') channel
          else 
            write(channel_name,'("M",i2)') channel
          end if
        case default
          channel_name='DNB'
      End Select
   
      CALL H5FOPEN_F( TRIM(filename), H5F_ACC_RDONLY_F, File_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open File"//TRIM(Filename))
     
      gname = "All_Data/VIIRS-"//TRIM(channel_name)//"-SDR_All"

      CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

      ! Take care the xxx_rh routines are doing an init, so it
      ! resets all variables
      Call viirs_sdr_data_rh(x%band(channel),group_id,err)
      THROWM(ERR.ne.0,"BAD RETURN")
      
      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))
      
      x%band(channel)%channel = channel
      x%band(channel)%channel_name = TRIM(channel_name)


      !/****** Read the top-level attributes ******/  
      Call jpss_meta_root_rh(x%root_att,file_id,"/",err)
      THROWM(ERR.ne.0,"BAD RETURN")
      

      !/****** Read the aggregate granule attributes ******/  
      gname = "Data_Products/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR"
      CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

      dname = "VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR_Aggr" 

      CALL jpss_meta_aggregate_rh(att,group_id,dname,err)
      THROWM(err.ne.0,"Cannot read attributes for "//TRIM(dname))

      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))
      
      CALL viirs_sdr_checkaggregate( x, att, err )
      THROWM(err.ne.0,"Cannot check Aggregate Attributes")

      CALL jpss_meta_aggregate_copy(x%agg_att, att, err)
      THROWM(err.ne.0,"Cannot Copy Aggregate Attributes")
      
      
!       !/****** Read the Granule-0 attributes ******/  
        ! keep only the last one
      gname = "Data_Products/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR"
      CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

      dname = "VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR_Gran_0" 

      CALL jpss_meta_granule_rh(x%gran0_att(channel),group_id,dname,err)
      THROWM(err.ne.0,"Cannot read attributes for "//TRIM(dname))

      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))
      


      !/****** Read the data product attributes ******/  
      dname = "/Data_Products/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR"

      CALL jpss_meta_dataproducts_rh(x%datap_att(channel),file_id,dname,err)
      THROWM(err.ne.0,"Cannot read attributes for "//TRIM(dname))

      CALL h5fclose_f(file_id, err) 
      THROWM(err.ne.0,"Cannot close file "//TRIM(Filename))

      
CATCH
      end subroutine viirs_sdr_load_channel

      subroutine viirs_sdr_save_channel( filename, bandname, channel, x, err, compress )
      ! Saves "All_Data" for a given channel/band to an HDF5 file filename
      ! saves root attributes
      ! saves aggregate attributes
      ! saves dataproduct attributes
      use hdf5
      USE H5LT
      USE aapp_hdf_mod


      implicit none

      character(len=*), intent(in) :: filename ! Name of file
      character(len=*), intent(in) :: bandname ! VIIRS Band Name: I M or DNB
      integer, intent(in)           :: channel
      type(viirs_sdr), intent(in)   :: x
      integer,intent(out)           :: err
      logical, optional, intent(in) :: compress 

      type(jpss_meta_aggregate) :: att

      character(len=256) :: gname, dname, aname
      character(len=3)   :: channel_name
      character(len=3)   :: band_name

      integer :: ifov
      character(len=1) :: nband

      INTEGER(HID_T) :: file_id, group_id, dataset_id, attr_id
      INTEGER :: SAERR
      LOGICAL :: attr_exists       ! .TRUE. if exists, .FALSE. otherwise
      INTEGER(HSIZE_T) :: DIMS(1)

TRY

      CALL H5FOPEN_F( TRIM(filename), H5F_ACC_RDWR_F, FILE_ID, ERR )
      THROWM(ERR.NE.0,"CANNOT OPEN "//TRIM(Filename))

! Group All_Data/VIIRS_channel-SDR_All
      gname = "/All_Data/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR_All"
#ifdef DEBUG
      write(0,*) "gname ",TRIM(gname)
#endif
      CALL MKPAR( file_id, gname, group_id, err )
      THROWM(err.ne.0,"Cannot create group "//TRIM(gname))

! Write data for channel
      Call viirs_sdr_data_wh(x%band(channel),group_id,err,compress=compress)
      THROWM(ERR.ne.0,"BAD RETURN")
      
      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))


! Group /Data_Products/VIIRS-channel-SDR/VIIRS-channel-SDR_Aggr" 
      gname =  "/Data_Products/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR"
      dname = TRIM(gname)//"/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR_Aggr" 
      DIMS(1) = 1
      CALL MKDSR( file_id, dname, dataset_id, err, dims )
      THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))

! Write channel Aggregate meta data
      CALL jpss_meta_aggregate_wh(x%agg_att,dataset_id,dname,err)
      THROWM(err.ne.0,"Cannot write attributes for "//TRIM(dname))
      
      CALL h5dclose_f( dataset_id, err)
      THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


! Group /Data_Products/VIIRS-channel-SDR/VIIRS-channel-SDR_Gran_0" 
      gname = "/Data_Products/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR"
      dname = TRIM(gname)//"/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR_Gran_0" 
      DIMS(1) = 1
      CALL MKDSR( file_id, dname, dataset_id, err, dims )
      THROWM(ERR.ne.0,"Cannot create dataset"//TRIM(dname))

      CALL jpss_meta_granule_wh(x%gran0_att(channel),dataset_id,dname,err)
      THROWM(err.ne.0,"Cannot write attributes for "//TRIM(dname))

      CALL h5dclose_f( dataset_id, err)
      THROWM(err.ne.0,"Cannot close dataset "//TRIM(dname))


! Write data product attributes
      dname = "/Data_Products/VIIRS-"//TRIM(x%band(channel)%channel_name)//"-SDR"

      CALL jpss_meta_dataproducts_wh(x%datap_att(channel),file_id,dname,err)
      THROWM(err.ne.0,"Cannot write attributes for "//TRIM(dname))


      CALL h5fclose_f(file_id, err) 
      THROWM(err.ne.0,"Cannot close file "//TRIM(Filename))

      
CATCH
      end subroutine viirs_sdr_save_channel

      subroutine viirs_sdr_geo_save( filename, bandname, x, err, compress )
      ! saves Geolocation for a given band to an HDF5 file filename
      use hdf5
      USE H5LT
      USE aapp_hdf_mod
      USE viirs_sdr_geo_def
      USE viirs_sdr_geo_io
      USE viirs_sdr_geo_mem


      implicit none

      character(len=*), intent(in) :: filename
      character(len=*), intent(in) :: bandname ! VIIRS Band Name: I M or DNB
      type(viirs_sdr_geo), intent(in) :: x
      integer,intent(out)             :: err
      logical, optional, intent(in)   :: compress 

      character(len=256) :: gname
      character(len=1)   :: nband

      INTEGER(HID_T) :: file_id, group_id      
      INTEGER :: SAERR

TRY

      CALL H5FOPEN_F( TRIM(filename), H5F_ACC_RDWR_F, FILE_ID, ERR )
      THROWM(ERR.NE.0,"CANNOT OPEN "//TRIM(Filename))

      
      nband = ADJUSTL(bandname)
      Select Case ( nband )
        case ('i','I')
          gname = "/All_Data/VIIRS-IMG-GEO_All"
        case ('m','M')
          gname = "/All_Data/VIIRS-MOD-GEO_All"
        case default
          gname = "/All_Data/VIIRS-DNB-GEO_All"
      End Select


! Group All_Data/VIIRS-band-GEO_All
      CALL MKPAR( file_id, gname, group_id, err )
      THROWM(err.ne.0,"Cannot create group "//TRIM(gname))

      Call viirs_sdr_geo_wh(x,group_id,err,compress=compress)
      THROWM(ERR.ne.0,"BAD RETURN")
      
      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))
      
      CALL h5fclose_f(file_id, err) 
      THROWM(err.ne.0,"Cannot close file "//TRIM(Filename))

CATCH
      end subroutine viirs_sdr_geo_save
      
      subroutine viirs_sdr_geo_load( filename, bandname, x, err )
      ! loads Geolocation for a given band from HDF5 file filename
      use hdf5
      USE H5LT
      USE aapp_hdf_mod
      USE viirs_sdr_geo_def
      USE viirs_sdr_geo_io
      USE viirs_sdr_geo_mem


      implicit none

      character(len=*), intent(in) :: filename
      character(len=*), intent(in) :: bandname ! VIIRS Band Name: I M or DNB
      type(viirs_sdr_geo), intent(inout) :: x
      integer,intent(out)::err

      character(len=256) :: gname
      character(len=1)   :: nband

      INTEGER(HID_T) :: file_id, group_id
      logical :: lext

TRY
      CALL H5FOPEN_F( TRIM(filename), H5F_ACC_RDONLY_F, File_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open File"//TRIM(Filename))

      nband = ADJUSTL(bandname)
      Select Case ( nband )
        case ('i','I')
          gname = "All_Data/VIIRS-IMG-GEO-TC_All"
        case ('m','M')
          gname = "All_Data/VIIRS-MOD-GEO-TC_All"
        case default
          gname = "All_Data/VIIRS-DNB-GEO-TC_All"
      End Select

      CALL H5LEXISTS_F( file_id, gname, lext, err )
      if( .not. lext ) then
        Select Case ( nband )
          case ('i','I')
            gname = "All_Data/VIIRS-IMG-GEO_All"
          case ('m','M')
            gname = "All_Data/VIIRS-MOD-GEO_All"
          case default
            gname = "All_Data/VIIRS-DNB-GEO_All"
        End Select
      endif

      CALL h5gopen_f(file_id, gname, group_id, err, H5P_DEFAULT_F)
      THROWM(err.ne.0,"Cannot open group "//TRIM(gname))

      Call viirs_sdr_geo_rh(x,group_id,err)
      THROWM(err.ne.0,"Cannot Read Geolocation for "//TRIM(gname))
      
      CALL h5gclose_f( group_id, err)
      THROWM(err.ne.0,"Cannot close group "//TRIM(gname))

      CALL h5fclose_f(file_id, err) 
      THROWM(err.ne.0,"Cannot close file "//TRIM(Filename))

CATCH
      end subroutine viirs_sdr_geo_load
            
      
      subroutine viirs_sdr_info( x, nchannels, npixels, nlines, nscans, ngranules, err )
      ! returns number of pixels/lines/scans/granules
      ! returns the real number of channels loaded
      implicit none

      type(viirs_sdr), intent(in) :: x
      integer,intent(out)::err
      integer, intent(out) :: nchannels, npixels, nlines, nscans, ngranules
      integer :: channel, mchannels
TRY
        
      nchannels = 0
      ngranules = 0
      nscans    = 0
      nlines    = 0
      npixels   = 0
      
      mchannels = size(x%band)
      DO channel = 1, mchannels
        IF(x%band(channel)%channel .NE. 0 ) THEN
          nchannels = nchannels + 1
        END IF
      END DO

      DO channel = 1, mchannels
        IF(x%band(channel)%channel .NE. 0 ) THEN
          ngranules   = size(x%band(channel)%ModeGran)
          nscans      = size(x%band(channel)%ModeScan)
          IF(associated(x%band(channel)%Reflectance)) THEN
            nlines    = size(x%band(channel)%reflectance,2)
            npixels   = size(x%band(channel)%reflectance,1)
          ELSE IF(associated(x%band(channel)%Radiance)) THEN
            nlines    = size(x%band(channel)%Radiance,2)
            npixels   = size(x%band(channel)%Radiance,1)
          ELSE IF(associated(x%band(channel)%BrightnessTemperature)) THEN
            nlines    = size(x%band(channel)%BrightnessTemperature,2)
            npixels   = size(x%band(channel)%BrightnessTemperature,1)
          ELSE
            err = 1 
            THROWM(ERR.ne.0,"Cannot determine number of lines")
          END IF
          EXIT
        ENDIF
      END DO
      

CATCH
      end subroutine viirs_sdr_info
      
      
      
      subroutine viirs_sdr_data_proc( x, err, clean)
      ! processing of the data part of the structure viirs_sdr_data
      ! applies scaling factors according to channels

      implicit none

      type(viirs_sdr_data), intent(inout) :: x
      integer,intent(out)::err
      logical, optional, intent(in) :: clean  ! if TRUE remove unsed arrays 

      logical :: scale_rad
      logical :: lclean
      integer :: nlines, npixels, ngranules
      integer :: line, pixel, granule
      integer :: l1, l2, goff
      integer :: lines_per_granule

TRY
        lclean = .false.
        IF ( Present( clean )) THEN
          lclean = clean
        END IF
        
        IF(associated(x%Reflectance)) THEN
          nlines    = size(x%Reflectance,2)
          npixels   = size(x%Reflectance,1)
        ELSE IF(associated(x%Radiance)) THEN
          nlines    = size(x%Radiance,2)
          npixels   = size(x%Radiance,1)
        ELSE IF(associated(x%BrightnessTemperature)) THEN
          nlines    = size(x%BrightnessTemperature,2)
          npixels   = size(x%BrightnessTemperature,1)
        ELSE IF(associated(x%Reflectances)) THEN
          nlines    = size(x%Reflectances,2)
          npixels   = size(x%Reflectances,1)
        ELSE
          err = 1 
          THROWM(ERR.ne.0,"Cannot determine number of lines")
        END IF
        ngranules   = size(x%ModeGran)
        lines_per_granule = nlines/ngranules
#ifdef DEBUG
      write(*,*) 'number of granules', ngranules
      write(*,*) 'number of lines', nlines
      write(*,*) 'number of pixels', npixels
      write(*,*) 'lines_per_granule', lines_per_granule
      write(0,*) x%Channel_name
#endif
      IF( (x%Channel_Name(1:1) .EQ. 'I' .and. x%channel .LE.  3) .or. &
        & (x%Channel_Name(1:1) .EQ. 'M' .and. x%channel .LE. 11) ) THEN

        ! apply scaling factors to Relectances

        IF(.NOT. associated(x%Reflectances)) then
          ALLOCATE( x%reflectances(size(x%reflectance,1),size(x%reflectance,2)), STAT=ERR)
          THROWM(ERR.ne.0,"Cannot allocate Relectances (real values) ")
        
          DO granule = 1, ngranules
            l1 = (granule - 1) * lines_per_granule +1
            l2 = min( l1+lines_per_granule-1, nlines)
            goff = (granule-1)*2
            DO line = l1,l2
              DO pixel = 1,npixels
          
                IF( x%reflectance(pixel,line) .LE. 65527 ) THEN
                   x%reflectances(pixel,line) = &
                     & x%reflectance(pixel,line) * x%ReflectanceFactors(1+goff) + &
                     & x%ReflectanceFactors(2+goff) 
                ELSE
                  x%reflectances(pixel,line) = -999.9
                END IF
              END DO
            END DO
          END DO
          IF ( lclean ) THEN
            deallocate(x%reflectance,stat=err)
            THROWM(err.ne.0,"CANNOT DEALLOCATE reflectance")
            nullify(x%reflectance)
            x%ReflectanceFactors(1) = 1.
            x%ReflectanceFactors(2) = 0.
          ENDIF
        END IF
      ENDIF
      
      scale_rad = .false.
      IF( (x%Channel_Name(1:1) .EQ. 'I' ) ) scale_rad = .true.
      IF( (x%Channel_Name(1:1) .EQ. 'M' ) ) THEN
        SELECT CASE ( x%Channel )
          case (1:2,6,8:12,14:16) 
             scale_rad = .true. 
          case default
        END SELECT
      END IF
      
      If ( scale_rad ) THEN

        ! apply scaling factors to Radiances
        DO granule = 1, ngranules
          l1 = (granule - 1) * lines_per_granule +1
          l2 = min( l1+lines_per_granule-1, nlines)
          goff = (granule-1)*2
          DO line = l1,l2
            DO pixel = 1,npixels

              IF( x%Radiance(pixel,line) .LE. 65527 ) THEN
                 x%Radiance(pixel,line) = &
                   & x%Radiance(pixel,line) * x%RadianceFactors(1+goff) + &
                   & x%RadianceFactors(2+goff) 
              ELSE
                x%Radiance(pixel,line) = -999.9
              END IF
            END DO
          END DO
          ! clear scaling factors when applied
          x%RadianceFactors(1+goff) = 1.
          x%RadianceFactors(2+goff) = 0.
        END DO
      END IF
      
      SELECT CASE ( x%Channel_Name )
        case ('I4','I5','M12','M14','M15','M16') 
          ! apply scaling factors to Brightness Temperatures
        DO granule = 1, ngranules
          l1 = (granule - 1) * lines_per_granule +1
          l2 = min( l1+lines_per_granule-1, nlines)
          goff = (granule-1)*2
          DO line = l1,l2
            DO pixel = 1,npixels

              IF( x%BrightnessTemperature(pixel,line) .LE. 65527 ) THEN
                 x%BrightnessTemperature(pixel,line) = &
                   & x%BrightnessTemperature(pixel,line) * x%BrightnessTemperatureFactors(1+goff) + &
                   & x%BrightnessTemperatureFactors(2+goff) 
              ELSE
                x%BrightnessTemperature(pixel,line) = -999.9
              END IF
            END DO
          END DO
          ! clear scaling factors when applied
          x%BrightnessTemperatureFactors(1+goff) = 1.
          x%BrightnessTemperatureFactors(2+goff) = 0.
        END DO

        case default
      END SELECT


CATCH
      end subroutine viirs_sdr_data_proc
      
      subroutine viirs_sdr_geo_proc( x, att, err )
      ! processing of the geelocation structure viirs_sdr_geo
      ! calculates the TAI offset and calculates the UTC time for each scan.    
      implicit none

      type(viirs_sdr_geo), intent(inout) :: x
      type(jpss_meta_aggregate), intent(in) :: att

      integer,intent(out)::err

      integer :: nscans, ngranules
      integer :: iscan
      integer :: taioffset ! offset UTC TAI in seconds
      integer :: taioffset_ms
      integer :: hour, minute
      real*8  :: second
      real*8  :: secs
      
TRY
      ! Estimate difference TAI UTC in seconds
      ! AggregateBeginningTime is UTC
      ! Starting Time is IET (1/1/1958) microsecond
 
      read(att%AggregateBeginningTime,"(i2,i2,f9.6)") hour, minute, second
      secs = hour*3600.0 + minute*60.0 + second

      read (att%AggregateBeginningDate,"(I4,I2,I2)") &
           & x%startyear, x%startmonth, x%startday

      read (att%AggregateEndingDate,"(I4,I2,I2)") &
           & x%endyear, x%endmonth, x%endday

      read(att%AggregateEndingTime,"(i2,i2,f9.6)") hour, minute, second
#ifdef DEBUG
      write(0,*) 'Beginning secs ',secs
      write(0,*) 'att%AggregateBeginningTime ',att%AggregateBeginningTime   
      write(0,*) "x%startyear, x%startmonth, x%startday ",&
           & x%startyear, x%startmonth, x%startday
      write(0,*) 'x%StartTime(1) ',x%StartTime(1)
      write(0,*) 'att%AggregateEndingTime ',att%AggregateEndingTime 
      write(0,*) "x%endyear, x%endmonth, x%endday ",&
           & x%startyear, x%startmonth, x%startday
      write(0,*) 'taioffset ',taioffset
#endif
      IF( x%StartTime(1) .GT. 0 ) THEN
        taioffset = floor((MOD(x%StartTime(1),86400000000_8))/1000000.0 - secs + 0.5)
#ifdef DEBUG
      write(0,*) 'taioffset ',taioffset
#endif
        if (taioffset < 0) taioffset = taioffset + 86400
#ifdef DEBUG
      write(0,*) 'taioffset ',taioffset
#endif
      ELSE
        ERR = 1
        THROWM(ERR.NE.0,"Unable to determine TAI offset ")
      END IF
      taioffset_ms = taioffset * 1000


      IF(.NOT. associated(x%UTC)) then
        ALLOCATE( x%UTC(size(x%StartTime)), STAT=ERR)
        THROWM(ERR.ne.0,"Cannot allocate UTC  ")
      ENDIF

      IF(.NOT. associated(x%UTCJulianDay)) then
        ALLOCATE( x%UTCJulianDay(size(x%StartTime)), STAT=ERR)
        THROWM(ERR.ne.0,"Cannot allocate UTCJulianDay  ")
      ENDIF 

      WHERE ( x%StartTime > 0 )

       x%UTC = MOD(x%StartTime,86400000000_8)/1000_8 - taioffset_ms

! add 8 years (1950-1958) 8*365 + 2 days (bissectiles years (1952 and 1956)
        x%UTCJulianDay=(x%StartTime - taioffset_ms*1000_8)/86400000000_8 + 8*365+2
      ELSEWHERE
        x%UTC = -999
	x%UTCJulianDay= -999
      ENDWHERE
#ifdef DEBUG
      write(*,*) 'latitude', size(x%latitude,1),size(x%latitude,2)
      write(*,*) 'SatelliteAzimuthAngle', size(x%SatelliteAzimuthAngle,1),size(x%SatelliteAzimuthAngle,2)
      write(*,*) 'Height', size(x%Height,1), size(x%Height,2)
      write(*,*) 'SCPosition', size(x%SCPosition,1),size(x%SCPosition,2)
#endif
CATCH
      end subroutine viirs_sdr_geo_proc
      
      subroutine viirs_sdr_checkaggregate( x, y, err )
    ! verifies agg_att Y is the same as the one contained in X, if any
    
      implicit none

      type(viirs_sdr), intent(inout) :: x
      type(jpss_meta_aggregate), intent(inout) :: y
      INTEGER, intent(out) :: ERR         ! Error code 


TRY
      
      IF( x%agg_att%AggregateBeginningOrbitNumber .EQ. 0 ) RETURN
      
      IF ( y%AggregateBeginningDate .NE. x%agg_att%AggregateBeginningDate ) ERR=1
      THROWM(ERR.NE.0,"different AggregateBeginningDate")

      IF ( y%AggregateBeginningGranuleID .NE. x%agg_att%AggregateBeginningGranuleID ) ERR=1
      THROWM(ERR.NE.0,"different AggregateBeginningGranuleID")
        
      IF ( y%AggregateBeginningOrbitNumber .NE. x%agg_att%AggregateBeginningOrbitNumber ) ERR=1
      THROWM(ERR.NE.0,"different AggregateBeginningOrbitNumber")
        
      IF ( y%AggregateBeginningTime .NE. x%agg_att%AggregateBeginningTime ) ERR=1
      THROWM(ERR.NE.0,"different AggregateBeginningTime")
        
      IF ( y%AggregateEndingDate .NE. x%agg_att%AggregateEndingDate ) ERR=1
      THROWM(ERR.NE.0,"different AggregateEndingDate")

      IF ( y%AggregateEndingGranuleID .NE. x%agg_att%AggregateEndingGranuleID ) ERR=1
      THROWM(ERR.NE.0,"different AggregateEndingGranuleID")
        
      IF ( y%AggregateEndingOrbitNumber .NE. x%agg_att%AggregateEndingOrbitNumber ) ERR=1
      THROWM(ERR.NE.0,"different AggregateEndingOrbitNumber")
        
      IF ( y% AggregateEndingTime .NE. x%agg_att%AggregateEndingTime ) ERR=1
      THROWM(ERR.NE.0,"different AggregateEndingTime")
        
      IF ( y% AggregateNumberGranules .NE. x%agg_att%AggregateNumberGranules ) ERR=1
      THROWM(ERR.NE.0,"different AggregateNumberGranules")
            
CATCH
      end subroutine viirs_sdr_checkaggregate
      
Character*256 Function aapp_dirname( path )
  Character(len=*), Intent(in) :: path
  
  Integer :: i
  aapp_dirname = ""
  i = Len( Trim( path ) ) - 1
  Do
    If( i .le. 0 ) Return
    If( path(i:i) .eq. '/' ) Exit
    i = i - 1
  EndDo
  aapp_dirname = path(1:i)
End Function

Character*256 Function aapp_basename( path )
  Character(len=*), Intent(in) :: path
  
  Integer :: i
  aapp_basename = ""
  i = Len( Trim( path ) ) - 1
  Do
    If( i .le. 0 ) Then
      i = 0
      Exit
    EndIf
    If( path(i:i) .eq. '/' ) Exit
    i = i - 1
  EndDo
  aapp_basename = path(i+1:)
End Function

#endif

END MODULE VIIRS_SDR_MOD
