!+ I/O routine for level 1C AVHRR data
!
      SUBROUTINE IOAVH1C1D ( COMMAND, AVH1Cunit, AVH1Crecord )
      IMPLICIT NONE
      
!------------------------------------------------------------------------------
!  $Id: ioavh1c1d.F 530 2017-06-27 11:07:32Z frna $
!  (C) NWP SAF
!------------------------------------------------------------------------------      
!------------------------------------------------------------------------------
! Name:
!   ioavh1c1d.F
!
! Purpose:
!   Interface to files containing level 1C or level 1D AVHRR data
!
! Method:
!   Performs one of several functions, according to the value of COMMAND :
!
!   "OPEN/READ"          : open file for readonly
!   "OPEN/WRITE"         : open file for write
!   "CLOSE"              : close file
!   "READ HEADER"        : read header into common
!   "READ DATA"          : read data record into common
!   "WRITE HEADER"       : write header from common
!   "WRITE DATA"         : write data from common
!
!   Note: COMMAND must be typed exactly, and in upper case.
!   Routine ends by calling "ErrorReport" to check on error status.
!   If "COMMAND" is not recognised, "ErrorReport"
!   will be called with fatal error status.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
!  Version     Date      Comment                  Author
!
!   1.0      20/04/09  Created from ioi1c.F    T.Labrot (MF)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  AVHRR level 1C header/data common blocks 
#include "avh1c.h"
!  Record length dividor 
#include "reclopt.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*(*) COMMAND         ! instruction on what function to perform
      INTEGER       AVH1Cunit       ! unit number
      INTEGER       AVH1Crecord     ! record number
!
! Local scalars:
      CHARACTER*80  errMessage      ! error message
      INTEGER       errStatus       ! error status: 0 OK, +ve for fatal error
      LOGICAL       wordswap_i      ! true if word swap needed for input file
      DATA wordswap_i /.FALSE./
!
! Function & Subroutine calls:
      External ErrorReport          ! error routine
!
!- End of header


! Act on value of COMMAND
! -----------------------

! 1.  OPEN/READ
!     ---------
      IF ( COMMAND .EQ. "OPEN/READ" ) THEN

         OPEN ( UNIT=AVH1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=avh1c_record_size_byte/rldiv, 
     &   IOSTAT=errStatus )

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=AVH1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=avh1c_record_size_byte/rldiv, 
     &   IOSTAT=errStatus )
     
         IF (errStatus.EQ.0) THEN
          CALL ml_wti('INFO','OPEN/READ 1C1D AVHRR file: UNIT=:',
     &                AVH1Cunit)
         ELSE
          CALL ml_wti('ERROR','OPEN/READ 1C1D AVHRR file: UNIT=:',
     &                AVH1Cunit)
          CALL aappexit(3)
         ENDIF 

! 2.  OPEN/WRITE
!     ----------
      ELSEIF ( COMMAND .EQ. "OPEN/WRITE" ) THEN

!        First try with STATUS="NEW", i.e. file doesn't yet exist
         OPEN ( UNIT=AVH1Cunit, STATUS="NEW", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=avh1c_record_size_byte/rldiv, 
     &   IOSTAT=errStatus )

!        If that didn't work, use STATUS="OLD"
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=AVH1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=avh1c_record_size_byte/rldiv, 
     &   IOSTAT=errStatus )
! N.B. This seems to be needed on HPs, where "unknown" is treated as "scratch"

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=AVH1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=avh1c_record_size_byte/rldiv, 
     &   IOSTAT=errStatus )

	 IF ( errStatus.EQ.0 ) THEN
          CALL ml_wti('INFO','OPEN/WRITE 1C1D AVHRR file:  UNIT=:',
     &                AVH1Cunit)
         ELSE
          CALL ml_wti('ERROR','OPEN/WRITE 1C1D AVHRR file: UNIT=:',
     &                AVH1Cunit)
          CALL aappexit(3)
         ENDIF 

! 3.  READ HEADER
!     ---- ------
      ELSEIF ( COMMAND .EQ. "READ HEADER" ) THEN

!        read first header record
         READ ( UNIT=AVH1Cunit,REC=AVH1Crecord,IOSTAT=errStatus ) 
     &   avh1c_hd
     
	 IF ( errStatus.EQ.0 ) THEN
          CALL ml_wti('INFO','READing 1C1D AVHRR header REC=:',
     &                AVH1Crecord)
         ELSE
          CALL ml_wti('ERROR','READing 1C1D AVHRR header REC=:',
     &                AVH1Crecord)
          CALL aappexit(3)
         ENDIF
         
         IF (avh1c_h_instrument .EQ. 16777216) THEN    !Hex 1000000
           wordswap_i = .TRUE.
           CALL wordswap(avh1c_hd(3), avh1c_record_size_i4-2)
         ENDIF 

! 4.  WRITE HEADER
!     ----- ------
      ELSEIF ( COMMAND .EQ. "WRITE HEADER" ) THEN

         WRITE ( UNIT=AVH1Cunit,REC=AVH1Crecord,IOSTAT=errStatus ) 
     &   avh1c_hd

         IF ( errStatus.EQ.0 ) THEN
          CALL ml_wti('INFO','WRITEing 1C1D AVHRR header REC=:',
     &                 AVH1Crecord)
         ELSE
          CALL ml_wti('ERROR','WRITEing 1C1D AVHRR header REC=:',
     &                 AVH1Crecord)
          CALL aappexit(3)
         ENDIF
          


! 5.  READ DATA
!     ---- ----
      ELSEIF ( COMMAND .EQ. "READ DATA" ) THEN

         READ ( UNIT=AVH1Cunit,REC=AVH1Crecord,IOSTAT=errStatus ) 
     &   avh1c_dt

         IF (wordswap_i) THEN
           CALL wordswap (avh1c_dt(1), 273)      !for integer*4
           CALL wordswap2(avh1c_dt(274), 14336)  !for integer*2 
           CALL wordswap (avh1c_dt(7441), 10)    !for integer*4
         ENDIF

         IF ( errStatus.EQ.0 ) THEN
!          CALL ml_wti('INFO','READing 1C1D AVHRR data REC=:',
!     &                AVH1Crecord)
         ELSE
          CALL ml_wti('ERROR','READing 1C1D AVHRR data REC=:',
     &                AVH1Crecord)
          CALL aappexit(3)
         ENDIF
          

! 6.  WRITE DATA
!     ----- ----
      ELSEIF ( COMMAND .EQ. "WRITE DATA" ) THEN

         WRITE ( UNIT=AVH1Cunit,REC=AVH1Crecord,IOSTAT=errStatus ) 
     &   avh1c_dt

         IF ( errStatus.EQ.0 ) THEN
!          CALL ml_wti('INFO','WRITEing 1C1D AVHRR data REC=:',
!     &                AVH1Crecord)
         ELSE
          CALL ml_wti('ERROR','WRITEing 1C1D AVHRR data REC=:',
     &                AVH1Crecord)
          CALL aappexit(3)
         ENDIF
          


! 7.  CLOSE
!     -----
      ELSEIF ( COMMAND .EQ. "CLOSE" ) THEN

         CLOSE ( UNIT=AVH1Cunit, IOSTAT=errStatus )

         IF ( errStatus.EQ.0 ) THEN
          CALL ml_wti('INFO','CLOSEing 1C1D AVHRR file, UNIT=:',
     &                AVH1Cunit)
         ELSE
          CALL ml_wti('ERROR','CLOSEing 1C1D AVHRR file, UNIT=:',
     &                AVH1Cunit)
          CALL aappexit(3)
         ENDIF


! 8.  COMMAND not recognized
!     ------- --- ----------
      ELSE

         errStatus = 999
         
         CALL ml_wt('ERROR','Command not recognised')
         CALL aappexit(3)
         
      ENDIF   ! (COMMAND)

      RETURN
      END
