! Name: $Id: maia.F90 548 2017-08-30 09:28:19Z frna $
!
! Purpose: 
!          Subroutines maia 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    01/05/2015   Pascale Roquet bug fixes
!  1.3    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.4    05/05/2017   Pascale Roquet add T2m to maia_par (31)
SUBROUTINE maia (idbg, new_box,  landsea, elev, field_id, box, pix_id, pix, maia_par)

! Authors       : lydie lavanant

! MAIA Cloud Detection
!-----------------------------------------------------------------------
! input:
!       idbg            debug level (info, debug)
!       new_box         new information at a box resolution
!       box             lat, lon, solar and satellite angles at the center of the box
!       pix_id          latitude, longitude of the pixel
!       pix             input observations (albedo en %, brightness temperature in K) 
!                               
! output:
!       maia_par    maia output mask information
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input
  type( debug ),    INTENT(in)          :: idbg         ! 
  type( topo_field),INTENT(in)          :: landsea, elev   ! surface topography landsea and elev
  Type (field_info),INTENT(in)          :: field_id
  type( box_id ),   INTENT(inout)       :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( pix_info),  INTENT(inout)       :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),  INTENT(in)          :: pix  ! pix observations (albedo in %, Tb in K)
  LOGICAL,          INTENT(in)          :: new_box  
! output
  type(maia_param), INTENT(out)                     :: maia_par ! mask outputs (see maia_Write_Output.f90)
  
! local 
  type( clim_field), save       :: clim_alb     ! 
  type( clim_field), save       :: clim_sst     ! 
  type( clim_field), save       :: clim_sstmoy  ! 
  type( nwp_field) , save       :: bg(2)        ! forecast information
  type( maia_thres), save               :: thres        ! 
  type( maia_ThresTables_sea) , save    :: tabsea       ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_ThresTables_land), save    :: tabland      ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_ThresTables_opaq), save    :: tabopaq      ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_VISThresTables)  , save    :: thvis_sea    ! visible threshold coefficients over sea 
  type( maia_VISThresTables)  , save    :: thvis_land   ! visible threshold coefficients over land
  REAL                        , save    :: reflec(nbreflecsol,nbreflecsat,nbreflecazi,3)
 
  type( maia_CMa)               :: CMa          ! 
  type( maia_CT)                :: CT           ! 
  type( maia_CH)                :: CH           ! 
        
  LOGICAL, SAVE :: first=.true.
  logical       :: ToDo=.false.
!------------------------------------------------------------------------
! First call: start the run
! Read all necessary threshold LUTs, Atlases, climatologies and forecasts
!------------------------------------------------------------------------

 if( first ) then
   write(*,*) "first call maia"
   if (idbg%debug) write(*,*) "first call maia"
   ! read IR and VIS thresholds tables
   call maia_setup (idbg, field_id, tabsea, tabland, tabopaq, thvis_sea, thvis_land, reflec)

   ! landsea + elevation atlases are read in main program
   ! call maia_Read_Topo (idbg, field_id, landsea, elev) 
 
   ! read climatology atlases
   call maia_Read_Clim (idbg, field_id, pix_id, 'albmax', clim_alb)
   call maia_Read_Clim (idbg, field_id, pix_id, 'sstmin', clim_sst)
   call maia_Read_Clim (idbg, field_id, pix_id, 'sstmoy', clim_sstmoy)

   ! read forecast files
   call maia_Read_Prev (idbg, 'Bg_BeforObs', bg(1)) 
   call maia_Read_Prev (idbg, 'Bg_AfterObs', bg(2)) 
   first=.false.
 end if
  
!-----------------------------------------------------------------------
! new information at the box resolution
!-----------------------------------------------------------------------
 if( new_box) then

    ! get environment from climatologies and forecast
    !------------------------------------------------
    call maia_GetClim ( idbg, box, clim_alb )   
    call maia_GetClim ( idbg, box, clim_sst )   
    call maia_GetClim ( idbg, box, clim_sstmoy) 

    call maia_GetPrev ( idbg, box, pix_id, bg ) 

    ! desert update
    ! -------------
    if (box % land .and. box % albedo_clim > cst_desertalbmin ) then
       box % desert = .true.
       box%tm = 3
    endif

    ! Tsurf of the box
    ! ----------------
    if (box % ts_prev > vmanq) then
        box % tsurf = box % ts_prev 
        if(box % tsurf<0) then
           write(*,*) box % ts_prev, box % alt
           stop 2
        endif
    else
        write(*,*) 'PB prev ', box%lat, box%lon
        stop 2
    end if 

    if(idbg % debug) then
       write(*,*) 'BoxEnv Box: day, glint, twilight, night, sea, land, coast, desert snow_always', &
                                   box % day, box % glint, box % twilight , box % night, &
                                   box % sea, box % land, box % coast, box % desert, box % snow_always
       write(*,"('BoxEnv: alt, sst_clim, alb_clim, t2m_prev, ts_prev, wv:',i8, 5f10.2)") &
                box%alt, box%sst_clim, box%albedo_clim, box%t2m_prev, box%ts_prev, box%cwv
       write(*,"('        t500, 700, 850:',7f10.2)") box%t500, box%t700, box%t850
       write(*,*) '  -----------------------------------------------------------------------'
    end if
                       
    ! Thresholds reset
    !-----------------
    call maia_Thres_reset (idbg, thres)

    ! get CMa IR and VIS thresholds
    !------------------------------
    call maia_GetThres_CMa (idbg, box, pix_id, reflec, thvis_sea, thvis_land, &
                            tabsea, tabland, thres )

    ! get CT thresholds
    ! -----------------
    call maia_GetThres_CT (idbg, box, tabopaq, thres )
 endif

!-----------------------------------------------------------------------
! Information at the pixel resolution
!-----------------------------------------------------------------------
  ! set environment and reset output
  !---------------------------------
  call maia_PixEnv_reset ( idbg, landsea, elev, box, pix_id, pix, CMa, CT, CH ) 

  if( idbg % info ) then
     write(*,*) '  -----------------------------------------------------------------------'
     call maia_Pr_InfoPix (pix_id, pix)
  endif


  ! SnowIce at Surface flag
  ! --------------------
  call maia_SnowIce_surf (idbg, box, pix, pix_id% snowice_surf) 


  ! Thin Cirrus detection
  ! ---------------------
  call maia_Flag_ThinCirrus (idbg, box, pix_id, pix, CMa, CMa%thin_cirrus)
  if(idbg % debug) write(*,*) 'Thin cirrus= ', CMa%thin_cirrus


  ! Cloud Mask
  ! ----------
  call maia_Cloud_Mask (idbg, box, pix_id, pix, thres, CMa)
  if(idbg % debug) then
     write(*,"('CMa OUT:',i8)") CMa % conf_clear
     write(*,*) '  -----------------------------------------------------------------------'
  end if
    

  ! Cloud Type
  ! ----------
  if (CMa % conf_value>=0 .and. CMa % conf_value < 0.7 ) then   !cloudy or probably cloudy
      call maia_Cloud_Type (idbg, box, pix_id, pix, thres, CT)
  endif


  ! Cloud Phase
  ! ----------
!  call maia_Cloud_Phase (idbg, box, pix, thphase, CMa, CT, 2)
!  if(idbg % debug) write(*,*) 'CPhase= ', CT%Phase


  ! Cloud Pressure
  ! --------------
  call maia_Cloud_Pressure (idbg, box, pix_id, pix, tabopaq%t108_tcld, CT, CH)


  ! Sand/dust and Vocanic Ash detections 
  ! ------------------------------------
  call maia_Flag_Dust      (idbg, box, pix, pix_id, thres, CT, CMa)     ! rajouter pix_id%sd_M06
  call maia_Flag_VolcanAsh (idbg, box, pix, pix_id, thres, CMa)
              
!-----------------------------------------------------------------------
! fill maia_par output
!-----------------------------------------------------------------------
  call maia_Fill_Output (idbg, box, pix_id, pix, CMa, CT, CH, maia_par )

!-----------------------------------------------------------------------
! Information from Moderate Resolution Neighbors
!-----------------------------------------------------------------------
  ToDo=.false.
  if ( ToDo) then
    ! CMa Confidence update from Cloud adjacency
    ! ------------------------------------------
    !  call maia_CMa_Neighbors (idbg, box, texture, pix, thres, CMa)
    !  call maia_CMa_Neighbors (idbg, box, pix, thres, CMa)
    !  if(idbg % debug) write(*,*) 'CMask Moderate texture= ', CMa%test

  endif

  
END SUBROUTINE maia
