! Name: $Id: maia_Analyse_Field.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: 
!          Subroutines maia_Analyse_Field 
!          Analyse imager data (lat, lon, Obs)
!            correct field_1b if necessary (apply solar zenith cos correction)
!          Define the work boxes :
!            Boxes are regularly spaced in lat/lon 
!            dimension of a box is 0.1 degre x 0.1 degre
!            fill the field_id (field_info) structure
!            with infomartion about the boxes
!            and allocate and fill pointers 
!               field_id%pix_box and field_id%lat_box
!         
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/05/2015   Lydie Lavanant/Pascale Roquet bug fixes
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!=====================================================================================
SUBROUTINE maia_Analyse_Field (idbg, field_1b, field_id )
!=====================================================================================

 USE MOD_MAIA_CONST
 USE mod_maia_types
IMPLICIT NONE

  type( debug ), intent(in)     :: idbg         ! 
  Type (field),  intent(inout)  :: field_1b
  Type (field_info), intent(out):: field_id

  integer :: ll, pp, lig, pix, ic, n, traite(field_1b % Nb_pixels,field_1b % Nb_Lines)
  real    :: my_lon
  logical :: isOver180=.FALSE.
!----------------------------------------------------------------- 
  field_id % mission = field_1b% mission
  field_id % ImgName = field_1b% ImgName
  
  write(*,*) ' field_1b% ImgName: ', field_1b% ImgName
  
  if(field_1b% ImgName == 'viirs' ) then
     ! verify some correct navigation in the field
     if(all(field_1b % lat(:,:) <-99.) ) then
        call ml_wt('ERROR',' No correct Latitudes in granule')
        stop 2   
     endif
     if(all(field_1b % lon(:,:) <-399.) ) then
        call ml_wt('ERROR',' No correct Longitudes in granule')
        stop 2   
     endif
     ! put to bad navigation,and vmanq in the pixels with bad quality
     do lig=1, field_1b % Nb_Lines
     do pix=1, field_1b % Nb_pixels

       if( all (field_1b % Qual_Obs(:,pix,lig) /=0) ) then
          field_1b % lat(pix,lig) = vmanq
          field_1b % lon(pix,lig) = vmanq
       endif
     end do
     end do
     ! verify some correct navigation in the field
     if(all(field_1b % lat(:,:) <-99.) .or. all(field_1b % lon(:,:) <-399.)) then
        call ml_wt('ERROR',' No correct Information in granule')
        stop 2     
     endif
     ! put to bad quality pixels those with bad navigation
     do lig=1, field_1b % Nb_Lines
     do pix=1, field_1b % Nb_pixels
       if (field_1b % lat(pix,lig) <-99. .or. field_1b % lon(pix,lig) <-399. ) then
           field_1b % Qual_Obs(:,pix,lig) = 1
          field_1b %      Obs(:,pix,lig) = vmanq
       endif
     end do
     end do
     ! effective processed lines
     field_1b % NLines_filled=0
     do lig=1, field_1b % Nb_Lines
        if( any(field_1b % lat(:,lig) >=-90) .and. any(field_1b % lat(:,lig) <=90) ) then
            field_1b % NLines_filled=field_1b % NLines_filled+1
        endif
     end do
     ! reflectance channels 
     do ic=1,7
     do lig=1, field_1b % Nb_Lines
     do pix=1, field_1b % Nb_pixels
        if(field_1b % sunzen (pix,lig) > vmanq .and. field_1b % sunzen(pix,lig)<90. .and. &
           field_1b % Obs(ic,pix,lig)>0) then
           field_1b % Obs(ic,pix,lig) = field_1b % Obs(ic,pix,lig) * cos(cdr* field_1b % sunzen(pix,lig) )
        endif
     end do
     end do
     end do
  endif

  ! definition des boites de travail
  call maia_ValMin2 (idbg, field_1b % lat, field_1b % Nb_pixels, field_1b % Nb_Lines, -99.5, field_id % LatM_min)
  field_id % LatM_max = MAXVAL (field_1b % lat)
  call maia_ValMin2 (idbg, field_1b % lon, field_1b % Nb_pixels, field_1b % Nb_Lines, -399.5, field_id % LonM_min)
  field_id % LonM_max = MAXVAL (field_1b % lon)
 
  if ( field_id % LonM_max > 178. .and. field_id % LonM_min < -178 ) then
    isOver180=.TRUE.
    field_id % isOver180=.TRUE.
  
    call maia_ValMinMaxLon(idbg, field_1b % lon, field_1b % Nb_pixels, field_1b % Nb_Lines, -399.5, &
                                 field_id % LonM_min,field_id % LonM_max)

    if(idbg%debug) then
       write(*,*) ">>> this granule is over longitude 180 "
       write(*,*) '>>>  LatMin LatMax LonMin LonMax :', &
                 field_id % LatM_min, field_id % LatM_max, field_id % LonM_min, field_id % LonM_max
    endif
 
  endif
  
  field_id % nbLat_box= 1+ INT ((field_id % LatM_max - field_id % LatM_min)*10)
  field_id % nbLon_box= 1+ INT ((field_id % LonM_max - field_id % LonM_min)*10)
  
  ! Longitude normalisation between -180 and +180 degrees
  call maia_Lon_Norm (.false., 'viirs ', field_id % LonM_min)
  call maia_Lon_Norm (.false., 'viirs ', field_id % LonM_max)
  
  if(idbg%debug) write(*,*) '  LonMin LonMax after normalisation ', field_id % LonM_min, field_id % LonM_max    
  write(*,*) '  nbLat_box, nbLon_box: ',field_id % nbLat_box, field_id % nbLon_box

  allocate ( field_id % nb_fov(field_id % nbLon_box+1,field_id % nbLat_box+1) )
  allocate ( field_id % pix_box(1000,field_id % nbLon_box+1,field_id % nbLat_box+1) )
  allocate ( field_id % lig_box(1000,field_id % nbLon_box+1,field_id % nbLat_box+1) )

  field_id % nb_fov(:,:)=0
  do lig=1, field_1b % Nb_Lines
  do pix=1, field_1b % Nb_pixels
     if(field_1b % lat(pix,lig) > -90. .and. field_1b % lon(pix,lig) > -180.) then
        ll=1+INT( (field_1b % lat(pix,lig) - field_id % LatM_min)*10 )
        if (field_1b % lon(pix,lig) < 0 .and. isOver180) then
            my_lon=field_1b % lon(pix,lig)+360.
        else
            my_lon=field_1b % lon(pix,lig)
        endif
        pp=1+INT(   ( my_lon - field_id % LonM_min)*10 )
        field_id % nb_fov(pp,ll) = field_id % nb_fov(pp,ll) + 1
        field_id % pix_box(field_id % nb_fov(pp,ll),pp,ll)=pix
        field_id % lig_box(field_id % nb_fov(pp,ll),pp,ll)=lig
     endif
  end do
  end do

  traite(:,:)=0
  do ll=1, field_id % nbLat_box
  do pp=1, field_id % nbLon_box

     if(field_id % nb_fov(pp,ll)>0) then
        do n=1, field_id % nb_fov(pp,ll)
           pix=field_id % pix_box(n, pp, ll)
           lig=field_id % lig_box(n, pp, ll)
           if(pix>0 .and. lig>0) traite(pix,lig)= traite(pix,lig)+1
        end do
     endif
  end do
  end do

 if(field_1b% ImgName == 'avhrr' ) then
   if(idbg%debug) then
       write(*,*) "verif missing fields channels 16 and 37"
       write(*,*) MAXVAL(field_1b % Obs(3,:,:)), MAXVAL(field_1b % Obs(4,:,:))
   endif
   if( MAXVAL(field_1b % Obs(3,:,:)) == vmanq .and.  MAXVAL(field_1b % Obs(4,:,:)) == vmanq ) then
      call ml_wt('ERROR',' Missing channels 16 and 37 data')
      stop 2
   endif
 endif
 
end subroutine maia_Analyse_Field
