! Name: $Id: maia_CMa_LN.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_LN 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant enhancement for quality flags
SUBROUTINE maia_CMa_LN  (idbg, pix_id, pix, box, thres, CMa)

! Authors       : lydie lavanant

! land - night
!max_num_tests = 7 (VIIRS), 5 (AVHRR)
! Gr 1. Emission Threshold          :   . BT108
! Gr 2. Emission Difference Tests   :   . BT108 – BT37 for TOC NDVI > thres
!                                       . BT87 - BT108          !for coherence seviri
!                                       . BT37 – BT108
!                                       . BT87 - BT37           !over desert coherence seviri
!                                       . BT108 - BT87          !large satsen coherence seviri
! Gr 5. Emission Thin Cirrus Test   :   . BT108 – BT120 
! Gr 6. texture                     :   . I4 & I43             !for coherence maiav3

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),      INTENT(in) :: idbg        ! 
  type( pix_info),  INTENT(in) :: pix_id        ! 
  type( pix_data),  INTENT(in) :: pix           ! 
  type( box_id ),     INTENT(in) :: box         ! 
  type( maia_thres), INTENT(in) :: thres        ! 
  type( maia_CMa ),  INTENT(out) :: CMa
  
! local
  REAL    :: conf_texture
  REAL    :: seuil108, s_cl_108, s_cld_108
  REAL    :: seuil10837, s_cl_10837, s_cld_10837
  INTEGER :: num_tests, max_num_tests, ngroup(ngroups_max), max_num_groups(ngroups_max)
  REAL    :: cc_group(ngroups_max,ngroups_max)
  REAL    :: seuil, s_cl, s_cld  
  real    :: btd, ts  
  logical :: ToDo
  
!======================================================================
   if(pix_id% ImgName == 'viirs') then
      max_num_tests = max_num_tests_VIIRS(5)
      max_num_groups(:) = max_num_group_VIIRS_LN(:)
   else
      max_num_tests = max_num_tests_AVHRR(5)
      max_num_groups(:) = max_num_group_AVHRR_LN(:)
   endif

   num_tests = 0
   ngroup(:)=0
   cc_group(:,:) = 0

   if(idbg%debug) write(*,*) 'CMa_LN ',pix % Qual_37, pix % Qual_87, pix % Qual_108, pix % Qual_120
   if(idbg%debug) write(*,*) 'CMa_LN ',box % tsurf

   !    ---------------------------
   !    1.      108 Threshold Test
   !    ---------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. &
       box % tsurf > sfc_temp_Min .AND. box % tsurf < sfc_temp_Max) THEN

      ! thresholds: from forecast or warmest obs
      if (pix_id % coast .and. box % sst_clim>vmanq .and. box% tsurf> vmanq) then
          ts = min(box % sst_clim, box % tsurf)
      elseif (box % ts_prev > vmanq) then
          ts=box % ts_prev - (0.65* pix_id % alt)/100.
      else
          ts=MAX( CMa_lstmin , pix % Obs_108 )
      end if 

      btd = ts - pix % Obs_108

      ! thresholds:
      IF (pix_id % coast) THEN
          seuil = MAX(thres % v_CMa_108_sea, thres % v_CMa_108_land)
      ELSE IF (pix_id % desert) THEN
          seuil = CMa_lst_desert
          if(pix_id% ImgName == 'avhrr') seuil = thres % v_CMa_108_land - 3.    
      ELSE
          seuil = thres % v_CMa_108_land
      ENDIF

      s_cl  = seuil - 2.0       ! confident clear
      s_cld = seuil + 2.0       ! confident cloudy

      ! Cloudflag
      IF (btd >= seuil)  CMa % Cld_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108)

      if(pix_id % snow_always) then
         if(CMa %  confcl_108>0.5) CMa %  confcl_108=0.6
      endif

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(1)=ngroup(1)+1
      cc_group(1,ngroup(1)) = CMa % confcl_108

      if(idbg%debug) then
         write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108' , CMa % confcl_108, btd, '>', seuil
      endif
   END IF

   !    ----------------------------------
   !    5.1     108 – 120 Thin Cirrus Test
   !    ----------------------------------
   ToDo=.true.!
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_120) THEN

      btd= pix % Obs_108 - pix % Obs_120

      ! threshold. For small temperature differences, set the threshold to a default value 
!      IF (pix_id % coast) THEN
      IF (pix_id % coast) THEN
          seuil = MAX(thres % v_CMa_108_120_sea, thres % v_CMa_108_120_land) 
      ELSE
          seuil = thres % v_CMa_108_120_land 
      ENDIF
      if(seuil < 0.1) seuil = CMa_LN_108_120

      s_cl  = seuil - 0.25
      s_cld = seuil + 0.5

      ! Cloud flag
      IF (btd > seuil) CMa % Cld_108_120 = .true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_120)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(5)=ngroup(5)+1
      cc_group(5, ngroup(5)) = CMa % confcl_108_120

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108_120', CMa % confcl_108_120, btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.5 37 – 108 Difference Test 
   !    ------------------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_37 .and. pix % Obs_37 > 210) THEN     !limit_37

      btd = pix % Obs_37 - pix % Obs_108

      ! thresholds
      IF (pix_id % coast) THEN
          seuil     =  MAX(thres % v_CMa_37_108_sea, thres % v_CMa_37_108_land) 
      ELSE
          seuil     =  thres % v_CMa_37_108_land
      ENDIF

      s_cl  = seuil-0.5         ! confident clear
      s_cld = seuil+0.5         ! confident cloudy

      ! Set the 37-108 Cloud flag
      IF (btd > seuil) CMa % Cld_37_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd,s_cld, seuil, s_cl , CMa % confcl_37_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_37_108

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '37_108', CMa % confcl_37_108, btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.2     108 – 37 Difference Test 
   !    ------------------------------------------
!   IF (pix % Qual_108 .and. pix % Qual_37 .and. &
!       pix % Obs_37 > limit_37 .and. pix_id % toc_ndvi>NIGHT_TOC_NDVI) THEN
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_37 .and. pix % Obs_37 > 210 ) THEN

      btd = pix % Obs_108 - pix % Obs_37

      !  thresholds
      IF (pix_id % coast) THEN
          seuil     =  MAX(thres % v_CMa_108_37_sea, thres % v_CMa_108_37_land)
      ELSE
          seuil     =  thres % v_CMa_108_37_land
      ENDIF

!      s_cl  = CMa_LN_108_37_Ccl  - (box % cwv * 0.6)   ! confident clear
!      s_cld = CMa_LN_108_37_Ccld - (box % cwv * 0.4)   ! confident cloudy
      s_cl  =  seuil - 0.5      ! confident clear
      s_cld = seuil + 0.5       ! confident cloudy

      ! Cloud flag
      IF (btd > seuil) CMa % Cld_108_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_108_37

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108_37', CMa % confcl_108_37,btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.3 87 – 108 Difference Test 
   !    ------------------------------------------
   ToDo=.true.!
   IF (ToDo .and. pix % Qual_87 .and. pix % Qual_108) THEN

      btd = pix % Obs_87 - pix % Obs_108

      !  thresholds:
      IF (pix_id % coast) THEN
          seuil = MAX(thres % v_CMa_87_108_sea, thres % v_CMa_87_108_land) 
      ELSE
          seuil = thres % v_CMa_87_108_land
      ENDIF
      s_cl  = seuil - 0.5       ! confident clear
      s_cld = seuil + 0.5       ! confident cloudy

      ! Set the 87_108_120 Cloud flag
      IF (btd > seuil) CMa % Cld_87_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_87_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_87_108

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '87_108', CMa % confcl_87_108, btd,'>',seuil
   ENDIF

   !    ------------------------------------------
   !    2.6     87 – 37 Difference Test over desert
   !    ------------------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_87 .and. pix % Qual_37 .and. pix_id % desert) THEN

      btd = pix % Obs_87 - pix % Obs_37

      !  thresholds
      seuil = thres % v_CMa_87_37_land
      s_cl  = seuil - 0.5       ! confident clear
      s_cld = seuil + 0.5       ! confident cloudy

      ! Set theCloud flag
      IF (btd > seuil) CMa % Cld_87_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_87_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_87_37

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '87_37 desert', CMa % confcl_87_37, btd,'>',seuil
   ENDIF

   !    -----------------------------------------------------
   !    Spatial Textures
   !    -----------------------------------------------------
   ToDo=.true.
   IF ( pix_id%coast .or. pix_id%alt>500 ) ToDo=.false.
   ! thresholds
   if(ToDo) then
      if(pix_id % texture_fromImager)then
         seuil108          = cstI_sd108l
         seuil10837        = cstI_sd10837l
      elseif (pix_id % texture_fromNeighbors) then
         seuil108          = cstA_sd108l
         seuil10837        = cstA_sd10837l
      endif   
   endif
   
   if(ToDo .and. pix_id% sd_108>vmanq .and. pix_id% sd_10837>vmanq .and. .not. pix_id % desert) then
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.25        ! confident cloudy

      s_cl_10837  = seuil10837 -0.25    ! confident clear
      s_cld_10837 = seuil10837 +0.25    ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 .and.pix_id% sd_10837 > seuil10837) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture
      CALL maia_ConfTest (idbg, pix_id% sd_10837, s_cld_10837, seuil10837, s_cl_10837, conf_texture)
      CMa % confcl_texture1= (CMa % confcl_texture1 + conf_texture )/2.
      if(CMa % confcl_texture1<0.25) CMa % confcl_texture1=0.25

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2,a5,f10.2,a3,f10.2)') 'texture1', CMa % confcl_texture1, &
                                               pix_id% sd_108,'>', seuil108, '  &  ',pix_id% sd_10837,'>', seuil

   elseif(ToDo .and. pix_id% sd_108>vmanq ) then        ! AVHRR channel 3.7mm not available in daytime
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.25        ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 ) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') 'texture1', CMa % confcl_texture1, pix_id% sd_108,'>', seuil108
   endif
                      
   
   !    -------------------------------
   !    Final mask confidence 
   !    -------------------------------
   call maia_CMa_ConfClear (idbg, ngroup, cc_group,  CMa)

   !    -------------------------------
   !    Quality flag
   !    -------------------------------
   if(idbg%debug) write(*,'(a15,2i10)') 'CMa_quality:', max_num_tests, num_tests
   call maia_CMa_quality (idbg, max_num_tests, num_tests, max_num_groups, ngroup, CMa % qual)
!   call maia_CMa_quality (idbg, max_num_tests, num_tests, CMa % qual)
   if(CMa % qual==0) CMa % conf_clear=vmanq

END SUBROUTINE maia_CMa_LN
