! Name: $Id: maia_CMa_LT.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_LT 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant enhancement for quality flags
SUBROUTINE maia_CMa_LT  (idbg, pix_id, pix, box, thres, CMa)

! Authors       : lydie lavanant

! land - day
!max_num_tests = 8 (VIIRS), 6 (AVHRR)
! Gr 1. Emission Threshold          :   . BT108                                  ! coherence maiav3
! Gr 2. Emission Difference Tests   :   . BT37 - BT40 lat 60S-60N & TOC NDVI >.2 (Todo=false)
!                                       . BT108- BT37 for TOC NDVI > 0.2
!                                       . BT87 - BT108                           !coherence seviri
!                                       . BT87 - BT37                            !over desert coherence seviri
!                                       . BT108- BT87                            !large satsen coherence seviri
! Gr 3. Reflectance Threshold Tests :   . Ref06  
!                                       . Ref08/Ref06 RatioTest                  (Todo=false)
! Gr 4. Reflectance Thin Cirrus     :   . Ref13                                  (Todo=false)
! Gr 5. Emission Thin Cirrus Test   :   . BT108- BT120 
! Gr 6. Texture			    :   . sd_108 & sd_10837

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),      INTENT(in)  :: idbg       ! 
  type( pix_info),  INTENT(in)  :: pix_id   ! 
  type( pix_data),  INTENT(in)  :: pix          ! 
  type( box_id ),     INTENT(in)  :: box        ! 
  type( maia_thres), INTENT(in)  :: thres       ! 
  type( maia_CMa ),  INTENT(inout) :: CMa
  
! local
  INTEGER :: num_tests, max_num_tests, ngroup(ngroups_max), max_num_groups(ngroups_max)
  REAL    :: cc_group(ngroups_max,ngroups_max)
  REAL    :: seuil, s_cl, s_cld  
  REAL    :: conf_texture
  REAL    :: seuil108, s_cl_108, s_cld_108
  REAL    :: seuil10837, s_cl_10837, s_cld_10837

  real    :: tsurf, btd
  real    :: obs06, obs08, ratio_obs06_obs08, ref_0604
  real    :: ratio_2, ratio_1, ratio_3, s_87_108
  logical :: ToDo, qual_0604
!======================================================================
   if(pix_id% ImgName == 'viirs') then
      max_num_tests = max_num_tests_VIIRS(6)
      max_num_groups(:) = max_num_group_VIIRS_LT(:)
   else
      max_num_tests = max_num_tests_AVHRR(6)
      max_num_groups(:) = max_num_group_AVHRR_LT(:)
      if(pix % Qual_37 .and. .not. pix % Qual_06) then
         max_num_tests = max_num_tests -1 ! tests with 0.6 not done
      endif
      if(pix % Qual_06 .and. .not. pix % Qual_37) then
         max_num_tests = max_num_tests -2 ! tests with 3.7 not done
      endif
   endif
   
   num_tests = 0
   ngroup(:)=0
   cc_group(:,:) = 9999

   !    ---------------------------------------------------------------------------------
   !    TESTs
   if(idbg%debug) then
      write(*,'(a15)') CMa % test_name
      write(*,'(a57)') 'Confident_clear (1:clear/0:cloudy) '
   endif
   !    ---------------------------------------------------------------------------------

   !    --------------------------
   !    1.      Tsurf Test
   !    --------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 ) THEN

      ! thresholds: from forecast or warmest obs
      if (box % ts_prev > vmanq) then
          tsurf=box % ts_prev - (0.65* pix_id % alt)/100.
      else
          tsurf=MAX( CMa_lstmin , pix % Obs_108 )
      end if 

      IF (pix_id % coast .and. pix_id % twilight) THEN
          seuil = MAX(thres % v_CMa_108_sea, thres % v_CMa_108_land)
      ELSE
          seuil = thres % v_CMa_108_land
      ENDIF

      btd = tsurf - pix % Obs_108

      s_cl  = seuil - 2.0
      s_cld = seuil + 2.0

      ! Cloud flag
      IF (btd >= seuil .or. pix % Obs_108 < cst_108ClearMin) CMa % Cld_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108)
      IF (pix % Obs_108 <= cst_108ClearMin) CMa %  confcl_108=0.
      
      if(pix_id % snow_always) then
         if(CMa %  confcl_108>0.5) CMa %  confcl_108=0.6
      endif

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(1)=1
      cc_group(1,1) = CMa % confcl_108

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108' , CMa % confcl_108, btd, '>', seuil
   END IF

   !    ----------------------------------
   !    5.      108 – 120 Thin Cirrus Test
   !    ----------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_120 ) THEN

      btd= pix % Obs_108 - pix % Obs_120

      ! threshold. For small temperature differences, set the threshold to a default value 
      IF (pix_id % coast .and. pix_id % twilight) THEN
          seuil = MAX(thres % v_CMa_108_120_sea, thres % v_CMa_108_120_land) 
      ELSE
          seuil = thres % v_CMa_108_120_land 
      ENDIF
      if(seuil < 0.1) seuil = CMa_LD_108_120
      s_cl  = seuil - 0.25
      s_cld = seuil + 0.5

      ! Cloud flag
      IF (btd > seuil) CMa %Cld_108_120=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_120)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(5)=ngroup(5)+1
      cc_group(5,ngroup(5)) = CMa % confcl_108_120

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108_120', CMa % confcl_108_120, btd,'>',seuil
   END IF

   !    ------------------------------------------------------------------------------------
   !    2.1     BT37 – BT40 Difference Test for lat 60S-60N and "Top Of Canopy" NDVI>thres
   !    ------------------------------------------------------------------------------------
   ToDo=.false.!ToDo=.true.
   if( pix_id % coast .or. pix_id % Glint) ToDo=.false.
   IF (ToDo .and. pix % Qual_37  .and. pix % Qual_40 .and. &
       pix % Obs_37>CMa_37_tempmin .and. pix % Obs_37<CMa_37_saturation .and. &
       pix_id % Lat < highLat .and. pix_id % Lat > lowLat .and. .not. pix_id%desert) THEN

      btd = pix % Obs_37 - pix % Obs_40

      ! thresholds
      seuil = box%cosol * CMa_LD_37_40  !
      s_cl  = box%cosol * CMa_LD_37_40_Ccl      ! confident clear
      s_cld = box%cosol * CMa_LD_37_40_Ccld     ! confident cloudy

      ! Cloud flag
      IF (btd >= seuil) CMa %Cld_37_40=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_37_40)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_37_40

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '37_40' , CMa % confcl_37_40, btd,'>',seuil
   END IF

   !    ----------------------------------------------------------------
   !    2.2     108 – 37 Difference Test 
   !    ----------------------------------------------------------------
   ToDo=.true.
   if( pix % Qual_108 .and. pix % Qual_87) then
       s_87_108 = -1.5 * (pix_id%satsec -1) -4.5
       if( (pix % Obs_87 - pix % Obs_108) > s_87_108) then
          ToDo=.true.
       else
          ToDo=.false.
       endif
   endif
   if(pix_id % coast .or. pix_id % glint) ToDo=.false.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_37 .and. &
       pix % Obs_37>CMa_37_tempmin .and. pix % Obs_37<CMa_37_saturation) THEN

      btd = pix % Obs_108 - pix % Obs_37

      !  thresholds
      IF (pix_id % coast .and. pix_id % twilight) THEN
          seuil     =  MAX(thres % v_CMa_108_37_sea, thres % v_CMa_108_37_land)
      ELSE
          seuil     =  thres % v_CMa_108_37_land
      ENDIF
      s_cl  = seuil - 2.        ! confident clear
      s_cld = seuil + 2.        ! confident cloudy

      ! Cloud flag
      IF (btd >seuil) CMa %Cld_108_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_108_37

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108_37' , CMa % confcl_108_37, btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.3     87 – 108 Difference Test 
   !    ------------------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_87 .and. pix % Qual_108 .and. thres % v_CMa_87_108_land>vmanq) THEN

      btd = pix % Obs_87 - pix % Obs_108

      !  thresholds: 
      seuil = thres % v_CMa_87_108_land
      s_cld = seuil + 0.5
      s_cl  = seuil - 0.5

      ! Set the 87_108_120 Cloud flag
      IF (btd >= seuil) CMa % Cld_87_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_87_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_87_108

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '87_108' , CMa % confcl_87_108, btd,'>',seuil
   ENDIF

   !    ------------------------------------------
   !    2.5 37 – 108 Difference Test
   !    ------------------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_37 .and. &
       pix % Obs_37>CMa_37_tempmin .and. pix % Obs_37<CMa_37_saturation) THEN

      btd = pix % Obs_37 - pix % Obs_108

      ! thresholds
      IF (pix_id % coast) THEN
          seuil     =  MAX(thres % v_CMa_37_108_sea, thres % v_CMa_37_108_land) 
      ELSE
          seuil     =  thres % v_CMa_37_108_land
      ENDIF

      s_cl  = seuil-2.          ! confident clear
      s_cld = seuil+2.          ! confident cloudy

      ! Set the 37-108 Cloud flag
      IF (btd > seuil) CMa % Cld_37_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd,s_cld, seuil, s_cl , CMa % confcl_37_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(ngroup(2),2) = CMa % confcl_37_108

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '37_108', CMa % confcl_37_108, btd,'>',seuil
   END IF

   !    -------------------------------------------------------------------
   !    3.4     06 or 04 Reflectance Test, depending on TOC NDVI conditions
   !    -------------------------------------------------------------------
   ToDo=.true.
   IF (pix_id % desert) THEN       
        ref_0604 = pix % Obs_04
        qual_0604= pix % Qual_04
        seuil    = thres % v_CMa_04_land        
   ELSE
        ref_0604 = pix % Obs_06
        qual_0604= pix % Qual_06
        seuil    = thres % v_CMa_06_land        
   END IF

   IF (ToDo .and. ref_0604>vmanq .and. qual_0604 .and. seuil>vmanq) then

      ! thresholds  
      s_cl  = seuil -1.5                ! confident clear
      s_cld = seuil +1.5                ! confident cloudy

      ! Cloud flag
      IF (ref_0604 >= seuil) CMa %Cld_06=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, ref_0604, s_cld, seuil, s_cl, CMa % confcl_06)
      
      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3,ngroup(3)) = CMa % confcl_06

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '06' , CMa % confcl_06, ref_0604,'>',seuil 
   ENDIF

   !    ------------------------------------
   !    3.2     08/06 Reflectance Ratio Test 
   !    ------------------------------------
   ToDo=.true.
   if(pix_id % coast .or. pix_id % twilight) ToDo=.false.
   IF (ToDo .and. pix % Qual_06 .and. pix % Qual_08 .and. pix % Obs_06>=CMa_LD_06_GEMI ) THEN

      ! Calculate the 08/06 ratio
      obs06 = pix % Obs_06
      obs08 = pix % Obs_08
      ratio_1 = 2.0 * (obs08-obs06) + (1.5*obs08) + (0.5*obs06)
      ratio_2 = obs08 + obs06 + 0.5
      ratio_3 = ratio_1 / ratio_2
      ratio_obs06_obs08 = ratio_3 * (1.0 - 0.25*ratio_3) - ((obs06 - 0.125) / (1.0 - obs06))

      !  thresholds
      seuil = CMa_LD_06_08
      s_cl  = CMa_LD_06_08_Ccl  ! confident clear
      s_cld = CMa_LD_06_08_Ccld ! confident cloudy

      ! Set the 06_08 Cloud flag
      IF ( ratio_obs06_obs08 <= seuil ) CMa % Cld_06_08=.true. 

     !  Confidence Clear level
     CALL maia_ConfTest (idbg, ratio_obs06_obs08, s_cld, seuil, s_cl, CMa % confcl_06_08)

     !  number of tests done
     num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3,ngroup(3)) = CMa % confcl_06_08

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '06/08' , CMa % confcl_06_08, ratio_obs06_obs08,'<',seuil
   ENDIF

   !    ------------------------------------------
   !    2.6     87 – 37 Difference Test over desert
   !    ------------------------------------------
   ToDo=.true.!
   IF (ToDo .and. pix % Qual_87 .and. pix % Qual_37 .and. pix_id % desert) THEN
      max_num_tests = max_num_tests+1

      btd = pix % Obs_87 - pix % Obs_37

      !  thresholds
      seuil = thres % v_CMa_87_37_land
      s_cl  = seuil - 0.5       ! confident clear
      s_cld = seuil + 0.5       ! confident cloudy

      ! Set theCloud flag
      IF (btd > seuil) CMa % Cld_87_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_87_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_87_37

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '87_37 desert', CMa % confcl_87_37, btd,'>',seuil
   ENDIF

   !    --------------------------------------------------------
   !    4.      13 high troposphere thin Cirrus Reflectance Test 
   !    --------------------------------------------------------
   ToDo=.false. ! delicat -> à travailler
   if(pix_id % coast .or. pix_id % twilight) ToDo=.false.
   IF (ToDo .and. pix % Qual_13 .and. box % cwv > CMa_DD_13_cwv_cutoff) THEN

      !  thresholds
      seuil = box%cosol * CMa_LD_13
      s_cl  = box%cosol * CMa_LD_13_Ccl ! confident clear
      s_cld = box%cosol * CMa_LD_13_Ccld        ! confident cloudy

      ! Cloud flag
      IF (pix % Obs_13 > seuil ) CMa %Cld_13=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix % Obs_13, s_cld, seuil, s_cl, CMa % confcl_13)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(4)=ngroup(4)+1
      cc_group(4,ngroup(4)) = CMa % confcl_13

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '13' , CMa % confcl_13, pix % Obs_13,'>',seuil
   ENDIF


   !    -----------------------------------------------------
   !    Spatial Textures
   !    -----------------------------------------------------
!   ToDo=.false.
!   IF (.not. pix_id%coast .and. pix_id%alt<500 .and. .not. pix_id % desert) ToDo=.true.
   ToDo=.true.
   IF ( pix_id%coast .or. pix_id%alt>500 ) ToDo=.false.

   ! thresholds
   if(ToDo) then
      if(pix_id % texture_fromImager)then
         seuil108          = cstI_sd108l   
         seuil10837        = cstI_sd10837l 
         if(pix_id%sunzen<90) then
            seuil108          = cstI_sd108l   + 1.67* (90.-pix_id%sunzen)
            seuil10837        = cstI_sd10837l + 1.67* (90.-pix_id%sunzen)
         endif
      elseif (pix_id % texture_fromNeighbors) then
         seuil108          = cstA_sd108l
         seuil10837        = cstA_sd10837l
         if(pix_id%sunzen<90) then
            seuil108          = cstA_sd108l   + 1.67* (90.-pix_id%sunzen)
            seuil10837        = cstA_sd10837l + 1.67* (90.-pix_id%sunzen)
         endif
      endif   
      if(seuil108   >  2.) seuil108=  2.
      if(seuil10837 >  2.) seuil10837= 2.
   endif
   
   if(ToDo .and. pix_id% sd_108>vmanq .and. pix_id% sd_10837>vmanq .and. .not. pix_id % desert) then
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.25        ! confident cloudy

      s_cl_10837  = seuil10837 -0.25    ! confident clear
      s_cld_10837 = seuil10837 +0.25    ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 .and.pix_id% sd_10837 > seuil10837) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture
      CALL maia_ConfTest (idbg, pix_id% sd_10837, s_cld_10837, seuil10837, s_cl_10837, conf_texture)
      CMa % confcl_texture1= (CMa % confcl_texture1 + conf_texture )/2.

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2,a5,f10.2,a3,f10.2)') 'texture1', CMa % confcl_texture1, &
                                               pix_id% sd_108,'>', seuil108, '  &  ',pix_id% sd_10837,'>', seuil
                                               
   elseif(ToDo .and. pix_id% sd_108>vmanq ) then        ! AVHRR channel 3.7mm not available in daytime
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.25        ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 ) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') 'texture1', CMa % confcl_texture1, &
                                                           pix_id% sd_108,'>', seuil108
   endif
    
    
   !    -------------------------------
   !    Final mask confidence 
   !    -------------------------------
   call maia_CMa_ConfClear (idbg, ngroup, cc_group,  CMa)

   if(idbg%debug) then
      write(*,'(a45, i5)') 'Conf_cld=0,prob_cld=1,prob_cl=2,conf_cl=3', CMa % conf_clear
   endif


   !    -------------------------------
   !    Quality flag
   !    -------------------------------
   call maia_CMa_quality (idbg, max_num_tests, num_tests, max_num_groups, ngroup, CMa % qual)
!   call maia_CMa_quality (idbg, max_num_tests, num_tests, CMa % qual)
   if(CMa % qual==0) CMa % conf_clear = vmanq
 
   if(idbg%debug) then
      write(*,'(a45, i5)') 'Conf_cl=0,prob_cl=1,prob_cld=2,conf_cld=3', CMa % conf_clear
      write(*,'(a45, i5)') 'Qual:high=0,medium=1,poor=2,bad=3', CMa % qual
   endif


END SUBROUTINE maia_CMa_LT
