! Name: $Id: maia_CMa_SD.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_SD 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant enhancement for quality flags
SUBROUTINE maia_CMa_SD  (idbg, pix_id, pix, box, thres, CMa)

! Authors       : lydie lavanant

! sea - day
! max_num_tests = 11 (VIIRS), 9 (AVHRR)
! Gr 1. Emission Threshold          :   . BT108                             !for coherence with maiav3
! Gr 2. Emission Difference Tests   :   . BT37 – BT40 for lat 60S-60N       (ToDo=false)
!                                       . BT108 – BT37 
!                                       . BT87 - BT108
! Gr 3. Reflectance Threshold Tests :   . Ref08  
!                                       . Ref06
!                                       . Ref08/Ref06
! Gr 4. Reflectance Thin Cirrus     :   . Ref16                             !for coherence with maiav3
!                                   :   . Ref13
! Gr 5. Emission Thin Cirrus Test   :   . BT108 – BT120 
! Gr 6. texture  if not coast       :   . I2                            
!                                       . I5

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),     INTENT(in) :: idbg         ! 
  type( pix_info), INTENT(in)   :: pix_id       ! 
  type( pix_data), INTENT(in)   :: pix          ! 
  type( box_id ),    INTENT(in) :: box          ! 
  type( maia_thres),INTENT(in)  :: thres        ! 
  type( maia_CMa),  INTENT(out) :: CMa
  
! local
  INTEGER :: num_tests, max_num_tests, max_num_groups(ngroups_max), ngroup(ngroups_max)
  REAL    :: cc_group(ngroups_max,ngroups_max)
  REAL    :: seuil08
  REAL    :: seuil108, s_cl_108, s_cld_108
  REAL    :: seuil10837, s_cl_10837, s_cld_10837
  REAL    :: seuil, s_cl, s_cld  
  REAL    :: conf_texture
  real    :: btd, btd108_120, ratio, sstloc
  logical :: ToDo

!======================================================================
   if(pix_id% ImgName == 'viirs') then
      max_num_tests = max_num_tests_VIIRS(3)
      max_num_groups(:) = max_num_group_VIIRS_SD(:)
   else
      max_num_tests = max_num_tests_AVHRR(3)
      max_num_groups(:) = max_num_group_AVHRR_SD(:)
   endif

   num_tests = 0
   ngroup(:)=0
   cc_group(:,:) = 9999

   !    ---------------------------------------------------------------------------------
   !    TESTs
   if(idbg%debug) then
      write(*,'(a10)') CMa % test_name
      write(*,'(a57)') 'Confident_clear (1:clear/0:cloudy), Seuil, btd '
   endif
   !    ---------------------------------------------------------------------------------

   !    1. SST Test
   !    --------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 ) THEN

      ! thresholds: compute local sst or use forecast
      if (box % sst_clim > CMa_sstfrozen .and. pix % Qual_120 .and. .not. pix_id % lake) then
          call maia_SST (idbg, pix_id, pix, box % sst_clim, sstloc)
          btd = box % sst_clim - sstloc
          seuil   = cst_sst     !seuil   = thres % v_CMa_sst !? 
      else                      ! -> use forecast
          btd = box % tsurf - pix % Obs_108
          seuil   = thres % v_CMa_108_sea
      end if

      s_cl  = seuil - 2.0       ! confident clear
      s_cld = seuil + 2.0       ! confident cloudy

      ! Cloud flag
      IF (btd >= seuil ) CMa % Cld_108=.true. 
      IF (pix % Obs_108 < 258.15 .and. (pix_id%lat>-60 .and. pix_id%lat<60) ) CMa % Cld_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108)
      IF (pix % Obs_108 < 258.15 .and. (pix_id%lat>-60 .and. pix_id%lat<60)) CMa %  confcl_108=0.

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(1)=1
      cc_group(1,1) = CMa % confcl_108

      if(idbg%debug) then
         write(*,'(a15, f7.2)') 'sstloc ', sstloc
         write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108' , CMa % confcl_108, btd, '>', seuil
      endif
   END IF

   !    5. 108 – 120 Thin Cirrus Test
   !    -----------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_120 ) THEN

      btd= pix % Obs_108 - pix % Obs_120

      ! thresholds: for long paths, set to a default value 
      IF ( ABS(pix_id % satsec-99.0) < 0.0001) THEN
         seuil = CMa_SD_108_120
      ELSE
         seuil = thres % v_CMa_108_120_sea
         if(seuil < 0.1) seuil = CMa_SD_108_120
      END IF

      s_cl  = seuil - 0.25
      s_cld = seuil + 0.5

      ! Cloud flag
      IF (btd > seuil) CMa % Cld_108_120=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_120)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(5)=ngroup(5)+1
      cc_group(5,ngroup(5)) = CMa % confcl_108_120

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108_120' , CMa % confcl_108_120, btd, '>', seuil
   END IF

   !    2.1 37 – 40 Difference Test for lat 60S-60N
   !    -----------------------------------------------------
   ToDo=.false.
   IF (ToDo .and. pix % Qual_37  .and. pix % Qual_40 .and. &
       pix % Obs_37>CMa_37_tempmin .and. pix % Obs_37<CMa_37_saturation .and. &
       box % Lat < highLat .and. box % Lat > lowLat .and. .not. box % Glint) THEN

      btd = pix % Obs_37 - pix % Obs_40

      ! thresholds
      seuil = box%cosol * CMa_SD_37_40  + 0.12 * pix_id%sunzen!
      s_cl  = box%cosol * CMa_SD_37_40_Ccl+ 0.12 * pix_id%sunzen   ! confident clear
      s_cld = box%cosol * CMa_SD_37_40_Ccld+ 0.12 * pix_id%sunzen  ! confident cloudy

      ! Cloud flag
      IF (btd > seuil ) CMa % Cld_37_40=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_37_40)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_37_40

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') ' 37_40', CMa % confcl_37_40, btd, '>', seuil
   END IF

   !    2.2 108 – 37 Difference Test with no sunglint
   !    ----------------------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_37 .and. &
       pix % Obs_37>CMa_37_tempmin .and. pix % Obs_37<CMa_37_saturation .and. &
       .not. box % Glint) THEN

      btd = pix % Obs_108 - pix % Obs_37

      !  thresholds
      seuil = thres % v_CMa_108_37_sea  !
      s_cl  = seuil -2. ! confident clear
      s_cld = seuil +2. ! confident cloudy


      ! Cloud flag
      IF (btd > seuil) CMa % Cld_108_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_108_37

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '108_37 ', CMa % confcl_108_37, btd, '>', seuil
   END IF

   !    2.3 87 – 108 Difference Test 
   !    ------------------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_87 .and. pix % Qual_108 .and. pix % Qual_120) THEN

      ! brightness temperature differences, BT87 – BT108 and BT108 – BT120
      btd       = pix % Obs_87 - pix % Obs_108
      btd108_120 = pix % Obs_108 - pix % Obs_120

      !  thresholds: using a 3-degree polynomial function of btd108_120
      seuil = thres % v_CMa_87_108_sea
      s_cl  = seuil - 0.5       ! confident clear
      s_cld = seuil + 0.5       ! confident cloudy

      ! Cloud flag
      IF (btd >= seuil) CMa % Cld_87_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_87_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_87_108

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '87_108 ', CMa % confcl_87_108, btd, '>', seuil
   ENDIF

   !    3.1 08 Reflectance Test 
   !    -----------------------
   ToDo=.true.  !   if(box % coast ) ToDo=.false.
   IF (ToDo .and. pix % Qual_08) THEN

      !  thresholds
      IF (.not. box % Glint .and. .not. pix_id % lake) THEN
         seuil = thres % v_CMa_08_sea
         s_cl  = seuil -1.5             ! confident clear
         s_cld = seuil +1.5             ! confident cloudy
      ELSE
         seuil = box%cosol * CMa_glint_08
         s_cl  = box%cosol * CMa_glint_08_Ccl   ! confident clear
         s_cld = box%cosol * CMa_glint_08_Ccld  ! confident cloudy
      ENDIF
      
      ! Cloud flag
      IF (pix % Obs_08 > seuil) CMa % Cld_08=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix % Obs_08, s_cld, seuil, s_cl, CMa % confcl_08)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3,ngroup(3)) = CMa % confcl_08

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '08' , CMa % confcl_08,pix % Obs_08, '>', seuil
   ENDIF
   
      
   !    16 Reflectance Test 
   !    -----------------------
   ToDo=.true.  !   if(box % coast ) ToDo=.false.
   IF (ToDo .and. pix % Qual_16) THEN

      !  thresholds
      IF (.not. box % Glint .and. .not. pix_id % lake) THEN
         seuil = thres % v_CMa_16_sea
         s_cl  = seuil -0.5             ! confident clear
         s_cld = seuil +0.5             ! confident cloudy
      ELSE
         seuil = box%cosol * CMa_glint_08
         s_cl  = box%cosol * CMa_glint_08_Ccl   ! confident clear
         s_cld = box%cosol * CMa_glint_08_Ccld  ! confident cloudy
      ENDIF
      
      ! Cloud flag
      IF (pix % Obs_16 > seuil) CMa % Cld_16=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix % Obs_16, s_cld, seuil, s_cl, CMa % confcl_16)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3,ngroup(3)) = CMa % confcl_16

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '16' , CMa % confcl_16,pix % Obs_16, '>', seuil
   ENDIF

   
   ratio = pix % Obs_08 / pix % Obs_06
   
   if (ratio<0.1) then
   
      ! 06 Reflectance Test 
      ! -----------------------
      ToDo=.true.       !   if(box % coast ) ToDo=.false.
      IF (ToDo .and. pix % Qual_06) THEN

         !       thresholds
         IF (.not. box % Glint .and. .not. pix_id % lake) THEN
            seuil = thres % v_CMa_06_sea
            s_cl  = seuil -1.5          ! confident clear
            s_cld = seuil +1.5          ! confident cloudy
         ELSE
            seuil = box%cosol * CMa_glint_06
            s_cl  = box%cosol * (CMa_glint_06-4)        ! confident clear
            s_cld = box%cosol * (CMa_glint_06+4)        ! confident cloudy
         ENDIF
      
         ! Cloud flag
         IF (pix % Obs_06 > seuil) CMa % Cld_06=.true. 

         !      Confidence Clear level
         CALL maia_ConfTest (idbg, pix % Obs_06, s_cld, seuil, s_cl, CMa % confcl_06)

         ! number of tests done
         num_tests = num_tests +1

         ! for the final Confidence level
         ngroup(3)=ngroup(3)+1
         cc_group(3,ngroup(3)) = CMa % confcl_06

         if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '06' , CMa % confcl_06,pix % Obs_06, '>', seuil
      ENDIF
   endif
   

   !    3.2 08/06 Reflectance Ratio Test 
   !    --------------------------------
   ToDo=.true.
   if(box % coast .or. pix_id % twilight) ToDo=.false.
   IF (ToDo .and. pix % Qual_06 .and. pix % Qual_08 ) THEN

      ratio = pix % Obs_08 / pix % Obs_06

      !  thresholds
      IF (box % Glint) THEN
         seuil = CMa_glintRatio
         s_cl  = CMa_glintRatio_Ccl
         s_cld = CMa_glintRatio_Ccld
      ELSE
         seuil = CMa_SD_06_08
         s_cl  = CMa_SD_06_08_Ccl
         s_cld = CMa_SD_06_08_Ccld
      ENDIF

      ! Cloud flag
      IF ( ratio > seuil ) CMa % Cld_06_08=.true. 

     !  Confidence Clear level
      CALL maia_ConfTest (idbg, ratio, s_cld, seuil, s_cl, CMa % confcl_06_08)

     !  number of tests done
     num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3,ngroup(3)) =CMa % confcl_06_08

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '08/06' , CMa % confcl_06_08,ratio, '>', seuil
   ENDIF


   !    --------------------------------------------------------
   !    4.      13 high troposphere thin Cirrus Reflectance Test 
   !    --------------------------------------------------------
   ToDo=.true.  
   if(box % coast .or. pix_id % twilight) ToDo=.false.
   IF (ToDo .and. pix % Qual_13 .and. box % cwv > CMa_DD_13_cwv_cutoff) THEN

      !  thresholds
      seuil = box%cosol * 1.7   !CMa_LD_13
      s_cl  = box%cosol * 1.2   !CMa_LD_13_Ccl! confident clear
      s_cld = box%cosol * 2.2   !CMa_LD_13_Ccld ! confident cloudy
      
      ! Cloud flag
      IF (pix % Obs_13 > seuil ) CMa %Cld_13=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix % Obs_13, s_cld, seuil, s_cl, CMa % confcl_13)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(4)=ngroup(4)+1
      cc_group(4,ngroup(4)) = CMa % confcl_13

      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') '13', CMa % confcl_13, pix % Obs_13,'>',seuil
   ENDIF


   !    -----------------------------------------------------
   !    6. Spatial Textures
   !    -----------------------------------------------------
   ToDo=.false.
   IF (.not. box%coast ) ToDo=.true.
   
   ! thresholds
   if(ToDo) then
      if(pix_id % texture_fromImager)then
         seuil108   = cstI_sd108s
         seuil08    = cstI_sd08 + 0.03*pix % Obs_08
         seuil10837 = cstI_sd10837sd
      elseif (pix_id % texture_fromNeighbors) then
         seuil108   = cstA_sd108s
         seuil08    = cstA_sd08 + 0.03*pix % Obs_08
         seuil10837 = cstA_sd10837sd
      endif   
   endif

   if(ToDo .and. pix_id% sd_108>vmanq .and. pix_id% sd_10837>vmanq) then
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.1         ! confident cloudy

      s_cl_10837  = seuil10837 -0.1     ! confident clear
      s_cld_10837 = seuil10837 +0.1     ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 .and.pix_id% sd_10837 > seuil10837) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture
      CALL maia_ConfTest (idbg, pix_id% sd_10837, s_cld_10837, seuil10837, s_cl_10837, conf_texture)
      CMa % confcl_texture1= (CMa % confcl_texture1 + conf_texture )/2.

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2, a3,f10.2,a3,f10.2)') &
                     'texture1', CMa % confcl_texture1, pix_id% sd_108,'>', seuil108,&
                                                        '&',pix_id% sd_10837,'>', seuil10837
                                                        
   elseif(ToDo .and. pix_id% sd_108>vmanq ) then        ! AVHRR channel 3.7mm not available in daytime
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.1         ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 ) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2)') 'texture1', CMa % confcl_texture1, &
                                                           pix_id% sd_108,'>', seuil108
   endif
   
   if(ToDo .and. pix_id% sd_08>vmanq) then
      s_cl  = seuil08 -0.1      ! confident clear
      s_cld = seuil08 +0.1      ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_08 > seuil08) CMa % Cld_texture1=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix_id% sd_08, s_cld, seuil08, s_cl, CMa % confcl_texture2)
!      CMa % confcl_texture2 = (0.5+CMa % confcl_texture2)/2.
      if(CMa % confcl_texture2 <=.25)  CMa % confcl_texture2 = 0.25

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture2
      
      if(idbg%debug) write(*,'(a15,f5.1,f10.2,a3,f10.2, a3,f10.2)') &
                     'texture2', CMa % confcl_texture2, pix_id% sd_08,'>', seuil08
   endif
                      

   !    -------------------------------
   !    Final mask confidence 
   !    -------------------------------
   call maia_CMa_ConfClear (idbg, ngroup, cc_group,  CMa )

   if(idbg%debug) then
      write(*,'(a45, i5)') 'Conf_cl=3,prob_cl=2,prob_cld=1,conf_cld=0', CMa % conf_clear
   endif

   !    -------------------------------
   !    Quality flag
   !    -------------------------------
   call maia_CMa_quality (idbg, max_num_tests, num_tests, max_num_groups, ngroup, CMa % qual)
!   call maia_CMa_quality (idbg, max_num_tests, num_tests, CMa % qual)

   if(CMa % qual==0) CMa % conf_clear=vmanq

   !    -------------------------------
   !    clear/cloudy/snow/ice flag
   !    -------------------------------
   CMa % clear = 0
   if(CMa % Cld_108     .or. CMa % cld_06_08   .or. CMa % cld_08      .or. CMa % cld_37_108 .or.&
      CMa % cld_87_108 .or. CMa % cld_108_37 .or. CMa % cld_37_40 .or. CMa % cld_108_120  ) then
      CMa % clear = 1
   endif


   if(idbg%debug .and. CMa % conf_clear==vmanq) then
      write(*,'(a45, i5)') 'Conf_cl=3,prob_cl=2,prob_cld=1,conf_cld=0', CMa % conf_clear
   endif

END SUBROUTINE maia_CMa_SD
