! Name: $Id: maia_CMa_SG.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_SG 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant enhancement for quality flags
SUBROUTINE maia_CMa_SG  (idbg, pix_id, pix, box, thres, CMa)

! Authors       : lydie lavanant

! sea - glint
! max_num_tests = 7 (VIIRS), 6 (AVHRR)
!Gr 1.  Emission Threshold          : BT108                           added for coherence with maiav3
!Gr 2.  Emission Difference Tests   : BT87 - BT108
!Gr 3.  Reflectance Threshold Tests : Ref08  
!                                     Ref16/Ref06                     (ToDo = false)
!                                     LowCloudInSunGlint (06, 37-108)
!Gr 5.  Emission Thin Cirrus Test   : BT108 - BT120 
!Gr 6.  Textures                     : sd_108
!                                   : sd_08

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),     INTENT(in) :: idbg         ! 
  type( pix_info), INTENT(in)   :: pix_id       ! 
  type( pix_data), INTENT(in)   :: pix          ! 
  type( box_id ),    INTENT(in) :: box          ! 
  type( maia_thres),INTENT(in)  :: thres        ! 
  type( maia_CMa),  INTENT(out) :: CMa
  
! local
  INTEGER :: num_tests, max_num_tests, ngroup(ngroups_max), max_num_groups(ngroups_max)
  REAL    :: cc_group(ngroups_max,ngroups_max)
  REAL    :: seuil, s_cl, s_cld  
  real    :: btd,  btd37_108, btd108_120, ratio, sstloc
  REAL    :: seuil08, s_cl_08, s_cld_08
  REAL    :: seuil108, s_cl_108, s_cld_108

  logical :: ToDo
!======================================================================
   if(pix_id% ImgName == 'viirs') then
      max_num_tests = max_num_tests_VIIRS(4)
      max_num_groups(:) = max_num_group_VIIRS_SG(:)
   else
      max_num_tests = max_num_tests_AVHRR(4)
      max_num_groups(:) = max_num_group_AVHRR_SG(:)
   endif

   num_tests = 0
   ngroup(:)=0
   cc_group(:,:) = 0

   !    ---------------------------------------------------------------------------------
   !    TESTs
   if(idbg%debug) then
      write(*,'(a10)') CMa % test_name
      write(*,'(a57)') 'Confident_clear (1:clear/0:cloudy), Seuil, btd '
   endif

   !    --------------------------
   !    1. SST Test
   !    --------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 ) THEN

      ! thresholds: compute local sst or use forecast
      if (box % sst_clim > CMa_sstfrozen .and. pix % Qual_120 .and. .not. pix_id % lake) then   
          call maia_SST (idbg, pix_id, pix, box % sst_clim, sstloc)
          btd = box % sst_clim - sstloc
          btd = box % sst_clim - sstloc
          seuil   = cst_sst     !seuil   = thres % v_CMa_sst !? 
      else                      ! -> use forecast
          btd = box % tsurf - pix % Obs_108
          seuil   = thres % v_CMa_108_sea
      end if

      s_cl  = seuil - 2.0
      s_cld = seuil + 2.0

      ! Cloud flag
      IF (btd >= seuil ) CMa % Cld_108=.true. 
      IF (pix % Obs_108 < 258.15 .and. (pix_id%lat>-60 .and. pix_id%lat<60) ) CMa % Cld_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108)
      IF (pix % Obs_108 < 258.15 .and. (pix_id%lat>-60 .and. pix_id%lat<60)) CMa %  confcl_108=0.

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(1)=1
      cc_group(1,1) = CMa % confcl_108

      if(idbg%debug) then
         write(*,'(a15, f7.2)') 'sstloc ', sstloc
         write(*,'(a15,f5.1,2f10.2)') '108 ' , CMa % confcl_108, seuil, btd
      endif
   END IF

   !    --------------------------
   !    5. 108 – 120 Thin Cirrus Test
   !    --------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_120 ) THEN

      btd= pix % Obs_108 - pix % Obs_120

      ! threshold. For long  paths, set the threshold to a default value 
      seuil= thres % v_CMa_108_120_sea
      IF ( seuil < 0.1 .or. ABS(box % satsec-99.0) < 0.0001) THEN
         seuil = CMa_SD_108_120
      END IF
      s_cl  = seuil - 0.25
      s_cld = seuil + 0.5

      ! Set the 108 120 Cloud flag
      IF (btd > seuil) CMa % Cld_108_120=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_120)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(5)=ngroup(5)+1
      cc_group(5, ngroup(5)) = CMa % confcl_108_120

      if(idbg%debug) write(*,'(a15,f5.1,2f10.2)') '108_120 ', CMa % confcl_108_120, seuil, btd
   END IF

   !    ------------------------------------------
   !    2.3 87 – 108 Difference Test 
   !    ------------------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_87 .and. pix % Qual_108 .and. pix % Qual_120) THEN

      ! Calculate the brightness temperature differences, BT87 – BT108 and BT108 – BT120
      btd        = pix % Obs_87  - pix % Obs_108
      btd108_120 = pix % Obs_108 - pix % Obs_120

      ! thresholds using a 3-degree polynomial function of btd108_120
!      seuil = coeff1 + ( coeff2 * btd108_120 ) + ( coeff3 * btd108_120*btd108_120 ) + &
!                       ( coeff4 * btd108_120*btd108_120*btd108_120 )
      seuil = thres % v_CMa_87_108_sea
      s_cl  = seuil - 0.5
      s_cld = seuil + 0.5

      ! Cloud flag
      IF (btd >= seuil) CMa % Cld_87_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_87_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_87_108

      if(idbg%debug) write(*,'(a15,f5.1,2f10.2)') '87_108 ', CMa % confcl_87_108, seuil, btd
   ENDIF

   !    -------------------
   !    3.1 08 Reflectance Test 
   !    -------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_08 ) THEN

      !  thresholds
      seuil = box%cosol * CMa_glint_08
      s_cl  = box%cosol * CMa_glint_08_Ccl      ! confident clear
      s_cld = box%cosol * CMa_glint_08_Ccld     ! confident cloudy
      if(seuil<thres % v_CMa_08_sea) then
         seuil = thres % v_CMa_08_sea
         s_cl= 0.9*seuil
         s_cld=1.1*seuil
      endif
      
      ! Set the 08 Cloud flag
      IF (pix % Obs_08 > seuil .and.  (pix % Obs_16 / pix % Obs_06)<1.1) CMa % Cld_08=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix % Obs_08, s_cld, seuil, s_cl, CMa % confcl_08)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3, ngroup(3)) = CMa % confcl_08

      if(idbg%debug) write(*,'(a15,f5.1,2f10.2)') '08 ', CMa % confcl_08, seuil, pix % Obs_08
   ENDIF

   !    ----------------------------
   !    3.2 06/16 Reflectance Ratio Test 
   !    ----------------------------
   ToDo=.false.
   if(box % coast ) ToDo=.false.
   IF (ToDo .and. pix % Qual_06 .and. pix % Qual_16 ) THEN

      ! Calculate the 08/06 ratio
      ratio = pix % Obs_16 / pix % Obs_06

      !  thresholds
      seuil = CMa_glintRatio
      s_cl  = CMa_glintRatio_Ccl
      s_cld = CMa_glintRatio_Ccld
      seuil = 1.1
      s_cl  = CMa_glintRatio_Ccl
      s_cld = CMa_glintRatio_Ccld
      
      ! Set the 06_08 Cloud flag
      IF ( ratio > seuil ) CMa % Cld_06_08=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, ratio, s_cld, seuil, s_cl, CMa % confcl_06_08)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3, ngroup(3)) = CMa % confcl_06_08

      if(idbg%debug) write(*,'(a15,f5.1,3f10.2)') '06/16 ', CMa % confcl_06_08, seuil, ratio
   ENDIF

   !    -------------------------------
   !    3.3 Low SunGlint Reflectance Test 
   !    -------------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_06 .and. pix % Qual_37 .and. pix % Qual_108 .and. &
       pix % Obs_37 < CMa_37_saturation .and. box%cosol < cosolmax_sea) THEN

      !  thresholds
      seuil = CMa_Glint_06 * box%cosol
      s_cl  = seuil     ! confident clear
      s_cld = seuil     ! confident cloudy

      ! cloud flag for Low Cloud in SunGlint
      btd37_108 = pix % Obs_37 - pix % Obs_108
      IF (pix % Obs_06 > seuil  .and. &
          btd37_108 > CMa_glint_37_108* box%cosol .and. &
          btd37_108 < CMa_glint06coef*pix % Obs_06) THEN
          CMa % Cld_LowSunGlint=.true. 
      ENDIF

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix % Obs_06, s_cld, seuil, s_cl, CMa % confcl_LowSunGlint)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3, ngroup(3)) = CMa % confcl_LowSunGlint

      if(idbg%debug) then
         write(*,'(a15,f5.1,5f10.2)') 'Low ', CMa % confcl_LowSunGlint, seuil, pix % Obs_06, &
                                              CMa_glint_37_108* box%cosol, btd37_108, CMa_glint06coef*pix % Obs_06
      endif
   ENDIF


   !    -----------------------------------------------------
   !    6. Spatial Textures
   !    -----------------------------------------------------
   ToDo=.false.
   IF (.not. box%coast ) ToDo=.true.
   
   ! thresholds
   if(ToDo) then
      if(pix_id % texture_fromImager)then
         seuil108         = cstI_sd108s + 0.35
         seuil08          = cstI_sd08  + 0.50 + 0.06*pix % Obs_08
      elseif (pix_id % texture_fromNeighbors) then
         seuil108         = cstA_sd108s + 0.35
         seuil08          = cstA_sd08  + 0.50 + 0.06*pix % Obs_08
      endif   
   endif

   if(ToDo .and. pix_id% sd_108>vmanq .and. pix % Qual_108) then
   
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.1         ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108, s_cl_108, CMa % confcl_texture1)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg % debug) write(*,*) 'texture1: ', pix_id% sd_108,'>', seuil108, CMa % confcl_texture1
   endif
   
   if(ToDo .and. pix_id% sd_08>vmanq .and. pix % Qual_08) then
   
      s_cl_08  = seuil08 -0.1   ! confident clear
      s_cld_08 = seuil08 +0.1   ! confident cloudy
      
      ! Set the Cloud flag
      IF (pix_id% sd_08 > seuil08)CMa % Cld_texture2=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix_id% sd_08,   s_cld_08,   seuil08, s_cl_08, CMa % confcl_texture2)
!      CMa % confcl_texture2 = (0.5+CMa % confcl_texture2)/2.
      if(CMa % confcl_texture2 <=.25)  CMa % confcl_texture2 = 0.25

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture2
      
      if(idbg % debug)write(*,*) 'texture2: ',  pix_id% sd_08,'>', seuil08, CMa % confcl_texture2
   endif

   
   !    -------------------------------
   !    Final mask confidence 
   !    -------------------------------
   call maia_CMa_ConfClear (idbg, ngroup, cc_group,  CMa )
   if(idbg%info) write(*,'(a20,i5)') 'CMa % conf_clear: ',CMa % conf_clear

   !    -------------------------------
   !    Quality flag
   !    -------------------------------
   call maia_CMa_quality (idbg, max_num_tests, num_tests, max_num_groups, ngroup, CMa % qual)
!   call maia_CMa_quality (idbg, max_num_tests, num_tests, CMa % qual)
   if(CMa % qual==0) CMa % conf_clear=vmanq

   if(idbg%info) then
      write(*,'(a45, i5)') 'Conf_cl=3,prob_cl=2,prob_cld=1,conf_cld=0', CMa % conf_clear
      write(*,'(a45, i5)') 'Qual:high=3,medium=2,poor=1,bad=0', CMa % qual
   endif
END SUBROUTINE maia_CMa_SG
