! Name: $Id: maia_CMa_SN.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_SN 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant add extreme cooling and enhancement for quality flags
SUBROUTINE maia_CMa_SN  (idbg, pix_id, pix, box, thres, CMa)

! Authors       : lydie lavanant

! sea - night

!max_num_tests = 7 (VIIRS), 6 (AVHRR)
!Gr 1.  Emission Threshold Test   : BT108 Test (SST or BT)
!Gr 2.  Emission Difference Test  : BT120 – BT37 for BT37 > BT37_limit (230K)
!                                   BT37 – BT108
!                                   BT108 – BT37
!                                   BT87 - BT108, seuil fct de 108-120
!Gr 5.  Emission Thin Cirrus Test : BT108 – BT120
!Gr 6.  Texture                   : sd_108 & sd_10837

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input
  type( debug ),     INTENT(in) :: idbg         ! 
  type( pix_info), INTENT(in)   :: pix_id       ! 
  type( pix_data), INTENT(in)   :: pix          ! 
  type( box_id ), INTENT(in)    :: box          ! 
  type( maia_thres),INTENT(in)  :: thres        ! 
  type( maia_CMa),  INTENT(out) :: CMa
  
! local
  INTEGER :: num_tests, max_num_tests, ngroup(ngroups_max), max_num_groups(ngroups_max)
  real    :: cc_group(ngroups_max,ngroups_max)
  REAL    :: seuil, s_cl, s_cld  
  REAL    :: conf_texture
  REAL    :: seuil108, s_cl_108, s_cld_108
  REAL    :: seuil10837, s_cl_10837, s_cld_10837
  real    :: btd, sstloc
  logical :: ToDo

! function
   
!======================================================================
   if(pix_id% ImgName == 'viirs') then
      max_num_tests = max_num_tests_VIIRS(1)
      max_num_groups(:) = max_num_group_VIIRS_SN(:)
   else
      max_num_tests = max_num_tests_AVHRR(1)
      max_num_groups(:) = max_num_group_AVHRR_SN(:)
   endif

   num_tests = 0
   ngroup(:)=0
   cc_group(:,:) = 0

   if(idbg%debug) write(*,'(a15, a50)') CMa % test_name, 'individual confidence 1.(clear) to 0.(cloudy)'
   !    ---------------------
   !    1. SST Test
   !    ---------------------
   IF (pix % Qual_108 ) THEN

      ! Set the threshold: compute local sst or use forecast
      if (box % sst_clim > CMa_sstfrozen .and. pix % Qual_120 .and. pix % Qual_37 .and. &
          .not. pix_id % lake) then     
          call maia_SST (idbg, pix_id, pix, box % sst_clim, sstloc)
          btd = box % sst_clim - sstloc
          seuil   = cst_sst     !seuil   = thres % v_CMa_sst !? 
      else                      ! -> use forecast
          btd = box % tsurf - pix % Obs_108
          seuil   = thres % v_CMa_108_sea
          if((box % ts_prev - box % t2m_prev)>5.) seuil = seuil + 0.5*(box % ts_prev - box % t2m_prev)
          ! extreme cooling
          if(box % ts_prev<263. ) then
             if((box % tsurf - thres % v_CMa_108_sea)< 255) then
                 seuil = seuil -5. - 0.8*(255 - (box % tsurf - thres % v_CMa_108_sea))
             else
                 seuil = seuil -5.
             endif
          endif
      end if

      s_cl  = seuil - 1.0       !2.0
      s_cld = seuil + 1.0       !2.0

      ! Set the 108 Cloud flag
      IF (btd >= seuil ) CMa % Cld_108=.true. 
      IF (pix % Obs_108 < 258.15 .and. (pix_id%lat>-60 .and. pix_id%lat<60) ) CMa % Cld_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108)
      IF (pix % Obs_108 < 258.15 .and. (pix_id%lat>-60 .and. pix_id%lat<60)) CMa %  confcl_108=0.

      ! number of tests done
      num_tests = num_tests +1
      
      ! for the final Confidence level
      ngroup(1)=1
      cc_group(1,1) = CMa % confcl_108

      if(idbg%debug) then
         write(*,'(a15,f6.2,f10.2,a3,f10.2)') '108' , CMa % confcl_108, btd, '>', seuil
         if(box % sst_clim > CMa_sstfrozen .and. (box % ts_prev - box % t2m_prev)>5.) write(*,*) &
           '108: sst frozen. account for a radiative surface effect of ', 0.5*(box % ts_prev - box % t2m_prev)
      endif
   END IF

   !    --------------------------
   !    5. 108 – 120 Thin Cirrus Test
   !    --------------------------
   IF (pix % Qual_108 .and. pix % Qual_120) THEN

      btd= pix % Obs_108 - pix % Obs_120

      ! Set the threshold.For small thresholds or long paths, set the threshold to a default value 
      seuil= thres % v_CMa_108_120_sea
      IF ( seuil < 0.1 .or. ABS(box % satsec-99.0) < 0.0001) THEN
         seuil = CMa_SN_108_120
      END IF
      s_cl  = seuil - 0.25
      s_cld = seuil + 0.5

      ! Set the 108 120 Cloud flag
      IF (btd > seuil) CMa % Cld_108_120=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl , CMa % confcl_108_120)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(5)=ngroup(5)+1
      cc_group(5,ngroup(5)) = CMa % confcl_108_120

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '108_120', CMa % confcl_108_120, btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.3 87 – 108 Difference Test 
   !    ------------------------------------------
   IF (pix % Qual_87 .and. pix % Qual_108 .and. pix % Qual_120) THEN

      btd      = pix % Obs_87 - pix % Obs_108

      !  thresholds:
      seuil = thres % v_CMa_87_108_sea

      s_cl  = seuil - 0.5
      s_cld = seuil + 0.5

      ! Set the 87_108_120 Cloud flag
      IF (btd > seuil) CMa % Cld_87_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl , CMa % confcl_87_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_87_108

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '87_108', CMa % confcl_87_108, btd,'>',seuil
   ENDIF

   !    ------------------------------------------
   !    2.5 37 – 108 Difference Test 
   !    ------------------------------------------
   IF (pix % Qual_108 .and. pix % Qual_37 .and. pix % Obs_37 > limit_37) THEN

      btd = pix % Obs_37 - pix % Obs_108

      !  thresholds
      seuil = thres % v_CMa_37_108_sea
      s_cl  = thres % v_CMa_37_108_sea -0.5     ! confident clear
      s_cld = thres % v_CMa_37_108_sea +0.5     ! confident cloudy

      ! Set the 37-108 Cloud flag
      IF (btd > seuil) CMa % Cld_37_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd,s_cld, seuil, s_cl , CMa % confcl_37_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_37_108

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '37_108', CMa % confcl_37_108, btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.4 120 – 37 Difference Test 
   !    ------------------------------------------
   IF (pix % Qual_120 .and. pix % Qual_37 .and. pix % Obs_37 > limit_37) THEN

      btd = pix % Obs_120 - pix % Obs_37

      !  thresholds
      seuil     = thres % v_CMa_120_37_sea
      s_cl  = thres % v_CMa_120_37_sea -0.5     ! confident clear
      s_cld = thres % v_CMa_120_37_sea +0.5     ! confident cloudy

      ! Set the 120_37 Cloud flag
      IF (btd > seuil) CMa % Cld_120_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_120_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_120_37

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '120_37', CMa % confcl_120_37,btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.2 108 – 37 Difference Test 
   !    ------------------------------------------
   IF (pix % Qual_108 .and. pix % Qual_37 .and. pix % Obs_37 > limit_37) THEN

      btd = pix % Obs_108 - pix % Obs_37

      !  thresholds
      seuil = thres % v_CMa_108_37_sea
      s_cl  = thres % v_CMa_108_37_sea -0.5     ! confident clear
      s_cld = thres % v_CMa_108_37_sea +0.5     ! confident cloudy

      ! Set the 120_37 Cloud flag
      IF (btd > seuil) CMa % Cld_108_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2,ngroup(2)) = CMa % confcl_108_37

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '108_37', CMa % confcl_108_37,btd,'>',seuil
   END IF
   

   !    -----------------------------------------------------
   !    Spatial Textures
   !    -----------------------------------------------------
   ToDo=.false.
   IF (.not. box%coast ) ToDo=.true.

   ! thresholds
   if(ToDo) then
      if(pix_id % texture_fromImager)then
         seuil108   = cstI_sd108s
         seuil10837 = cstI_sd10837sn
      elseif (pix_id % texture_fromNeighbors) then
         seuil108   = cstA_sd108s
         seuil10837 = cstA_sd10837sn
      endif   
   endif

   if(ToDo .and. pix_id% sd_108>vmanq .and. pix_id% sd_10837>vmanq) then
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.1         ! confident cloudy

      s_cl_10837  = seuil10837 -0.1     ! confident clear
      s_cld_10837 = seuil10837 +0.1     ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 .and.pix_id% sd_10837 > seuil10837) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture
      CALL maia_ConfTest (idbg, pix_id% sd_10837, s_cld_10837, seuil10837, s_cl_10837, conf_texture)
      CMa % confcl_texture1= (CMa % confcl_texture1 + conf_texture )/2.
      if(CMa % confcl_texture1<0.25) CMa % confcl_texture1=0.25

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2,a3,f10.2,a3,f10.2)') 'texture', CMa % confcl_texture1, &
                                        pix_id% sd_108,'>', seuil108, '&', pix_id% sd_10837,'>', seuil
    endif
                      
   
   !    -------------------------------
   !    Final mask confidence 
   !    -------------------------------
   call maia_CMa_ConfClear (idbg, ngroup, cc_group,  CMa )

   !    -------------------------------
   !    Quality flag
   !    -------------------------------
   call maia_CMa_quality (idbg, max_num_tests, num_tests, max_num_groups, ngroup, CMa % qual)
!   call maia_CMa_quality (idbg, max_num_tests, num_tests, CMa % qual)

   if(CMa % qual==0) CMa % conf_clear=vmanq


END SUBROUTINE maia_CMa_SN
