! Name: $Id: maia_CMa_ST.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: 
!          Subroutines maia_CMa_ST 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant add extreme cooling and enhancement for quality flags
!  1.4    21/06/2017   Pascale Roquet remove test on metop01
SUBROUTINE maia_CMa_ST  (idbg, pix_id, pix, box, thres, CMa)
! Authors       : lydie lavanant

! sea - twilight
! max_num_tests = 9 (VIIRS), 8 (AVHRR)
! Gr 1. Emission Threshold          : BT108
! Gr 2. Emission Difference Tests   : BT108 – BT37
!                                     BT87 - BT108
!                                     BT37 - BT108 (low cloud detection) for coherence with maiav3
!                                     BT120 – BT37                       for coherence with maiav3
! Gr 3. Reflectance Threshold Tests : Ref08  
!                                     Ref08/Ref06                        (ToDo =  false)
! Gr 4. Reflectance Thin Cirrus     : Ref16                              for coherence with maiav3
! Gr 5. Emission Thin Cirrus Test   : BT108 – BT120 
! Gr 6. texture                     : sd_108 & sd_10837                  for coherence maiav3 et seviri

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),     INTENT(in) :: idbg         ! 
  type( pix_info), INTENT(in)   :: pix_id       ! 
  type( pix_data), INTENT(in)   :: pix          ! 
  type( box_id ),    INTENT(in) :: box          ! 
  type( maia_thres ),INTENT(in):: thres         ! 
  type( maia_CMa), INTENT(out) :: CMa
  
! local
  INTEGER :: num_tests, max_num_tests, ngroup(ngroups_max), max_num_groups(ngroups_max)
  REAL    :: cc_group(ngroups_max,ngroups_max)
  REAL    :: seuil, s_cl, s_cld  
  REAL    :: conf_texture
  REAL    :: seuil108, s_cl_108, s_cld_108
  REAL    :: seuil10837, s_cl_10837, s_cld_10837
  real    :: btd, btd108_120, ratio, sstloc
  logical :: ToDo

!======================================================================
   if(pix_id% ImgName == 'viirs') then
      max_num_tests = max_num_tests_VIIRS(2)
      max_num_groups(:) = max_num_group_VIIRS_ST(:)
   else
      max_num_tests = max_num_tests_AVHRR(2)
      max_num_groups(:) = max_num_group_AVHRR_ST(:)
      if(pix % Qual_06) then
         if( .not. pix % Qual_37) then
            max_num_tests = max_num_tests -3 ! tests with 3.7 not done
         endif
      endif
      if(pix % Qual_37 .and. .not. pix % Qual_16) then
         max_num_tests = max_num_tests -1 ! tests with 1.6 not done
      endif
   endif

   num_tests = 0
   ngroup(:)=0
   cc_group(:,:) = 0

   if(idbg%debug) write(*,'(a15)') CMa % test_name
   if(idbg%debug) write(*,'(2a15)') 'satimg: ',pix_id % satimg

   !    --------------------------
   !    1. SST Test
   !    --------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 ) THEN

      ! thresholds: compute local sst or use forecast
      if (box % sst_clim > CMa_sstfrozen .and. pix % Qual_120  .and. .not. pix_id % Lake) then     
          call maia_SST (idbg, pix_id, pix, box % sst_clim, sstloc)
          btd = box % sst_clim - sstloc
          seuil   = cst_sst     !seuil   = thres % v_CMa_sst !? 
      else                      ! -> use forecast
          btd = box % tsurf - pix % Obs_108
          seuil   = thres % v_CMa_108_sea
          ! extreme cooling
          if(box % ts_prev<263. ) then
             if((box % tsurf - thres % v_CMa_108_sea)< 255) then
                 seuil = seuil -5. - 0.8*(255 - (box % tsurf - thres % v_CMa_108_sea))
             else
                 seuil = seuil -5.
             endif
          endif
      end if

      ! Set the 108 Cloud flag
      IF (btd >= seuil ) CMa % Cld_108=.true. 
      IF (pix % Obs_108 < 258.15 .and. (pix_id%lat>-60 .and. pix_id%lat<60) ) CMa % Cld_108=.true. 

      ! Confidence Clear level
      s_cld = seuil + 2.0
      s_cl  = seuil - 2.0
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108)
      IF (pix % Obs_108 < 258.15 .and. (pix_id%lat>-60 .and. pix_id%lat<60)) CMa %  confcl_108=0.

      ! number of tests done
      num_tests = num_tests +1
      
      ! for the final Confidence level
      ngroup(1)=1
      cc_group(1,1) = CMa % confcl_108

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '108' , CMa % confcl_108, btd, '>', seuil
   END IF

   !    --------------------------
   !    5. 108 – 120 Thin Cirrus Test
   !    --------------------------
   ToDo=.true.
   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_120 ) THEN

      btd= pix % Obs_108 - pix % Obs_120

      seuil = thres % v_CMa_108_120_sea
      s_cld = seuil + 0.5
      s_cl  = seuil - 0.25

      ! Cloud flag
      IF (btd > seuil) CMa % Cld_108_120=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_120)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(5)=ngroup(5)+1
      cc_group(5, ngroup(5)) = CMa % confcl_108_120

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '108_120', CMa % confcl_108_120, btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.2 108 – 37 Difference Test with no sunglint
   !    ------------------------------------------
   ToDo=.true.

   IF (ToDo .and. pix % Qual_108 .and. pix % Qual_37 .and. &
       pix % Obs_37>CMa_37_tempmin .and. pix % Obs_37<CMa_37_saturation .and. &
       .not. box % Glint) THEN

      btd = pix % Obs_108 - pix % Obs_37

      !  thresholds
      seuil = thres % v_CMa_108_37_sea  !
      s_cl  = seuil -0.5 !-2. ! confident clear
      s_cld = seuil +0.5 !+2. ! confident cloudy

      ! Cloud flag
      IF (btd > seuil) CMa % Cld_108_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_108_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_108_37

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '108_37', CMa % confcl_108_37,btd,'>',seuil
   END IF

   !    ------------------------------------------
   !    2.3 87 – 108 Difference Test 
   !    ------------------------------------------
   ToDo=.true.
   IF (pix % Qual_87 .and. pix % Qual_108 .and. pix % Qual_120) THEN

      btd = pix % Obs_87 - pix % Obs_108
      btd108_120 = pix % Obs_108 - pix % Obs_120

      !  thresholds: 
      seuil = thres % v_CMa_87_108_sea
      s_cl  = seuil - 0.5       ! confident clear
      s_cld = seuil + 0.5       ! confident cloudy

      ! Cloud flag
      IF (btd >= seuil) CMa % Cld_87_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_87_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_87_108

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '87_108', CMa % confcl_87_108, btd,'>',seuil
   ENDIF

   !    ------------------------------------------
   !    2.4 37 – 108 Difference Test 
   !    ------------------------------------------
   ToDo=.true.

   IF (ToDo .and. pix % Qual_37 .and. pix % Qual_108 .and. &
       pix % Obs_37>CMa_37_tempmin .and. pix % Obs_37<CMa_37_saturation ) THEN

      btd = pix % Obs_37 - pix % Obs_108

      !  thresholds
      seuil = thres % v_CMa_37_108_sea  !
      s_cl  = seuil             ! confident clear
      s_cld = seuil             ! confident cloudy

      ! Set the 108_37 Cloud flag
      IF (btd < seuil) CMa % Cld_37_108=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_37_108)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_37_108

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '37_108', CMa % confcl_37_108, btd,'>',seuil
   ENDIF

   !    ------------------------------------------
   !    2.5 120 – 37 Difference Test 
   !    ------------------------------------------
   ToDo=.true.

   IF (ToDo .and. pix % Qual_120 .and. pix % Qual_37 .and. &
       pix % Obs_37>CMa_37_tempmin .and. pix % Obs_37<CMa_37_saturation ) THEN

      ! Calculate the brightness temperature difference, btd
      btd = pix % Obs_120 - pix % Obs_37

      !  thresholds
      seuil = thres % v_CMa_120_37_sea
      s_cl  = seuil -0.5        ! confident clear
      s_cld = seuil +0.5        ! confident cloudy

      ! Set the 120_37 Cloud flag
      IF (btd > seuil) CMa % Cld_120_37=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, btd, s_cld, seuil, s_cl, CMa % confcl_120_37)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(2)=ngroup(2)+1
      cc_group(2, ngroup(2)) = CMa % confcl_120_37

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '120_37', CMa % confcl_120_37,btd,'>',seuil
   END IF

   !    -------------------
   !    3.1 08 Reflectance Test 
   !    -------------------
   ToDo=.true.
   IF (pix % Qual_08 ) THEN

      !  thresholds
      IF (.not. box % Glint .and. .not. pix_id % lake) THEN
         seuil = thres % v_CMa_08_sea
         s_cl  = seuil -1.5             ! confident clear
         s_cld = seuil +1.5             ! confident cloudy
!         seuil = box%cosol * CMa_SD_08
!         s_cl  = box%cosol * CMa_SD_08_Ccl     ! confident clear
!         s_cld = box%cosol * CMa_SD_08_Ccld    ! confident cloudy
      ELSE
         seuil = box%cosol * CMa_glint_08
         s_cl  = box%cosol * CMa_glint_08_Ccl           ! confident clear
         s_cld = box%cosol * CMa_glint_08_Ccld  ! confident cloudy
      ENDIF
      
      ! Set the 08 Cloud flag
      IF (pix % Obs_08 > seuil)  CMa % Cld_08=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix % Obs_08, s_cld, seuil, s_cl, CMa % confcl_08)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3, ngroup(3)) = CMa % confcl_08

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '08', CMa % confcl_08, pix % Obs_08,'>',seuil
   ENDIF

   !    ----------------------------
   !    3.2 08/06 Reflectance Ratio Test 
   !    ----------------------------
   ToDo=.false. !   
   IF (ToDo .and. pix % Qual_06 .and. pix % Qual_08 ) THEN

      ! Calculate the 08/06 ratio
      ratio = pix % Obs_08 / pix % Obs_06

      !  thresholds
      IF (box % Glint) THEN
         seuil = CMa_glintRatio
         s_cl  = CMa_glintRatio_Ccl
         s_cld = CMa_glintRatio_Ccld
      ELSE
         seuil = CMa_SD_06_08
         s_cl  = CMa_SD_06_08_Ccl
         s_cld = CMa_SD_06_08_Ccld
      ENDIF

      ! Set the 06_08 Cloud flag
      IF ( ratio > seuil ) CMa % Cld_06_08=.true. 

     !  Confidence Clear level
      CALL maia_ConfTest (idbg, ratio, s_cld, seuil, s_cl, CMa % confcl_06_08)

     !  number of tests done
     num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(3)=ngroup(3)+1
      cc_group(3, ngroup(3)) = CMa % confcl_06_08

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '06_08', CMa % confcl_06_08, ratio,'>',seuil
   ENDIF

   !    -------------------------------
   !    4. 16 Thin Cirrus Reflectance Test 
   !    -------------------------------
   ToDo=.true.
   IF (pix % Qual_16 ) THEN
 
     !   thresholds
      if(thres % v_CMa_16_sea>vmanq) then
         seuil = thres % v_CMa_16_sea
         s_cl  = seuil-0.4              ! confident clear
         s_cld = seuil+0.4              ! confident cloudy
      else
         seuil = box%cosol * CMa_SD_16
         s_cl  = box%cosol * CMa_SD_16_Ccl      ! confident clear
         s_cld = box%cosol * CMa_SD_16_Ccld     ! confident cloudy
      endif

      ! Set the M9 Cloud flag
      IF (pix % Obs_16 > seuil) CMa % Cld_16=.true. 

      ! Confidence Clear level
      CALL maia_ConfTest (idbg, pix % Obs_16, s_cld, seuil, s_cl, CMa % confcl_16)

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(4)=ngroup(4)+1
      cc_group(4, ngroup(4)) = CMa % confcl_16

      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') '16', CMa % confcl_16, pix % Obs_16,'>',seuil
   ENDIF

   
   !    -----------------------------------------------------
   !    Spatial Textures
   !    -----------------------------------------------------
   ToDo=.false.
   IF (.not. box%coast ) ToDo=.true.

   ! thresholds
   if(ToDo) then
      if(pix_id % texture_fromImager)then
         seuil108   = cstI_sd108s
         seuil10837 = cstI_sd10837sn
      elseif (pix_id % texture_fromNeighbors) then
         seuil108   = cstA_sd108s
         seuil10837 = cstA_sd10837sn
      endif   
   endif

   if(ToDo .and. pix_id% sd_108>vmanq .and. pix_id% sd_10837>vmanq) then
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.1         ! confident cloudy

      s_cl_10837  = seuil10837 -0.1     ! confident clear
      s_cld_10837 = seuil10837 +0.1     ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 .and.pix_id% sd_10837 > seuil10837) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture
      CALL maia_ConfTest (idbg, pix_id% sd_10837, s_cld_10837, seuil10837, s_cl_10837, conf_texture)
      CMa % confcl_texture1= (CMa % confcl_texture1 + conf_texture )/2.

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2,a3,f10.2,a3,f10.2)') 'texture', CMa % confcl_texture1, &
                                        pix_id% sd_108,'>', seuil108, '&', pix_id% sd_10837,'>', seuil
                                                        
   elseif(ToDo .and. pix_id% sd_108>vmanq ) then        ! AVHRR channel 3.7mm not available in daytime
      s_cl_108  = seuil108 -0.1         ! confident clear
      s_cld_108 = seuil108 +0.1         ! confident cloudy

      ! Set the Cloud flag
      IF (pix_id% sd_108 > seuil108 ) CMa % Cld_texture1=.true. 

      ! Confidence Clear level               
      CALL maia_ConfTest (idbg, pix_id% sd_108,   s_cld_108,   seuil108,   s_cl_108,  conf_texture)
      CMa % confcl_texture1=conf_texture

      ! number of tests done
      num_tests = num_tests +1

      ! for the final Confidence level
      ngroup(6)=ngroup(6)+1
      cc_group(6,ngroup(6)) = CMa % confcl_texture1
      
      if(idbg%debug) write(*,'(a15,f6.2,f10.2,a3,f10.2)') 'texture1', CMa % confcl_texture1, &
                                                           pix_id% sd_108,'>', seuil108
   endif
                      
 
   !    -------------------------------
   !    Final mask confidence 
   !    -------------------------------
   call maia_CMa_ConfClear (idbg, ngroup, cc_group,  CMa)

   !    -------------------------------
   !    Quality flag
   !    -------------------------------
   call maia_CMa_quality (idbg, max_num_tests, num_tests, max_num_groups, ngroup, CMa % qual)
!   call maia_CMa_quality (idbg, max_num_tests, num_tests, CMa % qual)
   if(CMa % qual==0) CMa % conf_clear=vmanq

END SUBROUTINE maia_CMa_ST
