! Name: $Id: maia_Cal_AtmCorrVis.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Cal_AtmCorrVis 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE  maia_Cal_AtmCorrVis (idbg, box, thvis, A0, A1, A2, ic)

! Authors       : HLeGleau, lydie lavanant

! computes the coefficients used in the simulation of channels 0.65, 0.86 or 1.6 micron
! 6S version 4 was used to compute the tables
! uses a continental aerosol of 35 km visibility

 USE MOD_MAIA_CONST
 USE MOD_maia_types

IMPLICIT NONE

  INTEGER,          INTENT(in)  :: ic           ! channel number (1, 2, 3)                    
  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( maia_VISThresTables), intent(in) :: thvis   ! tabulated threshold (over sea or land)
  REAL,             INTENT(out) :: A0,A1,A2     ! coefficients  

  REAL                  :: UO3          ! ozone content (cm NTP)
  REAL                  :: rphaz
  REAL(KIND=8)          :: musol,musat
  REAL(KIND=8)          :: longh2o,longo3,longgaz
  INTEGER(KIND=8)       :: IW,IOXY
  INTEGER(KIND=8)       :: iang,isol,isat,iazi
  REAL(KIND=8)          :: difang,difsol,difsat,difaz
  REAL(KIND=8)          :: satzen_scat
  INTEGER(KIND=8)       :: ksat,ksol,kazi
  REAL(KIND=8)          :: diffemis,difftransu,difftransd,gaztrans

  if(idbg%debug) then
    write(*,*) "maia_Cal_AtmCorrVis"  
  endif
!=======================================================
  a2 = -9999.
  a1 = -9999.
  a0 = -9999.
  UO3 = .35     !put to a default value

! Test on angles 
  if(box % sunzen > 89. .or. box % sunzen < 0..or.      &
     box % satzen > 89. .or. box % satzen < 0..or.      &
     abs(box % difazi) > 360.) then
     return
  endif

! computes the difference of azimut between 0 and 180
  rphaz = abs(box % difazi)
  if(rphaz > 180) rphaz = 360.-rphaz
  if(rphaz.eq.180.) rphaz = 179.9

! limit zenith range for aerosol emission computation
! because 6S simulation is doubtful
  satzen_scat = box % satzen
  if(box % satzen>=70.) satzen_scat = 69.9

! Calcul des cosinus des angles zenitaux, de l angle de diffusion
  musol = cos(cdr * box % sunzen)
  musat = cos(cdr * box % satzen)

! compute optical path for gazes
  longgaz = (1. / musol + 1. / musat) 
  longH2o = longgaz * box % cwv          
  longo3 =  longgaz * UO3           

! compute indices for gaz absorption

  IW = (longH2o - H2oMin)/H2oPas + 1 
  if(IW > NbH2o) IW = NbH2o
  if(IW < 1) IW = 1
  IOxy = (longgaz - OxyMin)/OxyPas + 1  
  if(IOxy > NbOxy) IOxy = NbOxy
  if(IOxy < 1) IOxy = 1
  
  gaztrans = thvis%TrH2O(IW,ic)*thvis%TrOxy(IOxy,ic)*(thvis%TrO3(ic)**longo3)

! compute indices for transmittance due to scattering

  iang =   (box % sunzen-difftransmin)/difftranspas + 1
  difang = (box % sunzen-difftransmin-(iang-1)*difftranspas)/difftranspas
  difftransd = thvis%transdiff(iang,ic)*(1.-difang) + thvis%transdiff(iang+1,ic)*difang
  iang =   (satzen_scat-difftransmin)/difftranspas + 1
  difang = (box % satzen-difftransmin-(iang-1)*difftranspas)/difftranspas
  difftransu = thvis%transdiff(iang,ic)*(1.-difang) + thvis%transdiff(iang+1,ic)*difang

! compute indices for emission due to scattering

  isol =   (box % sunzen-diffemissolmin)/diffemissolpas + 1
  difsol = (box % sunzen-diffemissolmin-(isol-1)*diffemissolpas)/diffemissolpas 
  isat =   (satzen_scat-diffemissatmin)/diffemissatpas + 1
  difsat = (box % satzen-diffemissatmin-(isat-1)*diffemissatpas)/diffemissatpas 
  iazi =  (rphaz-diffemisazimin)/diffemisazipas + 1
  difaz = (rphaz-diffemisazimin-(iazi-1)*diffemisazipas)/diffemisazipas 
        
! interpolation in aerosol emission tables (3 angles)
  diffemis = 0.
  do ksol=0,1,1
  do ksat=0,1,1
  do kazi=0,1,1
     diffemis = diffemis +                                              &
                  thvis % emisdiff(ksol+isol,ksat+isat,kazi+iazi,ic) *  &
                  ((1-difsat)*(1-ksat)+difsat*ksat) *                   &
                  ((1-difsol)*(1-ksol)+difsol*ksol) *                   &
                  ((1-difaz )*(1-kazi)+difaz *kazi) 
  enddo
  enddo
  enddo

  a0 = 100.*diffemis*gaztrans
  a1 = 100.*difftransu*difftransd*gaztrans
  a2 = thvis % spherediff(ic)
  
END SUBROUTINE maia_Cal_AtmCorrVis  
