! Name: $Id: maia_Cal_CoxMunk.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_Cal_CoxMunk 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE  maia_Cal_CoxMunk (idbg, lambda, box, albmax)

! f90 conversion : lydie lavanant       04/06/02

! computes the maximum reflectance over sea (Cox and Munck theory).

 USE MOD_MAIA_CONST
 USE MOD_maia_types

IMPLICIT NONE

  REAL,             INTENT(in) :: lambda
  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! lat, lon, solar and satellite angles at the center of the box
  REAL,             INTENT(out) :: albmax       ! in %

  REAL, PARAMETER :: xsal = 34.3                ! Cox salinity

  REAL :: muso,musa,mun,cos2chi,coschi,sinchi
  REAL :: val
  REAL :: nr,ni,R1,WW,SSS

!=============================================[ instructions ]==========  
  if(idbg%debug) then
    write(*,*) "maia_Cal_CoxMunk"  
  endif
  ! too high or low angles are not processed. protect division by 0
  if ( box % satzen>89. .or. box % satzen<0. .or. &
       box % sunzen>89. .or. box % sunzen<0. .or. abs(box % difazi)>360. ) then
       albmax=vmanq
       return
  endif
  !if(idbg % debug) write(*,*) 'maia_Cal_CoxMunk', box % satzen, box % sunzen, box % difazi
      
  ! refraction index and extinction coefficient over sea 
  call maia_Cal_WaterIndex (lambda,xsal,nr,ni)
  !if(idbg % debug) write(*,*) 'maia_Cal_CoxMunk', nr,ni

  ! cosinus of solar and local zentih angles
  muso = cos(cdr*box % sunzen)
  musa = cos(cdr*box % satzen)

  ! default value si angles extremes 
  if( ABS(muso+musa) < 0.001 ) then
     albmax=200.
  
  else

     cos2chi = sin(cdr*box % sunzen)*sin(cdr*box % satzen)*cos(cdr*box % difazi) +muso*musa
     if(cos2chi.ge.1.)  cos2chi = 0.99999
     if(cos2chi.le.-1.) cos2chi = -0.99999
  
     mun = (muso+musa) / (sqrt(2.*(1.+cos2chi)))
     coschi = sqrt(0.5*(1.+cos2chi))
     sinchi = sqrt(0.5*(1.-cos2chi))

     ! computes the Fresnel coefficients
     call maia_Cal_Fresnel (nr,ni,coschi,sinchi,R1)
     !if(idbg % debug) write(*,*) 'maia_Cal_CoxMunk', R1, mun, cos2chi, coschi, sinchi, muso, musa

     ! computes the maximum value of the specular reflection (Munck law) 
     ! and of the associated wind (limited to 20m/s)
     WW=((1./(mun*mun)-1.)-0.003)/0.00512
     if(WW.ge.0..and.WW.le.20.) then
         val = exp(-1.)/        (4*musa*muso*(abs(mun*mun-mun*mun*mun*mun)))
     else if(WW.lt.0.) then
         WW = 0.         
         SSS = 0.003+0.00512*WW
         val = exp( (1.-1./(mun*mun))/SSS) /(4*musa*muso*mun*mun*mun*mun*SSS)
     else if(WW.gt.20.) then
         WW = 20.         
         SSS = 0.003+0.00512*WW
         val = exp( (1.-1./(mun*mun))/SSS) /(4*musa*muso*mun*mun*mun*mun*SSS)
     endif   
      
     albmax = 100.*val*R1

     ! limited to 200%
     if(albmax > 200.) albmax=200.

  endif

END SUBROUTINE  maia_Cal_CoxMunk
