! Name: $Id: maia_Cloud_Phase.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: Determines a cloud phase for confidently cloudy pixels.
!          Subroutines maia_Cloud_Phase overlap_test Cirrus_test Mixed_phase_test Opaque_ice_test Cirrus2_test temp2rad 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/01/2014   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_Cloud_Phase (idbg, box, pix, Thres_Phase, CMa, CT, soft)

! Authors       : lydie lavanant

! Determines a cloud phase for confidently cloudy pixels.
! The following assignments are made to all pixels:
!        0:  Not Executable
!        1:  Clear (from Confidently Clear pixels)
!        2:  Partly Cloudy (from Probably Clear and Cloudy pixels)
!        3:  Water Cloud
!        4:  Supercooled Water or Mixed Phase Cloud
!        5:  Opaque Ice Cloud
!        6:  Cirrus (Non-Opaque) Cloud
!        7:  Cloud Overlap 
!        8:  uncertain

! 3 software sources:
! soft=1 from Bryan A. Baum for further information email: bryan.baum@ssec.wisc.edu
! soft=2 from:
!        Pavolonis, M. J., A. K. Heidinger and T. Uttal: Daytime Global Cloud Typing
!        from AVHRR and VIIRS: Algorithm Description, Validation,and Comparisons.
!        Journal of Applied Meteorology, 2005.
!        VIIRS Cloud Mask ATBD. December 2011
! soft=3 from:
!        Pavolonis M. J.: Advances in extracting cloud composition information from 
!        spaceborne infrared radiances- a robust alternative to brightness temperatures. 
!        Part1: theory. Journal of Applied Meteorology, 2010.

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),    INTENT(in)          :: idbg          
  type( box_id ),   INTENT(in)          :: box          
  type( pix_data),INTENT(in)            :: pix  
  type( maia_Thres_Phase), INTENT(in)   :: Thres_Phase  !
  type( maia_CMa), INTENT(in)           :: CMa   
  type( maia_CT),INTENT(inout)  :: CT    
  integer,          INTENT(in)          :: soft

! local
  real          :: Reflec_37
  real          :: Solar_cste, M_Rad_37, M_Rad_108, ems_37
  logical       :: overlap, cirrus, mixed_phase, opaque_ice
  real          :: dbt87_108
  real          :: Thres87_108, Thres13         ! seuils dynamiques

!=======================================================
 
  CT % phase = 0

!  IF (CMa % Conf_clear == 3) THEN                                      !0=confident clear
  IF (CMa % Conf_value  >= 0.7 .or. CMa % Conf_clear == 3) THEN         !0=confident clear
     CT % phase = 1                                                     ! Clear
!  ELSEIF (CMa % Conf_value  >= 0.7) THEN                                       
!     CT % phase = 2                                                    ! 2=probably clear
  ELSE

     ! -----------------------------------------------------
     if(soft==1) then   ! Baum/Menzel 
          
        dbt87_108 = pix % Obs_87 - pix % Obs_108  ! [8.5-11 micron] brightness temperature difference
                 
        if (pix % Obs_108 <= 238. .or. dbt87_108 >= 0.5) then
           CT % phase = 5                                               ! flag as ice cloud
        else if (pix % Obs_108 > 238. .or. dbt87_108 <-1.0) then
           CT % phase = 3                                               ! flag as water cloud
        else if (pix % Obs_108 >238. .and. pix % Obs_108 <268. .and. dbt87_108 >= -0.25 .and. dbt87_108 < 0.5) then
           CT % phase = 4                                               ! flag as mixed-phase cloud
        else if (pix % Obs_108 > 238. .and. pix % Obs_108 < 268. .and. dbt87_108 < -0.25  .and. dbt87_108 >= -1.) then
           CT % phase = 8                                               ! flag as uncertain phase 
        else if (pix % Obs_108 > 238. .and. dbt87_108 <= -1. ) then                  
           CT % phase = 3                                               ! flag as water cloud                               
        else if (pix % Obs_108 >= 285. .and. dbt87_108 <= -0.5 ) then                  
           CT % phase = 3                                               ! flag as water cloud     
        else 
           CT % phase = 8                                               ! flag what's left as uncertain
        endif         

     ! -----------------------------------------------------
     else if (soft==2) then     ! Pavolonis 2005 + atbd vcm 

     ! calculate simple solar reflectance component to M12: Reflec_37
     Solar_cste = 10.725412
     call temp2rad (pix % Obs_37, M_Rad_37)
     call temp2rad (pix % Obs_108, M_Rad_108)
     Reflec_37 = (M_Rad_37- M_Rad_108)/(Solar_cste*cos(box %sunzen) - M_Rad_108)

     ! pseudo-emissivity
     ems_37= M_Rad_37 / M_Rad_108

        IF (pix % Obs_108 <= 233.16) THEN       ! 
           call overlap_test(idbg, box, pix, Thres_Phase, ems_37, CMa, overlap)         ! overlap test
           if( overlap) then
               CT % phase = 7                   ! Overlap
           else                         
               call Cirrus_test (idbg, box, pix, Reflec_37, ems_37, cirrus)     ! cirrus test
               if( cirrus) then
                  CT % phase = 6                ! cirrus non-opaque
               else
                  CT % phase = 5                ! opaque ice
               endif                            
           endif

        ELSE IF (pix % Obs_108 <= 253.16 ) THEN         
           call overlap_test(idbg, box, pix, Thres_Phase, ems_37, CMa, overlap)         ! overlap test
           if( overlap) then
               CT % phase = 7                   ! Overlap
           else                         
               call Cirrus_test (idbg, box, pix, Reflec_37, ems_37, cirrus)     ! cirrus test
               if( cirrus) then
                  CT % phase = 6                ! cirrus non-opaque
               else
                  call Mixed_phase_test (idbg, box, pix, mixed_phase)           ! IR and/or NIR
                  if(mixed_phase) then
                     CT % phase = 4             ! Mixed Phase
                  else
                     CT % phase = 5             ! opaque ice
                  endif
               endif
           endif

        ELSE IF (pix % Obs_108 <= 273.16 ) THEN         
           call overlap_test(idbg, box, pix, Thres_Phase, ems_37, CMa, overlap)         ! overlap test
           if( overlap) then
               CT % phase = 7                   ! Overlap
           else                         
               call Cirrus_test (idbg, box, pix, Reflec_37, ems_37, cirrus)     ! cirrus test
               if( cirrus) then
                  CT % phase = 6                ! cirrus non-opaque
               else
                  call Opaque_ice_test (idbg, box, pix, opaque_ice)             ! IR and/or NIR
                  if(opaque_ice) then
                     CT % phase = 5             ! opaque ice
                  else if(box%day) then
                        Thres87_108 = 0.7       ! seuil à calculer avec hiscrtm
                        Thres13 = 2.5
                        call Cirrus2_test (idbg, box, pix, Reflec_37, Thres87_108, Thres13, Cirrus)     ! IR and/or NIR
                        if( cirrus) then
                            CT % phase = 6      ! cirrus non-opaque
                        else
                            CT % phase = 4      ! Mixed Phase
                        endif
                  else
                            CT % phase = 4      ! Mixed Phase
                  endif
               endif
           endif

        ELSE IF (pix % Obs_108 > 273.16 ) THEN  
           call overlap_test(idbg, box, pix, Thres_Phase, ems_37, CMa, overlap)         ! overlap test
           if( overlap) then
               CT % phase = 7                   ! Overlap
           else                         
               call Cirrus_test (idbg, box, pix, Reflec_37, ems_37, cirrus)             ! cirrus test
               if( cirrus) then
                  CT % phase = 6                ! cirrus non-opaque
               else
                  Thres87_108 = 0.5
                  Thres13 = 1.
                  call Cirrus2_test (idbg, box, pix, Reflec_37, Thres87_108, Thres13, Cirrus)           ! IR and/or NIR
                  if( cirrus) then
                     CT % phase = 6             ! cirrus non-opaque
                  else
                     CT % phase = 3             ! water
                  endif
               endif
           endif
  if( CMa%Conf_clear == 3) write(*,*) 'Phase: ', box%tm, CMa % Conf_value, CT%cld_type, CT % phase
        ENDIF

     ! -----------------------------------------------------
     elseif (soft==3) then      ! Pavolonis 2010 + atbd abi cm
        write(*,*) 'the method is not implemented'
        
        !ems_M14 = (pix % Obs_87 - Rad_clr_14) / (Rad_cld_14(Tcld) - Rad_clr_14)        ! Rad_clr: radiance claire
        !ems_M15 = (pix % Obs_108 - Rad_clr_15) / (Rad_cld_15(Tcld) - Rad_clr_15)       ! Rad_cld: radiance 'opaque' au nuage
        !ems_M16 = (pix % Obs_120 - Rad_clr_16) / (Rad_cld_16(Tcld) - Rad_clr_16)

        ! effective absorption optical depth ratio:
        !beta_14_15 = log(1-ems_M14) / log(1-ems_M15)
        !beta_16_15 = log(1-ems_M16) / log(1-ems_M15)

        ! define the opaque temperature level for which the ems==.98
        !Rad_cld = Rad_clr + (pix % M - Rad_clr)/0.98
        !do i=1, rttov_nlevels
        !   if(radiances_rttov(i) == Rad_cld ) T_opaque=temp(i)
        !end do

        ! tests:
        ! 1: low surface emissivity
        ! 2: beta_16_15 opaque cloud
        ! 3: opaque cloud temperature differences                       (ne marche que avec un canal sondeur 7.4mm)
        ! 4: overall opaque cloud: combine tests 1,2 3
       
        ! 5: WV multilayer cloud detection                              (ne marche que avec un canal sondeur 7.4mm)
        ! 6: IR Window multilayer cloud detection
        ! 7: overall multilayer cloud detection: combine tests 5 and 6

        ! 8: homogeneous freezing
        ! 9: beta_14_15 and Topaque_74                                  (ne marche que avec un canal sondeur 7.4mm)
        !10: beta_14_15 and Topaque_74_LocalRadiativeCenter             (ne marche que avec un canal sondeur 7.4mm)
        !11: beta_14_15 mid-level opaque ice clouds
        !12: beta_14_15 and Topaque_74 thin ice clouds                  (ne marche que avec un canal sondeur 7.4mm)
        !13: overall ice cloud detection: combine tests 8 to 12

        !14: descriminate opaque and semi-transparent ice clouds
        !15: beta_14_15 and Topaque_15: Mixed phase
        !16: super-cooled liquid water
     ! -----------------------------------------------------
     else
        write(*,*) 'pb method pour cloud phase'
     endif
ENDIF
END SUBROUTINE maia_Cloud_Phase

!=====================================================================================
subroutine overlap_test(idbg, box, pix, Thres_Phase, ems_37, CMa, overlap)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),           INTENT(in)   :: idbg         ! 
  type( box_id ),          INTENT(in)   :: box          ! info at the center of the box
  type( pix_data),        INTENT(in)    :: pix  ! pix observations (albedo in %, Tb in K)
  type( maia_Thres_Phase), INTENT(in)   :: Thres_Phase  !
  real,                    INTENT(in)   :: Ems_37
  type( maia_CMa),         INTENT(in)   :: CMa   
  logical,                 intent(out)  :: overlap

  integer, parameter    :: nb_sunzen=8 , nb_satzen=7
  integer, parameter    :: ideb_sunzen(8)=(/0, 10, 20, 30, 40, 50, 60, 70 /)
  integer, parameter    :: ideb_satzen(7)=(/0, 10, 20, 30, 40, 50, 60 /)
  integer, parameter    :: nb_scat= 18
  integer, parameter    :: ideb_scat(18)=(/0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100, 110, 120, 130, 140, 150, 160, 170 /)

  real :: R65, R138, s_108_120, s_10
  integer :: isun,isat, iscat
  if(idbg%debug) write(*,*) "overlap_test"  
  
  overlap=.false.

  ! init thresholds
  R65=pix % Obs_06/100.
  s_108_120 = Thres_Phase% A_overlap(isun,isat)*(R65**4) + Thres_Phase% B_overlap(isun,isat)*(R65**3) + &
                Thres_Phase% C_overlap(isun,isat)*(R65**2) + Thres_Phase% D_overlap(isun,isat)*R65 + &
                Thres_Phase% E_overlap(isun,isat)

  R138=pix % Obs_13/100.    
  if(box % sea .or. CMa % snowice_fl==1) then   !for water/ice/snow
     s_10=Thres_Phase% A_wis(iscat)*(R138**4) + Thres_Phase% B_wis(iscat)*(R138**3) +&
              Thres_Phase% C_wis(iscat)*(R138**2) + Thres_Phase% D_wis(iscat)*R138 + &
              Thres_Phase% E_wis(iscat) !water/ice/snow
  else  ! for land veget
     s_10=Thres_Phase% A_veg(iscat)*(R138**4) + Thres_Phase% B_veg(iscat)*(R138**3) +&
              Thres_Phase% C_veg(iscat)*(R138**2) + Thres_Phase% D_veg(iscat)*R138 +&
              Thres_Phase% E_veg(iscat) !water/ice/snow
  endif

  if(box % day ) then   
     if(box % sea ) then        
        ! Obs_16 > threshold    : function of ReflM9 and scatt geometry
        ! Obs_16/Obs_06< 1.     : eliminates cirrus over bright backgrounds
        ! Obs_108< 280.         : eliminates water clouds in dry atmospheres
        ! Obs_06> 35.           : eliminates thin cirrus over desert
        ! Obs_108-Obs_120>thres : SWBTD required to confirm overlap cloud fct of M5  
        ! Obs_108 <270.         : eliminates partially filled water clouds that may also have ReflM9> 35.
        if((box %lat>-50. .and. box %lat< 50. .and. pix % Obs_13> 2.5 .and. pix % Obs_13 < 40.) &
            .or.                                   (pix % Obs_13> 1.0 .and. pix % Obs_13 < 40.)) then
           if(pix % Obs_16 >s_10 .and. pix % Obs_16/pix % Obs_06< 1. .and.pix % Obs_108< 280.) then
              if(pix % Obs_13 >8.) then
                 overlap=.true.
              else                      
                 if( pix % Obs_06>35. .and. (pix % Obs_108-pix % Obs_120)> s_108_120 .and. pix % Obs_108<270.) then             
                    overlap=.true.               
                 endif 
              endif 
           endif 
        elseif((box %lat>-50. .and. box %lat< 50. .and. pix % Obs_16>10.).or. pix % Obs_16> 30.) then
           overlap=.true.                
        endif 

     elseif(box % land .or. box % coast .or. CMa % snowice_fl==1 ) then
        if( (box %lat>-40. .and. box %lat< 40. .and. pix % Obs_13> 2.7 .and. pix % Obs_13 < 40.) .or. &
            pix % Obs_13> 1.0 .and. pix % Obs_13 < 40.) then
           if(pix % Obs_16 >s_10 .and. pix % Obs_16/pix % Obs_06< 1. .and. pix % Obs_108< 280.) then                    
              if( pix % Obs_13 >12. .and. pix % Obs_06>35. .and. (pix % Obs_108-pix % Obs_120) > s_108_120) then
                 overlap=.true.
              endif              
           elseif( (box%lat>-50. .and. box%lat< 50. .and. pix % Obs_16> 10.) .or. pix % Obs_16> 30.) then
              if(pix % Obs_06>35. .and. (pix % Obs_108-pix % Obs_120) > s_108_120 .and. pix % Obs_108>270.)then
                 overlap=.true.          
              endif 
           endif 
        endif 

     elseif(box % desert ) then
        if(pix % Obs_04 >50. .and. pix%Obs_06>35..and. (pix%Obs_108-pix%Obs_120)> s_108_120 .and.pix%Obs_108< 270.) then
           if((box %lat>-50. .and. box %lat< 50. .and. pix % Obs_16>10.).or. pix % Obs_16> 30. ) then
              overlap=.true.             
           endif 
        endif
     
     else
        if(box % tm /=4) write(*,*) 'PB in cloud_phase' , box % tm
     endif 

  else !night
     if( box % sea .and. box % lat >=-30 .and. box % lat <=30) then
        if( (pix % Obs_108-pix % Obs_120)>0.58  .and.  (pix % Obs_108-pix % Obs_120) < 2.5) then
           if( Ems_37>1. .and. Ems_37> 2.6 ) then
              overlap=.true.             
           endif 
        endif 

     elseif(box % sea .and. (box % lat <-30 .or. box % lat >30)) then
        if( (pix % Obs_108-pix % Obs_120)>0.58  .and.  (pix % Obs_108-pix % Obs_120) < 2.0) then
           if( Ems_37>1. .and. Ems_37> 2.0 ) then
              overlap=.true.             
           endif 
        endif 

!     elseif ( (box % land .or. CMa % snowice_fl==1) .and. .not. box%desert ) then
     elseif (  .not. box%desert .and. .not. box%snow_always ) then
        if( (pix % Obs_108-pix % Obs_120)>0.58  .and.  (pix % Obs_108-pix % Obs_120) < 2.0) then
           if( Ems_37>1. .and. Ems_37> 2.0 ) then
              overlap=.true.             
           endif 
        endif 
     else
       if(box % tm /=3 .and. box % tm /=4) write(*,*) 'PB in cloud_phase' , &
          box %tm, box %lat, box %lon, CMa %snowice_fl, box%sea, box%land, box%coast, box%desert
     endif 

  endif

end subroutine  overlap_test

!=====================================================================================
subroutine Cirrus_test (idbg, box, pix, Reflec_37, Ems_37, cirrus)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),     INTENT(in) :: idbg         ! 
  type( box_id ),    INTENT(in) :: box          ! info at the center of the box
  type( pix_data), INTENT(in) :: pix    ! pix observations (albedo in %, Tb in K)
  real, INTENT(in)              :: Reflec_37, Ems_37
  logical, intent(out)          :: cirrus

  real  :: Ci_108_120, Ci_Day_108_120, Ci_Night_15_16   ! a definir
  
if(idbg%debug) write(*,*) "Cirrus_test"

  Ci_Night_15_16=1.5

  cirrus = .false.
  if(pix % Obs_108 < 265.) then ! a corriger avec une fct de satzen et M15
     Ci_108_120=0.7
  else
     Ci_108_120=1.5
  endif

  if(box % day .and. box%desert ) then
     IF (( pix % Obs_13 > 2.5 .AND. (pix % Obs_108-pix % Obs_120) > Ci_Day_108_120 .AND. Reflec_37 < 40.) .OR.&
          (pix % Obs_06 < 40. .AND. pix % Obs_13/pix % Obs_06 > 0.17)) THEN
        cirrus = .true.
     ENDIF
  else if(box % day ) then
     IF (( pix % Obs_13 > 2.5 .AND. (pix % Obs_108-pix % Obs_120) > Ci_Day_108_120 .AND. Reflec_37 < 15.) .OR.&
          (pix % Obs_06 < 40. .AND. pix % Obs_13/pix % Obs_06 > 0.17)) THEN
        cirrus = .true.
     ENDIF
  else
     IF (( (pix % Obs_108-pix % Obs_120) > Ci_Night_15_16 .AND. Ems_37 > 1.2) .OR. Ems_37 > 1.4) THEN
        cirrus = .true.
     ENDIF
  endif

end subroutine  Cirrus_test

!=====================================================================================
subroutine Mixed_phase_test (idbg, box, pix, Mixed_phase)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),     INTENT(in) :: idbg         
  type( box_id ),    INTENT(in) :: box          
  type( pix_data), INTENT(in)   :: pix  ! pix observations (albedo in %, Tb in K)
  logical, intent(out)          :: Mixed_phase

  real  :: Mixed108_120 

  if(idbg%debug) write(*,*) "Mixed_phase_test"

  Mixed108_120 = 0.5 +2.
  Mixed_phase = .false.
  if(box % day ) then
     IF ( ((pix % Obs_87-pix % Obs_108) < Mixed108_120 .AND. pix % Obs_108 > 238.) .OR. &
          pix % Obs_16/pix % Obs_06 > 1) THEN
         Mixed_phase = .true.
     ENDIF
  else  !night
     IF ( (pix % Obs_87-pix % Obs_108) < Mixed108_120 .AND. pix % Obs_108 > 238. ) THEN
         Mixed_phase = .true.
     ENDIF
  endif

end subroutine Mixed_phase_test

!=====================================================================================
subroutine  Opaque_ice_test (idbg, box, pix, opaque_ice)        
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),     INTENT(in) :: idbg         ! 
  type( box_id ),    INTENT(in) :: box          ! info at the center of the box
  type( pix_data), INTENT(in)   :: pix  ! pix observations (albedo in %, Tb in K)
  logical, intent(out)          :: opaque_ice

  real :: Mixed108_120
  if(idbg%debug) write(*,*) "Opaque_ice_test"

  Mixed108_120 = 0.5
  opaque_ice = .false.
  if(box % day ) then
     IF ((pix % Obs_87-pix % Obs_108) < Mixed108_120 .AND. pix % Obs_108 <263. .AND. pix % Obs_16/pix % Obs_06 < 1) THEN
         opaque_ice = .true.
     ENDIF
  else  !night
     IF ((pix % Obs_87-pix % Obs_108) < Mixed108_120 .AND. pix % Obs_108 <263. ) THEN
         opaque_ice = .true.
     ENDIF
  endif

end subroutine  Opaque_ice_test

!=====================================================================================
subroutine Cirrus2_test (idbg, box, pix, Reflec_37, Thres87_108, Thres13, Cirrus)
!=====================================================================================

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),     INTENT(in) :: idbg         
  type( box_id ),    INTENT(in) :: box          
  type( pix_data), INTENT(in)   :: pix  ! pix observations (albedo in %, Tb in K)
  real, INTENT(in)              :: Reflec_37, Thres87_108, Thres13
  logical, intent(out)          :: cirrus

  if(idbg%debug) write(*,*) "Cirrus2_test"

  cirrus = .false.
  IF ((pix % Obs_87-pix % Obs_108) >= Thres87_108 .AND. pix % Obs_13 >Thres13 ) THEN
     if((box%desert .and. Reflec_37<40.) .or. Reflec_37<15.) then
        cirrus = .true.
     endif  
  ENDIF
                                                        
end subroutine Cirrus2_test

subroutine temp2rad (temp, rad)

implicit none                                                     

  real, parameter :: c1=1.1910659E-05, c2=1.438833
  real          :: b1, b2, val_can
  real          :: rad, temp
  
  val_can=2666.67       ! canal 3.7
  
  b1=C1*val_can**3
  b2=C2*val_can
  
  if(temp > 0.) then
        rad = b1 / ( Exp( b2/temp  ) - 1. )
  else
     rad=-9999.
  endif
  
end subroutine temp2rad
