! Name: $Id: maia_Fill_Output.F90 548 2017-08-30 09:28:19Z frna $
!
! Purpose: 
!          Subroutines maia_Fill_Output 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.3    23/12/2016   Lydie Lavanant enhancement for quality flags
!  1.4    05/05/2017   Pascale Roquet add T2m to maia_par (31) 
!  1.5    23/08/2017   Pascale Roquet use a structure for maia output

SUBROUTINE maia_Fill_Output (idbg, box, pix_id, pix, CMa, CT, CH, maia_par)

!-----------------------------------------------------------------------
! Authors       : lydie lavanant
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),      INTENT(in) :: idbg       ! 
  type( box_id ),     INTENT(in) :: box        ! info at the center of the box
  type( pix_info ),   INTENT(in) :: pix_id   ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),    INTENT(in) :: pix  ! pix observations (albedo in %, Tb in K)
  type( maia_CMa),    INTENT(in) :: CMa ! 
  type( maia_CT),     INTENT(inout) :: CT               ! 
  type( maia_CH),     INTENT(in) :: CH          ! 

  type(maia_param), INTENT(out)    :: maia_par          ! mask outputs

!-----------------------------------------------------------------------


  ! 1  - Cloud Mask summary     0= confident clear / 1= cloudy / 3= clear over snow / 
  !                             4= clear over ice / 5= aerosol_dust_ash_fire
  if(CMa % conf_clear==3 ) maia_par%Cloud_Mask = 0
  if(CMa % conf_clear<3  ) maia_par%Cloud_Mask = 1
  if(CMa % snowice_fl==1 .and. CMa % conf_clear==3) then
     if(box%sea) then
        maia_par%Cloud_Mask = 4
     else
        maia_par%Cloud_Mask = 3
     endif
  endif
  if(maia_par%Cloud_Mask ==0 .and. pix_id%tm==4) maia_par%Cloud_Mask = 3        ! put to snow if clear and snowalways
  if(box%sea) then
     if(maia_par%Cloud_Mask ==0 .and.pix_id% snowice_surf==1) maia_par%Cloud_Mask = 4   ! put to snow if clear and snow from forecast
  endif

  ! 2  - Cloud Mask Quality     3=high / 2=medium / 1=poor / 0=bad
  maia_par%Cloud_Mask_Qual = CMa % qual
  !if (pix %Qual_tot==2) maia_par%Cloud_Mask_Qual =1

  ! 3  - Cloud Mask Confidence before Cloud Adjacency
  !      3=confident clear / 2=probably clear / 1=probably cloudy / 0=confident cloudy
  maia_par%Cloud_Mask_Conf = CMa % conf_clear
                                        
  ! 4  - Cloud Mask Adjacency from evaluation of surrounding pixels
  !      0=confident clear / 1=probably clear / 2=probably cloudy / 3=confident cloudy                                  
  maia_par%Cloud_Mask_Ad = CMa % Adjacent_conf_clear

  ! 5  - Sea: Split-Window Ts (SST)  for confident clear
  !      Land: LST using RTE from NWP profile and emissivity atlas for confident clear
  if( maia_par%Cloud_Mask == 0) then
     if( pix_id % tm == 0 .and. box % sstmoy_clim>vmanq ) then  
         call maia_SST (idbg, pix_id, pix, box % sstmoy_clim, maia_par%surf_temp)
     elseif( box % land ) then                          !land
!         maia_par%surf_temp     = maia_LST (pix, box)
     endif
  endif

  ! 7  - Box bg Tskin source    0= from climatology / 1= from forecast 
  ! 8  - Box Bg Tskin used              (K)
  if( box % tm == 0 .and. box% sst_clim>vmanq ) then
      maia_par%box_bg_tskin_src = 0
      maia_par%Box_Bg_Tskin_used = box% sst_clim
  elseif( box % tm == 0 .and. box% sst_clim==vmanq .and. box% tsurf>vmanq) then
      maia_par%box_bg_tskin_src = 1
      maia_par%Box_Bg_Tskin_used = box% tsurf
  elseif( box % tm /= 0 .and. box% tsurf>vmanq) then
      maia_par%box_bg_tskin_src = 1
      maia_par%Box_Bg_Tskin_used = box% tsurf
  else
      maia_par%box_bg_tskin_src = imanq
      maia_par%Box_Bg_Tskin_used = vmanq
  endif      

  ! 9  - Box Bg CWV source      0= from climatology / 1= from forecast
  ! 10 - Box Bg CWV used        (g/cm2)
  if( box% cwv_prev>vmanq ) then
      maia_par%box_bg_wv_src  = 1
      maia_par%Box_Bg_WV_Content_used = box% cwv_prev
  else
      maia_par%box_bg_wv_src  = 0
      maia_par%Box_Bg_WV_Content_used = vmanq
  endif      

  ! 11 - Box surface altitude (m) 
  maia_par%alt_surface  = box% alt

  ! 12 - Box atlas surface type 0= sea / 1= mixed / 2= land / 3= desert / 4= snow_always
  maia_par%Box_atlas_surface_type  = box% tm

  ! 13 - Box day_time           0= Night / 1= Twilight / 2= Day / 3= Sunglint
  maia_par%box_daytime  = box% day_time

  ! 14 - Box specular reflexion 0= no / 1= yes
  maia_par%specular_reflexion  = box % dcj

  !15 - pixel surface altitude (m)
   maia_par%alt_surface  = pix_id % alt
 
  !16 - pixel surface type     0= sea / 1= mixed / 2= land / 3= desert / 4= snow_always
   maia_par%pixel_surface_type  = pix_id % tm

  ! 17 - pixel snow/ice         0= no / 1= yes 
  !maia_par%snowice_fl  = CMa % snowice_fl ! not used in write

  ! 18 - cloud type. add cumuliform values
  CT % class= 20                                                ! undefined
  if(CMa % conf_clear>1 .and. pix_id % tm>0. ) CT % class= 1    ! clear or probably clear land
  if(CMa % conf_clear>1 .and. pix_id % tm == 0) CT % class= 2   ! clear or probably clear sea
  if(CMa % conf_clear>1 .and. pix_id % tm>0. ) CT % class= 1    ! clear or probably clear coast 
  
  if(maia_par%Cloud_Mask == 3.) then
     if( pix_id%tm==0) then
        CT % class= 4   ! snow sea
     else
        CT % class= 3   ! snow land
     endif
  endif
  
  if(maia_par%Cloud_Mask == 4.) then
     if( pix_id%tm==2) then
        CT % class= 3   ! snow land
     else
        CT % class= 4   ! snow sea
     endif
  endif
  if(CT % class ==1 .and. pix_id%tm==4) CT % class = 3  ! put to snow if clear and snowalways
  if(box%sea) then
     if(CT % class ==1 .and.pix_id% snowice_surf==1) CT % class = 4     ! put to snow if clear and snow from forecast
  endif
  
  if(maia_par%Cloud_Mask == 1.  ) then                  ! for probablycloudy and cloudy situations (cld_type==imanq if ProbablyClear)
    if(CT%cld_type== 1) CT % class= 6           ! very low and stratiform
    if(CT%cld_type== 2) CT % class= 8           ! low and stratiform
    if(CT%cld_type== 3) CT % class= 10          ! medium and stratiform
    if(CT%cld_type== 4) CT % class= 12          ! high opaque and stratiform
    if(CT%cld_type== 5) CT % class= 14          ! very high opaque and stratiform
    if(CT%cld_type== 6) CT % class= 17          ! high semitransparent thick 
    if(CT%cld_type== 7) CT % class= 16          ! high semitransparent meanly
    if(CT%cld_type== 8) CT % class= 15          ! high semitransparent thin
    if(CT%cld_type== 9) CT % class= 18          ! high semitransparent above
    if(CT%cld_type== 10) CT % class=19          ! fractional
  endif
  maia_par%cloud_type = CT % class

  ! 19 - cloud phase            
  maia_par%cloud_phase = CT % Phase

  ! 20 - thin cirrus detected   0= no /1= yes
  !maia_par%cirrus_detected = CT % thin_cirrus ! not used in write

  ! 21 - cloud shadow detected  0= no /1= yes
  !maia_par%cloud_shadow = CMa % shadow_fl

  ! 22 - cloud opacity          0= no /1= yes
  maia_par%cloud_opacity =CT % black

  ! 23 - cloud top temperature  (K) for confident cloudy and opaque
   maia_par%cloud_top_temp =CH % Tcld

  ! 24 - cloud top pressure     (hPa) for confident cloudy and opaque using NWP profile
   maia_par%cloud_top_press =CH % Pcld

  ! 25 - Heavy aerosol          0= no / 1= yes
  !maia_par%heavy_aerosol =CT % Heavy_Aerosol

  ! 26 - Dust                   0= no / 1= yes
  !maia_par%dust =CT % Dust_Candidate

  ! 27 - Volcanic Ash           0= no / 1= yes
  !maia_par%volcanic_ash =CT % Dust_VolcanicAsh

  ! 28 - Smoke                  0= no / 1= yes
  !maia_par%Smoke_Candidate =CT % Smoke_Candidate

  ! 29 - Fire                   0= no / 1= yes
  !maia_parActivefile_fl =CT % Activefire_fl

  ! 30 - Moderate pixel texture 0= no / 1= yes
  !maia_par%MNeihbors =CT % MNeighbors

  ! 31 - T2m prev 
  maia_par%temp_2m =  box% t2m_prev
  
  if( idbg%info ) then
      write(*,*) '  -----------------------------------------------------------------------'
      write(*,*) '  MAIA_PAR= '
      write(*,*) ' Cloud_mask , conf, qual: ', maia_par%cloud_mask, maia_par%Cloud_Mask_Conf, maia_par%Cloud_Mask_Qual
      write(*,*) ' Cloud_type : ', maia_par%cloud_type
      write(*,*) ' day_time : ', maia_par%box_daytime
      write(*,*) ' Box_Bg_Tskin_used  :', maia_par%Box_Bg_Tskin_used

  endif

END SUBROUTINE maia_Fill_Output
