! Name: $Id: maia_Flag_Dust.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: Determines a dust/sand flag 
!          Subroutines maia_Flag_Dust 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/01/2014   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
subroutine maia_Flag_Dust (idbg, box, pix, pix_id, thres, CT, CMa)

! Authors       : lydie lavanant

! Determines a dust/sand flag transported out of deserts over continental and oceanic surfaces (North Africa and adjacent seas)
 
! The night/sea test is based on the Sahara Dust Index using M12, M15, M16 (Merchant et al., 2006)
! The daytime tests are based on the M1/M5 ratio and M5 texture with adjacent M pixels and Bt differences M12-M15, M16-M15, M14-M15
! not applied for night/land

 USE MOD_MAIA_CONST
 USE mod_maia_types
IMPLICIT NONE

  type( debug )                         :: idbg         ! 
  type( box_id ),   INTENT(inout)       :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( pix_info)                       :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data)                       :: pix          ! viirs observations (albedo in %, Tb in K)
  type( maia_thres), INTENT(in)         :: thres        ! 
  integer                               :: CT
  type( maia_CMa),   INTENT(inout)      :: CMa          ! 

  real    :: SaharaDustIndex, s_87108, s_108Min, s_108MinAccurate
  real    :: s_DustLD_06Min, s_DustLD_06Max, s_DustLD_87108, rinter, s_DustSD_108min, s_DustSD_06Max
  real    :: s_DustSD_06Thick, s_DustSD_06Min, s_DustSD_16Min,s_DustSD_16MinAccurate
  real    :: s_DustSD_06MaxAccurate, s_DustSD_120108Min, s_DustSD_87108Min, s_DustSD_87108MinThick
  real    :: ndvi


  if( idbg % debug )  write(*,*) "maia_Flag_Dust"
  CMa % dust_fl=0     
         
  ! LAND and DAY
  ! -----------
  if(pix_id%land .and. pix_id%day) then
  
     ndvi = (pix % Obs_08-pix % Obs_06)/(pix % Obs_08 + pix % Obs_06)
     s_DustLD_06Min = thres % v_CMa_06_land + Thr_DustLD_offsetmin * box%cosol
     s_DustLD_06Max = thres % v_CMa_06_land + Thr_DustLD_offsetmax * box%cosol
  
     s_DustLD_87108 = Thr_DustLD_87108_MinAccurate
     rinter = Thr_DustLD_87108_offset06+ Thr_DustLD_87108_slope06 * pix%Obs_06
     if(s_DustLD_87108 > rinter) s_DustLD_87108 = rinter
          
     if( pix%Obs_108  > Thr_DustLD_min108         .and. pix%Obs_108   < Thr_DustLD_max108 .and. &
         pix%Obs_06   > s_DustLD_06Min            .and. pix%Obs_06    < s_DustLD_06Max    .and. &
         pix_id%sd_06 < Thr_DustLD_sd06*box%cosol .and. pix_id%sd_108 < Thr_DustLD_sd108)   then
         
         if( ( pix%Obs_37-pix%Obs_108 > Thr_DustLD_37108_1 .and. pix%Obs_120-pix%Obs_108 > Thr_DustLD_120108_1 ) .or.           &! from low clouds
             ( pix%Obs_37-pix%Obs_108 > Thr_DustLD_37108_2 .and. pix%Obs_120-pix%Obs_108 > Thr_DustLD_120108_2 ) ) then
               CMa % dust_fl=3 
            
         elseif( pix%Obs_120-pix%Obs_108 > Thr_DustLD_120108min .and.  &
             ( ( pix%Obs_87-pix%Obs_108  > s_DustLD_87108   .and. pix%Obs_06 < pix%Obs_16 * Thr_DustLD_0616maxaccurate ) .or.   & !from ice clouds
               ( pix%Obs_87-pix%Obs_108  > Thr_DustLD_87108 .and. pix%Obs_06 < pix%Obs_16 * Thr_DustLD_0616 ) )  ) then
               CMa % dust_fl=2 

         elseif( ndvi > Thr_DustLD_ndvi          .and. (CT ==19.or.CT==6.or.CT==8) .and.                                        &
                  pix_id%sd_M06 < Thr_DustLD_sdM06  .and. pix%Obs_108-pix%Obs_120 < Thr_DustLD_108120 ) then
               CMa % dust_fl=1 
         endif
     endif 
  endif

  ! SEA and DAY
  ! -----------
  if(pix_id%sea .and. pix_id%day) then
  
     s_DustSD_108min        = box%sst_clim + Thr_DustSD_108_offset + Thr_DustSD_108_slope*(box%satsec-1)
     s_DustSD_120108Min     = -thres % v_CMa_108_120_sea  + Thr_DustSD_120108_offset
     s_DustSD_87108MinThick = Thr_DustSD_87108_MinThick + Thr_DustSD_87108_slope *(box%satsec-1)
     s_DustSD_87108Min      = Thr_DustSD_87108_minRough + Thr_DustSD_87108_slope *(box%satsec-1)
     s_DustSD_06Max         = thres % v_CMa_06_sea + Thr_DustSD_06_offsetMax   * box%cosol
     s_DustSD_06Thick       = thres % v_CMa_06_sea + Thr_DustSD_06_offsetThick * box%cosol
     s_DustSD_06Min         = thres % v_CMa_06_sea + Thr_DustSD_06_offsetMin * box%cosol
     s_DustSD_06MaxAccurate = thres % v_CMa_06_sea + Thr_DustSD_06MaxAccurate * box%cosol       
     s_DustSD_16Min         = thres % v_CMa_16_sea + Thr_DustSD_16_offset * box%cosol      
     s_DustSD_16MinAccurate = thres % v_CMa_16_sea + Thr_DustSD_16_offsetAccurate * box%cosol    
     
     ! separation from clouds
     if( CMa%conf_clear<2               .and. CMa%thin_cirrus==0 .and.                                  &
          pix_id%sd_M06 < Thr_DustSD_sdM06 .and. pix%Obs_108-pix%Obs_120<Thr_DustSD_108120) then
         
             if(pix%Obs_04/pix%Obs_06 <Thr_DustSD_0406) then    
             if (pix_id%sd_108 < Thr_DustSD_sd108max  .and. pix%Obs_108>s_DustSD_108min .and.           &       
             pix%Obs_87-pix%Obs_108>s_DustSD_87108Min .and.                                             &
             
             ( (pix%Obs_06 <s_DustSD_06Max .and. pix_id%sd_06<Thr_DustSD_sd06Max*box%cosol .and.        &       
                pix_id%sd_08<Thr_DustSD_sd08Max* box%cosol .and.                                        &       
                pix_id%sd_10837<Thr_DustSD_sd10837Max)                                                  &       
               .or.                                                                                     &
               (pix%Obs_06>s_DustSD_06Thick .and. pix%Obs_87-pix%Obs_108>s_DustSD_87108MinThick         &       
                .and. pix_id%sd_06 < Thr_DustSD_sd06MaxThick * box%cosol)                               &       
               .or.                                                                                     &
               (pix%Obs_06<s_DustSD_06MaxAccurate .and.pix%Obs_06>=s_DustSD_06Max .and.                 &       
                pix%Obs_87-pix%Obs_108>Thr_DustSD_87108MinAccurate                .and.                 &       
                pix_id%sd_06<Thr_DustSD_sd06Max * box%cosol                       .and.                 &       
                pix_id%sd_08<Thr_DustSD_sd08Max * box%cosol .and.pix_id%sd_10837<Thr_DustSD_sd10837Max) &
             ) )CMa % dust_fl=1 
             endif
     endif
     ! separation from free water surface  (not done over seaice)
     if( CMa%conf_clear>=2              .and. CT<=2 .and.                                               &
          pix_id%sd_M06 < Thr_DustSD_sdM06 .and. pix%Obs_108-pix%Obs_120<Thr_DustSD_108120) then
     
         if(pix%Obs_04/pix%Obs_06 <  Thr_DustSD_0406 .and.                                               &
            pix%Obs_04            > (Thr_DustSD_04 *box%cosol + Thr_DustSD_04Slope*(box%cos_scat_ang-Thr_DustSD_04Offset))) then
                
             if((pix%Obs_16>Thr_DustSD_1606Min*pix%Obs_06 .and. pix%Obs_16>s_DustSD_16Min .and.         &
                 pix%Obs_120-pix%Obs_108>s_DustSD_120108Min)                                            &
               .or.                                                                                     &
               (pix%Obs_16>Thr_DustSD_1606Min*pix%Obs_06     .and.                                      &
                pix_id%sd_10837<Thr_DustSD_sd10837MaxCumulus .and.                                      &   
                pix%Obs_120-pix%Obs_108>Thr_DustSD_120108Min .and.pix%Obs_16>s_DustSD_16MinAccurate)    &
                .or.                                                                                    &
                (pix%Obs_16>Thr_DustSD_1606MinAccurate*pix%Obs_06 .and.pix%Obs_16>s_DustSD_16Min)       &
               .or.                                                                                     &
                (pix%Obs_06>s_DustSD_06Min .and. pix%Obs_120-pix%Obs_108>s_DustSD_120108Min)            &
             ) CMa % dust_fl=2 
             endif
         endif 
    endif
   
  ! SEA and NIGHT
  ! -------------
   if(pix_id%sea .and. pix_id%night) then
   
         SaharaDustIndex = Thr_DustSN_sdi_offset + &
                           Thr_DustSN_sdi_slope_3787*(pix%Obs_37-pix%Obs_87) + &
                           Thr_DustSN_sdi_slope_108120*(pix%Obs_108-pix%Obs_120)
         s_87108          = Thr_DustSN_87108Min + Thr_DustSN_87108_slope * (pix_id%satsec-1)
         s_108Min         = box%sst_clim + Thr_DustSN_108_offset + Thr_DustSN_108_slope*(pix_id%satsec-1)
         s_108MinAccurate = box%sst_clim + Thr_DustSN_108_offset_accurate + Thr_DustSN_108_slope*(pix_id%satsec-1)
         
         if( pix%Obs_120-pix%Obs_108 > Thr_DustSN_120108Min .and. pix%Obs_87-pix%Obs_108 > s_87108        .and. & 
             pix_id%sd_108           < Thr_DustSN_sd108Max  .and. pix_id%sd_10837 < Thr_DustSN_sd10837Max .and. &      
             ((SaharaDustIndex > Thr_DustSN_sdiMin          .and. pix%Obs_108 > s_108Min )  .or.                &
              (SaharaDustIndex > Thr_DustSN_sdiMinAccurate  .and. pix%Obs_108 > s_108MinAccurate))  )then         
               CMa % dust_fl=1 
         endif
   endif

end subroutine maia_Flag_Dust
