! Name: $Id: maia_Flag_ThinCirrus.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: Determines a thin cirrus flag
!          Subroutines maia_Flag_ThinCirrus 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/01/2014   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_Flag_ThinCirrus (idbg, box, pix_id, pix, CMa, Thin_Cirrus_flag)

! Authors       : lydie lavanant

! Determines a thin cirrus flag
! The tests consist of a brightness temperature difference threshold test in M15 – M16 at night 
! and a reflectance threshold tests using band 13 in the daytime.

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),    INTENT(in)  :: idbg         ! 
  type( box_id ),   INTENT(in)  :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( pix_info),  INTENT(in)  :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data),  INTENT(in)  :: pix          ! pix observations (albedo in %, Tb in K)
  type( maia_CMa), INTENT(in)   :: CMa  ! 
  integer,          INTENT(out) :: Thin_Cirrus_flag

! local
  real  :: low_cwv, high_cwv, seuil
  real  :: btd

!=======================================================
  low_cwv  = Thr_ThinCi_LowCwvCutoff_13  * cos (cdr * box % satzen)
  high_cwv = Thr_ThinCi_HighCwvCutoff_13 * cos (cdr * box % satzen)

  Thin_Cirrus_flag=0    ! NO CLOUD

  seuil  = 0.0

  IF (pix_id % night) THEN

     IF (pix % Qual_108 .and. pix % Qual_120 ) THEN
        ! Premiere approche: valeurs fixes à affiner en fonction de satzen avec hiscrtm en fct du COD        
        btd= pix % Obs_108 - pix % Obs_120
        IF ( btd > CMa_SN_108_120-0.25 .and. btd<CMa_SN_108_120) THEN
                Thin_Cirrus_flag=1      !CLOUD
        ELSE
                Thin_Cirrus_flag=0      !NO_CLOUD
        ENDIF
     ENDIF
  ELSE  ! Daytime

     IF (pix % Qual_13 .AND. box % cwv > low_cwv) THEN
        IF (CMa % snowice_fl ==1) THEN
           seuil  = Thr_ThinCiSID_13
        ELSE IF (pix_id % sea) THEN
           seuil  = Thr_ThinCiSD_13
        ELSE IF (pix_id % desert) THEN
           seuil  = Thr_ThinCiDD_13
        ELSE IF (pix_id % coast) THEN
           IF ( box % cwv > high_cwv ) THEN
              seuil = Thr_ThinCiCD_13 
           ELSE
              seuil = Thr_ThinCiCD_13 + (Thr_ThinCiDD_13-Thr_ThinCiCD_13)*(high_cwv - box%cwv)/(high_cwv - low_cwv)
           ENDIF
        ELSE 
           IF (box % cwv >= high_cwv ) THEN
              seuil = Thr_ThinCiLD_13
           ELSE
              seuil = Thr_ThinCiLD_13 + (Thr_ThinCiDD_13-Thr_ThinCiLD_13)* (high_cwv - box%cwv)/(high_cwv-low_cwv)
           ENDIF
        ENDIF
        seuil  = box%cosol * seuil
        IF (pix % Obs_13 >= seuil .AND. pix % Obs_13 < seuil+3.) THEN
           Thin_Cirrus_flag=1   !CLOUD
        ELSE
           Thin_Cirrus_flag=0   !NO_CLOUD
        ENDIF
        if(idbg % debug) then
           write(*,'(a15,i5,f10.2,a3,f10.2,a3,f10.2)') 'maia_ThinCirrus: ',CMa%snowice_fl, seuil,'<', pix%Obs_13,'<',seuil+3
        endif
     ENDIF
  ENDIF

END SUBROUTINE maia_Flag_ThinCirrus

          
