! Name: $Id: maia_GetThres_CMa_Land.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetThres_CMaLand 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetThres_CMaLand (idbg, box, tabland, thres)

! Authors       : lydie lavanant

!-----------------------------------------------------------------------
! to compute the IR thresholds used over land 
!       function of satsec_loc, wv, tsurf, alb
!       indsec from 1 to 16
!       satsec_loc : secant of local zenith angle
!-----------------------------------------------------------------------
! tab108_120_ln=        t108t120_veget_+3
! tab108_120_ld=        t108t120_veget_-10

! tabtsurf_ln_veg=      tsurft108_veget_+5 ! tabtsurf_*_des: 4k offset add in routine to veg
! tabtsurf_ld_veg=      tsurft108_veget_-5 ! tabtsurf_*_des: 4k offset add in routine to veg
! tabtsurf_ln_des=      tsurft108_veget_+5
! tabtsurf_ld_des=      tsurft108_veget_-5

! tab108_37_l_cold_veg=         t108t38_veget_+5
! tab108_37_l_warm_veg=         t108t38_veget_+5
! tab108_37_l_cold_des=         t108t38_desert_+5
! tab108_37_l_warm_des=         t108t38_desert_+5

! tab37_108_l_cold_veg=         t38t108_veget_+3
! tab37_108_l_warm_veg=         t38t108_veget_+0 
! tab37_108_l_cold_des=         t38t108_veget_+3 
! tab37_108_l_warm_des=         t38t108_veget_+0

! tab87_108_ln_veg      t087t108_veget_+5_night
! tab87_108_ld_veg      t087t108_veget_-5_day
! tab87_108_ln_des      t087t108_desert_+5_night
! tab87_108_ld_des      t087t108_desert_-5_day

! tab87_37_l_cold_veg   t087t038_veget_+5
! tab87_37_l_warm_veg   t087t038_veget_+5
! tab87_37_l_cold_des   t087t038_veget_+5
! tab87_37_l_warm_des   t087t038_veget_+5

! tab37_40_l_cold_veg   t038t040_veget_+0
! tab37_40_l_warm_veg   t038t040_veget_+0
! tab37_40_l_cold_des   t038t040_veget_+0
! tab37_40_l_warm_des   t038t040_veget_+0
!-----------------------------------------------------------------------

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( debug ),                INTENT(in) :: idbg      ! 
  type( box_id ),               INTENT(in) :: box       ! lat, lon, solar and sat angles at the center of the box
  type( maia_ThresTables_land),INTENT(in) :: tabland   ! tabulated threshold tables (nb_wv,nb_secant)
  type( maia_thres ),           INTENT(inout):: thres   ! dynamic thresholds in (deg*100)

! local:
  REAL              :: satsec_loc                                       ! value with borns 
  REAL, PARAMETER   :: max_noise = 0.3 
  REAL, PARAMETER   :: sec_maxval=2.5
  REAL, PARAMETER   :: sec_deb=1.0
  REAL, PARAMETER   :: sec_step=0.25
  REAL, PARAMETER   :: wv_deb=2.5               ! in kg/m2
  REAL, PARAMETER   :: wv_step=5.               ! in kg/m2

  INTEGER :: iw,isec 
  REAL    :: difsec,difw,diftsurf,difalb,difsol, difalt
  REAL    :: seuil,seuilc,seuilw,seuilveg,seuildes,seuild,seuiln
  REAL    :: tsurf, alb     

! functions called
  REAL :: maia_Interp_InLut
  
!=============================================[ instructions ]==========
  satsec_loc=box % satsec
  tsurf=box % tsurf
  alb= box % albedo_clim

! Preparation for interpolation in secant 
  if (satsec_loc  >  sec_maxval) satsec_loc=sec_maxval
  isec= 1 + (satsec_loc-sec_deb)/sec_step
  if (isec  >=  sec_nb) isec=sec_nb-1
  if (isec  <=  1) isec=1
  difsec = ( satsec_loc - sec_deb - (isec-1)*sec_step ) / sec_step  

! Preparation for interpolation in water vapor  
  iw= 1 + (box % cwv-wv_deb)/wv_step
  if (iw >= wv_nb) iw=wv_nb-1
  if (iw <= 1) iw=1
  difw = (box % cwv- wv_deb - (iw-1)*wv_step)/ wv_step

! Preparation for interpolation in cold waters 
  diftsurf=(tsurf - cst_tempcold) / (cst_tempwarm-cst_tempcold) 

! Preparation for interpolation in climatological reflectance 
  difalb=(alb - cst_veg_max_alb) / (cst_des_min_alb - cst_veg_max_alb) 

! Preparation for interpolation in solar zenith angles 
  difsol= (box % sunzen - cst_solhigh) / (cst_solnight - cst_solhigh) 

! altitude difference between fov and nwp
  if(box % altnwp > vmanq) then
     difalt=box % alt -box % altnwp
  else
     difalt=0.
  endif
  
  if(idbg%debug) then
    write(*,*) 'maia_GetThres_CMaLand: satsec, wv, alb, tsurf, sunzen', satsec_loc, box % cwv , alb, tsurf, box % sunzen     
    write(*,*) 'maia_GetThres_CMaLand: isec,iw,difsec,difw,difalt,difalb',isec, iw, difsec, difw, difalt, difalb
  end if

! compute the different thresholds
 if (alb < cst_veg_max_alb .or. box % snow_always) then

    if(box % sunzen > cst_solnight) then
      ! s_45         
      seuil=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec)
      if (seuil > vmanq) seuil=seuil+ max_noise  
      thres% v_CMa_108_120_land=seuil               
      ! s_t87t108  
      seuil=maia_Interp_InLut(tabland%tab87_108_ln_veg,difsec,difw,iw,isec)
      if (seuil > vmanq) seuil=seuil+ max_noise  
      thres% v_CMa_87_108_land=seuil   
      ! s_t87t37                
      seuilw=maia_Interp_InLut(tabland%tab87_37_l_warm_veg,difsec,difw,iw,isec) 
      seuilc=maia_Interp_InLut(tabland%tab87_37_l_cold_veg,difsec,difw,iw,isec)
      seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
      if (seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_87_37_land=seuil          
      ! s_ts4 
      seuil=maia_Interp_InLut(tabland%tabtsurf_ln_veg,difsec,difw,iw,isec) 
      if(seuil > vmanq) seuil=seuil+ max_noise+ cst_adiarate*difalt
      thres% v_CMa_108_land=seuil          

    else if(box % sunzen < cst_solhigh) then            
      ! s_45 
      seuild=maia_Interp_InLut(tabland%tab108_120_ld,difsec,difw,iw,isec)
      seuiln=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec)
      seuil= (1-difsol)*seuild + difsol*seuiln
      if(seuil > vmanq ) seuil=seuil+max_noise
      thres% v_CMa_108_120_land=seuil     
      ! s_t87t108
      seuil=maia_Interp_InLut(tabland%tab87_108_ld_veg,difsec,difw,iw,isec)
      if(seuil > vmanq ) seuil=seuil+max_noise
      thres% v_CMa_87_108_land=seuil     
      ! s_ts4 
      seuil=maia_Interp_InLut(tabland%tabtsurf_ld_veg,difsec,difw,iw,isec)
      if(seuil > vmanq ) seuil=seuil+max_noise+cst_adiarate*difalt
      thres% v_CMa_108_land=seuil       
    else      
      ! s_45 
      seuild=maia_Interp_InLut(tabland%tab108_120_ld,difsec,difw,iw,isec) 
      seuiln=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec) 
      seuil = (1-difsol)*seuild + difsol*seuiln
      if (seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_108_120_land=seuil          
      ! s_ t87t108
      seuild=maia_Interp_InLut(tabland%tab87_108_ld_veg,difsec,difw,iw,isec) 
      seuiln=maia_Interp_InLut(tabland%tab87_108_ln_veg,difsec,difw,iw,isec) 
      seuil = (1-difsol)*seuild + difsol*seuiln
      if (seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_87_108_land=seuil          
      ! s_ts4 
      seuild=maia_Interp_InLut(tabland%tabtsurf_ld_veg,difsec,difw,iw,isec) 
      seuiln=maia_Interp_InLut(tabland%tabtsurf_ln_veg,difsec,difw,iw,isec) 
      seuil = (1-difsol)*seuild + difsol*seuiln
      if (seuil > vmanq) seuil=seuil+max_noise+cst_adiarate*difalt
      thres% v_CMa_108_land=seuil            
    endif       !end of solzen     
            
    if(tsurf < cst_tempcold) then
      ! s_43          
      seuil=maia_Interp_InLut(tabland%tab108_37_l_cold_veg,difsec,difw,iw,isec)
      if(seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_108_37_land=seuil          
      ! s_34 
      seuil=maia_Interp_InLut(tabland%tab37_108_l_cold_veg,difsec,difw,iw,isec)
      if (seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_37_108_land=seuil                
      ! s_t87t37          
      seuil=maia_Interp_InLut(tabland%tab87_37_l_cold_veg,difsec,difw,iw,isec)
      if(seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_87_37_land=seuil                     
     ! s_t37t40 
      seuil=maia_Interp_InLut(tabland%tab37_40_l_cold_veg,difsec,difw,iw,isec)
      if (seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_37_40_land=seuil                
    else                  
      ! s_43 
      seuilw=maia_Interp_InLut(tabland%tab108_37_l_warm_veg,difsec,difw,iw,isec) 
      seuilc=maia_Interp_InLut(tabland%tab108_37_l_cold_veg,difsec,difw,iw,isec)
      seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
      if (seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_108_37_land=seuil          
      ! s_34 
      seuilw=maia_Interp_InLut(tabland%tab37_108_l_warm_veg,difsec,difw,iw,isec) 
      seuilc=maia_Interp_InLut(tabland%tab37_108_l_cold_veg,difsec,difw,iw,isec)
      seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
      if (seuil > vmanq) seuil=seuil+max_noise 
      thres% v_CMa_37_108_land=seuil       
      ! s_t87t37 
      seuilw=maia_Interp_InLut(tabland%tab87_37_l_warm_veg,difsec,difw,iw,isec) 
      seuilc=maia_Interp_InLut(tabland%tab87_37_l_cold_veg,difsec,difw,iw,isec)
      seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
      if (seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_87_37_land=seuil          
      ! s_t37t40 
      seuilw=maia_Interp_InLut(tabland%tab37_40_l_warm_veg,difsec,difw,iw,isec) 
      seuilc=maia_Interp_InLut(tabland%tab37_40_l_cold_veg,difsec,difw,iw,isec)
      seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
      if (seuil > vmanq) seuil=seuil+max_noise 
      thres% v_CMa_37_40_land=seuil       
    endif   !end of tsurf     
 
 else if (alb > cst_des_min_alb) then
   
   if(box % sunzen > cst_solnight) then            
     ! s_45          
     seuil=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec)
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_120_land=seuil          
     ! s_t87t108  
     seuil=maia_Interp_InLut(tabland%tab87_108_ln_des,difsec,difw,iw,isec)
     if (seuil > vmanq) seuil=seuil+ max_noise  
     thres% v_CMa_87_108_land=seuil               
      ! s_t87t37          
      seuil=maia_Interp_InLut(tabland%tab87_37_l_cold_des,difsec,difw,iw,isec)
      if(seuil > vmanq) seuil=seuil+max_noise
      thres% v_CMa_87_37_land=seuil                     
     ! s_ts4 
     seuil=maia_Interp_InLut(tabland%tabtsurf_ln_des,difsec,difw,iw,isec)
     seuil=seuil+ 4.
     if (seuil > vmanq) seuil=seuil+max_noise+cst_adiarate*difalt
     thres% v_CMa_108_land=seuil                  
   else if(box % sunzen < cst_solhigh) then            
     ! s_45 
     seuild=maia_Interp_InLut(tabland%tab108_120_ld,difsec,difw,iw,isec) 
     seuiln=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec)
     seuil = (1-difsol)*seuild + difsol*seuiln
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_120_land=seuil               
     ! s_t87t108  
     seuil=maia_Interp_InLut(tabland%tab87_108_ld_des,difsec,difw,iw,isec)
     if (seuil > vmanq) seuil=seuil+ max_noise  
     thres% v_CMa_87_108_land=seuil               
     ! s_ts4 
     seuil=maia_Interp_InLut(tabland%tabtsurf_ld_des,difsec,difw,iw,isec)
     seuil=seuil+4.
     if (seuil > vmanq) seuil=seuil+max_noise+cst_adiarate*difalt
     thres% v_CMa_108_land=seuil             
   else            
     ! s_45          
     seuild=maia_Interp_InLut(tabland%tab108_120_ld, difsec,difw,iw,isec)
     seuiln=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec)
     seuil = (1-difsol)*seuild + difsol*seuiln                 
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_120_land=seuil          
     ! s_t87t108  
     seuiln=maia_Interp_InLut(tabland%tab87_108_ln_des,difsec,difw,iw,isec)
     seuild=maia_Interp_InLut(tabland%tab87_108_ld_des,difsec,difw,iw,isec)
     seuil = (1-difsol)*seuild + difsol*seuiln                 
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_108_land=seuil               
     ! s_ts4 
     seuild=maia_Interp_InLut(tabland%tabtsurf_ld_des,difsec,difw,iw,isec) 
     seuild=seuild+4.
     seuiln=maia_Interp_InLut(tabland%tabtsurf_ln_des,difsec,difw,iw,isec)
     seuiln=seuiln+4.
     seuil = (1-difsol)*seuild + difsol*seuiln
     if (seuil > vmanq) seuil=seuil+max_noise+cst_adiarate*difalt
     thres% v_CMa_108_land=seuil 
   endif     !end of solzen
            
   if(tsurf  <  cst_tempcold) then
     ! s_43 
     seuil=maia_Interp_InLut(tabland%tab108_37_l_cold_des,difsec,difw,iw,isec)
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_37_land=seuil          
     ! s_34 
     seuil=maia_Interp_InLut(tabland%tab37_108_l_cold_des,difsec,difw,iw,isec)
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_37_108_land=seuil               
     ! s_t87t37          
     seuil=maia_Interp_InLut(tabland%tab87_37_l_cold_des,difsec,difw,iw,isec)
     if(seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_37_land=seuil                                  
      ! s_t37t40 
     seuil=maia_Interp_InLut(tabland%tab37_40_l_cold_des,difsec,difw,iw,isec)
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_37_40_land=seuil               
   else                  
     ! s_43 
     seuilw=maia_Interp_InLut(tabland%tab108_37_l_warm_des,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab108_37_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_37_land=seuil          
     ! s_34 
     seuilw=maia_Interp_InLut(tabland%tab37_108_l_warm_des,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab37_108_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
     if (seuil > vmanq) seuil=seuil+max_noise 
     thres% v_CMa_37_108_land=seuil
     ! s_t87t37          
     seuilw=maia_Interp_InLut(tabland%tab87_37_l_warm_des,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab87_37_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
     if(seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_37_land=seuil                                 
      ! s_t37t40 
     seuilw=maia_Interp_InLut(tabland%tab37_40_l_warm_des,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab37_40_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-diftsurf)*seuilc + diftsurf*seuilw
     if (seuil > vmanq) seuil=seuil+max_noise 
     thres% v_CMa_37_40_land=seuil
   endif      !end of tsurf 
         
 else
   
   if(box % sunzen  >  cst_solnight) then            
     ! s_45          
     seuil=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec)
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_120_land=seuil          
     ! s_t87t108  
     seuilveg=maia_Interp_InLut(tabland%tab87_108_ln_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tab87_108_ln_des,difsec,difw,iw,isec)
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_108_land=seuil             
     ! s_t87t37          
     seuilveg=maia_Interp_InLut(tabland%tab87_37_l_cold_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tab87_37_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if(seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_37_land=seuil                     
     ! s_ts4 
     seuilveg=maia_Interp_InLut(tabland%tabtsurf_ln_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tabtsurf_ln_des,difsec,difw,iw,isec)
     seuildes=seuildes+4.
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise+cst_adiarate*difalt
     thres% v_CMa_108_land=seuil                     
   else if(box % sunzen  <  cst_solhigh) then            
     ! s_45          
     seuild=maia_Interp_InLut(tabland%tab108_120_ld,difsec,difw,iw,isec)
     seuiln=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec)
     seuil = (1-difsol)*seuild + difsol*seuiln
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_120_land=seuil             
     ! s_t87t108  
     seuilveg=maia_Interp_InLut(tabland%tab87_108_ld_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tab87_108_ld_des,difsec,difw,iw,isec)
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_108_land=seuil             
     ! s_ts4 
     seuilveg=maia_Interp_InLut(tabland%tabtsurf_ld_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tabtsurf_ld_des,difsec,difw,iw,isec)
     seuildes=seuildes+4.
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise+cst_adiarate*difalt
     thres% v_CMa_108_land=seuil             
   else      
     ! s_45          
     seuild=maia_Interp_InLut(tabland%tab108_120_ld,difsec,difw,iw,isec)
     seuiln=maia_Interp_InLut(tabland%tab108_120_ln,difsec,difw,iw,isec)
     seuil = (1-difsol)*seuild + difsol*seuiln
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_120_land=seuil          
     ! s_t87t108  
     seuild=maia_Interp_InLut(tabland%tab87_108_ld_veg,difsec,difw,iw,isec)
     seuiln=maia_Interp_InLut(tabland%tab87_108_ln_veg,difsec,difw,iw,isec)
     seuilveg = (1-difsol)*seuild + difsol*seuiln
     seuild=maia_Interp_InLut(tabland%tab87_108_ld_des,difsec,difw,iw,isec)
     seuiln=maia_Interp_InLut(tabland%tab87_108_ln_des,difsec,difw,iw,isec)
     seuildes = (1-difsol)*seuild + difsol*seuiln
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_108_land=seuil             
     ! s_ts4 
     seuild=maia_Interp_InLut(tabland%tabtsurf_ld_veg,difsec,difw,iw,isec)
     seuiln=maia_Interp_InLut(tabland%tabtsurf_ln_veg,difsec,difw,iw,isec)
     seuilveg = (1-difsol)*seuild + difsol*seuiln
     seuild=maia_Interp_InLut(tabland%tabtsurf_ld_des,difsec,difw,iw,isec)
     seuild=seuild+4.
     seuiln=maia_Interp_InLut(tabland%tabtsurf_ln_des,difsec,difw,iw,isec)
     seuiln=seuiln+4.
     seuildes = (1-difsol)*seuild + difsol*seuiln
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise+cst_adiarate*difalt
     thres% v_CMa_108_land=seuil
   endif   !end of solzen   
            
   if(tsurf  <  cst_tempcold) then
     ! s_43 
     seuilveg=maia_Interp_InLut(tabland%tab108_37_l_cold_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tab108_37_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_37_land=seuil          
     ! s_34 
     seuilveg=maia_Interp_InLut(tabland%tab37_108_l_cold_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tab37_108_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq)seuil=seuil+max_noise
     thres% v_CMa_37_108_land=seuil                            
     ! s_t87t37          
     seuilveg=maia_Interp_InLut(tabland%tab87_37_l_cold_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tab87_37_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_37_land=seuil         
     ! s_t37t40 
     seuilveg=maia_Interp_InLut(tabland%tab37_40_l_cold_veg,difsec,difw,iw,isec)
     seuildes=maia_Interp_InLut(tabland%tab37_40_l_cold_des,difsec,difw,iw,isec)
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq)seuil=seuil+max_noise
     thres% v_CMa_37_40_land=seuil                            
   else               
     ! s_43 
     seuilw=maia_Interp_InLut(tabland%tab108_37_l_warm_veg,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab108_37_l_cold_veg,difsec,difw,iw,isec)
     seuilveg = (1-diftsurf)*seuilc + diftsurf*seuilw
     seuilw=maia_Interp_InLut(tabland%tab108_37_l_warm_des,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab108_37_l_cold_des,difsec,difw,iw,isec)
     seuildes = (1-diftsurf)*seuilc + diftsurf*seuilw
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_108_37_land=seuil          
     ! s_34 
     seuilw=maia_Interp_InLut(tabland%tab37_108_l_warm_veg,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab37_108_l_cold_veg,difsec,difw,iw,isec)  
     seuilveg = (1-diftsurf)*seuilc + diftsurf*seuilw
     seuilw=maia_Interp_InLut(tabland%tab37_108_l_warm_des,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab37_108_l_cold_des,difsec,difw,iw,isec)
     seuildes = (1-diftsurf)*seuilc + diftsurf*seuilw
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise 
     thres% v_CMa_37_108_land=seuil   
     ! s_t87t37          
     seuilw=maia_Interp_InLut(tabland%tab87_37_l_warm_veg,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab87_37_l_cold_veg,difsec,difw,iw,isec)
     seuilveg = (1-diftsurf)*seuilc + diftsurf*seuilw
     seuilw=maia_Interp_InLut(tabland%tab87_37_l_warm_des,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab87_37_l_cold_des,difsec,difw,iw,isec)
     seuildes = (1-diftsurf)*seuilc + diftsurf*seuilw
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise
     thres% v_CMa_87_37_land=seuil          
     ! s_t37t40 
     seuilw=maia_Interp_InLut(tabland%tab37_40_l_warm_veg,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab37_40_l_cold_veg,difsec,difw,iw,isec)  
     seuilveg = (1-diftsurf)*seuilc + diftsurf*seuilw
     seuilw=maia_Interp_InLut(tabland%tab37_40_l_warm_des,difsec,difw,iw,isec)
     seuilc=maia_Interp_InLut(tabland%tab37_40_l_cold_des,difsec,difw,iw,isec)
     seuildes = (1-diftsurf)*seuilc + diftsurf*seuilw
     seuil = (1-difalb)*seuilveg + difalb*seuildes
     if (seuil > vmanq) seuil=seuil+max_noise 
     thres% v_CMa_37_108_land=seuil   
   endif        !end of tsurf 

 endif    !end of alb
  if( idbg%debug ) then
        write(*,*) 'THRES land 108, 108-120, 37-108, 108-37, 87_108, 87_37 avant: ',                       &
                 thres % v_CMa_108_land, thres % v_CMa_108_120_land, thres% v_CMa_37_108_land, &
                 thres % v_CMa_108_37_land, thres % v_CMa_87_108_land, thres % v_CMa_87_37_land
  endif         

END SUBROUTINE maia_GetThres_CMaLand
