! Name: $Id: maia_GetTopo.F90 502 2016-11-15 10:22:21Z frna $
!
! Purpose: 
!          Subroutines maia_GetTopo 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    20/05/2016   Lydie Lavanant maia 4 version 4.3
SUBROUTINE maia_GetTopo ( idbg, landsea, elev, lat, lon_in, psize ,lsize, tm, alt )

! Authors       : lydie lavanant

! fichier resolution 0.01 degre
!   landsea=0   ! mer/inland-water
!   landsea=1   ! terre
!   landsea=2   ! desert
!   landsea=3   ! neige perenne
!   landsea=4   ! cote
!   landsea=5   ! inland water
! alt (m)

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE
  
  type( debug ), INTENT(in)     :: idbg ! 
  type( topo_field), INTENT(in) :: landsea, elev        ! surface topography landsea and elev
  REAL, INTENT(in)              :: lat, lon_in
  INTEGER, INTENT(in)           :: psize ,lsize
  INTEGER, INTENT(out)          :: tm, alt
  
  REAL          :: tab(20,20), value, lon
  REAL          :: rx, ry
  INTEGER       :: nx, ny, ix, iy, ii, jj, ixmax, ixmin, p, ik, k, box_size
 
! ---------------------------------------------------------------------------
  lon = lon_in
  tab(:,:)=vmanq

  ! taille de la boite
  nx= 1 ; ny=1
  if(psize>1) nx= INT(psize/2.)
  if(lsize>1) ny= INT(lsize/2.)

  ! surface elevation
  ! ----------------------------------------------------------------------------
  if(elev% norm180) then
    if(lon_in  >=  180. - elev% lon_inc/2.) lon = lon_in - 360.
    if(lon_in  <=  -180. - elev% lon_inc/2.) lon = lon_in + 360.
  else
    if(lon_in  >=  360. - elev% lon_inc/2.) lon = lon_in - 360.
    if(lon_in  <=     0. - elev% lon_inc/2.) lon = lon_in + 360.
  end if

  ! lower node of the grid for the pixel
  ry= 1+ (lat- elev % lat1)/elev % lat_inc
  rx= 1+ (lon- elev % lon1)/elev % lon_inc
  iy = INT(ry)  ! lower lat grid
  ix = INT(rx)  ! lower lon grid
  ry = ry-iy
  rx = rx-ix
  
  ! manage atlas borders in lat
  if(iy<1) iy=1
  if(iy+1>elev%nbl) iy=elev%nbl-1
  

  if(idbg % debug) then
     write(*,*) '     GetTopo:', lat, lon
     write(*,'(a20, 4f20.4)') '     GetTopo:', elev% lat1, elev% lon1, elev% lat_inc, elev% lon_inc
     write(*,*) '     GetTopo:', ix, iy, elev % data (ix, iy)
  endif  
  if(psize ==1 .and. lsize==1) then
     if(iy<1 .or. iy+1>elev%nbl) then
        write(*,*) '     GetTopo: outside atlas in Lat:', lat, iy, lsize, ny, elev % lat1, elev% nbl
        stop 2
     else
        ixmin=ix
        if(ix==0) ixmin = elev%nbp
        tab(1,1) =elev % data (ixmin, iy)
        tab(1,2) =elev % data (ixmin, iy+1)
        ixmax=ix+1
        if(ix==elev%nbp) ixmax=1
        tab(2,1) =elev % data (ixmax, iy)
        tab(2,2) =elev % data (ixmax, iy+1)
        
        call maia_interp_InGrid (tab(1:2,1:2), rx, ry, value)
        if(idbg % debug)  write(*,*) '     Getelev:', ix, iy, tab(1:2,1:2), value
        if(value <0)      write(*,*) '     Getelev:', lat, lon, rx, ry, tab(1:2,1:2), value
     endif
  else
     box_size=MIN( (iy-1) , (elev%nbl - iy))
     ny=MIN(box_size,ny)
     if(iy-ny<1 .or. iy+ny>elev%nbl) then
        write(*,*) '     GetTopo: outside atlas in Lat:', lat, iy, ny, elev % lat1, elev%nbl
        stop 2
     else
        p=0
        do k=-nx,nx
           p=p+1
           ik=ix+k
           if(ik<1) ik=ik + elev%nbp
           if(ik>elev%nbp) ik=ik - elev%nbp
           tab(p,1:1+2*ny) =elev % data (ik,iy-ny:iy+ny)
        end do
        call maia_valmoy ( idbg, nx,ny, tab, vmanq, value)
     endif
  endif
  alt = value
  if(idbg % debug) then
     write(*,*) '     GetTopo alt:', alt
  endif  

  ! surface type of the nearest node
  ! 0.005 to take into account that the landsea dataset was created from 0.005 resolution data
  ! ----------------------------------------------------------------------------
  if(landsea% norm180) then
    if(lon_in  >=  180. - landsea% lon_inc/2.) lon = lon_in - 360.
    if(lon_in  <=  -180. - landsea% lon_inc/2.) lon = lon_in + 360.
  else
    if(lon_in  >=  360. - landsea% lon_inc/2.) lon = lon_in - 360.
    if(lon_in  <=     0. - landsea% lon_inc/2.) lon = lon_in + 360.
  end if
  if(idbg % debug) then
     write(*,*) '     GetTopo:', lat, lon
     write(*,'(a20, 4f20.4)') '     GetTopo:', landsea% lat1, landsea% lon1, landsea% lat_inc, landsea% lon_inc
     write(*,'(a20, 4i20  )') '     GetTopo:', landsea%nbp, landsea%nbl
  endif  

  ! lower node of the grid for the pixel
  ry=  1+(lat+0.005- landsea % lat1)/landsea % lat_inc
  rx=  1+(lon-0.005- landsea % lon1)/landsea % lon_inc
  iy = INT(ry)  ! lower lat grid
  ix = INT(rx)  ! lower lon grid
  ry = ry-iy
  rx = rx-ix
  ii=ix
  if(rx>0.5) ii=ix+1
  if(ii==0) ii=landsea%nbp
  if(ii==landsea%nbp) ii=1
  jj=iy
  if(ry>0.5) jj=iy+1
  tm = landsea % data (ii, jj)
     
  box_size=MIN( (jj-1) , (landsea%nbl - jj))
  ny=MIN(box_size,ny)

  if(psize /=1 .or. lsize/=1) then
     do k=-nx,nx
        ik=ii+k
        if(ik<1) ik=ik + landsea%nbp
        if(ik>landsea%nbp) ik=ik - landsea%nbp
        if ( any( landsea % data (ik, jj-ny:jj+ny)==0) .and. & 
             any( landsea % data (ik, jj-ny:jj+ny)>0) ) tm=4 
        if ( any( landsea % data (ik, jj-ny:jj+ny)==4) ) tm=4 
     end do
  endif
  if(idbg % debug) then
     write(*,*) '     GetTopo tm:', tm
  endif  

END SUBROUTINE maia_GetTopo
