! Name: $Id: maia_PixEnv_reset.F90 514 2017-02-06 11:28:14Z frna $
!
! Purpose: reset pixel environment
!          Subroutines maia_PixEnv_reset 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/01/2014   Lydie Lavanant update Add control for missing channels; use 0.01 degres landsea atlas; update definition of CloudMask (bits 15-18).
!  1.2    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.3    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.4    23/01/2017   Pascale Roquet remove unused variables
SUBROUTINE maia_PixEnv_reset ( idbg, landsea, elev, box, pix_id, pix, CMa, CT, CH)

! Authors       : lydie lavanant

! set environment and reset CMa, CT, CH output

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

!     pix_id%tm = 0     ! sea + inland water
!     pix_id%tm = 1     ! coast
!     pix_id%tm = 2     ! land
!     pix_id%tm = 3     ! desert
!     pix_id%tm = 4     ! snow_always

  type( debug ), INTENT(in)        :: idbg         ! 
  type( topo_field),INTENT(in)     :: landsea, elev     ! surface topography landsea and elev
  type( box_id ),INTENT(inout)     :: box               ! box information
  type( pix_info ),INTENT(inout)   :: pix_id            ! pix_id information
  type( pix_data),  INTENT(in)     :: pix               ! pix observations (albedo in %, Tb in K)
  type( maia_CMa ), INTENT(inout)  :: CMa
  type( maia_CT), INTENT(inout)    :: CT                ! 
  type( maia_CH), INTENT(inout)    :: CH                ! 

  INTEGER                          :: ls, alt

!===================================================================

  ! Longitude normalisation between -180 and +180 degrees
  ! -----------------------------------------------------
  call maia_Lon_Norm (.false., 'data  ', pix_id % lon)

  ! relative azimuth angle  (deg)
  ! -----------------------------
  if(pix_id % difazi< -360.) then
     pix_id % difazi = ABS(pix_id % satazi - pix_id % sunazi)
     if( pix_id % difazi >  180. ) pix_id % difazi = pix_id % difazi - 360.
     if( pix_id % difazi < -180. ) pix_id % difazi = pix_id % difazi + 360.
  endif

  ! secant of local zenith angle
  ! -----------------------------
  pix_id % satsec = 1./cos(pix_id % satzen * cdr)

  ! glint conditions
  !-----------------
  pix_id % dcj = box % dcj
  pix_id % glint = box % glint

  ! 0: NIGHT /1: TWILIGHT /2: DAY /3: SUNGLINT
  ! -----------------------------
  pix_id % day      =.false.
  pix_id % night    =.false.
  pix_id % glint    =.false.
  pix_id % twilight =.false.

  if(box % dcj == 0 .and. pix % Qual_06 .and. pix % Qual_08 .and. &
     pix_id % sunzen < cst_sunzenmax_day) then               
        pix_id % day_time =2                       ! DAY           
        pix_id % day      =.true.
  else if (box % dcj == 1 .and. pix % Qual_06 .and. pix % Qual_08 .and. &
           pix_id % sunzen < cst_sunzenmax_day ) then        
        pix_id % day_time =3                       ! SUNGLINT              
        pix_id % glint    =.true.
  else if (pix % Qual_37 .and. pix_id % sunzen > cst_sunzenmin_night ) then                      
        pix_id % day_time =0                       ! NIGHT
        pix_id % night    =.true.
  else if (pix % Qual_37 .and. pix % Obs_37>vmanq .and. pix % Obs_06==vmanq .and. &
           pix_id % sunzen >= cst_sunzenmax_day) then                      
        pix_id % day_time =0                       ! NIGHT
        pix_id % night    =.true.
  else    
        if( pix_id % sunzen >= cst_sunzenmax_day) then
            pix_id % day_time = 1                  ! TWILIGHT
            pix_id % twilight =.true.
        else
            pix_id % day      = box % day
            pix_id % night    = box % night
            pix_id % glint    = box % glint
            pix_id % twilight = box % twilight
            pix_id % day_time = box % day_time
        endif
  endif 
   

  ! topography at pix_id resolution
  ! -----------------------------
  if(pix_id % tm < 0 .or. pix_id % alt <= -9000.) then  ! from user
     call maia_GetTopo ( idbg, landsea, elev, pix_id%lat, pix_id%lon, 1, 1, ls, alt)
  endif 

  pix_id%alt=alt

  pix_id % sea =                .false.  
  pix_id % coast =              .false.  
  pix_id % land =               .false.  
  pix_id % desert =             .false. 
  pix_id % snow_always =        .false. 

  if (ls ==0 ) then
     pix_id % sea  = .true. 
     pix_id%tm = 0
  elseif (ls ==1 ) then
     pix_id % land = .true. 
     pix_id%tm = 2
  elseif (ls ==2 ) then
     pix_id % desert = .true.
     pix_id%tm = 3
  elseif (ls ==3 ) then
     pix_id % snow_always = .true.
     pix_id%tm = 4
  elseif (ls ==4 ) then
     pix_id % coast = .true. 
     pix_id%tm = 1
  elseif (ls ==5 ) then         ! inland water
     pix_id % sea = .true. 
     pix_id%tm = 0              ! inland water put in sea
  else
     write(*,*) 'Pb in TOPO', pix_id%lat, pix_id%lon, ls, alt
     stop 2
  endif
  
  if (box % land .and. box % albedo_clim > cst_desertalbmin ) then
       pix_id % desert = .true.
       pix_id%tm = 3
  endif
  

  
  ! reset all flags
  call maia_reset_CMa (idbg, CMa)

  CT % black            =imanq          ! black-body flag (1= black-body)
  CT % cld_type         =imanq          ! cloud type    1=CT_VLOW to 10=CT_FRACTIONAL
  CT % class            =imanq          ! classification from 0 to 20 
  CT % Phase            =imanq  
  CT % Heavy_Aerosol    =imanq
  CT % Smoke_Candidate  =imanq
  CT % Dust_Candidate   =imanq
  CT % Dust_VolcanicAsh =imanq
  CT % Activefire_fl    =imanq
  CT % MNeighbors       =imanq

  CH % Tcld = vmanq
  CH % Pcld = vmanq

  ! the 3.7mm does not seem established well for few lines at the day/night transition for metop/avhrr 
  if(pix_id% ImgName == 'avhrr' .and. pix_id % night .and. pix % Qual_37 .and. pix % Qual_06) then
    if(pix % Obs_06>0.05 ) then
        pix_id % day_time =1                       ! TWILIGHT
        pix_id % twilight =.true.
        pix_id % night    =.false.
    endif
  endif
 
 END SUBROUTINE  maia_PixEnv_reset
      
