! Name: $Id: maia_Read_GribApi.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: Read the forecast in GRIB API format
!          Subroutines maia_Read_GribApi niveau mk_NWP_field init_NWP_field maia_hutorm sort_press ml2pl_91 ml2pl_137 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/01/2014   Lydie Lavanant update Add control for missing channels; use 0.01 degres landsea atlas; update definition of CloudMask (bits 15-18)
!  1.2    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.3    01/05/2015   Pascale Roquet bug fixes
!  1.4    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.5    01/06/2017   Pascale Roquet eccodes compatibility
!---------------------------------------------------------------------------  
!> maia_Read_GribApi subroutine
!
!> @author      lydie lavanant
!> </dl>
!> @brief       Read the forecast in GRIB API format <br>
!> @brief       Units are: Tempe (K) / Relative Humidity (%) or specific Humidity (ppmv) / 
!> @brief       Precipitable Water (kg/m2) / Pressure (Pa) / Altitude (m) / land/sea  (0=sea 1=land) <br>
!> @brief       verified for MF/Arpege, ECMWF and GFS  <br>
!---------------------------------------------------------------------------  
subroutine  maia_Read_GribApi (idbg, iuforecast, bg, all_ok)

#ifdef HAS_LIBECCODES
USE eccodes
#elif HAS_LIBGRIB_API
USE grib_api
#define CODES_SUCCESS 1
#endif

USE MOD_MAIA_CONST
USE MOD_maia_types

IMPLICIT NONE
        
  type( debug ),    INTENT(inout)  :: idbg     
  INTEGER,          INTENT(in)  :: iuforecast   ! input logical unit
  type( nwp_field),INTENT(inout):: bg           ! forecast field information
  LOGICAL,         intent(out)  :: all_ok       ! true if all fields found

#if defined(HAS_LIBGRIB_API) || defined(HAS_LIBECCODES)

  INTEGER, PARAMETER    :: nbniv_max=200                ! maximum number of temperature levels
  real                  :: pres_ml(nbniv_max)
  character(len=20)     :: name_space
  integer               :: igrib, iret, kiter, grib_count, status, date
  INTEGER               :: i, j, k, l, n, p, lev, kk, ik

  character(len=256)    :: key, value
  character(len=512)    :: info_key(10), info_val(10)
  character(len=2)      :: centre
  character(len=4)      :: typ_niv, field_api, humid_type

  integer               :: niv, nblev, nbl, nbp, np, nivp, NbOfPoints

  real(kind=8),dimension(:),allocatable     :: lats,lons,values

  LOGICAL               :: ch_ts, ch_t2m, ch_psurf, ch_cwv ! true if field found
  LOGICAL               :: ch_ci, ch_sd                    ! true if field found
  LOGICAL, allocatable  :: temp_ok(:), humid_ok(:)         ! true if fields found
  LOGICAL               :: t_level_ok, h_level_ok

  real                  :: scale_data, tcwv
  INTEGER               :: ig(7), ug(4) ! date and time
  REAL(KIND=8)          :: ij2
  real, allocatable     :: pres_cal(:), rm_cal(:), rm(:) 
  INTEGER               :: my_ech,my_day_count

  real, allocatable     :: bg_tm(:,:), bg_alt(:,:), pressions(:)
  real, allocatable     :: temp_atmos(:,:,:), humid_atmos(:,:,:)
  logical               :: found_p
  integer, allocatable  :: indx(:)
  integer               :: sens=1      ! increasing order


!=============================================[ instructions ]==========
  name_space='ls'               ! valid name_spaces are ls and mars

 ! get the characteristics of the NWP
  ! ------------------------------------   
  call grib_new_from_file (iuforecast, igrib, iret) 
  call grib_set(igrib,'missingValue',vmanq)

  call grib_get(igrib, 'latitudeOfFirstGridPointInDegrees', bg % lat1 ) 
  call grib_get(igrib, 'longitudeOfFirstGridPointInDegrees',bg % lon1 ) 
  call grib_get(igrib, 'latitudeOfLastGridPointInDegrees', bg %latend) 
  call grib_get(igrib, 'longitudeOfLastGridPointInDegrees',bg %lonend) 
  call grib_get(igrib, 'numberOfPointsAlongAParallel', bg %nbp)
  call grib_get(igrib, 'numberOfPointsAlongAMeridian', bg %nbl)
  call grib_get(igrib, 'iDirectionIncrementInDegrees', bg %lat_inc)
  call grib_get(igrib, 'jDirectionIncrementInDegrees', bg %lon_inc)

  if(bg %latend < bg % lat1 .and. bg %lat_inc>0) bg %lat_inc = -bg %lat_inc
  nbp = bg %nbp
  nbl = bg %nbl
  
  bg % circumferential =.false.
  if(bg %lonend + bg %lon_inc == bg % lon1+360.) bg % circumferential = .true.
  if(bg % circumferential)       call ml_wt('info','NWP cylindrical around the earth')

  ig(:)=0
  call grib_get(igrib,'date',date)
  call grib_get(igrib,'year',ig(1))
  call grib_get(igrib,'month',ig(2))
  call grib_get(igrib,'day',ig(3))
  call grib_get(igrib,'hour',ig(4))
  call grib_get(igrib,'minute',ig(5))
  call xigj(ig,bg%ij1)

  ug(:)=0

  call grib_get(igrib,'step',bg%ech)    !echeance
  my_ech=modulo(bg%ech,24)
  my_day_count=bg%ech/24
 
  ug(1) =my_ech 
  call xugj(ug,ij2)
  
  if(idbg%info) write(*,*) "ech: ",bg%ech," ij2 : ",ij2," my_day_count :",my_day_count
  
  ! date correspondant aux donnees
  bg%ij =bg%ij1+my_day_count+ij2

  ! normalisation NWP
  bg % norm180 = .true.
  if( bg % lonend >= 180.) bg % norm180 = .false.

  if(idbg%debug) then
     if(bg % norm180)       call ml_wt('info','NWP in [-180,180[')
     if(.not. bg % norm180) call ml_wt('info','NWP in [0,360[')
  endif

  if(idbg%info) then
     call xijg(bg%ij,ig)
     write(*,*) 'NWP forecast is for: ',bg%ij1, ij2, bg%ij, ig
     write(*,*) 'lat1, lon1=', bg % lat1, bg % lon1, bg % lat_inc, bg % lon_inc, &
                               bg % nbl, bg % nbp, bg % latend, bg % lonend
  endif

  nblev=200
  
  bg % mltype = 'Pres'
  allocate ( bg%lat(nbp,nbl), bg%lon(nbp,nbl))  
  allocate ( bg%ts(nbp,nbl) )                   ! surface temperature(K)
  allocate ( bg%t2m(nbp,nbl) )                  ! temperature 2m (K)
  allocate ( bg%cwv(nbp,nbl) )                  ! total water vapor content (g/cm2)
  allocate ( bg%psurf(nbp,nbl) )                ! psurf (hpa)
  allocate ( bg%alt(nbp,nbl), bg_alt(nbp,nbl) )                 ! surface roughness (m)
  allocate ( bg%seaice(nbp,nbl),bg%snow(nbp,nbl))! seaice cover (0-1), snowdepth (m)
  allocate ( bg%tm(nbp,nbl), bg_tm(nbp,nbl) )   ! 

  allocate ( temp_atmos(nblev,nbp,nbl))         ! temperature (K)
  allocate ( humid_atmos(nblev,nbp,nbl))        ! water vapour (% or kg/kg)
  allocate ( pressions(nblev))
  
  pressions(:)=vmanq ; temp_atmos(:,:,:)=vmanq ; humid_atmos(:,:,:)=vmanq

  ! initializations:
  call init_NWP_field (bg)
  ch_ts=.false. ; ch_t2m=.false. ; ch_psurf=.false. ; ch_cwv=.false. ; ch_ci=.false. ; ch_sd=.false.

  bg % nbniv =0  ;  np=0 ;  grib_count=0

  ! ------------------------------------   
  ! Loop on all the messages in the file
  ! ------------------------------------   
  DO WHILE (iret /= GRIB_END_OF_FILE)

     grib_count=grib_count+1

     call grib_keys_iterator_new (igrib, kiter, name_space)     !create a new iterator
      
     field_api='xxxx' ; typ_niv='xxxx' ; centre='xx'; n=0 ; niv=0
     do
        call grib_keys_iterator_next( kiter, iret)
       
        if (iret /= CODES_SUCCESS) then
            exit                !iret=1 if new iterator ;0=il n'y a plus d'elements
        endif
        n=n+1
         
        call grib_keys_iterator_get_name ( kiter, key)  ! key=nom du iterator
        call grib_get (igrib, trim(key), value, status) ! recupere les caracteristiques du champ
        info_key(n) =trim(key)
        info_val(n) =trim(value)

        if(trim(key)=='centre' .and.   trim(value)=='lfpw') centre='MF'
        if(trim(key)=='centre' .and.   trim(value)=='ecmf') centre='EC'

        if(trim(key)=='typeOfLevel' .and. trim(value)=='hybrid')                typ_niv='ml'
        if(trim(key)=='typeOfLevel' .and. trim(value)=='isobaricInhPa')         typ_niv='Pres'
        if(trim(key)=='typeOfLevel' .and. trim(value)=='surface')               typ_niv='surf'
        if(trim(key)=='typeOfLevel' .and. trim(value)=='heightAboveGround')     typ_niv='heig'

        if(trim(key)=='shortName' .and.   trim(value)=='o3')    field_api='o3'
        if(trim(key)=='shortName' .and.   trim(value)=='q')     field_api='q'
        if(trim(key)=='shortName' .and.   trim(value)=='t')     field_api='t'
        if(trim(key)=='shortName' .and.   trim(value)=='r')     field_api='rh'
        if(trim(key)=='shortName' .and.   trim(value)=='2t')    field_api='t2m'
        if(trim(key)=='shortName' .and.   trim(value)=='2d')    field_api='td2m'
        if(trim(key)=='shortName' .and.   trim(value)=='h')     field_api='alt'
        if(trim(key)=='shortName' .and.   trim(value)=='lsm')   field_api='tm'
        if(trim(key)=='shortName' .and.   trim(value)=='tcwv')  field_api='tcwv'
        if(trim(key)=='shortName' .and.   trim(value)=='tcc')   field_api='tcc'
        if(trim(key)=='shortName' .and.   trim(value)=='sp')    field_api='psol'
        if(trim(key)=='shortName' .and.   trim(value)=='skt')   field_api='ts'
        if(trim(key)=='shortName' .and.   trim(value)=='sr')    field_api='alt'
        if(trim(key)=='shortName' .and.   trim(value)=='sd')    field_api='sd'
        if(trim(key)=='shortName' .and.   trim(value)=='ci')    field_api='ci'

        if(trim(key)=='level' ) call niveau ( value , niv)    
     end do
     
     if(idbg%info) write(*,*) "find field_api:",field_api
     
     call grib_get (igrib,'numberOfPoints',NbOfPoints)
     if ( NbOfPoints .ne. bg%nbp * bg%nbl ) then
        write(*,*) "ERROR Grib inconsistency in number of points for ",field_api
        write(*,*) NbOfPoints,bg%nbp * bg%nbl
        STOP "ERROR Grib inconsistency in number of points"
     endif
    
     if(typ_niv=='ml') bg % mltype='ml'
     bg % origine = centre

     allocate(lats(NbOfPoints), lons(NbOfPoints), values(NbOfPoints))
 
     call grib_get_data (igrib,lats,lons,values)
     
     if ( lats(1) .ne. bg%lat1  .and.  lons(1) .ne. bg%lon1  .and. &
          lats(NbOfPoints) .ne. bg%latend  .and.  lons(NbOfPoints) .ne. bg%lonend ) then
           write(*,*) "ERROR Grib inconsistency in lat/lon for ",field_api
           write(*,*) NbOfPoints, bg%nbp * bg%nbl
           write(*,*) lats(1), bg%lat1, lons(1), bg%lon1
           write(*,*) lats(NbOfPoints), bg%latend, lons(NbOfPoints), bg%lonend
           write(*,'(12a12)') info_key(1:n)
           write(*,'(12a12)') info_val(1:n), field_api, typ_niv
           write(*,'(3a5,2i6)') centre, typ_niv, field_api, niv
           STOP "ERROR Grib inconsistency in lat/lon"
     endif 
     
     call mk_NWP_field  ( lats, NbOfPoints, nbp, nbl, 1., bg%lat )
     call mk_NWP_field  ( lons, NbOfPoints, nbp, nbl, 1., bg%lon )
     if(idbg%debug ) then
           write(*,*) NbOfPoints,bg%nbp * bg%nbl
           write(*,*) lats(1), bg%lat1, lons(1), bg%lon1
           write(*,*) lats(NbOfPoints), bg%latend, lons(NbOfPoints), bg%lonend
           write(*,'(12a12)') info_key(1:n)
           write(*,'(12a12)') info_val(1:n), field_api, typ_niv
           write(*,'(3a5,2i6)') centre, typ_niv, field_api, niv
     endif
     if(idbg%info .and. field_api /='xxxx') write(*,'(3a5,2i6)') centre, typ_niv, field_api, niv
     
     ! define altitude pressures
     if((field_api == 't' .or. field_api == 'rh' .or. field_api == 'q') .and. typ_niv=='Pres') then
        found_p=.false.
        do k=1,np
          if(pressions(k) == niv) found_p=.true.
        enddo
        if (.not. found_p ) then
           np=np+1
           pressions(np) =niv
        endif
     endif
 
     if(field_api ==  't2m' ) then              ! t2m 
        ch_t2m=.true.
        scale_data=1.
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, bg%t2m )
     endif

     if(field_api=='ts' .or. (typ_niv=='surf' .and. field_api=='t')) then               ! Ts 
        ch_ts=.true.
        scale_data=1.
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, bg%ts )
     endif

     if(field_api ==  'alt' ) then              ! altitude (Surface roughness) 
        bg%ch_alt=.true.
        scale_data=1.
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, bg_alt )
        bg%alt = bg_alt
        bg % alt_origine = centre
     endif

     if(field_api ==  'tm' ) then               ! land/sea
        bg%ch_tm=.true.
        scale_data=1.
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, bg_tm )
        bg%tm = bg_tm
        bg % tm_origine = centre
     endif

     if(field_api ==  'tcwv' ) then             ! precipitable water vapor (kg/m2)
        ch_cwv=.true.
        scale_data=1.                   
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, bg%cwv )
     endif

     if(field_api == 'psol' ) then              ! psol
        ch_psurf = .true.
        scale_data=1./100.
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, bg%psurf )
     endif

     if(field_api ==  'ci' ) then               ! seaice
        ch_ci=.true.
        scale_data=1.
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, bg%seaice )
     endif
     
     if(field_api ==  'sd' ) then               ! snowdepth 
        ch_sd=.true.
        scale_data=1.
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, bg%snow )
     endif

     if(field_api == 't' .and. typ_niv=='Pres') then    ! temperature on levels (°K)
        scale_data=1.
        do k=1,np
           if(pressions(k) == niv) nivp=k
        end do
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, temp_atmos(nivp,:,:) )
     endif
     if(field_api == 't' .and. typ_niv=='ml') then
        scale_data=1.
        np=np+1
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, temp_atmos(niv,:,:) )
     endif
     
     if( (field_api == 'rh' .or. field_api == 'q') .and. typ_niv=='Pres') then          ! relative humidity on levels (%) or 
        scale_data=1.                                   ! specific humidity on levels (kg/kg)
        humid_type=field_api
        do k=1,np
           if(pressions(k) ==niv) nivp=k
        end do
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, humid_atmos(nivp,:,:) )
     endif
     if((field_api == 'rh' .or. field_api == 'q') .and. typ_niv=='ml') then
        scale_data=1.
        call mk_NWP_field ( values, NbOfPoints, nbp, nbl, scale_data, humid_atmos(niv,:,:) )
     endif

     deallocate(lats, lons, values)

     call grib_keys_iterator_delete (kiter)     !delete the iterator
     call grib_release (igrib)
     call grib_new_from_file (iuforecast, igrib, iret)

  end do
  deallocate(bg_tm,bg_alt )
  ! -----------------------------------------------   
  ! End of the Loop on all the messages in the Grib
  ! -----------------------------------------------   

  ! ml type: compute pressure levels and interpolate on rttov+ mandatory levels.
  ! -------------------------------------------------------------------------
  if( bg % mltype=='ml') then
     pressions(1:n_rttov) = p_rttov(1:n_rttov)
     pressions(n_rttov+1: n_rttov+3) = bg% mandatory_press
     
     allocate(indx(n_rttov+4))  ! the last level is not processed
     call sort_press(pressions(1:n_rttov+4), n_rttov+4, indx, sens)
     bg % nbniv = n_rttov+3   
     
     allocate ( bg % pres (bg % nbniv))        
     allocate ( bg % temp (bg % nbniv,nbp,nbl))            ! temperature (K)
     allocate ( bg % humid(bg % nbniv,nbp,nbl))            ! water vapour (% or kg/kg)
     bg % pres(1: bg % nbniv) = pressions(indx(1:bg % nbniv))
     
     do j=1, nbl
     do i=1, nbp
        if(np==137) then
          call ml2pl_137( bg%psurf(i,j), pres_ml(1:137))
        else
          STOP "ERROR Gribapi: number of ml levels =/ 137"
        endif
        call maia_interp_Plog (pres_ml(1:np), temp_atmos (1:np,i,j),bg%pres,bg%temp(:,i,j) ,np,n_rttov+3)
        call maia_interp_Plog (pres_ml(1:np), humid_atmos(1:np,i,j),bg%pres,bg%humid(:,i,j),np,n_rttov+3)
     end do
     end do
  endif

  ! Pres type: sort the atmospheric fields by increasing pressure order
  ! --------------------------------------------------------
  if( bg % mltype=='Pres') then
     allocate(indx(np))
     call sort_press(pressions(1:np), np, indx, sens)
     bg % nbniv = np

     allocate ( bg % pres (bg % nbniv))        
     allocate ( bg % temp (bg % nbniv,nbp,nbl))            ! temperature (K)
     allocate ( bg % humid(bg % nbniv,nbp,nbl))            ! water vapour (% or kg/kg)
     bg % pres(1: bg % nbniv)      = pressions(indx(1:np))
     
     do k=1, bg % nbniv
        nivp=indx(k)
        bg % temp(k,1:nbp,1:nbl)  = temp_atmos(nivp,1:nbp,1:nbl)
        bg % humid(k,1:nbp,1:nbl) = humid_atmos(nivp,1:nbp,1:nbl)
     enddo  
  endif  
      
  ! test if all atmospheric fields are present below 100hPa
  ! -------------------------------------------------------
  allocate ( temp_ok(bg % nbniv), humid_ok(bg % nbniv) )        
  all_ok=.true.
  do i=1,bg % nbniv
      temp_ok(i) =.true. ; humid_ok(i)=.true.
      if(all(bg % temp(i,:,:) ==vmanq)) temp_ok(i) =.false.
      if(all(bg % humid(i,:,:)==vmanq)) humid_ok(i)=.false.
      if(bg % pres(i)>100) then
         all_ok = all_ok .and. temp_ok(i) .and. humid_ok(i)
      endif 
  end do
   
  if(idbg%info) then
     write(*,'(i5, a9, 200f7.1)') bg % nbniv, ' levels:', bg % pres
     write(*,'(a14, 30L7)') ' temp levels:', temp_ok
     write(*,'(a14, 30L7)')' humid levels:', humid_ok  
     write(*,*) 'fields found psurf:',ch_psurf,', t2m:',ch_t2m,', ts:',ch_ts,', alt:',bg%ch_alt,', tm:',bg%ch_tm,', cwv:',ch_cwv
     write(*,*) 'humid_type: ', humid_type, 'all_ok: ', all_ok
  endif
  deallocate(temp_atmos,humid_atmos,indx)
  
 ! test if mandatory levels for temperature are present: 850, 700, 500 hPa
 ! -----------------------------------------------------------------------
  t_level_ok=.true. ;  h_level_ok=.true.
  do k=bg % nbniv,1,-1
     do kk =1,bg%nmandatory
        if (bg % pres(k) == bg% mandatory_press(kk) ) bg%mandatory_ok=.true.
        if (bg % pres(k) == bg% mandatory_press(kk) .and. .not. temp_ok(k) )  t_level_ok = .false.
        if (bg % pres(k) == bg% mandatory_press(kk) .and. .not. humid_ok(k) ) h_level_ok = .false.
     enddo
  enddo
  all_ok= all_ok .and. t_level_ok .and. h_level_ok .and. all(bg%mandatory_ok)

  if(idbg%info) then
      write(*,*)' mandatory levels :', bg%mandatory_press
      write(*,*) 'mandatory o k:',  bg%mandatory_ok
      write(*,*) 't ok :', t_level_ok  
      write(*,*) 'h ok :', h_level_ok
  endif
  if(.not. all_ok ) then
     stop "incomplete grib file"   
     stop 9
  endif

  if(ch_ci .and. ch_sd) call ml_wt('info','NWP sea-ice and snowdepth available')
      
  ! Computes the total water vapor content if necessary
  ! ---------------------------------------------------
  if(.not. ch_cwv .and. all_ok) then
      if(idbg%info) write(*,*) 'compute the total water content'
      
      tcwv = vmanq
      allocate (rm (bg%nbniv))
      allocate (pres_cal (bg%nbniv))  
      allocate (rm_cal (bg%nbniv))
      do p=1,bg % nbp
        do l=1,bg % nbl

           rm(:)=0.
           do lev=1, bg % nbniv
              if(bg % temp (lev,p,l) > 0 .and. humid_type=='rh') then
                 call maia_hutorm (bg % pres (lev), bg % temp (lev,p,l), bg % humid(lev,p,l), rm(lev))
                 if(rm(lev) /=-1.) rm(lev)= rm(lev) /(1+ rm(lev))           ! specific humidity in g/g
              else
                 rm(lev) = bg % humid(lev,p,l)
              endif
           end do
           
           ik=0
           do k=bg % nbniv,1,-1
              if(rm(k)>vmanq) then
                 ik=ik+1
                 pres_cal(ik) =  bg % pres(k)
                 rm_cal(ik) = rm(k)
              endif
           end do

           call maia_Cal_Twvc (ik, pres_cal(1:ik), bg % psurf (p,l), rm_cal(1:ik), tcwv)

           if(tcwv  >  vmanq) then
              bg % cwv (p,l)= tcwv      !total water vapour content in kg/m2
              ch_cwv=.true.
           endif
        end do
      end do
      if(idbg%debug) write(*,*) 'MAXVAL(tcwv) ', MAXVAL(bg % cwv(:,:))
      deallocate(rm)
      deallocate(pres_cal)
      deallocate(rm_cal)

  end if  ! no precipitable water found

  ! tests if all necessary fields are present
  all_ok = all_ok .and. ch_psurf .and. ch_t2m .and. ch_ts .and. ch_cwv 

  
  if(.not. all_ok) write(*,*) 'maia_Read_GribApi. Not All Ok:'
  if(.not. all_ok .or. idbg%debug) then
      write(*,'(i5, a9, 30f7.1)') bg % nbniv, ' levels:', bg % pres
      write(*,'(a14, 30L7)') ' temp levels :', temp_ok
      write(*,'(a14, 30L7)')' humid levels :', humid_ok
      write(*,*)' mandatory levels :', bg%mandatory_press
      write(*,*) 'mandatory o k:',  bg%mandatory_ok
      write(*,*) 't ok :', t_level_ok  
      write(*,*) 'h ok :', h_level_ok
      write(*,*) 'fields found psurf:',ch_psurf,', t2m:',ch_t2m,', ts:',ch_ts,', alt:',bg%ch_alt,', tm:',bg%ch_tm,', cwv:',ch_cwv
      write(*,*) 'humid_type: ', humid_type
  endif
  
  if(.not. all_ok ) then
    stop "incomplete grib file"   
    stop 9
  endif
  deallocate ( temp_ok, humid_ok)
  deallocate (pressions)

  ! get constant surface elevation / altitude from another NWP
  ! ----------------------------------------------------------
  if (.not. bg% ch_alt .or. .not. bg% ch_tm ) then
      call ml_wt('WARNING', 'NWP Surface alt/tm comes from another NWP')
      call maia_Read_PrevConst(idbg, bg )
  endif

#else

  all_ok = .false.
  STOP "maia_Read_GribApi SHOULD NOT BE CALLED: no GRIB_API or ECCODES library"

#endif

end subroutine maia_Read_GribApi


 subroutine niveau (value, lev)

IMPLICIT NONE
   character(len=256) :: value
   character(len=1) :: niv(9)=(/'1','2','3','4','5','6','7','8','9'/)
   character(len=2) :: str2   
   character(len=3) :: str3   
   integer :: n, lev, num, dec,cen

   if(len(trim(value)) ==1) then
      do n=1,9
         if(trim(value)==niv(n))  lev=n 
      end do
      if(trim(value)=='0') lev=0
   elseif(len(trim(value)) ==2) then
      str2= trim(value)
      do n=1,9
         if(str2(1:1)==niv(n) )  dec=n
         if(str2(2:2)==niv(n) )  num=n
      end do
      if(str2(2:2)=='0') num=0
      lev=dec*10 + num
   elseif(len(trim(value)) ==3) then
      str3= trim(value)
      do n=1,9
         if(str3(1:1)==niv(n) )  cen=n
         if(str3(2:2)==niv(n) )  dec=n
         if(str3(3:3)==niv(n) )  num=n
      end do
      if(str3(2:2)=='0') dec=0
      if(str3(3:3)=='0') num=0
      lev=cen*100+dec*10 + num
   else
      if(trim(value)=='1000') lev=1000
   endif

 end subroutine niveau

subroutine mk_NWP_field (tab, nbval, nbp_bg, nbl_bg, scale_field, field_api)

IMPLICIT NONE

  integer, intent(in)           :: nbval, nbp_bg, nbl_bg
  real, intent(in)              :: scale_field
  real(kind=8), intent(in)      :: tab(nbval)
  real ,        intent(out)     :: field_api(nbp_bg, nbl_bg)    ! T 2m (K)

  INTEGER                       :: i, j, ie, je
  ie = 0
  do i = 1, nbp_bg
     ie = ie+1
     je = 0
     do j = 1, nbl_bg
        je = je +1
        field_api (ie,je) = scale_field*tab(i+(j-1)*nbp_bg) 
     end do
  end do

end subroutine mk_NWP_field

subroutine init_NWP_field (bg)

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

  type( nwp_field),INTENT(inout)  :: bg           ! forecast field information

  bg%lat(:,:) = vmanq
  bg%lon(:,:) = vmanq
!  bg%pres(:)  = vmanq
  bg%ts (:,:) = vmanq
  bg%t2m(:,:) = vmanq
  bg%cwv(:,:) = vmanq
  bg%alt(:,:) = vmanq
  bg%tm (:,:) = vmanq
  bg%seaice(:,:) = vmanq
  bg%snow(:,:) = vmanq
  bg%psurf(:,:)   = vmanq
!  bg%humid(:,:,:) = vmanq
!  bg%temp (:,:,:) = vmanq
  bg%ch_alt = .false.
  bg%ch_tm = .false.
  bg%origine='xx'
  bg%alt_origine='xx'
  bg%tm_origine='xx'
  bg%mandatory_ok =(/.false.,.false.,.false./)

end subroutine init_NWP_field

subroutine maia_hutorm (p, t, hum, hutorm)

! f90 conversion : lydie lavanant       04/06/02

! calcule le rapport de melange pour une humidite relative et
!  une temperature ambiante

IMPLICIT NONE

  REAL, INTENT(IN) :: p, t, hum
  REAL, INTENT(OUT) :: hutorm
  REAL :: esat, esit
  REAL :: e, ew
        
        if( t > 273.15 ) then
          ew = esat(t)
        else
          ew = esit(t)  
        endif
        
        e = hum * ew /100.

        if(p /= e) then
           hutorm = 0.621989 * e / (p -e)
        else
           hutorm = 0.
        endif   
        
END subroutine maia_hutorm

!-----------------------------------------------------------------------

FUNCTION esat(t)

!  goff and gratch formula for h2o saturated partial pressure with
!  respect to liquid h2o

  REAL :: esat
        a=23.8319
        b=2948.964
        c=5.028
        d=29810.16
        e=25.21935
        al=0.0699382
        am=2999.924
        esat=10.**(a-b/t-c*alog10(t)-d*exp(-1.*al*t)+e*exp(-1.*am/t))

END FUNCTION esat

!-----------------------------------------------------------------------
      
FUNCTION esit(t)

!  goff and gratch formula for h2o saturated partial pressure with
!  respect to ice

  REAL :: esit
        a=2.07023
        b=0.00320991
        c=2484.896
        d=3.56654
        esit=10.**(a-b*t-c/t+d*alog10(t))

END FUNCTION esit
subroutine sort_press(x_in, n, indx,sens )

  !ARGUMENTS
  integer, intent(in)   :: n, sens
  real, intent(in )     :: x_in(n)
  integer, intent(out)  :: indx(n)
  
  ! Local variables:
  real          :: x(n), tp, max, min
  integer       :: i, j,idx

  do i=1, n
     indx(i)=i
     x(i)=x_in(i)
  enddo

  if(sens>0) then       !increasing order
  do i=1, n
     min=x(i)
     do j=i, n-1
        if (x(j)< min) then
           tp=x(i)      
           idx=indx(i)
           x(i)=x(j)
           x(j)=tp
           indx(i)=indx(j)
           indx(j)=idx
           min=x(i) 
        endif
     enddo
  enddo
  
  else                  ! decreasing order
  do i=1, n-1
     max=x(i)
     do j=i+1, n
        if (x(j)> max) then
           tp=x(i)      
           idx=indx(i)
           x(i)=x(j)
           x(j)=tp
           indx(i)=indx(j)
           indx(j)=idx
           max=x(i) 
        endif
     enddo
  enddo
  endif

end subroutine sort_press
subroutine ml2pl_91(ps, pres)
        
!http://www.ecmwf.int/products/data/technical/model_levels/model_def_91.html

  real ,  Intent(in) :: ps
  real ,  Intent(out) :: pres(91)
  real :: phalf(91)
  
  real :: a(91)=(/&
        2.000040,3.980832,7.387186,12.908319,21.413612,33.952858,&     
        51.746601,76.167656,108.715561,150.986023,204.637451,271.356506,  &   
        352.824493,450.685791,566.519226,701.813354,857.945801,1036.166504, &    
        1237.585449,1463.163940,1713.709595,1989.874390,2292.155518,2620.898438,&     
        2976.302246,3358.425781,3767.196045,4202.416504,4663.776367,5150.859863, &    
        5663.156250,6199.839355,6759.727051,7341.469727,7942.926270,8564.624023,  &   
        9208.305664,9873.560547,10558.881836,11262.484375,11982.662109,12713.897461, &    
        13453.225586,14192.009766,14922.685547,15638.053711,16329.560547,16990.623047,&
        17613.281250,18191.029297,18716.968750,19184.544922,19587.513672,19919.796875,& 
        20175.394531,20348.916016,20434.158203,20426.218750,20319.011719,20107.031250, &    
        19785.357422,19348.775391,18798.822266,18141.296875,17385.595703,16544.585938, &
        15633.566406,14665.645508,13653.219727,12608.383789,11543.166992,10471.310547,&
        9405.222656,8356.252930,7335.164551,6353.920898,5422.802734,4550.215820,&
        3743.464355,3010.146973,2356.202637,1784.854614,1297.656128,895.193542,&
        576.314148,336.772369,162.043427,54.208336,6.575628,0.003160,&      
        0.000000 /)   
  real :: b(91)=(/&
        0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
        0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
        0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
        0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
        0.000000,0.000000,0.000000,0.000000,0.000000,0.000000,&
        0.000000,0.000000,0.000000,0.000000,0.000014,0.000055,&
        0.000131,0.000279,0.000548,0.001000,0.001701,0.002765,&
        0.004267,0.006322,0.009035,0.012508,0.016860,0.022189,&
        0.028610,0.036227,0.045146,0.055474,0.067316,0.080777,&
        0.095964,0.112979,0.131935,0.152934,0.176091,0.201520,&
        0.229315,0.259554,0.291993,0.326329,0.362203,0.399205,&
        0.436906,0.475016,0.513280,0.551458,0.589317,0.626559,&
        0.662934,0.698224,0.732224,0.764679,0.795385,0.824185,&
        0.850950,0.875518,0.897767,0.917651,0.935157,0.950274,&
        0.963007,0.973466,0.982238,0.989153,0.994204,0.997630,&
        1.000000/)
       
        do i=1,91
          phalf(i) = a(i)/100. + ( b(i) * ps )
        end do

        pres(1)= phalf(1)/2.
        do i=2,91
          pres(i)= (phalf(i) +phalf(i-1))/2.
        end do
        
        return
        end

subroutine ml2pl_137(ps, pres)
        
!http://www.ecmwf.int/products/data/technical/model_levels/model_def_91.html

  real ,  Intent(in) :: ps
  real ,  Intent(out) :: pres(137)
  real :: phalf(137)
  
  real :: a(137)=(/&
        2.000365,     3.102241,     4.666084,     6.827977,     9.746966,    13.605424,    18.608931,    24.985718,    32.985710, &
       42.879242,    54.955463,    69.520576,    86.895882,   107.415741,   131.425507,   159.279404,   191.338562,   227.968948, &     
      269.539581,   316.420746,   368.982361,   427.592499,   492.616028,   564.413452,   643.339905,   729.744141,   823.967834, &     
      926.344910,  1037.201172,  1156.853638,  1285.610352,  1423.770142,  1571.622925,  1729.448975,  1897.519287,  2076.095947, &     
     2265.431641,  2465.770508,  2677.348145,  2900.391357,  3135.119385,  3381.743652,  3640.468262,  3911.490479,  4194.930664, &  
     4490.817383,  4799.149414,  5119.895020,  5452.990723,  5798.344727,  6156.074219,  6526.946777,  6911.870605,  7311.869141, &    
     7727.412109,  8159.354004,  8608.525391,  9076.400391,  9562.682617, 10065.978516, 10584.631836, 11116.662109, 11660.067383, &
    12211.547852, 12766.873047, 13324.668945, 13881.331055, 14432.139648, 14975.615234, 15508.256836, 16026.115234, 16527.322266,&
    17008.789062, 17467.613281, 17901.621094, 18308.433594, 18685.718750, 19031.289062, 19343.511719, 19620.042969, 19859.390625,&
    20059.931641, 20219.664062, 20337.863281, 20412.308594, 20442.078125, 20425.718750, 20361.816406, 20249.511719, 20087.085938, &
    19874.025391, 19608.572266, 19290.226562, 18917.460938, 18489.707031, 18006.925781, 17471.839844, 16888.687500, 16262.046875, &
    15596.695312, 14898.453125, 14173.324219, 13427.769531, 12668.257812, 11901.339844, 11133.304688, 10370.175781,  9617.515625,&
     8880.453125,  8163.375000,  7470.343750,  6804.421875,  6168.531250,  5564.382812,  4993.796875,  4457.375000,  3955.960938,&
     3489.234375,  3057.265625,  2659.140625,  2294.242188,  1961.500000,  1659.476562,  1387.546875,  1143.250000,   926.507812,&
      734.992188,   568.062500,   424.414062,    302.476562,  202.484375,   122.101562,    62.781250,    22.835938,     3.757813,&
        0.000000 ,     0.000000 /)  
        
  real :: b(137)=(/&
        0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,&
        0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,&
        0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,&
        0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,&
        0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000000,&
        0.000000, 0.000000, 0.000000, 0.000000, 0.000000, 0.000007, 0.000024, 0.000059, 0.000112, 0.000199,&
        0.000340, 0.000562, 0.000890, 0.001353, 0.001992, 0.002857, 0.003971, 0.005378, 0.007133, 0.009261,&
        0.011806, 0.014816, 0.018318, 0.022355, 0.026964, 0.032176, 0.038026, 0.044548, 0.051773, 0.059728,& 
        0.068448, 0.077958, 0.088286, 0.099462, 0.111505, 0.124448, 0.138313, 0.153125, 0.168910, 0.185689,& 
        0.203491, 0.222333, 0.242244, 0.263242, 0.285354, 0.308598, 0.332939, 0.358254, 0.384363, 0.411125,& 
        0.438391, 0.466003, 0.493800, 0.521619, 0.549301, 0.576692, 0.603648, 0.630036, 0.655736, 0.680643,&
        0.704669, 0.727739, 0.749797, 0.770798, 0.790717, 0.809536, 0.827256, 0.843881, 0.859432, 0.873929,&  
        0.887408, 0.899900, 0.911448, 0.922096, 0.931881, 0.940860, 0.949064, 0.956550, 0.963352, 0.969513,& 
        0.975078, 0.980072, 0.984542, 0.988500, 0.991984, 0.995003, 0.997630, 1.000000/)
       
        do i=1,137
          phalf(i) = a(i)/100. + ( b(i) * ps )
        end do

        pres(1)= phalf(1)/2.
        do i=2,137
          pres(i)= (phalf(i) +phalf(i-1))/2.
        end do
        
        return
        end
