! Name: $Id: maia_Read_Prev.F90 530 2017-06-27 11:07:32Z frna $
!
! Purpose: 
!          Subroutines maia_Read_Prev 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    01/12/2014   Lydie Lavanant maia4 version 4.2
!  1.2    01/05/2015   Pascale Roquet bug fixes
!  1.3    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.4    01/06/2017   Pascale Roquet eccodes compatibility
subroutine maia_Read_Prev (idbg, filename, bg )
      
! Authors       : lydie lavanant

! calls lec_grib_api to read the forecast fields:
!       the air 2m temperature ,
!       the surface pressure + altitude and the temperature+humidity profile
! computes the total water vapor content from information of module mod_forecast
! returns the arrays bg_t2m and bg_wv and all relative information in module mod_atlas
! unit for T is K and for WV in g/cm2*100

#ifdef HAS_LIBECCODES
USE eccodes
#elif HAS_LIBGRIB_API
USE grib_api
#endif

USE MOD_MAIA_CONST
USE MOD_maia_types

IMPLICIT NONE

  type( debug ),       INTENT(in)       :: idbg         ! 
  character (len=11),  intent(in)       :: filename
  type( nwp_field),    INTENT(out)      :: bg   ! forecast field


#if defined(HAS_LIBGRIB_API) || defined(HAS_LIBECCODES)

  INTEGER :: iuforecast         ! logical unit for forecast file
  INTEGER :: ios
  LOGICAL :: all_ok

  INTEGER :: p, l, i, j, number
  REAL    :: lat, lon, mean
!=============================================[ instructions ]==========

  if(idbg%info) call ml_wt ('info','maia_Read_Prev '//filename)
  
  bg % exist= .true. 
        
! open  NWP file:
! --------------
  call grib_open_file (iuforecast, filename ,'r', ios)
  if(ios /= 0) then     
     call ml_wt('WARNING','No NWP ',filename)
     bg % exist = .false.
     return
  endif
        
! reads the data
! ---------------
  call maia_Read_GribApi (idbg, iuforecast, bg, all_ok)
  
  if(idbg%debug) then
     write(*,*) 'BG'
     write(*,'(10f8.2)')   bg % temp(1:bg%nbniv,INT(bg%nbp/2),INT(bg%nbl/2))  
     write(*,*)   bg % humid(1:bg%nbniv,INT(bg%nbp/2),INT(bg%nbl/2))    ! water vapour (ppmv)
     write(*,*)   bg % ts(INT(bg%nbp/2),INT(bg%nbl/2))  ! surface temperature(K)
     write(*,*)   bg % t2m(INT(bg%nbp/2),INT(bg%nbl/2)) ! temperature 2m (K)
     write(*,*)   bg % cwv(INT(bg%nbp/2),INT(bg%nbl/2)) ! total water vapor content (g/cm2)
     write(*,*)   bg % psurf(INT(bg%nbp/2),INT(bg%nbl/2))       ! psurf (hpa)
     write(*,*)   bg % alt(INT(bg%nbp/2),INT(bg%nbl/2)) ! surface roughness (m)
     write(*,*)   bg % tm(INT(bg%nbp/2),INT(bg%nbl/2))  ! 
  endif

! close file:
! ----------
  call grib_close_file (iuforecast)

! etalement de la ts terre vers la mer
! pour diminuer contraste sst/lst
! ------------------------------------
  do l=2,bg % nbl -1
  do p=2,bg % nbp -1
     ! sur mer en bord de cote
     if (bg%tm(p,l) == 0 ) then
        mean = 0.
        number = 0.
        do i=-1,1
        do j=-1,1
           if( bg%tm(p+i,l+j) ==1) then 
                 mean=mean+ bg%t2m(p+i,l+j) 
                 number= number+1
           endif
        end do
        end do
        if (number >= 1) bg%t2m(p,l)= mean / number
      endif
  end do
  end do
  
! ----------------------------------------------------------------------------
  ! some graphics
  !-------------------------    
  if(idbg%debug) then
     do l=1,bg % nbl,1
        lat= bg % lat1 + (l-1) * bg % lat_inc
!        do p=1,bg % nbp,1
!           lon=bg % lon1 + (p-1) * bg % lon_inc
!           call maia_Lon_Norm (.false., 'lon   ', lon)
         do i=1,10
           lon=1.5*(i-5)
           if(lon  <=   0.- bg % lon_inc/2.) lon = lon + 360.
           p= INT(1+ (lon - bg % lon1) /bg % lon_inc)
           
           write(*,'(3f10.3,i5)') lon, lat, 1.*bg % temp (bg%nbniv, p,l), p
!           write(304,'(3f10.3)') lon, lat, 1.*bg % cwv (p,l)
        end do
     end do
  endif
        
#else

   STOP "maia_Read_Prev SHOULD NOT BE CALLED: no GRIB_API or ECCODES library"

#endif

END subroutine maia_Read_Prev

