! Name: $Id: maia_Read_PrevConst.F90 557 2017-10-03 13:12:34Z frna $
!
! Purpose: Read constant fields in NWP
!          Subroutines maia_Read_PrevConst 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/01/2014   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.2    01/06/2017   Pascale Roquet eccodes compatibility
!---------------------------------------------------------------------------  
!> maia_Read_PrevConst subroutine
!
!> @author      lydie lavanant
!> @date        01/11/2014: version 4.0 
!> </dl>
!> @brief       Read the NWP constant parameters in GRIB API format <br>
!> @brief       Units are: Altitude (m) / land/sea  (0=sea 1=land) 
!---------------------------------------------------------------------------  
subroutine  maia_Read_PrevConst (idbg, bg)

#ifdef HAS_LIBECCODES
USE eccodes
#elif HAS_LIBGRIB_API
USE grib_api
#define CODES_SUCCESS 1
#endif

USE MOD_MAIA_CONST
USE MOD_maia_types

IMPLICIT NONE
        
  type( debug ),    INTENT(in)  :: idbg ! 
  type( nwp_field),INTENT(inout):: bg           ! forecast field information

#if defined(HAS_LIBGRIB_API) || defined(HAS_LIBECCODES)

  INTEGER               :: iu   ! logical unit for forecast file
  integer               :: i_grib , iret
  character(len=20)     :: name_space
  integer               :: kiter, status
  INTEGER               :: l,  p, ios

  character(len=256)    :: key, value
  character(len=2)      :: centre
  character(len=4)      :: field_api

  real(kind=8),allocatable :: lats(:),lons(:),values(:)

  integer               :: const_nbl, const_nbp, NbOfPoints
  LOGICAL               :: current_alt, current_tm              ! true if field found
  real, allocatable     :: bg_tm(:,:), bg_alt(:,:), lat(:,:), lon(:,:)
  real                  :: const_lat1, const_lon1, const_lat_inc, const_lon_inc, latend, lonend
  INTEGER               :: ipg, ilg                             ! nearest or lower grid node
  REAL                  :: rpg, rlg, latitude, longitude        ! grid node real value

!=============================================[ instructions ]==========
  name_space='ls'               ! valid name_spaces are ls and mars

  ! open file of NWP Constant fields:
  ! ---------------------------------
  call grib_open_file (iu, 'Bg_Const' ,'r', ios)
  if(ios /= 0) then     
     call ml_wt('WARNING','No NWP ','Bg_Const')
     return
  endif  
  
  ! get the characteristics of the NWP
  ! ------------------------------------   
  call grib_new_from_file (iu, i_grib, iret) 
  call grib_set(i_grib,'missingValue',vmanq)

  call grib_get(i_grib, 'latitudeOfFirstGridPointInDegrees', const_lat1 ) 
  call grib_get(i_grib, 'longitudeOfFirstGridPointInDegrees',const_lon1 ) 
  call grib_get(i_grib, 'latitudeOfLastGridPointInDegrees', latend) 
  call grib_get(i_grib, 'longitudeOfLastGridPointInDegrees',lonend) 
  call grib_get(i_grib, 'numberOfPointsAlongAParallel', const_nbp)
  call grib_get(i_grib, 'numberOfPointsAlongAMeridian', const_nbl)
  call grib_get(i_grib, 'iDirectionIncrementInDegrees', const_lat_inc)
  call grib_get(i_grib, 'jDirectionIncrementInDegrees', const_lon_inc)

  if(latend < const_lat1 .and. const_lat_inc>0) const_lat_inc = -const_lat_inc

  if(idbg%info) then
     write(*,*) 'NWP Const lat1, lon1=', const_lat1,const_lon1,const_lat_inc,const_lon_inc,const_nbl,const_nbp
  endif

  allocate ( lat(const_nbp,const_nbl), lon(const_nbp,const_nbl))        
  allocate ( bg_alt(const_nbp,const_nbl), bg_tm(const_nbp,const_nbl) )
  current_alt=.false. ; current_tm=.false.

  ! Loop on all the messages in the file
  ! ------------------------------------   
  DO WHILE (iret /= GRIB_END_OF_FILE)

     call grib_keys_iterator_new (i_grib, kiter, name_space)    !create a new iterator
      
     field_api='xxxx'
     do
        call grib_keys_iterator_next( kiter, iret)         
        if (iret /= CODES_SUCCESS) then
            exit                !iret=1 if new iterator ;0=il n'y a plus d'elements
        endif
         
        call grib_keys_iterator_get_name ( kiter, key)  ! key=nom du iterator
        call grib_get (i_grib, trim(key), value, status)        ! recupere les caracteristiques du champ

        if(trim(key)=='centre'    .and.   trim(value)=='lfpw')  centre='MF'
        if(trim(key)=='centre'    .and.   trim(value)=='ecmf')  centre='EC'
        if(trim(key)=='shortName' .and.   trim(value)=='h')     field_api='alt'
        if(trim(key)=='shortName' .and.   trim(value)=='lsm')   field_api='tm'

     end do
     if(idbg%info) write(*,*) 'field_api: ', field_api

     call grib_get (i_grib,'numberOfPoints',NbOfPoints)
     allocate(lats(NbOfPoints),lons(NbOfPoints),values(NbOfPoints))
 
     call grib_get_data (i_grib,lats,lons,values)
     call mk_NWP_field  ( lats, NbOfPoints, const_nbp, const_nbl, 1., lat )
     call mk_NWP_field  ( lons, NbOfPoints, const_nbp, const_nbl, 1., lon )
  
     if(field_api ==  'alt' .and. .not. bg% ch_alt) then                !surface elevation 
        current_alt=.true.
        call mk_NWP_field ( values, NbOfPoints, const_nbp, const_nbl, 1., bg_alt )
        bg % alt_origine = centre
     endif

     if(field_api ==  'tm' .and. .not. bg% ch_tm ) then                 ! land/sea
        current_tm=.true.
        call mk_NWP_field ( values, NbOfPoints, const_nbp, const_nbl, 1., bg_tm )
        bg % tm_origine = centre
     endif

     deallocate(lats, lons, values)

     call grib_keys_iterator_delete (kiter)     !delete the iterator
     call grib_release (i_grib)
     call grib_new_from_file (iu, i_grib, iret)

  end do
    
  ! close file:
  ! ----------
  call grib_close_file (iu)
    
  ! extract fields on bg grid
  do l=1,bg%nbl
     latitude= bg % lat1 + (l-1) * bg % lat_inc
     do p=1,bg%nbp
        longitude=bg % lon1 + (p-1) * bg % lon_inc
        
        rpg= 1+ (longitude - const_lon1) /const_lon_inc
        rlg= 1+ (latitude  - const_lat1) /const_lat_inc
        ipg = INT(rpg) ; ilg = INT(rlg)
        rpg = rpg-ipg ;  rlg = rlg-ilg
        if(current_alt .and. .not. bg%ch_alt) bg%alt(p,l)= bg_alt(ipg,ilg)
        if(current_tm  .and. .not. bg%ch_tm)  bg%tm(p,l) = bg_tm(ipg,ilg)
     end do
  end do    

  ! tests if all atmospheric fields are present
  if(current_alt .and. .not. bg%ch_alt) bg%ch_alt=.true.
  if(current_tm  .and. .not. bg%ch_tm)  bg%ch_tm =.true.
  if(.not. bg%ch_alt .or. .not. bg%ch_tm) then
      write(*,*) 'ch_alt, ch_tm:', bg%ch_alt, bg%ch_tm
  endif
  
  deallocate(bg_tm, bg_alt, lat, lon)

#else
  STOP "maia_Read_PrevConst SHOULD NOT BE CALLED: no GRIB_API or ECCODES library"

#endif

end subroutine maia_Read_PrevConst
