! Name: $Id: maia_SST.F90 596 2018-11-06 15:03:21Z frna $
!
! Purpose: 
!          Subroutines maia_SST 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/06/2013   Lydie Lavanant
!  1.1    20/05/2016   Lydie Lavanant maia 4 version 4.3
!  1.2    07/06/2017   Pascale Roquet update formulas for NOAA and METOP
!  1.3    28/09/2017   Pascale Roquet suppress write first computed SST
!  1.4    11/01/2018   Pascale Roquet update for JPSS
!  1.5    05/11/2018   Pascale Roquet update for Metop-C

subroutine maia_SST (idbg, pix_id, pix, TSclim, SST)

! Authors       : pierre leborgne, lydie lavanant

! computes the sea skin surface temperature tempsurfm  SST in K

USE MOD_MAIA_CONST
USE MOD_maia_types
IMPLICIT NONE

! input/output
  type( debug ),   INTENT(in)   :: idbg         ! 
  type( pix_info), INTENT(in)   :: pix_id       ! 
  type( pix_data), INTENT(in)   :: pix          ! 
  REAL, INTENT(in)              :: TSclim       ! climatological value of SST (K)  
  REAL, INTENT(out)             :: SST
  
  REAL :: steta
  REAL :: tclim
  REAL :: T37, T11, T12
  REAL :: SSTn, SSTd, k
  logical :: pr = .False.

! ---------------------------------------------------------------------------
  SST = vmanq
  
  steta = pix_id % satsec -1
      
 ! change to degrees celcius
  T37= pix % Obs_37 - 273.15
  T11= pix % Obs_108 - 273.15
  T12= pix % Obs_120 - 273.15
  
  if(TSclim> vmanq) then
  
     tclim= TSclim- 273.15

     if ( pix_id % sunzen < 90) then            !day
       ! NL algorithm                   
       call SST_form(pix_id % satimg, "NLC",a_nl, b_nl, c_nl, d_nl, e_nl, f_nl, g_nl, &
                         Tclim, T37, T11, T12, steta, SST,pr)
 
       if( idbg % debug ) then
          write(*,*) '       maia_SST, NL '
          write(*,*) '       coef_SST ', a_nl, b_nl, c_nl, d_nl, e_nl, f_nl, g_nl
          write(*,*) '       T11, T12, steta, SST', T11, T12, steta, SST + 273.15
       endif

     else if ( pix_id % sunzen > 110)        then                !night
       ! T37_1 algorithm
        call SST_form(pix_id %satimg, "T37",a_37, b_37, c_37, d_37, e_37, f_37,0.,&
                         Tclim, T37, T11, T12, steta, SST,pr)
        if( idbg % debug ) then
           write(*,*) '       maia_SST, T37_1 '
           write(*,*) '       coef_SST ', a_37, b_37, c_37, d_37, e_37, f_37
           write(*,*) '       T37, T11, T12, steta, SST', T37, T11, T12, steta, SST  + 273.15          
        endif
     else ! twillight compute SST with the 2 formulas  

      call SST_form(pix_id % satimg, "NLC",a_nl, b_nl, c_nl, d_nl, e_nl, f_nl, g_nl, &
                         Tclim, T37, T11, T12, steta, SSTd, pr)
      call SST_form(pix_id %satimg, "T37",a_37, b_37, c_37, d_37, e_37, f_37,0.,&
                         Tclim, T37, T11, T12, steta, SSTn, pr)
      k = (110 - pix_id %sunzen ) / (110 - 90 )

      SST = k * SSTd + ( 1 - k ) * SSTn 

      if( idbg % debug ) then
           write(*,*) "satimg: ",pix_id % satimg, " imgName: ", pix_id% ImgName ,"night?:", pix_id % night  
           write(*,*) '       maia_SST, NL '
           write(*,*) '       coef_SST ', a_nl, b_nl, c_nl, d_nl, e_nl, f_nl, g_nl
           write(*,*) '       T11, T12, steta, SST', T11, T12, steta, SST + 273.15
           write(*,*) '       maia_SST, T37_1 '
           write(*,*) '       coef_SST ', a_37, b_37, c_37, d_37, e_37, f_37
           write(*,*) '       T37, T11, T12, steta, SST', T37, T11, T12, steta, SST  + 273.15 
       endif
    
     endif
     SST = SST + 273.15
  endif
      
END subroutine maia_SST

subroutine SST_form (sat,algo,a,b,c,d,e,f,g,Tg,T37,T11,T12,S,SST,pr)
! operational SST formulas METEO-FRANCE DIROP/CMS 07/06/2017 from P.LeBorgne,S.Saux-Picart,G.Legendre
! T11s = filtred temperature here same as T11
! Tg : climatological sst
! S = 1 - secante
! Temperature in celcius
implicit none
 CHARACTER (LEN=6), INTENT(in):: sat
 CHARACTER (LEN=3), INTENT(in):: algo
 real, INTENT(in) :: a,b,c,d,e,f,g,Tg,T37,T11,T12,S
 real, INTENT(inout) :: SST
 logical, INTENT(in) :: pr
 LOGICAL, SAVE :: first=.true.
 real :: T3, T4, T5, Tf

! for NOAA16 and NOAA17 formula Tf=Tg value in degre C
! T3 T4 T5 : values in degre K
! output in degre C
 T4 = T11 + 273.15
 T5 = T12 + 273.15
 T3 = T37 + 273.15
 Tf = Tg
if (pr) then
  write(*,*) "inside SST_form"
  write(*,*) a,b,c,d,e,f,g,Tg,T37,T11,T12,S
endif
if (first) then
 call ml_wt("info", "compute SST "//sat//" "//algo)
endif
! METOP01 NLC : (a + b.S)T11 + (c + d.S + e.Tg)(T11s - T12s) + f + g.S
if (algo == "NLC" .and. sat == "meto01") then
  SST = (a + b * S) * T11 + (c + d * S + e * Tg)*(T11 - T12) + f + g * S

! METOP01 T37 : (a + b.S)T37 + (c + d.S)(T11s - T12s) + e + f.S
else if (algo == "T37" .and. sat == "meto01") then
  SST = (a + b * S) * T37 + (c + d * S) * (T11 - T12) + e + f * S

! METOP02 NL  : a.T11 + ( b.S + c.Tg )(T11 - T12) + d + e.S
else if (algo == "NLC" .and. sat == "meto02") then
  SST =  a * T11 + ( b * S + c * Tg )* (T11 - T12) + d + e * S

! METOP02 T37 : (a + b.S)T37 + (c + d.S)(T11s - T12s) + e + f.S
else if (algo == "T37" .and. sat == "meto02") then
  SST =  (a + b*S)*T37 + (c + d*S)*(T11 - T12) + e + f*S

! METOP03 NLC  :(a + b.S)T11 + (c + d.S + e.Tg)(T11s - T12s) + f + g.S
else if (algo == "NLC" .and. sat == "meto03") then
  SST =  (a + b * S) * T11 + (c + d * S + e * Tg)*(T11 - T12) + f + g * S

! METOP03 T37 : (a + b.S)T37 + (c + d.S)(T11s - T12s) + e + f.S
else if (algo == "T37" .and. sat == "meto03") then
  SST =  (a + b*S)*T37 + (c + d*S)*(T11 - T12) + e + f*S

! NPP and JPSS NL      : (a + b.S)T11 + (c + d.S + e.Tg)(T11s - T12s) + f + g.S
else if (algo == "NLC" .and. ( sat == "npp" .or. sat(1:4) == "jpss")) then
 SST = (a + b*S)*T11 + (c + d*S + e*Tg)*(T11 - T12) + f + g*S  

! NPP and JPSS T37     : (a + b.S)T37 + (c + d.S)(T11s - T12s) + e + f.S
else if (algo == "T37" .and. ( sat == "npp" .or. sat(1:4) == "jpss") ) then
  SST = (a + b * S)*T37 + (c + d*S)*(T11 - T12) + e + f*S

! NOAA18 NL   :  a.T11 + ( b.S + c.Tg )(T11 - T12) + d + e.S
else if (algo == "NLC" .and. sat == "noaa18") then
 SST =  a * T11 + ( b * S + c *Tg )*(T11 - T12) + d + e*S

! NOAA18 T37  : (a + b.S)T37 + (c + d.S)(T11 - T12) + e + f.S
else if (algo == "T37" .and. sat == "noaa18") then
  SST = (a + b * S)*T37 + (c + d*S)*(T11 - T12) + e + f*S

! NOAA19 NL   :  a.T11 + ( b.S + c.Tg )(T11 - T12) + d + e.S
else if (algo == "NLC" .and. sat == "noaa19") then
   SST =a * T11 + ( b * S + c *Tg )*(T11 - T12) + d + e*S

! NOAA19 T37  : (a + b.S)T37 + (c + d.S)(T11 - T12) + e + f.S
else if (algo == "T37" .and. sat == "noaa19") then
   SST = (a + b * S)*T37 + (c + d*S)*(T11 - T12) + e + f*S

! NOAA17 NL : .9404T4 + 0.0838Tf(T4-T5) + 1.1098(T4-T5)(sec(0)-1)-255.1277
else if (algo == "NLC" .and. sat == "noaa17") then
 SST = 0.9404 * T4 + 0.0838*Tf * (T4-T5) + 1.1098* (T4-T5) * S -255.1277

! NOAA17 T37 : .9684T4+.0334Tf(T3-T5) + 1.9245(sec(0)-1) -262.5276
else if (algo == "T37" .and. sat == "noaa17") then
 SST = 0.9684 * T4 + 0.0334 * Tf * (T3 -T5) + 1.9245 * S -262.5276

! NOAA16 NL : .9128T4 + 0.0803Tf(T4-T5)+.7709(T4-T5)(sec(0)-1)-247.7182
else if (algo == "NLC" .and. sat == "noaa16") then
 SST =  0.9128 * T4 + 0.0803 * Tf * (T4-T5) + 0.7709 * (T4-T5) * S - 247.7182

! NOAA16 T37 : 0.9622T4 + 0.0336Tf(T3-T5) + 1.6073(sec(0)-1) -261.3291
else if (algo == "T37" .and. sat == "noaa16") then
  SST = 0.9622 * T4 + 0.0336 * Tf * (T3-T5) + 1.6073* S -261.3291

else
   SST = -9999.
endif
if (first) then
 if (pr) then
   write(*,*) "SST=", SST
 endif
 first = .false.
endif

return 
end subroutine SST_form
