! Name: $Id: mod_maia_wrapper.F90 553 2017-09-29 12:34:44Z frna $
!
! Purpose: 
!          Subroutines maia 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    01/04/2017   Pascale Roquet
!  1.1    25/08/2017   Pascale Roquet  maia_par is an array of maia_param type structure
!  1.2    29/09/2017   Pascale Roquet change  #ifdef HAS_LIBHDF5
module mod_maia_wrapper

use mod_maia_const
use mod_maia_types
implicit none
integer, parameter :: map_box_psize=33
integer, parameter :: map_box_lsize=38

type(maia_param), allocatable ::  map_maia(:,:)

type(maia_param) :: maia_box (map_box_psize, map_box_lsize)

public :: maia_wrapper
contains

SUBROUTINE maia_wrapper( idbg_maia, maia_par, nb_lines, nb_pixels, writefile)

! call MAIA Cloud Detection for AVHRR file 
!-----------------------------------------------------------------------
! read a file nammed avh1c.h5 
! input writefile : logicial write a hdf5 file named maiaCT_out.h5
!                               
! output:
!       maia_par(31, nb_pixels,nb_Lines)    maia4 output mask information 
!       see documentation of maia_par in maia_Fill_0uput.F90
!-----------------------------------------------------------------------

     
 USE MOD_MAIA_CONST
 USE mod_maia_types

 IMPLICIT NONE
   integer, INTENT(in)      :: idbg_maia  ! debug level (0= nodebug, 2 is maximum)
   logical, INTENT(in)                     :: writefile
   type(maia_param), allocatable , INTENT(out)   :: maia_par(:,:)       ! output arguments for MAIA subroutine
   integer, INTENT(out)     :: nb_lines
   integer, INTENT(out)     :: nb_pixels


#ifdef HAS_LIBHDF5
! maia routine arguments
  type( debug )         :: idbg         ! 
  type( box_id )        :: box          ! lat, lon, solar and satellite angles at the center of the box
  type( pix_info)       :: pix_id       ! lat, lon, solar and satellite angles at the pixel
  type( pix_data)       :: pixel        ! viirs observations (albedo in %, Tb in K)
  LOGICAL               :: new_box
  Type (field)          :: field_Avhrr
  Type (field_info)     :: field_id
  type( topo_field)     :: landsea, elev        ! surface topography landsea and elev
                           
! Scalars:
  INTEGER :: il, ip, n,  ll, pp  ! loop variable
           
       
! file names I/O for current and previous granule
  CHARACTER(LEN=13), parameter  :: filename_in     ="avh1c.h5"
  CHARACTER(LEN=13), parameter  :: filename_out    ="maiaCT_out.h5"   

! variables for MAIA param in script
  LOGICAL :: idbg_main
           
! pour test duree execution
  real :: t10, t12 
  integer :: AllocStatus
!---- CODE ------------------------------------------------------------- 
  call cpu_time(t10)
  call ml_wt('INFO','ENTERING maia_avhrr v4')

!---------------------
! RETRIEVAL OF  LAUNCH OPTIONS
!---------------------

  idbg_main     =.false.
  idbg % info   =.false.
  idbg % debug  =.false.
  if(idbg_maia == 1) idbg_main=.true.
  if(idbg_maia >= 1) idbg % info=.true.
  if(idbg_maia == 2) idbg % debug=.true.
  idbg %nbw     =0
  idbg %nbw_bg  =0

  call ml_wti('INFO','idbg_maia',idbg_maia)
      
         
!--------------------------------
  ! lecture des champs data AVHRR
  write(*,*)   '  --------------------------------------------------------------------'
  call ml_wt('INFO','Read AVHRR field')
  call maia_read_Avh1c_hdf5 ( idbg, filename_in, field_Avhrr )

  if(idbg%debug) write(*,*) 'maia_read_Avhrr out'

  !--------------------------------
  ! analyse des champs data
  field_Avhrr% ImgName = 'avhrr'
  call maia_Analyse_field (idbg, field_Avhrr, field_id )

  write(*,*) '  AvhrrField                 :', field_id % mission
  write(*,*) '  Nb_Lines                   :', field_Avhrr % Nb_Lines
  write(*,*) '  Date                       :', field_Avhrr % ij(1), field_Avhrr % ij(field_Avhrr % Nb_Lines)
  write(*,*) '  LatMin LatMax LonMin LonMax:', field_id % LatM_min, field_id % LatM_max, &
                                               field_id % LonM_min, field_id % LonM_max
  write(*,*) '  sunzenMin Max satzenMin Max:', MINVAL(field_Avhrr % sunzen),MAXVAL(field_Avhrr % sunzen), &
                                               MINVAL(field_Avhrr % satzen),MAXVAL(field_Avhrr % satzen)
  write(*,*) '  MAXVAL CHAN 1 2 3 4 5 6    :', MAXVAL(field_Avhrr % Obs(1,:,:)), MAXVAL(field_Avhrr % Obs(2,:,:)), &
                                               MAXVAL(field_Avhrr % Obs(3,:,:)), MAXVAL(field_Avhrr % Obs(4,:,:)), &
                                               MAXVAL(field_Avhrr % Obs(5,:,:)), MAXVAL(field_Avhrr % Obs(6,:,:))
  write(*,*) '  nbLat_box, nbLon_box       :', field_id % nbLat_box, field_id % nbLon_box
  
!-----------------------------
  ! Voisinage
  write(*,*) '  -----------------------------------------------------------------------'
  call ml_wt('INFO','Select NEIGHBORS')
  call mk_voisinage (idbg, field_Avhrr)

!-----------------------------
  ! read landsea + elevation atlases  
  write(*,*) '  -----------------------------------------------------------------------'
  call ml_wt('INFO','Read landsea & elevation atlases')
  call maia_Read_Topo (idbg, field_id, landsea, elev) 
!-----------------------------
! init output arguments for MAIA subroutinew
  write(*,*) field_Avhrr % Nb_pixels,field_Avhrr % Nb_Lines
  call maia_param_array_init(  maia_par, field_Avhrr % Nb_pixels, field_Avhrr % Nb_Lines,AllocStatus)
  IF (AllocStatus /= 0) then
      write(*,*) "STOP not enough memory"
      stop
  endif 
! ----------------------------------------------------------------------------
  ! boucle sur les boites et les pixels dans les boites
  write(*,*) '  -----------------------------------------------------------------------'


  call ml_wt('INFO','Loop on Pixels')
  do ll=1, field_id % nbLat_box
  if ( MOD(ll,50) == 0 ) call ml_wti('INFO','box_lat number',ll)
  !call ml_wti('INFO','box_lat number',ll)
  do pp=1, field_id % nbLon_box

     if(field_id % nb_fov(pp,ll)>0) then
        new_box=.true.

        call maia_Box_reset ( idbg, box ) 
        call maia_Box_GetTopo ( idbg, pp, ll, field_id, field_Avhrr, landsea, elev, box)

        do n=1, field_id % nb_fov(pp,ll)

           ip=field_id % pix_box(n, pp, ll)
           il=field_id % lig_box(n, pp, ll)
           field_Avhrr % scanqual(il) = 0   
          
           call maia_Pixel_reset (idbg, pix_id, pixel ) 
           call maia_Fill_Input_Avhrr (idbg, ip, il, field_Avhrr, pix_id, pixel, box)
           
           if(pixel % usable) then

              call maia_Texture_FromNeighbors (idbg, ip, il, field_Avhrr, pix_id)

              
              call maia (idbg, new_box, landsea, elev, field_id, box, pix_id, pixel, maia_par(ip,il))
             
              idbg % info=.false. ;  idbg % debug=.false.
              new_box=.false.

           endif        !if(field_Avhrr % Obs(,ip,il)> vmanq)

        end do          !n=1, nb_fov(pp,ll)
     endif              !if(nb_fov(pp,ll)>0)
  end do
  end do
  if (writefile) then
    call ml_wt('INFO','Write Maia HDF5 file')
    call maia_Write_AvhCT_hdf5 (idbg, field_id, field_Avhrr, maia_par, filename_in, filename_out,.true.)
  endif
  nb_lines = field_Avhrr%Nb_Lines
  nb_pixels = field_Avhrr%Nb_pixels  

  call cpu_time(t12)
  call ml_wtr('INFO','TOT_TIME',t12-t10)
#else

  write(0,'("maia_Avhrr.exe has not been built with HDF-5 Fortran interface")')
  write(0,'("Please run configure with --external-libs=-Lxxx/lib -lhdf5hl_fortran")')
  write(0,'(" and --external-includes=-Lxxx/include")')
  write(0,'(" where xxx is the location of the HDF-5 library")')
  
#endif
end subroutine
end module mod_maia_wrapper
