! Name: $Id: map_viirs_adjacency_mod.F90 596 2018-11-06 15:03:21Z frna $
!!
!! Purpose: 
!!
!>   Calculate the nearest VIIRS pixels for a given one, taking care of the
!!   tie-bow pixel trim, and Earth rotation between scans.
!!   The algorithm is described in : 
!!      Joint Polar Satellite System (JPSS)
!!      Operational Algorithm Description (OAD)
!!      Document for Common Adjacency Software
!!   available at (as of Jan 2014):
!!     http://npp.gsfc.nasa.gov/sciencedocuments/2013-06/474-00097_OAD_Cmn-Adj_B.pdf
!!
!!   Original ADL algorithm has been enlarged to deal with any number of scans and for 
!!   neighborhood windows of any size.
!!
!! This module is highly inspired of the ADL c++ :
!!    ADL/CMN/Utilities/Adjacency/src/ProCmnAdjTable.cpp
!!
!! @author P. Brunel
!! @date October, 2014
!! 24/08/2015: Portability fix (pgf90): make MYCOSLAT a variable not a parameter.
!! 09/05/2016: Bug fix 
!!
!! Code Description:
!!   FORTRAN 90, following AAPP standards
!!
!! Current Code Owner: NWP SAF
!!
!! Copyright:
!!    This software was developed within the context of
!!    the EUMETSAT Satellite Application Facility on
!!    Numerical Weather Prediction (NWP SAF), under the
!!    Cooperation Agreement dated 25 November 1998, between
!!    EUMETSAT and the Met Office, UK, by one or more partners
!!    within the NWP SAF. The partners in the NWP SAF are
!!    the Met Office, ECMWF, KNMI and MeteoFrance.
!!

MODULE MAP_VIIRS_ADJACENCY_MOD



#include "aapp_throw.h"
USE MAP_VIIRS_ADJACENCY_DEF
USE MAP_VIIRS_CRIS_DEF

#ifdef HAS_LIBHDF5
USE hdf5
USE AAPP_HDF_MOD
#endif

  IMPLICIT NONE
  
  PUBLIC :: MAP_VIIRS_ADJACENCY_NEW
  PUBLIC :: MAP_VIIRS_ADJACENCY_FREE
  PUBLIC :: MAP_VIIRS_ADJACENCY_SAVE
  PUBLIC :: GETADJPIXEL
  PRIVATE
  INTEGER :: ILAT
!  REAL(KIND=8), PARAMETER, DIMENSION(-9000:9000) :: MYCOSLAT = (/ (COS(ILAT*3.141592654d0/180./100.), ILAT= -9000, 9000 ) /) !< Array for fast cosine calculations
  REAL(KIND=8), DIMENSION(-9000:9000) :: MYCOSLAT   !< Array for fast cosine calculations
  LOGICAL :: FIRST_MYCOSLAT=.TRUE.

  CONTAINS

    !> New instance of a adjacency structure
    SUBROUTINE MAP_VIIRS_ADJACENCY_NEW( THIS, LAT, LON, ERR)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS
    REAL, INTENT(IN), TARGET :: LAT(:,:)
    REAL, INTENT(IN), TARGET :: LON(:,:)
    INTEGER,          INTENT(OUT) ::ERR
    
    INTEGER :: NSCANS
    INTEGER :: NROWS
    INTEGER :: NCOLS

TRY

    NROWS = SIZE(LAT,2)
    NCOLS = SIZE(LAT,1)
    ! Take care of I channels
    IF( NCOLS .EQ. 3200 ) THEN
      NSCANS = nrows / 16
    ELSE
      NSCANS = nrows / 32
    ENDIF

    CALL NEW_TABLE(THIS, NSCANS, NROWS, NCOLS, &
            &  LAT, LON, ERR)
    THROW(ERR.NE.0)

    CALL INITTABLE(THIS)  ! see ProCmnAdjTable::initTable

CATCH
    END SUBROUTINE

    !> Frees an adjacency structure instance
    SUBROUTINE MAP_VIIRS_ADJACENCY_FREE( THIS, ERR)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS
    INTEGER,          INTENT(OUT) ::ERR
    
TRY

    CALL ADJTABLE_FREE(THIS,ERR)
    THROW(ERR.NE.0)

CATCH
    END SUBROUTINE

    !> Save an adjacency structure instance to an HDF5 file
    SUBROUTINE MAP_VIIRS_ADJACENCY_SAVE( THIS, F, PATH, ERR)
    IMPLICIT NONE
    TYPE(ADJTABLE),      INTENT(INOUT) :: THIS
    CHARACTER(LEN=*),    INTENT(IN)    :: F
    CHARACTER(LEN=*),    INTENT(IN)    :: PATH
    INTEGER,             INTENT(OUT)   ::ERR
    
    LOGICAL            :: PATHROOT
#ifdef HAS_LIBHDF5
    INTEGER(HID_T)     :: FILE_ID, G_ID
#endif

TRY
    ERR = 0
    PATHROOT = TRIM(PATH) .EQ. "/"

#ifdef HAS_LIBHDF5
    write(*,*) "CALL H5FCREATE_F sur file ",TRIM(F)
    CALL H5FCREATE_F( F, H5F_ACC_TRUNC_F, FILE_ID, ERR )
    THROWM(err.ne.0,"Cannot create File "//TRIM(F))

    IF( PATHROOT ) THEN
      G_ID=FILE_ID
    ELSE
      CALL MKPAR( FILE_ID, PATH, G_ID, ERR )
      THROWM(ERR.NE.0,"CANNOT CREATE GROUP "//TRIM(PATH)//" IN FILE "//TRIM(F))
    ENDIF

    CALL ADJTABLE_WH(THIS,G_ID,ERR, COMPRESS=.FALSE.)
    THROWM(ERR.NE.0,"CANNOT WRITE ADJTABLE IN FILE "//TRIM(F))

    IF(.NOT. PATHROOT) THEN
      CALL H5GCLOSE_F( G_ID, ERR )
      THROWM(ERR.NE.0,"CANNOT CLOSE GROUP "//TRIM(PATH)//" IN FILE "//TRIM(F))
    ENDIF

    CALL H5FCLOSE_F( FILE_ID, ERR )
    THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(F))
    write(*,*) "CALL H5FCLOSE_F sur file ",TRIM(F)

#endif

CATCH
    END SUBROUTINE


    !> Allocate all arrays of an adjacency structure
    !! and calculate the Row and Col size
    SUBROUTINE NEW_TABLE(THIS, NSCANS, NROWS, NCOLS, &
            &  LAT, LON, ERR)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS
    INTEGER, INTENT(IN) :: NSCANS
    INTEGER, INTENT(IN) :: NROWS
    INTEGER, INTENT(IN) :: NCOLS
    REAL, INTENT(IN), TARGET :: LAT(:,:)
    REAL, INTENT(IN), TARGET :: LON(:,:)

    INTEGER,          INTENT(OUT) ::ERR
    
    INTEGER :: COL, REFSCAN, REFROW
TRY
    !>- initialise
    CALL ADJTABLE_INIT(THIS)
    
    THIS%TOTALSCANS     = NSCANS 
    THIS%ROWSPERSCAN    = NROWS / NSCANS
    THIS%COLUMNSPERSCAN = NCOLS 

    IF ( NROWS .NE. NSCANS * THIS%ROWSPERSCAN ) THEN
      ERR=1
      THROWM(ERR.NE.0,"ERROR NEWTABLE")
    ENDIF

    !>- allocate arrays
    ALLOCATE( THIS%TOP( NSCANS, NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)
    ALLOCATE( THIS%BOTTOM( NSCANS, NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)
    ALLOCATE( THIS%PREV( NSCANS, NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)
    ALLOCATE( THIS%PREVCOLOFFSET( NSCANS, NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)
    ALLOCATE( THIS%NEXT( NSCANS, NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)
    ALLOCATE( THIS%NEXTCOLOFFSET( NSCANS, NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)
    ALLOCATE( THIS%PIXELTRIM( NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)
    ALLOCATE( THIS%ROWSIZE( NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)
    ALLOCATE( THIS%COLSIZE( NCOLS ), STAT=ERR )
    THROW(ERR.NE.0)

    ! SIZE IS [PIXELS, LINES] FOR LATITUDE AND OTHER ARRAYS
    ! REVERSE in C CODE
    
    THIS%LAT => LAT 
    THIS%LON => LON 
    !>- reference scan is the middle of the scans
    REFSCAN = THIS%TOTALSCANS / 2 
    !>- reference row is the middle row of RefScan
    REFROW = (REFSCAN * THIS%ROWSPERSCAN) + THIS%ROWSPERSCAN/2

    !>- calculate the row size for each pixel
    DO COL = 1, THIS%COLUMNSPERSCAN
      THIS%rowSize(COL) = GETDISTANCE(THIS, REFROW, COL, REFROW+1, COL) 
    END DO
    ! Empirical scaling of 0.8 (see rowsizecoef variable) 
    ! will be applied in tests below in order to manage correctly 
    ! the middle of the scan

    !>- calculate the column size for each pixel
    DO COL = 1, THIS%COLUMNSPERSCAN-1
      THIS%colSize(COL) = GETDISTANCE(THIS, REFROW, COL, REFROW, COL+1) 
    END DO
    THIS%colSize(THIS%COLUMNSPERSCAN) = THIS%colSize(THIS%COLUMNSPERSCAN-1)

    !>- set the pixel trim according to band M or I
    IF( THIS%COLUMNSPERSCAN .EQ. 3200 ) THEN
      CALL PIXEL_TRIM_INIT_M(THIS, NCOLS)
    ELSE
      CALL PIXEL_TRIM_INIT_I(THIS, NCOLS)
    ENDIF
    
CATCH
    END SUBROUTINE

    !> see ProCmnAdjTable::initTable
    SUBROUTINE INITTABLE(THIS)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS

    INTEGER :: SCAN, COL, SCANFIRSTROW
    INTEGER :: TOPINDEX, BOTTOMINDEX
  
    DO SCAN = 1, THIS%TOTALSCANS

      !>- Calculate the first row of the scan
      SCANFIRSTROW = (SCAN-1) * THIS%ROWSPERSCAN +1
      !>- Loop over each column
      DO COL = 1, THIS%COLUMNSPERSCAN
        !> - Calculate the valid top and bottom row in the scan
        TOPINDEX    = THIS%PIXELTRIM(COL)
        BOTTOMINDEX = THIS%ROWSPERSCAN - 1 - TOPINDEX
        TOPINDEX    = TOPINDEX + SCANFIRSTROW
        BOTTOMINDEX = BOTTOMINDEX + SCANFIRSTROW
      
        THIS%TOP(SCAN,COL) = TOPINDEX
        THIS%BOTTOM(SCAN,COL) = BOTTOMINDEX
      
        !> - calculate the previous and next neighbour
        CALL CALCULATEPREVNEIGHBOR(THIS, TOPINDEX, COL, TOPINDEX, BOTTOMINDEX, SCANFIRSTROW)
        CALL CALCULATENEXTNEIGHBOR(THIS, BOTTOMINDEX, COL, TOPINDEX, BOTTOMINDEX, SCANFIRSTROW)
      END DO
    END DO
    END SUBROUTINE

    !> see ProCmnAdjTable::calculatePrevNeighbor
    SUBROUTINE CALCULATEPREVNEIGHBOR(THIS, AROW, ACOL, ASTART, AEND, ASCANOFFSET)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS
    INTEGER ,       INTENT(IN)    ::  AROW, ACOL, ASTART, AEND, ASCANOFFSET
    
    INTEGER :: ADJROW, ROW, PREVSCANROW
    INTEGER :: SCAN, COLOFFSET
    REAL :: DMIN, DIST

    DMIN = 1.E30
    ADJROW = ASTART ! the closest row index
    
    
    ! verify there is a previous scan
    IF (ASCANOFFSET .GT. 1) THEN
      ! loop over the non-bowtie deleted rows of this column
      DO ROW=ASTART, AEND
        ! calculate the index of the row in the previous scan
        PREVSCANROW = ROW - THIS%ROWSPERSCAN
        ! calculate the distance between the two points
        DIST = GETDISTANCE(THIS, AROW, ACOL, PREVSCANROW, ACOL)
        ! if the distance is closer than previously calculated,
        ! remember the distance and the row
        IF (DIST .LE. DMIN) THEN
          DMIN = DIST 
          ADJROW = PREVSCANROW 
        ENDIF
      END DO
    ENDIF 

    SCAN = (ASCANOFFSET-1) / THIS%ROWSPERSCAN +1
    COLOFFSET = GETCOLOFFSET(THIS, AROW, ACOL, ADJROW, DMIN)

    DO WHILE ( DMIN .LT. THIS%rowSize(ACOL)*THIS%rowsizecoef .AND. &
             & ADJROW.GT.1 )

      ADJROW = ADJROW-1 
      DMIN = GETDISTANCE(THIS, AROW, ACOL, ADJROW, ACOL)
      COLOFFSET = GETCOLOFFSET(THIS, AROW, ACOL, ADJROW, DMIN)

    END DO
    
    THIS%PREV(SCAN,ACOL) = ADJROW
    THIS%PREVCOLOFFSET(SCAN,ACOL) = COLOFFSET


    END SUBROUTINE

    !> see ProCmnAdjTable::calculateNextNeighbor
    SUBROUTINE CALCULATENEXTNEIGHBOR(THIS, AROW, ACOL, ASTART, AEND, ASCANOFFSET)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS
    INTEGER ,       INTENT(IN)    :: AROW, ACOL, ASTART, AEND, ASCANOFFSET
    
    INTEGER :: ADJROW, ROW, NEXTSCANROW
    INTEGER :: SCAN, COLOFFSET
    REAL    :: DMIN, DIST

    DMIN = 1.E30
    ADJROW = ASTART ! the closest row index
    
    ! verify there is another scan
     IF (ASCANOFFSET .LT. ((THIS%TOTALSCANS - 1) * THIS%ROWSPERSCAN +1)) THEN

      ! find the closest pixel in the next scan
      DO ROW=ASTART, AEND
        !calculate the row index in the next scan
        NEXTSCANROW = THIS%ROWSPERSCAN + ROW
        ! calculate the distance between the two points
        DIST = GETDISTANCE(THIS, AROW, ACOL, NEXTSCANROW, ACOL)
        ! if the distance is closer than previously calculated,
        ! remember the distance and the row
        IF (DIST .LE. DMIN) THEN
           DMIN = DIST
           ADJROW = NEXTSCANROW
        ENDIF
      END DO
    END IF

    SCAN = (ASCANOFFSET-1) / THIS%ROWSPERSCAN +1
    COLOFFSET = GETCOLOFFSET(THIS, AROW, ACOL, ADJROW, DMIN)

    DO WHILE ( DMIN .LT. THIS%rowSize(ACOL)*THIS%rowsizecoef .AND. &
            &  ADJROW.LT.(THIS%ROWSPERSCAN * THIS%TOTALSCANS))
    
      ADJROW = ADJROW + 1 
      DMIN = GETDISTANCE(THIS, AROW, ACOL, ADJROW, ACOL)
      COLOFFSET = GETCOLOFFSET(THIS, AROW, ACOL, ADJROW, DMIN)

    END DO

    THIS%NEXT(SCAN,ACOL) = ADJROW
    THIS%NEXTCOLOFFSET(SCAN,ACOL) = COLOFFSET

    END SUBROUTINE

    !> see ProCmnAdjTable::getColOffset
    INTEGER FUNCTION GETCOLOFFSET(THIS, ACURROW, ACURCOL, AADJROW, ACURDIST)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS
    INTEGER ,       INTENT(IN)    :: ACURROW, ACURCOL, AADJROW
    REAL,           INTENT(INOUT) :: ACURDIST

    INTEGER :: OFFSET
    REAL :: DIST, LEFTDIST, RIGHTDIST

    OFFSET = 0
    DIST = ACURDIST 

    ! make sure there is a column to the left
    IF (ACURCOL .GT. 1) THEN
      LEFTDIST = GETDISTANCE(THIS, ACURROW, ACURCOL, AADJROW, ACURCOL - 1)
      IF (LEFTDIST .LT. DIST) THEN
         OFFSET = -1
         DIST = LEFTDIST
      ENDIF
    ENDIF
    
    ! make sure there is a column to the right
    ! NB: a priori erreur dans ProCmnAdjTable.cpp
    IF ( ACURCOL .LT. THIS%COLUMNSPERSCAN ) THEN
      RIGHTDIST = GETDISTANCE(THIS, ACURROW, ACURCOL, AADJROW, ACURCOL + 1) 
      IF (RIGHTDIST .LT. DIST) THEN
        OFFSET = 1
        DIST = RIGHTDIST 
      ENDIF
    ENDIF
    
    ACURDIST = DIST
    GETCOLOFFSET= OFFSET

    END FUNCTION

    !> see ProCmnAdjTable::calcGreatCircleDist.
    !> Returns a distance in metres
    !! Very fast calculation which makes the assumption that
    !! the pixels are close so the Earth is flat. The conversion of
    !! longitude difference to distance is done using a tabulated
    !! cosine of the latitude (1/100 degree resolution)
    !> The difference with great circle angles is about 1/1000
    REAL FUNCTION GETDISTANCE(SELF, AROW1, ACOL1, AROW2, ACOL2)
    IMPLICIT NONE
    REAL(KIND=8)          deg2rad          ! convert degrees to radians
    PARAMETER (deg2rad=3.141592654d0/180.d0)
    REAL(KIND=8)          conv             ! convert to metres
    PARAMETER (conv=deg2rad*6371013.0D0)

    TYPE(ADJTABLE), INTENT(INOUT) :: SELF
    INTEGER,        INTENT(IN)    :: AROW1
    INTEGER,        INTENT(IN)    :: ACOL1
    INTEGER,        INTENT(IN)    :: AROW2
    INTEGER,        INTENT(IN)    :: ACOL2

    REAL(KIND=8) :: LAT1, LON1, LAT2, LON2
    REAL(KIND=8) :: DLAT, DLON
    REAL(KIND=8) :: TEMPO, TEMPO1
    INTEGER :: TEMPO3

    IF( FIRST_MYCOSLAT ) THEN
      DO ILAT=-9000,9000
        MYCOSLAT(ILAT) =  COS(ILAT*3.141592654d0/180./100.)
      ENDDO
      FIRST_MYCOSLAT=.FALSE.
    ENDIF

    ! Take care of lat/lon dimensions
    !    SIZE IS [PIXELS, LINES] FOR LATITUDE AND FOLLOWING ARRAYS
    !      SO [COLS, ROWS]

    ! Following is an approximation which is valid for the VIIRS adjacency purpose
    ! The difference with great circle angles is about 1/1000
    ! For VIIRS adjancency, the distances that we calculate are between 0 and ~20km
    !   the maximum error observed on test cases is about 25 metres which is much less than one pixel
    ! Take care that the angles are kept in degree unit
    ! and the CoSine of the latitude has been tabulated with a precision of 1/100 degree
    ! CPU time for INITTABLE is divided at least by 3 (29sec -> 7sec for 8448 x 3200 VIIRS M data)!
    LAT1 = SELF%LAT(ACOL1, AROW1)
    LON1 = SELF%LON(ACOL1, AROW1)
    LAT2 = SELF%LAT(ACOL2, AROW2)
    LON2 = SELF%LON(ACOL2, AROW2)
    
    DLAT = LAT1 - LAT2
    DLON = ABS(LON1 - LON2)
    TEMPO3 = NINT(LAT1*100)
    if (TEMPO3 > 9000)  THEN
      TEMPO3 = 9000
    ENDIF
    if (TEMPO3 < -9000 ) THEN
      TEMPO3 =-9000
    ENDIF
    TEMPO1 = MYCOSLAT(TEMPO3)
    TEMPO =  (DLON*TEMPO1)**2 + DLAT**2
    GETDISTANCE = SQRT( TEMPO ) * conv

    END FUNCTION

    !> For a given position (aRow, aCol) in the image this function returns
    !! the unique pixel (Row, Col) that should be used
    !> Recursive function
    RECURSIVE LOGICAL FUNCTION GETADJPIXEL(THIS, AROW, ACOL, SCAN, ROW, COL) RESULT(GAP)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(IN)    :: THIS
    INTEGER,        INTENT(IN)    :: AROW, ACOL
    INTEGER,        INTENT(IN)    :: SCAN
    INTEGER,        INTENT(INOUT) :: ROW, COL

    INTEGER :: adjRow, adjCol, nrows
    INTEGER :: larow, lacol
    
    adjRow = aRow 
    adjCol = aCol 

    nrows = This%rowsPerScan * this%totalScans
  
    if ( (adjCol .LE. 0) .OR. (adjCol .GT. this%columnsPerScan)) THEN
      !write(*,*) "hors colonne"
      goto 888
    END IF
   
    if ( (scan .le. 0) .or. (scan .gt. this%totalScans ) ) THEN
      !write(*,*) "hors scan"
      goto 888
    endif

    if (adjRow .LT. this%top(scan,adjCol)) then
      adjRow =  1 + adjRow - this%top(scan,adjCol) + this%prev(scan,adjCol)
      if ((adjRow .LE. 0) .OR. (adjRow .GT. nrows)) then
          goto 888
      endif
      adjCol = adjCol + this%prevColOffset(scan,adjCol)

      if ( (adjCol .LE. 0) .OR. (adjCol .GT. this%columnsPerScan)) then
        goto 888
      endif
!pb
      if ( scan .eq. 1) then
        goto 888
      endif
      
      if ( adjRow .LT. this%top(scan-1,adjCol) ) then
        laRow = adjRow 
        laCol = adjCol 
        if ( .not. getAdjPixel(this, laRow, laCol, scan-1, adjRow, adjCol) ) then
          goto 888
        endif
      endif
  
    else if ( adjRow .GT. this%bottom(scan,adjCol) ) Then
      !  NB: il y a a priori une erreur dans l'OAD
      adjRow = adjRow - this%bottom(scan,adjCol) - 1 + this%next(scan,adjCol)
      if ((adjRow .LE. 0) .OR. (adjRow .GT. nrows)) THEN    
        goto 888
      endif
      adjCol = adjCol + this%nextColOffset(scan,adjCol)
      if ( (adjCol .LE. 0) .OR. (adjCol .GT. this%columnsPerScan)) then    
        goto 888
      endif
!pb
      if ( scan .eq. this%totalScans) then
        goto 888
      endif

      if (adjRow .GT. this%bottom(scan+1,adjCol)) then
        laRow = adjRow 
        laCol = adjCol 
        if ( .NOT. getAdjPixel(this, laRow, laCol, scan+1, adjRow, adjCol) ) then      
          goto 888
        endif
      endif
    endif

    if ((adjRow .LE. 0) .OR. (adjRow .GT. nrows)) then
      goto 888
    endif
    if ( (adjCol .LE. 0) .OR. (adjCol .GT. this%columnsPerScan)) then
      goto 888
    endif

    row = adjRow 
    col = adjCol 
    GAP = .true.
    return
  
888 CONTINUE   

    !// fprintf(stderr,"out\n");
    row = -1
    col = -1
    
    GAP= .false. 
    return

    end function

    !> Pixel trim for VIIRS M band
    SUBROUTINE PIXEL_TRIM_INIT_M(THIS, NCOLS)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS
    INTEGER,        INTENT(IN)    :: NCOLS
    
    INTEGER :: IVALUE, COL

! Less than replaced by less or equal; col limits unchanged
    DO COL = 1, NCOLS
        IF( COL .LE. 640 ) THEN
            IVALUE = 2
        ELSEIF( COL .LE. 1008 ) THEN
            IVALUE = 1
        ELSEIF( COL .LE. 2192 ) THEN
            IVALUE = 0
        ELSEIF( COL .LE. 2560 ) THEN
            IVALUE = 1
        ELSE   !( COL .LE. 3200 ) THEN
            IVALUE = 2
        ENDIF
        THIS%PIXELTRIM(COL) = IVALUE
    END DO
    
    END SUBROUTINE

    !> Pixel trim for VIIRS I band
    SUBROUTINE PIXEL_TRIM_INIT_I(THIS, NCOLS)
    IMPLICIT NONE
    TYPE(ADJTABLE), INTENT(INOUT) :: THIS
    INTEGER,        INTENT(IN)    :: NCOLS
    
    INTEGER :: IVALUE, COL

! Less than replaced by less or equal; col limits unchanged
    DO COL = 1, NCOLS
        IF( COL .LE. 1280 ) THEN
            IVALUE = 4
        ELSEIF( COL .LE. 2016 ) THEN
            IVALUE = 2
        ELSEIF( COL .LE. 4384 ) THEN
            IVALUE = 0
        ELSEIF( COL .LE. 5120 ) THEN
            IVALUE = 2
        ELSE   !( COL .LE. 6400 ) THEN
            IVALUE = 4
        ENDIF
        THIS%PIXELTRIM(COL) = IVALUE
    END DO

    END SUBROUTINE

    !> Free a Adjtable structure
    subroutine adjtable_free(x,err)
    integer, parameter :: nlen = 256   ! name length
    type(adjtable),intent(inout)::x
    integer,intent(out)::err
    character(len=nlen)::sname
TRY
    err = 0
    
!    sname='lat'
!    if(associated(x%lat))then
!      deallocate(x%lat,stat=err)
!      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
!      nullify(x%lat)
!    endif
!    sname='lon'
!    if(associated(x%lon))then
!      deallocate(x%lon,stat=err)
!      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
!      nullify(x%lon)
!    endif

! lat/lon pointers are not allocated, see new_table
    nullify(x%lat)
    nullify(x%lon)
    sname='pixelTrim'
    if(associated(x%pixelTrim))then
      deallocate(x%pixelTrim,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%pixelTrim)
    endif
    sname='top'
    if(associated(x%top))then
      deallocate(x%top,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%top)
    endif
    sname='bottom'
    if(associated(x%bottom))then
      deallocate(x%bottom,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%bottom)
    endif
    sname='prev'
    if(associated(x%prev))then
      deallocate(x%prev,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%prev)
    endif
    sname='prevColOffset'
    if(associated(x%prevColOffset))then
      deallocate(x%prevColOffset,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%prevColOffset)
    endif
    sname='next'
    if(associated(x%next))then
      deallocate(x%next,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%next)
    endif
    sname='nextColOffset'
    if(associated(x%nextColOffset))then
      deallocate(x%nextColOffset,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%nextColOffset)
    endif
    sname='rowSize'
    if(associated(x%rowSize))then
      deallocate(x%rowSize,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%rowSize)
    endif
    sname='colSize'
    if(associated(x%colSize))then
      deallocate(x%colSize,stat=err)
      THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
      nullify(x%colSize)
    endif

CATCH
    end subroutine

    !> Initialise a Adjtable structure
    subroutine adjtable_init(x)
    type(adjtable),intent(inout)::x
    
      x%totalScans=0
      x%rowsPerScan=0
      x%columnsPerScan=0
      x%rowsizecoef=0.8 ! Empirical factor for tests with rowSize
      nullify(x%lat)
      nullify(x%lon)
      nullify(x%pixelTrim)
      nullify(x%top)
      nullify(x%bottom)
      nullify(x%prev)
      nullify(x%prevColOffset)
      nullify(x%next)
      nullify(x%nextColOffset)
      nullify(x%rowSize)
      nullify(x%colSize)
    end subroutine

    
#ifdef HAS_LIBHDF5
    !> Read a Adjtable structure from an HDF5 file
    subroutine adjtable_rh(x,lun,err)
    integer, parameter :: nlen = 256   ! name length
    type(adjtable),intent(out)::x
    integer(hid_t),intent(in)::lun
    integer,intent(out)::err
    character(len=nlen)::sname
TRY
    
    call adjtable_init(x)
    
    sname='totalScans'
    call read_array_hdf(lun,sname,err,i0=x%totalScans)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='rowsPerScan'
    call read_array_hdf(lun,sname,err,i0=x%rowsPerScan)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='columnsPerScan'
    call read_array_hdf(lun,sname,err,i0=x%columnsPerScan)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='lat'
    call read_array_hdf(lun,sname,err,pr2=x%lat)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='lon'
    call read_array_hdf(lun,sname,err,pr2=x%lon)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='pixelTrim'
    call read_array_hdf(lun,sname,err,pi1=x%pixelTrim)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='top'
    call read_array_hdf(lun,sname,err,pi2=x%top)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='bottom'
    call read_array_hdf(lun,sname,err,pi2=x%bottom)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='prev'
    call read_array_hdf(lun,sname,err,pi2=x%prev)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='prevColOffset'
    call read_array_hdf(lun,sname,err,pi2=x%prevColOffset)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='next'
    call read_array_hdf(lun,sname,err,pi2=x%next)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='nextColOffset'
    call read_array_hdf(lun,sname,err,pi2=x%nextColOffset)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))
    
    sname='rowSize'
    call read_array_hdf(lun,sname,err,pr1=x%rowSize)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))

    sname='rowsizecoef'
    call read_array_hdf(lun,sname,err,r0=x%rowsizecoef)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))

    sname='colSize'
    call read_array_hdf(lun,sname,err,pr1=x%colSize)
    THROWM(err.ne.0,"CANNOT READ "//trim(sname))

    err=0
CATCH
    end subroutine

    !> Write a Adjtable structure from an HDF5 file
    subroutine adjtable_wh(x,lun,err, compress)
    integer, parameter :: nlen = 256   ! name length
    type(adjtable),intent(in)::x
    integer(hid_t),intent(in)::lun
    integer,intent(out)::err
    logical,intent(in),optional::compress
    character(len=nlen)::sname
TRY
    sname='totalScans'
    call write_array_hdf(lun,sname,&
      & 'Number of Scans',&
      & err,i0=x%totalScans  )
    THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    
    sname='rowsPerScan'
    call write_array_hdf(lun,sname,&
      & 'Rows Per Scan',&
      & err,i0=x%rowsPerScan  )
    THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    
    sname='columnsPerScan'
    call write_array_hdf(lun,sname,&
      & 'Columns Per Scan',&
      & err,i0=x%columnsPerScan  )
    THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    
    sname='lat'
    if(associated(x%lat))then
      call write_array_hdf(lun,sname,&
        & 'Latitude of each pixel (positive North) degree ',&
        & err,r2=x%lat , units = 'degrees' ,compress=compress)
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='lon'
    if(associated(x%lon))then
      call write_array_hdf(lun,sname,&
        & 'Longitude of each pixel (positive East) degree',&
        & err,r2=x%lon , units = 'degrees' ,compress=compress)
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='pixelTrim'
    if(associated(x%pixelTrim))then
      call write_array_hdf(lun,sname,&
        & 'Pixel trim cells',&
        & err,i1=x%pixelTrim  )
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='top'
    if(associated(x%top))then
      call write_array_hdf(lun,sname,&
        & 'Top row == minimum row index',&
        & err,i2=x%top  ,compress=compress)
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='bottom'
    if(associated(x%bottom))then
      call write_array_hdf(lun,sname,&
        & 'Bottom row == maximum row index',&
        & err,i2=x%bottom  ,compress=compress)
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='prev'
    if(associated(x%prev))then
      call write_array_hdf(lun,sname,&
        & 'Previous scan row index',&
        & err,i2=x%prev  ,compress=compress)
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='prevColOffset'
    if(associated(x%prevColOffset))then
      call write_array_hdf(lun,sname,&
        & 'Previous column offset',&
        & err,i2=x%prevColOffset  ,compress=compress)
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='next'
    if(associated(x%next))then
      call write_array_hdf(lun,sname,&
        & 'Next scan row index',&
        & err,i2=x%next  ,compress=compress)
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='nextColOffset'
    if(associated(x%nextColOffset))then
      call write_array_hdf(lun,sname,&
        & 'Next column offset',&
        & err,i2=x%nextColOffset  ,compress=compress)
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='rowSize'
    if(associated(x%rowSize))then
      call write_array_hdf(lun,sname,&
        & 'Across scan pixel size',&
        & err,r1=x%rowSize , units = 'm' )
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif
    
    sname='rowsizecoef'
    call write_array_hdf(lun,sname,&
      & 'Empirical factor for tests with rowSize',&
      & err,r0=x%rowsizecoef )
    THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

    sname='colSize'
    if(associated(x%colSize))then
      call write_array_hdf(lun,sname,&
        & 'Along scan pixel size',&
        & err,r1=x%colSize , units = 'm' )
      THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
    endif

    err=0
CATCH
    end subroutine
#endif

END MODULE
