! Name: $Id$
!> The map_viirs_cris_def holds the derived types for the viirs-cris mapping.
!!
!> Derived types:
!>- map_viirs_options
!> -   run time options provided by environment variables.
!>- crisfovaxes
!> -  contains the pre-calculated CrIS field of view axes definition
!>- cfovviirs
!> -  CrIS fov 
!> -  Position in VIIRS array
!> -  Adjacency window
!> -  Axes
!> -  CrIS FOR shape weights normalised to sum = 1
!> -  CrIS Ellipse 
!>- viirs_maia4.
!! holds the MAIA4 data
!>- viirs_location
!> description of the viirs location, latitude, longitudes , bow tie mask and quality flag.
!!
!! VIIRS adjacency algorithm
!!
!! @author 
!! 15/01/2014 P. Brunel
!!!! @author P. Brunel
!! @date October, 2014
!! 10/2018  Add clusters P. Roquet
MODULE MAP_VIIRS_CRIS_DEF

USE viirs_maia4_header_def
USE viirs_maia4_data_def
  IMPLICIT NONE

  INTEGER :: cris1Cunit=18    !< unit number for CRIS level 1C file
  INTEGER :: cris1Dunit=28    !< unit number for CRIS level 1D file
  INTEGER :: crisfunit=55     !< unit number for CRIS FDF file

  INTEGER, PARAMETER :: max_box_radiusl = 50 !< maximum half size of box (M band)
  INTEGER, PARAMETER :: max_box_radiusp = 50 !< maximum half size of box (M band)

  INTEGER, PARAMETER :: CL_NB_CHANNEL = 16 ! for cluster must be equal to SD_BAS_NBK in Nuees.h
  INTEGER, PARAMETER :: CL_NBCLASS = 6 ! for cluster
  INTEGER, PARAMETER :: CL_NBCLASS_TOTAL = 7 ! for cluster
  INTEGER, PARAMETER :: SD_BAS_NPI_IMAGER = 2048 ! must be equal the definition in Nuees.h 
  INTEGER, PARAMETER :: SD_BAS_NLI_MAX_UTIL_IMAGER = 160 ! must be equal the definition in Nuees.h 
  INTEGER, PARAMETER :: CL_MAXLENGH = SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER
  INTEGER, PARAMETER :: CL_NO_CLASSIFIED = 7
  REAL , PARAMETER :: pi            = 3.1415926536
  INTEGER, PARAMETER :: MAX_NB_PIXELS_IN_FOV = 2000*4 

  TYPE viirs_srf_type
    CHARACTER(LEN=3) :: NAME
    REAL :: CWN
    REAL :: EQW_WN
    REAL :: CWL
    REAL :: EQW_WL
    REAL :: SOLAR_IRRADIANCE
    REAL :: RadtoSIfactor ! Factor for radiance unit conversion 
    REAL :: RadtoCRISunitfactor  ! Factor for radiance unit conversion
    REAL :: RefltoRadfactor ! factor for reflectance to radiance conversion
    REAL :: ChannelNoise ! Channel Noise for clusters
    REAL :: ChannelWeight ! Channel Weight for clusters
  END TYPE 

  !> 3 options provided by environment variables
  TYPE map_viirs_options
    CHARACTER(LEN=1) :: VIIRS_BAND   = "M"
    INTEGER          :: DEBUG  = 0
    REAL             :: THRESHOLDGP = 50.0  !<Mapping Threshold percentage of good VIIRS pixels in mapping box or FOV
    LOGICAL          :: DO_CLUSTERS
    INTEGER          :: CLUSTER_SORT_CHANNEL
  END TYPE

  !> The structure which contains the regression coefficients
  !! of the axis lenght with satellite altitude
  TYPE crisfovaxes
     REAL :: refaltitude !< reference altitude (km)
     INTEGER :: viirsm(30,9) = 1600 !< VIIRS-M pixel approximation of CrIS center
     INTEGER :: viirsl(30,9) = 0    !< VIIRS-M line offset approximation of CrIS center
     ! Regression values:
     ! Axis = slope*altitude + intercept
     ! Definition of axis directions
     ! across scan semi-axis 1 = lower pixel numbers
     ! along scan semi-axis 1  = lower line numbers
     REAL :: acslope1(30,9) !< across scan semi-axis 1, slope (km/km)
     REAL :: acinter1(30,9) !< across scan semi-axis 1, intercept (km)
     REAL :: acslope2(30,9) !< across scan semi-axis 2, slope (km/km)
     REAL :: acinter2(30,9) !< across scan semi-axis 2, intercept (km)
     REAL :: alslope1(30,9) !< along  scan semi-axis 1, slope (km/km)
     REAL :: alinter1(30,9) !< along  scan semi-axis 1, intercept (km)
     REAL :: alslope2(30,9) !< along  scan semi-axis 2, slope (km/km)
     REAL :: alinter2(30,9) !< along  scan semi-axis 2, intercept (km)
  END TYPE

  !> All CrIS FOV information and the position and mapping in VIIRS image
  TYPE cfovviirs
     ! CrIS FOV:
     INTEGER :: line
     INTEGER :: for
     INTEGER :: fov
     INTEGER :: scnlinyr     !< scan line year
     INTEGER :: scnlindy     !< scan line day of year
     INTEGER :: scnlintime   !< scan line UTC time of day (ms)
     REAL    :: target_latitude  !< (deg)
     REAL    :: target_longitude !< (deg)
     REAL    :: target_altitude  !<  altitude (km)

     ! Position in VIIRS array
     INTEGER :: vpc    !< VIIRS pixel for center FOV
     INTEGER :: vlc    !< VIIRS line for center FOV
     INTEGER :: lps    !< lines per scan
     INTEGER :: lpm    !< last pixel move
     INTEGER :: llm    !< last line move
     REAL    :: latitude  !< (deg)
     REAL    :: longitude !< (deg)
     REAL    :: altitude  !<  altitude (km)

     INTEGER :: vnl    !< VIIRS number of lines in array
     INTEGER :: vnp    !< VIIRS number of pixels in array

     ! Adjacency window
     INTEGER :: radiusp    !< size in VIIRS pixels
     INTEGER :: radiusl    !< size in VIIRS lines

     ! size of semi-axes
     REAL :: acrossk(2) !< across track axes (km)
     REAL :: acrossp(2) !< across track axes (pixels)
     REAL :: alongk(2)  !< along track axes (km)
     REAL :: alongp(2)  !< along track axes (lines)

     ! VIIRS neighbours
     ! indices of adjacency pixels size (radiusp, raduisl)
     ! negative values if not valid
     !  -1 == tie bow pixel on first rows of firs scan
     !                  or last rows of last scans
     !  -2 == outside scans
     INTEGER, POINTER :: vp(:,:) !< index for VIIRS neighbours pixels 
     INTEGER, POINTER :: vl(:,:) !< index for VIIRS neighbours lines 

     REAL,POINTER ::ellipse(:,:) !< Ellipse 
                               !! <1 inside; =1 ellipse, >1 outside

     REAL, POINTER :: weights(:,:) !< CrIS FOR shape weights normalised to sum = 1
     
     REAL, POINTER :: ellipsep(:) !< CrIS Ellipse list of pixels
     REAL, POINTER :: ellipsel(:) !< CrIS Ellipse list of lines
  END TYPE

  !> small CrIS FOV information and the position and mapping in VIIRS image
  TYPE fov_type
     ! CrIS FOV:
     INTEGER :: line
     INTEGER :: for
     INTEGER :: fov
     INTEGER :: vnl    !< VIIRS number of lines in array
     INTEGER :: vnp    !< VIIRS number of pixels in array
     INTEGER :: vpc    !< VIIRS pixel for center FOV 
     INTEGER :: vlc    !< VIIRS line for center FOV 
     INTEGER :: nb_pixels  !< actual VIIRS number of pixels in ellipse
     INTEGER, DIMENSION(MAX_NB_PIXELS_IN_FOV):: vp !< index for VIIRS neighbours pixels inside fov
     INTEGER, DIMENSION(MAX_NB_PIXELS_IN_FOV) :: vl !< index for VIIRS neighbours lines inside fov    
  END TYPE

TYPE cforviirs
     ! CrIS FOV:
     INTEGER :: line
     INTEGER :: for
     type(fov_type):: fovs (9)
     INTEGER :: scnlinyr     !< scan line year
     INTEGER :: scnlindy     !< scan line day of year
     INTEGER :: scnlintime   !< scan line UTC time of day (ms)
     REAL    :: target_latitude  !< (deg) (= position FOV 5)
     REAL    :: target_longitude !< (deg) (= position FOV 5)
     REAL    :: target_altitude  !<  altitude (km) (= position FOV 5)

     ! Position in VIIRS array (= position FOV 5)
     INTEGER :: vpc    !< VIIRS pixel for center FOV == center of FOV 5
     INTEGER :: vlc    !< VIIRS line for center FOV == center of FOV 5
     INTEGER :: lps    !< lines per scan
     INTEGER :: lpm    !< last pixel move
     INTEGER :: llm    !< last line move
     REAL    :: latitude  !< (deg)
     REAL    :: longitude !< (deg)
     REAL    :: altitude  !<  altitude (km)

     INTEGER :: vnl    !< VIIRS number of lines in array
     INTEGER :: vnp    !< VIIRS number of pixels in array
     
     ! Adjacency window
     INTEGER :: radiusp    !< size in VIIRS pixels
     INTEGER :: radiusl    !< size in VIIRS lines

     INTEGER :: fpv ! first pixel in VIIRS grid
     INTEGER :: flv ! first line in VIIRS grid
     INTEGER :: lpv ! last pixel in VIIRS grid
     INTEGER :: llv ! last line in VIIRS grid

 
     ! VIIRS neighbours
     ! indices of adjacency pixels size (radiusp, raduisl)
     ! negative values if not valid
     !  -1 == tie bow pixel on first rows of firs scan
     !                  or last rows of last scans
     !  -2 == outside scans
     INTEGER, POINTER :: vp(:,:) !< index for VIIRS neighbours pixels 
     INTEGER, POINTER :: vl(:,:) !< index for VIIRS neighbours lines 
  END TYPE


  !> MAIA4 data
  TYPE viirs_maia4
    TYPE(viirs_maia4_header) :: header
    TYPE(viirs_maia4_data)   :: data
    INTEGER                  :: format_version = 0
    LOGICAL                  :: lmaia4 = .FALSE.
  END TYPE

  !> VIIRS geo-location, including the bow tie mask and a quality flag
  type viirs_location
    integer::totalScans !< Number of Scans
    integer::nlines !< Number of lines
    integer::rowsPerScan !< Rows Per Scan
    integer::columnsPerScan !< Columns Per Scan
    real,pointer,dimension(:,:)::latitude !< Latitude of each pixel (positive North) degree 
      !!    size is [pixels, lines]
    real,pointer,dimension(:,:)::longitude !< Longitude of each pixel (positive East) degree
      !!    size is [pixels, lines]
    integer, dimension(-90:90,-180:180) :: gridp = -1 !< 1 degree lat/lon grid; positive if some VIIRS pixel inside
    integer*8,pointer,dimension(:)::UTCJulianDay !< Starting Date of each scan in UTC ( julian day since 1/1/1950) 
    integer*8,pointer,dimension(:)::UTC !< Starting Time of each scan in UTC (millisecond of day) 
    logical,pointer,dimension(:,:)::MaskBowTie !< Flag for Bow-Tie pixels,  True == pixels is part of
    logical,pointer,dimension(:,:)::qual !< location quality flag True=OK
    logical,pointer,dimension(:,:)::pixqual !< logical true if bad quality pixel in one channel
    logical::testqual !< logical true if any bad quality pixel
  end type

  type clusterparameter
   double precision, dimension(CL_NB_CHANNEL) :: Noise
   integer, dimension(CL_NB_CHANNEL) :: Weight
   integer :: Nch
   integer :: SortChan
  end type

  type clusterviirs
    INTEGER :: line
    INTEGER :: for
    INTEGER :: fov
    real, dimension(CL_MAXLENGH * CL_NB_CHANNEL) :: RADVECT ! radiance vector for the cluster method must be in SI 
                                     ! radvect ( SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER) * SD_BAS_NBK) 
    integer :: nb_pixels
    integer :: nblin
    integer :: nbcol
    integer :: nbclass
    integer, dimension (CL_MAXLENGH ) :: IdPixel  ! cluster
    integer, pointer :: cluster_image(:,:) ! CL_MAXLENGH = SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER
    integer, pointer :: mask(:,:) ! nbcol, nblin  
    integer, dimension  (CL_NBCLASS_TOTAL,9) :: RadAnaNbPix !(NClass, 9)    ! number of pixel by class
    real, dimension (CL_NB_CHANNEL,CL_NBCLASS_TOTAL,9) :: RadAnaMean  !(NClass, Nchan, 9)   ! means for each channel/class
    real, dimension (CL_NB_CHANNEL,CL_NBCLASS_TOTAL,9) :: RadAnaStd  !(NClass, NChan, 9) ! std for each channel/class
    integer, pointer :: cluster_on_viirs_grid(:,:) ! cluster_image remapped on VIIRS raw grid
    logical :: first_time


  end type
END MODULE
