! Name: $Id$
!
!> Purpose: Manage VIIRS-CRIS mapping.
!!
!> Current Code Owner: NWP SAF
!!
!> COPYRIGHT.
!!    This software was developed within the context of the EUMETSAT Satellite
!!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!!
!!    Copyright 2014, EUMETSAT, All Rights Reserved.
!!
!! @author P. Brunel
!! @date October, 2014
!! 08/09/2017: MAIA4 format version changed from 2 to 3 (PB/PR)
!! 22/10/2018: Bug fixes and add clusters (P. Roquet) 
!! 05/110/2018: Bug fix (missing data for cluster statistics) (P. Roquet)
MODULE MAP_VIIRS_CRIS_MOD

#ifdef HAS_LIBHDF5

#include "aapp_throw.h"

USE HDF5
USE H5LT
USE AAPP_HDF_MOD
USE MAP_VIIRS_CRIS_DEF
USE MAP_VIIRS_ADJACENCY_DEF
USE MAP_VIIRS_ADJACENCY_MOD


  IMPLICIT NONE
  
INTERFACE
 SUBROUTINE clustersnothread( RadVect, NbPixImg, Nch, SortCh, Noise, Weight, IdPix ,NbClass, ErrCode) BIND(C)
    USE, INTRINSIC :: ISO_C_BINDING
    IMPLICIT NONE
    INTEGER, PARAMETER :: CL_NB_CHANNEL = 16 ! for cluster
    INTEGER, PARAMETER :: CL_NBCLASS = 6 ! for cluster
    INTEGER, PARAMETER :: SD_BAS_NPI_IMAGER = 2048
    INTEGER, PARAMETER :: SD_BAS_NLI_MAX_UTIL_IMAGER = 160
    INTEGER, PARAMETER :: CL_MAXLENGH = SD_BAS_NPI_IMAGER * SD_BAS_NLI_MAX_UTIL_IMAGER
    REAL(C_FLOAT), DIMENSION(CL_MAXLENGH * CL_NB_CHANNEL) :: RadVect
    INTEGER(C_INT) :: NbPixImg, NbClass, ErrCode, Nch, SortCh
    INTEGER(C_INT), DIMENSION (CL_MAXLENGH) :: IdPix
    INTEGER(C_INT), DIMENSION (CL_NB_CHANNEL) :: Weight
    REAL(C_DOUBLE), DIMENSION (CL_NB_CHANNEL) :: Noise
  END SUBROUTINE clustersnothread
END INTERFACE

  INTEGER, PARAMETER :: EL_NMAX=181 ! number of pixels for ellipse contour
  
  TYPE(CRISFOVAXES),save :: CRISAXES
  TYPE(viirs_srf_type) :: VIIRSSRF(22) ! Contains info on VIIRS SRF (16 channels M, 5 channels I, 1 DNB)
  PRIVATE :: crisaxes, VIIRSSRF

  PUBLIC :: MAP_VIIRS_OPTIONS_SETUP
  PUBLIC :: MAP_VIIRS_SETUP
  PUBLIC :: MAP_VIIRS_CRIS_FDF
  PUBLIC :: MAP_UPDATE_CLUSTER_PARAM
  PUBLIC :: MAP_VIIRS_PRINT_OPTIONS
  PUBLIC :: MAP_NEW
  PUBLIC :: MAP_FREE
  PUBLIC :: MAPFOR_FREE
  PUBLIC :: MAP_SAVE
  PUBLIC :: MAP_SAVE_DEBUG
  PUBLIC :: MAP_APPLY
  PUBLIC :: MAP_CONV_SDR_1D_H
  PUBLIC :: MAP_CONV_SDR_1D_D
  PUBLIC :: MAP_CONV_MAIA4
  PUBLIC :: MAP_VIIRS_FREE
  PUBLIC :: MAP_NEAREST_VIIRS
  PUBLIC :: MAP_CRIS_FOV_BOX
  PUBLIC :: MAP_CRIS_FOV_MASK
  PUBLIC :: MAP_VIIRS_SHIFT_RAD
  PUBLIC :: MAP_CLUSTER_SDR
  PUBLIC :: VIIRSLOC_SAVE
  PUBLIC :: CLUSTER_SAVE
  PUBLIC :: CLUSTER_SAVE_OUTPUT
  PUBLIC :: MAP_CRIS_FOR_BOX
  PUBLIC :: MAP_VIIRS_FOR_NEIGHBORHOOD
  PUBLIC :: MAP_FORMAP_SAVE
  PUBLIC :: MAP_CONV_CLUSTER
  PUBLIC :: MAP_PUT_MAPPING_IN_FOV
  PRIVATE

  CONTAINS
    !> Set the VIIRS mapping options.
    !> Options are given by environment variables:
    !>- MAP_VIIRS_CRIS_DEBUG  possible values [0,2] default is 0
    !>- VIIRS_BAND is the VIIRS Band Name [I,M] default is M
    !>- MAP_VIIRS_CRIS_THRESHOLD is the minimum percentage of good VIIRS
    !!     pixels in a mapping box for calculations of means/cloud-masks/etc
    !!     default is 50%
    SUBROUTINE MAP_VIIRS_OPTIONS_SETUP(OPTIONS, ERR)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(OUT) :: OPTIONS
    INTEGER,                 INTENT(OUT) :: ERR

    CHARACTER(LEN=16) :: MYSTRING


TRY
    !> Debug level (default is 0 == no debug)
    CALL GETENV('MAP_VIIRS_CRIS_DEBUG',MYSTRING)
    IF( MYSTRING .NE. "" ) THEN
      READ(MYSTRING,*,IOSTAT=ERR) OPTIONS%DEBUG
      THROWM(ERR.NE.0,"Cannot decode env variable MAP_VIIRS_CRIS_DEBUG")
    ENDIF

    !> Band Name (default is M)
    CALL GETENV('VIIRS_BAND',MYSTRING)
    IF( TRIM(MYSTRING) .NE. "" ) THEN
      OPTIONS%VIIRS_BAND(1:1) = TRIM(MYSTRING)
    ENDIF

    !> Mapping Threshold percentage of good VIIRS pixels in mapping box or FOV
    CALL GETENV('MAP_VIIRS_CRIS_THRESHOLD',MYSTRING)
    IF( TRIM(MYSTRING) .NE. "" ) THEN
      READ(MYSTRING,*,IOSTAT=ERR) OPTIONS%THRESHOLDGP
      THROWM(ERR.NE.0,"Cannot decode env variable MAP_VIIRS_CRIS_THRESHOLD")
   ENDIF

    OPTIONS%DO_CLUSTERS=.TRUE.
     CALL GETENV('MAP_DO_CLUSTERS',MYSTRING)
    IF( TRIM(MYSTRING) .NE. "" ) THEN
      IF(MYSTRING(1:1) .EQ. "N" .or. MYSTRING(1:1) .eq. "n" ) OPTIONS%DO_CLUSTERS=.FALSE.
    ENDIF
    !> Get sort channel for cluster by default M16 -> 16
    OPTIONS%CLUSTER_SORT_CHANNEL = 16
    CALL GETENV('MAP_CLUSTER_SORT_CHANNEL',MYSTRING)
    IF( TRIM(MYSTRING) .NE. "" ) THEN
      READ(MYSTRING,*,IOSTAT=ERR) OPTIONS%CLUSTER_SORT_CHANNEL
      THROWM(ERR.NE.0,"Cannot decode env variable CLUSTER_SORT_CHANNEL")
    ENDIF
    write(*,*) "MAP_CLUSTER_SORT_CHANNEL",OPTIONS%CLUSTER_SORT_CHANNEL
CATCH
    END SUBROUTINE
    !> Prepares the calculations.
    !> Reads the VIIRS SDR and GEO data in files viirssdr.H5 and viirsgeo.H5
    !!   which could be the same physical file.
    !!
    !> Optionaly reads the MAIA4 cloud mask which should be exactly on the same 
    !!   grid as the VIIRS SDR.
    !!
    !> Prepares the Bow-Tie flag array, the location quality flag array
    !!   and the crude lat/lon grid (1 x 1 degree).
    SUBROUTINE MAP_VIIRS_SETUP(OPTIONS, SDR, MAIA4, VIIRSLOC, ERR)
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MOD

    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN)  :: OPTIONS
    TYPE(VIIRS_SDR),         INTENT(OUT) :: SDR
    TYPE(VIIRS_MAIA4),       INTENT(OUT) :: MAIA4
    TYPE(VIIRS_LOCATION),    INTENT(OUT) :: VIIRSLOC
    INTEGER,                 INTENT(OUT) :: ERR
    
    INTEGER :: NCHANNELS, NPIXELS, NLINES, NSCANS, NGRANULES
    INTEGER :: I, J, IRCHAN, ILAT, ILON
    LOGICAL :: EXSDR, EXGEO, EXMAIA
    INTEGER :: VIIRS_NCHAN
    INTEGER :: VIIRS_LIST(22)

    INTEGER           :: IQ(3)
    REAL(KIND=8)      :: IJ1, IJ2
    LOGICAL           :: OK
TRY
    !>- Initialise the structure
    CALL VIIRS_LOCATION_INIT(VIIRSLOC)

    !>- Check input files existence
    INQUIRE(FILE="viirssdr.H5", EXIST=EXSDR)
    INQUIRE(FILE="viirsgeo.H5", EXIST=EXGEO)
    INQUIRE(FILE="maia4.H5",    EXIST=EXMAIA)

    !>- If VIIRS data
    IF( EXSDR .AND. EXGEO) THEN
      !> - Load geolocation and VIIRS channels.
      !> - Use all channels available in file
      CALL MAP_VIIRS_LIST_CHANS( "viirssdr.H5", TRIM(OPTIONS%VIIRS_BAND), VIIRS_NCHAN, VIIRS_LIST, ERR)
      THROWM(ERR.NE.0,"Cannot check number of VIIRS channels")

      !> - Read VIIRS SDR data and geolocation
      CALL VIIRS_SDR_LOAD(OPTIONS%VIIRS_BAND, SDR, (/"viirssdr.H5"/), err, &
          & CHANNELS=VIIRS_LIST(1:VIIRS_NCHAN), &
          & CLEAN=.TRUE., &
          & GEOLOCFILE="viirsgeo.H5")
      THROWM(ERR.NE.0,"Cannot load VIIRS SDR structure")

      !> - Get the some usefull variables
      CALL VIIRS_SDR_INFO( SDR, NCHANNELS, NPIXELS, NLINES, NSCANS, NGRANULES, ERR )
      IF( OPTIONS%DEBUG .GE. 2 ) THEN
        write(*,*) "nchannels ", nchannels
        write(*,*) "npixels   ", npixels
        write(*,*) "nlines    ", nlines
        write(*,*) "nscans    ", nscans
        write(*,*) "ngranules ", ngranules
      ENDIF

      !> - Use specific structure in order to contain GeoLocation and Mask
      VIIRSLOC%TOTALSCANS      = NSCANS
      VIIRSLOC%NLINES          = NLINES
      VIIRSLOC%ROWSPERSCAN     = NINT(1.0 * NLINES / NSCANS)
      VIIRSLOC%COLUMNSPERSCAN  = NPIXELS
      VIIRSLOC%LATITUDE       => SDR%GEO%LATITUDE
      VIIRSLOC%LONGITUDE      => SDR%GEO%LONGITUDE
      VIIRSLOC%UTCJULIANDAY   => SDR%GEO%UTCJULIANDAY
      VIIRSLOC%UTC            => SDR%GEO%UTC
      ALLOCATE(VIIRSLOC%MaskBOWTIE(npixels, nlines),stat=err)
      THROWM(ERR.NE.0,"Cannot  allocate MaskBOWTIE array")
      ALLOCATE(VIIRSLOC%qual(npixels, nlines),stat=err)
      THROWM(ERR.NE.0,"Cannot  allocate quality array")
      ALLOCATE(VIIRSLOC%pixqual(npixels, nlines),stat=err)
      THROWM(ERR.NE.0,"Cannot  allocate pixel quality array")
      CALL GEOLOC_UNCORRECT_TC(OPTIONS, SDR, VIIRSLOC, ERR)

      !> - Bow-Tie Mask is given by IR radiances
      IF( NCHANNELS .LE. 0 ) THEN
        WRITE(*,*) "NO VIIRS Channel data available; continue without data"
      ELSE
        IRCHAN = 0
        DO I=SIZE(SDR%BAND),1,-1
          IF(ASSOCIATED(SDR%BAND(I)%RADIANCE)) THEN
            IRCHAN = I
            EXIT
          ENDIF
        END DO
        VIIRSLOC%MASKBOWTIE = .FALSE.
        WHERE( SDR%BAND(IRCHAN)%RADIANCE < 1.E-10 )
          VIIRSLOC%MASKBOWTIE        = .TRUE.
        END WHERE
      ENDIF

      !> - Check VIIRS Location, keep only latitude > -90
      VIIRSLOC%QUAL = .TRUE.
      WHERE( SDR%GEO%LATITUDE .LT. -90.0 )
        VIIRSLOC%QUAL = .FALSE.
      END WHERE
      VIIRSLOC%TESTQUAL = ANY(VIIRSLOC%QUAL .EQV. .FALSE.) 

      !> - Create a grid (1deg x 1deg) of the VIIRS pixels
      DO J=1, NLINES,5
        DO I=1, NPIXELS,5
          IF( VIIRSLOC%QUAL(i,J) ) THEN
            ILAT = NINT(VIIRSLOC%LATITUDE(i,J))
            ILON = NINT(VIIRSLOC%LONGITUDE(i,J))
            VIIRSLOC%GRIDP(ILAT,ILON) = I
          ENDIF
        END DO
      END DO

      !> - If any VIIRS pixel above 85 latitude, consider all longitude
      !! grid nodes as viewed by VIIRS
      DO ILAT = 85, 90
        IF( ANY(VIIRSLOC%GRIDP(ILAT,:) .GT. 0 )) THEN
          DO ILON = -180,180
            IF(VIIRSLOC%GRIDP(ILAT,ILON) .LT.0 ) VIIRSLOC%GRIDP(ILAT,ILON) = 0
          END DO
        ENDIF
      END DO
      DO ILAT = -90, -85
        IF( ANY(VIIRSLOC%GRIDP(ILAT,:) .GT. 0 )) THEN
          DO ILON = -180,180
            IF(VIIRSLOC%GRIDP(ILAT,ILON) .LT.0 ) VIIRSLOC%GRIDP(ILAT,ILON) = 0
          END DO
        ENDIF
      END DO
    ENDIF

    !> - mask all bad VIIRS quality pixels
    viirsloc%pixqual= .False.
    DO I=SIZE(SDR%BAND),1,-1
     IF(ASSOCIATED(SDR%BAND(I)%RADIANCE)) THEN
        WHERE( SDR%BAND(I)%RADIANCE < 1.E-10 )
          VIIRSLOC%pixqual        = .TRUE.
        END WHERE
        IF( SDR%BAND(I)%CHANNEL_NAME(1:1) .EQ. "M" ) THEN
          WHERE( SDR%BAND(I)%QF1_VIIRSMBANDSDR .NE. 0)
            VIIRSLOC%pixqual        = .TRUE.
          END WHERE
        ELSEIF( SDR%BAND(I)%CHANNEL_NAME(1:1) .EQ. "I" ) THEN
           WHERE( SDR%BAND(I)%QF1_VIIRSIBANDSDR .NE. 0)
            VIIRSLOC%pixqual        = .TRUE.
          END WHERE
        ELSE
           WHERE (SDR%BAND(I)%QF1_VIIRSDNBBANDSDR .NE. 0) 
            VIIRSLOC%pixqual        = .TRUE.
           END WHERE
        ENDIF ! band name
     ENDIF ! associated
   END DO

    !>- Load maia4 file if exists
    IF ( EXMAIA ) THEN

      CALL MAIA4_LOAD("maia4.H5", maia4, err)
      THROWM(ERR.NE.0,"Cannot load MAIA structure")

      !> - Only consider MAIA4 format version 2 (Feb 2014)
      !> - Update to version 3 (Sept 2017)
      IF( MAIA4%format_version .NE. 3 ) THEN
        IF( OPTIONS%DEBUG .GE. 2 ) THEN
          WRITE(*,*) "MAIA4 format versions <3 are not considered in mapping"
        ENDIF
        CALL MAIA4_FREE(MAIA4, ERR)
        THROW(ERR.NE.0)
      ELSE

        MAIA4%LMAIA4 = .TRUE.
        !> - MAIA4 cannot be used without VIIRS SDR because the MAIA4 latitude/longitudes 
        !!  are masked with missing values in the tie bow areas.
        !!  Correct lat/long are requested by the adjacency algorithm.
        !! Check coherence with SDR
        OK =          MAIA4%HEADER%NB_LINES  .EQ. NLINES
        OK = OK .AND. MAIA4%HEADER%NB_PIXELS .EQ. NPIXELS
        IQ(2) = MAIA4%DATA%SCNLIN_DAY(1)
        IQ(3) = MAIA4%DATA%SCNLIN_TIME(1)
        IQ(1) = MAIA4%DATA%SCNLIN_YEAR(1)
        CALL XIQJ(IQ,IJ1)
        IJ2 = SDR%GEO%UTCJULIANDAY(1) + SDR%GEO%UTC(1)/86400.D3
        OK = OK .AND. ABS( IJ1 -IJ2) .LE. 1/86400.D0 ! less than 1 second
        
        IF( .NOT. OK ) THEN
          IF( OPTIONS%DEBUG .GE. 1 ) THEN
            WRITE(*,*) "MAIA4 NOT COHERENT WITH VIIRS SDR"
            WRITE(*,*) "lines  maia, sdr ",MAIA4%HEADER%NB_LINES,NLINES
            WRITE(*,*) "pixels maia, sdr ",MAIA4%HEADER%NB_PIXELS,NPIXELS
            WRITE(*,*) "start maia, sdr, diff(sec) ",IJ1,IJ2,(IJ1 -IJ2)*86400.0D0
            WRITE(*,*) "Continue without MAIA4 data"
          ENDIF
          !ERR=1
          !THROWM(ERR.NE.0,"MAIA4 NOT COHERENT WITH VIIRS SDR")
          MAIA4%LMAIA4 = .FALSE.
          CALL MAIA4_FREE(MAIA4, ERR)
          THROW(ERR.NE.0)
        ENDIF
      ENDIF
    ELSE
      ! Not fatal
      IF( OPTIONS%DEBUG .GE. 2 ) THEN
        WRITE(*,*) "MAIA4 data NOT available; continue without cloud mask"
      ENDIF
    ENDIF   

CATCH
    END SUBROUTINE

    !> Transform the terrain corrected geolocation to UNcorrected.
    !> The method is not optimal as it uses the navigation routine
    !! geogcart and cartgeo.
    !!
    !! Only pixels with an altitude above 50 metres are considered.
    !! The iterative process stops after 4 iterations or if
    !! new altitude is lower than 10 metres.
    !!
    !! This subroutine can take up to 30% of total run time
    !! of the viirs cris mapping. This depends on the relief.
    !! But one should reminds that this software should be run with 
    !! non terrain correction (GMODO GIMGO files).
    !!
    !! @todo improve processing time of the un-correct terrain
    !! geolocation
    SUBROUTINE GEOLOC_UNCORRECT_TC(OPTIONS, SDR, VIIRSLOC, ERR)
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MOD

    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN)  :: OPTIONS
    TYPE(VIIRS_SDR),         INTENT(INOUT) :: SDR
    TYPE(VIIRS_LOCATION),    INTENT(INOUT) :: VIIRSLOC
    INTEGER,                 INTENT(OUT) :: ERR
    
    INTEGER :: LINE, PIXEL, ISCAN

    REAL              :: SATPOS(3)
    REAL(KIND=8)      :: LAT_OM, LON_OM, ALT_OM, OM(3)
    REAL(KIND=8)      :: SM(3)
    REAL(KIND=8)      :: COEF, NORM_SM
    
    INTEGER           :: ITER
    !INTEGER           :: CDEB, CFIN, CRATE

TRY
    !CALL SYSTEM_CLOCK(COUNT=CDEB)
    ! Convert degrees and metres to radians and kilometers 
    IF (OPTIONS%DEBUG .EQ. 2 ) write(*,*) "GEOLOC_UNCORRECT_TC"
    DO LINE=1,VIIRSLOC%NLINES
      ISCAN =  (LINE-1) / VIIRSLOC%ROWSPERSCAN +1
      SATPOS(:)=SDR%GEO%SCPosition(:,ISCAN) / 1000.
    DO PIXEL=1,VIIRSLOC%COLUMNSPERSCAN
      ALT_OM = SDR%GEO%HEIGHT(PIXEL,LINE) / 1000.
      IF( ALT_OM .GT. 0.050 ) THEN
        ITER = 1
        LAT_OM = SDR%GEO%LATITUDE(PIXEL,LINE) * 3.14159/180.
        LON_OM = SDR%GEO%LONGITUDE(PIXEL,LINE) * 3.14159/180.
        CALL GEOGCART(LAT_OM, LON_OM, ALT_OM, OM)
        DO WHILE ( ABS(ALT_OM) .GT. 0.010 .AND. ITER .LT. 3)
          COEF = ALT_OM / COS(SDR%GEO%SatelliteZenithAngle(PIXEL,LINE) * 3.14159/180.)
          SM(:) = OM(:) - SATPOS(:)
          NORM_SM = SQRT( SM(1)*SM(1) + SM(2)*SM(2) + SM(3)*SM(3) )
          OM(:) = OM(:) + SM(:)*COEF/NORM_SM
          CALL CARTGEOG(OM, LAT_OM, LON_OM, ALT_OM, ERR)
          THROWM(ERR.NE.0,"Cannot converge")
          ITER = ITER +1
        END DO
        SDR%GEO%LATITUDE(PIXEL,LINE) = LAT_OM *180. /3.14159
        SDR%GEO%LONGITUDE(PIXEL,LINE) = LON_OM *180. /3.14159
        SDR%GEO%HEIGHT(PIXEL,LINE) = ALT_OM * 1000.
      ENDIF
    ENDDO
    ENDDO
    !CALL SYSTEM_CLOCK(COUNT=CFIN,COUNT_RATE=CRATE)
    !write(0,*) "duree sec: ",CDEB, CFIN, CRATE, (CFIN-CDEB)*1.0_8/CRATE

CATCH
    END SUBROUTINE


    !> Print Mapping global options
    SUBROUTINE MAP_VIIRS_PRINT_OPTIONS(OPTIONS)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS

    WRITE(*,*) "MAP_VIIRS_OPTIONS :"
    WRITE(*,*) "VIIRS BAND  : ", OPTIONS%VIIRS_BAND
    WRITE(*,*) "DEBUG LEVEL : ", OPTIONS%DEBUG
    WRITE(*,*) "THRESHOLD   : ", OPTIONS%THRESHOLDGP
    WRITE(*,*) "CLUSTER SORT CHANNEL    : ", OPTIONS%CLUSTER_SORT_CHANNEL
    RETURN
    END SUBROUTINE

    !> Read MAIA4 structure from HDF5 file
    SUBROUTINE MAIA4_LOAD(FILENAME, MAIA4, ERR)
USE viirs_maia4_header_io
USE viirs_maia4_data_io
    IMPLICIT NONE

    CHARACTER(LEN=*),  INTENT(IN)  :: FILENAME
    TYPE(VIIRS_MAIA4), INTENT(OUT) :: MAIA4
    INTEGER,           INTENT(OUT) :: ERR

    INTEGER(HID_T)    :: FILE_ID, GROUP_ID
    INTEGER           :: IVAL(1)
    CHARACTER(LEN=16) :: GNAME, SNAME
    LOGICAL           :: LEXT
TRY

    CALL H5FOPEN_F( TRIM(FILENAME), H5F_ACC_RDONLY_F, FILE_ID, ERR, H5P_DEFAULT_F)
    THROWM(ERR.NE.0,"Cannot open File"//TRIM(FILENAME))
    
    SNAME='format version'
    MAIA4%FORMAT_VERSION=1
    CALL H5AEXISTS_BY_NAME_F(FILE_ID, "/", SNAME, LEXT, ERR)
    IF( LEXT ) THEN
      CALL H5LTGET_ATTRIBUTE_INT_F(FILE_ID,"/",SNAME,IVAL(1),ERR)
      THROWM(ERR.NE.0,"Cannot read attribute "//TRIM(SNAME))
      MAIA4%FORMAT_VERSION=IVAL(1)
    ENDIF

    GNAME = "HEADER"
    CALL H5GOPEN_F(FILE_ID, GNAME, GROUP_ID, ERR, H5P_DEFAULT_F)
    THROWM(ERR.NE.0,"Cannot open group "//TRIM(GNAME))
    
    CALL VIIRS_MAIA4_HEADER_RH(MAIA4%HEADER,GROUP_ID,ERR)
    THROWM(ERR.NE.0,"Cannot read MAIAv4 Header")

    CALL H5GCLOSE_F( GROUP_ID, ERR)
    THROWM(ERR.NE.0,"Cannot close group "//TRIM(GNAME))

    GNAME = "DATA"
    CALL H5GOPEN_F(FILE_ID, GNAME, GROUP_ID, ERR, H5P_DEFAULT_F)
    THROWM(ERR.NE.0,"Cannot open group "//TRIM(GNAME))
    
    CALL VIIRS_MAIA4_DATA_RH(MAIA4%DATA,GROUP_ID,ERR)
    THROWM(ERR.NE.0,"Cannot read MAIAv4 Data")
    
    CALL H5GCLOSE_F( GROUP_ID, ERR)
    THROWM(ERR.NE.0,"Cannot close group "//TRIM(GNAME))
    
    CALL H5FCLOSE_F(FILE_ID, ERR) 
    THROWM(ERR.NE.0,"Cannot close file "//TRIM(FILENAME))

CATCH
    END SUBROUTINE

    !> Free MAIA4 structure
    SUBROUTINE MAIA4_FREE(MAIA4, ERR)
USE viirs_maia4_header_mem
USE viirs_maia4_data_mem
    IMPLICIT NONE

    TYPE(VIIRS_MAIA4), INTENT(INOUT) :: MAIA4
    INTEGER,             INTENT(OUT) :: ERR
TRY
    CALL VIIRS_MAIA4_HEADER_FREE(MAIA4%HEADER,ERR)
    THROWM(ERR.NE.0,"Cannot free  MAIA4%HEADER")
    
    CALL VIIRS_MAIA4_DATA_FREE(MAIA4%DATA,ERR)
    THROWM(ERR.NE.0,"Cannot free  MAIA4%DATA")
    
    MAIA4%LMAIA4         = .FALSE.
    MAIA4%FORMAT_VERSION = 0
CATCH
    END SUBROUTINE

    !> Shift radiances according to environment variables.
    !> This can be done only once as the original array is 
    !! overwritten.
    !!
    !! Could be called by main program
    !!
    !! This routine should only be used for testing/debugging purposes
    !!   that's why the environment variables are not loaded in the
    !!   setup routine
    SUBROUTINE MAP_VIIRS_SHIFT_RAD(OPTIONS, SDR, VIIRSLOC, ERR)
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MOD

    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(VIIRS_SDR),      INTENT(OUT)   :: SDR
    TYPE(VIIRS_LOCATION), INTENT(OUT)   :: VIIRSLOC
    INTEGER,              INTENT(OUT)   :: ERR
    
    INTEGER           :: NPIXELS, NLINES
    CHARACTER(LEN=32) :: CH_SHIFT_PIX, CH_SHIFT_LIN
    INTEGER           :: SHIFT_PIX, SHIFT_LIN
    REAL, ALLOCATABLE ::  TMPRAD(:,:)
    INTEGER           ::  P,L, NP, NL, I

TRY

    NPIXELS = SIZE(VIIRSLOC%MASKBOWTIE,1)
    NLINES  = SIZE(VIIRSLOC%MASKBOWTIE,2)

    CALL GETENV('SHIFT_PIX',CH_SHIFT_PIX)
    CALL GETENV('SHIFT_LIN',CH_SHIFT_LIN)
    IF( CH_SHIFT_PIX .EQ. "" ) THEN
      SHIFT_PIX=0
    ELSE
      READ(CH_SHIFT_PIX,*) SHIFT_PIX
      IF( OPTIONS%DEBUG .GE. 2 ) THEN
        write(*,*) "shift mapping ",SHIFT_PIX," pixels"
      ENDIF
    ENDIF
    IF( CH_SHIFT_LIN .EQ. "" ) THEN
      SHIFT_LIN=0
    ELSE
      READ(CH_SHIFT_LIN,*) SHIFT_LIN
      IF( OPTIONS%DEBUG .GE. 2 ) THEN
        write(*,*) "shift mapping ",SHIFT_LIN," lines"
      ENDIF
    ENDIF

    IF( SHIFT_LIN .NE. 0 .OR. SHIFT_PIX .NE. 0 ) THEN
      ALLOCATE(TMPRAD(npixels, nlines),stat=err)
      THROWM(ERR.NE.0,"Cannot  allocate TMPRAD array")
      DO I=1,SIZE(SDR%BAND)
        IF(ASSOCIATED(SDR%BAND(I)%RADIANCE)) THEN
          TMPRAD = SDR%BAND(I)%RADIANCE

          DO L=1,NLINES
          DO P=1,NPIXELS
            NP=P+SHIFT_PIX
            NL=L+SHIFT_LIN
            IF( NP .GT. 0 .AND. NP .LE. NPIXELS  .AND. &
              & NL .GT. 0 .AND. NL .LE. NLINES ) THEN
              IF( (.NOT. VIIRSLOC%MASKBOWTIE(P,L)) .AND. &
                & (.NOT. VIIRSLOC%MASKBOWTIE(NP,NL)) ) THEN
                TMPRAD (NP,NL) = SDR%BAND(I)%RADIANCE (P,L)
              ENDIF
            ENDIF
          END DO
          END DO
          SDR%BAND(I)%RADIANCE = TMPRAD
        ENDIF
      END DO
      DEALLOCATE(TMPRAD)
    ENDIF

CATCH
    END SUBROUTINE

    !> Read CrIS FDF file for Mapping with VIIRS
    SUBROUTINE  MAP_VIIRS_CRIS_FDF(Cfunit, satname, err)
    IMPLICIT NONE
    INTEGER,      INTENT(IN)  :: CFUNIT        ! unit numbers
    CHARACTER(LEN=3),    INTENT(IN)  :: SATNAME ! Satellite Short Name
    INTEGER,      INTENT(OUT) :: ERR
     
    CHARACTER(LEN=80)  :: INPUT_LINE    ! line read from instream data
    INTEGER            :: MC, LC, I, IR, IV
    REAL               :: REG(8)
    CHARACTER(LEN=3)   :: NAME

TRY
      OPEN ( UNIT=CFUNIT, STATUS="OLD", FORM="FORMATTED",&
     &       ACCESS="SEQUENTIAL", IOSTAT=ERR )

      ! If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( ERR.NE.0 ) &
     &  OPEN ( UNIT=CFUNIT, STATUS="UNKNOWN", FORM="FORMATTED",&
     &       ACCESS="SEQUENTIAL", IOSTAT=ERR )
      THROWM(ERR.NE.0,"Error opening cris FDF")

      ! look for sections in the file
      DO WHILE ( ERR.EQ.0 .AND. INPUT_LINE(1:3).NE."END" )
        INPUT_LINE = " "
        READ(CFUNIT,"(A80)",IOSTAT=ERR) INPUT_LINE
        THROWM(ERR.NE.0,"Error reading cris FDF")

        !>- Read section CRISFOVSIZE
        IF ( INPUT_LINE(1:11) .EQ. "CRISFOVSIZE" ) THEN

          ! Read file
          READ(CFUNIT,*) CRISAXES%REFALTITUDE

          DO I = 1, 270
          
            READ(CFUNIT,*,IOSTAT=ERR) MC, LC, IR, IV, REG(:)
            THROWM(ERR.NE.0,"Error reading cris FDF")

            CRISAXES%VIIRSM(IR,IV)   = MC
            CRISAXES%VIIRSL(IR,IV)   = LC
            CRISAXES%ACSLOPE1(IR,IV) = REG(1)
            CRISAXES%ACINTER1(IR,IV) = REG(2)
            CRISAXES%ACSLOPE2(IR,IV) = REG(3)
            CRISAXES%ACINTER2(IR,IV) = REG(4)
            CRISAXES%ALSLOPE1(IR,IV) = REG(5)
            CRISAXES%ALINTER1(IR,IV) = REG(6)
            CRISAXES%ALSLOPE2(IR,IV) = REG(7)
            CRISAXES%ALINTER2(IR,IV) = REG(8)
           
          END DO 

        ENDIF

      IF ( INPUT_LINE(1:7) .EQ. SATNAME // " EQW" .OR. &
           INPUT_LINE(1:7) .EQ. SATNAME // " EQW" ) THEN
         ! Read VIIRS Channel cwn eqw_wn cwl eqw_wl
          READ(CFUNIT,*,IOSTAT=ERR) INPUT_LINE
          THROWM(ERR.NE.0,"Error reading cris FDF")
          DO I = 1, 22
            READ(CFUNIT,*,IOSTAT=ERR) VIIRSSRF(I)%NAME, VIIRSSRF(I)%CWN, VIIRSSRF(I)%EQW_WN, &
                                      VIIRSSRF(I)%CWL, VIIRSSRF(I)%EQW_WL
            THROWM(ERR.NE.0,"Error reading cris FDF")
          write(*,*) VIIRSSRF(I)%NAME, VIIRSSRF(I)%CWN, VIIRSSRF(I)%EQW_WN,&
                     VIIRSSRF(I)%CWL, VIIRSSRF(I)%EQW_WL
          END DO
          READ(CFUNIT,*,IOSTAT=ERR) INPUT_LINE
          THROWM(ERR.NE.0,"Error reading cris FDF")
          DO I = 1, 22
            READ(CFUNIT,*,IOSTAT=ERR) NAME, VIIRSSRF(I)%SOLAR_IRRADIANCE
            THROWM(ERR.NE.0,"Error reading cris FDF")
            THROWM(NAME .NE. VIIRSSRF(I)%NAME,"Error reading cris FDF inconsistent file")
          END DO
          write(*,*) "solar irradiance"
          DO I = 1, 22
            write(*,*) VIIRSSRF(I)%NAME,VIIRSSRF(I)%SOLAR_IRRADIANCE
          END DO
          READ(CFUNIT,*,IOSTAT=ERR) INPUT_LINE
          write(*,*) INPUT_LINE
          THROWM(ERR.NE.0,"Error reading cris FDF")
          DO I = 1, 22
            READ(CFUNIT,*,IOSTAT=ERR) VIIRSSRF(I)%ChannelNoise
            THROWM(ERR.NE.0,"Error reading cris FDF")
            write(*,*) "channel noise channel",I, VIIRSSRF(I)%ChannelNoise
          END DO
          READ(CFUNIT,*,IOSTAT=ERR) INPUT_LINE
          write(*,*) INPUT_LINE
          THROWM(ERR.NE.0,"Error reading cris FDF")
          DO I = 1, 22
            READ(CFUNIT,*,IOSTAT=ERR) VIIRSSRF(I)%ChannelWeight
            THROWM(ERR.NE.0,"Error reading cris FDF")
            write(*,*) "channel weight channel",I, VIIRSSRF(I)%ChannelWeight
          END DO
         ! compute conversion factors
         DO I = 1, 22
          !>- VIIRS Radiances are stored in SDRs as W/(m^2 sr micron)
          !! we want radiances in the same unit as CrIS W/(m^2 sr cm^-1)*1E7
          !! so use nominal wavelenght and band equivalent width (EQW)
          !!      (from mapping_viirs_cris.fdf) to make the conversion
          !!
          !> - R(nu) = 1000 *R(Lambda)*EQW(Lambda)/EQW(nu)   with:
          !> - R(nu) in  mW/(m^2 sr cm^-1)
          !> - R(Lambda) in W/(m^2 sr micron)
          !> - Lambda in microns
          !> - nu in centimeters-1
          VIIRSSRF(I)%RadtoCRISunitfactor = 1000. * VIIRSSRF(I)%EQW_WL / VIIRSSRF(I)%EQW_WN * 1.0D4 
          !>   For cluster we need to have SI Units
          VIIRSSRF(I)%RadtoSIfactor = 0.00001 * 1000. * VIIRSSRF(I)%EQW_WL / VIIRSSRF(I)%EQW_WN
         END DO
      ENDIF
         
      ENDDO

      CLOSE(CFUNIT)
CATCH
    END SUBROUTINE

 SUBROUTINE  MAP_UPDATE_CLUSTER_PARAM(OPTIONS, SDR, CLUSTERPARAM, err)
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MOD 
   IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(VIIRS_SDR),      INTENT(IN)   :: SDR
    TYPE(clusterparameter), INTENT(OUT) :: clusterparam
    INTEGER,      INTENT(OUT) :: ERR
    INTEGER ::I,k
    err = 0
    k = 0
   clusterparam%SortChan = 0

    DO I =1,SIZE(SDR%BAND)
       IF (ASSOCIATED(SDR%BAND(I)%RADIANCE)) THEN
         k = k + 1
         clusterparam%noise(k) = VIIRSSRF(I)%ChannelNoise
         clusterparam%weight(k) = VIIRSSRF(I)%ChannelWeight
         IF ( I .eq. OPTIONS%CLUSTER_SORT_CHANNEL) THEN
           clusterparam%SortChan = k
           write(*,*) "sorting channel:", I, "->" , k
         ENDIF 
       ENDIF
    ENDDO
    clusterparam%Nch = k
    IF ( clusterparam%SortChan .eq. 0 ) THEN
       clusterparam%SortChan = clusterparam%Nch 
       write(*,*) "sorting channel:" , clusterparam%SortChan
    ENDIF
    IF (k .eq. 0 ) err = 1
    IF ( all(clusterparam%weight == 0 )) err = 1
    write(*,*) "paremeters for clusters (Noise Weight):"
    DO I= 1,clusterparam%Nch
      write(*,*) clusterparam%noise(I),  clusterparam%weight(i)
    ENDDO
    write(*,*) "Nb channels:",clusterparam%Nch

    END SUBROUTINE

    !> Retrieve the list of channels available in the SDR file
    SUBROUTINE MAP_VIIRS_LIST_CHANS( FILENAME, BANDNAME, NCHAN, LIST, ERR)
    IMPLICIT NONE
    
    CHARACTER(LEN=*), INTENT(IN) :: FILENAME
    CHARACTER(LEN=*), INTENT(IN) :: BANDNAME
    INTEGER, INTENT(OUT)         :: NCHAN
    INTEGER, INTENT(OUT)         :: LIST(:)
    INTEGER, INTENT(OUT)         :: ERR

    CHARACTER(LEN=256) :: GNAME
    CHARACTER(LEN=3)   :: CHANNEL_NAME, NBAND

    INTEGER        :: MCHAN
    INTEGER        :: I
    INTEGER        :: SAERR, HDERR
    INTEGER(HID_T) :: FILE_ID
    INTEGER(HID_T) :: GROUP_ID
TRY
    NBAND = ADJUSTL(BANDNAME)
    CHANNEL_NAME = ''
    SELECT CASE ( NBAND )
    CASE ('i','I')
      MCHAN = 5
    CASE ('m','M')
      MCHAN = 16
    CASE DEFAULT
      MCHAN = 1
    END SELECT
    
    CALL H5FOPEN_F( TRIM(FILENAME), H5F_ACC_RDONLY_F, FILE_ID, ERR, H5P_DEFAULT_F)
    THROWM(ERR.NE.0,"Cannot open File"//TRIM(FILENAME))

    NCHAN = 0
    LIST  = 0
    CALL H5ESET_AUTO_F( 0, SAERR )

    DO I = 1, MCHAN
      SELECT CASE ( NBAND )
      CASE ('i','I')
        WRITE(CHANNEL_NAME,'("I",i1)') i
      CASE ('m','M')
        IF ( I .LT. 10 ) THEN
            WRITE(CHANNEL_NAME,'("M",i1)') i
          ELSE 
            WRITE(CHANNEL_NAME,'("M",i2)') i
          END IF
       CASE DEFAULT
          CHANNEL_NAME='DNB'
      END SELECT

      GNAME = "All_Data/VIIRS-"//TRIM(CHANNEL_NAME)//"-SDR_All"
      CALL H5GOPEN_F(FILE_ID, TRIM(GNAME), GROUP_ID, HDERR, H5P_DEFAULT_F)
      IF( HDERR .EQ. 0 ) THEN
        NCHAN = NCHAN +1
        LIST(NCHAN) = I
        CALL H5GCLOSE_F( GROUP_ID, HDERR )
      ENDIF
    END DO
    CALL H5ESET_AUTO_F( 1, SAERR )

    CALL H5FCLOSE_F( FILE_ID, ERR )
    THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(FILENAME))
    
    IF( NCHAN .EQ. 0 ) THEN
      ERR=1
      THROWM(ERR.NE.0,"NO BAND "//TRIM(BANDNAME)//" CHANNELS IN FILE "//TRIM(FILENAME))
    ENDIF
    
CATCH
    END SUBROUTINE

    !> Create a new mapping structure, fills with CrIS 1c/1d info.
    !> Do not fills with CrIS data if the argument IFOR is 0
    !!  this is usefull for any other mapping facility
    SUBROUTINE MAP_NEW(OPTIONS, MAPPING, LCRIS1C, ifor, ifov)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN)    :: OPTIONS
    TYPE(CFOVVIIRS),         INTENT(INOUT) :: MAPPING
    LOGICAL,                 INTENT(IN)    :: LCRIS1C
    INTEGER,                 INTENT(IN)    :: IFOR
    INTEGER,                 INTENT(IN)    :: IFOV  ! only for 1c format

    !>- Initialise the structure
    CALL CFOVVIIRS_INIT(MAPPING)
    
    !>- If any CrIS data, gets the CrIS field of view data 
    !! Different routines for 1C and 1D formats
    IF( IFOR .GT. 0 ) THEN
      IF( LCRIS1C ) THEN
        CALL GET_CRIS1C_FOV( ifor,      &
         &    ifov,              &
         &    MAPPING%line,             &
         &    MAPPING%scnlinyr,         &
         &    MAPPING%scnlindy,         &
         &    MAPPING%scnlintime,       &
         &    MAPPING%target_latitude,  &
         &    MAPPING%target_longitude, &
         &    MAPPING%target_altitude)
         MAPPING%for        = ifor
         MAPPING%fov        = ifov
      ELSE
        CALL GET_CRIS1D_FOV( ifor,      &
         &    MAPPING%fov,              &
         &    MAPPING%line,             &
         &    MAPPING%scnlinyr,         &
         &    MAPPING%scnlindy,         &
         &    MAPPING%scnlintime,       &
         &    MAPPING%target_latitude,  &
         &    MAPPING%target_longitude, &
         &    MAPPING%target_altitude)
         MAPPING%for        = ifor
      ENDIF
    ENDIF

    IF( OPTIONS%DEBUG .GE. 2 .AND. IFOR .GT. 0) THEN
      WRITE(*,*)
      WRITE(*,*) "MAP_NEW"
      WRITE(*,*) "line ",MAPPING%line
      WRITE(*,*) "FOR  ",MAPPING%for
      WRITE(*,*) "FOV  ",MAPPING%fov
      WRITE(*,*) "LAT  ",MAPPING%target_latitude
      WRITE(*,*) "LON  ",MAPPING%target_longitude
      WRITE(*,*) "ALT  ",MAPPING%target_altitude
    ENDIF

    MAPPING%altitude   = MAPPING%target_altitude

    END SUBROUTINE

    !> Free memory for mapping structure
    SUBROUTINE MAP_FREE( MAPPING, ERR)
    IMPLICIT NONE

    TYPE(CFOVVIIRS), INTENT(INOUT) :: MAPPING
    INTEGER,         INTENT(OUT)   ::ERR

TRY

     CALL CFOVVIIRS_FREE(MAPPING,ERR)
     THROWM(ERR.NE.0,"CANNOT FREE CFOVVIIRS")

CATCH
    END SUBROUTINE

   !> Free memory for mapping structure
    SUBROUTINE MAPFOR_FREE( X, ERR)
    IMPLICIT NONE

    TYPE(CFORVIIRS), INTENT(INOUT) ::X
    INTEGER,         INTENT(OUT)   ::ERR

    character(len=64)::sname
TRY
sname='vp'
if(associated(x%vp))THEN
  deallocate(x%vp,stat=err)
  THROWM(ERR.NE.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%vp)
ENDIF
sname='vl'
if(associated(x%vl))THEN
  deallocate(x%vl,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%vl)
ENDIF


CATCH
    END SUBROUTINE

    !> Save mapping structure to HDF5 file
    SUBROUTINE MAP_SAVE( OPTIONS, MAPPING, ERR)
    IMPLICIT NONE
    TYPE(CFOVVIIRS),              INTENT(IN)  :: MAPPING
    TYPE(MAP_VIIRS_OPTIONS),      INTENT(IN)  :: OPTIONS
    INTEGER,                      INTENT(OUT) :: ERR
    
    CHARACTER(LEN=128) :: FN
    INTEGER(HID_T)     :: FILE_ID

TRY
    ERR = 0

    WRITE(FN,'("mapping-",a1,"-",i4.4,"-",i2.2,"-",i2.2,".H5")') &
          & OPTIONS%VIIRS_BAND, MAPPING%line, MAPPING%for,MAPPING%fov
    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) "save to ",TRIM(FN)

    CALL H5FCREATE_F( FN, H5F_ACC_TRUNC_F, FILE_ID, ERR )
    THROWM(ERR.NE.0,"Cannot create File "//TRIM(FN))

    CALL CFOVVIIRS_WH(MAPPING,FILE_ID,ERR, COMPRESS=.FALSE.)
    THROWM(ERR.NE.0,"CANNOT WRITE ADJTABLE IN FILE "//TRIM(FN))

    CALL H5FCLOSE_F( FILE_ID, ERR )
    THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(FN))
    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) TRIM(FN), " saved"
CATCH
    END SUBROUTINE

    !> Save viirsloc structure to HDF5 file
    SUBROUTINE VIIRSLOC_SAVE( OPTIONS, VIIRSLOC, ERR)
    IMPLICIT NONE
    TYPE(VIIRS_LOCATION),         INTENT(IN)  :: VIIRSLOC
    TYPE(MAP_VIIRS_OPTIONS),      INTENT(IN)  :: OPTIONS
    INTEGER,                      INTENT(OUT) :: ERR
    
    CHARACTER(LEN=128) :: FN
    INTEGER(HID_T)     :: FILE_ID
    CHARACTER(LEN=32)  :: SNAME
    INTEGER, ALLOCATABLE  :: TAB(:,:)
TRY
    ERR = 0

    WRITE(FN,'("viirs_location.H5")') 
    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) "save to ",TRIM(FN)

    CALL H5FCREATE_F( FN, H5F_ACC_TRUNC_F, FILE_ID, ERR )
    THROWM(ERR.NE.0,"Cannot create File "//TRIM(FN))
    
    ALLOCATE(TAB(viirsloc%COLUMNSPERSCAN,viirsloc%NLINES), STAT=ERR )
    THROWM(ERR.NE.0,"allocate error")
    TAB = 0
    WHERE (VIIRSLOC%maskbowtie ) 
     TAB = 1
    END WHERE
    SNAME='maskbowtie'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I2=TAB)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    TAB = 0
    WHERE (viirsloc%QUAL) 
     TAB = 1  
    END WHERE
    SNAME='qual'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I2=TAB)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME)) 
    TAB = 0
    WHERE (viirsloc%pixqual) 
     TAB = 1  
    END WHERE
    SNAME='pixqual'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I2=TAB)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    CALL H5FCLOSE_F( FILE_ID, ERR )
    THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(FN))

     IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) TRIM(FN), " saved"

CATCH
    END SUBROUTINE

   !> Save cforviirs structure
    SUBROUTINE MAP_FORMAP_SAVE( OPTIONS, FORMAP, ERR)
     IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS),      INTENT(IN)  :: OPTIONS
    TYPE(cforviirs), INTENT(IN)               :: FORMAP
    INTEGER,                      INTENT(OUT) :: ERR
    CHARACTER(LEN=128) :: FN
    INTEGER(HID_T)     :: FILE_ID
    CHARACTER(LEN=32)  :: SNAME
    INTEGER :: IFOV
TRY
    ERR = 0
    WRITE(FN,'("FORMAP-",a1,"-",i4.4,"-",i2.2,".H5")') &  
          & OPTIONS%VIIRS_BAND, FORMAP%line, FORMAP%for

    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) "save to ",TRIM(FN)

    CALL H5FCREATE_F( FN, H5F_ACC_TRUNC_F, FILE_ID, ERR )
    THROWM(ERR.NE.0,"Cannot create File "//TRIM(FN))
    
    SNAME='vp'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I2=FORMAP%VP)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    
    SNAME='vl'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I2=FORMAP%vl)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    DO IFOV=1,9
      WRITE(SNAME,'("VL_",i1)') IFOV
      CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I1=FORMAP%FOVS(IFOV)%VL)
      THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
      WRITE(SNAME,'("VP_",i1)') IFOV
      CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I1=FORMAP%FOVS(IFOV)%VP)
      THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
      WRITE(SNAME,'("NB_PIX_FOV",i1)') IFOV
      CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I0=FORMAP%FOVS(IFOV)%nb_pixels)
      THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
   ENDDO
  
    CALL H5FCLOSE_F( FILE_ID, ERR )
    THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(FN))




     IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) TRIM(FN), " saved"
CATCH
    END SUBROUTINE


   !> Save cluster structure to HDF5 file
    SUBROUTINE CLUSTER_SAVE( OPTIONS, CLUSTER, ERR)
    IMPLICIT NONE
    TYPE(CLUSTERVIIRS),           INTENT(IN)  :: CLUSTER
    TYPE(MAP_VIIRS_OPTIONS),      INTENT(IN)  :: OPTIONS
    INTEGER,                      INTENT(OUT) :: ERR
    
    CHARACTER(LEN=128) :: FN
    INTEGER(HID_T)     :: FILE_ID
    CHARACTER(LEN=32)  :: SNAME

TRY
    ERR = 0
    WRITE(FN,'("cluster-",a1,"-",i4.4,"-",i2.2,".H5")') &  
          & OPTIONS%VIIRS_BAND, CLUSTER%line, CLUSTER%for

    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) "save to ",TRIM(FN)

    CALL H5FCREATE_F( FN, H5F_ACC_TRUNC_F, FILE_ID, ERR )
    THROWM(ERR.NE.0,"Cannot create File "//TRIM(FN))
    
    SNAME='radvect'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, R1=CLUSTER%RADVECT)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    
    SNAME='cluster_image'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I2=CLUSTER%cluster_image)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME='cluster_on_viirs_grid'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I2=CLUSTER%cluster_on_viirs_grid)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME='mask'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, I2=CLUSTER%mask)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME="line"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,I0=CLUSTER%line)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME="for"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,I0=CLUSTER%for)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    

    SNAME="nb_pixels"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,I0=CLUSTER%nb_pixels)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME="nblin"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,I0=CLUSTER%nblin)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME="nbcol"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,I0=CLUSTER%nbcol)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
        
    SNAME="IdPixel"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,I1=CLUSTER%IdPixel)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

  
    SNAME="nbclass"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,I0=CLUSTER%nbclass)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME="RadAnaNbPix"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,I2=CLUSTER%RadAnaNbPix)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME="RadAnaMean"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,R3=CLUSTER%RadAnaMean)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    SNAME="RadAnaStd"
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR,R3=CLUSTER%RadAnaStd)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))

    CALL H5FCLOSE_F( FILE_ID, ERR )
    THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(FN))

    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) TRIM(FN), " saved"

CATCH
    END SUBROUTINE

   !> Save cluster output to HDF5 file
    SUBROUTINE CLUSTER_SAVE_OUTPUT( OPTIONS, FN, CREATE, CLUSTER, FORMAP, ERR)

    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS),      INTENT(IN)  :: OPTIONS
    CHARACTER(LEN=256),           INTENT(IN)  :: FN ! FileName
    LOGICAL,  INTENT(IN)                      :: CREATE
    TYPE(CLUSTERVIIRS),           INTENT(IN)  :: CLUSTER   
    TYPE(CFORVIIRS),              INTENT(IN)  :: FORMAP
    INTEGER,                      INTENT(OUT) :: ERR
    
    CHARACTER(LEN=128) :: PATH
    INTEGER(HID_T)     :: FILE_ID
    CHARACTER(LEN=32)  :: SNAME
    CHARACTER(LEN=32)  :: ANAME
    INTEGER(HID_T)     :: G_ID

TRY
    ERR = 0
   
    WRITE(PATH,'("cluster-",a1,"-",i4.4,"-",i2.2)') &  
          & OPTIONS%VIIRS_BAND, CLUSTER%line, CLUSTER%for
    WRITE(*,*) PATH
    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) "save to ",TRIM(FN)
    IF (CREATE) THEN
       CALL H5FCREATE_F( FN, H5F_ACC_TRUNC_F, FILE_ID, ERR )
       THROWM(ERR.NE.0,"Cannot create File "//TRIM(FN))
      
    ELSE
       CALL H5FOPEN_F( FN, H5F_ACC_RDWR_F, FILE_ID, ERR )
       THROWM(ERR.NE.0,"Cannot open File "//TRIM(FN))
    ENDIF

    CALL MKPAR (FILE_ID, PATH,G_ID, ERR)
    THROWM(ERR.NE.0,"Cannot create group"//TRIM(PATH))

    SNAME="line"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,I0=CLUSTER%line)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    ANAME="COMMENT"
    call h5ltset_attribute_string_f(G_ID,sname,aname,"Line number",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))
    
    SNAME="for"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,I0=CLUSTER%for)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))    
    call h5ltset_attribute_string_f(G_ID,sname,aname,"field of regard number",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))

    SNAME='cluster_image'
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR, I2=CLUSTER%cluster_image)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    call h5ltset_attribute_string_f(G_ID,sname,aname,"cluster image",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))
    
    SNAME='cluster_on_viirs_grid'
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR, I2=CLUSTER%cluster_on_viirs_grid)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    call h5ltset_attribute_string_f(G_ID,sname,aname,"cluster image on VIIRS Grid",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))

    SNAME="flv"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,I0=FORMAP%flv)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME)) 
    call h5ltset_attribute_string_f(G_ID,sname,aname,"first line in VIIRS grid",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))
    SNAME="fpv"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,I0=FORMAP%fpv)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME)) 
    call h5ltset_attribute_string_f(G_ID,sname,aname,"first pixel in VIIRS grid",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname)) 
    
    SNAME="llv"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,I0=FORMAP%llv)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    call h5ltset_attribute_string_f(G_ID,sname,aname,"last line in VIIRS grid",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))

    SNAME="lpv"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,I0=FORMAP%lpv)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME)) 
    call h5ltset_attribute_string_f(G_ID,sname,aname,"last pixel in VIIRS grid",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))

    SNAME="nb_clusters"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,I0=CLUSTER%nbclass)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    call h5ltset_attribute_string_f(G_ID,sname,aname,"number of clusters",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))


    SNAME="RadAnaNbPix"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,I2=CLUSTER%RadAnaNbPix)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    call h5ltset_attribute_string_f(G_ID,sname,aname,&
        &  "For each Field of View/cluster: Number of pixels in the cluster",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))


    SNAME="RadAnaMean"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,R3=CLUSTER%RadAnaMean)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    call h5ltset_attribute_string_f(G_ID,sname,aname, &
      &   "For each Field of View/cluster/channel: Mean Radiance (units: W/(m2.sr.m-1)",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))
    
     SNAME="RadAnaStd"
    CALL WRITE_ARRAY_HDF(G_ID,SNAME," ", ERR,R3=CLUSTER%RadAnaStd)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    call h5ltset_attribute_string_f(G_ID,sname,aname, &
     &    "For each Field of View/cluster/channel: Standard deviation (units: W/(m2.sr.m-1)",err)
    THROWM(err.ne.0,"CANNOT WRITE ATTRIBUTE "//trim(aname))


    CALL H5GCLOSE_F( G_ID, ERR )
    THROWM(ERR.NE.0,"CANNOT CLOSE GROUP"//TRIM(PATH))

    CALL H5FCLOSE_F( FILE_ID, ERR )
    THROWM(ERR.NE.0,"CANNOT CLOSE "//TRIM(FN))

    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) TRIM(FN), " saved"


CATCH
    END SUBROUTINE

    !> Save mapping structure to HDF5 file.
    !> Add information on original and adjacency lat/lon/dist
    SUBROUTINE MAP_SAVE_DEBUG(OPTIONS, MAPPING, TABLE, ERR)
    IMPLICIT NONE
    TYPE(CFOVVIIRS),          INTENT(IN)  :: MAPPING
    TYPE(MAP_VIIRS_OPTIONS),  INTENT(IN)  :: OPTIONS
    TYPE(ADJTABLE),           INTENT(IN)  :: TABLE
    INTEGER,                  INTENT(OUT) :: ERR

    INTEGER(HID_T)     :: file_id
    REAL               :: P
    CHARACTER(LEN=32)  :: SNAME
    CHARACTER(LEN=256) :: FN
    REAL, ALLOCATABLE  :: TAB(:,:), LAT(:,:), LON(:,:)

TRY
    WRITE(FN,'("mapping-",a1,"-",i4.4,"-",i2.2,"-",i2.2,".H5")') &
          & OPTIONS%VIIRS_BAND, MAPPING%line, MAPPING%for,MAPPING%fov
    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) "save to ",TRIM(FN)

    CALL H5FCREATE_F( FN, H5F_ACC_TRUNC_F, FILE_ID, ERR )
    THROWM(ERR.NE.0,"Cannot create File "//TRIM(FN))

    ALLOCATE(TAB(2*MAPPING%RADIUSP+1,2*MAPPING%RADIUSL+1), STAT=ERR )
    THROWM(ERR.NE.0,"allocate error")
    ALLOCATE(LAT(2*MAPPING%RADIUSP+1,2*MAPPING%RADIUSL+1), STAT=ERR )
    THROWM(ERR.NE.0,"allocate error")
    ALLOCATE(LON(2*MAPPING%RADIUSP+1,2*MAPPING%RADIUSL+1), STAT=ERR )
    THROWM(ERR.NE.0,"allocate error")
    
    if( MAPPING%vlc-MAPPING%radiusl .gt. 0                        .and. &
      & MAPPING%vlc+MAPPING%radiusl .le. size(TABLE%LAT,2) .and. &
      & MAPPING%vpc-MAPPING%radiusp .gt. 0                        .and. &
      & MAPPING%vpc+MAPPING%radiusp .le. size(TABLE%LAT,1) ) THEN

      CALL MAP_DISTANCE_FROM_CENTER2( &
         & TABLE%LAT(MAPPING%vpc,MAPPING%vlc), &
         & TABLE%LON(MAPPING%vpc,MAPPING%vlc), &
         & TABLE%LAT(MAPPING%vpc-MAPPING%radiusp:MAPPING%vpc+MAPPING%radiusp, &
         &                  MAPPING%vlc-MAPPING%radiusl:MAPPING%vlc+MAPPING%radiusl), &
         & TABLE%LON(MAPPING%vpc-MAPPING%radiusp:MAPPING%vpc+MAPPING%radiusp, &
         &                   MAPPING%vlc-MAPPING%radiusl:MAPPING%vlc+MAPPING%radiusl), &
         & tab, ERR)
      THROW(ERR.NE.0)
      !>- Store array of original distance from target center
      SNAME='distance_orig'
      CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, units = 'm', R2=TAB)
      THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
      !>- Store array of original latitudes
      SNAME='latitude_orig'
      CALL WRITE_ARRAY_HDF(FILE_ID,SNAME,"Latitudes before adjacency", ERR, R2=&
            &    TABLE%LAT(MAPPING%VPC-MAPPING%RADIUSP:MAPPING%VPC+MAPPING%RADIUSP, &
            &                  MAPPING%VLC-MAPPING%RADIUSL:MAPPING%VLC+MAPPING%RADIUSL))
      THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
      !>- Store array of original longitudes
      SNAME='longitude_orig'
      CALL WRITE_ARRAY_HDF(FILE_ID,SNAME,"Longitudes before adjacency", ERR, R2=&
            &  TABLE%LON(MAPPING%VPC-MAPPING%RADIUSP:MAPPING%VPC+MAPPING%RADIUSP, &
            &                   MAPPING%VLC-MAPPING%RADIUSL:MAPPING%VLC+MAPPING%RADIUSL))
      THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    ENDIF
  
    ! VIIRS indices of adjacency pixels size (radiusp, raduisl)
    !>- Apply mapping on latitudes and longitudes arrays
    CALL MAP_APPLY( MAPPING, P, RMIS=MAPPING%LATITUDE,  R2=TABLE%LAT,  OR2=LAT)

    CALL MAP_APPLY( MAPPING, P, RMIS=MAPPING%LONGITUDE, R2=TABLE%LON, OR2=LON)
    !>- Compute distance from target center for mapped pixels
    CALL MAP_DISTANCE_FROM_CENTER2( &
       & TABLE%LAT(MAPPING%vpc,MAPPING%vlc), &
       & TABLE%LON(MAPPING%vpc,MAPPING%vlc), &
       & LAT, LON, tab, ERR)
    THROW(ERR.NE.0)
    !>- Store array of adjacency distance from target center
    SNAME='distance_adja'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, units = 'm', R2=TAB)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    !>- Store array of adjacency latitudes
    SNAME='latitude_adja'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, R2=LAT)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    !>- Store array of adjacency longitudes
    SNAME='longitude_adja'
    CALL WRITE_ARRAY_HDF(FILE_ID,SNAME," ", ERR, R2=LON)
    THROWM(ERR.NE.0,"CANNOT WRITE "//TRIM(SNAME))
    !>- Store cfovviirs structure
    CALL CFOVVIIRS_WH(MAPPING,FILE_ID,ERR, COMPRESS=.FALSE.)
    THROWM(ERR.NE.0,"CANNOT WRITE ADJTABLE IN FILE "//TRIM(FN))

    DEALLOCATE(TAB, LAT, LON)

    CALL H5FCLOSE_F( FILE_ID, ERR )
    THROWM(ERR.NE.0,"Cannot close File "//TRIM(FN))
    IF( OPTIONS%DEBUG .GE. 2 ) WRITE(*,*) TRIM(FN), " saved"

CATCH
    END SUBROUTINE



    !> Applies mapping on a global 2 dimensions input array.
    !> input:
    !>        - mapping structure.
    !>        - I2;R2 array to map, dimensions shoud be strictly the same as the
    !!           original SDR data (all pixels, all lines).
    !>        - IMis/RMis optional missing values (integer/real) that will fill the output
    !!           array in case of missing VIIRS data (default is -999/-999.).
    !>
    !> output:
    !>         - OI2;OR2 array mapped 
    !!           shape should be the same as the Mapping structure VP array (or VL array).
    !>         - P is the percentage of expected pixels used in mapping for Box (0-100%)
    !!            there could be less pixels inside the mapping if Only_fov or Use_weights is used.
    !>
    !> If ONLY_FOV: any pixel outside nominal field of view is set to missing value.
    !> If USE_WEIGHTS: pixels are weighted according to FOV shape.
    SUBROUTINE MAP_APPLY( MAPPING, P, IMIS, RMIS, I2, R2, OI2, OR2, &
                       & ONLY_FOV, USE_WEIGHTS)

    IMPLICIT NONE
    TYPE(CFOVVIIRS),   INTENT(IN)  :: MAPPING
    REAL,              INTENT(OUT) :: P
    INTEGER, OPTIONAL, INTENT(IN)  :: IMIS
    REAL,    OPTIONAL, INTENT(IN)  :: RMIS
    INTEGER, OPTIONAL, INTENT(IN)  :: I2(:,:)
    REAL,    OPTIONAL, INTENT(IN)  :: R2(:,:)
    LOGICAL, OPTIONAL, INTENT(IN)  :: USE_WEIGHTS
    LOGICAL, OPTIONAL, INTENT(IN)  :: ONLY_FOV
    INTEGER, OPTIONAL, INTENT(OUT) :: OI2(:,:)
    REAL,    OPTIONAL, INTENT(OUT) :: OR2(:,:)
    
    INTEGER :: I, J, NI, NJ, N
    INTEGER :: IMISS = -999
    REAL    :: RMISS = -999.
    LOGICAL :: UW, OF
    
    NI = SIZE( MAPPING%VP, 1 )
    NJ = SIZE( MAPPING%VP, 2 )

    UW = .FALSE.
    IF( PRESENT(USE_WEIGHTS) ) THEN
      UW = USE_WEIGHTS
    ENDIF
    
    OF = .FALSE.
    IF( PRESENT(ONLY_FOV) ) THEN
      OF = ONLY_FOV
    ENDIF

    IF( PRESENT(IMIS) ) THEN
      IMISS = IMIS
    ENDIF
    IF( PRESENT(RMIS) ) THEN
      RMISS = RMIS
    ENDIF

    N    = 0
    IF( PRESENT(I2) .AND. PRESENT(OI2)) THEN
      DO I = 1, NI
      DO J = 1, NJ
        IF( MAPPING%VL(I,J) .GT. 0 ) THEN
          OI2(I,J) = I2(MAPPING%VP(I,J), MAPPING%VL(I,J))
          N = N+1
        ELSE
          OI2(I,J) = IMISS
        ENDIF
      END DO
      END DO
      IF ( UW ) THEN
        OI2 = NINT(OI2 * MAPPING%WEIGHTS)
      ENDIF
      IF ( OF ) THEN
        WHERE ( MAPPING%ELLIPSE > 1. ) 
          OI2 = IMISS
        END WHERE
      ENDIF

    ELSEIF( PRESENT(R2) .AND. PRESENT(OR2)) THEN
      DO I = 1, NI
      DO J = 1, NJ
        IF( MAPPING%VL(I,J) .GT. 0 ) THEN
          OR2(I,J) = R2(MAPPING%VP(I,J), MAPPING%VL(I,J))
          N = N+1
        ELSE
          OR2(I,J) = RMISS
        ENDIF
      END DO
      END DO
      IF ( UW ) THEN
        OR2 = OR2 * MAPPING%WEIGHTS
      ENDIF
      IF ( OF ) THEN
        WHERE ( MAPPING%ELLIPSE > 1. ) 
          OR2 = IMISS
        END WHERE
      ENDIF

    ENDIF
    P = N*100. / (NI*NJ)

    END SUBROUTINE

    !> Applies mapping and convolution on a global 2 dimensions input array.
    !> input:
    !>        - mapping structure
    !>        - I2;R2 array to map, dimensions shoud be strictly the same as the
    !!           original SDR data (all pixels, all lines)
    !>        - QF optional array of quality flags (same size as I2/R2), convolution
    !!           is performed on valid pixels (default no test)
    !>
    !> output:
    !>         - I1;R1 convolved value 
    !>         - P is the percentage of expected pixels used in convolution (0-100%)
    !>
    !> If ONLY_FOV: any pixel outside nominal field of view is set to missing value
    !!              and weights of 0/1 are used;
    !!      if not: use CrIS FOV shape (see weights calculations in MAP_CRIS_FOV_ELLIPSE).
    !!
    !> Weights have a nominal sum of 1.0 inside FOV
    SUBROUTINE MAP_CONV( MAPPING, P, I2, R2, I1, R1, QF, ONLY_FOV)
    IMPLICIT NONE
    TYPE(CFOVVIIRS),   INTENT(IN)  :: MAPPING
    REAL,              INTENT(OUT) :: P
    INTEGER, OPTIONAL, INTENT(IN)  :: I2(:,:)
    REAL,    OPTIONAL, INTENT(IN)  :: R2(:,:)
    INTEGER, OPTIONAL, INTENT(IN)  :: QF(:,:)
    INTEGER, OPTIONAL, INTENT(OUT) :: I1
    REAL,    OPTIONAL, INTENT(OUT) :: R1
    LOGICAL, OPTIONAL, INTENT(IN)  :: ONLY_FOV

    INTEGER      :: I, J, N, NI, NJ, Z
    REAL(KIND=8) :: RSUM, RW
    LOGICAL :: OF
    REAL    :: RMISS = -999.

    NI = SIZE( MAPPING%VP, 1 )
    NJ = SIZE( MAPPING%VP, 2 )
    OF = .FALSE.
    IF( PRESENT(ONLY_FOV) ) THEN
      OF = ONLY_FOV
    ENDIF


    N    = 0
    Z    = 0 ! number of bad pixels inside perimeter
    RSUM = 0.D0
    RW   = 0.D0
    IF ( PRESENT(QF) ) THEN
      IF( PRESENT(I2) .AND. PRESENT(I1) .AND. OF) THEN
        DO I = 1, NI
        DO J = 1, NJ
          IF( MAPPING%ELLIPSE(I,J) .LE. 1. .AND. MAPPING%VL(I,J) .GT. 0 .AND. QF(MAPPING%VP(I,J), MAPPING%VL(I,J)).EQ.0 ) THEN
            RSUM = RSUM + I2(MAPPING%VP(I,J), MAPPING%VL(I,J))
            N = N+1
          ELSEIF( MAPPING%ELLIPSE(I,J) .LE. 1.) THEN
            Z = Z+1
          ENDIF
        END DO
        END DO
        IF( N .GT. 0 ) THEN
          RSUM = RSUM / N
        ELSE
          RSUM = RMISS
        ENDIF
        I1 = NINT(RSUM)
        P = MIN(N*100.0_8 / (N + Z), 100.0_8)

      ELSEIF( PRESENT(R2) .AND. PRESENT(R1) .AND. OF) THEN
        DO I = 1, NI
        DO J = 1, NJ
          IF( MAPPING%ELLIPSE(I,J) .LE. 1. .AND. MAPPING%VL(I,J) .GT. 0 .AND. QF(MAPPING%VP(I,J), MAPPING%VL(I,J)).EQ.0 ) THEN
            RSUM = RSUM + R2(MAPPING%VP(I,J), MAPPING%VL(I,J))
            N = N+1
          ELSEIF( MAPPING%ELLIPSE(I,J) .LE. 1.) THEN
            Z = Z+1
          ENDIF
        END DO
        END DO
        IF( N .GT. 0 ) THEN
          R1 = RSUM / N
        ELSE
          R1 = RMISS
        ENDIF
        P = MIN(N*100.0_8 / (N + Z), 100.0_8)

      ELSEIF( PRESENT(I2) .AND. PRESENT(I1)) THEN
        DO I = 1, NI
        DO J = 1, NJ
          IF( MAPPING%VL(I,J) .GT. 0 .AND. QF(MAPPING%VP(I,J), MAPPING%VL(I,J)).EQ.0 ) THEN
            RSUM = RSUM + I2(MAPPING%VP(I,J), MAPPING%VL(I,J)) * &
                     &  MAPPING%WEIGHTS(I,J)
            RW = RW + MAPPING%WEIGHTS(I,J)
          ENDIF
        END DO
        END DO
        IF( RW .GT. 0. ) THEN
          RSUM = RSUM / RW
        ELSE
          RSUM = RMISS
        ENDIF
        I1 = NINT(RSUM)
        P = MIN(RW*100.0_8, 100.0_8)  ! total sum of weight should be one
      
      ELSEIF( PRESENT(R2) .AND. PRESENT(R1)) THEN
        DO I = 1, NI
        DO J = 1, NJ
          IF( MAPPING%VL(I,J) .GT. 0 ) THEN
            IF (QF(MAPPING%VP(I,J), MAPPING%VL(I,J)).EQ.0 ) THEN
              RSUM = RSUM + R2(MAPPING%VP(I,J), MAPPING%VL(I,J))* &
                    &   MAPPING%WEIGHTS(I,J)
              RW = RW + MAPPING%WEIGHTS(I,J)
            ENDIF
          ENDIF
        END DO
        END DO
        IF( RW .GT. 0. ) THEN
          R1 = RSUM / RW
        ELSE
          R1 = RMISS
        ENDIF
        P = MIN(RW*100.0_8, 100.0_8)  ! total sum of weight should be one

      ENDIF
    ELSE
      IF( PRESENT(I2) .AND. PRESENT(I1) .AND. OF) THEN
        DO I = 1, NI
        DO J = 1, NJ
          IF( MAPPING%ELLIPSE(I,J) .LE. 1. .AND. MAPPING%VL(I,J) .GT. 0) THEN
            RSUM = RSUM + I2(MAPPING%VP(I,J), MAPPING%VL(I,J))
            N = N+1
          ELSEIF( MAPPING%ELLIPSE(I,J) .LE. 1.) THEN
            Z = Z+1
          ENDIF
        END DO
        END DO
        IF( N .GT. 0 ) THEN
          RSUM = RSUM / N
        ELSE
          RSUM = RMISS
        ENDIF
        I1 = NINT(RSUM)
        P = MIN(N*100.0_8 / (N + Z), 100.0_8)
        
      ELSEIF( PRESENT(R2) .AND. PRESENT(R1) .AND. OF) THEN
        DO I = 1, NI
        DO J = 1, NJ
          IF( MAPPING%ELLIPSE(I,J) .LE. 1. .AND. MAPPING%VL(I,J) .GT. 0) THEN
            RSUM = RSUM + R2(MAPPING%VP(I,J), MAPPING%VL(I,J))
            N = N+1
          ELSEIF( MAPPING%ELLIPSE(I,J) .LE. 1.) THEN
            Z = Z+1
          ENDIF
        END DO
        END DO
        IF( N .GT. 0 ) THEN
          R1 = RSUM / N
        ELSE
          R1 = RMISS
        ENDIF
        P = MIN(N*100.0_8 / (N + Z), 100.0_8)

      ELSEIF( PRESENT(I2) .AND. PRESENT(I1)) THEN
        DO I = 1, NI
        DO J = 1, NJ
          IF( MAPPING%VL(I,J) .GT. 0) THEN
            RSUM = RSUM + I2(MAPPING%VP(I,J), MAPPING%VL(I,J)) * &
                     &  MAPPING%WEIGHTS(I,J)
            RW = RW + MAPPING%WEIGHTS(I,J)
          ENDIF
        END DO
        END DO
        IF( RW .GT. 0. ) THEN
          RSUM = RSUM / RW
        ELSE
          RSUM = RMISS
        ENDIF
        I1 = NINT(RSUM)
        P = MIN(RW*100.0_8, 100.0_8)  ! total sum of weight should be one

      ELSEIF( PRESENT(R2) .AND. PRESENT(R1)) THEN
        DO I = 1, NI
        DO J = 1, NJ
          IF( MAPPING%VL(I,J) .GT. 0) THEN
            RSUM = RSUM + R2(MAPPING%VP(I,J), MAPPING%VL(I,J))* &
                    &   MAPPING%WEIGHTS(I,J)
            RW = RW + MAPPING%WEIGHTS(I,J)
          ENDIF
        END DO
        END DO
        IF( RW .GT. 0. ) THEN
          R1 = RSUM / RW
        ELSE
          R1 = RMISS
        ENDIF
        P = MIN(RW*100.0_8, 100.0_8)  ! total sum of weight should be one

      ENDIF
    ENDIF
    
    END SUBROUTINE

  
    !> Prepare storage for CrIS 1D header AAPP format
    SUBROUTINE MAP_CONV_SDR_1D_H(OPTIONS, SDR, &
      & CRIS_1DNCHANVIIRS, CRIS1D_H_NVIIRSCHAN, CRIS1D_H_NVIIRSCHANUSED, &
      & CRIS1D_H_VIIRSCHANS, CRIS1D_H_VIIRSBANDS )
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MOD

    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(VIIRS_SDR), INTENT(IN)  :: SDR
    INTEGER,         INTENT(IN)  :: CRIS_1DNCHANVIIRS   ! Number of VIIRS channels in format
    INTEGER,         INTENT(OUT) :: CRIS1D_H_NVIIRSCHAN       !number of VIIRS channels in format
    INTEGER,         INTENT(OUT) :: CRIS1D_H_NVIIRSCHANUSED   !number of channels used
    INTEGER,         INTENT(OUT) :: CRIS1D_H_VIIRSCHANS(CRIS_1DNCHANVIIRS)  !Channel numbers
    INTEGER,         INTENT(OUT) :: CRIS1D_H_VIIRSBANDS(CRIS_1DNCHANVIIRS)  !Band numbers
                                                         ! 5=M, 6=I, 7-DN
    INTEGER :: I, IC
    
    ! In ATOVPP (sub ppcouth) we have:
    !  cris1d_h_nviirschan = cris_1dnchanviirs
    !  cris1d_h_nviirschanused = 0      !None at this stage
    ! So ensure that the index arrays are correctly set to 0
    !  if first time we fill this level 1D file
    IF(  CRIS1D_H_NVIIRSCHANUSED .EQ. 0 ) THEN
      CRIS1D_H_VIIRSCHANS(:)  = 0
      CRIS1D_H_VIIRSBANDS(:)  = 0    
    ENDIF


    ! The band number comes from BUFR descriptor 008076:
    ! http://www.wmo.int/pages/prog/www/WMOCodes/WMO306_vI2/LatestVERSION/WMO306_vI2_BUFRCREX_CodeFlag_en.pdf
    ! Channels indices
    !  1-16 M channels
    ! 17-21 I channels
    !    22 DNB
    DO I=1,SIZE(SDR%BAND)
      IF(ASSOCIATED(SDR%BAND(I)%RADIANCE) .OR. &
       & ASSOCIATED(SDR%BAND(I)%REFLECTANCES)) THEN
        IF( SDR%BAND(I)%CHANNEL_NAME(1:1) .EQ. "I" ) THEN
          IC = I + 16
          CRIS1D_H_VIIRSBANDS(IC) = 6
        ELSE IF( SDR%BAND(I)%CHANNEL_NAME(1:1) .EQ. "M" ) THEN
          IC = I 
          CRIS1D_H_VIIRSBANDS(IC) = 5
        ELSE
          IC = 22
          CRIS1D_H_VIIRSBANDS(22) = 7
        ENDIF
        CRIS1D_H_VIIRSCHANS(IC) = I
      ENDIF
    END DO

    ! calculates the number of stored channels
    CRIS1D_H_NVIIRSCHAN     = 0
    CRIS1D_H_NVIIRSCHANUSED = 0
    DO I = 1, CRIS_1DNCHANVIIRS
      IF( CRIS1D_H_VIIRSBANDS(I) .NE. 0 ) THEN
        CRIS1D_H_NVIIRSCHAN     = CRIS1D_H_NVIIRSCHAN +1
        CRIS1D_H_NVIIRSCHANUSED = CRIS1D_H_NVIIRSCHANUSED +1
      ENDIF
    END DO
    IF( OPTIONS%DEBUG .GE. 2 ) THEN
      WRITE(*,*) "cris1d_h_nviirschan     ",CRIS1D_H_NVIIRSCHAN
      WRITE(*,*) "cris1d_h_nviirschanused ",CRIS1D_H_NVIIRSCHANUSED
      WRITE(*,*) "cris1d_h_viirsbands     ",CRIS1D_H_VIIRSBANDS
      WRITE(*,*) "cris1d_h_viirschans     ",CRIS1D_H_VIIRSCHANS
    ENDIF
    
    END SUBROUTINE

    !> Do convolution for all available channels in SDR file for current mapping.
    !> Prepare storage for CrIS 1D format, using simple arrays.
    SUBROUTINE MAP_CONV_SDR_1D_D(OPTIONS, MAPPING, SDR,     &
               & CRIS_1DNCHANVIIRS,    CRIS_1DNFOR,         &
               & CRIS1D_H_VIIRSCHANS, &
               & CRIS1D_VIIRSRADIANCE, CRIS1D_VIIRSREFLECTANCE )
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MOD

    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFOVVIIRS),  INTENT(IN)  :: MAPPING
    TYPE(VIIRS_SDR),  INTENT(IN)  :: SDR
    INTEGER,          INTENT(IN)  :: CRIS_1DNCHANVIIRS   ! Number of VIIRS channels in format
    INTEGER,          INTENT(IN)  :: CRIS_1DNFOR         ! Number of FORs present in 1d
    INTEGER,          INTENT(IN)  :: CRIS1D_H_VIIRSCHANS(CRIS_1DNCHANVIIRS)  !Channel numbers
    INTEGER,          INTENT(OUT) :: CRIS1D_VIIRSRADIANCE(CRIS_1DNCHANVIIRS,CRIS_1DNFOR)! mW/(m^2 sr cm^-1) *1E7
    INTEGER,          INTENT(OUT) :: CRIS1D_VIIRSREFLECTANCE(CRIS_1DNCHANVIIRS,CRIS_1DNFOR)

    INTEGER :: I, SDRCHAN
    REAL    :: RAD, REFL, P
    INTEGER,POINTER,DIMENSION(:,:)::QF1_VIIRS ! Band Quality Flag for each pixel [pixels, lines]
    
    DO I=1,CRIS_1DNCHANVIIRS
    ! SDRCHAN is (1-16 for M, 1-5 for I, 1 for DNB)
      SDRCHAN = CRIS1D_H_VIIRSCHANS(I)
      IF(OPTIONS%DEBUG .GE. 2) WRITE(*,*) "CHAN",I,"SDRCHAN",SDRCHAN  
      IF( SDRCHAN .GT. 0 ) THEN
        IF( SDR%BAND(SDRCHAN)%CHANNEL_NAME(1:1) .EQ. "M" ) THEN
          QF1_VIIRS => SDR%BAND(SDRCHAN)%QF1_VIIRSMBANDSDR
        ELSEIF( SDR%BAND(SDRCHAN)%CHANNEL_NAME(1:1) .EQ. "I" ) THEN
          QF1_VIIRS => SDR%BAND(SDRCHAN)%QF1_VIIRSIBANDSDR
        ELSE
          QF1_VIIRS => SDR%BAND(SDRCHAN)%QF1_VIIRSDNBBANDSDR
        ENDIF
      ENDIF
      IF( SDRCHAN .GT. 0 ) THEN
        IF(ASSOCIATED(SDR%BAND(SDRCHAN)%RADIANCE)) THEN
          !>- Use the CrIS FOV shape (ONLY_FOV=.False.) and the channel quality flag
          CALL MAP_CONV( MAPPING, P, R2=SDR%BAND(SDRCHAN)%RADIANCE, R1=RAD, &
            & QF=QF1_VIIRS, ONLY_FOV=.False.)

          !>- VIIRS Radiances are stored in SDRs as W/(m^2 sr micron)
          !! we want radiances in the same unit as CrIS W/(m^2 sr cm^-1)*1E7
                  
          !>- Consider FOV only if percentage of usefull pixels in FOV
          !! is higher than threshold given in option
          IF( RAD .GT. 0.  .AND. P .GE. OPTIONS%THRESHOLDGP ) THEN
            RAD = RAD * VIIRSSRF(I)%RadtoCRISunitfactor
            CRIS1D_VIIRSRADIANCE(I,MAPPING%FOR) = RAD 
          ELSE
            CRIS1D_VIIRSRADIANCE(I,MAPPING%FOR) = 0
          ENDIF
        ENDIF
        
        IF(ASSOCIATED(SDR%BAND(SDRCHAN)%REFLECTANCES)) THEN
          CALL MAP_CONV( MAPPING, P, R2=SDR%BAND(SDRCHAN)%REFLECTANCES, R1=REFL)
          IF( P .GE. OPTIONS%THRESHOLDGP ) THEN
            CRIS1D_VIIRSREFLECTANCE(I,MAPPING%FOR) = REFL *1.0D7
          ELSE
            CRIS1D_VIIRSREFLECTANCE(I,MAPPING%FOR) = 0
          ENDIF
        ENDIF
      ENDIF
      NULLIFY(QF1_VIIRS)
    END DO

    END SUBROUTINE

    !> Do clustering for all available channels in SDR file for current mapping.
    !> Prepare storage for CrIS 1D format, using simple arrays.
    SUBROUTINE MAP_CLUSTER_SDR (OPTIONS, MAPPING, SDR, &
               & VIIRSLOC, CLUSTERPARAM, CLUSTER, ERR )
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MOD

    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFORVIIRS),  INTENT(INOUT)  :: MAPPING ! WARNING HERE MAPPING IS A FORMAP
    TYPE(VIIRS_SDR),  INTENT(IN)  :: SDR
    TYPE(VIIRS_LOCATION),INTENT(IN) :: VIIRSLOC
    TYPE(CLUSTERPARAMETER), INTENT(IN) ::CLUSTERPARAM
    TYPE(CLUSTERVIIRS),  INTENT(OUT) :: CLUSTER  ! Cluster structure
    INTEGER ,  INTENT(OUT) :: ERR
    INTEGER :: I, P,L,NL,NP,K, NCHAN, errcode, nbl,nbp
    INTEGER :: offset_chan
TRY
    ERR = 0

    IF (OPTIONS%VIIRS_BAND .eq. 'M') THEN
          offset_chan = 0
    ELSE 
        IF (OPTIONS%VIIRS_BAND .eq. 'I') THEN
          offset_chan = 16
        ELSE
          offset_chan = 21
        ENDIF
    ENDIF
    CLUSTER%line = MAPPING%line 
    CLUSTER%for = MAPPING%for

    cluster % radvect  = -999.
    NCHAN =  0
    write(*,*) "clusters for line ",MAPPING%line," for ",MAPPING%for
    NP = SIZE( MAPPING%VP, 1)
    NL = SIZE( MAPPING%VP, 2)
    cluster%nbcol = NP
    cluster%nblin = NL
    write(*,*) "cluster%nbcol cluster%nblin :" ,cluster%nbcol ,cluster%nblin 
    IF (associated(cluster%cluster_image)) THEN
         write(*,*) "ici deallocate cluster_image"
         deallocate(cluster%cluster_image)
         THROWM(ERR.NE.0,"Cannot  deallocate cluster%cluster_image")
    ENDIF
    allocate(cluster%cluster_image(np,nl),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate cluster%cluster_image")

    IF (.not. cluster%first_time) THEN
         deallocate(cluster%mask)
         THROWM(ERR.NE.0,"Cannot  deallocate cluster%mask array")
    ENDIF
    allocate(cluster%mask(np,nl),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate cluster%mask")
    

    DO I=1,SIZE(SDR%BAND)             
      IF(ASSOCIATED(SDR%BAND(I)%RADIANCE)) THEN
        write(*,*) "compute rad vect for channel",i
        NCHAN = NCHAN + 1
        k = CL_MAXLENGH * (NCHAN -1)
        write(*,*) "start here k:",k, " NL:",NL," NP:", NP," NCHAN",NCHAN
        ! compute mask
        IF (NCHAN .EQ. 1) THEN
           cluster % mask = 0
           DO L=1,NL
               DO P=1,NP   
                    ! maskbowtie = True if "missing data" 
                    ! pixqual = True if bad quality pixel in one channel
                     IF ( mapping%vp(p,l) .GT. 1 .and. mapping%vl(p,l) .GT. 1 ) THEN                
                       IF ( viirsloc%maskbowtie(mapping%vp(p,l),mapping%vl(p,l)) .OR. &
                       &    viirsloc%pixqual(mapping%vp(p,l),mapping%vl(p,l)) ) THEN
                         cluster % mask(p,l) = 1
                       ENDIF
                     ELSE
                       cluster % mask(p,l) = 1
                     ENDIF
               ENDDO
           ENDDO
        ENDIF
        ! compute radiance in SI unit
        DO L=1,NL
            DO P=1,NP   
                IF ( cluster % mask(p,l) .EQ. 0 ) THEN
                   k = k + 1
                   cluster % radvect ( k ) = &
                   VIIRSSRF(I+offset_chan)%RadtoSIfactor  * SDR%BAND(I)%RADIANCE(mapping%vp(p,l),mapping%vl(p,l)) 
                ENDIF
            END DO
        END DO
 
        IF ( NCHAN .EQ. 1 ) THEN
           cluster%nb_pixels = k
        ENDIF
      ENDIF ! IF ASSOCIATED
    END DO ! DO  I=1,SIZE(SDR%BAND)

    ! Call compute clusters
       IF (OPTIONS%DEBUG == 2 )THEN
         write(*,*) "call clusters" , "nb_pixels =",cluster%nb_pixels
         write(*,*) "cluster SortChan",clusterparam%SortChan
         write(*,*) "cluster Nch",clusterparam%Nch
         write(*,*) "Noise", clusterparam%Noise
         write(*,*) "Weight", clusterparam%Weight
      ENDIF
      if ( cluster%nb_pixels .le. options%THRESHOLDGP ) then
        write(*,*) "not enough valid pixels to call clusters"
        err= 1
        return
     endif
     call clustersnothread(cluster%radvect, cluster%nb_pixels, clusterparam%Nch, &
                            clusterparam%SortChan, clusterparam%Noise, clusterparam%Weight, &
                            cluster%IdPixel, cluster%nbclass, errcode)
      err = errcode
    ! restitute image
    cluster% cluster_image = CL_NO_CLASSIFIED ! no classified
    k = 0
    DO L=1,NL
       DO P=1,NP   
            IF ( cluster % mask(p,l) .EQ. 0 ) THEN
                k =  k + 1
                cluster% cluster_image(P,L) = cluster % IdPixel ( k ) 
            ENDIF
        END DO
    END DO

    ! reproj cluster_image on cluster_on_viirs_grid
    IF ( .not. cluster%first_time ) THEN
       deallocate(cluster%cluster_on_viirs_grid)
       THROWM(ERR.NE.0, "Cannot deallocate cluster_on_viirs_grid")
    ELSE
       write(*,*) "first time allocate cluster%cluster_on_viirs_grid"
    ENDIF 
    nbl = mapping%llv - mapping%flv + 1
    nbp = mapping%lpv - mapping%fpv + 1
    write(*,*) "nbp", nbp,"nbl",nbl
    allocate(cluster%cluster_on_viirs_grid(nbp,nbl))
    THROWM(ERR.NE.0, "Cannot allocate cluster_on_viirs_grid")
     
     cluster%cluster_on_viirs_grid = 0
     DO L=1,NL
       DO P=1,NP
         if (mapping%vp(p,l) .ge. 1 .and. mapping%vl(p,l) .ge. 1 ) then
         cluster%cluster_on_viirs_grid(mapping%vp(p,l)- mapping%fpv + 1,mapping%vl(p,l) - mapping%flv + 1)= &
                                  cluster% cluster_image(P,L)
         endif
       END DO
    END DO
  
    ! compute statistics on each fov and for each class
    CALL MAP_CONV_CLUSTER(OPTIONS,mapping,sdr,cluster, ERR)

    IF (OPTIONS%DEBUG == 2 ) THEN
      write(*,*) "save format for line:",mapping%line, " and for:",mapping%for
      CALL MAP_FORMAP_SAVE(OPTIONS, mapping, ERR)
      THROWM(ERR.NE.0, "Cannot save FORMAP")
    
    ENDIF 
    cluster%first_time = .False.
CATCH
    END SUBROUTINE

    !> Do convolution for MAIA4 data (version >=2).
    !> Prepare storage for CrIS 1D format, using classical arrays.
    !> Convolution is done taking into account for the CrIS ellipse but 
    !!   no weights applied. We want the true cloud information inside the FOV
    !!   so ONLY_FOV option is true
    SUBROUTINE MAP_CONV_MAIA4(OPTIONS, MAPPING, MAIA4, &
                            & CRIS1C_LANDFRAC,   &
                            & CRIS1C_LANDSEA,    &
                            & CRIS1C_CLOUDCOVER, &
                            & CRIS1C_CLOUDTOPHT, &
                            & CRIS1D_CLOUDFLAG, ERR)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFOVVIIRS),    INTENT(IN)  :: MAPPING
    TYPE(VIIRS_MAIA4),  INTENT(IN)  :: MAIA4
    INTEGER,            INTENT(OUT) :: CRIS1D_CLOUDFLAG(10) !CrIS cloud flag (user defined)
    INTEGER,            INTENT(OUT) :: CRIS1C_LANDFRAC     ! land fraction
    INTEGER,            INTENT(OUT) :: CRIS1C_LANDSEA      ! land/sea qualifier
    INTEGER,            INTENT(OUT) :: CRIS1C_CLOUDCOVER   ! cloud cover (percent)
    INTEGER,            INTENT(OUT) :: CRIS1C_CLOUDTOPHT   ! height of top of cloud (m) NOT AVAILABLE
    INTEGER,            INTENT(OUT) :: ERR

    INTEGER(KIND=4) :: I, J, NP, IMIS, IMASK
    REAL            :: P
    REAL(KIND=8)    :: RSUM
    INTEGER         :: IW
    INTEGER         :: NCLEAR, NCLOUDY, NCLEARS, NCLEARI, NAERO

    INTEGER(KIND=4), ALLOCATABLE :: TAB(:,:), TAB1(:,:), TAB2(:,:), TAB3(:,:)
TRY

    CRIS1D_CLOUDFLAG(:) = Z'FFFF'
    CRIS1C_LANDFRAC     = 100
    CRIS1C_LANDSEA      =   3
    CRIS1C_CLOUDCOVER   = 100
    CRIS1C_CLOUDTOPHT   =   0

    IF( MAIA4%FORMAT_VERSION .EQ. 1 ) THEN
       RETURN
    ENDIF

    ALLOCATE(TAB (2*MAPPING%RADIUSP+1,2*MAPPING%RADIUSL+1), &
         &   TAB1(2*MAPPING%RADIUSP+1,2*MAPPING%RADIUSL+1), &
         &   TAB2(2*MAPPING%RADIUSP+1,2*MAPPING%RADIUSL+1), &
         &   TAB3(2*MAPPING%RADIUSP+1,2*MAPPING%RADIUSL+1), &
         &   STAT=ERR )
    THROWM(ERR.NE.0,"allocate error")

!new global attribute : "format_version" currently equal to 2
! Cloud Mask
!   bits 0-2 Cloud Mask on pixels   (before Cloud Adjacency) 
!     0= clear
!     1= cloudy
!     2= clear over snow
!     3= clear over ice
!     4= aerosol_dust_ash_fire
!    bits 3-4  Cloud Mask Quality from the number of tests involved
!     3=high
!     2=medium
!     1=low
!     0=bad
!   bits 5-6  Cloud Mask Confidence from the proximity of thresholds
!     3=Clear
!     2=ProbablyClear
!     1=ProbablyCloudy
!     0=Cloudy
!   bits 7-9  Adjacent Pixel Cloud Mask from cloud confidence evaluation
!      of surrounding pixels 0=Not tested
!     0=Not tested
!     1=Clear
!     2=ProbablyClear
!     3=ProbablyCloudy
!     4=Cloudy
!   bits 10-12 Cloud Mask total (after Cloud Adjacency process If bits7-9==0 same as bits0-2)
!     0= clear
!     1= cloudy 
!     2= clear over snow 
!     3= clear over ice
!     4= aerosol_dust_ash_fire
!   bits 15-17 Land/Water Background      
!                                         0=Sea+inland Water
!                                          1=mixed
!                                          2=land
!                                          3=desert
!                                          4=snow
!   bits 18 Land surface type              0=no coast
!              (from landsea atlas)        1=coast
!
! CLOUD Top Pressure (hPa * 10) Missing value : -9999
! CLOUD Top Temperature (Celsius * 100) Missing value : -9999
! Retrieved surface temperature (Celsius * 100) for confident clear: retrieved surface
!    temperature in C*100:
!       over sea : SST Split-Window
!       over land: LST from RTE using forecast and emissivity atlas
!    Missing value: -9999
! FovQual bits 0-1 input info quality
!    0= ok, mask done with highest information quality for cloud mask
!    1= poor, some channels are discarded
!    2= bad, cloud mask not done


!>- CrIS 1D output
!> - 1 % clear
!> - 2 % cloudy
!> - 3 % clear over snow
!> - 4 % clear over ice
!> - 5 % aerosol_dust_ash_fire
!> - 6 mean CLOUD Top Pressure (hPa * 10) Missing value : -9999
!> - 7 mean CLOUD Top Temperature (Celsius * 100) Missing value : -9999
!> - 8 mean surface temperature over sea (Celsius * 100) Missing value : -9999
!> - 9 mean surface temperature over land (Celsius * 100) Missing value : -9999
!> -10 Pixel quality flags:  % of good  quality flags (fov_qual==0)

!>- CrIS 1C output
!> - cris1c_landfrac (0-100%)
!> - cris1c_landsea
!>  -   0 08 012
!>  -     Land/sea qualifier
!>  -     Code figure
!>  -     0 Land
!>  -     1 Sea
!>  -     2 Coast
!>  -     3 Missing value 
!> - cris1c_cloudcover (0-100%)
!> - cris1c_cloudtopht (m)
!> @todo Check LandSea and CloudTopHeight are correctly coded to BUFR in AAPP (no need for scaling)


    ! Number of VIIRS pixels in CrIS FOV
    TAB = 0
    WHERE ( MAPPING%ELLIPSE .LE. 1. ) 
      TAB=1
    END WHERE
    NP = SUM(TAB)

    ! Cloud mask on the Field of View only:
    IMIS = Z'FFFF'
    CALL MAP_APPLY( MAPPING, P, IMIS=IMIS,  &
                  & I2=MAIA4%DATA%CLOUDMASK,  OI2=TAB, ONLY_FOV=.TRUE.)

    ! First call for Map_apply, so test percentage of data available
    IF ( P .GT. OPTIONS%THRESHOLDGP ) THEN

      ! Clear/Cloudy mask choose either bits 0-2 or 10-12
      ! bits 0-2
      IMASK = Z'00000007'
      TAB1 = IAND(TAB, IMASK)
      ! bits 10-12
      !IMASK = Z'00001C00'
      !TAB1 = IAND(TAB, IMASK)
      !TAB1 = ISHFT(TAB1,-10)

      !   bits 5-6  Cloud Mask Confidence from the proximity of thresholds
      IMASK = Z'00000060'
      TAB3 = IAND(TAB, IMASK)
      TAB3 = ISHFT(TAB3,-5)

      ! Land/Sea Mask bits 15-17
      IMASK = Z'00038000'
      TAB2 = IAND(TAB, IMASK)
      TAB2 = ISHFT(TAB2,-15)
      
      NCLEAR  = 0
      NCLOUDY = 0
      NCLEARS = 0
      NCLEARI = 0
      NAERO   = 0

      DO I = 1, SIZE( MAPPING%VP, 1 )
      DO J = 1, SIZE( MAPPING%VP, 2 )
        ! We take the following definition fo CLEAR:
        !     Clear
        !  or Cloudy and Probably Clear
        IF( TAB1(I,J) .EQ. 0 ) THEN
          NCLEAR = NCLEAR +1
        ELSEIF( TAB1(I,J) .EQ. 1 .AND. TAB3(I,J) .EQ. 2) THEN
          NCLEAR = NCLEAR +1
        ELSEIF( TAB1(I,J) .EQ. 1 ) THEN
          NCLOUDY = NCLOUDY +1
        ELSEIF( TAB1(I,J) .EQ. 2 ) THEN
          NCLEARS = NCLEARS +1
        ELSEIF( TAB1(I,J) .EQ. 3 ) THEN
          NCLEARI = NCLEARI +1
        ELSEIF( TAB1(I,J) .EQ. 4 ) THEN
          NAERO = NAERO +1
        ENDIF
      END DO
      END DO

      CRIS1D_CLOUDFLAG(1) = NINT(NCLEAR*100./ NP)
      CRIS1D_CLOUDFLAG(2) = NINT(NCLOUDY*100./ NP)
      CRIS1D_CLOUDFLAG(3) = NINT(NCLEARS*100./ NP)
      CRIS1D_CLOUDFLAG(4) = NINT(NCLEARI*100./ NP)
      CRIS1D_CLOUDFLAG(5) = NINT(NAERO*100./ NP)

      ! Cloud Top Pressure
      IMIS = -9999
      CALL MAP_APPLY( MAPPING, P, IMIS=IMIS,  &
                    & I2=MAIA4%DATA%CloudTopPres,  OI2=TAB, ONLY_FOV=.TRUE.)
      RSUM = 0.D0
      IW   = 0
      DO I = 1, SIZE( MAPPING%VP, 1 )
      DO J = 1, SIZE( MAPPING%VP, 2 )
        IF( TAB(I,J) .GT. 0) THEN
          RSUM = RSUM + TAB(I,J) 
          IW = IW + 1
        ENDIF
      END DO
      END DO
      IF( IW .GT. 0 ) THEN
        CRIS1D_CLOUDFLAG(6) = NINT(RSUM/ IW)
      ELSE
        CRIS1D_CLOUDFLAG(6) = -9999
      ENDIF

      ! Cloud Top Temperature
      IMIS = -9999
      CALL MAP_APPLY( MAPPING, P, IMIS=IMIS,  &
                    & I2=MAIA4%DATA%CLOUDTOPTEMP,  OI2=TAB, ONLY_FOV=.TRUE.)
      RSUM = 0.D0
      IW   = 0
      DO I = 1, SIZE( MAPPING%VP, 1 )
      DO J = 1, SIZE( MAPPING%VP, 2 )
        IF( TAB(I,J) .GT. -9999) THEN
          RSUM = RSUM + TAB(I,J) 
          IW = IW + 1
        ENDIF
      END DO
      END DO
      IF( IW .GT. 0 ) THEN
        CRIS1D_CLOUDFLAG(7) = NINT(RSUM/ IW)
      ELSE
        CRIS1D_CLOUDFLAG(7) = -9999
      ENDIF

      ! surface temperature over sea (TAB2=0)
      IMIS = -9999
      CALL MAP_APPLY( MAPPING, P, IMIS=IMIS,  &
                    & I2=MAIA4%DATA%TSURF,  OI2=TAB, ONLY_FOV=.TRUE.)
      RSUM = 0.D0
      IW   = 0
      DO I = 1, SIZE( MAPPING%VP, 1 )
      DO J = 1, SIZE( MAPPING%VP, 2 )
        IF( TAB(I,J) .GT. -9999 .AND. TAB2(I,J) .EQ. 0) THEN
          RSUM = RSUM + TAB(I,J) 
          IW = IW + 1
        ENDIF
      END DO
      END DO
      IF( IW .GT. 0 ) THEN
        CRIS1D_CLOUDFLAG(8) = NINT(RSUM/ IW)
      ELSE
        CRIS1D_CLOUDFLAG(8) = -9999
      ENDIF

      !surface temperature over land (TAB2=2 or 3)
      RSUM = 0.D0
      IW   = 0
      DO I = 1, SIZE( MAPPING%VP, 1 )
      DO J = 1, SIZE( MAPPING%VP, 2 )
        IF( TAB(I,J) .GT. -9999 .AND. (TAB2(I,J) .EQ. 2 .OR. TAB2(I,J) .EQ. 3)) THEN
          RSUM = RSUM + TAB(I,J) 
          IW = IW + 1
        ENDIF
      END DO
      END DO
      IF( IW .GT. 0 ) THEN
        CRIS1D_CLOUDFLAG(9) = NINT(RSUM/ IW)
      ELSE
        CRIS1D_CLOUDFLAG(9) = -9999
      ENDIF

      !Pixel quality flags
      IMIS = Z'FFFF'
      CALL MAP_APPLY( MAPPING, P, IMIS=IMIS,  &
                    & I2=MAIA4%DATA%FOV_QUAL,  OI2=TAB, ONLY_FOV=.TRUE.)
      IMASK = Z'00000003'
      TAB = IAND(TAB, IMASK)

      ! Good flags, consider MAIA flags 0 (good) and 1 (poor) 
      ! with a weight of 1 and 0.5 respectively
      ! flag 1 is take into account because this fov_qual is very sensible 
      ! and often sets this flag
      RSUM   = 0
      DO I = 1, SIZE( MAPPING%VP, 1 )
      DO J = 1, SIZE( MAPPING%VP, 2 )
        IF( TAB(I,J) .EQ. 0) THEN
          RSUM = RSUM + 1
        ELSEIF( TAB(I,J) .EQ. 1) THEN
          RSUM = RSUM + 0.5
        ENDIF
      END DO
      END DO
      CRIS1D_CLOUDFLAG(10) = NINT(RSUM*100./ NP)


      ! Level 1C variables
      ! Land Fraction (sea = TAB2==0)
      IW   = 0
      DO I = 1, SIZE( MAPPING%VP, 1 )
      DO J = 1, SIZE( MAPPING%VP, 2 )
        IF( TAB2(I,J) .EQ. 0) THEN
          IW = IW + 1
        ENDIF
      END DO
      END DO
      CRIS1C_LANDFRAC = 100 - IW*100/NP

      IF(CRIS1C_LANDFRAC .GT. 95) THEN
        CRIS1C_LANDSEA = 0 ! Land
      ELSEIF(CRIS1C_LANDFRAC .LT.  5) THEN
        CRIS1C_LANDSEA = 1 ! Sea
      ELSE
        CRIS1C_LANDSEA = 2 ! Coast
      ENDIF

      CRIS1C_CLOUDCOVER = CRIS1D_CLOUDFLAG(2)
      
      
      IF( OPTIONS%DEBUG .GE. 2 ) THEN
        write(*,*) "cris1d_cloudflag         ",CRIS1D_CLOUDFLAG(1:5)
        write(*,*) "cris1d_cloudflag TCP     ",CRIS1D_CLOUDFLAG(6)/10.
        write(*,*) "cris1d_cloudflag TCT     ",CRIS1D_CLOUDFLAG(7)/100.
        write(*,*) "cris1d_cloudflag TSea    ",CRIS1D_CLOUDFLAG(8)/100.
        write(*,*) "cris1d_cloudflag TLand   ",CRIS1D_CLOUDFLAG(9)/100.
        write(*,*) "cris1d_cloudflag FovQual ",CRIS1D_CLOUDFLAG(10)
        write(*,*) "cris1c_landfrac          ",CRIS1C_LANDFRAC
        write(*,*) "cris1c_landsea           ",CRIS1C_LANDSEA
        write(*,*) "cris1c_cloudcover        ",CRIS1C_CLOUDCOVER
        write(*,*) "cris1c_cloudtopht        ",CRIS1C_CLOUDTOPHT
      ENDIF
    ENDIF

    DEALLOCATE(TAB, TAB1, TAB2, TAB3, STAT=ERR )
    THROWM(ERR.NE.0,"DeAllocate error")

CATCH
    END SUBROUTINE

    !> Do convolution for clusters .
    !> Prepare storage for CrIS 1D format and 1C format, using classical arrays.
    !> Convolution is done taking into account for the CrIS ellipse but 
    !!   no weights applied. We want the true cloud information inside the FOV
    !!   so ONLY_FOV option is true
    SUBROUTINE MAP_CONV_CLUSTER(OPTIONS, MAPPING, SDR, CLUSTER, &
                                &  ERR)
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MOD
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFORVIIRS),    INTENT(IN)  :: MAPPING ! Mapping structure for FOR
    TYPE(VIIRS_SDR )  ,    INTENT(IN)  ::SDR
    TYPE(clusterviirs),  INTENT(INOUT)  :: CLUSTER
    INTEGER,            INTENT(OUT) :: ERR
   
    INTEGER(KIND=4) :: ifov, class, chan, p, targetp,targetl,NB_PIX
    INTEGER(KIND=4) :: relativp, relativl, offset_chan
    REAL:: radiance
    INTEGER,POINTER,DIMENSION(:,:)::QF1_VIIRS ! Band Quality Flag for each pixel [pixels, lines]
    IF (OPTIONS%VIIRS_BAND .eq. 'M') THEN
          offset_chan = 0
    ELSE 
        IF (OPTIONS%VIIRS_BAND .eq. 'I') THEN
          offset_chan = 16
        ELSE
          offset_chan = 21
        ENDIF
    ENDIF
    err = 0
    cluster%RadAnaMean = 0.
    cluster%RadAnaStd = 0.
    cluster%RadAnaNbPix = 0
    DO ifov=1,9    
      NB_PIX = MAPPING%fovs(IFOV)%nb_pixels
      DO P=1,NB_PIX
            targetp = MAPPING%fovs(IFOV)%VP(P)
            targetl = MAPPING%fovs(IFOV)%VL(P)
            relativp = MAPPING%fovs(IFOV)%VP(P) - MAPPING%fpv + 1
            relativl = MAPPING%fovs(IFOV)%VL(P) - MAPPING%flv + 1
         DO class =1,CL_NBCLASS_TOTAL
           IF (cluster%cluster_on_viirs_grid( relativp,relativl) .eq. class ) THEN ! f class in fov
               cluster%RadAnaNbPix(class,ifov) = cluster%RadAnaNbPix(class,ifov) + 1
              DO chan=1,SIZE(SDR%BAND)
                 IF (ASSOCIATED(SDR%BAND(chan)%RADIANCE)) THEN
                   IF( SDR%BAND(chan)%CHANNEL_NAME(1:1) .EQ. "M" ) THEN
                          QF1_VIIRS => SDR%BAND(chan)%QF1_VIIRSMBANDSDR
                   ELSEIF( SDR%BAND(chan)%CHANNEL_NAME(1:1) .EQ. "I" ) THEN
                          QF1_VIIRS => SDR%BAND(chan)%QF1_VIIRSIBANDSDR
                   ELSE
                          QF1_VIIRS => SDR%BAND(chan)%QF1_VIIRSDNBBANDSDR
                   ENDIF
                   IF (SDR%BAND(chan)%RADIANCE(targetp,targetl) .GT. 0 &
                       .AND. QF1_VIIRS(targetp,targetl) .EQ. 0) THEN
                     radiance = VIIRSSRF(chan+offset_chan)%RadtoSIfactor * SDR%BAND(chan)%RADIANCE(targetp,targetl)
                     cluster%RadAnaMean(chan,class,ifov) = cluster%RadAnaMean(chan,class,ifov) + &
                                                            radiance
                     cluster%RadAnaStd(chan,class,ifov) = cluster%RadAnaStd(chan,class,ifov) + &
                                                          radiance*radiance
                   END IF ! rad > 0
                   NULLIFY(QF1_VIIRS )
                 END IF ! associated
              END DO !  chan
           ENDIF ! match class in fov
         END DO ! class             
     END DO ! P
        
    ! finish to compute mean and std
     DO class =1,CL_NBCLASS_TOTAL  
       IF  (cluster%RadAnaNbPix(class,ifov) .ge. 1 ) THEN
         DO chan=1,SIZE(SDR%BAND)
             IF (ASSOCIATED(SDR%BAND(chan)%RADIANCE)) THEN
                 cluster%RadAnaMean(chan,class,ifov)  = cluster%RadAnaMean(chan,class,ifov)/ &
                                                        cluster%RadAnaNbPix(class,ifov) 
             END IF ! ASSOCIATED
          END DO
       ENDIF ! cluster%RadAnaNbPix(class,ifov) .ge. 1
       IF  (cluster%RadAnaNbPix(class,ifov) .ge. 2 ) THEN
         DO chan=1,SIZE(SDR%BAND)
            IF (ASSOCIATED(SDR%BAND(chan)%RADIANCE)) THEN
               cluster%RadAnaStd(chan,class,ifov)  = (cluster%RadAnaStd(chan,class,ifov) - &
                                   cluster%RadAnaMean(chan,class,ifov)*cluster%RadAnaMean(chan,class,ifov)*&
                                                 cluster%RadAnaNbPix(class,ifov)) / &
                                                     (cluster%RadAnaNbPix(class,ifov)-1)
               cluster%RadAnaStd(chan,class,ifov)  = SQRT(cluster%RadAnaStd(chan,class,ifov))
            END IF ! ASSOCIATED
         END DO
       ENDIF ! cluster%RadAnaNbPix(class,ifov) .ge. 1
     END DO! class
   END DO !ifov

    IF( OPTIONS%DEBUG .GE. 2 ) THEN
       write(*,*) " RadAnaNbPix", cluster%RadAnaNbPix
    ENDIF

    END SUBROUTINE
    !> Great circle distance between a set of points and a center, in km.
    !> Inputs:
    !>- LatC/LonC center coordinates latitude, longitude (degrees)
    !>- Lat/Lon 2 dimensions arrays of points, latitude, longitude (degrees).
    !!
    !> Output:
    !>- Dist 2 dimensions arrays of distances (km).
    !!
    !> Only used in case of debug, called by MAP_SAVE_DEBUG subroutine
    !> Exact formula
    SUBROUTINE MAP_DISTANCE_FROM_CENTER2(LATC, LONC, LAT, LON, DIST, ERR)
    IMPLICIT NONE
    REAL(KIND=8)  ::  deg2rad                 ! convert degrees to radians
    PARAMETER (deg2rad=3.141592654d0/180.d0)

    REAL,    INTENT(IN)  :: LATC
    REAL,    INTENT(IN)  :: LONC
    REAL,    INTENT(IN)  :: LAT(:,:)
    REAL,    INTENT(IN)  :: LON(:,:)
    REAL,    INTENT(OUT) :: DIST(:,:)
    INTEGER, INTENT(OUT) :: ERR

    REAL(KIND=8), ALLOCATABLE :: COSANG(:,:)
TRY
    ERR = 0
    ALLOCATE(COSANG(SIZE(LAT,1), SIZE(LAT,2)), STAT=ERR )
    THROWM(ERR.NE.0,"Allocate error")

    COSANG = SIN(LATC* DEG2RAD) * SIN(LAT* DEG2RAD) + &
       & COS(LATC* DEG2RAD)*COS(LAT* DEG2RAD)*COS(LONC* DEG2RAD - LON* DEG2RAD)

    WHERE (COSANG .GT. 1.0)
      COSANG = 1.0
    END WHERE

    DIST =   ACOS(COSANG) * 6371.0130D3

    DEALLOCATE(COSANG, STAT=ERR )
    THROWM(ERR.NE.0,"DeAllocate error")

CATCH
    END SUBROUTINE

    !> Great circle distance between 2 points in km.
    !> Inputs:
    !>- LatC/LonC first point coordinates latitude, longitude (degrees).
    !>- Lat/Lon , second point coordinates latitude, longitude (degrees).
    !!
    !> Output:
    !>- Dist  distance (km)
    !!
    !> Used by routine MAP_CRIS_FOV_AXISPIX for axis length
    !> Exact formula
    SUBROUTINE MAP_DISTANCE_FROM_CENTER(LATC, LONC, LAT, LON, DIST)
    IMPLICIT NONE
    REAL(KIND=8)  ::  deg2rad                 ! convert degrees to radians
    PARAMETER (deg2rad=3.141592654d0/180.d0)

    REAL, INTENT(IN)  :: LATC
    REAL, INTENT(IN)  :: LONC
    REAL, INTENT(IN)  :: LAT
    REAL, INTENT(IN)  :: LON
    REAL, INTENT(OUT) :: DIST

    REAL(KIND=8) :: COSANG

    COSANG = SIN(LATC* DEG2RAD) * SIN(LAT* DEG2RAD) + &
       & COS(LATC* DEG2RAD)*COS(LAT* DEG2RAD)*COS(LONC* DEG2RAD - LON* DEG2RAD)

    IF (COSANG .GT. 1.0) THEN
      COSANG = 1.0
    ENDIF

    DIST =   ACOS(COSANG) * 6371.0130D3

    END SUBROUTINE

    !> Finds the nearest VIIRS location of an input CrIS lat/lon.
    !> If CrIS FOV is out of the VIIRS domain ERR is 97 or 99.
    SUBROUTINE  MAP_NEAREST_VIIRS(OPTIONS, MAPPING, VIIRSLOC, ERR, PREVPOS)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFOVVIIRS),      INTENT(INOUT) :: MAPPING
    TYPE(VIIRS_LOCATION), INTENT(IN)    :: VIIRSLOC
    INTEGER,              INTENT(OUT)   :: ERR
    INTEGER, OPTIONAL,    INTENT(IN)    :: PREVPOS(2)
    
    REAL(KIND=8)  :: IJV, IJC, PRECIS
    INTEGER       :: IQ(3)
    INTEGER       :: I, ISCAN, NSCANS, NLINES, ILAT, ILON
    LOGICAL       :: INIT
TRY
    ERR = 0

    !>- Check target is mainly in VIIRS image
    ILAT = NINT(MAPPING%TARGET_LATITUDE)
    ILON = NINT(MAPPING%TARGET_LONGITUDE)
    IF( VIIRSLOC%gridp(ilat,ilon) .LT. 0) THEN
        ERR = 97
      RETURN
    ENDIF


    NSCANS  = VIIRSLOC%TOTALSCANS
    NLINES  = SIZE(VIIRSLOC%LATITUDE,2)

    MAPPING%LPS = VIIRSLOC%ROWSPERSCAN

    !>- Use PREVPOS (previous position) if available and not 0
    INIT = .TRUE.
    IF( PRESENT(PREVPOS) ) THEN
      IF( PREVPOS(1) .GT. 0 ) THEN
        INIT=.FALSE.
        MAPPING%vpc = PREVPOS(1)
        MAPPING%vlc = PREVPOS(2)
      ENDIF
    ENDIF

    !>- If need initialisation
    IF( INIT ) THEN
      !> - First guess of VIIRS line calculated using datation
      IQ(1) = MAPPING%SCNLINYR
      IQ(2) = MAPPING%SCNLINDY
      IQ(3) = MAPPING%SCNLINTIME
    
      CALL XIQJ( IQ, IJC )
      PRECIS = 1.D0/86400.D0  ! One second precision
      ISCAN = 0
      DO I = 1, SIZE(VIIRSLOC%UTCJULIANDAY)
        IJV = VIIRSLOC%UTCJULIANDAY(I) + VIIRSLOC%UTC(I)/86400.D3
        IF( ABS(IJV-IJC) .LE. PRECIS ) THEN
          ISCAN = I
          EXIT
        ENDIF
      END DO
      
      IF( ISCAN .EQ. 0 ) THEN
        !> - If scan outside VIRRS data return error code 99
        ERR = 99
        RETURN
      ENDIF

      !> - Transform scan value in line
      IF( VIIRSLOC%COLUMNSPERSCAN .EQ. 3200 ) THEN
        MAPPING%VPC = 1600
      ELSE
        ! initial pixel position is given in M coordinates
        MAPPING%VPC = 3200
      ENDIF
      MAPPING%VLC = NINT((ISCAN-1+0.5)*(NLINES/NSCANS)) 

      !> - Estimate pixel from CrIS axis regression tables
      IF( VIIRSLOC%COLUMNSPERSCAN .EQ. 3200 ) THEN
        MAPPING%VPC = CRISAXES%VIIRSM(MAPPING%FOR,MAPPING%FOV)
      ELSE
        ! initial pixel position is given in M coordinates
        MAPPING%VPC = (CRISAXES%VIIRSM(MAPPING%FOR,MAPPING%FOV) - 1600) * 2 + 3200
      ENDIF
      MAPPING%VLC = MAPPING%VLC + CRISAXES%VIIRSL(MAPPING%FOR,MAPPING%FOV)
    ENDIF
    
    MAPPING%VNL = NLINES
    MAPPING%vnp = VIIRSLOC%COLUMNSPERSCAN
    IF( MAPPING%VLC .LE. 0 .OR. MAPPING%VLC.GT.MAPPING%VNL ) THEN
      ! DO NOT NEED TO GO FURTHER, FOV OUTSIDE VIIRS DATA
      !>- If line outside VIRRS data return error code 99
      ERR = 99
      RETURN
    ENDIF

    IF( ABS(MAPPING%TARGET_LATITUDE) < 89.0 ) THEN
      !>- Search nearest VIIRS pixel with method 1
      CALL MAP_NEAREST_VIIRS_MET1(OPTIONS, MAPPING, VIIRSLOC, ERR)
      IF( ERR.EQ.99 ) THEN
        !> - If NOT FOUND or outside VIIRS scans
        !! Try second method
        CALL MAP_NEAREST_VIIRS_MET2(OPTIONS, MAPPING, VIIRSLOC, ERR)
      ENDIF
    ELSE
      !>- Polar FOV, use method 2
      CALL MAP_NEAREST_VIIRS_MET2(OPTIONS, MAPPING, VIIRSLOC, ERR)
    ENDIF
    THROWM(ERR.NE.0 .AND. ERR.NE.99,"ERROR MAP_NEAREST_VIIRS")
    
    !>- ERR 99 is not fatal, just FOV outside VIIRS data
    IF( ERR.NE.99 ) THEN
      ! FOV found in VIIRS data
      MAPPING%LATITUDE  = VIIRSLOC%LATITUDE(MAPPING%VPC,MAPPING%VLC)
      MAPPING%LONGITUDE = VIIRSLOC%LONGITUDE(MAPPING%VPC,MAPPING%VLC)
    ENDIF
    
    !>- SECURITY CHECK on lines.
    !! Do not consider border FOVs
    IF( MAPPING%VLC .LE. 3 .OR. &
      & MAPPING%VLC .GE. MAPPING%VNL-3 ) THEN
      !write(*,*) "WARNING BORDER FOV "
      ERR=99
    ENDIF

    IF( ERR.NE.99 ) THEN
      MAPPING%LATITUDE  = VIIRSLOC%LATITUDE(MAPPING%VPC,MAPPING%VLC)
      MAPPING%LONGITUDE = VIIRSLOC%LONGITUDE(MAPPING%VPC,MAPPING%VLC)
    ENDIF

CATCH
    END SUBROUTINE

    !> Finds the nearest VIIRS location of an input CrIS lat/lon.
    !> Method 1:
    !! use iterative process with vector products
    SUBROUTINE  MAP_NEAREST_VIIRS_MET1(OPTIONS, MAPPING, VIIRSLOC, ERR)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFOVVIIRS),      INTENT(INOUT) :: MAPPING
    TYPE(VIIRS_LOCATION), INTENT(IN)    :: VIIRSLOC
    INTEGER,              INTENT(OUT)   :: ERR
    
    INTEGER, PARAMETER :: MAXITER = 20
    
    INTEGER      :: I, IP, IL, LINS
    INTEGER      :: IPM, ILM
    REAL         :: BOXLAT(2,2), BOXLON(2,2)
    LOGICAL      :: BOXMSK(2,2)
    REAL         :: MINLON, MAXLON
    REAL         :: DP, DL
    LOGICAL      :: MASKED
    LOGICAL      :: FOUND
    LOGICAL      :: INSIDE
    INTEGER      :: ITER
    INTEGER      :: VPCM(MAXITER), VLCM(MAXITER)
    REAL         :: XS,YS,ZS
    REAL         :: XF(2,2),YF(2,2),ZF(2,2)
    REAL         :: DIST, DMIN

    ERR = 0
    ITER = 0
    VPCM = 0
    VLCM = 0
    FOUND = .FALSE.

    IF( OPTIONS%DEBUG .GE. 2 ) THEN
      WRITE(*,*) "SEARCH ",MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE
    ENDIF

    !>- Loop until found or max iterations
SEARCH:    DO WHILE (.NOT. FOUND .AND. ITER.LT.MAXITER)
      ITER = ITER+1
      
      VPCM(ITER) = MAPPING%VPC
      VLCM(ITER) = MAPPING%VLC
      IF( OPTIONS%DEBUG .GE. 2 ) THEN
        write(*,*)
        write(*,*) "START ITER ",iter," with VPC VLC ",MAPPING%VPC,MAPPING%VLC
      ENDIF

      !>- If new position already found in previous iterations
      !! THEN it is an infinite loop, so return to calling routine with found=False
      DO I = ITER-1, MAX(ITER-4,1), -1
        IF( MAPPING%vpc .EQ. vpcm(I) .AND. MAPPING%vlc .EQ. vlcm(I)) THEN
          ! infinite loop
          FOUND = .FALSE.
          ITER = MAXITER+1
          CYCLE SEARCH
        ENDIF
      ENDDO

      ! Check pixel inside VIIRS array
      ! Take 2 pixels margin for +-1 tests
      IF( MAPPING%vpc .LE. 0 ) MAPPING%vpc = 2
      IF( MAPPING%vpc .GE. MAPPING%vnp ) MAPPING%vpc = MAPPING%vnp -1
      IF( MAPPING%vlc .LE. 0 ) MAPPING%vlc = 2
      IF( MAPPING%vlc .GE. MAPPING%vnl ) MAPPING%vlc = MAPPING%vnl -1

      IP = MAPPING%vpc
      IL = MAPPING%vlc

      BOXLAT = VIIRSLOC%LATITUDE  (IP:IP+1,IL:IL+1)
      BOXLON = VIIRSLOC%LONGITUDE (IP:IP+1,IL:IL+1)
      BOXMSK = VIIRSLOC%MASKBOWTIE(IP:IP+1,IL:IL+1)

      MASKED = ANY(BOXMSK .EQV. .TRUE.)

      !>- If any box pixel is masked (bow tie) try to move
      !! out the masked area by a number of lines = lines-per-scan/2
      IF(MASKED)  THEN
        ! at least one box corner is masked
        LINS = MOD( IL, MAPPING%LPS ) +1
        IF( MAPPING%LLM .GE. 0 ) THEN
          MAPPING%VLC = MAPPING%VLC + MAPPING%LPS/2
        ELSE
          MAPPING%VLC = MAPPING%VLC - MAPPING%LPS/2
        ENDIF     
        CYCLE SEARCH
      ENDIF

      !>- Normalise LONGITUDES to [0,360] or [-180,180]
      MINLON = MINVAL(BOXLON)
      MAXLON = MAXVAL(BOXLON)
      IF( (MAXLON * MINLON .LT. 0.) .AND. MAXLON .GT. 90. ) THEN
        ! normalise to [0,360]
        WHERE(BOXLON < 0.) 
          BOXLON = BOXLON + 360.0
        END WHERE
      ELSEIF( MAXLON - MINLON .GT. 270. ) THEN
        ! normalise to [-180,180]
        WHERE(BOXLON > 180.) 
          BOXLON = BOXLON - 360.0
        END WHERE
      END IF
      
      !>- Test target inside box
      CALL MAP_INSIDE_BOX(MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE,&
          &  BOXLAT, BOXLON, INSIDE)
      
      IF( INSIDE ) THEN
        FOUND = .TRUE.
      ELSE
        !>- if not found THEN MOVE to another box
        MAPPING%lpm = 0
        MAPPING%llm = 0
        CALL MAP_NEW_BOX(MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE, &
             & BOXLAT, BOXLON, DP, DL)
        IF( DP .LT. -9990. ) THEN
          !>- If new_box fails force move of one
          MAPPING%LPM = 1
          MAPPING%LLM = 1
          MAPPING%VPC = MAPPING%VPC + MAPPING%LPM
          MAPPING%VLC = MAPPING%VLC + MAPPING%LLM

        ELSEIF( ABS(DP) .LT. 1. .AND. ABS(DL) .LT. 1. ) THEN
        !>- If displacement is less than 1, force move of one
          IF( DP > 0.5 ) THEN
            MAPPING%LPM = 1
          ENDIF
          IF( DL > 0.5 ) THEN
            MAPPING%LLM = 1
          ENDIF
          IF( DP < -0.5 ) THEN
            MAPPING%LPM = -1
          ENDIF
          IF( DL < -0.5 ) THEN
            MAPPING%LLM = -1
          ENDIF
          MAPPING%VPC = MAPPING%VPC + MAPPING%LPM
          MAPPING%VLC = MAPPING%VLC + MAPPING%LLM
          ! FOV found if not masked
          FOUND = .NOT. VIIRSLOC%MASKBOWTIE(MAPPING%VPC,MAPPING%VLC)

        ELSE
          MAPPING%LPM = NINT(DP)
          MAPPING%LLM = NINT(DL)
          MAPPING%VPC = MAPPING%VPC + MAPPING%LPM
          MAPPING%VLC = MAPPING%VLC + MAPPING%LLM
        ENDIF
      ENDIF ! inside
            
    END DO SEARCH

    IF( .NOT. FOUND ) ERR = 99

    IF(FOUND ) THEN
       !>- Pixel is found
       !>- Check if any of the 3 other pixels in box is closer to the target
       CALL LATLON_TO_XYZ0D(MAPPING%TARGET_LATITUDE,  &
                          & MAPPING%TARGET_LONGITUDE, &
                          & XS, YS, ZS)
       CALL LATLON_TO_XYZ(2, 2, BOXLAT, BOXLON, XF, YF, ZF)
       DMIN=999.
       IP = 1
       IL = 1
       DMIN=SQRT((XS-XF(IP,IL))**2+(YS-YF(IP,IL))**2+(ZS-ZF(IP,IL))**2) *6370.

       IPM = 0
       ILM = 0
       DO IP =1,2
       DO IL =1,2
         DIST=SQRT((XS-XF(IP,IL))**2+(YS-YF(IP,IL))**2+(ZS-ZF(IP,IL))**2) *6370.
         IF(DIST.LT.DMIN) THEN
           IPM = IP
           ILM = IL
           DMIN = DIST
         ENDIF
       ENDDO
       ENDDO
       ! Take closer pixel only if not part of bow-tie
       IF( IPM .GT. 0 ) THEN
        IF (MAPPING%vpc +IPM-1 .GT. 0 .AND. &
            MAPPING%vpc +IPM-1 .LE. MAPPING%vnp .AND. &
            MAPPING%vlc +ILM-1 .GT. 0 .AND.  &
            MAPPING%vlc +ILM-1 .LE.MAPPING%vnl ) THEN
            IF( .NOT. &
            & VIIRSLOC%MASKBOWTIE(MAPPING%vpc +IPM-1,MAPPING%vlc +ILM-1)) THEN
              MAPPING%vpc = MAPPING%vpc +IPM-1
              MAPPING%vlc = MAPPING%vlc +ILM-1
              IF( OPTIONS%DEBUG .GE. 2) THEN
                 WRITE(*,*) "Take closer pixel for ip, il ",MAPPING%vpc,MAPPING%vlc
              ENDIF
            ENDIF
       ENDIF
       ENDIF
       IF( OPTIONS%DEBUG .GE. 2) THEN
         WRITE(*,*) "Distance to target ",DMIN,"km"
       ENDIF
     ENDIF

    IF(FOUND .AND. OPTIONS%DEBUG .GE. 2) THEN
       WRITE(*,*) "FOUND at p,l  ", MAPPING%vpc,MAPPING%vlc
       WRITE(*,*) "Iteration     ", ITER
       WRITE(*,*) "SEARCH val    ", MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE
       WRITE(*,*) "Found lat lon ", VIIRSLOC%LATITUDE(MAPPING%vpc,MAPPING%vlc), VIIRSLOC%LONGITUDE(MAPPING%vpc,MAPPING%vlc)
    ENDIF

    END SUBROUTINE

    !> Finds the nearest VIIRS location of an input CrIS lat/lon.
    !> Method 1:
    !!  use iterative process with vector products.
    !!
    !> Test version which allows a variable box size.
    SUBROUTINE  MAP_NEAREST_VIIRS_MET1_varbox(OPTIONS, MAPPING, VIIRSLOC, ERR)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFOVVIIRS),      INTENT(INOUT) :: MAPPING
    TYPE(VIIRS_LOCATION), INTENT(IN)    :: VIIRSLOC
    INTEGER,              INTENT(OUT)   :: ERR
    
    INTEGER, PARAMETER :: MAXITER = 20
    
    INTEGER      :: I, IP, IL, LINS
    REAL         :: BOXLAT(2,2), BOXLON(2,2)
    LOGICAL      :: BOXMSK(2,2)
    REAL         :: MINLON, MAXLON
    REAL         :: DP, DL
    LOGICAL      :: MASKED
    LOGICAL      :: FOUND
    LOGICAL      :: INSIDE
    INTEGER      :: ITER
    INTEGER      :: VPCM(MAXITER), VLCM(MAXITER)
    REAL         :: POSS(3),POSF(3),DIST
    INTEGER      :: BOXSIZE, N0, N1


    ERR = 0
    ITER = 0
    VPCM = 0
    VLCM = 0
    FOUND = .FALSE.

    IF( OPTIONS%DEBUG .GE. 2 ) THEN
      WRITE(*,*) "SEARCH ",MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE
    ENDIF

SEARCH:    DO WHILE (.NOT. FOUND .AND. ITER.LT.MAXITER)
      ITER = ITER+1
      
      VPCM(ITER) = MAPPING%VPC
      VLCM(ITER) = MAPPING%VLC
      IF( OPTIONS%DEBUG .GE. 2 ) THEN
        write(*,*)
        write(*,*) "START ITER ",iter," with VPC VLC ",MAPPING%VPC,MAPPING%VLC
      ENDIF

      DO I = ITER-1, MAX(ITER-4,1), -1
        IF( MAPPING%vpc .EQ. vpcm(I) .AND. MAPPING%vlc .EQ. vlcm(I)) THEN
          ! infinite loop
          FOUND = .FALSE.
          ITER = MAXITER+1
          CYCLE SEARCH
        ENDIF
      ENDDO

      BOXSIZE = 2
      IF ( (BOXSIZE/2)*2 .EQ. BOXSIZE ) THEN
        N0 =  BOXSIZE/2 -1
        N1 =  BOXSIZE/2
      ELSE
        N0 =  BOXSIZE/2
        N1 =  BOXSIZE/2
      ENDIF
      IF( MAPPING%vpc .LE. BOXSIZE/2 ) MAPPING%vpc = BOXSIZE/2 +1
      IF( MAPPING%vpc .GE. MAPPING%vnp - BOXSIZE/2) MAPPING%vpc = MAPPING%vnp -BOXSIZE/2 -1
      IF( MAPPING%vlc .LE. BOXSIZE/2 ) MAPPING%vlc = BOXSIZE/2 +1
      IF( MAPPING%vlc .GE. MAPPING%vnl - BOXSIZE/2) MAPPING%vlc = MAPPING%vnl -BOXSIZE/2 -1
      IP = MAPPING%vpc
      IL = MAPPING%vlc

      BOXLAT(1,1) = VIIRSLOC%LATITUDE  (IP-N0,IL-N0)
      BOXLON(1,1) = VIIRSLOC%LONGITUDE (IP-N0,IL-N0)
      BOXMSK(1,1) = VIIRSLOC%MASKBOWTIE(IP-N0,IL-N0)
      
      BOXLAT(2,1) = VIIRSLOC%LATITUDE  (IP+N1,IL-N0)
      BOXLON(2,1) = VIIRSLOC%LONGITUDE (IP+N1,IL-N0)
      BOXMSK(2,1) = VIIRSLOC%MASKBOWTIE(IP+N1,IL-N0)
      
      BOXLAT(1,2) = VIIRSLOC%LATITUDE  (IP-N0,IL+N1)
      BOXLON(1,2) = VIIRSLOC%LONGITUDE (IP-N0,IL+N1)
      BOXMSK(1,2) = VIIRSLOC%MASKBOWTIE(IP-N0,IL+N1)
      
      BOXLAT(2,2) = VIIRSLOC%LATITUDE  (IP+N1,IL+N1)
      BOXLON(2,2) = VIIRSLOC%LONGITUDE (IP+N1,IL+N1)
      BOXMSK(2,2) = VIIRSLOC%MASKBOWTIE(IP+N1,IL+N1)

      MASKED = ANY(BOXMSK .EQV. .TRUE.)
        !write(*,*) "masked ",BOXMSK

      IF(MASKED)  THEN
        ! at least one box corner is masked
        ! move outside maked area
        LINS = MOD( IL, MAPPING%LPS ) +1
        IF( MAPPING%LLM .GE. 0 ) THEN
          MAPPING%VLC = MAPPING%VLC + MAPPING%LPS/2
        ELSE
          MAPPING%VLC = MAPPING%VLC - MAPPING%LPS/2
        ENDIF     
        CYCLE SEARCH
      ENDIF

      ! Normalise LONGITUDES
      MINLON = MINVAL(BOXLON)
      MAXLON = MAXVAL(BOXLON)
      IF( (MAXLON * MINLON .LT. 0.) .AND. MAXLON .GT. 90. ) THEN
        ! normalise to [0,360]
        WHERE(BOXLON < 0.) 
          BOXLON = BOXLON + 360.0
        END WHERE
      ELSEIF( MAXLON - MINLON .GT. 270. ) THEN
        ! normalise to [-180,180]
        WHERE(BOXLON > 180.) 
          BOXLON = BOXLON - 360.0
        END WHERE
      END IF
      
      ! Test target inside box
      CALL MAP_INSIDE_BOX(MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE,&
          &  BOXLAT, BOXLON, INSIDE)
      
      IF( INSIDE ) THEN
        FOUND = .TRUE.
      ELSE
        ! MOVE to another box
        MAPPING%lpm = 0
        MAPPING%llm = 0
        CALL MAP_NEW_BOX(MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE, &
             & BOXLAT, BOXLON, DP, DL)
        IF( DP .LT. -9990. ) THEN
          ! If new_box fails force move of one
          MAPPING%LPM = 1
          MAPPING%LLM = 1
          MAPPING%VPC = MAPPING%VPC + MAPPING%LPM
          MAPPING%VLC = MAPPING%VLC + MAPPING%LLM

        ELSEIF( ABS(DP) .LT. 1. .AND. ABS(DL) .LT. 1. ) THEN
        ! if displacement is less than 1, force move of one
          IF( DP > 0.5 ) THEN
            MAPPING%LPM = 1
          ENDIF
          IF( DL > 0.5 ) THEN
            MAPPING%LLM = 1
          ENDIF
          IF( DP < -0.5 ) THEN
            MAPPING%LPM = -1
          ENDIF
          IF( DL < -0.5 ) THEN
            MAPPING%LLM = -1
          ENDIF
          MAPPING%VPC = MAPPING%VPC + MAPPING%LPM
          MAPPING%VLC = MAPPING%VLC + MAPPING%LLM
          ! FOV found if not masked
          FOUND = .NOT. VIIRSLOC%MASKBOWTIE(MAPPING%VPC,MAPPING%VLC)
        ELSE
          MAPPING%LPM = NINT(DP)
          MAPPING%LLM = NINT(DL)
          MAPPING%VPC = MAPPING%VPC + MAPPING%LPM * BOXSIZE/2
          MAPPING%VLC = MAPPING%VLC + MAPPING%LLM * BOXSIZE/2
        ENDIF
      ENDIF ! inside
            
    END DO SEARCH

    IF( .NOT. FOUND ) ERR = 99
    IF(FOUND .AND. OPTIONS%DEBUG .GE. 2) THEN
       WRITE(*,*) "FOUND at p,l  ", MAPPING%vpc,MAPPING%vlc
       WRITE(*,*) "Iteration     ", ITER
       WRITE(*,*) "SEARCH val    ", MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE
       WRITE(*,*) "Found lat lon ", VIIRSLOC%LATITUDE(MAPPING%vpc,MAPPING%vlc), &
                                  & VIIRSLOC%LONGITUDE(MAPPING%vpc,MAPPING%vlc)                     
    
       CALL LATLON_TO_XYZ0D(MAPPING%TARGET_LATITUDE,  &
                          & MAPPING%TARGET_LONGITUDE, &
                          & POSS(1), POSS(2), POSS(3))
                          
       CALL LATLON_TO_XYZ0D(VIIRSLOC%LATITUDE(MAPPING%VPC,MAPPING%VLC), &
                          & VIIRSLOC%LONGITUDE(MAPPING%VPC,MAPPING%VLC),&
                          & POSF(1), POSF(2), POSF(3))
       
       DIST=SQRT((POSS(1)-POSF(1))**2+(POSS(2)-POSF(2))**2+(POSS(3)-POSF(3))**2) *6370.
       WRITE(*,*) "distance km ",DIST
     ENDIF

    END SUBROUTINE

    !> Check if lat/lon is inside a box defined by the corners in lat/lon
    SUBROUTINE  MAP_INSIDE_BOX(LAT, LON, BOXLAT, BOXLON, INSIDE)
    IMPLICIT NONE
    REAL,    INTENT(IN)  :: LAT
    REAL,    INTENT(IN)  :: LON
    REAL,    INTENT(IN)  :: BOXLAT(2,2)
    REAL,    INTENT(IN)  :: BOXLON(2,2)
    LOGICAL, INTENT(OUT) :: INSIDE

    REAL :: PV1, PV2, PV3, PV4

     PV1 = PRODVECT(LAT,LON,           &
           &  BOXLAT(1,1),BOXLON(1,1), &
           &  BOXLAT(2,1),BOXLON(2,1)) 
                                      
     PV2 = PRODVECT(LAT,LON,           &
           &  BOXLAT(2,1),BOXLON(2,1), &
           &  BOXLAT(2,2),BOXLON(2,2)) 
                                      
     PV3 = PRODVECT(LAT,LON,           &
           &  BOXLAT(2,2),BOXLON(2,2), &
           &  BOXLAT(1,2),BOXLON(1,2)) 
                                      
     PV4 = PRODVECT(LAT,LON,           &
           &  BOXLAT(1,2),BOXLON(1,2), &
           &  BOXLAT(1,1),BOXLON(1,1)) 
  
     !> If the 4 vector products are positive the pixel is inside 
     !! the box
     INSIDE = MAX(PV1,PV2,PV3,PV4)*MIN(PV1,PV2,PV3,PV4) .GE. 0.

    RETURN
    END SUBROUTINE

    !> Estimate the position of a new box that could hold the 
    !! lat/lon values
    SUBROUTINE  MAP_NEW_BOX(LAT, LON, BOXLAT, BOXLON, DP, DL)
    IMPLICIT NONE
    REAL, INTENT(IN)  ::  LAT
    REAL, INTENT(IN)  ::  LON
    REAL, INTENT(IN)  ::  BOXLAT(2,2)
    REAL, INTENT(IN)  ::  BOXLON(2,2)
    REAL, INTENT(OUT) :: DP
    REAL, INTENT(OUT) :: DL

    REAL :: AX, AY, BX, BY, DX, DY, DET
    
    AX = BOXLAT(2,1) - BOXLAT(1,1)
    AY = BOXLON(2,1) - BOXLON(1,1)
    BX = BOXLAT(1,2) - BOXLAT(1,1)
    BY = BOXLON(1,2) - BOXLON(1,1)
    DX = LAT - BOXLAT(1,1)
    DY = LON - BOXLON(1,1)

    DET = AX*BY - AY*BX
    IF( ABS(DET) > 1.E-6 ) THEN
      DP = ( BY*DX - DY*BX ) / DET
      DL = ( AX*DY - DX*AY ) / DET
    ELSE
      DP = -9999.
      DL = -9999.
    ENDIF
    
    RETURN
    END SUBROUTINE

    !> Finds the nearest VIIRS location of an input CrIS lat/lon.
    !> Method 2: 
    !!  downscale boxes
    SUBROUTINE  MAP_NEAREST_VIIRS_MET2(OPTIONS, MAPPING, VIIRSLOC, ERR)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFOVVIIRS),      INTENT(INOUT) :: MAPPING
    TYPE(VIIRS_LOCATION), INTENT(IN)    :: VIIRSLOC
    INTEGER,              INTENT(OUT)   :: ERR
    
    INTEGER              :: IP, IL
    REAL, ALLOCATABLE    :: BOXLAT(:,:), BOXLON(:,:)
    LOGICAL, ALLOCATABLE :: BOXMSK(:,:)
    REAL, ALLOCATABLE    :: VX(:,:), VY(:,:), VZ(:,:)
    REAL, ALLOCATABLE    :: DIST(:,:)
    REAL                 :: CX(1), CY(1), CZ(1)

    REAL    :: DISTKM
    INTEGER :: MIND(2)
    INTEGER :: SAMPLING(4,3), S, NCALC
    INTEGER :: P, PSIZE, PHSIZE, SAMPLE
    INTEGER :: L, LSIZE, LHSIZE
    LOGICAL :: FOUND

TRY
    !>- 4 boxes sizes 40x200, 30x30, 18x18, 12x12
    !! which will contain latitude/longitude grids sampled by 20, 5, 3, 1 
    !
    ! box scales sampling, half size pixels, half size lines
    SAMPLING(1,1) = 20
    SAMPLING(1,2) = 20
    SAMPLING(1,3) = 100
    
    SAMPLING(2,1) = 5
    SAMPLING(2,2) = 15
    SAMPLING(2,3) = 15
    
    SAMPLING(3,1) = 3
    SAMPLING(3,2) = 9
    SAMPLING(3,3) = 9
    
    SAMPLING(4,1) = 1
    SAMPLING(4,2) = 6
    SAMPLING(4,3) = 6

    ERR = 0
    NCALC = 0

    IF( OPTIONS%DEBUG .GE. 2) THEN
      WRITE(*,*) "SEARCH 2 ",MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE
      WRITE(*,*) "   start with ",MAPPING%VPC, MAPPING%VLC
    ENDIF

    ! get Earth fixed coordinates for target
    CALL LATLON_TO_XYZ (1,1,(/MAPPING%TARGET_LATITUDE/), (/MAPPING%TARGET_LONGITUDE/),&
     &     CX,CY,CZ)

     !>- Loop on 4 box sizes
SAMP:    DO S = 1, SIZE(SAMPLING,1)
      ERR = 0
      SAMPLE=SAMPLING(S,1)
      PHSIZE=SAMPLING(S,2)
      LHSIZE=SAMPLING(S,3)
      PSIZE= 2*PHSIZE/SAMPLE +1
      LSIZE= 2*LHSIZE/SAMPLE +1
      
      NCALC = NCALC + PSIZE*LSIZE
      
      ALLOCATE(BOXLAT(-PHSIZE/SAMPLE:PHSIZE/SAMPLE,-LHSIZE/SAMPLE:LHSIZE/SAMPLE), &
             & BOXLON(-PHSIZE/SAMPLE:PHSIZE/SAMPLE,-LHSIZE/SAMPLE:LHSIZE/SAMPLE), &
             & BOXMSK(-PHSIZE/SAMPLE:PHSIZE/SAMPLE,-LHSIZE/SAMPLE:LHSIZE/SAMPLE), STAT=ERR)
      THROWM(ERR.NE.0,"CANNOT ALLOCATE BOXLAT BOXLON")

      ALLOCATE(VX(-PHSIZE/SAMPLE:PHSIZE/SAMPLE,-LHSIZE/SAMPLE:LHSIZE/SAMPLE), &
             & VY(-PHSIZE/SAMPLE:PHSIZE/SAMPLE,-LHSIZE/SAMPLE:LHSIZE/SAMPLE), &
             & VZ(-PHSIZE/SAMPLE:PHSIZE/SAMPLE,-LHSIZE/SAMPLE:LHSIZE/SAMPLE), &
             & STAT=ERR)
      THROWM(ERR.NE.0,"CANNOT ALLOCATE VX VY VZ")

      ALLOCATE(DIST(-PHSIZE/SAMPLE:PHSIZE/SAMPLE,-LHSIZE/SAMPLE:LHSIZE/SAMPLE), STAT=ERR)
      THROWM(ERR.NE.0,"CANNOT ALLOCATE DIST")

      IF( MAPPING%VPC-PHSIZE .LE. 0 ) MAPPING%VPC = 1+PHSIZE
      IF( MAPPING%VPC+PHSIZE .GE. MAPPING%VNP ) MAPPING%VPC = MAPPING%VNP -1 -PHSIZE
      IF( MAPPING%VLC-LHSIZE .LE. 0 ) MAPPING%VLC = 1+LHSIZE
      IF( MAPPING%VLC+LHSIZE .GE. MAPPING%VNL ) MAPPING%VLC = MAPPING%VNL -1 -LHSIZE

      IP = MAPPING%VPC
      IL = MAPPING%VLC

      !>- Fills with latitude/longitude and mask
      DO L = -LHSIZE/SAMPLE, LHSIZE/SAMPLE
        DO P = -PHSIZE/SAMPLE, PHSIZE/SAMPLE
          BOXLAT(P,L) = VIIRSLOC%LATITUDE(   IP + P*SAMPLE ,IL + L*SAMPLE)
          BOXLON(P,L) = VIIRSLOC%LONGITUDE(  IP + P*SAMPLE ,IL + L*SAMPLE)
          BOXMSK(P,L) = VIIRSLOC%MASKBOWTIE( IP + P*SAMPLE ,IL + L*SAMPLE)
        END DO
      END DO

      VX = 9999.
      VY = 9999.
      VZ = 9999. 

      !>- Convert all the lat/lons of the grid to x/y/z
      CALL LATLON_TO_XYZ (PSIZE,LSIZE,BOXLAT,BOXLON,VX,VY,VZ)
      
      !>- Compute distance for each node to the target
      DIST(:,:) = SQRT((VX(:,:)-CX(1))**2+(VY(:,:)-CY(1))**2+(VZ(:,:)-CZ(1))**2)
      WHERE (BOXMSK .EQV. .TRUE.)
        DIST=9999.
      END WHERE

      !>- Get the pixel for the minimum distance
      MIND = MINLOC(DIST)
      FOUND = .TRUE.

      ! distance in km
      DISTKM = DIST(-PHSIZE/SAMPLE + MIND(1) -1, &
                  & -LHSIZE/SAMPLE + MIND(2) -1) *6370.
                  
      DEALLOCATE(BOXLAT, BOXLON, BOXMSK)
      DEALLOCATE(VX,VY,VZ)
      DEALLOCATE(DIST)

      !>- IF Distance is less than 500m don't need to continue
      IF ( DISTKM .LE. 0.5 ) THEN
        FOUND = .TRUE.
        MAPPING%vpc = MAPPING%vpc + (MIND(1)-1)*SAMPLE+1 - PHSIZE -1
        MAPPING%vlc = MAPPING%vlc + (MIND(2)-1)*SAMPLE+1 - LHSIZE -1
        EXIT SAMP
      ENDIF
      
      ! MINLOC function returns arrays starting at 1
      IF(MIND(1) .EQ.   1   .OR. &
        &MIND(2) .EQ.   1   .OR. &
        &MIND(1) .EQ. PSIZE .OR. &
        &MIND(2) .EQ. LSIZE ) THEN
        !>- If pixel is on a boarder, return error code 99
        FOUND = .FALSE.
        ERR = 99
      END IF

      MAPPING%vpc = MAPPING%vpc + (MIND(1)-1)*SAMPLE+1 - PHSIZE -1
      MAPPING%vlc = MAPPING%vlc + (MIND(2)-1)*SAMPLE+1 - LHSIZE -1

    END DO SAMP

    IF(FOUND .AND. OPTIONS%DEBUG .GE. 2) THEN
       WRITE(*,*) "FOUND at p,l  ", MAPPING%VPC,MAPPING%VLC
       WRITE(*,*) "Sampling      ", S
       WRITE(*,*) "SEARCH val    ", MAPPING%TARGET_LATITUDE, MAPPING%TARGET_LONGITUDE
       WRITE(*,*) "Found lat lon ", VIIRSLOC%LATITUDE(MAPPING%VPC,MAPPING%VLC), VIIRSLOC%LONGITUDE(MAPPING%VPC,MAPPING%VLC)
    ENDIF

CATCH
    END SUBROUTINE

    !> Calculate the FOV Axis in km from regression table
    SUBROUTINE  MAP_CRIS_FOV_AXISKM(MAPPING)
    IMPLICIT NONE
    TYPE(CFOVVIIRS), INTENT(INOUT)   :: MAPPING
    
    REAL :: DH
    
    DH = MAPPING%TARGET_ALTITUDE - CRISAXES%REFALTITUDE
    MAPPING%ACROSSK(1) = DH*CRISAXES%ACSLOPE1(MAPPING%FOR,MAPPING%FOV) + &
                          & CRISAXES%ACINTER1(MAPPING%FOR,MAPPING%FOV)
    MAPPING%ACROSSK(2) = DH*CRISAXES%ACSLOPE2(MAPPING%FOR,MAPPING%FOV) + &
                          & CRISAXES%ACINTER2(MAPPING%FOR,MAPPING%FOV)
    MAPPING%ALONGK(1)  = DH*CRISAXES%ALSLOPE1(MAPPING%FOR,MAPPING%FOV) + &
                          & CRISAXES%ALINTER1(MAPPING%FOR,MAPPING%FOV)
    MAPPING%ALONGK(2)  = DH*CRISAXES%ALSLOPE2(MAPPING%FOR,MAPPING%FOV) + &
                           & CRISAXES%ALINTER2(MAPPING%FOR,MAPPING%FOV)

    END SUBROUTINE

    !> Calculate the 4 semi-axes of the CrIS FOV in VIIRS pixels/lines
    SUBROUTINE MAP_CRIS_FOV_AXISPIX(MAPPING, VIIRSLOC, ERR)
    IMPLICIT NONE
    TYPE(CFOVVIIRS), INTENT(INOUT)    :: MAPPING
    TYPE(VIIRS_LOCATION), INTENT(IN)  :: VIIRSLOC
    INTEGER,              INTENT(OUT) :: ERR

    REAL :: DREF ! semi-axis length in km
    REAL :: DPRE ! previous calculation
    REAL :: DIST ! distance from center box
    
    INTEGER :: PV, LV  ! Pixel/line in VIIRS
    INTEGER :: PM, LM  ! Pixel/line in mapping box
    
    LOGICAL :: FOUND
    
    ERR = 0
    
    !>- semi axis across 1
    LM   = MAPPING%RADIUSL +1
    DREF = MAPPING%ACROSSK(1)
    DPRE = 999.
    FOUND = .FALSE.
    !>- loop on pixels along the axis
    DO PM = MAPPING%RADIUSP +1, 1, -1
      PV = MAPPING%VP(PM, LM)
      LV = MAPPING%VL(PM, LM)
      IF( PV .GT. 0 .AND. LV .GT. 0 ) THEN
        !>- compute distance from center
        CALL MAP_DISTANCE_FROM_CENTER( &
           & MAPPING%LATITUDE, &
           & MAPPING%LONGITUDE, &
           & VIIRSLOC%LATITUDE(PV, LV), &
           & VIIRSLOC%LONGITUDE(PV, LV), &
           & DIST )
        DIST = DIST / 1000.0
      ENDIF
      IF( DIST .GT. DREF ) THEN
        !>- if distance is above the axis lenght, THEN stop loop
        FOUND = .TRUE.
        EXIT
      ENDIF
      DPRE = DIST
    END DO
    IF( .NOT. FOUND ) THEN
      ERR=98
      RETURN
    ENDIF
    MAPPING%ACROSSP(1) = MAPPING%RADIUSP +1 - &
                       & (PM + (DIST-DREF)/(DIST-DPRE))


    ! semi axis across 2
    LM   = MAPPING%RADIUSL +1
    DREF = MAPPING%ACROSSK(2)
    DPRE = 999.
    FOUND = .FALSE.
    DO PM = MAPPING%RADIUSP +1, 2*MAPPING%RADIUSP +1
      PV = MAPPING%VP(PM, LM)
      LV = MAPPING%VL(PM, LM)
      IF( PV .GT. 0 .AND. LV .GT. 0 ) THEN
        CALL MAP_DISTANCE_FROM_CENTER( &
           & MAPPING%LATITUDE, &
           & MAPPING%LONGITUDE, &
           & VIIRSLOC%LATITUDE(PV, LV), &
           & VIIRSLOC%LONGITUDE(PV, LV), &
           & DIST )
        DIST = DIST / 1000.0
      ENDIF
      IF( DIST .GT. DREF ) THEN
        FOUND = .TRUE.
        EXIT
      ENDIF
      DPRE = DIST
    END DO
    IF( .NOT. FOUND ) THEN
      ERR=98
      RETURN
    ENDIF
    MAPPING%ACROSSP(2) = PM - (DIST-DREF)/(DIST-DPRE) - (MAPPING%RADIUSP+1)


    ! semi axis along 1
    PM   = MAPPING%RADIUSP +1
    DREF = MAPPING%ALONGK(1)
    DPRE = 999.
    FOUND = .FALSE.
    DO LM = MAPPING%RADIUSL +1, 1, -1
      PV = MAPPING%VP(PM, LM)
      LV = MAPPING%VL(PM, LM)
      IF( PV .GT. 0 .AND. LV .GT. 0 ) THEN
        CALL MAP_DISTANCE_FROM_CENTER( &
           & MAPPING%LATITUDE, &
           & MAPPING%LONGITUDE, &
           & VIIRSLOC%LATITUDE(PV, LV), &
           & VIIRSLOC%LONGITUDE(PV, LV), &
           & DIST )
        DIST = DIST / 1000.0
      ENDIF
      IF( DIST .GT. DREF ) THEN
        FOUND = .TRUE.
        EXIT
      ENDIF
      !DPRE = DIST
    END DO
    IF( .NOT. FOUND ) THEN
      ERR=98
      RETURN
    ENDIF
    MAPPING%ALONGP(1) = MAPPING%RADIUSL +1 - &
                       & (LM + (DIST-DREF)/(DIST-DPRE))


    ! semi axis along 2
    PM   = MAPPING%RADIUSP +1
    DREF = MAPPING%ALONGK(2)
    DPRE = 999.
    FOUND = .FALSE.
    DO LM = MAPPING%RADIUSL +1, 2*MAPPING%RADIUSL +1
      PV = MAPPING%VP(PM, LM)
      LV = MAPPING%VL(PM, LM)
      IF( PV .GT. 0 .AND. LV .GT. 0 ) THEN
        CALL MAP_DISTANCE_FROM_CENTER( &
           & MAPPING%LATITUDE, &
           & MAPPING%LONGITUDE, &
           & VIIRSLOC%LATITUDE(PV, LV), &
           & VIIRSLOC%LONGITUDE(PV, LV), &
           & DIST )
        DIST = DIST / 1000.0
      ENDIF
      IF( DIST .GT. DREF ) THEN
        FOUND = .TRUE.
        EXIT
      ENDIF
      DPRE = DIST
    END DO
    IF( .NOT. FOUND ) THEN
      ERR=98
      RETURN
    ENDIF
    MAPPING%ALONGP(2) = LM - (DIST-DREF)/(DIST-DPRE) - (MAPPING%RADIUSL+1)

    RETURN
    END SUBROUTINE

    !> Compute the BOX size and fills with neighbours
    SUBROUTINE  MAP_CRIS_FOV_BOX(OPTIONS, MAPPING, TABLE, ERR)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFOVVIIRS), INTENT(INOUT)      :: MAPPING
    TYPE(ADJTABLE),  INTENT(IN)         :: TABLE
    INTEGER,         INTENT(OUT)        :: ERR

    REAL    :: V, V1, V2
    REAL    :: ENLARG = 1.20  ! 20% margin for FOV Shape
    INTEGER :: ICOL1, ICOL2, COEF
TRY
    !>- Compute FOV axis length in kilometres
    CALL MAP_CRIS_FOV_AXISKM(MAPPING)

    !>- Calculate the Adjacency window size 

    ! Take care of pixel size variation due to tie bow effect
    ! get pixel size at +- 10 pixels from center
    ICOL1 = MAX(MAPPING%VPC -10, 1)
    ICOL2 = MIN(MAPPING%VPC +10, TABLE%COLUMNSPERSCAN)

    V1 = MAPPING%ACROSSK(1) / TABLE%COLSIZE(ICOL1) *1000.0
    V2 = MAPPING%ACROSSK(2) / TABLE%COLSIZE(ICOL2) *1000.0 

    V = MAX(V1, V2)
    V = V * ENLARG
    MAPPING%RADIUSP = (INT(V/2)+1)*2 ! take even number

    V1 = MAPPING%ALONGK(1) / TABLE%ROWSIZE(ICOL1) *1000.0
    V2 = MAPPING%ALONGK(2) / TABLE%ROWSIZE(ICOL2) *1000.0

    V = MAX(MAPPING%ALONGK(1),MAPPING%ALONGK(2))/ TABLE%ROWSIZE(MAPPING%VPC) *1000.0
    V = V * ENLARG
    MAPPING%RADIUSL = (INT(V/2)+1)*2 ! take even number

    IF ( OPTIONS%DEBUG .GE. 2 ) THEN
      write(*,*) "RADIUS ACROSS K ",MAPPING%ACROSSK(1),MAPPING%ACROSSK(2)
      write(*,*) "RADIUS ALONG  K ",MAPPING%ALONGK(1),MAPPING%ALONGK(2)
      write(*,*) "RADIUS ROWSIZE  ",TABLE%ROWSIZE(MAPPING%VPC)," m"
      write(*,*) "RADIUS P avec elargissement ",MAPPING%RADIUSP
      write(*,*) "RADIUS L avec elargissement ",MAPPING%RADIUSL
    ENDIF

    ! Max Box size is defined for M Band channels
    COEF = 1
    IF( OPTIONS%VIIRS_BAND .EQ. "I" ) THEN
      COEF = 2
    ENDIF

    IF ( MAPPING%RADIUSP .GT. MAX_BOX_RADIUSP*COEF ) THEN
      ERR = 1
      THROWM(ERR.NE.0,"Box pixels radius too large")
    ENDIF

    IF ( MAPPING%RADIUSL .GT. MAX_BOX_RADIUSL*COEF ) THEN
      ERR = 1
      THROWM(ERR.NE.0,"Box lines radius too large")
    ENDIF

    !>- Allocate and fills box with VIIRS neighbors
    CALL MAP_VIIRS_NEIGHBORHOOD(MAPPING, TABLE, ERR)
    THROW(ERR.NE.0)

CATCH
    END SUBROUTINE

  !> Compute the BOX size and fills with neighbours for FOR (for clusters)
    SUBROUTINE  MAP_CRIS_FOR_BOX(OPTIONS, FORMAP, TABLE, ERR)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFORVIIRS), INTENT(INOUT)      :: FORMAP
    TYPE(ADJTABLE),  INTENT(IN)         :: TABLE
    INTEGER,         INTENT(OUT)        :: ERR

    INTEGER  :: IFOV, vlc,vpc
TRY
    !>- Calculate the Adjacency window size from FOV mappings
    FORMAP%flv = FORMAP%fovs(5)%vnl
    FORMAP%llv= 0
    FORMAP%fpv = FORMAP%fovs(5)%vnp
    FORMAP%lpv = 0
    FORMAP%line = FORMAP%fovs(5)%line
    FORMAP%for = FORMAP%fovs(5)%for
    FORMAP%vpc = FORMAP%fovs(5)%vpc
    FORMAP%vlc = FORMAP%fovs(5)%vlc
    FORMAP%vnl = FORMAP%fovs(5)%vnl
    FORMAP%vnp = FORMAP%fovs(5)%vnp
    
    DO IFOV = 1,9
        IF ( OPTIONS%DEBUG .EQ. 2 ) THEN 
            write(*,*) IFOV, MINVAL(FORMAP%fovs(IFOV)%VL(1:FORMAP%fovs(IFOV)%nb_pixels)), &
                             MAXVAL(FORMAP%fovs(IFOV)%VL(1:FORMAP%fovs(IFOV)%nb_pixels)), &
                             MINVAL(FORMAP%fovs(IFOV)%VP(1:FORMAP%fovs(IFOV)%nb_pixels)), &
                             MAXVAL(FORMAP%fovs(IFOV)%VP(1:FORMAP%fovs(IFOV)%nb_pixels))
        ENDIF
        FORMAP%flv = MIN (FORMAP%flv,MINVAL(FORMAP%fovs(IFOV)%VL(1:FORMAP%fovs(IFOV)%nb_pixels)))
        FORMAP%llv = MAX (FORMAP%llv,MAXVAL(FORMAP%fovs(IFOV)%VL(1:FORMAP%fovs(IFOV)%nb_pixels)))
        FORMAP%fpv = MIN (FORMAP%fpv,MINVAL(FORMAP%fovs(IFOV)%VP(1:FORMAP%fovs(IFOV)%nb_pixels)))
        FORMAP%lpv = MAX (FORMAP%lpv,MAXVAL(FORMAP%fovs(IFOV)%VP(1:FORMAP%fovs(IFOV)%nb_pixels)))
    END DO
    IF (OPTIONS%DEBUG .GE. 2) write(*,*) "FORMAP BOX:", &
            FORMAP%flv,FORMAP%llv,FORMAP%fpv,FORMAP%lpv
    ! FIND Radius
    FORMAP%RADIUSL = (FORMAP%llv - FORMAP%flv) / 2
    FORMAP%RADIUSP = (FORMAP%lpv - FORMAP%fpv) / 2

    ! new center 
    vlc =  FORMAP%flv + FORMAP%RADIUSL
    vpc =  FORMAP%fpv + FORMAP%RADIUSP
    
    write(*,*)  "center", FORMAP%vlc, FORMAP%vpc, "new:", vlc , vpc
    FORMAP%vpc = vpc
    FORMAP%vlc = vlc

    IF ( FORMAP%RADIUSL .eq. 0 ) THEN
     ERR = -1
    ENDIF
    IF ( OPTIONS%DEBUG .GE. 2 ) THEN
      write(*,*) "FOR ", FORMAP%FOR
      write(*,*) "RADIUSP ",FORMAP%RADIUSP
      write(*,*) "RADIUSL ",FORMAP%RADIUSL
    ENDIF
    IF (associated(FORMAP%vl)) THEN
       write(*,*) "deallocate FORMAP%vl"
       deallocate (FORMAP%vl,stat=err)
       THROWM(ERR.NE.0,"cannot deallocateFORMAP%vl" )
    ENDIF
    IF (associated(FORMAP%vp)) THEN
       write(*,*) "deallocate FORMAP%vp"
       deallocate (FORMAP%vp,stat=err)
       THROWM(ERR.NE.0,"cannot deallocateFORMAP%vp" )
    ENDIF

    !>- Allocate and fills box with VIIRS neighbors
    CALL MAP_VIIRS_FOR_NEIGHBORHOOD(FORMAP, TABLE, ERR)
    THROW(ERR.NE.0)

    ! new dimensions of the correponding BOX on VIIRS GRID
    FORMAP%flv = MIN(FORMAP%flv, MINVAL(FORMAP%VL, mask = (FORMAP%VL .ge. 0)))
    FORMAP%llv = MAX(FORMAP%llv, MAXVAL(FORMAP%VL, mask = (FORMAP%VL .ge. 0)))
    FORMAP%fpv = MIN(FORMAP%fpv, MINVAL(FORMAP%VP, mask = (FORMAP%VP .ge. 0)))
    FORMAP%lpv = MAX(FORMAP%lpv, MAXVAL(FORMAP%VP, mask = (FORMAP%VP .ge. 0)))
    IF (OPTIONS%DEBUG .GE. 2) write(*,*) "NEW FORMAT BOX:" ,&
            FORMAP%flv,FORMAP%llv,FORMAP%fpv,FORMAP%lpv
CATCH
    END SUBROUTINE

    !> Calculate the CrIS FOV shape
    !> Ellipse array contains the result of ellipse equation (==1 on the contour).
    !> Weights array contains the FOV shape weights function with a sum of 1.0.
    !> EllipseP/EllipseL conatins the FOV contour in VIIRS coordinates (only used for plotting).
    SUBROUTINE MAP_CRIS_FOV_ELLIPSE(MAPPING)
    IMPLICIT NONE
    REAL(KIND=8), PARAMETER  ::  PI=3.141592654d0

    TYPE(CFOVVIIRS), INTENT(INOUT)   :: MAPPING

    INTEGER :: PM, LM  ! Pixel/line in mapping box
    
    REAL    :: A, B, A1, A2, B1, B2   ! elipse axes in pixels/lines
    REAL    :: AS, BS ! square
    INTEGER :: U, V   ! Elipse center
    REAL    :: F, W, T
    INTEGER :: M, ELN
    REAL(KIND=8) :: SUMW
    
    U = MAPPING%RADIUSP +1
    V = MAPPING%RADIUSL +1

    ! ellipse equation 
    !  (x-u)*(x-u) + (y-u)*(y-u)
    !   ---------  +  ---------  = k
    !      a*a           b*b
    ! with
    !  u,v center
    !  a,b semi-axis
    !
    ! ellipse equation is k=1
    !   k < 1 for any (x,y) inside the ellipse
    !   k > 1 for outside
    ! Any proportional coef f applied on a and b will
    !   give k / (f*f)
    
    !>- CrIS FOV shape is given relative to the FOV angle
    !!!    Joint Polar Satellite System (JPSS)
    !!!    Cross Track Infrared Sounder (CrIS) Sensor Data Records (SDR)
    !!!    Algorithm theoretical Basis Document (ATBD)
    !!!    Table 3: FOV shape characteristics page 34
    !>-    FOV shape en degres
    !> -    Référence FOV=16.8mrad = 0.963degree
    !> -    70% of Peak Response Width > 0.8735   coef 0.8735/0.963 = 0.906
    !> -    50% of Peak Response Width = 0.942    coef 0.9420/0.963 = 0.978
    !> -    10% of Peak Response Width < 1.100    coef 1.1000/0.963 = 1.142
    !> -     3% of Peak Response Width < 1.238    coef 1.2380/0.963 = 1.286  

    ! consider average axis size for along track
    B = (MAPPING%ALONGP(1)+MAPPING%ALONGP(2))/2.
    BS = B*B

    A = MAPPING%ACROSSP(1)
    AS = A*A

    DO LM = 1, 2*MAPPING%RADIUSL +1
    DO PM = 1, MAPPING%RADIUSP
      MAPPING%ELLIPSE(PM, LM) = &
        (PM - U)*(PM - U)/AS + (LM-V)*(LM-V)/BS
    END DO
    END DO
    
    A = MAPPING%ACROSSP(2)
    AS = A*A

    DO LM = 1, 2*MAPPING%RADIUSL +1
    DO PM = MAPPING%RADIUSP +1, 2*MAPPING%RADIUSP +1
      MAPPING%ELLIPSE(PM, LM) = &
        (PM - U)*(PM - U)/AS + (LM-V)*(LM-V)/BS
    END DO
    END DO

    ! FOV shape
    MAPPING%WEIGHTS = SQRT(MAPPING%ELLIPSE)

    DO LM = 1, 2*MAPPING%RADIUSL +1
    DO PM = 1, 2*MAPPING%RADIUSP +1
      F = MAPPING%WEIGHTS(PM, LM)
      IF (F .GT. 1.286 ) THEN
          W = 0.0
      ELSE IF ( F .GT. 1.142 ) THEN
          W = 0.10 - ((F - 1.142) / (1.286 - 1.142) * 0.07)
      ELSE IF ( F .GT. 0.978 ) THEN
          W = 0.50 - ((F - 0.978) / (1.142 - 0.978) * 0.40)
      ELSE IF ( F .GT. 0.906 ) THEN
          W = 0.70 - ((F - 0.906) / (0.978 - 0.906) * 0.20)
      ELSE
          W = 0.70
      ENDIF
      MAPPING%WEIGHTS(PM, LM) = W
    END DO
    END DO
    SUMW = SUM(MAPPING%WEIGHTS)
    MAPPING%WEIGHTS = MAPPING%WEIGHTS * 1.D0 / SUMW

    ! FOV Contour; only used for plotting
    B = (MAPPING%ALONGP(1)+MAPPING%ALONGP(2))/2.
    B1 = B
    B2 = B

    A1 = MAPPING%ACROSSP(2)
    A2 = MAPPING%ACROSSP(1)
    ELN = SIZE(MAPPING%ELLIPSEP)
    DO M = 1, ELN
      T = M*2.0*PI/(ELN-1)
      IF(T .LT. PI/2.) THEN
          MAPPING%ELLIPSEP(M) = U + A1 * COS(T)
          MAPPING%ELLIPSEL(M) = V + B1 * SIN(T)
      ELSE IF(T .LT. PI) THEN
          MAPPING%ELLIPSEP(M) = U + A2 * COS(T)
          MAPPING%ELLIPSEL(M) = V + B1 * SIN(T)
      ELSEIF(T .LT. 3.*PI/2.) THEN
          MAPPING%ELLIPSEP(M) = U + A2 * COS(T)
          MAPPING%ELLIPSEL(M) = V + B2 * SIN(T)
      ELSE IF(T .LT. 2.*PI) THEN
          MAPPING%ELLIPSEP(M) = U + A1 * COS(T)
          MAPPING%ELLIPSEL(M) = V + B2 * SIN(T)
      ELSE
          MAPPING%ELLIPSEP(M) = U + A1 * COS(T)
          MAPPING%ELLIPSEL(M) = V + B1 * SIN(T)
      ENDIF
    END DO

    END SUBROUTINE

    !> Use the adjacency table to fill the neighborhood box around center FOV.
    !> This ensure all VIIRS pixels in box are regularly spaced 
    SUBROUTINE MAP_VIIRS_NEIGHBORHOOD(MAPPING, TABLE, ERR)
    IMPLICIT NONE
    TYPE(ADJTABLE),  INTENT(IN)    :: TABLE
    TYPE(CFOVVIIRS), INTENT(INOUT) :: MAPPING
    INTEGER,         INTENT(OUT)   :: ERR
    
    INTEGER :: SCAN, NX, NY, XSIZE, YSIZE
    INTEGER :: X, Y, I, J
    INTEGER :: XA, YA
    LOGICAL :: OK
    CHARACTER(LEN=32) :: MSG
TRY

    SCAN  = (MAPPING%vlc-1) / TABLE%rowsPerScan +1
    NY    = TABLE%totalScans * TABLE%rowsPerScan
    NX    = TABLE%columnsPerScan
    YSIZE = 2 * MAPPING%radiusl + 1
    XSIZE = 2 * MAPPING%radiusp + 1
    
    ALLOCATE(MAPPING%vp(XSIZE,YSIZE),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate MAPPING%vp array")
    ALLOCATE(MAPPING%vl(XSIZE,YSIZE),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate MAPPING%vl array")

    ! missing pixels are flagged with -2
    MAPPING%vl = - 2 
    MAPPING%vp = - 2 
    I = 0
    DO Y = MAPPING%vlc - MAPPING%radiusl , MAPPING%vlc + MAPPING%radiusl
      I = I + 1
      IF( Y .GT. 0 .AND. Y .LE. NY ) THEN
        J = 0
        DO X = MAPPING%vpc - MAPPING%radiusp , MAPPING%vpc + MAPPING%radiusp
          J = J+1
          IF( X .GT. 0 .AND. X .LE. NX ) THEN
            OK = GETADJPIXEL(TABLE, y, x, scan, ya, xa)
            IF ( OK ) THEN
              MAPPING%vp(J,I) = xa
              MAPPING%vl(J,I) = ya
            END IF
          ENDIF
        END DO
      END IF
   
    END DO

    IF( MAPPING%vpc .NE. MAPPING%vp(MAPPING%radiusp+1,MAPPING%radiusl+1) .OR. &
      & MAPPING%vlc .NE. MAPPING%vl(MAPPING%radiusp+1,MAPPING%radiusl+1)) THEN
      ! This should never occur; some kind of internal error
      WRITE(MSG,'("Line ",i4.4," FOR ",i2.2," FOV ",i2.2)') &
          & MAPPING%line, MAPPING%for,MAPPING%fov
      ERR=1
      THROWM(ERR.NE.0,"Neighborhood, centre has shifted "//TRIM(MSG))
    ENDIF

CATCH
    END SUBROUTINE

    !> Use the adjacency table to fill the neighborhood box around center FOR.
    !> This ensure all VIIRS pixels in box are regularly spaced 
    SUBROUTINE MAP_VIIRS_FOR_NEIGHBORHOOD(MAPPING, TABLE, ERR)
    IMPLICIT NONE
    TYPE(ADJTABLE),  INTENT(IN)    :: TABLE
    TYPE(CFORVIIRS), INTENT(INOUT) :: MAPPING
    INTEGER,         INTENT(OUT)   :: ERR
    
    INTEGER :: SCAN, NX, NY, XSIZE, YSIZE
    INTEGER :: X, Y, I, J
    INTEGER :: XA, YA
    LOGICAL :: OK
    CHARACTER(LEN=32) :: MSG
TRY

    SCAN  = (MAPPING%vlc-1) / TABLE%rowsPerScan +1
    NY    = TABLE%totalScans * TABLE%rowsPerScan
    NX    = TABLE%columnsPerScan
    YSIZE = 2 * MAPPING%radiusl + 1
    XSIZE = 2 * MAPPING%radiusp + 1
    
    ALLOCATE(MAPPING%vp(XSIZE,YSIZE),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate MAPPING%vp array")
    ALLOCATE(MAPPING%vl(XSIZE,YSIZE),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate MAPPING%vl array")

    ! missing pixels are flagged with -2
    MAPPING%vl = - 2 
    MAPPING%vp = - 2 
    I = 0
    DO Y = MAPPING%vlc - MAPPING%radiusl , MAPPING%vlc + MAPPING%radiusl
      I = I + 1
      IF( Y .GT. 0 .AND. Y .LE. NY ) THEN
        J = 0
        DO X = MAPPING%vpc - MAPPING%radiusp , MAPPING%vpc + MAPPING%radiusp
          J = J+1
          IF( X .GT. 0 .AND. X .LE. NX ) THEN
            OK = GETADJPIXEL(TABLE, y, x, scan, ya, xa)
            IF ( OK ) THEN
              MAPPING%vp(J,I) = xa
              MAPPING%vl(J,I) = ya
            END IF
          ENDIF
        END DO
      END IF
   
    END DO

    IF( MAPPING%vpc .NE. MAPPING%vp(MAPPING%radiusp+1,MAPPING%radiusl+1) .OR. &
      & MAPPING%vlc .NE. MAPPING%vl(MAPPING%radiusp+1,MAPPING%radiusl+1)) THEN
      ! This could sometimes occur

      WRITE(MSG,'("Line ",i4.4," FOR ",i2.2)') &
          & MAPPING%line, MAPPING%for
      WRITE(*,*) "WARNING MAPPING FOR: " , MSG
    ENDIF

CATCH
    END SUBROUTINE
   

    !> Determines the CrIS FOV mask in VIIRS data
    SUBROUTINE MAP_CRIS_FOV_MASK(OPTIONS, MAPPING, VIIRSLOC, ERR)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS), INTENT(IN) :: OPTIONS
    TYPE(CFOVVIIRS),      INTENT(INOUT) :: MAPPING
    TYPE(VIIRS_LOCATION), INTENT(IN)    :: VIIRSLOC
    INTEGER,              INTENT(OUT)   :: ERR
    
    INTEGER :: XSIZE, YSIZE, I, J, P, L
TRY
    !>- compute FOV axis in pixels/lines and size of mapping box
    CALL MAP_CRIS_FOV_AXISPIX(MAPPING, VIIRSLOC, ERR)
    IF( ERR.EQ.98 ) THEN
      IF( OPTIONS%DEBUG .GE. 2) THEN
        write(*,*) "UNABLE TO FIT FOV IN VIIRS BOX ERROR 98"
      ENDIF
      ! UNABLE TO FIT FOV IN VIIRS BOX
      RETURN
    ENDIF

    YSIZE = 2 * MAPPING%radiusl + 1
    XSIZE = 2 * MAPPING%radiusp + 1

    ALLOCATE(MAPPING%ELLIPSE(XSIZE,YSIZE),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate MAPPING%ellipse array")
    
    ALLOCATE(MAPPING%WEIGHTS(XSIZE,YSIZE),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate MAPPING%WEIGHTS array")
    
    ALLOCATE(MAPPING%ELLIPSEP(EL_NMAX),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate MAPPING%ELLIPSEP array")
    ALLOCATE(MAPPING%ELLIPSEL(EL_NMAX),stat=err)
    THROWM(ERR.NE.0,"Cannot  allocate MAPPING%ELLIPSEL array")

    !>- calculate the FOV ellipse and weights in mapping box
    CALL MAP_CRIS_FOV_ELLIPSE(MAPPING)
    
    IF( VIIRSLOC%TESTQUAL ) THEN
      DO I = 1, XSIZE
        DO J = 1, YSIZE
          P = MAPPING%VP(I,J)
          L = MAPPING%VL(I,J)
          IF( P .GT. 0 .AND. L .GT. 0 ) THEN
            IF( .NOT. VIIRSLOC%QUAL(P,L) ) THEN
              MAPPING%VP(I,J)      = -999
              MAPPING%VL(I,J)      = -999
              MAPPING%WEIGHTS(I,J) = -999
            ENDIF
          ENDIF
        END DO
      END DO
    ENDIF

CATCH
    END SUBROUTINE

    !> Clear memory for end of processing
    !> VIIRS SDR, MAIA, geolocoation
    SUBROUTINE MAP_VIIRS_FREE(SDR, MAIA4, VIIRSLOC, ERR)
USE VIIRS_SDR_DEF
USE VIIRS_SDR_MEM

    IMPLICIT NONE
    TYPE(VIIRS_SDR),      INTENT(INOUT) :: SDR
    TYPE(VIIRS_MAIA4),    INTENT(INOUT) :: MAIA4
    TYPE(VIIRS_LOCATION), INTENT(INOUT) :: VIIRSLOC
    INTEGER,              INTENT(OUT)   :: ERR

TRY

    CALL VIIRS_SDR_FREE(SDR,ERR) 
    THROWM(ERR.NE.0,"Cannot free VIIRS SDR structure")
    
    IF ( MAIA4%LMAIA4 ) THEN
      CALL MAIA4_FREE(MAIA4, ERR)
      THROWM(ERR.NE.0,"Cannot free MAIA4 structure")
    ENDIF
    
    CALL VIIRS_LOCATION_FREE(VIIRSLOC, ERR)
    THROWM(ERR.NE.0,"Cannot free viirs location")

CATCH
    END SUBROUTINE

    !> Calculate a vector product NA x NB
    REAL FUNCTION PRODVECT(XN,YN,XA,YA,XB,YB)
      IMPLICIT NONE
       REAL, INTENT(IN) :: XN
       REAL, INTENT(IN) :: YN
       REAL, INTENT(IN) :: XA
       REAL, INTENT(IN) :: YA
       REAL, INTENT(IN) :: XB
       REAL, INTENT(IN) :: YB

       PRODVECT = (XA-XN)*(YB-YN) - (XB-XN)*(YA-YN)
      RETURN
    END FUNCTION

    !> Calculate Earth fixed coordinates for lat/lon 2 dimensions arrays.
    !> This should be multiplied by Earth radius if km is wanted
    SUBROUTINE LATLON_TO_XYZ(N1,N2,LAT,LON,X,Y,Z)
      IMPLICIT NONE

#include "constants_preproc.h"

!Arguments:
      INTEGER, INTENT(IN)  :: N1,N2
      REAL,    INTENT(IN)  :: LAT(N1,N2),LON(N1,N2)
      REAL,    INTENT(OUT) :: X(N1,N2),Y(N1,N2),Z(N1,N2)

!Local variables:
      INTEGER I,J
      REAL COSLAT

      DO I=1,N1
        DO J=1,N2
            COSLAT = COS(LAT(I,J)*DEG2RAD)
            X(I,J) = COS(LON(I,J)*DEG2RAD)*COSLAT
            Y(I,J) = SIN(LON(I,J)*DEG2RAD)*COSLAT
            Z(I,J) = SIN(LAT(I,J)*DEG2RAD)
        ENDDO
      ENDDO
      END SUBROUTINE

    !> Calculate Earth fixed coordinates for lat/lon.
    !> This should be multiplied by Earth radius if km is wanted
    SUBROUTINE LATLON_TO_XYZ0D(LAT,LON,X,Y,Z)
      IMPLICIT NONE

#include "constants_preproc.h"

!Arguments:
      REAL,    INTENT(IN)  :: LAT,LON
      REAL,    INTENT(OUT) :: X,Y,Z

!Local variables:
      REAL COSLAT

      COSLAT = COS(LAT*DEG2RAD)
      X      = COS(LON*DEG2RAD)*COSLAT
      Y      = SIN(LON*DEG2RAD)*COSLAT
      Z      = SIN(LAT*DEG2RAD)

      END SUBROUTINE

!> Initialise CFovViirs structure
subroutine cfovviirs_init(x)
type(cfovviirs),intent(inout)::x

  x%line=0
  x%for=0
  x%fov=0
  x%scnlinyr=0
  x%scnlindy=0
  x%scnlintime=0
  x%target_latitude=0.
  x%target_longitude=0.
  x%target_altitude=0.
  x%latitude=0.
  x%longitude=0.
  x%altitude=0.
  x%vpc=0
  x%vlc=0
  x%lps=0
  x%lpm=0
  x%llm=0
  x%vnl=0
  x%vnp=0
  x%radiusp=0
  x%radiusl=0
  x%acrossk=0.
  x%acrossp=0.
  x%alongk=0.
  x%alongp=0.
  nullify(x%vp)
  nullify(x%vl)
  nullify(x%ellipse)
  nullify(x%weights)
  nullify(x%ellipsep)
  nullify(x%ellipsel)
end subroutine

!> Dispose CFovViirs structure
subroutine cfovviirs_free(x,err)
type(cfovviirs),intent(inout)::x
integer,intent(out)::err
character(len=64)::sname
TRY
err = 0

sname='vp'
if(associated(x%vp))THEN
  deallocate(x%vp,stat=err)
  THROWM(ERR.NE.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%vp)
ENDIF
sname='vl'
if(associated(x%vl))THEN
  deallocate(x%vl,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%vl)
ENDIF
sname='ellipse'
if(associated(x%ellipse))THEN
  deallocate(x%ellipse,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%ellipse)
ENDIF
sname='weights'
if(associated(x%weights))THEN
  deallocate(x%weights,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%weights)
ENDIF
sname='ellipsep'
if(associated(x%ellipsep))THEN
  deallocate(x%ellipsep,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%ellipsep)
ENDIF
sname='ellipsel'
if(associated(x%ellipsel))THEN
  deallocate(x%ellipsel,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%ellipsel)
ENDIF
CATCH
end subroutine

!> Read CFovViirs structure from HDF5 file
subroutine cfovviirs_rh(x,lun,err)
type(cfovviirs),intent(out)::x
integer(hid_t),intent(in)::lun
integer,intent(out)::err
character(len=64)::sname
TRY

call cfovviirs_init(x)

sname='line'
call read_array_hdf(lun,sname,err,i0=x%line)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='for'
call read_array_hdf(lun,sname,err,i0=x%for)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='fov'
call read_array_hdf(lun,sname,err,i0=x%fov)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='scnlinyr'
call read_array_hdf(lun,sname,err,i0=x%scnlinyr)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='scnlindy'
call read_array_hdf(lun,sname,err,i0=x%scnlindy)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='scnlintime'
call read_array_hdf(lun,sname,err,i0=x%scnlintime)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='target_latitude'
call read_array_hdf(lun,sname,err,r0=x%target_latitude)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='target_longitude'
call read_array_hdf(lun,sname,err,r0=x%target_longitude)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='target_altitude'
call read_array_hdf(lun,sname,err,r0=x%target_altitude)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='latitude'
call read_array_hdf(lun,sname,err,r0=x%latitude)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='longitude'
call read_array_hdf(lun,sname,err,r0=x%longitude)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='altitude'
call read_array_hdf(lun,sname,err,r0=x%altitude)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='vpc'
call read_array_hdf(lun,sname,err,i0=x%vpc)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='vlc'
call read_array_hdf(lun,sname,err,i0=x%vlc)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='lps'
call read_array_hdf(lun,sname,err,i0=x%lps)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='lpm'
call read_array_hdf(lun,sname,err,i0=x%lpm)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='llm'
call read_array_hdf(lun,sname,err,i0=x%llm)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='vnl'
call read_array_hdf(lun,sname,err,i0=x%vnl)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='vnp'
call read_array_hdf(lun,sname,err,i0=x%vnp)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='radiusp'
call read_array_hdf(lun,sname,err,i0=x%radiusp)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='radiusl'
call read_array_hdf(lun,sname,err,i0=x%radiusl)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='acrossk'
call read_array_hdf(lun,sname,err,r1=x%acrossk)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='acrossp'
call read_array_hdf(lun,sname,err,r1=x%acrossp)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='alongk'
call read_array_hdf(lun,sname,err,r1=x%alongk)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='alongp'
call read_array_hdf(lun,sname,err,r1=x%alongp)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='vp'
call read_array_hdf(lun,sname,err,pi2=x%vp)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='vl'
call read_array_hdf(lun,sname,err,pi2=x%vl)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='ellipse'
call read_array_hdf(lun,sname,err,pr2=x%ellipse)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='weights'
call read_array_hdf(lun,sname,err,pr2=x%weights)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='ellipsep'
call read_array_hdf(lun,sname,err,pr1=x%ellipsep)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

sname='ellipsel'
call read_array_hdf(lun,sname,err,pr1=x%ellipsel)
THROWM(err.ne.0,"CANNOT READ "//trim(sname))

err=0
CATCH
end subroutine

!> Write CFovViirs structure to HDF5 file
subroutine cfovviirs_wh(x,lun,err, compress)
type(cfovviirs),intent(in)::x
integer(hid_t),intent(in)::lun
integer,intent(out)::err
logical,intent(in),optional::compress
character(len=64)::sname
TRY
sname='line'
call write_array_hdf(lun,sname,&
  & 'CrIS Line number',&
  & err,i0=x%line  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='for'
call write_array_hdf(lun,sname,&
  & 'CrIS Field of Regard number',&
  & err,i0=x%for  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='fov'
call write_array_hdf(lun,sname,&
  & 'CrIS Field of View number',&
  & err,i0=x%fov  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='scnlinyr'
call write_array_hdf(lun,sname,&
  & 'Scan line year',&
  & err,i0=x%scnlinyr  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='scnlindy'
call write_array_hdf(lun,sname,&
  & 'Scan line day of year',&
  & err,i0=x%scnlindy  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='scnlintime'
call write_array_hdf(lun,sname,&
  & 'scan line UTC time of day',&
  & err,i0=x%scnlintime , units = 'milli seconds of day' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='target_latitude'
call write_array_hdf(lun,sname,&
  & 'Target Latitude (positive North) degree ',&
  & err,r0=x%target_latitude , units = 'degrees' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='target_longitude'
call write_array_hdf(lun,sname,&
  & 'Target Longitude (positive East) degree',&
  & err,r0=x%target_longitude , units = 'degrees' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='target_altitude'
call write_array_hdf(lun,sname,&
  & 'Target Ellipsoid-Geoid separation ',&
  & err,r0=x%target_altitude , units = 'kilometer' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='latitude'
call write_array_hdf(lun,sname,&
  & 'Latitude (positive North) degree ',&
  & err,r0=x%latitude , units = 'degrees' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='longitude'
call write_array_hdf(lun,sname,&
  & 'Longitude (positive East) degree',&
  & err,r0=x%longitude , units = 'degrees' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='altitude'
call write_array_hdf(lun,sname,&
  & 'Ellipsoid-Geoid separation ',&
  & err,r0=x%altitude , units = 'kilometer' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='vpc'
call write_array_hdf(lun,sname,&
  & 'VIIRS pixel for center of CrIS FOV',&
  & err,i0=x%vpc  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='vlc'
call write_array_hdf(lun,sname,&
  & 'VIIRS line for center of CrIS FOV',&
  & err,i0=x%vlc  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='lps'
call write_array_hdf(lun,sname,&
  & 'number of lines per scan',&
  & err,i0=x%lps  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='lpm'
call write_array_hdf(lun,sname,&
  & 'last pixel move',&
  & err,i0=x%lpm  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='llm'
call write_array_hdf(lun,sname,&
  & 'last line move',&
  & err,i0=x%llm  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='vnl'
call write_array_hdf(lun,sname,&
  & 'VIIRS number of lines in array',&
  & err,i0=x%vnl  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='vnp'
call write_array_hdf(lun,sname,&
  & 'VIIRS number of pixels in array',&
  & err,i0=x%vnp  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='radiusp'
call write_array_hdf(lun,sname,&
  & 'Adjacency window size in VIIRS pixels',&
  & err,i0=x%radiusp  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='radiusl'
call write_array_hdf(lun,sname,&
  & 'Adjacency window size in VIIRS lines',&
  & err,i0=x%radiusl  )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='acrossk'
call write_array_hdf(lun,sname,&
  & 'across track axes',&
  & err,r1=x%acrossk , units = 'km' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='acrossp'
call write_array_hdf(lun,sname,&
  & 'across track axes',&
  & err,r1=x%acrossp , units = 'pixels' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='alongk'
call write_array_hdf(lun,sname,&
  & 'along track axes',&
  & err,r1=x%alongk , units = 'km' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='alongp'
call write_array_hdf(lun,sname,&
  & 'along track axes',&
  & err,r1=x%alongp , units = 'lines' )
THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))

sname='vp'
if(associated(x%vp))THEN
  call write_array_hdf(lun,sname,&
    & 'VIIRS neighbours',&
    & err,i2=x%vp , units = 'pixels (-2 == outside scans)' ,compress=compress)
  THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
ENDIF

sname='vl'
if(associated(x%vl))THEN
  call write_array_hdf(lun,sname,&
    & 'VIIRS neighbours',&
    & err,i2=x%vl , units = 'lines (-2 == outside scans)' ,compress=compress)
  THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
ENDIF

sname='ellipse'
if(associated(x%ellipse))THEN
  call write_array_hdf(lun,sname,&
    & 'Ellipse ',&
    & err,r2=x%ellipse , units = '<1 inside; =1 ellipse, >1 outside' ,compress=compress)
  THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
ENDIF

sname='weights'
if(associated(x%weights))THEN
  call write_array_hdf(lun,sname,&
    & 'CrIS FOV shape weights ',&
    & err,r2=x%weights  ,compress=compress)
  THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
ENDIF

sname='ellipsep'
if(associated(x%ellipsep))THEN
  call write_array_hdf(lun,sname,&
    & 'CrIS FOV ellipse ',&
    & err,r1=x%ellipsep , units = 'VIIRS pixels' )
  THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
ENDIF

sname='ellipsel'
if(associated(x%ellipsel))THEN
  call write_array_hdf(lun,sname,&
    & 'CrIS FOV ellipse ',&
    & err,r1=x%ellipsel , units = 'VIIRS lines' )
  THROWM(err.ne.0,"CANNOT WRITE "//trim(sname))
ENDIF

err=0
CATCH
end subroutine

!> Initialise VIIRS_Location structure
subroutine viirs_location_init(x)
type(viirs_location),intent(inout)::x

  x%totalScans=0
  x%nlines=0
  x%rowsPerScan=0
  x%columnsPerScan=0
  x%testqual=.false.
  nullify(x%pixqual)
  nullify(x%latitude)
  nullify(x%longitude)
  nullify(x%UTCJulianDay)
  nullify(x%UTC)
  nullify(x%MaskBOWTIE)
  nullify(x%qual)

end subroutine

!> Dispose VIIRS_Location structure
subroutine viirs_location_free(x,err)
type(viirs_location),intent(inout)::x
integer,intent(out)::err
character(len=64)::sname
TRY
err = 0

!sname='latitude'
!if(associated(x%latitude))THEN
!  deallocate(x%latitude,stat=err)
!  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
!  nullify(x%latitude)
!ENDIF
!sname='longitude'
!if(associated(x%longitude))THEN
!  deallocate(x%longitude,stat=err)
!  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
!  nullify(x%longitude)
!ENDIF
!sname='UTCJulianDay'
!if(associated(x%UTCJulianDay))THEN
!  deallocate(x%UTCJulianDay,stat=err)
!  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
!  nullify(x%UTCJulianDay)
!ENDIF
!sname='UTC'
!if(associated(x%UTC))THEN
!  deallocate(x%UTC,stat=err)
!  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
!  nullify(x%UTC)
!ENDIF
nullify(x%latitude)
nullify(x%longitude)
nullify(x%UTCJulianDay)
nullify(x%UTC)

sname='MaskBOWTIE'
if(associated(x%MaskBOWTIE))THEN
  deallocate(x%MaskBOWTIE,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%MaskBOWTIE)
ENDIF
sname='pixqual'
if(associated(x%pixqual))THEN
  deallocate(x%pixqual,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%pixqual)
ENDIF
sname='QUAL'
if(associated(x%qual))THEN
  deallocate(x%qual,stat=err)
  THROWM(err.ne.0,"CANNOT DEALLOCATE "//trim(sname))
  nullify(x%qual)
ENDIF

CATCH
end subroutine

  !> put cforviirs in fov_type
    SUBROUTINE MAP_PUT_MAPPING_IN_FOV( OPTIONS, MAPPING, MYFOV)
    IMPLICIT NONE
    TYPE(MAP_VIIRS_OPTIONS),      INTENT(IN)  :: OPTIONS
    TYPE(cfovviirs), INTENT(IN)               :: MAPPING
    TYPE(fov_type),                INTENT(OUT) :: MYFOV
    INTEGER :: P,L, K  ,NP,NL

    K = 0
    MYFOV%line = MAPPING%LINE
    MYFOV%FOR= MAPPING%FOR
    MYFOV%FOV = MAPPING%FOV
    MYFOV%vpc= MAPPING%vpc
    MYFOV%vlc = MAPPING%vlc
    MYFOV%vnp= MAPPING%vnp
    MYFOV%vnl = MAPPING%vnl
    NP = SIZE( MAPPING%VP, 1 )
    NL = SIZE( MAPPING%VP, 2 )
    DO L=1,NL
     DO P=1,NP
       IF (MAPPING%weights(P,L) .gt. 0. .and. &
           MAPPING%VP(P,L) .ge. 1 .and. MAPPING%VL(P,L) .ge. 1 ) THEN
              K = K + 1
              MYFOV%vp(k) = MAPPING%VP(P,L)
              MYFOV%vl(k) = MAPPING%VL(P,L)
        ENDIF
      ENDDO
   ENDDO
   MYFOV%NB_PIXELS = K
   if (OPTIONS%DEBUG .eq. 2) THEN
   write(*,*) "MAX PIXEL IN FOV :", MYFOV%NB_PIXELS

   ENDIF

    END SUBROUTINE

#endif



END MODULE MAP_VIIRS_CRIS_MOD
