#! /bin/ksh
#
# Korn Shell for alleph: run the NOAA ephemeris scheme (short term)
#
# PROPRIETAIRE/OWNER
#       P. Brunel - A. Marsouin
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     13/02/95   initial            P. Brunel
#  2.0     April 2004  add 2lines, new satid file, better parametrisation
#                     for bulletin/satellites management
#                     do some cleaning 
#                              P. Brunel
#  2.1     Dec 2004   Minor change as suggested by Dirk Hein. N C Atkinson
#  2.2     Feb 2006   change config file P. Marguinaud
#  3.1     June 2006  add options -C and -O to allow the use outside 
#                     operationnal context and diverse configurations of
#                     priorities and outputs                 P. Brunel

#
# COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
#
# local function
function which_oper_bul {
  # Finds in the navigation parametrisation the orbital
  # bulletin which corresponds to the satellite given in argument 1
  # involved parameters are:
  # PAR_NAVIGATION_DEFAULT_LISTEBUL and PAR_NAVIGATION_DEFAULT_LISTESAT
  
  # if argument 2 and 3 are present then they are assumed as 
  # a default bulletin list (arg2) and a default satellite list (arg3)

  if [ $# = 1 ] ; then
    DEFAULT_LISTEBUL=${PAR_NAVIGATION_DEFAULT_LISTEBUL}
    DEFAULT_LISTESAT=${PAR_NAVIGATION_DEFAULT_LISTESAT}
  elif [ $# = 3 ] ; then
    DEFAULT_LISTEBUL=$2
    DEFAULT_LISTESAT=$3
  else
    echo $*
    echo "function which_oper_bul needs one or three arguments"
    echo " which_oper_bul sat [list_of_bulletins] [ list_of_satellites]"
    exit 1
  fi

  # array index
  typeset -i i=0
  
  # if no bulletin found then gives the default value to TBUS bulletins
  BUL=tbus

  SATIMG=$1
  set -A ABUL ${DEFAULT_LISTEBUL}
  for xSAT in  ${DEFAULT_LISTESAT}
  do
    if [ "${xSAT}" = "${SATIMG}" ] ; then
      BUL=${ABUL[i]}
      break
    fi
    i=`expr ${i} + 1`
  done
}



# MAIN

. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh


# 0 initialize the variables
#---------------------------

# variables for the TBUS
DIR_DATA_TBUS=${DIR_DATA_TBUS:=${DIR_NAVIGATION}/tbus_db}
if [ ! -d ${DIR_DATA_TBUS} ]; then
  DIR_DATA_TBUS=${DIR_NAVIGATION}/orb_elem
fi  
export DIR_DATA_TBUS

# variables for the ARGOS
DIR_DATA_ARGOS=${DIR_DATA_ARGOS:=${DIR_NAVIGATION}/argos_db}
if [ ! -d ${DIR_DATA_ARGOS} ]; then
  DIR_DATA_ARGOS=${DIR_NAVIGATION}/orb_elem
fi  
export DIR_DATA_ARGOS

# variables for the 2-Line
DIR_DATA_TLE=${DIR_DATA_TLE:=${DIR_NAVIGATION}/tle_db}
if [ ! -d ${DIR_DATA_TLE} ]; then
  DIR_DATA_TLE=${DIR_NAVIGATION}/orb_elem
fi  
export DIR_DATA_TLE

# variables for the spm
DIR_DATA_SPM=${DIR_DATA_SPM:=${DIR_NAVIGATION}/spm_db}
if [ ! -d ${DIR_DATA_SPM} ]; then
  DIR_DATA_SPM=${DIR_NAVIGATION}/orb_elem
fi  
export DIR_DATA_SPM

DIR_TMP=/tmp

# 1 usage
#-------- 
usage(){
	echo
	echo ' ALLEPH to run the NOAA ephemeris scheme (short term)'
	echo
	echo 'usage:  alleph  [-options]';
	echo '	options:';
	echo '  -s  satellite_list';
	echo "		default:${PAR_NAVIGATION_DEFAULT_LISTESAT}";
	echo '  -S  station_name		default:lannion';
	echo '  -b  bulletin_list';
	echo "		default:${PAR_NAVIGATION_DEFAULT_LISTEBUL}";
	echo '  -d  start_date			default:  today';
	echo '  -n  number of days (real)	default:    1.0';
	echo '  -i  increment in seconds (real)	default:  120.0';
	echo '  -c  search criteria (n/p)	default:nearest';
	echo '                      (n for nearest or p for preceeding)';
	echo
	echo '  -O  (no argument) final ephemerisis send to standard output';
	echo '         No single satellite ephemerisis file created';
	echo '         No tracking angle file created';
	echo '         default is to create files for ';
	echo '           single satellite ephemerisis  ${DIR_NAVIGATION}/ephe/ephe_SAT_YYYYMMDD.txt';
	echo '           multiple satellite ephemerisis  ${DIR_NAVIGATION}/ephe/ephe_YYYYMMDD.txt';
	echo '           tracking angles  ${DIR_NAVIGATION}/tracking/tracking_SAT_YYYYMMDD_NNNNN.txt';
	echo
	echo '  -C  (no argument) do not take care of conflicts ';
	echo '         default is to take care of antenna conflicts between';
	echo '         satellites for one antena system'
	echo '  -o  duration before the start of acquisition needed for the';
	echo '          antenna steering (seconds) default: 0';
	echo
	echo ' the date format could be a date or a date/hour string:';
	echo " for example -d 'dd/mm/yy  hh:mm:ss.sss' (2 spaces between yy hh)";
	echo "     or      -d 'dd/mm/yy  hh:mm'";
	echo "     or      -d dd/mm/yy";
	echo
	echo 'files are stored under ${DIR_NAVIGATION}:';
	echo '     ./satpos  ./ephe  ./tracking ';
	echo
	echo 'environment variables:'
	echo '	DIR_SATID    path for the satellite identification file'
	echo '	SATID_FILE    filename for the satellite identification file'
	echo '	DIR_STATIONS  path for the stations file'
        echo '  DIR_NAVIGATION  generic path for the argos dir argos_db'
	}

# 2 get the arguments
#--------------------
          while getopts s:S:b:d:n:i:c:o:CO c
          do
               case $c in
               s)	LISTESAT=$OPTARG;;
               S)	STATION=$OPTARG;;
               b)	LISTEBUL=$OPTARG;;
               d)	DATE=$OPTARG;;
               n)	NBD=$OPTARG;;
               i)	INC=$OPTARG;;
               c)	CRI=$OPTARG;;
               o)	POFF=$OPTARG;;
               C)	CARE_OF_CONFLICTS='no';;
               O)	STD_OUTPUT='yes';;
               \?)      usage
                        exit 2;;
               esac
          done

# 2.1 default values
#-------------------

STATION=${STATION='lannion'}

DATE=${DATE=`date +%d/%m/%y`}
NBD=${NBD='1.00'}
INC=${INC='120.00'}
CRI=${CRI='n'}
POFF=${POFF=0}
MOFF=`expr 0 - ${POFF}`
CARE_OF_CONFLICTS=${CARE_OF_CONFLICTS='yes'}
STD_OUTPUT=${STD_OUTPUT='no'}


if [ "${CRI}" = 'n' ] ; then
  ICRI=0
else
  ICRI=1
fi

set `echo ${DATE} | tr '/' ' '`
if [ $3 -gt 50 ] ; then
  DATEY=19$3$2$1
else
  DATEY=20$3$2$1
fi

if [ -z "${LISTEBUL}" ] ; then
  # Listbul argument not set
  # take default values from parametrisation
  LISTEBUL=${PAR_NAVIGATION_DEFAULT_LISTEBUL}
  ARG_LISTEBUL='no'
else
  ARG_LISTEBUL='yes'
fi
if [ -z "${LISTESAT}" ] ; then
  # ListSat argument not set
  # take default values from parametrisation
  LISTESAT=${PAR_NAVIGATION_DEFAULT_LISTESAT}
  ARG_LISTESAT='no'
else
  ARG_LISTESAT='yes'
fi

# Directory EPHE  (store ephemerisis files)
if [ "${STD_OUTPUT}" = 'yes' ] ; then
  log_line
  log_info "work for standard output only"
  log_info "no computation of tracking angles"
  log_line
  DIR_EPHE=${DIR_TMP}
else
  DIR_EPHE=${DIR_NAVIGATION}/ephe
  if [ ! -d ${DIR_EPHE} ]; then
    mkdir ${DIR_EPHE}
  fi
fi

# Directory TRACKING (for tracking angles)
if [ "${STD_OUTPUT}" = 'no' ] ; then
  DIR_TRACKING=${DIR_NAVIGATION}/tracking
  if [ ! -d ${DIR_TRACKING} ]; then
    mkdir ${DIR_TRACKING}
  fi
fi


# 3 loop on the satellites
#-------------------------

for SAT in ${LISTESAT}
do

typeset -l BUL
if [ "${ARG_LISTEBUL}"  = 'no' ] &&  [ "${ARG_LISTESAT}"  = 'no' ]; then
  which_oper_bul ${SAT}
elif [ "${ARG_LISTEBUL}"  = 'no' ] ; then
  # take bulletin name from the parametrisation lists
  which_oper_bul ${SAT}
else
  which_oper_bul ${SAT} "${LISTEBUL}" "${LISTESAT}"
fi


TBUS_INDEX=${DIR_DATA_TBUS}/tbus_${SAT}.index
ARGOS_INDEX=${DIR_DATA_ARGOS}/argos_${SAT}.index
TLE_INDEX=${DIR_DATA_TLE}/tle_${SAT}.index
SPM_INDEX=${DIR_DATA_SPM}/spm_${SAT}.index

SATPOSF="${DIR_TMP}/satpos_${SAT}_${DATEY}.txt$$"

if [ "${BUL}" = 'tbus' ] ; then
  BUL_PATH=${DIR_DATA_TBUS}
  BUL_INDEX=${TBUS_INDEX}
  SATPOS_CMD='satpost'
elif [ "${BUL}" = 'spm' ] ; then
  BUL_PATH=${DIR_DATA_SPM}
  BUL_INDEX=${SPM_INDEX}
  SATPOS_CMD='satposspm'
elif [ "${BUL}" = 'tle' ] ; then
  BUL_PATH=${DIR_DATA_TLE}
  BUL_INDEX=${TLE_INDEX}
  SATPOS_CMD='satpostle'
else
  BUL_PATH=${DIR_DATA_ARGOS}
  BUL_INDEX=${ARGOS_INDEX}
  SATPOS_CMD='satposa'
fi

log_info  'satellite   :'"${SAT}"
log_info  'satpos file :'"${SATPOSF}"
log_info  'bulletin    :'"${BUL}"

# 3.1 creates the temporary satpos file
#--------------------------------------

${SATPOS_CMD} -s $SAT -S $STATION -d $DATE -n $NBD -i $INC -c $CRI > ${SATPOSF}


crd=$?

if [ "${crd}" -ne 0 ] ; then
  rm -f ${SATPOSF}
fi

# 3.2 ephemeris
#--------------
if [ "${crd}" -eq 0 ] ; then
  ephe.exe < ${SATPOSF} > ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt
  crd=$?
else
  log_info pb satpos ${SAT} :ephe skipped
fi

# 3.3 tracking angles (only when storing ephemerisis files)
#----------------------------------------------------------
if [ "${STD_OUTPUT}" = 'no' ] ; then
  if [ "${crd}" -eq 0 ] ; then
    cd ${DIR_TRACKING}
#   tracking.exe < ${SATPOSF} >> ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt
    tracking.exe < ${SATPOSF} | cat >> ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt

#Previous line changed following report by Dirk Hein that on some systems the 
#append (>>) does not work and the file is over-written. The extra 'cat' 
#command seems to cure this, for reasons that are not clear. NCA 16 Dec 2004

    crd=$?
  else
    log_error "ephe or satpos ${SAT} :tracking skipped"
  fi
fi


if [ "${crd}" -eq 0 ] ; then
  sort -o ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt
fi

# remove temporary satpos file
rm -f ${SATPOSF}

done   # loop: for SAT in ${LISTESAT}


#4. mixes satellites then identifies antenna conflicts
#-----------------------------------------------------

touch ${DIR_EPHE}/ephe_${DATEY}.tmp
rm -f ${FORT}10
touch ${FORT}10

for SAT in ${LISTESAT}
do
  cat ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt >> ${DIR_EPHE}/ephe_${DATEY}.tmp
  echo ${SAT} >> ${FORT}10
done


# Take care of orbit conflicts for ONE antenna system
# and a possible delay for pointing the antenna
if [ "${CARE_OF_CONFLICTS}" = 'yes' ] ; then
  # take care of aquisition conflicts
  if [ "${POFF}" -eq 0 ] ; then
    sort ${DIR_EPHE}/ephe_${DATEY}.tmp | antcnft.exe  | sort | \
     sed 's/[ ][ ]*/	/g' > ${DIR_EPHE}/ephe_${DATEY}.tmp2
  else
    # same as above but uses a delay time between 2 trackings
    cat ${DIR_EPHE}/ephe_${DATEY}.tmp | driftephe.exe ${MOFF} | sort | \
     antcnft.exe | driftephe.exe ${POFF} | sort | \
     sed 's/[ ][ ]*/	/g' > ${DIR_EPHE}/ephe_${DATEY}.tmp2
  fi
else
  log_line
  log_info "satellites/antennas conflict ignored"
  log_line
  sort ${DIR_EPHE}/ephe_${DATEY}.tmp  | \
  sed 's/[ ][ ]*/	/g' > ${DIR_EPHE}/ephe_${DATEY}.tmp2
fi

# Send output to file or standard output
if [ "${STD_OUTPUT}" = 'yes' ] ; then
  cat ${DIR_EPHE}/ephe_${DATEY}.tmp2
else
  mv ${DIR_EPHE}/ephe_${DATEY}.tmp2 ${DIR_EPHE}/ephe_${DATEY}.txt
fi

rm -f fort.10
rm -f ${DIR_EPHE}/ephe_${DATEY}.tmp
rm -f ${DIR_EPHE}/ephe_${DATEY}.tmp2
rm -f ${DIR_TMP}/ephe_*${DATEY}.*

exit
