C LIBS =  ephtrack navtool satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: antcnft.F 448 2015-05-18 14:39:56Z frna $
!
        program antcnft
!
!BUT/PURPOSE 
!       ANTCNF (ANTenna CoNFlicT) identifies the acquisition conflicts for
!       a single antenna system
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       antcnf process an ephemeris file which contains several satellites and
!       is valid for only one station.
!       The ephemeris file for each satellite has been produced by ephem 
!       (and tracking), and the various files have been concatenated and
!       the final file has been sorted to be strictly chronological. 
!       antcnf modify this file to identify the orbits which are considered as 
!       conflict orbits.
!
!       A priority rule is established for the list of satellites read in 
!       unit 10, the first one having the higher priority, the second the
!       following...
!
!       Conflict rule:
!       When several orbits are overlapping the orbit with the higher priority
!       is kept and the other ones are identified as conflict orbits. The orbit
!       duration is not taken into account. No margin is considered to identify
!       overlapping orbits.
!       For conflict orbits the event field of the ephemeris message
!       becomes "start_conflict" or "stop_conflict".
!
!       
!       Inputs are:
!       standard input
!               ephemeris file
!       input file on unit 10 (fort.10)
!               satellite list (one by line)
!
!       Output
!       standard output
!               updated ephemeris file
!       
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   24 Nov 2006      Correct (start|stop)_conflict offset
!                                    P. Marguinaud
! 1.2   3  Mar 2015      satmax = 10 instead of 5 P.Brunel
    
!EXIT CODES    
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!DECLARATIONS
!
! Local parameters:
        integer prunit          ! logical unit for the priority satellite list
        parameter (prunit=10)
        integer satmax          ! maximum number of satellites to process
        parameter(satmax=10)
!
! variables in input
        character*80 scname             ! input satellite name
        character*16 satlist(satmax)    ! NOAA satelite name list

!
! Commons
!
! Functions
        integer genscid                 ! order number for a satellite name
        character*16 genscnam          ! generic satellite name
        integer prorder                 ! priority order
!
! 
! Local variables:
        character*82 line_ephe          ! ephemeris line message
        character*82 start_ephe(satmax) ! start_acq line message
        character*82 stop_ephe(satmax)  ! stop_acq line message
        integer orb_status(satmax)      ! orbit status indicator
!                          0= orbit considered as a conflict orbit
!                          1= the status conflict or not conflict is unknown
!
        character*16 satname            ! satellite name
        character*16 event              ! event message 

        integer ios                     ! I/O status variable
        integer ordernum                ! order number of a satellite
        integer nbsat                   ! number of satellites processed
        integer nbwait                  ! number of orbits in the waiting queue
        integer psat                    ! satellite priority number
        integer isat                    ! satellite number for loops
!
!
!
!
! 
!-
        call ml_wt('info','start')
!
!       Section 1
!       Reads the input parameters
!       --------------------------
!
!       Get the satellite name list
        nbsat=0
        read(prunit,'(a)',iostat=ios) scname
        do while(ios.eq.0 .and. nbsat.lt.satmax)
          ordernum= genscid(scname)
          if(ordernum.eq.0) then
            call ml_wt('notice','Input satellite'//scname)
            call ml_wt('notice',
     &       'satellite unknown in the identification file')
          else
            nbsat=nbsat+1
            satlist(nbsat) = genscnam(scname)
            orb_status(nbsat)=0
          endif

          read(prunit,'(a)',iostat=ios) scname
        end do

        if(nbsat.eq.0) then
          call ml_wt('error',
     &      'no satellite in the priority file')
          call ml_wt('error','ANTCNFT aborted')
          call aappexit(1)
        end if
!
!
!       Section 2
!       Loop over the ephemeris file
!       -----------------------------
        nbwait=0
        read(*,'(a)',iostat=ios) line_ephe
        satname=line_ephe(26:41)
        event=line_ephe(50:65)

        do while (ios.eq.0) 
          psat=prorder(nbsat,satlist,satname)

!         if event is not start_acq or stop_acq send it to standard output
          if(event(1:9) .ne. 'start_acq' .and. 
     &       event(1:8) .ne. 'stop_acq' ) then
             write(*,'(a)') line_ephe

!         start of acquisition
          elseif(event(1:9) .eq. 'start_acq' .and. psat.ne.0) then
            start_ephe(psat) = line_ephe
            stop_ephe(psat) = ' '
            orb_status(psat)=1
            nbwait=nbwait+1

!         stop of acquisition for a conflict orbit
          elseif(event(1:8) .eq. 'stop_acq' .and. 
     &           psat.ne.0                   .and.
     &           orb_status(psat).eq.0 )       then
            line_ephe(50:65) = 'stop_conflict   '
            write(*,'(a)') line_ephe

!         stop of acquisition
          elseif(event(1:8) .eq. 'stop_acq' .and. 
     &           psat.ne.0                   .and.
     &           orb_status(psat).eq.1 )       then
            stop_ephe(psat) = line_ephe

!           suppress the waiting acquisitions which are less prior
            if(psat.lt.nbsat) then
              do isat=psat+1,nbsat
                if(orb_status(isat).eq.1) then
                  call ml_wt('info','orbit conflict for '//
     &               start_ephe(isat)(26:31)//
     &             ' orbit '//start_ephe(isat)(33:38))
                  orb_status(isat)=0
                  nbwait=nbwait-1
                  start_ephe(isat)(50:65) = 'start_conflict  '
                  write(*,'(a)') start_ephe(isat)
                  if(stop_ephe(isat) .ne. ' ') then
                    stop_ephe(isat)(50:65) = 'stop_conflict   '
                    write(*,'(a)') stop_ephe(isat)
                  endif
                endif
              end do
            endif

!           output the ephe. message if there is only one in queue
            if(nbwait.eq.1) then
              write(*,'(a)') start_ephe(psat)
              write(*,'(a)') stop_ephe(psat)
              orb_status(psat)=0
              nbwait=0
            endif
          endif

        read(*,'(a)',iostat=ios) line_ephe
        satname=line_ephe(26:41)
        event=line_ephe(50:65)

        end do
!
!
        call ml_wt('info','normal end')
!
        stop

        end
        integer function prorder(nbsat,satlist,satname)
!+
!NOM/NAME $Id: antcnft.F 448 2015-05-18 14:39:56Z frna $
!       integer function prorder(nbsat,satlist,satname)
!    
!BUT/PURPOSE 
!       returns the priority order of a NOAA satellite
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer nbsat                   ! number of satellites
        character*16 satlist(nbsat)      ! satellite list
        character*16 satname             ! satellite name
!
!DECLARATIONS
!
!
!
! Local parameters
!
! Commons
!
! Functions
        logical cp_cus  ! f7cp  compares strings (ignorecase)
!
! Local variables
        integer isat    ! satellite index
! 
!-
!
        isat=1

        do while (.not.cp_cus(satlist(isat),satname) .and. 
     &            isat.le.nbsat)
          isat=isat+1
        end do
        
        if(isat.gt.nbsat) then
          prorder=0
        else
          prorder=isat
        endif

        return
        end
