C LIBS = ephtrack navtool satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: ephe.F 47 2006-08-21 08:44:41Z frna $
!
        program ephe
!
!BUT/PURPOSE 
!       EPHE creates the ephemeris file corresponding to a satpos file.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       For the whole satpos file duration, the program calculates
!         the time of the ascending and descending nodes
!         the times of start and end of acquistion by the station 
!           (read in satpos)
!       
!       input/output are on standard units (pipe is possible)
!       Inputs are:
!       standard input
!               satpos file 
!
!       Output
!       standard output
!               ephemeris file
!       
! Remarks:
!       The run is done for only one satellite due to the satpos file
!       The ephemeris file has to be sorted to be strictly chronological
!       
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!     
!EXIT CODES    
!       0 = OK
!       2=IO error
!       3=convergence error for node
!       4=convergence error for start/stop acquisition
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!DECLARATIONS
!
! Local variables:
!
        double precision ijstart! start julian instant
        double precision ijstop ! stop julian instant
        integer luin            ! logical unit for the satpos file
        integer luout           ! logical unit for the output
        integer ierr            ! return code 0=OK  
                                !             1=periods do not mach
                                !             2=IO error
                                !             3=convergence error for node
                                !             4=convergence error for acquis.
!
!
!
!
! 
!-
!       units are standard input and output
        luin=5
        luout=6
        
!       the whole satpos file is processed (times = 0)
        ijstart = 0.d0
        ijstop   = 0.d0

        call ml_wt('info','start')
        call sp_ephe(ijstart,ijstop,luin,luout,ierr)

        if(ierr.eq.2) then
          call ml_wt('error','I/O error on satpos file')
        elseif(ierr.eq.3) then
          call ml_wt('error','convergence error for node')
        elseif(ierr.eq.4) then
          call ml_wt('error','convergence error for acquisition')
        endif

        if(ierr.ne.0) then
          call ml_wti('error','stop',ierr)
          call aappexit(ierr)
        else
          call ml_wt('info','normal end')
        endif

        stop
        end
        subroutine sp_ephe(ijstart,ijstop,luin,luout,ierr)
!+
!NOM/NAME $Id: ephe.F 47 2006-08-21 08:44:41Z frna $
!       subroutine sp_ephe(ijstart,ijstop,luin,luout,ierr)
!
!BUT/PURPOSE 
!       reads the SATPOS file in unit "luin" between the start and stop julian
!       instants and outputs the ephemeris data on luout
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       It searches the "real" beginning of the satpos file by comparing the
!       first characters to the target "#SATPOS"
!       It checks whether the input period of time is included in the
!       satpos period of time.
!       For each time step
!         reads the satellite position-velocity
!         tests if the satellite cross the equator 
!            if yes then computes the time of the node (ascending or descending)
!         tests if the satellite enters or leaves the station acquisition area
!           if yes then computes the start/stop time of acquisition
!
!       The header variables are stored in the common  /h_satpos/
!       if ijstart is less or equal 0 the whole file is processed
!
!       return code 0=OK  
!             1=periods do not mach
!             2=IO error
!             3=convergence error for node
!             4=convergence error for station
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
!input
        double precision ijstart! start julian instant 
        double precision ijstop ! stop julian instant
        integer luin            ! logical unit for the satpos file
        integer luout           ! logical unit for the output
!output
        integer ierr            ! return code 0=OK  
                                !             1=periods do not mach
                                !             2=IO error
                                !             3=convergence error for node
                                !             4=convergence error for acquis.
!       
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision crd
        parameter (crd=180.d0/pi)
        double precision cdr
        parameter (cdr=pi/180.d0)
        double precision short_acq      ! minimum duration of an acqu. (5 mn)
        parameter (short_acq=5.d0)
!
!
! Commons
! Satpos common
#include "c_satpos.h"
!
! Stations common
#include "c_stations.h"
!
! Functions
!
! Local variables
        integer ios                     ! I/O status variable
        integer step1                   ! first step to store
        integer step2                   ! last step to store
        integer istep                   ! loop step number
        integer fstep                   ! file step number
        integer it                      ! time in satpos interval( tenth of sec)
        double precision timris         ! julian instant of sun rise
        double precision timset         ! julian instant of sun set
        double precision e_ijstart      ! julian instant of start of ephemeris
        double precision e_ijstop       ! julian instant of stop of ephemeris
        double precision elev           ! sun elevation which defines   (rad)
!                                         sunset and sunrise
        character*8 dummyst             ! dummy string
        character*22 ch_stdate          ! date string of file start 
        character*22 ch_epoch           ! epoch time string

!
!       satpos data set of iteration i-1
        integer day1                    ! julian date
        double precision sp_pos1(3)     ! Greenwich satellite position (km)
        double precision sp_vel1(3)     ! Greenwich satellite velocity (km)
        integer sp_orbnum1              ! orbit number
        integer sp_satlight1            ! satellite in daylight or nighttime
!                                          0=daylight 1=nighttime 2=penombra
        integer sp_satvis1              ! satellite seen from the station 
!                                          0=yes  1=no
!       satpos data set of iteration i
        integer day2                    ! julian date
        double precision sp_pos2(3)     ! Greenwich satellite position (km)
        double precision sp_vel2(3)     ! Greenwich satellite velocity (km)
        integer sp_orbnum2              ! orbit number
        integer sp_satlight2            ! satellite in daylight or nighttime
!                                          0=daylight 1=nighttime 2=penombra
        integer sp_satvis2              ! satellite seen from the station 
!                                          0=yes  1=no
! 
!
        double precision time           ! time variable for nodes
        double precision timesacq       ! time at start of acquisition
        double precision timeeacq       ! time at end of acquisition
        double precision dtstat         ! time inside a SATPOS interval after
        double precision dtnode         !   .. convergence for node or station
        double precision lnode          ! longitude of node (rad)
        double precision durmn          ! acq duration in minutes
        double precision pos_acq(3)     ! Greenwich sat. position at timesacq
!                                       !               or timeeacq
        integer orbitsacq               ! orbit number at start of acquisition

        character*16 eph_mes            ! ephemeris message
        integer ierrsrs                 ! error in sun rise/set calculation
        integer ierrcn                  ! error in convergence for node
        integer ierrca                  ! error in convergence for acquisition

!-
        ierr = 0
!
!       Section 1
!       reads the satpos header and initialize variables
!       ------------------------------------------------
        dummyst='********'
        ios=0

!       finds the satpos target
        do while (ios.eq.0 .and. dummyst(1:7).ne.'#SATPOS' )
          read(luin,'(a)',iostat=ios) dummyst
        end do

        if(ios.ne.0) then
          ierr=2
          return
        endif

!       reads the header

        read(luin,'(20x,a)') sp_satnam
        read(luin,'(20x,a)') sp_grsnam
        read(luin,'(20x,a)') ch_stdate
        call xicj(ch_stdate,sp_fstart)
        read(luin,'(20x,f6.2)') sp_nbdays
        read(luin,'(20x,f6.2)') sp_dt
        read(luin,'(20x,a)') sp_bultyp
        read(luin,'(20x,i2)') sp_bulsear
        read(luin,'(20x,a)') sp_bulfilna
        
        read(luin,'(20x,a)') ch_epoch
        call xicj(ch_epoch,sp_epoch)
        read(luin,'(20x,f10.4)') sp_semaxis
        read(luin,'(20x,f10.9)') sp_eccent
        read(luin,'(20x,f10.5)') sp_inclin
        read(luin,'(20x,f10.5)') sp_argper
        read(luin,'(20x,f10.5)') sp_rigasc
        read(luin,'(20x,f10.5)') sp_meanano
        if( sp_semaxis .gt. 10000.) then
           read(luin,'(20x,3(f12.4,1x))') sp_x, sp_y, sp_z
        else
           read(luin,'(20x,3(f10.4,1x))') sp_x, sp_y, sp_z
        endif
        read(luin,'(20x,3(f10.7,1x))') sp_vx, sp_vy, sp_vz
        read(luin,'(20x,f10.4)') sp_latstat
        read(luin,'(20x,f10.4)') sp_lonstat
        read(luin,'(20x,f10.4)') sp_altstat
        read(luin,'(20x,f10.4)') sp_sitestat
        read(luin,'(a)',iostat=ios) dummyst
        sp_inclin   = sp_inclin   * cdr
        sp_argper   = sp_argper   * cdr
        sp_rigasc   = sp_rigasc   * cdr
        sp_meanano  = sp_meanano  * cdr
        sp_latstat  = sp_latstat  * cdr
        sp_lonstat  = sp_lonstat  * cdr
        sp_sitestat = sp_sitestat * cdr


!       initialize the station variables
        namestat(1) = sp_grsnam
        latstat(1)  = sp_latstat
        lonstat(1)  = sp_lonstat
        altstat(1)  = sp_altstat
        sitestat(1) = sp_sitestat
        call initrack(latstat, lonstat, altstat, posstat, matstat)
        elev=0.d0


!       test the period of time:
        if(ijstart.gt.0.) then
!         the input period must be inside the satpos file range 
          if(ijstart.lt.sp_fstart .or. 
     &       ijstop.gt.sp_fstart+sp_nbdays) then
            ierr=1
            return
          endif
          e_ijstart= ijstart
          e_ijstop = ijstop
        else
          e_ijstart= sp_fstart
          e_ijstop = sp_fstart+sp_nbdays
        endif

!       compute the first step and the number of steps to read
        if(ijstart.gt.0.) then
          step1 = idint( (ijstart-sp_fstart) * 86400.d0 / sp_dt + 1.)
          step2 = idint( (ijstop -sp_fstart) * 86400.d0 / sp_dt + 1.) +1
        else
          step1=1
          step2 = (sp_nbdays * 86400.d0) / sp_dt + 1
        endif

        if(step1.gt.1) then
          do istep=1,step1-1
            read(luin,'(a)',iostat=ios) dummyst
          end do
        endif


!       Section 2

!       initialize for the first step (index 1)
!       ---------------------------------------
        sp_orbnum1=-1
        sp_satlight1=-1
        sp_satvis1=-1
        orbitsacq=-1
        day1 = 0

        if( sp_semaxis .gt. 10000.) then
           read(luin,'(i8,3f12.3,3f10.6,i6,2i4)',iostat=ios)
     &                  fstep, sp_pos1, sp_vel1,
     &                  sp_orbnum1, sp_satlight1, sp_satvis1
        else
           read(luin,'(i8,3f10.3,3f10.6,i6,2i4)',iostat=ios)
     &                  fstep, sp_pos1, sp_vel1,
     &                  sp_orbnum1, sp_satlight1, sp_satvis1
        endif



!       Loop over the SATPOS file
!       -------------------------
!       read values with the index 2
!       ----------------------------
        do istep=step1+1,step2
          if( sp_semaxis .gt. 10000.) then
             read(luin,'(i8,3f12.3,3f10.6,i6,2i4)',iostat=ios)
     &                  fstep, sp_pos2, sp_vel2,
     &                  sp_orbnum2, sp_satlight2, sp_satvis2
          else
             read(luin,'(i8,3f10.3,3f10.6,i6,2i4)',iostat=ios)
     &                  fstep, sp_pos2, sp_vel2,
     &                  sp_orbnum2, sp_satlight2, sp_satvis2
          endif

!         checks I/O error
!         ----------------
          if(fstep.eq.istep .and. ios.eq.0) then

!         tests and calculates the sun rise/set
!         -------------------------------------
          day2 = sp_fstart + ((fstep-1-1)*sp_dt)/86400.d0

          if(day2.ne.day1) then
           
            call sunriset(day2,elev,1,latstat,lonstat,
     &                    timris,timset,ierrsrs)
            if(ierrsrs.eq.0) then
              if( timris .ge. e_ijstart .and. 
     &            timris .le. e_ijstop ) then
                it = idnint( (timris - dint(timris))
     &                                   * 86400.d0* 10.d0 )
                time = int(timris) + it/86400.d0/10.d0
                call wephmes(luout,time,sp_satnam,
     &                  sp_orbnum2,'sun_rise',sp_grsnam(1:16))
              endif
              if( timset .ge. e_ijstart .and. 
     &            timset .le. e_ijstop ) then
                it = idnint( (timset - dint(timset))
     &                                   * 86400.d0* 10.d0 )
                time = int(timset) + it/86400.d0/10.d0
                call wephmes(luout,time,sp_satnam,
     &                  sp_orbnum2,'sun_set',sp_grsnam(1:16))
              endif
            endif
          endif

!         tests and calculates the ascending node
!         ---------------------------------------
          if(sp_orbnum2.ne.sp_orbnum1) then
           
            call intnode(sp_pos1, sp_vel1, sp_pos2, sp_vel2,
     &                  sp_dt, dtnode, lnode, ierrcn)
            if(ierrcn.eq.0) then
              time = sp_fstart + 
     &           ((fstep-1-1)*sp_dt  + dtnode)/86400.d0
              write(eph_mes,'(f8.3,8x)') lnode*crd
              call wephmes(luout,time,sp_satnam,sp_orbnum2,
     &                     'asc_node',eph_mes)
            endif
          endif

!         tests and calculates the descending node
!         ----------------------------------------
          if(sp_pos1(3).gt. 0.d0 .and. 
     &       sp_pos2(3).le. 0.d0) then
            
            call intnode(sp_pos1, sp_vel1, sp_pos2, sp_vel2,
     &                  sp_dt, dtnode, lnode, ierrcn)
            if(ierrcn.eq.0) then
              time = sp_fstart + 
     &           ((fstep-1-1)*sp_dt  + dtnode)/86400.d0
              write(eph_mes,'(f8.3,8x)') lnode*crd
              call wephmes(luout,time,sp_satnam,sp_orbnum1,
     &                     'dsc_node',eph_mes)
            endif
          endif

!         tests the start/stop of acquisition
!         -----------------------------------
          if(sp_satvis2.ne.sp_satvis1) then
            call intstat(posstat, matstat, sitestat, 
     &                    sp_pos1, sp_vel1, 
     &                    sp_pos2, sp_vel2,
     &                    sp_dt, dtstat, pos_acq , ierrca)
            if(ierrca.eq.0) then
              it = dtstat * 10.d0 + 0.5d0
              dtstat = it / 10.d0
              time = sp_fstart + 
     &           ((fstep-1-1)*sp_dt  + dtstat)/86400.d0


!             determine if it is a start of acquisition and the orbit number
              if(sp_satvis1.eq.1) then
                timesacq = time
!               determine the orbit number (in case of crossing the equator)
                if(pos_acq(3)*sp_pos2(3) .gt. 0.) then
                  orbitsacq=sp_orbnum2
                else
                  orbitsacq=sp_orbnum1
                endif
              endif

!             determine if it is a end of acquisition 
!             if the orbit number of start is unknown that end of acq is ignored
!                         (the start of acq. is before ijstart)
              if(sp_satvis1.eq.0 .and. orbitsacq .gt. 0) then
                timeeacq = time
                durmn = (timeeacq-timesacq) * 1440.d0

!               writes the information on acquisition
!               START_ACQ/STOP_ACQ
!               if the duration is less than "short_acq" then
!                 the information is SHORTACQ and the duration (mn) is output

                if(durmn .gt. short_acq) then
                  call wephmes(luout,timesacq,sp_satnam,orbitsacq,
     &                     'start_acq',sp_grsnam(1:16))
                  call wephmes(luout,timeeacq,sp_satnam,orbitsacq,
     &                     'stop_acq',sp_grsnam(1:16))
                else
                  write(eph_mes,"(f5.2,' min',7x)") durmn
                  call wephmes(luout,timesacq,sp_satnam,orbitsacq,
     &                        'short_acq',eph_mes)
                  call wephmes(luout,timesacq,sp_satnam,orbitsacq,
     &                        'start_short',sp_grsnam(1:16))
                  call wephmes(luout,timeeacq,sp_satnam,orbitsacq,
     &                        'stop_short',sp_grsnam(1:16))
                endif

              endif

            endif

          endif

!         move the index 2 to 1
!         ---------------------
          sp_pos1(1) = sp_pos2(1)
          sp_pos1(2) = sp_pos2(2)
          sp_pos1(3) = sp_pos2(3)
          sp_vel1(1) = sp_vel2(1)
          sp_vel1(2) = sp_vel2(2)
          sp_vel1(3) = sp_vel2(3)
          sp_orbnum1 = sp_orbnum2
          sp_satlight1 = sp_satlight2
          sp_satvis1 = sp_satvis2
          day1 = day2
!         
!         check I/O error or convergence error
!         ------------------------------------
!
!         end of I/O error test
          else
            ierr=2
            return
         endif

          if(ierrcn.ne.0) then
            ierr=3
            return
          endif

          if(ierrca.ne.0) then
            ierr=4
            return
          endif

        end do

        return
        end
