#!/bin/ksh
#
# Get the 2 lines elements from the SPACE-TRACK web site
# for the "weather satellites"
# The script uses the command wget because anonymous ftp is forbidden
# It also use gunzip because files are provided in compressed mode.
#
# global script variables needed which can be set in the parameter 
# file ou ATOVS_ENV file depending on the environment:
# DIR_DATA_TLE root directory for stotage of 2-line elements
# PAR_NAVIGATION_TLE_TIMEOUT time out for connexion
# PAR_NAVIGATION_TLE_URL_LOGIN URL for login on web site
# PAR_NAVIGATION_TLE_URL_DOWNLOAD file to download
# PAR_NAVIGATION_TLE_USER username for web site
# PAR_NAVIGATION_TLE_PASSWD password for web site
#
# Please note that wget is able to manage the time out connexion
#
# PROPRIETAIRE/OWNER
#       P. Brunel
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     02/01/2002      none	       P. Brunel
#  1.1     25/01/2005  update for Space-Track web site  P. Brunel
#  1.2     10/02/2005  create TLE home directory if does not exists P. Brunel
#                      and cleaning of temporary files
#  1.3     01/02/2006  change config file P. Marguinaud
#  1.4     20/02/2006  remove references to archipel
#          31/01/2008  add capability to retrieve several catalogs 
#                      and connection to Celestrack   P. Brunel
#  1.5     01/02/2008  add capability to retrieve plain text from space-track
#                      e.g. favourites page. N. Atkinson
#  1.6     21/09/2012  add new space-track site with API functionality. To use it,
#                      set PAR_NAVIGATION_TLE_URL_DOWNLOAD=https://beta.space-track.org
#                      in your ATOVS_ENV7 file. Catalogue numbers are specified via
#                      new environment variable PAR_NAVIGATION_TLE_CATALOGUE. NCA
#  1.7     26/02/2013  change default URL to the new space-track site.
#  1.8     18/04/2016  Remove code related to old space-track site; update default list (remove N16/17 and
#                      add FY3C); add ability to specify mixed urls in PAR_NAVIGATION_TLE_URL_DOWNLOAD.
#                      For example, to get Metop from EUMETSAT and NOAA from Space-track, you can put the
#                      following in ATOVS_ENV7:
#PAR_NAVIGATION_TLE_URL_DOWNLOAD='https://www.space-track.org'
#PAR_NAVIGATION_TLE_URL_DOWNLOAD+=' http://oiswww.eumetsat.int/metopTLEs/html/data_out/latest_m02_long_tle.txt'
#PAR_NAVIGATION_TLE_URL_DOWNLOAD+=' http://oiswww.eumetsat.int/metopTLEs/html/data_out/latest_m01_long_tle.txt'
#PAR_NAVIGATION_TLE_CATALOGUE="25338,28654,33591,37849,32958,37214,39260,25994,27424"  #N15,18,19,S-NPP,FY-3A/3B/3C,Aqua,Terra
#export PAR_NAVIGATION_TLE_URL_DOWNLOAD PAR_NAVIGATION_TLE_CATALOGUE
#
#  1.9     05/11/2018  Carriage returns need removing from NASA TLEs. 
#
#    This software was developed within the context of
#    the EUMETSAT Satellite Application Facility on
#    Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 25 November 1998, between
#    EUMETSAT and the Met Office, UK, by one or more partners
#    within the NWP SAF. The partners in the NWP SAF are
#    the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2005, EUMETSAT, All Rights Reserved.
# 
#

. "$AAPP_PREFIX/ATOVS_CONF"

DIR_DATA_TLE=${DIR_DATA_TLE:=${DIR_NAVIGATION}/orb_elem}
if [ ! -d ${DIR_DATA_TLE} ]; then
  mkdir -p ${DIR_DATA_TLE} 
fi  
TIMEOUT=${PAR_NAVIGATION_TLE_TIMEOUT:=60}


# default value for URL of the bulletins
DEF_URL_LOGIN='https://www.space-track.org/perl/login.pl'
URL_LOGIN=${PAR_NAVIGATION_TLE_URL_LOGIN:=${DEF_URL_LOGIN}}

#DEF_URL_DOWNLOAD='https://www.space-track.org/perl/dl.pl?ID=7'
DEF_URL_DOWNLOAD='https://www.space-track.org'
URL_DOWNLOAD=${PAR_NAVIGATION_TLE_URL_DOWNLOAD:=${DEF_URL_DOWNLOAD}}

TLE_USER=${PAR_NAVIGATION_TLE_USER:=xxxxxx}
TLE_PASSWD=${PAR_NAVIGATION_TLE_PASSWD:=xxxxxx}

# default catalogue numbers for new Space-track site: NOAA-15,18,19, NPP, Metop-A/B, FY3A/3B/3C, Terra, Aqua
DEF_NORAD_CATALOGUE_NUMBERS="25338,28654,33591,37849,29499,38771,32958,37214,39260,25994,27424"
PAR_NAVIGATION_TLE_CATALOGUE=${PAR_NAVIGATION_TLE_CATALOGUE:=${DEF_NORAD_CATALOGUE_NUMBERS}}

# Wget command
WGET='wget'

# characters \ are present to prevent any trouble with SCCS
TLE_FILE=`date +tle_%Y\%m\%d_\%H\%M.txt`
TLE_MONTH=`date +%Y-\%m`
echo " TWO LINES retrieved on ${DIR_DATA_TLE}/${TLE_MONTH}/${TLE_FILE}"

if [ ! -d ${DIR_DATA_TLE}/${TLE_MONTH} ]; then
  mkdir ${DIR_DATA_TLE}/${TLE_MONTH}
fi
rm -f ${DIR_DATA_TLE}/${TLE_MONTH}/${TLE_FILE}

for URL in ${URL_DOWNLOAD}; do
  echo "URL=$URL"

  if [[ "${URL}" = *space-track* ]]; then   #new space-track site

    ${WGET} -T ${TIMEOUT} --post-data="identity=${TLE_USER}&password=${TLE_PASSWD}" --cookies=on \
      --keep-session-cookies --save-cookies=cookies_spacetrack "${URL}/ajaxauth/login" -olog

    ${WGET} -T ${TIMEOUT} --keep-session-cookies --load-cookies=cookies_spacetrack -O weather.txt \
      "${URL}/basicspacedata/query/class/tle_latest/ORDINAL/1/NORAD_CAT_ID/${PAR_NAVIGATION_TLE_CATALOGUE}/format/tle"

    STATUS=$?
    if [ ${STATUS} != 0 ]; then
      echo "Unable to retrieve TwoLine from ${URL}"
      \rm -f cookies_spacetrack
      exit 1
    fi

    cat weather.txt | tr -d '\r' >>${DIR_DATA_TLE}/${TLE_MONTH}/${TLE_FILE}   #remove extra carriage returns
    \rm -f weather.txt

  else        #Not space-track (e.g. Celestrak or EUMETSAT). Assume page is already formatted correctly.

  # Get the file
    ${WGET}  -T ${TIMEOUT} ${URL} -O weather.txt
    STATUS=$?

  # rename file according to date and time of connection
  #-----------------------------------------------------

    if [ ${STATUS} -eq 0 ] ; then 
      cat weather.txt | tr -d '\r' >> ${DIR_DATA_TLE}/${TLE_MONTH}/${TLE_FILE}
      \rm -f weather.txt
    else
      echo "Unable to retrieve TwoLine from ${URL}"
      exit 1
    fi
  fi
done

exit 0
