C LIBS = ephtrack navtool tbus satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: lgephe.F 47 2006-08-21 08:44:41Z frna $
!
        program lgephe
!
!BUT/PURPOSE 
!       LGEPHE creates the ephemeris file for a long period (ex: a month),
!       one satellite and a list of stations.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       reads the input parameters (list of stations...)
!       uses the historical ephemeris utilities file of the satellite
!       calculates the ephemeris parameters (nodes and
!         acquisition times) assuming:
!               - circular orbit
!               - linear variation of the nodal period
!               - linear variation of the longitude increment
!
!       Inputs are:
!       standard input
!               satellite name, 
!               start date  (or date/hour)
!               number of days
!               filename of the long term ephem. 
!               list of station names
!
!       Output
!       standard output
!               ephemeris file
!       
! Remarks:
!       The run is done for only one satellite due to the historical file
!       The ephemeris file has to be sorted to be strictly chronological
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   March 03:   Cut lines exceeding 72 characters.  T.Labrot
!     
!EXIT CODES    
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       genscid
!       cp_lsc          ! last significant character (f7cp)
!       cp_ssf          ! subsitute the white spaces in a character variable
!       xdcj            ! converts a date string into a julian date (f7tp)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!DECLARATIONS
!
! Local parameters:
        integer lgeunit                 ! logical unit for the bulletin index
        parameter (lgeunit=10)
!
! variables in standard input
        character*12 scname             ! NOAA satellite name
        character*80 fstatname          ! Ground station name
        character*22 ch_stdate          ! start date dd/mm/yy  hh:mm:ss.sss
        character*256 lgephe_file       ! filename of the bulletin index
        real nbdays                     ! number of days (real value)
!
! Commons
!       variables for long term ephemeris generation
#include "c_lge.h"
!
!       array of ground station coordinates
#include "c_stations.h"
!
!
! Functions
        integer genscid                 ! order number for a satellite name
        integer cp_lsc                  ! last significant character (f7cp)
!
! 
! Local variables:
        integer ios                     ! I/O status variable
        integer iergst                  ! return code of gstatc
        integer iergel                  ! return code of lge_gelem
        integer ordernum                ! order number of a satellite
        integer len_date                ! length of the input date string
        integer luout                   ! output logical unit
        double precision ijstdate       ! julian start instant
        double precision ijendate       ! julian end instant
        character*22 refdate0h          ! string reference date at 0h
        integer nbs                     ! number of ground stations
        character*80 statname(maxstat)  ! Ground station names
!
        data refdate0h/'01/01/50  00:00:00.000'/
!
!
!
! 
!-
!
        luout = 6
        call ml_wt('info','start')
!
!       Section 1
!       Reads the input parameters
!       --------------------------
!
!       Get the satellite name
        read(*,'(a)',iostat=ios) scname
        if(ios.eq.0) then
          ordernum= genscid(scname)
          if(ordernum.eq.0) then
            call ml_wt('error','satellite unknown in '//
     &           'the satellite identification file '//scname)
            call ml_wt('error','aborted')
            call aappexit(1)
          endif
        else
          call ml_wti('error',
     &          'while reading the satellite name I/O',ios)
          call ml_wt('error','aborted')
          call aappexit(1)
        endif


!       Get the start date
!       and converse to a julian instant to search for a tbus
        read(*,'(a)',iostat=ios) ch_stdate
        if(ios.ne.0) then
          call ml_wti('error','while reading the start date I/O',ios)
          call ml_wt('error','aborted')
          call aappexit(1)
        endif
!       complete the date for the 0 hour if no hour is specified
        len_date = cp_lsc(ch_stdate)
        if(len_date.lt.22) then
          ch_stdate(len_date+1:22) = refdate0h(len_date+1:22)
        endif
        call xicj(ch_stdate,ijstdate)

!       Get the number of days
        read(*,*,iostat=ios) nbdays
        if(ios.ne.0) then
          call ml_wti('error','while reading the number of days I/O',
     &                 ios)
          call ml_wt('error','aborted')
          call aappexit(1)
        endif
        ijendate = ijstdate + nbdays


!       Get the historical ephemeris utilities file name
        read(*,'(a)',iostat=ios) lgephe_file
        if(ios.ne.0) then
          call ml_wti('error','while reading the historical '//
     &                       'ephemeris utilities  file I/O',ios)
          call ml_wt('error','aborted')
          call aappexit(1)
        endif

!       Get the station names
        nbs = 0
        read(*,'(a)',iostat=ios) fstatname
        do while (ios.eq.0 .and. nbs.lt.maxstat) 
          nbs = nbs + 1
          statname(nbs) = fstatname
          read(*,'(a)',iostat=ios) fstatname
        end do
!
!
!       Section 2
!       Finds opens and reads the historical ephemeris utilities file
!       -------------------------------------------------------------
        open (unit=lgeunit,file=lgephe_file,status='old',iostat=ios)
        if( ios.ne.0 ) then
          call ml_wti('error',
     &              'while openning the long term ephe. file I/O',ios)
          call ml_wt('error','file '//lgephe_file)
          call ml_wt('error','aborted')
          call aappexit(1)
        endif

        call lge_gelem(lgeunit,ijstdate,ijendate,iergel)
        if(iergel.ne.0) then
          call ml_wti('error','lge_gelem return code ',iergel)
          call ml_wt('error','aborted')
          call aappexit(1)
        endif

        close(unit=lgeunit,iostat=ios)
!
!
!       Section 3
!       Initialize the station coordinates
!       ----------------------------------
        if(nbs.gt.0) then
          call gstatc(nbs,statname,iergst)
          if(iergst.eq.0) then
            call ml_wti('error',
     &        'in finding the station coordinates I/O',iergst)
            call ml_wt('error','aborted')
            call aappexit(1)
          elseif(iergst.ne.nbs) then
            call ml_wt('warning','when looking for the '//
     &                           'station coordinates')
            call ml_wt('warning','some stations were not found')
            call ml_wti('warning','wanted stations  ',nbs)
            call ml_wti('warning','found stations ',iergst)
          endif
        endif
!       
!
!       Section 4
!       Computes and outputs the ephemeris messages
!       -------------------------------------------

        call lge_ephe(luout,scname,ijstdate,ijendate)

        call ml_wt('info','normal end')

        stop

        end
