C LIBS = ephtrack navtool tbus satid f7ml f7tp f7cp f7gp auxaapp
        program lgepheing
!+
!NOM/NAME $Id: lgepheing.F 47 2006-08-21 08:44:41Z frna $
!
!BUT/PURPOSE 
!       Ingest a tbus bulletin for long term ephemeris calculations.
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions
!          
!       An historical ephemeris utilities file is available for each satellite
!       It contains the following informations
!               time of the ascending node
!               orbit number
!               longitude of the ascending node
!               longitude increment
!               semi major axis
!               inclination
!               nodal period
!
!       lgepheing updates these files for a list of satellites given as input
!
!       Algorithm
!       ---------
!       1 reads the inputs (character strings)
!         -home directory of the tbus files
!         -tbus file name (following the directory)
!         for each satellite (until end of file or sat='end'):
!               -satellite name
!               -historical ephemeris utilities file name for that satellite
!
!       2 Creates the tbus filename and opens the file
!
!       3 For each satellite
!         3.1 Decodes the tbus part IV orbital elements
!         3.2 Writes the usefull parameters in the ephe. utilities file
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date         comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   March 03  Cut lines exceeding 72 characters.    T.Labrot         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
! functions
!       genscid
!       genscnam
!       cp_lsc          ! last significant character (f7cp)
!       cp_ssf  subsitute the white spaces in a character variable
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!EXIT CODE
!       0 = normal execution 
!
!DECLARATIONS
!
!
!
!
! Parameters
        integer nbsatmax                ! maximum number of satellites 
        parameter (nbsatmax=5)
        integer tbunit                  ! logical unit for the tbus file
        parameter (tbunit=10)
        integer lgeunit                 ! logical unit for the long term ephem..
        parameter (lgeunit=11)
        logical debug
        parameter (debug=.false.)
!
! Functions
        integer genscid                 ! order number for a satellite name
        character*16 genscnam           ! NOAA satellite name
        integer cp_lsc                  ! last significant character (f7cp)
!
! Inputs
        character*256 dirtbus           ! home directory for the tbus files
        character*256 tbus_file         ! filename of the tbus without the
!                                       !   home directory
        character*16 scname(nbsatmax)   ! satellite name list
        character*256 lgephe_file(nbsatmax)     ! filename of the long term eph.
!
! Local variables:
        integer ios                     ! I/O status variable
        integer isat                    ! current satellite number in the list
        integer nbsat                   ! number of satellites to process
        integer prtbus                  ! if no zero tbus is printed on stdout
        integer ordernum                ! order number of a satellite
        integer dcerr                   ! return code of tb_dc
        character*16 noaanam            ! NOAA name of the current satellite
        character*16 lsatname           ! temporary satellite name
        character*256 name              ! var. name for the files
        character*256 lgephe            ! temporary filename of the tbus index
        character*256 full_tbus_file    ! full filename of the tbus
! 
!-
        call ml_wt('info','start')
!
!       Section 1
!       Reads the input parameters
!       --------------------------
!       Get the home directory of the TBUS file
        read(*,'(a)',iostat=ios) dirtbus
        if(ios.ne.0) then
          call ml_wti('error','Input error   iostat =',ios)
          call ml_wt('error','for TBUS home directory ')
          call ml_wt('error','aborted ')
          call aappexit(2)
        endif
!       Get the short name of the TBUS file (which will be stored in the
!       historical ephemeris utilities file)
        read(*,'(a)',iostat=ios) tbus_file
        if(ios.ne.0) then
          call ml_wti('error','Input error   iostat =',ios)
          call ml_wt('error','for TBUS filename ')
          call ml_wt('error','aborted ')
          call aappexit(2)
        endif
!
!       Get the satellite list, the historical ephemeris utilities filename
        nbsat=0
        ios = 0
        do while (ios .eq. 0 .and. nbsat.lt.nbsatmax .and. 
     &            lsatname.ne.'end')
          read(*,'(a)',iostat=ios) lsatname
          read(*,'(a)',iostat=ios) lgephe
          if(ios.eq.0) then
            ordernum= genscid(lsatname)
            if(ordernum.ne.0) then
              nbsat = nbsat+1
              scname(nbsat) = lsatname
              lgephe_file(nbsat)  = lgephe
            else
              call ml_wt('notice','Input satellite '//lsatname)
              call ml_wt('notice',
     &                 'unknown in the satellite identification file')
            endif
          endif
        end do

!       
!       Section 2
!       Initialize
!       ----------
!
!       create the tbus filename (adding the dirtbus and 
!       suppressing the white spaces)
        name =  dirtbus(1:cp_lsc(dirtbus)) // '/' // tbus_file
        call cp_ssf(name,full_tbus_file,' ')
!
!       opens the tbus file
        open (unit=tbunit,file=full_tbus_file,status='old',iostat=ios)
        if( ios.ne.0 ) then
          call ml_wti('error','open error   iostat =',ios)
          call ml_wt('error','for the tbus file '//full_tbus_file)
          call ml_wt('error','aborted ')
          call aappexit(2)
        else
          call ml_wt('info','Ingests tbus '//full_tbus_file)
        endif

!
!       Section 3
!       Process a satellite
!       -------------------
        do isat=1, nbsat
          call ml_wt('info','Satellite '//scname(isat))
!
!         Section 3.1
!         Opens the satellite historical ephemeris utilities file
!         -------------------------------------------------------
          noaanam  = genscnam(scname(isat))
!
          open (unit=lgeunit,file=lgephe_file(isat),status='old',
     &          iostat=ios)
          if( ios.ne.0 ) then
!           create the file 
            call ml_wti('notice','Open old error   iostat =',ios)
            call ml_wt('notice','for long term ephemeris file')
            call ml_wt('notice','try to create ')
            open (unit=lgeunit,file=lgephe_file(isat),status='new',
     &            iostat=ios)
            if( ios.ne.0 ) then
              call ml_wti('error','Open new error   iostat =',ios)
              call ml_wt('error','for long term ephemeris file')
              call ml_wt('error','aborted ')
              call aappexit(2)
            else
              call ml_wt('notice',
     &                   'Creation of long term ephemeris file OK')
              write(lgeunit,"('# ',a16)") noaanam
              write(lgeunit,"('%        epoch_time      ',
     &        '               orbit',
     &        '   lan      loninc   semMaxis    inclin    nodal per')")
              rewind lgeunit
            endif
          endif
!
!         Section 3.2
!         Reads the Part IV and checks
!         ----------------------------
          if(debug) then
            prtbus = 1
          else
            prtbus = 0
          endif

          call tb_dc(noaanam,tbunit,prtbus,dcerr)
          if(dcerr.eq.0) then
        
!           Writes a record in the historical ephemeris utilities file
            call lge_wind(lgeunit)

          elseif(dcerr.eq.1) then

!           The tb_dc did not found a Part IV for the satellite
            call ml_wt('notice','NO Part IV for that satellite')

          elseif(dcerr.eq.2) then

!           The tb_dc found an I/O error
            call ml_wt('error','I/O error while decoding the Part IV')

          elseif(dcerr.eq.3) then

!           The tb_dc found a fatal error in checking
            call ml_wt('error','unable to process Part IV data')
            call ml_wt('error','aborted')
            call aappexit(1)

          endif

          close (lgeunit)

!
!       end of the current satellite ingest
        end do

        close (tbunit)

        call ml_wt('info','normal end')

        stop
        end
        subroutine lge_wind(lgeunit)
!+
!NOM/NAME $Id: lgepheing.F 47 2006-08-21 08:44:41Z frna $
!
!BUT/PURPOSE 
!           Writes a record in the historical ephemeris utilities file
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       cp_lsc          ! last significant character (f7cp)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
        integer lgeunit                 ! logical unit for the long term ephem..
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        double precision pi     ! PI constant  
        double precision twopi
        double precision crd    ! conversion factor radians to degrees 
        real dstest             ! threshold for the minimum distance test (km)

        parameter ( pi = 3.14159265358979311600d0)
        parameter ( twopi = 2.d0 * pi)
        parameter ( crd=180.0d0/pi)
        parameter ( dstest=1. )
!
! Commons
! common TBUS Part IV
#include "c_tbus4.h"
!
!
! Functions
!
! Local variables:
        integer ios                     ! I/O status variable
        double precision lan            ! longitude of the ascending node (rd)
        double precision loninc         ! longitude increment (rd/orbit)
        double precision dist
        logical launch
        character*10 dummyst            ! dummy string
        character*22 ice                ! string var. for epoch time
        character*22 icn                ! string var. for nodal period
! 
        double precision positod(3)     ! position in ITOD reference frame
        double precision velitod(3)     ! velocity in ITOD reference frame
        double precision posgrw(3)      ! position in Greenwich reference frame 
        double precision velgrw(3)      ! velocity in Greenwich reference frame
        double precision lat            ! geographic latitude           (rad)
        double precision lon            ! longitude                     (rad)
        double precision alt            ! altitude                      (km)
        integer ierr                    ! return code of cartgeo
!                                         0 = ok  
!                                         1 = iterations have not converged
        double precision dlon           ! difference of longitude of
!                                       ! the ascending node (rad)
!-
!
!       Identifies two cases bulletins which are not processed
!       - launch bulletin (position equal 0)
!       - bulletin is not given for an ascending node
!
!
        dist = dsqrt( tbus_x*tbus_x + tbus_y*tbus_y + tbus_z*tbus_z )
        launch = dist.le.dstest

        if(launch) then
          call ml_wt('notice','launch bulletin... cannot be used')
          return
        elseif( dabs(tbus_z) .gt. 1.d0 ) then
          call ml_wt('notice','bulletin not given for an ascending '//
     &           'node... cannot be used')
          return
        else

!         normal case, a record is writen

!         go to the end of the file
          read(lgeunit,'(a10)',iostat=ios) dummyst
          do while (ios.eq.0) 
             read(lgeunit,'(a10)',iostat=ios) dummyst
          end do

!         longitude of ascending node
          lan = tbus_rigasc - tbus_gha
          if( lan .gt. pi ) then
            lan = lan - twopi
          elseif( lan .lt. -pi) then
            lan = lan + twopi
          endif

!         Now compute the lan using the xyz poistion in True of date
!         and difference of the 2 calculated longitudes
          dlon=0.d0
          call ml_wt('info','new code')
          if(dabs(tbus_z).lt.1.d0 .and. tbus_vz.gt.0.d0) then
             positod(1) = tbus_x
             positod(2) = tbus_y
             positod(3) = tbus_z
             velitod(1) = tbus_vx
             velitod(2) = tbus_vy
             velitod(3) = tbus_vz
             call pvitodgrw(tbus_epoch,positod,velitod,posgrw,velgrw)
             call cartgeog(posgrw,lat,lon,alt,ierr)
             dlon = lon - lan
             if( dlon .gt. twopi) then
                dlon=dlon-twopi
             elseif( dlon .lt. -twopi) then
                dlon=dlon+twopi
             endif
          endif

!         longitude of ascending node increment
          loninc = ( -360.98565d0 * pi/180.d0 + tbus_rascmot ) *
     &                  tbus_nodper

!         conversion of the time variables in a readable string
          call xijc(tbus_epoch,ice)
          call xijc(tbus_nodper,icn)

!         the output units are degrees, km only if the 2 longitudes
!         are the same 
          if(dabs(dlon) .lt. 0.05d0) then
             write(lgeunit,
     &       '(f15.9,1x,a22,1x,i6,1x,f8.3,1x,f10.6,1x,
     &                               f9.4,1x,f9.6,1x,a12)',
     &                                            iostat=ios)
     &         tbus_epoch,ice, tbus_orbnum, lan*crd, loninc*crd, 
     &         tbus_semaxis, tbus_inclin*crd, icn(11:22)
         
             if(ios.ne.0) then
                call ml_wti('error','while writing to the end of '//
     &           'historical ephemeris utilities file I/O',ios)
             endif
          else
             call ml_wt('error','computed longitude of '//
     &           'ascending node is false, see TBUS')
          endif


        endif

        return

        end
