#! /bin/ksh
#
# Korn Shell for sateph: run the NOAA ephemeris scheme (short term)
# ( light alleph version without no tracking angle nor satellites mixing )
#
# PROPRIETAIRE/OWNER
#       P. Brunel - A. Marsouin
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     13/11/97   initial            P. Brunel/G.Legendre
#  2.0     July 2003  add 2lines, new satid file, better parametrisation
#                     for bulletin/satellites management  
#                     add offset in number of days for -d argument P. Brunel
#  2.1     15/06/05   remove specific MF-CMS context (ArchiPEL level1)
#                     P. Brunel
#  2.2     01/02/06   change config file P. Marguinaud
#  2.3     20/02/06   remove reference to archipel P. Marguinaud
#  2.4     07/06/06   remove initialisation of DIR_STATIONS P. Marguinaud
#
# COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
#     &/'%Z% %M%  version %I% on %G%'/       
#
function which_oper_bul {
  # Finds in the navigation parametrisation the orbital
  # bulletin which corresponds to the satellite given in argument 1
  # involved parameters are:
  # PAR_NAVIGATION_DEFAULT_LISTEBUL and PAR_NAVIGATION_DEFAULT_LISTESAT
  
  # if argument 2 and 3 are present then they are assumed as 
  # a default bulletin list (arg2) and a default satellite list (arg3)

  if [ $# = 1 ] ; then
    DEFAULT_LISTEBUL=${PAR_NAVIGATION_DEFAULT_LISTEBUL}
    DEFAULT_LISTESAT=${PAR_NAVIGATION_DEFAULT_LISTESAT}
  elif [ $# = 3 ] ; then
    DEFAULT_LISTEBUL=$2
    DEFAULT_LISTESAT=$3
  else
    echo $*
    echo "function which_oper_bul needs one or three arguments"
    echo " which_oper_bul sat [list_of_bulletins] [ list_of_satellites]"
    exit 1
  fi

  # array index
  typeset -i i=0
  
  # if no bulletin found then gives the default value to TBUS bulletins
  BUL=tbus

  SATIMG=$1
  set -A ABUL ${DEFAULT_LISTEBUL}
  for SBUL in  ${DEFAULT_LISTESAT}
  do
    if [ "${SBUL}" = "${SATIMG}" ] ; then
      BUL=${ABUL[i]}
      break
    fi
    i=`expr ${i} + 1`
  done
}


# AAPP environment
. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh
log_init $0

#-----------------------------------------------------

 
# 0 initialize the variables
#---------------------------
# SATellite IDentification
# Use operational satid file
SATID_FILE=${SATID_FILE:=satid.txt}
export DIR_SATID SATID_FILE

# variables for the ARGOS
DIR_DATA_ARGOS=${DIR_NAVIGATION}/argos_db
export DIR_DATA_ARGOS

# variables for the TBUS
DIR_DATA_TBUS=${DIR_NAVIGATION}/tbus_db
export DIR_DATA_TBUS

# variables for the 2 LINES
DIR_DATA_TLE=${DIR_NAVIGATION}/tle_db
export DIR_DATA_TLE

# variables for the spm
DIR_DATA_SPM=${DIR_NAVIGATION}/spm_db
export DIR_DATA_SPM


# repertoire SATPOS
DIR_SATPOS=${DIR_NAVIGATION}/satpos
if [ ! -d ${DIR_SATPOS} ]; then
  mkdir ${DIR_SATPOS}
fi

# repertoire EPHE 
DIR_EPHE=${DIR_NAVIGATION}/ephe
if [ ! -d ${DIR_EPHE} ]; then
  mkdir ${DIR_EPHE}
fi

# 1 usage
#-------- 
usage(){
	echo
	echo ' SATEPH to run the ephemeris scheme (short term)'
	echo
	echo 'usage:  sateph  [-options]';
	echo '	options:';
	echo '  -s  satellite_list';
	echo "		default:${PAR_NAVIGATION_DEFAULT_LISTESAT}";
	echo '  -S  station_name		default:lannion';
	echo '  -b  bulletin_list';
	echo "		default:${PAR_NAVIGATION_DEFAULT_LISTEBUL}";
	echo '  -d  start_date			default:  today';
	echo '  -n  number of days (real)	default:    1.0';
	echo '  -i  increment in seconds (real)	default:  120.0';
	echo '  -c  search criteria (n/p)	default:nearest';
	echo '                      (n for nearest or p for preceeding)';
	echo
	echo ' the date format could be a date or a date/hour string:';
	echo ' or an offset in days to the current day';
	echo " for example -d 'dd/mm/yy  hh:mm:ss.sss' (2 spaces between yy hh)";
	echo "     or      -d 'dd/mm/yy  hh:mm'";
	echo "     or      -d dd/mm/yy";
	echo "     or      -d '-3'   3 days ago";
	echo "     or      -d '4'    4 days after";
	echo
	echo 'files are stored under ${DIR_NAVIGATION}:';
	echo '     ./satpos  ./ephe  ./tracking ';
	echo
	echo 'environment variables:'
	echo '	DIR_SATID    path for the satellite identification file'
	echo '	DIR_STATIONS  path for the stations file'
        echo '  DIR_NAVIGATION  generic path for the argos dir argos_db'
	}

# 2 get the arguments
#--------------------
          while getopts s:S:b:d:n:i:c: c
          do
               case $c in
               s)	LISTESAT=$OPTARG;;
               S)	STATION=$OPTARG;;
               b)	LISTEBUL=$OPTARG;;
               d)	DATE=$OPTARG;;
               n)	NBD=$OPTARG;;
               i)	INC=$OPTARG;;
               c)	CRI=$OPTARG;;
               \?)      usage
                        exit 2;;
               esac
          done

# 2.1 default values
#-------------------
STATION=${STATION='lannion'}

DATE=${DATE=`date +%d/%m/%y`}
NBD=${NBD='1.00'}
INC=${INC='120.00'}
CRI=${CRI='n'}

if [ "${CRI}" = 'n' ] ; then
  ICRI=0
else
  ICRI=1
fi

log_info "DATE : ${DATE}"

#calculate length of date inout variable
LG_DATE=`echo ${DATE} | awk '{print length($0)}'`

#test wether it is a date or an offset on current day
if [ ${LG_DATE} -ge 8 ] ; then
  # the argument is a date dd/mm/yy
  set `echo "${DATE}" | tr '/' ' '`
  if [ $3 -gt 50 ] ; then
    DATEY=19$3$2$1
  else
    DATEY=20$3$2$1
  fi
else
  # use date_tool functions for conversion of dates
  . date_tool
  # the argument is an offset on current day
  ND=${DATE}
  TODAY=`date +%Y\%m\%d`
  # add ND days and return DATEY in yyyymmdd format
  DATEY=`date_pn ${TODAY} ${ND}`
  # convert yyyymmdd to dd/mm/yy format
  DATE=`date_evol_to_bbt ${DATEY}`
fi


if [ -z "${LISTEBUL}" ] ; then
  # Listbul argument not set
  # take default values from parametrisation
  LISTEBUL=${PAR_NAVIGATION_DEFAULT_LISTEBUL}
  ARG_LISTEBUL='no'
else
  ARG_LISTEBUL='yes'
fi
if [ -z "${LISTESAT}" ] ; then
  # ListSat argument not set
  # take default values from parametrisation
  LISTESAT=${PAR_NAVIGATION_DEFAULT_LISTESAT}
  ARG_LISTESAT='no'
else
  ARG_LISTESAT='yes'
fi

# 3 loop on the satellites
#-------------------------
for SAT in ${LISTESAT}
do

log_line
typeset -l BUL
if [ "${ARG_LISTEBUL}"  = 'no' ] &&  [ "${ARG_LISTESAT}"  = 'no' ]; then
  which_oper_bul ${SAT}
elif [ "${ARG_LISTEBUL}"  = 'no' ] ; then
  # take bulletin name from the parametrisation lists
  which_oper_bul ${SAT}
else
  which_oper_bul ${SAT} "${LISTEBUL}" "${LISTESAT}"
fi

SATPOSF="${DIR_SATPOS}/satpos_${SAT}_${DATEY}.txt"

log_info  'satellite   :'"${SAT}"
log_info  'satpos file :'"${SATPOSF}"
log_info  'bulletin    :'"${BUL}"

TBUS_INDEX=${DIR_DATA_TBUS}/tbus_${SAT}.index
ARGOS_INDEX=${DIR_DATA_ARGOS}/argos_${SAT}.index
TLE_INDEX=${DIR_DATA_TLE}/tle_${SAT}.index
SPM_INDEX=${DIR_DATA_SPM}/spm_${SAT}.index

if [ "${BUL}" = 'tbus' ] ; then
  BUL_PATH=${DIR_DATA_TBUS}
  BUL_INDEX=${TBUS_INDEX}
  SATPOS_CMD='satpost'
elif [ "${BUL}" = 'tle' ] ; then
  BUL_PATH=${DIR_DATA_TLE}
  BUL_INDEX=${TLE_INDEX}
  SATPOS_CMD='satpostle'
elif [ "${BUL}" = 'spm' ] ; then
  BUL_PATH=${DIR_DATA_SPM}
  BUL_INDEX=${SPM_INDEX}
  SATPOS_CMD='satposspm'
else
  BUL_PATH=${DIR_DATA_ARGOS}
  BUL_INDEX=${ARGOS_INDEX}
  SATPOS_CMD='satposa'
fi


# 3.1 creates the satpos file
#----------------------------

${SATPOS_CMD} -s $SAT -S $STATION -d $DATE -n $NBD -i $INC -c $CRI > ${SATPOSF}

crd=$?

if [ "${crd}" -ne 0 ] ; then
  rm -f ${SATPOSF}
fi

# 3.2 ephemeris
#--------------
log_info  'ephe file   :'"${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt"
if [ "${crd}" -eq 0 ] ; then
  ephe < ${SATPOSF} > ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt
  crd=$?
else
  log_error  pb satpos ${SAT} :ephe skipped
fi

if [ "${crd}" -eq 0 ] ; then
  sort ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt -o ${DIR_EPHE}/ephe_${SAT}_${DATEY}.txt
fi

done

exit
