C LIBS =  tbus brolyd satid navtool f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: satpost.F 47 2006-08-21 08:44:41Z frna $
!
        program satpost
!
!BUT/PURPOSE 
!       Creates a satellite position-velocity file for several days 
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       Process ONE satellite
!
!       Read and check the input parameters on standard input
!               satellite name, 
!               station name
!               start date : date (8char) or date_hour (12to22 char)
!               number of days (real value)
!               time step (seconds)
!               home directory for the tbus files
!               index file name for that satellite
!               bulletin search criteria
!       
!       Finds, opens and reads the TBUS file corresponding to the search
!               criteria
!
!       Initialize the station coordinates
!
!       Writes the header on standard output
!
!       Calculates the position for all the steps using the BROLYD 
!               extrapolation model
!
!       The standard output is supposed to be connected to a satpos file
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   19 Mai 1998    djmax is now 5.0 days due to problems in launch period
!                               P. Brunel 
!     
!EXIT CODES    
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       genscid
!       genscnam
!       cp_lsc          ! last significant character (f7cp)
!       cp_ssf          ! subsitute the white spaces in a character variable
!       xdcj            ! converts a date string into a julian date
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!DECLARATIONS
!
! Local parameters:
        integer bulunit         ! logical unit for the bulletin file
        parameter (bulunit=10)
        integer bulindu         ! logical unit for the bulletin index file
        parameter (bulindu=11)
        real dtmax              ! maximum range for search around the epoch
!                               !   time in number of days
        parameter(dtmax=6.0)
!
        real djmax              ! maximum range for processing the satpos file
!                               !   time in number of days ==> ERROR
        parameter(djmax=5.0)
!
        real djwar              ! maximum range for processing the satpos file
!                               !   time in number of days WARNING emmited
        parameter(djwar=2.0)
!
! variables in standard input
        character*16 scname     ! NOAA satellite name
        character*80 statname   ! Ground station name
        character*22 ch_stdate  ! start date dd/mm/yy  hh:mm:ss.sss
        character*256 dirbul    ! home directory of the tbus files
        character*256 bulind_file! filename of the bulletin index file
        real nbdays             ! number of days (real value)
        real timestep           ! time step in seconds
        integer bulsear         ! bulletin search criteria (0 is normal mode)
!                                   0 = nearest to the start date and valid
!                                   1 = nearest before the start date and valid
!
! Commons
! common TBUS Part IV
#include "c_tbus4.h"
!
! Functions
        integer genscid         ! order number for a satellite name (satid)
        character*16 genscnam   ! satellite name
        integer cp_lsc          ! last significant character (f7cp)
!
! 
! Local variables:
        integer ios                     ! I/O status variable
        integer prtbul                  ! if no zero bul. is printed on stdout
        integer ierdc                   ! return code of tb_dc
        integer ierg                    ! return code of tb_glpv or tb_gnv
        integer iergst                  ! return code of gstatc
        integer ordernum                ! satid order number of a satellite
        integer len_date                ! length of the input date string
        real dj                         ! diff tbus_epoch - ijstdate (days)
        double precision ijstdate       ! julian start instant
        character*16  satname           ! satellite name
        character*256 name              ! string variable for the filenames
        character*256 bulfilna          ! full filename of the tbus file
        character*22 refdate0h          ! string reference date at 0h
        data refdate0h/'01/01/50  00:00:00.000'/
!
!
!
! 
!-
!
        prtbul=1
        call ml_wt('info','SATPOST start')

!       Section 1
!       Reads the input parameters
!       --------------------------
!
!       Get the satellite name
        read(*,'(a)',iostat=ios) scname
        if(ios.eq.0) then
          ordernum= genscid(scname)
          if(ordernum.eq.0) then
            call ml_wt('error','satellite unknown in the satellite'//
     &         'identification file '//scname)
            call ml_wt('error','SATPOST aborted')
            call aappexit(1)
          endif
        else
          call ml_wti('error',
     &      'while reading the satellite name I/O status',ios)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif
        call ml_wt('info','SATPOST for satellite '//scname)

!       Get the station name
        read(*,'(a)',iostat=ios) statname
        if(ios.ne.0) then
          call ml_wti('error',
     &       'while reading the station name I/O status',ios)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif

!       Get the start date
        read(*,'(a)',iostat=ios) ch_stdate
        if(ios.ne.0) then
          call ml_wti('error',
     &       'while reading the start date I/O status',ios)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif
!
!       make a full date string (22 char.), completed with 0 fields
        len_date = cp_lsc(ch_stdate)
        if(len_date.lt.22) then
          ch_stdate(len_date+1:22) = refdate0h(len_date+1:22)
        endif
!       Converse to a julian instant
        call xicj(ch_stdate,ijstdate)

!       Get the number of days
        read(*,*,iostat=ios) nbdays
        if(ios.ne.0) then
          call ml_wti('error',
     &       'while reading the number of days I/O status',ios)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif

!       Get the time step  (seconds)
        read(*,*,iostat=ios) timestep
        if(ios.ne.0) then
          call ml_wti('error',
     &       'while reading the time step I/O status',ios)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif

!       Get the ref directory of the orbital elements TBUS file
        read(*,'(a)',iostat=ios) dirbul
        if(ios.ne.0) then
          call ml_wti('error',
     &       'while reading the tbus home directory I/O status',ios)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif

!       Get the tbus index file
        read(*,'(a)',iostat=ios) bulind_file
        if(ios.ne.0) then
          call ml_wti('error',
     &       'while reading the index filename I/O status',ios)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif

!       Get the bulletin search criteria
        read(*,*,iostat=ios) bulsear
        if(ios.ne.0) then
          call ml_wti('error',
     &    'while reading the bulletin search criteria I/O status',ios)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif
!
!
!       Section 2
!       Finds opens and reads the TBUS file
!       -----------------------------------
        open (unit=bulindu,file=bulind_file,status='old',iostat=ios)
        if( ios.ne.0 ) then
          call ml_wti('error',
     &       'while openning the tbus index file I/O status',ios)
          call ml_wt('error','filename '//bulind_file)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif


        if(bulsear.eq.0) then
          call tb_gnv(bulindu,ijstdate,dtmax,bulfilna,ierg)
        elseif(bulsear.eq.1) then
          call tb_glpv(bulindu,ijstdate,bulfilna,ierg)
        else
          call ml_wti('error',
     &    'Internal error bulsear different from 0 or 1 :',bulsear)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif

        close(unit=bulindu,iostat=ios)

        if( ierg .eq. 0 ) then
!         create the tbus filename (adding the path and 
!         suppressing the white spaces)
          name = 
     &       dirbul(1:cp_lsc(dirbul)) // '/' //
     &       bulfilna(1:cp_lsc(bulfilna))
          call cp_ssf(name,bulfilna,' ')
!         opens the tbus file
          open (unit=bulunit,file=bulfilna,status='old',iostat=ios)
          if( ios.ne.0 ) then
            call ml_wti('error',
     &       'while openning the tbus file I/O status',ios)
            call ml_wt('error','filename '//bulfilna)
            call ml_wt('error','SATPOST aborted')
            call aappexit(1)
          else
            call ml_wt('info','SATPOST using tbus file '//bulfilna)
            call tb_dc(scname,bulunit,prtbul,ierdc)
            if(ierdc.ne.0) then
              call ml_wti('error',
     &         'while decoding the tbus file status',ierdc)
              call ml_wt('error','SATPOST aborted')
              call aappexit(1)
            endif
          endif
          close (unit=bulunit,iostat=ios)
        else
          call ml_wt('error',
     &      'NO tbus file available for the date'//ch_stdate)
          call ml_wti('error','return code of tb_gnv/tb_glpv :',ierg)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif
!
        dj = sngl(tbus_epoch - ijstdate)
        if(abs(dj). gt. djmax) then
          call ml_wt('error',
     &      'Start date too far from the TBUS epoch time')
          call ml_wtr('error','number of days :',abs(dj))
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif

        if(abs(dj). gt. djwar) then
          call ml_wt('warning',
     &      'Start date too far from the TBUS epoch time')
          call ml_wtr('warning','number of days :',abs(dj))
          call ml_wtr('warning','SATPOST abort if nb days is',djmax)
        endif
!
!
!       Section 3
!       Initialize the station coordinates
!       ----------------------------------
        call gstatc(1,statname,iergst)
        if(iergst.ne.1) then
          call ml_wt('error','in finding the station coordinates'//
     &              ' for '//statname)
          call ml_wti('error','I/O status was ',iergst)
          call ml_wt('error','SATPOST aborted')
          call aappexit(1)
        endif
!       
!
!       
!       Section 4
!       writes the header on standard output
!       ------------------------------------
        satname = genscnam(scname)
        write(*,"('#SATPOS')")
        write(*,"('satellite           ',a)") 
     &                         satname(1:cp_lsc(satname))
        write(*,"('ground station      ',a)")
     &                         statname(1:cp_lsc(statname))
        write(*,"('start date          ',a22)") ch_stdate
        write(*,"('number of days      ',f6.2)") nbdays
        write(*,"('time step (s)       ',f6.2)") timestep
        write(*,"('orbital bulletin    ',a)") 'tbus'
        write(*,"('search criteria     ',i2)") bulsear
        write(*,"('orbital elem. file   ',a)")
     &                         bulfilna(1:cp_lsc(bulfilna))
!
!
!       Section 5
!       Calculates the position for all the steps
!       -----------------------------------------
        call tb_satpos(ijstdate,nbdays,timestep)

        call ml_wt('info','SATPOST end')
        stop
        end
