#! /bin/ksh
#
# Korn Shell to run satpostle.exe: create a satpos file with an TLE
#
# The command process only one satellite. The satpos file has a starting date
# and a duration of several days. The tle bulletin is searched
# according to the criteria nearest/preceeding.
#
# PROPRIETAIRE/OWNER
#       P. Brunel - A. Marsouin
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment           <author>)
#  1.0     02/01/2002 initial            P. Brunel
#  1.2     01/02/06   change config file P. Marguinaud
#  1.3     20/02/06   remove reference to archipel P. Marguinaud
#


# AAPP environment
. "$AAPP_PREFIX/ATOVS_CONF"

#-----------------------------------------------------


# 0 initialize the variables
#---------------------------
# SATellite IDentification mandatory
# so take care of default values 
SATID_FILE=${SATID_FILE:=satid.txt}
export DIR_SATID SATID_FILE

# variables for the TLE HOME directory
DIR_DATA_TLE=${DIR_DATA_TLE:=${DIR_NAVIGATION}/tle_db}
if [ ! -d ${DIR_DATA_TLE} ]; then
  DIR_DATA_TLE=${DIR_NAVIGATION}/orb_elem
fi  
 
# variable for the stations file
DIR_STATIONS=${DIR_STATIONS:=${DIR_NAVIGATION}}
export DIR_STATIONS

STATION=${STATION:=lannion}

# 1 usage
#-------- 
usage(){
	echo
	echo ' SATPOSA  creation of a satpos file with an TLE bulletin'
	echo
	echo 'usage : satposa [-o] [-s  satellite_name] [-S station_name]';
	echo '   [-d start_date]  [-n number_of_days]';
	echo '   [-i increment_in_seconds]  [-c search_criteria]';
	echo
	echo '	options:';
	echo '  -s  satellite_name		default: noaa14';
	echo '  -S  station_name		default:lannion';
	echo '  -d  start_date			default:  today 0h';
	echo '  -n  number of days (real)	default:    1.0';
	echo '  -i  increment in seconds (real)	default:  120.0';
	echo '  -c  search criteria (n/p)	default:nearest';
	echo '                      (n for nearest or p for preceeding)';
	echo '  -o  output on normalized satpos filename satpos_satellite_date.txt   ';
	echo '      default is standard output';
	echo
	echo ' the date format could be a date or a date/hour string:';
	echo " for example -d 'dd/mm/yy  hh:mm:ss.sss' (2 spaces between yy hh)";
	echo "     or      -d 'dd/mm/yy  hh:mm'";
	echo "     or      -d dd/mm/yy";
	echo
	echo 'environment variables:'
        echo '  DIR_SATID  path for the satellite identification file'
	echo '	SATID_FILE    filename for the satellite identification file'
        echo '  DIR_NAVIGATION  generic path for the tle dir tle_db'
        echo '  DIR_STATIONS  path for the stations file'
	}

# 2 get the arguments
#--------------------
          while getopts s:S:b:d:n:i:c:o c
          do
               case $c in
               s)	SAT=$OPTARG;;
               S)	STATION=$OPTARG;;
               d)	DATE=$OPTARG;;
               n)	NBD=$OPTARG;;
               i)	INC=$OPTARG;;
               c)	CRI=$OPTARG;;
               o)	OUT=yes;;
               \?)      usage
                        exit 2;;
               esac
          done

# 2.1 default values
#-------------------

SAT=${SAT='noaa14'}
STATION=${STATION='lannion'}
DATE=${DATE=`date +%d/%m/%y`}
NBD=${NBD='1.00'}
INC=${INC='120.00'}
CRI=${CRI='n'}
OUT=${OUT='no'}

# tle index filename
BUL_INDEX=${DIR_DATA_TLE}/tle_${SAT}.index

# tle home directory
BUL_PATH=${DIR_DATA_TLE}

# search criteria
if [ "${CRI}" = 'n' ] ; then
  ICRI=0
else
  ICRI=1
fi

# output on stdout or on file
if [ "${OUT}" = 'yes' ] ; then
  set `echo ${DATE} | tr '/' ' '`
  if [ $3 -gt 50 ] ; then
    DATEY=19$3$2$1
  else
    DATEY=20$3$2$1
  fi
  if [ ! -d ${DIR_NAVIGATION}/satpos ]; then
    mkdir ${DIR_NAVIGATION}/satpos 
  fi
  OUTPUT="${DIR_NAVIGATION}/satpos/satpos_${SAT}_${DATEY}.txt"
else
  OUTPUT=''
fi


# 3 makes the satpos file
#--------------------------
if [ "${OUT}" = 'yes' ] ; then
satpostle.exe << EOF > ${OUTPUT}
${SAT}
${STATION}
${DATE}
${NBD}
${INC}
${BUL_PATH}
${BUL_INDEX}
${ICRI}
EOF
crdsatpos=$?

 if [ ${crdsatpos} -ne 0 ] ; then
   rm -f ${OUTPUT}
 fi

 if [ ! -s ${OUTPUT} ] ; then
   rm -f ${OUTPUT}
 fi

else
satpostle.exe << EOF
${SAT}
${STATION}
${DATE}
${NBD}
${INC}
${BUL_PATH}
${BUL_INDEX}
${ICRI}
EOF
crdsatpos=$?

fi


exit ${crdsatpos}

