C LIBS =  tbus brolyd satid f7ml f7tp f7cp f7gp auxaapp
        program tbusing
!+
!NOM/NAME $Id: tbusing.F 47 2006-08-21 08:44:41Z frna $
!
!BUT/PURPOSE 
!       Ingests a TBUS bulletin and process one or several satellites
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       Information for each satellite is processed separately.
!
!       For each satellite 2 historical files are created or updated:
!       - tbus index file: relative to the tbus orbital parameters
!               each record contains epoch time, quality, tbus filename
!       - clock error file: contains all the clock error information which 
!               has been validated
!
!       The TBUS epoch may be at any position in the historical files which 
!       means that an old tbus can be inserted in the files.
!
!       Algorithm
!       ---------
!       1 reads the inputs (character strings)
!         -dirtbus: home directory of the tbus files
!         -tbus_file: tbus filename to be ingested 
!               full name of the tbus file = dirtbus//tbus_file
!         for each satellite (until end of file or sat='end'):
!           -scname: satellite name
!           -tbind_file: tbus index filename for that satellite   (full name)
!           -clkerr_file: clock error filename for that satellite  (full name)
!
!       2 Creates the tbus filename and opens the file
!
!       3 For each satellite
!         3.1 opens the tbus index file and the clock error file
!               if a file does not exist, creates it
!         3.2 Process the tbus part IV orbital elements
!               decodes
!               checks the quality of the tbus by comparing it to the last 
!                 preceeding valid in the tbus index file (orbit extrapolation
!                 is made by BROLYD)
!               updates the tbus index file with the current tbus information
!         3.3 Process the tbus part IV clock error information
!               decodes
!               controls the values by comparing them to the last preceeding  
!                 values stored in the clock error file
!               Updates the clock error file with the current tbus information
!
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date         comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   March    Cut the lines exceeding 72 characters.  T.Labrot
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       tb_dc                           Decodes TBUS Part IV orbital elements 
!               tb_chk                  Checks TBUS Part IV orbital elements
!               tb_prn                  Prints TBUS Part IV orbital elements
!       tb_ctrl                         Control the quality of  orbital elements
!               tb_glpv                 get last preceeding valid TBUS filename
!               tb_sdc                  TBUS orbital elements short decoding
!       tb_wind                         writes a record in the TBUS index file
!       clkerr_dc                       clock error decoding
!               clkdatnorm              clock date normalization
!               clkvalnorm              clock values normalization
!       clkerr_ctrl                     clock error control
!       clkerr_wind                     writes a record in the clock error file
!
!
! functions
!       genscid
!       noascnam
!       cp_lsc          ! last significant character (f7cp)
!       cp_ssf  subsitute the white spaces in a character variable
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!EXIT CODE
!       0 = normal execution 
!
!DECLARATIONS
!
!
!
!
! Parameters
        integer nbsatmax                ! maximum number of satellites 
        parameter (nbsatmax=5)
        integer tbunit                  ! logical unit for the tbus file
        parameter (tbunit=10)
        integer tbindu                  ! logical unit for the tbus index
        parameter (tbindu=11)
        integer clkerru                 ! logical unit for the clock error file
        parameter (clkerru=12)
        logical debug
        parameter (debug=.false.)
!
! Functions
        integer genscid                 ! order number for a satellite name
        character*6 noascnam            ! NOAA satellite name
        integer cp_lsc                  ! last significant character (f7cp)
!
! Inputs
        character*256 dirtbus           ! home directory for the tbus files
        character*256 tbus_file         ! filename of the tbus without the
!                                       !   home directory
        character*12 scname(nbsatmax)   ! satellite name list
        character*256 tbind_file(nbsatmax)      ! filename of the tbus index
        character*256 clkerr_file(nbsatmax)     ! filename of the clock error
!
! Local variables:
        integer ios                     ! I/O status variable
        integer isat                    ! current satellite number in the list
        integer nbsat                   ! number of satellites to process
        integer prtbus                  ! if no zero tbus is printed on stdout
        integer prtclk                  ! if no zero clock errors printed 
        integer ordernum                ! order number of a satellite
        integer dcerr                   ! return code of tb_dc
        integer qflag                   ! quality flag
        real exterr(2)                  ! extrapoaltion errors (km/day)
        character*6 noaanam             ! NOAA name of the current satellite
        character*12 lsatname           ! temporary satellite name
        character*256 name              ! var. name for the files
        character*256 tbind             ! temporary filename of the tbus index
        character*256 clkerr            ! temporary filename of the clock error
        character*256 full_tbus_file    ! full filename of the tbus
        logical new_clk_fil             ! new clock error file
! 
!-
!
!       Section 1
!       Reads the input parameters
!       --------------------------
!       Get the home directory of the TBUS file
        read(*,'(a)',iostat=ios) dirtbus
        if(ios.ne.0) then
          call ml_wti('error','Input error   iostat =',ios)
          call ml_wt('error','for TBUS home directory ')
          call ml_wt('error','TBUSING aborted ')
          call aappexit(2)
        endif
!       Get the short name of the TBUS file (which will be stored in the
!       tbus index file)
        read(*,'(a)',iostat=ios) tbus_file
        if(ios.ne.0) then
          call ml_wti('error','Input error   iostat =',ios)
          call ml_wt('error','for TBUS filename ')
          call ml_wt('error','TBUSING aborted ')
          call aappexit(2)
        endif
!
!       Get the satellite list, the index filename and the clock error filename
        nbsat=0
        read(*,'(a)',iostat=ios) lsatname
        read(*,'(a)',iostat=ios) tbind
        read(*,'(a)',iostat=ios) clkerr
        do while (ios .eq. 0 .and. nbsat.lt.nbsatmax .and. 
     &            lsatname.ne.'end')
          ordernum= genscid(lsatname)
          if(ordernum.ne.0) then
            nbsat = nbsat+1
            scname(nbsat) = lsatname
            tbind_file(nbsat)  = tbind
            clkerr_file(nbsat) = clkerr
          else
            call ml_wt('notice','Input satellite '//lsatname)
            call ml_wt('notice',
     &                 'unknown in the satellite identification file')
          endif
          read(*,'(a)',iostat=ios) lsatname
          read(*,'(a)',iostat=ios) tbind
          read(*,'(a)',iostat=ios) clkerr
        end do

        if(nbsat.eq.0) then
          call ml_wt('notice','NO satellite to ingest')
          call ml_wt('error','TBUSING aborted ')
          call aappexit(2)
        endif

!       
!       Section 2
!       Initialize
!       ----------
!
!       create the tbus filename (adding the dirtbus and 
!       suppressing the white spaces)
        name =  dirtbus(1:cp_lsc(dirtbus)) // '/' // tbus_file
        call cp_ssf(name,full_tbus_file,' ')
!
!       opens the tbus file
        open (unit=tbunit,file=full_tbus_file,status='old',iostat=ios)
        if( ios.ne.0 ) then
          call ml_wti('error','open error   iostat =',ios)
          call ml_wt('error','for the tbus file '//full_tbus_file)
          call ml_wt('error','TBUSING aborted ')
          call aappexit(2)
        else
          call ml_wt('info','Ingests tbus '//full_tbus_file)
        endif

!
!       Section 3
!       Process a satellite
!       -------------------
        do isat=1, nbsat
          call ml_wt('info','Satellite '//scname(isat))
          new_clk_fil = .false.
!
!         Section 3.1
!         Opens the tbus index file and the clock error file
!         --------------------------------------------------
          ordernum = genscid(scname(isat))
          noaanam  = noascnam(ordernum)
!
          open (unit=tbindu,file=tbind_file(isat),status='old',
     &          iostat=ios)
          if( ios.ne.0 ) then
!           create the file 
            call ml_wti('notice','Open old error   iostat =',ios)
            call ml_wt('notice','for TBUS index ')
            call ml_wt('notice','try to create ')
            open (unit=tbindu,file=tbind_file(isat),status='new',
     &            iostat=ios)
            if( ios.ne.0 ) then
              call ml_wti('error','Open new error   iostat =',ios)
              call ml_wt('error','for TBUS index ')
              call ml_wt('error','TBUSING aborted ')
              call aappexit(2)
            else
              call ml_wt('notice','Creation of tbus index OK')
              write(tbindu,"('# ',a6)") noaanam
              write(tbindu,"('%  epoch_time   f orbit  fwerr  ',"//
     &                     "              'bkerr     "//
     &                     "epoch_time_string   tbus_filename')")
              rewind tbindu
            endif
          endif


          open (unit=clkerru,file=clkerr_file(isat),status='old',
     &          iostat=ios)
          if( ios.ne.0 ) then
!           create the file 
            call ml_wti('notice','Open old error   iostat =',ios)
            call ml_wt('notice','for clock error file ')
            call ml_wt('notice','try to create ')
            open (unit=clkerru,file=clkerr_file(isat),status='new',
     &            iostat=ios)
            if( ios.ne.0 ) then
              call ml_wti('error','Open new error   iostat =',ios)
              call ml_wt('error','for clock error file')
              call ml_wt('error','TBUSING aborted ')
              call aappexit(2)
            else
              call ml_wt('notice','Creation of clock error file OK')
              write(clkerru,"('# ',a6)") noaanam
              write(clkerru,"('%  last = last clock correction')")
              write(clkerru,"('%  cerr = clock error')")
              write(clkerru,"('%  rate = error rate')")
              write(clkerru,"('%  next = next clock correction')")
              rewind clkerru
              new_clk_fil = .true.
            endif       
          endif
!
!         Section 3.2
!         Reads the Part IV and checks
!         ----------------------------
          if(debug) then
            prtbus = 1
          else
            prtbus = 0
          endif

!         decodes
          call tb_dc(noaanam,tbunit,prtbus,dcerr)
          if(dcerr.eq.0) then
        
!           Checks the quality of the tbus by comparing it to the last 
!           preceeding valid in the tbus index file
            call tb_ctrl(noaanam,dirtbus,tbindu,qflag,exterr)

!
!           Updates the tbus index file with the current tbus information
            call tb_wind(tbindu,qflag,exterr,tbus_file)

          elseif(dcerr.eq.1) then

!           The tb_dc did not found a Part IV for the satellite
!           the tbus is not referenced in the tbus index file
              call ml_wt('notice','NO Part IV for that satellite')

          elseif(dcerr.eq.2) then

!           The tb_dc found an I/O error, the tbus is not referenced in the
!           tbus index file
              call ml_wt('error','I/O error while decoding the Part IV')

          elseif(dcerr.eq.3) then
!           The tb_dc found a fatal error in the orbital elements 
!               it is usefull to store it
!           Updates the tbus index file with the current tbus information
            qflag=3
            exterr(1) = 0.
            exterr(2) = 0.
            call tb_wind(tbindu,qflag,exterr,tbus_file)

          endif

          close (tbindu)

        
!         Section 3.3
!         Process the tbus part IV clock error information
!         ------------------------------------------------
          if(debug) then
            prtclk = 1
          else
            prtclk = 0
          endif

!         if the part IV has been found by tb_dc then search the error clock
!         message
          if(dcerr.ne.1) then

            call clkerr_dc(noaanam,tbunit,prtclk,dcerr)
            if(dcerr.eq.0) then

!             Controls the values by comparing them to the last preceeding  
!             values stored in the clock error file
              if ( .not. new_clk_fil ) then
                call clkerr_ctrl(clkerru)
              endif
!
!             Updates the clock error file with the current tbus information
              call clkerr_wind(clkerru)

            endif

          endif
          close(clkerru)
!
!       end of the current satellite ingest
        end do

        close (tbunit)

        stop
        end
