C LIBS = ephtrack navtool navnoaa tle sgp satid f7ml f7tp f7cp f7gp auxaapp
      program tleing
!+
!NOM/NAME $Id: tleing.F 47 2006-08-21 08:44:41Z frna $
!
!BUT/PURPOSE 
!       Ingests an TLE bulletin and process one or several satellites
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       Information for each satellite is processed separately.
!
!       For each satellite an historical file is created or updated:
!       - tle index file: relative to the tle orbital parameters
!               each record contains epoch time, quality, tle filename
!
!       The TLE epoch may be at any position in the historical files which 
!       means that an old tle can be inserted in the files.
!
!       Algorithm
!       ---------
!       1 reads the inputs (character strings)
!         -dirtle: home directory of the tle files
!         -tle_file: tle filename to be ingested 
!               full name of the tle file = dirtle//tle_file
!         for each satellite (until end of file or sat='end'):
!           -scname: satellite name
!           -tleind_file: tle index filename for that satellite   (full name)
!
!       2 Creates the tle filename and opens the file
!
!       3 For each satellite
!         3.1 opens the tle index file and the clock error file
!               if a file does not exist, creates it
!         3.2 Process the tle orbital elements
!               decodes
!               checks the quality of the tle by comparing it to the last 
!                 preceeding valid in the tle index file (orbit extrapolation
!                 is made by MOSAIK)
!               updates the tle index file with the current tle information
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date         comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   04 Jan 2005      Modify format string split over 2 lines. N.Atkinson
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       tle_dc                          Decodes TLE orbital elements
!               tle_chk                 Checks TLE orbital elements
!               tle_prn                 Print TLE orbital elements
!       tle_ctrl                                Control the quality of  orbital elements
!               tle_glpv                        get last preceeding valid TLE filename
!               tle_sdc                 tle  orbital elements short decoding
!       tle_wind                                writes a record in the TLE index file
!
! functions
!       genscid
!       genscnam
!       cp_lsc          ! last significant character (f7cp)
!       cp_ssf  subsitute the white spaces in a character variable
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!EXIT CODE
!       0 = normal execution 
!
!DECLARATIONS
!
! Parameters
        integer nbsatmax                ! maximum number of satellites 
        parameter (nbsatmax=5)
        integer tleunit                 ! logical unit for the tle file
        parameter (tleunit=10)
        integer tleindu                 ! logical unit for the tle index
        parameter (tleindu=11)
        logical debug
        parameter (debug=.false.)
!
! Functions
        integer genscid                 ! order number for a satellite name
        character*16 genscnam           ! NOAA satellite name
        integer cp_lsc                  ! last significant character (f7cp)
!
! Inputs
        character*256 dirtle            ! home directory for the tle files
        character*256 tle_file  ! filename of the tle without the
!                                       !   home directory
        character*16 scname(nbsatmax)   ! satellite name list
        character*256 tleind_file(nbsatmax)     ! filename of the tle index
!
! Local variables:
        integer ios                     ! I/O status variable
        integer isat                    ! current satellite number in the list
        integer nbsat                   ! number of satellites to process
        integer prtle                   ! if no zero tle is printed on stdout
        integer ordernum                ! order number of a satellite
        integer dcerr                   ! return code of tle_dc
        integer qflag                   ! quality flag
        real exterr(2)                  ! extrapoaltion errors (km/day)
        character*16 noaanam            ! NOAA name of the current satellite
        character*16 lsatname           ! temporary satellite name
        character*256 name              ! var. name for the files
        character*256 tleind            ! temporary filename of the tle index
        character*256 full_tle_file     ! full filename of the tle
! 
!-
!
        call sgp_init
!
!       Section 1
!       Reads the input parameters
!       --------------------------
!       Get the ref directory of the TLE file
        read(*,'(a)',iostat=ios) dirtle
        if(ios.ne.0) then
          call ml_wti('error','Input error   iostat =',ios)
          call ml_wt('error','for TLE home directory ')
          call ml_wt('error','TLEING aborted ')
          call aappexit(2)
        endif
!       Get the short name of the TLE file (which will be stored in the
!        tle index file)
        read(*,'(a)',iostat=ios) tle_file
        if(ios.ne.0) then
          call ml_wti('error','Input error   iostat =',ios)
          call ml_wt('error','for TLE filename ')
          call ml_wt('error','TLEING aborted ')
          call aappexit(2)
        endif
!
!       Get the satellite list and the index filename
        nbsat=0
        read(*,'(a)',iostat=ios) lsatname
        read(*,'(a)',iostat=ios) tleind
        do while (ios .eq. 0 .and. nbsat.lt.nbsatmax 
     &            .and. lsatname.ne.'end')
          if(ios.eq.0) then
            ordernum= genscid(lsatname)
            if(ordernum.ne.0) then
              nbsat = nbsat+1
              scname(nbsat) = lsatname
              tleind_file(nbsat)  = tleind
            else
            call ml_wt('notice','Input satellite '//lsatname)
            call ml_wt('notice',
     &                 'unknown in the satellite identification file')
            endif
          endif
          read(*,'(a)',iostat=ios) lsatname
          read(*,'(a)',iostat=ios) tleind
        end do

        if(nbsat.eq.0) then
          call ml_wt('notice','NO satellite to ingest')
          call ml_wt('error','TLEING aborted ')
          call aappexit(2)
        endif
!       
!       Section 2
!       Initialize
!       ----------
!
!       create the tle filename (adding the dirtle and 
!       suppressing the white spaces)
        name =  dirtle(1:cp_lsc(dirtle)) // '/' // tle_file
        call cp_ssf(name,full_tle_file,' ')
!
!       opens the tle file
        open (unit=tleunit,file=full_tle_file,status='old',iostat=ios)
        if( ios.ne.0 ) then
          call ml_wti('error','open error   iostat =',ios)
          call ml_wt('error','for the tle file '//full_tle_file)
          call ml_wt('error','TLEING aborted ')
          call aappexit(2)
        else
          call ml_wt('info','Ingests tle '//full_tle_file)
        endif

!
!       Section 3
!       Process a satellite
!       -------------------
        do isat=1, nbsat
          call ml_wt('info','Satellite '//scname(isat))
!
!         Section 3.1
!         Opens the tle index file
!         --------------------------
          noaanam  = genscnam(scname(isat))
!
          open (unit=tleindu,file=tleind_file(isat),status='old',
     &          iostat=ios)
          if( ios.ne.0 ) then
!           create the file 
            call ml_wti('notice','Open old error   iostat =',ios)
            call ml_wt('notice','for TLE index ')
            call ml_wt('notice','try to create ')
            open (unit=tleindu,file=tleind_file(isat),status='new',
     &            iostat=ios)
            if( ios.ne.0 ) then
              call ml_wti('error','Open new error   iostat =',ios)
              call ml_wt('error','for TLE index ')
              call ml_wt('error','TLEING aborted ')
              call aappexit(2)
            else
              call ml_wt('notice','Creation of tle index OK')
              write(tleindu,"('# ',a16)") noaanam
              write(tleindu,"('%  epoch_time   f orbit  fwerr  " //
     &              "bkerr     epoch_time_string   tle_filename')")
              rewind tleindu
            endif
          endif

!
!         Section 3.2
!         Reads the message and checks
!         ----------------------------
          if(debug) then
            prtle = 1
          else
            prtle = 0
          endif

!         decodes
          call tle_dc(noaanam,tleunit,prtle,dcerr)
          if(dcerr.eq.0) then
        
!           Checks the quality of the tle by comparing it to the last 
!           preceeding valid in the tle index file
            call tle_ctrl(noaanam,dirtle,tleindu,qflag,exterr)

!
!           Updates the tle index file with the current tle information
            call tle_wind(tleindu,qflag,exterr,tle_file)

          elseif(dcerr.eq.1) then

!           The tle_dc did not found a bulletin for the satellite
!           the tle is not referenced in the tle index file
              call ml_wt('notice','NO bulletin for that satellite')

          elseif(dcerr.eq.2) then

!           The tle_dc found an I/O error, the tle is not referenced in the
!           tle index file
              call ml_wt('error',
     &                 'I/O error while decoding the bulletin')

          elseif(dcerr.eq.3) then
!           The tle_chk found a fatal error in the orbital elements 
!               it is usefull to store it
!           Updates the tle index file with the current tle information
            qflag=3
            exterr(1) = 0.
            exterr(2) = 0.
            call tle_wind(tleindu,qflag,exterr,tle_file)

          endif

          close (tleindu)

!
!       end of the current satellite ingest
        end do

        close (tleunit)

        stop
        end
