C LIBS = ephtrack navnoaa navtool satid f7ml f7tp f7cp f7gp auxaapp
!+
!NOM/NAME $Id: tracking.F 47 2006-08-21 08:44:41Z frna $
!
        program tracking
!
!BUT/PURPOSE 
!       TRACKING creates the antenna tracking angle files corresponding
!        to a satpos file (standard unit)
!       
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       An antenna tracking angle file is produced for each orbit which
!       is acquired by the station (even short acquisition). It contains
!       the site (including a refraction correction) and azimuth values.
!
!       The time step for calculations is an integer value expressed in seconds.
!       It is defined as a data statement in the main program
!       
!       Inputs are:
!       standard input
!               satpos file 
!
!       Output
!               antenna angle files  (tracking_ssssss_yyyymmdd_ooooo.txt)
!               where   ssssss          noaa satellite name
!                       yyyymmdd        date
!                       ooooo           orbit number
!       Standard Output
!               ephemeris message for each orbit giving 
!                       the maximum elevation angle
!                       the minimum (station-satellite , station-sun) angle
!                       starting and ending time for alignment
!                                        station-satellite-sun
!
! Remarks:
!       The run is done for only one satellite due to the satpos file
!       The standard output can be redirected to the ephemeris file, and in
!       that case it has to be sorted again to be strictly chronological.
!       
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   13 March 1998   alignment station-satellite-sun calculations  A. M P.B
!     
!EXIT CODES    
!       0 = OK
!       1 to 4 error
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!DECLARATIONS
!
! Local variables:
!
        double precision ijstart! start julian instant
        double precision ijstop ! stop julian instant
        integer luin            ! logical unit for the satpos file
        integer lumes           ! logical unit for the ephem. message
        integer lutrack         ! logical unit for the tracking files
        integer ierr            ! return code 0=OK  
!                               !  sp_read:
!                                       1=satellites id diferr  (fatal)
!                                       2=periods do not mach   (fatal)
!                                       3=not enough storage    (fatal)         
!                                       4=IO error              (fatal)         
!                               ! sp_track:  
!                               ! last error code of intstat, intposvel, I/O
!
        integer timestep        ! time step in seconds
        data timestep/1/

!       units 5 and 6 are standard input and output
        data luin/5/
        data lumes/6/
        data lutrack/10/
!
!
!
!
! 
!-      
!       the whole file is processed (times = 0)
        ijstart = 0.d0
        ijstop   = 0.d0

        call ml_wt('info','start')

        call sp_read(ijstart,ijstop,luin,'any',ierr)

        if(ierr.eq.0) then
          call sp_track(timestep,lumes,lutrack,ierr)
        endif

        if(ierr.ne.0) then
          call ml_wti('error','stop',ierr)
          call aappexit(ierr)
        else
          call ml_wt('info','normal end')
        endif

        stop
        end
        subroutine sp_track(timestep,lumes,lutrack,ierr)
!+
!NOM/NAME $Id: tracking.F 47 2006-08-21 08:44:41Z frna $
!       subroutine sp_track(timestep,lumes,lutrack,ierr)
!
!BUT/PURPOSE 
!       uses the SATPOS common to compute the antenna tracking angles for the
!       station for the whole stored period.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       It calculates the station coordinates
!       For all the stored steps
!               it tests if there is a new acquisition (sp_satvis = 0)
!               if yes 
!                 it computes the start of acquisition
!                 it opens the output tracking file
!                 while the site is higher than a threshold
!                       it computes the tracking angles 
!                 close the tracking file
!               else next step
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   March 03  Cut line exceeding 72 characters.    T.Labrot        
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
!input
        integer timestep        ! time step in seconds
        integer lumes           ! logical unit for the ephemeris message
        integer lutrack         ! logical unit for the tracking files
!output
        integer ierr            ! return code 0=OK  
!                               ! else last error code  (intstat, intposvel,I/O)
!       
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision crd
        parameter (crd = 180.d0/pi)
        double precision cdr
        parameter (cdr = pi/180.d0)
        double precision pi_2
        parameter (pi_2=pi/2.d0)
!
!
! Commons
!
! common for SATPOS header
! Satpos common
#include "c_satpos.h"
!
! Functions
        double precision refrac         ! refraction correction (rad)
        integer cp_lsc                  ! last significant character (f7cp)
!
! Local variables
        integer ios                     ! I/O status variable
        integer line                    ! line number in the satpos arrays
        integer orbsacq                 ! orbit number at start of acquisition
        integer orbnum                  ! orbit number at t
        integer ierrs                   ! intstat return code 0=OK 1=conv. error
        integer ierrp                   ! intposvel " code 0=OK 1=conv. error
        integer lens                    ! character length of satellite
        double precision zen            ! satellite zenith angle  (rad) 
        double precision azi            ! satellite azimut angle  (rad) 
        double precision dist           ! station - sat distance  (km)
        double precision site           ! satellite site angle (rad)
        double precision pos(3)         ! Greenwich satellite position (km)
        double precision vel(3)         ! Greenwich satellite velocity (km)
        double precision jstop          ! julian time of the last line of data
        double precision t              ! julian time for the calculation
        integer gt(7)                   ! gregorian time for the calculation
        double precision dtst           ! time of the start of acquisition
!                                       !      in the satpos time interval (s)
        
        double precision dt_track       ! time increment for tracking ang.(days)
        double precision posstat(3)     ! station cartesian position    (km)
        double precision matstat(3,3)   ! transformation matrix from Greenwich
!                                         to the station local ref. frame  
! 
        double precision sitemax        ! maximum site for an acquisition
        double precision tsitemax       ! time at sitemax (julian days)
        character*16 eph_mes            ! ephemeris message

        double precision spos(3)        ! sun position in Greenwich ref. frame
        double precision ss1, ss2, ss3  ! vector station-satellite coordinates
        double precision scaln          ! scalar product of normalized vectors
                                        ! station-satellite and  station-sun
        double precision ssang          ! angle (0 to pi) between the 2 vectors
                                        ! station-satellite and  station-sun    
        double precision ssangthr       ! threshold for ssang (problems can occur 
!                                       ! on the antenna reception when the angle
!                                       ! is less than the threshold. (rad)
        double precision ssangmin       ! minimum of ssang variable (rad)
        double precision tssangmin      ! time at ssangmin (julian days)
        double precision tssangstart    ! starting time of ssang < ssangthr (julian days)
        double precision tssangstop     ! ending time of ssang < ssangthr (julian days)

        character*80 track_file         ! filename of a tracking file :
!                                       !  tracking_ssssss_yyyymmdd_ooooo.txt
        character*16 ev_sat             ! output satellite text
        integer day,sec,nsec            ! variables to round time t
        character*22 cht                ! time string
        logical force                   ! true if fisrt satpos record is during an aquisition
!
        parameter (ssangthr=2.*cdr)     ! 2 degrees
!-
        ierr = 0
!
!       conversion of the time step in days
        dt_track = timestep / 86400.d0
!
!       time of the last line of data in the satpos common
        jstop = sp_jstart+((sp_nbli-1)*sp_dt)
!
!       initialize the station variables
        call initrack(sp_latstat, sp_lonstat, sp_altstat,
     &                posstat, matstat)
!
!       check lenght of satellite name
!       ------------------------------
        !lens = cp_lsc(sp_satnam)
        !if( lens .eq. 6 ) then
        !  ev_sat = sp_satnam(1:6)
        !elseif( lens .le. 6 ) then
        !  ev_sat = '______'
        !  ev_sat(1:lens) = sp_satnam(1:lens)
        !else
        !  ev_sat = sp_satnam(1:4)//sp_satnam(lens-1:lens)
        !endif
        lens = cp_lsc(sp_satnam)
        if( lens .le. 16 ) then
          ev_sat = '________________'
          ev_sat(1:lens) = sp_satnam(1:lens)
        else
          ev_sat = sp_satnam(1:14)//sp_satnam(lens-1:lens)
          lens = 16
        endif

!       if first record is 0 then do not search for the first acquisistion
!       -------------------------------------------------------
        force = .false.
        if( sp_satvis(1).eq.0 .and. sp_satvis(2) .eq. 0) then
           ! force start acquistion at line 2
           sp_satvis(1) = 1
           sp_satvis(2) = 0
           force = .true.
        endif
!
!       for all the stored values
!       -------------------------
        line = 2
        do while (line.le.sp_nbli)

!         searches the changing status of sp_satvis (from 0 to 1)
!         -------------------------------------------------------
           do while ( .not.( sp_satvis(line-1).eq.1 .and.
     &                       sp_satvis(line).eq.0 )       .and.
     &                       line.le.sp_nbli)
              line = line + 1
           end do
                
!         acquisition is found, process tracking angles
!         ---------------------------------------------
          if(line.le.sp_nbli) then
            sitemax = 0.d0
            tsitemax = 0.d0
            ssangmin = 999.d0
            tssangmin = 0.d0
            tssangstart = -1.d0
            tssangstop  = -1.d0
!               finds the time of the start of acq (rounded to the next second)
            if( line .eq. 2 .and. force ) then
               dtst = 0.
               ierrs = 0
               pos(1) = sp_pos(1,line-1)
               pos(2) = sp_pos(2,line-1)
               pos(3) = sp_pos(3,line-1)
            else
               call intstat(posstat, matstat, sp_sitestat, 
     &                    sp_pos(1,line-1), sp_vel(1,line-1), 
     &                    sp_pos(1,line),   sp_vel(1,line),
     &                    sp_dt, dtst, pos , ierrs)
            endif 
            if(ierrs.eq.0) then
            
              t = sp_jstart + 
     &           ((line-1-1)*sp_dt  + dtst)/86400.d0
!             time t is rounded to the next timestep second
              day = t
              sec = (t - day) *86400
              nsec = ((sec / timestep) +1) * timestep 
              t = day *1.0d0 + nsec/86400.d0

!               compute the first point
              call intposvel(1,t,pos,vel,orbsacq,ierrp)
              call trackang(posstat,matstat,1,pos,zen,azi,dist)
              site = pi_2 - zen
              site = site + refrac(site)


              if(orbsacq.ge.0) then
              
!               creates the tracking file name, and opens the file
              call xijg(t,gt)
              write(track_file,
     &            "('tracking_',a,'_',i4.4,2i2.2,'_',i5.5,'.txt')")
     &            ev_sat(1:lens),gt(1),gt(2),gt(3),orbsacq

!              write(track_file,
!     &            "('tracking_',a6,'_',i4.4,2i2.2,'_',i5.5,'.txt')")
!     &            ev_sat,gt(1),gt(2),gt(3),orbsacq
              open(unit=lutrack,file=track_file,status='unknown',
     &             iostat=ios)
              if(ios.eq.0 .and. ierrp.eq.0) then

                call ml_wt('info','processing track file '//track_file)
!               writes the header of the file
                write(lutrack,"('satellite           ',a)") 
     &                         ev_sat(1:lens)
                write(lutrack,"('station latitude    ',f10.4)")
     &                         sp_latstat * crd
                write(lutrack,"('station longitude   ',f10.4)")
     &                         sp_lonstat * crd
                write(lutrack,"('station altitude    ',f10.4)") 
     &                         sp_altstat
                write(lutrack,"('stat. minimum site  ',f10.4)")
     &                         sp_sitestat * crd
                call gic(cht)
                write(lutrack,"('processing time     ',a)") cht
                call xijc(t,cht)
                write(lutrack,"('tracking start time ',a)") cht
                write(lutrack,"('orbit at start time ',i6)") orbsacq
                write(lutrack,"('time step (s)       ',i6)") timestep
                write(lutrack,"('     site     azim')")

!               for each step with site higher than site minimum of the station
!               ----------------------------------------------------------------
                do while( site.gt.sp_sitestat .and. ierrp.eq.0 .and.
     &                    t+dt_track.le.jstop)
!                 call xijc(t,cht)
!                 write(lutrack,"(2f10.3,2x,a22)") site*crd, azi*crd, cht
                  write(lutrack,"(2f10.3)") site*crd, azi*crd
                  t = t + dt_track
                  call intposvel(1,t,pos,vel,orbnum,ierrp)
                  call trackang(posstat,matstat,1,pos,zen,azi,dist)
                  site = pi_2 - zen
                  site = site + refrac(site)
                  if(site .gt. sitemax ) then
                    sitemax = site
                    tsitemax = t
                  endif

                  call sungrw(t,spos,dist)
                  ss1 = pos(1) - posstat(1)
                  ss2 = pos(2) - posstat(2)
                  ss3 = pos(3) - posstat(3)
                  
                  scaln = (spos(1)*ss1 + spos(2)*ss2 + spos(3)*ss3) 
     &                    / dist / dsqrt(ss1*ss1 + ss2*ss2 +ss3*ss3)
                  ssang =  dacos(scaln)

                  if(ssang .lt. ssangmin ) then
                    ssangmin = ssang
                    tssangmin = t
                  endif

                  if(ssang .lt. ssangthr .and. 
     &               tssangstart.le.0.d0 ) then
                    tssangstart = t
                  endif

                  if(ssang .lt. ssangthr ) then
                    tssangstop = t
                  endif

                end do

!               writes on standard output the ephemeris message for the maximum
!               site reached during the tracking
                if(tsitemax.gt.0.d0) then
                  write(eph_mes,'(f5.2,9x)') sitemax*crd
                  call wephmes(lumes,tsitemax,sp_satnam,
     &                     orbsacq,'site_max',eph_mes)
                endif
!               writes on standard output the ephemeris message for the minimum
!               angle sun-station-satellite reached during the tracking
                if(tssangmin.gt.0.d0) then
                  write(eph_mes,'(f6.2,8x)') ssangmin*crd
                  call wephmes(lumes,tssangmin,sp_satnam,
     &                     orbsacq,'sssang_min',eph_mes)
                endif
!               writes on standard output the ephemeris message for the
!               threshold of angle sun-station-sattellite reached 
!               during the tracking
                if(tssangstart.gt.0.d0) then
                  write(eph_mes,'(f6.2,8x)') ssangthr*crd
                  call wephmes(lumes,tssangstart,sp_satnam,
     &                     orbsacq,'sssang_start',eph_mes)
                  call wephmes(lumes,tssangstop,sp_satnam,
     &                     orbsacq,'sssang_stop',eph_mes)
                endif

              elseif(ierrp.ne.0) then
                ierr = ierrp
                call ml_wt('error', 'internal error in the '//
     &           'satellite position interpolation')
                call ml_wti('error','orbit skipped',sp_orbnum(line))

              elseif(ios.ne.0) then
                ierr = ios
                call ml_wti('error',
     &            ' while openning the antenna tracking file I/O',ios)
                call ml_wt('error','file '//track_file)
                call ml_wti('error','orbit skipped',orbsacq)
              endif
              close(unit=lutrack,iostat=ios)
              
              else
                call ml_wti('notice',
     &              'orbit number less than 0 skipped',orbsacq)
              endif

            else
!             error in the intstat convergence
              ierr = ierrs
              call ml_wti('error','in the intstat convergence',ierrs)
              call ml_wti('error','orbit skipped',sp_orbnum(line))

            endif

          else
!           end of stored values skip to the end of while
          endif

          line = line+1

!       end of stored values
        end do

        return
        end

