        subroutine intnode(pos1, vel1, pos2, vel2, dt,
     &                    dtnode, lnode, ierr)
!+
!NOM/NAME $Id: intnode.F 25 2006-02-03 15:43:53Z frna $
!       subroutine intnode(pos1, vel1, pos2, vel2, dt, dtnode, lnode, ierr)
!
!BUT/PURPOSE 
!       calculates the relative time of ascending or descending node  by
!       using a cubic interpolation of position and zeroing the z component
!       of position.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the interval pos1 - pos2 must include the equator.
!       the satellite position and relative velocity are  
!       expressed by their cartesian coordinates in Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!       a 3 order polynomial is assumed for each position coordinate
!
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
! Inputs
!       first point
        double precision pos1(3)        ! satellite position (km)
        double precision vel1(3)        ! satellite velocity (km/s)
!       second point
        double precision pos2(3)        ! satellite position (km)
        double precision vel2(3)        ! satellite velocity (km/s)
        double precision dt             ! time interval (s)     
! Ouptuts
        double precision dtnode         ! time of the node in the 
!                                       !      time interval (s)
        double precision lnode          ! longitude of the node  (rad)  
        integer ierr                    ! return code 0=OK  1=convergence error
!       
!DECLARATIONS
!
!
!
! Local parameters
!
        integer maxiter                 ! maximum number of iterations
        parameter (maxiter=20)
        double precision epsiz          ! z threshold for convergence in km 
        parameter (epsiz =0.01)

! Commons
!

! Functions
!
! Local variables
        integer i               ! loop variable
        double precision t      ! time inside SATPOS interval (in seconds)
        double precision a3(3)  ! 3rd order polynomial coefficients for x,y,z
        double precision a2(3)  ! 2rd order polynomial coefficients for x,y,z
        double precision pos(3) ! interpolated position at t
        double precision t1, t2, t3     ! time variables for dichotomy   (s)
        double precision z1, z2, z3     ! z variables for dichotomy (km)
        double precision dt2, dt3       ! time step **2 and **3
        integer nbiter          ! number of iterations
! 
!-
        dt2 = dt*dt
        dt3 = dt2*dt



! the polynomial coefficients are calculated for the 3 components
        do i=1,3
          a2(i)= 3.d0*(pos2(i) - pos1(i))/dt2
     >         - (2.d0*vel1(i) + vel2(i))/ dt 
          a3(i)= 2.d0*(pos1(i) - pos2(i))/dt3
     >          + (vel1(i) + vel2(i) ) / dt2
        enddo

        t1 = 0.d0
        z1 = pos1(3)

        t2 = dt
        z2 = pos2(3)

        t = t1 -  z1 * (t2 - t1) / (z2 -z1)
        t3 = t
        z3 = a3(3)*t*t*t + a2(3)*t*t + vel1(3)*t + pos1(3)

        nbiter = 1

!       iterative process
        do while ( dabs(z3).gt.epsiz .and. nbiter.lt.maxiter )
          if( z3 * z1 .gt. 0.d0) then
!           3 and 1 are on the same side, keep 2 , change 1 by 3
            t1 = t3
            z1 = z3
          else
!           3 and 1 are on the opposite side, keep 1 , change 2 by 3
            t2 = t3
            z2 = z3
          endif

          if(dabs(z2-z1) .lt. 0.005) then
            t3 = t1
          else
            t3 = t1 -  z1 * (t2 - t1) / (z2 -z1)
          endif
          t = t3
          z3 = a3(3)*t*t*t + a2(3)*t*t + vel1(3)*t + pos1(3)
          nbiter = nbiter + 1

        end do
        

        if(nbiter.gt.maxiter) then
          ierr=1
          dtnode = -999.d0
          lnode = 0.d0

        else
          ierr=0          
          dtnode = t3
!         interpolation of the 3 components of the position vector at t=dtnode
          do i=1,3
            pos(i) = a3(i)*t*t*t + a2(i)*t*t + vel1(i)*t + pos1(i)
          end do
          lnode = datan2( pos(2) , pos(1) )

        endif
        

        return
        end
