        subroutine intstatc(statpos, statmat, sitestat, 
     &                     pos1, pos2,
     &                     dt1, dt2, dtstat, pos3, ierr)
!+
!NOM/NAME $Id: intstatc.F 25 2006-02-03 15:43:53Z frna $
!       subroutine intstac(statpos, statmat, sitestat, 
!     &                     pos1, pos2,
!     &                     dt, dtstat, pos3, ierr)
!BUT/PURPOSE 
!       calculates the relative time of start or end of acquisition
!       using a CIRCULAR orbit and zeroing site minus sitestat.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!   
!       the interval pos1 pos2 must include the start or end of acquisition.
!       the satellite position and relative velocity are  
!       expressed by their cartesian coordinates in Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
! Inputs
        double precision statpos(3)     ! station cartesian position    (km)
        double precision statmat(3,3)   ! transformation matrix from Greenwich
!                                         to the station local ref. frame  
        double precision sitestat       ! minimum site for the antenna
!       first point
        double precision pos1(3)        ! satellite position (km)
        double precision dt1            ! time - time of ascending node (day)   
!       second point
        double precision pos2(3)        ! satellite position (km)
        double precision dt2            ! time - time of ascending node (day)   
! Ouptuts
        double precision dtstat         ! time of the start/stop of acquisition
!                                       !      in the time interval (day)
        double precision pos3(3)        ! interpolated position at dtstat
        integer ierr                    ! return code 0=OK  1=convergence error
!       
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision pi_2
        parameter (pi_2=pi/2.d0)
        integer maxiter                         ! maximum number of iterations
        parameter (maxiter=20)
        double precision epsil                  ! threshold in site difference
        parameter (epsil=0.01*pi/180.d0)        !  for convergence (0.01 deg)
!
! Common declarations:
! common for long term ephemeris generation
#include "c_lge.h"
!
! Functions
        double precision refrac         ! refraction function (rad)
!
! Local variables
        double precision t      ! time inside the interval (in seconds)
        double precision sitecomp       ! site angle for comparison (rad) 
        double precision zen            ! satellite zenith angle  (rad) 
        double precision azi            ! satellite azimut angle  (rad) 
        double precision dist           ! station - sat distance  (km)

        double precision t1, t2, t3     ! time variables for dichotomy   (s)
        double precision site           ! site variable (rad)
        double precision dsite1,dsite2,dsite3 ! difference site - sitecomp (rad)
        double precision lat            ! satellite latitude    (rd)
        double precision lon            ! satellite longitude   (rd)
        integer nbiter                  ! number of iterations
       double precision motlan  ! motion of longitude of ascd. node     (rd/day)
! 
!-


        motlan = loninc / nodper
!       Section 1
!       Initialize the 3 points for dichotomy
!       -------------------------------------
!
!       the site value for comparison is the argument value + epsil
!       because of the use of a dichotomy on site-minimum_site
!       we want to converge with a site superior to the minimum site.
        sitecomp = sitestat + epsil
!
!       point 1
!       -------
        t1 = dt1
        call trackang(statpos,statmat,1,pos1,zen,azi,dist)
        site = pi_2 - zen
        site = site + refrac(site)
        dsite1 = site - sitecomp

!       point 2
!       -------
        t2 = dt2
        call trackang(statpos,statmat,1,pos2,zen,azi,dist)
        site = pi_2 - zen
        site = site + refrac(site)
        dsite2 = site - sitecomp

!       point 3
!       -------
        if(dabs(dsite1) .le. epsil) then
          t3 = t1
          dsite3 = dsite1

        elseif(dabs(dsite2) .le. epsil) then
          t3 = t2
          dsite3 = dsite2

        else
        t = t1 -  dsite1 * (t2 - t1) / (dsite2 -dsite1)
        t3 = t
        call satpoc(semaxis,inclin,lan,nodper,motlan,t3,
     &                 pos3,lat,lon)
        call trackang(statpos,statmat,1,pos3,zen,azi,dist)
        site = pi_2 - zen
        site = site + refrac(site)
        dsite3 = site - sitecomp
        endif

        nbiter = 1

!       Section 2
!       Dichotomy
!       ---------
        do while ( dabs(dsite3).gt.epsil .and.
     &             nbiter.lt.maxiter )
          if( dsite3 * dsite1 .gt. 0.d0) then
!           3 and 1 are on the same side, keep 2 , change 1 by 3
            t1 = t3
            dsite1 = dsite3
          else
!           3 and 1 are on the opposite side, keep 1 , change 2 by 3
            t2 = t3
            dsite2 = dsite3
          endif

          if(dabs(dsite2-dsite1) .lt. 1.d-5) then
            t3 = t1
          else
            t3 = t1 -  dsite1 * (t2 - t1) / (dsite2 -dsite1)
          endif
          t = t3
          call satpoc(semaxis,inclin,lan,nodper,motlan,t3,
     &                 pos3,lat,lon)
          call trackang(statpos,statmat,1,pos3,zen,azi,dist)
          site = pi_2 - zen
          site = site + refrac(site)
          dsite3 = site - sitecomp
          nbiter = nbiter + 1

        end do

!       Checks for convergence error
!       ----------------------------
        if(nbiter.gt.20) then
          ierr=1
          dtstat = -999.d0
        else
          ierr=0          
          dtstat = t3
        endif

        return
        end
