        subroutine lge_ephe(luout,satname,ijstart,ijstop)
!+
!NOM/NAME $Id: lge_ephe.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       calculates the long term ephemeris parameters, nodes and
!         acquisition times, for one satellite and several stations.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       For each orbit (starting at the time of ascending node)
!         outputs the ephemeris message for ascending node
!         computes the descending node
!         outputs the ephemeris message for descending node
!         For each time step into the orbit
!           computes the satellite position
!           for each station
!             tests if the satellite enters or leaves the acquisition area
!             if yes then computes the start/stop time of acquisition
!
!       Remarks:
!       The nodes are calculated assuming:
!               - linear variation of the nodal period
!               - linear variation of the longitude increment
!       The start/stop time of acquisition are calculated assuming a
!         circular orbit
!       Times are rounded to the nearest tenth of second due to the possible
!       uncertainties of the circular orbital calculations
!
!       the elements for ephemeris generation are stored in common lge_elem
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer luout                   ! output logical unit
        character*(*) satname           ! noaa satellite name
        double precision ijstart        ! julian start instant
        double precision ijstop         ! julian stop instant
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision twopi
        parameter (twopi = 2.d0 * pi)
        double precision crd
        parameter (crd=180.d0/pi)
        double precision pi_2
        parameter (pi_2=pi/2.d0)

        double precision timestep       ! time increment = 2 minutes (days)     
        parameter (timestep = 2.d0/1440.d0)
!
! Commons
!
!       variables for long term ephemeris generation
#include "c_lge.h"
!
!       array of ground station coordinates ( radians, km )
#include "c_stations.h"
!
! Functions
        double precision refrac         ! refraction function
!
! Local variables
        integer kstat                   ! station number
        character*16 eph_mes            ! ephemeris message
        double precision site           ! satellite elevation angle  (rad) 
        double precision sitec          ! site corrected for refraction (rad) 
        double precision zen            ! satellite zenith angle  (rad) 
        double precision azi            ! satellite azimut angle  (rad) 
        double precision dist           ! station - sat distance          (km)
        double precision tdn            ! time of descending node
        double precision ldn            ! longitude of descending node
        double precision pos1(3)        ! satellite position (km)
        double precision pos2(3)        ! satellite position (km)
        double precision lat            ! satellite latitude    (rd)
        double precision lon            ! satellite longitude   (rd)

        integer its                     ! rounded seconds
        integer stvis1(maxstat)         ! satellite seen from the station
        integer stvis2(maxstat)         !    ..... 0=yes  1=no
        integer orbitsacq(maxstat)      ! orbit number at start of acquisition
        integer orbnum1                 ! orbit number
        integer orbnum2                 ! orbit number
        double precision dtime1         ! time since ascending node
        double precision dtime2         ! time since ascending node
        double precision dtstat         ! time since ascending node
        double precision timeacq        ! time of node (asc. or desc.)
        double precision time1          ! time point 1
        double precision time2          ! time point 2
        double precision wtime          ! rounded time for output
        double precision pos_acq(3)     ! Greenwich sat. position at timeacq

        integer ierrcs                  ! error in convergence for acquisition
        double precision motlan ! motion of longitude of ascd. node     (rd/day)
! 
!-
!
!       initialize the arrays 
        do kstat = 1, nbstat
          stvis1(kstat) = -1
          orbitsacq(kstat) = -1
        end do
        motlan = loninc / nodper

!       Section 1
!       main loop on the time of ascending nodes
!       ----------------------------------------
        do while (tan .le. ijstop)

!
!         new ascending node
!         ------------------
!         output the message for ascending node 
!             (time is rounded to the tenth of second)
          if(tan.ge.ijstart .and. tan.le.ijstop) then
            its = idnint( (tan - dint(tan))* 86400.d0 * 10.d0 )
            wtime = int(tan) + its/86400.d0/10.d0
            write(eph_mes,'(f8.3,8x)') lan*crd
            call wephmes(luout,wtime,satname,orbnum,'asc_node',eph_mes)
          endif

!
!         time of the descending node = ascending + period/2.
!         ---------------------------------------------------
          tdn = tan + nodper/2.d0
          ldn = lan - pi + loninc/2.d0
          if( ldn .gt. pi ) then
            ldn = ldn - twopi
          elseif( ldn .lt. -pi) then
            ldn = ldn + twopi
          endif
!         output the message for descending node
!             (time is rounded to the tenth of second)
          if(tdn.ge.ijstart .and. tdn.le.ijstop) then
            its = idnint( (tdn - dint(tdn))* 86400.d0 * 10.d0 ) 
            wtime = int(tdn) + its/86400.d0/10.d0
            write(eph_mes,'(f8.3,8x)') ldn*crd
            call wephmes(luout,wtime,satname,orbnum,'dsc_node',eph_mes)
          endif


!         finds the start/stop of the possible acquisitions for all the stations
!         ----------------------------------------------------------------------
          time2 = tan
          do while (time2 .lt. tan+nodper)
            dtime2 = time2 - tan
            orbnum2 = orbnum

!           computes the satellite position at time2 (circular orbit)
!           ---------------------------------------------------------
            call satpoc(semaxis,inclin,lan,nodper,motlan,dtime2,
     &                 pos2,lat,lon)

!           for each station tests if there is a start/stop of acquisition
!           --------------------------------------------------------------
            do kstat = 1, nbstat
               call trackang(posstat(1,kstat),matstat(1,1,kstat),1,
     &                     pos2,zen,azi,dist)
               site = pi_2 - zen
               sitec = site + refrac(site)
               if(sitec.ge.sitestat(kstat)) then
                 stvis2(kstat)=0
               else
                 stvis2(kstat)=1
               endif


               if(stvis2(kstat).ne.stvis1(kstat) .and. 
     &                        stvis1(kstat).ne.-1) then

!                there is a start/stop for that station during the
!                last interval [time1,time2]; computes the time more precisely
                 dtime1 = time1 - tan
                 call intstatc(posstat(1,kstat), matstat(1,1,kstat),
     &                    sitestat(kstat), 
     &                    pos1, pos2 ,
     &                    dtime1, dtime2, dtstat, pos_acq , ierrcs)

                 timeacq = dtstat + tan

!                determine if it is a start of acquisition and the orbit number
                 if(stvis1(kstat).eq.1) then

!                  determine the orbit number (in case of crossing the equator)
                   if(pos_acq(3)*pos2(3) .gt. 0.) then
                     orbitsacq(kstat)=orbnum2
                   else
                     orbitsacq(kstat)=orbnum1
                   endif
                   if(timeacq.ge.ijstart .and. timeacq.le.ijstop) then

!                    output the start_acq event
!                      (time is rounded to the tenth of second)
                     its = idnint( (timeacq - dint(timeacq))
     &                                   * 86400.d0* 10.d0 )
                     wtime = int(timeacq) + its/86400.d0/10.d0
                     call wephmes(luout,wtime,satname,
     &                   orbitsacq(kstat),'start_acq',namestat(kstat))
                   endif
                 endif

!                determine if it is a end of acquisition 
!                if the orbit number of start is unknown that 
!                 end of acq is ignored
                 if(stvis1(kstat).eq.0 .and. orbitsacq(kstat) .gt. 0) 
     &           then
                   if(timeacq.ge.ijstart .and. timeacq.le.ijstop) then

!                    output the stop_acq event
!                      (time is rounded to the tenth of second)
                     its = idnint( (timeacq - dint(timeacq))
     &                                   * 86400.d0* 10.d0 )
                     wtime = int(timeacq) + its/86400.d0/10.d0
                     call wephmes(luout,wtime,satname,
     &                   orbitsacq(kstat),'stop_acq',namestat(kstat))
                   endif
                 endif

               endif
               stvis1(kstat) = stvis2(kstat)

            end do

!           copy point 2 to 1 for next interval
            pos1(1) = pos2(1)
            pos1(2) = pos2(2)
            pos1(3) = pos2(3)
            orbnum1 = orbnum2
            time1 = time2
            time2 = time2 + timestep    
!
!         end of loop on the stations
          enddo

!         compute the next orbit parameters
!         ---------------------------------
          nodper = nodper + dnodper*nodper
          loninc = loninc + dloninc*nodper
          orbnum = orbnum +1
          tan = tan + nodper
          motlan = loninc / nodper
          lan = lan + loninc
          if( lan .gt. pi ) then
            lan = lan - twopi
          elseif( lan .lt. -pi) then
            lan = lan + twopi
          endif


!       end of loop on the nodes
        end do

        return

        end
