        subroutine lge_gelem(lgeunit,ijstart,ijstop,ierr)
!+
!NOM/NAME $Id: lge_gelem.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       get and calculates the elements for ephemeris generation
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!
!       reads the parameters of the historical ephemeris utilities
!       file, preceeding ijstop. Store them in circular arrays of 30 elements
!       (to be adapted according to the long term ephemeris duration).
!
!       The stored values are used to compute linear regressions on the nodal
!       period and longitude increment.
!
!       Then calculates the reference orbit that will be used for 
!       ephemeris calculations. This orbit contains ijstart.
!
!       the elements for ephemeris generation are stored in common lge_elem
!       
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!input
        integer lgeunit                 ! logical unit of the index file
        double precision ijstart        ! start time for the ephemeris
        double precision ijstop         ! stop time for the ephemeris
!output
        integer ierr                    ! return code 
!                                       ! 0 = OK
!                               ! 1 = no good lge before the input date
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        double precision pi     ! PI constant
        double precision twopi  ! 2 * PI 
        double precision cdr    ! conversion factor degrees to radians 

        parameter ( pi = 3.14159265358979311600d0)
        parameter ( twopi = 2.d0 * pi )
        parameter ( cdr=pi/180.0d0)
!
! Commons
!
!       variables for long term ephemeris generation
#include "c_lge.h"
!
! Functions
!
! Local variables:
        integer ios                     ! I/O status variable
        character*6 dummyst             ! dummy string
        integer ielem
        integer kelem
        integer ifelem          ! first element
        integer ilelem          ! last element
        integer nbe             ! number of elements

!       variables for the file reading
        double precision ftan           ! time of ascending node        
        integer forbnum                 ! orbit number
        double precision flan           ! longitude of ascending node   (deg)
        double precision floninc        ! longitude increment (deg/orbit)       
        double precision fsemaxis       ! semi major axis (km)  
        double precision finclin        ! inclination   (deg)
        double precision fnodper        ! nodal period  (days)
        character*22 icn                ! string for nodal period

!       variables for the regression
        integer nbem
        parameter (nbem = 30)
        double precision rtan(nbem)     ! time of ascending node        
        integer rorbnum(nbem)           ! orbit number
        double precision rlan(nbem)     ! longitude of ascending node   (rad)
        double precision rloninc(nbem)  ! longitude increment (rad/orbit)       
        double precision rsemaxis(nbem) ! semi major axis (km)  
        double precision rinclin(nbem)  ! inclination   (rad)
        double precision rnodper(nbem)  ! nodal period  (days)
        double precision rlona, rlonb
        double precision rlonr
        double precision rnoda, rnodb
        double precision rnodr

! 
!-
!
        icn='01/01/50  00:00:00.000'

        ierr = 0

!       rewind the file
        rewind lgeunit

!       skip the header lines
        read(lgeunit,'(a6)',iostat=ios) dummyst
        do while ( ios.eq.0 .and.
     &            (dummyst(1:1).eq. '#' .or. dummyst(1:1).eq.'%'
     &                                  .or. dummyst.eq.'      ')  )
          read(lgeunit,'(a6)',iostat=ios) dummyst
        end do
        if(ios.eq.0) backspace lgeunit

!
!
        ielem = 0
        kelem = 0
        ifelem = 0
        nbe = 0

        read(lgeunit,
     &       '(f15.9,1x,22x,1x,i6,1x,f8.3,1x,f10.6,1x,
     &                               f9.4,1x,f9.6,1x,a12)',iostat=ios)
     &         ftan, forbnum, flan, floninc, 
     &         fsemaxis, finclin, icn(11:22)
        call xicj(icn,fnodper)


!       loop on the records
!       informations are stored in circular arrays of nbem elements
!       stops when epoch time is after the input stop time
!       ------------------------------------------------------------
        do while (ios.eq.0 .and. ftan.le.ijstop)

          ielem = ielem +1
          kelem = kelem +1

!         keep only the last nbem elements
          if( ielem.gt.nbem ) then
            ielem = 1
          endif

          rtan(ielem) = ftan
          rorbnum(ielem) = forbnum 
          rlan(ielem) = flan * cdr      
          rloninc(ielem) = floninc * cdr
          rsemaxis(ielem) = fsemaxis 
          rinclin(ielem) = finclin * cdr
          rnodper(ielem) = fnodper
    
          read(lgeunit,
     &       '(f15.9,1x,22x,1x,i6,1x,f8.3,1x,f10.6,1x,
     &                               f9.4,1x,f9.6,1x,a12)',iostat=ios)
     &         ftan, forbnum, flan, floninc, 
     &         fsemaxis, finclin, icn(11:22)
          call xicj(icn,fnodper)

        end do

!       finds first and last elements of the circular arrays
!       ----------------------------------------------------
        if(kelem .gt. nbem) then
          ifelem = ielem + 1
          if( ifelem .gt. nbem ) ifelem = 1
          nbe = nbem
        else
          ifelem = 1
          nbe = ielem
        endif
        ilelem = ielem

!
!       if the epoch of the last element is after the start time - 30 days
!
        if( rtan(ilelem)-ijstart .gt. -30.d0) then

!       compute the regressions on the nodal period and longitude increment
!       ------------------------------------------------------------------
          call reglin(rtan,rloninc,nbe,rlona,rlonb,rlonr)
          call reglin(rtan,rnodper,nbe,rnoda,rnodb,rnodr)

!         The elements that will be used in the next steps is choosen
!         between the first and the last according to the interval start/stop
!
!       Calculates the reference orbit that will be used for 
!       ephemeris calculations. This orbit contains ijstart
!       ---------------------------------------------------

!         initialize the ref. orbit with an orbit of the circular arrays
          if ( rtan(ilelem) .ge. ijstop ) then
            ielem = ifelem
          elseif ( rtan(ifelem) .ge. ijstart ) then
            ielem = ifelem
          elseif ( rtan(ilelem) .ge. ijstart ) then
            ielem = ifelem
          else
            ielem = ilelem
          endif

          tan = rtan(ielem)
          orbnum = rorbnum(ielem)
          lan = rlan(ielem)
          loninc = rlona*tan + rlonb
          dloninc = rlona
          semaxis = rsemaxis(ielem)
          inclin = rinclin(ielem)
          nodper = rnoda*tan + rnodb
          dnodper = rnoda
          ierr =0

!         increment the orbits to find the first node BEFORE ijstart
          if(tan .ge. ijstart) then
            do while (tan .gt. ijstart)
              orbnum = orbnum -1
              nodper = nodper - dnodper*nodper
              tan = tan - nodper
              loninc = loninc - dloninc*nodper
              lan = lan - loninc
              if( lan .gt. pi ) then
                lan = lan - twopi
              elseif( lan .lt. -pi) then
                lan = lan + twopi
              endif
            end do
          else
            do while (tan .lt. ijstart)
              orbnum = orbnum +1
              nodper = nodper + dnodper*nodper
              tan = tan + nodper
              loninc = loninc + dloninc*nodper
              lan = lan + loninc
              if( lan .gt. pi ) then
                lan = lan - twopi
              elseif( lan .lt. -pi) then
                lan = lan + twopi
              endif
            end do
            orbnum = orbnum -1
            nodper = nodper - dnodper*nodper
            tan = tan - nodper
            loninc = loninc - dloninc*nodper
            lan = lan - loninc
            if( lan .gt. pi ) then
              lan = lan - twopi
            elseif( lan .lt. -pi) then
              lan = lan + twopi
            endif
          endif
          
        else
!         could not process ephemeris because the start date is too far
!         from the last value of the lge file
          call ml_wt('error','could not process because the start date')
          call ml_wt('error',
     &         'is too far from the last value of the lge file')
          ierr = 1
        endif

        return

        end
      subroutine REGLIN(X,Y,n,a,b,r)
!
!=============================================[ documentation       ]===
!---------------------------------------------[ identification      ]---
!     auteur            : Gerard LEGENDRE
!     date de creation  : 19/02/88
!---------------------------------------------[ role                ]---
!     calcul la regression lineaire  X=a*Y+b
!     r=coefficient de correlation
!---------------------------------------------[ domaine de validite ]---
!     Aucune restriction
!---------------------------------------------[ modifications       ]---
!---------------------------------------------[ remarques           ]---
!---------------------------------------------[ syntaxe d'appel     ]---
!---------------------------------------------[ variables globales  ]---
!---------------------------------------------[ common              ]---
!---------------------------------------------[ entrees/sorties     ]---
!---------------------------------------------[ modules appeles     ]---
!=============================================[ declarations        ]===
       implicit none
!---------------------------------------------[ parameters          ]---
!---------------------------------------------[ arguments           ]---
       integer n
       double precision X(n)
       double precision Y(n)
       double precision a
       double precision b
       double precision r
!---------------------------------------------[ variables globales  ]---
!---------------------------------------------[ commons             ]---
!---------------------------------------------[ variables locales   ]---
       double precision SigmaX,MoyenneX,SigmaY,MoyenneY,Produit
       integer i
!---------------------------------------------[ equivalences        ]---
!---------------------------------------------[ data                ]---
!---------------------------------------------[ fonctions appelees  ]---
!=============================================[ instructions        ]===
       if(n.gt.1) then

         SigmaX  =0.0D0
         MoyenneX=0.0D0
         SigmaY  =0.0D0
         MoyenneY=0.0D0
         Produit =0.0D0

         do 10 i=1,n
           SigmaX=SigmaX+((X(i)-MoyenneX)**2)/i
           SigmaX=(i-1)*SigmaX/i
           MoyenneX=(i-1)*MoyenneX+X(i)
           MoyenneX=MoyenneX/i

           SigmaY=SigmaY+((Y(i)-MoyenneY)**2)/i
           SigmaY=(i-1)*SigmaY/i
           MoyenneY=(i-1)*MoyenneY+Y(i)
           MoyenneY=MoyenneY/i

           Produit=Produit+X(i)*Y(i)
10       continue

         a=(Produit/n-MoyenneX*MoyenneY)/SigmaX
         b=MoyenneY-a*MoyenneX
         r=(a**2)*SigmaX/SigmaY
         r=dsqrt(r)

         else
           a=0.D0
           b=Y(1)
           r=1.0D0
         endif
      return
!=============================================[ formats             ]===
      end
