        Subroutine earthloc(
     >         satid_sat, satid_inst,
     >         ipixel, iline,
     >         reftime,  
     >         att_mode, att, lat, lon,
     >               satzen, satazi, sunzen, sunazi,
     >         ierr)
!+
!NOM/NAME $Id: earthloc.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!        calculates the earth location for a viewed point
!        and the corresponding satellite and sun zenith and azimut angles
!
!DESCRIPTION 
!        The subroutine has been revisited. 
!        It does not calculate directly, but simply call the subroutines ad-hoc
!
!        Reference frames:
!        Rs is the spacecraft fixed reference frame
!        Ra is the nominal attitude reference frame
!        Rg is the Greenwich reference frame
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!       10 Sep 1997  error corrected in chapter 3. and 4.   A. Marsouin P.Brunel
!                    to calculate zenith angle close to 0 or pi
!       08 Oct 1997  zenith and azimut calculations revisited 
!                    around zenith and nadir        A. Marsouin P.Brunel
!       24 Nov 1997  change AAPP azimut definition  A. Marsouin P.Brunel
!       12 March 2001  error correction.
!                      change calculation of the viewed point close to the 
!                      south or north pole.         A. Marsouin P.Brunel
!
! 2.0   01 Jul 2003   entirely new version          A. Marsouin P.Brunel
!       01 Apr 2004    comments changed        A. Marsouin P.Brunel
!       15 Jul 2004   Remove multiple declaration of ierr  N C Atkinson
!          
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        Implicit None                                                     
!
!ARGUMENTS

!input 
        Integer satid_sat                ! satellite index in satid file
        Integer satid_inst                ! instrument index for satid
        Integer iline                   ! line number 
        Integer ipixel                        ! pixel number
        Integer att_mode                ! attitude mode
        Double Precision reftime        ! julian instant of the first line (reference)
        Double Precision att(3,3)         ! attitude error

! output
        Double Precision lat        ! viewed points latitudes          (rad)
        Double Precision lon        ! viewed points longitudes        (rad)
!          satellite and sun viewing angles at the viewed points:
        Double Precision satzen        ! satellite zenith angles         (rad)
        Double Precision satazi        ! satellite azimut angles         (rad)
        Double Precision sunzen        ! sun zenith angles                    (rad)
        Double Precision sunazi        ! sun azimut angles                   (rad)
        Integer ierr                        ! return code 

!DECLARATIONS
!
!
!
! Local variables        (distances in km, angles in radians)
        Integer orbnum                ! orbit number
        Real pixel
        Real line
        Double Precision Mat_Ra_Rg(3,3)        ! Ra to Rg conversion matrix
        Double Precision sm_rs(3)       ! viewing vector for optical axis
        Double Precision om_g(3)        ! viewed point in Greenwich ref. frame
        Double Precision sun_g(3)        ! sun position in Greenwich ref. frame
        Double Precision dist


        Double Precision time                ! julian instant
        Double Precision scnang                ! viewed points scanning angles        (rad)
        Double Precision pos(3)                ! Greenwich satellite position  (km)
        Double Precision vel(3)                ! Greenwich satellite velocity        (km/s)
        Character*22 cht                ! date-time string
!-
!
        pixel = ipixel
        line  = iline

        ! 1. calculates viewing vectors in Rs
        !------------------------------------
        Call lptoviewvect(
     &        pixel, line, satid_sat, satid_inst, reftime,
     &        time, sm_rs, scnang)

        ! 2. calculates satellite position and velocity in Rg
        !----------------------------------------------------
        Call intposvel(1,time,pos,vel,orbnum,ierr)
        If( ierr .Ne. 0 ) Then
           Call xijc(time,cht)
           Call ml_wti('error','intposvel return code',ierr)
           Call ml_wt('error','intposvel input time '//cht)
           Return
        End If

        ! 3. calculates the Ra <--> Rg transformation
        !--------------------------------------------
        Call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','snagre return code',ierr)
           Call ml_wti('error','snagre input att mode ',att_mode)
           Return
        Endif
        
        ! 4. calculates the optical axis coordinates in Rg
        !-------------------------------------------------
        Call earthpix( pos, att, Mat_Ra_Rg, sm_rs, om_g, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','earthpix return code',ierr)
           Return
        End If

        ! 5. calculates the viewed point latitude and longitude
        !------------------------------------------------------
        Call cartlalo(1, om_g, lat, lon)
        
        ! 6. satellite zenith and azimut angles at the viewed point
        !----------------------------------------------------------
        Call zenazi(1, lat, lon, pos, satzen, satazi, dist)
           
        ! 7. sun zenith and azimut angles at the viewed point
        !    (calculates sun position in Rg then the angles)
        !------------------------------------------------------
        Call sungrw(time, sun_g, dist)
        Call zenazi(1, lat, lon, sun_g, sunzen, sunazi, dist)
          
        Return
        End
