        Subroutine earthloc_iasi(isat, inst_iasi, inst_avhrr,
     >         reftime_a, reftime_i, ipixel, iline,
     >         att_mode, att,
     >         avhrr_oa_pix, avhrr_oa_lin, 
     >         avhrr_pix, avhrr_lin,
     >         lat_oa, lon_oa,
     >         lat, lon,
     >         satzen, satazi, sunzen, sunazi, ierr)

        Implicit None                                                     
!+
!NOM/NAME $Id: earthloc_iasi.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE
!        Calculate for the optical axis and the 4 IASI pixels:
!           the earth location
!           the corresponding AVHRR line and pixel numbers
!           the satellite and sun zenith and azimut angles
!
!DESCRIPTION 
!        It does not calculate directly, but call the subroutines ad-hoc
!
!       This routine simply applies the geometrical laws of the IASI
!       scanning. IT CORRESPONDS TO A DRAFT NAVIGATION, the operational
!       IASI navigation will be based on a coregistration with AVHRR
!
!       see lptoviewvect_iasi.F for more details on IASI scanning geometry
!
!        Reference frames:
!        Rs is the spacecraft fixed reference frame
!        Ra is the nominal attitude reference frame
!        Rg is the Greenwich reference frame
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!       01 April 2004    comments changed       A. Marsouin P.Brunel
!       07 Mar 2007   correct several bugs relative to misalignment angles
!                     update call to invloc  P. Brunel
!
!DECLARATIONS
!
!ARGUMENTS
!in
        Integer isat                    ! satellite index in satid file
        Integer inst_avhrr              ! avhrr instrument index for satid
        Integer inst_iasi               ! iasi  instrument index for satid
        integer att_mode                ! attitude mode
        Double Precision reftime_a      ! julian instant of AVHRR line 1
        Double Precision reftime_i      ! julian instant of IASI line 1
        Integer iline                   ! pixel number
        Integer ipixel                  ! line number 
        Double Precision att(3,3)       ! attitude error matrix (rad)
!out
! pixels 1-4 + center
        Integer ierr                    ! return code 
        Real avhrr_oa_pix               ! AVHRR pixel number of optical axis  
        Real avhrr_oa_lin               ! AVHRR line number of optical axis
        Real avhrr_pix(4)               ! AVHRR pixel number of the 4 IASI pixels
        Real avhrr_lin(4)               ! AVHRR line number of the 4 IASI pixels
        Double Precision lat_oa         ! latitude of optical axis      (rad)
        Double Precision lon_oa         ! longitude of optical axis     (rad)
        Double Precision lat(4)         ! latitudes of the 4 IASI pixels (rad)
        Double Precision lon(4)         ! longitudes of the 4 IASI pixels (rad)

!         satellite and sun viewing angles at the 4 IASI pixels:
        Double Precision satzen(4)      ! satellite zenith angles       (rad)
        Double Precision satazi(4)      ! satellite azimut angles       (rad)
        Double Precision sunzen(4)      ! sun zenith angles             (rad)
        Double Precision sunazi(4)      ! sun azimut angles             (rad)

! Local variables

        Integer i
        Real line_i
        Real pixel_i

        Double Precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        Integer orbnum                  ! orbit number
        Double Precision pos(3)         ! Greenwich satellite position  (km)
        Double Precision vel(3)         ! Greenwich satellite velocity  (km/s)
        Double Precision so_rs(3)       ! viewing vector for optical axis
        Double Precision sm_rs(3,4)     ! viewing vector for the 4 pixels
        Double Precision sm_rs_avhrr(3) ! viewing vector for the 4 pixels
        Double Precision oa_g(3)        ! optical center in Greenwich ref. frame
        Double Precision om_g(3)        ! IASI pixel in Greenwich ref. frame

        ! Time variables for AVHRR inverse navigation
        Double Precision time   ! julian instant of the line (julian day)
        Double Precision tfirst ! julian instant of the first line (julian day)
        Double Precision tlast  ! julian instant of the last line  (julian day)

        Double Precision time_i ! times of viewed points 
        Double Precision scnang ! scanning angles of viewed points (rad)
        Double Precision dist   ! distance (km)
        Double Precision sun_g(3)       ! sun position in Greenwich ref. frame
        Character*22 cht        ! date-time string
! 
!-            
        ! IASI pixel line (real numbers)
        pixel_i = ipixel
        line_i  = iline

        
        ! 1. calculates viewing vectors in Rs
        !------------------------------------
        Call lptoviewvect_iasi(
     &        pixel_i, line_i, isat, inst_iasi, reftime_i, time_i,
     &        so_rs, sm_rs, scnang)

        ! 2. calculates satellite position and velocity in Rg
        !----------------------------------------------------
        Call intposvel(1,time_i,pos,vel,orbnum,ierr)
        If( ierr .Ne. 0 ) Then
           Call xijc(time_i,cht)
           Call ml_wti('error','intposvel return code',ierr)
           Call ml_wt('error','intposvel input time '//cht)
           Return
        End If

        ! 3. calculates the Ra <--> Rg transformation
        !--------------------------------------------
        Call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','snagre return code',ierr)
           Call ml_wti('error','snagre input att mode ',att_mode)
           Return
        Endif

        ! 4. calculates the optical axis coordinates in Rg
        !-------------------------------------------------
        Call earthpix( pos, att, Mat_Ra_Rg, so_rs, oa_g, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','earthpix return code',ierr)
           Return
        End If
        ! calculates the viewed point latitude and longitude
        Call  cartlalo(1, oa_g, lat_oa, lon_oa)

        ! Now converts in AVHRR coordinates 
        tfirst = time_i - 60. / 86400d0
        tlast  = time_i + 60. / 86400d0
        Call invloc(isat, inst_avhrr,
     &                    att_mode, att, oa_g, tfirst, tlast,
     &                    time, sm_rs_avhrr, ierr)
        If( ierr .Ne. 0 ) Then
           Call ml_wti('error','invloc return code',ierr)
           Return
        End If

        Call viewvecttolp(time,sm_rs_avhrr, isat, inst_avhrr,
     &                reftime_a, avhrr_oa_pix, avhrr_oa_lin ,scnang)
        
        ! 5. calculates the 4 pixels coordinates in Rg
        !-------------------------------------------------
        Do i=1,4
           Call earthpix( pos, att, Mat_Ra_Rg, sm_rs(1,i), om_g, ierr)
           If( ierr .Ne. 0 ) Then
              Call ml_wti('error','earthpix return code',ierr)
              Return
           End If
           ! calculates the viewed point latitude and longitude
           Call  cartlalo(1, om_g, lat(i), lon(i))

           ! satellite zenith and azimut angles of the viewed point
           Call zenazi(1, lat(i), lon(i), pos, 
     &                    satzen(i), satazi(i), dist)

           !sun zenith and azimut angles of the viewed point
           ! calculates the sun position and the vector: viewed point-Sun in Rg
           Call sungrw(time, sun_g, dist)
           Call zenazi(1, lat(i), lon(i), sun_g, 
     &                    sunzen(i), sunazi(i), dist)
           
           ! Now converts in AVHRR coordinates 
           tfirst = time_i - 60. / 86400d0
           tlast  = time_i + 60. / 86400d0
           Call invloc(isat, inst_avhrr,
     &                    att_mode, att, om_g, tfirst, tlast,
     &                    time, sm_rs_avhrr, ierr)
           If( ierr .Ne. 0 ) Then
              Call ml_wti('error','invloc return code',ierr)
              Return
           End If

           Call viewvecttolp(time,sm_rs_avhrr, isat, inst_avhrr,
     1 reftime_a, avhrr_pix(i), avhrr_lin(i)  ,scnang)
          
        Enddo

        Return
        End
