        subroutine estimatt(nbp, att_mode, time, scnang, truepos,
     &                      yaw, roll, pitch,
     &                      nbv, lmkval, lmkerr, reserr, ierr)
!+
!NOM/NAME $Id: estimatt.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE
!
!       estimation of the pseudo attitude error (yaw, roll, pitch) 
!       using landmarks true positions and observed viewing vectors
!
!       This routine is restricted to a radiometer scanning in a plane normal 
!       to the satellite velocity. Scanning function number 0 only.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       The code is the same as in the previous version. Since AAPP v5 the 
!       scanning angle is calculated in the instrument reference frame Ri 
!       (instead of Rs before) so the calulated yaw roll and pitch is 
!       in fact a pseudo attitude corresponding to the sum of the 
!       attitude angles and the misalignment angles.
!
!       Several reference frames are needed for the calculations:
!       - a landmark is located in Greenwich reference frame (Rg)
!       - viewing direction can be calculated in the instrument fixed reference
!         frame (Ri) with the scaning angle
!       - Ri should be aligned with the satellite reference frame Rs, the
!         difference being the misalignment angles.
!       - Rs should be aligned with the nominal attitude reference frame Ra, the
!         difference being the yaw roll and pitch angles.
! 
!       Scanning angle definition:
!       angle from viewing vector to nadir, counted positively according to 
!       the direct orientation of Ra (from Z to X)
!       it is positive on the left of the satellite trajectory
!        
!       Attitude estimation method
!       --------------------------
!       The landmark true position is the geographical position
!       issued from an atlas.
!
!       The observed time and scanning angle must be obtained from the AVHRR 
!       image by an automatic adjustment algorithm or operator identification.
!
!       The observed unit viewing vector, directly related to scanning angle
!       is known in Ri.
!       That same vector can calculated in Rg using the true landmark position
!       and the satellite position corresponding to the observed time.
!       It is then transformed into Ra (using the satellite position-velocity).
!       The calculated vector in Ra is related to the observed vector in Ri
!       with the pseudo attitude matrix.
!
!       Using the small angles approximation for the pseudo attitude matrix
!       yaw,roll,pitch are the unknowns a linear system of 3*nbp equations:
!                   |  yaw  |
!               X * |  roll | = Y
!                   | pitch |
!
!       X is a 3*nbp lines by 3 columns matrix:
!               |    ...            ...         ...     |
!               |                                       |
!               |     0          sin(ang_i)     0       |
!       X =     | -sin(ang_i)      0        cos(ang_i)  |       
!               |     0         -cos(ang_i)     0       |
!               |    ...            ...         ...     |
!               |                                       |
!
!       Y is a 3*nbp lines column vector:
!               |       ...             |
!               |                       |
!               |  smux_i - cos(ang_i)  |
!       Y =     |  smuy_i               |
!               |  smuz_i - sin(ang_i)  |
!               |                       |
!               |       ...             |
!
!       i refers to the ith landmark 
!       smux_i, smuy_i,smuz_i: viewing vector coordinates in Ra 
!       ang_i: scanning angle
!
!       The least square resolution gives:                      
!                      |  yaw  |
!               X'*X * |  roll | = X'*Y         (X'= transpose of X)
!                      | pitch |
!       with the following matrix formulations:
!
!               |   h1    0    h2  |               |  y1  |
!       X'*X =  |   0    nbp    0  |  and   X'*Y = |  y2  |
!               |   h2    0    h3  |               |  y3  |
!
!       the h1, h2, h3, y1, y2, y3 formulas can be read directly in the code
!
!       See also: Rosborough at al., IEEE, vol. 32, no. 3, May 94
!
!       Algorithm
!       ---------
!
!       The subroutine estimatt:
!       - calculates the attitude error (yaw, roll, pitch),
!       - identifies valid or erroneous landmarks,
!       - gives a validity criteria to the attitude estimation.
!
!       A first estimation of the attitude error is made as explained above.
!       The residual error of each landmark is calculated with the 3 equations
!       associated to the landmark.
!       The residual error of the least square resolution is then calculated
!       as the r.m.s residual error over all landmarks.
!       Valid landmarks are identified with the following criteria:
!               landmark residual error < 2. * r.m.s residual error
!       If some landmarks are not valid, the valid landmarks are used 
!       to estimate a second attitude error value and to calculate again 
!       each landmark residual error and the r.m.s. residual error.
!       The attitude estimation is finally considered as valid if:
!          each landmark residual error < 3. * r.m.s residual error
!          r.m.s residual error  < errmax (=0.002 rad <==> 2 AVHRR pixels)
!          scanning angles range > angdif (=0.755 rad <==> 800 AVHRR pixels)
!          5 or more valid landmarks
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 2.0   01 Jul 2003  adaptation to new sub snagre          A. Marsouin P.Brunel
!       07 Mar 2007    correct several bugs relative to misalignment angles
!                      code unchanged, comments are modified                
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       call intposvel(1,t,pos,vel,orbnum,ier): satellite position and velocity
!         ier = 0  OK           ier = 1  t is out of SATPOS limits
!
!       call snagre(att_mode,pos,vel,Mat_Ra_Rg,ierr): 
!              calculations for Rg --> Ra transformation
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer nbp                     ! number of landmarks
        integer att_mode
        double precision time(nbp)      ! measured julian instants  (julian day)
        double precision scnang(nbp)    ! measured scanning angles      (rad)
        double precision truepos(3,nbp) ! true landmark positions       (km)
!         expressed in cartesian coordinates in Greenwich reference frame  
!
! Output
        double precision yaw            ! estimated yaw error           (rad)
        double precision roll           ! estimated roll error          (rad)
        double precision pitch          ! estimated pitch error         (rad)
        integer nbv                     ! number of valid landmarks
        logical lmkval(nbp)             ! landmark validity flags 
!                                         if true, the landmark is valid        
        double precision lmkerr(nbp)    ! landmark residual error       (rad)
        double precision reserr         ! residual error of the least
!                                         square resolution             (rad)
        integer ierr                    ! return code 
!       ierr = 0 : OK
!       ierr from 1 to 4: the attitude estimation is made but is not valid
!         ierr=1 : less than 5 valid landmarks
!         ierr=2 : scanning angles range < 0.755 rad
!         ierr=3 : r.m.s residual error  > 0.002 rad
!         ierr=4 : a landmark residual error > 3. * r.m.s residual error
!       ierr from 10 to 13: internal error, no attitude estimation
!         ierr=10 : only one valid landmark 
!         ierr=11 : a time value is out of SATPOS limits
!         ierr=12 : impossible least square resolution with all landmarks 
!         ierr=13 : impossible least square resolution with all valid landmarks !         ierr=14 : only one input landmark
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
        integer nbx                     ! maximum number of landmarks
        parameter ( nbx = 100 )
        double precision errmax         ! maximum of residual error     (rad)
        parameter ( errmax = 2.d-3)     !  <==> 2 AVHRR pixels
        double precision angdif         ! scanning angles minimum range (rad)
        parameter ( angdif = 0.755d0)   !  <==> 800 AVHRR pixels


! Commons
!
! Functions
!
! Local variables
! 
! satellite position and velocity in Greenwich reference frame:
        double precision satpos(3)      ! position vector at t          (km)
        double precision satvel(3)      ! velocity vector at t          (km/s)
        integer orbnum                  ! orbit number at t
        integer ier1                    ! return code for intposvel

        integer i                       ! loop variable

        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix

! satellite - true landmark position unit vector in Greenwich reference frame
        double precision smu_g(3)       ! 

! satellite - true landmark position unit vectors in Sat_vel_vert ref. frame
        double precision ux(nbx)        ! x coordinate for all landmarks
        double precision uy(nbx)        ! y coordinate for all landmarks
        double precision uz(nbx)        ! z coordinate for all landmarks

        double precision smdist         ! satellite-landmark distance
        double precision sinang         ! scanning angle sinus
        double precision cosang         ! scanning angle cosinus

! coefficients for the linear system least square resolution
        double precision h1, h2, h3     ! matrix coefficients of final system 
        double precision y1, y2, y3     ! vector coefficients of final system
        double precision denom          ! intermediate variable

        double precision err            ! current landmark residual error
        double precision er1, er2, er3  ! intermediate variables
        double precision lmkmax         ! maximum landmark residual error 
        double precision angmin         ! minimum of the scanning angles
        double precision angmax         ! maximum of the scanning angles
!       the minimum and maximum are calculated over the valid landmarks
!-
!

        ierr = 0

        if (nbp.ge.2) then

! 1. Calculates for all landmarks the unit viewing vector in Ra
!--------------------------------------------------------------
          do i=1,nbp
! calculates the satellite position and velocity
            call intposvel(1,time(i),satpos,satvel,orbnum,ier1)
            if ( ier1.eq.0) then
! calculates the Ra <--> Rg transformation
              call snagre(att_mode, satpos, satvel, mat_ra_rg, ierr)
! calculates the unit vector, satellite - true landmark position, in Rg
              smu_g(1) = truepos(1,i) - satpos(1)
              smu_g(2) = truepos(2,i) - satpos(2)
              smu_g(3) = truepos(3,i) - satpos(3)
              smdist = dsqrt(smu_g(1)*smu_g(1) + smu_g(2)*smu_g(2)
     &                       + smu_g(3)*smu_g(3) )
              smu_g(1) = smu_g(1) / smdist
              smu_g(2) = smu_g(2) / smdist
              smu_g(3) = smu_g(3) / smdist
! transforms into Ra
              ux(i) = mat_ra_rg(1,1)*smu_g(1)+
     >                mat_ra_rg(1,2)*smu_g(2)+
     >                mat_ra_rg(1,3)*smu_g(3)
              uy(i) = mat_ra_rg(2,1)*smu_g(1)+
     >                mat_ra_rg(2,2)*smu_g(2)+
     >                mat_ra_rg(2,3)*smu_g(3)
              uz(i) = mat_ra_rg(3,1)*smu_g(1)+
     >                mat_ra_rg(3,2)*smu_g(2)+
     >                mat_ra_rg(3,3)*smu_g(3)
! time is not consistent with the SATPOS interval already read
            else
              ierr = 11
            endif
          end do


! 2. First attitude estimation
!-----------------------------
          if (ierr.eq.0) then

! calculates yaw, roll, pitch 
            h1 = 0.d0
            h2 = 0.d0
            h3 = 0.d0
            y1 = 0.d0
            y2 = 0.d0
            y3 = 0.d0
            do i=1,nbp
              sinang = dsin(scnang(i))
              cosang = dcos(scnang(i))
              h1 = h1 + sinang * sinang
              h2 = h2 - sinang * cosang
              h3 = h3 + cosang * cosang
              y1 = y1 - uy(i) * sinang
              y2 = y2 + ux(i) * sinang  - uz(i) * cosang
              y3 = y3 + uy(i) * cosang
            end do
            denom =  h1 * h3 - h2 * h2
            if (dabs(denom).lt.1.d-12) then
              ierr = 12
            else
              yaw = ( y1 * h3 - h2 * y3 ) / denom
              roll = y2 / nbp
              pitch = ( h1 * y3 - h2 * y1 ) / denom

! calculates residual errors 
              reserr = 0.d0
              nbv = 0
              do i=1,nbp
                sinang = dsin(scnang(i))
                cosang = dcos(scnang(i))
                er1 = ux(i) - cosang - sinang * roll
                er2 = uy(i) + sinang * yaw - cosang * pitch
                er3 = uz(i) - sinang + cosang*roll
                err = dsqrt(er1*er1 + er2*er2 + er3*er3)
                lmkerr(i) = err
                reserr = reserr + err * err
              end do
              reserr = dsqrt( reserr / nbp )

! identifies valid landmarks
              lmkmax = 2.d0 * reserr
              do i=1,nbp
                if( lmkerr(i).le.lmkmax ) then
                  nbv = nbv + 1
                  lmkval(i) = .true.
                else
                  lmkval(i) = .false.
                endif
              end do
              if ( nbv.le.1 ) ierr = 10
            endif
          endif

! 3. Second attitude estimation in case some landmarks are not valid
!-------------------------------------------------------------------
          if (ierr.eq.0.and.nbv.ne.nbp) then

! calculates yaw, roll, pitch using valid landmarks
            h1 = 0.d0
            h2 = 0.d0
            h3 = 0.d0
            y1 = 0.d0
            y2 = 0.d0
            y3 = 0.d0
            do i=1,nbp
              if( lmkval(i) ) then
                sinang = dsin(scnang(i))
                cosang = dcos(scnang(i))
                h1 = h1 + sinang * sinang
                h2 = h2 - sinang * cosang
                h3 = h3 + cosang * cosang
                y1 = y1 - uy(i) * sinang
                y2 = y2 + ux(i) * sinang  - uz(i) * cosang
                y3 = y3 + uy(i) * cosang
              endif
            end do
            denom =  h1 * h3 - h2 * h2
            if (dabs(denom).lt.1.d-12) then
              ierr = 13
            else
              yaw = ( y1 * h3 - h2 * y3 ) / denom
              roll = y2 / nbp
              pitch = ( h1 * y3 - h2 * y1 ) / denom

! calculates residual errors for valid landmarks
              reserr = 0.d0
              angmin = 4.
              angmax = -4.
              do i=1,nbp
                if( lmkval(i) ) then
                  sinang = dsin(scnang(i))
                  cosang = dcos(scnang(i))
                  er1 = ux(i) - cosang - sinang * roll
                  er2 = uy(i) + sinang * yaw - cosang * pitch
                  er3 = uz(i) - sinang + cosang*roll
                  err = dsqrt(er1*er1 + er2*er2 + er3*er3)
                  lmkerr(i) = err
                  reserr = reserr + err * err
                endif
              end do
              reserr = dsqrt( reserr / nbv )
              lmkmax = 3.d0 * reserr
              do i=1,nbp
                if( lmkval(i) ) then
                  if( err.gt.lmkmax ) ierr = 4          
                endif
              end do
            endif
          endif

! 4. validation of the attitude estimation
!-----------------------------------------
          if( ierr.eq.0) then
            angmin = 4.
            angmax = -4.
            do i=1,nbp
              if( lmkval(i) ) then
                if( scnang(i).gt.angmax ) angmax = scnang(i)            
                if( scnang(i).lt.angmin ) angmin = scnang(i)            
              endif
            end do
            if( nbv.lt.5 ) ierr = 1
            if( (angmax-angmin).le.angdif ) ierr = 2
            if( reserr.gt.errmax ) ierr = 3
          endif

! only one landmark, no attitude estimation
        else
          ierr = 14
        endif

! in case of a fatal error, fixed values are assigned to output parameters
        if (ierr.ge.10) then
          yaw = -9.d0
          roll = -9.d0
          pitch = -9.d0
          reserr = -9.d0
          nbv = 0
          do i=1,nbp
            lmkval(i) = .false.
            lmkerr(i) = -9.d0
          end do
        endif

        return
        end
