        subroutine estimrp(nbp, att_mode, time, scnang, truepos,
     &                     fixyaw, roll, pitch, 
     &                     nbv, lmkval, sigr, sigp, ierr)
!+
!NOM/NAME $Id: estimrp.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE
!
!       pseudo attitude error (yaw, roll, pitch) 
!       estimation of the roll and pich errors assuming a fixed yaw value 
!       using landmarks true positions and observed viewing vectors
!       to be used with a small number of landmarks (less than 10)
!
!       This routine is restricted to a radiometer scanning in a plane normal 
!       to the satellite velocity. Scanning function number 0 only.
!       
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       The code is the same as in the previous version. Since AAPP v5 the 
!       scanning angle is calculated in the instrument reference frame Ri 
!       (instead of Rs before) so the calulated yaw roll and pitch is 
!       in fact a pseudo attitude corresponding to the sum of the 
!       attitude angles and the misalignment angles.
!
!       Several reference frames are needed for the calculations:
!       - a landmark is located in Greenwich reference frame (Rg)
!       - viewing direction can be calculated in the instrument fixed reference
!         frame (Ri) with the scaning angle
!       - Ri should be aligned with the satellite reference frame Rs, the
!         difference being the misalignment angles.
!       - Rs should be aligned with the nominal attitude reference frame Ra, the
!         difference being the yaw roll and pitch angles.
! 
!       Roll and pitch estimation method
!       --------------------------------
!       The landmark true position is the geographical position
!       issued from an atlas.
!
!       The observed time and scanning angle must be obtained from the AVHRR 
!       image by an automatic adjustment algorithm or operator identification.
!
!       The observed unit viewing vector, directly related to scanning angle
!       is known in Ri.
!       That same vector can calculated in Rg using the true landmark position
!       and the satellite position corresponding to the observed time.
!       It is then transformed into Ra (using the satellite position-velocity).
!       The calculated vector in Ra is related to the observed vector in Ri
!       with the pseudo attitude matrix.
!
!       Using the small angles approximation for the pseudo attitude matrix and
!       assuming that the yaw value is known (fixaw), each landmark produces
!       a linear system of 3 equations with 2 unknowns roll and pitch:
!
!                       sin(ang_i) * roll               =  smux_i - cos(ang_i)
!       -sin(ang_i) * fixyaw    + cos(ang_i) * pitch    =  smuy_i
!                       -cos(ang_i) * roll              =  smuz_i - sin(ang_i)  
!
!       i refers to the ith landmark 
!       smux_i, smuy_i,smuz_i: viewing vector coordinates in Ra 
!       ang_i: scanning angle!
!
!       This system can be solved directly to obtain roll and pitch values
!       associated to the ith landmark: roll_i and pitch_i
!
!       See also: Rosborough at al., IEEE, vol. 32, no. 3, May 94
!
!       Algorithm
!       ---------
!       The subroutine estimrp:
!       - calculates the roll and pitch errors,
!       - identifies valid or erroneous landmarks,
!       - gives a validity criteria to the attitude estimation.
!
!       Roll and pitch are calculated for each landmark as explained above.
!       Roll and pitch median are calculated over all landmarks.
!       Valid landmarks are identified with the following criteria:
!               | roll_i  - roll_median |  < 0.004 rad
!               | pitch_i - pitch_median | < 0.004 rad
!       The roll and pitch are finally calculated by an average over all
!       the valid landmarks (the standard-deviations are also calculated).
!       The roll and pitch estimation is considered as valid if:
!          3 or more valid landmarks
!          roll_st_dev < 0.002 rad  and  pitch_st_dev < 0.002 rad
!          for each landmark:   | roll_i  - roll_mean |   < 3 * roll_st_dev
!                               | pitch_i - pitch_mean |  < 3 * pitch_st_dev
!
!       This method (with the median) has been preferred to a root mean square
!       resolution because this subroutine aims to process a small number
!       of landmarks which possibly includes an erroneous landmark.
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 2.0   01 Jul 2003  adaptation to new sub snagre          A. Marsouin P.Brunel
!       07 Mar 2007    correct several bugs relative to misalignment angles
!                      code unchanged, comments are modified                
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       call intposvel(1,t,pos,vel,orbnum,ier): satellite position and velocity
!         ier = 0  OK           ier = 1  t is out of SATPOS limits
!
!       call snagre(att_mode,pos,vel,Mat_Ra_Rg,ierr): 
!              calculations for Rg --> Ra transformation
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer nbp                     ! number of landmarks
        integer att_mode
        double precision time(nbp)      ! measured julian instants  (julian day)
        double precision scnang(nbp)    ! measured scanning angles      (rad)
        double precision truepos(3,nbp) ! true landmark positions       (km)
!            expressed in cartesian coordinates in Greenwich reference frame  
        double precision fixyaw         ! fixed yaw error               (rad)

! Output
        double precision roll           ! estimated roll error          (rad)
        double precision pitch          ! estimated pitch error         (rad)
        integer nbv                     ! number of valid landmarks
        logical lmkval(nbp)             ! landmark validity flags 
!                                         if true, the landmark is valid        
        double precision sigr,sigp
        integer ierr                    ! return code 
!       ierr = 0 : OK
!       ierr from 1 to 4: the attitude estimation is made but is not valid
!         ierr=1 : less than 3 valid landmarks
!         ierr=3 : roll or pitch standard-deviation  > 0.002 rad
!         ierr=4 : a landmark roll or pitch error differs from the mean
!                  by more than 3 * standard-deviation
!       ierr from 10 to 13: internal error, no attitude estimation
!         ierr=10 : only one valid landmark 
!         ierr=11 : a time value is out of SATPOS limits
!         ierr=14 : no input landmark
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
        integer nbx                     ! maximum number of landmarks
        parameter ( nbx = 100 )
        double precision errmax         ! landmark roll and pitch maximum 
!                                         distance to the median  
        parameter ( errmax = 4.d-3)     !   4.d-3 rad <==> 4 AVHRR pixels
        double precision stdmax         ! roll and pitch maximum std-dev. 
        parameter ( stdmax = 2.d-3)     !   2.d-3 rad <==> 2 AVHRR pixels


! Commons
!
! Functions
!
! Local variables
! 
! satellite position and velocity in Greenwich reference frame:
        double precision satpos(3)      ! position vector at t          (km)
        double precision satvel(3)      ! velocity vector at t          (km/s)
        integer orbnum                  ! orbit number at t
        integer ier1                    ! return code for intposvel

        integer i                       ! loop variable

        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix

! satellite - true landmark position unit vector in Greenwich reference frame
        double precision smu_g(3)       ! 

! satellite - true landmark position unit vectors in Sat_vel_vert ref. frame
        double precision ux(nbx)        ! x coordinate for all landmarks
        double precision uy(nbx)        ! y coordinate for all landmarks
        double precision uz(nbx)        ! z coordinate for all landmarks

        double precision smdist         ! satellite-landmark distance
        double precision sinang         ! scanning angle sinus
        double precision cosang         ! scanning angle cosinus

        double precision rol(nbx)       ! landmark roll value
        double precision pit(nbx)       ! landmark pitch value
        double precision r,p,rol2,pit2  ! intermediate variables
!-
!
        nbv=0
        ierr = 0

        if (nbp.ge.1) then

! 1. Calculates for all landmarks the unit viewing vector in Ra
!--------------------------------------------------------------
          do i=1,nbp
! calculates the satellite position and velocity
            call intposvel(1,time(i),satpos,satvel,orbnum,ier1)
            if ( ier1.eq.0) then
! calculates the Ra <--> Rg transformation
              call snagre(att_mode, satpos, satvel, mat_ra_rg, ierr)
! calculates the unit vector, satellite - true landmark position, in Rg
              smu_g(1) = truepos(1,i) - satpos(1)
              smu_g(2) = truepos(2,i) - satpos(2)
              smu_g(3) = truepos(3,i) - satpos(3)
              smdist = dsqrt(smu_g(1)*smu_g(1) + smu_g(2)*smu_g(2)
     &                       + smu_g(3)*smu_g(3) )
              smu_g(1) = smu_g(1) / smdist
              smu_g(2) = smu_g(2) / smdist
              smu_g(3) = smu_g(3) / smdist
! transforms into Ra
              ux(i) = mat_ra_rg(1,1)*smu_g(1)+
     >                mat_ra_rg(1,2)*smu_g(2)+
     >                mat_ra_rg(1,3)*smu_g(3)
              uy(i) = mat_ra_rg(2,1)*smu_g(1)+
     >                mat_ra_rg(2,2)*smu_g(2)+
     >                mat_ra_rg(2,3)*smu_g(3)
              uz(i) = mat_ra_rg(3,1)*smu_g(1)+
     >                mat_ra_rg(3,2)*smu_g(2)+
     >                mat_ra_rg(3,3)*smu_g(3)
! calculates roll and pitch 
              sinang = dsin(scnang(i))
              cosang = dcos(scnang(i))
              rol(i) = ux(i) * sinang  - uz(i) * cosang
              pit(i) = ( uy(i) + fixyaw * sinang ) / cosang
! time is not consistent with the SATPOS interval already read
            else
              ierr = 11
            endif
          end do


! 2. Identification of the valid landmarks
!-----------------------------------------
          if (ierr.eq.0) then

! calculates roll and pitch median values
            call nav_median(nbp,rol,roll)
            call nav_median(nbp,pit,pitch)

! identifies valid landmarks
            do i=1,nbp
              if( dabs(rol(i)-roll)  .le. errmax .and.
     &            dabs(pit(i)-pitch) .le. errmax ) then
                nbv = nbv + 1
                lmkval(i) = .true.
              else
                lmkval(i) = .false.
              endif
            end do
            if ( nbv.le.1 ) ierr = 10
          endif

! 3. Attitude estimation using the valid landmarks
!-------------------------------------------------
          if (ierr.eq.0) then

            roll = 0.d0
            pitch = 0.d0
            rol2 = 0.d0
            pit2 = 0.d0
            do i=1,nbp
              if( lmkval(i) ) then
                r = rol(i)
                p = pit(i)
                roll = roll + r
                pitch = pitch + p
                rol2 = rol2 + r * r
                pit2 = pit2 + p * p
              endif
            end do
            roll = roll / nbv
            pitch = pitch / nbv  
            sigr = dsqrt( rol2 / nbv - roll*roll )
            sigp = dsqrt( pit2 / nbv - pitch*pitch )
            do i=1,nbp
              if(  lmkval(i) .and.
     &           ( dabs(rol(i)-roll)  .gt. (3.d0*sigr) .or.
     &             dabs(pit(i)-pitch) .gt. (3.d0*sigp)     ) ) ierr = 4
            end do

          endif

! 4. validation of the attitude estimation
!-----------------------------------------
          if( ierr.eq.0) then
            if( nbv.lt.3 ) ierr = 1
            if( sigr.gt.stdmax .or. sigp.gt.stdmax ) ierr = 3
          endif

! no input landmark, no attitude estimation
        else
          ierr = 14
        endif

! in case of a fatal error, fixed values are assigned to output parameters
        if (ierr.ge.10) then
          roll = -9.d0
          pitch = -9.d0
          sigr = -9.d0
          sigp = -9.d0
          nbv = 0
          do i=1,nbp
            lmkval(i) = .false.
          end do
        endif

        return
        end
        
        subroutine nav_median(nbp,array,med)
!+
!NOM/NAME $Id: estimrp.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!
!       calculates the median of several values
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer nbp                     ! number of values
        double precision array(nbp)     ! array of values
!
! Output
        double precision med            ! median of the input values
!
!DECLARATIONS
!
!
!
! Local parameters
        integer nbx                     ! maximum number of values
        parameter ( nbx = 100 )

! Commons
!
! Functions
!
! Local variables
        double precision ordarr(nbx)    ! array sorted by increasing values 
        integer i,j                     ! loop variables
        double precision val            ! intermediate variable
        integer n                       ! intermediate variable
!-
!
! copy the input array in an intermediate array
        if (nbp.ge.2) then
          do i=1,nbp
            ordarr(i) = array(i)
          end do

! sort the array ordarr by increasing values
          do i=1,nbp-1
            do j=i+1,nbp
              if( ordarr(j).lt.ordarr(i)) then
                val = ordarr(i)
                ordarr(i) = ordarr(j)
                ordarr(j) = val
              endif
            end do
          end do

! calculate the median
          n = nbp / 2
          if( nbp.eq.2*n) then
            med = ( ordarr(n) + ordarr(n+1) ) /2.d0
          else
            med = ordarr(n+1)
          endif   
        else
          med = array(1)
        endif

        return
        end
