        subroutine footprint(
     &        pixel, line, isat, inst, npoints, psi0, reftime, 
     &        time, sm_rs, sp_rs, ierr)
!+
!NOM/NAME $Id: footprint.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       Exact calculation in the spacecraft fixed reference frame Rs
!       of the vectors defining the footprint edge
!
!DESCRIPTION 
!        FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       The footprint is the surface on Earth corresponding to the IFOV
!
!       The following types of instrument scanning geometry are allowed:
!          0: scanning in a plane, no tilt
!          1: scanning in a plane, with tilt
!         2: conical scanning with or without tilt
!
!       Calculations are similar to those of lptoviewvect. 
!
!       The IFOV center line and pixel are converted into time and scanning angle.
!
!       The satellite-viewed point unit vector is calculated in Ri for:
!       - the IFOV center
!       - all point of the footprint edge

!       All vectors are converted from Ri to Rs taking into account
!       the instrument misalignment in yaw, roll and pitch.
!
!       The routine may calculate:
!       - a full footprint edge described by "npoints" points
!       - a unique point of the footprint edge (if npoints=1), corresponding
!         to an azimuth angle of "psi0" in Ri
!
!       Reference frames:
!          Ri instrument-fixed reference frame
!          Rs spacecraft-fixed reference frame
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!       01 Apr 2004    comments changed        A. Marsouin P.Brunel
!       15 Jul 2004    Fix declaration of r_i - "\" and "/" characters in
!                      comments were upsetting g77 compiler  N Atkinson
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
        real pixel                        ! pixel number
        real line                        ! line number 
        integer isat                        ! satellite index in satid file
        integer inst                        ! instrument index for satid
        integer npoints                        ! number of points for the footprint edge
        double precision psi0                ! initial azim angle for the footprint edge (rad)
        double precision reftime        ! julian instant of line 1
!out
        integer ierr
        double precision time           ! julian instant of the viewed point
        double precision sm_rs(3)       ! satellite - IFOV center unit vector in Rs
        double precision sp_rs(3, npoints) ! same for all points of the footprint edge
!        
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        integer max_nbp_edge
        parameter (max_nbp_edge = 360)
!
! Commons
!
#include "satid.h"
! Functions
!
! Local variables
        ! angles are expressed in radians
        integer i
        double precision timlin
        double precision alpha        ! scanning angle
        double precision beta        ! tilt angle
        double precision delta  ! half angle of the cone, for conical scanning only
        double precision phi    ! scanning azimuth, for conical scanning only
        double precision psi    ! azimuth angle for a point on the footprint edge
        double precision psi_step! step azimuth angle
        double precision eps    ! half angle of the cone corresponding to the IFOV
        double precision sp_r1(3) ! viewing vector in R1 reference frame
        double precision sm_ri(3) ! viewing vector in instrument reference frame
        double precision sp_ri(3, max_nbp_edge) ! for contour
        double precision y_i !
        double precision r_i ! Yaw Roll Pitch misalignment angles
        double precision p_i !
! 
!-
!        
        ierr = 0

        if( npoints .gt. max_nbp_edge ) then
           call ml_wti('error',
     &                'number of contour points is too large ',npoints)
           call ml_wti('error',
     &                'max allowed value is ',max_nbp_edge)
           ierr = 1
           return
        endif
           
        if ( npoints .gt. 1 ) then
           psi_step = 2. * pi / (npoints - 1) ! "azimuth" angle for contour
        else
           ! only one point
           ! calculation for psi0 angle
           psi_step = 0.
        end if

        ! Test IFOV definition
        if(ifov_mode(inst,isat) .ne. 0) then
           call ml_wti('error','IFOV mode is ',ifov_mode(inst,isat))
           call ml_wt('error',
     &                'footprint routine does not allow this mode')
           ierr = 2
           return
        endif
        ! take first value of IFOV mode parameters
        eps = ifov_mode_p(1, inst, isat) /2.
!        write(*,*) 'eps ',eps

        ! 1. calculate viewing vector in Ri frame according to scanning function
        ! Ri   Instrument Reference frame
        !-----------------------------------------------------------------------

        if( inst_scan_mode(inst, isat) .eq. 0 ) then
           ! Scanning in a plane, no tilt
           !
           ! definition of scanning function parameters
           ! 1 = step time per FOV  (jdays)
           ! 2 = offset time between start time and first pixel of a line (jdays)
           ! 3 = center to center FOV step angle (rad)
           ! 4 = pixel number at nadir
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
           timlin = reftime + (line-1) * 
     &             inst_scan_mode_p(5,inst,isat)

           time = timlin + inst_scan_mode_p(2,inst,isat) + 
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           alpha = (pixel-inst_scan_mode_p(4,inst,isat)) * 
     &             inst_scan_mode_p(3,inst,isat)

           ! calculates the satellite - viewed point unit vector in Rs
           sm_ri(1) = dcos(alpha)
           sm_ri(2) = 0.
           sm_ri(3) = dsin(alpha)

           do i = 1, npoints
              psi = psi0 + (i-1) * psi_step
              sp_ri(1,i) = dcos(alpha) - eps*dsin(psi)*dsin(alpha)
              sp_ri(2,i) =               eps*dcos(psi)
              sp_ri(3,i) = dsin(alpha) + eps*dsin(psi)*dcos(alpha)
!              write(*,*) 'sp_ri ',sp_ri(1,i),sp_ri(2,i),sp_ri(3,i)
           end do

        elseif( inst_scan_mode(inst, isat) .eq. 1 ) then
           ! Scanning in a plane with tilt
           ! 
           ! definition of scanning function parameters
           ! 1 = step time per FOV  (jdays)
           ! 2 = offset time between start time and first pixel of a line(jdays)
           ! 3 = center to center FOV step angle (rad)
           ! 4 = pixel number at subtrack
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
           ! 6 = tilt angle between nadir and scanning plane (rad)
           timlin = reftime + (line-1) * 
     &             inst_scan_mode_p(5,inst,isat)

           time = timlin + inst_scan_mode_p(2,inst,isat) + 
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           alpha = (pixel-inst_scan_mode_p(4,inst,isat)) * 
     &             inst_scan_mode_p(3,inst,isat)

           beta = inst_scan_mode_p(6,inst,isat)

           ! calculates the satellite - viewed point unit vector in Rs
           sm_ri(1) = dcos(beta)*dcos(alpha)
           sm_ri(2) = dsin(beta)*dcos(alpha)
           sm_ri(3) = dsin(alpha)

           do i = 1, npoints
              psi = psi0 + (i-1) * psi_step
              sp_ri(1,i) = dcos(beta)* 
     &                       (dcos(alpha) - eps*dsin(psi)*dsin(alpha)) -
     &                       eps*dsin(beta)*dcos(psi)
              sp_ri(2,i) = dsin(beta)* 
     &                       (dcos(alpha) - eps*dsin(psi)*dsin(alpha)) +
     &                       eps*dcos(beta)*dcos(psi)
              sp_ri(3,i) = dsin(alpha) + eps*dsin(psi)*dcos(alpha)
           end do


        elseif( inst_scan_mode(inst, isat) .eq. 2 ) then
           ! Conic scanning w/wo tilt
           !
           ! definition of scanning function parameters
           ! 1 = step time per FOV  (jdays)
           ! 2 = offset time between start time and first pixel of a line(jdays)
           ! 3 = center to center FOV step angle, centre to centre FOV (rad)
           ! 4 = pixel number at subtrack
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
           ! 6 = tilt angle between nadir and cone axis (rad)
           ! 7 = half cone angle (rad)
           ! 8 = forward scanning  >= 0
           !     backward scanning < 0
           timlin = reftime + (line-1) * 
     &             inst_scan_mode_p(5,inst,isat)

           time = timlin + inst_scan_mode_p(2,inst,isat) + 
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           beta  = inst_scan_mode_p(6,inst,isat)
           delta = inst_scan_mode_p(7,inst,isat)
           
           alpha = (pixel-inst_scan_mode_p(4,inst,isat)) * 
     &             inst_scan_mode_p(3,inst,isat)
           
           if ( inst_scan_mode_p(8,inst,isat)  .ge. 0. ) then
              ! forward scanning
              phi = alpha + pi
           else
              ! backward scanning
              phi = alpha
           endif

           ! calculates the satellite - viewed point unit vector in Rs
           sm_ri(1) = dcos(beta)*dcos(delta) - 
     &                          dsin(beta)*dsin(delta)*dcos(phi)
           sm_ri(2) = dsin(beta)*dcos(delta) +
     &                          dcos(beta)*dsin(delta)*dcos(phi)
           sm_ri(3) = dsin(delta)*dsin(phi)

           do i = 1, npoints
              psi = psi0 + (i-1) * psi_step
              sp_r1(1) = dcos(delta) - eps*dcos(psi)*dsin(delta)
              sp_r1(2) = dsin(delta)*dcos(phi) + 
     &                    eps*(
     &                      dcos(psi)*dcos(phi)*dcos(delta) - 
     &                                  dsin(psi)*dsin(phi)  )
              sp_r1(3) = dsin(delta)*dsin(phi) +
     &                    eps*(
     &                      dcos(psi)*dsin(phi)*dcos(delta) +
     &                                  dsin(psi)*dcos(phi)  )
              
              sp_ri(1,i) = dcos(beta)*sp_r1(1) - dsin(beta)*sp_r1(2)
              sp_ri(2,i) = dsin(beta)*sp_r1(1) + dcos(beta)*sp_r1(2)
              sp_ri(3,i) = sp_r1(3)
           end do

        else
           sm_ri(1) = 0.
           sm_ri(2) = 0.
           sm_ri(3) = 0.
           do i = 1, npoints
              sp_ri(1,i) = 0.
              sp_ri(2,i) = 0. 
              sp_ri(3,i) = 0.
           end do

        end if


        ! 2. calculate viewing vector in Rs
        ! Rs   Spacecraft-fixed Reference frame
        !--------------------------------------
        if (mis_align_mode(inst,isat) .eq. 0) then
           
           ! Yaw Roll Pitch misalignment definition
           if (mis_align_mode_n(inst,isat) .ge. 3) then
              ! yes 3 parameters
              y_i = mis_align_mode_p(1,inst,isat)
              r_i = mis_align_mode_p(2,inst,isat)
              p_i = mis_align_mode_p(3,inst,isat)
           else
              y_i = 0.
              r_i = 0.
              p_i = 0.
           endif

           sm_rs(1) =      sm_ri(1) + p_i*sm_ri(2) - r_i*sm_ri(3)
           sm_rs(2) = -p_i*sm_ri(1) +     sm_ri(2) + y_i*sm_ri(3)
           sm_rs(3) =  r_i*sm_ri(1) - y_i*sm_ri(2) +     sm_ri(3) 
           
           do i = 1, npoints
              sp_rs(1,i) =  
     &                      sp_ri(1,i) + p_i*sp_ri(2,i) - r_i*sp_ri(3,i)
              sp_rs(2,i) =
     &                 -p_i*sp_ri(1,i) +     sp_ri(2,i) + y_i*sp_ri(3,i)
              sp_rs(3,i) =
     &                  r_i*sp_ri(1,i) - y_i*sp_ri(2,i) +     sp_ri(3,i) 
           end do
        else
           !No misalignment, or unknown
           sm_rs(1) = sm_ri(1)
           sm_rs(2) = sm_ri(2)
           sm_rs(3) = sm_ri(3)
           do i = 1, npoints
              sp_rs(1,i) = sp_ri(1,i)
              sp_rs(2,i) = sp_ri(2,i) 
              sp_rs(3,i) = sp_ri(3,i) 
           end do
        endif

        return
        end
