        subroutine invloc(isat,inst,att_mode, att, om_g, tfirst,tlast,
     &                    time,sm_s,ierr)

!+
!NOM/NAME $Id: invloc.F 503 2016-11-15 12:00:50Z frna $
!
!BUT/PURPOSE 
!
!       calculates the time and scanning angle corresponding to a point
!       on the earth defined by its Greenwich coordinates  
!       for an orbit portion corresponding to the acquisition by a station
!       (inverse problem of the earth location)
!
!       This routine is restricted to a radiometer scanning in a plane normal 
!       to the satellite velocity. Scanning function number 0 only.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       Several reference frames are needed for the calculations:
!       - the viewed point is located in Greenwich reference frame (Rg)
!       - viewing direction must be calculated in the instrument fixed reference
!         frame (Ri) to obtain the scaning angle
!       - Ri should be aligned with the satellite reference frame Rs, the
!         difference being the misalignment angles.
!       - Rs should be aligned with the nominal attitude reference frame Ra, the
!         difference being the attitude angles.
! 
!
!       Scanning angle definition:
!       angle from viewing vector to nadir, counted positively according to 
!       the direct orientation of Ra (from Z to X)
!       it is positive on the left of the satellite trajectory
!
!       Algorithm
!       ---------
!
!       the Ra <--> Rg transformation depends upon the satellite position 
!       and velocity i.e. with time therefore the algorithm is iterative
!
!       there are two possibilities for the initial time value:
!       - the value obtained in the last call to the subroutine
!       - the time at the middle of the pass
!       the choice is made using the distance to the last viewed point
!
!       at iteration i:
!       the satellite position and velocity are calculated at time t(i)
!       the Ra <--> Rg transformation is calculated
!       the satellite - viewed point vector SM is calculated in Rg
!       it is transformed into Ra then into Rs using attitude errors
!       then into Ri using misalignment angles
!       the y coordinate of SM in Ri should be 0 (<==> M is in scanning plane)
!       a correction to the satellite argument is obtained from SM coordinates
!       it corresponds to a time correction dt and t(i+1) = t(i) + dt
!
!       iterations are stopped when dt < 0.001 second
!
!       if the iterative process has correctly converged
!       - time is compared to the instants of the first and last line
!       - the angle OMS (O: earth center, M: viewed point, S: satellite)
!       is compared to pi/2 in order to eliminate the cases 
!       where M is on the other side of the earth than S
!
!       Validity limits: 
!       --------------- 
!               tlast - tfirst < nodal period / 4

!       for longer time period: use invlorb
!
!       Remark:
!       ------
!       before calling invloc, the satpos file must have been read with
!       a time interval larger than [tfirst,tlast] (1 mn before, 1 mn after)
!
!        
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 2.0   01 Jul 2003  adaptation to new sub snagre          A. Marsouin P.Brunel
!       07 Mar 2007  correct several bugs relative to misalignment angles
!                    calculations take into account misalignement angles
!                    2 new arguments, iterations are performed in Ri
! 2.1   02 Nov 2016  At the end, only save om_old and time_old if the point is
!                    viewed.
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       call intposvel(1,t,pos,vel,orbnum,ier): satellite position and velocity
!         ier = 0  OK           ier = 1  t is out of SATPOS limits
!
!       call snagre(att_mode,pos,vel,Mat_Ra_Rg,ierr): 
!              calculations for Rg --> Ra transformation
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer isat                    ! satellite index in satid file
        integer inst                    ! instrument index for satid
        integer att_mode
        double precision att(3,3)! attitude error matrix                (rad)
        double precision om_g(3)        ! viewed point M in Greenwich ref. frame
        double precision tfirst ! julian instant of the first line (julian day)
        double precision tlast  ! julian instant of the last line  (julian day)
!
! Output
        double precision time   ! viewed point julian instant   (julian day)
! satellite - viewed point vector (SM):
        double precision sm_s(3)        !   in spacecraft fixed ref. frame
        integer ierr            ! return code 
!         0 = OK, the point may be viewed but scnang has to be checked 
!         1 = the point is not viewed, time is not in [tfirst,tlast]
!         2 = the point is not viewed, on other side of earth than satellite
!        10 = the iterative process has not converged
!        11 = viewed point not consistent with the SATPOS interval already read
!        12 = erroneous latitude input

!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters

        integer maxiter                 ! maximum number of iterations
        parameter (maxiter = 20)

        double precision tepsil         ! time threshold for convergence   (day)
        double precision ttoler         ! tolerance on t in [tfirst,tlast] (day)
        parameter (tepsil = 0.001 / 86400.d0,
     &             ttoler = 2.d0 * tepsil)

        double precision distmax        ! maximum distance to last viewed point
        parameter (distmax = 1000.d0)

        double precision pi, pi_2       ! pi,  pi/2
        parameter ( pi = 3.14159265358979311600D0 ,
     &              pi_2 = pi / 2.d0)

!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Commons
#include "c_satpos.h"
#include "satid.h"
!
! Functions
!
! Local variables  (distances in km, angles in radians)
!
        double precision meanmot        ! satellite mean motion       (rad/day)
        double precision om_old(3)      ! om_g value of last call to invloc 
        double precision time_old       ! julian instant of last call to invloc
        double precision dist           ! distance to the last viewed point 
        double precision dx, dy, dz     ! intermediate variables 

        double precision t              ! julian instant of current iteration
        double precision dt             ! correction applied to t
        integer nbiter                  ! number of iterations

! satellite position and velocity in Greenwich reference frame:
        double precision pos(3)         ! position vector at t          (km)
        double precision vel(3)         ! velocity vector at t          (km/s)
        integer orbnum                  ! orbit number at t
        integer ier                     ! return code for intposvel

        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix

! satellite - viewed point vector (SM):
        double precision sm_g(3)        !   in Greenwich reference frame
        double precision sm_v(3)        !   in vel_vert reference frame

        double precision satdis         ! distance satellite - earth center
        double precision deltarg        ! satellite argument correction
        double precision scalpro        ! scalar product MS by MO

        double precision sm_ri(3) ! satellite - viewed point unit vector in Ri
        double precision y_i !
        double precision r_i ! Yaw Roll Pitch misalignment angles
        double precision p_i !

        data time_old/ 0.d0 /
        data om_old/ 3 * 0.d0/  
!
!-
!
        save om_old, time_old
        ierr = 0
        time = -999.d0

        meanmot = mu / (sp_semaxis*sp_semaxis*sp_semaxis)
        meanmot = dsqrt(meanmot) * 86400.d0

        y_i = mis_align_mode_p(1,inst,isat)
        r_i = mis_align_mode_p(2,inst,isat)
        p_i = mis_align_mode_p(3,inst,isat)

! 2. calculates an initial value of time
!---------------------------------------

! calculates the distance to the viewed point of the previous call
          dx = om_g(1) - om_old(1)
          dy = om_g(2) - om_old(2)
          dz = om_g(3) - om_old(3)
          dist = dsqrt( dx*dx + dy*dy + dz*dz )
        
! calculates the initial time value
          if ( dist.le.distmax ) then
            t = time_old
          else
            t = ( tfirst + tlast ) / 2.d0
          endif

! 3. iterative process
!---------------------
          dt = 0.d0
          nbiter = 0
          ier = 0

 100      continue

            t = t + dt
            nbiter = nbiter + 1

! calculates the satellite position and velocity
            call intposvel(1,t,pos,vel,orbnum,ier)
            if ( ier.eq.0) then

! calculates the Ra <--> Rg transformation
              call snagre(att_mode, pos, vel, mat_ra_rg, ierr)

! calculates the SM vector, satellite - viewed point, in Rg
              sm_g(1) = om_g(1) - pos(1)
              sm_g(2) = om_g(2) - pos(2)
              sm_g(3) = om_g(3) - pos(3)

! transforms into Ra
              sm_v(1) = mat_ra_rg(1,1)*sm_g(1)+
     >                  mat_ra_rg(1,2)*sm_g(2)+
     >                  mat_ra_rg(1,3)*sm_g(3)
              sm_v(2) = mat_ra_rg(2,1)*sm_g(1)+
     >                  mat_ra_rg(2,2)*sm_g(2)+
     >                  mat_ra_rg(2,3)*sm_g(3)
              sm_v(3) = mat_ra_rg(3,1)*sm_g(1)+
     >                  mat_ra_rg(3,2)*sm_g(2)+
     >                  mat_ra_rg(3,3)*sm_g(3)

! transforms into Rs using attitude error matrix
              sm_s(1)=att(1,1)*sm_v(1)+att(1,2)*sm_v(2)+att(1,3)*sm_v(3)
              sm_s(2)=att(2,1)*sm_v(1)+att(2,2)*sm_v(2)+att(2,3)*sm_v(3)
              sm_s(3)=att(3,1)*sm_v(1)+att(3,2)*sm_v(2)+att(3,3)*sm_v(3)

! transforms into Ri (instrument frame )using misalignment angles
              sm_ri(1) =      sm_s(1) + p_i*sm_s(2) - r_i*sm_s(3)
              sm_ri(2) = -p_i*sm_s(1) +     sm_s(2) + y_i*sm_s(3)
              sm_ri(3) =  r_i*sm_s(1) - y_i*sm_s(2) +     sm_s(3)

! calculates the correction needed on the satellite argument
              satdis = dsqrt(pos(1)*pos(1)+pos(2)*pos(2)+pos(3)*pos(3))
              deltarg = - datan( sm_ri(2) / ( satdis - sm_ri(1) ) )
              dt = deltarg / meanmot    
            endif

            if( dabs(dt).gt.tepsil .and. nbiter.lt.maxiter 
     &               .and. ier.eq.0 ) goto 100


! 4. check the validity of the results: t value and geometry earth, M , S
!-------------------------------------------------------------------------

! iterative process does not converge
          if(nbiter.gt.maxiter) then
            ierr = 10

! viewed point corresponds to a time outside the SATPOS interval already read
          else if(ier.ne.0) then
            ierr = 11
          else

! the point is not viewed, the obtained time is not in [tfirst,tlast]
            if( t.lt.(tfirst - ttoler) .or. t.gt.(tlast + ttoler) ) then
              ierr = 1

            else

! the sign + or - of the scalar product MS by MO ( = SM by OM ) indicates 
! whether the angle OMS is lower or higher than pi/2
              scalpro =  sm_g(1) * om_g(1) + sm_g(2) * om_g(2)
     &                 + sm_g(3) * om_g(3)

! angle OMS < pi/2: the point is on other side of earth than the satellite
              if(scalpro.ge.0) then
                ierr = 2

! angle OMS > pi/2: the point is viewed
              else
                ierr = 0
                time = t
              endif

            endif

          endif 

! if the value of t obtained by iteration is correct, saves t and OM values 
          if (ierr .eq. 0) then
            om_old(1) = om_g(1)
            om_old(2) = om_g(2)
            om_old(3) = om_g(3)
            time_old = t
          else
            om_old(1) = 0.d0
            om_old(2) = 0.d0
            om_old(3) = 0.d0
            time_old = 0.d0
          endif   

        return
        end
