        subroutine invlorb(isat,inst,att_mode, att, om_g, tfirst, tlast,
     &                    time, sm_s, ierr)

!+
!NOM/NAME $Id: invlorb.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!
!       calculates the time and scanning angle corresponding to a point
!       on the earth defined by its Greenwich coordinates for a whole orbit
!       (inverse problem of the earth location)
!
!       This routine is restricted to a radiometer scanning in a plane normal 
!       to the satellite velocity. Scanning function number 0 only.
!       
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       Several reference frames are needed for the calculations:
!       - the viewed point is located in Greenwich reference frame (Rg)
!       - viewing direction must be calculated in the instrument fixed reference
!         frame (Ri) to obtain the scaning angle
!       - Ri should be aligned with the satellite reference frame Rs, the
!         difference being the misalignment angles.
!       - Rs should be aligned with the nominal attitude reference frame Ra, the
!         difference being the attitude angles.
!
!       Scanning angle definition:
!       angle from viewing vector to nadir, counted positively according to 
!       the direct orientation of Ra (from Z to X)
!       it is positive on the left of the satellite trajectory
! 
!       Algorithm
!       ---------
!
!       the Ra <--> Rg transformation depends upon the satellite position 
!       and velocity i.e. with time therefore the algorithm is iterative
!
!       determination of the initial time value for the iterative process:
!       the satellite position S is calculated for discrete time values
!       in [tfirst,tlast] separated by deltat
!       the value for which angle OMS (O: earth center, M: viewed point,
!       S: satellite) is the closest to pi is chosen as the initial time value
!        
!       at iteration i:
!       the satellite position and velocity are calculated at time t(i)
!       the Ra <--> Rg transformation is calculated
!       the satellite - viewed point vector SM is calculated in Rg
!       it is transformed into Ra then into Rs using attitude errors
!       then into Ri using misalignment angles
!       the y coordinate of SM in Ri should be 0 (<==> M is in scanning plane)
!       a correction to the satellite argument is obtained from SM coordinates
!       it corresponds to a time correction dt and t(i+1) = t(i) + dt
!
!       iterations are stopped when dt < 0.001 second
!
!       if the iterative process has correctly converged
!       - time is compared to the instants of the first and last line
!       - the angle OMS (O: earth center, M: viewed point, S: satellite)
!       is compared to pi/2 in order to eliminate the cases 
!       where M is on the other side of the earth than S
!
!       Validity limits: 
!       --------------- 
!               tlast - tfirst < nodal period
!
!       if tlast - tfirst < nodal period / 4 :  use invloc 
!       if tlast - tfirst is close to nodal period :  some points of the earth
!       may correspond to 2 solutions one close to tfirst, the other to tlast   
!
!       Remark:
!       ------
!       before calling invlorb, the satpos file must have been read with
!       a time interval larger than [tfirst,tlast] (5 mn before, 5 mn after)
!
!        
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 2.0   01 Jul 2003  adaptation to new sub snagre          A. Marsouin P.Brunel
!       07 Mar 2007  correct several bugs relative to misalignment angles
!                    calculations take into account misalignement angles
!                    2 new arguments, iterations are performed in Ri
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       call intposvel(1,t,pos,vel,orbnum,ier): satellite position and velocity
!         ier = 0  OK           ier = 1  t is out of SATPOS limits
!
!       call snagre(att_mode,pos,vel,Mat_Ra_Rg,ierr): 
!              calculations for Rg --> Ra transformation
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer isat                    ! satellite index in satid file
        integer inst                    ! instrument index for satid
        integer att_mode
        double precision att(3,3)! attitude error matrix                (rad)
        double precision om_g(3)        ! viewed point M in Greenwich ref. frame
        double precision tfirst ! julian instant of the first line (julian day)
        double precision tlast  ! julian instant of the last line  (julian day)
!
! Output
        double precision time   ! viewed point julian instant   (julian day)
! satellite - viewed point vector (SM):
        double precision sm_s(3)        !   in spacecraft fixed ref. frame
        integer ierr            ! return code 
!       0 = OK, the point may be viewed but scnang has to be checked 
!       1 = the point is not viewed, time is not in [tfirst,tlast]
!       2 = the point is not viewed, on other side of earth than satellite
!       10 = the iterative process has not converged
!       11 = viewed point not consistent with the SATPOS interval already read
!       12 = erroneous latitude input
!       13 = [tfirst,tlast] not consistent with the SATPOS interval already read
!       14 = internal error, can not found an initial time value

!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters

        integer maxiter                 ! maximum number of iterations
        parameter (maxiter = 20)

        double precision tepsil         ! time threshold for convergence   (day)
        double precision ttoler         ! tolerance on t in [tfirst,tlast] (day)
        double precision toler0         ! tolerance on t0 in [tfirst,tlast](day)
        parameter (tepsil = 0.001 / 86400.d0,
     &             ttoler = 2.d0 * tepsil,
     &             toler0 = 3.d0 / 1440.d0 )

        double precision deltat         ! time step to calculate initial time
!                                         in [tfirst,tlast]             (day)
        parameter (deltat = 15.d0 / 1440.d0)

        double precision pi, pi_2, twopi! pi,  pi/2, 2*pi
        parameter ( pi = 3.14159265358979311600D0 ,
     &              twopi = pi * 2.d0 ,
     &              pi_2 = pi / 2.d0)

!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Commons
#include "c_satpos.h"
#include "satid.h"
!
! Functions
!
! Local variables (distances in km, angles in radians)
!
        double precision meanmot        ! satellite mean motion  (rad/day)
        double precision t              ! julian instant of current iteration
        double precision t0             ! julian instant to start iterations
        double precision dt             ! correction applied to t
        integer nbiter                  ! number of iterations
        integer ipass                   ! number of passes in iterative process
        logical enditer                 ! if true, end of iterative process

! satellite position and velocity in Greenwich reference frame:
        double precision pos(3)         ! position vector at t          (km)
        double precision vel(3)         ! velocity vector at t          (km/s)
        integer orbnum                  ! orbit number at t
        integer ier                     ! return code for intposvel

        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix

! satellite - viewed point vector (SM):
        double precision sm_g(3)        !   in Greenwich reference frame
        double precision sm_v(3)        !   in vel_vert reference frame

        double precision satdis         ! distance satellite - earth center
        double precision deltarg        ! satellite argument correction
        double precision scalpro        ! scalar product MS by MO
        double precision scalpmin       ! minimum value of scalpro

        double precision nodper         ! approximate nodal period      (day)

        double precision sm_ri(3) ! satellite - viewed point unit vector in Ri
        double precision y_i !
        double precision r_i ! Yaw Roll Pitch misalignment angles
        double precision p_i !
!
!-
!
        ierr = 0
        time = -999.d0

        meanmot = mu / (sp_semaxis*sp_semaxis*sp_semaxis)
        meanmot = dsqrt(meanmot) * 86400.d0

        y_i = mis_align_mode_p(1,inst,isat)
        r_i = mis_align_mode_p(2,inst,isat)
        p_i = mis_align_mode_p(3,inst,isat)

! 2. calculates an initial value of time
!---------------------------------------

          scalpmin = 10.d12
          ierr = 14
          t = tfirst + deltat / 2.d0

          do while ( t.le.tlast )
            call intposvel(1,t,pos,vel,orbnum,ier)
            if ( ier.eq.0) then
              sm_g(1) = om_g(1) - pos(1)
              sm_g(2) = om_g(2) - pos(2)
              sm_g(3) = om_g(3) - pos(3)
              scalpro =  sm_g(1) * om_g(1) + sm_g(2) * om_g(2)
     &                 + sm_g(3) * om_g(3)
              if( scalpro.le.scalpmin ) then
                scalpmin = scalpro
                t0 = t
                ierr = 0
              endif
            else
              ierr = 13
            endif
            t = t + deltat
          end do

! 3. iterative process
!---------------------

          if (ierr.eq.0) then
            enditer = .false.
            ipass = 0
            nodper = twopi / meanmot

            do while ( .not.enditer )  
              dt = 0.d0
              nbiter = 0
              ier = 0
              t = t0
              ipass = ipass + 1

 100          continue
                t = t + dt
                nbiter = nbiter + 1

! calculates the satellite position and velocity
                call intposvel(1,t,pos,vel,orbnum,ier)
                if ( ier.eq.0) then

! calculates the Ra <--> Rg transformation
                  call snagre(att_mode, pos, vel, mat_ra_rg, ierr)

! calculates the SM vector, satellite - viewed point, in Rg
                  sm_g(1) = om_g(1) - pos(1)
                  sm_g(2) = om_g(2) - pos(2)
                  sm_g(3) = om_g(3) - pos(3)

! transforms into Ra
              sm_v(1) = mat_ra_rg(1,1)*sm_g(1)+
     >                  mat_ra_rg(1,2)*sm_g(2)+
     >                  mat_ra_rg(1,3)*sm_g(3)
              sm_v(2) = mat_ra_rg(2,1)*sm_g(1)+
     >                  mat_ra_rg(2,2)*sm_g(2)+
     >                  mat_ra_rg(2,3)*sm_g(3)
              sm_v(3) = mat_ra_rg(3,1)*sm_g(1)+
     >                  mat_ra_rg(3,2)*sm_g(2)+
     >                  mat_ra_rg(3,3)*sm_g(3)


! transforms into Rs using attitude error matrix
                  sm_s(1) = att(1,1)*sm_v(1) + att(1,2)*sm_v(2)
     &                    + att(1,3)*sm_v(3)
                  sm_s(2) = att(2,1)*sm_v(1) + att(2,2)*sm_v(2)
     &                    + att(2,3)*sm_v(3)
                  sm_s(3) = att(3,1)*sm_v(1) + att(3,2)*sm_v(2)
     &                    + att(3,3)*sm_v(3)

! transforms into Ri (instrument frame )using misalignment angles
                  sm_ri(1) =      sm_s(1) + p_i*sm_s(2) - r_i*sm_s(3)
                  sm_ri(2) = -p_i*sm_s(1) +     sm_s(2) + y_i*sm_s(3)
                  sm_ri(3) =  r_i*sm_s(1) - y_i*sm_s(2) +     sm_s(3)

! calculates the correction needed on the satellite argument
                  satdis = dsqrt(pos(1)*pos(1)
     &                          +pos(2)*pos(2)+pos(3)*pos(3))
                  deltarg = - datan( sm_ri(2) / ( satdis - sm_ri(1) ) )
                  dt = deltarg / meanmot        
                endif

              if ( dabs(dt).gt.tepsil .and. nbiter.lt.maxiter 
     &                   .and. ier.eq.0 ) go to 100

! iterative process does not converge
              if( ipass.eq.1 .and. nbiter.gt.maxiter ) then
                ierr = 10
                enditer = .true.

! viewed point corresponds to a time outside the SATPOS interval already read
              else if( ipass.eq.1 .and. ier.ne.0 ) then
                ierr = 11
                enditer = .true.

! viewed point is in [tfirst,tlast]
              else if( t.ge.(tfirst - ttoler)
     &                 .and. t.le.(tlast + ttoler) ) then
                enditer = .true.            
                ierr = 0

! the obtained time is not in [tfirst,tlast]
! if it is the first pass, try another t0
              else 
                ierr = 1
                if( ipass.eq.1 .and.  t.lt.(tfirst - ttoler) 
     &             .and. (t+nodper).le.(tlast + toler0) ) then
                  t0 = t + nodper
                else if( ipass.eq.1 .and.  t.gt.(tlast + ttoler) 
     &                   .and. (t-nodper).ge.(tfirst - toler0) ) then
                  t0 = t - nodper
                else
                  enditer = .true.
                endif

              endif

            end do
            
! 4. tests the viewed point geometry
!-----------------------------------

            if( ierr.eq.0 ) then

! the sign + or - of the scalar product MS by MO ( = SM by OM ) indicates 
! whether the angle OMS is lower or higher than pi/2
              scalpro =  sm_g(1) * om_g(1) + sm_g(2) * om_g(2)
     &                 + sm_g(3) * om_g(3)

! angle OMS < pi/2: the point is on other side of earth than the satellite
              if(scalpro.ge.0) then
                ierr = 2

! angle OMS > pi/2: the point is viewed
              else
                ierr = 0
                time = t
              endif

            endif

          endif


        return
        end
