        subroutine lptoviewvect(
     &        pixel,line,isat,inst,reftime,time,sm_rs,alpha)
!+
!NOM/NAME $Id: lptoviewvect.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!       Convert the line and pixel numbers into the viewed vector in
!       the spacecraft fixed reference frame Rs
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       This routine replaces the old ctimang subroutine 
!           (version 1 of navigation library). 
!
!       The following types of instrument scanning geometry are allowed:
!         0: scanning in a plane, no tilt
!         1: scanning in a plane, with tilt
!         2: conical scanning with or without tilt
!
!       The line and pixel are converted into a time and scanning angle.
!
!       The time and angle give the satellite-viewed point unit vector  
!       in the reference frame Ri which is fixed to the instrument.
!
!       The unit vector is converted from Ri to Rs taking into account
!       the instrument misalignment in yaw roll and pitch.
!          (the misalignment are supposed to be constant).
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995     ctimang     A. Marsouin P.Brunel
! 2.0   01 Jul 2003    new code     A. Marsouin P.Brunel
!       01 Apr 2004    comments changed A. Marsouin P.Brunel
!       15 Jul 2004    Fix declaration of r_i - "\" and "/" characters in
!                      comments were upsetting g77 compiler  N Atkinson
!       07 Mar 2007    correct several bugs relative to misalignment angles
!                      change the misalignment signs
!
        implicit none                                                     
!     
!ARGUMENTS
        real pixel                      ! pixel number
        real line                       ! line number 
        integer isat                    ! satellite index in satid file
        integer inst                    ! instrument index for satid
        double precision reftime        ! julian instant of line 1
!out
        double precision time           ! julian instant of the viewed point
        double precision sm_rs(3)       ! satellite - viewed point unit vector in Rs
        double precision alpha          ! scanning angle (rad)
!       
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
!
! Commons
!
#include "satid.h"
! Functions
!
! Local variables
        double precision timlin
        double precision beta
        double precision delta
        double precision phi
        double precision sm_ri(3) ! satellite - viewed point unit vector in Ri
        double precision y_i !
        double precision r_i ! Yaw Roll Pitch misalignment angles
        double precision p_i ! 

! 
!-
!       
        
        ! 1. calculate viewing vector in Ri frame according to scanning function
        ! Ri   Instrument Reference frame
        !-----------------------------------------------------------------------

        if( inst_scan_mode(inst, isat) .eq. 0 ) then
           ! Scanning in a plane, no tilt
           !
           ! definition of scanning function parameters
           ! 1 = step time per FOV  (jdays)
           ! 2 = offset time between start time and first pixel of a line (jdays)
           ! 3 = center to center FOV step angle (rad)
           ! 4 = pixel number at nadir
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
           timlin = reftime + (line-1) * 
     &             inst_scan_mode_p(5,inst,isat)

           time = timlin + inst_scan_mode_p(2,inst,isat) + 
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           alpha = (pixel-inst_scan_mode_p(4,inst,isat)) * 
     &             inst_scan_mode_p(3,inst,isat)

           ! calculates the satellite - viewed point unit vector in Ri
           sm_ri(1) = dcos(alpha)
           sm_ri(2) = 0.
           sm_ri(3) = dsin(alpha)


        elseif( inst_scan_mode(inst, isat) .eq. 1 ) then
           ! Scanning in a plane with tilt
           ! 
           ! definition of scanning function parameters
           ! 1 = step time per FOV  (jdays)
           ! 2 = offset time between start time and first pixel of a line(jdays)
           ! 3 = center to center FOV step angle (rad)
           ! 4 = pixel number at subtrack
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
           ! 6 = tilt angle between nadir and scanning plane (rad)
           timlin = reftime + (line-1) * 
     &             inst_scan_mode_p(5,inst,isat)

           time = timlin + inst_scan_mode_p(2,inst,isat) + 
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           alpha = (pixel-inst_scan_mode_p(4,inst,isat)) * 
     &             inst_scan_mode_p(3,inst,isat)

           beta = inst_scan_mode_p(6,inst,isat)

           ! calculates the satellite - viewed point unit vector in Ri
           sm_ri(1) = dcos(beta)*dcos(alpha)
           sm_ri(2) = dsin(beta)*dcos(alpha)
           sm_ri(3) = dsin(alpha)

        elseif( inst_scan_mode(inst, isat) .eq. 2 ) then
           ! Conic scanning w/wo tilt
           !
           ! definition of scanning function parameters
           ! 1 = step time per FOV  (jdays)
           ! 2 = offset time between start time and first pixel of a line(jdays)
           ! 3 = center to center FOV step angle, centre to centre FOV (rad)
           ! 4 = pixel number at subtrack
           ! 5 = step time between two lines (jdays)
           ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
           ! 6 = tilt angle between nadir and cone axis (rad)
           ! 7 = half cone angle (rad)
           ! 8 = forward scanning  >= 0
           !     backward scanning < 0
           timlin = reftime + (line-1) * 
     &             inst_scan_mode_p(5,inst,isat)

           time = timlin + inst_scan_mode_p(2,inst,isat) + 
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

           beta  = inst_scan_mode_p(6,inst,isat)
           delta = inst_scan_mode_p(7,inst,isat)
           
           alpha = (pixel-inst_scan_mode_p(4,inst,isat)) * 
     &             inst_scan_mode_p(3,inst,isat)
           
           if ( inst_scan_mode_p(8,inst,isat)  .ge. 0. ) then
              ! forward scanning
              phi = alpha + pi
           else
              ! backward scanning
              phi = alpha
           endif

           ! calculates the satellite - viewed point unit vector in Ri
           sm_ri(1) = dcos(beta)*dcos(delta) - 
     &                          dsin(beta)*dsin(delta)*dcos(phi)
           sm_ri(2) = dsin(beta)*dcos(delta) +
     &                          dcos(beta)*dsin(delta)*dcos(phi)
           sm_ri(3) = dsin(delta)*dsin(phi)

        else
           sm_ri(1) = 0.
           sm_ri(2) = 0.
           sm_ri(3) = 0.
           alpha    = 0.

        end if


        ! 2. calculate satellite - viewed point unit vector in Rs
        ! Rs   Spacecraft-fixed Reference frame
        !--------------------------------------------------------
        if (mis_align_mode(inst,isat) .eq. 0) then
           
           ! Yaw Roll Pitch misalignment definition
           if (mis_align_mode_n(inst,isat) .ge. 3) then
              ! yes 3 parameters
              y_i = mis_align_mode_p(1,inst,isat)
              r_i = mis_align_mode_p(2,inst,isat)
              p_i = mis_align_mode_p(3,inst,isat)
           else
              y_i = 0.
              r_i = 0.
              p_i = 0.
           endif

           sm_rs(1) =      sm_ri(1) - p_i*sm_ri(2) + r_i*sm_ri(3)
           sm_rs(2) =  p_i*sm_ri(1) +     sm_ri(2) - y_i*sm_ri(3)
           sm_rs(3) = -r_i*sm_ri(1) + y_i*sm_ri(2) +     sm_ri(3)
 
        else
           !No misalignment, or unknown
           sm_rs(1) = sm_ri(1)
           sm_rs(2) = sm_ri(2)
           sm_rs(3) = sm_ri(3)
        endif

        return
        end
