        subroutine lptoviewvect_iasi(
     &        pixel,line,isat,inst,reftime,time,so_rs,sm_rs,alpha)
!+
!NOM/NAME $Id: lptoviewvect_iasi.F 84 2007-05-16 14:52:37Z frna $
!
!BUT/PURPOSE 
!       Convert the line and pixel numbers of one IASI scan
!       into the viewed vector of the optical axis and of the 4 IASI pixels
!       in the spacecraft fixed reference frame Rs
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       This routine simply applies the geometrical laws of the IASI
!       scanning. IT CORRESPONDS TO A DRAFT NAVIGATION, the operational
!       IASI navigation will be based on a coregistration with AVHRR
!
!       For the optical axis:
!       The line and pixel are converted into a time and scanning angle.
!       The time and angle give the satellite-viewed point unit vector  
!       in the reference frame Ri which is fixed to the instrument.
!
!       For the 4 pixels:
!       The unit vector in Ri is calculated from:
!       - the time and angle of the optical axis 
!       - the angle between optical axis and each IASI pixel 
!       - an additional angle called "IASI scan azimuth"
!
!       The optical axis and 4 pixel unit vectors are converted 
!       from Ri to Rs taking into account the instrument misalignment
!       in yaw roll and pitch.
!          (the misalignment are supposed to be constant).
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!       01 Apr 2004    comments changed A. Marsouin P.Brunel
!       15 Jul 2004    Fix declaration of r_i - "\" and "/" characters in
!                      comments were upsetting g77 compiler  N Atkinson
!       07 Mar 2007    correct several bugs relative to misalignment angles
!                      change the misalignment signs
!
        implicit none                                                     
!     
!ARGUMENTS
        real pixel                      ! pixel number
        real line                       ! line number 
        integer isat                    ! satellite index in satid file
        integer inst                    ! instrument index for satid
        double precision reftime        ! julian instant of line 1
!out
        double precision time           ! array of julian instant
        double precision so_rs(3)       ! viewing vector for optical axis
        double precision sm_rs(3,4)     ! viewing vector for the 4 pixels
        double precision alpha          ! scanning angle for optical axis (rad)
!       
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)

        double precision pi_7_4, pi_5_4, pi_3_4, pi_1_4
        parameter ( pi_7_4 = 7.*pi/4.)
        parameter ( pi_5_4 = 5.*pi/4.)
        parameter ( pi_3_4 = 3.*pi/4.)
        parameter ( pi_1_4 = 1.*pi/4.)
!
! Commons
!
#include "satid.h"
! Functions
!
! Local variables
        integer i
        double precision timlin
        double precision dzeta      ! angular distance between optical
                                    !   axis and each IASI pixel (rad)
        double precision so_ri(3)   ! optical axis viewing vector in instrument Ref
        double precision sm_ri(3,4) ! viewing vector in instrument reference frame
        double precision y_i !
        double precision r_i ! Yaw Roll Pitch misalignment angles
        double precision p_i !

        double precision phi(4) ! iasi scan azimuth
        data phi/pi_7_4, pi_5_4, pi_3_4, pi_1_4/
! 
!-
!       
        ! This is scanning function number 3
        ! IASI scanning
        !-----------------------------------
        
        ! 1. calculate optical axis viewing vector in Ri frame 
        ! Ri   Instrument Reference frame
        !-----------------------------------------------------

        ! Scanning in a plane, no tilt
        !
        ! definition of scanning function parameters
        ! 1 = step time per FOV  (jdays)
        ! 2 = offset time between start time and first pixel of a line (jdays)
        ! 3 = center to center FOV step angle (rad)
        ! 4 = view number at nadir
        ! 5 = step time between two lines (jdays)
        ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
        ! 6 = angle between optical axis and any of the 4 pixels
        timlin = reftime + (line-1) * 
     &             inst_scan_mode_p(5,inst,isat)

        time = timlin + inst_scan_mode_p(2,inst,isat) + 
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

        alpha = (pixel-inst_scan_mode_p(4,inst,isat)) * 
     &             inst_scan_mode_p(3,inst,isat)

           ! calculates the satellite - viewed point unit vector in Rs
        so_ri(1) = dcos(alpha)
        so_ri(2) = 0.
        so_ri(3) = dsin(alpha)

        ! 2. calculate 4 viewing vectors in Ri frame 
        !-------------------------------------------
        dzeta = inst_scan_mode_p(6,inst,isat)

        do i = 1, 4
           sm_ri(1,i) = dcos(alpha) - dzeta*dsin(phi(i))*dsin(alpha)
           sm_ri(2,i) =               dzeta*dcos(phi(i))
           sm_ri(3,i) = dsin(alpha) + dzeta*dsin(phi(i))*dcos(alpha)
        end do


        ! 3. calculate all viewing vector in Rs
        ! Rs   Spacecraft-fixed Reference frame
        !--------------------------------------
        if (mis_align_mode(inst,isat) .eq. 0) then
           
           ! Yaw Roll Pitch misalignment definition
           if (mis_align_mode_n(inst,isat) .ge. 3) then
              ! yes 3 parameters
              y_i = mis_align_mode_p(1,inst,isat)
              r_i = mis_align_mode_p(2,inst,isat)
              p_i = mis_align_mode_p(3,inst,isat)
           else
              y_i = 0.
              r_i = 0.
              p_i = 0.
           endif

           so_rs(1) =      so_ri(1) + p_i*so_ri(2) - r_i*so_ri(3)
           so_rs(2) = -p_i*so_ri(1) +     so_ri(2) + y_i*so_ri(3)
           so_rs(3) =  r_i*so_ri(1) - y_i*so_ri(2) +     so_ri(3) 

           do i = 1, 4
              sm_rs(1,i)=     sm_ri(1,i)- p_i*sm_ri(2,i)+ r_i*sm_ri(3,i)
              sm_rs(2,i)= p_i*sm_ri(1,i)+     sm_ri(2,i)- y_i*sm_ri(3,i)
              sm_rs(3,i)=-r_i*sm_ri(1,i)+ y_i*sm_ri(2,i)+     sm_ri(3,i)
           end do
           
        else
           !No misalignment, or unknown
           so_rs(1) = so_ri(1)
           so_rs(2) = so_ri(2)
           so_rs(3) = so_ri(3)
           do i = 1, 4
              sm_rs(1,i)= sm_ri(1,i)
              sm_rs(2,i)= sm_ri(2,i)
              sm_rs(3,i)= sm_ri(3,i)
           end do
        endif

        return
        end
