        subroutine nav_1blin(reftime, inst, line_num,
     &          scnlinbit, qualind,
     &          attangtime , rollang , pitchang , yawang,
     &          navstat ,
     &          scalti , loc_ang , geog_ang )
!+
!NOM/NAME $Id: nav_1blin.F 47 2006-08-21 08:44:41Z frna $
!
!BUT/PURPOSE 
!       compute the level 1b variables for ONE scanning line and
!       for one instrument number
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   08 Oct 1997    correct azimuth for Tovs
!       AAPP navigation conventions for azimuth angle are:
!               0 at SOUTH, from -180 to 180, positive towards East 
!       Level 1b navigation conventions for azimuth angle are:
!               0 at NORTH, from -180 to 180, positive towards East 
!       24 Nov 1997  change AAPP azimut definition  A. Marsouin P.Brunel
!               Level 1b definition is adopted
! 1.2   May 1998 change 1B format    Marcel Derrien
!              geo_ang is now integer*4 with scaling factor 1.e+04
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       gp_sb16 gp_sb32         set bit routines of libf7gp 
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
! Inputs:
        double precision reftime! julian instant of the first line (reference)
        integer inst            ! level 1b instrument number
        integer line_num        ! line number
!
! Inputs/Outputs
! Following one-dimension arrays are indexed by line number
        integer*2 scnlinbit(*)  ! scan line bit field
!                bit 15 : (0 = ascending data; 1 = descending data)
        integer qualind(*)      !quality indicator bit field
!                bit 27 : (1 = earth location data not available)
        integer attangtime(*)   !Time associated with attitude angles(seconds)
        integer*2 rollang(*)    !Roll Angle in Degrees
        integer*2 pitchang(*)   !Pitch Angle in Degrees
        integer*2 yawang(*)     !Yaw Angle in Degrees

        integer*4 navstat(*)    ! Navigation Status Bit Field
!             bits 15 - 12: earth location indicator 
!                     (0 = earth location available; 
!                      1 = user ephemeris files greater than 24 hours old;
!                      2 = no earth location available)
!
!             bits 11 - 8: spacecraft attitude control 
!                     (0 = operating inYGC or NOMINAL mode;
!                      1 = operating in another mode;
!                      2 = attitude exceeds nominal tolerance;
!                      3 = both 1 and 2)
!
!            bits 7 - 4: attitude SMODE 
!                     (0 = NOMINAL mode;
!                      1 = rate nulling mode;
!                      2 = YGC mode; 
!                      3 = search mode;
!                      4 = coast mode)
!            bits 3 - 0: attitude PWT1P$AC 
!                     (0 = NOMINAL mode/no test; 
!                      1 = yaw axis test in progress;
!                      2 = roll axis test in progress; 
!                      3 = pitch axis test in progress)
        integer*2 scalti(*)     ! Spacecraft Altitude (MSL) in km     
!                                  in this scan line)
!
! Following two-dimension arrays are indexed by (angle_type,pixel_number)
! and are given for the current line
        integer*2 loc_ang(3,*)  !set of 3 angles in degrees for located points
!            first angle  : 10^2 x (solar zenith angle )
!            second angle : 10^2 x (satellite zenith angle )
!            third angle  : hrpt, gac, avhrr :10^2 x (relative azimuth angle )
!                         : sounders :10^2 x (local azimuth angle )
        integer*4 geog_ang(2,*) !lat/lon pair in degrees for located points
!                first  : 10^4 x (latitude )
!                second : 10^4 x (longitude )
!            (North latitude and East longitude are positive)
!       
!DECLARATIONS
!
!
!
! Local parameters
!
! Commons
! Satpos common
#include "c_satpos.h"
#include "c_1bnav.h"
!
! Functions
!
! Local variables
        integer pix             ! pixel number
        integer orbnum          ! orbit number
        integer ierr            ! return code
        integer reftime_sec     ! seconds of the day for reftime
        integer iqtime(3)       ! year, day_of_year, second of day array
        double precision difazi ! azimuth difference (rad)
        double precision latsat ! satellite geographical latitude (rad)
        double precision lonsat ! satellite geographical longitude (rad)
        double precision alti   ! satellite geographical altitude (km)
        character*22 cht        ! date-time string
        logical first           ! true if first time entering the subroutine
        data first/.true./
! 
        integer satid_sat               ! satellite index in satid file
        integer satid_inst              ! instrument index for satid
        integer att_mode                ! nominal attitude mode from satid file
        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        double precision sm_rs(3)       ! viewing vector for optical axis
        double precision om_g(3)        ! viewed point in Greenwich ref. frame
        double precision sun_g(3)       ! sun position in Greenwich ref. frame
        double precision dist

        ! functions
        integer genscid
        integer genattid
        integer convl1binst
!-
        save first, reftime_sec, satid_sat, att_mode

        if( first ) then
! compute the pixels array:
          do pix = 1, nb_pnav
            pixels(pix) = step_pnav*(pix-1) + first_pnav
          enddo
          call xijq(reftime,iqtime)
          reftime_sec = iqtime(3)

!       get information and identificators from the satid file
          satid_sat  =  genscid(sp_satnam)
          if( satid_sat .eq. 0) then
             call ml_wt('error',
     1    'error in getting satid satellite number for '//sp_satnam)
             call aappexit(1)
          end if

          att_mode =  genattid(satid_sat)

          first=.false.
        endif

        do pix = 1, nb_pnav
          lines(pix) = line_num
        enddo

        ! converts the 1b instrument number onto 
        ! the satid identification number 
        satid_inst = convl1binst(satid_sat, inst)

        do pix = 1, nb_pnav
           ! 1. calculates viewing vectors
           !------------------------------
           call lptoviewvect(
     &        pixels(pix), lines(pix), satid_sat, satid_inst, reftime,
     &        time(pix), sm_rs, scnang(pix))

           ! 2. calculates position velocity
           !--------------------------------------------
           call intposvel(1,time(pix),pos,vel,orbnum,ierr)
           if( ierr .ne. 0 ) then
              call xijc(time(1),cht)
              call ml_wti('error','intposvel return code',ierr)
              call ml_wt('error','intposvel input time '//cht)
              call aappexit(2)
           end if

           ! 3. calculates the Rv <--> Rg transformation
           !--------------------------------------------
           call snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
           if( ierr .ne. 0 ) then
              call ml_wti('error','snagre return code',ierr)
              call ml_wti('error','snagre input att mode ',att_mode)
              call aappexit(2)
              return
           endif

           ! 4. calculates the optical axis coordinates in Rg
           !-------------------------------------------------
           call earthpix( pos, att, Mat_Ra_Rg, sm_rs, om_g, ierr)
           if( ierr .ne. 0 ) then
              call ml_wti('error','earthpix return code',ierr)
              call aappexit(2)
              return
           end if

           ! 5. calculates the viewed point latitude and longitude
           !------------------------------------------------------
           call  cartlalo(1, om_g, lat(pix), lon(pix))

           ! 6. satellite zenith and azimut angles of the viewed point
           !----------------------------------------------------------
           call zenazi(1, lat(pix), lon(pix), pos, 
     &                    satzen(pix), satazi(pix), dist)
           
           ! 7. sun zenith and azimut angles of the viewed point
           ! calculates the sun position and the vector: viewed point-Sun in Rg
           !------------------------------------------------------
           call sungrw(time(pix), sun_g, dist)
           call zenazi(1, lat(pix), lon(pix), sun_g, 
     &                    sunzen(pix), sunazi(pix), dist)
        enddo



! ---------------------------------------------
!  conversions to 1b units and bit flags to set
! --------------------------------------------- 
          
! acsending or descending bit indicator (bit 15)
! we use the last computation of velocity 
               
        if(vel(3).ge.0.d0) then
          call gp_sb16(scnlinbit(line_num),15,0)
        else
          call gp_sb16(scnlinbit(line_num),15,1)
        endif


! set earth location bit to 0 in quality indicator bit field
        call gp_sb32(qualind(line_num),27,0)
     
! attitude variables
        attangtime(line_num) = reftime_sec  
        rollang(line_num)  = roll  * crd  * 1.e+03         
        pitchang(line_num) = pitch * crd  * 1.e+03         
        yawang(line_num)   = yaw   * crd  * 1.e+03 
        
! set all bits in navigation status bit field to zero
        navstat(line_num) = 0

! compute the satellite altitude (in km*10), using the
! last position.
        call cartgeog(pos,latsat,lonsat,alti,ierr)
        if ( ierr.eq.0 ) then
          scalti(line_num) = alti * 10       
        else
          call ml_wti('error','cartgeog return code',ierr)
          call aappexit(2)
        endif

! converts the angles from radians to degrees * 100
        if(inst.le.3) then
          do pix = 1, nb_pnav
            loc_ang(1,pix) = sunzen (pix) * crd * 1.e+02
            loc_ang(2,pix) = satzen (pix) * crd * 1.e+02
            difazi = abs(sunazi(pix)-satazi(pix)) 
            if ( difazi.gt.pi ) difazi = 2*pi - difazi
            loc_ang(3,pix) = difazi*crd * 1.e+02

            geog_ang(1,pix) = lat(pix) * crd * 1.e+04
            geog_ang(2,pix) = lon(pix) * crd * 1.e+04
          enddo

        else
          do pix = 1, nb_pnav
            loc_ang(1,pix) = sunzen(pix) * crd * 1.e+02
            loc_ang(2,pix) = satzen(pix) * crd * 1.e+02
            loc_ang(3,pix) = satazi(pix) * crd * 1.e+02

            geog_ang(1,pix) = lat(pix) * crd * 1.e+04
            geog_ang(2,pix) = lon(pix) * crd * 1.e+04
          enddo

        endif


        return
        end
