
        subroutine cartgeog(pos,lat,lon,alt,ierr)
!+
!NOM/NAME $Id: cartgeog.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       converts cartesian coordinates into geographical coordinates
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       input and output coordinates are expressed in Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!       the latitude and altitude are calculated iteratively
!       the convergence criteria are : 
!         latitude : 10**-7 radian  (6*10**-4 degree)  
!         altitude : 0.01 meter 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision pos(3)         ! cartesian position            (km)
!
! Output
        double precision lat            ! geographic latitude           (rad)
        double precision lon            ! longitude                     (rad)
        double precision alt            ! altitude                      (km)
        integer ierr                    ! return code
!                                         0 = ok  
!                                         1 = iterations have not converged
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Local parameters
        double precision pi             ! 
        parameter ( pi = 3.14159265358979311600D0 )
        double precision epsilh         ! altitude threshold for convergence(km)
        parameter(epsilh = 1.d-5)
        double precision epsill         ! latitude threshold for conver. (rad)
        parameter(epsill = 1.d-7)
        integer maxiter                 ! maximum number of iterations
        parameter(maxiter = 20)
!
! Commons
!
! Functions
!
! Local variables 
        double precision Ni             ! normal to ellipsoid at iteration i
        double precision Lati, hi       ! latitude, altitude at iteration i
        double precision Latim1,Him1    ! latitude, altitude at iteration i-1
        integer nbiter                  ! number of iterations  
        double precision P
!
!-
!
        nbiter = 0
        ierr = 0        
        p = dsqrt( pos(1)*pos(1) + pos(2)*pos(2) )

! the point is on the earth polar axis, no iteration
        if( p .lt. 1.d-5 ) then
          if(pos(3).gt.0d0) then
            lat = pi / 2.d0
            lon = 0.d0
            alt = pos(3) - rpo
          else
            lat = -pi / 2.d0
            lon = 0.d0
            alt = -pos(3) - rpo
          end if

! normal case, iterative calculations
        else
          lon = datan2( pos(2) , pos(1) )
          him1 = dsqrt( pos(2)*pos(2) + pos(3)*pos(3) 
     &               + pos(1)*pos(1) ) - req*dsqrt( 1 - flat )
          latim1= datan( (pos(3)/p) / ( 1.d0- (exct2*req)/(req+him1) ) )
          ni = req / dsqrt( 1.d0 - exct2 * dsin(latim1)*dsin(latim1))
          hi = p / dcos(latim1) - ni
          lati =  datan( (pos(3)/p) / ( 1.d0- (exct2*ni)/(ni+hi) ) )
          nbiter = 1

          do while( ( dabs(hi-him1) .ge. epsilh  .or.
     &                dabs(lati-latim1) .ge. epsill )
     &               .and. nbiter .lt. maxiter)
            nbiter = nbiter + 1
            latim1 = lati
            him1 = hi
            ni = req / dsqrt( 1.d0 - exct2 * dsin(latim1)*dsin(latim1))
            hi = p / dcos(latim1) - ni
            lati =  datan( (pos(3)/p) / ( 1.d0- (exct2*ni)/(ni+hi) ) )
          enddo
          lat = lati
          alt = hi
          if (nbiter.eq.maxiter) ierr = 1
        endif

        return
        end
