        subroutine initrack(lat,lon,alt,pos,mat)
!+
!NOM/NAME $Id: initrack.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE
!
!       calculates station values useful for the tracking angles calculations 
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the station is known in Greenwich reference frame 
!       by its geographic coordinates latitude, longitude, altitude
!       the corresponding cartesian coordinates are calculated
!
!       to obtain the tracking angles (zenith + azimut) the viewing vector
!       must be in the station local reference frame (zenith, south, east)
!       the transformation matrix from Greenwich to local reference frame
!       is therefore calculated
!
!       Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!       Station local reference frame:
!         Z: parallel to the local ascending geographic vertical
!         Y: in horizontal plan, positive toward east
!         X: in horizontal plan, positive toward south
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision lat            ! station geographical latitude (rad)
        double precision lon            ! station longitude             (rad)
        double precision alt            ! station altitude              (km)
!
! Output
        double precision pos(3)         ! station cartesian position    (km)
        double precision mat(3,3)       ! transformation matrix from Greenwich
!                                         to the station local ref. frame   
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Commons
!
! Functions
!
! Local variables
        double precision n              ! normal to ellipsoid at the station
        double precision slat,clat      ! latitude sinus and cosinus 
        double precision slon,clon      ! longitude sinus and cosinus 
! 
!-
!
        slat = dsin(lat)
        clat = dcos(lat)
        slon = dsin(lon)
        clon = dcos(lon)

! Calculates the station cartesian coordinates in Greenwich reference frame

        n = req / dsqrt( 1.d0 - exct2 * slat* slat)
        pos(1) = ( n + alt ) * clat * clon
        pos(2) = ( n + alt ) * clat * slon
        pos(3) = ( n * (1-exct2) + alt ) * slat

! Calculates the transformation matrix from Greenwich reference frame
! to the station local reference frame (u, v, w)
! the matrix lines are the u, v, w coordinates in Greenwich reference frame

! u in horizontal plan, positive toward south
        mat(1,1) =   slat * clon
        mat(2,1) =   slat * slon
        mat(3,1) = - clat

! v in horizontal plan, positive toward east
        mat(1,2) = - slon
        mat(2,2) =   clon
        mat(3,2) =   0.d0

! w parallel to the local ascending geographic vertical
        mat(1,3) =   clat * clon
        mat(2,3) =   clat * slon
        mat(3,3) =   slat

        return

        end
