        subroutine intposvel(nbp,time,pos,vel,orbnum,ierr)
!+
!NOM/NAME $Id: intposvel.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       interpolates the satellite position and (relative) velocity 
!       in Greenwich reference frame in the satpos file
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the satellite position and velocity are expressed  
!       by their cartesian coordinates in Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!       a 3 order polynomial is assumed for each position coordinate
!
!       the SATPOS interval for interpolation is deduced from time
!       if it differs from the last one (corresponding to last call)
!       the polynomial coefficients are calculated 
!       the position and velocity components are interpolated using
!       the new or previous polynomial coefficients
!   
!       output also the orbit number
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
! Inputs
        integer nbp                     ! number of calculations to be done
        double precision time(nbp)      ! julian instants
! Ouptuts
        double precision pos(3,nbp)     ! satellite positions (km)
        double precision vel(3,nbp)     ! satellite relative velocities (km/s)
!               pos and vel are in Greenwich reference frame
        integer orbnum(nbp)             ! orbit number at time
        integer ierr            ! return code
!                               ! 0 = OK
!                               ! 1 = some time values out of SATPOS limits
        
!       
!DECLARATIONS
!
!
!
! Local parameters
!
! Commons
! Satpos common
#include "c_satpos.h"
!
! Functions
!
! Local variables
        integer i               ! loop variable
        integer ip              ! loop variable 
        double precision t      ! time inside SATPOS interval (in seconds)
        double precision t2, t3 ! t2 = t**2   t3 = t**3
        integer nstep           ! highest SATPOS step number preceeding time
!                                 time(nstep) <  time  < time(nstep+1)
        integer nstep_last      ! last value of nstep   
        double precision a3(3)  ! 3rd order polynomial coefficients for x,y,z
        double precision a2(3)  ! 2rd order polynomial coefficients for x,y,z

        data nstep_last/0/
! 
!-
        save nstep_last, a3, a2

        ierr = 0

        do ip=1,nbp

          nstep = idint( (time(ip) - sp_jstart) * 86400.d0 / sp_dt + 1.)
          if( nstep.ge.1.and.nstep.le.(sp_nbli-1)) then

            if( nstep_last.ne.nstep ) then
! the time interval in SATPOS is not the same as in the previous call
! the polynomial coefficients are calculated for the 3 components
              nstep_last = nstep
              do i=1,3
                a2(i)= 3.d0*(sp_pos(i,nstep+1) - sp_pos(i,nstep))/sp_dt2
     >                - (2.d0*sp_vel(i,nstep) + sp_vel(i,nstep+1))/sp_dt 
                a3(i)= 2.d0*(sp_pos(i,nstep) - sp_pos(i,nstep+1))/sp_dt3
     >                + (sp_vel(i,nstep) + sp_vel(i,nstep+1) ) / sp_dt2
              enddo
            endif

! interpolation of the 3 components of the position and velocity vectors        
            t = (time(ip) - sp_jstart) * 86400.d0 - (nstep-1) * sp_dt
            t2 = t * t
            t3 = t * t2
            do i=1,3
              pos(i,ip) = a3(i)*t3 + a2(i)*t2 + sp_vel(i,nstep)*t
     >                    + sp_pos(i,nstep)
              vel(i,ip) = 3.d0*a3(i)*t2 + 2.d0*a2(i)*t + sp_vel(i,nstep)        
            enddo
        
!           finds the orbit number (tests the equator crossing)
            if(pos(3,ip)*sp_pos(3,nstep+1) .gt. 0.) then
              orbnum(ip)=sp_orbnum(nstep+1)
            else
              orbnum(ip)=sp_orbnum(nstep)
            endif

          else
            ierr = 1
          endif

        enddo

        return
        end
