!+ Compute angular position in orbit
!
      SUBROUTINE ORBIT_ANGLE (nlines,year,day_of_year,hour_offset,
     &  scan_time,pos_vector,vel_vector,angle)
      IMPLICIT NONE


c-----------------------------------------
! Name: $Id: orbit_angle.F 576 2018-01-03 10:18:19Z frna $
!
! Purpose:
!   Estimate the angle around the orbit relative to the Earth-Sun ecliptic plane.
!   This can be useful in bias correction.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      16/01/14   N.C.Atkinson
!  1.1      02/01/18   Fix case of d/m >= 1
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Arguments
! Inputs:
      INTEGER nlines
      INTEGER year                !Start year
      INTEGER day_of_year         !Start day
      INTEGER hour_offset         !offset in hours (=12 hours for MWTS/MWHS)
      INTEGER scan_time(nlines)   !milliseconds
      REAL pos_vector(3,nlines)   !metres
      REAL vel_vector(3,nlines)   !m/s

! Outputs:
      REAL angle(nlines)
      
! Local variables

      REAL hour_of_day(nlines),firsthour
      DATA firsthour/-1/
      REAL orbitplane_vector(3)
      REAL solarplane_vector(3)
      REAL refvector(3)
      REAL d, m, distance
      REAL lst, ha, jday
      REAL ecliptic_npole_RA_hours
      DATA ecliptic_npole_RA_hours/18.0/
      REAL ecliptic_npole_dec
      DATA ecliptic_npole_dec/1.16152/   !66.55 deg
      
      INTEGER i

! Functions

      REAL dotprod
      REAL vector_mod

#include "p_earth.h"
#include "constants_preproc.h"

! External calls: latlon_to_xyz, crossprod, dotprod (libatovpp)
!-------------------------------------------------------------------------------------------

!Loop over lines

      DO i=1,nlines

        distance = vector_mod(pos_vector(1,i))
        IF (distance.GT.1.0E7 .OR. distance .LT. 6.0E6) THEN
          angle(i) = 999.0
        ELSE

!Get hour of day and jday since 12h 1 Jan 2000
          hour_of_day(i) = scan_time(i)/3600000.0 + hour_offset
          IF (hour_of_day(i) .GT. 24.0)
     &           hour_of_day(i) = hour_of_day(i) - 24.0
          IF (firsthour .LT. 0) firsthour = hour_of_day(i)
          IF (hour_of_day(i) .LT. firsthour) 
     &           hour_of_day(i) = hour_of_day(i)+24   !it's the next day

          jday = INT((year-1901)*365.25D0 - 36524.99D0)
     &        + day_of_year + 364.5D0
     &        + hour_of_day(i)/24.0

!Local sidereal time and hour angle for ecliptic N pole, hence x,y,z vector

          lst = 100.46 + 0.985647352*jday + hour_of_day(i)*15    !lst in deg
          ha = lst*deg2rad - ecliptic_npole_RA_hours/24.0*2*PI   !ha in radians
          solarplane_vector(1) = cos(ha)*cos(ecliptic_npole_dec)
          solarplane_vector(2) = -sin(ha)*cos(ecliptic_npole_dec)
          solarplane_vector(3) = sin(ecliptic_npole_dec)

!Compute the orbit plane

          CALL crossprod(pos_vector(1,i), vel_vector(1,i),
     &      orbitplane_vector)
        
!Define a reference vector where the orbit plane and solar plane intersect

          CALL crossprod(solarplane_vector, orbitplane_vector,refvector) 

!Find the angle between the position of the satellite and the reference vector

          d = dotprod(pos_vector(1,i),refvector)
          m = vector_mod(pos_vector(1,i))*vector_mod(refvector)
!          IF (abs(d) .GE. m) THEN
!            angle(i) = 180.0
          IF (d/m .GE. 1) THEN
            angle(i) = 0.0
          ELSE IF (d/m .LE. -1) THEN
            angle(i) = 180.0
          ELSE 
            angle(i) = acos(d/m) * rad2deg
          ENDIF

!Finally resolve the sign ambiguity

          if (dotprod(solarplane_vector,pos_vector(1,i)) .lt. 0) 
     &      angle(i) = -angle(i)

        ENDIF
      ENDDO
      END
      
!-------------------------------------------------------------------------------
        
      FUNCTION vector_mod(a)
      real a(3)
      real vector_mod
      vector_mod = sqrt(a(1)*a(1)+a(2)*a(2)+a(3)*a(3))
      END
