       Subroutine read_pm_ut1utc ( iu, jul1950, 
     &              pmx, pmy, ut1utc, pm_ref_day, ier)
  !
  !NOM/NAME  $Id: read_pm_ut1utc.F 26 2006-03-01 12:28:20Z frna $        
  !
  ! Description
  ! Reads a file named 'finals2000A.data' on unit "iu" and extract the values
  ! of Polar motion (arcsec)  and time difference of UT1 and UTC (sec)
  ! corresponding to the input CNES Julian date "jul1950".
  ! 
  ! The file can be retrieved from the ftp site, it is updated weekly (Thursday),
  !  maia.usno.navy.mil/ser7/finals2000A.data
  !
  ! The file format is described on the web site:
  !    http://maia.usno.navy.mil/ser7/readme.finals2000A
  !
  ! Return codes are and output values:
  ! 0 data found in the file                               read values
  ! 1 input date is before 1992                             0.
  ! 2 open failed                                           0.
  ! 3 data file does not contain corresponding date         0.
  ! 4 end of file reached                                   0.
  !
  ! Note that original code is Fortran 90
  !
  ! Definition of dates
  !--------------------
  ! Julian day number (JD):
  ! defined to be 0 for the day starting at Greenwhich mean noon on 1 January 4713 B.C.
  ! 
  ! Modified Julian Date  (MJD):
  ! defined by MJD = JD - 2400000.5 MJD begins at midnight of the civil day
  !
  ! CNES Juilan date:
  ! defined by CNES = JD - 2433282.5  CNES JD begins at midnight of the civil day
  !                                   and 0_CNES = 1 January 1950
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method: The methodology is as follows:
  ! -Open file
  ! -Read until date is found or end of file
  ! -Close file
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !          15/04/2005 Original P. Brunel
  !
  ! Code Description:
  !   Language:           Fortran 77.
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  ! Imported Type Definitions:
  
      Implicit None

      !subroutine arguments:
      ! Intent(in)
       Integer   iu          ! unit number for data file
       Integer   jul1950     ! CNES julian days (day 0 = 01/01/1995)

      ! Intent(out)
       Integer pm_ref_day    ! CNES julian day for pm/ut1_utc

       Real     pmx          ! polar motion x coord (arcsec)
       Real     pmy          ! polar motion y coord (arcsec)
       Real     ut1utc       ! UT1 - UTC (sec)

       Integer   ier            ! return code 0 = OK
                                ! 1 = input date before 1992
                                ! 2 = open failed
                                ! 3 = missing date in file 
                                ! 4 = eof reached 

                                !local variables:
       Integer  ios
       Integer  ios_open
       Integer  ios_close
       Real  s_mjd              ! searched MJD (minus 0.1 days for tests on real values)

  ! format of data file
       Character*120  iers_fmt 

  ! One record of "finals2000A.data" file
       Integer  year
       Integer  month
       Integer  day
       Real  mjd                ! fractional Modified Julian Date (MJD)
       Character*1  bul_a_pm_flag ! IERS (I) or Prediction (P) flag for
                                  ! Bull. A polar motion values
       Real  pm_x               ! Bull. A PM-x (sec. of arc)
       Real  pm_x_error         ! error in PM-x (sec. of arc)
       Real  pm_y               ! Bull. A PM-y (sec. of arc)
       Real  pm_y_error         ! error in PM-y (sec. of arc)
       Character*1  bul_a_ut1_flag ! IERS (I) or Prediction (P) flag for 
                                   ! Bull. A UT1-UTC values
       Real  ut1_utc            ! Bull. A UT1-UTC (sec. of time)
       Real  ut1_utc_error      ! error in UT1-UTC (sec. of time)
       Real  lod                ! Bull. A LOD (msec. of time) -- NOT ALWAYS FILLED
       Real  lod_error          ! error in LOD (msec. of time) -- NOT ALWAYS FILLED
       Character*1 bul_a_nut_flag ! IERS (I) or Prediction (P) flag for
                                  ! Bull. A nutation values
       Real  dx                 ! Bull. A dX wrt IAU2000A Nutation (msec. of arc),
                                !  Free Core Nutation NOT Removed
       Real  dx_error           ! error in dX (msec. of arc)
       Real  dy                 ! Bull. A dY wrt IAU2000A Nutation (msec. of arc),
                                ! Free Core Nutation NOT Removed
       Real  dy_error           ! error in dY (msec. of arc)
       Real  b_pm_x             ! Bull. B PM-x (sec. of arc)
       Real  b_pm_y             ! Bull. B PM-y (sec. of arc)
       Real  b_ut1_utc          ! Bull. B UT1-UTC (sec. of time)
       Real  b_dx               ! Bull. B dX wrt IAU2000A Nutation (msec. of arc)
       Real  b_by               ! Bull. B dY wrt IAU2000A Nutation (msec. of arc)

       Integer jul1950_mjd  ! CNES Julian day 0 in MJD
       Integer cnes1992     ! 01/01/1992 CNES Julian day

       data jul1950_mjd/33282/
       data cnes1992/15340/

!- End of header --------------------------------------------------------

       iers_fmt = '(3i2,1x,f8.2,1x,a1,1x,2f9.6,1x,2f9.6,'//
     &  '2x,a1,2f10.7,1x,2f7.4,2x,a1,1x,2f9.3,1x,2f9.3,'//
     &  '2f10.6,f11.7,2f10.3)'

       ! Before 1992 assume 0.
       If ( jul1950 .lt. cnes1992 ) Then
          Write(*,*)
     &        'Before 1992 Polar motion and UT1-UTC are assigned to 0'
          ier    = 1
          pm_ref_day = 0
          pmx    = 0.
          pmy    = 0.
          ut1utc = 0.
          Return
       End If

       ! conversion from Juilan CNES 1950 to Modified Juilan Days
       ! and use a 0.01 margin for tests on real data
       s_mjd = jul1950 + jul1950_mjd - 0.01
       mjd   = 0.

       ! open file
       Open(unit=iu, file='finals2000A.data', status='old', 
     &       iostat=ios_open)
       If( ios_open .ne. 0 ) Then
          Write(*,*) 
     &     'error opening file finals2000A.data  ios: ',ios_open
          Write(*,*) 
     &     ' error file finals2000A.data not valid for input MJD date ',
     &       Nint(s_mjd)
          Write(*,*) 'Polar motion and UT1-UTC are assigned to 0'
          ier    = 2
          pmx    = 0.
          pmy    = 0.
          ut1utc = 0.
          pm_ref_day = 0
          Return
       End If

       ! loop until date is found
       ios = ios_open

       Do While ( mjd .lt. s_mjd .And. ios .eq. 0 )
          Read(unit=iu, fmt=iers_fmt, iostat=ios) 
     &     year, month, day, mjd, 
     &     bul_a_pm_flag, pm_x, pm_x_error, pm_y, pm_y_error, 
     &     bul_a_ut1_flag, ut1_utc, ut1_utc_error ,
     &     lod, lod_error, 
     &     bul_a_nut_flag, dx, dx_error, dy, dy_error, 
     &     b_pm_x, b_pm_y, b_ut1_utc, b_dx, b_by

       End Do

       ! close file
       Close(unit=iu, iostat=ios_close)

       ! at this point status is I/O error
       ! or found date mjd is higher or equal to search date s_mjd
       If ( ios .eq. 0 ) Then
          If( Abs(mjd - s_mjd) .lt. 0.1) Then
          ! This is the date we whant
             ier    = 0
             pmx    = pm_x
             pmy    = pm_y
             ut1utc = ut1_utc
             pm_ref_day = jul1950
          Else
             ! date not found in file
             Write(*,*) 
     &       'warning file finals2000A.data does not contain MJD date ',
     &        Nint(s_mjd)
             Write(*,*) 'Polar motion and UT1-UTC are assigned to 0'
             ier    = 3
             pmx    = 0.
             pmy    = 0.
             ut1utc = 0.
             pm_ref_day = 0
          End If
       Else
          ! end of file reached or I/O error
          Write(*,*) 
     &     'error file finals2000A.data not valid for input MJD date ',
     &       Nint(s_mjd)
          Write(*,*) 'i/o status ', ios
          Write(*,*) 'Polar motion and UT1-UTC are assigned to 0'
          ier    = 4
          pmx    = 0.
          pmy    = 0.
          ut1utc = 0.
          pm_ref_day = 0
       End If

       Return
       End
