       Subroutine read_tai_utc (iu, jul1950, tai_utc, tai_ref_day, ier)

  !NOM/NAME  $Id: read_tai_utc.F 26 2006-03-01 12:28:20Z frna $        
  !
  ! Description
  ! Reads a file named 'tai-utc.dat' on unit "iu" and extract the
  ! time difference of TAI (Temps Atomique International) and UTC (sec)
  ! corresponding to the nearest preceeding input CNES 1950 Julian date "jul1950".
  ! 
  ! The file can be retrieved from the ftp site,
  !  maia.usno.navy.mil/ser7/tai-utc.dat
  !
  !
  ! Return codes
  ! 0 = OK                value from the file
  ! 1 = date before 1961  value is 0s
  ! 2 = open failed       value is 0s or 33s if input date > 01/01/2006
  ! 3 = reading error     value is 0s or 33s if input date > 01/01/2006
  !
  ! Note that original code is Fortran90
  !
  ! Definition of dates
  !--------------------
  ! Julian day number (JD):
  ! defined to be 0 for the day starting at Greenwhich mean noon on 1 January 4713 B.C.
  !
  ! Modified Julian Date  (MJD):
  ! defined by MJD = JD - 2400000.5 MJD begins at midnight of the civil day
  !
  ! CNES Juilan date:
  ! defined by CNES = JD - 2433282.5  CNES JD begins at midnight of the civil day
  !                                   and 0_CNES = 1 January 1950
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Method: The methodology is as follows:
  ! -Open file
  ! -Read until a date interval is found or end of file
  ! -Close file
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !          15/04/2005 Original P. Brunel
  !
  ! Code Description:
  !   Language:           Fortran 77
  !
  ! Declarations:
  ! Modules used:
  ! Imported Parameters:
  ! Imported Type Definitions:
       Implicit None

  !subroutine arguments:
       !  Intent(in)
       Integer   iu             ! unit number for data file
       Integer  jul1950         ! CNES julian days (day 0 = 01/01/1950)

       ! Intent(out)
       ! Variables for Polar motion and TAI UT1 UTC conversions
       Integer  tai_ref_day     ! CNES julian day for TAI
       Real     tai_utc         ! TAI - UTC (sec)

       Integer  ier             ! return code 0 = OK
                                ! 1 = date before 1961
                                ! 2 = open failed
                                ! 3 = reading error

                                !local variables:
       Character*11  datafile 
       Integer  ios
       Integer  ios_open
       Integer  ios_close
       Integer  ios_read
       Real     s_mjd           ! searched MJD (minus 0.1 days for tests on real values)

       Double Precision  jd     ! Julian Date (JD) MJD = JD - 2400000.5

                                ! One record of "tai-utc.dat" file
       Character*80       line
       Integer            year
       Character*3        month
       Integer            day
       Double Precision   fjd   ! Julian Date (JD)
       Double Precision   prev_fjd ! Julian Date (JD), previous read
       Real               ftaiutc ! TAI - UTC (sec)
       Real               prev_ftaiutc ! TAI - UTC (sec), previous read
       Character*37 iers_fmt 

                                ! conversions between dates
       Integer           cnes1961 ! 01/01/1961 CNES Julian day
       Integer           cnes2006 ! 01/01/2006 CNES Julian day
       Integer           jul1950_mjd ! CNES Julian day 0 in MJD
       Double Precision  cnes_jd ! conversion CNES to Julian Date (JD)
       Double Precision  jd_2006 ! 01/01/2006 in Julian date (JD)
       Double Precision  jd_31122005 ! 31/12/2005 in Julian date (JD)

       Logical  found

       ! reminder format of data file
       ! 1999 JAN  1 =JD 2451179.5  TAI-UTC=  33.0       S + (MJD - 41317.) X 0.0      S
       data datafile /'tai-utc.dat'/
       data iers_fmt /'(1x,i4,1x,a3,1x,i2,5x,f9.1,10x,f12.7)'/
       data cnes1961    /  4018      / ! 01/01/1961 CNES Julian day
       data cnes2006    / 20454      / ! 01/01/2006 CNES Julian day
       data jul1950_mjd / 33282      / ! CNES Julian day 0 in MJD
       data cnes_jd     / 2433282.5d0/ ! conversion CNES to Julian Date (JD)
       data jd_2006     / 2453736.5d0/ ! 01/01/2006 in Julian date (JD)
       data jd_31122005 / 2453735.5d0/ ! 31/12/2005 in Julian date (JD)

!- End of header --------------------------------------------------------

       ! Before 1961 assume 0.
       If ( jul1950 .lt. cnes1961 ) Then
          Write(*,*) 'Before 1961 assume TAI-UTC = 0'
          ier     = 1
          tai_utc = 0.
          tai_ref_day = 0
          Return
       End If

       ! conversion from CNES Julian day to Julian day
       jd = jul1950 + cnes_jd

       ! Take 0.1 day margin for tests
       ! Leap seconds are introduced at 23h59:59 the day before Y/M/D of the file
       ! ie: 1997 JUL  1 means the leap second was the 1997 JUNE 30 23h59m60sec
       jd = jd +0.1

       ! Open file
       Open(unit=iu, file=datafile, status='old', iostat=ios_open)
       If( ios_open .ne. 0 ) Then
          Write(*,*) 'error opening file ',datafile,'  ios: ',ios_open
          If ( jul1950 .gt. cnes2006 ) Then
             Write(*,*) 'TAI-UTC assigned to 33'
             ier     = 2
             tai_utc = 33.
             tai_ref_day = cnes2006
          Else
             Write(*,*) 'TAI-UTC assigned to 0'
             ier     = 2
             tai_utc = 0.
             tai_ref_day = 0
          End If
          Return
       End If

       ios_read = ios_open
       fjd      = 1.e10
       ftaiutc  = 0.
       found    = .False.

       ! search dates interval including input argument date
       Do While ( .Not. found .And. ios_read .eq. 0 )
          prev_fjd = fjd
          prev_ftaiutc = ftaiutc

          Read(iu,'(a80)', iostat=ios_read) line

          If( ios_read .eq. 0 ) Then
             Read(line, fmt=iers_fmt, iostat=ios) 
     &        year, month, day, fjd, ftaiutc
             !   write(*, fmt=iers_fmt) &
             !        & year, month, day, fjd, ftaiutc, ios

             found = prev_fjd .le. jd  .And.  jd .lt. fjd
          End If
       End Do

       Close(unit=iu, iostat=ios_close)

       If ( found ) Then
          ! Everything OK, take start of interval
          ier     = 0
          tai_utc = prev_ftaiutc
          tai_ref_day = prev_fjd - cnes_jd
       Else
          If( ios .eq. 0 .And. prev_fjd .gt. jd_31122005
     &                             .And. prev_fjd .le. jd) Then
          ! Last internal read is OK
          ! Last date in file is after 31/12/2005
          ! Search date is after or equal to 01/01/2006
          ! This means that we are in a "normal" state the required date
          ! is after the last value of the file.
             ier     = 0
             tai_utc = prev_ftaiutc
             tai_ref_day = prev_fjd - cnes_jd
          Else
             ! either ios is invalid or a reading error occured before 
             ! the line dated 01/01/2006
             ! we can assign 33 sec if input date is after 2006
             Write(*,*) 'error reading file ',datafile,
     &                  ' for input J1950 date ', jul1950
             If ( jul1950 .gt. cnes2006 ) Then
                Write(*,*) 'TAI-UTC assigned to 33'
                ier     = 3
                tai_utc = 33.
                tai_ref_day = cnes2006
            Else
                Write(*,*) 'TAI-UTC assigned to 0'
                ier     = 3
                tai_utc = 0.
                tai_ref_day = 0
             End If
          End If
       End If

       Return
       End
