        subroutine snagre(att_mode, pos, vel, Mat_Ra_Rg, ierr)
!+
!NOM/NAME $Id: snagre.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       calculates the conversion matrix between Greenwich reference 
!       frame Rg and nominal attitude reference frame Ra
!
!       Vector Va in Ra is converted into vector Vg in Rg by:
!
!     Vg(1)    | Mat_Ra_Rg(1,1) Mat_Ra_Rg(2,1) Mat_Ra_Rg(3,1) |    Va(1)
!     Vg(2) =  | Mat_Ra_Rg(1,2) Mat_Ra_Rg(2,2) Mat_Ra_Rg(3,2) | x  Va(2)
!     Vg(3)    | Mat_Ra_Rg(1,3) Mat_Ra_Rg(2,3) Mat_Ra_Rg(3,3) |    Va(3)
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!       01 April 2004    comments changed       A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
! no
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer att_mode                ! attitude mode
                                        ! 0 = local normal pointing mode
                                        ! 1 = Yaw steering mode
                                        ! 2 = Geocentric mode
        double precision pos(3)         ! Greenwich satellite position  (km)
        double precision vel(3)         ! Greenwich satellite velocity  (km/s)
!
! Output
        double precision Mat_Ra_Rg(3,3) ! Ra to Rg conversion matrix
        integer ierr                    ! return code 
!       0 = OK  
!       9 = internal error, attitude mode not available
!
!
!DECLARATIONS
!
!
!
! Local parameters
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Local variables
! 
        double precision inervel(3)     ! sat. inertial velocity vector (km/s)
        double precision latgeoc        ! satellite geocentric latitude 
        double precision latgeog        ! satellite geographic latitude 
        double precision slgeoc, clgeoc ! sinus, cosinus latgeoc        
        double precision slgeog, clgeog ! sinus, cosinus latgeog
        double precision slon, clon     ! sinus, cosinus of satellite longitude
        double precision satdis         ! distance satellite - earth center
        double precision n              ! approx. ellipsoid normal for satellite
        double precision h              ! approx. satellite altitude
        double precision x              ! intermediate variable

        double precision u(3)   ! ) units vectors of the Ra
        double precision v(3)   ! ) reference frame expressed in
        double precision w(3)   ! ) Greenwich reference frame
!-
!
        ierr = 0

        ! local normal pointing mode 
        if( att_mode .eq. 0 ) then
           ! calculation of the inertial velocity 
           inervel(1) = vel(1) - earthrot * pos(2)
           inervel(2) = vel(2) + earthrot * pos(1)
           inervel(3) = vel(3)

           ! calculates the satellite geocentric latitude 
           latgeoc = datan2( pos(3), dsqrt(pos(1)*pos(1)+pos(2)*pos(2)))
           clgeoc = dcos(latgeoc)
           slgeoc = dsin(latgeoc)

           ! calculates the satellite geographic latitude 
           ! by an approximation with an accuracy of 10-3 degrees (about 0.1 AVHRR line)
           ! for an exact formulation call cartgeog (navtool
           satdis = dsqrt(pos(1)*pos(1)+pos(2)*pos(2)+pos(3)*pos(3))
           n = req / dsqrt( 1.d0 - exct2 * slgeoc * slgeoc )
           h = satdis - rpo / dsqrt( 1.d0 - exct2 * clgeoc *clgeoc)
           x = 1.d0 - exct2 * n / ( n + h )
           latgeog = datan( ( slgeoc/clgeoc ) / x )
           clgeog = dcos(latgeog)
           slgeog = dsin(latgeog)

           ! calculates the satellite longitude sinus and cosinus
           clon = pos(1) / ( satdis*clgeoc) 
           slon = pos(2) / ( satdis*clgeoc) 

           ! the unit vectors of Ra reference frame are calculated 
           ! in Greenwich reference frame

           ! u parallel to the satellite geographic vertical, toward the earth
           u(1) = - clgeog * clon
           u(2) = - clgeog * slon
           u(3) = - slgeog

           ! w normal to u and to the satellite inertial velocity
           w(1) = - u(2) * inervel(3) + u(3) * inervel(2)
           w(2) = - u(3) * inervel(1) + u(1) * inervel(3)
           w(3) = - u(1) * inervel(2) + u(2) * inervel(1)
           x = dsqrt(w(1)*w(1) + w(2)*w(2) + w(3)*w(3)) 
           w(1) = w(1) / x
           w(2) = w(2) / x
           w(3) = w(3) / x

           ! v = w x u
           v(1) = w(2) * u(3) - w(3) * u(2)
           v(2) = w(3) * u(1) - w(1) * u(3)
           v(3) = w(1) * u(2) - w(2) * u(1)


        ! Yaw steering mode
        elseif( att_mode .eq. 1 ) then

           ! calculates the satellite geocentric latitude 
           latgeoc = datan2( pos(3), dsqrt(pos(1)*pos(1)+pos(2)*pos(2)))
           clgeoc = dcos(latgeoc)
           slgeoc = dsin(latgeoc)

           ! calculates the satellite geographic latitude 
           ! by an approximation with an accuracy of 10-3 degrees (about 0.1 AVHRR line)
           ! for an exact formulation call cartgeog (navtool)
           satdis = dsqrt(pos(1)*pos(1)+pos(2)*pos(2)+pos(3)*pos(3))
           n = req / dsqrt( 1.d0 - exct2 * slgeoc * slgeoc )
           h = satdis - rpo / dsqrt( 1.d0 - exct2 * clgeoc *clgeoc)
           x = 1.d0 - exct2 * n / ( n + h )
           latgeog = datan( ( slgeoc/clgeoc ) / x )
           clgeog = dcos(latgeog)
           slgeog = dsin(latgeog)

           ! calculates the satellite longitude sinus and cosinus
           clon = pos(1) / ( satdis*clgeoc) 
           slon = pos(2) / ( satdis*clgeoc) 

           ! the unit vectors of Ra reference frame are calculated 
           ! in Greenwich reference frame

           ! u parallel to the satellite geographic vertical, toward the earth
           u(1) = - clgeog * clon
           u(2) = - clgeog * slon
           u(3) = - slgeog

           ! w normal to u and to the satellite velocity
           w(1) = - u(2) * vel(3) + u(3) * vel(2)
           w(2) = - u(3) * vel(1) + u(1) * vel(3)
           w(3) = - u(1) * vel(2) + u(2) * vel(1)
           x = dsqrt(w(1)*w(1) + w(2)*w(2) + w(3)*w(3)) 
           w(1) = w(1) / x
           w(2) = w(2) / x
           w(3) = w(3) / x

           ! v = w x u
           v(1) = w(2) * u(3) - w(3) * u(2)
           v(2) = w(3) * u(1) - w(1) * u(3)
           v(3) = w(1) * u(2) - w(2) * u(1)


        ! Geocentric mode
        elseif( att_mode .eq. 2 ) then
           ! calculation of the inertial velocity 
           inervel(1) = vel(1) - earthrot * pos(2)
           inervel(2) = vel(2) + earthrot * pos(1)
           inervel(3) = vel(3)

           satdis = dsqrt(pos(1)*pos(1)+pos(2)*pos(2)+pos(3)*pos(3))

           ! the unit vectors of Ra reference frame are calculated 
           ! in Greenwich reference frame

           ! u parallel to the satellite geographic vertical, toward the earth
           u(1) = - pos(1) / satdis
           u(2) = - pos(2) / satdis
           u(3) = - pos(3) / satdis

           ! w normal to u and to the satellite inertial velocity
           w(1) = - u(2) * inervel(3) + u(3) * inervel(2)
           w(2) = - u(3) * inervel(1) + u(1) * inervel(3)
           w(3) = - u(1) * inervel(2) + u(2) * inervel(1)
           x = dsqrt(w(1)*w(1) + w(2)*w(2) + w(3)*w(3)) 
           w(1) = w(1) / x
           w(2) = w(2) / x
           w(3) = w(3) / x

           ! v = w x u
           v(1) = w(2) * u(3) - w(3) * u(2)
           v(2) = w(3) * u(1) - w(1) * u(3)
           v(3) = w(1) * u(2) - w(2) * u(1)

        else
           ierr = 9
        endif

        ! Calculation of the matrix
        if(ierr .eq. 0 ) then
           mat_ra_rg(1,1) = u(1)
           mat_ra_rg(1,2) = u(2)
           mat_ra_rg(1,3) = u(3)
           mat_ra_rg(2,1) = v(1)
           mat_ra_rg(2,2) = v(2)
           mat_ra_rg(2,3) = v(3)
           mat_ra_rg(3,1) = w(1)
           mat_ra_rg(3,2) = w(2)
           mat_ra_rg(3,3) = w(3)
        else
           mat_ra_rg(1,1) = 0.
           mat_ra_rg(1,2) = 0.
           mat_ra_rg(1,3) = 0.
           mat_ra_rg(2,1) = 0.
           mat_ra_rg(2,2) = 0.
           mat_ra_rg(2,3) = 0.
           mat_ra_rg(3,1) = 0.
           mat_ra_rg(3,2) = 0.
           mat_ra_rg(3,3) = 0.
        end if

        return
        end
