        subroutine sp_read(ijstart,ijend,unit,scname,ierr)
!+
!NOM/NAME $Id: sp_read.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       reads the SATPOS file in unit "unit" between the start and end julian
!       instants. Verifies that the satellite is the correct one.
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       It searches the "real" beginning of the file by comparing the
!       first charcters to the target "#SATPOS"
!       It reads the satellite position-velocity values corresponding to time
!       interval slightly larger than [start time, end time], in order
!       to have a correct interpolation at the limits (sub. intposvel.F).
!       It checks whether the satellite id, period of time and storage
!       are correct.
!       the header variables are stored in the common  /h_satpos/
!       the satpos data for the whole pass are stored in the common /d_satpos/
!       and the data used in each position and velocity interpolation are
!       stored in the common /u_satpos/
!       all commons are included in the file c_satpos.h
!
!       No time check is done if the start time equals 0 (all file processed)
!       No satellite check is done if the scname is 'any'
!   
!       return code 0=OK  
!             1=satellites id diferr    (fatal)
!             2=periods do not mach     (fatal)
!             3=not enough storage      (fatal)         
!             4=IO error                (fatal)  
!             5=start/end not coherent  (fatal)         
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!       August 2000  add test on input dates  P. Brunel
!       Nov 2000     initialize ierr return code at 0  P. Brunel
!       Jul 2003     add error messages if problem on satellite names
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       genscid
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
        double precision ijstart! julian instant of the first pixel
        double precision ijend  ! julian instant of the last pixel
        integer unit            ! logical unit for the satpos file
        character*(*) scname    ! input satellite name
        integer ierr            ! return code 0=OK  
                                !             1=satellites id diferr    (fatal)
                                !             2=periods do not mach     (fatal)
                                !             3=not enough storage      (fatal)
                                !             4=IO error                (fatal)
!       
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision cdr
        parameter (cdr=pi/180.d0)
!
!
! Commons
! Satpos common
#include "c_satpos.h"
!
! Functions
        integer genscid
!
! Local variables
        integer ios                     ! I/O status variable
        integer snumarg                 ! s/c number (relative to the argument)
        integer snumsp                  ! s/c number (relative to the file)
        integer nbsteps                 ! total number of steps of satpos file
        integer step1                   ! first step to store
        integer step2                   ! last step to store
        integer istep                   ! loop step number
        integer fstep                   ! file step number
        integer nsp                     ! number of step stored
        character*8 dummyst             ! dummy string
        character*22 ch_stdate          ! file start date string
        character*22 ch_epoch           ! epoch time string
        character*22 ch_date            ! dummy date string
! 
!-
!       No rewind
!
        dummyst='********'
        ios=0
        ierr = 0

!       finds the satpos target
        do while (ios.eq.0 .and. dummyst(1:7).ne.'#SATPOS' )
          read(unit,'(a)',iostat=ios) dummyst
        end do

        if(ios.ne.0) then
            call ml_wt('error','while decoding the SATPOS file'//
     &        'no #SATPOS target')
            call ml_wt('error','satpos file maybe empty')
            ierr = 4
            return
         endif

!       reads the header and verifies the satellites names
        read(unit,'(20x,a)') sp_satnam
        if (scname(1:3) .ne. 'any' ) then
          snumsp  = genscid(sp_satnam)
          snumarg = genscid(scname)
          if( snumarg .ne. snumsp .or.
     &      snumarg.eq.0 .or. snumsp.eq.0) then
            call ml_wt('error','while decoding the SATPOS file'//
     &        'spacecraft names does not mach')
            call ml_wt('error',scname(1:8)//'   '//sp_satnam)
            call ml_wti('error','id for name in satpos file ',snumsp)
            call ml_wti('error','id for name in argument    ',snumarg)
            ierr = 1
            return
          endif
        endif

        read(unit,'(20x,a)') sp_grsnam
        read(unit,'(20x,a)') ch_stdate
        call xicj(ch_stdate,sp_fstart)
        read(unit,'(20x,f6.2)') sp_nbdays
        read(unit,'(20x,f6.2)') sp_dt
        read(unit,'(20x,a)') sp_bultyp
        read(unit,'(20x,i2)') sp_bulsear
        read(unit,'(20x,a)') sp_bulfilna
        
        read(unit,'(20x,a)') ch_epoch
        call xicj(ch_epoch,sp_epoch)
        read(unit,'(20x,f10.4)') sp_semaxis
        read(unit,'(20x,f10.9)') sp_eccent
        read(unit,'(20x,f10.5)') sp_inclin
        read(unit,'(20x,f10.5)') sp_argper
        read(unit,'(20x,f10.5)') sp_rigasc
        read(unit,'(20x,f10.5)') sp_meanano
        if( sp_semaxis .gt. 10000.) then
           read(unit,'(20x,3(f12.4,1x))') sp_x, sp_y, sp_z
        else
           read(unit,'(20x,3(f10.4,1x))') sp_x, sp_y, sp_z
        endif
        read(unit,'(20x,3(f10.7,1x))') sp_vx, sp_vy, sp_vz
        read(unit,'(20x,f10.4)') sp_latstat
        read(unit,'(20x,f10.4)') sp_lonstat
        read(unit,'(20x,f10.4)') sp_altstat
        read(unit,'(20x,f10.4)') sp_sitestat
        read(unit,'(a)',iostat=ios) dummyst
        sp_inclin = sp_inclin * cdr
        sp_argper = sp_argper * cdr
        sp_rigasc = sp_rigasc * cdr
        sp_meanano = sp_meanano * cdr
        sp_latstat = sp_latstat * cdr
        sp_lonstat = sp_lonstat * cdr
        sp_sitestat = sp_sitestat * cdr


!       test the period of time:
        
        if(ijstart.gt.10227) then  ! January 1st, 1978 (TIROS-N)
!         the input period must be inside the satpos range 
          if(ijstart.lt.sp_fstart .or. 
     &       ijend.gt.sp_fstart+sp_nbdays) then
            call ml_wt('error',
     &                 'periods do not mach when reading satpos')
            call xijc(sp_fstart,ch_date)
            call ml_wt('error','satpos start '//ch_date)
            call xijc(sp_fstart+sp_nbdays,ch_date)
            call ml_wt('error','satpos stop  '//ch_date)
            call xijc(ijstart,ch_date)
            call ml_wt('error','required start '//ch_date)
            call xijc(ijend,ch_date)
            call ml_wt('error','required end   '//ch_date)
            ierr=2
            return
          endif
        elseif(ijstart.ne.0.) then
           ! no tests for start == 0   (means start of file)
          call xijc(ijstart,ch_date)
          call ml_wt('error','invalid start date')
          call ml_wt('error','required start '//ch_date)
          ierr=5
          return
        endif

        if(ijend.lt.ijstart) then
          call ml_wt('error','invalid end date')
          call xijc(ijstart,ch_date)
          call ml_wt('error','required start '//ch_date)
          call xijc(ijend,ch_date)
          call ml_wt('error','required end   '//ch_date)
          ierr=5
          return
        endif

!       number of steps in satpos file
        nbsteps = (sp_nbdays * 86400.d0) / sp_dt + 1

!       computes first step, last step and number of steps to read
!       takes one more step than necessary on each side
        if(ijstart.gt.0.) then
          step1 = idint( (ijstart-sp_fstart) * 86400.d0 / sp_dt )
          step2 = idint( (ijend  -sp_fstart) * 86400.d0 / sp_dt ) +3
          if(step1.le.0) step1 = 1
          if(step2.gt.nbsteps) step2 = nbsteps
        else
          step1 = 1
          step2 = nbsteps
        endif
        sp_nbli = step2 - step1 + 1

        if(sp_nbli.gt.nspmax) then
          call ml_wt('error','not enough storage in c_satpos common')
          call ml_wti('error','maximum satpos records allowed',nspmax)
          call ml_wti('error','satpos records needed',sp_nbli)
          ierr = 3
          return
        endif

        if(step1.gt.1) then
          do istep=1,step1-1
            read(unit,'(a)',iostat=ios) dummyst
          end do
        endif

!       read and store the values
        nsp = 0
        do istep=step1,step2
          nsp = nsp+1
          if( sp_semaxis .gt. 10000.) then
          read(unit,'(i8,3f12.3,3f10.6,i6,2i4)',iostat=ios)
     &                  fstep, 
     &                  sp_pos(1,nsp), sp_pos(2,nsp), sp_pos(3,nsp),
     &                  sp_vel(1,nsp), sp_vel(2,nsp), sp_vel(3,nsp),
     &                  sp_orbnum(nsp),
     &                  sp_satlight(nsp), sp_satvis(nsp)
          else
          read(unit,'(i8,3f10.3,3f10.6,i6,2i4)',iostat=ios)
     &                  fstep, 
     &                  sp_pos(1,nsp), sp_pos(2,nsp), sp_pos(3,nsp),
     &                  sp_vel(1,nsp), sp_vel(2,nsp), sp_vel(3,nsp),
     &                  sp_orbnum(nsp),
     &                  sp_satlight(nsp), sp_satvis(nsp)
          end if
          if(fstep.ne.istep .or. ios.ne.0) then
            call ml_wti('error',
     &                  'I/O error while reading satpos file',ios)
            ierr=4
            return
          endif
        end do

!       compute the real time of the common data
        sp_jstart = sp_fstart + (step1-1) *sp_dt / 86400.d0
        sp_dt2 = sp_dt * sp_dt
        sp_dt3 = sp_dt2 * sp_dt

        return
        end
