        subroutine sunpos(time,pos,dist)
!+
!NOM/NAME $Id: sunpos.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       calculates the sun cartesian coordinates  
!       in an inertial mean-of-date reference frame (IMOD)
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the origin of the calculations is the ESOC software PEPSOC 
!       with the following comments:
!         the sun accuracy is about 0.01 deg.
!
!         ref: Vanflandern, Pulkkinen: low-precision formulae for planetary
!         positions. astroph. j. suppl. 41 pp. 391-411, nov. 1979
!
!       if time value differs from last one (corresponding to last call)
!       by less than 60 s, the sun position is not re-calculated
!       ( induced error: longitude < 0.001 deg  latitude < 0.0003 deg )
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision time   ! julian instant
!
! Output
        double precision pos(3) ! sun position in the mean inertial     (km)
!                                 reference frame of the date            
        double precision dist   ! sun-earth distance                    (km)
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
        double precision twopi          ! = 2 * pi
        parameter ( twopi = 6.283185307179586476d0 )
! 
        double precision cecl,secl      ! cosinus and sinus of the
!                                         obliquity of the ecliptic
        parameter ( cecl = 0.91747755, secl = 0.39778757 )
!       the value adopted is 23.439941 deg corresponding to 01/01/95
!       the variation is -0.000356 deg / 1000 days
!
        double precision dtime          ! time difference in day
        parameter (dtime = 60d0 / 86400.d0)
!
! Commons
!
! Functions
!
! Local variables
        double precision time_last      ! last value of time
        double precision pos_last(3)    ! last value of pos
        double precision dist_last      ! last value of dist
        double precision jul    ! julian instant starting on 1 January 2000, 12h
        double precision x7     ! mean solar longitude in the ecliptic
        double precision x8     ! sun mean anomaly
        double precision c8, s8 ! cosinus and sinus of x8
        double precision as     ! apparent solar longitude in the ecliptic
        double precision ry

        data time_last/0.d0/
! 
!-
!
        save time_last, pos_last, dist_last

        if ( dabs(time - time_last).gt.dtime ) then

!       time value differs from last one, the sun position is calculated
          time_last = time
          jul = time - 18262.5 d0
          x7 = dmod(0.779072 d0 + 0.273790931 d-2 * jul, 1.d0) * twopi
          x8 = dmod(0.993126 d0 + 0.27377785 d-2 * jul, 1.d0) * twopi
          s8 = dsin(x8)
          c8 = dcos(x8)
          as = x7 + s8 * (335.d-4 + 698.d-6 * c8)
          dist_last = 0.1496190d9 - 0.2505768d7 * c8
          pos_last(1) = dist_last * dcos(as)
          ry = dist_last * dsin(as)
          pos_last(2) = cecl*ry
          pos_last(3) = secl*ry
        endif

        dist = dist_last
        pos(1) = pos_last(1)
        pos(2) = pos_last(2)
        pos(3) = pos_last(3)

        return
        end
