        subroutine sunriset(julday,elev,nbp,lat,lon,timris,timset,ierr)
!+
!NOM/NAME $Id: sunriset.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       Calculates the sunrise and sunset times at a date for a list of points
!       (sunrise and sunset are defined by a given sun elevation)
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
! 
!       For each point on the earth surface:
!       - first, approximate sunrise and sunset times are calculated,
!       - then, separately for sunrise and sunset, an exact value is calculated
!       with an iterative scheme based on the direct calculation of the sun
!       position in Greenwich refrence frame, by subroutine sungrw.
!
!       Remarks:
!       -------
!
!       The absolute value of input argument elev should be low 
!       (corresponding to a few degrees)
!
!       Depending on the longitude,  sunset time < sunrise time  is possible
!
!       When day and night durations vary rapidly:
!       - only a sunrise or only a sunset may occur at a date
!         these cases are indicated by return code 300 or 400
!       - 2 sunset or 2 sunrise may occur on a day, only one is calculated
!         these cases are not indicated by a specific return code value
!         they can be noticed by calling sunriset on consecutive dates
!
!       Internal errors are expected to occur only close to the poles
!       or if abs(elev) is too high
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!       06 Jan 2000     modifications when no sunset or no sunrise  
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       xdjq : conversion julian date to (year,day of year)
!       initrack: calculates station values useful for the tracking angles
!       suniter: iterative calculation of a exact sunrise or sunset time
!       sungrw: calculates the sun position in Greenwich reference frame
!       trackang: calculates the zenith and azimut angles of a celestial object
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer julday                  ! julian day
        double precision elev           ! sun elevation which defines   (rad)
!                                         sunset and sunrise
        integer nbp                     ! number of points on earth surface
        double precision lat(nbp)       ! geographic latitudes          (rad)
        double precision lon(nbp)       ! longitudes                    (rad)
!
! Output
        double precision timris(nbp)    ! sunrise julian instants       (day)
        double precision timset(nbp)    ! sunset julian instants        (day)
        integer ierr(nbp)               ! return code
!       0  = ok  
!       100  = permanent day
!       200  = permanent night
!       300  = no sunrise on julday,  sunrise time is given for julday-1
!       400  = no sunset  on julday,  sunset  time is given for julday+1
!       internal errors:
!       10 = in the approximate sunrise time
!        1 = in the approximate sunset time
!       11 = in the approximate sunrise and sunset times
!       30 = reaches the maximum iteration number for sunrise
!        3 = reaches the maximum iteration number for sunset
!       33 = reaches the maximum iteration number for sunrise and sunset
!       15 = input latitude  is not in [-pi/2 , pi/2]
!       16 = input longitude is not in [-2*pi , 2*pi]
!       17 = sunrise time and sunset time have been found out of the day
!       18 = failure on sunrise calculation
!       19 = failure on sunset calculation
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision twopi
        parameter ( twopi = 2.d0 * pi)
        double precision pis2
        parameter ( pis2 = pi / 2.d0)
        double precision earthrev       ! speed of earth revolution around sun 
        parameter ( earthrev = twopi / 360.25 ) 
        double precision dlat           ! latitude deviation for checking
!                                         permanent day or night
        parameter (dlat=pi*1.d0/180.d0) ! in radians (<==> 1 degree )
        double precision durmin         ! minimum duration for day or night
        parameter (durmin=1.d0/1440.d0) ! in day   (<==> 1 minute)
!
! Commons
!
! Functions
!
! Local variables
        integer yearday(2)              ! date expressed in year and day of year
        double precision ssunlat        ! sinus of approximate sun latitude
        double precision csunlat        ! cosinus of approximate sun latitude
        double precision eqt            ! time factor to obtain sun longitude
        double precision lati           ! latitude of the current point
        double precision long           ! longitude of the current point
        double precision pos(3)         ! cartesian position of the current point
        double precision mat(3,3)       ! transformation matrix from Greenwich
!                                         to the local ref. frame at the current point
        double precision hrise          ! approximate sunrise hour      (day)
        double precision hset           ! approximate sunset hour       (day)
        double precision t0             ! sunrise or sunset approximate time
        double precision s,y,la,diflon  ! intermediate variables
        double precision trise          ! sunrise julian instant        (day)
        double precision tset           ! sunset julian instant         (day)
        double precision t2             ! intermediate julian instant   (day)
        double precision posgrw(3)      ! sun position in Greenwich ref. frame
        double precision dist           ! sun-earth distance
        double precision zen            ! sun zenith angle              (rad) 
        double precision azi            ! sun azimut angle              (rad) 
        integer i                       ! loop variable
        integer ierc                    ! return code of the current point
        integer riset                   ! indicator for sunrise or sunset
!               riset = 1 if sunrise  or  riset = -1 if sunset  
        integer ierise                  ! return code for suniter at sunrise
        integer ierset                  ! return code for suniter at sunset
        integer ier2                    ! intermediate return code for suniter 

!-
!
! 0. calculates approximate sun parameters of the day
!----------------------------------------------------
        call xdjq(julday,yearday)
        s = dsin(earthrev * (yearday(2) - 2))
        ssunlat = 0.398d0 * dsin(earthrev *(yearday(2)-81+2.d0*s) )
        csunlat = dsqrt( 1.d0 - ssunlat * ssunlat )
        eqt = ( 0.128*s + 0.164*dsin(earthrev*(yearday(2)+10)) ) / 24.d0

        do i = 1,nbp
          trise = -999.
          tset = -999.
          ierc = 0
          lati = lat(i)
          long = lon(i)
          if( dabs(lati).gt.pis2 ) ierc = 15
          if( dabs(long).gt.twopi ) ierc = 16
          call initrack(lati,long,0.d0,pos,mat)

          if( ierc.eq.0 ) then

! 1. calculates approximate times for sunset and sunrise
!-------------------------------------------------------
            y = (dsin(elev) - ssunlat*dsin(lati)) / (csunlat*dcos(lati))

! identify permanent day or night
            if ( y .lt. -1 ) then
              ierc = 400
              la = lati - dlat
              if( lati.lt.0.d0 ) la = lati + dlat
              y = (dsin(elev) - ssunlat * dsin(la)) / (csunlat*dcos(la))
              if ( y .lt. -1 )  ierc = 100
            else if ( y .gt. 1 ) then
              ierc = 500
              la = lati - dlat
              if( lati.lt.0.d0 ) la = lati + dlat
              y = (dsin(elev) - ssunlat * dsin(la)) / (csunlat*dcos(la))
              if ( y.gt. 1 ) ierc = 200
            endif

! calculates sunrise and sunset times
            if( ierc.eq.0  .or. ierc.ge.400 ) then
              diflon = dacos(y)
              hrise = 0.5d0 + eqt - (long + diflon) / twopi
              hset  = 0.5d0 + eqt - (long - diflon) / twopi
              if( hrise.lt.0.d0 )  hrise = hrise + 1.d0
              if( hrise.gt.1.d0 )  hrise = hrise - 1.d0
              if( hset .gt.1.d0 )  hset  = hset  - 1.d0
              if( hrise.lt.0.d0 .or. hrise.gt.1.d0 ) then
                ierc = 10
                if( hset .lt.0.d0 .or. hset .gt.1.d0 ) ierc = 11
              else if( hset .lt.0.d0 .or. hset .gt.1.d0 ) then
                ierc = 1
              endif
            endif

          endif
          
          if( ierc.eq.0 .or. ierc.ge.400 ) then

! 2. sunrise calculation
!-----------------------
            t0 = julday + hrise
            riset = 1
            call suniter(t0,riset,elev,pos,mat,trise,ierise)
            if(ierise.eq.0 .and. trise.lt.julday ) then
              t0 = trise + 1.d0
              call suniter(t0,riset,elev,pos,mat,t2,ier2)
              if(ier2.eq.0 .and. t2.lt. (julday+1) ) then
                trise = t2
                ierise = ier2
              endif
            else if(ierise.eq.0 .and. trise.ge. (julday+1) ) then
              t0 = trise - 1.d0
              call suniter(t0,riset,elev,pos,mat,trise,ierise)
            else if(ierise.ne.0 .and. hrise.le.0.125) then
              t0 = t0 + 1.d0
              call suniter(t0,riset,elev,pos,mat,trise,ierise)
            else if(ierise.ne.0 .and. hrise.ge.0.875) then
              t0 = t0 - 1.d0
              call suniter(t0,riset,elev,pos,mat,trise,ierise)
            endif

! 3. sunset calculation
!-----------------------
            t0 = julday + hset
            riset = -1
            call suniter(t0,riset,elev,pos,mat,tset,ierset)
            if(ierset.eq.0 .and. tset.lt.julday ) then
              t0 = tset + 1.d0
              call suniter(t0,riset,elev,pos,mat,tset,ierset)
            else if(ierset.eq.0 .and. tset.ge. (julday+1) ) then
              t0 = tset - 1.d0
              call suniter(t0,riset,elev,pos,mat,t2,ier2)
              if(ier2.eq.0 .and. t2.ge.julday ) then
                tset = t2
                ierset = ier2
              endif
            else if(ierset.ne.0 .and. hset.le.0.125) then
              t0 = t0 + 1.d0
              call suniter(t0,riset,elev,pos,mat,tset,ierset)
            else if(ierset.ne.0 .and. hset.ge.0.875) then
              t0 = t0 - 1.d0
              call suniter(t0,riset,elev,pos,mat,tset,ierset)
            endif

! 4. checks the results and calculates the return code
!-----------------------------------------------------
            if( ierise.eq.0 .and. ierset.eq.0
     &          .and. dabs(tset-trise).lt.durmin ) then
              trise = -999.d0
              tset = -999.d0
              t0 = trise + 1.d0/24.d0
              call sungrw(t0,posgrw,dist)
              call trackang(pos,mat,1,posgrw,zen,azi,dist)
              if( zen .le. pis2 ) then
                ierc = 100
              else
                ierc = 200
              endif     
            else if(ierc.ge.400 .and. ierise.eq.0.and.ierset.eq.0) then
                ierc = 0
            else if(ierc.ge.400) then
                ierc = ierc - 300
                trise = -999.d0
                tset = -999.d0
            else if( (ierise.eq.1 .or. ierise.eq.2)
     &                .and. ierset.eq.ierise ) then
              ierc = ierise * 100 
            else if( (ierise.eq.1 .or. ierset.eq.1) .and. 
     &               (dabs(hset-hrise).lt.1./24.d0 .or. 
     &                dabs(hset-hrise).gt.23.d0/24.d0) ) then
              trise = -999.d0
              tset = -999.d0
              ierc = 100 
            else if( (ierise.eq.2 .or. ierset.eq.2) .and. 
     &               (dabs(hset-hrise).lt.1./24.d0 .or.
     &                dabs(hset-hrise).gt.23.d0/24.d0) )then
              trise = -999.d0
              tset = -999.d0
              ierc = 200 
            else if(ierise.eq.3 .or. ierset.eq.3) then
              ierc = ierise * 10 + ierset
            else if( ierise.ne.0) then
              ierc = 18   
            else if( ierset.ne.0) then
              ierc = 19   
            else if( (trise.lt.julday .or. trise.ge. (julday+1)) .and.
     &               ( tset.lt.julday .or.  tset.ge. (julday+1)) ) then
                trise = -999.d0
                tset  = -999.d0
                ierc = 17
            else if( trise.lt.julday ) then
              ierc = 300
            else if( trise.ge.(julday+1) ) then
              trise = -999.d0
              ierc = 18
            else if( tset.lt.julday ) then
              tset = -999.d0
              ierc = 19
            else if( tset.ge. (julday+1) ) then
              ierc = 400
            endif

          endif

          ierr(i) = ierc
          timris(i) = trise
          timset(i) = tset
        end do

        return
        end

        subroutine suniter(t0,riset,elev,pos,mat,t,ierr)
!+
!NOM/NAME $Id: sunriset.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       Iterative calculation of a exact sunrise or sunset time
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       
!       This routine is designed to be called by subroutine suniter.
!
!       The iterative scheme based on the direct calculation of the sun
!       position in Greenwich refrence frame, by subroutine sungrw.
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!       06 Jan 2000  convergence threshold changed: 0.001 deg instead of 0.01
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!       sungrw: calculates the sun position in Greenwich reference frame
!       trackang: calculates the zenith and azimut angles of a celestial object
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision t0             ! initial julian time (approx.) (day)
        integer riset                   ! indicator for sunrise or sunset
!               riset = 1 if sunrise  or  riset = -1 if sunset  
        double precision elev           ! sun elevation which defines   (rad)
!                                         sunset and sunrise
        double precision pos(3)         ! cartesian position for (lat,lon)
        double precision mat(3,3)       ! transformation matrix from Greenwich
!                                         to the local ref. frame at (lat,lon)
!
! Output
        double precision t              ! final julian time (exact)     (day)
        integer ierr                    ! return code
!        0  = ok  
!        1 = failure when initializing the iterative scheme, always day
!        2 = failure when initializing the iterative scheme, always night
!        3 = reaches the maximum iteration number
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision pis2
        parameter ( pis2 = pi / 2.d0)
        double precision cdr
        parameter (cdr=pi/180.d0)

! the iterative scheme uses two times: t1,t2 and the solution is in [t1,t2]
        double precision dtmax          ! maximum range for initial [t1,t2]
        parameter (dtmax= 4.d0/ 24.d0)  ! in day   (<==> 4 hours)
        double precision dt             ! time step used to obtain initial t2
        parameter (dt=10.d0/1440.d0)    ! in day   (<==> 10 minutes)
        integer maxiter                 ! maximum number of iterations
        parameter (maxiter=20)
        double precision epsiz          ! zenith ang. threshold for convergence
        parameter (epsiz = 0.001 * cdr) ! in radians (<==>0.001 degree)
!
! Commons
!
! Functions
!
! Local variables
        double precision t1,t2,t3       ! sunrise or sunset times used in the
!                                         iterative interpolation scheme
        double precision posgrw(3)      ! sun position in Greenwich ref. frame
        double precision dist           ! sun-earth distance
        double precision delev1         ! sun elevation difference at t1 (rad)
!                                         ( = sun_elevation(t1) - elev ) 
        double precision delev2,delev3  ! sun elevation differences at t2,t3  
        double precision zen            ! sun zenith angle              (rad) 
        double precision azi            ! sun azimut angle              (rad) 
        integer nbiter                  ! number of iterations
        integer sign                    ! -1 or 1, used to obtain initial t2

! 
!-

! finds two times situated before and after sunrise or sunset
!
          ierr = 0
          t1 = t0
          call sungrw(t1,posgrw,dist)
          call trackang(pos,mat,1,posgrw,zen,azi,dist)
          delev1 = pis2 - zen - elev
          sign = riset
          if( delev1.ge.0.d0 ) sign = -riset
          t2 = t1
          delev2 = delev1
          do while ( delev1*delev2.gt.0.d0 .and. abs(t2-t1).le.dtmax )
            t2 = t2 + sign*dt
            call sungrw(t2,posgrw,dist)
            call trackang(pos,mat,1,posgrw,zen,azi,dist)
            delev2 = pis2 - zen - elev
          end do
        
! iterative scheme 
!
          if( delev1*delev2.le.0.d0 ) then
            t3 = t1 -  delev1 * (t2 - t1) / (delev2 - delev1)
            call sungrw(t3,posgrw,dist)
            call trackang(pos,mat,1,posgrw,zen,azi,dist)
            delev3 = pis2 - zen - elev
            nbiter = 1
            do while ( dabs(delev3).gt.epsiz .and. nbiter.lt.maxiter )
              if( delev3 * delev1 .gt. 0.d0) then
                t1 = t3
                delev1 = delev3
              else
                t2 = t3
                delev2 = delev3
              endif
              t3 = t1 -  delev1 * (t2 - t1) / (delev2 - delev1)
              call sungrw(t3,posgrw,dist)
              call trackang(pos,mat,1,posgrw,zen,azi,dist)
              delev3 = pis2 - zen - elev
              nbiter = nbiter + 1
            end do
            if( nbiter.le.maxiter ) then
              t = t3
            
! reaches the maximum iteration number in the iterative scheme
            else
              ierr = 3
              t = -999.d0
            endif

! failure when initializing the iterative scheme 
          else
            if( delev1.ge.0.d0 ) then
              ierr = 1
            else
              ierr = 2
            endif
            t = -999.d0
          endif

        return
        end

