        subroutine sunsat(satpos,sunpos,daynight)
!+
!NOM/NAME $Id: sunsat.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE
!
!       calculates whether the satellite is lit by the sun or in earth umbra 
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the satellite N and sun S positions are expressed in a reference frame
!       with origin O = earth center and Z axis = earth polar axis
!
!       if the satellite N is in earth umbra:
!       the direction satellite-sun intersects the earth ellipsoid at 
!       a point M which is situated between N and S
!       the satellite-sun direction (=NM direction) is known
!       the distance D=NM is unknown
!          vectorOM = vectorON + D * unitvectorNS
!       OM is on the earth ellipsoid therefore
!       D is the smallest positive root of a 2 order equation
!               a * x**2  + b * x + c = 0
!
!       the condition "satellite is in earth umbra" corresponds to
!               discriminant > 0 and b < 0
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision satpos(3)      ! satellite position 
        double precision sunpos(3)      ! sun position 
!       satpos and sunpos are in a reference frame with Z = earth polar axis
!
! Output
        integer daynight                ! day/nigth code for the satellite
!                                         = 0 day, satellite lit by sun
!                                         = 1 night, satellite in earth umbra
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
! Commons
!
! Functions
!
! Local variables
        double precision dist           ! satellite-sun distance
        double precision nsu(3)         ! satellite-sun unit vector
        double precision a,b,c,delta
! 
!-
!
! calculates the unit vector of the direction satellite-sun
        nsu(1) = sunpos(1) - satpos(1)
        nsu(2) = sunpos(2) - satpos(2)
        nsu(3) = sunpos(3) - satpos(3)
        dist = dsqrt( nsu(1)*nsu(1) + nsu(2)*nsu(2) + nsu(3)*nsu(3) )
        nsu(1) = nsu(1) / dist
        nsu(2) = nsu(2) / dist
        nsu(3) = nsu(3) / dist

! calculates the coefficients of the 2nd order equation 
        a = 1.d0 + (re2_rp2 - 1.d0) * nsu(3)*nsu(3)
        b = satpos(1) * nsu(1) + satpos(2) * nsu(2) 
     >      + re2_rp2 * satpos(3) * nsu(3)
        c = satpos(1) * satpos(1) + satpos(2) * satpos(2) 
     >      + re2_rp2 * satpos(3) * satpos(3) - re2
        delta = b * b - a * c
  
! the direction satellite-sun intersects the earth between N and S
        if (delta.ge.0.and.b.lt.0) then
          daynight = 1

! the direction satellite-sun does no intersect the earth 
        else
          daynight = 0
        endif

        return
        end
