        subroutine trackang(pos,mat,nbp,celobj,zen,azi,dist)
!+
!NOM/NAME $Id: trackang.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE
!
!       calculates the zenith and azimut angles of a celestial object 
!       viewed from a station on the earth
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       station and celestial object are located in Greenwich reference frame
!
!       the viewing vectors, station - celestial object, are calculated 
!       in the station local reference frame (zenith, south, east)
!       using the transformation matrix calculated by subroutine initrack 
!
!       the zenith and azimut are deduced from the viewing vector coordinates
!
!       Greenwich reference frame:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!       Station local reference frame:
!         Z: parallel to the local ascending geographic vertical
!         Y: in horizontal plan, positive toward east
!         X: in horizontal plan, positive toward south
!
!       AAPP conventions for azimut angle are those of Level 1b:
!               0 at NORTH, from -180 to 180, positive towards East 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!       08 Oct 1997  additional tests at zenith and nadir  A. Marsouin P.Brunel
!       24 Nov 1997  change azimut definition  A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision pos(3)         ! station cartesian position      (km)
        double precision mat(3,3)       ! transformation matrix from Greenwich
!                                         to the station local ref. frame
        integer nbp                     ! number of calculations to be done
        double precision celobj(3,nbp)  ! positions of a celestial object (km)
!                                         in Greenwich reference frame
! Output
        double precision zen(nbp)       ! celestial object zenith angles  (rad) 
        double precision azi(nbp)       ! celestial object azimut angles  (rad) 
        double precision dist(nbp)      ! station - object distances      (km)
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi             ! pi
        parameter ( pi = 3.14159265358979311600D0 )
        double precision epsil,epsnad   ! thresholds for zenith and nadir
        parameter (epsil = 1.d-05, epsnad = pi - epsil)
!
! Commons
!
! Functions
!
! Local variables
        integer i                       ! loop variable
        double precision x,y,z          ! viewing vector in Greenwich
        double precision xl,yl,zl       ! viewing vector in local ref. frame
! 
!-
!
        do i=1,nbp

! calculates the viewing vector: station-celestial object
          x = celobj(1,i) - pos(1)
          y = celobj(2,i) - pos(2)
          z = celobj(3,i) - pos(3)

! converts this vector into the station local reference frame
          xl = mat(1,1) * x +  mat(2,1) * y + mat(3,1) * z
          yl = mat(1,2) * x +  mat(2,2) * y + mat(3,2) * z
          zl = mat(1,3) * x +  mat(2,3) * y + mat(3,3) * z

! calculates the zenith and azimut angles
          dist(i) = dsqrt( xl*xl + yl*yl + zl*zl )
          zen(i) = dacos( zl/dist(i) )
          azi(i) = pi - datan2( yl , xl )
          if(azi(i).gt.pi) azi(i) = azi(i) -2*pi
          
          if( zen(i).lt.epsil ) then
            zen(i) = 0.d0
            azi(i) = 0.d0
          else if( zen(i).gt.epsnad ) then
            zen(i) = pi
            azi(i) = 0.d0
          endif
          
        enddo

        return
        end
