        subroutine zenazi(nbp, lat, lon, pos, zen, azi, dist)
!+
!NOM/NAME $Id: zenazi.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!
!       calculates the zenith angle, azimut angle and distance of  
!       a celestial object viewed from a point on the earth surface (alt=0)
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
! 
!       Greenwich reference frame Rg:
!         X: in equatorial plane toward Greenwich meridian
!         Y: in equatorial plane toward lon = 90 degrees east
!         Z: terrestrial polar axis
!
!       Point local reference frame Rl:
!         Z: parallel to the local ascending geographic vertical
!         Y: in horizontal plan, positive toward east
!         X: in horizontal plan, positive toward south
!
!       AAPP conventions for azimut angle are those of Level 1b:
!               0 at NORTH, from -180 to 180, positive towards East 
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
! lalocart
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer nbp                     ! number of calculations to be done
        double precision lat(nbp)       ! array of geographic latitudes (rad)
        double precision lon(nbp)       ! array of longitudes           (rad)
        double precision pos(3,nbp)     ! array of cartesian positions 
                                        !      in Rg of celestial object (km)
!
! Output
        double precision zen(nbp)       ! celestial object zenith angles  (rad) 
        double precision azi(nbp)       ! celestial object azimut angles  (rad) 
        double precision dist(nbp)      ! point - object distances        (km)
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
        double precision pi             ! pi
        parameter ( pi = 3.14159265358979311600D0 )
        double precision epsil,epsnad   ! thresholds for zenith and nadir
        parameter (epsil = 1.d-05, epsnad = pi - epsil)
!
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
!
! Local variables
        integer i                       ! loop variable
        integer ierr
        double precision slatm,clatm    ! point sinus & cosinus latitude
        double precision slonm,clonm    ! point sinus & cosinus longitude

        double precision ul_g(3)        ! ) units vectors of the point
        double precision vl_g(3)        ! ) local reference frame expressed in
        double precision wl_g(3)        ! ) Greenwich reference frame
        double precision ms_l(3)        ! point - object vector in Rl
        double precision om_g(3)        ! point cartesian position in Rg (km)
        double precision ms_g(3)        ! point - object in Rg  
! 
!-
!
        do i=1,nbp

            slatm = dsin(lat(i))
            clatm = dcos(lat(i))
            slonm = dsin(lon(i))
            clonm = dcos(lon(i))

! calculates the Rl unit vectors in Rg
! wl parallel to the local ascending geographic vertical
            wl_g(1) =   clatm * clonm
            wl_g(2) =   clatm * slonm
            wl_g(3) =   slatm
! vl in horizontal plan, positive toward east
            vl_g(1) = - slonm
            vl_g(2) =   clonm
            vl_g(3) =   0.d0
! ul in horizontal plan, positive toward south
            ul_g(1) =   slatm * clonm
            ul_g(2) =   slatm * slonm
            ul_g(3) = - clatm

            call lalocart(1,lat(i),lon(i),om_g,ierr)
            if( ierr .eq. 0 ) then
               ms_g(1) = pos(1,i) - om_g(1)
               ms_g(2) = pos(2,i) - om_g(2)
               ms_g(3) = pos(3,i) - om_g(3)
            
! transforms the point - object vector from Rg to Rl
               ms_l(1) =   ul_g(1)*ms_g(1) + ul_g(2)*ms_g(2)
     >                    + ul_g(3)*ms_g(3)
               ms_l(2) =   vl_g(1)*ms_g(1) + vl_g(2)*ms_g(2)
     >                    + vl_g(3)*ms_g(3)
               ms_l(3) =   wl_g(1)*ms_g(1) + wl_g(2)*ms_g(2)
     >                    + wl_g(3)*ms_g(3)

! calculates the object zenith and azimut angles
               dist(i) = dsqrt( ms_l(1) * ms_l(1) + ms_l(2) * ms_l(2)
     >                   + ms_l(3) * ms_l(3) )
               zen(i) = dacos( ms_l(3) / dist(i) )
               azi(i) = pi - datan2( ms_l(2) , ms_l(1) )

               if(azi(i).gt.pi) then
                  azi(i) = azi(i) -2*pi
               end if
               
               if( zen(i).lt.epsil ) then
                  zen(i) = 0.d0
                  azi(i) = 0.d0
               else if( zen(i).gt.epsnad ) then
                  zen(i) = pi
                  azi(i) = 0.d0
               endif
            else
               zen(i) = -999.d0
               azi(i) = -999.d0
            end if

        enddo
        return
        end
