        subroutine clkerr_ctrl(clkerru)
!+
!NOM/NAME $Id: clkerr_ctrl.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE
!       Controls the decoded values of the clock error information  
!       by comparing them to the preceeding values stored in historical file
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!       The clock error information consists of 5 parameters, each one having
!       its own date:
!       - clock error                           (err)
!       - clock error rate                      (rate)
!       - clock error after last correction     (last)
!       - next clock correction                 (next)
!       - clock error assigned to 0             (null)
!
!       the null parameter is ignored in this subroutine
!
!       For each parameter, the subroutine searches for the date
!       immediately preceeding the decoded clock error date
!
!       The following controls are performed to validate the decoded values: 
!
!       - the clock error is compared to a predicted value at the date
!       calculated with the last preceeding error and rate values
!       ( just after a clock correction, no control can be made )
!
!       - the clock error after last correction is considered as OK
!
!       - the next clock correction date is compared to the last correction date
!       (they must differ by at least 15 days)
!
!       - the clock error rate is tested by thresholds according to the 
!       preceding values stored in historical file. See rate_th variable
!       The tests have been modified in version 1.1 on Aug 7th 2001.
!
!       Remarks:
!       - no control if there is no information during more than 2 months
!       - some additionnal tests are made in order to eliminate repeated values
!       - the historical file is supposed to be chronological 
!         (but missing values with a 0 date can be inserted).
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   07 Aug 2001  modification of threshold for rate information 
!                    and allow empty lines
!                                    A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
        integer clkerru         ! logical unit of the index file
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        real err_val_errm       ! maximum variation for the current meas.
        parameter (err_val_errm = 0.2 )
!
!
! Commons
! Clock information
#include "c_clkerr.h"
!
! Functions
!
! Local variables:
        integer ios                     ! I/O status
        character*8 datst               ! date string
        character*6 dummyst             ! dummy string
        logical finished1               ! end of loop for variable last_date
        logical finished2               ! end of loop for variable err_date
        logical finished3               ! end of loop for variable rate_date
        logical finished4               ! end of loop for variable next_date
        integer f_last_date             ! date found for variable last_date
        integer f_err_date              ! date found for variable err_date 
        integer f_rate_date             ! date found for variable rate_date
        integer f_next_date             ! date found for variable next_date
        real f_last_val                 ! value found for variable last_val
        real f_err_val                  ! value found for variable err_val
        real f_rate_val                 ! value found for variable rate_val
        real inival                     ! initial value for the test
        integer inidate                 ! initial date for the test
        real val                        ! computed test value

        real rate_th                    ! rate thresholds (s)
        real d_rate                     ! rate difference (file - tbus) (s)

!       I/O variables
        character*4 typmes              ! type of variable
        integer fdate                   ! file date variable
        real fval                       ! file value variable

        logical warn_next
        integer warn_next_d
! 
!-
!
!       rewind the file
        rewind clkerru

!       skip the header lines (starting with space # or %)
        dummyst(1:1)=' '
        read(clkerru,'(a6)',iostat=ios) dummyst

        do while ( ios.eq.0 .and.
     &            (dummyst(1:1).eq. '#' .or. dummyst(1:1).eq.'%'
     &             .or. dummyst(1:1).eq.' ')  )
           read(clkerru,'(a6)',iostat=ios) dummyst
        end do
        if(ios.eq.0) backspace clkerru

!
        finished1=.false.
        finished2=.false.
        finished3=.false.
        finished4=.false.

!       if a date is not found the date and the corresponding value
!       are set to zero.
        f_last_date = 0
        f_last_val = 0.
        f_err_date = 0
        f_err_val = 0.
        f_rate_date = 0
        f_rate_val = 0.
        f_next_date = 0
        warn_next = .false.
        warn_next_d = 0

        ! read first line
        read(clkerru,'(a4,i6,f8.3)',iostat=ios) typmes,fdate,fval

        ! do while end of file
        do while (ios.eq.0) 

          if( (typmes.eq.'last')   .and.
     &        (fdate .le. err_date) .and.
     &        (fdate .gt. 0)       .and.
     &        (.not.finished1)     .and.
     &        (ios.eq.0)          ) then
            f_last_date=fdate
            f_last_val=fval
          elseif( (typmes.eq.'last')   .and.
     &            (fdate .gt. err_date) ) then
             finished1=.true.
          endif

          if( (typmes.eq.'cerr')   .and.
     &        (fdate .le. err_date) .and.
     &        (fdate .gt. 0)       .and.
     &        (.not.finished2)     .and.
     &        (ios.eq.0)          ) then
            f_err_date=fdate
            f_err_val=fval
          elseif( (typmes.eq.'cerr')   .and.
     &            (fdate .gt. err_date) ) then
             finished2=.true.
          endif

          if( (typmes.eq.'rate')   .and.
     &        (fdate .le. err_date) .and.
     &        (fdate .gt. 0)       .and.
     &        (.not.finished3)     .and.
     &        (ios.eq.0)          ) then
            f_rate_date=fdate
            f_rate_val=fval
          elseif( (typmes.eq.'rate')   .and.
     &            (fdate .gt. err_date) ) then
             finished3=.true.
          endif

          if( (typmes.eq.'next')   .and.
     &        (fdate .le. err_date) .and.
     &        (fdate .gt. 0)       .and.
     &        (.not.finished4)     .and.
     &        (ios.eq.0)          ) then
            f_next_date=fdate
          elseif( (typmes.eq.'next')   .and.
     &            (fdate .gt. err_date) ) then
             finished4=.true.
          endif

          if( (typmes.eq.'next')   .and.
     &        (abs(fdate - next_date).lt.15) .and.
     &        (abs(fdate - next_date).gt.0) .and.
     &        (fdate .gt. 0)       .and.
     &        (ios.eq.0)          ) then
                warn_next = .true.
                warn_next_d = fdate
          endif

          ! read next line
          read(clkerru,'(a4,i6,f8.3)',iostat=ios) typmes,fdate,fval

        ! end of do while
        end do
!
        !write(0,*) f_last_date, f_last_val
        !write(0,*) f_err_date, f_err_val
        !write(0,*) f_rate_date, f_rate_val
        !write(0,*) f_next_date
        !write(0,*) last_date, last_val
        !write(0,*) err_date, err_val
        !write(0,*) rate_date, rate_val
        !write(0,*) next_date

!
!       tests if the last correction date in the current message
!       is more recent that the date found in the historical file
        if( last_date .gt. f_last_date ) then
          f_last_date = last_date
          f_last_val = last_val
        endif

!       if "last" date are the same between the clock error file
!       and the tbus, and if the values are not the same.
!       we keep the one in the file, and send a warning message.
!       the user has to introduce the correct error in the file.
        if( (last_date .eq. f_last_date) .and. 
     &      (f_last_val .ne. last_val)   ) then
          call ml_wt('warning',
     &   'same date for last correction as the clock error file')
          call xdjc(last_date,datst)
          call ml_wt('warning','for date '//datst)
          call ml_wt('warning','but not same value')
          call ml_wt('warning',
     &    'we keep the information of the clock error file')
          call ml_wt('warning',
     & 'you must introduce the right value in clkerr_satimg.txt')
          call ml_wtr('warning','file value',f_last_val)
          call ml_wtr('warning','tbus value',last_val)
          last_date = 0
          last_val = -9.999
        endif

!       compute the value at the time of err_date
        if( f_next_date .gt. f_last_date .and. 
     &      f_next_date .gt. f_err_date) then
!         the most recent date is a reset !
!         the err_val is correct we do not have any possiblity for checking
          inival = 0.
          inidate = 0
        elseif( f_last_date .ge. f_err_date) then
!         the most recent date is an information on correction
          inival = f_last_val
          inidate = f_last_date
        else
!         normal case, we are after a measure
          inival = f_err_val
          inidate = f_err_date
        endif

!       test for a long time period between measurements
!       if there is more than 2 months without information
!       the value of the tbus is declared OK
        if( err_date - max(f_next_date,f_last_date,f_err_date)
     &                                                .gt. 60 ) then
          inival = 0.
          inidate = 0
        endif

!       if there is no stored value of the error rate we take
!       the current one
        if(f_rate_date.eq.0 .and. rate_date.gt.0) then
          f_rate_val = rate_val
        endif

        if(inidate.gt.0 .and. err_date.gt.0) then
!         test of the current measure
!         compute the theorical value for the date
          val = inival + (err_date-inidate)*f_rate_val

          if( abs(val - err_val) .lt. err_val_errm ) then
!           call xdjc(err_date,datst)
!           call ml_wtr('debug','Clock error OK ',err_val)
!           call ml_wt('debug','for date '//datst)
          else
            call ml_wt('info','Clock error rejected')
            call xdjc(err_date,datst)
            call ml_wt('info','for date '//datst)
            call ml_wtr('info','read value     ',err_val)
            call ml_wtr('info','expected value ',val)
            err_val = -9.999
            err_date = 0
          endif

!         test of the error rate
!         threshold is function of last stored value 
          if ( f_rate_date .gt. 0 ) then
             if    (abs(f_rate_val) .gt. 0.015) then
                rate_th = 0.005            ! 5ms for rate in [16, *]
             elseif(abs(f_rate_val) .gt. 0.010) then
                rate_th = 0.004            ! 4ms for rate in [11, 15]
             elseif(abs(f_rate_val) .gt. 0.005) then
                rate_th = 0.003            ! 3ms for rate in [6, 10]
             else
                rate_th = 0.002            ! 2ms for rate in [0, 5]
             endif
             rate_th = rate_th + 0.00001   ! add 1/100 msec for further tests
                                           ! with "real" values

!            difference stored - read
             d_rate = abs(f_rate_val - rate_val)

!            test of difference with thresold
             if( d_rate .gt. rate_th ) then
                call ml_wt('info','Clock error rate rejected')
                call xdjc(rate_date,datst)
                call ml_wt('info','for date '//datst)
                call ml_wtr('info','read value     ',rate_val)
                call ml_wtr('info','stored value   ',f_rate_val)
                call ml_wtr('info','threshold      ',rate_th)
                rate_val = -9.999
                rate_date = 0
             endif
          endif


!         test of the next clock correction date
!         rejected if the next clock correction date is less than 15 days 
!         after a stored "last correction date"
          if(next_date-f_last_date.lt. 15 .and. 
     &       next_date-f_last_date.gt.0 )  then
            call ml_wt('notice','Next clock corr. date rejected')
            call ml_wt('notice',
     &           'less than 15 days after the stored last corr.')
            call xdjc(next_date,datst)
            call ml_wt('notice','next corr. date '//datst)
            call xdjc(f_last_date,datst)
            call ml_wt('notice','last corr. date '//datst)
            next_date = 0
          elseif(next_date-f_last_date.eq.0) then
!               next corr. date rejected corresponding to an already 
!               stored last cor.
            next_date = 0
!           call ml_wt('debug','next corr. date rejected '//
!     &          'corresponding to an already stored last cor')
          endif


        else
!         all was done for nothing , there is no available data for
!         comparison, the value is set to OK
!         call xdjc(err_date,datst)
!         call ml_wt('debug','clock err: no poss. comparison')
        endif

        if(warn_next) then
          call ml_wt('warning',
     &              ' INTERVENTION NEEDED ON THE CLOCK error FILE')
          call ml_wt('warning',
     &       'there are 2 next correction messages within 15 days')
          call xdjc(next_date,datst)
          call ml_wt('warning','next corr. new '//datst)
          call xdjc(warn_next_d,datst)
          call ml_wt('warning','next corr. old '//datst)
        endif

!
        return

        end
