        subroutine clkerr_dc(scnam,utbus,iprint,ierr)
!+
!NOM/NAME $Id: clkerr_dc.F 208 2010-03-10 09:41:35Z frna $
!
!BUT/PURPOSE 
!       Decodes the clock error values stored in the plain language message at
!       the end of the TBUS Part IV. Checks if they are between allowed values
!
!DESCRIPTION 
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions   
!       The subroutine searches in the tbus file the spacecraft
!       international designation pattern,  decodes the clock error message and
!       controls the validity (min,max) of each variable.
!
!       Missing or erroneous informations are set to a fixed value
!               date 01/01/50 (day 0) values -9.999
!       The decoded clock errors are stored in a common : /c_clkerr/
!
!       The clock error information consist of 5 parameters, each one having
!       its own date:
!       - clock error                           (err)
!       - clock error rate                      (rate)
!       - clock error after last correction     (last)
!       - next clock correction                 (next)
!       - clock error assigned to 0             (null)
!
!       the null parameter is ignored in this subroutine
!----------------------------------------------------------------------------
! New format description reference:
! e-mail:
!Subject: AD: Changes to TBUSnn KWBC message
!From: Satinfo <satinfo@nesdis.noaa.gov>
!Date: Tue, 14 Jan 1997 09:03:21 +0500
!
!The following was transmitted to U.S. domestic and international (GTS)
!meteorological circuits on 10 January 1997.  It is repeated here for TBUS
!message users.
!  SUBJECT:    CHANGES TO SATELLITE APT PREDICT MESSAGES,
!              TBUS01 - 04 KWBC...EFFECTIVE JANUARY 29...1997.
!
!     (1) PART IV OF THE TBUS BULLETIN BEGINS WITH 5 LINES OF CODED
!  INFORMATION FOLLOWED BY PLAIN LANGUAGE INFORMATION CONCERNING
!  SATELLITE OPERATIONS.  WITHIN THE PLAIN LANGUAGE SECTION IS
!  INFORMATION ABOUT THE ONBOARD SPACECRAFT TIP CLOCK CORRECTION.
!
!     AN EXAMPLE OF THE CURRENT CLOCK CORRECTION INFORMATION FOLLOWS:
!  LAST TIP CLK CORR 12/31/95 CLK ERR AFTER CORR MINUS 0.1 SEC.
!  CLK ERR AS OF 04/15/96 PLUS 0.20 SEC.  ERR RATE AS OF 04/01/96
!  PLUS 2 MS/DAY (ESTIMATED).  NEXT CLK CORRECTION 08/20/96.
!
!     EFFECTIVE JANUARY 29 1997 AN ADDITIONAL CODED LINE
!  CONTAINING THE TIP CLOCK CORRECTION INFORMATION WILL BE ADDED
!  TO PART IV PRIOR TO THE PLAIN LANGUAGE SECTION.  THIS LINE
!  WILL HAVE 6 ALPHANUMERIC FIELDS CODED AS FOLLOWS:
!       MMDDYY - MONTH DAY YEAR OF LAST CLOCK CORRECTION
!       STTTTT - SIGN (P OR M) AND CLOCK ERROR AFTER CORRECTION
!                IN SECONDS TO 3 DECIMAL PLACES
!       STTTTT - SIGN (P OR M) AND CURRENT CLOCK ERROR IN SECONDS
!                TO 3 DECIMAL PLACES
!       MMDDYY - MONTH DAY YEAR OF MEASURED ERROR RATE
!       STTTTT - SIGN (P OR M) AND CLOCK ERROR RATE EXPRESSED AS
!                MILLISECONDS/DAY
!       MMDDYY - MONTH DAY YEAR OF NEXT CLOCK CORRECTION
!
!     IF THE NEXT CLOCK CORRECTION DATE (FIELD 6) IS UNKNOWN IT WILL
!  BE SET TO ALL ZEROES.  IF ANY ERRORS ARE UNKNOWN (FIELDS 2 3 OR 5)
!  THE FIELD WILL BE SET TO P99999.
!
!     THE PLAIN LANGUAGE EXAMPLE IN PARAGRAPH 2 ABOVE WOULD BE CODED
!  IN THE NEW FORMAT AS FOLLOWS:
!  123195 M00100 041596 P00200 040196 P00002 082096
!
!     FROM JANUARY 27 THROUGH FEBRUARY 12 1997 BOTH THE PLAIN LANGUAGE
!  AND CODED TIP CLOCK CORRECTION INFORMATION WILL BE CONTAINED IN PART IV
!  SO USERS MAY BECOME FAMILIAR WITH THE NEW CODE.  EFFECTIVE WITH THE
!  TBUS MESSAGES ISSUED FEBRUARY 13 1997 THE PLAIN LANGUAGE CLOCK
!  CORRECTION INFORMATION WILL BE DISCONTINUED AND ONLY THE CODED CLOCK
!  INFORMATION TRANSMITTED.
!
!     (2) EFFECTIVE ON MONTH/DAY 1996 THE SPARESPARE FIELD AT THE END OF
!  THE CODED PORTION OF PART IV WAS REPLACED WITH THE EQUATOR CROSSING
!  LONGITUDE OF THE EPOCH REFERENCE ORBIT NUMBER CONTAINED IN FIELD 2 OF
!  PART IV.  THE CROSSING LONGITUDE IS MEASURED AS EAST LONGITUDE TO 5
!  DECIMAL PLACES (LLLLLLLL).
!
!     USERS WITH QUESTIONS OR CONCERNS ABOUT THESE CHANGES ARE ADVISED
!  TO CONTACT E. HARROD / NOAA NESDIS ESP12 / IPD - INGEST SYSTEMS BRANCH
!  VIA EMAIL (EHARROD@NESDIS.NOAA.GOV) OR FAX (1-301-457-5199).  USERS ARE
!  ALSO DIRECTED TO THE NOAA SATELLITE INFORMATION WEB SITE AT URL
!  HTTP://PSBSGI1.NESDIS.NOAA.GOV/NOAASIS.HTML
!  AND THE SATELLITE NAVIGATION WEB SITE AT URL
!  HTTP://PSBSGI1.NESDIS.NOAA.GOV:8080/ISB/NAVIGATION/NAVPAGE.HTML
!  FOR INFORMATION SPECIFIC TO THIS CHANGE AND GENERAL NAVIGATION
!  INFORMATION.
!
!----------------------------------------------------------------------------
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   20 Jan 1997     new format       P. Brunel
! 1.2   15 Dec 2009     Ignore blank lines in Section 4, N C Atkinson
! 1.3   09 Mar 2010     Add ios checks
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       genscid         ! get the order number for a satellite name
!       scintdes        ! get the s/c international designc. for an order number
!       xdcj            ! conversion string date to juilan date
!       clkdatnorm      ! normalize a date
!       clkvalnorm      ! normalize an integer or real value
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
! input
        character*(*) scnam     ! any authorized NOAA s/c name
        integer utbus           ! logical unit for the TBUS I/O
        integer iprint          ! if no zero formatted output on stdout
! output
        integer ierr            ! return code
!                               ! 0 = OK
!                               ! 1 = TBUS Part IV for that satellite NOT found
!                               !     or I/O error in the file reading
!                               ! 2 = error in the checking
!
!DECLARATIONS
!
!
!
!
! Local parameters:
        integer datemin         ! minimum julian day (year 1970)
        integer datemax         ! maximum julian day (today)
        parameter (datemin=7300)
        real last_min           ! minimum value for last cor.  (-1.5 s)
        real last_max           ! maximum value for last cor.  ( 1.5 s)
        parameter (last_min=-1.5)
        parameter (last_max= 1.5)
        real err_min            ! minimum value for error   (-1.5 s)
        real err_max            ! maximum value for error   ( 1.5 s)
        parameter (err_min=-1.5)
        parameter (err_max= 1.5)
        real rate_min           ! minimum value for rate  (-50 ms/day)
        real rate_max           ! maximum value for rate  ( 50 ms/day)
        parameter (rate_min=-0.050)
        parameter (rate_max= 0.050)
!
! Commons
! Clock information
#include "c_clkerr.h"
!
! Functions
        integer genscid                 ! order number for a satellite name
        character*9 scintdes            ! international satellite designcation
!
! Local variables:
        integer i                       ! loop variables
        integer j
        integer l
        integer ios
        integer ordnum                  ! order satellite number
        integer lenmes                  ! length of the message stored
        integer ic                      ! index in fullmess string
        integer ich1                    ! index in fullmess string
        integer sig                     ! sign (-1, 1)
        integer intval                  ! variable for integer calculations
        integer d_newfmt                ! julian date of the new Tbus format (Feb 1, 97)
        real val                        ! temporary value
        character*9 intdes              ! international satellite designcation
        character*800 fullmess          ! full message with no blanck char.
        character*80 line               ! one line of tbus
        character*80 line1, line6       ! lines 1 and 6 of tbus part IV (after Feb 1, 97)
        character*8 chdate              ! string for date (dd/mm/yy)
        character*1 signc               ! character used for the sign decoding 
        logical ok                      ! true if the date and value are correct
        logical rderr                   ! true if there is a read error
        character*80  str
! 
       integer tbus_year                ! epoch date ...
       integer tbus_month               !   .....
       integer tbus_day

       integer tbus_dg(3)               ! epoch time (gregorian convention) 
       integer tbus_date                ! julian date corresponding to the tbus epoch time
       equivalence( tbus_dg(1),tbus_year   )
       equivalence( tbus_dg(2),tbus_month  )
       equivalence( tbus_dg(3),tbus_day    )

!
!-
!
        call xdcj('01/02/97',d_newfmt)

!       maximum date is TODAY, date of the run
        call gdj(datemax)
        
        ierr = 0
        do i = 1,800
          fullmess(i:i)=' '
        end do
!
!       get the international satellite designation
        ordnum = genscid(scnam)
        intdes = scintdes(ordnum)

!       search the first line of the tbus part IV for the satellite
!       ----------------------------------------------------------
        rewind utbus
        read(utbus,'(a80)',iostat=ios) line1
        do while ( ios.eq.0 .and. 
     &             line1(1:9).ne.intdes .and. line1(2:10).ne.intdes )
          read(utbus,'(a80)',iostat=ios) line1
        end do

!       skip the orbital elements (total = 5 lines) and read the clock error line
!       for TBUS after the 1st February 1997
        rderr= (ios.ne.0)
        read(utbus,'(a80)',iostat=ios) line
        do while (ios.eq.0.and.line(2:2).EQ.' ') 
          read(utbus,'(a80)',iostat=ios) line
        end do
        rderr=rderr .or. (ios.ne.0)
        read(utbus,'(a80)',iostat=ios) line
        do while (ios.eq.0.and.line(2:2).EQ.' ') 
          read(utbus,'(a80)',iostat=ios) line
        end do
        rderr=rderr .or. (ios.ne.0)
        read(utbus,'(a80)',iostat=ios) line
        do while (ios.eq.0.and.line(2:2).EQ.' ') 
          read(utbus,'(a80)',iostat=ios) line
        end do
        rderr=rderr .or. (ios.ne.0)
        read(utbus,'(a80)',iostat=ios) line
        do while (ios.eq.0.and.line(2:2).EQ.' ') 
          read(utbus,'(a80)',iostat=ios) line
        end do
        rderr=rderr .or. (ios.ne.0)
        read(utbus,'(a80)',iostat=ios) line6
        do while (ios.eq.0.and.line6(2:2).EQ.' ') 
          read(utbus,'(a80)',iostat=ios) line6
        end do
        rderr=rderr .or. (ios.ne.0)

!         store 10 lines of message with no blanck character
!         --------------------------------------------------
          j = 0
          do l = 1,10
            read(utbus,'(a80)',iostat=ios) line
            do while (ios.eq.0.and.line(2:4).EQ.'   ') 
              read(utbus,'(a80)',iostat=ios) line
            end do
            do i = 1,80
              if(line(i:i).ne.' ') then
                j = j+1
                fullmess(j:j) = line(i:i)
              endif
            end do
          end do
          lenmes = j

        if(.not.rderr) then

!         checks if there is a white space as the first character
!         and then shift to the left all the lines.
          if(line1(1:1).eq.' ') then
            do i=2,80
              line1(i-1:i-1) = line1(i:i)
              line6(i-1:i-1) = line6(i:i)
            end do
          endif

!         finds the tbus datation and substract 3 days for the
!         comparison because part IV is given for 3 days in advance
          read(line1(30:31),'(i2)') tbus_year
          read(line1(32:33),'(i2)') tbus_month
          read(line1(34:35),'(i2)') tbus_day
          call xdgj(tbus_dg,tbus_date)

          if(tbus_date-3 .ge. d_newfmt) then
!
! NEW format  (effective 1st February 1997)
!==========================================
             if (tbus_date-d_newfmt .le. 20) then
                call ml_wt('info','new TBUS format')
             endif

!       1 last tip clock correction
!       ---------------------------
             chdate='01/01/50'
             val=-9.999
             ic=1
             if(line6(ic:ic+5) .ne. '000000' .and. 
     &          line6(ic:ic+5) .ne. 'P99999') then

                chdate(1:2)=line6(ic+2:ic+3)
                chdate(4:5)=line6(ic:ic+1)
                chdate(7:8)=line6(ic+4:ic+5)
             
                signc=line6(ic+7:ic+7)
                read(line6(ic+8:ic+12),'(i5)') intval
                val =  intval * 1.d-03
                if(signc.eq.'M') then
                   val=-val
                endif
             endif
             call xdcj(chdate,last_date)
             last_val=val

!       2 current clock error
!       ---------------------
             chdate='01/01/50'
             val=-9.999
             ic=15
             if(line6(ic:ic+5) .ne. '000000' .and. 
     &          line6(ic:ic+5) .ne. 'P99999') then

                chdate(1:2)=line6(ic+2:ic+3)
                chdate(4:5)=line6(ic:ic+1)
                chdate(7:8)=line6(ic+4:ic+5)
             
                signc=line6(ic+7:ic+7)
                read(line6(ic+8:ic+12),'(i5)') intval
                val =  intval * 1.d-03
                if(signc.eq.'M') then
                   val=-val
                endif
             endif
             call xdcj(chdate,err_date)
             err_val=val

!       3 current clock error rate
!       --------------------------
             chdate='01/01/50'
             val=-9.999
             ic=29
             if(line6(ic:ic+5) .ne. '000000' .and. 
     &          line6(ic:ic+5) .ne. 'P99999') then

                chdate(1:2)=line6(ic+2:ic+3)
                chdate(4:5)=line6(ic:ic+1)
                chdate(7:8)=line6(ic+4:ic+5)
             
                signc=line6(ic+7:ic+7)
                read(line6(ic+8:ic+12),'(i5)') intval
                val =  intval * 1.d-03
                if(signc.eq.'M') then
                   val=-val
                endif
             endif
             call xdcj(chdate,rate_date)
             rate_val=val

!       4 next clock correction
!       -----------------------
             chdate='01/01/50'
             ic=43
             if(line6(ic:ic+5) .ne. '000000' .and. 
     &          line6(ic:ic+5) .ne. 'P99999') then

                chdate(1:2)=line6(ic+2:ic+3)
                chdate(4:5)=line6(ic:ic+1)
                chdate(7:8)=line6(ic+4:ic+5)
             
             endif
             call xdcj(chdate,next_date)

          else

!
! OLD format plain language information
!==========================================
             if (abs(d_newfmt-tbus_date) .le. 20) then
                call ml_wt('info','old TBUS format')
             endif

!       1 last tip clock correction
!       ---------------------------
!       search the first string : 'LAST TIP CLK CORR'
        ic = 1
        chdate='01/01/50'
        val=-999.

        ich1 = index(fullmess(ic:lenmes),'LASTTIPCLKCORR')

        if(ich1.ne.0) then
          ic = ich1+ic-1+14
          call clkdatnorm(fullmess(ic:ic+7),chdate)
          ic= ic +7
          ich1 = index(fullmess(ic:lenmes),'RCORR')
          ic = ic + ich1 -1 +5
          if(fullmess(ic:ic).eq.'M') then
            ic=ic+5
            sig= -1.
          elseif(fullmess(ic:ic).eq.'P') then
            ic=ic+4
            sig= 1.
          else
            sig= 1.
          endif
          call clkvalnorm(fullmess(ic:ic+4),val)
          if(val.gt.-999.) then
            val= val * sig
          endif
          ic=ic+4
        endif
        call xdcj(chdate,last_date)
        last_val = val

!       2 current clock error
!       ---------------------
!       search the string : 'CLK ERR AS OF'
        chdate='01/01/50'
        val=-999.

        ich1 = index(fullmess(ic:lenmes),'CLKERRASOF')
        if(ich1.ne.0) then
          ic = ich1 + ic -1 +10
          call clkdatnorm(fullmess(ic:ic+7),chdate)
          ic=ic+8
          if(fullmess(ic:ic).eq.'M') then
            ic=ic+5
            sig= -1.
          elseif(fullmess(ic:ic).eq.'P') then
            ic=ic+4
            sig=  1.
          else
            sig=  1.
          endif
          call clkvalnorm(fullmess(ic:ic+4),val)
          if(val.gt.-999.) then
            val= val * sig
          endif
          ic = ic +4
        endif
        call xdcj(chdate,err_date)
        err_val = val

!       3 current clock error rate
!       --------------------------
!       search the string : 'ERR RATE AS OF'
        chdate='01/01/50'
        val=-999.

        ich1 = index(fullmess(ic:lenmes),'RATEASOF')
        if(ich1.ne.0) then
          ic = ich1 + ic -1 +8
          call clkdatnorm(fullmess(ic:ic+7),chdate)
          ic=ic+8
          if(fullmess(ic:ic).eq.'M') then
            ic=ic+5
            sig= -1.
          elseif(fullmess(ic:ic).eq.'P') then
            ic=ic+4
            sig=  1.
          else
            sig=  1.
          endif
          call clkvalnorm(fullmess(ic:ic+4),val)
          if(val.gt.-999.) then
            val= val * sig /1000.
          endif
          ic = ic +4
        endif
        call xdcj(chdate,rate_date)
        rate_val = val

!       4 next clock correction
!       -----------------------
!       search the string : 'NEXT TIP CLOCK CORRECTION'
        chdate='01/01/50'

        ich1 = index(fullmess(ic:lenmes),'NEXT')
        if(ich1.ne.0) then
          ic = ich1 + ic -1 + 4
          ich1 = index(fullmess(ic:ic+24),'/')
          if(ich1.ne.0) then
            ic = ich1 +ic -1 -2
            call clkdatnorm(fullmess(ic:ic+7),chdate)
           endif
        endif
        call xdcj(chdate,next_date)

! end of test on new format
! =========================
          endif

!       5 ranges of possible values
!       ---------------------------
        
        ok = last_date.ge.datemin  .and. last_date.le.datemax .and.
     &       last_val.ge.last_min .and. last_val.le.last_max
        if(.not.ok) then
          last_date = 0
          last_val = -9.999
        endif

        ok = err_date.ge.datemin  .and. err_date.le.datemax .and.
     &       err_val.ge.err_min .and. err_val.le.err_max
        if(.not.ok) then
          err_date = 0
          err_val = -9.999
        endif

        ok = rate_date.ge.datemin  .and. rate_date.le.datemax .and.
     &       rate_val.ge.rate_min .and. rate_val.le.rate_max
        if(.not.ok) then
          rate_date = 0
          rate_val = -9.999
        endif

        !For next date check between last or datemin and today+6 months
        if(last_date.gt.0) then
          ok = next_date.ge.last_date  .and. next_date.le.datemax+180 
        else
          ok = next_date.ge.datemin  .and. next_date.le.datemax+180
        endif
        if(.not.ok) then
          next_date = 0
        endif

!       5 Find a leap second message
!       ----------------------------
!       search the string : 'LEAP'

        ich1 = index(fullmess(1:lenmes),'LEAP')
        if(ich1.ne.0) then
          call ml_wt('warning','Look at the TBUS informations')
          call ml_wt('warning',
     &                     'There will be a LEAP SECOND in few days')
          call ml_wt('warning',
     &        'The clock error file should have a clock correction')
          call ml_wt('warning',
     &       'If not insert a "next" at the date of the leap second')
        endif

        if(iprint.ge.1) then
          str = scnam
          call ml_wt('debug',
     &        'decoded values for the clock errors '//str)

          call xdjc(last_date,chdate)
          call ml_wtr('debug',
     &        'last clock correction  '//chdate,last_val)

          call xdjc(err_date,chdate)
          call ml_wtr('debug',
     &        'clock error as of      '//chdate,err_val)

          call xdjc(rate_date,chdate)
          call ml_wtr('debug',
     &        'clock error rate as of '//chdate,rate_val)

          call xdjc(next_date,chdate)
          call ml_wt('debug',
     &        'next clock correction  '//chdate)

        endif

        if( last_date.eq.0 .and. err_date.eq.0 .and.
     &       rate_date.eq.0 .and. next_date.eq.0 ) then
          ierr=2
        endif

        else
          ierr = 1
        endif

        return

        end
