        subroutine clkerr_get(clkerru,epoch,clkerr,ierr)
!+
!NOM/NAME $Id: clkerr_get.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       calculates the clock error at epoch using the clock error file
!       
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!       The clock error information consist of 6 parameters, each one having
!       its own date:
!       - clock error                           (cerr)
!       - clock error rate                      (rate)
!       - clock error after last correction     (last)
!       - next clock correction                 (next)
!       - clock error assigned to 0             (null)
!       - bias of clock                         (bias)
!
!       For each parameter, the subroutine searches for the date
!       immediately preceeding epoch.
!
!       It finds which information (err, last or next) is closest to epoch
!       and calculates the clock error at epoch using this information and 
!       the clock error rate value.
!       NB: if this information is 'next', epoch is just after a reset
!       the clock error value after reset is unknown and assigned to 0.
!       If there is no "last" or "cerr" corresponding to the "next" a
!       warning message is sent during 3 days. ierr = 0
!
!       The "null" parameter is introduced manually in the clock error file.
!       All orbits following that date will have a null clock error and ierr =0
!       Null must be used when the tbus values are doubious and removed when
!       tbus values are considered valid.
!       All the tbus values are stored during the doubious period (but not 
!       used). For a correct use off line the erroneous values must be 
!       eliminated when removing the null parameter.
!
!       The historical file is supposed to be chronological (but missing values
!       with a 0 date can be inserted).
!
!       If there is no information during 2 months (60 days) before epoch
!       the clock error is set to 0, ierr=1
!
!       If there is an information on bias, the corresponding value is
!       used after the "bias date". User can introduce several bias dates 
!       and values.
!       The bias is always added to the calculated clock error, even if the
!       "null" parameter is set, or if it is not possible to determine the 
!       clock error (ierr=1). We consider that the bias is independant from
!       the calculation of the clock error.
!       This information has been added on May 2001, to take into account
!       the error observed on NOAA-16 clock (1 sec).
!
!       To keep a complete historical file we recommand not to delete the
!       erroneous lines but to change their first character with a "z". They
!       will appear at the end of the file (sort after each tbusing) and
!       will be ignored.
! 
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   24 Nov 1997   null added & reset changed   AM PB
! 1.2   10 May 2001   bias information on clock error   AM PB
!                     loop on all lines of file, stops on end-of-file
! 1.3   07 Aug 2001   allow empty lines and replace continue loop
!                     by a do while loop  AM PB
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
! input
        integer clkerru         ! logical unit of the index file
        double precision epoch  ! epoch time (julian days)
! output
        real clkerr             ! clok error for the epoch time (s)
        integer ierr            ! return code 
!                               ! 0 = OK
!                               ! 1 = failed (end of file, clkerr=0.)
!
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
!
!
! Functions
!
! Local variables:
        integer ios                     ! I/O status
        character*6 dummyst             ! dummy string
        character*22 cht                ! string for date-hour
        character*40 line               ! message line
        logical finished1               ! end of loop for variable last_date
        logical finished2               ! end of loop for variable err_date
        logical finished3               ! end of loop for variable rate_date
        logical finished4               ! end of loop for variable next_date
        logical finished_bias           ! end of loop for variable bias
        logical nullerror               ! when error is assigned to ZERO
        double precision f_last_date    ! jul time found for variable last_date
        double precision f_err_date     ! jul time found for variable err_date 
        double precision f_rate_date    ! jul time found for variable rate_date
        double precision f_next_date    ! jul time found for variable next_date
        double precision f_bias_date    ! jul time found for variable bias
        real f_last_val                 ! value found for variable last_val
        real f_err_val                  ! value found for variable err_val
        real f_rate_val                 ! value found for variable rate_val
        real f_bias_val                 ! value found for variable bias
        real inival                     ! initial value for the test
        double precision inidate        ! initial jul time for the test
        double precision fjdate         ! julian time read in the file

!       I/O variables
        character*4 typmes              ! type of variable
        integer fdate                   ! file date variable
        real fval                       ! file value variable
        real dt                         ! time difference
! 
!-
!
!       rewind the file
        rewind clkerru

!       skip the header lines (starting with space # or %)
        dummyst(1:1)=' '
        read(clkerru,'(a6)',iostat=ios) dummyst
        do while ( ios.eq.0 .and.
     &            (dummyst(1:1).eq. '#' .or. dummyst(1:1).eq.'%'
     &             .or. dummyst(1:1).eq.' ')  )
           read(clkerru,'(a6)',iostat=ios) dummyst
        end do
        if(ios.eq.0) backspace clkerru

!
        finished1=.false.
        finished2=.false.
        finished3=.false.
        finished4=.false.
        finished_bias=.false.
        nullerror=.false.

!       if a date is not found the date and the corresponding value
!       are set to zero.
        f_last_date = 0.d0
        f_last_val  = 0.
        f_err_date  = 0.d0
        f_err_val   = 0.
        f_rate_date = 0.d0
        f_rate_val  = 0.
        f_next_date = 0.d0
        f_bias_date = 0.d0
        f_bias_val  = 0.

        ! read first line
        read(clkerru,'(a4,i6,f8.3)',iostat=ios) typmes,fdate,fval

        ! do while end of file
        do while (ios.eq.0) 

!       BEWARE we suppose that the tbus informations are given for the
!       date at 23H59MN. So we need to transform the dates into julian times
!       by : jtime = jdate + 1439./1440.  (23h59mn)
!
        fjdate = fdate*1.d0 + 1439.d0/1440.d0
!
          if( (typmes.eq.'last')   .and.
     &        (fjdate .le. epoch)  .and.
     &        (fjdate .gt. 0)      .and.
     &        (.not.finished1)     .and.
     &        (ios.eq.0)          ) then
            f_last_date=fjdate
            f_last_val=fval
          elseif( (typmes.eq.'last')   .and.
     &            (fjdate .gt. epoch) ) then
             finished1=.true.
          endif

          if( (typmes.eq.'cerr')   .and.
     &        (fjdate .le. epoch)  .and.
     &        (fjdate .gt. 0)      .and.
     &        (.not.finished2)     .and.
     &        (ios.eq.0)          ) then
            f_err_date=fjdate
            f_err_val=fval
          elseif( (typmes.eq.'cerr')   .and.
     &            (fjdate .gt. epoch) ) then
             finished2=.true.
          endif

          if( (typmes.eq.'rate')   .and.
     &        (fjdate .le. epoch)  .and.
     &        (fjdate .gt. 0)      .and.
     &        (.not.finished3)     .and.
     &        (ios.eq.0)          ) then
            f_rate_date=fjdate
            f_rate_val=fval
          elseif( (typmes.eq.'rate')   .and.
     &            (fjdate .gt. epoch) ) then
             finished3=.true.
          endif

          if( (typmes.eq.'next')   .and.
     &        (fjdate .le. epoch)  .and.
     &        (fjdate .gt. 0)      .and.
     &        (.not.finished4)     .and.
     &        (ios.eq.0)          ) then
            f_next_date=fjdate
          elseif( (typmes.eq.'next')   .and.
     &            (fjdate .gt. epoch) ) then
             finished4=.true.
          endif

          if( (typmes.eq.'null')   .and.
     &        (fjdate .le. epoch)  .and.
     &        (ios.eq.0)          ) then
            nullerror=.true.
            call xijc(fjdate,cht)
            call ml_wt('info',
     &         'null value manually inserted in the clock error file ')
            call ml_wt('info','starting on '//cht)
          endif

          if( (typmes.eq.'bias')   .and.
     &        (fjdate .le. epoch)  .and.
     &        (fjdate .gt. 0)      .and.
     &        (.not.finished_bias)     .and.
     &        (ios.eq.0)          ) then
            f_bias_date=fjdate
            f_bias_val=fval
          elseif( (typmes.eq.'bias')   .and.
     &            (fjdate .gt. epoch) ) then
             finished_bias=.true.
          endif

          ! read next line
          read(clkerru,'(a4,i6,f8.3)',iostat=ios) typmes,fdate,fval

          ! end of do while
          end do
!
!
!
!       search the clock information (err last or next) immediately 
!       preceeding epoch
        if( nullerror ) then
!          the null parameter appeared. Clock error is assigned to zero
!          ierr will be 0
           inival = 0
           inidate = epoch
           f_rate_val = 0
        elseif( f_next_date .gt. f_last_date .and.
     &      f_next_date .gt. f_err_date) then
!          the most recent date is a reset !
!          the value of the reset is unknown, we assign it to zero
!          ierr will be 0.
!          A warning message is output during 3 days after the clock reset.
           dt = epoch-f_next_date
           if(dt.le.3) then
              call xijc(f_next_date,cht)
              call ml_wt('warning','clock correction on '//cht)
              call ml_wt('warning','clock error is assigned to 0')
              fdate = f_next_date
              call ml_wt('warning','estimate it with landmarks and')
              call ml_wt('warning',
     &         'insert a last correction in the clock error file')
              write(line,"('last',i6,'   x.xxx',2x,a8)") fdate, cht(1:8)
              call ml_wt('warning',line)
           end if
           inival = 0
           inidate = epoch
           f_rate_val = 0
        elseif( f_last_date .ge. f_err_date) then
!          the most recent date is an information on correction
           inival = f_last_val
           inidate = f_last_date
        else
!          normal case, we are after a measure
           inival = f_err_val
           inidate = f_err_date
        endif

!       compute the value at the time of epoch
        if(inidate.gt.0) then
!         if the time difference is larger than 2 month (60 days)
!           the clock error is set to 0.
!         we take the last information on the clock error rate (f_rate_val)
!           even if it is an old one (small variations)
!
          dt = epoch-inidate
          if(dt .lt. 60.) then
            clkerr = inival + (epoch-inidate)*f_rate_val
            ierr = 0
          else
            clkerr = 0.
            ierr = 1
          endif
        else
!         all was done for nothing , there is no available data for
!         comparison, the value is set to 0
          clkerr = 0.
          ierr = 1
        endif


!       add bias to clock error
        if( f_bias_date .gt. 0.d0 ) then
           write(cht,"(f7.3)") f_bias_val
           call ml_wt('info','clock bias '//cht(1:7)//' (sec)')
           clkerr = clkerr + f_bias_val
        endif
!
        return

        end
