        subroutine tb_ctrl(scnam,path,tbindu,qflag,exterr)
!+
!NOM/NAME $Id: tb_ctrl.F 116 2008-06-27 09:17:04Z frna $
!
!BUT/PURPOSE 
!       controls the quality of the tbus orbital parameters
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions 
!       This modules controls the quality of the tbus orbital parameters
!       - if the decoded position-velocity is equal to zero the tbus is declared
!               a 'launch bulletin' and is declared OK. (The first bulletins 
!               emitted by NOAA do not contain the position-velocity.)
!       - converts the Brouwer mean elements into position-velocity at
!               epoch time using Brolyd. Compare the calculated position with
!               the position read in the tbus.
!               tbus is declared OK if the difference is less than 1km
!       - get the file name of the last preceeding valid tbus from the 
!               historical file
!       - opens the last preceeding valid tbus
!       - decodes the Brouwer-mean elements of the last preceeding valid tbus
!               (tb_sdc)
!       - closes the last preceeding valid tbus
!       - calculates extrapolation errors in km/day, using the Brolyd model
!               forward error of the last preceeding valid onto the current tbus
!               backward error of the current onto the last preceeding 
!               valid tbus
!       - the tbus is declared OK if the errors are less than 
!               6km/day (variables fwtest, bwtest)
!               6km + 0.1km/day * duration  (variables fwtest, bwtest, ratetest)
!
!       the tests with the last preceeding valid tbus are done only if the
!       time difference is less than 7 days (variable dtmax)
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   26 June 2008 make fwtest,bwtest variables not parameters, and allow them
!                     to be over-written by PAR_NAVIGATION_EXTRAP_LIMIT. NCA
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       cp_lsc          ! last significant character (f7cp)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!input
        character*(*) scnam     ! any authorized NOAA s/c name
        character*(*) path      ! path name for the index files
        integer tbindu          ! logical unit of the index file
!output
        integer qflag           ! quality flag
!                               ! 0 = OK
!                               ! 1 = missing position-velocity and orbit > 250
!                               ! 2 = erroneous orbital parameters
        real exterr(2)          ! extrapoaltion errors (km/day)
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        double precision pi     ! PI constant  
        double precision crd    ! conversion factor radians to degrees 
        real dstest             ! threshold for the minimum distance test (km)
        real sltest             ! threshold for position selft test (km)
        real ratetest           ! rate for fwd/bwd tests (km/day/day)
        real dtmax              ! max. time difference allowed for tests (days)
        integer lpv_unit        ! unit for the lpv tbus file

        parameter ( pi = 3.14159265358979311600d0)
        parameter ( crd=180.0d0/pi)
        parameter ( dstest=1. )
        parameter ( sltest=10. )
!       parameter ( ratetest=0.1 )
        parameter ( ratetest=0.0 )
        parameter ( dtmax=7. )
        parameter ( lpv_unit = 20 )
!
! Commons
! common TBUS Part IV
#include "c_tbus4.h"
!
! Brolyd common
#include "c_brolyd.h"
!
! Functions
        integer cp_lsc                  ! last significant character (f7cp)
!
! Local variables:
        integer ios                     ! I/O status variable
        character*256 name              ! var. name for the last... tbus file
        character*256 lpv_file          ! fullname of the last... tbus file
        integer ierglpv                 ! return code from glpv
        integer iersdc                  ! return code from sdc
        integer ioslpv                  ! I/O status for the tbus file openning 
        integer ielem                   ! element number
        double precision epocht         ! epoch time for tb_glpv
        double precision t0             ! epoch time of the current tbus
        double precision cmelem(6)      ! mean elements of the current tbus
        double precision t1             ! epoch time of the preceeding tbus
        double precision pmelem(6)      ! mean elements of the preceeding tbus
        double precision pposvel(6)     ! pos-vel elements of the prec. tbus
        double precision dist           ! distance earth-satellite (km)
        double precision oscgtod(6)     ! osculating elements (gama trueofdate)
        double precision pos(3)         ! position elements (gama trueofdate)
        double precision vel(3)         ! velocity elements (gama trueofdate)
        double precision slferr         ! self error in position      (km)
        double precision poserr_x       ! position error (x comp.) (km)
        double precision poserr_y       ! position error (y comp.) (km)
        double precision poserr_z       ! position error (z comp.) (km)
        double precision velmod         ! module of velocity   (km/s)
        double precision wkelem(6)      ! working elements for brolyd
        double precision auxelem(6)     ! auxiliary elements for brolyd
        logical launch                  ! launch bulletin if .true.
        logical test1                   ! test number 1 (self)
        logical test2                   ! test number 2 (forward)
        logical test3                   ! test number 3 (backward)

        real fwtest                     ! threshold for forward test (km/day)
        real bwtest                     ! threshold for backward test (km/day)
        character*10 cextrap            ! extrapolation threshold string
! 
!-
!
!       0. Initializations
!       ------------------

        fwtest=20.   !Extrapolation limit default
        bwtest=20.
        call GETENV('PAR_NAVIGATION_EXTRAP_LIMIT',cextrap)
        read(cextrap,*,err=100,end=100) fwtest
        read(cextrap,*,err=100,end=100) bwtest
        call ml_wt ('info','Extrapolalation limit is '//cextrap//
     &       ' km/day')
  100   continue

        t0 = tbus_epoch
        cmelem(1) = tbus_semaxis
        cmelem(2) = tbus_eccent
        cmelem(3) = tbus_inclin
        cmelem(4) = tbus_rigasc
        cmelem(5) = tbus_argper
        cmelem(6) = tbus_meanano

        do ielem=1,6
          wkelem(ielem) = cmelem(ielem)
        enddo

        exterr(1) = 0.
        exterr(2) = 0.

!       1. launch bulletin test (position equals 0)
!       -------------------------------------------
!       when we have a launch bulletin the position-velocity arrays
!       are not given.
!       a launch bulletin is always declared OK
!       Error if the orbit number is over 250 and position == 0
!
        dist = dsqrt( tbus_x*tbus_x + tbus_y*tbus_y + tbus_z*tbus_z )
        launch = dist.le.dstest

        if(launch .and. tbus_orbnum.lt.250) then
          qflag = 0
        elseif(launch .and. tbus_orbnum.ge.250) then
          qflag = 1
        else

!       2. self position test 
!       --------------------
!       computes the position-velocity from the mean elements for the
!       same instant and compare to the read values

          tto = 0.d0
          call brolyd(oscgtod,wkelem,0,1,1,auxelem)
          call brolyd(oscgtod,wkelem,2,2,1,auxelem)

          call celem(oscgtod, gm, pos, vel)
        
          slferr = dsqrt( (tbus_x-pos(1))**2 +
     &                    (tbus_y-pos(2))**2 +
     &                    (tbus_z-pos(3))**2 )

          test1 = slferr .lt. sltest

!       3. forward and backward tests
!       -----------------------------
!       get the last preceeding valid tbus filename
!       one minute is substracted to epoch in case of tbusing running twice 
!       with the same bulletin as input
          epocht = tbus_epoch-1.d0/1440d0
          t1 = epocht
          call tb_glpv(tbindu,epocht,lpv_file,ierglpv)

          if( ierglpv .eq. 0 ) then
!           create the tbus filename (adding the path and 
!           suppressing the white spaces) and opens the file
            name = 
     &       path(1:cp_lsc(path)) // '/' // lpv_file(1:cp_lsc(lpv_file))
            call cp_ssf(name,lpv_file,' ')
!           opens the tbus file
            open (unit=lpv_unit,file=lpv_file,status='old',
     &            iostat=ioslpv)

            if( ioslpv.ne.0 ) then
              call ml_wt('error','A TBUS FILE IS MISSING')
              call ml_wt('error','Filename '//lpv_file)
              call ml_wti('error','I/O status :',ioslpv)
            else

              call tb_sdc(scnam,lpv_unit,t1,pmelem,pposvel,iersdc)
              if(iersdc.ne.0) then
                call ml_wt('error','A TBUS FILE IS INCORRECT')
                call ml_wt('error','Filename '//lpv_file)
                call ml_wti('error','tb_sdc code :',iersdc)
              endif

            endif
            close (unit=lpv_unit,iostat=ios)

          else
            call ml_wt('warning',
     &                       'No valid TBUS before the current one')

          endif

!         if everything is OK we can process the tests
!             opens, reads, decoding and time difference
          if(ierglpv.eq.0 .and. iersdc.eq.0 .and. ioslpv.eq.0 .and.
     &                dabs(t0-t1).lt.dtmax ) then

!           forward test (preceeding to current)
!           -----------------------------------
            tto = (t0-t1) *86400.d0
            do ielem=1,6
              wkelem(ielem) = pmelem(ielem)
            enddo

            call brolyd(oscgtod,wkelem,0,1,1,auxelem)
            call brolyd(oscgtod,wkelem,2,2,1,auxelem)
            call celem(oscgtod, gm, pos, vel)
          
!           position error (extrapolation - tbus values) at t0
            poserr_x = pos(1)-tbus_x
            poserr_y = pos(2)-tbus_y
            poserr_z = pos(3)-tbus_z

!           velocity module at t0
            velmod = dsqrt( tbus_vx*tbus_vx +
     &                      tbus_vy*tbus_vy +
     &                      tbus_vz*tbus_vz )

!           along track forward error at t0
            exterr(1) = ( poserr_x*tbus_vx +
     &                    poserr_y*tbus_vy +
     &                    poserr_z*tbus_vz ) / velmod / (t0-t1)

            test2 = abs(exterr(1)) .lt. fwtest+ratetest*(t0-t1)

!           backward test (current to preceeding)
!           ------------------------------------
        
!           first tests if the pposvel is not equal to 0
!           in case of 0 the preceeding bulletin is a 'launch bulletin' 
!           and the backward test is not performed
            dist = dsqrt( pposvel(1)*pposvel(1) + 
     &                    pposvel(2)*pposvel(2) + 
     &                    pposvel(3)*pposvel(3) )
            if( dist .gt. dstest ) then
            
              tto = (t1-t0) *86400.d0
              do ielem=1,6
                wkelem(ielem) = cmelem(ielem)
              enddo

              call brolyd(oscgtod,wkelem,0,1,1,auxelem)
              call brolyd(oscgtod,wkelem,2,2,1,auxelem)
              call celem(oscgtod, gm, pos, vel)

!             position error (extrapolation - tbus values) at t1
              poserr_x = pos(1)-pposvel(1)
              poserr_y = pos(2)-pposvel(2)
              poserr_z = pos(3)-pposvel(3)

!             velocity module at t1
              velmod = dsqrt( pposvel(4)*pposvel(4) +
     &                        pposvel(5)*pposvel(5) +
     &                        pposvel(6)*pposvel(6) )

!             along track backward error at t1
              exterr(2) = ( poserr_x*pposvel(4) +
     &                      poserr_y*pposvel(5) +
     &                      poserr_z*pposvel(6) ) / velmod / (t0-t1)

              test3 = abs(exterr(2)) .lt. bwtest+ratetest*(t0-t1)

            else
!             preceeding bulletin is a 'launch bulletin' 
              test3 = .true.

            endif

          elseif(dabs(t0-t1).ge.dtmax) then
            call ml_wtdp('warning','Too many days without valid tbus',
     &                  t0-t1)
            test2 = .true.
            test3 = .true.

          else
!           no file is available for test; the tests are set to OK
            test2 = .true.
            test3 = .true.
          endif

          if(test1 .and. test2 .and. test3) then
            qflag = 0
          else
            call ml_wt('warning','The TBUS bulletin is rejected')
            call ml_wtdp('warning','self error  (km)',slferr)
            call ml_wtdp('warning','number of days',t0-t1)
            call ml_wtr('warning',
     &                           'forward error (km/day)',exterr(1))
            call ml_wtr('warning',
     &                           'backward error (km/day)',exterr(2))
            qflag = 2
          endif


        endif

        return

        end
