        subroutine tb_fnode(epoch,inielem,nodper,tan)
!+
!NOM/NAME $Id: tb_fnode.F 25 2006-02-03 15:43:53Z frna $
!       subroutine tb_fnode(epoch,inielem,nodper,tan)
!
!BUT/PURPOSE 
!       computes the nodal period and the time of ascending node (tan) of
!       the next orbit after epoch using extrapolation orbit model brolyd
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
! 
!       the orbital parameters inielem (from tbus) are used as input of brolyd
!       to calculate the satellite position at any time 
!
!       A node is calculated with an iterative process (dichotomy) 
!       to obtain a zero value of the satellite position Z component 
!
!       2 consecutive nodes are calculated in order to obtain a nodal period
!       node 1 = next ascending node after epoch (different from epoch)
!       node 2 = ascending node following node 1
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision epoch          ! epoch time of the elements 
        double precision inielem(6)     ! orbital elements 
!
! Output
        double precision nodper         ! nodal period          (days)
        double precision tan            ! time of ascending node (jdays)
!
!
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi,twopi               ! pi,  2 * pi
        parameter ( pi = 3.14159265358979311600D0 ,
     &              twopi = 2.d0 * pi)
        integer maxiter                 ! maximum number of iterations
        parameter (maxiter=20)
        double precision minz           ! minimum z in km 
        parameter (minz =0.01)
!
!
! Commons
! Brolyd common
#include "c_brolyd.h"
!
! Local variables
        integer ielem                   ! element number
        integer nbiter                  ! number of iterations
        double precision wkelem(6)      ! working elements for brolyd
        double precision oscgtod(6)     ! osculating elements (gama trueofdate)
        double precision auxelem(6)     ! auxiliary elements for brolyd
        double precision positod(3)     ! position in ITOD reference frame
        double precision velitod(3)     ! velocity in ITOD reference frame
        double precision dt             ! initial time increment(sec)
        double precision t1, t2, t3     ! time variables for dichotomy   (s)
        double precision z1, z2, z3     ! z variables for dichotomy (km)
        double precision n2             ! satellite mean motion (rad/s)
        double precision tan1           ! time of ascending node 1 (jdays)
        double precision tan2           ! time of ascending node 2 (jdays)
        integer ian                     ! variable for loop on the asc. nodes
! 
!-
!
!
!       initialize the brolyd model with the current tbus
!       -------------------------------------------------
        do ielem=1,6
          wkelem(ielem) = inielem(ielem)
        end do
        tto = 0.d0
        call brolyd(oscgtod,wkelem,0,1,1,auxelem)

!       set t1 at one second for the first calculation, in case
!       epoch very close to an ascending node.
        t1 = 1.d0
!
!       the time increment for the initial search is set to 5 minutes
        dt = 300.d0
!       
!       loop to find 2 ascending nodes
!       ------------------------------
        do ian = 1,2
!
!         finds the next 5 minutes interval containing an ascending node
!         --------------------------------------------------------------
!         initialise the points 1 and 2 for searching the interval 
!         crossing the equator
          do ielem=1,6
            wkelem(ielem) = inielem(ielem)
          end do
          tto = t1
          call brolyd(oscgtod,wkelem,2,2,1,auxelem)
          call celem(oscgtod, gm, positod, velitod)
          z1 = positod(3)

          do ielem=1,6
            wkelem(ielem) = inielem(ielem)
          end do
          t2 = t1+ dt
          tto = t2
          call brolyd(oscgtod,wkelem,2,2,1,auxelem)
          call celem(oscgtod, gm, positod, velitod)
          z2 = positod(3)

!         finds t1 and t2 including an ascending node
          do while( z1*z2.gt.0.d0 .or. z2.le.0.d0)

            z1 = z2
            t1 = t2

            do ielem=1,6
              wkelem(ielem) = inielem(ielem)
            end do
            t2 = t1 +dt
            tto = t2
            call brolyd(oscgtod,wkelem,2,2,1,auxelem)
            call celem(oscgtod, gm, positod, velitod)
            z2 = positod(3)
          end do

!         in the 5 minutes interval t1 t2
!         finds t3 time of ascending node by a dichotomy on the z variable
!         ----------------------------------------------------------------
          t3 = t1 - z1 * (t2 - t1) / (z2 - z1)
          tto = t3
          call brolyd(oscgtod,wkelem,2,2,1,auxelem)
          call celem(oscgtod, gm, positod, velitod)
          z3 = positod(3)
          nbiter = 1

          do while ( dabs(z3).gt.minz .and. nbiter.lt.maxiter )
            if( z3 * z1 .gt. 0.d0) then
!             3 and 1 are on the same side, keep 2 , change 1 by 3
              t1 = t3
              z1 = z3
            else
!             3 and 1 are on the opposite side, keep 1 , change 2 by 3
              t2 = t3
              z2 = z3
            endif

            if(dabs(z2-z1) .lt. 0.005) then
              t3 = t1
            else
              t3 = t1 -  z1 * (t2 - t1) / (z2 -z1)
            endif
            do ielem=1,6
              wkelem(ielem) = inielem(ielem)
            end do
            tto = t3
            call brolyd(oscgtod,wkelem,2,2,1,auxelem)
            call celem(oscgtod, gm, positod, velitod)
            z3 = positod(3)
            nbiter = nbiter + 1

          end do

!         the first ascending node is the argument tan
!         --------------------------------------------
          if(ian.eq.1) then
            tan1 = t3/86400.d0 + epoch
            tan = tan1

!           initialize t1 for the next search using an approximate value of
!           the nodal period deduced from the semi-major axis
            n2 = gm / (inielem(1)**3)
            t1 = t3 + twopi / dsqrt(n2) - dt

          else

            tan2 = t3/86400.d0 + epoch

          endif

        end do

        nodper = tan2 - tan1

        return

        end
