        subroutine tb_satpos(ijstdate,nbdays,timestep)
!+
!NOM/NAME $Id: tb_satpos.F 30 2006-04-06 15:03:47Z frna $
!       subroutine tb_satpos(ijstdate,nbdays,timestep)
!
!BUT/PURPOSE 
!       calculate the satellite positions and write them on the standard output
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       The standard output is supposed to be connected to a satpos file
!
!       The calculations are made since the start date (ijstdate) 
!       during several days (nbdays) with a time increment (timestep).
!
!       The orbital parameters are in the common c_tbus4.
!
!       for each time the following calculations are performed:
!       - the satellite position and velocity in the
!       inertial reference frame using the Brolyd extrapolation model.
!       - conversion into a Greenwich reference frame.
!       - orbit number deduced from the z component
!       - visibility from the station including refraction
!       - satellite in daylight or nighttime conditions
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision ijstdate       ! julian start instant
        real nbdays                     ! number of days (real value)
        real timestep                   ! time step in seconds
!
!DECLARATIONS
!
!
!
! Local parameters
!
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)
        double precision crd
        parameter (crd=180.d0/pi)
        double precision pi_2
        parameter (pi_2=pi/2.d0)
!
! Commons
! common TBUS Part IV
#include "c_tbus4.h"
!
! Brolyd common
#include "c_brolyd.h"
!
! ground station coordinates ( radians, km )
#include "c_stations.h"
!
!
! Functions
        double precision refrac         ! refraction function (rad)
!
! Local variables
        integer ielem                   ! element number
        double precision inielem(6)     ! working elements for brolyd
        double precision wkelem(6)      ! working elements for brolyd
        double precision oscgtod(6)     ! osculating elements (gama trueofdate)
        double precision auxelem(6)     ! auxiliary elements for brolyd
        double precision positod(3)     ! position in ITOD reference frame
        double precision velitod(3)     ! velocity in ITOD reference frame
        double precision posgrw(3)      ! position in Greenwich reference frame 
        double precision velgrw(3)      ! velocity in Greenwich reference frame
        double precision tini           ! initial time (sec)
        double precision tstep          ! time step (sec)
        double precision ijstep         ! julian instant of the step
        double precision zpre           ! preceeding value of the z component
        double precision znew           ! new value of the z component
        double precision site           ! satellite elevation angle  (rad) 
        double precision sitec          ! site corrected for refraction (rad) 
        double precision zen            ! satellite zenith angle  (rad) 
        double precision azi            ! satellite azimut angle  (rad) 
        double precision dist           ! station - sat distance          (km)
        double precision nodper         ! computed nodal period (days)
        double precision tan            ! computed time of ascending node(jdays)
        double precision possun(3)      ! sun position in Greenwich ref. frame
        double precision distsun        ! sun-earth distance (km)
        integer orbitan                 ! orbit number at tan
        integer orbstdate               ! orbit number at start date 
        integer nloop                   ! number of steps to be calc.
        integer istep                   ! step number
        integer orbnum                  ! orbit number
        integer codsun                  ! 
        integer codvis                  ! celestial object visibility codes
!                                         0 = object is visible from the station
!                                         1 = object is not visible 
        character*22 ch_epoch           ! tbus_epoch string
! 
!-
!
!       Section 1
!       writes the orbital parameters of the header 
!       -------------------------------------------
        call xijc(tbus_epoch,ch_epoch)
        write(*,"('tbus epoch time     ',a22)") ch_epoch
        write(*,"('semi-major axis     ',f10.4)") tbus_semaxis
        write(*,"('eccentricity        ',f10.9)") tbus_eccent
        write(*,"('inclination         ',f10.5)") 
     &                                   tbus_inclin * crd
        write(*,"('perigee argument    ',f10.5)") 
     &                                   tbus_argper * crd
        write(*,"('right ascension     ',f10.5)")
     &                                   tbus_rigasc * crd
        write(*,"('mean anomaly        ',f10.5)")
     &                                   tbus_meanano * crd
        write(*,"('x,y,z               ',3(f10.4,1x))")
     &                                   tbus_x, tbus_y, tbus_z
        write(*,"('vx,vy,vz            ',3(f10.7,1x))")
     &                                   tbus_vx, tbus_vy, tbus_vz
        write(*,"('station latitude    ',f10.4)")
     &                                   latstat(1) * crd
        write(*,"('station longitude   ',f10.4)")
     &                                   lonstat(1) * crd
        write(*,"('station altitude    ',f10.4)") altstat(1)
        write(*,"('stat. minimum site  ',f10.4)")
     &                         sitestat(1) * crd
        write(*,"('    step',
     &            '      x         y         z   ',
     &            '     vx        vy        vz      orb sun sta')")
!
!       Section 2
!       initialize the brolyd model with the current tbus
!       -------------------------------------------------
        inielem(1) = tbus_semaxis
        inielem(2) = tbus_eccent
        inielem(3) = tbus_inclin
        inielem(4) = tbus_rigasc
        inielem(5) = tbus_argper
        inielem(6) = tbus_meanano
        do ielem=1,6
          wkelem(ielem) = inielem(ielem)
        end do
        tto = 0.d0
        call brolyd(oscgtod,wkelem,0,1,1,auxelem)

!       Section 3
!       computes the loop variables and initial values
!       ----------------------------------------------
!       initial time difference start-tbus_epoch in seconds
        tini = (ijstdate - tbus_epoch) * 86400.d0

        tstep = timestep * 1.d0
        nloop = (nbdays * 86400.d0) / tstep + 1
        if((nloop-1) * tstep /86400.d0 .lt. nbdays) nloop = nloop+1

!       finds an initial ascending node (time and orbit number)

        dist = dsqrt( tbus_x*tbus_x + tbus_y*tbus_y + tbus_z*tbus_z )
        if(dist.le.1.d0 .or. dabs(tbus_z).gt.0.1 ) then
!         there is no value of the position of the satellite and/or
!         the position is not given for a node
!         then we must compute a reference:
!               next orbit following tbus_epoch
!               time of ascending node of this orbit
!               nodal period
          call tb_fnode(tbus_epoch,inielem,nodper,tan)
          orbitan = tbus_orbnum + 1
          if(tbus_nodper.gt.0.d0) then
            nodper = tbus_nodper
          endif
        else
          tan = tbus_epoch
          nodper = tbus_nodper
          orbitan = tbus_orbnum
        endif

!       computes the orbit number at stdate

        call tb_forb(tan,orbitan,nodper,tbus_epoch,inielem,
     &                  ijstdate,orbstdate)

        orbnum = orbstdate
        zpre = 1.d0
!
!       Section 4
!       computes the position for each time step
!       ---------------------------------------
        do istep=1,nloop

!         4.1 computes the osculating elements
!
!         tto is the time difference between the tbus and the time
!         of calculation, in seconds
          tto = tini + (istep -1)*tstep
!               ijstep is the julian time for the step
          ijstep = ijstdate + ( (istep-1)* tstep )/ 86400.d0

          do ielem=1,6
            wkelem(ielem) = inielem(ielem)
          end do
          call brolyd(oscgtod,wkelem,2,2,1,auxelem)

!         4.2 computes the greenwich elements

          call celem(oscgtod, gm, positod, velitod)
          call pvitodgrw(ijstep,positod,velitod,posgrw,velgrw)
        
!         4.3 orbit number

          znew = posgrw(3)
          if( zpre.lt.0.d0 .and. znew.ge.0.d0 ) then
            orbnum = orbnum+1
          endif
          zpre = znew

!         4.4 is the satellite seen from the station?

!         codvis = -1
          call trackang(posstat,matstat,1,posgrw,zen,azi,dist)
          site = pi_2 - zen
          sitec = site + refrac(site)
          if(sitec.ge.sitestat(1)) then
            codvis=0
          else
            codvis=1
          endif

!         4.5 is the satellite in daylight or nighttime conditions?

          codsun = -1
          call sungrw(ijstep,possun,distsun)
          call sunsat(posgrw,possun,codsun)

!         4.6 writes the results on standard output

!         call xijc(ijstep,ch_epoch)
          write(*,'(i8,3f10.3,3f10.6,i6,2i4,1x,a22)')
     &                  istep, 
     &                  posgrw, velgrw, orbnum,
     &                  codsun, codvis
!     &                  , ch_epoch


        end do

        return

        end
