        subroutine tb_sdc(scnam,utbus,epoch,melem,posvel,ierr)
!+
!NOM/NAME $Id: tb_sdc.F 210 2010-04-21 12:46:58Z frna $
!
!BUT/PURPOSE 
!       TBUS Part IV short decoding for a valid bulletin
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       For a satellite name it extracts the TBUS Part IV variables
!       which are necessary for orbit extrapolation.
!       This subroutine must be used only for a valid tbus (which has
!       been previously decoded by the subroutine tb_dc without error).
!
!       1 The subroutine searches the spacecraft international designcation
!       pattern.
!       2 Then it reads the values (uses internal fortran read)
!       3 Units are converted to radians, days, julian days...
!
!
!       The units are :
!       radians km km/s days julian_days
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 1995      no          A. Marsouin P.Brunel
! 1.1   21 Apr 2010     Ignore blank lines in Section 4 and add read checks. NCA
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       genscid         ! get the order number for a satellite name
!       scintdes        ! get the s/c international designc. for an order number
!       xigj            ! conversion from gregorian to julian (f7tp)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!input
        character*(*) scnam     ! any authorized NOAA s/c name
        integer utbus           ! logical unit for the TBUS I/O
!output
        double precision epoch  ! epoch time (julian days)
        double precision melem(6)! mean elements 
        double precision posvel(6)! position velocity elements 
        integer ierr            ! return code
!                               ! 0 = OK
!                               ! 1 = TBUS Part IV for that satellite NOT found
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
        double precision pi     ! PI constant  
        double precision cdr    ! conversion factor degrees to radians 

        parameter ( pi = 3.14159265358979311600d0)
        parameter ( cdr=pi/180.0d0)
!      
!
! Functions
        integer genscid                 ! order number for a satellite name
        character*9 scintdes            ! international satellite designcation
!
! Local variables:
        integer i
        integer ios
        integer intval                  ! variable for integer calculations
        integer ordnum                  ! order satellite number
        character*9 intdes              ! international satellite designcation
        character*80 line1, line2       ! the five lines of the part IV
        character*80 line3, line4, line5
        character*1 signc               ! charcter used for the sign decoding 
        logical rderr                   ! true if there is a read error
! 
       double precision tbus_epoch      ! epoch time    (julian days)

       double precision tbus_eccent     ! eccentricity
       double precision tbus_argper     ! argument of perigee   (rd)
       double precision tbus_rigasc     ! right ascension       (rd)
       double precision tbus_inclin     ! inclination           (rd)
       double precision tbus_meanano    ! mean anomaly          (rd)
       double precision tbus_semaxis    ! semi major axis       (km)

       double precision tbus_x          ! x component of the position (km)
       double precision tbus_y          ! y component of the position (km)
       double precision tbus_z          ! z component of the position (km)
       double precision tbus_vx         ! x component of the velocity (km/s)
       double precision tbus_vy         ! y component of the velocity (km/s)
       double precision tbus_vz         ! z component of the velocity (km/s)
       
       integer tbus_year                ! epoch date ...
       integer tbus_month               !   .......
       integer tbus_day
       integer tbus_hour
       integer tbus_minute
       integer tbus_second
       integer tbus_msecond

       integer tbus_dg(7)               ! epoch time (gregorian convention) 
       equivalence( tbus_dg(1),tbus_year    )
       equivalence( tbus_dg(2),tbus_month   )
       equivalence( tbus_dg(3),tbus_day     )
       equivalence( tbus_dg(4),tbus_hour    )
       equivalence( tbus_dg(5),tbus_minute  )
       equivalence( tbus_dg(6),tbus_second  )
       equivalence( tbus_dg(7),tbus_msecond )
!
!-
!
        ierr = 0
!
!       get the international satellite designation
        ordnum = genscid(scnam)
        intdes = scintdes(ordnum)

!       search the first line of the tbus part IV for the satellite
        rewind utbus
        read(utbus,'(a80)',iostat=ios) line1
        do while ( ios.eq.0 .and. 
     &             line1(1:9).ne.intdes .and. line1(2:10).ne.intdes )
          read(utbus,'(a80)',iostat=ios) line1
        end do

        if(ios.eq.0) then
         rderr= (ios.ne.0)
         read(utbus,'(a80)',iostat=ios) line2
         do while (ios.eq.0.and.line2(2:2).EQ.' ') 
           read(utbus,'(a80)',iostat=ios) line2
         end do
         rderr=rderr .or. (ios.ne.0)
         read(utbus,'(a80)',iostat=ios) line3
         do while (ios.eq.0.and.line3(2:2).EQ.' ') 
           read(utbus,'(a80)',iostat=ios) line3
         end do
         rderr=rderr .or. (ios.ne.0)
         read(utbus,'(a80)',iostat=ios) line4
         do while (ios.eq.0.and.line4(2:2).EQ.' ') 
           read(utbus,'(a80)',iostat=ios) line4
         end do
         rderr=rderr .or. (ios.ne.0)
         read(utbus,'(a80)',iostat=ios) line5
         do while (ios.eq.0.and.line5(2:2).EQ.' ') 
           read(utbus,'(a80)',iostat=ios) line5
         end do
         rderr=rderr .or. (ios.ne.0)

         if(.not.rderr) then

!         checks if there is a white space as the first character
!         and then shift to the left all the lines.
          if(line1(1:1).eq.' ') then
            do i=2,80
              line1(i-1:i-1) = line1(i:i)
              line2(i-1:i-1) = line2(i:i)
              line3(i-1:i-1) = line3(i:i)
              line4(i-1:i-1) = line4(i:i)
              line5(i-1:i-1) = line5(i:i)
            end do
          endif

!        decode the tbus part IV

          read(line1(30:31),'(i2)') tbus_year
          read(line1(32:33),'(i2)') tbus_month
          read(line1(34:35),'(i2)') tbus_day
          read(line1(36:37),'(i2)') tbus_hour
          read(line1(38:39),'(i2)') tbus_minute
          read(line1(40:41),'(i2)') tbus_second
          read(line1(42:44),'(i3)') tbus_msecond
          call xigj(tbus_dg,tbus_epoch)

          read(line2(19:26),'(i8)') intval
          tbus_eccent  = intval * 1.d-08

          read(line2(28:35),'(i8)') intval
          tbus_argper  =  intval * 1.d-05 * cdr

          read(line2(37:44),'(i8)') intval
          tbus_rigasc  =  intval * 1.d-05 * cdr

          read(line2(46:53),'(i8)') intval
          tbus_inclin =  intval * 1.d-05 * cdr

          read(line3( 1: 8),'(i8)') intval
          tbus_meanano =  intval * 1.d-05 *cdr

          read(line3(10:17),'(i8)') intval
          tbus_semaxis =  intval * 1.d-03

          signc=line3(19:19)
          read(line3(20:28),'(i9)') intval
          tbus_x =  intval * 1.d-04
          if(signc.eq.'M') then
            tbus_x=-tbus_x
          endif

          signc=line3(30:30)
          read(line3(31:39),'(i9)') intval
          tbus_y =  intval * 1.d-04
          if(signc.eq.'M') then
            tbus_y=-tbus_y
          endif

          signc=line3(41:41)
          read(line3(42:50),'(i9)') intval
          tbus_z =  intval * 1.d-04
          if(signc.eq.'M') then
            tbus_z=-tbus_z
          endif

          signc=line4(1:1)
          read(line4(2:9),'(i8)') intval
          tbus_vx =  intval * 1.d-06
          if(signc.eq.'M') then
            tbus_vx=-tbus_vx
          endif

          signc=line4(11:11)
          read(line4(12:19),'(i8)') intval
          tbus_vy =  intval * 1.d-06
          if(signc.eq.'M') then
            tbus_vy=-tbus_vy
          endif

          signc=line4(21:21)
          read(line4(22:29),'(i8)') intval
          tbus_vz =  intval * 1.d-06
          if(signc.eq.'M') then
            tbus_vz=-tbus_vz
          endif

          epoch = tbus_epoch
          melem(1) = tbus_semaxis
          melem(2) = tbus_eccent
          melem(3) = tbus_inclin
          melem(4) = tbus_rigasc
          melem(5) = tbus_argper
          melem(6) = tbus_meanano

          posvel(1) = tbus_x
          posvel(2) = tbus_y
          posvel(3) = tbus_z
          posvel(4) = tbus_vx
          posvel(5) = tbus_vy
          posvel(6) = tbus_vz

         else
           ierr = 2   !read error
         endif

        else
          ierr = 1
        endif

        return

        end
