        subroutine tle_dc(scnam,utle,iprint,ierr)
!+
!NOM/NAME $Id: tle_dc.F 399 2014-06-20 08:33:14Z frna $
!
!BUT/PURPOSE 
!       Decodes TLE orbital elements
!
!DESCRIPTION 
!     FORTRAN 77 witth UKMO/METEO-FRANCE accepted extensions   
!       For a satellite name it extracts all the TLE orbital element variables
!       checks them for minimum and maximum values and store them in a 
!       specific common /c_tle/
!
!       1 The subroutine searches the spacecraft TLE pattern.
!       2 Then it reads the values (uses internal fortran read)
!       3 Units are converted to radians, days, julian days...
!
!
!       The units for stored values are :
!       radians radians/day km km/s days julian_days etc... They are all 
!       specified in the common declaration
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 April 2004      no          A. Marsouin P.Brunel
! 1.1   12 July  2004   Split format strings in read statements  N C Atkinson
! 1.2   20 June  2014   Add tledes1 for compatibility with some older bulletins
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!       tle_chk         ! check the tle between min and max values
!       tle_prn         ! print the tle bulletin
!       genscid         ! get the order number for a satellite name  (libsatid)
!       cp_ups          ! conversion lower case to upper case (f7cp)
!       scintdes        ! international satellite designation (libsatid)
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
      implicit none                                                     
!
!ARGUMENTS 
!input
        character*(*) scnam     ! any authorized s/c name
        integer utle            ! logical unit for the TLE file
        integer iprint          ! if no zero formatted output on stdout
!                               ! if zero works silently
!output
        integer ierr            ! Out return code
!                               ! 0 = OK
!                               ! 1 = TLE elements for that sat. NOT found
!                               ! 2 = I/O error in the file reading
!                               ! 3 = fatal error in the checking
!
!DECLARATIONS
!
!
!
! Common declarations:
!
! Local parameters:
!      PI is already in common C2 of c_sgp.h
!
! Commons
! commonn TLE
#include "c_sgp.h"
#include "c_tle.h"
!
! Functions  (libsatid)
        integer genscid                 ! order number for a satellite name
        character*9 scintdes            ! international satellite designcation
!
! Local variables:
        integer ios
        integer ierchk                  ! return code of tle_chk
        integer ordnum                  ! order satellite number
        integer year
        integer iq(3)
        character*8 tledes,tledes1      ! TLE satellite designation
        character*9 satname             ! international satellite designation
        character*80 line               ! one line of message
        character*22 cht
! 
        double precision day_of_year
        integer  stdmm_exp
        integer  bstar_exp
        integer line1_id
        integer line2_id
!
!-
!
        ierr = 0
!
!       get the TLE satellite designcation (uppercase)
        ordnum  = genscid(scnam)
        satname = scintdes(ordnum)
        tledes = satname(3:4)//satname(6:9)//'  '
        tledes1 = satname(3:4)//satname(6:8)//'  '//satname(9:9)
        call cp_ups(tledes)

!       search the first line of the tle line 1 for the satellite
        rewind utle
        read(utle,'(a)',iostat=ios) line
        do while ( ios.eq.0 .and. line(10:17).ne.tledes
     &                      .and. line(10:17).ne.tledes1 )
          read(utle,'(a)',iostat=ios) line
        end do

        if(ios.eq.0) then

!        decode the tle elements
!       --------------------------

          read(line,'(i1,1x,i5,a1,1x,a8,1x,i2,f12.8,1x,f10.8,' //
     &                     '2(1x,f6.5,i2),1x,i1,1x,i4,i1)',iostat=ios) 
     &     line1_id,            ! line number of element data
     &     tle_satnumber,       ! satellite number (TLE)
     &     tle_satclas,         ! satellite classification
     &     tle_intdes,          ! International designator (3 parts)
     &     year,                ! Epoch last 2 digits of year
     &     day_of_year,         ! Epoch day of year and fractional portion of day
     &     tle_ftdmm,           ! first time derivative of the mean motion
     &     tle_stdmm,           ! second time derivative of the mean motion
     &     stdmm_exp,           ! """ exponent
     &     tle_bstar,               ! BSTAR drag term
     &     bstar_exp,           ! """ exponent
     &     tle_ephtype,         ! ephemeris type
     &     tle_elemnum,         ! element number
     &     tle_chksum1          ! check sum line 1
          
          if(line1_id .ne. 1) then
             ierr = 2
          endif

          ! TLE epoch time
          tle_epoch_tlf = year*1000.D0 + day_of_year

          ! CNES Julian date of epoch time
          if(year .gt. 70) then
             year = year + 1900
          else
             year = year + 2000
          endif
          iq(1) = year
          iq(2) = int(day_of_year)
          iq(3) = (day_of_year - int(day_of_year)) * 86400D3
          call xiqj(iq, tle_epoch)
          call xijc(tle_epoch, cht)

           ! first time derivative of the mean motion
          tle_ftdmm = tle_ftdmm * twopi/xmnpda/xmnpda

           ! second time derivative of the mean motion
          tle_stdmm = tle_stdmm * (10.**stdmm_exp)
          tle_stdmm = tle_stdmm * twopi/xmnpda/xmnpda/xmnpda

          tle_bstar = tle_bstar * (10.**bstar_exp) / AE

          read(utle,'(a)',iostat=ios) line
          read(line,'(i1,1x,i5,2(1X,F8.4),1X,F7.7,' //
     &                 '2(1X,F8.4),1X,F11.8,i5,i1)',iostat=ios) 
     &     line2_id,            ! line number of element data
     &     tle_satnumber,       ! satellite number (TLE)
     &     tle_inclin,          ! inclination
     &     tle_rigasc,          ! right ascension of the ascending node
     &     tle_eccent,          ! excentricity
     &     tle_argper,          ! argument of perigee
     &     tle_meanano,         ! mean anomaly
     &     tle_meanmotion,      ! mean motion
     &     tle_orbnum,          ! orbit number at epoch
     &     tle_chksum2          ! check sum line 2

          if(line2_id .ne. 2) then
             ierr = 2
          endif

          if(ios.eq.0) then
            tle_inclin = tle_inclin * pi/180.d0
            tle_rigasc = tle_rigasc * pi/180.d0
            tle_argper = tle_argper * pi/180.d0
            tle_meanano = tle_meanano * pi/180.d0
            tle_meanmotion = tle_meanmotion * twopi / xmnpda   ! rad/nim

            !estimate semi major axis from TLE elements

            !E1 common
            xmo    = tle_meanano
            xnodeo = tle_rigasc
            omegao = tle_argper
            eo     = tle_eccent
            xincl  = tle_inclin
            xno    = tle_meanmotion
            xndt2o = tle_ftdmm
            xndd6o = tle_stdmm
            bstar  = tle_bstar
            epoch  = tle_epoch_tlf

            ! use of the anomalistic period to approximately separate between 
            ! deep space (>225 minutes) and near Earth orbits 
            tle_deep=(twopi/tle_meanmotion .gt. 225.)


!           checks the TLE variables between allowed values
!           ------------------------------------------------
            call tle_chk(ierchk)
            if(ierchk.eq.2) then
              ierr=3
            endif

            if(iprint.ge.1) then
              call tle_prn(scnam)
            endif

          else
            ierr = 2
          endif

        else
          ierr = 1
        endif

        return

        end
