        subroutine tle_fnode(t0_epoch,inielem,deep,nodper,tan)
!+
!NOM/NAME $Id: tle_fnode.F 25 2006-02-03 15:43:53Z frna $
!       subroutine tle_fnode(t0_epoch,inielem,smasec,nodper,tan)
!
!BUT/PURPOSE 
!       computes the nodal period, and the time of ascending node (tan) of
!       the next orbit after t0_epoch using extrapolation orbit model sgp/sdp
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       the orbital parameters inielem (from 2-Line) are used as input of 
!       SGP/SDP to calculate the satellite position at any time 
!
!       A node is calculated with an iterative process (dichotomy) 
!       to obtain a zero value of the satellite position Z component 
!
!       2 consecutive nodes are calculated in order to obtain a nodal period
!       node 1 = next ascending node after t0_epoch (different from t0_epoch)
!       node 2 = ascending node following node 1
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 April 2004      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision t0_epoch       ! epoch time of the elements 
        double precision inielem(10)    ! orbital elements 
        logical deep                    ! true if deep space 
!
! Output
        double precision nodper         ! nodal period          (days)
        double precision tan            ! time of ascending node (jdays)
!
!
!DECLARATIONS
!
!
!
! Local parameters
! constants of the earth ellipsoid and rotation
#include "p_earth.h"
!
        integer maxiter                 ! maximum number of iterations
        parameter (maxiter=20)
        double precision minz           ! minimum z in km 
        parameter (minz =0.01)
!
!
! Commons
#include "c_sgp.h"
!
!
!
! Local variables
        integer nbiter                  ! number of iterations
        integer iflag                   ! flag for SGP4 
        double precision tsince         ! extrapolation duration (mn)
        double precision posvel(6)      ! position-velocity in TEME ref. frame
        double precision dt             ! initial time increment(sec)
        double precision t0
        double precision t1, t2, t3     ! time variables for dichotomy   (s)
        double precision z1, z2, z3     ! z variables for dichotomy (km)
        double precision tan1           ! time of ascending node 1 (jdays)
        double precision tan2           ! time of ascending node 2 (jdays)
        integer ian                     ! variable for loop on the asc. nodes
! 
!-
!
        t0 = t0_epoch
        t1 = t0
!
!       the time increment for the initial search is set to 5 minutes
        dt = 300.d0/86400.d0
!       
!       loop for finding 2 ascending nodes
!       ----------------------------------
        do ian = 1,2
!
!         finds the next 5 minutes interval containing an ascending node
!         --------------------------------------------------------------
!         initialise the points 1 and 2 for searching the interval 
!         crossing the equator
           xmo    = inielem(1)
           xnodeo = inielem(2)  
           omegao = inielem(3)
           eo     = inielem(4)
           xincl  = inielem(5)  
           xno    = inielem(6)
           xndt2o = inielem(7)
           xndd6o = inielem(8)
           bstar  = inielem(9)
           epoch  = inielem(10)

           !Compute position at epoch
           tsince = (t1 - t0) * 1440.
           iflag = 1
           if(deep) then
              call sdp4(iflag,tsince)
           else
              call sgp4(iflag,tsince)
           endif
           posvel(1) = x*XKMPER/AE
           posvel(2) = y*XKMPER/AE
           posvel(3) = z*XKMPER/AE
           posvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
           posvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
           posvel(6) = zdot*XKMPER/AE*XMNPDA/86400.

           z1 = posvel(3)

           t2 = t1+ dt
           xmo    = inielem(1)
           xnodeo = inielem(2)  
           omegao = inielem(3)
           eo     = inielem(4)
           xincl  = inielem(5)  
           xno    = inielem(6)
           xndt2o = inielem(7)
           xndd6o = inielem(8)
           bstar  = inielem(9)
           epoch  = inielem(10)

           !Compute position at epoch
           tsince = (t2 - t0) * 1440.
           iflag = 1
           if(deep) then
              call sdp4(iflag,tsince)
           else
              call sgp4(iflag,tsince)
           endif
           posvel(1) = x*XKMPER/AE
           posvel(2) = y*XKMPER/AE
           posvel(3) = z*XKMPER/AE
           posvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
           posvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
           posvel(6) = zdot*XKMPER/AE*XMNPDA/86400.

           z2 = posvel(3)

!          finds t1 and t2 including an ascending node
           do while( z1*z2.gt.0.d0 .or. z2.le.0.d0)

              z1 = z2
              t1 = t2

              t2 = t1 +dt
              xmo    = inielem(1)
              xnodeo = inielem(2)  
              omegao = inielem(3)
              eo     = inielem(4)
              xincl  = inielem(5)  
              xno    = inielem(6)
              xndt2o = inielem(7)
              xndd6o = inielem(8)
              bstar  = inielem(9)
              epoch  = inielem(10)

                                !Compute position at epoch
              tsince = (t2 - t0) * 1440.
              iflag = 1
              if(deep) then
                 call sdp4(iflag,tsince)
              else
                 call sgp4(iflag,tsince)
              endif
              posvel(1) = x*XKMPER/AE
              posvel(2) = y*XKMPER/AE
              posvel(3) = z*XKMPER/AE
              posvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
              posvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
              posvel(6) = zdot*XKMPER/AE*XMNPDA/86400.

              z2 = posvel(3)
           end do

!          in the 5 minutes interval t1 t2
!          finds t3 time of ascending node by a dichotomy on the z variable
!          ----------------------------------------------------------------
           t3 = t1 - z1 * (t2 - t1) / (z2 - z1)
           xmo    = inielem(1)
           xnodeo = inielem(2)  
           omegao = inielem(3)
           eo     = inielem(4)
           xincl  = inielem(5)  
           xno    = inielem(6)
           xndt2o = inielem(7)
           xndd6o = inielem(8)
           bstar  = inielem(9)
           epoch  = inielem(10)

           !Compute position at epoch
           tsince = (t3 - t0) * 1440.
           iflag = 1
           if(deep) then
              call sdp4(iflag,tsince)
           else
              call sgp4(iflag,tsince)
           endif
           posvel(1) = x*XKMPER/AE
           posvel(2) = y*XKMPER/AE
           posvel(3) = z*XKMPER/AE
           posvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
           posvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
           posvel(6) = zdot*XKMPER/AE*XMNPDA/86400.
           z3 = posvel(3)
           nbiter = 1

           do while ( dabs(z3).gt.minz .and. nbiter.lt.maxiter )
              if( z3 * z1 .gt. 0.d0) then
!             3 and 1 are on the same side, keep 2 , change 1 by 3
                 t1 = t3
                 z1 = z3
              else
!             3 and 1 are on the opposite side, keep 1 , change 2 by 3
                 t2 = t3
                 z2 = z3
              endif

              if(dabs(z2-z1) .lt. 0.005) then
                 t3 = t1
              else
                 t3 = t1 -  z1 * (t2 - t1) / (z2 -z1)
              endif
              xmo    = inielem(1)
              xnodeo = inielem(2)  
              omegao = inielem(3)
              eo     = inielem(4)
              xincl  = inielem(5)  
              xno    = inielem(6)
              xndt2o = inielem(7)
              xndd6o = inielem(8)
              bstar  = inielem(9)
              epoch  = inielem(10)

              !Compute position at epoch
              tsince = (t3 - t0) * 1440.
              iflag = 1
              if(deep) then
                 call sdp4(iflag,tsince)
              else
                 call sgp4(iflag,tsince)
              endif
              posvel(1) = x*XKMPER/AE
              posvel(2) = y*XKMPER/AE
              posvel(3) = z*XKMPER/AE
              posvel(4) = xdot*XKMPER/AE*XMNPDA/86400.
              posvel(5) = ydot*XKMPER/AE*XMNPDA/86400.
              posvel(6) = zdot*XKMPER/AE*XMNPDA/86400.

              z3 = posvel(3)
              nbiter = nbiter + 1

           end do

!          the first ascending node is the argument tan
!          --------------------------------------------
           if(ian.eq.1) then
              tan1 = t3
              tan = tan1

!             initialize t1 for the next search using an approximate value of
!             the nodal period deduced from the semi-major axis
              t1 = t3 + twopi / inielem(6) / 1440. -dt

           else
              
              tan2 = t3
              
           endif


        end do

        nodper = tan2 - tan1

        return

        end
