        subroutine tle_forb(tan,orbitan,nodper,
     &                     t0_epoch,inielem,deep,t,orbit)
!+
!NOM/NAME $Id: tle_forb.F 25 2006-02-03 15:43:53Z frna $
!       subroutine tle_forb(tan,orbitan,nodper,
!     &                     t0_epoch,inielem,deep,t,orbit)
!
!BUT/PURPOSE 
!       computes the orbit number at t strictly consistent with sgp/sdp

!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       An approximate value of the orbit number is calculated using
!       the nodal period and an initial ascending node (time and orbit number)
!
!       The orbital parameters inielem (from 2-Line) are used as input of
!       SGP/SDP to calculate the satellite position at t.
!       The value of the z component is used to correct the approximate
!       orbit number if t is close to an ascending node.
!
!       It also checks the particular problem of the satellite launch
!       (starting date at 0h could be a negative orbit number).
!
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 April 2004      no          A. Marsouin P.Brunel
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        integer orbitan                 ! orbit number at tan
        double precision tan            ! time of ascending node (jdays)
        double precision nodper         ! nodal period          (days)
        double precision t0_epoch               ! epoch time of the elements 
        double precision inielem(10)    ! orbital elements 
        logical deep                    ! true if deep space 
        double precision t              ! epoch time of the searched orbit
!
! Output
        integer orbit                   ! orbit number at t
!
!
!DECLARATIONS
!
!
!
! Local parameters
!
!
! Commons
#include "c_sgp.h"
!
!
!
! Local variables
        integer iflag                   ! flag for SGP4 
        double precision tsince         ! extrapolation duration (mn)
        double precision realorbit      ! real orbit number at start instant
        double precision posvelteme(6)  ! position velocity in TEME ref. frame
        double precision zstdate        ! z component of the position at stdate
        double precision vzstdate       ! z component of the velocity at stdate
        double precision resorb         ! residual part of the orbit difference
! 
!-
!
!       realorbit is the estimated real orbit number at t
!       and orbit is the corresponding integer value
        realorbit = orbitan + (t - tan)/ nodper


!       tests for launch of satellite (orbit at 0h could be negative)
        if(realorbit .gt.0.d0) then
          orbit = dint(realorbit)
        else
          orbit = dint(realorbit-1.d0)
        endif

!       compute the position of the satellite by using mosaik at t

!       initial time difference: t - epoch in seconds
        xmo    = inielem(1)
        xnodeo = inielem(2)  
        omegao = inielem(3)
        eo     = inielem(4)
        xincl  = inielem(5)  
        xno    = inielem(6)
        xndt2o = inielem(7)
        xndd6o = inielem(8)
        bstar  = inielem(9)
        epoch  = inielem(10)

        !Compute position at epoch
        tsince = (t - t0_epoch) * 1440.
        iflag = 1
        if(deep) then
           call sdp4(iflag,tsince)
        else
           call sgp4(iflag,tsince)
        endif
        posvelteme(1) = x*XKMPER/AE
        posvelteme(2) = y*XKMPER/AE
        posvelteme(3) = z*XKMPER/AE
        posvelteme(4) = xdot*XKMPER/AE*XMNPDA/86400.
        posvelteme(5) = ydot*XKMPER/AE*XMNPDA/86400.
        posvelteme(6) = zdot*XKMPER/AE*XMNPDA/86400.

        zstdate = posvelteme(3)
        vzstdate = posvelteme(6)


!       if we are near an ascending node (50 km) we must verify the estimated
!       orbit number. 

        if(dabs(zstdate).lt.50.d0 .and. vzstdate.gt.0.d0) then

!         resorb is the decimal residual value of the estimated orbit
          resorb = orbit*1.d0 - realorbit
!
!         the estimated orbit number (orbit) is correct in the following cases:
!         - z > 0   and resorb close to 0  (just after an ascending node)
!         - z < 0   and resorb close to 1  (just before an ascending node)
!         or it has to be corrected to be consistent with mosaik.

          if (abs(resorb) .lt. 0.5 .and. zstdate.lt.0.d0) then
            orbit = orbit - 1
          endif

          if (abs(resorb) .gt. 0.5 .and. zstdate.gt.0.d0) then
            orbit = orbit + 1
          endif

        endif

        return

        end
