        subroutine tle_satpos(ijstdate,nbdays,timestep)
!+
!NOM/NAME $Id: tle_satpos.F 534 2017-07-31 09:17:38Z frna $
!       subroutine tle_satpos(ijstdate,nbdays,timestep)
!
!BUT/PURPOSE 
!       calculate the satellite positions and write them on the standard output
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!       The standard output is supposed to be connected to a satpos file
!
!       The calculations are made since the start date (ijstdate) 
!       during several days (nbdays) with a time increment (timestep).
!
!       The orbital parameters are in the common c_tle.
!
!       for each time the following calculations are performed:
!       - the satellite position and velocity in the
!       inertial reference frame using the SGP/SDP extrapolation model.
!       - conversion into a Greenwich reference frame.
!       - orbit number deduced from the z component
!       - visibility from the station including refraction
!       - satellite in daylight or nighttime conditions
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 April 2004     no          A. Marsouin P.Brunel
! 1.1   03 August 2005  bug correction in the orbit number calculation
!                       at nearest ascending node;.  P. Brunel
! 1.2   25 June 2012    Make orbit numbers from EUMETSAT TLEs consistent with
!                       those from NORAD TLEs. N. Atkinson
! 1.3   28 July 2017    Roll over from orbit 99999 to 1, following NOAA
!                       announcements for NOAA-15 (PB/NA)
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!
!ARGUMENTS 
! Input
        double precision ijstdate       ! julian start instant
        real nbdays                     ! number of days (real value)
        real timestep                   ! time step in seconds
!
!DECLARATIONS
!
!
!
! Local parameters
!
        double precision lpi
        parameter ( lpi = 3.14159265358979311600d0)
        double precision crd
        parameter (crd=180.d0/lpi)
        double precision pi_2
        parameter (pi_2=lpi/2.d0)
!
! Commons
! commonn TLE
#include "c_tle.h"
#include "c_sgp.h"
!
!       ground station coordinates ( radians, km )
#include "c_stations.h"
!
!
! Functions
        double precision refrac         ! refraction function
!
! Local variables
        integer i                       ! loop variable
        integer iflag                   ! flag for SGP4 
        double precision tsince         ! extrapolation duration (mn)
        double precision t0             ! initial time for SGP/SDP
        double precision elem(10)       ! elements for mosaik
        double precision posvelteme(6)  ! position velocity in TEME ref. frame
        double precision posgrw(3)      ! position in Greenwich reference frame 
        double precision velgrw(3)      ! velocity in Greenwich reference frame
        double precision tini           ! initial time (sec)
        double precision tstep          ! time step (sec)
        double precision ijstep         ! julian instant of the step
        double precision zpre           ! z comp. for the ascending node test
        double precision znew           ! z comp. for the ascending node test
        double precision site           ! satellite elevation angle  (rad) 
        double precision sitec          ! site corrected for refraction (rad) 
        double precision zen            ! satellite zenith angle  (rad) 
        double precision azi            ! satellite azimut angle  (rad) 
        double precision dist           ! station - sat distance          (km)
        double precision nodper         ! computed nodal period (days)
        double precision tan            ! computed time of ascending node(jdays)
        double precision possun(3)      ! sun position in Greenwich ref. frame
        double precision distsun        ! sun-earth distance (km)
        integer orbitan                 ! orbit number at tan
        integer orbstdate               ! orbit number at start date 
        integer nloop                   ! number of steps to be calc.
        integer istep                   ! step number
        integer orbnum                  ! orbit number
        integer codsun                  ! 
        integer codvis                  ! celestial object visibility codes
!                                         0 = object is visible from the station
!                                         1 = object is not visible 
        character*22 ch_epoch           ! tle_epoch string
!
        logical deep
! 
!       for semi major axis
        double precision a1, cosio, theta2, x3thm1
        double precision eosq, betao2, betao, del1
        double precision ao, delo, aodp
!-
!
!       Section 1
!       compute the position velocity at epoch time
!       -------------------------------------------
        t0 = tle_epoch

        elem(1)  = tle_meanano
        elem(2)  = tle_rigasc
        elem(3)  = tle_argper
        elem(4)  = tle_eccent
        elem(5)  = tle_inclin
        elem(6)  = tle_meanmotion
        elem(7)  = tle_ftdmm
        elem(8)  = tle_stdmm
        elem(9)  = tle_bstar
        elem(10) = tle_epoch_tlf
        deep     = tle_deep

        !E1 common
        xmo    = elem(1)
        xnodeo = elem(2)  
        omegao = elem(3)
        eo     = elem(4)
        xincl  = elem(5)  
        xno    = elem(6)
        xndt2o = elem(7)
        xndd6o = elem(8)
        bstar  = elem(9)
        epoch  = elem(10)

        !Compute position at epoch
        tsince = 0.d0
        iflag = 1
        if(tle_deep) then
           call sdp4(iflag,tsince)
        else
           call sgp4(iflag,tsince)
        endif
        posvelteme(1) = x*XKMPER/AE
        posvelteme(2) = y*XKMPER/AE
        posvelteme(3) = z*XKMPER/AE
        posvelteme(4) = xdot*XKMPER/AE*XMNPDA/86400.
        posvelteme(5) = ydot*XKMPER/AE*XMNPDA/86400.
        posvelteme(6) = zdot*XKMPER/AE*XMNPDA/86400.

        !estimate semi major axis from TLE elements
        A1=(XKE/XNO)**TOTHRD
        COSIO=COS(XINCL)
        THETA2=COSIO*COSIO
        X3THM1=3.*THETA2-1.
        EOSQ=EO*EO
        BETAO2=1.-EOSQ
        BETAO=SQRT(BETAO2)
        DEL1=1.5*CK2*X3THM1/(A1*A1*BETAO*BETAO2)
        AO=A1*(1.-DEL1*(.5*TOTHRD+DEL1*(1.+134./81.*DEL1)))
        DELO=1.5*CK2*X3THM1/(AO*AO*BETAO*BETAO2)
        AODP=AO/(1.-DELO) * xkmper

!       Section 2
!       writes the header 
!       -----------------
        call xijc(tle_epoch,ch_epoch)
        write(*,"('tle epoch time      ',a22)") ch_epoch
        write(*,"('semi-major axis     ',f10.4)") aodp
        write(*,"('eccentricity        ',f10.9)") tle_eccent
        write(*,"('inclination         ',f10.5)") 
     &                                   tle_inclin * crd
        write(*,"('perigee argument    ',f10.5)") 
     &                                   tle_argper * crd
        write(*,"('right ascension     ',f10.5)")
     &                                   tle_rigasc * crd
        write(*,"('mean anomaly        ',f10.5)")
     &                                   tle_meanano * crd
        if( aodp .gt. 10000.) then
           write(*,"('x,y,z               ',3(f12.4,1x))")
     &                                   (posvelteme(i),i=1,3)
        else
           write(*,"('x,y,z               ',3(f10.4,1x))")
     &                                   (posvelteme(i),i=1,3)
        endif
        write(*,"('vx,vy,vz            ',3(f10.7,1x))")
     &                                   (posvelteme(i),i=4,6)
        write(*,"('station latitude    ',f10.4)")
     &                                   latstat(1) * crd
        write(*,"('station longitude   ',f10.4)")
     &                                   lonstat(1) * crd
        write(*,"('station altitude    ',f10.4)") altstat(1)
        write(*,"('stat. minimum site  ',f10.4)")
     &                         sitestat(1) * crd
        write(*,"('    step',
     &            '      x         y         z   ',
     &            '     vx        vy        vz      orb sun sta')")
!
!       Section 3
!       computes the loop variables
!       ---------------------------
!       initial time difference start-tle_epoch in seconds
        tini = (ijstdate - tle_epoch) * 86400.d0

        tstep = timestep * 1.d0
        nloop = (nbdays * 86400.d0) / tstep + 1
        if((nloop-1) * tstep /86400.d0 .lt. nbdays) nloop = nloop+1

        ! compute the nodal period (different from revs/days)
        ! and also the time of next ascending node (tan)
        call tle_fnode(tle_epoch,elem,deep,nodper,tan)

        ! Take care that reference for orbit numbers differs
        ! for 2LINE and TBUS by ONE.
        ! We keep TBUS reference.

        ! v1.2 25/06/12: NORAD TLEs are normally normally given at an ascending
        ! node, but EUMETSAT TLEs are not. The convention appears to be that
        ! NORAD orbit number is the number of the orbit that has just finished
        ! at the ascending node crossing. Hence a EUM TLE with an epoch just
        ! after the NORAD one will indicate the next orbit. AAPP defines
        ! "orbitan" to be the number of the orbit after the next equator
        ! crossing (see tle_forb.F).

        if ( abs(posvelteme(3)) .gt. 1.) then
           ! the position is not given for a node (Z > 1km)
           ! then we must compute an orbit number
!1.2           orbitan = tle_orbnum + 1 + nint((tan-tle_epoch) / nodper)
           orbitan = tle_orbnum + 1 + INT((tan-tle_epoch) / nodper)
        else
           ! if the elements are given for a node 
           ! then keep values given in the bulletin for orbit number and time
           ! of ascending node.
           tan = tle_epoch
           orbitan = tle_orbnum + 1
        end if

!       computes the orbit number at stdate

        call tle_forb(tan,orbitan,nodper,tle_epoch,elem,
     &                  deep,ijstdate,orbstdate)

        orbnum = orbstdate
        zpre = 1.d0
!
!       Section 4
!       compute the position for ecah time step
!       ---------------------------------------
        do istep=1,nloop

!         4.1 compute the osculating elements
!
!               ijstep is the julian time for the step
          ijstep = ijstdate + ( (istep-1)* tstep )/ 86400.d0
          !E1 common
          xmo    = elem(1)
          xnodeo = elem(2)  
          omegao = elem(3)
          eo     = elem(4)
          xincl  = elem(5)  
          xno    = elem(6)
          xndt2o = elem(7)
          xndd6o = elem(8)
          bstar  = elem(9)
          epoch  = elem(10)
          
          !Compute position at epoch
          tsince = (ijstep - t0) * 1440.
          iflag = 1
          if(tle_deep) then
             call sdp4(iflag,tsince)
          else
             call sgp4(iflag,tsince)
          endif
          posvelteme(1) = x*XKMPER/AE
          posvelteme(2) = y*XKMPER/AE
          posvelteme(3) = z*XKMPER/AE
          posvelteme(4) = xdot*XKMPER/AE*XMNPDA/86400.
          posvelteme(5) = ydot*XKMPER/AE*XMNPDA/86400.
          posvelteme(6) = zdot*XKMPER/AE*XMNPDA/86400.

!         4.2 compute the greenwich elements
          call pvtemegrw(ijstep,posvelteme(1),
     &                          posvelteme(4),posgrw,velgrw)
        
!         4.3 orbit number
          znew = posgrw(3)
          if( zpre.lt.0.d0 .and. znew.ge.0.d0 ) then
            orbnum = orbnum+1
            if ( orbnum .ge. 100000) orbnum = 1
          endif
          zpre = znew

!         4.4 is the satellite seen from the station?
          codvis = -1
          call trackang(posstat,matstat,1,posgrw,zen,azi,dist)
          site = pi_2 - zen
          sitec = site + refrac(site)
          if(sitec.ge.sitestat(1)) then
            codvis=0
          else
            codvis=1
          endif

!         4.5 is the satellite in daylight or nighttime conditions?
!             only computed when the satellite is seen from the station

          codsun = -1
          call sungrw(ijstep,possun,distsun)
          call sunsat(posgrw,possun,codsun)

!         4.6 writes the results on standard output

          if( aodp .gt. 10000.) then
             write(*,'(i8,3f12.3,3f10.6,i6,2i4,1x,a22)')
     &                  istep, 
     &                  posgrw, velgrw, orbnum,
     &                  codsun, codvis
          else
             write(*,'(i8,3f10.3,3f10.6,i6,2i4,1x,a22)')
     &                  istep, 
     &                  posgrw, velgrw, orbnum,
     &                  codsun, codvis
          endif


        end do

        return

        end
