C LIBS = atovin atov f7nl1b cal auxaapp
!+    Main program for module ATOVIN of ATOVS Processing Package
!
      PROGRAM ATOVIN
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: atovin.F 468 2015-11-20 08:31:27Z frna $
!
! Purpose:
!   - Read level 1B (A)TOVS data for (optionally) each of the four possible
!     (A)TOVS instruments: HIRS, AMSU-A, AMSU-B, MSU.
!   - Apply calibration coefficients to get radiances from instrument
!     counts, and convert radiances to brightness temperatures.
!   - For AMSU-A & B, correct radiances for antenna effects
!   - Perform quality control to check that the file increments
!     consistently in time, adding missing values where necessary.
!     Also basic quality control on the brightness temperatures.
!   - Write out as level 1C (A)TOVS data.
!
! Method:
! 1.  Read user requirements
! 2.  Set up variables
! 3.  Read in, process, and write out HIRS data     (if user requires)
! 4.  Read in, process, and write out AMSU-A data   (if user requires)
! 5.  Read in, process, and write out AMSU-B data   (if user requires)
! 6.  Read in, process, and write out MSU data      (if user requires)
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version    Date     Comment
!
!  1.0      2/2/96  Original code. (R.J.Renshaw)
!  1.1     12/12/96 Process ATOVS and TOVS.  RJR
!  1.2      5/3/03  Add FDFunit to INAMSA and INAMSB calls.  Keith Whyte
!  1.3     28/11/03 include mhs1b.h. (N.C.Atkinson)
!  1.4     24/11/04 Display CVS date rather than version
!  1.5     18/11/15 Include bits.h
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!
! (according to F77 standard, commons should be declared in top-level routine)
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  HIRS Data Variables for processing within ATOVIN 
#include "inhstore.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-B Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  MSU Data Variables for processing within ATOVIN 
#include "inmstore.h"
!  HIRS level 1B header/data common blocks 
#include "hrs1b.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!  AMSU-A level 1B header/data common blocks 
#include "ama1b.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  AMSU-B level 1B header/data common blocks 
#include "amb1b.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!  MHS level 1B header/data common blocks 
#include "mhs1b.h"
!  MSU level 1B header/data common blocks 
#include "msu1b.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      INTEGER       H1Bunit       ! unit number for HIRS level 1B file
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
      INTEGER       A1Bunit       ! unit number for AMSU-A level 1B file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       B1Bunit       ! unit number for AMSU-B level 1B file
      INTEGER       B1Cunit       ! unit number for AMSU-B level 1C file
      INTEGER       M1Bunit       ! unit number for MSU level 1B file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
      INTEGER       FDFunit       ! unit number for fixed data file
!
! Function & Subroutine calls:
      External INUSER             ! read user requirements
      External INSETU             ! set up variables
      External INHIRS             ! process HIRS
      External INAMSA            ! process AMSU-A
      External INAMSB            ! process AMSU-B
      External INMSU             ! process MSU
!
!- End of header

      WRITE(*,*)
      WRITE(*,*) "    ************************************************"
      WRITE(*,*) "    ***   Module ATOVIN $Date: 2005/04/15 13:40:02 $"
      WRITE(*,*) "    ************************************************"
      WRITE(*,*)

! 1.  Read user requirements
!     ---- ---- ------------

      CALL INUSER ( LHIRS, LAMSA, LAMSB, LMSU,
     &              H1Bunit, H1Cunit,
     &              A1Bunit, A1Cunit,
     &              B1Bunit, B1Cunit,
     &              M1Bunit, M1Cunit,
     &              FDFunit)


! 2.  Set up variables
!     --- -- ---------

      CALL INSETU ( LHIRS, LAMSA, LAMSB, LMSU,
     &              H1Bunit, H1Cunit,
     &              A1Bunit, A1Cunit,
     &              B1Bunit, B1Cunit,
     &              M1Bunit, M1Cunit,
     &              FDFunit)


! 3.  Read in, process, and write out HIRS data     (if user requires)
!     ---- --- -------- --- ----- --- ---- ----

      IF (LHIRS) CALL INHIRS ( H1Bunit, H1Cunit )


! 4.  Read in, process, and write out AMSU-A data   (if user requires)
!     ---- --- -------- --- ----- --- ------ ----

      IF (LAMSA) CALL INAMSA ( A1Bunit, A1Cunit, FDFunit )


! 5.  Read in, process, and write out AMSU-B data   (if user requires)
!     ---- --- -------- --- ----- --- ------ ----

      IF (LAMSB) CALL INAMSB ( B1Bunit, B1Cunit, FDFunit )


! 6.  Read in, process, and write out MSU data   (if user requires)
!     ---- --- -------- --- ----- --- --- ----

      IF (LMSU) CALL INMSU ( M1Bunit, M1Cunit )


      WRITE(*,*) " ATOVIN processing completed."

      STOP
      END
