#!/bin/ksh
#$Id: atovin.ksh,v 1.2 2005/04/15 13:40:02 frna Exp $
#
# script to run atovin program: process level 1b TOVS/ATOVS to level 1c
#
# usage: atovin [-f "infiles"] [instrument1] [instrument2] [instrument3]
# where: instrument1/2/3 can be any of HIRS, MSU, AMSU-A, AMSU-B
#        (upper or lower case).
#
# change 990427 to accomodate handling of 'stx-1 March99' amsu-b bias
# 23/11/04: add link to AMSU-B bias correction file
# 01/02/06: change config file P. Marguinaud
# 19/07/06: accept "MHS" in instrument list
# 19/07/12: Allow user to specify the input files via the -f "infiles" option.
#           File names must be in the same order as the requested instruments.
#           Output files will then have .l1c suffix.
# 06/08/12: Add quotes round [a-z] to avoid expansion by the shell.
###############################################################
echo atovin script params $*
. "$AAPP_PREFIX/ATOVS_CONF"

rm -f atovin.log

###############################################################
# Have input file names been specified? If not, default to aman.l1b, etc.
# The input files must be in the same order as the requested instruments.

hrs1bfile=hrsn.l1b   #default file names
ama1bfile=aman.l1b
amb1bfile=ambn.l1b
msu1bfile=msun.l1b

while getopts f: c
do
  case $c in
     f)	files="$OPTARG";;
    \?) echo "*** Unrecognised option in atovin, only -f is allowed"
        exit 2;;
  esac
done
shift `expr $OPTIND - 1`
#
# Have we got any arguments ?
if [[ $# = 0 ]]; then
   echo "*** Failure in atovin script  - no arguments"
   exit 1
fi

instruments="$(echo $* | tr '[a-z]' '[A-Z]' | sed s/MHS/AMSU-B/)"

if [ "$files" ]; then
  set $files         #load input files (if present) onto $*
  for instr in $instruments; do
    case $instr in
      HIRS)   hrs1bfile=$1;;
      AMSU-A) ama1bfile=$1;;
      AMSU-B) amb1bfile=$1;;
      MSU)    msu1bfile=$1;;
    esac
    shift    #next file name
  done  
fi

###############################################################
# Tell atovin.exe which instruments to process
# (converting instrument names to upper case)
echo " read $instruments " > atovin.input
#
###############################################################
# Assign files to unit numbers
rm -f ${FORT}*
#
# fixed data files
ln -s ${DIR_PREPROC}/fdf.dat ${FORT}10
#
# stx-1 March99 bias
ln -s ${DIR_PREPROC}/stx1_mar99corr.dat ${FORT}99
#
# AMSU-B bias corrections
ln -s ${PAR_CALIBRATION_COEF}/amsub/amsub_bias.dat ${FORT}17
#
# level 1b input files
ln -s $hrs1bfile ${FORT}11
ln -s $ama1bfile ${FORT}12
ln -s $amb1bfile ${FORT}13
ln -s $msu1bfile ${FORT}14
if [ $amb1bfile = ambn.l1b ] && [ ! -s ambn.l1b ] && [ -s mhsn.l1b ]; then
  ln -sf mhsn.l1b ${FORT}13    #use MHS if there is no AMSU-B
fi
#
# level 1c output files. Change the suffix to l1c and if the name contains the
# string "l1b" (e.g. output of AAPP_RUN_NOAA) change it to "l1c".
hrs1cfile=$(echo ${hrs1bfile%.*}.l1c | sed s/l1b/l1c/)
ama1cfile=$(echo ${ama1bfile%.*}.l1c | sed s/l1b/l1c/)
amb1cfile=$(echo ${amb1bfile%.*}.l1c | sed s/l1b/l1c/)
msu1cfile=$(echo ${msu1bfile%.*}.l1c | sed s/l1b/l1c/)
touch $hrs1cfile
touch $ama1cfile
touch $amb1cfile
touch $msu1cfile
ln -s $hrs1cfile ${FORT}21
ln -s $ama1cfile ${FORT}22
ln -s $amb1cfile ${FORT}23
ln -s $msu1cfile ${FORT}24
#
###############################################################
# Run atovin executable
atovin.exe < atovin.input >atovin.log 2>&1
exitc=$?
#
# Check error code
if [[ $exitc != 0 ]]; then
   echo "*** Error $exitc in atovin.exe"
   echo "*** Input file:"
   cat atovin.input
fi
#
# Clean up
rm -f atovin.input ${FORT}*
#
# Exit with the error code from the executable
exit $exitc
