C LIBS = atovin atov f7nl1b cal auxaapp
!+    Main program for module ATOVIN_ANTCORR of ATOVS Processing Package
!
      PROGRAM ATOVIN
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: atovin_antcorr.F 468 2015-11-20 08:31:27Z frna $
!
! Purpose:
!   Apply or remove antenna pattern corrections to AMSU-A / AMSU-B / MHS data 
!   supplied in level 1c format. There are two reasons why the user might want
!   to do this:
!   1) To process level 1b data that have been converted to level 1c format
!      (e.g. global METOP data from EUMETSAT)
!   2) To remove the AAPP antenna correction in order to use something
!      different.
!
! Method:
! 1.  Read user requirements
! 2.  Set up variables
! 3.  Read in, process, and write out AMSU-A data   (if user requires)
! 4.  Read in, process, and write out AMSU-B/MHS data   (if user requires)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2006, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0     02/11/06 Based on atovin. Nigel Atkinson
!  1.1     18/11/15 Include bits.h
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!
! (according to F77 standard, commons should be declared in top-level routine)
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-B Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  AMSU-B/MHS level 1C header/data common blocks 
#include "amb1c.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      INTEGER       H1Bunit       ! unit number for HIRS level 1B file
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
      INTEGER       A1Bunit       ! unit number for AMSU-A level 1B file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       B1Bunit       ! unit number for AMSU-B level 1B file
      INTEGER       B1Cunit       ! unit number for AMSU-B level 1C file
      INTEGER       M1Bunit       ! unit number for MSU level 1B file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
      INTEGER       FDFunit       ! unit number for fixed data file
      INTEGER       Mode          ! Mode=1 to apply, -1 to remove antenna corr
!
! Function & Subroutine calls:
      External INUSER             ! read user requirements
      External INSETU             ! set up variables
      External INHIRS             ! process HIRS
      External INAMSA            ! process AMSU-A
      External INAMSB            ! process AMSU-B
      External INMSU             ! process MSU
!
!- End of header

      WRITE(*,*)
      WRITE(*,*) "    ************************************************"
      WRITE(*,*) "    ***   Module ATOVIN_ANTCORR"
      WRITE(*,*) "    ************************************************"
      WRITE(*,*)

! 1.  Read user requirements
!     ---- ---- ------------

      CALL INUSER ( LHIRS, LAMSA, LAMSB, LMSU,
     &              H1Bunit, H1Cunit,
     &              A1Bunit, A1Cunit,
     &              B1Bunit, B1Cunit,
     &              M1Bunit, M1Cunit,
     &              FDFunit)

      READ(*,*) Mode
      
! 2.  Set up variables
!     --- -- ---------

      CALL INSETU ( LHIRS, LAMSA, LAMSB, LMSU,
     &              H1Bunit, H1Cunit,
     &              A1Bunit, A1Cunit,
     &              B1Bunit, B1Cunit,
     &              M1Bunit, M1Cunit,
     &              FDFunit)


! 4.  Read in, process, and write out AMSU-A data   (if user requires)
!     ---- --- -------- --- ----- --- ------ ----

      IF (LAMSA) CALL INAMSA_ANTCORR ( A1Cunit, FDFunit, Mode )


! 5.  Read in, process, and write out AMSU-B data   (if user requires)
!     ---- --- -------- --- ----- --- ------ ----

      IF (LAMSB) CALL INAMSB_ANTCORR ( B1Cunit, FDFunit, Mode )


      WRITE(*,*) " ATOVIN_ANTCORR processing completed."

      STOP
      END
