#!/bin/ksh
#$Id: atovin_antcorr.ksh 323 2012-08-09 13:09:46Z frna $
#
# script to run atovin_antcorr program: apply antenna corrections to level 1c
#
# usage: atovin_antcorr [-f "files"] [-z] [instrument1] [instrument2]
# where: instrument1/2 can be any of AMSU-A, AMSU-B, MHS
#        (upper or lower case).
#        use "-z" to undo an existing antenna correction
#
# 02/11/06: initial version, based on atovin.ksh. Nigel Atkinson 
# 19/07/12: Allow user to specify the input files via the -f "files" option.
#           File names must be in the same order as the requested instruments.
#           Output files will then have .l1c suffix. 
# 06/08/12: Add quotes round [a-z] to avoid expansion by the shell.
###############################################################

usage(){
  echo
  echo 'usage : atovin_antcorr [-f "infiles"] [-z] instruments'
  echo '  where instruments can be AMSU-A, AMSU-B or MHS'
  echo '  and the -z option may be used to undo an antenna correction'
  echo
}

ama1cfile=aman.l1c   #default file names
amb1cfile=ambn.l1c

# Set the mode (1 to apply corrections, -1 to remove)
mode=1
while getopts f:z c
do
  case $c in
     f)	files="$OPTARG";;
     z)	mode="-1";;
    \?) usage
        exit 2;;
  esac

done
shift `expr $OPTIND - 1`

# Have we got any arguments ?
if [[ $# = 0 ]]; then
   usage
   exit 1
fi

. "$AAPP_PREFIX/ATOVS_CONF"

rm -f atovin_antcorr.log

instruments="$(echo $* | tr '[a-z]' '[A-Z]' | sed s/MHS/AMSU-B/)"

if [ "$files" ]; then
  set $files         #load input files (if present) onto $*
  for instr in $instruments; do
    case $instr in
      AMSU-A) ama1cfile=$1;;
      AMSU-B) amb1cfile=$1;;
    esac
    shift    #next file name
  done  
fi

###############################################################
# Tell atovin_antcorr.exe which instruments to process
echo " read $instruments " > atovin_antcorr.input
echo $mode >> atovin_antcorr.input
#
###############################################################
# Assign files to unit numbers
rm -f ${FORT}*
#
# fixed data files
ln -sf ${DIR_PREPROC}/fdf.dat ${FORT}10
#
# level 1c files
ln -sf $ama1cfile ${FORT}22
ln -sf $amb1cfile ${FORT}23
if [ $amb1cfile = ambn.l1c ] && [ ! -s ambn.l1c ] && [ -s mhsn.l1c ]; then
  ln -sf mhsn.l1c ${FORT}23    #use MHS if there is no AMSU-B
fi
#
###############################################################
# Run atovin_antcorr executable
atovin_antcorr.exe < atovin_antcorr.input >atovin_antcorr.log 2>&1
exitc=$?
#
# Check error code
if [[ $exitc != 0 ]]; then
   echo "*** Error $exitc in atovin_antcorr.exe"
   echo "*** Input file:"
   cat atovin_antcorr.input
fi
#
# Clean up
rm -f atovin_antcorr.input ${FORT}*
#
# Exit with the error code from the executable
exit $exitc
