C LIBS = atovpp atov cal f7nl1b satid f7ml f7tp f7cp f7gp auxaapp
!+    Main program for module ATOVPP of ATOVS Processing Package
!
      PROGRAM ATOVPP
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: atovpp.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   - Read level 1C satellite data for (optionally) each of:
!     HIRS, AMSU-A, AMSU-B/MHS, MSU, IASI, ATMS, CrIS
!   - Identify and flag data unsuitable for subsequent use
!     in the main sounding application (e.g. contaminated by precipitation).
!   - (Optionally) Adjust brightness temperatures for effects undesirable
!     to some users (e.g. limb effects).
!   - Map data (BTs and flags) between the measurement grids of the
!     different instruments.
!   - Write out as level 1D (A)TOVS data, on the required grids.
!
! Method:
! 1.  Set up variables & read fixed data files
! 2.  Loop through blocks of data
! 3.  Write level 1D headers
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      11/06/98  AMSU-B to A mapping no longer done here. (R.J.Renshaw)
!  1.2      06/01/05  Add IASI (N.C.Atkinson)
!  1.3      23/09/08  Add IASIPC
!  1.4      13/08/10  Add ATMS and CrIS
!  1.5      07/02/19  Add HIRAS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!
! All common blocks are included here to comply strictly with the F77 standard.
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  AMSU-B/MHS level 1C header/data common blocks 
#include "amb1c.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!  IASI PC header/data common blocks 
#include "iasipc.h"
!  ATMS level 1C header/data common blocks 
#include "atms1c.h"
!  CRIS level 1C header/data common blocks 
#include "cris1c.h"
!  HIRAS level 1C header/data common blocks 
#include "hiras1c.h"
!  HIRS level 1D header/data common blocks 
#include "hrs1d.h"
! #include "hrs1dm.h"          ! HIRS/TOVS level 1D header/data common blocks
!  AMSU-A level 1D header/data common blocks 
#include "ama1d.h"
!  AMSU-B level 1D header/data common blocks 
#include "amb1d.h"
!  IASI level 1D header/data common blocks 
#include "iasi1d.h"
!  ATMS level 1D header/data common blocks 
#include "atms1d.h"
!  CRIS level 1D header/data common blocks 
#include "cris1d.h"
!  fixed data for processing HIRS 
#include "pphfdata.h"
!  fixed data for processing AMSU-A 
#include "ppafdata.h"
!  fixed data for processing AMSU-B 
#include "ppbfdata.h"
!  fixed data for processing MSU 
#include "ppmfdata.h"
!  fixed data for processing IASI 
#include "ppifdata.h"
!  block of HIRS data 
#include "pphstore.h"
!  block of AMSU-A data 
#include "ppastore.h"
!  block of AMSU-B data 
#include "ppbstore.h"
!  block of MSU data 
#include "ppmstore.h"
!  block of IASI data 
#include "ppistore.h"
!  block of ATMS data 
#include "ppatmsstore.h"
!  block of CRIS data 
#include "ppcrisstore.h"
!  block of HIRAS data 
#include "pphirasstore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B/MHS
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LIASI         ! .TRUE. to process IASI
      LOGICAL       LPCIASI       ! .TRUE. to process IASI PC
      LOGICAL       LATMS         ! .TRUE. to process ATMS
      LOGICAL       LCRIS         ! .TRUE. to process CRIS
      LOGICAL       LHIRAS        ! .TRUE. to process HIRAS
!
      LOGICAL       GHIRS         ! .TRUE. to output on HIRS grid
      LOGICAL       GAMSA         ! .TRUE. to output on AMSU-A grid
      LOGICAL       GAMSB         ! .TRUE. to output on AMSU-B grid
      LOGICAL       GIASI         ! .TRUE. to output on IASI grid
      LOGICAL       GATMS         ! .TRUE. to output on ATMS grid
      LOGICAL       GCRIS         ! .TRUE. to output on CRIS grid
      LOGICAL       GHIRAS        ! .TRUE. to output on HIRAS grid
!
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       B1Cunit       ! unit number for AMSU-B level 1C file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
      INTEGER       I1Cunit       ! unit number for IASI level 1C file
      INTEGER       IPCunit       ! unit number for IASI PC file
      INTEGER       atms1Cunit    ! unit number for ATMS level 1C file
      INTEGER       cris1Cunit    ! unit number for CRIS level 1C file
      INTEGER       hiras1Cunit   ! unit number for HIRAS level 1C file
!
      INTEGER       H1Dunit       ! unit number for HIRS level 1D file
      INTEGER       A1Dunit       ! unit number for AMSU-A level 1D file
      INTEGER       B1Dunit       ! unit number for AMSU-B level 1D file
      INTEGER       I1Dunit       ! unit number for IASI level 1D file
      INTEGER       atms1Dunit    ! unit number for ATMS level 1D file
      INTEGER       cris1Dunit    ! unit number for CRIS level 1D file
      INTEGER       hiras1Dunit   ! unit number for HIRAS level 1D file
!
      INTEGER       HLastLine     ! number of last HIRS line in store
      INTEGER       ALastLine     ! number of last AMSU-A line in store
      INTEGER       BLastLine     ! number of last AMSU-B line in store
      INTEGER       MLastLine     ! number of last MSU line in store
      INTEGER       ILastLine     ! number of last IASI line in store
      INTEGER       atmsLastLine  ! number of last ATMS line in store
      INTEGER       crisLastLine  ! number of last CRIS line in store
      INTEGER       hirasLastLine ! number of last HIRAS line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
                                  ! start date/time for the processing:
      INTEGER       Year0         !   year
      INTEGER       Day0          !   day
      INTEGER       Time0         !   time
!
      INTEGER       NBlocks       ! number of blocks dT to be processed
      LOGICAL       TOVS          ! TRUE if instrument is TOVS, else FALSE
!
      INTEGER       iBlock        ! loop index, looping through blocks of data
      INTEGER       imapping      ! loop index, looping through mappings
      INTEGER       NFOV_IASI     ! 4 FOVs (detectors) for IASI grid, else 1
      INTEGER       IFOV          ! loop index for IASI FOVs (range 1 to 4)
      LOGICAL       FirstFOV      ! First FOV in IASI loop
      LOGICAL       LastFOV       ! Last FOV in IASI loop
!
! Function & Subroutine calls:
      External PPSETUP              ! read 1C headers & set up mapping LUT
      External PPIN                 ! read in block of data (level 1C)
      External PPPROC1              ! pre-process block before mapping
      External PPMAP                ! map data between instrument grids
      External PPLUT_IASI           ! set up mapping for IASI
      External PPPROC2              ! pre-process block after mapping
      External PPOUT                ! write out block as level 1D
      External PPFINISH             ! write out level 1D headers
!
!- End of header

!     initialise for first call to PPIN
      DATA HLastLine, ALastLine, BLastLine, MLastLine, ILastLine,
     &  atmsLastLine,crisLastLine,hirasLastLine
     &    / 0, 0, 0, 0, 0, 0, 0, 0 /

      WRITE(*,*) "##############################"
      WRITE(*,*) "# Start ATOVS Pre-Processing #"
      WRITE(*,*) "##############################"

! 1.  Set up variables & read fixed data files
!     --- -- --------- - ---- ----- ---- -----

      CALL PPSETUP ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &               LATMS, LCRIS, LHIRAS,
     &               GHIRS, GAMSA, GAMSB, GIASI, GATMS, GCRIS, GHIRAS,
     &               H1Cunit, A1Cunit, B1Cunit, M1Cunit, I1Cunit,
     &               IPCunit, atms1Cunit, cris1Cunit, hiras1Cunit,
     &               H1Dunit, A1Dunit, B1Dunit, I1Dunit,
     &               atms1Dunit, cris1Dunit, hiras1Dunit,
     &               LSunit, HTunit,
     &               Year0, Day0, Time0, NBlocks, TOVS )


! 2.  Loop through blocks of data
!     ---- ------- ------ -- ----

      DO iBlock = 1, NBlocks

         WRITE(*,*)
         WRITE(*,"(""   Processing data block "",I3,"":"")") iBlock

! 2.1    Read block of level 1C data into buffers
         CALL PPIN ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &               LATMS, LCRIS, LHIRAS,
     &               Year0, Day0, Time0, iBlock,
     &               H1Cunit, A1Cunit, B1Cunit, M1Cunit, I1Cunit,
     &               IPCunit, atms1Cunit, cris1Cunit, hiras1Cunit,
     &               I1Dunit, cris1Dunit, hiras1Dunit,
     &               HLastLine, ALastLine,
     &               BLastLine, MLastLine, ILastLine, atmsLastLine,
     &               crisLastLine, hirasLastLine )

! 2.2    Pre-process data before mapping
         CALL PPPROC1 ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &           LATMS, LCRIS, LHIRAS,
     &           LSunit, HTunit, HLastLine, ALastLine, BLastLine, 
     &           MLastLine, ILastLine, atmsLastLine, crisLastLine,
     &           hirasLastLine)
           
! 2.3    Loop round up to 4 IASI FOVs and set up IASI mapping parameters

         DO IFOV = FirstFOV_IASI, LastFOV_IASI       !held in ppifdata.h
           FirstFOV = IFOV.EQ.FirstFOV_IASI
           LastFOV = IFOV.EQ.LastFOV_IASI

           IF (GIASI .AND. FirstFOV_IASI.NE.LastFOV_IASI) 
     &        CALL PPLUT_IASI(IFOV)   !Set up IASI mapping for each detector
         
! 2.4    map data between instrument grids
           DO imapping = 1, Nmappings
           
             IF ( MapInstr(imapping).EQ."ATMS" .AND.
     &            TargInstr(imapping).EQ."CRIS" ) THEN
               CALL map_atms_to_cris
             ELSE
           
              ! AMSU-B to A is done by PPAMSUA1 so don't do here
               IF ( .NOT. ( MapInstr(imapping).EQ."AMSU-B" .AND.
     &                     TargInstr(imapping).EQ."AMSU-A" ) ) THEN

                 IF ( FirstFOV .OR.
     &                TargInstr(imapping).EQ."IASI" ) THEN
                   CALL PPMAP ( MapInstr(imapping), TargInstr(imapping),
     &                    HLastLine, ALastLine, BLastLine, MLastLine,
     &                    ILastLine, IFOV )
                 ENDIF
               ENDIF
             ENDIF
           ENDDO  ! imapping
         ENDDO   ! IFOV

! 2.5    Process data after mapping (HIRAS not needed initially)
         CALL PPPROC2 ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LATMS, LCRIS,
     &           GHIRS, GAMSA, GAMSB, GIASI, GATMS, GCRIS,
     &           HLastLine, ALastLine, BLastLine, MLastLine, ILastLine, 
     &           atmsLastLine, crisLastLine)
           
! 2.6    Write out data as level 1D
         CALL PPOUT ( TOVS, GHIRS, GAMSA, GAMSB, GIASI, GATMS, GCRIS,
     &           GHIRAS,
     &           H1Dunit, A1Dunit, B1Dunit, I1Dunit,
     &           atms1Dunit, cris1Dunit, hiras1Dunit,
     &           HLastLine, ALastLine, BLastLine, ILastLine, 
     &           atmsLastLine, crisLastLine, hirasLastLine)

      ENDDO   ! iBlock


! 3.  Write level 1D headers
!     ----- ----- -- -------

      CALL PPFINISH ( TOVS, LAMSA, LAMSB, LMSU, LPCIASI,
     &                LATMS, LCRIS, LHIRAS,
     &                GHIRS, GAMSA, GAMSB, GIASI,
     &                GATMS, GCRIS, GHIRAS,
     &                H1Dunit, A1Dunit, B1Dunit, I1Dunit,
     &                atms1Dunit, cris1Dunit, hiras1Dunit )
      WRITE(*,*)
      WRITE(*,*) "############################"
      WRITE(*,*) "# End ATOVS Pre-Processing #"
      WRITE(*,*) "############################"


      STOP
      END
