#!/bin/ksh
# run ATOVPP
# 01/02/06   change config file P. Marguinaud
# 31/03/06   check whether IASI_eig_encode.dat needs updating. N. Atkinson
# 20/06/06   change == to = . NCA
# 21/01/08   for use with updated "iasi_eigenvectors" script
# 02/06/08   IASI.fdf should be in directory $DIR_IASI_PREPROC not $DIR_PREPROC
# 12/03/09   Add iasi pc input on unit 16. Add $IASI_FDF environment variable
# 06/09/10   Add ATMS and CrIS
# 19/07/12   Allow the user to set input file names via -f "infiles" option.
#            File names must be in the same order as the requested instruments.
#            Output files will be the same but with .l1d suffix and with 
#            "l1c" -> "l1d".
# 07/08/14   Change IASI.fdf to ${IASI_FDF} when checking npc
# 23/11/17   For CrIS, examine the end channel number of band 3 to set the
#            default fdf file: CRIS.fdf or CRIS_FSR.fdf.
# 07/02/19   Add HIRAS, processed in the same way as CrIS. 
#
# Optional environment variables:
#   $IASI_FDF          IASI fdf file which includes list of channels to 
#                      reconstruct (defaults to IASI.fdf)
#   $DIR_IASI_PREPROC  defaults to $DIR_PREPROC

usage(){
  echo
  echo 'usage : atovpp -g grids -i instruments [-r] [-f infiles]'
  echo
  echo '  where grids is a sub-set of "AMSU-A AMSU-B HIRS IASI ATMS CRIS HIRAS",'
  echo '  instruments is a sub-set of "AMSU-A AMSU-B HIRS IASI PCIASI ATMS CRIS HIRAS"'
  echo '  and infiles is an optional list of input files (same order as instruments).'
  echo '  "" quotes are necessary if there is more than 1 instrument or grid'
  echo 
  echo '  use -r to output reconstructed radiances in the IASI level 1d'
  echo  
  echo '  alternative syntax for backward compatibility: atovpp [A]TOVS [h]'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

IASI_FDF=${IASI_FDF:-IASI.fdf}
DIR_IASI_PREPROC=${DIR_IASI_PREPROC:-${DIR_PREPROC}}

#CRIS_FDF=${CRIS_FDF:-CRIS.fdf}    #comment out because we do this later
DIR_CRIS_PREPROC=${DIR_CRIS_PREPROC:-${DIR_PREPROC}}
DIR_HIRAS_PREPROC=${DIR_HIRAS_PREPROC:-${DIR_PREPROC}}

ATMS_FDF=${ATMS_FDF:-ATMS.fdf}
DIR_ATMS_PREPROC=${DIR_ATMS_PREPROC:-${DIR_PREPROC}}
ATMS_IGNORE=${ATMS_IGNORE:-atms_ignore.dat}

rm -f ${FORT}* atovpp.log
#
# set up inputs, according to input arguments
#
ATOVPP_USE_RR=${ATOVPP_USE_RR:-F}      #default is not to use RRs
while getopts g:i:rf: c
do
  case $c in
     g)	GRIDS=$OPTARG;;
     i)	INSTR=$OPTARG;;
     r) ATOVPP_USE_RR=T;;
     f) files="$OPTARG";;
     \?)      usage
              exit 2;;
  esac

done
export ATOVPP_USE_RR
      
#if the old syntax is being used, specify the instruments and grids

if [ ${#GRIDS} -eq 0 ] || [ ${#INSTR} -eq 0 ]; then
  shift `expr $OPTIND - 1`      #see if there is an argument
  if [ "$#" -eq 0 ]; then
    usage
    exit 1
  else 
    if [ $1 = ATOVS ]; then             #syntax of AAPP v5 and earlier
      INSTR="AMSU-A AMSU-B HIRS"
      if [ x$2 = xh ]; then             #second "h" argument is optional
        GRIDS="HIRS AMSU-B"
      else
        GRIDS=HIRS
      fi
    elif [ $1 = TOVS ]; then
      GRIDS=HIRS
      INSTR="MSU HIRS"
    else
      usage
      exit 1
    fi
  fi
fi    

#change "MHS" to AMSU-B" since MHS is processed as for AMSU-B in atovpp

INSTR=$(echo $INSTR | sed s/MHS/AMSU-B/)
GRIDS=$(echo $GRIDS | sed s/MHS/AMSU-B/)

#create the atovpp.inp file containing the required mappings

echo "instruments to read (level 1c):" $INSTR >atovpp.inp
echo "instrument grids to output (level 1d):" $GRIDS >>atovpp.inp

nmap=0

if [[ "$INSTR" = *AMSU-A* && "$INSTR" = *AMSU-B* ]] && \
   [[ "$GRIDS" = *AMSU-A* || "$GRIDS" = *HIRS* || "$GRIDS" = *IASI* ]]; then
  nmap=$(($nmap + 1))
  echo "$nmap) map AMSU-B to AMSU-A" >>atovpp.inp
fi

if [[ "$INSTR" = *AMSU-A* && "$INSTR" = *AMSU-B* ]] && \
   [[ "$GRIDS" = *AMSU-B* ]]; then
  nmap=$(($nmap + 1))
  echo "$nmap) map AMSU-A to AMSU-B" >>atovpp.inp
fi

if [[ "$INSTR" = *AMSU-A* ]] && \
   [[ "$GRIDS" = *HIRS* ]]; then
  nmap=$(($nmap + 1))
  echo "$nmap) map AMSU-A to HIRS" >>atovpp.inp
fi

if [[ "$INSTR" = *AMSU-A* ]] && \
   [[ "$GRIDS" = *IASI* ]]; then
  nmap=$(($nmap + 1))
  echo "$nmap) map AMSU-A to IASI" >>atovpp.inp
fi

if [[ "$INSTR" = *" MSU"* || "$INSTR" = MSU* ]] && \
   [[ "$GRIDS" = *HIRS* ]]; then
  nmap=$(($nmap + 1))
  echo "$nmap) map MSU to HIRS" >>atovpp.inp
fi

if [[ "$INSTR" = *ATMS* ]] && \
   [[ "$GRIDS" = *CRIS* ]]; then
  nmap=$(($nmap + 1))
  echo "$nmap) map ATMS to CRIS" >>atovpp.inp
fi

#If IASI Principal Components are required, call iasi_eigenvectors in order to 
#generate or update the binary eigenvectors files. Then make links for atovpp

#if [[ "$GRIDS" = *IASI* ]] && [[ "$INSTR" != *PCIASI* ]]; then
if [[ "$GRIDS" = *IASI* ]]; then
  npc=$(grep "!Number of PC scores" <${DIR_IASI_PREPROC}/${IASI_FDF} | awk '{print $1}')
  if [ npc -gt 0 ] ; then

    iasi_eigenvectors || exit 1

    IASI_SPECFILE=${IASI_SPECFILE:-iasi_eigenvectors_spec.dat}
    set $(awk '{print $1}' $DIR_IASI_PREPROC/$IASI_SPECFILE | grep -v "!")
    evapod=$5
    eig_encode=$6
    eig_decode=$7
    eig_noise=$8
    ln -s ${DIR_IASI_PREPROC}/$eig_encode ${FORT}46
    ln -s ${DIR_IASI_PREPROC}/$eig_decode ${FORT}47
    ln -s ${DIR_IASI_PREPROC}/$eig_noise  ${FORT}48
    ln -s ${DIR_IASI_PREPROC}/$evapod     ${FORT}49
    
  fi
fi

#
# level 1c input files
#
hrs1cfile=hrsn.l1c   #default file names
ama1cfile=aman.l1c
amb1cfile=ambn.l1c
msu1cfile=msun.l1c
iasi1cfile=iasi.l1c
iasipcfile=iasi.lpc
atms1cfile=atms.l1c
cris1cfile=cris.l1c
hiras1cfile=hiras.l1c

if [ "$files" ]; then
  set $files         #load input files (if present) onto $*
  for instr in $INSTR; do
    case $instr in
      HIRS)   hrs1cfile=$1;;
      AMSU-A) ama1cfile=$1;;
      AMSU-B) amb1cfile=$1;;
      MSU)    msu1cfile=$1;;
      IASI)   iasi1cfile=$1;;
      IASIPC) iasipcfile=$1;;
      ATMS)   atms1cfile=$1;;
      CRIS)   cris1cfile=$1;;
      HIRAS)  hiras1cfile=$1;;
    esac
    shift    #next file name
  done  
fi

#If ATMS is input, run atms_beamwidth.exe

rm -f atovpp.log
if [[ "$INSTR" = *ATMS* ]]; then
  atms_beamwidth $atms1cfile >>atovpp.log
fi

#If CRIS is input, use cris1c_h_endchan(3) to determine whether it is NSR or
#FSR, and set the appropriate default fdf file

if [[ "$INSTR" = *CRIS* ]]; then
  if [ -s "$cris1cfile" ]; then
    max_chan_number=$(od -An -t u4 -j 164 -N 4 $cris1cfile)
    if [ "$max_chan_number" -gt 2000 ]; then
      CRIS_FDF=${CRIS_FDF:-CRIS_FSR.fdf}
      echo "CrIS file is FSR with $max_chan_number channels"
    else
      CRIS_FDF=${CRIS_FDF:-CRIS.fdf}
      echo "CrIS file is NSR with $max_chan_number channels"
    fi
  else
    echo "Error: CrIS file $cris1cfile not found"
    exit 1
  fi
fi

#If HIRAS is input, use hiras1c_h_endchan(3) to determine whether it is NSR or
#FSR, and set the appropriate default fdf file

if [[ "$INSTR" = *HIRAS* ]]; then
  if [ -s "$hiras1cfile" ]; then
    max_chan_number=$(od -An -t u4 -j 164 -N 4 $hiras1cfile)
    if [ "$max_chan_number" -gt 2000 ]; then
      HIRAS_FDF=${HIRAS_FDF:-HIRAS_FSR.fdf}
      echo "HIRAS file is FSR with $max_chan_number channels"
    else
      HIRAS_FDF=${HIRAS_FDF:-HIRAS.fdf}
      echo "HIRAS file is NSR with $max_chan_number channels"
    fi
  else
    echo "Error: HIRAS file $hiras1cfile not found"
    exit 1
  fi
fi

ln -s $hrs1cfile ${FORT}11
ln -s $ama1cfile ${FORT}12
ln -s $amb1cfile ${FORT}13
ln -s $msu1cfile ${FORT}14
ln -s $iasi1cfile ${FORT}15
ln -s $iasipcfile ${FORT}16
ln -s $atms1cfile ${FORT}17
ln -s $cris1cfile ${FORT}18
ln -s $hiras1cfile ${FORT}19
if [ $amb1cfile = ambn.l1c ] && [ ! -s ambn.l1c ] && [ -s mhsn.l1c ]; then
  ln -sf mhsn.l1c ${FORT}13    #use MHS if there is no AMSU-B
fi
#
# level 1d output files
#
if [ $hrs1cfile = hrsn.l1c ]; then
  hrs1dfile=hirs.l1d
else
  hrs1dfile=$(echo ${hrs1cfile%.*}.l1d | sed s/l1c/l1d/)
fi
ama1dfile=$(echo ${ama1cfile%.*}.l1d | sed s/l1c/l1d/)
amb1dfile=$(echo ${amb1cfile%.*}.l1d | sed s/l1c/l1d/)
if [[ "$INSTR" = *PCIASI* ]]; then
  iasi1dfile=$(echo ${iasipcfile%.*}.l1d | sed s/lpc/l1d/)
else
  iasi1dfile=$(echo ${iasi1cfile%.*}.l1d | sed s/l1c/l1d/)
fi  
atms1dfile=$(echo ${atms1cfile%.*}.l1d | sed s/l1c/l1d/)
cris1dfile=$(echo ${cris1cfile%.*}.l1d | sed s/l1c/l1d/)
hiras1dfile=$(echo ${hiras1cfile%.*}.l1d | sed s/l1c/l1d/)

ln -s $hrs1dfile ${FORT}21
ln -s $ama1dfile ${FORT}22
ln -s $amb1dfile ${FORT}23
ln -s $iasi1dfile ${FORT}25
ln -s $atms1dfile ${FORT}27
ln -s $cris1dfile ${FORT}28
ln -s $hiras1dfile ${FORT}29
#
# fixed data files
ln -s ${DIR_PREPROC}/mapbitls.dat ${FORT}51
ln -s ${DIR_PREPROC}/maptopog.dat ${FORT}52
ln -s ${DIR_PREPROC}/HIRS.fdf  ${FORT}41
ln -s ${DIR_PREPROC}/AMSUA.fdf ${FORT}42
ln -s ${DIR_PREPROC}/AMSUB.fdf ${FORT}43
ln -s ${DIR_PREPROC}/MSU.fdf   ${FORT}44
ln -s ${DIR_IASI_PREPROC}/${IASI_FDF}  ${FORT}45
ln -s ${DIR_PREPROC}/LUT.fdf   ${FORT}50
ln -s ${DIR_PREPROC}/ppbgb2a.dat  ${FORT}70
ln -sf ${DIR_PREPROC}/safnwc_si_coast.dat safnwc_si_coast.dat  # Unit 53
ln -sf ${DIR_PREPROC}/safnwc_si_land.dat safnwc_si_land.dat    # Unit 53
ln -sf ${DIR_PREPROC}/safnwc_si_sea.dat safnwc_si_sea.dat      # Unit 53
ln -s ${DIR_ATMS_PREPROC}/${ATMS_FDF} ${FORT}54
ln -s ${DIR_CRIS_PREPROC}/${CRIS_FDF} ${FORT}55
ln -s ${DIR_CRIS_PREPROC}/${HIRAS_FDF} ${FORT}57
ln -s ${DIR_ATMS_PREPROC}/${ATMS_IGNORE} ${FORT}56
#Run atovpp.exe

atovpp.exe < atovpp.inp >>atovpp.log 2>&1
exitc=$?

rm -f ${FORT}* atovpp.inp safnwc*
exit $exitc
