#!/bin/ksh
#
# Run Avh2Hirs for TOVS and ATOVS
#
# To run this new version you have to add in ATOVS_ENV:
#  DIR_FORECAST=<< the directory of your forecasts >>
#		default is ${WRK}/forecast
#  DIR_MAIA2_ATLAS=<< the directory of the maia2 thresholds >>
#		no default 
#  DIR_MAIA2_THRESHOLDS=<< the directory of the maia2 thresholds >>
#		no default
#  FORECAST_FORMAT
#   if "grib" the pattern of the filename is YYYYMM/YYYYMMDDHHMN.HHH
#       with YYYYMMDDHHMN time and date of start of forecast and 
#       HHH the time period
#       and surface file is YYYYMM/YYYYMMDDHHMN.CST
#   else the pattern is  YYYYMM/YYYYMMDDHHMN_HHH.txt
#       and no surface file is needed  
#   default is "grib"
#  NFORPERDAY Nunmber of FORcasts PER DAY  can be 2 (00, 12H)
#		or 4 (00 06 12 18H)
#		default is 4
#
#  export DIR_FORECAST
#  export DIR_MAIA2_ATLAS
#  export DIR_MAIA2_THRESHOLDS
#  export FORECAST_FORMAT
#  export NFORPERDAY
#       01/02/06   change config file P. Marguinaud
#       09/06/06   support for metop ( SATNB= ) P.Marguinaud
#       20/06/06   change == to = . NCA
#       20/08/07   update for maia4
#       13/03/19   fix the "typeset" line for CentOS7 compatibility 


prep_env_maia()
{
. maia_env
. maia_date
# datation of the AVHRR level 1c file
#------------------------------------
FILE_1C=avh1c.h5
_YYYY=`get_attribute_h5 startdatayr ${FILE_1C}`
_DAY=`get_attribute_h5 startdatady ${FILE_1C}`
_TIME=`get_attribute_h5 startdatatime_ms ${FILE_1C}`
_MMDD=`date_MMDD ${_YYYY} ${_DAY}`
_HHMN=`date_HHMN ${_TIME}`
YYYYMMDD_Deb=${_YYYY}${_MMDD}
HHMN_Deb=${_HHMN}
log_info "start date:" ${YYYYMMDD_Deb}_${HHMN_Deb}

_YYYY=`get_attribute_h5 enddatayr ${FILE_1C}`
_DAY=`get_attribute_h5 enddatady ${FILE_1C}`
_TIME=`get_attribute_h5 enddatatime_ms ${FILE_1C}`
_MMDD=`date_MMDD ${_YYYY} ${_DAY}`
_HHMN=`date_HHMN ${_TIME}`
YYYYMMDD_End=${_YYYY}${_MMDD}
HHMN_End=${_HHMN}

log_info "stop date:" ${YYYYMMDD_End}_${HHMN_End}


case ${SATIMG} in
metop01|M01) SAT=M01;;
metop02|M02) SAT=M02;;
metop03|M03) SAT=M03;;
noaa16|N16) SAT=N16;;
noaa17|N17) SAT=N17;;
noaa18|N18) SAT=N18;;
noaa19|N19) SAT=N19;;
esac
maia4_env

if [ ! -s Bg_BeforObs -o ! -s Bg_AfterObs ]
then
  log_error "forecast Before or After not available"
  log_error "cannot run maia4"
  remove_maia4_env
  exit 1
fi
ls -l  Bg_BeforObs
ls -l  Bg_AfterObs


} 


. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh

if [ $# -ne 1 ]; then
  echo " Arguments expected!!"
  echo " Argument = TOVS or ATOVS"
fi

A_TOVS=$1
rm -f ${FORT}*



###############################################
# Main
###############################################


# point to avhrr level 1b file
#------------------------------
if [ ! -f ${WRK}/hrpt.l1b ]; then
  echo " No ${WRK}/hrpt.l1b available"
  exit 1
fi

l1b=${WRK}/hrpt.l1b


# point to hirs level 1d file
#-----------------------------
if [ ! -f ${WRK}/hirs.l1d ]; then
  echo " No ${WRK}/hirs.l1d available"
  exit 1
fi

#Finds the satellite and datation using the level 1d file
#--------------------------------------------------------
set `l1didf -i  ${WRK}/hirs.l1d`
SATIMG=$1
YYYYMMDD=$2
HHMN=$3
NNNNN=$4
BASE=`echo ${SATIMG} | cut -c1-4`
if [ "$BASE" = "noaa" ]
then
  SATNB=`echo ${SATIMG} | cut -c5-6`
else
  SATNB=`echo ${SATIMG} | cut -c6-7`
fi
UNIT=`expr ${SATNB} + 50`
MMC=`echo ${YYYYMMDD} | cut -c5-6`

# prepare avh1c file for avh2hirs and avhh2hirs_atovs if it is
# possible to run maia (forecasts available)

typeset -l RUN_MAIA=${RUN_MAIA:=no}     # default is not to run MAIA
echo "RUN_MAIA=${RUN_MAIA}"
if [ "${RUN_MAIA}" = "yes" ]
then
    l1c=${l1b}.l1c
    log_info "avhrrin"
    avhrrin -i ${l1b} -o ${l1c}
    # convert  1c to h5
    avh=${l1c}.h5
    log_info "convert to h5 file"
    rm -f avh1c.h5
    avh1c_to_hdf5.exe ${l1c} avh1c.h5
    if [ ! -f avh1c.h5 ]
    then
     log_warning "cannot run maia"
     RUN_MAIA=no
    fi
else
   rm -f avh1c.h5
   log_info "no maia4"
fi

# point to time and angle correction file (${FORT}64=n14)
#--------------------------------------------------------
if [ ! -f ${DIR_PREPROC}/cor_${SATIMG}.dat ]; then
  echo " No cor_${SATIMG}.dat  available"
  exit 1
else
  ln -s ${DIR_PREPROC}/cor_${SATIMG}.dat   ${FORT}${UNIT}   
fi

# Runs Avh2Hirs according to TOVS / ATOVS
#----------------------------------------
# prepare maia4 environement
if [ "${RUN_MAIA}" = "yes" ]
then
  prep_env_maia
fi
ln -s ${WRK}/hirs.l1d ${FORT}12
ln -s ${WRK}/hrpt.l1b  ${FORT}11

rm -f avh2hirs.log
if [ ${A_TOVS} = TOVS ]; then
  log_info "avh2hirs.exe"
  avh2hirs.exe > avh2hirs.log 2>&1
else
  log_info "avh2hirs_atovs.exe"
  avh2hirs_atovs.exe > avh2hirs.log 2>&1
fi


# Maia end
#----------------------
if [ "${RUN_MAIA}" = "yes" ]
then
remove_maia4_env
rm -f  avh1c.h5
fi
# Quality of the alignment AVHRR / HIRS
#--------------------------------------
cat ${FORT}22 >>  mapqual_${SATIMG}.txt   

# end of avh2hirs
#----------------
rm -f ${FORT}*

exit 0

