!LIBS = avh2hirs maia4 maia4IO aapp_viirs atovpp atov f7nl1b avhrcl satid f7ml f7tp f7cp f7gp auxaapp
!
!NOM/NAME $Id: avh2hirs_atovs.F 635 2019-04-15 15:08:00Z frna $
      program avh2hirs_atovs
!-----------------------------------------------------------------------
!
! ATOVS version of avh2hirs
!
! BUT/PURPOSE: 
!     Fill the 13 AVHRR variables for HIRS level 1d format.
!     This program creates a AVHRR cloud mask inside the HIRS ellipse
!     and calculates the cloud free percentage and surface temperature.
!
!     The first program was written by P. Dibben and R.J.Renshaw (UKMO)
!     all the mapping routines are provided by UKMO.
!
!     The AVHRR is read by areas of ~400 lines in level 1b format
!     it is converted in brightness temperature and radiances
!     by the same algorithm as avh1bto1c
!
!     The cloud mask is done by the use of the MAIA library
!     2 possibilities for the 13 AVHRR parameters according to 
!     avaibility of maia (maia needs forecasts and hdf5 library)
!     1- with maia 
!               1 - pourcentage clear AVHRR in HIRS FOV (*100)
!               2 - surface temperature (K*100)
!               3 - climatological temperature or t2m (K*100)
!               4 - mean AVHRR channel 3 over HIRS FOV (degK*100)
!               5 - mean AVHRR channel 4 over HIRS FOV (degK*100)
!               6 - mean AVHRR channel 5 over HIRS FOV (degK*100)
!               7 - percentage of black body in HIRS FOV (*100)
!               8 - top cloud temperature over HIRS FOV (degK*100)
!               9 - standard deviation corresponding to the top cloud T (degK*100)
!              10 - mean clear AVHRR channel 4 over HIRS FOV (degK*100)
!              11 - mean clear AVHRR channel 5 over HIRS FOV (degK*100)
!              12 - std  AVHRR channel 4 over HIRS FOV (degK*100)
!              13 - std  clear AVHRR channel 4 over HIRS FOV (degK*100)
!
!       The "mean Bts" are computed with the real
!       mean radiance (cloudy or clear) and converted in temperature.
!
!       2- without maia  only parameters 4,5,6 and 12 are filled
!        
!     Files are:
!      unit     Description
!       11      AVHRR level 1b
!       12      HIRS level 1d (TOVS == with MSU)
!       21      calibration parameter file
!       22      mapping quality historical file (H8-A4 st-dev)
!       50+sat number   time and angle correction file !
!       for both modes the "mean Bts" are computed with the real
!       mean radiance (cloudy or clear) and converted in temperature.
! 
!     Files are:
!      unit     Description
!       11      AVHRR level 1b
!       12      HIRS level 1d (TOVS == with MSU)
!       21      calibration parameter file
!       22      mapping quality historical file (H8-A4 st-dev)
!       50+sat number   time and angle correction file 
! 
! DESCRIPTION: FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!       see documentations     
!
! PROPRIETAIRE/OWNER:
!
! HISTORIQUE/HISTORY  (MODIFICATIONS/CHANGES)
!
! version       date            comment                 author
!   1.0         Oct 97                               P. Brunel 
!   2.0         dec 2000  upgrade MAIA 2.1     P. Brunel - L. Lavanant
!   3.0       8 Jan 2002  Add test and comments when variables count and 
!                         count_all equal 0.         T.Labrot
!   4.0         march 03  f90 test                   T.Labrot
!             Changed  type XXX/yyy/
!                   to type XXX
!                      parameter(XXX=yyy) or data XXX/yyy/
!   4.1         march 04  Modify for linux           N Atkinson / A Korpela
!   4.2         July 04  Add satid in call to lutmap  N Atkinson
!   4.3         May 06 Add support for metop PM/PB
!               Test if map_nlines > 0; without this test, coredump
!               on short AVHRR data; test if 285-t23 > 0 & 285-t31 > 0
!               in VAP calculation
!   4.4         Increase max number of scan lines from 160 to 350, to support 
!               MetOp fast extract from Svalbard
!   8.0         Use maia4 instead of maia2
!   8.1         September 17 fix line too long
!
! ARGUMENTS: 
!
!
! FUNCTIONS AND SUBROUTINES CALLS (RETURN CODES FOR FUNCTIONS):
!
!
!-----------------------------------------------------------------------
      use mod_maia_wrapper
      use mod_maia_types
      implicit none

!---- DECLARATIONS -----------------------------------------------------

!---- ARGUMENTS --------------------------------------------------------
       integer lupar,lu1b,lu1d,lucor,lumq
       parameter (lupar=21)     ! avhrr calibration parameter file
       parameter (lu1b=11)      ! lu for avhrr l1b
       parameter (lu1d=12)      ! lu for hirs l1d
       parameter (lumq=22)      ! lu for mapping quality
       data lucor/50/           ! lu for alignment corrections    
!---- COMMONS ----------------------------------------------------------
#include "avh1b.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  HIRS/ATOVS level 1D header/data common blocks 
#include "hrs1d.h"
#include "reclopt.h"
!
!---- PARAMETERS -------------------------------------------------------
!---- LOCAL VARIABLES --------------------------------------------------

      integer ios,c, prct

      ! Avhrr quality control
      integer good_lines(6),bad_lines(6)
      data good_lines/6*0/
      data bad_lines/6*0/
      integer mis_lines
      data mis_lines/0/      

      integer missing
      character*2 chn_name(6)
      data chn_name/' 1',' 2','3B',' 4',' 5','3A'/
      
      integer nb_lines                ! force integer*4 for avh_h_scnlin
      integer  i4_avh_h_satid         !NOAA spacecraft identification code

      ! time and angle corrections
      real      time_cor(90),scan_angle_cor(90)

      ! data areas for lutmap
      character*5 cmap_instr
      parameter (cmap_instr='AVHRR')
      character*4 ctarg_instr
      parameter (ctarg_instr='HIRS')

      integer   elms_per_coloc
      parameter (elms_per_coloc=3)

      integer   max_colocs
      parameter (max_colocs=40)
      
      integer   targ_nfovs
      parameter (targ_nfovs=56)
      
      integer   targ_lines
      parameter (targ_lines=5)
      
      integer   time_diff
      real      lut_buf(3,40,56,5)
      real      ht_of_sat,period_of_sat
      
      integer   targ_start_line
      data targ_start_line/4/
      
      integer   mult_factor
      parameter (mult_factor=10)
      
      integer   duration
      parameter (duration=32)

      integer   spatial_mode
      parameter (spatial_mode=0) 
      
      integer   spatial_mult       ! both dont care
      parameter (spatial_mult=100)
      
      integer   status
      integer   lmin
      integer   lmax
      integer   ind

      ! data areas for av_map
      integer avh_max_lines
      parameter (avh_max_lines=400)

      real        miss_data
      parameter (miss_data=-9999.9e02)
      
      character*80  errmessage
      integer   map_nfovs
      parameter (map_nfovs=2048)
      
      integer   map_nlines
      integer   map_mxch
      parameter (map_mxch=5)
      
      integer   map_line0,map_chan0,map_bitqc
      data map_line0/0/
      data map_chan0/0/
      data map_bitqc/0/
      
      integer   map_lut_lines
      data map_lut_lines/192/
      
      integer   map_surf(2048,avh_max_lines)
      real      map_bts(5,2048,avh_max_lines)   ! 5 chans, 2048 fovs
      real      map_rad(5,2048,avh_max_lines)   ! 5 chans, 2048 fovs
      integer   map_qc(2048,avh_max_lines)! fov quality
       integer  map_bitcloud1,map_bitcloud2
      data map_bitcloud1/1/
      data map_bitcloud2/2/
      
      integer   targ_nlines
      integer   targ_mxch
      parameter (targ_mxch=13)
      
      integer   targ_line0,targ_chan0,targ_bitqc
      data targ_line0/0/
      data targ_chan0/0/
      data targ_bitqc/0/
      
      real targ_angles(6,56,350)        ! geometry angles (deg)
        ! 1 = latitude, 2 = longitude
        ! 3 = local zenith angle   4 = local azimuth angle
        ! 5 = solar zenith angle   6 = solar azimuth angle
      real      targ_alt(56,350)        ! surface altitude (m)
      integer   targ_surf(56,350)       ! surface type (0=sea 1=mixed 2=land)
      real      targ_bts(13,56,350)    ! for mapped data
      integer   targ_qc(56,350)
      integer   amsu_wv(56,350)   ! amsu total water vap content (put to -999)
      integer   surf_option
      parameter (surf_option=0)
      
      integer   mode 

      
      integer clear_box(33,38)   ! beware ,see av_map parameters 
                                 ! map_box_psize and map_box_lsize
      
      integer   errstatus

      integer   avh_start_time, hrs_start_time

      integer irec
      integer ii
      integer jj

      integer offset_read
      integer targ_nlines_av
      integer map_lmin_pre
      integer map_lmax_pre
      integer lin_pre
      integer map_lmin_new
      integer tline
      integer mline
      integer map_lmin
      integer map_lmax
      integer map_lin

      character *7 satimg         ! satellite name (6char) noaaxx ex: noaa16 

! contenu integre en vapeur d'eau
      real t23, t31, t50 ! AMSU-A canals
      real DF1 ! identify sea ice
      real Z ! local zenith angle
      real A, B, CC ! coefficients
      real VAP(56,350) ! Total Precipitable Water
      integer value

! data areas for stats on (hirs8-avh4) diffs
      real*8     sum(56),sumsq(56),sum_all,sumsq_all
      integer    count(56),count_all
      real       diff,mean(56),sd(56),mean_all,sd_all
      integer    idiff,hist(-30:30)
       integer dq(2)    ! date (year, day of year)
       integer dg(3)    ! date (year month day)
! data for maia4

      integer nblines
      integer nbpixels
      integer idbg                 ! debug level (0= nodebug, 2 is maximum)
      integer k
      integer p
      integer l
      logical has_maia

!---- EQUIVALENCES -----------------------------------------------------
!---- FUNCTIONS --------------------------------------------------------

!---- EQUIVALENCES -----------------------------------------------------
!---- FUNCTIONS --------------------------------------------------------
!---- EXTERNALS --------------------------------------------------------
      External IOH1D        ! read HIRS/TOVS 1D scan line
!---- CODE ------------------------------------------------------------- 
      idbg = 0 
      mode = 1
#ifdef HAS_LIBHDF5
! maia4
      inquire(file='avh1c.h5', exist=has_maia)
      if (has_maia) then 
        mode = 2
        call ml_wt('info','entering avh2hirs maia4')
      call ml_wti('info','mode = ',mode)
        idbg = 0
        write(*,*) "call maia_wrapper"  
        call maia_wrapper(idbg,map_maia,nblines,
     &                  nbpixels,.FALSE.) 
        write(*,*) "maia4 processed avhrr array dim:",nblines,nbpixels


       endif
#else
      write(0,'("avh2hirs.exe has not been built with HDF-5 Fortran")')
      write(0,'("interface. Please run configure with")')
      write(0,'("--external-libs=-Lxxx/lib -lhdf5hl_fortran")')
      write(0,'(" and --external-includes=-Lxxx/include")')
      write(0,'(" where xxx is the location of the HDF-5 library")')
      write(0,'("continue processing without maia cloud mask")')
      has_maia = .false.

#endif

! AVHRR 1b
!==========
!ouverture du fichier avhrr l1b
!------------------------------
      open(unit=lu1b,
     &          iostat=ios,
     &          status='old',
     &          form='unformatted',
     &          access='direct',
     &          recl=reclavh/rldiv)

      !Need status='unknown' for Linux ifc compiler
      if (ios.ne.0)
     &  open(unit=lu1b,
     &          iostat=ios,
     &          status='unknown',
     &          form='unformatted',
     &          access='direct',
     &          recl=reclavh/rldiv)

      if(ios.ne.0) then
        call ml_wti('ERROR','pb open l1b',ios)
        call aappexit(2)
      endif
    
!lecture header
!--------------
      ios=0
      call avhhdr(lu1b,ios) 
      if(ios.ne.0) then
        call ml_wti('ERROR','pb read header 1b',ios)
        call aappexit(2)
      endif
      i4_avh_h_satid = avh_h_satid
      call ml_wti('info','avhrr satellite ',i4_avh_h_satid)
      map_nlines=avh_h_scnlin
      write(*,*) 'info map_nlines',map_nlines
      avh_start_time=avh_h_startdatatime            ! used for monitoring only

! HIRS 1d
!=========
!ouverture du fichier HIRS 1d
!-----------------------------
       CALL IOH1D ("OPEN/WRITE", lu1d, irec)
    
!lecture header
!--------------
       irec=1
       CALL IOH1D ("READ HEADER", lu1d, irec)

       call ml_wti('info','start orbit',hrs1d_h_startorbit)
       call ml_wti('info','nblines hirs',hrs1d_h_scnlin)
       if( hrs1d_h_satid .le. 4 ) then
         write(satimg,"('meto',i2.2)") hrs1d_h_satid
       else
         write(satimg,"('noaa',i2.2)") hrs1d_h_satid
       endif
       call ml_wt('info','hirs satellite '//satimg)
       targ_nlines=hrs1d_h_scnlin
       hrs_start_time=hrs1d_h_startdatatime 
       ht_of_sat = hrs1d_h_satht /10.           !altitude (km)
       period_of_sat =  hrs1d_h_period /60.     !periode (mn)

! read 1st hirs data record
!--------------------------
      irec = 1+hrs1d_h_hdrcnt
      CALL IOH1D ("READ DATA", lu1d, irec)
      call ml_wti('info','hirs line ',hrs1d_scnlin)
      call ml_wti('info','hirs time (msec)',hrs1d_scnlintime)

! read all Hirs Data and store viewing geometry and surface
!----------------------------------------------------------
      do tline= 1, targ_nlines
         irec = tline + hrs1d_h_hdrcnt
         CALL IOH1D ("READ DATA", lu1d, irec)
         do ii = 1, Hnfovs
            targ_angles(1,ii,tline) = hrs1d_latlon(1,ii) /10000.
            targ_angles(2,ii,tline) = hrs1d_latlon(2,ii) /10000.
            targ_angles(3,ii,tline) = hrs1d_angles(1,ii) /100.
            targ_angles(4,ii,tline) = hrs1d_angles(2,ii) /100.
            targ_angles(5,ii,tline) = hrs1d_angles(3,ii) /100.
            targ_angles(6,ii,tline) = hrs1d_angles(4,ii) /100.
            targ_alt(ii,tline) = hrs1d_surface(1,ii)
            targ_surf(ii,tline) = hrs1d_surface(2,ii)


            ! Contenu integre en vapeur d eau
            !---------------------------------
            t23=hrs1d_btemps(21,ii) /100.
            t31=hrs1d_btemps(22,ii) /100.
            t50=hrs1d_btemps(23,ii) /100.

            DF1=2.85 + 0.020*t23 - 0.028*t50  ! identify sea ice
            Z=(hrs1d_angles(1,ii) /100.)*3.14/180.

            if( (ABS(hrs1d_latlon(1,ii) /10000.) .le. 50 .or.
     &                                      DF1.le.0.2) .and.
     &          ( 285-t23 .gt. 0 ) .and. ( 285-t31 .gt. 0 )
     &      ) then
               A= 247.92 - (69.235 - 44.177*cos(Z))*cos(Z)
               B= -116.27
               CC= 73.409
               VAP(ii,tline)= A+ B*alog(285-t23) + CC*alog(285-t31)
               VAP(ii,tline)= VAP(ii,tline)* cos(Z)
            else  
               VAP(ii,tline)=0
            endif
            !---------------------------------
            
         end do
      end do
    
      do tline=1,targ_nlines
         do ii = 1, Hnfovs
            value=nint((VAP(ii,tline))/1.)
            amsu_wv(ii,tline)=value*10
         end do
      end do
      
! Time and angle correction
!=============================
! open and read the file
      lucor=lucor+hrs1d_h_satid
      call ml_wti('info','lucor ',lucor)
      open(unit=lucor,status='old',access='sequential',iostat=ios)

      !Need status='unknown' for Linux ifc compiler
      if (ios.ne.0)
     & open(unit=lucor,status='unknown',access='sequential',iostat=ios)

      do ii=1,10
        read(lucor,'(9f8.4)') (time_cor(jj),jj=(ii-1)*9+1,ii*9)
      enddo

      do ii=1,10
        read(lucor,'(9f8.4)') (scan_angle_cor(jj),jj=(ii-1)*9+1,ii*9)
      enddo
      close(lucor)
      call ml_wtr('info','time correction pixel 28 (sec)',time_cor(28))
      call ml_wtr('info','scan correction pixel 28 (deg)',
     &      scan_angle_cor(28))
      
      nb_lines=avh_h_scnlin
      call ml_wti('info','number of lines to process',nb_lines)


! Initialize Lut Map
!=====================
      time_diff=avh_start_time-hrs_start_time
      call ml_wti('info','avh2hirs: time diff (msecs)',time_diff)
      if(abs(time_diff/1000).gt.32) then
        call ml_wti('warning','avh2hirs: bad time diff (secs)',
     &           time_diff/1000)
      endif

! create look up table
      call lutmap(cmap_instr, ctarg_instr,
     &            targ_start_line,
     &            targ_nfovs, targ_lines,
     &            mult_factor, duration, time_diff,
     &            time_cor, scan_angle_cor, lut_buf,
     &            elms_per_coloc, max_colocs,
     &            ht_of_sat, period_of_sat, spatial_mode, spatial_mult,
     &            hrs1d_h_satid, hrs1d_h_wmosatid, status) 

! compute the minimum and maximum Avhrr line numbers for the 5 Hirs lines
      lmin=9999
      lmax=0
      do ii=1,5
         do jj=1,56
            if(lut_buf(1,2,jj,ii) .lt. lmin )lmin = lut_buf(1,2,jj,ii)
            ind=nint(lut_buf(3,1,jj,ii)) + 1
            if(lut_buf(1,ind,jj,ii).gt.lmax )lmax = lut_buf(1,ind,jj,ii)
         end do
      end do
      ! take a half HIRS fov for security for the minimum
      lmin=lmin - lut_buf(3,1,1,1)/2
      lmax=lmax + 1

      offset_read = 192/targ_lines * (targ_start_line-1)

      map_lmin_pre=-999
      map_lmax_pre=-999
      map_line0=-999

!Line loop
!==========
      do tline=1,targ_nlines,targ_lines
         call ml_wti('info','Target line',tline)
! read avhrr into buffer
         map_lmin=lmin+(tline/targ_lines)*192 - offset_read
         map_lmax=lmax+(tline/targ_lines)*192
         map_lin=0
         if(map_lmin.le.0) map_lmin=1
         if(map_lmax.gt.avh_h_scnlin) map_lmax=avh_h_scnlin
         if(map_lmax-map_lmin+1 .gt. avh_max_lines) then
            call ml_wt('error','trop de lignes a charger')
            call aappexit(2)
         end if
     
         map_lmin_new = map_lmin
         if ( idbg .eq. 2 ) then
           write(*,*) "map_lmin",map_lmin,"map_lmax",map_lmax
           write(*,*)  "map_lmax_pre",map_lmax_pre
         endif
         if(map_lmax_pre.gt.0) then
            if ( idbg .eq. 2 ) then
              write(*,*)'copies from ',map_lmin,' to ',map_lmax_pre 
              write(*,*)" map_lmin-map_line0:",map_lmin-map_line0
              write(*,*)" map_lmax_pre-map_line0:",
     &                               map_lmax_pre-map_line0
            end if  
            do lin_pre= map_lmin-map_line0, map_lmax_pre-map_line0
               map_lin = map_lin+1
               !put in "line" map_lin "line" lin_pre
               do ii=1,map_nfovs
                  do jj=1,map_mxch
                     map_bts(jj,ii,map_lin)=map_bts(jj,ii,lin_pre)
                  end do
                  do jj=1,5
                     map_rad(jj,ii,map_lin)=map_rad(jj,ii,lin_pre)
                  end do
                  if (has_maia) then
          call maia_param_copy_item( map_maia(ii,lin_pre),
     &                               map_maia(ii,map_lin))

                  endif
                  map_qc(ii,map_lin) = map_qc(ii,lin_pre)
               end do
            end do
            map_lmin_new = map_lmax_pre+1
         end if
              
         !read avhrr from map_lmin_new to map_lmax
         do mline=map_lmin_new, map_lmax
            map_lin = map_lin+1

            call avhdtr(lu1b,mline,ios) 
            if(ios.ne.0) then
               call ml_wti('ERROR','pb read l1b',ios)
               call aappexit(2)
            endif
            
            call avh_lbc(mline, 
     &                     map_rad(1,1,map_lin),
     &                     map_bts(1,1,map_lin),
     &                     map_qc(1,map_lin),
     &                     map_bitqc,
     &                     missing,
     &                     good_lines,
     &                     bad_lines)
            mis_lines = mis_lines + missing
          if (has_maia) then
            do ii = 1,map_nfovs
       call maia_param_copy_item( map_maia(ii,mline),
     &                            map_maia(ii,map_lin))
             
            end do
          endif
         enddo

         map_lmin_pre = map_lmin
         map_lmax_pre = map_lmax
     
! now map avhrr to hirs; mapped data returned in targ_bts
         map_line0=map_lmin-1
         targ_line0=tline-1
         map_nlines = map_lmax-map_lmin+1

         if( map_nlines .gt. 0 ) then

           targ_nlines_av=targ_lines
           if(targ_line0+targ_nlines_av .gt. targ_nlines ) then
              targ_nlines_av=targ_nlines-targ_line0
           endif

         call av_map_maia(
     &              has_maia, 
     &              map_nfovs, map_nlines, map_mxch, map_line0,
     &              map_chan0, map_bitqc, map_surf,
     &              map_rad, map_bts, 
     &              map_qc, 
     &              map_bitcloud1,
     &              targ_nfovs,targ_nlines_av,targ_mxch,targ_line0,
     &              targ_chan0,targ_bitqc,
     &              targ_surf(1,tline),
     &              targ_bts(1,1,tline),targ_qc,
     &              elms_per_coloc,max_colocs,targ_lines,
     &              targ_start_line,map_lut_lines,lut_buf,
     &              surf_option,mode,miss_data,
     &              errstatus,errmessage,
     &              clear_box)
         write(*,*) "la", errstatus
          if(errstatus.ne.0) then
             call ml_wti('warning','av_map status',errstatus)
             call ml_wt('warning',errmessage)
          end if 

        else
        
          call ml_wt( 'warning', ' map_nlines < 0 ' )
        
        endif
               
      enddo !line loop
      
      write(*,*) 'Differences H8-A4'
      do ii=1,56
         sum(ii)=0.
         count(ii)=0
         sumsq(ii)=0.
      end do

      call ml_wt('info','write AVHRR results to 1d file')

! Writes the AVHRR parameters into the Hirs 1d record
!====================================================
! Statistics are made for lines [6 , targ_nlines-6]
!
       !update the header with AVHRR present
       call gp_sb32(hrs1d_h_instruments,5,1)
       !ecriture header
       irec=1
       CALL IOH1D ("WRITE HEADER", lu1d, irec)
      
       do irec = 1+hrs1d_h_hdrcnt,targ_nlines+hrs1d_h_hdrcnt

          CALL IOH1D ("READ DATA", lu1d, irec)
          tline=hrs1d_scnlin
          do ii=1,56
             do jj=1,targ_mxch
                hrs1d_avhrr(jj,ii)=targ_bts(jj,ii,tline)
             end do
          end do
          CALL IOH1D ("WRITE DATA", lu1d, irec)

          ! Hirs 8 - AVHRR 4 statistics
          do ii=1,56
             if(tline.gt.5 .and. tline.lt.targ_nlines-5) then
                if(hrs1d_btemps(8,ii).ge.18000.and.
     &            (hrs1d_hirsqual(ii).eq.0.or.
     &             hrs1d_hirsqual(ii).eq.1048576) .and.
     &             targ_bts(5,ii,tline).ge.18000) then
                   diff=(hrs1d_btemps(8,ii) -targ_bts(5,ii,tline))/100.
                   idiff=nint(diff)
                   if(tline.gt.4) then
                      if(idiff.ge.-30.and.idiff.le.30)
     &                      hist(idiff)=hist(idiff)+1
                      sum(ii)=sum(ii)+diff
                      sumsq(ii)=sumsq(ii)+diff*diff
                      count(ii)=count(ii)+1  
                      sum_all=sum_all+diff
                      sumsq_all=sumsq_all+diff*diff
                      count_all=count_all+1
                   endif
                endif
             end if
             
          end do
       end do

       do ii=1,56
         if (count(ii) .ne. 0) then
           mean(ii)=sum(ii)/count(ii)
           sd(ii)=sqrt(sumsq(ii)/count(ii)-mean(ii)*mean(ii))
         else
           mean(ii)=miss_data
           sd(ii)=miss_data
           call ml_wti
     &      ('warning','No statistic H8-A4 for HIRS column',ii)
         endif    
       enddo
       
       if (count_all .ne. 0) then
         mean_all=sum_all/count_all
         sd_all=sqrt(sumsq_all/count_all-mean_all*mean_all)
       else
         mean_all=miss_data
         sd_all=miss_data
         call ml_wt('error','No statistic H8-A4 for all HIRS pixels')
         call ml_wt('error',
     &   'Check HIRS 8 , AVHRR 4 and HIRS quality control word')
       endif   
       
       ! writes the statistics to standard output
       print *,'Mean/sd/count of (h8-a4)',mean_all,sd_all,count_all
       do ii=1,56
          print *,'Mean/sd/count of (h8-a4) for pixel ',
     &            ii,mean(ii),sd(ii),count(ii)
       enddo

       
       ! writes the statistics to a formatted historical file
       dq(1) = hrs1d_h_startdatayr
       dq(2) = hrs1d_h_startdatady
       call xdqg(dq,dg) 
       write(lumq,'(i4,2i3.2,i6.5,57f5.2)') 
     &    dg(1),dg(2),dg(3), hrs1d_h_startorbit, 
     &    sd_all, (sd(ii),ii=1,56)


      !write AVHRR quality information on standard output
      write(*,*) 'Calibration quality control for each channel',
     &           ' in number of lines'
      write(*,'(10x,2a6)') '  good','   bad'
      do c=1,6
         write(*,'(a8,a2,2i6)') 'channel ',
     &        chn_name(c),good_lines(c),bad_lines(c)
      end do
      write(*,*) 'missing lines ',mis_lines

      prct = mis_lines*100/avh_h_scnlin
      if(prct .ge. 10) then
         call ml_wti('warning',
     &      'high number of AVHRR missing lines (%)',prct)
      endif

!close l1b l1d files 
!===================
      close (lu1b)
      CALL IOH1D ("CLOSE", lu1d, irec)
     

      call ml_wt('info','normal exit from avh2hirs')
      stop
      end


