C LIBS = atovpp atov auxaapp f7nl1b
!+
!
      PROGRAM CRIS_DEGRADE_FSR
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: cris_degrade_fsr.F 557 2017-10-03 13:12:34Z frna $
!
! Transform CrIS full-spectral-resolution (FSR) data to standard resolution.
! The input 1c data could be obtained from SDR (via cris_sdr) or could be from
! BUFR files (aapp_decodebufr_1c).
!
! Input:  2211 channels: 713 in band 1, 865 in band 2, 633 in band 3
!         Spectral sampling 0.625 cm^-1
!         Hanning apodised or unapodised
! Output: 1305 channels: 713 in band 1, 433 in band 2, 159 in band 3
!         Spectral sampling 0.625, 1.25, 2.5 cm^-1
!         Hanning apodised
!
! Method:
!   1) Transform the Hanning functions to interferogram domain
!   2) Read input file, looping over spectra
!      - Pad to power of 2 and add mirror image to make spectrum symmetric
!      - FFT to interferogram domain
!      - Truncate and modify the apodisation
!      - Inverse FFT to spectral domain
!      - Select the correct spectral range
!   3) Write to output file 
!
! The first and last point in bands 2 and 3 will have degraded accuracy.
!
! In principle this program could be extended to handle unapodised SDR with
! the extra 4 samples per band (total 2223). But this is not currently an
! option in cris_sdr, so is not implemented.

! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2016, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0   25/05/2016   Initial

#include "cris1c.h"
#include "ppparms.h"
#include "ppcrisfdata.h"

! Variables
      INTEGER   C1cunit       !Unit number
      DATA C1cunit/18/
      INTEGER   C1coutunit   !Unit number
      DATA C1coutunit/28/
      INTEGER   Iscan,Ichan,FOV,FofR,i  !Loop variables
      INTEGER   npad, recwords_save
      
      REAL h_in(2048)        !Hanning function in (and also for band 1 output)
      data h_in/0.54,0.23,2045*0.0,0.23/

      REAL h_out2(1024)      !Hanning function out for band 2
      data h_out2/0.54,0.23,1021*0.0,0.23/

      REAL h_out3(512 )      !Hanning function out for band 3
      data h_out3/0.54,0.23,509*0.0,0.23/
      
      REAL h_out2_ratio(1024), h_out3_ratio(512)
      REAL radiance(2048)    !With mirror image and some padding
      REAL r_out2(1024)
      REAL r_out3(512)
      LOGICAL first
      DATA first /.true./
!
!- End of header

      WRITE(*,*)'##############################'
      WRITE(*,*)'# Start cris_degrade_fsr.exe #'
      WRITE(*,*)'##############################'
      
! Compute the conversion arrays

      call SFFTCF(h_in,2048,11)    !Real output in array elements 1 to 1025     
      call SFFTCF(h_out2,1024,10)  !Real output in array elements 1 to 513
      call SFFTCF(h_out3,512,9)    !Real output in array elements 1 to 257

      do i=1,513
        h_out2_ratio(i) = h_out2(i)/h_in(i)
      enddo
      do i=1,257
        h_out3_ratio(i) = h_out3(i)/h_in(i)
      enddo

! Open the 1c files

      cris1c_h_recwords = cris_1crecl/4   !see cris1c.h
      CALL IOC1C("OPEN/READ",C1cunit,0)
      CALL IOC1C("READ HEADER",C1cunit,1)
      recwords_save = cris1c_h_recwords
      IF (cris1c_h_nchanused .ne. 2211) THEN
        write(*,*)'Error: this program expects 2211 channels'
        write(*,*)'   Number of channels input:',cris1c_h_nchanused
        CALL IOC1C("CLOSE",C1cunit,0)
        call aappexit(1)
      ENDIF

! Modify the data records

      DO Iscan=1,cris1c_h_scnlin
        cris1c_h_recwords = recwords_save
        CALL IOC1C("READ DATA",C1cunit,1+Iscan)

        DO FOV=1,cris_1cnfov
          DO FofR=1,cris_1cnfor

!Band 1 - if we need to apodise band 1, do it in spectral domain, no need for any FFTs.
            IF (IAND(cris1c_radianceflgs(FOV,FofR),1024) .NE. 0) THEN
              IF (first) THEN
                write(*,*) 'Unapodised input'
                first = .false.
              ENDIF
              DO Ichan=1,713
                radiance(Ichan+1) = cris1c_radiance(FOV,FofR,Ichan)
              ENDDO  
              radiance(1) = radiance(2)
              radiance(715) = radiance(714)
              DO Ichan=1,713
                cris1c_radiance(FOV,FofR,Ichan) = 0.23*radiance(Ichan) + 
     &             0.54*radiance(Ichan+1) + 0.23*radiance(Ichan+2) + 0.5
              ENDDO
            ENDIF

!Band 2
            npad = 80
            DO Ichan=npad+1,npad+865
              radiance(Ichan) = 
     &           cris1c_radiance(FOV,FofR,Ichan+713-npad)  !Band 2 spectrum
            ENDDO
            DO Ichan=1,npad
              radiance(Ichan) = radiance(npad+1)   !Padding at start
            ENDDO
            DO Ichan=npad+866,1025
              radiance(Ichan) = radiance(npad+865)   !Padding at end
            ENDDO
            DO Ichan=1026,2048
              radiance(Ichan) = radiance(2050-Ichan)   !Mirror image
            ENDDO

            CALL SFFTCF(radiance,2048,11)  !transform to interferogram
            IF (IAND(cris1c_radianceflgs(FOV,FofR),1024) .NE. 0) THEN   !If input is unapodised
              DO Ichan=1,513
                r_out2(Ichan) = radiance(Ichan)*h_out2(Ichan) !real part of interferogram
              ENDDO
            ELSE
              DO Ichan=1,513
                r_out2(Ichan) = radiance(Ichan)*h_out2_ratio(Ichan) !real part of interferogram
              ENDDO
            ENDIF
            DO Ichan=514,1024
              r_out2(Ichan) = 0.0   !imaginary part of interferogram
            ENDDO

            CALL SFFTCB(r_out2,1024,10)  !transform back to spectrum
            DO Ichan=1,433
              cris1c_radiance(FOV,FofR,Ichan+713) = 
     &            r_out2(Ichan+npad/2)/2  !/2 due to fft convention
            ENDDO

!Band 3
            npad = 100
            DO Ichan=npad+1,npad+633
              radiance(Ichan) = 
     &           cris1c_radiance(FOV,FofR,Ichan+713+865-npad)  !Band 3 spectrum
            ENDDO
            DO Ichan=1,npad
              radiance(Ichan) = radiance(npad+1)   !Padding at start
            ENDDO
            DO Ichan=npad+634,1025
              radiance(Ichan) = radiance(npad+633)   !Padding at end
            ENDDO
            DO Ichan=1026,2048
              radiance(Ichan) = radiance(2050-Ichan)   !Mirror image
            ENDDO

            CALL SFFTCF(radiance,2048,11)  !transform to interferogram
            IF (IAND(cris1c_radianceflgs(FOV,FofR),1024) .NE. 0) THEN   !If input is unapodised
              DO Ichan=1,257
                r_out3(Ichan) = radiance(Ichan)*h_out3(Ichan) !real part of interferogram
              ENDDO
              cris1c_radianceflgs(FOV,FofR) = 2048   !Set flag to "apodised"
            ELSE
              DO Ichan=1,257
                r_out3(Ichan) = radiance(Ichan)*h_out3_ratio(Ichan) !real part of interferogram
              ENDDO
            ENDIF
            DO Ichan=258,512
              r_out3(Ichan) = 0.0   !imaginary part of interferogram
            ENDDO

            CALL SFFTCB(r_out3,512,9)  !transform back to spectrum
            DO Ichan=1,159
              cris1c_radiance(FOV,FofR,Ichan+713+433) =
     &            r_out3(Ichan+npad/4)/4  !/4 due to fft convention
            ENDDO

          ENDDO
        ENDDO

        cris1c_h_nchan = 1305
        cris1c_h_nchanused = cris1c_h_nchan
        cris1c_h_recwords = array_size_cris1c - 
     &   (cris_1cnchan - cris1c_h_nchanused)*cris_1cnfov*cris_1cnfor
        IF (Iscan.EQ.1) THEN
          CALL IOC1C("OPEN/WRITE",C1coutunit,0)
          CALL IOC1C("WRITE HEADER",C1coutunit,1)
        ENDIF
        CALL IOC1C("WRITE DATA",C1coutunit,1+Iscan)
      ENDDO

! Close files

      CALL IOC1C("CLOSE",C1cunit,0)
      CALL IOC1C("CLOSE",C1coutunit,0)
      WRITE(*,*)'End cris_degrade_fsr.exe'

      END
