C LIBS = atovpp auxaapp
!+    Main program to transform reference eigenvectors to binary files suitable
!     for atovpp
!
      PROGRAM IASI_EIGENVECTORS
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: iasi_eigenvectors.F 235 2010-10-06 13:05:12Z frna $
!
! Purpose: Create binary data files for atovpp
!
! Method:
! 1) Read in reference eigenvectors, nedr profile (self apodised) and 
!    deapodisation function to convert from gaussian apodised to self apodised.
! 2) Write out the eigenvectors as binary, for encoding the self-apodised
!    noise-normalised radiances
! 3) Multiply the eigenvectors by the noise, apodise and write out as
!    binary, for decoding the PCs directly to gaussian apodised radiances
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      16/08/05  Original code. (N.C.Atkinson)
!  1.1      18/01/08  Extend to accept ECMWF PCA format
!  1.2      15/09/08  Use parameter Imaxnpc in ppparms.h instead of
!                     iasi_1dnpc in iasi1d.h
!  1.3      03/03/09  Add .dat suffix to file names if not already present (e.g.
!                     if input files are hdf5).
!  1.4      01/09/09  Add evfile to header record.
!  1.5      09/09/10  Only store 80 characters of evfile (for consistency with
!                     iasipc.h and ppifdata.h).
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!

!  general parameters for ATOVPP 
#include "ppparms.h"
!  Record length multiplier 
#include "reclopt.h"
!  Fundamental and physical constants 
#include "constants_preproc.h"

      INTEGER npoints_work           ! Number of points per band in the MTF file
      PARAMETER (npoints_work=512)
      INTEGER nbands                 ! Number of bands in the MTF file
      PARAMETER (nbands=45)
      REAL freq_band1                ! First wavenumber in MTF file
      PARAMETER (freq_band1=600.0)   
      REAL bandstep                  ! Band separation in wavenumbers
      PARAMETER (bandstep=50.0)
      INTEGER nnoise                 ! Number of discrete noise values
      PARAMETER (nnoise=44)
      REAL TRef                      ! Reference temperature for NEdT
      PARAMETER (TRef=280.0)
      INTEGER evsize
      PARAMETER (evsize=Inchan*Imaxnpc)
            
      REAL          eigenvectors(Inchan, Imaxnpc)
      DATA          eigenvectors/evsize*0.0/
      REAL          eigenmean(Inchan)
      REAL          nedr_self(Inchan)
      DATA          nedr_self/Inchan*0.0/
      REAL          mtf(nbands,npoints_work) !MTF correction 600-2800 cm^-1 
                                             !step 50 cm^-1, to go from gauss
                                             !apodised to self apodised
      REAL          noise_mult(nbands)
      REAL          spectrum(Inchan)
      REAL          mtfsum
      REAL          freq
      REAL          f
      REAL          bandfreq(nbands)
      REAL          inputnoise(nnoise)
      REAL          wdiff
      REAL          y, dBdT, nedt
      
      INTEGER       wavenum_noise(nnoise)
      INTEGER       eigversnb               ! version number
      INTEGER       ios                     ! I/O status
      INTEGER       ipc, ichan, iband, i    ! loop variables
      INTEGER       I1, I2                  ! array indices
      INTEGER       evnchan
      INTEGER       evmax_eofs
      INTEGER       evchannels(Inchan)
      INTEGER       eigformat
      INTEGER       num_evfiles, ifile, nchan, pos
      INTEGER       pc_to_use(3), pccount
      DATA          pc_to_use/3*0/
      INTEGER       startpc(3), endpc(3), startchan(3), endchan(3)
      DATA          startpc/3*0/,endpc/3*0/,startchan/3*0/,endchan/3*0/
      
      CHARACTER*80  Specfile
      CHARACTER*80  Inputline
      CHARACTER*80  noisefile
      CHARACTER*80  apodfile
      CHARACTER*80  eig_encode_file
      CHARACTER*80  eig_decode_file
      CHARACTER*80  eig_noise_file
      CHARACTER*4   evlevel
      CHARACTER*84  evfile(3)    !Assume we won't have more than 1 per IASI band
      CHARACTER*84  evfiletmp
          
! Function & Subroutine calls:
      External      deapodise_iasi

! Read the specification file

      CALL GETENV ('IASI_SPECFILE',Specfile)
      OPEN (10,file=Specfile)
      Inputline='!'
      DO WHILE (Inputline(1:1) .EQ. '!')
        READ(10,*) Inputline    !skip comments
      ENDDO
      READ(Inputline,*) eigformat
      READ(10,*) evlevel
      READ(10,*) eigversnb
      READ(10,*) noisefile
      READ(10,*) apodfile
      READ(10,*) eig_encode_file
      READ(10,*) eig_decode_file
      READ(10,*) eig_noise_file
      READ(10,*) num_evfiles
      
      IF (eigformat.eq.0.and.num_evfiles.ne.1) THEN
        write(*,*) 'Warning: only one file accepted with format 0)'
        num_evfiles = 1
      ENDIF
      
      DO I=1,num_evfiles
        READ(10,*) evfile(I)
        READ(10,*) pc_to_use(I)
        pos = index(evfile(I),'.gz')
        IF (pos.gt.0) THEN
          evfiletmp = evfile(I)
          evfiletmp(pos:pos+3) = '.dat'   !change .gz to .dat
          evfile(I) = evfiletmp
        ELSE
          pos = index(evfile(I),'.hdf5')
          IF (pos.gt.0) THEN
            evfiletmp = evfile(I)
            evfiletmp(pos:pos+4) = '.dat '   !change .hdf5 to .dat
            evfile(I) = evfiletmp
          ELSE
            pos = index(evfile(I),' ')    !No suffix, just find blank at end
            IF (pos.gt.0) THEN
              evfiletmp = evfile(I)
              evfiletmp(pos:pos+3) = '.dat'
              evfile(I) = evfiletmp
            ENDIF
          ENDIF
        ENDIF
      ENDDO
      CLOSE (10)
      
      IF (pc_to_use(1)+pc_to_use(2)+pc_to_use(3) .GT. Imaxnpc) THEN
        WRITE(*,*)'Error: too many PCs requested. Max is ',Imaxnpc
        call aappexit(1)
      ENDIF

      IF (eigformat .eq. 0) THEN

! Open and read the noise file (format 0 only). Assume this file is in the format used in Tony
! Lee's EUMETSAT contract report EUM/CO/03/1155/PS. Note that this has tab
! separators since it was originally Excel.

        OPEN (10,file=noisefile,status='OLD',iostat=ios)
        IF (ios .NE. 0) THEN
          write(*,*) 'Error opening ' // noisefile
          STOP
        ENDIF

        write(*,*)'Reading ' // noisefile
        READ (10,*) Inputline
        DO WHILE (Inputline(1:6).NE."Nombre")       !Wavenumber
          READ (10,*) Inputline
        ENDDO
        write(*,*)'Reading wavenumbers'
        READ(10,*) wavenum_noise

        DO WHILE (Inputline(1:7).NE."Bruit1A")      !Self apodised noise
          READ (10,*) Inputline(1:5),Inputline(6:7) !Allow for tab separators!
        ENDDO
        write(*,*)'Reading noise'
        READ(10,*) inputnoise
        write(*,*) 'inputnoise:',inputnoise
        CLOSE (10)
      
! Interpolate the noise and convert from NEdT at 280K to NEdR

        I1 = 1
        I2 = 2
        wdiff = wavenum_noise(I2) - wavenum_noise(I1)
        DO ichan=1,Inchan
          freq = (ichan-1)*0.25 + 645.0       !IASI wavenumber
          IF (freq.GT.wavenum_noise(I2).AND.I2.LT.nnoise) THEN
            I1 = I1+1          !Move on to next interval
            I2 = I2+1
            wdiff = wavenum_noise(I2) - wavenum_noise(I1)
          ENDIF
          f = (freq-wavenum_noise(I1))/wdiff
          nedt = f*inputnoise(I2) + (1-f)*inputnoise(I1)
          y = freq*C_2/Tref
          dBdT = C_1*freq*freq*freq/(exp(y)-1)**2 * exp(y) * y/Tref
          nedr_self(ichan) = nedt*dBdT
        ENDDO
      ENDIF   !Format=0
      
! Open and read the MTF file (level 1B eigenvectors only)
 
      IF (evlevel.EQ.'1B') THEN
        OPEN (10,file=apodfile,status='OLD',iostat=ios)
        IF (ios .NE. 0) THEN
          write(*,*) 'Error opening ' // apodfile
          STOP
        ENDIF
        READ (10,*) mtf
        CLOSE (10)
      ENDIF

! Open and read the eigenvectors file

      pccount = 0
      DO ifile=1,num_evfiles
        OPEN (10, file=evfile(ifile),iostat=ios)
        IF (ios .NE. 0) THEN
          write(*,*) 'Error opening ' // evfile(ifile)
          STOP
        ENDIF
        write(*,*)'Reading ' // evfile(ifile)
        write(*,*)'PCs to use: ',pc_to_use(ifile)

        IF (eigformat .eq. 0) THEN      !Format = 0
          nchan = 8461       !contains all channels
          READ (10,*) eigversnb
          DO ipc=1,pc_to_use(ifile)
            pccount = pccount + 1
            READ(10,*) (eigenvectors(I,pccount),I=1,nchan)
          ENDDO
        ELSE                            !Format = 1
          READ (10,*) nchan
          READ (10,*) (evchannels(I),I=1,nchan)
          READ (10,*) (nedr_self(evchannels(I)),I=1,nchan)
          READ (10,*) (eigenmean(evchannels(I)),I=1,nchan)
          READ (10,*) evmax_eofs
          IF (evmax_eofs .LT. pc_to_use(ifile)) THEN
            write(*,*)'Number of EOFs,',evmax_eofs,
     &         ', is too small. Expect at least',pc_to_use(ifile)
            call aappexit(1)
          ENDIF
          startchan(ifile) = evchannels(1)
          endchan(ifile) = evchannels(nchan)
          startpc(ifile) = pccount + 1
          DO ipc=1,pc_to_use(ifile)
            pccount = pccount + 1
            READ(10,*) (eigenvectors(evchannels(I),pccount),I=1,nchan)
          ENDDO
          endpc(ifile) = pccount
        ENDIF
        CLOSE (10)
      ENDDO

! If nedr is in units of W/m^2/cm^-1/str then convert to mW/m^2/cm^-1/str
! because those are the units used in ppispectra.F. Expect noise of around
! 1mW/m^2/cm^-1/str at 645cm^-1

      IF (nedr_self(1).GT.0.AND.nedr_self(1).LT.0.01) THEN
        WRITE(*,*) 'Converting noise units from W to mW'
        DO I=1,Inchan
          nedr_self(i) = nedr_self(i)*1000.0
        ENDDO
      ENDIF
      
! Create encode file

      OPEN (10, file=eig_encode_file,status='NEW',
     &    form='UNFORMATTED', access='DIRECT',
     &    recl=Inchan*4/rldiv, iostat=ios)
     
      IF (ios .NE. 0) THEN             !Try status='OLD'
        OPEN (10, file=eig_encode_file,status='OLD',
     &    form='UNFORMATTED', access='DIRECT',
     &    recl=Inchan*4/rldiv, iostat=ios)
     
        IF (ios .NE. 0) THEN
          write(*,*) 'Error creating ' // eig_encode_file
          STOP
        ENDIF
      ENDIF

      WRITE (10,rec=1) eigversnb,Inchan,pccount,eigformat,evlevel,
     &  startchan,endchan,startpc,endpc,
     &  evfile(1)(1:80),evfile(2)(1:80),evfile(3)(1:80)      !Header

! Write to encoding file. Keep this normalised.
 
      IF (eigformat.EQ.1) WRITE (10,rec=2) eigenmean 
      DO ipc=1,pccount
        DO ichan=1,Inchan
          spectrum(ichan) = eigenvectors(ichan,ipc)
        ENDDO

        WRITE (10,rec=ipc+eigformat+1) spectrum
      ENDDO
 
      CLOSE(10)
      
! Create decode file

      OPEN (10, file=eig_decode_file,status='NEW',
     &    form='UNFORMATTED', access='DIRECT',
     &    recl=Inchan*4/rldiv, iostat=ios)
     
      IF (ios .NE. 0) THEN             !Try status='OLD'
        OPEN (10, file=eig_decode_file,status='OLD',
     &    form='UNFORMATTED', access='DIRECT',
     &    recl=Inchan*4/rldiv, iostat=ios)
     
        IF (ios .NE. 0) THEN
          write(*,*) 'Error creating ' // eig_decode_file
          STOP
        ENDIF
      ENDIF

      WRITE (10,rec=1) eigversnb,Inchan,pccount,eigformat,evlevel,
     &  startchan,endchan,startpc,endpc,
     &  evfile(1)(1:80),evfile(2)(1:80),evfile(3)(1:80)      !Header

! To allow direct reconstruction of gaussian-apodised radiances from scores,
! denormalise and apodise, then write to decoding file. This file is not used in
! AAPP but may be used by user applications.
 
      IF (eigformat.EQ.1) THEN
        DO ichan=1,Inchan
          spectrum(ichan) = eigenmean(ichan)*nedr_self(ichan)
        ENDDO
        IF (evlevel.EQ.'1B') CALL deapodise_iasi(spectrum, mtf, -1)
        WRITE (10,rec=2) spectrum
      ENDIF 

      DO ipc=1,pccount
        DO ichan=1,Inchan
          spectrum(ichan) = eigenvectors(ichan,ipc)*nedr_self(ichan)
        ENDDO

        IF (evlevel.EQ.'1B') CALL deapodise_iasi(spectrum, mtf, -1)
        WRITE (10,rec=ipc+eigformat+1) spectrum

      ENDDO
 
      CLOSE(10)
      
! Create noise file

      OPEN (10, file=eig_noise_file,status='NEW',
     &    form='UNFORMATTED', access='DIRECT',
     &    recl=Inchan*4/rldiv, iostat=ios)
     
      IF (ios .NE. 0) THEN           !Try status='OLD'
        OPEN (10, file=eig_noise_file,status='OLD',
     &    form='UNFORMATTED', access='DIRECT',
     &    recl=Inchan*4/rldiv, iostat=ios)
     
        IF (ios .NE. 0) THEN
          write(*,*) 'Error creating ' // eig_noise_file
          STOP
        ENDIF
      ENDIF
      
      WRITE (10,rec=1) nedr_self
      CLOSE(10)
      write(*,*)'Finished iasi_eigenvectors.exe'

      END
