#!/bin/ksh
#
# Generate new binary eigenvectors files according to the specification given in
# $IASI_SPECFILE (default iasi_eigenvectors_spec.dat) in directory
# $DIR_IASI_PREPROC (default $DIR_PREPROC)
#
# Accepts the following types of eigenvector file:
# (i)  the original AAPP v6.1 file containing a version ID on the first line
#      followed by the eigenvectors. The mean radiance is not subtracted
#      before encoding. The noise profile at discrete wavenumbers is held
#      separately.
# (ii) output files from the ECMWF PCA package, containing:
#        number of channels
#        channel list
#        noise profile (all channels)
#        mean normalised radiance
#        number of eigenvectors available
#        the eigenvectors
#
# 21/01/2007 NCA
# 03/03/2009 Add hdf5 capability
#
################################################################################

. "$AAPP_PREFIX/ATOVS_CONF"

#Optional dedicated directory for IASI data files. Default $DIR_PREPROC

DIR_IASI_PREPROC=${DIR_IASI_PREPROC:-${DIR_PREPROC}}
IASI_SPECFILE=${IASI_SPECFILE:-iasi_eigenvectors_spec.dat}
export IASI_SPECFILE

cd $DIR_IASI_PREPROC

#Check spec file

if [ ! -s $IASI_SPECFILE ]; then
  echo "Error - missing $IASI_SPECFILE in $DIR_IASI_PREPROC"
  exit 1
fi

#Read the spec file, ignoring any comment lines

set $(awk '{print $1}' $IASI_SPECFILE | grep -v "!")
evformat=$1
evlevel=$2
evversion=$3
evnoise=$4
evapod=$5
eig_encode=$6
eig_decode=$7
eig_noise=$8
num_evfiles=$9
shift 9
evfiles=$*

#Check whether the output file already exists and is newer than the spec
#file. If so, we assume everything is up to date.

if [ -f $eig_encode ]; then
  if [ $eig_encode -nt $IASI_SPECFILE ]; then
    exit 0
  fi
fi

#unzip any gzipped eigenvector files. Use a ".dat" suffix.
#decode any hdf5 files. 

count=0
while [ $count -lt $num_evfiles ]; do
  file=$1
  num_evectors=$2
  if [ ! -s $file ]; then
    echo "Error - missing ${file} in $DIR_IASI_PREPROC"
    exit 1
  fi
  if [ ${file%%.gz} != $file ]; then                #gzipped ASCII format
    echo "unzipping ${file}"
    outfile=${file%%.gz}.dat
    gunzip -c ${file} > $outfile
    if [ $? != 0 ]; then
      echo "error unzipping ${file}"
      exit 1
    fi
  elif [[ "$(head -1 $file)" = *HDF* ]]; then       #hdf5 format
    outfile=${file%%.hdf5}.dat
    iasi_ev_readhdf5.exe $file $2 >$outfile
    if [ $? != 0 ]; then
      echo "iasi_ev_readhdf5.exe failed"
      exit 1
    fi
  else
    echo "Error: eigenvector file must be either HDF5 format or .gz suffix"
    exit 1
  fi

  count=$(expr $count + 1)
  shift
  shift     # 2 shifts because next line is number of eigenvectors to use
done

#Check apodisation file

if [ "$evlevel" = "1B" ]; then
  if [ ! -s ${evapod} ]; then
    echo "atovpp error - missing ${evapod} in $DIR_IASI_PREPROC"
    exit 1
  fi
fi

#Check input noise file (if format=0)

if [ "$evformat" = 0 ]; then       #original format used by AAPP v6.1
  if [ ! -s ${evnoise} ]; then
    echo "atovpp error - missing ${evnoise} in $DIR_IASI_PREPROC"
    exit 1
  fi
fi

#Call the Fortran executable

echo "Generating IASI eigenvectors binary files"
iasi_eigenvectors.exe
exitc=$?

#Tidy up

for file in $evfiles; do
  rm -f ${file%%.gz}.dat ${file%%.hdf5}.dat 
done

if [ $exitc != 0 ]; then
  echo "error in iasi_eigenvectors.exe"
  exit $exitc
fi
  
exit
