/* 
! Name: $Id: iasi_ev_readhdf5.c 416 2014-09-25 09:30:11Z frna $
!
! Purpose: C program to convert HDF5 eigenvector files to standard AAPP ASCII
! format (as used in the NWP SAF IASI PCA compression software).
!
! Interfaces with HDF5 Release 1.8, see http://www.hdfgroup.org/HDF5/
!
! Arguments:
!      file name
!      max number of eigenvectors to write (optional, otherwise writes all)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2009, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    03/03/2009  Original code. (N.C.Atkinson)
!  1.1    19/09/2010  Allow for channel numbering from 1 in version 103 onwards
!  1.2    07/08/2014  Old channel numbering applies to Metop-A only.
!
! Code Description: C
!
! C subroutines and functions called:
!     none
!
* LIBS = 
!-------------------------------------------------------------------------------
*/

#include <stdlib.h>
#include <stdio.h>

#ifdef HAS_LIBHDF5
#include "hdf5.h"
#endif

int main (int argc, char **argv){

#ifdef HAS_LIBHDF5

  hid_t   file_id, dataset_id, attr_id;        /* handles */
  herr_t  status;
  
  int     FirstChannel, NbrChannels, NbrEigenvectors;        /* attributes */
  double  *Eigenvalues, *Eigenvectors, *Mean, *Nedr;         /* data */
  
  int     i,j,k;           /* loop variables */
  int     maxEigenvectors;
  int     version;
  int     satnum;
  int     channeloffset;

  if (argc < 2) {
    fprintf(stderr, "usage: iasi_ev_readhdf5 filename [max_eigenvectors]\n");
    exit(1);
  }

/* channels were numbered from 0 in version 102 (for Metop-A), from 1 in later
   versions */
  sscanf(argv[1]+56,"%d",&version);
  sscanf(argv[1]+14,"%d",&satnum);
  
  if ( version <= 102 && satnum == 2) channeloffset=1;
  else channeloffset=0;

/****** Open the file ******/

  file_id = H5Fopen(argv[1], H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0){
    fprintf(stderr, "cannot open %s\n",argv[1]);
    exit(2);
  }

/****** Get the attributes ******/

  attr_id = H5Aopen(file_id, "FirstChannel", H5P_DEFAULT);
  if (attr_id < 0) {
    fprintf(stderr,"Error reading FirstChannel\n");
    exit(3);
  }
  status = H5Aread(attr_id, H5T_NATIVE_INT, &FirstChannel);
  H5Aclose(attr_id);
  
  attr_id = H5Aopen(file_id, "NbrChannels", H5P_DEFAULT);
  if (attr_id < 0) {
    fprintf(stderr,"Error reading NbrChannels\n");
    exit(3);
  }
  status = H5Aread(attr_id, H5T_NATIVE_INT, &NbrChannels);
  H5Aclose(attr_id);
  
  attr_id = H5Aopen(file_id, "NbrEigenvectors", H5P_DEFAULT);
  if (attr_id < 0) {
    fprintf(stderr,"Error reading NbrEigenvectors\n");
    exit(3);
  }
  status = H5Aread(attr_id, H5T_NATIVE_INT, &NbrEigenvectors);
  H5Aclose(attr_id);
  
  if (argc >= 3) sscanf(argv[2],"%d",&maxEigenvectors);
  else maxEigenvectors = NbrEigenvectors;

/****** Allocate memory for data values ******/

  Eigenvalues = (double *)calloc(NbrEigenvectors, sizeof(double));
  Eigenvectors = (double *)calloc(NbrChannels*NbrEigenvectors, sizeof(double));
  Mean = (double *)calloc(NbrChannels, sizeof(double));
  Nedr = (double *)calloc(NbrChannels, sizeof(double));

/****** Read the data ******/

  dataset_id = H5Dopen(file_id, "Eigenvalues",H5P_DEFAULT);
  if (dataset_id < 0) {
    fprintf(stderr,"Error reading Eigenvalues\n");
    exit(4);
  }
  status = H5Dread(dataset_id, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, 
                    Eigenvalues);
  H5Dclose(dataset_id);

  dataset_id = H5Dopen(file_id, "Eigenvectors",H5P_DEFAULT);
  if (dataset_id < 0) {
    fprintf(stderr,"Error reading Eigenvectors\n");
    exit(4);
  }
  status = H5Dread(dataset_id, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, 
                    Eigenvectors);
  H5Dclose(dataset_id);

  dataset_id = H5Dopen(file_id, "Mean",H5P_DEFAULT);
  if (dataset_id < 0) {
    fprintf(stderr,"Error reading Mean\n");
    exit(4);
  }
  status = H5Dread(dataset_id, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, 
                    Mean);
  H5Dclose(dataset_id);

  dataset_id = H5Dopen(file_id, "Nedr",H5P_DEFAULT);
  if (dataset_id < 0) {
    fprintf(stderr,"Error reading Nedr\n");
    exit(4);
  }
  status = H5Dread(dataset_id, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, 
                    Nedr);
  H5Dclose(dataset_id);

/****** Display ******/

  printf("%d %s\n",NbrChannels,argv[1]);/* Number of channels and file name */
  for (i=0; i<NbrChannels; i++){        /* Channels (numbered from 1 in output) */
    printf("%4d ",i+FirstChannel+channeloffset);
    if (i%10 == 9) printf("\n");
  }
  if (i%10 != 0) printf("\n");

  for (i=0; i<NbrChannels; i++){        /* Nedr (in mW/m^2/str/cm^-1) */
    printf("%9.7f ",Nedr[i]*1E5);
    if (i%10 == 9) printf("\n");
  }
  if (i%10 != 0) printf("\n");

  for (i=0; i<NbrChannels; i++){        /* Mean */
    printf("%9.4f ",Mean[i]);
    if (i%10 == 9) printf("\n");
  }
  if (i%10 != 0) printf("\n");

  printf("%d\n",maxEigenvectors);       /* Number of eigenvectors */
  for (j=0; j<maxEigenvectors; j++){    /* Eigenvectors */
    k = j*NbrChannels;
    for (i=0; i<NbrChannels; i++){
      printf("%11.8f ",Eigenvectors[k+i]);
      if (i%10 == 9) printf("\n");
    }
    if (i%10 != 0) printf("\n");
  }

/****** Clean up ******/

  free(Nedr);
  free(Mean);
  free(Eigenvectors);
  free(Eigenvalues);
  H5Fclose(file_id);
  return 0;

#else

  printf("iasi_ev_readhdf5.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  
#endif

}
