!+ I/O routine for level 1C AMSU-B data
!
      SUBROUTINE IOB1C ( COMMAND, B1Cunit, B1Crecord )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: iob1c.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   interface to files containing level 1C AMSU-B data
!
! Method:
!   Performs one of several functions, according to the value of COMMAND :
!
!   "OPEN/READ"          : open file for readonly
!   "OPEN/WRITE"         : open file for write
!   "CLOSE"              : close file
!   "READ HEADER"        : read header into common
!   "READ DATA"          : read data record into common
!   "WRITE HEADER"       : write header from common
!   "WRITE DATA"         : write data from common
!
!   Note: COMMAND must be typed exactly, and in upper case.
!   Routine ends by calling "ErrorReport" to check on error status.
!   If "COMMAND" is not recognised, "ErrorReport"
!   will be called with fatal error status.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      02/02/96  R.J.Renshaw
!  1.1      01/07/98  READONLY deleted in OPEN statement (non-standard)
!  1.2      09/03/04  Modify for Linux (N Atkinson / A Korpela)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVPP Processing 
#include "ppparms.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!  Record length dividor 
#include "reclopt.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*(*) COMMAND       ! instruction on what function to perform
      INTEGER       B1Cunit       ! unit number
      INTEGER       B1Crecord     ! record number
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      LOGICAL       wordswap_b    ! true if word swap needed for input file
      DATA wordswap_b /.FALSE./
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
!- End of header


! Act on value of COMMAND
! -----------------------

! 1.  OPEN/READ
!     ---------
      IF ( COMMAND .EQ. "OPEN/READ" ) THEN

         OPEN ( UNIT=B1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=amb_1crecl/rldiv, IOSTAT=errStatus )

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=B1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=amb_1crecl/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/READ 1C AMSU-B file: UNIT="",I3,"" RECL="",I6)")
     &                                B1Cunit, amb_1crecl/rldiv


! 2.  OPEN/WRITE
!     ----------
      ELSEIF ( COMMAND .EQ. "OPEN/WRITE" ) THEN

!        First try with STATUS="NEW", i.e. file doesn't yet exist
         OPEN ( UNIT=B1Cunit, STATUS="NEW", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=amb_1crecl/rldiv, IOSTAT=errStatus )

!        If that didn"t work, use STATUS="OLD"
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=B1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=amb_1crecl/rldiv, IOSTAT=errStatus )
! N.B. This seems to be needed on HPs, where "unknown" is treated as "scratch"

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=B1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=amb_1crecl/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/WRITE 1C AMSU-B file: UNIT="",I3,"" RECL="",I6)")
     &                                 B1Cunit, amb_1crecl/rldiv


! 3.  READ HEADER
!     ---- ------
      ELSEIF ( COMMAND .EQ. "READ HEADER" ) THEN

!        read first header record
         READ ( UNIT=B1Cunit,REC=B1Crecord,IOSTAT=errStatus ) amb1c_hd

         IF (amb1c_h_instrument .EQ. 184549376 .OR.     !Hex 0B000000 AMSU-B
     &       amb1c_h_instrument .EQ. 201326592) THEN    !Hex 0C000000 MHS
           wordswap_b = .TRUE.
           CALL wordswap(amb1c_hd(3), array_size_ab1c-2)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" READing 1C AMSU-B header: UNIT="",I3,"" REC ="",I6)")
     &                             B1Cunit, B1Crecord


! 4.  WRITE HEADER
!     ----- ------
      ELSEIF ( COMMAND .EQ. "WRITE HEADER" ) THEN

         WRITE ( UNIT=B1Cunit,REC=B1Crecord,IOSTAT=errStatus ) amb1c_hd

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" WRITEing 1C AMSU-B header: UNIT="",I3,"" REC="",I6)")
     &                              B1Cunit, B1Crecord


! 5.  READ DATA
!     ---- ----
      ELSEIF ( COMMAND .EQ. "READ DATA" ) THEN

         READ ( UNIT=B1Cunit,REC=B1Crecord,IOSTAT=errStatus ) amb1c_dt

         IF (wordswap_b) THEN
           CALL wordswap(amb1c_dt, array_size_ab1c)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" READing AMSU-B 1C data, UNIT="",I3,"" REC="",I6)")
     &                             B1Cunit, B1Crecord


! 6.  WRITE DATA
!     ----- ----
      ELSEIF ( COMMAND .EQ. "WRITE DATA" ) THEN

         WRITE ( UNIT=B1Cunit,REC=B1Crecord,IOSTAT=errStatus ) amb1c_dt

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" WRITEing AMSU-B 1C data, UNIT="",I3,"" REC="",I6)")
     &                              B1Cunit, B1Crecord


! 7.  CLOSE
!     -----
      ELSEIF ( COMMAND .EQ. "CLOSE" ) THEN

         CLOSE ( UNIT=B1Cunit, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" CLOSEing AMSU-B 1C file, UNIT="",I3)") B1Cunit


! 8.  COMMAND not recognized
!     ------- --- ----------
      ELSE

         errStatus = 999
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Command not recognised: "",A50)") COMMAND


      ENDIF   ! (COMMAND)


! 8.  Check error status
!     ----- ----- ------

      CALL ErrorReport ( errStatus, errMessage, "IOB1C")


      RETURN
      END
