!+ I/O routine for level 1C HIRAS data
!
      SUBROUTINE IOHIRAS1C ( COMMAND, H1Cunit, H1Crecord )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: iohiras1c.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   interface to files containing level 1C HIRAS data
!
! Method:
!   Performs one of several functions, according to the value of COMMAND :
!
!   "OPEN/READ"          : open file for readonly
!   "OPEN/WRITE"         : open file for write
!   "CLOSE"              : close file
!   "READ HEADER"        : read header into common
!   "READ DATA"          : read data record into common
!   "WRITE HEADER"       : write header from common
!   "WRITE DATA"         : write data from common
!
!   Note: COMMAND must be typed exactly, and in upper case.
!   Routine ends by calling "ErrorReport" to check on error status.
!   If "COMMAND" is not recognised, "ErrorReport"
!   will be called with fatal error status.
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    05/02/2019 Initial
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVPP Processing 
#include "ppparms.h"
!  HIRAS level 1C header/data common blocks 
#include "hiras1c.h"
!  Record length dividor 
#include "reclopt.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*(*) COMMAND       ! instruction on what function to perform
      INTEGER       H1Cunit       ! unit number
      INTEGER       H1Crecord     ! record number
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      LOGICAL       wordswap_c    ! true if word swap needed for input file
      DATA wordswap_c /.FALSE./
      INTEGER       I             ! Loop index
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
!- End of header


!Allow user to control the record size via hiras1c_h_recwords (in words)
      IF (hiras1c_h_recwords .LE. 0) 
     &  hiras1c_h_recwords = hiras_1crecl/4

! Act on value of COMMAND
! -----------------------

! 1.  OPEN/READ
!     ---------
!Note: hiras1c_h_recwords must be valid on input

      IF ( COMMAND .EQ. "OPEN/READ" ) THEN

         OPEN ( UNIT=H1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=hiras1c_h_recwords*4/rldiv,
     &   IOSTAT=errStatus )

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=H1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=hiras1c_h_recwords*4/rldiv,
     &   IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/READ 1C HIRAS file: UNIT="",I3,"" RECL="",I6)")
     &                             H1Cunit, hiras1c_h_recwords*4/rldiv


! 2.  OPEN/WRITE
!     ----------
      ELSEIF ( COMMAND .EQ. "OPEN/WRITE" ) THEN

!        First try with STATUS="NEW", i.e. file doesn't yet exist
         OPEN ( UNIT=H1Cunit, STATUS="NEW", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=hiras1c_h_recwords*4/rldiv,
     &   IOSTAT=errStatus )

!        If that didn"t work, use STATUS="OLD"
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=H1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=hiras1c_h_recwords*4/rldiv,
     &   IOSTAT=errStatus )
! N.B. This seems to be needed on HPs, where "unknown" is treated as "scratch"

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=H1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=hiras1c_h_recwords*4/rldiv, 
     &   IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/WRITE 1C HIRAS file: UNIT="",I3,"" RECL="",I6)")
     &                            H1Cunit, hiras1c_h_recwords*4/rldiv


! 3.  READ HEADER
!     ---- ------
      ELSEIF ( COMMAND .EQ. "READ HEADER" ) THEN

!        read first header record
         READ ( UNIT=H1Cunit,REC=H1Crecord,IOSTAT=errStatus ) 
     &        (hiras1c_hd(I),I=1,hiras1c_h_recwords)

         IF (hiras1c_h_instrument .EQ. 1812070400) THEN    !Hex 6C020000
           wordswap_c = .TRUE.
           CALL wordswap(hiras1c_hd(3), hiras1c_h_recwords-2)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" READing 1C HIRAS header: UNIT="",I3,"" REC ="",I6)")
     &                             H1Cunit, H1Crecord


! 4.  WRITE HEADER
!     ----- ------
      ELSEIF ( COMMAND .EQ. "WRITE HEADER" ) THEN

         WRITE ( UNIT=H1Cunit,REC=H1Crecord,IOSTAT=errStatus )
     &      (hiras1c_hd(I),I=1,hiras1c_h_recwords)

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" WRITEing 1C HIRAS header: UNIT="",I3,"" REC="",I6)")
     &                              H1Cunit, H1Crecord


! 5.  READ DATA
!     ---- ----
      ELSEIF ( COMMAND .EQ. "READ DATA" ) THEN

         READ ( UNIT=H1Cunit,REC=H1Crecord,IOSTAT=errStatus )
     &      (hiras1c_dt(I),I=1,hiras1c_h_recwords)

         IF (wordswap_c) THEN
           CALL wordswap(hiras1c_dt, hiras1c_h_recwords)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" READing HIRAS 1C data, UNIT="",I3,"" REC="",I6)")
     &                             H1Cunit, H1Crecord


! 6.  WRITE DATA
!     ----- ----
      ELSEIF ( COMMAND .EQ. "WRITE DATA" ) THEN

         WRITE ( UNIT=H1Cunit,REC=H1Crecord,IOSTAT=errStatus )
     &      (hiras1c_dt(I),I=1,hiras1c_h_recwords)

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" WRITEing HIRAS 1C data, UNIT="",I3,"" REC="",I6)")
     &                              H1Cunit, H1Crecord


! 7.  CLOSE
!     -----
      ELSEIF ( COMMAND .EQ. "CLOSE" ) THEN

         CLOSE ( UNIT=H1Cunit, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" CLOSEing HIRAS 1C file, UNIT="",I3)") H1Cunit


! 8.  COMMAND not recognized
!     ------- --- ----------
      ELSE

         errStatus = 999
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Command not recognised: "",A50)") COMMAND


      ENDIF   ! (COMMAND)


! 8.  Check error status
!     ----- ----- ------

      CALL ErrorReport ( errStatus, errMessage, "IOHIRAS1C")


      RETURN
      END
