!+ I/O routine for level 1C IASI data
!
      SUBROUTINE IOI1C ( COMMAND, I1Cunit, I1Crecord )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ioi1c.F 203 2010-02-19 11:07:05Z frna $
!
! Purpose:
!   interface to files containing level 1C IASI data
!
! Method:
!   Performs one of several functions, according to the value of COMMAND :
!
!   "OPEN/READ"          : open file for readonly
!   "OPEN/WRITE"         : open file for write
!   "CLOSE"              : close file
!   "READ HEADER"        : read header into common
!   "READ DATA"          : read data record into common
!   "WRITE HEADER"       : write header from common
!   "WRITE DATA"         : write data from common
!
!   Note: COMMAND must be typed exactly, and in upper case.
!   Routine ends by calling "ErrorReport" to check on error status.
!   If "COMMAND" is not recognised, "ErrorReport"
!   will be called with fatal error status.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      02/11/05  Created from ioh1c.F
!  1.1      19/02/10  Allow for v2 format (with IIS and cloud variables)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVPP Processing 
#include "ppparms.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!  Record length dividor 
#include "reclopt.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*(*) COMMAND       ! instruction on what function to perform
      INTEGER       I1Cunit       ! unit number
      INTEGER       I1Crecord     ! record number
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      LOGICAL       wordswap_i    ! true if word swap needed for input file
      DATA wordswap_i /.FALSE./
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
!- End of header


! Act on value of COMMAND
! -----------------------

! 1.  OPEN/READ
!     ---------
      IF ( COMMAND .EQ. "OPEN/READ" ) THEN

         OPEN ( UNIT=I1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_1crecl/rldiv, IOSTAT=errStatus )

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=I1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_1crecl/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/READ 1C IASI file: UNIT="",I3,"" RECL="",I6)")
     &                                I1Cunit, iasi_1crecl/rldiv


! 2.  OPEN/WRITE
!     ----------
      ELSEIF ( COMMAND .EQ. "OPEN/WRITE" ) THEN

!        First try with STATUS="NEW", i.e. file doesn't yet exist
         OPEN ( UNIT=I1Cunit, STATUS="NEW", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_1crecl/rldiv, IOSTAT=errStatus )

!        If that didn't work, use STATUS="OLD"
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=I1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_1crecl/rldiv, IOSTAT=errStatus )
! N.B. This seems to be needed on HPs, where "unknown" is treated as "scratch"

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=I1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_1crecl/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/WRITE 1C IASI file: UNIT="",I3,"" RECL="",I6)")
     &                                 I1Cunit, iasi_1crecl/rldiv


! 3.  READ HEADER
!     ---- ------
      ELSEIF ( COMMAND .EQ. "READ HEADER" ) THEN

!        read first header record
         READ ( UNIT=I1Cunit,REC=I1Crecord,IOSTAT=errStatus ) iasi1c_hd

         IF (iasi1c_h_instrument .EQ. 218103808) THEN    !Hex 0D000000
           wordswap_i = .TRUE.
           CALL wordswap(iasi1c_hd(3), array_size_iasi1c-2)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" READing 1C IASI header: UNIT="",I3,"" REC ="",I6)")
     &                             I1Cunit, I1Crecord


! 4.  WRITE HEADER
!     ----- ------
      ELSEIF ( COMMAND .EQ. "WRITE HEADER" ) THEN

         WRITE ( UNIT=I1Cunit,REC=I1Crecord,IOSTAT=errStatus ) iasi1c_hd

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" WRITEing 1C IASI header: UNIT="",I3,"" REC="",I6)")
     &                              I1Cunit, I1Crecord


! 5.  READ DATA
!     ---- ----
      ELSEIF ( COMMAND .EQ. "READ DATA" ) THEN

         READ ( UNIT=I1Cunit,REC=I1Crecord,IOSTAT=errStatus ) iasi1c_dt

         IF (wordswap_i) THEN
           CALL wordswap (iasi1c_dt(1), 880)
           CALL wordswap2(iasi1c_dt(881), 1044000)  !integer*2 iasi1c_scalrad
           CALL wordswap (iasi1c_dt(522881),array_size_iasi1c-522880)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" READing IASI 1C data, UNIT="",I3,"" REC="",I6)")
     &                             I1Cunit, I1Crecord


! 6.  WRITE DATA
!     ----- ----
      ELSEIF ( COMMAND .EQ. "WRITE DATA" ) THEN

         WRITE ( UNIT=I1Cunit,REC=I1Crecord,IOSTAT=errStatus ) iasi1c_dt

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" WRITEing IASI 1C data, UNIT="",I3,"" REC="",I6)")
     &                              I1Cunit, I1Crecord


! 7.  CLOSE
!     -----
      ELSEIF ( COMMAND .EQ. "CLOSE" ) THEN

         CLOSE ( UNIT=I1Cunit, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" CLOSEing IASI 1C file, UNIT="",I3)") I1Cunit


! 8.  COMMAND not recognized
!     ------- --- ----------
      ELSE

         errStatus = 999
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Command not recognised: "",A50)") COMMAND


      ENDIF   ! (COMMAND)


! 8.  Check error status
!     ----- ----- ------

      CALL ErrorReport ( errStatus, errMessage, "IOI1C")


      RETURN
      END
