!+ I/O routine for level 1C IASI data with Principal Components
!
      SUBROUTINE IOIPC ( COMMAND, IPCunit, IPCrecord )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id$
!
! Purpose:
!   interface to files containing level 1C IASI data with Principal Components
!
!
! Method:
!   Performs one of several functions, according to the value of COMMAND :
!
!   "OPEN/READ"          : open file for readonly
!   "OPEN/WRITE"         : open file for write
!   "CLOSE"              : close file
!   "READ HEADER"        : read header into common
!   "READ DATA"          : read data record into common
!   "WRITE HEADER"       : write header from common
!   "WRITE DATA"         : write data from common
!
!   Note: COMMAND must be typed exactly, and in upper case.
!   Routine ends by calling "ErrorReport" to check on error status.
!   If "COMMAND" is not recognised, "ErrorReport"
!   will be called with fatal error status.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date        Comment
!
!  1.0     12/09/2008  Created from ioi1c.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVPP Processing 
#include "ppparms.h"
!  IASI level PC header/data common blocks 
#include "iasipc.h"
!  Record length dividor 
#include "reclopt.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*(*) COMMAND       ! instruction on what function to perform
      INTEGER       IPCunit       ! unit number
      INTEGER       IPCrecord     ! record number
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      LOGICAL       wordswap_i    ! true if word swap needed for input file
      DATA wordswap_i /.FALSE./
      INTEGER       i,n           ! array index / number of elements
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
!- End of header


! Act on value of COMMAND
! -----------------------

! 1.  OPEN/READ
!     ---------
      IF ( COMMAND .EQ. "OPEN/READ" ) THEN

         OPEN ( UNIT=IPCunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_pcrecl/rldiv, IOSTAT=errStatus )

!        If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=IPCunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_pcrecl/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/READ PC IASI file: UNIT="",I3,"" RECL="",I6)")
     &                                IPCunit, iasi_pcrecl/rldiv


! 2.  OPEN/WRITE
!     ----------
      ELSEIF ( COMMAND .EQ. "OPEN/WRITE" ) THEN

!        First try with STATUS="NEW", i.e. file doesn't yet exist
         OPEN ( UNIT=IPCunit, STATUS="NEW", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_pcrecl/rldiv, IOSTAT=errStatus )

!        If that didn't work, use STATUS="OLD"
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=IPCunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_pcrecl/rldiv, IOSTAT=errStatus )
! N.B. This seems to be needed on HPs, where "unknown" is treated as "scratch"

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=IPCunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=iasi_pcrecl/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/WRITE PC IASI file: UNIT="",I3,"" RECL="",I6)")
     &                                 IPCunit, iasi_pcrecl/rldiv


! 3.  READ HEADER
!     ---- ------
      ELSEIF ( COMMAND .EQ. "READ HEADER" ) THEN

!        read first header record
         READ ( UNIT=IPCunit,REC=IPCrecord,IOSTAT=errStatus ) iasipc_hd

         IF (iasipc_h_instrument .EQ. 234881024) THEN    !Hex 0E000000
           wordswap_i = .TRUE.
           CALL wordswap(iasipc_hd(3), array_size_iasipc-2)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" READing PC IASI header: UNIT="",I3,"" REC ="",I6)")
     &                             IPCunit, IPCrecord


! 4.  WRITE HEADER
!     ----- ------
      ELSEIF ( COMMAND .EQ. "WRITE HEADER" ) THEN

         WRITE ( UNIT=IPCunit,REC=IPCrecord,IOSTAT=errStatus ) iasipc_hd

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" WRITEing PC IASI header: UNIT="",I3,"" REC="",I6)")
     &                              IPCunit, IPCrecord


! 5.  READ DATA
!     ---- ----
      ELSEIF ( COMMAND .EQ. "READ DATA" ) THEN

         READ ( UNIT=IPCunit,REC=IPCrecord,IOSTAT=errStatus ) iasipc_dt

         IF (wordswap_i) THEN
           CALL wordswap (iasipc_dt(1), 880)
           i = 881
           n = iasi_pcnchan*120 + iasi_pcnrecon*120 + iasi_pcnpc*120
           CALL wordswap2(iasipc_dt(i), n)        !Swap the 2-byte integers
           i = i + n
           n = array_size_iasipc - i
           CALL wordswap(iasipc_dt(i), n)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" READing IASI PC data, UNIT="",I3,"" REC="",I6)")
     &                             IPCunit, IPCrecord


! 6.  WRITE DATA
!     ----- ----
      ELSEIF ( COMMAND .EQ. "WRITE DATA" ) THEN

         WRITE ( UNIT=IPCunit,REC=IPCrecord,IOSTAT=errStatus ) iasipc_dt

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" WRITEing IASI PC data, UNIT="",I3,"" REC="",I6)")
     &                              IPCunit, IPCrecord


! 7.  CLOSE
!     -----
      ELSEIF ( COMMAND .EQ. "CLOSE" ) THEN

         CLOSE ( UNIT=IPCunit, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" CLOSEing IASI PC file, UNIT="",I3)") IPCunit


! 8.  COMMAND not recognized
!     ------- --- ----------
      ELSE

         errStatus = 999
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Command not recognised: "",A50)") COMMAND


      ENDIF   ! (COMMAND)


! 8.  Check error status
!     ----- ----- ------

      CALL ErrorReport ( errStatus, errMessage, "IOIPC")


      RETURN
      END
