!+ I/O routine for level 1C MWHS2 data
!
      SUBROUTINE IOMWHS21C ( COMMAND, MWHS21Cunit, MWHS21Crecord )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id$
!
! Purpose:
!   interface to files containing level 1C MWHS2 data
!
! Method:
!   Performs one of several functions, according to the value of COMMAND :
!
!   "OPEN/READ"          : open file for readonly
!   "OPEN/WRITE"         : open file for write
!   "CLOSE"              : close file
!   "READ HEADER"        : read header into common
!   "READ DATA"          : read data record into common
!   "WRITE HEADER"       : write header from common
!   "WRITE DATA"         : write data from common
!
!   Note: COMMAND must be typed exactly, and in upper case.
!   Routine ends by calling "ErrorReport" to check on error status.
!   If "COMMAND" is not recognised, "ErrorReport"
!   will be called with fatal error status.
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    02/12/2010 From iomwhs1c.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVPP Processing 
#include "ppparms.h"
!  MWHS2 level 1C header/data common blocks 
#include "mwhs21c.h"
!  Record length dividor 
#include "reclopt.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*(*) COMMAND       ! instruction on what function to perform
      INTEGER       MWHS21Cunit       ! unit number
      INTEGER       MWHS21Crecord     ! record number
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      LOGICAL       wordswap_mwhs    ! true if word swap needed for input file
      DATA wordswap_mwhs /.FALSE./
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
!- End of header


! Act on value of COMMAND
! -----------------------

! 1.  OPEN/READ
!     ---------
      IF ( COMMAND .EQ. "OPEN/READ" ) THEN

         OPEN ( UNIT=MWHS21Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=mwhs2_1crecl/rldiv,
     &   IOSTAT=errStatus )

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=MWHS21Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=mwhs2_1crecl/rldiv,
     &   IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/READ 1C MWHS2 file: UNIT="",I3,"" RECL="",I6)")
     &                                MWHS21Cunit, mwhs2_1crecl/rldiv


! 2.  OPEN/WRITE
!     ----------
      ELSEIF ( COMMAND .EQ. "OPEN/WRITE" ) THEN

!        First try with STATUS="NEW", i.e. file doesn't yet exist
         OPEN ( UNIT=MWHS21Cunit, STATUS="NEW", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=mwhs2_1crecl/rldiv,
     &   IOSTAT=errStatus )

!        If that didn"t work, use STATUS="OLD"
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=MWHS21Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=mwhs2_1crecl/rldiv,
     &   IOSTAT=errStatus )
! N.B. This seems to be needed on HPs, where "unknown" is treated as "scratch"

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=MWHS21Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=mwhs2_1crecl/rldiv, 
     &   IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" OPEN/WRITE 1C MWHS2 file: UNIT="",I3,"" RECL="",I6)")
     &                            MWHS21Cunit, mwhs2_1crecl/rldiv


! 3.  READ HEADER
!     ---- ------
      ELSEIF ( COMMAND .EQ. "READ HEADER" ) THEN

!        read first header record
         READ ( UNIT=MWHS21Cunit,REC=MWHS21Crecord,IOSTAT=errStatus ) 
     &        mwhs21c_hd

         IF (mwhs21c_h_instrument .EQ. 301989888) THEN    !Hex 12000000 (18)
           wordswap_mwhs = .TRUE.
           CALL wordswap(mwhs21c_hd(3), array_size_mwhs21c-2)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" READing 1C MWHS2 header: UNIT="",I3,"" REC ="",I6)")
     &                             MWHS21Cunit, MWHS21Crecord


! 4.  WRITE HEADER
!     ----- ------
      ELSEIF ( COMMAND .EQ. "WRITE HEADER" ) THEN

         WRITE ( UNIT=MWHS21Cunit,REC=MWHS21Crecord,IOSTAT=errStatus )
     &      mwhs21c_hd

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &"("" WRITEing 1C MWHS2 header: UNIT="",I3,"" REC="",I6)")
     &                              MWHS21Cunit, MWHS21Crecord


! 5.  READ DATA
!     ---- ----
      ELSEIF ( COMMAND .EQ. "READ DATA" ) THEN

         READ ( UNIT=MWHS21Cunit,REC=MWHS21Crecord,IOSTAT=errStatus )
     &      mwhs21c_dt

         IF (wordswap_mwhs) THEN
           CALL wordswap(mwhs21c_dt, array_size_mwhs21c)
         ENDIF

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" READing MWHS2 1C data, UNIT="",I3,"" REC="",I6)")
     &                             MWHS21Cunit, MWHS21Crecord


! 6.  WRITE DATA
!     ----- ----
      ELSEIF ( COMMAND .EQ. "WRITE DATA" ) THEN

         WRITE ( UNIT=MWHS21Cunit,REC=MWHS21Crecord,IOSTAT=errStatus )
     &      mwhs21c_dt

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" WRITEing MWHS2 1C data, UNIT="",I3,"" REC="",I6)")
     &                              MWHS21Cunit, MWHS21Crecord


! 7.  CLOSE
!     -----
      ELSEIF ( COMMAND .EQ. "CLOSE" ) THEN

         CLOSE ( UNIT=MWHS21Cunit, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" CLOSEing MWHS2 1C file, UNIT="",I3)") MWHS21Cunit


! 8.  COMMAND not recognized
!     ------- --- ----------
      ELSE

         errStatus = 999
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Command not recognised: "",A50)") COMMAND


      ENDIF   ! (COMMAND)


! 8.  Check error status
!     ----- ----- ------

      CALL ErrorReport ( errStatus, errMessage, "IOMWHS21C")


      RETURN
      END
