!+ Set up 1C AMSU-A scanline from 1B scanline within ATOVIN
!
      SUBROUTINE INAGET ( scan )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inaget.F 463 2015-09-28 15:29:53Z frna $
!
! Purpose:
!   Place information from common blocks /ama1bdt/ & /inastore/
!   into common block /ama1cdt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1B & 1C include files (ama1b.h & ama1c.h) are
!     compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Extract required information from 1B into 1C
!
! Current Code Owner: N.C.Atkinson
!
! History:
! Version  Date      Comment
!
!  1.0    12/12/96  Original code. (R.J.Renshaw)
!  1.1    17/01/97  lat/lon now 10^4
!  1.2    10/03/98  updated for 1c vn1.9 (instrument temps and solar angles)
!  1.3    20/05/98  updated for latest 1b include files
!                   (lat/lon * 10^4 and no version numbers)
!  1.4    08/06/98  bug fixed  - was picking up wrong counts
!  1.5    18/06/98  Bug fixed in converting azimuth angle to [0,360]
!  1.6    23/06/98  Add clock drift if not already done.
!  1.7    23/02/05  Convert counts to K*100 for ama1c_instrtemp1&2. N.C.Atkinson
!  1.8    11/09/15  Add ama1c_coldnedt, ama1c_warmnedt. Scan line as argument.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-A level 1B header/data common blocks 
#include "ama1b.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  Earth model, used to identify non-AAPP geolocation
#include "p_earth.h"
!
! Arguments:
      INTEGER       scan
!
! Local parameters:
      INTEGER       A1Bvnnb       ! version number of 1B format compatible
         PARAMETER (A1Bvnnb=1)    ! with the code in this subroutine
      INTEGER       A1Cvnnb       ! version number of 1C format compatible
         PARAMETER (A1Cvnnb=190)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chn3          ! = chn*3
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       i             ! general purpose loop variable
      INTEGER       angle         ! satellite azimuth angle (deg*100)
      INTEGER       year          ! scan line year
      INTEGER       day           ! scan line day of year
      INTEGER       time          ! scan line UTC time of day in milliseconds
      INTEGER       clockdrift    ! satellite clock drift delta in milliseconds
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
      REAL          rcount        ! raw count value as real number
!
! Local arrays:
      REAL          latitudes(Anfovs)   ! latitudes for one scanline
      REAL          longitudes(Anfovs)  ! longitudes for one scanline
      REAL          azimuths(Anfovs)    ! solar azimuth angles for one scanline
      REAL          zeniths(Anfovs)     ! solar zenith angles for one scanline
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeAdd              ! add an increment to a date/time
      External INSUNA               ! calculate solar zenith & azimuth angles
!
!- End of header

      SAVE first                    ! .TRUE. on first call

! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!!        Is the 1B include file compatible with this source code ?
!         IF ( A1Bvnnb .NE. ama_1bvnnb ) THEN
!            errStatus = -11  ! warning
!            errMessage = " "
!            WRITE ( errMessage,
!     &      "("" Code expects 1B format"",I6,"" but ama1b.h is "",I6)")
!     &           A1Bvnnb, ama_1bvnnb
!            CALL ErrorReport ( errStatus, errMessage, "INAGET")
!         ENDIF

!        Is the 1C include file compatible with this source code ?
         IF ( A1Cvnnb .NE. ama_1cvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but ama1c.h is "",I6)")
     &           A1Cvnnb, ama_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "INAGET")
         ENDIF

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Extract required information from 1B into 1C
!     ------- -------- ----------- ---- -- ---- --

! 1.1 scanline number
      ama1c_scnlin = ama_scnlin

! 1.2 scanline date/time    ( - assume already corrected for clock drift)
      ama1c_scnlinyr = ama_scnlinyr        ! scan line year
      ama1c_scnlindy = ama_scnlindy        ! scan line day of year
      ama1c_scnlintime = ama_scnlintime    ! UTC time of day in milliseconds

      year = ama1c_scnlinyr                ! scan line year
      day  = ama1c_scnlindy                ! scan line day of year
      time = ama1c_scnlintime              ! UTC time of day in milliseconds

!      - adding clock drift delta if not already done so
      IF ( .NOT.BTEST(INT(ama_scnlinbit),bits(14)) ) THEN  ! add clock drift delta
         clockdrift = ama_clockdrift       ! (msecs)    (ama_... is I2)
         IF ( clockdrift .LT. 0 ) clockdrift = clockdrift + 65536
         CALL TimeAdd ( year, day, time, clockdrift,
     &        ama1c_scnlinyr, ama1c_scnlindy, ama1c_scnlintime)
      ENDIF

! 1.3 other scanline information
      ama1c_qualind = ama_qualind          !quality indicator bit field
      ama1c_scanqual = ama_linqualflgs     !scan line quality flags

!     Quality control word for each channel
      DO chn = 1, Anchan
         ama1c_chanqual(chn) = ama_chqualflg(chn)
      ENDDO  ! chn

!     Instrument temperatures in K*100
      rcount = ama1_temps(33)              !AMSU-A1-1 RF shelf temp
      ama1c_instrtemp1 = 100*(ama1_h_digacoef(129)*1.0E-4  + 
     &                rcount*(ama1_h_digacoef(130)*1.0E-9  + 
     &                rcount*(ama1_h_digacoef(131)*1.0E-16 + 
     &                rcount* ama1_h_digacoef(132)*1.0E-20))) 
      rcount = ama2_temps(11)              !AMSU-A2 RF shelf temp
      ama1c_instrtemp2 = 100*(ama2_h_digacoef(41)*1.0E-4  + 
     &                rcount*(ama2_h_digacoef(42)*1.0E-9  + 
     &                rcount*(ama2_h_digacoef(43)*1.0E-16 + 
     &                rcount* ama2_h_digacoef(44)*1.0E-20))) 

      DO i = 1, 2
         ama1c_spare1(i) = 0               !spare
      ENDDO

! 1.4 Navigation

      ama1c_scalti = ama_scalti  ! sat altitude above ref. ellipsoid (km*10)

      DO fov = 1, Anfovs
         ama1c_latlon(1,fov) = ama_pos(fov*2-1) ! latitude
         ama1c_latlon(2,fov) = ama_pos(fov*2)   ! longitude
         latitudes(fov)  = REAL(ama1c_latlon(1,fov))*1.E-4
         longitudes(fov) = REAL(ama1c_latlon(2,fov))*1.E-4
      ENDDO  ! fov

!     angles
      DO fov = 1, Anfovs
         ama1c_angles(1,fov) = ama_ang(3*fov-1) ! zenith
         angle = ama_ang(3*fov)   ! azimuth
!        make sure azimuth between 0-360, rather than +/-180
         IF ( angle .LT. 0 ) angle = 36000 + angle
         ama1c_angles(2,fov) = angle
      ENDDO  ! fov

!     Calculate solar zenith/azimuth angles
      CALL INSUNA ( year, day, time, Anfovs, latitudes, longitudes,
     &              azimuths, zeniths )
      DO fov = 1, Anfovs
         ama1c_angles(3,fov) = NINT(zeniths(fov)*100.)  ! solar zenith
         ama1c_angles(4,fov) = NINT(azimuths(fov)*100.) ! solar azimuth
      ENDDO  ! fov

!     NESDIS 1B uses a "relative" azimuth, defined as satellite minus solar
!     angle. In AAPP it's the satellite angle. Allow for this difference.
!     Use site_id and modelid to identify NESDIS geolocation.

      IF (ama_h_siteid.EQ.'NSS'.AND.ama_h_modelid.NE.earth_modelid) THEN
        DO fov = 1, Anfovs
          angle = ama1c_angles(2,fov) + ama1c_angles(4,fov)
          IF (angle .GT. 36000) angle = angle - 36000
          ama1c_angles(2,fov) = angle
        ENDDO
      ENDIF

! 1.5 Initialise variables new in 1C
      DO i = 1, 2
         ama1c_spare2(i) = 0               !spare
      ENDDO
      DO chn = 1, Anchan
      DO fov = 1, Anfovs
         ama1c_btemps(chn,fov) = MDI       !brightness temperatures
      ENDDO  ! fov
      ENDDO  ! chn
      DO fov = 1, Anfovs
         ama1c_dataqual(fov) = 0           !qc word for each field of view
      ENDDO  ! fov


! 2.  Extract other information required here
!     ------- ----- ----------- -------- ----

! 2.1 Calibration coefficients (primary & secondary)
      DO chn = 1, Anchan
         chn3 = chn*3
         Acalib(1,1,chn) = REAL(ama_pmcal(chn3-2)) * 1.E-19  ! primary
         Acalib(1,2,chn) = REAL(ama_pmcal(chn3-1)) * 1.E-13
         Acalib(1,3,chn) = REAL(ama_pmcal(chn3))   * 1.E-09
         Acalib(2,1,chn) = REAL(ama_sdcal(chn3-2)) * 1.E-19  ! secondary
         Acalib(2,2,chn) = REAL(ama_sdcal(chn3-1)) * 1.E-13
         Acalib(2,3,chn) = REAL(ama_sdcal(chn3))   * 1.E-09
      ENDDO  ! chn

! 2.2 Counts from the instrument

!     A1
      DO fov = 1, Anfovs
      DO chn = 3, 15
         Acounts(chn,fov) = ama1_earth_data(chn+2,fov)
      ENDDO  ! chn
      ENDDO  ! fov

!     A2
      DO fov = 1, Anfovs
      DO chn = 1, 2
         Acounts(chn,fov) = ama2_earth_data(chn+2,fov)
      ENDDO  ! chn
      ENDDO  ! fov

! 2.3 NEDT
      DO chn = 1, Anchan
         IF (AcoldNEDT(chn,scan).GT. 0.0) THEN
           ama1c_coldnedt(chn) = AcoldNEDT(chn,scan)*1000
         ELSE
           ama1c_coldnedt(chn) = MDI
         ENDIF
         IF (AwarmNEDT(chn,scan).GT. 0.0) THEN
           ama1c_warmnedt(chn) = AwarmNEDT(chn,scan)*1000
         ELSE
           ama1c_warmnedt(chn) = MDI
         ENDIF
         IF (AcoldNEDT_rms(chn).GT. 0.0) THEN
           ama1c_h_coldnedt(chn) = AcoldNEDT_rms(chn)*1000
         ELSE
           ama1c_h_coldnedt(chn) = MDI
         ENDIF
         IF (AwarmNEDT_rms(chn).GT. 0.0) THEN
           ama1c_h_warmnedt(chn) = AwarmNEDT_rms(chn)*1000
         ELSE
           ama1c_h_warmnedt(chn) = MDI
         ENDIF
      ENDDO

      DO i = 1, 80-2*Anchan
         ama1c_filler(i) = 0      !filler
      ENDDO

      RETURN
      END
