!+ Determine whether moon is in the AMSU-A space view. Optionally correct the
!  data
      SUBROUTINE INAMOONCORR (Scan, A1BUnit)
      IMPLICIT NONE
!
! Name: $Id: inamooncorr.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!  If parameter mxlin_moon (inastore.h) is < number of scan lines (e.g. 1)
!  then flag those lines that are contaminated by the moon. 
!  If mxlin_moon is >=number of scan lines (e.g. 1000), use linear 
!  interpolation of the gain in order to correct the calibration coefficients.
!  Sets flags as follows:
!    MFlag=0 if all OK
!    MFlag=1 if AMSU-A1 is affected
!    MFlag=2 if AMSU-A2 is affected
!    MFlag=3 if both are affected
!    ama_linqualflgs bit 16: Moon in A1, uncorrected
!    ama_linqualflgs bit 17: Moon in A2, uncorrected
!    ama_linqualflgs bit 18: Moon in A1, corrected
!    ama_linqualflgs bit 19: Moon in A2, corrected
!  These bits are currently zero fill. (ama_linqualflgs is copied to
!  ama1c_scanqual in INAGET).
!
!  Current code owner: N.C.Atkinson
!
!  History:
!
!  Version  Date       Comment
!  1.0    12/06/01  Original code.
!  1.1    15/01/03  Ensure a1gain is used in A1 section.      Nigel Atkinson
!  1.2    05/03/04  Correct initialisation of c1 & c2.        Nigel Atkinson
!  1.3    19/03/04  Modifycoefs: new "have calibration" test  Aarno Korpela
!  1.4    18/10/04  Modifycoefs: check new values for pmcal are within
!                   range of integer*4                        Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
#include "inparms.h"      
#include "inastore.h"
#include "ama1b.h"
#include "bits.h"

! Arguments:
      INTEGER Scan, A1BUnit

! Local variables:
      INTEGER NewScan, Chan, Index, A1BRecord
      REAL a1gain(14,3)  !scan number+13 channels, 1=first, 2=last, 3=temporary
      REAL a2gain(3,3)   !scan number+2 channels, 1=first, 2=last, 3=temporary
      DATA a1gain, a2gain /42*0.0, 9*0.0/
      REAL Gain, NewGain, f1, f2
 
! Function and subroutine calls:
      EXTERNAL INAMOONTEST   !Calculates the moon-spaceview angles
!
!- End of header

! Option 1: Just flag the moon, don't correct data
      IF (mxlin_moon.LT.ama_h_scnlin) THEN
        IF (ama_qualind.GE.0) THEN     !Only proceed if 'do not use' flag clear
          CALL INAMOONTEST         !Result in MFlag
          IF (MOD(MFlag,2).EQ.1)   !Set line quality bit 16 if A1 contaminated
     &      ama_linqualflgs = IBSET(ama_linqualflgs, bits(16))
          IF (MFlag.GE.2)          !Set line quality bit 17 if A2 contaminated
     &      ama_linqualflgs = IBSET(ama_linqualflgs, bits(17))
        ENDIF 
        RETURN
      ENDIF

! Option 2: Linear interpolation of gain
      IF (ama_qualind .LT. 0) THEN    !Look at 'do not use' flag
        IF (Scan.EQ.1) THEN
          Moonflag(Scan) = 0   !Assume first scan is OK
        ELSE
          Moonflag(Scan) = Moonflag(Scan-1)  !Assume no change
        ENDIF
        RETURN       !Only proceed if 'do not use' flag is clear
      ENDIF

      IF (Moonflag(Scan) .EQ. -1) THEN    !Scan line not previously examined
 
        CALL INAMOONTEST
        Moonflag(Scan) = MFlag

        IF (MOD(MFlag,2).EQ.0) THEN    !A1 OK
          DO Chan=3,15
            CALL MODIFYCOEFS (Chan, Gain, 0.0)   !Calculate the gain
            a1gain(Chan-1,1) = Gain
          ENDDO
          a1gain(1,1)=Scan    !Store the scan number
        ENDIF

        IF (MFlag.LT.2) THEN    !A2 OK
          DO Chan=1,2
            CALL MODIFYCOEFS (Chan, Gain, 0.0)   !Calculate the gain
            a2gain(Chan+1,1) = Gain
          ENDDO
          a2gain(1,1)=Scan    !Store the scan number
        ENDIF

        IF (MFlag.NE.0) THEN      !First moon contaminated line

          NewScan = Scan
  100     NewScan = NewScan+1
          IF (NewScan.LE.ama_h_scnlin) THEN
            A1BRecord = NewScan + ama_h_hdrcnt
            CALL IOA1B ("READ DATA", A1BUnit, A1BRecord)   !Read next 1B scan 

            IF (ama_qualind .LT. 0) THEN   !Bad data. Ensure it is not used
              Moonflag(NewScan) = Moonflag(NewScan-1)
              MFlag = Moonflag(NewScan)
            ELSE
              CALL INAMOONTEST
              Moonflag(NewScan) = MFlag

              IF (MOD(MFlag,2).EQ.0) THEN              !A1 OK
                IF (MOD(Moonflag(NewScan-1),2).EQ.0) THEN
                  Index = 3       !Previous scan OK. Store results temporarily
                ELSE
                  Index = 2       !First good scan following bad period
                ENDIF
                DO Chan=3,15
                  CALL MODIFYCOEFS (Chan, Gain, 0.0)     !Calculate gains
                  a1gain(Chan-1,Index) = Gain
                ENDDO
                a1gain(1,Index) = NewScan
              ELSE
                IF (MOD(Moonflag(NewScan-1),2).EQ.0) THEN  !Bad scan following 
                  DO Chan=3,15                             ! good period
                    a1gain(Chan-1,1) = a1gain(Chan-1,3) !Use most recent good
                  ENDDO                                 ! data
                  a1gain(1,1) =  a1gain(1,3)
                ENDIF
              ENDIF

              IF (MFlag.LT.2) THEN                     !A2 OK
                IF (Moonflag(NewScan-1).LT.2) THEN
                  Index = 3       !Previous scan OK. Store results temporarily
                ELSE
                  Index = 2       !First good scan following bad period
                ENDIF
                DO Chan=1,2
                  CALL MODIFYCOEFS (Chan, Gain, 0.0)     !Calculate gains
                  a2gain(Chan+1,Index) = Gain
                ENDDO
                a2gain(1,Index) = NewScan
              ELSE
                IF (Moonflag(NewScan-1).LT.2) THEN   !First bad scan following
                  DO Chan=1,2                        ! good period
                    a2gain(Chan+1,1) = a2gain(Chan+1,3) !Use most recent good
                  ENDDO                                 ! data
                  a2gain(1,1) =  a2gain(1,3)
                ENDIF
              ENDIF
            ENDIF              

            IF (MFlag.NE.0) GOTO 100   !Continue until moon event ended

          ELSE   !End of file reached
            IF (MOD(MoonFlag(NewScan-1),2).EQ.1) a1gain(1,2) = 0.0 !Unavailable
            IF (MoonFlag(NewScan-1).GE.2)        a2gain(1,2) = 0.0 !Unavailable
          ENDIF

          A1BRecord = Scan + ama_h_hdrcnt
          CALL IOA1B ("READ DATA", A1BUnit, A1BRecord)  !Re-read original scan
        ENDIF
      ENDIF

! Now see if any interpolation is required for the gain

      IF (MOD(Moonflag(Scan),2) .EQ.1) THEN    !A1 needs interpolation
        IF ((a1gain(1,1).NE.0.0) .OR. (a1gain(1,2).NE.0.0)) THEN
          IF (a1gain(1,2).EQ.0.0) THEN
            f1 = 1.0       !No end point
            f2 = 0.0
          ELSE IF (a1gain(1,1).EQ.0.0) THEN
            f1 = 0.0       !No start point
            f2 = 1.0
          ELSE
            f1 = (a1gain(1,2)-Scan)/(a1gain(1,2)-a1gain(1,1))
            f2 = 1.0-f1
          ENDIF
          DO Chan=3,15
            NewGain = f1*a1gain(Chan-1, 1) + f2*a1gain(Chan-1, 2)
            Call Modifycoefs (Chan, Gain, NewGain)
          ENDDO
          Moonflag(Scan) = Moonflag(Scan)-1  !Successful A1 correction
          ama_linqualflgs = IBSET(ama_linqualflgs, bits(18)) !Flag it
        ENDIF
      ENDIF

      IF (Moonflag(Scan).GE.2) THEN    !A2 needs interpolation
        IF ((a2gain(1,1).NE.0.0) .OR. (a2gain(1,2).NE.0.0)) THEN
          IF (a2gain(1,2).EQ.0.0) THEN
            f1 = 1.0       !No end point
            f2 = 0.0
          ELSE IF (a2gain(1,1).EQ.0.0) THEN
            f1 = 0.0       !No start point
            f2 = 1.0
          ELSE
            f1 = (a2gain(1,2)-Scan)/(a2gain(1,2)-a2gain(1,1))
            f2 = 1.0-f1
          ENDIF
          DO Chan=1,2
            NewGain = f1*a2gain(Chan+1, 1) + f2*a2gain(Chan+1, 2)
            Call Modifycoefs (Chan, Gain, NewGain)
          ENDDO
          Moonflag(Scan) = Moonflag(Scan)-2  !Successful A2 correction
          ama_linqualflgs = IBSET(ama_linqualflgs, bits(19)) !Flag it
        ENDIF    !Otherwise do not attempt corrections
      ENDIF
      MFlag = Moonflag(Scan)

      IF (MOD(MFlag,2).EQ.1)    !Set line quality bit 16 if A1 contaminated
     &    ama_linqualflgs = IBSET(ama_linqualflgs, bits(16))
      IF (MFlag.GE.2)           !Set line quality bit 17 if A2 contaminated
     &    ama_linqualflgs = IBSET(ama_linqualflgs, bits(17))

      RETURN
      END
!
!
!
      SUBROUTINE Modifycoefs (Chan, Gain, NewGain)

! Calculate gain for specified AMSU-A channel.
! Optionally over-write the calibration parameters according to a new gain,
! where the impact is zero at the warm reference point, nominally 290K.

#include "ama1b.h"
      LOGICAL haveCalib
      INTEGER Chan
      REAL Gain, NewGain
      REAL a0, a1, a2, rwarm, rcold, xwarm, xcold, NewGain2, xcoldmod, u
      REAL xref
      REAL c1,c2
      DATA c1/1.1910439e-5/,c2/1.4387687/
!
      haveCalib = .TRUE.
      IF (ama_pmcal((Chan-1)*3+2).EQ.0) haveCalib = .FALSE.   !a1
      IF (ama_pmcal((Chan-1)*3+3).EQ.0) haveCalib = .FALSE.   !a0
      IF (.NOT.haveCalib) THEN
        Gain=0.0   !No calibration data
        RETURN
      ENDIF
      a0 = ama_pmcal((Chan-1)*3+3)*1E-9
      a1 = ama_pmcal((Chan-1)*3+2)*1E-13
      a2 = ama_pmcal((Chan-1)*3+1)*1E-19
      xref = -a0/a1       !Approx zero radiance counts
      a0 = a2*xref*xref   !Shift x origin
      a1 = a1 + 2*a2*xref
      w = ama_h_temrad((chan-1)*3+1)*1E-6
      rwarm = c1/c2 *w*w * 290.0    !Approx warm reference
      rcold = c1/c2 *w*w * 3.0      !Approx cold reference
      xwarm = xref + (rwarm-a0)/a1 * (1 - a2/(a1*a1)*(rwarm-a0)) !Inverse
      xcold = xref + (rcold-a0)/a1 * (1 - a2/(a1*a1)*(rcold-a0)) ! quadratic
      Gain = (xwarm-xcold)/(rwarm-rcold)

      IF (NewGain.NE.0.0) THEN
        NewGain2 = NewGain*NewGain
        xcoldmod = xwarm-NewGain*(rwarm-rcold)
        u = Gain*Gain*a2   !Retain the existing nonlinearity parameter

!v1.4: Compute the new values of a0, a1, a2, apply the scaling factors and 
!      check they are all within the range of an integer*4.
        a0 = (rwarm - xwarm/NewGain + u*(xcoldmod*xwarm/NewGain2))*1E9
        a1 = (1/NewGain - u*((xwarm+xcoldmod)/NewGain2))*1E13
        a2 = u/NewGain2*1E19
        IF (a0 .GE. -2147483648.0 .AND. a0 .LE. 2147483647.0 .AND.
     &      a1 .GE. -2147483648.0 .AND. a1 .LE. 2147483647.0 .AND.
     &      a2 .GE. -2147483648.0 .AND. a2 .LE. 2147483647.0) THEN
          ama_pmcal((Chan-1)*3+3) = a0
          ama_pmcal((Chan-1)*3+2) = a1
          ama_pmcal((Chan-1)*3+1) = a2
        ENDIF
      ENDIF
      RETURN
      END                   
