!+ process AMSU-A data within ATOVIN
!
      SUBROUTINE INAMSA ( A1Bunit, A1Cunit, FDFunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inamsa.F 461 2015-09-15 14:38:15Z frna $
!
! Purpose:
!   Controlling routine to read level 1b AMSU-A data,
!   convert AMSU-A electronic counts to radiances, then to
!   brightness temperatures, and write out as level 1c.
!
! Method:
! 1.  Open AMSU-A 1B & 1C files
! 2.  Read 1B header & set up 1C header
! 3.  Loop through scan lines, reading, processing & writing data
! 4.  Complete & write out 1C header
! 5.  Close files
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0    12/12/96  Original code. (R.J.Renshaw)
!  1.1    13/6/01   Add calls to moon routines (N.C.Atkinson)
!  1.2     5/3/03   Add call to INFDF to read antenna efficiencies
!  1.3    03/11/06  Fill in ama1c_h_vnantennacorr. NCA
!  1.4    15/09/15  Add NEDT calculation
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-A level 1B header/data common blocks 
#include "ama1b.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!
!   Scalar arguments with intent(In):
      INTEGER       A1Bunit       ! unit number for AMSU-A level 1B file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       FDFunit       ! unit number for fixed data file
!
! Local scalars:
      INTEGER       first_lineno  ! number of first scanline in the 1B file
      INTEGER       last_lineno   ! number of last scanline in the 1B file
      INTEGER       A1Brecord     ! record number for AMSU-A level 1B file
      INTEGER       A1Crecord     ! record number for AMSU-A level 1C file
      INTEGER       scnlin        ! loop variable over scanlines
      INTEGER       satid         ! For nesdis satid from 1b header
      INTEGER       isat          ! loop variable over satellites
      LOGICAL       first_line    ! true for first scanline in the 1B file
         DATA       first_line / .TRUE. /
      LOGICAL       write_data    ! .TRUE. if line is to be written out
!
! Function & Subroutine calls:
      External INAHDR               ! interpret AMSU-A level 1B header into 1C
      External INAGET               ! read AMSU-A 1B scan line
      External INAPRC               ! process AMSU-A scan line
      External IOA1B                ! I/O routine for level 1B AMSU-A data
      External IOA1C                ! I/O routine for level 1C AMSU-A data
!
!- End of header


! 1.  Open AMSU-A 1B & 1C files
!     ---- ---- -- - -- -----

      CALL IOA1B ("OPEN/READ", A1Bunit, A1Brecord )    ! 1B file
      CALL IOA1C ("OPEN/WRITE", A1Cunit, A1Crecord )   ! 1C file


! 2.  Read 1B header & set up 1C header
!     ---- -- ------ - --- -- -- ------

! 2.1 Read 1B header
      A1Brecord = 1   ! Headers start on record 1.
      CALL IOA1B ("READ HEADER", A1Bunit, A1Brecord )

!     point to last header record in 1B
      A1Brecord = ama_h_hdrcnt
      
!     read antenna efficiencies and fill in the version number
      satid = ama_h_satid
      CALL INFDF (FDFunit, satid)
      DO isat=1,Nsat
        IF (satid.EQ.NESDISid(isat)) THEN
          ama1c_h_vnantennacorr = FDFvnno(isat)
        ENDIF
      ENDDO

! 2.2 Calculate moon position at start and end of orbit
      CALL INAMOONINIT

! 2.3 Set up 1C header
      CALL INAHDR

! 2.4 Do an initial pass through the data to get information for NEDT
      DO scnlin = 1, ama_h_scnlin
         A1Brecord = A1Brecord + 1
         CALL IOA1B ("READ DATA", A1Bunit, A1Brecord )
         CALL INAGET_NEDT ( scnlin )
      ENDDO

! 2.5 Compute the NEDTs
      CALL INAPRC_NEDT

!     point to last header record in 1B
      A1Brecord = ama_h_hdrcnt

!     point to last header record in 1C
      A1Crecord = ama1c_h_hdrcnt


! 3.  Loop through scan lines, reading, processing & writing data
!     ---- ------- ---- ------ -------- ---------- - ------- ----

      DO scnlin = 1, ama_h_scnlin

! 3.1    Read a scanline
         A1Brecord = A1Brecord + 1
         CALL IOA1B ("READ DATA", A1Bunit, A1Brecord )

! 3.2    Test each scan for moon contamination
         CALL INAMOONCORR ( scnlin, A1BUnit )

! 3.3    Transfer 1B data into program and 1C arrays
         CALL INAGET ( scnlin )

! 3.3    Process the scanline
         CALL INAPRC ( write_data )

! 3.4    Write it out, if it contains any good data
         IF ( write_data ) THEN

            A1Crecord = A1Crecord + 1
            CALL IOA1C ("WRITE DATA", A1Cunit, A1Crecord )

!           Record the number of the first & last scanlines in the file
            IF ( first_line ) THEN
               first_lineno = ama1c_scnlin
               first_line = .FALSE.
            ENDIF
            last_lineno = ama1c_scnlin

         ENDIF

      ENDDO   ! (scnlin)


! 4.  Complete & write out 1C header
!     -------- - ----- --- -- ------

!     correct 1C header to show the correct number of scanlines
      ama1c_h_scnlin = A1Crecord - ama1c_h_hdrcnt

!     ..and the correct number of missing scanlines
      ama1c_h_misscnlin = (1+last_lineno-first_lineno) - ama1c_h_scnlin

!     write the 1C header
      A1Crecord = 1
      CALL IOA1C ("WRITE HEADER", A1Cunit, A1Crecord )

      WRITE(*,*) ama1c_h_scnlin," scanlines written to AMSU-A 1C file"
      WRITE(*,*)


! 5.  Close files
!     ----- -----

      CALL IOA1B ("CLOSE", A1Bunit, A1Brecord )   ! 1B file
      CALL IOA1C ("CLOSE", A1Cunit, A1Crecord )   ! 1C file


      RETURN
      END
