!+ process AMSU-A data within ATOVIN_ANTCORR
!
      SUBROUTINE INAMSA_ANTCORR ( A1Cunit, FDFunit, Mode )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inamsa_antcorr.F 58 2006-11-08 10:31:36Z frna $
!
! Purpose:
!   Routine to read level 1c AMSU-A data,
!   apply or remove antenna corrections, and write out as level 1c.
!
! Method:
! 1.  Open AMSU-A 1C files
! 2.  Loop through scan lines, reading, processing & writing data
! 3.  Close files
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2006, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0     02/11/06 Based on inamsa.F and inaprc.F. Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  Constants
#include "constants_preproc.h"
!
!   Scalar arguments with intent(In):
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       FDFunit       ! unit number for fixed data file
      INTEGER       Mode          ! Mode=1 to apply, -1 to remove antenna corr
!
! Local scalars:
      INTEGER       first_lineno  ! number of first scanline in the 1B file
      INTEGER       last_lineno   ! number of last scanline in the 1B file
      INTEGER       A1Crecord     ! record number for AMSU-A level 1C file
      INTEGER       scnlin        ! loop variable over scanlines
      INTEGER       satid         ! For WMO satid from 1c header
      INTEGER       chn           ! loop variable over all channels
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       isat          ! loop variable over satellites in FDF file
      REAL          radiance      ! radiance
      REAL          bri_temp      ! brightness temperature
      REAL          wave_no       ! wavenumber
      
!
! Function & Subroutine calls:
      External IOA1C                ! I/O routine for level 1C AMSU-A data
!
! Cosmic background radiance for each channel, calculated assuming 2.73 Kelvin
! Wavenumbers from calibration parameters file
      REAL Bspace(Anchan)
      DATA Bspace /
     & 1.14709E-05, 1.85760E-05, 3.95843E-05, 4.25294E-05, 4.34692E-05,
     & 4.44188E-05, 4.50567E-05, 4.57183E-05, 4.78323E-05, 4.78323E-05,
     & 4.78323E-05, 4.78323E-05, 4.78323E-05, 4.78323E-05, 8.24266E-05 /
      REAL wavenumbers(Anchan)
      DATA wavenumbers /
     & 0.793883,1.047391,1.677827,1.761218,1.787770,1.814589,1.832601,
     & 1.851281,1.911000,1.911000,1.911000,1.911000,1.911000,1.911000,
     & 2.968720 /
!
!- End of header


! 1.  Open AMSU-A 1C file
!     ---- ------ -- ----

      CALL IOA1C ("OPEN/READ", A1Cunit, A1Crecord )   ! 1C file


! 2.  Read 1C header
!     ---- -- ------

      A1Crecord = 1   ! Headers start on record 1.
      CALL IOA1C ("READ HEADER", A1Cunit, A1Crecord )

!     read antenna efficiencies
      satid = -1*ama1c_h_wmosatid  !INFDF interprets negative satid as WMO satid
      CALL INFDF (FDFunit, satid)

      IF (mode.EQ.1) THEN
        DO isat=1,Nsat
          IF (ama1c_h_wmosatid.EQ.WMOid(isat)) THEN
            IF (ama1c_h_vnantennacorr.NE.0.AND.
     &          ama1c_h_vnantennacorr.NE.MDI) THEN
              WRITE(*,*) 'WARNING: ama1c_h_vnantennacorr is ',
     &           ama1c_h_vnantennacorr,', expect zero'
            ENDIF
            WRITE(*,*) 'Apply antenna coefs version',FDFvnno(isat)
            ama1c_h_vnantennacorr = FDFvnno(isat)
          ENDIF
        ENDDO
      ELSE
        DO isat=1,Nsat
          IF (ama1c_h_wmosatid.EQ.WMOid(isat)) THEN
            IF (ama1c_h_vnantennacorr.NE.FDFvnno(isat).AND.
     &          ama1c_h_vnantennacorr.NE.MDI) THEN
              WRITE(*,*) 'WARNING: ama1c_h_vnantennacorr is ',
     &           ama1c_h_vnantennacorr,', expect ',FDFvnno(isat)
            ENDIF
            WRITE(*,*) 'Remove antenna coefs version',FDFvnno(isat)
            ama1c_h_vnantennacorr = 0
          ENDIF
        ENDDO
      ENDIF

!     compute APlanck
      DO chn = 1, Anchan
        wave_no = wavenumbers(chn)
        APlanck(1,chn) = C_1 * wave_no * wave_no * wave_no
        APlanck(2,chn) = C_2 * wave_no
        APlanck(3,chn) = 0.0        !No band corrections for AMSU-A
        APlanck(4,chn) = 1.0
      ENDDO

!     point to last header record in 1C
      A1Crecord = ama1c_h_hdrcnt

! 3.  Loop through scan lines, reading, processing & writing data
!     ---- ------- ---- ------ -------- ---------- - ------- ----

      DO scnlin = 1, ama1c_h_scnlin

! Read a scanline
         A1Crecord = A1Crecord + 1
         CALL IOA1C ("READ DATA", A1Cunit, A1Crecord )

! Check quality flag

         IF ( .NOT. BTEST( ama1c_qualind, bits(31) ) ! don't use
     &    .AND..NOT.BTEST( ama1c_qualind, bits(28) ) ! no calibration
     &     ) THEN

! Convert BTs to radiance

            DO fov = 1, Anfovs          
              DO chn = 1, Anchan
                IF (ama1c_btemps(chn,fov).GT.0) THEN
                  bri_temp = ama1c_btemps(chn,fov)*0.01
                  bri_temp = bri_temp*APlanck(4,chn) + APlanck(3,chn)
                  radiance = 
     &              APlanck(1,chn)/(exp(APlanck(2,chn)/bri_temp) - 1)

! Apply antenna corrections
!               We have values for the efficiency with which the antenna sees
!               the earth, the spacecraft, and space for each scan position.
!               Assume radiance from the spacecraft is the same as from the
!               earth (i.e. reflecting) and then adjust the antenna-observed
!               radiance to the value we expect from the earth alone.

                  IF (Mode.EQ.1) THEN
                    radiance=(radiance-Aeff_Space(fov,chn)*Bspace(chn))
     &                     /(Aeff_Earth(fov,chn) + Aeff_Platf(fov,chn))
                  ELSE
                    radiance = radiance
     &                *(Aeff_Earth(fov,chn)+Aeff_Platf(fov,chn))
     &                + Aeff_Space(fov,chn)*Bspace(chn)
                  ENDIF

! Convert radiance back to BT

                  bri_temp = APlanck(2,chn)         ! Kelvin
     &                     / log( 1. + APlanck(1,chn)/radiance )
                  bri_temp = (bri_temp-APlanck(3,chn)) / APlanck(4,chn)
                  ama1c_btemps(chn,fov) = NINT ( bri_temp*100. )
                ENDIF
              ENDDO
            ENDDO

! Write out the scan line

            CALL IOA1C ("WRITE DATA", A1Cunit, A1Crecord )

         ENDIF   ! scan line OK
      ENDDO   ! (scnlin)

      WRITE(*,*) ama1c_h_scnlin," scanlines written to AMSU-A 1C file"
      WRITE(*,*)

! 4.  Update header and close files
!     ------ ------ --- ----- -----

      CALL IOA1C ("WRITE HEADER", A1Cunit, 1 )
      CALL IOA1C ("CLOSE", A1Cunit, A1Crecord )


      RETURN
      END
