!+ process AMSU-B data within ATOVIN
!
      SUBROUTINE INAMSB ( B1Bunit, B1Cunit, FDFunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inamsb.F 461 2015-09-15 14:38:15Z frna $
!
! Purpose:
!   Controlling routine to read level 1b AMSU-B or MHS data,
!   convert AMSU-B electronic counts to radiances, then to
!   brightness temperatures, and write out as level 1c.
!
! Method:
! 1.  Open AMSU-B (or MHS) 1B & 1C files
! 2.  Read 1B header & set up 1C header
! 3.  Loop through scan lines, reading, processing & writing data
! 4.  Complete & write out 1C header
! 5.  Close files
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0    12/12/96  Original code. (R.J.Renshaw)
!  1.1    04/01/99  Call AMSU-B bias correction routines (N.C.Atkinson)
!  1.2     5/3/03   Add call to INFDF to read antenna efficiencies
!  1.3    26/05/04  Add MHS (N.C.Atkinson)
!  1.4    23/11/04  Read bias corrections from file (optional)
!  1.5    03/11/06  Fill in amb1c_h_vnantennacorr. NCA
!  1.6    15/09/15  Add NEDT calculation
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-B Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  AMSU-B level 1B header/data common blocks 
#include "amb1b.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!  AMSU-B unit numbers for AMB_READCORR 
#include "ambclin.h"
!
!   Scalar arguments with intent(In):
      INTEGER       B1Bunit       ! unit number for AMSU-B level 1B file
      INTEGER       B1Cunit       ! unit number for AMSU-B level 1C file
      INTEGER       FDFunit       ! unit number for fixed data file
!
! Local scalars:
      INTEGER       first_lineno  ! number of first scanline in the 1B file
      INTEGER       last_lineno   ! number of last scanline in the 1B file
      INTEGER       B1Brecord     ! record number for AMSU-B level 1B file
      INTEGER       B1Crecord     ! record number for AMSU-B level 1C file
      INTEGER       scnlin        ! loop variable over scanlines
      INTEGER       Tx            ! loop variable over transmitters
      INTEGER       satid         ! For nesdis satid from 1b header
      INTEGER       isat          ! loop variable over satellites
      LOGICAL       first_line    ! true for first scanline in the 1B file
         DATA       first_line / .TRUE. /
      LOGICAL       write_data    ! .TRUE. if line is to be written out
      LOGICAL       mhspresent    ! .TRUE. if we have MHS instead of AMSU-B
!
! Function & Subroutine calls:
      External INBHDR               ! interpret AMSU-B level 1B header into 1C
      External INMHSHDR             ! interpret MHS level 1B header into 1C
      External INBGET               ! read AMSU-B 1B scan line
      External INMHSGET             ! read MHS 1B scan line
      External INBPRC               ! process AMSU-B scan line
      External IOB1B                ! I/O routine for level 1B AMSU-B data
      External IOMHS1B              ! I/O routine for level 1B MHS data
      External IOB1C                ! I/O routine for level 1C AMSU-B data
!
!- End of header


! 1.  Open AMSU-B 1B & 1C files
!     ---- ---- -- - -- -----

      CALL IOB1B ("OPEN/READ", B1Bunit, B1Brecord )    ! 1B file
      CALL IOB1C ("OPEN/WRITE", B1Cunit, B1Crecord )   ! 1C file


! 2.  Read 1B header & set up 1C header
!     ---- -- ------ - --- -- -- ------

! 2.1 Read 1B header
      B1Brecord = 1   ! Headers start on record 1.
      CALL IOB1B ("READ HEADER", B1Bunit, B1Brecord )
      
! 2.2 See whether we are dealing with MHS. If so, re-read header
      IF (amb_h_datatyp .EQ. 12) THEN
        mhspresent = .true.
        CALL IOMHS1B ("READ HEADER", B1Bunit, B1Brecord )
      ELSE
        mhspresent = .false.
      ENDIF
      
!     point to last header record in 1B
      B1Brecord = amb_h_hdrcnt

! 2.3 read antenna efficiencies
      satid = amb_h_satid
      CALL INFDF (FDFunit, satid)
      DO isat=1,Nsat
        IF (satid.EQ.NESDISid(isat)) THEN
          amb1c_h_vnantennacorr = FDFvnno(isat)
        ENDIF
      ENDDO

!     Read RFI bias corrections from file, over-writing values in 1b header.
!     This has no effect if amsubcl has been run, but if the 1b file has come
!     from elsewhere (e.g. NESDIS) then it can be used to force the 1c file to
!     be compatible with the AAPP corrections. Uncomment if required.
!      amb_corcoefu = 17      
!      amb_cllgu = 6
!      CALL AMB_READCORR

! 2.4 See if reference power information is present in 1B header. If so, get
!     bias correction tables from 1B header and interpolate to 90 views.
!     Read scan 6 in order to pick up STX and SARR powers (unreliable on scans
!     1-5). Applies to AMSU-B only.

      IF (amb_h_TxPow(1).NE.0.AND..NOT.mhspresent) THEN
        CALL AMB_GETCORR
        CALL AMB_GETSTX1     !Read March 99 bias table from external file
        IF (amb_h_scnlin.GE.6) THEN
          CALL IOB1B ("READ DATA", B1Bunit, B1Brecord+6 )
          DO Tx=1,5
            STXPowTmp(Tx) = amb_ahk(22+Tx)
          ENDDO
        ENDIF
      ENDIF

! 2.5 Set up 1C header
      IF (.NOT.mhspresent) THEN
        CALL INBHDR
      ELSE
        CALL INMHSHDR

! 2.6 Do an initial pass through the MHS data to get information for NEDT
        DO scnlin = 1, amb_h_scnlin
           B1Brecord = B1Brecord + 1
           CALL IOMHS1B ("READ DATA", B1Bunit, B1Brecord )
           CALL INMHSGET_NEDT ( scnlin )
        ENDDO

! 2.7 Compute the MHS NEDTs
        CALL INMHSPRC_NEDT

      ENDIF

!     point to last header record in 1B
      B1Brecord = amb_h_hdrcnt

!     point to last header record in 1C
      B1Crecord = amb1c_h_hdrcnt


! 3.  Loop through scan lines, reading, processing & writing data
!     ---- ------- ---- ------ -------- ---------- - ------- ----

      DO scnlin = 1, amb_h_scnlin

! 3.1    Read a scanline
         B1Brecord = B1Brecord + 1
         IF (.NOT.mhspresent) THEN
           CALL IOB1B ("READ DATA", B1Bunit, B1Brecord )
         ELSE
           CALL IOMHS1B ("READ DATA", B1Bunit, B1Brecord )
         ENDIF
         
! 3.2    Transfer 1B data into program and 1C arrays
         IF (.NOT.mhspresent) THEN
           CALL INBGET
         ELSE
           CALL INMHSGET ( scnlin )
         ENDIF

!        Correct AMSU-B Earth-view data for transmitter interference
!        Only do this if reference powers are present
         IF (amb_h_TxPow(1).NE.0.AND..NOT.mhspresent) THEN
           CALL AMB_EARTHCORR
         ENDIF
         
! 3.3    Process the scanline
         CALL INBPRC ( write_data )
         
! 3.4    Write it out, if it contains any good data
         IF ( write_data ) THEN

            B1Crecord = B1Crecord + 1
            CALL IOB1C ("WRITE DATA", B1Cunit, B1Crecord )
            
!           Record the number of the first & last scanlines in the file
            IF ( first_line ) THEN
               first_lineno = amb1c_scnlin
               first_line = .FALSE.
            ENDIF
            last_lineno = amb1c_scnlin

         ENDIF

      ENDDO   ! (scnlin)


! 4.  Complete & write out 1C header
!     -------- - ----- --- -- ------

!     correct 1C header to show the correct number of scanlines
      amb1c_h_scnlin = B1Crecord - amb1c_h_hdrcnt

!     ..and the correct number of missing scanlines
      amb1c_h_misscnlin = (1+last_lineno-first_lineno) - amb1c_h_scnlin

!     write the 1C header
      B1Crecord = 1
      CALL IOB1C ("WRITE HEADER", B1Cunit, B1Crecord )

      WRITE(*,*) amb1c_h_scnlin," scanlines written to AMSU-B 1C file"
      WRITE(*,*)


! 5.  Close files
!     ----- -----

      CALL IOB1B ("CLOSE", B1Bunit, B1Brecord )   ! 1B file
      CALL IOB1C ("CLOSE", B1Cunit, B1Crecord )   ! 1C file


      RETURN
      END
