!+ process AMSU-B/MHS data within ATOVIN_ANTCORR
!
      SUBROUTINE INAMSB_ANTCORR ( B1Cunit, FDFunit, Mode )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inamsb_antcorr.F 58 2006-11-08 10:31:36Z frna $
!
! Purpose:
!   Routine to read level 1c AMSU-B/MHS data,
!   apply or remove antenna corrections, and write out as level 1c.
!
! Method:
! 1.  Open AMSU-B 1C files
! 2.  Loop through scan lines, reading, processing & writing data
! 3.  Close files
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2006, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0     02/11/06 Based on inbmsa.F and inbprc.F. Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-B/MHS Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  AMSU-B/MHS level 1C header/data common blocks 
#include "amb1c.h"
!  Constants
#include "constants_preproc.h"
!
!   Scalar arguments with intent(In):
      INTEGER       B1Cunit       ! unit number for AMSU-B/MHS level 1C file
      INTEGER       FDFunit       ! unit number for fixed data file
      INTEGER       Mode          ! Mode=1 to apply, -1 to remove antenna corr
!
! Local scalars:
      INTEGER       first_lineno  ! number of first scanline in the 1B file
      INTEGER       last_lineno   ! number of last scanline in the 1B file
      INTEGER       B1Crecord     ! record number for AMSU-B/MHS level 1C file
      INTEGER       scnlin        ! loop variable over scanlines
      INTEGER       satid         ! For WMO satid from 1c header
      INTEGER       chn           ! loop variable over all channels
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       isat          ! loop variable over satellites in FDF file
      REAL          radiance      ! radiance
      REAL          bri_temp      ! brightness temperature
      REAL          wave_no       ! wavenumber
      REAL          targ_temp     ! Internal target temperature
      REAL          earth_ang     ! Earth-view antenna angle
      REAL          space_ang     ! Space-view antenna angle
      
!
! Function & Subroutine calls:
      External IOB1C                ! I/O routine for level 1C AMSU-B/MHS data
!
! Cosmic background radiance for each channel, calculated assuming 2.73 Kelvin.
! The values are strictly speaking incorrect for MHS, but keep this subroutine
! consistent with inamsb.F
! Wavenumbers from calibration parameters file. (Initialise as AMSU-B, but
! over-write later for MHS)
      REAL Bspace(Bnchan)
      DATA Bspace /
     & 8.24175E-05, 1.15024E-04, 1.13024E-04, 1.13024E-04, 1.13024E-04 /
      REAL wavenumbers(Bnchan)
      DATA wavenumbers /2.9684, 5.0032, 6.1146, 6.1146, 6.1146 /
!
! Antenna reflectivities from fdf.dat
      REAL          reflectivity_diff(20)  ! AMSU antenna reflectivity diffs
      COMMON /amsu_reflectivity/ reflectivity_diff
!
!- End of header


! 1.  Open AMSU-B/MHS 1C file
!     ---- ---------- -- ----

      CALL IOB1C ("OPEN/READ", B1Cunit, B1Crecord )   ! 1C file


! 2.  Read 1C header
!     ---- -- ------

      B1Crecord = 1   ! Headers start on record 1.
      CALL IOB1C ("READ HEADER", B1Cunit, B1Crecord )

!     read antenna efficiencies
      satid = -1*amb1c_h_wmosatid  !INFDF interprets negative satid as WMO satid
      CALL INFDF (FDFunit, satid)

      IF (mode.EQ.1) THEN
        DO isat=1,Nsat
          IF (amb1c_h_wmosatid.EQ.WMOid(isat)) THEN
            IF (amb1c_h_vnantennacorr.NE.0.AND.
     &          amb1c_h_vnantennacorr.NE.MDI) THEN
              WRITE(*,*) 'WARNING: amb1c_h_vnantennacorr is ',
     &           amb1c_h_vnantennacorr,', expect zero'
            ENDIF
            WRITE(*,*) 'Apply antenna coefs version',FDFvnno(isat)
            amb1c_h_vnantennacorr = FDFvnno(isat)
          ENDIF
        ENDDO
      ELSE
        DO isat=1,Nsat
          IF (amb1c_h_wmosatid.EQ.WMOid(isat)) THEN
            IF (amb1c_h_vnantennacorr.NE.FDFvnno(isat).AND.
     &          amb1c_h_vnantennacorr.NE.MDI) THEN
              WRITE(*,*) 'WARNING: amb1c_h_vnantennacorr is ',
     &           amb1c_h_vnantennacorr,', expect ',FDFvnno(isat)
            ENDIF
            WRITE(*,*) 'Remove antenna coefs version',FDFvnno(isat)
            amb1c_h_vnantennacorr = 0
          ENDIF
        ENDDO
      ENDIF

      IF (amb1c_h_instrument .EQ. 12) THEN   !MHS
        wavenumbers(2) = 5.236956     !157 GHz
        wavenumbers(5) = 6.348092     !190.3 GHz
      ENDIF

!     compute BPlanck
      DO chn = 1, Bnchan
        wave_no = wavenumbers(chn)
        BPlanck(1,chn) = C_1 * wave_no * wave_no * wave_no
        BPlanck(2,chn) = C_2 * wave_no
        BPlanck(3,chn) = 0.0        !Ignore band corrections
        BPlanck(4,chn) = 1.0
      ENDDO

!     point to last header record in 1C
      B1Crecord = amb1c_h_hdrcnt

! Estimate space view angle and target temperature
      IF (amb1c_h_instrument .EQ. 12) THEN
        space_ang = 73.75 * deg2rad    !Assume MHS profile 0 but not critical
      ELSE
        space_ang = 75.0 * deg2rad     !Assume AMSU-B SV2 but not critical
      ENDIF
      targ_temp = 290.0  !Guess - not necessary to compute exactly

! 3.  Loop through scan lines, reading, processing & writing data
!     ---- ------- ---- ------ -------- ---------- - ------- ----

      DO scnlin = 1, amb1c_h_scnlin

! Read a scanline
         B1Crecord = B1Crecord + 1
         CALL IOB1C ("READ DATA", B1Cunit, B1Crecord )

! Check quality flag

         IF ( .NOT. BTEST( amb1c_qualind, bits(31) ) ! don't use
     &    .AND..NOT.BTEST( amb1c_qualind, bits(28) ) ! no calibration
     &     ) THEN

! Convert BTs to radiance

            DO fov = 1, Bnfovs          

! Compute antenna angle
              IF (amb1c_h_instrument .EQ. 12) THEN
                earth_ang = (fov-45.5)*10.0/9.0 * deg2rad   !MHS
              ELSE
                earth_ang = (fov-45.5)*1.1 * deg2rad        !AMSU-B
              ENDIF

              DO chn = 1, Bnchan
                IF (amb1c_btemps(chn,fov).GT.0) THEN
                  bri_temp = amb1c_btemps(chn,fov)*0.01
                  bri_temp = bri_temp*BPlanck(4,chn) + BPlanck(3,chn)
                  radiance = 
     &              BPlanck(1,chn)/(exp(BPlanck(2,chn)/bri_temp) - 1)

! Apply antenna corrections
!               We have values for the efficiency with which the antenna sees
!               the earth, the spacecraft, and space for each scan position.
!               Assume radiance from the spacecraft is the same as from the
!               earth (i.e. reflecting) and then adjust the antenna-observed
!               radiance to the value we expect from the earth alone.

                  IF (Mode.EQ.1) THEN
                    radiance=(radiance-Beff_Space(fov,chn)*Bspace(chn))
     &                     /(Beff_Earth(fov,chn) + Beff_Platf(fov,chn))
                  ELSE
                    radiance = radiance
     &                *(Beff_Earth(fov,chn)+Beff_Platf(fov,chn))
     &                + Beff_Space(fov,chn)*Bspace(chn)
                  ENDIF

! Convert radiance back to BT

                  bri_temp = BPlanck(2,chn)         ! Kelvin
     &                     / log( 1. + BPlanck(1,chn)/radiance )
                  bri_temp = (bri_temp-BPlanck(3,chn)) / BPlanck(4,chn)

! Apply (or remove) antenna reflectivity corrections if present
                  IF (reflectivity_diff(chn+15).NE.0.0) THEN
                    bri_temp = bri_temp + Mode*(
     &                reflectivity_diff(chn+15)*(targ_temp-bri_temp)*
     &                (cos(2*earth_ang) - cos(2*space_ang))/2.0)  
                  ENDIF

                  amb1c_btemps(chn,fov) = NINT ( bri_temp*100. )
                ENDIF
              ENDDO
            ENDDO

! Write out the scan line

            CALL IOB1C ("WRITE DATA", B1Cunit, B1Crecord )

         ENDIF   ! scan line OK
      ENDDO   ! (scnlin)

      WRITE(*,*) amb1c_h_scnlin," scanlines written to AMSU-B 1C file"
      WRITE(*,*)

! 4.  Update header and close files
!     ------ ------ --- ----- -----

      CALL IOB1C ("WRITE HEADER", B1Cunit, 1 )
      CALL IOB1C ("CLOSE", B1Cunit, B1Crecord )


      RETURN
      END
