!+  process AMSU-A scanline - level 1b to 1c
!
      SUBROUTINE INAPRC ( Write_data )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inaprc.F 61 2006-11-27 13:43:51Z frna $
!
! Purpose:
!   Convert counts to radiances to brightness temperatures
!   and perform some quality control.
!
! Method:
! 1.  Check scanline number & time of scanline
! 2.  Act on flags for the scanline from the 1B file
! 3.  Loop through scanline and channels, converting counts to BTs, with qc
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0    12/12/96   Original code. (R.J.Renshaw)
!  1.1    18/12/96   bug fixed: ama1c_dataqual now ama1c_dataqual(fov)
!  1.2    12/06/98   Acycle REAL not INTEGER
!  1.3    22/06/98   Added antenna correction code
!  1.4    08/01/01   Ensure correct adjustment to scanline number.
!  1.5    13/06/01   Set flag if moon contamination is present (N.C.Atkinson)
!  1.6    08/12/03   Time gap error not fatal if last line (N.C.Atkinson)
!  1.7    04/04/05   Set BTs to missing if chanqual flag bits 4 or 5 set (NCA)
!  1.8    21/11/06   Ignore scans with bad times but do not abort (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  AMSU-A level 1B header/data common blocks 
#include "ama1b.h"
!
! Subroutine arguments:
!
!   Scalar arguments with intent(out)
      LOGICAL       Write_data    ! .TRUE. if line is to be written out
!
! Local parameters:
      INTEGER       mx_dTime      ! upper limit for departure of scan time
         PARAMETER (mx_dTime=50)  ! from nominal value  (milliseconds)
!
! Local Scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       cal_set       ! use 1=primary or 2=secondary cal coeffs
      INTEGER       dline         ! jump in line number since last call
      INTEGER       dTime         ! time difference returned from s/r TimeSub
      INTEGER       errTime       ! error status returned from s/r TimeSub
      INTEGER       fov           ! loop variable over all fovs
      REAL          counts        ! averaged PRT counts from the instrument
      REAL          radiance      ! radiance calculated from counts
      REAL          bri_temp      ! brightness temperature calc from radiance
      LOGICAL       good_bts      ! .FALSE. if all channel BTs for the fov bad
      LOGICAL       good_data     ! .TRUE. if line has brightness temperatures
!
      INTEGER       prev_line     ! line number of previous scanline
         DATA       prev_line / MDI /
      INTEGER       prev_year     ! year of previous scanline
         DATA       prev_year / 0 /
      INTEGER       prev_day      ! day of previous scanline
         DATA       prev_day / 0 /
      INTEGER       prev_time     ! time of previous scanline
         DATA       prev_time / 0 /
      INTEGER       line_adjust   ! adjustment to line number to agree with
         DATA       line_adjust / 0 /                       ! scanline times
      INTEGER       chanqualprev(Anchan)  !Chan quality flag for previous line
         DATA       chanqualprev / 15*0 /
!
! Cosmic background radiance for each channel, calculated assuming 2.73 Kelvin
      REAL Bspace(Anchan)
      DATA Bspace /
     & 1.14709E-05, 1.85760E-05, 3.95843E-05, 4.25294E-05, 4.34692E-05,
     & 4.44188E-05, 4.50567E-05, 4.57183E-05, 4.78323E-05, 4.78323E-05,
     & 4.78323E-05, 4.78323E-05, 4.78323E-05, 4.78323E-05, 8.24266E-05 /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeSub              ! subtract two dates/times
!
      SAVE prev_line, prev_year, prev_day, prev_time, line_adjust
!
!- End of header


! 1.  Check scanline number & time of scanline
!     ----- -------- ------ - ---- -- --------

! 1.1 Innocent until proven guilty..
      good_data  = .TRUE.   ! good brightness temperatures for this line
      Write_data = .TRUE.   ! do process & write out this scanline

! 1.2 Adjust scanline number to be consistent with scanline times
      ama1c_scnlin = ama1c_scnlin + line_adjust

! 1.3 Calculate the scanline number from the date/time (to check)
      IF (prev_line.NE.MDI) THEN   ! not the 1st record read

         CALL TimeSub
     &        ( ama1c_scnlinyr, ama1c_scnlindy, ama1c_scnlintime,
     &          prev_year, prev_day, prev_time, dTime, errTime )
         errMessage = " "
         WRITE ( errMessage,
     &   "(""Scanline date/time in error"",3I9)")
     &   ama1c_scnlinyr, ama1c_scnlindy, ama1c_scnlintime
         CALL ErrorReport(errTime,errMessage,"INAPRC")

!        calculate how many scanlines this is since the last
         dline = NINT(REAL(dTime)/Acycle)

!        Ignore scan if date/time not incrementing by a multiple of Acycle
         IF ( ABS(dTime-NINT(REAL(dline)*Acycle)).GT.mx_dTime ) THEN
            errMessage = " "
            WRITE ( errMessage,
     &      "(""Time gaps between scanlines are wrong, lines"",3I9)")
     &      prev_line-line_adjust, ama1c_scnlin-line_adjust, dline
            errStatus = -131 ! warning
            Write_data = .FALSE.
            ama1c_scnlintime = prev_time  !keep the previous time
            ama1c_scnlin = prev_line      !  and line number
            dline = 0
            CALL ErrorReport(errStatus,errMessage,"INAPRC")
         ENDIF

!        Complain if scanline number and estimated scanline number differ
         IF ( ama1c_scnlin.NE.(prev_line+dline) ) THEN
            errMessage = " "
            WRITE ( errMessage,"(""Times for scanlines "",I5,
     &      "" and "",I5,"" differ by "",I3,"" scanline cycles"")")
     &      prev_line-line_adjust, ama1c_scnlin-line_adjust, dline
            errStatus = -132  ! warning
            CALL ErrorReport(errStatus,errMessage,"INAPRC")
!         and reset scanline & adjustment to what we think they should be
!            line_adjust = line_adjust + (dline-1)
!            ama1c_scnlin = ama1c_scnlin + (dline-1)
            line_adjust = line_adjust + prev_line + dline - ama1c_scnlin
            ama1c_scnlin = prev_line + dline
         ENDIF

      ENDIF  ! not the 1st record read

! 1.4 save values for next scanline
      prev_line = ama1c_scnlin
      prev_year = ama1c_scnlinyr
      prev_day = ama1c_scnlindy
      prev_time = ama1c_scnlintime


! 2.  Act on flags for the scanline from the 1B file
!     --- -- ----- --- --- -------- ---- --- -- ----

! 2.1 Set choice of calibration coefficients from "scanline quality flags"

      IF ( BTEST( ama1c_scanqual, bits(15) ) ! not calibrated, bad time
     & .OR.BTEST( ama1c_scanqual, bits(13) ) ! not calibrated, bad PRT
     & .OR.BTEST( ama1c_scanqual, bits(10) ) ! uncalibrated, instrument mode
     & ) THEN

         cal_set = 2 ! use secondary set of calibration coefficients
         DO fov = 1, Anfovs
            ama1c_dataqual(fov) = IBSET( ama1c_dataqual(fov), bits(30) )
         ENDDO

      ELSE

         cal_set = 1 ! use primary set of calibration coefficients

      ENDIF

! 2.2 "quality indicator": Cannot use any information from these fovs

      IF ( BTEST( ama1c_qualind, bits(30) ) ! time sequence error
     & .OR.BTEST( ama1c_qualind, bits(27) ) ! no earth location
     & ) THEN

         good_data  = .FALSE.  ! no brightness temperatures for this line
         Write_data = .FALSE.  ! don't write out this line

      ENDIF

! 2.3 "quality indicator": No BTs but can still map data to these fovs

      IF ( BTEST( ama1c_qualind, bits(31) ) ! don't use for product generation
     & .OR.BTEST( ama1c_qualind, bits(28) ) ! no calibration
     &  ) THEN

         good_data  = .FALSE.  ! no brightness temperatures for this line

!        Set BTs to missing and flag them
         DO fov = 1, Anfovs
            DO chn = 1, Anchan
               ama1c_btemps(chn,fov) = MDI
            ENDDO  ! chn
            ama1c_dataqual(fov) = IBSET(ama1c_dataqual(fov),bits(0))
         ENDDO  ! fov

      ENDIF


! 3.  Loop through scanline and channels, converting counts to BTs, with qc
!     ---- ------- -------- --- --------- ---------- ------ -- ---- ---- --

      IF ( good_data .AND. Write_data ) THEN
         DO fov = 1, Anfovs

!           Loop through the channels
            good_bts = .FALSE.
            DO chn = 1, Anchan

! 3.1          Convert counts to radiance (W/m2)
               counts = REAL(Acounts(chn,fov))
               radiance = -999.
               IF (ABS(counts).LT.99999.) THEN       ! (check data)
                  radiance = Acalib(cal_set,3,chn)
     &                     + Acalib(cal_set,2,chn)*counts
     &                     + Acalib(cal_set,1,chn)*counts*counts
               ENDIF  ! counts

! 3.2          Apply antenna corrections
!              We have values for the efficiency with which the antenna sees
!              the earth, the spacecraft, and space for each scan position.
!              Assume radiance from the spacecraft is the same as from the
!              earth (i.e. reflecting) and then adjust the antenna-observed
!              radiance to the value we expect from the earth alone.
               radiance = ( radiance - Aeff_Space(fov,chn)*Bspace(chn) )
     &                   /( Aeff_Earth(fov,chn) + Aeff_Platf(fov,chn) )

! 3.3          Convert radiance to brightness temperature (inverse Planck)
!              and modify to allow for bandwidth of channels
               bri_temp = -999.
               IF (radiance.GT.0.) THEN             ! (check data)
                  bri_temp = APlanck(2,chn)         ! Kelvin
     &                     / log( 1. + APlanck(1,chn)/radiance )
                  bri_temp = (bri_temp-APlanck(3,chn)) / APlanck(4,chn)
               ENDIF  ! radiance

! 3.4          Look at the channel quality flags. If bits 4 or 5 are set (no
!              good space/target counts), either for this line or the preceding
!              line, then the BTs are suspect and should be ignored.
               IF (IAND(ama1c_chanqual(chn),48).NE.0.OR.
     &             IAND(chanqualprev(chn),48).NE.0) bri_temp = -999.     

! 3.5          Quality control final brightness temperature
               IF ( bri_temp.GT.AminBT .AND. bri_temp.LT.AmaxBT ) THEN
                  ama1c_btemps(chn,fov) = NINT ( bri_temp*100. )  ! 100*deg K
                 good_bts = .TRUE.
               ELSE
!                 Set BT to missing, and flag it
                  ama1c_btemps(chn,fov) = MDI
                  ama1c_dataqual(fov) =
     &                           IBSET(ama1c_dataqual(fov),bits(chn))
               ENDIF  ! bri_temp

! 3.6          Look at moon flag and reject contaminated data. Retain the
!              brightness temperatures for non-critical applications.
               IF ((chn.LE.2 .AND. MFlag.GE.2).OR.            !Moon in AMSU-A2
     &             (chn.GT.2 .AND. MOD(MFlag,2).EQ.1)) THEN   !Moon in AMSU-A1
                  ama1c_dataqual(fov) =
     &                           IBSET(ama1c_dataqual(fov),bits(chn))
               ENDIF

            ENDDO  ! chn

!           If all BTs are bad, set bit0 of data quality flagword
            IF ( .NOT. good_bts ) ama1c_dataqual(fov) =
     &                            IBSET(ama1c_dataqual(fov),bits(0))

         ENDDO  ! fov
      ENDIF  ! good_data .AND. Write_data

      DO chn = 1, Anchan
        chanqualprev(chn) = ama1c_chanqual(chn)  !Update for next line
      ENDDO

      RETURN
      END
