!+ process NEDT for AMSU-A
!
      SUBROUTINE INAPRC_NEDT
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inaprc_nedt.F 459 2015-09-11 14:55:32Z frna $
!
! Purpose: Compute NEDT for each scan line. Based on EUMETSAT's method.
!
! Reference: EUM/OPS/TEN/11/3488, 2011, "Technical note for design ECP 299"
!
! Method: The NEDT used here is the standard deviation of 7 scan lines
!         of cold and warm counts, scaled by the gain.
!
! Current Code Owner: N.C.Atkinson
!
! History:
! Version  Date      Comment
!
!  1.0  10/09/2015  Initial. Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-A level 1B header/data common blocks 
#include "ama1b.h"
!  Constants 
#include "constants_preproc.h"
!
! Local scalars:
      INTEGER iscan, jscan, ichan, nsample
      LOGICAL okscan(7)
      INTEGER nscan(Anchan)

      REAL coldcal_mean, coldcal_sd
      REAL warmcal_mean, warmcal_sd
      REAL a0, a1, a2
      REAL drdc, dcdt, wave_no
      REAL drdt(Anchan)   !Slope of Planck function
!
!- End of header

      DO ichan = 1,Anchan
        wave_no = ama_h_temrad(ichan*3-2) * 1.E-06
        drdt(ichan) = C_1/C_2 * wave_no * wave_no
        AcoldNEDT_rms(ichan) = 0.0
        AwarmNEDT_rms(ichan) = 0.0
        nscan(ichan) = 0
      ENDDO

!Loop over scans and channels

      DO iscan=1,ama_h_scnlin
        DO ichan = 1,Anchan
          DO jscan=-3,3
            okscan(jscan+4) = .false.
          ENDDO

!Check which nearby scans are usable for this channel

          nsample = 0
          DO jscan=-3,3
            IF (iscan+jscan.GT.0.AND.iscan+jscan.LE.ama_h_scnlin) THEN
              IF (Acoldcals(1,ichan,iscan+jscan).NE.0 .AND.
     &            Acoldcals(2,ichan,iscan+jscan).NE.0 .AND.
     &            Awarmcals(1,ichan,iscan+jscan).NE.0 .AND.
     &            Awarmcals(2,ichan,iscan+jscan).NE.0 .AND.
     &            Acalibs(2,ichan,iscan+jscan).GT.0 ) THEN
                okscan(jscan+4) = .true.
                nsample = nsample + 2
              ENDIF
            ENDIF
          ENDDO           
        
!Get mean counts

          IF (nsample .gt. 0) THEN
            coldcal_mean = 0.0
            warmcal_mean = 0.0
            DO jscan=-3,3
              IF (okscan(jscan+4)) THEN
                coldcal_mean = coldcal_mean +              
     &            Acoldcals(1,ichan,iscan+jscan) +
     &            Acoldcals(2,ichan,iscan+jscan)
                warmcal_mean = warmcal_mean +              
     &            Awarmcals(1,ichan,iscan+jscan) +
     &            Awarmcals(2,ichan,iscan+jscan)
              ENDIF
            ENDDO
            coldcal_mean = coldcal_mean/nsample                
            warmcal_mean = warmcal_mean/nsample                

!Get std dev of counts

            coldcal_sd = 0.0
            warmcal_sd = 0.0
            DO jscan=-3,3
              IF (okscan(jscan+4)) THEN
                coldcal_sd = coldcal_sd +              
     &            (Acoldcals(1,ichan,iscan+jscan)-coldcal_mean)**2 +
     &            (Acoldcals(2,ichan,iscan+jscan)-coldcal_mean)**2
                warmcal_sd = warmcal_sd +              
     &            (Awarmcals(1,ichan,iscan+jscan)-warmcal_mean)**2 +
     &            (Awarmcals(2,ichan,iscan+jscan)-warmcal_mean)**2
              ENDIF
            ENDDO
            coldcal_sd = SQRT(coldcal_sd/(nsample-1))                
            warmcal_sd = SQRT(warmcal_sd/(nsample-1))                

!Get gain from the a0, a1, a2 coefs

            a0 = Acalibs(3,ichan,iscan)
            a1 = Acalibs(2,ichan,iscan)
            a2 = Acalibs(1,ichan,iscan)
            IF (a1 .GT. 0) THEN
              drdc = a1 - 2*a2*a0/a1       !slope at zero K (good approximation)
              dcdt = drdt(ichan)/drdc      !gain in counts per K

!Get NEDT

              AcoldNEDT(ichan,iscan) = coldcal_sd/dcdt                 
              AwarmNEDT(ichan,iscan) = warmcal_sd/dcdt
              AcoldNEDT_rms(ichan) = AcoldNEDT_rms(ichan) + 
     &             AcoldNEDT(ichan,iscan)**2
              AwarmNEDT_rms(ichan) = AwarmNEDT_rms(ichan) + 
     &             AwarmNEDT(ichan,iscan)**2
              nscan(ichan) = nscan(ichan) + 1
            ELSE
              AcoldNEDT(ichan,iscan) = -1.0                 
              AwarmNEDT(ichan,iscan) = -1.0
            ENDIF
          ELSE
            AcoldNEDT(ichan,iscan) = -1.0    !if no good samples
            AwarmNEDT(ichan,iscan) = -1.0
          ENDIF 

        ENDDO   ! channels
      ENDDO   ! scans

      DO ichan=1,Anchan
        IF (nscan(ichan) .GT. 0) THEN
          AcoldNEDT_rms(ichan) = SQRT(AcoldNEDT_rms(ichan)/nscan(ichan))
          AwarmNEDT_rms(ichan) = SQRT(AwarmNEDT_rms(ichan)/nscan(ichan))
        ELSE
          AcoldNEDT_rms(ichan) = -1.0
          AwarmNEDT_rms(ichan) = -1.0
        ENDIF
      ENDDO
      END
