!+ Set up 1C AMSU-B scanline from 1B scanline within ATOVIN
!
      SUBROUTINE INBGET
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inbget.F 635 2019-04-15 15:08:00Z frna $
!
! Purpose:
!   Place information from common blocks /amb1bdt/ & /inbstore/
!   into common block /amb1cdt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1B & 1C include files (amb1b.h & amb1c.h) are
!     compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Extract required information from 1B into 1C
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0    12/12/96  Original code. (R.J.Renshaw)
!  1.1    17/01/97  lat/lon now 10^4
!  1.2    10/03/98  updated for 1c vn1.9 (instrument temp and solar angles)
!  1.3    20/05/98  updated for latest 1b include files
!                   (lat/lon * 10^4 and no version numbers)
!  1.4    18/06/98  bug fixed in converting azimuth angle to [0,360]
!  1.5    23/06/98  Add clock drift if not already done.
!  1.6    21/12/98  Allow for AMSU-B counts values > 32767 (N.C.Atkinson)
!  1.7    23/02/05  Convert counts to K*100 for amb1c_instrtemp.
!  1.8    18/09/15  Allow for difference between NESDIS and AAPP azimuth angles.
!  1.9    09/04/19  Portability fix.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-B Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  AMSU-B level 1B header/data common blocks 
#include "amb1b.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!  Earth model, used to identify non-AAPP geolocation
#include "p_earth.h"
!
! Local parameters:
      INTEGER       B1Bvnnb       ! version number of 1B format compatible
         PARAMETER (B1Bvnnb=1)    ! with the code in this subroutine
      INTEGER       B1Cvnnb       ! version number of 1C format compatible
         PARAMETER (B1Cvnnb=190)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chn3          ! = chn*3
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       i             ! general purpose loop variable
      INTEGER       angle         ! satellite azimuth angle (deg*100)
      INTEGER       year          ! scan line year
      INTEGER       day           ! scan line day of year
      INTEGER       time          ! scan line UTC time of day in milliseconds
      INTEGER       clockdrift    ! satellite clock drift delta in milliseconds
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
      REAL          rcount        ! raw count value as real number
!
! Local arrays:
      REAL          latitudes(Bnfovs)   ! latitudes for one scanline
      REAL          longitudes(Bnfovs)  ! longitudes for one scanline
      REAL          azimuths(Bnfovs)    ! solar azimuth angles for one scanline
      REAL          zeniths(Bnfovs)     ! solar zenith angles for one scanline
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeAdd              ! add an increment to a date/time
      External INSUNA               ! calculate solar zenith & azimuth angles
!
!- End of header

      SAVE first                    ! .TRUE. on first call

! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!!        Is the 1B include file compatible with this source code ?
!         IF ( B1Bvnnb .NE. amb_1bvnnb ) THEN
!            errStatus = -11  ! warning
!            errMessage = " "
!            WRITE ( errMessage,
!     &      "("" Code expects 1B format"",I6,"" but amb1b.h is "",I6)")
!     &           B1Bvnnb, amb_1bvnnb
!            CALL ErrorReport ( errStatus, errMessage, "INBGET")
!         ENDIF

!        Is the 1C include file compatible with this source code ?
         IF ( B1Cvnnb .NE. amb_1cvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but amb1c.h is "",I6)")
     &           B1Cvnnb, amb_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "INBGET")
         ENDIF

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Extract required information from 1B into 1C
!     ------- -------- ----------- ---- -- ---- --

! 1.1 scanline number
      amb1c_scnlin = amb_scnlin

! 1.2 scanline date/time    ( - assume already corrected for clock drift)
      amb1c_scnlinyr = amb_scnlinyr        ! scan line year
      amb1c_scnlindy = amb_scnlindy        ! scan line day of year
      amb1c_scnlintime = amb_scnlintime    ! UTC time of day in milliseconds

      year = amb1c_scnlinyr                ! scan line year
      day  = amb1c_scnlindy                ! scan line day of year
      time = amb1c_scnlintime              ! UTC time of day in milliseconds

!      - adding clock drift delta if not already done so
      IF ( .NOT.BTEST(INT(amb_scnlinbit),bits(14)) ) THEN  ! add clock drift delta
         clockdrift = amb_clockdrift       ! (msecs)    (amb_... is I2)
         IF ( clockdrift .LT. 0 ) clockdrift = clockdrift + 65536
         CALL TimeAdd ( year, day, time, clockdrift,
     &        amb1c_scnlinyr, amb1c_scnlindy, amb1c_scnlintime)
      ENDIF

! 1.3 other scanline information
      amb1c_qualind = amb_qualind          !quality indicator bit field
      amb1c_scanqual = amb_linqualflgs     !scan line quality flags
!     Quality control word for each channel
      DO chn = 1, Bnchan
         amb1c_chanqual(chn) = amb_chqualflg(chn)
      ENDDO  ! chn

!     Instrument temperature in K*100
      rcount = amb_temps(3)                !AMSU-B Mixer chan 18-20 temp
      amb1c_instrtemp = 100*(amb_h_digacoef(9)*1.0E-2   + 
     &               rcount*(amb_h_digacoef(10)*1.0E-7  + 
     &               rcount*(amb_h_digacoef(11)*1.0E-12 + 
     &               rcount* amb_h_digacoef(12)*1.0E-18))) 

      DO i = 1, 2
         amb1c_spare1(i) = 0               !spare
      ENDDO  ! i

! 1.4 Navigation

      amb1c_scalti = amb_scalti  ! sat altitude above ref. ellipsoid (km*10)

      DO fov = 1, Bnfovs
         amb1c_latlon(1,fov) = amb_pos(fov*2-1) ! latitude
         amb1c_latlon(2,fov) = amb_pos(fov*2)   ! longitude
         latitudes(fov)  = REAL(amb1c_latlon(1,fov))*1.E-4
         longitudes(fov) = REAL(amb1c_latlon(2,fov))*1.E-4
      ENDDO  ! fov

!     angles
      DO fov = 1, Bnfovs
         amb1c_angles(1,fov) = amb_ang(3*fov-1) ! zenith
         angle = amb_ang(3*fov)   ! azimuth
!        make sure azimuth between 0-360, rather than +/-180
         IF ( angle .LT. 0 ) angle = 36000 + angle
         amb1c_angles(2,fov) = angle
      ENDDO  ! fov

!     Calculate solar zenith/azimuth angles
      CALL INSUNA ( year, day, time, Bnfovs, latitudes, longitudes,
     &              azimuths, zeniths )
      DO fov = 1, Bnfovs
         amb1c_angles(3,fov) = NINT(zeniths(fov)*100.)  ! solar zenith
         amb1c_angles(4,fov) = NINT(azimuths(fov)*100.) ! solar azimuth
      ENDDO  ! fov

!     NESDIS 1B uses a "relative" azimuth, defined as satellite minus solar
!     angle. In AAPP it's the satellite angle. Allow for this difference.
!     Use site_id and modelid to identify NESDIS geolocation.

      IF (amb_h_siteid.EQ.'NSS'.AND.amb_h_modelid.NE.earth_modelid) THEN
        DO fov = 1, Bnfovs
          angle = amb1c_angles(2,fov) + amb1c_angles(4,fov)
          IF (angle .GT. 36000) angle = angle - 36000
          amb1c_angles(2,fov) = angle
        ENDDO
      ENDIF

! 1.5 Initialise variables new in 1C
      DO i = 1, 2
         amb1c_spare2(i) = 0               !spare
      ENDDO  ! i
      DO chn = 1, Bnchan
      DO fov = 1, Bnfovs
         amb1c_btemps(chn,fov) = MDI       !brightness temperatures
      ENDDO  ! fov
      ENDDO  ! chn
      DO fov = 1, Bnfovs
         amb1c_dataqual(fov) = 0           !qc word for each field of view
      ENDDO  ! fov


! 2.  Extract other information required here
!     ------- ----- ----------- -------- ----

! 2.1 Calibration coefficients (primary & secondary)
      DO chn = 1, Bnchan
         chn3 = chn*3
         Bcalib(1,1,chn) = REAL(amb_pmcal(chn3-2)) * 1.E-16  ! primary
         Bcalib(1,2,chn) = REAL(amb_pmcal(chn3-1)) * 1.E-10
         Bcalib(1,3,chn) = REAL(amb_pmcal(chn3))   * 1.E-06
         Bcalib(2,1,chn) = REAL(amb_sdcal(chn3-2)) * 1.E-16  ! secondary
         Bcalib(2,2,chn) = REAL(amb_sdcal(chn3-1)) * 1.E-10
         Bcalib(2,3,chn) = REAL(amb_sdcal(chn3))   * 1.E-06
      ENDDO  ! chn

! 2.2 Counts from the instrument

      DO fov = 1, Bnfovs
         chn3 = fov*6 - 5
         DO chn = 1, Bnchan
            Bcounts(chn,fov) = mod(amb_earth_data(chn+chn3)+65536,65536)
         ENDDO  ! chn
      ENDDO  ! fov


      DO i = 1, 55-2*Bnchan
         amb1c_filler(i) = 0      !filler
      ENDDO


      RETURN
      END
