!+  process AMSU-B scanline - level 1b to 1c
!
      SUBROUTINE INBPRC ( Write_data )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inbprc.F 180 2010-01-08 12:04:35Z frna $
!
! Purpose:
!   Convert counts to radiances to brightness temperatures
!   and perform some quality control.
!
! Method:
! 1.  Check scanline number & time of scanline
! 2.  Act on flags for the scanline from the 1B file
! 3.  Loop through scanline and channels, converting counts to BTs, with qc
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0    12/12/96   Original code. (R.J.Renshaw)
!  1.1    18/12/96   bug fixed: amb1c_dataqual now amb1c_dataqual(fov)
!  1.2    12/06/98   Bcycle REAL not INTEGER
!  1.3    18/06/98   Added antenna correction code
!  1.4    20/01/99   Set 'missing' flag when data is flagged for
!                    interference bias change  - data is unreliable. RJR
!  1.5    22/02/99   Set 'missing' flag whenever STX-3 transmitter is on. RJR
!  1.6    26/10/00   Check for STX-3 transmitter made specific to NOAA-15. KWW
!  1.7    08/01/01   Ensure correct adjustment to scanline number.
!  1.8    25/11/04   Add MHS; add antenna reflectivity correction if present; 
!                    use constants_preproc.h. N C Atkinson
!  1.9    21/11/06   Ignore scans with bad times but do not abort (NCA)
!  1.10   04/01/10   Ignore interference bias change flag for NOAA-17 (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-B Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  AMSU-B level 1B header/data common blocks 
#include "amb1b.h"
!  MHS level 1B header/data common blocks 
#include "mhs1b.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!  Fundamental and physical constants 
#include "constants_preproc.h"
!
! Subroutine arguments:
!
!   Scalar arguments with intent(out)
      LOGICAL       Write_data    ! .TRUE. if line is to be written out
!
! Local parameters:
      INTEGER       mx_dTime      ! upper limit for departure of scan time
         PARAMETER (mx_dTime=50)  ! from nominal value  (milliseconds)
!
! Local Scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       cal_set       ! use 1=primary or 2=secondary cal coeffs
      INTEGER       dline         ! jump in line number since last call
      INTEGER       dTime         ! time difference returned from s/r TimeSub
      INTEGER       errTime       ! error status returned from s/r TimeSub
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       i             ! loop variable
      INTEGER       isat          ! index to NESDISid and NOAAid arrays
      REAL          counts        ! averaged PRT counts from the instrument
      REAL          radiance      ! radiance calculated from counts
      REAL          bri_temp      ! brightness temperature calc from radiance
      REAL          targ_temp     ! Internal target temperature
      REAL          earth_ang     ! Earth-view antenna angle
      REAL          space_ang     ! Space-view antenna angle
      LOGICAL       good_bts      ! .FALSE. if all channel BTs for the fov bad
      LOGICAL       good_data     ! .TRUE. if line has brightness temperatures
!
      INTEGER       prev_line     ! line number of previous scanline
         DATA       prev_line / MDI /
      INTEGER       prev_year     ! year of previous scanline
         DATA       prev_year / 0 /
      INTEGER       prev_day      ! day of previous scanline
         DATA       prev_day / 0 /
      INTEGER       prev_time     ! time of previous scanline
         DATA       prev_time / 0 /
      INTEGER       line_adjust   ! adjustment to line number to agree with
         DATA       line_adjust / 0 /                       ! scanline times
!
! Cosmic background radiance for each channel, calculated assuming 2.73 Kelvin
      REAL Bspace(Bnchan)
      DATA Bspace /
     & 8.24175E-05, 1.15024E-04, 1.13024E-04, 1.13024E-04, 1.13024E-04 /
!
! Antenna reflectivities from fdf.dat
      REAL          reflectivity_diff(20)  ! AMSU antenna reflectivity diffs
      COMMON /amsu_reflectivity/ reflectivity_diff
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeSub              ! subtract two dates/times
!
      SAVE prev_line, prev_year, prev_day, prev_time, line_adjust
!
!- End of header


! 1.  Check scanline number & time of scanline
!     ----- -------- ------ - ---- -- --------

! 1.1 Innocent until proven guilty..
      good_data  = .TRUE.   ! good brightness temperatures for this line
      Write_data = .TRUE.   ! do process & write out this scanline

! 1.2 Adjust scanline number to be consistent with scanline times
      amb1c_scnlin = amb1c_scnlin + line_adjust

! 1.3 Calculate the scanline number from the date/time (to check)
      IF (prev_line.NE.MDI) THEN   ! not the 1st record read

         CALL TimeSub
     &        ( amb1c_scnlinyr, amb1c_scnlindy, amb1c_scnlintime,
     &          prev_year, prev_day, prev_time, dTime, errTime )
         errMessage = " "
         WRITE ( errMessage,
     &   "(""Scanline date/time in error"",3I9)")
     &   amb1c_scnlinyr, amb1c_scnlindy, amb1c_scnlintime
         CALL ErrorReport(errTime,errMessage,"INBPRC")

!        calculate how many scanlines this is since the last
         dline = NINT(REAL(dTime)/Bcycle)

!        Ignore scan if date/time not incrementing by a multiple of Acycle
         IF ( ABS(dTime-NINT(REAL(dline)*Bcycle)).GT.mx_dTime ) THEN
            errMessage = " "
            WRITE ( errMessage,
     &      "(""Time gaps between scanlines are wrong, lines"",3I9)")
     &      prev_line-line_adjust, amb1c_scnlin-line_adjust, dline
            errStatus = -131 ! warning
            Write_data = .FALSE.
            amb1c_scnlintime = prev_time  !keep the previous time
            amb1c_scnlin = prev_line      !  and line number
            dline = 0
            CALL ErrorReport(errStatus,errMessage,"INBPRC")
         ENDIF

!        Complain if scanline number and estimated scanline number differ
         IF ( amb1c_scnlin.NE.(prev_line+dline) ) THEN
            errMessage = " "
            WRITE ( errMessage,"(""Times for scanlines "",I5,
     &      "" and "",I5,"" differ by "",I3,"" scanline cycles"")")
     &      prev_line-line_adjust, amb1c_scnlin-line_adjust, dline
            errStatus = -132  ! warning
            CALL ErrorReport(errStatus,errMessage,"INBPRC")
!         and reset scanline & adjustment to what we think they should be
!            line_adjust = line_adjust + (dline-1)
!            amb1c_scnlin = amb1c_scnlin + (dline-1)
            line_adjust = line_adjust + prev_line + dline - amb1c_scnlin
            amb1c_scnlin = prev_line + dline
         ENDIF

      ENDIF  ! not the 1st record read

! 1.4 save values for next scanline
      prev_line = amb1c_scnlin
      prev_year = amb1c_scnlinyr
      prev_day = amb1c_scnlindy
      prev_time = amb1c_scnlintime


! 2.  Act on flags for the scanline from the 1B file
!     --- -- ----- --- --- -------- ---- --- -- ----

! 2.1 Set choice of calibration coefficients from "scanline quality flags"

      IF ( BTEST( amb1c_scanqual, bits(15) ) ! not calibrated, bad time
     & .OR.BTEST( amb1c_scanqual, bits(13) ) ! not calibrated, bad PRT
     & .OR.BTEST( amb1c_scanqual, bits(10) ) ! uncalibrated, instrument mode
     & ) THEN

         cal_set = 2 ! use secondary set of calibration coefficients
         DO fov = 1, Bnfovs
            amb1c_dataqual(fov) = IBSET( amb1c_dataqual(fov), bits(30) )
         ENDDO

      ELSE

         cal_set = 1 ! use primary set of calibration coefficients

      ENDIF

! 2.2 "quality indicator": Cannot use any information from these fovs

      IF ( BTEST( amb1c_qualind, bits(30) ) ! time sequence error
     & .OR.BTEST( amb1c_qualind, bits(27) ) ! no earth location
     & ) THEN

         good_data  = .FALSE.  ! no brightness temperatures for this line
         Write_data = .FALSE.  ! don't write out this line

      ENDIF

! 2.3 "quality indicator": No BTs but can still map data to these fovs

!     First find satellite index for use with STX-3 test
      isat = 3            ! default NOAA-15
      DO i = 1,MaxSats
         IF ( NESDISid(i) .EQ. amb_h_satid )  isat=i
      ENDDO

      IF ( BTEST( amb1c_qualind, bits(31) ) ! don't use for product generation
     & .OR.BTEST( amb1c_qualind, bits(28) ) ! no calibration
     & .OR. (BTEST( amb1c_qualind, bits(6))
     &   .AND. NOAAid(isat) .NE. 17 )       ! interference bias changeover
     & .OR. ( amb_ahk(21) .EQ. 255
     &   .AND. NOAAid(isat) .EQ. 15 )) THEN ! STX-3 transmitter on for NOAA-15

         good_data  = .FALSE.  ! no brightness temperatures for this line

!        Set BTs to missing and flag them
         DO fov = 1, Bnfovs
            DO chn = 1, Bnchan
               amb1c_btemps(chn,fov) = MDI
            ENDDO  ! chn
            amb1c_dataqual(fov) = IBSET(amb1c_dataqual(fov),bits(0))
         ENDDO  ! fov

      ENDIF


! 3.  Loop through scanline and channels, converting counts to BTs, with qc
!     ---- ------- -------- --- --------- ---------- ------ -- ---- ---- --

      IF ( good_data .AND. Write_data ) THEN

! Get or estimate space view angle and target temperature
         IF (amb1c_h_instrument .EQ. 12) THEN      !MHS
           space_ang = 73.75 * deg2rad      !Assume profile 0 but not critical
           targ_temp = mhs_obcttemp(1) * 0.001
           IF (targ_temp.LT.260.OR.targ_temp.GT.320) targ_temp = 290.0 !Default
         ELSE                        !AMSU-B
           space_ang = 75.0 * deg2rad    !Assume SV2 but not critical
           targ_temp = 290.0         !Guess - not necessary to compute exactly
         ENDIF

         DO fov = 1, Bnfovs

! Compute antenna angle
            IF (amb1c_h_instrument .EQ. 12) THEN
              earth_ang = (fov-45.5)*10.0/9.0 * deg2rad   !MHS
            ELSE
              earth_ang = (fov-45.5)*1.1 * deg2rad        !AMSU-B
            ENDIF

!           Loop through the channels
            good_bts = .FALSE.
            DO chn = 1, Bnchan

! 3.1          Convert counts to radiance (W/m2)
               counts = REAL(Bcounts(chn,fov))
               radiance = -999.
               IF (ABS(counts).LT.99999.) THEN       ! (check data)
                  radiance = Bcalib(cal_set,3,chn)
     &                     + Bcalib(cal_set,2,chn)*counts
     &                     + Bcalib(cal_set,1,chn)*counts*counts
               ENDIF  ! counts

! 3.2          Apply antenna corrections
!              We have values for the efficiency with which the antenna sees
!              the earth, the spacecraft, and space for each scan position.
!              Assume radiance from the spacecraft is the same as from the
!              earth (i.e. reflecting) and then adjust the antenna-observed
!              radiance to the value we expect from the earth alone.
               radiance = ( radiance - Beff_Space(fov,chn)*Bspace(chn) )
     &                   /( Beff_Earth(fov,chn) + Beff_Platf(fov,chn) )

! 3.3          Convert radiance to brightness temperature (inverse Planck)
!              and modify to allow for bandwidth of channels

               bri_temp = -999.
               IF (radiance.GT.0.) THEN             ! (check data)
                  bri_temp = BPlanck(2,chn)         ! Kelvin
     &                     / log( 1. + BPlanck(1,chn)/radiance )
                  bri_temp = (bri_temp-BPlanck(3,chn)) / BPlanck(4,chn)

! 3.4          Apply antenna reflectivity corrections if present
                  IF (reflectivity_diff(chn+15).NE.0.0) THEN
                    bri_temp = bri_temp + 
     &                reflectivity_diff(chn+15)*(targ_temp-bri_temp)*
     &                (cos(2*earth_ang) - cos(2*space_ang))/2.0  
                  ENDIF

               ENDIF  ! radiance

! 3.5          Quality control final brightness temperature
               IF ( bri_temp.GT.BminBT .AND. bri_temp.LT.BmaxBT ) THEN
                  amb1c_btemps(chn,fov) = NINT ( bri_temp*100. )  ! 100*deg K
                  good_bts = .TRUE.
               ELSE
!                 Set BT to missing, and flag it
                  amb1c_btemps(chn,fov) = MDI
                  amb1c_dataqual(fov) =
     &                           IBSET(amb1c_dataqual(fov),bits(chn))
               ENDIF  ! bri_temp

            ENDDO  ! chn

!           If all BTs are bad, set bit0 of data quality flagword
            IF ( .NOT. good_bts ) amb1c_dataqual(fov) =
     &                            IBSET(amb1c_dataqual(fov),bits(0))

         ENDDO  ! fov
      ENDIF  ! good_data .AND. Write_data


      RETURN
      END
