!+ read fixed data file for ATOVIN
!
      SUBROUTINE INFDF (FDFunit, satid)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: infdf.F 570 2017-11-24 13:59:51Z frna $
!
! Purpose:
!   read fixed data file for ATOVIN and store data
!
! Method:
! 1.  Open file
! 2.  Read fixed data for any satellites in the file
!   For each satellite, read satellite id codes, orbit height and period,
!   and then (if ATOVS) sets of "antenna efficiencies" for AMSU-A and AMSU-B for
!   the specified satellite.
!   ( reference: "Measurements of the AMSU-B Antenna Pattern",
!   T.J.Hewison & R.Saunders, IEEE Transactions on Geoscience and Remote
!   Sensing, Vol 34, No 2, March 1996 )
!
!   Format of input is specified in AAPP documentation APP203F, vn of 12/12/96
!   Note that, generally, input data with "!" in the first column are ignored.
!
!   Antenna efficiencies are specified for each ATOVS satellite
!   in the fixed data file, but are read only if satid is specified.
!   INFDF is first called with satid as zero.
!
!   From version 1.5, also read the fractional antenna reflectivity difference:
!   1 - R(90 deg)/R(nadir). Positive for V pol channels.
!
!   Numbers are stored in commons /FDFinfo/, /inastore/ & /inbstore/.
!
!   From version 1.8 the user can specify which version of antenna coefs to use,
!   via environment variables FDFSAT and FDFVN. Default is to use the first set
!   of coefs (for a particular satellite) in the fdf.dat file. FDFSAT must be in
!   the same format as in the fdf.dat file, for example:
!     export FDFSAT="METO-02 NOAA18"
!     export FDFVN="2 1"
!   You only need to list the satellites for which you don't want the default.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0     22/2/96  Original code. (R.J.Renshaw)
!  1.1    12/12/96  Process ATOVS and TOVS. Many changes. RJR
!  1.2    03/03/98  Channel number included in FDF file. RJR
!  1.3    06/03/03  Add satid as argument. Read antenna efficiencies
!                   only if called with the appropriate satid. (Keith Whyte)
!  1.4    09/03/04  Modify for Linux (N Atkinson / A Korpela)
!  1.5    22/07/04  Read antenna reflectivity factors also (N Atkinson)
!  1.6    28/11/05  Support for Metop. PM/NCA
!  1.7    02/11/06  Allow negative satid to be interpreted as a WMO satid. NCA
!  1.8    14/05/07  Support environment variables FDFSAT and FDFVN.
!  1.9    30/05/07  Change open for g95
!  1.10   18/10/10  Add NPP
!  1.11   23/11/17  NOAA-19 is the last ATOVS, NOAA-20 is JPSS-1.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-A Data Variables for processing within ATOVIN 
#include "inastore.h"
!  AMSU-B Data Variables for processing within ATOVIN 
#include "inbstore.h"
!
! Subroutine arguments
!   Scalar arguments with intent(in):
      INTEGER       FDFunit       ! unit number for fixed data file
      INTEGER       satid         ! NESDIS satellite id for AMSU antenna
                                  ! efficiencies
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for opening & reading
      INTEGER       chn           ! loop variable over all channels
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       isat          ! loop variable over all satellites
      INTEGER       indx          ! index to instrument in input_line
      INTEGER       maxindx       ! upper limit on indx
      INTEGER       i             ! loop index
      INTEGER       WMOsatid      ! WMO satid
      CHARACTER*80  input_line    ! line read from unit FDFunit
      CHARACTER*5   series        ! satellite series
      CHARACTER*7   satidstr      ! series plus number
      CHARACTER*80  fdfsat,fdfvn  ! strings for holding environment variables
      CHARACTER*7   fdfsats(MaxSats) !List of satellites for version numbers
      INTEGER       fdfvns(MaxSats)  !List of version numbers
      INTEGER       lensat,lenvn     !String lengths
      INTEGER       n_vn             !Number of specified versions
      LOGICAL       vn_required

      REAL          reflectivity_diff(20)  ! AMSU antenna reflectivity diffs
      COMMON /amsu_reflectivity/ reflectivity_diff
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVIN error routine
!
!- End of header

      IF (satid.LT.0) THEN     !Interpret negative satid as a WMO satid
        WMOsatid = -satid
      ELSE
        WMOsatid = 0
      ENDIF 

      DO chn=1,20
        reflectivity_diff(chn)=0.0   !Initialise to zero
      ENDDO

! 1.1 Open file
!     ---- ----
#ifdef FORTRAN_OPEN_RW
      OPEN ( UNIT=FDFunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus, ACTION = 'READ' )
#else
      OPEN ( UNIT=FDFunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!If that didn't work, use STATUS="UNKNOWN" (needed for Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=FDFunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )
#endif

      WRITE ( errMessage,
     & "("" Error opening fixed data file, unit "",I8)" ) FDFunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"INFDF")

! 1.2 Read required antenna correction version numbers (optional)
!     -----------------------------------------------------------

      call getenv('FDFSAT',fdfsat)
      call getenv('FDFVN',fdfvn)
      lensat = len_trim(fdfsat)
      lenvn = len_trim(fdfvn)
      n_vn = 0
      if (lensat .ge. 7 .and. lenvn .ge. 1) then
        n_vn = (lensat + 1)/8      !7 character names plus space between them
        read(fdfsat,*,iostat=errStatus) fdfsats
        read(fdfvn,*,iostat=errStatus) fdfvns
      endif

! 2.  Read fixed data for any satellites in the file
!     ---- ----- ---- --- --- ---------- -- --- ----

! 2.1 Loop through lines to the end, looking for lines that aren't comments

      Nsat = 0   ! number of satellites found
      maxindx = LEN(input_line) - 6 ! upper limit for indx
      READ(FDFunit,"(A80)",IOSTAT=errStatus) input_line
      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )

         IF ( input_line(1:1) .NE. "!" ) THEN

!           interpret as a satellite name, e.g. "NOAA-14"
            CALL C2Upper ( input_line )  ! convert to upper case

!           look through the string for the satellite id (NOAA-??)
            indx = 0   ! pointer to character in input_line
            isat = 0   ! satellite id (e.g. 14 for NOAA-14)
            DO WHILE ( indx.LT.maxindx .AND. isat.EQ.0 )
               indx = indx + 1
               IF (( input_line(indx:indx+4) .EQ. "NOAA-" )
     >        .OR. ( input_line(indx:indx+4) .EQ. "METO-" )
     >        .OR. ( input_line(indx:indx+2) .EQ. "NPP" ) ) THEN
                   READ(input_line(indx+5:indx+6),"(I2)") isat
                   series = input_line(indx:indx+4)
                   satidstr = input_line(indx:indx+6)
                   errMessage = "Reading NOAA sat id"
                   errStatus = ABS (errStatus)   ! end-of-file returns -1
                   CALL ErrorReport(errStatus,errMessage,"INFDF")
               ENDIF
            ENDDO  ! WHILE

!           store this
            Nsat = Nsat + 1
            NOAAid(Nsat) = isat

!           now read the NESDIS satid, nominal satellite height, period and
!           WMO satid
            input_line = "!"
            errStatus = 0
            errMessage = " "
            WRITE ( errMessage,
     &              "(""Reading NESDIS sat id etc., "",A5,I2)")
     &              series,isat
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(FDFunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"INFDF")
            READ(input_line,*,IOSTAT=errStatus)
     &           NESDISid(Nsat), SatHt(Nsat), OrbitPd(Nsat), WMOid(Nsat)
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"INFDF")

! 2.2       If this is an ATOVS satellite then read AMSU antenna efficiencies
            IF ( (isat.GT.14.AND.isat.LE.19).OR.series.EQ."METO-" ) THEN

             errMessage = " "
             WRITE ( errMessage,
     &            "(""Reading antenna efficiencies, "",A5,I2)")
     &            series,isat
              
!            version number      
!            read the whole line into memory  (ignoring lines starting "!")
             input_line = "!"
             DO WHILE ( input_line(1:1).EQ."!" .AND. errStatus.EQ.0 )
                READ(FDFunit,"(A80)",IOSTAT=errStatus) input_line
             ENDDO
             IF ( errStatus.EQ.0 )
     &          READ(input_line,*,IOSTAT=errStatus) FDFvnno(Nsat)
             errStatus = ABS (errStatus)   ! end-of-file returns -1
             CALL ErrorReport(errStatus,errMessage,"INFDF")

             vn_required = .true.      !Is this is the correct version?
             DO i=1,n_vn
               IF (fdfsats(i) .eq. satidstr) THEN
                 IF (FDFvnno(Nsat) .eq. fdfvns(i)) then
                   IF (satid .ne. 0) 
     &               write(*,*)'Use fdf data version ',FDFvnno(Nsat),
     &               ' for ',satidstr
                 ELSE
                   IF (satid .ne. 0) 
     &               write(*,*)'Skip fdf data version ',FDFvnno(Nsat),
     &               ' for ',satidstr
                   vn_required = .false.
                 ENDIF
               ENDIF
             ENDDO

!            Check whether FDF data have already been read for this satellite.
!            This applies if FDFSAT and FDFVN have not been set.
!            Default is to accept the first set of coefficients.
             DO i=1,Nsat-1 
               IF (WMOid(i) .EQ. WMOid(Nsat)) THEN
                 IF (satid .ne. 0 .and. vn_required) 
     &             write(*,*)'Skip fdf data version ',FDFvnno(Nsat),
     &             ' for ',satidstr
                 vn_required = .false.
               ENDIF
             ENDDO

             IF ( .not. vn_required) THEN
               Nsat = Nsat - 1          !Ingore this entry completely
               errStatus = 0
               DO WHILE (errStatus.EQ.0.AND.input_line(1:1).NE."!".AND.
     &                   input_line(1:3).NE."END")
                 READ(FDFunit,"(A80)",IOSTAT=errStatus) input_line    
               ENDDO
              
             ELSE IF ( NESDISid(Nsat) .EQ. satid. OR. 
     &            WMOid(Nsat) .EQ. WMOsatid) THEN
              
! 2.2.1        Antenna efficiencies for AMSU-A
               DO chn = 1, Anchan
                  READ(FDFunit,*)  ! channel number
                  READ(FDFunit,"(10F8.6)",IOSTAT=errStatus)
     &             (Aeff_Earth(fov,chn),fov=1,Anfovs), !Earth efficiencies
     &             (Aeff_Platf(fov,chn),fov=1,Anfovs), ! Platform
     &             (Aeff_Space(fov,chn),fov=1,Anfovs)  ! Space
                  errStatus = ABS (errStatus)   ! end-of-file returns -1
                  CALL ErrorReport(errStatus,errMessage,"INFDF")
               ENDDO  ! chn

! 2.2.2        Antenna efficiencies for AMSU-B
               DO chn = 1, Bnchan
                  READ(FDFunit,*)  ! channel number
                  READ(FDFunit,"(10F8.6)",IOSTAT=errStatus)
     &             (Beff_Earth(fov,chn),fov=1,Bnfovs), !Earth efficiencies
     &             (Beff_Platf(fov,chn),fov=1,Bnfovs), ! Platform
     &             (Beff_Space(fov,chn),fov=1,Bnfovs)  ! Space
                  errStatus = ABS (errStatus)   ! end-of-file returns -1
                  CALL ErrorReport(errStatus,errMessage,"INFDF")
               ENDDO  ! chn
              
C1.5 Read antenna reflectivity factors if present - channel and value
C1.5 These follow straight on from the efficiencies

               DO WHILE (errStatus.EQ.0.AND.input_line(1:1).NE."!".AND.
     &                   input_line(1:3).NE."END")
                  READ(FDFunit,"(A80)",IOSTAT=errStatus) input_line
                 IF (errStatus.EQ.0.AND.input_line(1:1).NE."!".AND.
     &               input_line(1:3).NE."END") THEN
                   READ(input_line,*) chn, reflectivity_diff(chn)
                 ENDIF
               ENDDO

             ELSE
C              nlines = Anchan*(1+3*Anfovs/10) + Bnchan*(1+3*Bnfovs/10)
C              DO i=1,nlines
C                  READ(FDFunit,"(A80)",IOSTAT=errStatus) input_line
C                  errStatus = ABS (errStatus)   ! end-of-file returns -1
C                  CALL ErrorReport(errStatus,errMessage,"INFDF")
C              ENDDO

C1.5 Number of lines now varies depending on whether antenna reflectivities are
C1.5 present or not
               DO WHILE (errStatus.EQ.0.AND.input_line(1:1).NE."!".AND.
     &                   input_line(1:3).NE."END")
                 READ(FDFunit,"(A80)",IOSTAT=errStatus) input_line    
               ENDDO

             ENDIF  ! satid matches

            ENDIF  ! not a TOVS satellite

         ENDIF  ! ( input_line(1:1) .NE. "!" )

         READ(FDFunit,"(A80)",IOSTAT=errStatus) input_line

      ENDDO  ! WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )

      CLOSE (FDFunit)

      RETURN
      END
