!+ Set up 1C HIRS scanline from 1B scanline within ATOVIN
!
      SUBROUTINE INHGET ( scan )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inhget.F 463 2015-09-28 15:29:53Z frna $
!
! Purpose:
!   Place information from common blocks /hrs1bdt/ & /inhstore/
!   into common block /hrs1cdt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1B & 1C include files (hrs1b.h & hrs1c.h) are
!     compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Extract required information from 1B into 1C
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      5/2/96  Original code. (R.J.Renshaw)
!  1.1    12/12/96  No error status in call to INSUNA
!                   Flag BTs as missing when not an earth-view
!                   Calibration flagword deleted
!  1.2      2/1/97  Bug fixed  - loop over fovs in 2.3
!  1.3    17/01/97  lat/lon in 1C file now 10^4, but 10^2 in 1B file (sec 1.4)
!                   Hcaline new (see 2.3)
!  1.4    10/03/98  Updated for 1c vn1.9 (instrument temperature added)
!  1.5    20/05/98  updated for latest 1b include files
!                   (lat/lon * 10^4 and no version numbers)
!  1.6    18/06/98  bug fixed in converting azimuth angle to [0,360]
!  1.7    20/07/98  Adjust NESDIS counts to allow for 12th bit being signed
!  1.8    19/02/99  Fix: correct loop over hrs1c_filler
!  1.9    21/12/06  Convert counts to K*100 for hrs1c_instrtemp. N.C.Atkinson
!  1.10   18/09/15  Add hrs1c_coldnedt, hrs1c_warmnedt. Scan line as argument.
!                   Allow for difference between NESDIS and AAPP azimuth angles.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  HIRS Data Variables for processing within ATOVIN 
#include "inhstore.h"
!  HIRS level 1B header/data common blocks 
#include "hrs1b.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!  Earth model, used to identify non-AAPP geolocation
#include "p_earth.h"
!
! Arguments:
      INTEGER       scan
!
! Local parameters:
      INTEGER       H1Bvnnb       ! version number of 1B format compatible
         PARAMETER (H1Bvnnb=1)    ! with the code in this subroutine
      INTEGER       H1Cvnnb       ! version number of 1C format compatible
         PARAMETER (H1Cvnnb=190)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       jchn          ! channel number (comes from index_calib)
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       i             ! general purpose loop variable
      INTEGER       angle         ! satellite azimuth angle (deg*100)
      INTEGER       year          ! scan line year
      INTEGER       day           ! scan line day of year
      INTEGER       time          ! scan line UTC time of day in milliseconds
      INTEGER       clockdrift    ! satellite clock drift delta in milliseconds
      INTEGER       coldcal_use
      INTEGER       warmcal_use
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
      REAL          rcount        ! raw count value as real number
!
! Local arrays:
      REAL          latitudes(Hnfovs)   ! latitudes for one scanline
      REAL          longitudes(Hnfovs)  ! longitudes for one scanline
      REAL          azimuths(Hnfovs)    ! solar azimuth angles for one scanline
      REAL          zeniths(Hnfovs)     ! solar zenith angles for one scanline
      INTEGER       index_calib(Hnchan) ! decode order in which channels
!                                          are specified HIRS 1B file
      DATA index_calib / 1, 17, 2, 3, 13, 4, 18, 11, 19, 7,
     &                   8, 20, 10, 14, 6, 5, 15, 12, 16, 9 /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeAdd              ! add an increment to a date/time
      External INSUNA               ! calculate solar zenith & azimuth angles
!
!- End of header

      SAVE first                    ! .TRUE. on first call

! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!!        Is the 1B include file compatible with this source code ?
!         IF ( H1Bvnnb .NE. hrs_1bvnnb ) THEN
!            errStatus = -11  ! warning
!            errMessage = " "
!            WRITE ( errMessage,
!     &      "("" Code expects 1B format"",I6,"" but hrs1b.h is "",I6)")
!     &           H1Bvnnb, hrs_1bvnnb
!            CALL ErrorReport ( errStatus, errMessage, "INHGET")
!         ENDIF

!        Is the 1C include file compatible with this source code ?
         IF ( H1Cvnnb .NE. hrs_1cvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but hrs1c.h is "",I6)")
     &           H1Cvnnb, hrs_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "INHGET")
         ENDIF

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Extract required information from 1B into 1C
!     ------- -------- ----------- ---- -- ---- --

! 1.1 scanline number
      hrs1c_scnlin = hrs_scnlin

! 1.2 scanline date/time

      hrs1c_scnlinyr = hrs_scnlinyr        ! scan line year
      hrs1c_scnlindy = hrs_scnlindy        ! scan line day of year
      hrs1c_scnlintime = hrs_scnlintime    ! UTC time of day in milliseconds

      year = hrs1c_scnlinyr                ! scan line year
      day  = hrs1c_scnlindy                ! scan line day of year
      time = hrs1c_scnlintime              ! UTC time of day in milliseconds

!      - adding clock drift delta if not already done so
      IF ( .NOT.BTEST(INT(hrs_scnlinf),bits(14)) ) THEN ! add clock drift delta
         clockdrift = hrs_clockdrift       ! (msecs)    (hrs_... is I2)
         IF ( clockdrift .LT. 0 ) clockdrift = clockdrift + 65536
         CALL TimeAdd ( year, day, time, clockdrift,
     &        hrs1c_scnlinyr, hrs1c_scnlindy, hrs1c_scnlintime)
      ENDIF

! 1.3 other scanline information
      hrs1c_mjfrcnt = hrs_mjfrcnt          !major frame count
      hrs1c_qualind = hrs_qualind          !quality indicator bit field
      hrs1c_scanqual = hrs_linqualflgs     !scan line quality flags
!     Quality control word for each channel
      DO chn = 1, Hnchan
         jchn = index_calib(chn)  ! decode the order in the 1B file
         hrs1c_chanqual(jchn) = hrs_chqualflg(chn)
      ENDDO  ! chn

      rcount = hrs_elem(6,62)     !HIRS baseplate temperature
      if (hrs_h_bpcnttmp(1) .NE. 0) then    !non-zero in hirscl_algoV4
        hrs1c_instrtemp = 100*(hrs_h_bpcnttmp(1)*1.0E-6  + 
     &                 rcount*(hrs_h_bpcnttmp(2)*1.0E-9  + 
     &                 rcount*(hrs_h_bpcnttmp(3)*1.0E-14 + 
     &                 rcount*(hrs_h_bpcnttmp(4)*1.0E-17 + 
     &                 rcount*(hrs_h_bpcnttmp(5)*1.0E-21 + 
     &                 rcount* hrs_h_bpcnttmp(6)*1.0E-25)))))
      else
        hrs1c_instrtemp = hrs_elem(6,62)    !Coefs not available
      endif

      DO i = 1, 2
         hrs1c_spare1(i) = 0               !spare
      ENDDO  ! i

! 1.4 Navigation

      hrs1c_scalti = hrs_scalti  ! sat altitude above ref. ellipsoid (km*10)

      DO fov = 1, Hnfovs
         hrs1c_latlon(1,fov) = hrs_pos(fov*2-1) ! latitude
         hrs1c_latlon(2,fov) = hrs_pos(fov*2)   ! longitude
         latitudes(fov)  = REAL(hrs1c_latlon(1,fov))*1.E-4
         longitudes(fov) = REAL(hrs1c_latlon(2,fov))*1.E-4
      ENDDO  ! fov

!     angles
      DO fov = 1, Hnfovs
         hrs1c_angles(1,fov) = hrs_ang(3*fov-1) ! zenith
         angle = hrs_ang(3*fov)   ! azimuth
!        make sure azimuth between 0-360, rather than +/-180
         IF ( angle .LT. 0 ) angle = 36000 + angle
         hrs1c_angles(2,fov) = angle
         hrs1c_angles(3,fov) = hrs_ang(3*fov-2) ! solar zenith
      ENDDO  ! fov

!     Calculate solar azimuth angles
      CALL INSUNA ( year, day, time, Hnfovs, latitudes, longitudes,
     &              azimuths, zeniths )
      DO fov = 1, Hnfovs
         hrs1c_angles(4,fov) = NINT(azimuths(fov)*100.) ! solar azimuth
      ENDDO  ! fov

!     NESDIS 1B uses a "relative" azimuth, defined as satellite minus solar
!     angle. In AAPP it's the satellite angle. Allow for this difference.
!     Use site_id and modelid to identify NESDIS geolocation.

      IF (hrs_h_siteid.EQ.'NSS'.AND.hrs_h_modelid.NE.earth_modelid) THEN
        DO fov = 1, Hnfovs
          angle = hrs1c_angles(2,fov) + hrs1c_angles(4,fov)
          IF (angle .GT. 36000) angle = angle - 36000
          hrs1c_angles(2,fov) = angle
        ENDDO
      ENDIF

! 1.5 Initialise variables new in 1C
      DO i = 1, 2
         hrs1c_spare2(i) = 0               !spare
      ENDDO  ! i
      DO chn = 1, Hnchan
      DO fov = 1, Hnfovs
         hrs1c_btemps(chn,fov) = MDI       !brightness temperatures
      ENDDO  ! fov
      ENDDO  ! chn
      DO fov = 1, Hnfovs
         hrs1c_dataqual(fov) = 0           !qc word for each field of view
      ENDDO  ! fov


! 2.  Extract other information required here
!     ------- ----- ----------- -------- ----

! 2.1 Calibration coefficients (primary & secondary)
      DO chn = 1, Hnchan
         jchn = index_calib(chn)  ! decode the order in the 1B file
         Hcalib(1,1,jchn) = REAL(hrs_calcof(1,chn)) * 1.E-12  ! primary
         Hcalib(1,2,jchn) = REAL(hrs_calcof(2,chn)) * 1.E-09
         Hcalib(1,3,jchn) = REAL(hrs_calcof(3,chn)) * 1.E-06
         Hcalib(2,1,jchn) = REAL(hrs_scalcof(1,chn)) * 1.E-12 ! secondary
         Hcalib(2,2,jchn) = REAL(hrs_scalcof(2,chn)) * 1.E-09
         Hcalib(2,3,jchn) = REAL(hrs_scalcof(3,chn)) * 1.E-06
      ENDDO  ! chn

! 2.2 Counts from the instrument
      DO chn = 1, Hnchan
         jchn = index_calib(chn)  ! decode the order in the 1B file
         DO fov = 1, Hnfovs
            Hcounts(jchn,fov) = hrs_elem(chn+2,fov-1)
         ENDDO  ! fov
      ENDDO  ! chn

!     If this is NESDIS, subtract 4096 to allow for 12th bit being a sign-bit
      IF ( hrs_h_siteid .EQ. "NSS" ) THEN
         DO chn = 1, Hnchan
         DO fov = 1, Hnfovs
            Hcounts(chn,fov) = Hcounts(chn,fov) - 4096
         ENDDO  ! fov
         ENDDO  ! chn
      ENDIF

! 2.3 Is this an earth view ?
      Hcaline = hrs_scntyp.NE.0   ! .TRUE. for calibration lines

! 2.4 NEDT
      coldcal_use = 1
      warmcal_use = 1
      DO i=2,mxlin_nedt_hrs
        IF (Hcoldline(i).GT.0.AND.Hcoldline(i).LE.scan) THEN
          coldcal_use = i
        ENDIF
        IF (Hwarmline(i).GT.0.AND.Hwarmline(i).LE.scan) THEN
          warmcal_use = i
        ENDIF
      ENDDO

      DO chn = 1, Hnchan-1
         IF (HcoldNEDT(chn,coldcal_use).GT. 0.0) THEN
           hrs1c_coldnedt(chn) = HcoldNEDT(chn,coldcal_use)*1000
         ELSE
           hrs1c_coldnedt(chn) = MDI
         ENDIF
         IF (HwarmNEDT(chn,warmcal_use).GT. 0.0) THEN
           hrs1c_warmnedt(chn) = HwarmNEDT(chn,coldcal_use)*1000
         ELSE
           hrs1c_warmnedt(chn) = MDI
         ENDIF
         IF (HcoldNEDT_rms(chn).GT. 0.0) THEN
           hrs1c_h_coldnedt(chn) = HcoldNEDT_rms(chn)*1000
         ELSE
           hrs1c_h_coldnedt(chn) = MDI
         ENDIF
         IF (HwarmNEDT_rms(chn).GT. 0.0) THEN
           hrs1c_h_warmnedt(chn) = HwarmNEDT_rms(chn)*1000
         ELSE
           hrs1c_h_warmnedt(chn) = MDI
         ENDIF
      ENDDO

      DO i = 1, 119-2*Hnchan
         hrs1c_filler(i) = 0      !filler
      ENDDO


      RETURN
      END
