!+ Interpret HIRS level 1B header information within ATOVIN
!
      SUBROUTINE INHHDR
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inhhdr.F 484 2016-04-04 09:24:35Z frna $
!
! Purpose:
!   Extract required information from header records for use by ATOVIN.
!   Set up header information for HIRS level 1C file.
!
! Method:
! 1.  Check format version number and data type
!     HIRS level 1B header has already been read into common /hrs1bhd/
!     Check that format of 1B & 1C include files (hrs1b.h, hrs1c.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1B file is compatible with this subroutine,
!     and that data type from the header is 5 for HIRS.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Print out header info
! 3.  Extract required information from HIRS 1B header
!     (-and derive Planck coefficients).
! 4.  Set up level 1C header
!     Use /hrs1bhd/ to set up the header information for HIRS level 1C
!     in common /hrs1chd/.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      2/2/96  Original code. (R.J.Renshaw)
!  1.1     12/12/96 Conform to 1C format vn 1.8
!  1.2     10/03/98 Read orbit no's from hrs_h_dataname, not hrs_h_prblkid. RJR
!                   Updated for 1c vn1.9 (nothing relevant here)
!  1.3    20/05/98  checks on 1B version number commented out
!  1.4    24/11/03  Allow for new 1B format (hrs_h_l1bversyr >= 2003)
!  1.5    15/09/04  Use constants_preproc.h for fundamental constants - NCA
!  1.6    23/11/04  Allow for 2004 NESDIS 1b format update
!  1.7    29/11/05  Add wmosatid - NCA
!  1.8    04/04/16  Correct filler initialisation (account for NEdT)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  HIRS Data Variables for processing within ATOVIN 
#include "inhstore.h"
!  HIRS level 1B header/data common blocks 
#include "hrs1b.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!  Fundamental and physical constants 
#include "constants_preproc.h"
!
! Local parameters:
!
      INTEGER       H1Bvnnb       ! version number of 1B format compatible
         PARAMETER (H1Bvnnb=1)    ! with the code in this subroutine
      INTEGER       H1Cvnnb       ! version number of 1C format compatible
         PARAMETER (H1Cvnnb=190)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status (0 is OK, +ve is fatal)
      INTEGER       i             ! general purpose loop variables
      INTEGER       isat          ! loop variable over all satellites
      INTEGER       chn           ! loop variable over all channels
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      INTEGER       sat_index     ! satellite index (between 1 & MaxSats)
      INTEGER       sttorbit      ! last 2 digits, orbit number at start
      INTEGER       endorbit      ! last 2 digits, orbit number at end
      REAL          seconds       ! seconds (start/end of data)
      REAL          wave_no       ! channel wave number (cm-1)
!
! Function & Subroutine calls:
      External ConvDay            ! convert day of year to month/day
      External ErrorReport        ! ATOVIN/ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

!! 1.1 Is the 1B include file compatible with this source code ?
!!     -- --- -- ------- ---- ---------- ---- ---- ------ ----
!
!      IF ( H1Bvnnb .NE. hrs_1bvnnb ) THEN
!         errStatus = -11  ! warning
!         errMessage = " "
!         WRITE ( errMessage,
!     &   "("" Code expects 1B format "",I6,"" but hrs1b.h is "",I6)")
!     &        H1Bvnnb, hrs_1bvnnb
!         CALL ErrorReport ( errStatus, errMessage, "INHHDR")
!      ENDIF

! 1.2 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( H1Cvnnb .NE. hrs_1cvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but hrs1c.h is "",I6)")
     &        H1Cvnnb, hrs_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "INHHDR")
      ENDIF

!! 1.3 Is the 1B data compatible ?
!!     -- --- -- ---- ----------
!!     "HRS1B.H" contains its own version number "hrs_h_l1bversnb"
!      IF ( hrs_h_l1bversnb .NE. hrs_1bvnnb ) THEN
!         errStatus = -13  ! warning
!         errMessage = " "
!         WRITE ( errMessage,
!     &   "("" Format of level 1B HIRS file is "",I8,"", not "",I8)")
!     &        hrs_h_l1bversnb,hrs_1bvnnb
!         CALL ErrorReport ( errStatus, errMessage, "INHHDR")
!      ENDIF

! 1.4 Is the 1B data HIRS ?
!     -- --- -- ---- ----
!     "hrs_h_datatyp" from 1B header should be 5 for HIRS
      IF ( hrs_h_datatyp .NE. 5 ) THEN
         ErrStatus = 14
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Data type in 1B HIRS file is "",I8,""; not 5 for HIRS"")")
     &        hrs_h_datatyp
         CALL ErrorReport ( errStatus, errMessage, "INHHDR")
      ENDIF


! 2.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "########"
      WRITE(*,*) "# HIRS #"
      WRITE(*,*) "########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1B file, format version "",3I6)")
     &            hrs_h_l1bversnb, hrs_h_l1bversyr, hrs_h_l1bversdy
      WRITE(*,"(I4,"" header records in file: "",A42)")
     &            hrs_h_hdrcnt, hrs_h_dataname

!     calculate start and end dates/times from header:

!     ..start
      year = hrs_h_startdatayr
      yrday = hrs_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(hrs_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = hrs_h_enddatayr
      yrday = hrs_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(hrs_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines of which "",I6,
     &          "" earth-located and "",I6,"" missing."")")
     &          hrs_h_scnlin, hrs_h_callocsclin, hrs_h_misscnlin
      WRITE(*,*)


! 3.  Extract required information from HIRS 1B header
!     ------- -------- ----------- ---- ---- -- ------
!     From v1.4 need to check whether we are using old or new 1b format

! 3.1 temperature-radiance conversion  (Planck function - bandwidth corrected)
!                  order of channels = 1,2,3,4,...,18,19
!                  for nochannel = 1 to 19 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
!                  (scaling factor for wavenumber changes to 10^5
!                   for channels 13-19 inclusive)

!     IR channels
      DO chn = 1, Hnchan-1

!        Use wave number to calculate channel-dependent constants for Planck fn
         wave_no = REAL(hrs_h_tempradcnv(1,chn)) * 1.E-06
         IF ( chn.GE.13 ) wave_no = wave_no * 10.

!       (formula from Houghton "The Physics of Atmospheres",
!                   Cambridge Univ Press, p236, 2nd Edition - 1986)
!        - calculated radiances are W m(-2) sr(-1) cm(-1)
         HPlanck(1,chn) = C_1 * wave_no * wave_no * wave_no
         HPlanck(2,chn) = C_2 * wave_no

!        Store bandwidth corrections to adjust the brightness temperatures
         HPlanck(3,chn) = REAL(hrs_h_tempradcnv(2,chn)) * 1.E-06
         HPlanck(4,chn) = REAL(hrs_h_tempradcnv(3,chn)) * 1.E-06
      ENDDO  ! chn

!     Visible channel. Assume the scaling factors from the NOAA-N 1b format
!     because the values were off scale and hence meaningless before then.
!     In practice the values are not used in AAPP.
      IF (hrs_h_20solfiltirrad .EQ. 9999) THEN
        HPlanck(1,Hnchan) = REAL(MDI) * 1.0E-6       !Missing
      ELSE
        HPlanck(1,Hnchan) = REAL(INT(hrs_h_20solfiltirrad)) * 1.E-02
!        10^2 x (albedo-radiance ch20 solar filtered irradiance, W/m2)
      ENDIF     
      IF (hrs_h_20equifiltwidth .EQ. 9999) THEN
        HPlanck(2,Hnchan) = REAL(MDI) * 1.0E-6       !Missing
      ELSE
        HPlanck(2,Hnchan) = REAL(INT(hrs_h_20equifiltwidth)) * 1.E-04
!        10^4 x (albedo-radiance ch 20 equivalent filter width, cm^-1)
      ENDIF     

! 4.  Set up level 1C header
!     --- -- ----- -- ------

!!!   This code is compatible with files:
!!!   "hrs1b.h" November 1995 (CMS)
!!!   "hrs1c.h" vn1.9       10/03/98              Richard Renshaw (UKMO)

!     creation site for 1C data
      hrs1c_h_site   = SiteID              !file creation site ID (CHAR*3)
      hrs1c_h_cfill1 = " "                 !filler

!     creation site for original 1B data
!        - format starts "XXX.YY..." and we want XXX
      hrs1c_h_1bsite = hrs_h_dataname(1:3)
      hrs1c_h_cfill2 = " "                 !filler

      hrs1c_h_versnb = hrs_1cvnnb          !level 1C format version number
      hrs1c_h_versyr = hrs_1cvnyr          !level 1C format version year
      hrs1c_h_versdy = hrs_1cvndy          !level 1C format version day of year
      hrs1c_h_hdrcnt = 1 !count of header records in 1C file  - set to 1

! Convert satellite id from NESDIS 1B code to e.g. 14 for NOAA-14
      sat_index = 0
      DO isat = 1, MaxSats
         IF ( hrs_h_satid .EQ. NESDISid(isat) ) THEN
            hrs1c_h_satid = NOAAid(isat)
            hrs1c_h_wmosatid = WMOid(isat)
            sat_index = isat
         ENDIF
      ENDDO  ! isat
      IF ( sat_index .EQ. 0 ) THEN
         errStatus = 40  ! fatal
         errMessage = " "
         WRITE ( errMessage,"("" HIRS 1B satid not recognised: "",I8)")
     &           hrs_h_satid
         CALL ErrorReport ( errStatus, errMessage, "INHHDR")
      ENDIF

      hrs1c_h_instrument = hrs_h_datatyp   !data type code (HIRS=5)

      hrs1c_h_satht = NINT(SatHt(sat_index)*10.)!nominal sat height, km*10
      hrs1c_h_period = NINT(OrbitPd(sat_index)) !nominal orbit period (seconds)

! Derive orbit numbers from 1B "processing block id":
! "hrs_h_prblid" is of the format "Bxxxxxyy" where "xxxxx" is the orbit number
! at start of file, and "yy" is the final two digits at end of file
!      READ(hrs_h_prblkid,"(1X,I5)") hrs1c_h_startorbit !orbit number (at start)
      READ(hrs_h_dataname,"(32X,I5)") hrs1c_h_startorbit !orbit number at start
! derive orbit number at end of file
!      READ(hrs_h_prblkid,"(4X,I2)") sttorbit ! last 2 digits, start orbit no.
      READ(hrs_h_dataname,"(35X,I2)") sttorbit ! last 2 digits, start orbit no.
!      READ(hrs_h_prblkid,"(6X,I2)") endorbit ! last 2 digits, final orbit no.
      READ(hrs_h_dataname,"(37X,I2)") endorbit ! last 2 digits, final orbit no.
      hrs1c_h_endorbit = endorbit + 100*(hrs1c_h_startorbit/100)
      IF ( sttorbit.GT.endorbit )
     &     hrs1c_h_endorbit = hrs1c_h_endorbit + 100

      hrs1c_h_startdatayr = hrs_h_startdatayr!start of data set year
      hrs1c_h_startdatady = hrs_h_startdatady! (ditto..) day of the year
      hrs1c_h_startdatatime = hrs_h_startdatatime !(ditto..) UTC time of day,ms
      hrs1c_h_enddatayr = hrs_h_enddatayr!end of data set year
      hrs1c_h_enddatady = hrs_h_enddatady !(ditto..) day of the year
      hrs1c_h_enddatatime = hrs_h_enddatatime !(ditto..) UTC time of day,ms

! These two set later:
!     hrs1c_h_scnlin = hrs_h_scnlin   !count of scan lines in this data set
!     hrs1c_h_misscnlin = hrs_h_misscnlin !count of missing scan lines

      hrs1c_h_spare = 0               !spare

! temperature-radiance conversion coefficients
!                  order of channels = 1,2,3,4,...,18,19
!                  for nochannel = 1 to 19 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
!     IR channels
      DO chn = 1, Hnchan-1
         DO i = 1, 3
           hrs1c_h_tempradcnv(i,chn) = hrs_h_tempradcnv(i,chn)
         ENDDO  ! i
      ENDDO  ! chn

!     visible channel
      hrs1c_h_20solfiltirrad = NINT(HPlanck(1,Hnchan)*1E6)
!                  10^6 x (albedo-radiance ch20 solar filtered irradiance,W/m2)

      hrs1c_h_20equifiltwidth = NINT(HPlanck(2,Hnchan)*1E6)
!                  10^6 x (albedo-radiance ch 20 equivalent filter width cm^-1)

      DO i = 1, 1543
         hrs1c_h_filler(i) = 0    !filler
      ENDDO


      RETURN
      END
