!+ process HIRS data within ATOVIN
!
      SUBROUTINE INHIRS ( H1Bunit, H1Cunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inhirs.F 461 2015-09-15 14:38:15Z frna $
!
! Purpose:
!   Controlling routine to read level 1b HIRS data,
!   convert HIRS electronic counts to radiances, then to
!   brightness temperatures, and write out as level 1c.
!
! Method:
! 1.  Open HIRS 1B & 1C files
! 2.  Read 1B header & set up 1C header
! 3.  Loop through scan lines, reading, processing & writing data
! 4.  Complete & write out 1C header
! 5.  Close files
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      2/2/96  Original code. (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  HIRS Data Variables for processing within ATOVIN 
#include "inhstore.h"
!  HIRS level 1B header/data common blocks 
#include "hrs1b.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!
!   Scalar arguments with intent(In):
      INTEGER       H1Bunit       ! unit number for HIRS level 1B file
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
!
! Local scalars:
      INTEGER       first_lineno  ! number of first scanline in the 1B file
      INTEGER       last_lineno   ! number of last scanline in the 1B file
      INTEGER       H1Brecord     ! record number for HIRS level 1B file
      INTEGER       H1Crecord     ! record number for HIRS level 1C file
      INTEGER       scnlin        ! loop variable over scanlines
      LOGICAL       first_line    ! true for first scanline in the 1B file
         DATA       first_line / .TRUE. /
      LOGICAL       write_data    ! .TRUE. if line is to be written out
!
! Function & Subroutine calls:
      External INHHDR               ! interpret HIRS level 1B header into 1C
      External INHGET               ! read HIRS 1B scan line
      External INHGET_NEDT          ! read HIRS 1B scan line
      External INHPRC               ! process HIRS scan line
      External IOH1B                ! I/O routine for level 1B HIRS data
      External IOH1C                ! I/O routine for level 1C HIRS data
!
!- End of header


! 1.  Open HIRS 1B & 1C files
!     ---- ---- -- - -- -----

      CALL IOH1B ("OPEN/READ", H1Bunit, H1Brecord )    ! 1B file
      CALL IOH1C ("OPEN/WRITE", H1Cunit, H1Crecord )   ! 1C file


! 2.  Read 1B header & set up 1C header
!     ---- -- ------ - --- -- -- ------

! 2.1 Read 1B header
      H1Brecord = 1   ! Headers start on record 1.
      CALL IOH1B ("READ HEADER", H1Bunit, H1Brecord )

!     point to last header record in 1B
      H1Brecord = hrs_h_hdrcnt

! 2.2 Set up 1C header
      CALL INHHDR

! 2.3 Do an initial pass through the HIRS data to get NEDT
      DO scnlin = 1, hrs_h_scnlin
         H1Brecord = H1Brecord + 1
         CALL IOH1B ("READ DATA", H1Bunit, H1Brecord )
         CALL INHGET_NEDT ( scnlin )
      ENDDO

!     point to last header record in 1B
      H1Brecord = hrs_h_hdrcnt

!     point to last header record in 1C
      H1Crecord = hrs1c_h_hdrcnt


! 3.  Loop through scan lines, reading, processing & writing data
!     ---- ------- ---- ------ -------- ---------- - ------- ----

      DO scnlin = 1, hrs_h_scnlin

! 3.1    Read a scanline
         H1Brecord = H1Brecord + 1
         CALL IOH1B ("READ DATA", H1Bunit, H1Brecord )

! 3.2    Transfer 1B data into program and 1C arrays
         CALL INHGET ( scnlin )

! 3.3    Process the scanline
         CALL INHPRC ( write_data )

! 3.4    Write it out, if it contains any good data
         IF ( write_data ) THEN

            H1Crecord = H1Crecord + 1
            CALL IOH1C ("WRITE DATA", H1Cunit, H1Crecord )

!           Record the number of the first & last scanlines in the file
            IF ( first_line ) THEN
               first_lineno = hrs1c_scnlin
               first_line = .FALSE.
            ENDIF
            last_lineno = hrs1c_scnlin

         ENDIF

      ENDDO   ! (scnlin)


! 4.  Complete & write out 1C header
!     -------- - ----- --- -- ------

!     correct 1C header to show the correct number of scanlines
      hrs1c_h_scnlin = H1Crecord - hrs1c_h_hdrcnt

!     ..and the correct number of missing scanlines
      hrs1c_h_misscnlin = (1+last_lineno-first_lineno) - hrs1c_h_scnlin

!     write the 1C header
      H1Crecord = 1
      CALL IOH1C ("WRITE HEADER", H1Cunit, H1Crecord )

      WRITE(*,*) hrs1c_h_scnlin," scanlines written to HIRS 1C file"
      WRITE(*,*)


! 5.  Close files
!     ----- -----

      CALL IOH1B ("CLOSE", H1Bunit, H1Brecord )   ! 1B file
      CALL IOH1C ("CLOSE", H1Cunit, H1Crecord )   ! 1C file


      RETURN
      END
