!+  process HIRS scanline - level 1b to 1c
!
      SUBROUTINE INHPRC ( Write_data )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inhprc.F 61 2006-11-27 13:43:51Z frna $
!
! Purpose:
!   Convert counts to radiances to brightness temperatures
!   and perform some quality control.
!
! Method:
! 1.  Check scanline number & time of scanline
! 2.  Act on flags for the scanline from the 1B file
! 3.  Loop through scanline and channels, converting counts to BTs, with qc
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0    22/2/96    Original code. (R.J.Renshaw)
!  1.1    15/04/96   Bit 0 of Data Quality flagwords and calibration flagword
!                    now set when appropriate     - RJR
!  1.2    12/12/96   If scanline numbers and date/time increment inconsistently
!                    then say so and change scanline number.
!                    Calibration flag now for all fovs and channels in scanline
!  1.3    18/12/96   bug fixed: hrs1c_dataqual now hrs1c_dataqual(fov)
!  1.4    20/01/97   Hcaline new (see 2.3) so that calibration lines are
!                    written but with BTs flagged and set to MDI
!  1.5    12/06/98   Hcycle REAL not INTEGER
!  1.6    21/04/99   fix for zero hirs BTs at start of pass (Aarno Korpela)
!  1.7    08/01/01   Ensure correct adjustment to scanline number.
!  1.8    21/06/05   Don't check bit 28 of qualind. NCA
!  1.9    21/11/06   Ignore scans with bad times but do not abort (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  HIRS Data Variables for processing within ATOVIN 
#include "inhstore.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!
! Subroutine arguments:
!
!   Scalar arguments with intent(out)
      LOGICAL       Write_data    ! .TRUE. if line is to be written out
!
! Local parameters:
      INTEGER       mx_dTime      ! upper limit for departure of scan time
         PARAMETER (mx_dTime=50)  ! from nominal value  (milliseconds)
!
! Local Scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       cal_set       ! use 1=primary or 2=secondary cal coeffs
      INTEGER       dline         ! jump in line number since last call
      INTEGER       dTime         ! time difference returned from s/r TimeSub
      INTEGER       errTime       ! error status returned from s/r TimeSub
      INTEGER       fov           ! loop variable over all fovs
      REAL          counts        ! averaged PRT counts from the instrument
      REAL          radiance      ! radiance calculated from counts
      REAL          bri_temp      ! brightness temperature calc from radiance
      LOGICAL       good_bts      ! .FALSE. if all channel BTs for the fov bad
      LOGICAL       good_data     ! .TRUE. if line has brightness temperatures
!
      INTEGER       prev_line     ! line number of previous scanline
         DATA       prev_line / MDI /
      INTEGER       prev_year     ! year of previous scanline
         DATA       prev_year / 0 /
      INTEGER       prev_day      ! day of previous scanline
         DATA       prev_day / 0 /
      INTEGER       prev_time     ! time of previous scanline
         DATA       prev_time / 0 /
      INTEGER       line_adjust   ! adjustment to line number to agree with
         DATA       line_adjust / 0 /                       ! scanline times
C/NIWA add
      INTEGER       i
      LOGICAL       allzero
C/NIWA end
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeSub              ! subtract two dates/times
!
      SAVE prev_line, prev_year, prev_day, prev_time, line_adjust
!
!- End of header


! 1.  Check scanline number & time of scanline
!     ----- -------- ------ - ---- -- --------

! 1.1 Innocent until proven guilty..
      good_data  = .TRUE.   ! good brightness temperatures for this line
      Write_data = .TRUE.   ! do process & write out this scanline

! 1.2 Adjust scanline number to be consistent with scanline times
      hrs1c_scnlin = hrs1c_scnlin + line_adjust

! 1.3 Calculate the scanline number from the date/time (to check)
      IF (prev_line.NE.MDI) THEN   ! not the 1st record read

         CALL TimeSub
     &        ( hrs1c_scnlinyr, hrs1c_scnlindy, hrs1c_scnlintime,
     &          prev_year, prev_day, prev_time, dTime, errTime )
         errMessage = " "
         WRITE ( errMessage,
     &   "(""Scanline date/time in error"",3I9)")
     &   hrs1c_scnlinyr, hrs1c_scnlindy, hrs1c_scnlintime
         CALL ErrorReport(errTime,errMessage,"INHPRC")

!        calculate how many scanlines this is since the last
         dline = NINT(REAL(dTime)/Hcycle)

!        Ignore scan if date/time not incrementing by a multiple of Acycle
         IF ( ABS(dTime-NINT(REAL(dline)*Hcycle)).GT.mx_dTime ) THEN
            errMessage = " "
            WRITE ( errMessage,
     &      "(""Time gaps between scanlines are wrong, lines"",3I9)")
     &      prev_line-line_adjust, hrs1c_scnlin-line_adjust, dline
            errStatus = -131 ! warning
            Write_data = .FALSE.
            hrs1c_scnlintime = prev_time  !keep the previous time
            hrs1c_scnlin = prev_line      !  and line number
            dline = 0
            CALL ErrorReport(errStatus,errMessage,"INHPRC")
         ENDIF

!        Complain if scanline number and estimated scanline number differ
         IF ( hrs1c_scnlin.NE.(prev_line+dline) ) THEN
            errMessage = " "
            WRITE ( errMessage,"(""Times for scanlines "",I5,
     &      "" and "",I5,"" differ by "",I3,"" scanline cycles"")")
     &      prev_line-line_adjust, hrs1c_scnlin-line_adjust, dline
            errStatus = -132  ! warning
            CALL ErrorReport(errStatus,errMessage,"INHPRC")
!         and reset scanline & adjustment to what we think they should be
!            line_adjust = line_adjust + (dline-1)
!            hrs1c_scnlin = hrs1c_scnlin + (dline-1)
            line_adjust = line_adjust + prev_line + dline - hrs1c_scnlin
            hrs1c_scnlin = prev_line + dline
         ENDIF

      ENDIF  ! not the 1st record read

! 1.4 save values for next scanline
      prev_line = hrs1c_scnlin
      prev_year = hrs1c_scnlinyr
      prev_day = hrs1c_scnlindy
      prev_time = hrs1c_scnlintime


! 2.  Act on flags for the scanline from the 1B file
!     --- -- ----- --- --- -------- ---- --- -- ----

! 2.1 Set choice of calibration coefficients from "scanline quality flags"

      IF ( BTEST( hrs1c_scanqual, bits(15) ) ! not calibrated, bad time
     & .OR.BTEST( hrs1c_scanqual, bits(13) ) ! not calibrated, bad PRT
     & .OR.BTEST( hrs1c_scanqual, bits(10) ) ! uncalibrated, instrument mode
     & ) THEN

!      WRITE(*,*) ' not calibrated - use secondary set'
         cal_set = 2 ! use secondary set of calibration coefficients
         DO fov = 1, Hnfovs
            hrs1c_dataqual(fov) = IBSET( hrs1c_dataqual(fov), bits(30) )
         ENDDO

      ELSE

!      WRITE(*,*) ' using primary set'
         cal_set = 1 ! use primary set of calibration coefficients

      ENDIF

! 2.2 "quality indicator": Cannot use any information from these fovs

      IF ( BTEST( hrs1c_qualind, bits(30) ) ! time sequence error
     & .OR.BTEST( hrs1c_qualind, bits(27) ) ! no earth location
     & ) THEN

!      WRITE(*,*) ' no brightness temperatures  - do not write'
         good_data  = .FALSE.  ! no brightness temperatures for this line
         Write_data = .FALSE.  ! don't write out this line

      ENDIF

! 2.3 "quality indicator": No BTs but can still map data to these fovs

      IF ( BTEST( hrs1c_qualind, bits(31) ) ! don't use for product generation
     & .OR. Hcaline                         ! calibration line
     &  ) THEN

!Version 1.8: comment out the line below since NESDIS now use this flag for
!minor anomalies
!     & .OR.BTEST( hrs1c_qualind, bits(28) ) ! calibration anomaly

!      WRITE(*,*) ' no brightness temperatures  - do write'
         good_data  = .FALSE.  ! no brightness temperatures for this line

!        Set BTs to missing and flag them
         DO fov = 1, Hnfovs
            DO chn = 1, Hnchan
               hrs1c_btemps(chn,fov) = MDI
            ENDDO  ! chn
            hrs1c_dataqual(fov) = IBSET(hrs1c_dataqual(fov),bits(0))
         ENDDO  ! fov

      ENDIF


! 3.  Loop through scanline and channels, converting counts to BTs, with qc
!     ---- ------- -------- --- --------- ---------- ------ -- ---- ---- --

      IF ( good_data .AND. Write_data ) THEN
         DO fov = 1, Hnfovs

!      WRITE(*,*) ' fov ', fov
C/NIWA add
            allzero=.TRUE.
            DO i=1,20
               allzero = allzero.AND.(Hcounts(i,fov).EQ.0)
               ENDDO
C/NIWA end
!           Loop through the infra-red channels
            good_bts = .FALSE.
            DO chn = 1, Hnchan-1

! 3.1          Convert counts to radiance (W/m2)
               counts = REAL(Hcounts(chn,fov))
               radiance = -999.
C/NIWA         IF (ABS(counts).LT.9999.) THEN       ! (check data)
               IF (ABS(counts).LT.9999. .AND. .NOT.allzero) THEN       ! (check data)
                  radiance = Hcalib(cal_set,3,chn)
     &                     + Hcalib(cal_set,2,chn)*counts
     &                     + Hcalib(cal_set,1,chn)*counts*counts
               ENDIF  ! counts

! 3.2          Convert radiance to brightness temperature (inverse Planck)
!              and modify to allow for bandwidth of channels
               bri_temp = -999.
               IF (radiance.GT.0.) THEN             ! (check data)
                  bri_temp = HPlanck(2,chn)         ! Kelvin
     &                  / log( 1. + HPlanck(1,chn)/radiance )
                  bri_temp = (bri_temp-HPlanck(3,chn)) / HPlanck(4,chn)
               ENDIF  ! radiance

! 3.3          Quality control final brightness temperature
               IF ( bri_temp.GT.HminBT .AND. bri_temp.LT.HmaxBT ) THEN
                     hrs1c_btemps(chn,fov) = NINT ( bri_temp*100. )  ! 100*deg K
                 good_bts = .TRUE.
!      WRITE(*,'(I3,3F12.2)') chn, counts, radiance, bri_temp
               ELSE
!                 Set BT to missing, and flag it
                  hrs1c_btemps(chn,fov) = MDI
                  hrs1c_dataqual(fov) =
     &                           IBSET(hrs1c_dataqual(fov),bits(chn))
!      WRITE(*,*) 'failed', bri_temp
               ENDIF  ! bri_temp
!      WRITE(*,'(I3,3F12.2)') chn, counts, radiance, bri_temp
!      WRITE(*,*) ' ch, cnt, rad, bt ', chn, counts, radiance, bri_temp
!      WRITE(*,*) Hcalib(cal_set,3,chn), Hcalib(cal_set,2,chn),
!     &           Hcalib(cal_set,1,chn)
!      WRITE(*,*)

            ENDDO  ! chn

! 3.4       For HIRS-20, calibrate, but don't convert radiance to BT
            counts = REAL(Hcounts(Hnchan,fov))
            radiance = -999.
C/NIWA      IF (ABS(counts).LT.9999.) THEN
            IF (ABS(counts).LT.9999. .AND. .NOT.allzero) THEN
               radiance = Hcalib(cal_set,3,Hnchan)
     &                  + Hcalib(cal_set,2,Hnchan)*counts
     &                  + Hcalib(cal_set,1,Hnchan)*counts*counts
            ENDIF  ! counts

! 3.5       Quality control final HIRS-20 radiance
            IF (radiance.GT.0.) THEN
               hrs1c_btemps(Hnchan,fov) = NINT ( radiance*100. ) !10^2 * W/m2
            ELSE
               hrs1c_btemps(Hnchan,fov) = MDI
               hrs1c_dataqual(fov) =
     &                    IBSET(hrs1c_dataqual(fov),bits(Hnchan))
            ENDIF  ! radiance

!           If all BTs are bad, set bit0 of data quality flagword
            IF ( .NOT. good_bts ) hrs1c_dataqual(fov) =
     &                            IBSET(hrs1c_dataqual(fov),bits(0))

         ENDDO  ! fov
      ENDIF  ! good_data .AND. Write_data

      RETURN
      END

