!+ Interpret MSU level 1B header information within ATOVIN
!
      SUBROUTINE INMHDR
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inmhdr.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Extract required information from header records for use by ATOVIN.
!   Set up header information for MSU level 1C file.
!
! Method:
! 1.  Check format version number and data type
!     MSU level 1B header has already been read into common /msu1bhd/
!     Check that format of 1B & 1C include files (msu1b.h, msu1c.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1B file is compatible with this subroutine,
!     and that data type from the header is 6 for MSU.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Print out header info
! 3.  Extract required information from MSU 1B header
!     (-and derive Planck coefficients).
! 4.  Set up level 1C header
!     Use /msu1bhd/ to set up the header information for MSU level 1C
!     in common /msu1chd/.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0     19/3/96  Original code. (R.J.Renshaw)
!  1.1     12/12/96 Conform to 1C format vn 1.8
!  1.2     03/03/98 Read orbit no's from msu_h_dataname, not msu_h_prblkid. RJR
!  1.3    20/05/98  checks on 1B version number commented out
!  1.4    15/09/04  Use constants_preproc.h for fundamental constants - NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  MSU Data Variables for processing within ATOVIN 
#include "inmstore.h"
!  MSU level 1B header/data common blocks 
#include "msu1b.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!  Fundamental and physical constants 
#include "constants_preproc.h"
!
! Local parameters:
!
      INTEGER       M1Bvnnb       ! version number of 1B format compatible
         PARAMETER (M1Bvnnb=1)    ! with the code in this subroutine
      INTEGER       M1Cvnnb       ! version number of 1C format compatible
         PARAMETER (M1Cvnnb=180)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status (0 is OK, +ve is fatal)
      INTEGER       i             ! general purpose loop variables
      INTEGER       isat          ! loop variable over all satellites
      INTEGER       chn           ! loop variable over all channels
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      INTEGER       sat_index     ! satellite index (between 1 & MaxSats)
      INTEGER       sttorbit      ! last 2 digits, orbit number at start
      INTEGER       endorbit      ! last 2 digits, orbit number at end
      REAL          seconds       ! seconds (start/end of data)
      REAL          wave_no       ! channel wave number (cm-1)
!
! Function & Subroutine calls:
      External ConvDay            ! convert day of year to month/day
      External ErrorReport        ! ATOVIN/ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

!! 1.1 Is the 1B include file compatible with this source code ?
!!     -- --- -- ------- ---- ---------- ---- ---- ------ ----
!
!      IF ( M1Bvnnb .NE. msu_1bvnnb ) THEN
!         errStatus = -11  ! warning
!         errMessage = " "
!         WRITE ( errMessage,
!     &   "("" Code expects 1B format "",I6,"" but msu1b.h is "",I6)")
!     &        M1Bvnnb, msu_1bvnnb
!         CALL ErrorReport ( errStatus, errMessage, "INMHDR")
!      ENDIF

! 1.2 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( M1Cvnnb .NE. msu_1cvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but msu1c.h is "",I6)")
     &        M1Cvnnb, msu_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "INMHDR")
      ENDIF

!! 1.3 Is the 1B data compatible ?
!!     -- --- -- ---- ----------
!!     "MSU1B.H" contains its own version number "msu_h_l1bversnb"
!      IF ( msu_h_l1bversnb .NE. msu_1bvnnb ) THEN
!         errStatus = -13  ! warning
!         errMessage = " "
!         WRITE ( errMessage,
!     &   "("" Format of level 1B MSU file is "",I8,"", not "",I8)")
!     &        msu_h_l1bversnb,msu_1bvnnb
!         CALL ErrorReport ( errStatus, errMessage, "INMHDR")
!      ENDIF

! 1.4 Is the 1B data MSU ?
!     -- --- -- ---- ----
!     "msu_h_datatyp" from 1B header should be 6 for MSU
      IF ( msu_h_datatyp .NE. 6 ) THEN
         ErrStatus = 14
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Data type in 1B MSU file is "",I8,""; not 6 for MSU"")")
     &        msu_h_datatyp
         CALL ErrorReport ( errStatus, errMessage, "INMHDR")
      ENDIF


! 2.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "#######"
      WRITE(*,*) "# MSU #"
      WRITE(*,*) "#######"
      WRITE(*,*)
      WRITE(*,"("" reading level 1B file, format version "",3I6)")
     &            msu_h_l1bversnb, msu_h_l1bversyr, msu_h_l1bversdy
      WRITE(*,"(I4,"" header records in file: "",A42)")
     &            msu_h_hdrcnt, msu_h_dataname

!     calculate start and end dates/times from header
!     ..start
      year = msu_h_startdatayr
      yrday = msu_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(msu_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = msu_h_enddatayr
      yrday = msu_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(msu_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines of which "",I6,
     &          "" earth-located and "",I6,"" missing."")")
     &          msu_h_scnlin, msu_h_callocsclin, msu_h_misscnlin
      WRITE(*,*)


! 3.  Extract required information from MSU 1B header
!     ------- -------- ----------- ---- ---- -- ------

! 3.1 temperature-radiance conversion  (Planck function - bandwidth corrected)
!                  for nochannel = 1 to 4 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      DO chn = 1, Mnchan

!        Use wave number to calculate channel-dependent constants for Planck fn
         wave_no = REAL(msu_h_censperes(chn)) * 1.E-04   ! cm-1

!       (formula from Houghton "The Physics of Atmospheres",
!                   Cambridge Univ Press, p236, 2nd Edition - 1986)
!        - calculated radiances are W m(-2) sr(-1) cm(-1)
         MPlanck(1,chn) = C_1 * wave_no * wave_no * wave_no
         MPlanck(2,chn) = C_2 * wave_no

!        Store bandwidth corrections to adjust the brightness temperatures
!         - set to defaults, because Planck fn is nearly linear here
         MPlanck(3,chn) = 0.
         MPlanck(4,chn) = 1.

      ENDDO  ! chn

! 3.2 Non-linearity coefficients
      DO chn = 1, Mnchan
         Mnonlin(1,chn) = REAL(msu_h_nolincor(1,chn)) * 1.E-12
         Mnonlin(2,chn) = REAL(msu_h_nolincor(2,chn)) * 1.E-06
         Mnonlin(3,chn) = REAL(msu_h_nolincor(3,chn)) * 1.E-06
      ENDDO  ! chn


! 4.  Set up level 1C header
!     --- -- ----- -- ------

!!!   This code is compatible with files:
!!!   "msu1b.h" November 1995 (CMS)
!!!   "msu1c.h" vn1.8       12/12/96              Richard Renshaw (UKMO)

!     creation site for 1C data
      msu1c_h_site   = SiteID              !file creation site ID (CHAR*3)
      msu1c_h_cfill1 = " "                 !filler

!     creation site for original 1B data
!        - format starts "XXX.YY..." and we want XXX
      msu1c_h_1bsite = msu_h_dataname(1:3)
      msu1c_h_cfill2 = " "                 !filler

      msu1c_h_versnb = msu_1cvnnb          !level 1C format version number
      msu1c_h_versyr = msu_1cvnyr          !level 1C format version year
      msu1c_h_versdy = msu_1cvndy          !level 1C format version day of year
      msu1c_h_hdrcnt = 1 !count of header records in 1C file  - set to 1

! Convert satellite id from NESDIS 1B code to e.g. 14 for NOAA-14
      sat_index = 0
      DO isat = 1, MaxSats
         IF ( msu_h_satid .EQ. NESDISid(isat) ) THEN
            msu1c_h_satid = NOAAid(isat)
            sat_index = isat
         ENDIF
      ENDDO  ! isat
      IF ( sat_index .EQ. 0 ) THEN
         errStatus = 40  ! fatal
         errMessage = " "
         WRITE ( errMessage,"("" MSU 1B satid not recognised: "",I8)")
     &           msu_h_satid
         CALL ErrorReport ( errStatus, errMessage, "INMHDR")
      ENDIF

      msu1c_h_instrument = msu_h_datatyp   !data type code (MSU=6)

      msu1c_h_satht = NINT(SatHt(sat_index)*10.)!nominal sat height, km*10
      msu1c_h_period = NINT(OrbitPd(sat_index)) !nominal orbit period (seconds)

! Derive orbit numbers from 1B "processing block id":
! "msu_h_prblid" is of the format "Bxxxxxyy" where "xxxxx" is the orbit number
! at start of file, and "yy" is the final two digits at end of file
!      READ(msu_h_prblkid,"(1X,I5)") msu1c_h_startorbit !orbit number (at start)
      READ(msu_h_dataname,"(32X,I5)") msu1c_h_startorbit !orbit number at start
! derive orbit number at end of file
!      READ(msu_h_prblkid,"(4X,I2)") sttorbit ! last 2 digits, start orbit no.
      READ(msu_h_dataname,"(35X,I2)") sttorbit ! last 2 digits, start orbit no.
!      READ(msu_h_prblkid,"(6X,I2)") endorbit ! last 2 digits, final orbit no.
      READ(msu_h_dataname,"(37X,I2)") endorbit ! last 2 digits, final orbit no.
      msu1c_h_endorbit = endorbit + 100*(msu1c_h_startorbit/100)
      IF ( sttorbit.GT.endorbit )
     &     msu1c_h_endorbit = msu1c_h_endorbit + 100

      msu1c_h_startdatayr = msu_h_startdatayr!start of data set year
      msu1c_h_startdatady = msu_h_startdatady! (ditto..) day of the year
      msu1c_h_startdatatime = msu_h_startdatatime !(ditto..) UTC time of day,ms
      msu1c_h_enddatayr = msu_h_enddatayr!end of data set year
      msu1c_h_enddatady = msu_h_enddatady !(ditto..) day of the year
      msu1c_h_enddatatime = msu_h_enddatatime !(ditto..) UTC time of day,ms

! These two set later:
!     msu1c_h_scnlin = msu_h_scnlin   !count of scan lines in this data set
!     msu1c_h_misscnlin = msu_h_misscnlin !count of missing scan lines

      msu1c_h_spare1 = 0              !spare
      msu1c_h_spare2 = 0              !spare

! temperature-radiance conversion coefficients
!                  for nochannel = 1 to 4 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
!   - constants 1 & 2 set to defaults, because Planck fn is nearly linear here
      DO chn = 1, Mnchan
         msu1c_h_tempradcnv(1,chn) = msu_h_censperes(chn)*100 ! change to 1E06
         msu1c_h_tempradcnv(2,chn) = 0
         msu1c_h_tempradcnv(3,chn) = 1E06
      ENDDO  ! chn

      DO i = 1, 92
         msu1c_h_filler(i) = 0    !filler
      ENDDO


      RETURN
      END
