!+ Set up 1C AMSU-B scanline from MHS 1B scanline within ATOVIN
!
      SUBROUTINE INMHSGET ( scan )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inmhsget.F 484 2016-04-04 09:24:35Z frna $
!
! Purpose:
!   Place information from common blocks /mhs1bdt/ & /inbstore/
!   into common block /amb1cdt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1B & 1C include files (amb1b.h & amb1c.h) are
!     compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Extract required information from 1B into 1C
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0    02/7/03   Created from inbget.F v1.6    N C Atkinson
!  1.1    23/1/06   Allow for systems that have ICHAR as signed. NCA
!  1.2    04/7/14   Remove unnecessary "float" functions, which were causing
!                   portability problems when used with ICHAR.
!  1.3    18/09/15  Add amb1c_coldnedt, amb1c_warmnedt. Scan line as argument.
!                   Allow for difference between NESDIS and AAPP azimuth angles.
!  1.4    04/04/16  Bug fix: Anchan -> Bnchan in NEDT loop.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-B Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  MHS level 1B header/data common blocks 
#include "mhs1b.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!  Earth model, used to identify non-AAPP geolocation
#include "p_earth.h"
!
! Arguments:
      INTEGER       scan
!
! Local parameters:
      INTEGER       B1Bvnnb       ! version number of 1B format compatible
         PARAMETER (B1Bvnnb=1)    ! with the code in this subroutine
      INTEGER       B1Cvnnb       ! version number of 1C format compatible
         PARAMETER (B1Cvnnb=190)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chn3          ! = chn*3
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       i             ! general purpose loop variable
      INTEGER       angle         ! satellite azimuth angle (deg*100)
      INTEGER       year          ! scan line year
      INTEGER       day           ! scan line day of year
      INTEGER       time          ! scan line UTC time of day in milliseconds
      INTEGER       clockdrift    ! satellite clock drift delta in milliseconds
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Local arrays:
      REAL          latitudes(Bnfovs)   ! latitudes for one scanline
      REAL          longitudes(Bnfovs)  ! longitudes for one scanline
      REAL          azimuths(Bnfovs)    ! solar azimuth angles for one scanline
      REAL          zeniths(Bnfovs)     ! solar zenith angles for one scanline
      REAL          t                   ! temperature
      REAL          thermcnttmp(5)      ! thermistor coefficients
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeAdd              ! add an increment to a date/time
      External INSUNA               ! calculate solar zenith & azimuth angles
!
!- End of header

      SAVE first                    ! .TRUE. on first call
      SAVE thermcnttmp

! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!!        Is the 1B include file compatible with this source code ?
!         IF ( B1Bvnnb .NE. mhs_1bvnnb ) THEN
!            errStatus = -11  ! warning
!            errMessage = " "
!            WRITE ( errMessage,
!     &      "("" Code expects 1B format"",I6,"" but mhs1b.h is "",I6)")
!     &           B1Bvnnb, mhs_1bvnnb
!            CALL ErrorReport ( errStatus, errMessage, "INMHSGET")
!         ENDIF

!        Is the 1C include file compatible with this source code ?
         IF ( B1Cvnnb .NE. amb_1cvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but amb1c.h is "",I6)")
     &           B1Cvnnb, amb_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "INBGET")
         ENDIF

         first = .FALSE.
         
         thermcnttmp(1) = mhs_h_therm(1)*1.0E-4     !Conversion coefficients
         thermcnttmp(2) = mhs_h_therm(2)*1.0E-7
         thermcnttmp(3) = mhs_h_therm(3)*1.0E-10
         thermcnttmp(4) = mhs_h_therm(4)*1.0E-12
         thermcnttmp(5) = mhs_h_therm(5)*1.0E-15

      ENDIF  ! ( first )


! 1.  Extract required information from 1B into 1C
!     ------- -------- ----------- ---- -- ---- --

! 1.1 scanline number
      amb1c_scnlin = mhs_scnlin

! 1.2 scanline date/time    ( - assume already corrected for clock drift)
      amb1c_scnlinyr = mhs_scnlinyr        ! scan line year
      amb1c_scnlindy = mhs_scnlindy        ! scan line day of year
      amb1c_scnlintime = mhs_scnlintime    ! UTC time of day in milliseconds

      year = amb1c_scnlinyr                ! scan line year
      day  = amb1c_scnlindy                ! scan line day of year
      time = amb1c_scnlintime              ! UTC time of day in milliseconds

!      - adding clock drift delta if not already done so
      IF ( .NOT.BTEST(INT(mhs_scnlinbit),bits(14)) ) THEN  ! add clock drift delta
         clockdrift = mhs_clockdrift       ! (msecs)    (mhs_... is I2)
         IF ( clockdrift .LT. 0 ) clockdrift = clockdrift + 65536
         CALL TimeAdd ( year, day, time, clockdrift,
     &        amb1c_scnlinyr, amb1c_scnlindy, amb1c_scnlintime)
      ENDIF

! 1.3 other scanline information
      amb1c_qualind = mhs_qualind          !quality indicator bit field
      amb1c_scanqual = mhs_linqualflgs     !scan line quality flags
!     Quality control word for each channel
      DO chn = 1, Bnchan
         amb1c_chanqual(chn) = mhs_chqualflg(chn)
      ENDDO  ! chn

! Compute instrument temperature in units of 0.01 deg (see mhs_iwttmp.F)
!
      if (mhs_h_tsensid .eq. 0) then 
        t = ICHAR(mhs_temps(4)) !primary: LO H5 temperature [QBS5]
      else
        t = ICHAR(mhs_temps(1)) !secondary: LO H1 temperature [QBS1]
      endif  
      IF (t.LT.0) t = t + 256.0     !Ensure unsigned
      amb1c_instrtemp = NINT(100.0*(thermcnttmp(1) + t*(thermcnttmp(2) + 
     &    t*(thermcnttmp(3) + t*(thermcnttmp(4) + t*thermcnttmp(5))))))

      DO i = 1, 2
         amb1c_spare1(i) = 0               !spare
      ENDDO  ! i

! 1.4 Navigation

      amb1c_scalti = mhs_scalti  ! sat altitude above ref. ellipsoid (km*10)

      DO fov = 1, Bnfovs
         amb1c_latlon(1,fov) = mhs_pos(fov*2-1) ! latitude
         amb1c_latlon(2,fov) = mhs_pos(fov*2)   ! longitude
         latitudes(fov)  = REAL(amb1c_latlon(1,fov))*1.E-4
         longitudes(fov) = REAL(amb1c_latlon(2,fov))*1.E-4
      ENDDO  ! fov

!     angles
      DO fov = 1, Bnfovs
         amb1c_angles(1,fov) = mhs_ang(3*fov-1) ! zenith
         angle = mhs_ang(3*fov)   ! azimuth
!        make sure azimuth between 0-360, rather than +/-180
         IF ( angle .LT. 0 ) angle = 36000 + angle
         amb1c_angles(2,fov) = angle
      ENDDO  ! fov

!     Calculate solar zenith/azimuth angles
      CALL INSUNA ( year, day, time, Bnfovs, latitudes, longitudes,
     &              azimuths, zeniths )
      DO fov = 1, Bnfovs
         amb1c_angles(3,fov) = NINT(zeniths(fov)*100.)  ! solar zenith
         amb1c_angles(4,fov) = NINT(azimuths(fov)*100.) ! solar azimuth
      ENDDO  ! fov

!     NESDIS 1B uses a "relative" azimuth, defined as satellite minus solar
!     angle. In AAPP it's the satellite angle. Allow for this difference.
!     Use site_id and modelid to identify NESDIS geolocation.

      IF (mhs_h_siteid.EQ.'NSS'.AND.mhs_h_modelid.NE.earth_modelid) THEN
        DO fov = 1, Bnfovs
          angle = amb1c_angles(2,fov) + amb1c_angles(4,fov)
          IF (angle .GT. 36000) angle = angle - 36000
          amb1c_angles(2,fov) = angle
        ENDDO
      ENDIF

! 1.5 Initialise variables new in 1C
      DO i = 1, 2
         amb1c_spare2(i) = 0               !spare
      ENDDO  ! i
      DO chn = 1, Bnchan
      DO fov = 1, Bnfovs
         amb1c_btemps(chn,fov) = MDI       !brightness temperatures
      ENDDO  ! fov
      ENDDO  ! chn
      DO fov = 1, Bnfovs
         amb1c_dataqual(fov) = 0           !qc word for each field of view
      ENDDO  ! fov


! 2.  Extract other information required here
!     ------- ----- ----------- -------- ----

! 2.1 Calibration coefficients (primary & secondary)
      DO chn = 1, Bnchan
         chn3 = chn*3
         Bcalib(1,1,chn) = REAL(mhs_pmcal(chn3-2)) * 1.E-16  ! primary
         Bcalib(1,2,chn) = REAL(mhs_pmcal(chn3-1)) * 1.E-10
         Bcalib(1,3,chn) = REAL(mhs_pmcal(chn3))   * 1.E-06
         Bcalib(2,1,chn) = REAL(mhs_sdcal(chn3-2)) * 1.E-16  ! secondary
         Bcalib(2,2,chn) = REAL(mhs_sdcal(chn3-1)) * 1.E-10
         Bcalib(2,3,chn) = REAL(mhs_sdcal(chn3))   * 1.E-06
      ENDDO  ! chn

! 2.2 Counts from the instrument

      DO fov = 1, Bnfovs
         chn3 = fov*6 - 5
         DO chn = 1, Bnchan
            Bcounts(chn,fov) = mod(mhs_earth_data(chn+chn3)+65536,65536)
         ENDDO  ! chn
      ENDDO  ! fov

! 2.3 NEDT
      DO chn = 1, Bnchan
         IF (BcoldNEDT(chn,scan).GT. 0.0) THEN
           amb1c_coldnedt(chn) = BcoldNEDT(chn,scan)*1000
         ELSE
           amb1c_coldnedt(chn) = MDI
         ENDIF
         IF (BwarmNEDT(chn,scan).GT. 0.0) THEN
           amb1c_warmnedt(chn) = BwarmNEDT(chn,scan)*1000
         ELSE
           amb1c_warmnedt(chn) = MDI
         ENDIF
         IF (BcoldNEDT_rms(chn).GT. 0.0) THEN
           amb1c_h_coldnedt(chn) = BcoldNEDT_rms(chn)*1000
         ELSE
           amb1c_h_coldnedt(chn) = MDI
         ENDIF
         IF (BwarmNEDT_rms(chn).GT. 0.0) THEN
           amb1c_h_warmnedt(chn) = BwarmNEDT_rms(chn)*1000
         ELSE
           amb1c_h_warmnedt(chn) = MDI
         ENDIF
      ENDDO

      DO i = 1, 55-2*Bnchan
         amb1c_filler(i) = 0      !filler
      ENDDO

      RETURN
      END
