!+ Interpret MHS level 1B header information within ATOVIN
!
      SUBROUTINE INMHSHDR
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inmhshdr.F 484 2016-04-04 09:24:35Z frna $
!
! Purpose:
!   Extract required information from header records for use by ATOVIN.
!   Set up header information for AMSU-B level 1C file.
!
! Method:
! 1.  Check format version number and data type
!     MHS level 1B header has already been read into common /mhs1bhd/
!     Check that format of 1B & 1C include files (mhs1b.h, amb1c.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1B file is compatible with this subroutine,
!     and that data type from the header is 12 for MHS.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Print out header info
! 3.  Extract required information from MHS 1B header
!     (-and derive Planck coefficients).
! 4.  Set up level 1C header
!     Use /mhs1bhd/ to set up the header information for AMSU-B level 1C
!     in common /amb1chd/.
!
! Current Code Owner: N C Atkinson
!
! History:
! Version  Date      Comment
!
!  1.0    02/7/03   Created from inbhdr.F v1.5       N C Atkinson
!  1.1    29/11/05  Add wmosatid - NCA
!  1.2    21/09/06  Bug fix in deriving amb1c_h_endorbit - NCA
!  1.3    04/04/16  Correct filler initialisation (account for NEdT)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-B Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  MHS level 1B header/data common blocks 
#include "mhs1b.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!
! Local parameters:
!
      INTEGER       B1Bvnnb       ! version number of 1B format compatible
         PARAMETER (B1Bvnnb=1)    ! with the code in this subroutine
      INTEGER       B1Cvnnb       ! version number of 1C format compatible
         PARAMETER (B1Cvnnb=190)  ! with the code in this subroutine
!
! Constants from Cohen & Taylor "Physics Today" pp 9-13 Aug 1992
! Constants from the "Dictionary of Physics", Penguin Press, ed. V.H.Pitt.
      REAL          C_1           ! First radiation constant
         PARAMETER (C_1 = 1.191044e-05) ! mW/(sqm.ster.cm^-4)
      REAL          C_2           ! Second radiation constant
         PARAMETER (C_2 = 1.438769) ! K/cm^-1 ##
! These not needed:
!     REAL          C_Planck      ! Planck constant h
!        PARAMETER (C_Planck = 6.6260755E-34)   ! J s
!     REAL          C_Boltzman    ! Boltzman constant k
!        PARAMETER (C_Boltzman = 1.380658E-23)  ! J K(-1)
!     REAL          C_light       ! speed of light c
!        PARAMETER (C_light = 299792458.)      ! m s(-1)
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status (0 is OK, +ve is fatal)
      INTEGER       i             ! general purpose loop variables
      INTEGER       isat          ! loop variable over all satellites
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chn3          ! = chn*3
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      INTEGER       sat_index     ! satellite index (between 1 & MaxSats)
      INTEGER       sttorbit      ! last 2 digits, orbit number at start
      INTEGER       endorbit      ! last 2 digits, orbit number at end
      REAL          seconds       ! seconds (start/end of data)
      REAL          wave_no       ! channel wave number (cm-1)
!
! Function & Subroutine calls:
      External ConvDay            ! convert day of year to month/day
      External ErrorReport        ! ATOVIN/ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

!! 1.1 Is the 1B include file compatible with this source code ?
!!     -- --- -- ------- ---- ---------- ---- ---- ------ ----
!
!      IF ( B1Bvnnb .NE. amb_1bvnnb ) THEN
!         errStatus = -11  ! warning
!         errMessage = " "
!         WRITE ( errMessage,
!     &   "("" Code expects 1B format "",I6,"" but mhs1b.h is "",I6)")
!     &        B1Bvnnb, amb_1bvnnb
!         CALL ErrorReport ( errStatus, errMessage, "INMHSHDR")
!      ENDIF

! 1.2 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( B1Cvnnb .NE. amb_1cvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but amb1c.h is "",I6)")
     &        B1Cvnnb, amb_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "INBHDR")
      ENDIF

!! 1.3 Is the 1B data compatible ?
!!     -- --- -- ---- ----------
!!     "MHS1B.H" contains its own version number "mhs_h_l1bversnb"
!      IF ( mhs_h_l1bversnb .NE. amb_1bvnnb ) THEN
!         errStatus = -13  ! warning
!         errMessage = " "
!         WRITE ( errMessage,
!     &   "("" Format of level 1B MHS file is "",I8,"", not "",I8)")
!     &        mhs_h_l1bversnb,amb_1bvnnb
!         CALL ErrorReport ( errStatus, errMessage, "INMHSHDR")
!      ENDIF

! 1.4 Is the 1B data MHS ?
!     -- --- -- ---- ---
!     "mhs_h_datatyp" from 1B header should be 12 for MHS
      IF ( mhs_h_datatyp .NE. 12 ) THEN
         ErrStatus = 14
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Data type in 1B MHS file is "",I8,
     &     ""; not 12 for MHS"")")    mhs_h_datatyp
         CALL ErrorReport ( errStatus, errMessage, "INMHSHDR")
      ENDIF


! 2.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "##########"
      WRITE(*,*) "# MHS    #"
      WRITE(*,*) "##########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1B file, format version "",3I6)")
     &            mhs_h_l1bversnb, mhs_h_l1bversyr, mhs_h_l1bversdy
      WRITE(*,"(I4,"" header records in file: "",A42)")
     &            mhs_h_hdrcnt, mhs_h_dataname

!     calculate start and end dates/times from header:

!     ..start
      year = mhs_h_startdatayr
      yrday = mhs_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(mhs_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = mhs_h_enddatayr
      yrday = mhs_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(mhs_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines of which "",I6,
     &          "" earth-located and "",I6,"" missing."")")
     &          mhs_h_scnlin, mhs_h_callocsclin, mhs_h_misscnlin
      WRITE(*,*)


! 3.  Extract required information from MHS 1B header
!     ------- -------- ----------- ---- --- -- ------

! 3.1 temperature-radiance conversion  (Planck function - bandwidth corrected)
!                  order of channels = 1,2,3,4,5
!                      10^6 x central wavenumber
!                      10^6 x constant 1
!                      10^6 x constant 2

      DO chn = 1, Bnchan
         chn3 = chn*3

!        Use wave number to calculate channel-dependent constants for Planck fn
         wave_no = REAL(mhs_h_temrad(chn3-2)) * 1.E-06

!       (formula from Houghton "The Physics of Atmospheres",
!                   Cambridge Univ Press, p236, 2nd Edition - 1986)
!        - calculated radiances are W m(-2) sr(-1) cm(-1)
         BPlanck(1,chn) = C_1 * wave_no * wave_no * wave_no
         BPlanck(2,chn) = C_2 * wave_no

!        Store bandwidth corrections to adjust the brightness temperatures
         BPlanck(3,chn) = REAL(mhs_h_temrad(chn3-1)) * 1.E-06
         BPlanck(4,chn) = REAL(mhs_h_temrad(chn3))   * 1.E-06

      ENDDO  ! chn


! 4.  Set up level 1C header
!     --- -- ----- -- ------

!!!   This code is compatible with files:
!!!   "mhs1b.h"
!!!   "amb1c.h" vn1.9       10/03/98              Richard Renshaw (UKMO)

!     creation site for 1C data
      amb1c_h_site   = SiteID              !file creation site ID (CHAR*3)
      amb1c_h_cfill1 = " "                 !filler

!     creation site for original 1B data
!        - format starts "XXX.YY..." and we want XXX
      amb1c_h_1bsite = mhs_h_dataname(1:3)
      amb1c_h_cfill2 = " "                 !filler

      amb1c_h_versnb = amb_1cvnnb          !level 1C format version number
      amb1c_h_versyr = amb_1cvnyr          !level 1C format version year
      amb1c_h_versdy = amb_1cvndy          !level 1C format version day of year
      amb1c_h_hdrcnt = 1 !count of header records in 1C file  - set to 1

!! Convert satellite id from NESDIS 1B code to e.g. 14 for NOAA-14
      sat_index = 0
      DO isat = 1, MaxSats
         IF ( mhs_h_satid .EQ. NESDISid(isat) ) THEN
            amb1c_h_satid = NOAAid(isat)
            amb1c_h_wmosatid = WMOid(isat)
            sat_index = isat
         ENDIF
      ENDDO  ! isat
      IF ( sat_index .EQ. 0 ) THEN
         errStatus = 40  ! fatal
         errMessage = " "
         WRITE ( errMessage,"("" MHS 1B satid not recognised: "",
     &           I8)") mhs_h_satid
         CALL ErrorReport ( errStatus, errMessage, "INBHDR")
      ENDIF

      amb1c_h_instrument = mhs_h_datatyp   !data type code (MHS=12)

      amb1c_h_satht = NINT(SatHt(sat_index)*10.)!nominal sat height, km*10
      amb1c_h_period = NINT(OrbitPd(sat_index)) !nominal orbit period (seconds)

! Derive orbit numbers from 1B "processing block id":
! "mhs_h_prblkid" is of the format "Bxxxxxyy" where "xxxxx" is the orbit number
! at start of file, and "yy" is the final two digits at end of file
      READ(mhs_h_prblkid,"(1X,I5)") amb1c_h_startorbit !orbit number (at start)
! derive orbit number at end of file
      READ(mhs_h_prblkid,"(4X,I2)") sttorbit ! last 2 digits, start orbit no.
      READ(mhs_h_prblkid,"(6X,I2)") endorbit ! last 2 digits, final orbit no.
!      READ(mhs_h_prblkid,"(3X,I2)") sttorbit ! last 2 digits, start orbit no.
!      READ(mhs_h_prblkid,"(5X,I2)") endorbit ! last 2 digits, final orbit no.
      amb1c_h_endorbit = endorbit + 100*(amb1c_h_startorbit/100)
      IF ( sttorbit.GT.endorbit )
     &     amb1c_h_endorbit = amb1c_h_endorbit + 100

      amb1c_h_startdatayr = mhs_h_startdatayr!start of data set year
      amb1c_h_startdatady = mhs_h_startdatady! (ditto..) day of the year
      amb1c_h_startdatatime = mhs_h_startdatatime !(ditto..) UTC time of day,ms
      amb1c_h_enddatayr = mhs_h_enddatayr!end of data set year
      amb1c_h_enddatady = mhs_h_enddatady !(ditto..) day of the year
      amb1c_h_enddatatime = mhs_h_enddatatime !(ditto..) UTC time of day,ms

! These two set later:
!     amb1c_h_scnlin = mhs_h_scnlin   !count of scan lines in this data set
!     amb1c_h_misscnlin = mhs_h_misscnlin !count of missing scan lines

      amb1c_h_spare = 0               !spare

! temperature-radiance conversion coefficients
!                  order of channels = 1,2,3,4,5
!                      10^6 x central wavenumber
!                      10^6 x constant 1
!                      10^6 x constant 2
      DO chn = 1, Bnchan
         chn3 = chn*3
         amb1c_h_tempradcnv(1,chn) = mhs_h_temrad(chn3-2)
         amb1c_h_tempradcnv(2,chn) = mhs_h_temrad(chn3-1)
         amb1c_h_tempradcnv(3,chn) = mhs_h_temrad(chn3)
      ENDDO  ! chn

      DO i = 1, 1104
         amb1c_h_filler(i) = 0    !filler
      ENDDO


      RETURN
      END
