!+ process NEDT for MHS
!
      SUBROUTINE INMHSPRC_NEDT
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inmhsprc_nedt.F 459 2015-09-11 14:55:32Z frna $
!
! Purpose: Compute NEDT for each scan line. Based on EUMETSAT's method.
!
! Reference: EUM/OPS/TEN/11/3488, 2011, "Technical note for design ECP 299"
!
! Method: The NEDT used here is the weighted standard deviation of 7 scan lines
!         of cold and warm counts, scaled by the gain.
!
! Current Code Owner: N.C.Atkinson
!
! History:
! Version  Date      Comment
!
!  1.0  10/09/2015  Initial. Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  AMSU-B/MHS Data Variables for processing within ATOVIN 
#include "inbstore.h"
!  MHS level 1B header/data common blocks 
#include "mhs1b.h"
!  Constants 
#include "constants_preproc.h"
!
! Local scalars:
      INTEGER iscan, jscan, ichan, weightsum, i
      LOGICAL okscan(7)
      INTEGER nscan(Bnchan)

      REAL coldcal_mean, coldcal_sd
      REAL warmcal_mean, warmcal_sd
      REAL a0, a1, a2
      REAL drdc, dcdt, wave_no
      REAL drdt(Bnchan)   !Slope of Planck function

      REAL weights(7)
      DATA weights/1,2,3,4,3,2,1/
!
!- End of header

      DO ichan = 1,Bnchan
        wave_no = mhs_h_temrad(ichan*3-2) * 1.E-06
        drdt(ichan) = C_1/C_2 * wave_no * wave_no
        BcoldNEDT_rms(ichan) = 0.0
        BwarmNEDT_rms(ichan) = 0.0
        nscan(ichan) = 0
      ENDDO

!Loop over scans and channels

      DO iscan=1,mhs_h_scnlin
        DO ichan = 1,Bnchan
          DO jscan=-3,3
            okscan(jscan+4) = .false.
          ENDDO

!Check which nearby scans are usable for this channel
!If one values is missing, discard the scan

          weightsum = 0
          DO jscan=-3,3
            IF (iscan+jscan.GT.0.AND.iscan+jscan.LE.mhs_h_scnlin) THEN
              IF (Bcoldcals(1,ichan,iscan+jscan).NE.0 .AND.
     &            Bcoldcals(2,ichan,iscan+jscan).NE.0 .AND.
     &            Bcoldcals(3,ichan,iscan+jscan).NE.0 .AND.
     &            Bcoldcals(4,ichan,iscan+jscan).NE.0 .AND.
     &            Bwarmcals(1,ichan,iscan+jscan).NE.0 .AND.
     &            Bwarmcals(2,ichan,iscan+jscan).NE.0 .AND.
     &            Bwarmcals(3,ichan,iscan+jscan).NE.0 .AND.
     &            Bwarmcals(4,ichan,iscan+jscan).NE.0 .AND.
     &            Bcalibs(2,ichan,iscan+jscan).GT.0 ) THEN
                okscan(jscan+4) = .true.
                weightsum = weightsum + weights(jscan+4)
              ENDIF
            ENDIF
          ENDDO           
        
!Get weighted mean counts

          IF (weightsum .gt. 0) THEN
            coldcal_mean = 0.0
            warmcal_mean = 0.0
            DO jscan=-3,3
              IF (okscan(jscan+4)) THEN
                DO i=1,4
                  coldcal_mean = coldcal_mean + weights(jscan+4)*             
     &              Bcoldcals(i,ichan,iscan+jscan)
                  warmcal_mean = warmcal_mean + weights(jscan+4)*            
     &              Bwarmcals(i,ichan,iscan+jscan)
                ENDDO
              ENDIF
            ENDDO
            coldcal_mean = coldcal_mean/(4*weightsum)                
            warmcal_mean = warmcal_mean/(4*weightsum)               

!Get weighted std dev of counts

            coldcal_sd = 0.0
            warmcal_sd = 0.0
            DO jscan=-3,3
              IF (okscan(jscan+4)) THEN
                DO i=1,4
                  coldcal_sd = coldcal_sd + weights(jscan+4)*             
     &              (Bcoldcals(i,ichan,iscan+jscan)-coldcal_mean)**2
                  warmcal_sd = warmcal_sd + weights(jscan+4)*            
     &              (Bwarmcals(i,ichan,iscan+jscan)-warmcal_mean)**2
                ENDDO
              ENDIF
            ENDDO
            coldcal_sd = SQRT(coldcal_sd/(4*weightsum))                
            warmcal_sd = SQRT(warmcal_sd/(4*weightsum))                

!Get gain from the a0, a1, a2 coefs

            a0 = Bcalibs(3,ichan,iscan)
            a1 = Bcalibs(2,ichan,iscan)
            a2 = Bcalibs(1,ichan,iscan)
            IF (a1 .GT. 0) THEN
              drdc = a1 - 2*a2*a0/a1       !slope at zero K (good approximation)
              dcdt = drdt(ichan)/drdc      !gain in counts per K

!Get NEDT

              BcoldNEDT(ichan,iscan) = coldcal_sd/dcdt                 
              BwarmNEDT(ichan,iscan) = warmcal_sd/dcdt
              BcoldNEDT_rms(ichan) = BcoldNEDT_rms(ichan) + 
     &             BcoldNEDT(ichan,iscan)**2
              BwarmNEDT_rms(ichan) = BwarmNEDT_rms(ichan) + 
     &             BwarmNEDT(ichan,iscan)**2
              nscan(ichan) = nscan(ichan) + 1
            ELSE
              BcoldNEDT(ichan,iscan) = -1.0                 
              BwarmNEDT(ichan,iscan) = -1.0
            ENDIF
          ELSE
            BcoldNEDT(ichan,iscan) = -1.0    !if no good samples
            BwarmNEDT(ichan,iscan) = -1.0
          ENDIF 

        ENDDO   ! channels
      ENDDO   ! scans

      DO ichan=1,Bnchan
        IF (nscan(ichan) .GT. 0) THEN
          BcoldNEDT_rms(ichan) = SQRT(BcoldNEDT_rms(ichan)/nscan(ichan))
          BwarmNEDT_rms(ichan) = SQRT(BwarmNEDT_rms(ichan)/nscan(ichan))
        ELSE
          BcoldNEDT_rms(ichan) = -1.0
          BwarmNEDT_rms(ichan) = -1.0
        ENDIF
      ENDDO
      END
