!+  process MSU scanline - level 1b to 1c
!
      SUBROUTINE INMPRC ( Write_data )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inmprc.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Convert counts to radiances to brightness temperatures
!   and perform some quality control.
!
! Method:
! 1.  Check scanline number & time of scanline
! 2.  Act on flags for the scanline from the 1B file
! 3.  Loop through scanline and channels, converting counts to BTs, with qc
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0    19/3/96    Original code. (R.J.Renshaw)
!  1.1    15/04/96   Bit 0 of Data Quality flagwords and calibration flagword
!                    now set when appropriate     - RJR
!                    Calibration flag now for all fovs and channels in scanline
!  1.2    18/12/96   bug fixed: msu1c_dataqual now msu1c_dataqual(fov)
!  1.3    12/06/98   Mcycle REAL not INTEGER
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  MSU Data Variables for processing within ATOVIN 
#include "inmstore.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!
! Subroutine arguments:
!
!   Scalar arguments with intent(out)
      LOGICAL       Write_data    ! .TRUE. if line is to be written out
!
! Local parameters:
      INTEGER       mx_dTime      ! upper limit for departure of scan time
         PARAMETER (mx_dTime=50)  ! from nominal value  (milliseconds)
!
! Local Scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       cal_set       ! use 1=primary or 2=secondary cal coeffs
      INTEGER       fov           ! loop variable over all fovs
      REAL          counts        ! averaged PRT counts from the instrument
      REAL          radiance      ! radiance calculated from counts
      REAL          bri_temp      ! brightness temperature calc from radiance
      LOGICAL       good_bts      ! .FALSE. if all channel BTs for the fov bad
      LOGICAL       good_data     ! .TRUE. if line has brightness temperatures
!
      INTEGER       prev_line     ! line number of previous scanline
         DATA       prev_line / MDI /
      INTEGER       prev_year     ! year of previous scanline
         DATA       prev_year / 0 /
      INTEGER       prev_day      ! day of previous scanline
         DATA       prev_day / 0 /
      INTEGER       prev_time     ! time of previous scanline
         DATA       prev_time / 0 /
      INTEGER       dTime         ! time difference returned from s/r TimeSub
      INTEGER       errTime       ! error status returned from s/r TimeSub
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
      External TimeSub              ! subtract two dates/times
!
      SAVE prev_line, prev_year, prev_day, prev_time
!
!- End of header


! 1.  Check scanline number & time of scanline
!     ----- -------- ------ - ---- -- --------

! 1.1 Innocent until proven guilty..
      good_data  = .TRUE.   ! good brightness temperatures for this line
      Write_data = .TRUE.   ! do process & write out this scanline

! 1.2 When scanlines are incrementing properly, check time..
      IF ( msu1c_scnlin .EQ. prev_line+1 ) THEN
         CALL TimeSub
     &        ( msu1c_scnlinyr, msu1c_scnlindy, msu1c_scnlintime,
     &          prev_year, prev_day, prev_time, dTime, errTime )
         dTime = dTime - NINT(Mcycle)
         IF ( errTime.NE.0  .OR. ABS(dTime).GT.mx_dTime ) THEN
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Jump in successive date/time:"",6I8)" )
     &           prev_year,prev_day,prev_time,
     &           msu1c_scnlinyr, msu1c_scnlindy, msu1c_scnlintime
            errStatus = -1  ! warning
            CALL ErrorReport(errStatus,errMessage,"INMPRC")
         ENDIF ! (errTime.NE.0 .OR. ..dTime out of range..)

! 1.3 Else, check that scanline numbers aren't going backwards..
      ELSEIF ( msu1c_scnlin .LE. prev_line ) THEN
         errMessage = " "
         WRITE ( errMessage,
     &   "("" line numbers going backwards:"",I8,"" follows "",I8)")
     &        msu1c_scnlin, prev_line
         errStatus = 2   ! fatal
         CALL ErrorReport(errStatus,errMessage,"INMPRC")

! 1.4 If this wasn't the first scanline, then we've missed one..
      ELSEIF (prev_line.NE.MDI) THEN   !  we"ve skipped a line(s)
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Found scanline "",I8,"" ; expected "",I8)")
     &        msu1c_scnlin, prev_line+1
         errStatus = -3  ! warning
         CALL ErrorReport(errStatus,errMessage,"INMPRC")

      ENDIF  ! (msu1c_scnlin = prev_line+1)

! 1.5 save values for next scanline
      prev_line = msu1c_scnlin
      prev_year = msu1c_scnlinyr
      prev_day = msu1c_scnlindy
      prev_time = msu1c_scnlintime


! 2.  Act on flags for the scanline from the 1B file
!     --- -- ----- --- --- -------- ---- --- -- ----

! 2.1 Set choice of calibration coefficients from "scanline quality flags"

      IF ( BTEST( msu1c_scanqual, bits(15) ) ! not calibrated, bad time
     & .OR.BTEST( msu1c_scanqual, bits(13) ) ! not calibrated, bad PRT
     & .OR.BTEST( msu1c_scanqual, bits(10) ) ! uncalibrated, instrument mode
     & ) THEN

         cal_set = 2 ! use secondary set of calibration coefficients
         DO fov = 1, Mnfovs
            msu1c_dataqual(fov) = IBSET( msu1c_dataqual(fov), bits(30) )
         ENDDO

      ELSE

         cal_set = 1 ! use primary set of calibration coefficients

      ENDIF

! 2.2 "quality indicator": Cannot use any information from these fovs

      IF ( BTEST( msu1c_qualind, bits(30) ) ! time sequence error
     & .OR.BTEST( msu1c_qualind, bits(27) ) ! no earth location
     & ) THEN

         good_data  = .FALSE.  ! no brightness temperatures for this line
         Write_data = .FALSE.  ! don't write out this line

      ENDIF

! 2.3 "quality indicator": No BTs but can still map data to these fovs

      IF ( BTEST( msu1c_qualind, bits(31) ) ! don't use for product generation
     & .OR.BTEST( msu1c_qualind, bits(28) ) ! no calibration
     &  ) THEN

         good_data  = .FALSE.  ! no brightness temperatures for this line

!        Set BTs to missing and flag them
         DO fov = 1, Mnfovs
            DO chn = 1, Mnchan
               msu1c_btemps(chn,fov) = MDI
            ENDDO  ! chn
            msu1c_dataqual(fov) = IBSET(msu1c_dataqual(fov),bits(0))
         ENDDO  ! fov

      ENDIF


! 3.  Loop through scanline and channels, converting counts to BTs, with qc
!     ---- ------- -------- --- --------- ---------- ------ -- ---- ---- --

      IF ( good_data .AND. Write_data ) THEN
         DO fov = 1, Mnfovs

!           Loop through the channels
            good_bts = .FALSE.
            DO chn = 1, Mnchan

! 3.1          Correct counts for non-linearity
               counts = REAL(Mcounts(chn,fov))
               IF (ABS(counts).LT.9999.) THEN       ! (check data)
                  counts = Mnonlin(3,chn)
     &                   + Mnonlin(2,chn)*counts
     &                   + Mnonlin(1,chn)*counts*counts
               ENDIF  ! counts

! 3.2          Convert counts to radiance (W/m2)
               radiance = -999.
               IF (ABS(counts).LT.9999.) THEN       ! (check data)
                  radiance = Mcalib(cal_set,3,chn)
     &                     + Mcalib(cal_set,2,chn)*counts
     &                     + Mcalib(cal_set,1,chn)*counts*counts
               ENDIF  ! counts

! 3.3          Convert radiance to brightness temperature (inverse Planck)
!              and modify to allow for bandwidth of channels
               bri_temp = -999.
               IF (radiance.GT.0.) THEN             ! (check data)
                  bri_temp = MPlanck(2,chn)         ! Kelvin
     &                     / log( 1. + MPlanck(1,chn)/radiance )
                  bri_temp = (bri_temp-MPlanck(3,chn)) / MPlanck(4,chn)
               ENDIF  ! radiance

! 3.4          Quality control final brightness temperature
               IF ( bri_temp.GT.MminBT .AND. bri_temp.LT.MmaxBT ) THEN
                  msu1c_btemps(chn,fov) = NINT ( bri_temp*100. )  ! 100*deg K
                  good_bts = .TRUE.
               ELSE
!                 Set BT to missing, and flag it
                  msu1c_btemps(chn,fov) = MDI
                  msu1c_dataqual(fov) =
     &                           IBSET(msu1c_dataqual(fov),bits(chn))
               ENDIF  ! bri_temp

            ENDDO  ! chn

!           If all BTs are bad, set bit 0 of data quality flagword
            IF ( .NOT. good_bts ) msu1c_dataqual(fov) =
     &                         IBSET(msu1c_dataqual(fov),bits(0))

         ENDDO  ! fov
      ENDIF  ! good_data .AND. Write_data


      RETURN
      END
