!+ process MSU data within ATOVIN
!
      SUBROUTINE INMSU ( M1Bunit, M1Cunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inmsu.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Controlling routine to read level 1b MSU data,
!   convert MSU electronic counts to radiances, then to
!   brightness temperatures, and write out as level 1c.
!
! Method:
! 1.  Open MSU 1B & 1C files
! 2.  Read 1B header & set up 1C header
! 3.  Loop through scan lines, reading, processing & writing data
! 4.  Complete & write out 1C header
! 5.  Close files
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0     19/3/96  Original code. (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVIN Processing 
#include "inparms.h"
!  MSU Data Variables for processing within ATOVIN 
#include "inmstore.h"
!  MSU level 1B header/data common blocks 
#include "msu1b.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!
!   Scalar arguments with intent(In):
      INTEGER       M1Bunit       ! unit number for MSU level 1B file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
!
! Local scalars:
      INTEGER       first_lineno  ! number of first scanline in the 1B file
      INTEGER       last_lineno   ! number of last scanline in the 1B file
      INTEGER       M1Brecord     ! record number for MSU level 1B file
      INTEGER       M1Crecord     ! record number for MSU level 1C file
      INTEGER       scnlin        ! loop variable over scanlines
      LOGICAL       first_line    ! true for first scanline in the 1B file
         DATA       first_line / .TRUE. /
      LOGICAL       write_data    ! .TRUE. if line is to be written out
!
! Function & Subroutine calls:
      External INMHDR               ! interpret MSU level 1B header into 1C
      External INMGET               ! read MSU 1B scan line
      External INMPRC               ! process MSU scan line
      External IOM1B                ! I/O routine for level 1B MSU data
      External IOM1C                ! I/O routine for level 1C MSU data
!
!- End of header


! 1.  Open MSU 1B & 1C files
!     ---- ---- -- - -- -----

      CALL IOM1B ("OPEN/READ", M1Bunit, M1Brecord )    ! 1B file
      CALL IOM1C ("OPEN/WRITE", M1Cunit, M1Crecord )   ! 1C file


! 2.  Read 1B header & set up 1C header
!     ---- -- ------ - --- -- -- ------

! 2.1 Read 1B header
      M1Brecord = 1   ! Headers start on record 1.
      CALL IOM1B ("READ HEADER", M1Bunit, M1Brecord )

!     point to last header record in 1B
      M1Brecord = msu_h_hdrcnt

! 2.2 Set up 1C header
      CALL INMHDR

!     point to last header record in 1C
      M1Crecord = msu1c_h_hdrcnt


! 3.  Loop through scan lines, reading, processing & writing data
!     ---- ------- ---- ------ -------- ---------- - ------- ----

      DO scnlin = 1, msu_h_scnlin

! 3.1    Read a scanline
         M1Brecord = M1Brecord + 1
         CALL IOM1B ("READ DATA", M1Bunit, M1Brecord )

! 3.2    Transfer 1B data into program and 1C arrays
         CALL INMGET

! 3.3    Process the scanline
         CALL INMPRC ( write_data )

! 3.4    Write it out, if it contains any good data
         IF ( write_data ) THEN

            M1Crecord = M1Crecord + 1
            CALL IOM1C ("WRITE DATA", M1Cunit, M1Crecord )

!           Record the number of the first & last scanlines in the file
            IF ( first_line ) THEN
               first_lineno = msu1c_scnlin
               first_line = .FALSE.
            ENDIF
            last_lineno = msu1c_scnlin

         ENDIF

      ENDDO   ! (scnlin)


! 4.  Complete & write out 1C header
!     -------- - ----- --- -- ------

!     correct 1C header to show the correct number of scanlines
      msu1c_h_scnlin = M1Crecord - msu1c_h_hdrcnt

!     ..and the correct number of missing scanlines
      msu1c_h_misscnlin = (1+last_lineno-first_lineno) - msu1c_h_scnlin

!     write the 1C header
      M1Crecord = 1
      CALL IOM1C ("WRITE HEADER", M1Cunit, M1Crecord )

      WRITE(*,*) msu1c_h_scnlin," scanlines written to MSU 1C file"
      WRITE(*,*)


! 5.  Close files
!     ----- -----

      CALL IOM1B ("CLOSE", M1Bunit, M1Brecord )   ! 1B file
      CALL IOM1C ("CLOSE", M1Cunit, M1Crecord )   ! 1C file


      RETURN
      END
