!+  Compute solar zenith and azimuth angles
!
      SUBROUTINE INSUNA ( Year, Day, Time, N, Latitudes, Longitudes,
     &                    Azimuths, Zeniths )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: insuna.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Compute solar zenith and azimuth angles for an array of latitudes
!   and longitudes, for a given time and date.
!
! Method:
!   Air Almanac useful for checking GHA and DECL
!   Norton's Star Atlas for equation of time
!   Robinson N. Solar Radiation Ch 2 for useful introduction to
!   theory/terminology.
!
!   If input values are out of range, angles of zero will be returned.
!
! Current Code Owner: N.C.Atkinson
!
! History:
! Version  Date      Comment
!
!  1.01   30/10/90   R.W. Saunders
!  1.02   22/02/96   Argument errStatus added, some reformatting of code,
!                     & name changed from "S_SUN" for ATOVIN    - RJR
!  1.03   26/11/96   Argument errStatus removed   - RJR
!  1.04   19/12/97   Azimuth now -180/180 (was 0/360),
!                    and MDI multiplied by 100 to avoid confusion   - RJR
!  1.05   10/03/98   Azimuth now 0/360 as in BUFR   - RJR
!  1.06   15/09/04   Use constants_preproc.h for fundamental constants - NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Subroutine arguments:
!
!   Arguments with intent(in)
      INTEGER       Day                   ! Day in year (1-366)
      INTEGER       Year                  ! Year (eg 1984)
      INTEGER       Time                  ! Time GMT (millisecs from midnight)
      INTEGER       N                     ! number of points in arrays
      REAL          Latitudes(N)          ! latitudes (degrees, North +ve)
      REAL          Longitudes(N)         ! longitudes (degrees, East +ve)
!
!   Arguments with intent(out)
      REAL          Azimuths(N)           ! local solar azimuth angles, degrees
      REAL          Zeniths(N)            ! local solar zenith angles, degrees
      INTEGER       ErrStatus             ! error status (zero if all ok)
!
! Include:
#include "constants_preproc.h"
!
! Local Scalars:
      INTEGER       centday               ! century day
      INTEGER       i                     ! loop through spots 1 to N
      REAL          ang                   ! local hour angle
      REAL          azim                  ! solar azimuth angle
      REAL          cosaz                 ! cosine of solar azimuth
      REAL          decl                  ! solar declination (radians)
      REAL          eqnt                  ! "equation of time" (seconds)
      REAL          hour                  ! hour (+fraction) of day
      REAL          lat                   ! latitude
      REAL          lon                   ! longitude
      REAL          localtime             ! local solar time (hrs)
      REAL          rcd                   !fraction of days elapsed this century
      REAL          rcd2                  ! "rcd" squared
      REAL          rcosdc                ! cosine of solar declination
      REAL          rsindc                ! sine of solar declination
      REAL          rcosev                ! cosine of elevation angle
      REAL          rsinev                ! sine of elevation angle
      REAL          rcoslt                ! cosine of latitude
      REAL          rsinlt                ! sine of latitude
      INTEGER prev_day                    ! day from previous call
         DATA prev_day/0/
! Don't know what these are! :
      REAL sinalp, taneqn, ydeg, yrad
!
      SAVE prev_day, eqnt, rcosdc, rsindc
!
!- End of header

      Errstatus = 0     ! no problem yet
!
!   Perform range checking for inputs
      IF (( Time .GE. 0 .AND. Time .LE. 86400000) .AND.
     &   ( Year .GT. 1950 .AND. Year .LT. 2200 ) .AND.
     &   ( Day .GE. 1 .AND. Day .LE. 366) ) THEN
!
!________________________________________________________________________
!        Only call this section once per day
         IF ( prev_day .NE. Day)THEN
!
!           Century day (ie no. of days since 0-JAN-1900) - 2000 is a leap year
            centday = (Year-1900)*365 + Day + (Year-1901)/4
!
            rcd = REAL(centday) / 36525. ! Fraction of days elapsed this century
            rcd2 = rcd*rcd
            ydeg = (rcd * 36000.769 + 279.697) / 360.
            ydeg = MOD( ydeg , 1. ) * 360.
            yrad = ydeg * deg2rad
!
!       Compute equation of time (in seconds) for this day
!       (No reference for this but it gives the correct answers
!       when compared with table in Norton's Star Atlas)
!
            eqnt = - ( (93.0+14.23*rcd-0.0144*rcd2) * SIN(yrad) )
     &             - ( (432.5-3.71*rcd-0.2063*rcd2) * COS(yrad) )
     &             + ( (596.9-0.81*rcd-0.0096*rcd2) * SIN(2.*yrad) )
     &             - ( (1.4+0.28*rcd)               * COS(2.*yrad) )
     &             + ( (3.8+0.6*rcd)                * SIN(3.*yrad) )
     &             + ( (19.5-0.21*rcd-0.0103*rcd2)  * COS(3.*yrad) )
     &             - ( (12.8-0.03*rcd)              * SIN(4.*yrad) )
!
!           Get solar declination for given day (radians)
            sinalp = SIN((ydeg-eqnt/240.) * deg2rad)
            taneqn = 0.43382 - 0.00027*rcd
            decl = ATAN(taneqn*sinalp)
            eqnt = eqnt / 3600.                    ! Convert to hours
!
!           Sine and cosine of declination
            rsindc = SIN(decl)
            rcosdc = COS(decl)
!
            prev_day = Day
         ENDIF
!________________________________________________________________________
!
!        Loop through spots
         DO i = 1, N
            lat = Latitudes(i)
            lon = Longitudes(i)
!           check co-ordinates
            IF  (  lat.GE.-90.  .AND. lat.LE.90.
     &       .AND. lon.GE.-180. .AND. lon.LE.180. ) THEN

               rsinlt = SIN(lat*deg2rad)
               rcoslt = COS(lat*deg2rad)
               hour = REAL(Time) / 3600000. ! Convert msecs elapsed to GMT

!              Calculate solar zenith (degrees)
               localtime = lon/15. + eqnt + hour   ! Local solar time (hrs)
               ang = ( localtime*15. + 180. ) * deg2rad  ! Local hour angle
!              (when longitude is 0 this is the GHA given in the Air Almanac)
               rsinev = rsindc*rsinlt + rcosdc*rcoslt*COS(ang)
               rcosev = SQRT(1. - rsinev*rsinev)
               Zeniths(i) = (PI/2. - ASIN(rsinev)) * rad2deg
!
!              Calculate solar azimuth (degrees)
               cosaz = ( rsindc - rsinev*rsinlt) / (rcoslt*rcosev)
               cosaz = MAX ( cosaz, -1. )
               cosaz = MIN ( cosaz, 1. )
               azim = ACOS(cosaz)
               IF (MOD(localtime+72.,24.) .GE. 12.) azim = 2.*PI - azim
!               IF ( azim.GT.PI ) azim = 2.*PI - azim
               Azimuths(i) = azim * rad2deg

            ELSE
               Zeniths(i) = -9999.99       ! set to missing
               Azimuths(i) = -9999.99
            ENDIF   ! lat & lon ok

         ENDDO  ! i
!
      ELSE  ! input values out of range
         DO i = 1, N
            Zeniths(i) = -9999.99       ! set to missing
            Azimuths(i) = -9999.99
         ENDDO  ! i
      ENDIF
!
      RETURN
      END
