!+ read which instruments to process and set unit numbers
!
      SUBROUTINE INUSER ( LHIRS, LAMSA, LAMSB, LMSU,
     &                    H1Bunit, H1Cunit,
     &                    A1Bunit, A1Cunit,
     &                    B1Bunit, B1Cunit,
     &                    M1Bunit, M1Cunit,
     &                    FDFunit)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: inuser.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Read from standard input a list of instruments to process.
!   Set unit numbers for input/output files and for fixed data file.
!
! Method:
! 0.  Initialisation
! 1.  Which instruments to process ?
!   Format of input is specified in AAPP documentation APP203U, vn of 12/12/96
!   Note that input data with "!" in the first column are ignored.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0     22/2/96  Original code. (R.J.Renshaw)
!  1.1     19/3/96  Process ATOVS and TOVS.  RJR
!  1.2     12/12/96 User input simplified, following ATOVPP format.
!  1.3     24/11/04 Add MHS. (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Global variables:
!
      INTEGER       Ninstruments         ! number of instruments recognised:
      PARAMETER   ( Ninstruments = 5 )   !   HIRS, AMSU-A, AMSU-B, MSU, MHS
!
! Subroutine arguments
!   Scalar arguments with intent(out):
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B/MHS
      LOGICAL       LMSU          ! .TRUE. to process MSU
      INTEGER       H1Bunit       ! unit number for HIRS level 1B file
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
      INTEGER       A1Bunit       ! unit number for AMSU-A level 1B file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       B1Bunit       ! unit number for AMSU-B/MHS level 1B file
      INTEGER       B1Cunit       ! unit number for AMSU-B/MHS level 1C file
      INTEGER       M1Bunit       ! unit number for MSU level 1B file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
      INTEGER       FDFunit       ! unit number for fixed data file
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      INTEGER       errStatus     ! error status
                                  ! +ve => fatal error, -ve => warning
      CHARACTER*80  errMessage    ! error message
      INTEGER       i             ! loop variable over instruments
      INTEGER       indx          ! index to instrument in input_line
      INTEGER       maxindx       ! upper limit on indx
      INTEGER       ninst         ! number of instruments found
!
! Local Arrays:
      INTEGER       inst(Ninstruments)       ! list of instruments found
      INTEGER       leninstr(Ninstruments)   ! lengths of instrument names
      CHARACTER*6   cinst, cinstrument(Ninstruments) ! instrument names
      DATA cinstrument / "HIRS", "AMSU-A", "AMSU-B", "MSU", "MHS" /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN error routine
      External C2Upper            ! convert character string to upper case
!
!- End of header


! 0.  Initialisation
!     --------------

! 0.1 whether to read and process these instruments:
      LHIRS = .FALSE.
      LAMSA = .FALSE.
      LAMSB = .FALSE.
      LMSU  = .FALSE.

! 0.2 Define unit numbers
! (these could be read in if there is a real requirement for
!  unit numbers to be different for different runs of ATOVIN)

!     level 1b datasets
      H1Bunit = 11      ! unit number for HIRS level 1C file
      A1Bunit = 12      ! unit number for AMSU-A level 1C file
      B1Bunit = 13      ! unit number for AMSU-B level 1C file
      M1Bunit = 14      ! unit number for MSU level 1C file

!     level 1c datasets
      H1Cunit = 21      ! unit number for HIRS level 1C file
      A1Cunit = 22      ! unit number for AMSU-A level 1C file
      B1Cunit = 23      ! unit number for AMSU-B level 1C file
      M1Cunit = 24      ! unit number for MSU level 1C file

!     fixed data file
      FDFunit  = 10

! 0.3 count the length of the instrument names
      maxindx = LEN(cinstrument(1)) ! upper limit for indx
      DO i = 1, Ninstruments
         cinst = cinstrument(i)
         indx = maxindx   ! point to last character in string
         DO WHILE (cinst(indx:indx).EQ." ")
            indx = indx - 1
         ENDDO  ! WHILE
         leninstr(i) = indx
      ENDDO  ! i
      maxindx = LEN(input_line)+1 - LEN(cinstrument(1)) ! upper limit for indx


! 1.  Which instruments to process ?
!     ----- ----------- -- -------

!     read the whole line into memory  (ignoring lines starting "!")
      input_line = "!"
      errStatus = 0
      errMessage = "Error reading user input: input instruments"
      DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
         READ(*,"(A80)",IOSTAT=errStatus) input_line
      ENDDO

!     check for errors
      errStatus = ABS (errStatus)   ! end-of-file returns -1, which isn't fatal
      CALL ErrorReport(errStatus,errMessage,"INUSER")

!     convert to upper case
      CALL C2Upper ( input_line )

!     look through the whole string for instrument names, stopping at "!"
      indx = 1   ! pointer to character in input_line
      ninst = 0  ! number of instrument names found
      DO WHILE ( indx.LT.maxindx
     &           .AND. input_line(indx:indx+1).NE."!" )
         DO i = 1, Ninstruments
            IF  ( input_line(indx:indx+leninstr(i)-1)
     &           .EQ. cinstrument(i) ) THEN      ! found it here
                 ninst = ninst + 1
                 inst(ninst) = i
                 indx = indx + leninstr(i) - 1   ! move past this instrument
            ENDIF
         ENDDO  ! i
         indx = indx + 1
      ENDDO  ! WHILE

      IF ( ninst.EQ.0 ) THEN
         errMessage = "No instruments requested for processing"
         errStatus = 10
         CALL ErrorReport(errStatus,errMessage,"INUSER")
      ENDIF

!     see which instruments have been requested
      DO i = 1, ninst
         IF     ( inst(i).EQ.1 ) THEN
            LHIRS = .TRUE.
         ELSEIF ( inst(i).EQ.2 ) THEN
            LAMSA = .TRUE.
         ELSEIF ( inst(i).EQ.3 ) THEN
            LAMSB = .TRUE.
         ELSEIF ( inst(i).EQ.4 ) THEN
            LMSU  = .TRUE.
         ELSEIF ( inst(i).EQ.5 ) THEN
            LAMSB  = .TRUE.           !MHS is processed as for AMSU-B
         ENDIF
      ENDDO  !


      RETURN
      END
