! Name: $Id$
!> Some F77 routine in order to manage the 1C/1D input/output
!!
!> Current Code Owner: NWP SAF
!!
!> COPYRIGHT.
!!    This software was developed within the context of the EUMETSAT Satellite
!!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!!
!!    Copyright 2014, EUMETSAT, All Rights Reserved.
!!
!! @author P. Brunel
!! @date October, 2014
!!
! History:
! Version  Date      Comment
!
!  1.0    01/10/2014 Creation
!  1.1    10/09/2018 Bug Fix read correct cris1c_h_recwords P. Roquet
!  
!> Get CrIS 1D information for a given field of view
       SUBROUTINE GET_CRIS1D_FOV( ifor, ifov, line, 
     &    scnlinyr, scnlindy, scnlintime,
     &    latitude, longitude, altitude)
         implicit none

! CRIS level 1D header/data common blocks
#include "cris1d.h"

       INTEGER IFOR ! input FOR
       
       ! Outputs
       INTEGER IFOV
       INTEGER LINE
       integer scnlin            !scan line number
       integer scnlinyr          !scan line year
       integer scnlindy          !scan line day of year
       integer scnlintime        !scan line UTC time of day (ms)

       real latitude
       real longitude
       real altitude
 
        line       = cris1d_scnlin
        ifov       = mod(cris1d_FOV(ifor)-1,9)+1
      
        scnlinyr   = cris1d_scnlinyr
        scnlindy   = cris1d_scnlindy
        scnlintime = cris1d_scnlintime(ifor)
        latitude   = cris1d_latlon(1,ifor) * 1.E-04
        longitude  = cris1d_latlon(2,ifor) * 1.E-04
        altitude   = cris1d_scalti * 1.E-01

      RETURN
      END

!> Get CrIS 1C information for a given field of regard and field of view
      SUBROUTINE GET_CRIS1C_FOV( ifor, ifov, line, 
     &    scnlinyr, scnlindy, scnlintime,
     &    latitude, longitude, altitude)
         implicit none

! CRIS level 1c header/data common blocks
#include "cris1c.h"

       INTEGER IFOR ! input FOR
       INTEGER IFOV ! input FOV

       ! Outputs
       INTEGER LINE
       integer scnlin            !scan line number
       integer scnlinyr          !scan line year
       integer scnlindy          !scan line day of year
       integer scnlintime        !scan line UTC time of day (ms)
       real latitude
       real longitude
       real altitude
        line       = cris1c_scnlin
        scnlinyr   = cris1c_scnlinyr
        scnlindy   = cris1c_scnlindy
        scnlintime = cris1c_scnlintime(ifor)
        latitude   = cris1c_latlon(1,ifov,ifor) * 1.E-04
        longitude  = cris1c_latlon(2,ifov,ifor) * 1.E-04
        altitude   = cris1c_scalti * 1.E-01

      RETURN
      END

!> Read CrIS 1C header and returns number of scans, FOR, FOV
      SUBROUTINE GET_CRIS1C_HEADER(C1Cunit, hdrcnt, recperscan, 
     &           nfor, nfov, nscan)
      implicit none
#include "cris1c.h"
      INTEGER C1Cunit
      INTEGER hdrcnt, recperscan, nfov, nfor, nscan

      CALL IOC1C ("READ HEADER",C1Cunit , 1 )

      hdrcnt     = cris1c_h_hdrcnt
      recperscan = 1
      nfor       = cris1c_h_nfor
      nfov       = cris1c_h_nfov
      nscan      = cris1c_h_scnlin
      RETURN
      END

!> Read CrIS 1D header and returns number of scans, FOR, FOV
!! and list of viirs channels already filled
      SUBROUTINE GET_CRIS1D_HEADER(C1Dunit, hdrcnt, recperscan, 
     &           nfor, nfov, nscan,
     &           nviirschan, nviirschanused, 
     &           viirschans, viirsbands )

      implicit none
#include "cris1d.h"
      INTEGER C1Dunit
      INTEGER hdrcnt, recperscan, nfov, nfor, nscan
      integer nviirschan       !number of VIIRS channels in format
      integer nviirschanused   !number of channels used
      integer viirschans(cris_1dnchanviirs)  !Channel numbers
      integer viirsbands(cris_1dnchanviirs)  !Band numbers
                                                  ! 5=M, 6=I, 7-DN

      integer i
      
      CALL IOC1D ("READ HEADER", C1Dunit, 1 )

      hdrcnt     = cris1d_h_hdrcnt
      recperscan = cris1d_h_recperscan
      nfor       = cris1d_h_nfor
      nfov       = cris1d_h_nfov
      nscan      = cris1d_h_scnlin
      
      nviirschan     = cris1d_h_nviirschan
      nviirschanused = cris1d_h_nviirschanused

      DO I = 1, cris_1dnchanviirs
        viirschans(I) = cris1d_h_viirschans(I)
        viirsbands(I) = cris1d_h_viirsbands(I)
      END DO

      RETURN
      END

!> Update CrIS 1D haeder with VIIRS information and write header 
      SUBROUTINE PUT_CRIS1D_HEADER(C1Dunit, 
     &  nviirschan, nviirschanused, 
     &  viirschans, viirsbands )
      implicit none
#include "cris1d.h"
      INTEGER C1Dunit
      integer nviirschan       !number of VIIRS channels in format
      integer nviirschanused   !number of channels used
      integer viirschans(cris_1dnchanviirs)  !Channel numbers
      integer viirsbands(cris_1dnchanviirs)  !Band numbers
                                                  ! 5=M, 6=I, 7-DN
      integer i

      cris1d_h_nviirschan = nviirschan
      cris1d_h_nviirschanused = nviirschanused

      DO I = 1, cris_1dnchanviirs
        cris1d_h_viirschans(I) = viirschans(I)
        cris1d_h_viirsbands(I) = viirsbands(I)
      END DO

      CALL IOC1D ("WRITE HEADER", C1Dunit, 1 )

      RETURN
      END

!> Open CrIS 1C/1D files.
!>- Try to open 1D file
!>- If fails then try to open 1C file
!>- Returns 1 logical for each file format 
      SUBROUTINE OPEN_CRIS(C1Cunit, C1Dunit, LCRIS1C)
      ! no need for return code as any error will turn to a stop
      ! via errorrepert and aapp_exit
      IMPLICIT NONE

#include "cris1c.h"
#include "cris1d.h"
#include "reclopt.h"

      INTEGER C1Cunit, C1Dunit    ! input 
      LOGICAL LCRIS1C             ! output, true if 1C file available
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      LOGICAL       OK
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine

      IF (cris1c_h_recwords .LE. 0) 
     &  cris1c_h_recwords = cris_1crecl/4

!     Check 1D file is available for READ/WRITE
      OPEN ( UNIT=C1Dunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=cris_1drecl/rldiv, IOSTAT=errStatus )

!     If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=C1Dunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=cris_1drecl/rldiv, IOSTAT=errStatus )
     
      WRITE ( errMessage,
     &   "("" OPEN/READ 1D CRIS file: UNIT="",I3,"" RECL="",I6)")
     &                                C1Dunit, cris_1drecl/rldiv

!     Try to read first header record (just to verify in case of Unknown)
      READ ( UNIT=C1Dunit,REC=1,IOSTAT=errStatus ) cris1d_hd
      OK = errStatus.EQ.0
      OK = OK .AND. cris1d_h_atovppvn .GE. 1        !ATOVPP version number (test vns = 9000+)
      OK = OK .AND. cris1d_h_instruments .GE. 1     ! bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B, bit5=AVHRR, bit 6=IASI,
!        bit7=ATMS, bit8=CrIS)

      IF  ( OK ) THEN
        LCRIS1C = .FALSE.
      ! check record size of the 1D file
       write(*,*) "cris1d_h_recwords:",cris1d_h_recwords
       IF (cris1d_h_recwords .ne. cris_1drecl*4/rldiv ) THEN
          WRITE(*,*) "WARNING: record size of 1d is ",cris1d_h_recwords,
     &               "not", cris_1drecl*4/rldiv,":close/open the file"
          CLOSE (UNIT=C1Dunit)
          OPEN (UNIT=C1Dunit, STATUS="OLD", FORM="UNFORMATTED", 
     &          ACCESS="DIRECT",RECL=cris1d_h_recwords*4/rldiv,
     &          IOSTAT=errStatus)
!         If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
          IF ( errStatus.NE.0 ) 
     &     OPEN ( UNIT=C1Dunit, STATUS="UNKNOWN",FORM="UNFORMATTED",
     &           ACCESS="DIRECT", RECL=cris1d_h_recwords*4/rldiv, 
     &           IOSTAT=errStatus)
          
       ENDIF

      ELSE
         CLOSE ( UNIT=C1Dunit )
         errStatus = 0
         write(*,*) "error with open 1d: trying now 1c"

	 !Check 1C file READ Only

         cris1c_h_recwords = 48    !first part of header only

         OPEN ( UNIT=C1Cunit, 
     &    STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &    RECL=cris1c_h_recwords*4/rldiv, IOSTAT=errStatus )

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 ) THEN
           write(*,*) "here we use status unknown"
           OPEN ( UNIT=C1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &     ACCESS="DIRECT", RECL=cris1c_h_recwords*4/rldiv,
     &     IOSTAT=errStatus )
         END IF

         CALL IOC1C("READ HEADER",C1Cunit,1)   !Get cris1c_h_recwords
         CALL IOC1C("CLOSE",C1Cunit,1)

         OPEN ( UNIT=C1Cunit, STATUS="OLD", FORM="UNFORMATTED",
     &    ACCESS="DIRECT", RECL=cris1c_h_recwords*4/rldiv,
     &    IOSTAT=errStatus )

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 ) THEN
           write(*,*) "here we use status unknown"
           OPEN ( UNIT=C1Cunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &     ACCESS="DIRECT", RECL=cris1c_h_recwords*4/rldiv,
     &     IOSTAT=errStatus )
         END IF

         WRITE ( errMessage,
     &   "("" OPEN/READ 1C CRIS file: UNIT="",I3,"" RECL="",I6)")
     &                                C1Cunit, cris_1drecl/rldiv
         LCRIS1C = .TRUE.
      ENDIF

      CALL ErrorReport ( errStatus, errMessage, "OPEN_CRIS")
 
      RETURN
      END

!> Close CrIS 1C/1D files
      SUBROUTINE CLOSE_CRIS(C1Cunit, C1Dunit, LCRIS1C)
      IMPLICIT NONE

      INTEGER C1Cunit, C1Dunit
      LOGICAL LCRIS1C

      IF( LCRIS1C ) THEN
        CALL IOC1C ("CLOSE", C1Cunit, 0)
      ELSE
        CALL IOC1D ("CLOSE", C1Dunit, 0)
      ENDIF

      RETURN
      END

!> Depending on logical LCris1C reads CrIS 1c or 1d record
      SUBROUTINE READ_CRIS(C1Cunit, C1Dunit, LCRIS1C, record)
      IMPLICIT NONE
      INTEGER C1Cunit, C1Dunit, record
      LOGICAL LCRIS1C

      IF( LCRIS1C ) THEN
        CALL IOC1C ("READ DATA", C1Cunit, record)
      ELSE
        CALL IOC1D ("READ DATA", C1Dunit, record)
      ENDIF
      RETURN
      END

!> Move CrIS 1D VIIRS variables from common data to argument arrays
      SUBROUTINE GET_CRIS1D_VIIRS(viirsradiance, 
     &  viirsreflectance, cloudflag)
      IMPLICIT NONE
#include "cris1d.h"

      integer viirsradiance(cris_1dnchanviirs,cris_1dnfor)
      integer viirsreflectance(cris_1dnchanviirs,cris_1dnfor)
      integer cloudflag(10,cris_1dnfor)
      
      INTEGER IFOR, ICHAN
      
      DO IFOR  = 1, cris_1dnfor
      DO ICHAN = 1, cris_1dnchanviirs
        viirsradiance(ICHAN,IFOR) =
     &              cris1d_viirsradiance(ICHAN,IFOR) 
        viirsreflectance(ICHAN,IFOR) =
     &              cris1d_viirsreflectance(ICHAN,IFOR)
      END DO
      END DO

      DO IFOR  = 1, cris_1dnfor
      DO ICHAN = 1, 10
        cloudflag(ICHAN,IFOR) = cris1d_cloudflag(ICHAN,IFOR)  
      END DO
      END DO

      RETURN
      END


!> Move CrIS 1D VIIRS variables from argument arrays to common
      SUBROUTINE PUT_CRIS1D_VIIRS(viirsradiance,
     &  viirsreflectance, cloudflag)
      IMPLICIT NONE
#include "cris1d.h"

      integer viirsradiance(cris_1dnchanviirs,cris_1dnfor)
      integer viirsreflectance(cris_1dnchanviirs,cris_1dnfor)
      integer cloudflag(10,cris_1dnfor)
      
      INTEGER IFOR, ICHAN
      
      DO IFOR  = 1, cris_1dnfor
      DO ICHAN = 1, cris_1dnchanviirs
        cris1d_viirsradiance(ICHAN,IFOR)    = viirsradiance(ICHAN,IFOR)
        cris1d_viirsreflectance(ICHAN,IFOR) = 
     &                                     viirsreflectance(ICHAN,IFOR)
      END DO
      END DO

      DO IFOR  = 1, cris_1dnfor
      DO ICHAN = 1, 10
        cris1d_cloudflag(ICHAN,IFOR)    = cloudflag(ICHAN,IFOR)
      END DO
      END DO

      RETURN
      END

!> Move CrIS 1C VIIRS variables from common data to argument arrays
      SUBROUTINE GET_CRIS1C_VIIRS( cris1clandfrac, cris1clandsea,
     &  cris1ccloudcover, cris1ccloudtopht)
      IMPLICIT NONE
#include "cris1c.h"
      integer cris1clandfrac(cris_1cnfov,cris_1cnfor)     ! land fraction
      integer cris1clandsea(cris_1cnfov,cris_1cnfor)      ! land/sea qualifier
      integer cris1ccloudcover(cris_1cnfov,cris_1cnfor)   ! cloud cover (percent)
      integer cris1ccloudtopht(cris_1cnfov,cris_1cnfor)   ! height of top of cloud (m)

      INTEGER IFOR, IFOV
      
      DO IFOR = 1, cris_1cnfor
      DO IFOV = 1, cris_1cnfov
        cris1clandfrac(IFOV, IFOR)   = cris1c_landfrac(IFOV, IFOR)
        cris1clandsea(IFOV, IFOR)    = cris1c_landsea(IFOV, IFOR)
        cris1ccloudcover(IFOV, IFOR) = cris1c_cloudcover(IFOV, IFOR)
        cris1ccloudtopht(IFOV, IFOR) = cris1c_cloudtopht(IFOV, IFOR)
      END DO
      END DO

      RETURN
      END

!> Move CrIS 1C VIIRS variables from argument arrays to common
      SUBROUTINE PUT_CRIS1C_VIIRS(cris1clandfrac, cris1clandsea,
     &  cris1ccloudcover, cris1ccloudtopht)
      IMPLICIT NONE
#include "cris1c.h"
      integer cris1clandfrac(cris_1cnfov,cris_1cnfor)     ! land fraction
      integer cris1clandsea(cris_1cnfov,cris_1cnfor)      ! land/sea qualifier
      integer cris1ccloudcover(cris_1cnfov,cris_1cnfor)   ! cloud cover (percent)
      integer cris1ccloudtopht(cris_1cnfov,cris_1cnfor)   ! height of top of cloud (m)

      INTEGER IFOR, IFOV
      
      DO IFOR = 1, cris_1cnfor
      DO IFOV = 1, cris_1cnfov
        cris1c_landfrac(IFOV, IFOR)   = cris1clandfrac(IFOV, IFOR)
        cris1c_landsea(IFOV, IFOR)    = cris1clandsea(IFOV, IFOR)
        cris1c_cloudcover(IFOV, IFOR) = cris1ccloudcover(IFOV, IFOR)
        cris1c_cloudtopht(IFOV, IFOR) = cris1ccloudtopht(IFOV, IFOR)
      END DO
      END DO

      RETURN
      END

!> Write a CrIS 1D record
      SUBROUTINE WRITE_CRIS1D(C1Dunit, record)
      IMPLICIT NONE
#include "cris1d.h"

      INTEGER C1Dunit, record

      CALL IOC1D ("WRITE DATA", C1Dunit, record)

      RETURN
      END

!> Write a CrIS 1C record
      SUBROUTINE WRITE_CRIS1C(C1Cunit, record)
      IMPLICIT NONE
#include "cris1c.h"

      INTEGER C1Cunit, record

      CALL IOC1C ("WRITE DATA", C1Cunit, record)

      RETURN
      END

