!+ De-apodise IASI data
!
      SUBROUTINE deapodise_iasi(spectrum, mtf, mode)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: deapodise_iasi.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   IASI level 1c data are gaussian apodised. To calculate eigenvector scores
!   based on signal to noise ratio we must transform back to self-apodised
!   - with uniform noise as a function of optical path difference.
!
! Method: 
!   The MTF correction is a function of wavenumber. Split the spectrum into a
!   number of smaller bands of 256 samples, separated by 50cm^-1. Process each
!   band, then combine using linear interpolation.
! 
!   See chapter 3 of "Support Study on IASI level 1c Data Compression Final 
!   Report" by A.C.L.Lee and S.Bedford, EUMETSAT Contract EUM/CO/03/1155/PS,
!   Feb 2004. 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      16/08/05   N.C.Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!      
!  Parameters:

      INTEGER lead_in
      PARAMETER (lead_in=436)         !45 cm^-1 plus 256 points
      INTEGER lead_out
      PARAMETER (lead_out=415)        !39.75 cm^-1 plus 256 points
      INTEGER npoints_in
      PARAMETER (npoints_in=8461)     !645-2760 cm^-1 at spacing 0.25 cm^-1
      INTEGER npoints_pad
      PARAMETER (npoints_pad=npoints_in+lead_in+lead_out)
      INTEGER npoints_work
      PARAMETER (npoints_work=512)
      INTEGER pow2
      PARAMETER (pow2=9)              !2^9 = 512
      INTEGER band_separation
      PARAMETER (band_separation=200) !200 samples = 50 cm^-1 @ 0.25 cm^-1
      INTEGER firstband
      PARAMETER (firstband=257)       !First band centred on 600cm^-1
      INTEGER nbands
      PARAMETER (nbands=45)
      
!  Arguments:

      REAL    spectrum(npoints_in)     !Spectrum from 645-2760 cm^-1
      REAL    mtf(nbands,npoints_work) !MTF correction 600-2800 cm^-1 step 50 cm^-1
      INTEGER mode                     !+1 to deapodise, -1 to apodise
      
!  Local variables:
     
      REAL composite(npoints_pad)
      REAL composite_out(npoints_pad)
      REAL spec_work(npoints_work)
      
      REAL hanning(56)   !Hanning window, produced using PV-Wave command
      DATA hanning/      !  print,(hanning(112))(0:55),format='(6(F9.7,","))'
     &  0.0000000,0.0008008,0.0032007,0.0071920,0.0127620,0.0198927,
     &  0.0285613,0.0387401,0.0503964,0.0634930,0.0779878,0.0938344,
     &  0.1109821,0.1293760,0.1489571,0.1696627,0.1914264,0.2141786,
     &  0.2378463,0.2623539,0.2876227,0.3135718,0.3401181,0.3671765,
     &  0.3946604,0.4224817,0.4505514,0.4787795,0.5070754,0.5353488,
     &  0.5635089,0.5914657,0.6191294,0.6464114,0.6732245,0.6994827,
     &  0.7251019,0.7500000,0.7740973,0.7973167,0.8195835,0.8408267,
     &  0.8609781,0.8799732,0.8977510,0.9142548,0.9294317,0.9432329,
     &  0.9556143,0.9665362,0.9759637,0.9838665,0.9902194,0.9950019,
     &  0.9981987,0.9997997/

      REAL window(npoints_work)       !Window function
      DATA window/512*1.0/            !  - will be modified later
      
      REAL interp(npoints_work)       !Interpolation function
      DATA interp/256*0.0,256*1.0/    !  - will be modified later

      INTEGER i,iband                        !Loop indices
      INTEGER istart,start_interp,end_interp !Array indices
      INTEGER ios
      
      LOGICAL first
      DATA first/.TRUE./

! Function & Subroutine calls:

      EXTERNAL SFFTCF,SFFTCB           !Real -> Complex Fast Fourier Transform
!
!- End of header

      IF (first) THEN

!Complete the spectral window

        DO i=1,56
          window(i) = hanning(i)
          window(1+npoints_work-i) = hanning(i)
        ENDDO

!Compute the interpolation function. Just do the leading half - the rest is 1.0

        start_interp = npoints_work/2 - band_separation + 1
        end_interp = npoints_work/2
      
        DO i=start_interp,end_interp
          interp(i) = (i-start_interp+0.5)/(end_interp-start_interp+1)
        ENDDO
        
        first = .FALSE.
        
      ENDIF    !first time

!Pad the spectrum at either end

      DO i=1,npoints_in
        composite(i+lead_in) = spectrum(i)
      ENDDO

      DO i=1,lead_in
        composite(i) = spectrum(2+lead_in-i)   !Reverse but don't use 1st point
      ENDDO

      DO i=1,lead_out
        composite(npoints_in+lead_in+i) = spectrum(npoints_in-i)
      ENDDO

!Multiply by Hanning window at either end

      DO i=1,56
        composite(i) = composite(i)*window(i)
        composite(1+npoints_pad-i)=composite(1+npoints_pad-i)*window(i)
      ENDDO 

!Split the spectrum into bands and transform

      DO iband=1,nbands
        istart = 1 + (iband-1)*band_separation
        DO i=1,npoints_work
          spec_work(i) = composite(istart+i-1)
        ENDDO
        
        CALL SFFTCF(spec_work, npoints_work, pow2)     !To interferogram domain

!Multiply by the apodisation correction function and transform back.
!In spec_work, points npoints_work/2+2 onwards contain the imaginary part in
!reverse order. The mtf correction is symmetric.

        IF (mode .EQ. 1) THEN     !De-apodise
          DO i=1,npoints_work
            spec_work(i) = spec_work(i)*mtf(iband,i)
          ENDDO
        ELSE
          DO i=1,npoints_work     !Apodise (for testing)
            spec_work(i) = spec_work(i)/mtf(iband,i)
          ENDDO
        ENDIF
        CALL SFFTCB(spec_work, npoints_work, pow2)      !To spectral domain

!Put into the output array with linear interpolation between bands

        IF (iband.eq.1) THEN
          DO i=1,npoints_work
            composite_out(i) = spec_work(i)
          ENDDO
        ELSE
          DO i=1,npoints_work
            composite_out(istart+i-1) = spec_work(i)*interp(i) + 
     &                composite_out(istart+i-1)*(1.0-interp(i))   
          ENDDO
        ENDIF

      ENDDO       !Loop over bands

!Put back into the input array

      DO i=1,npoints_in
        spectrum(i) = composite_out(i+lead_in)
      ENDDO
      END
