!+ I/O routine for level 1D ATMS data
!
      SUBROUTINE IOATTHIN1D ( COMMAND, AT1Dunit, AT1Drecord )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ioatthin1d.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   interface to files containing thinned level 1D ATMS data
!
! Method:
!   Performs one of several functions, according to the value of COMMAND :
!
!   "OPEN/READ"          : open file for readonly
!   "OPEN/WRITE"         : open file for write
!   "CLOSE"              : close file
!   "READ HEADER"        : read header into common
!   "READ DATA"          : read data record into common
!   "WRITE HEADER"       : write header from common
!   "WRITE DATA"         : write data from common
!
!   Note: COMMAND must be typed exactly, and in upper case.
!   Routine ends by calling "ErrorReport" to check on error status.
!   If "COMMAND" is not recognised, "ErrorReport"
!   will be called with fatal error status.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   13/05/2011  Based on ioat1d.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  Parameters for ATOVPP Processing 
#include "ppparms.h"
!  ATMS level 1D header/data common blocks 
#include "atms1dthin.h"
!  Record length dividor 
#include "reclopt.h"
!
!   Scalar arguments with intent(In):
      CHARACTER*(*) COMMAND       ! instruction on what function to perform
      INTEGER       AT1Dunit       ! unit number
      INTEGER       AT1Drecord     ! record number
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
!- End of header


! Act on value of COMMAND
! -----------------------

! 1.  OPEN/READ
!     ---------
      IF ( COMMAND .EQ. "OPEN/READ" ) THEN

         OPEN ( UNIT=AT1Dunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=tatms_1drecl/rldiv, IOSTAT=errStatus )

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=AT1Dunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=tatms_1drecl/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &  "("" OPEN/READ 1D thinned ATMS file: UNIT="",I3,"" RECL="",I6)")
     &                                AT1Dunit, tatms_1drecl/rldiv


! 2.  OPEN/WRITE
!     ----------
      ELSEIF ( COMMAND .EQ. "OPEN/WRITE" ) THEN

!        First try with STATUS="NEW", i.e. file doesn't yet exist
         OPEN ( UNIT=AT1Dunit, STATUS="NEW", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=tatms_1drecl/rldiv, IOSTAT=errStatus )

!        If that didn"t work, use STATUS="OLD"
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=AT1Dunit, STATUS="OLD", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=tatms_1drecl/rldiv, IOSTAT=errStatus )
! N.B. This seems to be needed on HPs, where "unknown" is treated as "scratch"

!        If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( errStatus.NE.0 )
     &   OPEN ( UNIT=AT1Dunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &   ACCESS="DIRECT", RECL=tatms_1drecl/rldiv, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     & "("" OPEN/WRITE 1D thinned ATMS file: UNIT="",I3,"" RECL="",I6)")
     &                                 AT1Dunit, tatms_1drecl/rldiv


! 3.  READ HEADER
!     ---- ------
      ELSEIF ( COMMAND .EQ. "READ HEADER" ) THEN

!        read first header record
         READ ( UNIT=AT1Dunit,REC=AT1Drecord,IOSTAT=errStatus )
     &      tatms1d_hd

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     & "("" READing 1D thinned ATMS header: UNIT="",I3,"" REC ="",I6)")
     &                             AT1Dunit, AT1Drecord


! 4.  WRITE HEADER
!     ----- ------
      ELSEIF ( COMMAND .EQ. "WRITE HEADER" ) THEN

         WRITE ( UNIT=AT1Dunit,REC=AT1Drecord,IOSTAT=errStatus ) 
     &      tatms1d_hd

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     & "("" WRITEing 1D thinned ATMS header: UNIT="",I3,"" REC="",I6)")
     &                              AT1Dunit, AT1Drecord


! 5.  READ DATA
!     ---- ----
      ELSEIF ( COMMAND .EQ. "READ DATA" ) THEN

         READ ( UNIT=AT1Dunit,REC=AT1Drecord,IOSTAT=errStatus )
     &      tatms1d_dt

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" READing ATMS thinned 1D data, UNIT="",I3,"" REC="",I6)")
     &                             AT1Dunit, AT1Drecord


! 6.  WRITE DATA
!     ----- ----
      ELSEIF ( COMMAND .EQ. "WRITE DATA" ) THEN

         WRITE ( UNIT=AT1Dunit,REC=AT1Drecord,IOSTAT=errStatus )
     &         tatms1d_dt

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" WRITEing ATMS thinned 1D data, UNIT="",I3,"" REC="",I6)")
     &                              AT1Dunit, AT1Drecord


! 7.  CLOSE
!     -----
      ELSEIF ( COMMAND .EQ. "CLOSE" ) THEN

         CLOSE ( UNIT=AT1Dunit, IOSTAT=errStatus )

!        define error message
         errMessage = " "
         WRITE ( errMessage,
     &   "("" CLOSEing ATMS thinned 1D file, UNIT="",I3)") AT1Dunit


! 8.  COMMAND not recognized
!     ------- --- ----------
      ELSE

         errStatus = 999
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Command not recognised: "",A50)") COMMAND


      ENDIF   ! (COMMAND)


! 8.  Check error status
!     ----- ----- ------

      CALL ErrorReport ( errStatus, errMessage, "IOATTHIN1D")


      RETURN
      END
