!+ create Look-Up-Table (LUT)  (ATOVPP module)
      SUBROUTINE LUTMAP(CMAP_INSTR,CTARG_INSTR,
     &                  TARG_START_LINE,
     &                  TARG_FOVS_PER_LINE,TARG_LINES,
     &                  MULT_FACTOR,DURATION,TIME_DIFF,
     &                  TIME_COR,SCAN_ANGLE_COR,LUTBUF,
     &                  ELMS_PER_COLOC,MAX_COLOCS,
     &                  HT_OF_SAT,PERIOD_OF_SAT,
     &                  SPATIAL_MODE,SPATIAL_MULT,SATID,WMOSATID,STATUS)

! Name: $Id: lutmap.F 62 2006-12-13 10:42:15Z frna $
!
! Purpose:
!   To generate a Look-Up-Table (LUT) which will identify those mapping
!   fovs which are 'close to' (colocate with)  target fovs. The LUT will also
!   provide a weight for each mapping fov for each mapping mode (nearest
!   neighbour, bi-linear interpolation, spatial average) to be applied to the
!   BT of each colocating mapping fov to provide the mapped value.
!   An appropriate LUT must be produced before running BT_MAP, FLAG_MAP or
!   AV_MAP.
!
! Method:
!   The identification of the mapping fovs close to a target fov is done by
!   locating each mapping and target fov on a 2-dimensional rectangular
!   grid of which one axis represents the track of the sub-satellite point
!   on the earth's surface and the other axis represents an 'instantaneous'
!   scan line at some origin time (in practice the origin time is taken to be
!   the start time (fov 1 time) of the target line designated 'line 1').
!   Displacement in the along track direction is a function of time and
!   displacement in the along scan direction is determined by the scanning
!   geometry of the instruments (note displacements are distances along the
!   curved earth's surface).
!
!   Ellipse footprints are also constructed on the grid for each target
!   fov. Those mapping fovs which fall inside a target fov footprint are taken
!   to be colocated with the fov (the ellipse for target fovs can be
!   artificially increased if required to capture a reasonable number of
!   mapping fovs).
!
!   Note that the pattern of target/mapping fovs will repeat at regular
!   intervals so the LUT only has to cover this 'pattern repeat' time.
!
!   The generation of the LUT is based on 'nominal' geometry ie satellite
!   and instrument characteritics are assumed to be as given in a NESS 107
!   type document. Actual characteristics may be systematically different from
!   these. The effect of these differences can be investigated and the results
!   incorporated into a correction for the position of each target fov across
!   the scan. The correction is in the form of small increments assigned
!   to the time of the fov and to the scan angle assigned to the fov (note
!   these corrections are believed only to be important when mapping AVHRR
!   to HIRS (or to some other instrument) where the size of the corrections
!   is similar to the resolution of the AVHRR instrument).
!
!   An example of a LUT entry for a target fov is shown below -
!
!     818   1523      0      0      0      0      0      0      0      7
!       7     15  -1115  -6216      6     -1      0      0      0      0
!       7     16   3732  -6083      7     -1      0      0      0      0
!       8     14  -5982  -1095      5     -1      0      0      0      0
!       8     15  -1115   -962      1     65     94      0      0      0
!       8     16   3732   -829      2     19      4      0      0      0
!       9     15  -1115   4291      3     12      1      0      0      0
!       9     16   3732   4424      4      4      0      0      0      0
!       0      0      0      0      0      0      0      0      0      0
!       0      0      0      0      0      0      0      0      0      0
!       0      0      0      0      0      0      0      0      0      0
!       0      0      0      0      0      0      0      0      0      0
!       0      0      0      0      0      0      0      0      0      0
!       0      0      0      0      0      0      0      0      0      0
!       0      0      0      0      0      0      0      0      0      0
!
!   There are 15 lines and 10 elements per line.
!   Line 1 elements 1 and 2 give the position of the target fov within the
!   mapping fovs as a mapping line/spot position (*100) ie in the example
!   the fov is at mapping line 8.18 and mapping spot 15.23
!   Line 1 element 10 gives the number of mapping fovs colocating with the
!   target fov. In the example lines 2 to 8 provide information on each of the
!   7 colocating mapping fov.
!   Elements 1 and 2 give the line/spot of the mapping fov.
!   Elements 3 and 4 gives the scan/track position (Km) of the mapping fov
!   relative to the target fov (*100) ie the first mapping fov is displaced
!   11.15Km to the 'left' of the target fov and 62.16Km 'below' the target fov.
!   Element 5 gives the ranking of the distance of the mapping fov from the
!   target fov.
!   Element 6 gives the bi-linear interpolation weight (*100) for the fov
!   (-1 indicates that the fov takes no part).
!   Element 6 gives the spatial average weight *100) for the fov (the actual
!   weight will depend on options selected for LUTMAP).
!
!   A target fov LUT entry for mapping AVHRR data has a different structure -
!
!   16278 147344     21
!     153   1470   1477
!     154   1467   1480
!     155   1465   1481
!     156   1464   1483
!     157   1463   1484
!     158   1462   1484
!     159   1462   1485
!     160   1461   1485
!     161   1461   1486
!     162   1461   1486
!     163   1461   1486
!     164   1461   1486
!     165   1461   1485
!     166   1462   1485
!     167   1462   1485
!     168   1463   1484
!     169   1464   1483
!     170   1465   1482
!     171   1466   1481
!     172   1468   1479
!     173   1472   1475
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!       0      0      0
!
! There are 40 lines and 3 elements.
! Line 1 gives the position of the target fov within the mapping grid ie at
! map line number 162.78 and map scan position 1473.44. The third element
! gives the number of mapping 'colocating lines'. Subsequent lines provide the
! AVHRR line number and the range of AVHRR fovs for each colocating line.
!
!
! The basic data for mapping will normally be held in a 'level 1C' file for the
! mapping instrument and there will be a corresponding level 1C (or 1D) file
! for the target instrument data (into which the mapped data will be finally
! stored). Such files may contain data for the whole of a local satellite
! pass or the whole of one orbit of the satellite for global data. In order
! to process data on machines of limited memory the data in such files will
! be divided into blocks held in local buffers. LUTMAP will normally be
! run with a TIME_DIFF appropriate to the line 1 times of the original 1C
! (or 1D) files and then used on successive blocks of data with offset values
! providing a link between line numbers implicit in the local buffers and
! the line numbers of the original 1C (or 1D) files.



!
! STATUS returned
!        =0   OK, -ve warning, +ve fatal
!       -1=   number of mapping fovs truncated. Maximum number allowed =
!             MAX_COLOCS-1.
!             This Status might be returned when mapping AMSU-A to HIRS where
!             the HIRS fov is inflated. Problem could be avoided by
!             reducing the multiplier (MULT_FACTOR)
!       -2=   serious problem in calculating the range of mapping lines for
!             the LUT. The calculated start mapping line number
!             (MAP_START_LINE) has come out .LE.0 and will be set to 1. Some
!             mapping fovs may not be represented in the LUT.
!       -3=   only applicable where the mapping instrument is AVHRR. The rough
!             calculation to determine a range of mapping lines to `cover' a
!             target fov has given a negative start line. Some mapping fovs
!             may not be represented in the LUT.
!       -4=   only 3 out of 4 mapping fovs found for bi-linear interpolation
!             in WEIGHTS. Probable interpretation is that (when mapping MSU
!             to HIRS for example) the HIRS fov enlargement is insufficient
!             to include enough MSU fovs. Track interpolation only will be
!             done.
!        10=  mapping instrument not recognised
!        11=  target instrument not recognised
!        12=  no mapping fovs found. Might need to increase MULT_FACTOR.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
!             Version  Date      Comment
!
!             1.0      04/05/95  Original code. (P.C.Dibben)
!             1.1      18/03/96  Code adapted for ATOVPP (R.J.Renshaw)
!                      including: except for AVHRR, colocs in LUT are
!                      ordered in increasing distance from the target fov
!             1.2      04/03/97  Allow mapping of AVHRR to AMSU-A (R.J.Renshaw)
!             1.3      30/06/98  Include trap for when no mapping fovs found.
!                                Values of SPATIAL_MODE changed.
!             1.4      23/11/98  Introduction of SPATIAL_MODE=4 (spatial
!                                average with linear weighting). WEIGHTS
!                                rationalised. Weights will be generated
!                                only for specified SPATIAL_MODE (other weights
!                                not needed; will be faster).
!                                NB corresponding change made to PPBTMAP (v1.2)
!                                (P C Dibben)
!             1.5      06/12/00  Introduction of SPATIAL_MODE=5 (BG-convolution,
!                                calculation done in PPBGB2A).  (Keith Whyte)
!             1.6      20/07/04  Remove hard-coded scan parameters via call
!                                to ppscan_params
!             1.7      15/09/04  Use constants_preproc.h for fundamental 
!                                constants - NCA
!             1.8      23/12/04  Comment out hard-coded AVHRR scan params - NCA
!             1.9      22/06/05  Fix HIRS FOV at 1.4 deg unless we are mapping
!                                AVHRR - NCA
!             2.0      24/08/05  Add IASI - NCA
!             2.1      30/11/06  Allow MAP_START_LINE to be negative. This can
!                                occur if TIME_DIFF is large, e.g. when scan
!                                numbers return to 1 at start of new METOP 
!                                orbit. NCA
! 
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:

      IMPLICIT NONE
c-----------------------------------------
c
c

! Argument list (all input except LUTBUF and STATUS)
      CHARACTER*(*)  CMAP_INSTR  ! mapping instrument
      CHARACTER*(*)  CTARG_INSTR ! target instrument
      INTEGER
     &            TARG_START_LINE, ! a `reasonable' target start line for
                                 ! LUT; will be adjusted if necessary.
                                 ! reasonable values are -
                                 ! mapping AVHRR to HIRS   4
                                 ! mapping AMSU-A to HIRS  9
                                 ! mapping AMSU-B to A     3
                                 ! mapping MSU to HIRS    25
                                 ! NB if the value is adjusted in LUTMAP the
                                 ! new value must be used when using the LUT.
     &            TARG_FOVS_PER_LINE,  ! number of target fovs per line
                                       ! for HIRS 56, for AMSU-A 30
     &            TARG_LINES,    ! number of target lines in duration (must be
                                 ! consistent with targ instru and DURATION) eg
                                 ! HIRS and 32 secs   =5
                                 ! HIRS and 128 secs  =20
                                 ! AMSU-A/IASI and 32 secs =4
     &            MULT_FACTOR,   ! mult factor for target CONE_ANGLE (*10)
                                 ! eg 10 for AVHRR to HIRS
                                 !    10 for AMSU-B to AMSU-A
                                 !    70 for AMSU-A to HIRS
                                 !   220 for MSU to HIRS
     &            DURATION,      ! time period covered by LUT (secs)
                                 ! eg 32 secs except MSU to HIRS 128 secs
     &            TIME_DIFF,     ! (mapping line 1 time) - (target line 1 time)
                                 ! msec (see RTIME_DIFF below)
     &            ELMS_PER_COLOC, ! number of elements of information per
                                 ! colocation
                                 ! 3 for AVHRR to HIRS, else 10
     &            MAX_COLOCS     ! max number of colocations
                                 ! 75 for AVHRR to AMSU-A
                                 ! 40 for AVHRR to HIRS
                                 ! 30 for MSU to HIRS
                                 ! else 15
      REAL
     &            HT_OF_SAT,     ! Km
     &            PERIOD_OF_SAT  ! Mins
      INTEGER     SPATIAL_MODE   ! 1=nearest neighbour    (to calculate weights)
                                 ! 2=bilinear
                                 ! 3=spatial average with gaussian weighting
                                 ! 4=spatial average with linear weighting
                                 ! 5=BG-convolution
      INTEGER     SPATIAL_MULT   ! mult factor (*100) applied to ellipse scan
                                 ! axis to define distance of zero influence
                                 ! for a mapping fov (SPATIAL_MODE=4) or define
                                 ! sigma value for gaussian function
                                 ! (SPATIAL_MODE=3)
      INTEGER     SATID          ! satid as given at level 1c
      INTEGER     WMOSATID       ! WMO satid as given at level 1c
      INTEGER     STATUS

!Include
!  Fundamental and physical constants 
#include "constants_preproc.h"

!Local variables
      REAL
     &            TIME_COR(*),   ! empirically derived time correction for each
                                 ! scan position (up to 90) of target instru
                                 ! (secs; added). NB the correction will be
                                 ! for a target/mapping pair.
     &            SCAN_ANGLE_COR(*)  ! empirically derived scan angle
                                 ! corrections for each scan position (up to 90)
                                 ! of target instru (deg; added). NB the
                                 ! corrections will be for a target/mapping
                                 ! pair.
      REAL
     &            LUTBUF(ELMS_PER_COLOC,MAX_COLOCS,
     &                   TARG_FOVS_PER_LINE,TARG_LINES)
                                 ! output buffer for LUT



! These data areas are either fixed values (but with options) or are calculated
      INTEGER
     &            MAP_START_LINE, ! 1st line for mapping location
     &            MAP_END_LINE    ! last line for mapping location

      INTEGER
     &            MAP_LINES       ! number of mapping lines in duration


      INTEGER
     &            TARG_END_LINE   ! last target line for LUT

      REAL        RTIME_DIFF      ! real version of TIME_DIFF in secs


! Pointers for loading LUT buffer
      INTEGER
     &            SET_PNT,       ! from 1 to MAX_COLOCS
     &            ELEM_PNT       ! from 1 to ELMS_PER_COLOC

! For calculation of MAP start and end lines for LUT
      REAL
     &            TARG_FOV_TIME,          ! time of a target fov (secs)
     &            TARG_ELLIPSE_TIME       ! time associated with ellipse
                                          ! extremity of a targ fov (secs)
! Satellite data
      REAL        SAT_ANGULAR_SPEED       ! rate of rotation
                                          !(radians per sec) calc
                                          ! from PERIOD_OF_SAT
      REAL        HT_OF_SATT              ! Km, copy of HT_OF_SAT
                                          ! for Common below
      COMMON/SATELLITE/HT_OF_SATT,SAT_ANGULAR_SPEED

! Instrument data. 
! The 6 elements refer to AVHRR, HIRS, AMSU-A, AMSU-B, MSU, IASI.
      INTEGER     TARG_INSTRU    ! Target instrument identifier
                                 ! 1=AVHRR,2=HIRS,3=AMSU-A,4=AMSU-B,5=MSU,6=IASI
                                 ! used as pointer into buffers containing
                                 ! instrument dependent data
      INTEGER     MAP_INSTRU     ! Mapping instrument identifier - as
                                 ! TARG_INSTRU
      REAL        CONE_ANGLES(6)
      REAL        CONE_ANGLE      ! working value (deg)
      REAL        STEP_ANGLES(6)  ! angle (deg) between consecutive spots (deg)
      REAL        STEP_ANGLE
      REAL        STEP_TIMES(6)   ! time between consecutive spots (sec)
      REAL        STEP_TIME
      REAL        LINE_TIMES(6)   ! Line repetion rate (secs)
      REAL        LINE_TIME
      REAL        HALFCONE_IASI   ! Separation of 4 spots from centre (deg)
      INTEGER     SPOTS_PER_LINES(6) ! number of scan positions per line
      INTEGER     SPOTS_PER_LINE
      INTEGER     LEFT_TO_RIGHT_SCANS(6)  ! set to 1 if instrument scans left
C                                         ! to right (looking along satellite
C                                         ! velocity vector)
      INTEGER     LEFT_TO_RIGHT_SCAN

      COMMON/INSTRUMENT/CONE_ANGLE,STEP_ANGLE,STEP_TIME,LINE_TIME,
     &       HALFCONE_IASI,SPOTS_PER_LINE,LEFT_TO_RIGHT_SCAN

! Work areas
      REAL
     &            SPOT_TIME,     ! time associated with a fov (sec)
     &            SCAN_POS,      ! position of fov as displacement from Track
                                 ! axis (km)
     &            TRACK_POS,     ! position of fov as displacement from Scan
                                 ! axis (km)
     &            TARG_POS_BUFF(2,90,25),   ! Scan and track positions (kms)
                                 ! for up to 90 spots and 25 lines
     &            MAP_POS_BUFF(2,2048,360),    ! Scan and track positions (kms)
                                 ! for up to 2048 spots and 360 lines
                                 ! (04/03/97: 270 changed to 340 for AMSU-A)
                                 ! (21/12/00: 340 changed to 360 for AMSU-A TL)
     &            LOCUS_POINTS(60), ! 30 scan/track positions relative to
                                 ! centre (km) for top-right quadrant of
                                 ! ellipse (not used here)
     &            ELLIPS(3,90),  ! major (scan) axis, minor (track) axis (km),
                                 ! and `right hand' focus (km)
                                 ! for up to 90 ellipses
     &            TMPLUT(20,90), ! NEW 25/11/96: temporary store for LUT
                                 ! elements when re-ordering (nearest coloc first)

! the 6 data areas below are concerned with rough selection of AVHRR fovs
! within target ellipse.
     &            ELLIP_TRACK_MAX,  ! max track offset for target ellipse (km)
     &            ELLIP_TRACK_MIN,  ! min track offset for target ellipse (km)
     &            ELLIP_SCAN_MAX,   ! max scan offset for target ellipse (km)
     &            ELLIP_SCAN_MIN,   ! min scan offset for target ellipse (km)
     &            SCAN_KM(2048),    ! scan positions (km) for each fov on any
                                    ! AVHRR line.
     &            TRACK_GRAD        ! AVHRR line separation
      DATA        TRACK_GRAD/1.09/  ! (km)

! Data for locating target fov on map grid
      REAL
     &            TARG_NADIR,    ! nadir scan position for target
     &            MAP_NADIR,     ! nadir scan position for mapping
     &            TARG_MAP_POS(2,90,25), ! targ fov positions (up to 90 spots
                                 ! and 25 lines) as map scan position and map
                                 ! line position
     &            SCAN_ANG       ! scan angle (deg) for a fov

! Loop controls etc
      INTEGER     II,JJ,IIM,JJM,IIT,JJT,KK,KKM,KKT,ICOUNT,
     &            MAP_FOVS_PER_LINE,
     &            FIRST_SPOT,LAST_SPOT,START_LINE,END_LINE,
     &            START_SPOT,END_SPOT,IERR
      LOGICAL     SEQU, INSIDE
         DATA     SEQU/.FALSE./

! Functions
      INTEGER    NINT             ! convert real number to nearest integer

!- End of header



      STATUS = 0   ! no problems yet

! Get scan parameters
      CALL PPSCAN_PARAMS(SATID,WMOSATID,
     & CONE_ANGLES, STEP_ANGLES, STEP_TIMES, LINE_TIMES,
     & SPOTS_PER_LINES, LEFT_TO_RIGHT_SCANS, HALFCONE_IASI, IERR)

      IF (IERR.NE.0) THEN
        CALL PPSCAN_PARAMS(15,206,             !satid not found: try NOAA-15
     &   CONE_ANGLES, STEP_ANGLES, STEP_TIMES, LINE_TIMES,
     &   SPOTS_PER_LINES, LEFT_TO_RIGHT_SCANS, HALFCONE_IASI, IERR)

        IF (IERR.NE.0) THEN
          CALL ErrorReport(1,"Error getting scan parameters","LUTMAP")
        ELSE
          CALL ErrorReport(-1,"Using default scan parameters","LUTMAP")
        ENDIF
      ENDIF
C      write(*,*)'CONE_ANGLES:',CONE_ANGLES  
C      write(*,*)'STEP_ANGLES:',STEP_ANGLES  
C      write(*,*)'STEP_TIMES:',STEP_TIMES 
C      write(*,*)'LINE_TIMES:',LINE_TIMES
C      write(*,*)'HALFCONE_IASI:',HALFCONE_IASI
C      write(*,*)'SPOTS_PER_LINES:',SPOTS_PER_LINES 
C      write(*,*)'LEFT_TO_RIGHT_SCANS:',LEFT_TO_RIGHT_SCANS

! Define mapping instrument
      IF     ( CMAP_INSTR.EQ."AVHRR" ) THEN
                MAP_INSTRU = 1
      ELSEIF ( CMAP_INSTR.EQ."HIRS" ) THEN
                MAP_INSTRU = 2
      ELSEIF ( CMAP_INSTR.EQ."AMSU-A" ) THEN
                MAP_INSTRU = 3
      ELSEIF ( CMAP_INSTR.EQ."AMSU-B" ) THEN
                MAP_INSTRU = 4
      ELSEIF ( CMAP_INSTR.EQ."MSU" ) THEN
                MAP_INSTRU = 5
      ELSEIF ( CMAP_INSTR.EQ."IASI" ) THEN
                MAP_INSTRU = 6
      ELSE
         STATUS = 10
         GOTO 99   ! jump out of this routine
      ENDIF

! Define target instrument
      IF     ( CTARG_INSTR.EQ."AVHRR" ) THEN
                TARG_INSTRU = 1
      ELSEIF ( CTARG_INSTR.EQ."HIRS" ) THEN
                TARG_INSTRU = 2
      ELSEIF ( CTARG_INSTR.EQ."AMSU-A" ) THEN
                TARG_INSTRU = 3
      ELSEIF ( CTARG_INSTR.EQ."AMSU-B" ) THEN
                TARG_INSTRU = 4
      ELSEIF ( CTARG_INSTR.EQ."MSU" ) THEN
                TARG_INSTRU = 5
      ELSEIF ( CTARG_INSTR.EQ."IASI" ) THEN
                TARG_INSTRU = 6
      ELSE
         STATUS = 11
         GOTO 99   ! jump out of this routine
      ENDIF

! v1.9: Unless we are mapping AVHRR, fix the HIRS cone angle at 1.4 degrees 
! even if this is a HIRS/4 instrument with 0.7 degree field of view. Otherwise
! we would have too few AMSU samples contributing and the noise would be high.

      IF (CTARG_INSTR.EQ."HIRS".AND.CMAP_INSTR.NE."AVHRR") THEN
        CONE_ANGLES(2) = 1.4
      ENDIF

! Calculate satellite angular speed
      SAT_ANGULAR_SPEED=(2.0*PI)/(PERIOD_OF_SAT*60.0)

! Derive data from input data
      RTIME_DIFF=TIME_DIFF/1000.0                    ! secs
      MAP_LINES=NINT(DURATION/LINE_TIMES(MAP_INSTRU))
      HT_OF_SATT=HT_OF_SAT    ! to satisfy SATELLITE common


! Set up ellipse buffer (ELLIPS) for target instrument

      CONE_ANGLE=CONE_ANGLES(TARG_INSTRU)
      STEP_ANGLE=STEP_ANGLES(TARG_INSTRU)
      STEP_TIME=STEP_TIMES(TARG_INSTRU)
      SPOTS_PER_LINE=SPOTS_PER_LINES(TARG_INSTRU)
      LEFT_TO_RIGHT_SCAN=LEFT_TO_RIGHT_SCANS(TARG_INSTRU)
      LINE_TIME=LINE_TIMES(TARG_INSTRU)

      CONE_ANGLE=CONE_ANGLE*(MULT_FACTOR/10.0)

      DO JJ=1,SPOTS_PER_LINE
        CALL ELLIPSE(JJ,ELLIPS(1,JJ),ELLIPS(2,JJ),
     &  ELLIPS(3,JJ),LOCUS_POINTS)
      ENDDO


! Calculate range of mapping lines (MAP_START_LINE to MAP_END_LINE) which will
! `cover' the target fovs of the LUT (the range of target lines is
! TARG_START_LINE to TARG_END_LINE).
! A number of simplifications are made but all should give a larger range of
! mapping lines than is strictly necessary. Where AVHRR is the mapping line
! (with right to left scan) it will be assumed that the scan is instantaneous.
! Calc start line.
C      MAP_START_LINE=-1
      MAP_START_LINE=-99999
      TARG_START_LINE=TARG_START_LINE-1
      ICOUNT=0
C      DO WHILE(MAP_START_LINE.LE.0.AND.ICOUNT.LT.20)
      DO WHILE(MAP_START_LINE.LE.-99999.AND.ICOUNT.LT.20)
        ICOUNT=ICOUNT+1
        TARG_START_LINE=TARG_START_LINE+1
!       calculate time of earliest target fov
        TARG_FOV_TIME=(TARG_START_LINE-1)*LINE_TIMES(TARG_INSTRU)  ! secs.  time
!                       of targ line 1 relative to origin time will be zero.
        TARG_ELLIPSE_TIME=TARG_FOV_TIME-(ELLIPS(2,1)/
     &                  (SAT_ANGULAR_SPEED*rEarth))          ! secs  ie
!                       earlier in time than centre of the fov by the time
!                       equivalent of the ellipse track axis (strictly only
!                       half the axis required)
        TARG_ELLIPSE_TIME=TARG_ELLIPSE_TIME-RTIME_DIFF  ! now relative to
                                               ! time of first mapping line
        MAP_START_LINE=(TARG_ELLIPSE_TIME/LINE_TIMES(MAP_INSTRU))+1
      ENDDO
      IF(MAP_START_LINE.LE.-99999) THEN     ! map start line still invalid
        STATUS=-2
        MAP_START_LINE=1
      ENDIF

! Now similar calculation for end line
      TARG_END_LINE=TARG_START_LINE+(TARG_LINES-1)
      TARG_FOV_TIME=TARG_END_LINE*LINE_TIMES(TARG_INSTRU)   ! time of first fov
!                       for `next' line rather than last fov for end line
      TARG_ELLIPSE_TIME=TARG_FOV_TIME+(ELLIPS(2,1)/
     &                  (SAT_ANGULAR_SPEED*rEarth))
      TARG_ELLIPSE_TIME=TARG_ELLIPSE_TIME-RTIME_DIFF
      MAP_END_LINE=(TARG_ELLIPSE_TIME/LINE_TIMES(MAP_INSTRU))+1


! Set up location buffer for Target instrument (TARG_POS_BUFF) and
! locate target fovs within the mapping grid

      CONE_ANGLE=CONE_ANGLES(TARG_INSTRU)
      STEP_ANGLE=STEP_ANGLES(TARG_INSTRU)
      STEP_TIME=STEP_TIMES(TARG_INSTRU)
      SPOTS_PER_LINE=SPOTS_PER_LINES(TARG_INSTRU)
      LEFT_TO_RIGHT_SCAN=LEFT_TO_RIGHT_SCANS(TARG_INSTRU)
      LINE_TIME=LINE_TIMES(TARG_INSTRU)
      TARG_NADIR=SPOTS_PER_LINE/2.0+0.5
      MAP_NADIR=SPOTS_PER_LINES(MAP_INSTRU)/2.0+0.5

      DO II=TARG_START_LINE,TARG_END_LINE
        KKT=(II-TARG_START_LINE)+1
        DO JJ=1,SPOTS_PER_LINE
          CALL LOCATION(II,JJ,0.0,TIME_COR(JJ),
     &                  SCAN_ANGLE_COR(JJ),SCAN_POS,TRACK_POS)
          TARG_POS_BUFF(1,JJ,KKT)=SCAN_POS
          TARG_POS_BUFF(2,JJ,KKT)=TRACK_POS

!         now calculate position of target fov on mapping grid
!         calculate spot time of target fov
          SPOT_TIME=(II-1)*LINE_TIME+(JJ-1)*STEP_TIME+
     &              TIME_COR(JJ)                                        !secs
          SPOT_TIME=SPOT_TIME-RTIME_DIFF          ! now relative to map line 1

!         calculate scan angle for target fov
          IF(LEFT_TO_RIGHT_SCAN.EQ.1) THEN
            SCAN_ANG=(JJ-TARG_NADIR)*STEP_ANGLE+
     &               SCAN_ANGLE_COR(JJ)
          ELSE
            SCAN_ANG=(TARG_NADIR-JJ)*STEP_ANGLE+
     &               SCAN_ANGLE_COR(JJ)
          ENDIF

!         calculate scan position
          IF(LEFT_TO_RIGHT_SCANS(MAP_INSTRU).EQ.1) THEN
            TARG_MAP_POS(1,JJ,KKT)=(SCAN_ANG/STEP_ANGLES(MAP_INSTRU))+
     &                            MAP_NADIR
          ELSE
            TARG_MAP_POS(1,JJ,KKT)=MAP_NADIR-
     &                            (SCAN_ANG/STEP_ANGLES(MAP_INSTRU))
          ENDIF

!         calculate track position
!         first remove from SPOT_TIME that component due to the scan position
!         of the target fov to give the 'line time' for the fov
          SPOT_TIME=SPOT_TIME-
     &              (TARG_MAP_POS(1,JJ,KKT)-1)*STEP_TIMES(MAP_INSTRU)
!         convert time to a number of mapping lines
          TARG_MAP_POS(2,JJ,KKT)=1+SPOT_TIME/LINE_TIMES(MAP_INSTRU)
        ENDDO
      ENDDO


! Set up location buffer for Mapping instrument (MAP_POS_BUFF)

      CONE_ANGLE=CONE_ANGLES(MAP_INSTRU)
      STEP_ANGLE=STEP_ANGLES(MAP_INSTRU)
      STEP_TIME=STEP_TIMES(MAP_INSTRU)
      SPOTS_PER_LINE=SPOTS_PER_LINES(MAP_INSTRU)
      LEFT_TO_RIGHT_SCAN=LEFT_TO_RIGHT_SCANS(MAP_INSTRU)
      LINE_TIME=LINE_TIMES(MAP_INSTRU)

      DO II=MAP_START_LINE,MAP_END_LINE
        KKM=(II-MAP_START_LINE)+1
        DO JJ=1,SPOTS_PER_LINE
          CALL LOCATION(II,JJ,RTIME_DIFF,0.0,
     &                  0.0,SCAN_POS,TRACK_POS)
          MAP_POS_BUFF(1,JJ,KKM)=SCAN_POS
          MAP_POS_BUFF(2,JJ,KKM)=TRACK_POS
        ENDDO
      ENDDO
!     now fill SCAN_KM from a `representative' line
      DO II=1,2048
        SCAN_KM(II)=MAP_POS_BUFF(1,II,1)
      ENDDO


! Perform colocation selection for each target fov in the LUT
! and set up LUTBUF

      MAP_FOVS_PER_LINE=SPOTS_PER_LINES(MAP_INSTRU)

      IF(MAP_INSTRU.EQ.1) THEN          ! AVHRR

        DO IIT=TARG_START_LINE,TARG_END_LINE
          KKT=(IIT-TARG_START_LINE)+1   ! LUTBUF pointer for dimension 4
          DO JJT=1,TARG_FOVS_PER_LINE   ! LUTBUF pointer for dimension 3
            SET_PNT=1                   ! LUTBUF pointer for dimension 2
            ELEM_PNT=0                  ! LUTBUF pointer for dimension 1

            ELEM_PNT=ELEM_PNT+1
            LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=
     &      TARG_MAP_POS(2,JJT,KKT)
            ELEM_PNT=ELEM_PNT+1
            LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=
     &      TARG_MAP_POS(1,JJT,KKT)
            ELEM_PNT=0
            SET_PNT=SET_PNT+1

!           check which mapping fovs colocate with current target fov
!           first calculate rough start and end mapping lines/spots for
!           fov search (NB +-2 and +-5 are to ensure that rough search area
!           is greater than strictly necessary).
            ELLIP_TRACK_MAX=TARG_POS_BUFF(2,JJT,KKT)+
     &                      ELLIPS(2,JJT)/2.0
            ELLIP_TRACK_MIN=TARG_POS_BUFF(2,JJT,KKT)-
     &                      ELLIPS(2,JJT)/2.0
            ELLIP_SCAN_MAX=TARG_POS_BUFF(1,JJT,KKT)+
     &                      ELLIPS(1,JJT)/2.0
            ELLIP_SCAN_MIN=TARG_POS_BUFF(1,JJT,KKT)-
     &                      ELLIPS(1,JJT)/2.0
            START_LINE=NINT(ELLIP_TRACK_MIN/TRACK_GRAD-
     &                      RTIME_DIFF/LINE_TIMES(1))-2
            IF(START_LINE.LE.0) THEN   ! search start line invalid
              STATUS=-3
              START_LINE=1
            ENDIF
            END_LINE=NINT(ELLIP_TRACK_MAX/TRACK_GRAD-
     &                      RTIME_DIFF/LINE_TIMES(1))+2
            DO II=1,2048
              IF(SCAN_KM(II).LT.ELLIP_SCAN_MAX) THEN
                START_SPOT=II-5
                GOTO 10
              ENDIF
            ENDDO
 10         CONTINUE
            DO II=1,2048
              IF(SCAN_KM(II).LT.ELLIP_SCAN_MIN) THEN
                END_SPOT=II+5
                GOTO 20
              ENDIF
            ENDDO
 20         CONTINUE
!           check search lines are within LUT mapping lines
            IF(START_LINE.LT.MAP_START_LINE) START_LINE=MAP_START_LINE
            IF(END_LINE.GT.MAP_END_LINE) END_LINE=MAP_END_LINE

            DO IIM=START_LINE,END_LINE
              KKM=(IIM-MAP_START_LINE)+1  ! NB MAP_START_LINE is line 1 in
              DO JJM=START_SPOT,END_SPOT             ! MAP_POS_BUFF

                CALL COLOC(MAP_POS_BUFF(1,JJM,KKM),
     &                     MAP_POS_BUFF(2,JJM,KKM),
     &                     TARG_POS_BUFF(1,JJT,KKT),
     &                     TARG_POS_BUFF(2,JJT,KKT),
     &                     ELLIPS(1,JJT),
     &                     ELLIPS(2,JJT),
     &                     ELLIPS(3,JJT),
     &                     INSIDE)
                IF(INSIDE) THEN
                  IF(SEQU) THEN
                    LAST_SPOT=JJM
                  ELSE
                    SEQU=.TRUE.
                    FIRST_SPOT=JJM
                    LAST_SPOT=JJM
                  ENDIF
                ELSE
                   IF(SEQU) THEN
                    SEQU=.FALSE.
                    ELEM_PNT=ELEM_PNT+1
                    LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=IIM
                    ELEM_PNT=ELEM_PNT+1
                    LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=FIRST_SPOT
                    ELEM_PNT=ELEM_PNT+1
                    LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=LAST_SPOT
                    ELEM_PNT=0
                    SET_PNT=SET_PNT+1
                    IF(SET_PNT.GT.MAX_COLOCS) THEN
                      IF ( SPATIAL_MODE.GT.1) STATUS=-1  ! not expected to be
                                 ! a problem for nearest neighbour
                      GOTO 77    ! and abandon this fov
                    ENDIF
                  ENDIF
                ENDIF
              ENDDO
            ENDDO
  77        CONTINUE
            LUTBUF(ELMS_PER_COLOC,1,JJT,KKT)=SET_PNT-2
          ENDDO
        ENDDO

      ELSE                                    ! mapping instrument not AVHRR

        DO IIT=TARG_START_LINE,TARG_END_LINE
          KKT=(IIT-TARG_START_LINE)+1         ! LUTMAP pointer for dimension 4
          DO JJT=1,TARG_FOVS_PER_LINE         ! LUTMAP pointer for dimension 3
            SET_PNT=1                         ! LUTMAP pointer for dimension 2
            ELEM_PNT=0                        ! LUTMAP pointer for dimension 1

            ELEM_PNT=ELEM_PNT+1
            LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=
     &      TARG_MAP_POS(2,JJT,KKT)
            ELEM_PNT=ELEM_PNT+1
            LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=
     &      TARG_MAP_POS(1,JJT,KKT)
            ELEM_PNT=0
            SET_PNT=SET_PNT+1

!           check which mapping fovs are colocated with current target fov
            DO IIM=MAP_START_LINE,MAP_END_LINE
              KKM=(IIM-MAP_START_LINE)+1
              DO JJM=1,MAP_FOVS_PER_LINE

                CALL COLOC(MAP_POS_BUFF(1,JJM,KKM),
     &                     MAP_POS_BUFF(2,JJM,KKM),
     &                     TARG_POS_BUFF(1,JJT,KKT),
     &                     TARG_POS_BUFF(2,JJT,KKT),
     &                     ELLIPS(1,JJT),
     &                     ELLIPS(2,JJT),
     &                     ELLIPS(3,JJT),
     &                     INSIDE)
                IF(INSIDE) THEN
                  ELEM_PNT=ELEM_PNT+1
                  LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=IIM
                  ELEM_PNT=ELEM_PNT+1
                  LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=JJM
                  ELEM_PNT=ELEM_PNT+1
                  LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=
     &                    (MAP_POS_BUFF(1,JJM,KKM)-
     &                     TARG_POS_BUFF(1,JJT,KKT))
                  ELEM_PNT=ELEM_PNT+1
                  LUTBUF(ELEM_PNT,SET_PNT,JJT,KKT)=
     &                    (MAP_POS_BUFF(2,JJM,KKM)-
     &                     TARG_POS_BUFF(2,JJT,KKT))
                   ELEM_PNT=0
                   SET_PNT=SET_PNT+1
                   IF(SET_PNT.GT.MAX_COLOCS) THEN ! max allowed colocs exceeded
                      IF ( SPATIAL_MODE.GT.1) STATUS=-1  !  not expected to be
                                 ! a problem for nearest neighbour
                      GOTO 88    ! and abandon this fov
                  ENDIF
                ENDIF

              ENDDO
            ENDDO

 88         CONTINUE

            IF ( SET_PNT .LE. 2 ) THEN ! no colocs found
                 STATUS=12
                 GOTO 99   ! give up
            ENDIF

!           now calculate weights to be assigned to each mapping fov for
!           each mapping mode
            CALL WEIGHTS(ELMS_PER_COLOC,MAX_COLOCS,
     &                   TARG_FOVS_PER_LINE,TARG_LINES,
     &                   LUTBUF,KKT,JJT,SET_PNT-1,ELLIPS,
     &                   SPATIAL_MODE,SPATIAL_MULT,MAP_INSTRU,
     &                   STATUS)

            LUTBUF(ELMS_PER_COLOC,1,JJT,KKT)=SET_PNT-2

! NEW 25/11/96: Re-order colocations in the LUT with the nearest first
!               (saves having to do this within the mapping routine)
!
!       a)  Save LUT elements in temporary array
            DO KKM = 2, SET_PNT-1
               DO IIM = 1, ELMS_PER_COLOC
                  TMPLUT(IIM,KKM) = LUTBUF(IIM,KKM,JJT,KKT)
               ENDDO
            ENDDO
!
!       b)  Re-store LUT elements in ranked order
            DO KKM = 2, SET_PNT-1
               KK = TMPLUT(5,KKM) + 1
               DO IIM = 1, ELMS_PER_COLOC  ! not what IIM was defined for
                  LUTBUF(IIM,KK,JJT,KKT) = TMPLUT(IIM,KKM)
               ENDDO
            ENDDO
! (END OF NEW 25/11/96)

          ENDDO
        ENDDO
      ENDIF

 99   CONTINUE


      RETURN
      END

!
!
!     SUBROUTINE LOCATION.FOR
!
! Called by LUTMAP
!
! Given:-
!          data for a specific satellite,
!          data for a specific instrument,
!          a fov spot number for the instrument scan line,
!          a fov line number for the instrument scan line,
! To return:-
!          the fov position as a scan and track offset.
!
! NOTE:
!    1. 'Left' and 'right' will be determined when looking along the velocity
!       vector of the satellite.
!    2. The spot position will be in kms on a 2-dimensional rectangular
!       coordinate system whose Track axis ('Y' axis) is the sub-satellite
!       track along the earths surface (positive in the direction of the
!       velocity vector).
!       The Scan axis ('X' axis) is an 'instantaneous' scan line at the
!       origin time (and perpendicular to the Track axis), positive to the
!       right
!    3. The calculation will assume the geometry of the problem is 'nominal'
!       ie height of satellite, satellite speed, instrument scan angle
!       increment etc will be as given in a NESS 107 type document.
!    4. The 'spot' whose position is returned is that of the point of
!       intersection of the instrument beam axis and the earths surface
!    5. Distances will be curved distances along the earths surface.

! Functions
!      Double precision sine function                          SIND
!      Double precision inverse sine function                  ASIND

      SUBROUTINE LOCATION(LINE_NO,SPOT_NO,TIME_DIFF,TIME_COR,
     &                    SCAN_ANGLE_COR,SCAN_POS,TRACK_POS)

      IMPLICIT NONE
c-----------------------------------------
c
c

! Parameter data
      INTEGER
     &            LINE_NO,       ! line number of fov
     &            SPOT_NO        ! spot number of fov (the first fov for a
                                 ! scan line being number 1
      REAL
     &            TIME_DIFF,     ! (time of first line) - (origin time) (secs)
     &            TIME_COR,      ! empirical time correction for fov (secs -
                                 ! to be added)
     &            SCAN_ANGLE_COR, ! empirical scan angle correction for fov
                                 ! (deg - to be added)
     &            SCAN_POS,      ! scan position of fov as described in Note 2
     &            TRACK_POS      ! track position of fov as described in Note 2

! Satellite data
      REAL
     &            HT_OF_SAT,                       ! working value (km)
     &            SAT_ANGULAR_SPEED                ! rate of rotation
                                                   ! (radians per sec)

! Instrument data
      REAL
     &            CONE_ANGLE,                      ! working value (deg)
     &            STEP_ANGLE,                      ! working value (deg)
     &            STEP_TIME,                       ! working value (sec)
     &            LINE_TIME,                       ! working value (sec)
     &            HALFCONE_IASI                    ! working value (deg)
      INTEGER
     &            SPOTS_PER_LINE,                  ! working value
     &            LEFT_TO_RIGHT_SCAN               ! working value

! Earth and mathematical data
!  Fundamental and physical constants 
#include "constants_preproc.h"

      COMMON/INSTRUMENT/CONE_ANGLE,STEP_ANGLE,STEP_TIME,LINE_TIME,
     &       HALFCONE_IASI,SPOTS_PER_LINE,LEFT_TO_RIGHT_SCAN
      COMMON/SATELLITE/HT_OF_SAT,SAT_ANGULAR_SPEED


! Functions
      REAL
     &            SIND,
     &            ASIND

! Work areas
      REAL
     &            SPOT_TIME,     ! time of spot relative to origin time (secs)
     &            ANGLE,
     &            SCAN_ANG,
     &            NADIR_SPOT,

     &            SPOT_ANG       ! angular displacement of line from centre
                                 ! of earth to satellite to line from centre of
                                 ! earth to fov centre (deg)



!CODE STARTS

! Calculate Scan Position
!       locate centre of fov (as an angle) on an 'instantaneous'
!       scan line. The angle will be positive if the fov lies to the right of
!       the track axis.
        NADIR_SPOT=SPOTS_PER_LINE/2.0+0.5
        IF(LEFT_TO_RIGHT_SCAN.EQ.1) THEN
          SCAN_ANG=(SPOT_NO-NADIR_SPOT)*STEP_ANGLE+SCAN_ANGLE_COR   ! deg
        ELSE
          SCAN_ANG=(NADIR_SPOT-SPOT_NO)*STEP_ANGLE+SCAN_ANGLE_COR   ! deg
        ENDIF

!       using Sine rule calculate spot angle (see definition above).
!       ANGLE is angle between line from spot to centre of earth and spot to
!       satellite. NB this angle will be .GT. 90deg, value from Sine rule
!       is .LT. 90 and has to be adjusted
        ANGLE=ASIND((SIND(SCAN_ANG)*(rEarth+
     &           HT_OF_SAT))/rEarth)                         ! deg
        ANGLE=180.0-ANGLE
        SPOT_ANG=180.0-(SCAN_ANG+ANGLE)                            ! deg

!       scan position (a 'curved' distance along earth's surface)
!       is given by
        SCAN_POS=(SPOT_ANG*deg2rad)*rEarth              ! km


! Calculate Track Position

        SPOT_TIME=TIME_DIFF+
     &            (LINE_NO-1)*LINE_TIME+
     &            (SPOT_NO-1)*STEP_TIME+
     &            TIME_COR                                         !secs
        TRACK_POS=(SAT_ANGULAR_SPEED*SPOT_TIME)*rEarth       ! km


        RETURN
        END

!
!
!       SUBROUTINE ELLIPSE.FOR
!
! For a given satellite, instrument and beam angle multiplication factor to
! return for specified spot -
!       length of major axis (km)
!       length of minor axis (km)
!       'right-hand' focus of ellipse
!       30 'X/Y' positions representing the 'top right hand' quadrant of the
!       ellipse locus (which can be used to plot the ellipse)
!
! NOTE:
!    1  major axis is orientated along the scan axis and the minor axis
!       along the track axis.
!    2  'positional' information will be in kms on a rectangular coordinate
!       system whose origin will be the centre of the ellipse with the 'Y'
!       axis in the along track direction (positive in the direction of the
!       satellite velocity vector).

! Functions
!      Double precision sine function                          SIND
!      Double precision inverse sine function                  ASIND
!      Double precision tan function                           TAND
!      Double precision inverse tan function                   ATAND
!      Square root                                             SQRT
!      Nearest Integer                                         NINT


      SUBROUTINE ELLIPSE(SPOT,MAJOR_AXIS,MINOR_AXIS,FOCUS,
     &LOCUS_POINTS)

      IMPLICIT NONE
c-----------------------------------------
c
c

      INTEGER
     &            SPOT           ! spot number of fov for which ellipse
                                 ! requested

      REAL
     &            MAJOR_AXIS,    ! length of major axis of ellipse (kms)
                                 ! NB will be orientated along scan axis.
     &            MINOR_AXIS,    ! length of minor axis of ellipse (kms)
                                 ! NB will be orientated along track axis.
     &            FOCUS,         ! `right-hand' focus of ellipse (kms)
     &            LOCUS_POINTS(60)  ! 30 X/Y points (kms) relative to centre
                                 ! defining locus of ellipse in `top right hand
                                 ! quadrant' (see NO_POINTS below)
! Satellite data
      REAL
     &            HT_OF_SAT,                         ! working value (km)
     &            SAT_ANGULAR_SPEED            ! rads per sec

! Instrument data
      REAL
     &            CONE_ANGLE,                      ! working value (deg)

     &            STEP_ANGLE,                      ! working value (deg)
     &            STEP_TIME,                       ! working value (secs)
     &            LINE_TIME,                       ! working value (secs)
     &            HALFCONE_IASI                    ! working value (deg)
      INTEGER
     &            SPOTS_PER_LINE,                  ! working value
     &            LEFT_TO_RIGHT_SCAN               ! working value

! Earth and mathematical data
!  Fundamental and physical constants 
#include "constants_preproc.h"

      COMMON/INSTRUMENT/CONE_ANGLE,STEP_ANGLE,STEP_TIME,LINE_TIME,
     &       HALFCONE_IASI,SPOTS_PER_LINE,LEFT_TO_RIGHT_SCAN
      COMMON/SATELLITE/HT_OF_SAT,SAT_ANGULAR_SPEED

! Functions
      REAL
     &            SIND,
     &            ASIND,
     &            TAND,
     &            ATAND
      REAL
     &            SQRT
      INTEGER
     &            NINT

! Work areas
      REAL
     &            SCAN_ANG,      ! scan angle of instrument for specific fov
                                 ! (deg)
     &            SCAN_ANG1,SCAN_ANG2,  ! as SCAN_ANG but for points on ellipse
     &            SPOT_ANG,      ! angular displacement of line from centre
                                 ! of earth to satellite to line from centre of
                                 ! earth to spot centre (deg).
     &            SPOT_ANG1,SPOT_ANG2,  ! as SPOT_ANG but for points on ellipse
     &            DIST,          ! some defined distance in defined units
     &            ANGLE,         ! some defined angle
     &            ECEN           ! ecentricity of ellipse
      INTEGER
     &            NO_POINTS      ! number of points defining quadrant of
      PARAMETER  (NO_POINTS=30)  ! ellipse (see LOCUS_POINTS above)

! Loop controls etc     (commented out  - see comments at end of this routine)
!      INTEGER     II,JJ,KK
!      REAL        YY,XX_INCR,ASQR,BSQR,CSQR
      REAL        XX


!CODE STARTS

! Calculate Scan Position

!       first locate centre of ellipse (as an angle) on an 'instantaneous'
!       scan line. Note we are not interested here in absolute positions
!       so the sign convention has been suspended.
        SCAN_ANG=((SPOTS_PER_LINE/2.0-SPOT)+0.5)*STEP_ANGLE            ! deg

!       using Sine rule calculate spot angle (see definition above)
!       ANGLE is angle between line from spot to centre of earth and spot to
!       satellite. NB this angle will be .GT. 90deg, value from Sine rule
!       is .LT. 90 and has to be adjusted
        XX=(SIND(SCAN_ANG)*(rEarth+HT_OF_SAT))/rEarth
        IF(XX.GT.1.0.OR.XX.LT.-1.0) THEN   ! should not ! see note below
          XX=NINT(XX)
        ENDIF
        ANGLE=ASIND(XX)
        ANGLE=180.0-ANGLE
        SPOT_ANG=180.0-(SCAN_ANG+ANGLE)                                ! deg

!       now do similar calculation for extremities of ellipse in along
!       scan direction. Note that in some cases (eg when mapping MSU to
!       HIRS and the HIRS fov has to be expanded) the line from the satellite
!       to the extremity of the ellipse may not intersect the earth. In these
!       cases ANGLE is effectively set to 90deg (this is not likely to
!       cause trouble).
        SCAN_ANG1=((SPOTS_PER_LINE/2.0-SPOT)+0.5)*STEP_ANGLE+
     &           CONE_ANGLE/2.0                                        ! deg
        XX=(SIND(SCAN_ANG1)*(rEarth+HT_OF_SAT))/rEarth
        IF(XX.GT.1.0.OR.XX.LT.-1.0) THEN
          XX=NINT(XX)
        ENDIF
        ANGLE=ASIND(XX)
        ANGLE=180.0-ANGLE
        SPOT_ANG1=180.0-(SCAN_ANG1+ANGLE)                              ! deg

        SCAN_ANG2=((SPOTS_PER_LINE/2.0-SPOT)+0.5)*STEP_ANGLE-
     &           CONE_ANGLE/2.0   ! deg
        XX=(SIND(SCAN_ANG2)*(rEarth+HT_OF_SAT))/rEarth
        IF(XX.GT.1.0.OR.XX.LT.-1.0) THEN
          XX=NINT(XX)
        ENDIF
        ANGLE=ASIND(XX)
        ANGLE=180.0-ANGLE
        SPOT_ANG2=180.0-(SCAN_ANG2+ANGLE)                              ! deg

!       major axis of ellipse given by
        MAJOR_AXIS=((SPOT_ANG1-SPOT_ANG2)*deg2rad)*rEarth              ! km

!       we now need to calculate the minor axis so that we can get the
!       eccentricity and foci of the ellipse.
!       we will consider a right angled triangle given by the line from
!       the satellite to the spot and the line at right angles which is
!       the tangent to the earths surface at the spot in the direction
!       of the satellite velocity vector. The third line will be from
!       the satellite to the extremity of the ellipse, cutting the second line.

!       calc distance from the satellite to the spot via the Sine rule
        DIST=(rEarth*SIND(SPOT_ANG))/SIND(SCAN_ANG)             ! km

!       calculate the length of the `tangent' line in the above triangle
        DIST=DIST*TAND(CONE_ANGLE/2.0)                                 ! km

!       now consider the triangle consisting of the tangent line, a line
!       from the centre of the earth perpendicular to the tangent line and
!       a line from the earths centre to meet the tangent line through the
!       extremity of the ellipse. If ANGLE is the angle at the centre of the
!       earth then
        ANGLE=ATAND(DIST/rEarth)                                 ! deg
        MINOR_AXIS=((2.0*ANGLE)*deg2rad)*rEarth             ! km

        IF(MAJOR_AXIS.LT.MINOR_AXIS) THEN   ! set minor=major
          MINOR_AXIS=MAJOR_AXIS
        ENDIF

        ECEN=SQRT(1.0-MINOR_AXIS**2.0/MAJOR_AXIS**2.0)

        FOCUS=(MAJOR_AXIS/2.0)*ECEN                                    ! km

! Now calculate points to define the ellipse. This code commented out for
! LUTMAP where ellipse points are not required.

!        XX=0.0                   ! initial `X' (scan) distance from centre (km)
!        XX_INCR=(MAJOR_AXIS/2.0)/(NO_POINTS-1)  !incr in X between points (km)
!        ASQR=(MAJOR_AXIS/2.0)**2.0
!        BSQR=(MINOR_AXIS/2.0)**2.0
!        XX=XX-XX_INCR
!        KK=1                     ! a point count

!        DO JJ=1,NO_POINTS
!          XX=XX+XX_INCR
!          CSQR=XX*XX
!          IF(CSQR.GT.ASQR) THEN
!            YY=0.0
!          ELSE
!            YY=SQRT(BSQR*(1.0-CSQR/ASQR))
!          ENDIF
!          LOCUS_POINTS(KK)=XX
!          KK=KK+1
!          LOCUS_POINTS(KK)=YY
!          KK=KK+1
!        ENDDO

        RETURN
        END



      SUBROUTINE COLOC(MAP_SCAN_POS,MAP_TRACK_POS,
     &                 TARG_SCAN_POS,TARG_TRACK_POS,
     &                 MAJOR_AXIS,MINOR_AXIS,FOCUS,
     &                 INSIDE)

      IMPLICIT NONE
c-----------------------------------------
c
c

      REAL
     &         MAP_SCAN_POS,         ! km
     &         MAP_TRACK_POS,        ! km
     &         TARG_SCAN_POS,        ! km
     &         TARG_TRACK_POS,       ! km
     &         MAJOR_AXIS,           ! km
     &         MINOR_AXIS,           ! km
     &         FOCUS                 ! km

      LOGICAL
     &         INSIDE

      REAL   DISPL_SCAN,DISPL_TRACK,FR_P,FL_P

      INSIDE=.TRUE.

! check if mapping spot well away from target ellipse (NB this check is
! probably not relevant where the mapping fov is AVHRR due to the pre-check
! done before the call to COLOC)
      IF(ABS(MAP_SCAN_POS-TARG_SCAN_POS).GT.MAJOR_AXIS*0.6) THEN
        INSIDE=.FALSE.
        RETURN
      ENDIF
      IF(ABS(MAP_TRACK_POS-TARG_TRACK_POS).GT.MINOR_AXIS*0.6) THEN
        INSIDE=.FALSE.
        RETURN
      ENDIF

! do final precise check

!      get position of mapping spot relative to target
       DISPL_SCAN=MAP_SCAN_POS-TARG_SCAN_POS
       DISPL_TRACK=MAP_TRACK_POS-TARG_TRACK_POS

!      get distance of mapping spot from each focus
       FR_P=SQRT((DISPL_SCAN-FOCUS)**2+DISPL_TRACK**2)
       FL_P=SQRT((DISPL_SCAN+FOCUS)**2+DISPL_TRACK**2)

!      if combined distance is .GT. the length of the major axis of ellipse
!      then mapping spot lies outside target ellipse
       IF(FR_P+FL_P .GT. MAJOR_AXIS) THEN
         INSIDE=.FALSE.
       ENDIF

       RETURN
       END
!

      SUBROUTINE WEIGHTS(ELMS_PER_COLOC,MAX_COLOCS,
     &                   FOVS_PER_LINE,LINES,
     &                   LUTBUF,LINE,SPOT,PNT,ELLIPS,
     &                   SPATIAL_MODE,SPATIAL_MULT,MAP_INSTRU,
     &                   STATUS)
!
! To calculate 'weight' values for inclusion in the data record viz
!     element 5    rank of distances, 1=shortest distance
!             6    bi-linear interpolation weight
!             7    spatial average weight



      IMPLICIT NONE
c-----------------------------------------
c
c

      INTEGER
     &            ELMS_PER_COLOC,
     &            MAX_COLOCS,
     &            FOVS_PER_LINE,LINES,
     &            LINE,SPOT,
     &            PNT,
     &            SPATIAL_MODE,
     &            SPATIAL_MULT,
     &            MAP_INSTRU,
     &            STATUS

      REAL
     &            LUTBUF(ELMS_PER_COLOC,MAX_COLOCS,
     &                   FOVS_PER_LINE,LINES)

      REAL
     &            ELLIPS(3,90)      ! element 1 is scan axis of ellipse (km)
                                    ! for up to 90 scan positions

      REAL
     &            DIST(30),
     &            SCAN_DIST,
     &            TRACK_DIST,
     &            WEIGHT,
     &            SUM_WEIGHT,
     &            MAX_DIST,         ! in the context of a spatial average the
                                    ! distance at which a mapping fov would
                                    ! have no impact
     &            SIGMA,            ! sigma value for gaussian function
     &            TRACK_LIM         ! a critical value (km) to detect problem
                                    ! in bi-linear interpol (mapping instrument
                                    ! dependent).

      REAL
     &            SCAN,
     &            TRACK,
     &            SCANPP,
     &            SCANPM,
     &            SCANMM,
     &            SCANMP,
     &            TRACKPP,
     &            TRACKPM,
     &            TRACKMM,
     &            TRACKMP,
     &            SUM_SCAN,
     &            SUM_TRACK,
     &            MULT_SUM,
     &            PPWEIGHT,
     &            PMWEIGHT,
     &            MMWEIGHT,
     &            MPWEIGHT
      INTEGER
     &            PPII,PMII,MMII,MPII

      LOGICAL
     &            OK,OKKK,PP,PM,MM,MP

      INTEGER
     &            RANKS(30)         ! each element contains the rank of the
                                    ! distance held in the corresponding
                                    ! element of DIST (shortest ranked 1)

      INTEGER
     &            II,JJ,KK,IJ

! Functions
      REAL
     &            EXP

! Work out distances of each mapping fov from the target fov
      DO II=2,PNT
        SCAN_DIST=LUTBUF(3,II,SPOT,LINE)             ! Km
        TRACK_DIST=LUTBUF(4,II,SPOT,LINE)            ! Km
        DIST(II)=SQRT(SCAN_DIST**2+TRACK_DIST**2)    !Km
      ENDDO


! Note it is possible that two distances will be the same - it could be
! a bit awkward (eg when ranking) if this were the case. We will check here
! and make small adjustment to one distance if necessary.
      DO II=2,PNT
        DO JJ=2,PNT
          IF(II.NE.JJ) THEN
            IF(DIST(II).EQ.DIST(JJ)) THEN
              DIST(JJ)=DIST(JJ)+0.05
            ENDIF
          ENDIF
        ENDDO
      ENDDO

! Rank the distances and add to LUTBUF column 5 (NB done for all SPATIAL_MODEs)
      CALL SORT_DIST(DIST,RANKS,PNT)

      DO II=2,PNT
        LUTBUF(5,II,SPOT,LINE)=RANKS(II)
      ENDDO


      IF(SPATIAL_MODE.EQ.2) THEN    ! bi-linear interpolation
! Find the 4 mapping fovs for bi-linear interpolation.
! We wish to find the 4 mapping fovs which are `close to' the target fov and
! which collectively `surround' the target fov. Criteria - two fovs must have
! -ve scan/track displacements and two +ve scan/track displacements ie
! clockwise from top left hand corner of the `box' we have scan/track
! displacements -/+  +/+  +/-  -/-   and we search for these combinations
! from the nearest fov (NB in code below PP = scan displacement +ve, track
! displacement positive etc). Note displacements in all quadrants set
! positive for bi-linear calculation.


      PP=.FALSE.
      MM=.FALSE.
      PM=.FALSE.
      MP=.FALSE.
      OK=.FALSE.
      PPII=0          ! 'PNT' of LUTBUF containing '++' etc
      PMII=0
      MMII=0
      MPII=0
      JJ=1            ! check rank 1 first

      DO WHILE(.NOT.OK)    ! to force search on rank order
       DO II=2,PNT
        IF(.NOT.OK) THEN
         IF(RANKS(II).EQ.JJ) THEN
          JJ=JJ+1
          SCAN=LUTBUF(3,II,SPOT,LINE)              ! Km
          TRACK=LUTBUF(4,II,SPOT,LINE)             ! Km
          IF(SCAN.GE.0.AND.TRACK.GE.0.AND..NOT.PP) THEN
            PP=.TRUE.
            PPII=II
            SCANPP=SCAN
            TRACKPP=TRACK
          ELSE IF(SCAN.GE.0.AND.TRACK.LT.0.AND..NOT.PM) THEN
            PM=.TRUE.
            PMII=II
            SCANPM=SCAN
            TRACKPM=-TRACK
          ELSE IF(SCAN.LT.0.AND.TRACK.LT.0.AND..NOT.MM) THEN
            MM=.TRUE.
            MMII=II
            SCANMM=-SCAN
            TRACKMM=-TRACK
          ELSE IF(SCAN.LT.0.AND.TRACK.GE.0.AND..NOT.MP) THEN
            MP=.TRUE.
            MPII=II
            SCANMP=-SCAN
            TRACKMP=TRACK
          ENDIF
          IF(PP.AND.PM.AND.MM.AND.MP) THEN

            OK=.TRUE.

!           'consistency' check. The above process may not always select the
!           appropriate fovs. This is a consequence of the fact that the 4
!           fovs do not form a rectangle but form a trapezium (due to the time
!           delay (and hence along track movement) between successive fovs on
!           a scan line). If we have chosen appropriate fovs then MP and PP
!           track offsets should be similar (less than TRACK_LIM) (and ditto
!           for MM and PM) and scan offsets for PP and PM should be 'identical'
!           (nominally <2.0km) (and ditto for MP and MM).
!           Experience has identified two failure modes
!           (1) the fov which should have been selected for PM lies in PP and
!               wrong fovs are selected for PP and PM - the solution is to
!               force the fov selected in PP to be that for PM and select the
!               `next' PP fov (the calculation of Weights will be degraded
!               to a small degree).
!           (2) three successive fovs are selected from the same scan line.
!               (2a) If the middle of the 3 lies in MP then the MM fov will be
!               incorrect and the 'next' MM fov will replace it.
!               (2b) If the middle of the 3 lies in PM then the PP fov will be
!               incorrect and the 'next' PP fov will replace it.

            IF(MAP_INSTRU.EQ.5) THEN
              TRACK_LIM=20.0
            ELSE
              TRACK_LIM=3.0
            ENDIF

            IF(ABS(TRACKMP-TRACKPP).GT.TRACK_LIM .AND.
     &         ABS(TRACKMM-TRACKPM).GT.TRACK_LIM) THEN    ! fail mode 1
!             we now look for a LUTBUF entry which has a similar scan offset to
!             PM and a similar track offset to MP to be used for PP
              OKKK=.FALSE.
              DO KK=2,PNT
                SCAN=LUTBUF(3,KK,SPOT,LINE)        ! Km
                TRACK=LUTBUF(4,KK,SPOT,LINE)       ! Km
                IF(ABS(LUTBUF(3,PMII,SPOT,LINE)-SCAN).LT.2.0 .AND.
     &           ABS(LUTBUF(4,MPII,SPOT,LINE)-TRACK).LT.TRACK_LIM) THEN
                 OKKK=.TRUE.
                 PMII=PPII
                 SCANPM=SCANPP
                 TRACKPM=TRACKPP
                 PPII=KK
                 SCANPP=SCAN
                 TRACKPP=TRACK
                ENDIF
              ENDDO
              IF(.NOT.OKKK) THEN   ! ie have failed to find a new PP.
                PM=.FALSE.         ! We will
                PP=.FALSE.         ! assume we are at edge of scan and only
                TRACKPP=TRACKMP    ! do track interpolation (cf similar code
                TRACKPM=TRACKMM    ! below)
              ENDIF
            ENDIF


            IF(ABS(-TRACKMM-TRACKMP).LT.TRACK_LIM.AND.
     &         ABS(TRACKMP-TRACKPP).LT.TRACK_LIM) THEN    ! fail 2a
!             we now look for a LUTBUF entry which has a similar scan offset to
!             MP and a similar track offset to PM to be used for MM
              OKKK=.FALSE.
              DO KK=2,PNT
                SCAN=LUTBUF(3,KK,SPOT,LINE)        ! Km
                TRACK=LUTBUF(4,KK,SPOT,LINE)       ! Km
                IF(ABS(LUTBUF(3,MPII,SPOT,LINE)-SCAN).LT.2.0 .AND.
     &           ABS(LUTBUF(4,PMII,SPOT,LINE)-TRACK).LT.TRACK_LIM) THEN
                 OKKK=.TRUE.
                 MMII=KK
                 SCANMM=-SCAN
                 TRACKMM=-TRACK
                ENDIF
              ENDDO
              IF(.NOT.OKKK) THEN   ! ie have failed to find a new PP.
                MP=.FALSE.         ! We will
                MM=.FALSE.         ! assume we are at edge of scan and only
                                   ! do track interpolation (cf similar code
                                   ! below)
              ENDIF
            ENDIF

            IF(ABS(-TRACKMM-(-TRACKPM)).LT.TRACK_LIM.AND.
     &         ABS(-TRACKPM-TRACKPP).LT.TRACK_LIM) THEN    ! fail 2b
!             we now look for a LUTBUF entry which has a similar scan offset to
!             PM and a similar track offset to MP to be used for PP (as fail 1)
              OKKK=.FALSE.
              DO KK=2,PNT
                SCAN=LUTBUF(3,KK,SPOT,LINE)        ! Km
                TRACK=LUTBUF(4,KK,SPOT,LINE)       ! Km
                IF(ABS(LUTBUF(3,PMII,SPOT,LINE)-SCAN).LT.2.0 .AND.
     &           ABS(LUTBUF(4,MPII,SPOT,LINE)-TRACK).LT.TRACK_LIM) THEN
                 OKKK=.TRUE.
                 PPII=KK
                 SCANPP=SCAN
                 TRACKPP=TRACK
                ENDIF
              ENDDO
              IF(.NOT.OKKK) THEN   ! ie have failed to find a new PP.
                PM=.FALSE.         ! We will
                PP=.FALSE.         ! assume we are at edge of scan and only
                TRACKPP=TRACKMP    ! do track interpolation (cf similar code
                TRACKPM=TRACKMM    ! below)
              ENDIF
            ENDIF


          ENDIF
         ENDIF
        ENDIF

        IF(.NOT.OK.AND.JJ.EQ.PNT) THEN ! ie have checked each LUT entry but
                                   ! failed to find the 4 mapping fovs.
                                   ! may be edge of scan (where
                                   ! mapping scan does not extend beyond target
                                   ! scan) Or (in mapping MSU to HIRS for eg)
                                   ! the HIRS fov is too small and fails to
                                   ! capture 4 suitable fovs (STATUS set to -4)
                                   ! Track interpolation only used.
          IJ=0
          IF(MP) IJ=IJ+1
          IF(PP) IJ=IJ+1
          IF(PM) IJ=IJ+1
          IF(MM) IJ=IJ+1
!         Not a problem when we only need nearest neighbour
          IF(IJ.EQ.3 .AND. SPATIAL_MODE.GT.1) THEN
            STATUS=-4
          ENDIF


          OK=.TRUE.
          IF(MP.AND.MM) THEN
            PM=.FALSE.
            PP=.FALSE.
            TRACKPP=TRACKMP
            TRACKPM=TRACKMM
          ELSE IF(PM.AND.PP) THEN
            MP=.FALSE.
            MM=.FALSE.
          ELSE
          ENDIF

        ENDIF
       ENDDO
      ENDDO

!     Now calc weights from the displacements. NB the fact that we are working
!     with a trapezium rather than a rectangle violates bi-linear interpolation
!     assumption. However the `slope' will be small (about 3deg ??) so not
!     to worry.
      DO II=2,PNT            ! set bi-linear weights to -1
        LUTBUF(6,II,SPOT,LINE)=-1.0
      ENDDO

      IF(PP.AND.PM.AND.MM.AND.MP) THEN
       SUM_SCAN=SCANMP+SCANPP
       SUM_TRACK=TRACKPP+TRACKPM
       MULT_SUM=SUM_SCAN*SUM_TRACK
       MPWEIGHT=(TRACKPM*SCANPP)/MULT_SUM
       PPWEIGHT=(TRACKPM*SCANMP)/MULT_SUM
       PMWEIGHT=(TRACKPP*SCANMP)/MULT_SUM
       MMWEIGHT=(TRACKPP*SCANPP)/MULT_SUM


       LUTBUF(6,MPII,SPOT,LINE)=MPWEIGHT
       LUTBUF(6,PPII,SPOT,LINE)=PPWEIGHT
       LUTBUF(6,PMII,SPOT,LINE)=PMWEIGHT
       LUTBUF(6,MMII,SPOT,LINE)=MMWEIGHT
      ELSE          ! track only interpolation
       SUM_TRACK=TRACKPP+TRACKPM
       PPWEIGHT=(SUM_TRACK-TRACKPP)/SUM_TRACK
       PMWEIGHT=(SUM_TRACK-TRACKPM)/SUM_TRACK

       IF(MP) THEN
        LUTBUF(6,MPII,SPOT,LINE)=PPWEIGHT
       ENDIF
       IF(PP) THEN
        LUTBUF(6,PPII,SPOT,LINE)=PPWEIGHT
       ENDIF
       IF(PM) THEN
        LUTBUF(6,PMII,SPOT,LINE)=PMWEIGHT
       ENDIF
       IF(MM) THEN
        LUTBUF(6,MMII,SPOT,LINE)=PMWEIGHT
       ENDIF
      ENDIF

      ENDIF      ! bi-linear interpolation


! Spatial average with gaussian weights
      IF(SPATIAL_MODE.EQ.3) THEN
        SUM_WEIGHT=0.0
        SIGMA=ELLIPS(1,SPOT)*(SPATIAL_MULT/100.0)

        DO II=2,PNT
         WEIGHT=EXP(-((DIST(II)*DIST(II))/
     &          (2.0*SIGMA*SIGMA)))
         SUM_WEIGHT=SUM_WEIGHT+WEIGHT
        ENDDO

        DO II=2,PNT
         WEIGHT=EXP(-((DIST(II)*DIST(II))/
     &          (2.0*SIGMA*SIGMA)))
         LUTBUF(7,II,SPOT,LINE)=WEIGHT/SUM_WEIGHT
        ENDDO
      ENDIF
      

! Spatial average with linear weights (or dummy calculation for BG-convolution)
      IF(SPATIAL_MODE.EQ.4 .OR. SPATIAL_MODE.EQ.5) THEN
        SUM_WEIGHT=0.0
        MAX_DIST=ELLIPS(1,SPOT)*(SPATIAL_MULT/100.0)

        DO II=2,PNT
         WEIGHT=(MAX_DIST-DIST(II))/(MAX_DIST+DIST(II))
         SUM_WEIGHT=SUM_WEIGHT+WEIGHT
        ENDDO

        DO II=2,PNT
         WEIGHT=(MAX_DIST-DIST(II))/(MAX_DIST+DIST(II))
         LUTBUF(7,II,SPOT,LINE)=WEIGHT/SUM_WEIGHT
        ENDDO
      ENDIF



      RETURN
      END



!
!   Subroutine SORT_DIST.FOR  to flag each distance in a vector of distances
!   with its ranked distance. Smallest distance ranked 1.
!
      SUBROUTINE SORT_DIST(DISTS,RANKS,NUM)

      IMPLICIT NONE
c-----------------------------------------
c
c

      REAL
     &          DISTS(30)      ! contains NUM distances in some units (IN)
      INTEGER
     &          RANKS(30),     ! in each element contains rank of corresponding
                               ! element in DISTS  (OUT)
     &          NUM            ! number of distances 1-14

      INTEGER
     &          II,JJ


      DO II=1,30
        RANKS(II)=1
      ENDDO

      DO II=2,NUM
        DO JJ=2,NUM
          IF(DISTS(II).GT.DISTS(JJ)) RANKS(II)=RANKS(II)+1
        ENDDO
      ENDDO

      RETURN
      END
