      SUBROUTINE map_atms_to_cris
      IMPLICIT NONE

c-----------------------------------------
! Name: $Id: map_atms_to_cris.F 339 2012-11-26 13:30:05Z frna $
!
! Purpose:
! Use the supplied lat/lon values to obtain the positions of the CrIS samples in
! the ATMS array. This differs from the normal AAPP method of co-locating
! instruments; the advantage is that it uses the NOAA geolocations rather than
! making assumptions about the scan timing, alignment, etc.
!
! Method:
! 1) Use the mid-scan times to find the nearest ATMS line to each CrIS line
! 2) Convert all lat/lons to x/y/z/co-ordinates
! 3) Compute the ATMS co-ordinates of each CrIS spot (see below)
!
! Points are named as follows, where A, O, B are successive ATMS scans and C, O,
! D are successive ATMS scan positions. P is the CrIS point to be located.
!
!      B
!        P
!   C  O  D
!
!      A
!
!  Delta_y = 2*(OP x CD).(AB x CD) / |AB x CD|^2
!  Delta_x = -2*(OP x AB).(AB x CD) / |AB x CD|^2
!
! inputs (in ppcrisstore.h and ppatmsstore.h):
!   atmstime(atmsnfovs,atmsnlines)
!   atmslats(atmsnfovs,atmsnlines)
!   atmslons(atmsnfovs,atmsnlines)
!   cristime(crisnfor,crisnlines)
!   crislats(crisnfovs,crisnlines)  !fov 1-30 = detector 1, etc.
!   crislons(crisnfovs,crisnlines)
!
! outputs (in ppcrisstore.h):
!   crismapbt(atmsnchan,crisnfovs,crisnlines)   !mapped BT in K
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     20/08/10  N.C.Atkinson
!  1.1     03/07/2012 Set crisnedtcoldatms and crisnedtwarmatms
!  1.2     26/11/2012 Set mapped BT to missing if lunar contamination
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:

#include "ppparms.h"
#include "ppcrisstore.h"
#include "ppatmsstore.h"
#include "bits.h"

!Local variables:
      INTEGER ctimes(crisnlines)  !Time at centre of scan
      INTEGER atimes(atmsnlines)
      INTEGER cscan, ascan, cspot, aspot, cfor, cfov, achan, i
      INTEGER refscan(crisnlines) !Reference ATMS scan
      INTEGER aspotnums(crisnfor) !ATMS spot for each CrIS field of regard
      INTEGER scan0,scan1,scanN,spot0,spot1,spotN

      REAL ax(atmsnfovs,atmsnlines)  !Geolocation in x/y/z units
      REAL ay(atmsnfovs,atmsnlines)
      REAL az(atmsnfovs,atmsnlines)
      REAL cx(crisnfovs,crisnlines)  !Geolocation in x/y/z units
      REAL cy(crisnfovs,crisnlines)
      REAL cz(crisnfovs,crisnlines)
      
      REAL O_vec(3),A_vec(3),B_vec(3),C_vec(3),D_vec(3),P_vec(3)  !Vectors
      REAL AB_vec(3),CD_vec(3),OP_vec(3)
      REAL ABCD_vec(3),OPCD_vec(3),OPAB_vec(3)
      REAL ABfactor,CDfactor,norm,tdiff
      REAL scanindex,spotindex,dscan,dspot
      REAL coef00,coef01,coef10,coef11
      
      LOGICAL startgap, endgap

!Functions:
      REAL dotprod
!
!- End of header

      WRITE(*,*)'         Mapping ATMS to CrIS'

!Time stamp at centre of each scan

      DO cscan=1,crisnlines
        ctimes(cscan) = cristime(crisnfor/2,cscan)
      ENDDO
      DO ascan=1,atmsnlines
        atimes(ascan) = atmstime(atmsnfovs/2,ascan)
      ENDDO
  
!For each CrIS scan find the nearest ATMS scan

      DO cscan=1,crisnlines
        refscan(cscan) = -9999
        IF (crisrecdno(cscan) .NE. MDI) THEN
          DO ascan=1,atmsnlines
            IF (atmsrecdno(ascan) .NE. MDI) THEN
              tdiff = abs(ctimes(cscan)-atimes(ascan))
              IF (tdiff .lt. 2000) refscan(cscan) = ascan   !within 2 seconds
            ENDIF
          ENDDO
        ENDIF
      ENDDO

!Convert all the lat/lons to x/y/z

      CALL latlon_to_xyz (atmsnfovs,atmsnlines,atmslats,atmslons,
     &     ax,ay,az)
      CALL latlon_to_xyz (crisnfovs,crisnlines,crislats,crislons,
     &     cx,cy,cz)
  
!Reference ATMS spot numbers assuming 3:1 ratio in the scan steps

      DO i=1,crisnfor
        aspotnums(i) = i*3 + 2      !5,8,...92
      ENDDO

!Loop over CrIS scan lines

      DO cscan=1,crisnlines
        ascan = refscan(cscan)
        IF (ascan .GE. 1) THEN

          IF (ascan .EQ. 1) THEN
            startgap = .true.
          ELSE
            IF (atmsrecdno(ascan-1) .EQ. MDI .OR. 
     &          atimes(ascan)-atimes(ascan-1) .GT. 3000) THEN
              startgap = .true.
            ELSE
              startgap = .false.
            ENDIF
          ENDIF

          IF (ascan .EQ. atmsnlines) THEN
            endgap = .true.
          ELSE
            IF (atmsrecdno(ascan+1) .EQ. MDI .OR.
     &          atimes(ascan+1)-atimes(ascan) .GT. 3000) THEN
              endgap = .true.
            ELSE
              endgap = .false.
            ENDIF
          ENDIF

          IF (.not.(startgap .and. endgap)) THEN  !not an isolated ATMS line

!Loop over CrIS fields of regard

            DO cfor=1,crisnfor
              aspot = aspotnums(cfor)
              O_vec(1) = ax(aspot,ascan)  !same origin for all 9 FOVs
              O_vec(2) = ay(aspot,ascan)
              O_vec(3) = az(aspot,ascan)

              IF (startgap) THEN
                A_vec(1) = O_vec(1)
                A_vec(2) = O_vec(2)
                A_vec(3) = O_vec(3)
                B_vec(1) = ax(aspot,ascan+1)
                B_vec(2) = ay(aspot,ascan+1)
                B_vec(3) = az(aspot,ascan+1)
                ABfactor = 1.0
              ELSE IF (endgap) THEN
                A_vec(1) = ax(aspot,ascan-1)
                A_vec(2) = ay(aspot,ascan-1)
                A_vec(3) = az(aspot,ascan-1)
                B_vec(1) = O_vec(1)
                B_vec(2) = O_vec(2)
                B_vec(3) = O_vec(3)
                ABfactor = 1.0
              ELSE
                A_vec(1) = ax(aspot,ascan-1)
                A_vec(2) = ay(aspot,ascan-1)
                A_vec(3) = az(aspot,ascan-1)
                B_vec(1) = ax(aspot,ascan+1)
                B_vec(2) = ay(aspot,ascan+1)
                B_vec(3) = az(aspot,ascan+1)
                ABfactor = 2.0
              ENDIF

              C_vec(1) = ax(aspot-1,ascan)
              C_vec(2) = ay(aspot-1,ascan)
              C_vec(3) = az(aspot-1,ascan)
              D_vec(1) = ax(aspot+1,ascan)
              D_vec(2) = ay(aspot+1,ascan)
              D_vec(3) = az(aspot+1,ascan)
              CDfactor = 2.0

              DO i=1,3
                AB_vec(i) = B_vec(i)-A_vec(i)
                CD_vec(i) = D_vec(i)-C_vec(i)
              ENDDO
                
              CALL crossprod(AB_vec,CD_vec,ABCD_vec)
              norm = dotprod(ABCD_vec,ABCD_vec)
              DO i=1,3
                ABCD_vec(i) = ABCD_vec(i)/norm
              ENDDO

              DO cfov=1,9
                cspot = (cfov-1)*crisnfor + cfor
                P_vec(1) = cx(cspot,cscan)
                P_vec(2) = cy(cspot,cscan)
                P_vec(3) = cz(cspot,cscan)
                DO i=1,3
                  OP_vec(i) = P_vec(i) - O_vec(i)
                ENDDO
                
                CALL crossprod(OP_vec,CD_vec,OPCD_vec)
                CALL crossprod(OP_vec,AB_vec,OPAB_vec)

                scanindex = ascan + 
     &             CDfactor*dotprod(OPCD_vec,ABCD_vec)
                spotindex = aspot - 
     &             ABfactor*dotprod(OPAB_vec,ABCD_vec)

!If we are outside the permitted range, use the nearest available

                IF (scanindex .LT. 1.0) scanindex = 1.0
                IF (scanindex .GT. atmsnlines) scanindex = atmsnlines
                IF (spotindex .LT. 1.0) spotindex = 1.0
                IF (spotindex .GT. atmsnfovs) spotindex = atmsnfovs

!Bilinear interpolation of the 4 neighbouring brightness temperatures

                scan0 = INT(scanindex)
                scan1 = MIN(INT(scanindex+1), atmsnlines)
                spot0 = INT(spotindex)
                spot1 = MIN(INT(spotindex+1), atmsnfovs)
                scanN = MIN(NINT(scanindex),atmsnlines)   !Nearest
                spotN = MIN(NINT(spotindex),atmsnfovs)
                
!If one scan is missing, use only the other scan
                IF (atmsrecdno(scan0).NE.MDI .OR. 
     &              atmsrecdno(scan1).NE.MDI) THEN
                  IF (atmsrecdno(scan0).EQ.MDI) scan0 = scan1
                  IF (atmsrecdno(scan1).EQ.MDI) scan1 = scan0

                  dscan = scanindex - scan0
                  dspot = spotindex - spot0
                  coef00 = (1.0 - dscan)*(1.0 - dspot)
                  coef01 = dscan*(1.0 - dspot)
                  coef10 = (1.0 - dscan)*dspot
                  coef11 = dscan*dspot
                
                  DO achan=1,atmsnchan
                    IF (atmsbtemps(achan,spot0,scan0).GE.atmsminBT.AND.
     &                atmsbtemps(achan,spot0,scan1).GE.atmsminBT.AND.
     &                atmsbtemps(achan,spot1,scan0).GE.atmsminBT.AND.
     &                atmsbtemps(achan,spot1,scan1).GE.atmsminBT.AND.
     &                atmsbtemps(achan,spot0,scan0).LE.atmsmaxBT.AND.
     &                atmsbtemps(achan,spot0,scan1).LE.atmsmaxBT.AND.
     &                atmsbtemps(achan,spot1,scan0).LE.atmsmaxBT.AND.
     &                atmsbtemps(achan,spot1,scan1).LE.atmsmaxBT) THEN
                      crismapbt(achan,cspot,cscan) = 
     &                  coef00*atmsbtemps(achan,spot0,scan0) + 
     &                  coef01*atmsbtemps(achan,spot0,scan1) + 
     &                  coef10*atmsbtemps(achan,spot1,scan0) + 
     &                  coef11*atmsbtemps(achan,spot1,scan1)
                    ENDIF

!If nearest ATMS scan is lunar-contaminated, set the mapped BT to missing

                    IF (BTEST(atmschanqual(achan,scanN),BITS(9))) THEN
                      crismapbt(achan,cspot,cscan) = RMDI
                    ENDIF

                  ENDDO    !loop over ATMS channels
                ENDIF
                
!Nearest neighbour for the NWC SAF scattering variables

                IF (atmsrecdno(scanN).NE.MDI) THEN
                  crism0(cspot,cscan) = atmsm0(spotN,scanN)
                  crism1(cspot,cscan) = atmsm1(spotN,scanN)
                  crism2(cspot,cscan) = atmsm2(spotN,scanN)
                  crism3(cspot,cscan) = atmsm3(spotN,scanN)
                  crissi_nwcsaf(cspot,cscan) = 
     &               atmssi_nwcsaf(spotN,scanN)
                  crissi_nwcsaf_flag(cspot,cscan) = 
     &               atmssi_nwcsaf_flag(spotN,scanN)

!For NEDT use the nearest ATMS line at the centre of the CrIS scan

                  IF (cfov.eq.5.and.cfor.eq.crisnfor/2) THEN
                    DO achan=1,atmsnchan
                      crisnedtcoldatms(achan,cscan) =
     &                    atmsnedtcold(achan,scanN)
                      crisnedtwarmatms(achan,cscan) =
     &                    atmsnedtwarm(achan,scanN)
                    ENDDO
                  ENDIF
                ENDIF
              ENDDO    !loop over fov
            ENDDO    !loop over cfor
          ENDIF    !not an isolated line
        ENDIF    !ascan OK
      ENDDO    !loop over CrIS scan lines
      END

!*******************************************************************************

      SUBROUTINE latlon_to_xyz(n1,n2,lat,lon,x,y,z)
      IMPLICIT NONE
      
#include "constants_preproc.h"
      
!Arguments:
      INTEGER n1,n2
      REAL lat(n1,n2),lon(n1,n2),x(n1,n2),y(n1,n2),z(n1,n2)
     
!Local variables:
      INTEGER i,j
      REAL coslat
      
      DO i=1,n1
        DO j=1,n2
          coslat = cos(lat(i,j)*deg2rad)
          x(i,j) = cos(lon(i,j)*deg2rad)*coslat
          y(i,j) = sin(lon(i,j)*deg2rad)*coslat
          z(i,j) = sin(lat(i,j)*deg2rad)
        ENDDO
      ENDDO
      END

!*******************************************************************************

      FUNCTION dotprod(A,B)       !Scalar product of 3-element vectors
      IMPLICIT NONE
      REAL A(3), B(3)  !Inputs
      REAL dotprod     !output
      dotprod = A(1)*B(1) + A(2)*B(2) + A(3)*B(3)
      RETURN
      END

!*******************************************************************************

      SUBROUTINE crossprod(A,B,C)     !Vector product of 3-element vectors
      IMPLICIT NONE
      REAL A(3), B(3)  !Inputs
      REAL C(3)        !output
      C(1) = A(2)*B(3) - A(3)*B(2)
      C(2) = A(3)*B(1) - A(1)*B(3)
      C(3) = A(1)*B(2) - A(2)*B(1)
      RETURN
      END
