!+ apply median filter on instrument grid
!
      SUBROUTINE MEDIAN ( BTEMPS, BTqual, BitQC, Chan, Nchan,
     &                    SurfType, SurfOption, Nfovs, Nlines,
     &                    BoxM, BoxN, BTmed )

      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: median.F 5 2006-01-04 09:11:15Z frna $
!
! Purpose:
!   For each fov, calculate median BT in grid box centred on fov,
!   and subtract from BT at centre.
!
! Method:
!   For BoxM x BoxN box centred on each line/fov, calculate & store median BT
!   Size of box is necessarily reduced near edges of block. This is done
!   symmetrically as far as possible, down to a 2x2 box in each corner.
!
!   There is an option of treating each surface type separately.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      16/01/97  Bug fixed (see code). (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!   Scalar arguments with intent(In):
      INTEGER       BitQC         ! bit "BitQC" of "BTqual" is set if BT is bad
      INTEGER       Chan          ! channel number to use
      INTEGER       Nchan         ! 1st dimension of BTEMPS (num of channels)
      INTEGER       Nfovs         ! number of fovs per scanline
      INTEGER       BoxM          ! size of filter box in pixels, along scan
      INTEGER       BoxN          ! size of filter box in pixels, across scan
!                                   BoxM,N should both be odd numbers
      INTEGER       Nlines        ! number of lines in BTEMPS
      INTEGER       SurfOption    ! to indicate processing of surface types.
                                  !     0=map irrespective of surface types
                                  !     1=map only if surface type of mapping
                                  !       fov is same as that of target fov
!
!   Array arguments with intent(In):
      REAL          BTEMPS(Nchan,Nfovs,Nlines) ! brightness temperatures
      INTEGER       BTqual(Nfovs,Nlines)       ! QC flags for each fov
      INTEGER       SurfType(Nfovs,Nlines)     ! surface type of each fov
!
!   Array arguments with intent(Out):
      REAL          BTmed(Nfovs,Nlines)       ! median BT in box
!
! Local parameters:
      INTEGER       maxBoxMN       ! max value of BoxM*BoxN allowed
        PARAMETER ( maxBoxMN = 100)
      INTEGER       maxFOV         ! max no. of fovs / scanline allowed
        PARAMETER ( maxFOV = 100)
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       halfM, halfN  ! half of BoxM,N  (rounded down!)
      INTEGER       i             ! general purpose index
      INTEGER       line          ! loop index: pointer to scanline in BTEMPS
      INTEGER       fov           ! loop index: fov number
      LOGICAL       fov_ok        ! TRUE if box fov is OK to use
      INTEGER       linej         ! loop index: pointer to scanline in box
      INTEGER       fovj          ! loop index: fov number in box
      INTEGER       line0, line1  ! start and end line of each box
      INTEGER       p0, p1        ! start and end fov of each box
      INTEGER       dline, dfov   ! used in calculating the above
      INTEGER       n_store       ! number of BTs saved in bt_store
      INTEGER       surface       ! surface type (from SurfType) of box fov
      REAL          bt_median     ! median BT within the box
!
! Local arrays
      INTEGER       fov0(maxFOV)  ! start fov of each box, for each fov
      INTEGER       fov1(maxFOV)  ! end fov of each box, for each fov
      INTEGER       bt_rank(maxBoxMN)  ! ranking of BTs within box
      REAL          bt_store(maxBoxMN) ! save BTs within box & sort them
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External RRANK              ! rank real array by value
!
!- End of header

! 0.  Perform set-up & check parameters
!     ------- ------ - ----- ----------

! 0.1 Are there too many fovs per scanline ?
      IF ( Nfovs.LT.1 .OR. Nfovs.GT.maxFOV ) THEN
         errMessage = " "
         WRITE ( errMessage,
     &   "(""Nfovs = "",I8,""   - must be < "",I8)" ) Nfovs, maxFOV
         errStatus = 1  ! fatal
         CALL ErrorReport(errStatus,errMessage,"MEDIAN")
      ENDIF

! 0.2 Define halflength/width of box. BoxM and BoxN should(!) be odd numbers.
      halfM = BoxM / 2
      halfN = BoxN / 2
!     Is the box too big ?
      IF ( (2*halfM+1)*(2*halfN+1).GT.maxBoxMN ) THEN
         errMessage = " "
         WRITE ( errMessage,
     &   "(""BoxM,N = "",2I8,""   - must be < "",I8)" )
     &   BoxM, BoxN, maxBoxMN
         errStatus = 2  ! fatal
         CALL ErrorReport(errStatus,errMessage,"MEDIAN")
      ENDIF

! 0.3 For efficiency, calculate & store here which fovs should
!     be included in the box for each scan position.

      DO fov = 1, Nfovs

!        define the box width ..
         dfov = halfN
!        .. keeping the box symmetrical near the edges ..
         dfov = MIN( dfov, fov-1, Nfovs-fov )
         p0 = fov - dfov   ! first fov of box
         p1 = fov + dfov   ! last fov of box
!        .. but not too small
         IF ( halfM.GT.1 ) THEN
            IF ( fov.EQ.1 ) THEN
               p1 = fov + 1 ! last fov of box
            ELSEIF ( fov.EQ.Nfovs ) THEN
               p0 = fov - 1 ! first fov of box
            ENDIF
         ENDIF
!        Store fov numbers
         fov0(fov) = p0    ! first fov of box
         fov1(fov) = p1    ! last fov of box

      ENDDO  ! fov


! 1.  loop through lines & fovs, applying filter
!     ---- ------- ----- - ----- -------- ------

! 1.1 loop through scanlines
      DO line = 1, Nlines

! 1.2    define the box length ..
         dline = halfM
!        .. keeping the box symmetrical near the edges ..
         dline = MIN( dline, line-1, Nlines-line )
         line0 = line - dline   ! first line of box
         line1 = line + dline   ! last line of box
!        .. but not too small
         IF ( halfM.GT.1 ) THEN
            IF ( line.EQ.1 ) THEN
               line1 = line + 1 ! last line of box
            ELSEIF ( line.EQ.Nlines ) THEN
               line0 = line - 1 ! first line of box
            ENDIF
         ENDIF

! 1.3    loop through the fovs in this line
         DO fov = 1, Nfovs

            IF ( .NOT. BTEST(BTqual(fov,line),BitQC) ) THEN

               n_store = 0   ! number of valid BTs in box
               surface = SurfType(fov,line)

! 1.4          For this line & fov, store BTs in surrounding box
               DO linej = line0, line1
               DO fovj = fov0(fov), fov1(fov)

!                 check QC flag
                  fov_ok = .NOT. BTEST(BTqual(fovj,linej),BitQC)

!                 if required, check surface type (unless 'mixed')
                  IF (SurfOption.NE.0 .AND. surface.NE.1 )
     &               fov_ok = fov_ok .AND.
     &                        SurfType(fovj,linej) .EQ. surface

!                 store BT from box in array bt_store
                  IF (fov_ok) THEN
                     n_store = n_store + 1
                     bt_store(n_store) = BTEMPS(Chan,fovj,linej)
                  ENDIF  ! fov_ok

               ENDDO  ! fovj
               ENDDO  ! linej

! 1.5          look at the values in the box

!              rank BTs in increasing order
               CALL RRANK ( n_store, bt_store, bt_rank )

!              find the "median" value
               IF ( MOD(n_store,2).EQ.1 ) THEN  ! median is unique
! (16/01/97) old version had bug:  bt_median = bt_store(bt_rank(n_store/2))
                  bt_median = bt_store(bt_rank(1+n_store/2))
               ELSE                             ! median is ambiguous
                  i = n_store / 2               ! -take mean of either side
                  bt_median = 0.5 *
     &              (bt_store(bt_rank(i))+bt_store(bt_rank(i+1)))
               ENDIF

               BTmed(fov,line) = bt_median

            ENDIF  ! central fov OK

         ENDDO  ! fov

      ENDDO  ! line


      RETURN
      END
