!+
!
      SUBROUTINE MODIFY_BEAMWIDTH ( nx, ny, image, sampling_dist, 
     &   beamwidth, newwidth, mtfcutoff, nxaverage, nyaverage, qc_dist)
     
      IMPLICIT NONE
!-----------------------------------------
! Name: $Id: modify_beamwidth.F 444 2015-04-27 13:02:03Z frna $
!
! Purpose:
!   Manipulate the effective beam width of an image. For example, convert ATMS
!   to AMSU-A-like resolution while reducing the noise.
!
! Method:
!   1) Pad the image to a power of 2 in each dimension.
! If FFT technique is to be used then: 
!   2) Assuming Gaussian beam shapes, calcluate the input and output Modulation
!      Transfer Functions (MTF).
!   3) FFT image to frequency domain (2-D).
!   4) Multiply by output MTF divided by input MTF. If a cut-off is specified
!      (when attempting to make the beam width narrower), attenuate further
!      by an exponential function - factor of 2 at the cutoff. 
!   5) FFT back to image domain 
! Finally,
!   6) Over-write the input image, with averaging if requested.
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0   22/07/2010   N.C.Atkinson
!  1.1   18/07/2012   Add maxval (some bad ATMS data have BT >=330K)
!  1.2   23/04/2015   Allow nxaverage & nyaverage to be even - edge points
!                     weighted by factor 0.5.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

! Parameters
      INTEGER nxmax
      PARAMETER (nxmax=128)    !Max number of spots per scan line
      INTEGER nymax
      PARAMETER (nymax=4096)   !Max number of lines. Allow full orbit ATMS
      REAL minval
      PARAMETER (minval=0.0)   !Values less than this are treated as missing
      REAL maxval
      PARAMETER (maxval=33000.0) !Values more than this are treated as missing

! Arguments
      INTEGER nx, ny         !Size of image
      REAL    image(nx,ny)   !BT or radiance image
      REAL    sampling_dist  !typically degrees
      REAL    beamwidth      !ditto
      REAL    newwidth       !ditto
      REAL    mtfcutoff      !0.0 to 1.0
      INTEGER nxaverage      !Number of samples to average (or zero)
      INTEGER nyaverage      !Number of samples to average (or zero)
      INTEGER qc_dist        !Number of samples around missing data to discard
      
! Local variables
      INTEGER nxpad, nypad, dx, dy
      INTEGER i,j,k,ix,iy,ii,jj,deltax,minii,minjj,maxii,maxjj
      INTEGER jfirst
      INTEGER xpow2, ypow2
      INTEGER nxav2, nyav2, naverage
      REAL    mtfxin(nxmax),mtfxout(nxmax)
      REAL    mtfyin(nymax),mtfyout(nymax)
      REAL    mtfin,mtfout
      REAL    mtfpad(nxmax,nymax)
      REAL    imagepad(nxmax,nymax)
      REAL    work(nymax)
      REAL    f,df,factor
      REAL    PI, LN2, LN2LNcsquared, MTF_constant_in, MTF_constant_out
      LOGICAL missing, gooddata(nxmax,nymax)

      REAL    firstfactor_x  !set to 0.5 if nxaverage is >0 and even, else 1.0
      REAL    firstfactor_y  !set to 0.5 if nyaverage is >0 and even, else 1.0
      REAL    fx, fy

! Function & Subroutine calls:

      EXTERNAL SFFTCF,SFFTCB           !Real -> Complex Fast Fourier Transform

! End of declarations
!-----------------------------------------
      
      PI = 4.0*atan(1.0)
      LN2 = alog(2.0)
      IF (mtfcutoff .GT. 0.0) LN2LNcsquared = LN2/alog(mtfcutoff)**2
      nxav2 = nxaverage/2
      nyav2 = nyaverage/2
      naverage = nxaverage*nyaverage

      firstfactor_x = 1.0
      IF (nxav2*2.eq.nxaverage.and.nxaverage.ne.0) firstfactor_x = 0.5
      firstfactor_y = 1.0
      IF (nyav2*2.eq.nyaverage.and.nyaverage.ne.0) firstfactor_y = 0.5

!1) Pad the image up to the nearest power of 2 in each dimension, by reversing
!the points near the edge.

      xpow2 = INT(ALOG(nx*1.0)/LN2 + 1.0)
      ypow2 = INT(ALOG(ny*1.0)/LN2 + 1.0)
      nxpad = 2**xpow2
      nypad = 2**ypow2
      dx = (nxpad - nx)/2
      dy = (nypad - ny)/2
      
      IF (nypad .GT. nymax) THEN
        write(*,*)'Error: too many ATMS scan lines'
        write(*,*)'Please use fewer than',nymax, ' scan lines'
        write(*,*)'or increase the value of nymax in modify_beamwidth.F'
        CALL AAPPEXIT(1)
      ENDIF
      
!Loop over scan positions

      DO i=dx+1,dx+nx
        DO j=dy+1,dy+ny
          imagepad(i,j) = image(i-dx,j-dy)   !Take a copy of the input data
          IF (imagepad(i,j).LT.minval.OR.imagepad(i,j).GE.maxval) THEN
            gooddata(i-dx,j-dy) = .false.
          ELSE
            gooddata(i-dx,j-dy) = .true.
          ENDIF
        ENDDO

!Interpolate missing points in the along-track direction

        jfirst = -1
        missing = .false.
        
        DO j=dy+1,dy+ny
          IF (.not.missing) THEN
            IF (imagepad(i,j).GE.minval.AND.imagepad(i,j).LT.maxval)THEN
              jfirst = j
            ELSE
              missing = .true.
            ENDIF
          ELSE
            IF (imagepad(i,j).GE.minval.AND.imagepad(i,j).LT.maxval)THEN
              missing = .false.            !First good point after missing
              IF (jfirst .eq. -1) THEN
                DO k=dy+1,j-1
                  imagepad(i,k) = imagepad(i,j)      !Constant
                ENDDO
              ELSE
                DO k=jfirst+1,j-1
                  factor = (j-k)*1.0/(j-jfirst)      !Interpolate
                  imagepad(i,k) = imagepad(i,jfirst)*factor + 
     &                            imagepad(i,j)*(1.0-factor)
                ENDDO
              ENDIF
            ENDIF
          ENDIF
        ENDDO
        IF (missing) THEN         !Last scan is missing
          IF (jfirst .GE. 1) then
            DO k=jfirst+1,dy+ny
              imagepad(i,k) = imagepad(i,jfirst)     !Constant
            ENDDO
          ENDIF
        ENDIF          

!Continue padding the edges

        DO j=1,dy
          imagepad(i,j) = imagepad(i,dy+dy+2-j)
        ENDDO
        DO j=ny+dy+1,nypad
          imagepad(i,j) = imagepad(i,ny+dy+ny+dy-j)
        ENDDO
      ENDDO
      DO i=1,dx
        DO j=1,nypad
          imagepad(i,j) = imagepad(dx+dx+2-i,j)
        ENDDO
      ENDDO
      DO i=nx+dx+1,nxpad
        DO j=1,nypad
          imagepad(i,j) = imagepad(nx+dx+nx+dx-i,j)
        ENDDO
      ENDDO

!2) Compute the MTF modifications. Assume beams are Gaussian.

      IF (newwidth .GT. 0) THEN
        MTF_Constant_in = (PI*beamwidth/(2*sampling_dist))**2/LN2
        MTF_Constant_out = (PI*newwidth/(2*sampling_dist))**2/LN2
        df = 1.0/nxpad
        DO i=1,nxpad/2+1
          f = df*(i-1)      !DC to Nyquist
          mtfxin(i) = exp(-MTF_Constant_in * f**2)
          mtfxout(i) = exp(-MTF_Constant_out * f**2)
          IF (i.GT.1.AND.i.LT.nxpad/2+1) THEN
            mtfxin(nxpad-i+2) = mtfxin(i)
            mtfxout(nxpad-i+2) = mtfxout(i)
          ENDIF
        ENDDO
        df = 1.0/nypad
        DO i=1,nypad/2+1
          f = df*(i-1)      !DC to Nyquist
          mtfyin(i) = exp(-MTF_Constant_in * f * f)
          mtfyout(i) = exp(-MTF_Constant_out * f * f)
          IF (i.GT.1.AND.i.LT.nypad/2+1) THEN
            mtfyin(nypad-i+2) = mtfyin(i)
            mtfyout(nypad-i+2) = mtfyout(i)
          ENDIF
        ENDDO
        DO j=1,nypad
          DO i=1,nxpad
            mtfin = mtfxin(i)*mtfyin(j)
            mtfout = mtfxout(i)*mtfyout(j)
            if (mtfcutoff .GT. 0.0) THEN
              mtfpad(i,j) = (mtfout * 
     &          exp(-LN2LNcsquared*(alog(mtfout))**2))/mtfin
            else
              mtfpad(i,j) = mtfout/mtfin
            endif
          ENDDO
        ENDDO
      
!3) Fourier transform, line by line then column by column.
!After each FFT, points 1 to nxpad/2+1 contain the real part of the spectrum,
!the rest contain the imaginary part in reverse order.

        DO j=1,nypad
          CALL SFFTCF(imagepad(1,j),nxpad,xpow2)
        ENDDO
        DO i=1,nxpad
          DO j=1,nypad
            work(j) = imagepad(i,j)
          ENDDO
          CALL SFFTCF(work,nypad,ypow2)
          DO j=1,nypad
            imagepad(i,j) = work(j)
          ENDDO
        ENDDO

!4) Multiply the spectrum by the MTF factor

        DO j=1,nypad
          DO i=1,nxpad
            imagepad(i,j) = imagepad(i,j)*mtfpad(i,j)
          ENDDO
        ENDDO
      
!5) Inverse Fourier transform, column by column then line by line 

        DO i=1,nxpad
          DO j=1,nypad
            work(j) = imagepad(i,j)
          ENDDO
          CALL SFFTCB(work,nypad,ypow2)
          DO j=1,nypad
            imagepad(i,j) = work(j)
          ENDDO
        ENDDO
        DO j=1,nypad
          CALL SFFTCB(imagepad(1,j),nxpad,xpow2)
        ENDDO
      ENDIF   !New width is specified

!6) Over-write the input image (good points only)

      DO j=1,ny
        DO i=1,nx
          IF (gooddata(i,j)) THEN
            IF (nxav2 .EQ. 0 .AND. nyav2 .EQ. 0) THEN
              image(i,j) = imagepad(i+dx,j+dy)
            ELSE
              image(i,j) = 0.0             !Do averaging
              DO iy = -nyav2,nyav2
	        fy = 1.0
		IF (iy.eq.-nyav2.or.iy.eq.nyav2) fy = firstfactor_y
                DO ix = -nxav2,nxav2
	          fx = 1.0
		  IF (ix.eq.-nxav2.or.ix.eq.nxav2) fx = firstfactor_x
                  image(i,j)=image(i,j)+fx*fy*imagepad(i+dx+ix,j+dy+iy)
                ENDDO
              ENDDO
              image(i,j) = image(i,j)/naverage
            ENDIF
          ENDIF
        ENDDO
      ENDDO

!7) Reset points close to bad or missing points, based on qc_dist

      IF (qc_dist .GT. 0) THEN
        DO j=1,ny
          DO i=1,nx
            IF (.not. gooddata(i,j)) THEN
              minjj = MAX(j-qc_dist,1)
              maxjj = MIN(j+qc_dist,ny)
              DO jj=minjj,maxjj
                deltax = NINT(SQRT(REAL(qc_dist**2-(abs(jj-j)-0.5)**2)))
                minii = MAX(i-deltax,1)
                maxii = MIN(i+deltax,nx)
                DO ii=minii,maxii
                  image(ii,jj) = -999999.
                ENDDO
              ENDDO
            ENDIF
          ENDDO
        ENDDO
        DO j=1,qc_dist       !Reset the beginning and ending scans also
          DO i=1,nx
            image(i,j) = -999999.
          ENDDO
        ENDDO
        DO j=ny-qc_dist,ny
          DO i=1,nx
            image(i,j) = -999999.
          ENDDO
        ENDDO
      ENDIF

      RETURN
      END    
