!+ invert a real matrix
!
      SUBROUTINE MXINV ( A, N, LDA, SCRATCH, DET, TOL )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: mxinv.F 5 2006-01-04 09:11:15Z frna $
!
! Purpose:
!   invert a real matrix
!
! Method:
!   Solves systems of linear equations by inverting a square matrix
!   The standard Gauss-Jordan method is used: a partial pivot search
!   (one unused row), followed by Gauss-Jordan reduction.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Origin unknown!  (R.J.Renshaw)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!   Scalar arguments with intent(In):
      INTEGER       N             ! order of matrix A
      INTEGER       LDA           ! leading dimension of array A
      REAL          TOL    ! lower limit for the determinant"s partial products.
!                            Matrix A is declared singular once the partial
!                            product of pivot elements is less than or equal in
!                            magnitude to this parameter, which should be +ve
!
!     Array arguments with intent(InOut):
      REAL          A(LDA,LDA)    ! A contains the square matrix to be inverted
!                                   A is overwritten by the inverse of A
      INTEGER       SCRATCH(N,2)  ! work array
!
!     Scalar arguments with intent(Out):
      REAL          DET           ! determinant of A
!
! Local scalars:
      INTEGER       I, J, K       ! general purpose loop indices
      REAL          BIG           ! largest element in sub-matrix
      REAL          TEMP          ! temporary storage of array element
!
!- End of header


!     Search for largest element
!     ------ --- ------- -------

      DET=1.
      DO K=1,N
        SCRATCH(K,1)=K
        SCRATCH(K,2)=K
        BIG=A(K,K)         ! get diagonal element

!       Look for largest element in sub-matrix A(K:N,K:N)
        DO J=K,N
          DO I=K,N
            IF(ABS(BIG).LT.ABS(A(I,J)))THEN
              BIG=A(I,J)
              SCRATCH(K,1)=I
              SCRATCH(K,2)=J
            ENDIF
          ENDDO
        ENDDO


!       Re-order rows & columns
!       -------- ---- - -------

!       Interchange rows
        J=SCRATCH(K,1)
        IF(J.GT.K)THEN
          DO I=1,N
            TEMP=-A(K,I)
            A(K,I)=A(J,I)
            A(J,I)=TEMP
          ENDDO
        ENDIF

!       Interchange columns
        I=SCRATCH(K,2)
        IF(I.GT.K)THEN
          DO J=1,N
            TEMP=-A(J,K)
            A(J,K)=A(J,I)
            A(J,I)=TEMP
          ENDDO
        ENDIF


!       Check if matrix is near-singular
!       ----- -- ------ -- -------------

        DET=DET*BIG            ! partial product of pivots
        IF (ABS(DET).LE.TOL) RETURN

!       Divide column by minus pivot
!       (value of pivot element is contained in BIGA)
        DO I=1,N
          IF(I.NE.K)THEN
            A(I,K)=A(I,K)/(-BIG)
          ENDIF
        ENDDO


!       Reduce matrix
!       ------ ------

        DO I=1,N
          TEMP=A(I,K)
          DO J=1,N
            IF(I.NE.K .AND. J.NE.K)THEN
              A(I,J)=TEMP*A(K,J)+A(I,J)
            ENDIF
          ENDDO
        ENDDO

!       Divide row by pivot
        DO J=1,N
          IF(J.NE.K) A(K,J)=A(K,J)/BIG
        ENDDO
        A(K,K)=1./BIG     ! Replace pivot by reciprocal

      ENDDO  ! K


!     Final row and column interchange
!     ----- --- --- ------ -----------

      DO K=N,1,-1
        I=SCRATCH(K,1)
        IF(I.GT.K)THEN
          DO J=1,N
            TEMP=A(J,K)
            A(J,K)=-A(J,I)
            A(J,I)=TEMP
          ENDDO
        ENDIF

        J=SCRATCH(K,2)
        IF(J.GT.K)THEN
          DO I=1,N
            TEMP=A(K,I)
            A(K,I)=-A(J,I)
            A(J,I)=TEMP
          ENDDO
        ENDIF
      ENDDO  ! K


      RETURN
      END
