!+ Map AMSU-A to an individual HIRS fov
!
      SUBROUTINE PPA2H ( TFOV, TLine,
     &                   Ncolocs, List_fov, List_line, List_wt )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppa2h.F 211 2010-04-26 12:34:57Z frna $
!
! Purpose:
!   Select mapping fovs from those given and derive brightness temperatures
!   and other parameters at the specified target fov.
!
! Method:
! 0.  Initialisation
! 1.  Identify nearest neighbour 'good' mapping fov
!     'Good' is any fov with bit 0 clear
! 2.  Identify 'clear' mapping fovs
!     'Clear' means not flagged for: cloud, precip, scattering, or
!      for surface type that we can't handle.
! 3.  Select fovs to map
!     Strategy is:
!         IF ( number of 'clear'/'recognised surface' mapping fovs > 0 )
!            map all cloud-free fovs of same surface type as "NN clear/..."
!         ELSEIF (NN cloudy )
!            take nearest neighbour
!         ELSEIF (NN flagged for precip/scattering/surf-type)
!            map all fovs, regardless of surface type
!         ENDIF
! 4.  Are we mapping bad AMSU-B ?
!     If nearest neighbour lacks AMSU-B then don't worry about AMSU-B,
!     but if it has AMSU-B then don't map fovs that don't.
! 5.  Calculate mapped BTs
!
!     Selecting fovs calibrated with primary or secondary calibration:
!       if the nearest neighbour is primary, then use only primaries
!       else use primaries and secondaries together.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1       9/12/96  Bug corrected   - no longer check calibration flag in
!                                       section 1.
!  1.2      12/12/96  Don't skip bad target fovs
!  1.3      10/03/98  Store instrument temp from nearest AMSU-A scanline
!  1.4      10/05/98  extra pre-processing variables included
!  1.5      04/08/98  bug fixed: wrong surface type going into Hmapsurf
!  1.6      07/09/98  Mapping considerably simplified. Problems were found when
!                     the same one AMSU spot was mapped to more than one HIRS
!                     so mapping is now always bilinear from several spots.
!  1.7      June 00   Put a flag on spots that are declared coast with 
!                     micro-wave channels  
!  1.8      March 03  New flag for fewer colocations than expected (NCAtkinson) 
!  1.9      30/03/10  Make sure we don't include missing BTs. NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! block of HIRS data   
#include "pphstore.h"
! block of AMSU-A data   
#include "ppastore.h"
! bit numbers for intrinsic bit routines (eg BTEST)    
#include "bits.h"   
!
!   Arguments with intent(In):
      INTEGER       TFOV          ! target fov number
      INTEGER       TLine         ! number of target line in btemps array
      INTEGER       Ncolocs       ! no of colocated mapping FOVs
      INTEGER       List_line(*)  ! list of coloc line numbers (1st=nearest)
      INTEGER       List_fov(*)   !               spot numbers
      REAL          List_wt(*)    !               LUT weights
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       fov           ! mapping fov number
      INTEGER       line          ! number of mapping line in btemps array
      INTEGER       icoloc        ! loop variable over mapping FOVs
      INTEGER       isurf         ! surface type of nearest neighbour AMSU-A
      INTEGER       nchan         ! no of channels to be mapped
      INTEGER       ngood         ! no of fovs selected
      INTEGER       nn            ! index of nearest neighbour mapping fov
      INTEGER       pp_qc         ! pre-processor QC word
      INTEGER       qcmask        ! bitword picks out selected bits from Aqc
      LOGICAL       samesurf      ! .TRUE. if all AMSU-A are same surface type
      REAL          sum_weights   ! sum of weights, used if weights need
                                  !   to be re-calculated (i.e. QC flags set)
      REAL          weight        ! used in the mapping
      REAL          wt_norm       ! reciprocal of sum_weights
!
! Local arrays:
      REAL          sum(40)       ! sum of weights*BTs for each channel
      LOGICAL       use_coloc(40) ! mapping FOVs: .TRUE. to use

!- End of header

! 0.  Initialisation
!     --------------
!     Define a bitword to pick out selected bits from Aqc
      qcmask = 0
      qcmask = IBSET ( qcmask, bits(30) )  ! secondary calibration
      qcmask = IBSET ( qcmask, bits(29) )  ! AMSU-B used secondary calibration
      qcmask = IBSET ( qcmask, bits(28) )  ! AMSU-B missing
      qcmask = IBSET ( qcmask, bits(27) )  ! cloud cost
      qcmask = IBSET ( qcmask, bits(26) )  ! scattering test
      qcmask = IBSET ( qcmask, bits(23) )  ! mismatch between AMSU-A/B 89GHz values
      qcmask = IBSET ( qcmask, bits(22) )  ! mismatch in SURFELEV/ATOVPP surf

! 1.  Identify good mapping fovs
!     -------- ---- ------- ----

!     Select only mapping fovs with valid brightness temperatures
      ngood = 0
      nn = 0   ! index of nearest neighbour mapping fov
      pp_qc = 0
      DO icoloc = 1, Ncolocs
         fov  = List_fov(icoloc)
         line = List_line(icoloc)
         IF ( .NOT.BTEST(Aqc(fov,line),bits(0)) ) THEN  ! BTs present
            ngood = ngood + 1
            use_coloc(icoloc) = .TRUE.
!           pp_qc = pp_qc .OR. ( Aqc(fov,line) .AND. qcmask )   ! QC word
            pp_qc = IOR( pp_qc, IAND( Aqc(fov,line), qcmask ))  ! QC word
            IF ( ngood .EQ. 1 ) nn = icoloc  ! nearest neighbour
         ELSE
            use_coloc(icoloc) = .FALSE.
            pp_qc = IBSET ( pp_qc, bits(21))  !Fewer colocations than expected
         ENDIF
      ENDDO

      IF ( ngood.EQ.0 ) GOTO 999 ! no good mapping fovs

! 2.  Calculate mapped BTs
!     --------- ------ ---

! 2.1 number of channels (depending on whether we have AMSU-B)
      nchan = Anchan
      IF ( .NOT. BTEST ( pp_qc, bits(28) ) ) nchan = nchan + Bnchan

! 2.2 initialise summation variables
      sum_weights = 0.
      DO chn = 1, nchan
         sum(chn)  = 0.
      ENDDO  ! chn

! 2.3 Do the mapping: sum weights*BTs for those FOVs selected
!     If any BT for a channel is missing, set the mapped BT to missing
      DO icoloc = 1, ncolocs
         IF ( use_coloc(icoloc) ) THEN
            fov  = List_fov(icoloc)
            line = List_line(icoloc)
            weight = List_wt(icoloc)
            sum_weights = sum_weights + weight
!           loop through the channels
            DO chn = 1, nchan
              IF (Abtemps(chn,fov,line) .NE. RMDI) THEN
                IF (sum(chn) .NE. RMDI) THEN
                  sum(chn) = sum(chn) + weight*Abtemps(chn,fov,line)
                ENDIF
              ELSE
                sum(chn) = RMDI
              ENDIF
            ENDDO  ! chn
         ENDIF  ! use_coloc(icoloc)
      ENDDO  ! icoloc

! 2.4 Loop through channels again to calculate mapped BTs
      wt_norm = 1. / sum_weights
      DO chn = 1, nchan
         IF (sum(chn) .NE. RMDI) THEN
           sum(chn) = sum(chn) * wt_norm
         ELSE
           sum(chn) = RMDI
         ENDIF
      ENDDO  ! chn

! 3.  Are all AMSU-A the same surface type ?
!     --- --- ------ --- ---- ------- ----

!     surface type of nearest neighbour
      fov  = List_fov(nn)
      line = List_line(nn)
      isurf = Asurftype(fov,line)

!     check against others
      samesurf = .TRUE.
      DO icoloc = 1, ncolocs
         IF ( use_coloc(icoloc) ) THEN
            fov  = List_fov(icoloc)
            line = List_line(icoloc)
            samesurf = samesurf .AND. ( isurf .EQ. Asurftype(fov,line) )
         ENDIF  ! use_coloc(icoloc)
      ENDDO  ! icoloc

      IF ( .NOT.samesurf ) pp_qc = IBSET ( pp_qc, bits(31) )
      IF ( isurf .eq. 1 )  pp_qc = IBSET ( pp_qc, bits(31) )


! 4.  Store values in HIRS arrays
!     ----- ------ -- ---- ------

!     prepro qc word
      Hmapqc(TFOV,TLine) = pp_qc

!     Instrument temperatures are from nearest AMSU-A scanline
      line = List_line(nn)
      Hinstrtempa1(TLine) = Ainstrtemp1(line)
      Hinstrtempa2(TLine) = Ainstrtemp2(line)
      Hinstrtempb(TLine)  = Ainstrtempb(line)  ! & mapped value from AMSU-B

!     Mapped brightness temperatures
      DO chn = 1, nchan
         Hmapbt(chn,TFOV,TLine) = sum(chn)
      ENDDO  ! chn

 999  CONTINUE    ! jump here if no good mapping fovs found

      RETURN
      END
