!+ Read fixed data file for AMSU-A within ATOVPP
!
      SUBROUTINE PPAFDF ( Afunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppafdf.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Read fixed data file required for AMSU-A within ATOVPP.
!
! Method:
! 0.  initialisation
! 1.  open file
! 2.  look for sections in the file
! 3.  read and derive coefficients for precip/scattering tests PPAPCP
! 4.  read and derive coefficients for cirrus index PPACIRR
! 5.  read and derive numbers for surface identification PPASURF
! 6.  read bias corrections
! 7.  close file
!
!   File format is specified in AAPP documentation APP204F, vn of 12/04/96
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      17/12/96  ASecAng initialised here (0.2)
!  1.2      10/01/97  AXcost new (0.2)
!  1.3      20/05/98  AXcost replaced by ASrfThresh
!  1.4      15/08/00  Coefficients for microwave cirrus index (D.C.Jones)
!  1.5      09/03/04  Modify for Linux (N Atkinson / A Korpela)
!  1.6      15/09/04  Use constants_preproc.h for fundamental constants - NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"  
! fixed data for processing AMSU-A
#include "ppafdata.h"
! block of AMSU-A data 
#include "ppastore.h"        
!Fundamental and physical constants
#include "constants_preproc.h"    
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       Afunit        ! unit number for AMSU-A-specific data
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from file
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status value
      INTEGER       chn           ! loop index: channel number
      INTEGER       fov           ! loop index: fov number
      INTEGER       i, j          ! general purpose loop indices
!     for scattering coefficients:
      REAL          angle         ! surface zenith angle for the fov
      REAL          satang        ! satellite zenith angle
      REAL          ratio         ! ratio defined as (rEarth+SatHt) / rEarth
      REAL          x             ! secant(surface zenith angle) - 1.
!     for surface type:
      REAL          det           ! determinant from s/r MXINV
      INTEGER       iang          ! loop index over angles
      INTEGER       isurf         ! loop index over surface types
      INTEGER       k, m          ! loop indices over channels
      INTEGER       kx, mx        ! indices over reduced set of channels
!
! Local Arrays:
      INTEGER       SCRATCH(NASrfChMx,2)  ! work array for s/r MXINV
      REAL aa(4,4)                ! polynomial coefficients for PPASCAT tests
      REAL tb(Anchan+Bnchan)      ! BT means as read for PPASURF
      REAL tbc(Anchan+Bnchan,Anchan+Bnchan) ! BT cov mx as read for PPASURF
      REAL tbcx(NASrfChMx,NASrfChMx)   ! tbc on reduced set of channels
      REAL zASecAng(NASecAng) ! secant(angles) for PPASURF
      DATA zASecAng / 1.00, 1.25, 1.50, 1.75, 2.00 /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External MXINV                ! matrix inversion routine
      External RDINTS               ! read integers from character string

!- End of header


! 0.  initialisation
!     --------------

! 0.1 bias corrections
      DO fov = 1, Anfovs
      DO chn = 1, Anchan
         Abiases(chn,fov) = 0.
      ENDDO  ! chn
      ENDDO  ! fov

! 0.2 secants of fixed angles for PPASURF
      DO i = 1, NASecAng
         ASecAng(i) = zASecAng(i)
      ENDDO  ! i


! 1.  open file
!     ---- ----

      OPEN ( UNIT=Afunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!     If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=Afunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

      WRITE ( errMessage,
     & "(""Error opening AMSU-A fixed data file, unit "",I8)" ) Afunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPAFDF")


! 2.  look for sections in the file
!     ---- --- -------- -- --- ----

      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )
         input_line = " "
         READ(Afunit,"(A80)",IOSTAT=errStatus) input_line


         IF ( input_line(1:6) .EQ. "PREPRO" ) THEN

! 3.        read and derive coefficients for precip/scattering tests PPAPCP
!           ---- --- ------ ------------ --- ----------------- ---- -------

! 3.1       threshold for difference in AMSU-A/AMSUB 89GHz channels PPB2A

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading ABmax89"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(Afunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")
            READ(input_line,*,IOSTAT=errStatus) ABmax89 ! test threshold

! 3.2       logistic precipitation probability test PPCROSBY

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading Crosby numbers"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(Afunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")
            READ(input_line,"(4F8.3)",IOSTAT=errStatus)
     &           Acrosby1, Acrosby2, Acrosby3, Amaxcrosby

! 3.3       Grody light rainfall test PPGRODY

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading Grody numbers"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(Afunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")
            READ(input_line,"(2F8.3)",IOSTAT=errStatus)
     &           Agrody1, Agrody2

! 3.4       coefficients for scattering test PPASCAT
!           The equation we are are using is
!                   T15=a0 + T1 * a1 + T2 * a2 + T3 * a3
!           a0, a1, a2 and a3 are functions of secant(view angle) - 1
!           A third order polynomial is used to calculate a0, a1, a2 and a3.
!           The polynomial coefficents are read into "aa"

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading Amaxscat"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(Afunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")
            READ(input_line,*,IOSTAT=errStatus) Amaxscat ! test threshold

            READ(Afunit,"(4F13.8)",IOSTAT=errStatus)
     &                           ((aa(i,j),i=1,4),j=1,4)
            WRITE ( errMessage,
     &    "(""Error reading AMSU-A fdf scat, unit "",I8)" ) Afunit
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")

!           Calculate coefficients to predict AMSU-A channel 15 from 1,2,3.
            ratio = (rEarth+Asatht) / rEarth
            satang = -Aangle * REAL(Anfovs+1) / 2.
            DO fov = 1, Anfovs
               satang = satang + Aangle
!              convert zenith angle at satellite to z.angle (rad) at surface
               angle = ASIN ( SIN(satang*deg2rad) * ratio )
               x = 1./COS(angle) - 1.
               DO j = 1, 4
                  Ascat(j,fov) =
     &              aa(1,j) + aa(2,j)*x + aa(3,j)*x*x + aa(4,j)*x*x*x
               ENDDO  ! i
            ENDDO  ! fov

! 4.        read and derive coefficients for cirrus test PPACIRR
!           ---- --- ------ ------------ --- ------ ---- -------

! 4.1       coefficients for scattering test PPACIRR
!           The equation we are are using is
!                   T20=a0 + T1 * a1 + T16 * a2 + T17 * a3
!           a0, a1, a2 and a3 are functions of secant(view angle) - 1
!           A third order polynomial is used to calculate a0, a1, a2 and a3.
!           The polynomial coefficents are read into "aa"

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading ABmaxcirr"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(Afunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")
            READ(input_line,*,IOSTAT=errStatus) ABmaxcirr ! test threshold

            READ(Afunit,"(4F13.8)",IOSTAT=errStatus)
     &                           ((aa(i,j),i=1,4),j=1,4)
            WRITE ( errMessage,
     &    "(""Error reading AMSU-A fdf cirr, unit "",I8)" ) Afunit
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")

!           Calculate coefficients to predict AMSU-B channel 20 from A1,B16,B17.
            ratio = (rEarth+Asatht) / rEarth
            satang = -Aangle * REAL(Anfovs+1) / 2.
            DO fov = 1, Anfovs
               satang = satang + Aangle
!              convert zenith angle at satellite to z.angle (rad) at surface
               angle = ASIN ( SIN(satang*deg2rad) * ratio )
               x = 1./COS(angle) - 1.
               DO j = 1, 4
                  ABcirr(j,fov) =
     &              aa(1,j) + aa(2,j)*x + aa(3,j)*x*x + aa(4,j)*x*x*x
               ENDDO  ! i
            ENDDO  ! fov

! 5.        read and derive numbers for surface identification PPASURF
!           ---- --- ------ ------- --- ------- -------------- -------

! 5.1       Choice of channels for use in determining AMSU-A surface type

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading channels for PPASURF"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(Afunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")

!           Having read the line, extract channel numbers
            CALL RDINTS ( input_line, ASrfChn, NASrfCh )

!           Check all is ok
            IF ( NASrfCh.LT.1 .OR. NASrfCh.GT.NASrfChMx ) THEN
               errStatus  = 411  ! fatal
               errMessage = " "
               WRITE ( errMessage,
     &           "(""Too many/few channels for use in PPASURF: "",I8)" )
     &           NASrfCh
               CALL ErrorReport(errStatus,errMessage,"PPAFDF")
            ENDIF
            DO i = 1, NASrfCh
               IF ( ASrfChn(i).LT.1 .OR.
     &              ASrfChn(i).GT.Anchan+Bnchan ) THEN
                  errStatus  = 412  ! fatal
                  errMessage = " "
                  WRITE ( errMessage,
     &     "(""Channel out of range for PPASURF: "",I8)" ) ASrfChn(i)
               ENDIF
            ENDDO  ! i
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")

! 5.2       threshold for detecting cloud in the cost function

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading cloud threshold for PPASURF"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(Afunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")

!           Having read the line, read the threshold
            READ(input_line,*,IOSTAT=errStatus) ASrfThresh
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")

! 5.3       mean brightness temperatures

!           read comment line in the file
            READ(Afunit,*,IOSTAT=errStatus)
            WRITE ( errMessage,
     &  "(""Error reading coeffs for PPASURF, unit "",I8)" ) Afunit
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")

!           loop through surface types
            DO isurf = 1, NASrfTyp

               READ(Afunit,*,IOSTAT=errStatus)  ! comment line in the file
               errStatus = ABS (errStatus)   ! end-of-file returns -1
               CALL ErrorReport(errStatus,errMessage,"PPAFDF")

!              loop through angles
               DO iang = 1, NASecAng

                  READ(Afunit,"(6F13.3)",IOSTAT=errStatus)
     &                (tb(m),m=1,Anchan+Bnchan)
                  errStatus = ABS (errStatus)   ! end-of-file returns -1
                  CALL ErrorReport(errStatus,errMessage,"PPAFDF")
!                 store reduced set of channels
                  DO k = 1, NASrfCh
                     kx = ASrfChn(k)
                     AmeanBT(isurf,iang,k) = tb(kx)
                  ENDDO  ! k

               ENDDO  ! iang
            ENDDO  ! isurf

!           read comment line in the file
            READ(Afunit,*,IOSTAT=errStatus)
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")

! 5.4       brightness temperature covariance matrices
!           loop through surface types
            DO isurf = 1, NASrfTyp

               READ(Afunit,*,IOSTAT=errStatus)  ! comment line in the file
               errStatus = ABS (errStatus)   ! end-of-file returns -1
               CALL ErrorReport(errStatus,errMessage,"PPAFDF")

!              loop through angles
               DO iang = 1, NASecAng

                  DO m = 1, Anchan+Bnchan
                     READ(Afunit,"(6F13.4)",IOSTAT=errStatus)
     &               (tbc(k,m),k=1,Anchan+Bnchan)
                     errStatus = ABS (errStatus)   ! end-of-file returns -1
                     CALL ErrorReport(errStatus,errMessage,"PPAFDF")
                  ENDDO  ! m
!                 store reduced set of channels
                  DO k = 1, NASrfCh
                     kx = ASrfChn(k)
                     DO m = 1, NASrfCh
                        mx = ASrfChn(m)
                        tbcx(m,k) = tbc(mx,kx)
                     ENDDO  ! m
                  ENDDO  ! k

!                 invert the covariance matrix
                  CALL MXINV ( tbcx, NASrfCh, NASrfChMx, SCRATCH,
     &                         det, 0.01 )

!                 and store it
                  DO k = 1, NASrfCh
                  DO m = 1, NASrfCh
                     AcovBT(isurf,iang,m,k) = tbcx(m,k)
                  ENDDO  ! m
                  ENDDO  ! k

               ENDDO  ! iang
            ENDDO  ! isurf


         ELSEIF ( input_line(1:4) .EQ. "BIAS" ) THEN

! 6.        read bias-corrections
!           ---- ----------------

!           read scan-dependent bias corrections
            errMessage = "Reading BIAS section"
            errStatus = 0
            DO fov = 1, Anfovs
               IF ( errStatus.EQ.0 ) THEN
                  READ(Afunit,"(10F8.3)",IOSTAT=errStatus)
     &                       (Abiases(chn,fov),chn=1,Anchan)
               ENDIF
            ENDDO  ! fov
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPAFDF")


         ENDIF  ! input_line

      ENDDO  ! WHILE ( errStatus.EQ.0 )


! 7.  close file
!     ----- ----

      CLOSE(Afunit)


      RETURN
      END
