!+ Read & store level 1C AMSU-A data within ATOVPP
!
      SUBROUTINE PPAGET ( A1Cunit, ALastLine,
     &                    Year1, Day1, Time1,
     &                    Year2, Day2, Time2 )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppaget.F 62 2006-12-13 10:42:15Z frna $
!
! Purpose:
!   Update ATOVPP store of AMSU-A scanlines to hold data within
!   start and end dates/times specified (Year1/Day1/Time1 & Year2/Day2/Time2)
!   with an extra 1 or 2 scanlines at either end to help with the mapping.
!
! Method:
! 1.  Look for first suitable scanline in previous block
! 2.  Define record number to start reading from
! 3.  Set data store to all "missing"
! 4.  Read data till we come to end of time interval, or of dataset
!
!   The position of each scanline in the program store ("ppastore.h") is
!   determined solely by the date/time of the scanline number, relative to
!   that of the first line in store.  Thus the routine can cope with missing
!   scanlines, but can potentially be upset if the first line in store has a
!   corrupt date/time.
!
! Note:
!   "ALastLine" should be initialised as 0 before first calling this routine.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Check the scanline number from the date/time
!  1.2      11/06/98  Acycle REAL not INTEGER
!  1.3      21/04/99  border set = 2*Acycle instead of 1*Acycle
!  1.4      12/10/00  Check that first scanline OK  (Keith Whyte)
!  1.5      29/11/06  Use estimated scan line for setting scnlinold - 
!                     because METOP scan line can return to 1. Nigel Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! block of AMSU-A data
#include "ppastore.h" 
! AMSU-A level 1C header/data common blocks
#include "ama1c.h"
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"    
!
!   Scalar arguments with intent(In):
      INTEGER       A1Cunit       ! logical unit no. for AMSU-A 1C input
      INTEGER       ALastLine     ! number of last line stored in store
      ! Date/Time data block is to start from:
      INTEGER       Year1         !     year (e.g. 1995)
      INTEGER       Day1          !     day of year
      INTEGER       Time1         !     time (milliseconds)
      ! Date/Time data block is to end:
      INTEGER       Year2         !     year (e.g. 1995)
      INTEGER       Day2          !     day of year
      INTEGER       Time2         !     time (milliseconds)
!
! Local scalars:
      CHARACTER*80  errMessage    ! for calls to ErrorReport
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      INTEGER       border        ! for overlaps of data blocks (millisecs)
      INTEGER       dTime         ! time difference (milliseconds) from TimeSub
      INTEGER       errTime       ! error status from TimeSub
      INTEGER       A1Crecord     ! record number for AMSU-A level 1C file
      INTEGER       fov           ! loop variable over all fov's
      INTEGER       line          ! line number within data store
      INTEGER       no_data       ! bit word set to "missing/bad data"
      INTEGER       scnlin0       ! scanline number of "zeroth" line in store
      INTEGER       scnlinDT      ! scanline number derived from date/time
      INTEGER       scnlin        ! scanline number
      INTEGER       year          ! year number of scanline
      INTEGER       day           ! day number of scanline
      INTEGER       time          ! time of scanline (ms)
      INTEGER       scnlinOld     ! previous scanline number
      INTEGER       yearOld       ! year number of previous scanline
      INTEGER       dayOld        ! day number of previous scanline
      INTEGER       timeOld       ! time of previous scanline (ms)
!
! Function & Subroutine calls:
      External TimeSub              ! Subtract two dates/times
      External IOA1C                ! read AMSU-A 1C scan line
      External PPAIND               ! store scanline data in common/ppastore/
      External ErrorReport          ! ATOVPP error routine
!
!- End of header


! 1.  Look for first suitable scanline in previous block
!     ---- --- ----- -------- -------- -- -------- -----

      line = ALastLine + 1
      dTime   = -999
      border = NINT(Acycle*2.0)  ! ensure sufficient overlap with hirs block

!     (stop looking once scanlines in block are earlier than start time)
      DO WHILE (line.GT.1 .AND. dTime.LT.border)

         line = line - 1
         errTime = 1
         IF ( Arecdno(line).NE.MDI ) THEN    ! valid line
            CALL TimeSub ( Year1,       Day1,       Time1,
     &                     Ayear(line), Aday(line), Atime(line),
     &                     dTime, errTime)! dTime = start time - time of scan
!         errTime returned as non-zero if input is out of range (e.g. missing)
         ENDIF
         IF (errTime.NE.0) dTime = -999   ! go on looking

      ENDDO


! 2.  Define record number to start reading from
!     ------ ------ ------ -- ----- ------- ----

      IF ( dTime.GE.0 ) THEN
!        we've already read suitable data  - get record number from that
         A1Crecord = Arecdno(line)
      ELSE
!        no suitable scanlines already in block  - try next record
         IF ( ALastLine.GT.0 ) THEN
            A1Crecord = Arecdno(ALastLine) + 1
         ELSE  ! this is the first data to be read
            A1Crecord = Ahdrcnt + 1  !first data record in 1C file
         ENDIF
      ENDIF


! 3.  Set data store to all "missing"
!     --- ---- ----- -- ---  -------

!     Throughout this program, Arecdno() should be used to check
!     whether a scanline is present or not, and Aqc(bits(0))
!     to check whether the data is usable.

      no_data = 0
      no_data = IBSET ( no_data, bits(0) )

      DO line = 1, Anlines
         Arecdno(line) = MDI
         DO fov = 1, Anfovs
            Aqc(fov,line) = no_data
         ENDDO  ! fov
      ENDDO  ! line


! 4.  Read data till we come to end of time interval, or of dataset
!     ---- ---- ---- -- ---- -- --- -- ---- --------  -- -- -------

      ALastLine = 0  ! i.e. we have no data in storage
      errTime = 1
      dTime   = -999
      scnlin0 = MDI  ! scanline number of "zeroth" scanline in store

! 4.1 Loop through scanlines in dataset
      DO WHILE ( A1Crecord.LE.(Ahdrcnt+Ascnlin) .AND. dTime.LT.border )

!        Read a scan line
         CALL IOA1C ("READ DATA", A1Cunit, A1Crecord)
         scnlin = ama1c_scnlin
         year = ama1c_scnlinyr
         day  = ama1c_scnlindy
         time = ama1c_scnlintime

        IF ( ama1c_scnlin.GE.1 .AND. ama1c_scnlin.LE.9999
     &    .AND. .NOT. BTEST(ama1c_scanqual,bits(23)) ) THEN    ! if scanline OK

! 4.2    Calculate the scanline number from the date/time (to check)
         IF ( scnlin0 .NE. MDI ) THEN   ! not the 1st record read this call

            CALL TimeSub ( year, day, time,
     &                     yearOld,dayOld, timeOld, dTime, errTime )
            errMessage = " "
            CALL ErrorReport(errTime,errMessage,"PPAGET")

!           estimated scanline number
            scnlinDT = scnlinOld + NINT(REAL(dTime)/Acycle)

!           Complain if scanline number and estimated scanline number differ
            IF ( scnlinDT.NE.scnlin ) THEN
               errMessage = " "
               WRITE ( errMessage,
     &      "(""Scanline numbers and times inconsistent, lines"",3I9)")
     &          scnlinOld, scnlin, scnlinDT
               errStatus = -42  ! warning
               CALL ErrorReport(errStatus,errMessage,"PPAGET")
            ENDIF

         ENDIF  ! not the 1st record read

! 4.3    Store scanline in slot "line" in common/ppastore/

!        Calculate slot to put line into
         IF ( scnlin0 .EQ. MDI ) THEN   ! first record to be read this call
            scnlin0 = scnlin - 1
            line = 1
         ELSE
            line = scnlinDT - scnlin0   ! use estimated line number, not scnlin
         ENDIF

         IF ( line.LE.Anlines ) THEN  ! there's room
            ALastLine = line
            CALL PPAIND ( line )
            Arecdno(line) = A1Crecord
         ENDIF

! 4.4    save line-number/date/time to check against next scanline
!         scnlinOld = scnlin
         IF ( line .EQ. 1 ) THEN
           scnlinOld = scnlin
         ELSE
           scnlinOld = scnlinDT
         ENDIF
         yearOld = year
         dayOld  = day
         timeOld = time

! 4.5    Calculate dTime := date/time of scan - end date/time of block
         CALL TimeSub ( year, day, time,
     &                  Year2,Day2,Time2,  dTime, errTime )
!        errTime returned as non-zero if input is out of range (e.g. missing)
         IF (errTime.NE.0) dTime = -999   ! go on looking

        ENDIF    ! if scanline OK

!       Increment record number for next read
        A1Crecord = A1Crecord + 1

      ENDDO   ! WHILE ( A1Crecord.LE.(Ahdrcnt+Ascnlin) .AND. dTime.LT.border )

      WRITE(*,"(10X,""AMSU-A line numbers "",I4,"" to "",I4,
     &        "", with "",I3,"" missing lines"")")
     &        Alineno(1), Alineno(ALastLine),
     &       (scnlinOld-Alineno(1))-(ALastLine-1)
!     &       (Alineno(ALastLine)-Alineno(1))-(ALastLine-1)


      RETURN
      END
