!+ Transfer level 1C AMSU-A data from data record to program store
!
      SUBROUTINE PPAIND ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppaind.F 500 2016-10-28 13:02:15Z frna $
!
! Purpose:
!   AMSU-A 1C data, when read, is held in common/ama1cdt/. ATOVPP stores data
!   from several scanlines in its own internal array structure, contained
!   in common/ppastore/. This routine transfers a single scanline from
!   common/ama1cdt/ to slot "Line" in common/ppastore/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1C include file (ama1c.h) and AMSU-A store
!     (ppastore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Copy variables from common/ama1cdt/ to slot "Line" in common/ppastore/
! 2.  Check flags
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1C format vn 1.8
!                     Set Aqc(bit0) if any channels are missing
!  1.2      10/03/98  updated for 1c vn 1.9 (instrument temps and solar angles)
!  1.3      02/07/98  Aangles REAL not INTEGER
!  1.4      06/11/00  For NOAA-15, do not set Aqc(bit0) if channel 14 missing
!  1.5      29/04/02  For NOAA-15, do not set Aqc(bit0) if channel 11 missing
!  1.6      27/12/02  Don't check Ascanqual bits 18,19 (corrected for moon), but
!                      do check bits 16,17 (uncorrected moon-contaminated lines)
!  1.7      15/03/04  For NOAA-17, do not set Aqc(bit0) if channels 3-15 missing
!  1.8      27/07/07  Test bit 25 of qualind: status changes usually indicate
!                     bad data. 
!  1.9      17/09/07  For NOAA-16, do not set Aqc(bit0) if channel 4 missing
!  1.10     16/10/07  If last spot is missing, flag whole line as bad. This is
!                     often an indicator of corrupt data.
!  1.11     01/12/09  For MetOp-A do not set Aqc(bit0) if channel 7 missing
!  1.12     06/07/15  For MetOp-A do not set Aqc(bit0) if channel 8 missing
!  1.12     01/08/16  For MetOp-A do not set Aqc(bit0) if channel 3 missing
!  1.13     17/10/16  For MetOp-B do not set Aqc(bit0) if channel 15 missing
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"   
! general parameters for ATOVPP
#include "ppparms.h"         
! block of AMSU-A data
#include "ppastore.h"       
! AMSU-A level 1C header/data common blocks
#include "ama1c.h"          
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppastore/
!
! Local parameters:
      INTEGER       A1Cvnnb       ! version number of 1C format compatible
         PARAMETER (A1Cvnnb=190)  ! with the code in this subroutine
      INTEGER       Astvnnb       ! version number of "ppastore.h" compatible
         PARAMETER (Astvnnb=130)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chanbits      ! bit word with all channel bits set
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       qcmask        ! bitword picks out selected bits
      LOGICAL       badline       ! .TRUE. not to use a scanline
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first, chanbits
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!        Is the 1C include file compatible with this source code ?
         IF ( A1Cvnnb .NE. ama_1cvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but ama1c.h is "",I6)")
     &           A1Cvnnb, ama_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPAIND")
         ENDIF

!        Is the ppastore include file compatible with this source code ?
         IF ( Astvnnb .NE. ama_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects Astore format"",I6,"" but ppastore.h is"",I6)")
     &           Astvnnb, ama_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPAIND")
         ENDIF

!        set up a mask to pick out just the flags for the individual channels
         chanbits = 0

         IF ( ama1c_h_satid .EQ. 15 ) THEN
!
!  For NOAA-15, do not set bit for channels 11 or 14,
!                         to allow pseudo value to be set in pphoutd.F
            DO chn = 1, 10
               chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO
            DO chn = 12, 13
               chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO
            chanbits = IBSET ( chanbits, bits(15) )

         ELSE IF ( ama1c_h_satid .EQ. 16 ) THEN
!
!  For NOAA-16, do not set bit for channel 4
!
            DO chn = 1, 3
               chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO
            DO chn = 5, 15
              chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO

         ELSE IF ( ama1c_h_satid .EQ. 17 ) THEN
!
!  For NOAA-17, do not set bit for channels 3 to 15 (unless before 28 Oct 03)
!
            DO chn = 1, 2
               chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO
            IF (ama1c_h_startorbit.lt.6976) THEN
              DO chn = 3, 15
                chanbits = IBSET ( chanbits, bits(chn) )
              ENDDO
            ENDIF

         ELSE IF ( ama1c_h_satid .EQ. 1 ) THEN
!
!  For Metop-B do not set bit for channel 15
!
            DO chn = 1, 14
              chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO

         ELSE IF ( ama1c_h_satid .EQ. 2 ) THEN
!
!  For MetOp-A, do not set bit for channel 7 or 8
!
            DO chn = 1, 2
               chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO
            DO chn = 4, 6
               chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO
            DO chn = 9, 15
              chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO

         ELSE

            DO chn = 1, Anchan
               chanbits = IBSET ( chanbits, bits(chn) )
            ENDDO

         ENDIF

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from common/ama1cdt/ to slot "Line" in common/ppastore/
!     ---- --------- ---- --------------- -- ----  ----  -- ----------------

      Alineno(Line) = ama1c_scnlin                ! scanline number
      Ayear(Line) = ama1c_scnlinyr                ! scanline year
      Aday(Line) = ama1c_scnlindy                 ! scanline day of year
      Atime(Line) =  ama1c_scnlintime             ! scanline time (ms)
      Aqualind(Line) = ama1c_qualind              ! quality indicator bit field
      Ascanqual(Line) = ama1c_scanqual            ! scan line quality flags
      Ainstrtemp1(Line) = ama1c_instrtemp1        ! AMSU-A1 RF shelf temp
      Ainstrtemp2(Line) = ama1c_instrtemp2        ! AMSU-A2 RF shelf temp

      DO fov = 1, Anfovs
         Alats(fov,Line) = REAL(ama1c_latlon(1,fov))*1.E-4 !latitude(deg,N+ve)
         Alons(fov,Line) = REAL(ama1c_latlon(2,fov))*1.E-4 !longitude (E+ve)
         Aangles(1,fov,Line) = REAL(ama1c_angles(1,fov))*0.01 !local zenith
         Aangles(2,fov,Line) = REAL(ama1c_angles(2,fov))*0.01 !local azimith
         Aangles(3,fov,Line) = REAL(ama1c_angles(3,fov))*0.01 !solar zenith
         Aangles(4,fov,Line) = REAL(ama1c_angles(4,fov))*0.01 !solar azimith
      ENDDO  ! fov

      Aaltitude(Line) =  ama1c_scalti             ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
      DO fov = 1, Anfovs
      DO chn = 1, Anchan
         Abtemps(chn,fov,Line) = REAL(ama1c_btemps(chn,fov))*0.01 ! BTs (K)
      ENDDO  ! chn
      DO chn = Anchan+1, Anchan+Bnchan
         Abtemps(chn,fov,Line) = RMDI ! set AMSU-B to missing, awaiting mapping
      ENDDO  ! chn
      ENDDO  ! fov

!     ATOVPP QC word  (..just take bits 0 & 30 from level 1c data quality word)
      qcmask = 0
      qcmask = IBSET ( qcmask, bits(0) )  ! bit 0  = 'data missing'
      qcmask = IBSET ( qcmask, bits(30) ) ! bit 30 = 'secondary calibration'
      DO fov = 1, Anfovs
!        Aqc(fov,Line) = ama1c_dataqual(fov) .AND. qcmask
         Aqc(fov,Line) = IAND(ama1c_dataqual(fov), qcmask)
      ENDDO  ! fov

!     Set Aqc(bit0) if any channels are missing
      DO fov = 1, Anfovs
!        IF ( (ama1c_dataqual(fov).AND.chanbits).NE.0 )
         IF ( IAND(ama1c_dataqual(fov),chanbits).NE.0 )
     &         Aqc(fov,Line) = IBSET(Aqc(fov,Line),bits(0))
      ENDDO  ! fov


! 2.  Check flags
!     ----- -----

! 2.1 Flags for the whole scanline

!     data quality indicator
      badline = BTEST( Aqualind(Line), bits(31) ) !do not use for products
     &    .OR.  BTEST( Aqualind(Line), bits(27) ) !no earth location
     &    .OR.  BTEST( Aqualind(Line), bits(25) ) !instr status changed

!     scanline quality indicator
      badline = badline
     &    .OR.  BTEST( Ascanqual(Line), bits(23) )!bad time field
     &    .OR.  BTEST( Ascanqual(Line), bits(22) )!bad time field
     &    .OR.  BTEST( Ascanqual(Line), bits(20) )!sequence repeats
     &    .OR.  BTEST( Ascanqual(Line), bits(17) )!A2 moon contamination
     &    .OR.  BTEST( Ascanqual(Line), bits(16) )!A1 moon contamination
     &    .OR.  BTEST( Ascanqual(Line), bits(04) )!earth location unreasonable

!     If last spot is missing, flag whole line as bad
      badline = badline .OR. BTEST( Aqc(30,Line),bits(0) )

!     For bad scanline, flag each fov
      IF ( badline ) THEN
         DO fov = 1, Anfovs
            Aqc(fov,Line) = IBSET ( Aqc(fov,Line), bits(0) )
         ENDDO  ! fov
      ENDIF


      RETURN
      END
