!+ Read & interpret AMSU-A level 1C header within ATOVPP
!
      SUBROUTINE PPAINH ( A1Cunit, SatId, WMOSatId,
     &                    StartYearA, StartDayA, StartTimeA,
     &                    EndYearA,   EndDayA,   EndTimeA )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppainh.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Open input dataset and read level 1C AMSU-A header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     AMSU-A level 1C header has already been read into common /ama1chd/
!     Check that format of 1C include files (ama1c.h) is compatible with the
!     code in this subroutine. Check that format of 1C file is compatible with
!     this subroutine, and that data type from the header is 10 for AMSU-A.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from AMSU-A 1C header
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      10/12/96  SatId added, Ht and Period deleted
!  1.2      23/08/05  Add WMOSatId (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! block of AMSU-A data 
#include "ppastore.h"     
! AMSU-A level 1C header/data common blocks
#include "ama1c.h"           
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
!
! Local parameters:
      INTEGER       A1Cvnnb       ! version number of 1C format compatible
         PARAMETER (A1Cvnnb=190)  ! with the code in this subroutine
!
!     Alternative way of checking that version numbers match (at compile time)
!      REAL VNTEST1(A1Cvnnb-ama_1cvnnb+1), VNTEST2(ama_1cvnnb-A1Cvnnb+1)
!
!   Scalar arguments with intent(In):
      INTEGER       A1Cunit       ! logical unit no. for AMSU-A 1C input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 3 for METOP-1)
      ! start/end date/time of AMSU-A data:
      INTEGER       StartYearA, EndYearA
      INTEGER       StartDayA,  EndDayA
      INTEGER       StartTimeA, EndTimeA
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       A1Crecord     ! record number for AMSU-A level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOA1C                ! I/O routine for level 1C AMSU-A data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.1 Open 1C file
      CALL IOA1C ("OPEN/READ", A1Cunit, A1Crecord )   ! 1C file

! 1.2 Read 1C header
      A1Crecord = 1   ! Headers start on record 1.
      CALL IOA1C ("READ HEADER", A1Cunit, A1Crecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( A1Cvnnb .NE. ama_1cvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but ama1c.h is "",I6)")
     &        A1Cvnnb, ama_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPAINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "ama1C.H" contains its own version number "ama_h_l1cversnb"
      IF ( ama1c_h_versnb .NE. ama_1cvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level 1C AMSU-A file is "",I8,"", not "",I8)")
     &        ama1c_h_versnb,ama_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPAINH")
      ENDIF

! 2.3 Is the 1C data AMSU-A ?
!     -- --- -- ---- ------
!     "ama1c_h_instrument" from 1C header should be 10 for AMSU-A
      IF ( ama1c_h_instrument .NE. 10 ) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage, "("" Data type in 1C AMSU-A file is "",
     &           I8,""; not 10 for AMSU-A"")") ama1c_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "PPAINH")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "##########"
      WRITE(*,*) "# AMSU-A #"
      WRITE(*,*) "##########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            ama1c_h_versnb, ama1c_h_versyr, ama1c_h_versdy
      WRITE(*,"(I4,"" header records in file."")") ama1c_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") ama1c_h_satid

!     calculate start and end dates/times from header
!     ..start
      year = ama1c_h_startdatayr
      yrday = ama1c_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(ama1c_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = ama1c_h_enddatayr
      yrday = ama1c_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(ama1c_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          ama1c_h_scnlin, ama1c_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from AMSU-A 1C header
!     ------- -------- ----------- ---- ------ -- ------

      ASatId = ama1c_h_satid
      SatId = ASatId
      WMOSatId = ama1c_h_wmosatid
      StartDayA = ama1c_h_startdatady
      StartYearA = ama1c_h_startdatayr
      StartTimeA = ama1c_h_startdatatime
      EndDayA = ama1c_h_enddatady
      EndYearA = ama1c_h_enddatayr
      EndTimeA = ama1c_h_enddatatime
      Asatht = REAL(ama1c_h_satht)*0.1      !nominal sat height (km)
      Asatperiod = REAL(ama1c_h_period)/60. !nominal orbit period (minutes)
      Ahdrcnt = ama1c_h_hdrcnt       !count of header records in 1C file
      Ascnlin = ama1c_h_scnlin       !count of scanlines in 1C file
      Ainstruments = IBSET ( Ainstruments , bits(3) )   ! AMSU-A data present


      RETURN
      END
