!+ pre-process AMSU-A
!
      SUBROUTINE PPAMSUA1 ( ALastLine, BLastLine, LSunit, HTunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppamsua1.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   pre-process a block of level 1c AMSU-A data
!
! Method:
! 1.  Add scan-dependent bias corrections (BIASCORR)
! 2.  Get land/sea information & elevation (SURFELEV)
! 3.  Correct for limb effect and surface emissivity
! 4.  Map AMSU-B to AMSU-A grid (PPMAP)
! 5.  Test for precipitation (PPAPCP)
! 6.  Estimate surface type from the data (PPASURF)
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      16/01/97  Bug fixed: lat & lon not now multiplied by 0.01
!  1.2      11/06/98  Bug fixed: Nmappings no longer changed here
!  1.3      10/07/98  PPACORR added for consistency with HIRS pre-processing
!  1.4      14/11/03  Check whether AMSU-B really needed in PPASURF (NCAtkinson)
!  1.5      15/03/05  Add LandFrac
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! parameters and look-up tables for the mapping
#include "ppluts.h"
! fixed data for processing AMSU-A          
#include "ppafdata.h"
! block of AMSU-A data       
#include "ppastore.h"  
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
!
!   Scalar arguments with intent(In):
      INTEGER       ALastLine     ! number of last AMSU-A line in store
      INTEGER       BLastLine     ! number of last AMSU-B line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       imapping      ! loop index, looping through mappings
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! loop index
      LOGICAL       BMissing      ! true if AMSU-B missing for surface type
!
! Local arrays:
      REAL          lats(Anfovs)  ! array of latitudes for a scanline
      REAL          lons(Anfovs)  ! array of longitudes for a scanline
      INTEGER       surftype(Anfovs) !surface type for a scanline (SURFELEV)
      INTEGER       elevation(Anfovs)!elevation for a scanline (SURFELEV)
      INTEGER       LandFrac(Anfovs) !Land fraction(%) for a scanline (SURFELEV)
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External BIASCORR           ! scan-dependent bias corrections
      External SURFELEV           ! get land/sea information, (etc.)
      External PPACORR            ! Correct for limb effect and surface emiss
      External PPAPCP             ! test for precipitation, etc.
      External PPMAP              ! map data between instrument grids
      External PPASURF            ! estimate surface type from the data
!
!- End of header

      WRITE(*,"(10X,""Pre-processing AMSU-A before the mapping"")")

! 1.  Add scan-dependent bias corrections
!     --- -------------- ---- -----------

      CALL BIASCORR ( Abiases, Anchan+Bnchan, Anchan, Anfovs,
     &                ALastLine, Abtemps )


! 2.  Get land/sea information
!     --- -------- -----------

!     loop through scanlines not already processed
      DO line = 1, ALastLine

! 2.1    pick out lat's and lon's from scanline
         DO fov = 1, Anfovs
            lats(fov) = Alats(fov,line)        ! degrees N
            lons(fov) = Alons(fov,line)        ! degrees E
         ENDDO  ! fov

! 2.2    find surface type for each fov - using options read in PPUSER
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, Anfovs, AboxSize,
     &                   surftype, elevation, LandFrac )

! 2.3    store surface type & elevation
         DO fov = 1, Anfovs
            Asurftype(fov,line) = surftype(fov)
            Aelev(fov,line) = elevation(fov)
            ALandFrac(fov,line) = LandFrac(fov)
         ENDDO  ! fov

      ENDDO  ! line


! 3.  Correct for limb effect and surface emissivity
!     ------- --- ---- ------ --- ------- ----------

      CALL PPACORR ( ALastLine )


! 4.  Map AMSU-B to AMSU-A grid
!     --- ------ -- ------ ----

!     Note that this would normally be done from the ATOVPP main program
!     but is done here so that PPASURF can use AMSU-B channels before
!     the AMSU-A data is mapped to HIRS.

      DO imapping = 1, Nmappings
         IF ( MapInstr(imapping).EQ."AMSU-B" .AND.
     &        TargInstr(imapping).EQ."AMSU-A" ) THEN

            CALL PPMAP ("AMSU-B","AMSU-A",0,ALastLine,BLastLine,0)

         ENDIF
      ENDDO  ! imapping


! 5.  test for precipitation
!     ---- --- -------------

      CALL PPAPCP ( ALastLine )


! 6.  Estimate surface type from the data
!     -------- ------- ---- ---- --- ----

!    Uses channels from AMSU-A and optionally AMSU-B
!    If AMSU-B is not present, check to see if it is needed

      BMissing = .FALSE.
      IF (.NOT. BTEST(Ainstruments,bits(4))) THEN
        DO i=1,NASrfCh
          IF (ASrfChn(i) .GT. 15) BMissing = .TRUE. !Chans defined in AMSUA.fdf
        ENDDO
      ENDIF
      
      IF ( .NOT. BMissing ) THEN
         CALL PPASURF ( ALastLine )
      ELSE
         errStatus = -61
         errMessage =
     &   "Cannot guess AMSU-A surface type without mapped AMSU-B"
         CALL ErrorReport(errStatus,errMessage,"PPAMSUA1")
      ENDIF


      RETURN
      END
