!+ pre-process AMSU-B before the mapping
!
      SUBROUTINE PPAMSUB1 ( BLastLine, LSunit, HTunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppamsub1.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   pre-process a block of level 1c AMSU-B data
!
! Method:
! 1.  Add scan-dependent bias corrections (BIASCORR)
! 2.  Get land/sea information & elevation (SURFELEV)
! 3.  Correct for limb effect and surface emissivity
! 4.  Apply median filter to 89GHz channel
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1       9/12/96  include Bboxlv89
!  1.2      16/01/97  Bugs fixed: several Anfovs changed to Bnfovs
!                     lat & lon not now multiplied by 0.01
!  1.3      29/06/98  Bit 29 of Bqc becomes bit 20
!  1.4      10/07/98  PPBCORR added for consistency with HIRS pre-processing
!  1.5      15/03/05  Add LandFrac
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! fixed data for processing AMSU-B
#include "ppbfdata.h"        
! block of AMSU-B data
#include "ppbstore.h"       
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
!
!   Scalar arguments with intent(In):
      INTEGER       BLastLine     ! number of last AMSU-B line in store
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
!
! Local arrays:
      REAL          lats(Bnfovs)  ! array of latitudes for a scanline
      REAL          lons(Bnfovs)  ! array of longitudes for a scanline
      INTEGER       surftype(Bnfovs) !surface type for a scanline (SURFELEV)
      INTEGER       elevation(Bnfovs)!elevation for a scanline (SURFELEV)
      INTEGER       LandFrac(Bnfovs) !Land fraction(%) for a scanline (SURFELEV)
      REAL          btmed(Bnfovs,Bnlines) ! median BT in grid-box
!
! Function & Subroutine calls:
      External BIASCORR           ! scan-dependent bias corrections
      External SURFELEV           ! get land/sea information, (etc.)
      External PPBCORR            ! Correct for limb effect and surface emiss
      External MEDIAN             ! apply median filter to BTs in grid
!
!- End of header

      WRITE(*,"(10X,""Pre-processing AMSU-B before the mapping"")")

      Bpcpvn = 1000   ! precip/scattering detection version number
                      ! 0 means no detection was done
                      ! 1000-1999 intended for UKMO precip schemes
                      ! 9000+ for any test versions


! 1.  Add scan-dependent bias corrections
!     --- -------------- ---- -----------

      CALL BIASCORR( Bbiases, Bnchan, Bnchan, Bnfovs,
     &               BLastLine, Bbtemps )


! 2.  Get land/sea information
!     --- -------- -----------

!     loop through scanlines not already processed
      DO line = 1, BLastLine

! 2.1    pick out lat's and lon's from scanline
         DO fov = 1, Bnfovs
            lats(fov) = Blats(fov,line)        ! degrees N
            lons(fov) = Blons(fov,line)        ! degrees E
         ENDDO  ! fov

! 2.2    find surface type for each fov - using options read in PPUSER
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, Bnfovs, BboxSize,
     &                   surftype, elevation, LandFrac )

! 2.3    store surface type & elevation
         DO fov = 1, Bnfovs
            Bsurftype(fov,line) = surftype(fov)
            Belev(fov,line) = elevation(fov)
            BLandFrac(fov,line) = LandFrac(fov)
         ENDDO  ! fov

      ENDDO  ! line


! 3.  Correct for limb effect and surface emissivity
!     ------- --- ---- ------ --- ------- ----------

      CALL PPBCORR ( BLastLine )


! 4.  Apply median filter to 89GHz channel
!     ----- ------ ------ -- ----- -------

! 4.1 Calculate departure from median  (AMSU-B channel 1)
      CALL MEDIAN ( Bbtemps, Bqc, bits(0), 1, Bnchan,
     &              Bsurftype, 1, Bnfovs, BLastLine,
     &              Bboxlv89, Bboxlv89, btmed )


! 4.2 Flag those spots where departure from median is too large
      DO line = 1, BLastLine
      DO fov = 1, Bnfovs

         IF ( .NOT.BTEST(Bqc(fov,line),bits(0)) ) THEN
          IF ( ABS(Bbtemps(1,fov,line)-btmed(fov,line)) .GT. Bmaxlv89 )
     &             Bqc(fov,line) = IBSET ( Bqc(fov,line), bits(20) )
         ENDIF

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
