!+ pre-process AMSU-B after the mapping
!
      SUBROUTINE PPAMSUB2 ( BLastLine, LAMSA )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppamsub2.F 68 2007-02-23 14:40:47Z frna $
!
! Purpose:
!   pre-process a block of level 1c AMSU-B data
!
! Method:
!   (presumes we have mapped AMSU-A brightness temperatures)
!   Flag where AMSU-A and B 89GHz channels differ.
!   Recalculate AMSU-A scattering index using AMSU-B 89GHz instead of AMSU-A.
!   Calculate AMSU-AB cirrus index using channels A1,B16,B17,B20
!!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      29/06/98  Extensive changes to use mapped AMSU-A data
!  1.2      14/08/00  Added cirrus scattering index (D.C.Jones)
!  1.3      21/02/07  Check fov2 not zero. N C Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! fixed data for processing AMSU-A
#include "ppafdata.h"        
! block of AMSU-B data
#include "ppbstore.h"       
! block of AMSU-A data
#include "ppastore.h"       
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
!
!   Scalar arguments with intent(In):
      INTEGER       BLastLine     ! number of last AMSU-B line in store
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
!
! Local scalars:
      INTEGER       errStatus     ! error status (0 = OK)
      CHARACTER*80  errMessage    ! error message
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       fov2          ! fov number of nearest AMSU-A fov
      REAL          T1            ! brightness temp in AMSU-A channel 1 (K)
      REAL          T2            ! brightness temp in AMSU-A channel 2 (K)
      REAL          T3            ! brightness temp in AMSU-A channel 3 (K)
      REAL          T15           ! brightness temp in AMSU-B channel 1 (K)
      REAL          T17           ! brightness temp in AMSU-B channel 3 (K)
      REAL          T20           ! brightness temp in AMSU-B channel 5 (K)
      REAL          SI            ! scattering index
      REAL          CI            ! cirrus scattering index
      LOGICAL       scat          ! .TRUE. if scattering detected
      LOGICAL       cirrus        ! .TRUE. if cirrus detected
      REAL          avdiff_89_150(Bnfovs, Bnlines)   !89-150 difference (K)
      REAL          zen           ! Zenith angle
      INTEGER       lsma,lsmb     ! Land fractions (percent)
      REAL          m0,m1,m2,m3   ! Precipitation probabilities
      INTEGER       si_flag       ! Scattering index flag
      INTEGER       flagb         ! Used by scattering routines

! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External PPASCAT            ! scattering index
          REAL PPASCAT
      External PPACIRR            ! cirrus index
          REAL PPACIRR
!
!- End of header

      WRITE(*,"(10X,""Pre-processing AMSU-B after the mapping"")")

!     Check we have mapped AMSU-A, else we can't do pre-processing here
      IF ( .NOT. BTEST(Binstruments,bits(3)) ) THEN
         errStatus = -1
         errMessage =
     &   "Need AMSU-A brightness temps mapped to AMSU-B grid"
         CALL ErrorReport(errStatus,errMessage,"PPAMSUB2")
         GOTO 999
      ENDIF

! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, BLastLine
      DO fov = 1, Bnfovs

         T1  = BmapBT(1,fov,line)    ! mapped AMSU-A channel 1
         T2  = BmapBT(2,fov,line)    ! mapped AMSU-A channel 2
         T3  = BmapBT(3,fov,line)    ! mapped AMSU-A channel 3
         T15 = Bbtemps(1,fov,line)   ! AMSU-B channel 1 (89GHz)
         T17 = Bbtemps(2,fov,line)   ! AMSU-B channel 2 (150GHz)
         T20 = Bbtemps(5,fov,line)   ! AMSU-B channel 5 (183+-7GHz)

         fov2 = Bmapfov(fov,line)    ! fov no. of nearest AMSU-A fov

! 1.     Scattering test  (only over the sea)
!        ---------- ----

!        only do if channels present
         IF (  T1 .GT. RMDI  .AND.  T2 .GT. RMDI
     &   .AND. T3 .GT. RMDI  .AND.  T15 .GT. RMDI .AND. fov2.GT.0) THEN
            SI = PPASCAT ( T1, T2, T3, T15, fov2 )
            Bscatindx(fov,line) = SI
            IF ( Bsurftype(fov,line).EQ.0 ) THEN
               scat = SI .GT. Amaxscat
            ELSE
               scat = .FALSE.
            ENDIF
         ELSE
            Bscatindx(fov,line) = RMDI
            scat = .TRUE.  ! default is to err on side of caution
         ENDIF
         IF ( scat ) Bqc(fov,line) = IBSET ( Bqc(fov,line), bits(19) )

! 2.     Cirrus test  (only over the sea)
!        ------ ----

!        only do if channels present
         IF (  T1 .GT. RMDI  .AND.  T15 .GT. RMDI
     &   .AND. T17 .GT. RMDI  .AND.  T20 .GT. RMDI .AND. fov2.GT.0) THEN
            CI = PPACIRR ( T1, T15, T17, T20, fov2 )
            Bcirrindx(fov,line) = CI
            IF ( Bsurftype(fov,line).EQ.0 ) THEN
               cirrus = CI .GT. ABmaxcirr
            ELSE
               cirrus = .FALSE.
            ENDIF
         ELSE
            Bcirrindx(fov,line) = RMDI
            cirrus = .TRUE.  ! default is to err on side of caution
         ENDIF
         IF ( cirrus ) Bqc(fov,line) = IBSET ( Bqc(fov,line), bits(17) )

      ENDDO  ! fov
      ENDDO  ! line

 999  CONTINUE  ! jump here if we have no mapped AMSU-A data

! 3.    NWC-SAF scattering and precipitation test (from Ralf Bennartz)
!       -------------------------------------------------------------- 

      write(*,*)'         NWC-SAF scattering test'
      CALL average_T89_T150(BLastLine, 9, avdiff_89_150)  !Box of 9x9 spots

      DO line = 1, BLastLine
        DO fov = 1, Bnfovs
          T1  = BmapBT(1,fov,line)    ! mapped AMSU-A channel 1
          T15 = Bbtemps(1,fov,line)   ! AMSU-B channel 1 (89GHz)
          T17 = Bbtemps(2,fov,line)   ! AMSU-B channel 2 (150GHz)
          zen = Bangles(1,fov,line)   ! Local zenith angle
          flagb = 0
          lsmb = BLandFrac(fov,line)  ! AMSU-B land fraction
          lsma = BMapLandFrac(fov,line)  ! AMSU-A land fraction

C          write(*,*)'Inputs:',fov,line,T1,T15,T17,zen,flagb,lsma,lsmb,
C     +        avdiff_89_150(fov,line),Bsurftype(fov,line)

          CALL retrieve_one_si(T1,T15,T17,zen,flagb,lsma,lsmb,
     +                  avdiff_89_150(fov,line),SI,m0,m1,m2,m3,si_flag)

C          write(*,*)'Outputs:',SI,m0,m1,m2,m3,si_flag
          Bm0(fov,line) = m0
          Bm1(fov,line) = m1
          Bm2(fov,line) = m2
          Bm3(fov,line) = m3
          Bsi_nwcsaf(fov,line) = SI
          Bsi_nwcsaf_flag(fov,line) = si_flag
        ENDDO
      ENDDO

      RETURN
      END
