!+ Transfer data from program store to level 1D AMSU-A data record
!
      SUBROUTINE PPAOUTD ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppaoutd.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   ATOVPP stores data from several scanlines in its own internal array
!   structure, contained in common/ppastore/. This routine transfers a single
!   scanline from slot "Line" in common/ppastore/ to common/ama1ddt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1D include file (ama1d.h) and AMSU-A store
!     (ppastore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
!     Fill parts of 1D data record that won't change each scanline
! 1.  Copy variables from slot "Line" in common/ppastore/ to common/ama1ddt/
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1D format vn 1.9
!  1.2      17/01/97  Conform to 1D format vn 1.91
!  1.3      17/01/97  Conform to 1D format vn 1.92
!  1.4      17/01/97  Conform to ppastore format vn 1.3
!  1.5      15/06/98  Extra variables in ama1d_prepro
!  1.6      02/07/98  Aangles REAL not INTEGER
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! block of AMSU-A data
#include "ppastore.h"       
! AMSU-A level 1D header/data common blocks
#include "ama1d.h"          
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppastore/
!
! Local parameters:
      INTEGER       A1Dvnnb       ! version number of 1D format compatible
         PARAMETER (A1Dvnnb=193)  ! with the code in this subroutine
      INTEGER       Astvnnb       ! version number of "ppastore.h" compatible
         PARAMETER (Astvnnb=130)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov           ! loop variable over all fovs
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first                    ! .TRUE. on first call
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

! 0.1    Is the 1D include file compatible with this source code ?
         IF ( A1Dvnnb .NE. ama_1dvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1D format"",I6,"" but ama1d.h is "",I6)")
     &           A1Dvnnb, ama_1dvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPAOUTD")
         ENDIF

! 0.2    Is the ppastore include file compatible with this source code ?
         IF ( Astvnnb .NE. ama_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects Astore format"",I6,"" but ppastore.h is"",I6)")
     &           Astvnnb, ama_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPAOUTD")
         ENDIF

! 0.3    Fill parts of 1D data record that won't change each scanline
         DO i = 1, 88
            ama1d_filler(i) = MDI                     ! filler
         ENDDO  ! i

! 0.4    Reset "first" for subsequent calls
         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from slot "Line" in common/pphstore/ to common/ama1ddt/
!     ---- --------- ---- ----  ----  -- ---------------- -- ---------------

      ama1d_scnlin = Alineno(Line)                ! scanline number
      ama1d_scnlinyr = Ayear(Line)                ! scanline year
      ama1d_scnlindy = Aday(Line)                 ! scanline day of year
      ama1d_scnlintime = Atime(Line)              ! scanline time (ms)
      ama1d_qualind = Aqualind(Line)              ! quality indicator bit field
      ama1d_scanqual = Ascanqual(Line)            ! scan line quality flags
      ama1d_instrtemp1 = Ainstrtemp1(Line)        ! AMSU-A1 RF shelf temp
      ama1d_instrtemp2 = Ainstrtemp2(Line)        ! AMSU-A2 RF shelf temp
      ama1d_instrtempb = Ainstrtempb(Line)        ! AMSU-B Mixer ch 18-20 temp
      DO i = 1, 2
         ama1d_spare1(i) = MDI                    ! spare
      ENDDO  ! i

      DO fov = 1, Anfovs
         ama1d_latlon(1,fov) = NINT(Alats(fov,Line)*1.E4) !latitude (N +ve)
         ama1d_latlon(2,fov) = NINT(Alons(fov,Line)*1.E4) !longitude (E +ve)
         ama1d_surface(1,fov) = Aelev(fov,Line)   !surface height (m)
         ama1d_surface(2,fov) = Asurftype(fov,Line) !surface type
         DO i = 1, 4
            ama1d_angles(i,fov) = NINT(Aangles(i,fov,Line)*100.) ! scan angles:
!                                         (1,*,*) local zenith angle (deg*100)
!                                         (2,*,*) local azimuth angle (deg*100)
!                                         (3,*,*) solar zenith angle (deg*100)
!                                         (4,*,*) solar azimuth angle (deg*100)
         ENDDO  ! i
      ENDDO  ! fov

      ama1d_scalti = Aaltitude(Line)              ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
      DO i = 1, 2
         ama1d_spare2(i) = MDI                    ! spare
      ENDDO  ! i

!     AMSU-A & -B BTs (deg K * 100)
      DO fov = 1, Anfovs
      DO chn = 1, Anchan+Bnchan
         ama1d_btemps(chn,fov) = NINT(Abtemps(chn,fov,Line)*100.)
      ENDDO  ! chn
      ENDDO  ! fov

![user-defined:
      DO fov = 1, Anfovs
         ama1d_prepro(1,fov) = Aqc(fov,Line)       ! ATOVPP QC word for AMSU
         ama1d_prepro(2,fov) = Appsurf(fov,Line)   ! estimated surface type
         ama1d_prepro(3,fov) = NINT(Asurfcost(fov,Line)*100.) ! max prob cloud
         ama1d_prepro(4,fov) = NINT(Ascatindx(fov,Line)*100.) ! scattering index
         ama1d_prepro(5,fov) = NINT(Acrosby(fov,Line)*100.)   ! logistic precip
         DO i = 6, 7
            ama1d_prepro(i,fov) = MDI              ! spare
         ENDDO  ! i
      ENDDO  ! fov
! ... ]


      RETURN
      END
