!+ Set up (AMSU on AMSU-A grid) level 1D header within ATOVPP
!
      SUBROUTINE PPAOUTH ( LAMSB )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppaouth.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Set up level 1D header using information from 1C headers.
!
! Method:
! 1.  Check format version number and data type
!     AMSU-A level 1C header has already been read into common /ama1chd/
!     Check that format of 1C & 1D include files (ama1c.h, ama1d.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1C file is compatible with this subroutine.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Set up level 1D header
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1D format vn 1.9
!  1.2      17/01/97  Conform to 1D format vn 1.91
!  1.3      10/03/98  Conform to 1D format vn 1.92
!  1.4      15/06/98  Conform to 1D format vn 1.93
!  1.5      29/06/98  Use Ainstruments in output header
!  1.6      15/08/05  wmosatid instead of spare1 (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
! general parameters for ATOVPP
#include "ppparms.h"        
! parameters and look-up tables for the mapping
#include "ppluts.h"          
! fixed data for processing AMSU-A
#include "ppafdata.h"      
! fixed data for processing AMSU-B
#include "ppbfdata.h"       
! block of AMSU-A data 
#include "ppastore.h"       
! AMSU-A level 1C header/data common blocks
#include "ama1c.h"          
! AMSU-B level 1C header/data common blocks
#include "amb1c.h"          
! AMSU-A level 1D header/data common blocks
#include "ama1d.h"           
!
! Local parameters:
      INTEGER       A1Cvnnb       ! version number AMSU-A 1C format compatible
         PARAMETER (A1Cvnnb=190)  !    with the code in this subroutine
      INTEGER       B1Cvnnb       ! version number AMSU-B 1C format compatible
         PARAMETER (B1Cvnnb=190)  !    with the code in this subroutine
      INTEGER       A1Dvnnb       ! version number 1D format compatible
         PARAMETER (A1Dvnnb=193)  !    with the code in this subroutine
!
!   Scalar arguments with intent(In):
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! general purpose loop variables
      INTEGER       chn           ! loop variable over channels
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 1.1 Are the 1C include files compatible with this source code ?
!     --- --- -- ------- ----- ---------- ---- ---- ------ ----

!     AMSU-A
      IF ( A1Cvnnb .NE. ama_1cvnnb ) THEN
         errStatus = -111  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but ama1c.h is "",I6)")
     &        A1Cvnnb, ama_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPAOUTH")
      ENDIF

!     AMSU-B
      IF ( LAMSB .AND. B1Cvnnb .NE. amb_1cvnnb ) THEN
         errStatus = -112  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but amb1c.h is "",I6)")
     &        B1Cvnnb, amb_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPAOUTH")
      ENDIF

! 1.2 Is the 1D include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( A1Dvnnb .NE. ama_1dvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1D format "",I6,"" but ama1d.h is "",I6)")
     &        A1Dvnnb, ama_1dvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPAOUTH")
      ENDIF


! 2.  Set up level 1D header
!     --- -- ----- -- ------

!!!   This code is compatible with files:
!!!   "ama1c.h" vn1.9       10/03/98
!!!   "amb1c.h" vn1.9       10/03/98
!!!   "ama1d.h" vn1.92      10/03/98

! 2.1 General

!     creation site for 1D data
      ama1d_h_site   = SiteID              !file creation site ID (CHAR*3)
      ama1d_h_cfill1 = " "                 !filler

!     creation site for original 1B data
      ama1d_h_1bsite = ama1c_h_1bsite
      ama1d_h_cfill2 = " "                 !filler

      ama1d_h_versnb = ama_1dvnnb          !level 1d format version number
      ama1d_h_versyr = ama_1dvnyr          !level 1d format version year
      ama1d_h_versdy = ama_1dvndy          !level 1d format version day of year
      ama1d_h_hdrcnt = ama1c_h_hdrcnt      !count of header records
      ama1d_h_satid = ama1c_h_satid        !satellite id (e.g. 14 for NOAA-14)
      ama1d_h_grid = ama1c_h_instrument    !instrument grid code (AMSU-A=10)
      ama1d_h_satht = ama1c_h_satht        !nominal sat height, km*10
      ama1d_h_period = ama1c_h_period      !nominal orbit period (seconds)
      ama1d_h_startorbit = ama1c_h_startorbit !orbit number (at start)
      ama1d_h_startdatayr = ama1c_h_startdatayr!start of data set year
      ama1d_h_startdatady = ama1c_h_startdatady! (ditto) day of the year
      ama1d_h_startdatatime = ama1c_h_startdatatime !(ditto) UTC time of day,ms
      ama1d_h_endorbit = ama1c_h_endorbit  !orbit number (at end)
      ama1d_h_enddatayr = ama1c_h_enddatayr!end of data set year
      ama1d_h_enddatady = ama1c_h_enddatady!(ditto) day of the year
      ama1d_h_enddatatime = ama1c_h_enddatatime !(ditto) UTC time of day,ms
      ama1d_h_scnlin = ama1c_h_scnlin      !count of scan lines in this dataset
      ama1d_h_misscnlin = ama1c_h_misscnlin !count of missing scan lines
      ama1d_h_atovppvn = ATOVPPvn     !ATOVPP version number (test vns = 9000+)
      ama1d_h_instruments = Ainstruments   !instruments present
      ama1d_h_Aantenna = ama1c_h_vnantennacorr ! vn AMSU-A antenna corrections
      ama1d_h_wmosatid = ama1c_h_wmosatid      !WMO sat ID

! 2.2 temperature-radiance conversion coefficients

!     AMSU-A
!                  order of channels = 1,2,...,15 and AMSU-B 1,...,5
!                  for nochannel = 1 to 20 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
!     AMSU-A
      DO chn = 1, Anchan
      DO i = 1, 3
         ama1d_h_amsPlanck(i,chn) = ama1c_h_tempradcnv(i,chn)
      ENDDO  ! i
      ENDDO  ! chn

!     AMSU-B
      IF ( LAMSB ) THEN
         DO chn = 1, Bnchan
         DO i = 1, 3
            ama1d_h_amsPlanck(i,chn+Anchan) = amb1c_h_tempradcnv(i,chn)
         ENDDO  ! i
         ENDDO  ! chn
      ELSE
         DO chn = 1, Bnchan
         DO i = 1, 3
            ama1d_h_amsPlanck(i,chn+Anchan) = MDI
         ENDDO  ! i
         ENDDO  ! chn
      ENDIF

! 2.3 Pre-processing information

![user-defined:
      ama1d_h_dT  = dT         ! size of data batches processed within ATOVPP
      ama1d_h_mapBTmode = A2Hmode  ! mapping mode (AMSU-A to AMSU-A only)
      ama1d_h_Acorrvn  = Acorrvn        ! vn no AMSU-A limb/emiss correction
      ama1d_h_Apcpvn   = Apcpvn         ! vn no AMSU-A precip/scat detection
      ama1d_h_Bcorrvn  = Bcorrvn        ! vn no AMSU-B limb/emiss correction
      ama1d_h_Bpcpvn   = Bpcpvn         ! vn no AMSU-B precip/scat detection
      ama1d_h_Bantenna = amb1c_h_vnantennacorr ! vn AMSU-B antenna corrections
      ama1d_h_Ascat = Amaxscat !threshold value for AMSU-A scattering index
      ama1d_h_ACrosby = Amaxcrosby !threshold value for AMSU-A logistic precip
      ama1d_h_AB89 = ABmax89 !threshold value for AMSU-A/B 89GHz differences
      DO i = 1, 546
         ama1d_h_spare2(i) = MDI        !spare
      ENDDO  ! i
! ... ]


      RETURN
      END
