!+ Look for precipitation signal in AMSU-A
!
      SUBROUTINE PPAPCP ( ALastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppapcp.F 590 2018-06-05 10:51:49Z frna $
!
! Purpose:
!   Look for precipitation signal in AMSU-A and set flags accordingly.
!
! Method:
! 0.  Loop through scanlines and fields-of-view
! 1.  Scattering test
! 2.  Crosby, Ferraro & Wu test
! 3.  Grody light rainfall test
! 4.  Set flags
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      10/12/96  PPAMSU15 replaced by PPASCAT
!  1.2      20/05/98  Extra variables stored
!  1.3      18/06/98  Allow for missing data in the brightness temperatures
!  1.4      29/06/98  Bug fix  - PPASCAT and PPCROSBY both REAL not LOGICAL
!  1.5      19/02/99  Another bug fix  - correct check that BTs not missing
!  1.6      05/06/18  Don't set scat or crosby for Metop-B with failed AMSU 15
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! fixed data for processing AMSU-A
#include "ppafdata.h"       
! block of AMSU-A data
#include "ppastore.h"        
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
!
!   Scalar arguments with intent(In):
      INTEGER       ALastLine     ! number of last AMSU-A line in store
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      REAL          T1            ! brightness temp in AMSU-A channel 1 (K)
      REAL          T2            ! brightness temp in AMSU-A channel 2 (K)
      REAL          T3            ! brightness temp in AMSU-A channel 3 (K)
      REAL          T15           ! brightness temp in AMSU-A channel 15 (K)
      REAL          SI            ! scattering index
      REAL          probprecip    ! Crosby probability of precipitation
      LOGICAL       scat          ! .TRUE. if scattering detected
      LOGICAL       crosby        ! .TRUE. if Crosby test flags fov
      LOGICAL       grody         ! .TRUE. if Grody test flags fov
!
! Local arrays:
!
! Function & Subroutine calls:
      External PPASCAT            ! scattering index
          REAL PPASCAT
      External PPCROSBY           ! logistic regression
          REAL PPCROSBY
      External PPGRODY            ! light rainfall test
       LOGICAL PPGRODY
!
!- End of header

      Apcpvn = 1000   ! precip/scattering detection version number
                      ! 0 means no detection was done
                      ! 1000-1999 intended for UKMO precip schemes
                      ! 9000+ for any test versions

! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, ALastLine
      DO fov = 1, Anfovs

         T1  = Abtemps(1,fov,line)
         T2  = Abtemps(2,fov,line)
         T3  = Abtemps(3,fov,line)
         T15 = Abtemps(15,fov,line)


! 1.     Scattering test  (only over the sea)
!        ---------- ----

!        only do if channels present
         IF (ASatId .EQ. 1 .AND. T15 .LE. 0) THEN
            Ascatindx(fov,line) = RMDI
            scat = .FALSE.    !Metop-B failed channel 15
         ELSE IF (  T1 .GT. 0.  .AND.  T2 .GT. 0.
     &   .AND. T3 .GT. 0.  .AND.  T15 .GT. 0. ) THEN
            SI = PPASCAT ( T1, T2, T3, T15, fov )
            Ascatindx(fov,line) = SI
            IF ( Asurftype(fov,line).EQ.0 ) THEN
               scat = SI .GT. Amaxscat
            ELSE
               scat = .FALSE.
            ENDIF
         ELSE
            Ascatindx(fov,line) = RMDI
            scat = .TRUE.  ! default is to err on side of caution
         ENDIF


! 2.     Crosby, Ferraro & Wu test
!        ------  ------- - -- ----

         IF (ASatId .EQ. 1 .AND. T15 .LE. 0) THEN
            Acrosby(fov,line) = RMDI
            crosby = .FALSE.    !Metop-B failed channel 15
         ELSE IF (  T1 .GT. 0.  .AND.  T15 .GT. 0. ) THEN
            probprecip = PPCROSBY(T1,T15 )
            Acrosby(fov,line) = probprecip
            crosby = probprecip .GT. Amaxcrosby
         ELSE
            Acrosby(fov,line) = RMDI
            crosby = .TRUE.  ! default is to err on side of caution
         ENDIF


! 3.        Grody light rainfall test
!           ----- ----- -------- ----

         IF (  T1 .GT. 0.  .AND.  T2 .GT. 0. ) THEN
            grody = PPGRODY( T1, T2 )
         ELSE
            grody = .TRUE.  ! default is to err on side of caution
         ENDIF


! 4.     Set flags
!        --- -----

         IF ( scat )   Aqc(fov,line) = IBSET ( Aqc(fov,line), bits(26) )
         IF ( crosby ) Aqc(fov,line) = IBSET ( Aqc(fov,line), bits(25) )
         IF ( grody )  Aqc(fov,line) = IBSET ( Aqc(fov,line), bits(24) )

!        set flag for use by the mapping routines:
         IF ( scat .OR. crosby )
     &                 Aqc(fov,line) = IBSET ( Aqc(fov,line), bits(31) )

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
