!+ calculate scattering index
!
      FUNCTION PPASCAT ( TB1, TB2, TB3, TB15, FOV )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppascat.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   estimate AMSU-A channel 15 brightness temperature from channels 1,2,3
!
! Method:
!   Estimate AMSU-A channel 15 brightness temperature from channels 1,2,3
!   The equation we are are using is T15 = A1 + T1*A2 + T2*A3 + T3*A4
!   A1, A2, A3 and A4 are functions of the secant of the view angle - 1
!   Then calculate scattering index as estimated TB15 - observed TB15
!
! Theory: For long wavelengths (>3mm) very few objects cause significant
!         scattering. For wavelengths between 0.5 and 3mm an increasing
!         number of surfaces significantly scatter radiation e.g. sand
!         grains in a desert, brine pockets in sea ice, air pockets in sea
!         ice, snow grains, ripples on the ocean surface. At very short
!         wavelengths less than 0.5mm scattering by ice hydrometeors and
!         precipitation sized water droplets becomes significant. In practise
!         the relatively wide spacing of AMSU window channels 20, 30, 50, 90,
!         160 GHz means all these surfaces/particles appear to scatter most
!         significantly at high frequency and less at lower frequency. A simple
!         scattering index can be devised that predicts the 85.5/89/91 GHz
!         channel (89 in the case of AMSU-A; most radiometers have a channel
!         near 90 GHz) from the lower frequency channels. In the absence of
!         scattering brightness temperature generally increases with frequency
!         and therefore the prediction will always predict TB89>TB31 etc. But
!         in the presence of scattering the 89 GHz brightness temperature is
!         lowered. Therefore the low frequency channels overestimate the high
!         frequency channel. Therefore the difference between the predicted
!         and measured 89 GHz TB gives us a simple scattering index.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Derived from Steve English's PV-Wave code (27/10/95)
!  1.1      10/12/96  Changed to a logical function
!  1.2      20/05/98  Changed back to return just the S.I.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! fixed data for processing AMSU-A    
#include "ppafdata.h"       
!
!   Scalar arguments with intent(In):
      REAL          TB1           ! brightness temp in AMSU-A channel 1 (K)
      REAL          TB2           ! brightness temp in AMSU-A channel 2 (K)
      REAL          TB3           ! brightness temp in AMSU-A channel 3 (K)
      REAL          TB15          ! brightness temp in AMSU-A channel 15 (K)
      REAL          ETB15         ! estimated BT in AMSU-A channel 15 (K)
      INTEGER       FOV           ! fov number in scan
!
!   Return value of function:
      REAL          PPASCAT       ! scattering index = ETB15 - TB15
!
!- End of header


! 1.  Estimate channel 15
!     -------- ------- --

      ETB15 = Ascat(1,FOV)
     &      + Ascat(2,FOV)*TB1
     &      + Ascat(3,FOV)*TB2
     &      + Ascat(4,FOV)*TB3

! 2.  Calculate scattering index
!     --------- ---------- -----

      PPASCAT = ETB15 - TB15

      RETURN
      END
