!+ Estimate surface type from AMSU-A
!
      SUBROUTINE PPASURF ( ALastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppasurf.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!
!
! Method:
! 0.  Loop through scanlines and fields-of-view
! 1.  Interpolate means & covariances to scan angle
! 2.  Find most likely surface type
!
! Surface types:
! 1 = Bare young ice (i.e. new ice, no snow)
! 2 = Dry land (i.e. dry with or without significant vegetation)
! 3 = Dry snow (i.e. snow with water less than 2%, over land)
! 4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
! 5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
! 6 = Wet forest (i.e. established forest with wet canopy)
! 7 = Wet land (i.e. non-forested land with a wet surface)
! 8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
! 9 = Desert
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Derived from Steve English's PV-Wave code (27/10/95)
!  1.1      10/01/97  AXcost new  - multiplier for cost function without CLW
!  1.2      20/05/98  Now just calculate cost for cloud-free surfaces, and
!                     apply threshold on cost to detect cloud.
!  1.3      02/07/98  Aangles REAL not INTEGER
!  1.4      15/09/04  Use constants_preproc.h for fundamental constants - NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! fixed data for processing AMSU-A 
#include "ppafdata.h"      
! block of AMSU-A data
#include "ppastore.h"    
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
! Fundamental and physical constants
#include "constants_preproc.h"    
!
!   Scalar arguments with intent(In):
      INTEGER       ALastLine     ! number of last AMSU-A line in store
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       iang          ! loop index over angles
      INTEGER       isurf         ! loop index over surface types
      INTEGER       k, kx, m      ! loop indices over channels
      INTEGER       kang          ! index of where ASecAng .GE. secang
      INTEGER       surftype      ! derived surface type
      REAL          normalise     ! normalise J wrt number of channels
      REAL          J             ! cost function for a given surface type
      REAL          Jmin          ! minimum value of J for all surface types
      REAL          secang        ! secant(satellite zenith angle)
      REAL          sec_norm      ! normalisation factor for interpolation
      REAL          sum           ! sum of cost function over channels
!
! Local arrays:
      REAL meanBTx(NASrfTyp,NASrfChMx) !mean btemp difference vector at secang
      REAL covBTx(NASrfTyp,NASrfChMx,NASrfChMx) ! inverse cov mx at secang
      REAL diffBTx(NASrfChMx)     ! difference from mean btemps vector
!
!- End of header

      Asurfvn = 1000  ! surface identification version number
                      ! 0 means no identification was done
                      ! 1000-1999 intended for UKMO schemes
                      ! 9000+ for any test versions

!     normalise cost function according to the number of channels used
      normalise = 1. / REAL(NASrfCh*NASrfCh)

! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, ALastLine
      DO fov = 1, Anfovs

! 1.     Interpolate means & covariances to scan angle
!        ----------- ----- - ----------- -- ---- -----
! ??? Is this better done for the 15 standard view angles within PPAFDF ???

         secang = 1. / COS(Aangles(1,fov,line)*deg2rad) ! secant of view angle

!        Find specified angles either side of view angle
         kang = 0
         DO iang = 1, NASecAng-1
            IF ( secang.GE.ASecAng(iang)
     &     .AND. secang.LT.ASecAng(iang+1) ) kang = iang
         ENDDO  ! iang

!        Derive appropriate means & covariances
         IF ( kang .EQ. 0 ) THEN !secang>ASecAng(NASecAng):use ASecAng(NSecAng)
            DO isurf = 1, NASrfTyp
            DO k = 1, NASrfCh
               meanBTx(isurf,k) = AmeanBT(isurf,NASecAng,k)
               DO m = 1, NASrfCh
                  covBTx(isurf,m,k) = AcovBT(isurf,NASecAng,m,k)
               ENDDO  ! m
            ENDDO  ! k
            ENDDO  ! isurf
         ELSE                      ! sec < ASecAng(NASecAng): interpolate
            sec_norm = (secang-ASecAng(kang))
     &                 / (ASecAng(kang+1)-ASecAng(kang))
            DO isurf = 1, NASrfTyp
            DO k = 1, NASrfCh
               meanBTx(isurf,k) = AmeanBT(isurf,kang,k) + sec_norm*
     &                   (AmeanBT(isurf,kang+1,k)-AmeanBT(isurf,kang,k))
               DO m = 1, NASrfCh
                  covBTx(isurf,m,k) = AcovBT(isurf,kang,m,k) +
     &        sec_norm*(AcovBT(isurf,kang+1,m,k)-AcovBT(isurf,kang,m,k))
               ENDDO  ! m
            ENDDO  ! k
            ENDDO  ! isurf
         ENDIF  ! kang .EQ. 0


! 2.     Find most likely surface type
!        ---- ---- ------ ------- ----

!        We evaluate the cost function J = (to-tm).C^-1.(to-tm)T
!        to find the minimum value, looping through each surface type:
         Jmin = 9999.0
         DO isurf = 1, NASrfTyp

            DO k = 1, NASrfCh
               kx = ASrfChn(k)
               diffBTx(k) = Abtemps(kx,fov,line) - meanBTx(isurf,k)
            ENDDO  ! k

!           J = ABS(diffBTx times cov.matrix times diffBTx(transpose))
            J = 0.
            DO k = 1, NASrfCh
               sum = 0.
               DO m = 1, NASrfCh
                  sum = sum + covBTx(isurf,m,k)*diffBTx(m)
               ENDDO  ! m
               J = J + diffBTx(k)*sum
            ENDDO  ! k
            J = ABS(J)

!           Is this the best fit yet ?
            IF ( J .LT. Jmin ) THEN
               Jmin = J
               surftype = isurf
            ENDIF

         ENDDO  ! isurf

!        normalise according to the number of channels used
         Jmin = Jmin * normalise

!        apply threshold check to minimum J for cloud
         IF ( Jmin .GT. ASrfThresh ) THEN
            Aqc(fov,line) = IBSET ( Aqc(fov,line), bits(27) )
         ENDIF

!        if snow or sea-ice, check channel 1 to see if this is actually desert
         IF ( surftype.EQ.1 .OR. surftype.EQ.4 .OR. surftype.EQ.8 ) THEN
            IF ( Abtemps(1,fov,line).GT.275. ) surftype = 9    ! desert
         ENDIF

!        store minimum J and surface type
         Asurfcost(fov,line) = Jmin
         Appsurf(fov,line) = surftype

!        set flag if this disagrees with surface type from topography dataset
         IF ( Asurftype(fov,line).EQ.0 ) THEN      ! sea
            IF ( surftype.EQ.2 .OR. surftype.EQ.3 .OR. surftype.EQ.6
     &      .OR. surftype.EQ.7 .OR. surftype.EQ.9 )
     &           Aqc(fov,line) = IBSET ( Aqc(fov,line), bits(22) )
         ELSEIF ( Asurftype(fov,line).EQ.2 ) THEN  ! land
            IF ( surftype.EQ.5 )
     &           Aqc(fov,line) = IBSET ( Aqc(fov,line), bits(22) )
         ENDIF

!        set flag if this surface type is one we can't handle
         IF ( surftype.EQ.2 .OR. surftype.EQ.3 .OR. surftype.EQ.6
     &      .OR. surftype.EQ.7 .OR. surftype.EQ.9 )
     &           Aqc(fov,line) = IBSET ( Aqc(fov,line), bits(31) )

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
