!+ pre-process ATMS
!
      SUBROUTINE PPATMS1 ( atmsLastLine, LSunit, HTunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatms1.F 456 2015-07-27 14:58:53Z frna $
!
! Purpose:
!   pre-process a block of level 1c ATMS data
!
! Method:
! 1.  Get land/sea information & elevation (SURFELEV)
! 2.  Test for precipitation (PPAPCP)
! 3.  Estimate surface type from the data (PPASURF)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    02/09/2010  From ppamsua1.F
!  1.1    23/07/2015  Reduce BoxSize, used for surface type and elevation
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! parameters and look-up tables for the mapping
#include "ppluts.h"
! fixed data for processing ATMS         
#include "ppatmsfdata.h"
! block of ATMS data       
#include "ppatmsstore.h"  
! ATMS 1c       
#include "atms1c.h"  
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
!
!   Scalar arguments with intent(In):
      INTEGER       atmsLastLine  ! number of last ATMS line in store
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       imapping      ! loop index, looping through mappings
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! loop index
      INTEGER       BoxSize       ! box size for SURFELEV
!
! Local arrays:
      REAL          lats(atmsnfovs)  ! array of latitudes for a scanline
      REAL          lons(atmsnfovs)  ! array of longitudes for a scanline
      INTEGER       surftype(atmsnfovs) !surface type for a scanline (SURFELEV)
      INTEGER       elevation(atmsnfovs)!elevation for a scanline (SURFELEV)
      INTEGER       LandFrac(atmsnfovs) !Land fraction(%) for a scanline (SURFELEV)
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External SURFELEV           ! get land/sea information, (etc.)
      External PPAPCP             ! test for precipitation, etc.
      External PPASURF            ! estimate surface type from the data
!
!- End of header

      WRITE(*,"(10X,""Pre-processing ATMS before the mapping"")")

! 1.  Get land/sea information
!     --- -------- -----------

!     loop through scanlines not already processed
      DO line = 1, atmsLastLine

! 1.1    pick out lat's and lon's from scanline
         DO fov = 1, atmsnfovs
            lats(fov) = atmslats(fov,line)        ! degrees N
            lons(fov) = atmslons(fov,line)        ! degrees E
         ENDDO  ! fov

! 1.2    find surface type for each fov - using options read in PPUSER

   !For ATMS the beam width is different for different channels.
   !Ckeck the 1c header to see whether averaging has been done or not, and
   !scale atmsboxSize accordingly (assume atmsboxSize applies to channel 1).
   !Note that the AMSU-A box size (100km half-width) is very generous
   !(4 times the beam half-width).
   !Use channel 3 for surface typ and elevation.
         
         IF (atms1c_h_beamwidth(3).GT.0) THEN
           BoxSize = atms1c_h_beamwidth(3)*1E-3/5.2 * atmsboxSize
         ELSE IF (atms1c_h_nxaverage(3).GT.0) THEN
           BoxSize = atms1c_h_nxaverage(3)*atmsboxSize*1.1/5.2
         ELSE
           BoxSize = atmsboxSize*2.2/5.2
         ENDIF
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, atmsnfovs, BoxSize,
     &                   surftype, elevation, LandFrac )

! 1.3    store surface type & elevation
         DO fov = 1, atmsnfovs
            atmssurftype(fov,line) = surftype(fov)
            atmselev(fov,line) = elevation(fov)
            atmsLandFraca(fov,line) = LandFrac(fov)
         ENDDO  ! fov

! 1.4    repeat for channel 16, representing AMSU-B-like channels
         IF (atms1c_h_beamwidth(16).GT.0) THEN
           BoxSize = atms1c_h_beamwidth(16)*1E-3/5.2 * atmsboxSize
         ELSE IF (atms1c_h_nxaverage(16).GT.0) THEN
           BoxSize = atms1c_h_nxaverage(16)*atmsboxSize*1.1/5.2
         ELSE
           BoxSize = atmsboxSize*1.1/5.2
         ENDIF
         CALL SURFELEV ( LSunit, HTunit,
     &                   lats, lons, atmsnfovs, BoxSize,
     &                   surftype, elevation, LandFrac )

! 1.5    store land fraction
         DO fov = 1, atmsnfovs
            atmsLandFracb(fov,line) = LandFrac(fov)
         ENDDO  ! fov

      ENDDO  ! line


! 2.  test for precipitation
!     ---- --- -------------

      CALL PPATMSPCP ( atmsLastLine )


! 3.  Estimate surface type from the data
!     -------- ------- ---- ---- --- ----

      CALL PPATMSSURF ( atmsLastLine )


      RETURN
      END
