!+ Read fixed data file for ATMS within ATOVPP
!
      SUBROUTINE PPATMSFDF ( atmsfunit, atmsignoreunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatmsfdf.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Read fixed data file required for ATMS within ATOVPP.
!
! Method:
! 0.  initialisation
! 1.  open file
! 2.  look for sections in the file
! 3.  read and derive coefficients for precip/scattering tests PPAPCP
! 4.  read and derive coefficients for cirrus index PPACIRR
! 5.  read and derive numbers for surface identification PPASURF
! 6.  read bias corrections
! 7.  close file
!
!   File format is same as AMSU-A
!
! History:
! Version  Date      Comment
!
!  1.0    06/09/2010  From PPATMSFDF.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"  
! fixed data for processing ATMS
#include "ppatmsfdata.h"
! block of ATMS data 
#include "ppatmsstore.h"        
!Fundamental and physical constants
#include "constants_preproc.h"    
! ATMS 1c common blocks
#include "atms1c.h"
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       atmsfunit        ! unit number for ATMS-specific data
      INTEGER       atmsignoreunit   ! unit number for ATMS ignore file
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from file
      CHARACTER*80  errMessage    ! error message
      CHARACTER*30  Cline
      INTEGER       errStatus     ! error status value
      INTEGER       chn           ! loop index: channel number
      INTEGER       fov           ! loop index: fov number
      INTEGER       i, j          ! general purpose loop indices
      INTEGER       wmosatid
      INTEGER       ignore_number
!     for scattering coefficients:
      REAL          angle         ! surface zenith angle for the fov
      REAL          satang        ! satellite zenith angle
      REAL          ratio         ! ratio defined as (rEarth+SatHt) / rEarth
      REAL          x             ! secant(surface zenith angle) - 1.
!     for surface type:
      REAL          det           ! determinant from s/r MXINV
      INTEGER       iang          ! loop index over angles
      INTEGER       isurf         ! loop index over surface types
      INTEGER       k, m          ! loop indices over channels
      INTEGER       kx, mx        ! indices over reduced set of channels
!
! Local Arrays:
      INTEGER       SCRATCH(NatmsSrfChMx,2)  ! work array for s/r MXINV
      REAL aa(4,4)                ! polynomial coefficients for PPASCAT tests
      REAL tb(20)                 ! BT means as read for PPASURF for AMSU CHANNELS
      REAL tbc(20,20)             ! BT cov mx as read for PPASURF for AMSU CHANNELS
      REAL tbcx(NatmsSrfChMx,NatmsSrfChMx)   ! tbc on reduced set of channels
      REAL zatmsSecAng(NatmsSecAng) ! secant(angles) for PPASURF
      DATA zatmsSecAng / 1.00, 1.25, 1.50, 1.75, 2.00 /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External MXINV                ! matrix inversion routine
      External RDINTS               ! read integers from character string

!- End of header


! 0.  initialisation
!     --------------

! 0.1 bias corrections
      DO fov = 1, atmsnfovs
      DO chn = 1, atmsnchan
         atmsbiases(chn,fov) = 0.
      ENDDO  ! chn
      ENDDO  ! fov

! 0.2 secants of fixed angles for PPASURF
      DO i = 1, NatmsSecAng
         atmsSecAng(i) = zatmsSecAng(i)
      ENDDO  ! i


! 1.  open file
!     ---- ----

      OPEN ( UNIT=atmsfunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!     If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=atmsfunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

      WRITE ( errMessage,
     & "(""Error opening ATMS fixed data file, unit "",I8)" ) atmsfunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")


! 2.  look for sections in the file
!     ---- --- -------- -- --- ----

      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )
         input_line = " "
         READ(atmsfunit,"(A80)",IOSTAT=errStatus) input_line


         IF ( input_line(1:6) .EQ. "PREPRO" ) THEN

! 3.        read and derive coefficients for precip/scattering tests PPAPCP
!           ---- --- ------ ------------ --- ----------------- ---- -------

! 3.1       logistic precipitation probability test PPCROSBY

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading Crosby numbers"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(atmsfunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")
            READ(input_line,"(4F8.3)",IOSTAT=errStatus)
     &           atmscrosby1, atmscrosby2, atmscrosby3, atmsmaxcrosby

! 3.2       Grody light rainfall test PPGRODY

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading Grody numbers"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(atmsfunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")
            READ(input_line,"(2F8.3)",IOSTAT=errStatus)
     &           atmsgrody1, atmsgrody2

! 3.3       coefficients for scattering test PPASCAT
!           The equation we are are using is
!                   T15=a0 + T1 * a1 + T2 * a2 + T3 * a3
!           a0, a1, a2 and a3 are functions of secant(view angle) - 1
!           A third order polynomial is used to calculate a0, a1, a2 and a3.
!           The polynomial coefficents are read into "aa"

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading atmsmaxscat"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(atmsfunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")
            READ(input_line,*,IOSTAT=errStatus) atmsmaxscat ! test threshold

            READ(atmsfunit,"(4F13.8)",IOSTAT=errStatus)
     &                           ((aa(i,j),i=1,4),j=1,4)
            WRITE ( errMessage,
     &    "(""Error reading ATMS fdf scat, unit "",I8)" ) atmsfunit
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

!           Calculate coefficients to predict ATMS channel 16 from 1,2,3.
            ratio = (rEarth+atmssatht) / rEarth
            satang = -atmsangle * REAL(atmsnfovs+1) / 2.
            DO fov = 1, atmsnfovs
               satang = satang + atmsangle
!              convert zenith angle at satellite to z.angle (rad) at surface
               angle = ASIN ( SIN(satang*deg2rad) * ratio )
               x = 1./COS(angle) - 1.
               DO j = 1, 4
                  atmsscat(j,fov) =
     &              aa(1,j) + aa(2,j)*x + aa(3,j)*x*x + aa(4,j)*x*x*x
               ENDDO  ! i
            ENDDO  ! fov

! 4.        read and derive coefficients for cirrus test PPACIRR
!           ---- --- ------ ------------ --- ------ ---- -------

! 4.1       coefficients for scattering test PPACIRR
!           The equation we are are using is
!                   T20=a0 + T1 * a1 + T16 * a2 + T17 * a3
!           a0, a1, a2 and a3 are functions of secant(view angle) - 1
!           A third order polynomial is used to calculate a0, a1, a2 and a3.
!           The polynomial coefficents are read into "aa"

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading ABmaxcirr"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(atmsfunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")
            READ(input_line,*,IOSTAT=errStatus) atmsmaxcirr ! test threshold

            READ(atmsfunit,"(4F13.8)",IOSTAT=errStatus)
     &                           ((aa(i,j),i=1,4),j=1,4)
            WRITE ( errMessage,
     &    "(""Error reading ATMS fdf cirr, unit "",I8)" ) atmsfunit
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

!           Calculate coefficients to predict AMSU-B channel 20 from A1,B16,B17.
            ratio = (rEarth+atmssatht) / rEarth
            satang = -atmsangle * REAL(atmsnfovs+1) / 2.
            DO fov = 1, atmsnfovs
               satang = satang + atmsangle
!              convert zenith angle at satellite to z.angle (rad) at surface
               angle = ASIN ( SIN(satang*deg2rad) * ratio )
               x = 1./COS(angle) - 1.
               DO j = 1, 4
                  atmscirr(j,fov) =
     &              aa(1,j) + aa(2,j)*x + aa(3,j)*x*x + aa(4,j)*x*x*x
               ENDDO  ! i
            ENDDO  ! fov

! 5.        read and derive numbers for surface identification PPASURF
!           ---- --- ------ ------- --- ------- -------------- -------

! 5.1       Choice of channels for use in determining ATMS surface type

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading channels for PPASURF"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(atmsfunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

!           Having read the line, extract channel numbers
            CALL RDINTS ( input_line, atmsSrfChn, NatmsSrfCh )

!           Check all is ok
            IF ( NatmsSrfCh.LT.1 .OR. NatmsSrfCh.GT.NatmsSrfChMx ) THEN
               errStatus  = 411  ! fatal
               errMessage = " "
               WRITE ( errMessage,
     &           "(""Too many/few channels for use in PPASURF: "",I8)" )
     &           NatmsSrfCh
               CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")
            ENDIF
            DO i = 1, NatmsSrfCh
               IF ( atmsSrfChn(i).LT.1 .OR.
     &              atmsSrfChn(i).GT.atmsnchan ) THEN
                  errStatus  = 412  ! fatal
                  errMessage = " "
                  WRITE ( errMessage,
     &     "(""Channel out of range for PPASURF: "",I8)" ) atmsSrfChn(i)
               ENDIF
            ENDDO  ! i
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

! 5.2       threshold for detecting cloud in the cost function

!           read the whole line into memory  (ignoring lines starting "!")
            input_line = "!"
            errStatus = 0
            errMessage = "Error reading cloud threshold for PPASURF"
            DO WHILE ( input_line(1:1) .EQ. "!" .AND. errStatus.EQ.0 )
               READ(atmsfunit,"(A80)",IOSTAT=errStatus) input_line
            ENDDO
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

!           Having read the line, read the threshold
            READ(input_line,*,IOSTAT=errStatus) atmsSrfThresh
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

! 5.3       mean brightness temperatures

!           read comment line in the file
            READ(atmsfunit,*,IOSTAT=errStatus)
            WRITE ( errMessage,
     &  "(""Error reading coeffs for PPASURF, unit "",I8)" ) atmsfunit
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

!           loop through surface types
            DO isurf = 1, NatmsSrfTyp

               READ(atmsfunit,*,IOSTAT=errStatus)  ! comment line in the file
               errStatus = ABS (errStatus)   ! end-of-file returns -1
               CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

!              loop through angles
               DO iang = 1, NatmsSecAng

                  READ(atmsfunit,"(6F13.3)",IOSTAT=errStatus)
     &                (tb(m),m=1,20)      !AMSU-like channels
                  errStatus = ABS (errStatus)   ! end-of-file returns -1
                  CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")
!                 store reduced set of channels
                  DO k = 1, NatmsSrfCh
                     kx = atmsSrfChn(k)
                     atmsmeanBT(isurf,iang,k) = tb(kx)
                  ENDDO  ! k

               ENDDO  ! iang
            ENDDO  ! isurf

!           read comment line in the file
            READ(atmsfunit,*,IOSTAT=errStatus)
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

! 5.4       brightness temperature covariance matrices
!           loop through surface types
            DO isurf = 1, NatmsSrfTyp

               READ(atmsfunit,*,IOSTAT=errStatus)  ! comment line in the file
               errStatus = ABS (errStatus)   ! end-of-file returns -1
               CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

!              loop through angles
               DO iang = 1, NatmsSecAng

                  DO m = 1, 20  !AMSU-like channels
                     READ(atmsfunit,"(6F13.4)",IOSTAT=errStatus)
     &               (tbc(k,m),k=1,20)
                     errStatus = ABS (errStatus)   ! end-of-file returns -1
                     CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")
                  ENDDO  ! m
!                 store reduced set of channels
                  DO k = 1, NatmsSrfCh
                     kx = atmsSrfChn(k)
                     DO m = 1, NatmsSrfCh
                        mx = atmsSrfChn(m)
                        tbcx(m,k) = tbc(mx,kx)
                     ENDDO  ! m
                  ENDDO  ! k

!                 invert the covariance matrix
                  CALL MXINV ( tbcx, NatmsSrfCh, NatmsSrfChMx, SCRATCH,
     &                         det, 0.01 )

!                 and store it
                  DO k = 1, NatmsSrfCh
                  DO m = 1, NatmsSrfCh
                     atmscovBT(isurf,iang,m,k) = tbcx(m,k)
                  ENDDO  ! m
                  ENDDO  ! k

               ENDDO  ! iang
            ENDDO  ! isurf


         ELSEIF ( input_line(1:4) .EQ. "BIAS" ) THEN

! 6.        read bias-corrections
!           ---- ----------------

!           read scan-dependent bias corrections
            errMessage = "Reading BIAS section"
            errStatus = 0
            DO fov = 1, atmsnfovs
               IF ( errStatus.EQ.0 ) THEN
                  READ(atmsfunit,"(10F8.3)",IOSTAT=errStatus)
     &                       (atmsbiases(chn,fov),chn=1,atmsnchan)
               ENDIF
            ENDDO  ! fov
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")


         ENDIF  ! input_line

      ENDDO  ! WHILE ( errStatus.EQ.0 )


! 7.  close file
!     ----- ----

      CLOSE(atmsfunit)


! 8.  read the ignore file
!     --------------------

      OPEN ( UNIT=atmsignoreunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!     If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=atmsignoreunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

      WRITE ( errMessage,
     & "(""Error opening ATMS ignore file, unit "",I8)" ) atmsignoreunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPATMSFDF")

      wmosatid=999
      DO WHILE (wmosatid .NE. atms1c_h_wmosatid)
        read(atmsignoreunit,'(a30)',end=100) Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(atmsignoreunit,'(a30)',end=100) Cline
        ENDDO
        READ(Cline,*) wmosatid

        read(atmsignoreunit,'(a30)',end=100) Cline
        DO WHILE (Cline(1:1) .eq. '#')
          read(atmsignoreunit,'(a30)',end=100) Cline
        ENDDO
        READ(Cline,*) ignore_number

        DO chn=1,atmsnchan
          atms_ignore_channel(chn) = .FALSE.
        ENDDO
        DO i=1,ignore_number
          read(atmsignoreunit,'(a30)',end=100) Cline
          DO WHILE (Cline(1:1) .eq. '#')
            read(atmsignoreunit,'(a30)',end=100) Cline
          ENDDO
          READ(Cline,*) chn
          atms_ignore_channel(chn) = .TRUE.
        ENDDO
      ENDDO 
  100 IF (wmosatid .NE. atms1c_h_wmosatid) THEN
        WRITE(*,*) 'WARNING: sat id not matched: use',wmosatid
      ENDIF
        
      CLOSE(atmsignoreunit)

      RETURN
      END
