!+ Read & store level 1C ATMS data within ATOVPP
!
      SUBROUTINE PPATMSGET ( atms1Cunit, atmsLastLine,
     &                    Year1, Day1, Time1,
     &                    Year2, Day2, Time2 )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatmsget.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Update ATOVPP store of ATMS scanlines to hold data within
!   start and end dates/times specified (Year1/Day1/Time1 & Year2/Day2/Time2)
!   with an extra 1 or 2 scanlines at either end to help with the mapping.
!
! Method:
! 1.  Look for first suitable scanline in previous block
! 2.  Define record number to start reading from
! 3.  Set data store to all "missing"
! 4.  Read data till we come to end of time interval, or of dataset
!
!   The position of each scanline in the program store ("ppatmsstore.h") is
!   determined solely by the date/time of the scanline number, relative to
!   that of the first line in store.  Thus the routine can cope with missing
!   scanlines, but can potentially be upset if the first line in store has a
!   corrupt date/time.
!
! Note:
!   "atmsLastLine" should be initialised as 0 before first calling this routine.
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0   16/08/2010  Created from ppaget.F
!  
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! block of ATMS data
#include "ppatmsstore.h" 
! ATMS level 1C header/data common blocks
#include "atms1c.h"
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"    
!
!   Scalar arguments with intent(In):
      INTEGER       atms1Cunit       ! logical unit no. for ATMS 1C input
      INTEGER       atmsLastLine     ! number of last line stored in store
      ! Date/Time data block is to start from:
      INTEGER       Year1         !     year (e.g. 1995)
      INTEGER       Day1          !     day of year
      INTEGER       Time1         !     time (milliseconds)
      ! Date/Time data block is to end:
      INTEGER       Year2         !     year (e.g. 1995)
      INTEGER       Day2          !     day of year
      INTEGER       Time2         !     time (milliseconds)
!
! Local scalars:
      CHARACTER*80  errMessage    ! for calls to ErrorReport
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      INTEGER       border        ! for overlaps of data blocks (millisecs)
      INTEGER       dTime         ! time difference (milliseconds) from TimeSub
      INTEGER       errTime       ! error status from TimeSub
      INTEGER       atms1Crecord  ! record number for ATMS level 1C file
      INTEGER       fov           ! loop variable over all fov's
      INTEGER       line          ! line number within data store
      INTEGER       no_data       ! bit word set to "missing/bad data"
      INTEGER       scnlin0       ! scanline number of "zeroth" line in store
      INTEGER       scnlinDT      ! scanline number derived from date/time
      INTEGER       scnlin        ! scanline number
      INTEGER       year          ! year number of scanline
      INTEGER       day           ! day number of scanline
      INTEGER       time          ! time of scanline (ms)
      INTEGER       scnlinOld     ! previous scanline number
      INTEGER       yearOld       ! year number of previous scanline
      INTEGER       dayOld        ! day number of previous scanline
      INTEGER       timeOld       ! time of previous scanline (ms)
      INTEGER       scnlinOffset  ! difference between scnlin and scnlinDT
!
! Function & Subroutine calls:
      External TimeSub              ! Subtract two dates/times
      External IOAT1C               ! read ATMS 1C scan line
      External PPATMSIND            ! store scanline data in common/ppatmsstore/
      External ErrorReport          ! ATOVPP error routine
!
!- End of header


! 1.  Look for first suitable scanline in previous block
!     ---- --- ----- -------- -------- -- -------- -----

      scnlinOffset = 0
      line = atmsLastLine + 1
      dTime   = -999
      border = NINT(atmscycle*6.0)  ! ensure sufficient overlap with cris block

!     (stop looking once scanlines in block are earlier than start time)
      DO WHILE (line.GT.1 .AND. dTime.LT.border)

         line = line - 1
         errTime = 1
         IF ( atmsrecdno(line).NE.MDI ) THEN    ! valid line
            CALL TimeSub ( Year1,       Day1,       Time1,
     &                  atmsyear(line), atmsday(line), atmstime(1,line),
     &                  dTime, errTime)  ! dTime = start time - time of scan
!         errTime returned as non-zero if input is out of range (e.g. missing)
         ENDIF
         IF (errTime.NE.0) dTime = -999   ! go on looking

      ENDDO


! 2.  Define record number to start reading from
!     ------ ------ ------ -- ----- ------- ----

      IF ( dTime.GE.0 ) THEN
!        we've already read suitable data  - get record number from that
         atms1Crecord = atmsrecdno(line)
      ELSE
!        no suitable scanlines already in block  - try next record
         IF ( atmsLastLine.GT.0 ) THEN
            atms1Crecord = atmsrecdno(atmsLastLine) + 1
         ELSE  ! this is the first data to be read
            atms1Crecord = atmshdrcnt + 1  !first data record in 1C file
         ENDIF
      ENDIF


! 3.  Set data store to all "missing"
!     --- ---- ----- -- ---  -------

!     Throughout this program, atmsrecdno() should be used to check
!     whether a scanline is present or not, and atmsqc(bits(0))
!     to check whether the data is usable.

      no_data = 0
      no_data = IBSET ( no_data, bits(0) )

      DO line = 1, atmsnlines
         atmsrecdno(line) = MDI
         DO fov = 1, Anfovs
            atmsqc(fov,line) = no_data
         ENDDO  ! fov
      ENDDO  ! line


! 4.  Read data till we come to end of time interval, or of dataset
!     ---- ---- ---- -- ---- -- --- -- ---- --------  -- -- -------

      atmsLastLine = 0  ! i.e. we have no data in storage
      errTime = 1
      dTime   = -999
      scnlin0 = MDI  ! scanline number of "zeroth" scanline in store

! 4.1 Loop through scanlines in dataset
      DO WHILE ( atms1Crecord.LE.(atmshdrcnt+atmsscnlin) .AND.
     &   dTime.LT.border )

!        Read a scan line
         CALL IOAT1C ("READ DATA", atms1Cunit, atms1Crecord)
         scnlin = atms1c_scnlin
         year = atms1c_scnlinyr
         day  = atms1c_scnlindy
         time = atms1c_scnlintime(1)

        IF ( atms1c_scnlin.GE.1 .AND. atms1c_scnlin.LE.9999
     &    .AND. time .GE. 0
     &    .AND. .NOT. BTEST(atms1c_scanqual,bits(23)) ) THEN    ! if scanline OK

! 4.2    Calculate the scanline number from the date/time (to check)
         IF ( scnlin0 .NE. MDI ) THEN   ! not the 1st record read this call

            CALL TimeSub ( year, day, time,
     &                     yearOld,dayOld, timeOld, dTime, errTime )
            errMessage = " "
            CALL ErrorReport(errTime,errMessage,"PPATMSGET")

!           estimated scanline number
            scnlinDT = scnlinOld + NINT(REAL(dTime)/atmscycle)

!           Complain if scanline number and estimated scanline number differ
            IF ( scnlinDT.NE.scnlin+scnlinOffset ) THEN
               errMessage = " "
               WRITE ( errMessage,
     &      "(""Scanline numbers and times inconsistent, lines"",3I9)")
     &          scnlinOld, scnlin, scnlinDT
               errStatus = -42  ! warning
               CALL ErrorReport(errStatus,errMessage,"PPATMSGET")
               scnlinOffset = scnlinDT - scnlin
            ENDIF

         ENDIF  ! not the 1st record read

! 4.3    Store scanline in slot "line" in common/ppatmsstore/

!        Calculate slot to put line into
         IF ( scnlin0 .EQ. MDI ) THEN   ! first record to be read this call
            scnlin0 = scnlin - 1
            line = 1
         ELSE
            line = scnlinDT - scnlin0   ! use estimated line number, not scnlin
         ENDIF

         IF ( line.LE.atmsnlines ) THEN  ! there's room
            atmsLastLine = line
            CALL PPATMSIND ( line )
            atmsrecdno(line) = atms1Crecord
         ENDIF

! 4.4    save line-number/date/time to check against next scanline
         IF ( line .EQ. 1 ) THEN
           scnlinOld = scnlin
         ELSE
           scnlinOld = scnlinDT
         ENDIF
         yearOld = year
         dayOld  = day
         timeOld = time

! 4.5    Calculate dTime := date/time of scan - end date/time of block
         CALL TimeSub ( year, day, time,
     &                  Year2,Day2,Time2,  dTime, errTime )
!        errTime returned as non-zero if input is out of range (e.g. missing)
         IF (errTime.NE.0) dTime = -999   ! go on looking

        ENDIF    ! if scanline OK

!       Increment record number for next read
        atms1Crecord = atms1Crecord + 1

      ENDDO   ! WHILE ( atms1Crecord.LE.(atmshdrcnt+atmsscnlin) .AND. dTime.LT.border )

      WRITE(*,"(10X,""ATMS line numbers "",I4,"" to "",I4,
     &        "", with "",I3,"" missing lines"")")
!     &        atmslineno(1), atmslineno(atmsLastLine),
     &        atmslineno(1), scnlinOld,
     &       (scnlinOld-atmslineno(1))-(atmsLastLine-1)


      RETURN
      END
