!+ Transfer level 1C ATMS data from data record to program store
!
      SUBROUTINE PPATMSIND ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatmsind.F 317 2012-07-03 10:38:38Z frna $
!
! Purpose:
!   ATMS 1C data, when read, is held in common/atms1cdt/. ATOVPP stores data
!   from several scanlines in its own internal array structure, contained
!   in common/ppastore/. This routine transfers a single scanline from
!   common/atms1cdt/ to slot "Line" in common/ppastore/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1C include file (atms1c.h) and ATMS store
!     (ppastore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Copy variables from common/atms1cdt/ to slot "Line" in common/ppatmsstore/
! 2.  Check flags
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0    16/08/2010  Created from ppaind.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"   
! general parameters for ATOVPP
#include "ppparms.h"         
! block of ATMS data
#include "ppatmsstore.h"       
! ATMS level 1C header/data common blocks
#include "atms1c.h"          
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppastore/
!
! Local parameters:
      INTEGER       atms1Cvnnb    ! version number of 1C format compatible
         PARAMETER (atms1Cvnnb=1) ! with the code in this subroutine
      INTEGER       atmsstvnnb    ! version number of "ppastore.h" compatible
         PARAMETER (atmsstvnnb=1)    ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chanbits      ! bit word with all channel bits set
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       qcmask        ! bitword picks out selected bits
      LOGICAL       badline       ! .TRUE. not to use a scanline
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first, chanbits
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!        Is the 1C include file compatible with this source code ?
         IF ( atms1Cvnnb .NE. atms_1cvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but atms1c.h is "",I6)")
     &           atms1Cvnnb, atms_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPATMSIND")
         ENDIF

!        Is the ppastore include file compatible with this source code ?
         IF ( atmsstvnnb .NE. atms_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects atmsstore format"",I6,"" but " //
     &        "ppatmsstore.h is"",I6)") atmsstvnnb, atms_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPATMSIND")
         ENDIF

!        set up a mask to pick out just the flags for the individual channels
!        (modify here if any channels are bad)

         chanbits = 0

         DO chn = 1, atmsnchan
            chanbits = IBSET ( chanbits, bits(chn) )
         ENDDO

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from common/atms1cdt/ to slot "Line" in common/ppatmsstore/
!     ---- --------- ---- --------------- -- ----  ----  -- ----------------

      atmslineno(Line) = atms1c_scnlin                ! scanline number
      atmsyear(Line) = atms1c_scnlinyr                ! scanline year
      atmsday(Line) = atms1c_scnlindy                 ! scanline day of year
      atmsgranulequal(Line) = atms1c_granulequal      ! granule quality bit field
      atmsscanqual(Line) = atms1c_scanqual            ! scan line quality flags
      atmsgeolocqual(Line) = atms1c_geolocqual        ! geolocation quality flags
      DO chn = 1, atms_1cnchan
        atmschanqual(chn,Line) = atms1c_chanqual(chn)      ! channel quality flags
        atmsnedtcold(chn,Line) =  atms1c_nedtcold(chn)*0.0001
        atmsnedtwarm(chn,Line) =  atms1c_nedtwarm(chn)*0.0001
      ENDDO
      
      DO fov = 1, atmsnfovs
         atmstime(fov,Line) =  atms1c_scnlintime (fov)         ! time (ms)
         atmslats(fov,Line) = REAL(atms1c_latlon(1,fov))*1.E-4 !latitude(deg,N+ve)
         atmslons(fov,Line) = REAL(atms1c_latlon(2,fov))*1.E-4 !longitude (E+ve)
         atmsangles(1,fov,Line) = REAL(atms1c_angles(1,fov))*0.01 !local zenith
         atmsangles(2,fov,Line) = REAL(atms1c_angles(2,fov))*0.01 !local azimith
         atmsangles(3,fov,Line) = REAL(atms1c_angles(3,fov))*0.01 !solar zenith
         atmsangles(4,fov,Line) = REAL(atms1c_angles(4,fov))*0.01 !solar azimith
      ENDDO  ! fov

      atmsaltitude(Line) =  atms1c_scalti         ! satellite altitude (km*10)
                                                  ! above reference ellipsoid

      DO fov = 1, atmsnfovs
      DO chn = 1, atmsnchan
         atmsbtemps(chn,fov,Line) = REAL(atms1c_btemps(chn,fov))*0.01 ! BTs (K)
      ENDDO  ! chn
      ENDDO  ! fov

!     ATOVPP QC word  (not present in input 1C, so set to zero)
      DO fov = 1, atmsnfovs
         atmsqc(fov,Line) = 0
      ENDDO  ! fov



! 2.  Check flags
!     ----- -----

! Don't check flags for now, until we know which ones need to be acted upon

      RETURN
      END
