!+ Read & interpret ATMS level 1C header within ATOVPP
!
      SUBROUTINE PPATMSINH ( atms1Cunit, SatId, WMOSatId,
     &                    StartYearatms, StartDayatms, StartTimeatms,
     &                    EndYearatms,   EndDayatms,   EndTimeatms )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatmsinh.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Open input dataset and read level 1C ATMS header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     ATMS level 1C header has already been read into common /atms1chd/
!     Check that format of 1C include files (atms1c.h) is compatible with the
!     code in this subroutine. Check that format of 1C file is compatible with
!     this subroutine, and that data type from the header is 10 for ATMS.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from ATMS 1C header
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0   16/08/2010  Created from ppainh.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! block of ATMS data 
#include "ppatmsstore.h"     
! ATMS level 1C header/data common blocks
#include "atms1c.h"           
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
!
! Local parameters:
      INTEGER       atms1Cvnnb       ! version number of 1C format compatible
         PARAMETER (atms1Cvnnb=1)  ! with the code in this subroutine
!
!     Alternative way of checking that version numbers match (at compile time)
!      REAL VNTEST1(atms1Cvnnb-atms_1cvnnb+1), VNTEST2(atms_1cvnnb-atms1Cvnnb+1)
!
!   Scalar arguments with intent(In):
      INTEGER       atms1Cunit       ! logical unit no. for ATMS 1C input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 3 for METOP-1)
      ! start/end date/time of ATMS data:
      INTEGER       StartYearatms, EndYearatms
      INTEGER       StartDayatms,  EndDayatms
      INTEGER       StartTimeatms, EndTimeatms
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       atms1Crecord  ! record number for ATMS level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOAT1C               ! I/O routine for level 1C ATMS data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.1 Open 1C file
      CALL IOAT1C ("OPEN/READ", atms1Cunit, atms1Crecord )   ! 1C file

! 1.2 Read 1C header
      atms1Crecord = 1   ! Headers start on record 1.
      CALL IOAT1C ("READ HEADER", atms1Cunit, atms1Crecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( atms1Cvnnb .NE. atms_1cvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but atms1c.h is "",I6)")
     &        atms1Cvnnb, atms_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPATMSINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "atms1c.h" contains its own version number "atms_h_l1cversnb"
      IF ( atms1c_h_versnb .NE. atms_1cvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level 1C ATMS file is "",I8,"", not "",I8)")
     &        atms1c_h_versnb,atms_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPATMSINH")
      ENDIF

! 2.3 Is the 1C data ATMS ?
!     -- --- -- ---- ------
!     "atms1c_h_instrument" from 1C header should be 621 for ATMS
      IF ( atms1c_h_instrument .NE. 621 ) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage, "("" Data type in 1C ATMS file is "",
     &           I8,""; not 621 for ATMS"")") atms1c_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "PPATMSINH")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "##########"
      WRITE(*,*) "#  ATMS  #"
      WRITE(*,*) "##########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            atms1c_h_versnb, atms1c_h_versyr, atms1c_h_versdy
      WRITE(*,"(I4,"" header records in file."")") atms1c_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") atms1c_h_satid

!     calculate start and end dates/times from header
!     ..start
      year = atms1c_h_startdatayr
      yrday = atms1c_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(atms1c_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = atms1c_h_enddatayr
      yrday = atms1c_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(atms1c_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          atms1c_h_scnlin, atms1c_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from ATMS 1C header
!     ------- -------- ----------- ---- ------ -- ------

      atmsSatId = atms1c_h_satid
      SatId = atmsSatId
      WMOSatId = atms1c_h_wmosatid
      StartDayatms = atms1c_h_startdatady
      StartYearatms = atms1c_h_startdatayr
      StartTimeatms = atms1c_h_startdatatime
      EndDayatms = atms1c_h_enddatady
      EndYearatms = atms1c_h_enddatayr
      EndTimeatms = atms1c_h_enddatatime
      atmssatht = REAL(atms1c_h_satht)*0.1      !nominal sat height (km)
      atmssatperiod = REAL(atms1c_h_period)/60. !nominal orbit period (minutes)
      atmshdrcnt = atms1c_h_hdrcnt       !count of header records in 1C file
      atmsscnlin = atms1c_h_scnlin       !count of scanlines in 1C file
      atmsinstruments = IBSET ( atmsinstruments , bits(7) )   ! ATMS data present


      RETURN
      END
