!+ Look for precipitation signal in ATMS
!
      SUBROUTINE PPATMSPCP ( atmsLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatmspcp.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Look for precipitation signal in ATMS and set flags accordingly.
!
! Method:
! 0.  Loop through scanlines and fields-of-view
! 1.  Scattering test
! 2.  Cirrus test
! 3.  Crosby, Ferraro & Wu test
! 4.  Grody light rainfall test
! 5.  NWC SAF test
! 6.  Set flags
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    02/09/2010  From ppapcp.F and ppamsub2.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! fixed data for processing ATMS
#include "ppatmsfdata.h"       
! block of ATMS data
#include "ppatmsstore.h"        
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
!
!   Scalar arguments with intent(In):
      INTEGER       atmsLastLine     ! number of last ATMS line in store
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      REAL          T1            ! brightness temp in ATMS channel 1 (K)
      REAL          T2            ! brightness temp in ATMS channel 2 (K)
      REAL          T3            ! brightness temp in ATMS channel 3 (K)
      REAL          T16           ! brightness temp in ATMS channel 16 (K)
      REAL          T17           ! brightness temp in ATMS channel 17 (K)
      REAL          T18           ! brightness temp in ATMS channel 18 (K)
      REAL          zen           ! zenith angle
      REAL          SI            ! scattering index
      REAL          CI            ! cirrus index
      REAL          probprecip    ! Crosby probability of precipitation
      LOGICAL       scat          ! .TRUE. if scattering detected
      LOGICAL       cirrus        ! .TRUE. if cirrus detected
      LOGICAL       crosby        ! .TRUE. if Crosby test flags fov
      LOGICAL       grody         ! .TRUE. if Grody test flags fov
      REAL          m0,m1,m2,m3   ! Precipitation probabilities
      INTEGER       si_flag       ! Scattering index flag
      INTEGER       lsma          ! Land fraction (percent) AMSU-A-like
      INTEGER       lsmb          ! Land fraction (percent) AMSU-B-like
      REAL          flagb         ! Used by retrieve_one_si
!
! Local arrays:
      REAL          avdiff_89_166(atmsnfovs, atmsnlines)   !89-166 difference (K)
!
! Function & Subroutine calls:
      External PPATMSSCAT            ! scattering index
          REAL PPATMSSCAT
      External PPATMSCIRR            ! cirrus index
          REAL PPATMSCIRR
      External PPATMSCROSBY           ! logistic regression
          REAL PPATMSCROSBY
      External PPATMSGRODY            ! light rainfall test
       LOGICAL PPATMSGRODY
!
!- End of header

      atmspcpvn = 1000   ! precip/scattering detection version number
                      ! 0 means no detection was done
                      ! 1000-1999 intended for UKMO precip schemes
                      ! 9000+ for any test versions

!     Create average T89-T166 field for use with the NWC-SAF test

      CALL average_T89_T166_atms(atmsLastLine, 27, avdiff_89_166)  !Box of 27x27 spots

! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, atmsLastLine
      DO fov = 1, atmsnfovs

         T1  = atmsbtemps(1,fov,line)
         T2  = atmsbtemps(2,fov,line)
         T3  = atmsbtemps(3,fov,line)
         T16 = atmsbtemps(16,fov,line)
         T17 = atmsbtemps(17,fov,line)
         T18 = atmsbtemps(18,fov,line)
         zen = atmsangles(1,fov,line)
         lsmb = atmsLandFracb(fov,line)    !land frac for AMSU-B-like channels
         lsma = atmsLandFraca(fov,line)    !land frac for AMSU-A-like channels


! 1.     Scattering test  (only over the sea)
!        ---------- ----

!        only do if channels present
         IF (  T1 .GT. 0.  .AND.  T2 .GT. 0.
     &   .AND. T3 .GT. 0.  .AND.  T16 .GT. 0. ) THEN
            SI = PPATMSSCAT ( T1, T2, T3, T16, fov )
            atmsscatindx(fov,line) = SI
            IF ( atmssurftype(fov,line).EQ.0 ) THEN
               scat = SI .GT. atmsmaxscat
            ELSE
               scat = .FALSE.
            ENDIF
         ELSE
            atmsscatindx(fov,line) = RMDI
            scat = .TRUE.  ! default is to err on side of caution
         ENDIF

! 2.     Cirrus test  (only over the sea)
!        ------ ----

!        only do if channels present
         IF (  T1 .GT. RMDI  .AND.  T16 .GT. RMDI
     &   .AND. T17 .GT. RMDI  .AND.  T18 .GT. RMDI ) THEN
            CI = PPATMSCIRR ( T1, T16, T17, T18, fov )
            atmscirrindx(fov,line) = CI
            IF ( atmssurftype(fov,line).EQ.0 ) THEN
               cirrus = CI .GT. atmsmaxcirr
            ELSE
               cirrus = .FALSE.
            ENDIF
         ELSE
            atmscirrindx(fov,line) = RMDI
            cirrus = .TRUE.  ! default is to err on side of caution
         ENDIF

! 3.     Crosby, Ferraro & Wu test
!        ------  ------- - -- ----

         IF (  T1 .GT. 0.  .AND.  T16 .GT. 0. ) THEN
            probprecip = PPATMSCROSBY(T1,T16 )
            atmscrosby(fov,line) = probprecip
            crosby = probprecip .GT. atmsmaxcrosby
         ELSE
            atmscrosby(fov,line) = RMDI
            crosby = .TRUE.  ! default is to err on side of caution
         ENDIF


! 4.     Grody light rainfall test
!        ----- ----- -------- ----

         IF (  T1 .GT. 0.  .AND.  T2 .GT. 0. ) THEN
            grody = PPATMSGRODY( T1, T2 )
         ELSE
            grody = .TRUE.  ! default is to err on side of caution
         ENDIF


! 5.     NWC-SAF scattering and precipitation test (from Ralf Bennartz)
!        -------------------------------------------------------------- 

         CALL retrieve_one_si(T1,T16,T17,zen,flagb,lsma,lsmb,
     &                 avdiff_89_166(fov,line),SI,m0,m1,m2,m3,si_flag)
         atmsm0(fov,line) = m0
         atmsm1(fov,line) = m1
         atmsm2(fov,line) = m2
         atmsm3(fov,line) = m3
         atmssi_nwcsaf(fov,line) = SI
         atmssi_nwcsaf_flag(fov,line) = si_flag


! 6.     Set flags
!        --- -----

         IF ( scat )   atmsqc(fov,line) = 
     &                    IBSET ( atmsqc(fov,line), bits(26) )
         IF ( cirrus ) atmsqc(fov,line) = 
     &                    IBSET ( atmsqc(fov,line), bits(17) )
         IF ( crosby ) atmsqc(fov,line) =
     &                    IBSET ( atmsqc(fov,line), bits(25) )
         IF ( grody )  atmsqc(fov,line) =
     &                    IBSET ( atmsqc(fov,line), bits(24) )

!        set flag for use by the mapping routines:
         IF ( scat .OR. crosby )
     &          atmsqc(fov,line) = IBSET ( atmsqc(fov,line), bits(31) )

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
