!+ Transfer data from program store to level 1D ATMS data record
!
      SUBROUTINE PPATOUTD ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatoutd.F 339 2012-11-26 13:30:05Z frna $
!
! Purpose:
!   ATOVPP stores data from several scanlines in its own internal array
!   structure, contained in common/ppatmsstore/. This routine transfers a single
!   scanline from slot "Line" in common/ppatmsstore/ to common/atms1ddt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1D include file (atms1d.h) and ATMS store
!     (ppatmsstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
!     Fill parts of 1D data record that won't change each scanline
! 1.  Copy variables from slot "Line" in common/ppatmsstore/ to common/atms1ddt/
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   03/09/2010 Created from ppaoutd.F (N.C.Atkinson)
!  1.1   03/07/2012 Set atms1d_nedtwarm and atms1d_nedtcold
!  1.2   26/11/2012 Set BT to missing if lunar contamination
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! block of ATMS data
#include "ppatmsstore.h"       
! ATMS fixed data
#include "ppatmsfdata.h"       
! ATMS level 1D header/data common blocks
#include "atms1d.h"          
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppatmsstore/
!
! Local parameters:
      INTEGER       atms1Dvnnb     ! version number of 1D format compatible
         PARAMETER (atms1Dvnnb=1)  ! with the code in this subroutine
      INTEGER       atmsstvnnb     ! version number of "ppatmsstore.h" compatible
         PARAMETER (atmsstvnnb=1)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov           ! loop variable over all fovs
      LOGICAL       badfov        ! indicates 1 or more bad BTs
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first                    ! .TRUE. on first call
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

! 0.1    Is the 1D include file compatible with this source code ?
         IF ( atms1Dvnnb .NE. atms_1dvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1D format"",I6,"" but atms1d.h is "",I6)")
     &           atms1Dvnnb, atms_1dvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPATOUTD")
         ENDIF

! 0.2    Is the ppatmsstore include file compatible with this source code ?
         IF ( atmsstvnnb .NE. atms_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects atmsstore format"",I6,"" but "//
     &      "ppatmsstore.h is"",I6)") atmsstvnnb, atms_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPATOUTD")
         ENDIF

! 0.3    Reset "first" for subsequent calls
         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from slot "Line" in common/pphstore/ to common/atms1ddt/
!     ---- --------- ---- ----  ----  -- ---------------- -- ---------------

      atms1d_scnlin = atmslineno(Line)                ! scanline number
      atms1d_scnlinyr = atmsyear(Line)                ! scanline year
      atms1d_scnlindy = atmsday(Line)                 ! scanline day of year
      atms1d_granulequal = atmsgranulequal(Line)
      atms1d_scanqual = atmsscanqual(Line)
      atms1d_geolocqual = atmsgeolocqual(Line)
      DO chn = 1, atmsnchan
        atms1d_chanqual(chn) = atmschanqual(chn,Line)
        atms1d_nedtcold(chn) =  atmsnedtcold(chn,Line)*10000
        atms1d_nedtwarm(chn) =  atmsnedtwarm(chn,Line)*10000
      ENDDO

      DO fov = 1, atmsnfovs
         atms1d_scnlintime(fov) = atmstime(fov,Line)          ! scanline time (ms)
         atms1d_latlon(1,fov) = NINT(atmslats(fov,Line)*1.E4) !latitude (N +ve)
         atms1d_latlon(2,fov) = NINT(atmslons(fov,Line)*1.E4) !longitude (E +ve)
         atms1d_surface(1,fov) = atmselev(fov,Line)           !surface height (m)
         atms1d_surface(2,fov) = atmssurftype(fov,Line)       !surface type
         atms1d_landfrac(1,fov) = atmsLandFraca(fov,Line)     !land frac, AMSU-A-like channels
         atms1d_landfrac(2,fov) = atmsLandFracb(fov,Line)     !land frac, AMSU-B-like channels
         DO i = 1, 4
            atms1d_angles(i,fov) = NINT(atmsangles(i,fov,Line)*100.) ! scan angles:
!                                         (1,*,*) local zenith angle (deg*100)
!                                         (2,*,*) local azimuth angle (deg*100)
!                                         (3,*,*) solar zenith angle (deg*100)
!                                         (4,*,*) solar azimuth angle (deg*100)
         ENDDO  ! i
      ENDDO  ! fov

      atms1d_scalti = atmsaltitude(Line)              ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
!     ATMS BTs (deg K * 100) with QC flag
      DO fov = 1, atmsnfovs
        badfov = .FALSE.
        atms1d_atmsqual(FOV) = 0
        DO chn = 1, atmsnchan

!If spot is lunar-contaminated, set the BT to missing
          IF (BTEST(atmschanqual(chn,Line),BITS(9))) THEN
            atmsbtemps(chn,fov,Line) = RMDI
          ENDIF

          atms1d_btemps(chn,fov) = NINT(atmsbtemps(chn,fov,Line)*100.)
          IF (atms_ignore_channel(chn)) THEN
            atms1d_atmsqual(FOV) = 
     &         IBSET(atms1d_atmsqual(FOV),bits(chn+3))
          ELSE IF (atmsbtemps(chn,fov,Line).LT.atmsminBT .OR.
     &             atmsbtemps(chn,fov,Line).GT.atmsmaxBT) THEN
            badfov = .TRUE.
            atms1d_atmsqual(FOV) = 
     &         IBSET(atms1d_atmsqual(FOV),bits(chn+3))
          ENDIF
        ENDDO  ! chn
        IF (badfov) THEN
          atms1d_atmsqual(FOV) = IBSET(atms1d_atmsqual(FOV),bits(0))
        ENDIF
      ENDDO  ! fov

![user-defined:
      DO fov = 1, atmsnfovs
         atms1d_prepro(1,fov) = atmsqc(fov,Line)       ! ATOVPP QC word for AMSU
         atms1d_prepro(2,fov) = atmsppsurf(fov,Line)   ! estimated surface type
         atms1d_prepro(3,fov) = NINT(atmssurfcost(fov,Line)*100.) ! max prob cloud
         atms1d_prepro(4,fov) = NINT(atmsscatindx(fov,Line)*100.) ! scattering index
         atms1d_prepro(5,fov) = NINT(atmscirrindx(fov,Line)*100.) ! cirrus index
         atms1d_prepro(6,fov) = NINT(atmscrosby(fov,Line)*100.)   ! logistic precip
         IF (atmssi_nwcsaf(fov,Line).GT.-999.0) THEN
           atms1d_prepro(7,fov) = NINT(atmssi_nwcsaf(fov,Line)*100) ! Bennartz index
         ELSE
           atms1d_prepro(7,fov) = MDI
         ENDIF
         
         ! NWC-SAF Precip probabilities (each in range 0 to 15)
         i =  MIN(MAX(NINT(atmsm0(fov,Line)*15.0), 0), 15)        
         atms1d_prepro(8,fov) = i                                 !Bits 0 to 3 (none)
         i =  MIN(MAX(NINT(atmsm1(fov,Line)*15.0), 0), 15)        
         atms1d_prepro(8,fov) = IOR(atms1d_prepro(7,fov), i*16)   !Bits 4 to 7 (light)
         i =  MIN(MAX(NINT(atmsm2(fov,Line)*15.0), 0), 15)        
         atms1d_prepro(8,fov) = IOR(atms1d_prepro(7,fov), i*256)  !Bits 8 to 11 (moderate)
         i =  MIN(MAX(NINT(atmsm3(fov,Line)*15.0), 0), 15)        
         atms1d_prepro(8,fov) = IOR(atms1d_prepro(7,fov), i*4096) !Bits 12 to 15 (intense)

      ENDDO  ! fov
! ... ]


      RETURN
      END
