!+ Set up ATMS level 1D header within ATOVPP
!
      SUBROUTINE PPATOUTH
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppatouth.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Set up level 1D header using information from 1C headers.
!
! Method:
! 1.  Check format version number and data type
!     AMSU-A level 1C header has already been read into common /ama1chd/
!     Check that format of 1C & 1D include files (ama1c.h, atms1d.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1C file is compatible with this subroutine.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Set up level 1D header
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   16/08/2010  From ppaouth.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
! general parameters for ATOVPP
#include "ppparms.h"        
! parameters and look-up tables for the mapping
#include "ppluts.h"          
! fixed data for processing AMSU-A
#include "ppatmsfdata.h"      
! block of AMSU-A data 
#include "ppatmsstore.h"       
! ATMS level 1C header/data common blocks
#include "atms1c.h"          
! ATMS level 1D header/data common blocks
#include "atms1d.h"           
!
! Local parameters:
      INTEGER       AT1Cvnnb     ! version number ATMS 1C format compatible
         PARAMETER (AT1Cvnnb=1)  !    with the code in this subroutine
      INTEGER       AT1Dvnnb     ! version number 1D format compatible
         PARAMETER (AT1Dvnnb=1)  !    with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! general purpose loop variables
      INTEGER       chn           ! loop variable over channels
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 1.1 Are the 1C include files compatible with this source code ?
!     --- --- -- ------- ----- ---------- ---- ---- ------ ----

!     ATMS
      IF ( AT1Cvnnb .NE. atms_1cvnnb ) THEN
         errStatus = -111  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but atms1c.h is "",I6)")
     &        AT1Cvnnb, atms_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPATOUTH")
      ENDIF

! 1.2 Is the 1D include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( AT1Dvnnb .NE. atms_1dvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1D format "",I6,"" but atms1d.h is "",I6)")
     &        AT1Dvnnb, atms_1dvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPATOUTH")
      ENDIF


! 2.  Set up level 1D header
!     --- -- ----- -- ------

! 2.1 General

!     creation site for 1D data
      atms1d_h_site   = SiteID              !file creation site ID (CHAR*3)
      atms1d_h_cfill1 = " "                 !filler

!     creation site for original 1B data
      atms1d_h_1bsite = atms1c_h_1bsite
      atms1d_h_cfill2 = " "                 !filler

      atms1d_h_versnb = atms_1dvnnb          !level 1d format version number
      atms1d_h_versyr = atms_1dvnyr          !level 1d format version year
      atms1d_h_versdy = atms_1dvndy          !level 1d format version day of year
      atms1d_h_hdrcnt = atms1c_h_hdrcnt      !count of header records
      atms1d_h_satid = atms1c_h_satid        !satellite id (e.g. 14 for NOAA-14)
      atms1d_h_grid = atms1c_h_instrument    !instrument grid code (AMSU-A=10)
      atms1d_h_satht = atms1c_h_satht        !nominal sat height, km*10
      atms1d_h_period = atms1c_h_period      !nominal orbit period (seconds)
      atms1d_h_startorbit = atms1c_h_startorbit !orbit number (at start)
      atms1d_h_startdatayr = atms1c_h_startdatayr!start of data set year
      atms1d_h_startdatady = atms1c_h_startdatady! (ditto) day of the year
      atms1d_h_startdatatime = atms1c_h_startdatatime !(ditto) UTC time of day,ms
      atms1d_h_endorbit = atms1c_h_endorbit  !orbit number (at end)
      atms1d_h_enddatayr = atms1c_h_enddatayr!end of data set year
      atms1d_h_enddatady = atms1c_h_enddatady!(ditto) day of the year
      atms1d_h_enddatatime = atms1c_h_enddatatime !(ditto) UTC time of day,ms
      atms1d_h_scnlin = atms1c_h_scnlin      !count of scan lines in this dataset
      atms1d_h_misscnlin = atms1c_h_misscnlin !count of missing scan lines
      atms1d_h_atovppvn = ATOVPPvn     !ATOVPP version number (test vns = 9000+)
      atms1d_h_instruments = atmsinstruments   !instruments present
      atms1d_h_Aantenna = atms1c_h_vnantennacorr ! vn AMSU-A antenna corrections
      atms1d_h_wmosatid = atms1c_h_wmosatid      !WMO sat ID
      atms1d_h_centre = atms1c_h_centre          !WMO centre id
      atms1d_h_subcentre = atms1c_h_subcentre    !WMO subcentre id
      atms1d_h_recwords = array_size_atms1d
      atms1d_h_nchan = atms1c_h_nchan
      atms1d_h_nfov = atms1c_h_nfov
      atms1d_h_nchanused = atms1c_h_nchanused
      atms1d_h_wmosatid = atms1c_h_wmosatid
      atms1d_h_beamversion = atms1c_h_beamversion


! 2.2 temperature-radiance conversion coefficients, etc.

!     ATMS
!                  order of channels = 1,2,...,22
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
      DO chn = 1, atmsnchan
      atms1d_h_beamwidth(chn) = atms1c_h_beamwidth(chn)
      atms1d_h_cutoff(chn)    = atms1c_h_cutoff(chn)
      atms1d_h_nxaverage(chn) = atms1c_h_nxaverage(chn)
      atms1d_h_nyaverage(chn) = atms1c_h_nyaverage(chn)
      DO i = 1, 3
         atms1d_h_planck(i,chn) = atms1c_h_tempradcnv(i,chn)
      ENDDO  ! i
      atms1d_h_bandwidth(chn) = atms1c_h_bandwidth(chn)
      ENDDO  ! chn

! 2.3 Pre-processing information

![user-defined:
      atms1d_h_dT  = dT         ! size of data batches processed within ATOVPP
      atms1d_h_Acorrvn  = atmscorrvn        ! vn no AMSU-A limb/emiss correction
      atms1d_h_Apcpvn   = atmspcpvn         ! vn no AMSU-A precip/scat detection
      atms1d_h_Ascat = atmsmaxscat !threshold value for AMSU-A scattering index
      atms1d_h_Acirr = atmsmaxcirr !threshold value for cirrus index
      atms1d_h_ACrosby = atmsmaxcrosby !threshold value for AMSU-A logistic precip
! ... ]


      RETURN
      END
