!+ Map AMSU-B to an individual AMSU-A fov
!
      SUBROUTINE PPB2A ( TFOV, TLine,
     &                   Ncolocs, List_fov, List_line, List_wt )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppb2a.F 211 2010-04-26 12:34:57Z frna $
!
! Purpose:
!   Select mapping fovs from those given and derive brightness temperatures
!   and other parameters at the specified target fov.
!
! Method:
! 0.  Initialisation
! 1.  First try only fovs with primary calibration
! 2.  then if desperate accept those with secondary calibration
! 3.  check range of 89GHz channel over AMSU-A fov
! 4.  Finish setting the flags
! 5.  Calculate mapped BTs
!   Note that only those AMSU-B fovs with the same surface type as the AMSU-A
!   fov are mapped, unless the AMSU-A fov is of mixed type, in which case all
!   AMSU-B fovs are mapped.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Don't skip bad target fovs
!  1.2      10/03/98  Store instrument temp from nearest AMSU-B scanline
!  1.3      17/01/07  Get the 89GHz range check working: previously this flag
!                     was never set. Set bits 1-5 of qcmask, in order to to
!                     check all channels. NCA
!  1.4      30/03/10  Make sure we don't include missing BTs. NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h" 
! block of AMSU-A data
#include "ppastore.h"
! fixed data for processing AMSU-A   
#include "ppafdata.h"
! block of AMSU-B data
#include "ppbstore.h"
! bit numbers for intrinsic bit routines (eg BTEST)      
#include "bits.h"      
!
!   Arguments with intent(In):
      INTEGER       TFOV          ! target fov number
      INTEGER       TLine         ! number of target line in btemps array
      INTEGER       Ncolocs       ! no of colocated mapping FOVs
      INTEGER       List_line(*)  ! list of coloc line numbers (1st=nearest)
      INTEGER       List_fov(*)   !               spot numbers
      REAL          List_wt(*)    !               LUT weights
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       fov           ! mapping fov number
      INTEGER       line          ! number of mapping line in btemps array
      INTEGER       iflag         ! flagword
      INTEGER       icoloc        ! loop variable over mapping FOVs
      REAL          max89, min89  ! max/min value of 89GHz channel over fov
      INTEGER       ngood         ! no of fovs selected
      INTEGER       pp_surf       ! surface type from the nearest mapping fov
      INTEGER       pp_qc         ! pre-processor QC word
      INTEGER       qcmask        ! bitword picks out selected bits from Mqc
      LOGICAL       bad89         ! .TRUE. if max89-min89 exceeds threshold
      LOGICAL       mixed         ! .TRUE. if target fov has mixed surfaces
      REAL          sum_weights   ! sum of weights, used if weights need
                                  !   to be re-calculated (i.e. QC flags set)
      REAL          weight        ! used in the mapping
      REAL          wt_norm       ! reciprocal of sum_weights
!
! Local arrays:
      REAL          sum(40)       ! sum of weights*BTs for each channel
      LOGICAL       use_coloc(90) ! mapping FOVs: .TRUE. to use
!
!- End of header


! 0.  Initialisation
!     --------------

! 0.1 Initially, no mapping fovs have been selected
      ngood = 0
      DO icoloc = 1, Ncolocs
         use_coloc(icoloc) = .FALSE.
      ENDDO  ! icoloc
      pp_surf = MDI   ! surface type of nearest good fov
      pp_qc = 0
      pp_qc = IBSET(pp_qc,bits(28)) ! pre-processor QC flagword: AMSU-B missing


! 1.  First try only fovs with primary calibration
!     ----- --- ---- ---- ---- ------- -----------

      pp_qc = 0             ! pre-processor QC flagword
      ngood = 0             ! number of fovs chosen so far
      pp_surf = Asurftype(TFOV,TLine)     ! i.e. just the AMSU-A surface type
      mixed = pp_surf.EQ.1  ! mixed surface type
      qcmask = 0
      qcmask = IBSET ( qcmask, bits(0) )  ! bad/missing data
      qcmask = IBSET ( qcmask, bits(1) )
      qcmask = IBSET ( qcmask, bits(2) )
      qcmask = IBSET ( qcmask, bits(3) )
      qcmask = IBSET ( qcmask, bits(4) )
      qcmask = IBSET ( qcmask, bits(5) )
      qcmask = IBSET ( qcmask, bits(30) ) ! secondary calibration used

      DO icoloc = 1, Ncolocs
         fov  = List_fov(icoloc)
         line = List_line(icoloc)
!        iflag = Bqc(fov,line).AND.qcmask
         iflag = IAND(Bqc(fov,line),qcmask)
         IF ( iflag.EQ.0 .AND.
     &        (mixed .OR. pp_surf.EQ.Bsurftype(fov,line)) ) THEN
            use_coloc(icoloc) = .TRUE.
            ngood = ngood + 1
         ENDIF
      ENDDO  ! icoloc


! 2.  then if desperate accept those with secondary calibration
!     ---- -- --------- ------ ----- ---- --------- -----------

      IF ( ngood.EQ.0 ) THEN
!        set flag
         qcmask = IBCLR ( qcmask, bits(30) ) ! secondary calibration not used
         pp_qc = IBSET(pp_qc,bits(29))  ! secondary calibration flag
!        and try again
         DO icoloc = 1, Ncolocs
            fov  = List_fov(icoloc)
            line = List_line(icoloc)
!           iflag = Bqc(fov,line).AND.qcmask
            iflag = IAND(Bqc(fov,line),qcmask)
            IF ( iflag.EQ.0 .AND.
     &        (mixed .OR. pp_surf.EQ.Bsurftype(fov,line)) ) THEN
               use_coloc(icoloc) = .TRUE.
               ngood = ngood + 1
            ENDIF
         ENDDO  ! icoloc
      ENDIF  ! ngood


! 3.  check range of 89GHz channel over AMSU-A fov
!     ----- ----- -- ----- ------- ---- ------ ---

      max89 = 0.0
      min89 = 99999.0
      IF ( ngood.GT.0 ) THEN ! fovs found
         DO icoloc = 1, Ncolocs
            IF ( use_coloc(icoloc) ) THEN  ! fov ok
               fov  = List_fov(icoloc)
               line = List_line(icoloc)
               max89 = MAX ( max89, Bbtemps(1,fov,line) )
               min89 = MIN ( min89, Bbtemps(1,fov,line) )
            ENDIF
         ENDDO  ! icoloc
         bad89 = max89-min89 .GT. ABmax89
      ENDIF  ! ngood


! 4.  Finish setting the flags
!     ------ ------- --- -----

 400  CONTINUE  ! jump here if target fov bad

      IF ( ngood.EQ.0 ) THEN ! no fovs found
         pp_qc = IBCLR(pp_qc,bits(29))  ! secondary calibration flag
         pp_qc = IBSET(pp_qc,bits(28))  ! AMSU-B missing
      ELSE
         IF ( bad89 ) THEN
            pp_qc = IBSET(pp_qc,bits(23)) ! variance at 89GHz
            pp_qc = IBSET(pp_qc,bits(31)) ! warn the mapping
         ENDIF
      ENDIF  ! ngood

!     Aqc(TFOV,TLine) = Aqc(TFOV,TLine) .OR. pp_qc
      Aqc(TFOV,TLine) = IOR(Aqc(TFOV,TLine), pp_qc)

!     Store instrument temperature from nearest AMSU-B scanline
      Ainstrtempb(TLine)  = Binstrtemp(List_line(1))


! 5.  Calculate mapped BTs
!     --------- ------ ---

! 5.1 Count how many were selected
      ngood = 0
      DO icoloc = 1, ncolocs
         IF ( use_coloc(icoloc) ) ngood = ngood + 1
      ENDDO

      IF ( ngood .EQ. 0 ) THEN

! 5.2    No colocated mapping data selected  - set to missing
         DO chn = 1, Bnchan
            Abtemps(chn+Anchan,TFOV,TLine) = RMDI
         ENDDO  ! chn

      ELSE

! 5.3    initialise summation variables
         sum_weights = 0.
         DO chn = 1, Bnchan
            sum(chn)  = 0.
         ENDDO  ! chn

! 5.4    Do the mapping: sum weights*BTs for those FOVs selected
!        If any BT for a channel is missing, set the mapped BT to missing
         DO icoloc = 1, ncolocs
            IF ( use_coloc(icoloc) ) THEN
               fov  = List_fov(icoloc)
               line = List_line(icoloc)
               weight = List_wt(icoloc)
               sum_weights = sum_weights + weight
!              loop through the channels
               DO chn = 1, Bnchan
                 IF (Bbtemps(chn,fov,line) .NE. RMDI) THEN
                   IF (sum(chn) .NE. RMDI) THEN
                     sum(chn) = sum(chn) + weight*Bbtemps(chn,fov,line)
                   ENDIF
                 ELSE
                   sum(chn) = RMDI
                 ENDIF
               ENDDO  ! chn
            ENDIF  ! use_coloc(icoloc)
          ENDDO  ! icoloc

! 5.5     Loop through channels again, to calculate & store mapped BTs
          wt_norm = 1. / sum_weights
          DO chn = 1, Bnchan
             IF (sum(chn) .NE. RMDI) THEN
               Abtemps(chn+Anchan,TFOV,TLine) = sum(chn) * wt_norm
             ELSE
               Abtemps(chn+Anchan,TFOV,TLine) = RMDI
             ENDIF
          ENDDO  ! chn

       ENDIF                     ! ngood


      RETURN
      END
