!+ Transfer level 1C AMSU-B data from data record to program store
!
      SUBROUTINE PPBIND ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppbind.F 191 2010-01-27 13:42:50Z frna $
!
! Purpose:
!   AMSU-B 1C data, when read, is held in common/amb1cdt/. ATOVPP stores data
!   from several scanlines in its own internal array structure, contained
!   in common/ppbstore/. This routine transfers a single scanline from
!   common/amb1cdt/ to slot "Line" in common/ppbstore/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1C include file (amb1c.h) and AMSU-B store
!     (ppbstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Copy variables from common/amb1cdt/ to slot "Line" in common/ppbstore/
! 2.  Check flags
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1C format vn 1.8
!                     Set Bqc(bit0) if any channels are missing
!  1.2      10/03/98  Conform to 1C format vn 1.9
!                     - instrument temperature and solar angles added
!  1.3      29/06/98  Changes to Bqc
!  1.4      02/07/98  Bangles REAL not INTEGER
!  1.5      16/09/09  For NOAA-19, do not set Bqc(bit0) if channels 3/4 missing
!  1.6      27/01/10  For NOAA-15/16/17, do not set Bqc(bit0) if channels 3/4/5
!                     missing
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
! general parameters for ATOVPP
#include "ppparms.h"         
! block of AMSU-B data
#include "ppbstore.h"      
! AMSU-B level 1C header/data common blocks
#include "amb1c.h"          
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppbstore/
!
! Local parameters:
      INTEGER       B1Cvnnb       ! version number of 1C format compatible
         PARAMETER (B1Cvnnb=190)  ! with the code in this subroutine
      INTEGER       Bstvnnb       ! version number of "ppbstore.h" compatible
         PARAMETER (Bstvnnb=140)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chanbits      ! bit word with all channel bits set
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       qcmask        ! bitword picks out selected bits
      LOGICAL       badline       ! .TRUE. not to use a scanline
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first, chanbits
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!        Is the 1C include file compatible with this source code ?
         IF ( B1Cvnnb .NE. amb_1cvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but amb1c.h is "",I6)")
     &           B1Cvnnb, amb_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPBIND")
         ENDIF

!        Is the ppbstore include file compatible with this source code ?
         IF ( Bstvnnb .NE. amb_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects bstore format"",I6,"" but ppbstore.h is"",I6)")
     &           Bstvnnb, amb_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPBIND")
         ENDIF

!        set up a mask to pick out just the flags for the individual channels
         chanbits = 0

         IF ( amb1c_h_satid .EQ. 19 ) THEN
!
!  For NOAA-19, do not set bit for channels 3 or 4
!
           DO chn = 1, 2
             chanbits = IBSET ( chanbits, bits(chn) )
           ENDDO
           DO chn = 5, 5
             chanbits = IBSET ( chanbits, bits(chn) )
           ENDDO

         ELSE IF ( amb1c_h_satid.GE.15 .AND. amb1c_h_satid.LE.17 ) THEN
!
!  For NOAA-15/16/17, do not set bit for channels 3, 4 or 5
!
           DO chn = 1, 2
             chanbits = IBSET ( chanbits, bits(chn) )
           ENDDO

         ELSE
 
           DO chn = 1, Bnchan
             chanbits = IBSET ( chanbits, bits(chn) )
           ENDDO

         ENDIF

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from common/amb1cdt/ to slot "Line" in common/ppbstore/
!     ---- --------- ---- --------------- -- ----  ----  -- ----------------

      Blineno(Line) = amb1c_scnlin                ! scanline number
      Byear(Line) = amb1c_scnlinyr                ! scanline year
      Bday(Line) = amb1c_scnlindy                 ! scanline day of year
      Btime(Line) =  amb1c_scnlintime             ! scanline time (ms)
      Bqualind(Line) = amb1c_qualind              ! quality indicator bit field
      Bscanqual(Line) = amb1c_scanqual            ! scan line quality flags
      Binstrtemp(Line) = amb1c_instrtemp          ! AMSU-B Mixer chan 18-20 temp

      DO fov = 1, Bnfovs
         Blats(fov,Line) = REAL(amb1c_latlon(1,fov))*1.E-4 !latitude(deg,N+ve)
         Blons(fov,Line) = REAL(amb1c_latlon(2,fov))*1.E-4 !longitude (E+ve)
         Bangles(1,fov,Line) = REAL(amb1c_angles(1,fov))*0.01 !local zenith
         Bangles(2,fov,Line) = REAL(amb1c_angles(2,fov))*0.01 !local azimith
         Bangles(3,fov,Line) = REAL(amb1c_angles(3,fov))*0.01 !solar zenith
         Bangles(4,fov,Line) = REAL(amb1c_angles(4,fov))*0.01 !solar azimith
      ENDDO  ! fov

      Baltitude(Line) =  amb1c_scalti             ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
      DO fov = 1, Bnfovs
      DO chn = 1, Bnchan
         Bbtemps(chn,fov,Line) = REAL(amb1c_btemps(chn,fov))*0.01 ! BTs (K)
      ENDDO  ! chn
      DO chn = 1, Anchan
         BmapBT(chn,fov,Line) = RMDI
      ENDDO  ! chn
      ENDDO  ! fov

!     ATOVPP QC word  (..just take bits 0 & 30 from level 1c data quality word)
      qcmask = 0
      qcmask = IBSET ( qcmask, bits(0) )  ! bit 0  = 'data missing'
      qcmask = IBSET ( qcmask, bits(30) ) ! bit 30 = 'secondary calibration'
      DO fov = 1, Bnfovs
!        Bqc(fov,Line) = amb1c_dataqual(fov) .AND. qcmask  ! QC word for AMSU-B
         Bqc(fov,Line) = IAND(amb1c_dataqual(fov), qcmask)  ! QC word for AMSU-B
      ENDDO  ! fov

!     Set Bqc(bit0) if any channels are missing
      DO fov = 1, Bnfovs
!        IF ( (amb1c_dataqual(fov).AND.chanbits).NE.0 )
         IF ( IAND(amb1c_dataqual(fov), chanbits).NE.0 )
     &         Bqc(fov,Line) = IBSET(Bqc(fov,Line),bits(0))
      ENDDO  ! fov


! 2.  Check flags
!     ----- -----

! 2.1 Flags for the whole scanline

!     data quality indicator
      badline = BTEST( Bqualind(Line), bits(31) ) !do not use for products
     &    .OR.  BTEST( Bqualind(Line), bits(27) ) !no earth location

!     scanline quality indicator
      badline = badline
     &    .OR.  BTEST( Bscanqual(Line), bits(23) )!bad time field
     &    .OR.  BTEST( Bscanqual(Line), bits(22) )!bad time field
     &    .OR.  BTEST( Bscanqual(Line), bits(20) )!sequence repeats
     &    .OR.  BTEST( Bscanqual(Line), bits(19) )!TIP synch error
     &    .OR.  BTEST( Bscanqual(Line), bits(18) )!TIP frame sequence error
     &    .OR.  BTEST( Bscanqual(Line), bits(17) )!TIP status error
     &    .OR.  BTEST( Bscanqual(Line), bits(16) )!TIP parity error
     &    .OR.  BTEST( Bscanqual(Line), bits(04) )!earth location unreasonable

!     For bad scanline, flag each fov
      IF ( badline ) THEN
         DO fov = 1, Bnfovs
            Bqc(fov,Line) = IBSET ( Bqc(fov,Line), bits(0) )
         ENDDO  ! fov
      ENDIF


      RETURN
      END
