!+ Read & interpret AMSU-B level 1C header within ATOVPP
!
      SUBROUTINE PPBINH ( B1Cunit, SatId, WMOSatId,
     &                    StartYearB, StartDayB, StartTimeB,
     &                    EndYearB,   EndDayB,   EndTimeB )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppbinh.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Open input dataset and read level 1C AMSU-B header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     AMSU-B level 1C header has already been read into common /amb1chd/
!     Check that format of 1C include files (amb1c.h) is compatible with the
!     code in this subroutine. Check that format of 1C file is compatible with
!     this subroutine, and that data type from the header is 11 for AMSU-B or
!     12 for MHS.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from AMSU-B 1C header
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      10/12/96  SatId added, Ht and Period deleted
!  1.2      10/03/98  Updated for 1c vn 1.9 (nothing relevant here)
!  1.3       1/12/03  Add MHS
!  1.4      23/08/05  Add WMOSatId (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! block of AMSU-B data
#include "ppbstore.h"      
! AMSU-B level 1C header/data common blocks
#include "amb1c.h"           
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
!
! Local parameters:
      INTEGER       B1Cvnnb       ! version number of 1C format compatible
         PARAMETER (B1Cvnnb=190)  ! with the code in this subroutine
!
!     Alternative way of checking that version numbers match (at compile time)
!      REAL VNTEST1(B1Cvnnb-amb_1cvnnb+1), VNTEST2(amb_1cvnnb-B1Cvnnb+1)
!
!   Scalar arguments with intent(In):
      INTEGER       B1Cunit       ! logical unit no. for AMSU-B 1C input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 3 for METOP-1)
      ! start/end date/time of AMSU-B data:
      INTEGER       StartYearB, EndYearB
      INTEGER       StartDayB,  EndDayB
      INTEGER       StartTimeB, EndTimeB
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       B1Crecord     ! record number for AMSU-B level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOB1C                ! I/O routine for level 1C AMSU-B data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.1 Open 1C file
      CALL IOB1C ("OPEN/READ", B1Cunit, B1Crecord )   ! 1C file

! 1.2 Read 1C header
      B1Crecord = 1   ! Headers start on record 1.
      CALL IOB1C ("READ HEADER", B1Cunit, B1Crecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( B1Cvnnb .NE. amb_1cvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but amb1c.h is "",I6)")
     &        B1Cvnnb, amb_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPBINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "amb1C.H" contains its own version number "amb_h_l1cversnb"
      IF ( amb1c_h_versnb .NE. amb_1cvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level 1C AMSU-B file is "",I8,"", not "",I8)")
     &        amb1c_h_versnb,amb_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPBINH")
      ENDIF

! 2.3 Is the 1C data AMSU-B ?
!     -- --- -- ---- ----
!     "amb1c_h_instrument" from 1C header should be 11 for AMSU-B
      IF ( amb1c_h_instrument.NE.11.AND.amb1c_h_instrument.NE.12) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage, "("" Data type in 1C AMSU-B file is "",
     &        I8,""; not 11(12) for AMSU-B(MHS)"")") amb1c_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "PPBINH")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "##############"
      WRITE(*,*) "# AMSU-B/MHS #"
      WRITE(*,*) "##############"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            amb1c_h_versnb, amb1c_h_versyr, amb1c_h_versdy
      WRITE(*,"(I4,"" header records in file."")") amb1c_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") amb1c_h_satid

!     calculate start and end dates/times from header
!     ..start
      year = amb1c_h_startdatayr
      yrday = amb1c_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(amb1c_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = amb1c_h_enddatayr
      yrday = amb1c_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(amb1c_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          amb1c_h_scnlin, amb1c_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from AMSU-B 1C header
!     ------- -------- ----------- ---- ------ -- ------

      BSatId = amb1c_h_satid
      SatId = BSatId
      WMOSatId = amb1c_h_wmosatid
      StartDayB = amb1c_h_startdatady
      StartYearB = amb1c_h_startdatayr
      StartTimeB = amb1c_h_startdatatime
      EndDayB = amb1c_h_enddatady
      EndYearB = amb1c_h_enddatayr
      EndTimeB = amb1c_h_enddatatime
      Bsatht = REAL(amb1c_h_satht)*0.1      !nominal sat height (km)
      Bsatperiod = REAL(amb1c_h_period)/60. !nominal orbit period (minutes)
      Bhdrcnt = amb1c_h_hdrcnt       !count of header records in 1C file
      Bscnlin = amb1c_h_scnlin       !count of scanlines in 1C file
      Binstruments = IBSET ( Binstruments , bits(4) )   ! AMSU-B data present


      RETURN
      END
