!+ Transfer data from program store to level 1D AMSU-B data record
!
      SUBROUTINE PPBOUTD ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppboutd.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   ATOVPP stores data from several scanlines in its own internal array
!   structure, contained in common/ppbstore/. This routine transfers a single
!   scanline from slot "Line" in common/ppbstore/ to common/amb1ddt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1D include file (amb1d.h) and AMSU-B store
!     (ppbstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
!     Fill parts of 1D data record that won't change each scanline
! 1.  Copy variables from slot "Line" in common/ppbstore/ to common/amb1ddt/
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1D format vn 1.9
!  1.2      17/01/97  Conform to 1D format vn 1.91
!  1.3      10/03/98  Conform to 1D format vn 1.92
!  1.4      29/06/98  Include AMSU-A mapped to B (1D format vn 1.93)
!  1.5      02/07/98  Bangles REAL not INTEGER
!  1.6      01/08/98  Bsurfcost & Bscatindx *100 on output
!  1.7      17/08/00  Add cirrus index (1D format vn 1.94)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
!  block of AMSU-B data       
#include "ppbstore.h"        
! AMSU-B level 1D header/data common blocks
#include "amb1d.h"           
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppbstore/
!
! Local parameters:
      INTEGER       B1Dvnnb       ! version number of 1D format compatible
         PARAMETER (B1Dvnnb=194)  ! with the code in this subroutine
      INTEGER       Bstvnnb       ! version number of "ppbstore.h" compatible
         PARAMETER (Bstvnnb=140)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov           ! loop variable over all fovs
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first                    ! .TRUE. on first call
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

! 0.1    Is the 1D include file compatible with this source code ?
         IF ( B1Dvnnb .NE. amb_1dvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1D format"",I6,"" but amb1d.h is "",I6)")
     &           B1Dvnnb, amb_1dvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPBOUTD")
         ENDIF

! 0.2    Is the ppbstore include file compatible with this source code ?
         IF ( Bstvnnb .NE. amb_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects Bstore format"",I6,"" but ppbstore.h is"",I6)")
     &           Bstvnnb, amb_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPBOUTD")
         ENDIF

! 0.3    Reset "first" for subsequent calls
         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from slot "Line" in common/pphstore/ to common/amb1ddt/
!     ---- --------- ---- ----  ----  -- ---------------- -- ---------------

      amb1d_scnlin = Blineno(Line)                ! scanline number
      amb1d_scnlinyr = Byear(Line)                ! scanline year
      amb1d_scnlindy = Bday(Line)                 ! scanline day of year
      amb1d_scnlintime = Btime(Line)              ! scanline time (ms)
      amb1d_qualind = Bqualind(Line)              ! quality indicator bit field
      amb1d_scanqual = Bscanqual(Line)            ! scan line quality flags
      amb1d_instrtemp = Binstrtemp(Line)          ! AMSU-B Mixer chan 18-20 temp
      DO i = 1, 2
         amb1d_spare1(i) = MDI                    ! spare
      ENDDO  ! i

      DO fov = 1, Bnfovs
         amb1d_latlon(1,fov) = NINT(Blats(fov,Line)*1.E4) !latitude (N +ve)
         amb1d_latlon(2,fov) = NINT(Blons(fov,Line)*1.E4) !longitude (E +ve)
         amb1d_surface(1,fov) = Belev(fov,Line)   !surface height (m)
         amb1d_surface(2,fov) = Bsurftype(fov,Line) !surface type
         DO i = 1, 4
            amb1d_angles(i,fov) = NINT(Bangles(i,fov,Line)*100.) ! scan angles:
!                                         (1,*,*) local zenith angle (deg*100)
!                                         (2,*,*) local azimuth angle (deg*100)
!                                         (3,*,*) solar zenith angle (deg*100)
!                                         (4,*,*) solar azimuth angle (deg*100)
         ENDDO  ! i
      ENDDO  ! fov

      amb1d_scalti = Baltitude(Line)              ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
      DO i = 1, 2
         amb1d_spare2(i) = MDI                    ! spare
      ENDDO  ! i

!     mapped AMSU-A BTs (deg K * 100)
      DO fov = 1, Bnfovs
      DO chn = 1, Anchan
         amb1d_btemps(chn,fov) = NINT(BmapBT(chn,fov,Line)*100.)
      ENDDO  ! chn
      ENDDO  ! fov

!     AMSU-B BTs (deg K * 100)
      DO fov = 1, Bnfovs
      DO chn = 1, Bnchan
         amb1d_btemps(Anchan+chn,fov) = NINT(Bbtemps(chn,fov,Line)*100.)
      ENDDO  ! chn
      ENDDO  ! fov

![user-defined:
      DO fov = 1, Bnfovs
         amb1d_prepro(1,fov) = Bqc(fov,Line)       ! ATOVPP QC word for AMSU
         amb1d_prepro(2,fov) = Bmapsurf(fov,Line)  ! estimated surface type
         amb1d_prepro(3,fov) = NINT(Bsurfcost(fov,Line)*100.) !AMSU-A cloud test
         amb1d_prepro(4,fov) = NINT(Bscatindx(fov,Line)*100.) !scattering index
         amb1d_prepro(5,fov) = NINT(Bcirrindx(fov,Line)*1000.) !cirrus index
!         amb1d_prepro(6,fov) = MDI                 ! spare

!Add NWC-SAF scattering variables. For backwards compatibility, squeeze them
!into the spare prepro word. Scattering index is in range -327.68K to +327.67K,
!precip probs are in range 0 to 15. Users can redefine if necessary.

         i = MIN(MAX(NINT(Bsi_nwcsaf(fov,Line)*100), -32768), 32767)
         amb1d_prepro(6,fov) = i * 65536                        !Bits 16 to 31
         i =  MIN(MAX(NINT(Bm0(fov,Line)*15.0), 0), 15)        
         amb1d_prepro(6,fov) = IOR(amb1d_prepro(6,fov), i)      !Bits 0 to 3
         i =  MIN(MAX(NINT(Bm1(fov,Line)*15.0), 0), 15)        
         amb1d_prepro(6,fov) = IOR(amb1d_prepro(6,fov), i*16)   !Bits 4 to 7
         i =  MIN(MAX(NINT(Bm2(fov,Line)*15.0), 0), 15)        
         amb1d_prepro(6,fov) = IOR(amb1d_prepro(6,fov), i*256)  !Bits 8 to 11
         i =  MIN(MAX(NINT(Bm3(fov,Line)*15.0), 0), 15)        
         amb1d_prepro(6,fov) = IOR(amb1d_prepro(6,fov), i*4096) !Bits 12 to 15

      ENDDO  ! fov
! ... ]


      RETURN
      END
