!+ Set up (AMSU-B on AMSU-B grid) level 1D header within ATOVPP
!
      SUBROUTINE PPBOUTH ( LAMSA )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppbouth.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Set up level 1D header using information from 1C headers.
!
! Method:
! 1.  Check format version number and data type
!     AMSU-B level 1C header has already been read into common /amb1chd/
!     Check that format of 1C & 1D include files (amb1c.h, amb1d.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1C file is compatible with this subroutine.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Set up level 1D header
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1D format vn 1.9
!  1.2      17/01/97  Conform to 1D format vn 1.91
!  1.3      10/03/98  Conform to 1D format vn 1.92
!  1.4      29/06/98  Include AMSU-A mapped to B (1D format vn 1.93)
!  1.5      17/08/00  1D format vn 1.94 (for cirrus index)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  fixed data for processing AMSU-B 
#include "ppbfdata.h"
!  block of AMSU-A data 
#include "ppbstore.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!  AMSU-A level 1D header/data common blocks 
#include "amb1d.h"
!
!   Scalar arguments with intent(In):
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
!
! Local parameters:
      INTEGER       B1Cvnnb       ! version number AMSU-B 1C format compatible
         PARAMETER (B1Cvnnb=190)  !    with the code in this subroutine
      INTEGER       B1Dvnnb       ! version number 1D format compatible
         PARAMETER (B1Dvnnb=194)  !    with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! general purpose loop variables
      INTEGER       chn           ! loop variable over channels
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 1.1 Are the 1C include files compatible with this source code ?
!     --- --- -- ------- ----- ---------- ---- ---- ------ ----

      IF ( B1Cvnnb .NE. amb_1cvnnb ) THEN
         errStatus = -11  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but amb1c.h is "",I6)")
     &        B1Cvnnb, amb_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPBOUTH")
      ENDIF

! 1.2 Is the 1D include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( B1Dvnnb .NE. amb_1dvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1D format "",I6,"" but amb1d.h is "",I6)")
     &        B1Dvnnb, amb_1dvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPBOUTH")
      ENDIF


! 2.  Set up level 1D header
!     --- -- ----- -- ------

! 2.1 General

!     creation site for 1D data
      amb1d_h_site   = SiteID              !file creation site ID (CHAR*3)
      amb1d_h_cfill1 = " "                 !filler

!     creation site for original 1B data
      amb1d_h_1bsite = amb1c_h_1bsite
      amb1d_h_cfill2 = " "                 !filler

      amb1d_h_versnb = amb_1dvnnb          !level 1d format version number
      amb1d_h_versyr = amb_1dvnyr          !level 1d format version year
      amb1d_h_versdy = amb_1dvndy          !level 1d format version day of year
      amb1d_h_hdrcnt = amb1c_h_hdrcnt      !count of header records
      amb1d_h_satid = amb1c_h_satid        !satellite id (e.g. 14 for NOAA-14)
      amb1d_h_grid = amb1c_h_instrument    !instrument grid code (AMSU-B=11)
      amb1d_h_satht = amb1c_h_satht        !nominal sat height, km*10
      amb1d_h_period = amb1c_h_period      !nominal orbit period (seconds)
      amb1d_h_startorbit = amb1c_h_startorbit !orbit number (at start)
      amb1d_h_startdatayr = amb1c_h_startdatayr!start of data set year
      amb1d_h_startdatady = amb1c_h_startdatady! (ditto) day of the year
      amb1d_h_startdatatime = amb1c_h_startdatatime !(ditto) UTC time of day,ms
      amb1d_h_endorbit = amb1c_h_endorbit  !orbit number (at end)
      amb1d_h_enddatayr = amb1c_h_enddatayr!end of data set year
      amb1d_h_enddatady = amb1c_h_enddatady!(ditto) day of the year
      amb1d_h_enddatatime = amb1c_h_enddatatime !(ditto) UTC time of day,ms
      amb1d_h_scnlin = amb1c_h_scnlin      !count of scan lines in this dataset
      amb1d_h_misscnlin = amb1c_h_misscnlin !count of missing scan lines
      amb1d_h_atovppvn = ATOVPPvn     !ATOVPP version number (test vns = 9000+)
      amb1d_h_instruments = Binstruments   !instruments present
      amb1d_h_Bantenna = amb1c_h_vnantennacorr ! vn AMSU-B antenna corrections
      amb1d_h_wmosatid = amb1c_h_wmosatid      ! WMO sat ID

! 2.2 temperature-radiance conversion coefficients

! AMSU-A and B temperature-radiance conversion coefficients  (Planck function)
!                  order of channels = AMSU-A 1-15, AMSU-B 1-5
!                  (1,*) = 10^6 x (central wavenumber)
!                  (2,*) = 10^6 x (constant 1)
!                  (3,*) = 10^6 x (constant 2)
!
!     AMSU-A
      IF ( LAMSA ) THEN
         DO chn = 1, Anchan
         DO i = 1, 3
            amb1d_h_amsPlanck(i,chn) = ama1c_h_tempradcnv(i,chn)
         ENDDO  ! i
         ENDDO  ! chn
      ELSE
         DO chn = 1, Anchan
         DO i = 1, 3
            amb1d_h_amsPlanck(i,chn) = MDI
         ENDDO  ! i
         ENDDO  ! chn
      ENDIF
!
!     AMSU-B
      DO chn = 1, Bnchan
      DO i = 1, 3
         amb1d_h_amsPlanck(i,Anchan+chn) = amb1c_h_tempradcnv(i,chn)
      ENDDO  ! i
      ENDDO  ! chn

! 2.3 Pre-processing information

![user-defined:
      amb1d_h_Bcorrvn  = Bcorrvn        ! vn no AMSU-B limb/emiss correction
      amb1d_h_Bpcpvn   = Bpcpvn         ! vn no AMSU-B precip/scat detection
      DO i = 1, 2218
         amb1d_h_spare2(i) = MDI        !spare
      ENDDO  ! i
! ... ]


      RETURN
      END
