!+ map brightness temperatures between ATOVS instrument grids  (ATOVPP module)
!
      SUBROUTINE PPBTMAP ( CMap, CTarg,
     &           Map_Nlines,  Map_line0,
     &           Targ_Nfovs, Targ_Nlines, Targ_line0,
     &           Elms_per_coloc, Max_colocs, LUT_targ_lines,
     &           First_LUT_targ_line, LUT_map_lines, LUT, Mode, IFOV )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppbtmap.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   map brightness temperatures between ATOVS instrument grids
!   (Companion routine: LUTMAP)
!
!   Note that many of the mappings are commented out.
!   Their inclusion would require other modifications to ATOVPP,
!   such as the coding of appropriate pre-processing tests and of
!   the necessary FOV-selection routines PPH2B,..etc.
!
! Method:
!   SR LUTMAP generates a Look-Up-Table (LUT) which identifies those mapping
!   fovs which colocate with a target fov. The LUT also provides a weight for
!   each mapping fov for each mapping mode (bi-linear interpolation,
!   spatial average). The weights for the selected mode are applied to
!   the corresponding BTs and the resulting sum provides the mapped value.
!
!   Note a block of data is mapped by one call to BT_MAP.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  0.0      04/05/95  Original code. (P.C.Dibben)
!  1.0      27/11/96  Code adapted for ATOVPP. (R.J.Renshaw)
!  1.1      29/06/98  Added PPA2B
!  1.2      23/11/98  In section 1 intialisation; Mode = 1 (nearest neighbour
!                     is checked for and LUTref set =5 (and not 6 as before
!                     which demands bi-linear interpolation weight for
!                     colocation 1 is .gt. 0). NB change also made to
!                     lutmap (v1.4). (P C Dibben)
!  1.3      06/12/00  Added call of PPBGB2A for BG-Convolution (Ralf Bennartz)           
!  1.4      26/01/05  Add AMSU-A to IASI. Add IFOV as argument.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE U.K. MET. OFFICE.
!#########################################################################
!
! Declarations:
!
! Note:
! The Mapping and Target data arrays (eg Map_BTs) represent a block of
! data for processing. The arrays have implicit line numbers running (for
! Map_BTs) from 1 to Map_Nlines. These line numbers can be converted to
! "1C" line numbers by adding Map_line0 (or Targ_line0).
!
! Declarations:
!
! Subroutine arguments
      CHARACTER*(*)  CMap             ! mapping instrument
      CHARACTER*(*)  CTarg            ! target instrument
!                                      (i.e. HIRS, AMSU-A, AMSU-B, MSU)
!
!   Mapping data  - all with intent(In):
      INTEGER    Map_Nlines       ! number of lines in Map_BTs
      INTEGER    Map_line0        ! "1C" line number -1 of the 1st line
                                  ! represented in the Map buffers (eg Map_BTs)
!
!   Target data  - all with intent(In):
      INTEGER    Targ_Nfovs       ! number of fovs per line
      INTEGER    Targ_Nlines      ! number of lines in Targ_BTs
      INTEGER    Targ_line0       ! "1C" line number -1 of the 1st line
                                  ! represented in Targ_* (eg Targ_BTs)
!
!   LUTMAP data  - all with intent(In):
      INTEGER    Elms_per_coloc   ! number of elements of information per coloc
      INTEGER    Max_colocs       ! max number of colocations
      INTEGER    LUT_targ_lines   ! number of target lines represented in LUT
      INTEGER    First_LUT_targ_line ! first target line represented in LUT
      INTEGER    LUT_map_lines    ! number of mapping lines represented in LUT
      REAL       LUT(Elms_per_coloc,Max_colocs,Targ_Nfovs,
     &                   LUT_targ_lines)  ! look-up-table from LUTMAP
!
!   Mapping options  - intent(In):
      INTEGER    Mode             ! 1=nearest neighbour, 2=bilinear interp,
!                                   3=spatial averaging with gaussian weighting
!                                   4=spatial average with linear weighting
!                                   5=BG-convolution
      INTEGER    IFOV             !IASI detector FOV number (1 to 4)

! Local scalars:
      INTEGER    ncolocs          ! no of valid colocation entries from LUT
      INTEGER    map_line_offset  ! offset to be added to a LUT mapping
                                  ! line number to give a `1c" line number
      INTEGER    offset           ! used in calculating LUT entry line numbers
      INTEGER    modulo_line      ! used in calculating LUT entry line numbers
      INTEGER    no_entries       ! number of colocated fovs in LUT
      INTEGER    LUT_line         ! appropriate LUT entry line
      INTEGER    LUTref           ! entry in LUT for this mapping mode
      INTEGER    map_line         ! line number for mapped instrument
      INTEGER    map_fov          ! fov number for mapped instrument
      INTEGER    ii               ! loop variable, target instrument line number
      INTEGER    jj               ! loop variable, target instrument fov
      INTEGER    kk               ! loop variable, colocated fovs
      REAL       weight           ! used in the mapping
!
! Local arrays:
      INTEGER    list_line(40)    ! list of coloc line numbers (1st=nearest)
      INTEGER    list_fov(40)     !               spot numbers
      REAL       list_wt(40)      !               LUT weights
!
! Function & Subroutine calls:
      External PPA2H              ! select FOVs to use to map AMSU-A to HIRS
C     External PPB2H              !    (ditto)                AMSU-B to HIRS
      External PPM2H              !    (ditto)                 MSU to HIRS
C     External PPH2A              !    (ditto)                HIRS to AMSU-A
      External PPB2A              !    (ditto)                AMSU-B to -A
      External PPBGB2A            !    (ditto)                AMSU-B to -A
C     External PPH2B              !    (ditto)                HIRS to AMSU-B
      External PPA2B              !    (ditto)                AMSU-A to -B
      External PPA2I              !    (ditto)                AMSU-A to IASI
!
!- End of header


! 1.  Initialisation
!     --------------

! 1.1 Mapping mode
      IF ( Mode.EQ.1) THEN
          LUTref = 5  ! nearest neighbour
      ELSE IF ( Mode.EQ.2 ) THEN
          LUTref = 6  ! bilinear interpolation
      ELSE
          LUTref = 7  ! spatial averaging
      ENDIF


! 2.  Loop through all target lines
!     ---- ------- --- ------ -----

      DO ii = 1, Targ_Nlines

! 2.1   convert an ii line number into an entry line to the LUT (LUT)
        offset = Targ_line0 + ii                ! ie the "1C" line number
        offset = offset - First_LUT_targ_line   ! line number relative to the
                                                ! start of the LUT
        modulo_line = MOD(offset,LUT_targ_lines)
        IF (modulo_line.LT.0) modulo_line = modulo_line+LUT_targ_lines
        LUT_line = modulo_line + 1         ! appropriate LUT entry line

!       now calculate a mapping line offset ie the offset between the mapping
!       line number provided by the LUT and the "1C" mapping line number.
!       This offset will be some integer multiple of the number of mapping
!       lines in the duration of the LUT.
        IF (offset.GE.0) THEN
          map_line_offset = (offset/LUT_targ_lines) * LUT_map_lines
        ELSE
          IF (modulo_line.EQ.0) THEN
            map_line_offset = (offset/LUT_targ_lines) * LUT_map_lines
          ELSE
            map_line_offset = ((offset-LUT_targ_lines)/LUT_targ_lines)*
     &                        LUT_map_lines
          ENDIF
        ENDIF

! 2.2   Loop through all target fovs in line
        DO jj = 1, Targ_Nfovs

! 2.2.1    Produce a list of colocations which exist within the mapped data
           ncolocs = 0
           no_entries = NINT(LUT(Elms_per_coloc,1,jj,LUT_line)) !no of colocs
           DO kk = 2, no_entries+1  ! loop through colocations for target

              IF (LUT(LUTref,kk,jj,LUT_line).GT.0.) THEN   ! valid coloc

!               get fov/line number for colocated mapping fov
                map_fov  = NINT(LUT(2,kk,jj,LUT_line))
                map_line = NINT(LUT(1,kk,jj,LUT_line))    ! "1C" line number
     &                   + map_line_offset
                map_line = map_line - Map_line0     ! Map_BTs line number
                weight = LUT(LUTref,kk,jj,LUT_line) ! weight for the mapping

!               Check that line is in Map_BTs & that coloc is to be used
                IF (weight.GT.0. .AND.
     &              map_line.GE.1 .AND. map_line.LE.Map_Nlines) THEN
                   ncolocs = ncolocs + 1
                   list_fov(ncolocs)  = map_fov
                   list_line(ncolocs) = map_line
                   list_wt(ncolocs)   = weight
                ENDIF               ! Map line OK & coloc has +ve weight

              ENDIF                 ! valid colocation

           ENDDO  ! kk              ! loop through colocations for target


! 2.2.3    Call specific routine to derive mapped BTs
           IF ( ncolocs .GT. 0 ) THEN

              IF ( Mode.EQ.1 ) ncolocs = 1  ! nearest neighbour only

              IF ( CTarg.EQ."HIRS" ) THEN

! ***            AMSU-A to HIRS
                 IF ( CMap.EQ."AMSU-A" ) THEN
                    CALL PPA2H ( jj, ii,
     &                           ncolocs, list_fov, list_line, list_wt )

! ***            MSU to HIRS
                 ELSEIF ( CMap.EQ."MSU" ) THEN
                    CALL PPM2H ( jj, ii,
     &                           ncolocs, list_fov, list_line, list_wt )

                 ENDIF  ! CMap

              ELSEIF ( CTarg.EQ."AMSU-A" ) THEN

! ***            AMSU-B to AMSU-A
                 IF ( CMap.EQ."AMSU-B" ) THEN
                    IF (Mode.NE.5) THEN 
                       CALL PPB2A ( jj, ii,
     &                           ncolocs, list_fov, list_line, list_wt )
                    ELSE
                       CALL PPBGB2A ( jj, ii,
     &                           ncolocs, list_fov, list_line, list_wt )
                    ENDIF
                 ENDIF  ! CMap

              ELSEIF ( CTarg.EQ."AMSU-B" ) THEN

! ***            AMSU-A to AMSU-B
                 IF ( CMap.EQ."AMSU-A" ) THEN
                    CALL PPA2B ( jj, ii,
     &                           ncolocs, list_fov, list_line, list_wt )

                 ENDIF  ! CMap

              ELSEIF ( CTarg.EQ."IASI" ) THEN

! ***            AMSU-A to IASI
                 IF ( CMap.EQ."AMSU-A" ) THEN
                    CALL PPA2I ( jj, ii,
     &                     ncolocs, list_fov, list_line, list_wt, IFOV )
                 ENDIF  ! CMap

              ENDIF  ! CTarg

           ENDIF  ! ncolocs

          ENDDO  ! jj                      ! target fovs in line
        ENDDO  ! ii = 1, Targ_Nlines       ! target lines


      RETURN
      END
