!+ Look for precipitation signal in ATMS on CRIS grid
!
      SUBROUTINE PPCAPCP ( crislastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcapcp.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Look for precipitation signal in ATMS mapped to CRIS grid
!   and set flags accordingly.
!
! Method:
! 0.  Loop through scanlines and fields-of-view
! 1.  Scattering test
! 2.  Crosby, Ferraro & Wu test
! 3.  Grody light rainfall test
! 4.  Set flags
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   02/09/2010 Derived from subroutine ppiapcp. (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  fixed data for processing ATMS 
#include "ppatmsfdata.h"
!  IASI 1d format 
#include "cris1d.h"
!  block of IASI level 1d data 
#include "ppcrisstore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       crislastLine     ! number of last HIRS line in store
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       Afov          ! nearest ATMS fov to CrIS fov
      REAL          T1            ! brightness temp in ATMS channel 1 (K)
      REAL          T2            ! brightness temp in ATMS channel 2 (K)
      REAL          T3            ! brightness temp in ATMS channel 3 (K)
      REAL          T16           ! brightness temp in ATMS channel 16 (K)
      REAL          T17           ! brightness temp in ATMS channel 17 (K)
      REAL          T18           ! brightness temp in ATMS channel 18 (K)
      REAL          SI            ! scattering index
      REAL          CI            ! cirrus index
      REAL          probprecip    ! Crosby probability of precipitation
      LOGICAL       scat          ! .TRUE. if scattering detected
      LOGICAL       cirrus        ! .TRUE. if cirrus detected
      LOGICAL       crosby        ! .TRUE. if Crosby test flags fov
      LOGICAL       grody         ! .TRUE. if Grody test flags fov
!
! Local arrays:
      INTEGER NearestAfov(crisnfovs) ! nearest ATMS fov to each CRIS fov
      DATA NearestAfov / 270*0 /
      
!
! Function & Subroutine calls:
      External PPATMSSCAT            ! scattering index
          REAL PPATMSSCAT
      External PPATMSCIRR            ! scattering index
          REAL PPATMSCIRR
      External PPATMSCROSBY           ! logistic regression
          REAL PPATMSCROSBY
      External PPATMSGRODY            ! light rainfall test
       LOGICAL PPATMSGRODY
!
!- End of header

!Approximate the nearest ATMS fov to each CRIS fov
      IF (NearestAfov(1).EQ.0) THEN
        DO fov = 1, crisnfor
          NearestAfov(fov) = atmsnfovs/2 - (crisnfor/2)*3 + fov*3 - 1
          DO line = 1, 8
            NearestAfov(fov+line*crisnfor) = NearestAfov(fov)
          ENDDO
        ENDDO
      ENDIF


! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, crislastLine
      DO fov = 1, crisnfovs

         Afov = NearestAfov(fov)
         T1  = crismapbt(1,fov,line)
         T2  = crismapbt(2,fov,line)
         T3  = crismapbt(3,fov,line)
         T16 = crismapbt(16,fov,line)
         T17 = crismapbt(17,fov,line)
         T18 = crismapbt(18,fov,line)

! 1.     Scattering test  (only over the sea)
!        ---------- ----

!        only do if channels present
         IF (  T1 .GT. RMDI  .AND.  T2 .GT. RMDI
     &   .AND. T3 .GT. RMDI  .AND.  T16 .GT. RMDI ) THEN
            SI = PPATMSSCAT ( T1, T2, T3, T16, Afov )
            crisscatindx(fov,line) = SI
            IF ( crissurftype(fov,line).EQ.0 ) THEN
               scat = SI .GT. atmsmaxscat
            ELSE
               scat = .FALSE.
            ENDIF
         ELSE
            crisscatindx(fov,line) = RMDI
            scat = .TRUE.  ! default is to err on side of caution
         ENDIF


! 2.     Cirrus test  (only over the sea)
!        ------ ----

!        only do if channels present
         IF (  T1 .GT. RMDI  .AND.  T16 .GT. RMDI
     &   .AND. T17 .GT. RMDI  .AND.  T18 .GT. RMDI ) THEN
            CI = PPATMSCIRR ( T1, T16, T17, T18, Afov )
            criscirrindx(fov,line) = CI
            IF ( crissurftype(fov,line).EQ.0 ) THEN
               cirrus = CI .GT. atmsmaxcirr
            ELSE
               cirrus = .FALSE.
            ENDIF
         ELSE
            criscirrindx(fov,line) = RMDI
            cirrus = .TRUE.  ! default is to err on side of caution
         ENDIF


! 3.     Crosby, Ferraro & Wu test
!        ------  ------- - -- ----

         IF (  T1 .GT. RMDI  .AND.  T16 .GT. RMDI ) THEN
            probprecip = PPATMSCROSBY(T1,T16 )
            criscrosby(fov,line) = probprecip
            crosby = probprecip .GT. atmsmaxcrosby
         ELSE
            criscrosby(fov,line) = RMDI
            crosby = .TRUE.  ! default is to err on side of caution
         ENDIF


! 4.        Grody light rainfall test
!           ----- ----- -------- ----

         IF (  T1 .GT. RMDI  .AND.  T2 .GT. RMDI ) THEN
            grody = PPATMSGRODY( T1, T2 )
         ELSE
            grody = .TRUE.  ! default is to err on side of caution
         ENDIF


! 5.     Set flags
!        --- -----

         IF ( scat )
     &      crismapqc(fov,line) = IBSET ( crismapqc(fov,line), bits(26))
         IF ( cirrus )
     &      crismapqc(fov,line) = IBSET ( crismapqc(fov,line), bits(17))
         IF ( crosby )
     &      crismapqc(fov,line) = IBSET ( crismapqc(fov,line), bits(25))
         IF ( grody )
     &      crismapqc(fov,line) = IBSET ( crismapqc(fov,line), bits(24))

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
