!+ Estimate surface type from ATMS on CrIS grid
!
      SUBROUTINE PPCASURF ( crisLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcasurf.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!     Estimate surface type and flag cloud liquid water
!     using ATMS data mapped to the CrIS grid
!
! Method:
! 0.  Loop through scanlines and fields-of-view
! 1.  Interpolate means & covariances to scan angle
! 2.  Find most likely surface type
!
! Surface types:
! 1 = Bare young ice (i.e. new ice, no snow)
! 2 = Dry land (i.e. dry with or without significant vegetation)
! 3 = Dry snow (i.e. snow with water less than 2%, over land)
! 4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
! 5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
! 6 = Wet forest (i.e. established forest with wet canopy)
! 7 = Wet land (i.e. non-forested land with a wet surface)
! 8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
! 9 = Desert
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    02/09/2010  Derived from subroutine ppiasurf (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  fixed data for processing ATMS
#include "ppatmsfdata.h"
!  block of CrIS level 1d data 
#include "ppcrisstore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  Fundamental and physical constants 
#include "constants_preproc.h"
!
!   Scalar arguments with intent(In):
      INTEGER       crisLastLine     ! number of last IASI line in store
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       iang          ! loop index over angles
      INTEGER       isurf         ! loop index over surface types
      INTEGER       k, kx, m      ! loop indices over channels
      INTEGER       kang          ! index of where ASecAng .GE. secang
      INTEGER       surftype      ! derived surface type
      REAL          normalise     ! normalise J wrt number of channels
      REAL          J             ! cost function for a given surface type
      REAL          Jmin          ! minimum value of J for all surface types
      REAL          secang        ! secant(satellite zenith angle)
      REAL          sec_norm      ! normalisation factor for interpolation
      REAL          sum           ! sum of cost function over channels
!
! Local arrays:
      REAL meanBTx(NatmsSrfTyp,NatmsSrfChMx) !mean btemp difference vector at secang
      REAL covBTx(NatmsSrfTyp,NatmsSrfChMx,NatmsSrfChMx) ! inverse cov mx at secang
      REAL diffBTx(NatmsSrfChMx)     ! difference from mean btemps vector
!
!- End of header

!     normalise cost function according to the number of channels used
      normalise = 1. / REAL(NatmsSrfCh*NatmsSrfCh)

! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, crisLastLine
      DO fov = 1, crisnfovs

! 1.     Interpolate means & covariances to scan angle
!        ----------- ----- - ----------- -- ---- -----

         secang = 1. / COS(crisangles(1,fov,line)*deg2rad) ! secant of view angle

!        Find specified angles either side of view angle
         kang = 0
         DO iang = 1, NatmsSecAng-1
            IF ( secang.GE.atmsSecAng(iang)
     &     .AND. secang.LT.atmsSecAng(iang+1) ) kang = iang
         ENDDO  ! iang

!        Derive appropriate means & covariances
         IF ( kang .EQ. 0 ) THEN !secang>atmsSecAng(NatmsSecAng):use atmsSecAng(NSecAng)
            DO isurf = 1, NatmsSrfTyp
            DO k = 1, NatmsSrfCh
               meanBTx(isurf,k) = atmsmeanBT(isurf,NatmsSecAng,k)
               DO m = 1, NatmsSrfCh
                  covBTx(isurf,m,k) = atmscovBT(isurf,NatmsSecAng,m,k)
               ENDDO  ! m
            ENDDO  ! k
            ENDDO  ! isurf
         ELSE                      ! sec < atmsSecAng(NatmsSecAng): interpolate
            sec_norm = (secang-atmsSecAng(kang))
     &                 / (atmsSecAng(kang+1)-atmsSecAng(kang))
            DO isurf = 1, NatmsSrfTyp
            DO k = 1, NatmsSrfCh
               meanBTx(isurf,k) = atmsmeanBT(isurf,kang,k) + sec_norm*
     &             (atmsmeanBT(isurf,kang+1,k)-atmsmeanBT(isurf,kang,k))
               DO m = 1, NatmsSrfCh
                  covBTx(isurf,m,k) = atmscovBT(isurf,kang,m,k) +
     &        sec_norm*(atmscovBT(isurf,kang+1,m,k)-
     &                  atmscovBT(isurf,kang,m,k))
               ENDDO  ! m
            ENDDO  ! k
            ENDDO  ! isurf
         ENDIF  ! kang .EQ. 0


! 2.     Find most likely surface type
!        ---- ---- ------ ------- ----

!        We evaluate the cost function J = (to-tm).C^-1.(to-tm)T
!        to find the minimum value, looping through each surface type:
         Jmin = 9999.0
         DO isurf = 1, NatmsSrfTyp

            DO k = 1, NatmsSrfCh
               kx = atmsSrfChn(k)
               diffBTx(k) = crismapbt(kx,fov,line) - meanBTx(isurf,k)
            ENDDO  ! k

!           J = ABS(diffBTx times cov.matrix times diffBTx(transpose))
            J = 0.
            DO k = 1, NatmsSrfCh
               sum = 0.
               DO m = 1, NatmsSrfCh
                  sum = sum + covBTx(isurf,m,k)*diffBTx(m)
               ENDDO  ! m
               J = J + diffBTx(k)*sum
            ENDDO  ! k
            J = ABS(J)

!           Is this the best fit yet ?
            IF ( J .LT. Jmin ) THEN
               Jmin = J
               surftype = isurf
            ENDIF

         ENDDO  ! isurf

!        normalise according to the number of channels used
         IF (Jmin .LT. 9999.0) Jmin = Jmin * normalise

!        apply threshold check to minimum J for cloud
         IF ( Jmin .GT. atmsSrfThresh ) THEN
            crismapqc(fov,line) = IBSET ( crismapqc(fov,line), bits(2) )
         ENDIF

!        if snow or sea-ice, check channel 1 to see if this is actually desert
         IF ( surftype.EQ.1 .OR. surftype.EQ.4 .OR. surftype.EQ.8 ) THEN
            IF ( crismapbt(1,fov,line).GT.275. ) surftype = 9    ! desert
         ENDIF

!        store minimum J and surface type
         crissurfcost(fov,line) = Jmin
         crismapsurf(fov,line) = surftype

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
