!+ Transfer data from program store to level 1D CrIS data record
!
      SUBROUTINE PPCOUTD ( Line, Irec )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcoutd.F 317 2012-07-03 10:38:38Z frna $
!
! Purpose:
!   ATOVPP stores data from several scanlines in its own internal array
!   structure, contained in common/ppcrisstore/. This routine transfers a single
!   scanline from slot "Line" in common/ppcrisstore/ to common/cris1ddt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1D include file (cris1d.h) and CrIS store
!     (ppcrisstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
!     Fill parts of 1D data record that won't change each scanline
! 1.  Copy variables from slot "Line" in common/ppcrisstore/ to common/cris1ddt/
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   03/09/2010 Created from ppioutd.F (N.C.Atkinson)
!  1.1   03/07/2012 Fill cris1d_nedtcoldatms and cris1d_nedtwarmatms
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of CrIS data 
#include "ppcrisstore.h"
!  CrIS level 1D header/data common blocks 
#include "cris1d.h"
!  CrIS fixed data and user options 
#include "ppcrisfdata.h"
! ATMS fixed data
#include "ppatmsfdata.h"       
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppistore/
      INTEGER       Irec          ! CrIS 1d record (1 to 9)
!
! Local parameters:
      INTEGER       cris1Dvnnb       ! version number of 1D format compatible
         PARAMETER (cris1Dvnnb=1)    ! with the code in this subroutine
      INTEGER       crisstvnnb       ! version number of "ppistore.h" compatible
         PARAMETER (crisstvnnb=1)    ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov1d         ! loop variable over all 1d fovs
      INTEGER       fov           ! all CrIS fovs (max 270)
      LOGICAL       badfov        ! indicates 1 or more bad BTs
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first                    ! .TRUE. on first call
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

! 0.1    Is the 1D include file compatible with this source code ?
         IF ( cris1Dvnnb .NE. cris_1dvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1D format"",I6,"" but cris1d.h is "",I6)")
     &           cris1Dvnnb, cris_1dvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPCOUTD")
         ENDIF

! 0.2    Is the ppcrisstore include file compatible with this source code ?
         IF ( crisstvnnb .NE. cris_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects Istore format"",I6,"" but ppcrisstore.h"//
     &      " is "",I6)") crisstvnnb, cris_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPCOUTD")
         ENDIF

! 0.3    Fill parts of 1D data record that won't change each scanline

! 0.4    Reset "first" for subsequent calls
         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from slot "Line" in common/ppcrisstore/ to common/cris1ddt/
!     ---- --------- ---- ----  ----  -- ------------------- -- ---------------

!     scnlin, scnlinyr, scnlindy, scnlintime, scalti already filled in ppcris1d.F

      DO fov1d = 1, cris_1dnfor
          
         IF (nrec_cris.EQ.9) THEN
           fov = (irec-1)*30 + fov1d                   !Full resolution
         ELSE
           fov = (crisbestFOV(fov1d,Line)-1)*30 + fov1d   !Best of 9
         ENDIF
         
!     Lat, lon, angles, etc. have already been filled in ppcris1d.F

!     Surfelev outputs
         cris1d_surface(1,fov1d) = criselev(fov,Line)
         cris1d_surface(2,fov1d) = crissurftype(fov,Line)
         cris1d_surface(3,fov1d) = crislandfrac(fov,Line)
         
!     Mapped ATMS BTs (deg K * 100) and quality flag
         badfov = .FALSE.
         cris1d_crisqual(fov1d) = 0
         DO chn = 1, atmsnchan
           cris1d_btemps(chn,fov1d)= NINT(crismapbt(chn,fov,Line)*100.)
           IF (atms_ignore_channel(chn)) THEN
             cris1d_crisqual(fov1d) = 
     &         IBSET(cris1d_crisqual(fov1d),bits(chn+3))
           ELSE IF (crismapbt(chn,fov,Line).LT.atmsminBT .OR.
     &              crismapbt(chn,fov,Line).GT.atmsmaxBT) THEN
             badfov = .TRUE.
             cris1d_crisqual(fov1d) = 
     &         IBSET(cris1d_crisqual(fov1d),bits(chn+3))
           ENDIF
         ENDDO  ! chn

         cris1d_crisqual(fov1d) = crisqc(fov,Line)   ! QC word for IASI

         cris1d_prepro(1,fov1d) = crismapqc(fov,Line)    ! ATOVPP QC word for AMSU
         cris1d_prepro(2,fov1d) = crismapsurf(fov,Line)  ! estimated surface type
         cris1d_prepro(3,fov1d) = NINT(crissurfcost(fov,Line)*100.) ! max prob cloud
         cris1d_prepro(4,fov1d) = NINT(crisscatindx(fov,Line)*100.) ! scattering index
         cris1d_prepro(5,fov1d) = NINT(criscirrindx(fov,Line)*100.) ! cirrus index
         cris1d_prepro(6,fov1d) = NINT(criscrosby(fov,Line)*100.)   ! logistic precip
         IF (crissi_nwcsaf(fov,Line).GT.-999.0) THEN
           cris1d_prepro(7,fov1d) = NINT(crissi_nwcsaf(fov,Line)*100) ! Bennartz index
         ELSE
           cris1d_prepro(7,fov1d) = MDI
         ENDIF

         ! NWC-SAF Precip probabilities (each in range 0 to 15)
         i =  MIN(MAX(NINT(crism0(fov,Line)*15.0), 0), 15)        
         cris1d_prepro(8,fov1d) = i                                 !Bits 0-3 (none)
         i =  MIN(MAX(NINT(crism1(fov,Line)*15.0), 0), 15)        
         cris1d_prepro(8,fov1d) = IOR(cris1d_prepro(8,fov1d),i*16) !Bits 4-7 (light)
         i =  MIN(MAX(NINT(crism2(fov,Line)*15.0), 0), 15)        
         cris1d_prepro(8,fov1d) = IOR(cris1d_prepro(8,fov1d),i*256) !Bits 8-11 (moderate)
         i =  MIN(MAX(NINT(crism3(fov,Line)*15.0), 0), 15)        
         cris1d_prepro(8,fov1d) = IOR(cris1d_prepro(8,fov1d),i*4096)!Bits 12-15 (intense)

         DO i = 1, 13
            cris1d_emiss(i,fov1d) = MDI               ! reserved for emissivity
         ENDDO  ! i

!     Cloud flags
         DO i = 1, 10
            cris1d_cloudflag(i,fov1d) = crisCloudflag(i,fov1d,Line)
         ENDDO  ! i

      ENDDO  ! fov1d
             
      DO chn = 1, atmsnchan
        cris1d_nedtcoldatms(chn) = crisnedtcoldatms(chn,Line)*10000
        cris1d_nedtwarmatms(chn) = crisnedtwarmatms(chn,Line)*10000
      ENDDO

      RETURN
      END
