!+ Set up CRIS level 1D header within ATOVPP
!
      SUBROUTINE PPCOUTH (LATMS)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcouth.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Set up level 1D header using information from 1C headers.
!
! Method:
! 1.  Check format version number and data type
!     CRIS level 1C header has already been read into common /cris1chd/
!     Check that format of 1C & 1D include files (cris1c.h, cris1d.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1C file is compatible with this subroutine.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Set up level 1D header
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     12/08/05  Created from ppiouth.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  CRIS level 1D header/data common blocks 
#include "cris1d.h"
!  fixed data for processing CRIS 
#include "ppcrisfdata.h"
!  fixed data for processing ATMS 
#include "ppatmsfdata.h"
!  block of CRIS mapped data 
#include "ppcrisstore.h"
!  CRIS level 1C header/data common blocks 
#include "cris1c.h"
!  ATMS level 1C header/data common blocks 
#include "atms1c.h"
!
! Local parameters:
      INTEGER       cris1Cvnnb       ! version number HIRS 1C format compatible
         PARAMETER (cris1Cvnnb=1  )  !    with the code in this subroutine
      INTEGER       atms1Cvnnb       ! version number AMSU-A 1C format compatible
         PARAMETER (atms1Cvnnb=1  )  !    with the code in this subroutine
      INTEGER       cris1Dvnnb       ! version number 1D format compatible
         PARAMETER (cris1Dvnnb=1  )  !    with the code in this subroutine
!
!   Scalar arguments with intent(In):
      LOGICAL       LATMS         ! .TRUE. to process ATMS
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! general purpose loop variables
      INTEGER       chn           ! loop variable over channels
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 1.1 Are the 1C include files compatible with this source code ?
!     --- --- -- ------- ----- ---------- ---- ---- ------ ----

!     CRIS
      IF ( cris1Cvnnb .NE. cris_1cvnnb ) THEN
         errStatus = -111  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but cris1c.h is "",I6)")
     &        cris1Cvnnb, cris_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPCOUTH")
      ENDIF

!     ATMS
      IF ( LATMS .AND. atms1Cvnnb .NE. atms_1cvnnb ) THEN
         errStatus = -112  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but ama1c.h is "",I6)")
     &        atms1Cvnnb, atms_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPCOUTH")
      ENDIF


! 1.2 Is the 1D include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( cris1Dvnnb .NE. cris_1dvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1D format "",I6,"" but cris1d.h is "",I6)")
     &        cris1Dvnnb, cris_1dvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPCOUTH")
      ENDIF


! 2.  Set up level 1D header
!     --- -- ----- -- ------

! 2.1 General

!     creation site for 1D data
      cris1d_h_site   = SiteID              !file creation site ID (CHAR*3)
      cris1d_h_cfill1 = " "                 !filler

!     creation site for original 1B data
      cris1d_h_1bsite = cris1c_h_1bsite
      cris1d_h_cfill2 = " "                 !filler

      cris1d_h_versnb = cris_1dvnnb          !level 1d format version number
      cris1d_h_versyr = cris_1dvnyr          !level 1d format version year
      cris1d_h_versdy = cris_1dvndy          !level 1d format version day of year
      cris1d_h_hdrcnt = cris1c_h_hdrcnt      !count of header records
      cris1d_h_satid = cris1c_h_satid        !satellite id (e.g. 14 for NOAA-14)
      cris1d_h_grid = cris1c_h_instrument    !instrument grid code (CRIS=13)
      cris1d_h_satht = cris1c_h_satht        !nominal sat height, km*10
      cris1d_h_period = cris1c_h_period      !nominal orbit period (seconds)
      cris1d_h_startorbit = cris1c_h_startorbit !orbit number (at start)
      cris1d_h_startdatayr = cris1c_h_startdatayr!start of data set year
      cris1d_h_startdatady = cris1c_h_startdatady! (ditto) day of the year
      cris1d_h_startdatatime = cris1c_h_startdatatime !(ditto) UTC time of day,ms
      cris1d_h_endorbit = cris1c_h_endorbit  !orbit number (at end)
      cris1d_h_enddatayr = cris1c_h_enddatayr!end of data set year
      cris1d_h_enddatady = cris1c_h_enddatady!(ditto) day of the year
      cris1d_h_enddatatime = cris1c_h_enddatatime !(ditto) UTC time of day,ms
      cris1d_h_scnlin = cris1c_h_scnlin      !count of scan lines in this dataset
      cris1d_h_misscnlin = cris1c_h_misscnlin !count of missing scan lines
      cris1d_h_atovppvn = ATOVPPvn     !ATOVPP version number (test vns = 9000+)
      cris1d_h_instruments = crisinstruments   !instruments present
      cris1d_h_spare1 = MDI                !spare
      cris1d_h_wmosatid = cris1c_h_wmosatid  !WMO satellite id (3 for METOP-1)
      cris1d_h_centre = cris1c_h_centre      !WMO centre id
      cris1d_h_subcentre = cris1c_h_subcentre!WMO subcentre id
      cris1d_h_recwords = array_size_cris1d  !Size of record
      cris1d_h_nfov = 1                      !Number of FOVs in record
      cris1d_h_nfor = cris_1dnfor            !Number of FORs in record
      cris1d_h_nband = cris_1dnband          !Number of spectral bands
      DO i=1,cris1d_h_nband
        cris1d_h_startw(i) = cris1c_h_startw(i)
        cris1d_h_endw(i) = cris1c_h_endw(i)
        cris1d_h_startchan(i) = cris1c_h_startchan(i)
        cris1d_h_endchan(i) = cris1c_h_endchan(i)
      ENDDO

! 2.2 temperature-radiance conversion coefficients etc.

!     ATMS
      IF ( LATMS ) THEN
         cris1d_h_atmsbeamversion = atms1c_h_beamversion
         DO chn = 1, atmsnchan
         cris1d_h_atmsbeamwidth(chn) = atms1c_h_beamwidth(chn)
         cris1d_h_atmscutoff(chn)    = atms1c_h_cutoff(chn)
         cris1d_h_atmsnxaverage(chn) = atms1c_h_nxaverage(chn)
         cris1d_h_atmsnyaverage(chn) = atms1c_h_nyaverage(chn)
         DO i = 1, 3
            cris1d_h_atmsPlanck(i,chn) = atms1c_h_tempradcnv(i,chn)
         ENDDO  ! i
         ENDDO  ! chn
      ELSE
         cris1d_h_atmsbeamversion = MDI
         DO chn = 1, atmsnchan
         cris1d_h_atmsbeamwidth(chn) = MDI
         cris1d_h_atmscutoff(chn)    = MDI
         cris1d_h_atmsnxaverage(chn) = MDI
         cris1d_h_atmsnyaverage(chn) = MDI
         DO i = 1, 3
            cris1d_h_atmsPlanck(i,chn) = MDI
         ENDDO  ! i
         ENDDO  ! chn
      ENDIF

! 2.3 Pre-processing information

![user-defined:
      cris1d_h_dT  = dT         ! size of data batches processed within ATOVPP
      cris1d_h_mapBTmode = atms2crismode ! mapping mode
      cris1d_h_Acorrvn  = atmscorrvn        ! vn no ATMS limb/emiss correction
      cris1d_h_Apcpvn   = atmspcpvn         ! vn no ATMS precip/scat detection
      cris1d_h_Aantenna = atms1c_h_vnantennacorr ! vn ATMS antenna corrections
      cris1d_h_Ascat = atmsmaxscat !threshold value for AMSU-A scattering index
      cris1d_h_ACrosby = atmsmaxcrosby !threshold value for AMSU-A logistic precip
      cris1d_h_recperscan = nrec_cris  !1d records per scan (1 or 4)
      cris1d_h_fovmode = thinmode_cris
      DO i = 1, array_size_cris1dspare
         cris1d_h_spare2(i) = MDI        !spare
      ENDDO  ! i
! ... ]

! 2.4 CRIS variables

!      cris1d_h_eigvn is already filled (see ppifdf.F)
      cris1d_h_npc = cris_1dnpc        !PCs in format
      cris1d_h_npcused = npc_cris      !PCs used
      cris1d_h_nchan = cris_1dnchan    !Channels in format
      cris1d_h_nchanused = nchan_cris  !Channels used

      DO i=1,cris_1dnchan
        cris1d_h_channels(i) = channels_cris(i)
      ENDDO

! 2.5 VIIRS

      cris1d_h_nviirschan = cris_1dnchanviirs
      cris1d_h_nviirschanused = 0      !None at this stage

      RETURN
      END
