!+ CRIS cloud tests
!
      SUBROUTINE PPCRISCLOUD ( Testnumber, Clastline )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcriscloud.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!    NWP-independent cloud detection tests for CRIS
!
! Method:
!    This is identical to the tests for IASI.
!    Currently 2 tests are implemented:
!    1) Standard deviation of CRIS radiances for the 9 detectors at each scan 
!       position
!    2) Make use of mapped ATMS in order to simulate CRIS radiances
!    
!    Reference for both the above: Cheng et. al., 2006, 'The Use of Principal
!    Components Analysis (PCA) in Processing Simulated CRIS Data', ITSC-15,
!    poster A17
!
!    Results are stored in the crisCloudflag array, which has space for 10 cloud
!    tests. The last slot is used as a clear/cloudy flag (1 bit per test)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2007, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    17/08/2010 from ppicloud.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  CRIS 1c format 
#include "cris1c.h"
!  CRIS 1d format 
#include "cris1d.h"
!  block of CRIS level 1d data 
#include "ppcrisstore.h"
!  fixed data for processing CRIS 
#include "ppcrisfdata.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  constants 
#include "constants_preproc.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Testnumber    ! cloud test number
      INTEGER       Clastline     ! number of last CRIS line in store
                                  !  (or current line for test 1)
!
! Local scalars:
      REAL          radmean, radsum
      REAL          T4, T5, T6, csza, cview
      REAL          radiance, bri_temp, w, T, drdt, radsd, bt_pred
      INTEGER       scalrad(cris_1cnfov)
      INTEGER       fov, Ichan, ndet
      INTEGER       I,Idet,line     
!
!- End of header

!Update header record to indicate that this test has been performed
      cris1d_h_cloudtests=IBSET(cris1d_h_cloudtests,BITS(Testnumber-1))

      IF (Testnumber .eq. 1) THEN

!---------- Test 1: Standard deviation of 9 CRIS FOVs at 2390 cm^-1 -----------
!
!Reference: Cheng et. al., ITSC-15, poster A17
!
!This test must be done while the CRIS 1c are being read (see ppcris1d)
!------------------------------------------------------------------------------

        T = 280.0
        w = crisCloudW
        drdt = C_1*w*w*w/(exp(w*C_2/T)-1)**2 * exp(w*C_2/T) *C_2*w/(T*T)

        DO fov=1,cris_1dnfor
          radmean = 0.0
          ndet = 0
          DO Idet=1,9
            scalrad(Idet) = cris1c_radiance(Idet,fov,crisCloudindex)
            IF (scalrad(Idet).ne.0 .and. OKdetector_cris(Idet).eq.1)THEN
              ndet = ndet + 1
              radmean = radmean + scalrad(Idet)
            ENDIF
          ENDDO
          IF (ndet .ge. 2) THEN         !require at least 2 good detectors
            radmean = radmean/ndet
            radsd = 0.0
            DO Idet=1,9
              IF (scalrad(Idet).ne.0 .and.
     &                OKdetector_cris(Idet).eq.1) THEN
                radsd = radsd + (scalrad(Idet)-radmean)**2
              ENDIF
            ENDDO
            radsd = sqrt(radsd/(ndet-1))*1.0E-4/drdt

!Store 100*(std deviation referenced to 280K)

            crisCloudflag(Testnumber,fov,Clastline) = radsd*100

!Set master flag (word 10) if threshold is exceeded
!Assume NEdT at 280K is 0.42K

            IF (radsd .GT. 3*0.42) THEN
              crisCloudflag(10,fov,Clastline) =
     &         IBSET(crisCloudflag(10,fov,Clastline),BITS(Testnumber-1))
            ENDIF

          ELSE
            crisCloudflag(Testnumber,fov,Clastline) = MDI   !No CRIS
          ENDIF

        ENDDO

      ELSE IF (Testnumber .eq. 2) THEN       

!---------- Test 2: CRIS 2390 cm^-1 radiance predicted from AMSU -----------
!
!Reference: Cheng et. al., ITSC-15, poster A17
!------------------------------------------------------------------------------

        DO line = 1, Clastline
          DO fov=1,cris_1dnfor
            T4  = crismapbt(5,fov,line)   ! ATMS channel 5 = AMSU channel 4
            T5  = crismapbt(6,fov,line)
            T6  = crismapbt(7,fov,line)
            csza = COS(crisangles(3,fov,line)*deg2rad)    !Cos solar zenith angle
            cview = COS((fov-15.5)*3.33333*deg2rad)    !Cos scan angle
            IF (T4.GT.RMDI .AND. T5.GT.RMDI .AND. T6.GT.RMDI) THEN

              bt_pred = 44.007 - 0.272*T4 + 1.837*T5 - 0.738*T6
     &                  + 0.004*csza + 6.391*(1-cview)

!We already have the radiance (see ppcris1d)

              radiance = crisCloudflag(Testnumber,fov,line)*
     &              1.0E-4    ! mW m-2 sr-1 cm
              
              IF (radiance.GT.0.) THEN
                bri_temp = C_2*w / log(C_1*w*w*w/radiance + 1)

!Over-write crisCloudflag with the BT diff (*100)

                crisCloudflag(Testnumber,fov,line)=
     &                 (bt_pred - bri_temp)*100

!Set master flag (word 10) if threshold is exceeded

                IF (bt_pred - bri_temp .GT. 2.0) THEN
                  crisCloudflag(10,fov,line) =
     &              IBSET(crisCloudflag(10,fov,line),BITS(Testnumber-1))
                ENDIF

              ELSE
                crisCloudflag(Testnumber,fov,line) = MDI      !No valid CRIS
              ENDIF
     
            ELSE
              crisCloudflag(Testnumber,fov,line) = MDI      !No ATMS
            ENDIF
          ENDDO
        ENDDO
      
      ENDIF     !end test 2
      END

