!+ Read fixed data file for cris within ATOVPP
!
      SUBROUTINE PPCRISFDF (Cfunit,Ceigunit1,Ceigunit2,Cnoiseunit)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcrisfdf.F 635 2019-04-15 15:08:00Z frna $
!
! Purpose:
!   Read fixed data file required for CrIS within ATOVPP.
!
! Method:
! 1.  Read channel selection and thinning requirements
! 2.  Read eigenvectors for encoding (radiance to PC)
! 3.  Read eigenvectors for decoding (PC to radiance)
! 4.  Read noise profile, nedr (for generating reconstruction score)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 06 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    13/09/2010 From ppifdf.F
!  1.1    20/06/2012 Shorten name of "Use_reconstructed_radiances_cris" to less
!                    than 32 characters. NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1d format 
#include "cris1d.h"
!  fixed data for processing cris 
#include "ppcrisfdata.h"
!  fixed data for processing cris 
#include "reclopt.h"
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       Cfunit        ! unit numbers
      INTEGER       Ceigunit1
      INTEGER       Ceigunit2
      INTEGER       Cnoiseunit
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status value
      INTEGER       chn           ! loop index: channel number
      INTEGER       Ipc           ! loop index: PC number
      INTEGER       nchan_max     ! number of chans in eigenvector file
      INTEGER       npc_max       ! number of PCs in eigenvector file
      REAL*4        tmp(crisnchan)   ! temporary storage for eigenvectors

! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine

!- End of header


! 1.  open and read fdf file
!     ----------------------

      OPEN ( UNIT=Cfunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=Cfunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

      WRITE ( errMessage,
     & "("" Error opening cris fixed data file, unit "",I8)" ) Cfunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPCRISFDF")


!     look for sections in the file
!     ---- --- -------- -- --- ----

      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )
        input_line = " "
        READ(Cfunit,"(A80)",IOSTAT=errStatus) input_line

        IF ( input_line(1:5) .EQ. "START" ) THEN

!     Read file
!     ---- ----

          READ(Cfunit,*) thinmode_cris !Thinning mode: 0 = full resolution.
                                       !               1 = fixed detector(s)
                                       !               2 = warmest CrIS
                                       !               3 = hybrid 2&1
                                       !               4 = fixed sequence

          READ(Cfunit,*) nrec_cris    !records per scan line

          IF (thinmode_cris .EQ. 0) THEN
            READ(Cfunit,*) detector_use_cris(1)    !Not used for full res
          ELSE
            READ(Cfunit,*) (detector_use_cris(chn),chn=1,nrec_cris)
          ENDIF
          default_cris = detector_use_cris(1)

          IF (thinmode_cris .EQ. 0) THEN  !Over-write values if full res
            nrec_cris = 9       !Number of 1d records to output per scan
            DO chn=1,9
              detector_use_cris(chn) = chn
            ENDDO
          ENDIF

          READ(Cfunit,*) (OKdetector_cris(chn), chn=1,9) !status of the 9 detectors

          READ(Cfunit,*) latlimits_cris(1),latlimits_cris(2) !Latitude limits for option3

          READ(Cfunit,*) crischan      !cris channel for option 2

          READ(Cfunit,*) npc_cris      !Number of PC scores to output in 1d

          IF (npc_cris .GT. crismaxnpc) THEN
            WRITE (*,*) "Warning: cris fdf file specifies too many PCs"
            WRITE (*,*) "Maximum is crismaxnpc=", crismaxnpc
            WRITE (*,*) "Modify ppparms.h to increase and recompile"
            npc_cris = crismaxnpc
          ENDIF
          
          READ(Cfunit,*) chanvn_cris           !Channel selection version number
          cris1d_h_chanvn = chanvn_cris
          READ(Cfunit,*) nchan_cris       !Number of channels to output in 1d

          READ(Cfunit,*) (channels_cris(chn), chn=1,nchan_cris)

        ENDIF
      ENDDO

      CLOSE(Cfunit)

!Read environment variable ATOVPP_USE_RR
      
      CALL GETENV ('ATOVPP_USE_RR',input_line)
      IF (input_line(1:1) .EQ. 'T') THEN
        Use_reconstructed_rad_cris = .TRUE.
        WRITE(*,'(10X,A)') "Use reconstructed radiances"
      ELSE
        Use_reconstructed_rad_cris = .FALSE.
      ENDIF

! 2.  Read eigenvector file for encoding (direct access binary)
!     ----------------------------------

      IF (npc_cris .GT. 1) THEN

       IF (Ceigunit1 .GT. 0) THEN
        OPEN ( UNIT=Ceigunit1, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=crisnchan*4/rldiv, IOSTAT=errStatus )

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Ceigunit1, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=crisnchan*4/rldiv, IOSTAT=errStatus )

        WRITE ( errMessage,
     &   "("" Error opening cris eigenvector encode file, unit "",I8)" )
     &   Ceigunit1
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPCRISFDF")

        READ(Ceigunit1,rec=1) eigvn_cris,nchan_max,pccount_cris,
     &      eigformat_cris,evlevel_cris,
     &      startpcchan_cris,endpcchan_cris,startpc_cris,endpc_cris,
     &      evfile_cris   !Read header record
        npcbands_cris = 3
        IF (startpcchan_cris(3).EQ.0) npcbands_cris=2
        IF (startpcchan_cris(2).EQ.0) npcbands_cris=1
        IF (startpcchan_cris(1).EQ.0) THEN
          startpcchan_cris(1) = 1
          endpcchan_cris(1) = nchan_cris
          startpc_cris(1) = 1
          endpc_cris(1) = min(npc_cris,pccount_cris)
        ENDIF
        
        IF (nchan_max.LT.crisnchan) THEN
          WRITE ( errMessage,
     &     "("" Not enough channels in eigenvector file"")" )
          errStatus = 99
          CALL ErrorReport(errStatus,errMessage,"PPCRISFDF")
        ENDIF
          
        cris1d_h_eigvn = eigvn_cris       !Store version number of eigenvectors

        IF (eigformat_cris.EQ.1) THEN
          READ(Ceigunit1,rec=2) mean_encode_cris
        ELSE
          eigformat_cris = 0   !Only allow 0 and 1
        ENDIF

        IF (pccount_cris.LT.npc_cris) THEN
          WRITE (*,*) "WARNING: number of PCs requested is more than "//
     &       "number of eigenvectors in file",npc_cris,pccount_cris
          npc_cris = pccount_cris
        ENDIF

        DO Ipc=1,npc_cris
          READ(Ceigunit1,rec=Ipc+eigformat_cris+1) tmp
          DO chn=1,crisnchan
            eig_encode_cris(chn,Ipc) = tmp(chn)
          ENDDO
        ENDDO
      
        IF (evlevel_cris.NE.'1C') evlevel_cris = '1B'   !Only 1B and 1C supported

        CLOSE(Ceigunit1)
       ENDIF

! 3.  Read eigenvector file for decoding (direct access binary)
!     ----------------------------------

       IF (Ceigunit2 .GT. 0) THEN
        OPEN ( UNIT=Ceigunit2, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=crisnchan*4/rldiv, IOSTAT=errStatus )

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Ceigunit2, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=crisnchan*4/rldiv, IOSTAT=errStatus )

        WRITE ( errMessage,
     &   "("" Error opening cris eigenvector decode file, unit "",I8)" ) 
     &   Ceigunit2
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPCRISFDF")

        READ(Ceigunit2,rec=1) eigvn_cris,nchan_max,pccount_cris,
     &      eigformat_cris,evlevel_cris,
     &      startpcchan_cris,endpcchan_cris,startpc_cris,endpc_cris,
     &      evfile_cris   !Read header record
        npcbands_cris = 3
        IF (startpcchan_cris(3).EQ.0) npcbands_cris=2
        IF (startpcchan_cris(2).EQ.0) npcbands_cris=1
        IF (startpcchan_cris(1).EQ.0) THEN
          startpcchan_cris(1) = 1             !Default is all channels together
          endpcchan_cris(1) = nchan_cris
          startpc_cris(1) = 1
          endpc_cris(1) = min(npc_cris,pccount_cris)
        ENDIF

        IF (eigvn_cris.ne.cris1d_h_eigvn) THEN
          WRITE(*,*) "WARNING: eigenvector version number for " //
     &       "decoding differs from encoding",eigvn_cris,cris1d_h_eigvn
        ENDIF

        IF (eigformat_cris.EQ.1) THEN
          READ(Ceigunit2,rec=2) mean_decode_cris
        ELSE
          eigformat_cris = 0   !Only allow 0 and 1
        ENDIF

        IF (pccount_cris.LT.npc_cris) npc_cris = pccount_cris

        DO Ipc=1,npc_cris
          READ(Ceigunit2,rec=Ipc+eigformat_cris+1) tmp
          DO chn=1,crisnchan
            eig_decode_cris(chn,Ipc) = tmp(chn)
          ENDDO
        ENDDO
      
        IF (evlevel_cris.NE.'1C') evlevel_cris = '1B'   !Only 1B and 1C supported

        CLOSE(Ceigunit2)
       ENDIF

! 4.  Read nedr file (direct access binary)
!     --------------

       IF (Cnoiseunit .GT. 0) THEN
        OPEN ( UNIT=Cnoiseunit, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=crisnchan*4/rldiv, IOSTAT=errStatus )

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Cnoiseunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=crisnchan*4/rldiv, IOSTAT=errStatus )

        WRITE ( errMessage,
     &   "("" Error opening cris eigenvector file 2, unit "",I8)" ) 
     &   Cnoiseunit
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPCRISFDF")

        READ(Cnoiseunit,rec=1) nedr_cris
      
        CLOSE(Cnoiseunit)
       ENDIF

      ENDIF         ! npc_cris .GT. 1

!Initialise cloud tests flag
      cris1d_h_cloudtests = 0

      RETURN
      END
