!+ Read & store level 1C CRIS data within ATOVPP
!
      SUBROUTINE PPCRISGET ( cris1Cunit, cris1Dunit, crisLastLine,
     &                    Year1, Day1, Time1,
     &                    Year2, Day2, Time2 )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcrisget.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Update ATOVPP store of CRIS scanlines to hold data within
!   start and end dates/times specified (Year1/Day1/Time1 & Year2/Day2/Time2)
!   with an extra 1 or 2 scanlines at either end to help with the mapping.
!
! Method:
! 1.  Look for first suitable scanline in previous block
! 2.  Define record number to start reading from
! 3.  Set data store to all "missing"
! 4.  Read data till we come to end of time interval, or of dataset
!
!   The position of each scanline in the program store ("ppcrisstore.h") is
!   determined solely by the date/time of the scanline number, relative to
!   that of the first line in store.  Thus the routine can cope with missing
!   scanlines, but can potentially be upset if the first line in store has a
!   corrupt date/time.
!
! Note:
!   "crisLastLine" should be initialised as 0 before first calling this routine.
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0   16/08/2010  Created from ppatmsget.F
!  
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"
! block of CRIS data
#include "ppcrisstore.h" 
! CRIS fixed data 
#include "ppcrisfdata.h"
! CRIS level 1C header/data common blocks
#include "cris1c.h"
! CRIS level 1D header/data common blocks
#include "cris1d.h"
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"    
!
!   Scalar arguments with intent(In):
      INTEGER       cris1Cunit       ! logical unit no. for CRIS 1C input
      INTEGER       cris1Dunit       ! logical unit no. for CRIS 1D output
      INTEGER       crisLastLine     ! number of last line stored in store
      ! Date/Time data block is to start from:
      INTEGER       Year1         !     year (e.g. 1995)
      INTEGER       Day1          !     day of year
      INTEGER       Time1         !     time (milliseconds)
      ! Date/Time data block is to end:
      INTEGER       Year2         !     year (e.g. 1995)
      INTEGER       Day2          !     day of year
      INTEGER       Time2         !     time (milliseconds)
!
! Local scalars:
      CHARACTER*80  errMessage    ! for calls to ErrorReport
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      INTEGER       border        ! for overlaps of data blocks (millisecs)
      INTEGER       dTime         ! time difference (milliseconds) from TimeSub
      INTEGER       errTime       ! error status from TimeSub
      INTEGER       cris1Crecord  ! record number for CRIS level 1C file
      INTEGER       fov           ! loop variable over all fov's
      INTEGER       line          ! line number within data store
      INTEGER       no_data       ! bit word set to "missing/bad data"
      INTEGER       scnlin0       ! scanline number of "zeroth" line in store
      INTEGER       scnlinDT      ! scanline number derived from date/time
      INTEGER       scnlin        ! scanline number
      INTEGER       year          ! year number of scanline
      INTEGER       day           ! day number of scanline
      INTEGER       time          ! time of scanline (ms)
      INTEGER       scnlinOld     ! previous scanline number
      INTEGER       yearOld       ! year number of previous scanline
      INTEGER       dayOld        ! day number of previous scanline
      INTEGER       timeOld       ! time of previous scanline (ms)
      INTEGER       crisrec       ! loop index for records
      INTEGER       cris1Drecord  ! CRIS 1d record (1 to 9)
      INTEGER       scnlinOffset  ! difference between scnlin and scnlinDT
!
! Function & Subroutine calls:
      External TimeSub              ! Subtract two dates/times
      External IOC1C                ! read CRIS 1C scan line
      External IOC1D                ! write CRIS 1D scan line
      External PPCRISIND            ! store scanline data in common/ppcrisstore/
      External ErrorReport          ! ATOVPP error routine
!
!- End of header


! 1.  Look for first suitable scanline in previous block
!     ---- --- ----- -------- -------- -- -------- -----

      scnlinOffset = 0
      line = crisLastLine + 1
      dTime   = -999
      border = 0             ! block for final targ instr does not need border

!     (stop looking once scanlines in block are earlier than start time)
      DO WHILE (line.GT.1 .AND. dTime.LT.border)

         line = line - 1
         errTime = 1
         IF ( crisrecdno(line).NE.MDI ) THEN    ! valid line
            CALL TimeSub ( Year1,       Day1,       Time1,
     &                  crisyear(line), crisday(line), cristime(1,line),
     &                  dTime, errTime)  ! dTime = start time - time of scan
!         errTime returned as non-zero if input is out of range (e.g. missing)
         ENDIF
         IF (errTime.NE.0) dTime = -999   ! go on looking

      ENDDO


! 2.  Define record number to start reading from
!     ------ ------ ------ -- ----- ------- ----

      IF ( dTime.GE.0 ) THEN
!        we've already read suitable data  - get record number from that
         cris1Crecord = crisrecdno(line)
      ELSE
!        no suitable scanlines already in block  - try next record
         IF ( crisLastLine.GT.0 ) THEN
            cris1Crecord = crisrecdno(crisLastLine) + 1
         ELSE  ! this is the first data to be read
            cris1Crecord = crishdrcnt + 1  !first data record in 1C file
         ENDIF
      ENDIF


! 3.  Set data store to all "missing"
!     --- ---- ----- -- ---  -------

!     Throughout this program, crisrecdno() should be used to check
!     whether a scanline is present or not, and crisqc(bits(0))
!     to check whether the data is usable.

      no_data = 0
      no_data = IBSET ( no_data, bits(0) )

      DO line = 1, crisnlines
         crisrecdno(line) = MDI
         DO fov = 1, crisnfovs
            crisqc(fov,line) = no_data
         ENDDO  ! fov
      ENDDO  ! line


! 4.  Read data till we come to end of time interval, or of dataset
!     ---- ---- ---- -- ---- -- --- -- ---- --------  -- -- -------

      crisLastLine = 0  ! i.e. we have no data in storage
      errTime = 1
      dTime   = -999
      scnlin0 = MDI  ! scanline number of "zeroth" scanline in store

! 4.1 Loop through scanlines in dataset
      DO WHILE ( cris1Crecord.LE.(crishdrcnt+crisscnlin) .AND. 
     &   dTime.LT.border )

!        Read a scan line
         CALL IOC1C ("READ DATA", cris1Cunit, cris1Crecord)
         scnlin = cris1c_scnlin
         year = cris1c_scnlinyr
         day  = cris1c_scnlindy
         time = cris1c_scnlintime(1)

        IF ( cris1c_scnlin.GE.1 .AND. cris1c_scnlin.LE.9999
     &    .AND. time .GE. 0) THEN

! 4.2    Calculate the scanline number from the date/time (to check)
         IF ( scnlin0 .NE. MDI ) THEN   ! not the 1st record read this call

            CALL TimeSub ( year, day, time,
     &                     yearOld,dayOld, timeOld, dTime, errTime )
            errMessage = " "
            CALL ErrorReport(errTime,errMessage,"PPCRISGET")

!           estimated scanline number
            scnlinDT = scnlinOld + NINT(REAL(dTime)/criscycle)

!           Complain if scanline number and estimated scanline number differ
            IF ( scnlinDT.NE.scnlin+scnlinOffset ) THEN
               errMessage = " "
               WRITE ( errMessage,
     &      "(""Scanline numbers and times inconsistent, lines"",3I9)")
     &          scnlinOld, scnlin, scnlinDT
               errStatus = -42  ! warning
               CALL ErrorReport(errStatus,errMessage,"PPCRISGET")
               scnlinOffset = scnlinDT - scnlin
            ENDIF

         ENDIF  ! not the 1st record read

! 4.3    Store scanline in slot "line" in common/ppcrisstore/

!        Calculate slot to put line into
         IF ( scnlin0 .EQ. MDI ) THEN   ! first record to be read this call
            scnlin0 = scnlin - 1
            line = 1
         ELSE
            line = scnlinDT - scnlin0   ! use estimated line number, not scnlin
         ENDIF

         IF ( line.LE.crisnlines ) THEN  ! there's room
            crisLastLine = line
            CALL PPCRISIND ( line )
            crisrecdno(line) = cris1Crecord
         ENDIF

! 4.4    save line-number/date/time to check against next scanline
!         scnlinOld = scnlin
         IF ( line .EQ. 1 ) THEN
           scnlinOld = scnlin
         ELSE
           scnlinOld = scnlinDT
         ENDIF
         yearOld = year
         dayOld  = day
         timeOld = time

! 4.5    Calculate dTime := date/time of scan - end date/time of block
         CALL TimeSub ( year, day, time,
     &                  Year2,Day2,Time2,  dTime, errTime )
!        errTime returned as non-zero if input is out of range (e.g. missing)
         IF (errTime.NE.0) dTime = -999   ! go on looking

! 4.6    Process the CRIS scan line. Data are in common block in cris1c.h.
!        For full resolution data loop round the 9 CRIS detectors, otherwise
!        select the 'best' or the specified detector for each scan position.

         IF ( line.LE.crisnlines ) THEN  ! If there's room in storage arrays
         
           CALL PPcrisThin(line)        !Determine the 'best' detector

           DO crisrec = 1, nrec_cris
             CALL PPcris1D(crisrec,line)!Copy data to 1d common block and do PCI
                                        !analysis if required

! 4.7    Write out CRIS data as level 1d
!        Each 1c record has "nrec_cris" 1d records (ppcrisfdata.h)
!        Both also have a header record.

             cris1Drecord = nrec_cris*(crisrecdno(line)-2)+1+crisrec
             CALL IOC1D ("WRITE DATA", cris1Dunit, cris1Drecord)

           ENDDO    ! crisrec
         ENDIF    ! If there's room in storage arrays
        ENDIF    ! if scanline OK

!       Increment record number for next read
        cris1Crecord = cris1Crecord + 1

      ENDDO   ! WHILE ( cris1Crecord.LE.(crishdrcnt+crisscnlin) .AND. dTime.LT.border )

      WRITE(*,"(10X,""CRIS line numbers "",I4,"" to "",I4,
     &        "", with "",I3,"" missing lines"")")
!     &        crislineno(1), crislineno(crisLastLine),
     &        crislineno(1), scnlinOld,
     &       (scnlinOld-crislineno(1))-(crisLastLine-1)


      RETURN
      END
