!+ Transfer level 1C CRIS data from data record to program store
!
      SUBROUTINE PPCRISIND ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcrisind.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   CRIS 1C data, when read, is held in common/cris1cdt/. ATOVPP stores data
!   from several scanlines in its own internal array structure, contained
!   in common/ppcrisstore/. This routine transfers a single scanline from
!   common/cris1cdt/ to slot "Line" in common/ppcrisstore/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1C include file (cris1c.h) and CRIS store
!     (ppcrisstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Copy variables from common/cris1cdt/ to slot "Line" in common/ppcrisstore/
! 2.  Check flags
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     16/08/10  N.C.Atkinson, based on ppiind.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of CRIS data 
#include "ppcrisstore.h"
!  CRIS level 1C header/data common blocks 
#include "cris1c.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppcrisstore/
!
! Local parameters:
      INTEGER       cris1Cvnnb       ! version number of 1C format compatible
         PARAMETER (cris1Cvnnb=1)    ! with the code in this subroutine
      INTEGER       crisstvnnb       ! version number of "ppcrisstore.h" compatible
         PARAMETER (crisstvnnb=1)    ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chanbits      ! bit word with all channel bits set
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       det           ! detector
      INTEGER       pos           ! scan position
      INTEGER       qc0           ! qc flagword, bit 0 set
      LOGICAL       badline       ! .TRUE. not to use a scanline
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first, chanbits
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!        Is the 1C include file compatible with this source code ?
         IF ( cris1Cvnnb .NE. cris_1cvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but cris1c.h is "",I6)")
     &           cris1Cvnnb, cris_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPCRISIND")
         ENDIF

!        Is the ppcrisstore include file compatible with this source code ?
         IF ( crisstvnnb .NE. cris_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &           "("" Code expects Istore format"",I6,"" but " //
     &           "ppcrisstore.h is"",I6)")
     &           crisstvnnb, cris_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPCRISIND")
         ENDIF

!        set up a mask to pick out just the flags for the individual detectors
         chanbits = 0
         DO chn = 1, crisndet
            chanbits = IBSET ( chanbits, bits(chn) )
         ENDDO

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from common/cris1cdt/ to slot "Line" in common/ppcrisstore/
!     ---- --------- ---- --------------- -- ----  ----  -- ----------------

      crislineno(Line) = cris1c_scnlin                ! scanline number
      crisyear(Line) = cris1c_scnlinyr                ! scanline year
      crisday(Line) = cris1c_scnlindy                 ! scanline day of year
      
      DO fov = 1, crisnfor
         cristime(fov,Line) =  cris1c_scnlintime(fov) ! scanline time (ms)
      ENDDO

      DO fov = 1, crisnfovs      !store 270 fovs, 1-30=detector 1, etc. 
         det = (fov-1)/30 + 1   !detector
         pos = MOD(fov-1,30) + 1  !scan position
         crislats(fov,Line) = REAL(cris1c_latlon(1,det,pos))*1.E-4 !latitude(deg,N+ve)
         crislons(fov,Line) = REAL(cris1c_latlon(2,det,pos))*1.E-4 !longitude (E+ve)
         DO i = 1, 4
            crisangles(i,fov,Line) = REAL(cris1c_angles(i,det,pos))*0.01 ! scan angles:
!                                         (1,*,*) local zenith
!                                         (2,*,*) local azimuth
!                                         (3,*,*) solar zenith
!                                         (4,*,*) solar azimuth
         ENDDO  ! i
      ENDDO  ! fov

      DO fov = 1, crisnfovs
      DO chn = 1, atmsnchan
         crismapbt(chn,fov,Line) = RMDI
      ENDDO  ! chn
      ENDDO  ! fov

      DO fov = 1, crisnfovs
         crisqc(fov,Line) = 0  ! QC word for CRIS
      ENDDO  ! fov

!     initialise other variables
      qc0 = 0
      qc0 = IBSET(qc0,bits(0)) ! QC word: "AMSU missing"
      DO fov = 1, crisnfovs
         crismapsurf(fov,Line)  = MDI   ! estimated ATMS surface type
         crissurfcost(fov,Line) = RMDI  ! ATMS max prob cloud cost
         crisscatindx(fov,Line) = RMDI  ! ATMS scattering index
         criscrosby(fov,Line)   = RMDI  ! ATMS logistic precip probability
         crismapqc(fov,Line)    = qc0   ! ATMS QC word
      ENDDO  ! fov

! 2.  Check flags
!     ----- -----

! 2.1 Flags for the whole scanline

      badline = .FALSE.       !There are no whole-scanline flags at present

!     For bad scanline, flag each fov
      IF ( badline ) THEN
         DO fov = 1, Infovs
            crisqc(fov,Line) = IBSET ( crisqc(fov,Line), bits(0) )
         ENDDO  ! fov
      ENDIF


      RETURN
      END
