!+ Read & interpret CRIS level 1C header within ATOVPP
!
      SUBROUTINE PPCRISINH ( cris1Cunit, SatId, WMOSatId,
     &                    StartYearcris, StartDaycris, StartTimecris,
     &                    EndYearcris,   EndDaycris,   EndTimecris )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcrisinh.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose:
!   Open input dataset and read level 1C CRIS header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     CRIS level 1C header has already been read into common /cris1chd/
!     Check that format of 1C include files (cris1c.h) is compatible with the
!     code in this subroutine. Check that format of 1C file is compatible with
!     this subroutine, and that data type from the header is 10 for CRIS.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from CRIS 1C header
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0   16/08/2010  Created from ppainh.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! block of CRIS data 
#include "ppcrisstore.h"     
! CRIS level 1C header/data common blocks
#include "cris1c.h"           
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
!
! Local parameters:
      INTEGER       cris1Cvnnb       ! version number of 1C format compatible
         PARAMETER (cris1Cvnnb=1)    ! with the code in this subroutine
!
!     Alternative way of checking that version numbers match (at compile time)
!      REAL VNTEST1(cris1Cvnnb-cris_1cvnnb+1), VNTEST2(cris_1cvnnb-cris1Cvnnb+1)
!
!   Scalar arguments with intent(In):
      INTEGER       cris1Cunit       ! logical unit no. for CRIS 1C input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 3 for METOP-1)
      ! start/end date/time of CRIS data:
      INTEGER       StartYearcris, EndYearcris
      INTEGER       StartDaycris,  EndDaycris
      INTEGER       StartTimecris, EndTimecris
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       cris1Crecord  ! record number for CRIS level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOC1C                ! I/O routine for level 1C CRIS data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.0 Find the record length
      cris1c_h_recwords = array_size_cris1c - 
     &      cris_1cnchan*cris_1cnfov*cris_1cnfor   !excluding channels
      CALL IOC1C ("OPEN/READ", cris1Cunit, 0 )
      CALL IOC1C ("READ HEADER", cris1Cunit, 1 )
      CALL IOC1C ("CLOSE", cris1Cunit, 0)

! 1.1 Open 1C file with correct record length
      CALL IOC1C ("OPEN/READ", cris1Cunit, 0 )   ! 1C file

! 1.2 Read 1C header
      cris1Crecord = 1   ! Headers start on record 1.
      CALL IOC1C ("READ HEADER", cris1Cunit, cris1Crecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( cris1Cvnnb .NE. cris_1cvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but cris1c.h is "",I6)")
     &        cris1Cvnnb, cris_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPCRISINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "cris1c.h" contains its own version number "cris1c_h_versnb"
      IF ( cris1c_h_versnb .NE. cris_1cvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level 1C CRIS file is "",I8,"", not "",I8)")
     &        cris1c_h_versnb,cris_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPCRISINH")
      ENDIF

! 2.3 Is the 1C data CRIS ?
!     -- --- -- ---- ------
!     "cris1c_h_instrument" from 1C header should be 620 for CRIS
      IF ( cris1c_h_instrument .NE. 620 ) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage, "("" Data type in 1C CRIS file is "",
     &           I8,""; not 620 for CRIS"")") cris1c_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "PPCRISINH")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "##########"
      WRITE(*,*) "#  CRIS  #"
      WRITE(*,*) "##########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            cris1c_h_versnb, cris1c_h_versyr, cris1c_h_versdy
      WRITE(*,"(I4,"" header records in file."")") cris1c_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") cris1c_h_satid

!     calculate start and end dates/times from header
!     ..start
      year = cris1c_h_startdatayr
      yrday = cris1c_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(cris1c_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = cris1c_h_enddatayr
      yrday = cris1c_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(cris1c_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          cris1c_h_scnlin, cris1c_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from CRIS 1C header
!     ------- -------- ----------- ---- ---- -- ------

      crisSatId = cris1c_h_satid
      SatId = crisSatId
      WMOSatId = cris1c_h_wmosatid
      StartDaycris = cris1c_h_startdatady
      StartYearcris = cris1c_h_startdatayr
      StartTimecris = cris1c_h_startdatatime
      EndDaycris = cris1c_h_enddatady
      EndYearcris = cris1c_h_enddatayr
      EndTimecris = cris1c_h_enddatatime
      crissatht = REAL(cris1c_h_satht)*0.1      !nominal sat height (km)
      crissatperiod = REAL(cris1c_h_period)/60. !nominal orbit period (minutes)
      crishdrcnt = cris1c_h_hdrcnt       !count of header records in 1C file
      crisscnlin = cris1c_h_scnlin       !count of scanlines in 1C file
      crisinstruments = IBSET ( crisinstruments , bits(8) )   ! CRIS data present


      RETURN
      END
