!+ CRIS spectral analysis
!
      SUBROUTINE PPCRISSPECTRA ( Det, Ifor )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcrisspectra.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose: Compute PC scores for CRIS and store in level 1d
!
! Method:
! 1. Convert from scaled radiance to true radiance
! 2. Noise normalise
! 3. Compute PC scores using reference eigenvectors
! 4. Compute reconstructed radiances
! 5. Compute reconstruction score for each detector, noise normalised.
!    Ref: Goldberg et. al., IEEE Trans.Geosci.Rem.Sens., vol 41 no 2, Feb 2003
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0  17/08/2010  from ppispectra.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1c format 
#include "cris1c.h"
!  level 1d format 
#include "cris1d.h"
!  fixed data for processing CRIS 
#include "ppcrisfdata.h"
!
!   Scalar arguments with intent(In):

      INTEGER Det        !Detector (1 to 9)
      INTEGER Ifor       !FOV (1 to 30)

!   Local scalars:

      REAL Radiance(crisnchan)      !Radiance (mW/m^2/str/cm^-1)
      REAL NormRadiance(crisnchan)  !Noise normalised radiance
      REAL scaling               !Scaling factor
      REAL Recon                 !Reconstructed radiance (temporary)
      REAL Reconstructed(crisnchan) !Reconstructed radiances
      REAL Score                 !PC score (temporary)
      REAL Scores(cris_1dnpc)    !PC scores
      INTEGER I                  !Loop index
      INTEGER chn                !Channel index
      INTEGER Ipc                !PC index
      INTEGER iband
      INTEGER counts
      LOGICAL Done
      LOGICAL badcounts
      LOGICAL allzero
!
!- End of header      

! 1. Convert from scaled radiance to true radiance
!    ---------------------------------------------

! Original 1c radiances are in W/(m^2 str m^-1)*1E7. Multiply by a factor 1E-4
! to convert to mW/(m^2 str cm^-1).

      Done = .FALSE.
      badcounts = .FALSE.
      allzero = .TRUE.
      scaling = 1.0E-4
      DO chn=1,cris_1cnchan
        counts = cris1c_radiance(Det,Ifor,chn)
        IF (counts.EQ.MDI) THEN
          badcounts=.true.
        ELSE
          Radiance(chn) = counts*scaling
          IF (counts.NE.0) allzero = .FALSE. 
        ENDIF
      ENDDO
      
      IF (.NOT.badcounts.AND..NOT.allzero) THEN

! 2a. Noise normalise
!    ---------------

        DO chn=1,crisnchan
          NormRadiance(chn) = Radiance(chn)/nedr_cris(chn)
        ENDDO

! 2b. Subtract mean

        DO chn=1,crisnchan
          NormRadiance(chn) = NormRadiance(chn)-mean_encode_cris(chn)
        ENDDO

! 3. Compute PC scores
!    -----------------      

        DO iband=1,npcbands_cris
          DO Ipc=startpc_cris(iband),endpc_cris(iband)
            Score = 0.0
            DO chn=startpcchan_cris(iband),endpcchan_cris(iband)
              Score = Score + eig_encode_cris(chn,Ipc)*NormRadiance(chn)
            ENDDO
            cris1d_scores(Ipc,Ifor) = NINT(Score*2)  !Quantise at half NeDR
            Scores(Ipc) = cris1d_scores(Ipc,Ifor)/2.0
          ENDDO
        ENDDO

! 4. Compute reconstructed radiance (self apodised, noise normalised)
!    ------------------------------

        DO chn=1,crisnchan
          Recon = 0.0
          DO Ipc=1,npc_cris
            Recon = Recon + 
     &         eig_encode_cris(chn,Ipc)*Scores(Ipc)
          ENDDO
          Reconstructed(chn) = Recon
        ENDDO

! 5. Compute reconstruction score for each detector
!    ----------------------------------------------

        DO I=1,3
          Score = 0.0
          DO chn=cris1c_h_startchan(I),cris1c_h_endchan(I)
            Score = Score + 
     &       (Reconstructed(chn)-NormRadiance(chn))**2
          ENDDO
          Score = sqrt(Score/(cris1c_h_endchan(I) -
     &                        cris1c_h_startchan(I) + 1))
          cris1d_logfit(I,Ifor) = NINT(LOG10(Score)*1E4)  !10^4 times log score
                                                     !See BUFR descriptor 025052
        ENDDO

      ELSE        !if there are some missing scaled radiances

        DO Ipc=1,npc_cris
          cris1d_scores(Ipc,Ifor) = MDI
        ENDDO
        DO I=1,3
          cris1d_logfit(I,Ifor) = MDI
        ENDDO

      ENDIF

      RETURN
      END
