!+ CrIS thinning
!
      SUBROUTINE PPCRISTHIN ( line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppcristhin.F 547 2017-08-24 11:49:31Z frna $
!
! Purpose:
!   For each spot, choose 1 detector out of 9 for inclusion in the 1d output
!
! Method:
!   The following options are provided, via CRIS.fdf:
!   0) No thinning - include all 9 detectors
!   1) Fixed CRIS detector(s) (single detector or fixed combination)
!   2) Highest mean CRIS radiance for defined channel
!   3) Hybrid - scheme 1 in polar regions, scheme 2 elsewhere. Latitude
!      transition is defined in CRIS.fdf.
!   4) Fixed sequence of all 9 detectors (1 per scan line)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2007, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     04/07/11  N.C.Atkinson, based on ppithin.F
!  1.1     18/09/15  Bug fix: thin mode 2 and 3 were not working correctly
!  1.2     24/08/17  Get option 2 working if the 1c dataset has already been
!                    spectrally thinned.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  CRIS 1d format 
#include "cris1d.h"
!  block of CRIS data 
#include "ppcrisstore.h"
!  fixed data 
#include "ppcrisfdata.h"
!  CRIS level 1C header/data common blocks 
#include "cris1c.h"
!
!   Scalar arguments with intent(In):

      INTEGER line       !Scan line in ppistore

!   Local scalars:

      INTEGER   Cdet, Cfov       !Loop indices
      INTEGER   bestdet          !Best detector
      INTEGER   thinmode         !Copy of thinmode_CRIS
      INTEGER   cmaxrad          !Integer maximum radiance
      INTEGER   rad              !radiance
      INTEGER   FOVorder(9)      !For use with option 4
      INTEGER   crischan_index   !position of crischan in the input dataset
      DATA      crischan_index/1/
      INTEGER   bestchandiff
      INTEGER   chn, i
      DATA FOVorder /1,2,3,4,5,6,7,8,9/
      REAL      radiance, radsd, maxrad, minrad       !Radiance-derived values
      REAL      mean_radiance, raddiff, bestraddiff
      REAL      Lat              !Latitude
      LOGICAL   First
      DATA First/.true./
!
!- End of header

      IF (First) THEN
        First = .false.
        bestchandiff = 99999
        DO chn=1,cris1c_h_nchanused
          i = abs(cris1c_h_channels(chn)-crischan)
          IF (i .lt. bestchandiff) then
            crischan_index = chn
            bestchandiff = i
          ENDIF
        ENDDO
      ENDIF

      IF (nrec_CRIS .EQ. 1) THEN        !Only if thinning is required
        
!Thinning option 1: fixed detector for all FOVs and all scan lines
!Thinning option 4: fixed detector for all FOVs within a scan line

        IF (thinmode_CRIS .EQ. 1. OR. thinmode_CRIS .EQ. 4) THEN
          IF (thinmode_CRIS .EQ. 1) THEN
            DO Cfov=1,cris_1dnfor
              crisbestFOV(Cfov,line) = default_CRIS      !Fixed detector
            ENDDO
          ELSE
            crisbestFOV(1,line) = FOVorder(MOD(line,9) + 1)
            IF (OKdetector_cris(crisbestFOV(1,line)).EQ.0) THEN
              crisbestFOV(1,line) = default_CRIS         !Can't use that detector
            ENDIF
            DO Cfov=2,cris_1dnfor
              crisbestFOV(Cfov,line) = crisbestFOV(1,line)  !Fixed sequence
            ENDDO
          ENDIF
        ELSE
          DO Cfov=1,cris_1dnfor   !Loop over FOVs
            thinmode = thinmode_CRIS
            

!Thinning option 3: latitude dependent options. Thinmode is re-assigned
!depending on the latitude.

            IF (thinmode .EQ. 3) THEN
              Lat = cris1c_latlon(1,5,Cfov)*1.0E-4    !Central latitude
              IF (Lat.GT.latlimits_cris(1) .OR.
     &            Lat.LT.latlimits_cris(2)) THEN  !Polar
                thinmode = 1       !Fixed
                crisbestFOV(Cfov,line) = default_CRIS
              ELSE               !Not polar
                thinmode = 2       !Warmest CRIS
              ENDIF
            ENDIF

!Thinning option 2: Highest mean CRIS radiance for I/R channel

            IF (thinmode .EQ. 2) THEN
              cmaxrad = 0
              bestdet = default_CRIS             !Default
              DO Cdet=1,9
                rad = cris1c_radiance(Cdet,Cfov,crischan_index)
                IF (rad .GT. cmaxrad .AND. 
     &              OKdetector_cris(Cdet).EQ.1) THEN
                  bestdet = Cdet
                  cmaxrad = rad
                ENDIF
              ENDDO
              crisbestFOV(Cfov,line) = bestdet
            ENDIF    !thinmodethinmode_CRIS
          ENDDO    !Cfov loop
        ENDIF    !not mode 1
      ENDIF    !nrec_CRIS .EQ. 1
      END
