!+ Look for precipitation signal in AMSU-A on HIRS grid
!
      SUBROUTINE PPHAPCP ( HLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphapcp.F 590 2018-06-05 10:51:49Z frna $
!
! Purpose:
!   Look for precipitation signal in AMSU-A mapped to HIRS grid
!   and set flags accordingly.
!
! Method:
! 0.  Loop through scanlines and fields-of-view
! 1.  Scattering test
! 2.  Crosby, Ferraro & Wu test
! 3.  Grody light rainfall test
! 4.  Set flags
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      07/09/98 Derived from subroutine ppapcp. (R.J.Renshaw)
!  1.1      05/06/18 Don't set scat or crosby for Metop-B with failed AMSU 15
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! fixed data for processing AMSU-A
#include "ppafdata.h"       
! block of HIRS level 1d data
#include "pphstore.h"        
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"      
!
!   Scalar arguments with intent(In):
      INTEGER       HLastLine     ! number of last HIRS line in store
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       Afov          ! nearest AMSU-A fov to HIRS fov
      REAL          T1            ! brightness temp in AMSU-A channel 1 (K)
      REAL          T2            ! brightness temp in AMSU-A channel 2 (K)
      REAL          T3            ! brightness temp in AMSU-A channel 3 (K)
      REAL          T15           ! brightness temp in AMSU-A channel 15 (K)
      REAL          SI            ! scattering index
      REAL          probprecip    ! Crosby probability of precipitation
      LOGICAL       scat          ! .TRUE. if scattering detected
      LOGICAL       crosby        ! .TRUE. if Crosby test flags fov
      LOGICAL       grody         ! .TRUE. if Grody test flags fov
!
! Local arrays:
      INTEGER NearestAfov(Hnfovs) ! nearest AMSU-A fov to each HIRS fov
      DATA NearestAfov / 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 7, 8, 8,
     & 9, 9, 10, 10, 11, 11, 12, 13, 13, 14, 14, 15, 15, 16, 16, 17, 17,
     & 18, 18, 19, 20, 20, 21, 21, 22, 22, 23, 23, 24, 24, 25, 25, 26,
     & 27, 27, 28, 28, 29, 29, 30, 30 /
!
! Function & Subroutine calls:
      External PPASCAT            ! scattering index
          REAL PPASCAT
      External PPCROSBY           ! logistic regression
          REAL PPCROSBY
      External PPGRODY            ! light rainfall test
       LOGICAL PPGRODY
!
!- End of header

! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, HLastLine
      DO fov = 1, Hnfovs

         Afov = NearestAfov(fov)
         T1  = Hmapbt(1,fov,line)
         T2  = Hmapbt(2,fov,line)
         T3  = Hmapbt(3,fov,line)
         T15 = Hmapbt(15,fov,line)

! 1.     Scattering test  (only over the sea)
!        ---------- ----

!        only do if channels present
         IF (HSatId .EQ. 1 .AND. T15 .LE. 0) THEN
            Hscatindx(fov,line) = RMDI
            scat = .FALSE.    !Metop-B failed channel 15
         ELSE IF (  T1 .GT. RMDI  .AND.  T2 .GT. RMDI
     &   .AND. T3 .GT. RMDI  .AND.  T15 .GT. RMDI ) THEN
            SI = PPASCAT ( T1, T2, T3, T15, Afov )
            Hscatindx(fov,line) = SI
            IF ( Hsurftype(fov,line).EQ.0 ) THEN
               scat = SI .GT. Amaxscat
            ELSE
               scat = .FALSE.
            ENDIF
         ELSE
            Hscatindx(fov,line) = RMDI
            scat = .TRUE.  ! default is to err on side of caution
         ENDIF


! 2.     Crosby, Ferraro & Wu test
!        ------  ------- - -- ----

         IF (HSatId .EQ. 1 .AND. T15 .LE. 0) THEN
            Hcrosby(fov,line) = RMDI
            crosby = .FALSE.    !Metop-B failed channel 15
         ELSE IF (  T1 .GT. RMDI  .AND.  T15 .GT. RMDI ) THEN
            probprecip = PPCROSBY(T1,T15 )
            Hcrosby(fov,line) = probprecip
            crosby = probprecip .GT. Amaxcrosby
         ELSE
            Hcrosby(fov,line) = RMDI
            crosby = .TRUE.  ! default is to err on side of caution
         ENDIF


! 3.        Grody light rainfall test
!           ----- ----- -------- ----

         IF (  T1 .GT. RMDI  .AND.  T2 .GT. RMDI ) THEN
            grody = PPGRODY( T1, T2 )
         ELSE
            grody = .TRUE.  ! default is to err on side of caution
         ENDIF


! 4.     Set flags
!        --- -----

         IF ( scat )
     &      Hmapqc(fov,line) = IBSET ( Hmapqc(fov,line), bits(1) )
         IF ( crosby )
     &      Hmapqc(fov,line) = IBSET ( Hmapqc(fov,line), bits(25) )
         IF ( grody )
     &      Hmapqc(fov,line) = IBSET ( Hmapqc(fov,line), bits(24) )

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
