!+ Estimate surface type from AMSU-A on HIRS grid
!
      SUBROUTINE PPHASURF ( HLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphasurf.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!     Estimate surface type and flag cloud liquid water
!     using AMSU-A data mapped to the HIRS grid
!
! Method:
! 0.  Loop through scanlines and fields-of-view
! 1.  Interpolate means & covariances to scan angle
! 2.  Find most likely surface type
!
! Surface types:
! 1 = Bare young ice (i.e. new ice, no snow)
! 2 = Dry land (i.e. dry with or without significant vegetation)
! 3 = Dry snow (i.e. snow with water less than 2%, over land)
! 4 = Multi-year ice (i.e. old ice with snow [assumed dry] cover)
! 5 = Sea (i.e. open water, no islands, ice-free, WS=0 to 14m/s)
! 6 = Wet forest (i.e. established forest with wet canopy)
! 7 = Wet land (i.e. non-forested land with a wet surface)
! 8 = Wet snow (i.e. snow with water content > 2%, over land or ice)
! 9 = Desert
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      07/09/98  Derived from subroutine ppasurf
!  1.1      15/09/04  Use constants_preproc.h for fundamental constants - NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"         
! fixed data for processing AMSU-A
#include "ppafdata.h"        
! block of HIRS level 1d data
#include "pphstore.h"      
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
! Fundamental and physical constants
#include "constants_preproc.h"    
!
!   Scalar arguments with intent(In):
      INTEGER       HLastLine     ! number of last HIRS line in store
!
! Local scalars:
      INTEGER       line          ! loop index: pointer to scanline in buffer
      INTEGER       fov           ! loop index: fov number
      INTEGER       iang          ! loop index over angles
      INTEGER       isurf         ! loop index over surface types
      INTEGER       k, kx, m      ! loop indices over channels
      INTEGER       kang          ! index of where ASecAng .GE. secang
      INTEGER       surftype      ! derived surface type
      REAL          normalise     ! normalise J wrt number of channels
      REAL          J             ! cost function for a given surface type
      REAL          Jmin          ! minimum value of J for all surface types
      REAL          secang        ! secant(satellite zenith angle)
      REAL          sec_norm      ! normalisation factor for interpolation
      REAL          sum           ! sum of cost function over channels
!
! Local arrays:
      REAL meanBTx(NASrfTyp,NASrfChMx) !mean btemp difference vector at secang
      REAL covBTx(NASrfTyp,NASrfChMx,NASrfChMx) ! inverse cov mx at secang
      REAL diffBTx(NASrfChMx)     ! difference from mean btemps vector
!
!- End of header

!     normalise cost function according to the number of channels used
      normalise = 1. / REAL(NASrfCh*NASrfCh)

! 0.  Loop through scanlines and fields-of-view
!     ---- ------- --------- --- --------------

      DO line = 1, HLastLine
      DO fov = 1, Hnfovs

! 1.     Interpolate means & covariances to scan angle
!        ----------- ----- - ----------- -- ---- -----

         secang = 1. / COS(Hangles(1,fov,line)*deg2rad) ! secant of view angle

!        Find specified angles either side of view angle
         kang = 0
         DO iang = 1, NASecAng-1
            IF ( secang.GE.ASecAng(iang)
     &     .AND. secang.LT.ASecAng(iang+1) ) kang = iang
         ENDDO  ! iang

!        Derive appropriate means & covariances
         IF ( kang .EQ. 0 ) THEN !secang>ASecAng(NASecAng):use ASecAng(NSecAng)
            DO isurf = 1, NASrfTyp
            DO k = 1, NASrfCh
               meanBTx(isurf,k) = AmeanBT(isurf,NASecAng,k)
               DO m = 1, NASrfCh
                  covBTx(isurf,m,k) = AcovBT(isurf,NASecAng,m,k)
               ENDDO  ! m
            ENDDO  ! k
            ENDDO  ! isurf
         ELSE                      ! sec < ASecAng(NASecAng): interpolate
            sec_norm = (secang-ASecAng(kang))
     &                 / (ASecAng(kang+1)-ASecAng(kang))
            DO isurf = 1, NASrfTyp
            DO k = 1, NASrfCh
               meanBTx(isurf,k) = AmeanBT(isurf,kang,k) + sec_norm*
     &                   (AmeanBT(isurf,kang+1,k)-AmeanBT(isurf,kang,k))
               DO m = 1, NASrfCh
                  covBTx(isurf,m,k) = AcovBT(isurf,kang,m,k) +
     &        sec_norm*(AcovBT(isurf,kang+1,m,k)-AcovBT(isurf,kang,m,k))
               ENDDO  ! m
            ENDDO  ! k
            ENDDO  ! isurf
         ENDIF  ! kang .EQ. 0


! 2.     Find most likely surface type
!        ---- ---- ------ ------- ----

!        We evaluate the cost function J = (to-tm).C^-1.(to-tm)T
!        to find the minimum value, looping through each surface type:
         Jmin = 9999.0
         DO isurf = 1, NASrfTyp

            DO k = 1, NASrfCh
               kx = ASrfChn(k)
               diffBTx(k) = Hmapbt(kx,fov,line) - meanBTx(isurf,k)
            ENDDO  ! k

!           J = ABS(diffBTx times cov.matrix times diffBTx(transpose))
            J = 0.
            DO k = 1, NASrfCh
               sum = 0.
               DO m = 1, NASrfCh
                  sum = sum + covBTx(isurf,m,k)*diffBTx(m)
               ENDDO  ! m
               J = J + diffBTx(k)*sum
            ENDDO  ! k
            J = ABS(J)

!           Is this the best fit yet ?
            IF ( J .LT. Jmin ) THEN
               Jmin = J
               surftype = isurf
            ENDIF

         ENDDO  ! isurf

!        normalise according to the number of channels used
         Jmin = Jmin * normalise

!        apply threshold check to minimum J for cloud
         IF ( Jmin .GT. ASrfThresh ) THEN
            Hmapqc(fov,line) = IBSET ( Hmapqc(fov,line), bits(2) )
         ENDIF

!        if snow or sea-ice, check channel 1 to see if this is actually desert
         IF ( surftype.EQ.1 .OR. surftype.EQ.4 .OR. surftype.EQ.8 ) THEN
            IF ( Hmapbt(1,fov,line).GT.275. ) surftype = 9    ! desert
         ENDIF

!        store minimum J and surface type
         Hsurfcost(fov,line) = Jmin
         Hmapsurf(fov,line) = surftype

      ENDDO  ! fov
      ENDDO  ! line


      RETURN
      END
