!+ Transfer level 1C HIRS data from data record to program store
!
      SUBROUTINE PPHIND ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphind.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   HIRS 1C data, when read, is held in common/hrs1cdt/. ATOVPP stores data
!   from several scanlines in its own internal array structure, contained
!   in common/pphstore/. This routine transfers a single scanline from
!   common/hrs1cdt/ to slot "Line" in common/pphstore/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1C include file (hrs1c.h) and HIRS store
!     (pphstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Copy variables from common/hrs1cdt/ to slot "Line" in common/pphstore/
! 2.  Check flags
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1C format vn 1.8
!                     Set Hqc(bit0) if any channels are missing (not HIRS-20)
!  1.2      10/03/98  Conform to 1C format vn 1.9  - instr temp added
!  1.3      29/06/98  HmapBT initialised to RMDI
!  1.4      02/07/98  Hangles REAL not INTEGER
!  1.5      04/08/98  Other variables initialised in section(1)
!  1.6      07/09/98  Update version numbers
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of HIRS data 
#include "pphstore.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /pphstore/
!
! Local parameters:
      INTEGER       H1Cvnnb       ! version number of 1C format compatible
         PARAMETER (H1Cvnnb=190)  ! with the code in this subroutine
      INTEGER       Hstvnnb       ! version number of "pphstore.h" compatible
         PARAMETER (Hstvnnb=131)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chanbits      ! bit word with all channel bits set
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       qc0           ! qc flagword, bit 0 set
      LOGICAL       badline       ! .TRUE. not to use a scanline
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first, chanbits
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!        Is the 1C include file compatible with this source code ?
         IF ( H1Cvnnb .NE. hrs_1cvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but hrs1c.h is "",I6)")
     &           H1Cvnnb, hrs_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPHIND")
         ENDIF

!        Is the pphstore include file compatible with this source code ?
         IF ( Hstvnnb .NE. hrs_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects Hstore format"",I6,"" but pphstore.h is"",I6)")
     &           Hstvnnb, hrs_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPHIND")
         ENDIF

!        set up a mask to pick out just the flags for the individual channels
         chanbits = 0
         DO chn = 1, Hnchan-1   !  (except for HIRS-20)
            chanbits = IBSET ( chanbits, bits(chn) )
         ENDDO

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from common/hrs1cdt/ to slot "Line" in common/pphstore/
!     ---- --------- ---- --------------- -- ----  ----  -- ----------------

      Hlineno(Line) = hrs1c_scnlin                ! scanline number
      Hyear(Line) = hrs1c_scnlinyr                ! scanline year
      Hday(Line) = hrs1c_scnlindy                 ! scanline day of year
      Htime(Line) =  hrs1c_scnlintime             ! scanline time (ms)
      Hqualind(Line) = hrs1c_qualind              ! quality indicator bit field
      Hscanqual(Line) = hrs1c_scanqual            ! scan line quality flags
      Hinstrtemp(Line) = hrs1c_instrtemp          ! baseline temperature

      DO fov = 1, Hnfovs
         Hlats(fov,Line) = REAL(hrs1c_latlon(1,fov))*1.E-4 !latitude(deg,N+ve)
         Hlons(fov,Line) = REAL(hrs1c_latlon(2,fov))*1.E-4 !longitude (E+ve)
         DO i = 1, 4
            Hangles(i,fov,Line) = REAL(hrs1c_angles(i,fov))*0.01 ! scan angles:
!                                         (1,*,*) local zenith
!                                         (2,*,*) local azimuth
!                                         (3,*,*) solar zenith
!                                         (4,*,*) solar azimuth
         ENDDO  ! i
      ENDDO  ! fov

      Haltitude(Line) =  hrs1c_scalti             ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
      DO fov = 1, Hnfovs
      DO chn = 1, Hnchan
         Hbtemps(chn,fov,Line) = REAL(hrs1c_btemps(chn,fov))*0.01 !HIRS BTs (K)
      ENDDO  ! chn
      DO chn = 1, Anchan+Bnchan
         HmapBT(chn,fov,Line) = RMDI
      ENDDO  ! chn
      ENDDO  ! fov

      DO fov = 1, Hnfovs
         Hqc(fov,Line) = hrs1c_dataqual(fov)  ! QC word for HIRS
      ENDDO  ! fov

!     Set Hqc(bit0) if any channels are missing (except for HIRS-20)
      DO fov = 1, Hnfovs
!        IF ( (hrs1c_dataqual(fov).AND.chanbits).NE.0 )
         IF ( IAND(hrs1c_dataqual(fov),chanbits).NE.0 )
     &         Hqc(fov,Line) = IBSET(Hqc(fov,Line),bits(0))
      ENDDO  ! fov

!     initialise other variables
      qc0 = 0
      qc0 = IBSET(qc0,bits(0)) ! QC word: "AMSU missing"
      DO fov = 1, Hnfovs
         Hmapsurf(fov,Line)  = MDI   ! estimated AMSU-A surface type
         Hsurfcost(fov,Line) = RMDI  ! AMSU-A max prob cloud cost
         Hscatindx(fov,Line) = RMDI  ! AMSU-A scattering index
         Hcrosby(fov,Line)   = RMDI  ! AMSU-A logistic precip probability
         Hmapqc(fov,Line)    = qc0   ! AMSU QC word
      ENDDO  ! fov

! 2.  Check flags
!     ----- -----

! 2.1 Flags for the whole scanline

!     data quality indicator
      badline = BTEST( Hqualind(Line), bits(31) ) !do not use for products
     &    .OR.  BTEST( Hqualind(Line), bits(27) ) !no earth location

!     scanline quality indicator
      badline = badline
     &    .OR.  BTEST( Hscanqual(Line), bits(23) )!bad time field
     &    .OR.  BTEST( Hscanqual(Line), bits(22) )!bad time field
     &    .OR.  BTEST( Hscanqual(Line), bits(20) )!sequence repeats
     &    .OR.  BTEST( Hscanqual(Line), bits(19) )!TIP synch error
     &    .OR.  BTEST( Hscanqual(Line), bits(18) )!TIP frame sequence error
     &    .OR.  BTEST( Hscanqual(Line), bits(17) )!TIP status error
     &    .OR.  BTEST( Hscanqual(Line), bits(16) )!TIP parity error
     &    .OR.  BTEST( Hscanqual(Line), bits(04) )!earth location unreasonable

!     For bad scanline, flag each fov
      IF ( badline ) THEN
         DO fov = 1, Hnfovs
            Hqc(fov,Line) = IBSET ( Hqc(fov,Line), bits(0) )
         ENDDO  ! fov
      ENDIF


      RETURN
      END
