!+ Read & interpret HIRS level 1C header within ATOVPP
!
      SUBROUTINE PPHINH ( H1Cunit, SatId, WMOSatId,
     &                    StartYearH, StartDayH, StartTimeH,
     &                    EndYearH,   EndDayH,   EndTimeH )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphinh.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Open input dataset and read level 1C HIRS header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     HIRS level 1C header has already been read into common /hrs1chd/
!     Check that format of 1C include files (hrs1c.h) is compatible with the
!     code in this subroutine. Check that format of 1C file is compatible with
!     this subroutine, and that data type from the header is 5 for HIRS.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from HIRS 1C header
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  SatId added, Ht and Period deleted
!  1.2      15/06/98  Argument TOVS deleted
!  1.3      23/08/05  Add WMOSatId (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of HIRS data 
#include "pphstore.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
! Local parameters:
      INTEGER       H1Cvnnb       ! version number of 1C format compatible
         PARAMETER (H1Cvnnb=190)  ! with the code in this subroutine
!
!     Alternative way of checking that version numbers match (at compile time)
!      REAL VNTEST1(H1Cvnnb-hrs_1cvnnb+1), VNTEST2(hrs_1cvnnb-H1Cvnnb+1)
!
!   Scalar arguments with intent(In):
      INTEGER       H1Cunit       ! logical unit no. for HIRS 1C input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 3 for METOP-1)
      ! start/end date/time of HIRS data:
      INTEGER       StartYearH, EndYearH
      INTEGER       StartDayH,  EndDayH
      INTEGER       StartTimeH, EndTimeH
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       H1Crecord     ! record number for HIRS level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOH1C                ! I/O routine for level 1C HIRS data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.1 Open 1C file
      CALL IOH1C ("OPEN/READ", H1Cunit, H1Crecord )   ! 1C file

! 1.2 Read 1C header
      H1Crecord = 1   ! Headers start on record 1.
      CALL IOH1C ("READ HEADER", H1Cunit, H1Crecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( H1Cvnnb .NE. hrs_1cvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but hrs1c.h is "",I6)")
     &        H1Cvnnb, hrs_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPHINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "HRS1C.H" contains its own version number "hrs_h_l1cversnb"
      IF ( hrs1c_h_versnb .NE. hrs_1cvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level 1C HIRS file is "",I8,"", not "",I8)")
     &        hrs1c_h_versnb,hrs_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPHINH")
      ENDIF

! 2.3 Is the 1C data HIRS ?
!     -- --- -- ---- ----
!     "hrs1c_h_instrument" from 1C header should be 5 for HIRS
      IF ( hrs1c_h_instrument .NE. 5 ) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Data type in 1C HIRS file is "",I8,""; not 5 for HIRS"")")
     &        hrs1c_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "PPHINH")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "########"
      WRITE(*,*) "# HIRS #"
      WRITE(*,*) "########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            hrs1c_h_versnb, hrs1c_h_versyr, hrs1c_h_versdy
      WRITE(*,"(I4,"" header records in file."")") hrs1c_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") hrs1c_h_satid

!     calculate start and end dates/times from header
!     ..start
      year = hrs1c_h_startdatayr
      yrday = hrs1c_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(hrs1c_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = hrs1c_h_enddatayr
      yrday = hrs1c_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(hrs1c_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          hrs1c_h_scnlin, hrs1c_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from HIRS 1C header
!     ------- -------- ----------- ---- ---- -- ------

      HSatId = hrs1c_h_satid
      SatId = HSatId
      WMOSatId = hrs1c_h_wmosatid
      StartDayH = hrs1c_h_startdatady
      StartYearH = hrs1c_h_startdatayr
      StartTimeH = hrs1c_h_startdatatime
      EndDayH = hrs1c_h_enddatady
      EndYearH = hrs1c_h_enddatayr
      EndTimeH = hrs1c_h_enddatatime
      Hsatht = REAL(hrs1c_h_satht)*0.1      !nominal sat height (km)
      Hsatperiod = REAL(hrs1c_h_period)/60. !nominal orbit period (minutes)
      Hhdrcnt = hrs1c_h_hdrcnt       !count of header records in 1C file
      Hscnlin = hrs1c_h_scnlin       !count of scanlines in 1C file
      Hinstruments = IBSET ( Hinstruments , bits(0) )   ! HIRS data present

      RETURN
      END
