!+ HIRAS cloud tests
!
      SUBROUTINE PPHIRASCLOUD ( Testnumber, Clastline )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirascloud.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!    NWP-independent cloud detection tests for HIRAS
!
! Method:
!    This is identical to the tests for IASI.
!    Currently 2 tests are implemented:
!    1) Standard deviation of HIRAS radiances for the 4 detectors at each scan 
!       position
!    2) Make use of mapped MWTS/MWHS in order to simulate HIRAS radiances (not
!       yet implemented)
!    
!    Results are stored in the hirasCloudflag array, which has space for 10 cloud
!    tests. The last slot is used as a clear/cloudy flag (1 bit per test)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2007, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    08/02/2019 from ppcriscloud.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  HIRAS 1c format 
#include "hiras1c.h"
!  HIRAS 1d format 
#include "hiras1d.h"
!  block of HIRAS level 1d data 
#include "pphirasstore.h"
!  fixed data for processing HIRAS 
#include "pphirasfdata.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  constants 
#include "constants_preproc.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Testnumber    ! cloud test number
      INTEGER       Clastline     ! number of last HIRAS line in store
                                  !  (or current line for test 1)
!
! Local scalars:
      REAL          radmean, radsum
      REAL          T4, T5, T6, csza, cview
      REAL          radiance, bri_temp, w, T, drdt, radsd, bt_pred
      INTEGER       scalrad(hiras_1cnfov)
      INTEGER       fov, Ichan, ndet
      INTEGER       I,Idet,line     
!
!- End of header

!Update header record to indicate that this test has been performed
      hiras1d_h_cloudtests =
     &  IBSET(hiras1d_h_cloudtests,BITS(Testnumber-1))

      IF (Testnumber .eq. 1) THEN

!---------- Test 1: Standard deviation of 4 HIRAS FOVs at 2390 cm^-1 -----------
!
!Reference: Cheng et. al., ITSC-15, poster A17
!
!This test must be done while the HIRAS 1c are being read (see pphiras1d)
!------------------------------------------------------------------------------

        T = 280.0
        w = hirasCloudW
        drdt = C_1*w*w*w/(exp(w*C_2/T)-1)**2 * exp(w*C_2/T) *C_2*w/(T*T)

        DO fov=1,hiras_1dnfor
          radmean = 0.0
          ndet = 0
          DO Idet=1,4
            scalrad(Idet) = hiras1c_radiance(Idet,fov,hirasCloudindex)
            IF (scalrad(Idet).ne.0 .and. OKdetector_hiras(Idet).eq.1)
     &      THEN
              ndet = ndet + 1
              radmean = radmean + scalrad(Idet)
            ENDIF
          ENDDO
          IF (ndet .ge. 2) THEN         !require at least 2 good detectors
            radmean = radmean/ndet
            radsd = 0.0
            DO Idet=1,4
              IF (scalrad(Idet).ne.0 .and.
     &                OKdetector_hiras(Idet).eq.1) THEN
                radsd = radsd + (scalrad(Idet)-radmean)**2
              ENDIF
            ENDDO
            radsd = sqrt(radsd/(ndet-1))*1.0E-4/drdt

!Store 100*(std deviation referenced to 280K)

            hirasCloudflag(Testnumber,fov,Clastline) = radsd*100

!Set master flag (word 10) if threshold is exceeded
!Assume NEdT at 280K is 0.42K

            IF (radsd .GT. 3*0.42) THEN
              hirasCloudflag(10,fov,Clastline) =
     &        IBSET(hirasCloudflag(10,fov,Clastline),BITS(Testnumber-1))
            ENDIF

          ELSE
            hirasCloudflag(Testnumber,fov,Clastline) = MDI   !No HIRAS
          ENDIF

        ENDDO

      ELSE IF (Testnumber .eq. 2) THEN       

!---------- Test 2: HIRAS 2390 cm^-1 radiance predicted from AMSU -----------
!
!Reference: Cheng et. al., ITSC-15, poster A17
!------------------------------------------------------------------------------

        DO line = 1, Clastline
          DO fov=1,hiras_1dnfor
            T4  = hirasmapbt(3,fov,line)   ! MWTS channel 3 = AMSU channel 4
            T5  = hirasmapbt(4,fov,line)
            T6  = hirasmapbt(5,fov,line)
            csza = COS(hirasangles(3,fov,line)*deg2rad)    !Cos solar zenith angle
            cview = COS((fov-15.5)*3.33333*deg2rad)    !Cos scan angle
            IF (T4.GT.RMDI .AND. T5.GT.RMDI .AND. T6.GT.RMDI) THEN

              bt_pred = 44.007 - 0.272*T4 + 1.837*T5 - 0.738*T6
     &                  + 0.004*csza + 6.391*(1-cview)

!We already have the radiance (see pphiras1d)

              radiance = hirasCloudflag(Testnumber,fov,line)*
     &              1.0E-4    ! mW m-2 sr-1 cm
              
              IF (radiance.GT.0.) THEN
                bri_temp = C_2*w / log(C_1*w*w*w/radiance + 1)

!Over-write hirasCloudflag with the BT diff (*100)

                hirasCloudflag(Testnumber,fov,line)=
     &                 (bt_pred - bri_temp)*100

!Set master flag (word 10) if threshold is exceeded

                IF (bt_pred - bri_temp .GT. 2.0) THEN
                  hirasCloudflag(10,fov,line) =
     &             IBSET(hirasCloudflag(10,fov,line),BITS(Testnumber-1))
                ENDIF

              ELSE
                hirasCloudflag(Testnumber,fov,line) = MDI      !No valid HIRAS
              ENDIF
     
            ELSE
              hirasCloudflag(Testnumber,fov,line) = MDI      !No ATMS
            ENDIF
          ENDDO
        ENDDO
      
      ENDIF     !end test 2
      END

