!+ Read fixed data file for hiras within ATOVPP
!
      SUBROUTINE PPHIRASFDF (Cfunit,Ceigunit1,Ceigunit2,Cnoiseunit)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirasfdf.F 635 2019-04-15 15:08:00Z frna $
!
! Purpose:
!   Read fixed data file required for HIRAS within ATOVPP.
!
! Method:
! 1.  Read channel selection and thinning requirements
! 2.  Read eigenvectors for encoding (radiance to PC)
! 3.  Read eigenvectors for decoding (PC to radiance)
! 4.  Read noise profile, nedr (for generating reconstruction score)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 06 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    08/02/2019 From ppcrisfdf.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1d format 
#include "hiras1d.h"
!  fixed data for processing hiras 
#include "pphirasfdata.h"
!  fixed data for processing hiras 
#include "reclopt.h"
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       Cfunit        ! unit numbers
      INTEGER       Ceigunit1
      INTEGER       Ceigunit2
      INTEGER       Cnoiseunit
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status value
      INTEGER       chn           ! loop index: channel number
      INTEGER       Ipc           ! loop index: PC number
      INTEGER       nchan_max     ! number of chans in eigenvector file
      INTEGER       npc_max       ! number of PCs in eigenvector file
      REAL*4        tmp(hirasnchan)   ! temporary storage for eigenvectors

! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine

!- End of header


! 1.  open and read fdf file
!     ----------------------

      OPEN ( UNIT=Cfunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=Cfunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

      WRITE ( errMessage,
     & "("" Error opening hiras fixed data file, unit "",I8)" ) Cfunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPHIRASFDF")


!     look for sections in the file
!     ---- --- -------- -- --- ----

      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )
        input_line = " "
        READ(Cfunit,"(A80)",IOSTAT=errStatus) input_line

        IF ( input_line(1:5) .EQ. "START" ) THEN

!     Read file
!     ---- ----

          READ(Cfunit,*) thinmode_hiras !Thinning mode: 0 = full resolution.
                                       !               1 = fixed detector(s)
                                       !               2 = warmest HIRAS
                                       !               3 = hybrid 2&1
                                       !               4 = fixed sequence

          READ(Cfunit,*) nrec_hiras    !records per scan line

          IF (thinmode_hiras .EQ. 0) THEN
            READ(Cfunit,*) detector_use_hiras(1)    !Not used for full res
          ELSE
            READ(Cfunit,*) (detector_use_hiras(chn),chn=1,nrec_hiras)
          ENDIF
          default_hiras = detector_use_hiras(1)

          IF (thinmode_hiras .EQ. 0) THEN  !Over-write values if full res
            nrec_hiras = 4       !Number of 1d records to output per scan
            DO chn=1,4
              detector_use_hiras(chn) = chn
            ENDDO
          ENDIF

          READ(Cfunit,*) (OKdetector_hiras(chn), chn=1,4) !status of the 4 detectors

          READ(Cfunit,*) latlimits_hiras(1),latlimits_hiras(2) !Latitude limits for option3

          READ(Cfunit,*) hiraschan      !hiras channel for option 2

          READ(Cfunit,*) npc_hiras      !Number of PC scores to output in 1d

          IF (npc_hiras .GT. hirasmaxnpc) THEN
            WRITE (*,*) "Warning: hiras fdf file specifies too many PCs"
            WRITE (*,*) "Maximum is hirasmaxnpc=", hirasmaxnpc
            WRITE (*,*) "Modify ppparms.h to increase and recompile"
            npc_hiras = hirasmaxnpc
          ENDIF
          
          READ(Cfunit,*) chanvn_hiras           !Channel selection version number
          hiras1d_h_chanvn = chanvn_hiras
          READ(Cfunit,*) nchan_hiras       !Number of channels to output in 1d

          READ(Cfunit,*) (channels_hiras(chn), chn=1,nchan_hiras)

        ENDIF
      ENDDO

      CLOSE(Cfunit)

!Read environment variable ATOVPP_USE_RR
      
      CALL GETENV ('ATOVPP_USE_RR',input_line)
      IF (input_line(1:1) .EQ. 'T') THEN
        Use_reconstructed_rad_hiras = .TRUE.
        WRITE(*,'(10X,A)') "Use reconstructed radiances"
      ELSE
        Use_reconstructed_rad_hiras = .FALSE.
      ENDIF

! 2.  Read eigenvector file for encoding (direct access binary)
!     ----------------------------------

      IF (npc_hiras .GT. 1) THEN

       IF (Ceigunit1 .GT. 0) THEN
        OPEN ( UNIT=Ceigunit1, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=hirasnchan*4/rldiv, IOSTAT=errStatus)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Ceigunit1, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=hirasnchan*4/rldiv, IOSTAT=errStatus)

        WRITE ( errMessage,
     &   "("" Error opening hiras eigenvector encode file, unit "",I8)")
     &   Ceigunit1
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPHIRASFDF")

        READ(Ceigunit1,rec=1) eigvn_hiras,nchan_max,pccount_hiras,
     &      eigformat_hiras,evlevel_hiras,
     &      startpcchan_hiras,endpcchan_hiras,startpc_hiras,endpc_hiras,
     &      evfile_hiras   !Read header record
        npcbands_hiras = 3
        IF (startpcchan_hiras(3).EQ.0) npcbands_hiras=2
        IF (startpcchan_hiras(2).EQ.0) npcbands_hiras=1
        IF (startpcchan_hiras(1).EQ.0) THEN
          startpcchan_hiras(1) = 1
          endpcchan_hiras(1) = nchan_hiras
          startpc_hiras(1) = 1
          endpc_hiras(1) = min(npc_hiras,pccount_hiras)
        ENDIF
        
        IF (nchan_max.LT.hirasnchan) THEN
          WRITE ( errMessage,
     &     "("" Not enough channels in eigenvector file"")" )
          errStatus = 99
          CALL ErrorReport(errStatus,errMessage,"PPHIRASFDF")
        ENDIF
          
        hiras1d_h_eigvn = eigvn_hiras       !Store version number of eigenvectors

        IF (eigformat_hiras.EQ.1) THEN
          READ(Ceigunit1,rec=2) mean_encode_hiras
        ELSE
          eigformat_hiras = 0   !Only allow 0 and 1
        ENDIF

        IF (pccount_hiras.LT.npc_hiras) THEN
          WRITE (*,*) "WARNING: number of PCs requested is more than "//
     &       "number of eigenvectors in file",npc_hiras,pccount_hiras
          npc_hiras = pccount_hiras
        ENDIF

        DO Ipc=1,npc_hiras
          READ(Ceigunit1,rec=Ipc+eigformat_hiras+1) tmp
          DO chn=1,hirasnchan
            eig_encode_hiras(chn,Ipc) = tmp(chn)
          ENDDO
        ENDDO
      
        IF (evlevel_hiras.NE.'1C') evlevel_hiras = '1B'   !Only 1B and 1C supported

        CLOSE(Ceigunit1)
       ENDIF

! 3.  Read eigenvector file for decoding (direct access binary)
!     ----------------------------------

       IF (Ceigunit2 .GT. 0) THEN
        OPEN ( UNIT=Ceigunit2, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=hirasnchan*4/rldiv, IOSTAT=errStatus)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Ceigunit2, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=hirasnchan*4/rldiv, IOSTAT=errStatus)

        WRITE ( errMessage,
     &   "("" Error opening hiras eigenvector decode file, unit "",I8)") 
     &   Ceigunit2
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPHIRASFDF")

        READ(Ceigunit2,rec=1) eigvn_hiras,nchan_max,pccount_hiras,
     &      eigformat_hiras,evlevel_hiras,
     &      startpcchan_hiras,endpcchan_hiras,startpc_hiras,endpc_hiras,
     &      evfile_hiras   !Read header record
        npcbands_hiras = 3
        IF (startpcchan_hiras(3).EQ.0) npcbands_hiras=2
        IF (startpcchan_hiras(2).EQ.0) npcbands_hiras=1
        IF (startpcchan_hiras(1).EQ.0) THEN
          startpcchan_hiras(1) = 1             !Default is all channels together
          endpcchan_hiras(1) = nchan_hiras
          startpc_hiras(1) = 1
          endpc_hiras(1) = min(npc_hiras,pccount_hiras)
        ENDIF

        IF (eigvn_hiras.ne.hiras1d_h_eigvn) THEN
          WRITE(*,*) "WARNING: eigenvector version number for " //
     &       "decoding differs from encoding",
     &       eigvn_hiras,hiras1d_h_eigvn
        ENDIF

        IF (eigformat_hiras.EQ.1) THEN
          READ(Ceigunit2,rec=2) mean_decode_hiras
        ELSE
          eigformat_hiras = 0   !Only allow 0 and 1
        ENDIF

        IF (pccount_hiras.LT.npc_hiras) npc_hiras = pccount_hiras

        DO Ipc=1,npc_hiras
          READ(Ceigunit2,rec=Ipc+eigformat_hiras+1) tmp
          DO chn=1,hirasnchan
            eig_decode_hiras(chn,Ipc) = tmp(chn)
          ENDDO
        ENDDO
      
        IF (evlevel_hiras.NE.'1C') evlevel_hiras = '1B'   !Only 1B and 1C supported

        CLOSE(Ceigunit2)
       ENDIF

! 4.  Read nedr file (direct access binary)
!     --------------

       IF (Cnoiseunit .GT. 0) THEN
        OPEN ( UNIT=Cnoiseunit, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=hirasnchan*4/rldiv, IOSTAT=errStatus)

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Cnoiseunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=hirasnchan*4/rldiv, IOSTAT=errStatus)

        WRITE ( errMessage,
     &   "("" Error opening hiras eigenvector file 2, unit "",I8)" ) 
     &   Cnoiseunit
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPHIRASFDF")

        READ(Cnoiseunit,rec=1) nedr_hiras
      
        CLOSE(Cnoiseunit)
       ENDIF

      ENDIF         ! npc_hiras .GT. 1

!Initialise cloud tests flag
      hiras1d_h_cloudtests = 0

      RETURN
      END
