!+ Transfer level 1C HIRAS data from data record to program store
!
      SUBROUTINE PPHIRASIND ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirasind.F 635 2019-04-15 15:08:00Z frna $
!
! Purpose:
!   HIRAS 1C data, when read, is held in common/hiras1cdt/. ATOVPP stores data
!   from several scanlines in its own internal array structure, contained
!   in common/pphirasstore/. This routine transfers a single scanline from
!   common/hiras1cdt/ to slot "Line" in common/pphirasstore/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1C include file (hiras1c.h) and HIRAS store
!     (pphirasstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Copy variables from common/hiras1cdt/ to slot "Line" in common/pphirasstore/
! 2.  Check flags
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     11/02/19  N.C.Atkinson, based on ppcrisind.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of HIRAS data 
#include "pphirasstore.h"
!  HIRAS level 1C header/data common blocks 
#include "hiras1c.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /pphirasstore/
!
! Local parameters:
      INTEGER       hiras1Cvnnb       ! version number of 1C format compatible
         PARAMETER (hiras1Cvnnb=1)    ! with the code in this subroutine
      INTEGER       hirasstvnnb       ! version number of "pphirasstore.h" compatible
         PARAMETER (hirasstvnnb=1)    ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chanbits      ! bit word with all channel bits set
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       det           ! detector
      INTEGER       pos           ! scan position
      INTEGER       qc0           ! qc flagword, bit 0 set
      LOGICAL       badline       ! .TRUE. not to use a scanline
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first, chanbits
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!        Is the 1C include file compatible with this source code ?
         IF ( hiras1Cvnnb .NE. hiras_1cvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &     "("" Code expects 1C format"",I6,"" but hiras1c.h is "",I6)")
     &           hiras1Cvnnb, hiras_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPHIRASIND")
         ENDIF

!        Is the pphirasstore include file compatible with this source code ?
         IF ( hirasstvnnb .NE. hiras_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &           "("" Code expects Istore format"",I6,"" but " //
     &           "pphirasstore.h is"",I6)")
     &           hirasstvnnb, hiras_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPHIRASIND")
         ENDIF

!        set up a mask to pick out just the flags for the individual detectors
         chanbits = 0
         DO chn = 1, hirasndet
            chanbits = IBSET ( chanbits, bits(chn) )
         ENDDO

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from common/hiras1cdt/ to slot "Line" in common/pphirasstore/
!     ---- --------- ---- --------------- -- ----  ----  -- ----------------

      hiraslineno(Line) = hiras1c_scnlin                ! scanline number
      hirasyear(Line) = hiras1c_scnlinyr                ! scanline year
      hirasday(Line) = hiras1c_scnlindy                 ! scanline day of year
      
      DO fov = 1, hirasnfor
         hirastime(fov,Line) =  hiras1c_scnlintime(fov) ! scanline time (ms)
      ENDDO

      DO fov = 1, hirasnfovs      !store 116 fovs, 1-29=detector 1, etc. 
         det = (fov-1)/29 + 1   !detector
         pos = MOD(fov-1,29) + 1  !scan position
         hiraslats(fov,Line) = REAL(hiras1c_latlon(1,det,pos))*1.E-4 !latitude(deg,N+ve)
         hiraslons(fov,Line) = REAL(hiras1c_latlon(2,det,pos))*1.E-4 !longitude (E+ve)
         DO i = 1, 4
            hirasangles(i,fov,Line) = REAL(hiras1c_angles(i,det,pos))*0.01 ! scan angles:
!                                         (1,*,*) local zenith
!                                         (2,*,*) local azimuth
!                                         (3,*,*) solar zenith
!                                         (4,*,*) solar azimuth
         ENDDO  ! i
      ENDDO  ! fov

      DO fov = 1, hirasnfovs
      DO chn = 1, mwtsnchan
         hirasmapbt(chn,fov,Line) = RMDI
      ENDDO  ! chn
      ENDDO  ! fov

      DO fov = 1, hirasnfovs
         hirasqc(fov,Line) = 0  ! QC word for HIRAS
      ENDDO  ! fov

!     initialise other variables
      qc0 = 0
      qc0 = IBSET(qc0,bits(0)) ! QC word: "AMSU missing"
      DO fov = 1, hirasnfovs
         hirasmapsurf(fov,Line)  = MDI   ! estimated MWTS surface type
         hirassurfcost(fov,Line) = RMDI  ! MWTS max prob cloud cost
         hirasscatindx(fov,Line) = RMDI  ! MWTS scattering index
         hirascrosby(fov,Line)   = RMDI  ! MWTS logistic precip probability
         hirasmapqc(fov,Line)    = qc0   ! MWTS QC word
      ENDDO  ! fov

! 2.  Check flags
!     ----- -----

! 2.1 Flags for the whole scanline

      badline = .FALSE.       !There are no whole-scanline flags at present

!     For bad scanline, flag each fov
      IF ( badline ) THEN
         DO fov = 1, hirasnfovs
            hirasqc(fov,Line) = IBSET ( hirasqc(fov,Line), bits(0) )
         ENDDO  ! fov
      ENDIF


      RETURN
      END
