!+ Read & interpret HIRAS level 1C header within ATOVPP
!
      SUBROUTINE PPHIRASINH ( hiras1Cunit, SatId, WMOSatId,
     &                    StartYearhiras, StartDayhiras, StartTimehiras,
     &                    EndYearhiras,   EndDayhiras,   EndTimehiras )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirasinh.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Open input dataset and read level 1C HIRAS header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     HIRAS level 1C header has already been read into common /hiras1chd/
!     Check that format of 1C include files (hiras1c.h) is compatible with the
!     code in this subroutine. Check that format of 1C file is compatible with
!     this subroutine, and that data type from the header is 10 for HIRAS.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from HIRAS 1C header
!
! Current Code Owner: NWP SAF
!
! History:
! Version  Date      Comment
!
!  1.0   11/02/2019  Created from ppcrisinh.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
! general parameters for ATOVPP
#include "ppparms.h"        
! block of HIRAS data 
#include "pphirasstore.h"     
! HIRAS level 1C header/data common blocks
#include "hiras1c.h"           
! bit numbers for intrinsic bit routines (eg BTEST)
#include "bits.h"     
!
! Local parameters:
      INTEGER       hiras1Cvnnb       ! version number of 1C format compatible
         PARAMETER (hiras1Cvnnb=1)    ! with the code in this subroutine
!
!     Alternative way of checking that version numbers match (at compile time)
!      REAL VNTEST1(hiras1Cvnnb-hiras_1cvnnb+1), VNTEST2(hiras_1cvnnb-hiras1Cvnnb+1)
!
!   Scalar arguments with intent(In):
      INTEGER       hiras1Cunit       ! logical unit no. for HIRAS 1C input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 3 for METOP-1)
      ! start/end date/time of HIRAS data:
      INTEGER       StartYearhiras, EndYearhiras
      INTEGER       StartDayhiras,  EndDayhiras
      INTEGER       StartTimehiras, EndTimehiras
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       hiras1Crecord  ! record number for HIRAS level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOHIRAS1C                ! I/O routine for level 1C HIRAS data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.0 Find the record length
      hiras1c_h_recwords = array_size_hiras1c - 
     &      hiras_1cnchan*hiras_1cnfov*hiras_1cnfor   !excluding channels
      CALL IOHIRAS1C ("OPEN/READ", hiras1Cunit, 0 )
      CALL IOHIRAS1C ("READ HEADER", hiras1Cunit, 1 )
      CALL IOHIRAS1C ("CLOSE", hiras1Cunit, 0)

! 1.1 Open 1C file with correct record length
      CALL IOHIRAS1C ("OPEN/READ", hiras1Cunit, 0 )   ! 1C file

! 1.2 Read 1C header
      hiras1Crecord = 1   ! Headers start on record 1.
      CALL IOHIRAS1C ("READ HEADER", hiras1Cunit, hiras1Crecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( hiras1Cvnnb .NE. hiras_1cvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but hiras1c.h is "",I6)")
     &        hiras1Cvnnb, hiras_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPHIRASINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "hiras1c.h" contains its own version number "hiras1c_h_versnb"
      IF ( hiras1c_h_versnb .NE. hiras_1cvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level 1C HIRAS file is "",I8,"", not "",I8)")
     &        hiras1c_h_versnb,hiras_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPHIRASINH")
      ENDIF

! 2.3 Is the 1C data HIRAS ?
!     -- --- -- ---- ------
!     "hiras1c_h_instrument" from 1C header should be 955 for HIRAS
      IF ( hiras1c_h_instrument .NE. 955 ) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage, "("" Data type in 1C HIRAS file is "",
     &           I8,""; not 955 for HIRAS"")") hiras1c_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "PPHIRASINH")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "##########"
      WRITE(*,*) "#  HIRAS  #"
      WRITE(*,*) "##########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            hiras1c_h_versnb, hiras1c_h_versyr, hiras1c_h_versdy
      WRITE(*,"(I4,"" header records in file."")") hiras1c_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") hiras1c_h_satid

!     calculate start and end dates/times from header
!     ..start
      year = hiras1c_h_startdatayr
      yrday = hiras1c_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(hiras1c_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = hiras1c_h_enddatayr
      yrday = hiras1c_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(hiras1c_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          hiras1c_h_scnlin, hiras1c_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from HIRAS 1C header
!     ------- -------- ----------- ---- ---- -- ------

      hirasSatId = hiras1c_h_satid
      SatId = hirasSatId
      WMOSatId = hiras1c_h_wmosatid
      StartDayhiras = hiras1c_h_startdatady
      StartYearhiras = hiras1c_h_startdatayr
      StartTimehiras = hiras1c_h_startdatatime
      EndDayhiras = hiras1c_h_enddatady
      EndYearhiras = hiras1c_h_enddatayr
      EndTimehiras = hiras1c_h_enddatatime
      hirassatht = REAL(hiras1c_h_satht)*0.1      !nominal sat height (km)
      hirassatperiod = REAL(hiras1c_h_period)/60. !nominal orbit period (minutes)
      hirashdrcnt = hiras1c_h_hdrcnt       !count of header records in 1C file
      hirasscnlin = hiras1c_h_scnlin       !count of scanlines in 1C file
      hirasinstruments = IBSET ( hirasinstruments , bits(8) )   ! HIRAS data present


      RETURN
      END
