!+ Transfer data from program store to level 1D HIRAS data record
!
      SUBROUTINE PPHIRASOUTD ( Line, Irec )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirasoutd.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   ATOVPP stores data from several scanlines in its own internal array
!   structure, contained in common/pphirasstore/. This routine transfers a single
!   scanline from slot "Line" in common/pphirasstore/ to common/hiras1ddt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1D include file (hiras1d.h) and HIRAS store
!     (pphirasstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
!     Fill parts of 1D data record that won't change each scanline
! 1.  Copy variables from slot "Line" in common/pphirasstore/ to common/hiras1ddt/
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0   11/02/2019 Created from ppcoutd.F (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of HIRAS data 
#include "pphirasstore.h"
!  HIRAS level 1D header/data common blocks 
#include "hiras1d.h"
!  HIRAS fixed data and user options 
#include "pphirasfdata.h"
! MWTS fixed data
#include "ppmwts2fdata.h"       
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppistore/
      INTEGER       Irec          ! HIRAS 1d record (1 to 4)
!
! Local parameters:
      INTEGER       hiras1Dvnnb       ! version number of 1D format compatible
         PARAMETER (hiras1Dvnnb=1)    ! with the code in this subroutine
      INTEGER       hirasstvnnb       ! version number of "ppistore.h" compatible
         PARAMETER (hirasstvnnb=1)    ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov1d         ! loop variable over all 1d fovs
      INTEGER       fov           ! all HIRAS fovs (max 270)
      LOGICAL       badfov        ! indicates 1 or more bad BTs
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first                    ! .TRUE. on first call
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

! 0.1    Is the 1D include file compatible with this source code ?
         IF ( hiras1Dvnnb .NE. hiras_1dvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &     "("" Code expects 1D format"",I6,"" but hiras1d.h is "",I6)")
     &           hiras1Dvnnb, hiras_1dvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPHIRASOUTD")
         ENDIF

! 0.2    Is the pphirasstore include file compatible with this source code ?
         IF ( hirasstvnnb .NE. hiras_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &     "("" Code expects Istore format"",I6,"" but pphirasstore.h"//
     &     " is "",I6)") hirasstvnnb, hiras_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPHIRASOUTD")
         ENDIF

! 0.3    Fill parts of 1D data record that won't change each scanline

! 0.4    Reset "first" for subsequent calls
         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from slot "Line" in common/pphirasstore/ to common/hiras1ddt/
!     ---- --------- ---- ----  ----  -- ------------------- -- ---------------

!     scnlin, scnlinyr, scnlindy, scnlintime, scalti already filled in pphiras1d.F

      DO fov1d = 1, hiras_1dnfor
          
         IF (nrec_hiras.EQ.4) THEN
           fov = (irec-1)*29 + fov1d                   !Full resolution
         ELSE
           fov = (hirasbestFOV(fov1d,Line)-1)*29 + fov1d   !Best of 4
         ENDIF
         
!     Lat, lon, angles, etc. have already been filled in pphiras1d.F

!     Surfelev outputs
         hiras1d_surface(1,fov1d) = hiraselev(fov,Line)
         hiras1d_surface(2,fov1d) = hirassurftype(fov,Line)
         hiras1d_surface(3,fov1d) = hiraslandfrac(fov,Line)
         
!     Mapped MWTS BTs (deg K * 100) and quality flag
         badfov = .FALSE.
         hiras1d_hirasqual(fov1d) = 0
         DO chn = 1, mwtsnchan
           hiras1d_btemps(chn,fov1d)=NINT(hirasmapbt(chn,fov,Line)*100.)
           IF (mwts_ignore_channel(chn)) THEN
             hiras1d_hirasqual(fov1d) = 
     &         IBSET(hiras1d_hirasqual(fov1d),bits(chn+3))
           ELSE IF (hirasmapbt(chn,fov,Line).LT.mwtsminBT .OR.
     &              hirasmapbt(chn,fov,Line).GT.mwtsmaxBT) THEN
             badfov = .TRUE.
             hiras1d_hirasqual(fov1d) = 
     &         IBSET(hiras1d_hirasqual(fov1d),bits(chn+3))
           ENDIF
         ENDDO  ! chn

         hiras1d_hirasqual(fov1d) = hirasqc(fov,Line)   ! QC word for IASI

         hiras1d_prepro(1,fov1d) = hirasmapqc(fov,Line)    ! ATOVPP QC word for AMSU
         hiras1d_prepro(2,fov1d) = hirasmapsurf(fov,Line)  ! estimated surface type
         hiras1d_prepro(3,fov1d) = NINT(hirassurfcost(fov,Line)*100.) ! max prob cloud
         hiras1d_prepro(4,fov1d) = NINT(hirasscatindx(fov,Line)*100.) ! scattering index
         hiras1d_prepro(5,fov1d) = NINT(hirascirrindx(fov,Line)*100.) ! cirrus index
         hiras1d_prepro(6,fov1d) = NINT(hirascrosby(fov,Line)*100.)   ! logistic precip
         IF (hirassi_nwcsaf(fov,Line).GT.-999.0) THEN
           hiras1d_prepro(7,fov1d) = NINT(hirassi_nwcsaf(fov,Line)*100) ! Bennartz index
         ELSE
           hiras1d_prepro(7,fov1d) = MDI
         ENDIF

         ! NWC-SAF Precip probabilities (each in range 0 to 15)
         i =  MIN(MAX(NINT(hirasm0(fov,Line)*15.0), 0), 15)        
         hiras1d_prepro(8,fov1d) = i                                 !Bits 0-3 (none)
         i =  MIN(MAX(NINT(hirasm1(fov,Line)*15.0), 0), 15)        
         hiras1d_prepro(8,fov1d) = IOR(hiras1d_prepro(8,fov1d),i*16) !Bits 4-7 (light)
         i =  MIN(MAX(NINT(hirasm2(fov,Line)*15.0), 0), 15)        
         hiras1d_prepro(8,fov1d) = IOR(hiras1d_prepro(8,fov1d),i*256) !Bits 8-11 (moderate)
         i =  MIN(MAX(NINT(hirasm3(fov,Line)*15.0), 0), 15)        
         hiras1d_prepro(8,fov1d) = IOR(hiras1d_prepro(8,fov1d),i*4096)!Bits 12-15 (intense)

         DO i = 1, 13
            hiras1d_emiss(i,fov1d) = MDI               ! reserved for emissivity
         ENDDO  ! i

!     Cloud flags
         DO i = 1, 10
            hiras1d_cloudflag(i,fov1d) = hirasCloudflag(i,fov1d,Line)
         ENDDO  ! i

      ENDDO  ! fov1d
             
      DO chn = 1, mwtsnchan
        hiras1d_nedtcoldmwts(chn) = hirasnedtcoldmwts(chn,Line)*10000
        hiras1d_nedtwarmmwts(chn) = hirasnedtwarmmwts(chn,Line)*10000
      ENDDO

      RETURN
      END
