!+ Set up HIRAS level 1D header within ATOVPP
!
      SUBROUTINE PPHIRASOUTH (LMWTS)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirasouth.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Set up level 1D header using information from 1C headers.
!
! Method:
! 1.  Check format version number and data type
!     HIRAS level 1C header has already been read into common /hiras1chd/
!     Check that format of 1C & 1D include files (hiras1c.h, hiras1d.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1C file is compatible with this subroutine.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Set up level 1D header
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     11/02/19  Created from ppcouth.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  HIRAS level 1D header/data common blocks 
#include "hiras1d.h"
!  fixed data for processing HIRAS 
#include "pphirasfdata.h"
!  fixed data for processing MWTS 
#include "ppmwts2fdata.h"
!  block of HIRAS mapped data 
#include "pphirasstore.h"
!  HIRAS level 1C header/data common blocks 
#include "hiras1c.h"
!  MWTS level 1C header/data common blocks 
#include "mwts21c.h"
!
! Local parameters:
      INTEGER       hiras1Cvnnb       ! version number HIRS 1C format compatible
         PARAMETER (hiras1Cvnnb=1  )  !    with the code in this subroutine
      INTEGER       mwts1Cvnnb       ! version number AMSU-A 1C format compatible
         PARAMETER (mwts1Cvnnb=1  )  !    with the code in this subroutine
      INTEGER       hiras1Dvnnb       ! version number 1D format compatible
         PARAMETER (hiras1Dvnnb=1  )  !    with the code in this subroutine
!
!   Scalar arguments with intent(In):
      LOGICAL       LMWTS         ! .TRUE. to process MWTS
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! general purpose loop variables
      INTEGER       chn           ! loop variable over channels
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 1.1 Are the 1C include files compatible with this source code ?
!     --- --- -- ------- ----- ---------- ---- ---- ------ ----

!     HIRAS
      IF ( hiras1Cvnnb .NE. hiras_1cvnnb ) THEN
         errStatus = -111  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but hiras1c.h is "",I6)")
     &        hiras1Cvnnb, hiras_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPCOUTH")
      ENDIF

!     MWTS
      IF ( LMWTS .AND. mwts1Cvnnb .NE. mwts2_1cvnnb ) THEN
         errStatus = -112  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but ama1c.h is "",I6)")
     &        mwts1Cvnnb, mwts2_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPHIRASOUTH")
      ENDIF


! 1.2 Is the 1D include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( hiras1Dvnnb .NE. hiras_1dvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1D format "",I6,"" but hiras1d.h is "",I6)")
     &        hiras1Dvnnb, hiras_1dvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPCOUTH")
      ENDIF


! 2.  Set up level 1D header
!     --- -- ----- -- ------

! 2.1 General

!     creation site for 1D data
      hiras1d_h_site   = SiteID              !file creation site ID (CHAR*3)
      hiras1d_h_cfill1 = " "                 !filler

!     creation site for original 1B data
      hiras1d_h_1bsite = hiras1c_h_1bsite
      hiras1d_h_cfill2 = " "                 !filler

      hiras1d_h_versnb = hiras_1dvnnb          !level 1d format version number
      hiras1d_h_versyr = hiras_1dvnyr          !level 1d format version year
      hiras1d_h_versdy = hiras_1dvndy          !level 1d format version day of year
      hiras1d_h_hdrcnt = hiras1c_h_hdrcnt      !count of header records
      hiras1d_h_satid = hiras1c_h_satid        !satellite id (e.g. 14 for NOAA-14)
      hiras1d_h_grid = hiras1c_h_instrument    !instrument grid code (HIRAS=13)
      hiras1d_h_satht = hiras1c_h_satht        !nominal sat height, km*10
      hiras1d_h_period = hiras1c_h_period      !nominal orbit period (seconds)
      hiras1d_h_startorbit = hiras1c_h_startorbit !orbit number (at start)
      hiras1d_h_startdatayr = hiras1c_h_startdatayr!start of data set year
      hiras1d_h_startdatady = hiras1c_h_startdatady! (ditto) day of the year
      hiras1d_h_startdatatime = hiras1c_h_startdatatime !(ditto) UTC time of day,ms
      hiras1d_h_endorbit = hiras1c_h_endorbit  !orbit number (at end)
      hiras1d_h_enddatayr = hiras1c_h_enddatayr!end of data set year
      hiras1d_h_enddatady = hiras1c_h_enddatady!(ditto) day of the year
      hiras1d_h_enddatatime = hiras1c_h_enddatatime !(ditto) UTC time of day,ms
      hiras1d_h_scnlin = hiras1c_h_scnlin      !count of scan lines in this dataset
      hiras1d_h_misscnlin = hiras1c_h_misscnlin !count of missing scan lines
      hiras1d_h_atovppvn = ATOVPPvn     !ATOVPP version number (test vns = 9000+)
      hiras1d_h_instruments = hirasinstruments   !instruments present
      hiras1d_h_spare1 = MDI                !spare
      hiras1d_h_wmosatid = hiras1c_h_wmosatid  !WMO satellite id (3 for METOP-1)
      hiras1d_h_centre = hiras1c_h_centre      !WMO centre id
      hiras1d_h_subcentre = hiras1c_h_subcentre!WMO subcentre id
      hiras1d_h_recwords = array_size_hiras1d  !Size of record
      hiras1d_h_nfov = 1                      !Number of FOVs in record
      hiras1d_h_nfor = hiras_1dnfor            !Number of FORs in record
      hiras1d_h_nband = hiras_1dnband          !Number of spectral bands
      DO i=1,hiras1d_h_nband
        hiras1d_h_startw(i) = hiras1c_h_startw(i)
        hiras1d_h_endw(i) = hiras1c_h_endw(i)
        hiras1d_h_startchan(i) = hiras1c_h_startchan(i)
        hiras1d_h_endchan(i) = hiras1c_h_endchan(i)
      ENDDO

! 2.2 temperature-radiance conversion coefficients etc.

!     MWTS
      IF ( LMWTS ) THEN
         hiras1d_h_mwtsbeamversion = mwts21c_h_beamversion
         DO chn = 1, mwtsnchan
         hiras1d_h_mwtsbeamwidth(chn) = mwts21c_h_beamwidth(chn)
         hiras1d_h_mwtscutoff(chn)    = mwts21c_h_cutoff(chn)
         hiras1d_h_mwtsnxaverage(chn) = mwts21c_h_nxaverage(chn)
         hiras1d_h_mwtsnyaverage(chn) = mwts21c_h_nyaverage(chn)
         DO i = 1, 3
            hiras1d_h_mwtsPlanck(i,chn) = mwts21c_h_tempradcnv(i,chn)
         ENDDO  ! i
         ENDDO  ! chn
      ELSE
         hiras1d_h_mwtsbeamversion = MDI
         DO chn = 1, mwtsnchan
         hiras1d_h_mwtsbeamwidth(chn) = MDI
         hiras1d_h_mwtscutoff(chn)    = MDI
         hiras1d_h_mwtsnxaverage(chn) = MDI
         hiras1d_h_mwtsnyaverage(chn) = MDI
         DO i = 1, 3
            hiras1d_h_mwtsPlanck(i,chn) = MDI
         ENDDO  ! i
         ENDDO  ! chn
      ENDIF

! 2.3 Pre-processing information

![user-defined:
      hiras1d_h_dT  = dT         ! size of data batches processed within ATOVPP
      hiras1d_h_mapBTmode = mwts2hirasmode ! mapping mode
      hiras1d_h_Acorrvn  = mwtscorrvn        ! vn no MWTS limb/emiss correction
      hiras1d_h_Apcpvn   = mwtspcpvn         ! vn no MWTS precip/scat detection
      hiras1d_h_Aantenna = mwts21c_h_vnantennacorr ! vn MWTS antenna corrections
      hiras1d_h_Ascat = mwtsmaxscat !threshold value for AMSU-A scattering index
      hiras1d_h_ACrosby = mwtsmaxcrosby !threshold value for AMSU-A logistic precip
      hiras1d_h_recperscan = nrec_hiras  !1d records per scan (1 or 4)
      hiras1d_h_fovmode = thinmode_hiras
      DO i = 1, array_size_hiras1dspare
         hiras1d_h_spare2(i) = MDI        !spare
      ENDDO  ! i
! ... ]

! 2.4 HIRAS variables

!      hiras1d_h_eigvn is already filled (see ppifdf.F)
      hiras1d_h_npc = hiras_1dnpc        !PCs in format
      hiras1d_h_npcused = npc_hiras      !PCs used
      hiras1d_h_nchan = hiras_1dnchan    !Channels in format
      hiras1d_h_nchanused = nchan_hiras  !Channels used

      DO i=1,hiras_1dnchan
        hiras1d_h_channels(i) = channels_hiras(i)
      ENDDO

! 2.5 MERSI

      hiras1d_h_nmersichan = hiras_1dnchanmersi
      hiras1d_h_nmersichanused = 0      !None at this stage

      RETURN
      END
