!+ HIRAS thinning
!
      SUBROUTINE PPHIRASTHIN ( line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphirasthin.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   For each spot, choose 1 detector out of 4 for inclusion in the 1d output
!
! Method:
!   The following options are provided, via HIRAS.fdf:
!   0) No thinning - include all 4 detectors
!   1) Fixed HIRAS detector
!   2) Highest mean HIRAS radiance for defined channel
!   3) Hybrid - scheme 1 in polar regions, scheme 2 elsewhere. Latitude
!      transition is defined in HIRAS.fdf.
!   4) Fixed sequence of all 4 detectors (1 per scan line)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2007, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     08/02/2019
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  HIRAS 1d format 
#include "hiras1d.h"
!  block of HIRAS data 
#include "pphirasstore.h"
!  fixed data 
#include "pphirasfdata.h"
!  HIRAS level 1C header/data common blocks 
#include "hiras1c.h"
!
!   Scalar arguments with intent(In):

      INTEGER line       !Scan line in ppistore

!   Local scalars:

      INTEGER   Cdet, Cfov       !Loop indices
      INTEGER   bestdet          !Best detector
      INTEGER   thinmode         !Copy of thinmode_HIRAS
      INTEGER   cmaxrad          !Integer maximum radiance
      INTEGER   rad              !radiance
      INTEGER   FOVorder(4)      !For use with option 4
      INTEGER   hiraschan_index   !position of hiraschan in the input dataset
      DATA      hiraschan_index/1/
      INTEGER   bestchandiff
      INTEGER   chn, i
      DATA FOVorder /1,2,3,4/
      REAL      radiance, radsd, maxrad, minrad       !Radiance-derived values
      REAL      mean_radiance, raddiff, bestraddiff
      REAL      Lat              !Latitude
      LOGICAL   First
      DATA First/.true./
!
!- End of header

      IF (First) THEN
        First = .false.
        bestchandiff = 99999
        DO chn=1,hiras1c_h_nchanused
          i = abs(hiras1c_h_channels(chn)-hiraschan)
          IF (i .lt. bestchandiff) then
            hiraschan_index = chn
            bestchandiff = i
          ENDIF
        ENDDO
      ENDIF

      IF (nrec_HIRAS .EQ. 1) THEN        !Only if thinning is required
        
!Thinning option 1: fixed detector for all FOVs and all scan lines
!Thinning option 4: fixed detector for all FOVs within a scan line

        IF (thinmode_HIRAS .EQ. 1. OR. thinmode_HIRAS .EQ. 4) THEN
          IF (thinmode_HIRAS .EQ. 1) THEN
            DO Cfov=1,hiras_1dnfor
              hirasbestFOV(Cfov,line) = default_HIRAS      !Fixed detector
            ENDDO
          ELSE
            hirasbestFOV(1,line) = FOVorder(MOD(line,4) + 1)
            IF (OKdetector_hiras(hirasbestFOV(1,line)).EQ.0) THEN
              hirasbestFOV(1,line) = default_HIRAS         !Can't use that detector
            ENDIF
            DO Cfov=2,hiras_1dnfor
              hirasbestFOV(Cfov,line) = hirasbestFOV(1,line)  !Fixed sequence
            ENDDO
          ENDIF
        ELSE
          DO Cfov=1,hiras_1dnfor   !Loop over FOVs
            thinmode = thinmode_HIRAS
            

!Thinning option 3: latitude dependent options. Thinmode is re-assigned
!depending on the latitude.

            IF (thinmode .EQ. 3) THEN
              Lat = hiras1c_latlon(1,1,Cfov)*1.0E-4    !First latitude
              IF (Lat.GT.latlimits_hiras(1) .OR.
     &            Lat.LT.latlimits_hiras(2)) THEN  !Polar
                thinmode = 1       !Fixed
                hirasbestFOV(Cfov,line) = default_HIRAS
              ELSE               !Not polar
                thinmode = 2       !Warmest HIRAS
              ENDIF
            ENDIF

!Thinning option 2: Highest mean HIRAS radiance for I/R channel

            IF (thinmode .EQ. 2) THEN
              cmaxrad = 0
              bestdet = default_HIRAS             !Default
              DO Cdet=1,4
                rad = hiras1c_radiance(Cdet,Cfov,hiraschan_index)
                IF (rad .GT. cmaxrad .AND. 
     &              OKdetector_hiras(Cdet).EQ.1) THEN
                  bestdet = Cdet
                  cmaxrad = rad
                ENDIF
              ENDDO
              hirasbestFOV(Cfov,line) = bestdet
            ENDIF    !thinmodethinmode_HIRAS
          ENDDO    !Cfov loop
        ENDIF    !not mode 1
      ENDIF    !nrec_HIRAS .EQ. 1
      END
