!+ Transfer data from program store to level 1D HIRS data record (TOVS)
!
      SUBROUTINE PPHOUTDM ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphoutdm.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   ATOVPP stores data from several scanlines in its own internal array
!   structure, contained in common/pphstore/. This routine transfers a single
!   scanline from slot "Line" in common/pphstore/ to common/hrs1ddtm/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1D include file (hrs1dm.h) and HIRS store
!     (pphstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
!     Fill parts of 1D data record that won't change each scanline
! 1.  Copy variables from slot "Line" in common/pphstore/ to common/hrs1ddt/
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1D format vn 1.9
!  1.2      17/01/97  Conform to 1D format vn 1.91
!  1.3      02/07/98  Hangles REAL not INTEGER
!  1.4      07/09/98  Update version numbers
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of HIRS data 
#include "pphstore.h"
!  HIRS/TOVS level 1D header/data common blocks 
#include "hrs1dm.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /pphstore/
!
! Local parameters:
      INTEGER       H1Dvnnb       ! version number of 1D format compatible
         PARAMETER (H1Dvnnb=191)  ! with the code in this subroutine
      INTEGER       Hstvnnb       ! version number of "pphstore.h" compatible
         PARAMETER (Hstvnnb=131)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov           ! loop variable over all fovs
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first                    ! .TRUE. on first call
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

! 0.1    Is the 1D include file compatible with this source code ?
         IF ( H1Dvnnb .NE. hrs_1dvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &     "("" Code expects 1D format"",I6,"" but hrs1dm.h is "",I6)")
     &           H1Dvnnb, hrs_1dvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPHOUTDM")
         ENDIF

! 0.2    Is the pphstore include file compatible with this source code ?
         IF ( Hstvnnb .NE. hrs_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects Hstore format"",I6,"" but pphstore.h is"",I6)")
     &           Hstvnnb, hrs_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPHOUTDM")
         ENDIF

! 0.3    Fill parts of 1D data record that won't change each scanline

!        AVHRR (..until we have anything to put in here)
         DO fov = 1, Hnfovs
         DO i = 1, 13
            hrs1d_avhrr(i,fov) = MDI
         ENDDO  ! i
         ENDDO  ! fov

         DO i = 1, 8
            hrs1d_filler(i) = MDI                     ! filler
         ENDDO  ! i

! 0.4    Reset "first" for subsequent calls
         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from slot "Line" in common/pphstore/ to common/hrs1ddtm/
!     ---- --------- ---- ----  ----  -- ---------------- -- ----------------

      hrs1d_scnlin = Hlineno(Line)                ! scanline number
      hrs1d_scnlinyr = Hyear(Line)                ! scanline year
      hrs1d_scnlindy = Hday(Line)                 ! scanline day of year
      hrs1d_scnlintime = Htime(Line)              ! scanline time (ms)
      hrs1d_qualind = Hqualind(Line)              ! quality indicator bit field
      hrs1d_scanqual = Hscanqual(Line)            ! scan line quality flags
      DO i = 1, 2
         hrs1d_spare1(i) = MDI                    ! spare
      ENDDO  ! i

      DO fov = 1, Hnfovs
         hrs1d_latlon(1,fov) = NINT(Hlats(fov,Line)*10000.) !latitude (N +ve)
         hrs1d_latlon(2,fov) = NINT(Hlons(fov,Line)*10000.) !longitude (E +ve)
         hrs1d_surface(1,fov) = Helev(fov,Line)   !surface height (m)
         hrs1d_surface(2,fov) = Hsurftype(fov,Line) !surface type
         DO i = 1, 4
            hrs1d_angles(i,fov) = NINT(Hangles(i,fov,Line)*100.) ! scan angles:
!                                         (1,*,*) local zenith angle (deg*100)
!                                         (2,*,*) local azimuth angle (deg*100)
!                                         (3,*,*) solar zenith angle (deg*100)
!                                         (4,*,*) solar azimuth angle (deg*100)
         ENDDO  ! i
      ENDDO  ! fov

      hrs1d_scalti = Haltitude(Line)              ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
      hrs1d_spare2 = MDI                          ! spare

!     HIRS BTs (deg K * 100)
      DO fov = 1, Hnfovs
      DO chn = 1, Hnchan
         hrs1d_btemps(chn,fov) = NINT(Hbtemps(chn,fov,Line)*100.)
      ENDDO  ! chn
      ENDDO  ! fov

!     Mapped MSU BTs (deg K * 100)
      DO fov = 1, Hnfovs
      DO chn = 1, Mnchan
         hrs1d_btemps(chn+Hnchan,fov) = NINT(Hmapbt(chn,fov,Line)*100.)
      ENDDO  ! chn
      ENDDO  ! fov

      DO fov = 1, Hnfovs
         hrs1d_hirsqual(fov) = Hqc(fov,Line)     ! QC word for HIRS
      ENDDO  ! fov

![user-defined:
      DO fov = 1, Hnfovs
         hrs1d_prepro(1,fov) = Hmapqc(fov,Line)    ! ATOVPP QC word for MSU
         DO i = 2, 4
            hrs1d_prepro(i,fov) = MDI              ! spare
         ENDDO  ! i
      ENDDO  ! fov
! ... ]


      RETURN
      END
