!+ Set up HIRS/TOVS level 1D header within ATOVPP
!
      SUBROUTINE PPHOUTHM ( LMSU )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pphouthm.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Set up level 1D header using information from 1C headers.
!
! Method:
! 1.  Check format version number and data type
!     HIRS level 1C header has already been read into common /hrs1chd/
!     Check that format of 1C & 1D include files (hrs1c.h, hrs1dm.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1C file is compatible with this subroutine.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Set up level 1D header
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1D format vn 1.9
!  1.2      17/01/97  Conform to 1D format vn 1.91
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  fixed data for processing HIRS 
#include "pphfdata.h"
!  fixed data for processing MSU 
#include "ppmfdata.h"
!  block of HIRS data 
#include "pphstore.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!  HIRS/TOVS level 1D header/data common blocks 
#include "hrs1dm.h"
!
! Local parameters:
      INTEGER       H1Cvnnb       ! version number HIRS 1C format compatible
         PARAMETER (H1Cvnnb=190)  !    with the code in this subroutine
      INTEGER       M1Cvnnb       ! version number MSU 1C format compatible
         PARAMETER (M1Cvnnb=180)  !    with the code in this subroutine
      INTEGER       H1Dvnnb       ! version number 1D format compatible
         PARAMETER (H1Dvnnb=191)  !    with the code in this subroutine
!
!   Scalar arguments with intent(In):
      LOGICAL       LMSU          ! .TRUE. to process MSU
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! general purpose loop variables
      INTEGER       chn           ! loop variable over channels
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 1.1 Are the 1C include files compatible with this source code ?
!     --- --- -- ------- ----- ---------- ---- ---- ------ ----

!     HIRS
      IF ( H1Cvnnb .NE. hrs_1cvnnb ) THEN
         errStatus = -11  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but hrs1c.h is "",I6)")
     &        H1Cvnnb, hrs_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPHOUTHM")
      ENDIF

!     MSU
      IF ( LMSU .AND. M1Cvnnb .NE. msu_1cvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but msu1c.h is "",I6)")
     &        M1Cvnnb, msu_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPHOUTHM")
      ENDIF

! 1.2 Is the 1D include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( H1Dvnnb .NE. hrs_1dvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1D format "",I6,"" but hrs1dm.h is "",I6)")
     &        H1Dvnnb, hrs_1dvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPHOUTHM")
      ENDIF


! 2.  Set up level 1D header
!     --- -- ----- -- ------

!!!   This code is compatible with files:
!!!   "hrs1c.h" vn1.8       12/12/96
!!!   "hrs1dm.h" vn1.9      12/12/96

! 2.1 General

!     creation site for 1D data
      hrs1d_h_site   = SiteID              !file creation site ID (CHAR*3)
      hrs1d_h_cfill1 = " "                 !filler

!     creation site for original 1B data
      hrs1d_h_1bsite = hrs1c_h_1bsite
      hrs1d_h_cfill2 = " "                 !filler

      hrs1d_h_versnb = hrs_1dvnnb          !level 1d format version number
      hrs1d_h_versyr = hrs_1dvnyr          !level 1d format version year
      hrs1d_h_versdy = hrs_1dvndy          !level 1d format version day of year
      hrs1d_h_hdrcnt = hrs1c_h_hdrcnt      !count of header records
      hrs1d_h_satid = hrs1c_h_satid        !satellite id (e.g. 14 for NOAA-14)
      hrs1d_h_grid = hrs1c_h_instrument    !instrument grid code (HIRS=5)
      hrs1d_h_satht = hrs1c_h_satht        !nominal sat height, km*10
      hrs1d_h_period = hrs1c_h_period      !nominal orbit period (seconds)
      hrs1d_h_startorbit = hrs1c_h_startorbit !orbit number (at start)
      hrs1d_h_startdatayr = hrs1c_h_startdatayr!start of data set year
      hrs1d_h_startdatady = hrs1c_h_startdatady! (ditto) day of the year
      hrs1d_h_startdatatime = hrs1c_h_startdatatime !(ditto) UTC time of day,ms
      hrs1d_h_endorbit = hrs1c_h_endorbit  !orbit number (at end)
      hrs1d_h_enddatayr = hrs1c_h_enddatayr!end of data set year
      hrs1d_h_enddatady = hrs1c_h_enddatady!(ditto) day of the year
      hrs1d_h_enddatatime = hrs1c_h_enddatatime !(ditto) UTC time of day,ms
      hrs1d_h_scnlin = hrs1c_h_scnlin      !count of scan lines in this dataset
      hrs1d_h_misscnlin = hrs1c_h_misscnlin !count of missing scan lines
      hrs1d_h_atovppvn = ATOVPPvn     !ATOVPP version number (test vns = 9000+)
!     instruments present:
      hrs1d_h_instruments = 0
      hrs1d_h_instruments = IBSET(hrs1d_h_instruments,bits(0))  ! for HIRS
      IF ( LMSU )
     &   hrs1d_h_instruments = IBSET(hrs1d_h_instruments,bits(1)) ! for MSU
      hrs1d_h_wmosatid = hrs1c_h_wmosatid
      hrs1d_h_spare1 = MDI              !spare

! 2.2 temperature-radiance conversion coefficients

!     HIRS
!                  order of channels = 1,2,3,4,...,18,19
!                  for nochannel = 1 to 19 :
!                      10^6 x (nochannel central wavenumber)
!                      10^6 x (nochannel constant 1)
!                      10^6 x (nochannel constant 2)
!     IR channels
      DO chn = 1, Hnchan-1
      DO i = 1, 3
         hrs1d_h_hrsPlanck(i,chn) = hrs1c_h_tempradcnv(i,chn)
      ENDDO  ! i
      ENDDO  ! chn
!     visible channel
      hrs1d_h_20solfiltirrad= hrs1c_h_20solfiltirrad
!                  10^6 x (albedo-radiance ch20 solar filtered irradiance,W/m2)
      hrs1d_h_20equifiltwidth= hrs1c_h_20equifiltwidth
!                  10^6 x (albedo-radiance ch 20 equivalent filter width cm^-1)

      hrs1d_h_spare2 = MDI           !spare

!     MSU
      IF ( LMSU ) THEN
         DO chn = 1, Mnchan
         DO i = 1, 3
            hrs1d_h_msuPlanck(i,chn) = msu1c_h_tempradcnv(i,chn)
         ENDDO  ! i
         ENDDO  ! chn
      ELSE
         DO chn = 1, Mnchan
         DO i = 1, 3
            hrs1d_h_msuPlanck(i,chn) = MDI
         ENDDO  ! i
         ENDDO  ! chn
      ENDIF

! 2.3 Pre-processing information

![user-defined:
      hrs1d_h_dT  = dT         ! size of data batches processed within ATOVPP
      hrs1d_h_mapBTmode = M2Hmode  ! mapping mode (MSU to HIRS)
      hrs1d_h_Hcorrvn  = Hcorrvn        ! vn no HIRS limb/emiss correction
      hrs1d_h_Hcloudvn = Hcloudvn       ! vn no HIRS cloud correction
      hrs1d_h_Mcorrvn  = Mcorrvn        ! vn no MSU limb/emiss correction

      DO i = 1, 116
         hrs1d_h_spare3(i) = MDI           !spare
      ENDDO  ! i
! ... ]


      RETURN
      END
