!+ IASI cloud tests
!
      SUBROUTINE PPICLOUD ( Testnumber, Ilastline )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppicloud.F 182 2010-01-12 15:50:02Z frna $
!
! Purpose:
!    NWP-independent cloud detection tests for IASI
!
! Method:
!    Currently 2 tests are implemented:
!    1) Standard deviation of IASI radiances for the 4 detectors at each scan 
!       position
!    2) Make use of mapped AMSU in order to simulate IASI radiances
!    
!    Reference for both the above: Cheng et. al., 2006, 'The Use of Principal
!    Components Analysis (PCA) in Processing Simulated IASI Data', ITSC-15,
!    poster A17
!
!    Results are stored in the ICloudflag array, which has space for 10 cloud
!    tests. The last slot is used as a clear/cloudy flag (1 bit per test)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2007, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    19/03/2007 New. N C Atkinson
!  1.1    11/11/2008 Use the correct channel number and modify thresholds.
!                    (Had been incorrectly using 2400cm-1 window channel.)
!  1.2    18/05/2009 Re-organise to accommodate PC format input data
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  IASI 1c format 
#include "iasi1c.h"
!  IASI PC format 
#include "iasipc.h"
!  IASI 1d format 
#include "iasi1d.h"
!  block of IASI level 1d data 
#include "ppistore.h"
!  fixed data for processing IASI 
#include "ppifdata.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  constants 
#include "constants_preproc.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Testnumber    ! cloud test number
      INTEGER       Ilastline     ! number of last IASI line in store
                                  !  (or current line for test 1)
      INTEGER       BestDet       ! Detector to use (1 to 4)
      INTEGER       Ifov          ! FOV number (1 to 30)
!
! Local scalars:
      REAL          radmean, radsum
      REAL          T4, T5, T6, csza, cview
      REAL          radiance, bri_temp, w, T, drdt, radsd, bt_pred
      INTEGER       scalrad(4)
      INTEGER       fov, Ichan, ndet
      INTEGER       I,Idet,line     
      INTEGER       Cloudchan
      DATA Cloudchan / 6981 /
      DATA w / 2390.0 /
!
!- End of header

!Update header record to indicate that this test has been performed
      iasi1d_h_cloudtests=IBSET(iasi1d_h_cloudtests,BITS(Testnumber-1))

      IF (Testnumber .eq. 1) THEN

!---------- Test 1: Standard deviation of 4 IASI FOVs at 2390 cm^-1 -----------
!
!Reference: Cheng et. al., ITSC-15, poster A17
!
!This test must be done while the IASI 1c are being read (see ppipc1d)
!------------------------------------------------------------------------------

        T = 280.0
        drdt = C_1*w*w*w/(exp(w*C_2/T)-1)**2 * exp(w*C_2/T) *C_2*w/(T*T)

        DO fov=1,iasi_1dnfov
          radmean = 0.0
          ndet = 0
          DO Idet=1,4
            scalrad(Idet) = iasi1c_scalrad(Cloudchan,Idet,fov)
            IF (scalrad(Idet).ne.0 .and. OKdetector(Idet).eq.1) THEN
              ndet = ndet + 1
              radmean = radmean + scalrad(Idet)
            ENDIF
          ENDDO
          IF (ndet .ge. 2) THEN         !require at least 2 good detectors
            radmean = radmean/ndet
            radsd = 0.0
            DO Idet=1,4
              IF (scalrad(Idet).ne.0 .and. OKdetector(Idet).eq.1) THEN
                radsd = radsd + (scalrad(Idet)-radmean)**2
              ENDIF
            ENDDO
            radsd = sqrt(radsd/(ndet-1))*Scalefactors(Cloudchan)/drdt

!Store 100*(std deviation referenced to 280K)

            ICloudflag(Testnumber,fov,Ilastline) = radsd*100

!Set master flag (word 10) if threshold is exceeded
!Assume NEdT at 280K is 0.42K

            IF (radsd .GT. 3*0.42) THEN
              ICloudflag(10,fov,Ilastline) =
     &           IBSET(ICloudflag(10,fov,Ilastline),BITS(Testnumber-1))
            ENDIF

          ELSE
            ICloudflag(Testnumber,fov,Ilastline) = MDI   !No IASI
          ENDIF

        ENDDO

      ELSE IF (Testnumber .eq. 2) THEN       

!---------- Test 2: IASI 2390 cm^-1 radiance predicted from AMSU -----------
!
!Reference: Cheng et. al., ITSC-15, poster A17
!------------------------------------------------------------------------------

        DO line = 1, ILastLine
          DO fov=1,iasi_1dnfov
            T4  = Imapbt(4,fov,line)
            T5  = Imapbt(5,fov,line)
            T6  = Imapbt(6,fov,line)
            csza = COS(Iangles(3,fov,line)*deg2rad)    !Cos solar zenith angle
            cview = COS((fov-15.5)*3.33333*deg2rad)    !Cos scan angle
            IF (T4.GT.RMDI .AND. T5.GT.RMDI .AND. T6.GT.RMDI) THEN

              bt_pred = 44.007 - 0.272*T4 + 1.837*T5 - 0.738*T6
     &                  + 0.004*csza + 6.391*(1-cview)

!We already have the scaled radiance (see ppi1d)

              radiance = ICloudflag(Testnumber,fov,line)*
     &              Scalefactors(Cloudchan)
              
              IF (radiance.GT.0.) THEN
                bri_temp = C_2*w / log(C_1*w*w*w/radiance + 1)

!Over-write ICloudflag with the BT diff (*100)

                ICloudflag(Testnumber,fov,line)=(bt_pred - bri_temp)*100

!Set master flag (word 10) if threshold is exceeded

                IF (bt_pred - bri_temp .GT. 2.0) THEN
                  ICloudflag(10,fov,line) =
     &              IBSET(ICloudflag(10,fov,line),BITS(Testnumber-1))
                ENDIF

              ELSE
                ICloudflag(Testnumber,fov,line) = MDI      !No valid IASI
              ENDIF
     
            ELSE
              ICloudflag(Testnumber,fov,line) = MDI      !No AMSU
            ENDIF
          ENDDO
        ENDDO
      
      ENDIF     !end test 2
      END

