!+ Read fixed data file for IASI within ATOVPP
!
      SUBROUTINE PPIFDF (Ifunit,Ieigunit1,Ieigunit2,Inoiseunit,Imtfunit)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppifdf.F 382 2014-01-30 14:19:05Z frna $
!
! Purpose:
!   Read fixed data file required for IASI within ATOVPP.
!
! Method:
! 1.  Read channel selection and thinning requirements
! 2.  Read eigenvectors for encoding (radiance to PC)
! 3.  Read eigenvectors for decoding (PC to radiance)
! 4.  Read noise profile, nedr (for generating reconstruction score)
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    10/08/05  Original code. (N.C.Atkinson)
!  1.1    20/03/07  Read new OKdetector flags; initialise iasi1d_h_cloudtests
!  1.2    22/01/08  Read pccount, eigformat, evlevel, mean_encode_IASI,
!                     mean_decode_IASI
!  1.3    12/03/09  Use Imaxnpc instead of iasi_1dnpc.
!                   Only open eigenvector/noise/MTF file if unit is non zero.
!  1.4    01/09/09  Add evfile
!  1.5    30/01/14  Use initial value of npc_IASI if that is higher than the
!                   value in IASI.fdf.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1d format 
#include "iasi1d.h"
!  fixed data for processing IASI 
#include "ppifdata.h"
!  fixed data for processing IASI 
#include "reclopt.h"
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       Ifunit        ! unit numbers
      INTEGER       Ieigunit1
      INTEGER       Ieigunit2
      INTEGER       Inoiseunit
      INTEGER       Imtfunit
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status value
      INTEGER       chn           ! loop index: channel number
      INTEGER       Ipc           ! loop index: PC number
C      INTEGER       eigvn         ! eigenvector version number
      INTEGER       nchan_max     ! number of chans in eigenvector file
      INTEGER       npc_max       ! number of PCs in eigenvector file
      INTEGER       npc_IASI_tmp  ! value read in from IASI.fdf
      REAL*4        tmp(Inchan)   ! temporary storage for eigenvectors

! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine

!- End of header


! 1.  open and read fdf file
!     ----------------------

      OPEN ( UNIT=Ifunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!     If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=Ifunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

      WRITE ( errMessage,
     & "("" Error opening IASI fixed data file, unit "",I8)" ) Ifunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPIFDF")


!     look for sections in the file
!     ---- --- -------- -- --- ----

      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )
        input_line = " "
        READ(Ifunit,"(A80)",IOSTAT=errStatus) input_line

        IF ( input_line(1:5) .EQ. "START" ) THEN

!     Read file
!     ---- ----

          READ(Ifunit,*) thinmode_IASI !Thinning mode: 0 = full resolution.
                                       !               1 = fixed detector
                                       !               2 = warmest AVHRR
                                       !               3 = min variability
                                       !               4 = hybrid 2 and 3
                                       !               5 = warmest IASI
                                       !               6 = hybrid 5 and 1

          IF (thinmode_IASI .EQ. 0) THEN
            nrec_IASI = 4       !Number of 1d records to output per scan
            FirstFOV_IASI = 1
            LastFOV_IASI = 4
          ELSE
            nrec_IASI = 1
          ENDIF

          READ(Ifunit,*) thinmode_backup !backup mode if no AVHRR (1, 5 or 6)

          READ(Ifunit,*) default_IASI !Default detector

          IF (thinmode_IASI .NE. 1) THEN
            FirstFOV_IASI = 1                   !Need all 4 detectors
            LastFOV_IASI = 4
          ELSE
            FirstFOV_IASI = default_IASI        !Fixed detector mode
            LastFOV_IASI = default_IASI
          ENDIF

          READ(Ifunit,*) (OKdetector(chn), chn=1,4) !status of the 4 detectors

          READ(Ifunit,*) latlimits(1),latlimits(2)  !Latitude limits for options
                                                    !4 & 6 (N then S)
          READ(Ifunit,*) AVHRRchan_IASI  !AVHRR channel (1 to 6) for options 2-4

          READ(Ifunit,*) IASIchan      !IASI channel (1 to 8461) for options 5-6

          READ(Ifunit,*) npc_IASI_tmp  !Number of PC scores to output in 1d
          npc_IASI = MAX(npc_IASI,npc_IASI_tmp)  !Use initial value if higher

          IF (npc_IASI .GT. Imaxnpc) THEN
            WRITE (*,*) "Warning: IASI fdf file specifies too many PCs"
            WRITE (*,*) "Maximum is Imaxnpc=", Imaxnpc
            WRITE (*,*) "Modify ppparms.h to increase and recompile"
            npc_IASI = Imaxnpc
          ENDIF
          
          READ(Ifunit,*) chanvn           !Channel selection version number
          iasi1d_h_chanvn = chanvn
          READ(Ifunit,*) nchan_IASI       !Number of channels to output in 1d

          READ(Ifunit,*) (channels_IASI(chn), chn=1,nchan_IASI)

        ENDIF
      ENDDO

      CLOSE(Ifunit)

!Read environment variable ATOVPP_USE_RR
      
      CALL GETENV ('ATOVPP_USE_RR',input_line)
      IF (input_line(1:1) .EQ. 'T') THEN
        Use_reconstructed_radiances = .TRUE.
        WRITE(*,'(10X,A)') "Use reconstructed radiances"
      ELSE
        Use_reconstructed_radiances = .FALSE.
      ENDIF

! 2.  Read eigenvector file for encoding (direct access binary)
!     ----------------------------------

      IF (npc_IASI .GT. 1) THEN

       IF (Ieigunit1 .GT. 0) THEN
        OPEN ( UNIT=Ieigunit1, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=Inchan*4/rldiv, IOSTAT=errStatus )

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Ieigunit1, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=Inchan*4/rldiv, IOSTAT=errStatus )

        WRITE ( errMessage,
     &   "("" Error opening IASI eigenvector encode file, unit "",I8)" )
     &   Ieigunit1
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPIFDF")

        READ(Ieigunit1,rec=1) eigvn,nchan_max,pccount,eigformat,evlevel,
     &      startpcchan,endpcchan,startpc,endpc,evfile   !Read header record
        npcbands = 3
        IF (startpcchan(3).EQ.0) npcbands=2
        IF (startpcchan(2).EQ.0) npcbands=1
        IF (startpcchan(1).EQ.0) THEN
          startpcchan(1) = 1
          endpcchan(1) = 8461
          startpc(1) = 1
          endpc(1) = min(npc_iasi,pccount)
        ENDIF
        
        IF (nchan_max.LT.Inchan) THEN
          WRITE ( errMessage,
     &     "("" Not enough channels in eigenvector file"")" )
          errStatus = 99
          CALL ErrorReport(errStatus,errMessage,"PPIFDF")
        ENDIF
          
        iasi1d_h_eigvn = eigvn       !Store version number of eigenvectors

        IF (eigformat.EQ.1) THEN
          READ(Ieigunit1,rec=2) mean_encode_IASI
        ELSE
          eigformat = 0   !Only allow 0 and 1
        ENDIF

        IF (pccount.LT.npc_IASI) THEN
          WRITE (*,*) "WARNING: number of PCs requested is more than "//
     &       "number of eigenvectors in file",npc_IASI,pccount
          npc_IASI = pccount
        ENDIF

        DO Ipc=1,npc_IASI
          READ(Ieigunit1,rec=Ipc+eigformat+1) tmp
          DO chn=1,Inchan
            eig_encode_IASI(chn,Ipc) = tmp(chn)
          ENDDO
        ENDDO
      
        IF (evlevel.NE.'1C') evlevel = '1B'   !Only 1B and 1C supported

        CLOSE(Ieigunit1)
       ENDIF

! 3.  Read eigenvector file for decoding (direct access binary)
!     ----------------------------------

       IF (Ieigunit2 .GT. 0) THEN
        OPEN ( UNIT=Ieigunit2, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=Inchan*4/rldiv, IOSTAT=errStatus )

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Ieigunit2, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=Inchan*4/rldiv, IOSTAT=errStatus )

        WRITE ( errMessage,
     &   "("" Error opening IASI eigenvector decode file, unit "",I8)" ) 
     &   Ieigunit2
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPIFDF")

        READ(Ieigunit2,rec=1) eigvn,nchan_max,pccount,eigformat,evlevel,
     &      startpcchan,endpcchan,startpc,endpc,evfile   !Read header record
        npcbands = 3
        IF (startpcchan(3).EQ.0) npcbands=2
        IF (startpcchan(2).EQ.0) npcbands=1
        IF (startpcchan(1).EQ.0) THEN
          startpcchan(1) = 1             !Default is all channels together
          endpcchan(1) = 8461
          startpc(1) = 1
          endpc(1) = min(npc_iasi,pccount)
        ENDIF

        IF (eigvn.ne.iasi1d_h_eigvn) THEN
          WRITE(*,*) "WARNING: eigenvector version number for " //
     &       "decoding differs from encoding",eigvn,iasi1d_h_eigvn
        ENDIF

        IF (eigformat.EQ.1) THEN
          READ(Ieigunit2,rec=2) mean_decode_IASI
        ELSE
          eigformat = 0   !Only allow 0 and 1
        ENDIF

        IF (pccount.LT.npc_IASI) npc_IASI = pccount

        DO Ipc=1,npc_IASI
          READ(Ieigunit2,rec=Ipc+eigformat+1) tmp
          DO chn=1,Inchan
            eig_decode_IASI(chn,Ipc) = tmp(chn)
          ENDDO
        ENDDO
      
        IF (evlevel.NE.'1C') evlevel = '1B'   !Only 1B and 1C supported

        CLOSE(Ieigunit2)
       ENDIF

! 4.  Read nedr file (direct access binary)
!     --------------

       IF (Inoiseunit .GT. 0) THEN
        OPEN ( UNIT=Inoiseunit, STATUS="OLD", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=Inchan*4/rldiv, IOSTAT=errStatus )

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Inoiseunit, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &       ACCESS="DIRECT", RECL=Inchan*4/rldiv, IOSTAT=errStatus )

        WRITE ( errMessage,
     &   "("" Error opening IASI eigenvector file 2, unit "",I8)" ) 
     &   Inoiseunit
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPIFDF")

        READ(Inoiseunit,rec=1) nedr_IASI
      
        CLOSE(Inoiseunit)
       ENDIF

! 5.  Read MTF file (ASCII)
!     -------------

       IF (evlevel .EQ. '1B' .AND. Imtfunit .GT. 0) THEN
        OPEN ( UNIT=Imtfunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

!       If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
        IF ( errStatus.NE.0 )
     &    OPEN ( UNIT=Imtfunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )

        WRITE ( errMessage,
     &   "("" Error opening IASI MTF file, unit "",I8)" ) 
     &   Imtfunit
        errStatus = ABS (errStatus)   ! end-of-file returns -1
        CALL ErrorReport(errStatus,errMessage,"PPIFDF")

        READ(Imtfunit,*) mtf_IASI
      
        CLOSE(Imtfunit)
       ENDIF         ! evlevel = 1B
      ENDIF         ! npc_IASI .GT. 1

!Initialise cloud tests flag
      iasi1d_h_cloudtests = 0

      RETURN
      END
