!+ Transfer level 1C IASI data from data record to program store
!
      SUBROUTINE PPIIND ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppiind.F 316 2012-05-18 12:52:10Z frna $
!
! Purpose:
!   IASI 1C data, when read, is held in common/iasi1cdt/. ATOVPP stores data
!   from several scanlines in its own internal array structure, contained
!   in common/ppistore/. This routine transfers a single scanline from
!   common/iasi1cdt/ to slot "Line" in common/ppistore/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1C include file (iasi1c.h) and IASI store
!     (ppistore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Copy variables from common/iasi1cdt/ to slot "Line" in common/ppistore/
! 2.  Check flags
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     18/01/05  N.C.Atkinson, based on pphind.F
!  1.1     18/05/2012 Set I1Cvnnb to 2 for consistency with iasi1c.h.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of IASI data 
#include "ppistore.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppistore/
!
! Local parameters:
      INTEGER       I1Cvnnb       ! version number of 1C format compatible
         PARAMETER (I1Cvnnb=2)    ! with the code in this subroutine
      INTEGER       Istvnnb       ! version number of "ppistore.h" compatible
         PARAMETER (Istvnnb=1)    ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chanbits      ! bit word with all channel bits set
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov           ! loop variable over all fovs
      INTEGER       det           ! detector
      INTEGER       pos           ! scan position
      INTEGER       qc0           ! qc flagword, bit 0 set
      LOGICAL       badline       ! .TRUE. not to use a scanline
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first, chanbits
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!        Is the 1C include file compatible with this source code ?
         IF ( I1Cvnnb .NE. iasi_1cvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but iasi1c.h is "",I6)")
     &           I1Cvnnb, iasi_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPIIND")
         ENDIF

!        Is the ppistore include file compatible with this source code ?
         IF ( Istvnnb .NE. iasi_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects Istore format"",I6,"" but ppistore.h is"",I6)")
     &           Istvnnb, iasi_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPIIND")
         ENDIF

!        set up a mask to pick out just the flags for the individual detectors
         chanbits = 0
         DO chn = 1, Indet
            chanbits = IBSET ( chanbits, bits(chn) )
         ENDDO

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from common/iasi1cdt/ to slot "Line" in common/ppistore/
!     ---- --------- ---- --------------- -- ----  ----  -- ----------------

      Ilineno(Line) = iasi1c_scnlin                ! scanline number
      Iyear(Line) = iasi1c_scnlinyr                ! scanline year
      Iday(Line) = iasi1c_scnlindy                 ! scanline day of year
      Itime(Line) =  iasi1c_scnlintime(1)          ! scanline time (ms)

      DO fov = 1, Infovs      !store 120 fovs, 1-30=detector 1, etc. 
         det = (fov-1)/30 + 1   !detector
         pos = MOD(fov-1,30) + 1  !scan position
         Ilats(fov,Line) = REAL(iasi1c_latlon(1,det,pos))*1.E-4 !latitude(deg,N+ve)
         Ilons(fov,Line) = REAL(iasi1c_latlon(2,det,pos))*1.E-4 !longitude (E+ve)
         DO i = 1, 4
            Iangles(i,fov,Line) = REAL(iasi1c_angles(i,det,pos))*0.01 ! scan angles:
!                                         (1,*,*) local zenith
!                                         (2,*,*) local azimuth
!                                         (3,*,*) solar zenith
!                                         (4,*,*) solar azimuth
         ENDDO  ! i
      ENDDO  ! fov

      Ialtitude(Line) =  iasi1c_scalti            ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
      DO fov = 1, Infovs
      DO chn = 1, Anchan+Bnchan
         ImapBT(chn,fov,Line) = RMDI
      ENDDO  ! chn
      ENDDO  ! fov

      DO fov = 1, Infovs
         Iqc(fov,Line) = 0  ! QC word for IASI
      ENDDO  ! fov

!     initialise other variables
      qc0 = 0
      qc0 = IBSET(qc0,bits(0)) ! QC word: "AMSU missing"
      DO fov = 1, Infovs
         Imapsurf(fov,Line)  = MDI   ! estimated AMSU-A surface type
         Isurfcost(fov,Line) = RMDI  ! AMSU-A max prob cloud cost
         Iscatindx(fov,Line) = RMDI  ! AMSU-A scattering index
         Icrosby(fov,Line)   = RMDI  ! AMSU-A logistic precip probability
         Imapqc(fov,Line)    = qc0   ! AMSU QC word
      ENDDO  ! fov

! 2.  Check flags
!     ----- -----

! 2.1 Flags for the whole scanline

      badline = .FALSE.       !There are no whole-scanline flags at present

!     For bad scanline, flag each fov
      IF ( badline ) THEN
         DO fov = 1, Infovs
            Iqc(fov,Line) = IBSET ( Iqc(fov,Line), bits(0) )
         ENDDO  ! fov
      ENDIF


      RETURN
      END
