!+ Read & interpret IASI level 1C header within ATOVPP
!
      SUBROUTINE PPIINH ( I1Cunit, SatId, WMOSatId,
     &                    StartYearI, StartDayI, StartTimeI,
     &                    EndYearI,   EndDayI,   EndTimeI )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppiinh.F 182 2010-01-12 15:50:02Z frna $
!
! Purpose:
!   Open input dataset and read level 1C IASI header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     IASI level 1C header has already been read into common /iasi1chd/
!     Check that format of 1C include files (iasi1c.h) is compatible with the
!     code in this subroutine. Check that format of 1C file is compatible with
!     this subroutine, and that data type from the header is 10 for IASI.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from IASI 1C header
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      23/08/05  Based on ppainh.F. N.C.Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of IASI data 
#include "ppistore.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
! Local parameters:
      INTEGER       I1Cvnnb       ! version number of 1C format compatible
         PARAMETER (I1Cvnnb=1)  ! with the code in this subroutine
!
!   Scalar arguments with intent(In):
      INTEGER       I1Cunit       ! logical unit no. for IASI 1C input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 3 for METOP-1)
      ! start/end date/time of IASI data:
      INTEGER       StartYearI, EndYearI
      INTEGER       StartDayI,  EndDayI
      INTEGER       StartTimeI, EndTimeI
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       I1Crecord     ! record number for IASI level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
      INTEGER       Ichan,J
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOI1C                ! I/O routine for level 1C IASI data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.1 Open 1C file
      CALL IOI1C ("OPEN/READ", I1Cunit, I1Crecord )   ! 1C file

! 1.2 Read 1C header
      I1Crecord = 1   ! Headers start on record 1.
      CALL IOI1C ("READ HEADER", I1Cunit, I1Crecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( I1Cvnnb .NE. iasi_1cvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but iasi1c.h is "",I6)")
     &        I1Cvnnb, iasi_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "ppiINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "iasi1c.h" contains its own version number "iasi_h_l1cversnb"
      IF ( iasi1c_h_versnb .NE. iasi_1cvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level 1C IASI file is "",I8,"", not "",I8)")
     &        iasi1c_h_versnb,iasi_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "ppiINH")
      ENDIF

! 2.3 Is the 1C data IASI ?
!     -- --- -- ---- ------
!     "iasi1c_h_instrument" from 1C header should be 13 for IASI
      IF ( iasi1c_h_instrument .NE. 13 ) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage, "("" Data type in 1C IASI file is "",
     &           I8,""; not 13 for IASI"")") iasi1c_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "ppiinh")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "##########"
      WRITE(*,*) "# IASI   #"
      WRITE(*,*) "##########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            iasi1c_h_versnb, iasi1c_h_versyr, iasi1c_h_versdy
      WRITE(*,"(I4,"" header records in file."")") iasi1c_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") iasi1c_h_satid
      WRITE(*,"(""   WMO Satellite id is "",I4)") iasi1c_h_wmosatid

!     calculate start and end dates/times from header
!     ..start
      year = iasi1c_h_startdatayr
      yrday = iasi1c_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(iasi1c_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = iasi1c_h_enddatayr
      yrday = iasi1c_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(iasi1c_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          iasi1c_h_scnlin, iasi1c_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from IASI 1C header
!     ------- -------- ----------- ---- ------ -- ------

      ISatId = iasi1c_h_satid
      SatId = ISatId
      WMOSatId = iasi1c_h_wmosatid
      StartDayI = iasi1c_h_startdatady
      StartYearI = iasi1c_h_startdatayr
      StartTimeI = iasi1c_h_startdatatime
      EndDayI = iasi1c_h_enddatady
      EndYearI = iasi1c_h_enddatayr
      EndTimeI = iasi1c_h_enddatatime
      Isatht = REAL(iasi1c_h_satht)*0.1      !nominal sat height (km)
      Isatperiod = REAL(iasi1c_h_period)/60. !nominal orbit period (minutes)
      Ihdrcnt = iasi1c_h_hdrcnt       !count of header records in 1C file
      Iscnlin = iasi1c_h_scnlin       !count of scanlines in 1C file
      Iinstruments = IBSET ( Iinstruments , bits(6) )   ! IASI data present

! 5.  Compute the scale factor for each channel
!     -----------------------------------------

      J = 1
      DO Ichan=1,Inchan
        DO WHILE (Ichan .GT. iasi1c_h_endchan(J) .AND. J .LT. 10)
          J = J+1
        ENDDO
        Scalefactors(Ichan) = 10.0**(-iasi1c_h_scalefactor(J)+5) !mW/m^2/cm^-1/str
      ENDDO
  
      RETURN
      END
