!+ Controlling routine: read & store level 1C ATOVS data within ATOVPP
!
      SUBROUTINE PPIN ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &                  LATMS, LCRIS, LHIRAS,
     &                  Year0, Day0, Time0, iBlock,
     &                  H1Cunit, A1Cunit, B1Cunit, M1Cunit, I1Cunit,
     &                  IPCunit, atms1Cunit, cris1Cunit, hiras1Cunit,
     &                  I1Dunit, cris1Dunit, hiras1Dunit,
     &                  HLastLine, ALastLine,
     &                  BLastLine, MLastLine, ILastLine, atmsLastLine,
     &                  crisLastLine, hirasLastLine)
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppin.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Update ATOVPP buffers of HIRS/AMSU-A/-B/MSU scanlines to hold data within
!   start and end dates/times specified.
!
! Method:
! 1.  Calculate start/end time of block
! 2.  Read HIRS
! 3.  Read AMSU-A
! 4.  Read AMSU-B
! 5.  Read MSU
! 6.  Read IASI
! 7.  Read ATMS
! 8.  Read CRIS
! 9.  Read HIRAS
!   Calculate start and end dates/times for the block:
!                                         Year1/Day1/Time1, Year2/Day2/Time2.
!   For each instrument requested, call appropriate routine
!  (PPHGET/AGET/BGET/MGET) to read into store scanlines between these times.
!   These routines add an extra 2 scanlines at either end to help the mapping.
!
! Note:
!   The following should be initialised as 0 before first calling this routine:
!       HLastLine, ALastLine, BLastLine, MLastLine
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      15/08/05  Add IASI (N.C.Atkinson)
!  1.2      23/09/08  Add IASIPC (N.C.Atkinson)
!  1.3      16/08/10  Add ATMS and CrIS
!  1.4      07/02/19  Add HIRAS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!
!   Scalar arguments with intent(In):
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LIASI         ! .TRUE. to process IASI
      LOGICAL       LPCIASI       ! .TRUE. to process IASI PC
      LOGICAL       LATMS         ! .TRUE. to process ATMS
      LOGICAL       LCRIS         ! .TRUE. to process CRIS
      LOGICAL       LHIRAS        ! .TRUE. to process HIRAS
!
      ! start date/time for the datasets:
      INTEGER       Year0         !   year
      INTEGER       Day0          !   day
      INTEGER       Time0         !   time
!
      INTEGER       iBlock        ! loop index, looping through blocks
!
      INTEGER       H1Cunit       ! logical unit no. for HIRS 1C input
      INTEGER       A1Cunit       ! logical unit no. for AMSU-A 1C input
      INTEGER       B1Cunit       ! logical unit no. for AMSU-B 1C input
      INTEGER       M1Cunit       ! logical unit no. for MSU 1C input
      INTEGER       I1Cunit       ! logical unit no. for IASI 1C input
      INTEGER       atms1Cunit    ! logical unit no. for ATMS 1C input
      INTEGER       cris1Cunit    ! logical unit no. for CRIS 1C input
      INTEGER       hiras1Cunit   ! logical unit no. for HIRAS 1C input
      INTEGER       IPCunit       ! logical unit no. for IASI PC input
      INTEGER       I1Dunit       ! logical unit no. for IASI 1D output
      INTEGER       cris1Dunit    ! logical unit no. for CRIS 1D output
      INTEGER       hiras1Dunit   ! logical unit no. for HIRAS 1D output
!
      INTEGER       HLastLine     ! number of last HIRS line in store
      INTEGER       ALastLine     ! number of last AMSU-A line in store
      INTEGER       BLastLine     ! number of last AMSU-B line in store
      INTEGER       MLastLine     ! number of last MSU line in store
      INTEGER       ILastLine     ! number of last IASI line in store
      INTEGER       atmsLastLine  ! number of last ATMS line in store
      INTEGER       crisLastLine  ! number of last CrIS line in store
      INTEGER       hirasLastLine ! number of last HIRAS line in store
!
!   Scalar arguments with intent(InOut):
!
! Local scalars:
      ! Date/Time data block is to start from:
      INTEGER       dTime1        ! start time relative to Year0/Day0/Time0
      INTEGER       Year1         ! year (e.g. 1995)
      INTEGER       Day1          ! day of year
      INTEGER       Time1         ! time (milliseconds)
!
      ! Date/Time data block is to end:
      INTEGER       dTime2        ! end time relative to Year0/Day0/Time0
      INTEGER       Year2         ! year (e.g. 1995)
      INTEGER       Day2          ! day of year
      INTEGER       Time2         ! time (milliseconds)
!
      ! Just for print-out:
      INTEGER       hour1, hour2      ! start/end times in HH:MM:SS form
      INTEGER       minute1, minute2
      INTEGER       second1, second2
!
! Function & Subroutine calls:
      External PPHGET             ! read in block for HIRS
      External PPAGET             ! read in block for AMSU-A
      External PPBGET             ! read in block for AMSU-B
      External PPMGET             ! read in block for MSU
      External PPIGET             ! read in block for IASI
      External PPIPCGET           ! read in block for IASI PC
      External PPATMSGET          ! read in block for ATMS
      External PPCRISGET          ! read in block for CRIS
      External PPHIRASGET         ! read in block for HIRAS
      External TimeAdd            ! Add time interval to a date/time
!
!- End of header


! 1.  Calculate start/end time of block
!     --------- --------- ---- -- -----

!     start date/time
      dTime1 = dT*(iBlock-1)
      CALL TimeAdd ( Year0,Day0,Time0, dTime1, Year1,Day1,Time1 )

!     end date/time
      dTime2 = dTime1 + dT
      CALL TimeAdd ( Year0,Day0,Time0, dTime2, Year2,Day2,Time2 )

!     just for print-out:
      second1 = MOD( (Time1/1000), 60 )
      minute1 = MOD( (Time1/1000), 3600 )  / 60
      hour1 = Time1 / 3600000
      second2 = MOD( (Time2/1000), 60 )
      minute2 = MOD( (Time2/1000), 3600 )  / 60
      hour2 = Time2 / 3600000
      WRITE(*,"(10X,""Start/end times"",
     &      2(5X,I2.2,"":"",I2.2,""."",I2.2))")
     &      hour1, minute1, second1, hour2, minute2, second2


! 2.  Read HIRS
!     ---- ----

      IF (LHIRS) THEN

!        read data into buffer
         CALL PPHGET ( H1Cunit, HLastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LHIRS)


! 3.  Read AMSU-A
!     ---- ------

      IF (LAMSA) THEN

!        read data into buffer
         CALL PPAGET ( A1Cunit, ALastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LAMSA)


! 4.  Read AMSU-B
!     ---- ------

      IF (LAMSB) THEN

!        read data into buffer
         CALL PPBGET ( B1Cunit, BLastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LAMSB)


! 5.  Read MSU
!     ---- ---

      IF (LMSU) THEN

!        read data into buffer
         CALL PPMGET ( M1Cunit, MLastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LMSU)


! 6.  Read IASI
!     ---- ---=
!Note that PPIGET creates the IASI level 1d file and fills it with IASI data -
!to avoid storing in memory. We don't map IASI to any other grid.

      IF (LIASI) THEN

!        read data into buffer
         CALL PPIGET ( I1Cunit, I1Dunit, ILastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LIASI)

      IF (LPCIASI) THEN

!        read data into buffer
         CALL PPIPCGET ( IPCunit, I1Dunit, ILastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LPCIASI)


! 7.  Read ATMS
!     ---- ------

      IF (LATMS) THEN

!        read data into buffer
         CALL PPATMSGET ( atms1Cunit, atmsLastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LATMS)


! 8.  Read CRIS
!     ---- ------
!Note that PPCRISGET creates the CRIS level 1d file and fills it with CRIS data -
!to avoid storing in memory. We don't map CRIS to any other grid.

      IF (LCRIS) THEN

!        read data into buffer
         CALL PPCRISGET ( cris1Cunit, cris1Dunit, crisLastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LCRIS)


! 8.  Read HIRAS
!     ---- ------
!Note that PPHIRASGET creates the HIRAS level 1d file and fills it with 
!HIRAS data - to avoid storing in memory. We don't map HIRAS to any
!other grid.

      IF (LHIRAS) THEN

!        read data into buffer
         CALL PPHIRASGET ( hiras1Cunit, hiras1Dunit, hirasLastLine,
     &                 Year1, Day1, Time1,
     &                 Year2, Day2, Time2 )

      ENDIF   ! (LHIRAS)


      RETURN
      END
