!+ Transfer data from program store to level 1D IASI data record (ATOVS)
!
      SUBROUTINE PPIOUTD ( Line, Irec )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppioutd.F 70 2007-03-26 13:52:44Z frna $
!
! Purpose:
!   ATOVPP stores data from several scanlines in its own internal array
!   structure, contained in common/ppistore/. This routine transfers a single
!   scanline from slot "Line" in common/ppistore/ to common/iasi1ddt/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1D include file (iasi1d.h) and IASI store
!     (ppistore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
!     Fill parts of 1D data record that won't change each scanline
! 1.  Copy variables from slot "Line" in common/ppistore/ to common/iasi1ddt/
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     12/08/05  Created from pphoutd.F (N.C.Atkinson)
!  1.1     20/03/07  Copy info from ICloudflags. 1d version up to 2.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of IASI data 
#include "ppistore.h"
!  IASI level 1D header/data common blocks 
#include "iasi1d.h"
!  IASI fixed data and user options 
#include "ppifdata.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppistore/
      INTEGER       Irec          ! IASI 1d record (1 to 4)
!
! Local parameters:
      INTEGER       I1Dvnnb       ! version number of 1D format compatible
         PARAMETER (I1Dvnnb=2)    ! with the code in this subroutine
      INTEGER       Istvnnb       ! version number of "ppistore.h" compatible
         PARAMETER (Istvnnb=1)    ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       i             ! general purpose loop variable
      INTEGER       fov1d         ! loop variable over all 1d fovs
      INTEGER       fov           ! all IASI fovs (max 120)
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first                    ! .TRUE. on first call
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

! 0.1    Is the 1D include file compatible with this source code ?
         IF ( I1Dvnnb .NE. iasi_1dvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1D format"",I6,"" but iasi1d.h is "",I6)")
     &           I1Dvnnb, iasi_1dvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPIOUTD")
         ENDIF

! 0.2    Is the ppistore include file compatible with this source code ?
         IF ( Istvnnb .NE. iasi_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects Istore format"",I6,"" but ppistore.h is"",I6)")
     &           Istvnnb, iasi_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPIOUTD")
         ENDIF

! 0.3    Fill parts of 1D data record that won't change each scanline

! 0.4    Reset "first" for subsequent calls
         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from slot "Line" in common/ppistore/ to common/iasi1ddt/
!     ---- --------- ---- ----  ----  -- ---------------- -- ---------------

!     scnlin, scnlinyr, scnlindy, scnlintime, scalti already filled in ppi1d.F

      iasi1d_instrtempa1 = Iinstrtempa1(Line)      ! AMSU-A1 temperature
      iasi1d_instrtempa2 = Iinstrtempa2(Line)      ! AMSU-A2 temperature
      iasi1d_instrtempb = Iinstrtempb(Line)        ! AMSU-B temperature
      DO i = 1, 2
         iasi1d_spare1(i) = MDI                    ! spare
         iasi1d_spare2(i) = MDI                    ! spare
      ENDDO  ! i

      DO fov1d = 1, iasi_1dnfov
         
         IF (nrec_IASI.EQ.4) THEN
           fov = (irec-1)*30 + fov1d                   !Full resolution
         ELSE
           fov = (IbestFOV(fov1d,Line)-1)*30 + fov1d   !Best of 4
         ENDIF
         
!     Lat, lon, angles, etc. have already been filled in ppi1d.F

!     Surface height and type
         iasi1d_surface(1,fov1d) = Ielev(fov,Line)
         iasi1d_surface(2,fov1d) = Isurftype(fov,Line)

!     Mapped AMSU BTs (deg K * 100)
         DO chn = 1, Anchan+Bnchan
            iasi1d_btemps(chn,fov1d) = NINT(Imapbt(chn,fov,Line)*100.)
         ENDDO  ! chn

         iasi1d_iasiqual(fov1d) = Iqc(fov,Line)   ! QC word for IASI

         iasi1d_prepro(1,fov1d) = Imapqc(fov,Line)    ! ATOVPP QC word for AMSU
         iasi1d_prepro(2,fov1d) = Imapsurf(fov,Line)  ! estimated surface type
         iasi1d_prepro(3,fov1d) = NINT(Isurfcost(fov,Line)*100.) ! max prob cloud
         iasi1d_prepro(4,fov1d) = NINT(Iscatindx(fov,Line)*100.) ! scattering index
         iasi1d_prepro(5,fov1d) = NINT(Icrosby(fov,Line)*100.)   ! logistic precip
         DO i = 6, 7
            iasi1d_prepro(i,fov1d) = MDI              ! spare
         ENDDO  ! i
         DO i = 1, 13
            iasi1d_emiss(i,fov1d) = MDI               ! reserved for emissivity
         ENDDO  ! i

!     Cloud flags
         DO i = 1, 10
            iasi1d_cloudflag(i,fov1d) = ICloudflag(i,fov1d,Line)
         ENDDO  ! i

      ENDDO  ! fov1d


         

      RETURN
      END
