!+ Set up IASI level 1D header within ATOVPP
!
      SUBROUTINE PPIOUTH ( LAMSA, LAMSB )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppiouth.F 70 2007-03-26 13:52:44Z frna $
!
! Purpose:
!   Set up level 1D header using information from 1C headers.
!
! Method:
! 1.  Check format version number and data type
!     IASI level 1C header has already been read into common /iasi1chd/
!     Check that format of 1C & 1D include files (iasi1c.h, iasi1d.h)
!     are compatible with the code in this subroutine.
!     Check that format of 1C file is compatible with this subroutine.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 2.  Set up level 1D header
!
! Some parts of the 1D format are intended to be flexible, allowing
! different centres to customise the format for their own requirements.
! These parts are marked as '[user-defined ...]' below.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     12/08/05  Created from pphouth.F (N.C.Atkinson)
!  1.1     20/03/07  Increment I1Dvnnb to 2. Don't clear iasi1d_h_cloudtests.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  IASI level 1D header/data common blocks 
#include "iasi1d.h"
!  fixed data for processing IASI 
#include "ppifdata.h"
!  fixed data for processing AMSU-A 
#include "ppafdata.h"
!  fixed data for processing AMSU-B 
#include "ppbfdata.h"
!  block of IASI mapped data 
#include "ppistore.h"
!  HIRS level 1C header/data common blocks 
#include "iasi1c.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!
! Local parameters:
      INTEGER       I1Cvnnb       ! version number HIRS 1C format compatible
         PARAMETER (I1Cvnnb=1  )  !    with the code in this subroutine
      INTEGER       A1Cvnnb       ! version number AMSU-A 1C format compatible
         PARAMETER (A1Cvnnb=190)  !    with the code in this subroutine
      INTEGER       B1Cvnnb       ! version number AMSU-B 1C format compatible
         PARAMETER (B1Cvnnb=190)  !    with the code in this subroutine
      INTEGER       I1Dvnnb       ! version number 1D format compatible
         PARAMETER (I1Dvnnb=2  )  !    with the code in this subroutine
!
!   Scalar arguments with intent(In):
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i             ! general purpose loop variables
      INTEGER       chn           ! loop variable over channels
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
!
!- End of header


! 1.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 1.1 Are the 1C include files compatible with this source code ?
!     --- --- -- ------- ----- ---------- ---- ---- ------ ----

!     HIRS
      IF ( I1Cvnnb .NE. iasi_1cvnnb ) THEN
         errStatus = -111  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but iasi1c.h is "",I6)")
     &        I1Cvnnb, iasi_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPIOUTH")
      ENDIF

!     AMSU-A
      IF ( LAMSA .AND. A1Cvnnb .NE. ama_1cvnnb ) THEN
         errStatus = -112  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but ama1c.h is "",I6)")
     &        A1Cvnnb, ama_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPIOUTH")
      ENDIF

!     AMSU-B
      IF ( LAMSB .AND. B1Cvnnb .NE. amb_1cvnnb ) THEN
         errStatus = -113  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but amb1c.h is "",I6)")
     &        B1Cvnnb, amb_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPIOUTH")
      ENDIF

! 1.2 Is the 1D include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( I1Dvnnb .NE. iasi_1dvnnb ) THEN
         errStatus = -12  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1D format "",I6,"" but iasi1d.h is "",I6)")
     &        I1Dvnnb, iasi_1dvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPIOUTH")
      ENDIF


! 2.  Set up level 1D header
!     --- -- ----- -- ------

! 2.1 General

!     creation site for 1D data
      iasi1d_h_site   = SiteID              !file creation site ID (CHAR*3)
      iasi1d_h_cfill1 = " "                 !filler

!     creation site for original 1B data
      iasi1d_h_1bsite = iasi1c_h_1bsite
      iasi1d_h_cfill2 = " "                 !filler

      iasi1d_h_versnb = iasi_1dvnnb          !level 1d format version number
      iasi1d_h_versyr = iasi_1dvnyr          !level 1d format version year
      iasi1d_h_versdy = iasi_1dvndy          !level 1d format version day of year
      iasi1d_h_hdrcnt = iasi1c_h_hdrcnt      !count of header records
      iasi1d_h_satid = iasi1c_h_satid        !satellite id (e.g. 14 for NOAA-14)
      iasi1d_h_wmosatid = iasi1c_h_wmosatid  !WMO satellite id (3 for METOP-1)
      iasi1d_h_grid = iasi1c_h_instrument    !instrument grid code (IASI=13)
      iasi1d_h_satht = iasi1c_h_satht        !nominal sat height, km*10
      iasi1d_h_period = iasi1c_h_period      !nominal orbit period (seconds)
      iasi1d_h_startorbit = iasi1c_h_startorbit !orbit number (at start)
      iasi1d_h_startdatayr = iasi1c_h_startdatayr!start of data set year
      iasi1d_h_startdatady = iasi1c_h_startdatady! (ditto) day of the year
      iasi1d_h_startdatatime = iasi1c_h_startdatatime !(ditto) UTC time of day,ms
      iasi1d_h_endorbit = iasi1c_h_endorbit  !orbit number (at end)
      iasi1d_h_enddatayr = iasi1c_h_enddatayr!end of data set year
      iasi1d_h_enddatady = iasi1c_h_enddatady!(ditto) day of the year
      iasi1d_h_enddatatime = iasi1c_h_enddatatime !(ditto) UTC time of day,ms
      iasi1d_h_scnlin = iasi1c_h_scnlin      !count of scan lines in this dataset
      iasi1d_h_misscnlin = iasi1c_h_misscnlin !count of missing scan lines
      iasi1d_h_atovppvn = ATOVPPvn     !ATOVPP version number (test vns = 9000+)
      iasi1d_h_instruments = Iinstruments   !instruments present
      iasi1d_h_spare1 = MDI                !spare

! 2.2 temperature-radiance conversion coefficients

!     AMSU-A
      IF ( LAMSA ) THEN
         DO chn = 1, Anchan
         DO i = 1, 3
            iasi1d_h_amsPlanck(i,chn) = ama1c_h_tempradcnv(i,chn)
         ENDDO  ! i
         ENDDO  ! chn
      ELSE
         DO chn = 1, Anchan
         DO i = 1, 3
            iasi1d_h_amsPlanck(i,chn) = MDI
         ENDDO  ! i
         ENDDO  ! chn
      ENDIF

!     AMSU-B
      IF ( LAMSB ) THEN
         DO chn = 1, Bnchan
         DO i = 1, 3
            iasi1d_h_amsPlanck(i,chn+Anchan) = amb1c_h_tempradcnv(i,chn)
         ENDDO  ! i
         ENDDO  ! chn
      ELSE
         DO chn = 1, Bnchan
         DO i = 1, 3
            iasi1d_h_amsPlanck(i,chn+Anchan) = MDI
         ENDDO  ! i
         ENDDO  ! chn
      ENDIF

! 2.3 Pre-processing information

![user-defined:
      iasi1d_h_dT  = dT         ! size of data batches processed within ATOVPP
      iasi1d_h_mapBTmode = A2Imode  ! mapping mode (AMSU-A to IASI only)
      iasi1d_h_Acorrvn  = Acorrvn        ! vn no AMSU-A limb/emiss correction
      iasi1d_h_Apcpvn   = Apcpvn         ! vn no AMSU-A precip/scat detection
      iasi1d_h_Aantenna = ama1c_h_vnantennacorr ! vn AMSU-A antenna corrections
      iasi1d_h_Bcorrvn  = Bcorrvn        ! vn no AMSU-B limb/emiss correction
      iasi1d_h_Bpcpvn   = Bpcpvn         ! vn no AMSU-B precip/scat detection
      iasi1d_h_Bantenna = amb1c_h_vnantennacorr ! vn AMSU-B antenna corrections
      iasi1d_h_Ascat = Amaxscat !threshold value for AMSU-A scattering index
      iasi1d_h_ACrosby = Amaxcrosby !threshold value for AMSU-A logistic precip
      iasi1d_h_AB89 = ABmax89 !threshold value for AMSU-A/B 89GHz differences
      iasi1d_h_recperscan = nrec_IASI  !1d records per scan (1 or 4)
      iasi1d_h_fovmode = thinmode_IASI
      DO i = 1, array_size_iasi1dspare
         iasi1d_h_spare2(i) = MDI        !spare
      ENDDO  ! i
! ... ]

! 2.4 IASI variables

      DO i=1,10
        iasi1d_h_startchan(i) = iasi1c_h_startchan(i)
        iasi1d_h_endchan(i) = iasi1c_h_endchan(i)
        iasi1d_h_scalefactor(i) = iasi1c_h_scalefactor(i)
      ENDDO

!      iasi1d_h_eigvn is already filled (see ppifdf.F)
      iasi1d_h_npc = iasi_1dnpc        !PCs in format
      iasi1d_h_npcused = npc_IASI      !PCs used
      iasi1d_h_nchan = iasi_1dnchan    !Channels in format
      iasi1d_h_nchanused = nchan_IASI  !Channels used

      DO i=1,iasi_1dnchan
        iasi1d_h_channels(i) = channels_IASI(i)
      ENDDO

      RETURN
      END
