!+ Read & store level PC IASI data within ATOVPP
!
      SUBROUTINE PPIPCGET ( IPCunit, I1Dunit, ILastLine,
     &                    Year1, Day1, Time1,
     &                    Year2, Day2, Time2 )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppipcget.F 182 2010-01-12 15:50:02Z frna $
!
! Purpose:
!   Update ATOVPP store of IASI scanlines to hold data within
!   start and end dates/times specified (Year1/Day1/Time1 & Year2/Day2/Time2)
!   with an extra 1 or 2 scanlines at either end to help with the mapping.
!
! Method:
! 1.  Look for first suitable scanline in previous block
! 2.  Define record number to start reading from
! 3.  Set data store to all "missing"
! 4.  Read data till we come to end of time interval, or of dataset
!
!   The position of each scanline in the program store ("ppistore.h") is
!   determined solely by the date/time of the scanline number, relative to
!   that of the first line in store.  Thus the routine can cope with missing
!   scanlines, but can potentially be upset if the first line in store has a
!   corrupt date/time.
!
! Note:
!   "ILastLine" should be initialised as 0 before first calling this routine.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     22/09/08  N.C.Atkinson, based on ppiget.F.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  IASI 1d format 
#include "iasi1d.h"
!  block of IASI data 
#include "ppistore.h"
!  IASI fixed data 
#include "ppifdata.h"
!  IASI level PC header/data common blocks 
#include "iasipc.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       IPCunit       ! logical unit no. for IASI 1C input
      INTEGER       I1Dunit       ! logical unit no. for IASI 1D output
      INTEGER       ILastLine     ! number of last line stored in store
      ! Date/Time data block is to start from:
      INTEGER       Year1         !     year (e.g. 1995)
      INTEGER       Day1          !     day of year
      INTEGER       Time1         !     time (milliseconds)
      ! Date/Time data block is to end:
      INTEGER       Year2         !     year (e.g. 1995)
      INTEGER       Day2          !     day of year
      INTEGER       Time2         !     time (milliseconds)
!
! Local scalars:
      CHARACTER*80  errMessage    ! for calls to ErrorReport
      INTEGER       errStatus     ! error status: 0 OK, +ve for fatal error
      INTEGER       border        ! for overlaps of data blocks (millisecs)
      INTEGER       dTime         ! time difference (milliseconds) from TimeSub
      INTEGER       errTime       ! error status from TimeSub
      INTEGER       IPCrecord     ! record number for IASI level PC file
      INTEGER       fov           ! loop variable over all fov's
      INTEGER       line          ! line number within data store
      INTEGER       no_data       ! bit word set to "missing/bad data"
      INTEGER       scnlin0       ! scanline number of "zeroth" line in store
      INTEGER       scnlinDT      ! scanline number derived from date/time
      INTEGER       scnlin        ! scanline number
      INTEGER       year          ! year number of scanline
      INTEGER       day           ! day number of scanline
      INTEGER       time          ! time of scanline (ms)
      INTEGER       scnlinOld     ! previous scanline number
      INTEGER       yearOld       ! year number of previous scanline
      INTEGER       dayOld        ! day number of previous scanline
      INTEGER       timeOld       ! time of previous scanline (ms)
      INTEGER       Irec          ! loop index for records
      INTEGER       I1Drecord     ! IASI 1d record (1 to 4)

! Function & Subroutine calls:
      External TimeSub              ! Subtract two dates/times
      External IOIPC                ! read IASI PC scan line
      External IOI1D                ! write IASI 1D record
      External PPIPCIND             ! store scanline data in common/ppistore/
      External ErrorReport          ! ATOVPP error routine
!
!- End of header


! 1.  Look for first suitable scanline in previous block
!     ---- --- ----- -------- -------- -- -------- -----

      line = ILastLine + 1
      dTime   = -999
      border = 0             ! block for final targ instr does not need border

!     (stop looking once scanlines in block are earlier than start time)
      DO WHILE (line.GT.1 .AND. dTime.LT.border)

         line = line - 1
         errTime = 1
         IF ( Irecdno(line).NE.MDI ) THEN    ! valid line
            CALL TimeSub ( Year1,       Day1,       Time1,
     &                     Iyear(line), Iday(line), Itime(line),
     &                     dTime, errTime)! dTime = start time - time of scan
!         errTime returned as non-zero if input is out of range (e.g. missing)
         ENDIF
         IF (errTime.NE.0) dTime = -999   ! go on looking

      ENDDO


! 2.  Define record number to start reading from
!     ------ ------ ------ -- ----- ------- ----

      IF ( dTime.GE.0 ) THEN
!        we've already read suitable data  - get record number from that
         IPCrecord = Irecdno(line)
      ELSE
!        no suitable scanlines already in block  - try next record
         IF ( ILastLine.GT.0 ) THEN
            IPCrecord = Irecdno(ILastLine) + 1
         ELSE  ! this is the first data to be read
            IPCrecord = Ihdrcnt + 1  !first data record in 1C file
         ENDIF
      ENDIF


! 3.  Set data store to all "missing"
!     --- ---- ----- -- ---  -------

!     Throughout this program, Irecdno() should be used to check
!     whether a scanline is present or not, and Iqc(bit 0)
!     to check whether the data is present and usable.

      no_data = 0
      no_data = IBSET ( no_data, bits(0) )

      DO line = 1, Inlines
         Irecdno(line) = MDI
         DO fov = 1, Infovs
            Iqc(fov,line) = no_data
         ENDDO  ! fov
      ENDDO  ! line


! 4.  Read data till we come to end of time interval, or of dataset
!     ---- ---- ---- -- ---- -- --- -- ---- --------  -- -- -------

      ILastLine = 0  ! i.e. we have no data in storage
      errTime = 1
      dTime   = -999
      scnlin0 = MDI  ! scanline number of "zeroth" scanline in store

! 4.1 Loop through scanlines in dataset
      DO WHILE ( IPCrecord.LE.(Ihdrcnt+Iscnlin) .AND. dTime.LT.border )

!        Read a scan line
        CALL IOIPC ("READ DATA", IPCunit, IPCrecord)
        scnlin = iasipc_scnlin
        year = iasipc_scnlinyr
        day  = iasipc_scnlindy
        time = iasipc_scnlintime(1)

        IF ( iasipc_scnlin.GE.1 .AND. iasipc_scnlin.LE.9999 ) THEN

! 4.2    Calculate the scanline number from the date/time (to check)
         IF ( scnlin0 .NE. MDI ) THEN   ! not the 1st record read this call

            CALL TimeSub ( year, day, time,
     &                     yearOld,dayOld, timeOld, dTime, errTime )
            errMessage = " "
            CALL ErrorReport(errTime,errMessage,"PPIGET")

!           estimated scanline number
            scnlinDT = scnlinOld + NINT(REAL(dTime)/Icycle)

!           Complain if scanline number and estimated scanline number differ
            IF ( scnlinDT.NE.scnlin ) THEN
               errMessage = " "
               WRITE ( errMessage,
     &      "(""Scanline numbers and times inconsistent, lines"",3I9)")
     &          scnlinOld, scnlin, scnlinDT
               errStatus = -42  ! warning
               CALL ErrorReport(errStatus,errMessage,"PPIGET")
            ENDIF

         ENDIF  ! not the 1st record read

! 4.3    Store scanline in slot "line" in common/ppistore/

!        Calculate slot to put line into
         IF ( scnlin0 .EQ. MDI ) THEN   ! first record to be read this call
            scnlin0 = scnlin - 1
            line = 1
         ELSE
            line = scnlinDT - scnlin0   ! use estimated line number, not scnlin
         ENDIF

         IF ( line.LE.Inlines ) THEN  ! there's room
            ILastLine = line
            CALL PPIPCIND ( line )
            Irecdno(line) = IPCrecord
         ENDIF

! 4.4    save line-number/date/time to check against next scanline
!         scnlinOld = scnlin
         IF ( line .EQ. 1 ) THEN
           scnlinOld = scnlin
         ELSE
           scnlinOld = scnlinDT
         ENDIF
         yearOld = year
         dayOld  = day
         timeOld = time

! 4.5    Calculate dTime := date/time of scan - end date/time of block
         CALL TimeSub ( year, day, time,
     &                  Year2,Day2,Time2,  dTime, errTime )
!        errTime returned as non-zero if input is out of range (e.g. missing)
         IF (errTime.NE.0) dTime = -999   ! go on looking


! 4.6    Process the IASI scan line. Data are in common block in iasipc.h.
!        For full resolution data loop round the 4 IASI detectors, otherwise
!        select the 'best' or the specified detector for each scan position.

         IF ( line.LE.Inlines ) THEN  ! If there's room in storage arrays
         
           CALL PPIPCThin(line)        !Determine the 'best' detector

           DO Irec = 1, nrec_IASI
             CALL PPIPC1D(Irec,line)   !Copy data to 1d common block

! 4.7    Write out IASI data as level 1d
!        Each 1c record has "nrec_IASI" 1d records (ppifdata.h)
!        Both also have a header record.

             IF (nrec_IASI .EQ. 4) THEN
               I1Drecord = nrec_IASI*(Irecdno(line)-2) + 1 + Irec
             ELSE
               I1Drecord = Irecdno(line)
             ENDIF
             CALL IOI1D ("WRITE DATA", I1Dunit, I1Drecord)

           ENDDO    ! Irec
         ENDIF    ! If there's room in storage arrays
        ENDIF    ! if scanline OK

!       Increment record number for next read
        IPCrecord = IPCrecord + 1

      ENDDO   ! WHILE ( IPCrecord.LE.(Ihdrcnt+Iscnlin) .AND. dTime.LT.border )

      WRITE(*,"(10X,""IASI line numbers "",I4,"" to "",I4,
     &        "", with "",I3,"" missing lines"")")
     &        Ilineno(1), Ilineno(ILastLine),
!     &       (scnlinOld-Ilineno(1))-(ILastLine-1)
     &       (Ilineno(ILastLine)-Ilineno(1))-(ILastLine-1)
      WRITE(*,"(10X,""Number of IASI Principal Components:"",I4)") 
     &        npc_IASI
      WRITE(*,"(10X,""Number of IASI Channels to output:"",I4)") 
     &        nchan_IASI
      WRITE(*,"(10X,""IASI thinning mode:"",I4)") thinmode_IASI


      RETURN
      END
