!+ Read & interpret IASI level 1C PC header within ATOVPP
!
      SUBROUTINE PPIPCINH ( IPCunit, SatId, WMOSatId,
     &                    StartYearI, StartDayI, StartTimeI,
     &                    EndYearI,   EndDayI,   EndTimeI )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppipcinh.F 323 2012-08-09 13:09:46Z frna $
!
! Purpose:
!   Open input dataset and read IASI PC header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     IASI PC header has already been read into common /iasipchd/
!     Check that format of include files (iasipc.h) is compatible with the
!     code in this subroutine. Check that format of PC file is compatible with
!     this subroutine, and that data type from the header is 14 for IASI PC.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from IASI PC header
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      22/09/08  Based on ppiinh.F. N.C.Atkinson
!  1.1      09/08/12  IPCvnnb=2
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of IASI data 
#include "ppistore.h"
!  IASI level PC header/data common blocks 
#include "iasipc.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
! Local parameters:
      INTEGER       IPCvnnb       ! version number of PC format compatible
         PARAMETER (IPCvnnb=2)  ! with the code in this subroutine
!
!   Scalar arguments with intent(In):
      INTEGER       IPCunit       ! logical unit no. for IASI PC input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 3 for METOP-1)
      ! start/end date/time of IASI data:
      INTEGER       StartYearI, EndYearI
      INTEGER       StartDayI,  EndDayI
      INTEGER       StartTimeI, EndTimeI
      INTEGER       Ichan, J
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       IPCrecord     ! record number for IASI level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOIPC                ! I/O routine for level 1C IASI data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.1 Open 1C file
      CALL IOIPC ("OPEN/READ", IPCunit, IPCrecord )   ! 1C file

! 1.2 Read 1C header
      IPCrecord = 1   ! Headers start on record 1.
      CALL IOIPC ("READ HEADER", IPCunit, IPCrecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( IPCvnnb .NE. iasi_pcvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects PC format "",I6,"" but iasipc.h is "",I6)")
     &        IPCvnnb, iasi_pcvnnb
         CALL ErrorReport ( errStatus, errMessage, "ppipcINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "iasipc.h" contains its own version number "iasi_h_lpcversnb"
      IF ( iasipc_h_versnb .NE. iasi_pcvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level PC IASI file is "",I8,"", not "",I8)")
     &        iasipc_h_versnb,iasi_pcvnnb
         CALL ErrorReport ( errStatus, errMessage, "ppipcINH")
      ENDIF

! 2.3 Is the 1C data IASI ?
!     -- --- -- ---- ------
!     "iasipc_h_instrument" from PC header should be 14 for IASI
      IF ( iasipc_h_instrument .NE. 14 ) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage, "("" Data type in PC IASI file is "",
     &           I8,""; not 14 for IASIPC"")") iasipc_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "ppipcinh")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "##########"
      WRITE(*,*) "# IASIPC #"
      WRITE(*,*) "##########"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            iasipc_h_versnb, iasipc_h_versyr, iasipc_h_versdy
      WRITE(*,"(I4,"" header records in file."")") iasipc_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") iasipc_h_satid
      WRITE(*,"(""   WMO Satellite id is "",I4)") iasipc_h_wmosatid

!     calculate start and end dates/times from header
!     ..start
      year = iasipc_h_startdatayr
      yrday = iasipc_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(iasipc_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = iasipc_h_enddatayr
      yrday = iasipc_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(iasipc_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          iasipc_h_scnlin, iasipc_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from IASI 1C header
!     ------- -------- ----------- ---- ------ -- ------

      ISatId = iasipc_h_satid
      SatId = ISatId
      WMOSatId = iasipc_h_wmosatid
      StartDayI = iasipc_h_startdatady
      StartYearI = iasipc_h_startdatayr
      StartTimeI = iasipc_h_startdatatime
      EndDayI = iasipc_h_enddatady
      EndYearI = iasipc_h_enddatayr
      EndTimeI = iasipc_h_enddatatime
      Isatht = REAL(iasipc_h_satht)*0.1      !nominal sat height (km)
      Isatperiod = REAL(iasipc_h_period)/60. !nominal orbit period (minutes)
      Ihdrcnt = iasipc_h_hdrcnt       !count of header records in 1C file
      Iscnlin = iasipc_h_scnlin       !count of scanlines in 1C file
      Iinstruments = IBSET ( Iinstruments , bits(6) )   ! IASI data present

! 5.  Compute the scale factor for each channel
!     -----------------------------------------

      J = 1
      DO Ichan=1,Inchan
        DO WHILE (Ichan .GT. iasipc_h_endchan(J) .AND. J .LT. 10)
          J = J+1
        ENDDO
        Scalefactors(Ichan) = 10.0**(-iasipc_h_scalefactor(J)+5) !mW/m^2/cm^-1/str
      ENDDO

      RETURN
      END
