!+ IASI spectral analysis
!
      SUBROUTINE PPISPECTRA ( Det, Ifov )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppispectra.F 182 2010-01-12 15:50:02Z frna $
!
! Purpose: Compute PC scores for IASI and store in level 1d
!
! Method:
! 1. Convert from scaled radiance to true radiance
! 2. De-apodise
! 3. Compute PC scores using reference eigenvectors
! 4. Compute reconstructed radiances
! 5. Compute reconstruction score for each detector, noise normalised.
!    Ref: Goldberg et. al., IEEE Trans.Geosci.Rem.Sens., vol 41 no 2, Feb 2003
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0    07/11/05  Original code. (N.C.Atkinson)
!  1.1    22/01/08  Make use of evlevel, eigformat and mean_encode_IASI
!  1.2    04/03/09  Improve efficiency when eigenvectors are band separated
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  level 1c format 
#include "iasi1c.h"
!  level 1d format 
#include "iasi1d.h"
!  fixed data for processing IASI 
#include "ppifdata.h"
!
!   Scalar arguments with intent(In):

      INTEGER Det        !Detector (1 to 4)
      INTEGER Ifov       !FOV (1 to 30)

!   Local scalars:

      REAL Radiance(Inchan)      !Radiance (mW/m^2/str/cm^-1)
      REAL NormRadiance(Inchan)  !Noise normalised radiance
      REAL scaling               !Scaling factor
      REAL Recon                 !Reconstructed radiance (temporary)
      REAL Reconstructed(Inchan) !Reconstructed radiances
      REAL Score                 !PC score (temporary)
      REAL Scores(iasi_1dnpc)    !PC scores
      INTEGER I                  !Loop index
      INTEGER chn                !Channel index
      INTEGER Ipc                !PC index
      INTEGER iband
      INTEGER counts
      LOGICAL Done
      LOGICAL badcounts
      LOGICAL allzero
!
!- End of header      

! 1. Convert from scaled radiance to true radiance
!    ---------------------------------------------

! Original 1c radiances are in W/(m^2 str m^-1). Multiply by a factor 100000 to
! convert to mW/(m^2 str cm^-1).
! The BUFR format supports negative radiances of up to -5000 counts. Assume that
! counts more negative than this are in fact positive.

      Done = .FALSE.
      badcounts = .FALSE.
      allzero = .TRUE.
      DO I=1,10
        IF (.NOT. Done) THEN
          scaling = 10.0**(-iasi1c_h_scalefactor(I)*1.0)*100000.0
          DO chn=iasi1c_h_startchan(I),iasi1c_h_endchan(I)
            counts = iasi1c_scalrad(chn,Det,Ifov)
            IF (counts.EQ.-32768) THEN
              badcounts=.true.
            ELSE
              IF (counts.LT.-5000) counts = counts + 65536
              Radiance(chn) = counts*scaling
              IF (counts.NE.0) allzero = .FALSE. 
            ENDIF
          ENDDO
          IF (chn.GE.Inchan) Done = .TRUE.
        ENDIF
      ENDDO
      
! 2. De-apodise spectrum
!    -------------------

      IF (.NOT.badcounts.AND..NOT.allzero) THEN

        IF (evlevel.EQ.'1B') THEN
          CALL deapodise_iasi(Radiance, mtf_IASI, 1)
        ENDIF

! 3a. Noise normalise
!    ---------------

        DO chn=1,Inchan
          NormRadiance(chn) = Radiance(chn)/nedr_IASI(chn)
        ENDDO

! 3b. Subtract mean (if eigformat=1)

        IF (eigformat.EQ.1) THEN
          DO chn=1,Inchan
            NormRadiance(chn) = NormRadiance(chn)-mean_encode_IASI(chn)
          ENDDO
        ENDIF

! 4. Compute PC scores
!    -----------------      

        DO iband=1,npcbands
          DO Ipc=startpc(iband),endpc(iband)
            Score = 0.0
            DO chn=startpcchan(iband),endpcchan(iband)
              Score = Score + eig_encode_IASI(chn,Ipc)*NormRadiance(chn)
            ENDDO
            iasi1d_scores(Ipc,Ifov) = NINT(Score*2)  !Quantise at half NeDR
            Scores(Ipc) = iasi1d_scores(Ipc,Ifov)/2.0
          ENDDO
        ENDDO

! 5a. Compute reconstructed radiance (self apodised, noise normalised)
!    ------------------------------

        DO chn=1,Inchan
          Recon = 0.0
          DO Ipc=1,npc_IASI
            Recon = Recon + 
     &         eig_encode_IASI(chn,Ipc)*Scores(Ipc)
          ENDDO
          Reconstructed(chn) = Recon
        ENDDO

! 6. Compute reconstruction score for each detector
!    ----------------------------------------------

        DO I=1,3
          Score = 0.0
          DO chn=Firstchan(I),Lastchan(I)
            Score = Score + 
     &       (Reconstructed(chn)-NormRadiance(chn))**2
          ENDDO
          Score = sqrt(Score/(Lastchan(I)-Firstchan(I)+1))
          iasi1d_logfit(I,Ifov) = NINT(LOG10(Score)*1E4)  !10^4 times log score
                                                     !See BUFR descriptor 025052
        ENDDO

      ELSE        !if there are some missing scaled radiances

        DO Ipc=1,npc_IASI
          iasi1d_scores(Ipc,Ifov) = MDI
        ENDDO
        DO I=1,3
          iasi1d_logfit(I,Ifov) = MDI
        ENDDO

      ENDIF

      RETURN
      END
