!+ IASI thinning
!
      SUBROUTINE PPITHIN ( line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppithin.F 128 2008-10-07 09:33:03Z frna $
!
! Purpose:
!   For each spot, choose 1 detector out of 4 for inclusion in the 1d output
!
! Method:
!   The following options are provided, via IASI.fdf:
!   0) No thinning - include all 4 detectors
!   1) Fixed IASI detector (1 of 4)
!   2) Highest mean AVHRR radiance for defined AVHRR channel
!   3) Minimum variability in AVHRR radiances for defined channel
!   4) Hybrid - scheme 3 in polar regions, scheme 2 elsewhere. Latitude
!      transition is defined in IASI.fdf.
!   5) Highest mean IASI radiance for defined IASI channel
!   6) As for 5, but fixed detector in polar regions
!   7) Fixed sequence of all 4 detectors (1 per scan line)
!
!   If AVHRR is missing for all 4 spots, the system reverts to option 1, 5 or
!   6, depending on a switch in IASI.fdf.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     28/01/05  N.C.Atkinson
!  1.1     17/01/06  Refine options. NCA
!  1.2     12/07/06  Bug fix in mean latitude calculation. NCA
!  1.3     20/03/07  Make use of OKdetector. NCA
!  1.4     25/09/08  Make use of GQIsFlagQual
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  IASI 1d format 
#include "iasi1d.h"
!  block of IASI data 
#include "ppistore.h"
!  fixed data 
#include "ppifdata.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!
!   Scalar arguments with intent(In):

      INTEGER line       !Scan line in ppistore

!   Local scalars:

      INTEGER   Ispot, Iclass, Ichan, Idet, Ifov      !Loop indices
      INTEGER   bestdet          !Best detector
      INTEGER   rad              !Scaled AVHRR radiance
      INTEGER   sdev             !Scaled std dev of AVHRR radiance
      INTEGER   scale            !Scaling factor for AVHRR mean or sdev
      INTEGER   minclass, nclass !Number of classes
      INTEGER   thinmode         !Copy of thinmode_IASI
      INTEGER   imaxrad          !Integer maximum radiance
      INTEGER   FOVorder(4)      !For use with option 7
      DATA FOVorder /1,3,2,4/
      REAL      radiance, radsd, maxrad, minrad       !Radiance-derived values
      REAL      mean_radiance, raddiff, bestraddiff
      REAL      frac             !Fraction of AVHRR in a class
      REAL      Lat              !Latitude
      LOGICAL   AVHRRPresent     !Some AVHRR values are present for a scan pos
!
!- End of header

      Ichan = AVHRRchan_IASI     !AVHRR channel (1 to 6) for thinning test

      IF (nrec_IASI .EQ. 1) THEN        !Only if thinning is required
        
!Thinning option 1: fixed detector for all FOVs and all scan lines
!Thinning option 7: fixed detector for all FOVs within a scan line

        IF (thinmode_IASI .EQ. 1. OR. thinmode_IASI .EQ. 7) THEN
          IF (thinmode_IASI .EQ. 1) THEN
            DO Ifov=1,iasi_1dnfov
              IbestFOV(Ifov,line) = default_IASI      !Fixed detector
            ENDDO
          ELSE
            IbestFOV(1,line) = FOVorder(MOD(line,4) + 1)
            IF (OKdetector(IbestFOV(1,line)).EQ.0) THEN
              IbestFOV(1,line) = default_IASI         !Can't use that detector
            ENDIF
            DO Ifov=2,iasi_1dnfov
              IbestFOV(Ifov,line) = IbestFOV(1,line)  !Fixed sequence
            ENDDO
          ENDIF
        ELSE
          DO Ifov=1,iasi_1dnfov   !Loop over FOVs
            thinmode = thinmode_IASI
            
!For options 2, 3 or 4 see whether AVHRR data are present

            IF (thinmode .GE. 2 .AND. thinmode .LE. 4) THEN
              AVHRRPresent = .FALSE.
              DO Idet=1,4
                DO Iclass=1,7
                  IF (.not.AVHRRPresent) THEN
                    IF (iasi1c_avhrrmean(Ichan,Iclass,Idet,Ifov).GT.0) 
     &                AVHRRPresent = .TRUE.
                  ENDIF
                ENDDO
              ENDDO
            ENDIF
            IF (.not.AVHRRPresent) thinmode = thinmode_backup

!Thinning option 4 and 6: latitude dependent options. Thinmode is re-assigned
!depending on the latitude.

            IF (thinmode .EQ. 4 .OR. thinmode .EQ. 6) THEN
              Lat = (iasi1c_latlon(1,1,Ifov)
     &             + iasi1c_latlon(1,2,Ifov)
     &             + iasi1c_latlon(1,3,Ifov)
     &             + iasi1c_latlon(1,4,Ifov)) *0.25E-4    !Mean latitude
              IF (Lat.GT.latlimits(1).OR.Lat.LT.latlimits(2)) THEN  !Polar
                IF (thinmode .EQ. 4) THEN
                  thinmode = 3       !Min variability
                ELSE
                  thinmode = 1       !Fixed
                  IbestFOV(Ifov,line) = default_IASI
                ENDIF
              ELSE               !Not polar
                IF (thinmode .EQ. 4) THEN
                  thinmode = 2       !Warmest AVHRR
                ELSE
                  thinmode = 5       !Warmest IASI
                ENDIF
              ENDIF
            ENDIF

!Thinning option 2: Highest mean AVHRR radiance for I/R channel

            IF (thinmode .EQ. 2) THEN
              maxrad = 0.0
              bestdet = default_IASI             !Default
              DO Idet=1,4
                nclass = 0
                mean_radiance = 0.0
                DO Iclass=1,7
                  rad = iasi1c_avhrrmean(Ichan,Iclass,Idet,Ifov)
                  IF (rad .GT. 0) THEN
                    nclass = nclass + 1     !This class is present
                    scale = 
     &               iasi1c_avhmeanscalefactor(Ichan,Iclass,Idet,Ifov)
                    radiance = rad * 10.0**(-scale*1.0)
                    frac = iasi1c_avhrrfrac(Iclass,Idet,Ifov)*1.0E-6
C                    frac = 1.0  !TEST
                    mean_radiance = mean_radiance + radiance*frac
                  ENDIF
                ENDDO
C                IF (nclass.GT.0) mean_radiance = mean_radiance/nclass !TEST
                IF (mean_radiance.GT.maxrad .AND. 
     &                  OKdetector(Idet).EQ.1 .AND.
     &                  iasi1c_GQisFlagQual(Idet,Ifov).EQ.0) THEN
                  bestdet = Idet
                  maxrad = mean_radiance
                ENDIF
              ENDDO
              IbestFOV(Ifov,line) = bestdet

!Thinning option 3: Most homogeneous AVHRR
!Look for the FOV with the lowest number of classes. If two FOVs have the same
!number of classes choose the one with the lowest radiance spread

            ELSE IF (thinmode .EQ. 3) THEN
              minclass = 8
              bestdet = default_IASI             !Default
              bestraddiff = 1.0E10
              DO Idet=1,4
                nclass = 0            !Find number of used classes for each spot
                maxrad = 0.0          ! and the radiance spread
                minrad = 1.0E10
                DO Iclass=1,7
                  rad = iasi1c_avhrrmean(Ichan,Iclass,Idet,Ifov)
                  IF (rad .GT. 0) THEN
                    nclass = nclass + 1     !This class is present
                    scale = 
     &                iasi1c_avhmeanscalefactor(Ichan,Iclass,Idet,Ifov)
                    radiance = rad * 10.0**(-scale*1.0)
                    sdev = iasi1c_avhrrsdev(Ichan,Iclass,Idet,Ifov)
                    IF (sdev.GT.0) THEN   !Check that SDev values are present
                      scale = 
     &                 iasi1c_avhsdevscalefactor(Ichan,Iclass,Idet,Ifov)
                      radsd = sdev * 10.0**(-scale*1.0)
                    ELSE
                      radsd = 0.0
                    ENDIF
                    maxrad = max(maxrad,radiance+radsd)
                    minrad = min(minrad,radiance-radsd)
                  ENDIF
                ENDDO
                raddiff = maxrad-minrad
                IF (OKdetector(Idet).EQ.1 .AND.
     &                  iasi1c_GQisFlagQual(Idet,Ifov).EQ.0) THEN
                  IF (nclass.GT.0 .AND. nclass.LT.minclass) THEN    !New minimum
                    minclass = nclass
                    bestraddiff = raddiff
                    bestdet = Idet
                  ELSE IF (nclass .EQ. minclass) THEN    !Same number of classes
                    IF (raddiff .LT. bestraddiff) THEN   ! - check the spread
                      bestraddiff = raddiff
                      bestdet = Idet
                    ENDIF
                  ENDIF
                ENDIF
              ENDDO
              IbestFOV(Ifov,line) = bestdet

!Thinning option 5: Highest mean IASI radiance for I/R channel

            ELSE IF (thinmode .EQ. 5) THEN
              imaxrad = 0
              bestdet = default_IASI             !Default
              DO Idet=1,4
                rad = iasi1c_scalrad(IASIchan,Idet,Ifov)
                IF (rad .GT. imaxrad .AND. OKdetector(Idet).EQ.1 .AND.
     &                  iasi1c_GQisFlagQual(Idet,Ifov).EQ.0) THEN
                  bestdet = Idet
                  imaxrad = rad
                ENDIF
              ENDDO
              IbestFOV(Ifov,line) = bestdet
            ENDIF    !thinmodethinmode_IASI
          ENDDO    !Ifov loop
        ENDIF    !not mode 1
      ENDIF    !nrec_IASI .EQ. 1
      END
