!+
!NOM/NAME $Id: pplfdf.F 25 2006-02-03 15:43:53Z frna $
!+ Read fixed data file for PPLUT within ATOVPP
!
      SUBROUTINE PPLFDF ( Lfunit, SatId )
      IMPLICIT NONE
c----------------------------------------------------------------------
c version 1.1  15/01/01  correction of bug:            (TL)  
c          change       IF ( isat.EQ.SatId ) THEN
c              to       IF ( isat.NE.SatId ) THEN
c----------------------------------------------------------------------
c
! Name: $Id: pplfdf.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Read fixed data file with (optional) corrections for use by PPLUT
!
!   Note that many of the mappings are commented out.
!   Their inclusion would require other modifications to ATOVPP,
!   such as the coding of appropriate pre-processing tests and of
!   the necessary FOV-selection routines PPA2B,..etc.
!
! Method:
! 0.  initialisation
! 1.  open file
! 2.  read optional time/angle corrections for PPLUT
! 3.  close file
!
!   File format is specified in AAPP documentation APP204F, vn of 27/11/96
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      10/12/96  LUT corrections are satellite-dependent
!  1.2      13/11/98  A2B values now initialised.
!  1.3      09/03/04  Modify for Linux (N Atkinson / A Korpela)
!  1.4      23/08/05  Add IASI (NCA)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!
      INTEGER       Ninstruments         ! number of instruments recognised:
      PARAMETER   ( Ninstruments = 5 )   !   HIRS, AMSU-A, AMSU-B, MSU, IASI
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       Lfunit        ! unit number for LUT corrections
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status value
      INTEGER       fov           ! loop index: fov number
      INTEGER       i             ! loop variable over instruments
      INTEGER       indx          ! index to instrument in input_line
      INTEGER       isat          ! sat id from LUT file (e.g. 14 for NOAA-14)
      INTEGER       maxindx       ! upper limit on indx
      INTEGER       ninst         ! number of instruments found
      REAL          dummy         ! used to read past unwanted data
      CHARACTER*6   cmap          ! mapping instrument (e.g. "MSU")
      CHARACTER*6   ctarg         ! target instrument (e.g. "HIRS")
!
! Local Arrays:
      INTEGER       inst(Ninstruments)       ! list of instruments found
      INTEGER       leninstr(Ninstruments)    ! lengths of instrument names
      CHARACTER*6   cinstrument(Ninstruments) ! instrument names
      DATA cinstrument / "HIRS", "AMSU-A", "AMSU-B", "MSU", "IASI" /
!
! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine
      External C2Upper            ! convert character string to upper case

!- End of header


! 0.  initialisation
!     --------------

!     Set all LUT corrections to zero, in case they're not specified
!     in the LUT fixed data file.

! 0.1 target instrument = HIRS
      DO fov = 1, Hnfovs
         A2Htime_cor(fov) = 0.
         A2Hscan_cor(fov) = 0.
C         B2Htime_cor(fov) = 0.
C         B2Hscan_cor(fov) = 0.
         M2Htime_cor(fov) = 0.
         M2Hscan_cor(fov) = 0.
      ENDDO

! 0.2 target instrument = AMSU-A
      DO fov = 1, Anfovs
C         H2Atime_cor(fov) = 0.
C         H2Ascan_cor(fov) = 0.
         B2Atime_cor(fov) = 0.
         B2Ascan_cor(fov) = 0.
      ENDDO

! 0.3 target instrument = AMSU-B
      DO fov = 1, Bnfovs
C         H2Btime_cor(fov) = 0.
C         H2Bscan_cor(fov) = 0.
         A2Btime_cor(fov) = 0.
         A2Bscan_cor(fov) = 0.
      ENDDO

! 0.4 target instrument = IASI
      DO fov = 1, Inscanpos
         A2Itime_cor(fov) = 0.
         A2Iscan_cor(fov) = 0.
      ENDDO

! 0.5 count the length of the instrument names
      maxindx = LEN(cinstrument(1)) ! upper limit for indx
      DO i = 1, Ninstruments
         cmap = cinstrument(i)
         indx = maxindx   ! point to last character in string
         DO WHILE (cmap(indx:indx).EQ." ")
            indx = indx - 1
         ENDDO  ! WHILE
         leninstr(i) = indx
      ENDDO  ! i
      maxindx = LEN(input_line)+1 - LEN(cinstrument(1)) ! upper limit for indx


! 1.  open file
!     ---- ----

      OPEN ( UNIT=Lfunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus ) 

!     If that didn"t work, use STATUS="UNKNOWN" (Linux ifc compiler)
      IF ( errStatus.NE.0 )
     &  OPEN ( UNIT=Lfunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus ) 

      WRITE ( errMessage,
     & "("" Error opening LUT fixed data file, unit "",I8)" ) Lfunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPLFDF")


! 2.  read optional time/angle corrections for PPLUT
!     ---- -------- ---------- ----------- --- -----

! 2.1 Loop through lines to the end, looking for lines that aren't comments
      READ(Lfunit,"(A80)",IOSTAT=errStatus) input_line
      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )

         IF ( input_line(1:1) .NE. "!" ) THEN
!            not a comment line  -interpret it as a mapping e.g. "MSU to HIRS"
             CALL C2Upper ( input_line )  ! convert to upper case

!            look through the string for the satellite id (NOAA-?? or METOP-?)
             indx = 0   ! pointer to character in input_line
             isat = 0   ! satellite id (e.g. 14 for NOAA-14)
             DO WHILE ( indx.LT.maxindx .AND. isat.EQ.0 )
                indx = indx + 1
                IF  ( input_line(indx:indx+4) .EQ. "NOAA-" ) THEN
                    READ(input_line(indx+5:indx+6),"(I2)") isat
                ELSE IF ( input_line(indx:indx+5) .EQ. "METOP-" ) THEN
                    READ(input_line(indx+6:indx+6),"(I1)") isat
                ENDIF
             ENDDO  ! WHILE

!            look through the string for the first 2 instrument names
             indx = 0   ! pointer to character in input_line
             ninst = 0  ! number of instrument names found
             DO WHILE ( indx.LT.maxindx .AND. ninst.LT.2 )
                indx = indx + 1
                DO i = 1, Ninstruments
                   IF  ( input_line(indx:indx+leninstr(i)-1)
     &                  .EQ. cinstrument(i) ) THEN ! found it here
                        ninst = ninst + 1
                        inst(ninst) = i
                        indx = indx + leninstr(i)  ! move past this instrument
                   ENDIF
                ENDDO  ! i
             ENDDO  ! WHILE

             IF ( ninst.LT.2 ) THEN
                errMessage = "Mapping line not understood"
                errStatus = 22
                CALL ErrorReport(errStatus,errMessage,"PPLFDF")
             ENDIF

!            First name is the mapping instrument, second the target
             cmap  = cinstrument(inst(1))
             ctarg = cinstrument(inst(2))

!            Read in the corrections, depending on the mapping specified
              IF ( ctarg.EQ."HIRS" ) THEN

! ***            satellite id not wanted  - read past the data
!                 IF ( isat.EQ.SatId ) THEN
                 IF ( isat.NE.SatId ) THEN
!                   read time corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (dummy,fov=1,Hnfovs)
!                   read angle corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (dummy,fov=1,Hnfovs)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")

! ***            AMSU-A to HIRS
                 ELSEIF ( cmap.EQ."AMSU-A" ) THEN
!                   read time corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (A2Htime_cor(fov),fov=1,Hnfovs)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
!                   read angle corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (A2Hscan_cor(fov),fov=1,Hnfovs)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")

C! ***            AMSU-B to HIRS
C                 ELSEIF ( cmap.EQ."AMSU-B" ) THEN
C
C!                   read time corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (B2Htime_cor(fov),fov=1,Hnfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C!                   read angle corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (B2Hscan_cor(fov),fov=1,Hnfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C
! ***            MSU to HIRS
                 ELSEIF ( cmap.EQ."MSU" ) THEN

!                   read time corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (M2Htime_cor(fov),fov=1,Hnfovs)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
!                   read angle corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (M2Hscan_cor(fov),fov=1,Hnfovs)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")

                 ENDIF  ! cmap

              ELSEIF ( ctarg.EQ."AMSU-A" ) THEN

! ***            satellite id not wanted  - read past the data
                 IF ( isat.EQ.SatId ) THEN

!                   read time corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (dummy,fov=1,Anfovs)
!                   read angle corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (dummy,fov=1,Anfovs)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")

C! ***            HIRS to AMSU-A
C                 ELSEIF ( cmap.EQ."HIRS" ) THEN
C
C!                   read time corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (H2Atime_cor(fov),fov=1,Anfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C!                   read angle corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (H2Ascan_cor(fov),fov=1,Anfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C
! ***            AMSU-B to AMSU-A
                 ELSEIF ( cmap.EQ."AMSU-B" ) THEN
!                   read time corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (B2Atime_cor(fov),fov=1,Anfovs)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
!                   read angle corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (B2Ascan_cor(fov),fov=1,Anfovs)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")

C              ELSEIF ( ctarg.EQ."AMSU-B" ) THEN
C
C! ***            satellite id not wanted  - read past the data
C                 IF ( isat.EQ.SatId ) THEN
C
C!                   read time corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (dummy,fov=1,Bnfovs)
C!                   read angle corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (dummy,fov=1,Bnfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C
C! ***            HIRS to AMSU-B
C                 ELSEIF ( cmap.EQ."HIRS" ) THEN
C
C!                   read time corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (H2Btime_cor(fov),fov=1,Bnfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C!                   read angle corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (H2Bscan_cor(fov),fov=1,Bnfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C
C! ***            AMSU-A to AMSU-B
C                 ELSEIF ( cmap.EQ."AMSU-B" ) THEN
C                 IF ( cmap.EQ."AMSU-A" ) THEN
C
C!                   read time corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (A2Btime_cor(fov),fov=1,Bnfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C!                   read angle corrections
C                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
C     &                  (A2Bscan_cor(fov),fov=1,Bnfovs)
C                    errStatus = ABS (errStatus)   ! end-of-file returns -1
C                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
C
                 ENDIF  ! cmap

              ELSEIF ( ctarg.EQ."IASI" ) THEN

! ***            satellite id not wanted  - read past the data
                 IF ( isat.NE.SatId ) THEN
!                   read time corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (dummy,fov=1,Inscanpos)
!                   read angle corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (dummy,fov=1,Inscanpos)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")

! ***            AMSU-A to IASI
                 ELSEIF ( cmap.EQ."AMSU-A" ) THEN
!                   read time corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (A2Itime_cor(fov),fov=1,Inscanpos)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")
!                   read angle corrections
                    READ(Lfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (A2Iscan_cor(fov),fov=1,Inscanpos)
                    errStatus = ABS (errStatus)   ! end-of-file returns -1
                    CALL ErrorReport(errStatus,errMessage,"PPLFDF")

                 ENDIF  ! cmap
              ENDIF  ! ctarg

         ENDIF  ! input_line(1:1).NE."!"

         READ(Lfunit,"(A80)",IOSTAT=errStatus) input_line

      ENDDO  !  WHILE ( errStatus.EQ.0 )


! 3.  close file
!     ----- ----

      CLOSE(Lfunit)


      RETURN
      END
