!+ Set up mapping Look-Up-Tables
!
      SUBROUTINE PPLUT ( H1Cunit, A1Cunit, B1Cunit, M1Cunit, I1Cunit,
     &                   IPCunit,
     &                   LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &                   GHIRS, GAMSA, GAMSB, GIASI )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pplut.F 182 2010-01-12 15:50:02Z frna $
!
! Purpose:
!   Set up mapping Look-Up-Tables
!
! Method:
! 1.  Get scanline numbers & times from start of datasets
! 2.  Compute look-up tables for mapping. (LUTMAP)
!
!
! Note that many of the mappings produce error messages.
! Their inclusion would require other modifications to ATOVPP,
! such as the coding of appropriate pre-processing tests and of
! the necessary FOV-selection routines PPA2B,..etc.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      11/06/98  *cycle REAL not INTEGER
!  1.2      12/10/00  Bug fix. Sign of dTime adjustments reversed. (Keith Whyte)
!  1.3      20/07/04  Pass 1c satid as extra argument to LUTMAP (Nigel Atkinson)
!  1.4      18/01/05  Update for IASI (Nigel Atkinson)
!  1.5      23/09/08  Add IASI PC
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  HIRS level 1C header/data common blocks 
#include "hrs1c.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  AMSU-B level 1C header/data common blocks 
#include "amb1c.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!  IASI PC header/data common blocks 
#include "iasipc.h"
!  block of HIRS data 
#include "pphstore.h"
!  block of AMSU-A data 
#include "ppastore.h"
!  block of AMSU-B data 
#include "ppbstore.h"
!  block of MSU data 
#include "ppmstore.h"
!  block of IASI data 
#include "ppistore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
! Subroutine arguments:
!
!   Scalar arguments with intent(in)
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       B1Cunit       ! unit number for AMSU-B level 1C file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
      INTEGER       I1Cunit       ! unit number for IASI level 1C file
      INTEGER       IPCunit       ! unit number for IASI PC file
!
!   Scalar arguments with intent(InOut)
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LIASI         ! .TRUE. to process IASI
      LOGICAL       LPCIASI       ! .TRUE. to process IASI PC
!
      LOGICAL       GHIRS         ! .TRUE. to output on HIRS grid
      LOGICAL       GAMSA         ! .TRUE. to output on AMSU-A grid
      LOGICAL       GAMSB         ! .TRUE. to output on AMSU-B grid
      LOGICAL       GIASI         ! .TRUE. to output on IASI grid
!
! Local Scalars:
      INTEGER       H1Crecord     ! record number for HIRS level 1C file
      INTEGER       A1Crecord     ! record number for AMSU-A level 1C file
      INTEGER       B1Crecord     ! record number for AMSU-B level 1C file
      INTEGER       M1Crecord     ! record number for MSU level 1C file
      INTEGER       I1Crecord     ! record number for IASI level 1C file
      INTEGER       IPCrecord     ! record number for IASI PC file

!v1.4: Hyear1, Hday1, Htime1, Hline1 are now stored in common pphstore. 
!Similarly for other instruments.

      INTEGER       dTime         ! (mapping line 1 time) - (target line 1 time)
                                  ! (milliseconds)
      INTEGER       LUTduration   ! time period covered by LUT (secs)
!
      CHARACTER*6   cmap          ! mapping instrument
      CHARACTER*6   ctarg         ! target instrument
!                                      (i.e. HIRS, AMSU-A, AMSU-B, MSU, IASI)
      INTEGER       imapping      ! loop index, looping through mappings
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
!
! Function & Subroutine calls:
      External IOH1C                ! I/O routine for level 1C HIRS data
      External IOA1C                ! I/O routine for level 1C AMSU-A data
      External IOB1C                ! I/O routine for level 1C AMSU-B data
      External IOM1C                ! I/O routine for level 1C MSU data
      External IOI1C                ! I/O routine for level 1C IASI data
      External TimeSub              ! subtract two dates/times
      External LUTMAP               ! set up Look-Up-Tables for the mapping
      External ErrorReport          ! ATOVPP error routine
!- End of header

! 1.  Get scanline numbers & times from start of datasets
!     --- -------- ------- - ----- ---- ----- -- --------

! 1.1 HIRS
      IF (LHIRS) THEN

         H1Crecord = Hhdrcnt + 1  !first data record in 1C file

!        Read 1C records until we get a "good" one (with line number & time ok)
         DO WHILE ( H1Crecord.LE.(Hhdrcnt+Hscnlin)
     &       .AND. ( hrs1c_scnlin.LT.1 .OR. hrs1c_scnlin.GT.9999
     &        .OR.   BTEST(hrs1c_scanqual,bits(23)) ) )
            CALL IOH1C ("READ DATA", H1Cunit, H1Crecord )
            H1Crecord = H1Crecord + 1  ! go on to next record
         ENDDO

         IF ( hrs1c_scnlin.LT.1 .OR. hrs1c_scnlin.GT.9999
     &      .OR. BTEST(hrs1c_scanqual,bits(23)) ) THEN
!        no "good" HIRS data
            errMessage = "HIRS data all corrupt & so not processed"
            errStatus = -11  ! warning
            CALL ErrorReport(errStatus,errMessage,"PPLUT")
            LHIRS = .FALSE.
            GHIRS = .FALSE.
         ELSE
!        get date/time/line number
            Hyear1 = hrs1c_scnlinyr
            Hday1  = hrs1c_scnlindy
            Htime1 = hrs1c_scnlintime
            Hline1 = hrs1c_scnlin
         ENDIF

      ENDIF  ! LHIRS

! 1.2 AMSU-A
      IF (LAMSA) THEN

         A1Crecord = Ahdrcnt + 1  !first data record in 1C file

!        Read 1C records until we get a "good" one (with line number & time ok)
         DO WHILE ( A1Crecord.LE.(Ahdrcnt+Ascnlin)
     &       .AND. ( ama1c_scnlin.LT.1 .OR. ama1c_scnlin.GT.9999
     &        .OR.   BTEST(ama1c_scanqual,bits(23)) ) )
            CALL IOA1C ("READ DATA", A1Cunit, A1Crecord )
            A1Crecord = A1Crecord + 1  ! go on to next record
         ENDDO

         IF ( ama1c_scnlin.LT.1 .OR. ama1c_scnlin.GT.9999
     &      .OR. BTEST(ama1c_scanqual,bits(23)) ) THEN
!        no "good" AMSU-A data
            errMessage = " AMSU-A data all corrupt & so not processed"
            errStatus = -12  ! warning
            CALL ErrorReport(errStatus,errMessage,"PPLUT")
            LAMSA = .FALSE.
            GAMSA = .FALSE.
         ELSE
!        get date/time/line number
            Ayear1 = ama1c_scnlinyr
            Aday1  = ama1c_scnlindy
            Atime1 = ama1c_scnlintime
            Aline1 = ama1c_scnlin
         ENDIF

      ENDIF  ! LAMSA

! 1.3 AMSU-B
      IF (LAMSB) THEN

         B1Crecord = Bhdrcnt + 1  !first data record in 1C file

!        Read 1C records until we get a "good" one (with line number & time ok)
         DO WHILE ( B1Crecord.LE.(Bhdrcnt+Bscnlin)
     &       .AND. ( amb1c_scnlin.LT.1 .OR. amb1c_scnlin.GT.9999
     &        .OR.   BTEST(amb1c_scanqual,bits(23)) ) )
            CALL IOB1C ("READ DATA", B1Cunit, B1Crecord )
            B1Crecord = B1Crecord + 1  ! go on to next record
         ENDDO

         IF ( amb1c_scnlin.LT.1 .OR. amb1c_scnlin.GT.9999
     &      .OR. BTEST(amb1c_scanqual,bits(23)) ) THEN
!        no "good" AMSU-B data
            errMessage = "AMSU-B data all corrupt & so not processed"
            errStatus = -13  ! warning
            CALL ErrorReport(errStatus,errMessage,"PPLUT")
            LAMSB = .FALSE.
            GAMSB = .FALSE.
         ELSE
!        get date/time/line number
            Byear1 = amb1c_scnlinyr
            Bday1  = amb1c_scnlindy
            Btime1 = amb1c_scnlintime
            Bline1 = amb1c_scnlin
         ENDIF

      ENDIF  ! LAMSB

! 1.4 MSU
      IF (LMSU) THEN

         M1Crecord = Mhdrcnt + 1  !first data record in 1C file

!        Read 1C records until we get a "good" one (with line number & time ok)
         DO WHILE ( M1Crecord.LE.(Mhdrcnt+Mscnlin)
     &       .AND. ( msu1c_scnlin.LT.1 .OR. msu1c_scnlin.GT.9999
     &        .OR.   BTEST(msu1c_scanqual,bits(23)) ) )
            CALL IOM1C ("READ DATA", M1Cunit, M1Crecord )
            M1Crecord = M1Crecord + 1  ! go on to next record
         ENDDO

         IF ( msu1c_scnlin.LT.1 .OR. msu1c_scnlin.GT.9999
     &      .OR. BTEST(msu1c_scanqual,bits(23)) ) THEN
!        no "good" MSU data
            errMessage = "MSU data all corrupt & so not processed"
            errStatus = -14  ! warning
            CALL ErrorReport(errStatus,errMessage,"PPLUT")
            LMSU = .FALSE.
         ELSE
!        get date/time/line number
            Myear1 = msu1c_scnlinyr
            Mday1  = msu1c_scnlindy
            Mtime1 = msu1c_scnlintime
            Mline1 = msu1c_scnlin
         ENDIF

      ENDIF  ! LMSU

! 1.5 IASI
      IF (LIASI) THEN

         I1Crecord = Ihdrcnt + 1  !first data record in 1C file

!        Read 1C records until we get a "good" one (with line number ok)
         DO WHILE ( I1Crecord.LE.(Ihdrcnt+Iscnlin)
     &       .AND. ( iasi1c_scnlin.LT.1 .OR. iasi1c_scnlin.GT.9999 ) )
            CALL IOI1C ("READ DATA", I1Cunit, I1Crecord )
            I1Crecord = I1Crecord + 1  ! go on to next record
         ENDDO

         IF ( iasi1c_scnlin.LT.1 .OR. iasi1c_scnlin.GT.9999 ) THEN
!        no "good" IASI data
            errMessage = "IASI data all corrupt & so not processed"
            errStatus = -15  ! warning
            CALL ErrorReport(errStatus,errMessage,"PPLUT")
            LIASI = .FALSE.
         ELSE
!        get date/time/line number
            Iyear1 = iasi1c_scnlinyr
            Iday1  = iasi1c_scnlindy
            Itime1 = iasi1c_scnlintime(1)
            Iline1 = iasi1c_scnlin
         ENDIF

      ENDIF  ! LIASI


! 1.5 IASI PC
      IF (LPCIASI) THEN

         IPCrecord = Ihdrcnt + 1  !first data record in PC file

!        Read PC records until we get a "good" one (with line number ok)
         DO WHILE ( IPCrecord.LE.(Ihdrcnt+Iscnlin)
     &       .AND. ( iasipc_scnlin.LT.1 .OR. iasipc_scnlin.GT.9999 ) )
            CALL IOIPC ("READ DATA", IPCunit, IPCrecord )
            IPCrecord = IPCrecord + 1  ! go on to next record
         ENDDO

         IF ( iasipc_scnlin.LT.1 .OR. iasipc_scnlin.GT.9999 ) THEN
!        no "good" IASI data
            errMessage = "IASI PC data all corrupt & so not processed"
            errStatus = -15  ! warning
            CALL ErrorReport(errStatus,errMessage,"PPLUT")
            LIASI = .FALSE.
         ELSE
!        get date/time/line number
            Iyear1 = iasipc_scnlinyr
            Iday1  = iasipc_scnlindy
            Itime1 = iasipc_scnlintime(1)
            Iline1 = iasipc_scnlin
         ENDIF

      ENDIF  ! LIASI


! 2.  Compute look-up tables for mapping (not IASI - see pplut_iasi.F)
!     ------- ------- ------ --- -------

!     loop through requested mappings
      IF ( Nmappings.EQ.0.OR.
     &   ( Nmappings.EQ.1.AND.TargInstr(1).EQ."IASI") ) GOTO 999

      errStatus = 1   ! reset to zero if LUTs calculated ok
      DO imapping = 1, Nmappings

         cmap  = MapInstr(imapping)
         ctarg = TargInstr(imapping)

! 2.1    Map data to HIRS grid
         IF ( ctarg.EQ."HIRS" ) THEN

! 2.1.1     map AMSU-A
            IF    ( cmap.EQ."AMSU-A" ) THEN

!             Calculate time difference between instrument grids
!             dTime = time(first AMSU-A scanline in dataset) - time(first HIRS)
              CALL TimeSub ( Ayear1, Aday1, Atime1,
     &                       Hyear1, Hday1, Htime1, dTime, errStatus )
              errMessage = "Error with AMSU-A and HIRS scan times"
              errStatus = ABS(errStatus)  ! fatal if non-zero
              CALL ErrorReport(errStatus,errMessage,"PPLUT")

!             subtract time since AMSU-A scanline number 1
              dTime = dTime - NINT(Acycle*(Aline1-1))
!             add time since HIRS scanline number 1
              dTime = dTime + NINT(Hcycle*(Hline1-1))

              A2Hstart = 9     ! a 'reasonable' target start line for LUT
              LUTduration = NINT(A2Hnlines*Hcycle) ! duration of LUT (seconds)

              CALL LUTMAP ( "AMSU-A", "HIRS", A2Hstart, Hnfovs,
     &             A2Hnlines, A2Hmult, LUTduration, dTime, A2Htime_cor,
     &             A2Hscan_cor, A2HLUT, LUTelements, A2Hncolocs,
     &             Hsatht, Hsatperiod, A2Hmode, LUTmultfact,
     &             ama1c_h_satid, ama1c_h_wmosatid, errStatus )
              errMessage = "Error deriving AMSU-A to HIRS mapping LUT"

! 2.1.2     map AMSU-B
            ELSEIF( cmap.EQ."AMSU-B" ) THEN
               errStatus = 212
               WRITE(errMessage,*)
     &         "Cannot make LUT to map ", cmap, " to ", ctarg

! 2.1.3     map MSU
            ELSEIF( cmap.EQ."MSU" ) THEN

!             Calculate time difference between instrument grids
!             dTime = time(first MSU scanline in dataset) - time(first HIRS)
              CALL TimeSub ( Myear1, Mday1, Mtime1,
     &                       Hyear1, Hday1, Htime1, dTime, errStatus )
              errMessage = "Error with MSU and HIRS scan times"
              errStatus = ABS(errStatus)  ! fatal if non-zero
              CALL ErrorReport(errStatus,errMessage,"PPLUT")

!             subtract time since MSU scanline number 1
              dTime = dTime - NINT(Mcycle*(Mline1-1))
!             add time since HIRS scanline number 1
              dTime = dTime + NINT(Hcycle*(Hline1-1))

              M2Hstart = 25    ! a 'reasonable' target start line for LUT
              LUTduration = NINT(M2Hnlines*Hcycle) ! duration of LUT (seconds)

              CALL LUTMAP ( "MSU", "HIRS", M2Hstart, Hnfovs,
     &             M2Hnlines, M2Hmult, LUTduration, dTime, M2Htime_cor,
     &             M2Hscan_cor, M2HLUT, LUTelements, M2Hncolocs,
     &             Hsatht, Hsatperiod, M2Hmode, LUTmultfact,
     &             msu1c_h_satid, 0, errStatus)  !Don't bother with wmosatid
              errMessage = "Error deriving MSU to HIRS mapping LUT"

            ENDIF  ! CMap

! 2.2    Map data to AMSU-A grid
         ELSEIF ( ctarg.EQ."AMSU-A" ) THEN

! 2.2.1    map HIRS
           IF    ( cmap.EQ."HIRS" ) THEN
               errStatus = 221
               WRITE(errMessage,*)
     &         "Cannot make LUT to map ", cmap, " to ", ctarg

! 2.2.2    map AMSU-B
           ELSEIF( cmap.EQ."AMSU-B" ) THEN

!             Calculate time difference between instrument grids
!             dTime = time(first AMSU-B scanline in dataset)-time(first AMSU-A)
              CALL TimeSub ( Byear1, Bday1, Btime1,
     &                       Ayear1, Aday1, Atime1, dTime, errStatus )
              errMessage = "Error with AMSU-B and AMSU-A scan times"
              errStatus = ABS(errStatus)  ! fatal if non-zero
              CALL ErrorReport(errStatus,errMessage,"PPLUT")

!             subtract time since AMSU-B scanline number 1
              dTime = dTime - NINT(Bcycle*(Bline1-1))
!             add time since AMSU-A scanline number 1
              dTime = dTime + NINT(Acycle*(Aline1-1))

              B2Astart = 3     ! a 'reasonable' target start line for LUT
              LUTduration = NINT(B2Anlines*Acycle) ! duration of LUT (seconds)

              CALL LUTMAP ( "AMSU-B", "AMSU-A", B2Astart, Anfovs,
     &             B2Anlines, B2Amult, LUTduration, dTime, B2Atime_cor,
     &             B2Ascan_cor, B2ALUT, LUTelements, B2Ancolocs,
     &             Asatht, Asatperiod, B2Amode, LUTmultfact,
     &             amb1c_h_satid, amb1c_h_wmosatid, errStatus )
              errMessage = "Error deriving AMSU-B to A mapping LUT"

           ENDIF  ! cmap

! 2.3    Map data to AMSU-B grid
         ELSEIF ( ctarg.EQ."AMSU-B" ) THEN

! 2.3.1    map HIRS
           IF    ( cmap.EQ."HIRS" ) THEN
               errStatus = 231
               WRITE(errMessage,*)
     &         "Cannot make LUT to map ", cmap, " to ", ctarg

! 2.3.2    map AMSU-A
           ELSEIF( cmap.EQ."AMSU-A" ) THEN

!             Calculate time difference between instrument grids
!             dTime = time(first AMSU-A scanline in dataset)-time(first AMSU-B)
              CALL TimeSub ( Ayear1, Aday1, Atime1,
     &                       Byear1, Bday1, Btime1, dTime, errStatus )
              errMessage = "Error with AMSU-A and AMSU-B scan times"
              errStatus = ABS(errStatus)  ! fatal if non-zero
              CALL ErrorReport(errStatus,errMessage,"PPLUT")

!             subtract time since AMSU-A scanline number 1
              dTime = dTime - NINT(Acycle*(Aline1-1))
!             add time since AMSU-B scanline number 1
              dTime = dTime + NINT(Bcycle*(Bline1-1))

              A2Bstart = 40    ! a 'reasonable' target start line for LUT
              LUTduration = NINT(A2Bnlines*Bcycle) ! duration of LUT (seconds)

              CALL LUTMAP ( "AMSU-A", "AMSU-B", A2Bstart, Bnfovs,
     &             A2Bnlines, A2Bmult, LUTduration, dTime, A2Btime_cor,
     &             A2Bscan_cor, A2BLUT, LUTelements, A2Bncolocs,
     &             Bsatht, Bsatperiod, A2Bmode, LUTmultfact,
     &             ama1c_h_satid, ama1c_h_wmosatid, errStatus )
              errMessage = "Error deriving AMSU-A to B mapping LUT"

           ENDIF  ! cmap

         ENDIF  ! ctarg

!        check for errors in deriving the LUTs
         CALL ErrorReport(errStatus,errMessage,"PPLUT")

      ENDDO  ! imapping

 999  CONTINUE   ! jump here if no LUTs needed

      RETURN
      END
