!+ Set up mapping Look-Up-Tables
!
      SUBROUTINE PPLUT_IASI ( IFOV )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pplut_iasi.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Set up mapping Look-Up-Tables for IASI
!
! Method:
! 1.  Compute look-up tables for mapping. (LUTMAP)
!
! Scan line times and instrument viewing geometries have already been
! established by a call to PPLUT.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0     18/01/05  Initial version based on pplut.F. (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  parameters and look-up tables for the mapping 
#include "ppluts.h"
!  IASI level 1C header/data common blocks 
#include "iasi1c.h"
!  block of IASI data 
#include "ppistore.h"
!  AMSU-A level 1C header/data common blocks 
#include "ama1c.h"
!  block of AMSU-A data 
#include "ppastore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  fundamental and physical constants 
#include "constants_preproc.h"
!
! Subroutine arguments:
!
!   Scalar arguments with intent(in)
      INTEGER       IFOV          ! IASI FOV number (1 to 4)
!
! Local Scalars:
      INTEGER       dTime         ! (mapping line 1 time) - (target line 1 time)
                                  ! (milliseconds)
      INTEGER       LUTduration   ! time period covered by LUT (secs)
      INTEGER       I             ! Loop variable
!
      REAL          delta_x       ! IASI spot displacement in degrees
      REAL          delta_t       ! IASI spot displacement in seconds
      REAL          A2Iscan_cor_mod(Inscanpos) ! modified scan angle corrections
      REAL          A2Itime_cor_mod(Inscanpos) ! modified time corrections
!
      CHARACTER*6   cmap          ! mapping instrument
      CHARACTER*6   ctarg         ! target instrument
!                                      (i.e. HIRS, AMSU-A, AMSU-B, MSU)
      INTEGER       imapping      ! loop index, looping through mappings
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
!
! Instrument data common block as defined in lutmap.F. We need HALFCONE_IASI.
      REAL        CONE_ANGLE
      REAL        STEP_ANGLE
      REAL        STEP_TIME
      REAL        LINE_TIME
      REAL        HALFCONE_IASI   ! Separation of 4 spots from centre (deg)
      INTEGER     SPOTS_PER_LINE
      INTEGER     LEFT_TO_RIGHT_SCAN
      COMMON/INSTRUMENT/CONE_ANGLE,STEP_ANGLE,STEP_TIME,LINE_TIME,
     &       HALFCONE_IASI,SPOTS_PER_LINE,LEFT_TO_RIGHT_SCAN
!
! X and Y displacement (AMSU relative to IASI) for each of the 4 IASI FOVs
      INTEGER IX(4), IY(4)
      DATA IX /-1,-1, 1, 1/
      DATA IY / 1,-1,-1, 1/

! Function & Subroutine calls:
      External LUTMAP               ! set up Look-Up-Tables for the mapping
      External ErrorReport          ! ATOVPP error routine
!- End of header

! 1.  Compute look-up tables for mapping
!     ------- ------- ------ --- -------

!     loop through requested mappings
      IF ( Nmappings.EQ.0 ) GOTO 999
      errStatus = 1   ! reset to zero if LUTs calculated ok
      DO imapping = 1, Nmappings

         cmap  = MapInstr(imapping)
         ctarg = TargInstr(imapping)

! 1.1    Map data to IASI grid
         IF ( ctarg.EQ."IASI" ) THEN


! 1.1.1     set up geometry for this IASI FOV
            delta_x = IX(IFOV) * HALFCONE_IASI/SQRT(2.) !scan displacement in degrees
            delta_t = IY(IFOV) * HALFCONE_IASI/SQRT(2.) 
     &        *Isatht*Isatperiod*60/(360.0*rEarth)    !time displacement in secs

! Strictly speaking we should take into account the variation of satellite
! height around the orbit, but the above should be accurate enough.

            DO I=1,Inscanpos
              A2Iscan_cor_mod(I) = A2Iscan_cor(I) - delta_x    !Is sign correct?
              A2Itime_cor_mod(I) = A2Itime_cor(I) - delta_t
            ENDDO

! 1.1.2     map AMSU-A
            IF    ( cmap.EQ."AMSU-A" ) THEN

!             Calculate time difference between instrument grids
!             dTime = time(first AMSU-A scanline in dataset) - time(first IASI)
              CALL TimeSub ( Ayear1, Aday1, Atime1,
     &                       Iyear1, Iday1, Itime1, dTime, errStatus )
              errMessage = "Error with AMSU-A and IASI scan times"
              errStatus = ABS(errStatus)  ! fatal if non-zero
              CALL ErrorReport(errStatus,errMessage,"PPLUT_IASI")

!             subtract time since AMSU-A scanline number 1
              dTime = dTime - NINT(Acycle*(Aline1-1))
!             add time since IASIscanline number 1
              dTime = dTime + NINT(Icycle*(Iline1-1))

              A2Istart = 9     ! a 'reasonable' target start line for LUT
              LUTduration = NINT(A2Inlines*Icycle) ! duration of LUT (seconds)

              CALL LUTMAP ( "AMSU-A", "IASI", A2Istart, Inscanpos,
     &             A2Inlines, A2Imult, LUTduration, dTime, 
     &             A2Itime_cor_mod, A2Iscan_cor_mod, A2ILUT,
     &             LUTelements, A2Incolocs, Isatht, Isatperiod, A2Imode,
     &             LUTmultfact, ama1c_h_satid, ama1c_h_wmosatid,
     &             errStatus )
              errMessage = "Error deriving AMSU-A to IASI mapping LUT"
              
            ENDIF

!        check for errors in deriving the LUTs
         CALL ErrorReport(errStatus,errMessage,"PPLUT_IASI")

         ENDIF   !ctarg.EQ."IASI"
      ENDDO  ! imapping

 999  CONTINUE   ! jump here if no LUTs needed

      RETURN
      END
