!+ Map MSU to an individual HIRS fov
!
      SUBROUTINE PPM2H ( TFOV, TLine,
     &                   Ncolocs, List_fov, List_line, List_wt )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppm2h.F 415 2014-09-25 08:50:00Z frna $
!
! Purpose:
!   Select mapping fovs from those given and derive brightness temperatures
!   and other parameters at the specified target fov.
!
! Method:
! 0.  Initialisation
! 1.  First try only fovs with primary calibration
! 2.  then if desperate accept those with secondary calibration
! 3.  Set flags and surface type
! 4.  Calculate mapped BTs
!   Note that MSU fovs are accepted irrespective of surface type.
!   However a flag is set if the surface type of any of those selected
!   differs from that of the HIRS fov.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Don't skip bad target fovs
!  1.2        2/1/97  Bug fixed  - fov & line corrected in section 2
!  1.3      24/09/14  Make sure we don't include missing BTs (as in ppa2h.F)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of HIRS data 
#include "pphstore.h"
!  block of MSU data 
#include "ppmstore.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Arguments with intent(In):
      INTEGER       TFOV          ! target fov number
      INTEGER       TLine         ! number of target line in btemps array
      INTEGER       Ncolocs       ! no of colocated mapping FOVs
      INTEGER       List_line(*)  ! list of coloc line numbers (1st=nearest)
      INTEGER       List_fov(*)   !               spot numbers
      REAL          List_wt(*)    !               LUT weights
!
! Local scalars:
      INTEGER       chn           ! loop variable over all channels
      INTEGER       fov           ! mapping fov number
      INTEGER       line          ! number of mapping line in btemps array
      INTEGER       icoloc        ! loop variable over mapping FOVs
      INTEGER       ngood         ! no of fovs selected
      INTEGER       pp_surf       ! surface type from the nearest mapping fov
      INTEGER       pp_qc         ! pre-processor QC word
      INTEGER       qcmask        ! bitword picks out selected bits from Mqc
      LOGICAL       same_surf     ! .TRUE. so long as MSU same surface as HIRS
      REAL          sum_weights   ! sum of weights
      REAL          weight        ! weight used in the mapping
      REAL          wt_norm       ! reciprocal of sum_weights
!
! Local arrays:
      REAL         sum(40)        ! sum of weights*BTs for each channel
      LOGICAL      use_coloc(90)  ! mapping FOVs: .TRUE. to use
!
!- End of header


! 0.  Initialisation
!     --------------

! 0.1 Initially, no mapping fovs have been selected
      pp_qc = 0
      ngood = 0         ! number of fovs chosen so far
      DO icoloc = 1, Ncolocs
         use_coloc(icoloc) = .FALSE.
      ENDDO  ! icoloc


! 1.  First try only fovs with primary calibration
!     ----- --- ---- ---- ---- ------- -----------

      same_surf = .TRUE.  ! "MSU surf type = HIRS surf type"
      qcmask = 0
      qcmask = IBSET ( qcmask, bits(0) )  ! bad/missing data
      qcmask = IBSET ( qcmask, bits(30) ) ! secondary calibration used

      DO icoloc = 1, Ncolocs

         fov  = List_fov(icoloc)
         line = List_line(icoloc)
!        IF ( (Mqc(fov,line).AND.qcmask).EQ.0 ) THEN  ! primary calib ok
         IF ( IAND(Mqc(fov,line),qcmask).EQ.0 ) THEN  ! primary calib ok

            use_coloc(icoloc) = .TRUE.
            ngood = ngood + 1
!           Pick out nearest-neighbour surface type:
            IF ( ngood.EQ.1 ) THEN
               pp_surf = Msurftype(fov,line)
!              pp_qc = Mqc(fov,line).AND.qcmask
               pp_qc = IAND(Mqc(fov,line),qcmask)
            ENDIF
!           Note if this is the same surface type as HIRS
            same_surf = same_surf .AND.
     &                  Hsurftype(TFOV,TLine).EQ.Msurftype(fov,line)

         ENDIF

      ENDDO  ! icoloc


! 2.  then if desperate accept those with secondary calibration
!     ---- -- --------- ------ ----- ---- --------- -----------

      IF ( ngood.EQ.0 ) THEN

!        secondary calibration is now acceptable
         qcmask = IBCLR ( qcmask, bits(30) )

         DO icoloc = 1, Ncolocs

            fov  = List_fov(icoloc)
            line = List_line(icoloc)
!           IF ( (Mqc(fov,line).AND.qcmask).EQ.0 ) THEN  ! secondary calib ok
            IF ( IAND(Mqc(fov,line),qcmask).EQ.0 ) THEN  ! secondary calib ok

               use_coloc(icoloc) = .TRUE.
               ngood = ngood + 1
!              Pick out nearest-neighbour surface type:
               IF ( ngood.EQ.1 ) THEN
                  pp_surf = Msurftype(fov,line)
!                 pp_qc = Mqc(fov,line).AND.qcmask
                  pp_qc = IAND(Mqc(fov,line),qcmask)
               ENDIF
!              Note if this is the same surface type as HIRS
               same_surf = same_surf .AND.
     &                  Hsurftype(TFOV,TLine).EQ.Msurftype(fov,line)

            ENDIF

         ENDDO  ! icoloc

         IF ( ngood.GT.0 ) pp_qc = IBSET(pp_qc,bits(2))    ! 2ndary calib

      ENDIF  ! ngood


! 3.  Set flags and surface type
!     --- ----- --- ------- ----

 300  CONTINUE  ! jump here if target fov bad

      IF ( ngood.EQ.0 ) THEN ! no fovs found
         pp_qc = IBSET(PP_QC,bits(0))  ! MSU missing
         pp_surf = MDI
      ELSE
         IF ( .NOT.same_surf ) pp_qc = IBCLR(pp_qc,bits(3))  ! mixed surfaces
         IF ( ngood.LT.4 ) pp_qc = IBCLR(pp_qc,bits(1))    ! <4 MSU fovs
      ENDIF  ! ngood

      Hmapqc(TFOV,TLine) = pp_qc
      Hmapsurf(TFOV,TLine) = pp_surf


! 4.  Calculate mapped BTs
!     --------- ------ ---

      IF ( ngood .EQ. 0 ) THEN

! 4.1    No colocated mapping data selected  - set to missing
         DO chn = 1, Mnchan
            Hmapbt(chn,TFOV,TLine) = RMDI
         ENDDO  ! chn

      ELSE

! 4.2    initialise summation variables
         sum_weights = 0.
         DO chn = 1, Mnchan
            sum(chn)  = 0.
         ENDDO  ! chn

! 4.3    Do the mapping: sum weights*BTs for those FOVs selected
!        If any BT for a channel is missing, set the mapped BT to missing
         DO icoloc = 1, ncolocs
            IF ( use_coloc(icoloc) ) THEN
               fov  = List_fov(icoloc)
               line = List_line(icoloc)
               weight = List_wt(icoloc)
               sum_weights = sum_weights + weight
!              loop through the channels
               DO chn = 1, Mnchan
                 IF (Mbtemps(chn,fov,line) .NE. RMDI) THEN
                   IF (sum(chn) .NE. RMDI) THEN
                     sum(chn) = sum(chn) + weight*Mbtemps(chn,fov,line)
                   ENDIF
                 ELSE
                   sum(chn) = RMDI
                 ENDIF
               ENDDO  ! chn
            ENDIF  ! use_coloc(icoloc)
          ENDDO  ! icoloc

! 4.4     Loop through channels again, to calculate & store mapped BTs
          wt_norm = 1. / sum_weights
          DO chn = 1, Mnchan
             IF (sum(chn) .NE. RMDI) THEN
               Hmapbt(chn,TFOV,TLine) = sum(chn) * wt_norm
             ELSE
               Hmapbt(chn,TFOV,TLine) = RMDI
             ENDIF
          ENDDO  ! chn

       ENDIF                     ! ngood


      RETURN
      END
