!+ Read fixed data file for MSU within ATOVPP
!
      SUBROUTINE PPMFDF ( Mfunit )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppmfdf.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Read fixed data file required for MSU within ATOVPP.
!
! Method:
! 0.  initialisation
! 1.  open file
! 2.  look for sections in the file
! 3.  read numbers for the pre-processing (PREPRO) (nothing as yet)
! 4.  read bias corrections (BIAS)
! 5.  read limb-darkening curve (MSULIMB)
! 6.  close file
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      23/03/04  Modify for Linx (N.C.Atkinson)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  fixed data for processing MSU 
#include "ppmfdata.h"
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      INTEGER       Mfunit        ! unit number for MSU-specific data
!
! Local Scalars:
      CHARACTER*80  input_line    ! line read from instream data
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status value
      INTEGER       chn           ! loop index: channel number
      INTEGER       fov           ! loop index: fov number
      INTEGER       isurf         ! loop index: 1=sea, 2=land

! Function & Subroutine calls:
      External ErrorReport        ! ATOVPP error routine

!- End of header


! 0.  initialisation
!     --------------

! 0.1 bias corrections
      DO fov = 1, Mnfovs
      DO chn = 1, Mnchan
         Mbiases(chn,fov) = 0.
      ENDDO  ! chn
      ENDDO  ! fov

! 0.2 limb-darkening curve
      DO isurf = 1, 2   ! sea, land
      DO fov = 1, Mnfovs
      DO chn = 1, Mnchan
         Mlimb(chn,fov,isurf) = 0.
      ENDDO  ! chn
      ENDDO  ! fov
      ENDDO  ! isurf


! 1.  open file
!     ---- ----

      OPEN ( UNIT=Mfunit, STATUS="OLD", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )
      IF (errStatus .NE. 0)         !For Linux IFC
     &  OPEN ( UNIT=Mfunit, STATUS="UNKNOWN", FORM="FORMATTED",
     &       ACCESS="SEQUENTIAL", IOSTAT=errStatus )
      WRITE ( errMessage,
     & "(""Error opening MSU fixed data file, unit "",I8)" ) Mfunit
      errStatus = ABS (errStatus)   ! end-of-file returns -1
      CALL ErrorReport(errStatus,errMessage,"PPMFDF")


! 2.  look for sections in the file
!     ---- --- -------- -- --- ----

      DO WHILE ( errStatus.EQ.0 .AND. input_line(1:3).NE."END" )
         input_line = " "
         READ(Mfunit,"(A80)",IOSTAT=errStatus) input_line


         IF ( input_line(1:6) .EQ. "PREPRO" ) THEN

! 3.        read numbers for the pre-processing
!           ---- ------- --- --- --------------

            errMessage = "Reading PREPRO section"
            errStatus = 0
!           (as yet there are no numbers to read)
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPMFDF")


         ELSEIF ( input_line(1:4) .EQ. "BIAS" ) THEN

! 4.        read bias-corrections
!           ---- ----------------

!           read scan-dependent bias corrections
            errMessage = "Reading BIAS section"
            errStatus = 0
            DO fov = 1, Mnfovs
               IF ( errStatus.EQ.0 ) THEN
                  READ(Mfunit,"(10F8.3)",IOSTAT=errStatus)
     &                       (Mbiases(chn,fov),chn=1,Mnchan)
               ENDIF
            ENDDO  ! fov
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPMFDF")


         ELSEIF ( input_line(1:7) .EQ. "MSULIMB" ) THEN

! 5.        read limb-darkening curve
!           ---- -------------- -----

!           read limb-darkening curve
            errStatus = 0
            DO isurf = 1, 2   ! sea, land
            DO fov = 1, Hnfovs/2  ! assume limb effect is symmetric
               IF ( errStatus.EQ.0 )
     &            READ(Mfunit,"(10F8.3)",IOSTAT=errStatus)
     &                  (Mlimb(chn,fov,isurf),chn=1,Mnchan)
            ENDDO  ! fov
            ENDDO  ! isurf
            errStatus = ABS (errStatus)   ! end-of-file returns -1
            CALL ErrorReport(errStatus,errMessage,"PPMFDF")


         ENDIF  ! input_line

      ENDDO  ! WHILE ( errStatus.EQ.0 )


! 6.  close file
!     ----- ----

      CLOSE(Mfunit)


      RETURN
      END
