!+ Transfer level 1C MSU data from data record to program store
!
      SUBROUTINE PPMIND ( Line )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppmind.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   MSU 1C data, when read, is held in common/msu1cdt/. ATOVPP stores data
!   from several scanlines in its own internal array structure, contained
!   in common/pphstore/. This routine transfers a single scanline from
!   common/msu1cdt/ to slot "Line" in common/pphstore/.
!
! Method:
! 0.  Check format version numbers  (first call only)
!     Check that format of 1C include file (msu1c.h) and MSU store
!     (pphstore.h) are compatible with the code in this subroutine.
!     Call ErrorReport to print a warning if there is a problem.
! 1.  Copy variables from common/msu1cdt/ to slot "Line" in common/ppmstore/
! 2.  Check flags
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/12/96  Conform to 1C format vn 1.8
!                     Set Mqc(bit0) if any channels are missing
!  1.2      02/07/98  Mangles REAL not INTEGER
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of MSU data 
#include "ppmstore.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
!   Scalar arguments with intent(In):
      INTEGER       Line          ! line number for scanline in /ppmstore/
!
! Local parameters:
      INTEGER       M1Cvnnb       ! version number of 1C format compatible
         PARAMETER (M1Cvnnb=180)  ! with the code in this subroutine
      INTEGER       Mstvnnb       ! version number of "ppmstore.h" compatible
         PARAMETER (Mstvnnb=100)  ! with the code in this subroutine
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status for missing scanlines
      INTEGER       chn           ! loop variable over all channels
      INTEGER       chanbits      ! bit word with all channel bits set
      INTEGER       fov           ! loop variable over all fovs
      LOGICAL       badline       ! .TRUE. not to use a scanline
      LOGICAL first               ! .TRUE. on first call, then .FALSE.
         DATA first / .TRUE. /
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
      SAVE first, chanbits
!
!- End of header


! 0.  Check format version numbers  (first call only)
!     ----- ------ ------- -------

      IF ( first ) THEN

!        Is the 1C include file compatible with this source code ?
         IF ( M1Cvnnb .NE. msu_1cvnnb ) THEN
            errStatus = -11  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &      "("" Code expects 1C format"",I6,"" but msu1c.h is "",I6)")
     &           M1Cvnnb, msu_1cvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPMIND")
         ENDIF

!        Is the pphstore include file compatible with this source code ?
         IF ( Mstvnnb .NE. msu_stvnnb ) THEN
            errStatus = -12  ! warning
            errMessage = " "
            WRITE ( errMessage,
     &"("" Code expects Mstore format"",I6,"" but ppmstore.h is"",I6)")
     &           Mstvnnb, msu_stvnnb
            CALL ErrorReport ( errStatus, errMessage, "PPMIND")
         ENDIF

!        set up a mask to pick out just the flags for the individual channels
         chanbits = 0
         DO chn = 1, Mnchan
            chanbits = IBSET ( chanbits, bits(chn) )
         ENDDO

         first = .FALSE.

      ENDIF  ! ( first )


! 1.  Copy variables from common/msu1cdt/ to slot "Line" in common/ppmstore/
!     ---- --------- ---- --------------- -- ----  ----  -- ----------------

      Mlineno(Line) = msu1c_scnlin                ! scanline number
      Myear(Line) = msu1c_scnlinyr                ! scanline year
      Mday(Line) = msu1c_scnlindy                 ! scanline day of year
      Mtime(Line) =  msu1c_scnlintime             ! scanline time (ms)
      Mqualind(Line) = msu1c_qualind              ! quality indicator bit field
      Mscanqual(Line) = msu1c_scanqual            ! scan line quality flags

      DO fov = 1, Mnfovs
         Mlats(fov,Line) = REAL(msu1c_latlon(1,fov))*1.E-4 !latitude(deg,N+ve)
         Mlons(fov,Line) = REAL(msu1c_latlon(2,fov))*1.E-4 !longitude (E+ve)
         Mangles(1,fov,Line) = REAL(msu1c_angles(1,fov))*0.01 ! local zenith
         Mangles(2,fov,Line) = REAL(msu1c_angles(2,fov))*0.01 ! local azimuth
      ENDDO  ! fov

      Maltitude(Line) =  msu1c_scalti             ! satellite altitude (km*10)
                                                  ! above reference ellipsoid
      DO fov = 1, Mnfovs
      DO chn = 1, Mnchan
         Mbtemps(chn,fov,Line) = REAL(msu1c_btemps(chn,fov))*0.01 !MSU BTs (K)
      ENDDO  ! chn
      ENDDO  ! fov

      DO fov = 1, Mnfovs
         Mqc(fov,Line) = msu1c_dataqual(fov)  ! QC word for MSU
      ENDDO  ! fov

!     Set Mqc(bit0) if any channels are missing
      DO fov = 1, Mnfovs
!        IF ( (msu1c_dataqual(fov).AND.chanbits).NE.0 )
         IF ( IAND(msu1c_dataqual(fov),chanbits).NE.0 )
     &         Mqc(fov,Line) = IBSET(Mqc(fov,Line),bits(0))
      ENDDO  ! fov


! 2.  Check flags
!     ----- -----

! 2.1 Flags for the whole scanline

!     data quality indicator
      badline = BTEST( Mqualind(Line), bits(31) ) !do not use for products
     &    .OR.  BTEST( Mqualind(Line), bits(27) ) !no earth location

!     scanline quality indicator
      badline = badline
     &    .OR.  BTEST( Mscanqual(Line), bits(23) )!bad time field
     &    .OR.  BTEST( Mscanqual(Line), bits(22) )!bad time field
     &    .OR.  BTEST( Mscanqual(Line), bits(20) )!sequence repeats
     &    .OR.  BTEST( Mscanqual(Line), bits(19) )!TIP synch error
     &    .OR.  BTEST( Mscanqual(Line), bits(18) )!TIP frame sequence error
     &    .OR.  BTEST( Mscanqual(Line), bits(17) )!TIP status error
     &    .OR.  BTEST( Mscanqual(Line), bits(16) )!TIP parity error
     &    .OR.  BTEST( Mscanqual(Line), bits(04) )!earth location unreasonable

!     For bad scanline, flag each fov
      IF ( badline ) THEN
         DO fov = 1, Mnfovs
            Mqc(fov,Line) = IBSET ( Mqc(fov,Line), bits(0) )
         ENDDO  ! fov
      ENDIF


      RETURN
      END
