!+ Read & interpret MSU level 1C header within ATOVPP
!
      SUBROUTINE PPMINH ( M1Cunit, SatId,
     &                    StartYearM, StartDayM, StartTimeM,
     &                    EndYearM,   EndDayM,   EndTimeM )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppminh.F 25 2006-02-03 15:43:53Z frna $
!
! Purpose:
!   Open input dataset and read level 1C MSU header record.
!   Extract any required information.
!
! Method:
! 1.  Open file & read header
! 2.  Check format version number and data type
!     MSU level 1C header has already been read into common /msu1chd/
!     Check that format of 1C include files (msu1c.h) is compatible with the
!     code in this subroutine. Check that format of 1C file is compatible with
!     this subroutine, and that data type from the header is 6 for MSU.
!     Subroutine calls ErrorReport to print a warning if there is a problem.
! 3.  Print out header info
! 4.  Extract required information from MSU 1C header
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      10/12/96  SatId added, Ht and Period deleted
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of MSU data 
#include "ppmstore.h"
!  MSU level 1C header/data common blocks 
#include "msu1c.h"
!
! Local parameters:
      INTEGER       M1Cvnnb       ! version number of 1C format compatible
         PARAMETER (M1Cvnnb=180)  ! with the code in this subroutine
!
!   Scalar arguments with intent(In):
      INTEGER       M1Cunit       ! logical unit no. for MSU 1C input
!
!   Scalar arguments with intent(Out):
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      ! start/end date/time of MSU data:
      INTEGER       StartYearM, EndYearM
      INTEGER       StartDayM,  EndDayM
      INTEGER       StartTimeM, EndTimeM
!
! Local scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       M1Crecord     ! record number for MSU level 1C file
      INTEGER       year          ! year, e.g. 1995
      INTEGER       yrday         ! day of the year (1-366)
      INTEGER       month         ! month (start/end of data)
      INTEGER       day           ! day of month (start/end of data)
      INTEGER       hours         ! hours (start/end of data)
      INTEGER       minutes       ! minutes (start/end of data)
      REAL          seconds       ! seconds (start/end of data)
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVPP error routine
      External ConvDay              ! convert day of year to month/day
      External IOM1C                ! I/O routine for level 1C MSU data
!
!- End of header


! 1.  Open file & read header
!     ---- ---- - ---- ------

! 1.1 Open 1C file
      CALL IOM1C ("OPEN/READ", M1Cunit, M1Crecord )   ! 1C file

! 1.2 Read 1C header
      M1Crecord = 1   ! Headers start on record 1.
      CALL IOM1C ("READ HEADER", M1Cunit, M1Crecord )


! 2.  Check format version number and data type
!     ----- ------ ------- ------ --- ---- ----

! 2.1 Is the 1C include file compatible with this source code ?
!     -- --- -- ------- ---- ---------- ---- ---- ------ ----

      IF ( M1Cvnnb .NE. msu_1cvnnb ) THEN
         errStatus = -21  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Code expects 1C format "",I6,"" but msu1c.h is "",I6)")
     &        M1Cvnnb, msu_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPMINH")
      ENDIF

! 2.2 Is the 1C data compatible ?
!     -- --- -- ---- ----------
!     "msu1C.H" contains its own version number "msu_h_l1cversnb"
      IF ( msu1c_h_versnb .NE. msu_1cvnnb ) THEN
         errStatus = -22  ! warning
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Format of level 1C MSU file is "",I8,"", not "",I8)")
     &        msu1c_h_versnb,msu_1cvnnb
         CALL ErrorReport ( errStatus, errMessage, "PPMINH")
      ENDIF

! 2.3 Is the 1C data MSU ?
!     -- --- -- ---- ---
!     "msu1c_h_instrument" from 1C header should be 6 for MSU
      IF ( msu1c_h_instrument .NE. 6 ) THEN
         ErrStatus = 23  ! fatal
         errMessage = " "
         WRITE ( errMessage,
     &   "("" Data type in 1C MSU file is "",I8,""; not 6 for MSU"")")
     &        msu1c_h_instrument
         CALL ErrorReport ( errStatus, errMessage, "PPMINH")
      ENDIF


! 3.  Print out header info
!     ----- --- ------ ----

      WRITE(*,*) "#######"
      WRITE(*,*) "# MSU #"
      WRITE(*,*) "#######"
      WRITE(*,*)
      WRITE(*,"("" reading level 1C file, format version "",3I6)")
     &            msu1c_h_versnb, msu1c_h_versyr, msu1c_h_versdy
      WRITE(*,"(I4,"" header records in file."")") msu1c_h_hdrcnt
      WRITE(*,"(""       Satellite id is "",I4)") msu1c_h_satid

!     calculate start and end dates/times from header
!     ..start
      year = msu1c_h_startdatayr
      yrday = msu1c_h_startdatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(msu1c_h_startdatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   Start date/time  "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     ..end
      year = msu1c_h_enddatayr
      yrday = msu1c_h_enddatady
      CALL ConvDay (year, yrday, month, day )
      seconds = REAL(msu1c_h_enddatatime) * 0.001    ! original in millisecs
      hours = INT(seconds) / 3600
      seconds = seconds - REAL(hours*3600)
      minutes = INT(seconds) / 60
      seconds = seconds - REAL(minutes*60)
      WRITE(*,"(""   End date/time    "",2(I2.2,""/""),I4,2X,
     &          I2.2,"":"",I2.2,"":"",F4.1)")
     &          day,month,year, hours,minutes,seconds

!     number of scanlines
      WRITE(*,"(I6,"" scanlines, "",I6,"" missing."")")
     &          msu1c_h_scnlin, msu1c_h_misscnlin
      WRITE(*,*)


! 4.  Extract required information from MSU 1C header
!     ------- -------- ----------- ---- --- -- ------

      MSatId = msu1c_h_satid
      SatId = MSatId
      StartDayM = msu1c_h_startdatady
      StartYearM = msu1c_h_startdatayr
      StartTimeM = msu1c_h_startdatatime
      EndDayM = msu1c_h_enddatady
      EndYearM = msu1c_h_enddatayr
      EndTimeM = msu1c_h_enddatatime
      Msatht = REAL(msu1c_h_satht)*0.1      !nominal sat height (km)
      Msatperiod = REAL(msu1c_h_period)/60. !nominal orbit period (minutes)
      Mhdrcnt = msu1c_h_hdrcnt       !count of header records in 1C file
      Mscnlin = msu1c_h_scnlin       !count of scanlines in 1C file


      RETURN
      END
