!+ Create level 1D records from stores, and write out to 1D files
!
      SUBROUTINE PPOUT ( TOVS, GHIRS, GAMSA, GAMSB, GIASI, GATMS, GCRIS,
     &              GHIRAS,
     &              H1Dunit, A1Dunit, B1Dunit, I1Dunit,
     &              atms1Dunit, cris1Dunit, hiras1Dunit,
     &              HLastLine, ALastLine, BLastLine, ILastLine,
     &              atmsLastLine, crisLastLine, hirasLastLine )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppout.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Create level 1D records from stores, and write out to 1D files.
!
! Method:
! 1.  Write HIRS (TOVS or ATOVS)
! 2.  Write AMSU-A
! 3.  Write AMSU-B/MHS
! 4.  Write IASI
! 5.  Write ATMS
! 6.  Write CRIS
! 7.  Write HIRAS
!
!   PPOUT may overwrite the last record from the previous block, if the
!   same scanline has been processed within this block.  This is because
!   the last scanline in a block is at a disadvantage in the pre-processing,
!   e.g. when applying a horizontal filter. It is preferable to overwrite
!   it with the same scanline from the next block.  Similarly, the first
!   scanline from the current block may not be written, if it was already
!   processed as an "interior" line from the previous block.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      26/01/05  Add IASI. (N.C.Atkinson)
!  1.2      13/02/07  Bug fix in setting Iprev_recd. NCA
!  1.3      02/09/10  Add ATMS and CRIS
!  1.4      07/02/19  Add HIRAS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  block of HIRS data 
#include "pphstore.h"
!  block of AMSU-A data 
#include "ppastore.h"
!  block of AMSU-B data 
#include "ppbstore.h"
!  block of MSU data 
#include "ppmstore.h"
!  IASI 1d format 
#include "iasi1d.h"
!  block of IASI data 
#include "ppistore.h"
!  IASI fixed data and user options 
#include "ppifdata.h"
!  CRIS 1d format 
#include "cris1d.h"
!  block of CRIS data 
#include "ppcrisstore.h"
!  CRIS fixed data and user options 
#include "ppcrisfdata.h"
!  HIRAS 1d format 
#include "hiras1d.h"
!  block of HIRAS data 
#include "pphirasstore.h"
!  HIRAS fixed data and user options 
#include "pphirasfdata.h"
!  block of ATMS data 
#include "ppatmsstore.h"
!
!   Scalar arguments with intent(In):
      LOGICAL       TOVS          ! TRUE if instrument is TOVS, else FALSE
      LOGICAL       GHIRS         ! .TRUE. to output on HIRS grid
      LOGICAL       GAMSA         ! .TRUE. to output on AMSU-A grid
      LOGICAL       GAMSB         ! .TRUE. to output on AMSU-B grid
      LOGICAL       GIASI         ! .TRUE. to output on IASI grid
      LOGICAL       GATMS         ! .TRUE. to output on ATMS grid
      LOGICAL       GCRIS         ! .TRUE. to output on CRIS grid
      LOGICAL       GHIRAS        ! .TRUE. to output on HIRAS grid
!
      INTEGER       H1Dunit       ! logical unit no. for HIRS 1D output
      INTEGER       A1Dunit       ! logical unit no. for AMSU-A 1D output
      INTEGER       B1Dunit       ! logical unit no. for AMSU-B 1D output
      INTEGER       I1Dunit       ! logical unit no. for IASI 1D output
      INTEGER       atms1Dunit    ! logical unit no. for ATMS 1D output
      INTEGER       cris1Dunit    ! logical unit no. for CrIS 1D output
      INTEGER       hiras1Dunit   ! logical unit no. for HIRAS 1D output
!
      INTEGER       HLastLine     ! number of last HIRS line in store
      INTEGER       ALastLine     ! number of last AMSU-A line in store
      INTEGER       BLastLine     ! number of last AMSU-B line in store
      INTEGER       ILastLine     ! number of last IASI line in store
      INTEGER       atmsLastLine  ! number of last ATMS line in store
      INTEGER       crisLastLine  ! number of last CrIS line in store
      INTEGER       hirasLastLine ! number of last HIRAS line in store
!
! Local scalars:
      INTEGER       line          ! line number within data store
!
      INTEGER       H1Drecord   !record number for HIRS level 1D file
      INTEGER       Hprev_recd  !number of last HIRS record written previously
      INTEGER       A1Drecord   !record number for AMSU-A level 1D file
      INTEGER       Aprev_recd  !number of last AMSU-A record written previously
      INTEGER       B1Drecord   !record number for AMSU-B level 1D file
      INTEGER       Bprev_recd  !number of last AMSU-B record written previously
      INTEGER       I1Drecord   !record number for IASI level 1D file
      INTEGER       Iprev_recd  !number of last IASI record written previously
      INTEGER       Irec        !IASI/CrIS 1d record (1 to 4)
      INTEGER       atms1Drecord   !record number for ATMS level 1D file
      INTEGER       atmsprev_recd  !number of last ATMS record written previously
      INTEGER       cris1Drecord   !record number for CRIS level 1D file
      INTEGER       crisprev_recd  !number of last CRIS record written previously
      INTEGER       hiras1Drecord   !record number for HIRAS level 1D file
      INTEGER       hirasprev_recd  !number of last HIRAS record written previously
!
! Function & Subroutine calls:
      External PPHOUTD          ! create scanline record from common/pphstore/
      External IOH1D            ! write HIRS/ATOVS 1D scanline
      External PPHOUTDM         ! create scanline record from common/pphstore/
      External IOH1DM           ! write HIRS/TOVS 1D scanline
      External PPAOUTD          ! create scanline record from common/ppastore/
      External IOA1D            ! write AMSU-A 1D scanline
      External PPBOUTD          ! create scanline record from common/ppbstore/
      External IOB1D            ! write AMSU-B 1D scanline
      External PPIOUTD          ! create scanline record from common/ppistore/
      External IOI1D            ! write IASI 1D scanline
      External PPATOUTD         ! create scanline record from common/ppatmsstore/
      External IOAT1D           ! write ATMS 1D scanline
      External PPCOUTD          ! create scanline record from common/ppcrisstore/
      External IOC1D            ! write CRIS 1D scanline
      External PPHIRASOUTD      ! create scanline record from common/pphirasstore/
      External IOHIRAS1D        ! write HIRAS 1D scanline
!
      DATA  Hprev_recd, Aprev_recd, Bprev_recd, Iprev_recd / 4*-999 /
      SAVE  Hprev_recd, Aprev_recd, Bprev_recd, Iprev_recd ! for next call
      DATA  atmsprev_recd, crisprev_recd, hirasprev_recd / 3*-999 /
      SAVE  atmsprev_recd, crisprev_recd, hirasprev_recd ! for next call
      INTEGER I
!
!- End of header


! 1.  Write HIRS
!     ----- ----

      IF (GHIRS) THEN      ! output on HIRS grid required

!        loop through lines in store
         DO line = 1, HLastLine

            H1Drecord = Hrecdno(line)
!           Allowed to overwrite last line from previous call (see above):
            IF ( H1Drecord.NE.MDI .AND. H1Drecord.GE.Hprev_recd ) THEN

!              Separate routines for TOVS/ATOVS
               IF ( TOVS ) THEN

! 1.1             Create 1D data record from store
                  CALL PPHOUTDM ( line )

! 1.2             Write out 1D data record
                  CALL IOH1DM ("WRITE DATA", H1Dunit, H1Drecord)

               ELSE              ! ATOVS

! 1.3             Create 1D data record from store
                  CALL PPHOUTD ( line )

! 1.4             Write out 1D data record
                  CALL IOH1D ("WRITE DATA", H1Dunit, H1Drecord)

               ENDIF  ! TOVS

            ENDIF

         ENDDO  ! line
         Hprev_recd = H1Drecord  ! save for next call

      ENDIF   ! (GHIRS)


! 2.  Write AMSU-A
!     ----- ------

      IF (GAMSA) THEN     ! output on AMSU-A grid required

!        loop through lines in store
         DO line = 1, ALastLine

            A1Drecord = Arecdno(line)
!           Allowed to overwrite last line from previous call (see above):
            IF ( A1Drecord.NE.MDI .AND. A1Drecord.GE.Aprev_recd ) THEN

! 2.1          Create 1D data record from store
               CALL PPAOUTD ( line )

! 2.2          Write out 1D data record
               CALL IOA1D ("WRITE DATA", A1Dunit, A1Drecord)

            ENDIF

         ENDDO  ! line
         Aprev_recd = A1Drecord  ! save for next call

      ENDIF   ! (GAMSA)


! 3.  Write AMSU-B
!     ----- ------

      IF (GAMSB) THEN     ! output on AMSU-B grid required

!        loop through lines in store
         DO line = 1, BLastLine

            B1Drecord = Brecdno(line)
!           Allowed to overwrite last line from previous call (see above):
            IF ( B1Drecord.NE.MDI .AND. B1Drecord.GE.Bprev_recd ) THEN

! 3.1          Create 1D data record from store
               CALL PPBOUTD ( line )

! 3.2          Write out 1D data record
               CALL IOB1D ("WRITE DATA", B1Dunit, B1Drecord)

            ENDIF

         ENDDO  ! line
         Bprev_recd = B1Drecord  ! save for next call

      ENDIF   ! (GAMSB)

! 4.  Write IASI
!     ----- ----

      IF (GIASI) THEN     ! output on IASI grid required

!        loop through lines in store
         DO line = 1, ILastLine

!           Each 1c record has "nrec_IASI" 1d records (ppifdata.h)
!           Both also have a header record.

            DO Irec=1,nrec_IASI

              IF ( Irecdno(line).NE.MDI ) THEN
                IF (nrec_IASI .EQ. 4) THEN
                  I1Drecord = nrec_IASI*(Irecdno(line)-2) + 1 + Irec
                ELSE
                  I1Drecord = Irecdno(line)
                ENDIF

!           Allowed to overwrite last scan from previous call (see above):
                IF ( Irecdno(line).GE.Iprev_recd ) THEN

! 4.1          Read in the 1D record which has already been filled with IASI
!              instrument data

                   CALL IOI1D ("READ DATA", I1Dunit, I1Drecord)

! 4.2          Create 1D data record from store
                   CALL PPIOUTD ( line, Irec )

! 4.3          Write out 1D data record
                   CALL IOI1D ("WRITE DATA", I1Dunit, I1Drecord)

                ENDIF
                
              ENDIF  !Irecdno(line).NE.MDI

            ENDDO  ! Irec

         ENDDO  ! line

         Iprev_recd = Irecdno(ILastLine)  ! save 1c record for next call

      ENDIF   ! (GIASI)

! 5.  Write ATMS
!     ----- ----

      IF (GATMS) THEN     ! output on ATMS grid required

!        loop through lines in store
         DO line = 1, atmsLastLine

            atms1Drecord = atmsrecdno(line)
!           Allowed to overwrite last line from previous call (see above):
            IF ( atms1Drecord.NE.MDI .AND. atms1Drecord.GE.Aprev_recd ) 
     &        THEN

! 2.1          Create 1D data record from store
               CALL PPATOUTD ( line )

! 2.2          Write out 1D data record
               CALL IOAT1D ("WRITE DATA", atms1Dunit, atms1Drecord)

            ENDIF

         ENDDO  ! line
         atmsprev_recd = atms1Drecord  ! save for next call

      ENDIF   ! (GATMS)

! 4.  Write CRIS
!     ----- ----

      IF (GCRIS) THEN     ! output on CRIS grid required

!        loop through lines in store
         DO line = 1, crisLastLine

!           Each 1c record has "nrec_cris" 1d records (ppcrisfdata.h)
!           Both also have a header record.

            DO Irec=1,nrec_cris

              IF ( Irecdno(line).NE.MDI ) THEN
                IF (nrec_cris .GT. 1) THEN
                  cris1Drecord = nrec_cris*(crisrecdno(line)-2) + 1 
     &                + Irec
                ELSE
                  cris1Drecord = crisrecdno(line)
                ENDIF

!           Allowed to overwrite last scan from previous call (see above):
                IF ( crisrecdno(line).GE.crisprev_recd ) THEN

! 4.1          Read in the 1D record which has already been filled with CRIS
!              instrument data

                   CALL IOC1D ("READ DATA", cris1Dunit, cris1Drecord)

! 4.2          Create 1D data record from store
                   CALL PPCOUTD ( line, Irec )

! 4.3          Write out 1D data record
                   CALL IOC1D ("WRITE DATA", cris1Dunit, cris1Drecord)

                ENDIF
                
              ENDIF  !crisrecdno(line).NE.MDI

            ENDDO  ! Irec

         ENDDO  ! line

         crisprev_recd = crisrecdno(crisLastLine)  ! save 1c record for next call

      ENDIF   ! (GCRIS)

! 4.  Write HIRAS
!     ----- ----

      IF (GHIRAS) THEN     ! output on HIRAS grid required

!        loop through lines in store
         DO line = 1, hirasLastLine

!           Each 1c record has "nrec_hiras" 1d records (pphirasfdata.h)
!           Both also have a header record.

            DO Irec=1,nrec_hiras

              IF ( Irecdno(line).NE.MDI ) THEN
                IF (nrec_hiras .GT. 1) THEN
                  hiras1Drecord = nrec_hiras*(hirasrecdno(line)-2) + 1 
     &                + Irec
                ELSE
                  hiras1Drecord = hirasrecdno(line)
                ENDIF

!           Allowed to overwrite last scan from previous call (see above):
                IF ( hirasrecdno(line).GE.hirasprev_recd ) THEN

! 4.1          Read in the 1D record which has already been filled with HIRAS
!              instrument data

                   CALL IOHIRAS1D ("READ DATA", hiras1Dunit,
     &                hiras1Drecord)

! 4.2          Create 1D data record from store
                   CALL PPHIRASOUTD ( line, Irec )

! 4.3          Write out 1D data record
                   CALL IOHIRAS1D ("WRITE DATA", hiras1Dunit, 
     &                hiras1Drecord)

                ENDIF
                
              ENDIF  !hirasrecdno(line).NE.MDI

            ENDDO  ! Irec

         ENDDO  ! line

         hirasprev_recd = hirasrecdno(hirasLastLine)  ! save 1c record for next call

      ENDIF   ! (GHIRAS)


      RETURN
      END
