!+ Initialisation routine for ATOVPP
!
      SUBROUTINE PPSETUP ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &                     LATMS, LCRIS, LHIRAS,
     &                     GHIRS, GAMSA, GAMSB, GIASI, GATMS, GCRIS,
     &                     GHIRAS,
     &                     H1Cunit, A1Cunit, B1Cunit, M1Cunit, I1Cunit,
     &                     IPCunit, atms1Cunit, cris1Cunit, hiras1Cunit,
     &                     H1Dunit, A1Dunit, B1Dunit, I1Dunit,
     &                     atms1Dunit, cris1Dunit, hiras1Dunit,
     &                     LSunit, HTunit,
     &                     Year0, Day0, Time0, NBlocks, TOVS )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: ppsetup.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Read user requirements & options, level 1C headers, and set up program
!   variables such as mapping Look-Up-Tables and dates/times for processing.
!
! Method:
! 1.  Define bit numbers
! 2.  Read user requirements
! 3.  Read level 1C headers
! 4.  Read fixed data files
! 5.  Determine start/end dates/times for the processing, & number of blocks
! 6.  Set up mapping Look-Up-Tables
! 7.  Open files for level 1D output
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      10/12/96  SatId added
!  1.2      16/03/05  Scatfunit added (NCA)
!  1.3      12/08/05  Update for Metop, including addition of IASI (NCA)
!  1.4      13/08/10  Add ATMS and CrIS
!  1.5      30/01/14  Assign npc_IASI before call to PPIFDF
!  1.6      22/12/14  Add sat id range 706-708 for TOVS (NOAA-6&7 and T-N)
!  1.7      07/02/19  Add HIRAS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!  IASI 1d format
#include "iasi1d.h"
!  Fixed data for IASI 
#include "ppifdata.h"
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LIASI         ! .TRUE. to process IASI
      LOGICAL       LPCIASI       ! .TRUE. to process IASIPC
      LOGICAL       LATMS         ! .TRUE. to process ATMS
      LOGICAL       LCRIS         ! .TRUE. to process CRIS
      LOGICAL       LHIRAS        ! .TRUE. to process HIRAS
!
      LOGICAL       GHIRS         ! .TRUE. to output on HIRS grid
      LOGICAL       GAMSA         ! .TRUE. to output on AMSU-A grid
      LOGICAL       GAMSB         ! .TRUE. to output on AMSU-B grid
      LOGICAL       GIASI         ! .TRUE. to output on IASI grid
      LOGICAL       GATMS         ! .TRUE. to output on ATMS grid
      LOGICAL       GCRIS         ! .TRUE. to output on CRIS grid
      LOGICAL       GHIRAS        ! .TRUE. to output on HIRAS grid
!
      INTEGER       H1Cunit       ! unit number for HIRS level 1C file
      INTEGER       A1Cunit       ! unit number for AMSU-A level 1C file
      INTEGER       B1Cunit       ! unit number for AMSU-B level 1C file
      INTEGER       M1Cunit       ! unit number for MSU level 1C file
      INTEGER       I1Cunit       ! unit number for IASI level 1C file
      INTEGER       IPCunit       ! unit number for IASI PC file
      INTEGER       atms1Cunit    ! unit number for ATMS level 1C file
      INTEGER       cris1Cunit    ! unit number for CRIS level 1C file
      INTEGER       hiras1Cunit   ! unit number for HIRAS level 1C file
!
      INTEGER       H1Dunit       ! unit number for HIRS level 1D file
      INTEGER       A1Dunit       ! unit number for AMSU-A level 1D file
      INTEGER       B1Dunit       ! unit number for AMSU-B level 1D file
      INTEGER       I1Dunit       ! unit number for IASI level 1D file
      INTEGER       atms1Dunit    ! unit number for ATMS level 1D file
      INTEGER       cris1Dunit    ! unit number for CRIS level 1D file
      INTEGER       hiras1Dunit   ! unit number for HIRAS level 1D file
!
      INTEGER       HTunit        ! unit no. for heights dataset
      INTEGER       LSunit        ! unit no. for land/sea bitmap
!
!   Scalar arguments with intent(out)
      ! start date/time for the processing:
      INTEGER       Year0         !   year
      INTEGER       Day0          !   day
      INTEGER       Time0         !   time
!
      INTEGER       NBlocks       ! number of blocks dT to be processed
      LOGICAL       TOVS          ! TRUE if instrument is TOVS, else FALSE
!
! Local Scalars:
      INTEGER       Hfunit        ! unit number for HIRS-specific data
      INTEGER       Afunit        ! unit number for AMSU-A-specific data
      INTEGER       Bfunit        ! unit number for AMSU-B-specific data
      INTEGER       Mfunit        ! unit number for MSU-specific data
      INTEGER       Ifunit        ! unit number for IASI-specific data
      INTEGER       Ieigunit1     ! unit number for IASI eigenvectors (encoding)
      INTEGER       Ieigunit2     ! unit number for IASI eigenvectors (decoding)
      INTEGER       Inoiseunit    ! unit number for IASI noise
      INTEGER       Imtfunit      ! unit number for IASI MTF
      INTEGER       Lfunit        ! unit number for LUT corrections
      INTEGER       Scatfunit     ! unit number for NWC-SAF scattering files
      INTEGER       atmsfunit     ! unit number for ATMS
      INTEGER       crisfunit     ! unit number for CRIS
      INTEGER       hirasfunit    ! unit number for HIRAS
      INTEGER       atmsignoreunit ! unit number for ATMS ignore file
      INTEGER       criseigunit1  ! unit number for CRIS eigenvectors (encoding)
      INTEGER       criseigunit2  ! unit number for CRIS eigenvectors (decoding)
      INTEGER       hiraseigunit1  ! unit number for HIRAS eigenvectors (encoding)
      INTEGER       hiraseigunit2  ! unit number for HIRAS eigenvectors (decoding)
      INTEGER       H1Drecord     ! record number for HIRS level 1D file
      INTEGER       A1Drecord     ! record number for AMSU-A level 1D file
      INTEGER       B1Drecord     ! record number for AMSU-B level 1D file
      INTEGER       I1Drecord     ! record number for IASI level 1D file
      INTEGER       atms1Drecord  ! record number for ATMS level 1D file
      INTEGER       cris1Drecord  ! record number for CRIS level 1D file
      INTEGER       hiras1Drecord  ! record number for CRIS level 1D file
      INTEGER       SatId         ! satellite id (e.g. 14 for NOAA-14)
      INTEGER       WMOSatId      ! WMO satellite id (e.g. 205 for NOAA-14)
!
      CHARACTER*80  errMessage    ! error message
      INTEGER       errStatus     ! error status
      INTEGER       i, j          ! loop indices
!
      ! start/end date/time of HIRS data:
      INTEGER       startYearH, endYearH
      INTEGER       startDayH,  endDayH
      INTEGER       startTimeH, endTimeH
      ! start/end date/time of AMSU-A data:
      INTEGER       startYearA, endYearA
      INTEGER       startDayA,  endDayA
      INTEGER       startTimeA, endTimeA
      ! start/end date/time of AMSU-B data:
      INTEGER       startYearB, endYearB
      INTEGER       startDayB,  endDayB
      INTEGER       startTimeB, endTimeB
      ! start/end date/time of MSU data:
      INTEGER       startYearM, endYearM
      INTEGER       startDayM,  endDayM
      INTEGER       startTimeM, endTimeM
      ! start/end date/time of IASI data:
      INTEGER       startYearI, endYearI
      INTEGER       startDayI,  endDayI
      INTEGER       startTimeI, endTimeI
      ! start/end date/time of ATMS data:
      INTEGER       startYearATMS, endYearATMS
      INTEGER       startDayATMS,  endDayATMS
      INTEGER       startTimeATMS, endTimeATMS
      ! start/end date/time of CRIS data:
      INTEGER       startYearCRIS, endYearCRIS
      INTEGER       startDayCRIS,  endDayCRIS
      INTEGER       startTimeCRIS, endTimeCRIS
      ! start/end date/time of HIRAS data:
      INTEGER       startYearHIRAS, endYearHIRAS
      INTEGER       startDayHIRAS,  endDayHIRAS
      INTEGER       startTimeHIRAS, endTimeHIRAS
!
! Function & Subroutine calls:
      External PPUSER               ! read user requirements
      External PPHINH               ! read HIRS 1C header
      External PPAINH               ! read AMSU-A 1C header
      External PPBINH               ! read AMSU-B 1C header
      External PPMINH               ! read MSU 1C header
      External PPIINH               ! read IASI 1C header
      External PPIPCINH             ! read IASI PC header
      External PPATMSINH            ! read ATMS 1C header
      External PPCRISINH            ! read CRIS 1C header
      External PPHIRASINH           ! read HIRAS 1C header
      External PPHFDF               ! read HIRS fixed data file
      External PPAFDF               ! read AMSU-A fixed data file
      External PPBFDF               ! read AMSU-B fixed data file
      External PPMFDF               ! read MSU fixed data file
      External PPIFDF               ! read IASI fixed data file
      External PPATMSFDF            ! read ATMS fixed data file
      External PPCRISFDF            ! read CRIS fixed data file
      External PPHIRASFDF           ! read HIRAS fixed data file
      External PPLFDF               ! read LUT (mapping) fixed data file
      External PPLUT                ! set up mapping Look-Up-Table
      External PPTIME               ! start/end dates/times for the processing
      External IOH1D                ! I/O routine for level 1D HIRS/ATOVS data
      External IOH1DM               ! I/O routine for level 1D HIRS/TOVS data
      External IOA1D                ! I/O routine for level 1D AMSU-A data
      External IOB1D                ! I/O routine for level 1D AMSU-B data
      External IOI1D                ! I/O routine for level 1D IASI data
      External IOAT1D               ! I/O routine for level 1D ATMS data
      External IOC1D                ! I/O routine for level 1D CRIS data
      External IOHIRAS1D            ! I/O routine for level 1D HIRAS data
      External ErrorReport          ! ATOVPP error routine
!- End of header

! 1.  Define bit numbers   (in "bits.h")
!     ------ --- -------

! Convention used in 1B & 1C files is that an INTEGER*4 word has bits
! numbered 0-31, with bit 0 being the least significant bit.
! Some platforms take bit 31 as the LSB. Here we explicitly define
! the order of bits that we use, to keep the code portable.

      i = 1

      IF ( BTEST(i,0) ) THEN      !  bit 0 is LSB

         DO j = 0, 31
            bits(j)  =  j
         ENDDO  ! j

      ELSEIF( BTEST(i,31) ) THEN      !  bit 31 is LSB

         DO j = 0, 31
            bits(j)  =  31 - j
         ENDDO  ! j

      ELSE                            !  something wrong

         errStatus = 31
         errMessage =
     & " Bit problem: BTEST(1,0) and BTEST(1,31) are both FALSE"
         CALL ErrorReport ( errStatus, errMessage, "PPSETUP")

      ENDIF


! 2.  Read user requirements
!     ---- ---- ------------

      CALL PPUSER ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &              LATMS, LCRIS, LHIRAS,
     &              GHIRS, GAMSA, GAMSB, GIASI, GATMS, GCRIS, GHIRAS,
     &              H1Cunit, A1Cunit, B1Cunit, M1Cunit, I1Cunit,IPCunit,
     &              atms1Cunit, cris1Cunit, hiras1Cunit,
     &              H1Dunit, A1Dunit, B1Dunit, I1Dunit,
     &              atms1Dunit, cris1Dunit, hiras1Dunit,
     &              Hfunit, Afunit, Bfunit, Mfunit,
     &              Lfunit, LSunit, HTunit, Scatfunit,
     &              Ifunit, Ieigunit1, Ieigunit2, Inoiseunit, Imtfunit,
     &              atmsfunit, crisfunit, hirasfunit, atmsignoreunit,
     &              criseigunit1, criseigunit2,
     &              hiraseigunit1, hiraseigunit2 )


! 3.  Read level 1C headers
!     ---- ----- -- -------

! 3.1 HIRS
      IF ( LHIRS ) CALL PPHINH ( H1Cunit, SatId, WMOSatId,
     &                           startYearH, startDayH, startTimeH,
     &                           endYearH,   endDayH,   endTimeH )

! 3.2 AMSU-A
      IF ( LAMSA ) CALL PPAINH ( A1Cunit, SatId, WMOSatId,
     &                           startYearA, startDayA, startTimeA,
     &                           endYearA,   endDayA,   endTimeA )

! 3.3 AMSU-B
      IF ( LAMSB ) CALL PPBINH ( B1Cunit, SatId, WMOSatId,
     &                           startYearB, startDayB, startTimeB,
     &                           endYearB,   endDayB,   endTimeB )

! 3.4 MSU
      IF ( LMSU ) CALL PPMINH ( M1Cunit, SatId,    !WMOSatId not needed for MSU
     &                           startYearM, startDayM, startTimeM,
     &                           endYearM,   endDayM,   endTimeM )

! 3.5 IASI
      IF ( LIASI ) CALL PPIINH ( I1Cunit, SatId, WMOSatId,
     &                           startYearI, startDayI, startTimeI,
     &                           endYearI,   endDayI,   endTimeI )

! 3.6 IASIPC
      IF ( LPCIASI ) CALL PPIPCINH ( IPCunit, SatId, WMOSatId,
     &                           startYearI, startDayI, startTimeI,
     &                           endYearI,   endDayI,   endTimeI )

! 3.7 ATMS
      IF ( LATMS ) CALL PPATMSINH ( atms1Cunit, SatId, WMOSatId,
     &                       startYearATMS, startDayATMS, startTimeATMS,
     &                       endYearATMS,   endDayATMS,   endTimeATMS )

! 3.8 CRIS
      IF ( LCRIS ) CALL PPCRISINH ( cris1Cunit, SatId, WMOSatId,
     &                       startYearCRIS, startDayCRIS, startTimeCRIS,
     &                       endYearCRIS,   endDayCRIS,   endTimeCRIS )

! 3.81 HIRAS
      IF ( LHIRAS ) CALL PPHIRASINH ( hiras1Cunit, SatId, WMOSatId,
     &                   startYearHIRAS, startDayHIRAS, startTimeHIRAS,
     &                   endYearHIRAS,   endDayHIRAS,   endTimeHIRAS )

! 3.9 Is this a TOVS instrument ? (NOAA-14 and earlier)
      TOVS = SatId .LE. 14. AND. 
     &      (WMOSatId .EQ. 0 .OR.              !Accept old 1c format zero fill
     &      (WMOSatId .GE. 200 .AND. WMOSatId.LE.205).OR.
     &      (WMOSatId .GE. 706 .AND. WMOSatId.LE.708))


! 4.  Read fixed data files    (all except for topography files)
!     ---- ----- ---- -----

! 4.1 HIRS
      IF ( LHIRS ) CALL PPHFDF ( Hfunit, TOVS )

! 4.2 AMSU-A
      IF ( LAMSA ) CALL PPAFDF ( Afunit )

! 4.3 AMSU-B
      IF ( LAMSB ) CALL PPBFDF ( Bfunit )

! 4.4 MSU
      IF ( LMSU ) CALL PPMFDF ( Mfunit )

! 4.5 IASI. Only need the decode eigenvectors if working with PC format.
      npc_IASI = 0         !Use value from IASI.fdf
      IF ( LIASI ) CALL PPIFDF (Ifunit, Ieigunit1, Ieigunit2,
     &                          Inoiseunit, Imtfunit )
      IF ( LPCIASI ) CALL PPIFDF (Ifunit, 0, Ieigunit2, 0, 0)

! 4.6 LUT corrections
      CALL PPLFDF ( Lfunit, SatId )

! 4.7 Scattering data for AMSU-B and ATMS
      IF ( LAMSB .OR. LATMS ) CALL read_nwcsaf_scat_params (Scatfunit)

! 4.8 ATMS
      IF ( LATMS ) CALL PPATMSFDF ( atmsfunit, atmsignoreunit )

! 4.9 CRIS
      IF ( LCRIS) CALL PPCRISFDF (crisfunit, criseigunit1, criseigunit2)

! 4.91 HIRAS
      IF ( LHIRAS) CALL PPHIRASFDF (hirasfunit, hiraseigunit1,
     & hiraseigunit2)


! 5.  Determine start/end dates/times for the processing, & number of blocks
!     --------- --------- ----------- --- --- ----------- - ------ -- ------

      CALL PPTIME ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     & LATMS, LCRIS, LHIRAS,
     & startYearH, startDayH, startTimeH, endYearH, endDayH, endTimeH,
     & startYearA, startDayA, startTimeA, endYearA, endDayA, endTimeA,
     & startYearB, startDayB, startTimeB, endYearB, endDayB, endTimeB,
     & startYearM, startDayM, startTimeM, endYearM, endDayM, endTimeM,
     & StartYearI, StartDayI, StartTimeI, EndYearI, EndDayI, EndTimeI,
     & startYearATMS, startDayATMS, startTimeATMS, endYearATMS, 
     &    endDayATMS, endTimeATMS,
     & startYearCRIS, startDayCRIS, startTimeCRIS, endYearCRIS, 
     &    endDayCRIS, endTimeCRIS,
     & startYearHIRAS, startDayHIRAS, startTimeHIRAS, endYearHIRAS, 
     &    endDayHIRAS, endTimeHIRAS,
     & Year0, Day0, Time0, NBlocks )


! 6.  Set up mapping Look-Up-Tables (not ATMS/CrIS)
!     --- -- ------- --------------

      IF (.NOT. GCRIS .AND. .NOT. GATMS) THEN
        CALL PPLUT ( H1Cunit, A1Cunit, B1Cunit, M1Cunit, I1Cunit,
     &             IPCunit, LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     &             GHIRS, GAMSA, GAMSB, GIASI )
      ENDIF

      IF (LIASI.OR.LPCIASI) THEN
        IF (thinmode_IASI .EQ. 1) THEN
          CALL PPLUT_IASI( default_IASI )        !IASI in single detector mode
        ENDIF
      ELSE
        FirstFOV_IASI = 1         !Initialise these if IASI is not present
        LastFOV_IASI = 1
      ENDIF


! 7.  Open files for level 1D output
!     ---- ----- --- ----- -- ------

! 7.1 HIRS
      IF ( GHIRS ) THEN
         IF ( TOVS ) THEN
            CALL IOH1DM ("OPEN/WRITE", H1Dunit, H1Drecord )
         ELSE
            CALL IOH1D ("OPEN/WRITE", H1Dunit, H1Drecord )
         ENDIF
      ENDIF  ! GHIRS

! 7.2 AMSU-A
      IF ( GAMSA ) CALL IOA1D ("OPEN/WRITE", A1Dunit, A1Drecord )

! 7.3 AMSU-B
      IF ( GAMSB ) CALL IOB1D ("OPEN/WRITE", B1Dunit, B1Drecord )

! 7.4 IASI
      IF ( GIASI ) CALL IOI1D ("OPEN/WRITE", I1Dunit, I1Drecord )

! 7.5 ATMS
      IF ( GATMS ) CALL IOAT1D ("OPEN/WRITE", atms1Dunit, atms1Drecord )

! 7.6 CRIS
      IF ( GCRIS ) CALL IOC1D ("OPEN/WRITE", cris1Dunit, cris1Drecord )

! 7.7 HIRAS
      IF ( GHIRAS ) CALL IOHIRAS1D ("OPEN/WRITE", hiras1Dunit,
     & hiras1Drecord )

      RETURN
      END
