!+ Determine start/end dates/times for the processing
!
      SUBROUTINE PPTIME ( LHIRS, LAMSA, LAMSB, LMSU, LIASI, LPCIASI,
     & LATMS, LCRIS, LHIRAS,
     & StartYearH, StartDayH, StartTimeH, EndYearH, EndDayH, EndTimeH,
     & StartYearA, StartDayA, StartTimeA, EndYearA, EndDayA, EndTimeA,
     & StartYearB, StartDayB, StartTimeB, EndYearB, EndDayB, EndTimeB,
     & StartYearM, StartDayM, StartTimeM, EndYearM, EndDayM, EndTimeM,
     & StartYearI, StartDayI, StartTimeI, EndYearI, EndDayI, EndTimeI,
     & startYearATMS, startDayATMS, startTimeATMS, endYearATMS, 
     &    endDayATMS, endTimeATMS,
     & startYearCRIS, startDayCRIS, startTimeCRIS, endYearCRIS, 
     &    endDayCRIS, endTimeCRIS,
     & startYearHIRAS, startDayHIRAS, startTimeHIRAS, endYearHIRAS, 
     &    endDayHIRAS, endTimeHIRAS,
     & Year0, Day0, Time0, NBlocks )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: pptime.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!   Determine start/end dates/times for the processing
!
! Method:
! 1.  First guess: set start & end times to that of HIRS
! 2.  If we are using AMSU-A, and that is earlier/later, then use that instead
! 3.  If we are using AMSU-B, and that is earlier/later, then use that instead
! 4.  If we are using MSU, and that is earlier/later, then use that instead
! 5.  If we are using IASI, and that is earlier/later, then use that instead
! 6.  How many blocks size dT do we have ?
!   Choose the start date/time for processing to be the earliest of the
!   start dates/times of the instruments which are to be processed.
!   Similarly, take the end date/time to be the latest.
!   Also, check that instrument start times are similar!
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
!
!  1.0      27/11/96  Original code. (R.J.Renshaw)
!  1.1      12/08/98  Check instrument start times are similar. (R.J.Renshaw)
!                     (large differences give problems in lutmap)
!  1.2      12/08/05  Add IASI (N.C.Atkinson)
!  1.3      07/09/10  Add ATMS and CrIS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  general parameters for ATOVPP 
#include "ppparms.h"
!
! Subroutine arguments:
!   Scalar arguments with intent(in)
      LOGICAL       LHIRS         ! .TRUE. to process HIRS
      LOGICAL       LAMSA         ! .TRUE. to process AMSU-A
      LOGICAL       LAMSB         ! .TRUE. to process AMSU-B
      LOGICAL       LMSU          ! .TRUE. to process MSU
      LOGICAL       LIASI         ! .TRUE. to process IASI
      LOGICAL       LPCIASI       ! .TRUE. to process PCIASI
      LOGICAL       LATMS         ! .TRUE. to process ATMS
      LOGICAL       LCRIS         ! .TRUE. to process CRIS
      LOGICAL       LHIRAS        ! .TRUE. to process HIRAS
!
      ! start/end date/time of HIRS data:
      INTEGER       StartYearH, EndYearH
      INTEGER       StartDayH,  EndDayH
      INTEGER       StartTimeH, EndTimeH
      ! start/end date/time of AMSU-A data:
      INTEGER       StartYearA, EndYearA
      INTEGER       StartDayA,  EndDayA
      INTEGER       StartTimeA, EndTimeA
      ! start/end date/time of AMSU-B data:
      INTEGER       StartYearB, EndYearB
      INTEGER       StartDayB,  EndDayB
      INTEGER       StartTimeB, EndTimeB
      ! start/end date/time of MSU data:
      INTEGER       StartYearM, EndYearM
      INTEGER       StartDayM,  EndDayM
      INTEGER       StartTimeM, EndTimeM
      ! start/end date/time of IASI data:
      INTEGER       StartYearI, EndYearI
      INTEGER       StartDayI,  EndDayI
      INTEGER       StartTimeI, EndTimeI
      ! start/end date/time of ATMS data:
      INTEGER       StartYearATMS, EndYearATMS
      INTEGER       StartDayATMS,  EndDayATMS
      INTEGER       StartTimeATMS, EndTimeATMS
      ! start/end date/time of CRIS data:
      INTEGER       StartYearCRIS, EndYearCRIS
      INTEGER       StartDayCRIS,  EndDayCRIS
      INTEGER       StartTimeCRIS, EndTimeCRIS
      ! start/end date/time of HIRAS data:
      INTEGER       StartYearHIRAS, EndYearHIRAS
      INTEGER       StartDayHIRAS,  EndDayHIRAS
      INTEGER       StartTimeHIRAS, EndTimeHIRAS
!
!   Scalar arguments with intent(out)
      ! start date/time for the processing:
      INTEGER       Year0         !   year
      INTEGER       Day0          !   day
      INTEGER       Time0         !   time
!
      INTEGER       NBlocks       ! number of blocks dT to be processed
!
! Local Parameters:
      INTEGER       maxdtime      ! maximum difference in start times allowed
      PARAMETER   ( maxdtime = 300000 ) ! 300s in milliseconds
!
! Local Scalars:
      CHARACTER*80  errMessage    ! error message
      INTEGER       errTime       ! error status from TimeSub
      INTEGER       dtime         ! time difference (milliseconds) from TimeSub
!
      ! end date/time for the processing:
      INTEGER       year9         !   year
      INTEGER       day9          !   day
      INTEGER       time9         !   time
!
! Function & Subroutine calls:
      External TimeSub            ! Subtract two dates/times
      External ErrorReport        ! ATOVPP error routine
!
!- End of header


! 1.  First guess: set start & end times to that of HIRS or CRIS
!     ----- -----  --- ----- - --- ----- -- ---- -- ---- -- ----

      Year0 = MDI
      IF (LHIRS) THEN
         Year0 = StartYearH
         Day0  = StartDayH
         Time0 = StartTimeH
         year9 = EndYearH
         day9  = EndDayH
         time9 = EndTimeH
      ELSE IF (LCRIS) THEN
         Year0 = StartYearCRIS
         Day0  = StartDayCRIS
         Time0 = StartTimeCRIS
         year9 = EndYearCRIS
         day9  = EndDayCRIS
         time9 = EndTimeCRIS
      ELSE IF (LHIRAS) THEN
         Year0 = StartYearHIRAS
         Day0  = StartDayHIRAS
         Time0 = StartTimeHIRAS
         year9 = EndYearHIRAS
         day9  = EndDayHIRAS
         time9 = EndTimeHIRAS
      ENDIF


! 2.  If we are using AMSU-A, and that is earlier/later, then use that instead
!     -- -- --- ----- ------  --- ---- -- -------------  ---- --- ---- -------

      IF (LAMSA) THEN

         IF (Year0.EQ.MDI) THEN
!           set date/time to start/end time for AMSU-A if not using HIRS
            Year0 = StartYearA
            Day0  = StartDayA
            Time0 = StartTimeA
            year9 = EndYearA
            day9  = EndDayA
            time9 = EndTimeA
         ELSE
!           compare with AMSU-A start time
            CALL TimeSub (StartYearA,StartDayA,StartTimeA,
     &                    Year0,Day0,Time0,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                         "for HIRS and AMSU-A start dates/times"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF ( ABS(dtime) .GT. maxdtime ) THEN  ! difference too large
               errMessage =
     &      "AMSU-A or HIRS start time different from other instruments"
               CALL ErrorReport(2,errMessage,"PPTIME")
            ENDIF
            IF (dtime.LT.0) THEN  ! .. AMSU-A is earlier.
               Year0 = StartYearA
               Day0  = StartDayA
               Time0 = StartTimeA
            ENDIF
!           compare with AMSU-A end time
            CALL TimeSub (EndYearA,EndDayA,EndTimeA,
     &                    year9,day9,time9,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                           "for HIRS and AMSU-A end dates/times"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF (dtime.GT.0) THEN  ! .. AMSU-A is later.
               year9 = EndYearA
               day9  = EndDayA
               time9 = EndTimeA
            ENDIF
         ENDIF

      ENDIF   ! (LAMSA)


! 3.  If we are using AMSU-B, and that is earlier/later, then use that instead
!     -- -- --- ----- ------  --- ---- -- -------------  ---- --- ---- -------

      IF ( LAMSB ) THEN

         IF (Year0.EQ.MDI) THEN
!           set date/time to start/end time for AMSU-B if not using HIRS/AMSU-A
            Year0 = StartYearB
            Day0  = StartDayB
            Time0 = StartTimeB
            year9 = EndYearB
            day9  = EndDayB
            time9 = EndTimeB
         ELSE
!           compare with AMSU-B start time
            CALL TimeSub (StartYearB,StartDayB,StartTimeB,
     &                    Year0,Day0,Time0,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                                 "for AMSU-B start date/time"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF ( ABS(dtime) .GT. maxdtime ) THEN  ! difference too large
               errMessage =
     &      "AMSU-B start time different from HIRS or AMSU-A"
               CALL ErrorReport(3,errMessage,"PPTIME")
            ENDIF
            IF (dtime.LT.0) THEN  ! .. AMSU-B is earlier.
               Year0 = StartYearB
               Day0  = StartDayB
               Time0 = StartTimeB
            ENDIF
!           compare with AMSU-B end time
            CALL TimeSub (EndYearB,EndDayB,EndTimeB,
     &                    year9,day9,time9,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                                 "for AMSU-B end date/time"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF (dtime.GT.0) THEN  ! .. AMSU-B is later.
               year9 = EndYearB
               day9  = EndDayB
               time9 = EndTimeB
            ENDIF
         ENDIF

      ENDIF   ! (LAMSB)


! 4.  If we are using MSU, and that is earlier/later, then use that instead
!     -- -- --- ----- ---  --- ---- -- -------------  ---- --- ---- -------

      IF ( LMSU ) THEN

         IF (Year0.EQ.MDI) THEN
!           set date/time to start/end time for MSU if not using HIRS
            Year0 = StartYearM
            Day0  = StartDayM
            Time0 = StartTimeM
            year9 = EndYearM
            day9  = EndDayM
            time9 = EndTimeM
         ELSE
!           compare with MSU start time
            CALL TimeSub (StartYearM,StartDayM,StartTimeM,
     &                    Year0,Day0,Time0,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                                 "for MSU start date/time"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF ( ABS(dtime) .GT. maxdtime ) THEN  ! difference too large
               errMessage = "MSU start time different from HIRS"
               CALL ErrorReport(4,errMessage,"PPTIME")
            ENDIF
            IF (dtime.LT.0) THEN  ! .. MSU is earlier.
               Year0 = StartYearM
               Day0  = StartDayM
               Time0 = StartTimeM
            ENDIF
!           compare with MSU end time
            CALL TimeSub (EndYearM,EndDayM,EndTimeM,
     &                    year9,day9,time9,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                                 "for MSU end date/time"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF (dtime.GT.0) THEN  ! .. MSU is later.
               year9 = EndYearM
               day9  = EndDayM
               time9 = EndTimeM
            ENDIF
         ENDIF

      ENDIF   ! (LMSU)

! 5.  If we are using IASI, and that is earlier/later, then use that instead
!     -- -- --- ----- ----  --- ---- -- -------------  ---- --- ---- -------

      IF ( LIASI .OR. LPCIASI ) THEN

         IF (Year0.EQ.MDI) THEN
!           set date/time to start/end time for IASI if not using HIRS
            Year0 = StartYearI
            Day0  = StartDayI
            Time0 = StartTimeI
            year9 = EndYearI
            day9  = EndDayI
            time9 = EndTimeI
         ELSE
!           compare with IASI start time
            CALL TimeSub (StartYearI,StartDayI,StartTimeI,
     &                    Year0,Day0,Time0,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                                 "for IASI start date/time"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF ( ABS(dtime) .GT. maxdtime ) THEN  ! difference too large
               errMessage = "IASI start time different from AMSU"
               CALL ErrorReport(4,errMessage,"PPTIME")
            ENDIF
            IF (dtime.LT.0) THEN  ! .. IASI is earlier.
               Year0 = StartYearI
               Day0  = StartDayI
               Time0 = StartTimeI
            ENDIF
!           compare with IASI end time
            CALL TimeSub (EndYearI,EndDayI,EndTimeI,
     &                    year9,day9,time9,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                                 "for IASI end date/time"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF (dtime.GT.0) THEN  ! .. IASI is later.
               year9 = EndYearI
               day9  = EndDayI
               time9 = EndTimeI
            ENDIF
         ENDIF

      ENDIF   ! (LIASI)


! 6.  If we are using ATMS, and that is earlier/later, then use that instead
!     -- -- --- ----- ----  --- ---- -- -------------  ---- --- ---- -------

      IF ( LATMS ) THEN

         IF (Year0.EQ.MDI) THEN
!           set date/time to start/end time for ATMS
            Year0 = StartYearATMS
            Day0  = StartDayATMS
            Time0 = StartTimeATMS
            year9 = EndYearATMS
            day9  = EndDayATMS
            time9 = EndTimeATMS
         ELSE
!           compare with ATMS start time
            CALL TimeSub (StartYearATMS,StartDayATMS,StartTimeATMS,
     &                    Year0,Day0,Time0,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                                 "for ATMS start date/time"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF ( ABS(dtime) .GT. maxdtime ) THEN  ! difference too large
               errMessage = "ATMS start time different from ATMS"
               CALL ErrorReport(4,errMessage,"PPTIME")
            ENDIF
            IF (dtime.LT.0) THEN  ! .. ATMS is earlier.
               Year0 = StartYearATMS
               Day0  = StartDayATMS
               Time0 = StartTimeATMS
            ENDIF
!           compare with ATMS end time
            CALL TimeSub (EndYearATMS,EndDayATMS,EndTimeATMS,
     &                    year9,day9,time9,
     &                    dtime,errTime)
!                         fail when errTime indicates a fatal error (>0)
                          errMessage = " TimeSub reports error " //
     &                                 "for ATMS end date/time"
                          CALL ErrorReport(errTime,errMessage,"PPTIME")
            IF (dtime.GT.0) THEN  ! .. ATMS is later.
               year9 = EndYearATMS
               day9  = EndDayATMS
               time9 = EndTimeATMS
            ENDIF
         ENDIF

      ENDIF   ! (LATMS)


! 7.  How many blocks size dT do we have ?
!     --- ---- ------ ---- -- -- -- ----

      CALL TimeSub ( year9, day9, time9,
     &               Year0, Day0, Time0,
     &               dtime, errTime)
!                    fail when errTime indicates a fatal error (>0)
                     errMessage = " TimeSub reports error " //
     &                            "for start/end dates/times"
                     CALL ErrorReport(errTime,errMessage,"PPTIME")
      NBlocks = dtime/dT + 1


      RETURN
      END
