!+  return arrays of elevation & surface type for given latitudes/longitudes
!
      SUBROUTINE SURFELEV ( Unitno_ls, Unitno_ht,
     &                    Lats, Lons, Nspots, BoxSize,
     &                    SurfType, Elevations, LandFrac )
      IMPLICIT NONE
c-----------------------------------------
! Name: $Id: surfelev.F 401 2014-07-08 13:47:31Z frna $
!
! Purpose:
!    Return surface type (land/sea/mixed) and height for an array of
!    lat/lon points, using ITPP export package topography datasets
!    (1/6th degree x 1/6th degree lat/lon, heights in 100s of feet)
!    Height is for the nearest gridpoint, and is set to zero for sea spots.
!    Surface type is found by examining all points within (approx)
!    "BoxSize" kilometres of the spot. The spot is classified as land
!    (or sea) only if every point in this box is land (or sea). Otherwise,
!    its surface type is defined as mixed.
!
!    If either lat or lon are out of range then values of -999999 are
!    returned for both SurfType and Elevations.
!
! Method:
!
! 0) Read global land/sea bitmask into memory
!    Open heights file
! 1) Define size of box in terms of number of gridpoints in the x & y directions
! 2) Loop through the array of spots. For each spot, search within the box
!    for surface type, and also return the height of the nearest point.
!    To save performing too many READs, some of the height dataset records
!    are stored in array "store". This is a rolling archive, and is updated
!    as necessary. Its size "mx_store" can be tuned according to available
!    memory. Minimum value is 1.
!
!    This code is derived from s/r HRTOPO - part of University of Wisconsin
!    (Madison) International TOVS Processing Package (13/05/92 version)
!
!    Note that this routine uses the "BYTE" type declaration, which
!    isn't in the FORTRAN 77 standard.  BYTE is signed, i.e. 255 would
!    appear as -1. To get around this, dataset contains height in 100s
!    of feet, minus 128.  Adding 128 back again gives true elevation.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/07/95  Derived from s/r HRTOPO - part of University of Wisconsin
!         (Madison) International TOVS Processing Package (13/05/92 version)
!  1.1      12/12/96  Bug corrected affecting longitude wraparound at 0 deg.
!  1.2      17/01/97  Land (1) and Mixed (2) swapped around
!  1.3      04/09/98  Bug fix: fix when nhtrec < 0 now correct
!  1.4      06/07/99  More bug fixes, pointed out by R.Schraidt.
!                     a) check longitude within +/-180 degrees
!                     b) occasional error identifying 'mixed' surface type
!                        due to wrong count of points within box
!                     c) problem at 90N/0S boundary (..this is how the
!                        dataset is structured)
!  1.5      27/07/99  1.4 introduced new bug, fixed here:
!                         when BoxSize < 18km all spots returned as sea spots
!  1.6      05/08/99  Another bug: BoxSize incorrect in N-S direction
!                     (only half as big as intended)
!  1.7      24/01/00  Bug fix: gridpoints out by 1 in N-S and E-W directions
!  1.7bis   01/02/00  modification of the increment in the loop on the latitudes
!                     (P. Brunel and T. Labrot)  
!  1.8      12/06/03  Integer limits, increment and variable for "latitude box"
!                     loop (to avoid F77/F90 differences).  (K Whyte/N Atkinson)
!  1.9      09/03/04  Modify for Linux (N Atkinson / A Korpela)
!  1.10     15/09/04  Use constants_preproc.h for fundamental constants - NCA
!  1.11     11/03/05  Use character*1 instead of byte for portability - NCA
!  1.12     15/03/05  Add LandFrac - NCA
!  1.13     16/02/06  Ensure nhtrec is positive - NCA
!  1.14     19/07/11  Fix compilation error for null literal string - NCA
!  1.15     03/07/14  Separate the ICHAR and MOD functions for portability - NCA
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!
!  Record length dividor 
#include "reclopt.h"
!  Fundamental and physical constants 
#include "constants_preproc.h"
!
! Subroutine arguments
!   Scalar arguments with intent(in):
      INTEGER       Nspots        ! number of lats/lons to return values for
      INTEGER       Unitno_ht     ! unit no. for heights dataset "maptopog.dat"
      INTEGER       Unitno_ls     ! unit no. for land/sea bitmap "mapbitls.dat"
      INTEGER       BoxSize       ! size of box (in kilometres)
!
!   Array arguments with intent(in):
      REAL          Lats(*)       ! array of latitudes (degrees, South negative)
      REAL          Lons(*)       ! array of longitudes (degrees, West negative)
!
!   Array arguments with intent(out):
      INTEGER       Elevations(*) ! array of elevations (metres)
      INTEGER       SurfType(*)   ! surface types (0=sea, 1=mixed, 2=land)
      INTEGER       LandFrac(*)   ! Land fraction (percent)
!
! Local parameters:
      INTEGER       mx_store      ! max number of height records to store
        PARAMETER ( mx_store = 50 )  ! need be no higher than 188 maximum
      INTEGER       lenb          ! length of bitmap records (in I4 words)
        PARAMETER ( lenb = 70 )
      INTEGER       lenm          ! number of data points in bitmap records
        PARAMETER ( lenm = lenb - 2 )
      INTEGER       lenh          ! length of height records (in I1 words)
        PARAMETER ( lenh = 4320 )
      INTEGER       nrec          ! number of bitmap records
        PARAMETER ( nrec = 1080 )
      INTEGER       mlat          ! half the number of latitude gridpoints
        PARAMETER ( mlat = nrec / 2 )
      INTEGER       nwrd          ! nwrd+1 is bit no. within mbit for 1/6th degE
        PARAMETER ( nwrd = 2160 )
!
! Local Scalars:
      INTEGER       first_store   ! position in "store" of lowest stored record
      INTEGER       last_store    ! position in "store" of highest stored record
      INTEGER       i             ! general purpose loop variable
      INTEGER       ilat          ! latitude co-ordinate (SURFELEV grid)
      INTEGER       ilon          ! longitude co-ordinate (SURFELEV grid)
      INTEGER       irec          ! record number
      INTEGER       ispot         ! loop over Nspots
      INTEGER       ispot1        ! first spot to have a valid latitude
      INTEGER       istore        ! point to height record held within "store"
      INTEGER       iwrd          ! word number within lbuf holding height info
      INTEGER       jlat          ! latitude co-ordinate inside box
      INTEGER       jlon          ! longitude co-ordinate inside box
      INTEGER       klon          ! "jlon" adjusted for wraparound near 0dg East
      INTEGER       nhtrec        ! number of records in heights dataset
!     note that "nhtrec" is only valid with the 27/07/95 vn of maptopog onwards
      INTEGER       numb          ! count of number of land points in/from mbit
      INTEGER       sumbox        ! number of SURFELEV gridpoints within box
      INTEGER       sumland       ! count of number of land points within box
      INTEGER       surf_nearest  ! surface type of the centre-point of the box
      INTEGER       xbox          ! half-width in E-W direction of lat/lon box
      INTEGER       ybox          ! half-width in N-S direction of lat/lon box
      LOGICAL       first_call    ! .TRUE. when true
      INTEGER       elev          ! elevation of spot (in 100s of feet)
      REAL          latbox        ! latitude of point within box
      REAL          maxlat        ! maximum latitude for box
      REAL          minlat        ! minimum latitude for box
      INTEGER       jlat_ind      ! (latitude of point within box)*6
      REAL          xlat          ! absolute latitude of 1st spot
      REAL          xmult         ! multiplcation factor to adjust box size
                                  ! (as degrees longitude) according to latitude
!
! Local arrays:
      INTEGER       index_store(mx_store)! list of height records stored
      INTEGER*4     mbit(lenb)    ! bitmap record as read from Unitno_ls
                                  ! Last word is a count of no. of land points
      INTEGER*4     mbuf(lenm,nrec)! stored bitmap records "mbit"
      INTEGER       numbs(nrec)   ! number of land points in dataset up to
                                  ! and including points in mbuf(*,*)
      CHARACTER*1   store(lenh,mx_store)! program store of height records
      CHARACTER*1   lbuf(lenh)    ! record from heights dataset
!
! Function & Subroutine calls:
      External LBIT               ! examine a bit in an array
      INTEGER LBIT                ! LBIT returns an integer value (0 or 1)
      INTEGER ios                 ! Error return value
      CHARACTER*80  errMessage    ! error message
      CHARACTER*80  endianType    ! LITTLE_ENDIAN / BIG_ENDIAN
      CHARACTER*80  endianFlag    ! compiler flag used for endian conversion
      LOGICAL       needSwap      ! if LITTLE ENDIAN and no flag found, we need
                                  ! byte swapping to BIG_ENDIAN
!
      DATA first_call / .TRUE. /
      DATA first_store / 1 /, last_store / 1 /
      DATA index_store / mx_store * -1 /       ! i.e. no record numbers stored

      SAVE first_call, mbuf, numbs, nhtrec,
     &     store, index_store, first_store, last_store
!
!
! Function & Subroutine calls:
      External ErrorReport          ! ATOVIN/ATOVPP error routine
!
!- End of header


! 0.  Initialisation  (first call only)
!     --------------

      IF (first_call) THEN

! 0.1    open, read & close land/sea bit-map file

         OPEN ( UNIT=Unitno_ls, STATUS="OLD", FORM="UNFORMATTED",
     &          ACCESS="DIRECT", RECL=lenb*4/rldiv, IOSTAT=ios )

!        If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( ios.NE.0 )
     &   OPEN ( UNIT=Unitno_ls, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &          ACCESS="DIRECT", RECL=lenb*4/rldiv, IOSTAT=ios )

         if ( ios .ne. 0 ) then
           write(errMessage,901) ios,Unitno_ls
           CALL ErrorReport ( ABS(ios), errMessage, "SURFELEV")
         endif

         CALL getenv('FORM_TYPE',endianType)
         CALL getenv('BIGENDIANFLAG',endianFlag)
         needSwap = (endianType(1:6).EQ.'LITTLE' .AND. 
     &      endianFlag(1:1).EQ.' ')
         numb=0
         DO irec=1,nrec
            READ(Unitno_ls,rec=irec,ERR=900, IOSTAT=ios) mbit
            if (needSwap) CALL wordSwap( mbit,lenb )
!           count of number of land points read to date
            numbs(irec)=numb
!            IF (irec.LT.7) write(*,*) irec, mbit(lenb)
!           add number of land points read this record
            numb=numb+mbit(lenb)
!           store land/sea mask from this record
            DO i=1,lenm
               mbuf(i,irec)=mbit(i)
            ENDDO
         ENDDO

         CLOSE (Unitno_ls)

! 0.2    Open heights file, and get number of records

         OPEN ( UNIT=Unitno_ht, STATUS="OLD", FORM="UNFORMATTED",
     &          ACCESS="DIRECT", RECL=lenh/rldiv,iostat=ios )

!        If that didn't work, use STATUS="UNKNOWN" (Linux ifc compiler)
         IF ( ios.NE.0 )
     &   OPEN ( UNIT=Unitno_ht, STATUS="UNKNOWN", FORM="UNFORMATTED",
     &          ACCESS="DIRECT", RECL=lenh/rldiv,iostat=ios )

         if ( ios .ne. 0 ) then
           write(errMessage,902) ios,Unitno_ht
           CALL ErrorReport ( ABS(ios), errMessage, "SURFELEV")
         endif
         READ ( UNIT=Unitno_ht, rec=1,ERR=900, IOSTAT=ios) lbuf
         nhtrec = ICHAR(lbuf(1))
         IF (nhtrec.LT.0) nhtrec = nhtrec + 256    !ensure positive

         first_call = .FALSE.

      ENDIF


! 1.  Define size of box to look through, in terms of gridpoints
!     ------ ---- -- --- -- ---- -------- -- ----- -- ----------

! 1.1 latitude
      ybox = INT ( REAL(BoxSize*90*6*2) / (PI*rEarth) )
      ybox = MAX ( ybox, 1 )

! 1.2 longitude
!     take latitude of first point to be representative of them all
!     - potential problem near the Poles, where cos(latitude) is small
      ispot = 0
      ispot1 = 0 ! first spot to have a valid latitude
      DO WHILE ( ispot.LT.Nspots .AND. ispot1.EQ.0 )
         ispot = ispot + 1
         IF ( ABS(Lats(ispot)).LE.90. ) ispot1 = ispot
      ENDDO  ! WHILE
      IF ( ispot1.GT.0 ) THEN
         xlat  = MIN(ABS(Lats(ispot1)),85.)   ! & so set xlat to 85.0 there.
         xmult = COS ( xlat*PI / 180. )  ! (adjustment for latitude)
         xbox  = INT ( REAL(BoxSize*90*6*2) / (PI*rEarth*xmult) )
         xbox  = MAX ( xbox, 1 )
      ENDIF


! 2.  Loop through arrays of latitude & longitude
!     ---- ------- ------ -- -------- - ---------

      DO ispot = 1, Nspots
       IF ( ABS(Lats(ispot)).LE.90.
     & .AND. ABS(Lons(ispot)).LE.180. ) THEN  ! check lat/lon are ok

! 2.1    convert latitude and longitude to SURFELEV co-ordinates
!        (lat goes from 0-90N, 0-90S; lon goes from 0-360W !)
         ilat = NINT(Lats(ispot)*6.) + 1
!         IF (Lats(ispot).LT.-0.01) ilat = ABS(ilat) + mlat
         IF (ilat.LT.0) ilat = mlat - ilat
         ilat = MIN ( ilat, nrec-1)   ! 90S becomes 89.83S
         ilon = -NINT(Lons(ispot)*6.) + 1
         IF (ilon.LT.0) ilon = ilon + nwrd

! 2.2    Look through all gridpoints within "BoxSize" for surface type
!          - if we have both land and sea, then this is a "mixed" spot

!        This has to be modified for spots near a boundary of the grid.
         sumland = 0   !  number of land gridpoints within box
         sumbox  = 0   ! total number of gridpoints within box


!         DO latbox = MAX(Lats(ispot)-ybox/6.,-90.) ,
!     &               MIN(Lats(ispot)+ybox/6.,90.) , 1./6.

         minlat = MAX(Lats(ispot)-ybox/6.,-90.)
         maxlat = MIN(Lats(ispot)+ybox/6.,90.)
         
         if (ispot.EQ.99999) WRITE(6,*)       ! WEIRD !!!
         DO jlat_ind = 0, INT((maxlat-minlat)*6.0)

            ! find nearest dataset gridpoint
            latbox = jlat_ind/6.0 + minlat
            jlat = NINT(latbox*6.) + 1
            IF (jlat.LT.0) jlat = mlat - jlat
            jlat = MIN ( jlat, nrec-1)   ! 90S becomes 89.83S

!           load "mbit" with appropriate land/sea record
            DO i = 1, lenm
               mbit(i) = mbuf(i,jlat+1)
            ENDDO ! i

!           extract surface type from "mbuf" for each longitude in the box
            DO jlon = ilon-xbox, ilon+xbox
!              "wraparound" for Greenwich Meridian
!               12/12/96 bug corrected: klon = MOD(jlon-1,nwrd)+1
               IF (jlon.LT.0) THEN
                 klon = jlon + nwrd
               ELSEIF (jlon.GE.nwrd) THEN
                 klon = jlon - nwrd
               ELSE
                 klon = jlon
               ENDIF
               sumland = sumland + LBIT (klon+1, mbit)
            ENDDO ! jlon
            sumbox = sumbox + 2*xbox + 1    ! count number of points in box

         ENDDO ! latbox

!        Determine what surface type we have overall
         IF (sumland.EQ.0) THEN
            SurfType(ispot) = 0     ! sea
         ELSEIF (sumland.LT.sumbox) THEN
            SurfType(ispot) = 1     ! mixed
         ELSE
            SurfType(ispot) = 2     ! land
         ENDIF
         LandFrac(ispot) = (sumland*100.0)/sumbox

! 2.3    Pick out surface type of the nearest gridpoint (ilat,ilon)
!        - load "mbit" with appropriate land/sea record (may also be used later)
         DO i = 1, lenm
            mbit(i) = mbuf(i,ilat+1)
         ENDDO ! i
         surf_nearest = LBIT (ilon+1, mbit)

! 2.4    Where appropriate, extract the elevation from the nearest gridpoint
         IF (surf_nearest.EQ.0) THEN     ! sea spot => zero elevation

            Elevations(ispot) = 0

         ELSE                            ! land or mixed spot

! 2.4.1     Count up how many land spots in dataset up to & including this one
            numb = numbs(ilat+1) +1    ! the final "+1" because first "height"
            IF (ilon.NE.0) THEN        ! is actually the number of records
               DO i = 1, ilon
                  numb = numb + LBIT (i, mbit)
               ENDDO
            ENDIF

! 2.4.2     Calculate which record & word contain height for this spot
            irec = (numb-1) / lenh
            iwrd = numb - (lenh*irec)
            irec = irec + 1
! Check that record exists!
            IF (irec.GT.nhtrec) THEN
               WRITE(*,*) " nhtrec, numb, irec, ilat, ilon = ",
     &         nhtrec, numb, irec, ilat, ilon
!               irec = nhtrec       ! enough height records for all the
!               iwrd = lenh         ! land points in the land/sea mask!
            ENDIF

!           If this isn't a record already in memory, then read it

            IF  (   irec.GE.index_store(first_store)
     &        .AND. irec.LE.index_store(last_store)  ) THEN
! 2.4.3     record already in memory  - set istore to point to it
               istore = first_store + irec - index_store(first_store)
               istore = MOD ( istore-1, mx_store ) + 1   ! circular database

            ELSEIF ( irec .EQ. index_store(first_store)-1 ) THEN
! 2.4.4     record is adjacent to those in memory  - read in new record irec
!              First, move the "pointers" down by one..
               first_store = first_store - 1
               IF ( first_store.EQ.0 ) first_store = mx_store
               IF ( first_store.EQ.last_store ) THEN
                  last_store = last_store - 1  ! move last_store down
                  IF ( last_store.EQ.0 ) last_store = mx_store
               ENDIF
!              ..then read in the new record
               istore = first_store
               index_store(istore) = irec
               READ(Unitno_ht,rec=irec,ERR=900, IOSTAT=ios) lbuf
               DO i = 1, lenh
                  store(i,istore) = lbuf(i)
               ENDDO ! i

            ELSEIF ( irec .EQ. index_store(last_store)+1 ) THEN
! 2.4.5     record is adjacent to those in memory  - read in new record irec
!              First, move the "pointers" up by one..
               last_store = last_store + 1
               IF ( last_store.GT.mx_store ) last_store = 1
               IF ( last_store.EQ.first_store ) THEN
                  first_store = first_store + 1  ! move first_store up
                  IF ( first_store.GT.mx_store ) first_store = 1
               ENDIF
!              ..then read in the new record
               istore = last_store
               index_store(istore) = irec
               READ(Unitno_ht,rec=irec,ERR=900, IOSTAT=ios) lbuf
               DO i = 1, lenh
                  store(i,istore) = lbuf(i)
               ENDDO ! i

            ELSE
! 2.4.6     record is not adjacent to any in memory  - start afresh
!              First, set the pointers
               first_store = 1
               last_store  = 1
!              ..then read in the new record
               istore = 1
               index_store(istore) = irec
               READ(Unitno_ht,rec=irec,ERR=900, IOSTAT=ios) lbuf
               DO i = 1, lenh
                  store(i,istore) = lbuf(i)
               ENDDO ! i

            ENDIF  ! (find record in memory)

! 2.4.7     Extract the height, converting from 100s of feet to metres
!           Zero elevation has value 128 in "store"
!            elev = store(iwrd,istore) + 128 !This was for store declared "byte"
!            elev = MOD(ICHAR(store(iwrd,istore))+128,256)
            i = ICHAR(store(iwrd,istore))
            elev = MOD(i+128,256)
            Elevations(ispot) = NINT(30.48*REAL(elev))

         ENDIF   ! (surf_nearest.EQ.0)

       ELSE   ! lat/lon out of range

         SurfType(ispot)   = -999999
         Elevations(ispot) = -999999

       ENDIF  ! lat/lon ok
      ENDDO   ! ispot

      go to 1000
 900  continue
      write(errMessage,903) ios,Unitno_ht,irec
      CALL ErrorReport ( ABS(ios), errMessage, "SURFELEV")

C      write(*,'(a)')' SURFELEV: Error reading  elevation & surface type'
C      write(*,'(a,I3,a,I3,a,I10)')' SURFELEV: IOS = ',ios,' UNIT = ',
C     &         Unitno_ht,' RECNR = ',irec
C      call exit(2)
 1000 continue
      RETURN
901   format('SURFELEV: opening surface data type file, ios=',i3,
     &  ' unit=',i3)
902   format('SURFELEV: opening elevation data type file, ios=',i3,
     &  ' unit=',i3)
903   format('SURFELEV: when reading elevation/surface data, ios= ',i8,
     &  ' unit=',i3,' # of record=',i8)
      END

!+  return the value of a specified bit from an array
!
      FUNCTION LBIT ( J, Array )
      IMPLICIT NONE
c-----------------------------------------
c
c
!
! Purpose:
!   return the value of "j"th bit from variable "Array"
!   Here, bit "J" is counted within "Array" as follows:
!         Array(1)      Array(2)      Array(3)   ...
!  J =   1,2,..,32     33,...,64     65,...,96   ...
!   with bit 32 being the least significant bit within word Array(1)
!   (contrary to the convention used within AAPP, which counts
!    from 0 to 31, with bit 0 being the LSB).
!
! Method:
!   Divide "J" by 32 to find which the word number within "Array" containing
!   bit "J". Then take the remainder from 32 to find which bit to test,
!   using intrinsic function BTEST.
!
! Current Code Owner: R.J.Renshaw
!
! History:
! Version  Date      Comment
!
!  1.0      27/07/95  Derived from fn LBIT - part of University of Wisconsin
!         (Madison) International TOVS Processing Package (12/06/84 version)
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
! Global variables:
!  bit numbers for intrinsic bit routines (eg BTEST) 
#include "bits.h"
!
! Return value of function
      INTEGER      LBIT           ! 1 if bit is set; otherwise 0
!
! Function arguments
!
!   Scalar arguments:
      INTEGER      J              ! bit number to check
!
!   Array arguments:
      INTEGER*4    Array(*)       ! variable to be examined
!
! Local Scalars:
      INTEGER      jword          ! word no.(-1) within "Array" that holds bit J
      INTEGER      word           ! Array(jword+1)
      INTEGER      jbit           ! bit within "word" that holds bit J

!- End of header

      jword = (J-1)/32
      word  = Array(jword+1)

      jbit  = 32  -  (J - jword*32)

      IF (BTEST(word,bits(jbit))) THEN
         LBIT = 1
      ELSE
         LBIT = 0
      ENDIF

      RETURN
      END
