! Name: $Id$
      subroutine fill_maia(  has_maia,  tavh_b, 
     &                  ravh_b, map,
     &                  local, maia_info, clear_box)

! MAIA= MASQUE Avhrr pour Inversion Atovs
!       -      -          -         -
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! input:
!       maia_box(map_box_psize,map_box_lsize)  -- input computed maia parameters on box
!       has_maia           -- input logical if maia mask has been done
!       map_box_psize                     -- input pixel size of the box
!                               integer
!       map_box_lsize                     -- input line size of the box
!                               integer
!       tavh_b(map_box_psize,map_box_lsize,5)       -- input brightness temperature on box
!                               channels 1, 2 albedo en %
!                               channels 3, 4, 5 Tb (degK) 
!                               real 
!       ravh_b(map_box_psize,map_box_lsize;5)       -- input radiance on box
!                               channels 3, 4, 5  (mw/...) 
!                               real 
!       map(map_box_psize,map_box_lsize)            -- mapping (0: in the ellipse, 1 outside
!                                2 for missing lines )
!                               integer 
!       local                   -- output definition 
!                               true = local definition (for ICI)
!                               false = "GAC" parameters
!                               logical 
!                               
! output:
!       maia_info(13)    -- avhrr information parameters
!                          real
!
!               1 - pourcentage clear AVHRR in HIRS FOV (*100)
!               2 - surface temperature (K*100)
!               3 - climatological temperature or t2m (K*100)
!               4 - mean AVHRR channel 3 over HIRS FOV (degK*100)
!               5 - mean AVHRR channel 4 over HIRS FOV (degK*100)
!               6 - mean AVHRR channel 5 over HIRS FOV (degK*100)
!               7 - percentage of black body in HIRS FOV (*100)
!               8 - top cloud temperature over HIRS FOV (degK*100)
!               9 - standart deviation corresponding to the top cloud T (degK*100)
!              10 - mean clear AVHRR channel 4 over HIRS FOV (degK*100)
!              11 - mean clear AVHRR channel 5 over HIRS FOV (degK*100)
!              12 - std  AVHRR channel 4 over HIRS FOV (degK*100)
!              13 - std  clear AVHRR channel 4 over HIRS FOV (degK*100)
!
! 
!  clear_box(map_box_psize,map_box_lsize)
!            clear_box(j,i) = -1   ! outside ellipse
!            clear_box(j,i) = 1    ! cloudy inside ellispe
!            clear_box(j,i) = 0    ! clear inside ellipse
!-----------------------------------------------------------------------
! OWNER: L. Lavanant, P. Brunel         CMS. Nov 1999
!
!       version 2.1 on 11/2000  L. Lavanant
!       11/2000 Correction of arguments to function tempsurfm P. Brunel
!       11 June 01 Remove filtrage from external subroutines  T.Labrot
!       17 Aug 06  Change nb_el threshold from 100 to 90 as recommended by 
!                  MSC/JMA   N.Atkinson
!       15 May 17 originaly maia.F renammed to fill_maia maia4 replace maia2 P. Roquet
!       24 August 2017 use the new maia_param type

        use mod_maia_types
	use mod_maia_wrapper
        implicit none
#include "avh2hirs.h"
!
! --- parameters



! ... arguments   INPUT

        logical has_maia


        real tavh_b(map_box_psize,map_box_lsize,5)
        real ravh_b(map_box_psize,map_box_lsize,5)
        integer map(map_box_psize,map_box_lsize)
        logical local   

! ... arguments   OUTPUT
        real maia_info(13) ! maia info on HIRS pixel
        integer clear_box(map_box_psize,map_box_lsize)

! ... local scalars ..
      integer ic,nb_cl,nb_el,lside,i,j
      integer kval
      integer t4max_el                  ! K*100
      real temp4                        ! K*100
      real std,xmin,xmax,xmoy,rmoy,rmax
      real std4_el                      ! K*100
      real std4_cl                      ! K*100
      real t3, t4, t5
      integer c      

! ... local arrays
      real tavh_el(nb_el_max,5),ravh_el(nb_el_max,5)
      real sd33_el(nb_el_max)
      real tavh_cl(nb_el_max,5),ravh_cl(nb_el_max,5) 
      integer el_lut(2,nb_el_max)       ! LUT : tavh_b(el_lut(n)) == tavh_el(n)
      real tmoy_cl(5)                   ! alb: %*100  Tb: K*100  
      real tmoy_el(5)                   ! alb: %*100  Tb: K*100  
      real rmoy_el                      ! Rad: mw/cm2/st/cm-1
      real tsurf(nb_el_max)             ! K
      real tcloudtop(nb_el_max) 
      real t_surf
      real t2m_prev
      integer tm              ! surface type(0=sea 1=mixed 2=land)
      

      
! ... external subroutines ..
      external ppellip,xavg

! ... intrinsic functions ..
      intrinsic cos,int
      
! black body cloud
      real mtcloudtop,  sigmat4
      integer nblackbody
     

! Functions
      real avh_cir_r    ! conversion radiance to temp. (mW/cm2/st/cm-1 , K)

! other local variables
      integer nbp_in_ellipse
      integer nbland_in_ellipse
      integer nbsea_in_ellipse
      integer nbcoast_in_ellipse
!-----------------------------------------------------------------------
! initializations
!-----------------------------------------------------------------------

      do i=1,13
       maia_info(i)=-9999.
      end do
      if ( .not. has_maia )  then
         local = .False.
      endif
      do i=1,map_box_lsize                        ! 38
      do j=1,map_box_psize                        ! 33
         if (map(j,i).eq.1) then 
            clear_box(j,i)=-1           ! outside ellipse
         end if
         if (map(j,i).eq.0) then        ! inside ellipse 
             clear_box(j,i)=1 
         end if
      end do
      end do

!-----------------------------------------------------------------------
! ... PreProcessing of the AVHRR (Box to Ellipse)
! ... nb_el:  number of pixels in the ellipse including the edge of ellipse
! ... tavh_el: ravh_el: charts with only the nb_el data inside the ellipse 
! ... sd33_el:  standard deviation for each pixel in the ellipse computed on a 3*3 box
! ... tavh_el et sd33: in K*100 or Alb*100 
! ... ravh_el still in mw/...
!-----------------------------------------------------------------------
      lside = 3
      call ppellip(map_box_psize, map_box_lsize, tavh_b, ravh_b, map,
     +         lside,    tavh_el, ravh_el, sd33_el, el_lut, nb_el)

      if(nb_el.gt.90) then
!-----------------------------------------------------------------------
! ... inside the ellipse:
! ... mean of radiances + maximum radiance for channel 4
! ... conversion of radiance to temp. of the mean and the max
!-----------------------------------------------------------------------
         do ic=1,2
          call xavg(tavh_el(1,ic),nb_el,tmoy_el(ic),std,xmin,xmax)
         end do

         do ic=3,5
            call xavg(ravh_el(1,ic),nb_el,rmoy_el,std,xmin,rmax)
            tmoy_el(ic) = avh_cir_r(rmoy_el, ic-2) *100.
            if(ic.eq.4) then
               temp4  = avh_cir_r(rmax, ic-2)
            endif
         end do
         t4max_el=int(temp4*100.)
         call xavg(tavh_el(1,4),nb_el,xmoy,std4_el,xmin,xmax)

         ! mean values inside the ellipse
          maia_info(4) = tmoy_el(3)
          maia_info(5) = tmoy_el(4)
          maia_info(6) = tmoy_el(5)
          maia_info(12) = std4_el
   
         if (.not. has_maia ) then
          return
         endif


! compute t2m_prev and tm for hirs pixel  
      	t2m_prev = 0
      	tm = 0 
      	nbp_in_ellipse = 0
      	nbland_in_ellipse = 0
      	nbsea_in_ellipse = 0
      	nbcoast_in_ellipse = 0
      	nb_cl = 0 
      	do i=1,map_box_lsize                        ! 38
      	do j=1,map_box_psize  
          if (map(j,i) .eq. 0 ) then ! inside ellipse 
             t2m_prev = t2m_prev + maia_box(j,i)%temp_2m
             t_surf = t_surf + maia_box(j,i)%surf_temp
             nbp_in_ellipse = nbp_in_ellipse + 1
           ! maia4 surface type is 
           ! surface type(0=sea 1=mixed 2=land) from maia_box (0 sea A mix 2 ,3,4 land/dessert/snow)
             if (maia_box(j,i)%box_atlas_surface_type .eq. 0 ) then
                nbsea_in_ellipse = nbsea_in_ellipse + 1
             else 
               if  (maia_box(j,i)%box_atlas_surface_type .eq. 1 ) then
                 nbcoast_in_ellipse = nbcoast_in_ellipse + 1
               else 
                 nbland_in_ellipse = nbland_in_ellipse + 1
               endif
           endif
           ! cloud mask
           if ( maia_box(j,i)%cloud_mask_conf .eq. 3 ) then ! confidant clear
              nb_cl = nb_cl + 1                
              clear_box(j,i) = 0
              if (nb_cl .gt. nb_el_max ) then
                write(*,*) "WARNING nb_cl > nb_el_max",nb_cl,  nb_el_max
              else  
                do c = 1,5
                  ! keep clear pixels in tavh_cl and ravh_cl
                  tavh_cl(nb_cl,c) = tavh_b(j,i,c)
                  ravh_cl(nb_cl,c) = ravh_b(j,i,c)
                enddo
                tsurf(nb_cl) = maia_box(j,i)%surf_temp
              endif
           endif 
         endif

        end do
        end do

        if  (nbp_in_ellipse .ne. 0 ) then
          t2m_prev  =  t2m_prev / nbp_in_ellipse
        endif

       if ( nbsea_in_ellipse == nbp_in_ellipse ) then
         tm =  0 ! all clear pixels are sea pixels can compute tsurf moy and std
       else
         tm = -1
       endif 
!-----------------------------------------------------------------------
! Cloud top Temperature for the black body clouds
!-----------------------------------------------------------------------
        nblackbody = 0 ! number of opaq cloud
        mtcloudtop = 0
	sigmat4 = 0
 
        do i=1,map_box_lsize                        ! 38
        do j=1,map_box_psize  
          if (map(j,i) .eq. 0 ) then ! inside ellipse 
            if (maia_box(j,i)%cloud_opacity .eq. 1 .and. 
     &          maia_box(j,i)%cloud_top_temp .gt. 0. ) then
              ! cloud opacity
              nblackbody = nblackbody + 1
              tcloudtop(nblackbody) = maia_box(j,i)%cloud_top_temp
             endif
          endif
        end do
        end do     
        if (nblackbody .gt. 0 ) then
          call xavg(tcloudtop,nblackbody,mtcloudtop,sigmat4,xmin,xmax)
          maia_info(8) = mtcloudtop * 100.       ! en 100ieme de K
          maia_info(9) = sigmat4 * 100.          ! en 100ieme de degres
        endif

!-----------------------------------------------------------------------
! for the clear pixels:
!-----------------------------------------------------------------------
        do ic=1,5
          tmoy_cl(ic)=0
        end do
        std4_cl=0
      
        if(nb_cl .gt. 0) then
          kval=nb_cl
          do ic=1,2
             call xavg(tavh_cl(1,ic),kval,tmoy_cl(ic),std,xmin,xmax)
          end do      
          do ic = 3,5
            call xavg(ravh_cl(1,ic),kval,rmoy,std,xmin,xmax)
            tmoy_cl(ic) = avh_cir_r(rmoy, ic-2) *100.
          end do
         call xavg(tavh_cl(1,4),kval,xmoy,std4_cl,xmin,xmax)
          ! mean values for the clear pixels only 
          do ic=4,5
             maia_info(ic+6) = tmoy_cl(ic)
          end do
           ! standard deviation channel 4 for the clear pixels inside the ellipse
           maia_info(13) = std4_cl * 100 
        endif

 !-----------------------------------------------------------------------
! for all the pixels
!-----------------------------------------------------------------------
        ! black body porcent 
        maia_info(7) = nblackbody * 100. / nb_el
        ! clear pourcentage [0,1]
        if ( nb_el .ne. nbp_in_ellipse) then 
           write(*,*) "WARNING incoherent computation in fill_maia " 
         endif
         maia_info(1) = nb_cl*100./nb_el

        ! surface temperature (K*100) (only SST)
         if(tm.eq.0) then 
           if(maia_info(1).ge.5.00) then
             call xavg(tsurf,nb_cl,xmoy,std,xmin,xmax)
             maia_info(2) = xmoy*100.
          endif
        endif 
        ! t2m
         maia_info(3) = t2m_prev *100.



        

      end if ! nb_el (< 90) 


      return
      end 
