        subroutine footprint_iasi(
     &     pixel, line, isat, inst, npoints, reftime,
     &     time, so_rs, sm_rs, sop_rs, smp_rs, ierr)
!+
!NOM/NAME $Id: footprint_iasi.F 25 2006-02-03 15:43:53Z frna $
!
!BUT/PURPOSE 
!       Exact calculation in the spacecraft fixed reference frame of footprints
!       for the optical axis (virtual) and the 4 IASI pixels
!
!DESCRIPTION 
!       FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions 
!
!       The footprint is the surface on Earth corresponding to the IFOV
!       The routine calculates the vectors defining the footprint edge
!
!       Calculations are similar to those of footprint.F but adapted
!       to the particular case of IASI. See lptoviewvect_iasi.F for
!       more details on IASI scanning function.
!
!       The routine is intented to calculates a full footprint edge.
!       described by "npoints" points
!
!       Reference frames:
!          Ri instrument-fixed reference frame
!          Rs spacecraft-fixed reference frame
!
!PROPRIETAIRE/OWNER
!       P. Brunel - A. Marsouin
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
!(version   date        comment           <author>)
! 1.0   01 Jul 2003      no          A. Marsouin P.Brunel
!       01 Apr 2004    comments changed A. Marsouin P.Brunel
!       15 Jul 2004    Fix declaration of r_i for g77 compiler  N C Atkinson
!         
!
!FUNCTIONS AND SUBROUTINES CALLS  (RETURN CODES FOR FUNCTIONS)
!
!
!COPYRIGHT (c) Meteo-France (EVOLUNIX - ATOVS - EUMETSAT project <1994>)
!
        implicit none                                                     
!     
!ARGUMENTS
        real pixel                     ! pixel number
        real line                      ! line number 
        integer isat                   ! satellite index in satid file
        integer inst                   ! IASI index for satid
        integer npoints                ! num of points for the footprint edge
        double precision reftime       ! julian instant of line 1
!out
        integer ierr
        double precision time          ! julian instant of the viewed scan
        double precision so_rs(3)      ! viewing vector for optical axis
        double precision sop_rs(3, npoints)! viewing vector for all points 
                                       ! of the footprint edge of optical axis
                                       !   (virtual pixel)
!
        double precision sm_rs(3,4)    ! viewing vector for the 4 pixels
        double precision smp_rs(3, 4, npoints) ! viewing vector for all points
                                       ! of the footprint edge of the 4 pixels
!       
!DECLARATIONS
!
!
!
! Local parameters
        double precision pi
        parameter ( pi = 3.14159265358979311600d0)

        integer max_contour_npoints
        parameter (max_contour_npoints = 360)

        double precision pi_7_4, pi_5_4, pi_3_4, pi_1_4
        parameter ( pi_7_4 = 7.*pi/4.)
        parameter ( pi_5_4 = 5.*pi/4.)
        parameter ( pi_3_4 = 3.*pi/4.)
        parameter ( pi_1_4 = 1.*pi/4.)
!
! Commons
!
#include "satid.h"
! Functions
!
! Local variables
        integer i, j
        double precision timlin
        double precision alpha      ! scanning angle for optical axis (rad)
        double precision dzeta      ! angular distance between optical
                                    !   axis and each IASI pixel (rad)
        double precision so_ri(3)   ! optical axis viewing vector in instrument Ref
        double precision sm_ri(3,4) ! viewing vector in instrument reference frame
        double precision psi
        double precision psi_step
        double precision eps
        double precision sop_ri(3, max_contour_npoints) ! for contour
        double precision smp_ri(3, 4, max_contour_npoints) ! for contour

        double precision y_i !
        double precision r_i ! Yaw Roll Pitch misalignment angles
        double precision p_i !

        double precision phi(4) ! iasi scan azimuth
        data phi/pi_7_4, pi_5_4, pi_3_4, pi_1_4/
! 
!-
!       
        ! This is scanning function number 3
        ! IASI scanning
        !-----------------------------------
        
        ierr = 0

        if( npoints .gt. max_contour_npoints .or.
     &      npoints .lt. 5  ) then
           call ml_wti('error',
     &                'invalid number of contour points ',npoints)
           call ml_wt ('error',
     &                'min allowed value is 5')
           call ml_wti('error',
     &                'max allowed value is ',max_contour_npoints)
           ierr = 1
           return
        endif

        ! "azimuth" angle for contour
        psi_step = 2. * pi / (npoints - 1)

        ! Test IFOV definition
        if(ifov_mode(inst,isat) .ne. 0) then
           call ml_wti('error','IFOV mode is ',ifov_mode(inst,isat))
           call ml_wt('error',
     &                'footprint routine does not allow this mode')
           ierr = 2
           return
        endif
        ! take first value of IFOV mode parameters
        eps = ifov_mode_p(1, inst, isat) /2.

        ! 1. calculate optical axis viewing vector in Ri frame 
        ! Ri   Instrument Reference frame
        !-----------------------------------------------------

        ! Scanning in a plane, no tilt
        !
        ! definition of scanning function parameters
        ! 1 = step time per FOV  (jdays)
        ! 2 = offset time between start time and first pixel of a line (jdays)
        ! 3 = center to center FOV step angle (rad)
        ! 4 = view number at nadir
        ! 5 = step time between two lines (jdays)
        ! the medium pixel is the real pixel value at nadir(number of steps +1)/2.
        ! 6 = angle between optical axis and any of the 4 pixels
        timlin = reftime + (line-1) * 
     &             inst_scan_mode_p(5,inst,isat)

        time = timlin + inst_scan_mode_p(2,inst,isat) + 
     &             inst_scan_mode_p(1,inst,isat)*(pixel-1)

        alpha = (pixel-inst_scan_mode_p(4,inst,isat)) * 
     &             inst_scan_mode_p(3,inst,isat)

           ! calculates the satellite - viewed point unit vector in Rs
        so_ri(1) = dcos(alpha)
        so_ri(2) = 0.
        so_ri(3) = dsin(alpha)

        do i = 1, npoints
           psi = (i-1) * psi_step
           sop_ri(1,i) = dcos(alpha) - eps*dsin(psi)*dsin(alpha)
           sop_ri(2,i) =               eps*dcos(psi)
           sop_ri(3,i) = dsin(alpha) + eps*dsin(psi)*dcos(alpha)
        end do

        ! 2. calculate 4 viewing vectors in Ri frame 
        !-------------------------------------------
        dzeta = inst_scan_mode_p(6,inst,isat)

        do j = 1, 4
           sm_ri(1,j) = dcos(alpha) - dzeta*dsin(phi(j))*dsin(alpha)
           sm_ri(2,j) =               dzeta*dcos(phi(j))
           sm_ri(3,j) = dsin(alpha) + dzeta*dsin(phi(j))*dcos(alpha)
           do i = 1, npoints
              psi = (i-1) * psi_step
              smp_ri(1,j,i) = dcos(alpha) - 
     &                        dsin(alpha)*(
     &                          dzeta*dsin(phi(j)) + 
     &                          eps*dsin(phi(j)+psi)  )
              smp_ri(2,j,i) = dzeta*dcos(phi(j)) + 
     &                          eps*dcos(phi(j)+psi)
              smp_ri(3,j,i) = dsin(alpha) + 
     &                        dcos(alpha)*(
     &                          dzeta*dsin(phi(j)) + 
     &                          eps*dsin(phi(j)+psi)  )
           end do
        end do


        ! 3. calculate all viewing vector in Rs
        ! Rs   Spacecraft-fixed Reference frame
        !--------------------------------------
        if (mis_align_mode(inst,isat) .eq. 0) then
           
           ! Yaw Roll Pitch misalignment definition
           if (mis_align_mode_n(inst,isat) .ge. 3) then
              ! yes 3 parameters
              y_i = mis_align_mode_p(1,inst,isat)
              r_i = mis_align_mode_p(2,inst,isat)
              p_i = mis_align_mode_p(3,inst,isat)
           else
              y_i = 0.
              r_i = 0.
              p_i = 0.
           endif

           so_rs(1) =      so_ri(1) + p_i*so_ri(2) - r_i*so_ri(3)
           so_rs(2) = -p_i*so_ri(1) +     so_ri(2) + y_i*so_ri(3)
           so_rs(3) =  r_i*so_ri(1) - y_i*so_ri(2) +     so_ri(3) 

           do i = 1, 4
              sm_rs(1,i)=     sm_ri(1,i)+ p_i*sm_ri(2,i)- r_i*sm_ri(3,i)
              sm_rs(2,i)=-p_i*sm_ri(1,i)+     sm_ri(2,i)+ y_i*sm_ri(3,i)
              sm_rs(3,i)= r_i*sm_ri(1,i)- y_i*sm_ri(2,i)+     sm_ri(3,i)
           end do
           
           do i = 1, npoints
              sop_rs(1,i) =  
     &                  sop_ri(1,i) + p_i*sop_ri(2,i) - r_i*sop_ri(3,i)
              sop_rs(2,i) =
     &             -p_i*sop_ri(1,i) +     sop_ri(2,i) + y_i*sop_ri(3,i)
              sop_rs(3,i) =
     &              r_i*sop_ri(1,i) - y_i*sop_ri(2,i) +     sop_ri(3,i) 
           end do

           do j = 1, 4
              do i = 1, npoints
                 smp_rs(1,j,i) =  
     &             smp_ri(1,j,i) + p_i*smp_ri(2,j,i) - r_i*smp_ri(3,j,i)
                 smp_rs(2,j,i) =
     &        -p_i*smp_ri(1,j,i) +     smp_ri(2,j,i) + y_i*smp_ri(3,j,i)
                 smp_rs(3,j,i) =
     &         r_i*smp_ri(1,j,i) - y_i*smp_ri(2,j,i) +     smp_ri(3,j,i) 
              end do
           end do

        else
           !No misalignment, or unknown
           so_rs(1) = so_ri(1)
           so_rs(2) = so_ri(2)
           so_rs(3) = so_ri(3)
           do i = 1, 4
              sm_rs(1,i)= sm_ri(1,i)
              sm_rs(2,i)= sm_ri(2,i)
              sm_rs(3,i)= sm_ri(3,i)
           end do
           do i = 1, npoints
              sop_rs(1,i) = sop_ri(1,i)
              sop_rs(2,i) = sop_ri(2,i)
              sop_rs(3,i) = sop_ri(3,i)
           end do
           do j = 1, 4
              do i = 1, npoints
                 smp_rs(1,j,i) = smp_ri(1,j,i)
                 smp_rs(2,j,i) = smp_ri(2,j,i)
                 smp_rs(3,j,i) = smp_ri(3,j,i)
              end do
           end do
        endif

        return
        end
