#!/bin/ksh
#$Id: AAPP_RUN_FY1.ksh 123 2008-09-02 11:55:18Z frna $
#
# Korn Shell to run AAPP for FY1 imager data. This just extracts the AVHRR-like
# channels.
#
# AAPP_PREFIX environment variable must be defined
#
# PROPRIETAIRE/OWNER
#       NWP SAF
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2008, EUMETSAT, All Rights Reserved.
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment                         <author>)
#  1.0     29/01/08   initial                        Nigel Atkinson
#  1.1     27/08/08   add -u option
#

if [ "$AAPP_PREFIX" = "" ]; then
  echo 'Error: AAPP_PREFIX not defined'
  exit 2
fi
if [ ! -d "$AAPP_PREFIX" ]; then
  echo 'Error: AAPP_PREFIX is not a directory:' $AAPP_PREFIX
  exit 2
fi

. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh

# Usage and get input informations
#---------------------------------
usage(){
  echo 'usage : AAPP_RUN_FY1.ksh [options] file_name'
  echo '   -d date        : date of CHRPT data (required), in a format recognised'
  echo '                    by Unix date command, e.g. 20080129 or "29 Jan 2008")'
  echo '   -o outdir      : output directory (default $WRK)'
  echo '   -u             : use -u is input file is already unpacked'
  echo '   file_name      : CHRPT data file (full path or relative to current)'
}

datadate=""
unpacked=""
while getopts d:o:u c
do
  case $c in
    d) datadate=$OPTARG;;
    o) OUTDIR="$OPTARG";;
    u) unpacked="-u";;
    *) usage
       exit 2;;
  esac
done
shift `expr $OPTIND - 1`
OUTDIR=${OUTDIR:-$WRK}

if [ $# -ne 1 ] || [ "$datadate" = "" ]; then
  usage
  exit 2
fi

FILE=$1
log_info "start of processing ${FILE}"

# Convert to pseudo-NOAA format
#---------------------------------

daynum=$(date -d "$datadate" +%j)
rm -f fy1.hrp        #output file

convert_chrpt $unpacked $FILE fy1.hrp $daynum

if [ $? != 0 ]; then
  log_error Error converting ${FILE}
  exit 1
fi

# Check the satellite identifier
#-------------------------------
YEAR=$(date -d "$datadate" +%Y)
SATIMG_YYYYMMDD_HHMN=`hrpidf.exe<<EOF
${YEAR}
fy1.hrp
EOF`
if [ $? != 0 ]; then
  log_info  hrpidf.exe output ${SATIMG_YYYYMMDD_HHMN}
  log_error hrpidf file fy1.hrp not usable
  exit 1
fi
SATIMG=`echo ${SATIMG_YYYYMMDD_HHMN} | cut -b 1-6 `
if [[ $SATIMG != fy* ]]; then
  log_info  hrpidf.exe output ${SATIMG_YYYYMMDD_HHMN}
  log_error Not identified as fy series. Check the satid.txt file at $DIR_SATID
  exit 1
fi

# Make tle the default bulletin
#------------------------------
if [[ $PAR_NAVIGATION_DEFAULT_LISTESAT != *${SATIMG}* ]]; then
  export PAR_NAVIGATION_DEFAULT_LISTESAT=$SATIMG
  export PAR_NAVIGATION_DEFAULT_LISTEBUL=tle
fi

# Run AAPP_RUN_NOAA (no calibration)
#------------------

AAPP_RUN_NOAA -C -i "AVHRR" -Y $YEAR -o $OUTDIR fy1.hrp

if [ $? != 0 ]; then
  log_error Error in AAPP_RUN_NOAA for ${FILE}
  exit 1
fi
rm -f fy1.hrp                   #delete the converted HRPT file
rm -f $(ls $OUTDIR/hrpt_*.hrp)  #delete the link

# Rename fy1-04 -> fy1d in output file name
#------------------------------------------

file1b_tmp=$(ls $OUTDIR/hrpt_*.l1b)
file1b=$OUTDIR/$(echo $(basename $file1b_tmp) | sed s/fy1-04/fy1d/)
mv $file1b_tmp $file1b

# Run fy1cl
#----------

fy1cl $file1b

if [ $? != 0 ]; then
  log_error Error in fy1cl for ${FILE}
  exit 1
fi

log_info "Created $file1b"
log_info "end of processing ${FILE}"
