#!/bin/ksh
#$Id: AAPP_RUN_METOP.ksh 599 2018-11-09 11:53:44Z frna $
#
# Korn Shell to run AAPP for METOP data
#
# Environment variables required:
#   AAPP_PREFIX
#
# Environment variables required if IASI OPS-LRS is to be run:
#   DIR_IASICONFIG (directory containing IASI configuration files)
#   PATH_OPS (required if ops_process is not alrady in PATH) 
#
# Requirements:
# 1) All files to be processed are in a single directory
# 2) One file per instrument (i.e. dump mode)
# 3) File names follow the EUMETSAT convention, e.g. 
#   AMSA_xxx_00_M04_20020808181206Z_20020808195406Z_N_O_20020808201206Z
#   MHSx_xxx_00_M04_20020808181201Z_20020808195401Z_N_O_20020808201201Z
#   HIRS_xxx_00_M04_20020808181200Z_20020808195358Z_N_O_20020808201200Z
#   AVHR_xxx_00_M04_20020808181200Z_20020808182359Z_N_O_20020808201200Z
#   HKTM_xxx_00_M04_20020808181200Z_20020808195358Z_N_O_20020808201200Z
#   IASI_xxx_00_M04_20020808181200Z_20020808195358Z_N_O_20020808201200Z
#
# Usage: AAPP_RUN_METOP -i instruments -g grids -d indir -o outdir -b -c -z
#   where "instruments" can be any from the list "AMSU-A MHS HIRS AVHRR IASI"
#             (default "AMSU-A MHS HIRS AVHRR")
#         "grids" can be any from the list "AMSU-A MHS HIRS IASI"
#             (default "HIRS")
#         "indir" is the directory containing the input data (default current)
#         "outdir" is the directory for receiving the output data (default current)
#         -b option causes existing 1b files in the working directory to be re-used
#              (default is to delete them at the start)
#         -c option causes 1b files to be copied to dest directory rather than moved
#              (use prior to a second run with the -b option)
#         -z skips avh2hirs
#
#   If "instruments" is " " then only the HKTM file is processed
#   If "grids" is " " then atovpp is not run
#
#   If "HKTM" is included in the list of "instruments", then the OBT/UTC
#   correlation parameters in the instrument level 0 VIADRs are over-written 
#   with the parameters extracted from the HKTM file. This should not normally
#   be needed, but is provided as an option because some reception systems may
#   not have allowed for the change of units that EUMETSAT applied in Dec 2006
#   in order to comply with EPS.GGS.TN.980021.
#   
#   We may want to process a pass in two stages - a first step to get AMSU/HIRS/AVHRR
#   products out, then a second step to run IASI OPS-LRS and generate products
#   on IASI grid. (Note that OPS-LRS requires AVHRR l1b). In this case the commands
#   would be as follows:
#   AAPP_RUN_METOP -i "AMSU-A MHS HIRS AVHRR" -g "HIRS" -c
#   AAPP_RUN_METOP -i "AMSU-A MHS IASI" -g "IASI" -b
#
# IASI OPS-LRS is not automatically included in the AAPP v6 distribution. It
# must be requested by the user.
# 
# This script does not do BUFR encoding - the user may do this afterwards (level 1c)
# if needed using tool aapp_encodebufr_1c.
#
# If your HRPT system does not generate data in dump mode then you must either modify
# this script or concatenate the incoming L0 files (adjusting file names as
# necessary).
#
# PROPRIETAIRE/OWNER
#       NWP SAF
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2006, EUMETSAT, All Rights Reserved.
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# version  date    author          comment
# 1.0     19/07/06 Nigel Atkinson  initial
# 1.1     14/09/06 Nigel Atkinson  change test for ops_process; add comments
# 1.2     07/11/06 Nigel Atkinson  various corrections
#                                  use -ignore_degraded flags
# 1.3     29/01/07 Nigel Atkinson  option to over-write OBT/UTC correlation
#                                  parameters using values from HKTM file
# 1.4     19/03/07 Nigel Atkinson  update for latest OPS-LRS
# 1.5     12/03/09 Nigel Atkinson  Correct typo: log -> log_info
# 1.6     21/07/11 Nigel Atkinson  Change default GRIDS to " " if HIRS not present. 
# 1.7     04/10/11 Nigel Atkinson  Comment out call to convert_config_files
# 1.8	  27/06/11 Pascale Roquet  HKTM MMAM message
# 1.9     12/01/17 Nigel Atkinson  Copy IASI pfs file to output directory, as
#                                  it's needed for the CSPP_UW_HSRTV package.
# 1.10    24/08/17 Nigel Atkinson  Remove spm for AAPP v8. Add call to
#                                  avh_get_vis_coefs.
# 1.11    13/02/18 Pascale Roquet  Run MAIA_RUN_AVHRR if "$RUN_MAIA" = "yes"

if [ "$AAPP_PREFIX" = "" ]; then
  echo 'Error: AAPP_PREFIX not defined'
  exit 2
fi
if [ ! -d "$AAPP_PREFIX" ]; then
  echo 'Error: AAPP_PREFIX is not a directory:' $AAPP_PREFIX
  exit 2
fi

. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh


# Usage and get input informations
#---------------------------------
usage(){
  echo 'usage: AAPP_RUN_METOP -i instruments -g grids -d indir -o outdir -b -c -z'
}
	

while getopts i:g:d:o:bcz c
do
  case $c in
    i) INSTRUMENTS="$OPTARG";;
    g) GRIDS="$OPTARG";;
    d) INDIR="$OPTARG";;
    o) OUTDIR="$OPTARG";;
    b) KEEP1B=Y;;
    c) COPY1B=Y;;
    z) AVH2HIRS=N;;
    *) usage; exit 2;;
  esac
done
shift `expr $OPTIND - 1`

INSTRUMENTS=${INSTRUMENTS:-"AMSU-A MHS HIRS AVHRR"}
if [[ "$INSTRUMENTS" = *HIRS* ]]; then
  GRIDS=${GRIDS:-"HIRS"}
else
  GRIDS=${GRIDS:-" "}
fi
INDIR=${INDIR:-$PWD}
OUTDIR=${OUTDIR:-$PWD}
KEEP1B=${KEEP1B:-N}
COPY1B=${COPY1B:-N}
AVH2HIRS=${AVH2HIRS:-Y}

# Initialise
NNNNN=0
opserror=0

# Get list of input files and loop round them
#--------------------------------------------
#list HKTM first

cd $INDIR

files=$(ls HK*_???_00_M*Z 2>/dev/null)
files="${files} $(ls HI*_???_00_M*Z A*_???_00_M*Z M*_???_00_M*Z I*_???_00_M*Z 2>/dev/null)"

cd ${WRK}
log_info "Working directory is ${WRK}"
log_info "Input directory is ${INDIR}"
rm -f *.l1c *.l1d *.log *.par
if [ $KEEP1B = N ]; then
  rm -f *.l1b AVHR_xxx_1B*
fi
AMAdata=N; MHSdata=N; HIRSdata=N; AVHdata=N

for file in $files; do
  
  log_info "IN $file"
  instr=`echo $file | cut -c1-4`          #e.g. AMSA
  SATIMG=`echo $file | cut -c13-15`        #e.g. M02
  startdate=`echo $file | cut -c17-30`
  YYYYMMDD=`echo $startdate | cut -c1-8`
  YYYY=`echo $startdate | cut -c1-4`
  YY=`echo $startdate | cut -c3-4`
  MM=`echo $startdate | cut -c5-6`
  DD=`echo $startdate | cut -c7-8`
  HHMN=`echo $startdate | cut -c9-12`
  HH=`echo $startdate | cut -c9-10`
  MN=`echo $startdate | cut -c11-12`
  SS=`echo $startdate | cut -c13-14`
   
# Create the satpos file
#-----------------------

  FILE_SATPOS=${DIR_NAVIGATION}/satpos/satpos_${SATIMG}_${YYYYMMDD}.txt
  if [ ! -s $FILE_SATPOS ]; then

  # which type of bulletin for the requested satellite
  # this determines which satpos program to run
  #---------------------------------------------------
    i=0
    set -A ABUL ${PAR_NAVIGATION_DEFAULT_LISTEBUL}
    for SAT in ${PAR_NAVIGATION_DEFAULT_LISTESAT}
    do
      if [ "${SAT}" = "${SATIMG}" ] ; then
         BUL=${ABUL[i]}
      fi
      i=`expr ${i} + 1`
    done
    BUL=${BUL=tbus}
    echo "BUL = $BUL"
    case ${BUL} in
      tbus)  SATPOS_COM='satpost'
             BULING_COM='tbusing';;
      tle)   SATPOS_COM='satpostle'
             BULING_COM='tleing';;
      spm)   log_error "spm bulletins are no longer supported"
             exit 1;;
      *) SATPOS_COM='satpost'
         BULING_COM='tbusing';;
    esac

  #Run TBUS or TWO LINES ingest for the satellite
  #----------------------------------------------------
    ${BULING_COM} -s ${SATIMG}

  #Creation of the satpos file for the current day
  #-----------------------------------------------
    ${SATPOS_COM} -o -s ${SATIMG} -S ${STATION} -d ${DD}/${MM}/${YY} -n 1.2
    stat=$?
    if [ ! ${stat} -eq 0 ]; then
     log_error satpost error ${stat}
     exit 1
    elif [ ! -s ${FILE_SATPOS} ]; then
     log_error ${FILE_SATPOS} does not exist after satpost
     exit 1
    fi
  fi        #create satpos file

  #Now we are able to get the orbit number 
  #Use the date and time of data and the Satellite position file
  #-------------------------------------------------------------

  if [ $NNNNN = 0 ]; then
    NNNNN=$(sdh2orbnum -s $SATIMG -d $YYYYMMDD -h $HHMN)

    stat=$?
    if [ ! ${stat} -eq 0 ]; then
      log_error sdh2orbnum error ${stat}
      exit 1
    fi

    if [ ${NNNNN} -le 0 ] || [ ${NNNNN} -gt 100000 ]; then
      log_error invalid orbit number ${NNNNN}
      exit 1
    fi

    log_info "Satellite = $SATIMG"
    log_info "Date      = $YYYYMMDD"
    log_info "Hour      = $HHMN"
    log_info "Orbit     = $NNNNN"
  fi

#-----------------------------------------------------------------------------------------
# Get OBT/UTC correlation parameters from Admin message in HKTM file and
# over-write VIADR in instrument files (optional)
# 2 case for HKTM files : MMAM message or admin message
#-----------------------------------------------------------------------------------------

  if [[ $instr = HKTM && $INSTRUMENTS = *HKTM* ]]; then
    is_mmam=$(is-mmam.exe -pfsl0 ${INDIR}/${file})
    if [ "$is_mmam" == "TRUE" ] ; then
      mmam-main.exe -pfsl0 ${INDIR}/${file} /tmp/mmam$$.bz2
      bunzip2 /tmp/mmam$$.bz2
      OBTUTC="$(print-mmam-obt-utc.pl /tmp/mmam$$)"
      rm /tmp/mmam$$
      [ -z "$OBTUTC" ] && (log_error "can't get OBTUTC parameters from HKTM file" ; exit 1)
      patch_cmd=patch-level0-from-mmam.exe
      log_info "MMAM OBT/UTC parameters: $OBTUTC"
    else
      { log_error "MMAM not found, and old-style admin messages are no longer supported."; exit 1; }
      OBTUTC=$(admin-dump -pfsl0 -obtutc ${INDIR}/${file})  #retain the code for reference
      patch_cmd=patch-level0
      log_info "admin OBT/UTC parameters: $OBTUTC"
    fi 
    
      INSTRUMENTS="$(echo $INSTRUMENTS | sed s/HKTM//)"  #remove HKTM from list
      allfiles=$(ls $INDIR)
      for f in $allfiles; do
       if [[ $f != HKTM* ]]; then
         log_info "patching VIADR for $f with $patch_cmd"
         $patch_cmd $OBTUTC ${INDIR}/$f
       fi
      done

    
  fi

#-----------------------------------------------------------------------------------------
# AMSU-A calibration
#-----------------------------------------------------------------------------------------

  if [[ $instr = AMSA && $INSTRUMENTS = *AMSU-A* && ! -s aman.l1b ]]; then
    
    decom-amsua-metop ${INDIR}/${file} aman.l1b >decom-amsua-metop.log\
        || { log_error "decom-amsua-metop failed"; exit 1; }

    log_info "Running amsuacl"
    amsuacl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN aman.l1b \
        || { log_error "amsuacl failed"; exit 1; }
    AMAdata=Y

#-----------------------------------------------------------------------------------------
# MHS calibration
#-----------------------------------------------------------------------------------------

  elif [[ $instr = MHSx && $INSTRUMENTS = *MHS* && ! -s ambn.l1b ]]; then

    flags="-ignore_degraded_inst_mdr -ignore_degraded_proc_mdr"
    decom-mhs-metop $flags ${INDIR}/${file} ambn.l1b >decom-mhs-metop.log \
        || { log_error "decom-mhs-metop failed"; exit 1; }

    log_info "Running mhscl"
    mhscl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN ambn.l1b \
        || { log_error "mhscl failed"; exit 1; }
    MHSdata=Y

#-----------------------------------------------------------------------------------------
# HIRS calibration
#-----------------------------------------------------------------------------------------

  elif [[ $instr = HIRS && $INSTRUMENTS = *HIRS* && ! -s hrsn.l1b ]]; then

    flags="-ignore_degraded_inst_mdr -ignore_degraded_proc_mdr"
    decom-hirs-metop $flags ${INDIR}/${file} hrsn.l1b >decom-hirs-metop.log \
        || { log_error "decom-hirs-metop failed"; exit 1; }

    if [ "${HIRSCL_VERSION}" = 1 ]; then        #hirscl_algoV4

#   Manage the historic file. Run this first in case hirscl_algoV4 fails
      FILE_HISTORIC=${PAR_CALIBRATION_MONITOR}/${SATIMG}/hirs_historic.txt
      hirs_historic_file_manage -m $HIST_SIZE_HIGH -r $HIST_SIZE_LOW \
        -n $HIST_NMAX $FILE_HISTORIC

#   Creation of the hirs_b1aslope.txt file in the working directory
      hcalcb1_algoV4 -s $SATIMG -y $YYYY -m $MM -d $DD -h $HH  -n $MN \
         || { log_error "hcalcb1_algoV4 failed"; exit 1; }

#   HIRS calibration and navigation
      log_info "Running hirscl_algoV4"
      hirscl_algoV4 -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN hrsn.l1b \
        2>hirscl_algoV4.err || { log_error "hirscl_algoV4 failed"; exit 1; } 

    else          #hirscl original algorithm

      log_info "Running hirscl"
      hirscl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN hrsn.l1b \
        2>hirscl.err || { log_error "hirscl failed"; exit 1; } 

    fi
    HIRSdata=Y
    
#-----------------------------------------------------------------------------------------
# AVHRR calibration
#-----------------------------------------------------------------------------------------

  elif [[ $instr = AVHR && $INSTRUMENTS = *AVHR* && ! -s hrpt.l1b ]]; then

    log_info "Running avhrcl"
    flags="-ignore_degraded_inst_mdr -ignore_degraded_proc_mdr"
    decom-avhrr-metop $flags ${INDIR}/${file} hrpt.l1b >decom-avhrr-metop.log \
        || { log_error "decom-avhrr-metop failed"; exit 1; }

    avhrcl -c -l -s $SATIMG -d $YYYYMMDD -h $HHMN  -n $NNNNN hrpt.l1b \
      || { log_error "avhrcl failed"; exit 1; }

    avh_get_vis_coefs -s $SATIMG -d $YYYYMMDD hrpt.l1b  #operational visible coefs

    AVHdata=Y

#-----------------------------------------------------------------------------------------
# IASI
#-----------------------------------------------------------------------------------------

  elif [[ $instr = IASI && $INSTRUMENTS = *IASI* ]]; then

    IASIfile=$file                    #just record the file name
  
  fi

done    #loop over input files

# Check whether all required instruments (excluding IASI) have been processed

allok=Y
if [[ $INSTRUMENTS = *AMSU-A* && ! -s aman.l1b ]]; then
  log_error "aman.l1b missing or empty"
  allok=N
elif [[ $INSTRUMENTS = *MHS* && ! -s ambn.l1b ]]; then
  log_error "ambn.l1b missing or empty"
  allok=N
elif [[ $INSTRUMENTS = *HIRS* && ! -s hrsn.l1b ]]; then
  log_error "hrsn.l1b missing or empty"
  allok=N
elif [[ $INSTRUMENTS = *AVHRR* && ! -s hrpt.l1b ]]; then
  log_error "hrpt.l1b missing or empty"
  allok=N
fi

if [ $allok = N ]; then
  exit 1
fi


#-----------------------
# Preprocessing - atovin 
#-----------------------

#instrument list for atovin
INSTR1=$(echo $INSTRUMENTS | sed s/AVHRR// | sed s/IASI// | sed s/MHS/AMSU-B/)

if [ ${#INSTR1} -gt 1 ]; then
  log_info "Running atovin $INSTR1"
  atovin $INSTR1 || { log_error "atovin failed"; exit 1; }
fi

#-----------------------
# IASI OPS-LRS
#-----------------------
#In case of OPS-LRS failure we continue with running atovpp for ATOVS
#instruments, if that has been requested. Final exit code is 2 if OPS-LRS failed

if [[ $INSTRUMENTS = *IASI* ]]; then
  opserror=2
  if [ "$IASIfile" != "" ]; then
    
    if [ -s hrpt.l1b ]; then
      log_info "Running aapp-eps_avhrrl1b"    #convert to PFS L1B
      aapp-eps_avhrrl1b hrpt.l1b >/dev/null \
         || { log_error "aapp-eps_avhrrl1b failed"; }
    fi
      
    AVH1bfile="$(ls AVHR_xxx_1B*)"

    if [  "$RUN_MAIA" = "yes" ] ; then
      log_info "Running maia"
      MAIA4_RUN_AVHRR ${WRK}/hrpt.l1b || { log_error "MAIA_RUN_AVHRR failed"; exit 1; }      
      mv ${AVH1bfile} ${AVH1bfile}.old
      avhCT="$(ls avhCT*)"
      maia2pfs.exe ${AVH1bfile}.old  ${avhCT} ${AVH1bfile}|| { log_error "maia2pfs failed"; exit 1; }

    fi

    if [ "$AVH1bfile" = "" ]; then
      log_error "IASI requested but no AVHRR PFS 1b: cannot run OPS-LRS"
    else
      export METOP_ENV=NORMAL               #or TEST for simulated data
      if [ ! -d "$DIR_IASICONFIG" ]; then
        log_error "DIR_IASICONFIG not defined: cannot run OPS-LRS"
      else

        if [ -d "$PATH_OPS" ]; then
          export PATH=${PATH}:$PATH_OPS
        fi
        found=N                #check that ops_process is present in PATH
        for path in $(echo $PATH | sed y/":"/" "/); do
          if [ -f ${path}/ops_process ]; then found=Y; fi
        done        
        if [ $found = N ]; then
          log_error "ops_process was not found in PATH: cannot run OPS-LRS"
        else

          log_info "Running ops_process"
    #For OPS v6.1 onwards we don't need to convert the config files
    #      convert_config_files      #convert endianness if required
          rm -rf $SATIMG            #start a new output directory
          rm -f ${SATIMG}-pfsIASI1C.txt

          ops_process --satellite=$SATIMG --processing=DUMP \
            ${INDIR}/${IASIfile} \
            ${AVH1bfile}

          if [ -s ${SATIMG}-pfsIASI1C.txt ]; then
            IASI_outfile=$(cat ${SATIMG}-pfsIASI1C.txt)
            mkdir -p $OUTDIR
            cp $IASI_outfile $OUTDIR
            log_info "Converting IASI 1c to AAPP format"
            convert_iasi1c $IASI_outfile iasi.l1c
            if [ $? = 0 ]; then
              opserror=0     #all OPS-LRS steps OK
            else
              log_error "convert_iasi1c failed"
            fi
          else
            log_error "ops_process failed"
          fi

          ops_process --satellite=$SATIMG --stop

        fi
      fi
    fi
  fi
  if [ $opserror != 0 ]; then          #OPS-LRS error - continue without IASI
    INSTRUMENTS=$(echo $INSTRUMENTS | sed s/IASI//)
    GRIDS=$(echo $GRIDS | sed s/IASI//)
  fi
fi

#------------------------------------
# Preprocessing - atovpp and avh2hirs 
#------------------------------------

INSTR2=$(echo $INSTRUMENTS | sed s/AVHRR//)     #for atovpp

if [ ${#GRIDS} -gt 1 ]; then
  log_info "Running atovpp"
  atovpp -i "$INSTR2" -g "$GRIDS" || { log_error "atovpp failed"; exit 1; }

  if [[ $INSTRUMENTS = *AVHRR* && $GRIDS = *HIRS* && $AVH2HIRS = Y ]]; then
    log_info "avh2hirs ATOVS started"
    avh2hirs ATOVS || { log_error "avh2hirs failed"; exit 1; }
    log_info "avh2hirs ATOVS completed"
  fi
fi


#-------------------------------------------
# Re-name files and move to output directory
#-------------------------------------------

mkdir -p $OUTDIR
if [ $COPY1B = Y ]; then
  cmd=cp        #option for 1b files
else
  cmd=mv
fi


# AVHRR
if [ -f hrpt.l1b ] && [ $AVHdata = Y ]; then
  $cmd hrpt.l1b ${OUTDIR}/hrpt_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
fi

# HIRS
if [ -f hrsn.l1b ] && [ $HIRSdata = Y ]; then
  $cmd hrsn.l1b ${OUTDIR}/hirsl1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
fi
if [ -f hrsn.l1c ] && [ $HIRSdata = Y ]; then
  mv hrsn.l1c ${OUTDIR}/hirsl1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
fi
if [ -f hirs.l1d ]; then
  mv hirs.l1d ${OUTDIR}/hirsl1d_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1d
fi

#AMSU-A
if [ -f aman.l1b ] && [ $AMAdata = Y ]; then
  $cmd aman.l1b ${OUTDIR}/amsual1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
fi
if [ -f aman.l1c ] && [ $AMAdata = Y ]; then
  mv aman.l1c ${OUTDIR}/amsual1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
fi
if [ -f aman.l1d ]; then
  mv aman.l1d ${OUTDIR}/amsual1d_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1d
fi

#MHS
if [ -f ambn.l1b ] && [ $MHSdata = Y ]; then
  $cmd ambn.l1b ${OUTDIR}/mhsl1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
fi
if [ -f ambn.l1c ] && [ $MHSdata = Y ]; then
  mv ambn.l1c ${OUTDIR}/mhsl1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
fi
if [ -f ambn.l1d ]; then
  mv ambn.l1d ${OUTDIR}/mhsl1d_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1d
fi

#IASI
if [ -f iasi.l1c ]; then
  mv iasi.l1c ${OUTDIR}/iasil1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
fi
if [ -f iasi.l1d ]; then
  mv iasi.l1d ${OUTDIR}/iasil1d_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1d
fi

log_info "end of processing"
exit $opserror
