#!/bin/ksh
#$Id: AAPP_RUN_NOAA.ksh 546 2017-08-22 12:25:40Z frna $
#
# Korn Shell to run AAPP for NOAA data
#
# AAPP_PREFIX environment variable must be defined
#
# PROPRIETAIRE/OWNER
#       NWP SAF
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2006, EUMETSAT, All Rights Reserved.
#
# HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES) 
# (version   date      comment                         <author>)
#  1.0     10/02/99   initial                        Tiphaine Labrot
#  1.1     28/06/04   Allow for MHS                   Nigel Atkinson 
#  1.2     01/07/04   add Two Line orbital model      Pascal Brunel
#  1.3     18/02/05   add HIRS calibration version 4  Tiphaine Labrot
#  1.4     18/04/05   add call to hirs_historic_file_manage  Nigel Atkinson
#  1.5     01/07/05   redirect stderr for hirscl.            Nigel Atkinson 
#  1.6     08/06/06   use ATOVS_CONF. Tools prh* renamed to prl1b*.
#                     Rename to AAPP_RUN_NOAA                Nigel Atkinson
#  1.7     21/07/06   Add -i, -g, -o and -z options          Nigel Atkinson
#  1.8     29/01/08   Add -C, -L options to skip calibration/location. NCA
#  1.9     23/10/09   Allow ${HIRSCL_VERSION} to be satellite dependent and
#                     allow a version number >1 to skip HIRS calibration.
#                     e.g. NOAA-15 HIRS broke on 1 June 2009, so specify
#                     HIRSCL_VERSION="1 99"
#                     HIRSCL_SAT="default noaa15"
#  1.10    21/07/11   Change default GRIDS to " " if HIRS not present. 
#

if [ "$AAPP_PREFIX" = "" ]; then
  echo 'Error: AAPP_PREFIX not defined'
  exit 2
fi
if [ ! -d "$AAPP_PREFIX" ]; then
  echo 'Error: AAPP_PREFIX is not a directory:' $AAPP_PREFIX
  exit 2
fi

. "$AAPP_PREFIX/ATOVS_CONF"
. liblog.sh


# Usage and get input informations
#---------------------------------
usage(){
  echo 'usage : AAPP_RUN_NOAA.ksh [options] file_name'
  echo '   -D	          : debug on'
  echo '   -Y year        : year of the HRPT data default=current year'
  echo '   -i instruments : from list "AMSU-A AMSU-B HIRS MSU AVHRR DCS" (default all available)'
  echo '   -g grids       : from list "AMSU-A AMSU-B HIRS" (default "HIRS")'
  echo '   -o outdir      : output directory (default $WRK)'
  echo '   -z             : skip avh2hirs'
  echo '   -C             : skip calibration'
  echo '   -L             : skip Earth location'
  echo '   file_name      : HRPT data file (full path or relative to current)'
}
	
YEAR=`date +\%Y`
# default of YEAR is the current year.
# `date +\%Y` might not work on certain systems! Replace this command by one 
# that works on your system.

# NOTE:  Using YEAR-default can cause Y2K problems when processing in year 2000
# or later data from year 1999 or earlier. Use in such cases the YEAR parameter 
# of the chain processing script (i.e. this script).

calflag="-c"   #default is to calibrate
locflag="-l"   #default is to Earth locate

while getopts i:g:o:zDCLY: c
do
  case $c in
    i) INSTRUMENTS="$OPTARG";;
    g) GRIDS="$OPTARG";;
    o) OUTDIR="$OPTARG";;
    z) AVH2HIRS=N;;
    Y) YEAR=$OPTARG;;
    D) DEBUG='on';;
    C) calflag='';;
    L) locflag='';;
    *)       usage
       exit 2;;
  esac
done
shift `expr $OPTIND - 1`
export YEAR
OUTDIR=${OUTDIR:-$WRK}
AVH2HIRS=${AVH2HIRS:-Y}
if [ $(echo $OUTDIR | cut -c 1-1) != '/' ]; then
  OUTDIR=$PWD/$OUTDIR      #assume relative to current dir
  if [ ! -d $OUTDIR ]; then
    log_error "invalid output directory $OUTDIR"
    exit 1
  fi
fi

if [ $# -ne 1 ] ; then
  usage
  exit 2
fi
FILE=$1
callocflag="$calflag $locflag"
log_info "start of processing ${FILE}"

#Verify the pathname of the HRPT file
#------------------------------------
ROOT_FILE=`echo ${FILE} | cut -c 1-1`
if [ ! "${ROOT_FILE}" = '/' ] ; then
  FILE=$PWD/$FILE      #assume relative to current dir
fi
if [ ! -f $FILE ]; then
  log_error "nonexistent file $FILE"
fi

#Verify the validity of Year
#---------------------------
if [ ${YEAR} -lt 1978 ] || [ ${YEAR} -gt 2049 ]; then
   log_error invalid year ${YEAR}
   exit 1
fi


# Take the fundamental informations from the file
#------------------------------------------------

#Get the day of year and the hour of the data
#The Year is given as a script argument because it is not present
#in the HRPT format
SATIMG_YYYYMMDD_HHMN=`hrpidf.exe<<EOF
${YEAR}
${FILE}
EOF`
stat=$?
if [ ${stat} -ne 0 ] ; then
   log_line
   log_info  hrpidf.exe output ${SATIMG_YYYYMMDD_HHMN}
   log_error hrpidf file ${FILE} not usable
   exit 1
fi

SATIMG=`echo ${SATIMG_YYYYMMDD_HHMN} | cut -b 1-6 `
YYYYMMDD=`echo ${SATIMG_YYYYMMDD_HHMN} | cut -b 8-15 `
HHMN=`echo ${SATIMG_YYYYMMDD_HHMN} |cut -b 17-20 `
 
# which type of bulletin for the requested satellite
# this determines which satpos program to run
#---------------------------------------------------
  i=0
  set -A ABUL ${PAR_NAVIGATION_DEFAULT_LISTEBUL}
  for SAT in ${PAR_NAVIGATION_DEFAULT_LISTESAT}
  do
    if [ "${SAT}" = "${SATIMG}" ] ; then
       BUL=${ABUL[i]}
    fi
    i=`expr ${i} + 1`
  done
  BUL=${BUL=tbus}
  echo "BUL = $BUL"
  case ${BUL} in
    tbus)  SATPOS_COM='satpost'
           BULING_COM='tbusing';;
    tle)   SATPOS_COM='satpostle'
           BULING_COM='tleing';;
    *) SATPOS_COM='satpost'
       BULING_COM='tbusing';;
  esac

#Run TBUS or TWO LINES ingest for the satellite
#----------------------------------------------
${BULING_COM} -s ${SATIMG}

#Creation of the satpos file for the current day
#-----------------------------------------------
YYYY=`echo ${YYYYMMDD} | cut -b 1-4`
YY=`echo ${YYYYMMDD} | cut -b 3-4`
MM=`echo ${YYYYMMDD} | cut -b 5-6`
DD=`echo ${YYYYMMDD} | cut -b 7-8`
HH=`echo ${HHMN} | cut -b 1-2`
MN=`echo ${HHMN} | cut -b 3-4`
DIR_SATPOS=${DIR_NAVIGATION}/satpos
FILE_SATPOS=${DIR_SATPOS}/satpos_${SATIMG}_${YYYYMMDD}.txt
if [ ! -s ${FILE_SATPOS} ]; then
  ${SATPOS_COM} -o -s ${SATIMG} -S ${STATION} -d ${DD}/${MM}/${YY} -n 1.2
  stat=$?
  if [ ! ${stat} -eq 0 ]; then
   log_error satpost error ${stat}
   exit 1
  elif [ ! -s ${FILE_SATPOS} ]; then
   log_error ${FILE_SATPOS} does not exists after satpost
   exit 1
  fi
fi

#Now we are able to get the orbit number 
#Use the date and time of data and the Satellite position file
#-------------------------------------------------------------
SDH2ORBNUM_IJC="${DD}/${MM}/${YY}  ${HH}:${MN}:00.000"
NNNNN=`sdh2orbnum.exe <<EOF
${SATIMG}
${SDH2ORBNUM_IJC}
${FILE_SATPOS}
EOF`
stat=$?
if [ ! ${stat} -eq 0 ]; then
   log_error sdh2orbnum error ${stat}
   exit 1
fi

if [ ${NNNNN} -le 0 ] || [ ${NNNNN} -gt 100000 ]; then
   log_error invalid orbit number ${NNNNN}
   exit 1
fi


log_info "Satellite = $SATIMG"
log_info "Date      = $YYYYMMDD"
log_info "Hour      = $HHMN"
log_info "Orbit     = $NNNNN"

#Satellite number
#----------------
SATNB=`echo ${SATIMG} | cut -c5-6`

if [ ${SATNB} -le 14 ];then
  A_TOVS='TOVS'
else 
  A_TOVS='ATOVS'
fi  
log_info "A/TOVS    = $A_TOVS"

#Default instruments and grids
#-----------------------------
if [ $A_TOVS = 'TOVS' ]; then
  INSTRUMENTS=${INSTRUMENTS:-"MSU HIRS AVHRR DCS"}
else
  INSTRUMENTS=${INSTRUMENTS:-"AMSU-A AMSU-B HIRS AVHRR DCS"}
fi
if [[ "$INSTRUMENTS" = *HIRS* ]]; then
  GRIDS=${GRIDS:-"HIRS"}
else
  GRIDS=${GRIDS:-" "}
fi
INSTRUMENTS="$(echo $INSTRUMENTS | sed s/MHS/AMSU-B/)"
GRIDS="$(echo $GRIDS | sed s/MHS/AMSU-B/)"

AMA=0; AMB=0; HRS=0; AVH=0; MSU=0; DCS=0
for INSTR in $INSTRUMENTS; do
  case $INSTR in
    "AMSU-A") AMA=1;;
    "AMSU-B") AMB=1;;
    "HIRS")   HRS=1;;
    "AVHRR")  AVH=1;;
    "MSU")    MSU=1;;
    "DCS")    DCS=1;;
    *)        log_error invalid instrument requested $INSTR
              exit 1;;
  esac
done

#BEWARE Local directory is now $WRK
#====================================
cd ${WRK}
log_info Working directory is ${WRK}

#----------------
#  Decommutation 
#----------------
if [ ${A_TOVS} = 'TOVS' ]; then
  log_info "decom TOVS"
  cat << eof_inp >  decommutation.par
 1,0,0,0,0,$HRS,$MSU,$DCS,0,0,0,$AVH      ! OPTION NUMBERS
10,0,0,0,0,11,12,13,0,0,0,14              ! STREAM NO.S
${YEAR}                                   ! year of the data
0                                         ! operational mode
${NNNNN},${NNNNN}                         ! start and end orbit numbers
eof_inp
fi

if [ ${A_TOVS} = 'ATOVS' ]; then
  log_info "decom ATOVS"
  cat << eof_inp >  decommutation.par
 1,$HRS,$AMA,$AMA,$AMB,0,0,$DCS,0,0,0,$AVH  ! OPTION NUMBERS
10,11,15,15,16,0,0,13,0,0,0,14            ! STREAM NO.S
${YEAR}                                   ! year of the data
0                                         ! operational mode
${NNNNN},${NNNNN}                         ! start and end orbit numbers
eof_inp
fi

decommutation ${A_TOVS} decommutation.par ${FILE}  \
 || { log_error "decommutation failed"; exit 1; }


#------------------------
# Calibration/navigation 
#------------------------

#HIRS
#####



if [ $HRS = 1 ]; then

# Get the hirscl version number which can now be satellite dependent, e.g.
# HIRSCL_SAT="default noaa15"
# HIRSCL_VERSION="1 99"
# If the version number is greater than 1 then no calibration is done. 
#---------------------------------------------------------------------

  i=0
  HIRSCL_VERSION_USE=""
  set -A HVERSION ${HIRSCL_VERSION:-0}  #default 0 (the old algorithm)
  for SAT in ${HIRSCL_SAT:-default}; do
    if [ "${SAT}" = "${SATIMG}" ]; then
      HIRSCL_VERSION_USE=${HVERSION[i]}
    else
      HIRSCL_VERSION_DEF=${HVERSION[i]}
    fi
    i=`expr ${i} + 1`
  done
  HIRSCL_VERSION_USE=${HIRSCL_VERSION_USE:-${HIRSCL_VERSION_DEF}}

  if  [ "${HIRSCL_VERSION_USE}" -gt 1 ]; then     #no calibration, just navigation
    log_info "Running hirscl navigation only"
    hirscl $locflag -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN hrsn.l1b \
      2>hirscl.err || { log_error "hirscl failed"; exit 1; }

  elif [ "${HIRSCL_VERSION_USE}" = 0 ] || [ ${A_TOVS} = 'TOVS' ]; then
   
# the single algorithm available in AAPP1/2/3/4
#----------------------------------------------


#   HIRS l1b contents before HIRS calibration/navigation
    if [ -n "${DEBUG}" ] ; then
      prl1bhirs -s 1 -e 1 hrsn.l1b > prl1bhirs_before_calib.log  \
      || { log_error "prl1bhirs failed"; exit 1; }
    fi

#   HIRS calibration and navigation
    log_info "Running hirscl"
    hirscl $callocflag -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN hrsn.l1b \
    2>hirscl.err || { log_error "hirscl failed"; exit 1; } 

#   HIRS l1b contents after HIRS calibration/navigation
    if [ -n "${DEBUG}" ] ; then
      prl1bhirs -s 1 -e 1 hrsn.l1b > prl1bhirs_after_calib.log  \
      || { log_error "prl1bhirs failed"; exit 1; }
    fi

  else
    if [ "${HIRSCL_VERSION_USE}" = 1 ]; then
# the new algorithm (called algorithm version 4)
#----------------------------------------------- 

#   Manage the historic file. Run this first in case hirscl_algoV4 fails
      FILE_HISTORIC=${PAR_CALIBRATION_MONITOR}/${SATIMG}/hirs_historic.txt
      hirs_historic_file_manage -m $HIST_SIZE_HIGH -r $HIST_SIZE_LOW \
        -n $HIST_NMAX $FILE_HISTORIC

#   Creation of the hirs_b1aslope.txt file in the working directory
      hcalcb1_algoV4 -s $SATIMG -y $YYYY -m $MM -d $DD -h $HH  -n $MN \
      || { log_error "hcalcb1_algoV4 failed"; exit 1; }

#   HIRS l1b contents before HIRS calibration/navigation
      if [ -n "${DEBUG}" ] ; then
        prl1bhirs -s 1 -e 1 hrsn.l1b > prl1bhirs_before_calib.log  \
        || { log_error "prl1bhirs failed"; exit 1; }
      fi

#   HIRS calibration and navigation
      log_info "Running hirscl_algoV4"
      hirscl_algoV4 $callocflag -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN hrsn.l1b \
      2>hirscl_algoV4.err || { log_error "hirscl_algoV4 failed"; exit 1; } 

#   HIRS l1b contents after HIRS calibration/navigation
      if [ -n "${DEBUG}" ] ; then
        prl1bhirs -s 1 -e 1 hrsn.l1b > prl1bhirs_after_calib.log  \
        || { log_error "prl1bhirs failed"; exit 1; }
      fi
   
    else
      log_error "bad value for HIRSCL_VERSION in ATOVS_ENV"
      log_error "HIRS calibration/navigation failed"
      exit 1 
    fi
  fi
fi

#MSU
####

if [ ${A_TOVS} = 'TOVS' ] && [ $MSU = 1 ]; then

#  MSU l1b contents before MSU calibration/navigation
   if [ -n "${DEBUG}" ] ; then
     prl1bmsu msun.l1b > prl1bmsu_before_calib.log  \
     || { log_error "prl1bmsu failed"; exit 1; }
   fi

#  MSU calibration and navigation
   log_info "Running msucl"
   msucl $callocflag -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN msun.l1b \
   || { log_error "msucl failed"; exit 1; }

#  MSU l1b contents after MSU calibration/navigation
   if [ -n "${DEBUG}" ] ; then
     prl1bmsu msun.l1b > prl1bmsu_after_calib.log  \
     || { log_error "prl1bmsu failed"; exit 1; }
   fi
fi

#AMSU 
#####

if [ ${A_TOVS} = 'ATOVS' ]; then
  
#  AMSU-A calibration and navigation
  if [ $AMA = 1 ]; then
    log_info "Running amsuacl"
    amsuacl $callocflag -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN aman.l1b \
    || { log_error "amsuacl failed"; exit 1; }
  fi
  
  if [ $AMB = 1 ]; then
    if [ ${SATNB} -le 17 ];then
#    AMSU-B calibration and navigation   
      log_info "Running amsubcl"     
      amsubcl $callocflag -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN ambn.l1b \
      || { log_error "amsubcl failed"; exit 1; } 

    else
#     MHS calibration and navigation   
      log_info "Running mhscl"   
      mhscl $callocflag -s $SATIMG -d $YYYYMMDD -h $HHMN -n $NNNNN ambn.l1b \
      || { log_error "mhscl failed"; exit 1; } 
    fi
  fi
fi     

#AVHRR
######

if [ $AVH = 1 ]; then
# AVHRR l1b contents before AVHRR calibration/navigation
if [ -n "${DEBUG}" ] ; then
  prl1bavh -s 1 -e 1 hrpt.l1b > prl1bavh_before_calib.log \
  || { log_error "prl1bavh failed"; exit 1; }
fi

# AVHRR calibration and navigation
log_info "Running avhrcl"
avhrcl $callocflag -s $SATIMG -d $YYYYMMDD -h $HHMN  -n $NNNNN hrpt.l1b \
|| { log_error "avhrcl failed"; exit 1; }
  
avh_get_vis_coefs -s $SATIMG -d $YYYYMMDD hrpt.l1b  #operational visible coefs

# AVHRR l1b contents after AVHRR calibration/navigation
if [ -n "${DEBUG}" ] ; then
  prl1bavh -s 1 -e 1 hrpt.l1b > prl1bavh_after_calib.log \
  || { log_error "prl1bavh failed"; exit 1; }
fi
fi

#---------------
# Preprocessing 
#---------------

INSTR1="$(echo $INSTRUMENTS | sed s/AVHRR// | sed s/DCS//)"
if [ "$INSTR1" != "" ]; then
  log_info "Running atovin"
  atovin $INSTR1 || { log_error "atovin failed"; exit 1; }

  if [ "$GRIDS" != "" ]; then
    log_info "Running atovpp"
    atovpp -i "$INSTR1" -g "$GRIDS" || { log_error "atovpp failed"; exit 1; }
  fi

  if [ $AVH = 1 ] && [[ $GRIDS = *HIRS* ]] && [ $AVH2HIRS = Y ]; then
    log_info "avh2hirs ${A_TOVS} started"
    avh2hirs ${A_TOVS} || { log_error "avh2hirs failed"; exit 1; }
    log_info "avh2hirs ${A_TOVS} completed"
  fi
fi

#-----------------------------------
# Now move files with the full name
#-----------------------------------
chmod ugo+rw *.l1b *.l1c *.l1d *.log *.par 2>/dev/null 
if [ $AVH = 1 ]; then
  mv hrpt.l1b $OUTDIR/hrpt_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
fi

if [ $HRS = 1 ]; then
  mv hrsn.l1b $OUTDIR/hirsl1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
  mv hrsn.l1c $OUTDIR/hirsl1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
  if [[ $GRIDS = *HIRS* ]]; then
    mv hirs.l1d $OUTDIR/hirsl1d_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1d
  fi
fi

if [ $AMA = 1 ]; then
  mv aman.l1b $OUTDIR/amsual1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
  mv aman.l1c $OUTDIR/amsual1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
  if [[ $GRIDS = *AMSU-A* ]]; then
    mv aman.l1d $OUTDIR/amsual1d_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1d
  fi
fi

if [ $AMB = 1 ]; then
  if [ ${SATNB} -le 17 ];then
    mv ambn.l1b $OUTDIR/amsubl1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
    mv ambn.l1c $OUTDIR/amsubl1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
    if [[ $GRIDS = *AMSU-B* ]]; then
      mv ambn.l1d $OUTDIR/amsubl1d_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1d
    fi
  else
    mv ambn.l1b $OUTDIR/mhsl1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
    mv ambn.l1c $OUTDIR/mhsl1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
    if [[ $GRIDS = *AMSU-B* ]]; then
      mv ambn.l1d $OUTDIR/mhsl1d_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1d
    fi
  fi
fi

if [ $MSU = 1 ]; then
  mv msun.l1b $OUTDIR/msul1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
  mv msun.l1c $OUTDIR/msul1c_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1c
fi

if [ $DCS = 1 ]; then
  mv dcsn.l1b $OUTDIR/dcsl1b_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.l1b
fi

log_info "creation of a symbolic link for the HRPT file on ${WRK}"
log_info "source ${FILE}"
log_info "target hrpt_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.hrp"
ln -sf ${FILE} $OUTDIR/hrpt_${SATIMG}_${YYYYMMDD}_${HHMN}_${NNNNN}.hrp

log_info "end of processing ${FILE}"
exit
