#!/usr/bin/env python

"""$Id: aapp1c_quicklook.py 626 2019-03-12 10:37:57Z frna $

Quicklook display of AAPP level 1c/1d hdf5 data from sounders

Input: AAPP .h5 file

Output: To screen or png file or both

Projection: Plate Carree (i.e. equirectangular, true at equator)

Requirements: python2.7 or 3.x, with numpy, matplotlib, h5py, cartopy.
If you don't have cartopy, comment out the lines that refer to ccrs; it will still display the BT data.

For usage unstructions, type 

aapp1c_quicklook.py -h  

Works with AAPP-generated l1c.h5 or l1d.h5 files (see convert_to_hdf5), for instruments such as:
  AMSU, MHS, HIRS, ATMS, MWHS2, MWTS2, IRAS, CrIS, IASI

BT will be displayed if available, otherwise radiance (CrIS) or scaled radiance (IASI)
  
COPYRIGHT
   This software was developed within the context of the EUMETSAT Satellite
   Application Facility on Numerical Weather Prediction (NWP SAF), under the
   Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
   Met Office, UK, by one or more partners within the NWP SAF. The partners
   in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.

   Copyright 2018, EUMETSAT, All Rights Reserved.

History:
Version    Date     Comment
1.0    11/03/2019  Initial. NCA
       
"""

import numpy as np
import h5py as h5
import matplotlib.pyplot as plt
import sys
import cartopy.crs as ccrs
from argparse import ArgumentParser

parser = ArgumentParser()
parser.add_argument("-i", "--in", required=True, dest="inputfile", help="input h5 file")
parser.add_argument("-c", "--channel", dest="channel", default="1", help="Channel to display")
parser.add_argument("-r", "--range", dest="latlonrange", default='-180 180 -90 90', help="geographical limit: 'x0 x1 y0 y1', default '-180 180 -90 90'")
parser.add_argument("-s", "--symsize", dest="symsize", default=4, help="symbol size, default 4")
parser.add_argument("-nd", "--no_display", dest="display_image", action="store_false", default=True, help="don't display image on screen")
parser.add_argument("-ns", "--no_save", dest="save_image", action="store_false", default=True, help="don't save image")
parser.add_argument("-a", "--auto", dest="auto_region", action="store_true", default=False, help="auto range for lat/lon")
parser.add_argument("-H", "--high_resolution", dest="high_res", action="store_true", default=False, help="high resolution coastline")

args = parser.parse_args()

# Read arrays from h5 file

fid  =  h5.File(args.inputfile, 'r')
group  =  fid['/']
LATITUDE = 1.0E-4*np.ma.array(group['Geolocation/Latitude'])
LONGITUDE =  1.0E-4*np.ma.array(group['Geolocation/Longitude'])
btok = "/Data/btemps" in fid
radok = "/Data/radiance" in fid
scalradok = "/Data/scalrad" in fid
if btok:
    print ("Reading Data/btemps")
    BT = 1.0E-2*np.ma.array(group['Data/btemps'])
elif radok:
    print ("Reading Data/radiance")
    BT =  1.0E-4*np.ma.array(group['Data/radiance'])    #Convert to mW/m^2/sr/cm^-1
elif scalradok:
    print ("Reading Data/scalrad")
    BT =  1.0*np.ma.array(group['Data/scalrad'])     #no scaling for scalrad
fid.close()    

# Extract the required channel

if (len(BT.shape) == 2):
    image_array = BT[:,int(args.channel)-1]
elif (len(BT.shape) == 3):
    image_array = BT[:,:,int(args.channel)-1]
elif (len(BT.shape) == 4):
    image_array = BT[:,:,:,int(args.channel)-1]

# Convert the lat/lon bounds to an array

if args.auto_region:
    latlonrange = np.fromstring("0 0 0 0", dtype=float, sep=' ')
    latlonrange[0] = np.amin(LONGITUDE)
    latlonrange[1] = np.amax(LONGITUDE)
    latlonrange[2] = np.amin(LATITUDE)
    latlonrange[3] = np.amax(LATITUDE)
else:
    latlonrange = np.fromstring(args.latlonrange, dtype=float, sep=' ')
     
mask = ((LONGITUDE >= latlonrange[0]) & (LONGITUDE <= latlonrange[1]) & 
        (LATITUDE >= latlonrange[2]) & (LATITUDE <= latlonrange[3]) & (image_array >= 0))

LATITUDE=LATITUDE[mask]
LONGITUDE=LONGITUDE[mask]
image_array = image_array[mask]

# Change the default 8x6 inch plot
fig_size = plt.rcParams["figure.figsize"]
fig_size[0] = 18
fig_size[1] = 9
plt.rcParams["figure.figsize"] = fig_size 

# Plot the map
ax = plt.axes(projection=ccrs.PlateCarree())
ax.set_extent(latlonrange,crs=ccrs.PlateCarree())
if args.high_res:
    ax.coastlines('50m')
else:
    ax.coastlines()
gl = ax.gridlines(crs=ccrs.PlateCarree(), linewidth=1, color='gray', linestyle='--', draw_labels=True)
gl.xlabels_top = None

# Plot the data
cmap="jet"       #alternative: "RdBu_r"
plt.scatter(LONGITUDE,LATITUDE,s=int(args.symsize),c=image_array, marker="o", lw=0, cmap=cmap)
plt.title(args.inputfile)

# Draw the colour bar
cbar = plt.colorbar(orientation='vertical')
if btok:
    cbar.ax.set_ylabel('Channel '+args.channel+' BT (K)')
elif radok:
    cbar.ax.set_ylabel('Channel '+args.channel+' radiance (mW/m$^2$/sr/cm$^{-1}$)')
elif scalradok:
    cbar.ax.set_ylabel('Channel '+args.channel+' scaled radiance')

# Save or display image
if args.save_image:
    outfile=args.inputfile+"_"+args.channel+".png"
    plt.savefig(outfile)
    print(("created "+outfile))
if args.display_image:
    plt.show()



