C LIBS = aappbufr atov f7nl1b bufrdummy auxaapp atovin atov
!+ Decode level 1c BUFR data using the ECMWF BUFR library
!
      PROGRAM AAPP_DECODEBUFR_1C
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_decodebufr_1c.F 544 2017-08-15 15:03:58Z frna $
!
! Purpose: Decode BUFR 1c files and write out in AAPP 1c format
!
! Usage:
!    Uses the following enviroment variables:
!    BUFRDEC_FILE - name of BUFR file to decode
!    VERBOSE - Y to print first ob
!    FIRSTSCAN (optional) - first scan line required, numbered from 1 (IASI only)
!    LASTSCAN (optional) - last scan line required, numbered from 1 (IASI only)
!    Output is written to
!        ${FORT}10 for HIRS
!        ${FORT}11 for AMSU-A
!        ${FORT}12 for AMSU-B
!        ${FORT}13 for IASI, where ${FORT} is defined in the ATOVS_ENV file
!        ${FORT}14 for MSU
!        ${FORT}15 for PCIASI
!        ${FORT}14 for CrIS
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0      15/09/05  Created using template "decode_bufr.F" in the ECMWF library
!                    N.C.Atkinson
! 1.1      25/01/06  Skip over unrecognised messages and accept the new IASI
!                    TABLED descriptor. NCA
! 1.2      11/07/06  Accept IASI sequence 340001. NCA
! 1.3      08/09/06  Increase size of JSEC2 and JELEM, for use with BUFR library
!                    version 000310. Correct dimension of KBUFF.
! 1.4      25/01/08  Add initial call to INFDF.
! 1.5      25/11/09  Add the possibility to use 310008 for AMSU and MHS.
!                    Allow user to specify INSTRUMENT, in case the input file
!                    contains more than one instrument. Add MSU. Allow user
!                    to specify a particular WMO_SATID.
! 1.6      12/01/10  Add PCIASI and Enhanced IASI
! 1.7      19/07/10  Add CrIS
! 1.8      19/07/11  Fix compilation errors for null literal string. NCA
! 1.9      15/08/17  Set ENHANCED_IASI correctly when INSTRUMENT is set
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "ama1c.h"
#include "amb1c.h"
#include "hrs1c.h"
#include "msu1c.h"
#include "iasi1c.h"
#include "iasipc.h"
#include "atms1c.h"
#include "cris1c.h"

!Parameters. Note that the size of these parameters must be at least as large
!as the corresponding parameters in the ECMWF library (see bufrdc/parameter.F in
!version 000310, or individual routines e.g. bufrex.F in earlier versions).
!JBYTE must be larger than the longest message (which is IASI).

      INTEGER    JSUP,JSEC0,JSEC1,JSEC2,JSEC3,JSEC4,JELEM
      INTEGER    JBUFL,JBPW,JBYTE
      PARAMETER(JSUP =   9,JSEC0=   3,JSEC1= 40,JSEC2=4096 ,JSEC3=   4,
     &        JSEC4=   2,JELEM=160000,JBUFL=512000,JBYTE=2048000)

#ifdef JBPW_64
      PARAMETER(JBPW =  64)
#else
      PARAMETER(JBPW =  32)
#endif

      INTEGER KDLEN,KELEM,KVALS
      PARAMETER (KDLEN=1,KELEM=20000,KVALS=4096000)
!  Note: IASI 1c has 17676 data elements per ob, so KELEM must be >= this

!Variables

      INTEGER KBUFF(JBYTE)
      INTEGER KSUP(JSUP)  ,KSEC0(JSEC0),KSEC1(JSEC1)
      INTEGER KSEC2(JSEC2),KSEC3(JSEC3),KSEC4(JSEC4)
      INTEGER IRET,JRET,KRET,IERR,KERR  !Error codes
      INTEGER KBUFL          !Length of message
      INTEGER KEL            !Number of elements per ob in values array
      INTEGER NOBS           !Number of obs in a message
      INTEGER NELEM          !Number of elements per ob used
      INTEGER NMESS          !Number of messages
      INTEGER NBYTPW         !Bytes per word
      INTEGER I              !Loop index
      INTEGER NVIND          !Missing value indicator
      INTEGER NSCANS         !Number of scans processed
      INTEGER MODE           !1 to update header record, 0 to update data records
      INTEGER KTDLEN         !Number of descriptors
      INTEGER KTDLST(JELEM)  !List of descriptors
      INTEGER KTDEXL         !Number of entries in list of expanded descriptors
      INTEGER KTDEXP(JELEM)  !Descriptors as integer numbers
      INTEGER IUNIT          !Unit number for input BUFR file
      INTEGER WMO_SATID

      LOGICAL FIRSTMESS      !Used for printing the first message
      DATA FIRSTMESS/.TRUE./

      CHARACTER*256 BUFRDEC_FILE  !Name of file to decode
      CHARACTER*1   VERBOSE
      CHARACTER*64  CNAMES(KELEM) !Descriptor names
      CHARACTER*24  CUNITS(KELEM) !Descriptor units
      CHARACTER*80  CVALS(KELEM)  !For character descriptors (there are none)
      CHARACTER*6   CINSTR        !Instrument name e.g. AMSU-A
      CHARACTER*8   CDESCR        !Descriptor number as 6-character integer
      CHARACTER*6   INSTRUMENT    !Instrument requested, if input file contains
                                  !more than one instrument
      CHARACTER*6   CWMO_SATID    !WMO_SATID requested (optional)

      REAL*8 VALUES(KVALS)        !Array of decoded data

      LOGICAL ENHANCED_IASI       !True if sequence is 3-40-007
      DATA    ENHANCED_IASI /.false./

!-----------------------------------------------------------------------
! Function & Subroutine calls:
!-----------------------------------------------------------------------

      External PBOPEN,PBCLOSE,PBBUFR,BUSEL,BUFREX,BUS012

!-----------------------------------------------------------------------
!- End of header
!=======================================================================

!First read the general satellite info from the fdf.dat file on unit 20

      CALL INFDF (20, 0)

!Get requested instrument name, if one has been specified

      CALL GETENV('INSTRUMENT',INSTRUMENT)
      IF (INSTRUMENT(1:1).NE.' ') write(*,*)'Instrument=',INSTRUMENT
      IF (INSTRUMENT.EQ.'MHS') INSTRUMENT = 'AMSU-B'

!Get requested WMO sat id, if one has been specified

      WMO_SATID = -1
      CALL GETENV('WMO_SATID',CWMO_SATID)
      IF (CWMO_SATID(1:1).NE.' ') THEN
        read(CWMO_SATID,*) WMO_SATID
        write(*,*)'Requested sat id:',WMO_SATID
      ENDIF

!Initialise

      NBYTPW=JBPW/8       !Bytes per word
      IRET=0              !Return code for BUFR file reading
      KRET=0              !Return code for aapp_get_1c
      NMESS=0             !Number of messages
      NSCANS=0            !Number of scans processed
      hrs1c_scnlin=MDI    !Set to an invalid number
      ama1c_scnlin=MDI    !Set to an invalid number
      amb1c_scnlin=MDI    !Set to an invalid number

!Get name of BUFR file from environment variable

      CALL GETENV('BUFRDEC_FILE',BUFRDEC_FILE)
      CALL GETENV('VERBOSE',VERBOSE)

!Open BUFR file

      CALL PBOPEN(IUNIT,BUFRDEC_FILE,'R',IRET)
      IF(IRET.EQ.-1) STOP 'OPEN FAILED'
      IF(IRET.EQ.-2) STOP 'INVALID FILE NAME'
      IF(IRET.EQ.-3) STOP 'INVALID OPEN MODE SPECIFIED'

!Read BUFR message

      DO WHILE (IRET.EQ.0.AND.KRET.EQ.0)
        CALL PBBUFR(IUNIT,KBUFF,JBYTE,KBUFL,IRET) 
        IF(IRET.EQ.-1) WRITE(*,*)'End of BUFR file'
        IF(IRET.EQ.-2) STOP 'FILE HANDLING PROBLEM' 
        IF(IRET.EQ.-3) STOP 'ARRAY TOO SMALL FOR PRODUCT'
        NMESS = NMESS + 1
        write(*,*)'Read message',NMESS,' length',KBUFL

        IF (KBUFL.NE.0) THEN
          KBUFL = KBUFL/NBYTPW + 1      !Number of words

!Expand BUFR Message

          CALL BUS012(KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KSEC2,KERR)
          IF(KERR.NE.0) THEN
            WRITE(*,*)'ERROR IN BUS012: ',KERR
            WRITE(*,*)' BUFR MESSAGE NUMBER ',NMESS,' CORRUPTED.'
          ELSE        

!Decode BUFR Message

            KEL=KVALS/KSUP(6)           !Elements per ob in values array
            IF(KEL.GT.JELEM) KEL=JELEM
            CALL BUFREX(KBUFL,KBUFF,KSUP,KSEC0,KSEC1,KSEC2,KSEC3,KSEC4,
     1            KEL,CNAMES,CUNITS,KVALS,VALUES,CVALS,IERR)

            IF (IERR.NE.0.AND.IERR.NE.39) CALL aappexit(2)
          
            IF (IERR.EQ.39) THEN
              WRITE(*,*)'ERROR 39 IN BUFREX'
            ELSE

              NOBS=KSUP(6)
              NELEM=KSUP(5)
              WRITE(*,*)'Elements:',NELEM,', Subsets:',NOBS

!Get data descriptors and use sequence number to see which instrument we have.
!For IASI, accept either the first TABLEB descriptor (as in the EUMETSAT test
!data) or the TABLED sequence number.

              CALL BUSEL(KTDLEN,KTDLST,KTDEXL,KTDEXP,KERR)

              IF (INSTRUMENT(1:1).NE.' ') THEN
                CINSTR=INSTRUMENT               !User has fixed the instrument
                IF (KTDLST(1).EQ.340007) THEN
                  ENHANCED_IASI = .true.
                ELSE
                  ENHANCED_IASI = .false.
                ENDIF

              ELSE IF (KTDLST(1).EQ.310008) THEN         !HIRS 1c
              
                IF (VALUES(8).EQ.0) THEN
                  CINSTR='HIRS'
                ELSE IF (VALUES(8).EQ.1) THEN
                  CINSTR='MSU'
                ELSE IF (VALUES(8).EQ.2) THEN
                  CINSTR='SSU'
                ELSE IF (VALUES(8).EQ.3) THEN
                  CINSTR='AMSU-A'
                ELSE IF (VALUES(8).EQ.4.OR.VALUES(8).EQ.11) THEN
                  CINSTR='AMSU-B'       !includes MHS
                ELSE
                  WRITE(*,*) 'Instrument code not recognised',VALUES(8)
                  CINSTR='Z'
                ENDIF

              ELSE IF (KTDLST(1).EQ.310009) THEN    !AMSU-A 1c
              
                CINSTR='AMSU-A'

              ELSE IF (KTDLST(1).EQ.310010) THEN    !AMSU-B or MHS 1c
              
                CINSTR='AMSU-B'

              ELSE IF (KTDLST(1).EQ.001007        !IASI: 001007=first element
     &            .OR. KTDLST(1).EQ.361179        !      361179=NOAA test sequence
     &            .OR. KTDLST(1).EQ.340007        !      340007=Enhanced
     &            .OR. KTDLST(1).EQ.340001) THEN  !      340001=Standard
              
                CINSTR='IASI'

                IF (KTDLST(1).EQ.340007) THEN
                  ENHANCED_IASI = .true.
                ELSE
                  ENHANCED_IASI = .false.
                ENDIF

              ELSE IF (KTDLST(1).EQ.340008) THEN  !PCIASI
              
                CINSTR='PCIASI'

              ELSE IF (KTDLST(1).EQ.310060) THEN  !CrIS
              
                CINSTR='CRIS'

              ELSE IF (KTDLST(1).EQ.310061) THEN  !ATMS
              
                CINSTR='ATMS'

              ELSE
              
                WRITE(*,*) 'Sequence not recognised'
                CINSTR='Z'
                
              ENDIF 
              
              IF (CINSTR.NE.INSTRUMENT.AND.INSTRUMENT(1:1).NE.' ') 
     &              CINSTR='Z'
              
              IF (CINSTR.NE.'Z'.AND.CINSTR.NE.'SSU') THEN

!Print out the first observation of the first message

                IF (FIRSTMESS) THEN
                  FIRSTMESS = .FALSE.
                  WRITE(*,*)'KSEC0:',(KSEC0(I),I=1,JSEC0)
                  WRITE(*,*)'KSEC1:',(KSEC1(I),I=1,JSEC1)
                  WRITE(*,*)'KSEC2:',(KSEC2(I),I=1,64)     !limit to 64 words
                  WRITE(*,*)'KSEC3:',(KSEC3(I),I=1,JSEC3)
                  WRITE(*,*)'KSEC4:',(KSEC4(I),I=1,JSEC4)
                  WRITE(*,*)'KSUP:',(KSUP(I),I=1,JSUP)
                  IF (VERBOSE.EQ.'Y') THEN
                    DO I=1,NELEM
                      WRITE(CDESCR,'(2X,I6.6)') KTDEXP(I) 
                      WRITE(*,*) I,CDESCR,VALUES(I)
                    ENDDO
                  ENDIF
                ENDIF

!Fill the AAPP 1c data arrays

                MODE=0
                CALL AAPP_GET_1C(KEL,NOBS,VALUES,NSCANS,CINSTR,
     &            MODE,KRET,WMO_SATID,ENHANCED_IASI,KSEC1(3),KSEC1(16))
                                                !add centre and subcentre

              END IF      !Sequence OK              
            END IF      ! Decode OK
          END IF      ! Expansion OK
        END IF      ! Length >0
      ENDDO  !Loop over BUFR messages
      
!Update the header

      MODE=1
      IF (INSTRUMENT(1:1).NE.' ') CINSTR=INSTRUMENT
      IF (NSCANS.GT.0) THEN
        CALL AAPP_GET_1C(KEL,NOBS,VALUES,NSCANS,CINSTR,MODE,KRET,
     &      WMO_SATID,ENHANCED_IASI,KSEC1(3),KSEC1(16))
      ENDIF
                   
!Close BUFR file

      CALL PBCLOSE(IUNIT,JRET)
      END
