#!/bin/ksh
#
#$Id: aapp_decodebufr_1c.ksh 536 2017-08-09 10:40:30Z frna $
#
#Decode level 1c ATOVS or IASI BUFR files into AAPP binary format, using ECMWF
#BUFR libary
#
#Usage: aapp_decodebufr_1c [-i files] [-v]
#     -i  specifies input file names.
#         Defaults to "hrsn.bufr aman.bufr ambn.bufr mhsn.bufr iasi.bufr"
#     -v  verbose - prints the contents of the first observation
#
#Outputs: AAPP files with .l1c suffix
#
#Environment variable: BUFR_TABLES
#
#History:
# 1.1     04/01/05  initial version. N C Atkinson
# 1.2     25/01/05  refine options
# 1.3     01/02/06  change config file P. Marguinaud
# 1.4     25/01/08  link fdf.dat to unit 20. NCA
# 1.5     26/11/09  Add optional list of instruments (one per file); add MSU.
# 1.6     12/01/10  add PCIASI
# 1.7     19/07/10  add CrIS and ATMS
# 1.8     23/08/12  add MWTS and MWHS
#==============================================================================

usage(){
  echo
  echo 'usage : aapp_decodebufr_1c [-i files] [-v] [instruments]'
  echo
  echo '  where files is a list of files to decode.'
  echo '  Quotes " " are necessary if there is more than 1 file'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

if [ x$BUFR_TABLES = x ]; then
  echo Error: need to define environment variable BUFR_TABLES
  exit 1
fi

if [ ! -d $BUFR_TABLES ]; then
  echo Error: environment variable BUFR_TABLES does not point to a valid directory
  exit 1
fi

files="hrsn.bufr aman.bufr ambn.bufr mhsn.bufr iasi.bufr"   #default files
export VERBOSE=N        #default don't print first ob

while getopts vi: c
do
  case $c in
     v)	export VERBOSE=Y;;
     i)	files=$OPTARG;;
     \?)      usage
              exit 2;;
  esac

done

shift `expr $OPTIND - 1` #Leaves an optional list of instruments (one per file)

rm -f ${FORT}10 ${FORT}11 ${FORT}12 ${FORT}13 ${FORT}14 ${FORT}15 ${FORT}16 ${FORT}17 ${FORT}18 ${FORT}19 ${FORT}21

# fixed data files
ln -sf ${DIR_PREPROC}/fdf.dat ${FORT}20
ln -sf ${DIR_PREPROC}/atms_channels.dat ${FORT}30

for file in $files; do
  if [ -s $file ]; then
    if [ $# != 0 ]; then
      export INSTRUMENT=$1
      shift
    fi
    
    export BUFRDEC_FILE=$file
    echo processing $file
    aapp_decodebufr_1c.exe

    outfile=${file%.*}.l1c      #replace suffix by ".l1c" (except PCIASI)

    if [ -s ${FORT}10 ]; then
      echo created HIRS file $outfile
      mv ${FORT}10 $outfile

    elif [ -s ${FORT}11 ]; then
      echo created AMSU-A file $outfile
      mv ${FORT}11 $outfile

    elif [ -s ${FORT}12 ]; then
      echo created AMSU-B/MHS file $outfile
      mv ${FORT}12 $outfile

    elif [ -s ${FORT}13 ]; then
      echo created IASI file $outfile
      mv ${FORT}13 $outfile

    elif [ -s ${FORT}14 ]; then
      echo created MSU file $outfile
      mv ${FORT}14 $outfile

    elif [ -s ${FORT}15 ]; then
      outfile=${file%.*}.lpc      #replace suffix by ".lpc"
      echo created PCIASI file $outfile
      mv ${FORT}15 $outfile

    elif [ -s ${FORT}16 ]; then
      echo created CrIS file $outfile
      mv ${FORT}16 $outfile

    elif [ -s ${FORT}17 ]; then
      echo created ATMS file $outfile
      mv ${FORT}17 $outfile

    elif [ -s ${FORT}18 ]; then
      echo created MWTS file $outfile
      mv ${FORT}18 $outfile

    elif [ -s ${FORT}19 ]; then
      echo created MWHS file $outfile
      mv ${FORT}19 $outfile

    elif [ -s ${FORT}21 ]; then
      echo created IRAS file $outfile
      mv ${FORT}21 $outfile

    else
      echo Processing failed for $file
    fi

  fi
done
rm -f ${FORT}20 ${FORT}30
exit
