C LIBS = aappbufr atov atovpp f7nl1b f7tp bufrdummy auxaapp
!+ Encode AAPP level 1c data into BUFR using the ECMWF BUFR library
!
      PROGRAM AAPP_ENCODEBUFR_1C
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_encodebufr_1c.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose: Encode AAPP 1c data into BUFR
!
! Description:
!    Instrument to be encoded is held in environment variable ${INSTRUMENT}
!        - options are "HIRS", "AMSU-A", "AMSU-B", "MHS", "IASI"
!    Input 1c files are opened as follows:
!           ${FORT}10 HIRS
!           ${FORT}11 AMSU-A
!           ${FORT}12 AMSU-B/MHS
!           ${FORT}13 IASI
!           ${FORT}14 PCIASI
!           ${FORT}15 CRIS1D/HIRAS1D
!           ${FORT}16 ATMS1D
!           ${FORT}17 CRIS1C
!           ${FORT}18 ATMS1C
!           ${FORT}19 HIRS1D
!           ${FORT}20 IASI1D
!           ${FORT}21 AMSUB1D
!           ${FORT}22 MWTS/MWTS2
!           ${FORT}23 MWHS/MWHS2
!           ${FORT}24 IRAS
!           ${FORT}25 MWRI
!    Output BUFR file is created with file name ${BUFRENC_FILE}
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0      15/09/05  Created using template "create_bufr.F" in the ECMWF library
!                    N.C.Atkinson
! 1.1      25/01/06  Use sequence descriptor for IASI instead of DEFINE_IASI_1C
! 1.2      01/03/06  Use environment variables ORIGINATING_CENTRE and SUB_CENTRE
! 1.3      27/06/06  New sequence number for IASI 340001
! 1.4      08/09/06  Increase size of JSEC2 and JELEM, for use with BUFR library
!                    version 000310
! 1.5      30/06/08  Change message type to 3 (vertical soundings). Local 
!                    subtype defaults to EUMETSAT convention but can be
!                    over-written by $MESSAGE_SUBTYPE. International subtype
!                    is as per Common Code Table C13.
! 1.6      06/03/09  Add PCIASI.
! 1.7      23/11/09  Add Enhanced IASI
! 1.8      13/06/11  Add CRIS1D, ATMS1D, HIRS1D, IASI1D, AMSUB1D
! 1.9      20/09/12  Add MWTS, MWHS. Only read ATMS_THIN if processing ATMS1D. 
! 1.10     15/02/13  Add IRAS
! 1.11     07/06/13  For ATMS and CrIS, use data category 21, sub-categories 6
!                    and 5 respectively, in line with NESDIS data products.
! 1.12     23/05/14  Add MWTS2, MWHS2
! 1.13     19/08/14  Add MWRI by Qifeng Lu
! 1.14     09/07/15  Correct Timeoffset for MWHS2, MWTS2, MWRI
! 1.15     07/04/16  Add MWRI1D
! 1.16     11/10/16  Read environment variable SUB_CENTRE_ID into
!                    ISUBCENTREID and pass it to AAPP_PUT_1C instead of
!                    ISUBCENTRE (which is now used only in Section 1)
! 1.17     20/01/17  Switch to using class 33 quality descriptors for NEDT, as
!                    recommended by IPET-DRMM meeting in July 2015.
! 1.18     03/05/17  Double KVALS (to 8192000) to allow encoding of a whole
!                    CrIS scan, and double JELEM (to 320000) for consistency
!                    with recent versions of BUFRDC.
! 1.19     31/07/17  Add second to Section 1.
! 1.20     19/12/17  Correct TimeOffset for CRIS1D and ATMS1D
! 1.21     13/11/18  For ATMS1D and CRIS1D, revert to data category 3, sub-categories 40
!                    and 30 respectively, for continuity with existing level 1D applications.
! 1.22     11/02/19  Add HIRAS1D
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "ama1c.h"
#include "amb1c.h"
#include "hrs1c.h"
#include "iasi1c.h"
#include "iasipc.h"
#include "cris1c.h"
#include "atms1c.h"
#include "cris1d.h"
#include "hiras1d.h"
#include "atms1d.h"
#include "iasi1d.h"
#include "amb1d.h"
#include "mwts1c.h"
#include "mwhs1c.h"
#include "iras1c.h"
#include "mwts21c.h"
#include "mwhs21c.h"
#include "mwri1c.h"

!Parameters. Note that the size of these parameters must be at least as large
!as the corresponding parameters in the ECMWF library (see bufrdc/parameter.F in
!version 000310, or individual routines e.g. bufren.F in earlier versions)

      INTEGER    JSUP,JSEC0,JSEC1,JSEC2,JSEC3,JSEC4,JELEM
      INTEGER    JBUFL,JBPW
      PARAMETER(JSUP =   9,JSEC0=   3,JSEC1= 40,JSEC2=4096 ,JSEC3=    4,
     &        JSEC4=   2,JELEM=320000,JBUFL=512000)

#ifdef JBPW_64
      PARAMETER(JBPW =  64)
#else
      PARAMETER(JBPW =  32)
#endif

      INTEGER KDLEN,KELEM,KVALS
      PARAMETER (KDLEN=4,KELEM=20000,KVALS=8192000)
!  Note: IASI 1c has 17676 data elements per ob, so KELEM must be >= this
 
!Variables

      INTEGER KBUFR(JBUFL)
      INTEGER KDATA(KDLEN)
      INTEGER KSEC0(JSEC0),KSEC1(JSEC1)
      INTEGER KSEC2(JSEC2),KSEC3(JSEC3),KSEC4(JSEC4)
      INTEGER KBUFL          !Length of BUFR message in words
      INTEGER IRET,IERR,KERR !Error codes
      INTEGER NOBS           !Number of obs in a message
      INTEGER NELEM          !Number of elements per ob
      INTEGER NMESS          !Number of messages
      INTEGER NBYTPW         !Bytes per word
      INTEGER I              !Loop index
      INTEGER NVIND          !Missing value indicator
      INTEGER KPRUS          !Flag for re-using pointers
      INTEGER KPMISS         !Flag for handling out of range data
      INTEGER ILEN           !Number of bytes in message
      INTEGER KTDLEN         !Number of descriptors
      INTEGER KTDLST(JELEM)  !List of descriptors
      INTEGER IUNIT          !Unit number for output BUFR file
      INTEGER NOKEY          !For ECMWF RDB key
      INTEGER ICENTREID      !Centre code (1b/1c in Section 4)
      INTEGER ISUBCENTREID   !Centre code (1b/1c in Section 4)
      INTEGER IORIGCENTRE    !Centre code (Section 1)
      INTEGER ISUBCENTRE     !sub-centre code (Section 1)
      INTEGER IMASTER        !Version of master table
      INTEGER ILOCAL         !Version of local table
      INTEGER IEDITION       !BUFR edition number
      INTEGER IMESSAGE_SUBTYPE !Message subtype (locally defined)
      INTEGER ITHIN          !Number of samples/scans to increment
                             !If negative, only applied to samples
      INTEGER AAPP_VERSION   !e.g. 715 for v7.15
      LOGICAL ENHANCED_IASI
      LOGICAL USE_OB_TIME    !Use ob time or system time for Section 1
      INTEGER TimeOffset
      INTEGER YYYYMMDD(3)    !Year month day
      INTEGER hhmmss(3)      !Hour minute second

      CHARACTER*256 BUFRENC_FILE  !Name of BUFR file to create
      CHARACTER*80  CVALS(KELEM)  !For character descriptors (there are none)
      CHARACTER*3   CENTRE_ID     !Environment variable for centre code (1b/1c)
      CHARACTER*3   SUB_CENTRE_ID !Environment variable for centre code (1b/1c)
      CHARACTER*5   ORIGINATING_CENTRE  !Orig centre for section 1
      CHARACTER*5   SUB_CENTRE          !Sub-centre for section 1
      CHARACTER*3   MASTER_TABLE        !Master table version number
      CHARACTER*3   LOCAL_TABLE         !Local table version number
      CHARACTER*3   BUFR_EDITION        !BUFR edition number
      CHARACTER*3   MESSAGE_SUBTYPE     !Message subtype (locally defined)
      CHARACTER*3   CENHANCED_IASI      !'Y' or 'N'
      CHARACTER*3   CUSE_OB_TIME        !'Y' or 'N'
      CHARACTER*3   CATMS_THIN          !1 or 3 normally
      CHARACTER*3   CMWTS2_THIN         !1 or 3 normally
      CHARACTER*3   CMWRI_THIN         !1 or 3 normally
      CHARACTER*3   CMWHS2_THIN         !1 or 3 normally
      CHARACTER*3   CIRAS_THIN          !1 or 3 normally
      CHARACTER*5   CAAPP_VERSION

      CHARACTER*7   INSTRUMENT  !Instrument to encode (HIRS, AMSU-A, AMSU-B/MHS)
      CHARACTER*1   CENCODE_NEDT
      LOGICAL       ENCODE_NEDT

      REAL*8 VALUES(KVALS) !Array of data values
      REAL*8 RVIND         !Missing value indicator
      
!-----------------------------------------------------------------------
! Function & Subroutine calls:
!-----------------------------------------------------------------------

      External PBOPEN,PBCLOSE,BUPRQ,BUFREN,PBWRITE

!-----------------------------------------------------------------------
!- End of header
!=======================================================================

!Get instrument name

      CALL GETENV('INSTRUMENT',INSTRUMENT)
      write(*,*)'Instrument=',instrument
      CALL GETENV('ENCODE_NEDT',CENCODE_NEDT)
      IF (CENCODE_NEDT(1:1).EQ.'Y') THEN
        ENCODE_NEDT = .true.
      ELSE
        ENCODE_NEDT = .false.
      ENDIF

!Get environment variables if provided, otherwise set defaults

      IORIGCENTRE = 255     !255=Missing
      ISUBCENTRE = 0
      ICENTREID = 255
      ISUBCENTREID = 255
      IF (ENCODE_NEDT) THEN
        IMASTER = 26
      ELSE
        IMASTER = 13
      ENDIF
      ILOCAL = 0
      IEDITION = 4
      IMESSAGE_SUBTYPE = -1
      ITHIN = 1            !Default is to encode every spot (ATMS,MWTS2)
      ENHANCED_IASI = .false.
      USE_OB_TIME = .false.
      AAPP_VERSION = 0
      CALL GETENV('ORIGINATING_CENTRE',ORIGINATING_CENTRE)
      READ(ORIGINATING_CENTRE,*,ERR=99,END=99) IORIGCENTRE
   99 CALL GETENV('SUB_CENTRE',SUB_CENTRE)
      READ(SUB_CENTRE,*,ERR=100,END=100) ISUBCENTRE
  100 CALL GETENV('CENTRE_ID',CENTRE_ID)
      READ(CENTRE_ID,*,ERR=101,END=101) ICENTREID
  101 CALL GETENV('SUB_CENTRE_ID',SUB_CENTRE_ID)
      READ(SUB_CENTRE_ID,*,ERR=102,END=102) ISUBCENTREID
  102 CALL GETENV('MASTER_TABLE',MASTER_TABLE)
      READ(MASTER_TABLE,*,ERR=103,END=103) IMASTER
  103 CALL GETENV('LOCAL_TABLE',LOCAL_TABLE)
      READ(LOCAL_TABLE,*,ERR=104,END=104) ILOCAL
  104 CALL GETENV('BUFR_EDITION',BUFR_EDITION)
      READ(BUFR_EDITION,*,ERR=105,END=105) IEDITION
  105 CALL GETENV('MESSAGE_SUBTYPE',MESSAGE_SUBTYPE)
      READ(MESSAGE_SUBTYPE,*,ERR=106,END=106) IMESSAGE_SUBTYPE
  106 CALL GETENV('ENHANCED_IASI',CENHANCED_IASI)
      IF (CENHANCED_IASI(1:1) .EQ. 'Y') ENHANCED_IASI = .true.
  107 CALL GETENV('USE_OB_TIME',CUSE_OB_TIME)
      IF (CUSE_OB_TIME(1:1) .EQ. 'Y') USE_OB_TIME = .true.
  108 IF (INSTRUMENT.EQ. 'ATMS1D') THEN
        CALL GETENV('ATMS_THIN',CATMS_THIN)
        READ(CATMS_THIN,*,ERR=109,END=109) ITHIN
        WRITE(*,*)'ATMS_THIN=',ITHIN
      ENDIF
  109 IF (INSTRUMENT.EQ.'MWTS21D'.OR.INSTRUMENT.EQ.'MWTS2') THEN
        CALL GETENV('MWTS2_THIN',CMWTS2_THIN)
        READ(CMWTS2_THIN,*,ERR=110,END=110) ITHIN
        WRITE(*,*)'CMWTS2_THIN=',ITHIN
      ENDIF
  110 IF (INSTRUMENT.EQ.'MWHS21D'.OR.INSTRUMENT.EQ.'MWHS2') THEN
        CALL GETENV('MWHS2_THIN',CMWHS2_THIN)
        READ(CMWHS2_THIN,*,ERR=111,END=111) ITHIN
        WRITE(*,*)'CMWHS2_THIN=',ITHIN
      ENDIF
  111 IF (INSTRUMENT.EQ.'IRAS1D'.OR.INSTRUMENT.EQ.'IRAS') THEN
        CALL GETENV('IRAS_THIN',CIRAS_THIN)
        READ(CIRAS_THIN,*,ERR=112,END=112) ITHIN
        WRITE(*,*)'CIRAS_THIN=',ITHIN
      ENDIF
  112 IF (INSTRUMENT.EQ.'MWRI1D'.OR.INSTRUMENT.EQ.'MWRI') THEN
        CALL GETENV('MWRI_THIN',CMWRI_THIN)
        READ(CMWRI_THIN,*,ERR=113,END=113) ITHIN
        WRITE(*,*)'CMWRI_THIN=',ITHIN
      ENDIF
  113 CALL GETENV('AAPP_VERSION',CAAPP_VERSION)
      READ(CAAPP_VERSION,*,ERR=114,END=114) AAPP_VERSION
  114 CONTINUE
  
!Define BUFR sequence

      IF (INSTRUMENT.EQ.'HIRS') THEN

        KTDLST(1) = 310008
        IF (ENCODE_NEDT) THEN
          KTDLST(2) = 222000 !Quality information follows
          KTDLST(3) = 236000 !Define data present bit-map
          KTDLST(4) = 101156 !Repeat 1 descriptor 156 times
          KTDLST(5) = 031031 !Data present indicator, 0=yes, 1=no
          KTDLST(6) = 001031 !Originating centre for quality information
          KTDLST(7) = 001032 !Generating application
          KTDLST(8) = 101019 !Repeat 1 descriptor 19 times
          KTDLST(9) = 033089 !NEDT for warm target
          KTDLEN = 9
        ELSE
          KTDLEN = 1
        ENDIF
        NELEM = 1000   !Make this smaller than KELEM so that we can encode
                       !many scans if required

      ELSE IF (INSTRUMENT.EQ.'AMSU-A') THEN
      
        KTDLST(1) = 310009
        IF (ENCODE_NEDT) THEN
          KTDLST(2) = 222000 !Quality information follows
          KTDLST(3) = 236000 !Define data present bit-map
          KTDLST(4) = 101127 !Repeat 1 descriptor 127 times
          KTDLST(5) = 031031 !Data present indicator, 0=yes, 1=no
          KTDLST(6) = 001031 !Originating centre for quality information
          KTDLST(7) = 001032 !Generating application
          KTDLST(8) = 101015 !Repeat 1 descriptor 15 times
          KTDLST(9) = 033089 !NEDT for warm target
          KTDLEN = 9
        ELSE
          KTDLEN = 1
        ENDIF
        NELEM = 1000

      ELSE IF (INSTRUMENT.EQ.'AMSU-B'.OR.INSTRUMENT.EQ.'MHS') THEN
      
        KTDLST(1) = 310010
        IF (ENCODE_NEDT) THEN
          KTDLST(2) = 222000 !Quality information follows
          KTDLST(3) = 236000 !Define data present bit-map
          KTDLST(4) = 101067 !Repeat 1 descriptor 67 times
          KTDLST(5) = 031031 !Data present indicator, 0=yes, 1=no
          KTDLST(6) = 001031 !Originating centre for quality information
          KTDLST(7) = 001032 !Generating application
          KTDLST(8) = 101005 !Repeat 1 descriptor 5 times
          KTDLST(9) = 033089 !NEDT for warm target
          KTDLEN = 9
        ELSE
          KTDLEN = 1
        ENDIF
        NELEM = 1000
        
      ELSE IF (INSTRUMENT.EQ.'IASI') THEN
      
        IF (ENHANCED_IASI) THEN
          KTDLST(1) = 340007
        ELSE
          KTDLST(1) = 340001    !361179 with NOAA data
        ENDIF
        KTDLEN = 1
C        CALL DEFINE_IASI_1C(JELEM,KTDLST,KTDLEN)  !Use this if TABLED does not
C                                                  !contain 340001
        NELEM = KELEM   !Need the full allocation for IASI
        
      ELSE IF (INSTRUMENT.EQ.'PCIASI') THEN
      
        KTDLST(1) = 340008
        KTDLEN = 1
        NELEM = KELEM
        
      ELSE IF (INSTRUMENT.EQ.'CRIS') THEN
      
        KTDLST(1) = 310060
        KTDLEN = 1
        NELEM = KELEM
        
      ELSE IF (INSTRUMENT.EQ.'ATMS') THEN
      
        KTDLST(1) = 310061
        KTDLEN = 1
        NELEM = 1000
        
      ELSE IF (INSTRUMENT.EQ.'CRIS1D'.OR.INSTRUMENT.EQ.'ATMS1D'
     & .OR.INSTRUMENT.EQ.'HIRAS1D') THEN
      
        NELEM = 10000     !Up to 270 obs, so must be smaller than KVALS/270
        CALL DEFINE_CRIS_1D(JELEM,KTDLST,KTDLEN)

      ELSE IF (INSTRUMENT.EQ.'HIRS1D'.OR.INSTRUMENT.EQ.'HIRS1DM') THEN
      
        NELEM = KELEM
        CALL DEFINE_HIRS_1D(JELEM,KTDLST,KTDLEN)

      ELSE IF (INSTRUMENT.EQ.'AMSUB1D') THEN
      
        NELEM = KELEM
        CALL DEFINE_AMSUB_1D(JELEM,KTDLST,KTDLEN)

      ELSE IF (INSTRUMENT.EQ.'IASI1D') THEN
      
        NELEM = KELEM
        CALL DEFINE_IASI_1D(JELEM,KTDLST,KTDLEN)

      ELSE IF (INSTRUMENT.EQ.'MWTS' .OR. INSTRUMENT.EQ.'MWHS'
     &    .OR. INSTRUMENT.EQ.'IRAS' .OR. INSTRUMENT.EQ.'MWTS2'
     &    .OR. INSTRUMENT.EQ.'MWHS2' .OR. INSTRUMENT.EQ.'MWRI') THEN
      
        IF (INSTRUMENT.EQ.'MWRI') THEN
        NELEM = 1000
        ELSE
        NELEM = KELEM
        ENDIF
        CALL DEFINE_MWTS_MWHS_1C(JELEM,KTDLST,KTDLEN)

      ELSE IF (INSTRUMENT.EQ.'MWTS21D' .OR. INSTRUMENT.EQ.'MWHS21D'
     &    .OR. INSTRUMENT.EQ.'IRAS1D' .OR. INSTRUMENT.EQ.'MWRI1D') THEN
        NELEM = KELEM
        CALL DEFINE_FY3_GENERIC_1D(JELEM,KTDLST,KTDLEN)

      ELSE
      
        WRITE(*,*) 'Error: INSTRUMENT not recognised:',INSTRUMENT
        STOP
        
      ENDIF

!Add AAPP version number if this has been defined, plus CSPP version number for ATMS/CrIS

      IF (AAPP_VERSION .NE. 0) THEN
        KTDLEN = KTDLEN + 1
        KTDLST(KTDLEN) = 025060    !Software identification (numeric, 14 bits, max 16382)
        IF (INSTRUMENT.EQ.'ATMS'.OR.INSTRUMENT.EQ.'CRIS') THEN
          KTDLEN = KTDLEN + 1
          KTDLST(KTDLEN) = 025061    !Software identification and version number (12 character)
        ENDIF
      ENDIF

!Initialise

      RVIND = 1.7E38      !Missing value
      NMESS=0             !Number of messages
       
!Fill in the fixed parts of the message

C
C     SECTION 0 CONTENT
C
      KSEC0(1)=0      ! TOTAL LENGTH OF SECTION 0
      KSEC0(2)=0      ! TOTAL LENGTH OF BUFR MESSAGE
      KSEC0(3)=IEDITION      ! BUFR EDITION NUMBER
C
C     SECTION 1 CONTENT
C
      IF (IEDITION.GE.4) THEN
        KSEC1(1)=22    ! TOTAL LENGTH OF SECTION 1 (set to 18 for edition <= 3)
      ELSE
        KSEC1(1)=18
      ENDIF
      KSEC1(2)=IEDITION     ! BUFR EDITION NUMBER
      KSEC1(3)=IORIGCENTRE   
                     ! ORIGINATING CENTRE, e.g. 254=EUMETSAT, 98=ECMWF, 74=UKMO
                     ! See "Common Code Tables to Binary and Alphanumeric Codes"
                     ! at http://www.wmo.int/web/www/WMOCodes.html
                     ! NB: if the version number of the local table is 0 or 255
                     ! then this program will use ECMWF BUFR tables; otherwise
                     ! it will look for tables of the originating centre.
      KSEC1(4)=0     ! UPDATE SEQUENCE NUMBER
      KSEC1(5)=0     ! FLAG (PRESENCE OF SECTION 2), 0=no, 128=yes
      KSEC1(6)=3     ! BUFR MESSAGE TYPE, 21=radiances (satellite measured)
                     !                     3=vertical soundings (satellite)
      IF (INSTRUMENT.EQ.'CRIS'.OR.INSTRUMENT.EQ.'ATMS') THEN
        KSEC1(6)=21     ! BUFR MESSAGE TYPE, 21=radiances (satellite measured)
      ENDIF

         ! If no message subtype is given, replicate the EUMETSAT convention
      
      IF (IMESSAGE_SUBTYPE .LT. 0) THEN
        IF (INSTRUMENT.EQ.'HIRS'.OR.INSTRUMENT.EQ.'HIRS1D'
     &      .OR.INSTRUMENT.EQ.'HIRS1DM') THEN
          KSEC1(7)=0
        ELSE IF (INSTRUMENT.EQ.'AMSU-A') THEN
          KSEC1(7)=3
        ELSE IF (INSTRUMENT.EQ.'AMSU-B') THEN
          KSEC1(7)=4
        ELSE IF (INSTRUMENT.EQ.'MHS') THEN
          KSEC1(7)=11
        ELSE IF (INSTRUMENT.EQ.'IASI'.OR.INSTRUMENT.EQ.'PCIASI'
     &       .OR.INSTRUMENT.EQ.'IASI1D') THEN
          KSEC1(7)=221
        ELSE IF (INSTRUMENT.EQ.'CRIS'.OR.INSTRUMENT.EQ.'CRIS1D') THEN
          KSEC1(7)=202     !value from NOAA
        ELSE IF (INSTRUMENT.EQ.'ATMS'.OR.INSTRUMENT.EQ.'ATMS1D') THEN
          KSEC1(7)=203     !value from NOAA
        ELSE IF (INSTRUMENT.EQ.'MWTS'.OR.INSTRUMENT.EQ.'MWHS'
     &       .OR.INSTRUMENT.EQ.'IRAS') THEN
          KSEC1(7)=154     !ECMWF
        ELSE IF (INSTRUMENT.EQ.'MWRI') THEN
          KSEC1(7)=153     !ECMWF
        ELSE IF (INSTRUMENT.EQ.'MWTS21D'.OR.INSTRUMENT.EQ.'MWHS21D')THEN
          KSEC1(7)=0
        ELSE IF (INSTRUMENT.EQ.'IRAS1D'.OR.INSTRUMENT.EQ.'HIRAS1D')THEN
          KSEC1(7)=0
        ELSE IF (INSTRUMENT.EQ.'MWTS2'.OR.INSTRUMENT.EQ.'MWHS2')THEN
          KSEC1(7)=0
        ELSE IF (INSTRUMENT.EQ.'MWRI1D')THEN
          KSEC1(7)=0
        ENDIF
      ELSE
        KSEC1(7) = IMESSAGE_SUBTYPE
      ENDIF

      KSEC1(8)=ILOCAL! VERSION NUMBER OF LOCAL TABLE USED
                     !    Use EUMETSAT table 254011001 in order to get IASI
                     ! KSEC1(9) to (13) contain date and time (see below)
      KSEC1(14)=0    ! BUFR MASTER TABLE (ZERO) FOR METEOROLOGICAL DATA
      KSEC1(15)=IMASTER       ! VERSION NUMBER OF MASTER TABLE USED
      KSEC1(16)=ISUBCENTRE    ! ORIGINATING SUB-CENTRE

         ! The international sub-category in BUFR edition 4 is defined in 
         ! Common Code Table C13

      IF (IEDITION .GE. 4 )THEN
        IF (INSTRUMENT.EQ.'HIRS'.OR.INSTRUMENT.EQ.'HIRS1D'
     &     .OR.INSTRUMENT.EQ.'HIRS1DM') THEN
          KSEC1(17)=5
        ELSE IF (INSTRUMENT.EQ.'AMSU-A') THEN
          KSEC1(17)=3
        ELSE IF (INSTRUMENT.EQ.'AMSU-B'.OR.INSTRUMENT.EQ.'AMSUB1D') THEN
          KSEC1(17)=4
        ELSE IF (INSTRUMENT.EQ.'MHS') THEN
          KSEC1(17)=6
        ELSE IF (INSTRUMENT.EQ.'IASI'.OR.INSTRUMENT.EQ.'PCIASI'
     &       .OR.INSTRUMENT.EQ.'IASI1D') THEN
          KSEC1(17)=7
        ELSE IF (INSTRUMENT.EQ.'CRIS') THEN
          KSEC1(17)=5      !Sub-category of type 21: 5=CrIS
        ELSE IF (INSTRUMENT.EQ.'ATMS') THEN
          KSEC1(17)=6      !Sub-category of type 21: 6=ATMS
        ELSE IF (INSTRUMENT.EQ.'MWTS'.OR.INSTRUMENT.EQ.'MWHS'
     &       .OR.INSTRUMENT.EQ.'MWTS21D'.OR.INSTRUMENT.EQ.'MWHS21D'
     &       .OR.INSTRUMENT.EQ.'MWRI'.OR.INSTRUMENT.EQ.'MWRI1D' 
     &       .OR.INSTRUMENT.EQ.'MWTS2'.OR.INSTRUMENT.EQ.'MWHS2'
     &       .OR.INSTRUMENT.EQ.'ATMS1D') THEN
          KSEC1(17)=40     !Sub-category of type 3: 40=microwave temp/humidity sounding
        ELSE IF (INSTRUMENT.EQ.'CRIS1D'.OR.INSTRUMENT.EQ.'HIRAS1D') THEN
          KSEC1(17)=30     !Sub-category of type 3: 30=hyper-spectral temp/humidity sounding
        ELSE IF (INSTRUMENT.EQ.'IRAS'.OR.INSTRUMENT.EQ.'IRAS1D') THEN
          KSEC1(17)=20     !Sub-category of type 3: 20= IR temp/humidity sounding
        ENDIF
      ELSE
        KSEC1(17)=0    ! International sub-category (not used in Edition 3)
      ENDIF
      KSEC1(18)=0    ! Local ADP centre information
C
C     SECTION 2 CONTENT
C
      KSEC2(1)=52    ! TOTAL LENGTH OF SECTION 2 
                     !(52 when used for Report Data Base key)
      DO I=2,JSEC2
        KSEC2(I)=0
      ENDDO
C
C     SECTION 3 CONTENT
C
      KSEC3(1)=0     ! TOTAL LENGTH OF SECTION 3
      KSEC3(2)=0     ! RESERVED
      KSEC3(3)=0     ! Number of subsets (see below)
      KSEC3(4)=192   ! 128(observed) + 64(compressed)
!      KSEC3(4)=128   ! 128(observed)

!Set KPMISS,KPRUS into common block

      KPMISS=1   !1=Pack out-of-range data as missing (0=pack max value)
      KPRUS=1    !1=Always re-calculate pointers (0=re-use) 
      NOKEY=0    !0=Packs ECMWF RDB key (1=packs section 2 if needed but not 
                 !                       length of BUFR message in key)
      CALL BUPRQ(KPMISS,KPRUS,NOKEY)   

!Get name of BUFR file from environment variable

      CALL GETENV('BUFRENC_FILE',BUFRENC_FILE)

!Open BUFR file

      CALL PBOPEN(IUNIT,BUFRENC_FILE,'W',IRET)
      IF(IRET.EQ.-1) STOP 'OPEN FAILED'
      IF(IRET.EQ.-2) STOP 'INVALID FILE NAME'
      IF(IRET.EQ.-3) STOP 'INVALID OPEN MODE SPECIFIED'

!Main loop to create BUFR messages
      
      NOBS = -1
      DO WHILE (NOBS.NE.0)
      
!Read AAPP input files and fill the "VALUES" array

        CALL AAPP_PUT_1C(NELEM,NOBS,ICENTREID,ISUBCENTREID,VALUES,
     &     INSTRUMENT,ENHANCED_IASI,ITHIN,ENCODE_NEDT,AAPP_VERSION,
     &     CVALS)
     
        IF (NOBS.GT.0) THEN
          
          NMESS = NMESS + 1
          KSEC3(3)=NOBS       ! Number of subsets
          IF (INSTRUMENT.EQ.'PCIASI') THEN   !Delayed replication factors
            KDATA(1) = iasipc_h_nchanused
            DO I=1,3
              KDATA(I+1)=iasipc_h_endpc(I)-iasipc_h_startpc(I)+1
            ENDDO
          ELSE IF (INSTRUMENT.EQ.'CRIS1D') THEN
            KDATA(1) = cris1d_h_nchanused
            KDATA(2) = cris1d_h_npcused
            KDATA(3) = cris1d_h_nviirschanused
          ELSE IF (INSTRUMENT.EQ.'HIRAS1D') THEN
            KDATA(1) = hiras1d_h_nchanused
            KDATA(2) = hiras1d_h_npcused
            KDATA(3) = hiras1d_h_nmersichanused
          ELSE IF (INSTRUMENT.EQ.'MWTS21D') THEN
            KDATA(1) = mwts2_nchan
            KDATA(2) = mwts21c_h_nmapchanused
            KDATA(3) = 0
          ELSE IF (INSTRUMENT.EQ.'MWHS21D') THEN
            KDATA(1) = mwhs2_nchan
            KDATA(2) = mwhs21c_h_nmapchanused
            KDATA(3) = 0
          ELSE IF (INSTRUMENT.EQ.'MWRI1D') THEN
            KDATA(1) = mwri_nchan
            KDATA(2) = 0
            KDATA(3) = 0
          ELSE IF (INSTRUMENT.EQ.'IRAS1D') THEN
            KDATA(1) = iras_nchan
            KDATA(2) = iras1c_h_nmapchanused_mwts
            KDATA(3) = iras1c_h_nmapchanused_mwhs
          ELSE IF (INSTRUMENT.EQ.'CRIS') THEN
            KDATA(1) = cris1c_h_nchanused
          ELSE IF (INSTRUMENT.EQ.'ATMS') THEN
            KDATA(1) = atms1c_h_nchan
          ELSE IF (INSTRUMENT.EQ.'IASI1D') THEN
            KDATA(1) = iasi1d_h_nchanused
            KDATA(2) = iasi1d_h_npcused
          ELSE IF (INSTRUMENT.EQ.'MWTS') THEN
            KDATA(1) = 4
          ELSE IF (INSTRUMENT.EQ.'MWHS') THEN
            KDATA(1) = 5
          ELSE IF (INSTRUMENT.EQ.'MWTS2') THEN
            KDATA(1) = mwts2_nchan
          ELSE IF (INSTRUMENT.EQ.'MWRI') THEN
            KDATA(1) = mwri_nchan
          ELSE IF (INSTRUMENT.EQ.'MWHS2') THEN
            KDATA(1) = mwhs2_nchan
          ELSE IF (INSTRUMENT.EQ.'IRAS') THEN
            KDATA(1) = iras_nchan
          ELSE
            DO I=1,KDLEN
              KDATA(I)=0   !Delayed replication not used
            ENDDO
          ENDIF

!Put date and time into Section 1

          IF (INSTRUMENT.EQ.'IASI'.OR.INSTRUMENT.EQ.'PCIASI'
     &        .OR.INSTRUMENT.EQ.'CRIS') THEN
            TimeOffset=5
          ELSE IF (INSTRUMENT.EQ.'ATMS')THEN
            TimeOffset=6
          ELSE IF (INSTRUMENT.EQ.'CRIS1D'.OR.INSTRUMENT.EQ.'ATMS1D'
     &        .OR.INSTRUMENT.EQ.'HIRAS1D')THEN
            TimeOffset=9
          ELSE IF (INSTRUMENT.EQ.'MWTS'.OR.INSTRUMENT.EQ.'MWHS'
     &        .OR.INSTRUMENT.EQ.'MWTS2'.OR.INSTRUMENT.EQ.'MWHS2'
     &        .OR.INSTRUMENT.EQ.'MWRI'.OR.INSTRUMENT.EQ.'MWRI1D'
     &        .OR.INSTRUMENT.EQ.'IRAS'.OR.INSTRUMENT.EQ.'MWTS21D'
     &        .OR.INSTRUMENT.EQ.'MWHS21D'.OR.INSTRUMENT.EQ.'IRAS1D')THEN
            TimeOffset=8
          ELSE IF (INSTRUMENT.EQ.'HIRS1D'.OR.INSTRUMENT.EQ.'IASI1D'
     &       .OR.INSTRUMENT.EQ.'AMSUB1D'.OR.INSTRUMENT.EQ.'HIRS1DM')THEN
            TimeOffset=2
          ELSE
            TimeOffset=16
          ENDIF
          IF (USE_OB_TIME) THEN
            KSEC1(9)=VALUES(TimeOffset)     !Year 
            KSEC1(10)=VALUES(TimeOffset+1)  !Month 
            KSEC1(11)=VALUES(TimeOffset+2)  !Day 
            KSEC1(12)=VALUES(TimeOffset+3)  !Hour 
            KSEC1(13)=VALUES(TimeOffset+4)  !Minute
            IF (IEDITION.GE.4) KSEC1(18)=VALUES(TimeOffset+5) 
          ELSE
            CALL GDG(YYYYMMDD)  !date gregorienne
            CALL GHG(hhmmss)    !heure gregorienne
            KSEC1(9)=YYYYMMDD(1)
            KSEC1(10)=YYYYMMDD(2)
            KSEC1(11)=YYYYMMDD(3)
            KSEC1(12)=hhmmss(1)
            KSEC1(13)=hhmmss(2)
            IF (IEDITION.GE.4) KSEC1(18)=hhmmss(3)
          ENDIF
          
          if(KSEC1(2).le.3) then
            if(ksec1(9).gt.2000) then
               ksec1(9)=ksec1(9)-2000  !2 digit year for BUFR edition 3
            else                       !  and earlier
               ksec1(9)=ksec1(9)-1900
            end if
          end if

!Encode the message

          KERR=0
          CALL BUFREN( KSEC0,KSEC1,KSEC2,KSEC3,KSEC4,
     1             KTDLEN,KTDLST,KDLEN,KDATA,NELEM,
     2             KVALS,VALUES,CVALS,KBUFL,KBUFR,KERR)

          IF(KERR.GT.0) THEN
            CALL aappexit(2)
          ELSEIF(KERR.lt.0) then
            print*,'Encoding return_code=',kerr
          END IF 

          ILEN=KBUFL*JBPW/8     !Length of BUFR message in bytes

!Write to output file

          IERR=0
          CALL PBWRITE(IUNIT,KBUFR,ILEN,IERR)
          IF(IERR.LT.0) THEN
            PRINT*,'ERROR WRITING INTO TARGET FILE.'
            CALL aappexit(2)
          ELSE
            WRITE(*,*)'Written BUFR message',NMESS,' NOBS=',NOBS
          END IF

        ENDIF  !NOBS.NE.0
      ENDDO  !Loop over BUFR messages

!Close output file

      CALL PBCLOSE(IUNIT,IRET)
      END
