#!/bin/ksh
#
#$Id: aapp_encodebufr_1c.ksh 618 2019-02-12 10:54:03Z frna $
#
#Encode level 1c ATOVS or IASI AAPP files into BUFR, using ECMWF BUFR libary
#
#Usage: aapp_encodebufr_1c [-i files] instruments
#     -i  specifies input file names (same order as the instruments)
#         Defaults to "hrsn.l1c aman.l1c ambn.l1c mhsn.l1c iasi.l1c iasi.lpc"
#     instruments is a subset of "HIRS AMSU-A AMSU-B MHS IASI PCIASI"
#
#Outputs: BUFR files with .bufr suffix
#
#Environment variables:
#  BUFR_TABLES - directory containing BUFR tables (required)
#  ORIGINATING_CENTRE - for Section 1 (default 254=EUMETSAT, or 74 for level 1d)
#  SUB_CENTRE - for Section 1 (default 0)
#  MASTER_TABLE - version number of master table (default 13, or 16 for ATMS/CrIS)
#  LOCAL_TABLE - version number of local table (default 0, or 1 for level 1d)
#  CENTRE_ID - 1b/1c data originating centre, for section 4 (default 254=EUMETSAT)
#  BUFR_EDITION - BUFR edition number (default 4)
#  ENHANCED_IASI - set this to Y to use the newer sequence 3-40-007, otherwise
#                  defaults to 3-40-001
#  BUFR_FIRSTFOV_IASI1C - set to 0 to start FOV numbers from 0, for IASI 1c and
#                         pc BUFR output. This convention is used in EUMETSAT
#                         global stream. Otherwise defaults to 1.
#  
# If the version number of the local table is 0 or 255 then the ECMWF BUFR
# software will use master BUFR tables, e.g. B0000000000000013000, 
# D0000000000000013000
#
# Otherwise the software will require BUFR tables with names of the form
# B000cccccCCCCCVVVvvv, D000cccccCCCCCVVVvvv where ccccc is the sub-centre,
# CCCCC is the originating centre, VVV is the master table version number, vvv
# is the local table version number
#
#History:
# 1.1     27/01/05  initial version. N C Atkinson
# 1.2     01/02/06  change config file P. Marguinaud
# 1.3     01/03/06  add environment variables for originating centre, etc. NCA
# 1.4     11/09/06  add BUFR_EDITION; change default MASTER_TABLE to 13
# 1.5     14/02/07  add return code, PM
# 1.6     09/03/09  add PCIASI
# 1.7     24/11/09  add Enhanced IASI
# 1.8     04/03/10  add ATMS and CRIS
# 1.9     13/06/11  add HIRS1D, IASI1D and AMSUB1D
# 1.10    20/08/12  add MWTS and MWHS
# 1.11    14/09/12  change default MASTER_TABLE for ATMS/CrIS to 16, since the
#                   official v15 only has validation status for ATMS, and is
#                   different to v16.
# 1.12    15/02/13  add IRAS
# 1.13    25/04/14  use MASTER_TABLE v16 for PCIASI also, since v13 has a 
#                   different sequence for 340008.
# 1.14    23/05/14  Add MWTS21D and MWHS21D
# 1.15    19/08/14  Add MWRI by Qifeng Lu
# 1.16    15/09/15  Add -N flag for encoding NEDT (ATOVS only)
# 1.17    07/04/16  Add MWRI1D
# 1.18    07/12/16  Add -s option to skip some scans at start and end of file.
# 1.19    03/01/17  Create environment variable AAPP_VERSION from the AAPP_version.txt file.
#                   Enabled by default for level 1c, but can be suppressed with "-v none". 
# 1.20    20/01/17  Set default MASTER_TABLE=26 if ENCODE_NEDT=Y
# 1.21    10/10/17  Set default USE_OB_TIME=Y, as this is the recommended
#                   setting for DBNet.
# 1.22    13/12/17  Default is to include version number and NEDT for ATOVS only
#==============================================================================

usage(){
  echo
  echo 'usage : aapp_encodebufr_1c [-N | -n] [-i files] [-s skip_scans] [-v none|yes] instruments'
  echo
  echo '  where instruments is a list of instruments (HIRS AMSU-A AMSU-B MHS'
  echo '  IASI PCIASI HIRS1D IASI1D AMSUB1D MWTS MWHS IRAS MWTS2 MWHS2 MWRI MWTS21D'
  echo '  MWHS21D MWRI1D) and files is a list of files to encode.'
  echo '  Quotes " " are necessary if there is more than 1 file'
  echo '  Use -N to include NEDT information (ATOVS only) - default, or -n to suppress'
  echo '  By default, AAPP version number is appended to 1c sequences, use -v none to suppress'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

if [ x$BUFR_TABLES = x ]; then
  echo Error: need to define environment variable BUFR_TABLES
  exit 1
fi

if [ ! -d $BUFR_TABLES ]; then
  echo Error: environment variable BUFR_TABLES does not point to a valid directory
  exit 1
fi

files=""   #initialise
skip_scans=""

while getopts s:i:Nnv: c
do
  case $c in
     i)	files=$OPTARG;;
     s)	skip_scans=$OPTARG;;
     N)	encode_nedt=Y;;
     n)	encode_nedt=n;;
     v)	encode_version=$OPTARG;;
     \?)      usage
              exit 2;;
  esac

done

shift `expr $OPTIND - 1`
if [ $# -eq 0 ]; then       #no instruments specified
  usage
  exit 2
fi

export BUFR_SKIP_SCANS=$skip_scans

instruments=$*        #Get all the instruments

export USE_OB_TIME=${USE_OB_TIME:-Y}  #default is to use ob time in Section 1,
                                      #not the run time

# For ATMS and CrIS need at least master table version 16. Also for PCIASI.
if [[ $instruments = *CRIS* || $instruments = *ATMS* || $instruments = *PCIASI* ]]; then
  MASTER_TABLE=${MASTER_TABLE:-16}
fi

# For 1d encoding we need Met Office tables
if [[ $instruments = *CRIS1D* || $instruments = *HIRS1D* \
   || $instruments = *ATMS1D* || $instruments = *IASI1D* \
   || $instruments = *AMSUB1D* || $instruments = *MWTS21D* \
   || $instruments = *MWHS21D* || $instruments = *IRAS1D* \
   || $instruments = *MWRI1D* ]]; then
  LOCAL_TABLE=${LOCAL_TABLE:-1}
  ORIGINATING_CENTRE=${ORIGINATING_CENTRE:-74}   #74 = Met Office
fi

# Other defaults
export ORIGINATING_CENTRE=${ORIGINATING_CENTRE:-254}   #254 = EUMETSAT
export SUB_CENTRE=${SUB_CENTRE:-0}
export LOCAL_TABLE=${LOCAL_TABLE:-0}
export CENTRE_ID=${CENTRE_ID:-${ORIGINATING_CENTRE}}
export SUB_CENTRE_ID=${SUB_CENTRE_ID:-${SUB_CENTRE}}
export BUFR_EDITION=${BUFR_EDITION:-4}
export ENHANCED_IASI=${ENHANCED_IASI:-Y}     #This should be the default

if [ ${#files} != 0 ]; then
  set $files         #Put input files (if specified) in positional parameters
else
  shift $#           #Remove existing positional parameters
fi

err=0

for instrument in $instruments; do

  #For AAPP v8, the default is to encode NEDT and version number for ATOVS, but not for any other data types

  if [ $instrument = HIRS ] || [ $instrument = AMSU-A ] || [ $instrument = AMSU-B ] || [ $instrument = MHS ]; then
    export ENCODE_NEDT=${encode_nedt:-Y}
    encode_version=${encode_version:-yes}
  else
    export ENCODE_NEDT=N
    encode_version=${encode_version:-none}
  fi
  if [ $ENCODE_NEDT = Y ]; then
    export MASTER_TABLE=${MASTER_TABLE:-26}
    if [ $MASTER_TABLE -lt 26 ]; then
      echo "setting MASTER_TABLE=26"
      MASTER_TABLE=26
    fi
  else
    export MASTER_TABLE=${MASTER_TABLE:-13}
  fi
  if [ -s $AAPP_PREFIX/AAPP_version.txt ] && [[ $encode_version != [Nn]* ]]; then
    v=$(head -1 $AAPP_PREFIX/AAPP_version.txt)
    major=${v%%.*}
    minor=${v#*.}
    minor=${minor%%.*}   #ignore any sub-minor version numbers
    if [[ $minor = 0* ]]; then
      minor=0            #set minor version to 0 for beta releases
    fi 
    export AAPP_VERSION=$((100*$major + $minor))     #e.g. 7.14 -> 714, 8.1 -> 801
  else
    AAPP_VERSION=
  fi 
  echo "AAPP_VERSION=$AAPP_VERSION"
    
  case $instrument in
    HIRS)   infile=${1:-hrsn.l1c}
            ln -sf $infile ${FORT}10 ;;
    AMSU-A) infile=${1:-aman.l1c}
            ln -sf $infile ${FORT}11 ;;
    AMSU-B) infile=${1:-ambn.l1c}
            ln -sf $infile ${FORT}12 ;;
    MHS)    infile=${1:-mhsn.l1c}
            ln -sf $infile ${FORT}12 ;;     #MHS uses same unit as AMSU-B
    IASI)   infile=${1:-iasi.l1c}
            ln -sf $infile ${FORT}13 ;;
    PCIASI) infile=${1:-iasi.lpc}
            ln -sf $infile ${FORT}14 ;;
    CRIS1D) infile=${1:-cris.l1d}
            ln -sf $infile ${FORT}15 ;;
    HIRAS1D)infile=${1:-hiras.l1d}
            ln -sf $infile ${FORT}15 ;;
    ATMS1D) infile=${1:-atms.l1d}
            ln -sf $infile ${FORT}16 ;;
    CRIS)   infile=${1:-cris.l1c}
            ln -sf $infile ${FORT}17 ;;
    ATMS)   infile=${1:-atms.l1c}
            ln -sf $infile ${FORT}18 ;;
    HIRS1D) infile=${1:-hirs.l1d}
            ln -sf $infile ${FORT}19 ;;
    HIRS1DM)infile=${1:-hirs.l1d}
            ln -sf $infile ${FORT}19 ;;
    IASI1D) infile=${1:-iasi.l1d}
            ln -sf $infile ${FORT}20 ;;
    AMSUB1D) infile=${1:-ambn.l1d}
            ln -sf $infile ${FORT}21 ;;
    MWTS)   infile=${1:-mwts.l1c}
            ln -sf $infile ${FORT}22 ;;
    MWHS)   infile=${1:-mwhs.l1c}
            ln -sf $infile ${FORT}23 ;;
    MWTS2)  infile=${1:-mwts2.l1c}
            ln -sf $infile ${FORT}22 ;;
    MWHS2)  infile=${1:-mwhs2.l1c}
            ln -sf $infile ${FORT}23 ;;
    IRAS)   infile=${1:-iras.l1c}
            ln -sf $infile ${FORT}24 ;;
    MWTS21D)infile=${1:-mwts2.l1c}
            ln -sf $infile ${FORT}22 ;;
    MWHS21D)infile=${1:-mwhs2.l1c}
            ln -sf $infile ${FORT}23 ;;
    IRAS1D)infile=${1:-iras.l1c}
            ln -sf $infile ${FORT}24 ;;
    MWRI)   infile=${1:-mwri.l1c}
            ln -sf $infile ${FORT}25 ;;
    MWRI1D) infile=${1:-mwri.l1c}
            ln -sf $infile ${FORT}25 ;;
    *)      usage
            exit 2;;
  esac

  outfile=${infile%.*}.bufr
  
  if [ -f $infile ]; then
    if [ -s $infile ]; then
      export INSTRUMENT=$instrument
      export BUFRENC_FILE=$outfile
      rm -f $outfile
      echo processing $infile for $instrument
      aapp_encodebufr_1c.exe
      if [ $? -ne 0 ]
      then
        err=1
      fi
    else
      echo "Input file $infile is empty"
    fi
  else
    echo "Input file $infile does not exist"
  fi
  
  if [ $# != 0 ]; then
    shift         #Next input file
  fi
done

rm -f ${FORT}10 ${FORT}11 ${FORT}12 ${FORT}13 ${FORT}14
rm -f ${FORT}15 ${FORT}16 ${FORT}17 ${FORT}18 ${FORT}19 ${FORT}20 ${FORT}21
rm -f ${FORT}22 ${FORT}23 ${FORT}24 ${FORT}25 ${FORT}26
exit $err
