#!/bin/ksh
#$Id: avh_get_vis_coefs.ksh 559 2017-10-06 08:17:48Z frna $
#
# Get AVHRR visible channels coefficients from NOAA web site.
# Find the entry with the nearest "center time" to the requested time
# and print out the coefficients and the solar cal "data time"
#
# If an AVHRR 1b file is specified then the coefficients will be inserted into
# the "operational coefficients" in the data records.
#
# NOAA publish a new calibration file monthly, approx 3 to 4 weeks before
# the 'center' time in the file.
#
# PROPRIETAIRE/OWNER
#       NWP SAF
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
#
#    Copyright 2009, EUMETSAT, All Rights Reserved.
#
# History:
# 01/04/2009 Nigel Atkinson
# 26/10/2009 Allow for NOAA files with either 15 or 7 fields
# 09/11/2010 Update URL of NOAA web site
# 30/06/2011 Add the web page explicitly, not just the directory
# 13/10/2011 Remove unwanted <cr> in vis_coefs.inp and improve portability
# 29/10/2016 change nesdis URL to https
# 22/08/2017 For AAPP v8, use $AVHRR_VISIBLE_URL and $AVHRR_VISIBLE_URL_TIMEOUT in ATOVS_ENV8.
#            Accept BSD date command syntax.
#
##############################################################

usage(){
  echo
  echo 'usage : avh_get_vis_coefs -s satellite [-c "channels"] [-d date] [-u] [-z] [avh1bfile]'
  echo
  echo '  where satellite specifies one satellite (e.g. noaa18)'
  echo '  channels, defaults to "1 2 3a" (quotes needed)'
  echo '  date is in any format recognised by Linux date command, defaults to today'
  echo '    or if your system uses BSD (e.g. macOS) then date should be formatted YYYYmmdd'
  echo '  u option forces the NOAA coefficient file to be updated'
  echo '        (can set -d "" to prevent any further processing) '
  echo '  z option prevents automatic downloading of the coefficient file'
  echo '  avh1bfile is an AAPP-format AVHRR 1b file'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

#Web site where the NOAA files are found:

#url=https://www.star.nesdis.noaa.gov/smcd/spb/fwu/homepage/AVHRR/Op_Cal_AVHRR/Op_Cal_AVHRR.html
url=$AVHRR_VISIBLE_URL

#If a timeout is specified in ATOVS_ENV8, set the number of tries to 1
[ "$AVHRR_VISIBLE_URL_TIMEOUT" ] && timeout="-t 1 -T $AVHRR_VISIBLE_URL_TIMEOUT"

[ $url ] && remotedir=$(dirname $url)

dir=${PAR_CALIBRATION_COEF}/avhcl   #Local directory for storing the NOAA files
mkdir -p $dir

#See whether we have a GNU or BSD date command
gnudate=
[[ "$(date --version 2>/dev/null)" = *GNU* ]] && gnudate=Y

##############################################################
# process the input arguments
##############################################################

updatenow=N              #defaults
noupdate=N
datestr=$(date -u +%Y%m%d)
chans="1 2 3a"

while getopts s:c:d:uz c
do
  case $c in
     s)	sat=$OPTARG;;
     c)	chans=$OPTARG;;
     d)	datestr=$OPTARG;;
     u)	updatenow=Y;;
     z)	noupdate=Y;;
     \?)      usage
              exit 2;;
  esac
done

[ $url ] || noupdate=Y    #do not attempt to update if url is not set

shift `expr $OPTIND - 1`        
if [ "$sat" = "" ]; then
  usage
  exit 1
fi

if [ $# != 0 ]; then
  avh1bfile=$1
fi

if [[ $sat = noaa* ]]; then
  sat=N${sat#noaa}             #translate noaa18 -> N18
elif [[ $sat = M0[1-3] ]]; then
  sat=Metop${sat#M0}           #translate M02 -> Metop2
elif [[ $sat = metopa ]]; then
  sat=Metop2                   #translate metopa -> Metop2, etc.
elif [[ $sat = metopb ]]; then
  sat=Metop1
elif [[ $sat = metopc ]]; then
  sat=Metop3
elif [[ $sat = metop[1-3] ]]; then  #translate metop2 -> Metop2
  sat=Metop${sat#metop}
elif [[ $sat = metop0[1-3] ]]; then #translate metop02 -> Metop2
  sat=Metop${sat#metop0}
else
  echo "Unknown satellite $sat"
  exit 1
fi

##############################################################
# Loop round the channels
##############################################################

typeset -l chan      #lower case, i.e. 3a

outstring=""
n_out=0

for chan in $chans; do

  file=${sat}_AVHRR_Libya_ch${chan}.txt

##############################################################
# Retrieve calibration file from Internet if it needs updating
##############################################################

  if [ $noupdate = N ]; then

    if [ $updatenow = Y ]; then
      updateneeded=Y
    else
      updateneeded=N
      if [ -s $dir/$file ] && [ "$datestr" != "" ]; then
        set $(tail -1 $dir/$file)           #check whether an automatic update is needed
        if [ "$gnudate" ]; then
          lastsecs=$(date -u +%s -d $3)
          datasecs=$(date -u +%s -d $datestr)
        else
          lastsecs=$(date -jnu -f "%m/%d/%Y" $3 +%s)       #BSD date command has different syntax
          datasecs=$(date -jnu -f "%Y%m%d" $datestr +%s)   #Assume a standard format for datestr
        fi
        tdiff=$(expr $datasecs - $lastsecs)
        if [ $tdiff -gt 604800 ]; then #update if more than 7 days after center time
          updateneeded=Y
        fi
      else
        updateneeded=Y                 #file exists but is empty
      fi
      if [ "$(find $dir -name $file -atime -1)" != "" ]; then
        updateneeded=N                 #Don't try to update more than once per day
      fi
    fi

    if [ $updateneeded = Y ]; then
      wget $timeout ${remotedir}/${file} >wget.log 2>&1
      if [ $? != 0 ]; then
        echo "wget failed for $file"
        touch $dir/$file     #create dummy file to prevent multiple wget attempts
      else
        echo "downloaded $file"
        mv $file $dir
      fi
    fi
  fi

##############################################################
# Find the entry with "center time" nearest to the requested date
##############################################################

  if [ -s $dir/$file ] && [ "$datestr" != "" ]; then

    datasecs=$(date +%s -d $datestr)

    bestdiff='-1'

    while read line; do
      if [ "$line" != "" ]; then
        set $line
        if [ $# = 15 ] && [ ${#3} = 10 ]; then #check 15 fields and 3rd is length 10

          if [ "$gnudate" ]; then
            secs=$(date -u +%s -d $3)
          else
            secs=$(date -jnu -f "%m/%d/%Y" $3 +%s)
          fi
          secsdiff=$(expr $datasecs - $secs)
          if [ $secsdiff -ge 0 ]; then          #before the data time
            bestdiff=$secsdiff
            coefs="${12} ${13} ${14} ${15}"
            if [ "$gnudate" ]; then
              solarcaldate="$(date -u +"%Y %j" -d $2)"
            else
              solarcaldate="$(date -jnu -f "%m/%d/%Y" $2 +"%Y %j")"
            fi
          else                                  #after the data time
            secsdiff=$(expr $secs - $datasecs)
            if [ $secsdiff -lt $bestdiff ]; then
              bestdiff=$secsdiff
              coefs="${12} ${13} ${14} ${15}"
              if [ "$gnudate" ]; then
                solarcaldate="$(date -u +"%Y %j" -d $2)"
              else
                solarcaldate="$(date -jnu -f "%m/%d/%Y" $2 +"%Y %j")"
              fi
            fi
          fi

        elif [ $# = 7 ] && [ ${#3} = 10 ]; then #or 7 fields and 3rd is length 10

          secs=$(date +%s -d $3)
          secsdiff=$(expr $datasecs - $secs)
          if [ $secsdiff -ge 0 ]; then          #before the data time
            bestdiff=$secsdiff
            coefs="${4} ${5} ${6} ${7}"
            if [ "$gnudate" ]; then
              solarcaldate="$(date -u +"%Y %j" -d $2)"
            else
              solarcaldate="$(date -jnu -f "%m/%d/%Y" $2 +"%Y %j")"
            fi
          else                                  #after the data time
            secsdiff=$(expr $secs - $datasecs)
            if [ $secsdiff -lt $bestdiff ]; then
              bestdiff=$secsdiff
              coefs="${4} ${5} ${6} ${7}"
              if [ "$gnudate" ]; then
                solarcaldate="$(date -u +"%Y %j" -d $2)"
              else
                solarcaldate="$(date -jnu -f "%m/%d/%Y" $2 +"%Y %j")"
              fi
            fi
          fi
        fi
      fi
    done < $dir/$file

##############################################################
# Print out the coefficients
##############################################################

    if [ $bestdiff != -1 ]; then
      echo $chan $coefs $solarcaldate
      n_out=$(expr $n_out + 1)
      outstring="${outstring}${chan} ${coefs} ${solarcaldate}\n"
    fi
  fi
done
  
##############################################################
# Apply the coefficients to AVHRR 1b file
##############################################################

if [ -s "$avh1bfile" ] && [ $n_out -ge 1 ]; then
  echo "Updating $avh1bfile"
  echo $avh1bfile >vis_coefs.inp
  echo $n_out >>vis_coefs.inp
  printf "$outstring" >>vis_coefs.inp

  avh_get_vis_coefs.exe <vis_coefs.inp
  status=$?
  rm vis_coefs.inp
fi

exit $status
