C LIBS = f7nl1b auxaapp f7ml f7tp f7cp f7gp
!
      PROGRAM AVHRR_AAPP_TO_noaa
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: avhrr_aapp_to_class.F 482 2016-02-16 10:05:23Z frna $
!
! Purpose: Convert AVHRR files in AAPP format to NOAA format (16 bit,
!          big-endian)
!
! Inputs (read from std input):
!   Input  AAPP file name
!   Output NOAA file name
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  17/09/2013  N C Atkinson
! 1.1  11/02/2016  Correction: the offset term was scaled, should be quadratic.
!                  Use scale in avh_filler2 if it is set.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "avh1b.h"
#include "reclopt.h"    
#include "nl1b.h"

      character*200 aapp_file   !Can include directory
      character*200 noaa_file
      character*2 satid

      integer pos, endpos
      integer recl_noaa
      integer loc_rl
      integer ios
      integer irec
      integer i,j
      integer ichan
      integer nscan
      integer datatype
      integer datatype_swap
      integer headercount
      integer tmp1
      
      real irscale
      
      character*22016 avh_char_header   !length is that of AAPP
      character*22016 avh_char_data
      equivalence (avh_char_header, avh_hd(1))
      equivalence (avh_char_data, avh_dt(1))

      logical swap
      data swap/.false./
!
!- End of header

      read(*,'(A)') aapp_file
      read(*,'(A)') noaa_file

      recl_noaa = 22528   !16-bit format (AAPP is 22016)

!Is byte swapping needed? Make the output big-endian (as in NOAA/CLASS)

      avh_scnlin = 1
      if (ichar(avh_char_data(1:1)) .eq. 1) swap = .true.  !Little-endian platform

!Open the input and output files

      loc_rl=reclavh/rldiv
      open(1,file=aapp_file,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'AVHRR input file could not be opened, ',aapp_file
        call aappexit(1)
      endif

      loc_rl=recl_noaa/rldiv
      open(2,file=noaa_file,form='unformatted',access='direct',
     &     recl=loc_rl,status='replace',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'AVHRR output file could not be opened'
        call aappexit(1)
      endif

!Read and write the header record
      
      call avhhdr(1,ios)
      if(ios.ne.0) then
        write(*,*)'Unable to read header'
        call aappexit(1)
      endif

      avh_h_l1bversnb = 3    !Creating the v3 NOAA-N format without CLAVRx
      avh_h_l1bversyr = 2005
      avh_h_l1bversdy = 118  !Introduced April 28th 2005

      nscan = avh_h_scnlin
      datatype = avh_h_datatyp
      datatype_swap = datatype/256 + MOD(datatype,256)*256
      headercount = avh_h_hdrcnt

      if (swap) then                 !Force a byte swap
        avh_h_datatyp = datatype_swap
        CALL byteswap1b("AVH1BHD")   !byteswap_avh -> TRUE (in byteswap1b.F)
        avh_h_datatyp = datatype_swap
      endif

      write(2,rec=1,iostat=ios) avh_char_header
      
      if (swap) then                 !Cancel the swap
        avh_h_datatyp = datatype
        avh_h_hdrcnt = headercount
        CALL byteswap1b("AVH1BHD")   !byteswap_avh -> FALSE
      endif

!Loop over records

      do irec=1,nscan
        call avhdtr(1,irec,ios)
        if (ios .eq. 0) then

!Re-order the IR calibration coefficients and re-scale the quadratic coef.

          if (avh_filler2(1) .gt. 0) then
            irscale = 10.0**(-avh_filler2(1))
          else
            irscale = 1.0E-6   !default quadratic scale 10^6 if filler2 is not set
          endif
          
          do i=1,2   !set
            do j=1,1   !channel 3B
              tmp1 = avh_calir(1,i,j)
              avh_calir(1,i,j) = avh_calir(3,i,j)     !offset
              avh_calir(3,i,j) = NINT(tmp1*irscale/1.0E-6)     !quadratic -> 10^6
            enddo
            do j=2,3   !channels 4,5
              tmp1 = avh_calir(1,i,j)
              avh_calir(1,i,j) = avh_calir(3,i,j)     !offset
              avh_calir(3,i,j) = NINT(tmp1*irscale/1.0E-7)     !quadratic -> 10^7
            enddo
          enddo

!Re-scale the visible coefs. Ensure slope is positive, see Data Formats document

          do i=1,3   !set
            do j=1,3   !channel

  !slopes: 10^10 -> 10^7
              if (avh_calvis(1,i,j).lt.0) then
                avh_calvis(1,i,j) = 
     &               NINT(avh_calvis(1,i,j)*0.001 + 4294967.296)
              else
                avh_calvis(1,i,j) = NINT(avh_calvis(1,i,j)*0.001)
              endif
              if (avh_calvis(3,i,j).lt.0) then
                avh_calvis(3,i,j) = 
     &               NINT(avh_calvis(3,i,j)*0.001 + 4294967.296)
              else
                avh_calvis(3,i,j) = NINT(avh_calvis(3,i,j)*0.001)
              endif

  !offsets: 10^7 -> 10^6
              avh_calvis(2,i,j) = NINT(avh_calvis(2,i,j)*0.1)
              avh_calvis(4,i,j) = NINT(avh_calvis(4,i,j)*0.1)
            enddo
          enddo

!For avh_scnlinbit, reverse bit 0 (see Data Formats document section 8)

          tmp1 = avh_scnlinbit
          if (IAND(tmp1,1).EQ.1) then
            tmp1 = IAND(tmp1,65534)
          else
            tmp1 = IOR(tmp1,1)
          endif
          avh_scnlinbit = tmp1

!Write out data record

          if (swap) then
            avh_h_datatyp = datatype_swap
            CALL byteswap1b("AVH1BHD")   !byteswap_avh -> TRUE
            CALL byteswap1b("AVH1BDT")
            avh_h_datatyp = datatype
            avh_h_hdrcnt = headercount
            CALL byteswap1b("AVH1BHD")   !byteswap_avh -> FALSE
          endif
          write(2,rec=irec+1,iostat=ios) avh_char_data
          if (ios .ne. 0) then
            write(*,*)'Write failed for scan',irec
            call aappexit(1)
          endif
        else
          write(*,*)'Read failed for scan',irec
          call aappexit(1)
        endif
      enddo
      
      close(1)
      close(2)
      write(*,*)'Read   ',irec-1,' scans from ',aapp_file
      write(*,*)'Written',irec-1,' scans to   ',noaa_file

      END
