C LIBS = atovin f7nl1b auxaapp navtool f7ml f7tp f7cp f7gp
!
      PROGRAM AVHRR_GAC_CLASS_TO_AAPP_A_J
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: avhrr_gac_class_to_aapp_a-j.F 485 2016-04-07 15:30:28Z frna $
!
! Purpose: Convert AVHRR_GAC 1b files from NOAA/CLASS to AAPP format
!
!   The output is 2048 spots cross-track (as in the AAPP format), but retains
!   GAC resolution along-track (i.e. 1 scan in 3).
!
! Inputs (read from std input):
!   Input CLASS 1b file name, conforming to NSS.GHRR.N[A-J].D?????.*
!   Output AAPP file name
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  11/09/2013  N C Atkinson
! 1.1  25/04/2014  Allow for cases when avhgac_c_timecode_year is negative;
!                  assume 2 header records.
! 1.2  28/11/2014  Compute satellite height, zenith angles and relative azimuth Correct solar
!                  angles. Unpack telemetry containing raw data for
!                  calibration.
! 1.3  16/02/2016  Store scale factors in avh_filler2 and set avh_h_inststat1
!                  (see avh_lbc.F).
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "avh1b.h"
#include "reclopt.h"    
#include "avh1b_gac_tovs.h"
#include "nl1b.h"
#include "constants_preproc.h"
#include "inparms.h"

      character*200 class_file_in  !Can include directory
      character*80  class_file     !NSS.GHRR.N[A-J].D?????.*
      character*200 aapp_file
      character*2 satid
      character*6 satname
      character*6 satname_cal
      character*200 dir_calibration
      character*80 line

      integer pos
      integer recl_class
      integer loc_rl
      integer ios
      integer year
      integer day
      integer time
      integer irec
      integer scnlintime_old
      integer tdiff
      integer scandiff
      integer i,j,i3,i5,N
      integer cal
      integer ichan
      integer endpos
      integer iqscn(3)
      integer jdate
      integer ierr
      
      integer*2 avh_hrpt_flat(2045)   !5 channels x 409 spots
      integer*2 avh_hrpt_gac(5,409)
      equivalence(avh_hrpt_gac(1,1),avh_hrpt_flat(1))
      
      real widths(3,2)    !filter width, solar irradiance, wavelength * 2 chans
      real bandcor(3,3)   !band corrections and wavenumber * 3 chans

      logical swap
      data swap/.false./

      real centreangle_deg, scanangle_deg, zen_deg
      
      real latitudes(51), longitudes(51), azimuths(51), zeniths(51)
      real azi, relazi

      double precision satpos(3), nadpos(3), pixpos1(3), pixpos51(3)
      double precision pixvec1(3), pixvec51(3), normalvec(3), norm
      double precision latd, lond
      double precision earthrad_pix1, earthrad_pix51, earthrad_nad
      double precision zend, azid, distd, height_ratio, earth_ratio
      double precision centreangle, scanangle, height_km
!
!- End of header

      read(*,'(A)') class_file_in
      read(*,'(A)') aapp_file
      pos = 1
      do i=1,200
        if (class_file_in(i:i).eq."/") pos=i+1
      enddo
      class_file = class_file_in(pos:pos+41)
      
      if (class_file(5:7) .ne. "GHR") then
        write(*,*)'Error: input file name must be of the form'
        write(*,*)'NSS.GHRR.N[A-J].D?????.*'
        call aappexit(1)
      endif

      recl_class = 3220

!Is byte swapping needed? CLASS data are always big-endian

      avhgac_c_scnlin = 1
      if (ichar(avhgac_c_char(1:1)) .eq. 1) swap = .true.  !Little-endian platform

!Fixed items in header

      avh_h_siteid = 'nss'
      avh_h_l1bversnb = libl1bvers   !in nl1b.h
      avh_h_l1bversyr = libl1bversyr !in nl1b.h
      avh_h_l1bversdy = libl1bversdy !in nl1b.h
      avh_h_reclg = reclavh          !in avh1b.h
      avh_h_hdrcnt = 1
      avh_h_dataname = class_file
      avh_h_datatyp = 2              !2=GAC
      avh_h_tipsrc = 1               !1=GAC embedded
      avh_h_startdatady = 0
      avh_h_inststat1 = 63232          !Assume all channels on,
                                       !i.e. set bits 8-10 and 12-15
      avh_h_statchrecnb = 0

      satid = class_file(10:11)
      if (satid .eq. 'TN') then        !noaa05 (Tiros-N). This is an AAPP convention.
        satname = 'noaa05'
      else if (satid .eq. 'NA') then   !noaa06
        satname = 'noaa06'
      else if (satid .eq. 'NC') then   !noaa07
        satname = 'noaa07'
      else if (satid .eq. 'NE') then   !noaa08
        satname = 'noaa08'
      else if (satid .eq. 'NF') then   !noaa09
        satname = 'noaa09'
      else if (satid .eq. 'NG') then   !noaa10
        satname = 'noaa10'
      else if (satid .eq. 'NH') then   !noaa11
        satname = 'noaa11'
      else if (satid .eq. 'ND') then   !noaa12
        satname = 'noaa12'
      else if (satid .eq. 'NI') then   !noaa13
        satname = 'noaa13'
      else if (satid .eq. 'NJ') then   !noaa14
        satname = 'noaa14'
      else
        write(*,*)'Error: invalid satid ',satid
        call aappexit(1)
      endif

      avh_h_callocscnlin = 0
      scnlintime_old = 0
      
!Get channel info from AAPP data file
!avhcal.txt does not include all the older satellite, so use noaa11 by default.
!This will be rather inaccurate but is OK for a first guess.

      if (satname.eq.'noaa14' .or. satname.eq.'noaa12' .or. 
     &    satname.eq.'noaa11') then
        satname_cal = satname
      else
        satname_cal = 'noaa11'
        write(*,*)'Warning: using noaa11 channel information data' 
      endif

      dir_calibration = " "
      endpos = 1
      call getenv("DIR_CALIBRATION",dir_calibration)
      do i=1,LEN(dir_calibration)
        if (dir_calibration(i:i) .ne. " ") endpos = i
      enddo
      open(1,file=dir_calibration(1:endpos) // 
     &   "/coef/avhcl/avhcal.txt",form='formatted',status='old',
     &   iostat=ios)
      if(ios.ne.0) then
        write(*,*)'avhcal.txt could not be opened'
        call aappexit(1)
      endif
      line = " "
      ios = 0
      do while (index(line,satname_cal) .ne. 4 .and. ios .eq. 0)
        read(1,'(A80)',iostat=ios) line
      enddo
      if (ios .eq. 0) then
        do while (index(line,"equivalent") .ne. 3)
          read(1,'(A80)') line
        enddo
        read(1,*) line
        read(1,*) line
        read(1,*) widths
        do while (index(line,"band") .ne. 3)
          read(1,'(A80)') line
        enddo
        read(1,*) line
        read(1,*) bandcor
        close(1)

!visible channels
        do ichan=1,2
          avh_h_albcnv(1,ichan) = widths(2,ichan)*10  !solar filtered irradiance
          avh_h_albcnv(2,ichan) = widths(1,ichan)*1000  !filter width
        enddo

!IR channels
        avh_h_radtempcnv(1,1) = NINT(bandcor(1,1)*1D2)  !wavenumber 3B
        avh_h_radtempcnv(1,2) = NINT(bandcor(1,2)*1D3)  !wavenumber 4
        avh_h_radtempcnv(1,3) = NINT(bandcor(1,3)*1D3)  !wavenumber 5
        do ichan=1,3                     !see avh_hdu.F
          avh_h_radtempcnv (2,ichan) = 
     &         NINT( (-bandcor(2,ichan)/bandcor(3,ichan)) * 1D5 )
          avh_h_radtempcnv (3,ichan) = NINT((1./bandcor(3,ichan)) * 1D6)
        enddo

!noaa-5, 6, 8 and 10 do not have a channel 5, so repeat channel 4 information

        if (satid.eq.'noaa05' .or. satid.eq.'noaa06' .or.
     &      satid.eq.'noaa08' .or. satid.eq.'noaa10') then
          do i=1,3
            avh_h_radtempcnv(i,3) = avh_h_radtempcnv(i,2)  !no channel 5
          enddo
        endif
      else
        write(*,*)'could not find ' // satname_cal // ' in calcoef.dat'
        call aappexit(1)
      endif

!Open the input file

      loc_rl=recl_class/rldiv
      open(1,file=class_file_in,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'AVHRR_GAC input file could not be opened'
        call aappexit(1)
      endif
      
!Read the header record

      read(1,rec=1,iostat=ios) avhgac_c_char   !First record
      avh_h_satid = ICHAR(avhgac_c_char(1:1))
      avh_h_scnlin = ICHAR(avhgac_c_char(9:9))*256 + 
     &               ICHAR(avhgac_c_char(10:10))
      avh_h_prblkid = 'B' // avhgac_c_char(17:23)

!Open the output file and write the header

      loc_rl=reclavh/rldiv
      open(2,file=aapp_file,form='unformatted',access='direct',
     &     recl=loc_rl,status='replace',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'AVHRR output file could not be opened'
        call aappexit(1)
      endif
      call avhhdw(2,ios)
      if(ios.ne.0) then
        write(*,*)'Unable to write header for output file'
        call aappexit(1)
      endif
 
!Loop over records

      do irec=1,avh_h_scnlin
        read(1,rec=irec+2,iostat=ios) avhgac_c_char
        if (ios .eq. 0) then
          if (swap) CALL byteswap1b("GAC1BCL")

!Date, time, etc.

          avh_scnlin = avhgac_c_scnlin
          year = avhgac_c_timecode_year
          if (year.LT.0) year = year + 65536
          avh_scnlinyr = year / 512 + 1900
          if (avh_scnlinyr .lt. 1950) 
     &         avh_scnlinyr = avh_scnlinyr + 100
          avh_scnlindy = IAND(avhgac_c_timecode_year+0, 511)
          avh_scnlintime = avhgac_c_timecode_ms

          if (irec .eq. 1) then 
            scnlintime_old = avh_scnlintime 
          else
            tdiff = avh_scnlintime - scnlintime_old
            scnlintime_old = avh_scnlintime
            if (tdiff .lt. -43200000) tdiff = tdiff + 86400000
            scandiff = tdiff * 1.0 / 500 + 0.5    !0.5 sec per record
            if (scandiff .gt. 1) then
              avh_h_misscnlin = avh_h_misscnlin + scandiff - 1
              avh_h_datagaps = avh_h_datagaps + 1
            endif
          endif

          iqscn(1)=avh_scnlinyr
          iqscn(2)=avh_scnlindy
          iqscn(3)=avh_scnlintime
          call xdqj(iqscn,jdate)   !compute the julian day
          if (avh_h_startdatady .eq. 0) then
            avh_h_startdatayr = avh_scnlinyr
            avh_h_startdatady = avh_scnlindy
            avh_h_startdatatime = avh_scnlintime
            avh_h_startdatajd = jdate
          endif
          avh_h_enddatayr = avh_scnlinyr
          avh_h_enddatady = avh_scnlindy
          avh_h_enddatatime = avh_scnlintime
          avh_h_enddatajd = jdate

!Quality

          avh_qualind = 0
          if (IAND(avhgac_c_scanqual,Z'80000000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'80000000')  !do not use
          if (IAND(avhgac_c_scanqual,Z'20000000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'20000000')  !data gap
          if (IAND(avhgac_c_scanqual,Z'40000000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'40000000')  !time error
          if (IAND(avhgac_c_scanqual,Z'08000000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'10000000')  !no calibration
          if (IAND(avhgac_c_scanqual,Z'04000000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'08000000')  !no earth loc
          if (IAND(avhgac_c_scanqual,Z'01000000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'00000001')  !pseudo noise
          if (IAND(avhgac_c_scanqual,Z'00800000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'04000000')  !drop lock
          if (IAND(avhgac_c_scanqual,Z'00400000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'02000000')  !word error >0
          if (IAND(avhgac_c_scanqual,Z'00200000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'01000000')  !previously dropped lock
          if (IAND(avhgac_c_scanqual,Z'00100000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'00800000')  !flywheeling
          if (IAND(avhgac_c_scanqual,Z'00080000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'00400000')  !bit slippage

          if (IAND(avhgac_c_scanqual,Z'00008000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'00000200')  !parity error 1
          if (IAND(avhgac_c_scanqual,Z'00004000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'00000100')  !parity error 2
          if (IAND(avhgac_c_scanqual,Z'00002000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'00000080')  !parity error 3
          if (IAND(avhgac_c_scanqual,Z'00001000').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'00000040')  !parity error 4
          if (IAND(avhgac_c_scanqual,Z'00000800').ne.0) 
     &        avh_qualind = IOR(avh_qualind,Z'00000020')  !parity error 5

          if (IAND(avhgac_c_scanqual,Z'0000F800').eq.0) 
     &        avh_h_okdatafr = avh_h_okdatafr + 1           !no sync errors
          if (IAND(avhgac_c_scanqual,Z'04000000').ne.0) 
     &        avh_h_auxsyncerrsum = avh_h_auxsyncerrsum + 1 !Sum of aux sync errors
          if (IAND(avhgac_c_scanqual,Z'40000000').ne.0) 
     &        avh_h_timeseqerr = irec                     !record number of first occurrence
          if (IAND(avhgac_c_scanqual,Z'08000000').ne.0) 
     &        avh_h_locerrind =  irec                     !record number of first occurrence
          if (IAND(avhgac_c_scanqual,Z'0C000000').eq.0) 
     &        avh_h_callocscnlin = avh_h_callocscnlin + 1

          if (IAND(avhgac_c_scanqual,Z'02000000').eq.0) then
            avh_scnlinbit = 1        !Ascending with channel 3B   
          else
            avh_scnlinbit = -32767   !Descending with channel 3B
          endif

!Navigation

          do i=1,51
            avh_pos(1,i) = NINT(avhgac_c_latlon(1,i)*1.0E4/128)
            avh_pos(2,i) = NINT(avhgac_c_latlon(2,i)*1.0E4/128)
            avh_ang(1,i) = ICHAR(avhgac_c_solzen(i))*50
               !convert solar angle from 0.5 deg to 0.01 deg resolution.
          enddo

!Compute the solar azimuth angle

          year = avh_scnlinyr
          day = avh_scnlindy
          time = avh_scnlintime
          do i=1,51
            latitudes(i) = avh_pos(1,i)*1.0E-4
            longitudes(i) = avh_pos(2,i)*1.0E-4
          enddo
          N = 51
          CALL INSUNA ( year, day, time, N, latitudes, longitudes,
     &                    azimuths, zeniths )

!Estimate the satellite height using lat/lon of spots 1, 26 and 51.
!AVHRR scans 0.0540865 degrees per spot (see satid.txt).
!Navigation tie points are every 40 spots.

          IF (avh_pos(1,26).LE.900000.AND.avh_pos(1,26).GE.-900000) THEN


            latd = avh_pos(1,26)*1.0E-4*deg2rad
            lond = avh_pos(2,26)*1.0E-4*deg2rad
            call lalocart(1,latd,lond,nadpos,ierr)
            earthrad_nad = sqrt(nadpos(1)**2+nadpos(2)**2+nadpos(3)**2)

            latd = avh_pos(1,1)*1.0E-4*deg2rad
            lond = avh_pos(2,1)*1.0E-4*deg2rad
            call lalocart(1,latd,lond,pixpos1,ierr)
            earthrad_pix1 = 
     &        sqrt(pixpos1(1)**2+pixpos1(2)**2+pixpos1(3)**2)

            latd = avh_pos(1,51)*1.0E-4*deg2rad
            lond = avh_pos(2,51)*1.0E-4*deg2rad
            call lalocart(1,latd,lond,pixpos51,ierr)
            earthrad_pix51 = 
     &        sqrt(pixpos51(1)**2+pixpos51(2)**2+pixpos51(3)**2)

            DO i=1,3
              pixvec1(i) =  pixpos1(i)/earthrad_pix1
              pixvec51(i) =  pixpos51(i)/earthrad_pix51
            ENDDO
            
            centreangle = 0.5*acos(pixvec1(1)*pixvec51(1) + 
     &         pixvec1(2)*pixvec51(2) + 
     &         pixvec1(3)*pixvec51(3))  !semi-angle at centre of earth
            centreangle_deg = centreangle/deg2rad
                      
            scanangle_deg = 0.5*(1-51)*0.0540865*40
            scanangle = abs(scanangle_deg)*deg2rad

            height_ratio = sin(scanangle+centreangle)/sin(scanangle)
            height_km = earthrad_nad*(height_ratio-1.0)
            avh_scalti = 10.0*height_km + 0.5
             
!Zenith and azimuth angles for all spots, hence relative azimuth. 
!NOAA satellites use local normal pointing mode, so estimate the satellite
!position using the local normal at nadir. Neglect yaw/roll/pitch.

            earth_ratio = (semiMajor-semiDiff)/semiMajor
            normalvec(1) = earth_ratio*nadpos(1)
            normalvec(2) = earth_ratio*nadpos(2)
            normalvec(3) = 1.0/earth_ratio*nadpos(3)
            norm = sqrt(normalvec(1)**2 + normalvec(2)**2 + 
     &        normalvec(3)**2)
            DO i=1,3
              normalvec(i) = normalvec(i)/norm
              satpos(i) = nadpos(i) + normalvec(i)*height_km
            ENDDO
            
            DO i=1,51
              IF (i .ne. 26) THEN
                latd = avh_pos(1,i)*1.0D-4*deg2rad
                lond = avh_pos(2,i)*1.0D-4*deg2rad

                call zenazi(1,latd,lond,satpos,zend,azid,distd)

                avh_ang(2,i) = zend/deg2rad * 100 + 0.5   !store zenith angle

                relazi = abs(azimuths(i) - azid/deg2rad)
                if (relazi. gt. 360) relazi = relazi-360
                if (relazi .gt. 180) relazi = 360-relazi  !as in nav_1blin.F

                avh_ang(3,i) = relazi * 100 + 0.5    !store relative azimuth

              ENDIF
            ENDDO
            avh_ang(3,26) = avh_ang(3,27)   !nadir angles
            avh_ang(2,26) = 0

          ELSE
            avh_scalti = -32768
            do i=1,51
              avh_ang(2,i) = -32768
              avh_ang(3,i) = -32768
            enddo
          ENDIF

!Telemetry. 3 words per 4 bytes, as for the channel data.

          do i=1,103,3
            i3 = (i-1)/3 + 1
            if (i .ne. 103) then
              avh_telem(i+2) = IAND(avhgac_c_telem(i3),Z'3FF')
              avh_telem(i+1) = 
     &                 IAND(avhgac_c_telem(i3),Z'FFC00')/Z'400'
            endif
            avh_telem(i) = 
     &                 IAND(avhgac_c_telem(i3),Z'3FF00000')/Z'100000'
          enddo

!AVHRR channel data. 3 words per 4 bytes

          do i=1,2044,3
            i3 = (i-1)/3 + 1
            if (i .ne. 2044) then
              avh_hrpt_flat(i+2) = IAND(avhgac_c_video(i3),Z'3FF')
            endif
            avh_hrpt_flat(i+1) = 
     &                 IAND(avhgac_c_video(i3),Z'FFC00')/Z'400'
            avh_hrpt_flat(i) = 
     &                 IAND(avhgac_c_video(i3),Z'3FF00000')/Z'100000'
          enddo

!Fill in the points in the avh_hrpt(5,2048) array

          do i=1,409
            i5 = i*5
            do j=-2,1         !points 3-6, 8-11, ...
              do ichan=1,5
                avh_hrpt(ichan,i5+j) = avh_hrpt_gac(ichan,i)
              enddo
            enddo
          enddo
          do i=7,2042,5       !intermediate spots (take mean)
            do ichan=1,5
              avh_hrpt(ichan,i) = 
     &          (avh_hrpt(ichan,i-1)+avh_hrpt(ichan,i+1))/2
            enddo
          enddo
          do ichan=1,5        !first 2 and last 2
            avh_hrpt(ichan,1) = avh_hrpt(ichan,3)
            avh_hrpt(ichan,2) = avh_hrpt(ichan,3)
            avh_hrpt(ichan,2047) = avh_hrpt(ichan,2046)
            avh_hrpt(ichan,2048) = avh_hrpt(ichan,2046)
          enddo

!Calibration

          do ichan=1,2
            do j=1,3     !same for operational, test and pre-launch
              avh_calvis(1,j,ichan) = 
     &          NINT(avhgac_c_calcoef(1,ichan)/2.0**30 * 1E10) !slope
              avh_calvis(2,j,ichan) = 
     &          NINT(avhgac_c_calcoef(2,ichan)/2.0**22 * 1E7)  !intercept
              avh_calvis(3,j,ichan) = avh_calvis(1,1,ichan)
              avh_calvis(4,j,ichan) = avh_calvis(2,1,ichan)
              avh_calvis(5,j,ichan) = 512
            enddo
          enddo

          do ichan=3,5
            avh_calir(1,1,ichan-2) = 0                       !2nd order
            avh_calir(2,1,ichan-2) =
     &        NINT(avhgac_c_calcoef(1,ichan)/2.0**30 * 1E6)  !slope
            avh_calir(3,1,ichan-2) =
     &        NINT(avhgac_c_calcoef(2,ichan)/2.0**22 * 1E6)  !intercept
          enddo
          avh_filler2(1) = 9    !Scale for quadratic term
          avh_filler2(2) = 6    !Scale for linear term
          avh_filler2(3) = 6    !Scale for offset
                        
!Write out data record

          call avhdtw(2,irec,ios)
          if(ios.ne.0) then
            write(*,*)'Unable to write data record',irec
            call aappexit(1)
          endif
        else
          write(*,*)'Unable to read data record',irec
          call aappexit(1)
        endif  

      enddo
      
!Update header

      call avhhdw(2,ios)
      if(ios.ne.0) then
        write(*,*)'Unable to update header for output file'
        call aappexit(1)
      endif

      close(1)
      close(2)
      write(*,*)'Read   ',irec-1,' scans from ',class_file
      write(*,*)'Written',irec-1,' scans to   ',aapp_file

      END
