C LIBS = f7nl1b auxaapp f7ml f7tp f7cp f7gp
!
      PROGRAM AVHRR_LAC_CLASS_TO_AAPP_KLM
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: avhrr_gac_class_to_aapp_klm.F 478 2016-02-10 13:38:43Z frna $
!
! Purpose: Convert NOAA-[K-P] (ATOVS era) AVHRR GAC 1b files from
!          NOAA/CLASS (10 bit or 16 bit format) to AAPP format
!
!   The output is 2048 spots cross-track (as in the AAPP format), but retains
!   GAC resolution along-track (i.e. 1 scan in 3).
!
! Inputs (read from std input):
!   Input CLASS 1b file name, conforming to NSS.GHRR.N[K-P].D?????.*
!   Output AAPP file name
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  13/09/2013  N C Atkinson
! 1.1  13/01/2016  Store scale factors in avh_filler2 (see avh_lbc.F) and allow
!                  for quadratic scale change in April 2005.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "avh1b.h"
#include "reclopt.h"    
#include "nl1b.h"

      character*200 class_file_in  !Can include directory
      character*80  class_file     !NSS.GHRR.N[K-P].D?????.*
      character*200 aapp_file
      character*2 satid

      integer pos
      integer recl_class
      integer loc_rl
      integer ios
      integer irec
      integer i,j,i3,i5
      integer ichan
      integer file_size
      integer tmp1,tmp2
      integer endpos
      integer scaleq_ch45
      
      integer*2 avh_hrpt_flat(2045)   !5 channels x 409 spots
      integer*2 avh_hrpt_gac(5,409)
      equivalence(avh_hrpt_gac(1,1),avh_hrpt_flat(1))
      
      integer*4 avh_packed(682)
      
      character*22016 avh_char_header    !length is that of AAPP
      character*22016 avh_char_data
      equivalence (avh_char_header, avh_hd(1))
      equivalence (avh_char_data, avh_dt(1))

      logical swap
      data swap/.false./
!
!- End of header

      read(*,'(A)') class_file_in
      read(*,'(A)') aapp_file
      read(*,*) file_size

      if (MOD(file_size,4608).ne.0.and.MOD(file_size,5632).ne.0) then
        write(*,*)'Error: file size is not a multiple of the supported'
        write(*,*)' NOAA record lengths'
        call aappexit(1)
      endif

      pos = 1
      do i=1,200
        if (class_file_in(i:i).eq."/") pos=i+1
      enddo
      endpos = MIN(pos+79,200)
      class_file = class_file_in(pos:endpos)
      
      if (class_file(5:7) .ne. "GHR") then
        write(*,*)'Error: input file name must be of the form'
        write(*,*)'NSS.GHRR.N[K-P].D?????.*'
        call aappexit(1)
      endif

      recl_class = 4608    !assume packed format initially

!Is byte swapping needed? CLASS data are always big-endian

      avh_scnlin = 1
      if (ichar(avh_char_data(1:1)) .eq. 1) swap = .true.  !Little-endian platform

!Open the input and output files

      loc_rl=recl_class/rldiv
      open(1,file=class_file_in,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'AVHRR input file could not be opened'
        call aappexit(1)
      endif

      loc_rl=reclavh/rldiv
      open(2,file=aapp_file,form='unformatted',access='direct',
     &     recl=loc_rl,status='replace',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'AVHRR output file could not be opened'
        call aappexit(1)
      endif
      
!Read the header record. Because the format is the same as AAPP, it can go
!straight into the avh_hd array

      read(1,rec=1,iostat=ios) avh_char_header(1:recl_class)
      if (swap) CALL byteswap1b("AVH1BHD")
      if (avh_h_l1bversnb .gt. 2) then
        scaleq_ch45 = 100    !quadratic scale changed to 10^7 after April 2005
      else
        scaleq_ch45 = 1000   !original scale was 10^6
      endif
      avh_h_l1bversnb = libl1bvers
      avh_h_l1bversyr = libl1bversyr
      avh_h_l1bversdy = libl1bversdy
      call avhhdw(2,ios)
      if(ios.ne.0) then
        write(*,*)'Unable to write header for output file'
        call aappexit(1)
      endif

!Is it 10-bit packed format or 16-bit unpacked?

      recl_class = file_size/(avh_h_scnlin+1)
      if (recl_class .eq. 5632) then
        close(1)
        loc_rl=recl_class/rldiv
        open(1,file=class_file_in,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)
        write(*,*)'Input file is 16-bit format'
      else
        if (recl_class .ne. 4608) then
          write(*,*)'Error: Unsupported record length: ',recl_class
          write(*,*)'Records: ',avh_h_scnlin+1
          write(*,*)'File size: ',file_size
          call aappexit(1)
        else
          write(*,*)'Input file is 10-bit format'
        endif
      endif

!Loop over records

      do irec=1,avh_h_scnlin
        read(1,rec=irec+1,iostat=ios) 
     &     avh_char_data(1:recl_class)
        if (ios .eq. 0) then
          if (swap) CALL byteswap1b("AVH1BDT")

!Unpack the AVHRR channel data. 3 words per 4 bytes

          do ichan=1,5    !Copy the first 2045 elements of the hrpt array
            do i=1,409
              avh_hrpt_gac(ichan,i) = avh_hrpt(ichan,i)
            enddo
          enddo
          if (recl_class .eq. 4608) then
            do i=1,682
              tmp1 = avh_hrpt_flat(2*i-1)    !2-byte words -> 4-byte
              tmp2 = avh_hrpt_flat(2*i)
              if (tmp2.lt.0) tmp2 = tmp2+65536
              avh_packed(i) = tmp1*65536 + tmp2
            enddo
            do i=1,2044,3
              i3 = (i-1)/3 + 1
              if (i .ne. 2044) then
                avh_hrpt_flat(i+2) = IAND(avh_packed(i3),Z'3FF')
              endif
              avh_hrpt_flat(i+1) = 
     &                 IAND(avh_packed(i3),Z'FFC00')/Z'400'
              avh_hrpt_flat(i) = 
     &                 IAND(avh_packed(i3),Z'3FF00000')/Z'100000'
            enddo
          endif

!Fill in the points in the avh_hrpt(5,2048) array

          do i=1,409
            i5 = i*5
            do j=-2,1         !points 3-6, 8-11, ...
              do ichan=1,5
                avh_hrpt(ichan,i5+j) = avh_hrpt_gac(ichan,i)
              enddo
            enddo
          enddo
          do i=7,2042,5       !intermediate spots (take mean)
            do ichan=1,5
              avh_hrpt(ichan,i) = 
     &          (avh_hrpt(ichan,i-1)+avh_hrpt(ichan,i+1))/2
            enddo
          enddo
          do ichan=1,5        !first 2 and last 2
            avh_hrpt(ichan,1) = avh_hrpt(ichan,3)
            avh_hrpt(ichan,2) = avh_hrpt(ichan,3)
            avh_hrpt(ichan,2047) = avh_hrpt(ichan,2046)
            avh_hrpt(ichan,2048) = avh_hrpt(ichan,2046)
          enddo

!Re-order the IR calibration coefficients and re-scale the quadratic coef.

          do i=1,2   !set
            do j=1,1   !channel 3B
              tmp1 = avh_calir(3,i,j)
              avh_calir(3,i,j) = avh_calir(1,i,j)     !offset
              avh_calir(1,i,j) = tmp1*1000            !quadratic 10^6 -> 10^9
            enddo
            do j=2,3   !channels 4,5
              tmp1 = avh_calir(3,i,j)
              avh_calir(3,i,j) = avh_calir(1,i,j)     !offset
              avh_calir(1,i,j) = tmp1*scaleq_ch45     !quadratic 10^6 or 10^7 -> 10^9
            enddo
          enddo
          avh_filler2(1) = 9    !Scale for quadratic term
          avh_filler2(2) = 6    !Scale for linear term
          avh_filler2(3) = 6    !Scale for offset
          
!Re-scale the visible coefs

          do i=1,3   !set
            do j=1,3   !channel
              avh_calvis(1,i,j) = avh_calvis(1,i,j)*1000   !slope 10^7 -> 10^10
              avh_calvis(2,i,j) = avh_calvis(2,i,j)*10     !offset 10^6 -> 10^7
              avh_calvis(3,i,j) = avh_calvis(3,i,j)*1000   !slope
              avh_calvis(4,i,j) = avh_calvis(4,i,j)*10     !offset
            enddo
          enddo

!For avh_scnlinbit, reverse bit 0 (see Data Formats document section 8)

          tmp1 = avh_scnlinbit
          if (IAND(tmp1,1).EQ.1) then
            tmp1 = IAND(tmp1,65534)
          else
            tmp1 = IOR(tmp1,1)
          endif
          avh_scnlinbit = tmp1

!Write out data record

          call avhdtw(2,irec,ios)
          if(ios.ne.0) then
            write(*,*)'Unable to write data record',irec
            call aappexit(1)
          endif
        else
          write(*,*)'Unable to read data record',irec
          call aappexit(1)
        endif  

      enddo
        
      close(1)
      close(2)
      write(*,*)'Read   ',irec-1,' scans from ',class_file
      write(*,*)'Written',irec-1,' scans to   ',aapp_file

      END
