C LIBS = atovin atov auxaapp f7nl1b
!
      PROGRAM COMBINE_1C
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: combine_1c.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose: Combine together two 1c files (same instrument: HIRS, AMSU-A,
!   AMSU-B or IASI)
!
! Description:
!   Run this prior to atovpp to build up longer files from granules, if the
!   granules for different instruments are not time-synchronised.
!
!   The file on unit 10 is appended to the file on unit 11. They must be in
!   the correct time order, with no time gap between them.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2006, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  21/9/06  N C Atkinson
! 1.1  05/1/07  recognise MHS
! 1.2  08/11/10 allow for the case where an added file starts just after midnight
! 1.3  21/06/11 Add CrIS and ATMS
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "ppparms.h"
#include "hrs1c.h"
#include "ama1c.h"
#include "amb1c.h"
#include "iasi1c.h"
#include "cris1c.h"
#include "atms1c.h"

      INTEGER unitin, unitcombined
      DATA unitin/10/, unitcombined/11/
      INTEGER scnlin,endorbit,enddatayr,enddatady,enddatatime
      INTEGER scnlin_start,lasttime,tdiff,iscan,errStatus,instr_code
      INTEGER lastscan,linediff,I
     
      CHARACTER*6 instrument
!
!- End of header

!------------------------------------------------------------------------------
!1) Read the instrument type from standard input
!------------------------------------------------------------------------------

      READ(*,*) instrument

!------------------------------------------------------------------------------
!2) Process AMSU-A
!------------------------------------------------------------------------------

      IF (instrument.EQ."AMSU-A") THEN
        CALL IOA1C ("OPEN/WRITE",unitcombined,0)
        CALL IOA1C ("OPEN/READ",unitin,0)
        READ ( UNIT=unitcombined,REC=1,IOSTAT=errStatus ) ama1c_hd  !Exists?

        IF (errStatus.NE.0) THEN                 !Create new combined file
          CALL IOA1C ("READ HEADER",unitin,1)
          CALL IOA1C ("WRITE HEADER",unitcombined,1)
          DO iscan=1,ama1c_h_scnlin
            CALL IOA1C ("READ DATA",unitin,1+iscan)
            CALL IOA1C ("WRITE DATA",unitcombined,1+iscan)
          ENDDO
        ELSE
        
!Read the combined file header and the time and number of the last record

          CALL IOA1C ("READ HEADER",unitcombined,1)  !read combined file
          instr_code = ama1c_h_instrument
          scnlin_start = ama1c_h_scnlin
          CALL IOA1C ("READ DATA",unitcombined,1+ama1c_h_scnlin)
          lasttime = ama1c_scnlintime
          lastscan = ama1c_scnlin

!Read the input file header

          CALL IOA1C ("READ HEADER",unitin,1)
          scnlin = ama1c_h_scnlin
          endorbit = ama1c_h_endorbit
          enddatayr = ama1c_h_enddatayr
          enddatady = ama1c_h_enddatady
          enddatatime = ama1c_h_enddatatime
          IF (ama1c_h_instrument.NE.instr_code) THEN
            write(*,*)'Error: instruments do not match',
     &            ama1c_h_instrument,instr_code
            call aappexit(1)
          ENDIF

!Read in each new data record and append to the end of the combined file
!Check that the time of the first scan is valid

          DO iscan=1,scnlin
            CALL IOA1C ("READ DATA",unitin,1+iscan)
            IF (iscan.eq.1) THEN
              tdiff = ama1c_scnlintime - lasttime
              IF (tdiff.LT. -43200000) tdiff = tdiff + 86400000
              IF (tdiff.GT.Acycle*1.1.OR.tdiff.LT.Acycle*0.9) THEN
                write(*,*)'Error: data gap of ',tdiff,' milliseconds'
                call aappexit(1)
              ENDIF
              linediff = ama1c_scnlin - lastscan - 1
            ENDIF
            ama1c_scnlin = ama1c_scnlin - linediff   !Adjust scan line number
            CALL IOA1C ("WRITE DATA",unitcombined,1+scnlin_start+iscan)
          ENDDO

!Update the combined file header

          CALL IOA1C ("READ HEADER",unitcombined,1)
          ama1c_h_scnlin = scnlin_start + scnlin
          ama1c_h_endorbit = endorbit
          ama1c_h_enddatayr = enddatayr 
          ama1c_h_enddatady = enddatady
          ama1c_h_enddatatime = enddatatime
          CALL IOA1C ("WRITE HEADER",unitcombined,1)
        ENDIF

!Close files

        CALL IOA1C ("CLOSE",unitin,0)
        CALL IOA1C ("CLOSE",unitcombined,0)

!------------------------------------------------------------------------------
!3) Process AMSU-B/MHS
!------------------------------------------------------------------------------

      ELSE IF (instrument.EQ."AMSU-B".OR.instrument.EQ."MHS") THEN
        CALL IOB1C ("OPEN/WRITE",unitcombined,0)
        CALL IOB1C ("OPEN/READ",unitin,0)
        READ ( UNIT=unitcombined,REC=1,IOSTAT=errStatus ) amb1c_hd  !Exists?

        IF (errStatus.NE.0) THEN                 !Create new combined file
          CALL IOB1C ("READ HEADER",unitin,1)
          CALL IOB1C ("WRITE HEADER",unitcombined,1)
          DO iscan=1,amb1c_h_scnlin
            CALL IOB1C ("READ DATA",unitin,1+iscan)
            CALL IOB1C ("WRITE DATA",unitcombined,1+iscan)
          ENDDO
        ELSE
        
!Read the combined file header and the time and number of the last record

          CALL IOB1C ("READ HEADER",unitcombined,1)  !read combined file
          instr_code = amb1c_h_instrument
          scnlin_start = amb1c_h_scnlin
          CALL IOB1C ("READ DATA",unitcombined,1+amb1c_h_scnlin)
          lasttime = amb1c_scnlintime
          lastscan = amb1c_scnlin

!Read the input file header

          CALL IOB1C ("READ HEADER",unitin,1)
          scnlin = amb1c_h_scnlin
          endorbit = amb1c_h_endorbit
          enddatayr = amb1c_h_enddatayr
          enddatady = amb1c_h_enddatady
          enddatatime = amb1c_h_enddatatime
          IF (amb1c_h_instrument.NE.instr_code) THEN
            write(*,*)'Error: instruments do not match',
     &            amb1c_h_instrument,instr_code
            call aappexit(1)
          ENDIF

!Read in each new data record and append to the end of the combined file
!Check that the time of the first scan is valid

          DO iscan=1,scnlin
            CALL IOB1C ("READ DATA",unitin,1+iscan)
            IF (iscan.eq.1) THEN
              tdiff = amb1c_scnlintime - lasttime
              IF (tdiff.LT. -43200000) tdiff = tdiff + 86400000
              IF (tdiff.GT.Bcycle*1.1.OR.tdiff.LT.Bcycle*0.9) THEN
                write(*,*)'Error: data gap of ',tdiff,' milliseconds'
                call aappexit(1)
              ENDIF
              linediff = amb1c_scnlin - lastscan - 1
            ENDIF
            amb1c_scnlin = amb1c_scnlin - linediff   !Adjust scan line number
            CALL IOB1C ("WRITE DATA",unitcombined,1+scnlin_start+iscan)
          ENDDO

!Update the combined file header

          CALL IOB1C ("READ HEADER",unitcombined,1)
          amb1c_h_scnlin = scnlin_start + scnlin
          amb1c_h_endorbit = endorbit
          amb1c_h_enddatayr = enddatayr 
          amb1c_h_enddatady = enddatady
          amb1c_h_enddatatime = enddatatime
          CALL IOB1C ("WRITE HEADER",unitcombined,1)
        ENDIF

!Close files

        CALL IOB1C ("CLOSE",unitin,0)
        CALL IOB1C ("CLOSE",unitcombined,0)

!------------------------------------------------------------------------------
!4) Process HIRS
!------------------------------------------------------------------------------

      ELSE IF (instrument.EQ."HIRS") THEN
        CALL IOH1C ("OPEN/WRITE",unitcombined,0)
        CALL IOH1C ("OPEN/READ",unitin,0)
        READ ( UNIT=unitcombined,REC=1,IOSTAT=errStatus ) hrs1c_hd  !Exists?

        IF (errStatus.NE.0) THEN                 !Create new combined file
          CALL IOH1C ("READ HEADER",unitin,1)
          CALL IOH1C ("WRITE HEADER",unitcombined,1)
          DO iscan=1,hrs1c_h_scnlin
            CALL IOH1C ("READ DATA",unitin,1+iscan)
            CALL IOH1C ("WRITE DATA",unitcombined,1+iscan)
          ENDDO
        ELSE
        
!Read the combined file header and the time and number of the last record

          CALL IOH1C ("READ HEADER",unitcombined,1)  !read combined file
          instr_code = hrs1c_h_instrument
          scnlin_start = hrs1c_h_scnlin
          CALL IOH1C ("READ DATA",unitcombined,1+hrs1c_h_scnlin)
          lasttime = hrs1c_scnlintime
          lastscan = hrs1c_scnlin

!Read the input file header

          CALL IOH1C ("READ HEADER",unitin,1)
          scnlin = hrs1c_h_scnlin
          endorbit = hrs1c_h_endorbit
          enddatayr = hrs1c_h_enddatayr
          enddatady = hrs1c_h_enddatady
          enddatatime = hrs1c_h_enddatatime
          IF (hrs1c_h_instrument.NE.instr_code) THEN
            write(*,*)'Error: instruments do not match',
     &            hrs1c_h_instrument,instr_code
            call aappexit(1)
          ENDIF

!Read in each new data record and append to the end of the combined file
!Check that the time of the first scan is valid

          DO iscan=1,scnlin
            CALL IOH1C ("READ DATA",unitin,1+iscan)
            IF (iscan.eq.1) THEN
              tdiff = hrs1c_scnlintime - lasttime
              IF (tdiff.LT. -43200000) tdiff = tdiff + 86400000
              IF (tdiff.GT.Hcycle*1.1.OR.tdiff.LT.Hcycle*0.9) THEN
                write(*,*)'Error: data gap of ',tdiff,' milliseconds'
                call aappexit(1)
              ENDIF
              linediff = hrs1c_scnlin - lastscan - 1
            ENDIF
            hrs1c_scnlin = hrs1c_scnlin - linediff   !Adjust scan line number
            CALL IOH1C ("WRITE DATA",unitcombined,1+scnlin_start+iscan)
          ENDDO

!Update the combined file header

          CALL IOH1C ("READ HEADER",unitcombined,1)
          hrs1c_h_scnlin = scnlin_start + scnlin
          hrs1c_h_endorbit = endorbit
          hrs1c_h_enddatayr = enddatayr 
          hrs1c_h_enddatady = enddatady
          hrs1c_h_enddatatime = enddatatime
          CALL IOH1C ("WRITE HEADER",unitcombined,1)
        ENDIF

!Close files

        CALL IOH1C ("CLOSE",unitin,0)
        CALL IOH1C ("CLOSE",unitcombined,0)

!------------------------------------------------------------------------------
!5) Process IASI
!------------------------------------------------------------------------------

      ELSE IF (instrument.EQ."IASI") THEN
        CALL IOI1C ("OPEN/WRITE",unitcombined,0)
        CALL IOI1C ("OPEN/READ",unitin,0)
        READ ( UNIT=unitcombined,REC=1,IOSTAT=errStatus ) iasi1c_hd  !Exists?

        IF (errStatus.NE.0) THEN                 !Create new combined file
          CALL IOI1C ("READ HEADER",unitin,1)
          CALL IOI1C ("WRITE HEADER",unitcombined,1)
          DO iscan=1,iasi1c_h_scnlin
            CALL IOI1C ("READ DATA",unitin,1+iscan)
            CALL IOI1C ("WRITE DATA",unitcombined,1+iscan)
          ENDDO
        ELSE
        
!Read the combined file header and the time and number of the last record

          CALL IOI1C ("READ HEADER",unitcombined,1)  !read combined file
          instr_code = iasi1c_h_instrument
          scnlin_start = iasi1c_h_scnlin
          CALL IOI1C ("READ DATA",unitcombined,1+iasi1c_h_scnlin)
          lasttime = iasi1c_scnlintime(1)
          lastscan = iasi1c_scnlin

!Read the input file header

          CALL IOI1C ("READ HEADER",unitin,1)
          scnlin = iasi1c_h_scnlin
          endorbit = iasi1c_h_endorbit
          enddatayr = iasi1c_h_enddatayr
          enddatady = iasi1c_h_enddatady
          enddatatime = iasi1c_h_enddatatime
          IF (iasi1c_h_instrument.NE.instr_code) THEN
            write(*,*)'Error: instruments do not match',
     &            iasi1c_h_instrument,instr_code
            call aappexit(1)
          ENDIF

!Read in each new data record and append to the end of the combined file
!Check that the time of the first scan is valid

          DO iscan=1,scnlin
            CALL IOI1C ("READ DATA",unitin,1+iscan)
            IF (iscan.eq.1) THEN
              tdiff = iasi1c_scnlintime(1) - lasttime
              IF (tdiff.LT. -43200000) tdiff = tdiff + 86400000
              IF (tdiff.GT.Icycle*1.1.OR.tdiff.LT.Icycle*0.9) THEN
                write(*,*)'Error: data gap of ',tdiff,' milliseconds'
                call aappexit(1)
              ENDIF
              linediff = iasi1c_scnlin - lastscan - 1
            ENDIF
            iasi1c_scnlin = iasi1c_scnlin - linediff   !Adjust scan line number
            CALL IOI1C ("WRITE DATA",unitcombined,1+scnlin_start+iscan)
          ENDDO

!Update the combined file header

          CALL IOI1C ("READ HEADER",unitcombined,1)
          iasi1c_h_scnlin = scnlin_start + scnlin
          iasi1c_h_endorbit = endorbit
          iasi1c_h_enddatayr = enddatayr 
          iasi1c_h_enddatady = enddatady
          iasi1c_h_enddatatime = enddatatime
          CALL IOI1C ("WRITE HEADER",unitcombined,1)
        ENDIF

!Close files

        CALL IOI1C ("CLOSE",unitin,0)
        CALL IOI1C ("CLOSE",unitcombined,0)

!------------------------------------------------------------------------------
!6) Process CrIS
!------------------------------------------------------------------------------

      ELSE IF (instrument.EQ."CRIS") THEN
      
!First get the record length of the input file
        cris1c_h_recwords = array_size_cris1c - 
     &      cris_1cnchan*cris_1cnfov*cris_1cnfor !excluding channels
        CALL IOC1C ("OPEN/READ",unitin,0)
        CALL IOC1C ("READ HEADER",unitin,1)      !Read header
        CALL IOC1C ("CLOSE",unitin,1)
        CALL IOC1C ("OPEN/READ",unitin,0)        !Re-open with correct length
 
!Open the combined file
        CALL IOC1C ("OPEN/WRITE",unitcombined,0)
        READ ( UNIT=unitcombined,REC=1,IOSTAT=errStatus ) 
     &      (cris1c_hd(I),I=1,cris1c_h_recwords)  !Exists?

        IF (errStatus.NE.0) THEN                 !Create new combined file
          CALL IOC1C ("WRITE HEADER",unitcombined,1)
          DO iscan=1,cris1c_h_scnlin
            CALL IOC1C ("READ DATA",unitin,1+iscan)
            CALL IOC1C ("WRITE DATA",unitcombined,1+iscan)
          ENDDO
        ELSE
        
!Read the combined file header and the time and number of the last record

          CALL IOC1C ("READ HEADER",unitcombined,1)  !read combined file
          instr_code = cris1c_h_instrument
          scnlin_start = cris1c_h_scnlin
          CALL IOC1C ("READ DATA",unitcombined,1+cris1c_h_scnlin)
          lasttime = cris1c_scnlintime(1)
          lastscan = cris1c_scnlin

!Read the input file header

          CALL IOC1C ("READ HEADER",unitin,1)
          scnlin = cris1c_h_scnlin
          endorbit = cris1c_h_endorbit
          enddatayr = cris1c_h_enddatayr
          enddatady = cris1c_h_enddatady
          enddatatime = cris1c_h_enddatatime
          IF (cris1c_h_instrument.NE.instr_code) THEN
            write(*,*)'Error: instruments do not match',
     &            cris1c_h_instrument,instr_code
            call aappexit(1)
          ENDIF

!Read in each new data record and append to the end of the combined file
!Check that the time of the first scan is valid

          DO iscan=1,scnlin
            CALL IOC1C ("READ DATA",unitin,1+iscan)
            IF (iscan.eq.1) THEN
              tdiff = cris1c_scnlintime(1) - lasttime
              IF (tdiff.LT. -43200000) tdiff = tdiff + 86400000
              IF (tdiff.GT.criscycle*1.3.OR.tdiff.LT.criscycle*0.7) THEN
                write(*,*)'Error: data gap of ',tdiff,' milliseconds'
                call aappexit(1)
              ENDIF
              linediff = cris1c_scnlin - lastscan - 1
            ENDIF
            cris1c_scnlin = cris1c_scnlin - linediff   !Adjust scan line number
            CALL IOC1C ("WRITE DATA",unitcombined,1+scnlin_start+iscan)
          ENDDO

!Update the combined file header

          CALL IOC1C ("READ HEADER",unitcombined,1)
          cris1c_h_scnlin = scnlin_start + scnlin
          cris1c_h_endorbit = endorbit
          cris1c_h_enddatayr = enddatayr 
          cris1c_h_enddatady = enddatady
          cris1c_h_enddatatime = enddatatime
          CALL IOC1C ("WRITE HEADER",unitcombined,1)
        ENDIF

!Close files

        CALL IOC1C ("CLOSE",unitin,0)
        CALL IOC1C ("CLOSE",unitcombined,0)

!------------------------------------------------------------------------------
!7) Process ATMS
!------------------------------------------------------------------------------

      ELSE IF (instrument.EQ."ATMS") THEN
        CALL IOAT1C ("OPEN/WRITE",unitcombined,0)
        CALL IOAT1C ("OPEN/READ",unitin,0)
        READ ( UNIT=unitcombined,REC=1,IOSTAT=errStatus ) atms1c_hd  !Exists?

        IF (errStatus.NE.0) THEN                 !Create new combined file
          CALL IOAT1C ("READ HEADER",unitin,1)
          CALL IOAT1C ("WRITE HEADER",unitcombined,1)
          DO iscan=1,atms1c_h_scnlin
            CALL IOAT1C ("READ DATA",unitin,1+iscan)
            CALL IOAT1C ("WRITE DATA",unitcombined,1+iscan)
          ENDDO
        ELSE
        
!Read the combined file header and the time and number of the last record

          CALL IOAT1C ("READ HEADER",unitcombined,1)  !read combined file
          instr_code = atms1c_h_instrument
          scnlin_start = atms1c_h_scnlin
          CALL IOAT1C ("READ DATA",unitcombined,1+atms1c_h_scnlin)
          lasttime = atms1c_scnlintime(1)
          lastscan = atms1c_scnlin

!Read the input file header

          CALL IOAT1C ("READ HEADER",unitin,1)
          scnlin = atms1c_h_scnlin
          endorbit = atms1c_h_endorbit
          enddatayr = atms1c_h_enddatayr
          enddatady = atms1c_h_enddatady
          enddatatime = atms1c_h_enddatatime
          IF (atms1c_h_instrument.NE.instr_code) THEN
            write(*,*)'Error: instruments do not match',
     &            atms1c_h_instrument,instr_code
            call aappexit(1)
          ENDIF

!Read in each new data record and append to the end of the combined file
!Check that the time of the first scan is valid

          DO iscan=1,scnlin
            CALL IOAT1C ("READ DATA",unitin,1+iscan)
            IF (iscan.eq.1) THEN
              tdiff = atms1c_scnlintime(1) - lasttime
              IF (tdiff.LT. -43200000) tdiff = tdiff + 86400000
              IF (tdiff.GT.Bcycle*1.6.OR.tdiff.LT.Bcycle*0.7) THEN
                write(*,*)'Error: data gap of ',tdiff,' milliseconds'
                call aappexit(1)
              ENDIF
              linediff = atms1c_scnlin - lastscan - 1
            ENDIF
            atms1c_scnlin = atms1c_scnlin - linediff   !Adjust scan line number
            CALL IOAT1C ("WRITE DATA",unitcombined,1+scnlin_start+iscan)
          ENDDO

!Update the combined file header

          CALL IOAT1C ("READ HEADER",unitcombined,1)
          atms1c_h_scnlin = scnlin_start + scnlin
          atms1c_h_endorbit = endorbit
          atms1c_h_enddatayr = enddatayr 
          atms1c_h_enddatady = enddatady
          atms1c_h_enddatatime = enddatatime
          CALL IOAT1C ("WRITE HEADER",unitcombined,1)
        ENDIF

!Close files

        CALL IOAT1C ("CLOSE",unitin,0)
        CALL IOAT1C ("CLOSE",unitcombined,0)

      ENDIF
      END
