#!/bin/ksh
#
# $Id: convert_to_hdf5.ksh 536 2017-08-09 10:40:30Z frna $
#
# Convert AAPP l1c/l1d files to HDF-5
#
# History:
# 1.1   26/09/2014  NCA
# 1.2   23/02/2015  od needs the -v option
# 1.3   25/09/2015  Add mwhs2, mwts2, iras
# 1.4   13/01/2016  Add the instrument IDs for HIRS
# 1.5   27/01/2016  Add full res geo option for AVHRR (-G)
# 1.6   09/06/2016  Add geographical subset option for AVHRR l1c (-s)
# 1.7   08/08/2017  Add mwri
#
#==============================================================================

if [ "$AAPP_PREFIX" = "" ]; then
  echo 'Error: AAPP_PREFIX not defined'
  exit 2
fi
if [ ! -d "$AAPP_PREFIX" ]; then
  echo 'Error: AAPP_PREFIX is not a directory:' $AAPP_PREFIX
  exit 2
fi

. "$AAPP_PREFIX/ATOVS_CONF"

status=0

[ $# = 0 ] && { echo "Usage: convert_to_hdf5 [-c[0-9]] [-G] [-s lat lon nx ny] files"; \
  echo "where files is a list of AAPP-format l1c/l1d files"; \
  echo " -c[0-9] optionally invokes built-in gzip compression (0=none, 9=maximum)";
  echo " -G optionally writes out full resolution geolocation instead of tie points";
  echo " -s lat0 lon0 nx ny allows the user to specify a geographical subset for AVHRR 1c"; }

while [[ ${1:0:1} == - ]]; do
  if [[ $1 = -G ]]; then    #see if full resolution geo requested for AVHRR
    fullresgeo="$1"
    shift
  fi
  if [[ $1 = -c* ]]; then   #see if compression has been specified (0 to 9)
    clevel=${1#*c}
    clevel=${clevel:-5}     #default value if caller has used "-c"
    shift
  fi
  if [[ $1 = -s ]]; then    #subset for AVHRR
    shift; lat0="$1"
    shift; lon0="$1"
    shift; nx="$1"
    shift; ny="$1"
    shift
    clevel=${clevel:-0}     #need clevel to be set to something
  fi
done

for file in $*; do
  
  if [ -s $file ]; then
  
#First see whether it's level 1b. All level 1b have byte 17-22 as zero fill.

    set $(head -c 78 $file | od -v -An -t u2)
    if [ $9 = 0 ] && [ ${10} = 0 ] && [ ${11} = 0 ]; then
      level=1b
      datatyp=${39}
      ins=""
      case $datatyp in 
         1 |  256 ) ins=lac ;;
         2 |  512 ) ins=gac ;;
         3 |  768 ) ins=hrpt ;;
         4 | 1024 ) ins=frac ;;
         5 | 1280 ) ins=hrs ;;
         6 | 1536 ) ins=msu ;;
         7 | 1792 ) ins=ssu ;;
         8 | 2048 ) ins=dcs ;;
         9 | 2304 ) ins=sem ;;
        10 | 2560 ) ins=ama ;;
        11 | 2816 ) ins=amb ;;
        12 | 3072 ) ins=mhs ;;
        13 | 3328 ) ins=frac ;;
         * ) { echo "Error: cannot identify instrument"; status=1; }
      esac

      if [ "$ins" = lac ] || [ "$ins" = gac ] || [ "$ins" = hrpt ] || [ "$ins" = frac ]; then
        prog=hrp1b_to_hdf5.exe     #assume it's already in AAPP format, not NOAA
        echo $prog $fullresgeo $file ${file}.h5 $clevel
        $prog $fullresgeo $file ${file}.h5 $clevel || status=$?
      else
        echo "file type ${ins} level ${level} is not supported"
      fi

    else   #level 1c/1d

#Check that byte 8 is " " or 0

      set $(head -c 8 $file | od -v -An -t u1)
      if [ $8 != 32 ] && [ $8 != 0 ]; then
        echo "Error: input file is not level 1c/1d"
        status=1
      else 

#Read the first 88 bytes as u4 words, hence get instrument and level.
#If it's level 1d then word 22 contains the instrument bit field and word 21
#contains atovppvn. If it's level 1c then either word 21 is zero (HIRS) or word
#22 is zero (all others).
#Note that AVHRR l1c/l1d files from AAPP have instrument code 0, though the
#format spec says it should be 1. NOAA CLASS files have the flight model number.

        set $(head -c 88 $file | od -v -An -t u4)

        instr=$8

        case $instr in 
           0             ) ins=avh ;;
           1 |  16777216 ) ins=avh ;;
           5 |  83886080 ) ins=hrs ;;
          10 | 167772160 ) ins=ama ;;
          11 | 184549376 ) ins=amb ;;
          12 | 201326592 ) ins=amb ;;
          13 | 218103808 ) ins=iasi ;;
          14 | 234881024 ) ins=iasipc ;;
          17 | 285212672 ) ins=iras ;;
          18 | 301989888 ) ins=mwts2 ;;
          19 | 318767104 ) ins=mwhs2 ;;
          20 | 335544320 ) ins=mwri ;;
         620 | 1812070400 ) ins=cris ;;
         621 | 1828847616 ) ins=atms ;;
         301 | 755040256 ) ins=avh ;;
         302 | 771817472 ) ins=avh ;;
         303 | 788594688 ) ins=avh ;;
         304 | 805371904 ) ins=avh ;;
         305 | 822149120 ) ins=avh ;;
         306 | 838926336 ) ins=avh ;;
         307 | 855703552 ) ins=avh ;;
         308 | 872480768 ) ins=avh ;;
         309 | 889257984 ) ins=avh ;;
           * ) { echo "Error: cannot identify instrument"; exit 1; }
        esac

        if [ "$ins" = avh ]; then
          level=1c   #AVHRR has the same executable for 1c and 1d
        elif [ "$ins" = iasipc ]; then
          level=pc
          ins=iasi
        elif [ ${22} = 0 ] || [ ${21} = 0 ]; then
          level=1c
        else
          level=1d  
        fi
  
        prog=${ins}${level}_to_hdf5.exe

        if [ ${ins}${level} != avh1c ]; then
          fullresgeo=""     #AVHRR 1c only
        fi
        echo $prog $fullresgeo $file ${file}.h5 $clevel $lat0 $lon0 $nx $ny
        $prog $fullresgeo $file ${file}.h5 $clevel $lat0 $lon0 $nx $ny || status=$?

      fi      #byte 8 check
    fi      #level 1c/1d

  else
  
    echo "Cannot find $file"
    status=1

  fi
done

[ $status = 0 ] || echo "There were errors!" 
exit $status
