#!/usr/bin/perl -w
#
# $Id: gtsheaders_bufr.pl 562 2017-10-30 13:26:58Z frna $
#
# Convert BUFR messages into GTS bulletins by adding the headers described in 
# the WMO Manual on the GTS.
#
# See page A.II-15/30 for definition of the message structure and A.II-5/1 for
# definition of T1T2A1A2.
#
# COPYRIGHT
#    This software was developed within the context of the EUMETSAT Satellite
#    Application Facility on Numerical Weather Prediction (NWP SAF), under the
#    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
#    Met Office, UK, by one or more partners within the NWP SAF. The partners
#    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
#
#    Copyright 2017, EUMETSAT, All Rights Reserved. 
#
# 07/04/2008 Nigel Atkinson
# 25/04/2008 Make more robust - use the message length info in Section 0
# 20/06/2008 Allow specification of starting ii and add "--noincrement" option
# 17/07/2012 If YYGGgg is not specified by the user, take it from Section 1
# 16/10/2015 Correction 4096 -> 65536 in the message length computation
# 25/10/2017 Add to AAPP v8. Don't allow $seqcount to exceed 99.
##############################################################################

use strict;
use Getopt::Long;

sub usage
{
  print <<"END-OF-USAGE";
usage: gtsheaders_bufr.pl infile outfile T1T2A1A2[ii] CCCC [YYGGgg]
  where T1T2A1A2 is the product identifier (excluding the sequence number)
     For satellite data T1T2=IN and A1A2 is a geographical indicator
  CCCC is the originating centre (ICAO location indicator)
  YYGGgg is optional day-of-month hour minute (otherwise uses Section 1 time)
END-OF-USAGE
exit 0
}

my $noincrement = 0;
GetOptions ('noincrement' => \$noincrement);

my ($line, $len, $message, $messlen, $messlen4, $outlen, $startline, $seq); 
my ($char1, $char2, $char3, $len1, $len2, $len3, $bytesread, $line1);
my $recordcount = 0;
my $seqcount = 0;
my $startseq = 1;
my $lastddhhmn = "999999";
my $ddhhmn;

#Read arguments

my ($infile, $outfile, $t1t2a1a2, $cccc, $ddhhmn_arg) = @ARGV;

if ($ddhhmn_arg) {
  $ddhhmn = $ddhhmn_arg;
} else {
  $ddhhmn = "999999";
}

#Check number of arguments

usage if ( ! $cccc );

#Check length of arguments

if ( length($t1t2a1a2) == 6 ) {
  $startseq = substr($t1t2a1a2, 4, 2);
  $t1t2a1a2 = substr($t1t2a1a2, 0, 4)
} 

die "Error: length of T1T2A1A2 is not 4 bytes" if ( length($t1t2a1a2) != 4 );
die "Error: length of CCCC is not 4 bytes" if ( length($cccc) != 4 );
  
#Set up fixed part of header and end-of-message: 
#  <SOH><CR><CR><LF>000<CR><CR><LF>$t1t2a1a2ii $cccc $ddhhmn<CR><CR><LF>
#  <CR><CR><LF><ETX>

my $header1 = "\001\015\015\012000\015\015\012".$t1t2a1a2;
my $header2 = " ".$cccc." ".$ddhhmn."\015\015\012";   #after the "ii" sequence ID (n.b. changed later)
my $endmess = "\015\015\012\003";
my $terminator = "0000000000";
my $headerlen = length ($header1) + 2 + length ($header2);
my $endmesslen = length ($endmess);

$/ = "BUFR";       #set record terminator to look for start of each message

open (IN, $infile) || die "can't open $infile\n";
open (OUT, '>', $outfile) || die "can't create $outfile\n";

$line = <IN>;      #go to start of first message

while ($line = <IN>)   #read up to the next "BUFR"
{

#get the actual message length from the first 3 bytes

  $char1 = substr($line,0,1);
  $char2 = substr($line,1,1);
  $char3 = substr($line,2,1);
  ($len1,$len2,$len3) = (ord($char1),ord($char2),ord($char3));
  $messlen = ord($char1)*65536 + ord($char2)*256 + ord($char3);
  $messlen4 = $messlen - 4;  

#extract the message. Read more records in the unlikely event that there is an 
#embedded "BUFR".

  $len = length($line);
  while ($len < $messlen4)
  {
    die "end of file" if (not $line1 = <IN>);
    print "Warning: found embedded \"BUFR\", file could be corrupt\n";
    print "  current message length $len, expect $messlen4\n";
    $line = $line . $line1;
    $len = length($line);
  }
  $message = "BUFR".substr($line,0,$messlen4);
  
#Get Edition and Section 1 date (if YYGGgg is not specified by the user)
  if (! $ddhhmn_arg){
    my $edition = ord(substr($message,7,1));
    my ($dayofmonth, $hour, $minute);

    if ($edition == 4) {
      $dayofmonth = ord(substr($message,26,1));
      $hour = ord(substr($message,27,1));
      $minute = ord(substr($message,28,1));
    } else {
      $dayofmonth = ord(substr($message,22,1));
      $hour = ord(substr($message,23,1));
      $minute = ord(substr($message,24,1));
    }
    $ddhhmn = sprintf("%2.2d%2.2d%2.2d",$dayofmonth,$hour,$minute);
    $header2 = " ".$cccc." ".$ddhhmn."\015\015\012";   #update with new $ddhhmn
  }

#compute the total length to be output

  $outlen = $messlen + $headerlen + $endmesslen;
  $startline = sprintf("%8.8d00",$outlen);

#increment sequence count and write out the various parts of the message

  if ($noincrement || ($ddhhmn != $lastddhhmn)) {
    $seqcount = $startseq;
    $lastddhhmn = $ddhhmn;
  }
  $seq = sprintf("%2.2d",$seqcount);
  $recordcount++;
  $seqcount++;
  $seqcount = 1 if ($seqcount > 99);

  print OUT $startline.$header1.$seq.$header2.$message.$endmess;

  print "$recordcount $seq $ddhhmn $messlen\n";
}

if ($recordcount > 0) {
  print OUT $terminator;
} else {
  print "Warning: no BUFR records found in $infile\n"
}

close (IN);
close (OUT);
