C LIBS = f7nl1b auxaapp f7ml f7tp f7cp f7gp
!
      PROGRAM HIRS2_CLASS_TO_AAPP
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: hirs2_class_to_aapp.F 380 2014-01-23 12:35:13Z frna $
!
! Purpose: Convert HIRS/2 (TOVS) 1b files from NOAA/CLASS to AAPP format
!
! Inputs (read from std input):
!   Input CLASS 1b file name, conforming to NSS.HIRX.N[A-J].D?????.*
!   Output AAPP file name
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  30/08/2013  N C Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "hrs1b.h"
#include "reclopt.h"    
#include "hrs1b_class.h"
#include "nl1b.h"
#include "constants_preproc.h"

      character*200 class_file_in  !Can include directory
      character*80  class_file
      character*200 aapp_file
      character*2 satid
      character*6 satname
      character*200 dir_calibration
      character*80 line

      integer pos
      integer recl_class
      integer loc_rl
      integer ios
      integer year
      integer irec
      integer scnlintime_old
      integer tdiff
      integer scandiff
      integer i,j
      integer cal
      integer ichan
      integer endpos
      
      integer*2 i2wordm8,i2word7,i2word1023,i2word64
      data i2wordm8/Z'FFF8'/
      data i2word7/7/
      data i2word1023/1023/
      data i2word64/64/
      
      logical swap
      data swap/.false./

      real scanang
      real sinz
      real*8 wavenum(19)
      real*8 bandcor(2,19)
      real intercept

!
!- End of header

      read(*,'(A)') class_file_in
      read(*,'(A)') aapp_file
      pos = 1
      do i=1,200
        if (class_file_in(i:i).eq."/") pos=i+1
      enddo
      class_file = class_file_in(pos:pos+41)
      
      if (class_file(5:8) .ne. "HIRX") then
        write(*,*)'Error: input file name must be of the form'
        write(*,*)'NSS.HIRX.N[A-J].D?????.*'
        call aappexit(1)
      endif

      read(class_file,'(13X,I2)') year
      if (year.gt.50.and.year.lt.95) then
        recl_class = 4256
      else
        recl_class = 4253
        if (rldiv .ne. 1) then
          write(*,*)'Error: the HIRS 1b record length (4253) is an'
          write(*,*)'odd number of bytes, so record length specifier in'
          write(*,*)'"open" must be in bytes. For ifort, choose the'
          write(*,*)'compiler option "-assume byterecl"'
          call aappexit(1)
        endif
      endif

!Is byte swapping needed? CLASS data are always big-endian

      hrs_c_scnlin = 1
      if (ichar(hrs_c_char(1:1)) .eq. 1) swap = .true.  !Little-endian platform

!Fixed items in header

      hrs_h_siteid = 'nss'           !Make it lower case, so that atovin doesn't
                                     !subtract 4096 counts (see inhget.F)
      hrs_h_l1bversnb = libl1bvers   !in nl1b.h
      hrs_h_l1bversyr = libl1bversyr !in nl1b.h
      hrs_h_l1bversdy = libl1bversdy !in nl1b.h
      hrs_h_reclg = reclhrs          !in hrs1b.h
      hrs_h_hdrcnt = 1
      hrs_h_dataname = class_file
      hrs_h_datatyp = 5              !5=HIRS
      hrs_h_tipsrc = 2               !2=stored
      hrs_h_startdatady = 0

      satid = class_file(10:11)
      if (satid .eq. 'TN') then        !noaa05 (Tiros-N). This is an AAPP convention.
        satname = 'noaa05'
      else if (satid .eq. 'NA') then   !noaa06
        satname = 'noaa06'
      else if (satid .eq. 'NC') then   !noaa07
        satname = 'noaa07'
      else if (satid .eq. 'NE') then   !noaa08
        satname = 'noaa08'
      else if (satid .eq. 'NF') then   !noaa09
        satname = 'noaa09'
      else if (satid .eq. 'NG') then   !noaa10
        satname = 'noaa10'
      else if (satid .eq. 'NH') then   !noaa11
        satname = 'noaa11'
      else if (satid .eq. 'ND') then   !noaa12
        satname = 'noaa12'
      else if (satid .eq. 'NI') then   !noaa13
        satname = 'noaa13'
      else if (satid .eq. 'NJ') then   !noaa14
        satname = 'noaa14'
      else
        write(*,*)'Error: invalid satid ',satid
        call aappexit(1)
      endif

      hrs_h_callocsclin = 0
      scnlintime_old = 0
      
!Get channel info from AAPP data file

      dir_calibration = " "
      endpos = 1
      call getenv("DIR_CALIBRATION",dir_calibration)
      do i=1,LEN(dir_calibration)
        if (dir_calibration(i:i) .ne. " ") endpos = i
      enddo
      open(1,file=dir_calibration(1:endpos) // 
     &   "/coef/hirs/calcoef.dat",form='formatted',status='old',
     &   iostat=ios)
      if(ios.ne.0) then
        write(*,*)'calcoef.dat could not be opened'
        call aappexit(1)
      endif
      line = " "
      ios = 0
      do while (index(line,satname) .ne. 4 .and. ios .eq. 0)
        read(1,'(A80)',iostat=ios) line
      enddo
      if (ios .eq. 0) then
        do while (index(line,"central") .eq. 0)
          read(1,'(A80)') line
        enddo
        read(1,*) line
        read(1,*) wavenum
        read(1,*) line
        read(1,*) line
        read(1,*) line
        read(1,*) bandcor
        close(1)
        do ichan=1,12
          hrs_h_tempradcnv(1,ichan) = NINT(wavenum(ichan)*1D6)
        enddo
        do ichan=13,19
          hrs_h_tempradcnv(1,ichan) = NINT(wavenum(ichan)*1D5)
        enddo
        do ichan=1,19
          hrs_h_tempradcnv(2,ichan) = NINT(bandcor(1,ichan)*1D6)
          hrs_h_tempradcnv(3,ichan) = NINT(bandcor(2,ichan)*1D6)
        enddo
      else
        write(*,*)'could not find ' // satname // ' in calcoef.dat'
        call aappexit(1)
      endif

!Open the input file

      loc_rl=recl_class/rldiv
      open(1,file=class_file_in,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'HIRS input file could not be opened'
        call aappexit(1)
      endif
      
!Read the header record

      read(1,rec=1,iostat=ios) hrs_c_char   !First record
      hrs_h_satid = ICHAR(hrs_c_char(1:1))
      hrs_h_scnlin = ICHAR(hrs_c_char(9:9))*256 + 
     &               ICHAR(hrs_c_char(10:10))
      hrs_h_prblkid = 'B' // hrs_c_char(17:23)

!Open the output file and write the header

      loc_rl=reclhrs/rldiv
      open(2,file=aapp_file,form='unformatted',access='direct',
     &     recl=loc_rl,status='replace',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'HIRS output file could not be opened'
        call aappexit(1)
      endif
      call hrshdw(2,ios)
      if(ios.ne.0) then
        write(*,*)'Unable to write header for output file'
        call aappexit(1)
      endif
 
!Loop over records

      do irec=1,hrs_h_scnlin
        read(1,rec=irec+1,iostat=ios) hrs_c_char
        if (ios .eq. 0) then
          if (swap) CALL byteswap1b("HRS1BCL")

!Date, time, etc.

          hrs_scnlin = hrs_c_scnlin
          hrs_scnlinyr = hrs_c_timecode_year / 512 + 1900
          if (hrs_scnlinyr .lt. 1950) 
     &         hrs_scnlinyr = hrs_scnlinyr + 100
          hrs_scnlindy = IAND(hrs_c_timecode_year+0, 511)
          hrs_scnlintime = hrs_c_timecode_ms
          hrs_mjfrcnt = IAND(hrs_c_scanqual,255)/16
          hrs_scnpos = IAND(hrs_c_scanqual,15)
          hrs_scntyp = IAND(hrs_c_scanqual,Z'03000000')/Z'01000000'

          if (irec .eq. 1) then 
            scnlintime_old = hrs_scnlintime 
          else
            tdiff = hrs_scnlintime - scnlintime_old
            scnlintime_old = hrs_scnlintime
            if (tdiff .lt. -43200000) tdiff = tdiff + 86400000
            scandiff = tdiff * 1.0 / 6400 + 0.5
            if (scandiff .gt. 1) then
              hrs_h_misscnlin = hrs_h_misscnlin + scandiff - 1
              hrs_h_datagaps = hrs_h_datagaps + 1
            endif
          endif

          if (hrs_h_startdatady .eq. 0) then
            hrs_h_startdatayr = hrs_scnlinyr
            hrs_h_startdatady = hrs_scnlindy
            hrs_h_startdatatime = hrs_scnlintime
          endif
          hrs_h_enddatayr = hrs_scnlinyr
          hrs_h_enddatady = hrs_scnlindy
          hrs_h_enddatatime = hrs_scnlintime

!Quality

          hrs_qualind = 0
          if (IAND(hrs_c_scanqual,Z'80000000').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'80000000')  !do not use
          if (IAND(hrs_c_scanqual,Z'20000000').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'20000000')  !data gap
          if (IAND(hrs_c_scanqual,Z'40000000').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'40000000')  !time error
          if (IAND(hrs_c_scanqual,Z'00020000').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'08000000')  !no earth loc
          if (IAND(hrs_c_scanqual,Z'00010000').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'00000040')  !questionable earth loc
          if (IAND(hrs_c_scanqual,Z'00400000').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'00000800')  !some uncalibrated channels
          if (IAND(hrs_c_scanqual,Z'00004000').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'00080000')  !sync error
          if (IAND(hrs_c_scanqual,Z'00000400').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'00010000')  !parity error
          if (IAND(hrs_c_scanqual,Z'00000200').ne.0) 
     &        hrs_qualind = IOR(hrs_qualind,Z'00040000')  !aux frame sync error

          if (IAND(hrs_c_scanqual,Z'0000E200').eq.0) 
     &        hrs_h_okdatafr = hrs_h_okdatafr + 1           !no sync errors
          if (IAND(hrs_c_scanqual,Z'00000400').ne.0) 
     &        hrs_h_pacsparityerr = hrs_h_pacsparityerr + 1 !PACS (DACS?) parity errors
          if (IAND(hrs_c_scanqual,Z'00000200').ne.0) 
     &        hrs_h_auxsyncerrsum = hrs_h_auxsyncerrsum + 1 !Sum of aux sync errors
          if (IAND(hrs_c_scanqual,Z'40000000').ne.0) 
     &        hrs_h_timeseqerr = irec                     !record number of first occurrence
          if (IAND(hrs_c_scanqual,Z'00020000').ne.0) 
     &        hrs_h_locerrind =  irec                     !record number of first occurrence

!Re-arrange the first 2 element words. NESDIS format has 6 fill bits at the
!end of word 2, AAPP has them between words 1 and 2.

          do i=0,63
            hrs_elem(1,i) = IAND(hrs_c_hirsdata(1,i),i2wordm8)
            hrs_elem(2,i) = IAND(hrs_c_hirsdata(1,i),i2word7)*1024
     &                  + IAND(hrs_c_hirsdata(2,i)/i2word64,i2word1023)
!Channel data

            do j=3,22
              hrs_elem(j,i) = hrs_c_hirsdata(j,i)
            enddo
          enddo

!Minor frame quality

          do i=0,63 
            hrs_mnfrqual(i) = hrs_c_minorframequal(i)
          enddo

!Calibration
          
          cal = 0
          do ichan=1,20
            intercept = hrs_c_calcoef(3*ichan+60)/2.0**22

!Satellite-specific corrections detailed in POD user guide
            if ((ichan .eq. 1 .and. (satname .eq. 'noaa06' 
     &        .or. satname .eq. 'noaa07' .or. satname .eq. 'noaa08'
     &        .or. satname .eq. 'noaa10' .or. satname .eq. 'noaa11'
     &        .or. satname .eq. 'noaa13' .or. satname .eq. 'noaa14'))
     &     .or. (ichan .eq. 2 .and. satname .eq. 'noaa12')) then
              if (abs(intercept) .lt. 200) then
                intercept=(abs(intercept)+512)*intercept/abs(intercept)
              endif
            else if (ichan.eq.1 .and. satname.eq.'noaa12') then
              if (abs(intercept) .lt. 200) then
                intercept=(abs(intercept)+2048)*intercept/abs(intercept)
              else
                intercept=(abs(intercept)+1536)*intercept/abs(intercept)
              endif
            endif

            hrs_calcof(3,ichan) = NINT(intercept*1E6)   !intercept
            hrs_calcof(2,ichan) = 
     &        NINT((hrs_c_calcoef(3*ichan+59)/2.0**30)*1E9) !slope
            hrs_calcof(1,ichan) =
     &        NINT((hrs_c_calcoef(3*ichan+58)/2.0**44)*1E12) !2nd order
            if (hrs_calcof(2,ichan) .ne. 0) cal=1  !at least 1 chan calibrated

          enddo

          if (IAND(hrs_qualind,Z'08000000') .eq. 0) 
     &      hrs_h_callocsclin = hrs_h_callocsclin + cal

!Navigation. Since zen angles can be set to zero, work them out (ignoring
!roll error).

          hrs_scalti = hrs_c_height            !sat height in km
          do i=1,28
            scanang = (28.5-i)*1.8*deg2rad       !scan step is 1.8 deg
            sinz = (1 + hrs_scalti/6371.0)*sin(scanang)
            hrs_ang(3*i-1) = NINT(100*rad2deg*asin(sinz))
          enddo
          do i=29,56
            hrs_ang(3*i-1) = hrs_ang(170-3*i)
          enddo
          do i=1,56
            hrs_pos(2*i-1) = NINT(hrs_c_latlon(1,i)*1.0E4/128)
            hrs_pos(2*i)   = NINT(hrs_c_latlon(2,i)*1.0E4/128)
          enddo

!Write out data record

          call hrsdtw(2,irec,ios)
          if(ios.ne.0) then
            write(*,*)'Unable to write data record',irec
            call aappexit(1)
          endif
        else
          write(*,*)'Unable to read data record',irec
          call aappexit(1)
        endif  

      enddo
      
!Update header

      call hrshdw(2,ios)
      if(ios.ne.0) then
        write(*,*)'Unable to update header for output file'
        call aappexit(1)
      endif

      close(1)
      close(2)
      write(*,*)'Read   ',irec-1,' records from ',class_file
      write(*,*)'Written',irec-1,' records to   ',aapp_file

      END
