C LIBS = navnoaa navtool satid f7ml f7tp f7cp f7gp auxaapp
!
      program latlon_to_swath
!----------------------------------------------------------------------------
! Name: $Id: latlon_to_swath.F 507 2016-11-18 13:22:35Z frna $
!
! The following are read from standard input:
! 1) Name of satpos file
! 2) Start and end time of 1b file, in units of jd
! 3) A series of lat/lon pairs, for the points we want to convert
!
! Displays the pixel (x) and line (y) coordinates
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2016, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  13/09/2016  P Brunel / N Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
!------------------------------------------------------------------
!
      implicit none                                                     
!
!DECLARATIONS
!
! Local parameters:
      integer spunit       ! logical unit for the satpos index
      parameter (spunit=20)
      integer latlonunit       ! logical unit for the latlon file
      parameter (latlonunit=21)
!
!
! Commons
#include "c_satpos.h"
!
! Functions
! 
! Local variables:
      double precision yaw    ! estimated yaw error        (rad)
      double precision roll   ! estimated roll error        (rad)
      double precision pitch  ! estimated pitch error        (rad)

      integer ios            ! I/O status variable
      character*256 spname   ! var. name for the SATPOS file
      character*256 latlonfile   ! var. name for the lat/lon file
      character*22 st_date   ! date et heure de debut
      integer ierr           ! return code of sp_read

      integer satid_sat       ! satellite id in satid file
      integer satid_inst      ! instrument id for satellite id
      integer att_mode        ! attitude mode for satellite
      double precision att(3,3)     ! attitude error

      Double Precision lat    ! viewed points latitudes     (rad)
      Double Precision lon    ! viewed points longitudes    (rad)

      Double Precision tfirst,tlast, time, om_g(3), sm_s(3)

      Real line, pixel
      Double Precision scnang

! Functions
      integer genscid     ! libsatid
      integer geninstid   ! libsatid
      integer genattid    ! libsatid

      double precision pi
      parameter ( pi = 3.14159265358979311600d0)
!
!-
!
!    Reads the input parameters
!    --------------------------
!
!    Get the satpos file name
      read(*,'(a)',iostat=ios) spname
      if(ios.ne.0) then
        call ml_wti('error',
     &       'while reading the satpos filename I/O status',ios)
        call ml_wt('error','LOCTST aborted')
        call exit(1)
      endif

!    Get the attitude values yaw, roll, pitch (mrad)
      yaw = -100.0    !use default
      roll = -100.0 
      pitch = -100.0
!    convert the values to radians
      yaw = yaw*1.d-3
      roll = roll*1.d-3
      pitch = pitch*1.d-3


!    opens and reads the SATPOS file
!    -----------------------------------
      open (unit=spunit,file=spname,status='old',iostat=ios)
      if( ios.ne.0 ) then
        call ml_wti('error',
     &       'while openning the satpos file I/O status',ios)
        call ml_wt('error','filename '//spname)
        call ml_wt('error','LOCTST aborted')
        call exit(1)
      endif

    ! Read all the satpos file 
      call sp_read(0.d0,0.d0,spunit,'any',ierr)
      if( ierr.ne.0 ) then
        call ml_wti('error',
     &       'while reading the satpos file I/O status',ierr)
        call ml_wt('error','filename '//spname)
        call ml_wt('error','LOCTST aborted')
        call exit(1)
      endif
      close(unit=spunit,iostat=ios)


!    Get the Satid values for satellite and instrument
!    -------------------------------------------------
!       defines the satellite and instrument index 
!       for the satid file
!       Get the attitude mode for satellite
      satid_sat  =  genscid(sp_satnam)
      if( satid_sat .eq. 0) then
         call ml_wt('error',
     &      'error in getting satid satellite number for '//sp_satnam)
        call exit(1)
      end if

!   get instrument index in SATID file ; here for AVHRR
      satid_inst =  geninstid(satid_sat,'avhrr')
      if( satid_inst .eq. 0) then
         call ml_wt('error',
     &      'error in getting isatid nstrument number for avhrr')
        call exit(1)
      end if

!    Attitude
!    --------
!    Cheks the attitude values  if < -99 mrad take the default values
      if(yaw.lt.-99.d-03) then
        call def_att(sp_satnam,att_mode,yaw,roll,pitch)
!        call ml_wt('info','take default attitude values')
      endif

!   Calculate ATTITUDE matrix
      call calatt(yaw,roll,pitch,att)

!   Attitude mode definition 
      att_mode =  genattid(satid_sat)


!    Do calculations
!    ---------------

! Time of pass in julian days. Note this should be less thna one full orbit
! ~ check that tlast - tfirst is less than 100mn ( 100.0/1440.0 julian day)
!  tfirst and tlast should be the time of first and last lines of the 
!  given data file
!   so read from file:
!        tfirst=20898.375D0
!        tlast=20898.400D0
!   or calculated from string values:
!        call xicj("21/03/07  09:00:00.000",tfirst)
!        call xicj("21/03/07  09:36:00.000",tlast)

!  Read time from standard input
        read(*,*,iostat=ios) tfirst, tlast
        if( ios.ne.0 ) then
          call ml_wti('error',
     &       'reading tfirst and tlast',ios)
          call exit(1)
        endif

        call xijc(tfirst, st_date)
!        write(*,*) 'tfirst ',tfirst,'  ', st_date
        call xijc(tlast, st_date)
!        write(*,*) 'tlast  ',tlast,'  ', st_date

!   Loop over requested lat/lons
        read(*,'(a)',iostat=ios) latlonfile
        if( ios.ne.0 ) then
          call ml_wti('error',
     &       'reading latlonfile',ios)
          call exit(1)
        endif
        open (latlonunit,file=latlonfile,form='formatted',status='old')
        do while (ios.eq.0)
          read(latlonunit,*,iostat=ios) lat, lon 
          if (ios.ne.0) then
            close(latlonunit)
            call exit(0)
          endif    

!   Convert to radians
          lat=  lat * pi / 180.d0
          lon=  lon * pi / 180.d0


!   Convert position to Eath fixed frame
          call lalocart(1, lat, lon, om_g, ierr)
!          write(*,*) 'latitude  ',lat*180.d0/pi
!          write(*,*) 'longitude ',lon*180.d0/pi
!          write(*,*) 'satid_sat ',satid_sat
!          write(*,*) 'satid_inst ',satid_inst
!          write(*,*) 'om_g ',om_g
!          write(*,*) 'ierr ',ierr
!          write(*,*) 'tfirst,tlast ',tfirst,tlast

!   Call Inverse location routine that will provide 
!   a viewing vector in instrument frame
          time = 0.0
          ierr = 0
          call invloc(satid_sat,satid_inst,att_mode, att, om_g, 
     &                    tfirst,tlast,
     &                    time,sm_s,ierr)
          if(ierr .eq. 0) then
            call xijc(time, st_date)
!            write(*,*) 'time ',time,'  ', st_date
!            write(*,*) 'sm_s ', sm_s
!            write(*,*)

!    Convert viewing vector to line and pixel (scaning angle also provided)
            call viewvecttolp(
     &       time,sm_s,satid_sat,satid_inst,tfirst,pixel,line,scnang)
            write(*,*) NINT(pixel), NINT(line)
!            write(*,*) 'scnang ', scnang * 180.d0/pi, ' deg'
          else
            write(*,*) 0,0
          endif
        enddo     !end of loop
      
      end

