C LIBS = f7nl1b auxaapp f7ml f7tp f7cp f7gp
!
      PROGRAM MSU_CLASS_TO_AAPP
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: msu_class_to_aapp.F 380 2014-01-23 12:35:13Z frna $
!
! Purpose: Convert MSU 1b files from NOAA/CLASS to AAPP format
!
! Inputs (read from std input):
!   Input CLASS 1b file name, conforming to NSS.MSUX.N[A-J].D?????.*
!   Output AAPP file name
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  29/08/2013  N C Atkinson
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "msu1b.h"
#include "reclopt.h"    
#include "msu1b_class.h"
#include "nl1b.h"
#include "constants_preproc.h"

      character*200 class_file_in  !Can include directory
      character*80  class_file     !NSS.MSUX.N[A-J].D?????.*
      character*200 aapp_file
      character*2 satid

      integer pos
      integer recl_class
      integer loc_rl
      integer ios
      integer year
      integer irec
      integer scnlintime_old
      integer tdiff
      integer scandiff
      integer i,j
      integer cal
      integer ichan
      
      integer*2 i2word4095,i2word8,i2word16,i2word32,i2word48
      data i2word4095/4095/
      data i2word8/8/
      data i2word16/16/
      data i2word32/32/
      data i2word48/48/
      
      logical swap
      data swap/.false./

      real scanang
      real sinz
      real wavenum(4)
      data wavenum /1.6779,1.7927,1.8334,1.9331/  !same for all instruments

!
!- End of header

      read(*,'(A)') class_file_in
      read(*,'(A)') aapp_file
      pos = 1
      do i=1,200
        if (class_file_in(i:i).eq."/") pos=i+1
      enddo
      class_file = class_file_in(pos:pos+41)
      
      if (class_file(5:7) .ne. "MSU") then
        write(*,*)'Error: input file name must be of the form'
        write(*,*)'NSS.MSUX.N[A-J].D?????.*'
        call aappexit(1)
      endif

      read(class_file,'(13X,I2)') year
      if (year.gt.50.and.year.lt.95) then
        recl_class = 440
      else
        recl_class = 437
        if (rldiv .ne. 1) then
          write(*,*)'Error: the MSU 1b record length (437) is an'
          write(*,*)'odd number of bytes, so record length specifier in'
          write(*,*)'"open" must be in bytes. For ifort, choose the'
          write(*,*)'compiler option "-assume byterecl"'
          call aappexit(1)
        endif
      endif

!Is byte swapping needed? CLASS data are always big-endian

      msu_c_scnlin = 1
      if (ichar(msu_c_char(1:1)) .eq. 1) swap = .true.  !Little-endian platform

!Fixed items in header

      msu_h_siteid = 'nss'
      msu_h_l1bversnb = libl1bvers   !in nl1b.h
      msu_h_l1bversyr = libl1bversyr !in nl1b.h
      msu_h_l1bversdy = libl1bversdy !in nl1b.h
      msu_h_reclg = reclmsu          !in msu1b.h
      msu_h_hdrcnt = 1
      msu_h_dataname = class_file
      msu_h_datatyp = 6              !6=MSU
      msu_h_tipsrc = 2               !2=stored
      do i=1,4
        msu_h_censperes(i) = wavenum(i)*10000
      enddo
      msu_h_startdatady = 0

      satid = class_file(10:11)
      if (satid .eq. 'TN') then        !noaa05 (Tiros-N). This is an AAPP convention.
        msu_h_instid = 1340            !  instid are in msuhdu.F
      else if (satid .eq. 'NA') then   !noaa06
        msu_h_instid = 543
      else if (satid .eq. 'NC') then   !noaa07
        msu_h_instid = 1753
      else if (satid .eq. 'NE') then   !noaa08
        msu_h_instid = 3377
      else if (satid .eq. 'NF') then   !noaa09
        msu_h_instid = 2962
      else if (satid .eq. 'NG') then   !noaa10
        msu_h_instid = 2554
      else if (satid .eq. 'NH') then   !noaa11
        msu_h_instid = 2161
      else if (satid .eq. 'ND') then   !noaa12
        msu_h_instid = 3927
      else if (satid .eq. 'NI') then   !noaa13
        msu_h_instid = 0
      else if (satid .eq. 'NJ') then   !noaa14
        msu_h_instid = 3823
      else
        write(*,*)'Error: invalid satid ',satid
        call aappexit(1)
      endif

      msu_h_callocsclin = 0
      scnlintime_old = 0
      
!Open the input file

      loc_rl=recl_class/rldiv
      open(1,file=class_file_in,form='unformatted',access='direct',
     &     recl=loc_rl,status='old',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'MSU input file could not be opened'
        call aappexit(1)
      endif
      
!Read the header record

      read(1,rec=1,iostat=ios) msu_c_char   !First record
      msu_h_satid = ICHAR(msu_c_char(1:1))
      msu_h_scnlin = ICHAR(msu_c_char(9:9))*256 + 
     &               ICHAR(msu_c_char(10:10))
      msu_h_prblkid = 'B' // msu_c_char(17:23)

!Open the output file and write the header

      loc_rl=reclmsu/rldiv
      open(2,file=aapp_file,form='unformatted',access='direct',
     &     recl=loc_rl,status='replace',iostat=ios)
      if(ios.ne.0) then
        write(*,*)'MSU output file could not be opened'
        call aappexit(1)
      endif
      call msuhdw(2,ios)
      if(ios.ne.0) then
        write(*,*)'Unable to write header for output file'
        call aappexit(1)
      endif
 
!Loop over records

      do irec=1,msu_h_scnlin
        read(1,rec=irec+1,iostat=ios) msu_c_char
        if (ios .eq. 0) then
          if (swap) CALL byteswap1b("MSU1BCL")

!Date, time, etc.

          msu_scnlin = msu_c_scnlin
          msu_scnlinyr = msu_c_timecode_year / 512 + 1900
          if (msu_scnlinyr .lt. 1950) 
     &         msu_scnlinyr = msu_scnlinyr + 100
          msu_scnlindy = IAND(msu_c_timecode_year+0, 511)
          msu_scnlintime = msu_c_timecode_ms
          msu_mjfrcnt = IAND(msu_c_scanqual,255)/16
          msu_scnpos = IAND(msu_c_scanqual,15)

          if (irec .eq. 1) then 
            scnlintime_old = msu_scnlintime 
          else
            tdiff = msu_scnlintime - scnlintime_old
            scnlintime_old = msu_scnlintime
            if (tdiff .lt. -43200000) tdiff = tdiff + 86400000
            scandiff = tdiff * 1.0 / 25600 + 0.5
            if (scandiff .gt. 1) then
              msu_h_misscnlin = msu_h_misscnlin + scandiff - 1
              msu_h_datagaps = msu_h_datagaps + 1
            endif
          endif

          if (msu_h_startdatady .eq. 0) then
            msu_h_startdatayr = msu_scnlinyr
            msu_h_startdatady = msu_scnlindy
            msu_h_startdatatime = msu_scnlintime
          endif
          msu_h_enddatayr = msu_scnlinyr
          msu_h_enddatady = msu_scnlindy
          msu_h_enddatatime = msu_scnlintime

!Quality

          msu_qualind = 0
          if (IAND(msu_c_scanqual,Z'80000000').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'80000000')  !do not use
          if (IAND(msu_c_scanqual,Z'40000000').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'20000000')  !data gap
          if (IAND(msu_c_scanqual,Z'08000000').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'40000000')  !time error
          if (IAND(msu_c_scanqual,Z'02000000').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'08000000')  !no earth loc
          if (IAND(msu_c_scanqual,Z'01000000').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'00000040')  !questionable earth loc
          if (IAND(msu_c_scanqual,Z'00800000').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'00000800')  !some uncalibrated channels
          if (IAND(msu_c_scanqual,Z'00004000').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'00080000')  !sync error
          if (IAND(msu_c_scanqual,Z'00000400').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'00010000')  !parity error
          if (IAND(msu_c_scanqual,Z'00000200').ne.0) 
     &        msu_qualind = IOR(msu_qualind,Z'00040000')  !aux frame sync error

          if (IAND(msu_c_scanqual,Z'0000E200').eq.0) 
     &        msu_h_okdatafr = msu_h_okdatafr + 1           !no sync errors
          if (IAND(msu_c_scanqual,Z'00000400').ne.0) 
     &        msu_h_pacsparityerr = msu_h_pacsparityerr + 1 !PACS (DACS?) parity errors
          if (IAND(msu_c_scanqual,Z'00000200').ne.0) 
     &        msu_h_auxsyncerrsum = msu_h_auxsyncerrsum + 1 !Sum of aux sync errors
          if (IAND(msu_c_scanqual,Z'08000000').ne.0) 
     &        msu_h_timeseqerr = irec                     !record number of first occurrence
          if (IAND(msu_c_scanqual,Z'02000000').ne.0) 
     &        msu_h_locerrind =  irec                     !record number of first occurrence

!Channel data

          do i=1,8
            do j=1,14
              msu_data(i+(j-1)*8) = IAND(msu_c_msudata(i,j),i2word4095)
            enddo
          enddo
          
          cal = 0
          do ichan=1,4
            msu_scnlinqual(ichan) = 0
            msu_pmcal(3,ichan) = 
     &        NINT((msu_c_calcoef(2*ichan)/2.0**22)*1E8)   !intercept
            msu_pmcal(2,ichan) = 
     &        NINT((msu_c_calcoef(2*ichan-1)/2.0**30)*1E12)!slope
            msu_pmcal(1,ichan) = 0                         !2nd order
            if (msu_pmcal(2,ichan) .ne. 0) cal=1  !at least 1 chan is calibrated

            if (msu_data(3+ichan+88) .eq. 0 .or. 
     &        msu_data(3+ichan+88) .eq. 4095)
     &        msu_scnlinqual(ichan)= IOR(msu_scnlinqual(ichan),i2word16) !bad space

            if (msu_data(3+ichan+96) .eq. 0 .or. 
     &        msu_data(3+ichan+96) .eq. 4095)
     &        msu_scnlinqual(ichan)= IOR(msu_scnlinqual(ichan),i2word32) !bad BB

            if (msu_pmcal(2,ichan) .eq. 0 .and. 
     &        IAND(msu_scnlinqual(ichan),i2word48) .eq. 0)
     &        msu_scnlinqual(ichan) = IOR(msu_scnlinqual(ichan),i2word8) !band PRT

            msu_h_nolincor(1,ichan) = 
     &        NINT((msu_c_calcoef(4*ichan + 7)/2.0**44)*1E12)  !2nd
            msu_h_nolincor(2,ichan) = 
     &        NINT((msu_c_calcoef(4*ichan + 6)/2.0**30)*1E6)   !1st
            msu_h_nolincor(3,ichan) =
     &        NINT((msu_c_calcoef(4*ichan + 5)/2.0**22)*1E6)   !0th
          enddo

          if (IAND(msu_qualind,Z'08000000') .eq. 0) 
     &      msu_h_callocsclin = msu_h_callocsclin + cal

!Navigation. Since zen angles can be set to zero, work them out (ignoring
!roll error).

          msu_scalti = msu_c_height            !sat height in km
          do i=1,5
            scanang = (6-i)*9.47*deg2rad       !scan step is 9.47 deg
            sinz = (1 + msu_scalti/6371.0)*sin(scanang)
            msu_ang(2,i) = NINT(100*rad2deg*asin(sinz))
          enddo
          do i=7,11
            msu_ang(2,i) = msu_ang(2,12-i)
          enddo
          do i=1,11
            msu_pos(1,i) = NINT(msu_c_latlon(1,i)*1.0E4/128)
            msu_pos(2,i) = NINT(msu_c_latlon(2,i)*1.0E4/128)
          enddo

!Write out data record

          call msudtw(2,irec,ios)
          if(ios.ne.0) then
            write(*,*)'Unable to write data record',irec
            call aappexit(1)
          endif
        else
          write(*,*)'Unable to read data record',irec
          call aappexit(1)
        endif  

      enddo
      
!Update header

      call msuhdw(2,ios)
      if(ios.ne.0) then
        write(*,*)'Unable to update header for output file'
        call aappexit(1)
      endif

      close(1)
      close(2)
      write(*,*)'Read   ',irec-1,' scans from ',class_file
      write(*,*)'Written',irec-1,' scans to   ',aapp_file

      END
