#!/bin/ksh
#
#$Id: noaa_class_to_aapp.ksh 549 2017-08-30 12:40:28Z frna $
#
#Convert NOAA 1B satellite data from CLASS to AAPP format
#
#Instruments and formats supported:
#
#  Tiros-N to NOAA-14: 
#    MSU
#    HIRS/2
#    AVHRR and AVHRR/2 LAC and GAC 10-bit format
#
#  NOAA-15 to NOAA-19:
#    AVHRR/3 LAC and GAC 10-bit and 16-bit formats
#    The other instruments are already in AAPP format
#
#  The satellite identifier is extracted from the input file name, so it must be
#  in standard CLASS format, e.g. NSS.HIRX.N[A-P].D?????.*
#
#Links to format descriptions:
#  http://www.ncdc.noaa.gov/oa/pod-guide/ncdc/docs/intro.htm
#
#History:
#
# 1.0   12/09/2013  initial version. N C Atkinson
# 1.1   20/11/2014  allow for TOVS-era archive headers ("TBM" header)
#==============================================================================

. "$AAPP_PREFIX/ATOVS_CONF"

usage(){
  echo
  echo 'usage : noaa_class_to_aapp inputfile outputfile'
  echo
}

[ $# -lt 2 ] && { usage; exit 1; }

infile=$1
outfile=$2
[ -f $infile ] || { echo "Cannot find $1"; exit 1; }
infilename=$(basename $infile)

#Check whether there is an archive header - if so, bytes 31-72 contain the
#dataset name. If the dataset name is ASCII then the header is 512 bytes ("ARS"
#format). If it's EBCDIC (TOVS-era) then the header is 122 bytes ("TBM" format).
#Strip off the header.

header_id_ascii=$(head -c 41 $infile | tail -c 11)   #e.g. NSS.GHRR.NN
header_id_ebcdic=$(head -c 41 $infile | tail -c 11 | dd conv=ascii 2>/dev/null)
file_id=$(echo $infilename | head -c 11)

if [ "$header_id_ascii" = "$file_id" ]; then
  bs=512
elif [ "$header_id_ebcdic" = "$file_id" ]; then
  bs=122
else
  bs=""
fi
if [ "$bs" ]; then
  newfile=${infilename}.noheader
  if [ ! -f $newfile ]; then
    echo "Removing $bs byte archive header"
    dd bs=$bs skip=1 if=$infile of=$newfile
    if [ $? != 0 ]; then
      echo "Cannot create $newfile"
      exit 1
    fi
  else
    echo "using $newfile"
  fi
  infile=$newfile
  infilename=$(basename $infile)
fi

#Get the satellite and instrument from the file name

satid=$(echo $infilename | cut -c10-11)
instr=$(echo $infilename | cut -c5-7)
if [[ "$(stat --version 2>/dev/null)" = *GNU* ]]; then
  filesize=$(stat --format=%s $infile)
else
  filesize=$(stat -f %z $infile)
fi

if [[ $satid != N[A-P] && $satid != M[1-3] && $satid != TN ]]; then
  echo "wrong file name format"
  exit 1
fi

if [[ $satid = N[A-J] || $satid = TN ]]; then
  tovs=yes
else
  tovs=no
fi  

if [ $tovs = yes ]; then

##### TOVS era #####

  if [ $instr = MSU ]; then

msu_class_to_aapp.exe <<EOF
$infile
$outfile
EOF
exit $?

  elif [ $instr = HIR ]; then
  
hirs2_class_to_aapp.exe <<EOF
$infile
$outfile
EOF
exit $?

  elif [ $instr = GHR ]; then
  
avhrr_gac_class_to_aapp_a-j.exe <<EOF
$infile
$outfile
EOF
exit $?

  elif [ $instr = LHR ] || [ $instr = HRP ]; then
  
avhrr_lac_class_to_aapp_a-j.exe <<EOF
$infile
$outfile
EOF
exit $?

  else
    echo "unrecognised instrument type: $instr"
  fi

else

##### ATOVS era #####

  if [ $instr = GHR ]; then
  
avhrr_gac_class_to_aapp_klm.exe <<EOF
$infile
$outfile
$filesize
EOF
exit $?

  elif [ $instr = LHR ] || [ $instr = HRP ] || [ $instr = FRA ]; then
  
avhrr_lac_class_to_aapp_klm.exe <<EOF
$infile
$outfile
$filesize
EOF
exit $?

  else
    echo "unrecognised instrument type: $instr"
  fi

fi
