C LIBS = atovin atov auxaapp f7nl1b
!
      PROGRAM REORDER_1C
      IMPLICIT NONE
!----------------------------------------------------------------------------
! Name: $Id: reorder_1c.F 620 2019-02-15 12:41:33Z frna $
!
! Purpose: Sort a 1C file into time order, remove duplicate scans and
! create new scan line numbers
!
! Description:
!   This tool may be needed when working with archived ATOVS or TOVS data
!   (e.g. from ECMWF MARS archive).
!   Assumes all the scan line times are valid.
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! Version:
! 1.0  11/06/2010  N C Atkinson
! 1.1  12/01/2012  Add IASI, ATMS, CrIS
! 1.2  20/12/2018  Add MWHS2
! 1.3  14/02/2019  Add IRAS. Round scan step down for MWHS2 and IRAS because
!                  CMA BUFR files only give time to the nearest second. 
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:

#include "ppparms.h"
#include "hrs1c.h"
#include "ama1c.h"
#include "amb1c.h"
#include "msu1c.h"
#include "iasi1c.h"
#include "atms1c.h"
#include "cris1c.h"
#include "mwhs21c.h"
#include "iras1c.h"

      INTEGER unitin, unitout
      DATA unitin/10/, unitout/11/

      INTEGER MaxScans
      PARAMETER (MaxScans=10000)
      
      REAL*8 ScanSecs(MaxScans)
      INTEGER ScanYear(MaxScans)
      INTEGER ScanIndex(MaxScans)
      DATA ScanIndex/MaxScans*0/ 
      
      REAL*8 StartSecs, SecsInYear, Step
      DATA StartSecs/1.0D10/
      INTEGER StartYear
      DATA StartYear/10000/
      INTEGER LastScan
      DATA LastScan/0/

      INTEGER iscan, newscan
      INTEGER nscansin, nscansout

      CHARACTER*6 instrument
!
!- End of header

!------------------------------------------------------------------------------
!1) Read the instrument type from standard input
!------------------------------------------------------------------------------

      READ(*,*) instrument

!------------------------------------------------------------------------------
!2) Read in all the scan line times
!------------------------------------------------------------------------------

      IF (instrument.EQ."AMSU-A") THEN
        Step = Acycle*1.0D-3
        CALL IOA1C ("OPEN/READ",unitin,0)
        CALL IOA1C ("READ HEADER",unitin,1)
        CALL IOA1C ("OPEN/WRITE",unitout,0)
        nscansin = ama1c_h_scnlin
        DO iscan=1,nscansin
          CALL IOA1C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = ama1c_scnlinyr
          ScanSecs(iscan) = ama1c_scnlintime*1.0D-3
     &                     + ama1c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,ama1c_scnlinyr)
        ENDDO
      ELSE IF (instrument.EQ."AMSU-B".OR.instrument.EQ."MHS") THEN
        Step = Bcycle*1.0D-3
        CALL IOB1C ("OPEN/READ",unitin,0)
        CALL IOB1C ("READ HEADER",unitin,1)
        CALL IOB1C ("OPEN/WRITE",unitout,0)
        nscansin = amb1c_h_scnlin
        DO iscan=1,nscansin
          CALL IOB1C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = amb1c_scnlinyr
          ScanSecs(iscan) = amb1c_scnlintime*1.0D-3
     &                     + amb1c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,amb1c_scnlinyr)
        ENDDO
      ELSE IF (instrument.EQ."HIRS") THEN
        Step = Hcycle*1.0D-3
        CALL IOH1C ("OPEN/READ",unitin,0)
        CALL IOH1C ("READ HEADER",unitin,1)
        CALL IOH1C ("OPEN/WRITE",unitout,0)
        nscansin = hrs1c_h_scnlin
        DO iscan=1,nscansin
          CALL IOH1C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = hrs1c_scnlinyr
          ScanSecs(iscan) = hrs1c_scnlintime*1.0D-3
     &                     + hrs1c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,hrs1c_scnlinyr)
        ENDDO
      ELSE IF (instrument.EQ."MSU") THEN
        Step = Mcycle*1.0D-3
        CALL IOM1C ("OPEN/READ",unitin,0)
        CALL IOM1C ("READ HEADER",unitin,1)
        CALL IOM1C ("OPEN/WRITE",unitout,0)
        nscansin = msu1c_h_scnlin
        DO iscan=1,nscansin
          CALL IOM1C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = msu1c_scnlinyr
          ScanSecs(iscan) = msu1c_scnlintime*1.0D-3
     &                     + msu1c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,msu1c_scnlinyr)
        ENDDO
      ELSE IF (instrument.EQ."IASI") THEN
        Step = Icycle*1.0D-3
        CALL IOI1C ("OPEN/READ",unitin,0)
        CALL IOI1C ("READ HEADER",unitin,1)
        CALL IOI1C ("OPEN/WRITE",unitout,0)
        nscansin = iasi1c_h_scnlin
        DO iscan=1,nscansin
          CALL IOI1C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = iasi1c_scnlinyr
          ScanSecs(iscan) = iasi1c_scnlintime(1)*1.0D-3
     &                     + iasi1c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,iasi1c_scnlinyr)
        ENDDO
      ELSE IF (instrument.EQ."ATMS") THEN
        Step = atmscycle*1.0D-3
        CALL IOAT1C ("OPEN/READ",unitin,0)
        CALL IOAT1C ("READ HEADER",unitin,1)
        CALL IOAT1C ("OPEN/WRITE",unitout,0)
        nscansin = atms1c_h_scnlin
        DO iscan=1,nscansin
          CALL IOAT1C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = atms1c_scnlinyr
          ScanSecs(iscan) = atms1c_scnlintime(1)*1.0D-3
     &                     + atms1c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,atms1c_scnlinyr)
        ENDDO
      ELSE IF (instrument.EQ."CRIS") THEN
        Step = criscycle*1.0D-3
        CALL IOC1C ("OPEN/READ",unitin,0)
        CALL IOC1C ("READ HEADER",unitin,1)
        CALL IOC1C ("OPEN/WRITE",unitout,0)
        nscansin = cris1c_h_scnlin
        DO iscan=1,nscansin
          CALL IOAT1C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = cris1c_scnlinyr
          ScanSecs(iscan) = cris1c_scnlintime(1)*1.0D-3
     &                     + cris1c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,cris1c_scnlinyr)
        ENDDO
      ELSE IF (instrument.EQ."MWHS2") THEN
        Step = 2.0D0  !Minimum scan separation is 2 sec
        CALL IOMWHS21C ("OPEN/READ",unitin,0)
        CALL IOMWHS21C ("READ HEADER",unitin,1)
        CALL IOMWHS21C ("OPEN/WRITE",unitout,0)
        nscansin = mwhs21c_h_scnlin
        DO iscan=1,nscansin
          CALL IOMWHS21C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = mwhs21c_scnlinyr
          ScanSecs(iscan) = mwhs21c_scnlintime*1.0D-3
     &                     + mwhs21c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,mwhs21c_scnlinyr)
        ENDDO
      ELSE IF (instrument.EQ."IRAS") THEN
        Step = 6.0D0  !Minimum scan separation is 6 sec
        CALL IOIRAS1C ("OPEN/READ",unitin,0)
        CALL IOIRAS1C ("READ HEADER",unitin,1)
        CALL IOIRAS1C ("OPEN/WRITE",unitout,0)
        nscansin = iras1c_h_scnlin
        DO iscan=1,nscansin
          CALL IOIRAS1C ("READ DATA",unitin,1+iscan)
          ScanYear(iscan) = iras1c_scnlinyr
          ScanSecs(iscan) = iras1c_scnlintime*1.0D-3
     &                     + iras1c_scnlindy*86400.0D0
          StartYear = MIN(StartYear,iras1c_scnlinyr)
        ENDDO
      ENDIF

!------------------------------------------------------------------------------
!3) Find the start time and adjust times if there is a year transition
!------------------------------------------------------------------------------

      IF (MOD(StartYear,4).EQ.0) THEN
        SecsInYear = 366.0D0 * 86400.0D0
      ELSE
        SecsInYear = 365.0D0 * 86400.0D0
      ENDIF
      DO iscan = 1,nscansin
        IF (ScanYear(iscan) .EQ. StartYear) THEN
          StartSecs = MIN(StartSecs,ScanSecs(iscan))
        ELSE
          ScanSecs(iscan) = ScanSecs(iscan) + SecsInYear
        ENDIF
      ENDDO

!------------------------------------------------------------------------------
!4) Compute new scan line numbers
!------------------------------------------------------------------------------

      DO iscan = 1,nscansin
        NewScan = (ScanSecs(iscan) - StartSecs)/Step + 1.5
        IF (NewScan .LE. MaxScans) THEN
          LastScan = MAX(LastScan, NewScan)
          ScanIndex(NewScan) = iscan
        ELSE
          WRITE(*,*)'WARNING: Scan',iscan,' invalid: NewScan=',NewScan
        ENDIF
      ENDDO

!------------------------------------------------------------------------------
!5) Re-read input scans in the correct order and write to output file
!------------------------------------------------------------------------------

      nscansout = 0
      
      IF (instrument.EQ."AMSU-A") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOA1C ("READ DATA",unitin,1+ScanIndex(iscan))
            ama1c_scnlin = iscan
            nscansout = nscansout + 1
            CALL IOA1C ("WRITE DATA",unitout,1+nscansout)
            IF (nscansout.EQ.1) THEN
              ama1c_h_startdatayr = ama1c_scnlinyr
              ama1c_h_startdatady = ama1c_scnlindy
              ama1c_h_startdatatime = ama1c_scnlintime
            ENDIF
          ENDIF
        ENDDO
        ama1c_h_scnlin = nscansout
        ama1c_h_enddatayr = ama1c_scnlinyr
        ama1c_h_enddatady = ama1c_scnlindy
        ama1c_h_enddatatime = ama1c_scnlintime
        CALL IOA1C ("WRITE HEADER",unitout,1)
        CALL IOA1C ("CLOSE",unitout,0)
        CALL IOA1C ("CLOSE",unitin,0)

      ELSE IF (instrument.EQ."AMSU-B".OR.instrument.EQ."MHS") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOB1C ("READ DATA",unitin,1+ScanIndex(iscan))
            amb1c_scnlin = iscan
            nscansout = nscansout + 1
            CALL IOB1C ("WRITE DATA",unitout,1+nscansout)
            IF (nscansout.EQ.1) THEN
              amb1c_h_startdatayr = amb1c_scnlinyr
              amb1c_h_startdatady = amb1c_scnlindy
              amb1c_h_startdatatime = amb1c_scnlintime
            ENDIF
          ENDIF
        ENDDO
        amb1c_h_scnlin = nscansout
        amb1c_h_enddatayr = amb1c_scnlinyr
        amb1c_h_enddatady = amb1c_scnlindy
        amb1c_h_enddatatime = amb1c_scnlintime
        CALL IOB1C ("WRITE HEADER",unitout,1)
        CALL IOB1C ("CLOSE",unitout,0)
        CALL IOB1C ("CLOSE",unitin,0)

      ELSE IF (instrument.EQ."HIRS") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOH1C ("READ DATA",unitin,1+ScanIndex(iscan))
            hrs1c_scnlin = iscan
            nscansout = nscansout + 1
            CALL IOH1C ("WRITE DATA",unitout,1+nscansout)
            IF (nscansout.EQ.1) THEN
              hrs1c_h_startdatayr = hrs1c_scnlinyr
              hrs1c_h_startdatady = hrs1c_scnlindy
              hrs1c_h_startdatatime = hrs1c_scnlintime
            ENDIF
          ENDIF
        ENDDO
        hrs1c_h_scnlin = nscansout
        hrs1c_h_enddatayr = hrs1c_scnlinyr
        hrs1c_h_enddatady = hrs1c_scnlindy
        hrs1c_h_enddatatime = hrs1c_scnlintime
        CALL IOH1C ("WRITE HEADER",unitout,1)
        CALL IOH1C ("CLOSE",unitout,0)
        CALL IOH1C ("CLOSE",unitin,0)

      ELSE IF (instrument.EQ."MSU") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOM1C ("READ DATA",unitin,1+ScanIndex(iscan))
            msu1c_scnlin = iscan
            nscansout = nscansout + 1
            CALL IOM1C ("WRITE DATA",unitout,1+nscansout)
            IF (nscansout.EQ.1) THEN
              msu1c_h_startdatayr = msu1c_scnlinyr
              msu1c_h_startdatady = msu1c_scnlindy
              msu1c_h_startdatatime = msu1c_scnlintime
            ENDIF
          ENDIF
        ENDDO
        msu1c_h_scnlin = nscansout
        msu1c_h_enddatayr = msu1c_scnlinyr
        msu1c_h_enddatady = msu1c_scnlindy
        msu1c_h_enddatatime = msu1c_scnlintime
        CALL IOM1C ("WRITE HEADER",unitout,1)
        CALL IOM1C ("CLOSE",unitout,0)
        CALL IOM1C ("CLOSE",unitin,0)

      ELSE IF (instrument.EQ."IASI") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOI1C ("READ DATA",unitin,1+ScanIndex(iscan))
            iasi1c_scnlin = iscan
            nscansout = nscansout + 1
            CALL IOI1C ("WRITE DATA",unitout,1+nscansout)
            IF (nscansout.EQ.1) THEN
              iasi1c_h_startdatayr = iasi1c_scnlinyr
              iasi1c_h_startdatady = iasi1c_scnlindy
              iasi1c_h_startdatatime = iasi1c_scnlintime(1)
            ENDIF
          ENDIF
        ENDDO
        iasi1c_h_scnlin = nscansout
        iasi1c_h_enddatayr = iasi1c_scnlinyr
        iasi1c_h_enddatady = iasi1c_scnlindy
        iasi1c_h_enddatatime = iasi1c_scnlintime(1)
        CALL IOI1C ("WRITE HEADER",unitout,1)
        CALL IOI1C ("CLOSE",unitout,0)
        CALL IOI1C ("CLOSE",unitin,0)

      ELSE IF (instrument.EQ."ATMS") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOAT1C ("READ DATA",unitin,1+ScanIndex(iscan))
            atms1c_scnlin = iscan
            nscansout = nscansout + 1
            CALL IOAT1C ("WRITE DATA",unitout,1+nscansout)
            IF (nscansout.EQ.1) THEN
              atms1c_h_startdatayr = atms1c_scnlinyr
              atms1c_h_startdatady = atms1c_scnlindy
              atms1c_h_startdatatime = atms1c_scnlintime(1)
            ENDIF
          ENDIF
        ENDDO
        atms1c_h_scnlin = nscansout
        atms1c_h_enddatayr = atms1c_scnlinyr
        atms1c_h_enddatady = atms1c_scnlindy
        atms1c_h_enddatatime = atms1c_scnlintime(1)
        CALL IOAT1C ("WRITE HEADER",unitout,1)
        CALL IOAT1C ("CLOSE",unitout,0)
        CALL IOAT1C ("CLOSE",unitin,0)

      ELSE IF (instrument.EQ."CRIS") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOC1C ("READ DATA",unitin,1+ScanIndex(iscan))
            cris1c_scnlin = iscan
            nscansout = nscansout + 1
            CALL IOC1C ("WRITE DATA",unitout,1+nscansout)
            IF (nscansout.EQ.1) THEN
              cris1c_h_startdatayr = cris1c_scnlinyr
              cris1c_h_startdatady = cris1c_scnlindy
              cris1c_h_startdatatime = cris1c_scnlintime(1)
            ENDIF
          ENDIF
        ENDDO
        cris1c_h_scnlin = nscansout
        cris1c_h_enddatayr = cris1c_scnlinyr
        cris1c_h_enddatady = cris1c_scnlindy
        cris1c_h_enddatatime = cris1c_scnlintime(1)
        CALL IOC1C ("WRITE HEADER",unitout,1)
        CALL IOC1C ("CLOSE",unitout,0)
        CALL IOC1C ("CLOSE",unitin,0)

      ELSE IF (instrument.EQ."MWHS2") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOMWHS21C ("READ DATA",unitin,1+ScanIndex(iscan))
            mwhs21c_scnlin = iscan
              !Check first and last latitude
            IF (mwhs21c_latlon(1,1) .lt. 9999000 .and. 
     &          mwhs21c_latlon(1,mwhs2_nfovs) .lt. 9999000) then
              nscansout = nscansout + 1
              CALL IOMWHS21C ("WRITE DATA",unitout,1+nscansout)
              IF (nscansout.EQ.1) THEN
                mwhs21c_h_startdatayr = mwhs21c_scnlinyr
                mwhs21c_h_startdatady = mwhs21c_scnlindy
                mwhs21c_h_startdatatime = mwhs21c_scnlintime
              ENDIF
            ENDIF
          ENDIF
        ENDDO
        mwhs21c_h_scnlin = nscansout
        mwhs21c_h_enddatayr = mwhs21c_scnlinyr
        mwhs21c_h_enddatady = mwhs21c_scnlindy
        mwhs21c_h_enddatatime = mwhs21c_scnlintime
        CALL IOMWHS21C ("WRITE HEADER",unitout,1)
        CALL IOMWHS21C ("CLOSE",unitout,0)
        CALL IOMWHS21C ("CLOSE",unitin,0)

      ELSE IF (instrument.EQ."IRAS") THEN
        DO iscan = 1,LastScan
          IF (ScanIndex(iscan) .GT. 0) THEN
            CALL IOIRAS1C ("READ DATA",unitin,1+ScanIndex(iscan))
            iras1c_scnlin = iscan
              !Check first and last latitude
            IF (iras1c_latlon(1,1) .lt. 9999000 .and. 
     &          iras1c_latlon(1,iras_nfovs) .lt. 9999000) then
              nscansout = nscansout + 1
              CALL IOIRAS1C ("WRITE DATA",unitout,1+nscansout)
              IF (nscansout.EQ.1) THEN
                iras1c_h_startdatayr = iras1c_scnlinyr
                iras1c_h_startdatady = iras1c_scnlindy
                iras1c_h_startdatatime = iras1c_scnlintime
              ENDIF
            ENDIF
          ENDIF
        ENDDO
        iras1c_h_scnlin = nscansout
        iras1c_h_enddatayr = iras1c_scnlinyr
        iras1c_h_enddatady = iras1c_scnlindy
        iras1c_h_enddatatime = iras1c_scnlintime
        CALL IOIRAS1C ("WRITE HEADER",unitout,1)
        CALL IOIRAS1C ("CLOSE",unitout,0)
        CALL IOIRAS1C ("CLOSE",unitin,0)

      ENDIF

      WRITE(*,*)'Number of scans in: ',nscansin
      WRITE(*,*)'Number of scans out:',nscansout
      END
