!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C(KEL,NOBS,VALUES,NSCAN,CINSTR,MODE,KRET,
     &    WMO_SATID, ENHANCED_IASI, CENTRE, SUBCENTRE)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c.F 400 2014-06-26 08:49:16Z frna $
!
! Purpose: Get data from BUFR array and write it as level 1c AAPP format
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0      13/09/05  N.C.Atkinson
! 1.1      07/03/06  Compatible with NOAA simulated IASI data. NCA
! 1.2      20/09/06  Azimuth angles signed in AAPP, +ve in BUFR
! 1.3      08/12/06  Distinguish between AMSU-B and MHS in code table 002048
! 1.4      18/12/06  Correct hrs1c_mjfrcnt
! 1.5      04/01/07  Modify Mask033033 for 20 channels (was only 15)
! 1.6      27/03/07  Create dataqual flags if missing
! 1.7      22/05/07  Correct scaling factor for iasi1c_avhrrCGy & CGz
! 1.8      21/06/07  Improve robustness when there are missing times
! 1.9      05/10/07  Further improvement when there are missing times for IASI
! 1.10     26/11/07  Update comments giving IASI BUFR descriptor numbers
! 1.11     04/02/08  Height and period from fdf data where available
! 1.12     02/09/08  Bug fix: day numbers wrong in September 2008
! 1.13     03/12/08  Get the satids from fdf data rather than hard coding
! 1.14     26/11/09  Add MSU. Check the instrument type in word 8. Allow user
!                    to specify a particular WMO_SATID. Various mods to 
!                    work with ECMWF archived TOVS and ATOVS data.
! 1.15     11/01/10  Add PCIASI and Enhanced IASI 
! 1.16     19/05/10  Bug fix, the iasi1c_GQisFlagQual was only partially set.
! 2.0      16/07/10  Move the instrument-specific parts into own subroutines
!                    and add CrIS and ATMS
! 2.1      19/07/11  Fix compilation error for null literal string.
! 2.2      23/08/12  Add MWTS and MWHS
! 2.3      15/02/13  Add IRAS
! 2.4      20/06/14  Add MWTS2 and MWHS2
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "ama1c.h"
#include "amb1c.h"
#include "msu1c.h"
#include "hrs1c.h"
#include "iasi1c.h"
#include "iasipc.h"
#include "cris1c.h"
#include "atms1c.h"
#include "mwts1c.h"
#include "mwhs1c.h"
#include "iras1c.h"
#include "mwts21c.h"
#include "mwhs21c.h"

! Arguments:

      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      INTEGER NSCAN       !Number of scans processed
      INTEGER MODE        !0 to open 1c file and write data records, 
                          !1 to write header and close file
      INTEGER KRET        !0 if OK, 1 to stop
      INTEGER WMO_SATID   !Optional user-specified sat id
      CHARACTER*6  CINSTR !Instrument name
      REAL*8 VALUES(*)    !Data values
      LOGICAL ENHANCED_IASI
      INTEGER CENTRE      !From section 1 
      INTEGER SUBCENTRE   !From section 1 
      
! Parameters:

      REAL*8    Rmax
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      
! Local variables:

      INTEGER   H1cunit,A1cunit,B1cunit,I1cunit,M1cunit,Ipcunit,C1cunit,
     &        AT1cunit,MWTS1cunit,MWHS1cunit,IRAS1cunit
      DATA H1cunit/10/, A1cunit/11/, B1cunit/12/, I1cunit/13/,
     &        M1cunit/14/, Ipcunit/15/, C1cunit/16/, AT1cunit/17/,
     &        MWTS1cunit/18/, MWHS1cunit/19/, IRAS1cunit/21/
      
      INTEGER J         !Loop index
      INTEGER I         !Values index
      INTEGER K         !Index
       
      INTEGER Irec      
      INTEGER IOB       !Observation index

      LOGICAL First
      DATA First/.TRUE./
      
      CHARACTER*8 Firstscan,Lastscan
      INTEGER Ifirst,Ilast
      DATA Ifirst /1/, Ilast /999999/

!End of header
!------------------------------------------------------------------------------
      
      IF (MODE.EQ.0) THEN

        IF (First) THEN
          First=.FALSE.
          CALL GETENV('FIRSTSCAN',Firstscan)
          CALL GETENV('LASTSCAN',Lastscan)
          IF (Firstscan(1:1).NE." ") READ(Firstscan,*) Ifirst
          IF (Lastscan(1:1).NE." ")  READ(Lastscan,*)  Ilast
          IF (Firstscan(1:1).NE." " .OR. Lastscan(1:1).NE." ")
     &       WRITE(*,*)'Ifirst, Ilast:',Ifirst, Ilast
          NSCAN=0
          KRET=0
        ENDIF
        
!Loop round observations

        DO IOB=1,NOBS
        
          I = (IOB-1)*KEL       !Point to data in VALUES array
          
!----------------------------------------------------------------------------
!AMSU-A data
!----------------------------------------------------------------------------

          IF (CINSTR.EQ."AMSU-A".AND.VALUES(I+8).EQ.3.AND.
     &        (WMO_SATID.LE.0.OR.VALUES(I+7).EQ.WMO_SATID)) THEN

            CALL AAPP_GET_1C_AMSUA(NSCAN,I,IOB,NOBS,VALUES)

!----------------------------------------------------------------------------
!AMSU-B data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."AMSU-B".AND.
     &            (VALUES(I+8).EQ.4.OR.VALUES(I+8).EQ.11).AND.
     &        (WMO_SATID.LE.0.OR.VALUES(I+7).EQ.WMO_SATID)) THEN

            CALL AAPP_GET_1C_AMSUB(NSCAN,I,IOB,NOBS,VALUES)

!----------------------------------------------------------------------------
!MSU data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MSU".AND.VALUES(I+8).EQ.1.AND.
     &        (WMO_SATID.LE.0.OR.VALUES(I+7).EQ.WMO_SATID)) THEN

            CALL AAPP_GET_1C_MSU(NSCAN,I,IOB,NOBS,VALUES)

!----------------------------------------------------------------------------
!HIRS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."HIRS".AND.VALUES(I+8).EQ.0.AND.
     &        (WMO_SATID.LE.0.OR.VALUES(I+7).EQ.WMO_SATID)) THEN

            CALL AAPP_GET_1C_HIRS(NSCAN,I,IOB,NOBS,VALUES)

!----------------------------------------------------------------------------
!IASI data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."IASI") THEN

            CALL AAPP_GET_1C_IASI(NSCAN,I,IOB,NOBS,VALUES,
     &          ENHANCED_IASI,Ifirst,Ilast,KRET)

!----------------------------------------------------------------------------
!IASI PC data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."PCIASI") THEN

            CALL AAPP_GET_1C_PCIASI(NSCAN,I,IOB,NOBS,VALUES,
     &          Ifirst,Ilast,KRET)

!----------------------------------------------------------------------------
!CrIS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."CRIS") THEN

            CALL AAPP_GET_1C_CRIS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &            VALUES)

!----------------------------------------------------------------------------
!ATMS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."ATMS") THEN

            CALL AAPP_GET_1C_ATMS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &            VALUES)

!----------------------------------------------------------------------------
!MWTS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWTS") THEN

            CALL AAPP_GET_1C_MWTS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &            VALUES)

!----------------------------------------------------------------------------
!MWTS2 data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWTS2") THEN

            CALL AAPP_GET_1C_MWTS2(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &            VALUES)

!----------------------------------------------------------------------------
!MWHS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWHS") THEN

            CALL AAPP_GET_1C_MWHS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &            VALUES)

!----------------------------------------------------------------------------
!MWHS2 data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWHS2") THEN

            CALL AAPP_GET_1C_MWHS2(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &            VALUES)

!----------------------------------------------------------------------------
!IRAS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."IRAS") THEN

            CALL AAPP_GET_1C_IRAS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &            VALUES)

          ENDIF

!----------------------------------------------------------------------------
!All data types
!----------------------------------------------------------------------------

        ENDDO   !Loop round observations

      ELSE    !MODE.EQ.0

        Irec=1
        IF (CINSTR.EQ."HIRS")   CALL IOH1C("WRITE HEADER",H1cunit,Irec)
        IF (CINSTR.EQ."AMSU-A") CALL IOA1C("WRITE HEADER",A1cunit,Irec)
        IF (CINSTR.EQ."AMSU-B") CALL IOB1C("WRITE HEADER",B1cunit,Irec)
        IF (CINSTR.EQ."IASI")   CALL IOI1C("WRITE HEADER",I1cunit,Irec)
        IF (CINSTR.EQ."PCIASI") CALL IOIPC("WRITE HEADER",Ipcunit,Irec)
        IF (CINSTR.EQ."MSU")    CALL IOM1C("WRITE HEADER",M1cunit,Irec)
        IF (CINSTR.EQ."CRIS")   CALL IOC1C("WRITE HEADER",C1cunit,Irec)
        IF (CINSTR.EQ."ATMS")  CALL IOAT1C("WRITE HEADER",AT1cunit,Irec)
        IF (CINSTR.EQ."MWTS")  
     &    CALL IOMWTS1C("WRITE HEADER",MWTS1cunit,Irec)
        IF (CINSTR.EQ."MWTS2")  
     &    CALL IOMWTS21C("WRITE HEADER",MWTS1cunit,Irec)
        IF (CINSTR.EQ."MWHS")
     &    CALL IOMWHS1C("WRITE HEADER",MWHS1cunit,Irec)
        IF (CINSTR.EQ."MWHS2")
     &    CALL IOMWHS21C("WRITE HEADER",MWHS1cunit,Irec)
        IF (CINSTR.EQ."IRAS")
     &    CALL IOIRAS1C("WRITE HEADER",IRAS1cunit,Irec)
        IF (CINSTR.EQ."HIRS")   CALL IOH1C("CLOSE",H1cunit,Irec)
        IF (CINSTR.EQ."AMSU-A") CALL IOA1C("CLOSE",A1cunit,Irec)
        IF (CINSTR.EQ."AMSU-B") CALL IOB1C("CLOSE",B1cunit,Irec)
        IF (CINSTR.EQ."IASI")   CALL IOI1C("CLOSE",I1cunit,Irec)
        IF (CINSTR.EQ."PCIASI") CALL IOIPC("CLOSE",Ipcunit,Irec)
        IF (CINSTR.EQ."MSU")    CALL IOM1C("CLOSE",M1cunit,Irec)
        IF (CINSTR.EQ."CRIS")   CALL IOC1C("CLOSE",C1cunit,Irec)
        IF (CINSTR.EQ."ATMS")   CALL IOAT1C("CLOSE",AT1cunit,Irec)
        IF (CINSTR.EQ."MWTS")   CALL IOMWTS1C("CLOSE",MWTS1cunit,Irec)
        IF (CINSTR.EQ."MWTS2")  CALL IOMWTS21C("CLOSE",MWTS1cunit,Irec)
        IF (CINSTR.EQ."MWHS")   CALL IOMWHS1C("CLOSE",MWHS1cunit,Irec)
        IF (CINSTR.EQ."MWHS2")  CALL IOMWHS21C("CLOSE",MWHS1cunit,Irec)
        IF (CINSTR.EQ."IRAS")   CALL IOIRAS1C("CLOSE",IRAS1cunit,Irec)

      ENDIF   !MODE.EQ.1
      RETURN
      END

!-----------------------------------------------------------------------------
      
      FUNCTION FNSCALE(Rval,Scale)
      INTEGER FNSCALE
      REAL*8 Rval
      REAL*8 Scale
      
      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      IF (Rval.GT.Rmax) THEN
        FNSCALE = Imiss
      ELSE
        FNSCALE = NINT(Rval*Scale)
      ENDIF
      
      RETURN
      END

!-----------------------------------------------------------------------

      FUNCTION DAYOFYEAR(Y,M,D)
      INTEGER    DAYOFYEAR
      REAL*8     Y,M,D
      INTEGER    IY,IM,ID
      INTEGER    NOLEAP(12), LEAP(12)
      DATA       NOLEAP /0,31,59,90,120,151,181,212,243,273,304,334/
      DATA       LEAP   /0,31,60,91,121,152,182,213,244,274,305,335/
      IY = Y
      IM = M
      ID = D
      IF (MOD(IY,4).EQ.0) THEN         !OK up to 2100
        DAYOFYEAR = LEAP(IM) + ID
      ELSE
        DAYOFYEAR = NOLEAP(IM) + ID
      ENDIF
      RETURN
      END

!-----------------------------------------------------------------------

      SUBROUTINE UnBitSwap(BitIn, Mask, NumBitsIn, BitOut, NumBitsOut)
      IMPLICIT NONE

! purpose         : To reverse the steps taken by bitswap.f
!                 : bitswap.f takes an input bit field (BitIn) and sets the bits
!                 : in the output bit field (BitOut) detrmined by the
!                 : mapping in the Mask field.                     
!                                                                               
! calls           : BTEST  : To see if a bit is set
!                 : IBSET  : To set a bit
!                                                                               
! Description of Mask :
! 
! The mask is of the size of the input flag (bit) table. Each 
! element in the mask is the bit in the output flag table to set
! corresponding to the bit in the input flag table. 
!
! Arguments:

      INTEGER BitIn                !- i/p bit value (Integer form)
      INTEGER NumBitsIn            !- size of i/p bit table
      INTEGER Mask(NumBitsIn)      !- input bit mask (see description)
      INTEGER BitOut               !- o/p bit value (Integer form)
      INTEGER NumBitsOut           !- sze of o/p bit table

! Local variables:

      INTEGER I                    !- used for looping
      LOGICAL BitSet               !- TRUE if bit set
      
      BitIn = 0

      DO I = 1,NumBitsIn               !Loop over bits
        IF (Mask(I).GT.0) THEN
          BitSet = BTEST(BitOut,NumBitsOut-Mask(I))
          IF (BitSet) BitIn = IBSET(BitIn,NumBitsIn-I)
        ENDIF
      ENDDO

      RETURN
      END

