!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C_AMSUA(NSCAN,I,IOB,NOBS,VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c_amsua.F 543 2017-08-14 14:35:09Z frna $
!
! Purpose: Extract AMSU-A-specific values from BUFR decode array 
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    16/07/2010  From aapp_get_1c.F
! 1.1    08/04/2014  Allow output when ama1c_scnlin is zero
! 1.2    14/09/2015  Add NEDT if present 
! 1.3    14/08/2017  Correct array index for NEDT and Compute average warmnedt
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "ama1c.h"
#include "bits.h"

! Arguments:

      INTEGER NSCAN     !Number of scans processed
      INTEGER I         !Index to values
      INTEGER IOB       !Observation index
      INTEGER NOBS      !Number of observations
      REAL*8 VALUES(*)  !Data values
      
! Parameters:

      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:

      INTEGER   A1cunit   !Unit number
      DATA A1cunit/11/
      
      INTEGER FOVOffset
      DATA FOVOffset/0/
      INTEGER FOV       !Numbered from 1
      REAL*8 Spot       !Spot number (real)

      INTEGER J         !Loop index
      INTEGER K         !Index
      INTEGER BIT       !For use with unbitswap

      INTEGER    Mask033030(NumBitsIn) !- bit mask for 033030
      INTEGER    Mask033031(NumBitsIn) !- bit mask for 033031
      INTEGER    Mask033032(NumBitsIn) !- bit mask for 033032
      INTEGER    Mask033033(NumBitsIn) !- bit mask for 033033

      DATA Mask033030 /1,2,3,4,5,6,7,25*0/
      DATA Mask033031 /8*0,1,2,3,4,4*0,5,6,7,8,9,10,11,12,13,14,15,16,
     &                 17,3*0/
      DATA Mask033032 /26*0,1,2,3,4,5,6/
      DATA Mask033033 /0,1,9*0,21,20,19,18,17,16,15,14,13,12,11,10,9,
     &                 8,7,6,5,4,3,2,22/

      LOGICAL First
      DATA First/.TRUE./
      
      real*8  warmnedt_sumsq(Anchan)
      integer warmnedt_num(Anchan)
      DATA warmnedt_sumsq/Anchan*0.0D0/
      DATA warmnedt_num/Anchan*0/

! Functions

      INTEGER FNSCALE
      INTEGER DAYOFYEAR
      
!End of header
!------------------------------------------------------------------------------
      
!Check that the spot number is not missing; if it is, skip the ob
!Also, some encoding schemes number the FOV from 0, others from 1. Allow both.
      SPOT = VALUES(I+12)
      IF (SPOT .GT. Rmax) THEN
        WRITE(*,*)'Ob',IOB,' has missing FOV number'
        RETURN
      ENDIF
      IF (SPOT.EQ.0.0) FOVOffset=1      !FOVS numbered from 0
      FOV = SPOT + FOVOffset

!First time round, open the output file
      IF (First) THEN
        First=.FALSE.
        WRITE(*,*)'Creating 1c output dataset for AMSU-A'
        CALL IOA1C("OPEN/WRITE",A1cunit,0)
        ama1c_h_satht = 8330     !Initialise (get proper values later)
        ama1c_h_period = 6081
      ENDIF


      IF (FOV.EQ.1) THEN          !First spot
        ama1c_scnlin = FNSCALE(VALUES(I+11),1.0D0)
        ama1c_scnlinyr = FNSCALE(VALUES(I+16),1.0D0)
        IF (VALUES(I+16).GT.Rmax.OR.VALUES(I+17).GT.Rmax.OR.
     &              VALUES(I+18).GT.Rmax) THEN
          ama1c_scnlindy = Imiss
        ELSE
          ama1c_scnlindy = DAYOFYEAR(VALUES(I+16),VALUES(I+17),
     &                 VALUES(I+18))   
        ENDIF
        ama1c_scnlintime = FNSCALE(VALUES(I+19),3600.0D3)  !millisecs
        ama1c_scnlintime = ama1c_scnlintime + 
     &             FNSCALE(VALUES(I+20),60.0D3)   !Add minutes
        ama1c_scnlintime = ama1c_scnlintime + 
     &             FNSCALE(VALUES(I+21),1.0D3)    !Add seconds

        IF (VALUES(I+14).LT.Rmax) THEN
          Bit = VALUES(I+14)
          CALL UNBITSWAP(ama1c_qualind, Mask033030, NumBitsIn, 
     &           Bit, NumBitsOut)
        ELSE
          ama1c_qualind = Imiss
        ENDIF
        IF (VALUES(I+15).LT.Rmax) THEN
          Bit = VALUES(I+15)
          CALL UNBITSWAP(ama1c_scanqual, Mask033031, NumBitsIn, 
     &           Bit, NumBitsOut)
        ELSE
          ama1c_scanqual = Imiss
        ENDIF
        DO J=1,15
          IF (VALUES(I+42+(J-1)*6).LT.Rmax) THEN
            Bit = VALUES(I+42+(J-1)*6)
            CALL UNBITSWAP(ama1c_chanqual(J), Mask033032, 
     &           NumBitsIn, Bit, NumBitsOut)
          ELSE
            ama1c_chanqual(J) = Imiss
          ENDIF
        ENDDO
        
        ama1c_instrtemp1 = FNSCALE(VALUES(I+33),100.0D0)  !K*100
        ama1c_instrtemp2 = FNSCALE(VALUES(I+35),100.0D0)  !K*100
        ama1c_scalti = FNSCALE(VALUES(I+24),1.0D-2)       !km*10

!Fill part of header

        IF (NSCAN.EQ.0) THEN
          ama1c_h_startorbit = FNSCALE(VALUES(I+9),1.0D0)
          ama1c_h_startdatayr = ama1c_scnlinyr
          ama1c_h_startdatady = ama1c_scnlindy
          ama1c_h_startdatatime = ama1c_scnlintime
        ENDIF

      ENDIF      !FOV.EQ.1
              
!Fill the parts that change with each pixel

      DO J=1,2
        ama1c_latlon(J,FOV) = FNSCALE(VALUES(I+21+J),1.0D4) !10^4 x latlon
      ENDDO

      DO J=1,3,2      !Zenith angles
        ama1c_angles(J,FOV) = FNSCALE(VALUES(I+24+J),1.0D2) !10^2 x angle
      ENDDO

      DO J=2,4,2      !Azimuth angles (signed in AAPP, +ve in BUFR)
        ama1c_angles(J,FOV) = FNSCALE(VALUES(I+24+J),1.0D2) !10^2 x angle
        IF (ama1c_angles(J,FOV).NE.Imiss .AND.
     &              ama1c_angles(J,FOV).GT.18000) THEN
          ama1c_angles(J,FOV) = ama1c_angles(J,FOV) - 36000 !-180 to 180
        ENDIF
      ENDDO

      DO J=1,15
        ama1c_btemps(J,FOV) = 
     &             FNSCALE(VALUES(I+43+(J-1)*6),1.0D2) !10^2 x BT
      ENDDO

      IF (VALUES(I+29).LT.Rmax) THEN
        Bit = VALUES(I+29)
        CALL UNBITSWAP(ama1c_dataqual(FOV), Mask033033, 
     &           NumBitsIn, Bit, NumBitsOut)
      ELSE
        ama1c_dataqual(FOV) = 0  !If flag is missing, create it
        DO J=1,15
      IF (ama1c_btemps(J,FOV) .LE. AminBT*100 .OR.
     &    ama1c_btemps(J,FOV) .GE. AmaxBT*100 ) THEN
            ama1c_dataqual(FOV) = 
     &           IBSET(ama1c_dataqual(FOV),bits(J))
          ENDIF
        ENDDO 
      ENDIF
              
!Add NEDT if present
!i.e. if first bit map value is "false" and first bit map for BT is "true"

      IF (VALUES(I+130).EQ.1.AND.VALUES(I+172).EQ.0) THEN
        DO J=1,15
          ama1c_warmnedt(J) = FNSCALE(VALUES(I+J+258),1.0D3)
          if (ama1c_warmnedt(J).ne.Imiss.and.ama1c_warmnedt(J).ne.0)then
            warmnedt_sumsq(J) = warmnedt_sumsq(J)+VALUES(I+J+258)**2
            warmnedt_num(J) = warmnedt_num(J) + 1
          endif
        ENDDO
      ENDIF

! Write data record
 
      IF (FOV.EQ.30 .AND. ama1c_scnlin.GE.0) THEN
        NSCAN = NSCAN+1
        WRITE(6,*) 'Writing to data record',NSCAN
        CALL IOA1C("WRITE DATA", A1cunit, NSCAN+1) !1 header record
      ENDIF   !FOV.EQ.30

! Update header common block at conclusion of each message

      IF (IOB.EQ.NOBS) THEN
        ama1c_h_endorbit = FNSCALE(VALUES(I+9),1.0D0)
        ama1c_h_enddatayr = ama1c_scnlinyr
        ama1c_h_enddatady = ama1c_scnlindy
        ama1c_h_enddatatime = ama1c_scnlintime
        ama1c_h_scnlin = NSCAN
        ama1c_h_misscnlin = 0
        ama1c_h_vnantennacorr = FNSCALE(VALUES(I+10),1.0D0)
        DO J=1,15
          IF (VALUES(I+39+(J-1)*6) .LT. Rmax) THEN
            ama1c_h_tempradcnv(1,J) = 
     &               NINT(10**(VALUES(I+39+(J-1)*6))*1E4)    !10^6 x wavenumber
          ELSE
            ama1c_h_tempradcnv(1,J) = Imiss
          ENDIF
          ama1c_h_tempradcnv(2,J) = 
     &          FNSCALE(VALUES(I+40+(J-1)*6),1.0D6) !10^6 x CONST 1
          ama1c_h_tempradcnv(3,J) = 
     &          FNSCALE(VALUES(I+41+(J-1)*6),1.0D6) !10^6 x CONST 2
        ENDDO
        ama1c_h_site = "EUM"
        ama1c_h_cfill1 = " "
        ama1c_h_1bsite = "EUM"
        ama1c_h_cfill2 = " "
        ama1c_h_versnb = ama_1cvnnb
        ama1c_h_versyr = ama_1cvnyr
        ama1c_h_versdy = ama_1cvndy
        ama1c_h_hdrcnt = 1
        ama1c_h_wmosatid = FNSCALE(VALUES(I+7),1.0D0)
        IF (ama1c_h_wmosatid.GE.206) THEN
          ama1c_h_satid = ama1c_h_wmosatid-191   !NOAA satellites
        ELSE
          ama1c_h_satid = ama1c_h_wmosatid-2     !METOP
        ENDIF
        IF (ama1c_h_wmosatid.EQ.Imiss) ama1c_h_satid = Imiss
        ama1c_h_instrument = 10       !AMSU-A
! Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.ama1c_h_wmosatid) THEN
            ama1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            ama1c_h_period = OrbitPd(J)   !period, seconds
            ama1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO
        do J=1,Anchan
          if (warmnedt_num(J) .gt. 0) then
            ama1c_h_warmnedt(J) = 
     &       1000*sqrt(warmnedt_sumsq(J)/warmnedt_num(J))+0.5
          else
            ama1c_h_warmnedt(J) = ama1c_warmnedt(J)
          endif
        enddo
      
      ENDIF   !IOB.EQ.NOBS

      RETURN
      END

