!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C_ATMS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &     VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c_atms.F 610 2019-01-17 10:47:43Z frna $
!
! Purpose: Extract ATMS-specific values from BUFR decode array 
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    20/07/2010  New
! 1.1    03/07/2012  Correct NEdT scaling (make it same as the hdf5 SDR)
! 1.2    01/08/2017  atms1c_scnlin can be zero.
!                    Make enddatatime the time of the end of the last scan.
!                    Use the atms_channels.dat file instead of atmschandata.h
! 1.3    20/03/2018  Bug fix when reading atms_channels.dat
! 1.4    11/04/2018  Ensure header times are valid, i.e. ignore missing values
! 1.5    17/01/2019  BTs have been found to be unreliable if granule quality
!                    flag > 2, so set them to missing.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "atms1c.h"
#include "bits.h"
#include "constants_preproc.h"

! Arguments:

      INTEGER NSCAN     !Number of scans processed
      INTEGER I         !Index to values
      INTEGER IOB       !Observation index
      INTEGER NOBS      !Number of observations
      INTEGER CENTRE    !From section 1
      INTEGER SUBCENTRE !From section 1
      REAL*8 VALUES(*)  !Data values
      
! Parameters:

      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
! Local variables:

      INTEGER   AT1cunit   !Unit number
      DATA AT1cunit/17/
      
      INTEGER FOVOffset
      DATA FOVOffset/0/
      INTEGER FOV       !Field of view, numbered from 1
      REAL*8 Spot       !Spot number (real)

      INTEGER J         !Loop index
      INTEGER K         !Index
      INTEGER nchanused
      INTEGER chan
      INTEGER id, ichan
      REAL    channel_info(5,atms_1cnchan)
      DATA    channel_info /110*0/
      CHARACTER*80 line

      LOGICAL First
      DATA First/.TRUE./
      
! Functions

      INTEGER FNSCALE
      INTEGER DAYOFYEAR
      
!End of header
!------------------------------------------------------------------------------
      
!Check that the spot number is not missing; if it is, skip the ob
!Also, some encoding schemes number the FOV from 0, others from 1. Allow both.
      SPOT = VALUES(I+14)
      IF (SPOT .GT. Rmax) THEN
        WRITE(*,*)'Ob',IOB,' has missing FOV number'
        RETURN
      ENDIF
      IF (SPOT.EQ.0.0) FOVOffset=1      !FOVS numbered from 0
      FOV = SPOT + FOVOffset

!First time round, open the output file
      IF (First) THEN
        First=.FALSE.
        WRITE(*,*)'Creating 1c output dataset for ATMS'
        atms1c_h_satht = 8330
        atms1c_h_period = 6081
        atms1c_h_nchan = atms_1cnchan
        atms1c_h_recwords = array_size_atms1c
        atms1c_h_startdatatime = Imiss
        CALL IOAT1C("OPEN/WRITE",AT1cunit,0)
      ENDIF

      atms1c_scnlintime(FOV) = FNSCALE(VALUES(I+9),3600.0D3)  !millisecs
      IF (atms1c_scnlintime(FOV).NE.Imiss) THEN
        atms1c_scnlintime(FOV) = atms1c_scnlintime(FOV) + 
     &           FNSCALE(VALUES(I+10),60.0D3)  !Add minutes
        atms1c_scnlintime(FOV) = atms1c_scnlintime(FOV) + 
     &           FNSCALE(VALUES(I+11),1.0D3)   !Add seconds
      ENDIF

      IF (FOV.EQ.1) THEN  !First spot
        atms1c_scnlinyr = FNSCALE(VALUES(I+6),1.0D0)
        IF (VALUES(I+6).GT.Rmax.OR.VALUES(I+7).GT.Rmax.OR.
     &              VALUES(I+8).GT.Rmax) THEN
          atms1c_scnlindy = Imiss
        ELSE
          atms1c_scnlindy = DAYOFYEAR(VALUES(I+6),VALUES(I+7),
     &                 VALUES(I+8))   
        ENDIF

        atms1c_orbit = FNSCALE(VALUES(I+12),1.0D0)
        atms1c_scnlin = FNSCALE(VALUES(I+13),1.0D0)
        atms1c_granulequal = FNSCALE(VALUES(I+15),1.0D0)
        atms1c_scanqual = FNSCALE(VALUES(I+16),1.0D0)
        atms1c_geolocqual = FNSCALE(VALUES(I+17),1.0D0)
        atms1c_scalti = FNSCALE(VALUES(I+20),1.0D-2)      !km*10

        nchanused = VALUES(I+26)
        DO J=1,nchanused
          chan = FNSCALE(VALUES(I+18+J*9),1.0D0)
          IF (chan.NE.Imiss) THEN
            atms1c_nedtcold(chan) = FNSCALE(VALUES(I+24+J*9),1.0D4)
            atms1c_nedtwarm(chan) = FNSCALE(VALUES(I+25+J*9),1.0D4)
            atms1c_chanqual(chan) = FNSCALE(VALUES(I+26+J*9),1.0D0)
          ENDIF
        ENDDO

!Fill part of header

        IF (atms1c_h_startdatatime .LT. 0) THEN
          atms1c_h_startorbit = atms1c_orbit
          atms1c_h_startdatayr = atms1c_scnlinyr
          atms1c_h_startdatady = atms1c_scnlindy
          atms1c_h_startdatatime = atms1c_scnlintime(1)
        ENDIF
        atms1c_h_nchanused = VALUES(I+26)

      ENDIF      !FOV.EQ.1
              
!Fill the parts that change with each pixel

      DO J=1,2
        atms1c_latlon(J,FOV) = FNSCALE(VALUES(I+17+J),1.0D4) !10^4 x latlon
      ENDDO

      DO J=1,3,2      !Zenith angles
        atms1c_angles(J,FOV) = FNSCALE(VALUES(I+20+J),1.0D2) !10^2 x angle
      ENDDO

      DO J=2,4,2      !Azimuth angles (signed in AAPP, +ve in BUFR)
        atms1c_angles(J,FOV) = FNSCALE(VALUES(I+20+J),1.0D2) !10^2 x angle
        IF (atms1c_angles(J,FOV).NE.Imiss .AND.
     &              atms1c_angles(J,FOV).GT.18000) THEN
          atms1c_angles(J,FOV) = atms1c_angles(J,FOV) - 36000 !-180 to 180
        ENDIF
      ENDDO

!Brightness temps. Set to missing if granule quality flag is greater than 2
! (indicates bad or missing data in the downlink)

      nchanused = VALUES(I+26)
      DO J=1,nchanused
        chan = FNSCALE(VALUES(I+18+J*9),1.0D0)
        IF (chan.NE.Imiss .and. atms1c_granulequal.LE.2) THEN
          atms1c_anttemps(chan,FOV) = FNSCALE(VALUES(I+22+J*9),1.0D2)
          atms1c_btemps(chan,FOV) = FNSCALE(VALUES(I+23+J*9),1.0D2)
        ELSE
          atms1c_anttemps(chan,FOV) = Imiss
          atms1c_btemps(chan,FOV) = Imiss
        ENDIF
      ENDDO
             
! Write data record
 
      IF (FOV.EQ.atms_1cnfov .AND. atms1c_scnlin.GE.0) THEN
        NSCAN = NSCAN+1
        WRITE(6,*) 'Writing to data record',NSCAN
        CALL IOAT1C("WRITE DATA", AT1cunit, NSCAN+1) !1 header record
      ENDIF

! Update header common block at conclusion of each message

      IF (IOB.EQ.NOBS) THEN
        atms1c_h_nfov = atms_1cnfov
        atms1c_h_nchan = atms_1cnchan
        atms1c_h_nchanused = nchanused
        IF (atms1c_scnlintime(atms_1cnfov) .GE. 0) THEN
          atms1c_h_endorbit     = FNSCALE(VALUES(I+12),1.0D0)
          atms1c_h_enddatayr = atms1c_scnlinyr
          atms1c_h_enddatady = atms1c_scnlindy
          atms1c_h_enddatatime = atms1c_scnlintime(atms_1cnfov)
          if ((atms1c_scnlintime(1)-atms1c_scnlintime(atms_1cnfov))
     &      .gt.86392000) then
            atms1c_h_enddatady = atms1c_h_enddatady + 1
            if ((atms1c_h_enddatady .gt. 366) .or. 
     &         (MOD(atms1c_h_enddatayr,4) .ne. 0 .and. 
     &         atms1c_h_enddatady .gt. 365)) then
              atms1c_h_enddatady = 1
              atms1c_h_enddatayr = atms1c_h_enddatayr + 1
            endif
          endif
        endif
        atms1c_h_scnlin = NSCAN
        atms1c_h_misscnlin = 0
        atms1c_h_vnantennacorr = FNSCALE(VALUES(I+25),1.0D0)
        IF (VALUES(I+2).GT.0.AND.VALUES(I+2).LT.999) THEN
          WRITE (atms1c_h_site,'(I3.3)') INT(VALUES(I+2))
        ELSE
          atms1c_h_site = "xxx"
        ENDIF
        atms1c_h_cfill1 = " "
        IF (VALUES(I+3).GT.0.AND.VALUES(I+3).LT.999) THEN
          WRITE (atms1c_h_1bsite,'(I3.3)') INT(VALUES(I+3))
        ELSE
          atms1c_h_1bsite = "xxx"
        ENDIF
        atms1c_h_cfill2 = " "
        atms1c_h_versnb = atms_1cvnnb
        atms1c_h_versyr = atms_1cvnyr
        atms1c_h_versdy = atms_1cvndy
        atms1c_h_hdrcnt = 1
        atms1c_h_wmosatid = FNSCALE(VALUES(I+1),1.0D0)
        IF (atms1c_h_wmosatid.EQ.Imiss) THEN
          atms1c_h_satid = Imiss
        ELSE
          atms1c_h_satid = 225-atms1c_h_wmosatid               !1 for NPP
        ENDIF
        atms1c_h_instrument = FNSCALE(VALUES(I+4),1.0D0)       !621 for ATMS
        atms1c_h_centre = CENTRE
        atms1c_h_subcentre = SUBCENTRE

! Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.atms1c_h_wmosatid) THEN
            atms1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            atms1c_h_period = OrbitPd(J)   !period, seconds
            atms1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO
      
  !Read channel info from the atms_channels.dat file (same as in atms_sdr_out.F)
        IF (channel_info(1,1) .eq. 0) then
          OPEN (30,form='formatted',status='old')
          line="     "
          id = 0
          DO while (id .NE. atms1c_h_wmosatid)
            DO while (line(1:5) .ne. "# WMO")
              read(30,'(A)',END=100) line
            ENDDO
            read(30,*) id
            line="     "
            IF (id .eq. atms1c_h_wmosatid) THEN
              read(30,*) line
              read(30,*) channel_info
            ENDIF
          ENDDO
  100     CLOSE (30)
          DO ichan=1,atms_1cnchan
            atms1c_h_tempradcnv(1,ichan) = 
     &        channel_info(2,ichan)*1.0E13/C_light   !wavenumber*1E6
            atms1c_h_tempradcnv(2,ichan) = channel_info(4,ichan)*1.0E6  !offset
            atms1c_h_tempradcnv(3,ichan) = channel_info(5,ichan)*1.0E6  !slope
            atms1c_h_bandwidth(ichan) = channel_info(3,ichan)*1.0E6     !GHz*1E6
          ENDDO
        ENDIF

      ENDIF   !IOB.EQ.NOBS

      RETURN
      END

