!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C_CRIS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &          VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c_cris.F 536 2017-08-09 10:40:30Z frna $
!
! Purpose: Extract CrIS-specific values from BUFR decode array 
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    16/07/2010  New
! 1.1    15/10/2012  Correct scaling in surfht
! 1.2    21/06/2017  Updated cris1c.h with 12 extra channels
! 1.3    01/08/2017  Make enddatatime the time of the end of the last scan.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "cris1c.h"
#include "bits.h"

! Arguments:

      INTEGER NSCAN     !Number of scans processed
      INTEGER I         !Index to values
      INTEGER IOB       !Observation index
      INTEGER NOBS      !Number of observations
      INTEGER CENTRE    !From section 1
      INTEGER SUBCENTRE !From section 1
      REAL*8 VALUES(*)  !Data values
      
! Parameters:

      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
! Local variables:

      INTEGER   C1cunit   !Unit number
      DATA C1cunit/16/
      
      INTEGER FOVOffset
      DATA FOVOffset/0/
      INTEGER FofR      !Field of Regard, numbered from 1
      REAL*8 Spot       !Spot number (real)

      INTEGER J         !Loop index
      INTEGER K         !Index
      INTEGER BIT       !For use with unbitswap
      INTEGER FOV       !1 to 9
      INTEGER nchanused

      LOGICAL First
      DATA First/.TRUE./
      
! Functions

      INTEGER FNSCALE
      INTEGER DAYOFYEAR
      
!End of header
!------------------------------------------------------------------------------
      
!Check that the spot number is not missing; if it is, skip the ob
!Also, some encoding schemes number the FOV from 0, others from 1. Allow both.
      SPOT = VALUES(I+22)
      IF (SPOT .GT. Rmax) THEN
        WRITE(*,*)'Ob',IOB,' has missing Field of Regard number'
        RETURN
      ENDIF
      IF (SPOT.EQ.0.0) FOVOffset=1      !FOVS numbered from 0
      FofR = SPOT + FOVOffset

!First time round, define the record length and open the output file
      IF (First) THEN
        First=.FALSE.
        WRITE(*,*)'Creating 1c output dataset for CrIS'
        cris1c_h_satht = 8330
        cris1c_h_period = 6081
        cris1c_h_nchan = VALUES(I+57)
        cris1c_h_recwords = 
     &         11 + cris_1cnfor + cris_1cnband*cris_1cnfov + 
     &         (13 + 2*cris_1cnband + cris1c_h_nchan +12 + cris_1cnpc)
     &              *cris_1cnfov*cris_1cnfor
        CALL IOC1C("OPEN/WRITE",C1cunit,0)
      ENDIF

      FOV = FNSCALE(VALUES(I+23),1.0D0)

      cris1c_scnlintime(FofR) = FNSCALE(VALUES(I+8),3600.0D3)  !millisecs
      IF (cris1c_scnlintime(FofR).NE.Imiss) THEN
        cris1c_scnlintime(FofR) = cris1c_scnlintime(FofR) + 
     &           FNSCALE(VALUES(I+9),60.0D3)   !Add minutes
        cris1c_scnlintime(FofR) = cris1c_scnlintime(FofR) + 
     &           FNSCALE(VALUES(I+10),1.0D3)   !Add seconds
      ENDIF

      IF (FofR.EQ.1 .AND. (FOV.EQ.1.OR.cris_1cnfov.NE.9)) THEN  !First spot
        cris1c_scnlin = FNSCALE(VALUES(I+21),1.0D0)
        cris1c_scnlinyr = FNSCALE(VALUES(I+5),1.0D0)
        IF (VALUES(I+5).GT.Rmax.OR.VALUES(I+6).GT.Rmax.OR.
     &              VALUES(I+7).GT.Rmax) THEN
          cris1c_scnlindy = Imiss
        ELSE
          cris1c_scnlindy = DAYOFYEAR(VALUES(I+5),VALUES(I+6),
     &                 VALUES(I+7))   
        ENDIF

        cris1c_satxyz(1) = FNSCALE(VALUES(I+11),1.0D0)
        cris1c_satxyz(2) = FNSCALE(VALUES(I+12),1.0D0)
        cris1c_satxyz(3) = FNSCALE(VALUES(I+13),1.0D0)
        cris1c_ascdesc = FNSCALE(VALUES(I+20),1.0D0)
        cris1c_orbit = FNSCALE(VALUES(I+24),1.0D0)
        cris1c_scalti = FNSCALE(VALUES(I+26),1.0D-2)      !km*10
        cris1c_scanqualflgs = FNSCALE(VALUES(I+32),1.0D0)
        cris1c_geolocqual = FNSCALE(VALUES(I+55),1.0D0)

!Fill part of header

        IF (NSCAN.EQ.0) THEN
          cris1c_h_startorbit = cris1c_orbit
          cris1c_h_startdatayr = cris1c_scnlinyr
          cris1c_h_startdatady = cris1c_scnlindy
          cris1c_h_startdatatime = cris1c_scnlintime(1)
        ENDIF
        cris1c_h_nchanused = VALUES(I+57)

      ENDIF      !FofR.EQ.1
              
!Fill the parts that change with each pixel

      DO J=1,2
        cris1c_latlon(J,FOV,FofR) = FNSCALE(VALUES(I+13+J),1.0D4) !10^4 x latlon
      ENDDO

      DO J=1,3,2      !Zenith angles
        cris1c_angles(J,FOV,FofR) = FNSCALE(VALUES(I+15+J),1.0D2) !10^2 x angle
      ENDDO

      DO J=2,4,2      !Azimuth angles (signed in AAPP, +ve in BUFR)
        cris1c_angles(J,FOV,FofR) = FNSCALE(VALUES(I+15+J),1.0D2) !10^2 x angle
        IF (cris1c_angles(J,FOV,FofR).NE.Imiss .AND.
     &              cris1c_angles(J,FOV,FofR).GT.18000) THEN
          cris1c_angles(J,FOV,FofR) = cris1c_angles(J,FOV,FofR) - 36000 !-180 to 180
        ENDIF
      ENDDO

      cris1c_surfht(FOV,FofR) = FNSCALE(VALUES(I+25),1.0D0)
      cris1c_landfrac(FOV,FofR) = FNSCALE(VALUES(I+27),1.0D2)
      cris1c_landsea(FOV,FofR) = FNSCALE(VALUES(I+28),1.0D2)
      cris1c_cloudcover(FOV,FofR) = FNSCALE(VALUES(I+29),1.0D2)
      cris1c_cloudtopht(FOV,FofR) = FNSCALE(VALUES(I+30),1.0D2)
      cris1c_radianceflgs(FOV,FofR) = FNSCALE(VALUES(I+31),1.0D0)
      cris1c_calqualflgs(1,FOV) = FNSCALE(VALUES(I+38),1.0D0)
      cris1c_calqualflgs(2,FOV) = FNSCALE(VALUES(I+45),1.0D0)
      cris1c_calqualflgs(3,FOV) = FNSCALE(VALUES(I+52),1.0D0)
      cris1c_fovqualflgs(1,FOV,FofR) = FNSCALE(VALUES(I+39),1.0D0)
      cris1c_fovqualflgs(2,FOV,FofR) = FNSCALE(VALUES(I+46),1.0D0)
      cris1c_fovqualflgs(3,FOV,FofR) = FNSCALE(VALUES(I+53),1.0D0)
      cris1c_quality(FOV,FofR) = FNSCALE(VALUES(I+56),1.0D0)
      
      nchanused = VALUES(I+57)
      DO J=1,nchanused
        cris1c_radiance(FOV,FofR,J) = 
     &          FNSCALE(VALUES(I+57+J*2),1.0D7)   !Radiance W/m^2/str/cm^-1 *1E7
      ENDDO
             
! Write data record
 
      IF (FOV.EQ.cris_1cnfov .AND. FofR.EQ.cris_1cnfor .AND. 
     &      cris1c_scnlin.GT.0) THEN
        NSCAN = NSCAN+1
        WRITE(6,*) 'Writing to data record',NSCAN
        CALL IOC1C("WRITE DATA", C1cunit, NSCAN+1) !1 header record
      ENDIF   !FofR.EQ.30

! Update header common block at conclusion of each message

      IF (IOB.EQ.NOBS) THEN
!        cris1c_h_recwords = array_size_cris1c
        cris1c_h_nfov = cris_1cnfov
        cris1c_h_nfor = cris_1cnfor
        cris1c_h_nband = cris_1cnband
!        cris1c_h_nchan = cris_1cnchan
        cris1c_h_npc = cris_1cnpc
        cris1c_h_nchanused = nchanused
        cris1c_h_npcused = 0      !BUFR sequence does not currently have PCs
        cris1c_h_endorbit     = FNSCALE(VALUES(I+24),1.0D0)
        cris1c_h_startw(1)    = FNSCALE(VALUES(I+34),1.0D0) !m^-1
        cris1c_h_endw(1)      = FNSCALE(VALUES(I+35),1.0D0)
        cris1c_h_startchan(1) = FNSCALE(VALUES(I+36),1.0D0)
        cris1c_h_endchan(1)   = FNSCALE(VALUES(I+37),1.0D0)
        cris1c_h_startw(2)    = FNSCALE(VALUES(I+41),1.0D0)
        cris1c_h_endw(2)      = FNSCALE(VALUES(I+42),1.0D0)
        cris1c_h_startchan(2) = FNSCALE(VALUES(I+43),1.0D0)
        cris1c_h_endchan(2)   = FNSCALE(VALUES(I+44),1.0D0)
        cris1c_h_startw(3)    = FNSCALE(VALUES(I+48),1.0D0)
        cris1c_h_endw(3)      = FNSCALE(VALUES(I+49),1.0D0)
        cris1c_h_startchan(3) = FNSCALE(VALUES(I+50),1.0D0)
        cris1c_h_endchan(3)   = FNSCALE(VALUES(I+51),1.0D0)
        DO J=1,nchanused
          cris1c_h_channels(J) = FNSCALE(VALUES(I+56+J*2),1.0D0)
        ENDDO
        cris1c_h_enddatayr = cris1c_scnlinyr
        cris1c_h_enddatady = cris1c_scnlindy
        cris1c_h_enddatatime = cris1c_scnlintime(cris_1cnfor)
        if ((cris1c_scnlintime(1)-cris1c_scnlintime(cris_1cnfor))
     &      .gt.86392000) then
          cris1c_h_enddatady = cris1c_h_enddatady + 1
          if ((cris1c_h_enddatady .gt. 366) .or. 
     &         (MOD(cris1c_h_enddatayr,4) .ne. 0 .and. 
     &         cris1c_h_enddatady .gt. 365)) then
            cris1c_h_enddatady = 1
            cris1c_h_enddatayr = cris1c_h_enddatayr + 1
          endif
        endif
        cris1c_h_scnlin = NSCAN
        cris1c_h_misscnlin = 0
        IF (VALUES(I+2).GT.0.AND.VALUES(I+2).LT.999) THEN
          WRITE (cris1c_h_site,'(I3.3)') INT(VALUES(I+2))
        ELSE
          cris1c_h_site = "xxx"
        ENDIF
        cris1c_h_cfill1 = " "
        cris1c_h_1bsite = cris1c_h_site
        cris1c_h_cfill2 = " "
        cris1c_h_versnb = cris_1cvnnb
        cris1c_h_versyr = cris_1cvnyr
        cris1c_h_versdy = cris_1cvndy
        cris1c_h_hdrcnt = 1
        cris1c_h_wmosatid = FNSCALE(VALUES(I+1),1.0D0)
        IF (cris1c_h_wmosatid.EQ.Imiss) THEN
          cris1c_h_satid = Imiss
        ELSE
          cris1c_h_satid = 225-cris1c_h_wmosatid               !1 for NPP
        ENDIF
        cris1c_h_instrument = FNSCALE(VALUES(I+3),1.0D0)       !620 for CrIS
        cris1c_h_centre = CENTRE
        cris1c_h_subcentre = SUBCENTRE

! Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.cris1c_h_wmosatid) THEN
            cris1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            cris1c_h_period = OrbitPd(J)   !period, seconds
            cris1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO
      
      ENDIF   !IOB.EQ.NOBS

      RETURN
      END

