!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C_IASI(NSCAN,I,IOB,NOBS,VALUES,
     &   ENHANCED_IASI,Ifirst,Ilast,KRET)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c_iasi.F 545 2017-08-21 14:15:44Z frna $
!
! Purpose: Extract IASI-specific values from BUFR decode array 
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    16/07/2010  From aapp_get_1c.F
! 1.1    01/08/2017  Make enddatatime the time of the end of the last scan.
!                    Missing values for GEUMAvhrr1BQual, GEUMAvhrr1BCldFrac and
!                    GEUMAvhrr1BLandFrac set to -1, 255, 255 for consistency
!                    with original l1c datasets.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "iasi1c.h"
#include "bits.h"

! Arguments:

      INTEGER NSCAN     !Number of scans processed
      INTEGER I         !Index to values
      INTEGER IOB       !Observation index
      INTEGER NOBS      !Number of observations
      REAL*8 VALUES(*)  !Data values
      LOGICAL ENHANCED_IASI 
      INTEGER Ifirst,Ilast
      INTEGER KRET      !0 if OK, 1 to stop

! Parameters:

      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
! Local variables:

      INTEGER   I1cunit   !Unit number
      DATA I1cunit/13/
      
      INTEGER FOVOffset
      DATA FOVOffset/0/
      INTEGER FOV       !Numbered from 1
      REAL*8 Spot       !Spot number (real)

      INTEGER J         !Loop index
      INTEGER K         !Index

      INTEGER DET       !Detector number (IASI)
      INTEGER CLASS     !AVHRR class (IASI)
      INTEGER IBAND     !IASI band
      INTEGER ICHAN     !Channel index
      INTEGER ChOffset  !Channel number offset
      INTEGER SCANPOS   !Scan position (IASI)
      INTEGER TDIFF     !Time since start of file, ms (IASI)
      DATA TDIFF/0/
      INTEGER Scan0     !Scan number at start of file (IASI)
      DATA Scan0/0/
      INTEGER Time0     !Time at start of file (IASI)
      DATA Time0/0/
      INTEGER NSKIP     !Number of scans to skip before writing
      DATA NSKIP/0/

      INTEGER factor,itmp
      INTEGER loc_scalefactor(10)   !For use only with NOAA simulated IASI data
      INTEGER ref_scalefactor(10)
      DATA ref_scalefactor/7,7,7,8,8,9,9,9,9,9/
      INTEGER Offset                !For use with Enhanced IASI
      
      LOGICAL First
      DATA First/.TRUE./
      
! Functions

      INTEGER FNSCALE
      INTEGER DAYOFYEAR
      
!End of header
!------------------------------------------------------------------------------
      
!Check that the spot number is not missing; if it is, skip the ob
!Also, some encoding schemes number the FOV from 0, others from 1. Allow both.
      SPOT = VALUES(I+17)
      IF (SPOT .GT. Rmax) THEN
        WRITE(*,*)'Ob',IOB,' has missing FOV number'
        RETURN
      ENDIF
      IF (SPOT.EQ.0.0) FOVOffset=1      !FOVS numbered from 0
      FOV = SPOT + FOVOffset

!First time round, open the output file
      IF (First) THEN
        First=.FALSE.
        WRITE(*,*)'Creating 1c output dataset for IASI'
        CALL IOI1C("OPEN/WRITE",I1cunit,0)
        iasi1c_h_satht = 8330
        iasi1c_h_period = 6081
      ENDIF


      IF (FOV.EQ.1.OR.iasi1c_scnlinyr.EQ.Imiss) THEN   !First good spot
        iasi1c_granule = FNSCALE(VALUES(I+20),1.0D0)
        IF (VALUES(I+5).EQ.0) THEN
          IF (iasi1c_scnlinyr.EQ.0) THEN
            WRITE(*,*)'Please enter the year for IASI'
            READ(*,*) iasi1c_scnlinyr
          ENDIF
          VALUES(I+5) = iasi1c_scnlinyr
        ELSE 
          iasi1c_scnlinyr = FNSCALE(VALUES(I+5),1.0D0)
        ENDIF 
        IF (VALUES(I+5).GT.Rmax.OR.VALUES(I+6).GT.Rmax.OR.
     &      VALUES(I+7).GT.Rmax) THEN
          iasi1c_scnlindy = Imiss
        ELSE
          iasi1c_scnlindy = DAYOFYEAR(VALUES(I+5),VALUES(I+6),
     &                 VALUES(I+7))   
        ENDIF
        IF (ENHANCED_IASI) THEN
          Offset = 8
        ELSE
          Offset = 0
        ENDIF
        iasi1c_GQisQualIndex      = VALUES(I+23+Offset) !- 033061
        iasi1c_GQisQualIndexLoc   = VALUES(I+24+Offset) !- 033062
        iasi1c_GQisQualIndexRad   = VALUES(I+25+Offset) !- 033063
        iasi1c_GQisQualIndexSpect = VALUES(I+26+Offset) !- 033064
        iasi1c_GQisSysTecSondQual = VALUES(I+27+Offset) !- 033065
        iasi1c_scalti = FNSCALE(VALUES(I+21),1.0D-2)      !km*10
      ENDIF      !FOV.EQ.1
      
!Fill the parts that change with each pixel

      DET = MOD(FOV-1,4) + 1        !IASI Detector number (1 to 4)
      SCANPOS = (FOV-1)/4 + 1       !Scan position (1 to 30)
      
!If time is missing, initialise scnlintime(SCANPOS) using previous scan
      IF (VALUES(I+8) .GT. Rmax) THEN
        IF (DET .EQ. 1) THEN
          IF (NSCAN .NE. 0) THEN
            iasi1c_scnlintime(SCANPOS) = 
     &        MOD(iasi1c_scnlintime(SCANPOS) + 8000,86400000)
          ELSE
            iasi1c_scnlintime(SCANPOS) = Imiss !First scan - set missing
          ENDIF
        ENDIF
      ELSE
        iasi1c_scnlintime(SCANPOS)=FNSCALE(VALUES(I+8),3600.0D3)  !msec
        iasi1c_scnlintime(SCANPOS)=iasi1c_scnlintime(SCANPOS) + 
     &             FNSCALE(VALUES(I+9),60.0D3)   !Add minutes
        iasi1c_scnlintime(SCANPOS)=iasi1c_scnlintime(SCANPOS) + 
     &             FNSCALE(VALUES(I+10),1.0D3)   !Add seconds
      ENDIF

! Compute scan line number, allowing for missing scans. 8 seconds between scans.
! If scans are not numbered in the BUFR, take scan 1 as first requested record.

      IF (FOV.EQ.4.AND.(NSCAN+1).GE.Ifirst) THEN
        IF ((NSCAN+1).EQ.Ifirst) THEN
          IF (VALUES(I+19).LT.Rmax) THEN
            iasi1c_scnlin = VALUES(I+19)  !Scan line number
          ELSE
            iasi1c_scnlin = 1    !Scan line may be missing from BUFR
          ENDIF
          Scan0 = iasi1c_scnlin  !Time0 set later
        ELSE
          TDIFF = iasi1c_scnlintime(1) - Time0
          IF (TDIFF.LT.0) TDIFF = TDIFF + 86400000
          iasi1c_scnlin = NINT(TDIFF/8000.0) + Scan0
        ENDIF
      ENDIF
      
      IF (ENHANCED_IASI) THEN
        Offset = 9
        IF (VALUES(I+24).LT.Rmax) THEN    !Check first flag not missing
          iasi1c_GQisFlagQual(DET,SCANPOS) = VALUES(I+24)*2
     &      + VALUES(I+27)*4 + VALUES(I+30)*8         !- 033060
          IF (iasi1c_GQisFlagQual(DET,SCANPOS) .ne. 0) THEN
            iasi1c_GQisFlagQual(DET,SCANPOS) = 
     &        iasi1c_GQisFlagQual(DET,SCANPOS) + 1   !old flag
          ENDIF
          IF (VALUES(I+36).LT.Rmax) THEN
            iasi1c_GQisFlagQual(DET,SCANPOS) = 
     &        iasi1c_GQisFlagQual(DET,SCANPOS) + VALUES(I+36)*16 !- 040020
          ENDIF
        ENDIF
      ELSE
        Offset = 0
        iasi1c_GQisFlagQual(DET,SCANPOS) = VALUES(I+22)        !- 033060
      ENDIF       

      DO J=1,2
        iasi1c_latlon(J,DET,SCANPOS) = 
     &           FNSCALE(VALUES(I+10+J),1.0D4)      !10^4 x latlon
      ENDDO
      DO J=1,3,2      !Zenith angles
        iasi1c_angles(J,DET,SCANPOS) = 
     &           FNSCALE(VALUES(I+12+J),1.0D2)      !10^2 x angle
      ENDDO

      DO J=2,4,2      !Azimuth angles (signed in AAPP, +ve in BUFR)
        iasi1c_angles(J,DET,SCANPOS) = 
     &      FNSCALE(VALUES(I+12+J),1.0D2) !10^2 x angle
        IF (iasi1c_angles(J,DET,SCANPOS).NE.Imiss .AND.
     &      iasi1c_angles(J,DET,SCANPOS).GT.18000) THEN
          iasi1c_angles(J,DET,SCANPOS) =
     &            iasi1c_angles(J,DET,SCANPOS) - 36000 !-180 to 180
        ENDIF
      ENDDO

      DO J=1,8700        !Chan 1=645cm^-1, chan 8461=2760cm^-1
        IF (VALUES(I+2*J+57+Offset).LT.Rmax) THEN
          itmp = VALUES(I+2*J+57+Offset)    !- 014192
          if (itmp.GT.32768) itmp=itmp-65536
          iasi1c_scalrad(J,DET,SCANPOS) = itmp
        ELSE
          iasi1c_scalrad(J,DET,SCANPOS) = 0  !Set to zero if missing
                             !Radiance was probably -ve
        ENDIF
      ENDDO
      
      J = 17458+Offset

!Mapped AVHRR
      iasi1c_avhcombination = VALUES(I+J+1)           !- 025051
      J = J + 2
      
      DO CLASS=1,7
        iasi1c_avhrrCGy(CLASS,DET,SCANPOS) = 
     &       FNSCALE(VALUES(I+J),1.0D6)       !- 005060
        iasi1c_avhrrCGz(CLASS,DET,SCANPOS) = 
     &       FNSCALE(VALUES(I+J+1),1.0D6)             !- 005061
        iasi1c_avhrrfrac(CLASS,DET,SCANPOS) = 
     &       FNSCALE(VALUES(I+J+2),1.0D4)             !- 025085
                   !Convert percentage to 10^6 x fraction

        DO ICHAN=1,6
          K = J + 3 + (ICHAN-1)*5
          iasi1c_avhmeanscalefactor(ICHAN,CLASS,DET,SCANPOS)
     &             = FNSCALE(VALUES(I+K+1),1.0D0)     !- 025142
          iasi1c_avhrrmean(ICHAN,CLASS,DET,SCANPOS)
     &             = FNSCALE(VALUES(I+K+2),1.0D0)     !- 014047
          iasi1c_avhsdevscalefactor(ICHAN,CLASS,DET,SCANPOS)
     &             = FNSCALE(VALUES(I+K+3),1.0D0)     !- 025142
          iasi1c_avhrrsdev(ICHAN,CLASS,DET,SCANPOS)
     &             = FNSCALE(VALUES(I+K+4),1.0D0)     !- 014048
        ENDDO
        
        J = J + 33
      ENDDO  !- CLASS

!IIS radiance and cloud analysis

      IF (ENHANCED_IASI) THEN
        IF (VALUES(I+J) .LT. Rmax) THEN
          iasi1c_GEUMAvhrr1BCldFrac(DET,SCANPOS) = VALUES(I+J)  !- 020081
        ELSE
          iasi1c_GEUMAvhrr1BCldFrac(DET,SCANPOS) = 255
        ENDIF
        IF (VALUES(I+J+2) .LT. Rmax) THEN
          iasi1c_GEUMAvhrr1BLandFrac(DET,SCANPOS) = VALUES(I+J+2)  !- 020083
        ELSE
          iasi1c_GEUMAvhrr1BLandFrac(DET,SCANPOS) = 255
        ENDIF
        iasi1c_GIacAvgImagIIS(SCANPOS) = 
     &           FNSCALE(VALUES(I+J+4),1.0D6)         !- 040018
        iasi1c_GIacVarImagIIS(SCANPOS) = 
     &           FNSCALE(VALUES(I+J+5),1.0D6)         !- 040019
        IF (VALUES(I+J+6).LT.Rmax) THEN
          iasi1c_GEUMAvhrr1BQual(DET,SCANPOS) = 
     &           VALUES(I+J+6)                !- 040021
        ELSE
          IF (VALUES(I+J+7).LT.Rmax) THEN
            iasi1c_GEUMAvhrr1BQual(DET,SCANPOS) = 
     &           VALUES(I+J+7) + 128          !- 040022
          ELSE
            iasi1c_GEUMAvhrr1BQual(DET,SCANPOS) = -1   !set to -1 if missing
          ENDIF 
        ENDIF             
      ELSE
        iasi1c_GIacVarImagIIS(SCANPOS) = IMISS
        iasi1c_GIacAvgImagIIS(SCANPOS) = IMISS
        iasi1c_GEUMAvhrr1BCldFrac(DET,SCANPOS) = IMISS
        iasi1c_GEUMAvhrr1BLandFrac(DET,SCANPOS) = IMISS
        iasi1c_GEUMAvhrr1BQual(DET,SCANPOS) = IMISS
      ENDIF

! Scaling factors for IASI radiances.
! In EUMETSAT test data the channels are numbered from 1 at DC, so the first
! point is 645cm^-1 = 2581. However in NOAA test data the channels are numbered
! from 1 at 645cm^-1. Allow for both cases, defining channel 1 at 645cm^-1.

      IF (FOV.EQ.1) THEN
        DO J=1,10
          iasi1c_h_startchan(J)   = 
     &         FNSCALE(VALUES(I+28+Offset+(J-1)*3),1.0D0)         !- 025140
          if (J.EQ.1) ChOffset = iasi1c_h_startchan(J)-1
          if (iasi1c_h_startchan(J).GT.0)
     &       iasi1c_h_startchan(J) = iasi1c_h_startchan(J)
     &                        - ChOffset
          iasi1c_h_endchan(J)     =  
     &         FNSCALE(VALUES(I+29+Offset+(J-1)*3),1.0D0)         !- 025141
          if (iasi1c_h_endchan(J).GT.0)
     &       iasi1c_h_endchan(J) = iasi1c_h_endchan(J)
     &                        - ChOffset
          iasi1c_h_scalefactor(J) =  
     &         FNSCALE(VALUES(I+30+Offset+(J-1)*3),1.0D0)         !- 025142
        ENDDO
      ENDIF

!For NOAA simulated data the radiance scaling factors can change from ob to ob,
!which does not fit in with the AAPP 1c format. So fix the scaling factors.

      IF (VALUES(I+1).GE.993) THEN    !NOAA METOP simulation
        DO J=1,10
          loc_scalefactor(J) =  
     &         FNSCALE(VALUES(I+30+(J-1)*3),1.0D0)         !- 025142
          iasi1c_h_scalefactor(J) = ref_scalefactor(J)
          IF (loc_scalefactor(J).GT.ref_scalefactor(J)) THEN
            factor = 10**(loc_scalefactor(J)-ref_scalefactor(J))
            DO k=iasi1c_h_startchan(J),iasi1c_h_endchan(J)
              itmp = iasi1c_scalrad(K,DET,SCANPOS)
              if (itmp.LT.0) itmp = itmp + 65536
              itmp = itmp/factor
              if (itmp.GT.32768) itmp = itmp - 65536
              iasi1c_scalrad(K,DET,SCANPOS) = itmp
            ENDDO
          ELSE
            IF (loc_scalefactor(J).GT.0.AND.
     &         loc_scalefactor(J).LT.ref_scalefactor(J)) THEN
              WRITE(*,*)'Warning: scale factor',
     &               J,loc_scalefactor(J)
            ENDIF
          ENDIF
        ENDDO
      ENDIF          !End of NOAA correction
        
! Check for missing times in the first scan

      IF ((NSCAN+1).EQ.Ifirst.AND.FOV.EQ.120) THEN
        DO J=29,1,-1
          IF (iasi1c_scnlintime(J).EQ.Imiss.AND.
     &        iasi1c_scnlintime(J+1).NE.Imiss ) THEN
            iasi1c_scnlintime(J) = iasi1c_scnlintime(J+1) - 216
          ENDIF
        ENDDO
        DO J=2,30
          IF (iasi1c_scnlintime(J).EQ.Imiss.AND.
     &        iasi1c_scnlintime(J-1).NE.Imiss ) THEN
            iasi1c_scnlintime(J) = iasi1c_scnlintime(J+1) + 216
          ENDIF
        ENDDO
        iasi1c_h_startorbit = FNSCALE(VALUES(I+18),1.0D0)
        iasi1c_h_startdatayr = iasi1c_scnlinyr
        iasi1c_h_startdatady = iasi1c_scnlindy
        iasi1c_h_startdatatime = iasi1c_scnlintime(1)
        Time0 = iasi1c_scnlintime(1)
      ENDIF

! Write data record
 
      IF (FOV.EQ.120) THEN
        NSCAN = NSCAN+1
        IF (NSCAN.GE.Ifirst.AND.NSCAN.LE.Ilast) THEN
          WRITE(6,*) 'Writing to data record',NSCAN
          CALL IOI1C("WRITE DATA", I1cunit, NSCAN-NSKIP+1) !1 header record
        ELSE IF (NSCAN.LT.Ifirst) THEN
          NSKIP = NSKIP+1
        ENDIF
      ENDIF   !FOV.EQ.120

! Update main part of header common block after each scan

      IF (FOV.EQ.120.AND.NSCAN.GE.Ifirst.AND.NSCAN.LE.Ilast)
     &        THEN
        IF (NSCAN.EQ.Ilast) KRET=1     !No more scans required
        WRITE(*,*)'Updating header'
        iasi1c_h_endorbit = FNSCALE(VALUES(I+18),1.0D0)
        iasi1c_h_enddatayr = iasi1c_scnlinyr
        iasi1c_h_enddatady = iasi1c_scnlindy
        iasi1c_h_enddatatime = iasi1c_scnlintime(iasi_1cnfov)
        if ((iasi1c_scnlintime(1)-iasi1c_scnlintime(iasi_1cnfov))
     &      .gt.86392000) then
          iasi1c_h_enddatady = iasi1c_h_enddatady + 1
          if ((iasi1c_h_enddatady .gt. 366) .or. 
     &         (MOD(iasi1c_h_enddatayr,4) .ne. 0 .and. 
     &         iasi1c_h_enddatady .gt. 365)) then
            iasi1c_h_enddatady = 1
            iasi1c_h_enddatayr = iasi1c_h_enddatayr + 1
          endif
        endif
        iasi1c_h_scnlin = NSCAN-NSKIP
        iasi1c_h_misscnlin = 0
        iasi1c_h_site = "EUM"
        iasi1c_h_cfill1 = " "
        iasi1c_h_1bsite = "EUM"
        iasi1c_h_cfill2 = " "
        iasi1c_h_versnb = iasi_1cvnnb
        iasi1c_h_versyr = iasi_1cvnyr
        iasi1c_h_versdy = iasi_1cvndy
        iasi1c_h_hdrcnt = 1
        iasi1c_h_wmosatid = FNSCALE(VALUES(I+1),1.0D0)
        IF (iasi1c_h_wmosatid.GE.993) THEN         !NOAA METOP test data
          iasi1c_h_wmosatid = iasi1c_h_wmosatid - 990
          iasi1c_h_satid = iasi1c_h_wmosatid-2
        ELSE IF (iasi1c_h_wmosatid.GE.206) THEN
          iasi1c_h_satid = iasi1c_h_wmosatid-191   !NOAA satellites
        ELSE
          iasi1c_h_satid = iasi1c_h_wmosatid-2     !METOP
        ENDIF
        IF (iasi1c_h_wmosatid.EQ.Imiss) iasi1c_h_satid = Imiss
        iasi1c_h_instrument = 13       !IASI
! Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.iasi1c_h_wmosatid) THEN
            iasi1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            iasi1c_h_period = OrbitPd(J)   !period, seconds
            iasi1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO

      ENDIF   !IOB.EQ.NOBS

      RETURN
      END

