!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C_IRAS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &     VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c_iras.F 466 2015-10-21 14:26:02Z frna $
!
! Purpose: Extract IRAS-specific values from BUFR decode array 
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    15/02/2013  Create from aapp_get_1c_mwhs.F
! 1.1    30/07/2014  Use 008012 instead of 013040 for surface type.
! 1.2    21/10/2015  Correct iras1c_h_instrument 16 -> 17
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "iras1c.h"
#include "bits.h"
#include "constants_preproc.h"

! Arguments:

      INTEGER NSCAN     !Number of scans processed
      INTEGER I         !Index to values
      INTEGER IOB       !Observation index
      INTEGER NOBS      !Number of observations
      INTEGER CENTRE    !From section 1
      INTEGER SUBCENTRE !From section 1
      REAL*8 VALUES(*)  !Data values
      
! Parameters:

      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
! Local variables:

      INTEGER   IRAS1cunit   !Unit number
      DATA IRAS1cunit/21/
      
      INTEGER FOVOffset
      DATA FOVOffset/0/
      INTEGER FOV       !Field of view, numbered from 1
      REAL*8 Spot       !Spot number (real)

      INTEGER J         !Loop index
      INTEGER K         !Index
      INTEGER nchanused
      INTEGER chan

      LOGICAL First
      DATA First/.TRUE./
      
! Functions

      INTEGER FNSCALE
      INTEGER DAYOFYEAR
      
!End of header
!------------------------------------------------------------------------------
      
!Check that the spot number is not missing; if it is, skip the ob
!Also, some encoding schemes number the FOV from 0, others from 1. Allow both.
      SPOT = VALUES(I+7)
      IF (SPOT .GT. Rmax) THEN
        WRITE(*,*)'Ob',IOB,' has missing FOV number'
        RETURN
      ENDIF
      IF (SPOT.EQ.0.0) FOVOffset=1      !FOVS numbered from 0
      FOV = SPOT + FOVOffset

!First time round, open the output file
      IF (First) THEN
        First=.FALSE.
        WRITE(*,*)'Creating 1c output dataset for IRAS'
        iras1c_h_satht = 8350    !nominal height (km*10)
        iras1c_h_period = 6120   !nominal period (minutes)
        CALL IOIRAS1C("OPEN/WRITE",IRAS1cunit,0)
      ENDIF

      IF (FOV.EQ.1) THEN  !First spot
        iras1c_scnlinyr = FNSCALE(VALUES(I+8),1.0D0)
        IF (VALUES(I+8).GT.Rmax.OR.VALUES(I+9).GT.Rmax.OR.
     &              VALUES(I+10).GT.Rmax) THEN
          iras1c_scnlindy = Imiss
        ELSE
          iras1c_scnlindy = DAYOFYEAR(VALUES(I+8),VALUES(I+9),
     &                 VALUES(I+10))   
        ENDIF

        iras1c_scnlintime = FNSCALE(VALUES(I+11),3600.0D3)  !millisecs
        IF (iras1c_scnlintime.NE.Imiss) THEN
          iras1c_scnlintime = iras1c_scnlintime + 
     &           FNSCALE(VALUES(I+12),60.0D3)  !Add minutes
          iras1c_scnlintime = iras1c_scnlintime + 
     &           FNSCALE(VALUES(I+13),1.0D3)   !Add seconds
        ENDIF

        iras1c_scnlin = FNSCALE(VALUES(I+6),1.0D0)
        iras1c_scalti = Imiss

        DO J=1,iras_nchan
          chan = FNSCALE(VALUES(I+19+J*4),1.0D0)
          IF (chan.NE.Imiss) THEN
            iras1c_h_tempradcnv(1,chan) = 
     &        NINT(VALUES(I+20+J*4)/C_light*1.0D2)      !10^6*wavenumber cm^-1
            iras1c_h_tempradcnv(2,chan) = 0             !10^6*offset in K
            iras1c_h_tempradcnv(3,chan) = 1000000       !10^6*slope
          ENDIF
        ENDDO

!Fill part of header

        IF (NSCAN.EQ.0) THEN
          iras1c_h_startorbit = FNSCALE(VALUES(I+5),1.0D0)
          iras1c_h_startdatayr = iras1c_scnlinyr
          iras1c_h_startdatady = iras1c_scnlindy
          iras1c_h_startdatatime = iras1c_scnlintime
        ENDIF

      ENDIF      !FOV.EQ.1
              
!Fill the parts that change with each pixel

      DO J=1,2
        iras1c_latlon(J,FOV) = FNSCALE(VALUES(I+13+J),1.0D4) !10^4 x latlon
      ENDDO

      iras1c_surface(1,FOV) = FNSCALE(VALUES(I+16),1.0D0)  !Surface Height in m
      IF (VALUES(I+17) .EQ. 1) THEN
        iras1c_surface(2,FOV) = 0     !Sea
      ELSE IF (VALUES(I+17) .EQ. 2) THEN
        iras1c_surface(2,FOV) = 1     !Mixed
      ELSE IF (VALUES(I+17) .EQ. 0) THEN
        iras1c_surface(2,FOV) = 2     !Land
      ELSE
        iras1c_surface(2,FOV) = Imiss
      ENDIF

      DO J=1,3,2      !Zenith angles
        iras1c_angles(J,FOV) = FNSCALE(VALUES(I+17+J),1.0D2) !10^2 x angle
      ENDDO

      DO J=2,4,2      !Azimuth angles (signed in AAPP, +ve in BUFR)
        iras1c_angles(J,FOV) = FNSCALE(VALUES(I+17+J),1.0D2) !10^2 x angle
        IF (iras1c_angles(J,FOV).NE.Imiss .AND.
     &              iras1c_angles(J,FOV).GT.18000) THEN
          iras1c_angles(J,FOV) = iras1c_angles(J,FOV) - 36000 !-180 to 180
        ENDIF
      ENDDO

      DO J=1,iras_nchan
        chan = FNSCALE(VALUES(I+19+J*4),1.0D0)
        IF (chan.NE.Imiss) THEN
          iras1c_btemps(chan,FOV) = FNSCALE(VALUES(I+22+J*4),1.0D2)
        ENDIF
      ENDDO
             
! Write data record
 
      IF (FOV.EQ.iras_nfovs .AND. iras1c_scnlin.GT.0) THEN
        NSCAN = NSCAN+1
        WRITE(6,*) 'Writing to data record',NSCAN
        CALL IOIRAS1C("WRITE DATA", IRAS1cunit, NSCAN+1) !1 header record
      ENDIF

! Update header common block at conclusion of each message

      IF (IOB.EQ.NOBS) THEN
        iras1c_h_endorbit     = FNSCALE(VALUES(I+5),1.0D0)
        iras1c_h_enddatayr = iras1c_scnlinyr
        iras1c_h_enddatady = iras1c_scnlindy
        iras1c_h_enddatatime = iras1c_scnlintime
        iras1c_h_scnlin = NSCAN
        iras1c_h_misscnlin = 0
        IF (VALUES(I+1).GT.0.AND.VALUES(I+1).LT.999) THEN
          WRITE (iras1c_h_site,'(I3.3)') INT(VALUES(I+1))
        ELSE
          iras1c_h_site = "xxx"
        ENDIF
        iras1c_h_cfill1 = " "
        IF (VALUES(I+2).GT.0.AND.VALUES(I+2).LT.999) THEN
          WRITE (iras1c_h_1bsite,'(I3.3)') INT(VALUES(I+2))
        ELSE
          iras1c_h_1bsite = "xxx"
        ENDIF
        iras1c_h_cfill2 = " "
        iras1c_h_versnb = iras_1cvnnb
        iras1c_h_versyr = iras_1cvnyr
        iras1c_h_versdy = iras_1cvndy
        iras1c_h_hdrcnt = 1
        iras1c_h_wmosatid = FNSCALE(VALUES(I+3),1.0D0)
        IF (iras1c_h_wmosatid.EQ.Imiss) THEN
          iras1c_h_satid = Imiss
        ELSE
          iras1c_h_satid = iras1c_h_wmosatid-519               !1 for FY-3A
        ENDIF
        iras1c_h_instrument = 17   !see iras1c.h
     
      ENDIF   !IOB.EQ.NOBS

      RETURN
      END

