!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C_MSU(NSCAN,I,IOB,NOBS,VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c_msu.F 313 2012-02-28 14:51:43Z frna $
!
! Purpose: Extract MSU-specific values from BUFR decode array 
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    16/07/2010  From aapp_get_1c.F
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "msu1c.h"
#include "bits.h"

! Arguments:

      INTEGER NSCAN     !Number of scans processed
      INTEGER I         !Index to values
      INTEGER IOB       !Observation index
      INTEGER NOBS      !Number of observations
      REAL*8 VALUES(*)  !Data values
      
! Parameters:

      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:

      INTEGER   M1cunit   !Unit number
      DATA M1cunit/14/
      
      INTEGER FOVOffset
      DATA FOVOffset/0/
      INTEGER FOV       !Numbered from 1
      REAL*8 Spot       !Spot number (real)

      INTEGER J         !Loop index
      INTEGER K         !Index
      INTEGER BIT       !For use with unbitswap

      INTEGER    Mask033030(NumBitsIn) !- bit mask for 033030
      INTEGER    Mask033031(NumBitsIn) !- bit mask for 033031
      INTEGER    Mask033032(NumBitsIn) !- bit mask for 033032
      INTEGER    Mask033033(NumBitsIn) !- bit mask for 033033

      DATA Mask033030 /1,2,3,4,5,6,7,25*0/
      DATA Mask033031 /8*0,1,2,3,4,4*0,5,6,7,8,9,10,11,12,13,14,15,16,
     &                 17,3*0/
      DATA Mask033032 /26*0,1,2,3,4,5,6/
      DATA Mask033033 /0,1,9*0,21,20,19,18,17,16,15,14,13,12,11,10,9,
     &                 8,7,6,5,4,3,2,22/

      INTEGER msu1c_h_wmosatid   !This is not in the 1c format

      LOGICAL First
      DATA First/.TRUE./
      
! Functions

      INTEGER FNSCALE
      INTEGER DAYOFYEAR
      
!End of header
!------------------------------------------------------------------------------
      
!Check that the spot number is not missing; if it is, skip the ob
!Also, some encoding schemes number the FOV from 0, others from 1. Allow both.
      SPOT = VALUES(I+12)
      IF (SPOT .GT. Rmax) THEN
        WRITE(*,*)'Ob',IOB,' has missing FOV number'
        RETURN
      ENDIF
      IF (SPOT.EQ.0.0) FOVOffset=1      !FOVS numbered from 0
      FOV = SPOT + FOVOffset

!First time round, open the output file
      IF (First) THEN
        First=.FALSE.
        WRITE(*,*)'Creating 1c output dataset for MSU'
        CALL IOM1C("OPEN/WRITE",M1cunit,0)
        msu1c_h_satht = 8330
        msu1c_h_period = 6081
      ENDIF
 

      IF (FOV.EQ.1) THEN          !First spot
        msu1c_scnlin = FNSCALE(VALUES(I+11),1.0D0)
        msu1c_scnlinyr = FNSCALE(VALUES(I+16),1.0D0)
        IF (VALUES(I+16).GT.Rmax.OR.VALUES(I+17).GT.Rmax.OR.
     &      VALUES(I+18).GT.Rmax) THEN
          msu1c_scnlindy = Imiss
        ELSE
          msu1c_scnlindy = DAYOFYEAR(VALUES(I+16),VALUES(I+17),
     &                 VALUES(I+18))   
        ENDIF
        msu1c_scnlintime = FNSCALE(VALUES(I+19),3600.0D3)  !millisecs
        msu1c_scnlintime = msu1c_scnlintime + 
     &             FNSCALE(VALUES(I+20),60.0D3)   !Add minutes
        msu1c_scnlintime = msu1c_scnlintime + 
     &             FNSCALE(VALUES(I+21),1.0D3)    !Add seconds

        IF (VALUES(I+14).LT.Rmax) THEN
          Bit = VALUES(I+14)
          CALL UNBITSWAP(msu1c_qualind, Mask033030, NumBitsIn, 
     &           Bit, NumBitsOut)
        ELSE
          msu1c_qualind = 0     !Don't use Imiss
        ENDIF
        IF (VALUES(I+15).LT.Rmax) THEN
          Bit = VALUES(I+15)
          CALL UNBITSWAP(msu1c_scanqual, Mask033031, NumBitsIn, 
     &           Bit, NumBitsOut)
        ELSE
          msu1c_scanqual = 0     !Don't use Imiss
        ENDIF
        DO J=1,4
          IF (VALUES(I+42+(J-1)*6).LT.Rmax) THEN
            Bit = VALUES(I+42+(J-1)*6)
            CALL UNBITSWAP(msu1c_chanqual(J), Mask033032, 
     &           NumBitsIn, Bit, NumBitsOut)
          ELSE
            msu1c_chanqual(J) = 0     !Don't use Imiss
          ENDIF
        ENDDO
        
        msu1c_scalti = FNSCALE(VALUES(I+24),1.0D-2)      !km*10

!Fill part of header

        IF (NSCAN.EQ.0) THEN
          msu1c_h_startorbit = FNSCALE(VALUES(I+9),1.0D0)
          msu1c_h_startdatayr = msu1c_scnlinyr
          msu1c_h_startdatady = msu1c_scnlindy
          msu1c_h_startdatatime = msu1c_scnlintime
        ENDIF

      ENDIF      !FOV.EQ.1
      
!Fill the parts that change with each pixel

      DO J=1,2
        msu1c_latlon(J,FOV) = FNSCALE(VALUES(I+21+J),1.0D4) !10^4 x latlon
      ENDDO

      DO J=1,1        !Zenith angles (only the local zenith for MSU)
        msu1c_angles(J,FOV) = FNSCALE(VALUES(I+24+J),1.0D2) !10^2 x angle
      ENDDO

      DO J=2,2        !Azimuth angles (signed in AAPP, +ve in BUFR)
        msu1c_angles(J,FOV) = FNSCALE(VALUES(I+24+J),1.0D2) !10^2 x angle
        IF (msu1c_angles(J,FOV).NE.Imiss .AND.
     &      msu1c_angles(J,FOV).GT.18000) THEN
          msu1c_angles(J,FOV) = msu1c_angles(J,FOV) - 36000 !-180 to 180
        ENDIF
      ENDDO

      DO J=1,4
        msu1c_btemps(J,FOV) = 
     &            FNSCALE(VALUES(I+43+(J-1)*6),1.0D2) !10^2 x BT
      ENDDO

      IF (VALUES(I+29).LT.Rmax) THEN
        Bit = VALUES(I+29)
        CALL UNBITSWAP(msu1c_dataqual(FOV), Mask033033, 
     &           NumBitsIn, Bit, NumBitsOut)
      ELSE
        msu1c_dataqual(FOV) = 0  !If flag is missing, create it
        DO J=1,4
          IF (msu1c_btemps(J,FOV) .LE. MminBT*100 .OR.
     &        msu1c_btemps(J,FOV) .GE. MmaxBT*100 ) THEN
            msu1c_dataqual(FOV) = 
     &           IBSET(msu1c_dataqual(FOV),bits(J))
          ENDIF
        ENDDO 
      ENDIF
      
! Write data record
 
      IF (FOV.EQ.11 .AND. msu1c_scnlin.GT.0) THEN
        NSCAN = NSCAN+1
        WRITE(6,*) 'Writing to data record',NSCAN
        CALL IOM1C("WRITE DATA", M1cunit, NSCAN+1) !1 header record
      ENDIF   !FOV.EQ.11

! Update header common block at conclusion of each message

      IF (IOB.EQ.NOBS) THEN
        msu1c_h_endorbit = FNSCALE(VALUES(I+9),1.0D0)
        msu1c_h_enddatayr = msu1c_scnlinyr
        msu1c_h_enddatady = msu1c_scnlindy
        msu1c_h_enddatatime = msu1c_scnlintime
        msu1c_h_scnlin = NSCAN
        msu1c_h_misscnlin = 0
        DO J=1,4
          IF (VALUES(I+39+(J-1)*6) .LT. Rmax) THEN
            msu1c_h_tempradcnv(1,J) = 
     &       NINT(10**(VALUES(I+39+(J-1)*6))*1E4)    !10^6 x wavenumber
          ELSE
            msu1c_h_tempradcnv(1,J) = Imiss
          ENDIF
          msu1c_h_tempradcnv(2,J) = 
     &            FNSCALE(VALUES(I+40+(J-1)*6),1.0D6) !10^6 x CONST 1
          msu1c_h_tempradcnv(3,J) = 
     &            FNSCALE(VALUES(I+41+(J-1)*6),1.0D6) !10^6 x CONST 2
        ENDDO
        msu1c_h_site = "EUM"
        msu1c_h_cfill1 = " "
        msu1c_h_1bsite = "EUM"
        msu1c_h_cfill2 = " "
        msu1c_h_versnb = msu_1cvnnb
        msu1c_h_versyr = msu_1cvnyr
        msu1c_h_versdy = msu_1cvndy
        msu1c_h_hdrcnt = 1
        msu1c_h_wmosatid = FNSCALE(VALUES(I+7),1.0D0)
        msu1c_h_satid = Imiss
        msu1c_h_instrument = 6
! Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.msu1c_h_wmosatid) THEN
            msu1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            msu1c_h_period = OrbitPd(J)   !period, seconds
            msu1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO
        
      ENDIF   !IOB.EQ.NOBS

      RETURN
      END

