!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C_MWHS(NSCAN,I,IOB,NOBS,CENTRE,SUBCENTRE,
     &     VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c_mwhs.F 407 2014-07-30 14:15:48Z frna $
!
! Purpose: Extract MWHS-specific values from BUFR decode array 
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 1 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2012, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    23/08/2012  New
! 1.1    30/07/2014  Use 008012 instead of 013040 for surface type.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "mwhs1c.h"
#include "bits.h"
#include "constants_preproc.h"
#include "mwhschandata.h"

! Arguments:

      INTEGER NSCAN     !Number of scans processed
      INTEGER I         !Index to values
      INTEGER IOB       !Observation index
      INTEGER NOBS      !Number of observations
      INTEGER CENTRE    !From section 1
      INTEGER SUBCENTRE !From section 1
      REAL*8 VALUES(*)  !Data values
      
! Parameters:

      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
! Local variables:

      INTEGER   MWHS1cunit   !Unit number
      DATA MWHS1cunit/19/
      
      INTEGER FOVOffset
      DATA FOVOffset/0/
      INTEGER FOV       !Field of view, numbered from 1
      REAL*8 Spot       !Spot number (real)

      INTEGER J         !Loop index
      INTEGER K         !Index
      INTEGER nchanused
      INTEGER chan

      LOGICAL First
      DATA First/.TRUE./
      
! Functions

      INTEGER FNSCALE
      INTEGER DAYOFYEAR
      
!End of header
!------------------------------------------------------------------------------
      
!Check that the spot number is not missing; if it is, skip the ob
!Also, some encoding schemes number the FOV from 0, others from 1. Allow both.
      SPOT = VALUES(I+7)
      IF (SPOT .GT. Rmax) THEN
        WRITE(*,*)'Ob',IOB,' has missing FOV number'
        RETURN
      ENDIF
      IF (SPOT.EQ.0.0) FOVOffset=1      !FOVS numbered from 0
      FOV = SPOT + FOVOffset

!First time round, open the output file
      IF (First) THEN
        First=.FALSE.
        WRITE(*,*)'Creating 1c output dataset for MWHS'
        mwhs1c_h_satht = 8350    !nominal height (km*10)
        mwhs1c_h_period = 6120   !nominal period (minutes)
        CALL IOMWHS1C("OPEN/WRITE",MWHS1cunit,0)
      ENDIF

      IF (FOV.EQ.1) THEN  !First spot
        mwhs1c_scnlinyr = FNSCALE(VALUES(I+8),1.0D0)
        IF (VALUES(I+8).GT.Rmax.OR.VALUES(I+9).GT.Rmax.OR.
     &              VALUES(I+10).GT.Rmax) THEN
          mwhs1c_scnlindy = Imiss
        ELSE
          mwhs1c_scnlindy = DAYOFYEAR(VALUES(I+8),VALUES(I+9),
     &                 VALUES(I+10))   
        ENDIF

        mwhs1c_scnlintime = FNSCALE(VALUES(I+11),3600.0D3)  !millisecs
        IF (mwhs1c_scnlintime.NE.Imiss) THEN
          mwhs1c_scnlintime = mwhs1c_scnlintime + 
     &           FNSCALE(VALUES(I+12),60.0D3)  !Add minutes
          mwhs1c_scnlintime = mwhs1c_scnlintime + 
     &           FNSCALE(VALUES(I+13),1.0D3)   !Add seconds
        ENDIF

        mwhs1c_scnlin = FNSCALE(VALUES(I+6),1.0D0)
        mwhs1c_scalti = Imiss

        DO J=1,mwhs_nchan
          chan = FNSCALE(VALUES(I+19+J*4),1.0D0)
          IF (chan.NE.Imiss) THEN

!Use the nominal frequencies rather than the ones in the input BUFR file
!(may want to change this later)
            mwhs1c_h_tempradcnv(1,chan) = 
     &        NINT(mwhscentrefreq(chan)/C_light*1.0D13) !10^6*wavenumber cm^-1
            mwhs1c_h_tempradcnv(2,chan) = 
     &        NINT(mwhsbandcorr_offset(chan)*1.0D6)     !10^6*offset in K
            mwhs1c_h_tempradcnv(3,chan) = 
     &        NINT(mwhsbandcorr_slope(chan)*1.0D6)      !10^6*slope
            mwhs1c_h_bandwidth(chan) = 
     &        NINT(mwhsbandwidth(chan)*1E6)             !10^6*bandwidth in GHz
          ENDIF
        ENDDO

!Fill part of header

        IF (NSCAN.EQ.0) THEN
          mwhs1c_h_startorbit = FNSCALE(VALUES(I+5),1.0D0)
          mwhs1c_h_startdatayr = mwhs1c_scnlinyr
          mwhs1c_h_startdatady = mwhs1c_scnlindy
          mwhs1c_h_startdatatime = mwhs1c_scnlintime
        ENDIF

      ENDIF      !FOV.EQ.1
              
!Fill the parts that change with each pixel

      DO J=1,2
        mwhs1c_latlon(J,FOV) = FNSCALE(VALUES(I+13+J),1.0D4) !10^4 x latlon
      ENDDO

      mwhs1c_surface(1,FOV) = FNSCALE(VALUES(I+16),1.0D0)  !Surface Height in m
      IF (VALUES(I+17) .EQ. 1) THEN
        mwhs1c_surface(2,FOV) = 0     !Sea
      ELSE IF (VALUES(I+17) .EQ. 2) THEN
        mwhs1c_surface(2,FOV) = 1     !Mixed
      ELSE IF (VALUES(I+17) .EQ. 0) THEN
        mwhs1c_surface(2,FOV) = 2     !Land
      ELSE
        mwhs1c_surface(2,FOV) = Imiss
      ENDIF

      DO J=1,3,2      !Zenith angles
        mwhs1c_angles(J,FOV) = FNSCALE(VALUES(I+17+J),1.0D2) !10^2 x angle
      ENDDO

      DO J=2,4,2      !Azimuth angles (signed in AAPP, +ve in BUFR)
        mwhs1c_angles(J,FOV) = FNSCALE(VALUES(I+17+J),1.0D2) !10^2 x angle
        IF (mwhs1c_angles(J,FOV).NE.Imiss .AND.
     &              mwhs1c_angles(J,FOV).GT.18000) THEN
          mwhs1c_angles(J,FOV) = mwhs1c_angles(J,FOV) - 36000 !-180 to 180
        ENDIF
      ENDDO

      DO J=1,mwhs_nchan
        chan = FNSCALE(VALUES(I+19+J*4),1.0D0)
        IF (chan.NE.Imiss) THEN
          mwhs1c_btemps(chan,FOV) = FNSCALE(VALUES(I+22+J*4),1.0D2)
        ENDIF
      ENDDO
             
! Write data record
 
      IF (FOV.EQ.mwhs_nfovs .AND. mwhs1c_scnlin.GT.0) THEN
        NSCAN = NSCAN+1
        WRITE(6,*) 'Writing to data record',NSCAN
        CALL IOMWHS1C("WRITE DATA", MWHS1cunit, NSCAN+1) !1 header record
      ENDIF

! Update header common block at conclusion of each message

      IF (IOB.EQ.NOBS) THEN
        mwhs1c_h_endorbit     = FNSCALE(VALUES(I+5),1.0D0)
        mwhs1c_h_enddatayr = mwhs1c_scnlinyr
        mwhs1c_h_enddatady = mwhs1c_scnlindy
        mwhs1c_h_enddatatime = mwhs1c_scnlintime
        mwhs1c_h_scnlin = NSCAN
        mwhs1c_h_misscnlin = 0
        mwhs1c_h_vnantennacorr = Imiss
        IF (VALUES(I+1).GT.0.AND.VALUES(I+1).LT.999) THEN
          WRITE (mwhs1c_h_site,'(I3.3)') INT(VALUES(I+1))
        ELSE
          mwhs1c_h_site = "xxx"
        ENDIF
        mwhs1c_h_cfill1 = " "
        IF (VALUES(I+2).GT.0.AND.VALUES(I+2).LT.999) THEN
          WRITE (mwhs1c_h_1bsite,'(I3.3)') INT(VALUES(I+2))
        ELSE
          mwhs1c_h_1bsite = "xxx"
        ENDIF
        mwhs1c_h_cfill2 = " "
        mwhs1c_h_versnb = mwhs_1cvnnb
        mwhs1c_h_versyr = mwhs_1cvnyr
        mwhs1c_h_versdy = mwhs_1cvndy
        mwhs1c_h_hdrcnt = 1
        mwhs1c_h_wmosatid = FNSCALE(VALUES(I+3),1.0D0)
        IF (mwhs1c_h_wmosatid.EQ.Imiss) THEN
          mwhs1c_h_satid = Imiss
        ELSE
          mwhs1c_h_satid = mwhs1c_h_wmosatid-519               !1 for FY-3A
        ENDIF
        mwhs1c_h_instrument = 16   !see mwhs1c.h
     
      ENDIF   !IOB.EQ.NOBS

      RETURN
      END

