!+ Get data from BUFR array and write it as level 1c AAPP format

      SUBROUTINE AAPP_GET_1C_PCIASI(NSCAN,I,IOB,NOBS,VALUES,
     &   Ifirst,Ilast,KRET)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_get_1c_pciasi.F 545 2017-08-21 14:15:44Z frna $
!
! Purpose: Extract IASI-specific values from BUFR decode array 
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    16/07/2010  From aapp_get_1c.F
! 1.1    28/03/2011  Incorporate latest updates from aapp_get_1c.F
! 1.2    01/08/2017  Make enddatatime the time of the end of the last scan.
!                    Missing values for GEUMAvhrr1BQual, GEUMAvhrr1BCldFrac and
!                    GEUMAvhrr1BLandFrac set to -1, 255, 255 for consistency
!                    with original lpc datasets.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "iasipc.h"
#include "bits.h"

! Arguments:

      INTEGER NSCAN     !Number of scans processed
      INTEGER I         !Index to values
      INTEGER IOB       !Observation index
      INTEGER NOBS      !Number of observations
      REAL*8 VALUES(*)  !Data values
      INTEGER Ifirst,Ilast
      INTEGER KRET      !0 if OK, 1 to stop
      
! Parameters:

      REAL*8    Rmax
      INTEGER   Imiss
      PARAMETER (Rmax=1.6E+38)      !Missing value from BUFR is 1.7E+38
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
! Local variables:

      INTEGER   Ipcunit   !Unit number
      DATA Ipcunit/15/
      
      INTEGER FOVOffset
      DATA FOVOffset/0/
      INTEGER FOV       !Numbered from 1
      REAL*8 Spot       !Spot number (real)

      INTEGER J         !Loop index
      INTEGER K         !Index

      INTEGER DET       !Detector number (IASI)
      INTEGER CLASS     !AVHRR class (IASI)
      INTEGER IBAND     !IASI band
      INTEGER Ipc       !PC number (IASI)
      INTEGER Npc       !Number of PCs (IASI)
      INTEGER ICHAN     !Channel index
      INTEGER SCANPOS   !Scan position (IASI)
      INTEGER TDIFF     !Time since start of file, ms (IASI)
      DATA TDIFF/0/
      INTEGER Scan0     !Scan number at start of file (IASI)
      DATA Scan0/0/
      INTEGER Time0     !Time at start of file (IASI)
      DATA Time0/0/
      INTEGER NSKIP     !Number of scans to skip before writing
      DATA NSKIP/0/
      INTEGER Score     !PC score

      INTEGER factor,itmp
      REAL   ResidualRMS
      
      LOGICAL First
      DATA First/.TRUE./
      
! Functions

      INTEGER FNSCALE
      INTEGER DAYOFYEAR
      
!End of header
!------------------------------------------------------------------------------
      
!Check that the spot number is not missing; if it is, skip the ob
!Also, some encoding schemes number the FOV from 0, others from 1. Allow both.
      SPOT = VALUES(I+17)
      IF (SPOT .GT. Rmax) THEN
        WRITE(*,*)'Ob',IOB,' has missing FOV number'
        RETURN
      ENDIF
      IF (SPOT.EQ.0.0) FOVOffset=1      !FOVS numbered from 0
      FOV = SPOT + FOVOffset

!First time round, open the output file
      IF (First) THEN
        First=.FALSE.
        WRITE(*,*)'Creating 1c output dataset for PCIASI'
        CALL IOIPC("OPEN/WRITE",Ipcunit,0)
        iasipc_h_satht = 8330
        iasipc_h_period = 6081
      ENDIF


      IF (FOV.EQ.1.OR.iasipc_scnlinyr.EQ.Imiss) THEN   !First good spot
        iasipc_granule = FNSCALE(VALUES(I+20),1.0D0)
        iasipc_scnlinyr = FNSCALE(VALUES(I+5),1.0D0)
        IF (VALUES(I+5).GT.Rmax.OR.VALUES(I+6).GT.Rmax.OR.
     &      VALUES(I+7).GT.Rmax) THEN
          iasipc_scnlindy = Imiss
        ELSE
          iasipc_scnlindy = DAYOFYEAR(VALUES(I+5),VALUES(I+6),
     &                 VALUES(I+7))   
        ENDIF
        iasipc_GQisQualIndex      = VALUES(I+31)        !- 033061
        iasipc_GQisQualIndexLoc   = VALUES(I+32)        !- 033062
        iasipc_GQisQualIndexRad   = VALUES(I+33)        !- 033063
        iasipc_GQisQualIndexSpect = VALUES(I+34)        !- 033064
        iasipc_GQisSysTecSondQual = VALUES(I+35)        !- 033065
        iasipc_scalti = FNSCALE(VALUES(I+21),1.0D-2)      !km*10
      ENDIF      !FOV.EQ.1
      
!Fill the parts that change with each pixel

      DET = MOD(FOV-1,4) + 1        !IASI Detector number (1 to 4)
      SCANPOS = (FOV-1)/4 + 1       !Scan position (1 to 30)
      
!If time is missing, initialise scnlintime(SCANPOS) using previous scan
      IF (VALUES(I+8) .GT. Rmax) THEN
        IF (DET .EQ. 1) THEN
          IF (NSCAN .NE. 0) THEN
            iasipc_scnlintime(SCANPOS) = 
     &        MOD(iasipc_scnlintime(SCANPOS) + 8000,86400000)
          ELSE
            iasipc_scnlintime(SCANPOS) = Imiss !First scan - set missing
          ENDIF
        ENDIF
      ELSE
        iasipc_scnlintime(SCANPOS)=FNSCALE(VALUES(I+8),3600.0D3)  !msec
        iasipc_scnlintime(SCANPOS)=iasipc_scnlintime(SCANPOS) + 
     &             FNSCALE(VALUES(I+9),60.0D3)   !Add minutes
        iasipc_scnlintime(SCANPOS)=iasipc_scnlintime(SCANPOS) + 
     &             FNSCALE(VALUES(I+10),1.0D3)   !Add seconds
      ENDIF

! Compute scan line number, allowing for missing scans. 8 seconds between scans.
! If scans are not numbered in the BUFR, take scan 1 as first requested record.

      IF (FOV.EQ.4.AND.(NSCAN+1).GE.Ifirst) THEN
        IF ((NSCAN+1).EQ.Ifirst) THEN
          IF (VALUES(I+19).LT.Rmax) THEN
            iasipc_scnlin = VALUES(I+19)  !Scan line number
          ELSE
            iasipc_scnlin = 1    !Scan line may be missing from BUFR
          ENDIF
          Scan0 = iasipc_scnlin  !Time0 set later
        ELSE
          TDIFF = iasipc_scnlintime(1) - Time0
          IF (TDIFF.LT.0) TDIFF = TDIFF + 86400000
          iasipc_scnlin = NINT(TDIFF/8000.0) + Scan0
        ENDIF
      ENDIF

      IF (VALUES(I+24).LT.Rmax) THEN      !Check first flag not missing
        iasipc_GQisFlagQual(DET,SCANPOS) = VALUES(I+24)*2
     &      + VALUES(I+27)*4 + VALUES(I+30)*8         !- 030060
        IF (iasipc_GQisFlagQual(DET,SCANPOS) .ne. 0) THEN
          iasipc_GQisFlagQual(DET,SCANPOS) = 
     &        iasipc_GQisFlagQual(DET,SCANPOS) + 1   !old flag
        ENDIF
        IF (VALUES(I+36).LT.Rmax) THEN
          iasipc_GQisFlagQual(DET,SCANPOS) = 
     &      iasipc_GQisFlagQual(DET,SCANPOS) + VALUES(I+36)*16 !- 040020
        ENDIF
      ELSE
        iasipc_GQisFlagQual(DET,SCANPOS) = Imiss
      ENDIF

      DO J=1,2
        iasipc_latlon(J,DET,SCANPOS) = 
     &           FNSCALE(VALUES(I+10+J),1.0D4)      !10^4 x latlon
      ENDDO
      DO J=1,3,2      !Zenith angles
        iasipc_angles(J,DET,SCANPOS) = 
     &           FNSCALE(VALUES(I+12+J),1.0D2)      !10^2 x angle
      ENDDO

      DO J=2,4,2      !Azimuth angles (signed in AAPP, +ve in BUFR)
        iasipc_angles(J,DET,SCANPOS) = 
     &      FNSCALE(VALUES(I+12+J),1.0D2) !10^2 x angle
        IF (iasipc_angles(J,DET,SCANPOS).NE.Imiss .AND.
     &      iasipc_angles(J,DET,SCANPOS).GT.18000) THEN
          iasipc_angles(J,DET,SCANPOS) =
     &            iasipc_angles(J,DET,SCANPOS) - 36000 !-180 to 180
        ENDIF
      ENDDO

!Channel selection
      J = 37
      DO IBAND=1,10
        iasipc_h_startchan(IBAND) = FNSCALE(VALUES(I+J),1.0D0)
        iasipc_h_endchan(IBAND) = FNSCALE(VALUES(I+J+1),1.0D0)
        iasipc_h_scalefactor(IBAND) = 
     &              FNSCALE(VALUES(I+J+2),1.0D0)
        J = J + 3
      ENDDO
      iasipc_h_nchanused = VALUES(I+J)
      J = J+1
      DO ICHAN=1,iasipc_h_nchanused
        iasipc_h_channels(ICHAN) = FNSCALE(VALUES(I+J),1.0D0)
        IF (VALUES(I+J+1).LT.Rmax) THEN
          itmp = VALUES(I+J+1)    !- 014046
          if (itmp.GT.32768) itmp=itmp-65536
          iasipc_scalrad(ICHAN,DET,SCANPOS) = itmp
        ELSE
          iasipc_scalrad(ICHAN,DET,SCANPOS) = 0  !Set to zero if missing
        ENDIF
        J = J + 2
      ENDDO

!Instrument band definition
      DO IBAND=1,3
        IF (VALUES(I+J+3) .LT. Rmax) THEN
          ResidualRMS = VALUES(I+J+3)         !- 040016
          iasipc_logfit(IBAND,DET,SCANPOS) = 
     &          1.0E4*alog10(ResidualRMS)
        ELSE
          iasipc_logfit(IBAND,DET,SCANPOS) = Imiss
        ENDIF
        Npc = VALUES(I+J+5)                   !- 031002

        IF (FOV.EQ.1) THEN
          IF (IBAND.EQ.1) THEN
            iasipc_h_npcused = 0
            iasipc_h_npcbands = 0
          ENDIF
          iasipc_h_startpc(IBAND) = 0
          iasipc_h_endpc(IBAND) = 0
          iasipc_h_startpcchan(IBAND) =
     &               FNSCALE(VALUES(I+J),1.0D0)       !- 025140
          iasipc_h_endpcchan(IBAND) = 
     &               FNSCALE(VALUES(I+J+1),1.0D0)     !- 025141
          iasipc_h_quantization(IBAND) = 
     &               FNSCALE(VALUES(I+J+2),1.0D2)     !- 040015
          iasipc_h_evfilevn(IBAND) = 
     &               FNSCALE(VALUES(I+J+4),1.0D0)     !- 025062
     
          IF (Npc .GT. 0) THEN
            IF (IBAND .GT. 1) THEN
              iasipc_h_startpc(IBAND)= iasipc_h_endpc(IBAND-1)+1
            ELSE
              iasipc_h_startpc(IBAND)= 1
            ENDIF
            iasipc_h_endpc(IBAND)= iasipc_h_startpc(IBAND)+Npc-1
            iasipc_h_npcbands = iasipc_h_npcbands + 1
          ELSE
            iasipc_h_startpc(IBAND) = 0
            iasipc_h_endpc(IBAND) = 0
          ENDIF
          iasipc_h_npcused = iasipc_h_npcused + Npc
        ENDIF

!Principal component scores for band
        DO Ipc=1,Npc
          K = iasipc_h_startpc(IBAND)+Ipc-1
          Score = FNSCALE(VALUES(I+J+5+Ipc),1.0D0)    !- 040017
          iasipc_scores(K,DET,SCANPOS) = Score
        ENDDO

        J = J + 6 + Npc

      ENDDO   !IBAND

!Mapped AVHRR
      iasipc_avhcombination = VALUES(I+J+1)           !- 025051
      J = J + 2
      
      DO CLASS=1,7
        iasipc_avhrrCGy(CLASS,DET,SCANPOS) = 
     &       FNSCALE(VALUES(I+J),1.0D6)       !- 005060
        iasipc_avhrrCGz(CLASS,DET,SCANPOS) = 
     &       FNSCALE(VALUES(I+J+1),1.0D6)             !- 005061
        iasipc_avhrrfrac(CLASS,DET,SCANPOS) = 
     &       FNSCALE(VALUES(I+J+2),1.0D4)       !- 025085
                   !Convert precentage to 10^6 x fraction

        DO ICHAN=1,6
          K = J + 3 + (ICHAN-1)*5
          iasipc_avhmeanscalefactor(ICHAN,CLASS,DET,SCANPOS)
     &             = FNSCALE(VALUES(I+K+1),1.0D0)     !- 025142
          iasipc_avhrrmean(ICHAN,CLASS,DET,SCANPOS)
     &             = FNSCALE(VALUES(I+K+2),1.0D0)     !- 014047
          iasipc_avhsdevscalefactor(ICHAN,CLASS,DET,SCANPOS)
     &             = FNSCALE(VALUES(I+K+3),1.0D0)     !- 025142
          iasipc_avhrrsdev(ICHAN,CLASS,DET,SCANPOS)
     &             = FNSCALE(VALUES(I+K+4),1.0D0)     !- 014048
        ENDDO
        
        J = J + 33
      ENDDO  !- CLASS

!Cloud analysis

      IF (VALUES(I+J) .LT. Rmax) THEN
        iasipc_GEUMAvhrr1BCldFrac(DET,SCANPOS) = VALUES(I+J)  !- 020081
      ELSE
        iasipc_GEUMAvhrr1BCldFrac(DET,SCANPOS) = 255
      ENDIF
      IF (VALUES(I+J+2) .LT. Rmax) THEN
        iasipc_GEUMAvhrr1BLandFrac(DET,SCANPOS) = VALUES(I+J+2)  !- 020083
      ELSE
        iasipc_GEUMAvhrr1BLandFrac(DET,SCANPOS) = 255
      ENDIF
      iasipc_GIacAvgImagIIS(SCANPOS) = 
     &           FNSCALE(VALUES(I+J+4),1.0D6)         !- 040018
      iasipc_GIacVarImagIIS(SCANPOS) = 
     &           FNSCALE(VALUES(I+J+5),1.0D6)         !- 040019
      IF (VALUES(I+J+6).LT.Rmax) THEN
        iasipc_GEUMAvhrr1BQual(DET,SCANPOS) = 
     &           VALUES(I+J+6)                !- 040021
      ELSE
        IF (VALUES(I+J+7).LT.Rmax) THEN
          iasipc_GEUMAvhrr1BQual(DET,SCANPOS) = 
     &           VALUES(I+J+7) + 128          !- 040022
        ELSE
          iasipc_GEUMAvhrr1BQual(DET,SCANPOS) = -1   !set to -1 if missing
        ENDIF 
      ENDIF             
       
! Check for missing times in the first scan

      IF ((NSCAN+1).EQ.Ifirst.AND.FOV.EQ.120) THEN
        DO J=29,1,-1
          IF (iasipc_scnlintime(J).EQ.Imiss.AND.
     &        iasipc_scnlintime(J+1).NE.Imiss ) THEN
            iasipc_scnlintime(J) = iasipc_scnlintime(J+1) - 216
          ENDIF
        ENDDO
        DO J=2,30
          IF (iasipc_scnlintime(J).EQ.Imiss.AND.
     &        iasipc_scnlintime(J-1).NE.Imiss ) THEN
            iasipc_scnlintime(J) = iasipc_scnlintime(J+1) + 216
          ENDIF
        ENDDO
        iasipc_h_startorbit = FNSCALE(VALUES(I+18),1.0D0)
        iasipc_h_startdatayr = iasipc_scnlinyr
        iasipc_h_startdatady = iasipc_scnlindy
        iasipc_h_startdatatime = iasipc_scnlintime(1)
        Time0 = iasipc_scnlintime(1)
      ENDIF

! Write data record
 
      IF (FOV.EQ.120) THEN
        NSCAN = NSCAN+1
        IF (NSCAN.GE.Ifirst.AND.NSCAN.LE.Ilast) THEN
          WRITE(6,*) 'Writing to data record',NSCAN
          CALL IOIPC("WRITE DATA", Ipcunit, NSCAN-NSKIP+1) !1 header record
        ELSE IF (NSCAN.LT.Ifirst) THEN
          NSKIP = NSKIP+1
        ENDIF
      ENDIF   !FOV.EQ.120

! Update main part of header common block after each scan

      IF (FOV.EQ.120.AND.NSCAN.GE.Ifirst.AND.NSCAN.LE.Ilast)
     &        THEN
        IF (NSCAN.EQ.Ilast) KRET=1     !No more scans required
        WRITE(*,*)'Updating header'
        iasipc_h_endorbit = FNSCALE(VALUES(I+18),1.0D0)
        iasipc_h_enddatayr = iasipc_scnlinyr
        iasipc_h_enddatady = iasipc_scnlindy
        iasipc_h_enddatatime = iasipc_scnlintime(iasi_pcnfov)
        if ((iasipc_scnlintime(1)-iasipc_scnlintime(iasi_pcnfov))
     &      .gt.86392000) then
          iasipc_h_enddatady = iasipc_h_enddatady + 1
          if ((iasipc_h_enddatady .gt. 366) .or. 
     &         (MOD(iasipc_h_enddatayr,4) .ne. 0 .and. 
     &         iasipc_h_enddatady .gt. 365)) then
            iasipc_h_enddatady = 1
            iasipc_h_enddatayr = iasipc_h_enddatayr + 1
          endif
        endif
        iasipc_h_scnlin = NSCAN-NSKIP
        iasipc_h_misscnlin = 0
        iasipc_h_site = "EUM"
        iasipc_h_cfill1 = " "
        iasipc_h_1bsite = "EUM"
        iasipc_h_cfill2 = " "
        iasipc_h_versnb = iasi_pcvnnb
        iasipc_h_versyr = iasi_pcvnyr
        iasipc_h_versdy = iasi_pcvndy
        iasipc_h_hdrcnt = 1
        iasipc_h_wmosatid = FNSCALE(VALUES(I+1),1.0D0)
        iasipc_h_satid = iasipc_h_wmosatid-2     !METOP I/D
        IF (iasipc_h_wmosatid.EQ.Imiss) iasipc_h_satid = Imiss
        iasipc_h_instrument = 14       !PCIASI
! Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.iasipc_h_wmosatid) THEN
            iasipc_h_satht = SatHt(J)*10   !nominal ht, km*10
            iasipc_h_period = OrbitPd(J)   !period, seconds
            iasipc_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO
        iasipc_h_recl = array_size_iasipc*4
        iasipc_h_npc = iasi_pcnpc
        iasipc_h_nchan = iasi_pcnchan
        iasipc_h_nrchan = iasi_pcnrecon       
      ENDIF   !FOV.EQ.120

      RETURN
      END

