!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1C(KEL,NOBS,ICENTREID,ISUBCENTRE,VALUES,
     &   CINSTR,ENHANCED_IASI,ITHIN,ENCODE_NEDT,AAPP_VERSION,
     &   CVALS)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1c.F 618 2019-02-12 10:54:03Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0      12/09/05  N.C.Atkinson
! 1.1      26/01/06  Ensure IASI scaled radiances are unsigned.
!                    ICENTREID as an argument. NCA
! 1.2      27/06/06  Minimum IASI scaled radiance -5000 counts. NCA
! 1.3      11/07/06  Make "Sec" real and correct iasi1c_avhrrCGy/z scaling. NCA
! 1.4      07/09/06  Corrections to azimuth angles and iasi1c_granule. NCA
! 1.5      08/12/06  Distinguish between AMSU-B and MHS in code table 002048
! 1.6      15/12/06  Bug fix - DAY_TO_DATE was not working in December. NCA
! 1.7      06/09/07  Bug fix - AMSU-B/MHS channel numbers were zero. NCA
! 1.8      19/11/07  Bug fix - HIRS channel numbers were incorrect. NCA
! 1.9      26/11/07  Allow for missing iasi1c_h_startchan and endchan.
!                    Update comments giving IASI BUFR descriptor numbers.
! 1.10     03/09/08  Bug fix: day numbers wrong in September 2008 
! 1.11     06/03/09  Add PCIASI.
! 1.12     23/11/09  Add Enhanced IASI
! 2.0      23/09/10  Move the instrument-specific parts into own subroutines
!                    and add CrIS and ATMS
! 2.1      04/03/11  Add ISUBCENTRE as an argument, and add ATMS 1c
! 2.2      10/05/11  Add ITHIN
! 2.3      31/05/12  6 messages per scan for PCIASI to keep message length <50kb
! 2.4      20/08/12  6 messages per scan for CRIS to keep message length <50kb
! 2.5      15/02/13  Add IRAS
! 2.6      05/06/14  Add MWTS2,MWHS2,MWTS21D,MWHS21D,IRAS1D
! 2.7      19/08/14  Add MWRI by Qifeng Lu
! 2.8      10/09/14  Add support for TOVS-era HIRS files
! 2.9      08/01/16  Bug fix in DAY_TO_DATE (was getting month 0 at end of year)
! 2.10     07/04/16  Add MWRI1D
! 2.11     11/10/16  Pass ISUBCENTRE to AAPP_PUT_1D_HIRS
! 2.12     07/12/16  Option to skip scans at start and end of file
! 2.13     12/12/16  Extra arguments ENCODE_NEDT, AAPP_VERSION and CVALS
! 2.14     19/04/17  Use 1 message per scan for PCIASI and CrIS because of GTS
!                    restriction of 100 messages per file.
! 2.15     24/11/17  Add ISUBCENTRE in calls for AMSU-A, MHS and HIRS.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "ama1c.h"
#include "amb1c.h"
#include "hrs1c.h"
#include "iasi1c.h"
#include "iasipc.h"
#include "atms1c.h"
#include "cris1c.h"
#include "cris1d.h"
#include "hiras1d.h"
#include "atms1d.h"
#include "hrs1d.h"
#include "iasi1d.h"
#include "amb1d.h"
#include "mwts1c.h"
#include "mwhs1c.h"
#include "iras1c.h"
#include "mwts21c.h"
#include "mwhs21c.h"
#include "mwri1c.h"

! Arguments:

      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER ISUBCENTRE  !Sub-centre ID, e.g. 255=Missing
      CHARACTER*7  CINSTR !Instrument name
      LOGICAL ENHANCED_IASI !True if sequence is to be 3-40-007
      REAL*8 VALUES(*)    !Data values
      INTEGER ITHIN       !Number of samples/scans to increment
                          !If negative, then only the samples are incremented,
                          !every scan is used
      LOGICAL ENCODE_NEDT
      INTEGER AAPP_VERSION
      CHARACTER*80 CVALS(20000)  !For character descriptors (used for CSPP version number)
      
! Local variables:

      INTEGER   H1cunit,A1cunit,B1cunit,I1cunit,Ipcunit          !Unit numbers
      DATA H1cunit/10/, A1cunit/11/, B1cunit/12/, I1cunit/13/, 
     &     Ipcunit/14/
      INTEGER   C1dunit, AT1dunit          !Unit numbers
      DATA C1dunit/15/,  AT1dunit/16/
      INTEGER   C1cunit, AT1cunit          !Unit numbers
      DATA C1cunit/17/,  AT1cunit/18/
      INTEGER   H1dunit, I1dunit, B1dunit  !Unit numbers
      DATA H1dunit/19/,  I1dunit/20/, B1dunit/21/
      INTEGER   MWTS1cunit, MWHS1cunit, IRAS1cunit, MWRI1cunit
      DATA MWTS1cunit/22/, MWHS1cunit/23/,IRAS1cunit/24/,MWRI1cunit/25/
      
      INTEGER J         !Loop index
      INTEGER I         !Values index
      INTEGER K
       
      INTEGER FOV       !Numbered from 1
      INTEGER NFOV      !Number of FOVs per scan
      INTEGER Nrecords     !Number of scans
      INTEGER Irec      !Record loop index
      INTEGER Records_per_message  !Number of scans to encode in a BUFR message
      INTEGER TimeDelay !Milliseconds per FOV
      INTEGER FirstFOV,LastFOV,SplitFOV,SplitFOV1,SplitFOV2
      SAVE Nrecords,NFOV,Records_per_message,TimeDelay
      SAVE FirstFOV,LastFOV,SplitFOV,SplitFOV1,SplitFOV2
            
      INTEGER Current_record
      DATA Current_record/0/
      INTEGER FOVOffset
      DATA FOVOffset/0/    !Set to -1 if you want to index spots from 0
      INTEGER FOVOffset_iasi
      DATA FOVOffset_iasi/0/    !Set to -1 if you want to index spots from 0
      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec

      REAL*8 Spot       !Spot number (real)
      LOGICAL First
      DATA First/.TRUE./
      INTEGER Offset                !For use with Enhanced IASI

      CHARACTER*8 CFirstfov_iasi, CSkip_scans
      INTEGER Ifirst,Ilast,IFirstfov,Skip_scans

! Functions

      REAL*8 FNRSCALE
      
!End of header
!------------------------------------------------------------------------------
      
      IF (FIRST) THEN
      
        Skip_scans = 0
        CALL GETENV('BUFR_SKIP_SCANS',CSkip_scans)
        IF (CSkip_scans(1:1).NE." ") THEN
          READ(CSkip_scans,*) Skip_scans
          WRITE(*,*)'Number of scans to skip:',Skip_scans
        ENDIF

        CALL GETENV('BUFR_FIRSTFOV_IASI1C',CFirstfov_iasi)
        IF (CFirstfov_iasi(1:1).NE." ") THEN
          READ(CFirstfov_iasi,*) IFirstfov
          FOVOffset_iasi = IFirstfov - 1
          write(*,*) 'FOVOffset_iasi =',FOVOffset_iasi
        ENDIF

        IF (ITHIN.GT.0) THEN
          Current_record = 1 - ITHIN + Skip_scans
        ELSE
          Current_record = Skip_scans
        ENDIF

!Open the input file and read the 1c header

        WRITE(*,*)'Reading 1c/1d dataset for ',CINSTR
        Irec = 1

!Note: If SplitFOV1=SplitFOV2 and they are less than NFOV then the scan is split
!into 2 or more messages. Useful if you want to limit the message length e.g.
!for preparing bulletins for GTS. If SplitFOV1<SplitFOV2 then the scan is split
!into 2 unequal parts, with the split point incrementing for each message. This
!was implemented in order to get round the Met Office duplicate check in the
!case where there are several records per scan, all with the same time stamps.

!The number of obs per message should be limited to KVALS/NELEM (see main prog)
        IF (CINSTR.EQ."HIRS") THEN
          CALL IOH1C("OPEN/READ",H1cunit,Irec)
          CALL IOH1C("READ HEADER",H1cunit,Irec)
          Nrecords = hrs1c_h_scnlin
          TimeDelay = 100   !millisecs per FOV
          NFOV = 56
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 5       !Can be changed if required

        ELSE IF (CINSTR.EQ."AMSU-A") THEN
          CALL IOA1C("OPEN/READ",A1cunit,Irec)
          CALL IOA1C("READ HEADER",A1cunit,Irec)
          Nrecords = ama1c_h_scnlin
          TimeDelay = 203   !millisecs per FOV
          NFOV = 30
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 5       !Can be changed if required
          
        ELSE IF (CINSTR.EQ."AMSU-B".OR.CINSTR.EQ."MHS") THEN
          CALL IOB1C("OPEN/READ",B1cunit,Irec)
          CALL IOB1C("READ HEADER",B1cunit,Irec)
          Nrecords = amb1c_h_scnlin
          TimeDelay = 19   !millisecs per FOV
          NFOV = 90
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 5       !Can be changed if required
          
        ELSE IF (CINSTR.EQ."IASI") THEN
          CALL IOI1C("OPEN/READ",I1cunit,Irec)
          CALL IOI1C("READ HEADER",I1cunit,Irec)
          Nrecords = iasi1c_h_scnlin
          TimeDelay = 216   !millisecs per FOV
          NFOV = 30
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 1
          
        ELSE IF (CINSTR.EQ."PCIASI") THEN
          CALL IOIPC("OPEN/READ",Ipcunit,Irec)
          CALL IOIPC("READ HEADER",Ipcunit,Irec)
          Nrecords = iasipc_h_scnlin
          TimeDelay = 216   !millisecs per FOV
          NFOV = 30
!          SplitFOV1 = 5     !6 messages per scan  
!          SplitFOV2 = 5
          SplitFOV1 = NFOV   !1 message per scan
          SplitFOV2 = NFOV
          Records_per_message = 1
          
        ELSE IF (CINSTR.EQ."CRIS") THEN
          cris1c_h_recwords = array_size_cris1c
          CALL IOC1C("OPEN/READ",C1cunit,Irec)
          CALL IOC1C("READ HEADER",C1cunit,Irec)
          IF (cris1c_h_recwords .NE. array_size_cris1c) THEN
            CALL IOC1C("CLOSE",C1cunit,Irec)
            write(*,*)'Re-open with rec length',cris1c_h_recwords
            CALL IOC1C("OPEN/READ",C1cunit,Irec)  !Re-open with correct rec len
            CALL IOC1C("READ HEADER",C1cunit,Irec)
          ENDIF
          Nrecords = cris1c_h_scnlin
          NFOV = 30
!          SplitFOV1 = 5     !6 messages per scan 
!          SplitFOV2 = 5
          SplitFOV1 = NFOV   !1 message per scan
          SplitFOV2 = NFOV
          Records_per_message = 1
          
        ELSE IF (CINSTR.EQ."ATMS") THEN
          CALL IOAT1C("OPEN/READ",AT1cunit,Irec)
          CALL IOAT1C("READ HEADER",AT1cunit,Irec)
          Nrecords = atms1c_h_scnlin
          NFOV = 96
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 3
          
        ELSE IF (CINSTR.EQ."CRIS1D") THEN
          CALL IOC1D("OPEN/READ",C1dunit,Irec)
          CALL IOC1D("READ HEADER",C1dunit,Irec)
          Nrecords = cris1d_h_scnlin*cris1d_h_recperscan
          NFOV = 30
       !   SplitFOV1 = 15-cris1d_h_recperscan/2   !Split the scan into 2 unequal
       !   SplitFOV2 = 15+cris1d_h_recperscan/2   ! parts
       !   Records_per_message = 1
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = cris1d_h_recperscan
          
        ELSE IF (CINSTR.EQ."HIRAS1D") THEN
          CALL IOHIRAS1D("OPEN/READ",C1dunit,Irec)
          CALL IOHIRAS1D("READ HEADER",C1dunit,Irec)
          Nrecords = hiras1d_h_scnlin*hiras1d_h_recperscan
          NFOV = 29
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = hiras1d_h_recperscan
          
        ELSE IF (CINSTR.EQ."ATMS1D") THEN
          CALL IOAT1D("OPEN/READ",AT1dunit,Irec)
          CALL IOAT1D("READ HEADER",AT1dunit,Irec)
          Nrecords = atms1d_h_scnlin
          NFOV = 96
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 1
          
        ELSE IF (CINSTR.EQ."HIRS1D") THEN
          CALL IOH1D("OPEN/READ",H1dunit,Irec)
          CALL IOH1D("READ HEADER",H1dunit,Irec)
          Nrecords = hrs1d_h_scnlin
          TimeDelay = 100   !millisecs per FOV
          NFOV = 56
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 1

        ELSE IF (CINSTR.EQ."HIRS1DM") THEN
          CALL IOH1DM("OPEN/READ",H1dunit,Irec)
          CALL IOH1DM("READ HEADER",H1dunit,Irec)
          CALL HIRSM_RECORDS(Nrecords)
          TimeDelay = 100   !millisecs per FOV
          NFOV = 56
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 1

        ELSE IF (CINSTR.EQ."AMSUB1D") THEN
          CALL IOB1D("OPEN/READ",B1dunit,Irec)
          CALL IOB1D("READ HEADER",B1dunit,Irec)
          Nrecords = amb1d_h_scnlin
          TimeDelay = 19   !millisecs per FOV
          NFOV = 90
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 1

        ELSE IF (CINSTR.EQ."IASI1D") THEN
          CALL IOI1D("OPEN/READ",I1dunit,Irec)
          CALL IOI1D("READ HEADER",I1dunit,Irec)
          Nrecords = iasi1d_h_scnlin*iasi1d_h_recperscan
          NFOV = 30
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 1

        ELSE IF (CINSTR.EQ."MWTS") THEN
          CALL IOMWTS1C("OPEN/READ",MWTS1cunit,Irec)
          CALL IOMWTS1C("READ HEADER",MWTS1cunit,Irec)
          Nrecords = mwts1c_h_scnlin
          TimeDelay = 800    !Guess (16 sec for 15 FOVs + cal views + slew)
          NFOV = 15
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 50

        ELSE IF (CINSTR.EQ."MWHS") THEN
          CALL IOMWHS1C("OPEN/READ",MWHS1cunit,Irec)
          CALL IOMWHS1C("READ HEADER",MWHS1cunit,Irec)
          Nrecords = mwhs1c_h_scnlin
          TimeDelay = 18   !millisecs per FOV
          NFOV = 98
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 6

        ELSE IF (CINSTR.EQ."IRAS".OR.CINSTR.EQ."IRAS1D") THEN
          CALL IOIRAS1C("OPEN/READ",IRAS1cunit,Irec)
          CALL IOIRAS1C("READ HEADER",IRAS1cunit,Irec)
          Nrecords = IRAS1c_h_scnlin
          TimeDelay = 100    !Assume same as HIRS
          NFOV = 56
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 10

        ELSE IF (CINSTR.EQ."MWTS2".OR.CINSTR.EQ."MWTS21D") THEN
          CALL IOMWTS21C("OPEN/READ",MWTS1cunit,Irec)
          CALL IOMWTS21C("READ HEADER",MWTS1cunit,Irec)
          Nrecords = mwts21c_h_scnlin
          TimeDelay = 19    !Guess - same as AMSU-B
          NFOV = 90
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 5

        ELSE IF (CINSTR.EQ."MWRI".OR.CINSTR.EQ."MWRI1D") THEN
          CALL IOMWRI1C("OPEN/READ",MWRI1cunit,Irec)
          CALL IOMWRI1C("READ HEADER",MWRI1cunit,Irec)
          Nrecords = mwri1c_h_scnlin
          TimeDelay = 3    !Guess. Scan period=1.8 sec and there are 254 spots/scan
          NFOV = 254
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          IF (CINSTR.EQ."MWRI") Records_per_message = 5
          IF (CINSTR.EQ."MWRI1D") Records_per_message = 1

        ELSE IF (CINSTR.EQ."MWHS2".OR.CINSTR.EQ."MWHS21D") THEN
          CALL IOMWHS21C("OPEN/READ",MWHS1cunit,Irec)
          CALL IOMWHS21C("READ HEADER",MWHS1cunit,Irec)
          Nrecords = mwhs21c_h_scnlin
          TimeDelay = 19    !Guess - same as AMSU-B
          NFOV = 98
          SplitFOV1 = NFOV
          SplitFOV2 = NFOV
          Records_per_message = 5

        ELSE
          WRITE(*,*)'Error in aapp_put_1c: Invalid instrument ',CINSTR
          STOP
        ENDIF
        WRITE(*,*) 'Number of data records: ',Nrecords
        FirstFOV = 1
        LastFOV = SplitFOV1
        SplitFOV = SplitFOV1
        First=.FALSE.

      ENDIF   !First   

      NOBS = 0

!Loop round records (normally 1 scan per record, except for CRIS1D)

      DO Irec=1,Records_per_message

      IF (FirstFOV.EQ.1) THEN
        IF (ITHIN.GT.0) THEN
          Current_record = Current_record + ITHIN
        ELSE
          Current_record = Current_record + 1
        ENDIF
      ENDIF
      
!See if we have finished

      IF (Current_record.GT.Nrecords-Skip_scans) THEN
        IF (CINSTR.EQ."HIRS")   CALL IOH1C("CLOSE",H1cunit,0)
        IF (CINSTR.EQ."AMSU-A") CALL IOA1C("CLOSE",A1cunit,0)
        IF (CINSTR.EQ."AMSU-B".OR.CINSTR.EQ."MHS")
     &                          CALL IOB1C("CLOSE",B1cunit,0)
        IF (CINSTR.EQ."IASI")   CALL IOI1C("CLOSE",I1cunit,0)
        IF (CINSTR.EQ."PCIASI") CALL IOIPC("CLOSE",Ipcunit,0)
        IF (CINSTR.EQ."CRIS")   CALL IOC1C("CLOSE",C1cunit,0)
        IF (CINSTR.EQ."ATMS")   CALL IOAT1C("CLOSE",AT1cunit,0)
        IF (CINSTR.EQ."CRIS1D") CALL IOC1D("CLOSE",C1dunit,0)
        IF (CINSTR.EQ."HIRAS1D") CALL IOHIRAS1D("CLOSE",C1dunit,0)
        IF (CINSTR.EQ."ATMS1D") CALL IOAT1D("CLOSE",AT1dunit,0)
        IF (CINSTR.EQ."HIRS1D") CALL IOH1D("CLOSE",H1dunit,0)
        IF (CINSTR.EQ."HIRS1DM") CALL IOH1DM("CLOSE",H1dunit,0)
        IF (CINSTR.EQ."IASI1D") CALL IOI1D("CLOSE",I1dunit,0)
        IF (CINSTR.EQ."AMSUB1D") CALL IOB1D("CLOSE",B1dunit,0)
        IF (CINSTR.EQ."MWTS") CALL IOMWTS1C("CLOSE",MWTS1cunit,0)
        IF (CINSTR.EQ."MWHS") CALL IOMWHS1C("CLOSE",MWHS1cunit,0)
        IF (CINSTR.EQ."IRAS".OR.CINSTR.EQ."IRAS1D")
     &                        CALL IOIRAS1C("CLOSE",IRAS1cunit,0)
        IF (CINSTR.EQ."MWTS2".OR.CINSTR.EQ."MWTS21D")
     &                        CALL IOMWTS21C("CLOSE",MWTS1cunit,0)
        IF (CINSTR.EQ."MWRI".OR.CINSTR.EQ."MWRI1D")
     &                        CALL IOMWRI1C("CLOSE",MWRI1cunit,0)
        IF (CINSTR.EQ."MWHS2".OR.CINSTR.EQ."MWHS21D")
     &                        CALL IOMWHS21C("CLOSE",MWHS1cunit,0)
        RETURN
      ELSE

        DO FOV=FirstFOV,LastFOV
        
!----------------------------------------------------------------------------
!AMSU-A data
!----------------------------------------------------------------------------

          IF (CINSTR.EQ."AMSU-A") THEN
          
            CALL AAPP_PUT_1C_AMSUA(A1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ENCODE_NEDT, AAPP_VERSION)

!----------------------------------------------------------------------------
!AMSU-B/MHS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."AMSU-B".OR.CINSTR.EQ."MHS") THEN
          
            CALL AAPP_PUT_1C_AMSUB(B1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ENCODE_NEDT, AAPP_VERSION)

!----------------------------------------------------------------------------
!HIRS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."HIRS") THEN
          
            CALL AAPP_PUT_1C_HIRS(H1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ENCODE_NEDT, AAPP_VERSION)

!----------------------------------------------------------------------------
!IASI data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."IASI") THEN
            CALL AAPP_PUT_1C_IASI(I1cunit, Current_record, FOV,
     &         FOVOffset_iasi, ICENTREID, ENHANCED_IASI, NOBS, KEL,
     &         VALUES, AAPP_VERSION)
          
!----------------------------------------------------------------------------
!PCIASI data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."PCIASI") THEN
            CALL AAPP_PUT_1C_PCIASI(Ipcunit, Current_record, FOV,
     &         FOVOffset_iasi, ICENTREID, ENHANCED_IASI, NOBS, KEL,
     &         VALUES, AAPP_VERSION)
          
!----------------------------------------------------------------------------
!CrIS 1c data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."CRIS") THEN
            CALL AAPP_PUT_1C_CRIS(C1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, NOBS, KEL, VALUES,
     &         AAPP_VERSION, CVALS)
          
!----------------------------------------------------------------------------
!ATMS 1c data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."ATMS") THEN
            CALL AAPP_PUT_1C_ATMS(AT1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, NOBS, KEL, VALUES,
     &         AAPP_VERSION, CVALS)
          
!----------------------------------------------------------------------------
!CrIS 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."CRIS1D") THEN
            CALL AAPP_PUT_1D_CRIS(C1dunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, NOBS, KEL, VALUES)
          
!----------------------------------------------------------------------------
!HIRAS 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."HIRAS1D") THEN
            CALL AAPP_PUT_1D_HIRAS(C1dunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, NOBS, KEL, VALUES)
          
!----------------------------------------------------------------------------
!ATMS 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."ATMS1D") THEN
            CALL AAPP_PUT_1D_ATMS(AT1dunit, Current_record, FOV,
     &       FOVOffset, ICENTREID, ISUBCENTRE, NOBS, KEL, VALUES, 
     &       ABS(ITHIN))
          
!----------------------------------------------------------------------------
!HIRS 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."HIRS1D") THEN
            CALL AAPP_PUT_1D_HIRS(H1dunit, Current_record, FOV,
     &       FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &       VALUES)
          ELSE IF (CINSTR.EQ."HIRS1DM") THEN
            CALL AAPP_PUT_1D_HIRSM(H1dunit, Current_record, FOV,
     &       FOVOffset, ICENTREID, TimeDelay, NOBS, KEL, VALUES)

!----------------------------------------------------------------------------
!AMSUB 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."AMSUB1D") THEN
            CALL AAPP_PUT_1D_AMSUB(B1dunit, Current_record, FOV,
     &       FOVOffset, ICENTREID, TimeDelay, NOBS, KEL, VALUES)
          
!----------------------------------------------------------------------------
!IASI 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."IASI1D") THEN
            CALL AAPP_PUT_1D_IASI(I1dunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, NOBS, KEL, VALUES)
          
!----------------------------------------------------------------------------
!MWTS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWTS") THEN
          
            CALL AAPP_PUT_1C_MWTS(MWTS1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, AAPP_VERSION)

!----------------------------------------------------------------------------
!MWHS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWHS") THEN
          
            CALL AAPP_PUT_1C_MWHS(MWHS1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, AAPP_VERSION)

!----------------------------------------------------------------------------
!IRAS data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."IRAS") THEN
          
            CALL AAPP_PUT_1C_IRAS(IRAS1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, AAPP_VERSION)

!----------------------------------------------------------------------------
!MWTS2 1c data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWTS2") THEN
          
            CALL AAPP_PUT_1C_MWTS2(MWTS1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ABS(ITHIN), AAPP_VERSION)

!----------------------------------------------------------------------------
!MWRI 1c data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWRI") THEN

            CALL AAPP_PUT_1C_MWRI(MWRI1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ABS(ITHIN), AAPP_VERSION)

!----------------------------------------------------------------------------
!MWRI 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWRI1D") THEN
          
            CALL AAPP_PUT_1D_MWRI(MWRI1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ABS(ITHIN))


!----------------------------------------------------------------------------
!MWTS2 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWTS21D") THEN
          
            CALL AAPP_PUT_1D_MWTS2(MWTS1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ABS(ITHIN))

!----------------------------------------------------------------------------
!MWHS2 1c data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWHS2") THEN
          
            CALL AAPP_PUT_1C_MWHS2(MWHS1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ABS(ITHIN), AAPP_VERSION)

!----------------------------------------------------------------------------
!MWHS2 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."MWHS21D") THEN
          
            CALL AAPP_PUT_1D_MWHS2(MWHS1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ABS(ITHIN))

!----------------------------------------------------------------------------
!IRAS 1d data
!----------------------------------------------------------------------------

          ELSE IF (CINSTR.EQ."IRAS1D") THEN
          
            CALL AAPP_PUT_1D_IRAS(IRAS1cunit, Current_record, FOV,
     &         FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL,
     &         VALUES, ABS(ITHIN))

!----------------------------------------------------------------------------
!All data types
!----------------------------------------------------------------------------

          ELSE
            WRITE(*,*)'Error in aapp_put_1c: Invalid instrument ',CINSTR
          ENDIF
          
        ENDDO  !FOV

        IF (LastFOV .NE. NFOV) THEN
          FirstFOV = FirstFOV + SplitFOV    !Next part of the scan
          IF (SplitFOV1 .EQ. SplitFOV2) THEN
            LastFOV = MIN(FirstFOV + SplitFOV - 1, NFOV)
          ELSE
            LastFOV = NFOV
          ENDIF 
        ELSE
          IF (SplitFOV .LT. SplitFOV2) THEN
            SplitFOV = SplitFOV + 1         !Increment the split point
          ELSE
            SplitFOV = SplitFOV1
          ENDIF
          FirstFOV = 1
          LastFOV = SplitFOV
        ENDIF
      ENDIF  !Current_record.LE.Nrecords
      ENDDO  !Loop over scan lines
      RETURN
      END

!-----------------------------------------------------------------------------
      
      FUNCTION FNRSCALE(Ival,Scale)
      REAL*8  FNRSCALE
      INTEGER Ival
      REAL*8  Scale
      
      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E38)     !BUFR missing value
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      IF (Ival.EQ.Imiss) THEN
        FNRSCALE = Rmiss
      ELSE
        FNRSCALE = Ival*Scale
      ENDIF
      
      RETURN
      END

!-----------------------------------------------------------------------

      SUBROUTINE DAY_TO_DATE(Year,DayOfYear,Millisecs,
     &               Month,Day,Hour,Min,Sec)

      IMPLICIT NONE
      INTEGER    Year,DayOfYear,Millisecs    !Inputs
      INTEGER    Month,Day,Hour,Min          !Outputs
      REAL*8     Sec                         !Output
      INTEGER    I                           !Loop index
      INTEGER    Maxdays                     !Days in a year
      INTEGER    NOLEAP(13), LEAP(13)
      DATA       NOLEAP /0,31,59,90,120,151,181,212,243,273,304,334,365/
      DATA       LEAP   /0,31,60,91,121,152,182,213,244,274,305,335,366/
      LOGICAL    Leapyear

      IF (MOD(Year,4).EQ.0) THEN  !OK up to 2100
        Leapyear = .TRUE.
        Maxdays = 366
      ELSE
        Leapyear = .FALSE.
        Maxdays = 365
      ENDIF          
      
      IF (Millisecs.GE.86400000) THEN       !Wrap around - add 1 day
        Millisecs = Millisecs - 86400000
        DayOfYear = DayOfYear + 1
        IF (DayOfYear.GT.Maxdays) THEN
          Year = Year + 1
          DayOfYear = 1
        ENDIF
      ENDIF
      
      Month = 0
      IF (Leapyear) THEN
        DO I=2,13
          IF (LEAP(I).GE.DayOfYear.AND.Month.EQ.0) THEN
            Month = I-1
            Day = DayOfYear - LEAP(Month)
          ENDIF
        ENDDO
      ELSE
        DO I=2,13
          IF (NOLEAP(I).GE.DayOfYear.AND.Month.EQ.0) THEN
            Month = I-1
            Day = DayOfYear - NOLEAP(Month)
          ENDIF
        ENDDO
      ENDIF
      Hour = Millisecs/3600000
      Min = Millisecs/60000 - Hour*60
      Sec = MOD(Millisecs/1000.0D0,60.0D0)
      RETURN
      END

!-----------------------------------------------------------------------

      SUBROUTINE BitSwap(BitIn, Mask, NumBitsIn, BitOut, NumBitsOut)
      IMPLICIT NONE

! purpose         : To take an input bit field (BitIn) and set the bits
!                 : in the output bit field (BitOut) determined by the
!                 : mapping in the Mask field.                     
!                                                                               
! calls           : BTEST  : To see if a bit is set
!                 : IBSET  : To set a bit
!                                                                               
! Description of Mask :
! 
! The mask is of the size of the input flag (bit) table. Each 
! element in the mask is the bit in the output flag table to set
! corresponding to the bit in the input flag table. 
!
! Arguments:

      INTEGER BitIn                !- i/p bit value (Integer form)
      INTEGER NumBitsIn            !- size of i/p bit table
      INTEGER Mask(NumBitsIn)      !- input bit mask (see description)
      INTEGER BitOut               !- o/p bit value (Integer form)
      INTEGER NumBitsOut           !- sze of o/p bit table

! Local variables:

      INTEGER I                    !- used for looping
      LOGICAL BitSet               !- TRUE if bit set
      
      BitOut = 0

      DO I = 1,NumBitsIn
        IF (Mask(I).GT.0) THEN
          BitSet = BTEST(BitIn,NumBitsIn-I)
          IF (BitSet) BitOut = IBSET(BitOut,NumBitsOut-Mask(I))
        ENDIF
      ENDDO

      RETURN
      END

