!+ Read level 1c AAPP header/data records and put values in BUFR array
!
      SUBROUTINE AAPP_PUT_1D_HIRS(H1Dunit, Current_scan, FOV,
     &   FOVOffset, ICENTREID, ISUBCENTRE, TimeDelay, NOBS, KEL, VALUES)
      IMPLICIT NONE
!----------------------------------------------------------------------------
!
! Name: $Id: aapp_put_1d_hirs.F 534 2017-07-31 09:17:38Z frna $
!
! Purpose:
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2011, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0    10/06/2011  NCA. Format is that of ATOVSG/ATOVSL at Met Office.
! 1.1    26/06/2014  Round FovTime to nearest second before encoding
! 1.2    11/10/2016  Use ISUBCENTRE in the second instance of 001031,
!                    instead of repeating the ICENTREID. So it is not a
!                    "level 1b centre" but allows us to store the full
!                    centre details without changing the BUFR sequence.
! 1.3    31/07/2017  Correct channel 5 AVHRR and the scale of cloud fraction.
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Declarations:
!
!-----------------------------------------------------------------------
! Global variables:
!-----------------------------------------------------------------------

#include "inparms.h"
#include "hrs1d.h"

! Arguments:
      INTEGER H1Dunit      
      INTEGER Current_scan
      INTEGER ICENTREID   !Centre ID, e.g. 255=Missing
      INTEGER ISUBCENTRE  !Subcentre ID, e.g. 255=Missing
      INTEGER FOV         !Numbered from 1
      INTEGER FOVOffset   !Normally 0
      INTEGER TimeDelay   !Milliseconds per FOV
      INTEGER KEL         !Array elements per ob
      INTEGER NOBS        !Number of obs in BUFR message
      REAL*8 VALUES(*)    !Data values

! Parameters:

      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=1.7E+38)     !Missing value for BUFR
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      INTEGER    NumBitsIn          !- size of raw data bit fields
      PARAMETER  (NumBitsIn  = 32)
      INTEGER    NumBitsOut         !- set BUFR bit field width
      PARAMETER  (NumBitsOut = 24)

! Local variables:
      INTEGER I,J
      INTEGER Irec
      INTEGER BIT                   !For use with unbitswap
      INTEGER Mask033030(NumBitsIn) !- bit mask for 033030
      INTEGER Mask033031(NumBitsIn) !- bit mask for 033031
      INTEGER Mask002223(NumBitsIn) !- bit mask for 002223
      INTEGER Mask025214(NumBitsIn) !- bit mask for 025214
      INTEGER Mask025215(NumBitsIn) !- bit mask for 025215
      DATA Mask033030 /1,2,3,4,5,6,7,25*0/
      DATA Mask033031 /8*0,1,2,3,4,4*0,5,6,7,8,9,10,11,12,13,14,15,16,
     &                 17,3*0/
      DATA Mask002223 /26*0,11,10,9,0,2,1/
      DATA Mask025214 /1,2,3,4,5,6,7,8,9,10,18*0,11,12,13,14/
      DATA Mask025215 /0,1,8*0,2,22,21,20,19,18,17,16,15,14,13,12,11,10,
     &                 9,8,7,6,5,4,3,23/

      INTEGER scnlindy,scnlinyr,FovTime  !date and time
      INTEGER Month,Day,Hour,Min
      REAL*8  Sec

! Functions
      REAL*8 FNRSCALE

!End of header
!------------------------------------------------------------------------------

      IF (FOV.EQ.1) THEN
        irec = Current_scan + hrs1d_h_hdrcnt
        CALL IOH1D ("READ DATA", H1Dunit, irec)
        write(*,*)'Read scan',Current_scan
      ENDIF

!Point to data in VALUES array
      NOBS = NOBS + 1
      I = (NOBS-1)*KEL

      VALUES(i+1) = hrs1d_h_wmosatid              !- 001007       

!Date and time of the FOV
      scnlindy = hrs1d_scnlindy
      scnlinyr = hrs1d_scnlinyr
      FovTime = hrs1d_scnlintime+((FOV-1)*TimeDelay)
      FovTime = ((FovTime + 500)/1000)*1000    !Round to nearest second
      CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &    Month,Day,Hour,Min,Sec)

      VALUES(i+2) = scnlinyr                              !- 004001
      VALUES(i+3) = Month                                 !- 004002
      VALUES(i+4) = Day                                   !- 004003
      VALUES(i+5) = Hour                                  !- 004004
      VALUES(i+6) = Min                                   !- 004005
      VALUES(i+7) = Sec                                   !- 004006

      VALUES(i+8) = FNRSCALE(hrs1d_latlon(1,FOV),1.0D-4)  !- 005001
      VALUES(i+9) = FNRSCALE(hrs1d_latlon(2,FOV),1.0D-4)  !- 006001

! surface type: raw 1d data  : 0 = Sea, 1 = Mixed, 2 = Land
!             : BUFR 1d data : 0 = Land, 1 = Sea, 2 = Coast    !- 008012 

      IF (hrs1d_surface(2,FOV) .EQ. 0) THEN
        VALUES(i+10) = 1
      ELSEIF (hrs1d_surface(2,FOV) .EQ. 1) THEN
        VALUES(i+10) = 2
      ELSEIF (hrs1d_surface(2,FOV) .EQ. 2) THEN
        VALUES(i+10) = 0
      ELSE
        VALUES(i+10) = Rmiss
      ENDIF

!1d dataset creation site
      VALUES(i+11)  = 4                                     !- 008070
      VALUES(i+12)  = ICENTREID                             !- 001031

!1b dataset creation site
      VALUES(i+13)  = 2                                     !- 008070
      VALUES(i+14)  = ISUBCENTRE                            !- 001031

!orbit
      VALUES(i+15) = FNRSCALE(hrs1d_h_startorbit,1.0D0)     !- 005040

!atovpp version number
      VALUES(i+16) = FNRSCALE(hrs1d_h_atovppvn,1.0D0)       !- 055023

!instruments
      IF (hrs1d_h_instruments .NE. Imiss) THEN
        CALL BITSWAP(hrs1d_h_instruments, Mask002223, NumBitsIn, 
     &       Bit, 12)
        VALUES(i+17) = 1.0D0*Bit                            !- 002223
      ELSE
        VALUES(i+17) = Rmiss
      ENDIF

!scan line and FOV
      VALUES(i+18) = FNRSCALE(hrs1d_scnlin,1.0D0)           !- 005041
      VALUES(I+19) = FOV + FOVOffset                        !- 005043

!quality flags
      CALL BITSWAP(hrs1d_qualind, Mask033030, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+20) = 1.0D0*Bit                              !- 033030
      CALL BITSWAP(hrs1d_scanqual, Mask033031, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+21) = 1.0D0*Bit                              !- 033031

!preprocessing flags
      CALL BITSWAP(hrs1d_prepro(1,FOV), Mask025214, NumBitsIn, 
     &                   Bit, NumBitsOut)
      VALUES(i+22) = 1.0D0*Bit                              !- 025214
      IF (hrs1d_prepro(2,FOV) .GT. 0) THEN
        VALUES(i+23) = hrs1d_prepro(2,FOV) - 1              !- 008205
      ELSE
        VALUES(i+23) = Rmiss
      ENDIF
      VALUES(i+24) = FNRSCALE(hrs1d_prepro(3,FOV),1.0D-2)   !- 002224
      IF (VALUES(i+24).GT.163.82) VALUES(i+24) = Rmiss
      VALUES(i+25) = FNRSCALE(hrs1d_prepro(4,FOV),1.0D-2)   !- 025206
      IF (VALUES(i+25).GT.250.0) VALUES(i+25) = Rmiss
      VALUES(i+26) = FNRSCALE(hrs1d_prepro(5,FOV),1.0D-2)   !- 025207
      IF (VALUES(i+26).GT.126.0) VALUES(i+26) = Rmiss

!sensors and version numbers
      VALUES(i+27) = 0               !HIRS                  !- 002048      
      VALUES(i+28) = FNRSCALE(hrs1d_h_Hcorrvn,1.0D0)        !- 055024
      VALUES(i+29) = FNRSCALE(hrs1d_h_Hcloudvn,1.0D0)       !- 055025
      VALUES(i+30) = 3               !AMSU-A                !- 002048      
      VALUES(i+31) = FNRSCALE(hrs1d_h_Acorrvn,1.0D0)        !- 055024
      VALUES(i+32) = FNRSCALE(hrs1d_h_Apcpvn,1.0D0)         !- 055026
      VALUES(i+33) = 4               !AMSU-B/MHS            !- 002048      
      VALUES(i+34) = FNRSCALE(hrs1d_h_Bcorrvn,1.0D0)        !- 055024
      VALUES(i+35) = FNRSCALE(hrs1d_h_Bpcpvn,1.0D0)         !- 055026

!navigation
      VALUES(i+36) = FNRSCALE(hrs1d_scalti,1.0D2)           !- 007001
      VALUES(i+37) = FNRSCALE(hrs1d_surface(1,FOV),1.0D0)   !- 010001
      VALUES(i+38) = FNRSCALE(hrs1d_angles(1,FOV),1.0D-2)   !- 007024
      IF (hrs1d_angles(2,FOV).NE.Imiss .AND.
     &    hrs1d_angles(2,FOV).LT.0) THEN
        hrs1d_angles(2,FOV)=hrs1d_angles(2,FOV)+36000  !+ve
      ENDIF
      VALUES(i+39) = FNRSCALE(hrs1d_angles(2,FOV),1.0D-2)     !- 005021
      VALUES(i+40) = FNRSCALE(hrs1d_angles(3,FOV),1.0D-2)     !- 007025
      IF (hrs1d_angles(4,FOV).NE.Imiss .AND.
     &    hrs1d_angles(4,FOV).LT.0) THEN
        hrs1d_angles(4,FOV)=hrs1d_angles(4,FOV)+36000  !+ve
      ENDIF
      VALUES(i+41) = FNRSCALE(hrs1d_angles(4,FOV),1.0D-2)     !- 005022

!FOV quality
      IF (hrs1d_hirsqual(FOV) .NE. Imiss) THEN
        CALL BITSWAP(hrs1d_hirsqual(FOV), Mask025215, NumBitsIn, 
     &                   Bit, NumBitsOut)
        VALUES(i+42) = 1.0*Bit                                !- 025215
      ELSE
        VALUES(i+42) = Rmiss
      ENDIF

!Instrument temperatures
      VALUES(i+43) = 0                                        !- 002151
      VALUES(i+44) = FNRSCALE(hrs1d_instrtemp,1.0D-2)         !- 012064
      VALUES(i+45) = 3                                        !- 002151
      VALUES(i+46) = FNRSCALE(hrs1d_instrtempa1,1.0D-2)       !- 012064
      VALUES(i+47) = 4                                        !- 002151
      VALUES(i+48) = FNRSCALE(hrs1d_instrtempa2,1.0D-2)       !- 012064
      VALUES(i+49) = 6                                        !- 002151
      VALUES(i+50) = FNRSCALE(hrs1d_instrtempb,1.0D-2)        !- 012064

!-------------------------------
!Loop over HIRS channels 1 -> 19
!-------------------------------
      DO j = 1,Hnchan-1

!TOVS/ATOVS/AVHRR instrumentation channel number
        VALUES(i+51+(j-1)*5) = j                              !- 002150

!log_10 of (Temperature-radiance central wavenumber (per metre)) for ATOVS
        IF (hrs1d_h_hrsPlanck(1,j) .NE. Imiss) THEN
          IF (J.LT.13) THEN
            VALUES(i+52+(j-1)*5) =                         !Channels 1-12
     &        log10(0.0001 * hrs1d_h_hrsPlanck(1,j))          !- 025076
          ELSE
            VALUES(i+52+(j-1)*5) =                         !Channels 13-19
     &        log10(0.001 * hrs1d_h_hrsPlanck(1,j))           !- 025076
          ENDIF 
        ELSE
          VALUES(i+52+(j-1)*5) = Rmiss
        ENDIF

!Bandwidth correction coefficients 1 & 2
        VALUES(i+53+(j-1)*5) = 
     &            FNRSCALE(hrs1d_h_hrsPlanck(2,j),1.0D-6)     !- 025077
        VALUES(i+54+(j-1)*5) = 
     &            FNRSCALE(hrs1d_h_hrsPlanck(3,j),1.0D-6)     !- 025078

!Brightness temperatures
        VALUES(i+55+(j-1)*5) = 
     &            FNRSCALE(hrs1d_btemps(j,FOV),1.0D-2)        !- 012063

      ENDDO  !- j

!Channel 20
      VALUES(i+146) = 20                                        !- 002150
      VALUES(i+147) = FNRSCALE(hrs1d_h_20solfiltirrad,1.0D-6)   !- 025079
      VALUES(i+148) = FNRSCALE(hrs1d_h_20equifiltwidth,1.0D-10) !- 025080
      VALUES(i+149) = FNRSCALE(hrs1d_btemps(20,FOV),1.0D-2)     !- 014045

!-------------------------------
!Loop over AMSU channels 1 -> 20
!-------------------------------
      DO j = 1,Anchan+Bnchan

!TOVS/ATOVS/AVHRR instrumentation channel number (28 -> 47)
        VALUES(i+150+(j-1)*5) = j + 27                          !- 002150

!log_10 of (Temperature-radiance central wavenumber (per metre)) for ATOVS
        IF (hrs1d_h_amsPlanck(1,j) .NE. Imiss) THEN
           VALUES(i+151+(j-1)*5) = 
     &        log10(0.0001 * hrs1d_h_amsPlanck(1,j))            !-025076
        ELSE
          VALUES(i+151+(j-1)*5) = Rmiss
        ENDIF

!Bandwidth correction coefficients 1 & 2
        VALUES(i+152+(j-1)*5) = 
     &     FNRSCALE(hrs1d_h_amsPlanck(2,j),1.0D-6)              !- 025077
        VALUES(i+153+(j-1)*5) = 
     &     FNRSCALE(hrs1d_h_amsPlanck(3,j),1.0D-6)              !- 025078

!Brightness temperatures
        VALUES(i+154+(j-1)*5) = FNRSCALE(hrs1d_btemps(j+20,FOV),1.0D-2) !-012063

      ENDDO  !- j

!=======================================================================
! AVHRR data
! This applies to avh2hirs mode 1, i.e. local=.false. in av_map_maia_2.F
!=======================================================================

! fraction of clear pixels in HIRS FOV
! Note that 025085 has scale 0, so it is really a percentage
      IF (hrs1d_avhrr(1,FOV) .GE. 0) THEN
!        VALUES(i+250) = 0.01*hrs1d_avhrr(1,FOV)                 !- 025085
        VALUES(i+250) = hrs1d_avhrr(1,FOV)                      !- 025085
      ELSE
        VALUES(i+250) = Rmiss
      ENDIF

! channel 1
      VALUES(i+251) = 48        !channel                        !- 002150
      VALUES(i+252) = 4         !significance=mean              !- 008023
      VALUES(i+253) = 0         !pixel type mixed               !- 008072
      IF (hrs1d_avhrr(2,FOV).GE.0.AND.hrs1d_avhrr(2,FOV).LT.1270) THEN
        VALUES(i+254) = 0.1*hrs1d_avhrr(2,FOV)                  !- 014027
      ELSE
        VALUES(i+254) = Rmiss
      ENDIF
      VALUES(i+255) = 1         !pixel type clear               !- 008072
      IF (hrs1d_avhrr(7,FOV).GE.0.AND.hrs1d_avhrr(7,FOV).LT.1270) THEN
        VALUES(i+256) = 0.1*hrs1d_avhrr(7,FOV)                  !- 014027
      ELSE
        VALUES(i+256) = Rmiss
      ENDIF

! channel 2
      VALUES(i+257) = 49        !channel                        !- 002150
      VALUES(i+258) = 4         !significance=mean              !- 008023
      VALUES(i+259) = 0         !pixel type mixed               !- 008072
      IF (hrs1d_avhrr(3,FOV).GE.0.AND.hrs1d_avhrr(3,FOV).LT.1270) THEN
        VALUES(i+260) = 0.1*hrs1d_avhrr(3,FOV)                  !- 014027
      ELSE
        VALUES(i+260) = Rmiss
      ENDIF
      VALUES(i+261) = 1         !pixel type clear               !- 008072
      IF (hrs1d_avhrr(8,FOV).GE.0.AND.hrs1d_avhrr(8,FOV).LT.1270) THEN
        VALUES(i+262) = 0.1*hrs1d_avhrr(8,FOV)                  !- 014027
      ELSE
        VALUES(i+262) = Rmiss
      ENDIF

! channel 3a
      VALUES(i+263) = 50        !channel                        !- 002150
      VALUES(i+264) = 4         !significance=mean              !- 008023
      VALUES(i+265) = 0         !pixel type mixed               !- 008072
      VALUES(i+267) = 1         !pixel type clear               !- 008072
      IF (IAND(hrs1d_prepro(1,FOV),8) .NE. 0) THEN
        IF (hrs1d_avhrr(4,FOV).GE.0.AND.hrs1d_avhrr(4,FOV).LT.1270) THEN
          VALUES(i+266) = 0.1*hrs1d_avhrr(4,FOV)                !- 014027
        ELSE
          VALUES(i+266) = Rmiss
        ENDIF
        IF (hrs1d_avhrr(9,FOV).GE.0.AND.hrs1d_avhrr(9,FOV).LT.1270) THEN
          VALUES(i+268) = 0.1*hrs1d_avhrr(9,FOV)                !- 014027
        ELSE
          VALUES(i+268) = Rmiss
        ENDIF
      ELSE
        VALUES(i+266) = Rmiss
        VALUES(i+268) = Rmiss
      ENDIF
        
! channel 3b
      VALUES(i+269) = 51        !channel                        !- 002150
      VALUES(i+270) = 4         !significance=mean              !- 008023
      VALUES(i+271) = 0         !pixel type mixed               !- 008072
      VALUES(i+273) = 1         !pixel type clear               !- 008072
      IF (IAND(hrs1d_prepro(1,FOV),8) .EQ. 0) THEN
        IF (hrs1d_avhrr(4,FOV) .GE. 0) THEN
          VALUES(i+272) = 0.01*hrs1d_avhrr(4,FOV)               !- 012063
        ELSE
          VALUES(i+272) = Rmiss
        ENDIF
        IF (hrs1d_avhrr(9,FOV) .GE. 0) THEN
          VALUES(i+274) = 0.01*hrs1d_avhrr(9,FOV)               !- 012063
        ELSE
          VALUES(i+274) = Rmiss
        ENDIF
      ELSE
        VALUES(i+272) = Rmiss
        VALUES(i+274) = Rmiss
      ENDIF

! channel 4 (both mean and std dev)
      VALUES(i+275) = 52        !channel                        !- 002150
      VALUES(i+276) = 4         !significance=mean              !- 008023
      VALUES(i+277) = 0         !pixel type mixed               !- 008072
      IF (hrs1d_avhrr(5,FOV) .GE. 0) THEN
        VALUES(i+278) = 0.01*hrs1d_avhrr(5,FOV)                 !- 012063
      ELSE
        VALUES(i+278) = Rmiss
      ENDIF
      VALUES(i+279) = 1         !pixel type clear               !- 008072
      IF (hrs1d_avhrr(10,FOV) .GE. 0) THEN
        VALUES(i+280) = 0.01*hrs1d_avhrr(10,FOV)                !- 012063
      ELSE
        VALUES(i+280) = Rmiss
      ENDIF
      VALUES(i+281) = 10        !significance=stdev             !- 008023
      VALUES(i+282) = 0         !pixel type mixed               !- 008072
      IF (hrs1d_avhrr(12,FOV) .GE. 0) THEN
        VALUES(i+283) = 0.01*hrs1d_avhrr(12,FOV)                !- 012063
      ELSE
        VALUES(i+283) = Rmiss
      ENDIF
      VALUES(i+284) = 1         !pixel type clear               !- 008072
      IF (hrs1d_avhrr(13,FOV) .GE. 0) THEN
        VALUES(i+285) = 0.01*hrs1d_avhrr(13,FOV)                !- 012063
      ELSE
        VALUES(i+285) = Rmiss
      ENDIF

! channel 5
      VALUES(i+286) = 53        !channel                        !- 002150
      VALUES(i+287) = 4         !significance=mean              !- 008023
      VALUES(i+288) = 0         !pixel type mixed               !- 008072
!      IF (hrs1d_avhrr(10,FOV) .GE. 0) THEN   !correction 31/7/2017
      IF (hrs1d_avhrr(6,FOV) .GE. 0) THEN
        VALUES(i+289) = 0.01*hrs1d_avhrr(6,FOV)                 !- 012063
      ELSE
        VALUES(i+289) = Rmiss
      ENDIF
      VALUES(i+290) = 1         !pixel type clear               !- 008072
      IF (hrs1d_avhrr(11,FOV) .GE. 0) THEN
        VALUES(i+291) = 0.01*hrs1d_avhrr(11,FOV)                !- 012063
      ELSE
        VALUES(i+291) = Rmiss
      ENDIF

! emissivity (only used at Met Office - set to missing)
      DO j=1,13
        VALUES(i+291+J) = Rmiss
      ENDDO

      END
