! Name: $Id: ama1c_read_f.F 467 2015-11-17 10:27:43Z frna $
!
! Purpose: Read AMSU-A l1c file and pass information back to C calling routine.
!          Subroutines AMA1C_READ_HEADER and AMA1C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    16/09/2014  N C Atkinson
!  1.1    16/11/2015  Add coldnedt, warmnedt
!-----------------------------------------------------------------------

      SUBROUTINE AMA1C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_vnantennacorr, h_tempradcnv, h_wmosatid,
     & h_coldnedt, h_warmnedt)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 AMSUA_CHANS
      PARAMETER (AMSUA_CHANS=15)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_vnantennacorr
      INTEGER*4     h_tempradcnv(3,AMSUA_CHANS)
      INTEGER*4     h_wmosatid
      INTEGER*4     h_coldnedt(AMSUA_CHANS)
      INTEGER*4     h_warmnedt(AMSUA_CHANS)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "ama1c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open AMSU-A 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=ama_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read AMSU-A header'
      CALL IOA1C("READ HEADER",UNIT_1C,1)

      h_scnlin = ama1c_h_scnlin
      h_site = ama1c_h_site
      h_1bsite = ama1c_h_1bsite
      h_versnb = ama1c_h_versnb
      h_satid = ama1c_h_satid
      h_instrument = ama1c_h_instrument
      h_satht = ama1c_h_satht
      h_period = ama1c_h_period
      h_startorbit = ama1c_h_startorbit
      h_startdatayr = ama1c_h_startdatayr
      h_startdatady = ama1c_h_startdatady
      h_startdatatime = ama1c_h_startdatatime
      h_endorbit = ama1c_h_endorbit
      h_enddatayr = ama1c_h_enddatayr
      h_enddatady = ama1c_h_enddatady
      h_enddatatime = ama1c_h_enddatatime
      h_scnlin = ama1c_h_scnlin
      h_misscnlin = ama1c_h_misscnlin
      h_vnantennacorr = ama1c_h_vnantennacorr
      do ichan=1,AMSUA_CHANS
        do i=1,3
          h_tempradcnv(i,ichan) = ama1c_h_tempradcnv(i,ichan)
        enddo
        h_coldnedt(ichan) = ama1c_h_coldnedt(ichan)
        h_warmnedt(ichan) = ama1c_h_warmnedt(ichan)
      enddo
      h_wmosatid = ama1c_h_wmosatid

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE AMA1C_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, chanqual, instrtemp1, instrtemp2,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  scalti, btemps, dataqual, coldnedt, warmnedt)
      IMPLICIT NONE

! Parameters
      INTEGER*4 AMSUA_CHANS
      PARAMETER (AMSUA_CHANS=15)
      INTEGER*4 AMSUA_SPOTS
      PARAMETER (AMSUA_SPOTS=30)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 chanqual(AMSUA_CHANS,h_scnlin)
      INTEGER*4 instrtemp1(h_scnlin)
      INTEGER*4 instrtemp2(h_scnlin)
      INTEGER*4 latitude(AMSUA_SPOTS,h_scnlin)
      INTEGER*4 longitude(AMSUA_SPOTS,h_scnlin)
      INTEGER*4 satzen(AMSUA_SPOTS,h_scnlin)
      INTEGER*4 satazi(AMSUA_SPOTS,h_scnlin)
      INTEGER*4 solzen(AMSUA_SPOTS,h_scnlin)
      INTEGER*4 solazi(AMSUA_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 btemps(AMSUA_CHANS,AMSUA_SPOTS,h_scnlin)
      INTEGER*4 dataqual(AMSUA_SPOTS,h_scnlin)
      INTEGER*4 coldnedt(AMSUA_CHANS,h_scnlin)
      INTEGER*4 warmnedt(AMSUA_CHANS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot

#include "inparms.h"
#include "ama1c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read AMSU-A data records'
      DO iscan=1,h_scnlin
        CALL IOA1C("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = ama1c_scnlin
        scnlinyr(iscan) = ama1c_scnlinyr
        scnlindy(iscan) = ama1c_scnlindy
        scnlintime(iscan) = ama1c_scnlintime
        qualind(iscan) = ama1c_qualind
        scanqual(iscan) = ama1c_scanqual
        instrtemp1(iscan) = ama1c_instrtemp1
        instrtemp2(iscan) = ama1c_instrtemp2
        scalti(iscan) = ama1c_scalti
        DO ichan=1,AMSUA_CHANS
          chanqual(ichan,iscan) = ama1c_chanqual(ichan)
          coldnedt(ichan,iscan) = ama1c_coldnedt(ichan)
          warmnedt(ichan,iscan) = ama1c_warmnedt(ichan)
        ENDDO
        DO ispot=1,AMSUA_SPOTS
          DO ichan=1,AMSUA_CHANS
            btemps(ichan,ispot,iscan) = ama1c_btemps(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = ama1c_latlon(1,ispot)
          longitude(ispot,iscan) = ama1c_latlon(2,ispot)
          satzen(ispot,iscan) = ama1c_angles(1,ispot)
          satazi(ispot,iscan) = ama1c_angles(2,ispot)
          solzen(ispot,iscan) = ama1c_angles(3,ispot)
          solazi(ispot,iscan) = ama1c_angles(4,ispot)
          dataqual(ispot,iscan) = ama1c_dataqual(ispot)
        ENDDO
      ENDDO

      CALL IOA1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
