/* 
! Name: $Id: ama1d_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $
!
! Purpose: Subroutine to write AMSUB/MHS 1d data to HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      26/09/2014  N C Atkinson
!  1.1      04/10/2017  Change attribute "period_minutes" to "period_seconds"
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
 
#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <string.h>

#include "add_hdf5_attributes.h"
#include "ama1d_c.h"

int ama1d_to_hdf5_write(char *filename, int clevel){ 

  char *hdf5_encoder_version = "$Id: ama1d_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $";
  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[3];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         ichan;
  float       amsu_wavenumber[AMSU_CHANS];
  float       amsu_temprad_offset[AMSU_CHANS];
  float       amsu_temprad_slope[AMSU_CHANS];
  
  int         fill=-999999;
  float       btscale=0.01;
  float       latlonscale=0.0001;
  float       anglescale=0.01;
  float       altiscale=0.1;
  float       surfheightscale=1.0;

  for (ichan=0; ichan<AMSU_CHANS; ichan++){
    amsu_wavenumber[ichan] = 0.000001*h_amsPlanck[ichan][0];
    amsu_temprad_offset[ichan] = 0.000001*h_amsPlanck[ichan][1];
    amsu_temprad_slope[ichan] = 0.000001*h_amsPlanck[ichan][2];
  }

/* Create the HDF5 file */

  printf("Creating %s\n",filename);
  file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

/* Global attributes */

  attdims[0] = 1;

  add_string_attribute(file_id, "filename", filename);
  add_string_attribute(file_id, "site", h_site);
  add_string_attribute(file_id, "1bsite", h_1bsite);
  add_string_attribute(file_id, "hdf5_encoder_software_version", hdf5_encoder_version);
  add_int_attribute(file_id, "satid", &h_satid, attdims);
  add_int_attribute(file_id, "grid", &h_grid, attdims);
  add_string_attribute(file_id, "grid_key", "5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B; 12=MHS");
  add_int_attribute(file_id, "satht_hectometres", &h_satht, attdims);
  add_int_attribute(file_id, "period_seconds", &h_period, attdims);
  add_int_attribute(file_id, "startorbit", &h_startorbit, attdims);
  add_int_attribute(file_id, "startdatayr", &h_startdatayr, attdims);
  add_int_attribute(file_id, "startdatady", &h_startdatady, attdims);
  add_int_attribute(file_id, "startdatatime_ms", &h_startdatatime, attdims);
  add_int_attribute(file_id, "endorbit", &h_endorbit, attdims);
  add_int_attribute(file_id, "enddatayr", &h_enddatayr, attdims);
  add_int_attribute(file_id, "enddatady", &h_enddatady, attdims);
  add_int_attribute(file_id, "enddatatime_ms", &h_enddatatime, attdims);
  add_int_attribute(file_id, "scnlin", &h_scnlin, attdims);
  add_int_attribute(file_id, "misscnlin", &h_misscnlin, attdims);
  add_int_attribute(file_id, "atovppvn", &h_atovppvn, attdims);
  add_int_attribute(file_id, "instruments", &h_instruments, attdims);
  add_string_attribute(file_id, "instruments_key", "bit0=HIRS, bit1=MSU, bit3=AMSU-A, bit4=AMSU-B/MHS, bit5=AVHRR");
  add_int_attribute(file_id, "Aantenna", &h_Aantenna, attdims);
  add_int_attribute(file_id, "wmosatid", &h_wmosatid, attdims);
  add_int_attribute(file_id, "dT", &h_dT, attdims);
  add_int_attribute(file_id, "mapBTmode", &h_mapBTmode, attdims);
  add_int_attribute(file_id, "Acorrvn", &h_Acorrvn, attdims);
  add_int_attribute(file_id, "Apcpvn", &h_Apcpvn, attdims);
  add_int_attribute(file_id, "Bcorrvn", &h_Bcorrvn, attdims);
  add_int_attribute(file_id, "Bpcpvn", &h_Bpcpvn, attdims);
  add_int_attribute(file_id, "Bantenna", &h_Bantenna, attdims);
  add_int_attribute(file_id, "Ascat", &h_Ascat, attdims);
  add_int_attribute(file_id, "ACrosby", &h_ACrosby, attdims);
  add_int_attribute(file_id, "AB89", &h_AB89, attdims);

  attdims[0] = AMSU_CHANS;
  add_float_attribute(file_id, "AMSU_wavenumber", amsu_wavenumber, attdims);
  add_float_attribute(file_id, "AMSU_temprad_offset", amsu_temprad_offset, attdims);
  add_float_attribute(file_id, "AMSU_temprad_slope", amsu_temprad_slope, attdims);
  attdims[0] = 1;

/* Group /Data */

  group_id = H5Gcreate(file_id, "/Data", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets with one value per scan */

  rank = 1;
  dims[0] = h_scnlin;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/scnlin", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlin);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinyr", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinyr);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlindy", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlindy);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlintime", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlintime);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/qualind", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, qualind);
  status = add_string_attribute(dataset_id, "Description", "Quality indicator: see AAPP Formats document");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scanqual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scanqual);
  status = add_string_attribute(dataset_id, "Description", "Scan line quality: see AAPP Formats document");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/instrtemp1", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, instrtemp1);
  status = add_string_attribute(dataset_id, "Description", "AMSU-A1 instrument temperature");
  status = add_string_attribute(dataset_id, "Units", "K");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/instrtemp2", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, instrtemp2);
  status = add_string_attribute(dataset_id, "Description", "AMSU-A2 instrument temperature");
  status = add_string_attribute(dataset_id, "Units", "K");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/instrtempb", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, instrtempb);
  status = add_string_attribute(dataset_id, "Description", "AMSUB/MHS instrument temperature");
  status = add_string_attribute(dataset_id, "Units", "K");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scalti", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scalti);
  status = add_string_attribute(dataset_id, "Description", "Satellite altitude");
  status = add_string_attribute(dataset_id, "Units", "km");
  status = add_float_attribute(dataset_id, "Scale", &altiscale, attdims);
  H5Dclose(dataset_id);

/* Dataset btemps */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = AMA_SPOTS;
  dims[2] = AMSU_CHANS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/btemps", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, btemps);
  status = add_string_attribute(dataset_id, "Units", "K");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset prepro */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = AMA_SPOTS;
  dims[2] = 7;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/prepro", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, prepro);
  status = add_string_attribute(dataset_id, "Description", "Pre-processing output, see AAPP documentation");
  status = add_string_attribute(dataset_id, "Order_prepro",
   "1: QC word, 2: AMSU surface type, 3: Cost fn from PPASURF, 4: Scattering index, 5: Precip prob, 6-7: Spare");
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

  H5Pclose(plist);                    
  H5Gclose(group_id);

/* Group /Geolocation */

  group_id = H5Gcreate(file_id, "/Geolocation", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets latitude, longitude, surfheight, surftype */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = AMA_SPOTS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Geolocation/Latitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Longitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Surface_height", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, surfheight);
  status = add_string_attribute(dataset_id, "Description", "Surface height from atlas");
  status = add_string_attribute(dataset_id, "Units", "m");
  status = add_float_attribute(dataset_id, "Scale", &surfheightscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Surface_type", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, surftype);
  status = add_string_attribute(dataset_id, "Description", "Surface type from atlas: 0=sea, 1=mixed, 2=land");
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* Satellite and solar angles */

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* close the group and file */

  H5Sclose(dataspace_id);
  H5Pclose(plist);                    
  H5Gclose(group_id);
  H5Fclose(file_id);
}

#else

int hrs1c_to_hdf5_write(char *filename, int clevel){

  printf ("Error: unable to write HDF-5\n");
  exit(1);

}
#endif
