! Name: $Id: amb1c_write_f.F 609 2019-01-11 14:58:10Z frna $
!
! Purpose: Write AMSU-B/MHS l1c file using information from C calling routine.
!          Subroutines AMB1C_WRITE_HEADER and AMB1C_WRITE_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2019, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    04/01/2019  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE AMB1C_WRITE_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_vnantennacorr, h_tempradcnv, h_wmosatid,
     & h_coldnedt, h_warmnedt)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 AMSUB_CHANS
      PARAMETER (AMSUB_CHANS=5)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*4   h_site
      CHARACTER*4   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_vnantennacorr
      INTEGER*4     h_tempradcnv(3,AMSUB_CHANS)
      INTEGER*4     h_wmosatid
      INTEGER*4     h_coldnedt(AMSUB_CHANS)
      INTEGER*4     h_warmnedt(AMSUB_CHANS)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "amb1c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open AMSU-B/MHS 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="NEW", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=amb_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=amb_1crecl/rldiv, IOSTAT=errStatus )
        IF (errStatus.NE.0) THEN
          OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="UNKNOWN", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=amb_1crecl/rldiv, IOSTAT=errStatus )
          IF (errStatus.NE.0) THEN
            write(*,*)'Cannot open file ',filename(1:filename_length)
            call aappexit(1)
          ENDIF
        ENDIF
      ENDIF

      write(*,*)'Write AMSU-B/MHS header'
      amb1c_h_scnlin = h_scnlin
      amb1c_h_site = h_site
      amb1c_h_cfill1 = ' '
      amb1c_h_1bsite = h_1bsite
      amb1c_h_cfill2 = ' '
      amb1c_h_versnb = amb_1cvnnb
      amb1c_h_versyr = amb_1cvnyr
      amb1c_h_versdy = amb_1cvndy
      amb1c_h_hdrcnt = 1
      amb1c_h_satid = h_satid
      amb1c_h_instrument = h_instrument
      amb1c_h_satht = h_satht
      amb1c_h_period = h_period
      amb1c_h_startorbit = h_startorbit 
      amb1c_h_startdatayr = h_startdatayr
      amb1c_h_startdatady = h_startdatady
      amb1c_h_startdatatime = h_startdatatime
      amb1c_h_endorbit = h_endorbit
      amb1c_h_enddatayr = h_enddatayr
      amb1c_h_enddatady = h_enddatady
      amb1c_h_enddatatime = h_enddatatime
      amb1c_h_scnlin = h_scnlin
      amb1c_h_misscnlin = h_misscnlin
      amb1c_h_vnantennacorr = h_vnantennacorr
      do ichan=1,AMSUB_CHANS
        do i=1,3
          amb1c_h_tempradcnv(i,ichan) = h_tempradcnv(i,ichan) 
        enddo
        amb1c_h_coldnedt(ichan) = h_coldnedt(ichan)
        amb1c_h_warmnedt(ichan) = h_warmnedt(ichan)
      enddo
      amb1c_h_wmosatid = h_wmosatid

      CALL IOB1C("WRITE HEADER",UNIT_1C,1)

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE AMB1C_WRITE_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, chanqual, instrtemp,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  scalti, btemps, dataqual, coldnedt, warmnedt)
      IMPLICIT NONE

! Parameters
      INTEGER*4 AMSUB_CHANS
      PARAMETER (AMSUB_CHANS=5)
      INTEGER*4 AMSUB_SPOTS
      PARAMETER (AMSUB_SPOTS=90)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 chanqual(AMSUB_CHANS,h_scnlin)
      INTEGER*4 instrtemp(h_scnlin)
      INTEGER*4 latitude(AMSUB_SPOTS,h_scnlin)
      INTEGER*4 longitude(AMSUB_SPOTS,h_scnlin)
      INTEGER*4 satzen(AMSUB_SPOTS,h_scnlin)
      INTEGER*4 satazi(AMSUB_SPOTS,h_scnlin)
      INTEGER*4 solzen(AMSUB_SPOTS,h_scnlin)
      INTEGER*4 solazi(AMSUB_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 btemps(AMSUB_CHANS,AMSUB_SPOTS,h_scnlin)
      INTEGER*4 dataqual(AMSUB_SPOTS,h_scnlin)
      INTEGER*4 coldnedt(AMSUB_CHANS,h_scnlin)
      INTEGER*4 warmnedt(AMSUB_CHANS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot

#include "inparms.h"
#include "amb1c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Write AMSU-B/MHS data records'
      DO iscan=1,h_scnlin
        amb1c_scnlin = scnlin(iscan)
        amb1c_scnlinyr = scnlinyr(iscan)
        amb1c_scnlindy = scnlindy(iscan)
        amb1c_scnlintime = scnlintime(iscan)
        amb1c_qualind = qualind(iscan)
        amb1c_scanqual = scanqual(iscan)
        amb1c_instrtemp = instrtemp(iscan)
        amb1c_scalti = scalti(iscan)
        DO ichan=1,AMSUB_CHANS
          amb1c_chanqual(ichan) = chanqual(ichan,iscan)
          amb1c_coldnedt(ichan) = coldnedt(ichan,iscan)
          amb1c_warmnedt(ichan) = warmnedt(ichan,iscan)
        ENDDO
        DO ispot=1,AMSUB_SPOTS
          DO ichan=1,AMSUB_CHANS
            amb1c_btemps(ichan,ispot) = btemps(ichan,ispot,iscan)
          ENDDO
          amb1c_latlon(1,ispot) = latitude(ispot,iscan)
          amb1c_latlon(2,ispot) = longitude(ispot,iscan)
          amb1c_angles(1,ispot) = satzen(ispot,iscan)
          amb1c_angles(2,ispot) = satazi(ispot,iscan)
          amb1c_angles(3,ispot) = solzen(ispot,iscan)
          amb1c_angles(4,ispot) = solazi(ispot,iscan)
          amb1c_dataqual(ispot) = dataqual(ispot,iscan)
        ENDDO
        CALL IOB1C("WRITE DATA",UNIT_1C,iscan+1)
      ENDDO

      CALL IOB1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
