! Name: $Id: atms1c_read_f.F 456 2015-07-27 14:58:53Z frna $
!
! Purpose: Read ATMS l1c file and pass information back to C calling routine.
!          Subroutines ATMS1C_READ_HEADER and ATMS1C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    10/07/2015  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE ATMS1C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_vnantennacorr, h_tempradcnv, 
     & h_bandwidth, h_nchan, h_nfov, h_nchanused,
     & h_wmosatid, h_centre, h_subcentre, h_beamversion,
     & h_beamwidth, h_cutoff, h_nxaverage, h_nyaverage )

      IMPLICIT NONE
      
#include "inparms.h"
#include "atms1c.h"
#include "reclopt.h"

! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 ATMS_CHANS
      PARAMETER (ATMS_CHANS=atms_1cnchan)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_vnantennacorr
      INTEGER*4     h_tempradcnv(3,ATMS_CHANS)
      INTEGER*4     h_bandwidth(ATMS_CHANS)
      INTEGER*4     h_nchan
      INTEGER*4     h_nfov
      INTEGER*4     h_nchanused
      INTEGER*4     h_wmosatid
      INTEGER*4     h_centre
      INTEGER*4     h_subcentre
      INTEGER*4     h_beamversion
      INTEGER*4     h_beamwidth(ATMS_CHANS)
      INTEGER*4     h_cutoff(ATMS_CHANS)
      INTEGER*4     h_nxaverage(ATMS_CHANS)
      INTEGER*4     h_nyaverage(ATMS_CHANS)

! Local:
      INTEGER errStatus, i, ichan

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open ATMS 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=atms_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read ATMS header'
      CALL IOAT1C("READ HEADER",UNIT_1C,1)

      h_scnlin = atms1c_h_scnlin
      h_site = atms1c_h_site
      h_1bsite = atms1c_h_1bsite
      h_versnb = atms1c_h_versnb
      h_satid = atms1c_h_satid
      h_instrument = atms1c_h_instrument
      h_satht = atms1c_h_satht
      h_period = atms1c_h_period
      h_startorbit = atms1c_h_startorbit
      h_startdatayr = atms1c_h_startdatayr
      h_startdatady = atms1c_h_startdatady
      h_startdatatime = atms1c_h_startdatatime
      h_endorbit = atms1c_h_endorbit
      h_enddatayr = atms1c_h_enddatayr
      h_enddatady = atms1c_h_enddatady
      h_enddatatime = atms1c_h_enddatatime
      h_scnlin = atms1c_h_scnlin
      h_misscnlin = atms1c_h_misscnlin
      h_vnantennacorr = atms1c_h_vnantennacorr
      do ichan=1,ATMS_CHANS
        do i=1,3
          h_tempradcnv(i,ichan) = atms1c_h_tempradcnv(i,ichan)
        enddo
        h_bandwidth(ichan) = atms1c_h_bandwidth(ichan)
        h_beamwidth(ichan) = atms1c_h_beamwidth(ichan)
        h_cutoff(ichan) = atms1c_h_cutoff(ichan)
        h_nxaverage(ichan) = atms1c_h_nxaverage(ichan)
        h_nyaverage(ichan) = atms1c_h_nyaverage(ichan)
      enddo
      h_nchan = atms1c_h_nchan
      h_nfov = atms1c_h_nfov
      h_nchanused = atms1c_h_nchanused
      h_wmosatid = atms1c_h_wmosatid
      h_centre = atms1c_h_centre
      h_subcentre = atms1c_h_subcentre
      h_beamversion = atms1c_h_beamversion

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE ATMS1C_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  orbit, granulequal, scanqual, geolocqual, chanqual,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  scalti, nedtcold, nedtwarm, anttemps, btemps )

      IMPLICIT NONE

#include "inparms.h"
#include "atms1c.h"

! Parameters
      INTEGER*4 ATMS_CHANS
      PARAMETER (ATMS_CHANS=atms_1cnchan)
      INTEGER*4 ATMS_SPOTS
      PARAMETER (ATMS_SPOTS=atms_1cnfov)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(ATMS_SPOTS,h_scnlin)
      INTEGER*4 orbit(h_scnlin)
      INTEGER*4 granulequal(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 geolocqual(h_scnlin)
      INTEGER*4 chanqual(ATMS_CHANS,h_scnlin)
      INTEGER*4 latitude(ATMS_SPOTS,h_scnlin)
      INTEGER*4 longitude(ATMS_SPOTS,h_scnlin)
      INTEGER*4 satzen(ATMS_SPOTS,h_scnlin)
      INTEGER*4 solzen(ATMS_SPOTS,h_scnlin)
      INTEGER*4 satazi(ATMS_SPOTS,h_scnlin)
      INTEGER*4 solazi(ATMS_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 nedtcold(ATMS_CHANS,h_scnlin)
      INTEGER*4 nedtwarm(ATMS_CHANS,h_scnlin)
      INTEGER*4 anttemps(ATMS_CHANS,ATMS_SPOTS,h_scnlin)
      INTEGER*4 btemps(ATMS_CHANS,ATMS_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read ATMS data records'
      DO iscan=1,h_scnlin
        CALL IOAT1C("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = atms1c_scnlin
        scnlinyr(iscan) = atms1c_scnlinyr
        scnlindy(iscan) = atms1c_scnlindy
        orbit(iscan) = atms1c_orbit
        granulequal(iscan) = atms1c_granulequal
        scanqual(iscan) = atms1c_scanqual
        geolocqual(iscan) = atms1c_geolocqual
        scalti(iscan) = atms1c_scalti
        DO ichan=1,ATMS_CHANS
          chanqual(ichan,iscan) = atms1c_chanqual(ichan)
          nedtcold(ichan,iscan) = atms1c_nedtcold(ichan)
          nedtwarm(ichan,iscan) = atms1c_nedtwarm(ichan)
        ENDDO
        DO ispot=1,ATMS_SPOTS
          scnlintime(ispot,iscan) = atms1c_scnlintime(ispot)
          DO ichan=1,ATMS_CHANS
            anttemps(ichan,ispot,iscan) = atms1c_anttemps(ichan,ispot)
            btemps(ichan,ispot,iscan) = atms1c_btemps(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = atms1c_latlon(1,ispot)
          longitude(ispot,iscan) = atms1c_latlon(2,ispot)
          satzen(ispot,iscan) = atms1c_angles(1,ispot)
          satazi(ispot,iscan) = atms1c_angles(2,ispot)
          solzen(ispot,iscan) = atms1c_angles(3,ispot)
          solazi(ispot,iscan) = atms1c_angles(4,ispot)
        ENDDO
      ENDDO

      CALL IOAT1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
