! Name: $Id: atms_sdr_out.F 577 2018-01-15 13:58:49Z frna $
!
! Purpose: Fill ATMS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    13/10/2010  Original code. (N.C.Atkinson)
!  1.1    20/08/2012  Read environment variables: ATMS_ORBIT to over-write orbit
!                     number and ATMS_USE_ATEMPS (=Y) to over-write brightness
!                     temp with antenna temp.
!  1.2    25/09/2012  Set atms1c_h_centre and atms1c_h_subcentre
!  1.3    13/07/2015  Set atms1c_h_vnantennacorr to missing
!  1.4    12/12/2016  Add Software_Version
!  1.5    17/03/2017  Correct a -0.005K bias in btemps and atemps rounding
!  1.6    23/10/2017  Need last spot to have valid time, as well as first.
!  1.7    21/11/2017  Add wmosatid for JPSS-1 and JPSS-2
!
!-------------------------------------------------------------------------------

      SUBROUTINE ATMS_SDR_OUT (NScan, BeamTime_millisec, BT_real,
     & AT_real, 
     & NEdTCold, NEdTWarm, Latitude, Longitude, SatelliteAzimuthAngle,
     & SatelliteZenithAngle, SolarAzimuthAngle, SolarZenithAngle,
     & SCPosition, StartOrbit, EndOrbit, orbit_numbers, granulequal,
     & scanqual, chanqual, geolocqual,
     & AggregateBeginningDate, AggregateEndingDate, Mission_Name,
     & Software_Version )
     
      IMPLICIT NONE

      INTEGER   ATMS_GRANULES, ATMS_SCANS_PER_GRANULE, ATMS_SPOTS,
     &   ATMS_CHANS, ATMS_SCANS
      PARAMETER (ATMS_GRANULES=30)   !Same as in atms_sdr.h
      PARAMETER (ATMS_SCANS_PER_GRANULE=12)
      PARAMETER (ATMS_SPOTS=96)
      PARAMETER (ATMS_CHANS=22)
      PARAMETER (ATMS_SCANS=ATMS_SCANS_PER_GRANULE*ATMS_GRANULES)

! Arguments: 
!   SDR

      REAL NEdTCold(ATMS_CHANS,ATMS_SCANS)
      REAL NEdTWarm(ATMS_CHANS,ATMS_SCANS)
      CHARACTER*4 Mission_Name
      CHARACTER*16 Software_Version
      CHARACTER*9 AggregateBeginningDate
      CHARACTER*9 AggregateEndingDate
      INTEGER*4   StartOrbit
      INTEGER*4   EndOrbit

!   Geolocation

      REAL Latitude(ATMS_SPOTS,ATMS_SCANS);
      REAL Longitude(ATMS_SPOTS,ATMS_SCANS);
      REAL SatelliteAzimuthAngle(ATMS_SPOTS,ATMS_SCANS);
      REAL SatelliteZenithAngle(ATMS_SPOTS,ATMS_SCANS);
      REAL SolarAzimuthAngle(ATMS_SPOTS,ATMS_SCANS);
      REAL SolarZenithAngle(ATMS_SPOTS,ATMS_SCANS);
      REAL SCPosition(3,ATMS_SCANS);
      INTEGER*4 geolocqual(ATMS_SCANS)

!   Derived

      REAL      BT_real(ATMS_CHANS,ATMS_SPOTS,ATMS_SCANS)
      REAL      AT_real(ATMS_CHANS,ATMS_SPOTS,ATMS_SCANS)
      INTEGER*4 BeamTime_millisec(ATMS_SPOTS,ATMS_SCANS)
      INTEGER*4 NScan
      INTEGER*4 granulequal(ATMS_SCANS)
      INTEGER*4 scanqual(ATMS_SCANS)
      INTEGER*4 orbit_numbers(ATMS_SCANS)
      INTEGER*4 chanqual(ATMS_CHANS,ATMS_SCANS)

! Local variables

      INTEGER Nscan_use,iscan,ispot,ichan,unit,J,id
      INTEGER startyear, startmonth, startday
      INTEGER endyear, endmonth, endday
      REAL    xysq,z,lat,semiMinor,r_sat,r_ellipse
      CHARACTER*80 line
      REAL    channel_info(5,22)
      DATA    channel_info /110*0/
      CHARACTER*10 CATMS_ORBIT
      INTEGER atms_orbit
      CHARACTER*1 ATMS_USE_ATEMPS
      CHARACTER*5   ORIGINATING_CENTRE
      CHARACTER*5   SUB_CENTRE
      INTEGER IORIGCENTRE
      INTEGER ISUBCENTRE

! Functions

      INTEGER IDAYOFYEAR

! Include
#include "atms1c.h"
#include "inparms.h"
#include "constants_preproc.h"

! End of declarations
!-----------------------------------------------------------------------

      IF (Nscan .LE. 0) THEN
        WRITE(*,*)'Error in atms_sdr_out: Nscan=',Nscan
	CALL aappexit(1)
      ENDIF

      IORIGCENTRE = 255     !255=Missing
      ISUBCENTRE = 0
      CALL GETENV('ORIGINATING_CENTRE',ORIGINATING_CENTRE)
      READ(ORIGINATING_CENTRE,*,ERR=10,END=10) IORIGCENTRE
   10 CALL GETENV('SUB_CENTRE',SUB_CENTRE)
      READ(SUB_CENTRE,*,ERR=11,END=11) ISUBCENTRE
   11 CONTINUE

!Check whether all the scans have valid time.
      Nscan_use = 0
      DO iscan=1,Nscan
        IF (BeamTime_millisec(1,iscan).GT.0.AND.
     &      BeamTime_millisec(ATMS_SPOTS,iscan).GT.0) Nscan_use=iscan
      ENDDO    

      IF (Nscan_use .LE. 0) THEN
        WRITE(*,*)'Error in atms_sdr_out: no scans are usable'
	CALL aappexit(1)
      ENDIF

!Check whether the orbit number has been specified externally.
      atms_orbit = 0
      CALL GETENV('ATMS_ORBIT',CATMS_ORBIT)
      IF (CATMS_ORBIT(1:1).NE.' ') THEN
        read(CATMS_ORBIT,*) atms_orbit
        write(*,*)'Orbit number specified as:',atms_orbit
      ENDIF

!Check whether antenna temps are to be used instead of brightness temps
      CALL GETENV('ATMS_USE_ATEMPS',ATMS_USE_ATEMPS)

!Open the output file
      unit = 10
      CALL IOAT1C("OPEN/WRITE",unit,0)

!Read the general satellite info from the fdf.dat file on unit 20
      CALL INFDF (20, 0)      

      IF (Mission_Name(1:3) .EQ. 'NPP') THEN
        atms1c_h_wmosatid = 224
      ELSE IF (Mission_Name(2:3) .EQ. '01' .or. 
     &         Mission_Name(2:3) .EQ. '20') THEN
        atms1c_h_wmosatid = 225    !JPSS-1, NOAA-20
      ELSE IF (Mission_Name(2:3) .EQ. '02' .or. 
     &         Mission_Name(2:3) .EQ. '21') THEN
        atms1c_h_wmosatid = 226    !JPSS-2, NOAA-21
      ELSE
        atms1c_h_satid = MDI
        atms1c_h_wmosatid = MDI
      ENDIF

!Read the ATMS channel info on unit 21
      OPEN (21,form='formatted',status='old')
      id = 0
      DO while (id .NE. atms1c_h_wmosatid)
        line="     "
        DO while (line(1:5) .ne. "# WMO")
          read(21,'(A)',END=100) line
        ENDDO
        read(21,*) id
        IF (id .eq. atms1c_h_wmosatid) THEN
          read(21,*) line
          read(21,*) channel_info
        ENDIF
      ENDDO
  100 CLOSE (21)

      semiMinor = semiMajor - semiDiff

      atms1c_h_centre = IORIGCENTRE
      atms1c_h_subcentre = ISUBCENTRE
      atms1c_h_instrument = 621      !ATMS
      IF (atms_orbit .gt. 0) THEN
        atms1c_h_startorbit = atms_orbit    !Specified externally
        atms1c_h_endorbit = atms_orbit
      ELSE
        atms1c_h_startorbit = StartOrbit
        atms1c_h_endorbit = EndOrbit
      ENDIF
      atms1c_h_scnlin = NScan_use
      atms1c_h_misscnlin = 0
      atms1c_h_vnantennacorr = MDI
      atms1c_h_nchan = atms_1cnchan
      atms1c_h_nfov = atms_1cnfov
      atms1c_h_nchanused = atms_1cnchan
      atms1c_h_recwords = array_size_atms1c
      atms1c_h_hdrcnt = 1
      atms1c_h_versnb = atms_1cvnnb
      atms1c_h_versyr = atms_1cvnyr
      atms1c_h_versdy = atms_1cvndy
      DO ichan=1,ATMS_CHANS
        atms1c_h_tempradcnv(1,ichan) = 
     &    channel_info(2,ichan)*1.0E13/C_light   !wavenumber*1E6
        atms1c_h_tempradcnv(2,ichan) = channel_info(4,ichan)*1.0E6  !offset
        atms1c_h_tempradcnv(3,ichan) = channel_info(5,ichan)*1.0E6  !slope
        atms1c_h_bandwidth(ichan) = channel_info(3,ichan)*1.0E6     !GHz*1E6
      ENDDO

      READ (AggregateBeginningDate,'(I4,I2,I2)') startyear, startmonth,
     &        startday
      atms1c_h_startdatayr = startyear
      atms1c_h_startdatady = IDAYOFYEAR(startyear, startmonth, startday)
      atms1c_h_startdatatime = BeamTime_millisec(1,1)

      READ (AggregateEndingDate,'(I4,I2,I2)') endyear, endmonth, endday
      atms1c_h_enddatayr = endyear
      atms1c_h_enddatady = IDAYOFYEAR(endyear, endmonth, endday)
      atms1c_h_enddatatime = BeamTime_millisec(ATMS_SPOTS,NScan_use)

! Get nominal height and period from fdf file
      DO J=1,MaxSats
        IF (WMOid(J).EQ.atms1c_h_wmosatid) THEN
          atms1c_h_satht = SatHt(J)*10   !nominal ht, km*10
          atms1c_h_period = OrbitPd(J)   !period, seconds
          atms1c_h_satid = NOAAid(J)   !Satellite ID
        ENDIF
      ENDDO

      atms1c_h_software_version = Software_Version
      write(*,*) 'Software_Version:',atms1c_h_software_version

      CALL IOAT1C("WRITE HEADER",unit,1)
      
      DO iscan=1,NScan_use
        atms1c_scnlin = iscan
        IF (BeamTime_millisec(1,iscan) .GE. atms1c_h_startdatatime) THEN
          atms1c_scnlinyr = atms1c_h_startdatayr
          atms1c_scnlindy = atms1c_h_startdatady
        ELSE
          atms1c_scnlinyr = atms1c_h_enddatayr
          atms1c_scnlindy = atms1c_h_enddatady
        ENDIF
          
        DO ispot=1,ATMS_SPOTS
          atms1c_scnlintime(ispot) = BeamTime_millisec(ispot,iscan)
          atms1c_latlon(1,ispot) = 1E4*Latitude(ispot,iscan)
          atms1c_latlon(2,ispot) = 1E4*Longitude(ispot,iscan)
          atms1c_angles(1,ispot) = 1E2*SatelliteZenithAngle(ispot,iscan)
          atms1c_angles(2,ispot) =1E2*SatelliteAzimuthAngle(ispot,iscan)
          atms1c_angles(3,ispot) = 1E2*SolarZenithAngle(ispot,iscan)
          atms1c_angles(4,ispot) = 1E2*SolarAzimuthAngle(ispot,iscan)
        ENDDO
        DO ichan=1,ATMS_CHANS
          atms1c_nedtcold(ichan) = 1E4*NEdTCold(ichan,iscan)
          atms1c_nedtwarm(ichan) = 1E4*NEdTWarm(ichan,iscan)
          DO ispot=1,ATMS_SPOTS
            IF (BT_real(ichan,ispot,iscan).GT.0) THEN
              atms1c_btemps(ichan,ispot) =
     &            1E2*BT_real(ichan,ispot,iscan) + 0.5
            ELSE
              atms1c_btemps(ichan,ispot) = MDI
            ENDIF
            IF (AT_real(ichan,ispot,iscan).GT.0) THEN
              atms1c_anttemps(ichan,ispot) =
     &            1E2*AT_real(ichan,ispot,iscan) + 0.5
              IF (ATMS_USE_ATEMPS(1:1) .EQ. "Y") THEN
                atms1c_btemps(ichan,ispot) =
     &            1E2*AT_real(ichan,ispot,iscan) + 0.5
              ENDIF
            ELSE
              atms1c_anttemps(ichan,ispot) = MDI
            ENDIF
          ENDDO
          atms1c_chanqual(ichan) = chanqual(ichan,iscan)
        ENDDO
        
        atms1c_granulequal = granulequal(iscan)
        atms1c_scanqual = scanqual(iscan)
        atms1c_geolocqual = geolocqual(iscan)

        xysq = SCPosition(1,iscan)**2 + SCPosition(2,iscan)**2
        z = SCPosition(3,iscan)
        r_sat = sqrt(xysq + z**2)       !Centre of Earth to satellite
        lat = atan2(z,sqrt(xysq))
        r_ellipse = sqrt((semiMajor*cos(lat))**2 + 
     &                   (semiMinor*sin(lat))**2)
        atms1c_scalti = NINT((r_sat - r_ellipse)*0.01)    !km *10
        IF (atms_orbit .GT. 0) THEN
          atms1c_orbit = atms_orbit   !Specified externally 
        ELSE
          atms1c_orbit = orbit_numbers(iscan)
        ENDIF
        CALL IOAT1C("WRITE DATA",unit,iscan+1)
      ENDDO

      CALL IOAT1C("CLOSE",unit,0)

      RETURN
      END

!-----------------------------------------------------------------------

      FUNCTION IDAYOFYEAR(IY,IM,ID)
      INTEGER    IDAYOFYEAR
      INTEGER    IY,IM,ID
      INTEGER    NOLEAP(12), LEAP(12)
      DATA       NOLEAP /0,31,59,90,120,151,181,212,243,273,304,334/
      DATA       LEAP   /0,31,60,91,121,152,182,213,244,274,305,335/
      IF (MOD(IY,4).EQ.0) THEN         !OK up to 2100
        IDAYOFYEAR = LEAP(IM) + ID
      ELSE
        IDAYOFYEAR = NOLEAP(IM) + ID
      ENDIF
      RETURN
      END
