/*
! name: $Id: atms_sdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an ATMS SDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    12/10/2010  Original code. (N.C.Atkinson)
!  1.1    25/09/2012  Allow operation with TDR file only
!  1.2    25/02/2016  Bug fix: prevent negative time of day around midnight
!  1.3    12/12/2016  Add N_Software_Version
!  1.4    21/11/2017  Use attribute Platform_Short_Name instead of Mission_Name
!  1.5    11/04/2018  Compute taioffset from the first good time.
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "atms_sdr.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int atms_sdr_read(char *filename, char geo_char){
  
  int     igran,iscan,jscan,ispot,ichan;           /* Loop variables */
  hid_t   file_id, group_id, dataset_id, attr_id;  /* handles */
  hid_t   strtype;
  herr_t  status;
  int     hour, minute;
  double  second, secs;
  int     taioffset;
  long    taioffset_ms;
  char    name[80];
  int     file_is_TDR = 0;
  long long int firstBeamTime = 0;     
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannont open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */

/****** Read the top-level attributes ******/

  status = H5Tset_size (strtype, 4);   /* Set the length of the string */
  readattribute("Platform_Short_Name",file_id,strtype,Mission_Name);
  status = H5Tset_size (strtype, 79);
  if (geo_char == '/') {
    N_GEO_Ref[0] = '-';
    readattribute("N_GEO_Ref",file_id,strtype,N_GEO_Ref);
    if (N_GEO_Ref[0] == '-') {
      fprintf(stderr, "Error: N_GEO_Ref attribute not found: use -g option\n");
      exit(10);
    }
  }

/****** Open the group All_Data/ATMS-SDR_All ******/

  if (H5Lexists(file_id, "All_Data/ATMS-TDR_All", H5P_DEFAULT)) {
    fprintf(stderr,"Warning: Input file is TDR\n");
    file_is_TDR = 1;
  }
  if (file_is_TDR) group_id = H5Gopen(file_id, "All_Data/ATMS-TDR_All", H5P_DEFAULT);
  else             group_id = H5Gopen(file_id, "All_Data/ATMS-SDR_All", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group All_Data/ATMS-SDR_All\n");
    exit(10);
  }

/****** Read the data ******/

  readdataset("BeamTime", group_id, H5T_NATIVE_LLONG, BeamTime);
  if (file_is_TDR) {
    readdataset("AntennaTemperature", group_id, H5T_NATIVE_USHORT,
      BrightnessTemperature);
    readdataset("AntennaTemperatureFactors", group_id, H5T_NATIVE_FLOAT,
      BrightnessTemperatureFactors);
  } else {
    readdataset("BrightnessTemperature", group_id, H5T_NATIVE_USHORT,
      BrightnessTemperature);
    readdataset("BrightnessTemperatureFactors", group_id, H5T_NATIVE_FLOAT,
      BrightnessTemperatureFactors);
    readdataset("NEdTCold", group_id, H5T_NATIVE_FLOAT, NEdTCold);
    readdataset("NEdTWarm", group_id, H5T_NATIVE_FLOAT, NEdTWarm);
  }
  readdataset("QF1_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF1_GRAN_HEALTHSTATUS);
  readdataset("QF2_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF2_GRAN_HEALTHSTATUS);
  readdataset("QF3_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF3_GRAN_HEALTHSTATUS);
  readdataset("QF4_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF4_GRAN_HEALTHSTATUS);
  readdataset("QF5_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF5_GRAN_HEALTHSTATUS);
  readdataset("QF6_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF6_GRAN_HEALTHSTATUS);
  readdataset("QF7_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF7_GRAN_HEALTHSTATUS);
  readdataset("QF8_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF8_GRAN_HEALTHSTATUS);
  readdataset("QF9_GRAN_HEALTHSTATUS", group_id, H5T_NATIVE_UCHAR, QF9_GRAN_HEALTHSTATUS);
  readdataset("QF11_GRAN_QUADRATICCORRECTION", group_id, H5T_NATIVE_UCHAR,
    QF11_GRAN_QUADRATICCORRECTION);
  readdataset("QF12_SCAN_KAVPRTCONVERR", group_id, H5T_NATIVE_UCHAR,
    QF12_SCAN_KAVPRTCONVERR);
  readdataset("QF13_SCAN_WGPRTCONVERR", group_id, H5T_NATIVE_UCHAR,
    QF13_SCAN_WGPRTCONVERR);
  readdataset("QF14_SCAN_SHELFPRTCONVERR", group_id, H5T_NATIVE_UCHAR,
    QF14_SCAN_SHELFPRTCONVERR);
  readdataset("QF15_SCAN_KAVPRTTEMPLIMIT", group_id, H5T_NATIVE_UCHAR,
    QF15_SCAN_KAVPRTTEMPLIMIT);
  readdataset("QF16_SCAN_WGPRTTEMPLIMIT", group_id, H5T_NATIVE_UCHAR,
    QF16_SCAN_WGPRTTEMPLIMIT);
  readdataset("QF17_SCAN_KAVPRTTEMPCONSISTENCY", group_id, H5T_NATIVE_UCHAR,
    QF17_SCAN_KAVPRTTEMPCONSISTENCY);
  readdataset("QF18_SCAN_WGPRTTEMPCONSISTENCY", group_id, H5T_NATIVE_UCHAR,
    QF18_SCAN_WGPRTTEMPCONSISTENCY);
  readdataset("QF19_SCAN_ATMSSDR", group_id, H5T_NATIVE_UCHAR, QF19_SCAN_ATMSSDR);
  readdataset("QF20_ATMSSDR", group_id, H5T_NATIVE_UCHAR, QF20_ATMSSDR);
  readdataset("QF21_ATMSSDR", group_id, H5T_NATIVE_UCHAR, QF21_ATMSSDR);
  readdataset("QF22_ATMSSDR", group_id, H5T_NATIVE_UCHAR, QF22_ATMSSDR);

  H5Gclose(group_id);

/****** Open the group Data_Products/ATMS-SDR ******/

  if (file_is_TDR) group_id = H5Gopen(file_id, "Data_Products/ATMS-TDR", H5P_DEFAULT);
  else             group_id = H5Gopen(file_id, "Data_Products/ATMS-SDR", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group Data_Products/ATMS-SDR\n");
    exit(10);
  }

/****** Read the aggregate attribute data ******/

  if (file_is_TDR) dataset_id = H5Dopen(group_id, "ATMS-TDR_Aggr", H5P_DEFAULT);
  else             dataset_id = H5Dopen(group_id, "ATMS-SDR_Aggr", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open dataset ATMS-SDR_Aggr\n");
    exit(10);
  }

  status = H5Tset_size (strtype, 9);   /* Set the length of the string */
  readattribute("AggregateBeginningDate",dataset_id,strtype,AggregateBeginningDate);
  readattribute("AggregateEndingDate",dataset_id,strtype,AggregateEndingDate);
  status = H5Tset_size (strtype, 15);
  readattribute("AggregateBeginningTime",dataset_id,strtype,AggregateBeginningTime);
  readattribute("AggregateEndingTime",dataset_id,strtype,AggregateEndingTime);
  readattribute("AggregateBeginningOrbitNumber",dataset_id,H5T_NATIVE_LLONG,
    &AggregateBeginningOrbitNumber);
  readattribute("AggregateEndingOrbitNumber",dataset_id,H5T_NATIVE_LLONG,
    &AggregateEndingOrbitNumber);
  readattribute("AggregateNumberGranules",dataset_id,H5T_NATIVE_LONG,
    &AggregateNumberGranules);

  StartOrbit = AggregateBeginningOrbitNumber;
  EndOrbit = AggregateEndingOrbitNumber;
  NScan = AggregateNumberGranules*ATMS_SCANS_PER_GRANULE;
  
  H5Dclose(dataset_id);

/****** Read information from the granules ******/

  for (igran=0; igran<AggregateNumberGranules; igran++){
    if (file_is_TDR) sprintf(name,"ATMS-TDR_Gran_%d",igran);
    else             sprintf(name,"ATMS-SDR_Gran_%d",igran);
    
    dataset_id = H5Dopen(group_id, name, H5P_DEFAULT);
    if (dataset_id < 0) {
      fprintf(stderr,"Cannot open dataset %s\n",name);
      exit(10);
    }
    readattribute("N_Beginning_Orbit_Number", dataset_id, H5T_NATIVE_LLONG,
           &N_Beginning_Orbit_Number);
    status = H5Tset_size (strtype, 16);   /* Set the length of the string */
    readattribute("N_Software_Version", dataset_id, strtype, N_Software_Version);

    for (iscan=0;iscan<ATMS_SCANS_PER_GRANULE;iscan++) {
      jscan = igran*ATMS_SCANS_PER_GRANULE + iscan;
      orbit_numbers[jscan] = N_Beginning_Orbit_Number;
    }
    H5Dclose(dataset_id);
  }
    
  H5Gclose(group_id);

/****** Convert BTs ******/

  for (igran=0;igran<AggregateNumberGranules;igran++) {
    for (iscan=0;iscan<ATMS_SCANS_PER_GRANULE;iscan++) {
      jscan = igran*ATMS_SCANS_PER_GRANULE + iscan;
      
      for (ispot=0;ispot<ATMS_SPOTS;ispot++) {
        for (ichan=0;ichan<ATMS_CHANS;ichan++) {
          if (BrightnessTemperature[jscan][ispot][ichan] < 65528) {
            BT_real[jscan][ispot][ichan] = BrightnessTemperature[jscan][ispot][ichan]
               *BrightnessTemperatureFactors[igran*2] + BrightnessTemperatureFactors[igran*2 + 1];
          } else {
            BT_real[jscan][ispot][ichan] = RMDI;
          }
          if (file_is_TDR) AT_real[jscan][ispot][ichan] =
                           BT_real[jscan][ispot][ichan];
        }
      }
    }
  }
        

/****** Find the TAI offset and convert times ******/

  sscanf(AggregateBeginningTime,"%2d%2d%lf",&hour,&minute,&second);
  secs = hour*3600.0 + minute*60.0 + second;
  
  for (iscan=0;iscan<ATMS_SCANS;iscan++){      /* find the first valid time */
    for (ispot=0;ispot<ATMS_SPOTS;ispot++){
      if (BeamTime[iscan][ispot] > 0) {
        firstBeamTime = BeamTime[iscan][ispot];
        break;
      }
    }
    if (BeamTime[iscan][ispot] > 0) break;
  }

  if (firstBeamTime > 0) {
    taioffset = floor((firstBeamTime%86400000000LL)/1000000.0 - secs + 0.5);
    if (taioffset < 0) taioffset += 86400;
    taioffset_ms = taioffset*1000;
  } else {
    fprintf(stderr,"Warning: unable to determine TAI offset\n");
    taioffset_ms = 0;
  }
  printf("taioffset_ms = %d\n",taioffset_ms);

  for (iscan=0;iscan<ATMS_SCANS;iscan++) for (ispot=0;ispot<ATMS_SPOTS;ispot++){
    if (BeamTime[iscan][ispot] > 0) {
      BeamTime_millisec[iscan][ispot] = (BeamTime[iscan][ispot]%86400000000LL)/1000LL - taioffset_ms;
      if (BeamTime_millisec[iscan][ispot] < 0) BeamTime_millisec[iscan][ispot] += 86400000L;
    }
    else BeamTime_millisec[iscan][ispot] = MDI;
  }

/****** Process the quality flags ******/

/* Note that we define the bit order of the AAPP flags to be the same as the BUFR flags,
   with bit 1 being the most significant. In the SDR definition document (D34862-03E) 
   a "datum offset" of 0 means the least significant bit.
   In many cases one BUFR/AAPP flag corresponds to many SDR flags (e.g. where individual PRTs
   are flagged in the SDR). Here we set the BUFR/AAPP flag if one or more of the SDR flags is 
   non-zero. */

  for (iscan=0; iscan<NScan; iscan++){
    jscan = iscan/3;
    igran = iscan/ATMS_SCANS_PER_GRANULE;
    granulequal[iscan] = 0;
    
    if (QF11_GRAN_QUADRATICCORRECTION[igran]) granulequal[iscan]+=2;
    if (QF9_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=4;
    if (QF8_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=8;
    if (QF7_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=16;
    if (QF6_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=32;
    if (QF5_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=64;
    if (QF4_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=128;
    if (QF3_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=256;
    if (QF2_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=512;
    if (QF1_GRAN_HEALTHSTATUS[jscan]) granulequal[iscan]+=1024;

    scanqual[iscan] = 0;

    if (QF19_SCAN_ATMSSDR[iscan] & 32) scanqual[iscan]+=2;             /*BB ant pos err*/
    if (QF19_SCAN_ATMSSDR[iscan] & 16) scanqual[iscan]+=4;             /*SV ant pos err*/
    if (QF19_SCAN_ATMSSDR[iscan] & 8)  scanqual[iscan]+=8;             /*WG PRT sufficiency*/
    if (QF19_SCAN_ATMSSDR[iscan] & 4)  scanqual[iscan]+=16;            /*KAV PRT sufficiency*/
    if (QF19_SCAN_ATMSSDR[iscan] & 2)  scanqual[iscan]+=32;            /*Data gap*/
    if (QF19_SCAN_ATMSSDR[iscan] & 2)  scanqual[iscan]+=64;            /*Time sequence err*/
    if (QF18_SCAN_WGPRTTEMPCONSISTENCY[iscan])  scanqual[iscan]+=128;  /*WG PRT temp inconsistency*/
    if (QF17_SCAN_KAVPRTTEMPCONSISTENCY[iscan]) scanqual[iscan]+=256;  /*KAV PRT temp inconsistency*/
    if (QF16_SCAN_WGPRTTEMPLIMIT[iscan])        scanqual[iscan]+=512;  /*WG PRT out of range*/
    if (QF15_SCAN_KAVPRTTEMPLIMIT[iscan])       scanqual[iscan]+=1024; /*KAV PRT out of range*/
    if (QF14_SCAN_SHELFPRTCONVERR[iscan])       scanqual[iscan]+=2048; /*RF shelf computation failed*/
    if (QF13_SCAN_WGPRTCONVERR[iscan])          scanqual[iscan]+=4096; /*WG PRT computation failed*/
    if (QF12_SCAN_KAVPRTCONVERR[iscan])         scanqual[iscan]+=8192; /*KAV PRT computation failed*/

    for (ichan=0; ichan<ATMS_CHANS; ichan++){
      
      chanqual[iscan][ichan] = 0;
      
      if (QF22_ATMSSDR[iscan][ichan] & 240) chanqual[iscan][ichan]+=2;   /*BB view inconsistency*/
      if (QF22_ATMSSDR[iscan][ichan] & 15)  chanqual[iscan][ichan]+=4;   /*SP view inconsistency*/
      if (QF21_ATMSSDR[iscan][ichan] & 240) chanqual[iscan][ichan]+=8;   /*BB view out of range*/
      if (QF21_ATMSSDR[iscan][ichan] & 15)  chanqual[iscan][ichan]+=16;  /*SP view out of range*/
      if (QF20_ATMSSDR[iscan][ichan] & 16)  chanqual[iscan][ichan]+=32;  /*Insufficient BB view*/
      if (QF20_ATMSSDR[iscan][ichan] & 8)   chanqual[iscan][ichan]+=64;  /*Insufficient SP view*/
      if (QF20_ATMSSDR[iscan][ichan] & 4)   chanqual[iscan][ichan]+=128; /*Cal with fewer than preferred*/
      if (QF20_ATMSSDR[iscan][ichan] & 2)   chanqual[iscan][ichan]+=256; /*Gain error*/
      if (QF20_ATMSSDR[iscan][ichan] & 1)   chanqual[iscan][ichan]+=512; /*Moon in space view*/
      
    }    
  }

/****** Display ******/

  printf("Finished reading %s\n",filename);

/****** Clean up ******/

  H5Fclose(file_id);
  return 0;

}
#endif
