! Name: $Id: avh1b_to_hdf5_f.F 465 2015-10-13 12:43:03Z frna $
!
! Purpose: Read AVHRR l1b file and pass information back to C calling routine.
!          Subroutines openavh1b, readavh1bhd, readavh1bdt
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    19/05/2009  N C Atkinson
!  1.1    17/02/2015  Bug fix: solzen and satzen were transposed.
!                     Allow number of AVHRR scans to exceed 32767.
!-----------------------------------------------------------------------


      SUBROUTINE openavh1b(filename)
      IMPLICIT NONE
      
#include "avh1b.h"
#include "reclopt.h"

      CHARACTER*200 filename
      
      INTEGER I,len

      len = 200
      DO I=200,1,-1
        IF (ICHAR(filename(I:I)).EQ.0) len = I-1
      ENDDO
      
      OPEN(file=filename(1:len), unit=10, form='unformatted',
     &  access='direct',recl=reclavh/rldiv)
      write(*,*)'Opened file ',filename(1:len)
      
      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE readavh1bhd(scnlin,dataname,satid,startdatayr,
     &    startdatady,enddatayr,enddatady,startdatatime,enddatatime,
     &    ir_wavenumber,ir_radtemp_offset,ir_radtemp_slope,
     &    C_1copy,C_2copy)
     
      IMPLICIT NONE
      
#include "avh1b.h"
#include "constants_preproc.h"

      INTEGER      scnlin
      CHARACTER*42 dataname
      INTEGER*2    satid,startdatayr,startdatady,enddatayr,enddatady
      REAL         startdatatime,enddatatime,ir_wavenumber(3)
      REAL         ir_radtemp_offset(3),ir_radtemp_slope(3)
      REAL         C_1copy,C_2copy
      INTEGER      I

      READ(10,rec=1) avh_hd
      scnlin = avh_h_scnlin
      if (scnlin .lt. 0) scnlin = scnlin + 65536
      dataname = avh_h_dataname
      satid = avh_h_satid
      startdatayr = avh_h_startdatayr
      startdatady = avh_h_startdatady
      startdatatime = avh_h_startdatatime*1.0E-3
      enddatayr = avh_h_enddatayr
      enddatady = avh_h_enddatady
      enddatatime = avh_h_enddatatime*1.0E-3
      DO I=1,3
        ir_wavenumber(I) = avh_h_radtempcnv(1,I)*1E-3
        ir_radtemp_offset(I) = avh_h_radtempcnv(2,I)*1E-5
        ir_radtemp_slope(I) = avh_h_radtempcnv(3,I)*1E-6
      ENDDO
      ir_wavenumber(1) = avh_h_radtempcnv(1,1)*1E-2   !ch 3B
      
      C_1copy = C_1
      C_2copy = C_2        

      RETURN
      END
      
!-------------------------------------------------------------------------------
      
      SUBROUTINE readavh1bdt(irec,scaling,albedo_radiance,lat,lon,
     &   satzen,solzen,relaz,ch3b,
     &   scnlin,scnlinyr,scnlindy,scnlintime,qualind,scnlinqual,calqual)
      IMPLICIT NONE
      
#include "avh1b.h"

      INTEGER   irec
      INTEGER   scaling(6)         !ch 1,2,3a,3b,4,5
      INTEGER*2 albedo_radiance(avh_mxhrpchn,avh_mxpix)
      INTEGER*2 ch3b
      INTEGER*2 scnlin, scnlinyr, scnlindy, calqual(3)
      INTEGER*4 qualind, scnlinqual
      REAL*4    scnlintime
      REAL*4    lat(51),lon(51),satzen(51),solzen(51),relaz(51)

      INTEGER i,pix,chn,nvis
      INTEGER*2 I2_one
      DATA I2_one/1/
      
      REAL    coef(5),slope,intercept
      
      DO i=1,6
        scaling(i) = 100       !scaling 100
      ENDDO
      scaling(4) = 10000       !different scaling for channel 3b

      READ(10,rec=irec) avh_dt
      
!Date and time
      scnlin = avh_scnlin
      scnlinyr = avh_scnlinyr
      scnlindy = avh_scnlindy
      scnlintime = avh_scnlintime/1.0E3     !seconds

!Quality indicators
      qualind = avh_qualind
      scnlinqual = avh_scnlinqual
      DO i=1,3
        calqual(i) = avh_calqual(i)
      ENDDO

!See whether channel 3 is visible (3a) or IR (3b)
      ch3b = IAND(avh_scnlinbit, I2_one)
      IF (ch3b.EQ.I2_one) THEN
        nvis = 2
      ELSE
        nvis = 3
      ENDIF
      
!Calibrate the visible channels
      DO chn=1,nvis
        DO i=1,5
          coef(i) = avh_calvis(i,3,chn)  !pre-launch coef
        ENDDO
        DO pix=1,avh_mxpix
          IF (avh_hrpt(chn,pix) .LE. coef(5)) THEN
            slope = coef(1)*1E-10
            intercept = coef(2)*1E-7
          ELSE
            slope = coef(3)*1E-10
            intercept = coef(4)*1E-7
          ENDIF
          albedo_radiance(chn, pix) = NINT(scaling(chn)*
     &             (intercept + slope*avh_hrpt(chn,pix)))
        ENDDO
      ENDDO
      
!Calibrate the IR channels
      DO chn=nvis+1,5
        DO i=1,3
          coef(i) = avh_calir(i,1,chn-2)  !operational coef
        ENDDO
        DO pix=1,avh_mxpix
          albedo_radiance(chn, pix) = NINT(scaling(chn+1)*
     &             (coef(3)*1E-6 + coef(2)*1E-6*avh_hrpt(chn,pix) + 
     &              avh_hrpt(chn,pix)*coef(1)*1E-9*avh_hrpt(chn,pix)))
        ENDDO
      ENDDO
      
!Navigation     
        DO pix=1,51
          lat(pix) = avh_pos(1,pix)*1E-4
          lon(pix) = avh_pos(2,pix)*1E-4
          solzen(pix) = avh_ang(1,pix)*1E-2
          satzen(pix) = avh_ang(2,pix)*1E-2
          relaz(pix) = avh_ang(3,pix)*1E-2
        ENDDO
        
      RETURN
      END
      
!-------------------------------------------------------------------------------
      
