/* 
! Name: $Id: avh1b_to_hdf5_write.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Subroutine to write AVHRR data to HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      17/11/08  Original code. (N.C.Atkinson)
!  1.1      04/10/2017  Change attribute "period_minutes" to "period_seconds"
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
 
#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <stdio.h>
#include <string.h>
#include "add_hdf5_attributes.h"

int avh1b_to_hdf5_write(char *filename, char *dataname, int nscans, int *radscaling, 
              float startdatatime, float enddatatime,
              short satid, short startdatayr, short enddatayr, short startdatady, short enddatady,
              float *ir_wavenumber, float *ir_radtemp_offset, float *ir_radtemp_slope, float C_1, float C_2,
              short *hrpt, float *lat, float *lon, float *satzen, float *solzen,  float *relaz, 
              short *ch3b, short *scnlin,
              short *scnlinyr, short *scnlindy, float *scnlintime, int *qualind, 
              int *scnlinqual, short *calqual){

  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[3];       /* dimensions */
  hsize_t     cdims[3];
  herr_t      status;
  int         rank;
  int         values[6];
  int         i;
  
/* Create the HDF5 file */

  printf("Creating %s\n",filename);
  file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

/* Group /HEADER */

  group_id = H5Gcreate(file_id, "/HEADER", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  dims[0] = 1;

  dataspace_id = H5Screate_simple(1, dims, NULL); 
  strtype = H5Tcopy (H5T_C_S1);
  status = H5Tset_size (strtype, strlen(dataname));

  dataset_id = H5Dcreate(file_id, "/HEADER/dataname", strtype, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, strtype, H5S_ALL, H5S_ALL, H5P_DEFAULT, dataname);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/satid", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &satid);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/startdatayr", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &startdatayr);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/startdatady", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &startdatady);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/enddatayr", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &enddatayr);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/enddatady", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &enddatady);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/startdatatime", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &startdatatime);
  status = add_string_attribute(dataset_id, "Units","seconds");
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/enddatatime", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &enddatatime);
  status = add_string_attribute(dataset_id, "Units","seconds");
  H5Dclose(dataset_id);
  
  dataset_id = H5Dcreate(file_id, "/HEADER/constant_c1", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &C_1);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/constant_c2", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &C_2);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);
  dims[0] = 3;
  dataspace_id = H5Screate_simple(1, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/HEADER/ir_wavenumber", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ir_wavenumber);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/ir_radtemp_offset", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ir_radtemp_offset);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/HEADER/ir_radtemp_slope", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ir_radtemp_slope);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);
  H5Gclose(group_id);

/* Group /DATA */

  group_id = H5Gcreate(file_id, "/DATA", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

/* Dataset hrpt */

  rank = 3;
  dims[0] = nscans;
  dims[1] = 2048;
  dims[2] = 5;
  cdims[0] = 50;   /* chunking */
  cdims[1] = 64;
  cdims[2] = 5;
  plist  = H5Pcreate(H5P_DATASET_CREATE);
  status = H5Pset_chunk(plist, rank, cdims);
  status = H5Pset_deflate( plist, 6);         /* gzip compression */
  
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/DATA/hrpt", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, hrpt);
  H5Sclose(dataspace_id);
  H5Pclose(plist);                    

  status = add_string_attribute(dataset_id, "Units", 
                        "Albedo % or radiance mW/m^2/str/cm^-1");
  dims[0] = 6;
  status = add_int_attribute(dataset_id, "Scale factor ch1,2,3a,3b,4,5", radscaling, dims);

  H5Dclose(dataset_id);
  
/* ch3b, scnlin, scnlinyr, scnlindy, scnlintime, qualind, scnlinqual */

  rank = 1;
  dims[0] = nscans;
  cdims[0] = 50;
  plist  = H5Pcreate(H5P_DATASET_CREATE);
  status = H5Pset_chunk(plist, rank, cdims);
  status = H5Pset_deflate( plist, 6);

  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/DATA/ch3b", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ch3b);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/scnlin", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlin);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/year", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinyr);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/day", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlindy);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/seconds", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlintime);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/qualind", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, qualind);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/scnlinqual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinqual);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);  
  H5Pclose(plist);                    

/* calqual */

  rank = 2;
  dims[0] = nscans;
  dims[1] = 3;     /* 3 IR channels */
  cdims[0] = 50;
  cdims[1] = 3;
  plist  = H5Pcreate(H5P_DATASET_CREATE);
  status = H5Pset_chunk(plist, rank, cdims);
  status = H5Pset_deflate( plist, 6);
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/DATA/calqual", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calqual);
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);  
  H5Pclose(plist);                    

/* lat, lon */

  rank = 2;
  dims[0] = nscans;
  dims[1] = 51;
  cdims[0] = 50;
  cdims[1] = 51;
  plist  = H5Pcreate(H5P_DATASET_CREATE);
  status = H5Pset_chunk(plist, rank, cdims);
  status = H5Pset_deflate( plist, 6);
  
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/DATA/Latitude", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/Longitude", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/Sat_zenith", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/Sol_zenith", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/DATA/Rel_azimuth", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, relaz);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);
  H5Pclose(plist);                    

/* close the group and file */

  H5Gclose(group_id);
  H5Fclose(file_id);
}

/******************************************************************************/

#else

int avh1b_to_hdf5_write(char *filename, char *dataname,
              float startdatatime, float enddatatime,
              short startdatayr, short enddatayr,
              short startdatady, short enddatady, 
              int nscans, int *radscaling, 
              short *hrpt, float *lat, float *lon, float *satzen, float *solzen,  float *relaz, 
              short *ch3b, short *scnlin,
              short *scnlinyr, short *scnlindy, float *scnlintime, int *qualind, 
              int *scnlinqual, short *calqual){

/* Print an error message */

  printf ("Error: unable to write HDF-5\n");
  printf ("  avh1b_to_hdf5_write has not been built with compiler directive HAS_LIBHDF\n");
  exit(1);

}
#endif
