/* 
! Name: $Id: avh1c_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $
!
! Purpose: Subroutine to write AVHRR 1c data to HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      29/09/2014  N C Atkinson
!  1.1      23/02/2015  Add scalti
!  1.2      27/01/2016  Add full res geo option and NEdT
!  1.3      02/11/2016  Bug fix in scale attribute of tsurf
!  1.4      19/07/2017  Do not generate output if the requested area is not covered at all
!  1.5      04/10/2017  Change attribute "period_minutes" to "period_seconds"
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
 
#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <math.h>
#include <string.h>

#include "add_hdf5_attributes.h"
#include "avh1c_c.h"
#include "z_intex_51_to_2048.h"

int avh1c_to_hdf5_write(char *filename, int clevel, int fullresgeo, double lat0, double lon0, int nx, int ny){ 

  char *hdf5_encoder_version = "$Id: avh1c_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $";
  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[3];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         j;
  int         ichan;
  int         geo_subset;
  int         scan0, spot0, firstscan, lastscan, firstspot, lastspot;
  int         iscan, ispot, jscan_full, jscan_subset, jspot_full, jspot_subset;
  float       d, dmin, tmplat, tmplon, coslat, dxmin, dymin, latmin, lonmin;
  float       wavenumber[AVH_IRCHN];
  float       radtemp_offset[AVH_IRCHN];
  float       radtemp_slope[AVH_IRCHN];
  float       solfiltirrad[AVH_VISCHN];
  float       equifiltwidth[AVH_VISCHN];
  
  int         mask_done=0;
  int         fill=-999999;
  float       btscale=0.01;
  float       latlonscale=0.0001;
  float       anglescale=0.01;

  int *latitude_full;
  int *longitude_full;
  short *satzen_full;
  short *solzen_full;
  short *relazi_full;
  int *latitude_subset;
  int *longitude_subset;
  short *satzen_subset;
  short *solzen_subset;
  short *relazi_subset;
  short *btemps_subset;
  short *mask_subset;
  short *tsurf_subset;
  double H, lati[51],loni[51],azidi[51],zenSi[51],zensi[51];
  double lato[2048],lono[2048],azido[2048],zenSo[2048],zenso[2048];

  for (ichan=0; ichan<AVH_IRCHN; ichan++){
    wavenumber[ichan] = 0.001*h_radtempcnv[ichan][0];
    radtemp_offset[ichan] = 0.00001*h_radtempcnv[ichan][1];
    radtemp_slope[ichan] = 0.000001*h_radtempcnv[ichan][2];
  }
  wavenumber[0] = 0.01*h_radtempcnv[0][0]; /* scaling for channel 3B */

  for (ichan=0; ichan<AVH_VISCHN; ichan++){
    solfiltirrad[ichan] = 0.1*h_albcnv[ichan][0];
    equifiltwidth[ichan] = 0.001*h_albcnv[ichan][1];
  }

/* If a geographical subset has been requested, find the central spot/line,
   using the coarse geolocation. Do an approximate test that there is some coverage. */

  if ((nx > 0) && (ny > 0)){
    fullresgeo = 1;         /* need full res geolocation for this option */
    geo_subset = 1;
    dmin = 1.0E10;
    scan0 = 0;
    spot0 = 0;
    for (iscan=0; iscan<h_scnlin; iscan++) {
      i = iscan*51;
      for (ispot=0; ispot<51; ispot++){
        tmplat = 1.0E-4*latitude[ispot+i];
        tmplon = 1.0E-4*longitude[ispot+i];
        coslat = cos(tmplat*0.01745329);
        d = (tmplat - lat0)*(tmplat - lat0) + coslat*(tmplon - lon0)*(tmplon - lon0);
        if (d < dmin) {
          dmin = d;
          dxmin = 6371.0*coslat*abs(tmplon - lon0)*0.01745329;   /* in km */
          dymin = 6371.0*abs(tmplat - lat0)*0.01745329;
          latmin = tmplat;
          lonmin = tmplon;
          scan0 = iscan;
          spot0 = ispot*40 + 20;
        }
      }
    }
    printf("nearest lat/lon = %10.1f %10.1f\n",latmin,lonmin);
    printf("dxmin=%7.1f km, dymin=%7.1f km\n",dxmin,dymin);

    if ((dxmin < nx/2*1.1) && (dymin < ny/2*1.1)) {    /* 1.1km per AVHRR sample at nadir */
      firstscan = scan0 - ny/2;
      lastscan = firstscan + ny - 1;
      firstspot = spot0 - nx/2;
      lastspot = firstspot + nx - 1;
      if (firstscan < 0) firstscan = 0;
      if (lastscan >= h_scnlin) lastscan = h_scnlin-1;
      if (firstspot < 0) firstspot = 0;
      if (lastspot >= 2048) lastspot = 2047;
      nx = lastspot - firstspot + 1;
      ny = lastscan - firstscan + 1;
      printf ("scans: %d to %d, spots: %d to %d\n",firstscan,lastscan,firstspot,lastspot);
      printf ("Actual number of scans in subset: %d\n",ny);
      printf ("Actual number of spots in subset: %d\n",nx);
      latitude_subset = (int*)calloc(nx*ny, sizeof(int));
      longitude_subset = (int*)calloc(nx*ny, sizeof(int));
      satzen_subset = (short*)calloc(nx*ny, sizeof(short));
      solzen_subset = (short*)calloc(nx*ny, sizeof(short));
      relazi_subset = (short*)calloc(nx*ny, sizeof(short));
      btemps_subset = (short*)calloc(AVH_CHANS*nx*ny, sizeof(int));
      mask_subset = (short*)calloc(nx*ny, sizeof(short));
      tsurf_subset = (short*)calloc(nx*ny, sizeof(short));
    } else {
      firstscan = -1;  /* area is not covered: do not create output file */
    }
  } else {
    geo_subset = 0;
    firstscan = 0;
    lastscan = h_scnlin;
    firstspot = 0;
    lastspot = AVH_SPOTS-1;
    nx = AVH_SPOTS;
    ny = h_scnlin;
  }
        

/* Create the HDF5 file */

  if (firstscan >= 0){
  printf("Creating %s\n",filename);
  file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

/* Global attributes */

  attdims[0] = 1;

  add_string_attribute(file_id, "filename", filename);
  add_string_attribute(file_id, "site", h_site);
  add_string_attribute(file_id, "1bsite", h_1bsite);
  add_string_attribute(file_id, "hdf5_encoder_software_version", hdf5_encoder_version);
  add_int_attribute(file_id, "satid", &h_satid, attdims);
  add_int_attribute(file_id, "instrument", &h_instrument, attdims);
  add_int_attribute(file_id, "satht_hectometres", &h_satht, attdims);
  add_int_attribute(file_id, "period_seconds", &h_period, attdims);
  add_int_attribute(file_id, "startorbit", &h_startorbit, attdims);
  add_int_attribute(file_id, "startdatayr", &h_startdatayr, attdims);
  add_int_attribute(file_id, "startdatady", &h_startdatady, attdims);
  add_int_attribute(file_id, "startdatatime_ms", &h_startdatatime, attdims);
  add_int_attribute(file_id, "endorbit", &h_endorbit, attdims);
  add_int_attribute(file_id, "enddatayr", &h_enddatayr, attdims);
  add_int_attribute(file_id, "enddatady", &h_enddatady, attdims);
  add_int_attribute(file_id, "enddatatime_ms", &h_enddatatime, attdims);
/*  add_int_attribute(file_id, "scnlin", &h_scnlin, attdims); */
  add_int_attribute(file_id, "scnlin", &ny, attdims);
  add_int_attribute(file_id, "misscnlin", &h_misscnlin, attdims);
  add_int_attribute(file_id, "callocscnlin", &h_callocscnlin, attdims);
  add_int_attribute(file_id, "datagaps", &h_datagaps, attdims);
  add_int_attribute(file_id, "datatyp", &h_datatyp, attdims);
  add_string_attribute(file_id, "datatyp_key", "1 = LAC; 2 = GAC; 3 = HRPT");
  add_string_attribute(file_id, "infilename", h_infilename);
  add_int_attribute(file_id, "wmosatid", &h_wmosatid, attdims);
  add_int_attribute(file_id, "cpidsyr", &h_cpidsyr, attdims);
  add_int_attribute(file_id, "cpidsdy", &h_cpidsdy, attdims);
  add_int_attribute(file_id, "rollerr", &h_rollerr, attdims);
  add_int_attribute(file_id, "pitcherr", &h_pitcherr, attdims);
  add_int_attribute(file_id, "yawerr", &h_yawerr, attdims);

  attdims[0] = AVH_IRCHN;
  add_float_attribute(file_id, "AVHRR_wavenumber", wavenumber, attdims);
  add_float_attribute(file_id, "AVHRR_radtemp_offset", radtemp_offset, attdims);
  add_float_attribute(file_id, "AVHRR_radtemp_slope", radtemp_slope, attdims);
  attdims[0] = AVH_VISCHN;
  add_float_attribute(file_id, "AVHRR_solfiltirrad", solfiltirrad, attdims);
  add_float_attribute(file_id, "AVHRR_equifiltwidth", equifiltwidth, attdims);
  attdims[0] = 1;

/* Group /Data */

  group_id = H5Gcreate(file_id, "/Data", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets with one value per scan */

  rank = 1;
  dims[0] = ny;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/scnlin", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &scnlin[firstscan]);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinyr", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &scnlinyr[firstscan]);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlindy", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &scnlindy[firstscan]);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlintime", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &scnlintime[firstscan]);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/clockdrift", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &clockdrift[firstscan]);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinbit", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &scnlinbit[firstscan]);
  status = add_string_attribute(dataset_id, "Description", "Bit 0: 0=3A, 1=3B; bit 13: 1=corrected for TIP attitude; bit 14: 1=corrected for clock drift; bit 15: 0=ascending, 1=descending");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/qualind", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &qualind[firstscan]);
  status = add_string_attribute(dataset_id, "Description", "Quality indicator: see AAPP Formats document");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinqual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &scnlinqual[firstscan]);
  status = add_string_attribute(dataset_id, "Description", "Scan line quality: see AAPP Formats document");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/maskind", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &maskind[firstscan]);
  status = add_string_attribute(dataset_id, "Description", "Cloud mask done =1");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scalti", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &scalti[firstscan]);
  status = add_string_attribute(dataset_id, "Description", "Spacecraft altitude");
  status = add_string_attribute(dataset_id, "Units", "100m");
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Calibration quality and NEdT */

  rank = 2;
  dims[0] = ny;
  dims[1] = AVH_IRCHN;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/calqual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &calqual[AVH_IRCHN*firstscan]);
  status = add_string_attribute(dataset_id, "Description", "Calibration quality: see AAPP Formats document");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/nedt", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, &nedt[AVH_IRCHN*firstscan]);
  status = add_string_attribute(dataset_id, "Description", "NEdT at 300K for channels 3B, 4, 5");
  status = add_string_attribute(dataset_id, "Units", "K");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset btemps */

  rank = 3;
  dims[0] = ny;
  dims[1] = nx;
  dims[2] = AVH_CHANS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/btemps", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);

  if (geo_subset == 0) {
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, btemps);
  } else {
    printf ("Generating the btemps subset\n");
    for (iscan=0; iscan<ny; iscan++) {
      jscan_full = (iscan+firstscan) * AVH_SPOTS * AVH_CHANS;
      jscan_subset = iscan * nx * AVH_CHANS;
      for (ispot=0; ispot<nx; ispot++) {
        jspot_full = (ispot+firstspot)*AVH_CHANS;
        jspot_subset = ispot*AVH_CHANS;
        for (ichan=0; ichan<AVH_CHANS; ichan++) {
          btemps_subset[jscan_subset + jspot_subset + ichan] = btemps[jscan_full + jspot_full + ichan];
        }
        if (mask_done == 1) {
          mask_subset[jscan_subset + jspot_subset] = mask[jscan_full + jspot_full];
          tsurf_subset[jscan_subset + jspot_subset] = tsurf[jscan_full + jspot_full];
        }
      }
    }
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, btemps_subset);
  }

  status = add_string_attribute(dataset_id, "Units", "% (vis), K (IR)");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  status = add_string_attribute(dataset_id, "Description", "Reflectivity and brightness temperature");

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset mask (only if present) */

  for (i=0; i<h_scnlin; i++){
    if (maskind[i] != 0) mask_done=1;
  }
  if (mask_done == 1) {
    printf ("Cloud mask is present\n");
    rank = 2;
    dims[0] = ny;
    dims[1] = nx;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/mask", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    if (geo_subset == 0) {
      status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, mask);
    } else {
      status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, mask_subset);
    }
    status = add_string_attribute(dataset_id, "Description", "MAIA3 cloud mask, see AAPP documentation");
    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/tsurf", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    if (geo_subset == 0) {
      status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tsurf);
    } else {
      status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tsurf_subset);
    }
    status = add_string_attribute(dataset_id, "Description", "retrieved surface temperature");
    status = add_string_attribute(dataset_id, "Units", "deg C");
    status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);

  } else {
    printf ("Cloud mask is not present\n");
  }
  H5Pclose(plist);                    
  H5Gclose(group_id);

/* Group /Geolocation */

  group_id = H5Gcreate(file_id, "/Geolocation", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets latitude and longitude */

  /* interpolate 51 -> 2048 points if requested */
  if (fullresgeo) {
    latitude_full = (int*)calloc(h_scnlin*AVH_SPOTS, sizeof(int));
    longitude_full = (int*)calloc(h_scnlin*AVH_SPOTS, sizeof(int));
    satzen_full = (short*)calloc(h_scnlin*AVH_SPOTS, sizeof(short));
    solzen_full = (short*)calloc(h_scnlin*AVH_SPOTS, sizeof(short));
    relazi_full = (short*)calloc(h_scnlin*AVH_SPOTS, sizeof(short));
    for (iscan=0; iscan<h_scnlin; iscan++){
      H = scalti[iscan]*0.1;
      i = iscan*51;
      for (j=0; j<51; j++){
        lati[j] = 1.0E-4*latitude[j+i];
        loni[j] = 1.0E-4*longitude[j+i];
        azidi[j] = 1.0E-2*relazi[j+i];
        zenSi[j] = 1.0E-2*solzen[j+i];
        zensi[j] = 1.0E-2*satzen[j+i];
      }
      z_intex_51_to_2048 (H, azidi, zenSi, zensi, loni, lati, azido, zenSo, zenso, lono, lato);
      i = iscan*2048;
      for (j=0; j<2048; j++){
        latitude_full[j+i] = 10000 * lato[j];
        longitude_full[j+i] = 10000 * lono[j];
        relazi_full[j+i] = 100 * azido[j];
        solzen_full[j+i] = 100 * zenSo[j];
        satzen_full[j+i] = 100 * zenso[j];
      }
    }
    if (geo_subset) {
      for (iscan=0; iscan<ny; iscan++) {
        jscan_full = (iscan+firstscan) * AVH_SPOTS;
        jscan_subset = iscan * nx;
        for (ispot=0; ispot<nx; ispot++) {
          jspot_full = ispot+firstspot;
          jspot_subset = ispot;
          latitude_subset[jscan_subset + jspot_subset] = latitude_full[jscan_full + jspot_full];
          longitude_subset[jscan_subset + jspot_subset] = longitude_full[jscan_full + jspot_full];
          satzen_subset[jscan_subset + jspot_subset] = satzen_full[jscan_full + jspot_full];
          solzen_subset[jscan_subset + jspot_subset] = solzen_full[jscan_full + jspot_full];
          relazi_subset[jscan_subset + jspot_subset] = relazi_full[jscan_full + jspot_full];
        }
      }
      dims[0] = ny;
      dims[1] = nx;
    } 
    else {
      dims[0] = h_scnlin;
      dims[1] = AVH_SPOTS;
    }
  }
  else {
    dims[0] = h_scnlin;
    dims[1] = AVH_TIE;
  }

  rank = 2;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Geolocation/Latitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  if (geo_subset) { 
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude_subset);
    status = add_string_attribute(dataset_id, "Description", "Latitude subset");
  } else if (fullresgeo) {
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude_full);
    status = add_string_attribute(dataset_id, "Description", "Latitude all spots");
  } else {
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude);
    status = add_string_attribute(dataset_id, "Description", "Latitude spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Longitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  if (geo_subset) { 
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude_subset);
    status = add_string_attribute(dataset_id, "Description", "Longitude subset");
  } else if (fullresgeo) {
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude_full);
    status = add_string_attribute(dataset_id, "Description", "Longitude all spots");
  } else {
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude);
    status = add_string_attribute(dataset_id, "Description", "Longitude spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* Satellite and solar angles */

  if (geo_subset) { 
    dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen_subset);
    status = add_string_attribute(dataset_id, "Description", "Satellite zenith angle subset");
  } else if (fullresgeo) {   /* Store the full resolution angles as 16 bit numbers to save space */ 
    dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen_full);
    status = add_string_attribute(dataset_id, "Description", "Satellite zenith angle all spots");
  } else {
    dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen);
    status = add_string_attribute(dataset_id, "Description", "Satellite zenith angle spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  if (geo_subset) { 
    dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen_subset);
    status = add_string_attribute(dataset_id, "Description", "Solar zenith angle subset");
  } else if (fullresgeo) {
    dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen_full);
    status = add_string_attribute(dataset_id, "Description", "Solar zenith angle all spots");
  } else {
    dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen);
    status = add_string_attribute(dataset_id, "Description", "Solar zenith angle spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  if (geo_subset) { 
    dataset_id = H5Dcreate(file_id, "/Geolocation/Relative_azimuth_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, relazi_subset);
    status = add_string_attribute(dataset_id, "Description", "Relative azimuth angle subset");
  } else if (fullresgeo) {
    dataset_id = H5Dcreate(file_id, "/Geolocation/Relative_azimuth_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, relazi_full);
    status = add_string_attribute(dataset_id, "Description", "Relative azimuth angle all spots");
  } else {
    dataset_id = H5Dcreate(file_id, "/Geolocation/Relative_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, relazi);
    status = add_string_attribute(dataset_id, "Description", "Relative azimuth angle spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* close the group and file */

  H5Sclose(dataspace_id);
  H5Pclose(plist);                    
  H5Gclose(group_id);
  H5Fclose(file_id);
  printf ("Finished writing to HDF-5 file\n");
  } else {
    printf("No coverage, output file not created\n");
    exit(1);
  }
}

#else

int avh1c_to_hdf5_write(char *filename, int clevel, int fullresgeo, double lat0, double lon0, int nx, int ny){ 

  printf ("Error: unable to write HDF-5\n");
  exit(1);

}
#endif
