/* 
! Name: $Id: cris1c_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $
!
! Purpose: Subroutine to write CRIS 1c data to HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      09/07/2015  N C Atkinson
!  1.1      14/08/2015  Correct the radiance Units attribute
!  1.2      04/10/2017  Change attribute "period_minutes" to "period_seconds"
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
 
#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <string.h>
#include <math.h>

#include "add_hdf5_attributes.h"
#include "cris1c_c.h"

int cris1c_to_hdf5_write(char *filename, int clevel){ 

  char *hdf5_encoder_version = "$Id: cris1c_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $";
  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[5];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         ichan;
  int         ifor;
  int         ifov;
  
  int         fill=-999999;
  int         avhfill=255;
  float       latlonscale=0.0001;
  float       anglescale=0.01;
  float       radiancescale=1.0E-7;
  float       w[3];
  
/* Create the HDF5 file */

  printf("Creating %s\n",filename);
  file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

/* Global attributes */

  attdims[0] = 1;

  add_string_attribute(file_id, "filename", filename);
  add_string_attribute(file_id, "site", h_site);
  add_string_attribute(file_id, "1bsite", h_1bsite);
  add_string_attribute(file_id, "hdf5_encoder_software_version", hdf5_encoder_version);
  add_int_attribute(file_id, "satid", &h_satid, attdims);
  add_int_attribute(file_id, "instrument", &h_instrument, attdims);
  add_int_attribute(file_id, "satht_hectometres", &h_satht, attdims);
  add_int_attribute(file_id, "period_seconds", &h_period, attdims);
  add_int_attribute(file_id, "startorbit", &h_startorbit, attdims);
  add_int_attribute(file_id, "startdatayr", &h_startdatayr, attdims);
  add_int_attribute(file_id, "startdatady", &h_startdatady, attdims);
  add_int_attribute(file_id, "startdatatime_ms", &h_startdatatime, attdims);
  add_int_attribute(file_id, "endorbit", &h_endorbit, attdims);
  add_int_attribute(file_id, "enddatayr", &h_enddatayr, attdims);
  add_int_attribute(file_id, "enddatady", &h_enddatady, attdims);
  add_int_attribute(file_id, "enddatatime_ms", &h_enddatatime, attdims);
  add_int_attribute(file_id, "scnlin", &h_scnlin, attdims);
  add_int_attribute(file_id, "misscnlin", &h_misscnlin, attdims);
  add_int_attribute(file_id, "wmosatid", &h_wmosatid, attdims);
  add_int_attribute(file_id, "centre", &h_centre, attdims);
  add_int_attribute(file_id, "subcentre", &h_subcentre, attdims);
  add_int_attribute(file_id, "nfov", &h_nfov, attdims);
  add_int_attribute(file_id, "nfor", &h_nfor, attdims);
  add_int_attribute(file_id, "nband", &h_nband, attdims);
  add_int_attribute(file_id, "eigvn", &h_eigvn, attdims);
  add_int_attribute(file_id, "npcused", &h_npcused, attdims);
  add_int_attribute(file_id, "chanvn", &h_chanvn, attdims);
  add_int_attribute(file_id, "nchanused", &h_nchanused, attdims);

  attdims[0] = 3;

  for (i=0; i<3; i++) w[i] = h_startw[i]*0.01;
  add_float_attribute(file_id, "start_wavenumber", w, attdims);
  for (i=0; i<3; i++) w[i] = h_endw[i]*0.01;
  add_float_attribute(file_id, "end_wavenumber", w, attdims);
  add_int_attribute(file_id, "startchan", h_startchan, attdims);
  add_int_attribute(file_id, "endchan", h_endchan, attdims);

  if (h_nchanused > 0){
    attdims[0] = h_nchanused;
    add_int_attribute(file_id, "channels", h_channels, attdims);
  }
  attdims[0] = 1;

/* Group /Data */

  group_id = H5Gcreate(file_id, "/Data", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets with one value per scan */

  rank = 1;
  dims[0] = h_scnlin;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/scnlin", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlin);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinyr", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinyr);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlindy", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlindy);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scalti", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scalti);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/ascdesc", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ascdesc);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/orbit", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, orbit);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scanqualflgs", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scanqualflgs);
  status = add_string_attribute(dataset_id, "Description", "Scan quality flags: see cris1c.h");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/geolocqual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geolocqual);
  status = add_string_attribute(dataset_id, "Description", "Geolocation quality flags for each scan: see cris1c.h");
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Dataset scnlintime */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = CRIS_FOR;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/scnlintime", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlintime);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset satzyz */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = 3;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/satxyz", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satxyz);
  status = add_string_attribute(dataset_id, "Units", "m");
  status = add_string_attribute(dataset_id, "Description", "Satellite x,y,z position at mid scan");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset radianceflgs */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = CRIS_FOR;
  dims[2] = CRIS_FOV;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/radianceflgs", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, radianceflgs);
  status = add_string_attribute(dataset_id, "Description", "Radiance flags for each spot: see cris1c.h");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset calqualflgs */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = CRIS_FOV;
  dims[2] = 3;    /* bands */
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/calqualflgs", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calqualflgs);
  status = add_string_attribute(dataset_id, "Description", "Cal quality flags for each band and detector: see cris1c.h");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset radiance */

  if (h_nchanused > 0) {
    rank = 4;
    dims[0] = h_scnlin;
    dims[1] = CRIS_FOR;
    dims[2] = CRIS_FOV;
    dims[3] = h_nchan;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/radiance", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, radiance);
    status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr cm^-1)");
    status = add_float_attribute(dataset_id, "Scale", &radiancescale, attdims);

    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);
  }

/* Dataset scores */

  if (h_npcused > 0) {
    rank = 4;
    dims[0] = h_scnlin;
    dims[1] = CRIS_FOR;
    dims[2] = CRIS_FOV;
    dims[3] = h_npcused;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/scores", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scores);
    status = add_string_attribute(dataset_id, "Description", "Principal component scores");

    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);
  }

/* Group /Geolocation */

  H5Pclose(plist);                    
  H5Gclose(group_id);
  group_id = H5Gcreate(file_id, "/Geolocation", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets latitude and longitude */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = CRIS_FOR;
  dims[2] = CRIS_FOV;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Geolocation/Latitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Longitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* Satellite and solar angles */

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* Other geolocation variables */

  dataset_id = H5Dcreate(file_id, "/Geolocation/surfht", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, surfht);
  status = add_string_attribute(dataset_id, "Units", "m");
  status = add_string_attribute(dataset_id, "Description", "Surface height above ellipsiod");
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/landfrac", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, landfrac);
  status = add_string_attribute(dataset_id, "Units", "%");
  status = add_string_attribute(dataset_id, "Description", "Land fraction");
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/landsea", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, landsea);
  status = add_string_attribute(dataset_id, "Description", "0=land, 1=sea, 2=mixed, 3=unknown");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Group /Derived */

  H5Pclose(plist);                    
  H5Gclose(group_id);
  group_id = H5Gcreate(file_id, "/Derived", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Cloud cover and height */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = CRIS_FOR;
  dims[2] = CRIS_FOV;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Derived/cloudcover", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, cloudcover);
  status = add_string_attribute(dataset_id, "Units", "%");
  status = add_string_attribute(dataset_id, "Description", "Cloud cover");
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Derived/cloudtopht", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, cloudtopht);
  status = add_string_attribute(dataset_id, "Units", "m");
  status = add_string_attribute(dataset_id, "Description", "Cloud top height");
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* close the group and file */

  H5Sclose(dataspace_id);
  H5Pclose(plist);                    
  H5Gclose(group_id);
  H5Fclose(file_id);
}

#else

int cris1c_to_hdf5_write(char *filename, int clevel){

  printf ("Error: unable to write HDF-5\n");
  exit(1);

}
#endif
