/*
! name: $Id: cris_geo_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an CRIS Geolocation file in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    19/10/2010  Original code. (N.C.Atkinson)
!  1.1    25/02/2016  Bug fix: prevent negative time of day around midnight
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "cris_sdr.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "common_read.h"

int cris_geo_read(char *filename){
  
  int     iscan, ifor;                   /* Loop variables */
  hid_t   file_id, group_id, dataset_id;  /* handles */
  hid_t   strtype;
  herr_t  status;
  int     hour, minute;
  double  second, secs;
  int     taioffset;
  long    taioffset_ms;
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0){
    fprintf(stderr, "Cannot open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */

/****** Open the group CrIS-SDR-GEO_All ******/

  group_id  = H5Gopen(file_id, "All_Data/CrIS-SDR-GEO_All", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group All_Data/CrIS-SDR-GEO_All\n");
    exit(10);
  }

/****** Read the data ******/

  readdataset("FORTime", group_id, H5T_NATIVE_LLONG, FORTime);
  readdataset("Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
  readdataset("Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
  readdataset("SatelliteAzimuthAngle", group_id, H5T_NATIVE_FLOAT, SatelliteAzimuthAngle);
  readdataset("SatelliteZenithAngle", group_id, H5T_NATIVE_FLOAT, SatelliteZenithAngle);
  readdataset("SolarAzimuthAngle", group_id, H5T_NATIVE_FLOAT, SolarAzimuthAngle);
  readdataset("SolarZenithAngle", group_id, H5T_NATIVE_FLOAT, SolarZenithAngle);
  readdataset("Height", group_id, H5T_NATIVE_FLOAT, Height);
  readdataset("SCPosition", group_id, H5T_NATIVE_FLOAT, SCPosition);
  readdataset("SCVelocity", group_id, H5T_NATIVE_FLOAT, SCVelocity);
  readdataset("QF1_CRISSDRGEO", group_id, H5T_NATIVE_UCHAR, QF1_CRISSDRGEO);

  H5Gclose(group_id);

/****** Read the aggregate attribute data ******/

  group_id  = H5Gopen(file_id, "Data_Products/CrIS-SDR-GEO", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group Data_Products/CrIS-SDR-GEO\n");
    exit(10);
  }

  dataset_id = H5Dopen(group_id, "CrIS-SDR-GEO_Aggr", H5P_DEFAULT);
  if (dataset_id < 0) {
    fprintf(stderr,"Cannot open dataset CrIS-SDR-GEO_Aggr\n");
    exit(10);
  }

  status = H5Tset_size (strtype, 9);   /* Set the length of the string */
  readattribute("AggregateBeginningDate",dataset_id,strtype,AggregateBeginningDate);
  readattribute("AggregateEndingDate",dataset_id,strtype,AggregateEndingDate);
  status = H5Tset_size (strtype, 15);
  readattribute("AggregateBeginningTime",dataset_id,strtype,AggregateBeginningTime);
  readattribute("AggregateEndingTime",dataset_id,strtype,AggregateEndingTime);
  readattribute("AggregateBeginningOrbitNumber",dataset_id,H5T_NATIVE_LLONG,
    &AggregateBeginningOrbitNumber);
  readattribute("AggregateEndingOrbitNumber",dataset_id,H5T_NATIVE_LLONG,
    &AggregateEndingOrbitNumber);

  StartOrbit = AggregateBeginningOrbitNumber;
  EndOrbit = AggregateEndingOrbitNumber;
  
  H5Dclose(dataset_id);
  H5Gclose(group_id);

/****** Find the TAI offset and convert times ******/

  sscanf(AggregateBeginningTime,"%2d%2d%lf",&hour,&minute,&second);
  secs = hour*3600.0 + minute*60.0 + second;
  
  if (FORTime[0][0] > 0) {
    taioffset = floor((FORTime[0][0]%86400000000LL)/1000000.0 - secs + 0.5);
    if (taioffset < 0) taioffset += 86400;
    if (taioffset <120) {
      taioffset_ms = taioffset*1000;
      printf("taioffset = %d\n",taioffset);
    } else {
      fprintf(stderr,"Warning: unable to determine TAI offset\n");
      taioffset_ms = 0;
    }
  } else {
    fprintf(stderr,"Warning: unable to determine TAI offset\n");
    taioffset_ms = 0;
  }

  for (iscan=0;iscan<CRIS_SCANS;iscan++) for (ifor=0;ifor<CRIS_FORS;ifor++){
    if (FORTime[iscan][ifor] > 0) {
      FORTime_millisec[iscan][ifor] = (FORTime[iscan][ifor]%86400000000LL)/1000LL - taioffset_ms;
      if (FORTime_millisec[iscan][ifor] < 0) FORTime_millisec[iscan][ifor] += 86400000L;
    }
    else FORTime_millisec[iscan][ifor] = MDI;
  }

/****** Process the quality flags ******/

  for (iscan=0; iscan<CRIS_SCANS; iscan++){
    geolocqual[iscan] = QF1_CRISSDRGEO[iscan];
  }

/****** Display ******/

  printf("Finished reading %s\n",filename);

/****** Clean up ******/

  H5Fclose(file_id);
  return 0;

}
#endif
