! Name: $Id: cris_sdr_out.F 577 2018-01-15 13:58:49Z frna $
!
! Purpose: Fill CrIS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    19/10/2010  Original code. (N.C.Atkinson)
!  1.1    11/08/2011  Modify to only output 1305 channels in the 1c
!  1.2    20/08/2012  Read environment variable: CRIS_ORBIT to over-write orbit
!                     number.
!  1.3    25/09/2012  Set cris1c_h_centre and cris1c_h_subcentre
!  1.4    17/03/2015  Increase CRIS_GRANULES from 15 to 20; use SURFELEV to get
!                     surface height and type
!  1.5    10/07/2015  Correct scaling of cris1c_h_startw; set cris1c_satxyz.
!  1.6    16/05/2016  Extend to handle full-spectral-resolution files.
!                     The numbers of channels are passed as arguments.
!  1.7    04/07/2016  Set the apodisation flag in cris1c_radianceflgs.
!  1.8    12/12/2016  Add Software_Version
!  1.9    04/01/2017  Set cris1c_quality to 0 for consistency with NOAA
!  1.10   20/06/2017  Add cris1c_extraradiance, 12 extra channels at band edges
!  1.11   21/11/2017  Add wmosatid for JPSS-1 and JPSS-2
!
!-------------------------------------------------------------------------------

      SUBROUTINE CRIS_SDR_OUT (NScan,
     & cris_chans_lw, cris_chans_mw, cris_chans_sw,
     & FORTime_millisec,
     & ES_RealLW,ES_RealMW,ES_RealSW, 
     & Latitude, Longitude, SatelliteAzimuthAngle,
     & SatelliteZenithAngle, SolarAzimuthAngle, SolarZenithAngle,
     & SCPosition, SCVelocity, Height, StartOrbit, EndOrbit,
     & orbit_numbers, scanqual, calqual, fovqual, geolocqual,
     & AggregateBeginningDate, AggregateEndingDate, Mission_Name,
     & CApod, Software_Version )
     
      IMPLICIT NONE

! SDR parameters, also defined in cris_sdr.h
      INTEGER   CRIS_GRANULES, CRIS_SCANS_PER_GRANULE, CRIS_FORS,
     &   CRIS_FOVS, CRIS_SCANS, CRIS_BANDS
      PARAMETER (CRIS_GRANULES=30)
      PARAMETER (CRIS_SCANS_PER_GRANULE=4)
      PARAMETER (CRIS_FORS=30)
      PARAMETER (CRIS_FOVS=9)
      PARAMETER (CRIS_BANDS=3)
      PARAMETER (CRIS_SCANS=CRIS_SCANS_PER_GRANULE*CRIS_GRANULES)

! In going from SDR to AAPP level 1c we discard channels at edge of band (for
! consistency with NESDIS conventions).
      INTEGER CRIS_SKIP, CRIS_HALFSKIP
      PARAMETER (CRIS_SKIP=4)
      PARAMETER (CRIS_HALFSKIP=2)

! Band definitions for the AAPP level 1c file
      REAL CRIS_FIRSTW(3), CRIS_STEPW(3)
      DATA CRIS_FIRSTW/650.0, 1210.0, 2155.0/     !Wavenumbers, same in FSR and non-FSR
      DATA CRIS_STEPW/0.625, 1.25, 2.5/           !Changed later if FSR

      INTEGER CRIS_NCHAN(3)

! Arguments: 
!   SDR

      INTEGER    cris_chans_lw, cris_chans_mw, cris_chans_sw
      REAL       ES_RealLW(cris_chans_lw,CRIS_FOVS,CRIS_FORS,CRIS_SCANS)
      REAL       ES_RealMW(cris_chans_mw,CRIS_FOVS,CRIS_FORS,CRIS_SCANS)
      REAL       ES_RealSW(cris_chans_sw,CRIS_FOVS,CRIS_FORS,CRIS_SCANS)
      CHARACTER*4 Mission_Name
      CHARACTER*16 Software_Version
      CHARACTER*9 AggregateBeginningDate
      CHARACTER*9 AggregateEndingDate
      INTEGER*4   StartOrbit
      INTEGER*4   EndOrbit
      CHARACTER*20 CApod

!   Geolocation

      REAL Latitude(CRIS_FOVS,CRIS_FORS,CRIS_SCANS);
      REAL Longitude(CRIS_FOVS,CRIS_FORS,CRIS_SCANS);
      REAL SatelliteAzimuthAngle(CRIS_FOVS,CRIS_FORS,CRIS_SCANS);
      REAL SatelliteZenithAngle(CRIS_FOVS,CRIS_FORS,CRIS_SCANS);
      REAL SolarAzimuthAngle(CRIS_FOVS,CRIS_FORS,CRIS_SCANS);
      REAL SolarZenithAngle(CRIS_FOVS,CRIS_FORS,CRIS_SCANS);
      REAL Height(CRIS_FOVS,CRIS_FORS,CRIS_SCANS);
      REAL SCPosition(3,CRIS_SCANS);
      REAL SCVelocity(3,CRIS_SCANS);
      INTEGER*4 geolocqual(CRIS_SCANS)

!   Derived

      INTEGER*4 FORTime_millisec(CRIS_FORS,CRIS_SCANS)
      INTEGER*4 NScan
      INTEGER*4 orbit_numbers(CRIS_SCANS)
      INTEGER*4 scanqual(CRIS_SCANS)
      INTEGER*4 calqual(CRIS_BANDS,CRIS_FOVS,CRIS_SCANS)
      INTEGER*4 fovqual(CRIS_BANDS,CRIS_FOVS,CRIS_FORS,CRIS_SCANS)

! Local variables

      INTEGER iscan,ifor,ifov,ichan,jchan,iband,unit,j,i
      INTEGER startyear, startmonth, startday
      INTEGER endyear, endmonth, endday
      REAL    xysq,z,lat,semiMinor,r_sat,r_ellipse
      REAL    radiance(869)      !largest FSR array
      REAL    apod_radiance(869)
      REAL    Hamming(3)
      DATA    Hamming /0.54,0.23,0.0/
      REAL    Blackman(3)
      DATA    Blackman /0.42323, 0.248775, 0.03961/
      REAL    Apod(3)
      INTEGER NApod       !Number of non-zero apodisation coefficients
      CHARACTER*10 CCRIS_ORBIT
      INTEGER cris_orbit
      CHARACTER*5   ORIGINATING_CENTRE
      CHARACTER*5   SUB_CENTRE
      INTEGER IORIGCENTRE
      INTEGER ISUBCENTRE

      INTEGER surftype(CRIS_FOVS,CRIS_FORS) !surface type for a scanline (SURFELEV)
      INTEGER elevation(CRIS_FOVS,CRIS_FORS)!elevation for a scanline (SURFELEV)
      INTEGER landfrac(CRIS_FOVS,CRIS_FORS) !land fraction for a scanline (SURFELEV)
      INTEGER crisboxSize             ! size of grid-box used in SURFELEV
      PARAMETER ( crisboxSize = 20 )  ! km
      INTEGER LSunit, HTunit
      PARAMETER (LSunit=51)
      PARAMETER (HTunit=52)

! Functions

      INTEGER IDAYOFYEAR

! Include
#include "cris1c.h"
#include "inparms.h"
#include "constants_preproc.h"
#include "bits.h"

! End of declarations
!-----------------------------------------------------------------------

      CRIS_NCHAN(1) = cris_chans_lw
      CRIS_NCHAN(2) = cris_chans_mw
      CRIS_NCHAN(3) = cris_chans_sw
      IF (cris_chans_mw .GT. 439) CRIS_STEPW(2) = CRIS_STEPW(1)   !if FSR
      IF (cris_chans_sw .GT. 163) CRIS_STEPW(3) = CRIS_STEPW(1)

! Define bit numbers   (see ppsetup.F)
      i = 1
      IF ( BTEST(i,0) ) THEN      !  bit 0 is LSB
         DO j = 0, 31
            bits(j)  =  j
         ENDDO  ! j
      ELSEIF( BTEST(i,31) ) THEN      !  bit 31 is LSB
         DO j = 0, 31
            bits(j)  =  31 - j
         ENDDO  ! j
      ELSE                            !  something wrong
         write(*,*)'Bit problem: BTEST(1,0) and BTEST(1,31) both FALSE'
         CALL aappexit(1)
      ENDIF

      IF (Nscan .LE. 0) THEN
        WRITE(*,*)'Error in cris_sdr_out: Nscan=',Nscan
	CALL aappexit(1)
      ENDIF

! Set the originating centre and sub-centre
      IORIGCENTRE = 255     !255=Missing
      ISUBCENTRE = 0
      CALL GETENV('ORIGINATING_CENTRE',ORIGINATING_CENTRE)
      READ(ORIGINATING_CENTRE,*,ERR=10,END=10) IORIGCENTRE
   10 CALL GETENV('SUB_CENTRE',SUB_CENTRE)
      READ(SUB_CENTRE,*,ERR=11,END=11) ISUBCENTRE
   11 CONTINUE

!Check whether the orbit number has been specified externally.
      cris_orbit = 0
      CALL GETENV('CRIS_ORBIT',CCRIS_ORBIT)
      IF (CCRIS_ORBIT(1:1).NE.' ') THEN
        read(CCRIS_ORBIT,*) cris_orbit
        write(*,*)'Orbit number specified as:',cris_orbit
      ENDIF

!Set the apodisation constants
      write(*,'(A,A)')'Apodisation=',CApod
      IF (CApod(1:1) .EQ. 'H') THEN          !Hamming
        NApod = 2
        DO J=1,NApod
          Apod(J) = Hamming(J)
        ENDDO
      ELSE IF (CApod(1:1) .EQ. 'B') THEN     !Blackman_Harris
        NApod = 3
        DO J=1,3
          Apod(J) = Blackman(J)
        ENDDO
      ELSE
        NApod = 0    !None
      ENDIF

!Read the general satellite info from the fdf.dat file on unit 20
      CALL INFDF (20, 0)      

      semiMinor = semiMajor - semiDiff

      IF (Mission_Name(1:3) .EQ. 'NPP') THEN
        cris1c_h_wmosatid = 224
      ELSE IF (Mission_Name(2:3) .EQ. '01' .or. 
     &         Mission_Name(2:3) .EQ. '20') THEN
        cris1c_h_wmosatid = 225    !JPSS-1, NOAA-20
      ELSE IF (Mission_Name(2:3) .EQ. '02' .or.
     &         Mission_Name(2:3) .EQ. '21') THEN
        cris1c_h_wmosatid = 226    !JPSS-2, NOAA-21
      ELSE
        cris1c_h_satid = MDI
        cris1c_h_wmosatid = MDI
      ENDIF
      cris1c_h_centre = IORIGCENTRE
      cris1c_h_subcentre = ISUBCENTRE
      cris1c_h_instrument = 620      !CRIS
      IF (cris_orbit .gt. 0) THEN
        cris1c_h_startorbit = cris_orbit
        cris1c_h_endorbit = cris_orbit
      ELSE
        cris1c_h_startorbit = StartOrbit
        cris1c_h_endorbit = EndOrbit
      ENDIF
      cris1c_h_scnlin = NScan
      cris1c_h_misscnlin = 0
!      cris1c_h_nchan = cris_1cnchan
      cris1c_h_nchan = 
     &  CRIS_NCHAN(1)+CRIS_NCHAN(2)+CRIS_NCHAN(3)-3*CRIS_SKIP
      write(*,*)'Number of channels in each band in 1c: ',
     &  cris_chans_lw-CRIS_SKIP, cris_chans_mw-CRIS_SKIP,
     &  cris_chans_sw-CRIS_SKIP
      write(*,*)'Total channels in 1c: ',cris1c_h_nchan
      cris1c_h_nchanused = cris1c_h_nchan
      cris1c_h_nfov = cris_1cnfov
      cris1c_h_nfor = cris_1cnfor
      cris1c_h_nband = cris_1cnband
      cris1c_h_recwords =array_size_cris1c -
     &  (cris_1cnchan-cris1c_h_nchan)*cris_1cnfov*cris_1cnfor
      cris1c_h_hdrcnt = 1
      cris1c_h_versnb = cris_1cvnnb
      cris1c_h_versyr = cris_1cvnyr
      cris1c_h_versdy = cris_1cvndy
      cris1c_h_npc = cris_1cnpc
      cris1c_h_npcused = 0
      cris1c_h_eigvn = 0
      
      READ (AggregateBeginningDate,'(I4,I2,I2)') startyear, startmonth,
     &        startday
      cris1c_h_startdatayr = startyear
      cris1c_h_startdatady = IDAYOFYEAR(startyear, startmonth, startday)
      cris1c_h_startdatatime = FORTime_millisec(1,1)

      READ (AggregateEndingDate,'(I4,I2,I2)') endyear, endmonth, endday
      cris1c_h_enddatayr = endyear
      cris1c_h_enddatady = IDAYOFYEAR(endyear, endmonth, endday)
      cris1c_h_enddatatime = FORTime_millisec(CRIS_FORS, NScan)

! Channel wavenumbers *100
      DO iband=1,CRIS_BANDS
        cris1c_h_startw(iband) = CRIS_FIRSTW(iband)*100
        cris1c_h_endw(iband) = (CRIS_FIRSTW(iband) + 
     &    (CRIS_NCHAN(iband)-CRIS_SKIP-1)*CRIS_STEPW(iband))*100.0 + 0.5
      ENDDO

      cris1c_h_startchan(1) = 1
      cris1c_h_endchan(1) = cris1c_h_startchan(1) + CRIS_CHANS_LW - 
     &                          CRIS_SKIP - 1
      cris1c_h_startchan(2) = cris1c_h_endchan(1) + 1
      cris1c_h_endchan(2) = cris1c_h_startchan(2) + CRIS_CHANS_MW - 
     &                          CRIS_SKIP - 1
      cris1c_h_startchan(3) = cris1c_h_endchan(2) + 1
      cris1c_h_endchan(3) = cris1c_h_startchan(3) + CRIS_CHANS_SW - 
     &                          CRIS_SKIP - 1

      DO ichan=1,cris1c_h_endchan(3)
        cris1c_h_channels(ichan) = ichan
      ENDDO

! Get nominal height and period from fdf file
      DO J=1,MaxSats
        IF (WMOid(J).EQ.cris1c_h_wmosatid) THEN
          cris1c_h_satht = SatHt(J)*10   !nominal ht, km*10
          cris1c_h_period = OrbitPd(J)   !period, seconds
          cris1c_h_satid = NOAAid(J)   !Satellite ID
        ENDIF
      ENDDO

      cris1c_h_software_version = Software_Version
      write(*,*) 'Software_Version:',cris1c_h_software_version

!Open the output file and write the header
      unit = 10
      CALL IOC1C("OPEN/WRITE",unit,0)
      CALL IOC1C("WRITE HEADER",unit,1)
      
      DO iscan=1,NScan
        cris1c_scnlin = iscan
        IF (FORTime_millisec(1,iscan) .GE. cris1c_h_startdatatime) THEN
          cris1c_scnlinyr = cris1c_h_startdatayr
          cris1c_scnlindy = cris1c_h_startdatady
        ELSE
          cris1c_scnlinyr = cris1c_h_enddatayr
          cris1c_scnlindy = cris1c_h_enddatady
        ENDIF
          
! Get the surface height, type and land fraction using SURFELEV

        CALL SURFELEV(LSunit, HTunit,
     &   Latitude(1,1,iscan), Longitude(1,1,iscan), CRIS_FORS*CRIS_FOVS,
     &   crisboxSize, surftype, elevation, landfrac )

! Loop over FORs and FOVs

        DO ifor=1,CRIS_FORS
          cris1c_scnlintime(ifor) = FORTime_millisec(ifor,iscan)
          DO ifov=1,CRIS_FOVS
            cris1c_latlon(1,ifov,ifor) = 
     &              NINT(1E4*Latitude(ifov,ifor,iscan))
            cris1c_latlon(2,ifov,ifor) = 
     &              NINT(1E4*Longitude(ifov,ifor,iscan))
            cris1c_angles(1,ifov,ifor) =
     &              NINT(1E2*SatelliteZenithAngle(ifov,ifor,iscan))
            cris1c_angles(2,ifov,ifor) =
     &              NINT(1E2*SatelliteAzimuthAngle(ifov,ifor,iscan))
            cris1c_angles(3,ifov,ifor) =
     &              NINT(1E2*SolarZenithAngle(ifov,ifor,iscan))
            cris1c_angles(4,ifov,ifor) =
     &              NINT(1E2*SolarAzimuthAngle(ifov,ifor,iscan))

            cris1c_surfht(ifov,ifor) = elevation(ifov,ifor)   !in metres
            cris1c_landfrac(ifov,ifor) = landfrac(ifov,ifor)  !in percent

!Set the surftype according to BUFR descriptor 008012, with 0=land, 100=sea,
!200=coast. Avoids changes to the BUFR encode/decode routines.
            IF (surftype(ifov,ifor).EQ.0) THEN      !Sea
              cris1c_landsea(ifov,ifor) = 100
            ELSE IF (surftype(ifov,ifor).EQ.1) THEN  !Mixed
              cris1c_landsea(ifov,ifor) = 200
            ELSE IF (surftype(ifov,ifor).EQ.2) THEN  !Land
              cris1c_landsea(ifov,ifor) = 0
            ELSE
              cris1c_landsea(ifov,ifor) = 300
            ENDIF

            cris1c_cloudcover(ifov,ifor) = MDI
            cris1c_cloudtopht(ifov,ifor) = MDI

!NUCAPS team were planning to implement some additional quality checking, but
!have not done so yet (email from Yi Song, Dec 2016).
            cris1c_quality(ifov,ifor) = 0
            IF (CApod(1:1) .EQ. 'H') THEN
              cris1c_radianceflgs(ifov,ifor) = 2048  !Bit 4 of 15 (BUFR def)
            ELSE IF (CApod(1:1) .EQ. 'N') THEN
              cris1c_radianceflgs(ifov,ifor) = 1024  !Bit 5 of 15 (BUFR def)
            ENDIF
            DO iband=1,CRIS_BANDS
              DO ichan=1,CRIS_NCHAN(iband)
                IF (iband.EQ.1) THEN
                  radiance(ichan) = ES_RealLW(ichan,ifov,ifor,iscan)
                ELSE IF (iband.EQ.2) THEN
                  radiance(ichan) = ES_RealMW(ichan,ifov,ifor,iscan)
                ELSE
                  radiance(ichan) = ES_RealSW(ichan,ifov,ifor,iscan)
                ENDIF
              ENDDO

       !Apodise

              IF (NApod.NE.0) THEN
                apod_radiance(1) = radiance(1)
                apod_radiance(2) = radiance(2)
                DO ichan=3,CRIS_NCHAN(iband)-2
                  apod_radiance(ichan) = Apod(1)*radiance(ichan) +
     &              Apod(2)*(radiance(ichan-1) + radiance(ichan+1))
                  IF (NApod.EQ.3)
     &              apod_radiance(ichan) = apod_radiance(ichan) +
     &                Apod(3)*(radiance(ichan-2) + radiance(ichan+2))
                ENDDO
                apod_radiance(ichan) = radiance(ichan)
                apod_radiance(ichan+1) = radiance(ichan+1)
              ENDIF
              
       !Save radiance in 1c structure. Output is W/m^2/str/cm^-1 x 1E7,
       !input is mW/m^2/str/cm^-1, so scale by 1E4.
       
              DO ichan=1,CRIS_NCHAN(iband)-CRIS_SKIP
                jchan = ichan + cris1c_h_startchan(iband) - 1
                IF (radiance(ichan+CRIS_HALFSKIP).GT.-999.0) THEN
                  IF (NApod.NE.0) THEN
                    cris1c_radiance(ifov,ifor,jchan) =
     &                 NINT(1E4*apod_radiance(ichan+CRIS_HALFSKIP))  !Apodised
                  ELSE
                    cris1c_radiance(ifov,ifor,jchan) =
     &                 NINT(1E4*radiance(ichan+CRIS_HALFSKIP))   !Not apodised
                  ENDIF
                ELSE
                  cris1c_radiance(ifov,ifor,jchan) = MDI
                ENDIF
              ENDDO

              DO i=1,2   !First 2 samples in each band
                j = (iband-1)*4 + i
                IF (radiance(i).GT.-999.0) THEN
                  cris1c_extraradiance(ifov,ifor,j) =  
     &              NINT(1E4*radiance(i))
                ELSE
                  cris1c_extraradiance(ifov,ifor,j) = MDI 
                ENDIF
              ENDDO      
              DO i=1,2   !Last 2 samples in each band
                j = (iband-1)*4 + 2 + i
                IF (radiance(CRIS_NCHAN(iband)-2+i).GT.-999.0) THEN
                  cris1c_extraradiance(ifov,ifor,j) =  
     &              NINT(1E4*radiance(CRIS_NCHAN(iband)-2+i))
                ELSE
                  cris1c_extraradiance(ifov,ifor,j) = MDI 
                ENDIF
              ENDDO      
            ENDDO   !iband     
          ENDDO   !ifov
        ENDDO   !ifor
        
        IF (SCVelocity(3,iscan) .GE. 0) THEN   !Z component of velocity
          cris1c_ascdesc = 0    !Ascending
        ELSE
          cris1c_ascdesc = 1    !Descending
        ENDIF
        cris1c_scanqualflgs = scanqual(iscan)
        DO iband=1,CRIS_BANDS
          DO ifov=1,CRIS_FOVS
            cris1c_calqualflgs(iband,ifov) = calqual(iband,ifov,iscan)
            DO ifor=1,CRIS_FORS
              cris1c_fovqualflgs(iband,ifov,ifor) = 
     &          fovqual(iband,ifov,ifor,iscan)
            ENDDO
          ENDDO
        ENDDO
        cris1c_geolocqual = geolocqual(iscan)

        cris1c_satxyz(1) = SCPosition(1,iscan)
        cris1c_satxyz(2) = SCPosition(2,iscan)
        cris1c_satxyz(3) = SCPosition(3,iscan)
        xysq = SCPosition(1,iscan)**2 + SCPosition(2,iscan)**2
        z = SCPosition(3,iscan)
        r_sat = sqrt(xysq + z**2)       !Centre of Earth to satellite
        lat = atan2(z,sqrt(xysq))
        r_ellipse = sqrt((semiMajor*cos(lat))**2 + 
     &                   (semiMinor*sin(lat))**2)
        cris1c_scalti = NINT((r_sat - r_ellipse)*0.01)    !km *10
        IF (cris_orbit .GT. 0) THEN
          cris1c_orbit = cris_orbit
        ELSE
          cris1c_orbit = orbit_numbers(iscan)
        ENDIF

        CALL IOC1C("WRITE DATA",unit,iscan+1)
      ENDDO  !iscan

      CALL IOC1C("CLOSE",unit,0)

      RETURN
      END

!-----------------------------------------------------------------------

      FUNCTION IDAYOFYEAR(IY,IM,ID)
      INTEGER    IDAYOFYEAR
      INTEGER    IY,IM,ID
      INTEGER    NOLEAP(12), LEAP(12)
      DATA       NOLEAP /0,31,59,90,120,151,181,212,243,273,304,334/
      DATA       LEAP   /0,31,60,91,121,152,182,213,244,274,305,335/
      IF (MOD(IY,4).EQ.0) THEN         !OK up to 2100
        IDAYOFYEAR = LEAP(IM) + ID
      ELSE
        IDAYOFYEAR = NOLEAP(IM) + ID
      ENDIF
      RETURN
      END
