/*
! name: $Id: cris_sdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an CRIS SDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    19/10/2010  Original code. (N.C.Atkinson)
!  1.1    16/05/2016  Extend to handle full-spectral-resolution files.
!                     The number of channels is determined from the ES_Real datasets.
!  1.2    12/12/2016  Add N_Software_Version
!  1.3    21/11/2017  Use attribute Platform_Short_Name instead of Mission_Name.
!                     Allow for FSR group and dataset names, CrIS-FS-SDR*.
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "cris_sdr.h"
#include <stdio.h>
#include <stdlib.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int cris_sdr_read(char *filename, char geo_char){
  
  int     igran,iscan,jscan,ifov,ifor,iband,ichan;       /* Loop variables */
  hid_t   file_id, group_id, dataset_id, attr_id;  /* handles */
  hid_t   strtype, dspace_id;
  herr_t  status;
  hsize_t dims[4];   /* size of the radiance array */
  char    name[80];
  int     FSR=0;
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannont open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */

/****** Read the top-level attributes ******/

  status = H5Tset_size (strtype, 4);   /* Set the length of the string */
  readattribute("Platform_Short_Name",file_id,strtype,Mission_Name);
  status = H5Tset_size (strtype, 79);
  if (geo_char == '/') {
    N_GEO_Ref[0] = '-';
    readattribute("N_GEO_Ref",file_id,strtype,N_GEO_Ref);
    if (N_GEO_Ref[0] == '-') {
      fprintf(stderr, "Error: N_GEO_Ref attribute not found: use -g option\n");
      exit(10);
    }
  }

/****** Open the group All_Data/CrIS-SDR_All ******/

  if (! H5Lexists(file_id, "All_Data/CrIS-SDR_All", H5P_DEFAULT)) FSR=1;

  if (FSR) group_id = H5Gopen(file_id, "All_Data/CrIS-FS-SDR_All", H5P_DEFAULT);
  else group_id = H5Gopen(file_id, "All_Data/CrIS-SDR_All", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group All_Data/CrIS-SDR_All\n");
    exit(10);
  }

/****** How many channels are there? ******/

  dataset_id = H5Dopen(group_id, "ES_RealLW", H5P_DEFAULT);
  dspace_id = H5Dget_space(dataset_id);
  H5Sget_simple_extent_dims(dspace_id, dims, NULL);
  cris_chans_lw = dims[3];
  H5Dclose(dataset_id);

  dataset_id = H5Dopen(group_id, "ES_RealMW", H5P_DEFAULT);
  dspace_id = H5Dget_space(dataset_id);
  H5Sget_simple_extent_dims(dspace_id, dims, NULL);
  cris_chans_mw = dims[3];
  H5Dclose(dataset_id);

  dataset_id = H5Dopen(group_id, "ES_RealSW", H5P_DEFAULT);
  dspace_id = H5Dget_space(dataset_id);
  H5Sget_simple_extent_dims(dspace_id, dims, NULL);
  cris_chans_sw = dims[3];
  H5Dclose(dataset_id);

  printf ("Number of channels in each band in SDR file: %d %d %d\n",cris_chans_lw,cris_chans_mw,cris_chans_sw);
  
/****** Read the data ******/

  readdataset("ES_RealLW", group_id, H5T_NATIVE_FLOAT, ES_RealLW);
  readdataset("ES_RealMW", group_id, H5T_NATIVE_FLOAT, ES_RealMW);
  readdataset("ES_RealSW", group_id, H5T_NATIVE_FLOAT, ES_RealSW);
  readdataset("QF1_SCAN_CRISSDR", group_id, H5T_NATIVE_UCHAR, QF1_SCAN_CRISSDR);
  readdataset("QF2_CRISSDR", group_id, H5T_NATIVE_UCHAR, QF2_CRISSDR);
  readdataset("QF3_CRISSDR", group_id, H5T_NATIVE_UCHAR, QF3_CRISSDR);
  readdataset("QF4_CRISSDR", group_id, H5T_NATIVE_UCHAR, QF4_CRISSDR);

  H5Gclose(group_id);

/****** Open the group Data_Products/CrIS-SDR ******/

  if (FSR) group_id  = H5Gopen(file_id, "Data_Products/CrIS-FS-SDR", H5P_DEFAULT);
  else group_id  = H5Gopen(file_id, "Data_Products/CrIS-SDR", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group Data_Products/CrIS-SDR\n");
    exit(10);
  }


/****** Read the aggregate attribute data ******/

  if (FSR) dataset_id = H5Dopen(group_id, "CrIS-FS-SDR_Aggr", H5P_DEFAULT);
  else dataset_id = H5Dopen(group_id, "CrIS-SDR_Aggr", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open dataset CrIS-SDR_Aggr\n");
    exit(10);
  }

  readattribute("AggregateNumberGranules",dataset_id,H5T_NATIVE_LONG,
    &AggregateNumberGranules);

  NScan = AggregateNumberGranules*CRIS_SCANS_PER_GRANULE;
  
  H5Dclose(dataset_id);

/****** Read information from the granules ******/

  for (igran=0; igran<AggregateNumberGranules; igran++){
    if (FSR) sprintf(name,"CrIS-FS-SDR_Gran_%d",igran);
    else sprintf(name,"CrIS-SDR_Gran_%d",igran);
    
    dataset_id = H5Dopen(group_id, name, H5P_DEFAULT);
    if (dataset_id < 0) {
      fprintf(stderr,"Cannot open dataset %s\n",name);
      exit(10);
    }
    readattribute("N_Beginning_Orbit_Number", dataset_id, H5T_NATIVE_LLONG,
           &N_Beginning_Orbit_Number);
    status = H5Tset_size (strtype, 16);   /* Set the length of the string */
    readattribute("N_Software_Version", dataset_id, strtype, N_Software_Version);

    for (iscan=0;iscan<CRIS_SCANS_PER_GRANULE;iscan++) {
      jscan = igran*CRIS_SCANS_PER_GRANULE + iscan;
      orbit_numbers[jscan] = N_Beginning_Orbit_Number;
    }
    H5Dclose(dataset_id);
  }
    
  H5Gclose(group_id);

/****** Process the quality flags ******/

/* Note that we define the bit order of the AAPP flags to be the same as the BUFR flags,
   with bit 1 being the most significant. In the SDR definition document (D34862-03E) 
   a "datum offset" of 0 means the least significant bit.
   In many cases one BUFR/AAPP flag corresponds to many SDR flags (e.g. where individual PRTs
   are flagged in the SDR). Here we set the BUFR/AAPP flag if one or more of the SDR flags is 
   non-zero. */

  for (iscan=0; iscan<NScan; iscan++){
    
    scanqual[iscan] = 0;

    if (QF1_SCAN_CRISSDR[iscan] & 1) scanqual[iscan]+=4096;  /*Data gap*/
    if (QF1_SCAN_CRISSDR[iscan] & 2) scanqual[iscan]+=2048;  /*Timing sequence error*/
    if (QF1_SCAN_CRISSDR[iscan] & 4) scanqual[iscan]+=1024;  /*Lambda monitored quality error*/
    if (QF1_SCAN_CRISSDR[iscan] & 8) scanqual[iscan]+=512 ;  /*Invalid instrument temperatures*/
    if (QF1_SCAN_CRISSDR[iscan] & 16) scanqual[iscan]+=256;  /*Excess thermal drift*/

    for (ifov=0; ifov<CRIS_FOVS; ifov++) for (iband=0; iband<CRIS_BANDS; iband++){
      
      calqual[iscan][ifov][iband] = 0;
      
      if (QF2_CRISSDR[iscan][ifov][iband] & 1) calqual[iscan][ifov][iband]+=256;   /*Lunar intrusion*/
      if (QF2_CRISSDR[iscan][ifov][iband] & 2) calqual[iscan][ifov][iband]+=128;   /*Lunar intrusion*/
      
    }
    
    for (ifor=0; ifor<CRIS_FORS; ifor++) for (ifov=0; ifov<CRIS_FOVS; ifov++) 
         for  (iband=0; iband<CRIS_BANDS; iband++){

      fovqual[iscan][ifor][ifov][iband] = 0;

      if (QF3_CRISSDR[iscan][ifor][ifov][iband] & 1) fovqual[iscan][ifor][ifov][iband]+=262144; /*Degraded SDR quality*/
      if (QF3_CRISSDR[iscan][ifor][ifov][iband] & 2) fovqual[iscan][ifor][ifov][iband]+=131072; /*Invalid SDR quality*/
      if (QF3_CRISSDR[iscan][ifor][ifov][iband] & 4) fovqual[iscan][ifor][ifov][iband]+=65536;  /*Invalid geolocation*/
      if (QF3_CRISSDR[iscan][ifor][ifov][iband] & 8) fovqual[iscan][ifor][ifov][iband]+=32768;  /*Degraded rad cal*/
      if (QF3_CRISSDR[iscan][ifor][ifov][iband] & 16) fovqual[iscan][ifor][ifov][iband]+=16384; /*Invalid rad cal*/
      if (QF3_CRISSDR[iscan][ifor][ifov][iband] & 32) fovqual[iscan][ifor][ifov][iband]+=8192;  /*Degraded spectral cal*/
      if (QF3_CRISSDR[iscan][ifor][ifov][iband] & 64) fovqual[iscan][ifor][ifov][iband]+=4096;  /*Invalid spectral cal*/
      if (QF3_CRISSDR[iscan][ifor][ifov][iband] & 128) fovqual[iscan][ifor][ifov][iband]+=2048; /*Fringe count error corrected*/
      if (QF4_CRISSDR[iscan][ifor][ifov][iband] & 1) fovqual[iscan][ifor][ifov][iband]+=1024;   /*1=night*/
      if (QF4_CRISSDR[iscan][ifor][ifov][iband] & 2) fovqual[iscan][ifor][ifov][iband]+=512;    /*Invalid RDR*/
      if (QF4_CRISSDR[iscan][ifor][ifov][iband] & 4) fovqual[iscan][ifor][ifov][iband]+=256;    /*Significant fringe count error*/
      if (QF4_CRISSDR[iscan][ifor][ifov][iband] & 8) fovqual[iscan][ifor][ifov][iband]+=128;    /*Bit trim failed*/
    }

  }

/****** Display ******/

  printf("Finished reading %s\n",filename);

/****** Clean up ******/

  H5Fclose(file_id);
  return 0;

}
#endif
