! Name: $Id: hiras_sdr_out.F 634 2019-04-08 09:46:08Z frna $
!
! Purpose: Fill MWHS 1c structure and write out file
!
! Current Code Owner: NWP SAF
!
! At FSR, the channel numbers in each band (excluding 2 extra samples at band edges) are:
!    HIRAS: 1-777, 778-1642, 1643-2275
!    CrIS:  1-713, 714-1578, 1579-2211
!
! The start wavenumbers are: 650.0, 1210.0, 2155.0 for both instruments.
! HIRAS has 64 extra spectral samples in band 1, compared with CrIS.
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    05/02/2019  Initial
!  1.1    01/04/2019  Move the first call to IOHIRAS1C; Daycnt is 2-D;
!                     increase capacity to 4 granules (20 minutes)
!
!-------------------------------------------------------------------------------

      SUBROUTINE HIRAS_SDR_OUT (
     &  Count_scnlines,hiras_chans_lw,hiras_chans_mw1,hiras_chans_mw2,
     &  Orbit_Number, Orbit_Period,
     &  ES_RealLW, ES_RealMW1, ES_RealMW2, 
     &  Latitude, Longitude, Daycnt, Mscnt,
     &  SensorZenith, SensorAzimuth, SolarZenith, SolarAzimuth,
     &  LandSeaMask, Land_Cover, Height, QA_Score, QA_flag_Process, 
     &  QA_flag_Scnline,  
     &  Beginning_Date, Beginning_Time, Ending_Date, Ending_Time,
     &  Satellite_Name, CApod)
     
      IMPLICIT NONE

! SDR parameters, also defined in hiras_sdr.h
      INTEGER   HIRAS_GRANULES, HIRAS_SCANS_PER_GRANULE, HIRAS_FORS,
     &   HIRAS_FOVS, HIRAS_SCANS, HIRAS_BANDS, HIRAS_TOTAL_STEPS
      PARAMETER (HIRAS_GRANULES=4)
      PARAMETER (HIRAS_SCANS_PER_GRANULE=30)
      PARAMETER (HIRAS_FORS=29)
      PARAMETER (HIRAS_FOVS=4)
      PARAMETER (HIRAS_BANDS=3)
      PARAMETER (HIRAS_TOTAL_STEPS=40)
      PARAMETER (HIRAS_SCANS=HIRAS_SCANS_PER_GRANULE*HIRAS_GRANULES)

! In going from SDR to AAPP level 1c we discard channels at edge of band (for
! consistency with CrIS).
      INTEGER HIRAS_SKIP, HIRAS_HALFSKIP
      PARAMETER (HIRAS_SKIP=4)
      PARAMETER (HIRAS_HALFSKIP=2)

! Band definitions for the AAPP level 1c file
      REAL HIRAS_FIRSTW(3), HIRAS_STEPW(3)
      DATA HIRAS_FIRSTW/650.0, 1210.0, 2155.0/     !Wavenumbers, same in FSR and non-FSR
      DATA HIRAS_STEPW/0.625, 1.25, 2.5/           !Changed later if FSR

      INTEGER HIRAS_NCHAN(3)

! Arguments: 
      INTEGER*4    Count_scnlines
      INTEGER*4    hiras_chans_lw, hiras_chans_mw1, hiras_chans_mw2
      INTEGER*4    Orbit_Number
      INTEGER*4    Orbit_Period
      REAL ES_RealLW(hiras_chans_lw,HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      REAL ES_RealMW1(hiras_chans_mw1,HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      REAL ES_RealMW2(hiras_chans_mw2,HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)

      INTEGER*2    Daycnt(HIRAS_TOTAL_STEPS,HIRAS_SCANS)
      INTEGER*4    Mscnt(HIRAS_TOTAL_STEPS,HIRAS_SCANS)
      INTEGER*2    LandSeaMask(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      INTEGER*2    Land_Cover(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      CHARACTER*11 Beginning_Date, Ending_Date
      CHARACTER*13 Beginning_Time, Ending_Time
      CHARACTER*6  Satellite_Name
      CHARACTER*20 CApod

      REAL      Latitude(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      REAL      Longitude(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      INTEGER*2 SensorAzimuth(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      INTEGER*2 SensorZenith(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      INTEGER*2 SolarAzimuth(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      INTEGER*2 SolarZenith(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)
      INTEGER*2 Height(HIRAS_FOVS,HIRAS_FORS,HIRAS_SCANS)

      INTEGER*2 QA_Score(HIRAS_SCANS)
      INTEGER*2 QA_flag_Process(HIRAS_SCANS)
      INTEGER*4 QA_flag_Scnline(HIRAS_TOTAL_STEPS, HIRAS_SCANS)

! Local variables

      INTEGER iscan,ifor,ifov,ichan,jchan,iband,unit,j,i,irec,firstscan
      INTEGER startyear, startmonth, startday
      INTEGER endyear, endmonth, endday
      INTEGER year, month, day, hour, minute
      REAL    xysq,x,y,z,lat,lon,rad,semiMinor,r_sat,r_ellipse
      REAL    xpos(HIRAS_SCANS),ypos(HIRAS_SCANS),zpos(HIRAS_SCANS)
      REAL    second
      REAL    radiance(869)      !largest radiance array
      REAL    apod_radiance(869)
      REAL    Hamming(3)
      DATA    Hamming /0.54,0.23,0.0/
      REAL    Blackman(3)
      DATA    Blackman /0.42323, 0.248775, 0.03961/
      REAL    Apod(3)
      INTEGER NApod       !Number of non-zero apodisation coefficients
      CHARACTER*10 CCRIS_ORBIT
      INTEGER hiras_orbit
      CHARACTER*5   ORIGINATING_CENTRE
      CHARACTER*5   SUB_CENTRE
      INTEGER IORIGCENTRE
      INTEGER ISUBCENTRE
      REAL    RMDI
      DATA    RMDI/-9999.99/

      INTEGER surftype(HIRAS_FOVS,HIRAS_FORS) !surface type for a scanline (SURFELEV)
      INTEGER elevation(HIRAS_FOVS,HIRAS_FORS)!elevation for a scanline (SURFELEV)
      INTEGER landfrac(HIRAS_FOVS,HIRAS_FORS) !land fraction for a scanline (SURFELEV)
      INTEGER hirasboxSize             ! size of grid-box used in SURFELEV
      PARAMETER ( hirasboxSize = 20 )  ! km
      INTEGER LSunit, HTunit
      PARAMETER (LSunit=51)
      PARAMETER (HTunit=52)

!Function
      INTEGER    IDAYOFYEAR

! Include
#include "hiras1c.h"
#include "inparms.h"
#include "constants_preproc.h"

! End of declarations
!-----------------------------------------------------------------------

      HIRAS_NCHAN(1) = hiras_chans_lw
      HIRAS_NCHAN(2) = hiras_chans_mw1
      HIRAS_NCHAN(3) = hiras_chans_mw2
      IF (hiras_chans_mw1 .GT. 437) HIRAS_STEPW(2) = HIRAS_STEPW(1)   !if FSR
      IF (hiras_chans_mw2 .GT. 163) HIRAS_STEPW(3) = HIRAS_STEPW(1)

! Set the originating centre and sub-centre
      IORIGCENTRE = 255     !255=Missing
      ISUBCENTRE = 0
      CALL GETENV('ORIGINATING_CENTRE',ORIGINATING_CENTRE)
      READ(ORIGINATING_CENTRE,*,ERR=10,END=10) IORIGCENTRE
   10 CALL GETENV('SUB_CENTRE',SUB_CENTRE)
      READ(SUB_CENTRE,*,ERR=11,END=11) ISUBCENTRE
   11 CONTINUE

!Set the apodisation constants
      write(*,'(A,A)')'Apodisation=',CApod
      IF (CApod(1:1) .EQ. 'H') THEN          !Hamming
        NApod = 2
        DO J=1,NApod
          Apod(J) = Hamming(J)
        ENDDO
      ELSE IF (CApod(1:1) .EQ. 'B') THEN     !Blackman_Harris
        NApod = 3
        DO J=1,3
          Apod(J) = Blackman(J)
        ENDDO
      ELSE
        NApod = 0    !None
      ENDIF

!Read the general satellite info from the fdf.dat file on unit 20
      CALL INFDF (20, 0)      

      IF (Satellite_Name(1:5) .EQ. "FY-3D") THEN
        hiras1c_h_satid = 4
        hiras1c_h_wmosatid = 523
      ELSE
        hiras1c_h_satid = MDI
        hiras1c_h_wmosatid = MDI
      ENDIF
      hiras1c_h_centre = IORIGCENTRE
      hiras1c_h_subcentre = ISUBCENTRE

      READ(Ending_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Ending_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      hiras1c_h_enddatayr = year
      hiras1c_h_enddatady = IDAYOFYEAR(year, month, day)
      hiras1c_h_enddatatime = 1000*(hour*3600 + minute*60 + second)
      
      READ(Beginning_Date,'(I4,1X,I2,1X,I2)') year, month, day
      READ(Beginning_Time,'(I2,1X,I2,1X,F6.3)') hour, minute, second
      hiras1c_h_startdatayr = year
      hiras1c_h_startdatady = IDAYOFYEAR(year, month, day)
      hiras1c_h_startdatatime = 1000*(hour*3600 + minute*60 + second)

!      hiras1c_h_scnlin = Count_scnlines
      
      hiras1c_h_scnlin = 0
      DO iscan=1,Count_scnlines
        if (Daycnt(1,iscan) .gt. 0 .and. 
     &     Daycnt(HIRAS_TOTAL_STEPS,iscan) .gt. 0 .and.
     &     Latitude(1,15,iscan) .le. 90.0) then
          hiras1c_h_scnlin = hiras1c_h_scnlin + 1
        endif
      ENDDO

! Channel wavenumbers *100
      DO iband=1,HIRAS_BANDS
        hiras1c_h_startw(iband) = HIRAS_FIRSTW(iband)*100
        hiras1c_h_endw(iband) = (HIRAS_FIRSTW(iband) + 
     &   (HIRAS_NCHAN(iband)-HIRAS_SKIP-1)*HIRAS_STEPW(iband))*100.0+0.5
      ENDDO

      hiras1c_h_startchan(1) = 1
      hiras1c_h_endchan(1) = hiras1c_h_startchan(1) + HIRAS_CHANS_LW - 
     &                          HIRAS_SKIP - 1
      hiras1c_h_startchan(2) = hiras1c_h_endchan(1) + 1
      hiras1c_h_endchan(2) = hiras1c_h_startchan(2) + HIRAS_CHANS_MW1 - 
     &                          HIRAS_SKIP - 1
      hiras1c_h_startchan(3) = hiras1c_h_endchan(2) + 1
      hiras1c_h_endchan(3) = hiras1c_h_startchan(3) + HIRAS_CHANS_MW2 - 
     &                          HIRAS_SKIP - 1

      DO ichan=1,hiras1c_h_endchan(3)
        hiras1c_h_channels(ichan) = ichan
      ENDDO

      hiras1c_h_instrument = 955
      hiras1c_h_startorbit = Orbit_Number
      hiras1c_h_endorbit = Orbit_Number
      hiras1c_h_satht = 8360      !nominal
      hiras1c_h_period = Orbit_Period*60
      
      hiras1c_h_nchan = 
     &  HIRAS_NCHAN(1)+HIRAS_NCHAN(2)+HIRAS_NCHAN(3)-3*HIRAS_SKIP
      write(*,*)'Number of channels in each band in 1c: ',
     &  hiras_chans_lw-HIRAS_SKIP, hiras_chans_mw1-HIRAS_SKIP,
     &  hiras_chans_mw2-HIRAS_SKIP
      write(*,*)'Total channels in 1c: ',hiras1c_h_nchan
      hiras1c_h_nchanused = hiras1c_h_nchan
      hiras1c_h_nfov = hiras_1cnfov
      hiras1c_h_nfor = hiras_1cnfor
      hiras1c_h_nband = hiras_1cnband
      hiras1c_h_recwords =array_size_hiras1c -
     &  (hiras_1cnchan-hiras1c_h_nchan)*hiras_1cnfov*hiras_1cnfor
      hiras1c_h_hdrcnt = 1
      hiras1c_h_versnb = hiras_1cvnnb
      hiras1c_h_versyr = hiras_1cvnyr
      hiras1c_h_versdy = hiras_1cvndy
      hiras1c_h_npc = hiras_1cnpc
      hiras1c_h_npcused = 0
      hiras1c_h_eigvn = 0

      write(*,*)'hiras1c_h_recwords=',hiras1c_h_recwords

!Open the l1c file
      unit = 10
      CALL IOHIRAS1C("OPEN/WRITE",unit,0)
      
      CALL IOHIRAS1C("WRITE HEADER",unit,1)

      hiras1c_scnlin = 0

!Estimate satellite position in Greenwich co-ords - not present in the hdf5
!Could get a more accurate value from the satpos file, but this will do for
!now. Ignore elipticity of earth. Use nominal satellite height.

      rad = rEarth*1000 + hiras1c_h_satht*100   !distance from centre of earth
      DO iscan=1,Count_scnlines
        lat = Latitude(1,15,iscan)
        lon = Longitude(1,15,iscan)
        IF (lat .LT. 32767) THEN
          call latlon_to_xyz(1,1,Lat,Lon,x,y,z)
          xpos(iscan) = rad*x
          ypos(iscan) = rad*y
          zpos(iscan) = rad*z
        ELSE
          xpos(iscan) = RMDI
          ypos(iscan) = RMDI
          zpos(iscan) = RMDI
        ENDIF
      ENDDO

!Main loop

      irec = 1
      firstscan = 0
      DO iscan=1,Count_scnlines
       if (Daycnt(1,iscan) .gt. 0 .and. 
     &     Daycnt(HIRAS_TOTAL_STEPS,iscan) .gt. 0 .and.
     &     Latitude(1,15,iscan) .le. 90.0) then
        if (firstscan .eq. 0) firstscan = iscan
        hiras1c_scnlin = iscan - firstscan + 1
        hiras1c_satxyz(1) = xpos(iscan)
        hiras1c_satxyz(2) = ypos(iscan)
        hiras1c_satxyz(3) = zpos(iscan)
        hiras1c_scalti = hiras1c_h_satht   !Nominal height
        hiras1c_ascdesc = 0                !0 = ascending, 1=descending
        if (iscan .eq. 1) then
          if (zpos(iscan+1) .lt. zpos(iscan)) hiras1c_ascdesc = 1
        else
          if (zpos(iscan) .lt. zpos(iscan-1)) hiras1c_ascdesc = 1
        endif

        DO ifor=1,HIRAS_FORS
          if (Daycnt(ifor,iscan) .gt. 0) then
            hiras1c_scnlintime(ifor) = Mscnt(ifor,iscan) + 43200000   !12 hour offset
            IF (hiras1c_scnlintime(ifor) .GE. 86400000) THEN
              hiras1c_scnlintime(ifor)=hiras1c_scnlintime(ifor)-86400000
            ENDIF
          else
            hiras1c_scnlintime(ifor) = MDI
          endif
        ENDDO
        IF (hiras1c_scnlintime(1) - hiras1c_h_startdatatime .LT. 
     &             -43200000) THEN
          hiras1c_scnlindy = hiras1c_h_enddatady
          hiras1c_scnlinyr = hiras1c_h_enddatayr
        ELSE
          hiras1c_scnlindy = hiras1c_h_startdatady
          hiras1c_scnlinyr = hiras1c_h_startdatayr
        ENDIF
        
        DO ifor=1,HIRAS_FORS
          DO ifov=1,HIRAS_FOVS
            hiras1c_latlon(1,ifov,ifor) = 
     &              NINT(1E4*Latitude(ifov,ifor,iscan))
            hiras1c_latlon(2,ifov,ifor) = 
     &              NINT(1E4*Longitude(ifov,ifor,iscan))
            hiras1c_angles(1,ifov,ifor) = SensorZenith(ifov,ifor,iscan)
            hiras1c_angles(2,ifov,ifor) = SensorAzimuth(ifov,ifor,iscan)
            hiras1c_angles(3,ifov,ifor) = SolarZenith(ifov,ifor,iscan)
            hiras1c_angles(4,ifov,ifor) = SolarAzimuth(ifov,ifor,iscan)

            IF (hiras1c_angles(2,ifov,ifor) .lt. 0)    !0 to 36000
     &        hiras1c_angles(2,ifov,ifor) = 
     &            hiras1c_angles(2,ifov,ifor) + 65536
            IF (hiras1c_angles(4,ifov,ifor) .lt. 0)    !0 to 36000
     &        hiras1c_angles(4,ifov,ifor) =
     &            hiras1c_angles(4,ifov,ifor) + 65536

            hiras1c_surfht(ifov,ifor) = height(ifov,ifor,iscan)       !metres
            hiras1c_landfrac(ifov,ifor) = Land_Cover(ifov,ifor,iscan) !percent

!Set the surftype according to BUFR descriptor 008012, with 0=land, 1=sea,
!2=coast, 3=inland (missing in BUFR)
            IF (LandSeaMask(ifov,ifor,iscan).EQ.3) THEN
              hiras1c_landsea(ifov,ifor) = 1                   !Sea
            ELSE IF (LandSeaMask(ifov,ifor,iscan).EQ.1) THEN
              hiras1c_landsea(ifov,ifor) = 0                   !Land
            ELSE IF (LandSeaMask(ifov,ifor,iscan).EQ.5) THEN
              hiras1c_landsea(ifov,ifor) = 2                   !Mixed
            ELSE IF (LandSeaMask(ifov,ifor,iscan).EQ.2) THEN
              hiras1c_landsea(ifov,ifor) = 3                   !Inland water
            ELSE
              hiras1c_landsea(ifov,ifor) = MDI
            ENDIF              

            hiras1c_cloudcover(ifov,ifor) = MDI
            hiras1c_cloudtopht(ifov,ifor) = MDI

            DO iband=1,HIRAS_BANDS
              DO ichan=1,HIRAS_NCHAN(iband)
                IF (iband.EQ.1) THEN
                  radiance(ichan) = ES_RealLW(ichan,ifov,ifor,iscan)
                ELSE IF (iband.EQ.2) THEN
                  radiance(ichan) = ES_RealMW1(ichan,ifov,ifor,iscan)
                ELSE
                  radiance(ichan) = ES_RealMW2(ichan,ifov,ifor,iscan)
                ENDIF
              ENDDO

       !Apodise

              IF (NApod.NE.0) THEN
                apod_radiance(1) = radiance(1)
                apod_radiance(2) = radiance(2)
                DO ichan=3,HIRAS_NCHAN(iband)-2
                  apod_radiance(ichan) = Apod(1)*radiance(ichan) +
     &              Apod(2)*(radiance(ichan-1) + radiance(ichan+1))
                  IF (NApod.EQ.3)
     &              apod_radiance(ichan) = apod_radiance(ichan) +
     &                Apod(3)*(radiance(ichan-2) + radiance(ichan+2))
                ENDDO
                apod_radiance(ichan) = radiance(ichan)
                apod_radiance(ichan+1) = radiance(ichan+1)
              ENDIF
              
       !Save radiance in 1c structure. Output is W/m^2/str/cm^-1 x 1E7,
       !input is mW/m^2/str/cm^-1, so scale by 1E4.
       
              DO ichan=1,HIRAS_NCHAN(iband)-HIRAS_SKIP
                jchan = ichan + hiras1c_h_startchan(iband) - 1
                IF (radiance(ichan+HIRAS_HALFSKIP).GT.-999.0) THEN
                  IF (NApod.NE.0) THEN
                    hiras1c_radiance(ifov,ifor,jchan) =
     &                 NINT(1E4*apod_radiance(ichan+HIRAS_HALFSKIP))  !Apodised
                  ELSE
                    hiras1c_radiance(ifov,ifor,jchan) =
     &                 NINT(1E4*radiance(ichan+HIRAS_HALFSKIP))   !Not apodised
                  ENDIF
                ELSE
                  hiras1c_radiance(ifov,ifor,jchan) = MDI
                ENDIF
              ENDDO

              DO i=1,2   !First 2 samples in each band
                j = (iband-1)*4 + i
                IF (radiance(i).GT.-999.0) THEN
                  hiras1c_extraradiance(ifov,ifor,j) =  
     &              NINT(1E4*radiance(i))
                ELSE
                  hiras1c_extraradiance(ifov,ifor,j) = MDI 
                ENDIF
              ENDDO      
              DO i=1,2   !Last 2 samples in each band
                j = (iband-1)*4 + 2 + i
                IF (radiance(HIRAS_NCHAN(iband)-2+i).GT.-999.0) THEN
                  hiras1c_extraradiance(ifov,ifor,j) =  
     &              NINT(1E4*radiance(HIRAS_NCHAN(iband)-2+i))
                ELSE
                  hiras1c_extraradiance(ifov,ifor,j) = MDI 
                ENDIF
              ENDDO 
            ENDDO   !iband     
          ENDDO   !ifov
        ENDDO   !ifor

        irec = irec + 1
        CALL IOHIRAS1C("WRITE DATA",unit,irec)
       ENDIF  !Daycnt>0
      ENDDO  !iscan

      
      WRITE(*,*)'Number of scan lines in input file: ', Count_scnlines
      WRITE(*,*)'Number of scan lines with valid time/geolocation: ',
     &  hiras1c_h_scnlin

      CALL IOHIRAS1C("CLOSE",unit,0)

      RETURN
      END

