/*
! name: $Id: hiras_sdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an HIRAS SDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    05/02/2019  Initial
!  1.1    02/04/2019  Use Daycnt and Mscnt instead of attributes
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "hiras_sdr.h"
#include <stdio.h>
#include <stdlib.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int hiras_sdr_read(char *filename){
  
  hid_t   file_id, group_id, dataset_id;        /* handles */
  hid_t   strtype, dspace_id;
  herr_t  status;
  hsize_t dims[4];   /* size of the radiance array */

  FILE *fp;
  char cmd[100];
  int i,j;
  int startday=0, endday=0, startsec=0, endsec=0;
  char dateformat[30]="+'%Y-%m-%d %H:%M:%S.000'";

  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannot open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */


/****** Open the group "/" ******/

  group_id = H5Gopen(file_id, "/", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }

/****** Read the data ******/

  status = H5Tset_size (strtype, 6);   /* Set the length of the string */
  readattribute("Satellite Name",group_id,strtype,Satellite_Name);

  status = H5Tset_size (strtype, 11);   /* Set the length of the string */
  readattribute("Observing Beginning Date",group_id,strtype,Beginning_Date);
  readattribute("Observing Ending Date",group_id,strtype,Ending_Date);

  status = H5Tset_size (strtype, 13);   /* Set the length of the string */
  readattribute("Observing Beginning Time",group_id,strtype,Beginning_Time);
  readattribute("Observing Ending Time",group_id,strtype,Ending_Time);

  readattribute("Number Of Scans", group_id, H5T_NATIVE_INT, &Count_scnlines);
  readattribute("Orbit Number", group_id, H5T_NATIVE_INT, &Orbit_Number);
  readattribute("Orbit Period(min.)", group_id, H5T_NATIVE_INT, &Orbit_Period);

/****** How many channels are there? ******/

  dataset_id = H5Dopen(group_id, "Data/ES_RealLW", H5P_DEFAULT);
  dspace_id = H5Dget_space(dataset_id);
  H5Sget_simple_extent_dims(dspace_id, dims, NULL);
  hiras_chans_lw = dims[3];
  H5Dclose(dataset_id);
  dataset_id = H5Dopen(group_id, "Data/ES_RealMW1", H5P_DEFAULT);
  dspace_id = H5Dget_space(dataset_id);
  H5Sget_simple_extent_dims(dspace_id, dims, NULL);
  hiras_chans_mw1 = dims[3];
  H5Dclose(dataset_id);
  dataset_id = H5Dopen(group_id, "Data/ES_RealMW2", H5P_DEFAULT);
  dspace_id = H5Dget_space(dataset_id);
  H5Sget_simple_extent_dims(dspace_id, dims, NULL);
  hiras_chans_mw2 = dims[3];
  H5Dclose(dataset_id);

  readdataset("Data/ES_RealLW", group_id, H5T_NATIVE_FLOAT, ES_RealLW);
  readdataset("Data/ES_RealMW1", group_id, H5T_NATIVE_FLOAT, ES_RealMW1);
  readdataset("Data/ES_RealMW2", group_id, H5T_NATIVE_FLOAT, ES_RealMW2);

/* QA flags commented out for now as the global and local array dimensions are
   different */
/*
  readdataset("QA/QA_Score", group_id, H5T_NATIVE_SHORT, QA_Score);
  readdataset("QA/QA_flag_Process", group_id, H5T_NATIVE_SHORT, QA_flag_Process);
  readdataset("QA/QA_flag_Scnline", group_id, H5T_NATIVE_INT, QA_flag_Scnline);
*/

  readdataset("Geolocation/Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
  readdataset("Geolocation/Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
  readdataset("Geolocation/Sensor_Zenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
  readdataset("Geolocation/Sensor_Azimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
  readdataset("Geolocation/Solar_Zenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
  readdataset("Geolocation/Solar_Azimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);
  readdataset("Geolocation/LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
  readdataset("Geolocation/Land_Cover", group_id, H5T_NATIVE_SHORT, Land_Cover);
  readdataset("Geolocation/Daycnt", group_id, H5T_NATIVE_SHORT, Daycnt);
  readdataset("Geolocation/Mscnt", group_id, H5T_NATIVE_INT, Mscnt);

  printf("Finished reading %s\n",filename);

/* The attributes "Observing Beginning Date" etc. are not reliable. So use Daycnt and Mscnt instead */

  for (i=0; i<Count_scnlines; i++) {
    for (j=0; j<HIRAS_TOTAL_STEPS; j++) {
      if (Daycnt[i][j] > 0){
        if (startday == 0) {
          startday = Daycnt[i][j];
          startsec = Mscnt[i][j]/1000;
        }
        endday = Daycnt[i][j];
        endsec = Mscnt[i][j]/1000;
      }
    }
  }

/* Convert start and end times to string using the Linux "date" command */

  sprintf(cmd,"date -d '2000-01-01 + %d days + %d seconds + 43200 seconds' %s",startday,startsec,dateformat);
  fp = popen(cmd, "r");
  fscanf(fp,"%s %s",Beginning_Date,Beginning_Time);
  printf("Start: %s %s\n",Beginning_Date,Beginning_Time);
  pclose(fp);

  sprintf(cmd,"date -d '2000-01-01 + %d days + %d seconds + 43200 seconds' %s",endday,endsec,dateformat);
  fp = popen(cmd, "r");
  fscanf(fp,"%s %s",Ending_Date,Ending_Time);
  printf("End:   %s %s\n",Ending_Date,Ending_Time);
  pclose(fp);

/****** Clean up ******/

  H5Gclose(group_id);
  H5Fclose(file_id);
  return 0;

}
#endif
