/* 
! Name: $Id: hrp1b_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $
!
! Purpose: Subroutine to write AVHRR 1c data to HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      09/10/2014  N C Atkinson
!  1.1      13/10/2015  Add calvis and calir
!  1.2      27/01/2016  Add full res geo option and nedt
!  1.3      04/10/2017  Change attribute "period_minutes" to "period_seconds"
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
 
#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <string.h>

#include "add_hdf5_attributes.h"
#include "avh1b_c.h"
#include "z_intex_51_to_2048.h"

int nspots;

int hrp1b_to_hdf5_write(char *filename, int clevel, int fullresgeo){ 

  char *hdf5_encoder_version = "$Id: hrp1b_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $";
  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[3];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         j;
  int         ichan;
  int         iscan;
  float       wavenumber[AVH_IRCHN];
  float       radtemp_offset[AVH_IRCHN];
  float       radtemp_slope[AVH_IRCHN];
  float       solfiltirrad[AVH_VISCHN];
  float       equifiltwidth[AVH_VISCHN];
  
  int         mask_done=0;
  int         fill=-999999;
  float       btscale=0.01;
  float       latlonscale=0.0001;
  float       anglescale=0.01;
  float       rollangscale=0.001;
  float       altiscale=0.1;
  float       slopescale=1.0E-10;
  float       interscale=1.0E-7;
  float       irscale_out[3]={1.0E-6, 1.0E-6, 1.0E-6};

  int *latitude_full;
  int *longitude_full;
  short *satzen_full;
  short *solzen_full;
  short *relazi_full;
  double H, lati[51],loni[51],azidi[51],zenSi[51],zensi[51];
  double lato[2048],lono[2048],azido[2048],zenSo[2048],zenso[2048];
  
  for (i=0; i<2; i++) if (irscale[i] > 0) {
    irscale_out[i] = 1.0;
    for (j=0; j<irscale[i]; j++) irscale_out[i] *= 0.1;
  }

  for (ichan=0; ichan<AVH_IRCHN; ichan++){
    wavenumber[ichan] = 0.001*h_radtempcnv[ichan][0];
    radtemp_offset[ichan] = 0.00001*h_radtempcnv[ichan][1];
    radtemp_slope[ichan] = 0.000001*h_radtempcnv[ichan][2];
  }
  wavenumber[0] = 0.01*h_radtempcnv[0][0]; /* scaling for channel 3B */

  for (ichan=0; ichan<AVH_VISCHN; ichan++){
    solfiltirrad[ichan] = 0.1*h_albcnv[ichan][0];
    equifiltwidth[ichan] = 0.001*h_albcnv[ichan][1];
  }

/* Create the HDF5 file */

  printf("Creating %s\n",filename);
  file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

/* Global attributes */

  attdims[0] = 1;

  add_string_attribute(file_id, "filename", filename);
  add_string_attribute(file_id, "hdf5_encoder_software_version", hdf5_encoder_version);
  add_string_attribute(file_id, "siteid", h_siteid);
  add_short_attribute(file_id, "l1bversnb", &h_l1bversnb, attdims);
  add_short_attribute(file_id, "l1bversyr", &h_l1bversyr, attdims);
  add_short_attribute(file_id, "l1bversdy", &h_l1bversdy, attdims);
  add_string_attribute(file_id, "dataname", h_dataname);
  add_string_attribute(file_id, "prblkid", h_prblkid);
  add_short_attribute(file_id, "satid", &h_satid, attdims);
  add_short_attribute(file_id, "instid", &h_instid, attdims);
  add_short_attribute(file_id, "datatyp", &h_datatyp, attdims);
  add_string_attribute(file_id, "datatyp_key", "1 = LAC; 2 = GAC; 3 = HRPT; 4=FRAC");
  add_short_attribute(file_id, "tipsrc", &h_tipsrc, attdims);
  add_int_attribute(file_id, "startdatajd", &h_startdatajd, attdims);
  add_short_attribute(file_id, "startdatayr", &h_startdatayr, attdims);
  add_short_attribute(file_id, "startdatady", &h_startdatady, attdims);
  add_int_attribute(file_id, "startdatatime_ms", &h_startdatatime, attdims);
  add_int_attribute(file_id, "enddatajd", &h_enddatajd, attdims);
  add_short_attribute(file_id, "enddatayr", &h_enddatayr, attdims);
  add_short_attribute(file_id, "enddatady", &h_enddatady, attdims);
  add_int_attribute(file_id, "enddatatime_ms", &h_enddatatime, attdims);
  add_short_attribute(file_id, "cpidsyr", &h_cpidsyr, attdims);
  add_short_attribute(file_id, "cpidsdy", &h_cpidsdy, attdims);
  add_int_attribute(file_id, "inststat1", &h_inststat1, attdims);
  add_short_attribute(file_id, "statchrecnb", &h_statchrecnb, attdims);
  add_int_attribute(file_id, "inststat2", &h_inststat1, attdims);
  add_ushort_attribute(file_id, "scnlin", &h_scnlin, attdims);
  add_ushort_attribute(file_id, "misscnlin", &h_misscnlin, attdims);
  add_ushort_attribute(file_id, "callocscnlin", &h_callocscnlin, attdims);
  add_ushort_attribute(file_id, "datagaps", &h_datagaps, attdims);
  add_ushort_attribute(file_id, "okdatafr", &h_okdatafr, attdims);
  add_ushort_attribute(file_id, "pacsparityerr", &h_pacsparityerr, attdims);
  add_ushort_attribute(file_id, "auxsyncerrsum", &h_auxsyncerrsum, attdims);
  add_ushort_attribute(file_id, "timeseqerr", &h_timeseqerr, attdims);
  add_ushort_attribute(file_id, "timeseqerrcode", &h_timeseqerrcode, attdims);
  add_ushort_attribute(file_id, "socclockupind", &h_socclockupind, attdims);
  add_ushort_attribute(file_id, "locerrind", &h_locerrind, attdims);
  add_ushort_attribute(file_id, "locerrcode", &h_locerrcode, attdims);
  add_ushort_attribute(file_id, "pacsstatfield", &h_pacsstatfield, attdims);
  add_ushort_attribute(file_id, "pacsdatasrc", &h_pacsdatasrc, attdims);
  add_ushort_attribute(file_id, "ramp_cal_indicator", &h_racalind, attdims);
  add_ushort_attribute(file_id, "solarcalyr", &h_solarcalyr, attdims);
  add_ushort_attribute(file_id, "solarcaldy", &h_solarcaldy, attdims);
  add_ushort_attribute(file_id, "primary_cal_alg_id", &h_pcalalgind, attdims);
  add_ushort_attribute(file_id, "primary_cal_alg_option", &h_pcalalgopt, attdims);
  add_ushort_attribute(file_id, "secondary_cal_alg_id", &h_scalalgind, attdims);
  add_ushort_attribute(file_id, "secondary_cal_alg_option", &h_scalalgopt, attdims);
  if (strlen(h_modelid) == 0) {
    h_modelid[0] = ' ';
    h_modelid[1] = '\0';
  }
  add_string_attribute(file_id, "modelid", h_modelid);
  float nadir_location_tolerance_km = h_nadloctol*0.1;
  add_float_attribute(file_id, "nadir_location_tolerance_km", &nadir_location_tolerance_km, attdims);
  add_short_attribute(file_id, "locbit", &h_locbit, attdims);
  float rollerr_deg = h_rollerr*1.0E-3;
  add_float_attribute(file_id, "rollerr_deg", &rollerr_deg, attdims);
  float pitcherr_deg = h_pitcherr*1.0E-3;
  add_float_attribute(file_id, "pitcherr_deg", &pitcherr_deg, attdims);
  float yawerr_deg = h_yawerr*1.0E-3;
  add_float_attribute(file_id, "yawerr_deg", &yawerr_deg, attdims);

  float irttcoef[4][6];
  for (i=0; i<4; i++){
    irttcoef[i][0] = h_irttcoef[i][0]*1.0E-2;
    irttcoef[i][1] = h_irttcoef[i][1]*1.0E-5;
    irttcoef[i][2] = h_irttcoef[i][2]*1.0E-8;
    irttcoef[i][3] = h_irttcoef[i][3]*1.0E-11;
    irttcoef[i][4] = h_irttcoef[i][4]*1.0E-14;
    irttcoef[i][5] = h_irttcoef[i][5]*1.0E-17;
  }
  attdims[0] = 4;   /* 4 PRTs */
  attdims[1] = 6;   /* 6 coefs */
  dataspace_id = H5Screate_simple(2, attdims, NULL);
  attribute_id = H5Acreate(file_id, "irttcoef", H5T_IEEE_F32LE, dataspace_id, H5P_DEFAULT, H5P_DEFAULT);
  status = H5Awrite(attribute_id, H5T_NATIVE_FLOAT, irttcoef);
  H5Aclose(attribute_id);
  H5Sclose(dataspace_id);

  attdims[0] = AVH_VISCHN;
  add_float_attribute(file_id, "AVHRR_solfiltirrad", solfiltirrad, attdims);
  add_float_attribute(file_id, "AVHRR_equifiltwidth", equifiltwidth, attdims);
  attdims[0] = AVH_IRCHN;
  add_float_attribute(file_id, "AVHRR_wavenumber", wavenumber, attdims);
  add_float_attribute(file_id, "AVHRR_radtemp_offset", radtemp_offset, attdims);
  add_float_attribute(file_id, "AVHRR_radtemp_slope", radtemp_slope, attdims);
  attdims[0] = 1;

  add_short_attribute(file_id, "epoch_year", &h_epoyr, attdims);
  add_short_attribute(file_id, "epoch_day", &h_epody, attdims);
  add_int_attribute(file_id, "epoch_time_ms", &h_epotime, attdims);
  add_int_attribute(file_id, "semi_major_axis_cm", &h_smaxis, attdims);
  float eccen = h_eccen*1.0E-8;
  add_float_attribute(file_id, "eccentricity", &eccen, attdims);
  float inclination_deg = h_incli*1.0E-5;
  add_float_attribute(file_id, "inclination_deg", &inclination_deg, attdims);
  float argper_deg = h_argper*1.0E-5;
  add_float_attribute(file_id, "argper_deg", &argper_deg, attdims);
  float rascnod_deg = h_rascnod*1.0E-5;
  add_float_attribute(file_id, "right_ascension_ascending_node_deg", &rascnod_deg, attdims);
  float manom_deg = h_manom*1.0E-5;
  add_float_attribute(file_id, "mean_anomaly_deg", &manom_deg, attdims);
  add_int_attribute(file_id, "xpos_cm", &h_xpos, attdims);
  add_int_attribute(file_id, "ypos_cm", &h_ypos, attdims);
  add_int_attribute(file_id, "zpos_cm", &h_zpos, attdims);
  add_int_attribute(file_id, "xvel_cm/sec", &h_xvel, attdims);
  add_int_attribute(file_id, "yvel_cm/sec", &h_yvel, attdims);
  add_int_attribute(file_id, "zvel_cm/sec", &h_zvel, attdims);
  float earthsun_AU = h_earthsun*1.0E-6;
  add_float_attribute(file_id, "earthsun_AU", &earthsun_AU, attdims);

/* Group /Data */

  group_id = H5Gcreate(file_id, "/Data", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets with one value per scan */

  rank = 1;
  dims[0] = h_scnlin;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/scnlin", H5T_STD_U16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_USHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlin);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinyr", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinyr);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlindy", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlindy);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/clockdrift", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, clockdrift);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlintime", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlintime);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinbit", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinbit);
  status = add_string_attribute(dataset_id, "Description", "Bit 0: 0=3A, 1=3B; bit 13: 1=corrected for TIP attitude; bit 14: 1=corrected for clock drift; bit 15: 0=ascending, 1=descending");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/qualind", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, qualind);
  status = add_string_attribute(dataset_id, "Description", "Quality indicator: see AAPP Formats document");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinqual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinqual);
  status = add_string_attribute(dataset_id, "Description", "Scan line quality: see AAPP Formats document");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/cbiterr", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, cbiterr);
  status = add_string_attribute(dataset_id, "Description", "Count of bit errors in frame sync");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/navstat", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, navstat);
  status = add_string_attribute(dataset_id, "Description", "Navigation Status Bit Field, see AAPP documentation");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/attangtime", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, attangtime);
  status = add_string_attribute(dataset_id, "Description", "Time associated with TIP Euler angles");
  status = add_string_attribute(dataset_id, "Units", "second");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/rollang", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, rollang);
  status = add_string_attribute(dataset_id, "Description", "Roll angle");
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &rollangscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/pitchang", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, pitchang);
  status = add_string_attribute(dataset_id, "Description", "Pitch angle");
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &rollangscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/yawang", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, yawang);
  status = add_string_attribute(dataset_id, "Description", "Yaw angle");
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &rollangscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scalti", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scalti);
  status = add_string_attribute(dataset_id, "Description", "Spacecraft Altitude (MSL)");
  status = add_string_attribute(dataset_id, "Units", "km");
  status = add_float_attribute(dataset_id, "Scale", &altiscale, attdims);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Calibration quality and NEdT */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = AVH_IRCHN;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/calqual", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calqual);
  status = add_string_attribute(dataset_id, "Description", "Calibration quality: see AAPP Formats document");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/nedt", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, nedt);
  status = add_string_attribute(dataset_id, "Description", "NEdT at 300K for channels 3B, 4, 5");
  status = add_string_attribute(dataset_id, "Units", "K");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* telem */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = 103;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/telem", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, telem);
  status = add_string_attribute(dataset_id, "Description", 
    "The 103 first 10 bit words from HRPT, packed three (10 bit) words in four bytes, right justified.");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset hrpt */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = nspots;
  dims[2] = AVH_CHANS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/hrpt", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, hrpt);
  status = add_string_attribute(dataset_id, "Description", "Earth view counts");

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Datasets calvis and calir */
/* Separate the calvis into separate datasets for slope1, intercept1, slope2, intercept2, intersection, where
   the slopes are unsigned and the intercepts are signed */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = AVH_VISCHN;
  dims[2] = 3;     /* operational, test, pre-launch */
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/calvis_slope1", H5T_STD_U32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calvis);
  status = add_string_attribute(dataset_id, "Description", "Slope 1 [operational, test, pre-launch] for each visible channel");
  status = add_float_attribute(dataset_id, "Scale", &slopescale, attdims);

  dataset_id = H5Dcreate(file_id, "/Data/calvis_intercept1", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calvis+h_scnlin*AVH_VISCHN*3);
  status = add_string_attribute(dataset_id, "Description", "Intercept 1 [operational, test, pre-launch] for each visible channel");
  status = add_float_attribute(dataset_id, "Scale", &interscale, attdims);

  dataset_id = H5Dcreate(file_id, "/Data/calvis_slope2", H5T_STD_U32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_UINT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calvis+2*h_scnlin*AVH_VISCHN*3);
  status = add_string_attribute(dataset_id, "Description", "Slope 2 [operational, test, pre-launch] for each visible channel");
  status = add_float_attribute(dataset_id, "Scale", &slopescale, attdims);

  dataset_id = H5Dcreate(file_id, "/Data/calvis_intercept2", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calvis+3*h_scnlin*AVH_VISCHN*3);
  status = add_string_attribute(dataset_id, "Description", "Intercept 2 [operational, test, pre-launch] for each visible channel");
  status = add_float_attribute(dataset_id, "Scale", &interscale, attdims);

  dataset_id = H5Dcreate(file_id, "/Data/calvis_intersection", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calvis+4*h_scnlin*AVH_VISCHN*3);
  status = add_string_attribute(dataset_id, "Description", "Intersection [operational, test, pre-launch] for each visible channel");

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

  rank = 4;
  dims[0] = h_scnlin;
  dims[1] = AVH_IRCHN;
  dims[2] = 2;     /* operational, pre-launch */
  dims[3] = 3;     /* a0, a1, a2 */
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/calir", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, calir);
  status = add_string_attribute(dataset_id, "Description", "[a2, a1, a0], [operational, pre-launch] for each IR channel");
  attdims[0] = 3;
  status = add_float_attribute(dataset_id, "Scale a2, a1, a0", irscale_out, attdims);
  attdims[0] = 1;

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

  H5Pclose(plist);                    
  H5Gclose(group_id);

/* Group /Geolocation */

  group_id = H5Gcreate(file_id, "/Geolocation", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets latitude and longitude */

  /* interpolate 51 -> 2048 points if requested */
  if (fullresgeo) {
    latitude_full = (int*)calloc(h_scnlin*AVH_SPOTS, sizeof(int));
    longitude_full = (int*)calloc(h_scnlin*AVH_SPOTS, sizeof(int));
    satzen_full = (short*)calloc(h_scnlin*AVH_SPOTS, sizeof(short));
    solzen_full = (short*)calloc(h_scnlin*AVH_SPOTS, sizeof(short));
    relazi_full = (short*)calloc(h_scnlin*AVH_SPOTS, sizeof(short));
    for (iscan=0; iscan<h_scnlin; iscan++){
      H = scalti[iscan]*0.1;
      i = iscan*51;
      for (j=0; j<51; j++){
        lati[j] = 1.0E-4*latitude[j+i];
        loni[j] = 1.0E-4*longitude[j+i];
        azidi[j] = 1.0E-2*relazi[j+i];
        zenSi[j] = 1.0E-2*solzen[j+i];
        zensi[j] = 1.0E-2*satzen[j+i];
      }
      z_intex_51_to_2048 (H, azidi, zenSi, zensi, loni, lati, azido, zenSo, zenso, lono, lato);
      i = iscan*2048;
      for (j=0; j<2048; j++){
        latitude_full[j+i] = 10000 * lato[j];
        longitude_full[j+i] = 10000 * lono[j];
        relazi_full[j+i] = 100 * azido[j];
        solzen_full[j+i] = 100 * zenSo[j];
        satzen_full[j+i] = 100 * zenso[j];
      }
    }
    dims[1] = AVH_SPOTS;
  }
  else dims[1] = AVH_TIE;

  rank = 2;
  dims[0] = h_scnlin;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Geolocation/Latitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  if (fullresgeo) {
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude_full);
    status = add_string_attribute(dataset_id, "Description", "Latitude all spots");
  } else {
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude);
    status = add_string_attribute(dataset_id, "Description", "Latitude spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Longitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  if (fullresgeo) {
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude_full);
    status = add_string_attribute(dataset_id, "Description", "Longitude all spots");
  } else {
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude);
    status = add_string_attribute(dataset_id, "Description", "Longitude spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* Satellite and solar angles */

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  if (fullresgeo) {
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen_full);
    status = add_string_attribute(dataset_id, "Description", "Satellite zenith angle all spots");
  } else {
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen);
    status = add_string_attribute(dataset_id, "Description", "Satellite zenith angle spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  if (fullresgeo) {
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen_full);
    status = add_string_attribute(dataset_id, "Description", "Solar zenith angle all spots");
  } else {
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen);
    status = add_string_attribute(dataset_id, "Description", "Solar zenith angle spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Relative_azimuth_angle", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  if (fullresgeo) {
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, relazi_full);
    status = add_string_attribute(dataset_id, "Description", "Relative azimuth angle all spots");
  } else {
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, relazi);
    status = add_string_attribute(dataset_id, "Description", "Relative azimuth angle spots 25 to 2025 every 40 spots");
  }
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  H5Dclose(dataset_id);

/* close the group and file */

  H5Sclose(dataspace_id);
  H5Pclose(plist);                    
  H5Gclose(group_id);
  H5Fclose(file_id);
}

#else

int hrp1b_to_hdf5_write(char *filename, int clevel, int fullresgeo){ 

  printf ("Error: unable to write HDF-5\n");
  exit(1);

}
#endif
