! Name: $Id: hrs1c_read_f.F 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read HIRS l1c file and pass information back to C calling routine.
!          Subroutines HRS1C_READ_HEADER and HRS1C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    19/09/2014  N C Atkinson
!  1.1    16/11/2015  Add coldnedt, warmnedt
!-----------------------------------------------------------------------

      SUBROUTINE HRS1C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_tempradcnv,
     & h_20solfiltirrad, h_20equifiltwidth, h_wmosatid,
     & h_coldnedt, h_warmnedt)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 HIRS_CHANS
      PARAMETER (HIRS_CHANS=20)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_tempradcnv(3,HIRS_CHANS-1)
      INTEGER*4     h_20solfiltirrad
      INTEGER*4     h_20equifiltwidth
      INTEGER*4     h_wmosatid
      INTEGER*4     h_coldnedt(HIRS_CHANS)
      INTEGER*4     h_warmnedt(HIRS_CHANS)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "hrs1c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open HIRS 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=hrs_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read HIRS header'
      CALL IOH1C("READ HEADER",UNIT_1C,1)

      h_scnlin = hrs1c_h_scnlin
      h_site = hrs1c_h_site
      h_1bsite = hrs1c_h_1bsite
      h_versnb = hrs1c_h_versnb
      h_satid = hrs1c_h_satid
      h_instrument = hrs1c_h_instrument
      h_satht = hrs1c_h_satht
      h_period = hrs1c_h_period
      h_startorbit = hrs1c_h_startorbit
      h_startdatayr = hrs1c_h_startdatayr
      h_startdatady = hrs1c_h_startdatady
      h_startdatatime = hrs1c_h_startdatatime
      h_endorbit = hrs1c_h_endorbit
      h_enddatayr = hrs1c_h_enddatayr
      h_enddatady = hrs1c_h_enddatady
      h_enddatatime = hrs1c_h_enddatatime
      h_scnlin = hrs1c_h_scnlin
      h_misscnlin = hrs1c_h_misscnlin
      do ichan=1,HIRS_CHANS-1
        do i=1,3
          h_tempradcnv(i,ichan) = hrs1c_h_tempradcnv(i,ichan)
        enddo
        h_coldnedt(ichan) = hrs1c_h_coldnedt(ichan)
        h_warmnedt(ichan) = hrs1c_h_warmnedt(ichan)
      enddo
      h_20solfiltirrad = hrs1c_h_20solfiltirrad
      h_20equifiltwidth = hrs1c_h_20equifiltwidth
      h_wmosatid = hrs1c_h_wmosatid

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE HRS1C_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime, mjfrcnt,
     &  qualind, scanqual, chanqual, instrtemp,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  scalti, btemps, dataqual, coldnedt, warmnedt)
      IMPLICIT NONE

! Parameters
      INTEGER*4 HIRS_CHANS
      PARAMETER (HIRS_CHANS=20)
      INTEGER*4 HIRS_SPOTS
      PARAMETER (HIRS_SPOTS=56)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 mjfrcnt(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 chanqual(HIRS_CHANS,h_scnlin)
      INTEGER*4 instrtemp(h_scnlin)
      INTEGER*4 latitude(HIRS_SPOTS,h_scnlin)
      INTEGER*4 longitude(HIRS_SPOTS,h_scnlin)
      INTEGER*4 satzen(HIRS_SPOTS,h_scnlin)
      INTEGER*4 satazi(HIRS_SPOTS,h_scnlin)
      INTEGER*4 solzen(HIRS_SPOTS,h_scnlin)
      INTEGER*4 solazi(HIRS_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 btemps(HIRS_CHANS,HIRS_SPOTS,h_scnlin)
      INTEGER*4 dataqual(HIRS_SPOTS,h_scnlin)
      INTEGER*4 coldnedt(HIRS_CHANS,h_scnlin)
      INTEGER*4 warmnedt(HIRS_CHANS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot

#include "inparms.h"
#include "hrs1c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read HIRS data records'
      DO iscan=1,h_scnlin
        CALL IOH1C("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = hrs1c_scnlin
        scnlinyr(iscan) = hrs1c_scnlinyr
        scnlindy(iscan) = hrs1c_scnlindy
        scnlintime(iscan) = hrs1c_scnlintime
        qualind(iscan) = hrs1c_qualind
        scanqual(iscan) = hrs1c_scanqual
        instrtemp(iscan) = hrs1c_instrtemp
        scalti(iscan) = hrs1c_scalti
        DO ichan=1,HIRS_CHANS
          chanqual(ichan,iscan) = hrs1c_chanqual(ichan)
          coldnedt(ichan,iscan) = hrs1c_coldnedt(ichan)
          warmnedt(ichan,iscan) = hrs1c_warmnedt(ichan)
        ENDDO
        DO ispot=1,HIRS_SPOTS
          DO ichan=1,HIRS_CHANS
            btemps(ichan,ispot,iscan) = hrs1c_btemps(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = hrs1c_latlon(1,ispot)
          longitude(ispot,iscan) = hrs1c_latlon(2,ispot)
          satzen(ispot,iscan) = hrs1c_angles(1,ispot)
          satazi(ispot,iscan) = hrs1c_angles(2,ispot)
          solzen(ispot,iscan) = hrs1c_angles(3,ispot)
          solazi(ispot,iscan) = hrs1c_angles(4,ispot)
          dataqual(ispot,iscan) = hrs1c_dataqual(ispot)
        ENDDO
      ENDDO

      CALL IOH1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
