/* 
! Name: $Id: iasi1c_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $
!
! Purpose: Subroutine to write IASI 1c data to HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      30/09/2014  N C Atkinson
!  1.1      04/10/2017  Change attribute "period_minutes" to "period_seconds"
!  1.2      08/01/2019  Add scalti
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
 
#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <string.h>
#include <math.h>

#include "add_hdf5_attributes.h"
#include "iasi1c_c.h"

int iasi1c_to_hdf5_write(char *filename, int clevel){ 

  char *hdf5_encoder_version = "$Id: iasi1c_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $";
  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[5];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         ichan;
  int         ifor;
  int         ifov;
  
  int         fill=-999999;
  int         avhfill=255;
  float       latlonscale=0.0001;
  float       anglescale=0.01;
  float       scalefactor[10];
  float       avhrrscale=0.000001;
  float       altiscale=0.1;
  
  for (i=0; i<10; i++) {
    if (h_scalefactor[i] > 0) scalefactor[i] = pow(10.0,(-h_scalefactor[i]));
    else scalefactor[i] = 0.0;
  }

/* Create the HDF5 file */

  printf("Creating %s\n",filename);
  file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

/* Global attributes */

  attdims[0] = 1;

  add_string_attribute(file_id, "filename", filename);
  add_string_attribute(file_id, "site", h_site);
  add_string_attribute(file_id, "1bsite", h_1bsite);
  add_string_attribute(file_id, "hdf5_encoder_software_version", hdf5_encoder_version);
  add_int_attribute(file_id, "satid", &h_satid, attdims);
  add_int_attribute(file_id, "instrument", &h_instrument, attdims);
  add_int_attribute(file_id, "satht_hectometres", &h_satht, attdims);
  add_int_attribute(file_id, "period_seconds", &h_period, attdims);
  add_int_attribute(file_id, "startorbit", &h_startorbit, attdims);
  add_int_attribute(file_id, "startdatayr", &h_startdatayr, attdims);
  add_int_attribute(file_id, "startdatady", &h_startdatady, attdims);
  add_int_attribute(file_id, "startdatatime_ms", &h_startdatatime, attdims);
  add_int_attribute(file_id, "endorbit", &h_endorbit, attdims);
  add_int_attribute(file_id, "enddatayr", &h_enddatayr, attdims);
  add_int_attribute(file_id, "enddatady", &h_enddatady, attdims);
  add_int_attribute(file_id, "enddatatime_ms", &h_enddatatime, attdims);
  add_int_attribute(file_id, "scnlin", &h_scnlin, attdims);
  add_int_attribute(file_id, "misscnlin", &h_misscnlin, attdims);
  add_int_attribute(file_id, "wmosatid", &h_wmosatid, attdims);


/* Group /Data */

  group_id = H5Gcreate(file_id, "/Data", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets with one value per scan */

  rank = 1;
  dims[0] = h_scnlin;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/scnlin", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlin);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/granule", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, granule);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinyr", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinyr);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlindy", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlindy);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/avhcombination", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhcombination);
  status = add_string_attribute(dataset_id, "Description", "AVHRR channel combination");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisQualIndex", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisQualIndex);
  status = add_string_attribute(dataset_id, "Description", "NEDT estimated/expected");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisQualIndexLoc", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisQualIndexLoc);
  status = add_string_attribute(dataset_id, "Description", "IIS/AVHRR co-registration uncertainty");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisQualIndexRad", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisQualIndexRad);
  status = add_string_attribute(dataset_id, "Description", "NEDT estimated/expected radiometric");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisQualIndexSpect", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisQualIndexSpect);
  status = add_string_attribute(dataset_id, "Description", "NEDT estimated/expected spectral");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisSysTecSondQual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisSysTecSondQual);
  status = add_string_attribute(dataset_id, "Description", "System - TEC quality index for sounder");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scalti", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scalti);
  status = add_string_attribute(dataset_id, "Description", "Satellite altitude");
  status = add_string_attribute(dataset_id, "Units", "km");
  status = add_float_attribute(dataset_id, "Scale", &altiscale, attdims);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Dataset scnlintime */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = IASI_FOR;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/scnlintime", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlintime);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset scalrad */

  rank = 4;
  dims[0] = h_scnlin;
  dims[1] = IASI_FOR;
  dims[2] = IASI_FOV;
  dims[3] = IASI_CHANS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/scalrad", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scalrad);
  status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr m^-1)");
  attdims[0] = 10;
  add_int_attribute(dataset_id, "startchan", h_startchan, attdims);
  add_int_attribute(dataset_id, "endchan", h_endchan, attdims);
  add_float_attribute(dataset_id, "scalefactor", scalefactor, attdims);
  attdims[0] = 1;

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Group /AVHRR */

  H5Pclose(plist);                    
  H5Gclose(group_id);
  group_id = H5Gcreate(file_id, "/AVHRR", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* AVHRR datasets */

  rank = 5;
  dims[0] = h_scnlin;
  dims[1] = IASI_FOR;
  dims[2] = IASI_FOV;
  dims[3] = AVHRR_CLASS;
  dims[4] = AVHRR_CHANS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrmean", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrmean);
  status = add_string_attribute(dataset_id, "Description", "Mean AVHRR radiance for 6 channels and 7 cluster classes");
  status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr) for channels 1-3A, W/(m^2 sr m^-1) for channels 3B-5");
  status = add_string_attribute(dataset_id, "Scale", "10^(-avhrrmeanscalefactor)");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrsdev", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrsdev);
  status = add_string_attribute(dataset_id, "Description", "Std dev AVHRR radiance for 6 channels and 7 cluster classes");
  status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr) for channels 1-3A, W/(m^2 sr m^-1) for channels 3B-5");
  status = add_string_attribute(dataset_id, "Scale", "10^(-avhrrsdevscalefactor)");
  H5Dclose(dataset_id);

     /* the scaling factors can be stored as 8 bit words */
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrmean_scalefactor", H5T_STD_U8LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhmeanscalefactor);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrsdev_scalefactor", H5T_STD_U8LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhsdevscalefactor);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

  rank = 4;
  dims[0] = h_scnlin;
  dims[1] = IASI_FOR;
  dims[2] = IASI_FOV;
  dims[3] = AVHRR_CLASS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrfrac", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrfrac);
  status = add_string_attribute(dataset_id, "Description", "Fraction in each class");
  status = add_float_attribute(dataset_id, "Scale", &avhrrscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrCGy", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrCGy);
  status = add_string_attribute(dataset_id, "Description", "Y angular position of centre of gravity");
  status = add_string_attribute(dataset_id, "Units", "degrees");
  status = add_float_attribute(dataset_id, "Scale", &avhrrscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrCGz", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrCGz);
  status = add_string_attribute(dataset_id, "Description", "Z angular position of centre of gravity");
  status = add_string_attribute(dataset_id, "Units", "degrees");
  status = add_float_attribute(dataset_id, "Scale", &avhrrscale, attdims);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = IASI_FOR;
  dims[2] = IASI_FOV;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/GQisFlagQual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisFlagQual);
  status = add_string_attribute(dataset_id, "Description", "combines GQisFlagQual and GQisFlagQualDetailed, see AAPP documentation");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/GEUMAvhrr1BCldFrac", H5T_STD_U8LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GEUMAvhrr1BCldFrac);
  status = add_string_attribute(dataset_id, "Description", "Cloud fraction in IASI FOV from AVHRR 1B"); 
  status = add_string_attribute(dataset_id, "Units", "%");
  status = add_int_attribute(dataset_id, "FillValue", &avhfill, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/GEUMAvhrr1BLandFrac", H5T_STD_U8LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GEUMAvhrr1BLandFrac);
  status = add_string_attribute(dataset_id, "Description", "Land and coast fraction in IASI FOV from AVHRR 1B");
  status = add_string_attribute(dataset_id, "Units", "%");
  status = add_int_attribute(dataset_id, "FillValue", &avhfill, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/GEUMAvhrr1BQual", H5T_STD_I8LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GEUMAvhrr1BQual);
  status = add_string_attribute(dataset_id, "Description", "Bits 0-6: % snow/ice, or number of bad AVHRR pixels (0-126), Bit 7: 0 if AVHRR is OK");
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = IASI_FOR;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/AVHRR/GIacAvgImagIIS", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GIacAvgImagIIS);
  status = add_string_attribute(dataset_id, "Description", "Average of IIS image");
  status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr m^-1)");
  status = add_float_attribute(dataset_id, "Scale", &avhrrscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/GIacVarImagIIS", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GIacVarImagIIS);
  status = add_string_attribute(dataset_id, "Description", "Standard deviation of IIS image");
  status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr m^-1)");
  status = add_float_attribute(dataset_id, "Scale", &avhrrscale, attdims);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Group /Geolocation */

  H5Pclose(plist);                    
  H5Gclose(group_id);
  group_id = H5Gcreate(file_id, "/Geolocation", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets latitude and longitude */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = IASI_FOR;
  dims[2] = IASI_FOV;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Geolocation/Latitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Longitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* Satellite and solar angles */

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* close the group and file */

  H5Sclose(dataspace_id);
  H5Pclose(plist);                    
  H5Gclose(group_id);
  H5Fclose(file_id);
}

#else

int iasi1c_to_hdf5_write(char *filename, int clevel){ 

  printf ("Error: unable to write HDF-5\n");
  exit(1);

}
#endif
