! Name: $Id: iasi1c_write_f.F 608 2019-01-11 14:13:00Z frna $
!
! Purpose: Write IASI l1c file using information from C calling routine.
!          Subroutines IASI1C_WRITE_HEADER and IASI1C_WRITE_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2019, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    10/01/2019  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE IASI1C_WRITE_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_wmosatid,
     & h_startchan, h_endchan, h_scalefactor)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 IASI_CHANS
      PARAMETER (IASI_CHANS=8461)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_wmosatid
      INTEGER*4     h_startchan(10)
      INTEGER*4     h_endchan(10)
      INTEGER*4     h_scalefactor(10)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "iasi1c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open IASI 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="NEW", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=iasi_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=iasi_1crecl/rldiv, IOSTAT=errStatus )
        IF (errStatus.NE.0) THEN
          OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="UNKNOWN", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=iasi_1crecl/rldiv, IOSTAT=errStatus )
          IF (errStatus.NE.0) THEN
            write(*,*)'Cannot open file ',filename(1:filename_length)
            call aappexit(1)
          ENDIF
        ENDIF
      ENDIF

      write(*,*)'Write IASI header'
      iasi1c_h_scnlin = h_scnlin
      iasi1c_h_site = h_site
      iasi1c_h_cfill1 = ' '
      iasi1c_h_1bsite = h_1bsite
      iasi1c_h_cfill2 = ' '
      iasi1c_h_versnb = iasi_1cvnnb
      iasi1c_h_versyr = iasi_1cvnyr
      iasi1c_h_versdy = iasi_1cvndy
      iasi1c_h_hdrcnt = 1
      iasi1c_h_satid = h_satid
      iasi1c_h_instrument = h_instrument
      iasi1c_h_satht = h_satht
      iasi1c_h_period = h_period
      iasi1c_h_startorbit = h_startorbit 
      iasi1c_h_startdatayr = h_startdatayr
      iasi1c_h_startdatady = h_startdatady
      iasi1c_h_startdatatime = h_startdatatime
      iasi1c_h_endorbit = h_endorbit
      iasi1c_h_enddatayr = h_enddatayr
      iasi1c_h_enddatady = h_enddatady
      iasi1c_h_enddatatime = h_enddatatime
      iasi1c_h_scnlin = h_scnlin
      iasi1c_h_misscnlin = h_misscnlin
      iasi1c_h_wmosatid = h_wmosatid
      DO i=1,10
        iasi1c_h_startchan(i) = h_startchan(i)
        iasi1c_h_endchan(i) = h_endchan(i)
        iasi1c_h_scalefactor(i) = h_scalefactor(i)
      ENDDO

      CALL IOI1C("WRITE HEADER",UNIT_1C,1)

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE IASI1C_WRITE_DATA (
     &  h_scnlin,
     &  scnlin, granule, scnlinyr, scnlindy, scnlintime,
     &  GQisFlagQual, GQisQualIndex, GQisQualIndexLoc, GQisQualIndexRad,
     &  GQisQualIndexSpect, GQisSysTecSondQual,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  scalti, scalrad, 
     &  avhcombination, avhrrmean, avhmeanscalefactor,
     &  avhrrsdev, avhsdevscalefactor, avhrrfrac, avhrrCGy, avhrrCGz,
     &  GIacVarImagIIS, GIacAvgImagIIS, GEUMAvhrr1BCldFrac,
     &  GEUMAvhrr1BLandFrac, GEUMAvhrr1BQual)
      IMPLICIT NONE

! Parameters
      INTEGER*4 IASI_CHANS
      PARAMETER (IASI_CHANS=8461)
      INTEGER*4 IASI_FOR
      PARAMETER (IASI_FOR=30)
      INTEGER*4 IASI_FOV
      PARAMETER (IASI_FOV=4)
      INTEGER*4 AVHRR_CLASS
      PARAMETER (AVHRR_CLASS=7)
      INTEGER*4 AVHRR_CHANS
      PARAMETER (AVHRR_CHANS=6)
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 granule(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(IASI_FOR,h_scnlin)
      INTEGER*4 GQisFlagQual(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 GQisQualIndex(h_scnlin)
      INTEGER*4 GQisQualIndexLoc(h_scnlin)
      INTEGER*4 GQisQualIndexRad(h_scnlin)
      INTEGER*4 GQisQualIndexSpect(h_scnlin)
      INTEGER*4 GQisSysTecSondQual(h_scnlin)
      INTEGER*4 latitude(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 longitude(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 satzen(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 satazi(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 solzen(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 solazi(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*2 scalrad(IASI_CHANS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 avhcombination(h_scnlin)
      INTEGER*4 avhrrmean(AVHRR_CHANS,AVHRR_CLASS,IASI_FOV,IASI_FOR,
     &  h_scnlin)
      INTEGER*2 avhmeanscalefactor(AVHRR_CHANS,AVHRR_CLASS,IASI_FOV,
     &  IASI_FOR, h_scnlin)
      INTEGER*4 avhrrsdev(AVHRR_CHANS,AVHRR_CLASS,IASI_FOV,IASI_FOR,
     &  h_scnlin)
      INTEGER*2 avhsdevscalefactor(AVHRR_CHANS,AVHRR_CLASS,IASI_FOV,
     &  IASI_FOR, h_scnlin)
      INTEGER*4 avhrrfrac(AVHRR_CLASS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 avhrrCGy(AVHRR_CLASS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 avhrrCGz(AVHRR_CLASS,IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 GIacVarImagIIS(IASI_FOR,h_scnlin)
      INTEGER*4 GIacAvgImagIIS(IASI_FOR,h_scnlin)
      INTEGER*4 GEUMAvhrr1BCldFrac(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 GEUMAvhrr1BLandFrac(IASI_FOV,IASI_FOR,h_scnlin)
      INTEGER*4 GEUMAvhrr1BQual(IASI_FOV,IASI_FOR,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,ifov,ifor,iclass,errStatus,i

#include "inparms.h"
#include "iasi1c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Write IASI data records'
      DO iscan=1,h_scnlin
        iasi1c_scnlin = scnlin(iscan)
        iasi1c_granule = granule(iscan)
        iasi1c_scnlinyr = scnlinyr(iscan)
        iasi1c_scnlindy = scnlindy(iscan)
        iasi1c_scalti = scalti(iscan)
        iasi1c_avhcombination = avhcombination(iscan)
        iasi1c_GQisQualIndex = GQisQualIndex(iscan)
        iasi1c_GQisQualIndexLoc = GQisQualIndexLoc(iscan)
        iasi1c_GQisQualIndexRad = GQisQualIndexRad(iscan)
        iasi1c_GQisQualIndexSpect = GQisQualIndexSpect(iscan)
        iasi1c_GQisSysTecSondQual = GQisSysTecSondQual(iscan)
        DO ifor=1,IASI_FOR
          iasi1c_scnlintime(ifor) = scnlintime(ifor,iscan)
          DO ifov=1,IASI_FOV
            iasi1c_GQisFlagQual(ifov,ifor) =
     &            GQisFlagQual(ifov,ifor,iscan)
            DO ichan=1,IASI_CHANS
              iasi1c_scalrad(ichan,ifov,ifor) = 
     &            scalrad(ichan,ifov,ifor,iscan)
            ENDDO
            iasi1c_latlon(1,ifov,ifor) = latitude(ifov,ifor,iscan)
            iasi1c_latlon(2,ifov,ifor) = longitude(ifov,ifor,iscan)
            iasi1c_angles(1,ifov,ifor) = satzen(ifov,ifor,iscan)
            iasi1c_angles(2,ifov,ifor) = satazi(ifov,ifor,iscan)
            iasi1c_angles(3,ifov,ifor) = solzen(ifov,ifor,iscan)
            iasi1c_angles(4,ifov,ifor) = solazi(ifov,ifor,iscan)
            DO iclass=1,AVHRR_CLASS
              DO ichan=1,AVHRR_CHANS
                iasi1c_avhrrmean(ichan,iclass,ifov,ifor) =
     &              avhrrmean(ichan,iclass,ifov,ifor,iscan)
                iasi1c_avhmeanscalefactor(ichan,iclass,ifov,ifor) = 
     &              avhmeanscalefactor(ichan,iclass,ifov,ifor,iscan)
                iasi1c_avhrrsdev(ichan,iclass,ifov,ifor) = 
     &              avhrrsdev(ichan,iclass,ifov,ifor,iscan)
                iasi1c_avhsdevscalefactor(ichan,iclass,ifov,ifor) = 
     &              avhsdevscalefactor(ichan,iclass,ifov,ifor,iscan)
              ENDDO
              iasi1c_avhrrfrac(iclass,ifov,ifor) =
     &           avhrrfrac(iclass,ifov,ifor,iscan)
              iasi1c_avhrrCGy(iclass,ifov,ifor) =
     &           avhrrCGy(iclass,ifov,ifor,iscan)
              iasi1c_avhrrCGz(iclass,ifov,ifor) =
     &           avhrrCGz(iclass,ifov,ifor,iscan)
            ENDDO
            iasi1c_GEUMAvhrr1BCldFrac(ifov,ifor) =
     &         GEUMAvhrr1BCldFrac(ifov,ifor,iscan)
            iasi1c_GEUMAvhrr1BLandFrac(ifov,ifor) =
     &         GEUMAvhrr1BLandFrac(ifov,ifor,iscan)
            iasi1c_GEUMAvhrr1BQual(ifov,ifor) =
     &         GEUMAvhrr1BQual(ifov,ifor,iscan)
          ENDDO
          iasi1c_GIacVarImagIIS(ifor) = GIacVarImagIIS(ifor,iscan)
          iasi1c_GIacAvgImagIIS(ifor) = GIacAvgImagIIS(ifor,iscan)
        ENDDO  !ifor
        CALL IOI1C("WRITE DATA",UNIT_1C,iscan+1)
      ENDDO  !iscan

      CALL IOI1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
