/* 
! Name: $Id: iasi1d_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $
!
! Purpose: Subroutine to write IASI 1c data to HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2008, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      30/09/2014  N C Atkinson
!  1.1      04/10/2017  Change attribute "period_minutes" to "period_seconds"
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
 
#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <string.h>
#include <math.h>

#include "add_hdf5_attributes.h"
#include "iasi1d_c.h"

int iasi1d_to_hdf5_write(char *filename, int clevel){ 

  char *hdf5_encoder_version = "$Id: iasi1d_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $";
  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[5];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         ichan;
  int         ifor;
  int         ifov;
  float       amsu_wavenumber[MAP_CHANS];
  float       amsu_temprad_offset[MAP_CHANS];
  float       amsu_temprad_slope[MAP_CHANS];
  
  int         fill=-999999;
  int         avhfill=255;
  float       latlonscale=0.0001;
  float       btscale=0.01;
  float       anglescale=0.01;
  float       scalefactor[10];
  float       avhrrscale=0.000001;
  float       logfitscale=0.0001;
  float       altiscale=0.1;
  float       surfheightscale=1.0;
  float       scorescale=0.5;
  
  for (i=0; i<10; i++) {
    if (h_scalefactor[i] > 0) scalefactor[i] = pow(10.0,(-h_scalefactor[i]));
    else scalefactor[i] = 0.0;
  }

  for (ichan=0; ichan<MAP_CHANS; ichan++){
    amsu_wavenumber[ichan] = 0.000001*h_amsPlanck[ichan][0];
    amsu_temprad_offset[ichan] = 0.000001*h_amsPlanck[ichan][1];
    amsu_temprad_slope[ichan] = 0.000001*h_amsPlanck[ichan][2];
  }

/* Create the HDF5 file */

  printf("Creating %s\n",filename);
  file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

/* Global attributes */

  attdims[0] = 1;

  add_string_attribute(file_id, "filename", filename);
  add_string_attribute(file_id, "site", h_site);
  add_string_attribute(file_id, "1bsite", h_1bsite);
  add_string_attribute(file_id, "hdf5_encoder_software_version", hdf5_encoder_version);
  add_int_attribute(file_id, "satid", &h_satid, attdims);
  add_int_attribute(file_id, "grid", &h_grid, attdims);
  add_string_attribute(file_id, "grid_key", "5=HIRS; 6=MSU; 10=AMSU-A; 11=AMSU-B; 12=MHS; 13=IASI");
  add_int_attribute(file_id, "instruments", &h_instruments, attdims);
  add_string_attribute(file_id, "instruments_key", "bit3=AMSU-A, bit4=AMSU-B/MHS, bit6=IASI");
  add_int_attribute(file_id, "satht_hectometres", &h_satht, attdims);
  add_int_attribute(file_id, "period_seconds", &h_period, attdims);
  add_int_attribute(file_id, "startorbit", &h_startorbit, attdims);
  add_int_attribute(file_id, "startdatayr", &h_startdatayr, attdims);
  add_int_attribute(file_id, "startdatady", &h_startdatady, attdims);
  add_int_attribute(file_id, "startdatatime_ms", &h_startdatatime, attdims);
  add_int_attribute(file_id, "endorbit", &h_endorbit, attdims);
  add_int_attribute(file_id, "enddatayr", &h_enddatayr, attdims);
  add_int_attribute(file_id, "enddatady", &h_enddatady, attdims);
  add_int_attribute(file_id, "enddatatime_ms", &h_enddatatime, attdims);
  add_int_attribute(file_id, "scnlin", &h_scnlin, attdims);
  add_int_attribute(file_id, "misscnlin", &h_misscnlin, attdims);
  add_int_attribute(file_id, "wmosatid", &h_wmosatid, attdims);
  add_int_attribute(file_id, "atovppvn", &h_atovppvn, attdims);
  add_int_attribute(file_id, "eigvn", &h_eigvn, attdims);
  add_int_attribute(file_id, "npcused", &h_npcused, attdims);
  add_int_attribute(file_id, "chanvn", &h_chanvn, attdims);
  add_int_attribute(file_id, "nchanused", &h_nchanused, attdims);
  if (h_nchanused > 0){
    attdims[0] = h_nchanused;
    add_int_attribute(file_id, "IASI_channels", h_channels, attdims);
    attdims[0] = 1;
  }
  add_int_attribute(file_id, "dT", &h_dT, attdims);
  if (h_instruments & 24) {   /* if AMSU-A/MHS present (bit 3,4) */
    attdims[0] = MAP_CHANS;
    add_float_attribute(file_id, "AMSU_wavenumber", amsu_wavenumber, attdims);
    add_float_attribute(file_id, "AMSU_temprad_offset", amsu_temprad_offset, attdims);
    add_float_attribute(file_id, "AMSU_temprad_slope", amsu_temprad_slope, attdims);
    attdims[0] = 1;
    add_int_attribute(file_id, "mapBTmode", &h_mapBTmode, attdims);
    add_string_attribute(file_id, "mapBTmode_key", "1=nearest neighbour, 2=bilinear interpolation, 3=spatial averaging");
    add_int_attribute(file_id, "Acorrvn", &h_Acorrvn, attdims);
    add_int_attribute(file_id, "Apcpvn", &h_Apcpvn, attdims);
    add_int_attribute(file_id, "Aantenna", &h_Aantenna, attdims);
    add_int_attribute(file_id, "Bcorrvn", &h_Bcorrvn, attdims);
    add_int_attribute(file_id, "Bpcpvn", &h_Bpcpvn, attdims);
    add_int_attribute(file_id, "Bantenna", &h_Bantenna, attdims);
    add_int_attribute(file_id, "Ascat_threshold_K", &h_Ascat, attdims);
    add_int_attribute(file_id, "ACrosby_threshold_K", &h_ACrosby, attdims);
    add_int_attribute(file_id, "AB89_threshold_K", &h_AB89, attdims);
  }
  add_int_attribute(file_id, "cloudtests", &h_cloudtests, attdims);
  add_int_attribute(file_id, "recperscan", &h_recperscan, attdims);
  add_int_attribute(file_id, "fovmode", &h_fovmode, attdims);

/* Group /Data */

  group_id = H5Gcreate(file_id, "/Data", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets with one value per record */

  rank = 1;
  dims[0] = h_scnlin*h_recperscan;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/scnlin", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlin);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinyr", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinyr);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlindy", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlindy);
  H5Dclose(dataset_id);
  if (h_instruments & 8) {   /* if AMSU-A present (bit 3) */
    dataset_id = H5Dcreate(file_id, "/Data/instrtempa1", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, instrtempa1);
    status = add_string_attribute(dataset_id, "Description", "AMSU-A1 instrument temperature");
    status = add_string_attribute(dataset_id, "Units", "K");
    status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
    H5Dclose(dataset_id);
    dataset_id = H5Dcreate(file_id, "/Data/instrtempa2", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, instrtempa2);
    status = add_string_attribute(dataset_id, "Description", "AMSU-A2 instrument temperature");
    status = add_string_attribute(dataset_id, "Units", "K");
    status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
    H5Dclose(dataset_id);
  }
  if (h_instruments & 16) {   /* if MHS present (bit 4) */
    dataset_id = H5Dcreate(file_id, "/Data/instrtempb", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, instrtempb);
    status = add_string_attribute(dataset_id, "Description", "MHS instrument temperature");
    status = add_string_attribute(dataset_id, "Units", "K");
    status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
    H5Dclose(dataset_id);
  }
  dataset_id = H5Dcreate(file_id, "/Data/scalti", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scalti);
  status = add_string_attribute(dataset_id, "Description", "Satellite altitude");
  status = add_string_attribute(dataset_id, "Units", "km");
  status = add_float_attribute(dataset_id, "Scale", &altiscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/avhcombination", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhcombination);
  status = add_string_attribute(dataset_id, "Description", "AVHRR channel combination");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisQualIndex", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisQualIndex);
  status = add_string_attribute(dataset_id, "Description", "NEDT estimated/expected");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisQualIndexLoc", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisQualIndexLoc);
  status = add_string_attribute(dataset_id, "Description", "IIS/AVHRR co-registration uncertainty");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisQualIndexRad", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisQualIndexRad);
  status = add_string_attribute(dataset_id, "Description", "NEDT estimated/expected radiometric");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisQualIndexSpect", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisQualIndexSpect);
  status = add_string_attribute(dataset_id, "Description", "NEDT estimated/expected spectral");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/GQisSysTecSondQual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisSysTecSondQual);
  status = add_string_attribute(dataset_id, "Description", "System - TEC quality index for sounder");
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Dataset scnlintime */

  rank = 2;
  dims[0] = h_scnlin*h_recperscan;
  dims[1] = IASI_FOR;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/scnlintime", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlintime);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/iasiqual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, iasiqual);
  status = add_string_attribute(dataset_id, "Description", "IASI FOV quality: see AAPP Formats document");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/FOV", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, FOV);
  status = add_string_attribute(dataset_id, "Description", "FOV number (1 to 4) for each field of regard");
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Dataset btemps */

  if (h_instruments & 24) {    /* If AMSU/MHS present (bits 3 or 4) */
    rank = 3;
    dims[0] = h_scnlin*h_recperscan;
    dims[1] = IASI_FOR;
    dims[2] = MAP_CHANS;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/btemps", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, btemps);
    status = add_string_attribute(dataset_id, "Units", "K");
    status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
    status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);

    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);
  }

/* Dataset prepro */

  if (h_instruments & 24) {    /* If AMSU/MHS present (bits 3 or 4) */
    rank = 3;
    dims[0] = h_scnlin*h_recperscan;
    dims[1] = IASI_FOR;
    dims[2] = 7;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/prepro", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, prepro);
    status = add_string_attribute(dataset_id, "Description", "Pre-processing output, see AAPP documentation");
    status = add_string_attribute(dataset_id, "Order_prepro",
     "1: QC word, 2: Surface type, 3: Cost fn from PPASURF, 4: Scattering index, 5: Precip. prob., 6-7: Spare");
    status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);

    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);
  }

/* Dataset logfit */

  if (h_npcused > 0){      /* omit if there are no PCs */
    rank = 3;
    dims[0] = h_scnlin*h_recperscan;
    dims[1] = IASI_FOR;
    dims[2] = 3;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/logfit", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, logfit);
    status = add_string_attribute(dataset_id, "Description", "Log10 noise-normalised reconstruction score for 3 bands");
    status = add_float_attribute(dataset_id, "Scale", &logfitscale, attdims);

    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);
  }

/* Dataset cloudflag */

  rank = 3;
  dims[0] = h_scnlin*h_recperscan;
  dims[1] = IASI_FOR;
  dims[2] = 10;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/cloudflag", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, cloudflag);
  status = add_string_attribute(dataset_id, "Description", "Result of up to 10 cloud tests");

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset emiss*/

  rank = 3;
  dims[0] = h_scnlin*h_recperscan;
  dims[1] = IASI_FOR;
  dims[2] = 13;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }

  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/emiss", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, emiss);
  status = add_string_attribute(dataset_id, "Description", "13 emissivity parameters");

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset scalrad and scores */

  if (h_nchanused > 0){      /* omit if there are no channels */
    rank = 3;
    dims[0] = h_scnlin*h_recperscan;
    dims[1] = IASI_FOR;
    dims[2] = h_nchanused;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/scalrad", H5T_STD_I16LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scalrad);
    status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr m^-1)");
    attdims[0] = 10;
    add_int_attribute(dataset_id, "startchan", h_startchan, attdims);
    add_int_attribute(dataset_id, "endchan", h_endchan, attdims);
    add_float_attribute(dataset_id, "scalefactor", scalefactor, attdims);
    attdims[0] = 1;

    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);
  }
  if (h_npcused > 0){      /* omit if there are no PCs */
    rank = 3;
    dims[0] = h_scnlin*h_recperscan;
    dims[1] = IASI_FOR;
    dims[2] = h_npcused;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }

    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/scores", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scores);
    status = add_string_attribute(dataset_id, "Description", "PC scores for noise-normalised radiances. See attribute eigvn for eigenvector version.");
    add_float_attribute(dataset_id, "Scale", &scorescale, attdims);

    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);
  }

/* Group /AVHRR */

  H5Pclose(plist);                    
  H5Gclose(group_id);
  group_id = H5Gcreate(file_id, "/AVHRR", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* AVHRR datasets */

  rank = 4;
  dims[0] = h_scnlin*h_recperscan;
  dims[1] = IASI_FOR;
  dims[2] = AVHRR_CLASS;
  dims[3] = AVHRR_CHANS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrmean", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrmean);
  status = add_string_attribute(dataset_id, "Description", "Mean AVHRR radiance for 6 channels and 7 cluster classes");
  status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr) for channels 1-3A, W/(m^2 sr m^-1) for channels 3B-5");
  status = add_string_attribute(dataset_id, "Scale", "10^(-avhrrmeanscalefactor)");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrsdev", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrsdev);
  status = add_string_attribute(dataset_id, "Description", "Std dev AVHRR radiance for 6 channels and 7 cluster classes");
  status = add_string_attribute(dataset_id, "Units", "W/(m^2 sr) for channels 1-3A, W/(m^2 sr m^-1) for channels 3B-5");
  status = add_string_attribute(dataset_id, "Scale", "10^(-avhrrsdevscalefactor)");
  H5Dclose(dataset_id);

     /* the scaling factors can be stored as 8 bit words */
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrmean_scalefactor", H5T_STD_U8LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhmeanscalefactor);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrsdev_scalefactor", H5T_STD_U8LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_SHORT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhsdevscalefactor);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

  rank = 3;
  dims[0] = h_scnlin*h_recperscan;
  dims[1] = IASI_FOR;
  dims[2] = AVHRR_CLASS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrfrac", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrfrac);
  status = add_string_attribute(dataset_id, "Description", "Fraction in each class");
  status = add_float_attribute(dataset_id, "Scale", &avhrrscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrCGy", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrCGy);
  status = add_string_attribute(dataset_id, "Description", "Y angular position of centre of gravity");
  status = add_string_attribute(dataset_id, "Units", "degrees");
  status = add_float_attribute(dataset_id, "Scale", &avhrrscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/AVHRR/avhrrCGz", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, avhrrCGz);
  status = add_string_attribute(dataset_id, "Description", "Z angular position of centre of gravity");
  status = add_string_attribute(dataset_id, "Units", "degrees");
  status = add_float_attribute(dataset_id, "Scale", &avhrrscale, attdims);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

  rank = 2;
  dims[0] = h_scnlin*h_recperscan;
  dims[1] = IASI_FOR;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/GQisFlagQual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, GQisFlagQual);
  status = add_string_attribute(dataset_id, "Description", "combines GQisFlagQual and GQisFlagQualDetailed, see AAPP documentation");
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Group /Geolocation */

  H5Pclose(plist);                    
  H5Gclose(group_id);
  group_id = H5Gcreate(file_id, "/Geolocation", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets latitude, longitude, surfheight, surftype */

  rank = 2;
  dims[0] = h_scnlin*h_recperscan;
  dims[1] = IASI_FOR;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Geolocation/Latitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Longitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Surface_height", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, surfheight);
  status = add_string_attribute(dataset_id, "Description", "Surface height from atlas");
  status = add_string_attribute(dataset_id, "Units", "m");
  status = add_float_attribute(dataset_id, "Scale", &surfheightscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Surface_type", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, surftype);
  status = add_string_attribute(dataset_id, "Description", "Surface type from atlas: 0=sea, 1=mixed, 2=land");
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* Satellite and solar angles */

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* close the group and file */

  H5Sclose(dataspace_id);
  H5Pclose(plist);                    
  H5Gclose(group_id);
  H5Fclose(file_id);
}

#else

int iasi1d_to_hdf5_write(char *filename, int clevel){ 

  printf ("Error: unable to write HDF-5\n");
  exit(1);

}
#endif
