/* 
! Name: $Id: iasipc_from_hdf5_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Subroutine to read IASI pc data from HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2019, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      09/01/2019  N C Atkinson
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>

#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <string.h>
#include <math.h>

#include "aapp_hdf5.h"

#include "common_read.h"
#include "iasipc_c.h"

int iasipc_from_hdf5_read(char *filename){ 

  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[3];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         ichan;
  float       quantization_real[3];
  float       scalefactor[10];
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannot open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */

/****** Read top-level attributes ******/

  status = H5Tset_size (strtype, 3);
  readattribute("site",file_id,strtype,h_site);
  readattribute("1bsite",file_id,strtype,h_1bsite);
  readattribute("satid",file_id,H5T_NATIVE_INT,&h_satid);
  readattribute("instrument",file_id,H5T_NATIVE_INT,&h_instrument);
  readattribute("satht_hectometres",file_id,H5T_NATIVE_INT,&h_satht);
  readattribute("period_seconds",file_id,H5T_NATIVE_INT,&h_period);
  readattribute("startorbit",file_id,H5T_NATIVE_INT,&h_startorbit);
  readattribute("startdatayr",file_id,H5T_NATIVE_INT,&h_startdatayr);
  readattribute("startdatady",file_id,H5T_NATIVE_INT,&h_startdatady);
  readattribute("startdatatime_ms",file_id,H5T_NATIVE_INT,&h_startdatatime);
  readattribute("endorbit",file_id,H5T_NATIVE_INT,&h_endorbit);
  readattribute("enddatayr",file_id,H5T_NATIVE_INT,&h_enddatayr);
  readattribute("enddatady",file_id,H5T_NATIVE_INT,&h_enddatady);
  readattribute("enddatatime_ms",file_id,H5T_NATIVE_INT,&h_enddatatime);
  readattribute("scnlin",file_id,H5T_NATIVE_INT,&h_scnlin);
  readattribute("misscnlin",file_id,H5T_NATIVE_INT,&h_misscnlin);
  readattribute("wmosatid",file_id,H5T_NATIVE_INT,&h_wmosatid);
  readattribute("eigvn",file_id,H5T_NATIVE_INT,&h_eigvn);
  readattribute("npcused",file_id,H5T_NATIVE_INT,&h_npcused);
  readattribute("chanvn",file_id,H5T_NATIVE_INT,&h_chanvn);
  readattribute("nchanused",file_id,H5T_NATIVE_INT,&h_nchanused);
  readattribute("rchanvn",file_id,H5T_NATIVE_INT,&h_rchanvn);
  readattribute("nrchanused",file_id,H5T_NATIVE_INT,&h_nrchanused);
  readattribute("evfilevn",file_id,H5T_NATIVE_INT,h_evfilevn);
  if (h_evfilevn > 0){
    status = H5Tset_size (strtype, 81);
    readattribute("evfile1",file_id,strtype,h_evfile1);
    readattribute("evfile2",file_id,strtype,h_evfile2);
    readattribute("evfile3",file_id,strtype,h_evfile3);
  }
  if (h_nchanused > 0){
    readattribute("IASI_channels",file_id,H5T_NATIVE_INT,h_channels);
  }
  if (h_nrchanused > 0){
    readattribute("IASI_recon_channels",file_id,H5T_NATIVE_INT,h_rchannels);
  }

/****** Allocate memory ******/

  scnlin = (int*)calloc(h_scnlin, sizeof(int));
  granule = (int*)calloc(h_scnlin, sizeof(int));
  scnlinyr = (int*)calloc(h_scnlin, sizeof(int));
  scnlindy = (int*)calloc(h_scnlin, sizeof(int));
  scnlintime = (int*)calloc(h_scnlin*IASI_FOR, sizeof(int));
  GQisFlagQual = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  GQisQualIndex = (int*)calloc(h_scnlin, sizeof(int));
  GQisQualIndexLoc = (int*)calloc(h_scnlin, sizeof(int));
  GQisQualIndexRad = (int*)calloc(h_scnlin, sizeof(int));
  GQisQualIndexSpect = (int*)calloc(h_scnlin, sizeof(int));
  GQisSysTecSondQual = (int*)calloc(h_scnlin, sizeof(int));
  latitude = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  longitude = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  satzen = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  solzen = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  satazi = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  solazi = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  scalti = (int*)calloc(h_scnlin, sizeof(int));
  scalrad = (short*)calloc(h_scnlin*IASI_SPOTS*IASI_PCCHANS, sizeof(short));
  reconrad = (short*)calloc(h_scnlin*IASI_SPOTS*IASI_PCCHANS, sizeof(short));
  scores = (int*)calloc(h_scnlin*IASI_SPOTS*IASI_PCS, sizeof(int));
  logfit = (int*)calloc(h_scnlin*IASI_SPOTS*3, sizeof(int));
  avhcombination = (int*)calloc(h_scnlin, sizeof(int));
  avhrrmean          = (int*)calloc(h_scnlin*IASI_SPOTS*AVHRR_CHANS*AVHRR_CLASS, sizeof(int));
  avhmeanscalefactor = (short*)calloc(h_scnlin*IASI_SPOTS*AVHRR_CHANS*AVHRR_CLASS, sizeof(short));
  avhrrsdev          = (int*)calloc(h_scnlin*IASI_SPOTS*AVHRR_CHANS*AVHRR_CLASS, sizeof(int));
  avhsdevscalefactor = (short*)calloc(h_scnlin*IASI_SPOTS*AVHRR_CHANS*AVHRR_CLASS, sizeof(short));
  avhrrfrac = (int*)calloc(h_scnlin*IASI_SPOTS*AVHRR_CLASS, sizeof(int));
  avhrrCGy = (int*)calloc(h_scnlin*IASI_SPOTS*AVHRR_CLASS, sizeof(int));
  avhrrCGz = (int*)calloc(h_scnlin*IASI_SPOTS*AVHRR_CLASS, sizeof(int));
  GIacVarImagIIS = (int*)calloc(h_scnlin*IASI_FOR, sizeof(int));
  GIacAvgImagIIS = (int*)calloc(h_scnlin*IASI_FOR, sizeof(int));
  GEUMAvhrr1BCldFrac = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  GEUMAvhrr1BLandFrac = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));
  GEUMAvhrr1BQual = (int*)calloc(h_scnlin*IASI_SPOTS, sizeof(int));

  printf("finished allocating\n");

/****** Open and read the group "/Data" ******/

  group_id = H5Gopen(file_id, "/Data", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }
  readdataset("scnlin",group_id,H5T_NATIVE_INT,scnlin);
  readdataset("granule",group_id,H5T_NATIVE_INT,granule);
  readdataset("scnlinyr",group_id,H5T_NATIVE_INT,scnlinyr);
  readdataset("scnlindy",group_id,H5T_NATIVE_INT,scnlindy);
  readdataset("scnlintime",group_id,H5T_NATIVE_INT,scnlintime);
  readdataset("avhcombination",group_id,H5T_NATIVE_INT,avhcombination);
  readdataset("GQisQualIndex",group_id,H5T_NATIVE_INT,GQisQualIndex);
  readdataset("GQisQualIndexLoc",group_id,H5T_NATIVE_INT,GQisQualIndexLoc);
  readdataset("GQisQualIndexRad",group_id,H5T_NATIVE_INT,GQisQualIndexRad);
  readdataset("GQisQualIndexSpect",group_id,H5T_NATIVE_INT,GQisQualIndexSpect);
  readdataset("GQisSysTecSondQual",group_id,H5T_NATIVE_INT,GQisSysTecSondQual);
  readdataset("GQisFlagQual",group_id,H5T_NATIVE_INT,GQisFlagQual);
  readdataset("scalti",group_id,H5T_NATIVE_INT,scalti);
  if (h_nchanused > 0) {
    readdataset("scalrad",group_id,H5T_NATIVE_SHORT,scalrad);
    dataset_id = H5Dopen(group_id, "scalrad", H5P_DEFAULT);
    readattribute("startchan",dataset_id,H5T_NATIVE_INT,h_startchan);
    readattribute("endchan",dataset_id,H5T_NATIVE_INT,h_endchan);
    readattribute("scalefactor",dataset_id,H5T_NATIVE_FLOAT,scalefactor);
    for (i=0; i<10; i++) {
      if (scalefactor[i] > 0.0) h_scalefactor[i] = -1.0*log10((double)scalefactor[i])+0.5;
      else h_scalefactor[i] = -999999;
    }
    H5Dclose(dataset_id);
  }
  if (h_nrchanused > 0) {
    readdataset("reconrad",group_id,H5T_NATIVE_SHORT,reconrad);
    if (h_nchanused == 0) {
      dataset_id = H5Dopen(group_id, "reconrad", H5P_DEFAULT);
      readattribute("startchan",dataset_id,H5T_NATIVE_INT,h_startchan);
      readattribute("endchan",dataset_id,H5T_NATIVE_INT,h_endchan);
      readattribute("scalefactor",dataset_id,H5T_NATIVE_FLOAT,scalefactor);
      for (i=0; i<10; i++) {
        if (scalefactor[i] > 0.0) h_scalefactor[i] = -1.0*log10((double)scalefactor[i])+0.5;
        else h_scalefactor[i] = -999999;
      }
      H5Dclose(dataset_id);
    }
  }
  if (h_npcused > 0) {
    readdataset("logfit",group_id,H5T_NATIVE_INT,logfit);
    readdataset("scores",group_id,H5T_NATIVE_INT,scores);
    dataset_id = H5Dopen(group_id, "scores", H5P_DEFAULT);
    readattribute("Quantization",dataset_id,H5T_NATIVE_FLOAT,quantization_real);
    readattribute("startpcchan",dataset_id,H5T_NATIVE_INT,h_startpcchan);
    readattribute("endpcchan",dataset_id,H5T_NATIVE_INT,h_endpcchan);
    readattribute("startpc",dataset_id,H5T_NATIVE_INT,h_startpc);
    readattribute("endpc",dataset_id,H5T_NATIVE_INT,h_endpc);
    H5Dclose(dataset_id);
    h_quantization[0] = 100*quantization_real[0];
    h_quantization[1] = 100*quantization_real[1];
    h_quantization[2] = 100*quantization_real[2];
  }
  H5Gclose(group_id);

/****** Open and read the group "/AVHRR" ******/

  group_id = H5Gopen(file_id, "/AVHRR", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }
  readdataset("avhrrmean",group_id,H5T_NATIVE_INT,avhrrmean);
  readdataset("avhrrsdev",group_id,H5T_NATIVE_INT,avhrrsdev);
  readdataset("avhrrmean_scalefactor",group_id,H5T_NATIVE_SHORT,avhmeanscalefactor);
  readdataset("avhrrsdev_scalefactor",group_id,H5T_NATIVE_SHORT,avhsdevscalefactor);
  readdataset("avhrrfrac",group_id,H5T_NATIVE_INT,avhrrfrac);
  readdataset("avhrrCGy",group_id,H5T_NATIVE_INT,avhrrCGy);
  readdataset("avhrrCGz",group_id,H5T_NATIVE_INT,avhrrCGz);
  readdataset("GEUMAvhrr1BCldFrac",group_id,H5T_NATIVE_INT,GEUMAvhrr1BCldFrac);
  readdataset("GEUMAvhrr1BLandFrac",group_id,H5T_NATIVE_INT,GEUMAvhrr1BLandFrac);
  readdataset("GEUMAvhrr1BQual",group_id,H5T_NATIVE_INT,GEUMAvhrr1BQual);
  readdataset("GIacAvgImagIIS",group_id,H5T_NATIVE_INT,GIacAvgImagIIS);
  readdataset("GIacVarImagIIS",group_id,H5T_NATIVE_INT,GIacVarImagIIS);
  H5Gclose(group_id);

/****** Open and read the group "/Geolocation" ******/

  group_id = H5Gopen(file_id, "/Geolocation", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }
  readdataset("Latitude",group_id,H5T_NATIVE_INT,latitude);
  readdataset("Longitude",group_id,H5T_NATIVE_INT,longitude);
  readdataset("Satellite_zenith_angle",group_id,H5T_NATIVE_INT,satzen);
  readdataset("Solar_zenith_angle",group_id,H5T_NATIVE_INT,solzen);
  readdataset("Satellite_azimuth_angle",group_id,H5T_NATIVE_INT,satazi);
  readdataset("Solar_azimuth_angle",group_id,H5T_NATIVE_INT,solazi);
  H5Gclose(group_id);

/****** Clean up ******/

  H5Fclose(file_id);
  printf("Finished reading %s\n",filename);
  return 0;

}

#else

int iasipc_from_hdf5_read(char *filename){

  printf ("Error: unable to read HDF-5\n");
  exit(1);

}
#endif
