! Name: $Id: iras1c_read_f.F 464 2015-09-29 12:51:29Z frna $
!
! Purpose: Read IRAS l1c file and pass information back to C calling routine.
!          Subroutines IRAS1C_READ_HEADER and IRAS1C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    29/09/2015  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE IRAS1C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_tempradcnv, h_wmosatid,
     & h_nchan, h_nmapchan, h_nmapchanused_mwts, h_nmapchanused_mwhs)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 IRAS_CHANS
      PARAMETER (IRAS_CHANS=26)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_tempradcnv(3,IRAS_CHANS)
      INTEGER*4     h_wmosatid
      INTEGER*4     h_nchan
      INTEGER*4     h_nmapchan
      INTEGER*4     h_nmapchanused_mwts
      INTEGER*4     h_nmapchanused_mwhs

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "iras1c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open IRAS 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=iras_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read IRAS header'
      CALL IOIRAS1C("READ HEADER",UNIT_1C,1)

      h_scnlin = iras1c_h_scnlin
      h_site = iras1c_h_site
      h_1bsite = iras1c_h_1bsite
      h_versnb = iras1c_h_versnb
      h_satid = iras1c_h_satid
      h_instrument = iras1c_h_instrument
      h_satht = iras1c_h_satht
      h_period = iras1c_h_period
      h_startorbit = iras1c_h_startorbit
      h_startdatayr = iras1c_h_startdatayr
      h_startdatady = iras1c_h_startdatady
      h_startdatatime = iras1c_h_startdatatime
      h_endorbit = iras1c_h_endorbit
      h_enddatayr = iras1c_h_enddatayr
      h_enddatady = iras1c_h_enddatady
      h_enddatatime = iras1c_h_enddatatime
      h_scnlin = iras1c_h_scnlin
      h_misscnlin = iras1c_h_misscnlin
      do ichan=1,IRAS_CHANS
        do i=1,3
          h_tempradcnv(i,ichan) = iras1c_h_tempradcnv(i,ichan)
        enddo
      enddo
      h_wmosatid = iras1c_h_wmosatid
      h_nchan = iras1c_h_nchan
      h_nmapchan = iras1c_h_nmapchan
      h_nmapchanused_mwts = iras1c_h_nmapchanused_mwts
      h_nmapchanused_mwhs = iras1c_h_nmapchanused_mwhs

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE IRAS1C_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  geo_qc, scnlin_qc, cal_qc, pixel_qc, instrtemp,
     &  latitude, longitude, satzen, solzen, satazi, solazi, scalti,
     &  satposition, satvelocity, orbitangle,
     &  btemps, btmap, dataqual)
      IMPLICIT NONE

! Parameters
      INTEGER*4 IRAS_CHANS
      PARAMETER (IRAS_CHANS=26)
      INTEGER*4 IRAS_MAPCHANS
      PARAMETER (IRAS_MAPCHANS=28)
      INTEGER*4 IRAS_SPOTS
      PARAMETER (IRAS_SPOTS=56)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 geo_qc(h_scnlin)
      INTEGER*4 scnlin_qc(h_scnlin)
      INTEGER*4 cal_qc(h_scnlin)
      INTEGER*4 pixel_qc(IRAS_SPOTS,h_scnlin)
      INTEGER*4 instrtemp(9,h_scnlin)
      INTEGER*4 latitude(IRAS_SPOTS,h_scnlin)
      INTEGER*4 longitude(IRAS_SPOTS,h_scnlin)
      INTEGER*4 satzen(IRAS_SPOTS,h_scnlin)
      INTEGER*4 satazi(IRAS_SPOTS,h_scnlin)
      INTEGER*4 solzen(IRAS_SPOTS,h_scnlin)
      INTEGER*4 solazi(IRAS_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 satposition(3,h_scnlin)
      INTEGER*4 satvelocity(3,h_scnlin)
      INTEGER*4 orbitangle(h_scnlin)
      INTEGER*4 btemps(IRAS_CHANS,IRAS_SPOTS,h_scnlin)
      INTEGER*4 btmap(IRAS_MAPCHANS,IRAS_SPOTS,h_scnlin)
      INTEGER*4 dataqual(IRAS_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,i

#include "inparms.h"
#include "iras1c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read IRAS data records'
      DO iscan=1,h_scnlin
        CALL IOIRAS1C("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = iras1c_scnlin
        scnlinyr(iscan) = iras1c_scnlinyr
        scnlindy(iscan) = iras1c_scnlindy
        scnlintime(iscan) = iras1c_scnlintime
        geo_qc(iscan) = iras1c_geo_qc
        scnlin_qc(iscan) = iras1c_scnlin_qc
        cal_qc(iscan) = iras1c_cal_qc
        DO i=1,9
          instrtemp(i,iscan) = iras1c_instrtemp(i)
        ENDDO
        scalti(iscan) = iras1c_scalti
        DO ispot=1,IRAS_SPOTS
          pixel_qc(ispot,iscan) = iras1c_pixel_qc(ispot)
          DO ichan=1,IRAS_CHANS
            btemps(ichan,ispot,iscan) = iras1c_btemps(ichan,ispot)
          ENDDO
          DO ichan=1,IRAS_MAPCHANS
            btmap(ichan,ispot,iscan) = iras1c_btmap(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = iras1c_latlon(1,ispot)
          longitude(ispot,iscan) = iras1c_latlon(2,ispot)
          satzen(ispot,iscan) = iras1c_angles(1,ispot)
          satazi(ispot,iscan) = iras1c_angles(2,ispot)
          solzen(ispot,iscan) = iras1c_angles(3,ispot)
          solazi(ispot,iscan) = iras1c_angles(4,ispot)
          dataqual(ispot,iscan) = iras1c_dataqual(ispot)
        ENDDO
        DO i=1,3
          satposition(i,iscan) = iras1c_satposition(i)
          satvelocity(i,iscan) = iras1c_satvelocity(i)
        ENDDO
        orbitangle(iscan) = iras1c_orbitangle
      ENDDO

      CALL IOIRAS1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
