/* 
! Name: $Id: iras1c_to_hdf5_write.c 638 2019-04-24 11:31:42Z frna $
!
! Purpose: Subroutine to write IRAS 1c data to HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2015, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      29/09/2015  N C Atkinson
!  1.1      04/10/2017  Change attribute "period_minutes" to "period_seconds"
!  1.2      08/01/2019  Add scalti and dataqual
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
 
#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <string.h>

#include "add_hdf5_attributes.h"
#include "iras1c_c.h"

int iras1c_to_hdf5_write(char *filename, int clevel){ 

  char *hdf5_encoder_version = "$Id";
  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[3];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         ichan;
  float       wavenumber[IRAS_CHANS];
  float       temprad_offset[IRAS_CHANS];
  float       temprad_slope[IRAS_CHANS];
  
  int         fill=-999999;
  float       btscale=0.01;
  float       latlonscale=0.0001;
  float       anglescale=0.01;
  float       altiscale=0.1;

  for (ichan=0; ichan<IRAS_CHANS; ichan++){
    wavenumber[ichan] = 0.0001*h_tempradcnv[ichan][0];
    temprad_offset[ichan] = 0.000001*h_tempradcnv[ichan][1];
    temprad_slope[ichan] = 0.000001*h_tempradcnv[ichan][2];
  }

/* Create the HDF5 file */

  printf("Creating %s\n",filename);
  file_id = H5Fcreate(filename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);

/* Global attributes */

  attdims[0] = 1;

  add_string_attribute(file_id, "filename", filename);
  add_string_attribute(file_id, "site", h_site);
  add_string_attribute(file_id, "1bsite", h_1bsite);
  add_string_attribute(file_id, "hdf5_encoder_software_version", hdf5_encoder_version);
  add_int_attribute(file_id, "satid", &h_satid, attdims);
  add_int_attribute(file_id, "instrument", &h_instrument, attdims);
  add_int_attribute(file_id, "satht_hectometres", &h_satht, attdims);
  add_int_attribute(file_id, "period_seconds", &h_period, attdims);
  add_int_attribute(file_id, "startorbit", &h_startorbit, attdims);
  add_int_attribute(file_id, "startdatayr", &h_startdatayr, attdims);
  add_int_attribute(file_id, "startdatady", &h_startdatady, attdims);
  add_int_attribute(file_id, "startdatatime_ms", &h_startdatatime, attdims);
  add_int_attribute(file_id, "endorbit", &h_endorbit, attdims);
  add_int_attribute(file_id, "enddatayr", &h_enddatayr, attdims);
  add_int_attribute(file_id, "enddatady", &h_enddatady, attdims);
  add_int_attribute(file_id, "enddatatime_ms", &h_enddatatime, attdims);
  add_int_attribute(file_id, "scnlin", &h_scnlin, attdims);
  add_int_attribute(file_id, "misscnlin", &h_misscnlin, attdims);
  add_int_attribute(file_id, "wmosatid", &h_wmosatid, attdims);
  add_int_attribute(file_id, "nmapchanused_mwts", &h_nmapchanused_mwts, attdims);
  add_int_attribute(file_id, "nmapchanused_mwhs", &h_nmapchanused_mwhs, attdims);

  attdims[0] = IRAS_CHANS;
  add_float_attribute(file_id, "IRAS_wavenumber", wavenumber, attdims);
  add_float_attribute(file_id, "IRAS_temprad_offset", temprad_offset, attdims);
  add_float_attribute(file_id, "IRAS_temprad_slope", temprad_slope, attdims);
  attdims[0] = 1;

/* Group /Data */

  group_id = H5Gcreate(file_id, "/Data", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets with one value per scan */

  rank = 1;
  dims[0] = h_scnlin;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Data/scnlin", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlin);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlinyr", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlinyr);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlindy", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlindy);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlintime", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlintime);
  status = add_string_attribute(dataset_id, "Units", "ms");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/geo_qc", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geo_qc);
  status = add_string_attribute(dataset_id, "Description", "Geolocation quality");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scnlin_qc", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scnlin_qc);
  status = add_string_attribute(dataset_id, "Description", "Scan line quality");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/cal_qc", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, cal_qc);
  status = add_string_attribute(dataset_id, "Description", "Calibration quality");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/orbitangle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, orbitangle);
  status = add_string_attribute(dataset_id, "Description", "Angle round orbit relative to solar plane");
  status = add_string_attribute(dataset_id, "Units", "deg");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/scalti", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, scalti);
  status = add_string_attribute(dataset_id, "Description", "Satellite altitude");
  status = add_string_attribute(dataset_id, "Units", "km");
  status = add_float_attribute(dataset_id, "Scale", &altiscale, attdims);
  H5Dclose(dataset_id);

  H5Sclose(dataspace_id);

/* Satellite position and velocity */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = 3;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/satposition", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satposition);
  status = add_string_attribute(dataset_id, "Description", "Satellite position x,y,z");
  status = add_string_attribute(dataset_id, "Units", "m");
  H5Dclose(dataset_id);
  dataset_id = H5Dcreate(file_id, "/Data/satvelocity", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satvelocity);
  status = add_string_attribute(dataset_id, "Description", "Satellite velocity vx,vy,vz");
  status = add_string_attribute(dataset_id, "Units", "m/s");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Pixel QC */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = IRAS_SPOTS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/pixel_qc", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, pixel_qc);
  status = add_string_attribute(dataset_id, "Description", "Pixel quality");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Instrument temps */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = 9;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/instrtemp", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, instrtemp);
  status = add_string_attribute(dataset_id, "Description", "Instrument temperatures");
  status = add_string_attribute(dataset_id, "Units", "K");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset btemps */

  rank = 3;
  dims[0] = h_scnlin;
  dims[1] = IRAS_SPOTS;
  dims[2] = IRAS_CHANS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/btemps", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, btemps);
  status = add_string_attribute(dataset_id, "Units", "K (1-20), uW/(cm^2 sr nm) (21-26)");
  status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  status = add_string_attribute(dataset_id, "Description", "Brightness temperature (1-20), spectral irradiance (21-26)");

  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

/* Dataset btmap */

  if (h_nmapchanused_mwts > 0 || h_nmapchanused_mwhs > 0) {
    rank = 3;
    dims[0] = h_scnlin;
    dims[1] = IRAS_SPOTS;
    dims[2] = IRAS_MAPCHANS;
    if (clevel > 0){
      status = H5Pset_chunk(plist, rank, dims);
      status = H5Pset_deflate( plist, clevel);         /* gzip compression */
    }
    dataspace_id = H5Screate_simple(rank, dims, NULL); 
    dataset_id = H5Dcreate(file_id, "/Data/btmap", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, btmap);
    status = add_string_attribute(dataset_id, "Units", "K");
    status = add_float_attribute(dataset_id, "Scale", &btscale, attdims);
    status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
    status = add_string_attribute(dataset_id, "Description", "Mapped brightness temperature");

    H5Dclose(dataset_id);
    H5Sclose(dataspace_id);
  }

/* dataqual */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = IRAS_SPOTS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 
  dataset_id = H5Dcreate(file_id, "/Data/dataqual", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, dataqual);
  status = add_string_attribute(dataset_id, "Description", "Data quality: see AAPP Formats document");
  H5Dclose(dataset_id);
  H5Sclose(dataspace_id);

  H5Pclose(plist);                    
  H5Gclose(group_id);

/* Group /Geolocation */

  group_id = H5Gcreate(file_id, "/Geolocation", H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
  plist  = H5Pcreate(H5P_DATASET_CREATE);

/* Datasets latitude and longitude */

  rank = 2;
  dims[0] = h_scnlin;
  dims[1] = IRAS_SPOTS;
  if (clevel > 0){
    status = H5Pset_chunk(plist, rank, dims);
    status = H5Pset_deflate( plist, clevel);         /* gzip compression */
  }
  dataspace_id = H5Screate_simple(rank, dims, NULL); 

  dataset_id = H5Dcreate(file_id, "/Geolocation/Latitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, latitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Longitude", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, longitude);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &latlonscale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* Satellite and solar angles */

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_zenith_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solzen);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Satellite_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, satazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

  dataset_id = H5Dcreate(file_id, "/Geolocation/Solar_azimuth_angle", H5T_STD_I32LE, dataspace_id, H5P_DEFAULT, plist, H5P_DEFAULT);
  status = H5Dwrite(dataset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, solazi);
  status = add_string_attribute(dataset_id, "Units", "degree");
  status = add_float_attribute(dataset_id, "Scale", &anglescale, attdims);
  status = add_int_attribute(dataset_id, "FillValue", &fill, attdims);
  H5Dclose(dataset_id);

/* close the group and file */

  H5Sclose(dataspace_id);
  H5Pclose(plist);                    
  H5Gclose(group_id);
  H5Fclose(file_id);
}

#else

int iras1c_to_hdf5_write(char *filename, int clevel){ 

  printf ("Error: unable to write HDF-5\n");
  exit(1);

}
#endif
