! Name: $Id: iras1c_write_f.F 607 2019-01-11 10:12:41Z frna $
!
! Purpose: Write IRAS l1c file using information from C calling routine.
!          Subroutines IRAS1C_WRITE_HEADER and IRAS1C_WRITE_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2019, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    08/01/2019  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE IRAS1C_WRITE_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_tempradcnv, h_wmosatid,
     & h_nmapchanused_mwts, h_nmapchanused_mwhs)

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 IRAS_CHANS
      PARAMETER (IRAS_CHANS=26)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*4   h_site
      CHARACTER*4   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_tempradcnv(3,IRAS_CHANS)
      INTEGER*4     h_wmosatid
      INTEGER*4     h_nmapchanused_mwts
      INTEGER*4     h_nmapchanused_mwhs

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "iras1c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open IRAS 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="NEW", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=iras_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=iras_1crecl/rldiv, IOSTAT=errStatus )
        IF (errStatus.NE.0) THEN
          OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &     STATUS="UNKNOWN", FORM="UNFORMATTED", ACCESS="DIRECT",
     &     RECL=iras_1crecl/rldiv, IOSTAT=errStatus )
          IF (errStatus.NE.0) THEN
            write(*,*)'Cannot open file ',filename(1:filename_length)
            call aappexit(1)
          ENDIF
        ENDIF
      ENDIF

      write(*,*)'Write IRAS header'
      iras1c_h_scnlin = h_scnlin
      iras1c_h_site = h_site
      iras1c_h_cfill1 = ' '
      iras1c_h_1bsite = h_1bsite
      iras1c_h_cfill2 = ' '
      iras1c_h_versnb = iras_1cvnnb
      iras1c_h_versyr = iras_1cvnyr
      iras1c_h_versdy = iras_1cvndy
      iras1c_h_hdrcnt = 1
      iras1c_h_satid = h_satid
      iras1c_h_instrument = h_instrument
      iras1c_h_satht = h_satht
      iras1c_h_period = h_period
      iras1c_h_startorbit = h_startorbit 
      iras1c_h_startdatayr = h_startdatayr
      iras1c_h_startdatady = h_startdatady
      iras1c_h_startdatatime = h_startdatatime
      iras1c_h_endorbit = h_endorbit
      iras1c_h_enddatayr = h_enddatayr
      iras1c_h_enddatady = h_enddatady
      iras1c_h_enddatatime = h_enddatatime
      iras1c_h_scnlin = h_scnlin
      iras1c_h_misscnlin = h_misscnlin
      do ichan=1,IRAS_CHANS
        do i=1,3
         iras1c_h_tempradcnv(i,ichan) = h_tempradcnv(i,ichan) 
        enddo
      enddo
      iras1c_h_wmosatid = h_wmosatid
      iras1c_h_nchan = iras_nchan
      iras1c_h_nmapchan = iras_nmapchan
      iras1c_h_nmapchanused_mwts = h_nmapchanused_mwts
      iras1c_h_nmapchanused_mwhs = h_nmapchanused_mwhs

      CALL IOIRAS1C("WRITE HEADER",UNIT_1C,1)

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE IRAS1C_WRITE_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime, instrtemp,
     &  latitude, longitude, satzen, solzen, satazi, solazi,
     &  scalti, satposition, satvelocity, orbitangle, btemps,
     &  btmap, dataqual)
      IMPLICIT NONE

! Parameters
      INTEGER*4 IRAS_CHANS
      PARAMETER (IRAS_CHANS=26)
      INTEGER*4 IRAS_MAPCHANS
      PARAMETER (IRAS_MAPCHANS=28)
      INTEGER*4 IRAS_SPOTS
      PARAMETER (IRAS_SPOTS=56)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 instrtemp(9,h_scnlin)
      INTEGER*4 latitude(IRAS_SPOTS,h_scnlin)
      INTEGER*4 longitude(IRAS_SPOTS,h_scnlin)
      INTEGER*4 satzen(IRAS_SPOTS,h_scnlin)
      INTEGER*4 satazi(IRAS_SPOTS,h_scnlin)
      INTEGER*4 solzen(IRAS_SPOTS,h_scnlin)
      INTEGER*4 solazi(IRAS_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 satposition(3,h_scnlin)
      INTEGER*4 satvelocity(3,h_scnlin)
      INTEGER*4 orbitangle(h_scnlin)
      INTEGER*4 btemps(IRAS_CHANS,IRAS_SPOTS,h_scnlin)
      INTEGER*4 btmap(IRAS_MAPCHANS,IRAS_SPOTS,h_scnlin)
      INTEGER*4 dataqual(IRAS_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,i

#include "inparms.h"
#include "iras1c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Write IRAS data records'
      DO iscan=1,h_scnlin
        iras1c_scnlin = scnlin(iscan)
        iras1c_scnlinyr = scnlinyr(iscan)
        iras1c_scnlindy = scnlindy(iscan)
        iras1c_scnlintime = scnlintime(iscan)
        DO i=1,9
          iras1c_instrtemp(i) = instrtemp(i,iscan)
        ENDDO
        iras1c_scalti = scalti(iscan)
        DO i=1,3
          iras1c_satposition(i) = satposition(i,iscan)
          iras1c_satvelocity(i) = satvelocity(i,iscan)
        ENDDO
        iras1c_orbitangle = orbitangle(iscan)
        DO ispot=1,IRAS_SPOTS
          DO ichan=1,IRAS_CHANS
            iras1c_btemps(ichan,ispot) = btemps(ichan,ispot,iscan)
          ENDDO
          DO ichan=1,IRAS_MAPCHANS
            iras1c_btmap(ichan,ispot) = btmap(ichan,ispot,iscan)
          ENDDO
          iras1c_latlon(1,ispot) = latitude(ispot,iscan)
          iras1c_latlon(2,ispot) = longitude(ispot,iscan)
          iras1c_angles(1,ispot) = satzen(ispot,iscan)
          iras1c_angles(2,ispot) = satazi(ispot,iscan)
          iras1c_angles(3,ispot) = solzen(ispot,iscan)
          iras1c_angles(4,ispot) = solazi(ispot,iscan)
          iras1c_dataqual(ispot) = dataqual(ispot,iscan)
        ENDDO
        CALL IOIRAS1C("WRITE DATA",UNIT_1C,iscan+1)
      ENDDO

      CALL IOIRAS1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
