/*
! name: $Id: iras_sdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an IRAS SDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2013, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    31/01/2013  Adapt from mwts_sdr_read.c (N.C.Atkinson)
!  1.1    11/06/2014  Add FY-3C
!  1.2    12/09/2014  Correct the string comparisons (was not working for FY-3B)
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "iras_sdr.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int iras_sdr_read(char *filename){
  
  int     igran,iscan,jscan,ispot,ichan;        /* Loop variables */
  hid_t   file_id, group_id, dataset_id;        /* handles */
  hid_t   strtype;
  herr_t  status;
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannont open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */


/****** Open the group "/" ******/

  group_id = H5Gopen(file_id, "/", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }

/****** Read the data ******/

  status = H5Tset_size (strtype, 6);   /* Set the length of the string */
  readattribute("Satellite Name",group_id,strtype,Satellite_Name);

  status = H5Tset_size (strtype, 11);   /* Set the length of the string */
  readattribute("Observing Beginning Date",group_id,strtype,Beginning_Date);
  readattribute("Observing Ending Date",group_id,strtype,Ending_Date);

  status = H5Tset_size (strtype, 13);   /* Set the length of the string */
  readattribute("Observing Beginning Time",group_id,strtype,Beginning_Time);
  readattribute("Observing Ending Time",group_id,strtype,Ending_Time);

  readattribute("Number Of Scans", group_id, H5T_NATIVE_INT, &Count_scnlines);

/* Don't use "Orbit Number" because that is not valid in direct readout data */
  if ((strncmp(Satellite_Name,"FY-3A",5)==0) || (strncmp(Satellite_Name,"FY-3B",5)==0)) {
    status = H5Tset_size (strtype, 6);   /* Set the length of the string */
    readattribute("Orbit", group_id, strtype, Orbit_string);
    sscanf(Orbit_string,"%d",&Orbit_Number);
    readattribute("Orbit Period", group_id, H5T_NATIVE_INT, &Orbit_Period);
  } else {
    readattribute("Orbit Number", group_id, H5T_NATIVE_INT, &Orbit_Number);
    readattribute("Orbit Period(min.)", group_id, H5T_NATIVE_INT, &Orbit_Period);
  }

/* Various temperatures. One value per file, but may want to derive these on a
   per-scan basis in the future */
  if ((strncmp(Satellite_Name,"FY-3A",5)==0) || (strncmp(Satellite_Name,"FY-3B",5)==0)) {
    readattribute("ira_bb_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[0]);
    readattribute("ira_board_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[1]);
    readattribute("ira_box_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[2]);
    readattribute("ira_colder2_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[3]);
    readattribute("ira_main_optical_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[4]);
    readattribute("ira_mirror_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[5]);
    readattribute("ira_modulator_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[6]);
    readattribute("ira_relay_optic_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[7]);
    readattribute("ira_wheel_tempave", group_id, H5T_NATIVE_FLOAT, &Ave_Instr_Temp[8]);
  }
  readattribute("ira_central_wn", group_id, H5T_NATIVE_FLOAT, Central_Wn);
  if ((strncmp(Satellite_Name,"FY-3A",5)==0) || (strncmp(Satellite_Name,"FY-3B",5)==0)) {
    readdataset("FY3A_IRAS_TB", group_id, H5T_NATIVE_FLOAT, Earth_Obs_BT);
    readdataset("Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
    readdataset("Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
    readdataset("Height", group_id, H5T_NATIVE_SHORT, Height);
    readdataset("LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
    readdataset("SensorZenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
    readdataset("SensorAzimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
    readdataset("SolarZenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
    readdataset("SolarAzimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);
    readdataset("Ira_cal_qc", group_id, H5T_NATIVE_INT, cal_qc);
    readdataset("Ira_pixel_qc", group_id, H5T_NATIVE_INT, pixel_qc);
    readdataset("Ira_geo_qc", group_id, H5T_NATIVE_SHORT, geo_qc);
    readdataset("Ira_scnlin_qc", group_id, H5T_NATIVE_INT, scnlin_qc);
    readdataset("Scnlin_mscnt", group_id, H5T_NATIVE_INT, Scan_Time);
    readdataset("Scnlin_daycnt", group_id, H5T_NATIVE_INT, Scan_Day);
    readdataset("EVS_orb_pos", group_id, H5T_NATIVE_FLOAT, EVS_orb_pos);
  } else {
    readdataset("Data/IRAS_TB", group_id, H5T_NATIVE_FLOAT, Earth_Obs_BT);
    readdataset("Geolocation/Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
    readdataset("Geolocation/Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
    readdataset("Geolocation/DEM", group_id, H5T_NATIVE_SHORT, Height);
    readdataset("Geolocation/LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
    readdataset("Geolocation/SensorZenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
    readdataset("Geolocation/SensorAzimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
    readdataset("Geolocation/SolarZenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
    readdataset("Geolocation/SolarAzimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);
    readdataset("QA/Ira_scnlin_qc", group_id, H5T_NATIVE_INT, scnlin_qc);
    readdataset("Data/Scnlin_mscnt", group_id, H5T_NATIVE_INT, Scan_Time);
    readdataset("Data/Scnlin_daycnt", group_id, H5T_NATIVE_INT, Scan_Day);
    for (iscan=0; iscan<IRAS_SCANS; iscan++){
      EVS_orb_pos[iscan][0] = 0.0;    /* not available */
      EVS_orb_pos[iscan][1] = 0.0;
      EVS_orb_pos[iscan][2] = 0.0;
      cal_qc[iscan] = 0;
      geo_qc[iscan] = 0;
      for (ispot=0;ispot<IRAS_SPOTS;ispot++) pixel_qc[iscan][ispot] = 0;
    }
  }

  printf("Finished reading %s\n",filename);

/****** Clean up ******/

  H5Gclose(group_id);
  H5Fclose(file_id);
  return 0;

}
#endif
