/*
! name: $Id: mwhs_sdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an MWHS SDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    30/11/2010  Original code. (N.C.Atkinson)
!  1.1    03/10/2012  Add Height and LandSeaMask (as for mwts)
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "mwhs_sdr.h"
#include <stdio.h>
#include <stdlib.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int mwhs_sdr_read(char *filename){
  
  int     igran,iscan,jscan,ispot,ichan;        /* Loop variables */
  hid_t   file_id, group_id, dataset_id;        /* handles */
  hid_t   strtype;
  hid_t   time_vdata_tid;
  hid_t   instr_vdata_tid;
  herr_t  status;
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannont open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */


/****** Open the group "/" ******/

  group_id = H5Gopen(file_id, "/", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }

/****** Read the data ******/

  status = H5Tset_size (strtype, 6);   /* Set the length of the string */
  readattribute("Satellite Name",group_id,strtype,Satellite_Name);

  status = H5Tset_size (strtype, 11);   /* Set the length of the string */
  readattribute("Observing Beginning Date",group_id,strtype,Beginning_Date);
  readattribute("Observing Ending Date",group_id,strtype,Ending_Date);

  status = H5Tset_size (strtype, 13);   /* Set the length of the string */
  readattribute("Observing Beginning Time",group_id,strtype,Beginning_Time);
  readattribute("Observing Ending Time",group_id,strtype,Ending_Time);

  readattribute("Count_scnlines", group_id, H5T_NATIVE_INT, &Count_scnlines);
  readattribute("Orbit Number", group_id, H5T_NATIVE_INT, &Orbit_Number);
  readattribute("Orbit Period", group_id, H5T_NATIVE_INT, &Orbit_Period);

  readdataset("Earth_Obs_BT", group_id, H5T_NATIVE_FLOAT, Earth_Obs_BT);
  readdataset("Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
  readdataset("Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
  readdataset("Height", group_id, H5T_NATIVE_SHORT, Height);
  readdataset("LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
  readdataset("SensorZenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
  readdataset("SensorAzimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
  readdataset("SolarZenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
  readdataset("SolarAzimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);
  readdataset("Black_Body_View_Ang", group_id, H5T_NATIVE_SHORT, Black_Body_View_Ang);
  readdataset("Space_View_Ang", group_id, H5T_NATIVE_SHORT, Space_View_Ang);

  readdataset("GPS_Position_Time", group_id, H5T_NATIVE_INT, GPS_Position_Time);
  readdataset("GPS_Position_Data", group_id, H5T_NATIVE_INT, GPS_Position_Data);

  printf("Finished reading %s\n",filename);

/****** Get the raw time info and instrument temp 
        (code based on example at http://beige.ucs.indiana.edu/I590/node144.html) ******/

  H5Gclose(group_id);
  group_id = H5Gopen(file_id, "GROUP_TABLE", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open GROUP_TABLE\n");
    exit(10);
  }
  dataset_id = H5Dopen(group_id, "Time Vdata", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open dataset Time Vdata\n");
    exit(10);
  }

  time_vdata_tid = H5Tcreate(H5T_COMPOUND, sizeof(t_time_vdata));
  H5Tinsert(time_vdata_tid, "Package_Counter", HOFFSET(t_time_vdata, Package_Counter), H5T_NATIVE_SHORT);
  H5Tinsert(time_vdata_tid, "RawScanLine_dayTime_for_1th", HOFFSET(t_time_vdata, RawScanLine_dayTime_for_1th), H5T_NATIVE_INT);
  H5Tinsert(time_vdata_tid, "RawScanLine_dayTime_for_49th", HOFFSET(t_time_vdata, RawScanLine_dayTime_for_49th), H5T_NATIVE_INT);
  H5Tinsert(time_vdata_tid, "RawScanLine_dayTime_for_50th", HOFFSET(t_time_vdata, RawScanLine_dayTime_for_50th), H5T_NATIVE_INT);
  H5Tinsert(time_vdata_tid, "RawScanLine_dayTime_for_98th", HOFFSET(t_time_vdata, RawScanLine_dayTime_for_98th), H5T_NATIVE_INT);
  H5Tinsert(time_vdata_tid, "RawScanLine_msTime_for_1th", HOFFSET(t_time_vdata, RawScanLine_msTime_for_1th), H5T_NATIVE_INT);
  H5Tinsert(time_vdata_tid, "RawScanLine_msTime_for_49th", HOFFSET(t_time_vdata, RawScanLine_msTime_for_49th), H5T_NATIVE_INT);
  H5Tinsert(time_vdata_tid, "RawScanLine_msTime_for_50th", HOFFSET(t_time_vdata, RawScanLine_msTime_for_50th), H5T_NATIVE_INT);
  H5Tinsert(time_vdata_tid, "RawScanLine_msTime_for_98th", HOFFSET(t_time_vdata, RawScanLine_msTime_for_98th), H5T_NATIVE_INT);
  H5Tinsert(time_vdata_tid, "Quality_Flag_for_Time_Data", HOFFSET(t_time_vdata, Quality_Flag_for_Time_Data), H5T_NATIVE_SHORT);

  status = H5Dread(dataset_id, time_vdata_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, time_vdata);
  for (iscan=0; iscan<Count_scnlines; iscan++){
    if ((time_vdata[iscan].Quality_Flag_for_Time_Data == 0) &&
        (time_vdata[iscan].RawScanLine_dayTime_for_1th == time_vdata[iscan].RawScanLine_dayTime_for_49th) &&
        (time_vdata[iscan].RawScanLine_dayTime_for_49th == time_vdata[iscan].RawScanLine_dayTime_for_50th) &&
        (time_vdata[iscan].RawScanLine_dayTime_for_50th == time_vdata[iscan].RawScanLine_dayTime_for_98th)) {
      Raw_Time[iscan] = time_vdata[iscan].RawScanLine_msTime_for_1th;
    }
    else Raw_Time[iscan] = -1;
    Package_Counter[iscan] = time_vdata[iscan].Package_Counter;
  }
  H5Dclose(dataset_id);
  
  dataset_id = H5Dopen(group_id, "Instrument Performance Vdata", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open dataset Instrument Performance Vdata\n");
    exit(10);
  }
  instr_vdata_tid = H5Tcreate(H5T_COMPOUND, sizeof(t_instr_vdata));
  H5Tinsert(instr_vdata_tid, "Mixer_183GHz_Temperature", HOFFSET(t_instr_vdata, Mixer_183GHz_Temperature), H5T_NATIVE_FLOAT);

  status = H5Dread(dataset_id, instr_vdata_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, instr_vdata);
  for (iscan=0; iscan<Count_scnlines; iscan++){
    Mixer_Temp[iscan] = instr_vdata[iscan].Mixer_183GHz_Temperature;
  }
  H5Dclose(dataset_id);


/****** Clean up ******/

  H5Gclose(group_id);
  H5Fclose(file_id);
  return 0;

}
#endif
