! Name: $Id: mwri1c_read_f.F 542 2017-08-11 14:00:03Z frna $
!
! Purpose: Read MWRI l1c file and pass information back to C calling routine.
!          Subroutines MWRI1C_READ_HEADER and MWRI1C_READ_DATA
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    08/08/2017  N C Atkinson
!-----------------------------------------------------------------------

      SUBROUTINE MWRI1C_READ_HEADER (
     & filename, filename_length,
     & h_site, h_1bsite, h_versnb,
     & h_satid, h_instrument, h_satht, h_period,
     & h_startorbit, h_startdatayr, h_startdatady, h_startdatatime,
     & h_endorbit, h_enddatayr, h_enddatady, h_enddatatime,
     & h_scnlin, h_misscnlin, h_vnantennacorr, h_tempradcnv, h_wmosatid,
     & h_bandwidth, h_nchan, h_nmapchan, h_nmapchanused, h_beamversion,
     & h_beamwidth, h_cutoff, h_nxaverage, h_nyaverage )

      IMPLICIT NONE
      
! Parameters
      INTEGER   UNIT_1C
      PARAMETER (UNIT_1C=10)
      INTEGER*4 MWRI_CHANS
      PARAMETER (MWRI_CHANS=10)

! Arguments: 
      CHARACTER*(*) filename
      INTEGER       filename_length
      CHARACTER*3   h_site
      CHARACTER*3   h_1bsite
      INTEGER*4     h_versnb
      INTEGER*4     h_satid
      INTEGER*4     h_instrument
      INTEGER*4     h_satht
      INTEGER*4     h_period
      INTEGER*4     h_startorbit
      INTEGER*4     h_startdatayr
      INTEGER*4     h_startdatady
      INTEGER*4     h_startdatatime
      INTEGER*4     h_endorbit
      INTEGER*4     h_enddatayr
      INTEGER*4     h_enddatady
      INTEGER*4     h_enddatatime
      INTEGER*4     h_scnlin
      INTEGER*4     h_misscnlin
      INTEGER*4     h_vnantennacorr
      INTEGER*4     h_tempradcnv(3,MWRI_CHANS)
      INTEGER*4     h_wmosatid
      INTEGER*4     h_bandwidth(MWRI_CHANS)
      INTEGER*4     h_nchan
      INTEGER*4     h_nmapchan
      INTEGER*4     h_nmapchanused
      INTEGER*4     h_beamversion
      INTEGER*4     h_beamwidth(MWRI_CHANS)
      INTEGER*4     h_cutoff(MWRI_CHANS)
      INTEGER*4     h_nxaverage(MWRI_CHANS)
      INTEGER*4     h_nyaverage(MWRI_CHANS)

! Local:
      INTEGER errStatus, i, ichan

#include "inparms.h"
#include "mwri1c.h"
#include "reclopt.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Open MWRI 1c file ',filename(1:filename_length)
      OPEN ( UNIT=UNIT_1C, FILE=filename(1:filename_length), 
     &   STATUS="OLD", FORM="UNFORMATTED", ACCESS="DIRECT",
     &   RECL=mwri_1crecl/rldiv, IOSTAT=errStatus )
      IF (errStatus.NE.0) THEN
        write(*,*)'Cannot open file ',filename(1:filename_length)
        call aappexit(1)
      ENDIF

      write(*,*)'Read MWRI header'
      CALL IOMWRI1C("READ HEADER",UNIT_1C,1)

      h_scnlin = mwri1c_h_scnlin
      h_site = mwri1c_h_site
      h_1bsite = mwri1c_h_1bsite
      h_versnb = mwri1c_h_versnb
      h_satid = mwri1c_h_satid
      h_instrument = mwri1c_h_instrument
      h_satht = mwri1c_h_satht
      h_period = mwri1c_h_period
      h_startorbit = mwri1c_h_startorbit
      h_startdatayr = mwri1c_h_startdatayr
      h_startdatady = mwri1c_h_startdatady
      h_startdatatime = mwri1c_h_startdatatime
      h_endorbit = mwri1c_h_endorbit
      h_enddatayr = mwri1c_h_enddatayr
      h_enddatady = mwri1c_h_enddatady
      h_enddatatime = mwri1c_h_enddatatime
      h_scnlin = mwri1c_h_scnlin
      h_misscnlin = mwri1c_h_misscnlin
      h_vnantennacorr = mwri1c_h_vnantennacorr
      do ichan=1,MWRI_CHANS
        do i=1,3
          h_tempradcnv(i,ichan) = mwri1c_h_tempradcnv(i,ichan)
        enddo
        h_bandwidth(ichan) = mwri1c_h_bandwidth(ichan)
        h_beamwidth(ichan) = mwri1c_h_beamwidth(ichan)
        h_cutoff(ichan) = mwri1c_h_cutoff(ichan)
        h_nxaverage(ichan) = mwri1c_h_nxaverage(ichan)
        h_nyaverage(ichan) = mwri1c_h_nyaverage(ichan)
      enddo
      h_wmosatid = mwri1c_h_wmosatid
      h_nchan = mwri1c_h_nchan
      h_nmapchan = mwri1c_h_nmapchan
      h_nmapchanused = mwri1c_h_nmapchanused
      h_beamversion = mwri1c_h_beamversion

      RETURN
      END
      
!-------------------------------------------------------------------------------

      SUBROUTINE MWRI1C_READ_DATA (
     &  h_scnlin,
     &  scnlin, scnlinyr, scnlindy, scnlintime,
     &  qualind, scanqual, chanqual, instrtemp1, instrtemp2,
     &  latitude, longitude, satzen, solzen, satazi, solazi, scalti,
     &  satposition, satvelocity, orbitangle,
     &  btemps, btmap, dataqual)
      IMPLICIT NONE

! Parameters
      INTEGER*4 MWRI_CHANS
      PARAMETER (MWRI_CHANS=10)
      INTEGER*4 MWRI_MAPCHANS
      PARAMETER (MWRI_MAPCHANS=10)
      INTEGER*4 MWRI_SPOTS
      PARAMETER (MWRI_SPOTS=254)
      INTEGER*4 UNIT_1C
      PARAMETER (UNIT_1C=10)

! Arguments: 
      INTEGER*4 h_scnlin
      INTEGER*4 scnlin(h_scnlin)
      INTEGER*4 scnlinyr(h_scnlin)
      INTEGER*4 scnlindy(h_scnlin)
      INTEGER*4 scnlintime(h_scnlin)
      INTEGER*4 qualind(h_scnlin)
      INTEGER*4 scanqual(h_scnlin)
      INTEGER*4 chanqual(MWRI_CHANS,h_scnlin)
      INTEGER*4 instrtemp1(h_scnlin)
      INTEGER*4 instrtemp2(h_scnlin)
      INTEGER*4 latitude(MWRI_SPOTS,h_scnlin)
      INTEGER*4 longitude(MWRI_SPOTS,h_scnlin)
      INTEGER*4 satzen(MWRI_SPOTS,h_scnlin)
      INTEGER*4 satazi(MWRI_SPOTS,h_scnlin)
      INTEGER*4 solzen(MWRI_SPOTS,h_scnlin)
      INTEGER*4 solazi(MWRI_SPOTS,h_scnlin)
      INTEGER*4 scalti(h_scnlin)
      INTEGER*4 satposition(3,h_scnlin)
      INTEGER*4 satvelocity(3,h_scnlin)
      INTEGER*4 orbitangle(h_scnlin)
      INTEGER*4 btemps(MWRI_CHANS,MWRI_SPOTS,h_scnlin)
      INTEGER*4 btmap(MWRI_MAPCHANS,MWRI_SPOTS,h_scnlin)
      INTEGER*4 dataqual(MWRI_SPOTS,h_scnlin)

! Local
      INTEGER iscan,ichan,ispot,i

#include "inparms.h"
#include "mwri1c.h"

! End of declarations
!-----------------------------------------------------------------------

      write(*,*)'Read MWRI data records'
      DO iscan=1,h_scnlin
        CALL IOMWRI1C("READ DATA",UNIT_1C,iscan+1)
        scnlin(iscan) = mwri1c_scnlin
        scnlinyr(iscan) = mwri1c_scnlinyr
        scnlindy(iscan) = mwri1c_scnlindy
        scnlintime(iscan) = mwri1c_scnlintime
        qualind(iscan) = mwri1c_qualind
        scanqual(iscan) = mwri1c_scanqual
        instrtemp1(iscan) = mwri1c_instrtemp1
        instrtemp2(iscan) = mwri1c_instrtemp2
        scalti(iscan) = mwri1c_scalti
        DO ichan=1,MWRI_CHANS
          chanqual(ichan,iscan) = mwri1c_chanqual(ichan)
        ENDDO
        DO ispot=1,MWRI_SPOTS
          DO ichan=1,MWRI_CHANS
            btemps(ichan,ispot,iscan) = mwri1c_btemps(ichan,ispot)
          ENDDO
          DO ichan=1,MWRI_MAPCHANS
            btmap(ichan,ispot,iscan) = mwri1c_btmap(ichan,ispot)
          ENDDO
          latitude(ispot,iscan) = mwri1c_latlon(1,ispot)
          longitude(ispot,iscan) = mwri1c_latlon(2,ispot)
          satzen(ispot,iscan) = mwri1c_angles(1,ispot)
          satazi(ispot,iscan) = mwri1c_angles(2,ispot)
          solzen(ispot,iscan) = mwri1c_angles(3,ispot)
          solazi(ispot,iscan) = mwri1c_angles(4,ispot)
          dataqual(ispot,iscan) = mwri1c_dataqual(ispot)
        ENDDO
        DO i=1,3
          satposition(i,iscan) = mwri1c_satposition(i)
          satvelocity(i,iscan) = mwri1c_satvelocity(i)
        ENDDO
        orbitangle(iscan) = mwri1c_orbitangle
      ENDDO

      CALL IOMWRI1C("CLOSE",UNIT_1C,0)

      RETURN
      END
      
