/*
! name: $Id: mwri2_sdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an mwri SDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    08/04/2014  From mwts2_sdr_read.c (N.C.Atkinson)
!  1.1    05/02/2016  Get it working for FY-3A/3B
!  1.2    09/08/2018  Version for FY-3D with 266 spots
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "mwri2_sdr.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int mwri2_sdr_read(char *filename){
  
  hid_t   file_id, group_id, dataset_id;        /* handles */
  hid_t   strtype;
  herr_t  status;
  
  float   Scan_Time_and_Period[MWRI_SCANS][4];   /* Used for FY-3A/3B */
  int     iscan;
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannont open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */

/****** Open the group "/" ******/

  group_id = H5Gopen(file_id, "/", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }

/****** Read the data ******/

  status = H5Tset_size (strtype, 6);   /* Set the length of the string */
  readattribute("Satellite Name",group_id,strtype,Satellite_Name);

  status = H5Tset_size (strtype, 11);   /* Set the length of the string */
  readattribute("Observing Beginning Date",group_id,strtype,Beginning_Date);
  readattribute("Observing Ending Date",group_id,strtype,Ending_Date);

  status = H5Tset_size (strtype, 13);   /* Set the length of the string */
  readattribute("Observing Beginning Time",group_id,strtype,Beginning_Time);
  readattribute("Observing Ending Time",group_id,strtype,Ending_Time);

  readattribute("Number Of Scans", group_id, H5T_NATIVE_SHORT, &Count_scnlines);
  readattribute("Orbit Number", group_id, H5T_NATIVE_SHORT, &Orbit_Number);

  if ((strncmp(Satellite_Name,"FY-3A",5) == 0) || (strncmp(Satellite_Name,"FY-3B",5) == 0)) {

    readattribute("Orbit Period", group_id, H5T_NATIVE_SHORT, &Orbit_Period);

    readdataset("EARTH_OBSERVE_BT_10_to_89GHz", group_id, H5T_NATIVE_SHORT, Earth_Obs_BT); 
    readattribute("QA_Scan_Flag", group_id, H5T_NATIVE_SHORT, QA_Scan_Flag);
    readattribute("QA_Pixel_Flag", group_id, H5T_NATIVE_SHORT, QA_Ch_Flag);
    readdataset("Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
    readdataset("Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
    readdataset("LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
    readdataset("Height", group_id, H5T_NATIVE_SHORT, DEM);
    readdataset("SensorZenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
    readdataset("SensorAzimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
    readdataset("SolarZenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
    readdataset("SolarAzimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);
    readdataset("Scan_Time_and_Period", group_id, H5T_NATIVE_FLOAT, Scan_Time_and_Period);

    for (iscan=0; iscan<Count_scnlines; iscan++){    /* convert to FY-3C variables */
      Scnlin_daycnt[iscan] = Scan_Time_and_Period[iscan][0];
      Scnlin_mscnt[iscan][0] = Scan_Time_and_Period[iscan][1];
      Scnlin_mscnt[iscan][1] = Scan_Time_and_Period[iscan][3];
    }
    
    /* Store QA_Pixel_Flag in the QA_Ch_Flag variable; not clear whether it's the
       same as QA_Ch_Flag on FY-3C.
       Not available: Pixel_View_Angle, scnlin_qc */

  } else if (strncmp(Satellite_Name,"FY-3C",5) == 0) {

    readattribute("Orbit Period(min.)", group_id, H5T_NATIVE_SHORT, &Orbit_Period);

    readdataset("Data/EARTH_OBSERVE_BT_10_to_89GHz", group_id, H5T_NATIVE_SHORT, Earth_Obs_BT); 
    readdataset("QA/QA_Ch_Flag", group_id, H5T_NATIVE_SHORT, QA_Ch_Flag);
    readdataset("QA/QA_Scan_Flag", group_id, H5T_NATIVE_SHORT, QA_Scan_Flag);
    readdataset("Geolocation/Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
    readdataset("Geolocation/Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
    readdataset("Data/LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
    readdataset("Data/DEM", group_id, H5T_NATIVE_SHORT, DEM);
    readdataset("Geolocation/SensorZenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
    readdataset("Geolocation/SensorAzimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
    readdataset("Geolocation/SolarZenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
    readdataset("Geolocation/SolarAzimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);
    readdataset("Data/Scan_daycnt", group_id, H5T_NATIVE_SHORT, Scnlin_daycnt);
    readdataset("Data/Scan_mscnt", group_id, H5T_NATIVE_FLOAT, Scnlin_mscnt);

    /* Not available: Pixel_View_Angle, scnlin_qc */

  } else {    /* FY-3D */

    readattribute("Orbit Period(min.)", group_id, H5T_NATIVE_SHORT, &Orbit_Period);

    readdataset("Calibration/EARTH_OBSERVE_BT_10_to_89GHz", group_id, H5T_NATIVE_SHORT, Earth_Obs_BT); 
    readdataset("QA/QA_Ch_Flag", group_id, H5T_NATIVE_SHORT, QA_Ch_Flag);
    readdataset("QA/QA_Scan_Flag", group_id, H5T_NATIVE_SHORT, QA_Scan_Flag);
    readdataset("Geolocation/Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
    readdataset("Geolocation/Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
    readdataset("Calibration/LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
    readdataset("Calibration/DEM", group_id, H5T_NATIVE_SHORT, DEM);
    readdataset("Geolocation/Sensor_Zenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
    readdataset("Geolocation/Sensor_Azimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
    readdataset("Geolocation/Solar_Zenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
    readdataset("Geolocation/Solar_Azimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);
    readdataset("Calibration/Scan_Daycnt", group_id, H5T_NATIVE_SHORT, Scnlin_daycnt);
    readdataset("Calibration/Scan_Mscnt", group_id, H5T_NATIVE_FLOAT, Scnlin_mscnt);

    /* Not available: Pixel_View_Angle, scnlin_qc */

  }


/****** Clean up ******/

  H5Gclose(group_id);
  H5Fclose(file_id);
  return 0;

}
#endif
