/* 
! Name: $Id: mwts21c_from_hdf5_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Subroutine to read MWTS2 1c data from HDF-5 file
!
! Method: 
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2019, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date       Comment
!
!  1.0      08/01/2019  N C Atkinson
!
! Code Description: C
!
! C subroutines and functions called:
!     HDF-5 library routines
!
!-------------------------------------------------------------------------------
*/
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "aapp_hdf5.h"

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include <string.h>

#include "common_read.h"
#include "mwts21c_c.h"

int mwts21c_from_hdf5_read(char *filename){ 

  hid_t       file_id;
  hid_t       group_id;
  hid_t       dataspace_id;
  hid_t       dataset_id;
  hid_t       attribute_id;
  hid_t       plist;
  hid_t       strtype;       /* type ID string */
  hsize_t     dims[3];       /* dimensions */
  hsize_t     attdims[3];    /* attribute dimensions */
  herr_t      status;
  int         rank;
  int         i;
  int         ichan;
  float       wavenumber[MWTS2_CHANS];
  float       temprad_offset[MWTS2_CHANS];
  float       temprad_slope[MWTS2_CHANS];
  float       bandwidth[MWTS2_CHANS];
  float       beamwidth[MWTS2_CHANS];
  float       cutoff[MWTS2_CHANS];
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannot open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */

/****** Read top-level attributes ******/

  status = H5Tset_size (strtype, 4);
  readattribute("site",file_id,strtype,h_site);
  readattribute("1bsite",file_id,strtype,h_1bsite);
  readattribute("satid",file_id,H5T_NATIVE_INT,&h_satid);
  readattribute("instrument",file_id,H5T_NATIVE_INT,&h_instrument);
  readattribute("satht_hectometres",file_id,H5T_NATIVE_INT,&h_satht);
  readattribute("period_seconds",file_id,H5T_NATIVE_INT,&h_period);
  readattribute("startorbit",file_id,H5T_NATIVE_INT,&h_startorbit);
  readattribute("startdatayr",file_id,H5T_NATIVE_INT,&h_startdatayr);
  readattribute("startdatady",file_id,H5T_NATIVE_INT,&h_startdatady);
  readattribute("startdatatime_ms",file_id,H5T_NATIVE_INT,&h_startdatatime);
  readattribute("endorbit",file_id,H5T_NATIVE_INT,&h_endorbit);
  readattribute("enddatayr",file_id,H5T_NATIVE_INT,&h_enddatayr);
  readattribute("enddatady",file_id,H5T_NATIVE_INT,&h_enddatady);
  readattribute("enddatatime_ms",file_id,H5T_NATIVE_INT,&h_enddatatime);
  readattribute("scnlin",file_id,H5T_NATIVE_INT,&h_scnlin);
  readattribute("misscnlin",file_id,H5T_NATIVE_INT,&h_misscnlin);
  readattribute("vnantennacorr",file_id,H5T_NATIVE_INT,&h_vnantennacorr);
  readattribute("wmosatid",file_id,H5T_NATIVE_INT,&h_wmosatid);
  readattribute("nmapchanused",file_id,H5T_NATIVE_INT,&h_nmapchanused);
  readattribute("beamversion",file_id,H5T_NATIVE_INT,&h_beamversion);
  readattribute("MWTS2_wavenumber",file_id,H5T_NATIVE_FLOAT,wavenumber);
  readattribute("MWTS2_temprad_offset",file_id,H5T_NATIVE_FLOAT,temprad_offset);
  readattribute("MWTS2_temprad_slope",file_id,H5T_NATIVE_FLOAT,temprad_slope);
  readattribute("bandwidth",file_id,H5T_NATIVE_FLOAT,bandwidth);
  readattribute("beamwidth",file_id,H5T_NATIVE_FLOAT,beamwidth);
  readattribute("cutoff",file_id,H5T_NATIVE_FLOAT,cutoff);
  readattribute("nxaverage",file_id,H5T_NATIVE_INT,h_nxaverage);
  readattribute("nyaverage",file_id,H5T_NATIVE_INT,h_nyaverage);

  for (ichan=0; ichan<MWTS2_CHANS; ichan++){
    h_tempradcnv[ichan][0] = floor(wavenumber[ichan]*1E6 + 0.5);
    h_tempradcnv[ichan][1] = floor(temprad_offset[ichan]*1E6 + 0.5);
    h_tempradcnv[ichan][2] = floor(temprad_slope[ichan]*1E6 + 0.5);
    h_bandwidth[ichan] = floor(bandwidth[ichan]*1E6 + 0.5);
    h_beamwidth[ichan] = floor(beamwidth[ichan]*1E3 + 0.5);
    h_cutoff[ichan] = floor(cutoff[ichan]*1E3 + 0.5);
  }

/****** Allocate memory ******/

  scnlin = (int*)calloc(h_scnlin, sizeof(int));
  scnlinyr = (int*)calloc(h_scnlin, sizeof(int));
  scnlindy = (int*)calloc(h_scnlin, sizeof(int));
  scnlintime = (int*)calloc(h_scnlin, sizeof(int));
  qualind = (int*)calloc(h_scnlin, sizeof(int));
  scanqual = (int*)calloc(h_scnlin, sizeof(int));
  chanqual = (int*)calloc(h_scnlin*MWTS2_CHANS, sizeof(int));
  instrtemp1 = (int*)calloc(h_scnlin, sizeof(int));
  instrtemp2 = (int*)calloc(h_scnlin, sizeof(int));
  latitude = (int*)calloc(h_scnlin*MWTS2_SPOTS, sizeof(int));
  longitude = (int*)calloc(h_scnlin*MWTS2_SPOTS, sizeof(int));
  satzen = (int*)calloc(h_scnlin*MWTS2_SPOTS, sizeof(int));
  solzen = (int*)calloc(h_scnlin*MWTS2_SPOTS, sizeof(int));
  satazi = (int*)calloc(h_scnlin*MWTS2_SPOTS, sizeof(int));
  solazi = (int*)calloc(h_scnlin*MWTS2_SPOTS, sizeof(int));
  scalti = (int*)calloc(h_scnlin, sizeof(int));
  satposition = (int*)calloc(h_scnlin*3, sizeof(int));
  satvelocity = (int*)calloc(h_scnlin*3, sizeof(int));
  orbitangle = (int*)calloc(h_scnlin, sizeof(int));
  btemps = (int*)calloc(h_scnlin*MWTS2_SPOTS*MWTS2_CHANS, sizeof(int));
  btmap = (int*)calloc(h_scnlin*MWTS2_SPOTS*MWTS2_MAPCHANS, sizeof(int));
  dataqual = (int*)calloc(h_scnlin*MWTS2_SPOTS, sizeof(int));

/****** Open and read the group "/Data" ******/

  group_id = H5Gopen(file_id, "/Data", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }
  readdataset("scnlin",group_id,H5T_NATIVE_INT,scnlin);
/*  for (i=0; i<h_scnlin; i++) printf("%d %d\n",i,scnlin[i]); */
  readdataset("scnlinyr",group_id,H5T_NATIVE_INT,scnlinyr);
  readdataset("scnlindy",group_id,H5T_NATIVE_INT,scnlindy);
  readdataset("scnlintime",group_id,H5T_NATIVE_INT,scnlintime);
  readdataset("qualind",group_id,H5T_NATIVE_INT,qualind);
  readdataset("scanqual",group_id,H5T_NATIVE_INT,scanqual);
  readdataset("chanqual",group_id,H5T_NATIVE_INT,chanqual);
  readdataset("instrtemp1",group_id,H5T_NATIVE_INT,instrtemp1);
  readdataset("instrtemp2",group_id,H5T_NATIVE_INT,instrtemp2);
  readdataset("scalti",group_id,H5T_NATIVE_INT,scalti);
  readdataset("satposition",group_id,H5T_NATIVE_INT,satposition);
  readdataset("satvelocity",group_id,H5T_NATIVE_INT,satvelocity);
  readdataset("orbitangle",group_id,H5T_NATIVE_INT,orbitangle);
  readdataset("btemps",group_id,H5T_NATIVE_INT,btemps);
  if (h_nmapchanused > 0) readdataset("btmap",group_id,H5T_NATIVE_INT,btmap);
  readdataset("dataqual",group_id,H5T_NATIVE_INT,dataqual);
  H5Gclose(group_id);

/****** Open and read the group "/Geolocation" ******/

  group_id = H5Gopen(file_id, "/Geolocation", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }
  readdataset("Latitude",group_id,H5T_NATIVE_INT,latitude);
  readdataset("Longitude",group_id,H5T_NATIVE_INT,longitude);
  readdataset("Satellite_zenith_angle",group_id,H5T_NATIVE_INT,satzen);
  readdataset("Solar_zenith_angle",group_id,H5T_NATIVE_INT,solzen);
  readdataset("Satellite_azimuth_angle",group_id,H5T_NATIVE_INT,satazi);
  readdataset("Solar_azimuth_angle",group_id,H5T_NATIVE_INT,solazi);
  H5Gclose(group_id);

/****** Clean up ******/

  H5Fclose(file_id);
  printf("Finished reading %s\n",filename);
  return 0;

}

#else

int mwts21c_from_hdf5_read(char *filename){

  printf ("Error: unable to read HDF-5\n");
  exit(1);

}
#endif
