/*
! name: $Id: mwts2_sdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an MWTS2 SDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    08/04/2014  From mwts_sdr_read.c (N.C.Atkinson)
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "mwts2_sdr.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int mwts2_sdr_read(char *filename){
  
  hid_t   file_id, group_id, dataset_id;        /* handles */
  hid_t   strtype;
  herr_t  status;
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannont open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */


/****** Open the group "/" ******/

  group_id = H5Gopen(file_id, "/", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }

/****** Read the data ******/

  status = H5Tset_size (strtype, 6);   /* Set the length of the string */
  readattribute("Satellite Name",group_id,strtype,Satellite_Name);

  status = H5Tset_size (strtype, 11);   /* Set the length of the string */
  readattribute("Observing Beginning Date",group_id,strtype,Beginning_Date);
  readattribute("Observing Ending Date",group_id,strtype,Ending_Date);

  status = H5Tset_size (strtype, 13);   /* Set the length of the string */
  readattribute("Observing Beginning Time",group_id,strtype,Beginning_Time);
  readattribute("Observing Ending Time",group_id,strtype,Ending_Time);

  readattribute("Number Of Scans", group_id, H5T_NATIVE_INT, &Count_scnlines);
  readattribute("Orbit Number", group_id, H5T_NATIVE_INT, &Orbit_Number);
  readattribute("Orbit Period(min.)", group_id, H5T_NATIVE_INT, &Orbit_Period);

  readdataset("Data/Earth_Obs_BT", group_id, H5T_NATIVE_SHORT, Earth_Obs_BT);
  if (strncmp(Satellite_Name,"FY-3C",5)==0) {
    readdataset("Data/Earth_Obs_Angle", group_id, H5T_NATIVE_FLOAT, Earth_Obs_Angle);
    readdataset("Data/Quality_Flag_Channels", group_id, H5T_NATIVE_SHORT, Quality_Flag_Channels);
    readdataset("Data/Quality_Flag_Scnlin", group_id, H5T_NATIVE_SHORT, Quality_Flag_Scnlin);
  } else {       /* FY-3D */
    readdataset("Geolocation/Earth_Obs_Angle", group_id, H5T_NATIVE_FLOAT, Earth_Obs_Angle_all);
    readdataset("QA/Quality_Flag_Channels", group_id, H5T_NATIVE_SHORT, Quality_Flag_Channels);
    readdataset("QA/Quality_Flag_Scnlin", group_id, H5T_NATIVE_SHORT, Quality_Flag_Scnlin);
  }
  readdataset("Geolocation/Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
  readdataset("Geolocation/Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
  readdataset("Geolocation/LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
  readdataset("Geolocation/DEM", group_id, H5T_NATIVE_SHORT, DEM);
  readdataset("Geolocation/SensorZenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
  readdataset("Geolocation/SensorAzimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
  readdataset("Geolocation/SolarZenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
  readdataset("Geolocation/SolarAzimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);

/* Time is an 8-element array (year, month, day, hour, minute, second, millisecond, days since 2000-01-01) */

  readdataset("Data/Time", group_id, H5T_NATIVE_INT, Time);

  printf("Finished reading %s\n",filename);

/****** Clean up ******/

  H5Gclose(group_id);
  H5Fclose(file_id);
  return 0;

}
#endif
