/*
! name: $Id: mwts_sdr_read.c 635 2019-04-15 15:08:00Z frna $
!
! Purpose: Read the contents of an MWHS SDR in HDF5 format
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 01 December 2006, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2010, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0    30/11/2010  Original code. (N.C.Atkinson)
!
! Code Description: C
!
!-------------------------------------------------------------------------------
*/

#ifdef HAS_LIBHDF5

#include "hdf5.h"
#include "mwts_sdr.h"
#include <stdio.h>
#include <stdlib.h>

#include "aapp_hdf5.h"
#include "common_read.h"

int mwts_sdr_read(char *filename){
  
  int     igran,iscan,jscan,ispot,ichan;        /* Loop variables */
  hid_t   file_id, group_id, dataset_id;        /* handles */
  hid_t   strtype;
  herr_t  status;
  
/****** Open the file ******/

  file_id = H5Fopen(filename, H5F_ACC_RDONLY, H5P_DEFAULT);
  if (file_id < 0) {
    fprintf(stderr,"Cannont open file %s\n",filename);
    exit(10);
  }

  strtype = H5Tcopy (H5T_C_S1);    /* Initialise */


/****** Open the group "/" ******/

  group_id = H5Gopen(file_id, "/", H5P_DEFAULT);
  if (group_id < 0) {
    fprintf(stderr,"Cannot open group\n");
    exit(10);
  }

/****** Read the data ******/

  status = H5Tset_size (strtype, 6);   /* Set the length of the string */
  readattribute("Satellite Name",group_id,strtype,Satellite_Name);

  status = H5Tset_size (strtype, 11);   /* Set the length of the string */
  readattribute("Observing Beginning Date",group_id,strtype,Beginning_Date);
  readattribute("Observing Ending Date",group_id,strtype,Ending_Date);

  status = H5Tset_size (strtype, 13);   /* Set the length of the string */
  readattribute("Observing Beginning Time",group_id,strtype,Beginning_Time);
  readattribute("Observing Ending Time",group_id,strtype,Ending_Time);

  readattribute("Number Of Scans", group_id, H5T_NATIVE_INT, &Count_scnlines);
  readattribute("Orbit Number", group_id, H5T_NATIVE_INT, &Orbit_Number);
  readattribute("Orbit Period", group_id, H5T_NATIVE_INT, &Orbit_Period);
  readattribute("Ave.. Instru.Temp", group_id, H5T_NATIVE_FLOAT, Ave_Instr_Temp);

  readdataset("Earth_Obs_BT", group_id, H5T_NATIVE_FLOAT, Earth_Obs_BT);
  readdataset("Latitude", group_id, H5T_NATIVE_FLOAT, Latitude);
  readdataset("Longitude", group_id, H5T_NATIVE_FLOAT, Longitude);
  readdataset("Height", group_id, H5T_NATIVE_SHORT, Height);
  readdataset("LandSeaMask", group_id, H5T_NATIVE_SHORT, LandSeaMask);
  readdataset("SensorZenith", group_id, H5T_NATIVE_SHORT, SensorZenith);
  readdataset("SensorAzimuth", group_id, H5T_NATIVE_SHORT, SensorAzimuth);
  readdataset("SolarZenith", group_id, H5T_NATIVE_SHORT, SolarZenith);
  readdataset("SolarAzimuth", group_id, H5T_NATIVE_SHORT, SolarAzimuth);
  readdataset("Black_Body_View_Ang", group_id, H5T_NATIVE_SHORT, Black_Body_View_Ang);
  readdataset("cal_qc", group_id, H5T_NATIVE_SHORT, cal_qc);
  readdataset("Cal_Coefficient", group_id, H5T_NATIVE_INT, Cal_Coefficient);

  readdataset("EVS_orb_pos", group_id, H5T_NATIVE_FLOAT, EVS_orb_pos);

/* Position_Time seems to be the most accurate, but is not always available - so
   report "Scnlin_mscnt" instead */

/*  readdataset("Position_Time", group_id, H5T_NATIVE_INT, Scan_Time); */
  readdataset("Scnlin_mscnt", group_id, H5T_NATIVE_INT, Scan_Time);

  printf("Finished reading %s\n",filename);

/****** Clean up ******/

  H5Gclose(group_id);
  H5Fclose(file_id);
  return 0;

}
#endif
