      SUBROUTINE BYTESWAP1B(code)
!+
!NOM/NAME $Id: byteswap1b.F 427 2014-11-28 13:20:26Z frna $
!
!---------------------------------------------------------------------------
!BUT/PURPOSE
!     Byte swap level 1b data in order to read 1b data generated from an
!     external source. Level 1b datasets contain a mixture of 1, 2 and 4 byte
!     variables, held in common blocks, so need to define the swapping required
!     for each data type.
!
!     Examines the data type when reading in the header to determine whether
!     a swap is needed, and stores the result ready for the data records.
!
!DESCRIPTION
!     FORTRAN 77 with UKMO/METEO-FRANCE accepted extensions
!
!PROPRIETAIRE/OWNER
!     Nigel Atkinson, Met Office
!
!HISTORIQUE/HISTORY (MODIFICATIONS/CHANGES)
!(version  date   author         comment)
! 1.1     8/3/04  N C Atkinson   Initial version
! 1.2     2/4/04  N C Atkinson   Add MHS
! 1.3    27/7/04  T Labrot       Bug correction: 
!                                Bug: We never pass in HRS1Bhd part
! 1.4    19/10/04 N C Atkinson   Update to NESDIS 2004 formats
! 1.5    24/11/04 N C Atkinson   Remove second HIRS header - no longer needed.
!                                Byte swap AMSU-B header even if it's really MHS
! 1.6    13/04/05 N C Atkinson   Add AVHRR
! 1.7    28/02/12 N C Atkinson   HRS1BHD2 never used
! 1.8    30/08/13 N C Atkinson   Add MSU1BCL, HRS1BCL, LAC1BCL, GAC1BCL (TOVS CLASS data)
! 1.9    28/11/14 N C Atkinson   Change pos_lac1bc and pos_gac1bc to make "telem" 4-byte words
!ARGUMENTS
!Input:
!     code = "AMB1BHD","AMB1BDT","AMA1BHD","AMA1BDT","HRS1BHD", 
!             "HRS1BDT","MSU1BHD","MSU1BDT","MHS1BHD","MHS1BDT",
!             "AVH1BHD","AVH1BDT","MSU1BCL","HRS1BCL","LAC1BCL","GAC1BCL"
!
!FUNCTIONS AND SUBROUTINES CALLS: None
!
!#########################################################################
! COPYRIGHT        THE CONTENTS OF THIS FILE ARE CROWN COPYRIGHT. THE
!                  SOFTWARE, DATA AND OTHER PROPRIETARY INFORMATION
!                  CONTAINED THEREIN IS SUPPLIED FOR BONAFIDE RESEARCH
!                  PURPOSES. COMMERCIAL OR BUSINESS USE IS NOT PERMITTED
!                  WITHOUT THE WRITTEN AUTHORITY OF THE MET OFFICE.
!                  AT FIRZROY RD., EXETER, DEVON, EX1 3PB, U.K.
!#########################################################################
!
!--DECLARATIONS-------------------------------------------------------------

      IMPLICIT NONE
      
#include "amb1b.h"
#include "ama1b.h"
#include "hrs1b.h"
#include "msu1b.h"
#include "mhs1b.h"
#include "avh1b.h"
#include "msu1b_class.h"
#include "hrs1b_class.h"
#include "avh1b_lac_tovs.h"
#include "avh1b_gac_tovs.h"

      CHARACTER*7 code
      CHARACTER*1 ch
      INTEGER   pos1,pos2,I,J
      LOGICAL   byteswap
      DATA      byteswap/.FALSE./ 
      LOGICAL   byteswap_avh
      DATA      byteswap_avh/.FALSE./ 
      
C! The following sections give the byte positions where the number of bytes per
C! word changes (fillers don't matter). Also define character arrays and
C! equivalence them to the common blocks defined in *1b.h .
C!-------------------------------------------------------------------------
C! AMSU-B
C!-------------------------------------------------------------------------
      CHARACTER*1 amb_hdc(reclamb)
      CHARACTER*1 amb_dtc(reclamb)
      EQUIVALENCE (amb_hd(1),amb_hdc(1))
      EQUIVALENCE (amb_dt(1),amb_dtc(1))

      INTEGER*2 pos_amb1bhd(23)
      INTEGER*2 nbyte_amb1bhd(23)

      DATA pos_amb1bhd   /1,5,23,73,81,85,89,97,101,105,109,121,125,129,
     & 133,249,401,409,425,497,705,1001,2859/
      DATA nbyte_amb1bhd /1,2, 1, 2, 4, 2, 4, 2,  4,  2,  1,  4,  2,  4,
     &   2,  4,  1,  2,  4,  2,  4,   2,   1/     
      
      INTEGER*2 pos_amb1bdt(16)
      INTEGER*2 nbyte_amb1bdt(16)

      DATA pos_amb1bdt   /1,9,13,25,33,61,191,197,205,753,1473,2673,
     & 2681, 2745,2749,2805/
      DATA nbyte_amb1bdt /2,4, 2, 4, 2, 4,  2,  4,  2,  4,  2,  4,  2,
     &    4,   2,   1/      
      
C!-------------------------------------------------------------------------
C! AMSU-A
C!-------------------------------------------------------------------------
      CHARACTER*1 ama_hdc(reclama)
      CHARACTER*1 ama_dtc(reclama)
      EQUIVALENCE (ama_hd(1),ama_hdc(1))
      EQUIVALENCE (ama_dt(1),ama_dtc(1))

      INTEGER*2 pos_ama1bhd(25)
      INTEGER*2 nbyte_ama1bhd(25)

      DATA pos_ama1bhd   /1,5,23,73,81,85,89,97,101,105,121,125,129,
     & 137,141,145,421,881,889,905,2357,2369,2373,2377,2401/
      DATA nbyte_ama1bhd /1,2, 1, 2, 4, 2, 4, 2,  4,  2,  4,  2,  4,
     &   2,  4,  2,  4,  1,  2,  4,   2,   4,   2,   4,   1/     
      
      INTEGER*2 pos_ama1bdt(18)
      INTEGER*2 nbyte_ama1bdt(18)

      DATA pos_ama1bdt   /1,9,13,25,33,81,445,457,465,653,897,2149,2153,
     & 2185,2509,2513,2545,2557/
      DATA nbyte_ama1bdt /2,4, 2, 4, 2, 4,  2,  4,  2,  4,  2,   4,   1,
     &    2,   4,   1,   2,   1/      
      
C!-------------------------------------------------------------------------
C! HIRS
C!-------------------------------------------------------------------------
      CHARACTER*1 hrs_hdc(reclhrs)
      CHARACTER*1 hrs_dtc(reclhrs)
      EQUIVALENCE (hrs_hd(1),hrs_hdc(1))
      EQUIVALENCE (hrs_dt(1),hrs_dtc(1))

      INTEGER*2 pos_hrs1bhd(27)
      INTEGER*2 nbyte_hrs1bhd(27)

      DATA pos_hrs1bhd   /1,5,23,73,81,85,89,97,101,105,111,117,121,
     & 125,129,193,749,761,769,785,1885,1889,1969,1973,1977,1981,2005/
      DATA nbyte_hrs1bhd /1,2, 1, 2, 4, 2, 4, 2,  4,  2,  1,  4,  2,
     &   4,  2,  4,  2,  1,  2,  4,   2,   4,   2,   4,   2,   4,   1/     
      
      INTEGER*2 pos_hrs1bdt(14)
      INTEGER*2 nbyte_hrs1bdt(14)

      DATA pos_hrs1bdt   /1,9,13,29,37,77,157,637,649,657,1001,1449,
     &4545, 4549/
      DATA nbyte_hrs1bdt /2,4, 2, 4, 2, 1,  4,  2,  4,  2,  4,  2,  4,
     &    1/      
      
C!-------------------------------------------------------------------------
C! MHS
C!-------------------------------------------------------------------------
      CHARACTER*1 mhs_hdc(reclmhs)
      CHARACTER*1 mhs_dtc(reclmhs)
      EQUIVALENCE (mhs_hd(1),mhs_hdc(1))
      EQUIVALENCE (mhs_dt(1),mhs_dtc(1))

      INTEGER*2 pos_mhs1bhd(24)
      INTEGER*2 nbyte_mhs1bhd(24)

      DATA pos_mhs1bhd   /1,5,23,73,81,85,89,97,101,105,111,121,125,129,
     & 133,281,493,501,517,885,1757,1761,1765,1789/
      DATA nbyte_mhs1bhd /1,2, 1, 2, 4, 2, 4, 2,  4,  2,  1,  4,  2,  4,
     &   2,  4,  1,  2,  4,  2,   4,   2,   4,   1/     
      
      INTEGER*2 pos_mhs1bdt(21)
      INTEGER*2 nbyte_mhs1bdt(21)

      DATA pos_mhs1bdt   /1,9,13,17,21,23,25,33,61,185,197,205,753,1473,
     & 2673,2751,2769,2789,2841,2865,2869/
      DATA nbyte_mhs1bdt /2,4, 2, 4, 2, 1, 4, 2, 4,  2,  4,  2,  4,   2,
     &    1,   2,   4,   1,   2,   4,   1/      
      
C!-------------------------------------------------------------------------
C! AVHRR
C!-------------------------------------------------------------------------
      CHARACTER*1 avh_hdc(reclavh)
      CHARACTER*1 avh_dtc(reclavh)
      EQUIVALENCE (avh_hd(1),avh_hdc(1))
      EQUIVALENCE (avh_dt(1),avh_dtc(1))

      INTEGER*2 pos_avh1bhd(20)
      INTEGER*2 nbyte_avh1bhd(20)

      DATA pos_avh1bhd   /1,5,23,73,81,85,89,97,101,105,117,121,125,129,
     & 257,329,337,353,425,689/
      DATA nbyte_avh1bhd /1,2, 1, 2, 4, 2, 4, 2,  4,  2,  4,  2,  4,  2,
     &   4,  1,  2,  4,  2,  1/     
      
      INTEGER*2 pos_avh1bdt(10)
      INTEGER*2 nbyte_avh1bdt(10)

      DATA pos_avh1bdt   /1,9,13,25,33,49,321,641,1057,21883/
      DATA nbyte_avh1bdt /2,4, 2, 4, 2, 4,  2,  4,   2,    1/      
      
C!-------------------------------------------------------------------------
C! MSU CLASS
C!-------------------------------------------------------------------------
      CHARACTER*1 msu1bc(400)
      EQUIVALENCE (msu1bc(1),msu_c_char)

      INTEGER*2 pos_msu1bc(4)
      INTEGER*2 nbyte_msu1bc(4)

      DATA pos_msu1bc   /1, 5, 113, 385/
      DATA nbyte_msu1bc /2, 4,   2,   1/     
           
C!-------------------------------------------------------------------------
C! HIRS/2 CLASS
C!-------------------------------------------------------------------------
      CHARACTER*1 hrs1bc(3844)
      EQUIVALENCE (hrs1bc(1),hrs_c_char)

      INTEGER*2 pos_hrs1bc(4)
      INTEGER*2 nbyte_hrs1bc(4)

      DATA pos_hrs1bc   /1, 5, 737, 3781/
      DATA nbyte_hrs1bc /2, 4,   2,    1/     
           
C!-------------------------------------------------------------------------
C! LAC CLASS (TOVS)
C!-------------------------------------------------------------------------
      CHARACTER*1 lac1bc(14800)
      EQUIVALENCE (lac1bc(1),avhlac_c_char)

      INTEGER*2 pos_lac1bc(8)
      INTEGER*2 nbyte_lac1bc(8)

!      DATA pos_lac1bc   /1, 5, 53, 105, 449, 14105, 14125, 14127/
      DATA pos_lac1bc   /1, 5, 53, 105, 309, 14105, 14125, 14127/
      DATA nbyte_lac1bc /2, 4,  1,   2,   4,     1,     2,     1/     
           
C!-------------------------------------------------------------------------
C! GAC CLASS (TOVS)
C!-------------------------------------------------------------------------
      CHARACTER*1 gac1bc(3220)
      EQUIVALENCE (gac1bc(1),avhgac_c_char)

      INTEGER*2 pos_gac1bc(8)
      INTEGER*2 nbyte_gac1bc(8)

!      DATA pos_gac1bc   /1, 5, 53, 105, 449, 3177, 3197, 3199/
      DATA pos_gac1bc   /1, 5, 53, 105, 309, 3177, 3197, 3199/
      DATA nbyte_gac1bc /2, 4,  1,   2,   4,    1,    2,    1/     
           
!--END OF DECLARATIONS------------------------------------------------------

      IF (CODE(1:7) .EQ. "AMB1BHD") THEN

!Note: If we have read the header in as AMSU-B but it's really MHS, do the 
!byte swapping anyway as for AMSU-B. The calling program should then check the
!instrument ID

        IF (amb_h_datatyp .EQ. 2816 .OR.     !Hex 0B00 =AMSU-B
     &      amb_h_datatyp .EQ. 3072) THEN    !Hex 0C00 =MHS
          byteswap = .TRUE.
          DO I=1,22
            pos1 = pos_amb1bhd(I)
            pos2 = pos_amb1bhd(I+1)-1
            IF (nbyte_amb1bhd(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = amb_hdc(J)
                amb_hdc(J) = amb_hdc(J+1)
                amb_hdc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_amb1bhd(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = amb_hdc(J)
                amb_hdc(J) = amb_hdc(J+3)
                amb_hdc(J+3) = ch
                ch = amb_hdc(J+1)
                amb_hdc(J+1) = amb_hdc(J+2)
                amb_hdc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO  
        ELSE
          byteswap = .FALSE.
        ENDIF

      ELSE IF (CODE(1:7) .EQ. "AMB1BDT") THEN

        IF (byteswap) THEN
          DO I=1,15
            pos1 = pos_amb1bdt(I)
            pos2 = pos_amb1bdt(I+1)-1
            IF (nbyte_amb1bdt(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = amb_dtc(J)
                amb_dtc(J) = amb_dtc(J+1)
                amb_dtc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_amb1bdt(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = amb_dtc(J)
                amb_dtc(J) = amb_dtc(J+3)
                amb_dtc(J+3) = ch
                ch = amb_dtc(J+1)
                amb_dtc(J+1) = amb_dtc(J+2)
                amb_dtc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ENDIF
        
      ELSE IF (CODE(1:7) .EQ. "AMA1BHD") THEN

        IF (ama_h_datatyp .EQ. 2560) THEN   !Hex 0A00
          byteswap = .TRUE.
          DO I=1,24
            pos1 = pos_ama1bhd(I)
            pos2 = pos_ama1bhd(I+1)-1
            IF (nbyte_ama1bhd(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = ama_hdc(J)
                ama_hdc(J) = ama_hdc(J+1)
                ama_hdc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_ama1bhd(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = ama_hdc(J)
                ama_hdc(J) = ama_hdc(J+3)
                ama_hdc(J+3) = ch
                ch = ama_hdc(J+1)
                ama_hdc(J+1) = ama_hdc(J+2)
                ama_hdc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ELSE
          byteswap = .FALSE.
        ENDIF

      ELSE IF (CODE(1:7) .EQ. "AMA1BDT") THEN

        IF (byteswap) THEN
          DO I=1,17
            pos1 = pos_ama1bdt(I)
            pos2 = pos_ama1bdt(I+1)-1
            IF (nbyte_ama1bdt(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = ama_dtc(J)
                ama_dtc(J) = ama_dtc(J+1)
                ama_dtc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_ama1bdt(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = ama_dtc(J)
                ama_dtc(J) = ama_dtc(J+3)
                ama_dtc(J+3) = ch
                ch = ama_dtc(J+1)
                ama_dtc(J+1) = ama_dtc(J+2)
                ama_dtc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ENDIF
        
!      ELSE IF (CODE(1:7) .EQ. "HRS1BHD" .AND. 
!     &        CODE(1:8) .NE. "HRS1BHD2" ) THEN
      ELSE IF (CODE(1:7) .EQ. "HRS1BHD") THEN

        IF (hrs_h_datatyp .EQ. 1280) THEN   !Hex 0500
          byteswap = .TRUE.
          DO I=1,26
            pos1 = pos_hrs1bhd(I)
            pos2 = pos_hrs1bhd(I+1)-1
            IF (nbyte_hrs1bhd(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = hrs_hdc(J)
                hrs_hdc(J) = hrs_hdc(J+1)
                hrs_hdc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_hrs1bhd(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = hrs_hdc(J)
                hrs_hdc(J) = hrs_hdc(J+3)
                hrs_hdc(J+3) = ch
                ch = hrs_hdc(J+1)
                hrs_hdc(J+1) = hrs_hdc(J+2)
                hrs_hdc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ELSE
          byteswap = .FALSE.
        ENDIF

      ELSE IF (CODE(1:7) .EQ. "HRS1BDT") THEN

        IF (byteswap) THEN
          DO I=1,13
            pos1 = pos_hrs1bdt(I)
            pos2 = pos_hrs1bdt(I+1)-1
            IF (nbyte_hrs1bdt(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = hrs_dtc(J)
                hrs_dtc(J) = hrs_dtc(J+1)
                hrs_dtc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_hrs1bdt(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = hrs_dtc(J)
                hrs_dtc(J) = hrs_dtc(J+3)
                hrs_dtc(J+3) = ch
                ch = hrs_dtc(J+1)
                hrs_dtc(J+1) = hrs_dtc(J+2)
                hrs_dtc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ENDIF
        
        
      ELSE IF (CODE(1:7) .EQ. "MSU1BHD") THEN

        IF (msu_h_datatyp .EQ. 1536) THEN   !Hex 0600
          byteswap = .TRUE.
          WRITE(*,*) 'byteswap1b: MSU not implemented'
          STOP
        ELSE
          byteswap = .FALSE.
        ENDIF

      ELSE IF (CODE(1:7) .EQ. "MSU1BDT") THEN
        IF (byteswap) THEN
          WRITE(*,*) 'byteswap1b: MSU not implemented'
          STOP
        ENDIF

      ELSE IF (CODE(1:7) .EQ. "MHS1BHD") THEN

        IF (mhs_h_datatyp .EQ. 3072) THEN   !Hex 0C00
          byteswap = .TRUE.
          DO I=1,23
            pos1 = pos_mhs1bhd(I)
            pos2 = pos_mhs1bhd(I+1)-1
            IF (nbyte_mhs1bhd(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = mhs_hdc(J)
                mhs_hdc(J) = mhs_hdc(J+1)
                mhs_hdc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_mhs1bhd(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = mhs_hdc(J)
                mhs_hdc(J) = mhs_hdc(J+3)
                mhs_hdc(J+3) = ch
                ch = mhs_hdc(J+1)
                mhs_hdc(J+1) = mhs_hdc(J+2)
                mhs_hdc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ELSE
          byteswap = .FALSE.
        ENDIF

      ELSE IF (CODE(1:7) .EQ. "MHS1BDT") THEN

        IF (byteswap) THEN
          DO I=1,20
            pos1 = pos_mhs1bdt(I)
            pos2 = pos_mhs1bdt(I+1)-1
            IF (nbyte_mhs1bdt(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = mhs_dtc(J)
                mhs_dtc(J) = mhs_dtc(J+1)
                mhs_dtc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_mhs1bdt(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = mhs_dtc(J)
                mhs_dtc(J) = mhs_dtc(J+3)
                mhs_dtc(J+3) = ch
                ch = mhs_dtc(J+1)
                mhs_dtc(J+1) = mhs_dtc(J+2)
                mhs_dtc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ENDIF
        
      ELSE IF (CODE(1:7) .EQ. "AVH1BHD") THEN

        IF (avh_h_datatyp .EQ. 768 .OR.      !Hex 0300 = HRPT
     &      avh_h_datatyp .EQ. 256 .OR.      !Hex 0100 = LAC
     &      avh_h_datatyp .EQ. 512 .OR.      !Hex 0200 = GAC
     &      avh_h_datatyp .EQ. 3328) THEN    !Hex 0D00 = FRAC
          byteswap_avh = .TRUE.
          DO I=1,19
            pos1 = pos_avh1bhd(I)
            pos2 = pos_avh1bhd(I+1)-1
            IF (nbyte_avh1bhd(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = avh_hdc(J)
                avh_hdc(J) = avh_hdc(J+1)
                avh_hdc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_avh1bhd(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = avh_hdc(J)
                avh_hdc(J) = avh_hdc(J+3)
                avh_hdc(J+3) = ch
                ch = avh_hdc(J+1)
                avh_hdc(J+1) = avh_hdc(J+2)
                avh_hdc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ELSE
          byteswap_avh = .FALSE.
        ENDIF

      ELSE IF (CODE(1:7) .EQ. "AVH1BDT") THEN

        IF (byteswap_avh) THEN
          DO I=1,9
            pos1 = pos_avh1bdt(I)
            pos2 = pos_avh1bdt(I+1)-1
            IF (nbyte_avh1bdt(I) .EQ. 2) THEN
              DO J=pos1,pos2,2
                ch = avh_dtc(J)
                avh_dtc(J) = avh_dtc(J+1)
                avh_dtc(J+1) = ch
              ENDDO
            ELSE IF (nbyte_avh1bdt(I) .EQ. 4) THEN 
              DO J=pos1,pos2,4
                ch = avh_dtc(J)
                avh_dtc(J) = avh_dtc(J+3)
                avh_dtc(J+3) = ch
                ch = avh_dtc(J+1)
                avh_dtc(J+1) = avh_dtc(J+2)
                avh_dtc(J+2) = ch
              ENDDO
            ENDIF
          ENDDO
        ENDIF
        
      ELSE IF (CODE(1:7) .EQ. "MSU1BCL") THEN

!For MSU CLASS, always swap if the routine is called

        DO I=1,3
          pos1 = pos_msu1bc(I)
          pos2 = pos_msu1bc(I+1)-1
          IF (nbyte_msu1bc(I) .EQ. 2) THEN
            DO J=pos1,pos2,2
              ch = msu1bc(J)
              msu1bc(J) = msu1bc(J+1)
              msu1bc(J+1) = ch
            ENDDO
          ELSE IF (nbyte_msu1bc(I) .EQ. 4) THEN 
            DO J=pos1,pos2,4
              ch = msu1bc(J)
              msu1bc(J) = msu1bc(J+3)
              msu1bc(J+3) = ch
              ch = msu1bc(J+1)
              msu1bc(J+1) = msu1bc(J+2)
              msu1bc(J+2) = ch
            ENDDO
          ENDIF
        ENDDO
        
      ELSE IF (CODE(1:7) .EQ. "HRS1BCL") THEN

!For MSU CLASS, always swap if the routine is called

        DO I=1,3
          pos1 = pos_hrs1bc(I)
          pos2 = pos_hrs1bc(I+1)-1
          IF (nbyte_hrs1bc(I) .EQ. 2) THEN
            DO J=pos1,pos2,2
              ch = hrs1bc(J)
              hrs1bc(J) = hrs1bc(J+1)
              hrs1bc(J+1) = ch
            ENDDO
          ELSE IF (nbyte_hrs1bc(I) .EQ. 4) THEN 
            DO J=pos1,pos2,4
              ch = hrs1bc(J)
              hrs1bc(J) = hrs1bc(J+3)
              hrs1bc(J+3) = ch
              ch = hrs1bc(J+1)
              hrs1bc(J+1) = hrs1bc(J+2)
              hrs1bc(J+2) = ch
            ENDDO
          ENDIF
        ENDDO
        
      ELSE IF (CODE(1:7) .EQ. "LAC1BCL") THEN

!For LAC from CLASS, always swap if the routine is called

        DO I=1,7
          pos1 = pos_lac1bc(I)
          pos2 = pos_lac1bc(I+1)-1
          IF (nbyte_lac1bc(I) .EQ. 2) THEN
            DO J=pos1,pos2,2
              ch = lac1bc(J)
              lac1bc(J) = lac1bc(J+1)
              lac1bc(J+1) = ch
            ENDDO
          ELSE IF (nbyte_lac1bc(I) .EQ. 4) THEN 
            DO J=pos1,pos2,4
              ch = lac1bc(J)
              lac1bc(J) = lac1bc(J+3)
              lac1bc(J+3) = ch
              ch = lac1bc(J+1)
              lac1bc(J+1) = lac1bc(J+2)
              lac1bc(J+2) = ch
            ENDDO
          ENDIF
        ENDDO
        
      ELSE IF (CODE(1:7) .EQ. "GAC1BCL") THEN

!For GAC from CLASS, always swap if the routine is called

        DO I=1,7
          pos1 = pos_gac1bc(I)
          pos2 = pos_gac1bc(I+1)-1
          IF (nbyte_gac1bc(I) .EQ. 2) THEN
            DO J=pos1,pos2,2
              ch = gac1bc(J)
              gac1bc(J) = gac1bc(J+1)
              gac1bc(J+1) = ch
            ENDDO
          ELSE IF (nbyte_gac1bc(I) .EQ. 4) THEN 
            DO J=pos1,pos2,4
              ch = gac1bc(J)
              gac1bc(J) = gac1bc(J+3)
              gac1bc(J+3) = ch
              ch = gac1bc(J+1)
              gac1bc(J+1) = gac1bc(J+2)
              gac1bc(J+2) = ch
            ENDDO
          ENDIF
        ENDDO
        
      ELSE
        WRITE(*,*) 'byteswap1b: Code ',CODE,' not recognised'
        STOP
      ENDIF
      RETURN
      END
