! LIBS = 

PROGRAM cris_bufr_add_clusters

!-----------------------------------------------------------------------
!
! Name: $Id: cris_bufr_add_clusters.F90 616 2019-01-29 14:54:55Z frna $
!
! Purpose: Append VIIRS cluster analysis to a level 1c "CRISFSR" BUFR file
!          created using eccodes_encodebufr_1c. The BUFR file must use the
!          FSR BUFR sequence, but does not have to be FSR data.
!
! For details of how to generate the input cluster and BUFR files, see the
! script VIIRS_CRIS_MAIA4_RUN
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2019, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     29/01/2019 Initial
!
! Code Description:
!   Fortran90
!
! Declarations:

#ifdef HAS_LIBECCODES
#ifdef HAS_LIBHDF5

  USE HDF5
  USE eccodes
  implicit none

  character(len=200) :: bufrfile_in, clusterfile, bufrfile_out, fdffile, fdfdir
  character(len=4)   :: cscan
  character(len=2)   :: cfor
  character(len=3)   :: satname
  character(len=17)  :: groupname
  character(len=10)  :: formatstr(2447)    !Accommodate 2223 channels + (7 classes x 16 channels x 2)
  character(len=80)  :: descriptor
  character(len=80)  :: inputline
  integer            :: i, j, k, iret, ifile, ofile, ibufr, obufr, narg, hdferr
  integer            :: nsubsets, iscan, ifor, ifov, iclass, ispot, ichan
  integer            :: nchan_cris, nchan_extra, satid, ntotal
  logical            :: link_exists
  integer,      dimension(7,9)       :: RadAnaNbPix_for  !for one FOR
  integer,      dimension(7,9,30)    :: RadAnaNbPix      !for all FORs
  real,         dimension(16,7,9)    :: RadAnaMean_for, RadAnaStd_for  !for one FOR
  real(kind=8), dimension(16,7,9,30) :: RadAnaMean, RadAnaStd          !for all FORs
  real(kind=8), dimension(270) :: rvalues, rvalues1, rvalues2  !assume 270 CrIS spots per message
  real(kind=8), dimension(4)   :: srfinfo             !cwn eqw_wn cwl eqw_wl for one channel
  real(kind=8), dimension(16)  :: conversion          !Conversion from W/(m^2 sr m^-1) to W/(m^2 sr um) for 16 channels

  INTEGER(HID_T) :: file_id       ! File identifier
  INTEGER(HID_T) :: dset_id       ! Dataset identifier
  INTEGER(HID_T) :: attr_id       ! Attribute identifier
  INTEGER(HID_T) :: grp_id        ! Group identifier
  INTEGER(HSIZE_T) :: dims(3)

!-------End of declarations-----------------------------------------------------------

  narg = command_argument_count()
  if (narg < 3) then
    write(*,*)'Usage: cris_bufr_add_clusters.exe bufrfile_in clusterfile bufrfile_out'
    stop
  endif

  call get_command_argument(1,bufrfile_in)
  call get_command_argument(2,clusterfile)
  call get_command_argument(3,bufrfile_out)

  do i=1,9
    write(formatstr(i),'(A)') '(A,I1,A)'
  enddo
  do i=10,99
    write(formatstr(i),'(A)') '(A,I2,A)'
  enddo
  do i=100,999
    write(formatstr(i),'(A)') '(A,I3,A)'
  enddo
  do i=1000,2447
    write(formatstr(i),'(A)') '(A,I4,A)'
  enddo

!Open the input clusters file

  CALL h5open_f(hdferr)
  write(*,*)'open (read): ',trim(clusterfile)
  CALL h5fopen_f (clusterfile, H5F_ACC_RDWR_F, file_id, hdferr)
  if (hdferr /= 0) then
    write(*,*)'Error opening ',clusterfile
    stop
  endif
  dims(1) = 16   !channels
  dims(2) = 7    !clusters
  dims(3) = 9    !fovs

!Read the input BUFR file and clone each message

  write(*,*) 'open (read): ',trim(bufrfile_in)
  call codes_open_file(ifile, bufrfile_in, 'r')
  write(*,*)'open (write): ',trim(bufrfile_out)
  call codes_open_file(ofile, bufrfile_out, 'w')
  call codes_bufr_new_from_file(ifile,ibufr,iret)   !first message

  satid = 0
  satname = 'XXX'

  do while (iret /= CODES_END_OF_FILE)
    call codes_clone(ibufr, obufr)
    call codes_set(obufr,'skipExtraKeyAttributes', 1, iret)
    call codes_set(obufr,'unpack', 1)

!Get the satellite ID and read the viirs to cris fdf file
    if (satid == 0) then
      call codes_get(obufr,'satelliteIdentifier',satid)
      if (satid == 224) satname = 'NPP'
      if (satid == 225) satname = 'J01'
      if (satid == 226) satname = 'J02'
      call getenv('DIR_PREPROC',fdfdir)
      fdffile = trim(fdfdir) // '/mapping_viirs_cris.fdf'
      write(*,*) 'get bandwidth data for satellite ',satname,' from ',trim(fdffile)
      open(10,file=fdffile,status='old')
      inputline='   '
      do while (inputline(1:3) /= satname)
        read(10,'(A)',iostat=iret) inputline
        if (iret /= 0) then
          write(*,*) 'Error: cannot find ',satname,' in ',fdffile
          STOP
        endif
      enddo
      read(10,'(A)') inputline
      do ichan=1,16
        read(10,'(A)') inputline
        read(inputline(4:80),*) srfinfo
        conversion(ichan) = 100*srfinfo(2)/srfinfo(4)
                  !Convert back to W/(m^2 sr um) using the inverse of formula in map_viirs_cris_mod.F90, which is:
                  !VIIRSSRF(I)%RadtoSIfactor = 0.00001 * 1000. * VIIRSSRF(I)%EQW_WL / VIIRSSRF(I)%EQW_WN
      enddo
      close(10)
    endif
    write(*,'(A1)',advance="no") '.'

!Get the scan line number
    call codes_get(obufr,'numberOfSubsets',nsubsets)
    if (nsubsets == 270) then     !this code only works for 1 scan per message
      call codes_get(obufr,'extendedDelayedDescriptorReplicationFactor',nchan_cris)
      call codes_get(obufr,'delayedDescriptorReplicationFactor',nchan_extra)
      call codes_get(obufr,'scanLineNumber',iscan)
      write(cscan,'(I4.4)') iscan

!For each FOR, see if the corresponding group exists. If yes, read the VIIRS mean and stdev.
      RadAnaMean = CODES_MISSING_DOUBLE
      RadAnaStd = CODES_MISSING_DOUBLE
      RadAnaNbPix = 0
      do ifor=1,30
        write(cfor,'(I2.2)') ifor
        groupname = 'cluster-M-' // cscan // '-' // cfor
        call h5lexists_f(file_id, groupname, link_exists, hdferr)
        if (link_exists) then
          call h5gopen_f(file_id, groupname, grp_id, hdferr)
          call h5dopen_f(grp_id, 'RadAnaMean', dset_id, hdferr)
          call h5dread_f(dset_id, H5T_NATIVE_REAL, RadAnaMean_for, dims, hdferr)
          RadAnaMean(:,:,:,ifor) = RadAnaMean_for
          call h5dclose_f(dset_id, hdferr)
          call h5dopen_f(grp_id, 'RadAnaStd', dset_id, hdferr)
          call h5dread_f(dset_id, H5T_NATIVE_REAL, RadAnaStd_for, dims, hdferr)
          RadAnaStd(:,:,:,ifor) = RadAnaStd_for
          call h5dclose_f(dset_id, hdferr)
          call h5dopen_f(grp_id, 'RadAnaNbPix', dset_id, hdferr)
          call h5dread_f(dset_id, H5T_NATIVE_INTEGER, RadAnaNbPix_for, dims, hdferr)
          RadAnaNbPix(:,:,ifor) = RadAnaNbPix_for
          call h5dclose_f(dset_id, hdferr)
          call h5gclose_f(grp_id, hdferr)
        endif
      enddo

!Store the data in the BUFR fields

      i = nchan_cris + nchan_extra    !used for radiance
      j = 0                           !used for first order statistics
      k = nchan_cris + nchan_extra    !used for channel number

      do iclass=1,7
        do ichan=1,16
          call codes_set(obufr,descriptor,rvalues2,iret)
          ispot = 0
          do ifor=1,30
            do ifov=1,9
              ispot = ispot + 1
              if (RadAnaMean(ichan,iclass,ifov,ifor) /= CODES_MISSING_DOUBLE) then
                rvalues1(ispot) = RadAnaMean(ichan,iclass,ifov,ifor)*conversion(ichan)
                rvalues2(ispot) = RadAnaStd(ichan,iclass,ifov,ifor)*conversion(ichan)
              else
                rvalues1(ispot) = CODES_MISSING_DOUBLE
                rvalues2(ispot) = CODES_MISSING_DOUBLE
              endif
              if (ichan == 1) then
                ntotal = SUM(RadAnaNbPix(:,ifov,ifor))
                if (ntotal > 0) then
                  rvalues(ispot) = RadAnaNbPix(iclass,ifov,ifor)*100.0/ntotal
                else
                  rvalues(ispot) = CODES_MISSING_DOUBLE
                endif
              endif
            enddo
          enddo
          if (ichan == 1) then
            write(descriptor,formatstr(iclass)) '#',iclass,'#fractionOfClearPixelsInHirsFov'
            call codes_set(obufr,descriptor,rvalues,iret)
          endif
          k = k + 1
          write(descriptor,formatstr(k)) '#',k,'#channelNumber'
          call codes_set(obufr,descriptor,ichan,iret)
          j = j + 1
          write(descriptor,formatstr(j)) '#',j,'#firstOrderStatistics'
          call codes_set(obufr,descriptor,4,iret)    !4=mean
          i = i + 1 
          write(descriptor,formatstr(i)) '#',i,'#channelRadiance'
          call codes_set(obufr,descriptor,rvalues1,iret)
          j = j + 1
          write(descriptor,formatstr(j)) '#',j,'#firstOrderStatistics'
          call codes_set(obufr,descriptor,10,iret)   !10=stdev
          i = i + 1 
          write(descriptor,formatstr(i)) '#',i,'#channelRadiance'
          call codes_set(obufr,descriptor,rvalues2,iret)
        enddo
      enddo
    else
      write(*,*) 'Unexpected number of subsets: ',nsubsets
    endif

!Write output message

    call codes_set(obufr,'pack',1)
    call codes_write(obufr, ofile)
    call codes_release(obufr)
    call codes_release(ibufr)
    call codes_bufr_new_from_file(ifile,ibufr,iret)   !next message
  enddo

  call codes_close_file(ifile)
  call codes_close_file(ofile)
  CALL h5fclose_f(file_id, hdferr)
  CALL h5close_f(hdferr)

  write(*,*)
  write(*,*)'Finished writing ',trim(bufrfile_out)
#else
  write(*,*) 'cris_bufr_add_clusters.exe has not been built with hdf5'
#endif
#else
  write(*,*) 'cris_bufr_add_clusters.exe has not been built with eccodes'
#endif

END PROGRAM cris_bufr_add_clusters
