C LIBS = ecbufr atov f7nl1b f7tp auxaapp atovin atov
!+ Decode AAPP level 1c BUFR into AAPP format using ecCodes

      program eccodes_decodebufr_1c

#ifdef HAS_LIBECCODES
      use eccodes
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_decodebufr_1c.F 594 2018-11-01 12:46:39Z frna $
!
! Purpose: BUFR decoder for l1c BUFR, using ecCodes library
!
!    See https://software.ecmwf.int/wiki/display/ECC/ecCodes+Home
!
!    Inputs:
!      - $BUFRDEC_FILE defaults to 'infile.bufr'
!      - FDF.dat on ${FORT}.20
!    Outputs:
!      - l1c file on ${FORT}.10
!
!    Assumptions:
!      - Data are in time order
!      - All spots are present (no thinning)
!      - Spot 1 marks the start of a new scan. 
!    Normally a BUFR message can be expected to contain a whole number of scans,
!    but the code does not rely on this.
! 
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     24/01/2017 Initial
! 1.1     01/11/2018 Set skipExtraKeyAttributes for improved speed with
!                    ecCodes v2.9.0 onwards. No effect with earlier versions.
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines called:
!   codes_open_file, codes_set, codes_get, codes_release,
!   codes_bufr_new_from_file, codes_close_file
! AAPP subroutines called:
!   INFDF, AAPPEXIT
!
! Declarations:

#include "inparms.h"

  !Parameters and fixed items
      integer fdfunit
      DATA fdfunit/20/      !fdf file on fort.20
      integer out1cunit
      DATA out1cunit/10/     !output 1c file on fort.10
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      CHARACTER*7   CINSTR        !Instrument name e.g. AMSU-A
      CHARACTER*7   INSTRUMENT    !Instrument requested, if input file contains
                                  !more than one instrument
      CHARACTER*200 BUFRDEC_FILE

  !Loop variables, etc. 
      integer ibufr, iret, i
      integer imessage
      logical nedt_present
      integer ifile

  !Allocatable
      integer, dimension(:), allocatable :: descriptors

!-----------------------------------------------------------------------

!First read the general satellite info from the fdf.dat file on unit 20

      CALL INFDF (fdfunit, 0)

!Get environment variables if provided, otherwise set defaults

      CALL GETENV('BUFRDEC_FILE',BUFRDEC_FILE)
      if (BUFRDEC_FILE .eq. '') BUFRDEC_FILE='infile.bufr'
      CALL GETENV('INSTRUMENT',INSTRUMENT)
      IF (INSTRUMENT(1:1).NE.' ') 
     &  write(*,*)'Requested instrument: ',INSTRUMENT
      IF (INSTRUMENT.EQ.'MHS') INSTRUMENT = 'AMSU-B'
      write(*,*)'Input file: ',BUFRDEC_FILE(1:LEN_TRIM(BUFRDEC_FILE))

!Open the input file

      call codes_open_file(ifile,BUFRDEC_FILE,'r')
      call codes_bufr_new_from_file(ifile,ibufr,iret)
      imessage = 0

!Main loop over messages

      do while (iret .NE. CODES_END_OF_FILE)

  !unpack the data values
        call codes_set(ibufr,"skipExtraKeyAttributes",1,iret)
        call codes_set(ibufr,"unpack",1)

  !Get the descriptors
        call codes_get(ibufr,'unexpandedDescriptors',descriptors)
        do i=1,size(descriptors)
          if (descriptors(i) .eq. 222000) then   !Quality information follows
            nedt_present = .true.
            if (imessage .eq. 0) write(*,*)'NEDT values are present'
          endif
        enddo

  !Determine the instrument
        if (INSTRUMENT(1:1).NE.' ') then
          CINSTR=INSTRUMENT               !User has fixed the instrument
        else if (descriptors(1).EQ.310008) then
          CINSTR='HIRS'
        else if (descriptors(1) .eq. 310009) then
          CINSTR='AMSU-A'
        else if (descriptors(1) .eq. 310010) then
          CINSTR='AMSU-B'     !AMSU-B or MHS
        else if (descriptors(1) .eq. 340007) then
          CINSTR='IASI'
        else if (descriptors(1) .eq. 340008) then
          CINSTR='PCIASI'
        else if (descriptors(1) .eq. 310061) then
          CINSTR='ATMS'
        else if (descriptors(1) .eq. 310060) then
          CINSTR='CRIS'
        endif        
        if (imessage .eq. 0 .and. INSTRUMENT(1:1).EQ.' ') 
     &    write(*,*)'Instrument: ',CINSTR

  !Read the data for the requested instrument

        SELECT CASE (CINSTR)
          CASE ('HIRS') 
            call eccodes_get_1c_hirs(ibufr,out1cunit,imessage,
     &       nedt_present)
          CASE ('AMSU-A') 
            call eccodes_get_1c_amsua(ibufr,out1cunit,imessage,
     &       nedt_present)
          CASE ('AMSU-B') 
            call eccodes_get_1c_amsub(ibufr,out1cunit,imessage,
     &       nedt_present)
          CASE ('IASI') 
            call eccodes_get_1c_iasi(ibufr,out1cunit,imessage)
          CASE ('PCIASI') 
            call eccodes_get_1c_pciasi(ibufr,out1cunit,imessage)
          CASE ('MWHS') 
            call eccodes_get_1c_mwhs(ibufr,out1cunit,imessage)
          CASE ('MWHS2') 
            call eccodes_get_1c_mwhs2(ibufr,out1cunit,imessage)
          CASE ('MWTS') 
            call eccodes_get_1c_mwts(ibufr,out1cunit,imessage)
          CASE ('MWTS2') 
            call eccodes_get_1c_mwts2(ibufr,out1cunit,imessage)
          CASE ('IRAS') 
            call eccodes_get_1c_iras(ibufr,out1cunit,imessage)
          CASE ('MWRI') 
            call eccodes_get_1c_mwri(ibufr,out1cunit,imessage)
          CASE ('ATMS') 
            call eccodes_get_1c_atms(ibufr,out1cunit,imessage)
          CASE ('CRIS') 
            call eccodes_get_1c_cris(ibufr,out1cunit,imessage)
          CASE ('CRISFSR') 
            call eccodes_get_1c_crisfsr(ibufr,out1cunit,imessage)
        CASE DEFAULT
          write(*,*)'Invalid instrument: ',CINSTR
          call aappexit(1)
        END SELECT

  !Release the bufr message
        call codes_release(ibufr)

  !Load the next bufr message
        call codes_bufr_new_from_file(ifile,ibufr,iret)

      enddo   !iret .NE. CODES_END_OF_FILE

  !Close the output files
      call codes_close_file(ifile)
      if (imessage .gt. 0) then
        close(out1cunit)
      else
        write(*,*)'Error: no data found for ',CINSTR
      endif
#else
      write(*,*)
     &  'eccodes_decodebufr_1c has not been built with eccodes enabled'
      write(*,*)'Please re-run "configure", with appropriate values for'
      write(*,*)' --external-libs and --external-includes'
      write(*,*)'You need the libraries -leccodes -leccodes_f90'
      write(*,*)'Then re-run "make"'
      write(*,*)'If eccodes is not centrally-installed, you may need'
      write(*,*)' to set LD_LIBRARY_PATH'
#endif
      end program eccodes_decodebufr_1c
