C LIBS =  ecbufr atov atovpp f7nl1b f7tp auxaapp
!+ Encode AAPP level 1c data into BUFR using ecCodes

      program eccodes_encodebufr_1c

#ifdef HAS_LIBECCODES
      use eccodes
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_encodebufr_1c.F 619 2019-02-12 12:49:41Z frna $
!
! Purpose: BUFR encoder for l1c BUFR, using ecCodes library
!
!    See https://software.ecmwf.int/wiki/display/ECC/ecCodes+Home
!
!    Inputs:
!      - l1c file on ${FORT}.10
!    Outputs:
!      - $BUFRENC_FILE defaults to 'outfile.bufr'
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 29 June 2011, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     24/01/2017 Initial
! 1.1     01/05/2018 Default dataSubCategory 205 for CRISFSR
! 1.2     12/02/2019 Add HIRAS
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines called:
!   codes_open_file, codes_set, codes_get, codes_release,
!   codes_bufr_new_from_file, codes_close_file
! AAPP subroutines called:
!   AAPPEXIT
!
! Declarations:

  !Parameters and fixed items
      integer in1cunit
      DATA in1cunit/10/     !input on fort.10

  !Loop variables, etc. 
      integer ibufr, iret
      integer i, imessage, irec, iscan, isub, ispot, ichan, jchan
      integer outfile
      integer number_of_messages
      integer nsubsets

  !To hold optional environment variables
      INTEGER ICENTREID        !Centre code (1c in Section 4)
      INTEGER ISUBCENTREID     !Centre code (1c in Section 4)
      INTEGER ICENTREID_1B     !Centre code (1b in Section 4)
      INTEGER ISUBCENTREID_1B  !Centre code (1b in Section 4)
      INTEGER IORIGCENTRE      !Centre code (Section 1)
      INTEGER ISUBCENTRE       !sub-centre code (Section 1)
      INTEGER IMASTER          !Version of master table
      INTEGER ILOCAL           !Version of local table
      INTEGER IMESSAGE_SUBTYPE !Message subtype (locally defined)
      INTEGER AAPP_VERSION     !e.g. 715 for v7.15

      CHARACTER*256 BUFRENC_FILE        !Name of BUFR file to create
      CHARACTER*5   ORIGINATING_CENTRE  !Orig centre for section 1
      CHARACTER*5   SUB_CENTRE          !Sub-centre for section 1
      CHARACTER*3   MASTER_TABLE        !Master table version number
      CHARACTER*3   LOCAL_TABLE         !Local table version number
      CHARACTER*3   MESSAGE_SUBTYPE     !Message subtype (locally defined)
      CHARACTER*3   CENTRE_ID           !Centre code (1c)
      CHARACTER*3   SUB_CENTRE_ID       !Centre code (1c)
      CHARACTER*3   CENTRE_ID_1B        !Centre code (1b)
      CHARACTER*3   SUB_CENTRE_ID_1B    !Centre code (1b)
      CHARACTER*5   CAAPP_VERSION       !Numeric, e.g. 714 for v7.14
      CHARACTER*1   CENCODE_NEDT        !Y or N
      CHARACTER*7   INSTRUMENT  !Instrument to encode (HIRS, AMSU-A, AMSU-B/MHS)
      LOGICAL       ENCODE_NEDT
      LOGICAL       finished
      
!--------------------------------------------------------------

!Get environment variables if provided, otherwise set defaults

      CALL GETENV('INSTRUMENT',INSTRUMENT)
      write(*,*)'Instrument=',INSTRUMENT
      CALL GETENV('BUFRENC_FILE',BUFRENC_FILE)
      if (BUFRENC_FILE .eq. '') BUFRENC_FILE='outfile.bufr'
      write(*,*)'Output file: ',BUFRENC_FILE(1:LEN_TRIM(BUFRENC_FILE))

      CALL GETENV('ENCODE_NEDT',CENCODE_NEDT)
      IF (CENCODE_NEDT(1:1).EQ.'Y') THEN
        ENCODE_NEDT = .true.
      ELSE
        ENCODE_NEDT = .false.
      ENDIF

      IORIGCENTRE = 255     !255=Missing
      ISUBCENTRE = 0
      ICENTREID = 255
      ISUBCENTREID = 255
      IF (ENCODE_NEDT) THEN
        IMASTER = 26
      ELSE
        IMASTER = 16
      ENDIF
      ILOCAL = 0
      IMESSAGE_SUBTYPE = -1
      AAPP_VERSION = 0

      CALL GETENV('ORIGINATING_CENTRE',ORIGINATING_CENTRE)
      READ(ORIGINATING_CENTRE,*,ERR=99,END=99) IORIGCENTRE
   99 CALL GETENV('SUB_CENTRE',SUB_CENTRE)
      READ(SUB_CENTRE,*,ERR=100,END=100) ISUBCENTRE
  100 CALL GETENV('CENTRE_ID',CENTRE_ID)
      READ(CENTRE_ID,*,ERR=101,END=101) ICENTREID
  101 CALL GETENV('SUB_CENTRE_ID',SUB_CENTRE_ID)
      READ(SUB_CENTRE_ID,*,ERR=102,END=102) ISUBCENTREID
  102 ICENTREID_1B = ICENTREID
      ISUBCENTREID_1B = ISUBCENTREID
      CALL GETENV('CENTRE_ID_1B',CENTRE_ID_1B)
      READ(CENTRE_ID_1B,*,ERR=103,END=103) ICENTREID_1B
  103 CALL GETENV('SUB_CENTRE_ID_1B',SUB_CENTRE_ID_1B)
      READ(SUB_CENTRE_ID_1B,*,ERR=104,END=104) ISUBCENTREID_1B
  104 CALL GETENV('MASTER_TABLE',MASTER_TABLE)
      READ(MASTER_TABLE,*,ERR=105,END=105) IMASTER
  105 CALL GETENV('LOCAL_TABLE',LOCAL_TABLE)
      READ(LOCAL_TABLE,*,ERR=106,END=106) ILOCAL
  106 CALL GETENV('MESSAGE_SUBTYPE',MESSAGE_SUBTYPE)
      READ(MESSAGE_SUBTYPE,*,ERR=107,END=107) IMESSAGE_SUBTYPE
  107 CALL GETENV('AAPP_VERSION',CAAPP_VERSION)
      READ(CAAPP_VERSION,*,ERR=108,END=108) AAPP_VERSION
  108 CONTINUE

         ! If no message subtype is given, replicate the previous AAPP convention
      
      IF (IMESSAGE_SUBTYPE .LT. 0) THEN
        IF (INSTRUMENT.EQ.'HIRS'.OR.INSTRUMENT.EQ.'HIRS1D'
     &      .OR.INSTRUMENT.EQ.'HIRS1DM') THEN
          IMESSAGE_SUBTYPE=0
        ELSE IF (INSTRUMENT.EQ.'AMSU-A') THEN
          IMESSAGE_SUBTYPE=3
        ELSE IF (INSTRUMENT.EQ.'AMSU-B') THEN
          IMESSAGE_SUBTYPE=4
        ELSE IF (INSTRUMENT.EQ.'MHS') THEN
          IMESSAGE_SUBTYPE=11
        ELSE IF (INSTRUMENT.EQ.'IASI'.OR.INSTRUMENT.EQ.'PCIASI'
     &       .OR.INSTRUMENT.EQ.'IASI1D') THEN
          IMESSAGE_SUBTYPE=221
        ELSE IF (INSTRUMENT.EQ.'CRIS'.OR.INSTRUMENT.EQ.'CRIS1D') THEN
          IMESSAGE_SUBTYPE=202     !value from NOAA
        ELSE IF (INSTRUMENT.EQ.'CRISFSR') THEN
          IMESSAGE_SUBTYPE=205     !value from NOAA
        ELSE IF (INSTRUMENT.EQ.'ATMS'.OR.INSTRUMENT.EQ.'ATMS1D') THEN
          IMESSAGE_SUBTYPE=203     !value from NOAA
        ELSE IF (INSTRUMENT.EQ.'MWTS'.OR.INSTRUMENT.EQ.'MWHS'
     &       .OR.INSTRUMENT.EQ.'IRAS') THEN
          IMESSAGE_SUBTYPE=154     !ECMWF
        ELSE IF (INSTRUMENT.EQ.'MWRI') THEN
          IMESSAGE_SUBTYPE=153     !ECMWF
        ELSE IF (INSTRUMENT.EQ.'MWTS21D'.OR.INSTRUMENT.EQ.'MWHS21D')THEN
          IMESSAGE_SUBTYPE=0
        ELSE IF (INSTRUMENT.EQ.'IRAS1D')THEN
          IMESSAGE_SUBTYPE=0
        ELSE IF (INSTRUMENT.EQ.'MWTS2'.OR.INSTRUMENT.EQ.'MWHS2')THEN
          IMESSAGE_SUBTYPE=0
        ELSE IF (INSTRUMENT.EQ.'HIRAS')THEN
          IMESSAGE_SUBTYPE=0
        ELSE IF (INSTRUMENT.EQ.'MWRI1D')THEN
          IMESSAGE_SUBTYPE=0
        ENDIF
      ENDIF

      finished = .false.
      imessage = 0
      iscan = 0

!Loop over BUFR messages

      do while (.not. finished)

  !Initialise eccodes

        call codes_bufr_new_from_samples(ibufr,'BUFR4',iret)
        if (iret/=CODES_SUCCESS) then
          print *,'ERROR creating BUFR from BUFR4'
          call aappexit(1)
        endif

  !Set some of the fixed items in Section 1
  
        call codes_set(ibufr,'edition',4)
        call codes_set(ibufr,'masterTableNumber',0)
        call codes_set(ibufr,'updateSequenceNumber',0)
        call codes_set(ibufr,'observedData',1)
        call codes_set(ibufr,'compressedData',1)
        call codes_set(ibufr,'bufrHeaderCentre',IORIGCENTRE)
        call codes_set(ibufr,'bufrHeaderSubCentre',ISUBCENTRE)
        call codes_set(ibufr,'masterTablesVersionNumber',IMASTER)
        call codes_set(ibufr,'localTablesVersionNumber',ILOCAL)
        call codes_set(ibufr,'dataSubCategory',IMESSAGE_SUBTYPE)

  !Read the data for the requested instrument and create the message

        SELECT CASE (INSTRUMENT)
          CASE ('HIRS') 
            call eccodes_put_1c_hirs(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,ICENTREID_1B,
     &       ISUBCENTREID_1B,AAPP_VERSION,ENCODE_NEDT)
          CASE ('HIRS1D') 
            call eccodes_put_1d_hirs(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID)
          CASE ('AMSU-A') 
            call eccodes_put_1c_amsua(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,ICENTREID_1B,
     &       ISUBCENTREID_1B,AAPP_VERSION,ENCODE_NEDT)
          CASE ('AMSU-B', 'MHS') 
            call eccodes_put_1c_amsub(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,ICENTREID_1B,
     &       ISUBCENTREID_1B,AAPP_VERSION,ENCODE_NEDT)
          CASE ('IASI') 
            call eccodes_put_1c_iasi(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,AAPP_VERSION)
          CASE ('PCIASI') 
            call eccodes_put_1c_pciasi(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,AAPP_VERSION)
          CASE ('MWHS') 
            call eccodes_put_1c_mwhs(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)
          CASE ('MWHS2') 
            call eccodes_put_1c_mwhs2(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)
          CASE ('MWHS21D') 
            call eccodes_put_1d_mwhs2(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID)
          CASE ('MWTS') 
            call eccodes_put_1c_mwts(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)
          CASE ('MWTS2') 
            call eccodes_put_1c_mwts2(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)
          CASE ('MWTS21D') 
            call eccodes_put_1d_mwts2(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID)
          CASE ('IRAS') 
            call eccodes_put_1c_iras(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)
          CASE ('MWRI') 
            call eccodes_put_1c_mwri(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)
          CASE ('ATMS') 
            call eccodes_put_1c_atms(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)
          CASE ('CRIS') 
            call eccodes_put_1c_cris(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,AAPP_VERSION)
          CASE ('CRISFSR') 
            call eccodes_put_1c_crisfsr(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,AAPP_VERSION)
          CASE ('HIRAS') 
            call eccodes_put_1c_hiras(ibufr,in1cunit,imessage,iscan,
     &       finished,ICENTREID,AAPP_VERSION)
        CASE DEFAULT
          write(*,*)'Invalid instrument: ',INSTRUMENT
          call aappexit(1)
        END SELECT

  !Write to output file

        call codes_set(ibufr,'pack',1)
        if (imessage .eq. 1) then
          call codes_open_file(outfile,BUFRENC_FILE,'w')
        else
          call codes_open_file(outfile,BUFRENC_FILE,'a')
        endif
        call codes_write(ibufr,outfile)
        call codes_close_file(outfile)
        call codes_release(ibufr)

      enddo  !while (.not. finished)
#else
      write(*,*)
     &  'eccodes_encodebufr_1c has not been built with eccodes enabled'
      write(*,*)'Please re-run "configure", with appropriate values for'
      write(*,*)' --external-libs and --external-includes'
      write(*,*)'You need the libraries -leccodes -leccodes_f90'
      write(*,*)'Then re-run "make"'
      write(*,*)'If eccodes is not centrally-installed, you may need'
      write(*,*)' to set LD_LIBRARY_PATH'
#endif
      end program eccodes_encodebufr_1c
