#!/bin/ksh
#
#$Id: eccodes_encodebufr_1c.ksh 619 2019-02-12 12:49:41Z frna $
#
#Encode level 1c/1d AAPP files into BUFR, using ecCodes
#
#Usage: See usage() below
#
#Outputs: BUFR files with .bufr suffix
#
#Environment variables:
#  ORIGINATING_CENTRE - for Section 1 (default 254=EUMETSAT, or 74 for level 1d)
#  SUB_CENTRE - for Section 1 (default 0)
#  MASTER_TABLE - version number of master table (default 14, or 16 for ATMS/CrIS, 
#    or 26 if NEDT is required)
#  LOCAL_TABLE - version number of local table (default 0, or 1 for level 1d)
#  CENTRE_ID - 1b/1c data originating centre, for section 4 (default 254=EUMETSAT)
#  
#History:
# 1.1     30/03/17  Initial version, based on aapp_encodebufr_1c. N C Atkinson
# 1.2     13/12/17  Default is to include version number and NEDT for ATOVS only
# 1.3     30/01/18  If "USE_VASS_SEQUENCE" is set, use local table until master table v30 is released
# 1.4     12/06/18  Add "export ECCODES_BUFR_SET_TO_MISSING_IF_OUT_OF_RANGE=1"
# 1.5     10/12/18  Default MASTER_TABLE=30 for FY-3 instruments if USE_VASS_SEQUENCE=Y
# 1.6     12/02/19  Add HIRAS
#==============================================================================

usage(){
  echo
  echo 'usage : eccodes_encodebufr_1c [-N | -n] [-i files] [-s skip_scans] [-v none|yes] instruments'
  echo
  echo '  where instruments is a list of instruments: HIRS AMSU-A AMSU-B MHS'
  echo '    IASI PCIASI ATMS CRIS CRISFSR MWTS MWHS IRAS MWTS2 MWHS2 MWRI'
  echo '    HIRS1D MWHS21D MWTS21D HIRAS'
  echo '  and files is a list of files to encode.'
  echo '  Quotes " " are necessary if there is more than 1 file'
  echo '  Use -N to include NEDT information (ATOVS only)'
  echo '  By default, AAPP version number is appended to 1c sequences, use -v none to suppress'
  echo
}

. "$AAPP_PREFIX/ATOVS_CONF"

export ECCODES_BUFR_SET_TO_MISSING_IF_OUT_OF_RANGE=1

files=""   #initialise
skip_scans=""

while getopts s:i:Nnv: c
do
  case $c in
     i)	files=$OPTARG;;
     s)	skip_scans=$OPTARG;;
     N)	encode_nedt=Y;;
     n)	encode_nedt=n;;
     v)	encode_version=$OPTARG;;
     \?)      usage
              exit 2;;
  esac

done

shift `expr $OPTIND - 1`
if [ $# -eq 0 ]; then       #no instruments specified
  usage
  exit 2
fi

export BUFR_SKIP_SCANS=$skip_scans

instruments=$*        #Get all the instruments

# For some 1d encoding we need Met Office tables
if [[ $instruments = *CRIS1D* || $instruments = *HIRS1D* \
   || $instruments = *ATMS1D* || $instruments = *IASI1D* \
   || $instruments = *AMSUB1D* ]]; then
  LOCAL_TABLE=${LOCAL_TABLE:-1}
  ORIGINATING_CENTRE=${ORIGINATING_CENTRE:-74}   #74 = Met Office

  definitions=${AAPP_PREFIX}/AAPP/data/tools_eccodes/definitions
  if [ "$ECCODES_DEFINITION_PATH" = "" ]; then
    echo 'Error: you need to define environment variable ECCODES_DEFINITION_PATH'
    echo 'e.g. export ECCODES_DEFINITION_PATH=$DIR_ECCODES/share/eccodes/definitions'
    echo 'where DIR_ECCODES is the ecCodes installation directory'
    exit 1
  elif [[ "$ECCODES_DEFINITION_PATH" != *${definitions}* ]]; then
    ECCODES_DEFINITION_PATH=${ECCODES_DEFINITION_PATH}:${definitions}
  fi
fi

# Other defaults
export ORIGINATING_CENTRE=${ORIGINATING_CENTRE:-254}   #254 = EUMETSAT
export SUB_CENTRE=${SUB_CENTRE:-0}
export LOCAL_TABLE=${LOCAL_TABLE:-0}
export CENTRE_ID=${CENTRE_ID:-${ORIGINATING_CENTRE}}
export SUB_CENTRE_ID=${SUB_CENTRE_ID:-${SUB_CENTRE}}
export BUFR_EDITION=${BUFR_EDITION:-4}

if [ ${#files} != 0 ]; then
  set $files         #Put input files (if specified) in positional parameters
else
  shift $#           #Remove existing positional parameters
fi

err=0

for instrument in $instruments; do

  #For AAPP v8, the default is to encode NEDT and version number for ATOVS, but not for any other data types

  if [ $instrument = HIRS ] || [ $instrument = AMSU-A ] || [ $instrument = AMSU-B ] || [ $instrument = MHS ]; then
    export ENCODE_NEDT=${encode_nedt:-Y}
    encode_version=${encode_version:-yes}
  else
    export ENCODE_NEDT=N
    encode_version=${encode_version:-none}
  fi
  if [ $instrument = MWHS2 ] || [ $instrument = MWTS2 ] || [ $instrument = IRAS ] || [ $instrument = MWHS ] || [ $instrument = MWTS ]; then
    if [ "$USE_VASS_SEQUENCE" = Y ]; then 
      export MASTER_TABLE=${MASTER_TABLE:-30}
      if [ $MASTER_TABLE -lt 30 ]; then       #If MASTER_TABLE <30, use Beijing's local table
        echo "setting MASTER_TABLE=29, LOCAL_TABLE=1, ORIGINATING_CENTRE=39"
        MASTER_TABLE=29
        LOCAL_TABLE=1
        ORIGINATING_CENTRE=39
      fi
      export MESSAGE_SUBTYPE=${MESSAGE_SUBTYPE:-0}
    fi
  fi
  if [ $ENCODE_NEDT = Y ]; then
    export MASTER_TABLE=${MASTER_TABLE:-26}
    if [ $MASTER_TABLE -lt 26 ]; then
      echo "setting MASTER_TABLE=26"
      MASTER_TABLE=26
    fi
  else
    export MASTER_TABLE=${MASTER_TABLE:-16}
  fi
  if [ -s $AAPP_PREFIX/AAPP_version.txt ] && [[ $encode_version != [Nn]* ]]; then
    v=$(head -1 $AAPP_PREFIX/AAPP_version.txt)
    major=${v%%.*}
    minor=${v#*.}
    minor=${minor%%.*}   #ignore any sub-minor version numbers
    if [[ $minor = 0* ]]; then
      minor=0            #set minor version to 0 for beta releases
    fi 
    export AAPP_VERSION=$((100*$major + $minor))     #e.g. 7.14 -> 714, 8.1 -> 801
  else
    AAPP_VERSION=
  fi 
  echo "AAPP_VERSION=$AAPP_VERSION"
    
  case $instrument in
    HIRS)   infile=${1:-hrsn.l1c};;
    AMSU-A) infile=${1:-aman.l1c};;
    AMSU-B) infile=${1:-ambn.l1c};;
    MHS)    infile=${1:-mhsn.l1c};;
    IASI)   infile=${1:-iasi.l1c};;
    PCIASI) infile=${1:-iasi.lpc};;
    CRIS1D) infile=${1:-cris.l1d};;
    ATMS1D) infile=${1:-atms.l1d};;
    CRIS)   infile=${1:-cris.l1c};;
    CRISFSR)infile=${1:-cris.l1c};;
    HIRAS)  infile=${1:-hiras.l1c};;
    ATMS)   infile=${1:-atms.l1c};;
    HIRS1D) infile=${1:-hirs.l1d};;
    HIRS1DM)infile=${1:-hirs.l1d};;
    IASI1D) infile=${1:-iasi.l1d};;
    AMSUB1D) infile=${1:-ambn.l1d};;
    MWTS)   infile=${1:-mwts.l1c};;
    MWHS)   infile=${1:-mwhs.l1c};;
    MWTS2)  infile=${1:-mwts2.l1c};;
    MWHS2)  infile=${1:-mwhs2.l1c};;
    IRAS)   infile=${1:-iras.l1c};;
    MWTS21D)infile=${1:-mwts2.l1c};;
    MWHS21D)infile=${1:-mwhs2.l1c};;
    IRAS1D)infile=${1:-iras.l1c};;
    MWRI)   infile=${1:-mwri.l1c};;
    MWRI1D) infile=${1:-mwri.l1c};;
    *)      usage
            exit 2;;
  esac
  ln -sf $infile ${FORT}10

  outfile=${infile%.*}.bufr
  
  if [ -f $infile ]; then
    if [ -s $infile ]; then
      export INSTRUMENT=$instrument
      export BUFRENC_FILE=$outfile
      rm -f $outfile
      echo processing $infile for $instrument
      eccodes_encodebufr_1c.exe
      if [ $? -ne 0 ]
      then
        err=1
      fi
    else
      echo "Input file $infile is empty"
    fi
  else
    echo "Input file $infile does not exist"
  fi
  
  if [ $# != 0 ]; then
    shift         #Next input file
  fi
done

rm -f ${FORT}10
exit $err
