      subroutine eccodes_get_1c_atms(ibufr,out1cunit,imessage)
      
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_get_1c_atms.F 610 2019-01-17 10:47:43Z frna $
!
! Purpose: BUFR decoder subroutine for AMSU-A l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     28/04/2017 Initial
! 1.1     20/03/2018 Bug fix when reading atms_channels.dat
! 1.2     11/04/2018 Allow for missing date/time
! 1.3     21/11/2018 orbitNumber can change within a message
! 1.4     17/01/2019 BTs have been found to be unreliable if granule quality
!                    flag > 2, so set them to missing.
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_get
! AAPP subroutines: ioat1c, UNBITSWAP
!
! Declarations:

#include "inparms.h"
#include "atms1c.h"
#include "constants_preproc.h"

!Functions
      integer FNSCALEEC, FNISCALEEC, DAYOFYEAR

!Arguments
      integer ibufr
      integer out1cunit
      integer imessage
      
#ifdef HAS_LIBECCODES

!Variables
      integer   Imiss
      parameter (Imiss=-999999)     !AAPP missing value integer

  !Miscellaneous variables 
      integer i, j, irec, iscan, isub, ispot, ichan, jchan, id
      integer sensor
      integer nsubsets
      integer scans_to_store
      integer scans_to_write
      integer skipscan
      REAL    channel_info(5,atms_1cnchan)
      DATA    channel_info /110*0/
      CHARACTER*80 line

  !Variables to be saved between calls
      integer, save      :: spots_to_write
      character*10, save :: formatstr(atms_1cnchan)

  !AAPP data store
      integer, dimension(:,:), allocatable :: atms1c_dt_store
      integer, dimension(:), allocatable   :: scans
      integer, dimension(:), allocatable   :: spots

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      integer*4, dimension(:), allocatable :: descriptors
      character*80 :: descriptor

  !Date/time arrays and scalars
      integer, dimension(:), allocatable :: years
      integer, dimension(:), allocatable :: months
      integer, dimension(:), allocatable :: days
      integer, dimension(:), allocatable :: hours
      integer, dimension(:), allocatable :: minutes
      real*8, dimension(:), allocatable  :: seconds

      real*8  Year,Month,Day      !for passing to DAYOFYEAR
      integer millisecs, jday
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_orbit,
     & index_granulequal, index_scanqual, 
     & index_geolocqual, index_chanqual,
     & index_latlon, index_angles,
     & index_scalti, index_nedtcold, index_nedtwarm, 
     & index_anttemps, index_btemps

!-----------------------------------------------------------------------

      call codes_get(ibufr,'satelliteInstruments',sensor)
      if (sensor .ne. 621) then
        write(*,*) 'Skip message. Sensor=',sensor
        return
      endif

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,atms_1cnchan
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_atms1c
          atms1c_dt(i) = i
        enddo
        index_scnlin = atms1c_scnlin
        index_scnlinyr = atms1c_scnlinyr
        index_scnlindy = atms1c_scnlindy
        index_scnlintime = atms1c_scnlintime(1)
        index_orbit = atms1c_orbit
        index_granulequal = atms1c_granulequal
        index_scanqual = atms1c_scanqual
        index_geolocqual = atms1c_geolocqual
        index_chanqual = atms1c_chanqual(1)
        index_latlon = atms1c_latlon(1,1)
        index_angles = atms1c_angles(1,1)
        index_scalti = atms1c_scalti
        index_nedtcold = atms1c_nedtcold(1)
        index_nedtwarm = atms1c_nedtwarm(1)
        index_anttemps = atms1c_anttemps(1,1)
        index_btemps = atms1c_btemps(1,1)
      
!Initialise some header variables
        call codes_get(ibufr,'bufrHeaderCentre',i)
        atms1c_h_centre = i
        call codes_get(ibufr,'centre',i)
        if (i .ge. 0 .and. i .lt. 999) then
          write(atms1c_h_site,'(I3.3)') i
        else
          atms1c_h_site = "xxx"
        endif
        call codes_get(ibufr,'bufrHeaderSubCentre',i)
        atms1c_h_subcentre = i
        call codes_get(ibufr,'subCentre',i)
        if (i .ge. 0 .and. i .le. 999) then
          write(atms1c_h_1bsite,'(I3.3)') i
        else
          atms1c_h_1bsite = "xxx"
        endif
        atms1c_h_cfill1 = ' '
        atms1c_h_cfill2 = ' '
        atms1c_h_scnlin = 0
        atms1c_h_hdrcnt = 1
        atms1c_h_startdatatime = -1
        atms1c_h_versnb = atms_1cvnnb
        atms1c_h_versyr = atms_1cvnyr
        atms1c_h_versdy = atms_1cvndy
        atms1c_h_instrument = 621       !ATMS
        atms1c_h_nfov = atms_1cnfov
        atms1c_h_nchan = atms_1cnchan
        atms1c_h_nchanused = atms_1cnchan
        atms1c_h_recwords = array_size_atms1c
        atms1c_h_misscnlin = 0
        spots_to_write = 0

        call codes_get(ibufr,'centre',atms1c_h_centre)
        call codes_get(ibufr,'subCentre',atms1c_h_subcentre)
        call codes_get(ibufr,'satelliteIdentifier',atms1c_h_wmosatid)
        atms1c_h_satid = atms1c_h_wmosatid-223   !NPP=224 -> 1
        IF (atms1c_h_wmosatid .eq. Imiss) atms1c_h_satid = Imiss
        !!call codes_get(ibufr,'orbitNumber',atms1c_h_startorbit)
        call codes_get(ibufr,
     &    'satelliteAntennaCorrectionsVersionNumber',i)
        atms1c_h_vnantennacorr = FNISCALEEC(i,1)

  !Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.atms1c_h_wmosatid) THEN
            atms1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            atms1c_h_period = OrbitPd(J)   !period, seconds
            atms1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO

  !Read channel info from the atms_channels.dat file (same as in atms_sdr_out.F)
        OPEN (30,form='formatted',status='old')
        line="     "
        id = 0
        DO while (id .NE. atms1c_h_wmosatid)
          DO while (line(1:5) .ne. "# WMO")
            read(30,'(A)',END=100) line
          ENDDO
          read(30,*) id
          line="     "
          IF (id .eq. atms1c_h_wmosatid) THEN
            read(30,*) line
            read(30,*) channel_info
          ENDIF
        ENDDO
  100   CLOSE (30)
        DO ichan=1,atms_1cnchan
          atms1c_h_tempradcnv(1,ichan) = 
     &      channel_info(2,ichan)*1.0E13/C_light   !wavenumber*1E6
          atms1c_h_tempradcnv(2,ichan) = channel_info(4,ichan)*1.0E6  !offset
          atms1c_h_tempradcnv(3,ichan) = channel_info(5,ichan)*1.0E6  !slope
          atms1c_h_bandwidth(ichan) = channel_info(3,ichan)*1.0E6     !GHz*1E6
        ENDDO

  !Open the output file and write header

        call ioat1c("OPEN/WRITE", out1cunit, 0)
        call ioat1c("WRITE HEADER", out1cunit, 1)

      endif     !(imessage .eq. 1)

  !Read the total number of subsets. Normally an integer multiple of the number
  !of spots, but make the code general in case scans are split across messages. 

      call codes_get(ibufr,'numberOfSubsets',nsubsets)
      scans_to_store = (nsubsets+spots_to_write)/atms_1cnfov + 1    !add 1 scan to be safe

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))
      allocate(scans(nsubsets))
      allocate(spots(nsubsets))
      allocate(years(nsubsets))
      allocate(months(nsubsets))
      allocate(days(nsubsets))
      allocate(hours(nsubsets))
      allocate(minutes(nsubsets))
      allocate(seconds(nsubsets))
      allocate(atms1c_dt_store(array_size_atms1c,scans_to_store))
      atms1c_dt_store = 0

  !if there is a partial scan left over from previous message, load the data
  !into the first scan
      if (spots_to_write .ne. 0) then
        do i=1,array_size_atms1c
          atms1c_dt_store(i,1) = atms1c_dt(i)
        enddo
      endif

  !FOV number
      call codes_get(ibufr,'fieldOfViewNumber',spots)
      do isub = 1,nsubsets
        if (spots(isub).le.0.or.spots(isub).gt.atms_1cnfov) then
          write(*,*)'Error: invalid fov number',spots(isub)
          call aappexit(1)
        endif
      enddo
      if (imessage .eq. 1) then
        if (spots(1).ne.1) then
          skipscan = 1      !ignore initial partial scan if there is one
          spots_to_write = nsubsets + 1 - spots(1)
        else
          skipscan = 0
          spots_to_write = nsubsets
        endif
      else
        skipscan = 0
        spots_to_write = spots_to_write + nsubsets
      endif

  !scan number
      call codes_get(ibufr,'scanLineNumber',ivalues)
      scans(1) = 1
      do isub = 2,nsubsets
        scans(isub) = scans(isub-1)
        if (spots(isub).eq.1) scans(isub) = scans(isub) + 1
      enddo
      do isub = 1,nsubsets
        atms1c_dt_store(index_scnlin,scans(isub)) = ivalues(isub)
      enddo

  !granulequal
      call codes_get(ibufr,'granuleLevelQualityFlags',ivalues)
      do isub = 1,nsubsets
        atms1c_dt_store(index_granulequal,scans(isub))
     &       = FNISCALEEC(ivalues(isub),1)
      enddo

  !scanqual
      call codes_get(ibufr,'scanLevelQualityFlags',ivalues)
      do isub = 1,nsubsets,atms_1cnfov
        atms1c_dt_store(index_scanqual,scans(isub)) =
     &      FNISCALEEC(ivalues(isub),1)
      enddo

  !geolocqual
      call codes_get(ibufr,'geolocationQuality',ivalues)
      do isub = 1,nsubsets,atms_1cnfov
        atms1c_dt_store(index_geolocqual,scans(isub))
     &       = FNISCALEEC(ivalues(isub),1)
      enddo

  !date/time
      call codes_get(ibufr,'year',years)
      call codes_get(ibufr,'month',months)
      call codes_get(ibufr,'day',days)
      call codes_get(ibufr,'hour',hours)
      call codes_get(ibufr,'minute',minutes)
      call codes_get(ibufr,'second',seconds)
        
  !transform the date/time variables
      do isub = 1,nsubsets
        if (spots(isub).eq.1) then
          if (days(isub) .ne. CODES_MISSING_LONG) then
            Year = years(isub)
            Month = months(isub)
            Day = days(isub)
            atms1c_dt_store(index_scnlinyr, scans(isub)) = Year
            jday = DAYOFYEAR(Year,Month,Day)
            atms1c_dt_store(index_scnlindy, scans(isub)) = jday
          else
            atms1c_dt_store(index_scnlinyr, scans(isub)) = Imiss
            atms1c_dt_store(index_scnlindy, scans(isub)) = Imiss
          endif
        endif
        if (hours(isub) .ne. CODES_MISSING_LONG) then
          millisecs = hours(isub)*3600000 + minutes(isub)*60000 
     &                  + seconds(isub)*1000   
          atms1c_dt_store(index_scnlintime+spots(isub)-1, scans(isub)) =
     &       millisecs
        else
          atms1c_dt_store(index_scnlintime+spots(isub)-1, scans(isub)) =
     &       Imiss
        endif
      enddo

  !orbit
      call codes_get(ibufr,'orbitNumber',ivalues)
      if (imessage .eq. 1) atms1c_h_startorbit = ivalues(1)
      atms1c_h_endorbit = ivalues(nsubsets)
      do isub = 1,nsubsets,atms_1cnfov
        atms1c_dt_store(index_orbit,scans(isub)) = ivalues(isub)
      enddo

  !latitude
      call codes_get(ibufr,'latitude',rvalues)
      do isub = 1,nsubsets
        atms1c_dt_store(index_latlon+2*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !longitude
      call codes_get(ibufr,'longitude',rvalues)
      do isub = 1,nsubsets
        atms1c_dt_store(index_latlon+2*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !scalti
      call codes_get(ibufr,'height',rvalues)
      do isub = 1,nsubsets,atms_1cnfov
        atms1c_dt_store(index_scalti,scans(isub)) = rvalues(isub)
      enddo

  !sat zenith
      call codes_get(ibufr,'satelliteZenithAngle',rvalues)
      do isub = 1,nsubsets
        atms1c_dt_store(index_angles+4*spots(isub)-4, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !sat azimuth (0 to 360 deg)
      call codes_get(ibufr,'bearingOrAzimuth',rvalues)
      do isub = 1,nsubsets
        atms1c_dt_store(index_angles+4*spots(isub)-3, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar zenith
      call codes_get(ibufr,'solarZenithAngle',rvalues)
      do isub = 1,nsubsets
        atms1c_dt_store(index_angles+4*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar azimuth (0 to 360 deg)
      call codes_get(ibufr,'solarAzimuth',rvalues)
      do isub = 1,nsubsets
        atms1c_dt_store(index_angles+4*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !Channel parameters - already got these from data file. 

  !chanqual (chan,scan)
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelDataQualityFlags'
        call codes_get(ibufr, descriptor, ivalues)
        do isub = 1,nsubsets,atms_1cnfov
          atms1c_dt_store(index_chanqual+ichan-1, scans(isub)) = 
     &       FNISCALEEC(ivalues(isub),1)
        enddo   
      enddo

  !btemps (chan,spot,scan). Set BTs to missing if the granule quality flag is
  !greater than 2 (indicates bad or missing data in the downlink).
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        call codes_get(ibufr, descriptor, rvalues)
        do isub = 1,nsubsets
          if (atms1c_dt_store(index_granulequal,scans(isub)) .le. 2)then
            atms1c_dt_store(index_btemps + ichan-1 + 
     &        atms_1cnchan*(spots(isub)-1), scans(isub)) =
     &        FNSCALEEC(rvalues(isub), 1.0D2)
          else
            atms1c_dt_store(index_btemps + ichan-1 + 
     &        atms_1cnchan*(spots(isub)-1), scans(isub)) = Imiss
          endif
        enddo
      enddo

  !anttemps (chan,spot,scan). Apply extra QC: set BTs to missing if the
  !granule quality flag is greater than 2 (probably bad data in the downlink).
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#antennaTemperature'
        call codes_get(ibufr, descriptor, rvalues)
        do isub = 1,nsubsets
          if (atms1c_dt_store(index_granulequal,scans(isub)) .le. 2)then
            atms1c_dt_store(index_anttemps + ichan-1 + 
     &        atms_1cnchan*(spots(isub)-1), scans(isub)) =
     &        FNSCALEEC(rvalues(isub), 1.0D2)
          else
            atms1c_dt_store(index_anttemps + ichan-1 + 
     &        atms_1cnchan*(spots(isub)-1), scans(isub)) = Imiss
          endif
        enddo
      enddo

  !nedtcold
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#noiseEquivalentDeltaTemperatureWhileViewingColdTarget'
        call codes_get(ibufr, descriptor, rvalues)
        do isub = 1,nsubsets,atms_1cnfov
          atms1c_dt_store(index_nedtcold + ichan-1, scans(isub)) =
     &        FNSCALEEC(rvalues(isub), 1.0D4)
        enddo
      enddo

  !nedtwarm
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#noiseEquivalentDeltaTemperatureWhileViewingWarmTarget'
        call codes_get(ibufr, descriptor, rvalues)
        do isub = 1,nsubsets,atms_1cnfov
          atms1c_dt_store(index_nedtwarm + ichan-1, scans(isub)) =
     &        FNSCALEEC(rvalues(isub), 1.0D4)
        enddo
      enddo

  !CSPP version number, if present
      call codes_get(ibufr,'unexpandedDescriptors',descriptors)
      do i=1,size(descriptors)
        if (descriptors(i) .eq. 25061) then
          call codes_get(ibufr,'softwareVersionNumber',
     &       atms1c_h_software_version)
        endif
      enddo

  !save the data
      scans_to_write = spots_to_write/atms_1cnfov
      DO iscan=1,scans_to_write
        spots_to_write = spots_to_write - atms_1cnfov
        do i=1,array_size_atms1c
          atms1c_dt(i) = atms1c_dt_store(i,iscan+skipscan)
        enddo
        atms1c_h_scnlin = atms1c_h_scnlin + 1
        if (atms1c_h_startdatatime .lt. 0) then
          atms1c_h_startdatatime = atms1c_scnlintime(1)
          atms1c_h_startdatady = atms1c_scnlindy
          atms1c_h_startdatayr = atms1c_scnlinyr
        endif
        atms1c_h_enddatatime = atms1c_scnlintime(atms_1cnfov)
        atms1c_h_enddatady = atms1c_scnlindy
        atms1c_h_enddatayr = atms1c_scnlinyr
        if ((atms1c_scnlintime(1)-atms1c_scnlintime(atms_1cnfov))
     &      .gt.86392000) then
          atms1c_h_enddatady = atms1c_h_enddatady + 1
          if ((atms1c_h_enddatady .gt. 366) .or. 
     &         (MOD(atms1c_h_enddatayr,4) .ne. 0 .and. 
     &         atms1c_h_enddatady .gt. 365)) then
            atms1c_h_enddatady = 1
            atms1c_h_enddatayr = atms1c_h_enddatayr + 1
          endif
        endif
        call ioat1c("WRITE DATA", out1cunit,
     &      atms1c_h_scnlin+atms1c_h_hdrcnt)
      ENDDO

  !Update the header
      call ioat1c("WRITE HEADER", out1cunit, 1)
      write(*,*)'Message:',imessage, ' subsets:',nsubsets,
     &  ' scan count:',atms1c_h_scnlin

  !If there is a partial scan left over, save the data
      if (spots_to_write .ne. 0) then
        do i=1,array_size_atms1c
          atms1c_dt(i) = atms1c_dt_store(i,scans_to_write+1)
        enddo
      endif

  !Deallocate arrays before exiting the subroutine
      if(allocated(ivalues)) deallocate(ivalues)
      if(allocated(rvalues)) deallocate(rvalues)
      if(allocated(scans)) deallocate(scans)
      if(allocated(spots)) deallocate(spots)
      if(allocated(years)) deallocate(years)
      if(allocated(months)) deallocate(months)
      if(allocated(days)) deallocate(days)
      if(allocated(hours)) deallocate(hours)
      if(allocated(minutes)) deallocate(minutes)
      if(allocated(seconds)) deallocate(seconds)
      if(allocated(atms1c_dt_store)) deallocate(atms1c_dt_store)
#endif

      end subroutine eccodes_get_1c_atms
