      subroutine eccodes_get_1c_cris(ibufr,out1cunit,imessage)
      
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_get_1c_cris.F 601 2018-11-21 15:57:14Z frna $
!
! Purpose: BUFR decoder subroutine for CRIS l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     04/05/2017 Initial
! 1.1     17/05/2018 Allocate fovs and fors
! 1.2     21/11/2018 orbitNumber can change within a message
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_get
! AAPP subroutines: ioc1c, UNBITSWAP
!
! Declarations:

#include "inparms.h"
#include "cris1c.h"

!Functions
      integer FNSCALEEC, FNISCALEEC, DAYOFYEAR

!Arguments
      integer ibufr
      integer out1cunit
      integer imessage
      logical nedt_present
      
#ifdef HAS_LIBECCODES

!Variables
      integer   Imiss
      parameter (Imiss=-999999)     !AAPP missing value integer
      integer   Cnfovs
      parameter (Cnfovs=cris_1cnfov*cris_1cnfor)

  !Miscellaneous variables 
      integer i, j, irec, iscan, isub, jsub, ispot, ichan, jchan, kchan
      integer pos, pos0, iband, sensor
      integer nsubsets, scans_to_store, scans_to_write, skipscan

  !Variables to be saved between calls
      integer, save      :: spots_to_write
      character*10, save :: formatstr(cris_1cnchan)

  !AAPP data store
      integer, dimension(:,:), allocatable :: cris1c_dt_store
      integer, dimension(:), allocatable   :: scans
      integer, dimension(:), allocatable   :: spots
      integer, dimension(:), allocatable   :: fovs
      integer, dimension(:), allocatable   :: fors
      integer   scalrad_i4(1)
      integer*2 scalrad_i2(2)
      equivalence(scalrad_i4(1),scalrad_i2(1))

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      integer*4, dimension(:), allocatable :: descriptors
      character*80 :: descriptor

  !Date/time arrays and scalars
      integer, dimension(:), allocatable :: years
      integer, dimension(:), allocatable :: months
      integer, dimension(:), allocatable :: days
      integer, dimension(:), allocatable :: hours
      integer, dimension(:), allocatable :: minutes
      real*8, dimension(:), allocatable  :: seconds
      integer scnlindy,scnlinyr,FovTime  !date and time
      real*8  Year,Month,Day      !for passing to DAYOFYEAR
      integer millisecs, jday
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin,index_scnlinyr,index_scnlindy,
     & index_scnlintime,index_latlon,index_angles,
     & index_satxyz,index_ascdesc,index_orbit,index_scalti,
     & index_surfht,index_landfrac,index_landsea,index_cloudcover,
     & index_cloudtopht,index_radianceflgs,index_scanqualflgs,
     & index_calqualflgs,index_fovqualflgs,index_geolocqual,
     & index_quality,index_scores,index_logfit,index_radiance

!-----------------------------------------------------------------------

      call codes_get(ibufr,'#1#satelliteInstruments',sensor)
      if (sensor .ne. 620) then
        write(*,*) 'Skip message. Sensor=',sensor
        return
      endif

!number of channels
      call codes_get(ibufr,
     & 'extendedDelayedDescriptorReplicationFactor',cris1c_h_nchan)
      cris1c_h_nchanused = cris1c_h_nchan
      cris1c_h_recwords = 
     &         11 + cris_1cnfor + cris_1cnband*cris_1cnfov + 
     &         (13 + 2*cris_1cnband + cris1c_h_nchan +12 + cris_1cnpc)
     &              *cris_1cnfov*cris_1cnfor

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,min(9,cris_1cnchan)
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,min(99,cris_1cnchan)
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo
        do ichan=100,min(999,cris_1cnchan)
          write(formatstr(ichan),'(A)') '(A,I3,A)'
        enddo
        do ichan=1000,cris_1cnchan
          write(formatstr(ichan),'(A)') '(A,I4,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_cris1c
          cris1c_dt(i) = i
        enddo
        index_scnlin = cris1c_scnlin
        index_scnlinyr = cris1c_scnlinyr
        index_scnlindy = cris1c_scnlindy
        index_scnlintime = cris1c_scnlintime(1)
        index_latlon = cris1c_latlon(1,1,1)
        index_angles = cris1c_angles(1,1,1)
        index_satxyz = cris1c_satxyz(1)
        index_ascdesc = cris1c_ascdesc
        index_orbit = cris1c_orbit
        index_scalti = cris1c_scalti
        index_surfht = cris1c_surfht(1,1)
        index_landfrac = cris1c_landfrac(1,1)
        index_landsea = cris1c_landsea(1,1)
        index_cloudcover = cris1c_cloudcover(1,1)
        index_cloudtopht = cris1c_cloudtopht(1,1)
        index_radianceflgs = cris1c_radianceflgs(1,1)
        index_scanqualflgs = cris1c_scanqualflgs
        index_calqualflgs = cris1c_calqualflgs(1,1)
        index_fovqualflgs = cris1c_fovqualflgs(1,1,1)
        index_geolocqual = cris1c_geolocqual
        index_quality = cris1c_quality(1,1)
        index_scores = cris1c_scores(1,1,1)
        index_logfit = cris1c_logfit(1,1,1)
        index_radiance = cris1c_radiance(1,1,1)
      
!Initialise some header variables
        call codes_get(ibufr,'bufrHeaderCentre',i)
        cris1c_h_centre = i
        call codes_get(ibufr,'centre',i)
        if (i .ge. 0 .and. i .lt. 999) then
          write(cris1c_h_site,'(I3.3)') i
        else
          cris1c_h_site = "xxx"
        endif
        call codes_get(ibufr,'bufrHeaderSubCentre',i)
        cris1c_h_subcentre = i
        if (i .ge. 0 .and. i .le. 999) then  !NB: no "subcentre" in the sequence 
          write(cris1c_h_1bsite,'(I3.3)') i
        else
          cris1c_h_1bsite = "xxx"
        endif
        cris1c_h_cfill1 = ' '
        cris1c_h_cfill2 = ' '
        cris1c_h_scnlin = 0
        cris1c_h_hdrcnt = 1
        cris1c_h_startdatatime = -1
        cris1c_h_versnb = cris_1cvnnb
        cris1c_h_versyr = cris_1cvnyr
        cris1c_h_versdy = cris_1cvndy
        cris1c_h_instrument = 620       !CRIS
        cris1c_h_nfov = cris_1cnfov
        cris1c_h_nfor = cris_1cnfor
        cris1c_h_nband = cris_1cnband
        cris1c_h_npc = cris_1cnpc
        cris1c_h_npcused = 0      !BUFR sequence does not have PCs
        spots_to_write = 0

        call codes_get(ibufr,'satelliteIdentifier',cris1c_h_wmosatid)

        cris1c_h_satid = cris1c_h_wmosatid-223   !NPP=224 -> 1
        IF (cris1c_h_wmosatid .eq. Imiss) cris1c_h_satid = Imiss
        !!call codes_get(ibufr,'orbitNumber',cris1c_h_startorbit)

  !Nominal height and period are not in the BUFR, so get them from fdf file
        DO J=1,MaxSats
          IF (WMOid(J).EQ.cris1c_h_wmosatid) THEN
            cris1c_h_satht = SatHt(J)*10   !nominal ht, km*10
            cris1c_h_period = OrbitPd(J)   !period, seconds
            cris1c_h_satid = NOAAid(J)     !Satellite ID
          ENDIF
        ENDDO

  !Open the output file and write header

        call ioc1c("OPEN/WRITE", out1cunit, 0)
        call ioc1c("WRITE HEADER", out1cunit, 1)

      endif     !(imessage .eq. 1)

      !!call codes_get(ibufr,'orbitNumber',cris1c_h_endorbit)

  !Read the total number of subsets. Normally an integer multiple of the number
  !of spots, but make the code general in case scans are split across messages. 

      call codes_get(ibufr,'numberOfSubsets',nsubsets)
      scans_to_store = (nsubsets+spots_to_write)/Cnfovs + 1    !add 1 scan to be safe

      allocate(fovs(nsubsets))
      allocate(fors(nsubsets))
      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))
      allocate(scans(nsubsets))
      allocate(spots(nsubsets))
      allocate(years(nsubsets))
      allocate(months(nsubsets))
      allocate(days(nsubsets))
      allocate(hours(nsubsets))
      allocate(minutes(nsubsets))
      allocate(seconds(nsubsets))
      allocate(cris1c_dt_store(array_size_cris1c,scans_to_store))
      cris1c_dt_store = 0

  !if there is a partial scan left over from previous message, load the data
  !into the first scan
      if (spots_to_write .ne. 0) then
        do i=1,array_size_cris1c
          cris1c_dt_store(i,1) = cris1c_dt(i)
        enddo
      endif

  !Orbit number
      call codes_get(ibufr,'orbitNumber',ivalues)
      if (imessage .eq. 1) cris1c_h_startorbit = ivalues(1)
      cris1c_h_endorbit = ivalues(nsubsets)

  !FOV number
      call codes_get(ibufr,'fieldOfViewNumber',fovs)
      call codes_get(ibufr,'fieldOfRegardNumber',fors)
      do isub = 1,nsubsets
        spots(isub) = fovs(isub) + (fors(isub)-1)*cris_1cnfov
        if (spots(isub).le.0.or.spots(isub).gt.Cnfovs) then
          write(*,*)'Error: invalid fov number',spots(isub)
          call aappexit(1)
        endif
      enddo
      if (imessage .eq. 1) then
        if (spots(1).ne.1) then
          skipscan = 1      !ignore initial partial scan if there is one
          spots_to_write = nsubsets + 1 - spots(1)
        else
          skipscan = 0
          spots_to_write = nsubsets
        endif
      else
        skipscan = 0
        spots_to_write = spots_to_write + nsubsets
      endif

  !scan number
      call codes_get(ibufr,'scanLineNumber',ivalues)
      scans(1) = 1
      do isub = 2,nsubsets
        scans(isub) = scans(isub-1)
        if (spots(isub).eq.1) scans(isub) = scans(isub) + 1
      enddo
      do isub = 1,nsubsets
        cris1c_dt_store(index_scnlin,scans(isub)) = ivalues(isub)
      enddo

  !date/time
      call codes_get(ibufr,'year',years)
      call codes_get(ibufr,'month',months)
      call codes_get(ibufr,'day',days)
      call codes_get(ibufr,'hour',hours)
      call codes_get(ibufr,'minute',minutes)
      call codes_get(ibufr,'second',seconds)
        
  !transform the date/time variables
      do isub = 1,nsubsets
        if (spots(isub).eq.1) then
          Year = years(isub)
          Month = months(isub)
          Day = days(isub)
          cris1c_dt_store(index_scnlinyr, scans(isub)) = Year
          jday = DAYOFYEAR(Year,Month,Day)
          cris1c_dt_store(index_scnlindy, scans(isub)) = jday
        endif
        if (MOD(spots(isub),9) .eq. 1) then
          millisecs = hours(isub)*3600000 + minutes(isub)*60000 
     &                  + seconds(isub)*1000   
          cris1c_dt_store(index_scnlintime+spots(isub)/9, scans(isub)) = 
     &                  millisecs
        endif
      enddo

  !satxyz
      call codes_get(ibufr,
     &  'DistanceFromEarthCentreInDirectionOf0DegreesLongitude',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_satxyz,scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D0)
      enddo

      call codes_get(ibufr,
     &  'DistanceFromEarthCentreInDirection90DegreesEast',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_satxyz+1,scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D0)
      enddo

      call codes_get(ibufr,
     &  'DistanceFromEarthCentreInDirectionOfNorthPole',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_satxyz+2,scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D0)
      enddo

  !latitude
      call codes_get(ibufr,'latitude',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_latlon+2*spots(isub)-2, scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !longitude
      call codes_get(ibufr,'longitude',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_latlon+2*spots(isub)-1, scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !sat zenith
      call codes_get(ibufr,'satelliteZenithAngle',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_angles+4*spots(isub)-4, scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !sat azimuth (0 to 360 deg in BUFR, signed in AAPP)
      call codes_get(ibufr,'bearingOrAzimuth',rvalues)
      do isub = 1,nsubsets
        if (rvalues(isub).ne.CODES_MISSING_DOUBLE .and. 
     &      rvalues(isub).gt.180.0D0) then
          rvalues(isub)=rvalues(isub)-360.0D0
        endif
        cris1c_dt_store(index_angles+4*spots(isub)-3, scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar zenith
      call codes_get(ibufr,'solarZenithAngle',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_angles+4*spots(isub)-2, scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar azimuth (0 to 360 deg in BUFR, signed in AAPP)
      call codes_get(ibufr,'solarAzimuth',rvalues)
      do isub = 1,nsubsets
        if (rvalues(isub).ne.CODES_MISSING_DOUBLE .and. 
     &      rvalues(isub).gt.180.0D0) then
          rvalues(isub)=rvalues(isub)-360.0D0
        endif
        cris1c_dt_store(index_angles+4*spots(isub)-1, scans(isub))
     &    = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !ascending/descending
      call codes_get(ibufr,'orbitQualifier',ivalues)
      do isub = 1,nsubsets,Cnfovs
        cris1c_dt_store(index_ascdesc,scans(isub))
     &    = FNISCALEEC(ivalues(isub),1)
      enddo

  !Orbit
      call codes_get(ibufr,'orbitNumber',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_orbit,scans(isub))
     &    = FNSCALEEC(rvalues(isub), 1.0D0)
      enddo

  !surfht
      call codes_get(ibufr,'heightOfLandSurface',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_surfht+spots(isub)-1, scans(isub))
     &    = FNSCALEEC(rvalues(isub), 1.0D0)
      enddo
        
  !scalti
      call codes_get(ibufr,'height',rvalues)
      do isub = 1,nsubsets,Cnfovs
        cris1c_dt_store(index_scalti,scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D-2)  !store as km*10
      enddo

  !landfrac
      call codes_get(ibufr,'landFraction',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_landfrac+spots(isub)-1,scans(isub)) =
     &     FNSCALEEC(rvalues(isub),1.0D2)
      enddo
        
  !landsea
      call codes_get(ibufr,'landOrSeaQualifier',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_landsea+spots(isub)-1,scans(isub)) =
     &     FNSCALEEC(rvalues(isub),1.0D2)
      enddo
        
  !cloudcover
      call codes_get(ibufr,'cloudCoverTotal',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_cloudcover+spots(isub)-1,scans(isub)) =
     &     FNSCALEEC(rvalues(isub),1.0D2)
      enddo
        
  !cloudtopht
      call codes_get(ibufr,'heightOfTopOfCloud',rvalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_cloudtopht+spots(isub)-1,scans(isub)) =
     &     FNSCALEEC(rvalues(isub),1.0D2)
      enddo
        
  !radianceflgs
      call codes_get(ibufr,'radianceTypeFlags',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_radianceflgs+spots(isub)-1,scans(isub)) =
     &     FNISCALEEC(ivalues(isub),1)
      enddo

  !scanqualflgs
      call codes_get(ibufr,'scanLevelQualityFlags',ivalues)
      do isub = 1,nsubsets,Cnfovs
        cris1c_dt_store(index_scanqualflgs,scans(isub)) =
     &     FNISCALEEC(ivalues(isub),1)
      enddo

  !Band definitions and flags, band 1
      call codes_get(ibufr,'#1#waveNumber',i)
      cris1c_h_startw(1) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#2#waveNumber',i)
      cris1c_h_endw(1) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#1#startChannel',i)
      cris1c_h_startchan(1) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#1#endChannel',i)
      cris1c_h_endchan(1) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#1#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_calqualflgs+3*fovs(isub)-2, scans(isub)) =
     &     FNISCALEEC(ivalues(isub),1)
      enddo
      call codes_get(ibufr,'#1#fieldOfViewQualityFlags',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_fovqualflgs+3*fovs(isub)-2, scans(isub)) =
     &     FNISCALEEC(ivalues(isub),1)
      enddo

  !Band definitions and flags, band 2
      call codes_get(ibufr,'#3#waveNumber',i)
      cris1c_h_startw(2) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#4#waveNumber',i)
      cris1c_h_endw(2) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#2#startChannel',i)
      cris1c_h_startchan(2) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#2#endChannel',i)
      cris1c_h_endchan(2) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#2#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_calqualflgs+3*fovs(isub)-1, scans(isub)) =
     &     FNISCALEEC(ivalues(isub),1)
      enddo
      call codes_get(ibufr,'#2#fieldOfViewQualityFlags',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_fovqualflgs+3*fovs(isub)-1, scans(isub)) =
     &     FNISCALEEC(ivalues(isub),1)
      enddo

  !Band definitions and flags, band 3
      call codes_get(ibufr,'#5#waveNumber',i)
      cris1c_h_startw(3) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#6#waveNumber',i)
      cris1c_h_endw(3) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#3#startChannel',i)
      cris1c_h_startchan(3) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#3#endChannel',i)
      cris1c_h_endchan(3) = FNISCALEEC(i,1)
      call codes_get(ibufr,'#3#calibrationQualityFlags',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_calqualflgs+3*fovs(isub), scans(isub)) =
     &     FNISCALEEC(ivalues(isub),1)
      enddo
      call codes_get(ibufr,'#3#fieldOfViewQualityFlags',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_fovqualflgs+3*fovs(isub), scans(isub)) =
     &     FNISCALEEC(ivalues(isub),1)
      enddo

  !geolocqual
      call codes_get(ibufr,'geolocationQuality',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_geolocqual,scans(isub))
     &    = FNISCALEEC(ivalues(isub), 1)
      enddo

  !quality
      call codes_get(ibufr,'qualityInformation',ivalues)
      do isub = 1,nsubsets
        cris1c_dt_store(index_quality+spots(isub)-1,scans(isub))
     &    = FNISCALEEC(ivalues(isub), 1)
      enddo

  !channel number
      if(allocated(ivalues)) deallocate(ivalues)
      call codes_get(ibufr, 'channelNumber', ivalues)
      do ichan=1,cris1c_h_nchanused
        cris1c_h_channels(ichan) = ivalues(ichan)
      enddo

  !Radiance
      do ichan=1,cris1c_h_nchanused
        write(descriptor,formatstr(ichan)) '#',ichan,'#channelRadiance'
        call codes_get(ibufr,descriptor,rvalues)
        do isub = 1,nsubsets
          cris1c_dt_store(index_radiance + spots(isub)-1 + 
     &      (ichan-1)*Cnfovs, scans(isub)) = 
     &        FNSCALEEC(rvalues(isub),1.0D7)   !Radiance W/m^2/str/cm^-1 *1E7
        enddo
      enddo

  !CSPP version number, if present
      call codes_get(ibufr,'unexpandedDescriptors',descriptors)
      do i=1,size(descriptors)
        if (descriptors(i) .eq. 25061) then
          call codes_get(ibufr,'softwareVersionNumber',
     &       cris1c_h_software_version)
        endif
      enddo

  !save the data
      scans_to_write = spots_to_write/Cnfovs
      DO iscan=1,scans_to_write
        spots_to_write = spots_to_write - Cnfovs
        do i=1,array_size_cris1c
          cris1c_dt(i) = cris1c_dt_store(i,iscan+skipscan)
        enddo
        cris1c_h_scnlin = cris1c_h_scnlin + 1
        if (cris1c_h_startdatatime .lt. 0) then
          cris1c_h_startdatatime = cris1c_scnlintime(1)
          cris1c_h_startdatady = cris1c_scnlindy
          cris1c_h_startdatayr = cris1c_scnlinyr
        endif
        cris1c_h_enddatatime = cris1c_scnlintime(cris_1cnfor)
        cris1c_h_enddatady = cris1c_scnlindy
        cris1c_h_enddatayr = cris1c_scnlinyr
        if ((cris1c_scnlintime(1)-cris1c_scnlintime(cris_1cnfor))
     &      .gt.86392000) then
          cris1c_h_enddatady = cris1c_h_enddatady + 1
          if ((cris1c_h_enddatady .gt. 366) .or. 
     &         (MOD(cris1c_h_enddatayr,4) .ne. 0 .and. 
     &         cris1c_h_enddatady .gt. 365)) then
            cris1c_h_enddatady = 1
            cris1c_h_enddatayr = cris1c_h_enddatayr + 1
          endif
        endif
        call ioc1c("WRITE DATA", out1cunit,
     &      cris1c_h_scnlin+cris1c_h_hdrcnt)
      ENDDO

  !Update the header
      call ioc1c("WRITE HEADER", out1cunit, 1)
      write(*,*)'Message:',imessage, ' subsets:',nsubsets,
     &  ' channels:',cris1c_h_nchan,' scan count:',cris1c_h_scnlin

  !If there is a partial scan left over, save the data
      if (spots_to_write .ne. 0) then
        do i=1,array_size_cris1c
          cris1c_dt(i) = cris1c_dt_store(i,scans_to_write+1)
        enddo
      endif

  !Deallocate arrays before exiting the subroutine
      if(allocated(ivalues)) deallocate(ivalues)
      if(allocated(rvalues)) deallocate(rvalues)
      if(allocated(scans)) deallocate(scans)
      if(allocated(spots)) deallocate(spots)
      if(allocated(fovs)) deallocate(fovs)
      if(allocated(fors)) deallocate(fors)
      if(allocated(years)) deallocate(years)
      if(allocated(months)) deallocate(months)
      if(allocated(days)) deallocate(days)
      if(allocated(hours)) deallocate(hours)
      if(allocated(minutes)) deallocate(minutes)
      if(allocated(seconds)) deallocate(seconds)
      if(allocated(cris1c_dt_store)) deallocate(cris1c_dt_store)

#endif
      end subroutine eccodes_get_1c_cris
