      subroutine eccodes_get_1c_iras(ibufr,out1cunit,imessage)
      
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_get_1c_iras.F 602 2018-12-11 10:00:10Z frna $
!
! Purpose: BUFR decoder subroutine for IRAS l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     30/03/2017 Initial
! 1.1     02/10/2017 Correct iras1c_h_instrument
! 1.2     06/11/2017 Support the CMA sequence 310072
! 1.3     27/04/2018 Add inland water category
! 1.4     21/11/2018 orbitNumber can change within a message
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_get
! AAPP subroutines: ioiras1c
!
! Declarations:

#include "iras1c.h"
#include "constants_preproc.h"

!Functions
      integer FNSCALEEC, DAYOFYEAR

!Arguments
      integer ibufr
      integer out1cunit
      integer imessage
      logical nedt_present
      
#ifdef HAS_LIBECCODES

!Variables
      integer   Imiss
      parameter (Imiss=-999999)     !AAPP missing value integer

  !Miscellaneous variables 
      integer i, j, irec, iscan, isub, ispot, ichan, jchan
      integer sensor
      integer nsubsets
      integer scans_to_store
      integer scans_to_write
      integer skipscan
      real*8  rtmp

  !Variables to be saved between calls
      integer, save      :: spots_to_write
      character*10, save :: formatstr(iras_nchan)

  !AAPP data store
      integer, dimension(:,:), allocatable :: iras1c_dt_store
      integer, dimension(:), allocatable   :: scans
      integer, dimension(:), allocatable   :: spots

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      integer*4, dimension(:), allocatable :: descriptors
      character*80 :: descriptor

  !Date/time arrays and scalars
      integer, dimension(:), allocatable :: years
      integer, dimension(:), allocatable :: months
      integer, dimension(:), allocatable :: days
      integer, dimension(:), allocatable :: hours
      integer, dimension(:), allocatable :: minutes
      real*8, dimension(:), allocatable  :: seconds

      real*8  Year,Month,Day      !for passing to DAYOFYEAR
      integer millisecs, jday
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_geo_qc, index_scnlin_qc, 
     & index_cal_qc, index_pixel_qc, index_instrtemp,
     & index_latlon, index_surface, index_angles,
     & index_scalti, index_satposition, index_satvelocity, 
     & index_orbitangle, index_btemps, index_btmap,
     & index_dataqual, index_filler

  !Using the CMA sequence for VASS?
      logical :: USE_VASS_SEQUENCE = .false.

!-----------------------------------------------------------------------

      call codes_get(ibufr,'unexpandedDescriptors',descriptors)
      if (descriptors(1) .eq. 310072) then
        USE_VASS_SEQUENCE = .true.
      endif
      deallocate(descriptors)

      call codes_get(ibufr,'satelliteInstruments',sensor)
      if (sensor .ne. 933) then
        write(*,*) 'Skip message. Sensor=',sensor
        return
      endif

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,iras_nchan
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_iras1c
          iras1c_dt(i) = i
        enddo
        index_scnlin = iras1c_scnlin
        index_scnlinyr = iras1c_scnlinyr
        index_scnlindy = iras1c_scnlindy
        index_scnlintime = iras1c_scnlintime
        index_geo_qc = iras1c_geo_qc
        index_scnlin_qc = iras1c_scnlin_qc
        index_cal_qc = iras1c_cal_qc
        index_pixel_qc = iras1c_pixel_qc(1)
        index_instrtemp = iras1c_instrtemp(1)
        index_latlon = iras1c_latlon(1,1)
        index_surface = iras1c_surface(1,1)
        index_angles = iras1c_angles(1,1)
        index_scalti = iras1c_scalti
        index_satposition = iras1c_satposition(1)
        index_satvelocity = iras1c_satvelocity(1)
        index_orbitangle = iras1c_orbitangle
        index_btemps = iras1c_btemps(1,1)
        index_btmap = iras1c_btmap(1,1)
        index_dataqual = iras1c_dataqual(1)
        index_filler = iras1c_filler(1)
      
!Initialise some header variables
        call codes_get(ibufr,'bufrHeaderCentre',i)
        if (i .ge. 0 .and. i .lt. 999) then
          write(iras1c_h_site,'(I3.3)') i
        else
          iras1c_h_site = "xxx"
        endif
        call codes_get(ibufr,'bufrHeaderSubCentre',i)
        if (i .ge. 0 .and. i .le. 999) then
          write(iras1c_h_1bsite,'(I3.3)') i
        else
          iras1c_h_1bsite = "xxx"
        endif
        iras1c_h_cfill1 = ' '
        iras1c_h_cfill2 = ' '
        iras1c_h_scnlin = 0
        iras1c_h_hdrcnt = 1
        iras1c_h_startdatatime = -1
        iras1c_h_versnb = iras_1cvnnb
        iras1c_h_versyr = iras_1cvnyr
        iras1c_h_versdy = iras_1cvndy
        iras1c_h_instrument = 17       !IRAS
        spots_to_write = 0

        call codes_get(ibufr,'satelliteIdentifier',iras1c_h_wmosatid)
        IF (iras1c_h_wmosatid .eq. CODES_MISSING_LONG) THEN
          iras1c_h_wmosatid = Imiss
          iras1c_h_satid = Imiss
        ELSE
          iras1c_h_satid = iras1c_h_wmosatid-519     !1 for FY-3A
        ENDIF
        !!call codes_get(ibufr,'orbitNumber',iras1c_h_startorbit)
        iras1c_h_satht = 8360   !nominal ht, km*10
        iras1c_h_period = 6120   !period, seconds
        iras1c_h_nchan = iras_nchan
        iras1c_h_nmapchan = iras_nmapchan

  !Open the output file and write header

        call ioiras1c("OPEN/WRITE", out1cunit, 0)
        call ioiras1c("WRITE HEADER", out1cunit, 1)

      endif     !(imessage .eq. 1)

      !!call codes_get(ibufr,'orbitNumber',iras1c_h_endorbit)

  !Read the total number of subsets. Normally an integer multiple of the number
  !of spots, but make the code general in case scans are split across messages. 

      call codes_get(ibufr,'numberOfSubsets',nsubsets)
      scans_to_store = (nsubsets+spots_to_write)/iras_nfovs + 1    !add 1 scan to be safe

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))
      allocate(scans(nsubsets))
      allocate(spots(nsubsets))
      allocate(years(nsubsets))
      allocate(months(nsubsets))
      allocate(days(nsubsets))
      allocate(hours(nsubsets))
      allocate(minutes(nsubsets))
      allocate(seconds(nsubsets))
      allocate(iras1c_dt_store(array_size_iras1c,scans_to_store))
      iras1c_dt_store = 0

  !if there is a partial scan left over from previous message, load the data
  !into the first scan
      if (spots_to_write .ne. 0) then
        do i=1,array_size_iras1c
          iras1c_dt_store(i,1) = iras1c_dt(i)
        enddo
      endif

  !Orbit number
      call codes_get(ibufr,'orbitNumber',ivalues)
      if (imessage .eq. 1) iras1c_h_startorbit = ivalues(1)
      iras1c_h_endorbit = ivalues(nsubsets)

  !FOV number
      call codes_get(ibufr,'fieldOfViewNumber',spots)
      do isub = 1,nsubsets
        if (spots(isub).le.0.or.spots(isub).gt.iras_nfovs) then
          write(*,*)'Error: invalid fov number',spots(isub)
          call aappexit(1)
        endif
      enddo
      if (imessage .eq. 1) then
        if (spots(1).ne.1) then
          skipscan = 1      !ignore initial partial scan if there is one
          spots_to_write = nsubsets + 1 - spots(1)
        else
          skipscan = 0
          spots_to_write = nsubsets
        endif
      else
        skipscan = 0
        spots_to_write = spots_to_write + nsubsets
      endif

  !scan number
      call codes_get(ibufr,'scanLineNumber',ivalues)
      scans(1) = 1
      do isub = 2,nsubsets
        scans(isub) = scans(isub-1)
        if (spots(isub).eq.1) scans(isub) = scans(isub) + 1
      enddo
      do isub = 1,nsubsets
        iras1c_dt_store(index_scnlin,scans(isub)) = ivalues(isub)
      enddo

  !date/time
      call codes_get(ibufr,'year',years)
      call codes_get(ibufr,'month',months)
      call codes_get(ibufr,'day',days)
      call codes_get(ibufr,'hour',hours)
      call codes_get(ibufr,'minute',minutes)
      call codes_get(ibufr,'second',seconds)
        
  !transform the date/time variables
      do isub = 1,nsubsets
        if (spots(isub).eq.1) then
          Year = years(isub)
          Month = months(isub)
          Day = days(isub)
          iras1c_dt_store(index_scnlinyr, scans(isub)) = Year
          jday = DAYOFYEAR(Year,Month,Day)
          iras1c_dt_store(index_scnlindy, scans(isub)) = jday
          millisecs = hours(isub)*3600000 + minutes(isub)*60000 
     &                  + seconds(isub)*1000   
          iras1c_dt_store(index_scnlintime, scans(isub)) = millisecs
        endif
      enddo

  !latitude
      call codes_get(ibufr,'latitude',rvalues)
      do isub = 1,nsubsets
        iras1c_dt_store(index_latlon+2*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !longitude
      call codes_get(ibufr,'longitude',rvalues)
      do isub = 1,nsubsets
        iras1c_dt_store(index_latlon+2*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D4)
      enddo

  !scalti (VASS sequence only)
      if (USE_VASS_SEQUENCE) then
        call codes_get(ibufr,'heightOfStation',rvalues)
        do isub = 1,nsubsets,iras_nfovs
          iras1c_dt_store(index_scalti,scans(isub)) = 
     &        FNSCALEEC(rvalues(isub),1.0D-2)
        enddo
      else
        do isub = 1,nsubsets,iras_nfovs
          iras1c_dt_store(index_scalti,scans(isub)) = Imiss
        enddo
      endif

  !height
      if (USE_VASS_SEQUENCE) then
        call codes_get(ibufr,'nonCoordinateHeight',rvalues)
      else
        call codes_get(ibufr,'height',rvalues)
      endif
      do isub = 1,nsubsets
        iras1c_dt_store(index_surface+2*spots(isub)-2, scans(isub)) = 
     &      FNSCALEEC(rvalues(isub),1.0D0)
      enddo

  !landsea
      if (USE_VASS_SEQUENCE) then
        call codes_get(ibufr,'surfaceFlag',ivalues)   !013040
           !0 Land
           !1 Reserved
           !2 Near coast
           !3 Ice
           !4 Possible ice
           !5 Ocean
           !6 Coast
           !7 Inland water*
           !8 Snow cover
           !9 Sea ice
           !10 Standing water
           !11 Snow
        do isub = 1,nsubsets
          IF (ivalues(isub) .EQ. 5) THEN
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=0  !Sea
          ELSE IF (ivalues(isub) .EQ. 7) THEN
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=3  !Inland water
          ELSE IF (ivalues(isub) .EQ. 2 .or. ivalues(isub) .EQ. 6 .or.
     &          ivalues(isub) .EQ. 9) THEN
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=1  !Mixed
          ELSE IF (ivalues(isub) .EQ. 1 .OR.
     &             ivalues(isub) .EQ. CODES_MISSING_LONG) THEN
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))
     &          =Imiss
          ELSE
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=2  !Land
          ENDIF
        enddo
      else
        call codes_get(ibufr,'landOrSeaQualifier',ivalues)
        do isub = 1,nsubsets
          IF (ivalues(isub) .EQ. 1) THEN
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=0  !Sea
          ELSE IF (ivalues(isub) .EQ. 2) THEN
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=1  !Mixed
          ELSE IF (ivalues(isub) .EQ. 0) THEN
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))=2  !Land
          ELSE
           iras1c_dt_store(index_surface+2*spots(isub)-1,scans(isub))
     &          = Imiss
          ENDIF
        enddo
      endif

  !sat zenith
      call codes_get(ibufr,'satelliteZenithAngle',rvalues)
      do isub = 1,nsubsets
        iras1c_dt_store(index_angles+4*spots(isub)-4, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !sat azimuth (0 to 360 deg)
      call codes_get(ibufr,'bearingOrAzimuth',rvalues)
      do isub = 1,nsubsets
        iras1c_dt_store(index_angles+4*spots(isub)-3, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar zenith
      call codes_get(ibufr,'solarZenithAngle',rvalues)
      do isub = 1,nsubsets
        iras1c_dt_store(index_angles+4*spots(isub)-2, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !solar azimuth (0 to 360 deg)
      call codes_get(ibufr,'solarAzimuth',rvalues)
      do isub = 1,nsubsets
        iras1c_dt_store(index_angles+4*spots(isub)-1, scans(isub))
     &       = FNSCALEEC(rvalues(isub),1.0D2)
      enddo

  !Centre frequency in Hz, or wavelength for the VASS sequence
      if (USE_VASS_SEQUENCE) then
        do ichan=1,iras_nchan
          write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#satelliteChannelWavelength'
          call codes_get(ibufr, descriptor, rvalues)
          iras1c_h_tempradcnv(1,ichan) = 
     &          NINT(1.0D2/rvalues(1))   !10^4*wavenumber cm^-1

          write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient1'
          call codes_get(ibufr, descriptor, rvalues)
          iras1c_h_tempradcnv(2,ichan) = 
     &          NINT(1.0D6*rvalues(1))   !10^6*offset in K

          write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient2'
          call codes_get(ibufr, descriptor, rvalues)
          iras1c_h_tempradcnv(3,ichan) = 
     &          NINT(1.0D6*rvalues(1))   !10^6*slope
        enddo
      else
        do ichan=1,iras_nchan
          write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#satelliteChannelCentreFrequency'
          call codes_get(ibufr, descriptor, rvalues)
          iras1c_h_tempradcnv(1,ichan) = 
     &          NINT(rvalues(1)/C_light*1.0D2)   !10^4*wavenumber cm^-1
          iras1c_h_tempradcnv(2,ichan) = 0
          iras1c_h_tempradcnv(3,ichan) = 1000000
        enddo
      endif

  !btemps (chan,spot,scan)
      do ichan=1,iras_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        call codes_get(ibufr, descriptor, rvalues)
        do isub = 1,nsubsets
          rtmp = rvalues(isub)
          iras1c_dt_store(
     &        index_btemps+ichan-1+iras_nchan*(spots(isub)-1), 
     &        scans(isub)) = FNSCALEEC(rtmp, 1.0D2)
        enddo
      enddo

  !save the data
      scans_to_write = spots_to_write/iras_nfovs
      DO iscan=1,scans_to_write
        spots_to_write = spots_to_write - iras_nfovs
        do i=1,array_size_iras1c
          iras1c_dt(i) = iras1c_dt_store(i,iscan+skipscan)
        enddo
        iras1c_h_scnlin = iras1c_h_scnlin + 1
        if (iras1c_h_startdatatime .lt. 0) then
          iras1c_h_startdatatime = iras1c_scnlintime
          iras1c_h_startdatady = iras1c_scnlindy
          iras1c_h_startdatayr = iras1c_scnlinyr
        endif
        iras1c_h_enddatatime = iras1c_scnlintime
        iras1c_h_enddatady = iras1c_scnlindy
        iras1c_h_enddatayr = iras1c_scnlinyr
        call ioiras1c("WRITE DATA",out1cunit,
     &    iras1c_h_scnlin+iras1c_h_hdrcnt)
      ENDDO

  !Update the header
      call ioiras1c("WRITE HEADER", out1cunit, 1)
      write(*,*)'Message:',imessage, ' subsets:',nsubsets,
     &  ' scan count:',iras1c_h_scnlin


  !If there is a partial scan left over, save the data
      if (spots_to_write .ne. 0) then
        do i=1,array_size_iras1c
          iras1c_dt(i) = iras1c_dt_store(i,scans_to_write+1)
        enddo
      endif

  !Deallocate arrays before exiting the subroutine
      if(allocated(ivalues)) deallocate(ivalues)
      if(allocated(rvalues)) deallocate(rvalues)
      if(allocated(scans)) deallocate(scans)
      if(allocated(spots)) deallocate(spots)
      if(allocated(years)) deallocate(years)
      if(allocated(months)) deallocate(months)
      if(allocated(days)) deallocate(days)
      if(allocated(hours)) deallocate(hours)
      if(allocated(minutes)) deallocate(minutes)
      if(allocated(seconds)) deallocate(seconds)
      if(allocated(iras1c_dt_store)) deallocate(iras1c_dt_store)
#endif

      end subroutine eccodes_get_1c_iras
