      subroutine eccodes_put_1c_atms(ibufr,in1cunit,imessage,iscan,
     &  finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1c_atms.F 592 2018-08-09 14:36:40Z frna $
!
! Purpose: BUFR encoder subroutine for ATMS l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     27/04/2017 Initial
! 1.1     02/08/2018 Skip any scans that do not have valid scan time
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: ioat1c
!
! Declarations:

#include "constants_preproc.h"
#include "inparms.h"
#include "atms1c.h"

!Functions
      REAL*8 FNRSCALEEC

!Arguments
      integer ibufr
      integer in1cunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code (1c in Section 4)
      INTEGER ISUBCENTREID    !Centre code (1c in Section 4)
      INTEGER AAPP_VERSION    !e.g. 715 for v7.15
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer scans_per_message
      PARAMETER (scans_per_message = 3)  !Use several scans for longer messages
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      INTEGER Polarisation(22)
      DATA Polarisation/7,7,13*6,7,6*6/       !7=QV, 6=QH

  !Miscellaneous variables 
      integer i, irec, isub, ispot, ichan, jchan, jscan
      integer number_of_messages
      integer nsubsets
      real*8  rtmp
      real*8  freq

  !Input data store
      integer atms1c_dt_store(array_size_atms1c,scans_per_message)
      integer scans(scans_per_message*atms_1cnfov)
      integer spots(scans_per_message*atms_1cnfov)

  !Date/time arrays
      integer years(atms_1cnfov,scans_per_message)
      integer months(atms_1cnfov,scans_per_message)
      integer days(atms_1cnfov,scans_per_message)
      integer hours(atms_1cnfov,scans_per_message)
      integer minutes(atms_1cnfov,scans_per_message)
      real*8  seconds(atms_1cnfov,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Min
      real*8  Sec
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_orbit,
     & index_granulequal, index_scanqual, 
     & index_geolocqual, index_chanqual,
     & index_latlon, index_angles,
     & index_scalti, index_nedtcold, index_nedtwarm, 
     & index_anttemps, index_btemps

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(atms_1cnchan)

!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,atms_1cnchan
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_atms1c
          atms1c_dt(i) = i
        enddo
        index_scnlin = atms1c_scnlin
        index_scnlinyr = atms1c_scnlinyr
        index_scnlindy = atms1c_scnlindy
        index_scnlintime = atms1c_scnlintime(1)
        index_orbit = atms1c_orbit
        index_granulequal = atms1c_granulequal
        index_scanqual = atms1c_scanqual
        index_geolocqual = atms1c_geolocqual
        index_chanqual = atms1c_chanqual(1)
        index_latlon = atms1c_latlon(1,1)
        index_angles = atms1c_angles(1,1)
        index_scalti = atms1c_scalti
        index_nedtcold = atms1c_nedtcold(1)
        index_nedtwarm = atms1c_nedtwarm(1)
        index_anttemps = atms1c_anttemps(1,1)
        index_btemps = atms1c_btemps(1,1)

!Open the l1c file and read the header

        call ioat1c("OPEN/READ", in1cunit, 0)
        call ioat1c("READ HEADER", in1cunit, 1)

      endif
        
      i = scans_per_message-1
      number_of_messages = (atms1c_h_scnlin+i)/scans_per_message
      write(*,*)'Message:',imessage,' of',number_of_messages
      
      nsubsets = 0
      do jscan=1,scans_per_message
        if (iscan .lt. atms1c_h_scnlin) then
          atms1c_scnlintime(1) = -1

!Find the next scan that has valid scan time

          do while (atms1c_scnlintime(1) .lt. 0 .and. 
     &         iscan .lt. atms1c_h_scnlin)
            iscan = iscan + 1
            irec = iscan + atms1c_h_hdrcnt
            call ioat1c("READ DATA", in1cunit, irec)
          enddo

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,atms_1cnfov
            scnlindy = atms1c_scnlindy
            scnlinyr = atms1c_scnlinyr
            FovTime = atms1c_scnlintime(ispot)
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Min
            seconds(ispot,jscan) = Sec
          enddo

          nsubsets = nsubsets + atms_1cnfov
          do i=1,array_size_atms1c
            atms1c_dt_store(i,jscan) = atms1c_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo

!Section 1 information

      call codes_set(ibufr,'dataCategory',21)               !21 = radiances
      call codes_set(ibufr,'internationalDataSubCategory',6)!6 = ATMS in CCT-C13
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      call codes_set(ibufr,
     & 'inputExtendedDelayedDescriptorReplicationFactor',atms_1cnchan)
      if (AAPP_VERSION .ne. 0) then
        allocate(ivalues(3))
        ivalues=(/ 310061, 25060, 25061 /)    !AAPP and CSPP version numbers
      else  
        allocate(ivalues(1))
        ivalues=(/ 310061 /)
      endif
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      call codes_set(ibufr,'centre',ICENTREID)
      call codes_set(ibufr,'subCentre',ISUBCENTREID)
      call codes_set(ibufr,'satelliteIdentifier',atms1c_h_wmosatid)
      call codes_set(ibufr,'satelliteInstruments',621)    !ATMS
      call codes_set(ibufr,'satelliteClassification',3)
      call codes_set(ibufr,'satelliteAntennaCorrectionsVersionNumber',
     &  FNRSCALEEC(atms1c_h_vnantennacorr,1.0D0))

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = 1,nsubsets
        scans(isub) = (isub+atms_1cnfov-1)/atms_1cnfov
        spots(isub) = MOD(isub-1,atms_1cnfov)+1
      enddo

!Encode the data variables

  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = 
     &     MOD(atms1c_dt_store(index_scnlin,scans(isub)),255)   !8 bits limit
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = spots(isub)
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !granulequal
      do isub = 1,nsubsets
        ivalues(isub) = atms1c_dt_store(index_granulequal,scans(isub))
      enddo
      call codes_set(ibufr,'granuleLevelQualityFlags',ivalues)
        
  !scanqual
      do isub = 1,nsubsets
        ivalues(isub) = atms1c_dt_store(index_scanqual,scans(isub))
      enddo
      call codes_set(ibufr,'scanLevelQualityFlags',ivalues)
        
  !geolocqual
      do isub = 1,nsubsets
        ivalues(isub) = atms1c_dt_store(index_geolocqual,scans(isub))
      enddo
      call codes_set(ibufr,'geolocationQuality',ivalues)
        
  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        rvalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',rvalues)
        
  !Orbit
      do isub = 1,nsubsets
        rvalues(isub) = FNRSCALEEC(atms1c_dt_store(index_orbit,
     &        scans(isub)), 1.0D0)
      enddo
      call codes_set(ibufr,'orbitNumber',rvalues)

  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(atms1c_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)
        
  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(atms1c_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !scalti
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(atms1c_dt_store(index_scalti,
     &        scans(isub)), 1.0D2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'height',rvalues)
        
  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(atms1c_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(atms1c_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(atms1c_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(atms1c_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !channel number
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelNumber'
        call codes_set(ibufr, descriptor, ichan)
      enddo

  !centre frequency
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#satelliteChannelCentreFrequency'
        if (atms1c_h_tempradcnv(1,ichan).NE.Imiss) THEN
          freq = C_light*atms1c_h_tempradcnv(1,ichan)*1.0E-6*100
        else
          freq = CODES_MISSING_DOUBLE
        endif
        call codes_set(ibufr, descriptor,freq)
      enddo

  !bandwidth
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#satelliteChannelBandWidth'
        call codes_set(ibufr, descriptor, 
     &       FNRSCALEEC(atms1c_h_bandwidth(ichan),1.0D3))
      enddo

  !polarisation
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#antennaPolarization'
        call codes_set(ibufr, descriptor, Polarisation(ichan))
      enddo

  !chanqual
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelDataQualityFlags'
        do isub = 1,nsubsets
          i = atms1c_dt_store(index_chanqual+ichan-1, scans(isub))
          ivalues(isub) = i
        enddo   
        call codes_set(ibufr,descriptor,ivalues)
      enddo

  !anttemps (chan,spot,scan)
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#antennaTemperature'
        do isub = 1,nsubsets
          rtmp = FNRSCALEEC(atms1c_dt_store(index_anttemps + 
     &     ichan-1 + atms_1cnchan*(spots(isub)-1), scans(isub)), 1.0D-2)
          rvalues(isub) = rtmp
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !btemps (chan,spot,scan)
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        do isub = 1,nsubsets
          rtmp = FNRSCALEEC(atms1c_dt_store(index_btemps + 
     &     ichan-1 + atms_1cnchan*(spots(isub)-1), scans(isub)), 1.0D-2)
          rvalues(isub) = rtmp
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !nedtcold
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#noiseEquivalentDeltaTemperatureWhileViewingColdTarget'
        do isub = 1,nsubsets
          if (atms1c_nedtcold(ichan) .GE. 0) THEN
            rtmp = FNRSCALEEC(atms1c_dt_store(index_nedtcold + ichan-1,
     &        scans(isub)), 1.0D-4)
            rvalues(isub) = rtmp
          else
            rvalues(isub) = CODES_MISSING_DOUBLE
          endif
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !nedtwarm
      do ichan=1,atms_1cnchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#noiseEquivalentDeltaTemperatureWhileViewingWarmTarget'
        do isub = 1,nsubsets
          if (atms1c_nedtwarm(ichan) .GE. 0) THEN
            rtmp = FNRSCALEEC(atms1c_dt_store(index_nedtwarm + ichan-1,
     &        scans(isub)), 1.0D-4)
            rvalues(isub) = rtmp
          else
            rvalues(isub) = CODES_MISSING_DOUBLE
          endif
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !AAPP and CSPP versions
      if (AAPP_VERSION .NE. 0) then
        call codes_set(ibufr,'softwareIdentification',AAPP_VERSION)
        call codes_set(ibufr,'softwareVersionNumber',
     &     atms1c_h_software_version)
      endif

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (iscan .eq. atms1c_h_scnlin) then
        finished = .true.
        call ioat1c("CLOSE", in1cunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1c_atms
