      subroutine eccodes_put_1c_iras(ibufr,in1cunit,imessage,iscan,
     &  finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1c_iras.F 586 2018-04-30 10:32:45Z frna $
!
! Purpose: BUFR encoder subroutine for IRAS l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     30/03/2017 Initial
! 1.1     03/11/2017 Support the CMA sequence 310072, proposed at IPET-CM in
!                    July 2017, via environment variable USE_VASS_SEQUENCE=Y
! 1.2     12/02/2018 Change tovsOrAtovsProductQualifier to 
!                    verticalSoundingProductQualifier and 
!                    bandwidthCorrectionCoefficient1ForAtovs to
!                    bandwidthCorrectionCoefficient1 (for MASTER_TABLE >=29)
! 1.3     27/04/2018 Add inland water category
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: ioiras1c
!
! Declarations:

#include "inparms.h"
#include "iras1c.h"
#include "constants_preproc.h"

!Functions
      REAL*8 FNRSCALEEC

!Arguments
      integer ibufr
      integer in1cunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code (1c in Section 4)
      INTEGER ISUBCENTREID    !Centre code (1c in Section 4)
      INTEGER AAPP_VERSION    !e.g. 715 for v7.15
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer TimeDelay
      DATA TimeDelay/100/      !milliseconds per spot, assume same as HIRS
      integer scans_per_message
      PARAMETER (scans_per_message = 10)  !Use several scans for longer messages

  !Miscellaneous variables 
      integer i, irec, isub, ispot, ichan, jchan, jscan
      integer number_of_messages
      integer nsubsets
      real*8  rtmp

  !Input data store
      integer iras1c_dt_store(array_size_iras1c,scans_per_message)
      integer scans(scans_per_message*iras_nfovs)
      integer spots(scans_per_message*iras_nfovs)

  !Date/time arrays
      integer years(iras_nfovs,scans_per_message)
      integer months(iras_nfovs,scans_per_message)
      integer days(iras_nfovs,scans_per_message)
      integer hours(iras_nfovs,scans_per_message)
      integer minutes(iras_nfovs,scans_per_message)
      real*8  seconds(iras_nfovs,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Min
      real*8  Sec
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_geo_qc, index_scnlin_qc, 
     & index_cal_qc, index_pixel_qc, index_instrtemp,
     & index_latlon, index_surface, index_angles,
     & index_scalti, index_satposition, index_satvelocity, 
     & index_orbitangle, index_btemps, index_btmap,
     & index_dataqual, index_filler

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(2*iras_nchan)

  !Use the CMA sequence for VASS?
      character*1 :: CUSE_VASS_SEQUENCE = 'n'
      logical, save :: USE_VASS_SEQUENCE = .false.

!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

        CALL GETENV('USE_VASS_SEQUENCE',CUSE_VASS_SEQUENCE)
        if (CUSE_VASS_SEQUENCE(1:1) .eq. 'Y' .or. 
     &      CUSE_VASS_SEQUENCE(1:1) .eq. 'y') then
          USE_VASS_SEQUENCE = .true.
        endif

!Format string to use with multi-channel descriptors
        do ichan=1,9
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo
        do ichan=10,2*iras_nchan
          write(formatstr(ichan),'(A)') '(A,I2,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_iras1c
          iras1c_dt(i) = i
        enddo
        index_scnlin = iras1c_scnlin
        index_scnlinyr = iras1c_scnlinyr
        index_scnlindy = iras1c_scnlindy
        index_scnlintime = iras1c_scnlintime
        index_geo_qc = iras1c_geo_qc
        index_scnlin_qc = iras1c_scnlin_qc
        index_cal_qc = iras1c_cal_qc
        index_pixel_qc = iras1c_pixel_qc(1)
        index_instrtemp = iras1c_instrtemp(1)
        index_latlon = iras1c_latlon(1,1)
        index_surface = iras1c_surface(1,1)
        index_angles = iras1c_angles(1,1)
        index_scalti = iras1c_scalti
        index_satposition = iras1c_satposition(1)
        index_satvelocity = iras1c_satvelocity(1)
        index_orbitangle = iras1c_orbitangle
        index_btemps = iras1c_btemps(1,1)
        index_btmap = iras1c_btmap(1,1)
        index_dataqual = iras1c_dataqual(1)
        index_filler = iras1c_filler(1)
      
!Open the l1c file and read the header

        call ioiras1c("OPEN/READ", in1cunit, 0)
        call ioiras1c("READ HEADER", in1cunit, 1)

      endif
        
      i = scans_per_message-1
      number_of_messages = (iras1c_h_scnlin+i)/scans_per_message
      write(*,*)'Message:',imessage,' of',number_of_messages
      
      nsubsets = 0
      do jscan=1,scans_per_message
        if (iscan .lt. iras1c_h_scnlin) then
          iscan = iscan + 1
          irec = iscan + iras1c_h_hdrcnt
          call ioiras1c("READ DATA", in1cunit, irec)

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,iras_nfovs
            scnlindy = iras1c_scnlindy
            scnlinyr = iras1c_scnlinyr
            FovTime = iras1c_scnlintime + (ispot-1)*TimeDelay
            if (.not. USE_VASS_SEQUENCE) then
              FovTime = ((FovTime + 500)/1000)*1000    !Round to nearest second
            endif
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Min
            seconds(ispot,jscan) = Sec
          enddo

          nsubsets = nsubsets + iras_nfovs
          do i=1,array_size_iras1c
            iras1c_dt_store(i,jscan) = iras1c_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo

!Section 1 information

      allocate(ivalues(1))
      ivalues=(/ iras_nchan /)
      call codes_set(ibufr,'inputDelayedDescriptorReplicationFactor',
     &  ivalues)
      deallocate(ivalues)

      call codes_set(ibufr,'dataCategory',3)                  !Table C-13
      if (USE_VASS_SEQUENCE) then
        call codes_set(ibufr,'internationalDataSubCategory',8)  !VASS
      else
        call codes_set(ibufr,'internationalDataSubCategory',20) !I/R
      endif
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      if (USE_VASS_SEQUENCE) then
        allocate(ivalues(1))
        ivalues=(/ 310072 /)
      else if (AAPP_VERSION .ne. 0) then
        allocate(ivalues(29))
        ivalues=(/
     &    1033, 1034, 1007, 2019, 5040,
     &    201136, 5041, 201000, 5043, 301011,
     &    301013, 301021, 7002, 8012, 7024,
     &    5021, 7025, 5022, 108000, 31001,
     &    5042, 2153, 2154, 201132, 202129,
     &    12063, 202000, 201000, 25060 /)
      else  
        allocate(ivalues(28))
        ivalues=(/
     &    1033, 1034, 1007, 2019, 5040,
     &    201136, 5041, 201000, 5043, 301011,
     &    301013, 301021, 7002, 8012, 7024,
     &    5021, 7025, 5022, 108000, 31001,
     &    5042, 2153, 2154, 201132, 202129,
     &    12063, 202000, 201000 /)
      endif
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      if (USE_VASS_SEQUENCE) then
        call codes_set(ibufr,'verticalSoundingProductQualifier',3)
      endif
      call codes_set(ibufr,'centre',ICENTREID)
      call codes_set(ibufr,'subCentre',ISUBCENTREID)
      call codes_set(ibufr,'satelliteIdentifier',iras1c_h_wmosatid)
      call codes_set(ibufr,'satelliteInstruments',933)
      call codes_set(ibufr,'orbitNumber',iras1c_h_startorbit)

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = 1,nsubsets
        scans(isub) = (isub+iras_nfovs-1)/iras_nfovs
        spots(isub) = MOD(isub-1,iras_nfovs)+1
      enddo

!Encode the data variables

  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = iras1c_dt_store(index_scnlin,scans(isub))
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = spots(isub)
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        rvalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',rvalues)
        
  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iras1c_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)
        
  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iras1c_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !Satellite height (VASS sequence only)
      if (USE_VASS_SEQUENCE) then
        do isub = 1,nsubsets
          i = iras1c_dt_store(index_scalti, scans(isub))
          if (i .eq. -999999) then
            rvalues(isub) = 832000    !default (as used by CMA)
          else
            rvalues(isub) = FNRSCALEEC(i, 1.0D2)
          endif
        enddo
        call codes_set(ibufr,'heightOfStation',rvalues)
      endif

  !Surface height
      do isub = 1,nsubsets
        i = iras1c_dt_store(index_surface+2*spots(isub)-2, scans(isub))
        rvalues(isub) = FNRSCALEEC(i, 1.0D0)
      enddo
      if (USE_VASS_SEQUENCE) then
        call codes_set(ibufr,'nonCoordinateHeight',rvalues)
      else
        call codes_set(ibufr,'height',rvalues)
      endif

  !Surface type AAPP 0=sea, 1=mixed, 2=land, 3=inland water; 013040 0=land, 5=sea, 6=coast, 7=inland water
      if (USE_VASS_SEQUENCE) then
        do isub = 1,nsubsets
          i=iras1c_dt_store(index_surface+2*spots(isub)-1, scans(isub))
          IF (i .EQ. 0) THEN
            ivalues(isub) = 5
          ELSE IF (i .EQ. 1) THEN
            ivalues(isub) = 6
          ELSE IF (i .EQ. 2) THEN
            ivalues(isub) = 0
          ELSE IF (i .EQ. 3) THEN
            ivalues(isub) = 7
          ELSE
            ivalues(isub) = CODES_MISSING_LONG
          ENDIF
        enddo
        call codes_set(ibufr,'surfaceFlag',ivalues)

  !Surface type AAPP 0=sea, 1=mixed, 2=land, 3=inland water; 008012 0=land, 1=sea, 2=coast
      else
        do isub = 1,nsubsets
          i=iras1c_dt_store(index_surface+2*spots(isub)-1, scans(isub))
          IF (i .EQ. 0) THEN
            ivalues(isub) = 1
          ELSE IF (i .EQ. 1) THEN
            ivalues(isub) = 2
          ELSE IF (i .EQ. 2) THEN
            ivalues(isub) = 0
          ELSE IF (i .EQ. 3) THEN
            ivalues(isub) = 2    !encode inland water as coast
          ELSE
            ivalues(isub) = CODES_MISSING_LONG
          ENDIF
        enddo
        call codes_set(ibufr,'landOrSeaQualifier',ivalues)
      endif

  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iras1c_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iras1c_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iras1c_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(iras1c_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !channel number
      do ichan=1,iras_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelNumber'
        call codes_set(ibufr, descriptor, ichan)
      enddo

  !Centre frequency in Hz, or wavelength for the VASS sequence
      if (USE_VASS_SEQUENCE) then
        do ichan=1,iras_nchan
          write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#satelliteChannelWavelength'
          call codes_set(ibufr, descriptor, 
     &      1.0D0/(iras1c_h_tempradcnv(1,ichan)*1.0D-2))
        enddo
      else
        do ichan=1,iras_nchan
          write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#satelliteChannelCentreFrequency'
          call codes_set(ibufr, descriptor, 
     &      C_light*iras1c_h_tempradcnv(1,ichan)*1.0D-2)
        enddo
      endif

  !Bandwidth correction coefficients (VASS sequence)
      if (USE_VASS_SEQUENCE) then
        do ichan=1,iras_nchan
          write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient1'
          call codes_set(ibufr, descriptor, 
     &       1.0D-6*iras1c_h_tempradcnv(2,ichan))
        enddo
        do ichan=1,iras_nchan
          write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#bandwidthCorrectionCoefficient2'
          call codes_set(ibufr, descriptor, 
     &       1.0D-6*iras1c_h_tempradcnv(3,ichan))
        enddo
      endif

  !btemps (chan,spot,scan)
      do ichan=1,iras_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        do isub = 1,nsubsets
          i = iras1c_dt_store(index_btemps + 
     &        ichan-1+iras_nchan*(spots(isub)-1),scans(isub))
          if (i .ge. 0 .and. i .lt. 65535) then
            rvalues(isub) = i*1.0D-2
          else
            rvalues(isub) = CODES_MISSING_DOUBLE
          endif
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !AAPP version
      if (AAPP_VERSION .NE. 0 .and. .not. USE_VASS_SEQUENCE)
     &    call codes_set(ibufr,'softwareIdentification',AAPP_VERSION)

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (iscan .eq. iras1c_h_scnlin) then
        finished = .true.
        call ioiras1c("CLOSE", in1cunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1c_iras
