      subroutine eccodes_put_1c_mwts(ibufr,in1cunit,imessage,iscan,
     &  finished,ICENTREID,ISUBCENTREID,AAPP_VERSION)

#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_put_1c_mwts.F 536 2017-08-09 10:40:30Z frna $
!
! Purpose: BUFR encoder subroutine for MWTS l1c
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     12/05/2017 Initial
!
! Code Description:
!   Fortran90, but using FORTRAN77 fixed-formatting, in order to interface with
!   existing AAPP "include" files.
!
! ecCodes subroutines: codes_set
! AAPP subroutines: iomwts1c
!
! Declarations:

#include "inparms.h"
#include "mwts1c.h"
#include "constants_preproc.h"

!Functions
      REAL*8 FNRSCALEEC

!Arguments
      integer ibufr
      integer in1cunit
      integer imessage
      integer iscan
      logical finished
      INTEGER ICENTREID       !Centre code (1c in Section 4)
      INTEGER ISUBCENTREID    !Centre code (1c in Section 4)
      INTEGER AAPP_VERSION    !e.g. 715 for v7.15
      
#ifdef HAS_LIBECCODES

  !Parameters and fixed items
      integer TimeDelay
      DATA TimeDelay/800/      !milliseconds per spot, guess
                               !(16 sec for 15 FOVs + cal views + slew)
      integer scans_per_message
      PARAMETER (scans_per_message = 50)  !Use several scans for longer messages

  !Miscellaneous variables 
      integer i, irec, isub, ispot, ichan, jchan, jscan
      integer number_of_messages
      integer nsubsets
      real*8  rtmp

  !Input data store
      integer mwts1c_dt_store(array_size_mwts1c,scans_per_message)
      integer scans(scans_per_message*mwts_nfovs)
      integer spots(scans_per_message*mwts_nfovs)

  !Date/time arrays
      integer years(mwts_nfovs,scans_per_message)
      integer months(mwts_nfovs,scans_per_message)
      integer days(mwts_nfovs,scans_per_message)
      integer hours(mwts_nfovs,scans_per_message)
      integer minutes(mwts_nfovs,scans_per_message)
      real*8  seconds(mwts_nfovs,scans_per_message)
      integer scnlindy,scnlinyr,FovTime  !date and time
      integer Month,Day,Hour,Min
      real*8  Sec
      
  !Index variables for AAPP l1c array
      integer, save :: index_scnlin, index_scnlinyr, index_scnlindy,
     & index_scnlintime, index_qualind, index_scanqual, 
     & index_chanqual, index_instrtemp1, index_instrtemp2,
     & index_spare1, index_latlon, index_surface, index_angles,
     & index_scalti, index_spare2, index_btemps,
     & index_dataqual, index_filler

  !Arrays that are passed to eccodes
      real*8,    dimension(:), allocatable :: rvalues
      integer*4, dimension(:), allocatable :: ivalues
      character*80 descriptor
      
  !Variables to be saved between calls
      character*10, save :: formatstr(mwts_nchan)
      
!--------------------------------------------------------------

      imessage = imessage + 1

      if (imessage .eq. 1) then

!Format string to use with multi-channel descriptors
        do ichan=1,mwts_nchan
          write(formatstr(ichan),'(A)') '(A,I1,A)'
        enddo

!Find the position in the AAPP data record of each variable
        do i=1,array_size_mwts1c
          mwts1c_dt(i) = i
        enddo
        index_scnlin = mwts1c_scnlin
        index_scnlinyr = mwts1c_scnlinyr
        index_scnlindy = mwts1c_scnlindy
        index_scnlintime = mwts1c_scnlintime
        index_qualind = mwts1c_qualind
        index_scanqual = mwts1c_scanqual
        index_chanqual = mwts1c_chanqual(1)
        index_instrtemp1 = mwts1c_instrtemp1
        index_instrtemp2 = mwts1c_instrtemp2
        index_spare1 = mwts1c_spare1(1)
        index_latlon = mwts1c_latlon(1,1)
        index_surface = mwts1c_surface(1,1)
        index_angles = mwts1c_angles(1,1)
        index_scalti = mwts1c_scalti
        index_spare2 = mwts1c_spare2(1)
        index_btemps = mwts1c_btemps(1,1)
        index_dataqual = mwts1c_dataqual(1)
        index_filler = mwts1c_filler(1)
      
!Open the l1c file and read the header

        call iomwts1c("OPEN/READ", in1cunit, 0)
        call iomwts1c("READ HEADER", in1cunit, 1)

      endif
        
      i = scans_per_message-1
      number_of_messages = (mwts1c_h_scnlin+i)/scans_per_message
      write(*,*)'Message:',imessage,' of',number_of_messages
      
      nsubsets = 0
      do jscan=1,scans_per_message
        if (iscan .lt. mwts1c_h_scnlin) then
          iscan = iscan + 1
          irec = iscan + mwts1c_h_hdrcnt
          call iomwts1c("READ DATA", in1cunit, irec)

          !Convert year, day_of_year, milliseconds to YYMMDD HHMMSS

          do ispot=1,mwts_nfovs
            scnlindy = mwts1c_scnlindy
            scnlinyr = mwts1c_scnlinyr
            FovTime = mwts1c_scnlintime + (ispot-1)*TimeDelay
            FovTime = ((FovTime + 500)/1000)*1000    !Round to nearest second
            CALL DAY_TO_DATE(scnlinyr,scnlindy,FovTime,
     &           Month,Day,Hour,Min,Sec)            
            years(ispot,jscan) = scnlinyr
            months(ispot,jscan) = Month
            days(ispot,jscan) = Day
            hours(ispot,jscan) = Hour
            minutes(ispot,jscan) = Min
            seconds(ispot,jscan) = Sec
          enddo

          nsubsets = nsubsets + mwts_nfovs
          do i=1,array_size_mwts1c
            mwts1c_dt_store(i,jscan) = mwts1c_dt(i)
          enddo
          irec = irec + 1
        endif
      enddo

!Section 1 information

      allocate(ivalues(1))
      ivalues=(/ mwts_nchan /)
      call codes_set(ibufr,'inputDelayedDescriptorReplicationFactor',
     &  ivalues)
      deallocate(ivalues)

      call codes_set(ibufr,'dataCategory',3)                  !Table C-13
      call codes_set(ibufr,'internationalDataSubCategory',40) !microwave
      call codes_set(ibufr,'typicalYear',years(1,1))
      call codes_set(ibufr,'typicalMonth',months(1,1))
      call codes_set(ibufr,'typicalDay',days(1,1))
      call codes_set(ibufr,'typicalHour',hours(1,1))
      call codes_set(ibufr,'typicalMinute',minutes(1,1))
      call codes_set(ibufr,'typicalSecond',seconds(1,1))
      call codes_set(ibufr,'numberOfSubsets',nsubsets)
      if (AAPP_VERSION .ne. 0) then
        allocate(ivalues(29))
        ivalues=(/
     &    1033, 1034, 1007, 2019, 5040,
     &    201136, 5041, 201000, 5043, 301011,
     &    301013, 301021, 7002, 8012, 7024,
     &    5021, 7025, 5022, 108000, 31001,
     &    5042, 2153, 2154, 201132, 202129,
     &    12063, 202000, 201000, 25060 /)
      else  
        allocate(ivalues(28))
        ivalues=(/
     &    1033, 1034, 1007, 2019, 5040,
     &    201136, 5041, 201000, 5043, 301011,
     &    301013, 301021, 7002, 8012, 7024,
     &    5021, 7025, 5022, 108000, 31001,
     &    5042, 2153, 2154, 201132, 202129,
     &    12063, 202000, 201000 /)
      endif
      call codes_set(ibufr,'unexpandedDescriptors',ivalues)
      deallocate(ivalues)
      call codes_set(ibufr,'centre',ICENTREID)
      call codes_set(ibufr,'subCentre',ISUBCENTREID)
      call codes_set(ibufr,'satelliteIdentifier',mwts1c_h_wmosatid)
      call codes_set(ibufr,'satelliteInstruments',934)
      call codes_set(ibufr,'orbitNumber',mwts1c_h_startorbit)

      allocate(ivalues(nsubsets))
      allocate(rvalues(nsubsets))

      do isub = 1,nsubsets
        scans(isub) = (isub+mwts_nfovs-1)/mwts_nfovs
        spots(isub) = MOD(isub-1,mwts_nfovs)+1
      enddo

!Encode the data variables

  !Scan number
      do isub = 1,nsubsets
        ivalues(isub) = mwts1c_dt_store(index_scnlin,scans(isub))
      enddo
      call codes_set(ibufr,'scanLineNumber',ivalues)
        
  !FOV number
      do isub = 1,nsubsets
        ivalues(isub) = spots(isub)
      enddo
      call codes_set(ibufr,'fieldOfViewNumber',ivalues)
        
  !year
      do isub = 1,nsubsets
        ivalues(isub) = years(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'year',ivalues)
        
  !month
      do isub = 1,nsubsets
        ivalues(isub) = months(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'month',ivalues)
        
  !day
      do isub = 1,nsubsets
        ivalues(isub) = days(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'day',ivalues)
        
  !hour
      do isub = 1,nsubsets
        ivalues(isub) = hours(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'hour',ivalues)
        
  !minute
      do isub = 1,nsubsets
        ivalues(isub) = minutes(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'minute',ivalues)
        
  !second
      do isub = 1,nsubsets
        rvalues(isub) = seconds(spots(isub),scans(isub))
      enddo
      call codes_set(ibufr,'second',rvalues)
        
  !latitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwts1c_dt_store(index_latlon+2*spots(isub)-2,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'latitude',rvalues)
        
  !longitude
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwts1c_dt_store(index_latlon+2*spots(isub)-1,
     &        scans(isub)), 1.0D-4)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'longitude',rvalues)
        
  !Surface height
      do isub = 1,nsubsets
        i = mwts1c_dt_store(index_surface+2*spots(isub)-2, scans(isub))
        rvalues(isub) = FNRSCALEEC(i, 1.0D0)
      enddo
      call codes_set(ibufr,'height',rvalues)

  !Surface type AAPP 0=sea, 1=mixed, 2=land; 008012 0=land, 1=sea, 2=coast
      do isub = 1,nsubsets
        i = mwts1c_dt_store(index_surface+2*spots(isub)-1, scans(isub))
        IF (i .EQ. 0) THEN
          ivalues(isub) = 1
        ELSE IF (i .EQ. 1) THEN
          ivalues(isub) = 2
        ELSE IF (i .EQ. 2) THEN
          ivalues(isub) = 0
        ELSE
          ivalues(isub) = CODES_MISSING_LONG
        ENDIF
      enddo
      call codes_set(ibufr,'landOrSeaQualifier',ivalues)

  !sat zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwts1c_dt_store(index_angles+4*spots(isub)-4,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'satelliteZenithAngle',rvalues)
        
  !sat azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwts1c_dt_store(index_angles+4*spots(isub)-3,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'bearingOrAzimuth',rvalues)
        
  !solar zenith
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwts1c_dt_store(index_angles+4*spots(isub)-2,
     &        scans(isub)), 1.0D-2)
        rvalues(isub) = rtmp
      enddo
      call codes_set(ibufr,'solarZenithAngle',rvalues)
        
  !solar azimuth
      do isub = 1,nsubsets
        rtmp = FNRSCALEEC(mwts1c_dt_store(index_angles+4*spots(isub)-1,
     &        scans(isub)), 1.0D-2)
        if (rtmp.LT.0 .and. rtmp.GT.-360) rtmp = rtmp + 360.0D0
        rvalues(isub) = rtmp 
      enddo
      call codes_set(ibufr,'solarAzimuth',rvalues)

  !channel number
      do ichan=1,mwts_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#channelNumber'
        call codes_set(ibufr, descriptor, ichan)
      enddo

  !Bandwidth in Hz
      do ichan=1,mwts_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#satelliteChannelBandWidth'
        call codes_set(ibufr, descriptor, 
     &       FNRSCALEEC(mwts1c_h_bandwidth(ichan),1.0D3))
      enddo

  !Centre frequency in Hz
      do ichan=1,mwts_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#satelliteChannelCentreFrequency'
        call codes_set(ibufr, descriptor, 
     &    C_light*mwts1c_h_tempradcnv(1,ichan)*1.0D-4)
      enddo

  !btemps (chan,spot,scan)
      do ichan=1,mwts_nchan
        write(descriptor,formatstr(ichan)) '#',ichan,
     &       '#brightnessTemperature'
        do isub = 1,nsubsets
          i = mwts1c_dt_store(index_btemps + 
     &        ichan-1+mwts_nchan*(spots(isub)-1),scans(isub))
          if (i .ge. 0 .and. i .lt. 65535) then
            rvalues(isub) = i*1.0D-2
          else
            rvalues(isub) = CODES_MISSING_DOUBLE
          endif
        enddo
        call codes_set(ibufr,descriptor,rvalues)
      enddo

  !AAPP version
      if (AAPP_VERSION .NE. 0)
     &    call codes_set(ibufr,'softwareIdentification',AAPP_VERSION)

  !Deallocate arrays before exiting the subroutine
      deallocate(ivalues)
      deallocate(rvalues)

  !Have we finished?
      if (iscan .eq. mwts1c_h_scnlin) then
        finished = .true.
        call iomwts1c("CLOSE", in1cunit, 0)
      endif

#endif
      RETURN
      END subroutine eccodes_put_1c_mwts
