!-----------------------------------------------------------------------
!
! Name: $Id: eccodes_utils.F 519 2017-05-05 10:04:04Z frna $
!
! Purpose: Utility functions/subroutines for ecCodes BUFR tools
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 December 2016, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2017, EUMETSAT, All Rights Reserved.
!
! History:
! Version  Date      Comment
! 1.0     30/03/2017 Initial
!----------------------------------------------------------------------

      FUNCTION FNSCALEEC(Rval,Scale)   !Used in decoding real values
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

      INTEGER FNSCALEEC
      REAL*8 Rval
      REAL*8 Scale
      
#ifdef HAS_LIBECCODES
      REAL*8    Rmiss
      INTEGER   Imiss
      PARAMETER (Rmiss=CODES_MISSING_DOUBLE)
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      IF (Rval.EQ.Rmiss) THEN
        FNSCALEEC = Imiss
      ELSE
        FNSCALEEC = NINT(Rval*Scale)
      ENDIF     
#endif
      RETURN
      END

!------------------------------------------------------------------------

      FUNCTION FNISCALEEC(Ival,Scale)   !Used in decoding integers
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

      INTEGER FNISCALEEC
      INTEGER Ival
      INTEGER Scale
      
#ifdef HAS_LIBECCODES
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      IF (Ival.EQ.CODES_MISSING_LONG) THEN
        FNISCALEEC = Imiss
      ELSE
        FNISCALEEC = Ival*Scale
      ENDIF
#endif
      RETURN
      END

!------------------------------------------------------------------------

      FUNCTION FNJSCALEEC(Ival,Scale)   !Used in encoding to create integers
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

      INTEGER FNJSCALEEC
      INTEGER Ival
      INTEGER Scale
      
#ifdef HAS_LIBECCODES
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      IF (Ival.EQ.Imiss) THEN
        FNJSCALEEC = CODES_MISSING_LONG
      ELSE
        FNJSCALEEC = Ival*Scale
      ENDIF
#endif
      RETURN
      END

!------------------------------------------------------------------------

      FUNCTION FNRSCALEEC(Ival,Scale)  !Used in encoding to create reals
#ifdef HAS_LIBECCODES
      use eccodes
#endif
      implicit none

      REAL*8  FNRSCALEEC
      INTEGER Ival
      REAL*8  Scale
      
#ifdef HAS_LIBECCODES
      INTEGER   Imiss
      PARAMETER (Imiss=-999999)     !AAPP missing value integer
      
      IF (Ival.EQ.Imiss) THEN
        FNRSCALEEC = CODES_MISSING_DOUBLE
      ELSE
        FNRSCALEEC = Ival*Scale
      ENDIF
#endif
      RETURN
      END

!-----------------------------------------------------------------------

      FUNCTION DAYOFYEAR(Y,M,D)
      INTEGER    DAYOFYEAR
      REAL*8     Y,M,D
      INTEGER    IY,IM,ID
      INTEGER    NOLEAP(12), LEAP(12)
      DATA       NOLEAP /0,31,59,90,120,151,181,212,243,273,304,334/
      DATA       LEAP   /0,31,60,91,121,152,182,213,244,274,305,335/
      IY = Y
      IM = M
      ID = D
      IF (MOD(IY,4).EQ.0) THEN         !OK up to 2100
        DAYOFYEAR = LEAP(IM) + ID
      ELSE
        DAYOFYEAR = NOLEAP(IM) + ID
      ENDIF
      RETURN
      END

!-----------------------------------------------------------------------

      SUBROUTINE DAY_TO_DATE(Year,DayOfYear,Millisecs,
     &               Month,Day,Hour,Min,Sec)

      IMPLICIT NONE
      INTEGER    Year,DayOfYear,Millisecs    !Inputs
      INTEGER    Month,Day,Hour,Min          !Outputs
      REAL*8     Sec                         !Output
      INTEGER    I                           !Loop index
      INTEGER    Maxdays                     !Days in a year
      INTEGER    NOLEAP(13), LEAP(13)
      DATA       NOLEAP /0,31,59,90,120,151,181,212,243,273,304,334,365/
      DATA       LEAP   /0,31,60,91,121,152,182,213,244,274,305,335,366/
      LOGICAL    Leapyear

      IF (MOD(Year,4).EQ.0) THEN  !OK up to 2100
        Leapyear = .TRUE.
        Maxdays = 366
      ELSE
        Leapyear = .FALSE.
        Maxdays = 365
      ENDIF          
      
      IF (Millisecs.GE.86400000) THEN       !Wrap around - add 1 day
        Millisecs = Millisecs - 86400000
        DayOfYear = DayOfYear + 1
        IF (DayOfYear.GT.Maxdays) THEN
          Year = Year + 1
          DayOfYear = 1
        ENDIF
      ENDIF
      
      Month = 0
      IF (Leapyear) THEN
        DO I=2,13
          IF (LEAP(I).GE.DayOfYear.AND.Month.EQ.0) THEN
            Month = I-1
            Day = DayOfYear - LEAP(Month)
          ENDIF
        ENDDO
      ELSE
        DO I=2,13
          IF (NOLEAP(I).GE.DayOfYear.AND.Month.EQ.0) THEN
            Month = I-1
            Day = DayOfYear - NOLEAP(Month)
          ENDIF
        ENDDO
      ENDIF
      Hour = Millisecs/3600000
      Min = Millisecs/60000 - Hour*60
      Sec = MOD(Millisecs/1000.0D0,60.0D0)
      RETURN
      END

!-----------------------------------------------------------------------

      SUBROUTINE BitSwap(BitIn, Mask, NumBitsIn, BitOut, NumBitsOut)
      IMPLICIT NONE

! purpose         : To take an input bit field (BitIn) and set the bits
!                 : in the output bit field (BitOut) determined by the
!                 : mapping in the Mask field.                     
!                                                                               
! calls           : BTEST  : To see if a bit is set
!                 : IBSET  : To set a bit
!                                                                               
! Description of Mask :
! 
! The mask is of the size of the input flag (bit) table. Each 
! element in the mask is the bit in the output flag table to set
! corresponding to the bit in the input flag table. 
!
! Arguments:

      INTEGER BitIn                !- i/p bit value (Integer form)
      INTEGER NumBitsIn            !- size of i/p bit table
      INTEGER Mask(NumBitsIn)      !- input bit mask (see description)
      INTEGER BitOut               !- o/p bit value (Integer form)
      INTEGER NumBitsOut           !- sze of o/p bit table

! Local variables:

      INTEGER I                    !- used for looping
      LOGICAL BitSet               !- TRUE if bit set
      
      BitOut = 0

      DO I = 1,NumBitsIn
        IF (Mask(I).GT.0) THEN
          BitSet = BTEST(BitIn,NumBitsIn-I)
          IF (BitSet) BitOut = IBSET(BitOut,NumBitsOut-Mask(I))
        ENDIF
      ENDDO

      RETURN
      END

!-----------------------------------------------------------------------

      SUBROUTINE UnBitSwap(BitIn, Mask, NumBitsIn, BitOut, NumBitsOut)
      IMPLICIT NONE

! purpose         : To reverse the steps taken by bitswap.f
!                 : bitswap.f takes an input bit field (BitIn) and sets the bits
!                 : in the output bit field (BitOut) detrmined by the
!                 : mapping in the Mask field.                     
!                                                                               
! calls           : BTEST  : To see if a bit is set
!                 : IBSET  : To set a bit
!                                                                               
! Description of Mask :
! 
! The mask is of the size of the input flag (bit) table. Each 
! element in the mask is the bit in the output flag table to set
! corresponding to the bit in the input flag table. 
!
! Arguments:

      INTEGER BitIn                !- i/p bit value (Integer form)
      INTEGER NumBitsIn            !- size of i/p bit table
      INTEGER Mask(NumBitsIn)      !- input bit mask (see description)
      INTEGER BitOut               !- o/p bit value (Integer form)
      INTEGER NumBitsOut           !- sze of o/p bit table

! Local variables:

      INTEGER I                    !- used for looping
      LOGICAL BitSet               !- TRUE if bit set
      
      BitIn = 0

      DO I = 1,NumBitsIn               !Loop over bits
        IF (Mask(I).GT.0) THEN
          BitSet = BTEST(BitOut,NumBitsOut-Mask(I))
          IF (BitSet) BitIn = IBSET(BitIn,NumBitsIn-I)
        ENDIF
      ENDDO

      RETURN
      END
