#!/bin/ksh
#
# Check IASI BRD, CTX, GRD and ODB configuration files for endian-ness and
# convert as necessary. The first word is a version number in all cases;
# assume 1-65535 for 4-byte words, 1-256 for 2-byte words.
#
# DIR_IASICONFIG to be specified either as an argument or as an environment
# variable.
#
# v1.1 22/6/06 N C Atkinson
# v1.2 21/8/06 Bug fix: conversions were not working on little-endian systems
# v1.3 06/9/06 change syntax of od command, for clarity
# v1.4 04/10/11 Add '-b' option for use with OPS-LRS v6-0
###############################################################################

usage(){
  echo 'Usage: convert_config_files [-b] DIR_IASICONFIG'
  echo '  use -b (big-endian) for OPS-LRS v6-0 onwards'
}

endian=native      #default (for versions of OPS-LRS prior to 6-0)
while getopts b c
do
  case $c in
    b) endian=big;;
    *) usage; exit 2;;
  esac
done
shift `expr $OPTIND - 1`

DIR_IASICONFIG="${DIR_IASICONFIG:-$1}"

if [ "$DIR_IASICONFIG" = "" ]; then
  usage
  exit 1
fi
if [ ! -d "$DIR_IASICONFIG" ]; then
  echo "$DIR_IASICONFIG is not a valid directory"
  exit 1
fi

. "$AAPP_PREFIX/ATOVS_CONF"

cd $DIR_IASICONFIG

for file in $(ls IASI_BRD*); do    #BRD: version no ranges from 1 to 65535
  swap=n
  if [ $endian = native ]; then
    set $(od -t u4 -N 4 $file)        #integer4 stored as $2
    [ $2 -ge 65536 ] && swap=y
  else
    set $(od -t u2 -N 4 $file)
    [ $2 -ne 0 ] && swap=y
  fi
  if [ $swap = y ]; then
    echo converting $file
    mkdir -p orig                     #save the original files
    cnes_iasi_brd-swapb $file ${file}_new \
      || { echo "Error in cnes_iasi_brd-swapb"; exit 1; }
    mv $file orig
    mv ${file}_new $file
  fi
done
for file in $(ls IASI_CTX*); do    #CTX: version no ranges from 1 to 255
  swap=n
  if [ $endian = native ]; then
    set $(od -t u2 -N 2 $file)        #integer2 stored as $2
    [ $2 -ge 256 ] && swap=y
  else
    set $(od -t u1 -N 2 $file)
    [ $2 -ne 0 ] && swap=y
  fi
  if [ $swap = y ]; then
    echo converting $file
    mkdir -p orig                     #save the original files
    cnes_iasi_ctx-swapb $file ${file}_new \
      || { echo "Error in cnes_iasi_ctx-swapb"; exit 1; }
    mv $file orig
    mv ${file}_new $file
  fi
done
for file in $(ls IASI_GRD*); do    #GRD: version no ranges from 1 to 65535
  swap=n
  if [ $endian = native ]; then
    set $(od -t u4 -N 4 $file)        #integer4 stored as $2
    [ $2 -ge 65536 ] && swap=y
  else
    set $(od -t u2 -N 4 $file)
    [ $2 -ne 0 ] && swap=y
  fi
  if [ $swap = y ]; then
    echo converting $file
    mkdir -p orig                     #save the original files
    cnes_iasi_grd-swapb $file ${file}_new \
      || { echo "Error in cnes_iasi_grd-swapb"; exit 1; }
    mv $file orig
    mv ${file}_new $file
  fi
done
for file in $(ls IASI_ODB*); do    #ODB: version no ranges from 1 to 65535
  swap=n
  if [ $endian = native ]; then
    set $(od -t u4 -N 4 $file)        #integer4 stored as $2
    [ $2 -ge 65536 ] && swap=y
  else
    set $(od -t u2 -N 4 $file)
    [ $2 -ne 0 ] && swap=y
  fi
  if [ $swap = y ]; then
    echo converting $file
#    ulimit -s unlimited               #maximum stack size
    mkdir -p orig                     #save the original files
    cnes_iasi_odb-swapb $file ${file}_new \
      || { echo "Error in cnes_iasi_odb-swapb"; exit 1; }
    mv $file orig
    mv ${file}_new $file
  fi
done

