/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2006/2
 * 
 * SYNOPSIS  : Create an XML OBT file from a L0 file.
 *             We extract OBT/UTC information from the VIADR.
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 * 
 * HISTORY   : 09/02/2007 : Print out a non-null CCU_OBT field
 *                          for OPS.
 *             10/11/2010 : Change float -> double in a_step computation. NCA.
 *
 * VERSION   : 1.0
 *
 * LIBS = 
 */
#include <stdio.h>
#include "eps_metopl0.h"
#include <string.h>
#include <errno.h>
#include <unistd.h>

int main( int argc, char* argv[] ) {
  FILE *fpr, *fpw;
  t_eps_metopl0 ER;
  int done = 0;
  
  if( argc < 2 ) {
    fprintf( stderr, "Usage: %s ...._L0_xxx_... [ xxxx_OBT_... ]\n", argv[0] );
    exit( 0 );
  }
  
  if( ( fpr = fopen( argv[1], "r" ) ) == NULL ) {
    fprintf( stderr, "Cannot open %s for reading: %s\n", argv[1], strerror( errno ) );
    exit( 1 );
  }
  
  if( argc == 3 ) {
    if( ( fpw = fopen( argv[2], "w" ) ) == NULL ) {
      fprintf( stderr, "Cannot open %s for writing: %s\n", argv[2], strerror( errno ) );
      exit( 1 );
    }
  } else {
    fpw = stdout;
  }
  
  eps_metopl0_check_subclass = 0;
  eps_metopl0_check_subclass_version = 0;
  
  while( get_eps_metopl0( &ER, fpr ) ) {

    if( ER.grh.record_class == VIADR ) {

      t_viadr_0 *v0 = ER.content.VIADR_0;
      double a_step = ( (double)v0->CLOCK_STEP[0] * 0x10000 
	              + (double)v0->CLOCK_STEP[1] ) / 1e12; /* ~ 1/256s */

      eps_integer8 OBT_UTC_TIME_rtime = v0->OBT_UTC_TIME.rtime;
      eps_integer4 UTC_OBT_TIME_days  = ER.grh.record_start.days - 1;
      eps_integer8 UTC_OBT_TIME_msec  = ER.grh.record_start.milliseconds;
      eps_integer4 UTC_OBT_TIME_usec  = 0;
      double sec;

      sec = 
      ( 24. * 3600. * ( UTC_OBT_TIME_days - v0->UTC_OBT_TIME.days )      
      + 1e-3 * ( UTC_OBT_TIME_msec - v0->UTC_OBT_TIME.msec ) 
      + 1e-6 * ( UTC_OBT_TIME_usec - v0->UTC_OBT_TIME.usec ) );

      OBT_UTC_TIME_rtime += sec / a_step;
      
      
      fprintf( fpw,
"<?xml version=\"1.0\"?>\n"
"<!DOCTYPE FD_OBT_UTC_Time_Correlation SYSTEM \"FD_OBT_UTC_Time_Correlation.dtd\">\n"
"<FD_OBT_UTC_Time_Correlation>\n"
"  <UTC_LEAP_SECOND>00000:0000000000:00000</UTC_LEAP_SECOND>\n"
"  <CORRELATION>\n"
"    <CCU_OBT>%lld</CCU_OBT>\n"
"    <UTC>%5.5d:%10.10lld:%5.5d</UTC>\n"
"    <A_STEP>%15.10lf</A_STEP>\n"
"    <B_OFFSET>0.0</B_OFFSET>\n"
"  </CORRELATION>\n"
"</FD_OBT_UTC_Time_Correlation>\n",
           OBT_UTC_TIME_rtime, UTC_OBT_TIME_days,
	   UTC_OBT_TIME_msec,  UTC_OBT_TIME_usec,
	   10e5 * a_step );
      done++;   
      break;      
    }
    free_eps_metopl0( &ER );
  }
  
  fclose( fpr );
  fclose( fpw );
  
  if( done )
    exit( 0 );
  else {
    fprintf( stderr, "Cannot find VIADR0 within %s\n", argv[1] );
    if( argc == 3 )
      unlink( argv[2] );
    exit( 1 );
  }
}
