/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/10
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.2
 *
 * HISTORY   : 20/10/2005 : Change BYTE_ORDER to AAPP_BYTE_ORDER
 *                          Change LITTLE_ENDIAN to AAPP_LITTLE_ENDIAN
 *                          Change BIG_ENDIAN to AAPP_BIG_ENDIAN
 *             26/01/2006 : Add ifndef AAPP_BIG_ENDIAN to avoid
 *                          redefining AAPP_BIG_ENDIAN & AAPP_LITTLE_ENDIAN
 *             03/08/2006 : Change eps_integer from 4b to 8b
 *             30/07/2009 : Add read_eps_nreal8, etc.
 */
#ifndef EPS_BUFFER_H
#define EPS_BUFFER_H
#include "eps_types.h"
#include <stdio.h>
#include <stdlib.h>

#ifdef HAS_LIBHDF5
#include <hdf5.h>
#endif


#define AAPP_BIG_ENDIAN 4321
#define AAPP_LITTLE_ENDIAN 1234

/*
  Buffer type for reading EUMETSAT data format
  We read a whole record within a buffer, then read each
  data field from buffer
*/


typedef struct {
  char* p;
  int lp;
  char* c;
  int lc;
} buffer;


/* General buffer methods */

int scan_buffer( const char* f, void* a, buffer* b, int l );

int read_buffer( void *p, int n, buffer* w );

int write_buffer( void *p, int n, buffer* w );

int reset_buffer( buffer* w );

int init_buffer( buffer* w, int n );

int free_buffer( buffer* w );

int status_buffer( buffer *w );

#define getmem( n ) malloc( n )
#define freemem( p ) free( p )

int read_int_buffer( void *p, int t, int n, buffer* w );

int write_int_buffer( void *p, int t, int n, buffer* w );

int print_int( const char *string, char* a, FILE *fp, int s, int t, int ndim, const int *dims );

#define read_eps_uinteger1( p, B, n ) read_int_buffer( p, 1, n, B )
#define read_eps_uinteger2( p, B, n ) read_int_buffer( p, 2, n, B )
#define read_eps_uinteger4( p, B, n ) read_int_buffer( p, 4, n, B )
#define read_eps_integer1( p, B, n ) read_int_buffer( p, 1, n, B )
#define read_eps_integer2( p, B, n ) read_int_buffer( p, 2, n, B )
#define read_eps_integer4( p, B, n ) read_int_buffer( p, 4, n, B )
#define read_eps_integer( p, B, n ) read_int_buffer( p, 8, n, B )

#define write_eps_uinteger1( p, B, n ) write_int_buffer( p, 1, n, B )
#define write_eps_uinteger2( p, B, n ) write_int_buffer( p, 2, n, B )
#define write_eps_uinteger4( p, B, n ) write_int_buffer( p, 4, n, B )
#define write_eps_integer1( p, B, n ) write_int_buffer( p, 1, n, B )
#define write_eps_integer2( p, B, n ) write_int_buffer( p, 2, n, B )
#define write_eps_integer4( p, B, n ) write_int_buffer( p, 4, n, B )
#define write_eps_integer( p, B, n ) write_int_buffer( p, 8, n, B )

#define print_eps_uinteger1( L, p, fp, ndim, dims ) print_int( L, (char*)p, fp, 0, 1, ndim, dims )
#define print_eps_uinteger2( L, p, fp, ndim, dims ) print_int( L, (char*)p, fp, 0, 2, ndim, dims )
#define print_eps_uinteger4( L, p, fp, ndim, dims ) print_int( L, (char*)p, fp, 0, 4, ndim, dims )
#define print_eps_integer1( L, p, fp, ndim, dims ) print_int( L, (char*)p, fp, 1, 1, ndim, dims )
#define print_eps_integer2( L, p, fp, ndim, dims ) print_int( L, (char*)p, fp, 1, 2, ndim, dims )
#define print_eps_integer4( L, p, fp, ndim, dims ) print_int( L, (char*)p, fp, 1, 4, ndim, dims )
#define print_eps_integer( L, p, fp, ndim, dims ) print_int( L, (char*)p, fp, 1, 8, ndim, dims )
#define print_eps_uinteger( L, p, fp, ndim, dims ) print_int( L, (char*)p, fp, 0, 8, ndim, dims )

#define init_eps_integer( pi ) *( pi ) = 0
#define init_eps_uinteger( pi ) *( pi ) = 0

int read_ascii_eps_integer( eps_integer *pi, buffer *B, int len );

int read_ascii_eps_uinteger( eps_uinteger *pi, buffer *B, int len );

int write_ascii_eps_integer( const char* label, eps_integer *pi, buffer *B, int len );

int write_ascii_eps_uinteger( const char* label, eps_uinteger *pi, buffer *B, int len );


int read_vint_buffer( void *p, int t, int n, buffer* w );

int write_vint_buffer( void *p, int t, int n, buffer* w );

int print_vint( const char *string, char* a, FILE *fp, int t, int ndim, const int *dims );

#define read_eps_vinteger4( p, B, n ) read_vint_buffer( p, 4, n, B )
#define write_eps_vinteger4( p, B, n ) write_vint_buffer( p, 4, n, B )
#define print_eps_vinteger4( L, p, fp, ndim, dims ) print_vint( L, (char*)p, fp, 4, ndim, dims )

int scale_eps_integer( const eps_integer *pi, eps_real *pr, int n, float s );

int scale_eps_integer2( const eps_integer2 *pi, eps_real4 *pr, int n, float s );

int scale_eps_integer4( const eps_integer4 *pi, eps_real4 *pr, int n, float s );

int scale_eps_uinteger( const eps_uinteger *pi, eps_real *pr, int n, float s );

int scale_eps_uinteger2( const eps_uinteger2 *pi, eps_real4 *pr, int n, float s );

int scale_eps_uinteger4( const eps_uinteger4 *pi, eps_real4 *pr, int n, float s );

int scale_eps_vinteger4( const eps_vinteger4 *pi, eps_real4 *pr, int n, float s );

/* eps_real methods */

int read_eps_real_buffer( void *p, int t, int n, buffer* w );

int write_eps_real_buffer( void *p, int t, int n, buffer* w );

int print_eps_real( const char *string, char* a, FILE *fp, int t, int ndim, const int *dims );

int read_eps_nreal_buffer( void *p, int t, int n, buffer* w );

int write_eps_nreal_buffer( void *p, int t, int n, buffer* w );

#define read_eps_real8( p, B, n ) read_eps_real_buffer( p, 8, n, B )
#define read_eps_real4( p, B, n ) read_eps_real_buffer( p, 4, n, B )

#define write_eps_real8( p, B, n ) write_eps_real_buffer( p, 8, n, B )
#define write_eps_real4( p, B, n ) write_eps_real_buffer( p, 4, n, B )

#define print_eps_real4( L, p, fp, ndim, dims ) print_eps_real( L, (char*)p, fp, 4, ndim, dims )
#define print_eps_real8( L, p, fp, ndim, dims ) print_eps_real( L, (char*)p, fp, 8, ndim, dims )

#define read_eps_nreal8( p, B, n ) read_eps_nreal_buffer( p, 8, n, B )
#define read_eps_nreal4( p, B, n ) read_eps_nreal_buffer( p, 4, n, B )

#define write_eps_nreal8( p, B, n ) write_eps_nreal_buffer( p, 8, n, B )
#define write_eps_nreal4( p, B, n ) write_eps_nreal_buffer( p, 4, n, B )

#define print_eps_nreal4( L, p, fp, ndim, dims ) print_eps_real( L, (char*)p, fp, 4, ndim, dims )
#define print_eps_nreal8( L, p, fp, ndim, dims ) print_eps_real( L, (char*)p, fp, 8, ndim, dims )


#define init_eps_bitfield( pb ) 

int read_ascii_eps_bitfield( void* p, buffer *w, int t );

int read_eps_bitfield_buffer( void *p, int t, int n, buffer* w );

int write_ascii_eps_bitfield( const char *label, void *p, buffer *w, int t );

int write_eps_bitfield_buffer( void *p, int t, int n, buffer* w );

int print_eps_bitfield( const char *string, char* a, FILE *fp, int t, int ndim, const int *dims );

int get_bit( char* eps_bitfield, int nbit );

int set_bit( char* eps_bitfield, int nbit, int b );



int read_eps_ccsdstime_buffer( eps_ccsdstime *T, int n, buffer* w );

int write_eps_ccsdstime_buffer( eps_ccsdstime *T, int n, buffer* w );

int print_eps_ccsdstime( const char *string, eps_ccsdstime* a, FILE *fp, int ndim, const int *dims );


#define read_eps_ccsdstime( p, B, n ) read_eps_ccsdstime_buffer( p, n, B )

#define write_eps_ccsdstime( p, B, n ) write_eps_ccsdstime_buffer( p, n, B )


/* ccsds pph, psh, obt */


int read_pph_buffer( t_pph *pph, int n, buffer* w );

int write_pph_buffer( t_pph *P, int n, buffer* w );

int read_psh_buffer( t_psh *psh, int n, buffer* w );

int write_psh_buffer( t_psh *P, int n, buffer* w );

int read_obt_buffer( t_obt *sbt, int n, buffer* w );

int write_obt_buffer( t_obt *O, int n, buffer* w );

#define read_pph( p, B, n ) read_pph_buffer( p, n, B )

#define write_pph( p, B, n ) write_pph_buffer( p, n, B )

#define read_psh( p, B, n ) read_psh_buffer( p, n, B )

#define write_psh( p, B, n ) write_psh_buffer( p, n, B )

#define read_obt( p, B, n ) read_obt_buffer( p, n, B )

#define write_obt( p, B, n ) write_obt_buffer( p, n, B )

int size_pph( t_pph *pph );

int size_psh( t_psh *psh );

int size_obt( t_obt *obt );



int read_eps_byte_buffer( eps_byte *p, int n, buffer *w );

int write_eps_byte_buffer( eps_byte *p, int n, buffer *w );


#define read_eps_byte( p, B, n ) read_eps_byte_buffer( (eps_byte*)p, n, B )

#define write_eps_byte( p, B, n ) write_eps_byte_buffer( (eps_byte*)p, n, B )


int read_eps_boolean_buffer( eps_boolean *p, int n, buffer *w );

int write_eps_boolean_buffer( eps_boolean *p, int n, buffer *w );

int print_eps_boolean( const char *label, eps_boolean *p, FILE *fp, int ndim, const int *dims );

#define read_eps_boolean( p, B, n ) read_eps_boolean_buffer( p, n, B )

#define write_eps_boolean( p, B, n ) write_eps_boolean_buffer( p, n, B )

#define init_eps_boolean( pb ) *( pb ) = 0

int read_ascii_eps_boolean( eps_boolean *pi, buffer *B, int len );

int write_ascii_eps_boolean( const char* label, eps_boolean *pi, buffer *B, int len );




int print_string( const char *label, eps_byte *s, FILE *fp, int len, int ndim, const int *dims );

#define read_string( p, B, t, n ) read_eps_byte_buffer( (eps_byte*)p, ( t ) * ( n ), B )

#define write_string( p, B, t, n ) write_eps_byte_buffer( (eps_byte*)p, ( t ) * ( n ), B )

#define init_string( s, l ) memset( s, 'x', l )

int read_ascii_string( char* s, buffer *B, int len );

int write_ascii_string( const char* label, char* s, buffer *B, int len );


int read_ascii_eps_gentime( eps_gentime *gt, buffer *B );

int read_ascii_eps_longgentime( eps_longgentime *lgt, buffer *B );

int write_ascii_eps_gentime( const char* label, eps_gentime *gt, buffer *B );

int write_ascii_eps_longgentime( const char* label, eps_longgentime *lgt, buffer *B );

int init_eps_gentime( eps_gentime *gt );

int init_eps_longgentime( eps_longgentime *lgt );



int read_label( const char** lb, int *len, buffer *B );

extern int decode_endian;
extern int encode_endian;
extern int nreal_endian;


int skip_line( buffer *B );

int print_eps_gentime( const char* label, eps_gentime *gt, FILE *fp, int, int* );
int print_eps_longgentime( const char* label, eps_longgentime *gt, FILE *fp, int, int* );


/* HDF5 methods */

#ifdef HAS_LIBHDF5

int write_epsh5_integer2( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );
int write_epsh5_integer4( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );

int write_epsh5_uinteger( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );
int write_epsh5_uinteger1( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );
int write_epsh5_uinteger2( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );
int write_epsh5_uinteger4( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );

int write_epsh5_vinteger4( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );

int write_epsh5_ccsdstime( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );
int write_epsh5_ShortCDSTime( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );
int write_epsh5_longgentime( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );
int write_epsh5_gentime( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );

int write_epsh5_string( char *p, hid_t gid, int n, char *dname );

int write_epsh5_byte( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );

int write_epsh5_boolean( void *p, hid_t gid, int rank, hsize_t *dims, char *dname );

#endif  /* HAS_LIBHDF5 */

#endif
