/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2009/12
 * 
 * AUTHOR    : Pascal Brunel
 *
 * SYNOPSIS  : aapp1dpfs.c: Program for conversion of cloudmask from AAPP AVHRR
 *             1D to PFS 1b.
 *
 * It takes three arguments:
 *         - Level 1b PFS file, converted to PFS format by aapp-eps_avhrrl1b
 *         - AAPP AVHRR level 1d file
 *         - output level 1b PFS file with cloud mask.
 *
 * Input 1d and 1b PFS should be the same for start and end times, no 
 * control is done. It could be improved in order to give the capability to
 * output in the same PFS file as the input, or to generate the output name
 * with operational PFS name.
 *
 * HISTORY   : 23/03/2010 : Integrate in AAPP
 *
 * LIBS = eps_avhrrl1b_6.5 aapp_avhrrl1c eps_common 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "aapp_avhrrl1c.h"
#include "eps_avhrrl1b_6.5.h"
#include "eps_grh.h"
#include "eps_types.h"
#include "eps_buffer.h"

/* macro pour copier une chaine de bits de i1 a i2 */
#define get_ibit(x, i1, i2) \
  (x >> i1) & ( (1 << (i2-i1+1))-1 )

int main( int argc, char* argv[] ) {
  t_header H;
  t_record R;
  t_eps_avhrrl1b_6_5 ER;
  buffer BR, BH;
  FILE *fpa, *fpe, *fpw;

  int i, k;
  int decode_endian_aapp; 
  int cloudy, sea, cloudmask, qualflag, snowice, nonproc;

  if( argc < 4 ) {
    printf( "Usage: %s PFS.l1b AVHRR.l1d PFS.l1b\n", argv[0] );
    exit( 0 );
  }

  init_buffer( &BH, size_header( &H ) );
  init_buffer( &BR, size_record( &R ) );
  
  memset( BH.p, 0, size_header( &H ) );
  memset( BR.p, 0, size_record( &R ) );
  memset( &H, 0, sizeof( H ) );
  memset( &R, 0, sizeof( R ) );

  fpe = fopen( argv[1], "r" );
  if( fpe == NULL ) {
    printf( "Error opening readonly file %s\n", argv[1] );
    exit( 1 );
  }
  fpa = fopen( argv[2], "r" );
  if( fpa == NULL ) {
    printf( "Error opening readonly file %s\n", argv[2] );
    exit( 1 );
  }
  fpw = fopen( argv[3], "w" );
  if( fpw == NULL ) {
    printf( "Error opening write file %s\n", argv[3] );
    exit( 1 );
  }

  /* lecture header AAPP */
  fread( BH.p, size_header( &H ), 1, fpa );
  {
    /* grok aapp endianness */
    int yy1, yy2;
    unsigned char* p = (unsigned char*)BH.p + 12;

    yy1 = ((p[0]*256 + p[1])*256 + p[2])*256 + p[3];
    yy2 = ((p[3]*256 + p[2])*256 + p[1])*256 + p[0];
    if( ( yy1 > 1900 ) && ( yy1 < 2100 ) ) { 
      decode_endian_aapp = AAPP_BIG_ENDIAN;
      fprintf(stdout,"lecture AAPP Big endian\n");
    } else if( ( yy2 > 1900 ) && ( yy2 < 2100 ) ) {
      decode_endian_aapp = AAPP_LITTLE_ENDIAN;
      fprintf(stdout,"lecture AAPP Little endian\n");
    } else {
      fprintf( 
        stderr, 
        "Could not figure out endian order of AAPP file %s\n", argv[0] 
      );
      fprintf( 
        stderr, 
        "yy1 yy2 %d %d\n", yy1, yy2 
      );
      exit( 1 );
    }

  }

  decode_endian = decode_endian_aapp;
  read_header( &H, &BH, 1 );
  /* print_header( "HEADER:", &H, stdout, 0, NULL ); */

  k = 0;
  decode_endian = AAPP_BIG_ENDIAN;
  while (get_eps_avhrrl1b_6_5 (&ER, fpe))
    {
      k++;
      /* printf ("%d, %d\n", k, ER.grh.record_class); */
      if( ER.grh.record_class == MPHR ) {
        printf ("%67.67s \n",ER.content.MPHR->PRODUCT_NAME);
      }
      if( ER.grh.record_class == MDR ) {
          /* lecture AAPP */
          decode_endian = decode_endian_aapp;
          fread( BR.p, size_record( &R ), 1, fpa );
          reset_buffer( &BR );
          read_record( &R, &BR, 1 );

          /* mise a jour masque du PFS + terre/mer + neige/glace */
          memset (ER.content.MDR_1B->CLOUD_INFORMATION, 0, 
                    sizeof (ER.content.MDR_1B->CLOUD_INFORMATION));
          for( i = 0; i < 2048; i++ ) {
            cloudy = get_ibit ( R.avh1c_mask[i], 12, 12);
            sea    = get_ibit ( R.avh1c_mask[i], 13, 13);
            cloudmask  = get_ibit ( R.avh1c_mask[i], 6, 10);
            qualflag   = get_ibit ( R.avh1c_mask[i], 1, 2);
            snowice    = cloudmask==3 || cloudmask == 4;
            nonproc    = cloudmask==0;
            /* set_bit arguments are: pointer, bit position(0=MS bit), value.
               The EPS Generic Product Format Specification defines bit 0 as 
               the LS, so when setting bits according to Table 4.4.1.1 of the
               AVHRR PFS document we need to invert the bit number. */
            if ( (qualflag != 1) && (! nonproc) ) {
              if ( cloudy ) {
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-15, 1 );
              } else {
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-14, 1 );
              }
              if ( !sea ) {
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-1, 1 );
              }
              if ( snowice ) {
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-4, 1 );
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-5, 1 );
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-6, 1 );
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-7, 1 );
              }
            }
          }
          printf( "." ); fflush( stdout );
        }

      /* ecriture PFS */
      encode_endian = AAPP_BIG_ENDIAN;
      put_eps_avhrrl1b_6_5 (&ER, fpw);
      free_eps_avhrrl1b_6_5 (&ER);

      decode_endian = AAPP_BIG_ENDIAN;
    }

  free_buffer( &BH );
  free_buffer( &BR );
  printf( "\n" ); fflush( stdout );

/*
      // a faire des la lecture du MPHR
      if( argc != 3 ) { 
        char new[68];
        sprintf( new, "%67.67s", ER.content.MPHR->PRODUCT_NAME);
        rename( temp, new );
      }
*/

  return 0;
}
