/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Decodes AMSUA L0 and pass packets to AAPP
 *             AMSUA decoding routines.
 *
 * VERSION   : 1.7
 *
 * HISTORY   : 16/01/2006 : Add LIBS info
 *             26/01/2006 : Add SPACECRAFT_ID variable
 *                          Add call to aapp_finish
 *                          Fill dummies with zeroes
 *             01/02/2006 : Changes basic types to eps types
 *                          Make internal routines private
 *             30/10/2006 : Check for input file
 *             02/10/2006 : Rewrite of the processing loop
 *                          Messages on two columns
 *                          Change day offset
 *                          Force processing of corrupt
 *                          data to introduce dummy
 *                          records
 *             09/11/2006 : Add 8s to line datation 
 *                          because traditional NOAA processing will
 *                          remove them.
 *             16/11/2006 : Fix broken lines datation
 *             26/11/2013 : Prevent infinite loop if file ends before matching
 *                          seq_count1 and seq_count2 are found
 *             19/11/2014 : Skip over duplicate MDRs
 *             17/11/2017 : Ensure seq_count2 == seq_count1 before the main loop:
 *                          corrects an error in the 26/11/2013 update.
 *
 * LIBS = metop_amsua obtutc ccsds eps_metopl0 eps_common metop_common
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ccsds.h"
#include "eps_metopl0.h"
#include "obtutc.h"
#include "amsua-aapp.h"
#include "common-aapp.h"
#include "eps_types.h"

#define SIZE_OF_MHS_PACKET 1300
#define SIZE_OF_AMSUA1_PACKET 620
#define SIZE_OF_AMSUA2_PACKET 620
#define SIZE_OF_AMSUB_PACKET 1950

#define SECONDS_AMSUA_LINE 8.

/*
 * remove_fillwords takes bufi if length si, bufo of length so, 
 * and removes fillwords contained in the data stream.
 * Fillwords are 2 bytes long, and specified in fillword[]
 */

static int remove_fillwords( 
  char *bufi, char* bufo, int si, int so, char fillword[2] 
) {
  int i;
  int c;
	       
		   
  for( i = 0, c = 0; i < si; i += 2 ) {
    if( ( bufi[i+0] == fillword[0] )
     && ( bufi[i+1] == fillword[1] )
    ) {
    } else {
      if( c > so ) return -1;
      bufo[c+0] = bufi[i+0];
      bufo[c+1] = bufi[i+1];
      c += 2;
    }
  }

}


/*
 * main_amsout behaves like aapp's amsout, except that it
 * sets up dummy buffers when needed, figures out the
 * satellite id from the MPHR, and pass the correct option
 * array for processing AMSUA
 */

static void main_amsout( 
  t_mphr *m0, char *amsua1, char *amsua2, 
  eps_integer4 yr, eps_integer4 msech, eps_integer4 systim[3], eps_integer4 *ios 
) {
/*
 * This is the Fortran interface of aapp's amsout
 *
 * INTEGER*4 OPTN(12)     -- Option numbers
 * INTEGER*2 AMSUA1(620)  -- AMSU-A1 scan line
 * INTEGER*2 AMSUA2(156)  -- AMSU-A2 scan line
 * INTEGER*2 AMSUB(1950)  -- AMSU-B scan line
 * CHARACTER*1 MHS(3900)  -- 3 MHS scan lines
 * INTEGER*2 AMANAG(71)   -- AMSU Analogue data
 * INTEGER*2 AMVALID(71)  -- AMSU analog telemetry validity
 * INTEGER YR             -- Year of data (e.g. 1997).
 * INTEGER MSECA          -- Time of data
 * INTEGER SYSTIM(3)      -- System year, dayno, msecs from midnight
 * INTEGER SATID          -- NOAA Satellite id
 * INTEGER AERR           -- AMSU-A Error flag (0=OK)
 * INTEGER BERR(3)        -- AMSU-B Error flag (0=OK)
 * LOGICAL*1 FULLAM       -- Set to false when AMSU scan line written
 * LOGICAL*1 GOTIME       -- Set to TRUE when first TIP time read
 * INTEGER STARTORB       -- start orbit number
 * INTEGER ENDORB         -- end orbit number
 * LOGICAL*1 MHSPRESENT   -- Set to TRUE for NOAA-N,N'
 *
 */
  eps_integer4 optn[12] = { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
  eps_integer4 satid;
  eps_integer4 aerr[1] = { 0 };
  eps_integer4 berr[3] = { 0, 0, 0 };

  eps_integer2 amanag[71]; /* dummy analog data */
  eps_integer2 amvalid[71]; /* dummy analog validity data */
  eps_integer4 fullam;
  eps_integer4 gotime;
  eps_integer4 mhspresent;
  char SPACECRAFT_ID[4];
  eps_integer4 msecs;


  /* AAPP's amsout requires real dummy arrays */

  eps_integer2 amsua1_dummy[SIZE_OF_AMSUA1_PACKET];
  eps_integer2 amsua2_dummy[SIZE_OF_AMSUA2_PACKET];
  eps_integer2 amsub_dummy[SIZE_OF_AMSUB_PACKET];
  char mhs_dummy[SIZE_OF_MHS_PACKET*3];
  eps_integer4 day;
  
  
  /* Fills dummies with zeroes */

  memset( amanag, 0, 71 * sizeof( eps_integer2 ) );
  memset( amvalid, 0, 71 * sizeof( eps_integer2 ) );
  memset( amsua1_dummy, 0, SIZE_OF_AMSUA1_PACKET * sizeof( eps_integer2 ) );
  memset( amsua2_dummy, 0, SIZE_OF_AMSUA2_PACKET * sizeof( eps_integer2 ) );
  memset( amsub_dummy, 0, SIZE_OF_AMSUB_PACKET * sizeof( eps_integer2 ) );
  memset( mhs_dummy, 0, SIZE_OF_MHS_PACKET * 3 );


  optn[2] = 1;
  optn[3] = 1;


  amsua1 = amsua1 ? amsua1 : (char*)amsua1_dummy;
  amsua2 = amsua2 ? amsua2 : (char*)amsua2_dummy;

  memcpy( SPACECRAFT_ID, m0->SPACECRAFT_ID, 3 );
  SPACECRAFT_ID[3] = '\0';
  satid = aapp_genscid( SPACECRAFT_ID );
  fullam = 1;
  gotime = 1;
  mhspresent = 0;
  aerr[0] = 0;


  msecs = systim[2];
  day = systim[1];
  
  aapp_amsday( day, msecs );

  aapp_amsout(
    optn, amsua1, amsua2, (char*)amsub_dummy, (char*)mhs_dummy, 
    (char*)amanag, (char*)amvalid, yr, msech, systim, satid, aerr, berr,
    fullam, gotime, m0->ORBIT_START, m0->ORBIT_END, mhspresent
  );
  
}



/*
 * callbacks for opening and closing AMSUA write unit
 */

static const eps_integer4 amsl = 4;

static int amsua_l1b_open( char* filename ) {
  aapp_init( amsl, filename );
}

static int amsua_l1b_close() {
  aapp_finish( amsl );
}



/*
 * callback for writing header of AMSUA AAPP file.
 */


static int amsua_l1b_write_header() {
  eps_integer4 ios = 0;
  aapp_amahdw( amsl, &ios );
  return ios;
}


/*
 * Writes a AMSUA1,2 pair of ccsds packets. We decode the pph/psh/sbt,
 * and apply the obt/utc correction using the viadr0, then we remove
 * fillwords, and pass the packets to amsout
 */

static eps_real8 current_ij1950 = 0.;

static int amsua_l1b_write_record( char* amsua1, char* amsua2, t_viadr_0 *v0, t_mphr *m0 ) {
  packet_primary_header pph1, pph2;
  packet_secondary_header psh1, psh2;
  satellite_binary_time sbt1, sbt2;
  eps_integer4 yr;
  eps_integer4 msech;
  eps_integer4 systim[3] = { 0, 0, 0 };
  char *buffer1 = NULL, *buffer2 = NULL;
  char fillword[] = { 0, 1 };
  eps_integer4 ios = 0;

  

  if( amsua1 != NULL ) {
    ccsds_read_packet_primary_header( &pph1, amsua1 );
    ccsds_read_packet_secondary_header( &psh1, amsua1 + 6 );
    ccsds_read_satellite_binary_time( &sbt1, amsua1 + 6 + 8 );
  }

  if( amsua2 != NULL ) {
    ccsds_read_packet_primary_header( &pph2, amsua2 );
    ccsds_read_packet_secondary_header( &psh2, amsua2 + 6 );
    ccsds_read_satellite_binary_time( &sbt2, amsua2 + 6 + 8 );
  }
  
  if( amsua1 != NULL ) {
    printf( " %d %d %d %d %d %d %d | %d %d %d ", 
      pph1.version, 
      pph1.type, 
      pph1.secondary_header_flag, 
      pph1.APID, 
      pph1.sequence_flag, 
      pph1.packet_seq_count, 
      pph1.packet_length, 
      psh1.days, 
      psh1.msec, 
      psh1.usec
    );
  } else {
    printf( " DUMMY RECORD AMSUA1.................      " );
  }

  printf( "  *  " );

  if( amsua2 != NULL ) {
    printf( " %d %d %d %d %d %d %d | %d %d %d ", 
      pph2.version, 
      pph2.type, 
      pph2.secondary_header_flag, 
      pph2.APID, 
      pph2.sequence_flag, 
      pph2.packet_seq_count, 
      pph2.packet_length, 
      psh2.days, 
      psh2.msec, 
      psh2.usec
    );
  } else {
    printf( " DUMMY RECORD AMSUA2.................      " );
  }

  printf( " * " );

  {
    eps_real8 ij2000, ij1950;
    eps_integer4 iq[3];
    packet_secondary_header *psh = NULL;
    satellite_binary_time *sbt = NULL;

    if( amsua1 ) {
      psh = &psh1;
      sbt = &sbt1;
    }
    if( amsua2 ) {
      psh = &psh2;
      sbt = &sbt2;
    }


    if( psh != NULL ) {

      /* time correction using OBT */

      obtutc_viadr0_jj2000( psh, sbt, v0, m0, &ij2000 );
      obtutc_jj2000_to_jj1950( &ij2000, &ij1950 );
      
    } else {   
    
      ij1950 = current_ij1950 + SECONDS_AMSUA_LINE / ( 24. * 3600. );
      
    }
    
    current_ij1950 = ij1950;
    
    /* 
     * we add here 8s, because noaa processing
     * will remove these 8s
     */
    
    ij1950 += 8. / ( 24. * 3600. );

    aapp_xijq( &ij1950, iq );
    
    printf( "  %d, %d, %d", iq[0], iq[1], iq[2] );
      
    
    printf( "\n" );
    
    systim[0] = iq[0];
    systim[1] = iq[1];
    systim[2] = iq[2];
    
    
    yr    = systim[0];
            systim[1];
    msech = systim[2];
    



    
  }

  if( amsua1 != NULL ) {

    buffer1 = (char*)malloc( pph1.packet_length + 1 - 8 );
    memset( buffer1, 0, pph1.packet_length + 1 - 8 );

    remove_fillwords( 
      amsua1 + 6 + 8 + 6, buffer1, pph1.packet_length + 1 - 8, pph1.packet_length + 1 - 8, fillword
    );
    SWAP_INT_BUFFER( buffer1, 2, ( pph1.packet_length + 1 - 8 ) / 2 );

  }

  if( amsua2 != NULL ) {

    buffer2 = (char*)malloc( pph2.packet_length + 1 - 8 );

    remove_fillwords( 
      amsua2 + 6 + 8 + 6, buffer2, pph2.packet_length + 1 - 8, pph2.packet_length + 1 - 8, fillword
    );
    SWAP_INT_BUFFER( buffer2, 2, ( pph2.packet_length + 1 - 8 ) / 2 );

  }


  
  main_amsout( m0, buffer1, buffer2, yr, msech, systim, &ios );

  if( amsua1 )
  free( buffer1 );
  if( amsua2 )
  free( buffer2 );
  
}

static int compare_seq_counts( int seq_count1, int seq_count2 ) {
  
  if( ( seq_count1 < 0 ) || ( seq_count2 < 0 ) ) {
    printf( "Cannot compare negative seq_counts\n" );
    exit( 1 );
  }
  
  if( seq_count1 == seq_count2 )
    return 0;
  else if( ( ( seq_count1 - seq_count2 + 0x4000 ) % 0x4000 ) > ( 0x4000 / 2 ) )
    return -1;
  else 
    return +1;

}


/*
 * main is tricky here, because we are processing AMSUA1 and AMSUA2 at the same
 * time ( AAPP requires this ). 
 * We have to handle the case of missing packets of AMSUA1, AMSUA2, or both.
 * we return 
 
   +1 when we found a valid record with data; seq_count is updated
   
    0 when there is still data, but next record has not data; seq_count is updated;
    fpr is reset
    
   -1 when there is no data anymore
   
 */


static int get_next_mdr( t_eps_metopl0* pEM0, FILE* fpr, int* pseq_count, int apid ) {
  t_eps_metopl0* qEM0 = NULL;
  packet_primary_header pph;
  packet_secondary_header psh;
  long fpos = ftell( fpr );
  int pseq_diff;

  
  while( get_eps_metopl0( pEM0, fpr ) ) {
    if( ( pEM0->grh.record_class == MDR ) 
     && ( pEM0->grh.instrument_group != DUMMY ) 
    ) {
      ccsds_read_packet_primary_header( &pph, pEM0->content.MDR_0->INST_DATA );

      if( pph.APID == apid ) {
        qEM0 = pEM0;
        break;
      }
    }
    free_eps_metopl0( pEM0 );
  }
  
  /* we found nothing, we signal EOF */
  if( qEM0 == NULL )
    return -1;
    
  
  if( *pseq_count < 0 ) { /* first time we get here */
  
    *pseq_count = pph.packet_seq_count;
    return +1;
 
  } else { /* we have a valid MDR and pseq_count is valid */
  
    pseq_diff = pph.packet_seq_count % 0x4000 - *pseq_count;
    pseq_diff = (pseq_diff + 0x2000) % 0x4000 - 0x2000;
    
    if (pseq_diff > 0) *pseq_count = ( *pseq_count + 1 ) % 0x4000;
    
    if( *pseq_count == ( pph.packet_seq_count % 0x4000 ) ) { /* the normal case */
      return +1;
    } else { /* 
              * If pseq_diff is positive this is a missing scan so we go back in the fpr
              * free the record and return 0. If pseq_diff is negative or zero it's a duplicate
              * MDR so move on to the next.
              */
      if (pseq_diff > 0) {
        fseek( fpr, fpos, SEEK_SET );
        free_eps_metopl0( qEM0 );
      }
      return 0;
    }
  
  } 
  
  
  
}

int main( int argc, char* argv[] ) {
  FILE *fpr1, *fpr2;
  t_eps_metopl0 EM_0, MDR_1, MDR_2; 
  
  int seq_count1   = -1;
  int seq_count2   = -1;
  
  t_mphr* mphr0 = NULL;
  t_viadr_0* viadr0 = NULL;
  
  int c1, c2;
  
  int c;


  if( argc != 3 ) {
    printf( "Usage: %s AMSU_xxx_00_M01_... .l1b\n", argv[0] );
    exit( 1 );
  }

  /* we open the file twice, for reading A1 and A2 packets */
  fpr1 = fopen( argv[1], "r" );
  fpr2 = fopen( argv[1], "r" );
  if( fpr1 == NULL ) {
    printf( "Cannot open `%s' for reading\n", argv[1] );
    return 1;
  }

  eps_metopl0_check_subclass = 0;
  eps_metopl0_check_subclass_version = 0;

  amsua_l1b_open( argv[2] );

  get_eps_metopl0( &EM_0, fpr1 );
  mphr0 = EM_0.content.MPHR;
  
  while( get_eps_metopl0( &EM_0, fpr1 ) ) {
    if( EM_0.grh.record_class == VIADR ) {
      viadr0 = EM_0.content.VIADR_0;
      break;
    }
    free_eps_metopl0( &EM_0 );
  }
  
  if( viadr0 == NULL ) {
     printf( "No VIADR was found in this product\n" );
     return 1;
  }
  

  
  amsua_l1b_write_header();

  c1 = get_next_mdr( &MDR_1, fpr1, &seq_count1, 39 );
  c2 = get_next_mdr( &MDR_2, fpr2, &seq_count2, 40 );
  
  if( ( c1 > 0 ) && ( c2 > 0 ) ) { /* we have records of both A1 & A2 */

    while( compare_seq_counts( seq_count1, seq_count2 ) < 0 && c1 >= 0 ) { /* seek forward in AMSUA1 */

      amsua_l1b_write_record(
        c1 > 0                   ? MDR_1.content.MDR_0->INST_DATA : NULL, 
        seq_count1 == seq_count2 ? MDR_2.content.MDR_0->INST_DATA : NULL, 
        viadr0, mphr0
      );
        
      if( c1 > 0 )
        free_eps_metopl0( &MDR_1 );
        
      c1 = get_next_mdr( &MDR_1, fpr1, &seq_count1, 39 );
      
      /* we should have seq_count1 == seq_count2 now */
    }
        
    while( compare_seq_counts( seq_count1, seq_count2 ) > 0 && c2 >= 0 ) { /* seek forward in AMSUA2 */

      amsua_l1b_write_record(
        seq_count2 == seq_count1 ? MDR_1.content.MDR_0->INST_DATA : NULL, 
        c2 > 0                   ? MDR_2.content.MDR_0->INST_DATA : NULL, 
        viadr0, mphr0
      );
        
      if( c2 > 0 )
        free_eps_metopl0( &MDR_2 );
        
      c2 = get_next_mdr( &MDR_2, fpr2, &seq_count2, 40 );
      
      /* we should have seq_count2 == seq_count1 now */
    }
    
    
  } 



  while( 1 ) {
  
    amsua_l1b_write_record(
      c1 > 0 ? MDR_1.content.MDR_0->INST_DATA : NULL,
      c2 > 0 ? MDR_2.content.MDR_0->INST_DATA : NULL,
      viadr0, mphr0
    );
    
    if( c1 > 0 )
      free_eps_metopl0( &MDR_1 );
    if( c2 > 0 )
      free_eps_metopl0( &MDR_2 );
    
    c1 = get_next_mdr( &MDR_1, fpr1, &seq_count1, 39 );
    c2 = get_next_mdr( &MDR_2, fpr2, &seq_count2, 40 );
    
    /* get back in step if needed */
    while (compare_seq_counts(seq_count1,seq_count2) < 0 && c1 >= 0)
      c1 = get_next_mdr( &MDR_1, fpr1, &seq_count1, 39 );
    while (compare_seq_counts(seq_count1,seq_count2) > 0 && c2 >= 0)
      c2 = get_next_mdr( &MDR_2, fpr2, &seq_count2, 40 );
  
    if( ( c1 < 0 ) && ( c2 < 0 ) )
      break;

  }
  
  if( c1 > 0 )
    free_eps_metopl0( &MDR_1 );
  if( c2 > 0 )
    free_eps_metopl0( &MDR_2 );
    
    
  
  

  amsua_l1b_write_header();


  amsua_l1b_close();


  return 0;
}

