/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Main program for decoding AVHRR L0 and passing
 *             instruments packets to AAPP core routines.
 *
 * VERSION   : 1.5
 *
 * HISTORY   : 12/01/2006 : cast buffer to (unsigned char*)
 *                          in call to main_avhrdc
 *             16/01/2006 : Add LIBS info
 *             26/01/2006 : Add SPACECRAFT_ID variable
 *                          Add call to finish
 *             01/02/2006 : Change basic types to eps_types
 *                          Make internal routines private
 *             02/06/2006 : Set exit code
 *             30/10/2006 : Add -ignore_degraded_inst_mdr
 *                          and -ignore_degraded_proc_mdr
 *                          options
 *                          For dummy lines, set the date
 *                          of previous processed line,
 *                          so that if the last line of the pass
 *                          is a dummy, we have a valid header
 *             16/11/2006 : Fix broken lines datation
 *             20/10/2010 : Trap for CLOCK_STEP=0
 *             19/11/2010 : Check for APID change during a data gap
 *             16/05/2011 : Add check on pph.packet_length (sometimes the last
 *                          value is corrupt)
 *             13/07/2011 : Add check on psh.days
 * 
 * LIBS = obtutc ccsds metop_avhrr metop_common eps_metopl0 eps_common
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ccsds.h"
#include "eps_metopl0.h"
#include "obtutc.h"
#include "avhrr-aapp.h"
#include "common-main.h"
#include "common-aapp.h"
#include "eps_types.h"


#define SECONDS_AVHRR_LINE 0.166666666666667

/*
 * Expands 10-bit packed data in 16bit integers
 *
 */

static void decode( unsigned char* buf10, hword* buf16, int n10 ) {
  int i = 0;
  int j = 0;
  int n = 10;
  int a = 0, b = 0, c = 0;
  int x = 0;
  
  while( i < n10 ) {  

    x = a;
    
    while( n > 0 ) {
      a = buf10[i];
      x = x * 256 + a;
      i++;
      n -= 8;    
    }
        
    x >>= - n;
    n += 10;
    
    a &= ( 1 << ( 10 - n ) ) - 1;  
  
    buf16[j] = x;
   
    j++;
  } 
  
}

/*
 * main_avhrdc prepares a HRPT minor frame to pass to AAPP
 */

static void main_avhrdc( t_mphr *m0, unsigned char* buf10, eps_integer4 orbmf, 
                  eps_integer4 iqcur[3], int apid ) {
  hword buf16[10355];
  hword hrptmf[11090];


  memset( hrptmf, 0, sizeof( hrptmf ) );

  if( buf10 != NULL ) {
    int i;
    decode( buf10, buf16, 12944 );

  
    /* Channel 3A/3B */
  
    hrptmf[6] = ( apid == 103 ) ? 1 : 0;
    hrptmf[7] = ( apid == 103 ) ? 1 : 0;
  
    /* 
      Telemetry 
      - Ramp calibration ( 5 channels )   : 5w
      - AVHRR internal target temperature : 3w
      - AVHRR patch temperature           : 1w
    */

    hrptmf[12] = buf16[50];
    hrptmf[13] = buf16[51];
    hrptmf[14] = buf16[52];
    hrptmf[15] = buf16[53];
    hrptmf[16] = buf16[54];
    
    hrptmf[17] = buf16[10295 + 2];
    hrptmf[18] = buf16[10295 + 3];
    hrptmf[19] = buf16[10295 + 4];
    
    hrptmf[20] = buf16[10300 + 2];
    
    /*
      AVHRR internal target data : 30w
    */
    
    for( i = 0; i < 10; i++ ) {
      hrptmf[22 + 3 * i ] = buf16[10305 + 2 + 5 * i];
      hrptmf[23 + 3 * i ] = buf16[10305 + 3 + 5 * i];
      hrptmf[24 + 3 * i ] = buf16[10305 + 4 + 5 * i];
    }
  
    /*
      Space data : 50w
    */
    
    for( i = 0; i < 50; i++ ) {
      hrptmf[52 + i] = buf16[i];
    }
    
    /* 
      Earth data
    */
    
    for( i = 0; i < 10240; i++ ) {
      hrptmf[750 + i] = buf16[55 + i];
    }
  
   
  }
  
/*
 *  Interface to AAPP's avhrc
 *  input : integer opmode           !( 0 = off-line, 1= real time)
 *  input : startorb                 ! revolution in which is th beginning of data set
 *  input : endorb                   ! revolution in which is the end of data set
 *  input : integer isystq(2)        ! system quant. time (tp library)
 *  input : integer satid            ! satellite id. order number from tiros-n (1)
 *  input : integer iqcur(3)         ! current year,day and millsecs.
 *  input : integer orbmf            ! orbit minor frame number ,AVHRR scan line number
 *  input : integer nbmiscn          ! number of missing scan lines
 *  input : integer hmfn             ! hrpt minor frame number
 *  input : integer*2 hrptmf(11090)  ! hrpt minor frame to process   
 */  
  {
    eps_integer4 opmode;
    eps_integer4 isystq[2];
    eps_integer4 satid;
    
    eps_integer4 nbmiscn;
    static eps_integer4 hmfn = 1;
    char SPACECRAFT_ID[4];
    
    opmode = 0;

    memcpy( SPACECRAFT_ID, m0->SPACECRAFT_ID, 3 );
    SPACECRAFT_ID[3] = '\0';
    satid = aapp_genscid( SPACECRAFT_ID );

    nbmiscn = 0;
    
    aapp_avhrdc( 
      opmode, m0->ORBIT_START, m0->ORBIT_END, isystq, satid, iqcur, orbmf,
      nbmiscn, hmfn, (char*)hrptmf
    );
    
  
    hmfn++;
    hmfn = hmfn % 3 + 1;
  
  }
  
  
}

/*
 * Here we record the current record number and the current
 * systim. We need this to provide dummy record with 
 * valid time information.
 * If the pass starts with dummy records, then these are
 * skipped until a valid MDR shows up.
 *
 */

static eps_integer4 orbmf = 1;
static eps_real8 current_ij1950 = 0.;
static int current_record_number = 0;
static int APID_save = -1, msec_save = -1, days_save = -1, packet_length_save = -1;

/* 
 * avhrr_l1b_write_record writes a record, after applying the time
 * correction
 */

static int avhrr_l1b_write_dummy( t_viadr_0 *v0, t_mphr *m0 ) {
  eps_integer4 systim[3];
  eps_real8 ij1950;
  eps_integer4 iq[3];
  eps_integer4 yr, msech;
  
  if( current_record_number == 0 ) 
    return -1;
  
  ij1950 = current_ij1950 + SECONDS_AVHRR_LINE / ( 24. * 3600. );
  
  aapp_xijq( &ij1950, iq );

  yr    = systim[0] = iq[0];
          systim[1] = iq[1];
  msech = systim[2] = iq[2];
  
  main_avhrdc( m0, NULL, orbmf, systim, 103 );
  printf( " DUMMY RECORD........................\n" );
  orbmf++;
  
  current_ij1950 = ij1950;
  
  return 1;
}

static int avhrr_l1b_write_record( char* buffer, t_viadr_0 *v0, t_mphr *m0 ) {
  packet_primary_header pph;
  packet_secondary_header psh;
  satellite_binary_time sbt;
  eps_integer4 yr;
  eps_integer4 msech;
  eps_integer4 systim[3];

  ccsds_read_pph_psh_sbt( &pph, &psh, &sbt, buffer );

  /* If there has been an APID change, check the scan times to see if
     dummy scans are needed - since they will not have been added from
     common-main */

  if ( ( APID_save > 0 ) && ( APID_save != pph.APID )){
    int msecdiff;
    int daydiff;
    int scandiff;
    int i;
    msecdiff = psh.msec - msec_save;
    daydiff = psh.days - days_save;
    if (msecdiff < 0 && daydiff == 1) msecdiff += 86400000;
    scandiff = msecdiff/(SECONDS_AVHRR_LINE*1000.0) + 0.5;
    if (scandiff > 1) printf ("Number of dummy scans to add: %d\n", scandiff-1);
    for (i=0; i<scandiff; i++){
      avhrr_l1b_write_dummy( v0, m0 );
    }
  }

  printf( " %d %d %d %d %d %d %d | %d %d\n", 
    pph.version, 
    pph.type, 
    pph.secondary_header_flag, 
    pph.APID, 
    pph.sequence_flag, 
    pph.packet_seq_count, 
    pph.packet_length, 
    psh.days, 
    psh.msec
  );

  /* Check pph.packet_length, assuming the first is correct */
  
  if ( ( packet_length_save < 0) || ((packet_length_save == pph.packet_length) && 
       ((psh.days == days_save) || (psh.days == days_save+1)) && 
       (psh.msec >= 0) && (psh.msec < 86400000)) ) {

  APID_save = pph.APID;
  days_save = psh.days;
  msec_save = psh.msec;
  packet_length_save = pph.packet_length;

  {
    eps_real8 ij2000, ij1950;
    eps_integer4 iq[3];

    /* time correction using OBT */

    if (v0->CLOCK_STEP[0] == 0 && v0->CLOCK_STEP[1] == 0){
      printf("Warning: obt-utc clock step is zero\n");
      return 1;
    }
    obtutc_viadr0_jj2000( &psh, &sbt, v0, m0, &ij2000 );
    obtutc_jj2000_to_jj1950( &ij2000, &ij1950 );

    aapp_xijq( &ij1950, iq );

    yr    = systim[0] = iq[0];
            systim[1] = iq[1];
    msech = systim[2] = iq[2];
    
    current_ij1950 = ij1950;
  }
  
  current_record_number++;

  main_avhrdc( 
    m0,
    (unsigned char*)buffer + SIZE_OF_PACKET_PRIMARY_HEADER + SIZE_OF_PACKET_SECONDARY_HEADER 
    + SIZE_OF_SATELLITE_BINARY_TIME, orbmf, systim, pph.APID );
    
  orbmf++;
  
  }    /* end of check on pph.packet_length */

  return 1;
}

/*
 * callbacks for opening, closing and writing header
 */

static const eps_integer4 avhl = 4;

static int avhrr_l1b_open( char* filename ) {
  aapp_init( avhl, filename );
}

static int avhrr_l1b_close() {
  aapp_finish( avhl );
}

static int avhrr_l1b_write_header() {
  eps_integer4 ios = 0;
  aapp_avhhdw( avhl, &ios );
  return ios;
}


/*
 * main program, we use generic loop handler
 */


int main( int argc, char* argv[] ) {

  if( argc < 3 ) {
    printf( "Usage: %s [ -ignore_degraded_inst_mdr "
            "-ignore_degraded_proc_mdr ] AVHR_xxx_00_M01_... .l1b\n", argv[0] );
    exit( 1 );
  }

  return common_main(
    argc, argv, avhrr_l1b_open, 
    avhrr_l1b_write_header, avhrr_l1b_write_record,
    avhrr_l1b_write_dummy, avhrr_l1b_close
  );

}
  
