/* 
 * Name: $Id$
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Opens an AAPP AVHRR 1B file and converts it ( partially )
 *             to PFS format, for use in IASI-OPS. The endianess of 
 *             AAPP input file is auto-detected.
 *
 * VERSION   : 1.2
 *
 * HISTORY   : 20/10/2005 : Change BYTE_ORDER to AAPP_BYTE_ORDER
 *                          Change LITTLE_ENDIAN to AAPP_LITTLE_ENDIAN
 *                          Change BIG_ENDIAN to AAPP_BIG_ENDIAN
 *
 *             27/10/2005 : Satellite name correction M01/M02/M03/M04
 *             16/01/2006 : Add LIBS info
 *             26/01/2006 : Add fclose for fpr & fpw
 *             01/02/2006 : Make internal routines private
 *                          Use eps_types in calls to AAPP
 *             14/02/2006 : Change name to eps_avhrrl1b-main.c
 *             11/08/2006 : Change %d to %qd (because eps_integer now 8 bytes)
 *             21/08/2006 : Check input file opened successfully. NCA
 *             01/09/2006 : Change %qd to %lld
 *             09/02/2006 : Add ORBIT_START & ORBIT_END to MPHR
 *             11/04/2007 : Add CH*_SOLAR_FILTERED_IRRADIANCE,
 *                          CH*_EQUIVALENT_FILTER_WIDTH,
 *                          CH*_CENTRAL_WAVENUMBER, CH*_CONSTANT1,
 *                          CH*_CONSTANT2_SLOPE in GIADR_ANALOG.
 *             19/07/2011 : Bug fix in z_intex_51_to_103. Point 103 was wrong.
 *             13/10/2011 : Ensure calvis slope is positive
 *             24/09/2018 : Pascale Roquet correct radiance unit in comment
 * 
 * LIBS = metop_common eps_avhrrl1b_6.5 aapp_avhrrl1b eps_common
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <time.h>
#include <math.h>

#include "common-aapp.h"

#include "eps_buffer.h"
#include "eps_avhrrl1b_6.5.h"
#include "aapp_avhrrl1b.h"
#include "eps_grh.h"
#include "eps_types.h"


#define JULIAN_OFFSET_2000_1950 18262

/*
 * Convert year/day of year to number of days since 1/1/2000
 */

static int scnlinyr_scnlindy_to_days( int scnlinyr, int scnlindy ) {
  eps_real8 ij0;
  eps_integer4 ig0[7] = { 2000, 1, 1, 0, 0, 0, 0 };

  ig0[0] = scnlinyr;

  aapp_xigj( ig0, &ij0 );

  return ij0 + ( scnlindy - 1 ) - JULIAN_OFFSET_2000_1950;
}

/*
 * Retrieves the localtime in a eps_gentime struct
 */

static int now_to_gentime( eps_gentime* gt ) {
  struct tm TM;
  time_t T;
  T = time( &T );
  localtime_r( &T, &TM );
  gt->YYYY = TM.tm_year + 1900;
  gt->MM   = TM.tm_mon + 1;
  gt->DD   = TM.tm_mday;
  gt->HH   = TM.tm_hour; 
  gt->mm   = TM.tm_min;
  gt->ss   = TM.tm_sec;
}


/*
 * Converts a (x,y,z) to a (lat,lon)
 */

static int xyz_to_ll(
  double x, double y, double z, double *plon, double *plat
) {
  double r;
  const double epsilon1 = 1e-15;
  const double epsilon2 = 1e-8;
  double xocl, yocl;
  double lon, lat;

  r = sqrt( x * x + y * y + z * z );
  x = x / r;
  y = y / r;
  z = z / r;

  if( z > ( + 1. - epsilon1 ) ) {
    lon = 0.;
    lat = +90.;
  } else if( z < ( -1. + epsilon1 ) ) {
    lon = 0.;
    lat = -90.;
  } else {
    lat = asin( z ); /* radians */
    xocl = x / cos( lat );
    if( fabs( xocl ) > ( +1 - epsilon2 ) ) {
      yocl = y / cos( lat );
      /* radians */
      lon = asin( yocl );
      lon = x > 0 ? lon : - lon + M_PI; 
    } else {
      /* radians */
      lon = acos( xocl );
      lon = y > 0 ? lon : - lon; 
    }
    lat = 180. * lat / M_PI; /* degrees */
    lon = 180. * lon / M_PI; /* degrees */
    while( lon > 180 ) {
      lon -= 360;
    }
  }

  plon[0] = lon;
  plat[0] = lat;

}

/*
 * (lat,lon) -> (x,y,z)
 */

static int ll_to_xyz( 
  double lon, double lat, double *px, double *py, double *pz 
) {
  px[0] = cos( M_PI * lon / 180. ) * cos( M_PI * lat / 180. );
  py[0] = sin( M_PI * lon / 180. ) * cos( M_PI * lat / 180. );
  pz[0] = sin( M_PI * lat / 180. );
}

/*
 * Extrapolates 51 navigated points to 103, as required in PFS.
 * The 51 points are spot 24, 64, ... 1984, 2024 (counting from 0).
 * The 103 points are 4, 24, .... 2044.
 */

int z_intex_51_to_103(
  double* zeni,               /* degrees */
  double t,
  double* loni, double* lati, /* degrees */
  double* lono, double* lato  /* degrees */
) {
  double alpha[103], gamma[103]; /* radians */
  double x[103], y[103], z[103];
  int i;
  double a1, a3;
  int j1, j2, j3;
  double c21, c31, c32;
  double r;

  /* Compute alpha (scan angle), gamma (angle at centre of earth) and x/y/z,
     for the original 51 points */

  for( i = 0; i < 51; i++ ) {
    alpha[2*i+1] = asin( t * sin( ( M_PI / 180. ) * zeni[i] ) );
    gamma[2*i+1] = ( M_PI / 180. ) * zeni[i] - alpha[2*i+1];
    ll_to_xyz( loni[i], lati[i], x + 2*i+1, y + 2*i+1, z + 2*i+1 );
    lono[2*i+1] = loni[i];
    lato[2*i+1] = lati[i];
  }

  /* Interpolate alpha */

  {
    r = ( 4 - 24. ) / 40.;
    alpha[0] = ( 1. - r ) * alpha[1] + r * alpha[3];
  }

  for( i = 0; i < 50; i++ ) {
    alpha[2*i+2] = ( alpha[2*i+1] + alpha[2*i+3] ) / 2;
  }

  {
  /* Bug fix 19/07/2011
   *    r = ( 2024 - 1984. ) / 40.;
   *    alpha[102] = ( 1. - r ) * alpha[1984] + r * alpha[2024];
   */
    r = ( 2024 - 2044. ) / 40.;
    alpha[102] = ( 1. - r ) * alpha[101] + r * alpha[99];
  }

  /* Compute the missing gamma values */

  /* Bug fix 19/07/2011
   *     for( i = 0; i < 51; i++ ) {
   */
  for( i = 0; i < 52; i++ ) {
    gamma[2*i] = asin( sin( alpha[2*i] ) / t ) - alpha[2*i];
  }

  /* Interpolate x,y,z and then convert back to lat/lon */

  {
    j1 = 1;
    j2 = 0;
    j3 = 3;
    c31 = cos( gamma[j3] - gamma[j1] );
    c21 = cos( gamma[j2] - gamma[j1] );
    c32 = cos( gamma[j3] - gamma[j2] );
    a1 = c21 - c31 * c32;
    a3 = c32 - c31 * c21;
    x[j2] = a1 * x[j1] + a3 * x[j3];
    y[j2] = a1 * y[j1] + a3 * y[j3];
    z[j2] = a1 * z[j1] + a3 * z[j3];

    xyz_to_ll( x[j2], y[j2], z[j2], lono + j2, lato + j2 );
  }

  for( i = 0; i < 50; i++ ) {
    j1 = 2*i+1;
    j2 = j1 + 1;
    j3 = j2 + 1;
    c21 = cos( gamma[j2] - gamma[j1] );
    c31 = cos( gamma[j3] - gamma[j1] );
    c32 = cos( gamma[j3] - gamma[j2] );
    a1 = c21 - c31 * c32;
    a3 = c32 - c31 * c21;
    x[j2] = a1 * x[j1] + a3 * x[j3];
    y[j2] = a1 * y[j1] + a3 * y[j3];
    z[j2] = a1 * z[j1] + a3 * z[j3];

    xyz_to_ll( x[j2], y[j2], z[j2], lono + j2, lato + j2 );
  }

  {
    j1 = 101;
    j2 = 102;
    j3 = 99;
    c31 = cos( gamma[j3] - gamma[j1] );
    c21 = cos( gamma[j2] - gamma[j1] );
    c32 = cos( gamma[j3] - gamma[j2] );
    a1 = c21 - c31 * c32;
    a3 = c32 - c31 * c21;
    x[j2] = a1 * x[j1] + a3 * x[j3];
    y[j2] = a1 * y[j1] + a3 * y[j3];
    z[j2] = a1 * z[j1] + a3 * z[j3];

    xyz_to_ll( x[j2], y[j2], z[j2], lono + j2, lato + j2 );
  }

}


int main( int argc, char* argv[] ) {
  FILE *fpr, *fpw;
  t_eps_avhrrl1b_6_5 ER;

  buffer BH, BR, BHx, BRx;
  t_header H;
  t_record R;
  int i = 0;
  int j;
  int line;
  
  long offset[4];
  char temp[32];
  

  BH.c  = BH.p  = aapp_avh1bhd_addr;
  BH.lc = BH.lp = size_header( &H );

  BR.c  = BR.p  = aapp_avh1bdt_addr;
  BR.lc = BR.lp = size_record( &R );

  init_buffer( &BHx, size_header( &H ) );
  init_buffer( &BRx, size_record( &R ) );


  if( argc < 2 ) {
    printf( "Usage: %s .l1b\n", argv[0] );
    exit( 1 );
  }

  fpr = fopen( argv[1], "r" );
  if( fpr == NULL ) {
    printf( "Error opening file %s\n", argv[1] );
    exit( 1 );
  }
  
  if( argc == 3 ) {
    fpw = fopen( argv[2], "w" );
  } else {
    int i;
    int fd;
    for( i = 0; i < 8; i++ ) {
      temp[i] = 'X';
    }
    temp[8] = '\0';
    fd = mkstemp( temp );
    fpw = fdopen( fd, "w+" );
  }


  fread( BHx.p, BHx.lp, 1, fpr );

  {
    /* grok aapp endianness */
    int yy1, yy2;
    unsigned char* p = (unsigned char*)BHx.p + 6;

    yy1 = p[0] * 256 + p[1];
    yy2 = p[1] * 256 + p[0];
    if( ( yy1 > 1900 ) && ( yy1 < 2100 ) ) { 
      decode_endian = AAPP_BIG_ENDIAN;
    } else if( ( yy2 > 1900 ) && ( yy2 < 2100 ) ) {
      decode_endian = AAPP_LITTLE_ENDIAN;
    } else {
      fprintf( 
        stderr, 
        "Could not figure out endian order of AAPP file %s\n", argv[0] 
      );
      exit( 1 );
    }

  }


  read_header( &H, &BHx, 1 );

  encode_endian = AAPP_BYTE_ORDER;
  write_header( &H, &BH, 1 );
  encode_endian = AAPP_BIG_ENDIAN;
  

  /* MPHR */ {

    static const char* SATS[] = { "M01", "M02", "M03", "M04" };

    ER.content.MPHR = (t_mphr*)calloc( 1, sizeof( t_mphr ) );
    
    init_mphr( ER.content.MPHR, 1 );
    
    ER.grh.record_class = MPHR;
    ER.grh.instrument_group = GENERIC;
    ER.grh.record_subclass = 0;
    ER.grh.record_subclass_version = 2;
  
    ER.grh.record_size = size_mphr( ER.content.MPHR );
  
    ER.grh.record_start.days = H.avh_h_startdatajd - JULIAN_OFFSET_2000_1950;
    ER.grh.record_stop.days  = H.avh_h_enddatajd   - JULIAN_OFFSET_2000_1950;
    
    ER.grh.record_start.milliseconds = H.avh_h_startdatatime;
    ER.grh.record_stop.milliseconds  = H.avh_h_enddatatime;
    
    memset( ER.content.MPHR->PARENT_PRODUCT_NAME_1, ' ', 67 );
    memcpy( ER.content.MPHR->PARENT_PRODUCT_NAME_1, H.avh_h_dataname, 42 );
    
    memcpy( ER.content.MPHR->INSTRUMENT_ID, "AVHR", 4 );
    memcpy( ER.content.MPHR->INSTRUMENT_MODEL, "  1", 3 );
    
    memcpy( ER.content.MPHR->PROCESSING_LEVEL, "1B", 2 );

    if( ( H.avh_h_satid >= 11 ) && ( H.avh_h_satid <= 14 ) ) {
      memcpy( ER.content.MPHR->SPACECRAFT_ID, SATS[H.avh_h_satid-11], 3 );
    } else {
      memcpy( ER.content.MPHR->SPACECRAFT_ID, "xxx", 3 );
    }

    {
      char c[3];
      int i;
      memcpy( c, H.avh_h_siteid, 3 );
      /* to uppercase */
      for( i = 0; i < 3; i++ ) {
        if( ( c[i] >= 'a' ) && ( c[i] <= 'z' ) ) {
          c[i] -= 32;
        }
      }
      memcpy( ER.content.MPHR->PROCESSING_CENTRE, c, 3 );
      memcpy( ER.content.MPHR->RECEIVING_GROUND_STATION, c, 3 );
    }

    {
      eps_integer4 jg[3], hh, mm, ss;
      char YYYYMMDDhhmmssZ[16];
      eps_integer4 jj;
     
      jj = H.avh_h_startdatajd; 
      aapp_xdjg( &jj, jg );
      hh = ( H.avh_h_startdatatime / 3600000 );
      mm = ( H.avh_h_startdatatime % 3600000 ) / 60000;
      ss = ( H.avh_h_startdatatime %   60000 ) /  1000;
      sprintf( 
        YYYYMMDDhhmmssZ, 
        "%4.4d%2.2d%2.2d%2.2d%2.2d%2.2dZ",
        jg[0], jg[1], jg[2], hh, mm, ss
      );

      ER.content.MPHR->SENSING_START.YYYY = jg[0];
      ER.content.MPHR->SENSING_START.MM   = jg[1];
      ER.content.MPHR->SENSING_START.DD   = jg[2];
      ER.content.MPHR->SENSING_START.HH   = hh;
      ER.content.MPHR->SENSING_START.mm   = mm;
      ER.content.MPHR->SENSING_START.ss   = ss;

      ER.content.MPHR->RECEIVE_TIME_START
      = ER.content.MPHR->SENSING_START_THEORETICAL 
      = ER.content.MPHR->SENSING_START;

      jj = H.avh_h_startdatajd; 
      aapp_xdjg( &jj, jg );
      hh = ( H.avh_h_enddatatime / 3600000 );
      mm = ( H.avh_h_enddatatime % 3600000 ) / 60000;
      ss = ( H.avh_h_enddatatime %   60000 ) /  1000;
      sprintf( 
        YYYYMMDDhhmmssZ, 
        "%4.4d%2.2d%2.2d%2.2d%2.2d%2.2dZ",
        jg[0], jg[1], jg[2], hh, mm, ss
      );

      ER.content.MPHR->SENSING_END.YYYY = jg[0];
      ER.content.MPHR->SENSING_END.MM   = jg[1];
      ER.content.MPHR->SENSING_END.DD   = jg[2];
      ER.content.MPHR->SENSING_END.HH   = hh;
      ER.content.MPHR->SENSING_END.mm   = mm;
      ER.content.MPHR->SENSING_END.ss   = ss;

      ER.content.MPHR->RECEIVE_TIME_END
      = ER.content.MPHR->SENSING_END_THEORETICAL 
      = ER.content.MPHR->SENSING_END;

    }

    ER.content.MPHR->TOTAL_MPHR  = 1;
    ER.content.MPHR->TOTAL_SPHR  = 1;
    ER.content.MPHR->TOTAL_IPR   = 3;
    ER.content.MPHR->TOTAL_GIADR = 2;
    ER.content.MPHR->TOTAL_MDR = H.avh_h_scnlin;
    ER.content.MPHR->TOTAL_RECORDS = 1 + 1 + 3 + 2 + H.avh_h_scnlin;
    
    
    ER.content.MPHR->DURATION_OF_PRODUCT = 
    ER.content.MPHR->MILLISECONDS_OF_DATA_PRESENT = 
    ( H.avh_h_enddatajd - H.avh_h_startdatajd ) * 3600 * 1000
    + ( H.avh_h_enddatatime - H.avh_h_startdatatime );
   
    now_to_gentime( &ER.content.MPHR->PROCESSING_TIME_START );
    ER.content.MPHR->PROCESSING_TIME_END = 
    ER.content.MPHR->PROCESSING_TIME_START;
    {
      char n[68];
      sprintf( n, 
        "%4.4s_%3.3s_%2.2s_%3.3s_"
        "%4.4lld%2.2lld%2.2lld%2.2lld%2.2lld%2.2lldZ_"
        "%4.4lld%2.2lld%2.2lld%2.2lld%2.2lld%2.2lldZ_"
        "%1.1s_%1.1s_"
        "%4.4lld%2.2lld%2.2lld%2.2lld%2.2lld%2.2lldZ", 
        ER.content.MPHR->INSTRUMENT_ID,
        ER.content.MPHR->PRODUCT_TYPE,
        ER.content.MPHR->PROCESSING_LEVEL,
        ER.content.MPHR->SPACECRAFT_ID,
        ER.content.MPHR->SENSING_START.YYYY,
        ER.content.MPHR->SENSING_START.MM,
        ER.content.MPHR->SENSING_START.DD,
        ER.content.MPHR->SENSING_START.HH,
        ER.content.MPHR->SENSING_START.mm,
        ER.content.MPHR->SENSING_START.ss,
        ER.content.MPHR->SENSING_END.YYYY,
        ER.content.MPHR->SENSING_END.MM,
        ER.content.MPHR->SENSING_END.DD,
        ER.content.MPHR->SENSING_END.HH,
        ER.content.MPHR->SENSING_END.mm,
        ER.content.MPHR->SENSING_END.ss,
        ER.content.MPHR->PROCESSING_MODE,
        ER.content.MPHR->DISPOSITION_MODE,
        ER.content.MPHR->PROCESSING_TIME_START.YYYY,
        ER.content.MPHR->PROCESSING_TIME_START.MM,
        ER.content.MPHR->PROCESSING_TIME_START.DD,
        ER.content.MPHR->PROCESSING_TIME_START.HH,
        ER.content.MPHR->PROCESSING_TIME_START.mm,
        ER.content.MPHR->PROCESSING_TIME_START.ss
      );
      memcpy( ER.content.MPHR->PRODUCT_NAME, n, 67 );

      if( argc != 3 ) {
      
        rename( temp, n );
      
      }

    }
      
    {
      int orb1, orb2;
      char buffer[16];

      memcpy( buffer, H.avh_h_dataname + 32, 5 );
      buffer[5] = '\0';
      orb1 = atoi( buffer );

      memcpy( buffer, H.avh_h_dataname + 37, 2 );
      buffer[2] = '\0';
      orb2 = ( orb1 / 100 ) * 100 + atoi( buffer );
      
      ER.content.MPHR->ORBIT_START = orb1;
      ER.content.MPHR->ORBIT_END   = orb2;
    }

    put_eps_avhrrl1b_6_5( &ER, fpw );
    
    free_eps_avhrrl1b_6_5( &ER );
 
  }
  
  /* SPHR */ {
    ER.content.SPHR = (t_sphr*)calloc( 1, sizeof( t_sphr ) );

    init_sphr( ER.content.SPHR, 1 );
    
    ER.grh.record_class = SPHR;
    ER.grh.instrument_group = AVHRR;
    ER.grh.record_subclass = 0;
    ER.grh.record_subclass_version = 3;
  
    ER.grh.record_size = size_sphr( ER.content.SPHR );
  
    ER.grh.record_start.days = H.avh_h_startdatajd - JULIAN_OFFSET_2000_1950;
    ER.grh.record_stop.days  = H.avh_h_enddatajd   - JULIAN_OFFSET_2000_1950;
    
    ER.grh.record_start.milliseconds = H.avh_h_startdatatime;
    ER.grh.record_stop.milliseconds  = H.avh_h_enddatatime;
    
    ER.content.SPHR->EARTH_VIEWS_PER_SCANLINE = 2048;
    ER.content.SPHR->NAV_SAMPLE_RATE = 20;

    put_eps_avhrrl1b_6_5( &ER, fpw );
    
    free_eps_avhrrl1b_6_5( &ER );
  
  }



  offset[0] = ftell( fpw );

  /* IPR placeholder */ for( i = 0; i < 3; i++ ) {
  
    ER.content.IPR = (t_ipr*)calloc( 1, sizeof( t_ipr ) );
    
    ER.grh.record_class = IPR;
    ER.grh.instrument_group = GENERIC;
    ER.grh.record_subclass = 0;
    ER.grh.record_subclass_version = 1;
  
    ER.grh.record_size = size_ipr( ER.content.IPR );
  
    ER.grh.record_start.days = H.avh_h_startdatajd - JULIAN_OFFSET_2000_1950;
    ER.grh.record_stop.days  = H.avh_h_enddatajd   - JULIAN_OFFSET_2000_1950;
    
    ER.grh.record_start.milliseconds = H.avh_h_startdatatime;
    ER.grh.record_stop.milliseconds  = H.avh_h_enddatatime;

    put_eps_avhrrl1b_6_5( &ER, fpw );
    
    free_eps_avhrrl1b_6_5( &ER );
  
  }


  offset[1] = ftell( fpw );
  
  /* GIADR RADIANCE */ {
  
    ER.content.GIADR_RADIANCE = (t_giadr_radiance*)
    calloc( 1, sizeof( t_giadr_radiance ) );
    
    ER.grh.record_class = GIADR;
    ER.grh.instrument_group = AVHRR;
    ER.grh.record_subclass = 1;
    ER.grh.record_subclass_version = 3;
  
    ER.grh.record_size = size_giadr_radiance( ER.content.GIADR_RADIANCE );
  
    ER.grh.record_start.days = H.avh_h_startdatajd - JULIAN_OFFSET_2000_1950;
    ER.grh.record_stop.days  = H.avh_h_enddatajd   - JULIAN_OFFSET_2000_1950;
    
    ER.grh.record_start.milliseconds = H.avh_h_startdatatime;
    ER.grh.record_stop.milliseconds  = H.avh_h_enddatatime;

    ER.content.GIADR_RADIANCE->CH1_SOLAR_FILTERED_IRRADIANCE 
      = H.avh_h_albcnv[0][0];
    ER.content.GIADR_RADIANCE->CH1_EQUIVALENT_FILTER_WIDTH 
      = H.avh_h_albcnv[0][1];

    ER.content.GIADR_RADIANCE->CH2_SOLAR_FILTERED_IRRADIANCE 
      = H.avh_h_albcnv[1][0];
    ER.content.GIADR_RADIANCE->CH2_EQUIVALENT_FILTER_WIDTH 
      = H.avh_h_albcnv[1][1];

    ER.content.GIADR_RADIANCE->CH3A_SOLAR_FILTERED_IRRADIANCE 
      = H.avh_h_albcnv[2][0];
    ER.content.GIADR_RADIANCE->CH3A_EQUIVALENT_FILTER_WIDTH 
      = H.avh_h_albcnv[2][1];


    ER.content.GIADR_RADIANCE->CH3B_CENTRAL_WAVENUMBER 
      = H.avh_h_radtempcv[0][0];
    ER.content.GIADR_RADIANCE->CH3B_CONSTANT1 
      = H.avh_h_radtempcv[0][1];
    ER.content.GIADR_RADIANCE->CH3B_CONSTANT2_SLOPE 
      = H.avh_h_radtempcv[0][2];

    ER.content.GIADR_RADIANCE->CH4_CENTRAL_WAVENUMBER 
      = H.avh_h_radtempcv[1][0];
    ER.content.GIADR_RADIANCE->CH4_CONSTANT1 
      = H.avh_h_radtempcv[1][1];
    ER.content.GIADR_RADIANCE->CH4_CONSTANT2_SLOPE 
      = H.avh_h_radtempcv[1][2];

    ER.content.GIADR_RADIANCE->CH5_CENTRAL_WAVENUMBER 
      = H.avh_h_radtempcv[2][0];
    ER.content.GIADR_RADIANCE->CH5_CONSTANT1 
      = H.avh_h_radtempcv[2][1];
    ER.content.GIADR_RADIANCE->CH5_CONSTANT2_SLOPE 
      = H.avh_h_radtempcv[2][2];


    put_eps_avhrrl1b_6_5( &ER, fpw );
    
    free_eps_avhrrl1b_6_5( &ER );
  
  }
  
  offset[2] = ftell( fpw );
  
  /* GIADR ANALOG */ {
  
    ER.content.GIADR_ANALOG = (t_giadr_analog*)
    calloc( 1, sizeof( t_giadr_analog ) );
    
    ER.grh.record_class = GIADR;
    ER.grh.instrument_group = AVHRR;
    ER.grh.record_subclass = 2;
    ER.grh.record_subclass_version = 2;
  
    ER.grh.record_size = size_giadr_analog( ER.content.GIADR_ANALOG );
  
    ER.grh.record_start.days = H.avh_h_startdatajd - JULIAN_OFFSET_2000_1950;
    ER.grh.record_stop.days  = H.avh_h_enddatajd   - JULIAN_OFFSET_2000_1950;
    
    ER.grh.record_start.milliseconds = H.avh_h_startdatatime;
    ER.grh.record_stop.milliseconds  = H.avh_h_enddatatime;

    put_eps_avhrrl1b_6_5( &ER, fpw );
    
    free_eps_avhrrl1b_6_5( &ER );  
  
  }

  
  offset[3] = ftell( fpw );
  
  for( line = 0; ; line++  ) {
    t_eps_avhrrl1b_6_5 ER;
    
  
    bzero( &ER, sizeof( ER ) );
  

    fread( BRx.p, BRx.lp, 1, fpr );
    if( feof( fpr ) ) { break; }
    reset_buffer( &BRx );
    read_record( &R, &BRx, 1 );
    reset_buffer( &BR );
    encode_endian = AAPP_BYTE_ORDER;
    write_record( &R, &BR, 1 );
    encode_endian = AAPP_BIG_ENDIAN;

    printf( "." ); fflush( stdout );

    ER.content.MDR_1B = (t_mdr_1b*)calloc( 1, sizeof( t_mdr_1b ) );
    ER.content.MDR_1B->EARTH_VIEWS_PER_SCANLINE = 2048;
    ER.content.MDR_1B->NUM_NAVIGATION_POINTS = 103;
    
    init_mdr_1b( ER.content.MDR_1B, 1 );


    ER.grh.record_class = MDR;
    ER.grh.instrument_group = AVHRR;
    ER.grh.record_subclass = 2;
    ER.grh.record_subclass_version = 4;
    
    ER.grh.record_size = size_mdr_1b( ER.content.MDR_1B );


    ER.grh.record_start.days =
    ER.grh.record_stop.days  =
    scnlinyr_scnlindy_to_days( R.avh_scnlinyr, R.avh_scnlindy );
    
    ER.grh.record_start.milliseconds =
    ER.grh.record_stop.milliseconds  =
    R.avh_scnlintime;



    /* MDR/DIGITAL_B_DATA */ {
      int i;
      eps_integer4 avh_h_inststat1 = H.avh_h_inststat1;
      for( i = 0; i < 16; i++ ) {
        set_bit( ER.content.MDR_1B->DIGITAL_B_DATA, 15 - i, avh_h_inststat1 % 2 );
	avh_h_inststat1 = avh_h_inststat1 >> 1;
      }
      /* 3A/3B */
      if( R.avh_scnlinbit & 0x00000001 ) {
        set_bit( ER.content.MDR_1B->DIGITAL_B_DATA, 8, 0 ); /* 3B */
      } else {
        set_bit( ER.content.MDR_1B->DIGITAL_B_DATA, 8, 1 ); /* 3A */
      }
    }


    /* MDR/FRAME_INDICATOR */ {
      /* 3A/3B */
      if( R.avh_scnlinbit & 0x00000001 ) {
        set_bit( ER.content.MDR_1B->FRAME_INDICATOR, 16, 0 ); /* 3B */
      } else {
        set_bit( ER.content.MDR_1B->FRAME_INDICATOR, 16, 1 ); /* 3A */
      }
    }

    /* MDR/CALIBRATION_QUALITY */ {
      int i, j;
      for( j = 0; j < 3; j++ ) {
        eps_integer2 avh_calqual = R.avh_calqual[j];
        for( i = 0; i < 16; i++ ) {
          set_bit( ER.content.MDR_1B->CALIBRATION_QUALITY[j], 15 - i, avh_calqual % 2 );
	  avh_calqual = avh_calqual >> 1;
	}
      }
    }
/*
    memcpy(
      ER.content.MDR_1B->CALIBRATION_QUALITY,
      &R.avh_calqual, 3 * 4
    );
*/

    /* MDR/SCAN_LINE_QUALITY */ {
      int i;
      eps_integer4 avh_scnlinqual = R.avh_scnlinqual;
      for( i = 0; i < 32; i++ ) {
        set_bit( ER.content.MDR_1B->SCAN_LINE_QUALITY, 31 - i, avh_scnlinqual % 2 );
        avh_scnlinqual = avh_scnlinqual >> 1;
      }
    }
/* 
    memcpy(
      &ER.content.MDR_1B->SCAN_LINE_QUALITY,
      &R.avh_scnlinqual, 4
    );
*/

    /* MDR/QUALITY_INDICATOR */ {
      int i;
      eps_integer4 avh_qualind = R.avh_qualind;
      for( i = 0; i < 32; i++ ) {
        set_bit( ER.content.MDR_1B->QUALITY_INDICATOR, 31 - i, avh_qualind % 2 );
        avh_qualind = avh_qualind >> 1;
      }
    }
/* 
    memcpy( 
      &ER.content.MDR_1B->QUALITY_INDICATOR, 
      &R.avh_qualind, 4 
    );
*/

    /* MDR/NAVIGATION_STATUS */ {
      int i;
      eps_integer4 avh_navstat = R.avh_navstat;
      for( i = 0; i < 32; i++ ) {
        set_bit( ER.content.MDR_1B->NAVIGATION_STATUS, 31 - i, avh_navstat % 2 );
	avh_navstat = avh_navstat >> 1;
      }
    }
/* 
    memcpy( 
      &ER.content.MDR_1B->NAVIGATION_STATUS, 
      &R.avh_navstat,
      4
    );
*/

  
/*
      subroutine avh_lbc(line, avh_rad, avh_bt, avh_qc, bit_qc,
     &                     missing, good_lines, bad_lines)
      integer line		! scan line number
      real avh_rad(5,2048) 	! radiances mW/(m2.sr.cm-1)
      real avh_bt(5,2048)	! brigthness temperature (K) and albedo
      integer avh_qc(2048)	! QC word for each avhrr fov
                                !    (bit 'bit_QC' is 0=good, 1=bad fov)
      integer bit_qc		! bit number for quality
      integer missing		! line quality  0=present   1=missing
      integer good_lines(6)
      integer bad_lines(6)
*/

    {  
      float avh_rad[2048*5];
      float avh_bt[2048*5];
      int avh_qc[2048];
      int bit_qc;
      int missing;
      int good_lines[6];
      int bad_lines[6];
    
      bzero( avh_rad, sizeof( avh_rad ) );
      bzero( avh_bt, sizeof( avh_bt ) );
      bzero( avh_qc, sizeof( avh_qc ) );
      bit_qc = 0;
      missing = 0;
      bzero( good_lines, sizeof( good_lines ) );
      bzero( bad_lines, sizeof( bad_lines ) );
    
    
      aapp_avh_lbc( 
        line, avh_rad, avh_bt, avh_qc, &bit_qc, 
        &missing, good_lines, bad_lines 
      );
  
      {
        int i, j;
      
        for( j = 0; j < 2; j++ ) {
          for( i = 0; i < 2048; i++ ) {
            ER.content.MDR_1B->SCENE_RADIANCES[j][i]
            = 100 * ( ( H.avh_h_albcnv[j][0] / 10. ) * 
            avh_bt[ 5 * i + j ] / ( 100. * 3.14159 ) );
          }
        }
        
        for( j = 2; j < 3; j++ ) {
          if( ( R.avh_scnlinbit % 2 ) == 0 ) { /* 3A */
            for( i = 0; i < 2048; i++ ) {
              ER.content.MDR_1B->SCENE_RADIANCES[j][i]
              = 10000 * ( ( H.avh_h_albcnv[j][0] / 10. ) * 
              avh_bt[ 5 * i + j ] / ( 100. * 3.14159 ) );
            }
          } else { /* 3B */
            for( i = 0; i < 2048; i++ ) {
              ER.content.MDR_1B->SCENE_RADIANCES[j][i]
              = 10000 * avh_rad[ 5 * i + j ];
            }
          }
        }
        
        for( j = 3; j < 5; j++ ) {
          for( i = 0; i < 2048; i++ ) {
            ER.content.MDR_1B->SCENE_RADIANCES[j][i]
            = 100 * avh_rad[ 5 * i + j ];
          }
        }
      
	/* Interpolate EARTH_LOCATIONS */ {
          const double RT = 6366;
          double zeni[51];
          double loni[51];
          double lati[51];
	  double lono[103];
	  double lato[103];
	  int i, j;
	  double t;

	  t = RT / ( RT + R.avh_scalti / 10. );

	  for( i = 0; i < 51; i++ ) {
            zeni[i] = R.avh_ang[i][1] / 1e2;
            lati[i] = R.avh_pos[i][0] / 1e4;
	    loni[i] = R.avh_pos[i][1] / 1e4;
          }

          z_intex_51_to_103( zeni, t, loni, lati, lono, lato );

	  for( i = 0; i < 103; i++ ) {
            ER.content.MDR_1B->EARTH_LOCATIONS[i][0] = lato[i] * 1e4;
            ER.content.MDR_1B->EARTH_LOCATIONS[i][1] = lono[i] * 1e4;
          }

        /* Interpolate ANGULAR_RELATIONS (linear approximation).
           Since the l1b file only stores the relative azimuth (solar minus
           satellite) we store this in the 3rd slot and leave 4th slot empty. */

          for( j = 0; j < 3; j++ ) {
            for( i = 0; i < 51; i++ ) {
              ER.content.MDR_1B->ANGULAR_RELATIONS[2*i+1][j] = R.avh_ang[i][j];
            }
            for( i = 2; i < 101; i+=2 ) {
              ER.content.MDR_1B->ANGULAR_RELATIONS[i][j] = 
                (0.5*ER.content.MDR_1B->ANGULAR_RELATIONS[i-1][j] + 
                 0.5*ER.content.MDR_1B->ANGULAR_RELATIONS[i+1][j]);
            }
            ER.content.MDR_1B->ANGULAR_RELATIONS[0][j] = 
              2.0*ER.content.MDR_1B->ANGULAR_RELATIONS[1][j] - 
                  ER.content.MDR_1B->ANGULAR_RELATIONS[2][j];
            ER.content.MDR_1B->ANGULAR_RELATIONS[102][j] = 
              2.0*ER.content.MDR_1B->ANGULAR_RELATIONS[101][j] - 
                  ER.content.MDR_1B->ANGULAR_RELATIONS[100][j];
          }

        }


        
        i = 0;
        ER.content.MDR_1B->EARTH_LOCATION_FIRST[0]
        = ER.content.MDR_1B->EARTH_LOCATIONS[i][0];
        ER.content.MDR_1B->EARTH_LOCATION_FIRST[1]
        = ER.content.MDR_1B->EARTH_LOCATIONS[i][1];
    
        i = 102;
        ER.content.MDR_1B->EARTH_LOCATION_LAST[0]
        = ER.content.MDR_1B->EARTH_LOCATIONS[i][0];
        ER.content.MDR_1B->EARTH_LOCATION_LAST[1]
        = ER.content.MDR_1B->EARTH_LOCATIONS[i][1];
    
      }
  
  
      ER.content.MDR_1B->TIME_ATTITUDE = R.avh_attangtime;
      
      ER.content.MDR_1B->EULER_ANGLE[0] = R.avh_rollang;
      ER.content.MDR_1B->EULER_ANGLE[1] = R.avh_pitchang;
      ER.content.MDR_1B->EULER_ANGLE[2] = R.avh_yawang;
      

      ER.content.MDR_1B->SPACECRAFT_ALTITUDE = R.avh_scalti;
      
      
/*
        <array name="ANGULAR_RELATIONS" length="$NP">
            <array length="4">
                <field type="integer2" scaling-factor="10^2" units="deg" 
                description="Angular relationships: 
                solar zenith angle, 
                satellite zenith angle, 
                solar azimuth angle, 
                satellite azimuth angle  
                (points 25 to 2025, each 40th point, 
                possible reduction to every 20th point)"/>
            </array>
        </array>
       integer*2 avh_ang(3,51)    !set of 3 angles in degrees for point 25
!                                  to point 2025 every 40 points
!                first angle  : 10^2 x (solar zenith angle )
!                second angle : 10^2 x (satellite zenith angle )
!                third angle  : 10^2 x (relative azimuth angle )
*/
      
     
     ER.content.MDR_1B->COUNT_ERROR_FRAME = R.avh_cbiterr;


     for( i = 0; i < 3; i++ ) {
      
        ER.content.MDR_1B->CH123A_CURVE_SLOPE1[i] 
        = R.avh_calvis[i][0][0] 
        * 10e7 / 10e10;
        ER.content.MDR_1B->CH123A_CURVE_INTERCEPT1[i] 
        = R.avh_calvis[i][0][1] 
        * 10e6 / 10e7;
        if (R.avh_calvis[i][0][2] >= 0)
          ER.content.MDR_1B->CH123A_CURVE_SLOPE2[i] 
          = R.avh_calvis[i][0][2]
          * 10e7 / 10e10;
        else 
          ER.content.MDR_1B->CH123A_CURVE_SLOPE2[i] 
          = (R.avh_calvis[i][0][2]+4294967296.0)
          * 10e7 / 10e10;
        ER.content.MDR_1B->CH123A_CURVE_INTERCEPT2[i] 
        = R.avh_calvis[i][0][3] 
        * 10e6 / 10e7;
        ER.content.MDR_1B->CH123A_CURVE_INTERCEPTION[i] 
        = R.avh_calvis[i][0][4];

        ER.content.MDR_1B->CH123A_TEST_CURVE_SLOPE1[i] 
        = R.avh_calvis[i][1][0] 
        * 10e7 / 10e10;
        ER.content.MDR_1B->CH123A_TEST_CURVE_INTERCEPT1[i] 
        = R.avh_calvis[i][1][1] 
        * 10e6 / 10e7;
        if (R.avh_calvis[i][1][2] >= 0)
          ER.content.MDR_1B->CH123A_TEST_CURVE_SLOPE2[i] 
          = R.avh_calvis[i][1][2] 
          * 10e7 / 10e10;
        else
          ER.content.MDR_1B->CH123A_TEST_CURVE_SLOPE2[i] 
          = (R.avh_calvis[i][1][2]+4294967296.0)
          * 10e7 / 10e10;
        ER.content.MDR_1B->CH123A_TEST_CURVE_INTERCEPT2[i] 
        = R.avh_calvis[i][1][3] 
        * 10e6 / 10e7;
        ER.content.MDR_1B->CH123A_TEST_CURVE_INTERCEPTION[i] 
        = R.avh_calvis[i][1][4];

        ER.content.MDR_1B->CH123A_PRELAUNCH_CURVE_SLOPE1[i] 
        = R.avh_calvis[i][2][0] 
        * 10e7 / 10e10;
        ER.content.MDR_1B->CH123A_PRELAUNCH_CURVE_INTERCEPT1[i] 
        = R.avh_calvis[i][2][1] 
        * 10e6 / 10e7;
        if (R.avh_calvis[i][2][2] >= 0)
          ER.content.MDR_1B->CH123A_PRELAUNCH_CURVE_SLOPE2[i] 
          = R.avh_calvis[i][2][2] 
          * 10e7 / 10e10;
        else
          ER.content.MDR_1B->CH123A_PRELAUNCH_CURVE_SLOPE2[i] 
          = (R.avh_calvis[i][2][2]+4294967296.0)
          * 10e7 / 10e10;
        ER.content.MDR_1B->CH123A_PRELAUNCH_CURVE_INTERCEPT2[i] 
        = R.avh_calvis[i][2][3] 
        * 10e6 / 10e7;
        ER.content.MDR_1B->CH123A_PRELAUNCH_CURVE_INTERCEPTION[i] 
        = R.avh_calvis[i][2][4];
 
        ER.content.MDR_1B->CH3B45_SECOND_TERM[i]
        = R.avh_calir[i][0][0] 
        * 10e9 / 10e6;
        ER.content.MDR_1B->CH3B45_FIRST_TERM[i]
        = R.avh_calir[i][0][1] 
        * 10e6 / 10e6;
        ER.content.MDR_1B->CH3B45_ZEROTH_TERM[i]
        = R.avh_calir[i][0][2] 
        * 10e6 / 10e6;
   
        ER.content.MDR_1B->CH3B45_TEST_SECOND_TERM[i]
        = R.avh_calir[i][1][0] 
        * 10e9 / 10e6;
        ER.content.MDR_1B->CH3B45_TEST_FIRST_TERM[i]
        = R.avh_calir[i][1][1] 
        * 10e6 / 10e6;
        ER.content.MDR_1B->CH3B45_TEST_ZEROTH_TERM[i]
        = R.avh_calir[i][1][2] 
        * 10e6 / 10e6;
   

     }
      
    }


    put_eps_avhrrl1b_6_5( &ER, fpw );

    free_eps_avhrrl1b_6_5( &ER );

  }


  fseek( fpw, offset[0], SEEK_SET );

  /* IPR */ {
  
    ER.content.IPR = (t_ipr*)calloc( 1, sizeof( t_ipr ) );
    
    ER.grh.record_class = IPR;
    ER.grh.instrument_group = GENERIC;
    ER.grh.record_subclass = 0;
    ER.grh.record_subclass_version = 1;
  
    ER.grh.record_size = size_ipr( ER.content.IPR );
  
    ER.grh.record_start.days = H.avh_h_startdatajd - JULIAN_OFFSET_2000_1950;
    ER.grh.record_stop.days  = H.avh_h_enddatajd   - JULIAN_OFFSET_2000_1950;
    
    ER.grh.record_start.milliseconds = H.avh_h_startdatatime;
    ER.grh.record_stop.milliseconds  = H.avh_h_enddatatime;
    

    ER.content.IPR->TARGET_RECORD_CLASS     = GIADR;
    ER.content.IPR->TARGET_INSTRUMENT_GROUP = AVHRR;
    ER.content.IPR->TARGET_RECORD_SUBCLASS  = 1;
    ER.content.IPR->TARGET_RECORD_OFFSET    = offset[1];

    put_eps_avhrrl1b_6_5( &ER, fpw );


    ER.content.IPR->TARGET_RECORD_CLASS     = GIADR;
    ER.content.IPR->TARGET_INSTRUMENT_GROUP = AVHRR;
    ER.content.IPR->TARGET_RECORD_SUBCLASS  = 2;
    ER.content.IPR->TARGET_RECORD_OFFSET    = offset[2];

    put_eps_avhrrl1b_6_5( &ER, fpw );


    ER.content.IPR->TARGET_RECORD_CLASS     = MDR;
    ER.content.IPR->TARGET_INSTRUMENT_GROUP = AVHRR;
    ER.content.IPR->TARGET_RECORD_SUBCLASS  = 2;
    ER.content.IPR->TARGET_RECORD_OFFSET    = offset[3];

    put_eps_avhrrl1b_6_5( &ER, fpw );

    free_eps_avhrrl1b_6_5( &ER );
  
  }

  fclose( fpw );
  fclose( fpr );
  
  return 0;
  
}
