/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Reads HIRS PFS L0, extract instrument packets
 *             and pass them to AAPP core routines.
 *
 * VERSION   : 1.5
 *
 * HISTORY   : 12/01/2006 : include "hirs-aapp.h"
 *             16/01/2006 : Add LIBS info
 *             26/01/2006 : Add SPACECRAFT_ID variable
 *                          Add call to aapp_finish
 *             01/02/2006 : Change basic types to eps_types
 *                          Make internal subroutines private
 *             02/06/2006 : Set exit code
 *             30/10/2006 : Add -ignore_degraded_inst_mdr
 *                          and -ignore_degraded_proc_mdr
 *                          options
 *             31/10/2006 : For dummy lines, set the date
 *                          of previous processed line,
 *                          so that if the last line of the pass
 *                          is a dummy, we have a valid header
 *             16/11/2006 : Fix broken lines datation
 * 
 * LIBS = metop_hirs metop_common ccsds obtutc eps_metopl0 eps_common
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ccsds.h"
#include "eps_metopl0.h"
#include "obtutc.h"
#include "common-main.h"
#include "common-aapp.h"
#include "hirs-aapp.h"
#include "eps_types.h"

#define SECONDS_HIRS_LINE 6.4

/*
 * Interface to AAPP's hirout
 */

static void main_hirout( 
  t_mphr *m0, char *buffer, eps_integer4 yr, 
  eps_integer4 msech, eps_integer4 systim[3] 
) {
/*
 *   integer*2 hirs(1152)      -- hirs scan line
 *   integer*2 hianag(1)       -- hirs analogue data
 *   integer yr                -- year of data (e.g. 1997).
 *   integer msech             -- tip msecs from midnight
 *   integer systim(3)         -- current year, dayno, msecs from midnight
 *   integer satid             -- noaa satellite id
 *   integer err               -- error flag (0=ok)
 *   logical*1 fullhi          -- set to true when hirs scan line filled
 *   logical*1 gotime          -- set to true when first valid tip time 
 *                                available
 *   integer*4 startorb,endorb -- orbit number (start and end)
 *   integer hirs3             -- if set to 1 assumes hirs 3 otherwise hirs 2
 */
  eps_integer4 satid;
  eps_integer4 err = 0;
  eps_integer4 fullhi = 1;
  eps_integer4 gotime = 1;
  eps_integer4 hirs3 = 1;
  char dummy[2304];
  char SPACECRAFT_ID[4];

  memcpy( SPACECRAFT_ID, m0->SPACECRAFT_ID, 3 );
  SPACECRAFT_ID[3] = '\0';
  satid = aapp_genscid( SPACECRAFT_ID );


  memset( dummy, 0, sizeof( dummy ) );
  buffer = ( buffer == NULL ) ? dummy : buffer;
  aapp_hirout( 
    buffer, 0, yr, msech, systim, satid, &err, fullhi, 
    gotime, m0->ORBIT_START, m0->ORBIT_END, hirs3
  );
}

/*
 * Callbacks for writing a record to AAPP file.
 */

/*
 * Here we record the current record number and the current
 * systim. We need this to provide dummy record with 
 * valid time information.
 * If the pass starts with dummy records, then these are
 * skipped until a valid MDR shows up.
 *
 */

static eps_real8 current_ij1950 = 0.;
static int current_record_number = 0;

static int hirs_l1b_write_record( char* buffer, t_viadr_0 *v0, t_mphr *m0 ) {
  packet_primary_header pph;
  packet_secondary_header psh;
  satellite_binary_time sbt;
  eps_integer4 yr;
  eps_integer4 msech;
  eps_integer4 systim[3];

  current_record_number++;

  ccsds_read_packet_primary_header( &pph, buffer );
  ccsds_read_packet_secondary_header( &psh, buffer + 6 );
  ccsds_read_satellite_binary_time( &sbt, buffer + 6 + 8 );

  printf( " %d %d %d %d %d %d %d | %d %d %d\n", 
    pph.version, 
    pph.type, 
    pph.secondary_header_flag, 
    pph.APID, 
    pph.sequence_flag, 
    pph.packet_seq_count, 
    pph.packet_length, 
    psh.days, 
    psh.msec, 
    psh.usec
  );

  {
    eps_real8 ij2000, ij1950;
    eps_integer4 iq[3];
        
    /* time correction using OBT */

    obtutc_viadr0_jj2000( &psh, &sbt, v0, m0, &ij2000 );
    obtutc_jj2000_to_jj1950( &ij2000, &ij1950 );

    aapp_xijq( &ij1950, iq );
    
    current_ij1950 = ij1950;
  
    yr    = systim[0] = iq[0];
            systim[1] = iq[1];
    msech = systim[2] = iq[2];
  
  }

  aapp_hirday( systim[1] );

  SWAP_INT_BUFFER( 
    buffer +  6 + 8 + 6, 2, 
    ( pph.packet_length + 1 - ( 8 + 6 + 2 ) ) / 2
  );
  
  main_hirout( m0, buffer +  6 + 8 + 6, yr, msech, systim );
    

}

static int hirs_l1b_write_dummy( t_viadr_0 *v0, t_mphr *m0 ) {
  eps_integer4 systim[3];
  eps_integer4 yr;
  eps_integer4 msech;
  eps_integer4 iq[3];
  eps_real8 ij1950;
  
  if( current_record_number == 0 ) 
    return -1;
  
  ij1950 = current_ij1950 + SECONDS_HIRS_LINE / ( 24. * 3600. );
  
  aapp_xijq( &ij1950, iq );
  
  yr    = systim[0] = iq[0];
          systim[1] = iq[1];
  msech = systim[2] = iq[2];
  
  
  main_hirout( m0, NULL, yr, msech, systim );
  printf( " DUMMY RECORD........................\n" );
  
  current_ij1950 = ij1950;
  
  return 1;
}

/*
 * Open, close, write header callbacks
 */

static const eps_integer4 hrsl = 4;

static int hirs_l1b_open( char* filename ) {
  aapp_init( hrsl, filename );
}

static int hirs_l1b_close() {
  aapp_finish( hrsl );
}

static int hirs_l1b_write_header() {
  eps_integer4 ios = 0;
  aapp_hrshdw( hrsl, &ios );
  return ios;
}

/*
 * main program, here we use the common loop
 */
    
int main( int argc, char* argv[] ) {

  if( argc < 3 ) {
    printf( "Usage: %s [ -ignore_degraded_inst_mdr "
            "-ignore_degraded_proc_mdr ] HIRS_xxx_00_M01_... .l1b\n", argv[0] );
    exit( 1 );
  }

  return common_main(
    argc, argv, hirs_l1b_open, hirs_l1b_write_header,
    hirs_l1b_write_record, hirs_l1b_write_dummy,
    hirs_l1b_close
  );

}
