/* Name: $Id$ */
/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2017/02
 #
 * AUTHOR    : Pascale Roquet from aapp1dpfs.c
 *
 * SYNOPSIS  : maia2pfs.c: Program to include maia cloudmask to PFS 1b.
 *
 * It takes three arguments:
 *         - Level 1b PFS file, converted to PFS format by aapp-eps_avhrrl1b
 *         - AAPP maia4 hdf5 file format
 *         - output level 1b PFS file with cloud mask.
 *
 * Input files (maia and 1b PFS) should be the same for start and end times, 
 * a succint control is done (line numbers and start date/time 
 * (but the precision of this control is only seconds (precision time start in pfs file))
 *
 *  In direct broadcast processing PFS L1B files are obtained with :
 *  PFS L0 -> decom-avhrr-metop -> avh_get_vis_coef -> avhrcl ->  aapp-eps_avhrrl1b 
 *  maia4 hdf5 file are obtained with :
 *  PFS LO -> decom-avhrr-metop-> avh_get_vis_coef -> avhrcl -> avhrrin -> convert_to_hdf5 -> maia4
 *
 * HISTORY   : 03/03/2017 : Creation
 *
 * LIBS = eps_avhrrl1b_6.5 aapp_avhrrl1c eps_common
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "eps_avhrrl1b_6.5.h"
#include "eps_grh.h"
#include "eps_types.h"
#include "eps_buffer.h"
#ifdef HAS_LIBHDF5_HL
#include "hdf5.h"
#include "hdf5_hl.h"
#endif

/* macro pour copier une chaine de bits de i1 a i2 */
#define get_ibit(x, i1, i2) \
  (x >> i1) & ( (1 << (i2-i1+1))-1 )
int yisleap(int year)
{
    return (year % 4 == 0 && year % 100 != 0) || (year % 400 == 0);
}
int get_yday(int year, int mon, int day)
{
    static const int days[2][13] = {
        {0, 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334},
        {0, 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335}
    };
    int leap = yisleap(year);
    return days[leap][mon] + day;
}
int main( int argc, char* argv[] ) {

#ifdef HAS_LIBHDF5_HL

  t_eps_avhrrl1b_6_5 ER;
  buffer BR, BH;
  FILE  *fpe, *fpw;
  char dset[80];
  int i, k;
  int decode_endian_aapp; 
  int cloudy, sea, cloudmask, qualflag, snowice, nonproc;

  hid_t file_id_maia, dataset_id;
  herr_t  status;
  int * cloud_mask_dset;
  int * fov_qual_dset;
  int * fov_qual;
  int landwaterbg;
  int input_info_qual;
  int Nb_lines,Nb_pixels;
  int start_year, start_day, start_time;
  int start_day_pfs, start_time_seconds_pfs;
  int * avh_mask;
  int nblines;
  
  encode_endian = AAPP_BIG_ENDIAN;
  
  if( argc < 4 ) {
    fprintf( stderr,"Usage: %s PFS.l1b AVHRR.l1d PFS.l1b\n", argv[0] );
    exit( 0 );
  }

  fpe = fopen( argv[1], "r" );
  if( fpe == NULL ) {
    fprintf(stderr, "Error opening readonly file %s\n", argv[1] );
    exit( 1 );
  }
  fprintf(stderr,"opening file %s\n",argv[2]);

  file_id_maia = H5Fopen(argv[2] , H5F_ACC_RDONLY, H5P_DEFAULT);
  if( file_id_maia < 0 ) {
    fprintf(stderr, "Error opening readonly file %s\n", argv[2] );
    exit( 1 );
  }

  fpw = fopen( argv[3], "w" );
  if( fpw == NULL ) {
    printf( "Error opening write file %s\n", argv[3] );
    exit( 1 );
  }

  /* read maia AVH5 */
   strcpy(dset,"/DATA/CloudMask");

   hsize_t dims[2];
   H5T_class_t classid;
   size_t  sizetyp;
   fprintf(stderr,"read info on dataset %s\n",dset);
   H5LTget_dataset_info(file_id_maia,dset, dims, &classid, &sizetyp);

   fprintf(stderr,"dimension dset %d %d %d %d\n", dims[0],dims[1], classid, sizetyp);
   cloud_mask_dset = calloc( (size_t) dims[0]* dims[1], sizetyp);
   fov_qual_dset =  calloc( (size_t) dims[0]* dims[1], sizetyp);
   nblines =  dims[0];

   if ( cloud_mask_dset == NULL || fov_qual_dset == NULL ) {
      fprintf( stderr,"Error calloc\n");
    exit( 1 );
   }

   fprintf(stderr,"read datasets\n"); 
   status = H5LTread_dataset_int(file_id_maia ,dset, cloud_mask_dset);
   if (status != 0 ) {
     printf( "Error read\n");
     exit( 1 );
   }   
   
   strcpy(dset,"/DATA/fov_qual");
   status = H5LTread_dataset_int(file_id_maia ,dset, fov_qual_dset);
   if (status != 0 ) {
     printf( "Error read\n");
    exit( 1 );   

   }
   /* todo : check fortran/C order raw/lines */

   strcpy(dset,"/HEADER/start_time");
   status = H5LTread_dataset_int(file_id_maia ,dset, &start_time);
   if (status != 0 ) {
     printf( "Error read\n");
    exit( 1 );   
   }
  strcpy(dset,"/HEADER/start_day");
   status = H5LTread_dataset_int(file_id_maia ,dset, &start_day);
   if (status != 0 ) {
     printf( "Error read\n");
    exit( 1 );   
   }
  strcpy(dset,"/HEADER/start_year");
   status = H5LTread_dataset_int(file_id_maia ,dset, &start_year);
   if (status != 0 ) {
     printf( "Error read\n");
    exit( 1 );   
   }
   printf ("Cloud Mask tart_year: %d, start_day: %d , start_time: %d \n", start_year, start_day, start_time); 

  /* loops on MDR */
  int mdr = -1 ;
  while (get_eps_avhrrl1b_6_5 (&ER, fpe))
    {
      k++;
      /* printf ("%d, %d\n", k, ER.grh.record_class); */
      if( ER.grh.record_class == MPHR ) {
        printf ("PRODUCT_NAME: %67.67s \n",ER.content.MPHR->PRODUCT_NAME);
        printf ("SENSING_START.YYYY: %d \n",ER.content.MPHR->SENSING_START.YYYY);
        printf ("SENSING_START.mm: %d \n",ER.content.MPHR->SENSING_START.MM);
        printf ("SENSING_START.dd: %d \n",ER.content.MPHR->SENSING_START.DD);
        printf ("SENSING_START.hh: %d \n",ER.content.MPHR->SENSING_START.HH);
        printf ("SENSING_START.mm: %d \n",ER.content.MPHR->SENSING_START.mm);
        printf ("SENSING_START.ss: %d \n",ER.content.MPHR->SENSING_START.ss);
        start_day_pfs = get_yday(ER.content.MPHR->SENSING_START.YYYY, 
                                 ER.content.MPHR->SENSING_START.MM,
                                 ER.content.MPHR->SENSING_START.DD);
        start_time_seconds_pfs = ER.content.MPHR->SENSING_START.HH * 60*60 + 
                                 ER.content.MPHR->SENSING_START.mm *60 +
                          	 ER.content.MPHR->SENSING_START.ss ;
        printf("AVHRR PFS start_day:%d ,milliseconds %d \n", start_day_pfs, start_time_seconds_pfs * 1000);
        if (  start_day_pfs != start_day ||
              ER.content.MPHR->SENSING_START.YYYY != start_year || 
              start_time_seconds_pfs !=  start_time / 1000 ){
          fprintf(stderr,"ERROR: files are incompatible: start time are different\n");
          exit(1);
         }
        printf ("TOTAL_MDR :%d \n",ER.content.MPHR->TOTAL_MDR);
        if ( ER.content.MPHR->TOTAL_MDR != nblines ) {
          fprintf(stderr,"ERROR: files are incompatible: number of lines is different\n");
          exit(1);
         }
      }
      if( ER.grh.record_class == MDR ) {
          /* lecture AAPP */
          mdr++;
          reset_buffer( &BR );
          avh_mask = &cloud_mask_dset [ mdr * 2048];
          fov_qual = &fov_qual_dset [ mdr * 2048];

          /* mise a jour masque du PFS + terre/mer + neige/glace */
          memset (ER.content.MDR_1B->CLOUD_INFORMATION, 0, 
                    sizeof (ER.content.MDR_1B->CLOUD_INFORMATION));
          for( i = 0; i < 2048; i++ ) {
#if DEBUG
              if (i == 0 ) fprintf(stderr, "%d %d %d\n", mdr,i,avh_mask[i]);
              if (i == 1 ) fprintf(stderr, "%d %d %d\n", mdr,i,avh_mask[i]);
              if (i == 2 ) fprintf(stderr, "%d %d %d\n", mdr,i,avh_mask[i]);
#endif
            cloudmask  = get_ibit ( avh_mask[i], 10, 12 );
            cloudy =  cloudmask == 1;
            landwaterbg = get_ibit (avh_mask[i], 15,17);
            sea =  landwaterbg == 0;
            snowice    = cloudmask==3 || cloudmask == 4;

            qualflag   = get_ibit ( avh_mask[i], 3, 4);
            input_info_qual = get_ibit( fov_qual[i],0,1);

            nonproc    = input_info_qual == 2;
            /* set_bit arguments are: pointer, bit position(0=MS bit), value.
               The EPS Generic Product Format Specification defines bit 0 as 
               the LS, so when setting bits according to Table 4.4.1.1 of the
               AVHRR PFS document we need to invert the bit number. */
            if ( (qualflag < 3) && (! nonproc) ) {
              if ( cloudy ) {
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-15, 1 );
              } else {
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-14, 1 );
              }
              if ( !sea ) {
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-1, 1 );
              }
              if ( snowice ) {
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-4, 1 );
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-5, 1 );
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-6, 1 );
                set_bit( ER.content.MDR_1B->CLOUD_INFORMATION[i], 15-7, 1 );
              }
            }
          }
           fflush( stdout );
        }

      /* ecriture PFS */

      put_eps_avhrrl1b_6_5 (&ER, fpw);
      free_eps_avhrrl1b_6_5 (&ER);

    }


  printf( "\n" ); fflush( stdout );
   (void) 
  (void) H5close();
  
  return 0;

#else

  printf("maia2pfs.exe has not been built with HDF-5\n");
  printf("Please run \"configure\" with --external-libs=\"-Lxxx/lib -lhdf5 -lhdf5_hl\"\n");
  printf("  and --external-includes=\"-Lxxx/include\"\n");
  printf("  where xxx is the location of the HDF-5 library\n");
  return 1;
  

#endif
}



