/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Reads in MHS in PFS L0, extracts packets
 *             and pass them to AAPP decoding routines.
 *             MHS packets should be passed by chunks of 
 *             three.
 *
 * VERSION   : 1.5
 *
 * HISTORY   : 12/01/2006 : include <strings.h>
 *                          include "common-main.h"
 *             16/01/2006 : Add LIBS info
 *             26/01/2006 : Add SPACECRAFT_ID variable
 *                          Add aapp_finish call
 *                          Fill dummies with zeroes
 *             01/02/2006 : Change types to eps_types
 *                          Make internal routines private
 *             02/06/2006 : Set exit code
 *             30/10/2006 : Add -ignore_degraded_inst_mdr
 *                          and -ignore_degraded_proc_mdr
 *                          options
 *             06/11/2006 : Correct datation for MHS
 *             09/11/2006 : Add 16s to line datation 
 *                          because traditionnal NOAA processing will
 *                          remove them.
 *             16/11/2006 : Fix broken lines datation
 * 
 * LIBS = metop_mhs metop_common ccsds obtutc eps_metopl0 eps_common
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include "ccsds.h"
#include "eps_metopl0.h"
#include "obtutc.h"
#include "mhs-aapp.h"
#include "common-aapp.h"
#include "common-main.h"
#include "eps_types.h"

#define SIZE_OF_MHS_PACKET 1300
#define SIZE_OF_AMSUA1_PACKET 620
#define SIZE_OF_AMSUA2_PACKET 620
#define SIZE_OF_AMSUB_PACKET 1950

#define SECONDS_MHS_LINE 8.

/*
 * Interface to AAPP's amsout. We take care of setting options and
 * dummies when needed.
 */

static void main_amsout( 
  t_mphr *m0, char *mhs, eps_integer4 yr, eps_integer4 msech, 
  eps_integer4 systim[3], eps_integer4 *ios 
) {
/* AAPP's amsout prototype
 * INTEGER*4 OPTN(12)     -- Option numbers
 * INTEGER*2 AMSUA1(620)  -- AMSU-A1 scan line
 * INTEGER*2 AMSUA2(156)  -- AMSU-A2 scan line
 * INTEGER*2 AMSUB(1950)  -- AMSU-B scan line
 * CHARACTER*1 MHS(3900)  -- 3 MHS scan lines
 * INTEGER*2 AMANAG(71)   -- AMSU Analogue data
 * INTEGER*2 AMVALID(71)  -- AMSU analog telemetry validity
 * INTEGER YR             -- Year of data (e.g. 1997).
 * INTEGER MSECA          -- Time of data
 * INTEGER SYSTIM(3)      -- System year, dayno, msecs from midnight
 * INTEGER SATID          -- NOAA Satellite id
 * INTEGER AERR           -- AMSU-A Error flag (0=OK)
 * INTEGER BERR(3)        -- AMSU-B Error flag (0=OK)
 * LOGICAL*1 FULLAM       -- Set to false when AMSU scan line written
 * LOGICAL*1 GOTIME       -- Set to TRUE when first TIP time read
 * INTEGER STARTORB       -- start orbit number
 * INTEGER ENDORB         -- end orbit number
 * LOGICAL*1 MHSPRESENT   -- Set to TRUE for NOAA-N,N'
 */
  eps_integer4 optn[12] = { 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0 };
  eps_integer4 satid;
  eps_integer4 aerr[1] = { 0 };
  eps_integer4 berr[3] = { 0, 0, 0 };

  hword amanag[71]; /* dummy analog data */
  hword amvalid[71]; /* dummy analog validity data */
  eps_integer4 fullam;
  eps_integer4 gotime;
  eps_integer4 mhspresent;
  char dummy[SIZE_OF_MHS_PACKET * 3];

  /* AAPP's amsout requires real dummy arrays */

  hword amsua1_dummy[SIZE_OF_AMSUA1_PACKET];
  hword amsua2_dummy[SIZE_OF_AMSUA2_PACKET];
  hword amsub_dummy[SIZE_OF_AMSUB_PACKET];
  eps_integer4 day;
  char SPACECRAFT_ID[4];
  eps_integer4 msecs;

  /* Fills dummies with zeroes */

  memset( amanag, 0, 71 * sizeof( hword ) );
  memset( amvalid, 0, 71 * sizeof( hword ) );
  memset( dummy, 0, SIZE_OF_MHS_PACKET * 3 );
  memset( amsua1_dummy, 0, SIZE_OF_AMSUA1_PACKET * sizeof( hword ) );
  memset( amsua2_dummy, 0, SIZE_OF_AMSUA2_PACKET * sizeof( hword ) );
  memset( amsub_dummy, 0, SIZE_OF_AMSUB_PACKET * sizeof( hword ) );
  memset( dummy, 0, SIZE_OF_MHS_PACKET * 3 );


  memcpy( SPACECRAFT_ID, m0->SPACECRAFT_ID, 3 );
  SPACECRAFT_ID[3] = '\0';
  satid = aapp_genscid( SPACECRAFT_ID );



  mhs = ( mhs == NULL ) ? dummy : mhs;


  fullam = 1;
  gotime = 1;
  mhspresent = 1;


  msecs = systim[2];
  day = systim[1];
  
  aapp_amsday( day, msecs );

  aapp_amsout(
    optn, (char*)amsua1_dummy, (char*)amsua2_dummy, 
    (char*)amsub_dummy, (char*)mhs, (char*)amanag,
    (char*)amvalid, yr, msech, systim, satid, aerr, berr,
    fullam, gotime, m0->ORBIT_START, m0->ORBIT_END, mhspresent
  );
}

/*
 * callbacks for opening, closing, writing header to AAPP 1B file
 */

static const eps_integer4 mhsl  = 4;

static int mhs_l1b_open( char* filename ) {
  aapp_init( mhsl, filename );
}

static int mhs_l1b_close() {
  aapp_finish( mhsl );
}

static int mhs_l1b_write_header() {
  eps_integer4 ios = 0;
  aapp_mhshdw( mhsl, &ios );
  return ios;
}


static eps_real8 current_ij1950 = 0.;
static int current_record_number = 0;

/*
 * AAPP's expects mhs packets to be packed in rows of three.
 * mhs_l1b_write_recordx3 sends three consecutive records
 * to AAPP.
 */
static int mhs_l1b_write_recordx3( 
  char *inst, char* head, t_viadr_0 *v0, t_mphr *m0 
) {
  packet_primary_header pph;
  packet_secondary_header psh;
  satellite_binary_time sbt;
  eps_integer4 yr;
  eps_integer4 msech;
  eps_integer4 systim[3];
  eps_integer4 day;
  eps_integer4 ios = 0;

  
  ccsds_read_pph_psh_sbt( &pph, &psh, &sbt, head );

  {
    eps_real8 ij2000, ij1950;
    eps_integer4 iq[3];


    if( pph.APID == 0 ) { /* invalid data */
      ij1950 = current_ij1950 + SECONDS_MHS_LINE / ( 24 * 3600. );
      
      if( current_record_number == 0 ) 
        return 0;
      
    } else {
      /* time correction using OBT */
      obtutc_viadr0_jj2000( &psh, &sbt, v0, m0, &ij2000 );
      obtutc_jj2000_to_jj1950( &ij2000, &ij1950 );
    }

    current_record_number += 3;

    
    current_ij1950 = ij1950;

    /* 
     * we add here 16s, because noaa processing
     * will remove these 16s
     */
    
    ij1950 += 16. / ( 24. * 3600. );
    
    aapp_xijq( &ij1950, iq );

    yr    = systim[0] = iq[0];
            systim[1] = iq[1];
    msech = systim[2] = iq[2];
    
    printf( " *  %d, %d, %d", systim[0], systim[1], systim[2] );
    
  }

  main_amsout( m0, inst, yr, msech, systim, &ios );

}

/*
 * mhs_l1b_write_record keeps mhs packets and calls mhs_l1b_write_recordx3
 * every third packet
 */

static int mhs_l1b_write_record( char *buffer, t_viadr_0 *v0, t_mphr *m0 ) {
  static char head[
    SIZE_OF_PACKET_PRIMARY_HEADER + 
    SIZE_OF_PACKET_SECONDARY_HEADER + 
    SIZE_OF_SATELLITE_BINARY_TIME
  ];
  static int i = 0;
  static char inst[SIZE_OF_MHS_PACKET * 3];
  packet_primary_header pph;
  packet_secondary_header psh;
  satellite_binary_time sbt;

  ccsds_read_pph_psh_sbt( &pph, &psh, &sbt, buffer );

  if( pph.APID != 0 )
    printf( " %d %d %d %d %d %d %d | %d %d %d | %d %d", 
      pph.version, 
      pph.type, 
      pph.secondary_header_flag, 
      pph.APID, 
      pph.sequence_flag, 
      pph.packet_seq_count, 
      pph.packet_length, 
      psh.days, 
      psh.msec, 
      psh.usec,
      sbt.coarse_time,
      sbt.fine_time
    );

  if( i == 0 ) {
    memcpy( head, buffer,
      SIZE_OF_PACKET_PRIMARY_HEADER + 
      SIZE_OF_PACKET_SECONDARY_HEADER + 
      SIZE_OF_SATELLITE_BINARY_TIME
    );
  }

  bzero( inst + i * SIZE_OF_MHS_PACKET, SIZE_OF_MHS_PACKET );
  memcpy( 
    inst + 12 + i * SIZE_OF_MHS_PACKET, buffer 
    + SIZE_OF_PACKET_PRIMARY_HEADER + SIZE_OF_PACKET_SECONDARY_HEADER 
    + SIZE_OF_SATELLITE_BINARY_TIME, 
    SIZE_OF_MHS_PACKET - 12
  );

  i++; i %= 3;

  if( i == 0 ) {
    mhs_l1b_write_recordx3( inst, head, v0, m0 );
  } 
  
  printf( "\n" );
}

static int mhs_l1b_write_dummy( t_viadr_0 *v0, t_mphr *m0 ) {
  eps_integer4 ios;
  char dummy[SIZE_OF_MHS_PACKET];
  memset( dummy, 0, sizeof( dummy ) );
  printf( " DUMMY RECORD............................................." );
  mhs_l1b_write_record( dummy, v0, m0 );
}

int main( int argc, char* argv[] ) {

  if( argc < 3 ) {
    printf( "Usage: %s [ -ignore_degraded_inst_mdr "
            "-ignore_degraded_proc_mdr ] MHSx_xxx_00_M01_... .l1b\n", argv[0] );
    exit( 1 );
  }

  return common_main(
    argc, argv, mhs_l1b_open,
    mhs_l1b_write_header, mhs_l1b_write_record,
    mhs_l1b_write_dummy, mhs_l1b_close
  );

}
