/* Name: $Id$ */
/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2017/02
 #
 * AUTHOR    : Pascale Roquet from aapp1dpfs.c
 *
 * SYNOPSIS  : pfs_2avh1c.c: Program to convert PFSAVHRR1B file in avh1c file.
 *
 * It takes three arguments:
 *         - Level 1b PFS file, converted to PFS format
 *         - output aapp 1c avhrr file without cloud mask
 *
 * 
 *
 * HISTORY   : 03/03/2017 : Creation
 *
 * LIBS = eps_avhrrl1b_6.5 aapp_avhrrl1c eps_common
 */
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>

#include "eps_avhrrl1b_6.5.h"
#include "eps_maia_avh1c.h"
#include "eps_buffer.h"
#include "eps_types.h"

/* 
** SETBIT(val,bit): set bit to 1 in val
** GETNBIT(val,p,n): reads n bits starting at position p 
                     in val and right justify
** PUTNBIT(val,put,p,n): put n bits from put to val starting at p
                     ( to be restricted to unsigned val)
#define SETBIT(val,bit) ((val) | (0x1<<(bit)))
#define GETNBIT(val,p,n) ( ((val) >> (p)) & ~(~0 << (n)) )
#define PUTNBIT(val,put,p,n) ( ((((val) >> ((p)+(n)))) << ((p)+(n))) | ((put) << (p)) | ((~(~0 << (p))) & (val)) )
*/
/* PFS SCENE_RADIANCES :
 Scene radiance for channels 1, 2, 3a or 3b, 4 & 5. 
 Channels 1,2, 3a in units of W/(m 2 .sr).
 Channels 3b, 4, 5 in units of mW/(m 2 .sr.cm -1 ).
 Channels 1,2,4 & 5 with scale factor = 2.
 Channels 3a or 3b with scale factor = 4.
*/
#define DAY_OFFSET_2000_1970 10957

int SETBIT( int *val, int bit )    { *val = *val | (0x1 << bit) ; }
int SETBIT16( short *val, int bit ){ *val = *val | (0x1 << bit) ; }
int GETNBIT(int  val, int p,int n) { 
    int num;
    num =(val >> p)  & ~(~0 << n) ; 
    return num;
}
int PUTNBIT(int *val,int put,int p,int n) { 
    *val= ( (*val >> (p+n)) << (p+n) ) | (put << p) | ((~(~0 << p)) & *val) ;
}

int day2000_to_yyyy_yday( int day2000, int* yyyy, int *yday ) {
  struct tm st;
  time_t t;
  t = ( DAY_OFFSET_2000_1970 + day2000 ) * 24 * 3600;
  if( localtime_r( &t, &st ) ) {
    *yyyy = 1900 + st.tm_year;
    *yday = 1 + st.tm_yday;
    return 1;
  } else
    return -1;
}

int planck( double *temp, const double *rad, double nu, double B, double C, int n ) { /* K, mW/(m2.sr/cm-1), cm-1 */
  const double c1 =1.1910659e-05, c2 =1.438833;
  int i;
 double myrad;
  for( i = 0; i < n; i++ ) {
    myrad=rad[i];
    if (myrad <= 0 ) myrad=1.e-12;
    temp[i] = ( c2 * nu / log( 1 + c1 * nu * nu * nu / myrad ) - B )/ C;
  }
}


int main( int argc, char* argv[] ) {
  FILE *fpr;
  FILE *fpw;
  t_eps_avhrrl1b_6_5 ER;
  t_header H;
  t_record R;
  buffer BH;
  buffer BR;
  int line = 0;
  int linedummy = 0;
  double wavenumbers[3]; /* cm-1 */
  double b[3], c[3], f[3];


  if( argc < 3 ) {
    fprintf( stderr, "Usage: %s AVHR_1C_xxx_... .l1c\n", argv[0] );
    exit( 1 );
  }
  
  fpr = fopen( argv[1], "r" );
  fpw = fopen( argv[2], "w" );

  memset( &H, 0, sizeof( H ) );
  memset( &R, 0, sizeof( R ) );
  
  encode_endian = AAPP_BYTE_ORDER;
  
  init_buffer( &BH, size_header( &H ) );
  init_buffer( &BR, size_record( &R ) );
  
  write_header( &H, &BH, 1 );
  
  fwrite( BH.p, BH.lp, 1, fpw ); 
  reset_buffer( &BH );

  while( get_eps_avhrrl1b_6_5( &ER, fpr ) ) {
    
    /* printf( " ER.grh.record_class = %d\n",  ER.grh.record_class); */ 
        
    if( ER.grh.instrument_group == DUMMY ) {
        ++linedummy;
        printf( " linedummy = %d\n", linedummy );
        if( linedummy > 10 ) {
            /* nombre de lignes mis a la valeur reelle */
            H.avh1c_h_scnlin = line;
            
            /* on ecrit le header */
            rewind( fpw );
            write_header( &H, &BH, 1 );
            fwrite( BH.p, BH.lp, 1, fpw );
            reset_buffer( &BH );
            return 0;
        }
    }
        
    if( ER.grh.instrument_group != DUMMY ) {
    
    switch( ER.grh.record_class ) {
      case MPHR:
      
        memcpy( H.avh1c_h_siteid, ER.content.MPHR->RECEIVING_GROUND_STATION, 3 );
        memcpy( H.avh1c_h_l1bsiteid, ER.content.MPHR->PROCESSING_CENTRE, 3 );
        H.avh1c_h_versnb = 4;
        H.avh1c_h_versyr = 2006;	/* premieres donnees au cms */
        H.avh1c_h_versdy = 332;		/* premieres donnees au cms */
        H.avh1c_h_hdrcnt = 1;
        
        {
          char *name = ER.content.MPHR->PARENT_PRODUCT_NAME_1; 
          int i = 0;
          printf( "%c",' ');
	  for( i = 0; i < 67; i++ ) {
          H.avh1c_h_pfsdataname[i] = ER.content.MPHR->PARENT_PRODUCT_NAME_1[i];
          printf( "%c", H.avh1c_h_pfsdataname[i]);
          }
          H.avh1c_h_pfsdataname[67] = ' ';
          printf( "\n");
                             
          for( ; ( i < 67 - 42 ) && name[0] == ' '; i++, name++ );
          memset( H.avh1c_h_dataname, ' ', 42 );
          memcpy( H.avh1c_h_dataname, name, 42 );
        }

        H.avh1c_h_startorbit = ER.content.MPHR->ORBIT_START;
        printf( " no orbite = %d\n", H.avh1c_h_startorbit);

	{
	  struct {
	    const char* name;
	    int satid;
	  } SATS[] = { 
	    { "M01", 11 }, 
	    { "M02", 12 }, 
	    { "M03", 13 }, 
	    { "M04", 14 }, /* test */
	    { NULL, 0 }
	  };
	  int i;
	  int satid = 0;
	  for( i = 0; SATS[i].name; i++ ) {
	    if( ! strncmp( SATS[i].name, ER.content.MPHR->SPACECRAFT_ID, 3 ) ) {
	      satid = SATS[i].satid;
	      break;
	    }
	  }
	  H.avh1c_h_satid = satid;
	  H.avh1c_h_instid= 1;		/* AVHRR */
          H.avh1c_h_datatyp = 4;	/* global */
	  
	  {
	    int yyyy, yday;
	    day2000_to_yyyy_yday( ER.grh.record_start.days, &yyyy, &yday );
            H.avh1c_h_startdatayr = yyyy;
            H.avh1c_h_startdatady = yday;
	    day2000_to_yyyy_yday( ER.grh.record_stop.days, &yyyy, &yday );
            H.avh1c_h_enddatayr   = yyyy;
            H.avh1c_h_enddatady   = yday;
	  }
          H.avh1c_h_startdatatime = ER.grh.record_start.milliseconds;
          H.avh1c_h_enddatatime   = ER.grh.record_stop.milliseconds;
	  
          H.avh1c_h_scnlin = ER.content.MPHR->TOTAL_MDR;
          printf( " TOTAL_MDR %d\n", H.avh1c_h_scnlin); 
          
          H.avh1c_h_callocscnlin = ER.content.MPHR->TOTAL_MDR - 
	  ER.content.MPHR->COUNT_DEGRADED_INST_MDR - ER.content.MPHR->COUNT_DEGRADED_PROC_MDR;
          H.avh1c_h_rascnod = 10e2 * ER.content.MPHR->RIGHT_ASCENSION;
        }
      
      break;  /* MPHR */
      
      case MDR: {
        memset( &R, 0, sizeof( R ) ); /* erase record */
        R.avh1c_scnlin = ++line;
        
	{
	  int yyyy, yday;
	  day2000_to_yyyy_yday( ER.grh.record_start.days, &yyyy, &yday );
          R.avh1c_scnlinyr = yyyy;
          R.avh1c_scnlindy = yday;
	}
        R.avh1c_scnlintime = ER.grh.record_start.milliseconds;
        
        if( line == 1 ) printf( " scnlintime de la premiere line %d\n", R.avh1c_scnlintime);
         
        if( ER.grh.instrument_group != DUMMY ) {
          t_mdr_1b* mdr1b = ER.content.MDR_1B;
	  
          /* avh_h_inststat1 */ {
            int i;
            eps_integer4 avh_h_inststat = 0;
            if( get_bit( mdr1b->DIGITAL_B_DATA, 0 ) == 1 ) SETBIT(&avh_h_inststat,15);   /* motor telemetry */
            if( get_bit( mdr1b->DIGITAL_B_DATA, 1 ) == 1 ) SETBIT(&avh_h_inststat,14);   /* elect telemetry */
            if( get_bit( mdr1b->DIGITAL_B_DATA, 2 ) == 1 ) SETBIT(&avh_h_inststat,13);   /* 1 */
            if( get_bit( mdr1b->DIGITAL_B_DATA, 3 ) == 1 ) SETBIT(&avh_h_inststat,12);   /* 2 */
            if( get_bit( mdr1b->DIGITAL_B_DATA, 8 ) == 1 ) SETBIT(&avh_h_inststat,11);   /* 3A=1 */
            if( get_bit( mdr1b->DIGITAL_B_DATA, 8 ) == 0 ) SETBIT(&avh_h_inststat,10);   /* 3B=1 */
            if( get_bit( mdr1b->DIGITAL_B_DATA, 6 ) == 1 ) SETBIT(&avh_h_inststat,9);    /* 4 */
            if( get_bit( mdr1b->DIGITAL_B_DATA, 7 ) == 1 ) SETBIT(&avh_h_inststat,8);    /* 5 */
             
	    if( line == 1 ) {
	      H.avh1c_h_inststat1 = avh_h_inststat;
	      if( get_bit( mdr1b->DIGITAL_B_DATA, 8 ) == 1 ) printf(" l1 canal 3A \n"); 	      
	      if( get_bit( mdr1b->DIGITAL_B_DATA, 8 ) == 0 ) printf(" l1 canal 3B \n"); 
	      
	    } else {
	      if( ( H.avh1c_h_inststat1 != avh_h_inststat ) 
	       && ( H.avh1c_h_statchrecnb == 0 ) )
	      {
	        H.avh1c_h_inststat2 = avh_h_inststat;
		H.avh1c_h_statchrecnb = line;
	        if( get_bit( mdr1b->DIGITAL_B_DATA, 8 ) == 1 ) printf(" canal 3A line %d \n",line); 	      
	        if( get_bit( mdr1b->DIGITAL_B_DATA, 8 ) == 0 ) printf(" canal 3B line %d \n",line); 
	      }
	    }
	  } /* avh_h_inststat1 */
	  
          /* avh_qualind */ {
            int i;
            eps_integer4 avh_qualind = 0;
	    for( i = 0; i < 32; i++ ) {
	      avh_qualind <<= 1;  
	      if( get_bit( mdr1b->QUALITY_INDICATOR, i ) ) {
	        avh_qualind++;
	      } 
	    }
	    R.avh1c_qualind = avh_qualind;
	    if(R.avh1c_qualind  != 0 ) printf(" Bad qualind line %d \n",line); 
          } /* avh_qualind */
	  
	  /* avh_scnlinqual */ {
            int i;
            eps_integer4 avh_scnlinqual = 0;
	    for( i = 0; i < 32; i++ ) {
	      avh_scnlinqual <<= 1;  
	      if( get_bit( mdr1b->SCAN_LINE_QUALITY, i ) ) 
	        avh_scnlinqual++;
	    }
	    R.avh1c_scnlinqual = avh_scnlinqual;
	    if(avh_scnlinqual != 0 ) printf(" Bad scnlinqual line %d \n",line); 
	  } /* avh_scnlinqual */
	
	  /* avh_calqual */ {
	    int i, j;
	    eps_integer2 avh_calqual;
	    for( j = 0; j < 3; j++ ) {
	      avh_calqual = 0;
	      for( i = 0; i < 16; i++ ) {
	        avh_calqual <<= 1;
	        if( get_bit( &( mdr1b->CALIBRATION_QUALITY[j][0] ), i ) )
	          avh_calqual++;
	      }
	      R.avh1c_calqual[j] = avh_calqual;
	    }
	  } /* avh_calqual */
          
          /* avh_scnlinbit */ {
           R.avh1c_scnlinbit =0;
           if( get_bit( mdr1b->DIGITAL_B_DATA, 8 ) == 0 ) SETBIT16(&R.avh1c_scnlinbit,0); /* 3B */
          } /* avh_scnlinbit */
          	  
	  H.avh1c_h_rollerr  = mdr1b->EULER_ANGLE[0];
	  H.avh1c_h_pitcherr = mdr1b->EULER_ANGLE[1];
	  H.avh1c_h_yawerr   = mdr1b->EULER_ANGLE[2];
	
	  R.avh1c_scalti = mdr1b->SPACECRAFT_ALTITUDE;

	  /*  avh_ang, avh_pos */ {
	    if( mdr1b->NUM_NAVIGATION_POINTS == 103 ) {
	      int i;
	      for( i = 0; i < 51; i++ ) {
	        R.avh1c_pos[i][0] = mdr1b->EARTH_LOCATIONS[2*i+1][0];
	        R.avh1c_pos[i][1] = mdr1b->EARTH_LOCATIONS[2*i+1][1];
	        R.avh1c_ang[i][0] = mdr1b->ANGULAR_RELATIONS[2*i+1][0]; /* solar zenith */
	        R.avh1c_ang[i][1] = mdr1b->ANGULAR_RELATIONS[2*i+1][1]; /* satellite zenith */
	        R.avh1c_ang[i][2] = abs( mdr1b->ANGULAR_RELATIONS[2*i+1][2]
	                               - mdr1b->ANGULAR_RELATIONS[2*i+1][3] ); /* satellite zenith */
	      }
	    } else if( mdr1b->NUM_NAVIGATION_POINTS == 51 ) {
	      int i;
	      for( i = 0; i < 51; i++ ) {
	        R.avh1c_pos[i][0] = mdr1b->EARTH_LOCATIONS[i][0];
	        R.avh1c_pos[i][1] = mdr1b->EARTH_LOCATIONS[i][1];
	        R.avh1c_ang[i][0] = mdr1b->ANGULAR_RELATIONS[i][0]; /* solar zenith */
	        R.avh1c_ang[i][1] = mdr1b->ANGULAR_RELATIONS[i][1]; /* satellite zenith */
	        R.avh1c_ang[i][2] = abs( mdr1b->ANGULAR_RELATIONS[i][2]
	                               - mdr1b->ANGULAR_RELATIONS[i][3] ); /* satellite zenith */
	      }
	    }
	  } /*  avh_ang, avh_pos */
	  
	  {
	    double rad[2048], temp[2048];
	    int i, j, j3;
	    	      
            if(  get_bit( mdr1b->DIGITAL_B_DATA, 8 ) == 1 )  /* 3A */
	      j3 = 3;
	    else 
	      j3 = 2; 
	    	    
	    for( j = 0; j < j3; j++ ) { 
	      for( i = 0; i < 2048; i++ ) {  
	        R.avh1c_btemps[i][j] = 100 * 10. * ( ER.content.MDR_1B->SCENE_RADIANCES[j][i] / ( j == 2 ? 100. : 1. ) ) * M_PI / f[j];
              } 
	    }
	    
	    for( j = j3; j < 5; j++ ) { 
	      for( i = 0; i < 2048; i++ ) {
	        rad[i] = ER.content.MDR_1B->SCENE_RADIANCES[j][i] / ( j == 2 ? 10000. : 100. );
	      }
	      planck( temp, rad, wavenumbers[j-2], b[j-2], c[j-2], 2048 );
	      for( i = 0; i < 2048; i++ ) {
	        R.avh1c_btemps[i][j] = 100 * temp[i]; 
	      }
	    }

	    if(R.avh1c_scnlin==3) printf(" line 3: RADIANCES ch3 %d\n",R.avh1c_btemps[1300][2]);
	    	  
	  }
        }
    
        write_record( &R, &BR, 1 );
	fwrite( BR.p, BR.lp, 1, fpw );
        reset_buffer( &BR );
    
        break;
      }   /* MDR */   
      
      case GIADR: if( ER.grh.record_subclass == 1 ) {

        wavenumbers[0] = ER.content.GIADR_RADIANCE->CH3B_CENTRAL_WAVENUMBER / 10e1;
        wavenumbers[1] = ER.content.GIADR_RADIANCE->CH4_CENTRAL_WAVENUMBER  / 10e2;
        wavenumbers[2] = ER.content.GIADR_RADIANCE->CH5_CENTRAL_WAVENUMBER  / 10e2 ;
	
	f[0] = ER.content.GIADR_RADIANCE->CH1_SOLAR_FILTERED_IRRADIANCE;
	f[1] = ER.content.GIADR_RADIANCE->CH2_SOLAR_FILTERED_IRRADIANCE;
	f[2] = ER.content.GIADR_RADIANCE->CH3A_SOLAR_FILTERED_IRRADIANCE;
        fprintf(stderr," CH1_SOLAR_FILTERED_IRRADIANCE:%f\n",ER.content.GIADR_RADIANCE->CH1_SOLAR_FILTERED_IRRADIANCE/10.);
	fprintf(stderr," CH2_SOLAR_FILTERED_IRRADIANCE:%f\n",ER.content.GIADR_RADIANCE->CH2_SOLAR_FILTERED_IRRADIANCE/10.);
	fprintf(stderr," CH3a_SOLAR_FILTERED_IRRADIANCE:%f\n",ER.content.GIADR_RADIANCE->CH3A_SOLAR_FILTERED_IRRADIANCE/10.);
	
	b[0] = -ER.content.GIADR_RADIANCE->CH3B_CONSTANT1 / 100000.;
	b[1] = -ER.content.GIADR_RADIANCE->CH4_CONSTANT1  / 100000.;
	b[2] = -ER.content.GIADR_RADIANCE->CH5_CONSTANT1  / 100000.;
	
	c[0] = 1. / ( ER.content.GIADR_RADIANCE->CH3B_CONSTANT2_SLOPE / 1000000. );
	c[1] = 1. / ( ER.content.GIADR_RADIANCE->CH4_CONSTANT2_SLOPE  / 1000000. );
	c[2] = 1. / ( ER.content.GIADR_RADIANCE->CH5_CONSTANT2_SLOPE  / 1000000. );
	
	/* printf(
            " wavenumber %d/%d/%d %d/%d/%d %d/%d/%d\n",
            ER.content.GIADR_RADIANCE->CH3B_CENTRAL_WAVENUMBER,
            ER.content.GIADR_RADIANCE->CH3B_CONSTANT1, 
            ER.content.GIADR_RADIANCE->CH3B_CONSTANT2_SLOPE,
            ER.content.GIADR_RADIANCE->CH4_CENTRAL_WAVENUMBER,
            ER.content.GIADR_RADIANCE->CH4_CONSTANT1, 
            ER.content.GIADR_RADIANCE->CH4_CONSTANT2_SLOPE,
            ER.content.GIADR_RADIANCE->CH5_CENTRAL_WAVENUMBER,
            ER.content.GIADR_RADIANCE->CH5_CONSTANT1, 
            ER.content.GIADR_RADIANCE->CH5_CONSTANT2_SLOPE
          ); */

	printf(
            " solar filters %d/%d/%d\n",
            ER.content.GIADR_RADIANCE->CH1_SOLAR_FILTERED_IRRADIANCE,
            ER.content.GIADR_RADIANCE->CH2_SOLAR_FILTERED_IRRADIANCE, 
            ER.content.GIADR_RADIANCE->CH3A_SOLAR_FILTERED_IRRADIANCE
        ); 

	printf(
            " wavenumber %f/%f/%f %f/%f/%f %f/%f/%f\n",
            wavenumbers[0],b[0], c[0],
            wavenumbers[1],b[1], c[1],
            wavenumbers[2],b[2], c[2]
        );
	
	break;
      } /* GIADR */
      
    } 	/* ER.grh.record_class */
        
    free_eps_avhrrl1b_6_5( &ER );
    
    }	/* ER.grh.instrument_group */
    }	/* get_eps_avhrrl1b_6_5 */

  H.avh1c_h_scnlin =  line ;
  printf( " NB LIGNES traitees %d\n", H.avh1c_h_scnlin);
  printf( " scnlintime de la derniere line %d\n", R.avh1c_scnlintime);

  rewind( fpw );
  write_header( &H, &BH, 1 );
  fwrite( BH.p, BH.lp, 1, fpw );
  reset_buffer( &BH );

  return 0;
}


