/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 *
 * SYNOPSIS  : ccsds packets decoding
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.0
 *
 */
#include "ccsds.h"
#include <string.h>

/*
 * ccsds_swap_int_buffer swaps bytes in an integer array
 * p is a pointer to the integer array, t is the size 
 * of an integer ( 1, 2, 4, 8 bytes ), and n is the size
 * of the array.
 *
 */

int ccsds_swap_int_buffer( char *p, int t, int n ) {
  int i, j;
  char c;
  for( i = 0; i < n; i++ ) {
    for( j = 0; j < t / 2; j++ ) {
      c = p[ t * i + j ];
      p[ t * i + j ] = p[ t * ( i + 1 ) - ( j + 1 ) ];
      p[ t * ( i + 1 ) - ( j + 1 ) ] = c;
    }
  }
}

/*
 * ccsds_read_packet_primary_header read the pph of a ccsds
 * packet
 */

int ccsds_read_packet_primary_header( 
  packet_primary_header *pph, char* buffer 
) {
  hword pi;

  memcpy( &pi, buffer, 2 );
  SWAP_INT_BUFFER( &pi, 2, 1 );

  pph->APID = pi & 0x7ff;                pi >>= 11;
  pph->secondary_header_flag = pi & 0x1; pi >>= 1;
  pph->type = pi & 0x1;                  pi >>= 1;
  pph->version = pi & 0x7;               pi >>= 3;

  memcpy( &pi, buffer + 2, 2 );
  SWAP_INT_BUFFER( &pi, 2, 1 );

  pph->packet_seq_count = pi & 0x3fff; pi >>= 14;
  pph->sequence_flag = pi & 0x3;       pi >>= 2;

  memcpy( &pi, buffer + 4, 2 );
  SWAP_INT_BUFFER( &pi, 2, 1 );

  pph->packet_length = pi;

}

/*
 * reads ccsds packet secondary header from buffer
 */

int ccsds_read_packet_secondary_header( 
  packet_secondary_header *psh, char* buffer
) {
  dword ts;

  memcpy( &ts, buffer, 8 );
  SWAP_INT_BUFFER( &ts, 8, 1 );

  psh->usec = ts & 0xffff;     ts >>= 16;
  psh->msec = ts & 0xffffffff; ts >>= 32;
  psh->days = ts & 0xffff;     ts >>= 16;

}

/*
 * reads satellite binary time from buffer
 */

int ccsds_read_satellite_binary_time( 
  satellite_binary_time *sbt, char* buffer
) {
  hword hw;
  qword qw;

  sbt->coarse_time = 0;
  sbt->fine_time = 0;

  memcpy( &qw, buffer + 0, 1 ); /* should be 0 */

  memcpy( &qw, buffer + 1, 1 );
  sbt->coarse_time += qw * 256 * 256;

  memcpy( &qw, buffer + 2, 1 );
  sbt->coarse_time += qw * 256;

  memcpy( &qw, buffer + 3, 1 );
  sbt->coarse_time += qw;

  memcpy( &hw, buffer + 4, 2 );
  SWAP_INT_BUFFER( &hw, 2, 1 );
  sbt->fine_time += hw;

  return 1;
}

/*
 * we read pph, psh, sbt in one shot
 */

int ccsds_read_pph_psh_sbt(
  packet_primary_header *pph, packet_secondary_header *psh,
  satellite_binary_time *sbt, char* buffer
) {
  ccsds_read_packet_primary_header( pph, buffer );
  ccsds_read_packet_secondary_header( 
    psh, buffer + SIZE_OF_PACKET_PRIMARY_HEADER );
  ccsds_read_satellite_binary_time( sbt,
    buffer + SIZE_OF_PACKET_PRIMARY_HEADER + SIZE_OF_PACKET_SECONDARY_HEADER 
  );
}

/* 
 * here, we decode a full ccsds from a file stream
 */

int ccsds_fread_packet( ccsds_packet* packet, FILE* fpr ) {
  char buf6[6];
  char buf8[8];

  if( fread( buf6, 6, 1, fpr ) != 1 ) {
    return -1;
  } 
  
  ccsds_read_packet_primary_header( &packet->pph, buf6 );
  
  if( packet->pph.secondary_header_flag == 1 ) {
    if( fread( buf8, 8, 1, fpr ) != 1 ) {
      return -1;
    } 
    ccsds_read_packet_secondary_header( &packet->psh, buf8 );
  }


  fseek( fpr, - ( 6 + 8 ), SEEK_CUR );

  packet->packet = (char*)malloc( packet->pph.packet_length + 1 + 6 );
  
  if( 
      fread( packet->packet, packet->pph.packet_length + 1 + 6, 1, fpr ) != 1 
  ) {
    free( packet->packet );
    return -1;
  }

  ccsds_read_satellite_binary_time( &packet->sbt, packet->packet
  + SIZE_OF_PACKET_PRIMARY_HEADER + SIZE_OF_PACKET_SECONDARY_HEADER );

  return packet->pph.packet_length + 1 + 6;
}

/*
 * ccsds_bread_packet reads a packet from a memory buffer
 */

int ccsds_bread_packet( ccsds_packet* packet, char* buf ) {
  ccsds_read_packet_primary_header( &packet->pph, buf );

  if( packet->pph.secondary_header_flag == 1 ) {
    ccsds_read_packet_secondary_header( &packet->psh, buf + 6 );
  }

  packet->packet = (char*)malloc( packet->pph.packet_length + 1 + 6 );
  memcpy( packet->packet, buf, packet->pph.packet_length + 1 + 6 );

  return packet->pph.packet_length + 1 + 6;
}

void ccsds_free_packet( ccsds_packet *packet ) {
  free( packet->packet );
}
