! Name: $Id: convert_avh1b.F 530 2017-06-27 11:07:32Z frna $
!
! Purpose: Convert avhrr level 1b PFS format to AAPP format (fixed record length)
!
! Method: This file contains a collection of Fortran subroutines that are 
!         called by the main "c" routine
!
! Current Code Owner: NWP SAF
!
! COPYRIGHT
!    This software was developed within the context of the EUMETSAT Satellite
!    Application Facility on Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 16 December 2003, between EUMETSAT and the
!    Met Office, UK, by one or more partners within the NWP SAF. The partners
!    in the NWP SAF are the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2005, EUMETSAT, All Rights Reserved.
!
! History:
! Version    Date     Comment
!
!  1.0      28/11/06  N C Atkinson
!  1.1      09/05/07  Fix endianness issues, especially the 3a/3b indicator
!                     Add ascending/descending indicator in scnlinbit
!                     Make azimuth angle agree with AAPP convention
!                     Update status words in header
!  1.2      22/08/07  Accommodate NOAA GAC with 409 spots per line.
!                     Interpolate to maintain 2048 points in output.
!  1.3      16/10/07  Shift GAC product by 2.5 samples - to remove observed
!                     offset
!  1.4      11/01/08  Bug fix in mdravh1b in converting "days" to year/day
!  1.5      25/02/09  Minimum count 1 in mode 1
!  1.6      04/07/11  Initialise avh_scnlin and limit to 32766.
!  1.7      10/02/14  Correct scaling of the quality flags (16->256, 256->65536,
!                     4096->16777216)
!  1.8      03/07/14  Separate IAND and ICHAR for portability.
!  1.9      03/02/16  Store the EUMETSAT CALIBRATION_QUALITY in avh_filler3.
!                     Store calibration scaling in avh_filler2.
!  1.10     07/03/14  Control negatives values in SCENES_RADIANCES
!
! Code Description:
!   FORTRAN 77, following AAPP standards
!
! Functions and subroutines called:
!-------------------------------------------------------------------------------

      subroutine openavh1b
      implicit none

#include "avh1b.h"
#include "nl1b.h"
#include "reclopt.h"    

      integer loc_rl, ios, iset, ichan
      
      character*1 cmode
      integer mode,factors(6)
      common /modecom / mode,factors
      
      loc_rl=reclavh/rldiv
      open(10,form='unformatted',access='direct',
     &          recl=loc_rl,status='new',iostat=ios)

      if (ios.ne.0)
     &  open(10,form='unformatted',access='direct',
     &          recl=loc_rl,status='unknown',iostat=ios)

      if(ios.ne.0) then
        write(*,*) 'Error opening 1b file'
        call aappexit(2)
      endif

      avh_h_siteid = 'XXX'
      avh_h_blank = ' '
      avh_h_l1bversnb = libl1bvers
      avh_h_l1bversyr = libl1bversyr
      avh_h_l1bversdy = libl1bversdy
      avh_h_reclg = 22016
      avh_h_blksz = 0
      avh_h_hdrcnt = 1
      avh_h_datatyp = 1       !1=LAC, 2=GAC, 3=HRPT
      avh_h_startdatayr = 0     !update later
      avh_h_scnlin = 0
      avh_scnlin = 0
      avh_h_callocscnlin = 0
      avh_h_misscnlin = 0
      avh_h_inststat1 = 0
      avh_h_inststat2 = 0
      avh_h_statchrecnb = 0

!See whether the user wants to limit output to 10 bits
      call getenv('CONVERT_AVH1B_MODE',cmode)
      read(cmode,*,iostat=ios) mode
      if (ios.ne.0) mode=0         !default is to keep PFS values
      if (mode.ne.0) then
        factors(1)=5   !ch 1
        factors(2)=10  !ch 2
        factors(3)=32  !ch 3a
        factors(4)=32  !ch 3b
        factors(5)=32  !ch 4
        factors(6)=32  !ch 5
      endif
           
      return
      end
      
!------------------------------------------------------------------------------
      subroutine finishavh1b
      implicit none
#include "avh1b.h"

      write (10, rec=1) avh_hd
      close (10)
      write(*,*) avh_h_scnlin,
     &     ' scans written to output file on unit 10'
      return
      end

!------------------------------------------------------------------------------
      subroutine mdravh1b(
     &           days,
     &           milliseconds,
     &           EARTH_VIEWS_PER_SCANLINE,
     &           SCENE_RADIANCES,
     &           NAVIGATION_STATUS,
     &           SPACECRAFT_ALTITUDE,
     &           NUM_NAVIGATION_POINTS,
     &           ANGULAR_RELATIONS,
     &           EARTH_LOCATIONS,
     &           QUALITY_INDICATOR,
     &           SCAN_LINE_QUALITY,
     &           CALIBRATION_QUALITY,
     &           DIGITAL_B_DATA)
      
      implicit none
      integer*2   days
      integer*4   milliseconds
      integer*2   EARTH_VIEWS_PER_SCANLINE
      integer*2   SCENE_RADIANCES(2048,5)
      character*1 NAVIGATION_STATUS(4)
      integer*4   SPACECRAFT_ALTITUDE
      integer*2   NUM_NAVIGATION_POINTS
      integer*2   ANGULAR_RELATIONS(4,103)
      integer*4   EARTH_LOCATIONS(2,103)
      character*1 QUALITY_INDICATOR(4)
      character*1 SCAN_LINE_QUALITY(4)
      character*1 CALIBRATION_QUALITY(2,3)
      character*1 DIGITAL_B_DATA(2)
      
#include "avh1b.h"
      
      integer ispot,ispot2,jspot,ichan,jchan,kchan,status,i
      integer*2 ch3a
      real dlon,w

      integer mode,factors(6)
      common /modecom / mode,factors
      
      if (avh_scnlin .ge. 32766) then
        if (avh_scnlin .eq. 32766) write(*,*) 'Max scans reached'
        avh_scnlin = 32767
        return
      endif

!Get year and day. Approximation valid 2000 to 2099.
!Note that "days" is numbered from 0 on 1 Jan 2000
      avh_scnlin = avh_scnlin + 1
      avh_scnlinyr = INT((days+0.1)/365.25)
      avh_scnlindy = days - INT(avh_scnlinyr*365.25 + 0.9) + 1
      avh_scnlinyr = avh_scnlinyr + 2000
      avh_scnlintime = milliseconds
      avh_qualind = ICHAR(QUALITY_INDICATOR(1))*16777216 +
     &              ICHAR(QUALITY_INDICATOR(2))*65536 +
     &              ICHAR(QUALITY_INDICATOR(3))*256 +
     &              ICHAR(QUALITY_INDICATOR(4))
      avh_scnlinqual = ICHAR(SCAN_LINE_QUALITY(1))*16777216 + 
     &                 ICHAR(SCAN_LINE_QUALITY(2))*65536 +
     &                 ICHAR(SCAN_LINE_QUALITY(3))*256 +
     &                 ICHAR(SCAN_LINE_QUALITY(4))

!Store the EUMETSAT quality flag in avh_filler3. Bits 8-15 contain NEdT * 100.
!Store the AAPP version in avh_calqual: only bits 0-2 and 4-7 are used.
      avh_filler3(1) = ICHAR(CALIBRATION_QUALITY(1,1))*256 + 
     &                 ICHAR(CALIBRATION_QUALITY(2,1))
      avh_filler3(2) = ICHAR(CALIBRATION_QUALITY(1,2))*256 + 
     &                 ICHAR(CALIBRATION_QUALITY(2,2))
      avh_filler3(3) = ICHAR(CALIBRATION_QUALITY(1,3))*256 + 
     &                 ICHAR(CALIBRATION_QUALITY(2,3))
      avh_calqual(1) = IAND(avh_filler3(1)+0,247)   !+0 to get default integer
      avh_calqual(2) = IAND(avh_filler3(2)+0,247)
      avh_calqual(3) = IAND(avh_filler3(3)+0,247)

      avh_navstat = ICHAR(NAVIGATION_STATUS(1))*16777216 + 
     &              ICHAR(NAVIGATION_STATUS(2))*65536 + 
     &              ICHAR(NAVIGATION_STATUS(3))*256 + 
     &              ICHAR(NAVIGATION_STATUS(4)) 
      avh_scalti = SPACECRAFT_ALTITUDE   !10*altitude in km
      
      i = ICHAR(DIGITAL_B_DATA(2))
      ch3a = IAND(i,128)/128       !1=3a, 0=3b
      avh_scnlinbit = 1-ch3a                 !AAPP l1b has 0=3a, 1=3b

      dlon = (EARTH_LOCATIONS(2,103) - EARTH_LOCATIONS(2,1))*1.0E-4
      if (dlon .lt. -180.0) dlon = dlon + 360.0
      if (dlon .gt. 180.0) dlon = dlon - 360.0
      if (dlon .gt. 0) then                      !AVHRR has right-to-left scan
        avh_scnlinbit = avh_scnlinbit - 32768    !bit 15 set for descending
      endif
      
!Update channel status words in the header.
!Note that the DIGITAL_B status words for channel 3A and 3B appear to be always
!1 in the PFS files, so don't use them.
      status = ICHAR(DIGITAL_B_DATA(1))*256      !Just bits 8-16
      status = IAND(status,62208)                !Clear bits 10-11
      if (ch3a .eq. 1) then
        status = IOR(status,2048)                !Set bit 11
      else
        status = IOR(status,1024)                !Set bit 10
      endif
      if (avh_h_inststat1.eq.0) then
        avh_h_inststat1 = status
      else if (status.ne.avh_h_inststat1.and.avh_h_inststat2.eq.0) then
        avh_h_inststat2 = status
        avh_h_statchrecnb = avh_scnlin
      endif

!Get Earth view data
      if (mode.eq.0) then     !mode 0 (16 bit scaled radiances)
        do ichan=1,5
          IF (EARTH_VIEWS_PER_SCANLINE.EQ.2048) THEN
            do ispot=1,2048
             if (SCENE_RADIANCES(ispot,ichan) .lt. -1000) THEN
! integer overflow
                SCENE_RADIANCES(ispot,ichan) = 32767
              endif
             if (SCENE_RADIANCES(ispot,ichan) .lt. 0) THEN
                SCENE_RADIANCES(ispot,ichan) = 0
              endif
              avh_hrpt(ichan,ispot) = SCENE_RADIANCES(ispot,ichan)
            enddo
          ELSE
            kchan = (ichan-1)*409
            do ispot=1,409        !GAC
              avh_hrpt(ichan,ispot*5) = SCENE_RADIANCES(ispot+kchan,1)
            enddo
          ENDIF
        enddo
      else                    !mode 1 (10 bit limit with minimum value 1)
        do ichan=1,5
          jchan = ichan
          if (ichan.ge.3+ch3a) jchan=ichan+1
          IF (EARTH_VIEWS_PER_SCANLINE.EQ.2048) THEN
            do ispot=1,2048
              avh_hrpt(ichan,ispot) = 
     &            MAX(SCENE_RADIANCES(ispot,ichan)/factors(jchan),1)
            enddo
          ELSE
            kchan = (ichan-1)*409
            do ispot=1,409        !GAC
  !temporarily store GAC samples in spots 5, 10, ... 2045
              avh_hrpt(ichan,ispot*5) = 
     &            MAX(SCENE_RADIANCES(ispot+kchan,1)/factors(jchan),1)
            enddo
          ENDIF
        enddo
      endif
      IF (EARTH_VIEWS_PER_SCANLINE.NE.2048) THEN  !Interpolate GAC for scan line
  !1st GAC sample is mean of spots 1-4, 2nd is mean of spots 6-9, last is mean
  !of spots 2041-2044

        do ichan=1,5
          do ispot=1,2                       !First 2 spots - no interpolation
            avh_hrpt(ichan,ispot) = avh_hrpt(ichan,5)
          enddo
          do ispot=2043,2048                 !Last 6 spots - no interpolation
            avh_hrpt(ichan,ispot) = avh_hrpt(ichan,2045)
          enddo
          do ispot=1,408
            do jspot=1,5
              if (jspot .ne. 3) then
                w = (jspot-0.5)/5.0          !samples 1,2,4,5
                avh_hrpt(ichan,ispot*5+jspot-3) = 
     &             avh_hrpt(ichan,ispot*5)*(1.0-w)
     &           + avh_hrpt(ichan,ispot*5+5)*w
              endif
            enddo
            avh_hrpt(ichan,ispot*5) =        !finally over-write the original
     &             avh_hrpt(ichan,ispot*5)*0.5
     &           + avh_hrpt(ichan,ispot*5+5)*0.5
          enddo
        enddo
      ENDIF

!Get navigation data
      do ispot=1,51
        IF (NUM_NAVIGATION_POINTS.GT.51) THEN
          ispot2 = ispot*2
        ELSE
          ispot2 = ispot
        ENDIF
        avh_pos(1,ispot) = EARTH_LOCATIONS(1,ispot2)
        avh_pos(2,ispot) = EARTH_LOCATIONS(2,ispot2)
        avh_ang(1,ispot) = ANGULAR_RELATIONS(1,ispot2)  !solar zenith
        avh_ang(2,ispot) = ANGULAR_RELATIONS(2,ispot2)  !satellite zenith
        avh_ang(3,ispot) = abs(ANGULAR_RELATIONS(3,ispot2)  !Relative azimuth
     &                    -ANGULAR_RELATIONS(4,ispot2))     ! = solar minus sat
        if (avh_ang(3,ispot) .gt. 18000) 
     &     avh_ang(3,ispot) = 36000-avh_ang(3,ispot)!To agree with standard AAPP
      enddo
      
! Save the data record to disk
      write (10, rec=avh_scnlin+avh_h_hdrcnt) avh_dt
      
! Update the header
      if (EARTH_VIEWS_PER_SCANLINE.EQ.409) avh_h_datatyp = 2  !GAC
      if (avh_h_startdatayr.eq.0) then
        avh_h_startdatajd = days + 18262     !from 00h,1 Jan 1950 
        avh_h_startdatayr = avh_scnlinyr
        avh_h_startdatady = avh_scnlindy
        avh_h_startdatatime = avh_scnlintime
      endif
      avh_h_enddatajd = days + 18262     !from 00h,1 Jan 1950 
      avh_h_enddatayr = avh_scnlinyr
      avh_h_enddatady = avh_scnlindy
      avh_h_enddatatime = avh_scnlintime
      avh_h_scnlin = avh_scnlin
! Check bits 27 and 28 of the quality indicator
      if ( IAND(avh_qualind,402653184).eq.0) then
        avh_h_callocscnlin = avh_h_callocscnlin + 1
      endif
      return
      end

!------------------------------------------------------------------------------
      subroutine mphravh1b(SPACECRAFT_ID, ORBIT_START, ORBIT_END,
     & X_VELOCITY, Y_VELOCITY, Z_VELOCITY, SEMI_MAJOR_AXIS,
     & ECCENTRICITY, INCLINATION, PERIGEE_ARGUMENT, RIGHT_ASCENSION,
     & MEAN_ANOMALY)
      implicit none
      character*3 SPACECRAFT_ID
      integer     ORBIT_START, ORBIT_END, X_VELOCITY, Y_VELOCITY, 
     &            Z_VELOCITY, SEMI_MAJOR_AXIS, ECCENTRICITY,
     &            INCLINATION, PERIGEE_ARGUMENT, RIGHT_ASCENSION,
     &            MEAN_ANOMALY
     
      integer id
      real velocity, period, height, eccen, mean_orbit
#include "avh1b.h"

      read(SPACECRAFT_ID,'(2X,I1)') id
      avh_h_satid = 10 + id               !see satid.txt file
      avh_h_smaxis = SEMI_MAJOR_AXIS*100  !convert from mm(?) to km*1E5
      avh_h_eccen = ECCENTRICITY*100      !convert scaling 10^6 -> 10^8
      avh_h_incli = INCLINATION*100       !deg*10^3 -> 10^5
      avh_h_argper = PERIGEE_ARGUMENT*100 !deg*10^3 -> 10^5
      avh_h_rascnod = RIGHT_ASCENSION*100 !deg*10^3 -> 10^5
      avh_h_manom = MEAN_ANOMALY*100      !deg*10^3 -> 10^5
      return
      end

!------------------------------------------------------------------------------
      subroutine giadravh1b(CH1_SOLAR_FILTERED_IRRADIANCE,
     &  CH1_EQUIVALENT_FILTER_WIDTH,
     &  CH2_SOLAR_FILTERED_IRRADIANCE,
     &  CH2_EQUIVALENT_FILTER_WIDTH,
     &  CH3A_SOLAR_FILTERED_IRRADIANCE,
     &  CH3A_EQUIVALENT_FILTER_WIDTH,
     &  CH3B_CENTRAL_WAVENUMBER,
     &  CH3B_CONSTANT1,
     &  CH3B_CONSTANT2_SLOPE,
     &  CH4_CENTRAL_WAVENUMBER,
     &  CH4_CONSTANT1,
     &  CH4_CONSTANT2_SLOPE,
     &  CH5_CENTRAL_WAVENUMBER,
     &  CH5_CONSTANT1,
     &  CH5_CONSTANT2_SLOPE);

      implicit none
      integer*2 CH1_SOLAR_FILTERED_IRRADIANCE,
     &  CH1_EQUIVALENT_FILTER_WIDTH,
     &  CH2_SOLAR_FILTERED_IRRADIANCE,
     &  CH2_EQUIVALENT_FILTER_WIDTH,
     &  CH3A_SOLAR_FILTERED_IRRADIANCE,
     &  CH3A_EQUIVALENT_FILTER_WIDTH
     
      integer*4 CH3B_CENTRAL_WAVENUMBER,
     &  CH3B_CONSTANT1,
     &  CH3B_CONSTANT2_SLOPE,
     &  CH4_CENTRAL_WAVENUMBER,
     &  CH4_CONSTANT1,
     &  CH4_CONSTANT2_SLOPE,
     &  CH5_CENTRAL_WAVENUMBER,
     &  CH5_CONSTANT1,
     &  CH5_CONSTANT2_SLOPE

#include "avh1b.h"
      
      integer iset,ichan
      real rad_to_albedo(3)

      integer mode,factors(6)
      common /modecom / mode,factors     
      
      avh_h_albcnv(1,1) = CH1_SOLAR_FILTERED_IRRADIANCE  !1E1 W/m^2 
      avh_h_albcnv(1,2) = CH2_SOLAR_FILTERED_IRRADIANCE  !1E1
      avh_h_albcnv(1,3) = CH3A_SOLAR_FILTERED_IRRADIANCE !1E1
      avh_h_albcnv(2,1) = CH1_EQUIVALENT_FILTER_WIDTH  !1E3 um
      avh_h_albcnv(2,2) = CH2_EQUIVALENT_FILTER_WIDTH  !1E3
      avh_h_albcnv(2,3) = CH3A_EQUIVALENT_FILTER_WIDTH !1E3
      avh_h_radtempcnv(1,1) = CH3B_CENTRAL_WAVENUMBER    !1E2 cm^-1
      avh_h_radtempcnv(1,2) = CH4_CENTRAL_WAVENUMBER     !1E3
      avh_h_radtempcnv(1,3) = CH5_CENTRAL_WAVENUMBER     !1E3
      avh_h_radtempcnv(2,1) = CH3B_CONSTANT1        !1E5 K
      avh_h_radtempcnv(2,2) = CH4_CONSTANT1         !1E5
      avh_h_radtempcnv(2,3) = CH5_CONSTANT1         !1E5
      avh_h_radtempcnv(3,1) = CH3B_CONSTANT2_SLOPE       !1E6 K/K
      avh_h_radtempcnv(3,2) = CH4_CONSTANT2_SLOPE        !1E6
      avh_h_radtempcnv(3,3) = CH5_CONSTANT2_SLOPE        !1E6
      
! Integrated radiance to albedo: A = 100*pi*R/F where F is solar flux

      rad_to_albedo(1) = 100 * 3.1415927 / (0.1*avh_h_albcnv(1,1))
      rad_to_albedo(2) = 100 * 3.1415927 / (0.1*avh_h_albcnv(1,2))
      rad_to_albedo(3) = 100 * 3.1415927 / (0.1*avh_h_albcnv(1,3))
      
C      write(*,*) 'rad_to_albedo=',rad_to_albedo

! Calibration coefs - same for all scans

! I/R channels
! AAPP scale factor for linear term is 1E6, units mW/(m^2.sr.cm^-1)
! The EPS scale factor is 1E2 for ch 4/5 and 1E4 for ch 3b

      do iset=1,2             !operational, pre-launch
        do ichan=1,1
          avh_calir(1,iset,ichan) = 0         !quadratic
          avh_calir(2,iset,ichan) = 1E2       !linear ch 3b
          avh_calir(3,iset,ichan) = 0         !offset
        enddo
        do ichan=2,3
          avh_calir(1,iset,ichan) = 0         !quadratic
          avh_calir(2,iset,ichan) = 1E4       !linear 
          avh_calir(3,iset,ichan) = 0         !offset
        enddo
      enddo

!Store scaling factors in avh_filler2, for use by avhrrin.
      avh_filler2(1) = 6 
      avh_filler2(2) = 6 
      avh_filler2(3) = 6 

! Visible channels
! AAPP scale factor is 1E10 for albedo
! EPS scale factor is 1E2 for ch 1/2, 1E4 for ch 3a, units W/(m^2.sr)

      do iset=1,3             !operational, test, pre-launch
        do ichan=1,2
          avh_calvis(1,iset,ichan) = 1E8*rad_to_albedo(ichan)   !slope 1
          avh_calvis(2,iset,ichan) = 0                          !intercept 1
          avh_calvis(3,iset,ichan) = 1E8*rad_to_albedo(ichan)   !slope 2
          avh_calvis(4,iset,ichan) = 0                          !intercept 2
          avh_calvis(5,iset,ichan) = 0                          !intersection
        enddo
        do ichan=3,3
          avh_calvis(1,iset,ichan) = 1E6*rad_to_albedo(ichan)   !slope 1
          avh_calvis(2,iset,ichan) = 0                          !intercept 1
          avh_calvis(3,iset,ichan) = 1E6*rad_to_albedo(ichan)   !slope 2
          avh_calvis(4,iset,ichan) = 0                          !intercept 2
          avh_calvis(5,iset,ichan) = 0                          !intersection
        enddo
      enddo

!Adjust the calibration coefs if the user want to limit counts to 10 bits

      if (mode.ne.0) then
        do iset=1,3             !operational, test, pre-launch
          do ichan=1,3
            avh_calvis(1,iset,ichan) =
     &                avh_calvis(1,iset,ichan)*factors(ichan)
            avh_calvis(3,iset,ichan) =
     &                avh_calvis(3,iset,ichan)*factors(ichan)
          enddo
        enddo
        do iset=1,2             !operational, pre-launch
          do ichan=1,3
            avh_calir(2,iset,ichan) = 
     &                avh_calir(2,iset,ichan)*factors(ichan+3)
          enddo
        enddo
      endif
      
      return
      end
