/* Name: $Id$
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/10
 *
 * SYNOPSIS  : implements low level read/write operations
 *             for PFS format ( and other ).
 *             There are read/write routines for different
 *             basic eps types, integer4, integer2, time, etc...
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.2
 *
 * HISTORY   : 20/10/2005 : Change BYTE_ORDER to AAPP_BYTE_ORDER
 *                          Change LITTLE_ENDIAN to AAPP_LITTLE_ENDIAN
 *                          Change BIG_ENDIAN to AAPP_BIG_ENDIAN
 *             08/02/2006 : Change print_string for printing characters
 *             09/02/2006 : Change format to %le in print_eps_real
 *             30/05/2006 : Correct bug in read_label
 *             03/08/2006 : Change eps_integer from 4b to 8b
 *             01/09/2006 : Change %d to %lld in write_ascii_* routines
 *             10/11/2006 : Change malloc to calloc in init_buffer
 *             11/08/2009 : Add read_eps_nreal_buffer, write_eps_nreal_buffer
 *                          and other minor updates
 *             01/09/2018 : Add hdf5 functions  (P. Brunel)
 *
 */
#include "eps_buffer.h"
#include "eps_types.h"
#include "eps_grh.h"
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#ifdef HAS_LIBHDF5
#include <hdf5.h>
#endif
/* default values */

int encode_endian = AAPP_BIG_ENDIAN;
int decode_endian = AAPP_BIG_ENDIAN;
int nreal_endian  = AAPP_BIG_ENDIAN;


#if defined( AAPP_BYTE_ORDER ) && AAPP_BYTE_ORDER == AAPP_LITTLE_ENDIAN

#define SWAP_READ_INT_BUFFER( p, n, t, k ) \
  do { if( decode_endian == AAPP_BIG_ENDIAN ) swap_int_buffer( (char*)( p ), n, t, k ); } while( 0 )
#define SWAP_WRITE_INT_BUFFER( p, n, t, k ) \
  do { if( encode_endian == AAPP_BIG_ENDIAN ) swap_int_buffer( (char*)( p ), n, t, k ); } while( 0 )

#else

#define SWAP_READ_INT_BUFFER( p, n, t, k ) \
  do { if( decode_endian == AAPP_LITTLE_ENDIAN ) swap_int_buffer( (char*)( p ), n, t, k ); } while( 0 )
#define SWAP_WRITE_INT_BUFFER( p, n, t, k ) \
  do { if( encode_endian == AAPP_LITTLE_ENDIAN ) swap_int_buffer( (char*)( p ), n, t, k ); } while( 0 )

#endif
#ifdef HAS_LIBHDF5
int write_epsh5_integer2( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset. */
   dataset_id = H5Dcreate(gid, dname, H5T_NATIVE_INT16, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT16, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (eps_integer2*)p);

   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;

}
int write_epsh5_integer4( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset. */
   dataset_id = H5Dcreate(gid, dname, H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (eps_integer4*)p);

   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;

}


int write_epsh5_uinteger1( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset. */
   dataset_id = H5Dcreate(gid, dname, H5T_NATIVE_UINT8, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_UINT8, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (eps_uinteger1*)p);

   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;

}

int write_epsh5_uinteger2( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset. */
   dataset_id = H5Dcreate(gid, dname, H5T_NATIVE_UINT16, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_UINT16, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (eps_uinteger2*)p);

   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;

}
int write_epsh5_uinteger4( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset. */
   dataset_id = H5Dcreate(gid, dname, H5T_NATIVE_UINT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_UINT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (eps_uinteger4*)p);

   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;

}

int write_epsh5_uinteger( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset. */
   dataset_id = H5Dcreate(gid, dname, H5T_NATIVE_UINT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_UINT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (eps_uinteger*)p);

   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;

}
int write_epsh5_vinteger4( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){
   hid_t       dataset_id, dataspace_id, grp_v;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */

   int n;
   int i;
   n = 1;
   for( i = 0; i < rank; i++ ) {
      n *= dims[i];
   }
//typedef struct {
  //eps_byte scale;
  //eps_integer4 value;
//} eps_vinteger4;
        
   char* scale = (char*)getmem( n );  
   int*  value = (int*)getmem( n *4);  

   for( i = 0; i < n; i++ ) {
      scale[i] = ((eps_vinteger4*)p)[i].scale;
      value[i] = ((eps_vinteger4*)p)[i].value;
    }

   /* Create the group. */
   grp_v = H5Gcreate(gid, dname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "scale", H5T_NATIVE_INT8, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT8, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (char*)scale);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset value */
   dataset_id = H5Dcreate(grp_v, "value", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   ///* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)value);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);

   /* Close the group. */
   status = H5Gclose(grp_v);

   //freemem( (eps_byte*)scale );
   freemem( (char*)scale );
   //freemem( (eps_integer4*)value );
   freemem( (int*)value );

   return 0;
}

int write_epsh5_ccsdstime( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){
   hid_t       dataset_id, dataspace_id, grp_v;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */

   int n;
   int i;
   n = 1;
   for( i = 0; i < rank; i++ ) {
      n *= dims[i];
   }
//typedef struct {
  //eps_uinteger2 days;
  //eps_uinteger4 milliseconds;
//} eps_ccsdstime;
        
   short* days = (short*)getmem( n *2);  
   int*  milliseconds = (int*)getmem( n *4);  

   for( i = 0; i < n; i++ ) {
      days[i] = ((eps_ccsdstime*)p)[i].days;
      milliseconds[i] = ((eps_ccsdstime*)p)[i].milliseconds;
    }

   /* Create the group. */
   grp_v = H5Gcreate(gid, dname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "days", H5T_NATIVE_INT16, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT16, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (char*)days);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset value */
   dataset_id = H5Dcreate(grp_v, "milliseconds", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   ///* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)milliseconds);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);

   /* Close the group. */
   status = H5Gclose(grp_v);

   //freemem( (eps_byte*)scale );
   freemem( (short*)days );
   //freemem( (eps_integer4*)value );
   freemem( (int*)milliseconds );

   return 0;
}

int write_epsh5_ShortCDSTime( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){
   hid_t       dataset_id, dataspace_id, grp_v;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */

   int n;
   int i;
   n = 1;
   for( i = 0; i < rank; i++ ) {
      n *= dims[i];
   }
        
   short* days = (short*)getmem( n *2);  
   int*  milliseconds = (int*)getmem( n *4);  

   for( i = 0; i < n; i++ ) {
      days[i] = ((ShortCDSTime*)p)[i].days;
      milliseconds[i] = ((ShortCDSTime*)p)[i].milliseconds;
    }

   /* Create the group. */
   grp_v = H5Gcreate(gid, dname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "days", H5T_NATIVE_INT16, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT16, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (char*)days);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset value */
   dataset_id = H5Dcreate(grp_v, "milliseconds", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   ///* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)milliseconds);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);

   /* Close the group. */
   status = H5Gclose(grp_v);

   //freemem( (eps_byte*)scale );
   freemem( (short*)days );
   //freemem( (eps_integer4*)value );
   freemem( (int*)milliseconds );

   return 0;
}


int write_epsh5_longgentime( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){
   hid_t       dataset_id, dataspace_id, grp_v;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */

   int n;
   int i;
   n = 1;
   for( i = 0; i < rank; i++ ) {
      n *= dims[i];
   }
//typedef struct {
  //eps_integer YYYY;
  //eps_integer MM;
  //eps_integer DD;
  //eps_integer HH;
  //eps_integer mm;
  //eps_integer ss;
  //eps_integer uuu;
//} eps_longgentime;
        
   int*YYYY= (int*)getmem( n *4);  
   int*MM  = (int*)getmem( n *4);  
   int*DD  = (int*)getmem( n *4);  
   int*HH  = (int*)getmem( n *4);  
   int*mm  = (int*)getmem( n *4);  
   int*ss  = (int*)getmem( n *4);  
   int*uuu = (int*)getmem( n *4);  

   for( i = 0; i < n; i++ ) {
      YYYY[i] = ((eps_longgentime*)p)[i].YYYY;
      MM[i]   = ((eps_longgentime*)p)[i].MM;
      DD[i]   = ((eps_longgentime*)p)[i].DD;
      HH[i]   = ((eps_longgentime*)p)[i].HH;
      mm[i]   = ((eps_longgentime*)p)[i].mm;
      ss[i]   = ((eps_longgentime*)p)[i].ss;
      uuu[i]  = ((eps_longgentime*)p)[i].uuu;
    }

   /* Create the group. */
   grp_v = H5Gcreate(gid, dname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "YYYY", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)YYYY);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "MM", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)MM);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "DD", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)DD);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "HH", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)HH);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "mm", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)mm);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "ss", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)ss);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "uuu", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)uuu);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);




   /* Close the group. */
   status = H5Gclose(grp_v);

   freemem( (int*)YYYY );
   freemem( (int*)MM );
   freemem( (int*)DD );
   freemem( (int*)HH );
   freemem( (int*)mm );
   freemem( (int*)ss );
   freemem( (int*)uuu );

   return 0;
}

int write_epsh5_gentime( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){
   hid_t       dataset_id, dataspace_id, grp_v;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */

   int n;
   int i;
   n = 1;
   for( i = 0; i < rank; i++ ) {
      n *= dims[i];
   }
//typedef struct {
  //eps_integer YYYY;
  //eps_integer MM;
  //eps_integer DD;
  //eps_integer HH;
  //eps_integer mm;
  //eps_integer ss;
//} eps_gentime;
        
   int*YYYY= (int*)getmem( n *4);  
   int*MM  = (int*)getmem( n *4);  
   int*DD  = (int*)getmem( n *4);  
   int*HH  = (int*)getmem( n *4);  
   int*mm  = (int*)getmem( n *4);  
   int*ss  = (int*)getmem( n *4);  

   for( i = 0; i < n; i++ ) {
      YYYY[i] = ((eps_gentime*)p)[i].YYYY;
      MM[i]   = ((eps_gentime*)p)[i].MM;
      DD[i]   = ((eps_gentime*)p)[i].DD;
      HH[i]   = ((eps_gentime*)p)[i].HH;
      mm[i]   = ((eps_gentime*)p)[i].mm;
      ss[i]   = ((eps_gentime*)p)[i].ss;
    }

   /* Create the group. */
   grp_v = H5Gcreate(gid, dname, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "YYYY", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)YYYY);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "MM", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)MM);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "DD", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)DD);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "HH", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)HH);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "mm", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)mm);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);

   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset scale */
   dataset_id = H5Dcreate(grp_v, "ss", H5T_NATIVE_INT32, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT32, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (int*)ss);
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);
   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);




   /* Close the group. */
   status = H5Gclose(grp_v);

   freemem( (int*)YYYY );
   freemem( (int*)MM );
   freemem( (int*)DD );
   freemem( (int*)HH );
   freemem( (int*)mm );
   freemem( (int*)ss );

   return 0;
}

int write_epsh5_string( char *p, hid_t gid, int n, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */

   hid_t       dtype;
   size_t      size;


   dtype = H5Tcopy (H5T_C_S1);
   status = H5Tset_size (dtype, n);

   hsize_t dims[1] =  {1};
   dataspace_id = H5Screate_simple(1, dims, NULL);

   dataset_id = H5Dcreate(gid, dname, dtype, dataspace_id, H5P_DEFAULT,
       H5P_DEFAULT, H5P_DEFAULT);

   status = H5Dwrite (dataset_id, dtype, H5S_ALL, H5S_ALL, H5P_DEFAULT, p);
   
   
   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;

}


int write_epsh5_byte( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset. */
   dataset_id = H5Dcreate(gid, dname, H5T_NATIVE_INT8, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_INT8, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (eps_byte*)p);

   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;

}


int write_epsh5_boolean( void *p, hid_t gid, int rank, hsize_t *dims, char *dname ){

   hid_t       dataset_id, dataspace_id;  /* identifiers */
   herr_t      status;   /* Create the data space for the dataset. */


   dataspace_id = H5Screate_simple(rank, dims, NULL);
   /* Create the dataset. */
   dataset_id = H5Dcreate(gid, dname, H5T_NATIVE_UINT8, dataspace_id, 
                          H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

   /* Write the data to the dataset using default transfer properties.*/
    status = H5Dwrite(dataset_id, H5T_NATIVE_UINT8, H5S_ALL, H5S_ALL,
         H5P_DEFAULT, (eps_boolean*)p);

   /* End access to the dataset and release resources used by it. */
   status = H5Dclose(dataset_id);

   /* Terminate access to the data space. */ 
   status = H5Sclose(dataspace_id);
   return 0;
}
#endif
static int swap_int_buffer( char *p, int t, int n, int k ) {
  int i, j;
  char c;
  for( i = 0; i < n; i++ ) {
    for( j = 0; j < t / 2; j++ ) {
      c = p[ t * i + j ];
      p[ t * i + j ] = p[ t * ( i + 1 ) - ( j + 1 ) ];
      p[ t * ( i + 1 ) - ( j + 1 ) ] = c;
    }
  }
}



int scan_buffer( const char* f, void* a, buffer* b, int l ) {
  sscanf( b->c, f, a );
  b->c += l;
}

int read_buffer( void *p, int n, buffer* w ) {
  int k = ( n > w->lc ) ? w->lc : n;
  memcpy( p, w->c, k );
  w->lc -= k;
  w->c += k;
  return k;
}

int write_buffer( void *p, int n, buffer* w ) {
  int k = ( n > w->lc ) ? w->lc : n;
  memcpy( w->c, p, k );
  w->lc -= k;
  w->c += k;
  return k;
}

int reset_buffer( buffer* w ) {
  w->c = w->p;
  w->lc = w->lp;
}

int init_buffer( buffer* w, int n ) {
  w->p = (char*)calloc( 1, n );
  w->c = w->p;
#ifdef DEBUG 
  memset( w->c, 0, n );
  printf( " init_buffer = 0x%x %d\n", w->p, n );
#endif
  w->lc = w->lp = n;
}

int free_buffer( buffer* w ) {
#ifdef DEBUG 
  printf( " free_buffer = 0x%x\n", w->p );
#endif
  free( w->p );
}

int status_buffer( buffer *w ) {
  return 1;
}


/* int methods */

int read_int_buffer( void *p, int t, int n, buffer* w ) {
  int k = 0;
  
  k = read_buffer( p, t * n, w );
  SWAP_READ_INT_BUFFER( p, t, n, k );
  
  return k;
}

int write_int_buffer( void *p, int t, int n, buffer* w ) {
  int k = 0;
  
  k = write_buffer( p, t * n, w );
  SWAP_WRITE_INT_BUFFER( w->c - k, t, n, k );
  
  return k;
}

int print_int( const char *string, char* a, FILE *fp, int s, int t, int ndim, const int *dims ) {
  int i;
  int k = t;
  int x;
  char label[16];
  int l;

  
  switch( ndim ) {
  
    case 0:
      l = 1;
    break;
    
    case 1:
      l = dims[0];
    break;

    default:
      fprintf( fp, string );
      fprintf( fp, "\n" );
      for( i = 1; i < ndim; i++ ) {
        k *= dims[i];
      }
      for( i = 0; i < dims[0]; i++ ) {
        sprintf( label, "[%d]", i ); 
        print_int( label, a + k * i, fp, s, t, ndim-1, dims + 1 );
      }
      return 0;
  }
  
  fprintf( fp, "%-50s = ", string );

  for( i = 0; i < l; i++ ) {
    x = ( 
      s ? ( t == 1 ? (( eps_integer1*)a)[i] : t == 2 ? (( eps_integer2*)a)[i] : t == 4 ? (( eps_integer4*)a)[i] : 0  )
        : ( t == 1 ? ((eps_uinteger1*)a)[i] : t == 2 ? ((eps_uinteger2*)a)[i] : t == 4 ? ((eps_uinteger4*)a)[i] : 0  )
    );
    fprintf( fp, "%d ", x );
  }
  
  fprintf( fp, "\n" );
  
  return 0;
}

int read_ascii_eps_uinteger( eps_uinteger *pi, buffer *B, int len ) {
  eps_uinteger x = 0;
  int i;
  for( i = 0; ( i < len ) && ( B->lc > 0 ); i++, B->lc--, B->c++ ) {
     if( B->c[0] == ' ' ) continue;
     if( ! isdigit( B->c[0] ) ) { x = 0; break; }
     x = 10 * x + B->c[0] - '0';
  }
  *pi = x;
}

int read_ascii_eps_integer( eps_integer *pi, buffer *B, int len ) {
  eps_integer x = 0;
  int i;
  int s = 1;
  for( i = 0; ( i < len ) && ( B->lc > 0 ); i++, B->lc--, B->c++ ) {
     if( B->c[0] == ' ' ) continue;
     if( B->c[0] == '-' ) { s = -1; continue; }
     if( ! isdigit( B->c[0] ) ) { x = 0; break; }
     x = 10 * x + B->c[0] - '0';
  }
  *pi = s * x;
}

int write_ascii_eps_integer( const char* label, eps_integer *pi, buffer *B, int len ) {
  char buf[256];
  int l, lbuf;
  eps_integer x;
  int s;
  

  sprintf( buf, "%-30s= ", label );
  lbuf = strlen( buf );
  if( lbuf < B->lc ) {
    memcpy( B->c, buf, lbuf );
    B->c += lbuf;
    B->lc -= lbuf;
  }

  if( *pi < 0 ) {
    x = - *pi; s = -1;
  } else {
    x = + *pi; s = +1;
  }
    

  for( l = 255; x; l-- ) {
    buf[l] = ( x % 10 ) + '0'; 
    x = x / 10;
  }
  if( l == 255 ) {
    buf[l] = '0';
    l--;
  }
  if( s < 0 ) {
    buf[l] = '-';
    l--;
  }
  if( l < 255 - len ) {
    memset( buf, 'x', 256 );
    l = 0;
    goto done;
  }
  while( l > 255 - len ) {
    buf[l] = ' ';
    l--;
  }
  l++;


done:
  if( len < B->lc ) {
    memcpy( B->c, buf + l, len );
    B->c += len;
    B->c[0] = '\n';
    B->c++;
    B->lc -= len + 1;
  }
}

int write_ascii_eps_uinteger( const char* label, eps_uinteger *pi, buffer *B, int len ) {
  char buf[256];
  int l, lbuf;
  eps_uinteger x;


  sprintf( buf, "%-30s= ", label );
  lbuf = strlen( buf );
  if( lbuf < B->lc ) {
    memcpy( B->c, buf, lbuf );
    B->c += lbuf;
    B->lc -= lbuf;
  }

  x = + *pi;

  for( l = 255; x; l-- ) {
    buf[l] = ( x % 10 ) + '0';
    x = x / 10;
  }
  if( l == 255 ) {
    buf[l] = '0';
    l--;
  }
  if( l < 255 - len ) {
    memset( buf, 'x', 256 );
    l = 0;
    goto done;
  }
  while( l > 255 - len ) {
    buf[l] = ' ';
    l--;
  }
  l++;


done:
  if( len < B->lc ) {
    memcpy( B->c, buf + l, len );
    B->c += len;
    B->c[0] = '\n';
    B->c++;
    B->lc -= len + 1;
  }
}


/* vint methods */

int read_vint_buffer( void *p, int t, int n, buffer* w ) {
  int k = 0;
  
  if( t == 4 ) {
    eps_vinteger4* vi = (eps_vinteger4*)p;
    int i;
    
    
    for( i = 0; i < n; i++ ) {
    
      k += read_buffer( &vi->scale, 1, w );
      k += read_buffer( &vi->value, t, w );
      SWAP_READ_INT_BUFFER( &vi->value, t, 1, k );
      vi++;
      
    }
    
    
  } else {
    
      fprintf( stderr, 
        "read_int_buffer does not support eps_vinteger%d ( not yet ! )\n", t 
      );
      exit( 1 );
    
  }
  
  
  return k;
}

int write_vint_buffer( void *p, int t, int n, buffer* w ) {
  int k = 0;
  
  if( t == 4 ) {
    eps_vinteger4* vi = (eps_vinteger4*)p;
    int i;
    int l;
    
    for( i = 0; i < n; i++ ) {
    
      l = write_buffer( &vi->scale, 1, w ); k += l;
      l = write_buffer( &vi->value, t, w ); k += l;
      SWAP_WRITE_INT_BUFFER( w->c - l, t, 1, k );
      vi++;
      
    }
    
    
  } else {
  
      fprintf( stderr, 
        "write_int_buffer does not support eps_vinteger%d ( not yet ! )\n", t 
      );
      exit( 1 );
  
  }
  
  return k;
}

int print_vint( const char *string, char* a, FILE *fp, int t, int ndim, const int *dims ) {
  int i;
  int k =  ( t == 2 ? sizeof( eps_vinteger2 ) : t == 4 ? sizeof( eps_vinteger4 ) : 0 );
  int xs, xv;
  char label[16];
  int l;

  
  switch( ndim ) {
  
    case 0:
      l = 1;
    break;
    
    case 1:
      l = dims[0];
    break;

    default:
      fprintf( fp, string );
      fprintf( fp, "\n" );
      for( i = 1; i < ndim; i++ ) {
        k *= dims[i];
      }
      for( i = 0; i < dims[0]; i++ ) {
        sprintf( label, "[%d]", i ); 
        print_vint( label, a + k * i, fp, t, ndim-1, dims + 1 );
      }
      return 0;
  }
  
  fprintf( fp, "%-50s = ", string );

  for( i = 0; i < l; i++ ) {
    xs = ( 
      t == 2 ? ((eps_vinteger2*)a)[i].scale : t == 4 ? ((eps_vinteger4*)a)[i].scale : 0 
    );
    xv = ( 
      t == 2 ? ((eps_vinteger2*)a)[i].value : t == 4 ? ((eps_vinteger4*)a)[i].value : 0 
    );
    fprintf( fp, "(%d,%d) ", xs, xv );
  }
  
  fprintf( fp, "\n" );
  
  return 0;
}

int scale_eps_integer( const eps_integer *pi, eps_real *pr, int n, float s ) {
  int i;
  for( i = 0; i < n; i++ ) {
    pr[i] = (eps_real)pi[i] / (eps_real)s;
  }
}

int scale_eps_integer2( const eps_integer2 *pi, eps_real4 *pr, int n, float s ) {
  int i;
  for( i = 0; i < n; i++ ) {
    pr[i] = (eps_real4)pi[i] / (eps_real4)s;
  }
}

int scale_eps_integer4( const eps_integer4 *pi, eps_real4 *pr, int n, float s ) {
  int i;
  for( i = 0; i < n; i++ ) {
    pr[i] = (eps_real4)pi[i] / (eps_real4)s;
  }
}

int scale_eps_uinteger( const eps_uinteger *pi, eps_real *pr, int n, float s ) {
  int i;
  for( i = 0; i < n; i++ ) {
    pr[i] = (eps_real)pi[i] / (eps_real)s;
  }
}

int scale_eps_uinteger2( const eps_uinteger2 *pi, eps_real4 *pr, int n, float s ) {
  int i;
  for( i = 0; i < n; i++ ) {
    pr[i] = (eps_real4)pi[i] / (eps_real4)s;
  }
}

int scale_eps_uinteger4( const eps_uinteger4 *pi, eps_real4 *pr, int n, float s ) {
  int i;
  for( i = 0; i < n; i++ ) {
    pr[i] = (eps_real4)pi[i] / (eps_real4)s;
  }
}

int scale_eps_vinteger4( const eps_vinteger4 *pi, eps_real4 *pr, int n, float s ) {
  int i;
  static float S[] = { 
    1E00, 1E01, 1E02, 1E03, 1E04, 1E05, 1E06, 1E07, 1E08, 1E09, 
    1E10, 1E11, 1E12, 1E13, 1E14, 1E15, 1E16, 1E17, 1E18, 1E19, 
    1E20, 1E21, 1E22, 1E23, 1E24, 1E25, 1E26, 1E27, 1E28, 1E29, 
  };
  for( i = 0; i < n; i++ ) {
    pr[i] = (eps_real4)pi[i].value / ( (eps_real4)s * (eps_real4) S[ pi[i].scale ] );
  }
}



/* eps_real methods */

int read_eps_real_buffer( void *p, int t, int n, buffer* w ) {
  return read_int_buffer( p, t, n, w );
}

int write_eps_real_buffer( void *p, int t, int n, buffer* w ) {
  return write_int_buffer( p, t, n, w );
}

int read_eps_nreal_buffer( void *p, int t, int n, buffer* w ) {
  int k = 0;
  
  k = read_buffer( p, t * n, w );
  if( nreal_endian != AAPP_BYTE_ORDER )
    SWAP_READ_INT_BUFFER( p, t, n, k );
  
  return k;
}

int write_eps_nreal_buffer( void *p, int t, int n, buffer* w ) {
  int k = 0;
  
  k = write_buffer( p, t * n, w );
  if( nreal_endian != AAPP_BYTE_ORDER )
    SWAP_WRITE_INT_BUFFER( p, t, n, k );
  
  return k;
}

int print_eps_real( const char *string, char* a, FILE *fp, int t, int ndim, const int *dims ) {
  int i;
  int k = t;
  double x;
  char label[16];
  int l;

  
  switch( ndim ) {
  
    case 0:
      l = 1;
    break;
    
    case 1:
      l = dims[0];
    break;

    default:
      fprintf( fp, string );
      fprintf( fp, "\n" );
      for( i = 1; i < ndim; i++ ) {
        k *= dims[i];
      }
      for( i = 0; i < dims[0]; i++ ) {
        sprintf( label, "[%d]", i ); 
        print_eps_real( label, a + k * i, fp, t, ndim-1, dims + 1 );
      }
      return 0;
  }
  
  fprintf( fp, "%-50s = ", string );

  for( i = 0; i < l; i++ ) {
    x = ( 
      t == 4 ? ((eps_real4*)a)[i] : t == 8 ? ((eps_real8*)a)[i] : 0 
    );
    fprintf( fp, "%le ", x );
  }
  
  fprintf( fp, "\n" );
  
  return 0;
}

/* eps_bitfield methods */

int read_ascii_eps_bitfield( void* p, buffer *w, int t ) {
  int i, j;
  unsigned char x, *px;
  char b;
  
  px = (unsigned char*)p;
  for( i = 0; i < ( t / 8 ); i++ ) {
    x = 0;
    for( j = 0; j < 8; j++ ) {
      read_buffer( &b, 1, w );
      x = x * 2 + ( ( b == '1' ) ? 1 : 0 );
    }
    px[i] = x;
  }
}

int read_eps_bitfield_buffer( void *p, int t, int n, buffer* w ) {
  int k = read_buffer( p, t * n, w );
  return k;
}

int write_ascii_eps_bitfield( const char *label, void *p, buffer *w, int t ) {
  char buf[256];
  int l;
  int i, j;
  unsigned char x, *px;
  
  sprintf( buf, "%-30s= ", label );
  px = (unsigned char*)p;
  
  for( i = 0; i < ( t / 8 ); i++ ) {
    x = px[i];
    for( j = 0; j < 8; j++ ) {
      buf[ 31 + ( i + 1 ) * 8 - j ] = ( x % 2 ) ? '1' : '0';
      x /= 2;
    }
  }
  buf[ 32 + t ] = '\n';
  write_buffer( buf, 32 + t + 1, w );
}



int write_eps_bitfield_buffer( void *p, int t, int n, buffer* w ) {
  int k = write_buffer( p, t * n, w );
  return k;
}

int print_ascii_eps_bitfield( ) {

}

int print_eps_bitfield( const char *string, char* c, FILE *fp, int t, int ndim, const int *dims ) {
  int i, j;
  int k = 1;
  char label[16];
  int l;
  unsigned char* a = (unsigned char*)c;

  
  switch( ndim ) {
  
    case 0:
      l = 1;
    break;
    
    case 1:
      l = dims[0];
    break;

    default:
      fprintf( fp, string );
      fprintf( fp, "\n" );
      for( i = 1; i < ndim; i++ ) {
        k *= dims[i];
      }
      for( i = 0; i < dims[0]; i++ ) {
        sprintf( label, "[%d]", i ); 
        print_eps_bitfield( label, (char*)(a + k * i), fp, t, ndim-1, dims + 1 );
      }
      return 0;
  }
  
  fprintf( fp, "%-50s = ", string );


  k = 0;
  for( i = 0; i < l; i++ ) {
    for( j = 0; j < t / 8; j++ ) {
      fprintf( fp, "%2.2x", (int)a[k] );
      k++;
    }
    fprintf( fp, " " );
  }
  
  fprintf( fp, "\n" );
  
  return 0;
}

int get_bit( char* eps_bitfield, int nbit ) {
  while( nbit >= 8 ) { eps_bitfield++; nbit -= 8; }
  return ( 0x1 & ( *eps_bitfield >> ( 8 - nbit - 1 ) ) );
}

int set_bit( char* eps_bitfield, int nbit, int b ) {
  while( nbit >= 8 ) { eps_bitfield++; nbit -= 8; }
  /* Ugly */
  *eps_bitfield = b 
            ? *eps_bitfield |   ( 1 << ( 8 - nbit - 1 ) )
            : *eps_bitfield & ~ ( 1 << ( 8 - nbit - 1 ) );
}

/* eps_ccsdstime methods */

int read_eps_ccsdstime_buffer( eps_ccsdstime *T, int n, buffer* w ) {
  int k = 0;
  int i;
  for( i = 0; i < n; i++ ) {
    k += read_int_buffer( &T->days, 2, 1, w );
    k += read_int_buffer( &T->milliseconds, 4, 1, w );
    T++;
  }
  return k;
}

int write_eps_ccsdstime_buffer( eps_ccsdstime *T, int n, buffer* w ) {
  int k = 0;
  int i;
  for( i = 0; i < n; i++ ) {
    k += write_int_buffer( &T->days, 2, 1, w );
    k += write_int_buffer( &T->milliseconds, 4, 1, w );
    T++;
  }
  return k;
}

int print_eps_ccsdstime( const char *string, eps_ccsdstime* a, FILE *fp, int ndim, const int *dims ) {
  int i;
  int k = sizeof( eps_ccsdstime );
  char label[16];
  int l;

  
  switch( ndim ) {
  
    case 0:
      l = 1;
    break;
    
    case 1:
      l = dims[0];
    break;

    default:
      fprintf( fp, string );
      fprintf( fp, "\n" );
      for( i = 1; i < ndim; i++ ) {
        k *= dims[i];
      }
      for( i = 0; i < dims[0]; i++ ) {
        sprintf( label, "[%d]", i ); 
        print_eps_ccsdstime( label, a + k * i, fp, ndim-1, dims + 1 );
      }
      return 0;
  }
  
  fprintf( fp, "%-50s = ", string );

  for( i = 0; i < l; i++ ) {
    fprintf( fp, "(%d,%d) ", a[i].days, a[i].milliseconds );

  }
  
  fprintf( fp, "\n" );
  
  return 0;
}

/* ccsds stuff pph, psh, obt */


int read_pph_buffer( t_pph *pph, int n, buffer* w ) {
  eps_uinteger2 pi;
  int i;
  
  for( i = 0; i < n; i++, pph++ ) {

    read_buffer( &pi, 2, w );
    SWAP_READ_INT_BUFFER( &pi, 2, 1, 2 );

    pph->APID = pi & 0x7ff;                pi >>= 11;
    pph->secondary_header_flag = pi & 0x1; pi >>= 1;
    pph->type = pi & 0x1;                  pi >>= 1;
    pph->version = pi & 0x7;               pi >>= 3;

    read_buffer( &pi, 2, w );
    SWAP_READ_INT_BUFFER( &pi, 2, 1, 2 );

    pph->packet_seq_count = pi & 0x3fff; pi >>= 14;
    pph->sequence_flag = pi & 0x3;       pi >>= 2;

    read_buffer( &pi, 2, w );
    SWAP_READ_INT_BUFFER( &pi, 2, 1, 2 );

    pph->packet_length = pi;
  
  }
}

int write_pph_buffer( t_pph *P, int n, buffer* w ) {

}

int read_psh_buffer( t_psh *psh, int n, buffer* w ) {
  eps_uinteger8 ts;
  int i;

  for( i = 0; i < n; i++, psh++ ) { 
    read_buffer( &ts, 8, w );
    SWAP_READ_INT_BUFFER( &ts, 8, 1, 8 );

    psh->usec = ts & 0xffff;     ts >>= 16;
    psh->msec = ts & 0xffffffff; ts >>= 32;
    psh->days = ts & 0xffff;     ts >>= 16;
  }

}

int write_psh_buffer( t_psh *P, int n, buffer* w ) {

}

int read_obt_buffer( t_obt *sbt, int n, buffer* w ) {
  eps_uinteger2 hw;
  eps_uinteger1 qw;
  int i;
  
  for( i = 0; i < n; i++, sbt++ ) {

    sbt->coarse_time = 0;
    sbt->fine_time = 0;

    read_buffer( &qw, 1, w ); /* should be 0 */

    read_buffer( &qw, 1, w );
    sbt->coarse_time += qw * 256 * 256;

    read_buffer( &qw, 1, w );
    sbt->coarse_time += qw * 256;

    read_buffer( &qw, 1, w );
    sbt->coarse_time += qw;

    read_buffer( &hw, 2, w );
    SWAP_READ_INT_BUFFER( &hw, 2, 1, 8 );
    sbt->fine_time += hw; 

  }

  return 1;

}

int write_obt_buffer( t_obt *O, int n, buffer* w ) {

}

int size_pph( t_pph *pph ) {
  return SIZE_OF_PACKET_PRIMARY_HEADER;
}

int size_psh( t_psh *psh ) {
  return SIZE_OF_PACKET_SECONDARY_HEADER; 
}

int size_obt( t_obt *obt ) {
  return SIZE_OF_SATELLITE_BINARY_TIME;
}





/* eps_byte methods */

int read_eps_byte_buffer( eps_byte *p, int n, buffer* w ) {
  return read_buffer( p, n, w );
}

int write_eps_byte_buffer( eps_byte *p, int n, buffer* w ) {
  return write_buffer( p, n, w );
}

/* eps_boolean methods */

int read_eps_boolean_buffer( eps_boolean *p, int n, buffer* w ) {
  return read_buffer( p, n, w );
}

int write_eps_boolean_buffer( eps_boolean *p, int n, buffer* w ) {
  return write_buffer( p, n, w );
}

int print_eps_boolean( const char *string, eps_boolean *p, FILE *fp, int ndim, const int *dims ) {
  int i;
  int k = 1;
  int x;
  char label[16];
  int l;

  
  switch( ndim ) {
  
    case 0:
      l = 1;
    break;
    
    case 1:
      l = dims[0];
    break;

    default:
      fprintf( fp, string );
      fprintf( fp, "\n" );
      for( i = 1; i < ndim; i++ ) {
        k *= dims[i];
      }
      for( i = 0; i < dims[0]; i++ ) {
        sprintf( label, "[%d]", i ); 
        print_eps_boolean( label, p + k * i, fp, ndim-1, dims + 1 );
      }
      return 0;
  }
  
  fprintf( fp, "%-50s = ", string );

  for( i = 0; i < l; i++ ) {
    x = p[i];
    fprintf( fp, "%d ", x ? 1 : 0 );
  }
  
  fprintf( fp, "\n" );
  
  return 0;
}

int read_ascii_eps_boolean( eps_boolean *pi, buffer *B, int len ) {
  *pi = B->c[0] ? 1 : 0;
  B->c++; B->lc--;
}

int write_ascii_eps_boolean( const char* label, eps_boolean *pi, buffer *B, int len ) {
  int k = sprintf( B->c, "%-30s= %1d\n", label, *pi ? 1 : 0 );
  B->c += k;
  B->lc -= k;
}




int print_string( const char *label, eps_byte *s, FILE *fp, int len, int ndim, const int *dims ) {
  int i;
  fprintf( fp, "%-50s = ", label );
  for( i = 0; i < len; i++ ) {
    fprintf( fp, "%d ", s[i] );
  }
  fprintf( fp, "\n" );
  return 0;
}

int read_ascii_string( char* s, buffer *B, int len ) {
  int i = 0;
  while( ( B->lc > 0 ) && ( i < len ) ) {
    if( B->c[0] == '\n' ) break;
    s[i] = B->c[0];
    i++; B->c++; B->lc--;
  }
  return i;
}

int write_ascii_string( const char* label, char* s, buffer *B, int len ) {
  char buf[256];
  int l;
  int i;
  l = sprintf( buf, "%-30s= ", label );
  for( i = 0; i < len; i++ ) {
    buf[l+i] = s[i];
  }
  l += len;
  if( l < B->lc ) {
    memcpy( B->c, buf, l );
    B->c += l;
    B->c[0] = '\n';
    B->c++;
    B->lc -= l + 1;
  }
}


/* eps_gentime */


int read_ascii_eps_gentime( eps_gentime *gt, buffer *B ) {
  read_ascii_eps_integer( &gt->YYYY, B, 4 );
  read_ascii_eps_integer( &gt->MM, B, 2 );
  read_ascii_eps_integer( &gt->DD, B, 2 );
  read_ascii_eps_integer( &gt->HH, B, 2 );
  read_ascii_eps_integer( &gt->mm, B, 2 );
  read_ascii_eps_integer( &gt->ss, B, 2 );
}

int read_ascii_eps_longgentime( eps_longgentime *lgt, buffer *B ) {
  read_ascii_eps_integer( &lgt->YYYY, B, 4 );
  read_ascii_eps_integer( &lgt->MM, B, 2 );
  read_ascii_eps_integer( &lgt->DD, B, 2 );
  read_ascii_eps_integer( &lgt->HH, B, 2 );
  read_ascii_eps_integer( &lgt->mm, B, 2 );
  read_ascii_eps_integer( &lgt->ss, B, 2 );
  read_ascii_eps_integer( &lgt->uuu, B, 3 );
}

int write_ascii_eps_gentime( const char* label, eps_gentime *gt, buffer *B ) {
  char buf[256];
  int len;
  len = sprintf( 
    buf, "%-30s= %4.4lld%2.2lld%2.2lld%2.2lld%2.2lld%2.2lldZ", 
    label, gt->YYYY, gt->MM, gt->DD, 
    gt->HH, gt->mm, gt->ss
  );
  if( len < B->lc ) {
    memcpy( B->c, buf, len );
    B->c += len;
    B->c[0] = '\n';
    B->c++;
    B->lc -= len + 1;
  }
}

int write_ascii_eps_longgentime( const char* label, eps_longgentime *lgt, buffer *B ) {
  char buf[256];
  int len;
  len = sprintf( 
    buf, "%-30s= %4.4lld%2.2lld%2.2lld%2.2lld%2.2lld%2.2lld%3.3lldZ", 
    label, lgt->YYYY, lgt->MM, lgt->DD, 
    lgt->HH, lgt->mm, lgt->ss, lgt->uuu
  );
  if( len < B->lc ) {
    memcpy( B->c, buf, len );
    B->c += len;
    B->c[0] = '\n';
    B->c++;
    B->lc -= len + 1;
  }
}



int init_eps_gentime( eps_gentime *gt ) {
  gt->YYYY = 0;
  gt->MM = 0;
  gt->DD = 0;
  gt->HH = 0;
  gt->mm = 0;
  gt->ss = 0;
}

int init_eps_longgentime( eps_longgentime *lgt ) {
  lgt->YYYY = 0;
  lgt->MM = 0;
  lgt->DD = 0;
  lgt->HH = 0;
  lgt->mm = 0;
  lgt->ss = 0;
  lgt->uuu = 0;
}

int print_eps_gentime( 
  const char* label, 
  eps_gentime *gt, 
  FILE *fp, int ndim, 
  int* dims 
) {
}

int print_eps_longgentime( 
  const char* label, 
  eps_longgentime *gt, 
  FILE *fp, int ndim, 
  int* dims 
) {
}







int skip_line( buffer *B ) {
  /* goto eol */
  if( B->c[0] != '\n' ) {
    while( B->lc > 0 ) {
      if( B->c[0] == '\n' ) break;
      B->c++; B->lc--;
    }
  }
  if( B->lc > 0 ) {
    B->c++; B->lc--;
  }
}

int read_label( const char** lb, int *len, buffer *B ) {
  int i = 0;
  *lb = (const char*)B->c;
  while( B->c[i] != ' ' ) {
    i++;
    if( i > B->lc ) return 0;
  }
  B->lc -= 32;
  B->c  += 32;
  *len = i;
    
  return 1;
}







