/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * VERSION   : 1.3
 *
 * HISTORY   : 01/02/2006 : Changes types to eps_types
 *             30/10/2006 : Check for input file
 *                          Add call for writing dummy
 *                          records
 *                          Add -ignore_degraded_inst_mdr
 *                          and -ignore_degraded_inst_mdr
 *                          options
 *             31/10/2006 : Correct bug with mphr0 and viadr0 
 *                          pointers
 *                          Activate check on seq_count to
 *                          detect missing MDRs
 *             25/01/2007 : Check change of APID for transition 
 *                          of 3a/3b on AVHRR.
 *             19/11/2014 : Ignore MDRs with a sequence number
 *                          earlier than the most recent one.
 *             23/03/2015 : Bug fix: previous update did not correctly allow 
 *                          for sequence wrap-around: change 0x2000 to 0x6000
 *             20/01/2015 : Update seq_count only if packet counter is valid
 */
#include "common-main.h"
#include "eps_metopl0.h"
#include "ccsds.h"

/*
 * common_main implements the main processing loop for generic
 * instruments.
 * We keep track of the MPHR, VIADR, and check the packet.seq_count
 * so that we replace a missing packet with a dummy.
 * Callback routines are called when appropriate :
 *   l1b_write_header_cb writes header to 1B file
 *   l1b_write_record_cb writes record to 1B file
 *   l1b_write_dummy_cb writes a dummy record to 1B file.
 *   l1b_close_cb closes 1B file.
 */
int common_main( 
  int argc, char* argv[], l1b_open l1b_open_cb,
  l1b_write_header l1b_write_header_cb,
  l1b_write_record l1b_write_record_cb,
  l1b_write_dummy l1b_write_dummy_cb,
  l1b_close l1b_close_cb
) {
  FILE *fpr;
  t_eps_metopl0 EM0;
  t_mphr* mphr0 = NULL;
  t_viadr_0* viadr0 = NULL;
  int seq_count = -1, APID = -1;
  packet_primary_header pph;
  int ignore_degraded_inst_mdr = 0, 
      ignore_degraded_proc_mdr = 0;
  int iarg = 1;
  int seq_diff = 0;
  int seq_max = -1;
  int seq_max_diff;

  
  while( 1 ) {
  
    if( iarg >= argc ) {
      printf( "Not enough arguments\n" );
      return 1;
    }
  
    if( ! strcmp( "-ignore_degraded_proc_mdr", argv[iarg] ) ) {
      ignore_degraded_proc_mdr++;
      iarg++;
    } else if( ! strcmp( "-ignore_degraded_inst_mdr", argv[iarg] ) ) {    
      ignore_degraded_inst_mdr++;
      iarg++;
    } else {
      break;
    }
  
  }
  
  if( ignore_degraded_proc_mdr ) 
    printf( "Warning: DEGRADED_PROC_MDR will not be taken into account!!\n" );

  if( ignore_degraded_inst_mdr ) 
    printf( "Warning: DEGRADED_INST_MDR will not be taken into account!!\n" );

  fpr = fopen( argv[iarg], "r" );
  
  if( fpr == NULL ) {
    printf( "Cannot open `%s' for reading\n", argv[iarg] );
    return 1;
  }

  eps_metopl0_check_subclass = 0;
  eps_metopl0_check_subclass_version = 0;

  l1b_open_cb( argv[iarg+1] );
  while( get_eps_metopl0( &EM0, fpr ) ) {
  
  
   if( EM0.grh.record_class == MDR ) {

      if( mphr0 == NULL ) {
        printf( "No MPHR was found in this product\n" );
        return 1;
      }
        
      if( viadr0 == NULL ) {
        printf( "No VIADR was found in this product\n" );
        return 1;
      }

      if( EM0.grh.instrument_group != DUMMY ) {
        if( ( ( EM0.content.MDR_0->DEGRADED_INST_MDR == 0 ) || ignore_degraded_inst_mdr ) 
         && ( ( EM0.content.MDR_0->DEGRADED_PROC_MDR == 0 ) || ignore_degraded_proc_mdr )
        ) {

          ccsds_read_packet_primary_header( &pph, EM0.content.MDR_0->INST_DATA );

          if( ( seq_count >= 0 ) && ( APID == pph.APID ) ) { /* The test with APID is for
                                                                AVHRR; when there is a channel 3
                                                                transition, the seq_count changes
                                                                too */

    /* Does the last packet counter exceed the current maximum? If so, reset the maximum */

            seq_max_diff = (seq_count - seq_max + 0x6000 ) % 0x4000 - 0x2000;         
            if ( seq_max_diff > 0 ) seq_max = seq_count;

    /* Does the current packet counter exceed the current maximum? If >1, add dummy records.
       If not, it's a duplicate so ignore */

            seq_diff = (pph.packet_seq_count - seq_max + 0x6000 ) % 0x4000 - 0x2000;

            if (seq_diff > 1) {
              int i;
              for(
                i = seq_count;
                i != ( pph.packet_seq_count - 1 + 0x4000 ) % 0x4000;
                i = ( i + 1 ) % 0x4000
              ) {
                l1b_write_dummy_cb( viadr0, mphr0 );
              }
              seq_diff = 1;
            }
          } else {
            seq_max = pph.packet_seq_count;     /* Reset if first time round or after channel 3 transition */
            seq_diff = 1;
          }

    /* Write the line and update counter only if seq_diff is valid */

          if (seq_diff == 1) {
            seq_count = pph.packet_seq_count;
            APID = pph.APID;
            l1b_write_record_cb( EM0.content.MDR_0->INST_DATA, viadr0, mphr0 );
          }

        } else {
           l1b_write_dummy_cb( viadr0, mphr0 );
        }
      } 

    } 

    switch( EM0.grh.record_class ) {
      case MPHR:
        mphr0 = EM0.content.MPHR;
      break;
      case VIADR:
        viadr0 = EM0.content.VIADR_0;
      break;
      default:
        free_eps_metopl0( &EM0 );
    }
  }
  l1b_write_header_cb();
  l1b_close_cb();

  return 0;
}
