#include <math.h>
#include "z_intex_51_to_2048.h"

static int xyz_to_ll(
  double x, double y, double z, double *plon, double *plat
) {
  double r;
  const double epsilon1 = 1e-15;
  const double epsilon2 = 1e-8;
  double xocl, yocl;
  double lon, lat;

  r = sqrt( x * x + y * y + z * z );
  x = x / r;
  y = y / r;
  z = z / r;

  if( z > ( + 1. - epsilon1 ) ) {
    lon = 0.;
    lat = +90.;
  } else if( z < ( -1. + epsilon1 ) ) {
    lon = 0.;
    lat = -90.;
  } else {
    lat = asin( z ); /* radians */
    xocl = x / cos( lat );
    if( fabs( xocl ) > ( +1 - epsilon2 ) ) {
      yocl = y / cos( lat );
      /* radians */
      lon = asin( yocl );
      lon = x > 0 ? lon : - lon + M_PI; 
    } else {
      /* radians */
      lon = acos( xocl );
      lon = y > 0 ? lon : - lon; 
    }
    lat = 180. * lat / M_PI; /* degrees */
    lon = 180. * lon / M_PI; /* degrees */
    while( lon > 180 ) {
      lon -= 360;
    }
  }

  plon[0] = lon;
  plat[0] = lat;

}

static int ll_to_xyz( 
  double lon, double lat, double *px, double *py, double *pz 
) {
  px[0] = cos( M_PI * lon / 180. ) * cos( M_PI * lat / 180. );
  py[0] = sin( M_PI * lon / 180. ) * cos( M_PI * lat / 180. );
  pz[0] = sin( M_PI * lat / 180. );
}



int z_intex_51_to_2048( /* all angles and lat/lon in degrees */
  double H, /* kms */
  double* azidi,
  double* zenSi,
  double* zensi,              
  double* loni, double* lati, 
  double* azido,
  double* zenSo,
  double* zenso,
  double* lono, double* lato 
) {
  double alpha[2048], gamma[2048];  /* radians */
  double x[2048], y[2048], z[2048]; 
  int i, j;
  double a1, a3;
  int j1, j2, j3;
  double c21, c31, c32;
  double s21, s31, s32;
  double c1, c2, c3;
  double s1, s2, s3;
  double r;
  double t;
  const double RT = 6366;


  
  t = RT / ( RT + H );   

  for( i = 0; i < 51; i++ ) {
    alpha[24+40*i] = asin( t * sin( ( M_PI / 180. ) * zensi[i] ) );
    gamma[24+40*i] = ( M_PI / 180. ) * zensi[i] - alpha[24+40*i];
    ll_to_xyz( loni[i], lati[i], x + 24+40*i, y + 24+40*i, z + 24+40*i );
    lono[24+40*i] = loni[i];
    lato[24+40*i] = lati[i];
    zenSo[24+40*i] = zenSi[i];
    azido[24+40*i] = azidi[i];
  }

  /* Extrapolate alpha */
  for( i = 0; i < 24; i++ ) {
    r = ( i - 24. ) / 40.;
    alpha[i] = ( 1. - r ) * alpha[24] + r * alpha[64];
  }
  /* Interpolate alpha */
  for( i = 0; i < 50; i++ ) {
    for( j = 1; j < 40; j++ ) {
      r = j / 40.;
      alpha[24+40*i+j] = ( 1. - r ) * alpha[24+40*i] + r * alpha[24+40*i+40];
    }
  }
  /* Extrapolate alpha */
  for( i = 2025; i < 2048; i++ ) {
    r = ( i - 1984. ) / 40.;
    alpha[i] = ( 1. - r ) * alpha[1984] + r * alpha[2024];
  }

  /* Compute gamma & zenso */
  for( i = 0; i < 2048; i++ ) {
    zenso[i] = asin( sin( alpha[i] ) / t );
    gamma[i] = zenso[i] - alpha[i];
  }
  

  /* Set exact values for zenso on known points */
  for( i = 0; i < 51; i++ ) {
    zenso[24+40*i] = zensi[i] * M_PI / 180.;
  }
  
  /* Extrapolate */
  {
    j1 = 24;
    j3 = j1 + 40;
    c31 = cos( gamma[j3] - gamma[j1] );
    for( j = 0; j < 24; j++ ) {
      j2 = j;
      c21 = cos( gamma[j2] - gamma[j1] );
      c32 = cos( gamma[j3] - gamma[j2] );
      a1 = c21 - c31 * c32;
      a3 = c32 - c31 * c21;
      x[j2] = a1 * x[j1] + a3 * x[j3];
      y[j2] = a1 * y[j1] + a3 * y[j3];
      z[j2] = a1 * z[j1] + a3 * z[j3];
   
      xyz_to_ll( x[j2], y[j2], z[j2], lono + j2, lato + j2 );
    }
  }

  /* Interpolate */
  for( i = 0; i < 50; i++ ) {
    j1 = 24+40*i;
    j3 = j1 + 40;
    c31 = cos( gamma[j3] - gamma[j1] );
    for( j = 1; j < 40; j++ ) {
      j2 = j1 + j;
      c21 = cos( gamma[j2] - gamma[j1] );
      c32 = cos( gamma[j3] - gamma[j2] );
      a1 = c21 - c31 * c32;
      a3 = c32 - c31 * c21;
      x[j2] = a1 * x[j1] + a3 * x[j3];
      y[j2] = a1 * y[j1] + a3 * y[j3];
      z[j2] = a1 * z[j1] + a3 * z[j3];
   
      xyz_to_ll( x[j2], y[j2], z[j2], lono + j2, lato + j2 );
    }
    
    
  }

  /* Extrapolate */
  {
    j1 = 2024;
    j3 = j1 - 40;
    c31 = cos( gamma[j3] - gamma[j1] );
    for( j = 2025; j < 2048; j++ ) {
      j2 = j;
      c21 = cos( gamma[j2] - gamma[j1] );
      c32 = cos( gamma[j3] - gamma[j2] );
      a1 = c21 - c31 * c32;
      a3 = c32 - c31 * c21;
      x[j2] = a1 * x[j1] + a3 * x[j3];
      y[j2] = a1 * y[j1] + a3 * y[j3];
      z[j2] = a1 * z[j1] + a3 * z[j3];
   
      xyz_to_ll( x[j2], y[j2], z[j2], lono + j2, lato + j2 );
    }
  }
  

  /* Extrapolate */
  for( i = 0; i < 24; i++ ) {
    r = ( gamma[i]  - gamma[24] ) 
      / ( gamma[64] - gamma[24] );
    zenSo[i] = ( 1. - r ) * zenSo[24] + r * zenSo[64];
  }
  /* Interpolate */
  for( i = 0; i < 50; i++ ) {
    for( j = 1; j < 40; j++ ) {
      r = ( gamma[24+40*i+j]  - gamma[24+40*i] ) 
        / ( gamma[24+40*i+40] - gamma[24+40*i] );
      zenSo[24+40*i+j] = ( 1. - r ) * zenSo[24+40*i] + r * zenSo[24+40*i+40];
    }
  }
  /* Extrapolate */
  for( i = 2025; i < 2048; i++ ) {
    r = (   gamma[i]  - gamma[1984] ) 
      / ( gamma[2024] - gamma[1984] );
    zenSo[i] = ( 1. - r ) * zenSo[1984] + r * zenSo[2024];
  }
  
  
  
  /* Extrapolate */
  for( i = 0; i < 24; i++ ) {
    r = ( gamma[i]  - gamma[24] ) 
      / ( gamma[64] - gamma[24] );
    azido[i] = ( 1. - r ) * azido[24] + r * azido[64];
  }
  /* Interpolate */
  for( i = 0; i < 50; i++ ) {
    for( j = 1; j < 40; j++ ) {
      r = ( gamma[24+40*i+j]  - gamma[24+40*i] ) 
        / ( gamma[24+40*i+40] - gamma[24+40*i] );
      azido[24+40*i+j] = ( 1. - r ) * azido[24+40*i] + r * azido[24+40*i+40];
    }
  }
  /* Extrapolate */
  for( i = 2025; i < 2048; i++ ) {
    r = (   gamma[i]  - gamma[1984] ) 
      / ( gamma[2024] - gamma[1984] );
    azido[i] = ( 1. - r ) * azido[1984] + r * azido[2024];
  }





  for( i = 0; i < 2048; i++ ) {
    zenso[i] = ( 180. / M_PI ) * zenso[i];
  }

}







