/*
 * COPYRIGHT : Meteo-France/EUMETSAT/SAFNWP
 *
 * DATE      : 2005/5
 * 
 * AUTHOR    : Philippe.Marguinaud@meteo.fr
 *
 * SYNOPSIS  : Performs OBT/UTC calculations
 *
 * VERSION   : 1.2
 *
 * HISTORY   : 26/01/2006 : Change 0x0100000000
 *                          constant to 0x0100000000LL
 *             21/02/2006 : Correct a_step calculation bug
 *             20/10/2010 : Trap for a_step=0, avoiding infinite loop
 *
 */
#include "eps_metopl0.h"
#include "ccsds.h"
#include "obtutc.h"
#include "common-aapp.h"
#include "eps_types.h"


static const eps_uinteger8 obt_max = 0x0100000000LL;

/*
 * obtutc_a_b_jj2000 applies the obt/utc correction
 * using ( a_step, b_offset ). psh_a is the approximate
 * utc time, sbt the satellite binary time. The
 * result is in jj_r ( julian day/2000 )
 */

int obtutc_a_b_jj2000( 
  packet_secondary_header *psh_a, 
  satellite_binary_time *sbt,
  double a_step, double b_offset,
  t_mphr *m0,
  eps_real8 *jj_r
) {
  eps_uinteger8 obt_r; /* in 1/256s ticks */
  double utc_a, utc_r; /* in seconds since 1/1/2000 */
  
  if (a_step == 0.0) {
    printf("Warning: obt-utc clock step step is zero\n");
    *jj_r = 0.0;
  } else {
  obt_r = sbt->fine_time / 256 + sbt->coarse_time * 256;
  utc_r = a_step * obt_r + b_offset;
  utc_a = psh_a->days * 24. * 3600. + psh_a->msec / 1000.;

  while( utc_r < ( utc_a - a_step * obt_max / 2 ) ) {
    utc_r = utc_r + a_step * obt_max;
  }
  while( utc_r > ( utc_a + a_step * obt_max / 2 ) ) {
    utc_r = utc_r - a_step * obt_max;
  }

  *jj_r = utc_r / ( 24 * 3600 ); /* convert to julian/2000 */
  }

  return 1;
}

/*
 * convert a psh_a to a julian day/2000 
 */

int obtutc_psh_jj2000( packet_secondary_header *psh_a, eps_real8 *jj_r ) {
  double utc_a = psh_a->days * 24. * 3600. + psh_a->msec / 1000.;
  *jj_r = utc_a / ( 24 * 3600 );
  return 1;
}

/*
 * obtutc_viadr0_jj2000 applies the obt/utc correction using
 * the viadr ( computes a_step/b_offset and pass it to 
 * obtutc_a_b_jj2000 )
 */

int obtutc_viadr0_jj2000(
  packet_secondary_header *psh_a,
  satellite_binary_time *sbt,
  t_viadr_0 *v0,
  t_mphr *m0,
  eps_real8 *jj_r
) {
  double a_step = ( (double)v0->CLOCK_STEP[0] * 0x10000 
	          + (double)v0->CLOCK_STEP[1] ) / 1e12; /* ~ 1/256s */
  double b_offset = (double)v0->UTC_OBT_TIME.days * 24. * 3600.
	          + (double)v0->UTC_OBT_TIME.msec / 1000.
		  + (double)v0->UTC_OBT_TIME.usec / 1000000.
		  - a_step * (double)v0->OBT_UTC_TIME.rtime; 
  /* b_offset in seconds since 1/1/2000 */

  return obtutc_a_b_jj2000( psh_a, sbt, a_step, b_offset, m0, jj_r );
}

/*
 * obtutc_jj2000_to_jj1950 converts a jj2000 to a jj1950 ( used in AAPP )
 */

int obtutc_jj2000_to_jj1950( 
  eps_real8 *jj2000, eps_real8 *jj1950
) {
  eps_real8 ij0;
  eps_integer4 ig0[7] = { 2000, 1, 1, 0, 0, 0, 0 };
  aapp_xigj( ig0, &ij0 );
  *jj1950 = *jj2000 + ij0;
}


