package IASI::NAVIGATION;

=head1 NAME

IASI::NAVIGATION

=head1 SYNOPSIS

  use IASI::NAVIGATION;

  my $satp = satp_iasi_navigation( s => 'metop02', d => 20070129 ); 
  make_iasi_navigation( s => 'metop02', d => 20070129 );

=head1 DESCRIPTION

C<IASI::NAVIGATION> is a module for managing specific IASI navigation data.
It implements routines to create satpos files, ingest SPM bulletins.
Assumes that the environment variable C<DIR_NAVIGATION> is properly set.
With version OPS-LRS V6-0+p12 2line elements are default, previoulsy this was SPOT elements


=head1 ROUTINES

=over 4

=cut


use strict;
use File::Path;
use FileHandle;
use File::Basename;
use Fcntl qw( LOCK_EX LOCK_UN O_RDWR );
use base qw( Exporter );
our @EXPORT = qw( make_iasi_navigation satp_iasi_navigation add_bulletin_iasi_navigation );

use liblog;

use METOP::MISC;

sub lock_iasi_navigation {

=item lock_iasi_navigation

  &lock_iasi_navigation();

Internal. Lock the navigation directory so that other process do not 
interfere.

=cut


  'FileHandle'->new( ">$ENV{DIR_NAVIGATION}/.lock" );
  my $lock = 'FileHandle'->new( "$ENV{DIR_NAVIGATION}/.lock", O_RDWR );
  chmod( 0666, "$ENV{DIR_NAVIGATION}/.lock" );
  flock( $lock, LOCK_EX );
  return $lock;
}

sub unlock_iasi_navigation {

=item unlock_iasi_navigation

  &unlock_iasi_navigation();

Unlock navigation directory.

=cut

  my $lock  = shift;
  flock( $lock, LOCK_UN );
}


sub satp_iasi_navigation {

=item satp_iasi_navigation

  my $satp = &satp_iasi_navigation( s => 'metop02', d => 20070129 );

Returns the full path of the satpos file to be used for processing.

=cut

  my ( $opts ) = @_;
  return sprintf( "$ENV{DIR_NAVIGATION}/satpos/satpos_%s_%8.8d.txt", @{$opts}{ qw( s d ) } );
}


sub make_iasi_navigation {

=item make_iasi_navigation

  &make_iasi_navigation( s => 'metop02', d => 20070129 );

Creates satpos file for the specified hour and satellite, using
satposspm or satpostle.
satpostle is the new default

=cut

  my ( $opts ) = @_;
  
  my $satp = satp_iasi_navigation( $opts );

  my $lock = &lock_iasi_navigation();
  

  unless( -f $satp ) {
    my ( $YY, $MM, $DD ) = ( $opts->{d} =~ m/^\d\d(\d\d)(\d\d)(\d\d)$/o );
       execute( 
         'satpostle', '-s', $opts->{s}, '-d', "$DD/$MM/$YY", '-n', '1.5', '-o',  $satp )
    or execute( 
         'satposspm', '-s', $opts->{s}, '-d', "$DD/$MM/$YY", '-n', '1.5', '-o',  $satp )
    or do {
      log_error( "Cannot create $satp" );
      flock( $lock, LOCK_UN );
      goto ERROR;
    };

  }

  unlock_iasi_navigation( $lock );
  return 1;

ERROR:

  return;

}

sub add_bulletin_iasi_navigation {

=item add_bulletin_iasi_navigation

  &add_bulletin_iasi_navigation( s => 'metop02', 'HKTM_...' );

Ingests a HTKM file, and extracts SPM bulletins.

=cut 



  my ( $opts, $spm ) = @_;
  my $lock = &lock_iasi_navigation();
  my $c = execute( qw( spming -s ), $opts->{s}, $spm );
  unlock_iasi_navigation( $lock );
  return $c;
}


1;

=back

=head1 AUTHORS

Philippe.Marguinaud@meteo.fr

=head1 COPYRIGHT

Meteo-France/EUMETSAT/SAFNWP

=head1 HISTORY

=over 4

=item 01/03/2007

First version for HRPT processing.

=back

=cut

