package IASI::OPS;


=head1 NAME

IASI::OPS

=head1 SYNOPSIS

  use IASI::OPS;

  my $OPS = 'IASI::OPS'->new( s => 'metop02', 
    d => 200070215, h => 1243, P => 'DUMP' );

  $OPS->start();

  $OPS->process(
    'IASI_P71_00_M02_20070305192427Z_20070305192737Z_N_O_20070305192759Z',
    'AVHR_P70_00_M02_20070305224355Z_20070305224455Z_N_O_20070305224512Z',
    'AVHR_P70_00_M02_20070305224455Z_20070305224555Z_N_O_20070305224608Z'
  );

  # save produced files.

  $OPS->stop();


=head1 DESCRIPTION

This package implements the class C<IASI::OPS>, which interacts with the OPS
software. Methods below fall in two distinct categories: those which 
monitors the OPS and directly send it commands, and those which works at
a higher level data management, preparation of work-orders.
This class implements both DUMP and GRANULE processing.

=head2 DUMP PROCESSING

The IASI level 0 data is contained within a single file, and AVHRR 1B data may
be present in several non-overlapping contiguous files; all data is encoded
in PFS format:

   |==================== IASI 00 ==================|
  |=== AVHRR 1B ===|==== AVHRR 1B ==|== AVHRR 1B ===|

Ideally, AVHRR data should begin before and end after IASI; if it does not,
then the navigation of produced IASI level 1C data will be affected for 
data not covered by AVHRR.

=head2 GRANULE PROCESSING

IASI level 0 data shall come in in chunks of 3 minutes, with 10s overlap.
AVHRR data shall be provided in contiguous 1 minute granules:

  |===== IASI 00 ====|              |===== IASI 00 ====|
                   |===== IASI 00 ====|              |===== IASI 00 ====|
   |=== AVHRR ===|=== AVHRR ===|=== AVHRR ===|=== AVHRR ===|=== AVHRR ===|

Data shall be provided in chronological order, with respect to the end of each
granule.

=head1 HISTORY

=over 4

=item 06/10/2008 Change SVM creation routine for OPS_V4.0+p123.
=item 30/01/2010 compatibility perl

=babk

=head1 METHODS

=over 4

=cut



use strict;
use Sys::Hostname;
use IO::Socket::INET;
use FileHandle;
use File::Basename;
use File::Path;
use File::Copy;
use Data::Dumper;
use liblog;

use Cwd qw( cwd );
use Storable qw( nfreeze thaw );

use METOP::MISC;
use METOP::PFS;

use IASI::NAVIGATION;
use IASI::OPS::REPORT;
use IASI::OPS::WORKORDER;

my ( $SECONDS_AVHRR_MARGIN ) = ( 20 );

# interaction with OPS software

sub new {

=item new 

Constructor. Environment variables C<DIR_NAVIGATION> and 
C<DIR_IASICONFIG> shall be set.

=cut

  my $class = shift;
  my %args = @_;

  for my $dir (qw( DIR_NAVIGATION DIR_IASICONFIG )) {
    unless( exists( $ENV{$dir} ) && ( -d $ENV{$dir} ) ) {
      log_error( "Environment variable $dir is not a valid directory" );
      die( "\n" );
    }
  }

  
  # change to directory = sat

  ( -d $args{s} ) or mkpath( $args{s} );
  chdir( $args{s} );


  my ( $n ) = ( &pfs_satellite_name( $args{s} ) =~ m/0*(\d+)/o );

  my ( $ADR_MSG_SERVER_PORT_BASE, $APP_OPSIASI_PORT_BASE ) = 
  ( $ENV{ADR_MSG_SERVER_PORT_BASE} || 4000, $ENV{APP_OPSIASI_PORT_BASE} || 6000 );

  @ENV{ qw( REP_WORKING_ROOT ADR_MSG_SERVER_PORT APP_OPSIASI_PORT ) }
  = ( &cwd(), $ENV{ADR_MSG_SERVER_PORT} = $ADR_MSG_SERVER_PORT_BASE + $n, 
      $ENV{APP_OPSIASI_PORT} = $APP_OPSIASI_PORT_BASE + $n );

  my $self = -f 'processiasi.pl' 
           ? do 'processiasi.pl' 
           : bless { pfsIASI1C => [ ], %args }, $class;
  
  return $self;
}

sub done {

=item done

Returns true if the orbit has already been processed.

=cut

  my $self = shift;


  return -d ".keep/$self->{n}";
}


sub running {

=item running

Returns true if the OPS is currently running.

=cut

  return -f 'processiasi.pl';
}


my $fh_mp__mainprocess_eo;

sub wait4ack { # wait for a particular pattern to appear in the output file of MP__MainProcess

=item wait4ack

Wait for a pattern to appear in the F<MP__MainProcess.eo>.

  $self->wait4ack(
    regex => qr/ACK START/o,
    timeout => 60,  # optional
  );

=cut


  my ( $self, %args ) = @_;

  my $regex = $args{regex};
  my $timeout = $args{timeout} || 0;
  
  $fh_mp__mainprocess_eo ||= 'FileHandle'->new( "<OPS/bin/MP__MainProcess.eo" );

  $fh_mp__mainprocess_eo or return '';
  
  # Add 2 log lines 
  # P. Brunel 15 July 2013
  log_info( "WAIT4ACK: $regex   TIMEOUT: $timeout" );

  my $i = 0;
  MAIN: while( 1 ) {
    while( my $line = <$fh_mp__mainprocess_eo> ) {
      chomp ( $line );

      if( $line =~ $regex ) {
        log_info( "WAIT4ACK TROUVE: $regex" );
        return $line;
      }
    }
    sleep( 2. ); 
    ( $i++ % 5 ) or $self->up() or return; # check every 10s
    ( ! $timeout ) or ( $i * 2 < $timeout ) or return; # timeout has expired
  }
  
}


sub mainloop {

=item mainloop

Server-side mainloop, wait for incoming requests and process them, by
issuing commands in the OPS pipe. Inactivity timeout 1800s.

=cut

  my $self = shift;
  
  my ( $pipe, $ss ) = @{ $self }{ qw( pipe socket ) };

  my $i = 1;
  
  while( 1 ) {

    # the server will shut down by itself after 1800s of inactivity

    { 
      use IO::Select;
    
      my $is = 'IO::Select'->new( $ss );
    
      my ( $r, $w, $e ) = 'IO::Select'->select( $is, undef, $is, 1800 );

      if( $e && @$e ) {
        $self->abort( "Server socket error : $!\n" );
      }
      
      unless( $r && @$r ) { # timeout
        $self->abort( "IASI OPS TIMEOUT" );
      }
    
    }
    my $s = $ss->accept();
    
    

    my $length = <$s>;

    my $message = '';
    while( length( $message ) < $length ) {

      $s->read( my $buf, $length - length( $message ) );
      $message .= $buf;
    }
    my $c = thaw( $message );

    $pipe->print( "$c->{method} $i @{$c->{args}}\n" );
    
    log_info( "$c->{method} @{$c->{args}}" );

    # command received

    my $regex0 = qr/ACK $c->{method} $i 0/; 
    
    my $line0 = $self->wait4ack( regex => $regex0, timeout => 60 );

    $line0 or last; # something went wrong

    # command processed

    my $regex1 = qr/ACK $c->{method} $i ([1\d])/;

    my $timeout1 = 1800;

    if( $c->{method} eq 'STEP' ) {
      if( exists $ENV{OPS_TIMEOUT} ) {
        my $wo = 'IASI::OPS::WORKORDER'->new( file => $c->{args}[0] );
	$wo->read();
	my ( $SensingStart, $SensingStop ) = @{ $wo }{ qw( SensingStart SensingStop ) };
	my $dt = ( to_time_t( $SensingStop ) - to_time_t( $SensingStart ) );
        $timeout1 = $dt / 60 * $ENV{OPS_TIMEOUT} + 60;   
	log_info( "TIMEOUT = $timeout1 s ( DATA = $dt s )" );
      }
    }

    my $line1 = $self->wait4ack( regex => $regex1, timeout => $timeout1 );

    $line1 or last; # something went wrong

    $line1 =~ $regex1;

    # error status ( must be 1 )

    my $e = $1 && ( $1 == 1 ) ? 1 : 0;

    if( lc( $c->{method} ) eq 'stop' ) { 
      $self->wait4ack( regex => qr/ACK START/, timeout => 60 );
    }

    $i++;
    $s->print( "$e\n" );
    $s->close();

    ( lc( $c->{method} ) eq 'stop' ) && last;
    
    
  }


  my @pid = $self->ops_pids();
  
  for( grep { $_ ne $$ } @pid ) {
    kill( 9, $_ );
  }

  unlink( 'processiasi.pl' );
  exit( 0 );

}


sub install { # locate OPS package and make a copy

=item install

Locates the OPS in the C<PATH> environment variable and make a local copy.

=cut


  my $self = shift;

  my @PATH = split( m/:/, $ENV{PATH} );
  my ( $OPS ) = grep { -f "$_/MP__MainProcess" } @PATH;
  
  0 && do {
    local $" = "\n";
    print "@PATH\n";
  };
  
  $OPS or die "Cannot find OPS package\n";
  
  $OPS = dirname( $OPS );
  
  system( "cp -r $OPS/. OPS" )
  && die "Cannot find OPS package\n";
  
  # prepare working directories
  
  mkpath( $_ ) for( 
    qw( to_pgf input/avhr input/obtu input/svmu 
    input/auxd input/iasi/.done tmp/data ) 
  );

}

sub start {

=item start

Starts the software: make a local copy ( C<install> ) and
prepare navigation satpos file. Set up of environment variables,
load configuration file F<OPS.env>.
The server script is started here.
The object ( C<$self> ) is saved to F<processiasi.pl>.

=cut

  my $self = shift;

  ( -f 'processiasi.pl' ) 
    && die "OPS already running\n"; 

  &make_iasi_navigation( $self ) 
    or die "Cannot setup navigation data for IASI\n";

  $ENV{SATPOS_PATH} = &satp_iasi_navigation( $self );

  # 
  
  log_info( "COPY IASI OPS SOFTWARE" );

  $self->install();
  
  # start the OPS software
  
  log_info( "IASI OPS WARMING UP..." );

  $ENV{SPACECRAFT_ID} = &pfs_satellite_name( $self->{s} );


  system( 'ksh', '-c', " cd OPS/bin ; "
          . ". ./OPS.env ; "
          . "export ADR_MSG_SERVER_PORT=$ENV{ADR_MSG_SERVER_PORT} ; "
          . "./MP__MainProcess.sh " );
  sleep( 1 );
  my $err = $self->wait4ack( regex => qr/ACK START/, timeout => 60 );

  $err or die "Cannot start the OPS\n";

  my $ss = 'IO::Socket::INET'->new(
    LocalAddr  => &hostname(),
    LocalPort  => $ENV{APP_OPSIASI_PORT},
    Proto      => 'tcp',
    Type       => SOCK_STREAM,
    Listen     => 10,
    Reuse      => 1,
  ) or die "Cannot create a server socket : $!\n";

  log_info( "OPS IASI START" );


  my $pid;
  ( $pid = fork() ) or do { # fork off server; initialize new process
  
    use POSIX qw( setsid );

    setsid();

    {
      my %pids = map { 
                  local $/ = undef; 
		  ( my $f = $_ ) =~ s,(?:OPS/bin/|\.pid$),,go;
	          my $fh = 'FileHandle'->new( "<$_" ); 
	          ( $f, <$fh> )
	       } ( <OPS/bin/*.pid> );
	       
      $pids{OPS} = $$;
      chomp for( values( %pids ) );
      
      $self->{pids} = \%pids;

      local $Data::Dumper::Terse = 1;
      'FileHandle'->new( '>processiasi.pl' )->print( Dumper( $self ) );
    }

    

    # set pipe & server socket
    
    my $pipe = 'FileHandle'->new( ">tmp/cmd_fromPGF" );
    $pipe->autoflush( 1 );

    @{ $self }{ qw( pipe socket ) } = ( $pipe, $ss );
    
    # setup SIGTERM
    
    $SIG{TERM} = sub { delete $SIG{TERM}; $self->abort( "CAUGHT SIGTERM" ); };
    
    # 
    
    $self->mainloop();

  };

  $ss->close();

  $self->{pids}{OPS} = $pid;

  while( 1 ) { 
    return 1 if( -f 'processiasi.pl' );
    sleep( 5 );
  }

}

sub stop {

=item stop

Stops the server.

=cut

  my $self = shift;

  $self->send_command( 'stop' );

  log_info( "OPS IASI STOP" );

  # should not be necessary, but we never know

  my @pid = $self->ops_pids();
  
  for( @pid ) {
    kill( 9, $_ );
  }

  $self->cleanup();

}

sub cleanup {

=item cleanup

Clean-up of processes, semaphores and data used by OPS.

=cut

  my $self = shift;
  #
  
  {
    # Remove semaphores before they fill up
    use IPC::SysV qw( IPC_RMID );
    use FileHandle;
    my $fn = 'OPS/bin/semaphores.txt';
    if( my $fh = 'FileHandle'->new( "<$fn" ) ) {
      while( my $semid = <$fh> ) {
        chomp( $semid ); 
        semctl( hex( $semid ), 0, IPC_RMID, my $c = 0 );
      }
    }
    unlink( $fn );
  }

  my $d = ".keep/$self->{n}";
  rmtree( $d );
  mkpath( $d );
# print( STDERR "$_ -> $d/$_\n" ) for( qw( OPS input tmp to_pgf ) );
  rename( $_, "$d/$_" ) for( qw( OPS input tmp to_pgf ) );
  unlink( 'processiasi.pl' );
  # rmtree( $_ ) for( qw( OPS input tmp to_pgf ) );

}


sub ops_pids {

=item ops_pids

Returns PIDs of OPS process while running.

=cut

  my $self = shift;
  return values( %{ $self->{pids} } );
}

sub up {

=item up

Checks that the OPS is not dead.

=cut

  my $self = shift;

  # check if the OPS is running
  
  my @pid = $self->ops_pids();
  
  my $ok = 1;
  
  $ok &&= kill( 0, $_ ) for( @pid );

  # Print expected running PIDs and the ones which are down
  # P. Brunel 15 July 2013
  if ( ! $ok ) {
    print (Dumper ($self->{pids}));
    for( @pid ) {
      my $ppid=$_;
      my $pup=kill( 0, $ppid );  

      if ( $pup == 0 ) {
        log_info( "PID DOWN  $ppid $pup");
      }
    }
  }

  return $ok;
}


sub abort : method {

=item abort

Stops the OPS in case of emergency.

=cut

  my ( $self, $mess ) = @_;
  
  $mess ||= '';
  
  chomp( $mess );
  
  my @pid = $self->ops_pids();
  
#  print Dumper( $self, \@pid );
  
  kill( 15, $_ ) 
    for( grep { $_ ne $$ } @pid ); # should not suicide

  sleep( 2 );

  kill( 9, $_ ) 
    for( grep { $_ ne $$ } @pid ); # should not suicide
  
  log_error( "OPS IASI ABORT $mess" );

  print STDERR "OPS IASI ABORT : $mess !!\n" x 10;
  
  unlink( 'processiasi.pl' );

  $self->cleanup();
  
  if( $ENV{OPS_REPORT_ERROR_COMMAND} ) {
    local $Data::Dumper::Terse = 1;
    my $m = { 
      ops => $self,
      message => $mess,
    };
    my $fhp = 'FileHandle'->new( "| $ENV{OPS_REPORT_ERROR_COMMAND}" );
    $fhp->print( Dumper( $m ) );
    $fhp->close();
  }
  
  exit( 1 );
}


sub send_command {

=item send_command

Sends a command from the client-side to the server.
These are the commands the OPS may accept:
START, STEP, STOP, SUSPEND, ...

=cut

  my ( $self, $method, @args ) = @_;

  return unless( $self->up() );
  
  # connect to opsiasi daemon & send request

  my $cs = 'IO::Socket::INET'->new(
    PeerAddr => &hostname(),
    PeerPort => $ENV{APP_OPSIASI_PORT},
    Type     => SOCK_STREAM,
    Proto    => "tcp",
  ) or die "Cannot open client socket : $!\n";
  $cs->autoflush( 1 );

  my $message = nfreeze( { method => uc( $method ), args => \@args } );

  $cs->print( length( $message ) . "\n" );
  $cs->print( $message );
  $cs->print( "\n" );
  my $ans = <$cs> || '';
  
  chomp( $ans );
  
  return $ans;
}

sub AUTOLOAD {

=item AUTOLOAD

Proxies to C<send_command>.

=cut

  my $self = shift;
  ( my $method = our $AUTOLOAD ) =~ s/^.*:://o;
  die "Unknown method $method called on $self\n"
    unless( $method =~ m/^(?:step|suspend|resume)$/o );

  return $self->send_command( $method, @_ );
}

sub DESTROY {

=item DESTROY

=cut

  my $self = shift;
}







# processing subroutines go here


sub get_avhrr_granules { # select AVHRR granules matching a IASI granule

=item get_avhrr_granules

Returns the list of PFS handles for the AVHRR granules matching a IASI
granule.

=cut

  my ( $self, $i ) = @_;
  
  my ( $si, $ei ) = ( $i->et_sensing_start(), $i->et_sensing_end() );
  my @a = 
    grep { 
      my ( $sa, $ea ) = ( $_->et_sensing_start(), $_->et_sensing_end() );
         ( ( ( $si - $SECONDS_AVHRR_MARGIN ) < $sa ) && ( $sa < $ei + $SECONDS_AVHRR_MARGIN ) )
      || ( ( ( $si - $SECONDS_AVHRR_MARGIN ) < $ea ) && ( $ea < $ei + $SECONDS_AVHRR_MARGIN ) )
    }
    sort { $a->et_sensing_start() <=> $b->et_sensing_start() }
    map { 'METOP::PFS'->new( file => $_ ) } 
    ( <input/avhr/*> );

  return @a;
}




sub get_new_workorder { # creates new workder based on IASI granule 

=item get_new_workorder

Creates a new workorder for a IASI granule.

=cut

  my ( $self, %args ) = @_; # iasi => iasi l0
                            # last => 1 if last granule

  my $i = $args{iasi};

  my ( $SensingStart, $SensingStop ) = 
    ( @{ $i->{header} }{ qw( SENSING_START SENSING_END ) } );
    
  my @A = $self->get_avhrr_granules( $i );

  my @W = sort map { s,input/,,o; $_ } <input/IASI_9_wo_???>;

  my @X = ( 
            <input/auxd/IASI_BRD_*>, 
            <input/auxd/IASI_GRD_*>, 
            <input/auxd/IASI_ODB_*>, 
            <input/auxd/IASI_CTX_*>, 
            <input/obtu/xxxx_OBT_*>,
            <input/svmu/xxxx_SVM_*>,
	  );
	  
  my @M = ( 
            <input/auxd/IASI_ENG_01_*>, 
            <input/auxd/IASI_VER_01_*>, 
            <input/auxd/IASI_xxx_1A_*>, 
            <input/auxd/IASI_xxx_1B_*>, 
            <input/auxd/IASI_xxx_1C_*>, 
	  );



  my ( $c ) = @W ? ( $W[-1] =~ m/^IASI_9_wo_0*(\d+)$/o ) : 0;
  $c++;


  my $TimeIntervalFlag;
  if( $self->{P} eq 'DUMP' ) {
    $TimeIntervalFlag = 'Full';
  } else {
    $TimeIntervalFlag = $c == 1 ? 'First' : $args{last} ? 'Last' : 'Middle';
  }


  if( $c == 1 ) {
    $SensingStart = to_YYYYMMDDHHmmssZ( to_time_t( $SensingStart ) + 8 );
  } else {
    # re-read last work-order to see where we stopped
    my $wo ='IASI::OPS::WORKORDER'->new( file => "input/$W[-1]" );
    $wo->read();
    $SensingStart = $wo->{SensingStop};
  }
  
  my $wo = sprintf( 'IASI_9_wo_%3.3d', $c );
  my $xwo = 'IASI::OPS::WORKORDER'->new( 
    file	   => "input/$wo", 
    UnProcData     => "input/iasi/$i->{header}{PRODUCT_NAME}",
    ProcessingType => "L1a", 
    SensingStart   => $SensingStart, 
    SensingStop    => $SensingStop, 
    AuxData	   => [ 
      @X, map( { "input/avhr/$_->{header}{PRODUCT_NAME}" } @A )
    ], 
    TimeIntervalCounter => 1, 
    TimeIntervalFlag	=> $TimeIntervalFlag, 
    ProdModel	   => [ @M ]
  );

  $xwo->write();
  
  return $xwo;
}

sub get_unprocessed_granules {

=item get_unprocessed_granules

Returns the list of PFS handles of unprocessed IASI granules.

=cut

  my $self = shift;
  return sort { $a->et_sensing_start() <=> $b->et_sensing_start() }
         map { 'METOP::PFS'->new( file => $_ ) } 
         ( <input/iasi/*> );
}

sub get_last_granule {

=item get_last_granule

Returns the last unprocessed granule.

=cut

  my $self = shift;
  my @i = $self->get_unprocessed_granules();
  return pop( @i );
}


sub process_last_granule { # process last IASI granule inconditionally

=item process_last_granule

Process last IASI granule inconditionally. 

=cut

  my ( $self, %args ) = @_; # last => 1 : last granule ( Last in Work-Order )
  

  return unless( my $i = $self->get_last_granule() );

  return $self->process_granule( iasi => $i, %args );
}
  

sub process_granule {

=item process_granule

Process the granule passed as argument inconditionally ( except too short granules ).

=cut

  my ( $self, %args ) = @_;

  my $i = $args{iasi};


  if( ( $i->et_sensing_end() - $i->et_sensing_start() ) < 20 ) {
    log_info( "SKIP $i->{header}{PRODUCT_NAME} < 20s" );
    return;
  }
  
  log_info( "PROCESS $i->{header}{PRODUCT_NAME}" );

  my $wo = $self->get_new_workorder( iasi => $i, last => $args{last} );
  
  my $base = basename( $wo->{file} );
  
  my $sta = $self->step( "input/$base" );

  #log_info( "SLEEP 30sec" );
  #sleep( 30. ); # ajoute par P. Brunel le 20 juin 2013 pour tester sur MetOp-B (M01)

  my $rpt = 'IASI::OPS::REPORT'->new( file => "to_pgf/$base\_001.rpt" );
  
  $rpt->read() or die "Cannot open $base\_001.rpt\n";
  
  $sta or die "STEP COMMAND FAILED ( $rpt->{StageInfo}{LogMessage} )\n";

  my @log = $rpt->LogMessage();


  {
    local $" = "; ";
    log_info( "OPS IASI : @log" );
  }

  if( my @iasil1c = $rpt->GeneratedFile_1C() ) {
  

    push @{ $self->{pfsIASI1C} }, map { "to_pgf/$_" } @iasil1c; 

    
    rename( $wo->{UnProcData}, 'input/iasi/.done/' 
  	  . basename( $wo->{UnProcData} ) ); 
    
    log_info( "OPS IASI : @iasil1c CREATED" );
    
  } else {
  
    die "NO L1C FILE\n";
    
  }
    

  return 1;
}

sub store_pfsAVHR1B {

=item store_pfsAVHR1B

Store AVHRR 1B for later use.

=cut


  my ( $self, $file ) = @_;
  my $a = 'METOP::PFS'->new( file => $file );
  my %a = %{ $a->{header} };

  log_info( "STORE $a{PRODUCT_NAME}" );

  link( $file, "input/avhr/$a{PRODUCT_NAME}" ) or
  copy( $file, "input/avhr/$a{PRODUCT_NAME}" ) or
  die( "Could not copy $file to input/avhr/$a{PRODUCT_NAME}" );

}


sub process_pfsAVHR1B {

=item process_pfsAVHR1B

On reception of AVHRR granule, see if we can process a IASI granule.

=cut

  my ( $self, $file ) = @_;
  
  $self->store_pfsAVHR1B( $file );
  
  # we have an AVHRR granule; see if we can 
  # process last received IASI granule
  

  return unless( my $i = $self->get_last_granule() );

  my $a = 'METOP::PFS'->new( file => $file );

  return unless( $a->et_sensing_end() > ( $i->et_sensing_end() + $SECONDS_AVHRR_MARGIN ) );


  $self->process_last_granule();
  
  
}

sub create_obt { # create a OBT/UTC file from l0

=item create_obt

Creates the OBT/UTC file from the level 0 VIADR.

=cut

  my ( $self, $file ) = @_;
  my $YYYYMMDDHHmmssZ = to_YYYYMMDDHHmmssZ( time() );

  my $pfssn = &pfs_satellite_name( $self->{s} );

  my $OBT = "xxxx_OBT_xx_$pfssn\_xxxxxxxxxxxxxxZ_xxxxxxxxxxxxxxZ_$YYYYMMDDHHmmssZ\_xxxx_METOOBTUTC";

  log_info( "CREATE $OBT" );
  execute( 'eps_metopl0-obt-xml', $file, "input/obtu/$OBT" ) 
  or die "Cannot create OBT/UTC\n";
}

sub create_svm { # create a SVM file from satpos.txt; we only fill UMBRA information

=item create_svm

Creates the SVM file from the F<satpos.txt>. We massage the file to keep only UMBRA_END fields.

=cut

  my ( $self, $file ) = @_;
  my $YYYYMMDDHHmmssZ = to_YYYYMMDDHHmmssZ( time() );

  my $pfssn = &pfs_satellite_name( $self->{s} );

  my $SVM = "xxxx_SVM_xx_${pfssn}_20010101000000Z_20500101000000Z_$YYYYMMDDHHmmssZ\_FDFx_FDGEOEVENT";

  log_info( "CREATE $SVM" );
  execute( 'satpos-svm', $file, "input/svmu/$SVM.x" ) 
  or die( "Cannot create SVM\n" );
  
  my $fhi = 'FileHandle'->new( "<input/svmu/$SVM.x" );
  my $fho = 'FileHandle'->new( ">input/svmu/$SVM" );

# add filter on NIGHT_STR and NIGHT_END events
# add a space for UMBRA_END filter
# P. Brunel 15 July 2013

  while( my $line = <$fhi> ) {
    $fho->print( $line ) if( $line =~ m/ UMBRA_END/o );
    $fho->print( $line ) if( $line =~ m/ NIGHT_/o );
  }
  $fhi->close();
  $fho->close();

  unlink( "input/svmu/$SVM.x" );
}

sub get_configuration {

=item get_configuration

Select configuration data files from the C<$DIR_IASICONFIG> directory.

=cut

  my $self = shift;

  my $dir = $ENV{DIR_IASICONFIG};

  my $YYYYMMDDHHmmss = "$self->{d}$self->{h}00";
  
  my @prefix = qw( 
    IASI_BRD_xx IASI_CTX_xx IASI_GRD_xx IASI_ODB_xx 
    IASI_ENG_01 IASI_VER_01 IASI_xxx_1A IASI_xxx_1B IASI_xxx_1C 
  );
  
  my @C;
 
  my $pfssn = &pfs_satellite_name( $self->{s} );
 
  for my $p ( @prefix ) {
    my @c;
    for my $f ( sort map { basename( $_ ) } <$dir/$p*$pfssn*> ) {
      #print "file ",$f,"\n";
      my ( $YYYYMMDDHHmmss1, $YYYYMMDDHHmmss2 ) = ( $f =~ m/^\w{4}_\w{3}_\w{2}_${pfssn}_(\w{14})\w_(\w{14})\w/ );

      if( ( ( $YYYYMMDDHHmmss1 eq 'xxxxxxxxxxxxxx' ) || ( $YYYYMMDDHHmmss1 le $YYYYMMDDHHmmss ) )
       && ( ( $YYYYMMDDHHmmss2 eq 'xxxxxxxxxxxxxx' ) || ( $YYYYMMDDHHmmss le $YYYYMMDDHHmmss2 ) )
      ) {
        push @c, $f;
      }
    }
    @c or die "Cannot find aux data of type $p\n";
    push @C, @c;
  }
  
  for( @C ) {
    log_info( "SELECT CONFIG $_" );
    symlink( "$dir/$_", "input/auxd/$_" );
  }
  
  
}

sub store_pfsIASI00 {

=item store_pfsIASI00

Store IASI level 0 for later use.

=cut

  my ( $self, $file ) = @_;
  my $p = 'METOP::PFS'->new( file => $file );
  my %h = %{ $p->{header} };

  log_info( "STORE $h{PRODUCT_NAME}" );

  link( $file, "input/iasi/$h{PRODUCT_NAME}" ) or
  copy( $file, "input/iasi/$h{PRODUCT_NAME}" ) or
  die( "Could not copy $file to input/iasi/$h{PRODUCT_NAME}" );

}


sub process_pfsIASI00 {

=item process_pfsIASI00

Set-up of configuration and OBT/UTC and SVM file.
See if some IASI data is pending and process it now.

=cut

  my ( $self, $file ) = @_;

  # if this is the first IASI granule, then we create OBT/UTC file, SVM
  # and initialize configuration
  
  ( <input/iasi/IASI_*> or <input/iasi/.done/IASI_*> ) or do {
    $self->create_obt( $file );
    my $satp = &satp_iasi_navigation( $self );
    $self->create_svm( $satp );
  };

  #
  
  ( <input/auxd/*> )
    or $self->get_configuration();

  # we have a IASI granule; if another IASI 
  # granule is waiting, we process it now
  
  $self->process_last_granule();

  #

  $self->store_pfsIASI00( $file );
}


sub process {

=item process

Dispatch to other process_* methods.
Force processing in case of dump.

=cut

  my ( $self, @argv ) = @_;

  for my $argv ( @argv ) {

    my $p = 'METOP::PFS'->new( file => $argv );

    $p or do {
      die( "$argv is not PFS\n" );
    };

    my ( $INST, $LV ) = @{ $p->{header} }{ qw( INSTRUMENT_ID PROCESSING_LEVEL ) };
  
    my $method = "process_pfs$INST$LV";
    if( $self->can( $method ) ) {
      $self->$method( $argv );
    } else {
      die( "$0 cannot process args of type pfs$INST$LV\n" );
    }


  }
  
  if( $self->{P} eq 'DUMP' ) { # Dump mode, we force processing
    $self->process_last_granule();
  }

}

sub store {

=item store

Dispatch to store_* methods.

=cut

  my ( $self, $file ) = @_;
  my $p = 'METOP::PFS'->new( file => $file );
  my ( $INST, $LV ) = @{ $p->{header} }{ qw( INSTRUMENT_ID PROCESSING_LEVEL ) };
  my $method = "store_pfs$INST$LV";
  if( $self->can( $method ) ) {
    $self->$method( $file );
  } else {
    die( "$0 cannot store args of type pfs$INST$LV\n" );
  }
}

sub finish {

=item finish

Process pending data.

=cut

  my ( $self, $file ) = @_;

  $self->store( $file );

  if( my @i = $self->get_unprocessed_granules() ) {
    my $l = pop( @i );

    for my $i ( @i ) {
      $self->process_granule( iasi => $i );
    }
  
    $self->process_granule( iasi => $l, last => 1 );

  }
  
}





1;

=back

=head1 SEE ALSO

L<NWPSAF-MF-UD-006>, L<IA-SB-2100-9462-CNE>, L<IA-DF-0000-2006-CNE>, L<AAPP_6.3>,
L<ops_process>

=head1 AUTHORS

Philippe.Marguinaud@meteo.fr

=head1 COPYRIGHT

Meteo-France/EUMETSAT/SAFNWP

=head1 HISTORY

=over 4

=item 01/03/2007

First version for HRPT processing.

=item 03/05/2007

Add a timeout for STEP commands.

=back

=cut

