package IASI::OPS::WORKORDER;

use strict;
use FileHandle;
use XML::Mini::Document;

=head1 NAME

IASI::OPS::WORKORDER

=head1 SYNOPSIS

  my $wo = 'IASI::OPS::WORKORDER'->new( file => 'IASI_1_wo_001' );
  $wo->read();

  $wo->write();

=head1 DESCRIPTION

This modules prepares a work-order for IASI-OPS. All work-order fields
are required to be filled:

=over 4

=item ProcessingType

=item SensingStart

=item SensingStop

=item TimeIntervalFlag

=item TimeIntervalCounter

=item UnProcData

=item AuxData

This is a list

=item ProdModel

This is a list

=back

=head1 METHODS

=cut

sub new {

=item new

Constructor.

=cut

  my $class = shift;
  return bless { @_ }, $class;
}

sub read : method {

=item read

Reads and parses a workorder file.

=cut

  my $self = shift;
  my $xml = do { 
    my $fh = 'FileHandle'->new( "<$self->{file}" ); 
    $fh->getline() for( 1 .. 2 ); # skip <?xml..> and <!DOCTYPE..>
    local $/ = undef; <$fh> 
  };
  $self->{xml} = $xml;
  my $p = 'XML::Mini::Document'->new();
  $p->parse( $xml );
  %$self = ( %$self, %{ ( $p->toHash() )->{PPF_Work_Order} } );
}

sub write : method { # we write the wo by hand

=item write

Writes a workorder to disk.

=cut

  my $self = shift;
  my $fh = 'FileHandle'->new( ">$self->{file}" );
  $fh->print( << "EOF" );
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE PPF_Work_Order SYSTEM "PPF_Work_Order.dtd">
<PPF_Work_Order>
<ProcessingType>$self->{ProcessingType}</ProcessingType>
<SensingStart>$self->{SensingStart}</SensingStart>
<SensingStop>$self->{SensingStop}</SensingStop>
<TimeIntervalFlag>$self->{TimeIntervalFlag}</TimeIntervalFlag>
<TimeIntervalCounter>$self->{TimeIntervalCounter}</TimeIntervalCounter>
<UnProcData>$self->{UnProcData}</UnProcData>
EOF
  for my $a ( @{ $self->{AuxData} } ) {
    $fh->print( "<AuxData>$a</AuxData>\n" );
  }
  for my $m ( @{ $self->{ProdModel} } ) {
    $fh->print( "<ProdModel>$m</ProdModel>\n" );
  }
  $fh->print( "</PPF_Work_Order>\n" );
}



1;

=back

=head1 SEE ALSO

L<IASI::OPS>, L<IASI::OPS::REPORT>, L<XML::Mini::Document>

=head1 AUTHORS

Philippe.Marguinaud@meteo.fr

=head1 COPYRIGHT

Meteo-France/EUMETSAT/SAFNWP

=head1 HISTORY

=over 4

=item 01/03/2007

First version for HRPT processing.

=back

=cut

