package METOP::MISC;

# 16/12/2011 update to to_YYYYMMDDHHmmssZ and to_time_t subroutines
# due to issues on some 64bits Linux RHEL6 systems 
# P. Brunel N. Atkinson

use strict;

use Getopt::Std;
use Data::Dumper;
use FileHandle;
use File::Basename;

use base qw( Exporter );
our @EXPORT = qw( getoptions cat execute to_YYYYMMDDHHmmssZ 
  to_time_t touch pfs_satellite_name aapp_satellite_name );

use liblog;

=head1 NAME

METOP::MISC

=head1 SYNOPSIS

  use METOP::MISC;
  ...

=head1 DESCRIPTION

Miscellaneous helper functions for IASI OPS main module.

=head1 ROUTINES

=over 4

=cut


sub getoptions {

=item getoptions

Parses options from command line. C<-s>, satellite, C<-d> YYYYMMDD,
C<-h> HHmm, C<-n> orbit number. Satellite name should be C<Mnn>, 
C<metopnn>; C<metopnn> is always returned.

=cut

  getopt( 'Ssdhn', \my %opts );
  $opts{s} &&= &aapp_satellite_name( $opts{s} );
  return \%opts;
}


sub cat {

=item cat

Prints out a file's content to stdout.

=cut

  my $fh = 'FileHandle'->new( "<$_[0]" );
  local $/ = undef;
  $fh && print <$fh>
}


sub execute {

=execute

Executes a command keeping its stdout and stderr, and 
prints them out if an error occur.

=cut

  my @cmd = @_;
  my $log = 'execute-' . basename( $cmd[0] ) . '.log';
  my $cmd = "@cmd > $log 2>&1";
  my $ret;
  log_info( $cmd );
  if( system( $cmd ) ) {
    if( $? == -1 ) {
      log_error( "$0: \"@cmd\" failed to execute: $!" );
    } elsif( my $s = $? & 127 ) {
      log_error( "$0: \"@cmd\" received signal $s" );
    } elsif( my $c = $? >> 8 ) {
      log_error( "$0: \"@cmd\" returned $c" );
    } 
    &cat( $log );
    $ret = 0;
  } else {
    $ret = 1;
  }

  unlink( $log );
  
  return $ret;
}



sub to_YYYYMMDDHHmmssZ {

=item to_YYYYMMDDHHmmssZ

Convert a C<time_t> number to a C<YYYYMMDDHHmmssZ> format.

=cut

  my ( $ss, $mn, $hh, $dd, $mm, $yyyy ) = localtime( $_[0] );
  $mm++;
  $yyyy += 1900;
  return sprintf( '%4.4d%2.2d%2.2d%2.2d%2.2d%2.2dZ', $yyyy, $mm, $dd, $hh, $mn, $ss );
}

sub to_time_t {

=item to_time_t

Convert a C<YYYYMMDDHHmmssZ> to a C<time_t> number.

=cut

  use POSIX qw( mktime );
  my ( $yyyy, $mm, $dd, $hh, $mn, $ss )
   = ( $_[0] =~ m/^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})Z$/o );
  $mm--;
  $yyyy -= 1900;
  for( $yyyy, $mm, $dd, $hh, $mn, $ss ) {
    s/^0*(.+)$/$1/go;
  }
  return mktime( $ss, $mn, $hh, $dd, $mm, $yyyy );
}

sub touch {

=item touch

Creates an empty file.

=cut

  ( -f $_[0] ) or 'FileHandle'->new( ">$_[0]" );
}

sub pfs_satellite_name {

=item pfs_satellite_name

Satellite name conversion: metopA, metop02, M02 converted to M02.

=cut

  my $sn = shift;
  if( $sn =~ m/^m(\d+)$/io ) {
    return sprintf( 'M%2.2d', $1 );
  }
  if( $sn =~ m/^metop(\d+)$/io ) {
    return sprintf( 'M%2.2d', $1 );
  }
  if( $sn =~ m/^metop([a-z])$/io ) {
    return sprintf( 'M%2.2d', ord( lc( $1 ) ) - ord( 'a' ) + 1 );
  }
}

sub aapp_satellite_name {

=item aapp_satellite_name

Satellite name conversion: M02, metop02, metopA converted to metop02.

=cut

  my $sn = shift;
  if( $sn =~ m/^m(\d+)$/io ) {
    return sprintf( 'metop%2.2d', $1 );
  }
  if( $sn =~ m/^metop(\d+)$/io ) {
    return sprintf( 'metop%2.2d', $1 );
  }
  if( $sn =~ m/^metop([a-z])$/io ) {
    return sprintf( 'metop%2.2d', ord( lc( $1 ) ) - ord( 'a' ) + 1 );
  }
}



1;


=back

=head1 SEE ALSO

=head1 AUTHORS

Philippe.Marguinaud@meteo.fr

=head1 COPYRIGHT

Meteo-France/EUMETSAT/SAFNWP

=head1 HISTORY

=over 4

=item 01/03/2007

First version for HRPT processing.

=back

=cut

