package METOP::PFS;

use strict;
use FileHandle;
use POSIX qw( mktime );
use Data::Dumper;
use Fcntl qw( SEEK_CUR SEEK_SET );


=head1 NAME

METOP::PFS

=head1 SYNOPSIS

  use METOP::PFS;
  my $pfs = 'METOP::PFS'->open( file => 
  'IASI_P71_00_M02_20070305224145Z_20070305224455Z_N_O_20070305224515Z' );

  print $pfs->{header}{INSTRUMENT_TYPE};

=head1 DESCRIPTION

This module provides access to PFS files headers. It can parse the MPHR
and update it.

=head1 METHODS

=over 4

=cut

sub new {
  my $class = shift;
  return $class->open( @_ );
}

sub open : method {

=item open

Open a PFS file and parse its header.

=cut


  my $class = shift;
  my $self = bless { @_ }, $class;

  my $f = $self->{file};

  my $fh = 'FileHandle'->new( "<$f" );
  $fh or return;

  my $buf;
  $fh->read( $buf, 20 );

  $self->{grh} = $buf;

  $fh->read( $buf, 3307 - 20 );

  my %h = ( $buf =~ m/^(\w+)\s+=\s+(\S.*\S|\S)$/gom );

  $self->{header} = \%h;

  return $self;
}

sub et_sensing_start {

=item et_sensing_start

Returns the C<time_t> of the C<SENSING_START> field.

=cut

  my $self = shift;
  my ( $yyyy, $mm, $dd, $hh, $mn, $ss ) 
   = ( $self->{header}{SENSING_START} =~ 
       m/^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})Z$/o );
  $mm--;
  $yyyy -= 1900;
  return mktime( $ss, $mn, $hh, $dd, $mm, $yyyy );
}

sub et_sensing_end {

=item et_sensing_end

Returns the C<time_t> of the C<SENSING_END> field.

=cut

  my $self = shift;
  my ( $yyyy, $mm, $dd, $hh, $mn, $ss ) 
   = ( $self->{header}{SENSING_END} =~ 
       m/^(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})Z$/o );
  $mm--;
  $yyyy -= 1900;
  return mktime( $ss, $mn, $hh, $dd, $mm, $yyyy );
}

sub create_empty {
  my $self = shift;
  my %args = @_;
  my $fh = 'FileHandle'->new( ">$args{file}" );
  $fh->print( ' ' x 19, "\n" );
  my $header = $self->{header};
  $fh->print( << "EOF" );
SPACECRAFT_ID  = $header->{SPACECRAFT_ID}
SENSING_START  = $header->{SENSING_START}
ORBIT_START    = $header->{ORBIT_START}
TOTAL_MDR      = 0
EOF
}

sub update_mphr {

=item update_mphr

Writes the C<header> field to file.

=cut

  my $self = shift;

  my $fh = 'FileHandle'->new( "+<$self->{file}" );
  $fh->seek( 20, SEEK_SET );
  while( my $line = <$fh> ) {
    while( my ( $k1, $v1 ) = each( %{ $self->{header} } ) ) {
      my ( $k2, $s, $v2 ) = ( $line =~ m/^(\w+)(\s*)=(.*)/o );
      my $n = length( $v2 );
      ( $k1 eq $k2 ) && do {
        $fh->seek( - length( $line ), SEEK_CUR );
        $fh->printf( "$k2$s=%${n}s\n", $v1 );
      };
    }
    last if( $fh->tell() >= 3307 );
  }
  $fh->close();
}



1;

=back

=head1 SEE ALSO

L<METOP::PFS::GRH>, L<EPS.GGS.SPE.96167>

=head1 AUTHORS

Philippe.Marguinaud@meteo.fr

=head1 COPYRIGHT

Meteo-France/EUMETSAT/SAFNWP

=head1 HISTORY

=over 4

=item 01/03/2007

First version for HRPT processing.

=back

=cut

