static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/CMN/src/CMN__Securite.cc,v 1.3 2011/11/25 10:17:39 dpauly Exp $";
//@ ------------------------------------------------------------------*/
//@ Composant : CMN
//@ MODULE : CMN__Securite
//@
//@ Commentaire :
//@    Ce module gere les fonctionnalites de gestion des aspects
//@    securite:
//@      Inhiber les cores: CMN__Securite_NoCore
//@      Capter les signaux: CMN__Securite_Signaux
//@      Positionne l'environnement: CMN__Securite_Environ
//@      Controler les privileges: CMN__Securite_Init
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 13/04/2000 : Pierre-Louis Agnetti
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-159-CN : 26/05/2005 : JL. PASCAL
//@ modif positionnement du PATH
//@ fiche de correction correspondante : IA-2100-FC-159-THA
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-3316(DM-2100-OPS) : 18/05/2017 : MPE
//@ PORTAGE OPS SOUS RHEL 6.5 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//
#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/resource.h>

#include "CMN__Commun.h"
#include "CFG__Configuration.h"
#include "CMN__Trace.h"
#include "CMN__Securite.h"

// Migration RHEL 6.5
#ifdef LINUX
#include <cstdlib>
#endif

//====================================================================*/
// Declaration des define internes
//====================================================================*/


//======================================================================
// Declaration des fonctions internes
//======================================================================

static void CMN__Securite_NoCore();
static void CMN__Securite_Signaux(void (*fterm)(int),void (*fexit)(int));
static void CMN__Securite_Environ();

//======================================================================
// Declaration des variables globales
//======================================================================

//======================================================================
//             FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Securite_Init
//@
//@ Description : Initialisation des concepts securite en 
//@               appelant les differents modules necessaires
//@               CMN__Securite_NoCore, CMN__Securite_Signaux,
//@               CMN__Securite_Environ
//@
//@ Parametres :
//@               (ENTREE) pointeur de fonction arret propre ou NULL
//@               (ENTREE) pointeur de fonction arret rapide ou NULL
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void CMN__Securite_Init(void (*fterm)(int),void (*fexit)(int))
{
  //@ Inhibition des cores
  CMN__Securite_NoCore();

  //@ Positionnement des signaux
  CMN__Securite_Signaux(fterm,fexit);

  //@ Positionnement des variables d'environnement
  CMN__Securite_Environ();
}


//======================================================================
//             FONCTIONS INTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Securite_NoCore
//@
//@ Description : Inhibe la creation de fichier core en mode non DEBUG
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Securite_NoCore()
{
#ifndef DEBUG
  struct rlimit rlp;
  int crset=0;
  
  //@ Positionner la limite de taille du fichier core a 0
   rlp.rlim_max=0;
   rlp.rlim_cur=0;
   crset=setrlimit(RLIMIT_CORE,&rlp);
   TR1(9,"Set limit core cr:%d",crset);
#endif
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Securite_Signaux
//@
//@ Description : Inhibe tous les signaux et traite l'appel d'une 
//@               fonction pour SIGTERM et SIGQUIT,SIGILL,SIGSEGV
//@
//@ Parametres :
//@               (ENTREE) pointeur de fonction arret propre
//@               (ENTREE) pointeur de fonction arret rapide
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Securite_Signaux(void (*fterm)(int),void (*fexit)(int))
{
   sigset_t sig_set;
   sigset_t old_set;
   struct sigaction saterm;
   struct sigaction saexit;
   

   //@ Initialiser la structure signal
   sigemptyset(&sig_set);
   
   //@ Ignorer les signaux normaux
   //@ SIGINT,SIGHUP,SIGABRT,SIGCHLD
   sigaddset(&sig_set,SIGINT);
   sigaddset(&sig_set,SIGHUP);
   sigaddset(&sig_set,SIGABRT);
   sigaddset(&sig_set,SIGCHLD);
   
   //@ SI une fonction est donnee pour
   //@ l'arret normal, la positionner
   //@ sur le signal SIGTERM
   if (fterm==NULL)
     {
       //@ positionner la procedure sur les signaux
       sigaddset(&sig_set,SIGTERM);
     }
   //@ SINON ignorer le signal
   else
     {
       saterm.sa_handler=fterm;
       sigemptyset(&saterm.sa_mask);
       saterm.sa_flags=0;
       sigaction(SIGTERM,&saterm,NULL);
     }
   //@ FIN SI

   //@ SI une fonction est donnee pour
   //@ l'arret rapide, la positionner
   //@ sur les signaux d'urgence
   if (fexit==NULL)
     {
       //@ positionner la procedure sur les signaux
       sigaddset(&sig_set,SIGQUIT);
       sigaddset(&sig_set,SIGILL);
       sigaddset(&sig_set,SIGTRAP);
       sigaddset(&sig_set,SIGIOT);
       sigaddset(&sig_set,SIGFPE);
       sigaddset(&sig_set,SIGBUS);
       sigaddset(&sig_set,SIGSEGV);
     }
   //@ SINON ignorer les signaux
   else
     {
       saexit.sa_handler=fexit;
       sigemptyset(&saexit.sa_mask);
       saexit.sa_flags=0;
       sigaction(SIGQUIT,&saexit,NULL);
       sigaction(SIGILL,&saexit,NULL);
       sigaction(SIGTRAP,&saexit,NULL);
       sigaction(SIGIOT,&saexit,NULL);
       sigaction(SIGFPE,&saexit,NULL);
       sigaction(SIGBUS,&saexit,NULL);
       sigaction(SIGSEGV,&saexit,NULL);
     }
   //@ FIN SI

   sigprocmask(SIG_BLOCK,&sig_set,&old_set);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : CMN__Securite_Environ
//@
//@ Description : Positionne les variables d'environnement
//@               Positionnement des signaux
//@               Inhibition des core
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
static void CMN__Securite_Environ()
{
  // Variable de travail
  static char path[CMN__LG_CHAINE_FIXE];
  
  
  //@ Demarrage dans le repertoire racine du logiciel
  chdir(CFG__Configuration::GetRepertoire("RACINE_OPS").c_str());
  
  //@ Positionnement du PATH
  // FA 159
  sprintf(path,"PATH=/bin:/sbin:/usr/bin:/usr/sbin:%s:%s",
	  CFG__Configuration::GetRepertoire("BIN").c_str(),
	  CFG__Configuration::GetRepertoire("SHELL").c_str());
  putenv(path);
   
  //@ Positionnement IFS SHELL
  // FT-2765
  putenv(static_cast<char *>("IFS= \t\n"));
  putenv(static_cast<char *>("SHELL=/bin/sh"));
  // ...FT-2765

  //@ Positionnement de umask
  /* SMM-M4-FA-56-CN */
  umask(026);
}

