//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/ALG/src/SD_ALG_AVHRR.h,v 1.4 2012/07/10 16:49:16 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_ALG
//@ MODULE : SD_ALG_AVHRR
//@
//@ Comment :
//@    include file for the SD_ALG_AVHRR class which implements the
//@    algorithm used to compare IASI and AVHRR images in order to :
//@ - analyse IIS type
//@ - geolocalize IASI L1 products
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ VERSION : 27/02/2003 : [reference :] M. Jung
//@ --------------------------------------------------------------------
//@ PM-ID : IA-CR-0000-242-CNE : 06/08/2004 : B. MASSART
//@ 44_GEO is now called if Cdegraded cases #9 and #15 are raised.
//@ See IA-2100-CR-242-THA.
//@ --------------------------------------------------------------------
//@ PM-ID : IA-2100-CR-292-CNE : 10/03/2006 : B.MASSART
//@ Function SD_ALG_AVHRR_GetSubSatLatLon is added to set up 
//@ MPHR_SUBSAT_LATITUDE/LONGITUDE_START/END fields in Level1 products.
//@ See IA-2100-CR-292-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2521(DM-2100-OPS) : 24/04/2009 : S. BRUNEL
//@ Calcul de 4 pseudo-canaux AVHRR (Day 2, point 2)
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2751(FA-2100-OPS) : 22/03/2010 : S.BRUNEL
//@ Probleme de reproductibilite sur les variables 
//@ SUBSAT_XXXX_START et SUBSAT_XXXX_END
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2997(DM-2100-OPS) : 18/06/2012 : L. CASPAR
//@ Modification des entrees de 41_CCS et 44_GEO,et ajout des constantes  
//@ SD_ALG_USE_AVHRR_DATA et SD_ALG_DELTA_TIME pour calculer la 
//@ geolocalisation sans l'AVHRR
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#ifndef SD_ALG_AVHRR_h
#define SD_ALG_AVHRR_h


// inserer ici les fichiers include
#include <stdio.h>       /* entree/sortie */
#include <stdlib.h>      /* alloc memoire, ...*/
#include <string.h>      /* chaines de carateres */
#include <unistd.h>      /* gestion pipes, ... */
#include <time.h>        /* gestion de la date */
#include <sys/param.h>   /* constantes systeme */

#include "SD_BAS_Contexte.h"
#include "SD_BAS_AVHRR.h"
#include "SD_BAS_Global.h"
#include "SD_BAS_Niveau1.h"
#include "SD_BAS_Niveau0.h"
#include "SD_BAS_ConfAutre.h"
#include "SD_BAS_ConfStable.h"
#include "SD_BAS_Produits.h"
#include "ERR__Erreur.h"
#include "SD__Message.h"





/*====================================================================*/
//   External define  Declarations
/*====================================================================*/
#define ERR_BASE_AVHRR                          1340 /*SD_ALG_ERR_BASE_AVHRR */
#define SD_ALG_AVHRR_ERR_ALLOCATION             (ERR_BASE_AVHRR+1)
#define SD_ALG_AVHRR_ERR_METOP2AVHRR            (ERR_BASE_AVHRR+2)
#define SD_ALG_AVHRR_ERR_DOMAIN                 (ERR_BASE_AVHRR+3)
#define SD_ALG_AVHRR_ERR_METOP2GEOD             (ERR_BASE_AVHRR+4)
#define SD_ALG_AVHRR_ERR_MOINDRES_CARRES        (ERR_BASE_AVHRR+5)
#define SD_ALG_AVHRR_ERR_CHANNEL_UNAVAILABLE    (ERR_BASE_AVHRR+6)
#define SD_ALG_AVHRR_ERR_NUL_DISTANCE           (ERR_BASE_AVHRR+7)
#define SD_ALG_AVHRR_ERR_NUL_EIG_VAL            (ERR_BASE_AVHRR+9)
#define SD_ALG_AVHRR_ERR_EIG_VAL_PROCESSING     (ERR_BASE_AVHRR+11)
#define SD_ALG_AVHRR_ERR_METOP2IASI             (ERR_BASE_AVHRR+13)


#define SD_BAS_ERR_SET_AVHRR                   1

/*====================================================================*/
// Constant Declaration
/*====================================================================*/
#define SD_ALG_AVHRR_MODE_AVHRR 0
#define SD_ALG_AVHRR_MODE_IIS   1
#define SD_ALG_AZIM 0
#define SD_ALG_ELEV 1
#define SD_ALG_MAX_ITER 9
#define SD_ALG_COINS 4

// FT-2997
#define SD_ALG_USE_AVHRR_DATA 0
#define SD_ALG_DELTA_TIME 17000
// ...FT-2997

// Definitions pour SD_ALG_ConstruireAvhrrChannelState
#define AVHRR1         1
#define AVHRR2         2
#define AVHRR3         3
#define AVHRR4         4
#define AVHRR5         5
#define AVHRR3a      0x0080
#define AVHRR3b      0x0000
#define MASK_AVHRR1  0x2000
#define MASK_AVHRR2  0x1000
#define MASK_AVHRR3a 0x0800
#define MASK_AVHRR3b 0x0400
#define MASK_AVHRR4  0x0200
#define MASK_AVHRR5  0x0100
#define MASK_AVHRR3  0x0080

/*====================================================================*/
// Exported types Declaration
/*====================================================================*/

/*====================================================================*/
// External variables Declaration
/*====================================================================*/
#ifdef __cplusplus
extern "C" {
#endif

/*====================================================================*/
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_40_IAC
//@
//@ Description : algorithme de co-registration Imageur-AVHRR
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@          numero de ligne SN traite
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  Contexte : Contexte
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  MPHRN1 :  MPHR produit N1
//@         (ENTREE)  GIrcImg : image calibree
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (Entree/Sortie)  InfoDet : info det du produit ENG
//@                 pour le SN (on choisira la date UTC
//@                 ( on peut aussi trouver ceci avec InfoDet1ENG)
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_40_IAC(
  t_SD_BAS_DataThread         *ctx,
  t_SD_BAS_DataThreadProduit  *ctp,
  t_SD_BAS_Contexte           *Contexte,
  t_SD_BAS_ConfAutre          *ConfAutre,
  t_SD_BAS_ConfStable         *ConfStable,
  t_SD_BAS_MPHR               *Mphr,
  double                      *GIrcImg,
  t_SD_BAS_LigneAVHRR        **BandeauAVHRR,
  int                          nbligbandeau,
  t_SD_BAS_Liste_Var          *InfoDet1ENG
);

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_41_CCS
//@
//@ Description : analyse des radiances dans les FOV sondeurs
//@
//@ Commentaire :
//@    Les flags de disponibilte des produits "ChannelState" sont
//@    generes a l'interieur de 41_CCS (pas passes en parametres)
//@    A la demande du CNES "EigValCutoff" a ete rajoute dans les
//@    parametres.
//@
//@ Parameters :
//@         (ENTREE)  ctx .......... : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE)  ConfAutre .... : fichier de conf autre
//@         (ENTREE)  ConfStable ... : fichier de conf stable
//@         (ENTREE)  MPHRN1 ....... : MPHR produit N1
//@         (ENTREE)  GIrcImg ...... : image calibree
//@         (ENTREE)  AvhrrListeLin  : Suite de lignes AVHRR chargees en memoire
//@         (ENTREE)  AvhrrNbLin ... : Taille de la liste de lignes
//@
//@         (ENTSOR)  InfoDet1C .... : info det du produit 1C
//@                   pour le SN (on choisira la date UTC
//@
//@         (SORTIE)  ProduitCourant : pour pointer sur l'image utile
//@         (SORTIE)  InfoDet1ENG .. : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_41_CCS(
  t_SD_BAS_DataThread         *ctx,                // t_SD_BAS_DataThread*
  t_SD_BAS_ConfAutre          *ConfAutre,          // t_SD_BAS_ConfAutre*
  t_SD_BAS_ConfStable         *ConfStable,         // t_SD_BAS_ConfStable*
  t_SD_BAS_MPHR               *MphrN1,             // t_SD_BAS_MPHR*
  double                      *GIrcImg,            // double( (SD_BAS_IMCO * SD_BAS_IMLI) * SD_BAS_SN)
  t_SD_BAS_LigneAVHRR        **AvhrrListeLinIN,    // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  int                          AvhrrNbLinIN,       // int(1)
  double                       EigValCutoff,       // double(1) (= 1.0E-2)
  t_SD_BAS_Liste_Var           InfoDet1C,          // t_SD_BAS_Liste_Var
  t_SD_BAS_DataThreadProduit  *ProduitCourant,     // t_SD_BAS_DataThreadProduit*
  t_SD_BAS_Liste_Var           InfoDet1ENG         // t_SD_BAS_Liste_Var
);

// FT-2521
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_42_PCH
//@
//@ Description : Calcul de 4 pseudo-canaux AVHRR
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx .......... : contexte de l'etape (numero de ligne SN traite)
//@         (ENTREE)  ConfAutre .... : fichier de conf autre
//@         (ENTREE)  ConfStable ... : fichier de conf stable
//@         (ENTREE)  spectre1C .... : spectre IASI de niveau 1C
//@         (ENTREE)  InfoDet1C .... : info det du produit 1C
//@         (SORTIE)  InfoDet1ENG .. : info det du produit ENG
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_42_PCH(
  // Entree
  t_SD_BAS_DataThread *ctx,
  t_SD_BAS_ConfAutre *ConfAutre,
  t_SD_BAS_ConfStable *ConfStable,                 
  double *spectre1C, // spectre IASI de niveau 1C
  t_SD_BAS_Liste_Var  InfoDet1C,
  // Sortie
  t_SD_BAS_Liste_Var  InfoDet1ENG
);
// ...FT-2521

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_ALG_44_GEO
//@
//@ Description : Geolocalisation des produits IASI
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape numero de ligne SN traite
//@         (ENTREE)  ctp : variables temporaires
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE)  ConfStable : fichier de conf stable
//@         (ENTREE)  StatMphr : Infos globales MPHR
//@         (ENTREE)  MPHRN1 :  MPHR produit N1
//@         (Entree)  InfoDet1ENG : info det du produit ENG
//@         (Sortie)  InfoDet1ABC : info det du produit 1ABC pour le SN (on choisira la date UTC
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  OffsetIISAvhrr : Offset IIS-AVHRR dans le repere
//@                                     AVHRR (ligne,colonne)
//@         (ENTREE)  flag_geo_avhrr_miss : vaut VRAI si CG9 ou CG15
//@
//@ Return Status : test
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_ALG_44_GEO(
       t_SD_BAS_DataThread         *ctx,
       t_SD_BAS_DataThreadProduit  *ctp,
       t_SD_BAS_ConfAutre          *ConfAutre,
       t_SD_BAS_ConfStable         *ConfStable,
       t_SD_BAS_StatMphr           *StatMphr,
       t_SD_BAS_MPHR               *Mphr,
       t_SD_BAS_Liste_Var          *InfoDet1ENG,
       t_SD_BAS_Liste_Var          *InfoDet1ABC,
       int                          nbligbandeau,
       t_SD_BAS_LigneAVHRR        **BandeauAVHRR,
  		/* CR242 */
		// FT-2997
		double                      *OffsetIISAvhrrRI,	//Sortie de calcOffset
		// ...FT-2997
		t_CMN__Booleen              flag_geo_avhrr_miss 
);

/* CR292 */
//@ ------------------------------------------------------------------*/
//@ Method : SD_ALG_AVHRR_GetSubSatLatLon
//@
//@ objet :  estime la position subsatellite a partir de la date ENG_GEPSDATIASI
//@
//@
//@ Parametres :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  ConfAutre : fichier de conf autre
//@         (ENTREE/SORTIE)  StatMphr : Infos globales MPHR
//@         (ENTREE)  Mphr :  MPHR produit N1
//@         (ENTREE)  InfoDet1ENG : info det du produit ENG
//@
//@ Return Status :
//@         t_CMN_GLB_Statut
//@         CMN__STATUT_OK                    si la fonction a reussi
//@         SD_ALG_AVHRR_GetSubSatLatLon_ALRT si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2751
t_CMN__Statut SD_ALG_AVHRR_GetSubSatLatLon(
  t_SD_BAS_DataThread         *ctx,
  t_SD_BAS_ConfAutre          *ConfAutre,
  t_SD_BAS_StatMphr           *StatMphr,
  t_SD_BAS_MPHR               *Mphr,
  t_SD_BAS_Liste_Var          *InfoDet1ENG,
  int                         nombreLignes);
// ...FT-2751
/* FIN CR292 */

// FT-2997
t_CMN__Statut SD_ALG_CalcOffset(
  // Entrees pour MetopAVHRR
  int                   CCSMode,         // int(1)
  t_SD_BAS_DataThread  *Contexte_courant,   // t_SD_BAS_DataThread*
  t_SD_BAS_MPHR        *MphrN1,             // t_SD_BAS_MPHR*
  t_SD_BAS_LigneAVHRR **AvhrrListeLin,      // t_SD_BAS_LigneAVHRR * ( AvhrrNbLin)
  int                   AvhrrNbLin,         // int(1)
  int                   AvhrrLinDeb,        // int(1)
  int                   AvhrrLinFin,        // int(1)
  int                   AvhrrColDeb,        // int(1)
  int                   AvhrrColFin,        // int(1)
  // Entrees
  t_SD_BAS_SCDSTime     IasiTime,           // t_SD_BAS_SCDSTime*
  double                IasiScanAngle,      // double(1)
  double               *IasiMetopMatRot,    // double( SD_BAS_DIM_ROT_MATRICE * SD_BAS_DIM_ROT_MATRICE)
  double               *OffsetSondIISModel, // double( SD_BAS_MODEL * SD_BAS_CCD * SD_BAS_PN)
  t_SD_BAS_SCDSTime     t_FE,               // t_SD_BAS_SCDSTime*
  double                MetopPeriod,        // double(1)
  double               *OffsetIISAvhrr,     // double( SD_BAS_DEUX)
  // Sorties
  int                  *FlagDateNOK,        // unsigned int(1)
  double               *OffsetSondIIS,      // double( SD_BAS_DEUX)
  double               *OffsetSondAvhrr,    // double( SD_BAS_DEUX)
  double 			   *OffsetIISAvhrrRI    // double( SD_BAS_DEUX)
);
// ...FT-2997

//======================================================================
#ifdef __cplusplus
}
#endif


#endif

