//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_BanqueSpectrale
//@
//@ Comment :
//@    Code file for the  SD_BAS_BanqueSpectrale module which
//@    encapsulates the acces to the spectral data base file
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include "SD_BAS_BanqueSpectrale.h"

#include <stdio.h>       /* entree/sortie */
#include <fcntl.h>       /* O_RDWR, ... */

// FT-2765
#include "SD_BAS_SwapEndian.h"
// ...FT-2765

/*====================================================================*/
// Local Constant Declaration 
/*====================================================================*/
static const int SD_BAS_SIZE_INT4      = 4;
static const int SD_BAS_SIZE_COMPLEX32 = 8;
static const int SD_BAS_SIZE_REAL64    = 8;
// FT-2765
static const int SD_BAS_SIZE_REAL32    = 4;
// ...FT-2765

/* macro de detection d'erreur sur les fread() */
#define SD_BAS_BS_ERR_FREAD(ret, nb_octets)\
{\
   if ((ret) != (nb_octets))\
   {\
     cr_op   = SD_BAS_ERR_VERSION_BS;\
     msg_err = SD_BAS_MSG_BS;\
     num_lig = __LINE__;\
     goto fin;\
   }\
}

/* Gestion des erreurs */
#define SD_BAS_ERR_VERSION_BS      (SD_BAS_ERR_BASE_BanqueSpectrale + 0)
static char* SD_BAS_MSG_BS = "SPECTRAL DATA BASE READ ERROR";
char * SD_BAS_MSG_VERSION_BS = "INVALID SPECTRAL DATABASE FILE VERSION";
const int SD_BAS_ERR_SET_BS = 1;

/*====================================================================*/
// Local types Declaration
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================

//======================================================================
// Global variables Definition
//======================================================================


//======================================================================
//					FONCTIONS EXTERNES
//======================================================================

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ChargerBS
//@
//@ Description : ouverture lecture et fermeture du fichier de banque
//@ spectrale indique en argument
//@
//@
//@ Parameters :
//@			(ENTREE)  Fichier : nom du fichier de bande spectrale
//@			(ENTREE/SORTIE)  BS : banque spectrale lue
//@  			(ENTREE)  ctx : contexte de l'etape
//@
//@ Return Status :
//@			t_CMN__Statut
//@			CMN__STATUT_OK si la fonction a reussi
//@		        SD_BAS_BS_ERR_OUVERTURE si on ne peut pas ouvrir le fichier
//@                     SD_BAS_BS_ERR_FERMETURE si on ne peut pas fermer le fichier
//@                     SD_BAS_BS_ERR_LECTURE   si on ne peut lire
//@                     SD_BAS_VERSION si la longueur du fichier ou le numero
//@                     d'enregistrement sont different de ceux predefinis
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ChargerBS(
		char * Fichier,
		t_SD_BAS_BanqueSpectrale * BS,
		t_SD_BAS_DataThread *ctx
			)
{
  static const char tcProcName_F[] = "SD_BAS_ChargerBS";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  int              num_lig;             /* num de la ligne ou l'erreur est detectee */
  char*            msg_err;             /* pointeur vers le message d'erreur */
  FILE*	           fp = NULL;
  int              ret;
  int              nb_octets_a_lire;
  unsigned int     days;
  // FT-2765
  int i; 
  // ...FT-2765

   /* ouverture du fichier */
  fp = fopen(Fichier, "rb");
  if (fp == (FILE*)NULL)
  {
    /* affectation des variables de gestion des erreurs */
    cr_op   = SD_BAS_ERR_BAS_NIVEAU;
    msg_err = SD_BAS_MSG_OPEN;
    num_lig = __LINE__;
    goto fin;
  }

  /* Lecture des donnees */

  /* Issue Number of the format of the configuration file */
  ret = fread(&(BS->IDefIssueIcd), SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefIssueIcd),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Revision Number of the format of the configuration file */
  ret = fread(&(BS->IDefRevisionIcd), SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefRevisionIcd),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Identification of this spectral database */
  ret = fread(&(BS->IDefSdbID), SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSdbID),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Date of the validity period of the IASI l1 PPS spectral database */
  ret = fread(&days, SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&days,SD_BAS_SIZE_INT4,1);
  // ...FT-2765
  BS->IDefSdbDate.day = (unsigned short)days;

  ret = fread(&BS->IDefSdbDate.millis, SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSdbDate.millis),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Number of the METOP satellite */
  ret = fread(&(BS->IDefMETOPNumber), SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefMETOPNumber),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Spectral width of the spectral functions sampling */
  ret = fread(&BS->IDefSafDWn, SD_BAS_SIZE_REAL64, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSafDWn),SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* Number of the sample corresponding to the first instrument function */
  ret = fread(&BS->IDefSafNsfirst, SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSafNsfirst),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Number of the sample corresponding to the last instrument function */
  ret = fread(&BS->IDefSafNslast, SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSafNslast),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Number of points for the sampling of the OPD */
  ret = fread(&BS->IDefSafNsOpd, SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSafNsOpd),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Maximum of optical path difference represented by the self-apodization function */
  ret = fread(&BS->IDefSafOpdMax, SD_BAS_SIZE_REAL64, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSafOpdMax),SD_BAS_SIZE_REAL64,1);
  // ...FT-2765

  /* Number of columns of the grid of interferometric axis positions */
  ret = fread(&BS->IDefSdbGridNbCol, SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSdbGridNbCol),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Number of lines of the grid of interferometric axis positions */
  ret = fread(&BS->IDefSdbGridNbLin, SD_BAS_SIZE_INT4, 1, fp);
  SD_BAS_BS_ERR_FREAD(ret, 1);
 	// FT-2765
  SWAP_INT_BUFFER(&(BS->IDefSdbGridNbLin),SD_BAS_SIZE_INT4,1);
  // ...FT-2765

  /* Y angular coordinates of the grid points */
// jlp
  nb_octets_a_lire = SD_BAS_SIZE_REAL64*SD_BAS_CDX*SD_BAS_SDB_NB_COL;
  ret = fread(BS->IDefSdbGridY, 1, nb_octets_a_lire, fp);
  SD_BAS_BS_ERR_FREAD(ret, nb_octets_a_lire);
 	// FT-2765
  SWAP_INT_BUFFER(BS->IDefSdbGridY,SD_BAS_SIZE_REAL64,SD_BAS_CDX*SD_BAS_SDB_NB_COL);
  // ...FT-2765

  /* Z angular coordinates of the grid points */
// jlp
  nb_octets_a_lire = SD_BAS_SIZE_REAL64*SD_BAS_CDX*SD_BAS_SDB_NB_LIG;
  ret = fread(BS->IDefSdbGridZ, 1, nb_octets_a_lire, fp);
  SD_BAS_BS_ERR_FREAD(ret, nb_octets_a_lire);
 	// FT-2765
  SWAP_INT_BUFFER(BS->IDefSdbGridZ,SD_BAS_SIZE_REAL64,SD_BAS_CDX*SD_BAS_SDB_NB_LIG);
  // ...FT-2765

  /* Apodisation functions in the spectral database */
// jlp
  nb_octets_a_lire = SD_BAS_SIZE_COMPLEX32 * SD_BAS_SDB_NB_LIG * SD_BAS_SDB_NB_COL
    * SD_BAS_CDX * SD_BAS_PN * SD_BAS_SDB_NFSMAX * SD_BAS_SAF_NS_OPD;
  ret = fread(BS->ISdbGridAf, 1, nb_octets_a_lire, fp);
  SD_BAS_BS_ERR_FREAD(ret, nb_octets_a_lire);
 	// FT-2765
  	SWAP_INT_BUFFER(BS->ISdbGridAf,SD_BAS_SIZE_REAL32,\
		SD_BAS_SDB_NB_LIG*SD_BAS_SDB_NB_COL*SD_BAS_CDX*SD_BAS_PN*SD_BAS_SDB_NFSMAX*SD_BAS_SAF_NS_OPD*2);
  // ...FT-2765

  /* Spectral calibration functions in the spectral database */
// jlp
  nb_octets_a_lire = SD_BAS_SIZE_REAL64 * SD_BAS_SDB_NB_LIG*SD_BAS_SDB_NB_COL
    * SD_BAS_CDX * SD_BAS_PN * SD_BAS_SDB_NFSMAX;
  ret = fread(BS->ISdbGridFcs, 1, nb_octets_a_lire, fp);
  SD_BAS_BS_ERR_FREAD(ret, nb_octets_a_lire);
 	// FT-2765
  SWAP_INT_BUFFER(BS->ISdbGridFcs,SD_BAS_SIZE_REAL64,SD_BAS_SDB_NB_LIG*SD_BAS_SDB_NB_COL*SD_BAS_CDX*SD_BAS_PN*SD_BAS_SDB_NFSMAX);
  // ...FT-2765

  /* Spectral shifts computed at the grid points */
// jlp
  nb_octets_a_lire = SD_BAS_SIZE_REAL64 * SD_BAS_SDB_NB_LIG * SD_BAS_SDB_NB_COL
    * SD_BAS_WNP_NB_SW * SD_BAS_CDX * SD_BAS_PN;
  ret = fread(BS->ISdbGridWnShift, 1, nb_octets_a_lire, fp);
  SD_BAS_BS_ERR_FREAD(ret, nb_octets_a_lire);
 	// FT-2765
  SWAP_INT_BUFFER(BS->ISdbGridWnShift,SD_BAS_SIZE_REAL64,SD_BAS_SDB_NB_LIG*SD_BAS_SDB_NB_COL*SD_BAS_WNP_NB_SW*SD_BAS_CDX*SD_BAS_PN);
  // ...FT-2765

  /* Fermeture du fichier */
  ret = fclose(fp);
  if (ret == EOF)
  {
    /* affectation des variables de gestion des erreurs */
    cr_op   = SD_BAS_ERR_BAS_NIVEAU;
    msg_err = SD_BAS_MSG_CLOSE;
    num_lig = __LINE__;
    goto fin;
  }

  //verification version revision
  if(BS->IDefIssueIcd > SD_BAS_BANQUE_SPECTRALE_VERSION.major ||
     (BS->IDefIssueIcd ==  SD_BAS_BANQUE_SPECTRALE_VERSION.major && 
      BS->IDefRevisionIcd > SD_BAS_BANQUE_SPECTRALE_VERSION.minor))
    {
      cr_op   = SD_BAS_ERR_VERSION_BS;
      msg_err = SD_BAS_MSG_VERSION_BS;
      num_lig = __LINE__;
      goto fin;
    }
  /* fin de la fonction : gestion des erreurs */
  fin:
  if ( cr_op != CMN__STATUT_OK )
  {
    /* mise a jour de la variable ctx->cr.errinterne */
    SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
			SD_BAS_ERR_SET_BS,
			SD_BAS_ERR_NUM_JDB,
			tcProcName_F,
			num_lig,
			msg_err);


  }

  /* sortie de la fonction */
  return(cr_op); 
}

//======================================================================
//					FONCTIONS INTERNES
//======================================================================
