//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_LibESSL
//@
//@ Comment :
//@    Code file for the  SD_BAS_LibESSL class which encapsulates the
//@    acces to the IBM ESSL librarie used for   :
//@ - Fast Fourier Transform.
//@ - Spline/linear Interpolation.
//@     - least Square approximation.
//@ - .....
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>
// FT-2765
#include <time.h>
#include <pthread.h>

static pthread_mutex_t fftw3_mutex = PTHREAD_MUTEX_INITIALIZER;
// ...FT-2765

#include "SD_BAS_LibESSL.h"

// FT-2765
#include "SD_BAS_AUXLibESSL.h"

// P. Brunel 30/08/2011
// Ne pas placer de ifdef autour de ces includes
// En Spline 2D c est Ngmath qui est utilise de toute facon
// La macro USE_SPLINE_NUMREC est definie dans le fichier SD_BAS_AUXLibESSL.h
#include "lapack-lite.h"
#include "ngmath-lite.h"
#include "numrec-lite.h"
// ...P. Brunel 30/08/2011

#ifdef ESSL_TEST
#define ESSL_TEST_HOME "/tmp"
#define ESSL_TEST_BEGIN( ESSL_PROC ) \
  { \
    static pthread_mutex_t test_essl_mutex = PTHREAD_MUTEX_INITIALIZER; \
    static int essl_test_done = 0; \
    pthread_mutex_lock( &test_essl_mutex ); \
    if( 1 ) { \
      FILE *fp = fopen( ESSL_TEST_HOME #ESSL_PROC, "a" ); \
      essl_test_done++; \
      printf( " essl_test_done = %d\n", essl_test_done );
#define ESSL_TEST_END \
      fclose( fp ); \
    } \
    pthread_mutex_unlock( &test_essl_mutex ); \
  }
#endif
// ...FT-2765

/*====================================================================*/
/* Local Constant Declaration                                         */
/*====================================================================*/
/*====================================================================*/
/* Local types Declaration                                            */
/*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================

//======================================================================
// Global variables Definition
//======================================================================

static char *TAILLE_FFT_INCOMPATIBLE = "Taille de FFT incompatible";
static char *ERREUR_CONVERGENCE = "Erreur de convergence";
static char *ERREUR_NFIRST = "Erreur sur le parametre nFirst";
static char *SD_BAS_DGESVF = "Error in LibESSL DGESVF routine";
static char *SD_BAS_INTERP2D = "Error in 2D interpolation routine : 1D interpolation required";

// FT-2765
#ifdef OPS_USE_ESSL
static int (*iusadr) () = enotrm;
static int ierno       = 2001;    // Premiere erreur sur les arguments
static int inoal       = 256;     // Pour permettre un nombre d'erreurs illimite
#ifdef DEBUG
static int inomes      = 0;       // Pour conserver les messages d'erreur sur stdout
#else
static int inomes      = 256;
#endif
static int itrace      = 0;
static int irange      = 2200;    // pour continuer dans tous les  cas

#define SD_GES_MARGE 0
#endif
// ...FT-2765


//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FFT1DirDR2C
//@
//@ Description : transformee de fourier directe 1D ( reel vers complexe)
//@ pour des pb d'efficacite il est souhaitable ici de regrouper le calcul de
//@ plusieurs FFT . si ce n'est pas possible utiliser l' argument d'appel Init
//@
//@
//@ Parameters :
//@         (ENTREE)  init : initialisation ou calcul (1 ou 0)
//@         (ENTREE)  nbpts : nombre de points de la FFT a traiter
//@         (ENTREE)  nbfft : nombre de FFT a traiter
//@         (ENTREE)  xr : input constitue de plusieurs tableaux
//@                 de reels contigus
//@         (ENTREE)  aux1 : tableau temporaire
//@         (ENTREE)  aux2 : tableau temporaire
//@         (ENTREE)  naux1 : taille du tableau temporaire aux1
//@         (ENTREE)  naux2 : taille du tableau temporaire aux2
//@         (ENTREE)  scale : facteur de normalisation
//@         (ENTREE)  yc : output constitue de plusieurs tableaux
//@                 de complexes contigus
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2765
AUX_SD_BAS_FFT1DirDR2C * get_AUX_SD_BAS_FFT1DirDR2C()
{
  AUX_SD_BAS_FFT1DirDR2C* t = NULL;
  if( SD_BAS_MacMALLOC( t, AUX_SD_BAS_FFT1DirDR2C, 1 ) == FALSE )
  {
    return NULL;
  }
  memset (t, 0, sizeof(AUX_SD_BAS_FFT1DirDR2C));
  return t;
}

void free_AUX_SD_BAS_FFT1DirDR2C(
		AUX_SD_BAS_FFT1DirDR2C *t
		)
{
#ifdef OPS_USE_ESSL
  if( t ) {
    if( t->aux1 )
      SD_BAS_MacFREE( t->aux1 );
    if( t->aux2 )
      SD_BAS_MacFREE( t->aux2 );
    SD_BAS_MacFREE( t );
  }
#endif
#ifdef OPS_USE_FFTW
  if( t ) {
    if( t->plan ) {
      pthread_mutex_lock( &fftw3_mutex );
      fftw_destroy_plan( t->plan );
      pthread_mutex_unlock( &fftw3_mutex );
    }
    SD_BAS_MacFREE( t );
  }
#endif
#ifdef OPS_USE_FFT_NUMREC
  if( t ) {
    SD_BAS_MacFREE( t );
  }
#endif
#ifdef OPS_USE_FFT_FFTPACK
  if( t ) {
    SD_BAS_MacFREE( t->wsave );
    SD_BAS_MacFREE( t );
  }
#endif
}

static int init_AUX_SD_BAS_FFT1DirDR2C(
  AUX_SD_BAS_FFT1DirDR2C *t,
  int* n,
  int howmany,
  double* in,
  dcmplx* out
)
{
#ifdef OPS_USE_ESSL
  int naux1;
  int naux2;

  if(*n <= 2048)
    {
      naux1 = 22000;                          // cf doc lib ESSL p. 738 730 (max dcft drcft)
      naux2 = 20000;
    }
  else
    {
      naux1 = (int)(20000 + 2.28*(*n));       // cf doc lib ESSL p. 738 730 (max dcft drcft)
      naux2 = (int)(20000 + 2.28*(*n));
    }

  if( t->naux1 != naux1 )
    {
      t->naux1 = naux1;
      if( t->aux1 )
        SD_BAS_MacFREE( t->aux1 );
      if( SD_BAS_MacMALLOC( t->aux1, double, t->naux1+SD_GES_MARGE ) == FALSE )
        return 1;
    }

  if( t->naux2 != naux2 )
    {
      t->naux2 = naux2;
      if( t->aux2 )
        SD_BAS_MacFREE( t->aux2 );
      if( SD_BAS_MacMALLOC( t->aux2, double, t->naux2+SD_GES_MARGE ) == FALSE )
        return 1;
    }

  errset(&ierno,&inoal,&inomes,&itrace,&iusadr,&irange);

  return drcft(1, in, *n, out, *n/2+1, n, howmany, 1, 1, t->aux1,
               &t->naux1, t->aux2, &t->naux2);
#endif
#ifdef OPS_USE_FFTW
  pthread_mutex_lock( &fftw3_mutex );
  if( t->plan ) {
    fftw_destroy_plan( t->plan );
    t->plan = NULL;
  }
  t->in  = in;
  t->out = out;
  // with FFTW_ESTIMATE in & out arrays are not overwritten
  t->plan = fftw_plan_many_dft_r2c(
                1, n, howmany,
                in, NULL, 1, *n,
                (fftw_complex*)out, NULL, 1, *n,
                FFTW_ESTIMATE );
  pthread_mutex_unlock( &fftw3_mutex );
  return t->plan ? 0 : 1;
#endif
#ifdef OPS_USE_FFT_FFTPACK
  if( SD_BAS_MacMALLOC( t->wsave, dcmplx, (2 * *n + 15) ) == FALSE )
    return 1;
  dfftpack_dffti (*n, (double*)t->wsave);
#endif
  return 0;
}
// ...FT-2765

t_CMN__Statut SD_BAS_FFT1DirDR2C(
        t_SD_BAS_DataThread * ctx,
        int  init,
        int  *nbpts,
        int  nbfft,
        double * xr,
        // FT-2765
        AUX_SD_BAS_FFT1DirDR2C * aux1,
        // ...FT-2765
        double * aux2,
        int naux1,
        int naux2,
        double  scale,
        dcmplx *yc
            )
{
static const char tcProcName_F[] = "SD_BAS_FFT1DirDR2C";
// Entree dans la fonction
//@ on suppose ici que les tableaux en entree on ete reorganises par permutation circulaire
//@ de [-n/2 a n/2] vers [0 n] (AC)
//@ il faut egalement ici avoir range les donnees dans un tableau de taille acceptable par
//@ ESSL cf p 720 et avoir complete par des 0 si besoin ( zero padding)
//@  appel fonction ESSL DRCFT
//@ init = 1 ou 0( pas d'initialisation centralisee ???)
//@ stride =1 ( ecart entre 2 elements)
//@ x = input constitue de plusieurs tableaux de reels contigus
//@ inc2x = offset en nb d'elements entre les debuts de chaque tableau en entree ( = n)
//@ n = taille de chaque tableau  nbpts
//@ m = nombre de  tableau  nbfft
//@ isign = signe de l'exp dans la FFT  = -1
//@ scale = facteur de normalisation (AD) ( a priori 1/2N)
//@ aux1 et aux2 = tableaux de donnees temporaires
//@ naux1 et naux2 = taille des tableaux de donnees temporaires
//@ inc2y = offset en nb d'element entre les debuts de chaque tableau de sortie ( = n/2+1
//@ y = output  constitue de plusieurs tableaux de complexes contigues
//@ on suppose ici que les tableaux en entree on ete reorganises par permutation circulaire
//@ de [-n/2 a n/2] vers [0 n]
//@ on suppose ici que les tableaux en sortie seront reorganises par l'appelant
//@ par permutation circulaire (AC) en supprimant aussi si besoin le zero padding
//@ de [0 a n/2 ] vers [-n/2 a n/2]
// on trace ici si besoin toutes les variables en sortie

  t_CMN__Statut cr_op=CMN__STATUT_OK;


    int irc = 0;


    if( init ) {
      irc = init_AUX_SD_BAS_FFT1DirDR2C( aux1, nbpts, nbfft, xr, yc );
    } else {
  #ifdef OPS_USE_ESSL
      irc = drcft(0, xr, *nbpts, yc, *nbpts/2+1, nbpts, nbfft, 1, scale, aux1->aux1,
                  &aux1->naux1, aux1->aux2, &aux1->naux2);
  #endif
  #ifdef OPS_USE_FFTW
      fftw_execute_dft_r2c( aux1->plan, xr, (fftw_complex*)yc );
      {
        int i;
        for( i = 0; i < ((*nbpts)/2+1) * nbfft; i++ ) {
          yc[i]._data._re = yc[i]._data._re * scale;
          yc[i]._data._im = yc[i]._data._im * scale;
        }
      }
  #endif
#ifdef OPS_USE_FFT_NUMREC
    if (nbfft == 1)
      {
	int i;
        for (i = 0; i < (*nbpts)/2; i++)
          {
            yc[i]._data._re = xr[2*i+0];
            yc[i]._data._im = xr[2*i+1];
          }
	fft_numrec_r2c ((double*)yc, (*nbpts), 1, &irc);

        yc[(*nbpts)/2]._data._re = yc[0]._data._im;
        yc[(*nbpts)/2]._data._im = 0;
        yc[0]._data._im = 0;

        for( i = 0; i < ((*nbpts)/2+1) * nbfft; i++ ) {
          yc[i]._data._re = + yc[i]._data._re * scale;
          yc[i]._data._im = - yc[i]._data._im * scale;
        }
      }
    else
      {
        irc = 1;
      }
#endif
#ifdef OPS_USE_FFT_FFTPACK
    {
      int i;

//    dfftpack_dffti (*nbpts, (double*)aux1->wsave);

      for (i = 0; i < *nbpts/2; i++)
        {
          yc[i]._data._re = xr[2*i+0];
          yc[i]._data._im = xr[2*i+1];
        }
      
      dfftpack_dfftf (*nbpts, (double*)yc, (double*)aux1->wsave); 
      
      for (i = 0; i < *nbpts/2+1; i++)
        { 
          yc[i]._data._re = scale * yc[i]._data._re;
          yc[i]._data._im = scale * yc[i]._data._im;
        }
      
      {
        double * c = (double*)yc;
        for (i = *nbpts; i >= 1; i--)
          c[i] = c[i-1];
        yc[0]._data._im = 0;
        yc[*nbpts/2]._data._im = 0;
      }

    }
#endif
  #ifdef ESSL_TEST
        ESSL_TEST_BEGIN( SD_BAS_FFT1DirDR2C );
        fwrite( nbpts, sizeof( *nbpts ), 1, fp );
        fwrite( &nbfft, sizeof( nbfft ), 1, fp );
        fwrite( xr, sizeof( double ), *nbpts * nbfft, fp );
        fwrite( &scale, sizeof( double ), 1, fp );
        fwrite( yc, sizeof( dcmplx ), ((*nbpts)/2+1) * nbfft, fp );
        ESSL_TEST_END;
  #endif
    }

    if( irc == 1 ) {
      cr_op = SD_BAS_LibESSL_ERR_TAILLE_FFT;
      //@ on recupere le premier message d'erreur pour l'emission d'une trace
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
                          SD_BAS_LibESSL_ERR_TAILLE_FFT,tcProcName_F,__LINE__,TAILLE_FFT_INCOMPATIBLE);
      //@ on imprime les message d'erreurs sur la sortie standard
      #ifdef DEBUG
          SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, TAILLE_FFT_INCOMPATIBLE);
      #endif
  }

    // sortie de la fonction
    return cr_op;
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FFT1InvDC2R
//@
//@ Description : transformee de fourier inverse 1D (  complexe vers reel)
//@ pour des pb d'efficacite il est souhaitable ici de regrouper le calcul de
//@ plusieurs FFT . si ce n'est pas possible utiliser l' argument d'appel Init
//@
//@
//@ Parameters :
//@         (ENTREE)  init : initialisation ou calcul (1 ou 0)
//@         (ENTREE)  nbpts : nombre de points de la FFT a traiter
//@         (en tenant compte de la symetrie)
//@         (ENTREE)  nbfft : nombre de FFT a traiter
//@         (ENTREE)  xc : input constitue de plusieurs tableaux
//@                 de complexes contigus
//@         (ENTREE)  aux1 : tableau temporaire
//@         (ENTREE)  aux2 : tableau temporaire
//@         (ENTREE)  naux1 : taille du tableau temporaire aux1
//@         (ENTREE)  naux2 : taille du tableau temporaire aux2
//@         (ENTREE)  scale : facteur de normalisation
//@         (ENTREE)  yr : output constitue de plusieurs tableaux
//@                 de complexes contigus
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2765
AUX_SD_BAS_FFT1InvDC2R * get_AUX_SD_BAS_FFT1InvDC2R()
{
  AUX_SD_BAS_FFT1InvDC2R* t = NULL;
  if( SD_BAS_MacMALLOC( t, AUX_SD_BAS_FFT1InvDC2R, 1 ) == FALSE ) {
    return NULL;
  }
  memset(t, 0, sizeof(AUX_SD_BAS_FFT1InvDC2R));
  return t;
}

static int init_AUX_SD_BAS_FFT1InvDC2R(
  AUX_SD_BAS_FFT1InvDC2R* t,
  int* n,
  int howmany,
  dcmplx* in,
  double* out
)
{
#ifdef OPS_USE_ESSL

#endif
#ifdef OPS_USE_FFTW
  pthread_mutex_lock( &fftw3_mutex );
  if( t->plan ) {
    fftw_destroy_plan( t->plan );
    t->plan = NULL;
  }
  t->in = in;
  t->out = out;
  // with FFTW_ESTIMATE in & out arrays are not overwritten
  t->plan = fftw_plan_many_dft_c2r(
                  1, n, howmany,
                  (fftw_complex*)in,  NULL, 1, *n,
                  out, NULL, 1, *n,
                  FFTW_ESTIMATE );
  pthread_mutex_unlock( &fftw3_mutex );
  return t->plan ? 0 : 1;
#endif
}

void free_AUX_SD_BAS_FFT1InvDC2R(
  AUX_SD_BAS_FFT1InvDC2R *t
) {
  if( t ) {
#ifdef OPS_USE_ESSL

#endif
#ifdef OPS_USE_FFTW
    if( t->plan ) {
      pthread_mutex_lock( &fftw3_mutex );
      fftw_destroy_plan( t->plan );
      pthread_mutex_unlock( &fftw3_mutex );
    }
#endif
    SD_BAS_MacFREE( t );
  }
}
// ...FT-2765

t_CMN__Statut SD_BAS_FFT1InvDC2R(
        t_SD_BAS_DataThread * ctx,
        int  init,
        int  *nbpts,
        int  nbfft,
        dcmplx * xc,
        // ...FT-2765
        AUX_SD_BAS_FFT1InvDC2R * aux1,
        // ...FT-2765
        double * aux2,
        int naux1,
        int naux2,
        double  scale,
        double *yr
            )
{
// Entree dans la fonction
//@ on suppose ici que les tableaux en entree on ete reorganises
//@ de [0 a n/2]  (AC) en tenant compte de la symetrie et du  zero padding
//@  appels fonction ESSL DCRFT
//@ init = 1 ou  0
//@ x = input constitue de plusieurs tableaux de complexes contigus
//@ inc2x = offset en nb d'elements entre les debuts de chaque tableau en entree ( = n)
//@ n = taille de chaque tableau  nbpts
//@ m = nombre de  tableau  nbfft
//@ isign = signe de l'exp dans la FFT  = +1
//@ scale = facteur de normalisation (AD) ( a priori 1/2N)
//@ aux1 et aux2 = tableaux de donnees temporaires
//@ inc2y = offset en nb d'element entre les debuts de chaque tableau de sortie ( = n/2+1
//@ y = output  constitue de plusieurs tableaux de reels contigues
//@ on suppose ici que les tableaux en entree on ete reorganises par permutation circulaire
//@ de [-n/2 a n/2] vers [0 n]
//@ on suppose ici que les tableaux en sortie seront reorganises par l'appelant
//@  par permutation circulaire apres avoir supprime le zeo padding
//@ de [0 a 2*n ] vers [-n a n]
// on trace ici si besoin toutes les variables en sortie
	// FT-2765

  static const char tcProcName_F[] = "SD_BAS_FFT1InvDC2R";
  t_CMN__Statut cr_op=CMN__STATUT_OK;

    int irc = 0;

    if( init )
    {
      irc = init_AUX_SD_BAS_FFT1InvDC2R( aux1, nbpts, nbfft, xc, yr );
    }
    else
    {
  #ifdef ESSL_TEST
        ESSL_TEST_BEGIN( SD_BAS_FFT1InvDC2R );
        fwrite( nbpts, sizeof( *nbpts ), 1, fp );
        fwrite( &nbfft, sizeof( nbfft ), 1, fp );
        fwrite( xc, sizeof( dcmplx ), *nbpts * nbfft, fp );
        fwrite( &scale, sizeof( double ), 1, fp );
        ESSL_TEST_END;
  #endif
  #ifdef OPS_USE_ESSL
  #endif

  #ifdef OPS_USE_FFTW
      fftw_execute_dft_c2r( aux1->plan, (fftw_complex*)xc, yr );
      {
        int i;
        for( i = 0; i < (*nbpts) * nbfft; i++ )
        {
          yr[i] = yr[i] * scale;
        }
      }
  #endif
    }

    if( irc == 1 ) {
        cr_op = SD_BAS_LibESSL_ERR_TAILLE_FFT;
        //@ on recupere le premier message d'erreur pour l'emission d'une trace
        SD_BAS_TEST_ERR_INT(
        		ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
        		SD_BAS_LibESSL_ERR_TAILLE_FFT,
        		tcProcName_F,
        		__LINE__,
        		TAILLE_FFT_INCOMPATIBLE
        		);
        //@ on imprime les message d'erreurs sur la sortie standard
        #ifdef DEBUG
            SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, TAILLE_FFT_INCOMPATIBLE);
        #endif
    }

    // sortie de la fonction
    return cr_op;
}
// ...FT-2765

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FFT1DirDC2C
//@
//@ Description : transformee de fourier directe 1D ( complexe vers complexe)
//@ pour des pb d'efficacite il est souhaitable ici de regrouper le calcul de
//@ plusieurs FFT . si ce n'est pas possible utiliser l' argument d'appel Init
//@
//@
//@ Parameters :
//@         (ENTREE)  isign : sens de la FFT (1=directe, -1=inverse)
//@         (ENTREE)  init : initialisation ou calcul (1 ou 0)
//@         (ENTREE)  nbpts : nombre de points de la FFT a traiter
//@         (ENTREE)  nbfft : nombre de FFT a traiter
//@         (ENTREE)  xr : input constitue de plusieurs tableaux
//@                 de reels contigus
//@         (ENTREE)  aux1 : tableau temporaire
//@         (ENTREE)  aux2 : tableau temporaire
//@         (ENTREE)  naux1 : taille du tableau temporaire aux1
//@         (ENTREE)  naux2 : taille du tableau temporaire aux2
//@         (ENTREE)  scale : facteur de normalisation
//@         (ENTREE)  yc : output constitue de plusieurs tableaux
//@                 de complexes contigus
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2765
AUX_SD_BAS_FFT1DirInvDC2C * get_AUX_SD_BAS_FFT1DirInvDC2C()
{
  AUX_SD_BAS_FFT1DirInvDC2C *t = NULL;
  if( SD_BAS_MacMALLOC( t, AUX_SD_BAS_FFT1DirInvDC2C, 1 ) == FALSE ) {
    return NULL;
  }
  memset(t, 0, sizeof(AUX_SD_BAS_FFT1DirInvDC2C));
  return t;
}

static int init_AUX_SD_BAS_FFT1DirInvDC2C(
  AUX_SD_BAS_FFT1DirInvDC2C *t,
  int* n,
  int howmany,
  dcmplx* in,
  dcmplx* out,
  double scale,
  int isign
)
{
#ifdef OPS_USE_ESSL
  int naux1;
  int naux2;

  if(*n <= 2048)
    {
      naux1 = 22000;
      naux2 = 20000;
    }
  else
    {
      naux1 = (int)(20000 + 2.28*(*n));       // cf doc lib ESSL p. 738 730 (max dcft drcft)
      naux2 = (int)(20000 + 2.28*(*n));
    }

  if( t->naux1 != naux1 )
    {
      t->naux1 = naux1;
      if( t->aux12 )
        SD_BAS_MacFREE( t->aux12 );
      if( SD_BAS_MacMALLOC( t->aux12, double, t->naux1+SD_GES_MARGE ) == FALSE )
        return 1;
    }

  if( t->naux2 != naux2 )
    {
      t->naux2 = naux2;
      if( t->aux22 )
        SD_BAS_MacFREE( t->aux22 );
      if( SD_BAS_MacMALLOC( t->aux22, double, t->naux2+SD_GES_MARGE ) == FALSE )
        return 1;
    }

  errset(&ierno,&inoal,&inomes,&itrace,&iusadr,&irange);

  return dcft(1, in, 1, *n, out, 1, *n, n, howmany, isign, scale,
              t->aux12, &t->naux1, t->aux22, &t->naux2);
#endif
#ifdef OPS_USE_FFTW

  isign = isign > 0 ? FFTW_FORWARD : FFTW_BACKWARD;

  pthread_mutex_lock( &fftw3_mutex );
  if( t->plan ) {
    fftw_destroy_plan( t->plan );
    t->plan = NULL;
  }
  t->in  = in;
  t->out = out;
  // with FFTW_ESTIMATE in & out arrays are not overwritten
  t->plan = fftw_plan_many_dft(
                 1, n, howmany,
                 (fftw_complex*)in,  NULL, 1, *n,
                 (fftw_complex*)out, NULL, 1, *n,
                 isign, FFTW_ESTIMATE );
  pthread_mutex_unlock( &fftw3_mutex );
  return t->plan ? 0 : 1;
#endif
#ifdef OPS_USE_FFT_FFTPACK
  if( SD_BAS_MacMALLOC( t->wsave, dcmplx, (4 * *n + 15) ) == FALSE )
    return 1;
  dfftpack_zffti (*n, (double*)t->wsave);
#endif
  return 0;
}

void free_AUX_SD_BAS_FFT1DirInvDC2C(
  AUX_SD_BAS_FFT1DirInvDC2C * t
) {
#ifdef OPS_USE_ESSL
  if( t ) {
    if( t->aux12 )
      SD_BAS_MacFREE( t->aux12 );
    if( t->aux22 )
      SD_BAS_MacFREE( t->aux22 );
    SD_BAS_MacFREE( t );
  }
#endif
#ifdef OPS_USE_FFTW
  if( t ) {
    if( t->plan ) {
      pthread_mutex_lock( &fftw3_mutex );
      fftw_destroy_plan( t->plan );
      pthread_mutex_unlock( &fftw3_mutex );
    }
    SD_BAS_MacFREE( t );
  }
#endif
#ifdef OPS_USE_FFT_NUMREC
  if( t ) {
    SD_BAS_MacFREE( t );
  }
#endif
#ifdef OPS_USE_FFT_FFTPACK
  if( t ) {
    SD_BAS_MacFREE( t->wsave );
    SD_BAS_MacFREE( t );
  }
#endif
}

// ...FT-2765

t_CMN__Statut SD_BAS_FFT1DirInvDC2C(
        t_SD_BAS_DataThread * ctx,
        int isign,
        int  init,
        int  *nbpts,
        int  nbfft,
        dcmplx * xc,
        // FT-2765
        AUX_SD_BAS_FFT1DirInvDC2C * aux1,
        // ...FT-2765
        double * aux2,
        int naux1,
        int naux2,
        double  scale,
        dcmplx *yc
            )
{
static const char tcProcName_F[] = "SD_BAS_FFT1DirInvDC2C";
// Entree dans la fonction
//@ on suppose ici que les tableaux en entree on ete reorganises par permutation circulaire
//@ de [-n/2 a n/2] vers [0 n] (AC)
//@ il faut egalement ici avoir range les donnees dans un tableau de taille acceptable par
//@ ESSL cf p 720 et avoir complete par des 0 si besoin ( zero padding)
//@  appel fonction ESSL DCFT
//@ init = 1 ou 0( pas d'initialisation centralisee ???)
//@ stride =1 ( ecart entre 2 elements)
//@ x = input constitue de plusieurs tableaux de reels contigus
//@ inc2x = offset en nb d'elements entre les debuts de chaque tableau en entree ( = n)
//@ n = taille de chaque tableau  nbpts
//@ m = nombre de  tableau  nbfft
//@ isign = signe de l'exp dans la FFT  = -1
//@ scale = facteur de normalisation (AD) ( a priori 1/2N)
//@ aux1 et aux2 = tableaux de donnees temporaires
//@ naux1 et naux2 = taille des tableaux de donnees temporaires
//@ inc2y = offset en nb d'element entre les debuts de chaque tableau de sortie ( = n/2+1
//@ y = output  constitue de plusieurs tableaux de complexes contigues
//@ on suppose ici que les tableaux en entree on ete reorganises par permutation circulaire
//@ de [-n/2 a n/2] vers [0 n]
//@ on suppose ici que les tableaux en sortie seront reorganises par l'appelant
//@ par permutation circulaire (AC) en supprimant aussi si besoin le zero padding
//@ de [0 a n/2 ] vers [-n/2 a n/2]
// on trace ici si besoin toutes les variables en sortie

  t_CMN__Statut cr_op = CMN__STATUT_OK;

    int irc;                // Return code

    irc = 0;

    if( init ) {
      irc = init_AUX_SD_BAS_FFT1DirInvDC2C(
               aux1, nbpts, nbfft, xc, yc,
               scale, isign );
    } else {
  #ifdef OPS_USE_ESSL
      irc = dcft(0, xc, 1, *nbpts, yc, 1, *nbpts, nbpts, nbfft, isign, scale, aux1->aux12,
                  &aux1->naux1, aux1->aux22, &aux1->naux2);
  #endif
  #ifdef OPS_USE_FFTW
      fftw_execute_dft( aux1->plan, (fftw_complex*)xc, (fftw_complex*)yc );
      {
        int i;
        for( i = 0; i < (*nbpts) * nbfft; i++ ) {
          yc[i]._data._re = yc[i]._data._re * scale;
          yc[i]._data._im = yc[i]._data._im * scale;
        }
      }
  #endif
#ifdef OPS_USE_FFT_NUMREC
   if (nbfft == 1)
     {
       int i;
       for( i = 0; i < (*nbpts) * nbfft; i++ ) {
         yc[i]._data._re = xc[i]._data._re;
         yc[i]._data._im = xc[i]._data._im;
       }
       fft_numrec_c2c ((double*)yc, *nbpts, -isign, &irc);
       for( i = 0; i < (*nbpts) * nbfft; i++ ) {
         yc[i]._data._re = yc[i]._data._re * scale;
         yc[i]._data._im = yc[i]._data._im * scale;
       }
     }
   else
     {
       irc = 1;
     }
#endif
#ifdef OPS_USE_FFT_FFTPACK
  {
    int i;

    for (i = 0; i < *nbpts; i++)
      yc[i] = xc[i];

    if (isign < 0)
      dfftpack_zfftb (*nbpts, (double*)yc, (double*)aux1->wsave);
    else
      dfftpack_zfftf (*nbpts, (double*)yc, (double*)aux1->wsave);
    
    for (i = 0; i < *nbpts; i++)
      { 
        yc[i]._data._re = scale * yc[i]._data._re;
        yc[i]._data._im = scale * yc[i]._data._im;
      } 

  }
#endif
  #ifdef ESSL_TEST
        ESSL_TEST_BEGIN( SD_BAS_FFT1DirInvDC2C );
        fwrite( &isign, sizeof( isign ), 1, fp );
        fwrite( nbpts, sizeof( *nbpts ), 1, fp );
        fwrite( &nbfft, sizeof( nbfft ), 1, fp );
        fwrite( xc, sizeof( dcmplx ), *nbpts * nbfft, fp );
        fwrite( &scale, sizeof( double ), 1, fp );
        fwrite( yc, sizeof( dcmplx ), *nbpts * nbfft, fp );
        ESSL_TEST_END;
  #endif
    }

    if( irc == 1 ) {
        cr_op = SD_BAS_LibESSL_ERR_TAILLE_FFT;



        //@ on recupere le premier message d'erreur pour l'emission d'une trace
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
                            SD_BAS_LibESSL_ERR_TAILLE_FFT,tcProcName_F,__LINE__,TAILLE_FFT_INCOMPATIBLE);
        //@ on imprime les message d'erreurs sur la sortie standard
        #ifdef DEBUG
            SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, TAILLE_FFT_INCOMPATIBLE);
        #endif
    }
    // sortie de la fonction
    return(cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_MoindresCarres1D
//@
//@ Description : resolution par moindres carres d'un systeme lineaire
//@ de type A X = B
//@ les lignes ont ete constitues et allouees par l'appelant ainsi que
//@ les vecteurs b et s
//@
//@ Parameters :
//@         (ENTREE)  ctx : gestion des erreurs
//@         (ENTREE)  nbeq : nombre d'equations = nombre de lignes de A
//@         (ENTREE)  ninc : nombre d'inconnues = nombre de colonnes de A
//@                         (dans la cas de l'ajustement d'un polynome de
//@                         degre N, ninc = N+1)
//@         (ENTREE)  a : input constitue d'une matrice nbeq*ninc
//@                       de double contigus ranges par colonnes
//@                       (ordre fortran)
//@         (ENTREE)  b : input constitue d'un vecteur nbeq
//@                       de double contigus
//@         (SORTIE)  s : vecteur de taille ninc contenant les valeurs
//@                       singulieres de A (alloue par l'appelant)
//@         (ENTREE)  tau : tolerance d'erreur sur l'estimation
//@         (SORTIE)  x : resultat constitue d'un vecteur de taille ninc
//@                       de double contigus
//@         (SORTIE)  rmse : RMSE de l'ajustement
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_MoindresCarres1D(
        t_SD_BAS_DataThread * ctx,
        int  nbeq,
        int  ninc,
        double * a,
        double * b,
        double * s,
        double tau,
        double * x,
        double * rmse
            )
{
// FT-2765
#ifdef OPS_USE_ESSL
// ...FT-2765
	static const char tcProcName_F[] = "SD_BAS_MoindresCarres1D";
    t_CMN__Statut cr_op=CMN__STATUT_OK;

    int i,j;                // Compteurs
    double best;

    int iopt;               // Type de calcul a realiser
    int ldab;               // Dimensions max de a et b
    int ldx;                // Dimension de x
    int nb;                 // Nombre de colonnes de b
    int m;                  // Nombre de lignes de la matrice a
    int n;                  // Nombre de colonnes de la matrice a
    double *aux=NULL;       // Tableau temporaire
    int naux;               // Taille du tableau temporaire aux

    double *a_tmp=NULL;     // Recopiage de a;
    double *b_tmp=NULL;     // Recopiage de b;


    // Gestion de erreurs
    int irc;                // Return status des fonctions ESSL
    char *msg_err;          // Message d'erreur
    int num_lig;            // Ligne d'erreur

    errset(&ierno,&inoal,&inomes,&itrace,&iusadr,&irange);

    // Initialisations pour l'algorithme
    n = ninc;
    m = nbeq;
    iopt = 12;              // Pour tout calculer
    ldab = (n > m)?n:m;     // Max des dimensions de A
    nb = 1;                 // Vecteur
    ldx = n;

    naux = ldab*ldab;
    if(naux < 6) naux =6;   // Impose par ESSL

    // Allocation de la memoire
    if(    SD_BAS_MacMALLOC(aux,double,ldab*ldab) == FALSE
        || SD_BAS_MacMALLOC(a_tmp,double,ldab*ldab) == FALSE
        || SD_BAS_MacMALLOC(b_tmp,double,ldab) == FALSE
      )
    {
        cr_op   = SD_BAS_LibESSL_ERR_ALLOCATION;
        msg_err = (char *)SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;

        goto fin;
    }



    // Copie de a et b pour le calcul final du residu
    if(a_tmp != NULL && b_tmp != NULL)
    {
        for(i=0; i<m*n; i++) a_tmp[i] = a[i];
        for(i=0; i<m; i++) b_tmp[i] = b[i];
    }


    // Appel de la fonction DGESVF
    irc = dgesvf(iopt,a,ldab,b,ldab,nb,s,m,n,aux,&naux);

    if(irc != 0)
    {
        cr_op   = SD_BAS_LibESSL_ERR_DGESVF;
        msg_err = (char *)SD_BAS_DGESVF;
        num_lig = __LINE__;

        goto fin;
    }

    // Appel de la fonction DGESVS
    dgesvs(a,ldab,b,ldab,nb,s,x,ldx,m,n,tau);


    // Calcul de la racine carree de la moyenne quadratique des residus d'ajustement
    if(a_tmp != NULL && b_tmp != NULL)
    {
        *rmse = 0;
        for(i=0; i<m; i++)
        {
            best = 0.0;
            for(j=0; j<n; j++)
            {
                best = best + a_tmp[j*m+i]*x[j];
            }

            *rmse += (best-b_tmp[i])*(best-b_tmp[i]);
        }
        *rmse = sqrt(*rmse/m);
    }

    /* recuperation des valeurs initiales */
    for(i=0; i<m*n; i++) a[i] = a_tmp[i];
    for(i=0; i<m; i++) b[i] = b_tmp[i];

    if(a_tmp) free(a_tmp);
    if(b_tmp) free(b_tmp);
    if(aux) free(aux);

fin :

    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_BASE_ESSL,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

    // sortie de la fonction
    return(cr_op);

// FT-2765
#else
    static const char tcProcName_F[] = "SD_BAS_MoindresCarres1D";
    t_CMN__Statut cr_op=CMN__STATUT_OK;

    int i,j;                // Compteurs
    double best;

    int m;
    int n;
    int maxmn;
    double rcond;
    int rank;
    double *work = NULL;
    int lwork; double dlwork;
    int info;

    double *a_tmp = NULL;
    double *b_tmp = NULL;


    int irc = 0;
    char *msg_err;
    int num_lig;

#ifdef ESSL_TEST
      ESSL_TEST_BEGIN( SD_BAS_MoindresCarres1D );
      fwrite( &nbeq, sizeof( nbeq ), 1, fp );
      fwrite( &ninc, sizeof( ninc ), 1, fp );
      fwrite( a, sizeof( double ), nbeq * ninc,fp );
      fwrite( b, sizeof( double ), nbeq, fp );
      fwrite( s, sizeof( double ), ninc, fp );
      fwrite( &tau, sizeof( double ), 1, fp );
      ESSL_TEST_END;
#endif

    m = nbeq;
    n = ninc;
    rcond = tau / 10000;

    maxmn = ( n > m ) ? n : m;

    lwork = -1; info = 0; // query LAPACK
    lapack_dgelss( nbeq, ninc, 1, NULL, maxmn, NULL, maxmn, NULL, rcond, &rank, &dlwork, lwork, &info );

    if( info != 0 ) {
      cr_op = SD_BAS_LibESSL_ERR_DGESVF;
      msg_err = (char *)SD_BAS_DGESVF;
      num_lig = __LINE__;
      goto fin;
    }


    lwork = dlwork;

    if(  SD_BAS_MacMALLOC( work,  double, lwork         ) == FALSE
      || SD_BAS_MacMALLOC( a_tmp, double, m * n         ) == FALSE
      || SD_BAS_MacMALLOC( b_tmp, double, maxmn         ) == FALSE
      ){
      cr_op   = SD_BAS_LibESSL_ERR_ALLOCATION;
      msg_err = (char *)SD_BAS_MSG_ALLOC;
      num_lig = __LINE__;
      goto fin;
    }

    for( i = 0; i < m * n;  i++ ) { a_tmp[i] = 0; }
    for( i = 0; i < maxmn ; i++ ) { b_tmp[i] = 0; }

    for( i = 0; i < m * n; i++ ) {
      a_tmp[ i ] = a[ i ];
    }
    for( i = 0; i < m; i++ ) {
      b_tmp[ i ] = b[ i ];
    }

    lapack_dgelss( nbeq, ninc, 1, a_tmp, m, b_tmp, maxmn, s, rcond, &rank, work, lwork, &info );

    if( info != 0 ) {
      cr_op = SD_BAS_LibESSL_ERR_DGESVF;
      msg_err = (char *)SD_BAS_DGESVF;
      num_lig = __LINE__;
      goto fin;
    }

    for( i = 0; i < n; i++ ) { x[i] = b_tmp[i]; }

    *rmse = 0;
    for( i = 0; i < m; i++ ) {
      best = 0.0;
      for( j = 0; j < n; j++ ) {
        best = best + a[ j * m + i ] * x[ j ];
      }
      *rmse += ( best - b[ i ] ) * ( best - b[ i ] );
    }
    *rmse = sqrt( *rmse / m );

    SD_BAS_MacFREE( work );
    SD_BAS_MacFREE( a_tmp );
    SD_BAS_MacFREE( b_tmp );



fin :

    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK ) {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_BASE_ESSL,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }


  return cr_op;
#endif
// ...FT-2765

}



//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpSpline1D
//@
//@ Description : interpolation spline d'une serie de valeurs 1D triees
//@ par ordre croissant
//@ cette fonction doit etre appelee une premiere fois avec init=0
//@ puis  elle est appelee pour des entrees similaires
//@ avec init=4 pour reutiliser c (AC)
//@
//@ Parameters :
//@         (ENTREE)  init : initialisation ou calcul (0 ou 4)
//@         (ENTREE)  ne : nombre de points en entree
//@         (ENTREE)  xe : abscisse des points en entree
//@         (ENTREE)  ye : ordonnees des points en entree
//@         (ENTREE/SORTIE)  c : tableau des coef precalcules de taille 4*ne
//@         (ENTREE)  ns : nombre de points en sortie
//@         (ENTREE)  xs : abscisse des points en sortie
//@         (SORTIE)  ys : ordonnees des points en sortie
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2765
t_CMN__Statut SD_BAS_InterpSpline1D(
        int  init,
        int  ne,
        double * xe,
        double * ye,
        // FT-2765
        AUX_SD_BAS_InterpSpline1D * c,
        // ...FT-2765
        int  ns,
        double * xs,
        double * ys
            )
{
	  static const char tcProcName_F[] = "SD_BAS_InterpSpline1D";
	  t_CMN__Statut cr_op=CMN__STATUT_OK;

#ifdef USE_SPLINE_NGMATH

  const double sigma = 1.0;

#endif

  if( init <= 3 ) {

#ifdef OPS_USE_ESSL

		  int initialisation;
	    if( c->y2 ) {
	      SD_BAS_MacFREE( c->y2 );
	    }
	    if( SD_BAS_MacMALLOC( c->y2, double, 4*ne ) == FALSE ) {
	      cr_op   = SD_BAS_LibESSL_ERR_ALLOCATION;
	      return cr_op;
	    }
	    errset(&ierno,&inoal,&inomes,&itrace,&iusadr,&irange);

	    initialisation = init;
	    dcsint(xe,ye,c->y2,ne,&initialisation,xs,ys,ns);


#else
#ifdef USE_SPLINE_LITE

    if( c->as ) { 
      SD_BAS_MacFREE( c->as );
    }

    if( SD_BAS_MacMALLOC( c->as, double, 5 * ne ) == FALSE ) {
      cr_op = SD_BAS_LibESSL_ERR_ALLOCATION;
      return cr_op;
    }
    spline_spline( xe, ye, NULL, NULL, c->as, ne, 0, 1 );

#endif

#ifdef USE_SPLINE_NGMATH

    double *temp;
    int ierr; 
    
    if( c->yp ) {
      SD_BAS_MacFREE( c->yp );
    }
    
    if( ( SD_BAS_MacMALLOC( c->yp, double, ne ) == FALSE )
     || ( SD_BAS_MacMALLOC( temp, double, ne )  == FALSE ) 
    ) {
      cr_op   = SD_BAS_LibESSL_ERR_ALLOCATION;
      return cr_op;
    }
    c->lintrvl = 1;
    
    ngmath_dcurv1( ne, xe, ye, 0., 0., 3, c->yp, temp, sigma, &ierr );
    
    SD_BAS_MacFREE( temp );

#endif

#ifdef USE_SPLINE_NUMREC

	    if( c->y2 ) {
	      SD_BAS_MacFREE( c->y2 );
	    }
	    if( SD_BAS_MacMALLOC( c->y2, double, ne ) == FALSE ) {
	      cr_op   = SD_BAS_LibESSL_ERR_ALLOCATION;
	      return cr_op;
	    }
	    numrec_spline( xe - 1, ye - 1, ne, 1e99, 1e99, c->y2 - 1 );

#endif
#endif

	  } else {

#ifdef OPS_USE_ESSL

	    int initialisation;
	    initialisation = init;
	    dcsint(xe,ye,c->y2,ne,&initialisation,xs,ys,ns);

#else

#ifdef ESSL_TEST
	      ESSL_TEST_BEGIN( SD_BAS_InterpSpline1D );
	      fwrite( &ne, sizeof( ne ), 1, fp );
	      fwrite( xe, sizeof( double ), ne, fp );
	      fwrite( ye, sizeof( double ), ne, fp );
	      fwrite( &ns, sizeof( ns ), 1, fp );
	      fwrite( xs, sizeof( double ), ns, fp );
	      ESSL_TEST_END;
#endif
    
#ifdef USE_SPLINE_LITE
    spline_spline( xe, ye, xs, ys, c->as, ne, ns, 0 );
#endif

#ifdef USE_SPLINE_NGMATH
    {
      int i;
      for( i = 0; i < ns; i++ ) {
        ys[i] = ngmath_dcurv2( xs[i], ne, xe, ye, c->yp, sigma, &c->lintrvl );
      }  
    }
#endif

#ifdef USE_SPLINE_NUMREC
	    numrec_splintm( xe - 1, ye - 1, c->y2 - 1, ne, xs - 1, ys - 1, ns );
#endif

#endif
	  }

	  return cr_op;
}

AUX_SD_BAS_InterpSpline1D * get_AUX_SD_BAS_InterpSpline1D()
{
	  AUX_SD_BAS_InterpSpline1D *t = NULL;
	  if( SD_BAS_MacMALLOC( t, AUX_SD_BAS_InterpSpline1D, 1 ) == FALSE ) {
	    return NULL;
	  }
#ifdef OPS_USE_ESSL
	  memset (t, 0, sizeof(AUX_SD_BAS_InterpSpline1D));
#else

#ifdef USE_SPLINE_LITE
  t->as = NULL;
#endif
#ifdef USE_SPLINE_NGMATH
  t->lintrvl = 1;
  t->yp = NULL;
#endif
#ifdef USE_SPLINE_NUMREC
	  t->y2 = NULL;
#endif
#endif

	  return t;
}

void free_AUX_SD_BAS_InterpSpline1D( AUX_SD_BAS_InterpSpline1D * t )
{
	  if( t ) {
#ifdef OPS_USE_ESSL
	    if( t->y2 ) {
	      SD_BAS_MacFREE( t->y2 );
	    }
#else
#ifdef USE_SPLINE_LITE
    if( t->as ) {
      SD_BAS_MacFREE( t->as );
    }
#endif
#ifdef USE_SPLINE_NGMATH
    if( t->yp ) {
      SD_BAS_MacFREE( t->yp );
    }
#endif
#ifdef USE_SPLINE_NUMREC
	    if( t->y2 ) {
	      SD_BAS_MacFREE( t->y2 );
	    }
#endif
#endif
    SD_BAS_MacFREE( t );
  }

}
 // ...FT-2765

	//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpSpline2D
//@
//@ Description : interpolation spline d'une serie de valeurs 2D triees
//@ par ordre croissant pour un point
//@
//@ Parameters :
//@         (ENTREE)  ne : nombre de points de x1 et de x2 en entree
//@         (ENTREE)  xe1 : abscisse des points en entree
//@         (ENTREE)  xe2 : abscisse des points en entree 
//@         (ENTREE)  ye : ordonnees des points en entree ranges par colonne (ne*ne)
//@         (ENTREE)  nc : nombre de points de c
//@         (ENTREE/SORTIE)  c : tableau des coef precalcules de taille (11ne + 3)
//@         (ENTREE)  xs1 : abscisse du point en sortie
//@         (ENTREE)  xs2 : abscisse du point en sortie
//@         (SORTIE)  ys : ordonnees du point en sortie
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InterpSpline2D(
        int  ne,
        double * xe1,
        double * xe2,
        double * ye,
        int  nc,
        double * c,
        double * xs1,
        double * xs2,
        double * ys
            )
{
// FT-2765
#ifdef OPS_USE_ESSL
// ...FT-2765

	// Entree dans la fonction
//@ 1- appel fonction ESSL DCSINT
//@ x tableau initial  des abcisses trie = xe
//@ y tableau initial  des ordonnees trie = ye
//@ n = ne
//@ init = condition d'initialisation ( 0 ou 4)
//@
//@ c tableau des coef interne associees a x , y
//@ t = xs
//@ m = ns
//@ s = ys

    t_CMN__Statut cr_op=CMN__STATUT_OK;

    errset(&ierno,&inoal,&inomes,&itrace,&iusadr,&irange);

    dcsin2(xe1,xe2,ye,ne,ne,ne,xs1,xs2,1,1,ys,1,c,&nc);


// sortie de la fonction
return(cr_op);
#else

  static char tcProcName_F[] = "SD_BAS_InterpSpline2D";
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  int ierr;
  double *zp = NULL;
  double *temp = NULL;
  int lintrvlx, lintrvly;

#ifdef ESSL_TEST
      ESSL_TEST_BEGIN( SD_BAS_InterpSpline2D );
      fwrite( &ne, sizeof( ne ), 1, fp );
      fwrite( xe1, sizeof( double ), ne, fp );
      fwrite( xe2, sizeof( double ), ne, fp );
      fwrite( xe2, sizeof( double ), ne * ne, fp );
      fwrite( &nc, sizeof( nc ), 1, fp );
      fwrite( xs1, sizeof( double ), 1, fp );
      fwrite( xs2, sizeof( double ), 1, fp );
      ESSL_TEST_END;
#endif

  lintrvlx = 1;
  lintrvly = 1;


  if(
     ( SD_BAS_MacMALLOC( zp, double, 3 * ne * ne ) == FALSE )
  || ( SD_BAS_MacMALLOC( temp, double, 3 * ne ) == FALSE )
  ) {
    cr_op = SD_BAS_LibESSL_ERR_ALLOCATION;
    return cr_op;
  }

  // FT-2765
  ngmath_dsurf1(
    ne, ne, xe1, xe2, ye, ne,
    NULL, NULL, NULL, NULL, 0., 0., 0., 0.,
    255, zp, temp, 1., &ierr
  );

  *ys = ngmath_dsurf2(
    *xs1, *xs2, ne, ne, xe1, xe2, ye, ne, zp, 1.,
    &lintrvlx, &lintrvly
  );
  // ...FT-2765

  SD_BAS_MacFREE( zp );
  SD_BAS_MacFREE( temp );



  // sortie de la fonction
  return cr_op;
#endif
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpLineaire1D
//@
//@ Description : interpolation lineaire d'une serie de valeurs 1D triees
//@ par ordre croissant
//@
//@ Parameters :
//@         (ENTREE)  ne : nombre de points en entree
//@         (ENTREE)  xe : abscisse des points en entree
//@         (ENTREE)  ye : ordonnees des points en entree
//@         (ENTREE/SORTIE)  auxi : tableau de travail
//@         (ENTREE)  ns : nombre de points en sortie
//@         (ENTREE)  xs : abscisse des points en sortie
//@         (SORTIE)  ys : ordonnees des points en sortie
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
// FT-2765
t_CMN__Statut SD_BAS_InterpLineaire1D(
        int  ne,
        double * xe,
        double * ye,
        AUX_SD_BAS_InterpLineaire1D * auxi,
        int  ns,
        double * xs,
        double * ys
            )
{
    static const char tcProcName_F[] = "SD_BAS_InterpLineaire1D";

#ifdef OPS_USE_ESSL
// Entree dans la fonction
//@ 1- appel fonction ESSL DTPINT
//@ x tableau initial  des abcisses trie = xe
//@ y tableau initial  des ordonnees trie = ye
//@ n = ne
//@ nint = 2
//@ naux = 2 + ns
//@ aux = auxi
//@ t = xs
//@ m = ns
//@ s = ys

    t_CMN__Statut cr_op=CMN__STATUT_OK;

    int nint = 2;       // Nombre de coefficients du polynome d'interpolation
    int naux = 2+ns;

    if( auxi->auxi ) {
      SD_BAS_MacFREE( auxi->auxi );
    }
    SD_BAS_MacMALLOC( auxi->auxi, double, naux );

    errset(&ierno,&inoal,&inomes,&itrace,&iusadr,&irange);

    // Appel de la fonction DTPINT
    dtpint(xe,ye,ne,nint,xs,ys,ns,auxi->auxi,&naux);

#else
  t_CMN__Statut cr_op = CMN__STATUT_OK;
  int i;

#ifdef ESSL_TEST
      ESSL_TEST_BEGIN( SD_BAS_InterpLineaire1D );
      fwrite( &ne, sizeof( ne ), 1, fp );
      fwrite( xe, sizeof( double ), ne, fp );
      fwrite( ye, sizeof( double ), ne, fp );
      fwrite( &ns, sizeof( ns ), 1, fp );
      fwrite( xs, sizeof( double ), ns, fp );
      ESSL_TEST_END;
#endif

  if( SD_BAS_MacMALLOC( auxi->yp, double, ne ) == FALSE ) {
    cr_op   = SD_BAS_LibESSL_ERR_ALLOCATION;
    return cr_op;
  }

  for( i = 1; i < ne; i++ ) {
    auxi->yp[i] = ( ye[i] - ye[i-1] ) / ( xe[i] - xe[i-1] );
  }
  auxi->yp[0]  = auxi->yp[1];


  {
    int is,ie;
    ie = 0;
    for( is = 0; is < ns; is++ ) {
      while( ( xs[is] > xe[ie] ) && ( ie < ne ) ) {
        ie++;
      }
      if( ie < ne ) {
        ys[is] = ye[ie] + auxi->yp[ie] * ( xs[is] - xe[ie] );
      } else {
        ys[is] = ye[ne-1] + auxi->yp[ne-1] * ( xs[is] - xe[ne-1] );
      }
    }
  }

  
  return cr_op;
#endif
}

AUX_SD_BAS_InterpLineaire1D * get_AUX_SD_BAS_InterpLineaire1D()
{
  AUX_SD_BAS_InterpLineaire1D *t = NULL;
  if( SD_BAS_MacMALLOC( t, AUX_SD_BAS_InterpLineaire1D, 1 ) == FALSE )
  {
    return NULL;
  }
  memset (t, 0, sizeof(AUX_SD_BAS_InterpLineaire1D));
  return t;
}

void free_AUX_SD_BAS_InterpLineaire1D( AUX_SD_BAS_InterpLineaire1D * t )
{
  if( t )
  {
#ifdef OPS_USE_ESSL
    if( t->auxi ) {
      SD_BAS_MacFREE( t->auxi );
    }
#else
    if( t->yp ) {
      SD_BAS_MacFREE( t->yp );
    }
#endif
    SD_BAS_MacFREE( t );
  }
}
// FT-2765

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpBiLineaire2D
//@
//@ Description : interpolation bilineaire d'une serie de valeurs 2D triees
//@ par ordre croissant
//@
//@ ATTENTION !! si les xs et ys ne sont pas compris dans les intervalles
//@ [xemin,xemax] et [yemin,yemax] les zs correspondants sont extrapoles
//@
//@ Parameters :
//@         (ENTREE)  init : initialisation ou calcul (1 ou 0)
//@         (ENTREE)  nxe : nombre de colonnes de l'image de ref
//@         (ENTREE)  nye : nombre de lignes de l'image de ref
//@         (ENTREE)  xe : coord x pour l'image de ref( trie)
//@         (ENTREE)  ye : coord y pour l'image de ref ( trie)
//@         (ENTREE)  ze : radiometrie de l'imag de ref : tableau (nxe*nye) range par
//@                         lignes (ligne 1,ligne 2, ... , ligne nye)
//@         (ENTREE)  nxs : nombre de colonnes de l'imagette a reechantillonner (nblig, nbcol)
//@         (ENTREE)  nys : nombre de lignes de l'imagette a reechantillonner (nblig, nbcol)
//@         (ENTREE)  xs : coord x pour l'imagette a reechatillonner( trie)
//@         (ENTREE)  ys : coord y pour l'imagette a reechatillonner( trie)
//@         (ENTREE/sortie)  ratio : ratio precalcules taille = 2*(nxs+nys)
//@                                     (pour t et u et les indices)
//@         (SORTIE)  zs : radiometrie de l'imag reechantillonnee : tableau (nxs*nys)
//@
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InterpLineaire2D(
        t_SD_BAS_DataThread * ctx,
        int init,
        int   nxe,
        int   nye,
        double * xe,
        double * ye,
        double * ze,
        int  nxs,
        int  nys,
        double * xs,
        double * ys,
        double * zs,
        double *ratio
            )
{
// Entree dans la fonction
    static const char tcProcName_F[] = "SD_BAS_InterpLineaire2D";
    char *msg_err;          // Message d'erreur
    int num_lig;            // Ligne d'erreur

// faire une interpolation 2D optimisee en precalculant les ratio
// si init = 0
// puis en recherchant la place de chaque point et en appliquant les
// ratio precedents
// utiliser un code inspire de polder !!!
// pour la theorie elle est simple cf annexe 7 DA7

    t_CMN__Statut cr_op=CMN__STATUT_OK;

    int k,l;    // Compteurs de boucle
    int i,j;    // Indice temporaires
    int ctxe;   // Compteur sur les indices des xe
    int ctye;   // Compteur sur les indices des ye

    double f1,f2,f3,f4; // Doubles temporaires
    double t,u;         // Doubles temporaires


     if (nxe <= 1 || nye <= 1 )
    {
         msg_err = (char *)SD_BAS_INTERP2D;
        num_lig = __LINE__;
	cr_op=SD_BAS_ERR_BASE_ESSL;

        goto fin;
    }

    // Initialisation si INIT != 0
    if(init != 0)
    {
        // Calcul des ratios
        // Organisation du tableau ratio
        // 1er vecteur de taille nxs  : indice de xeinf(xs) dans xe
        // 2eme vecteur de taille nxs  : ratio t(xs) (cf. spec)
        // 3eme vecteur de taille nys  : indice de yeinf(xs) dans xe
        // 4eme vecteur de taille nys  : ratio u(xs) (cf. spec)


        // Pour chaque xs on cherche xeinf et xesup, les xe immediatement
        // inferieur et superieur et l'indice correspondant a xeinf

        // Pour chaque ys on cherche yeinf et yesup, les ye immediatement
        // inferieur et superieur et l'indice correspondant a yeinf

        // ATTENTION : les xe et ye doivent etre tries !

        ctxe=0;
        ctye=0;

        for(k=0; k<nxs; k++)
        {
            // Recherche du xeinf. Comme les tableaux sont ordonnes, on part
            // du precedent
            while(ctxe < nxe-2 && xe[ctxe] < xs[k])
            {
                ctxe++;
            }
            // On peut avoir depasse de 1
            if(ctxe > 0 && xe[ctxe] > xs[k]) ctxe--;


            // Calcul des ratios
            ratio[k] = ctxe;
            ratio[nxs+k] = (xs[k] - xe[ctxe]) / (xe[ctxe+1] - xe[ctxe]);
        }


        for(l=0; l<nys; l++)
        {
            // Recherche du yeinf. Comme les tableaux sont ordonnes, on part
            // du precedent
            while(ctye < nye -2 && ye[ctye] < ys[l])
            {
                ctye++;
            }
            // On peut avoir depasse de 1
            if(ctye > 0 && ye[ctye] > ys[l]) ctye--;


            // Calcul des ratios
            ratio[2*nxs + l]  = ctye;
            ratio[2*nxs + nys + l] = (ys[l] - ye[ctye]) / (ye[ctye+1] - ye[ctye]);
        }
    }


    // Calcul de l'interpolation si INIT = 0
    else
    {
        for(l=0; l<nys; l++)
        {
            for(k=0; k<nxs; k++)
            {
                // Recuperation de l'indice min xe
                i = (int)ratio[k];
                // Recuperation de l'indice min ye
                j = (int)ratio[2*nxs + l];
                // Recuperation du ratio t
                t = ratio[nxs+k];
                // Recuperation du ratio u
                u = ratio[2*nxs + nys + l];

                // Calcul des fonctions intermediaires
                f1 = ze[j*nxe + i];
                f2 = ze[j*nxe + i + 1];
                f3 = ze[(j + 1)*nxe + i + 1];
                f4 = ze[(j + 1)*nxe + i];

                // Calcul de la valeur interpolee
                zs[l*nxs + k] = (1-t)*(1-u)*f1 + t*(1-u)*f2 + t*u*f3 + (1-t)*u*f4;
            }
        }
    }

fin :

    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_BASE_ESSL,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err);

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

return(cr_op);

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ValVectPropre
//@
//@ Description : calcul des valeurs propres et des vecteurs propres
//@ d'une matrice carree
//@
//@
//@ Parameters :
//@         (ENTREE)  nbpts : nombre de pts
//@         (ENTREE)  a : matrice ( overwritten ESSL & LAPACK ) // FT-2765
//@         (SORTIE)  auxv : tableau temporaire
//@         (SORTIE) valpropre : valeurs propres
//@         (SORTIE) vecpropre : vecteurs propres
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ValVectPropre(
        t_SD_BAS_DataThread * ctx,
        int   nbpts,
        double * a,
        double * auxv,
        dcmplx * valpropre,
        dcmplx * vecpropre
        )
{
static const char tcProcName_F[] = "SD_BAS_ValVectPropre";
t_CMN__Statut cr_op=CMN__STATUT_OK;

#ifdef OPS_USE_ESSL
//@ 1- appel fonction ESSL DGEEV
//@ iopt = 1
//@ select = dummy
//@ a = matrice
//@ lda = nbpts
//@ ldz = nbpts
//@ n = nbpts
//@ aux = auxv
//@ naux = 2*nbpts
//@ w = valpropre
//@ ldz = nbpts
//@ z = vecpropre

    int irc;            // Status de retour

    int iopt;
    int lda;            // Taille de la matrice a;
    int ldz;            // Nombre mas de valeurs propres
    int select;         // Pas utilise si iopt = 1
    int naux;           // Taille de l'espace tampon

    errset(&ierno,&inoal,&inomes,&itrace,&iusadr,&irange);

    // Initialisations relatives a DGEEV
    iopt = 1;           // Pour calculer les valeurs propres et vecteur propres
    lda = nbpts;
    ldz = nbpts;
    naux = 2*nbpts;     // Cf recommandations ESSL

    // Appel de la fonction ESSL DGEEV
    irc = dgeev(iopt,a,lda,valpropre,vecpropre,ldz,&select,nbpts,auxv,&naux);

    // En cas d'erreur de calcul dans la fonction DGEEV
    if(irc != 0)
    {
        cr_op = CMN__STATUT_NOK;

        //@ on recupere le premier message d'erreur pour l'emission d'une trace
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
                            SD_BAS_LibESSL_ERR_CONVERG,tcProcName_F,__LINE__,ERREUR_CONVERGENCE);
        //@ on imprime les message d'erreurs sur la sortie standard
        #ifdef DEBUG
            SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERREUR_CONVERGENCE);
        #endif

    }


// sortie de la fonction
return(cr_op);
#else
 int irc;            // Status de retour
 irc = 0;
 int n;
 int lda;
 double *wr = NULL, *wi = NULL; // real & imag parts of eigenvalues
 double *vr = NULL;             // eigenvectors ( to be recombined )
 int lwork;
 double *work = NULL;
 double dlwork;
 int ldvr;
 int info;
 int i,j;

 n = nbpts;
 lda = nbpts;
 ldvr = n;

#ifdef ESSL_TEST
     ESSL_TEST_BEGIN( SD_BAS_ValVectPropre );
     fwrite( &nbpts, sizeof( nbpts ), 1, fp );
     fwrite( a, sizeof( double ), nbpts * nbpts, fp );
     ESSL_TEST_END;
#endif
  lwork = -1; info = 0; // query LAPACK
  lapack_dgeev( "N", "V", n, a, lda, wr, wi, NULL, 1, vr, ldvr, &dlwork, lwork, &info );
  lwork = dlwork;

 if(  SD_BAS_MacMALLOC( work,  double, lwork ) == FALSE
   || SD_BAS_MacMALLOC( vr,    double, n * n ) == FALSE
   || SD_BAS_MacMALLOC( wr,    double, n     ) == FALSE
   || SD_BAS_MacMALLOC( wi,    double, n     ) == FALSE
   ) {
     cr_op   = SD_BAS_LibESSL_ERR_ALLOCATION;
     SD_BAS_TEST_ERR_INT(
       ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
       SD_BAS_LibESSL_ERR_ALLOCATION,tcProcName_F,
       __LINE__,(char *)SD_BAS_MSG_ALLOC
     );
     return cr_op;
 }

 irc = 0;
 lapack_dgeev( "N", "V", n, a, lda, wr, wi, NULL, 1, vr, ldvr, work, lwork, &info );

  
 if( info != 0 ) {
   irc = 1;
 }

 for( i = 0; i < n; i++ ) {
   valpropre[i]._data._re = wr[i];
   valpropre[i]._data._im = wi[i];
 }

 for( i = 0; i < n; i++ ) {
   if( wi[i] != (double)0 ) {
     for( j = 0; j < n; j++ ) {
       vecpropre[ n * i         + j ]._data._re = + vr[ n * i         + j ];
       vecpropre[ n * i         + j ]._data._im = + vr[ n * ( i + 1 ) + j ];
       vecpropre[ n * ( i + 1 ) + j ]._data._re = + vr[ n * i         + j ];
       vecpropre[ n * ( i + 1 ) + j ]._data._im = - vr[ n * ( i + 1 ) + j ];
     }
     i++;
   } else {
     for( j = 0; j < n; j++ ) {
       vecpropre[ n * i + j ]._data._re = + vr[ n * i + j ];
       vecpropre[ n * i + j ]._data._im = 0;
     }
   }
 }

 SD_BAS_MacFREE( work );
 SD_BAS_MacFREE( vr );
 SD_BAS_MacFREE( wr );
 SD_BAS_MacFREE( wi );



fin:
 if( irc != 0 ) {
     cr_op = CMN__STATUT_NOK;

     //@ on recupere le premier message d'erreur pour l'emission d'une trace
     SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
                         SD_BAS_LibESSL_ERR_CONVERG,tcProcName_F,__LINE__,ERREUR_CONVERGENCE);
     //@ on imprime les message d'erreurs sur la sortie standard
     #ifdef DEBUG
         SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERREUR_CONVERGENCE);
     #endif
 }

 return cr_op;
#endif
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PreparationFFTR
//@
//@ Description : Preparation d'un signal reel en vue de l'injecter dans
//@               l'un des algorithmes de FFT precedents
//@
//@
//@ Parameters :
//@         (ENTREE)  x : signal a preparer
//@         (ENTREE)  nfirst : position du premier point de x dans le spectre
//@                             originel centre. nfirst min vaut 1
//@         (ENTREE)  nlast : position du dernier point de x dans le spectre
//@                             originel centre. nlast min vaut 1
//@         (SORTIE)  y : signal prepare, alloue par l'appelant
//@         (ENTREE)  nFFT : taille de ny
//@         (ENTREE)  xpadde : tableau temporaire de taille ny pour le calcul
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_PreparationFFTR(
        t_SD_BAS_DataThread * ctx,
        double *x,
        int nfirst,
        int nlast,
        double *y,
        int nFFT
)
{
    static const char tcProcName_F[] = "SD_BAS_PreparationFFTR";
    t_CMN__Statut cr_op=CMN__STATUT_OK;

    int nx = nlast - nfirst + 1;    // Nb de points du signal
    int nzg;                        // Nombre de zeros a ajouter a gauche
    int nf;
    int milieu=nFFT/2;              // Position du milieu du signal
    double *pt_x=NULL;              // Pointeur sur x
    double *pt_y=NULL;              // Pointeur sur double


    // Gestion des erreurs
    if(nx > nFFT)
    {
        cr_op = SD_BAS_LibESSL_ERR_NFIRST;
        //@ on recupere le premier message d'erreur pour l'emission d'une trace
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
                            SD_BAS_LibESSL_ERR_TAILLE_FFT,tcProcName_F,__LINE__,ERREUR_NFIRST)
        //@ on imprime les message d'erreurs sur la sortie standard
        #ifdef DEBUG
            SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERREUR_NFIRST);
        #endif
    }


    // Nombre de zeros a ajouter a gauche
    nzg = (nFFT - nx + 1)/2;

    // Point du signal qui se retrouvera etre le premier point du signal padde
    nf = nFFT/2 - nzg;



    // nFFT est TOUJOURS PAIR
    //=========================================================================

    //@ On range d'abord les elements correspondant aux frequences positives
    pt_y = y;
    pt_x = x + nf + nfirst - 1;

    while(pt_y < y + nx - nf)
    {
        *pt_y = *pt_x;
        pt_y++;
        pt_x++;
    }

    //@ Les zeros
    while(pt_y < y - nf + nFFT )
    {
        *pt_y = 0.0;
        pt_y++;
    }


    //@ Puis ceux correspondant aux frequences negatives
    pt_x = x + nfirst - 1;
    while(pt_y < y + nFFT)
    {
        *pt_y = *pt_x;
        pt_y++;
        pt_x++;
    }


return cr_op;
}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PreparationFFTC
//@
//@ Description : Preparation d'un signal COMPLEXE en vue de l'injecter dans
//@               l'un des algorithmes de FFT precedents
//@
//@
//@ Parameters :
//@         (ENTREE)  x : signal a preparer
//@         (ENTREE)  nfirst : position du premier point de x dans le spectre
//@                             originel centre. nfirst min vaut 1
//@         (ENTREE)  nlast : position du dernier point de x dans le spectre
//@                             originel centre. nlast min vaut 1
//@         (SORTIE)  y : signal prepare, alloue par l'appelant
//@         (ENTREE)  nFFT : taille de ny
//@         (ENTREE)  xpadde : tableau temporaire de taille ny pour le calcul
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_PreparationFFTC(
        t_SD_BAS_DataThread * ctx,
        dcmplx *x,
        int nfirst,
        int nlast,
        dcmplx *y,
        int nFFT
)
{
    static const char tcProcName_F[] = "SD_BAS_PreparationFFTR";
    t_CMN__Statut cr_op=CMN__STATUT_OK;

    int nx = nlast - nfirst + 1;
    int nzg;                        // Nombre de zeros a ajouter a gauche
    int nf;
    int milieu=nFFT/2;  // Position du milieu du signal
    dcmplx *pt_x=NULL;       // Pointeur sur x
    dcmplx *pt_y=NULL;       // Pointeur sur double




    // Gestion des erreurs
    if(nx > nFFT)
    {
        cr_op = SD_BAS_LibESSL_ERR_NFIRST;
        //@ on recupere le premier message d'erreur pour l'emission d'une trace
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,SD_BAS_ERR_SET_ESSL,
                            SD_BAS_LibESSL_ERR_TAILLE_FFT,tcProcName_F,__LINE__,ERREUR_NFIRST)
        //@ on imprime les message d'erreurs sur la sortie standard
        #ifdef DEBUG
            SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERREUR_NFIRST);
        #endif
    }


    // Nombre de zeros a ajouter a gauche
    nzg = (nFFT - nx + 1)/2;

    // Point du signal qui se retrouvera etre le premier point du signal padde
    nf = nFFT/2 - nzg;



    // nFFT est TOUJOURS PAIR
    //=========================================================================

    //@ On range d'abord les elements correspondant aux frequences positives
    pt_y = y;
    pt_x = x + nf + nfirst - 1;

    while(pt_y < y + nx - nf)
    {
        RE(*pt_y) = RE(*pt_x);
        IM(*pt_y) = IM(*pt_x);
        pt_y++;
        pt_x++;
    }

    //@ Les zeros
    while(pt_y < y - nf + nFFT )
    {
        RE(*pt_y) = 0.0;
        IM(*pt_y) = 0.0;
        pt_y++;
    }


    //@ Puis ceux correspondant aux frequences negatives
    pt_x = x + nfirst - 1;
    while(pt_y < y + nFFT)
    {
        RE(*pt_y) = RE(*pt_x);
        IM(*pt_y) = IM(*pt_x);
        pt_y++;
        pt_x++;
    }



return cr_op;
}




//======================================================================
//                  FONCTIONS INTERNES
//======================================================================
