//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_OperateurBase.c,v 1.3 2011/11/25 10:17:45 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_OperateurBase
//@
//@ Comment :
//@    Code file for the  SD_BAS_OperateurBase class which implements
//@    specific mathematical methods   :
//@ - Plank functions.
//@ - .....
//@ these methods are hand made in order to facilitate the
//@ portabilite of the software
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>
#include <math.h>

#include "SD_BAS_OperateurBase.h"
/* Declaration des fonctions importees */

/*====================================================================*/
// Local Constant Declaration
/*====================================================================*/

/*====================================================================*/
// Local types Declaration
/*====================================================================*/

//======================================================================
// Internal fonctions Declaration
//======================================================================
#define max(x,y)        ((x<=y) ? y : x)
#define min(x,y)        ((x>=y) ? y : x)

const double INVERSERACINEPI=0.56418958354775628694807945156077;


//======================================================================
// Global variables Definition
//======================================================================



//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_Planck
//@
//@ Description : Calcule la fonction de planck pour  une temperature T
//@ et un nombre d'onde nu
//@
//@
//@ Parameters :
//@         (ENTREE)  T : temperature du corps noir ( oK)
//@         (ENTREE)  nu: nombre d'onde ( m-1)
//@         (ENTREE)  IDefPlkPK1: Precalcul de 2*h*c*c
//@         (ENTREE)  IDefPlkPK2: Precalcul de h*c/k
//@
//@ Return Status :
//@         B : luminance ( w/(m2 sr m-1)
//@
//@ ------------------------------------------------------------------*/
double SD_BAS_Planck (
    double const T,     /* temperature */
    double const nu,    /* nombre d'onde */
    double IDefPlkPK1,  /* Precalcul de 2*h*c*c */
    double IDefPlkPK2   /* Precalcul de h*c/k */
    )
{
//@  cf annexe 7 du DA7 et P 55 du DA 3
//@      on prevoit ici de mettre en dur les constantes pour des pb de performances
//@ h = 6.6260755   10-34   J.s la constante de Planck
//@ k = 1.380658   10-23   J.K-1    la constante de Boltzmann
//@ c = 2.99792458   108   m.s-1    la vitesse de la lumiere dans le vide

return IDefPlkPK1*nu*nu*nu/(exp(IDefPlkPK2*nu/T)-1);
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PlanckInv
//@
//@ Description : Calcule l'inverse de la fonction de planck pour  une
//@ luminance B et un nombre d'onde nu
//@
//@
//@ Parameters :
//@         (ENTREE)  B : luminance ( w/(m2 sr m-1)
//@         (ENTREE)  nu: nombre d'onde ( m-1)
//@         (ENTREE)  IDefPlkPK1: Precalcul de 2*h*c*c
//@         (ENTREE)  IDefPlkPK2: Precalcul de h*c/k
//@
//@ Return Status :
//@          T : temperature du corps noir ( oK)
//@
//@ ------------------------------------------------------------------*/
double SD_BAS_PlanckInv (
    double const B,         /* luminance */
    double const nu,    /* nombre d'onde */
    double IDefPlkPK1,  /* Precalcul de 2*h*c*c */
    double IDefPlkPK2   /* Precalcul de h*c/k */
    )
{
//@  cf annexe 7 du DA7 et P 55 du DA 3
//@      on prevoit ici de mettre en dur les constantes pour des pb de performances
//@ h = 6.6260755   10-34   J.s la constante de Planck
//@ k = 1.380658   10-23   J.K-1    la constante de Boltzmann
//@ c = 2.99792458   108   m.s-1    la vitesse de la lumiere dans le vide

  if (B <= 0.0)
  {
    return 0.0;
  }
  else
  {
    return IDefPlkPK2*nu/log(1+IDefPlkPK1*nu*nu*nu/B);
  }
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_PlanckDeriv
//@
//@ Description : Calcule la derivee de la fonction de planck par
//@ rapport a la temp pour temperature T et un nombre d'onde nu
//@
//@
//@ Parameters :
//@         (ENTREE)  T : temperature du corps noir ( oK)
//@         (ENTREE)  nu: nombre d'onde ( m-1)
//@         (ENTREE)  IDefPlkPK1: Precalcul de 2*h*c*c
//@         (ENTREE)  IDefPlkPK2: Precalcul de h*c/k
//@
//@ Return Status :
//@         dBdt : derivee de planck  ( w/(m2 sr m-1 T )
//@
//@ ------------------------------------------------------------------*/
double SD_BAS_PlanckDeriv (
    double const T,       /* temperature */
    double const nu,          /* nombre d'onde */
    double IDefPlkPK1,  /* Precalcul de 2*h*c*c */
    double IDefPlkPK2   /* Precalcul de h*c/k */
    )
{
//@  cf annexe 7 du DA7 et P 55 du DA 3
//@      on prevoit ici de mettre en dur les constantes pour des pb de performances
//@ h = 6.6260755   10-34   J.s la constante de Planck
//@ k = 1.380658   10-23   J.K-1    la constante de Boltzmann
//@ c = 2.99792458   108   m.s-1    la vitesse de la lumiere dans le vide

    double X = exp(IDefPlkPK2*nu/T);
    double Y = nu*nu/T;

// sortie de la fonction
return IDefPlkPK1*IDefPlkPK2*Y*Y*X/((X-1)*(X-1));
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_FctAmortissement
//@
//@ Description : calcul de la fonction d'ammortissement des bord
//@ d'une fenetre spectrale avant TF
//@ remarque : cette fonction allouant de la memoire elle ne doit pas etre
//@     utilisee de maniere intensive sous peine de pb de perf
//@
//@ Parameters :
//@         (ENTREE)  nbpts: nombre de points du signal a amortir
//@         (ENTREE)  n: amortisement sigma exprime en nombre de points
//@         (ENTREE)  deltanu: largeur d'un echantillon
//@         (sortie)  amort : fonction d'ammortissement
//@         ( allouee par l'appelant)
//@         (ENTREE)  G: tableau temporaire de taille 6*n + 1
//@
//@ Return Status :
//@         t_CMN__Statut
//@             CMN__STATUT_OK si la fonction a reussi
//@             CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut  SD_BAS_FctAmortissement (
    int const nbpts,    /* nombre de points du signal a amortir */
    int const n,        /* nombre de points pour l'amort sigmas*/
    double const deltanu,   /* largeur d'un echantillon */
    double *amort,      /* fonction d'ammortissement */
    double *G               /* Fonction d'amortissement G(nu) */
    )

{
    t_CMN__Statut cr_op=CMN__STATUT_OK;
    double sigma;           /* Sigma pour l'amortissement */
    double invsigma;        /* Inverse de sigma */
    double invsigsurracpi;  /* Tmp pour le precalcul de 1/(sigma*sqrt(pi)) */
    double *pt_finG;        /* Pointeur de comptage temporaire sur G */
    double *pt_debG;        /* Pointeur de comptage temporaire sur G */
    double *pt_finS;        /* Pointeur de comptage temporaire sur la sortie */
    double *pt_debS;        /* Pointeur de comptage temporaire sur la sorite */
    double tmp_freq = 0.0;  /* Frequence tmp */
    double dtmp = 0.0;      /* Double tmp */
    double invsomnoy = 0.0; /* Inverse de la somme des coef de G */
    int i,p,d;              /* Compteurs divers */

    double nnew;


    nnew = (double) n / 4.0; // Pour prendre en compte l'erreur du CNES
    //@ calcul de sigma = n * deltanu
    sigma = nnew * deltanu;
    if (sigma != 0.0)
    {
      invsigma = 1.0/sigma;
    }

    if(G==NULL)
    {
        cr_op = CMN__STATUT_NOK;
        goto fin;
    }

    if(nnew == 0.0)
    {
        //@ Normalisation du noyau
        pt_debG = G;
        pt_finG = G + 1;

        while(pt_debG < pt_finG)
        {
           *pt_debG = 1.0;
           pt_debG++;
        }
    }
    else
    {
        // Precalcul de 1/(sigma*sqrt(pi))
        invsigsurracpi = invsigma * INVERSERACINEPI;

        tmp_freq = deltanu;

        //@ definition de la fonction gaussienne sur -3n 3n pts allocation dyn)
        //@ G (n) = 1/(n*deltanu sqrt(pi)) * exp ( -(nu/n)**2 )
        //@ On se place au milieu et on ne calcule que la moitie des coefs
        // Sauf le coef central
        for (i = 1 ; i <= n ; i++)
        {
            dtmp = tmp_freq*invsigma;
            dtmp = dtmp*dtmp;

            G[n - i] = exp(-(log(2.0) * dtmp))*invsigsurracpi;

            //@ La deuxieme moitie des coeffs est calculee par symetrie
            G[n + i] = G[n - i];
            tmp_freq += deltanu;

            //@ Calcule de la somme des coefs du noyau
            invsomnoy += G[n - i] * 2.0;
        }

        //@ Calcul du coef central
        pt_debG = G + n;
        *pt_debG = invsigsurracpi;
        invsomnoy += invsigsurracpi;

        invsomnoy = 1.0/invsomnoy;

        //@ Normalisation du noyau
        pt_debG = G;
        pt_finG = G + 2 * n +1;

        while(pt_debG < pt_finG)
        {
            *pt_debG *= invsomnoy;
            pt_debG++;
        }
    }



    //@ convolution de C par G pour obtenir amort

    pt_debS = amort;
    pt_finS = amort + nbpts - 1;

    //@ Convolution a l'interieur de l'intervalle [nud,nuf]
    //@ Toujours sur la moitie des points
    //@ La convolution par une porte est calculee comme la somme des coeff
    //@ du filtre sur un certain intervalle, intersection du filtre et de
    //@ la porte

    i = 0;  // Compteur de l'indice i de amort en cours de calcul
    p = 0;  // Premier indice du filtre qui correspond a un 1 de la porte
    d = 0;  // Dernier indice du filtre qui corresponnd a un 1 de la porte


    while (pt_debS <= pt_finS)
    {
        //@ Calcul de l'intersection de la porte et des coefficients du filtre
        //@ Coordonnee du debut de l'intersection
        p = max(2 * n - i,0);
        //@ Coordonnee de la fin de l'intersection
        d = min(nbpts-i-1, 2 * n);

        pt_debG = G + p;
        pt_finG = G + d + 1;
        *pt_debS = 0.0;

        //@ Convolution => en fait somme des coeffs du filtres correspondant
        //@ aux coeffs non nuls de la porte
        while(pt_debG < pt_finG)
        {
            *pt_debS += *pt_debG;
            pt_debG++;
        }

        //@ Utilisation de la symetrie
        *pt_finS = *pt_debS;

        pt_debS++;
        pt_finS--;
        i++;
    }


fin :
// sortie de la fonction
return(cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_EvolutionModele
//@
//@ Description : calcul de l'evolution d'une variable en fonction de la
//@       date courante
//@ remarque : cette fonction allouant de la memoire elle ne doit pas etre
//@     utilisee de maniere intensive sous peine de pb de perf
//@
//@ Parameters :
//@         (ENTREE)  T: date courante
//@         (ENTREE)  Tfe: date de fin de l'eclipse precedente
//@         (ENTREE)  Ts: periode orbitale en SECONDES
//@         (ENTREE)  Model: modele d'evolution
//@
//@ Return Status :
//@         x : valeur evaluee
//@
//@ ------------------------------------------------------------------*/
double  SD_BAS_EvolutionModele (
     t_SD_BAS_SCDSTime T,
     t_SD_BAS_SCDSTime Tfe,
     double Ts,
     double *Model
    )
{
//@  cf annexe 7 du DA7
//@ Le modele a la forme generale suivante :
//@     f(t) = a0*(1+a1*(t-t0)) + (1+b1*(t-t0))*g(u),
//@     avec u = t-tFE mod. TS  (A7-8)
//@ ou
//@ �   t est la date courante
//@ �   t0 une date de reference
//@ �   tFE est la date de fin de l?eclipse precedente
//@ �   TS la periode orbitale du satellite METOP
//@ �   g(u) est une fonction donnee par une tabulation
//@  Les parametres permettant de calculer la fonction f(t)
//@  par la formule (A7-8) sont fournis dans un tableau de 200 reels
//@  decrit ci-dessous. Les dates t0, t et tFE sont fournies dans le format
//@  Short CCSDS time (AC) (cf. [DR46])
//@
//@ Fonction Modele_orbital (t, tFE, TS, Model)
//@       Si (n>0) alors
//@          Calculer u = n * [(t-tFE) mod. TS] / TS
//@             i = Partie Entiere (u)
//@             x = u - i
//@         Si (i = n-1) Alors  g = gn-1*(1-x) + g0*x
//@         Sinon           g =  gi*(1-x) + gi+1*x
//@       Sinon             g = 0
//@       Fsi
//@      Retourner a0*(1+a1*(t-t0)) + (1+b1*(t-t0))*g

    double ft = 0.0;
    double g = 0.0;
    double u,x;
    int i;
    unsigned int tmp;
    unsigned int tmpf;
    unsigned int TsMilli;

    // Recuperation des parametres du modele
    int n           = (int)Model[5];
    double a0       = Model[2];
    double a1       = Model[3];
    double b1       = Model[4];
    double Mday     = Model[0];
    double Mmillis  = Model[1];
    double *Mg      = &Model[6];

    if(n > 0)
    {
        //@  Calcul de t-tFE en ms
        tmp = (T.day - Tfe.day)*86400000 + (T.millis - Tfe.millis);

        TsMilli = (unsigned int)(Ts*1000);
        u = (double)n * (double)(tmp % TsMilli) / (double)TsMilli;
        i = (int)floor(u);
        x= u - (double)i;

        if(i == n-1)
            g = (1.0-x)*Mg[n-1] + x*Mg[0];
        else
            g = (1.0-x)*Mg[i] + x*Mg[i+1];
    }

    //@ Calcul de T - T0 en jours
    tmpf = (T.day - Mday) + (T.millis - Mmillis)/86400000.0;

    //@ Estimation du parametre
    ft = a0*(1+a1*tmpf) + g*(1+b1*tmpf);

return ft;
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_Extremum2D
//@
//@ Description : calcul de la position de l'extremum d'une variable
//@ dont on connait la valeur en 9 points
//@ remarque : cette fonction doit etre appelee apres resolution du systeme
//@ lineaire par moindres carres ( fonction SD_BAS_MoindresCarres1D)
//@ remarque : on suppose que les points de la grille sont numerote de la
//@ maniere suivante ( de -1 a 1 pour x et de -1 a 1 pour y) ceci est
//@ utilise lors de la resolution par moindres carre de SD_BAS_MoindresCarres1D
//@
//@ Parameters :
//@         (ENTREE)  x : vecteur de 6 doubles contigus x1,x2,x3,x4,x5,x6
//@                     coefficients du paraboloide ajuste.
//@         (ENTREE)  eps : moyenne quadratique des residus d'ajustement
//@         (SORTIE)  res : vecteur de 4 doubles alloue par l'appelant qui contiendra
//@                         en sortie :
//@                             - Ymax
//@                             - Zmax
//@                             - residu d'ajustement
//@                             - Qualite de l'estimateur
//@
//@         (ENTREE)  dcmax : decalage central entier 
//@         (ENTREE)  dlmax : decalage central entier 
//@
//@ Return Status :
//@         SD_BAS_PAS_EXTREMUM
//@         SD_BAS_HORS_GRILLE
//@         CMN__STATUT_OK si la fonction a reussi
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut  SD_BAS_Extremum2D (
        double * x,
        double eps,
        double *res,
	int dcmax,
	int dlmax
    )
{
//@  cf annexe 7 du DA7
//@ On cherche a determiner (Ym,Zm) en approximant f par un paraboloide P.
//@ P(Y, Z) = x1 + x2Y + x3Z + x4Y2 + x5Z2 + x6YZ
//@ xi a ete calcule prealablement par resolution par moindres carres
//@ Position de l?extremum
//@ 1 (Ym, Zm) est alors calcule comme l'extremum de P(Y,Z)
//@   qui est donne par les equations (cf DA7)
//@   La resolution de ce systeme lineaire 2 x 2 fournit immediatement
//@    (cf DA7) xm et ym
//@   Si     D != 0
//@   condition d'erreur : D >=0 pas d'extremum
//@   condition d'erreur : xm et ym en dehors de la grille (<0 ou > 2)
//@ 2 calcul de lambda1 et lambda2 (valeurs propres)
//@ 3 calcul du demi grand axe a = Posmaxqual
//@   Cette valeur sera consideree comme critere de qualite de la determination de
//@  l?extremum en prenant  une estimation de epsilon  proportionnelle a la moyenne
//@  quadratique des residus d?ajustement du polynome (res)
//@ 4 calcul du maximum en xm ym avec le modele
//@  correlqual= P(xm,ym)
// sortie de la fonction

    t_CMN__Statut cr_op = CMN__STATUT_OK;

    double D;                   // Determinant du systeme
    double lambda1, lambda2;    // Valeurs propres
    double t1,t2;               // Valeurs de calcul intermediaires


    //@ Calcul du determinant du systeme
    D = (*(x+5)) * (*(x+5)) - 4 * (*(x+3)) * (*(x+4));

    t1 = *(x+3) + *(x+4);
    t2 = sqrt( (*(x+3) - *(x+4)) * (*(x+3) - *(x+4)) + ((*(x+5))  * (*(x+5))) );

    //@ Calcul des valeurs propores
    lambda1 = 0.5*(t1+t2);
    lambda2 = 0.5*(t1-t2);


    //@ On calcule tout le temps l'extremum, les NaN se propagent...
    //@ Ymax
    *res        = ( 2 * (*(x+1)) * (*(x+4)) - (*(x+2)) * (*(x+5)) )/D;

    //@ Zmax
    *(res+1)    = ( 2 * (*(x+2)) * (*(x+3)) - (*(x+1)) * (*(x+5)))/D;

    //@ Vmax
    *(res+2)    = *x + *(x+1) * (*res) + *(x+2) * (*(res+1))
                     + *(x+3) * (*res) *(*res)
                     + *(x+4) * (*(res+1)) *(*(res+1))
                     + *(x+5) * (*res) *(*(res+1));
//correction suite tel DB du 27/10 
//    *(res+2)    = eps;

    //@ Calcul de l'estimateur de qualite
    /* le facteur 3 fait reference au fait que les valeurs du polynome sont */
    /* connues avec une precision eps estimee a 3 sigma des residus */
    *(res+3)    = max(3*eps/sqrt(fabs(lambda1)) , 3*eps/sqrt(fabs(lambda2)));

   //@ Cas d'erreur
    if(*res < dlmax-1 || *res > dlmax+1 )
        cr_op=SD_BAS_HORS_GRILLE;

    if(*(res+1) < dcmax-1.0 || *(res+1) > dcmax+1 )
        cr_op=SD_BAS_HORS_GRILLE;

    if(D >= 0.0)
        cr_op=SD_BAS_PAS_EXTREMUM;

return(cr_op);
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_CalculPuissance2Sup
//@
//@ Description : calcule la puissance de 2 immediatement superieure ou egale
//@
//@
//@ Parameters :
//@     (ENTREE)  : valeur entiere dont on cherche la puissance de 2 
//@                 immediatement superieure ou egale
//@
//@ Return Status : 
//@ aucun
//@
//@ ------------------------------------------------------------------*/
int SD_BAS_CalculPuissance2Sup (int x)
{
  int y = 1;
  while(y < x){y = y << 1;}
  return (y);
}


//@ ------------------------------------------------------------------
//@ Methode : SD_BAS_ConvertirDateJourJulien
//@
//@ Description : Convertit une date au format en jour Julien
//@ (ENTREE)  date : date au format t_SD_BAS_GeneralTime
//@ (SORTIE)  jour_julien : date en jours juliens (0.0 au 1/1/2000 00:00:00.0)
//@
//@ Parametres :
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------
int SD_BAS_ConvertirDateJourJulien(
  t_SD_BAS_GeneralTime * date, 
  double *               jour_julien)

{
  /* Variables locales */
  /*-------------------*/
  t_CMN__Statut CodeRetour = CMN__STATUT_OK;
#ifdef OPS_DEBUG
  char          Fonction[] = "SD_BAS_ConvertirDateJourJulien";
#endif
  
  double         juliens;
  int            annee_i;
  int            mois_i;
  int            reste_entier;
  t_CMN__Booleen bissextile;
  
  /* Initialiser les variables locales */
  /*-----------------------------------*/
  
  /* Corps de l'operation */
  /*----------------------*/
#ifdef OPS_DEBUG
  printf ("Debut de %s\n", Fonction);
#endif
    
  /* Initialisation du nombre de jours juliens a zero */
  juliens = 0.0;

  /* POUR les annees comprises entre C_ANNEE_0 et l'annee donnee */
  /* FAIRE */
  for (annee_i = 2000 ; annee_i <= date->year; annee_i++)
  {
    /* CALL Est_bissextile */
    bissextile = SD_BAS_EstBissextile(annee_i);

    /* SI annee bissextile ALORS */
    if ( bissextile == CMN__FAUX )
    {
     	/* le mois de fevrier contient 28 jours */
      jours_par_mois[mois_fevrier] = 28.0;

    /* SINON */
    }
    else
    {
      /* le mois de fevrier contient 29 jours */
      jours_par_mois[mois_fevrier] = 29.0;
    }
    /* FINSI Est annee bissextile */

    /* SI ce n'est pas l'annee donnee ALORS */
    if (annee_i != date->year)
    {
      /* Calcul de la somme des jours de l'annee */
      juliens = juliens + JOURS_EN_JANVIER   + jours_par_mois[mois_fevrier]
                      		+ JOURS_EN_MARS      + JOURS_EN_AVRIL
		                      + JOURS_EN_MAI       + JOURS_EN_JUIN
		                      + JOURS_EN_JUILLET   + JOURS_EN_AOUT
                      		+ JOURS_EN_SEPTEMBRE + JOURS_EN_OCTOBRE
		                      + JOURS_EN_NOVEMBRE  + JOURS_EN_DECEMBRE;
    }
    /* FINSI ce n'est pas l'annee donnee */
  }
  /* FINFAIRE des annees */
      
  /* SI au moins un mois ecoule de l'annee ALORS */
  if ( date->month > mois_janvier)
  {
    /* POUR les mois ecoules de l'annee donnee FAIRE */
    for (mois_i=mois_janvier; mois_i < date->month; mois_i = mois_i + 1)
    {
      /* Additionner les jours de ces mois ecoules */
      juliens = juliens + jours_par_mois[mois_i];
    }
    /* FINFAIRE */
  }
  /* FINSI mois different de janvier */

  /* Ajout des jours du mois en cours moins le premier janvier */
  juliens = juliens + date->day - 1; /* zero au 1 janvier a 00:00:00.00 */

  /* Calcul de la partie fractionnaire du jour julien */
  juliens = juliens 
    + (double)date->hour / 24.0
    + (double)date->min / (24.0 * 60.0)
    + (double)date->sec / (24.0 * 60.0 * 60.0);
    + (double)date->millis/(1000*24.0 * 60.0 * 60.0);
  
  /* Ecriture du resultat */
  *jour_julien = juliens;
    
fin :
#ifdef OPS_DEBUG
  printf ("Fin de %s\n", Fonction);
#endif
  return(CodeRetour);
} /* fin de SD_BAS_ConvertirDateJourJulien */


//@ ------------------------------------------------------------------
//@ Methode : SD_BAS_EstBissextile
//@
//@ Description : teste si une annee est bissextile
//@ (ENTREE)  annee 
//@
//@ Parametres :
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------
t_CMN__Booleen SD_BAS_EstBissextile(const int annee)

{
  /*U Variables locales */
  t_CMN__Booleen bissextile;
  int            reste_entier;

  /* Corps de l'operation */

  /* Intialisation de la valeur a 365 jours */
  bissextile = CMN__FAUX;
   
  /* Les annees bissextiles sont un multiples de 4 */
  /* SI annee multiple de 4 ALORS */
  reste_entier = annee % 4;
  if ( reste_entier == 0 )
  {
   /* C'est a priori une annee bissextile */
    bissextile = CMN__VRAI;
  }
  /* FINSI annee multiple de 4 */

  /* SI annee multiple de cent ALORS */
  reste_entier = annee % 100;
  if (reste_entier == 0)
  {
    /* Il y a recalage et donc annee non bissextile */
    bissextile = CMN__FAUX;
  }
  /* FINSI annee multiple de cent */

  /* SI annee multiple de 400 ALORS */
  reste_entier = annee % 400;
  if (reste_entier == 0)
  {
    /* Tous les quatre cents ans le recalage n'est pas effectue */
    /* et donc c'est une annee bissextile de 366 jours */
    bissextile = CMN__VRAI;
  }
  /* FINSI annee multiple de 400 */
  
  /* SORTIE */
  return (bissextile);
} /* Fin de SD_BAS_EstBissextile */


//======================================================================
//                  FONCTIONS INTERNES
//======================================================================

