//$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/BAS/src/SD_BAS_OperateurLoc.c,v 1.6 2014/05/14 15:41:10 lcaspar Exp $
//@ --------------------------------------------------------------------
//@ Composant : SD
//@ Sous-composant : SD_BAS
//@ MODULE : SD_BAS_OperateurLoc
//@
//@ Comment :
//@    Code file for the  SD_BAS_OperateurLoc class which includes the
//@    geometrical algorithms used to transform the coordinates  :
//@ - from METOP  to earth
//@ - from earth to METOP
//@     -.....
//@ this is done thanks to Metop libraries which are non-thread-safe
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 27/02/2003 : [reference :] M. Jung
//@ Fonctions de bases de geolocalisation
//@ --------------------------------------------------------------------
//@ FA-ID :IA--CR-2100-257-CNE : 1/02/2005 : JL PASCAL
//@ Mise a jour de l'origine des tableaux de latitude de 25 a 5
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-160-CN : 6/06/2005 : JL. PASCAL
//@ correction test pour supporter donnees AVHRR degradees
//@ fiche de correction correspondante : IA-2100-FC-160-THA
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-200-CN : 21/05/2007 : R. MORENO
//@ PB de loc IASI, le CG  est systematiquement leve pour
//@ SN 11.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2997(DM-2100-OPS) : 12/06/2012 : L. CASPAR
//@ Ajout de fonctions pour calculer la geolocalisation sans l'AVHRR
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-3130(FA-2100-OPS) : 06/01/2013 : L. CASPAR
//@ Ajout de tests d'erreur
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include <stdio.h>
#include <pthread.h>
#include <math.h>

#include "SD_BAS_OperateurBase.h"
#include "SD_BAS_OperateurLoc.h"


// declaration des fonctions importees
#ifdef OPS_USE_METOPLIB
#include "metop_pointing.h"
#include "metop_orbit.h"
#include "metop_lib.h"
#else
#include "navigation.h"
#endif 

//*====================================================================*/
//* Local Constant Declaration
//*====================================================================*/
/*  CR257 */
#define   FIRST_AVHRR_COL 5     // Le bandeau AVHRR contient des infos
// sousechantillonnees entre le pixel 5 et 2045 par pas de SOUSECH_AVHRR_COL
#define   SOUSECH_AVHRR_COL 20     // Le bandeau AVHRR contient des infos
// sousechantillonnees entre le pixel 25 et 2025 par pas de SOUSECH_AVHRR_COL
#define SOUSECH_AVHRR_LIG 10     // pour optimisation de la recherche

const double SD_BAS_180=180.;  

// FT-2997
// duree entre 2 lignes, en secondes
const double SD_BAS_LINE_STEP_TIME = 1. / 6.;
// frequence de numerisation
const double SD_BAS_DIGIT_FREQ = 39936.;
// pixel number at Nadir
const double SD_BAS_PIX_NB_NADIR = 1024.45;
// delta du temps de balayage
const double SD_BAS_SCAN_DELTA_TIME = (34. / 1000.) / 86400.;
	
const int SD_BAS_MAX_ITER = 30;
const double SD_BAS_TIME_THRES = 0.001 / 86400.;
const double SD_BAS_TTOLER = 2. * 0.001 / 86400.;	
// ...FT-2997


//*====================================================================*/
//* Local types Declaration
//*====================================================================*/
//======================================================================
// Internal fonctions Declaration
//======================================================================
#include "SD_BAS_Niveau0.h"
static t_CMN__Statut SD_BAS_InterpolLatLon(
       t_SD_BAS_DataThread * ctx,
        const double  x,
        const double  y,
        const double  z,
        const int  colentier,
        const int  ligentier,
        t_SD_BAS_LigneAVHRR** BandeauAVHRR,
        int nbligbandeau,
        double * col,
        double *lig
            );
            
// FT-2997
static t_CMN__Statut SD_BAS_AzElVectPos(
        // Entrees 
        const double azimuth,
        const double elevation,
        // Sorties
        double VectPosRM[SD_BAS_DIM_ROT_MATRICE]
            );

static t_CMN__Statut SD_BAS_VectPosAzEl(
        // Entree
        const double VectPosRM[SD_BAS_DIM_ROT_MATRICE],
        // Sorties
        double * azimuth,
        double * elevation
            );
            
static t_CMN__Statut SD_BAS_TimeScanAngle2ColLin(
        // Entrees 
        const double AvhrrTime,
        const double AvhrrScanAngle,
        const double RefTime,
        // Sorties
        double * AvhrrLin,
        double * AvhrrCol
            );

static t_CMN__Statut SD_BAS_ColLin2TimeScanAngle(
        // Entrees
        const double AvhrrLin,
        const double AvhrrCol,
        const double RefTime,
        // Sorties
        double * AvhrrTime,
        double * AvhrrScanAngle
            );

static t_CMN__Statut SD_BAS_latlon2avhrr(
        // Entrees
		t_SD_BAS_DataThread * ctx,
        const double lat,
        const double lon,
        const double alt,
        double aocs[3],
        double att[3], 
        double datt[3],
        long iatt,
        const double timefirst,
        const double timelast,
        double mjdr_t[2],
        double xm_t[6],
        double x_t[6],
        // Sorties
        double * AvhrrTime,
        double * AvhrrScanAngle
            );
            
static t_CMN__Statut SD_BAS_Init_mo_orbit(
        // Entrees
		t_SD_BAS_DataThread * ctx,
		t_SD_BAS_MPHR *       MphrN0,
        // Sorties
    	long * iatt,
        double mjdr_t[2],
        double xm_t[6],
        double x_t[6],
   		double res_orbit_t[54],     
		double 	acc_t[3]
            );
// ...FT-2997

//======================================================================
// Global variables Definition
//======================================================================

// Mutex pour les variables conditionnelles POSIX
static     pthread_mutex_t   s_MutexTravaille;

// Messages d'erreur
static char *ERR_SEARCH="search problem : point not found";
static char *ERR_INTERP="Interpolation problem : bad interval";
static char *ERR_EXTRAP="Interpolation problem : extrapolation greater than 1 point";
static char *SD_BAS_OPLOC_MSG_ERR_METOP2GEOD="Error during SD_BAS_MetopGeod";
static char *SD_BAS_OPLOC_MSG_ERR_GEOD2AVHRR="Error during SD_BAS_GeodAVHRR";
static char *SD_BAS_OPLOC_MSG_ERR_AVHRR2GEOD="Error during SD_BAS_AVHRRGeod";
static char *SD_BAS_OPLOC_MSG_ERR_AZEL2VECTPOS="Error during SD_BAS_AzEl2VectPos";
static char *SD_BAS_OPLOC_MSG_ERR_VECTPOS2AZEL="Error during SD_BAS_VectPos2AzEl";
static char *SD_BAS_OPLOC_MSG_ERR_LATLON2AVHRR_CONV="latlon2avhrr problem : iterative process does not converge";
static char *SD_BAS_OPLOC_MSG_ERR_LATLON2AVHRR_TTOLER="latlon2avhrr problem : the obtaines time is not beetween timefirst and timelast";
static char *SD_BAS_OPLOC_MSG_ERR_INITMOORBIT="Error during SD_BAS_Init_mo_orbit";
//FA160 test supplementaires
static char *SD_BAS_OPLOC_MSG_ERR_SPLINE2D="Error during SD_BAS_interpSpline2D : result inpredictible";
static char *SD_BAS_OPLOC_MSG_ERR_GEOD2METOP="Error during SD_BAS_GeodMetop";
// CR 232 mise en place configuration variable
static char * s_METOP_ENV;
static char *SD_BAS_MSG_METOPENV_ND="environnement variable METOP_ENV not defined";
static char *SD_BAS_MSG_METOPENV_NC="environnement variable METOP_ENV not must belong to [TEST,NORMAL]";
/* variable de positionnement envirt METOP*/
const char * METOP_ENV = "METOP_ENV";
const char * METOP_ENV_TEST = "TEST";
const char * METOP_ENV_NORMAL = "NORMAL";




//======================================================================
//                  FONCTIONS EXTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_MetopGeod
//@
//@ Description : conversion des coordonnees du referentiel Metop
//@     vers le referentiel geodesique (lat, lon)
//@ cette fonction ne peut etre appelee qu'apres la reservation de la
//@ librairie Metop (SD_BAS_Reserver)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         ( numero de ligne SN traite
//@         (ENTREE)  IasiTime[2] : date du SN traite
//@         (ENTREE)  MphrN0 :  MPHR produit N0
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  azim : coordonnees des pts dans le repere METOP (azim) ( EN RADIAN )
//@         (ENTREE)  elev : coordonnees des pts dans le repere METOP (elev) ( EN RADIAN )
//@         (SORTIE)  lat : latitude
//@         (SORTIE)  lon : longitude
//@         (SORTIE)  Ang_mesY : angles de mesure METOP (az)( EN RADIAN )
//@         (SORTIE)  Ang_mesZ : angles de mesure METOP (el) ( EN RADIAN )
//@         (SORTIE)  Azim_sol : angles solaire (az) ( EN RADIAN )
//@         (SORTIE)  Elev_sol : angles solaire (el) ( EN RADIAN )
//@         (SORTIE)  EarthSatelDist : disance Terre/Satellite
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_MetopGeod(
  t_SD_BAS_DataThread * ctx,
  t_SD_BAS_SCDSTime *   IasiTime,
  t_SD_BAS_MPHR *       MphrN0,
  int                   nbpts,
  double *              azim,
  double *              elev,
  double *              lat,
  double *              lon,
  double *              Ang_mesY,
  double *              Ang_mesZ,
  double *              Azim_sol,
  double *              Elev_sol,
  double *              EarthSatelDist)

{
#ifdef OPS_USE_METOPLIB
  static const char tcProcName_F[] = "SD_BAS_MetopGeod";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  long mode_t;
  long ierr_orbit_t[4];
  long ierr_tar[15];       // mode d'activation et status vector
  double mjdr_t[2];
  double mjdp_t[2];
  double pos_t[3];
  double vel_t[3];       /* position et vitesse au noeud acendant de l'orbite */
  long status;                        /* Main status flag */
  double xm_t[6];
  double x_t[6];          // valeurs de Kelpler
  double acc_t[3];
  double res_orbit_t[54];    // valeurs temporaires (acceleration)
  long n;                 /* Number of error messages */
  long func_id;                           /* CFI function ID */
//  char msg[MO_MAX_COD][MO_MAX_STR];       /* Error messages vector */
  t_SD_BAS_MetopMSG * msg;       /* Error messages vector */
  int i;                  /* indice */
  long   iatt;                /* parametre d'activation de mp_target */
  double aocs_g[3];
  double att[3];
  double datt[3];
  long   idir,iray,ieres;         /* parametre d'activation de mp_target */
  double   dir[8];
  double freq;           /* parametre d'activation de mp_target */
  double res_tar[75];         /* resultat d'activation de mp_target */
  double dummy = 0.0;
  t_SD_BAS_GeneralTime state_vector_time;

  msg=ctx->ptmsg;
  state_vector_time.year = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->year;
  state_vector_time.month = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->month;
  state_vector_time.day = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->day;
  state_vector_time.hour = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->hour;
  state_vector_time.min = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->min;
  state_vector_time.sec = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->sec;
  state_vector_time.millis = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->millis;

  //-----------------------
  // Entree dans la fonction
  //@ calcul de la position du satellite et de l'acceleration
  //@ on met le mode de trace au maximum
  //-----------------------
  mo_silent();
  mp_silent();
  
  //-----------------------
  //@ 1 - initialisation
  //@  mode : MO_INIT
  //-----------------------
  // correction validation : on travaille en mode degrade (cf RM)
//  mode_t = MO_INIT;
//  mode_t = MO_INIT_GENSTATE;
// CR 232 mise en place configuration variable
 if (strcmp(s_METOP_ENV,METOP_ENV_TEST)==0)
    {
    // cas test
    mode_t = MO_INIT_GENSTATE;
    iatt = 2;
    }
  else
    {
    // cas nominal
    mode_t = MO_INIT;
    iatt = 3;
    }
  
  
  //-----------------------
  //@  mjdr : a utiliser en 2
  //@  xm : a utiliser en 2
  //@  mjdp : issu du MPHR N1 ( comment ? )
  //-----------------------
  // absolute UTC
  SD_BAS_ConvertirDateJourJulien (&state_vector_time, &(mjdp_t[0]));
  mjdp_t[1]=0.0; // DUT constante par orbite a definir ( entre 0 et 1 )
  
  //-----------------------
  //@  pos : issu du MPHR N1  ( comment ?)
  //-----------------------
  pos_t[0]=*(MphrN0->listevar.tab_var[MPHR_X_POSITION].val.pr8); //MPHR_X_POSITION
  pos_t[1]=*(MphrN0->listevar.tab_var[MPHR_Y_POSITION].val.pr8); //MPHR_Y_POSITION
  pos_t[2]=*(MphrN0->listevar.tab_var[MPHR_Z_POSITION].val.pr8); //MPHR_Z_POSITION
  
  //-----------------------
  //@  vel : issu du MPHR N1  (comment ?)
  //-----------------------
  vel_t[0]=*(MphrN0->listevar.tab_var[MPHR_X_VELOCITY].val.pr8); //MPHR_X_VELOCITY
  vel_t[1]=*(MphrN0->listevar.tab_var[MPHR_Y_VELOCITY].val.pr8); //MPHR_Y_VELOCITY
  vel_t[2]=*(MphrN0->listevar.tab_var[MPHR_Z_VELOCITY].val.pr8); //MPHR_Z_VELOCITY

  //-----------------------
  //@  acc :
  //@  res :
  //@  ierr : erreur
  //@ appel mo_orbit avec les arguments :
  //-----------------------
  status = mo_orbit(
    &mode_t, 
    mjdr_t, 
    xm_t, 
    mjdp_t,
    x_t, 
    pos_t, 
    vel_t, 
    acc_t, 
    res_orbit_t, 
    ierr_orbit_t);
  
  if (status != MO_OK)
  {
    func_id = MO_ORBIT_ID;
    //si erreur appel mo_vector_msg pour recuperation du libelle
    mo_vector_msg(&func_id, ierr_orbit_t, &n, msg);
    
    //@ on recupere le premier message d'erreur pour l'emission d'une trace
     cr_op   = SD_BAS_ERR_MP_ORBIT;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_ORBIT,
      tcProcName_F,
      __LINE__,
      msg[0])
     goto fin;
      
    //@ on imprime les message d'erreurs sur la sortie standard
#ifdef OPS_DEBUG
    SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, msg[0] );
    mo_print_msg(&n, msg);
    fprintf(stdout, "\n");
#endif

  }

  //-----------------------
  //@ 2 - propagation
  //@  mode : MO_PROPAG+MO_ORBIT_RES_BAS
  //-----------------------
  mode_t = MO_PROPAG+MO_ORBIT_RES_BAS+MO_ORBIT_RES_AUX;
  
  //-----------------------
  //@  mjdr : issu de 1
  //@  xm : issu de 1
  //@  mjdp : date du sous-cycle = IASITime (AC) - mjdr
  //-----------------------
  mjdp_t[0]=(double)IasiTime->day+((double)(IasiTime)->millis/(1000.*86400.));
  mjdp_t[1]=dummy;
 
  //-----------------------
  //@  pos : a utiliser dans la suite
  //@  vel : a utiliser dans la suite
  //@  acc : a  utiliser dans la suite
  //@  res :
  //@  ierr :
  //@ appel mo_orbit avec les arguments :
  //-----------------------
  status = mo_orbit(
    &mode_t, 
    mjdr_t, 
    xm_t, 
    mjdp_t,
    x_t, 
    pos_t, 
    vel_t, 
    acc_t, 
    res_orbit_t, 
    ierr_orbit_t);
  
  //si erreur appel mo_vector_msg pour recuperation du libelle
  if (status != MO_OK)
  {
    func_id = MO_ORBIT_ID;
    
    //si erreur appel mo_vector_msg pour recuperation du libelle
    mo_vector_msg(&func_id, ierr_orbit_t, &n, msg);
    
    //@ on recupere le premier message d'erreur pour l'emission d'une trace
     cr_op   = SD_BAS_ERR_MP_ORBIT;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_ORBIT,
      tcProcName_F,
      __LINE__,
      msg[0])
    goto fin;
    
    //@ on imprime les message d'erreurs sur la sortie standard
#ifdef OPS_DEBUG
    SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
    mo_print_msg(&n, msg);
    fprintf(stdout, "\n");
#endif
  }
  
  //-----------------------
  //@ Pour chacun des points x y
  //-----------------------
  n=0;
  
  aocs_g[0]= -0.1660;
  aocs_g[1]= 0.0508;
  aocs_g[2]= 3.9570;
  // correction validation : on travaille sans correction d'attitude
//CR 232 mise en place environnement variable
//  iatt=2;
//  iatt=3;
  
  att[0] = *(MphrN0->listevar.tab_var[MPHR_PITCH_ERROR].val.pr8); //  MPHR_PITCH_ERROR
  att[1] = *(MphrN0->listevar.tab_var[MPHR_ROLL_ERROR].val.pr8);  // MPHR_ROLL_ERROR
  att[2] = *(MphrN0->listevar.tab_var[MPHR_YAW_ERROR].val.pr8);   // MPHR_YAW_ERROR
  
  datt[0] = 0.0;        /* MetOp-1 SRAR mispointing rates [deg/s] */
  datt[1] = 0.0;
  datt[2] = 0.0;
  
  idir=MP_INTER_1ST;
  iray=MP_NO_REF;
  freq=0.;
  ieres=MP_TARG_RES_SAT2TARG+MP_TARG_RES_TARG2SUN;
  i=0;
  
  while ( i < nbpts )
  {
    //-----------------------
    //@ on appelle la fonction mp_target ([DA18])
    //@ Les arguments de cette fonction sont fournis par (cf. \ufffd7.1.3. de [DA18]) :
    //@ mjdp : date du sous-cycle = IASITime ( normalement issu de mo_orbit)
    //@ pos : position du satellite a la date mjdp obtenue par propagation des parametres orbitaux a la date mjdp ([DA19])
    //@       a l?aide de la fonction ESA mo_orbit et des parametres orbitaux fournis dans les entetes du niveau 0.
    //@ vel : acceleration, fournie egalement par mo_orbit
    //@ acc : acceleration, fournie egalement par mo_orbit
    //@ iatt=3
    //-----------------------
    //-----------------------
    //@ aocs : valeur par defaut preconisees dans [DA18], de toute facon cela ne sert a rien
    //@ att : YAW_ERROR, ROLL_ERROR, PITCH_ERROR fournie dans l?entete du niveau 0
    //-----------------------
    //-----------------------
    //@ datt : 0
    //-----------------------
    //-----------------------
    //@ idir : 1, intersection visee METOP et Terre
    //-----------------------
    //-----------------------
    //@ dir : direction de la ligne de visee (altitude sur Terre = 0 et delta vitesse = 0)
    //@ dir[0]= x dir[1]= y
    //-----------------------
//  conversion en degre 
    dir[0]=azim[i]*SD_BAS_180/M_PI;
    dir[1]=elev[i]*SD_BAS_180/M_PI;
    dir[2]=0.;
    dir[3]=dummy;
    dir[4]=0.;
    dir[5]=0.;
    dir[6]=dummy;
    dir[7]=dummy;
    
    //-----------------------
    //@ iray : 0 , pas de ray-tracing
    //@ freq : 0
    //-----------------------

    //-----------------------
    //@ ieres = Parametres permettant d?obtenir les sorties 1, 2, 3 e, 5, 66 et 67, soit a priori :
    //@ MP_TARG_RES_SAT2TARG+MP_TARG_RES_TARG2SUN
    //@ appel mp_target avec les arguments :
    //-----------------------
    status = mp_target(
      mjdp_t, 
      pos_t, 
      vel_t, 
      acc_t, 
      &iatt, 
      aocs_g, 
      att, 
      datt,
      &idir, 
      dir, 
      &iray, 
      &freq, 
      &ieres, 
      res_tar, 
      ierr_tar);

    //@ si erreur appel mp_vector_msg pour recuperation du libelle
    if (status != MP_OK)
    {
      func_id = MP_TARGET_ID;
#ifdef OPS_DEBUG
      //si erreur appel mo_vector_msg pour recuperation du libelle
      mp_vector_msg(&func_id, ierr_tar, &n, msg);
#endif
     
     cr_op   = SD_BAS_ERR_MP_TARGET;
      //@ on recupere le premier message d'erreur pour l'emission d'une trace
      SD_BAS_TEST_ERR_INT(
        ctx->cr.errinterne,
        SD_BAS_ERR_SET_OpLoc,
        SD_BAS_ERR_MP_TARGET,
        tcProcName_F,
        __LINE__,
        msg[0])
     goto fin;
      
      //@ on imprime les message d'erreurs sur la sortie standard
#ifdef OPS_DEBUG
      SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
      mp_print_msg(&n, msg);
      fprintf(stdout, "\n");
#endif
    }

    //-----------------------
    //@     Les coordonnees geodesiques sont disponibles dans les parametres de sortie 3 et 5 :
    //@ lat lon
    //@     Les angles de mesure sont disponibles dans les parametres de sortie 1 et 2 :
    //@     Ang_mes1 Ang_mes2
    //@     Les angles solaires sont disponibles dans les parametres de sortie 66 et 67 :
    //@     La distance terre/satellite est disponible dans le parametre 1 :
    //@ Ang_sol1 Ang_sol2
    //-----------------------
    EarthSatelDist[i] = res_orbit_t[MO_ORBIT_RES_ORBIT_RAD];
//    lat[i]=res_tar[MP_TARG_RES_GEOC_LAT]*M_PI/SD_BAS_180;
// la latitude a utiliser est la latitude geodesique
    lat[i]=res_tar[MP_TARG_RES_GEOD_LAT]*M_PI/SD_BAS_180;
//  conversion entre -180 et +180 et en radian
    if (res_tar[MP_TARG_RES_GEOC_LONG] <= SD_BAS_180)
      lon[i]=res_tar[MP_TARG_RES_GEOC_LONG]*M_PI/SD_BAS_180;
    else
      lon[i]=-(2*SD_BAS_180-res_tar[MP_TARG_RES_GEOC_LONG])*M_PI/SD_BAS_180;
       
    Ang_mesY[i]=res_tar[MP_TARG_RES_TARG2SAT_AZ_TOP]*M_PI/SD_BAS_180;
    Ang_mesZ[i]=res_tar[MP_TARG_RES_TARG2SAT_EL_TOP]*M_PI/SD_BAS_180;
    Azim_sol[i]=res_tar[MP_TARG_RES_TARG2SUN_AZ_TOP]*M_PI/SD_BAS_180;
    Elev_sol[i]=res_tar[MP_TARG_RES_TARG2SUN_EL_TOP]*M_PI/SD_BAS_180;

#ifdef OPS_DEBUG
    printf("Resulats relatifs au point %d\n",i);
    printf("   EarthSatelDist : %lf\n",EarthSatelDist[i]);
    printf("   Lat : %lf\n",lat[i]);
    printf("   Lon : %lf\n",lon[i]);
    printf("   Ang_mesY : %lf\n",Ang_mesY[i]);
    printf("   Ang_mesZ : %lf\n",Ang_mesZ[i]);
    printf("   Azim_sol : %lf\n",Azim_sol[i]);
    printf("   Elev_sol : %lf\n",Elev_sol[i]);
#endif
    i++;
  }
  //@ fin Pour chacun des points x y

  // on trace ici si besoin toutes les variables en sortie

fin:
// sortie de la fonction
//if (status != MO_OK) cr_op=CMN__STATUT_NOK;
return(cr_op);
#else
  static const char tcProcName_F[] = "SD_BAS_MetopGeod";
  t_CMN__Statut cr_op = CMN__STATUT_OK;


  double time;
  double pos[3];
  double vel[3];  
  double att[3][3]; 
  int iatt;
  int status = 0;    
  int i;
  
  status = navigation_ops_init();
  
  if( status != 0 ) {
    cr_op = SD_BAS_ERR_MP_ORBIT;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_ORBIT,
      tcProcName_F,
      __LINE__,
      "navigation_ops_init");
     goto fin;
  }

  time = (double)IasiTime->day + ( (double)(IasiTime)->millis / ( 1000. * 86400. ) );
  time += 18262;
  
  status = navigation_ops_intposvel( time, pos, vel );

  if( status != 0 ) {
    // P. Brunel 23 juillet 2013
    // ajout de messages
    printf("   intposvel time : %lf\n",time);
    printf("   iasitime day from 1950: %d\n", IasiTime->day + 18262);
    printf("   iasitime seconds: %lf\n", IasiTime->millis/1000.);

    cr_op = SD_BAS_ERR_MP_ORBIT;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_ORBIT,
      tcProcName_F,
      __LINE__,
      "navigation_ops_intposvel");
     goto fin;
  }
  
  /* MetopLib */
  if( strcmp( s_METOP_ENV, METOP_ENV_TEST ) == 0 ) {
    iatt = 2; /* NOAA */
  } else {
    iatt = 3; /* METOP */
  }
  
  /* translate to AAPP */
  switch( iatt ) {
    case 2:
      iatt = 0;
    break;
    case 3:
      iatt = 1;
    break;
  }
  
  navigation_calatt( 
    *(MphrN0->listevar.tab_var[MPHR_YAW_ERROR].val.pr8),
    *(MphrN0->listevar.tab_var[MPHR_ROLL_ERROR].val.pr8),
    *(MphrN0->listevar.tab_var[MPHR_PITCH_ERROR].val.pr8),
    att
  );
  

#ifdef OPS_DEBUG
    printf("pb appel navigation_ops_metop_to_earth pour nbpts %d\n",nbpts);
    printf("pb    time : %lf\n",time);
    printf("pb    pos: %lf  %lf  %lf\n",pos[0],pos[1],pos[2]);
    printf("pb    vel: %lf  %lf  %lf\n",vel[0],vel[1],vel[2]);
    i = 0;
    while ( i < nbpts )
    {
      printf("pb  i azim, elev: %d  %lf  %lf\n",i, azim[i],elev[i]);
      i++;
    }
#endif


  status = navigation_ops_metop_to_earth( 
            time, pos, vel, iatt, att, nbpts,
            azim, elev, lat, lon, 
            Ang_mesZ, Ang_mesY,
            Elev_sol, Azim_sol,
            EarthSatelDist );
            
  if( status != 0 ) {
    #ifdef OPS_DEBUG
    printf("pb    erreur navigation_ops_metop_to_earth status : %d\n",status);
    #endif

    cr_op = SD_BAS_ERR_MP_TARGET;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_TARGET,
      tcProcName_F,
      __LINE__,
      "navigation_ops_metop_to_earth");
    goto fin;
  }

#ifdef OPS_DEBUG
    printf("pb    Lat(0) Lon(0): %lf %lf\n",lat[0],lon[0]);
#endif


fin:
  return cr_op;
#endif
}


//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_GeodMetop
//@
//@ Description : conversion des coordonnees du referentiel Metop
//@     vers le referentiel geodesique (lat, lon)
//@ cette fonction ne peut etre appelee qu'apres la reservation de la
//@ librairie Metop (SD_BAS_Reserver)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@             numero de ligne SN traite
//@         (ENTREE)  IasiTime[2] : date du SN traite
//@         (ENTREE)  MphrN0 :  MPHR produit N0
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  lat : latitude (RADIAN)
//@         (ENTREE)  lon : longitude (RADIAN)
//@         (SORTIE)  azim : coordonnees des pts dans le repere METOP (RADIAN)
//@         (SORTIE)  elev : coordonnees des pts dans le repere METOP (RADIAN)
//@
//@ Return Status :
//@         t_CMN__Statut
//@         CMN__STATUT_OK si la fonction a reussi
//@         CMN__STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_GeodMetop(
  t_SD_BAS_DataThread * ctx,
  t_SD_BAS_SCDSTime *   IasiTime,
  t_SD_BAS_MPHR *       MphrN0,
  int                   nbpts,
  double *              lat,
  double *              lon,
  double *              azim,
  double *              elev)
        
{
#ifdef OPS_USE_METOPLIB
  static const char tcProcName_F[] = "SD_BAS_GeodMetop";
  t_CMN__Statut cr_op=CMN__STATUT_OK;
  long mode_t;
  long ierr_orbit_t[4];
  long ierr_s[4];      // mode d'activation et status vector
  double mjdr_t[2];
  double mjdp_t[2];
  double pos_t[3];
  double vel_t[3];       /* position et vitesse au noeud acendant de l'orbite */
  long status;                        /* Main status flag */
  double xm_t[6];
  double x_t[6];          // valeurs de Kelpler
  double acc_t[3];
  double res_orbit_t[54];    // valeurs temporaires (acceleration)
  long n;                 /* Number of error messages */
  long func_id;                           /* CFI function ID */
//   char msg[MO_MAX_COD][MO_MAX_STR];       /* Error messages vector */
   t_SD_BAS_MetopMSG * msg;       /* Error messages vector */
  int i;                  /* indice */
  long   iatt;                /* parametre d'activation de mp_stavis */
  double aocs_g[3];
  double att[3];
  double datt[3];
  double   sta[4];            /* parametre d'activation de mp_stavis */
  double res_sta[31];         /* resultat d'activation de mp_stavis */
  double dummy = 0.0;
  t_SD_BAS_GeneralTime state_vector_time;

  msg=ctx->ptmsg;
  state_vector_time.year = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->year;
  state_vector_time.month = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->month;
  state_vector_time.day = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->day;
  state_vector_time.hour = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->hour;
  state_vector_time.min = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->min;
  state_vector_time.sec = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->sec;
  state_vector_time.millis = 
    (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->millis;
    
  // Entree dans la fonction
  //-----------------------
  // Entree dans la fonction
  //@ calcul de la position du satellite et de l'acceleration
  //@ on met le mode de trace au maximum
  //-----------------------
  mo_silent();
  mp_silent();
  //-----------------------
  //@ 1 - initialisation
  //@  mode : MO_INIT
  //-----------------------
  // correction validation : on travaille en mode degrade (cf RM)
//  mode_t = MO_INIT;
//  mode_t = MO_INIT_GENSTATE;
// CR 232 mise en place environnement variable
  if (strcmp(s_METOP_ENV,METOP_ENV_TEST)==0)
    {
    // cas test
    mode_t = MO_INIT_GENSTATE;
    iatt = 2;
    }
  else
    {
    // cas nominal
    mode_t = MO_INIT;
    iatt = 3;
    }
  
  //-----------------------
  //@  mjdr : a utiliser en 2
  //@  xm : a utiliser en 2
  //@  mjdp : issu du MPHR N1 ( comment ? )
  //-----------------------
  // absolute UTC
  SD_BAS_ConvertirDateJourJulien (&state_vector_time, &(mjdp_t[0]));
  mjdp_t[1]=0; // DUT constante par orbite a definir ( entre 0 et 1 )
  
  //-----------------------
  //@  pos : issu du MPHR N1  ( comment ?)
  //-----------------------
  pos_t[0]=*(MphrN0->listevar.tab_var[MPHR_X_POSITION].val.pr8); //MPHR_X_POSITION
  pos_t[1]=*(MphrN0->listevar.tab_var[MPHR_Y_POSITION].val.pr8);; //MPHR_Y_POSITION
  pos_t[2]=*(MphrN0->listevar.tab_var[MPHR_Z_POSITION].val.pr8);; //MPHR_Z_POSITION
  
  //-----------------------
  //@  vel : issu du MPHR N1  (comment ?)
  //-----------------------
  vel_t[0]=*(MphrN0->listevar.tab_var[MPHR_X_VELOCITY].val.pr8); //MPHR_X_VELOCITY
  vel_t[1]=*(MphrN0->listevar.tab_var[MPHR_Y_VELOCITY].val.pr8); //MPHR_Y_VELOCITY
  vel_t[2]=*(MphrN0->listevar.tab_var[MPHR_Z_VELOCITY].val.pr8); //MPHR_Z_VELOCITY

  //-----------------------
  //@  acc :
  //@  res :
  //@  ierr : erreur
  //@ appel mo_orbit avec les arguments :
  //-----------------------
  status = mo_orbit(
    &mode_t, 
    mjdr_t, 
    xm_t, 
    mjdp_t,
    x_t, 
    pos_t, 
    vel_t, 
    acc_t, 
    res_orbit_t, 
    ierr_orbit_t);
    
  if (status != MO_OK)
  {
    func_id = MO_ORBIT_ID;
    //si erreur appel mo_vector_msg pour recuperation du libelle
    mo_vector_msg(&func_id, ierr_orbit_t, &n, msg);
    
    //@ on recupere le premier message d'erreur pour l'emission d'une trace
     cr_op   = SD_BAS_ERR_MP_ORBIT;
     SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_ORBIT,
      tcProcName_F,
      __LINE__,
      msg[0])
       goto fin;
      
    //@ on imprime les message d'erreurs sur la sortie standard
#ifdef OPS_DEBUG
    SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
    mo_print_msg(&n, msg);
    fprintf(stdout, "\n");
#endif
  }

  //-----------------------
  //@ 2 - propagation
  //@  mode : MO_PROPAG+MO_ORBIT_RES_BAS
  //-----------------------
  mode_t = MO_PROPAG+MO_ORBIT_RES_BAS;
  
  //-----------------------
  //@  mjdr : issu de 1
  //@  xm : issu de 1
  //@  mjdp : date du sous-cycle = IASITime (AC) - mjdr
  //-----------------------
  mjdp_t[0]=(IasiTime)->day+((double)(IasiTime)->millis/(1000.*86400.));
  mjdp_t[1]=dummy;

  //-----------------------
  //@  pos : a utiliser dans la suite
  //@  vel : a utiliser dans la suite
  //@  acc : a  utiliser dans la suite
  //@  res :
  //@  ierr :
  //@ appel mo_orbit avec les arguments :
  //-----------------------
  status = mo_orbit(
    &mode_t, 
    mjdr_t, 
    xm_t, 
    mjdp_t,
    x_t, 
    pos_t, 
    vel_t, 
    acc_t, 
    res_orbit_t, 
    ierr_orbit_t);
    
  //si erreur appel mo_vector_msg pour recuperation du libelle
  if (status != MO_OK)
  {
    func_id = MO_ORBIT_ID;
    //si erreur appel mo_vector_msg pour recuperation du libelle
    mo_vector_msg(&func_id, ierr_orbit_t, &n, msg);
    
    //@ on recupere le premier message d'erreur pour l'emission d'une trace
    cr_op   = SD_BAS_ERR_MP_ORBIT;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_ORBIT,
      tcProcName_F,
      __LINE__,
      msg[0])
     goto fin;
      
    //@ on imprime les message d'erreurs sur la sortie standard
#ifdef OPS_DEBUG
    SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
    mo_print_msg(&n, msg);
    fprintf(stdout, "\n");
#endif
  }
  
  //-----------------------
  //@ Pour chacun des points x y
  //-----------------------
  // correction validation : on travaille sans correction d'attitude
//CR 232 mise en place environnement variable
//  iatt=2;
//  iatt=3;
  aocs_g[0]= -0.1660;
  aocs_g[1]= 0.0508;
  aocs_g[2]= 3.9570;
  att[0] = *(MphrN0->listevar.tab_var[MPHR_PITCH_ERROR].val.pr8);  //  MPHR_PITCH_ERROR
  att[1] = *(MphrN0->listevar.tab_var[MPHR_ROLL_ERROR].val.pr8);   // MPHR_ROLL_ERROR
  att[2] = *(MphrN0->listevar.tab_var[MPHR_YAW_ERROR].val.pr8);    // MPHR_YAW_ERROR
  datt[0] = 0.0;        /* MetOp-1 SRAR mispointing rates [deg/s] */
  datt[1] = 0.0;
  datt[2] = 0.0;

  i=0;
  while ( i < nbpts )
  {
    //-----------------------
    //@   on appelle la fonction mp_stavis ([DA18])
    //@   Les arguments de cette fonction sont fournis par (cf. \ufffd7.1.3. de [DA18]) :
    //@     mjdp : date du sous-cycle = IASITime (issu de mo_orbit)
    //@     pos : position du satellite a la date mjdp obtenue par propagation des parametres orbitaux a la date mjdp ([DA19])
    //@      a l?aide de la fonction ESA mo_orbit et des parametres orbitaux fournis dans les entetes du niveau 0.
    //@      L?interface etant tres simple, elle n?est pas detaillee ici
    //@     vel : acceleration, fournie egalement par mo_orbit
    //@     acc : acceleration, fournie egalement par mo_orbit
    //@     iatt=3
    //-----------------------
    //-----------------------
    //@     aocs : valeur par defaut preconisees dans [DA18], de toute facon cela ne sert a rien
    //@     att : YAW_ERROR, ROLL_ERROR, PITCH_ERROR fournie dans l?entete du niveau 0
    //-----------------------
    //-----------------------
    //@     datt : 0
    //-----------------------
    //-----------------------
    //@ sta :
    //@     Sta[0] : latitude geodesique
    //@     Sta[1] : longitude geodesique
    //@     Sta[2] : altitude nulle
    //@     Sta[3] : masque d?elevation : -90 (parametre non utilise ici)
    //-----------------------
//  conversion en degre 
    sta[0] = lon[i]*SD_BAS_180/M_PI;               /* Ground station geocentric longitude [deg] */
//  conversion entre 0 et 360 
    if (sta[0] >= 0.)
      sta[0]=sta[0];
    else
    // erreur de signe
      sta[0]=(2*SD_BAS_180+sta[0]);
    sta[1] = lat[i]*SD_BAS_180/M_PI;       /* Ground station geodetic latitude [deg] */
    sta[2] = 0.0;          /* Ground station geodetic altitude [m] */
    sta[3] = -90.;         /* Ground station minimum elevation link [deg] */

    //@ appel fonction MP_Stavis
    status = mp_stavis(
      mjdp_t, 
      pos_t, 
      vel_t, 
      acc_t, 
      &iatt, 
      aocs_g, 
      att, 
      datt, 
      sta, 
      res_sta, 
      ierr_s);
      
    if (status != MP_OK)
    {
      func_id = MP_STAVIS_ID;
#ifdef OPS_DEBUG
				 	//@ si erreur appel mp_vector_msg pour recuperation du libelle
      mp_vector_msg(&func_id, ierr_s, &n, msg);
#endif
      //@ on recupere le premier message d'erreur pour l'emission d'une trace
    cr_op   = SD_BAS_ERR_MP_STAVIS;
      SD_BAS_TEST_ERR_INT(
        ctx->cr.errinterne,
        SD_BAS_ERR_SET_OpLoc,
        SD_BAS_ERR_MP_STAVIS,
        tcProcName_F,
        __LINE__,
        msg[0])
     goto fin;
        
      //@ on imprime les message d'erreurs sur la sortie standard
#ifdef OPS_DEBUG
      SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
      mp_print_msg(&n, msg);
      fprintf(stdout, "\n");
#endif
    }

    //-----------------------
    //@ Les sorties utiles de mp_stavis sont les parametres 3 et 4 :
    //@  elevation et azimuth satellite vers station, dont on deduit la direction
    //@ satellite-cible dans le repere METOP. ( cf FD )
    //-----------------------
// conversion en radian 
    azim[i]=res_sta[MP_STA_RES_SAT2STA_AZ_SRAR]*M_PI/SD_BAS_180;
    elev[i]=res_sta[MP_STA_RES_SAT2STA_EL_SRAR]*M_PI/SD_BAS_180;

    i++;
  }
  //@ fin Pour chacun des points x y

  // on trace ici si besoin toutes les variables en sortie

fin:
// sortie de la fonction
//if (status != MO_OK) cr_op=CMN__STATUT_NOK;
return(cr_op);
#else
  static const char tcProcName_F[] = "SD_BAS_GeodMetop";
  t_CMN__Statut cr_op = CMN__STATUT_OK;

  double time;
  double pos[3];
  double vel[3];  
  double att[3][3]; 
  int iatt;
  int status = 0;    

  status = navigation_ops_init();
  
  if( status != 0 ) {
    cr_op = SD_BAS_ERR_MP_ORBIT;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_ORBIT,
      tcProcName_F,
      __LINE__,
      "navigation_ops_init");
     goto fin;
  }

  time = (double)IasiTime->day + ( (double)(IasiTime)->millis / ( 1000. * 86400. ) );
  time += 18262;
  
  status = navigation_ops_intposvel( time, pos, vel );

  if( status != 0 ) {
    cr_op = SD_BAS_ERR_MP_ORBIT;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_ORBIT,
      tcProcName_F,
      __LINE__,
      "navigation_ops_intposvel");
     goto fin;
  }
  
  /* MetopLib */
  if( strcmp( s_METOP_ENV, METOP_ENV_TEST ) == 0 ) {
    iatt = 2; /* NOAA */
  } else {
    iatt = 3; /* METOP */
  }
  

  /* Translate to AAPP */  
  switch( iatt ) {
    case 2:
      iatt = 0;
    break;
    case 3:
      iatt = 1;
    break;
  }
  
  navigation_calatt( 
    *(MphrN0->listevar.tab_var[MPHR_YAW_ERROR].val.pr8),
    *(MphrN0->listevar.tab_var[MPHR_ROLL_ERROR].val.pr8),
    *(MphrN0->listevar.tab_var[MPHR_PITCH_ERROR].val.pr8),
    att
  );


  status = navigation_ops_earth_to_metop( 
   pos, vel, iatt,
   att, nbpts, lat, lon,
   azim, elev );
               
  if( status != 0 ) {
    cr_op = SD_BAS_ERR_MP_TARGET;
    SD_BAS_TEST_ERR_INT(
      ctx->cr.errinterne,
      SD_BAS_ERR_SET_OpLoc,
      SD_BAS_ERR_MP_TARGET,
      tcProcName_F,
      __LINE__,
      "navigation_ops_earth_to_metop");
    goto fin;
  }
  

fin:
  return cr_op;
#endif
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_GeodAVHRR
//@
//@ Description : recherche dans le bandeau AVHRR des pixels correspondants
//@     aux coordonnees geodesiques (lat, lon)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  lat : latitude (RADIAN)
//@         (ENTREE)  lon : longitude (RADIAN)
//@         (ENTREE)  indicedeb : indice de la premiere ligne AVHRR utile
//@         (ENTREE)  indicefin : indice de la derniere ligne AVHRR utile
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  coldeb : numero de col du premier pixel AVHRR a analyser
//@         (ENTREE)  colfin : numero de col du dernier pixel AVHRR a analyser
//@         (SORTIE)  col : numero de col du pixel AVHRR trouve (fractionnel)
//@         (SORTIE)  lig : numero de lig du pixel AVHRR trouve (fractionnel)
//@         (SORTIE)  date : date de la ligne AVHRR du pixel trouve
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_GeodAVHRR(
        t_SD_BAS_DataThread * ctx,
        int  nbpts,
        double * lat,
        double * lon,
        int indicedeb,
        int indicefin,
        t_SD_BAS_LigneAVHRR** BandeauAVHRR,
        int nbligbandeau,
        int coldeb,
        int colfin,
        double * col,
        double *lig,
        t_SD_BAS_SCDSTime * date
            )
{
static const char tcProcName_F[] = "SD_BAS_GeodAVHRR";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction
//@ On utilise les resultats produits par les appels a mp_target (cf. ci-dessus).
//@ On utilise les coordonnees geodesiques (latitude, longitude)
//@ et on va rechercher dans le produit Avhrr les pixels qui correspondent le mieux :
//@ -   IASITime est la date du SN courant, on va explorer les lignes AVHRR comprises entre
//@  IASITime - IDefDeltaAvhrrDTbefore(SN) et IDefDeltaAvhrrDTafter(SN)
//@ -   On restreint encore plus la recherche, en ne cherchant que les elements de colonne
//@ compris entre IDefDeltaAvhrrColLeft(SN) et IDefDeltaAvhrrColRight(SN).
//@ A priori ces valeurs seront fixees respectivement a 1 et 2048 (SD_BAS_NPI_AVHRR), et
//@ seront affinees par la suite pour eventuellement ameliorer les performances
//@ -   Le resultat est constitue de la date et du numero de colonne du point AVHRR dont les
//@  coordonnees (parametre EARTH_LOCATION dans le ADR-1B) correspondent le mieux.

    int il;             // Compteur sur les lignes du bandeau AVHRR
    int ic;             // Compteur sur les colonnes du bandeau AVHRR
    int i;              // Compteur sur le nombre de points
    int ligmin;         // Ligne AVHRR donnant la distance min
    int colmin;         // Colonne AVHRR donnant la distance min
    double dmin;        // Distance min precedente

  
    int ng_coldeb;      // Coldeb dans le tableau sousechantillonne
    int ng_colfin;      // Colfin dans le tableau sousechantillonne
    int ng_ligdeb;      // Ligdeb dans le tableau sousechantillonne
    int ng_ligfin;      // Ligfin dans le tableau sousechantillonne
    int nbcoltabsousech;// Nb de colonnes dans le tableau sousechantillonne
    int fin,taille; //variables temporaires
    double const1;
    double coslat,sinlat,coslon,sinlon;
    double x,y,z;
    double xav,yav,zav;
    int ilprec,ilsuiv;
    int il0,il1;
    int ic0,ic1;
    int interp11,interp12;
    int interp21,interp22;
    int interp31,interp32;
    int ligse[6*SOUSECH_AVHRR_COL+3];
    int colse[6*SOUSECH_AVHRR_COL+3];
    double latse[6*SOUSECH_AVHRR_COL+3];
    double lonse[6*SOUSECH_AVHRR_COL+3];
    int colentier,ligentier;
    double lattemp[10];
    double lontemp[10];
    

    // Attention : le tableau des latitudes et longitudes AVHRR est sousechantillonne
    // d'un facteur SOUSECH_AVHRR_COL pour chaque ligne du bandeau
    // sousech entre la colonne 25 et 2025 par pas de SOUSECH_AVHRR_COL (20 par defaut)

    //@ Calcul de la taille du tableau sousechantillonne
    nbcoltabsousech = SD_BAS_SIZE_OF_DATA_GEOLOC;


/*  CR257 */
// sousech entre la colonne 5 et 2045 par pas de SOUSECH_AVHRR_COL (20 par defaut)
    ng_coldeb = (coldeb - FIRST_AVHRR_COL)/SOUSECH_AVHRR_COL;
//FA 2OO
    ng_colfin = (colfin - FIRST_AVHRR_COL)/SOUSECH_AVHRR_COL + 1;
// ..FA 200
//    ng_coldeb = (coldeb - 25)/SOUSECH_AVHRR_COL;
//    ng_colfin = (colfin - 25)/SOUSECH_AVHRR_COL;
    ng_ligdeb = indicedeb;
    ng_ligfin = indicefin;

    //@ Si le bandeau sort des limites, on le recadre dans les limites
    if(ng_coldeb < 0) ng_coldeb=0;
    if(ng_colfin > nbcoltabsousech-1) ng_colfin=nbcoltabsousech-1;

    if(ng_ligdeb < 0) ng_ligdeb=0;
    if(ng_ligfin > nbligbandeau-1) ng_ligfin=nbligbandeau-1;
   const1 = (double)M_PI/(double)SD_BAS_180;
     for(i=0; i<nbpts; i++)
    {
        ligmin=0;
        colmin=0;
        dmin = 1e99;
	il=ng_ligdeb;
	ic=ng_coldeb;
// calcul de la position du point initial au sol
        coslat = cos(lat[i]);
        sinlat = sin(lat[i]);
        coslon = cos(lon[i]);
        sinlon = sin(lon[i]);
	
	x = coslat*coslon; 
	y = coslat*sinlon; 
	z = sinlat; 

//--------------------------------------------------------------------
// recherche du point le plus proche en projetant sur une sphere de rayon 1
//--------------------------------------------------------------------

// premiere passe : on ous echantillonne d'un facteur SOUSECH_AVHRR_LIG en lignes
	fin = 0;
	while ((il-ng_ligdeb)*(ng_colfin-ng_coldeb) +(ic-ng_coldeb) < (ng_ligfin-ng_ligdeb)*(ng_colfin-ng_coldeb) && fin == 0 )
	 {
                //@ Recuperation  des latitude et longitude AVHRR
                // Attention : ic est compris entre 0 et 2025 alors que le
                // bandeau est sousechantillonne
                coslat = cos(BandeauAVHRR[il]->Image.latitude[ic]*const1);
                sinlat = sin(BandeauAVHRR[il]->Image.latitude[ic]*const1);
                coslon = cos(BandeauAVHRR[il]->Image.longitude[ic]*const1);
                sinlon = sin(BandeauAVHRR[il]->Image.longitude[ic]*const1);
	        xav = coslat*coslon; 
	        yav = coslat*sinlon; 
	        zav = sinlat; 
		
		if (  (xav-x)*(xav-x) + (yav-y)*(yav-y) + (zav-z)*(zav-z) < dmin )
                 {
                    dmin = (xav-x)*(xav-x) + (yav-y)*(yav-y) + (zav-z)*(zav-z);
                    colmin = ic;
                    ligmin = il;
                }
		  
		
	       ic++;
	       if (ic==ng_colfin)
	        { ic=ng_coldeb; 
		if (il == ng_ligfin) 
		  {
		  fin = 1;
		  }
		else
		  {
		  il=il+SOUSECH_AVHRR_LIG;	
	          if (il > ng_ligfin) il = ng_ligfin ;
		  }
		}
	
	 }
//  fin premiere passe 

// deuxieme passe on traite sur un nombre de ligne  +-SOUSECH_AVHRR_LIG et sur 3 colonnes 
	// cas on ous echantillonne d'un facteur SOUSECH_AVHRR_COL en lignes
	fin = 0;
	il0=MAX(0,ligmin-SOUSECH_AVHRR_LIG);
	il1=MIN(ng_ligfin,ligmin+SOUSECH_AVHRR_LIG);
	il=il0;
	ic0=MAX(0,colmin-1);
	ic1=MIN(ng_colfin,colmin+1);
	ic=ic0;
        ligmin=0;
        colmin=0;
        dmin = 1e99;
	while ((il-il0)*(ic1-ic0) +(ic-ic0) < (il1-il0)*(ic1-ic0) )
	 {
                //@ Recuperation  des latitude et longitude AVHRR
                // Attention : ic est compris entre 0 et 2025 alors que le
                // bandeau est sousechantillonne
                coslat = cos(BandeauAVHRR[il]->Image.latitude[ic]*const1);
                sinlat = sin(BandeauAVHRR[il]->Image.latitude[ic]*const1);
                coslon = cos(BandeauAVHRR[il]->Image.longitude[ic]*const1);
                sinlon = sin(BandeauAVHRR[il]->Image.longitude[ic]*const1);
	        xav = coslat*coslon; 
	        yav = coslat*sinlon; 
	        zav = sinlat; 
		
		if (  (xav-x)*(xav-x) + (yav-y)*(yav-y) + (zav-z)*(zav-z) < dmin )
                 {
                    dmin = (xav-x)*(xav-x) + (yav-y)*(yav-y) + (zav-z)*(zav-z);
                    colmin = ic;
                    ligmin = il;
                 }
		  
		
	       ic++;
	       if (ic==ic1)
	        { ic=ic0; 
//correction BMA
 		  il++;
// 		  il=il++;	
		}
	
	 }
//  fin deuxieme passe 

// on est ici en bordure du bandeau
//FA160 le test est affine pour ne plus se plater dans image utile de ccs

        if ( colmin <= ng_coldeb ||  colmin >= ng_colfin -1
	  || ligmin <= ng_ligdeb || ligmin >=ng_ligfin -1)
	 {
	 // on est ici en bordure du bandeau
            cr_op = CMN__STATUT_NOK;
            //@ on recupere le premier message d'erreur pour l'emission d'une trace
            SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                                SD_BAS_ERR_SET_OpLoc,
                                SD_BAS_OpLoc_ERR_INTERP,
                                tcProcName_F,
                                __LINE__,
                                ERR_SEARCH)
            //@ on imprime les message d'erreurs sur la sortie standard
            #ifdef OPS_DEBUG
                SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERR_SEARCH);
            #endif
	    goto fin;
	 }
	 
       // A ce stade le point le plus pres est strictement compris dans le bandeau dans la geometrie sousechantillonnee
       // appel a la fonction d'interpolation dans le plan tengent
       colentier = colmin;
       ligentier = ligmin;
       cr_op=SD_BAS_InterpolLatLon(
       		ctx,
        	x,
        	y,
       		z,
       		colentier,
        	ligentier,
        	BandeauAVHRR,
        	nbligbandeau,
        	&col[i],
       		&lig[i]
                );
	// FT-3130
	if(lig[i] <= 0 ||  lig[i] >= nbligbandeau || col[i] <= 0 || col[i] >= 2045)
	{
		cr_op = CMN__STATUT_NOK;
	}
	// .. FT-3130
        if( cr_op != CMN__STATUT_OK)
  	{
            cr_op = CMN__STATUT_NOK;
            //@ on recupere le premier message d'erreur pour l'emission d'une trace
            SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                                SD_BAS_ERR_SET_OpLoc,
                                SD_BAS_OpLoc_ERR_INTERP,
                                tcProcName_F,
                                __LINE__,
                                ERR_SEARCH)
            //@ on imprime les message d'erreurs sur la sortie standard
            #ifdef OPS_DEBUG
                SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERR_SEARCH);
            #endif
	    goto fin;
  	}

        if( cr_op != CMN__STATUT_OK)
  	{
            cr_op = CMN__STATUT_NOK;
            //@ on recupere le premier message d'erreur pour l'emission d'une trace
            SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                                SD_BAS_ERR_SET_OpLoc,
                                SD_BAS_OpLoc_ERR_INTERP,
                                tcProcName_F,
                                __LINE__,
                                ERR_SEARCH)
            //@ on imprime les message d'erreurs sur la sortie standard
            #ifdef OPS_DEBUG
                SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERR_SEARCH);
            #endif
	    goto fin;
  	}
 
 
 

//--------------------------------------------------------------------
// fin recherche du point le plus proche en projetant sur une sphere de rayon 1
//--------------------------------------------------------------------

    	// le resultat est mis dans la variable de sortie
	 
/*  CR257 */
// sousech entre la colonne 5 et 2045 par pas de SOUSECH_AVHRR_COL (20 par defaut)
        col[i] = col[i] + FIRST_AVHRR_COL;
//        col[i] = col[i] + 25;
	
// test retour

//       cr_op= SD_BAS_AVHRRGeod(
//        ctx,
//        1,
//        BandeauAVHRR,
//        nbligbandeau,
//        &col[i],
//        &lig[i],
//        lattemp,
//        lontemp
//            );





//   if (ctx->ctxtrt.SN == 1 || ctx->ctxtrt.SN == 15 || ctx->ctxtrt.SN == 30)
//    {
//    printf("SN: %d col()= %lf lig()= %lf \n",ctx->ctxtrt.SN,col[i] ,lig[i]);
//    printf("SN: %d latinit(deg)= %lf loninit(deg)= %lf \n",ctx->ctxtrt.SN,lat[i]*(double)SD_BAS_180/M_PI ,lon[i]*(double)SD_BAS_180/M_PI);
//    printf("SN: %d latinit(deg)= %lf loninit(deg)= %lf \n",ctx->ctxtrt.SN,lattemp[0]*(double)SD_BAS_180/M_PI ,lontemp[0]*(double)SD_BAS_180/M_PI);
//    printf("SN: %d ecartlat(deg)= %lf ecartlon(deg)= %lf ecartlon(m)= %lf  ecartlat(m)= %lf\n",ctx->ctxtrt.SN,(lattemp[0]-lat[i])*(double)SD_BAS_180/M_PI ,(lontemp[0]-lon[i])*(double)SD_BAS_180/M_PI,cos(lat[i])*6300000*(lontemp[0]-lon[i]),6300000*(lattemp[0]-lat[i]));
//    }
   }
fin:
// sortie de la fonction
return(cr_op);

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_AVHRRGeod
//@
//@ Description : conversion de coordonnees AVHRR fractionnaires en
//@               coordonnees geodesiques lat,lon)
//@
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  col : colonnes fractionnaires des pixels AVHRR
//@         (ENTREE)  lig : lignes fractionnaires des pixels AVHRR
//@         (SORTIE)  lat : latitudes correspondant aux points AVHRR (RADIAN)
//@         (SORTIE)  lon : longitudes correspondant aux points AVHRR (RADIAN)
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_AVHRRGeod(
        t_SD_BAS_DataThread * ctx,
        int  nbpts,
        t_SD_BAS_LigneAVHRR** BandeauAVHRR,
        int nbligbandeau,
        double * col,
        double *lig,
        double *lat,
        double *lon
            )
{
static const char tcProcName_F[] = "SD_BAS_AVHRRGeod";
t_CMN__Statut cr_op=CMN__STATUT_OK;

    int pt;
    double ncol,nlig;            // Indice de colonne fractionnaire dans le tableau sousechantillonne
    double ncolinterp;

    int xp[9],yp[9];
    // Latitudes et longitudes de ces points
     double t,u;
    double latavhrr[9];
    double lonavhrr[9];
    double colavhrr[9];
    double ligavhrr[9];
    int ct;
    double coslat,sinlat;
    double coslon,sinlon;
    double xav[9];
    double yav[9];
    double zav[9];
    double xPpA,yPpA,zPpA;
    double normePpA;
    double xorthoav[9];
    double yorthoav[9];
    double zorthoav[9];
    double coef,invcoeff;
    double i1[3];
    double j1[3];
    double coefij;
    double i1vectj1[3];
    int choix;
    double ui1orthoav[9];
    double vj1orthoav[9];
    double up,vp;
    double tl,tc;
    double opx,opy,opz;
    double OPvecti1j1[3];
    double const1;
    int nbcoltabsousech;// Nb de colonnes dans le tableau sousechantillonne
    double c[100];
    double xe1[3],xe2[3],ye[9];
    int nc,ne,i;
    int il,ic;
//FA160
   char *msg_err;          // Message d'erreur
    int num_lig;
// Entree dans la fonction
    const1 = (double)M_PI/(double)SD_BAS_180;


    // Attention : le tableau des latitudes et longitudes AVHRR est sousechantillonne
    // d'un facteur SOUSECH_AVHRR_COL pour chaque ligne du bandeau
    // sousech entre la colonne 25 et 2025 par pas de SOUSECH_AVHRR_COL (20 par defaut)


    // Calcul de la taille du tableau sousechantillonne
/*  CR257 */
// sousech entre la colonne 5 et 2045 par pas de SOUSECH_AVHRR_COL (20 par defaut)
    nbcoltabsousech = SD_BAS_SIZE_OF_DATA_GEOLOC;
//    nbcoltabsousech = 2000/SOUSECH_AVHRR_COL;


    //@ Boucle sur le nombre de points a traiter
    for(pt=0; pt<nbpts; pt++)
    {

        //@ Calcul de la positionfractionnaire dans le tableau des lat et lon sousechantillonnees AVHRR
/*  CR257 */
// sousech entre la colonne 5 et 2045 par pas de SOUSECH_AVHRR_COL (20 par defaut)
        ncol = (col[pt] - FIRST_AVHRR_COL)/SOUSECH_AVHRR_COL;
//        ncol = (col[pt] - 25)/SOUSECH_AVHRR_COL;
        nlig=lig[pt];
        // FT-3130
	if ( (int)ncol <= 0 ||  (int)ncol >= SD_BAS_SIZE_OF_DATA_GEOLOC -1 
	  || (int)nlig <= 0 || (int)nlig >=nbligbandeau-1)
	//if ( ncol <= 0 ||  ncol >= SD_BAS_SIZE_OF_DATA_GEOLOC -1 
	//  || nlig <= 0 || nlig >=nbligbandeau-1)
        // FT-3130
	 {
	 // on est ici en bordure du bandeau

            cr_op = CMN__STATUT_NOK;
            //@ on recupere le premier message d'erreur pour l'emission d'une trace
            SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                                SD_BAS_ERR_SET_OpLoc,
                                SD_BAS_OpLoc_ERR_INTERP,
                                tcProcName_F,
                                __LINE__,
                                ERR_SEARCH)
            //@ on imprime les message d'erreurs sur la sortie standard
            #ifdef OPS_DEBUG
                SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERR_SEARCH);
            #endif
	    // FT-3130
	    msg_err = "Bordure de bandeau";
	    num_lig = __LINE__;
     	    // FT-3130
	    goto fin;
	 }

        //@ Recuperation des des 4 points du tableau sousechantillonne utilises pour l'interpolation
        // A ce stade colmin contient la colonne dans la geometrie sousechantillonnee
        // Rappel : la ligne sousechantillonnee contient nbcoltabsousech pixels utiles

	// 1 : on extrait tout d'abord du bandeau AVHRR les 8 points encadrant le point entier lle plus proche
	//
	//   1		2	3
	//        C1       C2
	//   4	   ptentier (5) 6	
	//        C3       C4
	//   7		8	9
  	for (il=(int)nlig-1,ct=1;il<=(int)nlig+1;il++)
   	{
  	  for (ic=(int)ncol-1;ic<=(int)ncol+1;ic++,ct++)
  	   {
           latavhrr[IND1(ct)]=BandeauAVHRR[il]->Image.latitude[ic]*const1;
           lonavhrr[IND1(ct)]=BandeauAVHRR[il]->Image.longitude[ic]*const1;
  	   colavhrr[IND1(ct)]=ic;
  	   ligavhrr[IND1(ct)]=il;
  	   }
   	}

 	// le cas de la singularite en latitude  n'est pas traite (cela revient a ne pas interpoler localement )


      // 2 : on determine les coordonnees des 4 points  dans le repere terrestre (sphere unite)
  	for (ct=1;ct<=9;ct++)
   	{
    	  coslat = cos(latavhrr[IND1(ct)]);
  	  sinlat = sin(latavhrr[IND1(ct)]);
  	  coslon = cos(lonavhrr[IND1(ct)]);
 	  sinlon = sin(lonavhrr[IND1(ct)]);
  	  xav[IND1(ct)]=coslon*coslat;
 	  yav[IND1(ct)]=sinlon*coslat;
 	  zav[IND1(ct)]=sinlat;
    	}

	// 3 : projection dans le referentiel  stereoscopique au point 5
	// interpolation des u v dans le referentiel stereoscopique
	//     on se place ici dans le repere terrestre 
	//    les point obtenus sont appeles P1 P2 ... P9
  	for (ct=1;ct<=9;ct++)
   	{
 	  xorthoav[IND1(ct)]=0.;
 	  xorthoav[IND1(ct)]=0.;
 	  xorthoav[IND1(ct)]=0.;
   	// le point central a pour coordonnees 0, 0
 	  if (ct != 5)
 	   {
	// calcul du vecteur P'A dans le referentiel terrestre
	// P'A = OP + OA
 	    xPpA= xav[IND1(5)] + xav[IND1(ct)];   
 	    yPpA= yav[IND1(5)] + yav[IND1(ct)];   
  	    zPpA= zav[IND1(5)] + zav[IND1(ct)]; 
  	    normePpA =  xPpA*xPpA + yPpA*yPpA + zPpA*zPpA;
  	    coef = 4./normePpA;
	// calcul du vecteur PB dans le referentiel terrestre
	// PB = -2OP + P'A * 4/||P 'A||^2
  	     xorthoav[IND1(ct)]= -2 * xav[IND1(5)] + coef * xPpA;
 	     yorthoav[IND1(ct)]= -2 * yav[IND1(5)] + coef * yPpA;
 	     zorthoav[IND1(ct)]= -2 * zav[IND1(5)] + coef * zPpA;
 	   }
 	}
	
	// 4 : on determine de nouveaux vecteurs utilise comme base du plan tangent i1 et j1  a partir de la formule:
	//    j1 = P5P8 / || P5P8||
        coef = sqrt(xorthoav[IND1(8)]*xorthoav[IND1(8)] + yorthoav[IND1(8)]*yorthoav[IND1(8)] + zorthoav[IND1(8)]*zorthoav[IND1(8)]);
        invcoeff= 1./coef;
        j1[0]= xorthoav[IND1(8)]*invcoeff;
        j1[1]= yorthoav[IND1(8)]*invcoeff;
        j1[2]= zorthoav[IND1(8)]*invcoeff; 
	//    on en deduit les coordonnees de  P8 dans ce nouveau repere (a , b)  : 
	//    P8 ( || P5P8||,0)
 	ui1orthoav[IND1(8)]=0.;
 	vj1orthoav[IND1(8)]=coef;
	//    i1 = P5P6 / || P5P6||
 	coef = sqrt(xorthoav[IND1(6)]*xorthoav[IND1(6)] + yorthoav[IND1(6)]*yorthoav[IND1(6)] + zorthoav[IND1(6)]*zorthoav[IND1(6)]);
 	invcoeff= 1./coef;
 	i1[0]= xorthoav[IND1(6)]*invcoeff;
 	i1[1]= yorthoav[IND1(6)]*invcoeff;
 	i1[2]= zorthoav[IND1(6)]*invcoeff; 
	//    on en deduit les coordonnees de  P6 dans ce nouveau repere (a , b)  : 
	//    P6 ( || P5P6||,0)
        ui1orthoav[IND1(6)]=coef;
        vj1orthoav[IND1(6)]=0.;
 

	// 5 : pour les point  1 2 3 4 7 9   on le projette dans la projection locale stereoscopique associe au point 5
	//     on se place ici dans le repere i1, j1 
	//   OP = u i1 + v j1, le calcul de a et b se realise en prenant le produit vectoriel
 	i1vectj1[0]=(i1[1]*j1[2] - i1[2]*j1[1]);
 	i1vectj1[1]=(i1[2]*j1[0] - i1[0]*j1[2]);
 	i1vectj1[2]=(i1[0]*j1[1] - i1[1]*j1[0]);
 	if ( fabs(i1vectj1[0]) >= 0.01 ) 
 	 {
	  coefij= 1./i1vectj1[0];
	  choix=0;
 	 }
	else if ( fabs(i1vectj1[1]) >= 0.01  )
	 {
 	 coefij= 1./i1vectj1[1];
	  choix=1;
	 }
	else
	 {
 	 coefij= 1./i1vectj1[2];
 	 choix=2;
	 }
	vj1orthoav[IND1(5)]=0.;
	ui1orthoav[IND1(5)]=0.;
 	for (ct=1;ct<=9;ct++)
  	 {
   
   	// le point central a pour coordonnees 5
  	 if (ct != 6 && ct != 8 && ct != 5)
    	  {
	//  OPx ^ i1 = v j1 ^ i1   d'ou v = OPx ^ i1 / j1 ^ i1
  	   OPvecti1j1[0]=(yorthoav[IND1(ct)]*i1[2] - zorthoav[IND1(ct)]*i1[1]);
  	   OPvecti1j1[1]=(zorthoav[IND1(ct)]*i1[0] - xorthoav[IND1(ct)]*i1[2]);
 	   OPvecti1j1[2]=(xorthoav[IND1(ct)]*i1[1] - yorthoav[IND1(ct)]*i1[0]);
  	   vj1orthoav[IND1(ct)]= OPvecti1j1[choix]*-coefij;
	//  OPx ^ j1 = u i1 ^ j1   d'ou u =  OPx ^ j1 / i1 ^ j1
  	   OPvecti1j1[0]=(yorthoav[IND1(ct)]*j1[2] - zorthoav[IND1(ct)]*j1[1]);
 	   OPvecti1j1[1]=(zorthoav[IND1(ct)]*j1[0] - xorthoav[IND1(ct)]*j1[2]);
 	   OPvecti1j1[2]=(xorthoav[IND1(ct)]*j1[1] - yorthoav[IND1(ct)]*j1[0]);
  	   ui1orthoav[IND1(ct)]= OPvecti1j1[choix]*coefij;
    	  }
   	 }
	

	// 6 :l'interpolation spline2D en u   est alors menee sur la zone
	// ne= 3
 	ne=3;
	// xe1= ligavhrr [ 1,2,3]
	// xe2= colavhrr [ 1,4,7]
 	for (i=0;i<3;i++)
 	 {
  	  xe1[i]=colavhrr[i];
//FA160 test sur la croissane des tableaux
	  if (i>0 && xe1[i] <= xe1[i-1])
	   {
            cr_op   = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
            msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
            num_lig = __LINE__;
       	    goto fin;
	   }
 	  xe2[i]=ligavhrr[i*3+1];
//FA160
	  if (i>0 && xe2[i] <= xe2[i-1])
	   {
            cr_op   = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
            msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
            num_lig = __LINE__;
       	    goto fin;
	   }
  	 }
	// ye = ui1orthoav [1 2 3 4 5 6 7 8 9]
	// nc = 11 ne+3
 	nc=11*ne+3;
	// c
	// xs1 = ncol
	// ys1 = nlig
	// ys = up
 	cr_op= SD_BAS_InterpSpline2D(
      		ne,
        	xe1,
        	xe2,
        	ui1orthoav,
        	nc,
        	c,
        	&ncol,
        	&nlig,
        	&up);
 	if(cr_op != CMN__STATUT_OK)
    	 {
    		// FT-3130
		msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
    		cr_op = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
		num_lig = __LINE__;
		// .. FT-3130
    	 	goto fin;
    	 }
	// 6 bis:l'interpolation spline2D en v   est alors menee sur la zone
	// ne= 3
 	ne=3;
	// xe1= colavhrr [ 1,2,3]
	// xe2= ligavhrr [ 1,4,7]
 	for (i=0;i<3;i++)
 	 {
  	  xe1[i]=colavhrr[i];
//FA160 test sur la croissance des valeurs
	  if (i>0 && xe1[i] <= xe1[i-1])
	   {
            cr_op   = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
            msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
            num_lig = __LINE__;
       	    goto fin;
	   }
  	  xe2[i]=ligavhrr[i*3+1];
//FA160
	  if (i>0 && xe2[i] <= xe2[i-1])
	   {
            cr_op   = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
            msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
            num_lig = __LINE__;
       	    goto fin;
	   }
 	 }
	// ye = ui1orthoav [1 2 3 4 5 6 7 8 9]
	// nc = 11 ne+3
 	nc=11*ne+3;
	// c
	// xs1 = ncol
	// ys1 = nlig
	// ys = up
 	cr_op= SD_BAS_InterpSpline2D(
      		ne,
        	xe1,
        	xe2,
        	vj1orthoav,
        	nc,
        	c,
        	&ncol,
        	&nlig,
        	&vp);
 	if(cr_op != CMN__STATUT_OK)
    	 {
    		// FT-3130
		msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
	    	cr_op = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
		num_lig = __LINE__;
		// .. FT-3130
	 	goto fin;
    	 }
	
	//  7 : retour en latitude longitude par la formule:
	//        P5P =  u i1 + v j1
	//	  P5P= P5 O + O P  d'ou OP = P5P + OP5 = up i1 + vp j1 + OP5   
        opx=up*i1[0] + vp*j1[0] + xav[IND1(5)];
        opy=up*i1[1] + vp*j1[1] + yav[IND1(5)];
        opz=up*i1[2] + vp*j1[2] + zav[IND1(5)];

	//        le retour en latitude logitude se fait par 
	// lat = atan(z/sqrt(x2+y2))   et du meme signe que z ( x > 0 lon >0 x < lon < 0)
	// lon = atan (y/x) et du meme signe que x ( x > 0 lon >0 x < lon < 0)
         lon[pt] = atan2 (opy,opx);
         lat[pt] =  atan(opz/sqrt(opx*opx+opy*opy)) ;
        	
	
	// 

    }

fin:
//FA160
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_OpLoc,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err)

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }
// sortie de la fonction
return(cr_op);

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_MetopAVHRR
//@
//@ Description : Conversion de coordonnees Metop -> raster AVHRR
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape : gestion des erreurs
//@         (ENTREE)  IasiTime[2] : date du SN traite
//@         (ENTREE)  Mphr :  MPHR produit N0
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  azim[nbpts] : coordonnees des pts dans le repere METOP (azim)
//@         (ENTREE)  elev[nbpts] : coordonnees des pts dans le repere METOP (elev)
//@         (ENTREE)  indicedeb : indice de la premiere ligne AVHRR utile
//@         (ENTREE)  indicefin : indice de la derniere ligne AVHRR utile
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  AvhrrColLeft : numero de col du premier pixel AVHRR a analyser
//@         (ENTREE)  AvhrrColRight : numero de col du dernier pixel AVHRR a analyser
//@         (SORTIE)  lat : latitude
//@         (SORTIE)  lon : longitude
//@         (SORTIE)  Ang_mesY : angles de mesure METOP
//@         (SORTIE)  Ang_mesZ : angles de mesure METOP
//@         (SORTIE)  Azim_sol : angles solaire
//@         (SORTIE)  Elev_sol : angles solaire
//@         (SORTIE)  ColAVHRR : Colonnes AVHRR correspondant aux points metop (fraction de pixel)
//@         (SORTIE)  LigAVHRR : Lignes AVHRR correspondant aux points metop (fraction de pixel)
//@         (SORTIE)  EarthSatelDist : disance Terre/Satellite
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_MetopAVHRR
(
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_SCDSTime * IasiTime,
    t_SD_BAS_MPHR * MphrN0,
    int  nbpts,
    double * azim,
    double * elev,
    int indicedeb,
    int indicefin,
    t_SD_BAS_LigneAVHRR** BandeauAVHRR,
    int nbligbandeau,
    int AvhrrColLeft,
    int AvhrrColRight,
    double *lat,              // Sortie SD_BAS_MetopGeod
    double *lon,              // Sortie SD_BAS_MetopGeod
    double *Ang_mesY,         // Sortie SD_BAS_MetopGeod
    double *Ang_mesZ,         // Sortie SD_BAS_MetopGeod
    double *Azim_sol,         // Sortie SD_BAS_MetopGeod
    double *Elev_sol,         // Sortie SD_BAS_MetopGeod
    double *ColAVHRR,
    double *LigAVHRR,
    double *EarthSatelDist
)
{
    static const char tcProcName_F[] = "SD_BAS_MetopAVHRR";
    t_CMN__Statut cr_op=CMN__STATUT_OK;


    t_SD_BAS_SCDSTime *date;  // Sortie SD_BAS_GeodAVHRR

    // Gestion de erreurs
    t_CMN__Statut irc;      // Statuts de sortie des fonctions appelees
    char *msg_err;          // Message d'erreur
    int num_lig;
    int i;
    
    #ifdef OPS_DEBUG
        printf("\nEntree dans la fonction SD_BAS_MetopAVHRR\n=====================================\n");
    #endif
    date=NULL;

    // Allocation de la memoire
    if(SD_BAS_MacMALLOC(date,t_SD_BAS_SCDSTime,nbpts) == FALSE)
    {
        cr_op   = SD_ALG_OPLOC_ERR_ALLOCATION;
        msg_err = (char *)SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;

        goto fin;
    }

    //@ Conversion METOP => Geodesique
    #ifdef OPS_DEBUG
            printf("\nAppel de SD_BAS_MetopGeod\n");
    #endif
   SD_BAS_ReserverOperateur();
   #ifdef OPS_DEBUG
        printf("pb call SD_BAS_MetopGeod from SD_BAS_OperateurLoc line %d \n",__LINE__);
        for(i=0; i<nbpts; i++)
        {
            printf("pb  point %d  azim %lf elev %lf \n",i,azim[i],elev[i]);
        }
    #endif

     irc = SD_BAS_MetopGeod(ctx,IasiTime,MphrN0,nbpts,azim,elev,
                           lat,lon,Ang_mesY,Ang_mesZ,Azim_sol,Elev_sol,
                           EarthSatelDist);

   SD_BAS_LibererOperateur();
    #ifdef OPS_DEBUG
    if(irc != CMN__STATUT_OK)
    {
        printf("pb  erreur SD_BAS_MetopGeod => Code retour %d\n",irc);
    }
    #endif
    
    if(irc != CMN__STATUT_OK)
    {
        cr_op   = SD_BAS_OPLOC_ERR_METOP2GEOD;
        msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_METOP2GEOD;
        num_lig = __LINE__;
	goto fin;
    }
    

    #ifdef OPS_DEBUG
         printf("SD_BAS_MetopGeod => Code retour ");
         if(irc != CMN__STATUT_OK) printf(" NOK\n");
         else printf("OK\n");
    #endif


    //@ Geodesique => AVHRR
    #ifdef OPS_DEBUG
            printf("\nAppel de SD_BAS_GeodAVHRR\n");
    #endif
    irc = SD_BAS_GeodAVHRR(ctx,nbpts,lat,lon,indicedeb,indicefin,BandeauAVHRR,nbligbandeau,
                            AvhrrColLeft,AvhrrColRight,ColAVHRR,LigAVHRR, date);

    if(irc != CMN__STATUT_OK)
    {
        cr_op   = SD_BAS_OPLOC_ERR_GEOD2AVHRR;
        msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_GEOD2AVHRR;
        num_lig = __LINE__;
	goto fin;
    }
    #ifdef OPS_DEBUG
         printf("SD_BAS_MetopGeod => Code retour ");
         if(irc != CMN__STATUT_OK) printf(" NOK\n");
         else printf("OK\n");
    #endif

fin:
   SD_BAS_MacFREE(date);
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_OpLoc,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err)

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

// sortie de la fonction
return(cr_op);
}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_AVHRRMetop
//@
//@ Description : Conversion de coordonnees AVHRR (fractionnaires) -> Metop
//@
//@ Parameters :
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE)  IasiTime[2] : date du SN traite
//@         (ENTREE)  MphrN0 :  MPHR produit N0
//@         (ENTREE)  nbpts : nombre de points a traiter
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (ENTREE)  col : colonnes fractionnaires des pixels AVHRR
//@         (ENTREE)  lig : lignes fractionnaires des pixels AVHRR
//@         (SORTIE)  azim : azim Metop
//@         (SORTIE)  elev : elev Metop
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_AVHRRMetop
(
    t_SD_BAS_DataThread * ctx,
    t_SD_BAS_SCDSTime * IasiTime,
    t_SD_BAS_MPHR * MphrN0,
    int  nbpts,
    t_SD_BAS_LigneAVHRR** BandeauAVHRR,
    int nbligbandeau,
    double * col,
    double * lig,
    double * azim,
    double * elev
)
{
    static const char tcProcName_F[] = "SD_BAS_MetopAVHRR";
    t_CMN__Statut cr_op=CMN__STATUT_OK;


    // Gestion de erreurs
    t_CMN__Statut irc;      // Statuts de sortie des fonctions appelees
    char *msg_err;          // Message d'erreur
    int num_lig;

    // Variables internes
    double *lat;        // Sorties de SD_BAS_AVHRRGeod
    double *lon;        // Sorties de SD_BAS_AVHRRGeod
    lat=NULL;
    lon=NULL;
    // Allocation de la memoire
    if(    SD_BAS_MacMALLOC(lat,double,nbpts) == FALSE
        || SD_BAS_MacMALLOC(lon,double,nbpts) == FALSE
      )
    {
        cr_op   = SD_ALG_OPLOC_ERR_ALLOCATION;
        msg_err = (char *)SD_BAS_MSG_ALLOC;
        num_lig = __LINE__;
    }

    //@ Conversion AVHRR -> geodesique
    irc =  SD_BAS_AVHRRGeod(ctx,nbpts,BandeauAVHRR,nbligbandeau,col,lig,lat,lon);
    if(irc != CMN__STATUT_OK)
    {
        cr_op   = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
        msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_AVHRR2GEOD;
        num_lig = __LINE__;

        goto fin;
    }

    //@ Conversion geodesique -> Metop
   SD_BAS_ReserverOperateur();
    irc = SD_BAS_GeodMetop(ctx,IasiTime,MphrN0,nbpts,lat,lon,azim,elev);
   SD_BAS_LibererOperateur();
    if(irc != CMN__STATUT_OK)
    {
        cr_op   = SD_BAS_OPLOC_ERR_GEOD2METOP;
        msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_GEOD2METOP;
        num_lig = __LINE__;
	goto fin;
    }



fin:
   SD_BAS_MacFREE(lat);
   SD_BAS_MacFREE(lon);
    // Traitement des erreurs
    //================================================================
    
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_OpLoc,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err)

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }

// sortie de la fonction
return(cr_op);

}

// FT-2997
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_MetopAVHRR_SansAVHRR
	//@
	//@ Description : Conversion de coordonnees Metop -> raster AVHRR
	//@
	//@ Parameters :
	//@         (ENTREE)  ctx : contexte de l'etape : gestion des erreurs
	//@         (ENTREE)  nbpts : nombre de points a traiter
	//@         (ENTREE)  IasiTime[2] : date du SN traite
	//@         (ENTREE)  MphrN0 :  MPHR produit N0
	//@         (ENTREE)  azimuth : coordonnees des pts dans le repere METOP (azim)
	//@         (ENTREE)  elevation : coordonnees des pts dans le repere METOP (elev)
	//@         (ENTREE)  TimeFirst : date de debut (IasiTime - DeltaTime)
	//@         (ENTREE)  TimeLast : date de fin (IasiTime + DeltaTime)
	//@         (ENTREE)  DateFirstLineAvhrr : nombre de lignes total du bandeau
	//@         (SORTIE)  AvhrrCol[nbpts] : Colonnes AVHRR correspondant aux points metop (fraction de pixel)
	//@         (SORTIE)  AvhrrLin[nbpts] : Lignes AVHRR correspondant aux points metop (fraction de pixel)
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_MetopAVHRR_SansAVHRR
	(
	    // Entrees
	    t_SD_BAS_DataThread * ctx,
	    int  nbpts,
	    t_SD_BAS_SCDSTime * IasiTime,
	    t_SD_BAS_MPHR * MphrN0,
    	double * azimuth,
    	double * elevation,
	    double TimeFirst,
	    double TimeLast,
	    double DateFirstLineAvhrr,
	    
	    // Sorties
	    double * AvhrrLin, 
	    double * AvhrrCol   
	)
	{
	#ifdef OPS_USE_METOPLIB
	    static const char tcProcName_F[] = "SD_BAS_MetopAVHRR_SansAVHRR";
	    t_CMN__Statut cr_op=CMN__STATUT_OK;

	    double EarthSatelDist;
	    double lat;
	    double lon;
	    double alt;
	    double AvhrrTime;
	    double AvhrrScanAngle;
	    int i;
	    
	    double dummy = 0.0;
	    	    
		/* Declaration des variables pour les appels a mo_orbit */
		long mode_t;

	    double mjdr_t[2];
	    double xm_t[6];
	    double x_t[6];

		double mjdp_t[2];
		double pos_t[3];
		double vel_t[3];		// position et vitesse au noeud acendant de l'orbite
		double acc_t[3];
		double res_orbit_t[54]; // valeurs temporaires (acceleration)

		/* Declaration des variables pour les appels a mp_target */
	  	double aocs_g[3];
	  	long iatt;            	// parametre d'activation de mp_target
	  	double att[3];
	  	double datt[3];
	  	long idir,iray,ieres; 	// parametres d'activation de mp_target
	  	double dir[8];
	  	double freq;            // parametre d'activation de mp_target
	  	double res_tar[75];     // resultat d'activation de mp_target

	    // Gestion de erreurs
  		long status;            // Main status flag
  		long func_id;           // CFI function ID
		long ierr_orbit_t[4];
		long ierr_tar[15];       // mode d'activation et status vector  		
  		long n;                 // Number of error messages
  		t_SD_BAS_MetopMSG * msg;// Error messages vector
  			
	    #ifdef OPS_DEBUG
	        printf("\nEntree dans la fonction SD_BAS_MetopAVHRR_SansAVHRR MetOpLib\n=====================================\n");
	    #endif
	    
		//-----------------------
		// code sart here
		//-----------------------		
  		msg=ctx->ptmsg;

		mo_silent(); 
		mp_silent();
  
		//-----------------------
  		//@ 1 - initialisation des paramA�tres  
  		//@ d'entrA�e pour l'appel A� mo_orbit
  		//@ call SD_BAS_Init_mo_orbit
  		//-----------------------
		cr_op = SD_BAS_Init_mo_orbit(// Entrees
						  			 ctx,
						  			 MphrN0,
							         // Sorties
								 	 &iatt,
							         mjdr_t,
							         xm_t,
							         x_t,
							   		 res_orbit_t,
									 acc_t     
							         );
        if( cr_op != CMN__STATUT_OK)
  		{
            cr_op = CMN__STATUT_NOK;
            
            //@ on recupere le premier message d'erreur pour l'emission d'une trace
            SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                                SD_BAS_ERR_SET_OpLoc,
                                SD_BAS_OPLOC_ERR_METOP2GEOD,
                                tcProcName_F,
                                __LINE__,
                                SD_BAS_OPLOC_MSG_ERR_INITMOORBIT);

            //@ on imprime les message d'erreurs sur la sortie standard
            #ifdef OPS_DEBUG
                SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, SD_BAS_OPLOC_MSG_ERR_INITMOORBIT);
            #endif

	    	goto fin;
  		}

		//-----------------------
        //@ 2 - propagation
	  	//@ Call mo_orbit with propagation mode :
	  	//@ MO_PROPAG + MO_ORBIT_RES_BAS + MO_ORBIT_RES_AUX
	  	//-----------------------
	  	mode_t = MO_PROPAG + MO_ORBIT_RES_BAS + MO_ORBIT_RES_AUX;		  
	  	//-----------------------
	  	//@  mjdr_t : from initialisation mode
	  	//@  xm_t : from initialisation mode
	  	//@  mjdp_t : date du sous-cycle = IASITime (AC) - mjdr
	  	//-----------------------
	  	mjdp_t[IND1(1)]= (double)IasiTime->day+((double)(IasiTime)->millis/(1000.*86400.));
	  	mjdp_t[IND1(2)]= dummy;
		
	  	//-----------------------
		//@ x_t : from initialisation mode
	  	//@ pos_t : used below
	  	//@ vel_t : used below
	  	//@ acc_t : used below
	  	//@ res_orbit_t :
	  	//@ ierr_orbit_t :
	  	//-----------------------
	  	
	  	//-----------------------
	  	//@ call mo_orbit with propagation mode
	  	//-----------------------
	  	status = mo_orbit(	&mode_t, 
	    				  	mjdr_t, 
	    					xm_t, 
	    					mjdp_t,
	    					x_t, 
	    					pos_t, 
	    					vel_t, 
	    					acc_t, 
	    					res_orbit_t, 
	    					ierr_orbit_t);
	    
	  	// mo_orbit error management
	  	if (status != MO_OK)
	  	{
	    	func_id = MO_ORBIT_ID;
	    	// si erreur appel mo_vector_msg pour recuperation du libelle
	    	status = mo_vector_msg(&func_id, ierr_orbit_t, &n, msg);
	    
	    	//@ on recupere le premier message d'erreur pour l'emission d'une trace
	    	cr_op   = SD_BAS_ERR_MP_ORBIT;
	    	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
	      						SD_BAS_ERR_SET_OpLoc,
	      						SD_BAS_ERR_MP_ORBIT,
	      						tcProcName_F,
	      						__LINE__,
	      						msg[0]);
	      						
			#ifdef OPS_DEBUG
		    	//@ print error message on standard output
	    		SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
	    		status = mo_print_msg(&n, msg);
	    		fprintf(stdout, "\n");
			#endif
	      						
	     	goto fin;		      
	  	}	  	
		// output : orbit radius
		EarthSatelDist = res_orbit_t[MO_ORBIT_RES_ORBIT_RAD];
		//-----------------------
	  	//@ Initialise input parameters for call to mp_target
    	//@ The parameters of this function are provided in Â§7.1.3. of [DA18]) :
    	//@ aocs : valeur par defaut preconisees dans [DA18], de toute facon cela ne sert a rien
	  	//-----------------------
  		aocs_g[IND1(1)]= -0.1660;
  		aocs_g[IND1(2)]= 0.0508;
  		aocs_g[IND1(3)]= 3.9570;

		//-----------------------
	  	//@ correction validation : on travaille sans correction d'attitude
	  	//@ CR 232 mise en place environnement variable
	  	//-----------------------
		//  iatt=2;	/* Local normal pointing CNES Specification */
		//  iatt=3; /* Local normal pointing + Yaw steering */
  
    	//-----------------------
    	//@ att : YAW_ERROR, ROLL_ERROR, PITCH_ERROR fournie dans l'entete du niveau 0
    	//-----------------------
  		att[IND1(1)] = *(MphrN0->listevar.tab_var[MPHR_PITCH_ERROR].val.pr8); // MPHR_PITCH_ERROR
  		att[IND1(2)] = *(MphrN0->listevar.tab_var[MPHR_ROLL_ERROR].val.pr8);  // MPHR_ROLL_ERROR
  		att[IND1(3)] = *(MphrN0->listevar.tab_var[MPHR_YAW_ERROR].val.pr8);   // MPHR_YAW_ERROR

		//-----------------------
	  	//@ MetOp-1 SRAR mispointing rates [deg/s]
    	//@ datt : 0
	  	//-----------------------
  		datt[IND1(1)] = 0.0;        
  		datt[IND1(2)] = 0.0;
  		datt[IND1(3)] = 0.0;
  
		//-----------------------
	  	//@ idir : first intersection point satellite - earth
	  	//-----------------------
   		idir=MP_INTER_1ST;
   		
		//-----------------------
	  	//@ No refraction
    	//@ iray : 0 , no ray-tracing
    	//@ freq : 0
    	//-----------------------
  		iray = MP_NO_REF;
  		freq = 0.;
  		
     	//-----------------------
    	//@ mp_target outputs : ieres = Parameters to get the output 1, 2, 3 e, 5, 66 et 67, so :
    	//@ MP_TARG_RES_SAT2TARG // + MP_TARG_RES_TARG2SUN
  		//-----------------------  
  		ieres = MP_TARG_RES_SAT2TARG; // + MP_TARG_RES_TARG2SUN;

  		//-----------------------
  		//@ For each points x y
  		//-----------------------
	  	n=0;
  		
  		i=0;
  		while ( i < nbpts )
  		{
	    	//-----------------------
	    	//@ Call subroutine SD_BAS_VectPosAzEl to compute azimuth and elevation angles.
	    	//@ Angles are in degrees for METOP POINTING tools
	    	//-----------------------
//	    	SD_BAS_VectPosAzEl(VectPosRM[i], &azimuth, &elevation);
	    	
	    	//-----------------------
	    	//@ dir : direction de la ligne de visee (altitude sur Terre = 0 et delta vitesse = 0)
	    	//@ dir[0]= x dir[1]= y
	    	//-----------------------
			//  conversion en degre 
	    	dir[IND1(1)] = azimuth[i] * SD_BAS_180 / M_PI;
	    	dir[IND1(2)] = elevation[i] * SD_BAS_180 / M_PI;
	    	dir[IND1(3)] = 0.;
	    	dir[IND1(4)] = dummy;
	    	dir[IND1(5)] = 0.;
	    	dir[IND1(6)] = 0.;
	    	dir[IND1(7)] = dummy;
	    	dir[IND1(8)] = dummy;
	
	    	//-----------------------
	    	//@ appel mp_target avec les arguments :
	    	//@ mjdp : date du sous-cycle = IASITime ( normalement issu de mo_orbit)
	    	//@ pos : position du satellite a la date mjdp obtenue par propagation des parametres orbitaux a la date mjdp ([DA19])
	    	//@       a l'aide de la fonction ESA mo_orbit et des parametres orbitaux fournis dans les entetes du niveau 0.
	    	//@ vel : acceleration, fournie egalement par mo_orbit
	    	//@ acc : acceleration, fournie egalement par mo_orbit
	    	//-----------------------
	    	//-----------------------
	    	status = mp_target(	mjdp_t, pos_t, 
						     	vel_t, acc_t, 
						      	&iatt, aocs_g, 
						      	att, datt,
						      	&idir, dir, 
						      	&iray, &freq, 
						      	&ieres, res_tar, 
						      	ierr_tar);

    		//@ si erreur appel mp_vector_msg pour recuperation du libelle
    		if (status != MP_OK)
    		{
	      		func_id = MP_TARGET_ID;
				#ifdef OPS_DEBUG
	      			//si erreur appel mo_vector_msg pour recuperation du libelle
	      			mp_vector_msg(&func_id, ierr_tar, &n, msg);
				#endif
	     
	     		cr_op = SD_BAS_ERR_MP_TARGET;
	      		//@ on recupere le premier message d'erreur pour l'emission d'une trace
	      		SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
	        						SD_BAS_ERR_SET_OpLoc,
	        						SD_BAS_ERR_MP_TARGET,
	        						tcProcName_F,
	        						__LINE__,
	        						msg[0]);
	      		//@ on imprime les message d'erreurs sur la sortie standard
				#ifdef OPS_DEBUG
	      			SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, msg[0] );
	      			mp_print_msg(&n, msg);
	      			fprintf(stdout, "\n");
				#endif
	     		goto fin;      
    		}

	    	//-----------------------
	    	//@ outputs : Geodetic latitude of the target [deg] -90:90
	    	//-----------------------
    		lat = res_tar[MP_TARG_RES_GEOD_LAT];
	    	//-----------------------
	    	//@ outputs : Geocentric longitude of the target [deg] 0:360
	    	//-----------------------
    		lon = res_tar[MP_TARG_RES_GEOC_LONG];
	    	//-----------------------
	    	//@ outputs : Geodetic altitude of the target [m]
	    	//-----------------------
    		// alt = res_tar[MP_TARG_RES_GEOD_ALT];
    		alt = 0.0;
			#ifdef OPS_DEBUG
	    		printf("Resulats relatifs au point %d\n",i);
	    		printf("   EarthSatelDist : %lf\n",EarthSatelDist);
	    		printf("   Lat : %lf\n",lat);
	    		printf("   Lon : %lf\n",lon);
	    		printf("   alt : %lf\n",alt);
			#endif
	    	 
	    	//-----------------------
	    	//@ Call subroutine SD_BAS_latlon2avhrr to compute 
	    	//@ AVHRR Time and scan angle (iterative process) 
	    	//-----------------------
       		SD_BAS_latlon2avhrr(ctx, lat, 
       							lon, alt, 
       							aocs_g, att, 
       							datt, iatt,
       							TimeFirst, TimeLast,
       							mjdr_t, xm_t, x_t,
       							&AvhrrTime,
       							&AvhrrScanAngle);

			#ifdef OPS_DEBUG
	    		printf("Resulats relatifs au point %d\n",i);
	    		printf("   AvhrrTime : %lf\n", AvhrrTime);
	    		printf("   AvhrrScanAngle : %lf\n", AvhrrScanAngle);
			#endif
       		
	    	//-----------------------
	    	//@ Call subroutine SD_BAS_TimeScanAngle2ColLin to transforme 
	    	//@ Time and scanAngle AVHRR into Column and Line number 
	    	//-----------------------
       		SD_BAS_TimeScanAngle2ColLin(AvhrrTime, 
       									AvhrrScanAngle, 
       									DateFirstLineAvhrr, 
       									&AvhrrLin[i],
       									&AvhrrCol[i]);
			#ifdef OPS_DEBUG
	    		printf("   AvhrrLin : %lf\n", AvhrrLin[i]);
	    		printf("   AvhrrCol : %lf\n", AvhrrCol[i]);
			#endif
           	i++;
		} //@ fin Pour chacun des points x y

	fin:
		// sortie de la fonction
		//if (status != MO_OK) cr_op=CMN__STATUT_NOK;
		return(cr_op);

	#else
      // P. Brunel 29 aout 2013 
      // navigation_ops_init
      // convert IASI time to AAPP Julian days
      // navigation_calatt
      //   use IASI time
      //   navigation_ops_intposvel
      //
      // loop on pixels
      //    use IASI time
      //     navigation_ops_metop_to_earth
      //     convert TimeFirst, TimeLast to AAPP Julian days
      //     inverse navigation between TimeFirst, TimeLast, 
      //        ->get AVHRR time, scan angle, line and pixel
      
      static const char tcProcName_F[] = "SD_BAS_MetopAVHRR_SansAVHRR";
      t_CMN__Statut cr_op=CMN__STATUT_OK;

      double timeIasi;
      double timeAvhrr;
      double tfirst;
      double tlast;
      double reftime;
      double pos[3];
      double vel[3]; 
      double omg[3]; 
      double att[3][3]; 
      int iatt;
      int status = 0; 
      int nbppercall = 1; 
      int i;

      double lat[1];
      double lon[1];
      double Elev_sat[1];
      double Azim_sat[1];
      double Elev_sol[1];
      double Azim_sol[1];
      double EarthSatelDist[1];

      double alpha, timlin, pixel, line;
    #ifdef OPS_DEBUG
        printf("\nEntree dans la fonction SD_BAS_MetopAVHRR_SansAVHRR AAPP\n=====================================\n");
    #endif

      //navigation_ops_init
      status = navigation_ops_init();
      
      if( status != 0 ) {
        cr_op = SD_BAS_ERR_MP_ORBIT;
        SD_BAS_TEST_ERR_INT(
          ctx->cr.errinterne,
          SD_BAS_ERR_SET_OpLoc,
          SD_BAS_ERR_MP_ORBIT,
          tcProcName_F,
          __LINE__,
          "navigation_ops_init");
         goto fin;
      }
      
      
      //convert IASI time to AAPP Julian days
      timeIasi = (double)IasiTime->day + ( (double)(IasiTime)->millis / ( 1000. * 86400. ) );
      timeIasi += 18262;
    
      //    convert TimeFirst, TimeLast to AAPP Julian days
      tfirst   = TimeFirst + 18262;
      tlast    = TimeLast  + 18262;
      reftime  = DateFirstLineAvhrr  + 18262;

      //  navigation_ops_intposvel
      status = navigation_ops_intposvel( timeIasi, pos, vel );
        #ifdef OPS_DEBUG
        printf("   intposvel timeIasi : %lf\n",timeIasi);
        printf("   iasitime day from 1950: %d\n", IasiTime->day + 18262);
        printf("   iasitime seconds: %lf\n", IasiTime->millis/1000.);
        printf("   intposvel status : %d\n",status);
        printf("   pos  : %lf %lf %lf\n",pos[0],pos[1],pos[2]);
        printf("   vel  : %lf %lf %lf\n",vel[0],vel[1],vel[2]);
        #endif

      if( status != 0 ) {
        printf("   intposvel timeIasi : %lf\n",timeIasi);
        printf("   iasitime day from 1950: %d\n", IasiTime->day + 18262);
        printf("   iasitime seconds: %lf\n", IasiTime->millis/1000.);
     
        cr_op = SD_BAS_ERR_MP_ORBIT;
        SD_BAS_TEST_ERR_INT(
          ctx->cr.errinterne,
          SD_BAS_ERR_SET_OpLoc,
          SD_BAS_ERR_MP_ORBIT,
          tcProcName_F,
          __LINE__,
          "navigation_ops_intposvel");
         goto fin;
      }
    
      //navigation_calatt
      /* MetopLib */
      if( strcmp( s_METOP_ENV, METOP_ENV_TEST ) == 0 ) {
        iatt = 2; /* NOAA */
      } else {
        iatt = 3; /* METOP */
      }
      
      /* translate to AAPP */
      switch( iatt ) {
        case 2:
          iatt = 0;
        break;
        case 3:
          iatt = 1;
        break;
      }
      
      navigation_calatt( 
        *(MphrN0->listevar.tab_var[MPHR_YAW_ERROR].val.pr8),
        *(MphrN0->listevar.tab_var[MPHR_ROLL_ERROR].val.pr8),
        *(MphrN0->listevar.tab_var[MPHR_PITCH_ERROR].val.pr8),
        att
      );
    
      // loop on pixels  
      //-----------------------
      //@ For each points x y
      //-----------------------
      i = 0;
      while ( i < nbpts )
      {
        //   use IASI time
        //    navigation_ops_metop_to_earth
        #ifdef OPS_DEBUG
         printf("Entrees de  navigation_ops_metop_to_earth pour point %d\n",i);
         printf("  timeIasi  : %lf\n",timeIasi);
         printf("  pos  : %lf %lf %lf\n",pos[0],pos[1],pos[2]);
         printf("  vel  : %lf %lf %lf\n",vel[0],vel[1],vel[2]);
         printf("  azimuth    : %lf\n",azimuth[i]);
         printf("  elevation  : %lf\n",elevation[i]);
        #endif

        status = navigation_ops_metop_to_earth( 
              timeIasi, pos, vel, iatt, att, nbppercall,
              &azimuth[i], &elevation[i],
              lat, lon, 
              Elev_sat, Azim_sat,
              Elev_sol, Azim_sol,
              EarthSatelDist );
              
        #ifdef OPS_DEBUG
         printf("Resulats relatifs au point %d\n",i);
         printf("   status : %d\n",status);
         printf("   EarthSatelDist : %lf\n",EarthSatelDist[0]);
         printf("   Lat (deg): %lf\n",lat[0] * SD_BAS_180 / M_PI);
         printf("   Lon (deg): %lf\n",lon[0] * SD_BAS_180 / M_PI);
        #endif
              
        if( status != 0 ) {
          cr_op = SD_BAS_ERR_MP_TARGET;
          SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne,
            SD_BAS_ERR_SET_OpLoc,
            SD_BAS_ERR_MP_TARGET,
            tcProcName_F,
            __LINE__,
            "navigation_ops_metop_to_earth");
          goto fin;
        }     

        
        //    inverse navigation between TimeFirst, TimeLast
        //       ->get AVHRR time, scan angle, line and pixel
        status = navigation_ops_invloc(iatt, att, 
                           lat[0], lon[0],
                           tfirst,tlast, 
                           reftime,
                           &alpha, &timlin, &pixel, &line);
        #ifdef OPS_DEBUG
          printf("Resulats relatifs au point %d\n",i);
          printf("   status : %d\n",status);
          printf("   AvhrrTime : %lf\n", timlin-18262);
          printf("   AvhrrScanAngle : %lf\n", alpha);
          printf("   AvhrrLin : %lf\n", line);
          printf("   AvhrrCol : %lf\n", pixel);
        #endif
        
        if( status != 0 ) {
          cr_op = SD_BAS_ERR_MP_TARGET;
          SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne,
            SD_BAS_ERR_SET_OpLoc,
            SD_BAS_ERR_MP_TARGET,
            tcProcName_F,
            __LINE__,
            "navigation_ops_invloc");
          goto fin;
        }
    
        AvhrrLin[i] = line;
        AvhrrCol[i] = pixel;
        

        i++;
      } //@ fin Pour chacun des points x y
    
    

fin:


    // sortie de la fonction
    return(cr_op);
	#endif
	}

	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_AVHRRMetop_SansAVHRR
	//@
	//@ Description : Conversion de coordonnees AVHRR (fractionnaires) -> Metop
	//@
	//@ Parameters :
	//@         (ENTREE)  ctx : contexte de l'etape
	//@         (ENTREE)  nbpts : nombre de points a traiter
	//@         (ENTREE)  IasiTime[2] : date du SN traite
	//@         (ENTREE)  MphrN0 :  MPHR produit N0
	//@         (ENTREE)  AvhrrLin : Lignes AVHRR correspondant aux points metop (fraction de pixel)
	//@         (ENTREE)  AvhrrCol : Colonnes AVHRR correspondant aux points metop (fraction de pixel)
	//@         (ENTREE)  DateFirstLineAvhrr : nombre de lignes total du bandeau
	//@         (SORTIE)  azim : azim Metop
	//@         (SORTIE)  elev : elev Metop
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_AVHRRMetop_SansAVHRR
	(
	    // Entrees
	    t_SD_BAS_DataThread * ctx,
	    int  nbpts,
	    t_SD_BAS_SCDSTime * IasiTime,
	    t_SD_BAS_MPHR * MphrN0,
	    double * AvhrrLin, 
	    double * AvhrrCol,   
	    double DateFirstLineAvhrr,
	    // Sorties
    	double * azim,
    	double * elev
	)
	{
#ifdef OPS_USE_METOPLIB
    	static const char tcProcName_F[] = "SD_BAS_AVHRRMetop_SansAVHRR";
    	t_CMN__Statut cr_op=CMN__STATUT_OK;

	    double lat;
	    double lon;
	    double alt;
	    double azimuth;
	    double elevation;
	    double AvhrrTime;
	    double AvhrrScanAngle;
	    int i;
		double VectPosSta[3];
	    double dummy = 0.0;	    
	    	    
		/* Declaration des variables pour les appels a mo_orbit */
		long mode_t;
		double mjdp_t[2];
		double pos_t[3];
		double vel_t[3];		// position et vitesse au noeud acendant de l'orbite
		double acc_t[3];
		double res_orbit_t[54]; // valeurs temporaires (acceleration)

		/* Declaration des variables pour les appels a mp_target */
	  	double aocs_g[3];
	  	long iatt;            	// parametre d'activation de mp_target
	  	double att[3];
	  	double datt[3];
	  	long idir,iray,ieres; 	// parametres d'activation de mp_target
	  	double dir[8];
	  	double freq;            // parametre d'activation de mp_target
	  	double res_tar[75];     // resultat d'activation de mp_target
	    double mjdr_t[2];
	    double xm_t[6];
	    double x_t[6];

		/* Declaration des variables pour les appels a mp_stavis */
		double sta[4];
		double res_sta[31];

	    // Gestion de erreurs
  		long status;            // Main status flag
  		long func_id;           // CFI function ID
		long ierr_orbit_t[4];
		long ierr_tar[15];       // mode d'activation et status vector  		
		long ierr_sta[4];       // mode d'activation et status vector  		
  		long n;                 // Number of error messages
  		t_SD_BAS_MetopMSG * msg;// Error messages vector

	    #ifdef OPS_DEBUG
	        printf("\nEntree dans la fonction SD_BAS_AVHRRMetop_SansAVHRR MetOpLib\n=====================================\n");
	    #endif
	    
		//-----------------------
		//@ code sart here
		//-----------------------
  		msg=ctx->ptmsg;

		mo_silent(); 
		mp_silent();
  
		//-----------------------
  		//@ Initialise input parameters for call to mo_orbit
  		//@ call SD_BAS_Init_mo_orbit
  		//-----------------------
		cr_op = SD_BAS_Init_mo_orbit(// Entrees
						  			 ctx,
						  			 MphrN0,
							         // Sorties
								 	 &iatt,
							         mjdr_t,
							         xm_t,
							         x_t,
							   		 res_orbit_t,
									 acc_t     
							         );
        if( cr_op != CMN__STATUT_OK)
  		{
            cr_op = CMN__STATUT_NOK;
            
            //@ on recupere le premier message d'erreur pour l'emission d'une trace
            SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                                SD_BAS_ERR_SET_OpLoc,
                                SD_BAS_OPLOC_MSG_ERR_GEOD2METOP,
                                tcProcName_F,
                                __LINE__,
                                SD_BAS_OPLOC_MSG_ERR_INITMOORBIT);

            //@ on imprime les message d'erreurs sur la sortie standard
            #ifdef OPS_DEBUG
                SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, SD_BAS_OPLOC_MSG_ERR_INITMOORBIT);
            #endif

	    	goto fin;
  		}

		//-----------------------
	  	//@ Initialise input parameters for call to mp_target
    	//@ The parameters of this function are provided in Â§7.1.3. of [DA18]) :
    	//@ aocs : valeur par defaut preconisees dans [DA18], de toute facon cela ne sert a rien
	  	//-----------------------
  		aocs_g[IND1(1)]= -0.1660;
  		aocs_g[IND1(2)]= 0.0508;
  		aocs_g[IND1(3)]= 3.9570;

		//-----------------------
	  	//@ correction validation : on travaille sans correction d'attitude
	  	//@ CR 232 mise en place environnement variable
	  	//-----------------------
		iatt=3; /* Local normal pointing + Yaw steering */
  
    	//-----------------------
    	//@ att : YAW_ERROR, ROLL_ERROR, PITCH_ERROR fournie dans l'entete du niveau 0
    	//-----------------------
  		att[IND1(1)] = *(MphrN0->listevar.tab_var[MPHR_PITCH_ERROR].val.pr8); // MPHR_PITCH_ERROR
  		att[IND1(2)] = *(MphrN0->listevar.tab_var[MPHR_ROLL_ERROR].val.pr8);  // MPHR_ROLL_ERROR
  		att[IND1(3)] = *(MphrN0->listevar.tab_var[MPHR_YAW_ERROR].val.pr8);   // MPHR_YAW_ERROR

		//-----------------------
	  	//@ MetOp-1 SRAR mispointing rates [deg/s]
    	//@ datt : 0
	  	//-----------------------
  		datt[IND1(1)] = 0.0;        
  		datt[IND1(2)] = 0.0;
  		datt[IND1(3)] = 0.0;
  
		//-----------------------
	  	//@ idir : first intersection point satellite - earth
	  	//-----------------------
   		idir=MP_INTER_1ST;
   		
		//-----------------------
	  	//@ No refraction
    	//@ iray : 0 , no ray-tracing
    	//@ freq : 0
    	//-----------------------
  		iray = MP_NO_REF;
  		freq = 0.;
  		
     	//-----------------------
    	//@ mp_target outputs : ieres = Parameters to get the output 1, 2, 3 e, 5, 66 et 67, so :
    	//@ MP_TARG_RES_SAT2TARG // + MP_TARG_RES_TARG2SUN
  		//-----------------------  
  		ieres = MP_TARG_RES_SAT2TARG; // + MP_TARG_RES_TARG2SUN;

  		//-----------------------
  		//@ For each points x y
  		//-----------------------  		
  		i = 0;
  		while ( i < nbpts )
  		{
	    	//-----------------------
	    	//@ Call subroutine SD_BAS_ColLin2TimeScanAngle to transform
	    	//@ AVHRR Column and line number in Time and ScanAngle.
	    	//-----------------------
	    	SD_BAS_ColLin2TimeScanAngle(AvhrrLin[i], 
	    								AvhrrCol[i],
	    								DateFirstLineAvhrr,
	    								&AvhrrTime,
	    								&AvhrrScanAngle);
	    	
	    	//-----------------------
	    	//@ Compute vector satellite-station coordinates in SRAR.
	    	//-----------------------
	    	VectPosSta[IND1(1)] = sin(AvhrrScanAngle);
	    	VectPosSta[IND1(2)] = 0.;
	    	VectPosSta[IND1(3)] = -cos(AvhrrScanAngle);
	    	
	    	//-----------------------
	    	//@ Call subroutine SD_BAS_VectPosAzEl 
	    	//@ to compute azimuth et elevation angles.
	    	//-----------------------
	    	SD_BAS_VectPosAzEl(VectPosSta, &azimuth, &elevation);
	    	
	    	//-----------------------
	    	//@ Compute satellite state vector at avhrr time.
			//-----------------------
			//-----------------------
		  	//@ Call mo_orbit with propagation mode :
		  	//@ MO_PROPAG
	  		//-----------------------
	  		mode_t = MO_PROPAG;		  
		  	//-----------------------
		  	//@  mjdr_t : from initialisation mode
		  	//@  xm_t : from initialisation mode
		  	//@  mjdp_t : AVHRR Time
		  	//-----------------------
		  	mjdp_t[IND1(1)] = AvhrrTime;
		  	mjdp_t[IND1(2)] = dummy;
		  	//-----------------------
			//@ x_t : from initialisation mode
		  	//@ pos_t : used below
		  	//@ vel_t : used below
		  	//@ acc_t : used below
		  	//@ res_orbit_t :
		  	//@ ierr_orbit_t :
		  	//-----------------------
	  	
		  	//-----------------------
		  	//@ call mo_orbit with propagation mode
		  	//-----------------------
		  	status = mo_orbit(	&mode_t, 
		    				  	mjdr_t, 
		    					xm_t, 
		    					mjdp_t,
		    					x_t, 
		    					pos_t, 
		    					vel_t, 
		    					acc_t, 
		    					res_orbit_t, 
		    					ierr_orbit_t);
	    
		  	// mo_orbit error management
		  	if (status != MO_OK)
		  	{
		    	func_id = MO_ORBIT_ID;
		    	// si erreur appel mo_vector_msg pour recuperation du libelle
		    	status = mo_vector_msg(&func_id, ierr_orbit_t, &n, msg);
		    
		    	//@ on recupere le premier message d'erreur pour l'emission d'une trace
		    	cr_op   = SD_BAS_ERR_MP_ORBIT;
		    	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
		      						SD_BAS_ERR_SET_OpLoc,
		      						SD_BAS_ERR_MP_ORBIT,
		      						tcProcName_F,
		      						__LINE__,
		      						msg[0]);
		      						
				#ifdef OPS_DEBUG
			    	//@ print error message on standard output
		    		SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
		    		status = mo_print_msg(&n, msg);
		    		fprintf(stdout, "\n");
				#endif
		      						
		     	goto fin;		      
		  	}

	    	//-----------------------
	    	//@ dir : direction de la ligne de visee (altitude sur Terre = 0 et delta vitesse = 0)
	    	//@ dir[0]= x dir[1]= y
	    	//-----------------------
			//  conversion en degre 
	    	dir[IND1(1)] = azimuth * SD_BAS_180 / M_PI;
	    	dir[IND1(2)] = elevation * SD_BAS_180 / M_PI;
	    	dir[IND1(3)] = 0.;
	    	dir[IND1(4)] = dummy;
	    	dir[IND1(5)] = 0.;
	    	dir[IND1(6)] = 0.;
	    	dir[IND1(7)] = dummy;
	    	dir[IND1(8)] = dummy;
	
	    	//-----------------------
	    	//@ appel mp_target avec les arguments :
	    	//@ mjdp : date du sous-cycle = IASITime ( normalement issu de mo_orbit)
	    	//@ pos : position du satellite a la date mjdp obtenue par propagation des parametres orbitaux a la date mjdp ([DA19])
	    	//@       a l'aide de la fonction ESA mo_orbit et des parametres orbitaux fournis dans les entetes du niveau 0.
	    	//@ vel : acceleration, fournie egalement par mo_orbit
	    	//@ acc : acceleration, fournie egalement par mo_orbit
	    	//-----------------------
	    	//-----------------------
	    	status = mp_target(	mjdp_t, pos_t, 
						     	vel_t, acc_t, 
						      	&iatt, aocs_g, 
						      	att, datt,
						      	&idir, dir, 
						      	&iray, &freq, 
						      	&ieres, res_tar, 
						      	ierr_tar);

    		//@ si erreur appel mp_vector_msg pour recuperation du libelle
    		if (status != MP_OK)
    		{
	      		func_id = MP_TARGET_ID;
				#ifdef OPS_DEBUG
	      			//si erreur appel mo_vector_msg pour recuperation du libelle
	      			mp_vector_msg(&func_id, ierr_tar, &n, msg);
				#endif
	     
	     		cr_op = SD_BAS_ERR_MP_TARGET;
	      		//@ on recupere le premier message d'erreur pour l'emission d'une trace
	      		SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
	        						SD_BAS_ERR_SET_OpLoc,
	        						SD_BAS_ERR_MP_TARGET,
	        						tcProcName_F,
	        						__LINE__,
	        						msg[0]);
	      		//@ on imprime les message d'erreurs sur la sortie standard
				#ifdef OPS_DEBUG
	      			SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
	      			mp_print_msg(&n, msg);
	      			fprintf(stdout, "\n");
				#endif
	     		goto fin;      
    		}

	    	//-----------------------
	    	//@ outputs : Geodetic latitude of the target [deg] -90:90
	    	//-----------------------
    		lat = res_tar[MP_TARG_RES_GEOD_LAT];
	    	//-----------------------
	    	//@ outputs : Geocentric longitude of the target [deg] 0:360
	    	//-----------------------
    		lon = res_tar[MP_TARG_RES_GEOC_LONG];
	    	//-----------------------
	    	//@ outputs : Geodetic altitude of the target [m]
	    	//-----------------------
    		// alt = res_tar[MP_TARG_RES_GEOD_ALT];
    		alt = 0.0;
    		
			#ifdef OPS_DEBUG
	    		printf("Resulats relatifs au point %d\n",i);
	    		printf("   Lat : %lf\n",lat);
	    		printf("   Lon : %lf\n",lon);
	    		printf("   alt : %lf\n",alt);
			#endif
			
	    	//-----------------------
	    	//@ Compute satellite state vector at IASI time.
			//-----------------------
			//-----------------------
		  	//@ Call mo_orbit with propagation mode :
		  	//@ MO_PROPAG
	  		//-----------------------
	  		mode_t = MO_PROPAG;		  
		  	//-----------------------
		  	//@  mjdr_t : from initialisation mode
		  	//@  xm_t : from initialisation mode
		  	//@  mjdp_t : IASI Time
		  	//-----------------------
		  	mjdp_t[IND1(1)] = (double)IasiTime->day + ( (double)(IasiTime)->millis / (1000.*86400.) );
		  	mjdp_t[IND1(2)] = dummy;
		  	//-----------------------
			//@ x_t : from initialisation mode
		  	//@ pos_t : used below
		  	//@ vel_t : used below
		  	//@ acc_t : used below
		  	//@ res_orbit_t :
		  	//@ ierr_orbit_t :
		  	//-----------------------
	  	
		  	//-----------------------
		  	//@ call mo_orbit with propagation mode
		  	//-----------------------
		  	status = mo_orbit(	&mode_t, 
		    				  	mjdr_t, 
		    					xm_t, 
		    					mjdp_t,
		    					x_t, 
		    					pos_t, 
		    					vel_t, 
		    					acc_t, 
		    					res_orbit_t, 
		    					ierr_orbit_t);
	    
		  	// mo_orbit error management
		  	if (status != MO_OK)
		  	{
		    	func_id = MO_ORBIT_ID;
		    	// si erreur appel mo_vector_msg pour recuperation du libelle
		    	status = mo_vector_msg(&func_id, ierr_orbit_t, &n, msg);
		    
		    	//@ on recupere le premier message d'erreur pour l'emission d'une trace
		    	cr_op   = SD_BAS_ERR_MP_ORBIT;
		    	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
		      						SD_BAS_ERR_SET_OpLoc,
		      						SD_BAS_ERR_MP_ORBIT,
		      						tcProcName_F,
		      						__LINE__,
		      						msg[0]);
		      						
				#ifdef OPS_DEBUG
			    	//@ print error message on standard output
		    		SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
		    		status = mo_print_msg(&n, msg);
		    		fprintf(stdout, "\n");
				#endif
		      						
		     	goto fin;		      
		  	}
			
	    	//-----------------------
	    	//@ Compute satellite position in SRAR coordinate system
	    	//@ at IASI time, and lat-lon.
		  	//@ Call mp_stavis
		  	//@ aocs_g : same as mp_target
		  	//@ att : same as mp_target 
		  	//@ datt : same as mp_target 
		  	//@ iatt : same as mp_target 
		  	//-----------------------
		  	sta[IND1(1)] = lon;	// ground station geocentric longitude [deg]
		  	sta[IND1(2)] = lat;	// ground station geodetic latitude [deg]	
		  	sta[IND1(3)] = alt;	// ground station geodetic altitude [m]
		  	sta[IND1(4)] = -90.;	// ground station minimum elevation link [deg]

		  	//-----------------------
		  	//@ call mp_stavis
		  	//-----------------------
			status = mp_stavis(	mjdp_t, 
						      	pos_t, 
						      	vel_t, 
						      	acc_t, 
						      	&iatt, 
						      	aocs_g, 
						      	att, 
						      	datt, 
						      	sta, 
						      	res_sta, 
						      	ierr_sta);
    		if (status != MP_OK)
    		{
      			func_id = MP_STAVIS_ID;
			 	//@ si erreur appel mp_vector_msg pour recuperation du libelle
    			mp_vector_msg(&func_id, ierr_sta, &n, msg);
      			//@ on recupere le premier message d'erreur pour l'emission d'une trace
    			cr_op   = SD_BAS_ERR_MP_STAVIS;
      			SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        							SD_BAS_ERR_SET_OpLoc,
        							SD_BAS_ERR_MP_STAVIS,
        							tcProcName_F,
        							__LINE__,
        							msg[0]);
      			//@ on imprime les message d'erreurs sur la sortie standard
				#ifdef OPS_DEBUG
      				SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg[0] );
      				mp_print_msg(&n, msg);
      				fprintf(stdout, "\n");
				#endif
     			goto fin;       
    		}
		  	//-----------------------
    		// output : satellite to station azimuth SRAR coordinate system
		  	//-----------------------
    		azimuth = res_sta[MP_STA_RES_SAT2STA_AZ_SRAR];
		  	//-----------------------
    		// output : satellite to station elevation SRAR coordinate system
		  	//-----------------------
    		elevation = res_sta[MP_STA_RES_SAT2STA_EL_SRAR];
 		  	//-----------------------
    		// output : satellite to station range EF CS [m]
 		  	//-----------------------
    		//DistStaSat = res_sta[MP_STA_RES_RANGE];
		  	//-----------------------
    		// convert angles from degrees to radian
		  	//-----------------------
		  	azim[i] = azimuth * M_PI / SD_BAS_180;
		  	elev[i] = elevation * M_PI / SD_BAS_180;
		  	#ifdef OPS_DEBUG
	    		printf("   Azimuth   : %lf\n",azim[i]);
	    		printf("   Elevation : %lf\n",elev[i]);
			#endif

    		//-----------------------
		  	//@ Computes unitary vector coordinates in SRAR
		  	//@ call SD_BAS_AzElVectPos
		  	//-----------------------
//		  	cr_op = SD_BAS_AzElVectPos(azimuth, elevation, VectPosRM[i]);
//        	if( cr_op != CMN__STATUT_OK)
//  			{
//            	cr_op = CMN__STATUT_NOK;
//            	//@ on recupere le premier message d'erreur pour l'emission d'une trace
//            	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
//                                	SD_BAS_ERR_SET_OpLoc,
//                                	cr_op,
//                                	tcProcName_F,
//                                	__LINE__,
//                                	SD_BAS_OPLOC_MSG_ERR_AZEL2VECTPOS)
//            	//@ on imprime les message d'erreurs sur la sortie standard
//            	#ifdef OPS_DEBUG
//                	SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, SD_BAS_OPLOC_MSG_ERR_AZEL2VECTPOS);
//            	#endif
//	    		goto fin;
//  			}
			i++;
  		} //@ fin Pour chacun des points x y 

	fin:
	// sortie de la fonction
	return(cr_op);
	
	#else
      // P. Brunel 29 aout 2013 
      // navigation_ops_init
      // convert IASI time to AAPP Julian days
      // navigation_calatt
      // loop on pixels
      //    calculate AVHRR time (call SD_BAS_ColLin2TimeScanAngle)
      //    convert AVHRR time to AAPP Julian days
      //    use AVHRR time
      //    navigation_ops_intposvel
      //    navogation_ops_metop_to_earth
      //    use IASI time
      //    navigation_ops_intposvel
      //    navigation_ops_earth_to_metop
      
      // Note that DateFirstLineAvhrr, TimeFirst, TimeLast in bothe routines SD_BAS_*_SansAVHRR
      // are calculated in SD_ALG_AVHRR from IasiTime like the following:
      // DateFirstLineAvhrr = (double)IasiTime.day + ( (double)(IasiTime).millis / (1000.*86400.) );
              			

      static const char tcProcName_F[] = "SD_BAS_AVHRRMetop_SansAVHRR";
      t_CMN__Statut cr_op=CMN__STATUT_OK;

      double lat[1];
      double lon[1];
      double azimuth[1];
      double elevation[1];
      double AvhrrTime;
      double AvhrrScanAngle;
      int i;
      double VectPosSta[3];

      int nbppercall = 1;

      double timeAvhrr;
      double timeIasi;
      double pos[3];
      double vel[3];  
      double att[3][3];
      double Elev_sat[1];
      double Azim_sat[1];
      double Elev_sol[1];
      double Azim_sol[1];
      double EarthSatelDist[1];
      double az;
      double el;
      
      int iatt;
      int status = 0;
      #ifdef OPS_DEBUG
        printf("\nEntree dans la fonction SD_BAS_MetopAVHRR_SansAVHRR AAPP\n=====================================\n");
      #endif

      // navigation_ops_init
      status = navigation_ops_init();
      
      if( status != 0 ) {
        cr_op = SD_BAS_ERR_MP_ORBIT;
        SD_BAS_TEST_ERR_INT(
          ctx->cr.errinterne,
          SD_BAS_ERR_SET_OpLoc,
          SD_BAS_ERR_MP_ORBIT,
          tcProcName_F,
          __LINE__,
          "navigation_ops_init");
         goto fin;
      }
     
      // convert IASI time to AAPP Julian days
      timeIasi = (double)IasiTime->day + ( (double)(IasiTime)->millis / ( 1000. * 86400. ) );
      timeIasi += 18262;
     
      // navigation_calatt
      /* MetopLib */
      if( strcmp( s_METOP_ENV, METOP_ENV_TEST ) == 0 ) {
        iatt = 2; /* NOAA */
      } else {
        iatt = 3; /* METOP */
      }
     
      /* Translate to AAPP */  
      switch( iatt ) {
        case 2:
          iatt = 0;
        break;
        case 3:
          iatt = 1;
        break;
      }
     
      navigation_calatt( 
        *(MphrN0->listevar.tab_var[MPHR_YAW_ERROR].val.pr8),
        *(MphrN0->listevar.tab_var[MPHR_ROLL_ERROR].val.pr8),
        *(MphrN0->listevar.tab_var[MPHR_PITCH_ERROR].val.pr8),
        att
      );
     
      // loop on pixels  		//-----------------------
      //@ For each points x y
      //-----------------------  		
      i = 0;
      while ( i < nbpts )
      {
        //    calculate AVHRR time (call SD_BAS_ColLin2TimeScanAngle)
        //-----------------------
        //@ Call subroutine SD_BAS_ColLin2TimeScanAngle to transform
        //@ AVHRR Column and line number in Time and ScanAngle.
        //-----------------------
        SD_BAS_ColLin2TimeScanAngle(AvhrrLin[i], 
                                    AvhrrCol[i],
                                    DateFirstLineAvhrr,
                                    &AvhrrTime,
                                    &AvhrrScanAngle);
        //-----------------------
        //@ Compute vector satellite-station coordinates in SRAR.
        //-----------------------
        VectPosSta[IND1(1)] = sin(AvhrrScanAngle);
        VectPosSta[IND1(2)] = 0.;
        VectPosSta[IND1(3)] = -cos(AvhrrScanAngle);
        
        //-----------------------
        //@ Call subroutine SD_BAS_VectPosAzEl 
        //@ to compute azimuth et elevation angles.
        //-----------------------
        SD_BAS_VectPosAzEl(VectPosSta, &az, &el);
        azimuth[0]   = az;
        elevation[0] = el;
        
        //    convert AVHRR time to AAPP Julian days
        timeAvhrr = AvhrrTime + 18262;
     
        //    use AVHRR time
        //    navigation_ops_intposvel
        status = navigation_ops_intposvel( timeAvhrr, pos, vel );
        #ifdef OPS_DEBUG
        printf("   intposvel timeAvhrr : %lf\n",timeAvhrr);
        printf("   status : %d\n",status);
        printf("   AvhrrTime: %lf\n",AvhrrTime );
        printf("   pos  : %lf %lf %lf\n",pos[0],pos[1],pos[2]);
        printf("   vel  : %lf %lf %lf\n",vel[0],vel[1],vel[2]);
        #endif
     
        if( status != 0 ) {
          cr_op = SD_BAS_ERR_MP_ORBIT;
          SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne,
            SD_BAS_ERR_SET_OpLoc,
            SD_BAS_ERR_MP_ORBIT,
            tcProcName_F,
            __LINE__,
            "navigation_ops_intposvel");
           goto fin;
        }
        
        #ifdef OPS_DEBUG
         printf("Entrees de  navigation_ops_metop_to_earth pour point %d\n",i);
         printf("  timeAvhrr  : %lf\n",timeAvhrr);
         printf("  pos  : %lf %lf %lf\n",pos[0],pos[1],pos[2]);
         printf("  vel  : %lf %lf %lf\n",vel[0],vel[1],vel[2]);
         printf("  azimuth    : %lf\n",azimuth[i]);
         printf("  elevation  : %lf\n",elevation[i]);
        #endif     
        //    navigation_ops_metop_to_earth
        status = navigation_ops_metop_to_earth( 
                timeAvhrr, pos, vel, iatt, att, nbppercall,
                azimuth, elevation, lat, lon, 
                Elev_sat, Azim_sat,
                Elev_sol, Azim_sol,
                EarthSatelDist );
                
        #ifdef OPS_DEBUG
         printf("Resulats relatifs au point %d\n",i);
         printf("   status : %d\n",status);
         printf("   EarthSatelDist : %lf\n",EarthSatelDist[0]);
         printf("   Lat (deg): %lf\n",lat[0] * SD_BAS_180 / M_PI);
         printf("   Lon (deg): %lf\n",lon[0] * SD_BAS_180 / M_PI);
        #endif

        if( status != 0 ) {
          cr_op = SD_BAS_ERR_MP_TARGET;
          SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne,
            SD_BAS_ERR_SET_OpLoc,
            SD_BAS_ERR_MP_TARGET,
            tcProcName_F,
            __LINE__,
            "navigation_ops_metop_to_earth");
          goto fin;
        }

        //    use IASI time
        //    navigation_ops_intposvel
        status = navigation_ops_intposvel( timeIasi, pos, vel );
             #ifdef OPS_DEBUG
        printf("   intposvel timeIasi : %lf\n",timeIasi);
        printf("   status : %d\n",status);
        printf("   AvhrrTime: %lf\n",AvhrrTime );
        printf("   pos  : %lf %lf %lf\n",pos[0],pos[1],pos[2]);
        printf("   vel  : %lf %lf %lf\n",vel[0],vel[1],vel[2]);
        #endif

        if( status != 0 ) {
          cr_op = SD_BAS_ERR_MP_ORBIT;
          SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne,
            SD_BAS_ERR_SET_OpLoc,
            SD_BAS_ERR_MP_ORBIT,
            tcProcName_F,
            __LINE__,
            "navigation_ops_intposvel");
           goto fin;
        }
        #ifdef OPS_DEBUG
         printf("Entrees de  navigation_ops_earth_to_metop pour point %d\n",i);
         printf("  pos  : %lf %lf %lf\n",pos[0],pos[1],pos[2]);
         printf("  vel  : %lf %lf %lf\n",vel[0],vel[1],vel[2]);
         printf("   Lat (deg): %lf\n",lat[0] * SD_BAS_180 / M_PI);
         printf("   Lon (deg): %lf\n",lon[0] * SD_BAS_180 / M_PI);
        #endif     

        //    navigation_ops_earth_to_metop
        status = navigation_ops_earth_to_metop( 
         pos, vel, iatt,
         att, nbppercall, lat, lon,
         azimuth, elevation );
         
        #ifdef OPS_DEBUG
         printf("Sortie de  navigation_ops_earth_to_metop pour point %d\n",i);
         printf("   status : %d\n",status);
         printf("  azimuth    : %lf\n",azimuth[0]);
         printf("  elevation  : %lf\n",elevation[0]);
        #endif     
      
        if( status != 0 ) {
          cr_op = SD_BAS_ERR_MP_TARGET;
          SD_BAS_TEST_ERR_INT(
            ctx->cr.errinterne,
            SD_BAS_ERR_SET_OpLoc,
            SD_BAS_ERR_MP_TARGET,
            tcProcName_F,
            __LINE__,
            "navigation_ops_earth_to_metop");
          goto fin;
        }
     
        azim[i] = azimuth[0];
        elev[i] = elevation[0];

        #ifdef OPS_DEBUG
          printf("   Azimuth   : %lf\n",azim[i]);
          printf("   Elevation : %lf\n",elev[i]);
        #endif

        i++;
      } //@ fin Pour chacun des points x y 
     
fin:
      // sortie de la fonction
      return(cr_op);
#endif
}
// ...FT-2997



//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InitialiserOperateur
//@
//@ Description : cette fonction permet de creer les mutex utiles pour rendre
//@     threadsafe metop_pointing (bloquante)
//@     afin de la rendre thread safe
//@
//@
//@ Parameters :  ctx
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_InitialiserOperateur(
  t_SD_BAS_DataThread * ctx)
{
static const char tcProcName_F[] = "SD_BAS_InitialiserOperateur";
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction

//@ appel fonction de creation du mutex
pthread_mutex_init(&s_MutexTravaille,NULL);

// CR 232 recupareation var env
s_METOP_ENV = getenv(METOP_ENV);

if(s_METOP_ENV == NULL)
  {
    SD_BAS_TEST_ERR_INT( 
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_OpLoc, 
      SD_BAS_MSG_METOPENV, 
      tcProcName_F, 
      __LINE__, 
      SD_BAS_MSG_METOPENV_ND);
    cr_op = SD_BAS_MSG_METOPENV;
    goto fin;
  }
if (strcmp(s_METOP_ENV,METOP_ENV_TEST)!= 0 && strcmp(s_METOP_ENV,METOP_ENV_NORMAL)!= 0)
  {
    SD_BAS_TEST_ERR_INT( 
      ctx->cr.errinterne, 
      SD_BAS_ERR_SET_OpLoc, 
      SD_BAS_MSG_METOPENV, 
      tcProcName_F, 
      __LINE__, 
      SD_BAS_MSG_METOPENV_NC);
    cr_op = SD_BAS_MSG_METOPENV;
    goto fin;
  }
fin:
// sortie de la fonction
return(cr_op);

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_DetruireOperateur
//@
//@ Description : cette fonction permet de detruire les mutex utiles pour rendre
//@     threadsafe metop_pointing (bloquante)
//@     afin de la rendre thread safe
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_DetruireOperateur(
            )
{
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction

//@ appel fonction de suppression du mutex
pthread_mutex_destroy(&s_MutexTravaille);


fin:
// sortie de la fonction
return(cr_op);

}
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_ReserverOperateur
//@
//@ Description : cette fonction permet de reserver l'acces a la librairie
//@     threadsafe metop_pointing (bloquante)
//@     afin de la rendre thread safe
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_ReserverOperateur(
            )
{
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction

//@ appel fonction de creation du mutex
pthread_mutex_lock(&s_MutexTravaille);


fin:
// sortie de la fonction
return(cr_op);

}

//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_LibererOperateur
//@
//@ Description : cette fonction permet de liberer l'acces a la librairie
//@     threadsafe metop_pointing (bloquante)
//@     afin de la rendre thread safe
//@
//@
//@ Parameters :  aucun
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
t_CMN__Statut SD_BAS_LibererOperateur(
            )
{
t_CMN__Statut cr_op=CMN__STATUT_OK;
// Entree dans la fonction

//@ appel fonction de creation du mutex
pthread_mutex_unlock(&s_MutexTravaille);


fin:
// sortie de la fonction
return(cr_op);

}

//======================================================================
//                  FONCTIONS INTERNES
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Procedure : SD_BAS_InterpolLatLon
//@
//@ Description : cette fonction permet d'interpoler dans les donnes 
//@ de latitude/longitude contenus dans les donnees AVHRR afin d'en extraire
//@ une valeur reel (l,c) qui corrspond au mieux)
//@ pour cela a partir d'une grille locale on effectue une projection stereoscopique
//@ et on interpole bilineairement dans le plan projete
//@
//@
//@ Parameters :  
//@         (ENTREE)  ctx : contexte de l'etape
//@         (ENTREE) x : point recherche(repere terrestre)
//@         (ENTREE) y : point recherche(repere terrestre)
//@         (ENTREE) z : point recherche(repere terrestre)
//@         (ENTREE) colentier : point (col)  le plus proche du point precedent dans la grille AVHRR
//@        (ENTREE) ligentier : point (lig)  le plus proche du point precedent dans la grille AVHRR
//@         (ENTREE)  BandeauAVHRR : bandeau AVHRR charge en memoire
//@         (ENTREE)  nbligbandeau : nombre de lignes total du bandeau
//@         (SORTIE)  col : colonne fractionnaires du point cherche
//@         (SORTIE)  lig : ligne fractionnaires des point cherche
//@
//@ Return Status :
//@         t_CMN__Statut
//@         SD_BAS_STATUT_OK si la fonction a reussi
//@         SD_BAS_STATUT_NOK si la fonction a echoue
//@
//@ ------------------------------------------------------------------*/
  t_CMN__Statut SD_BAS_InterpolLatLon(
       t_SD_BAS_DataThread * ctx,
        const double  x,
        const double  y,
        const double  z,
        const int  colentier,
        const int  ligentier,
        t_SD_BAS_LigneAVHRR** BandeauAVHRR,
        int nbligbandeau,
        double * col,
        double *lig
            )
{
static const char tcProcName_F[] = "SD_BAS_InterpolLatLon";
t_CMN__Statut cr_op=CMN__STATUT_OK;
double latavhrr[9];
double lonavhrr[9];
double colavhrr[9];
double ligavhrr[9];
double xav[9];
double yav[9];
double zav[9];
double xPpA,yPpA,zPpA;
double coslat,sinlat;
double coslon,sinlon;
double normePpA;
double xorthoav[9];
double yorthoav[9];
double zorthoav[9];
double coef,invcoeff;
double coefij;
double i1[3];
double j1[3];
double i1vectj1[3];
double OPvecti1j1[3];
double ui1orthoav[9];
double vj1orthoav[9];
double xortho,yortho,zortho;
double u,v,tu,tv;
double dusuiv,dvsuiv;
double puiss1,puiss2,puiss3,puiss4;
int coincellule;
int choix;
int util;
int il,ic,ct;
//double u1,v1,u2,v3;
double const1;
double c[100];
double xe1[3],xe2[3],ye[9];
int nc,ne,i;
//FA160
char *msg_err;          // Message d'erreur
int num_lig;
// Entree dans la fonction
 const1 = (double)M_PI/(double)SD_BAS_180;

// 1 : on extrait tout d'abord du bandeau AVHRR les 8 points encadrant le point entier lle plus proche
//
//   1		2	3
//        C1       C2
//   4	   ptentier (5) 6	
//        C3       C4
//   7		8	9
  for (il=ligentier-1,ct=1;il<=ligentier+1;il++)
   {
    for (ic=colentier-1;ic<=colentier+1;ic++,ct++)
     {
     latavhrr[IND1(ct)]=BandeauAVHRR[il]->Image.latitude[ic]*const1;
     lonavhrr[IND1(ct)]=BandeauAVHRR[il]->Image.longitude[ic]*const1;
     ligavhrr[IND1(ct)]=il;
     colavhrr[IND1(ct)]=ic*SOUSECH_AVHRR_COL;
     }
   }

// 2 : on determine les coordonnees des points  dans le repere terrestre (sphere unite)
  for (ct=1;ct<=9;ct++)
   {
    coslat = cos(latavhrr[IND1(ct)]);
    sinlat = sin(latavhrr[IND1(ct)]);
    coslon = cos(lonavhrr[IND1(ct)]);
    sinlon = sin(lonavhrr[IND1(ct)]);
    xav[IND1(ct)]=coslon*coslat;
    yav[IND1(ct)]=sinlon*coslat;
    zav[IND1(ct)]=sinlat;
    }


// 4 : pour les points 1 2 3 4  et 6 7 8 9 on les projette dans la projection locale stereoscopique associe au point 5
//     on se place ici dans le repere terrestre 
//    les point obtenus sont appeles P1 P2 ... P9
  for (ct=1;ct<=9;ct++)
   {
   xorthoav[IND1(ct)]=0.;
   xorthoav[IND1(ct)]=0.;
   xorthoav[IND1(ct)]=0.;
   // le point central a pour coordonnees 0, 0
   if (ct != 5)
    {
// calcul du vecteur P'A dans le referentiel terrestre
// P'A = OP + OA
      xPpA= xav[IND1(5)] + xav[IND1(ct)];   
      yPpA= yav[IND1(5)] + yav[IND1(ct)];   
      zPpA= zav[IND1(5)] + zav[IND1(ct)]; 
      normePpA =  xPpA*xPpA + yPpA*yPpA + zPpA*zPpA;
      coef = 4./normePpA;
// calcul du vecteur PB dans le referentiel terrestre
// PB = -2OP + P'A * 4/||P 'A||^2
      xorthoav[IND1(ct)]= -2 * xav[IND1(5)] + coef * xPpA;
      yorthoav[IND1(ct)]= -2 * yav[IND1(5)] + coef * yPpA;
      zorthoav[IND1(ct)]= -2 * zav[IND1(5)] + coef * zPpA;
    }
   }


// 5 : on determine de nouveaux vecteurs utilise comme base du plan tangent i1 et j1  a partir de la formule:
//    j1 = P5P8 / || P5P8||
 coef = sqrt(xorthoav[IND1(8)]*xorthoav[IND1(8)] + yorthoav[IND1(8)]*yorthoav[IND1(8)] + zorthoav[IND1(8)]*zorthoav[IND1(8)]);
 invcoeff= 1./coef;
 j1[0]= xorthoav[IND1(8)]*invcoeff;
 j1[1]= yorthoav[IND1(8)]*invcoeff;
 j1[2]= zorthoav[IND1(8)]*invcoeff; 
//    on en deduit les coordonnees de P8  dans ce nouveau repere (a , b)  : 
// P8 ( 0,|| P5P8||)
 ui1orthoav[IND1(8)]=0.;
 vj1orthoav[IND1(8)]=coef;
//   on determine de nouveaux vecteurs utilise comme base du plan tangent i1 et j1  a partir de la formule:
//    i1 = P5P6 / || P5P6||
 coef = sqrt(xorthoav[IND1(6)]*xorthoav[IND1(6)] + yorthoav[IND1(6)]*yorthoav[IND1(6)] + zorthoav[IND1(6)]*zorthoav[IND1(6)]);
 invcoeff= 1./coef;
 i1[0]= xorthoav[IND1(6)]*invcoeff;
 i1[1]= yorthoav[IND1(6)]*invcoeff;
 i1[2]= zorthoav[IND1(6)]*invcoeff; 
//    on en deduit les coordonnees de  P6 dans ce nouveau repere (a , b)  : 
// P6 ( || P5P6||,0)
 ui1orthoav[IND1(6)]=coef;
 vj1orthoav[IND1(6)]=0.;
 
 
// 6 : pour les points 1 2 3 4 7 9  on les projette dans la projection locale stereoscopique associe au point 5
//     on se place ici dans le repere i1, j1 
//   OP = u i1 + v j1, le calcul de a et b se realise en prenant le produit vectoriel
 i1vectj1[0]=(i1[1]*j1[2] - i1[2]*j1[1]);
 i1vectj1[1]=(i1[2]*j1[0] - i1[0]*j1[2]);
 i1vectj1[2]=(i1[0]*j1[1] - i1[1]*j1[0]);
 if ( fabs(i1vectj1[0]) >= 0.01 ) 
  {
  coefij= 1./i1vectj1[0];
  choix=0;
  }
 else if ( fabs(i1vectj1[1]) >= 0.01  )
  {
  coefij= 1./i1vectj1[1];
  choix=1;
  }
 else
  {
  coefij= 1./i1vectj1[2];
  choix=2;
  }
  
vj1orthoav[IND1(5)]=0.;
ui1orthoav[IND1(5)]=0.;
 for (ct=1;ct<=9;ct++)
   {
   
   // le point central a pour coordonnees 5
   if (ct != 6 && ct != 8 && ct != 5)
     {
//  OPx ^ i1 = v j1 ^ i1   d'ou v = OPx ^ i1 / j1 ^ i1
     OPvecti1j1[0]=(yorthoav[IND1(ct)]*i1[2] - zorthoav[IND1(ct)]*i1[1]);
     OPvecti1j1[1]=(zorthoav[IND1(ct)]*i1[0] - xorthoav[IND1(ct)]*i1[2]);
     OPvecti1j1[2]=(xorthoav[IND1(ct)]*i1[1] - yorthoav[IND1(ct)]*i1[0]);
     vj1orthoav[IND1(ct)]= OPvecti1j1[choix]*-coefij;
//  OPx ^ j1 = u i1 ^ j1   d'ou u =  OPx ^ j1 / i1 ^ j1
     OPvecti1j1[0]=(yorthoav[IND1(ct)]*j1[2] - zorthoav[IND1(ct)]*j1[1]);
     OPvecti1j1[1]=(zorthoav[IND1(ct)]*j1[0] - xorthoav[IND1(ct)]*j1[2]);
     OPvecti1j1[2]=(xorthoav[IND1(ct)]*j1[1] - yorthoav[IND1(ct)]*j1[0]);
     ui1orthoav[IND1(ct)]= OPvecti1j1[choix]*coefij;
     }
   }



// 7 :on  projette le point recherche (x , y , z)  dans la projection locale stereoscopique associe au point 5
// calcul du vecteur P'A dans le referentiel terrestre
// P'A = OP + OA
 xPpA= xav[IND1(5)] + x;   
 yPpA= yav[IND1(5)] + y;   
 zPpA= zav[IND1(5)] + z; 
 normePpA =  xPpA*xPpA + yPpA*yPpA + zPpA*zPpA;
 coef = 4./normePpA;
// calcul du vecteur PB dans le referentiel terrestre
// PB = -2OP + P'A * 4/||P 'A||^2
 xortho= -2 * xav[IND1(5)] + coef * xPpA;
 yortho= -2 * yav[IND1(5)] + coef * yPpA;
 zortho= -2 * zav[IND1(5)] + coef * zPpA;
// calcul du vecteur PB dans le referentiel i1 j1
//  OPx ^ i1 = v j1 ^ i1   d'ou v = OPx ^ i1 / j1 ^ i1
 OPvecti1j1[0]=(yortho*i1[2] - zortho*i1[1]);
 OPvecti1j1[1]=(zortho*i1[0] - xortho*i1[2]);
 OPvecti1j1[2]=(xortho*i1[1] - yortho*i1[0]);
 v= OPvecti1j1[choix]*-coefij;
//  OPx ^ j1 = u i1 ^ j1   d'ou u =  OPx ^ j1 / i1 ^ j1
 OPvecti1j1[0]=(yortho*j1[2] - zortho*j1[1]);
 OPvecti1j1[1]=(zortho*j1[0] - xortho*j1[2]);
 OPvecti1j1[2]=(xortho*j1[1] - yortho*j1[0]);
 u= OPvecti1j1[choix]*coefij;

// 7 : pour le point recherche (u,v) on cherche a connaitre la cellule auquelle il appartiend ( c1 C2 C3 c4 )
 coincellule = 0;
 if ( u < 0  && v < 0 ) coincellule=1;
else if ( u < 0  && v >= 0 ) coincellule=4;
else if ( u >= 0  && v < 0 ) coincellule=2;
else if ( u >= 0  && v >= 0 ) coincellule=5;


 if ( coincellule == 0 )
  {
    // on est ici en dehors de la grille
      cr_op = CMN__STATUT_NOK;
    //@ on recupere le premier message d'erreur pour l'emission d'une trace
      SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                                SD_BAS_ERR_SET_OpLoc,
                                SD_BAS_OpLoc_ERR_INTERP,
                                tcProcName_F,
                                __LINE__,
                                ERR_SEARCH)
     	// on imprime les message d'erreurs sur la sortie standard
      	#ifdef OPS_DEBUG
                SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, ERR_SEARCH);
      	#endif
     	// FT-3130
	msg_err = "Bordure de bandeau";
	num_lig = __LINE__;
	// .. FT-3130
      	goto fin;
  }


// 8 :l'interpolation spline2D en colonne et en ligne  est alors menee sur la zone
// ne= 3
 ne=3;
// xe1= ui1orthoav [ 1,2,3]
// xe2= vj1orthoav [ 1,4,7]
 for (i=0;i<3;i++)
  {
  xe1[i]=ui1orthoav[i];
//FA160 test sur la croissance des valeurs
  if (i>0 && xe1[i] <= xe1[i-1])
   {
    cr_op   = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
    msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
    num_lig = __LINE__;
    goto fin;
    }
  xe2[i]=vj1orthoav[i*3+1];
//FA160 test sur la croissance des valeurs
   if (i>0 && xe2[i] <= xe2[i-1])
    {
    cr_op   = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
    msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
    num_lig = __LINE__;
    goto fin;
    }
  }
// ye = colavhrr [1 2 3 4 5 6 7 8 9]
// nc = 11 ne+3
 nc=11*ne+3;
// c
// xs1 = u
// ys1 = v
// ys = col
 cr_op= SD_BAS_InterpSpline2D(
        ne,
        xe1,
        xe2,
        colavhrr,
        nc,
        c,
        &u,
        &v,
        col);
 if(cr_op != CMN__STATUT_OK)
    {
    	// FT-3130
	msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
    	cr_op = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
	num_lig = __LINE__;
	// .. FT-3130
    	goto fin;
    }

// 8 :l'interpolation spline2D en ligne  est alors menee sur la zone
// ne= 3
 ne=3;
// xe1= ui1orthoav [ 1,2,3]
// xe2= vj1orthoav [ 1,4,7]
// ye = ligavhrr [1 2 3 4 5 6 7 8 9]
// nc = 11 ne+3
// c
// xs1 = u
// ys1 = v
// ys = lig
 cr_op= SD_BAS_InterpSpline2D(
        ne,
        xe1,
        xe2,
        ligavhrr,
        nc,
        c,
        &u,
        &v,
        lig);
 if(cr_op != CMN__STATUT_OK)
    {
    	// FT-3130
	msg_err = (char *)SD_BAS_OPLOC_MSG_ERR_SPLINE2D;
    	cr_op = SD_BAS_OPLOC_ERR_AVHRR2GEOD;
	num_lig = __LINE__;
	// .. FT-3130
    	goto fin;
    }


fin:
//FA160
    // Traitement des erreurs
    //================================================================
    if ( cr_op != CMN__STATUT_OK )
    {
        // mise a jour de la variable ctx->cr.errinterne
        SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        SD_BAS_ERR_SET_OpLoc,
        cr_op,
        tcProcName_F,
        num_lig,
        msg_err)

        // on imprime le message d'erreur sur la sortie standard
        #ifdef OPS_DEBUG
            SD_BAS_PRINT_ERR_INT(ctx->cr.errinterne.nomfonction,
                                 ctx->cr.errinterne.numligne,
                                 ctx->cr.errinterne.libelle);
        #endif
    }
// sortie de la fonction
return(cr_op);

}

// FT-2997
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_AzElVectPos
	//@
	//@ Description : cette fonction permet de calculer a partir de l'azimuth 
	//@ et de l'angle d'elevation, le vecteur position correspondant.
	//@
	//@
	//@ Parameters :  
	//@         (ENTREE) azimuth : angle d'azimuth, exprime en radian, dans le repere IASI [0;2Pi]
	//@         (ENTREE) elevation : angle d'elevation, exprime en radian, dans le repere IASI [-Pi/2;Pi/2] 
	//@         (SORTIE) VectPosRM : Vecteur position dans le repere METOP
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_AzElVectPos(
	        // Entrees
	        const double azimuth,
	        const double elevation,
	        // Sortie
	        double VectPosRM[SD_BAS_DIM_ROT_MATRICE]
	            )
	{
		static const char tcProcName_F[] = "SD_BAS_AzElVectPos";
		t_CMN__Statut cr_op=CMN__STATUT_OK;	// sortie de la fonction
					
		double azim = (3 * M_PI /2) - azimuth;
		
		if (azim < 0) 
		{
			azim = azim + (2 * M_PI);
		}		
		if (azim > 2 * M_PI) 
		{
			azim = azim - (2 * M_PI);
		}
				
	   	// Calcul du vecteur position 
	   	VectPosRM[IND1(1)]= cos (elevation) * cos (azim);
	   	VectPosRM[IND1(2)]= cos (elevation) * sin (azim);	
	   	VectPosRM[IND1(3)]= -sin (elevation);		
	
		return(cr_op);	
	}            
	
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_VectPosAzEl
	//@
	//@ Description : cette fonction permet de calculer a partir du vecteur  
	//@ position, l'azimuth et l'angle d'elevation correspondant.
	//@
	//@
	//@ Parameters :  
	//@         (ENTREE) VectPosRM : Vecteur position dans le repere METOP
	//@         (SORTIE) azimuth : angle d'azimuth, exprime en radian, dans le repere IASI [0;2Pi]
	//@         (SORTIE) elevation : angle d'elevation, exprime en radian, dans le repere IASI [-Pi/2;Pi/2] 
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_VectPosAzEl(
	        // Entree
	        const double VectPosRM[SD_BAS_DIM_ROT_MATRICE],
	        // Sorties
	        double * azim,
	        double * elev
	            )
	{
		static const char tcProcName_F[] = "SD_BAS_VectPosAzEl";
		t_CMN__Statut cr_op=CMN__STATUT_OK;	// sortie de la fonction
		
		double azimuth = *azim;
		
		// Calcul de l'azimuth
		azimuth = atan2 (VectPosRM[IND1(2)],VectPosRM[IND1(1)]);
		// L'axe reference est la vitesse du satellite
		azimuth = (3 * M_PI /2) - azimuth;
		if (azimuth < 0) 
		{
			azimuth = azimuth + (2 * M_PI);
		}		
		if (azimuth > 2 * M_PI) 
		{
			azimuth = azimuth - (2 * M_PI);
		}
		*azim = azimuth;
		
		// Calcul de l'elevation
		*elev = atan2 (-VectPosRM[IND1(3)], 
						sqrt ( (VectPosRM[IND1(1)] * VectPosRM[IND1(1)]) 
								+ (VectPosRM[IND1(2)] * VectPosRM[IND1(2)]) ) );
		 
		return(cr_op);	
	}            
	            
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_TimeScanAngle2ColLin
	//@
	//@ Description : cette fonction permet de calculer les numeros de ligne  
	//@ et de colonne AVHRR correspondants a une date et un angle de balayage.
	//@
	//@
	//@ Parameters :  
	//@         (ENTREE) AvhrrTime : time
	//@         (ENTREE) AvhrrScanAngle : angle de balayage
	//@         (ENTREE) RefTime : instant de la premiÃ¨re ligne AVHRR
	//@         (SORTIE) AvhrrLin : numero de ligne AVHRR
	//@         (SORTIE) AvhrrCol : numero de colonne AVHRR 
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_TimeScanAngle2ColLin(
	        // Entrees
	        const double AvhrrTime,
	        const double AvhrrScanAngle,
	        const double RefTime,
	        // Sorties
	        double * AvhrrLin,
	        double * AvhrrCol
	            )
	{
		static const char tcProcName_F[] = "SD_BAS_TimeScanAngle2ColLin";
		t_CMN__Statut cr_op=CMN__STATUT_OK;	// sortie de la fonction
				
		// Calcul du numero de ligne AVHRR
		double AvhrrLineTime = AvhrrTime - SD_BAS_SCAN_DELTA_TIME;
		*AvhrrLin = ((AvhrrLineTime - RefTime) / (SD_BAS_LINE_STEP_TIME / 86400.)) + 1.;

		// Calcul du numero de colonne AVHRR
		*AvhrrCol = SD_BAS_PIX_NB_NADIR + 
					(AvhrrScanAngle / (2. * M_PI / (SD_BAS_LINE_STEP_TIME * SD_BAS_DIGIT_FREQ) ) );		
	
		return(cr_op);	
	}            
	
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_ColLin2TimeScanAngle
	//@
	//@ Description : cette fonction permet de calculer la date et l'angle   
	//@ de balayage correspondants a un numero de ligne et de colonne AVHRR .
	//@
	//@
	//@ Parameters :  
	//@         (ENTREE) AvhrrLin : numero de ligne AVHRR
	//@         (ENTREE) AvhrrCol : numero de colonne AVHRR 
	//@         (ENTREE) RefTime : instant de la premiÃ¨re ligne AVHRR
	//@         (SORTIE) AvhrrTime : timetime [julian day]
	//@         (SORTIE) AvhrrScanAngle : angle de balayage exprime en radian
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_ColLin2TimeScanAngle(
	        // Entrees
	        const double AvhrrLin,
	        const double AvhrrCol,
	        const double RefTime,
	        // Sorties
	        double * AvhrrTime,
	        double * AvhrrScanAngle
	            )
	{
		static const char tcProcName_F[] = "SD_BAS_ColLin2TimeScanAngle";
		t_CMN__Statut cr_op=CMN__STATUT_OK;	// sortie de la fonction
		
		// Calcul de AvhrrTime
		double AvhrrLineTime = RefTime + ((AvhrrLin - 1) * (SD_BAS_LINE_STEP_TIME/86400));
		*AvhrrTime = AvhrrLineTime + SD_BAS_SCAN_DELTA_TIME;		
		
		// Calcul de AvhrrScanAngle
		*AvhrrScanAngle = (AvhrrCol - SD_BAS_PIX_NB_NADIR) * 
						(2 * M_PI / (SD_BAS_LINE_STEP_TIME * SD_BAS_DIGIT_FREQ) );
							
		return(cr_op);	
	}            
	
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_latlon2avhrr
	//@
	//@ Description : cette fonction permet de calculer, dans le repere AVHRR  
	//@ la date et l'angle de balayage correspondant a un point situe sur 
	//@ Terre defini par sa latitude et sa longitude.
	//@
	//@
	//@ Parameters :  
	//@         (ENTREE) ctx : contexte de l'etape
	//@         (ENTREE) lat : latitude geodesique de la cible
	//@         (ENTREE) lon : longitude geodesique de la cible
	//@         (ENTREE) alt : altitude geodesique de la cible
	//@         (ENTREE) aocs : valeur du tangage, du roulis et du lacet
	//@         (ENTREE) att : valeur d'erreur du tangage, du roulis et du lacet 
	//@         (ENTREE) datt : MetOp-1 SRAR mispointing rates [deg/s]
	//@         (ENTREE) iatt : parametre d'activation de mp_target
	//@         (ENTREE) timefirst : time first line 
	//@         (ENTREE) timelast : time last line
	//@         (ENTREE) mjdr_t : UTC of the true ascending node 
	//@                           from mo_orbit init mode
	//@         (ENTREE) xm_t : Mean Kepler elements at ascending node 
	//@                         from mo_orbit init mode
	//@         (ENTREE) x_t : Osculating Kepler elements of the initial 
	//@                        state vector from mo_orbit init mode
	//@         (SORTIE) AvhrrTime : time [julian day]
	//@         (SORTIE) AvhrrScanAngle : angle de balayage exprime en radian
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_latlon2avhrr(
	        // Entrees
  			t_SD_BAS_DataThread * ctx,
	        const double lat,
	        const double lon,
	        const double alt,
	        double aocs[3],
	        double att[3], 
	        double datt[3],
	        long iatt,
	        const double timefirst,
	        const double timelast,
	        double mjdr_t[2],
	        double xm_t[6],
	        double x_t[6],
	        // Sorties
	        double * AvhrrTime,
	        double * AvhrrScanAngle
	            )
	{
	#ifdef OPS_USE_METOPLIB

		static const char tcProcName_F[] = "SD_BAS_latlon2avhrr";
		t_CMN__Statut cr_op=CMN__STATUT_OK;	// sortie de la fonction
		
		double t;					// julian instant of current iteration
		double Dt;					// correction applied to t [Julian Day]
		int nbIter;					// number of iterations
		double scalpro;				// scalar product MS by MO	
		double azimuth;				// azimuth angle in SRAR coordinate system [deg]
		double elevation;			// elevation angle in SRAR coordinate system [deg]
		double UnitVect[3];			// unitary vector of the station position in SRAR
		double VectPosSta[3];		// cartesian station position in SRAR
		double SatTransVel;			// transversal orbit velocity component [m/s]
		double DistStaSat;			// station to satellite range [m]
		double dummy = 0.;
		
		/* Declaration des variables pour les appels a mo_orbit */
		long mode_t;
		double mjdp_t[2];
		double pos_t[3];
		double vel_t[3];			// position et vitesse au noeud acendant de l'orbite
		double acc_t[3];
		double res_orbit_t[54]; 	// valeurs temporaires (acceleration)
		
		/* Declaration des variables pour les appels a mp_stavis */
		double sta[4];
		double res_sta[31];

		long status;            	// Main status flag 
		long func_id;           	// CFI function ID 
		long ierr_orbit_t[4];
		long ierr_sta[4];       	// mode d'activation et status vector
		long n;
    	t_SD_BAS_MetopMSG * msg_err;	// Message d'erreur
		
		//-----------------------
		// code sart here
		//-----------------------
		mo_silent(); 
		mp_silent();
		
	    // output variables initialisation values
	    *AvhrrTime = -999.;
	    *AvhrrScanAngle = -999.;
	    
	    // local variables initialisation values 
	    t = timefirst;
	    Dt = 0.;
	    
 		// iterative process
 		nbIter = 0;
 		while (nbIter < SD_BAS_MAX_ITER)
    	{
    		t = t + Dt;
    		nbIter = nbIter + 1;
    		
    		//-----------------------
		  	//@ Calculate the satellite position and velocity
		  	//@ Call mo_orbit with propagation mode :
		  	//@ MO_PROPAG + MO_ORBIT_RES_BAS + MO_ORBIT_RES_AUX
		  	//-----------------------
		  	mode_t = MO_PROPAG + MO_ORBIT_RES_BAS + MO_ORBIT_RES_AUX;		  
		  	//-----------------------
		  	//@  mjdr_t : from initialisation mode
		  	//@  xm_t : from initialisation mode
		  	//-----------------------
		  	mjdp_t[IND1(1)]= t;
		  	mjdp_t[IND1(2)]= dummy;
		  	//-----------------------
			//@ x_t : from initialisation mode
		  	//@ pos_t : used below
		  	//@ vel_t : used below
		  	//@ acc_t : used below
		  	//@ res_orbit_t :
		  	//@ ierr_orbit_t :
		  	//-----------------------
		  	
		  	//-----------------------
		  	//@ call mo_orbit with propagation mode
		  	//-----------------------
		  	status = mo_orbit(	&mode_t, 
		    				  	mjdr_t, 
		    					xm_t, 
		    					mjdp_t,
		    					x_t, 
		    					pos_t, 
		    					vel_t, 
		    					acc_t, 
		    					res_orbit_t, 
		    					ierr_orbit_t);
		    
		  	// mo_orbit error management
		  	if (status != MO_OK)
		  	{
		    	func_id = MO_ORBIT_ID;
		    	// si erreur appel mo_vector_msg pour recuperation du libelle
		    	status = mo_vector_msg(&func_id, ierr_orbit_t, &n, msg_err);
		    
		    	//@ on recupere le premier message d'erreur pour l'emission d'une trace
		    	cr_op   = SD_BAS_ERR_MP_ORBIT;
		    	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
		      						SD_BAS_ERR_SET_OpLoc,
		      						SD_BAS_ERR_MP_ORBIT,
		      						tcProcName_F,
		      						__LINE__,
		      						msg_err[0]);
		      						
				#ifdef OPS_DEBUG
			    	//@ print error message on standard output
		    		SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg_err[0] );
		    		status = mo_print_msg(&n, msg_err);
		    		fprintf(stdout, "\n");
				#endif
		      						
		     	goto fin;		      
		  	}
		  	
		  	// output : tranversal orbit velocity
		  	SatTransVel = res_orbit_t[MO_ORBIT_RES_TRANS_ORB_VEL];

    		//-----------------------
		  	//@ Computes satellite position in Satellite Relative Actual Reference coordinate system
		  	//@ Call mp_stavis
		  	//@ aocs_g : same as mp_target
		  	//@ att : same as mp_target 
		  	//@ datt : same as mp_target 
		  	//@ iatt : same as mp_target 
		  	//-----------------------
		  	sta[IND1(1)] = lon;	// ground station geocentric longitude [deg]
		  	sta[IND1(2)] = lat;	// ground station geodetic latitude [deg]	
		  	sta[IND1(3)] = alt;	// ground station geodetic altitude [m]
		  	sta[IND1(4)] = -90.;	// ground station minimum elevation link [deg]

		  	//-----------------------
		  	//@ call mp_stavis
		  	//-----------------------
			status = mp_stavis(	mjdp_t, 
						      	pos_t, 
						      	vel_t, 
						      	acc_t, 
						      	&iatt, 
						      	aocs, 
						      	att, 
						      	datt, 
						      	sta, 
						      	res_sta, 
						      	ierr_sta);
    		if (status != MP_OK)
    		{
      			func_id = MP_STAVIS_ID;
			 	//@ si erreur appel mp_vector_msg pour recuperation du libelle
    			mp_vector_msg(&func_id, ierr_sta, &n, msg_err);
      			//@ on recupere le premier message d'erreur pour l'emission d'une trace
    			cr_op   = SD_BAS_ERR_MP_STAVIS;
      			SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
        							SD_BAS_ERR_SET_OpLoc,
        							SD_BAS_ERR_MP_STAVIS,
        							tcProcName_F,
        							__LINE__,
        							msg_err[0]);
      			//@ on imprime les message d'erreurs sur la sortie standard
				#ifdef OPS_DEBUG
      				SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__,msg_err[0] );
      				mp_print_msg(&n, msg_err);
      				fprintf(stdout, "\n");
				#endif
     			goto fin;       
    		}
    		
		  	//-----------------------
    		// output : satellite to station azimuth SRAR coordinate system
		  	//-----------------------
    		azimuth = res_sta[MP_STA_RES_SAT2STA_AZ_SRAR];
		  	//-----------------------
    		// output : satellite to station elevation SRAR coordinate system
		  	//-----------------------
    		elevation = res_sta[MP_STA_RES_SAT2STA_EL_SRAR];
 		  	//-----------------------
    		// output : satellite to station range EF CS [m]
 		  	//-----------------------
    		DistStaSat = res_sta[MP_STA_RES_RANGE];
		  	//-----------------------
    		// convert angles from degrees to radian
		  	//-----------------------
		  	azimuth = azimuth * M_PI / SD_BAS_180;
		  	elevation = elevation * M_PI / SD_BAS_180;
		  			  	
    		//-----------------------
		  	//@ Computes unitary vector coordinates in SRAR
		  	//@ call SD_BAS_AzElVectPos
		  	//-----------------------
		  	cr_op = SD_BAS_AzElVectPos(azimuth, elevation, UnitVect);
        	if( cr_op != CMN__STATUT_OK)
  			{
            	cr_op = CMN__STATUT_NOK;
            	//@ on recupere le premier message d'erreur pour l'emission d'une trace
            	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                                	SD_BAS_ERR_SET_OpLoc,
                                	cr_op,
                                	tcProcName_F,
                                	__LINE__,
                                	SD_BAS_OPLOC_MSG_ERR_AZEL2VECTPOS)
            	//@ on imprime les message d'erreurs sur la sortie standard
            	#ifdef OPS_DEBUG
                	SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, SD_BAS_OPLOC_MSG_ERR_AZEL2VECTPOS);
            	#endif
	    		goto fin;
  			}
  			
    		//-----------------------
		  	//@ Computes vector satellite-station coordinates in SRAR
		  	//-----------------------
			VectPosSta[IND1(1)] = DistStaSat * UnitVect[IND1(1)];		  	
			VectPosSta[IND1(2)] = DistStaSat * UnitVect[IND1(2)];		  	
			VectPosSta[IND1(3)] = DistStaSat * UnitVect[IND1(3)];
			
			Dt = -VectPosSta[1] / (SatTransVel * 86400);
			
			if (abs(Dt) > SD_BAS_TIME_THRES) nbIter = SD_BAS_MAX_ITER;
    	} // Fin while
    	
    	//-----------------------
		//@ check the validity of the results
		//@ iterative process does not converge
		//-----------------------
    	if (nbIter > SD_BAS_MAX_ITER) 
		{
        	cr_op = CMN__STATUT_NOK;
        	//@ on recupere le premier message d'erreur pour l'emission d'une trace
        	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                            	SD_BAS_ERR_SET_OpLoc,
                            	cr_op,
                            	tcProcName_F,
                            	__LINE__,
                            	SD_BAS_OPLOC_MSG_ERR_LATLON2AVHRR_CONV);
        	//@ on imprime les message d'erreurs sur la sortie standard
        	#ifdef OPS_DEBUG
            	SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, SD_BAS_OPLOC_MSG_ERR_LATLON2AVHRR_CONV);
        	#endif
    		goto fin;
		}
		else if ((t < (timefirst - SD_BAS_TTOLER)) || (t > (timelast + SD_BAS_TTOLER)))
		{
    		cr_op = CMN__STATUT_NOK;
		
        	//@ on recupere le premier message d'erreur pour l'emission d'une trace
        	SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
                            	SD_BAS_ERR_SET_OpLoc,
                            	cr_op,
                            	tcProcName_F,
                            	__LINE__,
                            	SD_BAS_OPLOC_MSG_ERR_LATLON2AVHRR_TTOLER);
        	//@ on imprime les message d'erreurs sur la sortie standard
        	#ifdef OPS_DEBUG
            	SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, SD_BAS_OPLOC_MSG_ERR_LATLON2AVHRR_TTOLER);
        	#endif
    		goto fin;
		}	
		
		*AvhrrTime = t;
		*AvhrrScanAngle = atan2(VectPosSta[IND1(1)], -VectPosSta[IND1(3)]);
		
fin:	
		return(cr_op);

	#else
	static const char tcProcName_F[] = "SD_BAS_latlon2avhrr";
	t_CMN__Statut cr_op=CMN__STATUT_OK;
	// sortie de la fonction
	return(cr_op);
	#endif

	} 
	
	
	//@ ------------------------------------------------------------------*/
	//@ Procedure : SD_BAS_Init_mo_orbit
	//@
	//@ Description : cette fonction permet d'initialiser Metop orbit
	//@
	//@
	//@ Parameters :  
	//@         (ENTREE) ctx : contexte de l'etape
	//@         (ENTREE) MphrN0 :  MPHR produit N0
	//@         (SORTIE) iatt :  
	//@         (SORTIE) mjdr_t : UTC of the true ascending node 
	//@                           from mo_orbit init mode
	//@         (SORTIE) xm_t : Mean Kepler elements at ascending node 
	//@                         from mo_orbit init mode
	//@         (SORTIE) x_t : Osculating Kepler elements of the initial 
	//@                        state vector from mo_orbit init mode
	//@         (SORTIE) res_orbit_t : valeurs temporaires (acceleration)
	//@         (SORTIE) acc_t : valeurs temporaires (acceleration)
	//@
	//@ Return Status :
	//@         t_CMN__Statut
	//@         SD_BAS_STATUT_OK si la fonction a reussi
	//@         SD_BAS_STATUT_NOK si la fonction a echoue
	//@
	//@ ------------------------------------------------------------------*/
	t_CMN__Statut SD_BAS_Init_mo_orbit(
	        // Entrees
  			t_SD_BAS_DataThread * ctx,
  			t_SD_BAS_MPHR *       MphrN0,
	        // Sorties
        	long * iatt,
	        double mjdr_t[2],
	        double xm_t[6],
	        double x_t[6],
	   		double res_orbit_t[54],     
			double 	acc_t[3]
	            )
	{
	#ifdef OPS_USE_METOPLIB

		static const char tcProcName_F[] = "SD_BAS_Init_mo_orbit";
		t_CMN__Statut cr_op = CMN__STATUT_OK;	// sortie de la fonction
			
  		t_SD_BAS_GeneralTime state_vector_time;
  		
  		long mode_t;
        
  		long ierr_orbit_t[4];
  		double mjdp_t[2];
  		double pos_t[3];
  		double vel_t[3];       			/* position et vitesse au noeud acendant de l'orbite */
  		long status;                    /* Main status flag */
  		long n;                 		/* Number of error messages */
  		long func_id;                  	/* CFI function ID */
    		t_SD_BAS_MetopMSG * msg;       	/* Error messages vector */
  		
  		msg=ctx->ptmsg;
		state_vector_time.year = (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->year;
  		state_vector_time.month = (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->month;
  		state_vector_time.day = (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->day;
  		state_vector_time.hour = (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->hour;
  		state_vector_time.min = (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->min;
  		state_vector_time.sec = (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->sec;
  		state_vector_time.millis = (MphrN0->listevar.tab_var[MPHR_STATE_VECTOR_TIME].val.pgentime)->millis;

		//-----------------------
  		//@ Initialisation
  		//@ mode : MO_INIT
  		//-----------------------
  		// correction validation : on travaille en mode degrade (cf RM)
		//  mode_t = MO_INIT;
		//  mode_t = MO_INIT_GENSTATE;
		// CR 232 mise en place configuration variable
 		if (strcmp(s_METOP_ENV,METOP_ENV_TEST)==0)
    	{
    		// cas test
    		mode_t = MO_INIT_GENSTATE;
    		*iatt = 2;
    	}
  		else
    	{
    		// cas nominal
    		mode_t = MO_INIT;
    		*iatt = 3;
    	}
  
  		//-----------------------
  		//@  mjdr : a utiliser en 2
  		//@  xm : a utiliser en 2
  		//@  mjdp : issu du MPHR N1 ( comment ? )
  		//-----------------------
  		// absolute UTC
  		SD_BAS_ConvertirDateJourJulien (&state_vector_time, &(mjdp_t[0]));
  		mjdp_t[1]=0.0; // DUT constante par orbite a definir ( entre 0 et 1 )
	  
  		//-----------------------
  		//@  pos : issu du MPHR N1  ( comment ?)
  		//-----------------------
  		pos_t[0]=*(MphrN0->listevar.tab_var[MPHR_X_POSITION].val.pr8); //MPHR_X_POSITION
  		pos_t[1]=*(MphrN0->listevar.tab_var[MPHR_Y_POSITION].val.pr8); //MPHR_Y_POSITION
  		pos_t[2]=*(MphrN0->listevar.tab_var[MPHR_Z_POSITION].val.pr8); //MPHR_Z_POSITION
  
		//-----------------------
  		//@  vel : issu du MPHR N1  (comment ?)
  		//-----------------------
  		vel_t[0]=*(MphrN0->listevar.tab_var[MPHR_X_VELOCITY].val.pr8); //MPHR_X_VELOCITY
  		vel_t[1]=*(MphrN0->listevar.tab_var[MPHR_Y_VELOCITY].val.pr8); //MPHR_Y_VELOCITY
  		vel_t[2]=*(MphrN0->listevar.tab_var[MPHR_Z_VELOCITY].val.pr8); //MPHR_Z_VELOCITY
	
  		//-----------------------
		//@  acc :
  		//@  res :
  		//@  ierr : erreur
  		//@ appel mo_orbit avec les arguments :
  		//-----------------------
  		status = mo_orbit( &mode_t, 
    						mjdr_t, 
    						xm_t, 
    						mjdp_t,
    						x_t, 
    						pos_t, 
    						vel_t, 
    						acc_t, 
    						res_orbit_t, 
    						ierr_orbit_t);
  
  		if (status != MO_OK)
  		{
    		func_id = MO_ORBIT_ID;
    		//si erreur appel mo_vector_msg pour recuperation du libelle
    		mo_vector_msg(&func_id, ierr_orbit_t, &n, msg);
    
    		//@ on recupere le premier message d'erreur pour l'emission d'une trace
     		cr_op   = SD_BAS_ERR_MP_ORBIT;
    		SD_BAS_TEST_ERR_INT(ctx->cr.errinterne,
      							SD_BAS_ERR_SET_OpLoc,
      							SD_BAS_ERR_MP_ORBIT,
      							tcProcName_F,
      							__LINE__,
      							msg[0]);
    		//@ on imprime les message d'erreurs sur la sortie standard
			#ifdef OPS_DEBUG
    			SD_BAS_PRINT_ERR_INT(tcProcName_F, __LINE__, msg[0] );
    			mo_print_msg(&n, msg);
    			fprintf(stdout, "\n");
			#endif

     		goto fin;      
  		}
fin:	
		return(cr_op);
	#else
	static const char tcProcName_F[] = "SD_BAS_Init_mo_orbit";
	t_CMN__Statut cr_op=CMN__STATUT_OK;
	// sortie de la fonction
	return(cr_op);
	#endif

	} 
// ...FT-2997
