static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_OBTUTC.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ -------------------------------------------------------------------
//@ Composant : SD_DON
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_OBTUTC
//@ 
//@ Comment :
//@    this module implement the SD_DON_OBTUTC class
//@    used to offer the method to correct OBT Onboard time
//@    it contains also acces to OBT-UTC XML aux datafile
//@ 					
//@ -------------------------------------------------------------------
//@ HISTORIQUE
//@ -------------------------------------------------------------------
//@ VERSION : 18/12/2002 : Olivier Duffez
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include "SD_DON_OBTUTC.h"

#include <iomanip>

//
// Definition des constantes
//
// constante multiplicative pour passer des 1/256 secondes aux secondes
const double SD_DON_OBTUTC::OBT_TO_SEC = 256.0;


//======================================================================
//					 METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_OBTUTC
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@			Aucun
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_OBTUTC::SD_DON_OBTUTC()
  : m_DataXml(),
    m_DateOBTREF(0),
    m_DateUTCREF(),
    m_aStep(0.),
    m_bOffset(0.),
    m_Charge(CMN__FAUX)
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_OBTUTC
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@			Aucun
//@
//@ Return Status : 
//@			Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_OBTUTC::~SD_DON_OBTUTC()
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : Lire
//@
//@ Description : Cette methode permet de lire les informations
//@  stockees dans le fichier XML OBT UTC donne sous forme de fichier
//@  de donnees auxiliaires
//@
//@ Parameters :
//@			(ENTREE)  NomFichier : Nom du fichier de donnees aux
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_OBTUTC::Lire(const t_Chaine & NomFichier) 
  throw (ERR__Exception)
{
  //@ on teste si le fichier a deja ete charge
  if (m_Charge == CMN__FAUX)
    {
      //@ Initialisation du parsing du fichier
      m_DataXml.ParseFile(NomFichier);

      //@ Charge l'arbre XML en memoire et stocke les infos utiles dans les
      //@ attributs de l'objet courant :
      //@   k0 : issu de CCU_OBT
      //@   a  : issu de A_STEP
      //@   t0 : issu de CCU_UTC
      //@   b  : a priori inutile
      //@ le tag utilise est en principe le premier tag <CORRELATION> du fichier
      m_DataXml.ParseOBTUTC(*this);

      // mise a jour de l'indicateur de chargement
      m_Charge = CMN__VRAI;
    }
}


//@ ------------------------------------------------------------------*/
//@ Methode : CorrigeOBT
//@
//@ Description : Cette methode permet de transformer une date OBT
//@ issue du N0 en date UTC corrigee
//@
//@
//@ Parameters :
//@			(ENTREE)  DateOBT : date OBT a corriger
//@     (SORTIE)  DateUTC : date UTC corrigee
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
void SD_DON_OBTUTC::CorrigeOBT(t_SD_BAS_OBTTime & DateOBT,
                               t_Date &           DateUTC)
  throw (ERR__Exception)
{
  double dk = 0.;   // difference entre la date OBT fournie et celle de ref
  double nbSeconde = 0.;  
  
  // Entree dans la fonction 
  
  // ceci fait l'objet d'une demande interne pour ajouter cette methode a la 
  // classe CMN__Date 
  
  //@ on teste si le fichier a deja ete charge
  if (m_Charge == CMN__FAUX)
    {
      ERR__Exception exception;
      
      //@ Generation d'une exception
      exception.SetErrorCode(ERR__Exception::SD_DON_OBT_UNLOADED);
      exception.SetParametre("OBTUTC file has not been loaded");
      THROW(exception);
    }
 
  //@ The interpretation of the 4 fields delivered in any CORRELATION record 
  //@ is detailed below.
  //@ Assuming the following notations:
  //@ N = 2**32 , the number of OBT values, 0 <= OBT <= N - 1.
  //@ k0, extracted from tag <CCU_OBT>, the value of OBT counter at a certain instant0
  //@ t0, extracted from tag <UTC>, the UTC time at the same instant0
  //@ a,  extracted from tag <A_STEP>
  //@ b,  extracted from tag <B_OFFSET>
  //@ The conversion relies on the basic relation (t is the UTC corresponding to OBT k) :
  //@ t = t0 + a * (k - k0) or if preferred t = a * k + b
  //@
  //@ However, in order to deal with a potential rollover of the OBT,
  //@ the conversion shall be slightly enhanced. The following lines suggest
  //@ a possible algorithm.
  //@ Actually, as long as k and k0 correspond to not too distant instants,
  //@ one can assume that a rollover has occured if and only if |k-k0| > N/2
  //@ This yields to:
  
  // dk = k - k0, exprime en nb de 1/256 seconde
  dk = (double)(((long long)DateOBT) - m_DateOBTREF);
  
#ifdef DEBUG
//  cout << "dk = " << setprecision(15) << dk << endl;
#endif
  
  //@ if dk > N/2 then dk = dk - N
  if (dk > SD_BAS_NOBT / 2)
    {
      dk -= SD_BAS_NOBT;
    }
  //@ if dk < -N/2 then dk = dk + N
  else if (dk < -SD_BAS_NOBT / 2)
    {
      dk += SD_BAS_NOBT;
    }
  
  //@ at this point dk == (k - k0) modulo N and -N/2 << dk << N/2
  //@ t = t0 + a * dk (in micro-second)
  // on ajoute un nombre de secondes stocke dans un double (precision us)

  nbSeconde = m_aStep * dk / 1000000.;

  DateUTC = m_DateUTCREF + nbSeconde;                        
  
#ifdef DEBUG
//  cout << "DateOBT             = " << DateOBT << endl;
//  cout << "m_DateOBTREF        = " << m_DateOBTREF << endl;
//  cout << "m_aStep             = " << m_aStep << endl;
//  cout << "apres correction dk = " << setprecision(15) << dk << endl;
//  cout << "nombre secondes     = " << nbSeconde << endl;
//  cout << "t0                  = " << m_DateUTCREF.Chaine() << endl;
//  cout << "date UTC            = " << DateUTC.Chaine() << endl;
#endif
}


//======================================================================
//					METHODES PRIVEES 
//======================================================================
