static char rcs[]="$Header: /root/indusdrci/artf84050/IASI_OPS/OPS/SD/DON/src/SD_DON_OSV.cc,v 1.3 2011/11/25 10:17:45 dpauly Exp $";
//@ --------------------------------------------------------------------
//@ Composant : SD_DON
//@ Sous-composant : SD_DON
//@ CLASSE : SD_DON_OSV
//@ 
//@ Comment :
//@    this module implement the SD_DON_OSV class
//@    used to offer the method to determine the end of the last
//@    orbit
//@    it contains also acces to OSV xml aux datafile
//@           
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 18/12/2002 : Olivier Duffez
//@ --------------------------------------------------------------------
//@ DM-ID :IA-2100-DM-293-CNE : 08/03/2006 : B. GUENDE
//@ Reading strings IP_START and IP_END is no more case sensitive.
//@ See IA-2100-DM-293-THA.
//@ --------------------------------------------------------------------
//@ FA-ID : IA-FT-2364(FA-2100-OPS) : 20/09/2010 : S. BRUNEL
//@ Product without  manoeuvre flag raised 
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

#include "SD_DON_OSV.h"

//
// Constantes
//

// valeurs possibles du flag representant le mode de plateforme
const t_Chaine SD_DON_OSV::FLAG_ANX       = "ANX";
const t_Chaine SD_DON_OSV::FLAG_OOP_END   = "OOP_END";
const t_Chaine SD_DON_OSV::FLAG_IP_END    = "IP_END";
const t_Chaine SD_DON_OSV::FLAG_IP_START  = "IP_START";
const t_Chaine SD_DON_OSV::FLAG_OOP_START = "OOP_START";

//======================================================================
//           METHODES PUBLIQUES 
//======================================================================
//@ ------------------------------------------------------------------*/
//@ Methode : SD_DON_OSV
//@
//@ Description : Constructeur
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_OSV::SD_DON_OSV()
  : m_NomFichier(""),
    m_DataXml(),
    m_Charge(false)
{
}

//@ ------------------------------------------------------------------*/
//@ Methode : ~SD_DON_OSV
//@
//@ Description : Destructeur
//@
//@
//@ Parameters :
//@     Aucun
//@
//@ Return Status : 
//@     Aucune
//@
//@ ------------------------------------------------------------------*/
SD_DON_OSV::~SD_DON_OSV()
{
  //@ liberation du fichier XML en memoire par m_DataXml
}


//@ ------------------------------------------------------------------*/
//@ Methode : Lire
//@
//@ Description : Cette methode permet de lire les informations
//@ stockees dans le fichier xml OSV donne sous forme de fichier
//@  de donnees auxiliaires
//@
//@
//@ Parameters :
//@     (ENTREE)  NomFichier : Nom du fichier OSV 
//@
//@ Return Status : 
//@
//@ ------------------------------------------------------------------*/
void SD_DON_OSV::Lire(const t_Chaine & NomFichier) 
  throw (ERR__Exception)
{
  //@ on teste si le fichier a deja ete charge
  if (!m_Charge)
    {
     try
       {
      //@ Initialisation du parsing du fichier
        m_DataXml.ParseFile(NomFichier);
      // mise a jour de l'indicateur de chargement
        m_Charge = true;
       }
     //@ Traitement des exceptions
     // FT-2765
     catch (ERR__Exception e)
     // ...FT-2765
       {
        m_Charge = false;
        // FT-2765
        THROW(e);
        // ...FT-2765
        }
      
    }
}

//@ ------------------------------------------------------------------*/
//@ Methode : GetModePlateforme
//@
//@ Description : Cette methode permet de recuperer le mode plateforme 
//@ pour une date donnee 
//@
//@
//@ Parameters :
//@		(ENTREE)  dateDebutUTC : date UTC corrige (debut de ligne)
//@		(ENTREE)  dateFinUTC : date UTC corrige (fin de ligne)
//@     (SORTIE)  mode    : mode plateforme
//@		(SORTIE)  flagManoeuvre : vrai si la ligne comporte une manoeuvre 
//@									sinon false
//@
//@ Return Status : aucun
//@
//@ ------------------------------------------------------------------*/
// FT-2364
void SD_DON_OSV::GetModePlateforme(
						t_Date& dateDebutUTC, 
						t_Date& dateFinUTC, 
						e_BAS_ModePlateforme& mode,
						bool &flagManoeuvre)
// ...FT-2364
{
  // Flag lu dans le fichier XML OSV, sous forme de chaine
  t_Chaine flag;

  // FT-2364
  flagManoeuvre = false;
  // ...FT-2364

  //@ Initialisation par defaut du mode plateforme a PTF_NORMAL
  mode = PTF_NORMAL;

  //@ on verifie que le fichier a deja ete charge
  if (m_Charge)
    {
      //@ Parcourt de l'arbre XML en suivant l'algorithme suivant :
      //@ on se positionne en debut de fichier prealablement charge
      //@ tprec = -1
      //@ TANT que il existe des elements
      //@   deplacement dans le fichier  de maniere a se positionner sur le 
      //@   prochain element de type : <ORBIT_START>
      //@   recuperation du tag <FLAG> dans cet element
      //@   recuperation du tag <TIME> dans cet element
      //@   conversion de la chaine de caractere en t_date t
      //@ si t < dateUTC  alors flagmode = flag  
      //@ fin tantque
      // FT-2364
      m_DataXml.ParseOSV(dateDebutUTC, dateFinUTC, flag, flagManoeuvre);
      // ...FT-2364
      
      flag.Upper(); /* CR293 */
      
      //@ si le <flag> vaut ANX ou OOP_END ou IP_END alors le mode est normal
      if ( (flag == FLAG_ANX)     ||
           (flag == FLAG_OOP_END) ||
       (flag == FLAG_IP_END) )
        {
          mode = PTF_NORMAL;
        }
      
  //@ si le <flag> vaut IP_START alors le mode est manoeuvre dans le plan 
      else if (flag == FLAG_IP_START)
        {
          mode = PTF_PLAN;
        }
      
      //@ si le <flag> vaut O0P_START alors le mode est manoeuvre hors du plan.  
      else if (flag == FLAG_OOP_START)
        {
          mode = PTF_HORS_PLAN;
        }
      
      // sinon on envoit une exception
      else
        {
          
#ifdef OPS_DEBUG
          cout << "Le flag '" << flag << "' est inconnu." << endl;
#endif
          
          ERR__Exception exception;
          exception.SetErrorCode(ERR__Exception::INVALID_TREE);
          exception.SetParametre("OSV file with invalid element FLAG.");
          THROW(exception);
        }
    }
}


//======================================================================
//          METHODES PRIVEES 
//======================================================================
