// $Header: /root/indusdrci/artf84050/IASI_OPS/OPS/WOM/src/WOM__WorkOrder.cc,v 1.3 2011/11/25 10:17:46 dpauly Exp $
//@ --------------------------------------------------------------------
//@ Composant : WOM
//@ CLASSE : WOM__WorkOrder
//@
//@ Commentaire :
//@    Ce fichier contient la classe WOM__WorkOrder, qui gere
//@    un Work Order et son (ou ses) traitement(s) associe(s)
//@
//@ --------------------------------------------------------------------
//@ HISTORIQUE
//@ --------------------------------------------------------------------
//@ VERSION : 15/11/2002 : P. CABANE
//@ Creation du fichier
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-111-CN : 09/02/2004 : B. MASSART
//@ the lack of output product filename is taken into account in the method 
//@ WOM__WorkOrder::CreerRapport and listed in the report file
//@ --------------------------------------------------------------------
//@ FA-ID :IA-2100-FA-171-CN : 09/03/2006 : B. MASSART
//@ Spacecraft ID in the variable "NomProdxxxTronque" is defined in MP.env.
//@ See IA-2100-FA-171-THA.
//@ --------------------------------------------------------------------
//@ DM-ID : IA-FT-2765(DM-2100-OPS) : 21/03/2011 : S. BRUNEL
//@ PORTAGE OPS SOUS LINUX 32 BITS
//@ --------------------------------------------------------------------
//@ FIN-HISTORIQUE
//@ --------------------------------------------------------------------

//
// Fichiers include
//

// STL
#include <iostream>

// Standard
#include <unistd.h>
#include <values.h>
#include <math.h>

// Projet
#include "CMN__Commun.h"
#include "CFG__Configuration.h"
#include "CMN_PRS_Parser.h"
#include "SD_BAS_Global.h"
#include "SD_DON_Traitement.h"
#include "WOM__WorkOrder.h"


//
// Definition des constantes 
//
const t_Chaine WOM__WorkOrder::ENV_DUREE_GRANULE    = "DUREE_GRANULE";
const int      WOM__WorkOrder::DEFAUT_DUREE_GRANULE = 22*8;
const t_Chaine WOM__WorkOrder::EXT_RAPPORT          = "_001.rpt";
const t_Chaine WOM__WorkOrder::MSG_NO_MHS_TELEMETRY = "No MHSTelemetry has been defined.";
const t_Chaine WOM__WorkOrder::MSG_END_OF_STAGE     = "End of stage.";
const t_Chaine WOM__WorkOrder::ERR_REPORT_CREATION  = "Unable to create the report";
const t_Chaine WOM__WorkOrder::BAD_NUMBER_OF_TMT    = "Bad number of treatments for this dump: ";

/* FA171 */
const t_Chaine WOM__WorkOrder::s_SPACECRAFT_ID      = "SPACECRAFT_ID";
const t_Chaine WOM__WorkOrder::SPACECRAFT_ID_DEFAUT = "M01";

//=====================================================================
//                        METHODES PUBLIQUES
//=====================================================================

//@ ------------------------------------------------------------------*/
//@ Methode : WOM__WorkOrder
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__WorkOrder::WOM__WorkOrder()
  : m_Id(-1), m_Nom(""), m_Filename(""), m_DureeGranule(0)
{
  //@ Initialiser les attributs

  //@ Recuperer la variable d'environnement DUREE_GRANULE
  double duree = CFG__Configuration::GetNumerique(ENV_DUREE_GRANULE);

  //@ SI la variable n'est pas definie
  if (duree == MAXDOUBLE)
    {

#ifdef OPS_DEBUG
      cout << "Valeur defaut utilisee pour la duree d'un granule" 
           << endl;
#endif

      //@ Affecter la valeur par defaut a la duree d'un granule
      m_DureeGranule = DEFAUT_DUREE_GRANULE;
    }
  else
    {
      m_DureeGranule = int(duree);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : WOM__WorkOrder
//@
//@ Description : Constructeur
//@
//@ Parametres :
//@               (ENTREE) id : Identifiant du Work Order
//@               (ENTREE) nom : Nom du Work Order
//@               (ENTREE) fichier : Nom du fichier Work Order
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__WorkOrder::WOM__WorkOrder(int id, t_Chaine nom, t_Chaine fichier)
  : m_Id(id), m_Nom(nom), m_Filename(fichier), m_DureeGranule(0)
{
  //@ Initialiser les attributs

  //@ Recuperer la variable d'environnement DUREE_GRANULE
  double duree = CFG__Configuration::GetNumerique(ENV_DUREE_GRANULE);

  //@ SI la variable n'est pas definie
  if (duree == MAXDOUBLE)
    {

#ifdef OPS_DEBUG
      cout << "Valeur defaut utilisee pour la duree d'un granule" 
           << endl;
#endif

      //@ Affecter la valeur par defaut a la duree d'un granule
      m_DureeGranule = DEFAUT_DUREE_GRANULE;
    }
  else
    {
      m_DureeGranule = int(duree);
    }
  //@ FIN SI
}


//@ ------------------------------------------------------------------*/
//@ Methode : ~WOM__WorkOrder
//@
//@ Description : Destructeur
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
WOM__WorkOrder::~WOM__WorkOrder()
{
}


//@ ------------------------------------------------------------------*/
//@ Methode : IdentifierTraitement
//@
//@ Description : Decompose le Work Order en un ou plusieurs 
//@               traitements
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__WorkOrder::IdentifierTraitement() throw (ERR__Exception)
{
  //@ Nombre de traitements a inserer: par defaut, un seul
  int            nbTraitement = 1;

  //@ Temoin de creation de traitements de type DUMP
  t_CMN__Booleen EstUnDump    = CMN__FAUX;

  // Compteur iteration
  int            i            = 0;

 
  //@ Tester le type du Work Order
  //@ SI le type est FULL
  if (m_Data.TimeIntervalFlag() == 
      CMN_DAT_WorkOrder::WORK_ORDER_TYPE_FULL)
    {
      double dureeDump = 0.0;
      double nbTemp    = 0.0;
      t_Date dateStart;
      t_Date dateStop;


      //@ Cas d'un dump: 1 WorkOrder = n traitements
      //@ avec n > 1
      EstUnDump = CMN__VRAI;

#ifdef DEBUG
      cout << "WOM: Le traitement est un dump" << endl;
#endif

      //@ Calculer le nombre de traitements
      dateStart = m_Data.SensingStart();
      dateStop  = m_Data.SensingStop();

      dureeDump = dateStop - dateStart;
      nbTemp = dureeDump / double(m_DureeGranule);
      // FT-2765
      nbTraitement = static_cast<int>(floor(nbTemp));
      // ...FT-2765
      if (nbTemp != nbTraitement)
        {
          nbTraitement++;
        }

#ifdef DEBUG
      cout << "WOM: duree du dump : " << dureeDump << endl;
      cout << "WOM: nb Temp       : " << nbTemp << endl;
      cout << "WOM: nb traitements: " << nbTraitement << endl;
#endif

      //@ SI le nombre de traitements n'est pas superieur a 1
      if (!(nbTraitement > 1))
        {
          ERR__Exception exception;
          
          //@ Emission exception WOM_INVALID_WO
          exception.SetErrorCode(ERR__Exception::WOM_INVALID_WO);
          exception.SetParametre(BAD_NUMBER_OF_TMT +
                                 t_Chaine(nbTraitement));
          THROW(exception);
        }
      //@ FIN SI le nombre de traitements n'est pas superieur a 1
    }
  //@ FIN SI le type est FULL
  
  
  //@ En captant les exceptions
  try
    {
      //@ POUR le nombre de traitements a creer
      for (i=0; i<nbTraitement; i++)
        {
          //@ Creer un nouveau traitement a partir des
          //@ informations contenues dans le Work Order
          SD_DON_Traitement* traitement = new SD_DON_Traitement(m_Data);
          
          //@ Traitements particuliers pour un DUMP
          //@ SI c'est un DUMP
          if (EstUnDump == CMN__VRAI)
            {
              //@ Modification des dates et du SlideId
              traitement->T0(m_Data.SensingStart() + i*m_DureeGranule);
              traitement->Counter(i+1);

              //@ SI le traitement n'est pas le dernier
              if (i < (nbTraitement-1))
                {
                  //@ Positionner la date de fin a la date de debut
                  //@ + la duree d'un granule
                  traitement->T1(traitement->T0() + m_DureeGranule);

                  //@ Si c'est le premier traitement
                  if (i == 0)
                    {
                      //@ Positionner le SlideId a FIRST
                      traitement->SlideId(FIRST);
                    }
                  //@ SINON
                  else
                    {
                      //@ Positionner le SlideId a MIDDLE
                      traitement->SlideId(MIDDLE);
                    }
                  //@ FIN SI
                }
              //@ SINON
              else
                {
                  //@ Positionner la date de fin a la date de fin
                  //@ du WorkOrder
                  traitement->T1(m_Data.SensingStop());

                  //@ Positionner le SlideId a LAST
                  traitement->SlideId(LAST);
                }
              //@ FIN SI le traitement n'est pas le dernier
            }
          //@ FIN SI c'est un DUMP
 
#ifdef DEBUG
          cout << "WOM: affichage traitement a ajouter:" << endl; 
          traitement->Afficher();
#endif
          
          //@ Insertion du traitement dans la liste des traitements
          m_ListeTraitement.push_back(traitement);
        } 
      //@ FIN POUR le nombre de traitements a creer
    }

  //@ Traiter les exceptions
  catch (ERR__Exception& exception)
  {
    //@ Effacer la liste de traitement
    t_ListeTraitement::iterator i = m_ListeTraitement.begin();
    while (i != m_ListeTraitement.end())
      {
        i = m_ListeTraitement.erase(i);
      }
    m_ListeTraitement.clear();

    //@ Re-emission exception
    throw(exception);
  }
}


//@ ------------------------------------------------------------------*/
//@ Methode : TraitementCourant
//@
//@ Description : Identifie le traitement courant du Work Order
//@
//@ Parametres :
//@               Aucun
//@
//@ Valeur de retour :
//@               Traitement courant
//@
//@ ------------------------------------------------------------------*/
SD_DON_Traitement* WOM__WorkOrder::TraitementCourant() 
  throw (ERR__Exception)
{
  // Iterateur pour la liste des traitements
  t_ListeTraitement::iterator pInfo;

  // Temoin de recherche
  t_CMN__Booleen              trouve = CMN__FAUX;

 
  //@ Recherche du traitement courant
  pInfo = m_ListeTraitement.begin();
  
  //@ TANT QUE il existe des traitements ET QUE le traitement
  //@ courant n'a pas ete trouve
  while (pInfo != m_ListeTraitement.end() && (trouve == CMN__FAUX))
    {
      //@ SI l'etat du traitement indique qu'il n'a pas ete lance
      if ((*pInfo)->Etat() == SD_DON_Traitement::SD_TMT_TO_BE_RUN)
        {
          //@ Le traitement courant a ete trouve
          //@ Mettre a jour le temoin de recherche
          trouve = CMN__VRAI;
        }
      //@ SINON ce n'est pas le traitement courant
      else
        {
          //@ Passer au traitement suivant
          pInfo++;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE

  //@ SI le traitement n'a pas ete trouve
  if (trouve != CMN__VRAI)
    {
      trouve = CMN__VRAI;
      // on choisit le premier par defaut
      pInfo = m_ListeTraitement.begin();
      
//      ERR__Exception exception;
//    
//      //@ Emission exception WOM_TMT_NOT_FOUND
//      exception.SetErrorCode(ERR__Exception::WOM_TMT_NOT_FOUND);
//      exception.SetParametre("TraitementCourant()");
//      THROW(exception);
    }

  //@ Retourner le traitement trouve
  return (*pInfo);
}


//@ ------------------------------------------------------------------*/
//@ Methode : MettreAJour
//@
//@ Description : Met a jour le Work Order en fonction des 
//@               informations contenues dans le traitement
//@               passe en parametre
//@
//@ Parametres :
//@               (ENTREE) traitement : Traitement acheve
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__WorkOrder::MettreAJour(SD_DON_Traitement& traitement)
{
  // Iterateur pour la liste des traitements
  SD_DON_Traitement* traitementCourant = NULL;

 
  //@ Recherche du traitement courant
  traitementCourant = TraitementCourant();

  //@ Mettre a jour le traitement courant:
  //@  - Etat
  traitementCourant->Etat(traitement.Etat());
  
  //@  - PCD
  traitementCourant->PCD(traitement.PCD());
  
  //@  - StartTime
  traitementCourant->StartTime(traitement.StartTime());

  //@  - StopTime
  traitementCourant->StopTime(traitement.StopTime());
  
  //@  - ElapsedTime
  traitementCourant->ElapsedTime(traitement.ElapsedTime());
  
  //@  - UserTime
  traitementCourant->UserTime(traitement.UserTime());
  
  //@  - LogMessage
  traitementCourant->LogMessage(traitement.LogMessage());
  
  //@  - GeneratedFile
  traitementCourant->GeneratedFile(traitement.GeneratedFile());
      
  //@  - Liste des produits en entree
  traitementCourant->ListeProduitE(traitement.ListeProduitE());
  
  //@  - Liste des donnees auxiliaires generees
  traitementCourant->ListeDonneesAux(traitement.ListeDonneesAux());
  
  //@  - Liste des produits generes
  traitementCourant->ListeProduitS(traitement.ListeProduitS());
}


//@ ------------------------------------------------------------------*/
//@ Methode : EstTermine
//@
//@ Description : Teste s'il existe encore des traitements a executer
//@               pour le Work Order
//@
//@ Parametres :
//@               Aucun 
//@
//@ Valeur de retour :
//@               CMN__VRAI si tous les traitements ont ete executes
//@               CMN__FAUX sinon
//@
//@ ------------------------------------------------------------------*/
t_CMN__Booleen WOM__WorkOrder::EstTermine()
{
  // Iterateur pour la liste des traitements
  t_ListeTraitement::iterator pInfo;

  // Valeur de retour de la methode
  t_CMN__Booleen              estFini = CMN__VRAI;


  //@ Le Work Order est termine si tous ses traitements sont acheves
  pInfo = m_ListeTraitement.begin();

  //@ TANT QUE il existe des traitements ET QUE le traitement
  //@ a ete execute
  while (pInfo != m_ListeTraitement.end() && (estFini == CMN__VRAI))
    {
      //@ SI l'etat du traitement indique qu'il n'a pas ete lance
      if ((*pInfo)->Etat() == SD_DON_Traitement::SD_TMT_TO_BE_RUN)
        {
          //@ Le Work Order n'est pas termine
          //@ Mettre a jour la valeur de retour
          estFini = CMN__FAUX;
        }
      //@ SINON le traitement courant a ete execute
      else
        {
          //@ Passer au traitement suivant
          pInfo++;
        }
      //@ FIN SI
    }
  //@ FIN TANT QUE


  //@ Retourner la valeur de retour de la methode
  return estFini; 
}


//@ ------------------------------------------------------------------*/
//@ Methode : CreerRapport
//@
//@ Description : Cree un fichier de rapport pour le Work Order
//@
//@ Parametres :
//@               (ENTREE/SORTIE) rapport : Nom du rapport cree:
//@                      - en entree, le repertoire ou il se trouve
//@                      - en sortie le nom court du fichier rapport
//@               (SORTIE) listeProduit : Liste des fichiers de
//@                                       produits generes par 
//@                                       le Work Order
//@               (SORTIE) listeAux : Liste des fichiers de donnees
//@                                   auxiliaires generes par le
//@                                   Work Order
//@
//@ Valeur de retour :
//@               Aucune
//@
//@ ------------------------------------------------------------------*/
void WOM__WorkOrder::CreerRapport(t_Chaine& rapport, 
                                  t_ListeChaine& listeProduit,
                                  t_ListeChaine& listeAux)
{
  // Nom complet du fichier rapport
  t_Chaine                    fichierRapport("");

  // Iterateur pour la liste des traitements
  t_ListeTraitement::iterator pInfo;

  // Iterateur pour la liste des fichiers utilises en entree
  t_ListeChaine::iterator     pChaine;

  // Liste des StageInfo
  t_ListeStageInfo            listeStageInfo;

  // Temoin d'arret
  t_CMN__Booleen              continuer = CMN__VRAI;

  // Indicateur de presence des noms de produits de niveau 1
  t_CMN__Booleen              ProduitVERTrouve = CMN__FAUX;
  t_CMN__Booleen              ProduitENGTrouve = CMN__FAUX;
  t_CMN__Booleen              Produit1ATrouve  = CMN__FAUX;
  t_CMN__Booleen              Produit1BTrouve  = CMN__FAUX;
  t_CMN__Booleen              Produit1CTrouve  = CMN__FAUX;
  t_CMN__Booleen              ProduitCTXTrouve = CMN__FAUX;

  /* FA171 */
  t_Chaine                    SpacecraftID("");
  t_Chaine                    NomProdVERTronqueSCID("");
  t_Chaine                    NomProdENGTronqueSCID("");
  t_Chaine                    NomProd1ATronqueSCID("");
  t_Chaine                    NomProd1BTronqueSCID("");
  t_Chaine                    NomProd1CTronqueSCID("");
  t_Chaine                    NomProdCTXTronqueSCID("");
  /* FIN FA171 */
  
  //@ En captant les exceptions
  try
  {
    CMN_PRS_Parser parser;
    
    /* FA171 */
    //@ Recuperer l'identificateur du satellite
    SpacecraftID = getenv(s_SPACECRAFT_ID.c_str());
    
    if (SpacecraftID.empty())
    {
      SpacecraftID=SPACECRAFT_ID_DEFAUT;
    }
    
    NomProdVERTronqueSCID = NomProdVERTronque + SpacecraftID;
    NomProdENGTronqueSCID = NomProdENGTronque + SpacecraftID;
    NomProd1ATronqueSCID  = NomProd1ATronque  + SpacecraftID;
    NomProd1BTronqueSCID  = NomProd1BTronque  + SpacecraftID;
    NomProd1CTronqueSCID  = NomProd1CTronque  + SpacecraftID;
    NomProdCTXTronqueSCID = NomProdCTXTronque + SpacecraftID;
    /* FIN FA171 */
    
    //@ Construire le nom du fichier rapport: 
    //@   - parametre rapport (chemin du repertoire temporaire) + 
    //@     nom du WorkOrder + extension rapport
    //@ et le nom du rapport:
    //@   - nom du WorkOrder + extension rapport
    fichierRapport = rapport + m_Nom + EXT_RAPPORT;
    rapport = m_Nom + EXT_RAPPORT;
    
    //@ Construire la liste des StageInfo: 
    //@ 1 StageInfo pour 1 traitement realise
    pInfo = m_ListeTraitement.begin();
    
    //@ TANT QUE il existe des traitements ET QUE le traitement
    //@ a ete execute
    while (pInfo != m_ListeTraitement.end() && 
          (continuer == CMN__VRAI))
    {
      //@ SI l'etat du traitement indique qu'il n'a pas ete lance
      if ((*pInfo)->Etat() == SD_DON_Traitement::SD_TMT_TO_BE_RUN)
      {
        //@ Mettre a jour le temoin d'arret
        continuer = CMN__FAUX;
      }
      
      //@ SINON le traitement courant a ete execute
      else
      {
        // StageInfo
        CMN_DAT_StageInfo stageInfo;
        
        // GeneratedFile
        t_GeneratedFile generatedFileProduit = (*pInfo)->GeneratedFile();
        t_GeneratedFile generatedFileAux;
        
        //@ Definir le StageName
        stageInfo.StageName((*pInfo)->TypeTraitementStr());
        
        //@ POUR tous les fichiers utilises en entree
        for (pChaine = (*pInfo)->ListeProduitE().begin(); 
             pChaine != (*pInfo)->ListeProduitE().end(); 
             pChaine++)
        {
          //@ Ajouter le fichier dans la liste des fichiers utilises 
          stageInfo.AjouterUsedFileName(*pChaine);
        }
        //@ FIN POUR
        
        // Remarque: des fichiers ne sont produits que pour le
        // dernier traitement de la liste m_ListeTraitement
        
        //@ POUR tous les fichiers produits
        for (pChaine = (*pInfo)->ListeProduitS().begin(); 
             pChaine != (*pInfo)->ListeProduitS().end(); 
             pChaine++)
        {
          //@ SI le traitement s'est bien deroule
          if ((*pInfo)->Etat() == SD_DON_Traitement::SD_TMT_OK)
          {
            //@ Ajouter une section GeneratedFile pour le 
            //@ fichier
            generatedFileProduit.FileName(*pChaine);              
            stageInfo.AjouterGeneratedFile(generatedFileProduit);
            
            //@ Mettre a jour les flag d'existence
            /* FA171 */
            if ( !strncmp((*pChaine).c_str(), NomProdVERTronqueSCID.c_str(), SD_BAS_LGNOMPRODUITL1TRONQUE) )
            {
              ProduitVERTrouve = CMN__VRAI;
            }
            
            /* FA171 */
            if ( !strncmp((*pChaine).c_str(), NomProdENGTronqueSCID.c_str(), SD_BAS_LGNOMPRODUITL1TRONQUE) )
            {
              ProduitENGTrouve = CMN__VRAI;
            }
            
            /* FA171 */
            if ( !strncmp((*pChaine).c_str(), NomProd1ATronqueSCID.c_str(), SD_BAS_LGNOMPRODUITL1TRONQUE) )
            {
              Produit1ATrouve = CMN__VRAI;
            }
            
            /* FA171 */
            if ( !strncmp((*pChaine).c_str(), NomProd1BTronqueSCID.c_str(), SD_BAS_LGNOMPRODUITL1TRONQUE) )
            {
              Produit1BTrouve = CMN__VRAI;
            }
            
            /* FA171 */
            if ( !strncmp((*pChaine).c_str(), NomProd1CTronqueSCID.c_str(), SD_BAS_LGNOMPRODUITL1TRONQUE) )
            {
              Produit1CTrouve = CMN__VRAI;
            }
          }
          //@ FIN SI le traitement s'est bien deroule
          
          //@ Ajouter le fichier dans la liste des fichiers
          //@ produits
          listeProduit.push_back(*pChaine);
        }
        //@ FIN POUR
        
        //@ SI Le produit VER manque dans les produits de sortie ALORS
	       if ( (ProduitVERTrouve == CMN__FAUX) && 
	            ((*pInfo)->TypeTraitement() == CMN__StageTM::L1a) )
        {
          //@ Inserer le nom tronque dans le rapport
          generatedFileProduit.FileName(NomProdVERTronqueSCID); /* FA171 */
          stageInfo.AjouterGeneratedFile(generatedFileProduit);
        }
        
        //@ SI Le produit ENG manque dans les produits de sortie ALORS
	       if ( (ProduitENGTrouve == CMN__FAUX) && 
	            ( ((*pInfo)->TypeTraitement() == CMN__StageTM::L1a) || 
               ((*pInfo)->TypeTraitement() == CMN__StageTM::L1b) || 
               ((*pInfo)->TypeTraitement() == CMN__StageTM::L1c) ) )
        {
          //@ Inserer le nom tronque dans le rapport
          generatedFileProduit.FileName(NomProdENGTronqueSCID); /* FA171 */
          stageInfo.AjouterGeneratedFile(generatedFileProduit);
        }
        
        //@ SI Le produit 1A manque dans les produits de sortie ALORS
	       if ( (Produit1ATrouve == CMN__FAUX) && 
	            ((*pInfo)->TypeTraitement() == CMN__StageTM::L1a) )
        {
          //@ Inserer le nom tronque dans le rapport
          generatedFileProduit.FileName(NomProd1ATronqueSCID); /* FA171 */
          stageInfo.AjouterGeneratedFile(generatedFileProduit);
        }
        
        //@ SI Le produit 1B manque dans les produits de sortie ALORS
	       if ( (Produit1BTrouve == CMN__FAUX) && 
	            ( ((*pInfo)->TypeTraitement() == CMN__StageTM::L1a) || 
               ((*pInfo)->TypeTraitement() == CMN__StageTM::L1b) ) )
        {
          //@ Inserer le nom tronque dans le rapport
          generatedFileProduit.FileName(NomProd1BTronqueSCID); /* FA171 */
          stageInfo.AjouterGeneratedFile(generatedFileProduit);
        }
        
        //@ SI Le produit 1C manque dans les produits de sortie ALORS
	       if ( (Produit1CTrouve == CMN__FAUX) &&
	            ( ((*pInfo)->TypeTraitement() == CMN__StageTM::L1a) || 
               ((*pInfo)->TypeTraitement() == CMN__StageTM::L1b) || 
               ((*pInfo)->TypeTraitement() == CMN__StageTM::L1c) ) )
        {
          //@ Inserer le nom tronque dans le rapport
          generatedFileProduit.FileName(NomProd1CTronqueSCID); /* FA171 */
          stageInfo.AjouterGeneratedFile(generatedFileProduit);
        }
        
        //@ POUR tous les fichiers auxiliaires generes
        for (pChaine = (*pInfo)->ListeDonneesAux().begin(); 
             pChaine != (*pInfo)->ListeDonneesAux().end(); 
             pChaine++)
        {
          //@ SI le traitement s'est bien deroule
          if ((*pInfo)->Etat() == SD_DON_Traitement::SD_TMT_OK)
          {
            //@ Ajouter une section GeneratedFile pour le fichier
            generatedFileAux.FileName(*pChaine);
            stageInfo.AjouterGeneratedFile(generatedFileAux);
            
            /* FA171 */
            if ( !strncmp((*pChaine).c_str(), NomProdCTXTronqueSCID.c_str(), SD_BAS_LGNOMPRODUITL1TRONQUE) )
            {
              ProduitCTXTrouve = CMN__VRAI;
            }
          }
          //@ FIN SI le traitement s'est bien deroule
          
          //@ Ajouter le fichier dans la liste des fichiers
          //@ auxiliaires
          listeAux.push_back(*pChaine);
        }
        //@ FIN POUR
        
        //@ SI Le produit CTX manque dans les produits de sortie ALORS
	       if ( (ProduitCTXTrouve == CMN__FAUX) &&
	            ((*pInfo)->TypeTraitement() == CMN__StageTM::L1a) )
        {
          //@ Inserer le nom tronque dans le rapport
          generatedFileProduit.FileName(NomProdCTXTronqueSCID); /* FA171 */
          stageInfo.AjouterGeneratedFile(generatedFileProduit);
        }
        
        //@ Definir le MHSTelemetry
        stageInfo.MHSTelemetry(MSG_NO_MHS_TELEMETRY);
        
        //@ Ajouter une section ProcessingInfo pour le fichier:
        //@  - start time
        stageInfo.ProcessingStart((*pInfo)->StartTime());
        
        //@  - stop time
        stageInfo.ProcessingStop((*pInfo)->StopTime());
        
        //@  - elapsed time
        stageInfo.ElapsedTime((*pInfo)->ElapsedTime());
        
        //@  - user time
        stageInfo.UserTime((*pInfo)->UserTime());
        
        //@ Renseigner le message de log
        if ((*pInfo)->LogMessage().empty())
        {
          stageInfo.LogMessage(MSG_END_OF_STAGE);
        }
        else
        {
          stageInfo.LogMessage((*pInfo)->LogMessage());
        }
        
        //@ Ajouter le StageInfo dans la liste
        listeStageInfo.push_back(stageInfo);
        
        //@ Passer au traitement suivant
        pInfo++;
      }
      //@ FIN SI
    }
    //@ FIN TANT QUE il existe des traitements 
    //@ ET QUE le traitement a ete execute
    
    //@ Initialiser le parsing du fichier WorkOrder
    parser.ParseFile(m_Filename);
    
    //@ Creer le fichier rapport sous format XML
    parser.EcrireReport(fichierRapport, listeStageInfo);
  }

  //@ Traiter les exceptions
  catch (ERR__Exception &exception) 
  {
    //@ Re-emission de l'exception
    exception.SetParametre(ERR_REPORT_CREATION); 
    throw(exception);
  }
}


//=====================================================================
//                        METHODES PRIVEES
//=====================================================================
