////////////////////////////////////////////////////////////////////////////////
//
//	File: amsua1reader.hpp
//
//	Provides: AMSUA1Reader filter
//
//	Purpose: Convert NOAA AMSU-A L1b products to AMSU-A1 source packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef amsua1reader_defined
#define amsua1reader_defined

#include "config.hpp"
#include "amsuareader.hpp"
#include "ccsdsproducer.hpp"
#include "ccsdsgeneratorbase.hpp"

#define Config Config::AMSUA1

class AMSUA1Reader :
	public AMSUAReader,
	public CCSDSProducer,
	public CCSDSGeneratorBase<CCSDS_AMSUA1>
{
public:

	AMSUA1Reader(ProductProducer& _source, const string& config_file = "")
		: AMSUAReader(_source)
		{
			if (!config_file.empty())
			{
				Config::file.read(config_file);
			}

			// check for no input file
			if (!data)
			{
				return;
			}

			// prepare for source packet output

			ccsds_set_sequence_count(
				Config::initial_sequence_count);

			if (Config::utc_ignore_input_time)
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);

				ccsds_modify_utc_base_time(data->gen);
			}
			else
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);
			}

			ccsds_set_obt_zero_time(
				data->gen);

			ccsds_apply_obt_offsets(
				Config::obt_coarse_offset,
				Config::obt_fine_offset,
				Config::obt_fixed_fine_time);

//	    if (Config::obt_fixed_fine_time)
//	    {
//		cout << "AMSUA1 FIXED FINE TIME true\n";
//	    }
//	    else
//	    {
//		cout << "AMSUA1 FIXED FINE TIME false\n";
//	    }


		}


    //
    // get_next_ccsds
    //


    virtual CCSDS* get_next_ccsds(void)
		{
			if (!source.get_current_record())
			{
				if (spinner.get())
				{
					spinner->end();
				}
				return NULL;
			}
			else
			{
				data = (NOAA_L1b_AMSUA_Data*)source.get_current_record();
			}

			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			}
			else
			{
				spinner->tick();
			}

			// prepare next source packet

			ccsds_initialise();
//	    cout << "amsua1reader calling set_time fixed_time is "
			// << ccsds_obt_fixed_fine_time << endl;
			ccsds_set_time(data->gen);
			// copy scan line number into sequence number if configured
			if (Config::scan_line_number_as_sequence_count)
			{
				ccsds_set_sequence_count(data->gen.scan_line_number+
										 Config::initial_sequence_count);
			}

			const int APP_DATA_WORDS = 1040; // 2080 bytes
			const int MEASURE_MODE_WORDS = 620; // 1240 bytes
			const int UNIQUE_MODE_WORDS = 560; // 1120 bytes

			// prepare filler mask
			Mask mask[APP_DATA_WORDS]; // note each mask element equals two
			// bytes
			fill_mask(mask, APP_DATA_WORDS, Config::filler_pattern);
			speckle_mask(mask, APP_DATA_WORDS, Config::filler_random_speckle);

			// check mode
			if ((data->a1.digital_a.housekeeping[0]&2) == 2)
			{
				// measurement mode
				check_mask(mask, APP_DATA_WORDS, MEASURE_MODE_WORDS);
				AMSUA1MeasurementScienceData sci;
				fill_science_data(sci);
				apply_mask((uint16_be*)ccsds_packet.payload, (uint16_be*)&sci, mask,
						   APP_DATA_WORDS);
			}
			else
			{
				// unique mode
				check_mask(mask, APP_DATA_WORDS, UNIQUE_MODE_WORDS);
				// 560*2 = 1120 sci data bytes
				AMSUA1UniqueScienceData sci;
				fill_science_data(sci);
				apply_mask((uint16_be*)ccsds_packet.payload, (uint16_be*)&sci, mask,
						   APP_DATA_WORDS);
			}

			record_count++;
			return ccsds_finalise();
		}

    unsigned int amsua_count(unsigned int noaa_value)
		{
			return (noaa_value << 1);
		}

    unsigned int amsua_refpos(unsigned int noaa_value)
		{
			int referr = Config::reflector_pos_error_bit;
			if (referr != 0 && referr != 1)
			{
				error << "Configuration parameter Reflector-Pos-Error-Bit "
					  << "must be set to 0 or 1\n";
				referr = 0;
			}
			referr <<= 1;

			return ((noaa_value<<2) | referr);
		}

    unsigned int amsua_temp(unsigned int noaa_value)
		{
			return (noaa_value << 1);
		}

    unsigned int amsua_refvoltage(unsigned int noaa_value)
		{
			return (noaa_value << 1);
		}

    void fill_science_data(AMSUA1MeasurementScienceData& sci)
		{
			// header
			sci.sync_sequence[0] = 0xff;
			sci.sync_sequence[1] = 0xff;
			sci.sync_sequence[2] = 0xff;
			sci.serial_number = Config::serial_number;
			sci.digital_housekeeping[0] = data->a1.digital_a.housekeeping[0];
			sci.digital_housekeeping[1] = data->a1.digital_a.housekeeping[1];
			sci.digital_housekeeping[2] = data->a1.digital_a.housekeeping[2];
			sci.digital_housekeeping[3] = Config::unit_id;

			// data
//	    memcpy((char*)sci.scene, (char*)data->a1.scene,
//		   1020 + 60 + 92 + 60);

			// scene data

			for(int i=0; i<30; i++)
			{
				sci.scene[i].first_reading[0] =
					amsua_refpos(data->a1.scene[i].first_reading[0]);
				sci.scene[i].first_reading[1] =
					amsua_refpos(data->a1.scene[i].first_reading[1]);
				sci.scene[i].second_reading[0] =
					amsua_refpos(data->a1.scene[i].second_reading[0]);
				sci.scene[i].second_reading[1] =
					amsua_refpos(data->a1.scene[i].second_reading[1]);
				for(int j=0; j<13; j++)
				{
					sci.scene[i].counts[j] =
						amsua_count(data->a1.scene[i].counts[j]);
				}
			}

			// cold calibration

			sci.cold_calibration.first_reading[0] =
				amsua_refpos(data->a1.cold_calibration.first_reading[0]);
			sci.cold_calibration.first_reading[1] =
				amsua_refpos(data->a1.cold_calibration.first_reading[1]);
			sci.cold_calibration.second_reading[0] =
				amsua_refpos(data->a1.cold_calibration.second_reading[0]);
			sci.cold_calibration.second_reading[1] =
				amsua_refpos(data->a1.cold_calibration.second_reading[1]);
			for(int i=0; i<13; i++)
			{
				sci.cold_calibration.counts_1[i] =
					amsua_count(data->a1.cold_calibration.counts_1[i]);
				sci.cold_calibration.counts_2[i] =
					amsua_count(data->a1.cold_calibration.counts_2[i]);
			}

			// temperature sensors + reference voltage

			for(int i=0; i<45; i++)
			{
				sci.temperature_sensors[i] =
					amsua_temp(data->a1.temperature_sensors[i]);
			}
			sci.reference_voltage =
				amsua_refvoltage(data->a1.temperature_sensors[45]);

			// warm calibration

			sci.warm_calibration.first_reading[0] =
				amsua_refpos(data->a1.warm_calibration.first_reading[0]);
			sci.warm_calibration.first_reading[1] =
				amsua_refpos(data->a1.warm_calibration.first_reading[1]);
			sci.warm_calibration.second_reading[0] =
				amsua_refpos(data->a1.warm_calibration.second_reading[0]);
			sci.warm_calibration.second_reading[1] =
				amsua_refpos(data->a1.warm_calibration.second_reading[1]);
			for(int i=0; i<13; i++)
			{
				sci.warm_calibration.counts_1[i] =
					amsua_count(data->a1.warm_calibration.counts_1[i]);
				sci.warm_calibration.counts_2[i] =
					amsua_count(data->a1.warm_calibration.counts_2[i]);
			}
		}

    void fill_science_data(AMSUA1UniqueScienceData& sci)
		{
			// header
			sci.sync_sequence[0] = 0xff;
			sci.sync_sequence[1] = 0xff;
			sci.sync_sequence[2] = 0xff;
			sci.serial_number = Config::serial_number;
			sci.digital_housekeeping[0] = data->a1.digital_a.housekeeping[0];
			sci.digital_housekeeping[1] = data->a1.digital_a.housekeeping[1];
			sci.digital_housekeeping[2] = data->a1.digital_a.housekeeping[2];
			sci.digital_housekeeping[3] = Config::unit_id;

			// scene data
//	    memcpy((char*)sci.scene, (char*)data->a1.scene,
//		   1020);

			for(int i=0; i<30; i++)
			{
				sci.scene[i].first_reading[0] =
					amsua_refpos(data->a1.scene[i].first_reading[0]);
				sci.scene[i].first_reading[1] =
					amsua_refpos(data->a1.scene[i].first_reading[1]);
				sci.scene[i].second_reading[0] =
					amsua_refpos(data->a1.scene[i].second_reading[0]);
				sci.scene[i].second_reading[1] =
					amsua_refpos(data->a1.scene[i].second_reading[1]);
				for(int j=0; j<13; j++)
				{
					sci.scene[i].counts[j] =
						amsua_count(data->a1.scene[i].counts[j]);
				}
			}

			// temperature sensors + reference voltage

//	    memcpy((char*)sci.temperature_sensors,
//		   (char*)data->a1.temperature_sensors, 92);

			for(int i=0; i<45; i++)
			{
				sci.temperature_sensors[i] =
					amsua_temp(data->a1.temperature_sensors[i]);
			}
			sci.reference_voltage =
				amsua_refvoltage(data->a1.temperature_sensors[45]);
		}

    //
    // output_text_summary
    //

    void output_text_summary( ostream& os = cout )
		{
			string start_time = TimeConv::abs_year_day_millisecond_to_string(
				start_year,
				start_day,
				start_millisecond);

			string end_time = TimeConv::abs_year_day_millisecond_to_string(
				data->gen.scan_line_year,
				data->gen.scan_line_day_of_year,
				data->gen.scan_line_millisecond_of_day);

			os << "AMSUA1Reader report:\n"
			   << "    " << record_count << " scan lines processed ("
			   << start_scan_line_number << "-" << data->gen.scan_line_number
			   << ")\n    Product start time " << start_time << endl
			   << "    Product end time " << end_time << endl
			   << "    Samples per scan line 13\n"
			   << endl;
		}

    //
    // output_text_config
    //

    void output_text_config( ostream& os = cout )
		{
			os << "AMSUA1Reader configuration:\n"
			   << "    Serial number " << Config::serial_number << endl
			   << "    Unit-ID " << Config::serial_number << endl
			   << "    Filler pattern '"
			   << static_cast<string>(Config::filler_pattern) << "'\n"
			   << "    Filler random speckle " << Config::filler_random_speckle
			   << endl
			   << "    Scan line number as sequence count " <<
				(static_cast<bool>(Config::scan_line_number_as_sequence_count)?
				 "on":"off") << endl
			   << "    Initial sequence count "
			   << Config::initial_sequence_count << endl
			   << "    UTC days offset " << Config::utc_days_offset << endl
			   << "    UTC milliseconds offset " <<
				Config::utc_milliseconds_offset << endl
			   << "    UTC microseconds offset " <<
				Config::utc_microseconds_offset << endl
			   << "    Ignore UTC time " <<
				(static_cast<bool>(Config::utc_ignore_input_time)?"on":"off")
			   << endl
			   << "    OBT coarse offset " << Config::obt_coarse_offset << endl
			   << "    OBT fine offset " << Config::obt_fine_offset << endl
			   << "    OBT fixed fine time " << Config::obt_fixed_fine_time
			   << endl << endl;
		}

    EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

};

#undef Config

#endif
