////////////////////////////////////////////////////////////////////////////////
//
//	File: amsua2reader.hpp
//
//	Provides: AMSUA2Reader filter
//
//	Purpose: Convert NOAA AMSU-A L1b products to AMSU-A2 source packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef amsua2reader_defined
#define amsua2reader_defined

#include "config.hpp"
#include "interface.hpp"
#include "amsuareader.hpp"
#include "ccsdsproducer.hpp"
#include "ccsdsgeneratorbase.hpp"

#define Config Config::AMSUA2

class AMSUA2Reader :
	public AMSUAReader,
	public CCSDSProducer,
	public CCSDSGeneratorBase<CCSDS_AMSUA2>
{
public:

	AMSUA2Reader(ProductProducer& _source, const string& config_file = "")
		: AMSUAReader(_source)
		{
			if (!config_file.empty())
			{
				Config::file.read(config_file);
			}

			// check for no input file
			if (!data)
			{
				return;
			}

			// prepare for source packet output

			ccsds_set_sequence_count(
				Config::initial_sequence_count);

			if (Config::utc_ignore_input_time)
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);

				ccsds_modify_utc_base_time(data->gen);
			}
			else
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);
			}

			ccsds_set_obt_zero_time(
				data->gen);

			ccsds_apply_obt_offsets(
				Config::obt_coarse_offset,
				Config::obt_fine_offset,
				Config::obt_fixed_fine_time);

//		if (Config::obt_fixed_fine_time)
//		{
//		cout << "AMSUA2 FIXED FINE TIME true\n";
//		}
//		else
//		{
//		cout << "AMSUA2 FIXED FINE TIME false\n";
//		}

		}


	//
	// get_next_ccsds
	//


	virtual CCSDS* get_next_ccsds(void)
		{
			if (!source.get_current_record())
			{
				if (spinner.get())
				{
					spinner->end();
				}
				return NULL;
			}

			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			}
			else
			{
				spinner->tick();
			}

			// prepare next source packet

			ccsds_initialise();
//		cout << "amsua2reader calling set_time fixed_time is "
			// << ccsds_obt_fixed_fine_time << endl;
			ccsds_set_time(data->gen);
			// copy scan line number into sequence number if configured
			if (Config::scan_line_number_as_sequence_count)
			{
				ccsds_set_sequence_count(data->gen.scan_line_number+
										 Config::initial_sequence_count);
			}

			// prepare filler mask
			Mask mask[560]; // note each mask element equals two bytes
			fill_mask(mask, 560, Config::filler_pattern);
			speckle_mask(mask, 560, Config::filler_random_speckle);

			// check mode
			if ((data->a2.digital_a.housekeeping[0]&2) == 2)
			{
				// measurement mode
				check_mask(mask, 560, 156); // 156*2 = 312 sci data bytes
				AMSUA2MeasurementScienceData sci;
				fill_science_data(sci);
				apply_mask((uint16_be*)ccsds_packet.payload, (uint16_be*)&sci, mask,
						   560);
			}
			else
			{
				// unique mode
				check_mask(mask, 560, 144); // 144*2 = 288 sci data bytes
				AMSUA2UniqueScienceData sci;
				fill_science_data(sci);
				apply_mask((uint16_be*)ccsds_packet.payload, (uint16_be*)&sci, mask,
						   560);
			}

//		change_record(current_record+1);

			source.skip_records(1);
			if (source.get_current_record())
			{
				data = (NOAA_L1b_AMSUA_Data*)source.get_current_record();
			}
			record_count++;

			return ccsds_finalise();
		}

	unsigned int amsua_count(unsigned int noaa_value)
		{
			return noaa_value << 1;
		}

	unsigned int amsua_refpos(unsigned int noaa_value)
		{
			int referr = Config::reflector_pos_error_bit;
			if (referr != 0 && referr != 1)
			{
				error << "Configuration parameter Reflector-Pos-Error-Bit "
					  << "must be set to 0 or 1\n";
				referr = 0;
			}
			referr <<= 1;

			return ((noaa_value<<2) | referr);
		}

	unsigned int amsua_temp(unsigned int noaa_value)
		{
			return (noaa_value << 1);
		}

	unsigned int amsua_refvoltage(unsigned int noaa_value)
		{
			return (noaa_value << 1);
		}

	void fill_science_data(AMSUA2MeasurementScienceData& sci)
		{
			// header
			sci.sync_sequence[0] = 0xff;
			sci.sync_sequence[1] = 0xff;
			sci.sync_sequence[2] = 0xff;
			sci.serial_number = Config::serial_number;
			sci.digital_housekeeping[0] = data->a2.digital_a.housekeeping[0];
			sci.digital_housekeeping[1] = data->a2.digital_a.housekeeping[1];
			sci.digital_housekeeping[2] = data->a2.digital_a.housekeeping[2];
			sci.digital_housekeeping[3] = Config::unit_id;

			// scene data

//		memcpy((char*)sci.scene, (char*)data->a2.scene,
//		   240 + 12 + 40 + 12);

			for(int i=0; i<30; i++)
			{
				sci.scene[i].first_reading =
					amsua_refpos(data->a2.scene[i].first_reading);
				sci.scene[i].second_reading =
					amsua_refpos(data->a2.scene[i].second_reading);
				for(int j=0; j<2; j++)
				{
					sci.scene[i].counts[j] =
						amsua_count(data->a2.scene[i].counts[j]);
				}
			}

			// cold calibration

			sci.cold_calibration.first_reading =
				amsua_refpos(data->a2.cold_calibration.first_reading);
			sci.cold_calibration.second_reading =
				amsua_refpos(data->a2.cold_calibration.second_reading);
			for(int i=0; i<2; i++)
			{
				sci.cold_calibration.counts_1[i] =
					amsua_count(data->a2.cold_calibration.counts_1[i]);
				sci.cold_calibration.counts_2[i] =
					amsua_count(data->a2.cold_calibration.counts_2[i]);
			}

			// temperature sensors + reference voltage

			for(int i=0; i<19; i++)
			{
				sci.temperature_sensors[i] =
					amsua_temp(data->a2.temperature_sensors[i]);
			}
			sci.reference_voltage =
				amsua_refvoltage(data->a2.temperature_sensors[19]);

			// warm calibration

			sci.warm_calibration.first_reading =
				amsua_refpos(data->a2.warm_calibration.first_reading);
			sci.warm_calibration.second_reading =
				amsua_refpos(data->a2.warm_calibration.second_reading);
			for(int i=0; i<2; i++)
			{
				sci.warm_calibration.counts_1[i] =
					amsua_count(data->a2.warm_calibration.counts_1[i]);
				sci.warm_calibration.counts_2[i] =
					amsua_count(data->a2.warm_calibration.counts_2[i]);
			}
		}

	void fill_science_data(AMSUA2UniqueScienceData& sci)
		{
			// header
			sci.sync_sequence[0] = 0xff;
			sci.sync_sequence[1] = 0xff;
			sci.sync_sequence[2] = 0xff;
			sci.serial_number = Config::serial_number;
			sci.digital_housekeeping[0] = data->a2.digital_a.housekeeping[0];
			sci.digital_housekeeping[1] = data->a2.digital_a.housekeeping[1];
			sci.digital_housekeeping[2] = data->a2.digital_a.housekeeping[2];
			sci.digital_housekeeping[3] = Config::unit_id;

			// scene data
//		memcpy((char*)sci.scene, (char*)data->a2.scene,
//		   240);

			for(int i=0; i<30; i++)
			{
				sci.scene[i].first_reading =
					amsua_refpos(data->a2.scene[i].first_reading);
				sci.scene[i].second_reading =
					amsua_refpos(data->a2.scene[i].second_reading);
				for(int j=0; j<2; j++)
				{
					sci.scene[i].counts[j] =
						amsua_count(data->a2.scene[i].counts[j]);
				}
			}

			// temperature sensors + reference voltage

//		memcpy((char*)sci.temperature_sensors,
//		   (char*)data->a2.temperature_sensors, 40);

			for(int i=0; i<19; i++)
			{
				sci.temperature_sensors[i] =
					amsua_temp(data->a2.temperature_sensors[i]);
			}
			sci.reference_voltage =
				amsua_refvoltage(data->a2.temperature_sensors[19]);
		}

	//
	// output_text_summary
	//

	void output_text_summary( ostream& os = cout )
		{
			string start_time = TimeConv::abs_year_day_millisecond_to_string(
				start_year,
				start_day,
				start_millisecond);

			string end_time = TimeConv::abs_year_day_millisecond_to_string(
				data->gen.scan_line_year,
				data->gen.scan_line_day_of_year,
				data->gen.scan_line_millisecond_of_day);

			os << "AMSUA2Reader report:\n"
			   << "	   " << record_count << " scan lines processed ("
			   << start_scan_line_number << "-" << data->gen.scan_line_number
			   << ")\n	  Product start time " << start_time << endl
			   << "	   Product end time " << end_time << endl
			   << "	   Samples per scan line 2\n"
			   << endl;
		}

	//
	// output_text_config
	//

	void output_text_config( ostream& os = cout )
		{
			os << "AMSUA2Reader configuration:\n"
			   << "	   Serial number " << Config::serial_number << endl
			   << "	   Unit-ID " << Config::serial_number << endl
			   << "	   Filler pattern '"
			   << static_cast<string>(Config::filler_pattern) << "'\n"
			   << "	   Filler random speckle " << Config::filler_random_speckle
			   << endl
			   << "	   Scan line number as sequence count " <<
				(static_cast<bool>(Config::scan_line_number_as_sequence_count)?
				 "on":"off") << endl
			   << "	   Initial sequence count "
			   << Config::initial_sequence_count << endl
			   << "	   UTC days offset " << Config::utc_days_offset << endl
			   << "	   UTC milliseconds offset " <<
				Config::utc_milliseconds_offset << endl
			   << "	   UTC microseconds offset " <<
				Config::utc_microseconds_offset << endl
			   << "	   Ignore UTC time " <<
				(static_cast<bool>(Config::utc_ignore_input_time)?"on":"off")
			   << endl
			   << "	   OBT coarse offset " << Config::obt_coarse_offset << endl
			   << "	   OBT fine offset " << Config::obt_fine_offset << endl
			   << "	   OBT fixed fine time " << Config::obt_fixed_fine_time
			   << endl << endl;
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}
};

#undef Config

#endif
