////////////////////////////////////////////////////////////////////////////////
//
//	File: amsuareader.hpp
//
//	Provides: AMSUAReader filter
//
//	Purpose: Generate text display or graphics from NOAA AMSU-A L1b products
//
//	Copyright (c) 2002, 2003, 2004, 2006 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef amsuareader_defined
#define amsuareader_defined

#include "interface.hpp"
#include "noaa.hpp"
#include "imageproducer.hpp"
#include "amsuabase.hpp"

class AMSUAReader :
	public Interface,
	public ImageProducer,
	public AMSUABase
{
public:

	//
	// constructor
	//

	AMSUAReader(
		ProductProducer& _source,
		bool _display_a1=true,
		bool _display_a2=true)
		: source(_source),
		  display_a1(_display_a1),
		  display_a2(_display_a2),
		  record_count(0)
		{
			assert(sizeof(NOAA_L1b_AMSUA_Header)==NOAA_L1b_AMSUA_Header::SIZE);
			assert(sizeof(NOAA_L1b_AMSUA_Data)==NOAA_L1b_AMSUA_Data::SIZE);

			data = (NOAA_L1b_AMSUA_Data*)source.get_current_record();
			header = (NOAA_L1b_AMSUA_Header*)source.get_product_header();

			if (!data || !header)
			{
				gerror = -1;
				return;
			}

			if (header &&
				header->gen.data_type_code != NOAA_L1b_GEN_Header::DTC_AMSUA)
			{
				error << "This program can only read AMSU-A Level 1b products.\n"
					  << "(Data Type Code of " << header->gen.data_type_code
					  << " found)\n";
			}

			// read initial times
			start_scan_line_number = data->gen.scan_line_number;
			start_year = data->gen.scan_line_year;
			start_day = data->gen.scan_line_day_of_year;
			start_millisecond = data->gen.scan_line_millisecond_of_day;

		}

	//
	// output_text_header
	//

	void output_text_header( ostream& os = cout )
		{
			// archive header
			source.output_text_archive_header(os);

			// generic header
			source.output_text_gen_header(os);

			// amsu-a specific header
			os << "AMSU-A1 instument status			: "
			   << header->instrument_status_a1 << endl
			   << "AMSU-A2 instument status			: "
			   << header->instrument_status_a2 << endl
			   << "Count of data records			: "
			   << header->count_of_data_records << endl
			   << "Count of missing scan lines		: "
			   << header->count_of_missing_scan_lines << endl
			   << "Count of data gaps				: "
			   << header->count_of_data_gaps << endl
			   << "Frames without frame sync errors : "
			   << header->frames_without_frame_sync_errors << endl
			   << "PACS detected TIP parity errors	: "
			   << header->pacs_detected_tip_parity_errors << endl
			   << endl;
		}

	//
	// output_text_record
	//

	void output_text_record(int record_no = -1, bool show_header = false, ostream& os = cout)
		{
			source.skip_records(record_no);
			data = reinterpret_cast<NOAA_L1b_AMSUA_Data*>(
				source.get_current_record());
			if (!data)
			{
				os << "Cannot access record " << record_no << endl;
				return;
			}

			// Introduction

			if (show_header)
			{
				source.output_text_gen_record(record_no, os);
			}

			os << endl;

			if (display_a1)
			{
				output_text_amsua1_record(show_header, os);
			}

			if (display_a2)
			{
				output_text_amsua2_record(show_header, os);
			}
		}

	//
	// output_text_amsua1_record
	//

	void output_text_amsua1_record(bool show_header, ostream& os = cout)
		{
			//
			// AMSU-A1 section
			//

			if (show_header)
			{
				os << "AMSU-A1\n-------\n\n";
			}

			// AMSU-A1 Digital A Telemetry

			os << "AMSU-A1 Digital A Telemetry\n---------------------------\n";

			AMSUABase::output_digital_telemetry(
				os,
				data->a1.digital_a.sync_sequence,
				data->a1.digital_a.serial_number,
				data->a1.digital_a.housekeeping);

			// AMSU-A1 Scene Data

			os << "AMSU-A1 Scene Data\n------------------\n";
			os << "Position 1st reading 2nd reading Counts\n"
			   << "			1	  2		1	  2		3	  4		5	  6		7\n"
			   << "			  8		9	  10	11	  12	13	  14	15\n\n";

//				   12345 12345 12345 12345 12345 12345 12345 12345 12345

			for(int i=0; i<30; i++)
			{
				os << "	  " << setiosflags(ios::fixed) << setw(2)
				   << setfill(' ') << (i+1) << "	"
				   << setiosflags(ios::fixed) << setw(5) << setfill('0')
				   << data->a1.scene[i].first_reading[0] << " "
				   << setiosflags(ios::fixed) << setw(5)
				   << data->a1.scene[i].first_reading[1] << " "
				   << setiosflags(ios::fixed) << setw(5)
				   << data->a1.scene[i].second_reading[0] << " "
				   << setiosflags(ios::fixed) << setw(5)
				   << data->a1.scene[i].second_reading[1] << " ";
				for(int j=0; j<13; j++)
				{
					os << setiosflags(ios::fixed) << setw(5)
					   << data->a1.scene[i].counts[j] << " ";
					if (j==4)
					{
						os << "\n			";
					}
				}
				os << endl;
			}
			os << endl;

			// AMSU-A1 Cold Calibration

			os << "AMSU-A1 Cold Calibration\n------------------------\n"
			   << "Reflector 1 first reading  : "
			   << data->a1.cold_calibration.first_reading[0] << endl
			   << "Reflector 2 first reading  : "
			   << data->a1.cold_calibration.first_reading[1] << endl
			   << "Reflector 1 second reading : "
			   << data->a1.cold_calibration.second_reading[0] << endl
			   << "Reflector 2 second reading : "
			   << data->a1.cold_calibration.second_reading[1] << endl
			   << "Counts	1	  2\n";
			for(int i=0; i<13; i++)
			{
				os << "	   " << ntofs(i+3,2,' ') << " : "
				   << ntofs(data->a1.cold_calibration.counts_1[i]) << " "
				   << ntofs(data->a1.cold_calibration.counts_2[i]) << endl;
			}
			os << endl;

			// AMSU-A1 Temperature sensors
			os << "AMSU-A1 Temperature sensors\n---------------------------\n";
			for(int i=0; i<46; i++)
			{
				os << ntofs(i+1,2,' ') << ": "
				   << ntofs(data->a1.temperature_sensors[i]) << "  ";
				if (i%6==5)
				{
					os << endl;
				}
			}
			os << endl << endl;

			// AMSU-A1 Warm Calibration

			os << "AMSU-A1 Warm Calibration\n------------------------\n"
			   << "Reflector 1 first reading  : "
			   << data->a1.warm_calibration.first_reading[0] << endl
			   << "Reflector 2 first reading  : "
			   << data->a1.warm_calibration.first_reading[1] << endl
			   << "Reflector 1 second reading : "
			   << data->a1.warm_calibration.second_reading[0] << endl
			   << "Reflector 2 second reading : "
			   << data->a1.warm_calibration.second_reading[1] << endl
			   << "Counts	1	  2\n";
			for(int i=0; i<13; i++)
			{
				os << "	   " << ntofs(i+3,2,' ') << " : "
				   << ntofs(data->a1.warm_calibration.counts_1[i]) << " "
				   << ntofs(data->a1.warm_calibration.counts_2[i]) << endl;
			}
			os << endl;

			// AMSU-A1 Digital B Telemetry

			if (show_header)
			{
				os << "AMSU-A1 Digital B Telemetry\n"
				   << "---------------------------\n"
				   << "Invalid Word Flags : "
				   << ntobs(data->a1.digital_b.invalid_word_bit_flags,16)
				   << endl << "Housekeeping		  : "
				   << ntobs(data->a1.digital_b.housekeeping,16) << endl
				   << endl;

				// AMSU-A1 Analog Housekeeping
				os << "AMSU-A1 Analogue Housekeeping\n"
				   << "-----------------------------\n"
				   << "Invalid Word Flags : "
				   << ntobs(data->a1.analog_housekeeping.
							invalid_word_bit_flags,32)
				   << "\nWords:\n";
				for(int i=0; i<28; i++)
				{
					os << ntofs(i+1,2,' ') << ": "
					   << ntofs(data->a1.analog_housekeeping.words[i]) << "	 ";
					if (i%6==5)
					{
						os << endl;
					}
				}
				os << endl;
			}
		}

	//
	// output_text_amsua2_record
	//

	void output_text_amsua2_record(bool show_header, ostream& os = cout)
		{
			// AMSU-A2 Section

			if (show_header)
			{
				os << "AMSU-A2\n-------\n\n";
			}
			// AMSU-A2 Digital A Telemetry

			os << "AMSU-A2 Digital A Telemetry\n---------------------------\n";

			AMSUABase::output_digital_telemetry(
				os,
				data->a2.digital_a.sync_sequence,
				data->a2.digital_a.serial_number,
				data->a2.digital_a.housekeeping);

			// AMSU-A2 Scene Data

			os << "AMSU-A2 Scene Data\n------------------\n"
			   << "Position	 First	  Second   Counts\n"
			   << "			 reading  reading  Ch#1		Ch#2\n\n";

			for(int i=0; i<30; i++)
			{
				os << "	  " << setiosflags(ios::fixed) << setw(2)
				   << setfill(' ') << (i+1) << "	 "
				   << setiosflags(ios::fixed) << setw(5) << setfill('0')
				   << data->a2.scene[i].first_reading << "	  "
				   << setiosflags(ios::fixed) << setw(5)
				   << data->a2.scene[i].second_reading << "	   "
				   << setiosflags(ios::fixed) << setw(5)
				   << data->a2.scene[i].counts[0] << "	  "
				   << setiosflags(ios::fixed) << setw(5)
				   << data->a2.scene[i].counts[1] << endl;
			}
			os << endl;

			// AMSU-A2 Cold Calibration

			os << "AMSU-A2 Cold Calibration\n------------------------\n"
			   << "Reflector first reading	: "
			   << data->a2.cold_calibration.first_reading << endl
			   << "Reflector second reading : "
			   << data->a2.cold_calibration.second_reading << endl
			   << "			 Channels\n"
			   << "			 1	   2\n"
			   << "Counts 1: " << ntofs(data->a2.cold_calibration.counts_1[0])
			   << " " << ntofs(data->a2.cold_calibration.counts_2[0]) << endl
			   << "		  2: " << ntofs(data->a2.cold_calibration.counts_1[1])
			   << " " << ntofs(data->a2.cold_calibration.counts_2[1]) << endl
			   << endl;

			// AMSU-A2 Temperature sensors
			os << "AMSU-A2 Temperature sensors\n---------------------------\n";
			for(int i=0; i<20; i++)
			{
				os << ntofs(i+1,2,' ') << ": "
				   << ntofs(data->a2.temperature_sensors[i]) << "  ";
				if (i%6==5)
				{
					os << endl;
				}
			}
			os << endl << endl;

			// AMSU-A2 Warm Calibration

			os << "AMSU-A2 Warm Calibration\n------------------------\n"
			   << "Reflector first reading	: "
			   << data->a2.warm_calibration.first_reading << endl
			   << "Reflector second reading : "
			   << data->a2.warm_calibration.second_reading << endl
			   << "			 Channels\n"
			   << "			 1	   2\n"
			   << "Counts 1: " << ntofs(data->a2.warm_calibration.counts_1[0])
			   << " " << ntofs(data->a2.warm_calibration.counts_2[0]) << endl
			   << "		  2: " << ntofs(data->a2.warm_calibration.counts_1[1])
			   << " " << ntofs(data->a2.warm_calibration.counts_2[1]) << endl
			   << endl;

			// AMSU-A2 Digital B Telemetry

			if (show_header)
			{
				os << "AMSU-A2 Digital B Telemetry\n"
				   << "---------------------------\n"
				   << "Invalid Word Flags : "
				   << ntobs(data->a2.digital_b.invalid_word_bit_flags,16)
				   << endl << "Housekeeping		  : "
				   << ntobs(data->a2.digital_b.telemetry,16) << endl
				   << endl;

				// AMSU-A2 Analog Housekeeping
				os << "AMSU-A2 Analogue Housekeeping\n"
				   << "-----------------------------\n"
				   << "Invalid Word Flags : "
				   << ntobs(data->a2.analog_housekeeping.
							invalid_word_bit_flags,32)
				   << "\nWords:\n";
				for(int i=0; i<28; i++)
				{
					os << ntofs(i+1,2,' ') << ": "
					   << ntofs(data->a2.analog_housekeeping.words[i]) << "	 ";
					if (i%6==5)
					{
						os << endl;
					}
				}
				os << endl;
			}
		}


	//
	// get_image_count
	//

	virtual int get_image_count(void)
		{
			return AMSUABase::get_image_count();
		}

	//
	// write_image
	//

	virtual void write_image(int number, Image* image)
		{
			AMSUABase::write_image(number, image);
		}

	//
	// prepare_images
	//

	virtual void prepare_images(void)
		{
			spinner.reset(new Spinner(string("Preparing images")));

			// prepare normalisation parameters
			for(int i=0; i<30; i++)
			{
				amsua_min[i] = 65535;
				amsua_max[i] = 0;
			}

			// extract data

			while(source.get_current_record())
			{
				data = (NOAA_L1b_AMSUA_Data*)source.get_current_record();

				AMSUAScanLine scan_line;
				for(int x=0; x<30; x++)
				{
					for(int sample=0; sample<15; sample++)
					{
						uint16_be a;
						if (sample < 2)
						{
							// take 2 pixels from AMSU-A2 section
							a = data->a2.scene[x].counts[sample];
						}
						else
						{
							// take other 13 pixels from AMSU-A1 section
							a = data->a1.scene[x].counts[sample-2];
						}

						if (a>amsua_max[sample])
						{
							amsua_max[sample] = a;
						}

						if (a<amsua_min[sample])
						{
							amsua_min[sample] = a;
						}

						scan_line.pixels[x].channels[sample] = a;
					}
				}

				source.skip_records(1);
				amsua_scan_lines.push_back(scan_line);
				spinner->tick();
			}

			spinner->end();
		}

	//
	// fill_mask
	//	(for use by derived classes)
	//

	enum Mask { MASK_DATA , MASK_FILLER };

	void fill_mask(Mask* mask, int mask_length, string pattern)
		{
			if (pattern.empty())
			{
				for(int i=0; i<mask_length; i++)
				{
					mask[i] = MASK_DATA;
				}
				return;
			}
			int pattern_pos=0;
			int mask_pos = 0;

			while(mask_pos < mask_length)
			{
				switch(pattern[pattern_pos])
				{
				case 'X': // data
					mask[mask_pos++] = MASK_DATA;
					break;

				case 'x': // 50% chance data
					if (random()%2==0)
					{
						mask[mask_pos++] = MASK_DATA;
					}
					break;

				case '?': // coin flip
					mask[mask_pos++] = (random()%2==0?MASK_DATA:MASK_FILLER);
					break;

				case ';': // 50% change filler
					if (random()%2==0)
					{
						mask[mask_pos++] = MASK_FILLER;
					}
					break;

				case '.': // filler
					mask[mask_pos++] = MASK_FILLER;
					break;

				default:
					error << "ASUAReader:: illegal character '"
						  << pattern[pattern_pos] << "' in filler pattern\n";

				}

				pattern_pos++;
				if (pattern_pos==pattern.length())
				{
					pattern_pos = 0;
				}
			}
		}

	//
	// speckle_mask
	//

	void speckle_mask(Mask* mask, int mask_length, int speckle_count)
		{
			for(int i=0; i<speckle_count; i++)
			{
				int pos = random()%mask_length;
				mask[pos] = mask[pos]==MASK_DATA?MASK_FILLER:MASK_DATA;
			}
		}

	//
	// check_mask
	//

	void check_mask(Mask* mask, int mask_length, int data_required)
		{
			// first pass, analyse the current ask state
			int data_count = 0;
			int filler_count = 0;
			for(int i=0; i<mask_length; i++)
			{
				switch(mask[i])
				{
				case MASK_DATA:
					data_count++;
					break;
				case MASK_FILLER:
					filler_count++;
					break;
				}
			}

			// positive error means too much data, negative error
			// means too much filler
			int error_count = data_count - data_required;

			// second pass, correct the mask
			for(int i=mask_length-1; i>=0 && error!=0; i--)
			{
				if (error_count > 0 && mask[i] == MASK_DATA)
				{
					// found some data we can remove
					mask[i] = MASK_FILLER;
					error_count--;
				}
				if (error_count < 0 && mask[i] == MASK_FILLER)
				{
					// found some filler we can remove
					mask[i] = MASK_DATA;
					error_count++;
				}
			}
		}

	//
	// apply_mask
	//

	void apply_mask(uint16_be* dest, uint16_be* science_data, Mask* mask, int length)
		{
			int data_pos = 0;
			for(int i=0; i<length; i++)
			{
				if (mask[i] == MASK_DATA)
				{
					dest[i] = science_data[data_pos++];
				}
				else
				{
					dest[i] = 0x1;
				}
			}
		}

protected:

	ProductProducer& source;
	NOAA_L1b_AMSUA_Header* header;
	NOAA_L1b_AMSUA_Data* data;

	int current_record;

	bool display_a1;
	bool display_a2;

	unsigned int start_scan_line_number;
	unsigned int start_year;
	unsigned int start_day;
	unsigned int start_millisecond;
	int record_count;

};

#endif
