////////////////////////////////////////////////////////////////////////////////
//
//	File: amsubreader.hpp
//
//	Provides: AMSUBReader filter
//
//	Purpose: Display AMSU-B products or generate images
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef amsub_defined
#define amsub_defined

#include "noaa.hpp"
#include "imageproducer.hpp"
#include "image.hpp"

class AMSUBReader :
	public ImageProducer

{
public:

	typedef NOAA_L1b_AMSUB_Data::Sample Sample;

	//
	// constructor
	//

	AMSUBReader(ProductProducer& _source)
		: source(_source)
		{
			assert(NOAA_L1b_AMSUB_Header::SIZE==sizeof(NOAA_L1b_AMSUB_Header));
			assert(NOAA_L1b_AMSUB_Data::SIZE==sizeof(NOAA_L1b_AMSUB_Data));
			header = (NOAA_L1b_AMSUB_Header*)source.get_product_header();

			if (header && header->gen.data_type_code !=
				NOAA_L1b_GEN_Header::DTC_AMSUB)
			{
				error << "This program can only read AMSU-B products\n";
				return;
			}
		}

	//
	// output_text_header
	//

	void output_text_header( ostream& os = cout )
		{
			// archive header
			source.output_text_archive_header(os);

			// generic header
			source.output_text_gen_header(os);

			// AMBU-B specific header
			os << "AMSU-B instument status word 0	  : "
			   << ntobs(header->instrument_status[3], 8) << endl
			   << "						   word 1	  : "
			   << ntobs(header->instrument_status[2], 8) << endl
			   << "						   word 2	  : "
			   << ntobs(header->instrument_status[1], 8) << endl
			   << "						   word 3	  : "
			   << static_cast<int>(header->instrument_status[0]);
			switch(header->instrument_status[0])
			{
			case 0 : os << " (EM)"; break;
			case 4 : os << " (PFM)"; break;
			case 8 : os << " (FM2)"; break;
			case 12 : os << " (FM3)"; break;
			default: os << " (unkown FM)"; break;
			}
			os << endl
			   << "Count of data records			  : "
			   << header->count_of_data_records << endl
			   << "Count of good scan lines			  : "
			   << header->count_of_good_scan_lines << endl
			   << "Count of missing scan lines		  : "
			   << header->count_of_missing_scan_lines << endl
			   << "Count of data gaps				  : "
			   << header->count_of_data_gaps << endl
			   << "Count of frames without frame sync : "
			   << header->count_of_frames_without_frame_sync << endl
			   << "PACS detected TIP parity errors	  : "
			   << header->pacs_detected_tip_parity_errors << endl
			   << "PRT Coefficients\n";
			for(int prt=0; prt<7; prt++)
			{
				for(int coeff=0; coeff<4; coeff++)
				{
					os << "PRT#" << prt << " Coeff " << coeff
					   << "						: "
					   << header->prt_coeffs[prt][coeff] << endl;
				}
			}
			os << endl;
		}

	//
	// display_samples
	//

	void display_samples(textoutstream_type& os, Sample* samples, int count)
		{
			os << "Sample no.	Shaft pos Ch#16 Ch#17 Ch#18 Ch#19 Ch#20\n";
			for(int i=0; i<count; i++)
			{
				os << "		" << ntofs(i+1,2,' ') << "		"
				   << ntofs(samples[i].shaft_pos) << "	   ";
				for(int j=0; j<5; j++)
				{
					os << ntofs(samples[i].channels[j]) << " ";
				}
				os << endl;
			}
		}

	//
	// output_text_record
	//

	void output_text_record(int record_no = -1, bool show_header = false, ostream& os = cout)
		{
			source.skip_records(record_no);
			data = (NOAA_L1b_AMSUB_Data*)source.get_current_record();
			if (!data)
			{
				os << "Cannot access record " << record_no << endl;
				return;
			}

			// Introduction

			if (show_header)
			{
				source.output_text_gen_record(record_no, os);
			}


			os << "Sensor Data\n-----------\n\n";
			display_samples(os, data->sensor_data, 90);
			os << endl;

			os << "Space View\n----------\n\n";
			display_samples(os, data->space_view, 4);
			os << endl;

			os << "Target View\n-----------\n\n";
			display_samples(os, data->target_view,4);
			os << endl;

			if (show_header)
			{
				os << "Digital A Telemetry\n-------------------\n\n"
				   << "Invalid word flags	: "
				   << ntobs(data->digital_a.invalid_word_bits,32) << endl
				   << "Digital Data A01 MSB : "
				   << ntobs(data->digital_a.data_words[0]>>8,8) << endl
				   << "					LSB : "
				   << ntobs(data->digital_a.data_words[0]&255,8) << endl
				   << "Digital Data A02 MSB : "
				   << ntobs(data->digital_a.data_words[1]>>8,8) << endl
				   << "					LSB : "
				   << ntobs(data->digital_a.data_words[1]&255,8) << endl;
				for(int i=2; i<26; i++)
				{
					os << "Digital Data A" << ntofs(i+1,2,'0');
					if (i>=10 && i<=16)
					{
						os << " (PRT#" << (i-9) << ") : "
						   << ntofs(data->digital_a.data_words[i]);

						// for calculate actual temperature
						double temp1 = static_cast<double>(
							header->prt_coeffs[i-10][0]) * 10e-3;

						double temp2 = static_cast<double>(
							header->prt_coeffs[i-10][1]) * 10e-8 *
							static_cast<double>(
								data->digital_a.data_words[i]);

						double temp3 = static_cast<double>(
							header->prt_coeffs[i-10][2]) * 10e-13 *
							pow(static_cast<double>(
									data->digital_a.data_words[i]),2.0);

						double temp4 = static_cast<double>(
							header->prt_coeffs[i-10][3]) * 10e-19 *
							pow(static_cast<double>(
									data->digital_a.data_words[i]),3.0);

						double temp = temp1 + temp2 + temp3 + temp4 - 273.15;

						os << "	 (" << dto2dp(temp) << "C)\n";
						// << temp1 << "  ,	 " << temp2 << "  ,	 " << temp3
						// << "	 ,	" << temp4 << endl;
					}
					else
					{
						os << "			: "
						   << ntofs(data->digital_a.data_words[i]) << endl;
					}
				}
				os << endl;

				os << "Analog Telemetry\n----------------\n\n"
				   << "Invalid word flags : "
				   << ntobs(data->analog.invalid_word_bits,32) << endl;
				for(int i=0; i<28; i++)
				{
					os << "Word " << ntofs(i+1,2,'0') << " : "
					   << ntofs(data->analog.words[i]) << endl;
				}
			}
		}

	//
	// prepare_images
	//

	void prepare_images(void)
		{
			spinner.reset(new Spinner("Preparing image data"));

			uint16_be lo[5];
			uint16_be hi[5];
			for(int i=0; i<5; i++)
			{
				lo[i] = 65535;
				hi[i] = 0;
			}

			while(source.get_current_record())
			{
				data = (NOAA_L1b_AMSUB_Data*)source.get_current_record();
				ScanLine scan_line;

				for(int j=0; j<90; j++)
				{
					for(int k=0; k<5; k++)
					{
						uint16_be s = data->sensor_data[j].channels[k];
						if (s>hi[k])
						{
							hi[k] = s;
						}
						if (s!=0 && s<lo[k])
						{
							lo[k] = s;
						}
						scan_line.samples[j].channels[k] = s;
					}
				}

				source.skip_records(1);
				scan_lines.push_back(scan_line);
				spinner->tick();
			}

			spinner->end();

			// sort out normalisation factors
			for(int i=0; i<5; i++)
			{
				offset[i] = static_cast<double>(lo[i]);
				gain[i] = 255.0/(static_cast<double>(hi[i])-
								 static_cast<double>(lo[i]));
				info << "channel " << i << " lo " << lo[i] << " hi " << hi[i]
					 << " offset " << offset[i] << " gain " << gain[i] << endl;
			}

		}

	//
	// get_image_count
	//

	int get_image_count(void)
		{
			return 5;
		}

	//
	// write_image
	//

	void write_image(int number, Image* image)
		{
			spinner.reset(new Spinner("Assembling image"));

			image->set_filename_suffix(string(".")+ntos(number+1));
			image->set_attributes(Image::IMAGE_MONO, 90, scan_lines.size());
			image->begin_output();

			byte row[90];
			for(ScanLines::iterator iter=scan_lines.begin();
				iter!=scan_lines.end();
				iter++)
			{
				for(int x=0; x<90; x++)
				{
					row[x] = static_cast<byte>((static_cast<double>(
													(*iter).samples[x].channels[number])-
												offset[number])*gain[number]);
				}
				image->write_line(row);

				spinner->tick();

			}

			image->end_output();
			spinner->end();
		}

protected:

	ProductProducer& source;
	NOAA_L1b_AMSUB_Header* header;
	NOAA_L1b_AMSUB_Data* data;

	struct ScanLine
	{
		struct Sample
		{
			uint16_be channels[5];
		} samples[90];
	};
	typedef deque<ScanLine> ScanLines;
	ScanLines scan_lines;

	double offset[5];
	double gain[5];


};

#endif
