////////////////////////////////////////////////////////////////////////////////
//
//	File: amsubtomhs.hpp
//
//	Provides: AMSUBToMHS filter
//
//	Purpose: Convert AMSU-B products to MHS source packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef amsubtomhs_defined
#define amsubtomhs_defined

#include "config.hpp"
#include "amsubreader.hpp"
#include "ccsdsgeneratorbase.hpp"
#include "ccsdsproducer.hpp"
#include "mhstempconv.hpp"

#define Config Config::MHS

class AMSUBToMHS :
	public AMSUBReader,
	public CCSDSProducer,
	public CCSDSGeneratorBase<CCSDS_MHS>
{
public:

	//
	// constructor
	//

	AMSUBToMHS(ProductProducer& _source, const string& config_file)
		: AMSUBReader(_source)
		, record_count(0)
		{
			data = (NOAA_L1b_AMSUB_Data*)source.get_current_record();

			if (!config_file.empty())
			{
				Config::file.read(config_file);
			}

			if (!header || !data)
			{
//		error << "Unable to read input product\n";
				return;
			}

			subcommutation_code = Config::subcommutation_initial_value;

			// prepare CCSDS section

			ccsds_set_sequence_count(
				Config::initial_sequence_count);

			if (Config::utc_ignore_input_time)
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);

				ccsds_modify_utc_base_time(data->gen);
			}
			else
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);
			}

			ccsds_set_obt_zero_time(
				data->gen);

			ccsds_apply_obt_offsets(
				Config::obt_coarse_offset,
				Config::obt_fine_offset,
				Config::obt_fixed_fine_time);

			start_scan_line = data->gen.scan_line_number;
			start_year = data->gen.scan_line_year;
			start_day = data->gen.scan_line_day_of_year;
			start_millisecond = data->gen.scan_line_millisecond_of_day;

		}

	//
	// output_text_summary
	//

	void output_text_summary( ostream& os = cout )
		{
			if (!data || !header)
			{
				return;
			}

			string start_time = TimeConv::abs_year_day_millisecond_to_string(
				start_year,
				start_day,
				start_millisecond);

			string end_time = TimeConv::abs_year_day_millisecond_to_string(
				data->gen.scan_line_year,
				data->gen.scan_line_day_of_year,
				data->gen.scan_line_millisecond_of_day);

			os << "AMSUBToMHS report:\n"
			   << "	   " << record_count << " scan lines processed ("
			   << start_scan_line << "-" << (data->gen.scan_line_number)
			   << ")\n"
			   << "	   Product start time " << start_time << endl
			   << "	   Product end time " << end_time << endl
			   << endl;
		}

	//
	// output_text_record
	//

	virtual void output_text_record(
		int record = -1,
		bool show_header = false,
		ostream& os = cout)
		{
			AMSUBReader::output_text_record(record,show_header,os);
		}

	//
	// output_text_header
	//

	virtual void output_text_header( ostream& os = cout )
		{
			AMSUBReader::output_text_header(os);
		}

	//
	// prepare_images
	//

	virtual void prepare_images(void)
		{
			AMSUBReader::prepare_images();
		}

	//
	// get_image_count
	//

	virtual int get_image_count(void)
		{
			return AMSUBReader::get_image_count();
		}

	//
	// write_image
	//

	virtual void write_image(int number, Image* image)
		{
			AMSUBReader::write_image(number,image);
		}

	//
	// get_next_ccsds
	//

	CCSDS* get_next_ccsds(void)
		{
			if (!source.get_current_record())
			{
				if (spinner.get())
				{
					spinner->end();
				}
				return NULL;
			}

			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			}
			spinner->tick();

			// copy scan line number into sequence number if configured
			if (Config::scan_line_number_as_sequence_count)
			{
				ccsds_set_sequence_count(data->gen.scan_line_number+
										 Config::initial_sequence_count);
			}

			// prepare source packet
			ccsds_initialise();

			// set utc time
			ccsds_set_time(data->gen);

			// fill the mode field

			ccsds_packet.payload.mode =
				((Config::scan_mode.get()&15)<<4) +
				((Config::pie_id.get()&1)<<3) +
				((subcommutation_code&7));

			// advance subcommutation code
			subcommutation_code += Config::subcommutation_mod_value.get();
			if (subcommutation_code > Config::subcommutation_max_value.get())
			{
				subcommutation_code = Config::subcommutation_min_value.get();
			}
			if (subcommutation_code < Config::subcommutation_min_value.get())
			{
				subcommutation_code = Config::subcommutation_max_value.get();
			}

			// fill the telecommand acknowledgement field

			ccsds_packet.payload.telecommand[0] =
				((Config::tc_clean.get()&1) << 7) +
				((Config::tc_conforms.get()&1) << 6) +
				((Config::tc_recognised.get()&1) << 5) +
				((Config::tc_legal.get()&1) << 4) +
				((Config::fdm_motor_current_trip_status.get()&1) << 3) +
				((Config::tc_application_id.get()&7));

			ccsds_packet.payload.telecommand[1] =
				(Config::tc_application_process_id.get()&255);

			ccsds_packet.payload.telecommand[2] =
				(Config::tc_packet_sequence_count.get()&255);

			ccsds_packet.payload.telecommand[3] =
				((Config::tc_packet_sequence.get()&63) << 2) +
				((Config::tc_received_count.get()&3));

			ccsds_packet.payload.telecommand[4] =
				((Config::monitor_current_fault.get()&1) << 7) +
				((Config::thermistor_monitor_fault.get()&1) << 6) +
				((Config::switch_fault.get()&1) << 5) +
				((Config::processor_fault.get()&1) << 4) +
				((Config::rdm_motor_current_trip_status.get()&1) << 3) +
				((Config::dc_offset_error.get()&1) << 2) +
				((Config::scan_control_error.get()&1) << 1) +
				((Config::ref_ck_error.get()&1));

			// fill switch status section

			ccsds_packet.payload.switch_status[0] =
				((Config::receiver_channel_h1&1)<<0) +
				((Config::receiver_channel_h1_local_oscillator&1)<<1) +
				((Config::receiver_channel_h2&1)<<2) +
				((Config::receiver_channel_h2_local_oscillator&1)<<3) +
				((Config::receiver_channel_h3_h4_frontend&1)<<4) +
				((Config::receiver_channel_h3_h4_local_oscillator&1)<<5) +
				((Config::receiver_channel_h3_backend&1)<<6) +
				((Config::receiver_channel_h4_backend&1)<<7);

			ccsds_packet.payload.switch_status[1] =
				((Config::receiver_channel_h5&1)<<0) +
				((Config::receiver_channel_h5_local_oscillator&1)<<1) +
				((Config::rx_cv&1)<<2) +
				((Config::receiver_operational&1)<<3) +
				((Config::scan_mechanism_operational_heaters&1)<<4) +
				((Config::auxiliary_operational_heaters&1)<<5) +
				((Config::signal_processing_electronics&1)<<6) +
				((Config::prom_failure&1)<<7);

			ccsds_packet.payload.switch_status[2] =
				((Config::rdm_zero_position_sensors&1)<<0) +
				((Config::fdm_zero_position_sensors&1)<<1) +
				((Config::rdm_motor_sensors&1)<<2) +
				((Config::fdm_motor_sensors&1)<<3) +
				((Config::rdm_motor_supply&1)<<4) +
				((Config::fdm_motor_supply&1)<<5) +
				((Config::rdm_motor_current_trip_enabled&1)<<6) +
				((Config::fdm_motor_current_trip_enabled&1)<<7);

			// fill temperature measurements

			ccsds_packet.payload.temperature[0] = conv_temperature(Config::t1);
			ccsds_packet.payload.temperature[1] = conv_temperature(Config::t2);
			ccsds_packet.payload.temperature[2] = conv_temperature(Config::t3);
			ccsds_packet.payload.temperature[3] = conv_temperature(Config::t4);
			ccsds_packet.payload.temperature[4] = conv_temperature(Config::t5);
			ccsds_packet.payload.temperature[5] = conv_temperature(Config::t6);
			ccsds_packet.payload.temperature[6] = conv_temperature(Config::t7);
			ccsds_packet.payload.temperature[7] = conv_temperature(Config::t8);
			ccsds_packet.payload.temperature[8] = conv_temperature(Config::t9);
			ccsds_packet.payload.temperature[9] = conv_temperature(Config::t10);
			ccsds_packet.payload.temperature[10] =
				conv_temperature(Config::t11);
			ccsds_packet.payload.temperature[11] =
				conv_temperature(Config::t12);
			ccsds_packet.payload.temperature[12] =
				conv_temperature(Config::t13);
			ccsds_packet.payload.temperature[13] =
				conv_temperature(Config::t14);
			ccsds_packet.payload.temperature[14] =
				conv_temperature(Config::t15);
			ccsds_packet.payload.temperature[15] =
				conv_temperature(Config::t16);
			ccsds_packet.payload.temperature[16] =
				conv_temperature(Config::t17);
			ccsds_packet.payload.temperature[17] =
				conv_temperature(Config::t18);
			ccsds_packet.payload.temperature[18] =
				conv_temperature(Config::t19);
			ccsds_packet.payload.temperature[19] =
				conv_temperature(Config::t20);
			ccsds_packet.payload.temperature[20] =
				conv_temperature(Config::t21);
			ccsds_packet.payload.temperature[21] =
				conv_temperature(Config::t22);
			ccsds_packet.payload.temperature[22] =
				conv_temperature(Config::t23);
			ccsds_packet.payload.temperature[23] =
				conv_temperature(Config::t24);

			// fill current consumption

			// 5v secondary current
			ccsds_packet.payload.current[0] = static_cast<unsigned int>(
				(Config::v5_secondary_current/1000.0) /
				Config::v5_secondary_current_k);

			// 8v secondary current
			ccsds_packet.payload.current[1] = static_cast<unsigned int>(
				((Config::v8_secondary_current/1000.0) -
				 Config::v8_secondary_current_k2) /
				Config::v8_secondary_current_k1);

			// 15v receiver current
			ccsds_packet.payload.current[2] = static_cast<unsigned int>(
				((Config::v15_receiver_current/1000.0) -
				 Config::v15_receiver_current_k2) /
				Config::v15_receiver_current_k1);

			// -15v receiver current
			ccsds_packet.payload.current[3] = static_cast<unsigned int>(
				((Config::nv15_receiver_current/1000.0)-
				 Config::nv15_receiver_current_k2) /
				Config::nv15_receiver_current_k1);

			// RDM motor current
			ccsds_packet.payload.current[4] = static_cast<unsigned int>(
				(Config::rdm_motor_current/1000.0) /
				Config::rdm_motor_current_k);

			// FDM motor current
			ccsds_packet.payload.current[5] = static_cast<unsigned int>(
				(Config::fdm_motor_current/1000.0) /
				Config::fdm_motor_current_k);

			// fill status word

			ccsds_packet.payload.status_word =
				((Config::dc_offset_valid&1) << 7) +
				((Config::scan_control_valid&1) << 6) +
				((Config::profile&3) << 4);

			// fill signal processing status

			ccsds_packet.payload.signal_status[0] =
				Config::channel_h1_dc_offset_word&255;

			ccsds_packet.payload.signal_status[1] =
				Config::channel_h2_dc_offset_word&255;

			ccsds_packet.payload.signal_status[2] =
				Config::channel_h3_dc_offset_word&255;

			ccsds_packet.payload.signal_status[3] =
				Config::channel_h4_dc_offset_word&255;

			ccsds_packet.payload.signal_status[4] =
				Config::channel_h5_dc_offset_word&255;

			ccsds_packet.payload.signal_status[5] =
				((Config::h1_valid&1) << 7) +
				((Config::h2_valid&1) << 6) +
				((Config::h3_valid&1) << 5) +
				((Config::h4_valid&1) << 4) +
				((Config::h5_valid&1) << 3) +
				(Config::spe_mux_code);

			ccsds_packet.payload.signal_status[6] =
				((Config::h1_gain&7) << 5) +
				((Config::h2_gain&7) << 2);

			ccsds_packet.payload.signal_status[7] =
				((Config::h3_gain&7) << 5) +
				((Config::h4_gain&7) << 2);

			ccsds_packet.payload.signal_status[8] =
				(Config::h5_gain&7) << 5;

			// fill sample data

			// earth view
			memcpy((char*)ccsds_packet.payload.earth_view,
				   (char*)data->sensor_data,
				   sizeof(Sample)*90);
			for(int i=0; i<90; i++)
			{
				offset_shaft_position(ccsds_packet.payload.earth_view[i]);
			}

			// space view
			memcpy((char*)ccsds_packet.payload.space_view,
				   (char*)data->space_view,
				   sizeof(Sample)*4);
			for(int i=0; i<4; i++)
			{
				offset_shaft_position(ccsds_packet.payload.space_view[i]);
			}

			// obct view
			memcpy((char*)ccsds_packet.payload.obct_view,
				   (char*)data->target_view,
				   sizeof(Sample)*4);
			for(int i=0; i<4; i++)
			{
				offset_shaft_position(ccsds_packet.payload.obct_view[i]);
			}

			// fill obct values

			for(int i=0; i<5; i++)
			{
				ccsds_packet.payload.prt[i] =
					mhs_prt_k_to_count(
						amsub_prt_count_to_k(
							data->digital_a.data_words[10+i],
							header->prt_coeffs[i]));
			}

			ccsds_packet.payload.prt_cal[0] =
				Config::prt_cal_1;
			ccsds_packet.payload.prt_cal[1] =
				Config::prt_cal_2;
			ccsds_packet.payload.prt_cal[2] =
				Config::prt_cal_3;

			source.skip_records(1);

			if (source.get_current_record())
			{
				data = (NOAA_L1b_AMSUB_Data*)source.get_current_record();
			}

			record_count++;
			return ccsds_finalise();
		}

	//
	// conv_temperature
	//

	// Translate count values from the 24 MHS temp sensors to C.
	// This has to be done via a look up table.
	int conv_temperature(double temp)
		{
			for(int i=255; i>=0; i--)
			{
				if (mhs_temp_conv(i) > temp)
				{
					return i;
				}
			}
			return 0;
		}

	//
	// output_text_config
	//

	virtual void output_text_config( textoutstream_type& os )
		{
			os << "AMSUBToMHS configuration:\n"
			   << "	   Scan mode " << Config::scan_mode << endl
			   << "	   PIE-ID " << Config::pie_id << endl
			   << "	   Subcommutation initial value "
			   << Config::subcommutation_initial_value << endl
			   << "	   Subcommutation mod value "
			   << Config::subcommutation_mod_value << endl
			   << "	   Subcommutation min value "
			   << Config::subcommutation_min_value << endl
			   << "	   Subcommutation max value "
			   << Config::subcommutation_max_value << endl
			   << "	   TC clean " << Config::tc_clean << endl
			   << "	   TC conforms " << Config::tc_conforms << endl
			   << "	   TC recognised " << Config::tc_recognised << endl
			   << "	   TC legal " << Config::tc_legal << endl
			   << "	   FDM motor current trip status "
			   << Config::fdm_motor_current_trip_status << endl
			   << "	   TC application id " << Config::tc_application_id << endl
			   << "	   TC application process id "
			   << Config::tc_application_process_id << endl
			   << "	   TC packet sequence count "
			   << Config::tc_packet_sequence_count << endl
			   << "	   TC packet sequence " << Config::tc_packet_sequence
			   << endl
			   << "	   TC received count " << Config::tc_received_count << endl
			   << "	   Monitor current fault "
			   << Config::monitor_current_fault << endl
			   << "	   Thermistor monitor fault "
			   << Config::thermistor_monitor_fault << endl
			   << "	   Switch fault " << Config::switch_fault << endl
			   << "	   Processor fault " << Config::processor_fault << endl
			   << "	   RDM motor current trip status "
			   << Config::rdm_motor_current_trip_status << endl
			   << "	   DC offset error " << Config::dc_offset_error << endl
			   << "	   Scan control error " << Config::scan_control_error
			   << endl
			   << "	   REF CK error " << Config::ref_ck_error << endl
			   << "	   Receiver channel H1 " << Config::receiver_channel_h1
			   << endl
			   << "	   Receiver channel H1 local oscilator A/B "
			   << Config::receiver_channel_h1_local_oscillator << endl
			   << "	   Receiver channel H2 " << Config::receiver_channel_h2
			   << endl
			   << "	   Receiver channel H2 local oscillator A/B "
			   << Config::receiver_channel_h2_local_oscillator << endl
			   << "	   Receiver channel H3/H4 frontend "
			   << Config::receiver_channel_h3_h4_frontend << endl
			   << "	   Receiver channel H3/H4 local oscillator A/B "
			   << Config::receiver_channel_h3_h4_local_oscillator << endl
			   << "	   Receiver channel H5 " << Config::receiver_channel_h5
			   << endl
			   << "	   Receiver channel H5 local oscillator A/B "
			   << Config::receiver_channel_h5_local_oscillator << endl
			   << "	   RX CV " << Config::rx_cv << endl
			   << "	   Receiver operational " << Config::receiver_operational
			   << endl
			   << "	   Scan mechanism operational heaters "
			   << Config::scan_mechanism_operational_heaters << endl
			   << "	   Auxiliary operational heaters "
			   << Config::auxiliary_operational_heaters << endl
			   << "	   Signal processing electronics "
			   << Config::signal_processing_electronics << endl
			   << "	   PROM failure " << Config::prom_failure << endl
			   << "	   RDM zero position sensors "
			   << Config::rdm_zero_position_sensors << endl
			   << "	   FDM zero position sensors "
			   << Config::fdm_zero_position_sensors << endl
			   << "	   RDM motor sensors A/B "
			   << Config::rdm_motor_sensors << endl
			   << "	   FDM motor sensors A/B "
			   << Config::fdm_motor_sensors << endl
			   << "	   RDM motor supply " << Config::rdm_motor_supply << endl
			   << "	   FDM motor supply " << Config::rdm_motor_supply << endl
			   << "	   RDM motor current trip enabled "
			   << Config::rdm_motor_current_trip_enabled << endl
			   << "	   FDM motor current trip enabled "
			   << Config::fdm_motor_current_trip_enabled << endl
			   << "	   Temperature T1 " << Config::t1 << endl
			   << "	   Temperature T2 " << Config::t2 << endl
			   << "	   Temperature T3 " << Config::t3 << endl
			   << "	   Temperature T4 " << Config::t4 << endl
			   << "	   Temperature T5 " << Config::t5 << endl
			   << "	   Temperature T6 " << Config::t6 << endl
			   << "	   Temperature T7 " << Config::t7 << endl
			   << "	   Temperature T8 " << Config::t8 << endl
			   << "	   Temperature T9 " << Config::t9 << endl
			   << "	   Temperature T10 " << Config::t10 << endl
			   << "	   Temperature T11 " << Config::t11 << endl
			   << "	   Temperature T12 " << Config::t12 << endl
			   << "	   Temperature T13 " << Config::t13 << endl
			   << "	   Temperature T14 " << Config::t14 << endl
			   << "	   Temperature T15 " << Config::t15 << endl
			   << "	   Temperature T16 " << Config::t16 << endl
			   << "	   Temperature T17 " << Config::t17 << endl
			   << "	   Temperature T18 " << Config::t18 << endl
			   << "	   Temperature T19 " << Config::t19 << endl
			   << "	   Temperature T20 " << Config::t20 << endl
			   << "	   Temperature T21 " << Config::t21 << endl
			   << "	   Temperature T22 " << Config::t22 << endl
			   << "	   Temperature T23 " << Config::t23 << endl
			   << "	   Temperature T24 " << Config::t24 << endl
			   << "	   5v secondary current k "
			   << Config::v5_secondary_current_k << endl
			   << "	   8v secondary current k1 "
			   << Config::v8_secondary_current_k1 << endl
			   << "	   8v secondary current k2 "
			   << Config::v8_secondary_current_k2 << endl
			   << "	   15v receiver current k1 "
			   << Config::v15_receiver_current_k1 << endl
			   << "	   15v receiver current k2 "
			   << Config::v15_receiver_current_k2 << endl
			   << "	   -15v receiver current k1 "
			   << Config::nv15_receiver_current_k1 << endl
			   << "	   -15v receiver current k2 "
			   << Config::nv15_receiver_current_k2 << endl
			   << "	   RDM motor current k "
			   << Config::rdm_motor_current_k << endl
			   << "	   FDM motor current k "
			   << Config::fdm_motor_current_k << endl
			   << "	   5v secondary current "
			   << Config::v5_secondary_current << endl
			   << "	   8v secondary current "
			   << Config::v8_secondary_current << endl
			   << "	   15v receiver current "
			   << Config::v15_receiver_current << endl
			   << "	   -15v receiver current "
			   << Config::nv15_receiver_current << endl
			   << "	   RDM motor current "
			   << Config::rdm_motor_current << endl
			   << "	   FDM motor current "
			   << Config::fdm_motor_current << endl
			   << "	   DC offset valid "
			   << Config::dc_offset_valid << endl
			   << "	   Scan control valid "
			   << Config::scan_control_valid << endl
			   << "	   Profile " << Config::profile << endl
			   << "	   Channel H1 DC offset "
			   << Config::channel_h1_dc_offset_word << endl
			   << "	   Channel H2 DC offset "
			   << Config::channel_h2_dc_offset_word << endl
			   << "	   Channel H3 DC offset "
			   << Config::channel_h3_dc_offset_word << endl
			   << "	   Channel H4 DC offset "
			   << Config::channel_h4_dc_offset_word << endl
			   << "	   Channel H5 DC offset "
			   << Config::channel_h5_dc_offset_word << endl
			   << "	   Channel H1 Valid " << Config::h1_valid << endl
			   << "	   Channel H2 Valid " << Config::h2_valid << endl
			   << "	   Channel H3 Valid " << Config::h3_valid << endl
			   << "	   Channel H4 Valid " << Config::h4_valid << endl
			   << "	   Channel H5 Valid " << Config::h5_valid << endl
			   << "	   SPE mux code " << Config::spe_mux_code << endl
			   << "	   Channel H1 gain " << Config::h1_gain << endl
			   << "	   Channel H2 gain " << Config::h2_gain << endl
			   << "	   Channel H3 gain " << Config::h3_gain << endl
			   << "	   Channel H4 gain " << Config::h4_gain << endl
			   << "	   Channel H5 gain " << Config::h5_gain << endl
			   << "	   PRT CAL 1 " << Config::prt_cal_1 << endl
			   << "	   PRT CAL 2 " << Config::prt_cal_2 << endl
			   << "	   PRT CAL 3 " << Config::prt_cal_3 << endl
			   << "	   Scan line number as sequence count "
			   << (Config::scan_line_number_as_sequence_count?"on":"off")
			   << endl
			   << "	   Initial sequence count "
			   << Config::initial_sequence_count << endl
			   << "	   UTC days offset " << Config::utc_days_offset << endl
			   << "	   UTC milliseconds offset " <<
				Config::utc_milliseconds_offset << endl
			   << "	   UTC microseconds offset " <<
				Config::utc_microseconds_offset << endl
			   << "	   Ignore UTC time " <<
				(Config::utc_ignore_input_time?"on":"off") << endl
			   << "	   OBT coarse offset " << Config::obt_coarse_offset << endl
			   << "	   OBT fine offset " << Config::obt_fine_offset << endl
			   << "	   OBT fixed fine time " << Config::obt_fixed_fine_time
			   << endl << endl;

		}

	unsigned int mhs_prt_k_to_count(double k)
		{
//		cout << "(mhs_prt_k_to_count k=" << k;
			double c = k - 273.15;
//		cout << "c=" << c;
			unsigned int result = static_cast<unsigned int>(
				34.744*c + 1940.7024);
//		cout << " count=" << result << ")\n";
			return result;

		}

	double amsub_prt_count_to_k(unsigned int count, int16_be* coeffs)
		{
//		cout << "(amsub_prt_count_to_k count=" << count << " coeff1="
			// << coeffs[0] << " coeff2=" << coeffs[1] << " coeff3="
			// << coeffs[2] << " coeff4=" << coeffs[3];

			double temp1 = static_cast<double>(coeffs[0]) * 10e-3;

			double temp2 = static_cast<double>(coeffs[1]) * 10e-8 *
				static_cast<double>(count);

			double temp3 = static_cast<double>(coeffs[2]) *
				10e-13 * pow(static_cast<double>(count),2.0);

			double temp4 = static_cast<double>(coeffs[3]) *
				10e-19 * pow(static_cast<double>(count),3.0);

//		cout << " temp1=" << temp1 << " temp2=" << temp2 << " temp3="
			// << temp3 << " temp4=" << temp4 << " k="
			// << (temp1+temp2+temp3+temp4) << ")\n";

			return (temp1 + temp2 + temp3 + temp4);
		}

	//
	// offset_shaft_position
	//

	void offset_shaft_position(MHSPayload::Sample& s)
		{
			const int MAX_SHAFT_POS = 51200;

			int pos = s.shaft_pos;
			pos += Config::shaft_position_offset;
			if (pos >= MAX_SHAFT_POS)
			{
				pos -= MAX_SHAFT_POS;
			}

			if (pos < 0)
			{
				pos += MAX_SHAFT_POS;
			}

			s.shaft_pos = pos;
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:

	int subcommutation_code;

	unsigned int start_scan_line;
	unsigned int start_year;
	unsigned int start_day;
	unsigned int start_millisecond;
	int record_count;

};

#undef Config

#endif
