////////////////////////////////////////////////////////////////////////////////
//
//	File: args.cpp
//
//	Provides: Args class
//
//	Purpose: Command line argument parsing
//
//	Copyright (c) 2005 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "args.hpp"

Args::Args(void)
{
}

Args::~Args(void)
{
	for(Options::iterator iter=options.begin();
		iter!=options.end();
		iter++)
	{
		delete (*iter).setter;
	}
}

void Args::add(int i, const string& full_name, int short_code, int flags,
			   const string& help, const string& doc)
{
	assert(i==0);
	add_impl(NULL, full_name, short_code, flags, help, doc);
}

void Args::add(int* i, const string& full_name, int short_code, int flags,
			   const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<long long, int>(i), full_name, short_code,
			 (flags&OPTIONS_MASK)|LONGLONG, help, doc);
}

void Args::add(unsigned int* i, const string& full_name, int short_code,
			   int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<unsigned long long, unsigned int>(i),
			 full_name, short_code, (flags&OPTIONS_MASK)|LONGLONG, help, doc);
}

void Args::add(long long* i, const string& full_name, int short_code,
			   int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<long long, long long>(i), full_name,
			 short_code, (flags&OPTIONS_MASK)|LONGLONG, help, doc);
}

void Args::add(unsigned long long* i, const string& full_name, int short_code,
			   int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<unsigned long long, unsigned long long>(i),
			 full_name, short_code, (flags&OPTIONS_MASK)|LONGLONG, help, doc);
}

void Args::add(bool* b, const string& full_name, int short_code, int flags,
			   const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<bool, bool>(b), full_name, short_code,
			 (flags&OPTIONS_MASK), help, doc);
}

void Args::add(string* s, const string& full_name, int short_code, int flags,
			   const string& help, const string& doc)
{
	if ((flags&TYPE_MASK)!= STRING && (flags&TYPE_MASK)!=FILENAME &&
		(flags&TYPE_MASK)!=DIRNAME)
	{
		flags = STRING;
	}
	add_impl(new ArgSetter::Setter<string, string>(s), full_name, short_code,
			 flags, help, doc);
}

void Args::add(deque<int>* s, const string& full_name, int short_code,
			   int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<deque<long long>, deque<int> >(s),
			 full_name, short_code, (flags&OPTIONS_MASK)|MULTIPLE|STRING, help,
			 doc);
}

void Args::add(deque<unsigned int>* s, const string& full_name, int short_code,
			   int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<deque<unsigned long long>,
			 deque<unsigned int> >(s), full_name, short_code,
			 (flags&OPTIONS_MASK)|MULTIPLE|STRING, help, doc);
}

void Args::add(deque<long long>* s, const string& full_name, int short_code,
			   int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<deque<long long>, deque<long long> >(s),
			 full_name, short_code, (flags&OPTIONS_MASK)|MULTIPLE|STRING, help,
			 doc);
}

void Args::add(deque<unsigned long long>* s, const string& full_name,
			   int short_code, int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<deque<unsigned long long>,
			 deque<unsigned long long> >(s), full_name, short_code,
			 (flags&OPTIONS_MASK)|MULTIPLE|STRING, help, doc);
}

void Args::add(deque<string>* s, const string& full_name, int short_code,
			   int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<deque<string>, deque<string> >(s),
			 full_name, short_code, (flags&OPTIONS_MASK)|MULTIPLE|STRING, help,
			 doc);
}

void Args::add(double* d, const string& full_name, int short_code,
			   int flags, const string& help, const string& doc)
{
	add_impl(new ArgSetter::Setter<double, double >(d),
			 full_name, short_code, (flags&OPTIONS_MASK)|MULTIPLE|DOUBLE, help,
			 doc);
}

void Args::add_impl(ArgSetter::SetterBase* setter, const string& full_name,
					int short_code, int flags, const string& help,
					const string& doc)
{
	for(Options::iterator iter=options.begin();
		iter!=options.end();
		iter++)
	{
		if ((*iter).short_code == short_code && short_code != 0)
		{
			string a;
			stringstream str;
			str << short_code;
			str >> a;
			if (isascii(short_code))
			{
				a += " (";
				a += static_cast<char>(short_code);
				a += ")";
			}
			throw_error(string("Internal error: duplicate short option code ") +
						a);
		}
	}

	Option option;
	option.setter = setter;
	option.full_name = full_name;
	option.short_code = short_code;
	option.flags = flags;
	option.help = help;
	option.doc = doc;
	option.used = false;
	option.value_set = false;
	options.push_back(option);
}

void Args::add_group(string text)
{
	if (text[text.length()-1]!=':')
	{
		text += ":";
	}
	Option option;
	option.setter = NULL;
	option.short_code = 0;
	option.flags = OPTION_GROUP;
	option.help = text;
	options.push_back(option);
}

void Args::set_doc_head(const string& _doc_head)
{
	doc_head = _doc_head;
}

void Args::set_doc_tail(const string& _doc_tail)
{
	doc_tail = _doc_tail;
}

void Args::set_package(const string& _package)
{
	package = _package;
}

void Args::set_app(const string& _app)
{
	app = _app;
}

void Args::set_version(const string& _version)
{
	version = _version;
}

void Args::set_maintainer(const string& _maintainer)
{
	maintainer = _maintainer;
}

void Args::set_copyright(const string& _copyright)
{
	copyright = _copyright;
}

void Args::set_license(int)
{
	license = string("See LICENSE file for conditions of usage and copying.");
}

void Args::set_usage(const string& _usage)
{
	usage = _usage;
}

void Args::set_author(const string& _author)
{
	author = _author;
}

void Args::add_related_prog(const string& prog)
{
	other_progs.push_back(prog);
}

void Args::add_example(const string& command, const string& desc)
{
	Example example;
	example.command = command;
	example.desc = desc;
	examples.push_back(example);
}

void Args::add_env_var(const string& name, const string& desc)
{
	EnvVar var;
	var.name = name;
	var.desc = desc;
	env_vars.push_back(var);
}

void Args::set_args(int user_argc, char* user_argv[])
{
	add_group("Program information:");
	add(0, "version", ARGS_VERSION, NONE, "show version information and exit");
	add(0, "help", '?', NONE, "show help message and exit");
	if (examples.empty())
	{
		add(0, "long-help", LONG_HELP, NONE, "show extended help");
	}
	else
	{
		string exam;
		if (!examples.empty())
		{
			exam = " and examples";
		}
		add(0, "long-help", LONG_HELP, NONE, string("show extended help")+exam);
	}
	add(0, "docbook-help", DOCBOOK_HELP, ONLY_SHOW_IN_LONG_HELP,
		"show extended help text formatted as Docbook XML");

	bool found_default = false;
	Option default_option;
	default_option.setter = NULL;
	default_option.short_code = PARAMETER;
	default_option.flags = MULTIPLE;
	for(Options::iterator iter=options.begin();
		iter!=options.end();
		iter++)
	{
		if (((*iter).flags&DEFAULT_OPTION)!=0)
		{
			if (found_default)
			{
				throw_error(string("Internal error: options ") +
							default_option.full_name + " and " + (*iter).full_name +
							" are both default options");
			}
			default_option = *iter;
			found_default = true;
		}
	}

	Option* mode = &default_option;
	bool found_first = false; // record whether we have found >0 instances where
	// multiple values are allowed
	string last_arg;

	for(int a=1; a<user_argc; a++)
	{
		string arg = user_argv[a];

		// 3rd test allows values to be negative numbers, ie '--int -50'
		if (arg.length()>1 && arg[0]=='-' &&
			arg.find_first_not_of("-+0123456789")!=string::npos)
		{
			bool matched = false;
			for(Options::iterator iter=options.begin();
				iter!=options.end() && !matched;
				iter++)
			{
				if (arg==string(string("-")+static_cast<char>(
									(*iter).short_code)) ||
					arg==string(string("--")+(*iter).full_name))
				{
					matched = true;

					if ((*iter).short_code == '?')
					{
						show_help();
						exit(0);
					}
					else if ((*iter).short_code == LONG_HELP)
					{
						show_long_help();
						exit(0);
					}
					else if ((*iter).short_code == DOCBOOK_HELP)
					{
						show_docbook_help();
						exit(0);
					}
					else if ((*iter).short_code == ARGS_VERSION)
					{
						show_version();
						exit(0);
					}

					if (mode->short_code!=PARAMETER &&
						(mode->flags&DEFAULT_OPTION)==0)
					{
						if ((mode->flags&MULTIPLE)==0 ||
							((mode->flags&MULTIPLE)!=0 && !found_first))
						{
							throw_error(string("Required value missing for ") +
										"option " + last_arg);
						}
					}

					if ((*iter).used)
					{
						throw_error(string("Only one instance of option ") +
									arg + " may be specified");
					}

					if (((*iter).flags&TYPE_MASK) == 0)
					{
						if ((*iter).setter)
						{
							(*iter).setter->match("y");
						}
						else
						{
							// the flag has no value associated
							Arg newarg;
							newarg.short_code = (*iter).short_code;
							newarg.data_type = (*iter).flags & TYPE_MASK;
							argv.push_front(newarg);
						}
					}
					else
					{
						mode = &(*iter);
						if ((mode->flags&MULTIPLE)==0)
						{
							(*iter).used = true;
						}
						found_first = false;
					}
				}
			}

			if (!matched)
			{
				throw_error(string("Unknown option ")+arg);
			}

			last_arg = arg;
		}
		else
		{
			if (mode->value_set && (mode->flags&MULTIPLE)==0)
			{
				string option = string("--") + mode->full_name;
				if (mode->short_code!=0 && isascii(mode->short_code))
				{
					option += "/-";
					option += static_cast<char>(mode->short_code);
				}
				throw_error(string("Only one value for option ") + option +
							" may be set - found additional value '" + arg +
							"'");
			}
			if (mode->setter)
			{
				if (!mode->setter->match(arg))
				{
					throw_error(string("Could not parse value for option ") +
								mode->full_name);
				}
			}
			else
			{
				Arg newarg;
				newarg.short_code = mode->short_code;
				newarg.value = arg;
				newarg.data_type = mode->flags & TYPE_MASK;
				argv.push_front(newarg);
			}
			if ((mode->flags&MULTIPLE)==0)
			{
				mode = &default_option;
			}
			found_first = true;
			mode->value_set = true;
		}
	}

	// check if the user named a single-values option but didn't give the value
	if ((mode->short_code!=PARAMETER) && (mode->flags&MULTIPLE)==0 &&
		(mode->flags&DEFAULT_OPTION)==0)
	{
		string short_name;
		if (mode->short_code!=0 && isascii(mode->short_code))
		{
			short_name = string("/-") + static_cast<char>(mode->short_code);
		}
		throw_error(string("No value given for parameter --") +
					mode->full_name + short_name);
	}
}

int Args::get_next(void)
{
	if (argv.empty())
	{
		return DONE;
	}

	Arg& arg = *argv.rbegin();
	int res = arg.short_code;
	current_string = arg.value;
	argv.pop_back();

	return res;
}

string Args::get_string(void)
{
	return current_string;
}

long long Args::get_longlong(void)
{
	return strtoll(current_string.c_str(), NULL, 0);
}

double Args::get_double(void)
{
	return atof(current_string.c_str());
}

bool Args::get_bool(void)
{
	if (current_string[0]=='1' || current_string[0]=='y' ||
		current_string[0]=='Y' || current_string[0]=='t' ||
		current_string[0]=='T')
	{
		return true;
	}
	else
	{
		return false;
	}
}

string Args::xml_safe(const string& in, bool use_para)
{
	string result;
	if (use_para)
	{
		result = "<para>";
	}
	for(string::const_iterator iter=in.begin();
		iter!=in.end();
		iter++)
	{
		if ((*iter)=='<')
		{
			result += '[';
		}
		else if ((*iter)=='>')
		{
			result += ']';
		}
		else if ((*iter)=='\n')
		{
			if (use_para)
			{
				result += "</para><para>";
			}
			else
			{
				result += '\n';
			}
		}
		else
		{
			result += (*iter);
		}
	}
	if (use_para)
	{
		result += "</para>";
	}
	return result;
}

void Args::throw_error(const string& message)
{
	if (!app.empty())
	{
		cerr << app << ": ";
	}
	cerr << message << endl << "Try '" << app
		 << " --help' for more information\n";
	exit(-1);
}

void Args::show_version(void)
{
	cout << app << " (" << package << ") " << version << " built on "
		 << __DATE__ << endl << endl << copyright << endl << license << endl;
}

void Args::show_help(void)
{
	cout << "usage: " << app << " ";
	if (!options.empty())
	{
		cout << "OPTIONS ";
	}
	cout << usage << endl;
	cout << doc_head << endl << endl;
	show_options(false);
}

void Args::show_long_help(void)
{
	cout << "usage: " << app << " ";
	if (!options.empty())
	{
		cout << "OPTIONS ";
	}
	cout << usage << endl;
	if (!doc_head.empty())
	{
		cout << doc_head << endl;
	}
	cout << endl;
	show_options(true);
	cout << endl;
	if (!doc_tail.empty())
	{
		cout << doc_tail << endl << endl;
	}
	if (!env_vars.empty())
	{
		cout << "The following environment variables are used:\n";
		for(EnvVars::iterator iter=env_vars.begin();
			iter!=env_vars.end();
			iter++)
		{
			cout << "  " << (*iter).name
				 << string(max(static_cast<int>(3),static_cast<int>(18-(*iter).name.length())), ' ')
				 << (*iter).desc << endl;
		}
		cout << endl;
	}
	if (!examples.empty())
	{
		cout << "Examples of usage:\n\n";
		for(Examples::iterator iter = examples.begin();
			iter!=examples.end();
			iter++)
		{
			cout << "> " << (*iter).command << endl
				 << "	 " << (*iter).desc << endl;
		}
		cout << endl;
	}
	if (!other_progs.empty())
	{
		cout << "See also ";
		for (Progs::iterator iter=other_progs.begin();
			 iter!=other_progs.end();
			 iter++)
		{
			if (iter!=other_progs.begin())
			{
				cout << ", ";
			}
			cout << (*iter);
		}
		cout << ".\n\n";
	}
	if (!maintainer.empty())
	{
		cout << "Report bugs to " << maintainer << endl;
	}
}

void Args::show_docbook_help(void)
{
	cout << "<!DOCTYPE refentry PUBLIC \"-//OASIS//DTD DocBook XML "
		 << "V4.1.2//EN\"\n"
		 << "\"http://www.oasis-open.org/docbook/xml/4.1.2/docbookx.dtd\">\n"
		 << "<refentry id=\"" << app << "\">\n"
		 << "\n"
		 << "<refentryinfo>\n"
		 << "  <title>" << PACKAGE << "</title>\n"
//	 << "  <productname>" << "Eumetsat" << "</productname>\n"
		 << "  <date> " << __DATE__ << "</date>\n"
		 << "</refentryinfo>\n"
		 << "<refmeta>\n"
		 << "  <refentrytitle>" << app << "</refentrytitle>\n"
		 << "  <manvolnum>1</manvolnum>\n"
		 << "</refmeta>\n"
		 << "\n"
		 << "<refnamediv>\n"
		 << "  <refname>" << app << "</refname>\n"
		 << "  <refpurpose>" << doc_head << "</refpurpose>\n"
		 << "</refnamediv>\n"
		 << "\n"
		 << "<refsynopsisdiv>\n"
		 << "  <cmdsynopsis>\n"
		 << "	 <command>" << app << "</command>\n";
	if (!options.empty())
	{
		for(Options::iterator iter=options.begin();
			iter!=options.end();
			iter++)
		{
			if (!((*iter).flags&OPTION_GROUP))
			{
				string rep;
				if ((*iter).flags&MULTIPLE)
				{
					rep = " rep='repeat'";
				}
				string name = (*iter).get_double_name();
				string param;
				switch((*iter).flags&TYPE_MASK)
				{
				case STRING:
					param = " <replaceable>arg</replaceable>";
					break;

				case FILENAME:
					param = " <replaceable>filename</replaceable>";
					break;

				case DIRNAME:
					param = " <replaceable>dirname</replaceable>";
					break;

				case LONGLONG:
				case DOUBLE:
					param = " <replaceable>n</replaceable>";
					break;

				case TIME:
				case ABS_TIME:
				case REL_TIME:
					param = " <replaceable>time</replaceable>";
					break;
				}

				cout << "	   <arg" << rep << ">" << name << param
					 << "</arg>\n";
			}
		}
	}
	if (!usage.empty())
	{
		cout << "<arg><replaceable>" << xml_safe(usage,false)
			 << "</replaceable></arg>" << endl;
	}
	cout << "  </cmdsynopsis>\n"
		 << "</refsynopsisdiv>\n"
		 << "\n";

	if (!doc_tail.empty())
	{
		cout << "<refsect1>\n"
			 << "  <title>DESCRIPTION</title>\n"
			 << xml_safe(doc_tail, true)
			 << "</refsect1>\n";
	}

	cout << "<refsect1>\n"
		 << "  <title>OPTIONS</title>\n";
	bool in_group = false;
	for(Options::iterator iter=options.begin();
		iter!=options.end();
		iter++)
	{
		if ((*iter).flags&OPTION_GROUP)
		{
			if (in_group)
			{
				cout << "	 </variablelist>\n"
					 << "  </refsect2>\n";
			}
			cout << "  <refsect2><title>" << (*iter).help << "</title>\n"
				 << "	 <variablelist>\n";
			in_group = true;
		}
		else if (!((*iter).flags&HIDDEN))
		{
			if (!in_group)
			{
				cout << "	 <refsect2><title></title>\n"
					 << "	 <variablelist>\n";
				in_group = true;
			}
			cout << "	 <varlistentry>\n";
			string term = (*iter).get_double_name();
			cout << "	   <term>" << term << "</term>\n"
				 << "	   <listitem>\n";
			string help = (*iter).help;
			if (!help.empty() && help[help.length()-1]!='.')
			{
				help += '.';
			}
			string doc = (*iter).doc;
			if (!doc.empty() && doc[doc.length()-1]!='.')
			{
				doc += '.';
			}
			cout << "	   <para>" << xml_safe(help,false) << " "
				 << xml_safe(doc,false) << "</para>\n";
			cout << "	   </listitem>\n"
				 << "	 </varlistentry>\n";
		}
	}
	if (in_group)
	{
		cout << "	 </variablelist>\n"
			 << "  </refsect2>\n";
	}
	cout << "</refsect1>\n";

	if (!env_vars.empty())
	{
		cout << "<refsect1>\n"
			 << "  <title>ENVIRONMENT VARIABLES</title>"
			 << "  <para>The following environment variables are used by "
			 << "the application:</para>\n"
			 << "  <variablelist>\n";
		for(EnvVars::iterator iter=env_vars.begin();
			iter!=env_vars.end();
			iter++)
		{
			cout << "  <varlistentry><term><envar>" << (*iter).name
				 << "</envar></term>\n"
				 << "  <listitem><para>" << (*iter).desc
				 << "</para></listitem></varlistentry>\n";
		}
		cout << "</variablelist>";
		cout << "</refsect1>\n";
	}

	if (!examples.empty())
	{
		cout << "<refsect1>\n"
			 << "  <title>EXAMPLES</title>\n";
		for(Examples::iterator iter=examples.begin();
			iter!=examples.end();
			iter++)
		{
			string desc = (*iter).desc;
			if (desc[desc.length()-1]!=':')
			{
				desc += ':';
			}
			cout << "  <para>" << desc << "</para>\n"
				 << "  <para><command>" << (*iter).command
				 << "</command></para>\n";
		}
		cout << "</refsect1>\n";
	}

	if (!author.empty())
	{
		cout << "<refsect1>\n"
			 << "  <title>AUTHOR</title>\n"
			 << "  <para>Written by " << author << ".</para>\n"
			 << "</refsect1>\n";
	}

	if (!maintainer.empty())
	{
		cout << "<refsect1>\n"
			 << "  <title>REPORTING BUGS</title>\n"
			 << "  <para>Report bugs to <email>" << maintainer
			 << "</email>.</para>\n"
			 << "</refsect1>\n";
	}

	if (!copyright.empty() || !license.empty())
	{
		cout << "<refsect1>\n"
			 << "  <title>COPYRIGHT</title>\n"
			 << "  <para>" << copyright << "</para>\n"
			 << "  <para>" << license << "</para>\n"
			 << "</refsect1>\n";
	}

	if (!other_progs.empty())
	{
		cout << "<refsect1>\n"
			 << "  <title>SEE ALSO</title>\n"
			 << "  <para>";
		for(Progs::iterator iter=other_progs.begin();
			iter!=other_progs.end();
			iter++)
		{
			if (iter!=other_progs.begin())
			{
				cout << ", ";
			}
			cout << "<command>" << (*iter) << "</command>";
		}
		cout << "\n	 </para>\n"
			 << "</refsect1>\n";
	}

	cout << "</refentry>\n";
}

void Args::show_options(bool include_doc)
{
	const string INDENT = "	 ";
	const unsigned int rcolumn = 20;
	const unsigned int rcolumn_doc = 22;

	cout << "Where OPTIONS can include:\n";
	for(Options::iterator iter=options.begin();
		iter!=options.end();
		iter++)
	{
		if (((*iter).flags&HIDDEN) ||
			(((*iter).flags&ONLY_SHOW_IN_LONG_HELP) && !include_doc))
		{
			continue;
		}

		if ((*iter).flags & OPTION_GROUP)
		{
			// display an break between option groups
			cout << endl << (*iter).help << endl;
		}
		else
		{
			// display a normal option

			// first the long and short chars
			string lcolumn = INDENT + (*iter).get_double_name();

			// display the option's argument type, if any
			int type = (*iter).flags & TYPE_MASK;

			// see if multiple values are allowed
			string mult;
			if (((*iter).flags) & MULTIPLE)
			{
				mult = "...";
			}

			switch(type)
			{
			case STRING:
				lcolumn += " <arg" + mult + ">";
				break;

			case FILENAME:
				lcolumn += " <file" + mult + ">";
				break;

			case DIRNAME:
				lcolumn += " <dir" + mult + ">";
				break;

			case LONGLONG: // intentioal drop through
			case DOUBLE:
				lcolumn += " <n" + mult + ">";
				break;

			case TIME:
				lcolumn += " <time>";
				break;

			case ABS_TIME:
				lcolumn += " <time>";
				break;

			case REL_TIME:
				lcolumn += " <+time>";
				break;
			}

			// If the option name if too long, place the description on the
			// next line
			if (lcolumn.length() < rcolumn)
			{
				cout << lcolumn << string(rcolumn-lcolumn.length(), ' ')
					 << (*iter).help << endl;
			}
			else
			{
				cout << lcolumn << endl << string(rcolumn, ' ')
					 << (*iter).help << endl;
			}

			// show the doc string only in extended help modes
			if (include_doc && !(*iter).doc.empty())
			{
				cout << string(rcolumn_doc, ' ') << (*iter).doc << endl;
			}
		}
	}
}

string Args::Option::get_double_name(void) const
{
	string term;
	if (short_code!=0 && isascii(short_code))
	{
		term = "-";
		term += static_cast<char>(short_code);
	}
	if (!full_name.empty())
	{
		if (!term.empty())
		{
			term += ", ";
		}
		term += "--" + full_name;
	}
	if (flags&DEFAULT_OPTION)
	{
		term = string("[") + term + "]";
	}
	return term;
}
