////////////////////////////////////////////////////////////////////////////////
//
//	File: args.hpp
//
//	Provides: Args class
//
//	Purpose: Command line argument parsing
//
//	Copyright (c) 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

namespace ArgSetter
{
	class SetterBase
	{
	public:
		virtual ~SetterBase(void) {}
		virtual bool match(const string& str) =0;
	};

	template <class GenericType, class SpecificType> class Setter :
		public SetterBase
	{
	};

	template <class SpecificType> class Setter<deque<long long>, SpecificType>
		: public SetterBase
	{
	public:
		Setter<deque<long long>, SpecificType>(SpecificType* _variable)
		: variable(_variable)
			{
			}

		bool match(const string& input)
			{
				if (input.find_first_not_of("+-0123456789") != string::npos)
				{
					cerr << "Non-numeric value passed to numeric option\n";
					return false;
				}
				stringstream str;
				str << input;
				long long transform;
				str >> transform;
				variable->push_back(transform);
				return true;
			}

	private:
		SpecificType* variable;
	};

	template <class SpecificType> class Setter<
		deque<unsigned long long>, SpecificType> : public SetterBase
	{
	public:
		Setter<deque<unsigned long long>, SpecificType>(
			SpecificType* _variable)
		: variable(_variable)
			{
			}

		bool match(const string& input)
			{
				if (input.find_first_not_of("+-0123456789") != string::npos)
				{
					cerr << "Non-numeric value passed to numeric option\n";
					return false;
				}
				if (input.find_first_of("-") != string::npos)
				{
					cerr << "Negative value passed to option which does not "
						 << "accept negative values\n";
					return false;
				}
				stringstream str;
				str << input;
				long long transform;
				str >> transform;
				variable->push_back(transform);
				return true;
			}

	private:
		SpecificType* variable;
	};

	template <> class Setter<deque<string>,deque<string> > : public SetterBase
	{
	public:
		Setter<deque<string>,deque<string> >(deque<string>* _variable)
		: variable(_variable)
			{
			}

		bool match(const string& input)
			{
				variable->push_back(input);
				return true;
			}

	private:
		deque<string>* variable;
	};

	// this catches all int-style data types
	template <class SpecificType> class Setter<long long, SpecificType> :
		public SetterBase
	{
	public:
		Setter<long long, SpecificType>(SpecificType* _variable)
		: variable(_variable)
			{
			}

		bool match(const string& input)
			{
				if (input.find_first_not_of("+-0123456789") != string::npos)
				{
					cerr << "Non-numeric value passed to numeric option\n";
					return false;
				}
				stringstream str;
				str << input;
				str >> (*variable);
				return true;
			}

	private:
		SpecificType* variable;
	};

	template <class SpecificType> class Setter<
		unsigned long long, SpecificType> : public SetterBase
	{
	public:
		Setter<unsigned long long, SpecificType>(SpecificType* _variable)
		: variable(_variable)
			{
			}

		bool match(const string& input)
			{
				if (input.find_first_not_of("+-0123456789") != string::npos)
				{
					cerr << "Non-numeric value passed to numeric option\n";
					return false;
				}
				if (input.find_first_of("-") != string::npos)
				{
					cerr << "Negative value passed to option which does not "
						 << "accept negative values\n";
					return false;
				}
				stringstream str;
				str << input;
				str >> (*variable);
				return true;
			}

	private:
		SpecificType* variable;
	};

	template <> class Setter<bool,bool> : public SetterBase
	{
	public:
		Setter<bool,bool>(bool* _variable)
		: variable(_variable)
			{
			}

		bool match(const string& input)
			{
				if (input[0]=='1' || input[0]=='y' || input[0]=='Y' ||
					input[0]=='t' || input[0]=='T')
				{
					*variable = true;
				}
				else
				{
					*variable = false;
				}
				return true;
			}

	private:
		bool* variable;
	};

	template <> class Setter<string,string> : public SetterBase
	{
	public:
		Setter<string,string>(string* _variable)
		: variable(_variable)
			{
			}

		bool match(const string& input)
			{
				*variable = input;
				return true;
			}

	private:
		string* variable;
	};

	template <> class Setter<double,double> : public SetterBase
	{
	public:
		Setter<double,double>(double* _variable)
		: variable(_variable)
			{
			}

		bool match(const string& input)
			{
				if (input.find_first_not_of("0123456789-.") != string::npos)
				{
					cerr << "Cannot interpret argument '" << input << "' as a floating point number.\n";
					return false;
				}
				*variable = atof(input.c_str());
				return true;
			}

	private:
		double* variable;
	};
}

class Args
{
public:

	// data types to be given for an option
	enum { NONE = 0x0000 };
	enum { STRING = 0x0001 };
	enum { FILENAME = 0x0002 };
	enum { DIRNAME = 0x003 };
	enum { LONGLONG = 0x0004 };
	enum { DOUBLE = 0x0005 };
	enum { BOOL = 0x0006 };
	enum { TIME = 0x0007 };
	enum { ABS_TIME = 0x0008 };
	enum { REL_TIME = 0x0009 };
	enum { TYPE_MASK = 0x000f };

	// modifiers for options
	enum { DEFAULT_OPTION = 0x0010 };
	enum { MULTIPLE = 0x0020 };
	enum { HIDDEN = 0x0040 };
	enum { ONLY_SHOW_IN_LONG_HELP = 0x0080 };
	enum { OPTION_GROUP = 0x0100 };
	enum { MULTIPLE_MASK = 0xffef };
	enum { OPTIONS_MASK = 0xfff0 };

	// misc flags
	enum { DONE = 0x1000 };
	enum { GPL = 0x1001 };

	// return codes from get_next()
	enum { PARAMETER = 0x2000 };
	enum { ARGS_VERSION = 0x2001 };
	enum { HELP = 0x2002 };
	enum { LONG_HELP = 0x2003 };
	enum { DOCBOOK_HELP = 0x2004 };
	enum { USER_OPTION = 0x2005 };

	Args(void);
	~Args(void);

	void add(int, const string& full_name, int short_code, int flags,
			 const string& help = "", const string& doc = "");
	void add(int*, const string& full_name, int short_code, int flags,
			 const string& help = "", const string& doc = "");
	void add(unsigned int*, const string& full_name, int short_code, int flags,
			 const string& help = "", const string& doc = "");
	void add(long long*, const string& full_name, int short_code, int flags,
			 const string& help = "", const string& doc = "");
	void add(unsigned long long*, const string& full_name, int short_code,
			 int flags, const string& help = "", const string& doc = "");
	void add(string*, const string& full_name, int short_code, int flags,
			 const string& help = "", const string& doc = "");
	void add(bool*, const string& full_name, int short_code, int flags,
			 const string& help = "", const string& doc = "");
	void add(deque<string>*, const string& full_name, int short_code, int flags,
			 const string& help = "", const string& doc = "");
	void add(deque<int>*, const string& full_name, int short_code, int flags,
			 const string& help = "", const string& doc = "");
	void add(deque<unsigned int>*, const string& full_name, int short_code,
			 int flags, const string& help = "", const string& doc = "");
	void add(deque<long long>*, const string& full_name, int short_code,
			 int flags, const string& help = "", const string& doc = "");
	void add(deque<unsigned long long>*, const string& full_name,
			 int short_code, int flags, const string& help = "",
			 const string& doc = "");
	void add(double*, const string& full_name,
			 int short_code, int flags, const string& help = "",
			 const string& doc = "");

	void add_group(string text);
	void set_doc_head(const string& _doc_head);
	void set_doc_tail(const string& _doc_tail);
	void set_package(const string& _package);
	void set_app(const string& _app);
	void set_version(const string& _version);
	void set_maintainer(const string& _maintainer);
	void set_copyright(const string& _copyright);
	void set_license(int);
	void set_usage(const string& _usage);
	void set_author(const string& _author);

	/// to be inserted into the 'see other' section
	void add_related_prog(const string& prog);

	void add_example(const string& command, const string& desc);

	/// useful environment variables
	void add_env_var(const string& name, const string& desc);

	void set_args(int argc, char* argv[]);
	int get_next(void);

	string get_string(void);
	long long get_longlong(void);
	double get_double(void);
	bool get_bool(void);

	void throw_error(const string& message);
	void show_version(void);
	void show_help(void);
	void show_long_help(void);
	void show_docbook_help(void);
	void show_options(bool include_doc);

private:

	void add_impl(ArgSetter::SetterBase* setter, const string& full_name,
				  int short_code, int flags, const string& help,
				  const string& doc = "");

	string xml_safe(const string& input, bool use_para);

	string doc_head;
	string doc_tail;
	string package;
	string app;
	string version;
	string maintainer;
	string copyright;
	string license;
	string usage;
	string author;

	typedef vector<string> Progs;
	Progs other_progs;

	struct Option
	{
		ArgSetter::SetterBase* setter;
		string full_name;
		int short_code;
		int flags;
		string help;
		string doc;
		bool used;
		bool value_set;
		string get_double_name(void) const;
	};

	typedef vector<Option> Options;
	Options options;

	struct Example
	{
		string command;
		string desc;
	};
	typedef vector<Example> Examples;
	Examples examples;

	struct EnvVar
	{
		string name;
		string desc;
	};
	typedef vector<EnvVar> EnvVars;
	EnvVars env_vars;

	string current_string;

	struct Arg
	{
		int short_code;
		string value;
		int data_type;
	};

	typedef deque<Arg> Argv;
	Argv argv;
};
