////////////////////////////////////////////////////////////////////////////////
//
//	File: avhrr_to_ccsds.cpp
//
//	Provides: avhrr_to_ccsds executable
//
//	Purpose: Extract a stream of source packets from a NOAA L1b AVHRR product
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "args.hpp"
#include "filereader.hpp"
#include "noaaproductreader.hpp"
#include "readfromstdin.hpp"
#include "multifilewriter.hpp"
#include "writetostdout.hpp"
#include "singlefilewriter.hpp"
#include "avhrrreader.hpp"
#include "ccsdsextractor.hpp"
#include "imagewriter.hpp"
#include "avhrrinterpolator.hpp"
#include "productextractor.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "avhrr_to_ccsds";
const string CONFIG_DIR_VAR = "METOPIZER_HOME";
const string DEFAULT_CONFIG_FILE = "avhrr.config";
const string MAINTAINER = "elson@eumetsat.int";
const string COPYRIGHT = "Copyright (c) 2006 Eumetsat.";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	string in_file;
	string ccsds_file;
	string image_file;
	string config_file;
	string write_product;

	bool interpolate = false;
	bool jpeg = false;
	int bps=-1;
	string channels;

	bool verbose = true;

	Args args;
	args.set_doc_head("nExtract a stream of MetOp source packets from a NOAA L1b AVHRR product");
	args.set_doc_tail("");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright(COPYRIGHT);
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_env_var("METOPIZER_HOME", "Location to search for avhrr.config file");

	args.add_group("Input settings:");
	args.add(&in_file, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input filename");
	args.add(&ccsds_file, "out", 'o', Args::FILENAME, "Select output filename");
	args.add(&image_file, "image", 0, 0, "Write output as a set of image files (default=TIFF)");
	args.add(&config_file, "config", 0, Args::FILENAME, "Specify config file");

	args.add_group("Processing");
	args.add(&bps, "bps", 0, 0, "Specify bits per pixel of input product (8, 10 or 16)");
	args.add(&interpolate, "interpolate", 0, 0, "Interpolate GAC resolution input to LAC resolution");
	args.add(&write_product, "write-product", 0, Args::FILENAME, "Write interpolated product back to disk");
#ifndef _NO_JPEG
	args.add(&jpeg, "jpeg", 0, 0, "Write image in JPEG format");
#endif
	args.add(0, "quiet", 'q', 0, "Omit additional messages");
	args.add(0, "", 'z', 0, "Use simple busy animations");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case 'q':
			verbose = false;
			break;

		case Args::USER_OPTION:
			_debug = &cout;
			break;

		case 'z':
			Spinner::use_simple_spinner();
			break;
		}
	}

	// if no config file specified check for an implicit one
	if (config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_file = string(config_dir_var) + "/" + DEFAULT_CONFIG_FILE;
		}
		else
		{
			// use the default filename METOPIZER_HOME
			config_file = string(DEFAULT_METOPIZER_HOME) + "/" + DEFAULT_CONFIG_FILE;
		}
	}

	// fix streams if we're streaming to stdout
	if (image_file.empty() && ccsds_file.empty() && write_product.empty())
	{
		_info = &cerr;
		_warning = &cerr;
		verbose = false;
		Spinner::restream(&onull);
	}

	FileProducer* file_source;
	if (in_file.empty())
	{
		file_source = new ReadFromStdin();
	}
	else
	{
		file_source = new FileReader(in_file);
	}

	NOAAProductReader noaa_reader(*file_source, bps, channels);

	AVHRRInterpolator* interpolator = NULL;
	AVHRRReader* avhrr_reader;

	if (interpolate)
	{
		interpolator = new AVHRRInterpolator(noaa_reader, bps, channels);
		avhrr_reader = new AVHRRReader(*interpolator, config_file);
	}
	else
	{
		avhrr_reader = new AVHRRReader(noaa_reader, config_file, bps, channels);
	}

	if (!write_product.empty())
	{
		// write product file back to disk

		ProductExtractor
			extractor(interpolator?
					  dynamic_cast<ProductProducer&>(*interpolator)
					  : dynamic_cast<ProductProducer&>(noaa_reader));

		SingleFileWriter writer(&extractor, write_product);

		if (verbose && !gerror)
		{
			file_source->output_text_summary();
			noaa_reader.output_text_summary();
			if (interpolator)
			{
				interpolator->output_text_summary();
			}
			extractor.output_text_summary();
			writer.output_text_summary();
		}
	}
	else if (!image_file.empty())
	{
		// image output

		Image::Format image_format = Image::IMAGE_TIFF;
#ifndef _NO_JPEG
		if (jpeg)
		{
			image_format = Image::IMAGE_JPEG;
		}
#endif
		ImageWriter writer(*avhrr_reader, image_file, image_format);

		if (verbose && !gerror)
		{
			file_source->output_text_summary();
			noaa_reader.output_text_summary();
			if (interpolator)
			{
				interpolator->output_text_summary();
			}
			avhrr_reader->output_text_summary();
			writer.output_text_summary();
		}
	}
	else
	{
		// source packet output

		CCSDSExtractor extractor(*avhrr_reader);

		if (ccsds_file.empty())
		{
			WriteToStdout writer(&extractor);
		}
		else
		{
			SingleFileWriter writer(&extractor, ccsds_file);

			if (verbose && !gerror)
			{
				file_source->output_text_summary();
				noaa_reader.output_text_summary();
				if (interpolator)
				{
					interpolator->output_text_summary();
				}
				avhrr_reader->output_text_summary();
				extractor.output_text_summary();
				writer.output_text_summary();
			}
		}
	}

	delete file_source;
	delete interpolator;
	delete avhrr_reader;

	return 0;
}
