////////////////////////////////////////////////////////////////////////////////
//
//	File: avhrrbase.hpp
//
//	Provides: AVHRRBase ABC
//
//	Purpose: General product read functions for NOAA AVHRR products
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef avhrrbase_defined
#define avhrrbase_defined

#include "productproducer.hpp"

class AVHRRBase
{
protected:

	AVHRRBase(ProductProducer& _source, int user_bps = -1,
			  const string& user_channels = "")
		: source(_source)
		{

			DEBUG("avhrrbase constructor bps is " << user_bps
				  << " channels is '" << user_channels << "'\n");

			// we must configure 3 parameters:
			// bps bits per sample
			// channels 0-4 booleans
			// sample_count samples per scan line

			// these are deduced as follows:
			// - if the user has specified the values, those are used
			// - otherwise if a noaa archive header is present that is used
			// - otherwise if the scan line size indicates 10bps OR 8 bps 5
			// channel
			//	 OR 16bps 5 channel, we rely assume thats ok.
			// - otherwise fail.

			// in all cases if verbose output is selected the process can be
			// viewed.

//		int record_size = header.gen.logical_record_length;

			// set bits per sample
			if (user_bps > -1)
			{
				bps = user_bps;
				bps_source = USER;
			}
			else if (source.get_archive_header())
			{
				char buffer[3];
				buffer[0] = source.get_archive_header()->
					query.sensor_data_word_size[0];
				buffer[1] = source.get_archive_header()->
					query.sensor_data_word_size[1];
				buffer[2] = '\0';
				bps = atoi(buffer);
				if (bps != 8 && bps != 10 && bps != 16)
				{
					error << "Bad bps found in archive header '"
						  << buffer << "'\n";
					return;
				}
				bps_source = ARCHIVE_HEADER;
			}
			else
			{
				bps_source = RECORD_SIZE;
				switch(source.get_record_size())
				{
				case 3584: // 8bps GAC
					bps = 8;
					break;
				case 4608: // 10bps GAC
					bps = 10;
					break;
				case 5632: // 16bps GAC
					bps = 16;
					break;
				case 12288: // 8bps LAC
					bps = 8;
					break;
				case 15872: // 10bps LAC
					bps = 10;
					break;
				case 22528: // 16bps LAC
					bps = 16;
					break;
				default:
					error << "Cannot determine bits per sample - use --bps "
						  << "option or use a product with the SAA archive "
						  << "header attached\n";
					return;
				}
			}

			// determine which channels are in use
			if (!user_channels.empty())
			{
				channel_count = 0;
				for(int i=0; i<5; i++)
				{
					channels[i] = tolower(user_channels[i])=='y' ? 1 : 0;
					if (channels[i])
					{
						channel_count++;
					}
				}
				channel_source = USER;
			}
			else if (source.get_archive_header())
			{
				channel_count = 0;
				for(int i=0; i<5; i++)
				{
					if (source.get_archive_header()->
						query.channel_select_flags[i] == 'Y')
					{
						channels[i] = true;
						channel_count++;
					}
					else
					{
						channels[i] = false;
					}
				}
				channel_source = ARCHIVE_HEADER;
			}
			else
			{
				if (source.get_record_size()==3584
					|| source.get_record_size()==4608
					|| source.get_record_size()==5632
					|| source.get_record_size()==12288
					|| source.get_record_size()==15872
					|| source.get_record_size()==22528)
				{
					channel_source = RECORD_SIZE;
					for(int i=0; i<5; i++)
					{
						channels[i] = true;
					}
					channel_count = 5;
				}
				else
				{
					error << "Cannot which channels are in use - specify "
						  << "--channels option or use a product with the "
						  << "SAA archive header attached\n";
					return;
				}
			}

			// determine number of samples (LAC=2048, GAC=409)
			uint16_be dtc =
				source.get_product_header()->data_type_code;

			if (dtc==1 || dtc==3 || dtc==13)
			{
				// its LAC/HRPT
				sample_count = 2048;
			}
			else if (dtc==2)
			{
				// its GAC
				sample_count=409;
			}
			else
			{
				error << "AVHRRReader cannot process products with data type "
					  << " code of " << dtc << endl;
				return;
			}

//		cout << "avhrr block size " << source.get_record_size()
//		 << " sample_count " << sample_count << endl;

		}

	typedef NOAA_L1b_AVHRR_Data::Sample AVHRRSample;

	//
	// get_8bps_sensor_data
	//

	int get_8bps_sensor_data(NOAA_L1b_AVHRR_Data* data, int number)
		{
			uint32_be base = data->sensor_data[number/4];
			return (((base>>((3-number)*8))&255)<<2);
		}

	//
	// get_10bps_sensor_data
	//

	int get_10bps_sensor_data(NOAA_L1b_AVHRR_Data* data, int number)
		{
			uint32_be base = data->sensor_data[number/3];
			switch(2-(number%3))
			{
			case 0 : return (base & 0x3ff);
			case 1 : return ((base>>10) & 0x3ff);
			case 2 : return ((base>>20) & 0x3ff);
			default: return 0;
			}
		}

	//
	// get_16bps_sensor_data
	//

	int get_16bps_sensor_data(NOAA_L1b_AVHRR_Data* data, int number)
		{
			uint32_be base = data->sensor_data[number/2];
			if (number%2==0)
			{
				return ((base>>16)&1023);
			}
			else
			{
				return (base&1023);
			}
		}

	//
	// get_product_sample
	//

	AVHRRSample get_product_sample(NOAA_L1b_AVHRR_Data* data, int sample)
		{
			if (sample<0)
			{
				sample = 0;
			}
			if (sample >= sample_count)
			{
				sample = sample_count;
			}

			AVHRRSample result;
//		if (sample<0 || sample>=sample_count)
//		{
//		result.channels[0] = 0;
//		result.channels[1] = 0;
//		result.channels[2] = 0;
//		result.channels[3] = 0;
//		result.channels[4] = 0;
//		return result;
//		}
//		else
			{
				for(int i=0; i<5; i++)
				{
					if (channels[i])
					{
						switch(bps)
						{
						case 8:
							result.channels[i] =
								get_8bps_sensor_data(data,sample*
													 channel_count+i);
							break;
						case 10:
							result.channels[i] =
								get_10bps_sensor_data(data,sample*
													  channel_count+i);
							break;
						case 16:
							result.channels[i] =
								get_16bps_sensor_data(data, sample*
													  channel_count+i);
							break;
						}
					}
					else
					{
						result.channels[i] = 0;
					}
				}
			}
			return result;
		}


	//
	// output_sources_summary
	//

	void output_source_summary( textoutstream_type& os )
		{
			os << "	   Input samples per scan line " << sample_count << endl
			   << "	   Bits per sample " << bps;
			if (bps_source==USER)
			{
				os << " (user specified)\n";
			}
			else if (bps_source==ARCHIVE_HEADER)
			{
				os << " (read from archive header)\n";
			}
			else
			{
				os << " (deduced from record size)\n";
			}
			os << "	   Channel 1 " << (channels[0]?"on":"off");
			if (channel_source==USER)
			{
				os << " (user spacified)\n";
			}
			else if (channel_source==ARCHIVE_HEADER)
			{
				os << " (read from archive header)\n";
			}
			else
			{
				os << " (deduced from record size)\n";
			}
			os << "			   2 " << (channels[1]?"on":"off") << endl
			   << "			   3 " << (channels[2]?"on":"off") << endl
			   << "			   4 " << (channels[3]?"on":"off") << endl
			   << "			   5 " << (channels[4]?"on":"off") << endl;
		}

protected:

	ProductProducer& source;

	enum Source { USER, ARCHIVE_HEADER, RECORD_SIZE };
	Source bps_source;
	Source channel_source;

	int bps;
	int channel_count;
	bool channels[5];
	int sample_count; // samples per scan line


};

#endif
