////////////////////////////////////////////////////////////////////////////////
//
//	File: avhrrimagebase.hpp
//
//	Provides: AVHRRImageBase ABC
//
//	Purpose: Graphics output functions for the AVHRR
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef avhrrimagebase_defined
#define avhrrimagebase_defined

class AVHRRImageBase
{
public:

	typedef NOAA_L1b_AVHRR_Data::Sample AVHRRSample;

	//
	// get_image_count
	//

	int get_image_count(void)
		{
			return 8;
		}

	//
	// write_image
	//

	void write_image(int number, Image* image)
		{
			if (number < 5)
			{
				write_monochrome_image(image, number);
			}
			else if (number == 5)
			{
				write_composite_image(image);
			}
			else if (number == 6)
			{
				write_cloud_image(image);
			}
			else if (number == 7)
			{
				write_vegetation_image(image);
			}
		}

	//
	// write_monochrome_image
	//

	void write_monochrome_image(Image* image, int number)
		{
			spinner.reset(new Spinner(string("Assembling image ")+
									  ntos(number+1)));

			int width = (*avhrr_scan_lines.begin()).size();
			int height = avhrr_scan_lines.size();

			image->set_filename_suffix(string("."+ntos(number+1)));
			image->set_attributes( Image::IMAGE_MONO,
								   width,
								   height);
			image->begin_output();

			byte* row = new byte[width];

			for(AVHRRScanLines::iterator line=avhrr_scan_lines.begin();
				line!=avhrr_scan_lines.end();
				line++)
			{
				for(int x=0; x<width; x++)
				{
					row[x] = (*line)[x].channels[number]>>2;
//			row[x] = (((*line)[x].channels[number]>>2) & (128+64+32)) << 0;
				}
				image->write_line(row);
				spinner->tick();
			}

			delete[] row;
			image->end_output();
			spinner->end();
		}

	//
	// write_composite_image
	//

	void write_composite_image(Image* image)
		{
			spinner.reset(new Spinner(string("Assembling composite image")));

			int width = (*avhrr_scan_lines.begin()).size();
			int height = avhrr_scan_lines.size();

			image->set_filename_suffix(".composite");
			image->set_attributes(Image::IMAGE_COLOUR,
								  width,
								  height);
			image->begin_output();

			byte* row = new byte[width*3];

			for(AVHRRScanLines::iterator line=avhrr_scan_lines.begin();
				line!=avhrr_scan_lines.end();
				line++)
			{
				for(int x=0; x<((*line).size()); x++)
				{
					// original colour map
//			image->set_pixel(Pos(x,y),Col(
//					 (*line)[x].channels[1]>>2,
//					 (*line)[x].channels[2]>>2,
//					 (*line)[x].channels[0]>>2));
					// seashark colour map
//			image->set_pixel(Pos(x,y),Col(
//					 (*line)[x].channels[0]>>2,
//					 (*line)[x].channels[1]>>2,
//					 (*line)[x].channels[3]>>2));

					row[x*3] = (*line)[x].channels[0]>>2;
					row[x*3+1] = (*line)[x].channels[1]>>2;
					row[x*3+2] = (*line)[x].channels[3]>>2;
				}
				image->write_line(row);
				spinner->tick();
			}

			delete[] row;
			image->end_output();
			spinner->end();
		}

	//
	// write_cloud_image
	//

	void write_cloud_image(Image* image)
		{
			spinner.reset(new Spinner(string("Assembling cloud image")));

			int width = (*avhrr_scan_lines.begin()).size();
			int height = avhrr_scan_lines.size();

			image->set_filename_suffix(".cloud");
			image->set_attributes(Image::IMAGE_COLOUR,
								  width,
								  height);
			image->begin_output();

			byte* row = new byte[width*3];

			for(AVHRRScanLines::iterator line=avhrr_scan_lines.begin();
				line!=avhrr_scan_lines.end();
				line++)
			{
				for(int x=0; x<((*line).size()); x++)
				{
					AVHRRSample& sample = (*line)[x];

					// seashark colour selection
//			int red = sample.channels[0];
//			int green = sample.channels[1];
//			int blue = sample.channels[3];
					// original colour selection
					int red = sample.channels[1];
					int green = sample.channels[2];
					int blue = sample.channels[0];

					// tint dark areas blue (otherwise the oceans just come
					// out black)
					if (red<192 && green<192 && blue<192) // 192
					{
						blue += 192;
					}

					// cloud test.
					// we use channel 4 for extract cloud data
//			int cloud = get_noaa_earth_view_word(x*5+3);
					int cloud = sample.channels[3];

					// if the cloud is heavy do a plain copy to the main pixel
					if (cloud >= 560) // 580
					{
						red = cloud;
						green = cloud;
						blue = cloud;
					}

					// if the cloud is a bit less heavy blend it into the main
					// pixel
					// This avoids sharp edges around the borders of clouds
					if (cloud >= 540 && cloud < 560) // 560 580
					{
						red = (red+cloud)/2;
						green = (green+cloud)/2;
						blue = (blue+cloud)/2;
					}

					// now the purple trap
					double purpleness = (((red+blue)/2)-green)/
							fabs(static_cast<double>(red-blue));
					if (purpleness>2.0)
					{
						green = (red+blue)/2;
					}

//			image->set_pixel(Pos(x,y),Col(red>>2,green>>2,blue>>2));
					row[x*3] = red>>2;
					row[x*3+1] = green>>2;
					row[x*3+2] = blue>>2;
				}
				image->write_line(row);
				spinner->tick();
			}

			image->end_output();
			delete[] row;
			spinner->end();
		}

	//
	// write_vegetation_image
	//

	void write_vegetation_image(Image* image)
		{
			spinner.reset(new Spinner(string("Assembling vegetation image")));
//		info << "Assembling vegetation image\n";

			int width = (*avhrr_scan_lines.begin()).size();
			int height = avhrr_scan_lines.size();

			image->set_filename_suffix(".vegetation");
			image->set_attributes(Image::IMAGE_COLOUR,
								  width,
								  height);
			image->begin_output();

			byte* row = new byte[width*3];

			double* veg = NULL;
			int* red = NULL;
			int* green = NULL;
			int* blue = NULL;

			try
			{
				veg = new double[width*height];
				red = new int[width*height];
				green = new int[width*height];
				blue = new int[width*height];
			}
			catch( ... )
			{
				delete[] veg;
				delete[] red;
				delete[] green;
				delete[] blue;
				delete image;

				error << "Out of memory\n";
				return;
			}

			double mi = 0;
			double ma = 0;

			int y=0;
			for(AVHRRScanLines::iterator line=avhrr_scan_lines.begin();
				line!=avhrr_scan_lines.end();
				line++)
			{
				for(int x=0; x<width; x++)
				{
					AVHRRSample& sample = (*line)[x];

					double a = sample.channels[0];
					double b = sample.channels[1];

					double pixel = (b-a)/(b+a);

					if (pixel < mi)
					{
						mi = pixel;
					}

					if (pixel > ma)
					{
						ma = pixel;
					}

					veg[x+y*width] = pixel;
					red[x+y*width] = sample.channels[0];
					green[x+y*width] = sample.channels[1];
					blue[x+y*width] = sample.channels[2];

				}
				spinner->tick();
				y++;
			}

//		spinner->pause();
//		info << "Unnormalised image min " << mi << " max " << ma << endl;
//		spinner->restart();

			for(int y=0; y<height; y++)
			{
				for(int x=0; x<width; x++)
				{
					int r = red[x+y*width]>>2;
					int g = green[x+y*width]>>2;
					int b = blue[x+y*width]>>2;
					double pixel = veg[x+y*width];
					pixel = static_cast<int>((pixel-mi) * 250.0 / (ma-mi));

					if (pixel > 128)
					{
						int offset = static_cast<int>(pixel-128);
						r-=(offset/2);
						g+=(offset/2);
						b-=(offset/2);

						if (r<0) r=0;
						if (g>255) g=255;
						if (b<0) b=0;
					}

//			image->set_pixel(Pos(x,y), Col(r,g,b));
					row[x*3] = r;
					row[x*3+1] = g;
					row[x*3+2] = b;
				}
				image->write_line(row);
				spinner->tick();
			}
			delete[] veg;
			delete[] red;
			delete[] green;
			delete[] blue;

			image->end_output();
			spinner->end();
		}

protected:

	typedef vector<AVHRRSample> AVHRRScanLine;
	typedef deque<AVHRRScanLine> AVHRRScanLines;
	AVHRRScanLines avhrr_scan_lines;

};

#endif
