////////////////////////////////////////////////////////////////////////////////
//
//	File: avhrrinterpolator.hpp
//
//	Provides: AVHRRInterpolator filter
//
//	Purpose: Use linear interpolation to convert NOAA L1b AVHRR GAC products
//			 to LAC format
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef avhrrinterpolator_defined
#define avhrrinterpolator_defined

#include "avhrrbase.hpp"

const double AVHRR_SCAN_LINE_TIME = 166.66667;

class AVHRRInterpolator :
	public AVHRRBase,
	public ProductProducer,
	public FileProducer
{
public:

	typedef NOAA_L1b_AVHRR_Data::Sample Sample;
	enum { OUTPUT_SIZE = 22528 };
	enum { INPUT_SAMPLES = 409 };
	enum { OUTPUT_SAMPLES = 2048 };

//	  enum { MILLISECONDS_PER_SCAN_LINE = 10 };

	//
	// constructor
	//

	AVHRRInterpolator(ProductProducer& _source, int user_bpp = -1,
					  const string& user_channels = "")
		: AVHRRBase(_source, user_bpp, user_channels)
		, output_record(NULL)
		, low_input(NULL)
		, high_input(NULL)
		, input_record_count(0)
		{
			// check number of samples (LAC=2048, GAC=409)
			if (source.get_product_header()->data_type_code==2)
			{
				// its GAC
			}
			else
			{
				error << "avhrrinterpolator only accepts AVHRR GAC input "
					  << "products\n";
				return;
			}

			// duplicate source product header
			//	The output is not a real header block, just enough
			//	to trick the AVHRRReader filter (?)
			output_header = reinterpret_cast<NOAA_L1b_AVHRR_Header*>
				(new byte[OUTPUT_SIZE]);
			memcpy(
				output_header,
				source.get_product_header(),
				source.get_record_size());

			// change it to LAC
			output_header->gen.data_type_code = NOAA_L1b_GEN_Header::DTC_LAC;

			// zero filler bytes
			for(int i=source.get_record_size(); i<OUTPUT_SIZE; i++)
			{
				byte* pos = reinterpret_cast<byte*>(output_header)+i;
				*((uint16_be*)pos) = OUTPUT_SIZE;
			}

			if (source.get_archive_header())
			{
				output_archive_header.reset(new NOAA_Archive_Header);

				// copy input archive header
				memcpy(
					(byte*)(output_archive_header.get()),
					source.get_archive_header(),
					sizeof(NOAA_Archive_Header));

				// change it to 5 channel
				strncpy(output_archive_header.get()->
						query.channel_select_flags, "YYYYY", 5);
				// 16 bps
				strncpy(output_archive_header.get()->
						query.sensor_data_word_size, "16", 2);
				// increase record length
				strncpy(output_archive_header.get()->
						data.record_size, " 22528", 6);
				// and increase number of scan lines
				char buffer[7];
				strncpy(
					buffer,
					source.get_archive_header()->data.number_of_records,
					6);
				buffer[8] = '\0';
				int existing_scanline_count = atoi(buffer);

//		sprintf(output_archive_header.get()->
//			data.number_of_records, "%6d",
//			existing_scanline_count*3);

				// fix the record count in the archive header
				sprintf(buffer,
						"%6d",
						((existing_scanline_count-2)*3)-1);
				strncpy(output_archive_header.get()->data.number_of_records,
						buffer,
						6);

				// patch the data set name
				output_archive_header.get()->query.dataset_name[4] = 'L';

				DEBUG("avhrrinterpolator:: source has "
					  << existing_scanline_count
					  << " outputting " << existing_scanline_count*3
					  << " lines\n");
			}
			else
			{
				DEBUG("avhrrinterpolator:: source has no archive header\n");
			}

			low_input = reinterpret_cast<NOAA_L1b_AVHRR_Data*>
				(new byte[source.get_record_size()]);
			high_input = reinterpret_cast<NOAA_L1b_AVHRR_Data*>
				(new byte[source.get_record_size()]);

			// prepare the intepolator by reading in one scan line

			if (!source.get_current_record())
			{
				error << "avhrrinterpolator:: no inputs\n";
			}
			memcpy(
				high_input,
				source.get_current_record(),
				source.get_record_size());

			// set initial times
			start_year = source.get_current_record()->scan_line_year;
			start_day = source.get_current_record()->scan_line_day_of_year;
			start_millisecond = source.get_current_record()->
				scan_line_millisecond_of_day;

//		cout << "initial times " << start_year << " / " << start_day
			// << " / " << start_millisecond << endl;

			// set product header scan line count ...
			output_header->count_of_data_records =
				((output_header->count_of_data_records-1)*3)+1;

			// ... and end time
//		unsigned int end_year;
//		unsigned int end_day;
//		unsigned int end_millisecond;
//		calculate_scan_line_time(
//		output_header->count_of_data_records-1,
//		end_year,
//		end_day,
//		end_millisecond);
//		output_header->gen.end_year = end_year;
//		output_header->gen.end_day_of_year = end_day;
//		output_header->gen.end_millisecond = end_millisecond;
//		output_header->gen.end_day += static_cast<unsigned int>(
//		AVHRR_SCAN_LINE_TIME*(output_header->count_of_data_records-1))
//		% (1000*60*60*24);

			// advance to line zero
			output_record_number = -1;
			advance();

			if (!low_input || !high_input)
			{
				error << "Input product too small\n";
				return;
			}
		}

	//
	// destructor
	//

	~AVHRRInterpolator(void)
		{
			delete[] reinterpret_cast<byte*>(output_header);
			delete[] reinterpret_cast<byte*>(low_input);
			delete[] reinterpret_cast<byte*>(high_input);
		}

	//
	// get_archive_header
	//

	NOAA_Archive_Header* get_archive_header(void)
		{
			return output_archive_header.get();
		}

	//
	// get_product_header
	//

	NOAA_L1b_GEN_Header* get_product_header(void)
		{
			return reinterpret_cast<NOAA_L1b_GEN_Header*>(output_header);
		}

	//
	// get_current_record
	//

	NOAA_L1b_GEN_Data* get_current_record(void) const
		{
			if (low_input && high_input)
			{
				return (NOAA_L1b_GEN_Data*)output_record;
			}
			else
			{
				return NULL;
			}
		}

	//
	// skip_records
	//

	void skip_records(unsigned int offset)
		{
			for(int i=0; i<offset; i++)
			{
				if (low_input && high_input)
				{
					advance();
				}
			}
		}

	//
	// advance
	//
	void advance(void)
		{
			output_record_number++;

			if (output_record_number%3==0)
			{
				// draw in new data

				// remember old low_input
				NOAA_L1b_AVHRR_Data* temp = low_input;

				// copy old high_input to new low_input
				low_input = high_input;

				// prepare to read in new high_input
				high_input = temp;

				source.skip_records(1);
				// check for eof
				if (!source.get_current_record())
				{
					delete[] (byte*)low_input;
					low_input = NULL;
					delete[] (byte*)high_input;
					high_input = NULL;
					// don't delete output_record as the client may be
					// holding a pointer to it
//			delete[] (byte*)output_record;
//			output_record = NULL;
					return;
				}

				input_record_count++;

				// read in new high_input
				// (nb. this isn't quite nessesary since we could call
				// source.get_current_record() as needed instead.
				// But this way makes the actual interpolation code a bit
				// cleaner
				memcpy((char*)high_input, (char*)source.get_current_record(),
					   source.get_record_size());
			}

			DEBUG("avhrrinterpolator calculating record "
				  << output_record_number << endl);

			if (output_record_number==0)
			{
				base_scan_line_number = low_input->gen.scan_line_number;
			}

//		if (output_record)
//		{
//		delete[] reinterpret_cast<byte*>(output_record);
//		}

			if (!output_record)
			{
				output_record = (NOAA_L1b_AVHRR_Data*)new byte[OUTPUT_SIZE];
			}

			// copy first 1264 bytes over with no changes (telemetry,
			// calibration etc)
			memcpy(output_record, low_input, 1264);

			for(int i=0; i<OUTPUT_SAMPLES; i++)
			{
				Sample output;

				int pos1 = min(i/5, 408);
				int pos2 = min(i/5+1, 408);

				Sample tl = get_product_sample(low_input, pos1);
				Sample tr = get_product_sample(low_input, pos2);
				Sample bl = get_product_sample(high_input, pos1);
				Sample br = get_product_sample(high_input, pos2);

				int xmod = i%5;
				int ymod = output_record_number%3;

				for(int j=0; j<5; j++)
				{
					output.channels[j] = (
						tl.channels[j]*(4-xmod)*(2-ymod) +
						tr.channels[j]*xmod*(2-ymod) +
						bl.channels[j]*(4-xmod)*ymod +
						br.channels[j]*xmod*ymod)/8;

				}

				uint16_be* pos = reinterpret_cast<uint16_be*>(
					reinterpret_cast<byte*>(output_record)+1264+i*10);

				*pos = output.channels[0];
				*(pos+1) = output.channels[1];
				*(pos+2) = output.channels[2];
				*(pos+3) = output.channels[3];
				*(pos+4) = output.channels[4];
			}

			// modify scan line number
			unsigned int new_scan_line =
				base_scan_line_number+output_record_number;
			output_record->gen.scan_line_number = new_scan_line;

			// modify scan line time
			unsigned int new_year;
			unsigned int new_day;
			unsigned int new_millisecond;
			calculate_scan_line_time(
				output_record_number,
				new_year,
				new_day,
				new_millisecond);
			output_record->gen.scan_line_year = new_year;
			output_record->gen.scan_line_day_of_year = new_day;
			output_record->gen.scan_line_millisecond_of_day = new_millisecond;

//		cout << "scan line (" << output_record_number << ") " << new_year
			// << " / " << new_day << " / " << new_millisecond << endl;

//		output_record_number = no;
//		return reinterpret_cast<NOAA_L1b_GEN_Data*>(output_record);
		}

	//
	// get_record_size
	//

	int get_record_size(void) const
		{
			return OUTPUT_SIZE; // we convert everything to LAC 16bpp 5
			// channels
		}

	//
	// produce_files
	//

	bool produce_files(FileConsumer& dest)
		{
			return false;
		}

	//
	// get_next_file
	//

	istream_type* get_next_file(void)
		{
			return NULL;
		}

	//
	// output_text_summary
	//

	void output_text_summary( ostream& os = cout )
		{
			os << "AVHRRInterpolator report:\n"
			   << "	   Input " << input_record_count << " records\n"
			   << "	   Output " << output_record_number << " records\n";

			AVHRRBase::output_source_summary(os);

			os << endl;
		}

	//
	// output_text_archive_header
	//

	void output_text_archive_header( ostream& os = cout )
		{
		}

	//
	// output_gen_header
	//

	void output_text_gen_header( ostream& os = cout )
		{
		}

	//
	// output_gen_text_record
	//

	void output_text_gen_record(int, ostream& os = cout )
		{
		}

	void calculate_scan_line_time(
		int scan_line,
		unsigned int& years,
		unsigned int& days,
		unsigned int& milliseconds)
		{
			const int MILLISECONDS_PER_DAY = 1000*60*60*24;
			const int DAYS_PER_YEAR = 365;

			// set the initial times
			years = start_year;
			days = start_day;
			milliseconds = start_millisecond;

			// add basic offset
			milliseconds += static_cast<int>(scan_line * AVHRR_SCAN_LINE_TIME);

			// correct days if theres an overflow
			days += static_cast<unsigned int>(milliseconds/
											  MILLISECONDS_PER_DAY);
			milliseconds %= MILLISECONDS_PER_DAY;

			// correct year if theres an overflow
			years += static_cast<unsigned int>(days/DAYS_PER_YEAR);
			days %= DAYS_PER_YEAR;
		}


private:

	auto_ptr<NOAA_Archive_Header> output_archive_header;

	NOAA_L1b_AVHRR_Header* output_header;

	NOAA_L1b_AVHRR_Data* output_record;
	NOAA_L1b_AVHRR_Data* low_input;
	NOAA_L1b_AVHRR_Data* high_input;
	int output_record_number;
	int input_record_count;

	unsigned int start_year;
	unsigned int start_day;
	unsigned int start_millisecond;

	// track the scan line number to use
	int base_scan_line_number;

	int bps;
	int channel_count;
	bool channels[5];
};

#endif
