////////////////////////////////////////////////////////////////////////////////
//
//	File: avhrrreader.hpp
//
//	Provides: AVHRRReader filter
//
//	Purpose: Convert NOAA AVHRR LAC L1b products to source packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef avhrrreader_defined
#define avhrrreader_defined

#include "ccsds.hpp"
#include "ccsdsproducer.hpp"
#include "config.hpp"
#include "ccsdsgeneratorbase.hpp"
#include "noaa.hpp"
#include "productproducer.hpp"
#include "image.hpp"
#include "imageproducer.hpp"
#include "avhrrbase.hpp"
#include "avhrrimagebase.hpp"

#define Config Config::AVHRR

class AVHRRReader :
	public AVHRRBase,
	public AVHRRImageBase,
	public CCSDSGeneratorBase<CCSDS_AVHRR>,
	public CCSDSProducer,
	public ImageProducer
{
public:

	AVHRRReader( ProductProducer& _source, const string& config_file = "",
				 int user_bps = -1, const string& user_channels = "")
		: AVHRRBase(_source, user_bps, user_channels)
		, record_count(0)
		{
			assert(NOAA_L1b_AVHRR_Data::SIZE == sizeof(NOAA_L1b_AVHRR_Data));

			if (!config_file.empty())
			{
				Config::file.read(config_file);
			}

			header = reinterpret_cast<NOAA_L1b_AVHRR_Header*>(
				source.get_product_header());
			if (!header)
			{
				return;
			}

			data = reinterpret_cast<NOAA_L1b_AVHRR_Data*>(
				source.get_current_record());


			// prepare for source packet output

//		ccsds_set_sequence_count(
//		Config::initial_sequence_count);

			apid_103_count = Config::initial_sequence_count_103;
			apid_104_count = Config::initial_sequence_count_104;

			if (Config::utc_ignore_input_time)
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);

				ccsds_modify_utc_base_time(data->gen);
			}
			else
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);
			}

			ccsds_set_obt_zero_time(
				data->gen);

			ccsds_apply_obt_offsets(
				Config::obt_coarse_offset,
				Config::obt_fine_offset,
				Config::obt_fixed_fine_time);

			// read initial times
			start_scan_line_number = data->gen.scan_line_number;
			start_year = data->gen.scan_line_year;
			start_day = data->gen.scan_line_day_of_year;
			start_millisecond = data->gen.scan_line_millisecond_of_day;

		}

	//
	// output_text_summary
	//

	void output_text_summary( ostream& os = cout )
		{
			if (!data || !header)
			{
				return;
			}

			string start_time = TimeConv::abs_year_day_millisecond_to_string(
				start_year,
				start_day,
				start_millisecond);

			string end_time = TimeConv::abs_year_day_millisecond_to_string(
				data->gen.scan_line_year,
				data->gen.scan_line_day_of_year,
				data->gen.scan_line_millisecond_of_day);

			os << "AVHRRReader report:\n"
			   << "	   " << record_count << " scan lines processed ("
			   << start_scan_line_number << "-"
			   << (start_scan_line_number+record_count-1) << ")\n"
			   << "	   Product start time " << start_time << endl
			   << "	   Product end time " << end_time << endl;

			AVHRRBase::output_source_summary(os);

			os << endl;
		}

	//
	// output_text_config
	//

	void output_text_config( ostream& os = cout )
		{
			os << "AVHRRReader configuration:\n"
//		   << "	   Scan line number as sequence count " <<
//		(Config::scan_line_number_as_sequence_count?"on":"off") << endl
//		   << "	   Initial sequence count "
//		   << Config::initial_sequence_count << endl
			   << "	   Initial sequence count 103 " <<
				(Config::initial_sequence_count_103) << endl
			   << "	   Initial sequence count 104 " <<
				(Config::initial_sequence_count_104) << endl
			   << "	   UTC days offset " << Config::utc_days_offset << endl
			   << "	   UTC milliseconds offset " <<
				Config::utc_milliseconds_offset << endl
			   << "	   UTC microseconds offset " <<
				Config::utc_microseconds_offset << endl
			   << "	   Ignore UTC time " <<
				(Config::utc_ignore_input_time?"on":"off") << endl
			   << "	   OBT coarse offset " << Config::obt_coarse_offset << endl
			   << "	   OBT fine offset " << Config::obt_fine_offset << endl
			   << "	   OBT fixed fine time " << Config::obt_fixed_fine_time
			   << endl << endl;
		}

	//
	// output_text_header
	//

	void output_text_header( ostream& os = cout )
		{
			source.output_text_archive_header(os);
			source.output_text_gen_header(os);

			os << "Channels active ch#1		 : "
			   << ((header->instrument_status&(1<<13))!=0) << endl
			   << "				   ch#2		 : "
			   << ((header->instrument_status&(1<<12))!=0) << endl
			   << "				   ch#3a	 : "
			   << ((header->instrument_status&(1<<11))!=0) << endl
			   << "				   ch#3b	 : "
			   << ((header->instrument_status&(1<<10))!=0) << endl
			   << "				   ch#4		 : "
			   << ((header->instrument_status&(1<<9))!=0) << endl
			   << "				   ch#5		 : "
			   << ((header->instrument_status&(1<<8))!=0) << endl
			   << "Count of data records	 : " << header->count_of_data_records
			   << endl
			   << "Count of good scan lines	 : " << header->count_of_good_scan_lines
			   << endl
			   << "Count of missing lines	 : "
			   << header->count_of_missing_scan_lines << endl
			   << "Count of data gaps		 : " <<header->count_of_data_gaps << endl;
		}

	//
	// output_text_record
	//

	void output_text_record( int record_no,
							 bool show_header = false,
							 ostream& os = cout)
		{
//		data = reinterpret_cast<NOAA_L1b_AVHRR_Data*>(
//		source.get_data_record(record_no));

			source.skip_records(record_no);
			if (!source.get_current_record())
			{
				os << "Cannot access record " << record_no << endl;
				return;
			}

			data = (NOAA_L1b_AVHRR_Data*)source.get_current_record();

			// Introduction

			if (show_header)
			{
				source.output_text_gen_record(record_no, os);
			}

			// Navigation

			os << "Navigation\n----------\n\n"
			   << "Computed yaw steering\n"
			   << "	   roll angle		: " << data->nav.computed_roll_angle << endl
			   << "	   pitch angle		: " << data->nav.computed_pitch_angle << endl
			   << "	   yaw angle		: " << data->nav.computed_yaw_angle << endl
			   << endl
			   << "Total applied correction\n"
			   << "	   roll angle		: " << data->nav.applied_roll_angle << endl
			   << "	   pitch angle		: " << data->nav.applied_pitch_angle << endl
			   << "	   yaw angle		: " << data->nav.applied_yaw_angle << endl
			   << endl
			   << "Earth locations (lat,lon)\n";

			for(int i=0; i<51; i++)
			{
				os << "	   " << ntofs(i,2,' ') << " (FOV " << ntofs(i*40+25, 4, ' ') << ") : "
				   << setiosflags(ios::fixed) << setw(5)
				   << (data->nav.earth_locations[i].latitude/10000.0) << " deg, "
				   << (data->nav.earth_locations[i].longitude/10000.0) << " deg" << endl;
			}

			os << endl;

			// Telemetry

			os << "Telemetry\n---------\n\n"
			   << "Ramp calibration ch#1 : "
			   << data->telemetry.ramp_calibration.channels[0] << endl
			   << "					ch#2 : "
			   << data->telemetry.ramp_calibration.channels[1] << endl
			   << "					ch#3 : "
			   << data->telemetry.ramp_calibration.channels[2] << endl
			   << "					ch#4 : "
			   << data->telemetry.ramp_calibration.channels[3] << endl
			   << "					ch#5 : "
			   << data->telemetry.ramp_calibration.channels[4] << endl << endl
			   << "				  PRT #1 : " << data->telemetry.prt[0] << endl
			   << "					  #2 : " << data->telemetry.prt[1] << endl
			   << "					  #3 : " << data->telemetry.prt[2] << endl
			   << endl
			   << "	   Patch Temperature : "
			   << data->telemetry.patch_temperature << endl
			   << endl;

			// Back Scan

			os << "Back Scan\n----------\n\n"
			   << "Sample ch#3 ch#4 ch#5\n";

			for(int i=0; i<10; i++)
			{
				os << "	   " << setiosflags(ios::fixed) << setw(1) << i << "  "
				   << setiosflags(ios::fixed) << setw(4) << setfill('0')
				   << data->back_scan[i].channel_3 << " "
				   << setiosflags(ios::fixed) << setw(4) << setfill('0')
				   << data->back_scan[i].channel_4 << " "
				   << setiosflags(ios::fixed) << setw(4) << setfill('0')
				   << data->back_scan[i].channel_5 << endl;
			}
			os << endl;

			// Space View

			os << "Space View\n----------\n\n"
			   << "Sample ch#1 ch#2 ch#3 ch#4 ch#5\n";

			for(int i=0; i<10; i++)
			{
				os << "		" << setiosflags(ios::fixed) << setw(1) << i;
				for(int j=0; j<5; j++)
				{
					os << " " << setiosflags(ios::fixed) << setw(4)
					   << setfill('0')
					   << data->space_data[i].channels[j];
				}
				os << endl;
			}
			os << endl;

			// Earth View

			os << "Earth view\n----------\n\n"
			   << "Sample ch#1 ch#2 ch#3 ch#4 ch#5	   "
			   << "Sample ch#1 ch#2 ch#3 ch#4 ch#5\n";

			// set output stream to force 4 char outputs, right aligned, z
			// ero padded
			os << setiosflags(ios::fixed) << setw(4);

			// write earth views
			for(int i=0; i<sample_count; i+=2)
			{
				os << setiosflags(ios::fixed) << setfill(' ')
				   << setw(4) << i << "	 ";

				AVHRRBase::AVHRRSample sample = get_product_sample(data, i);
				for(int j=0; j<5; j++)
				{
					os << " " << setiosflags(ios::fixed) << setfill('0')
					   << setw(4)
					   << sample.channels[j];
				}

				os << "		" << setiosflags(ios::fixed) << setfill(' ')
				   << setw(4) <<
					(i+1) << "	";

				sample = get_product_sample(data, i+1);
				for(int j=0; j<5; j++)
				{
					os << " " << setiosflags(ios::fixed) << setfill('0')
					   << setw(4)
					   << sample.channels[j];
				}

				os << endl;
			}

			// restore stream settings
			os << resetiosflags(ios::fixed);

		}

	//
	// get_next_ccsds
	//

	CCSDS* get_next_ccsds(void)
		{
			if (!source.get_current_record())
			{
				if (spinner.get())
				{
					spinner->end();
				}
				return NULL;
			}

			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			};
			spinner->tick();

			if (sample_count != 2048)
			{
				spinner->end();
				error << "Internal error: GAC input has not been "
					  << "interpolated\n";
				gerror = -1;
				return NULL;
			}

			// copy scan line number into sequence number if configured
//		if (Config::scan_line_number_as_sequence_count)
//		{
//		ccsds_set_sequence_count(data->gen.scan_line_number+
//			Config::initial_sequence_count);
//		}

			// prepare source packet
			ccsds_initialise();

			// set utc time
			ccsds_set_time(data->gen);

			// set application id
			APID apid;
//		if (header->instrument_status & 128 == 0)
			if ((data->gen.scan_line_bit_field&3) == 0)
			{
				apid = APID_AVHRR_3B;
				ccsds_set_sequence_count(apid_104_count);
				apid_104_count++;
			}
			else
			{
				apid = APID_AVHRR_3A;
				ccsds_set_sequence_count(apid_103_count);
				apid_103_count++;
			}
			ccsds_packet.primary_header.set_apid(apid);

			// prepare to write application data section
			app_write_ptr = ccsds_packet.payload.data;
			app_write_bit = 7;

			// write space view pixels
			for(int i=0; i<10; i++)
			{
				write_sample( data->space_data[i] );
			}

			// write ramp calibration
			write_sample( data->telemetry.ramp_calibration );

			// write earth view
			for(int i=0; i<2048; i++)
			{
//		NOAA_L1b_LAC_Data::Sample s;
//		for(int j=0; j<5; j++)
//		{
//			s.channels[j] = get_noaa_earth_view_word(i*5+j);
//		}
				write_sample(get_product_sample(data, i));
			}

			// write ir_target_temp
			AVHRRBase::AVHRRSample s;
			s.channels[0] = 0;
			s.channels[1] = 0;
			s.channels[2] = data->telemetry.prt[0];
			s.channels[3] = data->telemetry.prt[1];
			s.channels[4] = data->telemetry.prt[2];
			write_sample(s);

			// write patch_temp
			for(int i=0; i<5; i++)
			{
				s.channels[i] = data->telemetry.patch_temperature;
			}
			write_sample(s);

			// write ir_back_scan
			for(int i=0; i<10; i++)
			{
				AVHRRBase::AVHRRSample s;
				s.channels[0] = 0;
				s.channels[1] = 0;
				s.channels[2] = data->back_scan[i].channel_3;
				s.channels[3] = data->back_scan[i].channel_4;
				s.channels[4] = data->back_scan[i].channel_5;

				write_sample(s);
			}

			write_bit(0);
			write_bit(0);

			source.skip_records(1);

			if (source.get_current_record())
			{
				data = (NOAA_L1b_AVHRR_Data*)source.get_current_record();
			}
			record_count++;

			return ccsds_finalise();
		}

	//
	// write_sample
	//

	void write_sample(const AVHRRBase::AVHRRSample& s)
		{
			for(int i=0; i<5; i++)
			{
				// nb actualy the and is not needed
				write_10_bits( s.channels[i] & 1023 );
			}
		}

	//
	// write_10_bits
	//	writes to ccsds packet
	//

	void write_10_bits(unsigned int s)
		{
			for(int i=0; i<10; i++)
			{
				write_bit( (s & (1<<9))>0 );
				s <<= 1;
			}
		}

	//
	// write_bit
	//	writes to ccsds packet
	//

	void write_bit(bool s)
		{
			if (s)
			{
				(*app_write_ptr) |= (1<<app_write_bit);
			}

			if (app_write_bit > 0)
			{
				app_write_bit--;
			}
			else
			{
				app_write_bit = 7;
				app_write_ptr++;
			}
		}

	//
	// get_image_count
	//

	int get_image_count(void)
		{
			if (!header)
			{
				return 0;
			}
			else
			{
				return AVHRRImageBase::get_image_count();
			}
		}

	//
	// write_image
	//

	void write_image(int number, Image* image)
		{
			AVHRRImageBase::write_image(number, image);
		}

	//
	// prepare_images
	//

	void prepare_images(void)
		{
			spinner.reset(new Spinner("Reading in image data"));

			int y=0;
			while(source.get_current_record())
			{
				data = (NOAA_L1b_AVHRR_Data*)source.get_current_record();

				// initialise new scan line
				avhrr_scan_lines.push_back(AVHRRScanLine());
				AVHRRScanLine& line = avhrr_scan_lines.back();
				line.resize(sample_count);

				// read line
				for(int x=0; x<sample_count; x++)
				{
					line[x] = get_product_sample(data, x);
				}

				y++;
				source.skip_records(1);
				spinner->tick();
				record_count++;
			}
			spinner->end();
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:

	int apid_103_count;
	int apid_104_count;

	NOAA_L1b_AVHRR_Header* header;
	NOAA_L1b_AVHRR_Data* data;

	byte* app_write_ptr;
	int app_write_bit;

	unsigned int start_scan_line_number;
	unsigned int start_year;
	unsigned int start_day;
	unsigned int start_millisecond;
	int record_count;

};

#undef Config

#endif
