////////////////////////////////////////////////////////////////////////////////
//
//	File: buffer.hpp
//
//	Provides: misc structs
//
//	Purpose: Code for representing buffers
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef buffer_defined
#define buffer_defined


// pure binary, non instrument specific packet

class Buffer
{
public:
	Buffer(void)
		: data(NULL)
		, owner(false)
		, length(0)
		{
			//cout << "Buffer default constructor\n";
		}

	// Warning: dangerous constructor.
	//get_length() and copy operations won't work
	// until the Packet Primary Header 'size' field has
	// been set
	Buffer(long _length)
		: data(new byte[_length]),
		  owner(true),
		  length(_length)
		{
			//cout << "Buffer length constructor\n";
		}


	Buffer(byte* source, int size, bool deep_copy)
		{
			length = size;
			if (deep_copy)
			{
				data = new byte[size];
				memcpy(data, source, size);
				owner = true;
			}
			else
			{
				data = source;
				owner = false;
			}
		}

	Buffer(const Buffer& other)
		{
			// deep copy, owns memory
			if (other.get_data())
			{
				long len = other.get_length();
				data = new byte[len];
				memcpy(data, other.get_data(), len);
				owner = true;
			}
			else
			{
				data = NULL;
				owner = false;
			}
		}

	Buffer(const Buffer* other)
		{
			//cout << "ccsads copy constructor\n";
			// deep copy, owns memory
			long len = other->get_length();
			data = new byte[len];
			memcpy(data, other->get_data(), len);
			owner = true;
		}

	~Buffer(void)
		{
			if (owner)
			{
				//cout << "Buffer destructor deleting array\n";
				delete[] data;
			}
			else
			{
				//cout << "Buffer destructor not deleting array\n";
			}
		}

	Buffer& operator = (const Buffer& other)
		{
			// deep copy, take ownership
			long len = other.get_length();
			data = new byte[len];
			memcpy(data, other.get_data(), len);
			owner = true;
			return *this;
		}


//	   byte* get_data(void) const
//	{
//		return data;
//	}


//	   void set_data(byte new_data, long offset)
//	{
//		byte* data_ptr = reinterpret_cast<byte*>(&data + offset);
//		*data_ptr = new_data;
//	}

	long get_length(void) const
		{
			return length;
		}

	byte* get_data(void) const
		{
			return &(data[0]);
		}


	void set_byte(byte new_data, int offset)
		{
			get_data()[offset] = new_data;
		}

	bool set_bit(int bit, int offset)
		{
			bool result = false;
			int byte_offset = offset/8;
			int bit_offset = offset - (byte_offset*8);

			if (bit_offset > 0 && bit_offset < 8)
			{
				int a = 0;
				int b = get_data()[byte_offset];

				a |= ((b>>(8-bit_offset))&((1<<bit_offset)-1))<<(8-bit_offset);
				if ( ((b>>(7-bit_offset))&1) != (bit&1) )
				{
					result = true;
				}
				a |= (bit&1)<<(7-bit_offset);
				a |= b&((1<<(7-bit_offset))-1);
				get_data()[byte_offset] = a;
			}
			else
			{
				int a = 0;
				int b = get_data()[byte_offset];
				if ( ((b>>7)&1) != (bit&1) )
				{
					result = true;
				}
				a |= (bit&1)<<7;
				a |= b&((1<<7)-1);
				get_data()[byte_offset] = a;
			}

			return result;
		}

	void flip_bit(int offset)
		{
			int byte_offset = offset/8;
			int bit_offset = offset - (byte_offset*8);

			if (bit_offset > 0 && bit_offset < 8)
			{
				int a = 0;
				int b = get_data()[byte_offset];

				a |= ((b>>(8-bit_offset))&((1<<bit_offset)-1))<<(8-bit_offset);
				if ( static_cast<int>((b>>(7-bit_offset))&1) == 0 )
				{
					a |= 1<<(7-bit_offset);
				}
				a |= b&((1<<(7-bit_offset))-1);
				get_data()[byte_offset] = a;
			}
			else
			{
				int a = 0;
				int b = get_data()[byte_offset];
				if ( static_cast<int>((b>>7)&1) == 0 )
				{
					a |= 1<<7;
				}
				a |= b&((1<<7)-1);
				get_data()[byte_offset] = a;
			}
		}


private:
	byte* data;
	bool owner;
	long length;
};


#endif
