////////////////////////////////////////////////////////////////////////////////
//
// File: buffer_corrupt_filter.cpp
//
// Provides: buffer_corrupt_filter executable
//
// Purpose: Enables the modification/corruption of any input file/buffer
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "buffer.hpp"
#include "filereader.hpp"
#include "bufferreader.hpp"
#include "readfromstdin.hpp"
#include "buffercorruptfilter.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"
#include "textsummarydisplayer.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "buffer_corrupt_filter";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	string in_file;
	string out_file;
	int buffer_size = -1;
	bool verbose = true;

	int skip = -1;
	int count = -1; 
	Info byte_info;
	int byte_offset_start = -1;
	int byte_offset_end = -1;
	string byte_increment = "both";

	int bit_offset_start = -1;
	int bit_offset_end = -1;
	int bit_width_val = -1;
	string bit_width_key = "";
	int bit_offset_start_relative = -2147483647;
	int bit_offset_end_relative = -2147483647;
	Info bit_info;
	int max_corruptions = -1;

	int input_ctr = 0;


	enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_SIZE, ARGV_SKIP, ARGV_CNT, 
		   ARGV_BYT_OFFST, ARGV_BYT_VAL, ARGV_BIT_OFFST, 
		   ARGV_BIT_VAL, ARGV_INC , ARGV_WIDTH , ARGV_MAX } state = ARGV_IN;

	string help_msg = 
		string("\nThe buffer_corrupt_filter enables the modification and corruption of any file/buffer.\n\n")
		+ "Usage:\n\n" + APPNAME
		+ " [-i|--in <filename>]"
		+ " [-o|--out <filename>]"
		+ " [-z|--size <N>]"
		+ " [-s|--skip <N>]"
		+ " [-c|--count <N>]"
		+ " [--for-byte-offset <N1|N1 N2>]"
		+ " [--set-byte <random|N1|N1 N2>]"
		+ " [--for-bit-offset <N1|N1 N2>]"
		+ " [--set-bit <random|N1|N1 N2|flip>]"
		+ " [--set-bit-width <N|random|N random>]"
		+ " [--set-increment <row|column>]"
		+ " [--set-max-corruptions <N>]"
		+ " [-m|--simple]"
		+ " [-h|--help]"
		+ " [-v|--version]\n\n"
		+ "Note: To avoid unnecessary typing, most of the options above have short hand names"
		+ " by which they can be refereed to (-i, -o, -z, -s, -c, -t, -m, -h and -v).\n\n"
		+ "where:\n\n"
		+ "--in <filename(s)> : Allows an input file to be specified.\n\n"
		+ "--out <filename> : Allows the name of an output file to be specified.\n\n"
		+ "--size <N> : Allows users to split a file into smaller packets. If left"
		+ " undefined, a default size of 1024 bytes is used. If the default size of 1024"
		+ " bytes is found to be larger than the size of the input file, the total size of"
		+ " the input file will be used instead.\n\n"
		+ "--skip <N> : Allows N packets to be skipped before applying any corruptions.\n\n"
		+ "--count <N> : Allows corruptions to be applied to only N packets starting at"
		+ " the first packet or after skipping a number of packets if the --skip option is used.\n\n"
		+ "--for-byte-offset <N1|N1 N2> : Allows corruptions to be introduced"
		+ " to a single byte at byte offset N1 or to a range of bytes"
		+ " starting at byte offset N1 and ending at byte offset N2."
		+ " Note that if the corruption is to be applied to the first byte,"
		+ " the byte offset must be set to zero. Hence, the byte offset"
		+ " must be greater or equal to zero and of course less than the"
		+ " packet length.\n\n"
		+ "--set-byte <random|N1|N1 N2> : Allows a single byte per packet or a"
		+ " range of bytes per packet to be set to random values, to be"
		+ " set to a single value N1, or to be filled with a number"
		+ " pattern starting with a value N1 and incremented by a value N2."
		+ " Note that N1 and N2 can not be set to anything greater than 255."
		+ " While N1 can not be less than zero, the increment N2 can be"
		+ " assigned a negative value.\n\n"
		+ "--for-bit-offset <N1|N1 N2> : Allows corruptions to be introduced"
		+ " to a single bit at bit offset N1 or to a range of bits"
		+ " starting at bit offset N1 and ending at bit offset N2."
		+ " Note that if the corruption is to be applied to the first bit,"
		+ " the bit offset must be set to zero. Hence, the bit offset"
		+ " must be greater or equal to zero and of course less than the"
		+ " packet length. Further more, if the numbers N1 or N2 are preceded"
		+ " with a '-/+' symbol, they will be assumed to be relative offsets. This"
		+ " allows users to combine both the --for-byte-offset and the"
		+ " --for-bit-offset options in the following way:\n\n"
		+ "	   If --for-byte-offset N1 --for-bit-offset = +N2\n"
		+ "	   Then start offset = (N1*8)+N2 bits\n\n"
		+ "	   If --for-byte-offset N1 and --for-bit-offset +N2 N3\n"
		+ "	   Then start offset = (N1*8)+N2 bits and end offset = N3 bits\n\n"
		+ "	   If --for-byte-offset N1 and --for-bit-offset +N2 +N3\n"
		+ "	   Then start offset = (N1*8)+N2 bits and end offset = (N1*8)+N2+N3 bits\n\n"
		+ "	   If --for-byte-offset N1 N2 and --for-bit-offset +N3 +N4\n"
		+ "	   Then start offset = (N1*8)+N3 bits and end offset = (N2*8)+N4 bits\n\n"
		+ "	   If --for-byte-offset N1 N2 and --for-bit-offset +N3\n"
		+ "	   Then start offset = (N1*8)+N3 bits and end offset = N2*8 bits\n\n"
		+ "--set-bit <random|N1|N1 N2|flip> : Allows a single bit per packet or a"
		+ " range of bits per packet to be set to random values, to be"
		+ " set to a single value N1(0/1), or to be filled with a number"
		+ " pattern starting with a value N1(0/1) and incremented by a value N2(0/1)."
		+ " Note that N1 and N2 can not be set to anything other than 0 and 1"
		+ " and hence, only patterns such as 01010101 or 10101010 are possible. The"
		+ " final flip option can be used to change single or a range of bits from 0 to 1"
		+ " or vice versa.\n\n"
		+ "--set-bit-width <N|random|N random> : The --set-bit-width option will only work and can"
		+ " only be set if the --for-byte/bit-offset options are used to define a range of"
		+ " bytes/bits to corrupt. The width option can be set to a number N, the key word random"
		+ " or to both random and a numeric value at the same time. Assuming we want to modify bits"
		+ " starting at bit offset N1 and ending at bit offset N2:\n\n"
		+ "	   If --set-bit-width N then the following bits will be modified\n"
		+ "				N1, N1+N, N1+2N, N1+3N ... N1+xN\n"
		+ "	   as long as N1+xN is less than or equal to N2\n\n"
		+ "	   If --set-bit-width random then the following bits will be modified\n"
		+ "				N1, N1+R1, N1+(R1+R2), N1+(R1+R2+R3), N1+(R1+R2+R3+...+Rx)\n"
		+ "	   as long as N1+(R1+R2+R3+...+Rx) is less than or equal to N2. Note that\n"
		+ "	   R1, R2, Rx represents different random numbers determined by the code.\n\n"
		+ "	   If --set-bit-width random N or --set-bit-width N random, then the value\n"
		+ "	   of N is simply used by the code to place a maximum limit on the random\n"
		+ "	   numbers so that they do not exceed the value of N.\n\n"
		+ "Note that the width cannot be set to a value greater than the difference"
		+ " between offsets N1 and N2.\n\n"
		+ "--set-increment <column|row> : This option only becomes"
		+ " relevant if one is attempting to assign a number pattern to a"
		+ " range of bits/bytes per packet. For example a user might want to"
		+ " corrupt byte offsets 101 to 105 (--for-byte-offset 101 105)"
		+ " with a number pattern: 1, 3, 5, 7, 9, ... (--set-byte 1 2)."
		+ " If the --set-increment option is left undefined, the"
		+ " resulting corruption per packet will be as follows:\n\n"
		+ "	  Byte Offset  101	102	 103  104  105\n"
		+ "	  Packet 1	 1	  3	   5	7	 9\n"
		+ "	  Packet 2	 3	  5	   7	9	 11\n"
		+ "	  Packet 3	 5	  7	   9	11	 13\n"
		+ "	  Packet 4	 7	  9	   11	13	 15\n"
		+ "	  Packet 5	 9	  11   13	15	 17\n"
		+ "	  Packet 6	 11	  13   15	17	 19\n\n"
		+ "Note that the pattern has been increment over both column"
		+ " (Byte Offset) and row (Packet). Hence, if the"
		+ " --set-increment option is left undefined, the default"
		+ " option is to apply the pattern over both columns and rows."
		+ " If however, the --set-increment is set to column, the"
		+ " pattern will only be incremented over the different byte"
		+ " offsets and will result into the following corruption:\n\n"
		+ "	  Byte Offset  101	102	 103  104  105\n"
		+ "	  Packet 1	 1	  3	   5	7	 9\n"
		+ "	  Packet 2	 1	  3	   5	7	 9\n"
		+ "	  Packet 3	 1	  3	   5	7	 9\n"
		+ "	  Packet 4	 1	  3	   5	7	 9\n"
		+ "	  Packet 5	 1	  3	   5	7	 9\n"
		+ "	  Packet 6	 1	  3	   5	7	 9\n\n"
		+ "Similarly, if the --set-increment option is set to row,"
		+ " the pattern will only be incremented over the different packets"
		+ " and will result into the following corruption:\n\n"
		+ "	  Byte Offset  101	102	 103  104  105\n"
		+ "	  Packet 1	 1	  1	   1	1	 1\n"
		+ "	  Packet 2	 3	  3	   3	3	 3\n"
		+ "	  Packet 3	 5	  5	   5	5	 5\n"
		+ "	  Packet 4	 7	  7	   7	7	 7\n"
		+ "	  Packet 5	 9	  9	   9	9	 9\n"
		+ "	  Packet 6	   11	11	 11	  11   11\n\n"
		+ "Note that --set-increment will have a similar effect on the"
		+ " --set-bit option.\n\n"
		+ "--set-max-corruptions <N> :	Allows a limit to be set on the maximum number of"
		+ " corruptions that a user might like to introduce.\n\n"
		+ "--simple : Use simple animations while processing.\n\n"
		+ "--help : Shows usage information.\n\n"
		+ "--version : Shows version information.\n\n";



	for(int x=1; x<argc; x++)
	{	
		string arg(argv[x]);

		if (arg == "--version" || arg == "-v")
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__ 
				 << endl << endl;
			return 0;
		}
		else if (arg == "--help" || arg == "-h")
		{
			cout << help_msg;
			return 0;
		}
		else if (arg == "--in" || arg == "-i")
		{
			state = ARGV_IN;
		}
		else if (arg == "--out" || arg == "-o")
		{
			state = ARGV_OUT;
		}
		else if (arg == "--size" || arg == "-z")
		{
			state = ARGV_SIZE;
		}
		else if (arg == "--quiet")
		{
			verbose = false;
		}
		else if (arg == "--debug")
		{
			_debug = &cout;
		}
		else if (arg == "-m" || arg == "--simple")
		{
			Spinner::use_simple_spinner();
		}
		else if (arg == "--skip" || arg == "-s")
		{
			state = ARGV_SKIP;
		}
		else if (arg == "--count" || arg == "-c")
		{
			state = ARGV_CNT;
		}
		else if (arg == "--for-byte" || arg == "--filter-byte" ||
				 arg == "--for-bytes" || arg == "--filter-bytes" ||
				 arg == "--for-byte-offset" || arg == "--filter-byte-offset")

		{
			input_ctr = 0;
			state = ARGV_BYT_OFFST;
		}
		else if (arg == "--set-byte" || arg == "--set-bytes")
		{
			input_ctr = 0;
			state = ARGV_BYT_VAL;
		}
		else if (arg == "--for-bit" || arg == "--filter-bit" ||
				 arg == "--for-bits" || arg == "--filter-bits" ||
				 arg == "--for-bit-offset" || arg == "--filter-bit-offset")

		{
			input_ctr = 0;
			state = ARGV_BIT_OFFST;
		}
		else if (arg == "--set-bit" || arg == "--set-bits")
		{
			input_ctr = 0;
			state = ARGV_BIT_VAL;
		}
		else if (arg == "--set-increment" || arg == "--set-inc" || arg == "-inc")
		{
			state = ARGV_INC;
		}
		else if (arg == "--set-bit-width" || arg == "--set-width" || arg == "-width")
		{
			input_ctr = 0;
			state = ARGV_WIDTH;
		}
		else if (arg == "--set-max-corruptions" || arg == "--set-max" || arg == "-max")
		{
			state = ARGV_MAX;
		}
		else
		{
			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << arg << "'\n";
				return -1;

			case ARGV_IN:
				if (!in_file.empty())
				{
					error << "Only one input file can be specified\n";
					return -1;
				}
				else
				{
					in_file = arg;
					//state = ARGV_NONE;
				}
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file can be specified\n";
					return -1;
				}
				else
				{
					out_file = arg;
					//state = ARGV_NONE;
				}
				break;

			case ARGV_SIZE:
				if (buffer_size != -1)
				{
					error << "Buffer size can only accept one input.\n";
					return -1;
				}
				else
				{
					buffer_size = lexical_cast<int>(arg);
					state = ARGV_NONE;
				}
				break;

			case ARGV_SKIP:
				if (skip != -1)
				{
					error << "The option --skip only accepts one numeric input\n";
					return -1;
				}
				else
				{		
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n";
						return -1;
					}
					skip = lexical_cast<int>(arg);
					state = ARGV_NONE;
				}
				break;

			case ARGV_CNT:
				if (count != -1)
				{
					error << "The option --count only accepts one numeric input\n";
					return -1;
				}
				else
				{		
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n";
						return -1;
					}
					count = lexical_cast<int>(arg);
					state = ARGV_NONE;
				}
				break;

			case ARGV_BYT_VAL:
				if (input_ctr > 1)
				{		
					error << "Parameter not recognised '" << arg << "'\n"
						  << "The option --set-byte only accepts the inputs "
						  << "random/single number/two numbers separated by a space. "
						  << "For example:\n  --set-byte random"
						  << "\n  --set-byte 3"
						  << "\n  --set-byte 2 1\n";
					return -1;
				}
				else if (arg == "random" && byte_info.key == Info::OFF && input_ctr < 2)
				{
					byte_info.key = Info::RANDOM;
					input_ctr++;
					state = ARGV_NONE;
				}
				else
				{	 
					if (byte_info.start == -1 && byte_info.key != Info::RANDOM && input_ctr < 2)
					{
						if (arg.find_first_not_of("0123456789")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-byte only accepts the inputs "
								  << "random/single number/two numbers separated by a space. "
								  << "For example:\n  --set-byte random"
								  << "\n  --set-byte 3"
								  << "\n  --set-byte 2 1\n";
							return -1;
						}
						byte_info.start = lexical_cast<int>(arg);
						if (byte_info.start > 255)
						{
							error << "A byte can not be set to anything greater than 255\n";
							return -1;
						}
						byte_info.key = Info::SINGLE;
						input_ctr++;
					}
					else if (byte_info.increment == 0 && byte_info.start != -1 
							 && byte_info.key != Info::RANDOM && input_ctr < 2)
					{
						if (arg.find_first_not_of("-0123456789")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-byte only accepts the inputs "
								  << "random/single number/two numbers separated by a space. "
								  << "For example:\n  --set-byte random"
								  << "\n  --set-byte 3"
								  << "\n  --set-byte 2 1\n";
							return -1;
						}
						byte_info.increment = lexical_cast<int>(arg);
						if (byte_info.increment > 255)
						{
							error << "A byte can not be set to anything greater than 255\n";
							return -1;
						}
						byte_info.key = Info::INCREMENT;
						input_ctr++;
					}
					else
					{
						error << "The option --set-byte only accepts the inputs "
							  << "random/single number/two numbers separated by a space. "
							  << "For example:\n  --set-byte random"
							  << "\n  --set-byte 3"
							  << "\n  --set-byte 2 1\n";
						return -1; 
					}
				}
				break;

			case ARGV_BYT_OFFST:  
				if (input_ctr > 1)
				{		
					error << "Parameter not recognised '" << arg << "'\n"
						  << "The option --for-byte-offset only accepts the inputs "
						  << "single number/two numbers separated by a space. "
						  << "For example:\n  --for-byte-offset 28"
						  << "\n  --for-byte-offset 28 48\n";
					return -1;
				}
				else if (byte_offset_start == -1 && input_ctr < 2)
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --for-byte-offset only accepts the inputs "
							  << "single number/two numbers separated by a space. "
							  << "For example:\n  --for-byte-offset 28"
							  << "\n  --for-byte-offset 28 48\n";
						return -1;
					}
					byte_offset_start = lexical_cast<int>(arg);
					input_ctr++;
				}
				else if (byte_offset_end == -1 && byte_offset_start != -1 && input_ctr < 2)
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --for-byte-offset only accepts the inputs "
							  << "single number/two numbers separated by a space. "
							  << "For example:\n  --for-byte-offset 28"
							  << "\n  --for-byte-offset 28 48\n";
						return -1;
					}
					byte_offset_end = lexical_cast<int>(arg);
					input_ctr++;
				}
				else
				{
					error << "The option --for-byte-offset only accepts the inputs "
						  << "single number/two numbers separated by a space. "
						  << "For example:\n  --for-byte-offset 28"
						  << "\n  --for-byte-offset 28 48\n";
					return -1;
				}
				break;

		
			case ARGV_BIT_OFFST:
				if (input_ctr > 1)
				{		
					error << "Parameter not recognised '" << arg << "'\n"
						  << "The option --for-bit-offset only accepts the inputs "
						  << "N/N1 N2. "
						  << "For example:\n  --for-bit-offset 28"
						  << "\n  --for-bit-offset 28 48\n";
					return -1;
				}
				else if (arg.find_first_not_of("-+0123456789")!=string::npos)
				{
					error << "Unacceptable input '" << arg << "'\n"
						  << "The option --for-bit-offset only accepts the inputs "
						  << "N/N1 N2. "
						  << "For example:\n  --for-bit-offset 28"
						  << "\n  --for-bit-offset 28 48\n";
					return -1;
				}		
				else if (arg[0]=='+' || arg[0]=='-')
				{//Relative Offset
			
					int number = lexical_cast<int>(arg.substr(1));
					if (arg[0]=='-')
					{
						number = number * -1;
					}
					if (bit_offset_start_relative == -2147483647 && bit_offset_start == -1 && input_ctr < 2)
					{
						bit_offset_start_relative = number;
						input_ctr++;
					}
					else if (bit_offset_end_relative == -2147483647 && bit_offset_end == -1 && input_ctr < 2)
					{
						bit_offset_end_relative = number;
						input_ctr++;
					}
					else
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --for-bit-offset only accepts two inputs "
							  << "to mark the start and end of the corruptions\n";
						return -1;
					}
				}
				else
				{//Offset

					int number = lexical_cast<int>(arg);
					if (bit_offset_start == -1 && bit_offset_start_relative == -2147483647 && input_ctr < 2)
					{
						bit_offset_start = number;
						input_ctr++;
					}
					else if (bit_offset_end == -1 && bit_offset_end_relative == -2147483647 && input_ctr < 2)
					{
						bit_offset_end = number;
						input_ctr++;
					}
					else
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --for-bit-offset only accepts two inputs "
							  << "to mark the start and end of the corruptions.\n";
						return -1;
					}
				}
				break;

			case ARGV_BIT_VAL:
				if (input_ctr > 1)
				{		
					error << "Parameter not recognised '" << arg << "'\n"
						  << "The option --set-bit only accepts the inputs "
						  << "random/single number/two numbers separated by a space/flip. "
						  << "For example:\n  --set-bit random"
						  << "\n  --set-bit flip"
						  << "\n  --set-bit 1 "
						  << "\n  --set-bit 0 1\n";
					return -1;
				}
				else if (arg == "random" && bit_info.key == Info::OFF && input_ctr < 2)
				{
					bit_info.key = Info::RANDOM;
					input_ctr++;
					state = ARGV_NONE;
				}
				else if (arg == "flip" && bit_info.key == Info::OFF && input_ctr < 2) 
				{
					bit_info.key = Info::FLIP;
					input_ctr++;
					state = ARGV_NONE;
				}		
				else
				{	 
					if (bit_info.start == -1 && bit_info.key != Info::RANDOM 
						&& bit_info.key != Info::FLIP && input_ctr < 2)
					{
						if (arg.find_first_not_of("01")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-bit only accepts the inputs "
								  << "random/single number/two numbers separated by a space/flip. "
								  << "For example:\n  --set-bit random"
								  << "\n  --set-bit flip"
								  << "\n  --set-bit 1 "
								  << "\n  --set-bit 0 1\n";
							return -1;
						}
						bit_info.start = lexical_cast<int>(arg);
						if (bit_info.start > 1)
						{
							error << "A bit can only be set to 0 or 1\n";
							return -1;
						}
						bit_info.key = Info::SINGLE;
						input_ctr++;
					}
					else if (bit_info.increment == 0 && bit_info.start != -1 
							 && bit_info.key != Info::RANDOM 
							 &&	 bit_info.key != Info::FLIP && input_ctr < 2)
					{
						if (arg.find_first_not_of("-01")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-bit only accepts the inputs "
								  << "random/single number/two numbers separated by a space/flip. "
								  << "For example:\n  --set-byte random"
								  << "\n  --set-bit flip"
								  << "\n  --set-bit 3"
								  << "\n  --set-bit 2 1\n";
							return -1;
						}
						bit_info.increment = lexical_cast<int>(arg);
						if (bit_info.increment > 1)
						{
							error << "A bit can only be set to 0 or 1\n";
							return -1;
						}
						bit_info.key = Info::INCREMENT;
						input_ctr++;
					}
					else
					{
						error << "The option --set-bit only accepts the inputs "
							  << "random/single number/two numbers separated by a space/flip. "
							  << "For example:\n  --set-bit random"
							  << "\n  --set-bit flip"
							  << "\n  --set-bit 3"
							  << "\n  --set-bit 2 1\n";
						return -1; 
					}
				}
				break;

			case ARGV_INC:	
				if (arg == "column" || arg == "col" && byte_increment == "both")
				{
					byte_increment = "column";
					state = ARGV_NONE;
				}
				else if (arg == "row" && byte_increment == "both")
				{
					byte_increment = "row";
					state = ARGV_NONE;
				}
				else
				{				
					error << "Unacceptable input '" << arg << "'\n"
						  << "The option --set-increment only accepts the inputs "
						  << "column or row. For example:\n	 --set-increment column"
						  << "\n  --set-increment row\n";
					return -1;
				}
				break;

			case ARGV_WIDTH:
				if (input_ctr > 1)
				{		
					error << "Parameter not recognised '" << arg << endl
						  << "The option --set-bit-width only accepts the an input "
						  << "number or an input number and keyword. The only accepted "
						  << "keyword is random or ran. For example:\n	--set-bit-width 5 "
						  << "\n  --set-increment 5 random"
						  << "\n  --set-increment random 5\n";
					return -1;
				}
				if (arg == "random" && bit_width_key == "" && input_ctr < 2)
				{
					bit_width_key = "random";
					input_ctr++;
				}
				else if (arg == "ran" && bit_width_key == "" && input_ctr < 2)
				{
					bit_width_key = "random";
					input_ctr++;
				}
				else
				{
					if (bit_width_val == -1 && input_ctr < 2)
					{
						if (arg.find_first_not_of("0123456789")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-bit-width only accepts the an input "
								  << "number or an input number and keyword. The only accepted "
								  << "keyword is random or ran. For example:\n	--set-bit-width 5 "
								  << "\n  --set-increment 5 random"
								  << "\n  --set-increment random 5\n";
							return -1;
						}
						bit_width_val = lexical_cast<int>(arg);
						input_ctr++;
					}
					else
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --set-bit-width only accepts the an input "
							  << "number or an input number and keyword. The only accepted "
							  << "keyword is random or ran. For example:\n	--set-bit-width 5 "
							  << "\n  --set-increment 5 random"
							  << "\n  --set-increment random 5\n";
						return -1;
					}
				}
				break;

			case ARGV_MAX:
				if (max_corruptions != -1)
				{
					error << "The option --set-max-corruptions only accepts one numeric input.\n";
					return -1;
				}
				else
				{		
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n";
						return -1;
					}
					max_corruptions = lexical_cast<int>(arg);
					state = ARGV_NONE;
				}
				break;

			}
		}
	}
	
	if (buffer_size < 1)
	{
		buffer_size = 1024;
	}

	if (in_file.empty())
	{	
		error << "An input file must be specified\n";
		return -1;
	}

	if (out_file.empty())
	{
		error << "An output file must be specified\n";
		return -1;
	}

	if (!verbose)
	{
		spinner->restream(&onull);
	}

	FileProducer* first_source;
	first_source = new FileReader(in_file);

	BufferReader buffer(*first_source, buffer_size);
	BufferCorruptFilter corrupt(buffer, buffer_size, skip, count, byte_info, bit_info,
								byte_offset_start, byte_offset_end, byte_increment,
								bit_offset_start, bit_offset_end, bit_offset_start_relative, 
								bit_offset_end_relative, bit_width_val, bit_width_key,
								max_corruptions);

	if (!corrupt.is_valid())
	{
		error << "Exit...\n";
		return -1;
	}

	FileConsumer* writer;
	if (out_file.empty())
	{
		writer = new WriteToStdout(&corrupt);
	}
	else
	{
		writer = new SingleFileWriter(&corrupt, out_file);
	}

	if (verbose && !gerror)
	{
		if (!corrupt.is_valid())
		{
			error << "\n*** WARNING: The output produced might be"
				  << " incomplete as errors were detected"
				  << " during the processing.\n\n";
			return -1;
		}
		else
		{
			first_source->output_text_summary();
			TextSummaryDisplayer d2(buffer);
			TextSummaryDisplayer d3(corrupt);
			writer->output_text_summary();
		}
	}

	delete first_source;
	delete writer;

	return 0;
}

