////////////////////////////////////////////////////////////////////////////////
//
//	File: buffercorruptfilter.hpp
//
//	Provides: BufferCorruptFilter filter
//
//	Purpose:  Enables the corruption of any input file
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef buffercorruptfilter_defined
#define buffercorruptfilter_defined

#include "bufferproducer.hpp"


struct Info
{
	enum {OFF, SINGLE, INCREMENT, RANDOM, FLIP} key;
	int start;
	int increment;
	int temp;
	int ctr;

	Info(void)
		{
			key = OFF;
			start = -1;
			increment = 0;
			temp = -1;
			ctr = 0;
		}

	Info(const Info& in)
		{
			key = in.key;
			start = in.start;
			increment = in.increment;
			temp = in.temp;
			ctr = in.ctr;
		}

};


class BufferCorruptFilter :
	public BufferProducer,
	public TextSummaryProducer,
	public FileProducer
{
public:

	//
	// constructor
	//

	BufferCorruptFilter(BufferProducer& _source , int _buffer_size, int _skip, int _count,
						Info _byte_info, Info _bit_info, int _byte_offset_start,
						int _byte_offset_end, string _byte_increment, int _bit_offset_start,
						int _bit_offset_end, int _bit_offset_start_relative,
						int _bit_offset_end_relative, int _bit_width_val,
						string _bit_width_key, int _max_corruptions)
		: source(&_source),
		  valid(true),
		  buffer_size(_buffer_size),
		  skip_val(_skip),
		  count_val(_count),
		  byte_info(_byte_info),
		bit_info(_bit_info),
		byte_offset_start(_byte_offset_start),
		byte_offset_end(_byte_offset_end),
		byte_increment(_byte_increment),
		bit_offset_start(_bit_offset_start),
		bit_offset_end(_bit_offset_end),
		bit_offset_start_relative(_bit_offset_start_relative),
		bit_offset_end_relative(_bit_offset_end_relative),
		bit_width_val(_bit_width_val),
		bit_width_key(_bit_width_key),
		max_corruptions(_max_corruptions),
		random_width(false)
		{

			// seed the pseudo random number generator with the current time
			srand(static_cast<unsigned>(time(0)));

			packet_count = 0;
			copied_count = 0;
			corrupt_count = 0;
			corruption_ctr = 0;
			total_byte_corruptions = 0;
			total_bit_corruptions = 0;

			if (max_corruptions == -1)
			{
				max_corruptions = 2147483647;
			}

			error_check(byte_info, 0, 255, "byte");
			error_check(bit_info, 0, 1, "byte");

			if (byte_info.key != Info::OFF && byte_offset_start == -1)
			{
				error << "NOTE: To successfully introduce corruptions the"
					  << " --for- and --set- options must be used together."
					  << " The --set-byte option can only be used together"
					  << " with the --for-byte-offest. The --set-bit option"
					  << " can be used with either the --for-byte-offset or"
					  << " the --for-bit-offest options.\n\n";
				valid = false;
			}

			if (bit_info.key != Info::OFF && byte_offset_start == -1 && bit_offset_start == -1)
			{
				error << "NOTE: To successfully introduce corruptions the"
					  << " --for- and --set- options must be used together."
					  << " The --set-byte option can only be used together"
					  << " with the --for-byte-offest. The --set-bit option"
					  << " can be used with either the --for-byte-offset or"
					  << " the --for-bit-offest options.\n\n";
				valid = false;
			}


			// Handle byte settings
			if (byte_offset_start > byte_offset_end && byte_offset_end != -1)
			{
				int tmp_offset = byte_offset_end;
				byte_offset_end = byte_offset_start;
				byte_offset_start = tmp_offset;
			}
			if (byte_offset_start >= buffer_size)
			{
				error << "\nSpecified byte offset is equal to or greater "
					  << "than the packet size ("
					  << buffer_size << " bytes).\n";
				valid = false;
				//return NULL;
			}
			if (byte_offset_end >= buffer_size)
			{
				byte_offset_end = buffer_size - 1;
			}


			// Handle bit settings
			if (bit_offset_start == -1 &&  byte_offset_start != -1)
			{
				bit_offset_start = byte_offset_start * 8;
			}
			if (bit_offset_end == -1 && byte_offset_end != -1)
			{
				bit_offset_end = byte_offset_end * 8;
			}


			// Handle relative bit settings
			if (bit_offset_start_relative != -2147483647 && bit_offset_start != -1)
			{
				bit_offset_start = bit_offset_start + bit_offset_start_relative;
				if (bit_offset_start < 0)
				{
					bit_offset_start = -1;
				}

			}
			else if (bit_offset_start_relative != -2147483647 && bit_offset_start == -1)
			{
				bit_offset_start = bit_offset_start_relative;
				if (bit_offset_start < 0)
				{
					bit_offset_start = -1;
				}
			}

			if (bit_offset_end_relative != -2147483647 && bit_offset_end != -1)
			{
				bit_offset_end = bit_offset_end + bit_offset_end_relative;
				if (bit_offset_end < 0)
				{
					bit_offset_end = -1;
				}
			}
			else if (bit_offset_end_relative != -2147483647 && bit_offset_start != -1 && bit_offset_end == -1)
			{
				bit_offset_end = bit_offset_start + bit_offset_end_relative;
				if (bit_offset_end < 0)
				{
					bit_offset_end = -1;
				}
			}


			if (bit_offset_start > bit_offset_end && bit_offset_end != -1)
			{
				int tmp_offset = bit_offset_end;
				bit_offset_end = bit_offset_start;
				bit_offset_start = tmp_offset;
			}
			if (((double)bit_offset_start/8.0) >= (double)buffer_size)
			{
				error << "\nSpecified bit offset is equal to or greater "
					  << "than the packet size ("
					  << (buffer_size*8) << " bits).\n";
				valid = false;
				//return NULL;
			}
			if (((double)bit_offset_end/8.0) >= (double)buffer_size)
			{
				bit_offset_end = (buffer_size*8) - 1;
			}

		}



	//
	// error_check - Info structures
	//
	void error_check(Info test, int min, int max, string field_name)
		{
			if (test.key == Info::SINGLE || test.key == Info::INCREMENT)
			{
				if (test.start < min || test.start > max)
				{
					valid = false;
					error << "\nThe value of the input field set-"
						  << field_name << " is out of range: "
						  << test.start << " ("
						  << min << "-" << max << ").\n";
				}
			}

			if (test.key == Info::INCREMENT)
			{
				if ( (test.increment > max) || (test.increment < (max * -1)) )
				{
					valid = false;
					error << "\nThe value of the input field set-"
						  << field_name << " is out of range: "
						  << test.increment << "\nNote that the increment"
						  << " or step value must be set to a number"
						  << " with a magnitude less than or equal to"
						  << " the maximum value the field of interest"
						  << " can accept (which in this case is "
						  << max << ").\n";
				}
			}

		}


	//
	// print_info
	//

	string print_info (Info info)
		{
			if (info.key == Info::OFF)
			{
				return "no";
			}
			else if (info.key == Info::RANDOM)
			{
				return "random";
			}
			else if (info.key == Info::FLIP)
			{
				return "flip";
			}
			else if (info.key == Info::SINGLE)
			{
				return lexical_cast<string>(info.start);
			}
			else if (info.key == Info::INCREMENT)
			{
				return (lexical_cast<string>(info.start) + ", "
						+ lexical_cast<string>(info.increment));
			}
			else
			{
				return "<unknown>";
			}
		}




	//
	// output_text_summary
	//

	virtual void output_text_summary( textoutstream_type& os )
		{
			if (count_val == -1)
			{
				count_val = packet_count - skip_val;
			}
			if (count_val > packet_count)
			{
				count_val = packet_count;
			}
			if (skip_val >= packet_count)
			{
				skip_val = packet_count;
				count_val = 0 ;
			}
			if ((skip_val + count_val) > packet_count)
			{
				count_val = packet_count - skip_val;
			}

			string byteOffset;
			if (byte_offset_start != -1 && byte_offset_end != -1)
			{
				byteOffset = (lexical_cast<string>(byte_offset_start)
							  + ", " + lexical_cast<string>(byte_offset_end));
			}
			else if (byte_offset_start != -1)
			{
				byteOffset = lexical_cast<string>(byte_offset_start);
			}
			else
			{
				byteOffset = "none";
			}

			string bitOffset;
			if (bit_offset_start != -1 && bit_offset_end != -1)
			{
				bitOffset = (lexical_cast<string>(bit_offset_start)
							 + ", " + lexical_cast<string>(bit_offset_end));
			}
			else if (bit_offset_start != -1)
			{
				bitOffset = lexical_cast<string>(bit_offset_start);
			}
			else
			{
				bitOffset = "none";
			}

			string bitOffsetRelative;
			if (bit_offset_start_relative != -2147483647 && bit_offset_end_relative != -2147483647)
			{
				bitOffsetRelative = (lexical_cast<string>(bit_offset_start_relative)
									 + ", " + lexical_cast<string>(bit_offset_end_relative));
			}
			else if (bit_offset_start_relative != -2147483647)
			{
				bitOffsetRelative = lexical_cast<string>(bit_offset_start_relative) + ", 0";
			}
			else if (bit_offset_end_relative != -2147483647)
			{
				bitOffsetRelative = "0, " + lexical_cast<string>(bit_offset_end_relative);
			}
			else
			{
				bitOffsetRelative = "none";
			}

			string bitWidth;
			if (bit_width_val != -1 && bit_width_key == "random")
			{
				bitWidth = (lexical_cast<string>(bit_width_val)
							+ ", " + bit_width_key);
			}
			else if (bit_width_val != -1 && bit_width_key != "random")
			{
				bitWidth = lexical_cast<string>(bit_width_val);
			}
			else if (bit_width_key == "random" && bit_width_val == -1)
			{
				bitWidth = bit_width_key;
			}
			else
			{
				bitWidth = "none";
			}

			long byte_corruptions_per_packet = 0;
			long bit_corruptions_per_packet = 0;
			if (corrupt_count>0)
			{
				byte_corruptions_per_packet = total_byte_corruptions/corrupt_count;
				bit_corruptions_per_packet = total_bit_corruptions/corrupt_count;
			}

			string maxCorrupt;
			if (max_corruptions >  0 && max_corruptions < 2147483647)
			{
				maxCorrupt = lexical_cast<string>(max_corruptions);
			}
			else
			{
				maxCorrupt = "none";
			}

			os << "BufferCorruptFilter Report:\n"
			   << "	   " << "Corruption Constraints\n"
			   << "		   " << "Selected Byte Offset	 : " << byteOffset << endl
			   << "		   " << "Relative Bit Offset	 : " << bitOffsetRelative << endl
			   << "		   " << "Selected Bit Offset	 : " << bitOffset << endl
			   << "		   " << "Bit Width				 : " << bitWidth << endl
			   << "		   " << "Increment Mode			 : " << byte_increment << endl
			   << "		   " << "Max Corruptions		 : " << maxCorrupt << endl
			   << "	   " << "Corruptions Applied\n"
			   << "		   " << "set-byte				 : " << print_info(&byte_info) << endl
			   << "		   " << "set-bit				 : " << print_info(&bit_info) << endl
			   << "	   " << "Corruption Statistics\n"
			   << "		   " << packet_count << " packets read in total\n"
			   << "		   " << skip_val << " packets skipped\n"
			   << "		   " << count_val << " packets counted\n"
			   << "		   " << copied_count << " packets copied\n"
			   << "		   " << corrupt_count << " packets corrupted\n"
			   << "		   " << byte_corruptions_per_packet << " average applied byte corruptions per packet\n"
			   << "		   " << bit_corruptions_per_packet << " average applied bit corruptions per packet\n\n";

		}



	//
	// random_range
	//

	int random_range(int lowest_number, int highest_number)
		{
			if (lowest_number > highest_number){
				swap(lowest_number, highest_number);
			}

			double range = highest_number - lowest_number + 1;
			return lowest_number + int(range * rand()/(RAND_MAX + 1.0));
		}



	//
	// increment
	//

	int increment(Info input, string param_name, int max_val)
		{
			int result = 0;
			int step = input.increment;
			int ctr = input.ctr;

			if ( (step <= max_val) && (step >= (-1.0 * max_val)) )
			{
				if (ctr == 0)
				{
					result = input.start;
				}
				else
				{
					result = input.temp + step;
				}

				if (result > max_val)
				{
					result = result - (max_val + 1);
				}

				if (result < 0)
				{
					result = result + (max_val + 1);
				}

				return result;
			}
			else
			{
				if (ctr == 0)
				{
					error << "\n*** WARNING: The value of the input field " << param_name << " is out of "
						  << "range and is thus being ignored. Note that the increment or step value "
						  << "must be set to a value with a magnitude less than or equal to the maximum "
						  << "value the field of interest can accept (which in this case is " << max_val << ").\n";
				}
				valid = false;
				return -1;
			}

		}




	//
	// get_next_buffer
	//

	virtual Buffer* get_next_buffer(void)
		{

			Buffer* result = source->get_next_buffer();

			// Check to ensure result is meaningful
			if (!result)
			{
				return NULL;
			}

			DEBUG("get_next_buffer: buffer size before = " << buffer_size << endl);

			if (buffer_size > result->get_length() && result->get_length() != 0)
			{
				buffer_size = result->get_length();
			}
			DEBUG("get_next_buffer: buffer size after = " << buffer_size << endl);


			corruption_ctr = 0;

			if (skip_val == -1)
			{
				skip_val = 0;
			}
			if (packet_count < skip_val)
			{
				// return unmodified packet
				packet_count++;
				copied_count++;
				return result;
			}
			if ((count_val != -1) && (packet_count >= (skip_val + count_val)))
			{
				// return unmodified packet
				packet_count++;
				copied_count++;
				return result;
			}



			// Corrupting a range of bytes at a given offset
			if (byte_offset_start != -1 && byte_offset_end == -1)
			{
				uint8 tmp_byte_val;
				if (byte_info.key == Info::RANDOM && corruption_ctr <= max_corruptions)
				{
					tmp_byte_val = (uint8)random_range(0, 255);
					result->set_byte(tmp_byte_val, byte_offset_start);
					corruption_ctr++;
				}
				else
				{
					if (byte_info.start != -1 && byte_info.increment == 0 &&
						byte_info.key == Info::SINGLE && corruption_ctr <= max_corruptions)
					{
						tmp_byte_val = (uint8)byte_info.start;
						result->set_byte(tmp_byte_val, byte_offset_start);
						corruption_ctr++;
					}
					else if (byte_info.start != -1 && byte_info.increment != 0 &&
							 byte_info.key == Info::INCREMENT && corruption_ctr <= max_corruptions)
					{
						int tmp = increment (byte_info, "set-byte", 255);
						byte_info.ctr++;
						if (tmp != -1)
						{
							byte_info.temp = tmp;
							tmp_byte_val = (uint8)tmp;
							result->set_byte(tmp_byte_val, byte_offset_start);
							corruption_ctr++;
						}
					}
				}
			}
			else if (byte_offset_start != -1 && byte_offset_end != -1)
			{
				uint8 tmp_byte_val;
				if (byte_info.key == Info::RANDOM)
				{
					for (int i=byte_offset_start; i<=byte_offset_end; i++)
					{
						if (corruption_ctr <= max_corruptions)
						{
							tmp_byte_val = (uint8)random_range(0, 255);
							result->set_byte(tmp_byte_val, i);
							corruption_ctr++;
						}
					}
				}
				else
				{
					if (byte_info.start != -1 && byte_info.increment == 0 &&
						byte_info.key == Info::SINGLE)
					{
						tmp_byte_val = (uint8)byte_info.start;
						for (int i=byte_offset_start; i<=byte_offset_end; i++)
						{
							if (corruption_ctr <= max_corruptions)
							{
								result->set_byte(tmp_byte_val, i);
								corruption_ctr++;
							}
						}
					}
					else if (byte_info.start != -1 && byte_info.increment != 0 &&
							 byte_info.key == Info::INCREMENT)
					{
						int tmp;
						if (byte_increment == "row" || byte_increment == "both")
						{
							tmp = increment (byte_info, "set-byte", 255);
							byte_info.ctr++;
						}
						else
						{
							tmp = byte_info.start;
						}

						if (tmp != -1)
						{
							byte_info.temp = tmp;
							tmp_byte_val = (uint8)tmp;
							int ctr = 0;
							for (int i=byte_offset_start; i<=byte_offset_end; i++)
							{
								if (corruption_ctr <= max_corruptions)
								{
									if (byte_increment == "column" || byte_increment == "both")
									{
										tmp_byte_val = byte_info.temp + (byte_info.increment * ctr);
										ctr++;
									}
									result->set_byte(tmp_byte_val, i);
									corruption_ctr++;
								}
							}

						}
					}
				}
			}

			if (corruption_ctr > 0)
			{
				total_byte_corruptions += corruption_ctr;
			}



			// Corrupting a range of bits at a given offset
			if (bit_offset_start != -1 && bit_offset_end == -1)
			{
				int tmp_bit_val;
				if (bit_info.key == Info::RANDOM && corruption_ctr <= max_corruptions)
				{
					tmp_bit_val = random_range(0, 1);
					if (result->set_bit(tmp_bit_val, bit_offset_start))
					{
						corruption_ctr++;
					}
				}
				else if (bit_info.key == Info::FLIP && corruption_ctr <= max_corruptions)
				{
					result->flip_bit(bit_offset_start);
					corruption_ctr++;
				}
				else
				{
					if (bit_info.start != -1 && bit_info.increment == 0 &&
						bit_info.key == Info::SINGLE && corruption_ctr <= max_corruptions)
					{
						if (result->set_bit(bit_info.start, bit_offset_start))
						{
							corruption_ctr++;
						}
					}
					else if (bit_info.start != -1 && bit_info.increment != 0 &&
							 bit_info.key == Info::INCREMENT && corruption_ctr <= max_corruptions)
					{
						int tmp = increment (bit_info, "set-bit", 1);
						bit_info.ctr++;
						if (tmp != -1)
						{
							bit_info.temp = tmp;
							if (result->set_bit(tmp, bit_offset_start))
							{
								corruption_ctr++;
							}
						}
					}
				}
			}
			else if (bit_offset_start != -1 && bit_offset_end != -1)
			{
				int tmp_bit_val;
				int width = 1;
				int limit = 2;
				int range = bit_offset_end - bit_offset_start + 1;
				if (range < 1)
				{
					range = 1;
				}

				if (bit_width_val > -1 && bit_width_key != "random")
				{
					width = bit_width_val + 1;
					if (width < 1)
					{
						width = 1;
					}
					if (width > range)
					{
						width = range;
					}
				}
				else
				{
					if (bit_width_key == "random")
					{
						if (bit_width_val > -1)
						{
							limit = bit_width_val;
							if (limit < 2)
							{
								limit = 2;
							}
							if (limit > range)
							{
								limit = range;
							}
						}
						else
						{
							int max = 1;
							if (max_corruptions > 0)
							{
								max = max_corruptions;
							}
							else
							{
								max = random_range(0, range);
							}

							limit = range/ max;
							if (limit < 2)
							{
								limit = 2;
							}
							if (limit > range)
							{
								limit = range;
							}
						}
						width = random_range(1, limit);
						random_width = true;
					}
				}

				if (bit_info.key == Info::RANDOM)
				{
					for (int i=bit_offset_start; i<=bit_offset_end; i=i+width)
					{
						if (corruption_ctr <= max_corruptions)
						{
							tmp_bit_val = random_range(0, 1);
							if (result->set_bit(tmp_bit_val, i))
							{
								corruption_ctr++;
							}
							if (random_width)
							{
								width = random_range(1, limit);
							}
						}
					}
				}
				else if (bit_info.key == Info::FLIP)
				{
					for (int i=bit_offset_start; i<=bit_offset_end; i=i+width)
					{
						if (corruption_ctr <= max_corruptions)
						{
							result->flip_bit(i);
							corruption_ctr++;
							if (random_width)
							{
								width = random_range(1, limit);
							}
						}
					}
				}
				else
				{
					if (bit_info.start != -1 && bit_info.increment == 0 &&
						bit_info.key == Info::SINGLE)
					{
						for (int i=bit_offset_start; i<=bit_offset_end; i=i+width)
						{
							if (corruption_ctr <= max_corruptions)
							{
								if(result->set_bit(bit_info.start, i))
								{
									corruption_ctr++;
								}
								if (random_width)
								{
									width = random_range(1, limit);
								}
							}
						}
					}
					else if (bit_info.start != -1 && bit_info.increment != 0 &&
							 bit_info.key == Info::INCREMENT)
					{
						int tmp;
						if (byte_increment == "row" || byte_increment == "both")
						{
							tmp = increment (bit_info, "set-bit", 1);
							bit_info.ctr++;
						}
						else
						{
							tmp = bit_info.start;
						}

						if (tmp != -1)
						{
							bit_info.temp = tmp;
							int ctr = 0;
							for (int i=bit_offset_start; i<=bit_offset_end; i=i+width)
							{
								if (corruption_ctr <= max_corruptions)
								{
									if (byte_increment == "column" || byte_increment == "both")
									{
										tmp = bit_info.temp + (bit_info.increment * ctr);
										ctr++;
									}
									if (result->set_bit(tmp, i))
									{
										corruption_ctr++;
									}
									if (random_width)
									{
										width = random_range(1, limit);
									}
								}
							}

						}
					}
				}
			}


			packet_count++;
			if (corruption_ctr > 0)
			{
				corrupt_count++;
				total_bit_corruptions = ( total_bit_corruptions
										  + corruption_ctr
										  - total_byte_corruptions/packet_count);
			}
			else
			{
				copied_count++;
			}

			DEBUG("Packet " << packet_count
				  << "	Byte Corruptions " << total_byte_corruptions
				  << "	Bit Corruptions " << total_bit_corruptions << endl);

			return result;

		}



	//
	// produce_files
	//

	virtual bool produce_files(FileConsumer& dest)
		{
			Buffer* buff;
			while((buff=get_next_buffer()))
			{
				ostream_type* os = dest.get_named_output_stream("buffer.out");
				os->write((char*)buff->get_data(),buffer_size);
			}
			return true;
		}


	//
	// get_next_file
	//

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}


	//
	// is_valid
	//

	bool is_valid(void) const
		{
			return valid;
		}


private:

	BufferProducer* source;
	bool valid;
	int buffer_size;

	int packet_count;
	int copied_count;
	int corrupt_count;
	int corruption_ctr;
	long total_byte_corruptions;
	long total_bit_corruptions;

	int skip_val;
	int count_val;

	Info byte_info;
	Info bit_info;

	int byte_offset_start;
	int byte_offset_end;
	string byte_increment;
	int bit_offset_start;
	int bit_offset_end;
	int bit_offset_start_relative;
	int bit_offset_end_relative;
	int bit_width_val;
	string bit_width_key;
	int max_corruptions;

	bool random_width;



};

#endif
