////////////////////////////////////////////////////////////////////////////////
//
//	File: cadu_displayer.cpp
//
//	Provides: cadu_displayer executable
//
//	Purpose: Display information on CADU packet streams
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "readfromstdin.hpp"
#include "textrecorddisplayer.hpp"
#include "tvcdudisplayer.hpp"
#include "cadureader.hpp"
#include "cadutocvcdu.hpp"
#include "cvcdutovcdu.hpp"
#include "args.hpp"
#include "vcdutotvcdu.hpp"
#include "caduwriter.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

int gerror = 0; // global error
const string APPNAME = "cadu_displayer";
const string MAINTAINER = "elson@eumetsat.int";

int main(int argc, char* argv[])
{
	ofstream onull("/dev/null");
	_debug = &onull;
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;

	deque<string> in_files;
	bool compact = false;
	string write_cadus;
	bool pad = false;

	Args args;
	args.set_doc_head("View CADU packets.");
	args.set_doc_tail("");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2005 Eumetsat.");
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("tvcdu_displayer");
	args.add_related_prog("tvcdu_to_ccsds");
	args.add_related_prog("ccsds_displayer");

	args.add_example("cadu_displayer file.cadu", "");

	args.add(&in_files, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input file(s)", "If omitted packets are read from standard input.");
	args.add(&compact, "compact", 0, 0, "Show compact output style");
	args.add(&write_cadus, "write-cadus", 0, 0, "Instead of normal output write a stream of pure CADU packets to <filename>", "Any gaps between packets or breaks in data in the original file will be removed. CADUs are identified always by looking for a correct sync mark that is 1024 or more bytes after the previous one.");
	args.add(&pad, "pad", 0, 0, "Add 4 bytes of padding data after each CADU if the --write-cadus option is used");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case Args::USER_OPTION:
			_debug = &cout;
			break;
		}
	}

	spinner->restream(&onull);

	// set UTC time zone
	TimeConv::set_time_zone();

	FileProducer* first_source;
	if (in_files.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_files);
	}

	CADUReader cadu_reader(*first_source);

	if (write_cadus.empty())
	{
		CADUToCVCDU cadu_to_cvcdu(&cadu_reader);
		CVCDUToVCDU cvcdu_to_vcdu(&cadu_to_cvcdu, false, false, false); // strip filler, strip encript, rs checks
		VCDUTotVCDU vcdu_to_tvcdu(cvcdu_to_vcdu, true); // ignore times
		tVCDUDisplayer tvcduer(vcdu_to_tvcdu, compact, false); // show tvcdu
		TextRecordDisplayer displayer(tvcduer);
	}
	else
	{
		CADUWriter cadu_writer(cadu_reader, write_cadus, pad);
		first_source->output_text_summary();
		cadu_reader.output_text_summary();
		cadu_writer.output_text_summary();
	}

	delete first_source;
	return 0;
}
