////////////////////////////////////////////////////////////////////////////////
//
//	File: cadu_generator.cpp
//
//	Provides: cadu_generator executable
//
//	Purpose: Synthesize a stream of Fill CADU packets
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "args.hpp"
#include "cadugenerator.hpp"
#include "singlefilewriter.hpp"
#include "caduextractor.hpp"
#include "writetostdout.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "cadu_generator";
const string MAINTAINER = "elson@eumetsat.de";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	string out_file;

	bool verbose = true;
	unsigned int initial_vccount = 0;
	unsigned int mod_vccount = 1;
	string scount;
	double rate = 3.5;
	int scid = 12;

	Args args;
	args.set_doc_head("Synthesize a stream of CADU packets.");
	args.set_doc_tail("");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2006 Eumetsat.");
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("cadu_displayer");

	args.add_example("cadu_generator --count 240s --init 10000 --mod 1 -o dummy.cadu","");

	args.add(&out_file, "out", 'o', 0, "Output file name. If ommited and standard output is a terminal, normal processing is carried out and a report displayed by no files written. If ommited and standard output is not a terminal, CCSDS packets are streamed to it.");
	args.add(&initial_vccount, "init", 0, 0, "Initial Virtual Channel counter value.", "Range 0 to 2^24-1");
	args.add(&mod_vccount, "mod", 0, 0, "Offset to apply to subsequent VC counter values. Default is 0.", "Counters wrap around at 2^24");
	args.add(&scount, "count", 'c', 0, "Number of Fill CADUs to create. Can be specified as a plain number or a time with prefix s (seconds) e.g. --count 120s");
	args.add(&rate, "rate", 0, 0, "Nominal data rate if --count is used with the 's' prefix. Specified as a number with units of Mb/s. Defaults to 3.5");
	args.add(&scid, "scid", 0, 0, "Spacecraft ID. Default is 12 (Metop-2)");

	args.add(0, "quiet", 'q', 0, "Omit additional messages");
	args.add(0, "", 'z', 0, "Use simple busy animations");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case 'q':
			verbose = false;
			break;

		case Args::USER_OPTION:
			_debug = &cout;
			break;

		case 'z':
			Spinner::use_simple_spinner();
			break;
		}
	}

	// fix streams if writing to stdout
	if (out_file.empty())
	{
		if (isatty(1)!=1)
		{
			verbose = false;
		}
		else
		{
			cerr << "This program cannot stream out to a terminal. Either specify an output file or pipe into another process.\n";
			return -1;
		}
	}

	if (scount.empty())
	{
		cerr << "Count option must be specified.\n";
		return -1;
	}

	unsigned int count;
	if (scount.find_first_not_of("0123456789") != string::npos)
	{
		char type = scount[scount.length()-1];
		if (type != 's')
		{
			cerr << "Value of --count parameter not understood. Use either a plain integer value e.g. 5, or a time with seconds prefix e.g. 10s or 240.41s\n";
			return -1;
		}
		string temp = scount.substr(0,scount.length()-1);
		if (temp.find_first_not_of("0123456789.") != string::npos)
		{
			cerr << "Value of --count parameter not understood. Use either a plain integer value e.g. 5, or a time with seconds prefix e.g. 10s or 240.41s\n";
			return -1;
		}
		double s = atof(temp.c_str());
		count = static_cast<unsigned int>(s * rate * 1000000.0 / 8192); // required no. bits over bits per CADU
	}
	else
	{
		count = atol(scount.c_str());
	}

	CADUGenerator source(count, initial_vccount, mod_vccount, scid);
	CADUExtractor extractor(&source);

	if (out_file.empty())
	{
		WriteToStdout writer(&extractor);

		source.output_text_summary();
		extractor.output_text_summary();
		writer.output_text_summary();
	}
	else
	{
		SingleFileWriter writer(&extractor, out_file);

		source.output_text_summary();
		extractor.output_text_summary();
		writer.output_text_summary();
	}

	return 0;
}
