////////////////////////////////////////////////////////////////////////////////
//
//	File: cadu_orbit_splitter.cpp
//
//	Provides: cadu_orbit_splitter executable
//
//	Purpose: splits a CADU file into dumps
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "readfromstdin.hpp"
#include "cadureader.hpp"
#include "cadutocvcdu.hpp"
#include "cvcdutovcdu.hpp"
#include "args.hpp"
#include "caduwriter.hpp"
#include "cadusplitter.hpp"
#include "multifilewriter.hpp"
#include "writetonull.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

int gerror = 0; // global error
const string APPNAME = "cadu_orbit_splitter";
const string MAINTAINER = "elson@eumetsat.int";

int main(int argc, char* argv[])
{
	ofstream onull("/dev/null");
	_debug = &onull;
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;

	deque<string> in_files;
	string output_prefix;
	string orbit_len;
	string orbit_overlap;
	bool shift_gras_obt = false;
	bool apid1_only = false;

	Args args;
	args.set_doc_head("Split CADU packets into dumps.");
	args.set_doc_tail("");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2006 Eumetsat.");
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("tvcdu_displayer");
	args.add_related_prog("tvcdu_to_ccsds");
	args.add_related_prog("ccsds_displayer");
	args.add_related_prog("cadu_displayer");

	args.add_example("cadu_orbit_splitter file.cadu -o splits --orbit 102m --overlap 42s", "");

	args.add(&in_files, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input file(s)", "If omitted packets are read from standard input.");
	args.add(&output_prefix, "out", 'o', 0, "Output prefix filename");
	args.add(&orbit_len, "duration", 0, 0, "Duration of orbit. Postfix number with m/s for minutes/seconds");
	args.add(&orbit_overlap, "overlap", 0, 0, "Duration of dump overlap. Postfix number with m/s for minutes/seconds");
	args.add(&shift_gras_obt, "shift-gras-obt", 0, 0, "Interpret GRAS OBT header times using the correct notation, shifted 8 bits upwards relative to other instruments (APID 480-511)");
	args.add(&apid1_only, "apid1-only", 0, 0, "Only use OBT values of APID 1 packets when deciding where to cut orbits");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case Args::USER_OPTION:
			_debug = &cout;
			break;
		}
	}

	spinner->restream(&onull);

	// set UTC time zone
	TimeConv::set_time_zone();

	FileProducer* source;
	if (in_files.empty())
	{
		source = new ReadFromStdin();
	}
	else
	{
		source = new FileReader(in_files);
	}

	long long len = orbit_len.empty() ? -1 : TimeConv::duration_to_int(orbit_len);
	long long overlap = orbit_overlap.empty() ? -1 : TimeConv::duration_to_int(orbit_overlap);

	CADUReader reader(*source);
	CADUSplitter splitter(&reader, len, overlap, shift_gras_obt, apid1_only);

	FileConsumer* target;
	if (output_prefix.empty())
	{
		target = new WriteToNull(splitter);
	}
	else
	{
		target = new MultiFileWriter(splitter, output_prefix);
	}

	source->output_text_summary();
	reader.output_text_summary();
	splitter.output_text_summary();
	target->output_text_summary();

	return 0;
}
