////////////////////////////////////////////////////////////////////////////////
//
//	File: cadu_scid_setter.cpp
//
//	Provides: cadu_scid_setter executable
//
//	Purpose: Modify the SCID value of CADU files
//
//	Copyright (c) 2008 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "readfromstdin.hpp"
#include "cadureader.hpp"
#include "vcduscidsetter.hpp"
#include "args.hpp"
#include "caduwriter.hpp"
#include "cadutocvcdu.hpp"
#include "cvcdutovcdu.hpp"
#include "vcdutocadu.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

int gerror = 0; // global error
const string APPNAME = "cadu_scid_setter";
const string MAINTAINER = "mike.elson@eumetsat.int";

int main(int argc, char* argv[])
{
	ofstream onull("/dev/null");
	_debug = &onull;
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;

	deque<string> in_files;
	string out_file;
	int new_scid;

	Args args;
	args.set_doc_head("Modify SCID values in CADU packets.");
	args.set_doc_tail("SCID values:\n\t11 - METOP 1\n\t12 - METOP 2\n\t13 - METOP 3\n\t14 - METOP SIM");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2008 Eumetsat.");
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("cadu_displayer");

	args.add_example("cadu_scid_setter input.cadu --set-scid 5 --out output.cadu", "");

	args.add(&in_files, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input file(s)", "If omitted packets are read from standard input.");
	args.add(&out_file, "out", 'o', Args::FILENAME, "Select output filename", "If ommited packets are written to standard output");
	args.add(&new_scid, "set-scid", 's', 0, "New SCID value");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case Args::USER_OPTION:
			_debug = &cout;
			break;
		}
	}

	spinner->restream(&onull);

	// set UTC time zone
	TimeConv::set_time_zone();

	FileProducer* first_source;
	if (in_files.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_files);
	}

	CADUReader cadu_reader(*first_source);
	CADUToCVCDU cadu_to_cvcdu(&cadu_reader);
	const bool strip_filler = false;
	const bool strip_encrypt = true;
	const bool rs_checks = false;
	CVCDUToVCDU cvcdu_to_vcdu(&cadu_to_cvcdu, strip_filler, strip_encrypt, rs_checks);
	VCDUSCIDSetter scid_setter(cvcdu_to_vcdu, new_scid);
	VCDUToCADU vcdu_to_cadu(&scid_setter);

	if (out_file.empty())
	{
		cerr << "No output name specified\n";
		return -1;
	}

	const bool pad = false;
	CADUWriter writer(vcdu_to_cadu, out_file, pad);

	first_source->output_text_summary();
	cadu_reader.output_text_summary();
	scid_setter.output_text_summary();
	writer.output_text_summary();

	delete first_source;
	return 0;
}
