////////////////////////////////////////////////////////////////////////////////
//
//	File: cadu_to_tvcdu.cpp
//
//	Provides: cadu_to_tvcdu executable
//
//	Purpose: Convert a stream of CADU packets to tVCDU packets
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "cadureader.hpp"
#include "cadutocvcdu.hpp"
#include "cvcdutovcdu.hpp"
#include "vcdutotvcdu.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"
#include "tvcduextractor.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "cadu_to_tvcdu";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_files;
	string out_file;

	bool verbose = true;
	bool strip_filler = false;
	bool strip_encrypt = false;
	bool ignore_times = false;
	bool rs = true;

	enum { ARGV_NONE, ARGV_IN, ARGV_OUT } state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__
				 << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on "
				 << __DATE__
				 << "\nConvert a stream of CADU packets to tVCDU packets.\n\n"
				 << "usage:\n" << APPNAME << " OPTIONS\n\nwhere OPTIONS can "
				 << "include:\n"
				 << "[--in, -i] <filename> : select input file\n"
				 << "--out, -o <filename> : select output file\n"
				 << "--strip-filler : remove Fill-VCDUs from output file\n"
				 << "--strip-encrypt : remove encrpyted packets from output\n"
				 << "--rs : perform Reed-Solomon correction and remove CVCDU "
				 << "packets which cannot be corrected.\n"
				 << "--include-filler : include Fill-VCDUs in output file\n"
				 << "--include-encrypt : include encrypted VCDUs in output\n"
				 << "--no-rs : do not perform any Reed-Solomon checks\n"
				 << "--ignore-times : set the initial time of the generated "
				 << "tVCDU headers to midnight 01/01/2000.\n"
				 << "-z : Use simple animation"
				 << "--help : display this message\n"
				 << "--version : display software version\n\n"
				 << "Default settings :\n"
				 << "  strip-filler -> " << (strip_filler?"yes":"no") << endl
				 << "  strip-encrypt -> " << (strip_encrypt?"yes":"no") << endl
				 << "  rs checks -> " << (rs?"yes":"no")
				 << endl << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_OUT;
		}
		else if (!strcmp(argv[x], "--quiet"))
		{
			verbose = false;
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else if (!strcmp(argv[x], "--strip-filler"))
		{
			strip_filler = true;
		}
		else if (!strcmp(argv[x], "--strip-encrypt"))
		{
			strip_encrypt = true;
		}
		else if (!strcmp(argv[x], "--rs"))
		{
			rs = true;
		}
		else if (!strcmp(argv[x], "--include-filler"))
		{
			strip_filler = false;
		}
		else if (!strcmp(argv[x], "--include-encrypt"))
		{
			strip_encrypt = false;
		}
		else if (!strcmp(argv[x], "--no-rs"))
		{
			rs = false;
		}
		else if (!strcmp(argv[x], "--ignore-times"))
		{
			ignore_times = true;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else
		{
			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				in_files.push_back(argv[x]);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					cerr << "Only one output filename can be specified.\n";
					return -1;
				}
				else
				{
					out_file = argv[x];
				}
				break;
			}
		}
	}

	// fix streams if writing to stdout
	if (out_file.empty() || in_files.empty())
	{
		_info = &onull;
		_warning = &onull;
	}

	if (in_files.empty())
	{
		error << "No input files specified\n";
		return -1;
	}

	FileReader source(in_files);
	CADUReader cadu_reader(source);
	CADUToCVCDU cadu_to_cvcdu(&cadu_reader);
	CVCDUToVCDU cvcdu_to_vcdu(&cadu_to_cvcdu,
							  strip_filler,
							  strip_encrypt,
							  rs);
	VCDUTotVCDU vcdu_to_tvcdu(cvcdu_to_vcdu, ignore_times);
	tVCDUExtractor extractor(vcdu_to_tvcdu);

	if (out_file.empty())
	{
		WriteToStdout x(&extractor);
	}
	else
	{
		SingleFileWriter x(&extractor, out_file);
		if (verbose && !gerror)
		{
			source.output_text_summary();
			cadu_reader.output_text_summary();
			cadu_to_cvcdu.output_text_summary();
			cvcdu_to_vcdu.output_text_summary();
			vcdu_to_tvcdu.output_text_summary();
			extractor.output_text_summary();
			x.output_text_summary();
		}
	}

	return 0;
}
