////////////////////////////////////////////////////////////////////////////////
//
//	File: cadusplitter.cpp
//
//	Provides: CADUSplitter class
//
//	Purpose:
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "cadusplitter.hpp"
#include "cadutocvcdu.hpp"
#include "cvcdutovcdu.hpp"
#include "ccsds.hpp"
#include "fileconsumer.hpp"

const int DEFAULT_DURATION = 102*60*1000;
const int DEFAULT_OVERLAP = 8*60*1000;

CADUSplitter::CADUSplitter(CADUProducer* _source,
						   long long _duration,
						   long long _overlap,
						   bool _shift_gras_obt,
						   bool _apid1_only)
	: source(_source),
	  duration(_duration),
	  overlap(_overlap),
	  shift_gras_obt(_shift_gras_obt),
	  apid1_only(_apid1_only),
	  current_file(NULL),
	  file_counter(0),
	  cadu_counter(0)
{
	current_fileinfo.fresh = true;
	current_fileinfo.rollovers = 0;
	current_fileinfo.cadu_counter = 0;

	if (duration==-1)
	{
		duration = DEFAULT_DURATION;
	}

	if (overlap==-1)
	{
		overlap = DEFAULT_OVERLAP;
	}

	CADUToCVCDU* cadu_to_cvcdu = new CADUToCVCDU(this);
	CVCDUToVCDU* cvcdu_to_vcdu = new CVCDUToVCDU(cadu_to_cvcdu,false,false,false);
	vcdu_chain.reset(cvcdu_to_vcdu);
	cout << "Orbit duration " << duration << " overlap " << overlap << endl;
}

CADUSplitter::~CADUSplitter(void)
{
	vcdu_chain->delete_source();
}

void CADUSplitter::output_text_summary(ostream& os)
{
	os << "CADUSplitter report:\n";
	for(FileInfos::iterator iter=fileinfos.begin(); iter!=fileinfos.end(); iter++)
	{
		os << "	   " << (*iter).first
		   << " initial OBT " << (*iter).second.initial_obt
		   << " final OBT " << (*iter).second.final_obt
		   << " CADU count " << (*iter).second.cadu_counter << endl;
	}
	os << endl;
}

istream_type* CADUSplitter::get_next_file(void)
{
	return NULL;
}

bool CADUSplitter::produce_files(FileConsumer& _writer)
{
	writer = &_writer;
	OBTTime obt;

	while((current_cadu = source->get_next_cadu()))
	{
		cadu_counter++;
		current_fileinfo.cadu_counter++;

		store_cadu(current_cadu);
		CADU dummy(*current_cadu);

		VCDU* vcdu = vcdu_chain->get_next_vcdu();
		VCID vcid = vcdu->get_vcid();
		unsigned int fhp = vcdu->mpdu.get_first_header_pointer();

		if (vcid!=VCID_FILL && fhp != 2047)
		{
			if (fhp>=882)
			{
				cerr << "WARNING bad packet - FIRST_HEADER_POINTER cannot be "
					 << ">882 (found " << fhp << ")\n";
			}
			else
			{
				while(fhp<VCDU::PAYLOAD_SIZE)
				{
					// payload size is 886
					// ccsds primary header is 6 bytes
					if (fhp > (VCDU::PAYLOAD_SIZE-20))
					{
						//cerr << "(cannot retrieve full primary header with fhp " << fhp << ")\n";
						fhp = VCDU::PAYLOAD_SIZE;
					}
					else
					{
						CCSDS ccsds(reinterpret_cast<byte*>(&vcdu->mpdu)+fhp+2, false);
						if ((!apid1_only) || ccsds.get_apid()==1)
						{

							obt = *ccsds.get_ancillary_data();
							obt.normalise();
							if (shift_gras_obt && ccsds.get_apid()>=448 && ccsds.get_apid()<=511)
							{
								obt = make_obt_time(obt.get_shifted_obt_coarse(), obt.get_shifted_obt_fine());
							}

							if (current_fileinfo.fresh)
							{
								current_fileinfo.initial_obt = obt;
								current_fileinfo.fresh = false;
							}

							if (!initial_obt.get())
							{
								cout << "Start of file " << file_counter << " at CADU " << cadu_counter << " OBT " << obt << endl;
								initial_obt.reset(new OBTTime(obt));
								set_dump_boundaries(*initial_obt);
							}

							if (overlap_queue.empty() && obt > begin_overlap)
							{
								// try to catch OBT wrap arounds
								if (fabs(static_cast<double>(obt-begin_overlap)) < (8000000LL*65536LL))
								{
									cout << "Begin recording overlap for dump " << (file_counter+1) << " at CADU " << cadu_counter << " OBT " << obt << "\n";
									overlap_queue.push(dummy);
								}
							}

							if (obt > end_dump)
							{
								// try to catch OBT wrap arounds
								if (fabs(static_cast<double>(obt-begin_overlap)) < (8000000LL*65536LL))
								{
									cout << "End of dump " << file_counter << " at CADU " << cadu_counter << " OBT " << obt << "\n\n";
									current_fileinfo.final_obt = obt;
									finalise_file();
									new_file();
									flush_overlap();
									set_dump_boundaries(obt);
								}
							}
						}

						if (fhp < 880)
						{
							fhp += ccsds.get_packet_length()+7;
						}
						else
						{
							fhp += VCDU::PAYLOAD_SIZE;
						}
					}
				}
			}
		}
	}

	current_fileinfo.final_obt = obt;
	finalise_file();
	return true;
}

void CADUSplitter::store_cadu(CADU* cadu)
{
	if (!current_file)
	{
		new_file();
	}
	if (!overlap_queue.empty())
	{
		overlap_queue.push(*cadu);
	}
	current_file->write(reinterpret_cast<char*>(cadu), CADU::SIZE);
}

void CADUSplitter::flush_overlap(void)
{
	while(!overlap_queue.empty())
	{
		current_file->write(reinterpret_cast<char*>(&overlap_queue.front()), CADU::SIZE);
		overlap_queue.pop();
	}
}

void CADUSplitter::finalise_file(void)
{
	if (current_file)
	{
		fileinfos[writer->get_filename_base() + lexical_cast<string>(file_counter-1)] = current_fileinfo;
	}
}

void CADUSplitter::new_file(void)
{
	current_file = writer->get_named_output_stream(lexical_cast<string>(file_counter++));
	current_fileinfo.rollovers = 0;
	current_fileinfo.cadu_counter = 0;
	current_fileinfo.fresh = true;
}

void CADUSplitter::set_dump_boundaries(const OBTTime& start_time)
{
//	  cout << "durartion " << duration << " overlap " << overlap << endl;
	end_dump = start_time + duration*65536/1000;
	begin_overlap = end_dump - overlap*66536/1000;

	end_dump.normalise();
	begin_overlap.normalise();

	cout << "Set dump start time to " << start_time << endl
		 << " dump end time " << end_dump << endl
		 << " begin overlap time " << begin_overlap << endl;
}

CADU* CADUSplitter::get_next_cadu(void)
{
	return current_cadu;
}
