////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds.cpp
//
//	Provides: misc structs
//
//	Purpose: Code for representing source packets
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"

string uint16tosfloat(uint16_be num)
{
	unsigned int s = (num >> 15) & 1;
	unsigned int e = (num >> 7) & 255;
	unsigned int f = num & 127;


	if (e == 255 && f != 0)
	{
		return "NaN";
	}

	if (e == 255 && f == 0)
	{
		if (s == 0)
		{
			return "Infinity";
		}
		else
		{
			return "-Infinity";
		}
	}

	if ((e > 0 && e < 255) || (e == 0 && f != 0))
	{
		stringstream strStream;
		double result = 0.0;
		double fraction = 0.0;

		for (int ctr=0; ctr<7; ctr++)
		{
			fraction += ((f>>(6-ctr)) & 1) * pow(2.0, -1*(ctr+1));
		}

		result = pow(2.0, (int)(e - 127)) * (1 + fraction);

		if (s == 0)
		{
			strStream << result;
		}
		else
		{
			strStream << "-" << result;
		}
		return strStream.str();
	}

	if ( e == 0 && f == 0)
	{
		return "0.0";
	}

	return "";
}

string uint16tofloat(uint16_be msw, uint16_be lsw)
{

	unsigned int s = (msw >> 15) & 1;
	unsigned int ex = (msw >> 7) & 255;
	unsigned int f = ((msw & 127)<<16) + lsw;


	if (ex == 255 && f != 0)
	{
		return "NaN";
	}

	if (ex == 255 && f == 0)
	{
		if (s == 0)
		{
			return "Infinity";
		}
		else
		{
			return "-Infinity";
		}
	}

	if ((ex > 0 && ex < 255) || (ex == 0 && f != 0))
	{
		stringstream strStream;
		double result = 0.0;
		double fraction = 0.0;

		for (int ctr=0; ctr<23; ctr++)
		{
			fraction += ((f>>(22-ctr)) & 1) * pow(2.0, -1*(ctr+1));
		}

		result = pow(2.0, (int)(ex - 127)) * (1 + fraction);

		if (s == 0)
		{
			strStream << result;
		}
		else
		{
			strStream << "-" << result;
		}
		return strStream.str();
	}

	if ( ex == 0 && f == 0)
	{
		return "0.0";
	}

	return "";
}


//
//
// CCSDSPrimaryHeader
//
//

unsigned int CCSDSPrimaryHeader::get_byte(int pos) const
{
	return static_cast<unsigned int>(data[pos]);
}

unsigned int CCSDSPrimaryHeader::get_version_number(void) const
{
	return (get_byte(0)>>5);
}

unsigned int CCSDSPrimaryHeader::get_type(void) const
{
	return ((get_byte(0)>>4)&1);
}

unsigned int CCSDSPrimaryHeader::get_secondary_header_flag(void) const
{
	return ((get_byte(0)>>3)&1);
}

unsigned int CCSDSPrimaryHeader::get_apid(void) const
{
    return (((get_byte(0)&3)<<8)+get_byte(1));
}

unsigned int CCSDSPrimaryHeader::get_sequence_flags(void) const
{
	return (get_byte(2)>>6);
}

unsigned int CCSDSPrimaryHeader::get_sequence_count(void) const
{
	return (((get_byte(2)&63)<<8)+get_byte(3));
}

unsigned int CCSDSPrimaryHeader::get_packet_length(void) const
{
	return ((get_byte(4)<<8)+get_byte(5));
}

void CCSDSPrimaryHeader::set_byte(int pos, unsigned int value)
{
	data[pos] = value;
}

void CCSDSPrimaryHeader::set_version_number(unsigned int vn)
{
	unsigned int b = get_byte(0);
	b &= 31;
	b |= (vn<<5);
	set_byte(0, b);
}

void CCSDSPrimaryHeader::set_type(unsigned int t)
{
	unsigned int b = get_byte(0);
	b &= (224 | 15);
	b |= (t<<4);
	set_byte(0, b);
}

void CCSDSPrimaryHeader::set_secondary_header_flag(unsigned int shf)
{
	unsigned int b = get_byte(0);
	b &= (240 | 7);
	b |= (shf<<3);
	set_byte(0, b);
}

void CCSDSPrimaryHeader::set_apid(unsigned int apid)
{
	unsigned int b1 = get_byte(0);
	unsigned int b2 = get_byte(1);

	b1 &= 248;
	b1 |= (apid>>8);

	b2 = apid&255; // not needed

	set_byte(0, b1);
	set_byte(1, b2);
}

void CCSDSPrimaryHeader::set_sequence_flags(unsigned int sf)
{
	unsigned int b = get_byte(2);
	b &= 63;
	b |= (sf<<6);
	set_byte(2,b);
}

void CCSDSPrimaryHeader::set_sequence_count(CCSDSSequenceCount sc)
{
	unsigned int b1 = get_byte(2);
	unsigned int b2 = get_byte(3);

	b1 &= 192;
	b1 |= ((sc>>8)&63);

	b2 = sc&255; // not needed

	set_byte(2, b1);
	set_byte(3, b2);
}

void CCSDSPrimaryHeader::set_packet_length(unsigned int pl)
{
	set_byte(4, pl>>8);
	set_byte(5, pl&255); // and not needed
}


//
//
// UTCTime
//
//

bool UTCTime::operator > (const UTCTime& rhs) const
{
	if (days != rhs.days)
	{
		return (days > rhs.days);
	}
	else if (milliseconds != rhs.milliseconds)
	{
		return (milliseconds > rhs.milliseconds);
	}
	else
	{
		return (microseconds > rhs.microseconds);
	}
}

bool UTCTime::operator < (const UTCTime& rhs) const
{
	if (days != rhs.days)
	{
		return (days < rhs.days);
	}
	else if (milliseconds != rhs.milliseconds)
	{
		return (milliseconds < rhs.milliseconds);
	}
	else
	{
		return (microseconds < rhs.microseconds);
	}
}

bool UTCTime::operator != (const UTCTime& rhs) const
{
	return (!(days==rhs.days
			  && milliseconds==rhs.milliseconds
			  && microseconds==rhs.microseconds));
}

unsigned int UTCTime::get_utc_days(void) const
{
	return days;
}

unsigned int UTCTime::get_utc_milliseconds(void) const
{
	return milliseconds;
}

unsigned int UTCTime::get_utc_microseconds(void) const
{
	return microseconds;
}

void UTCTime::set_utc_days(unsigned int _days)
{
	days = _days;
}

void UTCTime::set_utc_milliseconds(unsigned int _milliseconds)
{
	milliseconds = _milliseconds;
}

void UTCTime::set_utc_microseconds(unsigned int _microseconds)
{
	microseconds = _microseconds;
}

UTCTime make_utc_time(unsigned int _days,
					  unsigned int _milliseconds,
					  unsigned int _microseconds)
{
	UTCTime result;
	result.days = _days;
	result.milliseconds = _milliseconds;
	result.microseconds = _microseconds;
	return result;
}

textoutstream_type& operator << (textoutstream_type& os, const UTCTime& msh)
{
	os << TimeConv::utc_days_milliseconds_to_string(
		msh.get_utc_days(), msh.get_utc_milliseconds()) << "."
	   << ntofs(msh.get_utc_milliseconds()%1000, 3, '0')
	   << ntofs(msh.get_utc_microseconds(), 3, '0');

	return os;
}

long long UTCTime::operator - (const UTCTime& rhs) const
{
//	  cout << "days, rhs.days : "		  << days		  << " , " << rhs.days << endl
//	 << "millisec, rhs.millisec : " << milliseconds << " , " << rhs.milliseconds << endl;
	long long result = (days - rhs.days)*static_cast<long long>(MILLISECONDS_PER_DAY);
	result += static_cast<long long>(milliseconds) - static_cast<long long>(rhs.milliseconds);
	return result;
}

UTCTime& UTCTime::operator += (long long mod_milli)
{
//	  cout << "days " << days << " ms " << milliseconds
//	 << " mod_milli " << mod_milli << endl;

	long long new_milli = milliseconds;
	new_milli += mod_milli;

//	  cout << "new_milli " << new_milli
//	 << " (days " << (new_milli/static_cast<long long>(MILLISECONDS_PER_DAY)) << ")\n";

	if (new_milli > MILLISECONDS_PER_DAY)
	{
		days += (new_milli/MILLISECONDS_PER_DAY);
		milliseconds = new_milli % MILLISECONDS_PER_DAY;
	}
	else if (new_milli < 0)
	{
		days += (new_milli/MILLISECONDS_PER_DAY) - 1;
		milliseconds = MILLISECONDS_PER_DAY-((-new_milli)%MILLISECONDS_PER_DAY);
	}
	else
	{
		milliseconds = new_milli;
	}

//	  cout << "final days " << days << " ms " << milliseconds << endl;

	return (*this);
}

// increment UTC time by mod_micro microseconds
void UTCTime::mod_micro (long long mod_micro)
{
//	  cout << "days " << days << " ms " << milliseconds
//	 << " mod_milli " << mod_milli << endl;

	int new_days = days;
	long long new_milli = milliseconds;
	long long new_micro = microseconds;

	new_micro += mod_micro;

	if (new_micro >= 1000)
	{
		new_milli += (new_micro/1000);
		new_micro %= 1000;
	}
	else if (new_micro < 0)
	{
		new_milli--;
		new_milli += (new_micro/1000);
		new_micro = (new_micro%1000)+1000;
	}

//	  cout << "new_milli " << new_milli
//	 << " (days " << (new_milli/static_cast<long long>(MILLISECONDS_PER_DAY)) << ")\n";

	if (new_milli >= MILLISECONDS_PER_DAY)
	{
		new_days += (new_milli/MILLISECONDS_PER_DAY);
		new_milli = new_milli % MILLISECONDS_PER_DAY;
	}

	if (new_milli < 0)
	{
		new_days += (new_milli/MILLISECONDS_PER_DAY) - 1;
		new_milli = MILLISECONDS_PER_DAY-((-new_milli)%MILLISECONDS_PER_DAY);
	}

	days = new_days;
	milliseconds = new_milli;
	microseconds = new_micro;

//	  cout << "final days " << days << " ms " << milliseconds << endl;
}

int64 UTCTime::diff_micro(const UTCTime& other) const
{
	long long days_diff = static_cast<long long>(days) - static_cast<long long>(other.days);
	long long milli_diff = static_cast<long long>(milliseconds) - static_cast<long long>(other.milliseconds);
	long long micro_diff = static_cast<long long>(microseconds) - static_cast<long long>(other.microseconds);
	return ((days_diff*MICROSECONDS_PER_DAY)+(milli_diff*MICROSECONDS_PER_MILLISECOND)+micro_diff);
}

UTCTime UTCTime::operator + (int64 arg) const	 // add milliseconds to a UTC time
{
	UTCTime result = (*this);
	result += arg;
	return result;
}


//
//
// OBTTime	(generic bit48 format)
//
//

unsigned int OBTTime::get_obt_coarse(void) const
{
	return obt_coarse;
}

unsigned int OBTTime::get_obt_fine(void) const
{
	return obt_fine;
}

void OBTTime::set_obt_coarse(unsigned int c)
{
	obt_coarse = c&0xffffff;
}

void OBTTime::set_obt_fine(unsigned int f)
{
	obt_fine = f;
}

unsigned int OBTTime::get_shifted_obt_coarse(void) const
{
	return (obt_coarse >> 8);
}

unsigned int OBTTime::get_shifted_obt_fine(void) const
{
	return (((obt_coarse&0xff)<<8) | (obt_fine>>8));
}

unsigned int OBTTime::get_shifted_obt_veryfine(void) const
{
	return (obt_fine & 0xff);
}

long long OBTTime::subtract_shifted(const OBTTime& o) const
{
	unsigned long long a = (static_cast<unsigned long long>(get_shifted_obt_coarse())<<24) | (static_cast<unsigned long long>(get_shifted_obt_fine())<<8) | static_cast<unsigned long long>(get_shifted_obt_veryfine());
	unsigned long long b = (static_cast<unsigned long long>(o.get_shifted_obt_coarse())<<24) | (static_cast<unsigned long long>(o.get_shifted_obt_fine())<<8) | static_cast<unsigned long long>(o.get_shifted_obt_veryfine());
	return (static_cast<long long>(a) - static_cast<long long>(b));
}

string OBTTime::shifted_to_string(void) const
{
	return ntos(get_shifted_obt_coarse()) + "/" + ntos(get_shifted_obt_fine()) + "/" + ntos(get_shifted_obt_veryfine());
}

textoutstream_type& operator << (textoutstream_type& os, const OBTTime aad)
{
	os << "Seconds: " << aad.get_obt_coarse()
	   << " Seconds*2^-16: " << aad.get_obt_fine();
	return os;
}

long long OBTTime::operator - (const OBTTime& o) const
{
	unsigned long long a = ((static_cast<unsigned long long>(get_obt_coarse())<<16) | get_obt_fine());
	unsigned long long b = ((static_cast<unsigned long long>(o.get_obt_coarse())<<16) | (o.get_obt_fine()));
	return (static_cast<long long>(a) - static_cast<long long>(b));
}

OBTTime OBTTime::operator + (long long o) const
{
//	  cout << get_obt_coarse() << " " << get_obt_fine() << endl;
	unsigned long long temp = (static_cast<unsigned long long>(get_obt_coarse())<<16) +
		static_cast<unsigned long long>(get_obt_fine());
	//	unsigned long long temp2 = temp;
	temp += o;
//	  OBTTime result;
//	  result.obt_coarse = (temp >> 16) & 0xffffff;
//	  result.obt_fine = temp & 0xffff;
//	  cout << "OBTTime::operator plus this " << (*this) << " mod " << o << endl
//	 << " result " << result << endl
//	 << " temp " << temp << " temp2 " << temp2 << end
	return make_obt_time(temp>>16, temp&0xffff);
}

OBTTime OBTTime::operator - (long long o) const
{
	return ((*this)+(-o));
}

bool OBTTime::operator > (const OBTTime& o) const
{
	if (obt_coarse!=o.obt_coarse)
	{
		return (obt_coarse>o.obt_coarse);
	}
	else
	{
		return (obt_fine>o.obt_fine);
	}
}

bool OBTTime::operator != (const OBTTime& o) const
{
	return (obt_coarse!=o.obt_coarse || obt_fine!=o.obt_fine);
}

OBTTime make_obt_time(unsigned int _obt_coarse, unsigned int _obt_fine)
{
	OBTTime result;
	result.obt_coarse = _obt_coarse;
	result.obt_fine = _obt_fine;
	return result;
}

void OBTTime::normalise(void)
{
	obt_coarse %= (1<<24);
}

//
//
// CCSDS
//
//

CCSDS::CCSDS(void)
	: data(NULL)
	, owner(false)
{
//		cout << "ccsds default constructor\n";
}

CCSDS::CCSDS(int length)
//	  : data(reinterpret_cast<GEN_CCSDS*>(new byte[length]))
	: data(new byte[length]),
	  owner(true)
{
//		cout << "ccsds len constructor\n";
}

CCSDS::CCSDS(byte* source, bool deep_copy)
{
	if (deep_copy)
	{
//	int length = (reinterpret_cast<GEN_CCSDS*>(source)->
//			  header.get_packet_length())+7;
		int length = reinterpret_cast<CCSDSPrimaryHeader*>(source)->
			get_packet_length()+7;
		data = new byte[length];
		memcpy(data, source, length);

		owner = true;
	}
	else
	{
		data = source;
		owner = false;
	}
}

CCSDS::CCSDS(const CCSDS& other)
{
	// deep copy, owns memory
	if (other.get_data())
	{
		int length = other.get_length();
		data = new byte[length];
		memcpy(data, other.get_data(), length);
		owner = true;
	}
	else
	{
		data = NULL;
		owner = false;
	}
}

CCSDS::CCSDS(const CCSDS* other)
{
	// deep copy, owns memory
	int length = other->get_length();
	data = new byte[length];
	memcpy(data, other->get_data(), length);
	owner = true;
}


void CCSDS_SATELLITE::create_sband_output(void) const
{
	s_band.create_text_output(info);
	//packet->create_text_output(info);
}



CCSDS::~CCSDS(void)
{
	if (owner)
	{
		delete[] data;
	}
}

CCSDS& CCSDS::operator = (const CCSDS& other)
{
	// deep copy, take ownership
	int length = other.get_length();
	data = new byte[length];
	memcpy(data, other.get_data(), length);
	owner = true;
	return *this;
}

byte* CCSDS::get_data(void) const
{
	return data;
}

void CCSDS::set_data(unsigned int new_data, unsigned int offset)
{
	uint16_be* data_ptr = reinterpret_cast<uint16_be*>(get_payload() + offset);
	*data_ptr = new_data;
}

void CCSDS::set_byte(uint8 new_data, unsigned int offset)
{
	data[offset] = new_data;
}
int CCSDS::get_version_number(void) const
{
	return (get_primary_header()->get_version_number());
}

void CCSDS::set_version_number(unsigned int version) const
{
	get_primary_header()->set_version_number(version);
}

int CCSDS::get_type(void) const
{
	return (get_primary_header()->get_type());
}

void CCSDS::set_type(unsigned int type) const
{
	get_primary_header()->set_type(type);
}

int CCSDS::get_secondary_header_flag(void) const
{
	return (get_primary_header()->get_secondary_header_flag());
}

void CCSDS::set_secondary_header_flag(unsigned int flag) const
{
	get_primary_header()->set_secondary_header_flag(flag);
}

APID CCSDS::get_apid(void) const
{
	return static_cast<APID>(get_primary_header()->get_apid());
}

void CCSDS::set_apid(unsigned int apid) const
{
	get_primary_header()->set_apid(apid);
}

int CCSDS::get_sequence_flags(void) const
{
	return (get_primary_header()->get_sequence_flags());
}

void CCSDS::set_sequence_flags(unsigned int flag) const
{
	get_primary_header()->set_sequence_flags(flag);
}

CCSDSSequenceCount CCSDS::get_sequence_count(void) const
{
	return static_cast<CCSDSSequenceCount>(
		get_primary_header()->get_sequence_count());
}

void CCSDS::set_sequence_count(CCSDSSequenceCount sc)
{
	get_primary_header()->set_sequence_count(sc);
}

int CCSDS::get_length(void) const
{
	return (get_packet_length() + 7);
}

void CCSDS::set_packet_length(unsigned int pl) const
{
	get_primary_header()->set_packet_length(pl);
}

int CCSDS::get_packet_length(void) const
{
	return (get_primary_header()->get_packet_length());
}

int CCSDS::get_utc_days(void) const
{
	return (get_secondary_header()->get_utc_days());
}

void CCSDS::set_utc_days(unsigned int days)
{
	get_secondary_header()->set_utc_days(days);
}

int CCSDS::get_utc_milliseconds(void) const
{
	return (get_secondary_header()->get_utc_milliseconds());
}

void CCSDS::set_utc_milliseconds(unsigned int milliseconds)
{
	get_secondary_header()->set_utc_milliseconds(milliseconds);
}

int CCSDS::get_utc_microseconds(void) const
{
	return (get_secondary_header()->get_utc_microseconds());
}

void CCSDS::set_utc_microseconds(unsigned int microseconds)
{
	get_secondary_header()->set_utc_microseconds(microseconds);
}

byte* CCSDS::get_gras_data() const
{
	return reinterpret_cast<byte*>(get_ancillary_data());
}

unsigned int CCSDS::get_obt_coarse(void) const
{
	return (get_ancillary_data()->get_obt_coarse());
}

void CCSDS::set_obt_coarse(unsigned int c)
{
	get_ancillary_data()->set_obt_coarse(c);
}

unsigned int CCSDS::get_obt_fine(void) const
{
	return (get_ancillary_data()->get_obt_fine());
}

void CCSDS::set_obt_fine(unsigned int f)
{
	get_ancillary_data()->set_obt_fine(f);
}

unsigned int CCSDS::get_shifted_obt_coarse(void) const
{
	return (get_ancillary_data()->get_shifted_obt_coarse());
}

unsigned int CCSDS::get_shifted_obt_fine(void) const
{
	return (get_ancillary_data()->get_shifted_obt_fine());
}

unsigned int CCSDS::get_shifted_obt_veryfine(void) const
{
	return (get_ancillary_data()->get_shifted_obt_veryfine());
}

bool CCSDS::has_secondary_header(void) const
{
	// note: we don't trust the 2ndary header flag since
	// people don't seem to set it properly
	if (get_apid()==APID_MHS && get_length()==REDUCED_MHS_LENGTH)
	{
		return false;
	}
	else
	{
		return true;
	}
}

byte* CCSDS::get_payload(int offset) const
{
	if (has_secondary_header())
	{
		return (data+sizeof(CCSDSPrimaryHeader)+sizeof(UTCTime)+
				sizeof(OBTTime)+offset);
	}
	else
	{
		return (data+sizeof(CCSDSPrimaryHeader)+sizeof(OBTTime)+
				offset);
	}
}

int CCSDS::get_payload_length(void) const
{
	if (has_secondary_header())
	{
		return (get_length()-22);
	}
	else
	{
		return (get_length()-14);
	}
}

int CCSDS::get_error_control(void) const
{
	byte* end_payload = get_payload() + get_payload_length();
	return (*(reinterpret_cast<uint16_be*>(end_payload)));
}

void CCSDS::set_error_control(unsigned int ec)
{
	uint16_be* end_payload = reinterpret_cast<uint16_be*>(get_payload() +
														  get_payload_length());
	*end_payload = ec;
}

uint16_be CCSDS::calculate_vertical_parity(void) const
{
	uint16_be checksum = 0;
	for(int i=0; i<((get_length()-2)/2); i++)
	{
		checksum ^= ((uint16_be*)get_data())[i];
	}
	return checksum;
}

uint16_be CCSDS::calculate_crc(bool invert_initial_value) const
{
	const int CCITT_CRC_GEN = 0x1021;
	unsigned short crc = invert_initial_value ? 0x0 : 0xffff;

	for(int j=0; j<(get_length()-2); j++)
	{
		unsigned short dataByte = static_cast<unsigned short>(get_data()[j])<< 8;
		for (int i = 8; i > 0; i--)
		{
			if ((dataByte ^ crc) & 0x8000)
			{
				crc = (crc << 1) ^ CCITT_CRC_GEN;
			}
			else
			{
				crc <<= 1 ;
			}
			dataByte <<= 1;
		}
	}
	return crc;
}

uint16_be CCSDS::calculate_error_control(void) const
{
	if (get_apid()<=6 || (get_apid()>=448 && get_apid()<=511))
	{
		// GRAS, satellite and nav packets have PEC set to zero
		return 0;
	}
	else
	{
		if ((get_apid()>=130 && get_apid()<=180))
		{
			return calculate_crc(false);
		}
		else if (get_apid()==384)
		{
			return calculate_crc(true);
		}
		else
		{
			return calculate_vertical_parity();
		}
	}
}

CCSDSPrimaryHeader* CCSDS::get_primary_header(void) const
{
	return reinterpret_cast<CCSDSPrimaryHeader*>(data);
}

UTCTime* CCSDS::get_secondary_header(void) const
{
	if (has_secondary_header())
	{
		return reinterpret_cast<UTCTime*>(
			data+sizeof(CCSDSPrimaryHeader));
	}
	else
	{
		cerr << "This function is not implemented for Reduced MS inputs\n";
		assert(false);
		return NULL;
	}
}

OBTTime* CCSDS::get_ancillary_data(void) const
{
	if (has_secondary_header())
	{
		return reinterpret_cast<OBTTime*>(
			data+sizeof(CCSDSPrimaryHeader)+sizeof(UTCTime));
	}
	else
	{
		return reinterpret_cast<OBTTime*>(
			data+sizeof(CCSDSPrimaryHeader));
	}
}

