////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds.hpp
//
//	Provides: misc structs
//
//	Purpose: Code for representing source packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsds_defined
#define ccsds_defined

#include "sband.hpp"
#include "common.hpp"

typedef unsigned int CCSDSSequenceCount;
typedef unsigned int APID;

string uint16tosfloat(uint16_be num);
string uint16tofloat(uint16_be msw, uint16_be lsw);

const unsigned int APID_MHS = 34;
const unsigned int APID_SEM = 37;
const unsigned int APID_HIRS = 38;
const unsigned int APID_AMSUA_1 = 39;
const unsigned int APID_AMSUA_2 = 40;
const unsigned int APID_AVHRR_3A = 103;
const unsigned int APID_AVHRR_3B = 104;
const unsigned int APID_GRAS_MEASUREMENT_1 = 448;  //<== also = CCSDS_GRAS_NAV::APID
const unsigned int APID_GRAS_MEASUREMENT_2 = 480;  //<== also = CCSDS_GRAS::APID
const unsigned int APID_GRAS_NAVIGATION_1 = 2;
const unsigned int APID_GRAS_NAVIGATION_2 = 3;
const unsigned int APID_IASI_PX1 = 130;
const unsigned int APID_IASI_PX2 = 135;
const unsigned int APID_IASI_PX3 = 140;
const unsigned int APID_IASI_PX4 = 145;
const unsigned int APID_IASI_IPx = 150;
const unsigned int APID_IASI_VPx = 160;
const unsigned int APID_IASI_AP = 180;
const unsigned int APID_ASCAT_FIRST = 208;
const unsigned int APID_ASCAT_LAST = 256;
const unsigned int APID_GOME = 384;
const unsigned int APID_SAT = 1;
const unsigned int APID_REDUCED = 3;
const unsigned int APID_NAV = 2;
const unsigned int APID_ADMIN = 6;

const int REDUCED_MHS_LENGTH = 1300;

const int MAX_UTC_DAY = 1<<16;
const int MAX_UTC_MILLISECOND = 86400000;
const int MAX_UTC_MICROSECOND = 1000;
const int MAX_OBT_FINE = 1<<16;
const int MAX_OBT_COARSE = 1<<24;
const long long MAX_MHS_OBT_COARSE = 1LL<<32;

const long long OBT_DIFF_ROLLOVER_THRESHOLD = 65000LL*static_cast<long long>(MAX_OBT_FINE);

struct CCSDSPrimaryHeader
{
	byte data[6];

	unsigned int get_byte(int pos) const;
	unsigned int get_version_number(void) const;
	unsigned int get_type(void) const;
	unsigned int get_secondary_header_flag(void) const;
	unsigned int get_apid(void) const;
	unsigned int get_sequence_flags(void) const;
	unsigned int get_sequence_count(void) const;
	unsigned int get_packet_length(void) const;
	void set_byte(int pos, unsigned int value);
	void set_version_number(unsigned int vn);
	void set_type(unsigned int t);
	void set_secondary_header_flag(unsigned int shf);
	void set_apid(unsigned int apid);
	void set_sequence_flags(unsigned int sf);
	void set_sequence_count(CCSDSSequenceCount sc);
	void set_packet_length(unsigned int pl);
};

struct UTCTime
{
	enum { SIZE = 8 };

	uint16_be days PACKED;
	uint32_be milliseconds PACKED;
	uint16_be microseconds PACKED;

	bool operator < (const UTCTime& rhs) const;
	bool operator > (const UTCTime& rhs) const;
	bool operator != (const UTCTime& rhs) const;
	long long operator - (const UTCTime& rhs) const;
	UTCTime operator + (int64 arg) const;	 //	 add milliseconds to a UTC time
	UTCTime& operator += (long long mod_milli);
	void mod_micro (long long mod_micro);
	long long diff_micro(const UTCTime& other) const;
	unsigned int get_utc_days(void) const;
	unsigned int get_utc_milliseconds(void) const;
	unsigned int get_utc_microseconds(void) const;
	void set_utc_days(unsigned int days);
	void set_utc_milliseconds(unsigned int milliseconds);
	void set_utc_microseconds(unsigned int microseconds);
};

UTCTime make_utc_time(unsigned int _days = 0,
					  unsigned int _milliseconds = 0,
					  unsigned int _microseconds = 0);

textoutstream_type& operator << (textoutstream_type& os, const UTCTime& msh);

struct OBTTime
{
	enum { SIZE = 6 };
	enum { FINE_TICKS_PER_COARSE_TICK = 65536 };  // (2^16)

	uint32_be obt_coarse PACKED;
	uint16_be obt_fine PACKED;

	unsigned int get_obt_coarse(void) const;
	unsigned int get_obt_fine(void) const;
	void set_obt_coarse(unsigned int c);
	void set_obt_fine(unsigned int f);

	unsigned int get_shifted_obt_coarse(void) const;
	unsigned int get_shifted_obt_fine(void) const;
	unsigned int get_shifted_obt_veryfine(void) const;
	long long subtract_shifted(const OBTTime& o) const;

	long long operator - (const OBTTime& o) const;
	bool operator > (const OBTTime& o) const;
	bool operator != (const OBTTime& o) const;
	OBTTime operator + (long long mod) const; // argument is in fine ticks
	OBTTime operator - (long long mod) const;

	string shifted_to_string(void) const;
	void normalise(void); // modulate coarse time with 2^24
};

OBTTime make_obt_time(unsigned int _obt_coarse = 0,
					  unsigned int _obt_fine = 0);

textoutstream_type& operator << (textoutstream_type& os, const OBTTime aad);

// helper structure used by binary CCSDS class
struct GEN_CCSDS
{
	CCSDSPrimaryHeader header PACKED;
	UTCTime secondary_header PACKED;
	OBTTime ancillary_data PACKED;
	byte payload PACKED;
};

// pure binary, non instrument specific source packet

class CCSDS
{
public:
	CCSDS(void);

	// warning: dangerous constructor.
	// get_length() and copy operations won't work
	// until the Packet Primary Header 'size' field has
	// been set
	CCSDS(int length);
	CCSDS(byte* source, bool deep_copy);
	CCSDS(const CCSDS& other);
	CCSDS(const CCSDS* other);
	~CCSDS(void);
	CCSDS& operator = (const CCSDS& other);

	byte* get_data(void) const;
	void set_data(unsigned int new_data, unsigned int offset);
	void set_byte(uint8 new_data, unsigned int offset);

	int get_version_number(void) const;
	void set_version_number(unsigned int version) const;

	int get_type(void) const;
	void set_type(unsigned int type) const;

	int get_secondary_header_flag(void) const;
	void set_secondary_header_flag(unsigned int flag) const;

	APID get_apid(void) const;
	void set_apid(unsigned int apid) const;

	int get_sequence_flags(void) const;
	void set_sequence_flags(unsigned int flag) const;

	CCSDSSequenceCount get_sequence_count(void) const;

	void set_sequence_count(CCSDSSequenceCount sc);

	// get_length returns the actual total length of the packet
	int get_length(void) const;

	int get_packet_length(void) const;
	void set_packet_length(unsigned int pl) const;

	UTCTime* get_secondary_header(void) const;

	int get_utc_days(void) const;
	void set_utc_days(unsigned int days);

	int get_utc_milliseconds(void) const;
	void set_utc_milliseconds(unsigned int milliseconds);

	int get_utc_microseconds(void) const;
	void set_utc_microseconds(unsigned int microseconds);

	bool has_secondary_header(void) const;
	OBTTime* get_ancillary_data(void) const;
	byte* get_gras_data() const;

	unsigned int get_obt_coarse(void) const;
	void set_obt_coarse(unsigned int c);

	unsigned int get_obt_fine(void) const;
	void set_obt_fine(unsigned int f);

	unsigned int get_shifted_obt_coarse(void) const;
	unsigned int get_shifted_obt_fine(void) const;
	unsigned int get_shifted_obt_veryfine(void) const;

	byte* get_payload(int offset = 0) const;
	int get_payload_length(void) const;

	int get_error_control(void) const;
	void set_error_control(unsigned int ec);

	uint16_be calculate_error_control(void) const;
	uint16_be calculate_vertical_parity(void) const;
	uint16_be calculate_crc(bool invert_initial_value) const;

private:
	byte* data;
	bool owner;

	CCSDSPrimaryHeader* get_primary_header(void) const;
};

//
//
// Instrument specific CCSDS Application Data
//
//


//
// AVHRR payload
//

struct AVHRRPayload
{
	enum { SIZE = 12944 };

//	  struct Sample
//	  {
//	uint16_be channel[5];
//	unsigned int channel_1 : 10;
//	unsigned int channel_2 : 10;
//	unsigned int channel_3 : 10;
//	unsigned int channel_4 : 10;
//	unsigned int channel_5 : 10;
//	  };

//	  Sample space_view[10];
//	  Sample ramp_calibration;
//	  Sample earth_view[2048];
//	  Sample ir_target_temp;
//	  Sample patch_temp;
//	  Sample ir_back_scan[10];

	byte data[12944];
};

//
// AMSU-A1 payload
//

struct AMSUA1MeasurementScienceData
{
	enum { SIZE = 1240 };

	uint8 sync_sequence[3];
	uint8 serial_number;
	uint8 digital_housekeeping[4];

	struct
	{
		uint16_be first_reading[2];
		uint16_be second_reading[2];
		uint16_be counts[13];
	} scene[30];

	struct
	{
		uint16_be first_reading[2];
		uint16_be second_reading[2];
		uint16_be counts_1[13];
		uint16_be counts_2[13];
	} cold_calibration;

	uint16_be temperature_sensors[45];
	uint16_be reference_voltage;

	struct
	{
		uint16_be first_reading[2];
		uint16_be second_reading[2];
		uint16_be counts_1[13];
		uint16_be counts_2[13];
	} warm_calibration;
};

struct AMSUA1UniqueScienceData
{
	enum { SIZE = 1120 };

	uint8 sync_sequence[3];
	uint8 serial_number;
	uint8 digital_housekeeping[4];

	struct
	{
		uint16_be first_reading[2];
		uint16_be second_reading[2];
		uint16_be counts[13];
	} scene[30];

	uint16_be temperature_sensors[45];
	uint16_be reference_voltage;
};

//
// AMSU-A2 payload
//

struct AMSUA2MeasurementScienceData
{
	enum { SIZE = 312 };

	uint8 sync_sequence[3];
	uint8 serial_number;
	uint8 digital_housekeeping[4];

	struct
	{
		uint16_be first_reading;
		uint16_be second_reading;
		uint16_be counts[2];
	} scene[30];

	struct
	{
		uint16_be first_reading;
		uint16_be second_reading;
		uint16_be counts_1[2];
		uint16_be counts_2[2];
	} cold_calibration;

	uint16_be temperature_sensors[19];
	uint16_be reference_voltage;

	struct
	{
		uint16_be first_reading;
		uint16_be second_reading;
		uint16_be counts_1[2];
		uint16_be counts_2[2];
	} warm_calibration;
};

struct AMSUA2UniqueScienceData
{
	enum { SIZE = 288 };

	uint8 sync_sequence[3];
	uint8 serial_number;
	uint8 digital_housekeeping[4];

	struct
	{
		uint16_be first_reading;
		uint16_be second_reading;
		uint16_be counts[2];
	} scene[30];

	uint16_be temperature_sensors[19];
	uint16_be reference_voltage;
};

//
// MHS payload
//

struct MHSPayload
{
	enum { SIZE = 1286 };

	struct Sample
	{
		uint16_be shaft_pos;
		uint16_be channels[5];
	};

	uint8 mode;
	uint8 telecommand[5];
	uint8 switch_status[3];
	uint8 temperature[24];
	uint8 current[6];
	uint8 status_word;
	uint8 signal_status[9];

	Sample earth_view[90];
	Sample space_view[4];
	Sample obct_view[4];

	uint16_be prt[5];
	uint16_be prt_cal[3];

	byte space[45];
};

//
// HIRS payload
//

struct HIRSPayload
{
	enum { SIZE = 2304 };

	struct Element
	{
		byte data[36];
	} elements[64];
};

//
//
// Instrument Specific Source Packets
//
//


//
// SATELLITE Source Packet
//

struct CCSDS_SATELLITE
{
	enum { SIZE = 544 };
	enum { APID = 1 };

//	   CCSDS_SATELLITE(void);
//	   ~CCSDS_SATELLITE(void);
//	   void create_sband_output(void) const;

	CCSDSPrimaryHeader primary_header PACKED; //6
	UTCTime secondary_header PACKED; //8
	OBTTime ancillary_data PACKED; //6
	SBand s_band PACKED; //512
	byte filler[12] PACKED; //12

	void create_sband_output(void) const;

};



//
// AVHRR Source Packet
//

struct CCSDS_AVHRR
{
	enum { SIZE = 12966 };
	enum { APID = 103 };

	CCSDSPrimaryHeader primary_header PACKED;
	UTCTime secondary_header PACKED;
	OBTTime ancillary_data PACKED;
	AVHRRPayload payload PACKED;
	uint16_be error_control PACKED;
};

struct CCSDS_AMSUA1
{
	enum { SIZE = 2102 };
	enum { APID = 39 };

	// size 6
	CCSDSPrimaryHeader primary_header PACKED;
	// size 8
	UTCTime secondary_header PACKED;
	// size 6
	OBTTime ancillary_data PACKED;
	// size 2080
	byte payload[2080] PACKED;
	// size 2
	uint16_be error_control PACKED;
};

struct CCSDS_AMSUA2
{
	enum { SIZE = 1142 };
	enum { APID = 40 };

	CCSDSPrimaryHeader primary_header PACKED;
	UTCTime secondary_header PACKED;
	OBTTime ancillary_data PACKED;
	byte payload[1120] PACKED;
	uint16_be error_control PACKED;
};

struct CCSDS_MHS
{
	enum { SIZE = 1308 };
	enum { APID = 34 };

	CCSDSPrimaryHeader primary_header PACKED;
	UTCTime secondary_header PACKED;
	OBTTime ancillary_data PACKED;
	MHSPayload payload PACKED;
	uint16_be error_control PACKED;
};

struct CCSDS_HIRS
{
	enum { SIZE = 2326 };
	enum { APID = 38 };

	CCSDSPrimaryHeader primary_header PACKED;
	UTCTime secondary_header PACKED;
	OBTTime ancillary_data PACKED;
	HIRSPayload payload PACKED;
	uint16_be error_control PACKED;
};



//
// IASI Spectrum Source Packet Ancillary Data
//

struct IASI_Spec_AncillaryData
{
	enum { SIZE = 306 };

	//SBT at Time Tag
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine PACKED; //2
	uint16_be word[150]; //300

};



struct CCSDS_IASI_Spec_PX1
{
	enum { SIZE = 323 };
	enum { APID = 130 };

	CCSDSPrimaryHeader primary_header PACKED; //6
	UTCTime secondary_header PACKED; //8
	IASI_Spec_AncillaryData ancillary_data PACKED; //306
	byte payload PACKED; //1
	//IASI_Spec_Payload payload PACKED; //
	uint16_be error_control PACKED; //2
};



//
// IASI Image Source Packet Ancillary Data
//

struct IASI_Image_AncillaryData
{
	enum { SIZE = 36 };

	//SBT at Time Tag
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine PACKED; //2
	uint16_be word[15]; //30

};


struct CCSDS_IASI_Image
{
	enum { SIZE = 53 };
	enum { APID = 150 };

	CCSDSPrimaryHeader primary_header PACKED; //6
	UTCTime secondary_header PACKED; //8
	IASI_Image_AncillaryData ancillary_data PACKED; //36
	byte payload PACKED; //1
	uint16_be error_control PACKED; //2
};



//
// IASI Verification Source Packet Ancillary Data
//

struct IASI_Verif_AncillaryData
{
	enum { SIZE = 66 };

	//SBT at Time Tag
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine PACKED; //2
	uint16_be word[30]; //60

};


struct CCSDS_IASI_Verif
{
	enum { SIZE = 83 };
	enum { APID = 160 };

	CCSDSPrimaryHeader primary_header PACKED; //6
	UTCTime secondary_header PACKED; //8
	IASI_Verif_AncillaryData ancillary_data PACKED; //66
	byte payload PACKED; //1
	uint16_be error_control PACKED; //2
};



//
// IASI Ancillary Source Packet Ancillary Data
//

struct IASI_Anc_AncillaryData
{
	enum { SIZE = 126 };

	//SBT at Time Tag
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine PACKED; //2
	uint16_be word[60] PACKED; //120

};



//
// IASI Ancillary Source Packet Payload
//

struct IASI_Anc_Payload
{
	enum { SIZE = 640 };
	uint16_be word[320] PACKED;
};



struct CCSDS_IASI_Anc
{
	enum { SIZE = 782 };
	enum { APID = 180 };

	CCSDSPrimaryHeader primary_header PACKED; //6
	UTCTime secondary_header PACKED; //8
	IASI_Anc_AncillaryData ancillary_data PACKED; //126
	IASI_Anc_Payload payload PACKED; //640
	uint16_be error_control PACKED; //2
};

//
// GOME Source Packet Payload
//

struct GOME_FPA
{
	uint16_be indicator;
	uint16_be data[1024];
};

struct CCSDS_GOME
{
	enum { SIZE = 18738 };
	enum { APID = 384 };

	CCSDSPrimaryHeader primary_header PACKED; // 0
	UTCTime utc_time PACKED; // 6
	OBTTime obt_time PACKED; // 14
	uint16_be hk[478] PACKED; // 20
	uint16_be pmd[680] PACKED; // 976
	GOME_FPA fpa[2][4] PACKED; // 2336
	uint16_be error_control PACKED; // 18736
	// 18738
};

//
// ASCAT Nominal Source Packet Ancillary Data
//

struct ASCAT_NOM_AncillaryData
{
	enum { SIZE = 132 };

	//SBT at Time Tag
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine PACKED; //2

	uint16_be pri_count_time_tag PACKED;
	uint8 tag_field PACKED;
	uint8 ground_processor_flags PACKED;
	uint16_be pri_count PACKED;
	uint16_be soft_config PACKED;
	uint16_be param_config PACKED;
	uint16_be spare PACKED;
	uint16_be inst_config PACKED;
	uint16_be SFE_Temp[6] PACKED; //12
	uint16_be Antenna_Temp[12] PACKED; //24
	uint16_be receiver_gain PACKED;
	uint16_be out_of_range_count PACKED;
	uint16_be int_trans_power[4] PACKED; //8
	uint16_be int_refl_power[4] PACKED; //8
	uint16_be int_calib_power[4] PACKED; //8
	uint16_be calib_power[24] PACKED; //48

};


//
// ASCAT Nominal Source Packet Payload
//


struct ASCAT_NOM_Payload
{
	enum { SIZE = 512 };
	uint16_be data[256] PACKED;
};



struct CCSDS_ASCAT_NOM
{
	enum { SIZE = 660 };
	enum { APID = 192 }; //APID = 192 to 255

	CCSDSPrimaryHeader primary_header PACKED; //6
	UTCTime secondary_header PACKED; //8
	ASCAT_NOM_AncillaryData ancillary_data PACKED; //132
	ASCAT_NOM_Payload payload PACKED; //512
	// byte payload PACKED; //1
	uint16_be error_control PACKED; //2
};



//
// ASCAT Raw Source Packet Ancillary Data
//

struct ASCAT_RAW_AncillaryData
{
	enum { SIZE = 72 };

	//SBT at Time Tag
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine PACKED; //2

	uint16_be pri_count_time_tag PACKED;
	uint8 tag_field PACKED;
	uint8 ground_processor_flags PACKED;
	uint16_be pri_count PACKED;
	uint16_be soft_config PACKED;
	uint16_be param_config PACKED;
	uint16_be spare PACKED;
	uint16_be inst_config PACKED;
	uint16_be SFE_Temp[6] PACKED; //12
	uint16_be Antenna_Temp[12] PACKED; //24
	uint16_be receiver_gain PACKED;
	uint16_be out_of_range_count PACKED;
	uint16_be int_trans_power PACKED;
	uint16_be int_refl_power PACKED;
	uint16_be int_calib_power PACKED;
	uint16_be echo_samples PACKED;
	uint16_be calib_samples PACKED;
	uint16_be noise_samples PACKED;
};




struct CCSDS_ASCAT_RAW
{
	enum { SIZE = 89 };
	enum { APID = 192 }; //APID = 192 to 255

	CCSDSPrimaryHeader primary_header PACKED; //6
	UTCTime secondary_header PACKED; //8
	ASCAT_RAW_AncillaryData ancillary_data PACKED; //72
	byte payload PACKED; //1
	uint16_be error_control PACKED; //2
};



//
// GRAS Source Packet Ancillary Data
//

struct GRAS_MEASUREMENT_AncillaryData
{
	enum { SIZE = 10 };

	//SBT at Time Tag
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine	 PACKED; //2

	uint16_be len PACKED; //2
	uint16_be inf PACKED; //2
};


struct CCSDS_GRAS_MEASUREMENT
{
	enum { SIZE = 24 };
	//enum { APID = 448,480 };

	CCSDSPrimaryHeader primary_header	PACKED; //6
	UTCTime			   secondary_header PACKED; //8

	GRAS_MEASUREMENT_AncillaryData ancillary_data PACKED; //10
};


// New GRAS Stuff

struct GRAS_NAVIGATION_Basic_Block
{
	enum { SIZE = 26 };

	//OBT43Time	 OBT_NSM			PACKED; //6
	uint24_be	  OBT_coarse		 PACKED;
	uint16_be	  OBT_fine			 PACKED;
	uint8	   OBT_very_fine_NSM  PACKED;

	UTCTime	   UTC				  PACKED; //8
	int32_be	  POS_X				 PACKED; //4
	int32_be	  POS_Y				 PACKED; //4
	int32_be	  POS_Z				 PACKED; //4
};


struct GRAS_NAVIGATION_Incremental_Block
{
	enum { SIZE = 16 };

	//OBT27Time	 DOBT_NSM			 PACKED; //4
	uint8	   DOBT_coarse		   PACKED;
	uint16_be	  DOBT_fine			  PACKED;
	uint8	   DOBT_very_fine_NSM  PACKED;

	uint24_be	  DUTC				  PACKED; //3
	int24_be	  DX				  PACKED; //3
	int24_be	  DY				  PACKED; //3
	int24_be	  DZ				  PACKED; //3
	//byte		 data[12]			 PACKED;
};


struct CCSDS_GRAS_NAVIGATION_Nominal
{
	enum { SIZE = 160 };
	//enum { APID = 2 };

	CCSDSPrimaryHeader primary_header	PACKED; //6
	UTCTime			   secondary_header PACKED; //8

	// PMC Time
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine	 PACKED; //2

	GRAS_NAVIGATION_Basic_Block		  basic_block		   PACKED; //26
	GRAS_NAVIGATION_Incremental_Block incremental_block[7] PACKED; //112
	byte							  filler[2]			   PACKED; //2
};


struct CCSDS_GRAS_NAVIGATION_Reduced
{
	enum { SIZE = 64 };
	//enum { APID = 3 };

	CCSDSPrimaryHeader primary_header	PACKED; //6
	UTCTime			   secondary_header PACKED; //8

	// PMC Time
	uint32_be obt_coarse PACKED; //4
	uint16_be obt_fine	 PACKED; //2

	GRAS_NAVIGATION_Basic_Block basic_block PACKED; //26
	byte						filler[18]	PACKED; //18
};

#endif
