////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_apid_filter.cpp
//
//	Provides: ccsds_apid_filter executable
//
//	Purpose: Selectively remove CCSDS packets froma stream by checking their
//			 application id.
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "ccsdsapidfilter.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "ccsds_apid_filter";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_file;
	string out_file;
	StringList filter;

	bool verbose = true;
	int prepacket_header = 0;
	bool testfilter = false;

	enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_FILTER, ARGV_PREPACKET_HEADER}
	state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__
				 << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << endl << APPNAME << " " << PACKAGE_VERSION << " built on "
				 << __DATE__
				 << "\nSelectively remove CCSDS packets from a stream by "
				 << "filtering on the Application Id field.\n\n"
				 << "usage:\n" << APPNAME
				 << " [--in|-i filename(s)]"
				 << " [--out|-o filename]"
				 << " [--filter x ...]"
				 << " [--test-filter]"
				 << " [--show-apids]"
				 << " [--prepacket-header x]"
				 << " [-z]"
				 << " [--help|-h]"
				 << " [--version|-v]\n\n";
			return 0;
		}
		else if (!strcmp(argv[x], "--show-apids"))
		{
			cout << "Filter parameter(s) (multiple parameters seperates by "
				 << "spaces) can be a name from the list below or a single "
				 << "number. Some names have multiple APID numbers associated "
				 << "with them, and if these names are used the filter allows "
				 << "multiple APID codes through.\n";

			CCSDSAPIDFilter::display();
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_OUT;
		}
		else if (!strcmp(argv[x], "--filter"))
		{
			state = ARGV_FILTER;
		}
		else if (!strcmp(argv[x], "--test-filter"))
		{
			testfilter = true;
		}
		else if (!strcmp(argv[x], "--prepacket-header"))
		{
			state = ARGV_PREPACKET_HEADER;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else if (!strcmp(argv[x], "--quiet"))
		{
			verbose = false;
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else
		{
			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				in_file.push_back(argv[x]);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file can be specified\n";
					return -1;
				}
				else
				{
					out_file = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_FILTER:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				filter.push_back(string(argv[x]));
				break;

			case ARGV_PREPACKET_HEADER:
				if (prepacket_header != 0)
				{
					error << "More than one --prepacket_header argument "
						  << "found\n";
					return -1;
				}
				else
				{
					prepacket_header = atoi(argv[x]);
					state = ARGV_NONE;
				}
				break;
			}
		}
	}

	if (testfilter)
	{
		CCSDSAPIDFilter x(filter);
		x.output_text_summary();
		return 0;
	}

	// sort out streams
	if (out_file.empty())
	{
		verbose = false;
	}

	if (!verbose)
	{
		spinner->restream(&onull);
	}

	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source,prepacket_header);
	CCSDSAPIDFilter apid_filter(ccsds_reader, filter);

	if (!apid_filter.is_valid())
	{
		error << "Cannot process filter string\n";
		return -1;
	}

	FileConsumer* writer;
	if (out_file.empty())
	{
		writer = new WriteToStdout(&apid_filter);
	}
	else
	{
		writer = new SingleFileWriter(&apid_filter, out_file);
	}

	if (verbose && !gerror)
	{
		first_source->output_text_summary();
		ccsds_reader.output_text_summary();
		apid_filter.output_text_summary();
		writer->output_text_summary();
	}

	delete first_source;
	delete writer;

	return 0;
}

