////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_corrupt_filter.cpp
//
//	Provides: ccsds_corrupt_filter executable
//
//	Purpose: To enable the corruption of CCSDS packet common fields.
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "ccsdsapidfilter.hpp"
#include "readfromstdin.hpp"
#include "ccsdscorruptfilter.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "ccsds_corrupt_filter";

bool decode_input (string *opt, int *val, string argv, string name);

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_file;
	string out_file;

	bool verbose = true;

	StringList selected_apid;
	int skip_val = -1;
	int count_val = -1;
	int sequence_counter_start = -1;
	int sequence_counter_end = -1;
	//double micro_sec_start = -1;
	//double micro_sec_end = -1;

	string version_number_opt = "";
	string type_opt = "";
	string secondary_header_flag_opt = "";
	string apid_opt = "";
	string sequence_flag_opt = "";
	string sequence_counter_opt = "";
	string packet_length_opt = "";
	string utc_days_opt = "";
	string utc_milliseconds_opt = "";
	string utc_microseconds_opt = "";
	string obt_coarse_opt = "";
	string obt_fine_opt = "";
	string pec_opt = "";

	int version_number_val = -1;
	int type_val = -1;
	int secondary_header_flag_val = -1;
	int apid_val = -1;
	int sequence_flag_val = -1;
	int sequence_counter_val = -1;
	int packet_length_val = -1;
	int utc_days_val = -1;
	int utc_milliseconds_val = -1;
	int utc_microseconds_val = -1;
	int obt_coarse_val = -1;
	int obt_fine_val = -1;
	int pec_val = -1;

	int byte_val1 = -1;
	int byte_val2 = 0;
	int byte_offset_start = -1;
	int byte_offset_end = -1;
	string byte_opt = "no";
	string byte_increment = "both";

	short_cds_time start_time;
	int start_time_relative = -1;
	short_cds_time end_time;
	int end_time_relative = -1;

	bool collapse_flag = false;

	int input_ctr = 0;

	enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_SKIP, ARGV_CNT,
		   ARGV_FOR_APID, ARGV_FOR_SEQ_CNT, ARGV_FOR_UTC,
		   ARGV_SET_VERSION, ARGV_SET_TYPE, ARGV_SET_SND_HEADER,
		   ARGV_SET_APID, ARGV_SET_SEQ_FLG, ARGV_SET_SEQ_CNT,
		   ARGV_SET_PACKET_LEN, ARGV_SET_UTC_DAY,
		   ARGV_SET_UTC_MIL_SEC, ARGV_SET_UTC_MIC_SEC,
		   ARGV_SET_OBT_COARSE, ARGV_SET_OBT_FINE, ARGV_SET_PEC,
		   ARGV_BYT_VAL, ARGV_BYT_OFFST, ARGV_BYT_INC } state = ARGV_IN;

	string ccsds_packet_structure =
		string("\n--------------------------------------------------------------------------------------------------\n")
		+ "|				 PACKET PRIMARY HEADER				 |				PACKET DATA FIELD			|\n"
		+ "|					   (6 Bytes)					 |				(Max:65536 Bytes)			|\n"
		+ "--------------------------------------------------------------------------------------------------\n"
		+ "|   Packet Identification	|Packet Seq.   |Packet	 |Secondary|		 Source Data			|\n"
		+ "|							|Control	   |Length	 |Header   |---------------------------|----|\n"
		+ "|							|			   |		 |		   |		  User Data		   |PEC |\n"
		+ "|		 (2 bytes)			|(2 bytes)	   |(2 bytes)|(8 bytes)|						   |	|\n"
		+ "--------------------------------------------------------------------------------------------------\n"
		+ "|Version|Type |Sec.	|APID	|Seq.  |Seq.   |		 |UTC-Time |   Ancillary   |Application|	|\n"
		+ "|Number |Ind. |Header|		|Flags |Counter|		 |Stamp	   |	 Data	   |   Data	   |	|\n"
		+ "|	   |	 |Flag	|		|	   |	   |		 |		   |---------------|		   |	|\n"
		+ "|	   |	 |		|		|	   |	   |		 |		   |OBT |Ancillary |		   |	|\n"
		+ "|	   |	 |		|		|	   |	   |		 |		   |	|Info Field|		   |	|\n"
		+ "|3 bits |1 bit|1 bit |11 bits|2 bits|14 bits|		 |		   |	|		   |		   |	|\n"
		+ "--------------------------------------------------------------------------------------------------\n"
		+ "|		  16 bits			|	16 bits	   | 16 bits | 64 bits | 48 | Variable | Variable  | 16 |\n"
		+ "|							|			   |		 |		   |bits|		   |		   |bits|\n"
		+ "--------------------------------------------------------------------------------------------------\n";
	//0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789
	//0			10		  20		30		  40		50		  60		70		  80		90	   97

	//ccsds_packet_structure = ccsds_packet_structure.replace('-', '_');
	//ccsds_packet_structure = ccsds_packet_structure.replace('|', (char)195);


	string help_msg = string("\nThe ccsds_corrupt_filter enables the corruption of CCSDS packet common fields.")
		+ " Apart from the variable ancillary and application data sections,"
		+ " the remaining sections of CCSDS packets are common to all packet types."
		+ " Additional functionality to corrupt any part of the packet is provided"
		+ " using the --set-byte option (this option must be used wisely to avoid"
		+ " introducing unwanted corruptions).\n\n"
		+ "Usage:\n\n" + APPNAME
		+ " [-i|--in <filename(s)>]"
		+ " [-o|--out <filename>]"
		+ " [-s|--skip <N>]"
		+ " [-c|--count <N>]"
		+ " [For Options]"
		+ " [Set Options]"
		+ " [Byte Options]"
		+ " [-l|--collapse-length]"
		+ " [-a|--show-apids]"
		+ " [-t|--show-ccsds-table]"
		+ " [-z]"
		+ " [-h|--help]"
		+ " [-v|--version]\n\n"
		+ "Note: To avoid unnecessary typing, all the options above have short hand names"
		+ " by which they can be refereed to (-i, -o, -s, -c, -l, -a, -t, -h and -v).\n\n"
		+ "where:\n\n"
		+ "--in <filename(s)> : Allows multiple input files to be specified. Note that"
		+ " multiple input file names should be delimited by spaces. If the '--in'"
		+ " option is omitted, stdin is used by default.\n\n"
		+ "--out <filename> : Allows the name of an output file to be specified. If"
		+ " omitted stdout is used.\n\n"
		+ "--skip <N> : Allows N packets to be skipped before applying any corruptions.\n\n"
		+ "--count <N> : Allows corruptions to be applied to only N packets starting at"
		+ " the first packet or after skipping a number of packets if the --skip option is used.\n\n"
		+ "For Options : The '--for-' options discussed below allow the corruption"
		+ " constraints to be defined and thus allow single or a group of packets"
		+ " to be isolated and corrupted while leaving any packets that do not meet"
		+ " the constraints untouched.\n\n"
		+ "Set Options :  The '--set-' options discussed below allow the exact nature"
		+ " of the corruptions to be defined.\n\n"
		+ "Byte Options: The byte options discussed below are a special case that"
		+ " allow corruptions to be applied to any part of a packet.\n\n"
		+ "--collapse-length : Only applicable if used together with the"
		+ " --set-packet-length option and otherwise ignored. If the"
		+ " --set-packet-length option is used with out this option, the"
		+ " value of the packet length is simply corrupted without effecting"
		+ " the original physical size of the input packet. If however, this option"
		+ " is used, the physical size of the input packet will be either increased"
		+ " or collapsed so that it matches any value the packet length may be set"
		+ " to (this option must be used wisely as it effects the structure of the"
		+ " input packet and will probably make the packet impossible to read or"
		+ " decode using existing tools such EPSView or the MetOpizer ccsds_displayer).\n\n"
		+ "--show-apids : Shows a list of recognised APID types and the APID numbers"
		+ " that they represent. The recognised types or their numeric values can"
		+ " be used with the --for-apid option to only apply corruptions to a select"
		+ " number of packets that match the specified APID constraints.\n\n"
		+ "--show-ccsds-table : Displays a table illustrating the structure"
		+ " of CCSDS packets (Note that the screen must be wide enough for"
		+ " the table to be displayed properly.)\n\n"
		+ "-z : Use simplified animation while processing\n\n"
		+ "--help : Shows usage information.\n\n"
		+ "--version : Shows version information.\n\n";

	string help_msg_for = string("\nFor Options\n-----------\n\n")
		+ "--for-apid <apid(s)> : Allows single or multiple APIDs to be"
		+ " specified either by name or number. Only packets with APIDs"
		+ " matching the ones specified will be corrupted. To obtain a"
		+ " full list of APID names and numbers use the --show-apids option.\n\n"
		+ "--for-sequence-counter <start_ctr> <end_ctr> : Only packets"
		+ " with sequence counters greater or equal to start_ctr and"
		+ " less than or equal to end_ctr will be corrupted.\n\n"
		+ "--for-utc-time <start_time> <end_time> : Only packets with"
		+ " UTC-times greater or equal to start_time and less than or"
		+ " equal to end_time will be corrupted. Start and end times"
		+ " can be specified in two ways: a relative time and an absolute"
		+ " time. The absolute time must be in the format YYYYMMDDhhmmssZ,"
		+ " where the letters respectively represent Years, Months, Days,"
		+ " hours, minutes and seconds (e.g. 20041223151139Z). The relative"
		+ " time can be defined as an offset of +N(ms|s|m), where N"
		+ " represents a number and	 must be followed by units of"
		+ " millisecond(ms), seconds(s) or minutes(m) (e.g. +2m)."
		+ " The relative start time is always relative to the UTC-time of"
		+ " the first packet. The relative end time is a little more"
		+ " complicated. If an absolute start time is followed by a relative"
		+ " end time, then the relative end time will be relative to the"
		+ " absolute start time. If a relative start time is followed by"
		+ " a relative end time, then the relative end time will be relative"
		+ " to the UTC time of the first packet offset by the relative start"
		+ " time. It might sound a little complicated in words, but"
		+ " mathematically its quite simple:\n"
		+ "	  if start_time = A	   then	 start_time = A\n"
		+ "	  if end_time	= +5m  then	 end_time	= A + 5m\n"
		+ "	  if start_time = +2m  then	 start_time = T1 + 2m\n"
		+ "	  if end_time	= +5m  then	 end_time	= T1 + 2m + 5m\n"
		+ "Where A represents a specified absolute time, T1 represents the"
		+ " UTC-time of the first CCSDS packet read and +2m and +5m represent"
		+ " 2 and 5 minute relative time offsets.\n\n";

	string help_msg_set = string("\nSet Options\n-----------\n\n")
		+ "--set-version-number <random|N1|N1 N2>\n"
		+ "--set-type <random|N1|N1 N2>\n"
		+ "--set-secondary-header <random|N1|N1 N2>\n"
		+ "--set-apid <random|N1|N1 N2>\n"
		+ "--set-sequence-flag <random|N1|N1 N2>\n"
		+ "--set-sequence-counter <random|N1|N1 N2>\n"
		+ "--set-packet-length <random|N1|N1 N2>\n"
		+ "--set-utc-day <random|N1|N1 N2>\n"
		+ "--set-utc-millisecond <random|N1|N1 N2>\n"
		+ "--set-utc-microsecond <random|N1|N1 N2>\n"
		+ "--set-obt-coarse <random|N1|N1 N2>\n"
		+ "--set-obt-fine <random|N1|N1 N2>\n"
		+ "--set-pec <off|random|N1|N1 N2>\n\n"
		+ "The functions of the above set options are obvious and do not need"
		+ " to be individually explained. Each set option allows one of the"
		+ " CCSDS packet common fields to be corrupted. Note that three types"
		+ " of corruptions can be introduced:\n\n"
		+ "1. The first type of corruption allows the fields of consecutive"
		+ " CCSDS packets to be set to an appropriate* random number.\n\n"
		+ "2. The second type of corruption allows the fields of consecutive"
		+ " CCSDS packets to be set to an appropriate* single number (N1).\n\n"
		+ "3. The third type of corruption allows the fields of consecutive"
		+ " CCSDS packets to be filled with an appropriate* number series or"
		+ " patter starting with the number N1 and incremented by the number N2."
		+ " For example if N1=0 and N2=2, then consecutive packets will be filled"
		+ " with the numbers 0, 2, 4, 6, ...   If the field has a maximum value"
		+ " that is reached, the pattern or series will simply wrap around"
		+ " and continue. For example if the maximum acceptable value is 7,"
		+ " then the pattern will be: 0, 2, 4, 6, 0,  2, 4, 6, 0, ...  Note"
		+ " that N2 can also be assigned a negative value and thus produce"
		+ " increasing and decreasing number patterns.\n\n"
		+ "The term appropriate* has been used in the above paragraphs to"
		+ " imply that the corruptions applied must be greater or equal to"
		+ " the minimum value the field can accept and of course less than"
		+ " or equal to the maximum value the field can accept. The code"
		+ " attempts to handle all this automatically. If however, the user"
		+ " does input values that are out of range, they will be prompted"
		+ " with error messages and advised to revise their inputs.\n\n"
		+ "The only exception to the above set options is the final --set-pec"
		+ " option. This option has an additional off mode. The Packet Error"
		+ " Control (PEC) field is automatically calculated and updated"
		+ " whenever a corruption is introduced. Of course it can itself"
		+ " be corrupted in the usual three ways above. However, should"
		+ " a need arise to apply some corruptions without updating the"
		+ " PEC, the off mode can be used to simply do nothing and keep"
		+ " the PEC of the initial input CCSDS packet.\n\n";

	string help_msg_byte = string("\nByte Options\n------------\n\n")
		+ "Unlike the set options described above, the byte options allow"
		+ " corruptions to be applied to any part of a CCSDS packet.\n\n"
		+ "--for-byte-offset <N1|N1 N2> :  Corruptions can be introduced"
		+ " to a single byte at byte offset N1 or to a range of bytes"
		+ " starting at byte offset N1 and ending at byte offset N2."
		+ " Note that if the corruption is to be applied to the first byte,"
		+ " the byte offset must be set to zero. Hence, the byte offset"
		+ " must be greater or equal to zero and of course less than the"
		+ " packet length.\n\n"
		+ "--set-byte <random|N1|N1 N2> : Similar to the set options,"
		+ " the --set-byte option allows a single byte per packet or a"
		+ " range of bytes per packet to be set to random values, to be"
		+ " set to a single value N1, or to be filled with a number"
		+ " pattern starting with a value N1 and incremented by a value N2."
		+ " Note that N1 and N2 can not be set to anything greater than 255."
		+ " While N1 can not be less than zero, the increment N2 can be"
		+ " assigned a negative value.\n\n"
		+ "--set-byte-increment <column|row> : This option only becomes"
		+ " relevant if one is attempting to assign a number pattern to a"
		+ " range of bytes per packet. For example a user might want to"
		+ " corrupt byte offsets 101 to 105 (--for-byte-offset 101 105)"
		+ " with a number pattern: 1, 3, 5, 7, 9, ... (--set-byte 1 2)."
		+ " If the --set-byte-increment option is left undefined, the"
		+ " resulting corruption per packet will be as follows:\n\n"
		+ "	  Byte Offset  101	102	 103  104  105\n"
		+ "	  Packet 1	 1	  3	   5	7	 9\n"
		+ "	  Packet 2	 3	  5	   7	9	 11\n"
		+ "	  Packet 3	 5	  7	   9	11	 13\n"
		+ "	  Packet 4	 7	  9	   11	13	 15\n"
		+ "	  Packet 5	 9	  11   13	15	 17\n"
		+ "	  Packet 6	 11	  13   15	17	 19\n\n"
		+ "Note that the pattern has been increment over both column"
		+ " (Byte Offset) and row (Packet). Hence, if the"
		+ " --set-byte-increment option is left undefined, the default"
		+ " option is to apply the pattern over both columns and rows."
		+ " If however, the --set-byte-increment is set to column, the"
		+ " pattern will only be incremented over the different byte"
		+ " offsets and will result into the following corruption:\n\n"
		+ "	  Byte Offset	 101  102  103	104	 105\n"
		+ "	  Packet 1	 1	  3	   5	7	 9\n"
		+ "	  Packet 2	 1	  3	   5	7	 9\n"
		+ "	  Packet 3	 1	  3	   5	7	 9\n"
		+ "	  Packet 4	 1	  3	   5	7	 9\n"
		+ "	  Packet 5	 1	  3	   5	7	 9\n"
		+ "	  Packet 6	 1	  3	   5	7	 9\n\n"
		+ "Similarly, if the --set-byte-increment option is set to row,"
		+ " the pattern will only be incremented over the different packets"
		+ " and will result into the following corruption:\n\n"
		+ "	  Byte Offset	 101  102  103	104	 105\n"
		+ "	  Packet 1	 1	  1	   1	1	 1\n"
		+ "	  Packet 2	 3	  3	   3	3	 3\n"
		+ "	  Packet 3	 5	  5	   5	5	 5\n"
		+ "	  Packet 4	 7	  7	   7	7	 7\n"
		+ "	  Packet 5	 9	  9	   9	9	 9\n"
		+ "	  Packet 6	   11	11	 11	  11   11\n\n";

	string help_msg_example = string("\nExamples\n--------\n\n")
		+ "ccsds_corrupt_filter -i amsua.ccsds -o amsua.ccsds.corrupt"
		+ " --for-apid 39 40 --for-sequence-counter 3 15"
		+ " --set-version-number 1 2 --set-sequence-flag random\n\n"
		+ "ccsds_corrupt_filter -i amsua.ccsds -o amsua.ccsds.corrupt"
		+ " --for-apid amsu-a1 amsu-a2 --for-sequence-counter 3 15"
		+ " --for-utc-time +1m +3m"
		+ " --set-version-number 1 2 --set-sequence-flag random\n\n"
		+ "ccsds_corrupt_filter -i amsua.ccsds -o amsua.ccsds.corrupt"
		+ " --for-apid amsu-a1 amsu-a2 --for-sequence-counter 3 15"
		+ " --for-utc-time 20020227105801Z 20020227105901Z"
		+ " --set-version-number 1 2 --set-sequence-flag random\n\n";

//	   string help_msg = string("\nThe ccsds_corrupt_filter enables the corruption of CCSDS packet common fields.")
//	+ " Apart from the variable ancillary and application data sections,"
//	+ " the remaining sections of CCSDS packets are common to all packet types."
//	+ " Additional functionality to corrupt any part of the packet is provided"
//	+ " using the --set-byte option (this option must be used wisely to avoid"
//	+ " introducing unwanted corruptions).\n\n"
//	+ "usage:\n" + APPNAME
//	+ " [-i|--in <filename(s)>]"
//	+ " [-o|--out <filename>]"
//	+ " [-s|--skip <N>]"
//	+ " [-c|--count <N>]"
//	+ " [--for-apid <apid(s)>]"
//	+ " [--for-sequence-counter <start_ctr> <end_ctr>]"
//	+ " [--for-utc-time <start_time> <end_time>]"
//	+ " [--set-version-number <random|N1|N1 N2>]"
//	+ " [--set-type <random|N1|N1 N2>]"
//	+ " [--set-secondary-header <random|N1|N1 N2>]"
//	+ " [--set-apid <random|N1|N1 N2>]"
//	+ " [--set-sequence-flag <random|N1|N1 N2>]"
//	+ " [--set-sequence-counter <random|N1|N1 N2>]"
//	+ " [--set-packet-length <random|N1|N1 N2>]"
//	+ " [--set-utc-day <random|N1|N1 N2>]"
//	+ " [--set-utc-millisecond <random|N1|N1 N2>]"
//	+ " [--set-utc-microsecond <random|N1|N1 N2>]"
//	+ " [--set-obt-coarse <random|N1|N1 N2>]"
//	+ " [--set-obt-fine <random|N1|N1 N2>]"
//	+ " [--set-pec <off|random|N1|N1 N2>]"
//		+ " [--for-byte-offset <N1|N1 N2>]"
//		+ " [--set-byte <random|N1|N1 N2>]"
//	+ " [--set-byte-increment <column|row>]"
//	+ " [-l|--collapse-length]"
//	+ " [-a|--show-apids]"
//	+ " [-t|--show-ccsds-table]"
//	+ " [-h|--help]"
//	+ " [-v|--version]\n\n";


	for(int x=1; x<argc; x++)
	{
		string arg(argv[x]);

		if (arg == "--version" || arg == "-v")
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__
				 << endl;
			return 0;
		}
		else if (arg == "--help" || arg == "-h")
		{
			cout << help_msg;
			cout << help_msg_for;
			cout << help_msg_set;
			cout << help_msg_byte;
			cout << help_msg_example;
			return 0;
		}
		else if (arg == "--help-for" || arg == "-hf" || arg == "-h-f")
		{
			cout << help_msg_for;
			return 0;
		}
		else if (arg == "--help-set" || arg == "-hs" || arg == "-h-s")
		{
			cout << help_msg_set;
			return 0;
		}
		else if (arg == "--help-byte" || arg == "-hb" || arg == "-h-b")
		{
			cout << help_msg_byte;
			return 0;
		}
		else if (arg == "--help-example" || arg == "-hx" || arg == "-h-x")
		{
			cout << help_msg_example;
			return 0;
		}
		else if (arg == "--show-ccsds-table" || arg == "--show-table" ||
				 arg == "--show-ccsds" || arg == "--table" || arg == "-t")
		{
			cout << ccsds_packet_structure;
			return 0;
		}
		else if (arg == "--in" || arg == "-i")
		{
			state = ARGV_IN;
		}
		else if (arg == "--out" || arg == "-o")
		{
			state = ARGV_OUT;
		}
		else if (arg == "--quiet")
		{
			verbose = false;
		}
		else if (arg == "--debug")
		{
			_debug = &cout;
		}
		else if (arg == "--skip" || arg == "-s")
		{
			state = ARGV_SKIP;
		}
		else if (arg == "--count" || arg == "-c")
		{
			state = ARGV_CNT;
		}
		else if (arg == "--collapse-length" || arg == "--collapse" || arg == "-l")
		{
			collapse_flag = true;
		}
		else if (arg == "--show-apids" || arg == "-a")
		{
			cout << "\nThe option --for-apid accepts multiple input parameters seperates "
				 << "by spaces. These inputs can be a name from the list below or a single "
				 << "number. Some names have multiple APID numbers associated "
				 << "with them. Using such names will have the same result as defining "
				 << "all its associated APID numbers seperately.\n\n";

			CCSDSAPIDFilter::display();
			return 0;
		}
		else if (arg == "--for-apid" || arg == "--filter-apid")
		{
			state = ARGV_FOR_APID;
		}
		else if (arg == "--for-sequence-counter" || arg == "--filter-sequence-counter")
		{
			input_ctr = 0;
			state = ARGV_FOR_SEQ_CNT;
		}
		else if (arg == "--for-utc-time" || arg == "--filter-utc-time")
		{
			input_ctr = 0;
			state = ARGV_FOR_UTC;
		}
		else if (arg == "--set-version-number")
		{
			state = ARGV_SET_VERSION;
		}
		else if (arg == "--set-type")
		{
			state = ARGV_SET_TYPE;
		}
		else if (arg == "--set-secondary-header")
		{
			state = ARGV_SET_SND_HEADER;
		}
		else if (arg == "--set-apid")
		{
			state = ARGV_SET_APID;
		}
		else if (arg == "--set-sequence-flag")
		{
			state = ARGV_SET_SEQ_FLG;
		}
		else if (arg == "--set-sequence-counter")
		{
			state = ARGV_SET_SEQ_CNT;
		}
		else if (arg == "--set-packet-length")
		{
			state = ARGV_SET_PACKET_LEN;
		}
		else if (arg == "--set-utc-day")
		{
			state = ARGV_SET_UTC_DAY;
		}
		else if (arg == "--set-utc-millisecond" || arg == "--set-utc-milliseconds" ||
				 arg == "--set-utc-millisec" || arg == "--set-utc-milli")
		{
			state = ARGV_SET_UTC_MIL_SEC;
		}
		else if (arg == "--set-utc-microsecond" || arg == "--set-utc-microseconds" ||
				 arg == "--set-utc-microsec" || arg == "--set-utc-micro")
		{
			state = ARGV_SET_UTC_MIC_SEC;
		}
		else if (arg == "--set-obt-coarse" || arg == "--set-obt-c")
		{
			state = ARGV_SET_OBT_COARSE;
		}
		else if (arg == "--set-obt-fine" || arg == "--set-obt-f")
		{
			state = ARGV_SET_OBT_FINE;
		}
		else if (arg == "--set-pec")
		{
			state = ARGV_SET_PEC;
		}
		else if (arg == "--set-byte")
		{
			state = ARGV_BYT_VAL;
		}
		else if (arg == "--for-offset" || arg == "--filter-offset" ||
				 arg == "--for-byte-offset" || arg == "--filter-byte-offset" ||
				 arg == "--for-byte" || arg == "--filter-byte")

		{
			state = ARGV_BYT_OFFST;
		}
		else if (arg == "--set-byte-increment" || arg == "--set-byte-inc" ||
				 arg == "--set-increment" || arg == "--set-inc")
		{
			state = ARGV_BYT_INC;
		}
		else if (arg == "-z")
		{
			Spinner::use_simple_spinner();
		}
		else
		{
			switch(state)
			{

			case ARGV_NONE:
				cerr << "Parameter not recognised '" << arg << endl;
				return -1;

			case ARGV_IN:
				in_file.push_back(arg);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file can be specified\n";
					return -1;
				}
				else
				{
					out_file = arg;
					state = ARGV_NONE;
				}
				break;

			case ARGV_SKIP:
				if (skip_val != -1)
				{
					error << "The option --skip only accepts one numeric input\n";
					return -1;
				}
				else
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n";
						return -1;
					}
					skip_val = lexical_cast<int>(arg);
					state = ARGV_NONE;
				}
				break;

			case ARGV_CNT:
				if (count_val != -1)
				{
					error << "The option --count only accepts one numeric input\n";
					return -1;
				}
				else
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n";
						return -1;
					}
					count_val = lexical_cast<int>(arg);
					state = ARGV_NONE;
				}
				break;

			case ARGV_FOR_APID:
				selected_apid.push_back(arg);
				break;

			case ARGV_FOR_SEQ_CNT:
				if (arg.find_first_not_of("0123456789")!=string::npos)
				{
					error << "Unacceptable input '" << arg << "'\n";
					return -1;
				}
				if (sequence_counter_start == -1)
				{
					sequence_counter_start = lexical_cast<int>(arg);
					sequence_counter_end = sequence_counter_start;
					input_ctr++;
				}
				else if (sequence_counter_end == -1 ||
						 sequence_counter_end == sequence_counter_start &&
						 input_ctr < 2)
				{
					sequence_counter_end = lexical_cast<int>(arg);
					input_ctr++;
				}
				else
				{
					error << "The option --for-sequence-counter only accepts two inputs "
						  << "to mark the start and end of the corruptions to be applied.\n";
					return -1;
				}
				break;

			case ARGV_FOR_UTC:
				if (arg.find_first_not_of("0123456789zZ+mMsS")!=string::npos &&
					arg.length()<16)
				{
					error << "Unacceptable input '" << arg << "'\n"
						  << "Format of time not recognised. Use absolute "
						  << "time YYYYMMDDhhmmssZ or relative "
						  << "+<number><ms|s|m>\n";
					return -1;
				}
				if (arg[0]=='+')
				{
					// relative time specified
					int number = TimeConv::duration_to_int(arg.substr(1));
					if (number == -1)
					{
						return -1;
					}
					if (start_time_relative == -1 && input_ctr < 2)
					{
						//cout << "Using relative start time of +" << number << "ms\n";
						start_time_relative = number;
						input_ctr++;
					}
					else if (end_time_relative == -1 && input_ctr < 2)
					{
						//cout << "Using relative end time of +" << number << "ms\n";
						end_time_relative = number;
						input_ctr++;
					}
					else
					{
						error << "The option --for-utc-time only accepts two inputs "
							  << "to mark the start and end of the corruptions to be applied.\n";
						return -1;
					}
				}
				else
				{
					short_cds_time cds = short_cds_time(arg);
					if (cds == short_cds_time())
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "Format of time not recognised. Use absolute "
							  << "time YYYYMMDDhhmmssZ or relative "
							  << "+<number><ms|s|m>\n";
						return -1;
					}
					else if (start_time == short_cds_time()	 && input_ctr < 2)
					{
						start_time = cds;
						input_ctr++;
					}
					else if (end_time == short_cds_time() && input_ctr < 2)
					{
						end_time = cds;
						input_ctr++;
					}
					else
					{
						error << "The option --for-utc-time only accepts two inputs "
							  << "to mark the start and end of the corruptions to be applied.\n";
						return -1;
					}
				}
				break;

			case ARGV_SET_VERSION:
				if (!decode_input(&version_number_opt , &version_number_val, arg, "version-number"))
				{
					return -1;
				}
				break;

			case ARGV_SET_TYPE:
				if (!decode_input(&type_opt , &type_val, arg, "type"))
				{
					return -1;
				}
				break;

			case ARGV_SET_SND_HEADER:
				if (!decode_input(&secondary_header_flag_opt , &secondary_header_flag_val, arg, "secondary-header-flag"))
				{
					return -1;
				}
				break;

			case ARGV_SET_APID:
				if (!decode_input(&apid_opt , &apid_val, arg, "apid"))
				{
					return -1;
				}
				break;

			case ARGV_SET_SEQ_FLG:
				if (!decode_input(&sequence_flag_opt , &sequence_flag_val, arg, "sequence-flag"))
				{
					return -1;
				}
				break;

			case ARGV_SET_SEQ_CNT:
				if (!decode_input(&sequence_counter_opt , &sequence_counter_val, arg, "sequence-counter"))
				{
					return -1;
				}
				break;

			case ARGV_SET_PACKET_LEN:
				if (!decode_input(&packet_length_opt , &packet_length_val, arg, "packet-length"))
				{
					return -1;
				}
				break;

			case ARGV_SET_UTC_DAY:
				if (!decode_input(&utc_days_opt , &utc_days_val, arg, "utc-day"))
				{
					return -1;
				}
				break;

			case ARGV_SET_UTC_MIL_SEC:
				if (!decode_input(&utc_milliseconds_opt , &utc_milliseconds_val, arg, "utc-millisecond"))
				{
					return -1;
				}
				break;

			case ARGV_SET_UTC_MIC_SEC:
				if (!decode_input(&utc_microseconds_opt , &utc_microseconds_val, arg, "utc-microsecond"))
				{
					return -1;
				}
				break;

			case ARGV_SET_OBT_COARSE:
				if (!decode_input(&obt_coarse_opt , &obt_coarse_val, arg, "obt-coarse"))
				{
					return -1;
				}
				break;

			case ARGV_SET_OBT_FINE:
				if (!decode_input(&obt_fine_opt , &obt_fine_val, arg, "obt-fine"))
				{
					return -1;
				}
				break;

			case ARGV_SET_PEC:
				if (!decode_input(&pec_opt , &pec_val, arg, "pec"))
				{
					return -1;
				}
				break;

			case ARGV_BYT_VAL:
				if (arg == "random" && byte_opt == "no")
				{
					byte_opt = "random";
					state = ARGV_NONE;
				}
				else
				{
					if (byte_val1 == -1 && byte_opt != "random")
					{
						if (arg.find_first_not_of("0123456789")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-byte only accepts the inputs "
								  << "random/single number/two numbers separated by a space. "
								  << "For example:\n--set-byte random, "
								  << "\n--set-byte 3, "
								  << "\n--set-byte 2 1\n";
							return -1;
						}
						byte_val1 = lexical_cast<int>(arg);
						if (byte_val1 > 255)
						{
							error << "A byte can not be set to anything greater than 255\n";
							return -1;
						}
						byte_opt = "yes";
					}
					else if (byte_val2 == 0 && byte_val1 != -1 && byte_opt != "random")
					{
						if (arg.find_first_not_of("-0123456789")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-byte only accepts the inputs "
								  << "random/single number/two numbers separated by a space. "
								  << "For example:\n--set-byte random, "
								  << "\n--set-byte 3, "
								  << "\n--set-byte 2 1\n";
							return -1;
						}
						byte_val2 = lexical_cast<int>(arg);
						if (byte_val2 > 255)
						{
							error << "A byte can not be set to anything greater than 255\n";
							return -1;
						}
						byte_opt = "yes";
					}
					else
					{
						error << "The option --set-byte only accepts the inputs "
							  << "random/single number/two numbers separated by a space. "
							  << "For example:\n--set-byte random, "
							  << "\n--set-byte 3, "
							  << "\n--set-byte 2 1\n";
						return -1;
					}
				}
				break;

			case ARGV_BYT_OFFST:
				if (byte_offset_start == -1)
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --for-byte-offset only accepts the inputs "
							  << "single number/two numbers separated by a space. "
							  << "For example:\n--for-byte-offset 28, "
							  << "\n--for-byte-offset 28 48\n";
						return -1;
					}
					byte_offset_start = lexical_cast<int>(arg);
				}
				else if (byte_offset_end == -1 && byte_offset_start != -1)
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --for-byte-offset only accepts the inputs "
							  << "single number/two numbers separated by a space. "
							  << "For example:\n--for-byte-offset 28, "
							  << "\n--for-byte-offset 28 48\n";
						return -1;
					}
					byte_offset_end = lexical_cast<int>(arg);
				}
				else
				{
					error << "The option --for-byte-offset only accepts the inputs "
						  << "single number/two numbers separated by a space. "
						  << "For example:\n--for-byte-offset 28, "
						  << "\n--for-byte-offset 28 48\n";
					return -1;
				}
				break;


			case ARGV_BYT_INC:
				if (arg == "column" || arg == "col" && byte_increment == "both")
				{
					byte_increment = "column";
					state = ARGV_NONE;
				}
				else if (arg == "row" && byte_increment == "both")
				{
					byte_increment = "row";
					state = ARGV_NONE;
				}
				else
				{
					error << "Unacceptable input '" << arg << "'\n"
						  << "The option --set-byte-increment only accepts the inputs "
						  << "column or row. For example:\n--set-byte-increment column, "
						  << "\n--set-byte-increment row\n";
					return -1;
				}
				break;

			}
		}
	}



	if (!verbose)
	{
		spinner->restream(&onull);
	}


	if (byte_opt == "no" && byte_offset_start == -1 && byte_val1 == -1)
	{
		// No byte corruption has been defined and thus leave as is
	}
	else
	{
		if (byte_opt == "random" && byte_offset_start == -1)
		{
			error << "NOTE: The options --for-byte-offset and --set-byte must be used together\n";
			return -1;
		}
		else if (byte_offset_start == -1 || byte_val1 == -1 && byte_opt == "yes")
		{
			error << "NOTE: The options --for-byte-offset and --set-byte must be used together\n";
			return -1;
		}
	}


	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source, 0);
	CCSDSCorruptFilter corrupt(ccsds_reader, skip_val, count_val, selected_apid,
							   start_time, end_time, start_time_relative, end_time_relative,
							   sequence_counter_start, sequence_counter_end,
							   version_number_opt, type_opt, secondary_header_flag_opt,
							   apid_opt, sequence_flag_opt, sequence_counter_opt,
							   packet_length_opt,  utc_days_opt, utc_milliseconds_opt,
							   utc_microseconds_opt, obt_coarse_opt, obt_fine_opt, pec_opt,
							   version_number_val, type_val, secondary_header_flag_val, apid_val,
							   sequence_flag_val, sequence_counter_val, packet_length_val,
							   utc_days_val, utc_milliseconds_val, utc_microseconds_val,
							   obt_coarse_val, obt_fine_val, pec_val, byte_opt, byte_val1,
							   byte_val2, byte_offset_start, byte_offset_end, byte_increment, collapse_flag);
	if (!corrupt.is_valid())
	{
		error << "Exit...\n";
		return -1;
	}

	FileConsumer* writer;
	if (out_file.empty())
	{
		writer = new WriteToStdout(&corrupt);
	}
	else
	{
		writer = new SingleFileWriter(&corrupt, out_file);
	}

	if (verbose && !gerror)
	{
		if (!corrupt.is_valid())
		{
			error << "\n*** WARNING: The output produced might be"
				  << " corrupt or incomplete as errors were detected"
				  << " during the processing.\n\n";
			return -1;
		}
		else
		{
			first_source->output_text_summary();
			ccsds_reader.output_text_summary();
			corrupt.output_text_summary();
			writer->output_text_summary();
		}
	}

	delete first_source;
	delete writer;

	return 0;
}


bool decode_input (string *opt, int *val, string argv, string name)
{
	string error_msg = string ("Unacceptable input '") + argv +	 "'\n";

	if (argv.length() > 10)
	{
		error << "Input too large: "
			  << argv << endl;
		return false;
	}

	if (name == "pec")
	{
		error_msg = error_msg
			+ "The option --set-" + name + " only accepts the inputs "
			+ "off/random/single number/two numbers separated by a space. "
			+ "For example:\n--set-" + name + " off\n"
			+ "--set-" + name + " random\n"
			+ "--set-" + name + " 3\n"
			+ "--set-" + name + " 2 1\n";
	}
	else
	{
		error_msg = error_msg
			+ "The option --set-" + name + " only accepts the inputs "
			+ "random/single number/two numbers separated by a space. "
			+ "For example:\n--set-" + name + " random\n"
			+ "--set-" + name + " 3\n"
			+ "--set-" + name + " 2 1\n";
	}

	if (argv == "random")
	{
		*opt = "random";
	}
	else if (argv == "off" && name == "pec")
	{
		*opt = "off";
	}
	else
	{
		if (*val == -1 && *opt != "random" && *opt != "off")
		{
			if (argv.find_first_not_of("0123456789")!=string::npos)
			{
				error << error_msg;
				return false;
			}
			*val = lexical_cast<int>(argv);
			*opt = "yes";
		}
		else if (*opt == "" || *opt == "yes")
		{
			if (argv.find_first_not_of("-0123456789")!=string::npos)
			{
				error <<  error_msg;
				return false;
			}
			*opt = argv;
		}
		else
		{
			error << "Unrecognised parameter '" << argv << "'. Run '"
				  << APPNAME << " --help' for a list of options\n";
			return false;
		}
	}

	return true;
}
