///////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_demultiplexer.cpp
//
//	Provides: 
//
//	Purpose: 
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "multifilewriter.hpp"
#include "ccsdsdemultiplexer.hpp"

ostream* _debug;
ostream* _info;
ostream* _warning;
ostream* _error;
int gerror = 0;

ofstream onull("/dev/null");
const string APPNAME = "ccsds_demultiplexer";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_file;
	string out_file;

	bool verbose = true;
	int prepacket_header = 0;

	enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_PREPACKET_HEADER } 
	state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__ 
				 << endl << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__ 
				 << endl << endl;

			cout << "usage " << APPNAME
				 << " [--in|-i filename(s)]"
				 << " [--out|-o filename_prefix]"
				 << " [--prepacket-header x]"
				 << " [--verbose]"
				 << " [-z]"
				 << " [--help|-h]"
				 << " [--version|-v]"
				 << "\n\nDisplay a summary of the CCSDS packets in the "
				 << "input file.\n";
		
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_OUT;
		}
		else if (!strcmp(argv[x], "--prepacket-header"))
		{
			state = ARGV_PREPACKET_HEADER;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else if (!strcmp(argv[x], "--verbose"))
		{
			verbose = true;
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else
		{
			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				in_file.push_back(argv[x]);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output prefix can be specified\n";
					return -1;
				}
				else
				{
					out_file = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_PREPACKET_HEADER:
				if (prepacket_header != 0)
				{
					error << "More than one --prepacket_header argument "
						  << "found\n";
					return -1;
				}
				else
				{
					prepacket_header = atoi(argv[x]);
					state = ARGV_NONE;
				}
				break;
			}
		}
	}

	if (out_file.empty())
	{
		error << "Output filename prefix must be specified\n";
		return -1;
	}
	
	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source,prepacket_header);
	CCSDSDemultiplexer demultiplexer(ccsds_reader, false);
	MultiFileWriter writer(ccsds_reader, out_file);

	if (verbose && !gerror)
	{
		first_source->output_text_summary();
		ccsds_reader.output_text_summary();
		demultiplexer.output_text_summary();
		writer.output_text_summary();
	}

	delete first_source;

	return 0;
}

