////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_displayer.cpp
//
//	Provides: ccsds_displayer executable
//
//	Purpose: Display information on single CCSDS packets or streams of packets
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "args.hpp"
//#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "ccsdspacketdisplayer.hpp"
#include "ccsdsstreamdisplayer.hpp"
//#include "textrecorddisplayer.hpp"
//#include "textconfigdisplayer.hpp"
#include "ccsdsimageavhrr.hpp"
#include "ccsdsimageiasi.hpp"
#include "imagewriter.hpp"
#include "multifilewriter.hpp"
#include "ccsdsstatscalculator.hpp"
//#include "statsdisplayer.hpp"
#include "ccsdscountfilter.hpp"
#include "ccsdsshowgaps.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

int gerror = 0; // global error
const string APPNAME = "ccsds_displayer";
const string CONFIG_DIR_VAR = "METOPIZER_HOME";
//const string DEFAULT_CONFIG_DIR = ".metopizer";
const string DEFAULT_CONFIG_FILE = "ccsdsdisplayer.config";
const string DEFAULT_IASI_CONFIG_FILE = "iasi_spectrum.config";
const string MAINTAINER = "elson@eumetsat.int";
const string COPYRIGHT = "Copyright (c) 2005 Eumetsat.";

int main(int argc, char* argv[])
{
	ofstream onull("/dev/null");
	_debug = &onull;
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;

	deque<string> in_file;
	string image_avhrr;
	string image_iasi_iis;
	string config_file;
	string iasi_config_file;

	bool show_packets = false;
	bool verbose = true;
	bool hex = false;
	int prepacket_header = 0;
	bool header_only = false;
	bool check_config = false;
	bool check_iasi_config = false;
#ifndef _NO_JPEG
	bool jpeg = false;
#endif
	bool show_stats = false;
	bool show_gaps = false;
	bool hflip = false;
	bool vflip = false;
	bool invert = false;
	bool iis_trim = false;
	bool show_seq_diffs = false;
	bool unshift_gras_obt = false;
	bool show_viadr = false;

	int count = -1;
	int skip = 0;
	deque<int> select_utc_gaps;
	deque<int> select_grh_gaps;
	deque<int> select_obt_gaps;

	Args args;
	args.set_doc_head("Display information on single CCSDS packets or streams of packets.");
	args.set_doc_tail("By default the input is displayed in Packet format if a single CCSDS packet is present, or in Stream format otherwise.\nThe inputs may also be EPS L0 products instead of raw source packets.");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright(COPYRIGHT);
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("ccsds_count_filter");
	args.add_related_prog("noaa_l1b_displayer");

	args.add_example("ccsds_displayer *.ccsds", "Display a summay of all packets in all .ccsds files in the current directory");
	args.add_example("ccsds_displayer -p mhs.ccsds", "Show the contents of mhs.ccsds with each packet fully broken down");
	args.add_example("ccsds_displayer avhrr.ccsds --skip 500 --count 20000 --show-utc-gaps 680", "Display the index number (counting from 0), sequence count, UTC and OBT time of all packets where there is a gap of 680ms between successive time stamps. Only packets 500-20500 are scanned.");

	args.add_env_var("METOPIZER_HOME", "Location to search for ccsdsdisplayer.config file");

	args.add_group("Input settings:");
	args.add(&in_file, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input file(s)", "If omitted packets are read from standard input");
	args.add(&prepacket_header, "prepacket-header", 0, 0, "Skip <n> bytes before reading each CCSDS packet");

	args.add_group("Text settings:");
	args.add(&show_packets, "packets", 'p', 0, "Show output as individual packets");
	args.add(&hex, "hex", 0, 0, "Display packet contents as a hex dump");
	args.add(&header_only, "headers", 'h', 0, "Only display the generic headers of each packet");
	args.add(&show_stats, "show-stats", 0, 0, "Display channel count statistics instead of normal output");
	args.add(&show_gaps, "show-gaps", 0, 0, "Display time difference between each packet and the next (implies --headers if neither --headers nor --packets is specified");
	args.add(&show_seq_diffs, "show-seq-diffs", 0, 0, "Display sequence counter deltas from each packet ot the next");
	args.add(&unshift_gras_obt, "unshift-gras-obt", 0, 0, "Interpret GRAS OBT header times using the incorrect notation, not shifted 8 bits upwards relative to other instruments (APID 480-511)");
	args.add(&show_viadr, "show-viadr", 0, 0, "Display content of the viadr-l0-obt2utc records as they are found");

	args.add_group("Image settings:");
	args.add(&image_avhrr, "image-avhrr", 0, Args::FILENAME, "Output 5 channel images in TIFF format of AVHRR input packets", "ISPs belonging to any other instrument are ignored");
	args.add(&image_iasi_iis, "image-iasi-iis", 0, Args::FILENAME, "OUtput IASI IIS image to <filename>");
#ifndef _NO_JPEG
	args.add(&jpeg, "jpeg", 'j', 0, "Use Jpeg output instead of TIFF", "Only applies if the --image-avhrr option is selected");
#endif
	args.add(&hflip, "hflip", 0, 0, "Flip images horizontally (around the vertical axis) (IASI only)");
	args.add(&vflip, "vflip", 0, 0, "Flip images vertically (around the horizontal axis) (IASI only)");
	args.add(&invert, "invert", 0, 0, "Invert image colours (IASI only)");
	//args.add(&iis_trim, "iis-trim", 0, 0, "Trim the vertical lines from each IASI IIS row, so the data from adjacent lines do not overlap");

	args.add_group("Packet selection:");
	args.add(&count, "count", 'c', 0, "Display only <n> packets");
	args.add(&skip, "skip", 'k', 0, "Skip <n> packets before processing");
	args.add(&select_utc_gaps, "select-utc-gaps", 0, 0, "Display packet number(s) which have UTC time differences of <n> ms", "Normal output is suppressed");
	args.add(&select_grh_gaps, "select-grh-gaps", 0, 0, "Display packet number(s) which have GRH time differences of <n> ms (L0 products only)", "Normal output is suppressed");
	args.add(&select_obt_gaps, "select-obt-gaps", 0, 0, "Display packet number(s) which have OBT time differences of <n> s^-16", "Normal output is suppressed");

	args.add_group("Config settings:");
	args.add(&config_file, "config", 0, Args::FILENAME, "Use a config file");
	args.add(&check_config, "check-config", 0, 0, "Check config file");
	args.add(&check_iasi_config, "check-iasi-config", 0, 0, "Check IASI config file");
	args.add(0, "quiet", 'q', 0, "Omit additional messages");
	args.add(0, "", 'z', 0, "Use simple busy animations");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case 'q':
			verbose = false;
			break;

		case Args::USER_OPTION:
			_debug = &cout;
			break;

		case 'z':
			Spinner::use_simple_spinner();
			break;
		}
	}

	if (header_only || hex)
	{
		show_packets = true;
	}

	if (!verbose || isatty(1)!=1)
	{
		spinner->restream(&onull);
	}

	// if no config file specified check for an implicit one
	if (config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_file = string(config_dir_var) + "/" + DEFAULT_CONFIG_FILE;
		}
		else //if (getenv("HOME"))
		{
			// use the default filename
			//config_file = string(getenv("HOME")) + "/" + DEFAULT_CONFIG_DIR +
			//	"/" + DEFAULT_CONFIG_FILE;
			config_file = string(DEFAULT_METOPIZER_HOME) + "/" + DEFAULT_CONFIG_FILE;
		}
	}

	// if no iasi_config file specified check for an implicit one
	if (iasi_config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			iasi_config_file = string(config_dir_var) + "/" +
				DEFAULT_IASI_CONFIG_FILE;
		}
		else //if (getenv("HOME"))
		{
			// use the default filename
			//iasi_config_file = string(getenv("HOME")) + "/" +
			//	DEFAULT_CONFIG_DIR +
			//	"/" + DEFAULT_IASI_CONFIG_FILE;
			iasi_config_file = string(DEFAULT_METOPIZER_HOME) + "/" + DEFAULT_IASI_CONFIG_FILE;
		}
	}

	// set UTC time zone
	TimeConv::set_time_zone();

	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source, prepacket_header, show_viadr);
	CCSDSCountFilter count_filter(ccsds_reader, skip, count);

	if (show_stats)
	{
		// statistics display
		CCSDSStatsCalculator calc;
		calc.first_pass(count_filter);

		if (!in_file.empty())
		{
			FileReader filereader2(in_file);
			CCSDSReader ccsdsreader2(filereader2);
			calc.second_pass(ccsdsreader2);
		}
		else
		{
			info << "(not showing standard deviation because the input is a "
				 << "pipe, not a file)\n";
		}

		calc.output_text_summary();
		return 0;
	}

	if (!select_utc_gaps.empty() ||
		!select_obt_gaps.empty() ||
		!select_grh_gaps.empty())
	{
		CCSDSShowGaps show_gaps(count_filter,
								select_utc_gaps,
								select_grh_gaps,
								select_obt_gaps);
		return 0;
	}

	if (!image_avhrr.empty())
	{
		CCSDSImageAvhrr imager(count_filter);
#ifndef _NO_JPEG
		ImageWriter writer(imager, image_avhrr,
						   jpeg?Image::IMAGE_JPEG:Image::IMAGE_TIFF);
#else
		ImageWriter writer(imager, image_avhrr, Image::IMAGE_TIFF);
#endif
		if (verbose)
		{
			first_source->output_text_summary();
			count_filter.output_text_summary();
			imager.output_text_summary();
			writer.output_text_summary();
		}
		return 0;
	}

	if (!image_iasi_iis.empty())
	{
		CCSDSImageIASI imager(count_filter, hflip, vflip, invert, iis_trim);
#ifndef _NO_JPEG
		ImageWriter writer(imager, image_iasi_iis,
						   jpeg?Image::IMAGE_JPEG:Image::IMAGE_TIFF);
#else
		ImageWriter writer(imager, image_iasi_iis, Image::IMAGE_TIFF);
#endif
		if (verbose)
		{
			first_source->output_text_summary();
			count_filter.output_text_summary();
			imager.output_text_summary();
			writer.output_text_summary();
		}
		return 0;
	}

	if (show_packets || show_gaps)
	{
		CCSDSPacketDisplayer packet_displayer(
			count_filter,
			config_file,
			iasi_config_file);

		packet_displayer.output_text_record(cout,
											hex,
											header_only,
											show_gaps,
											show_seq_diffs,
											!unshift_gras_obt);
	}
	else
	{
		CCSDSStreamDisplayer stream_displayer(count_filter, !unshift_gras_obt);
		stream_displayer.output_text_summary();
	}

	delete first_source;
	return 0;
}
