////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_iasi_flags_fix.cpp
//
//	Provides: ccsds_iasi_flags_fix executable
//
//	Purpose: Use to correct IASI CCSDS packet flag settings. 
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "ccsdsiasiflagsfix.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "ccsds_iasi_flags_fix";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_file;
	string out_file;
	//string config_file;

	StringList filter;

	bool verbose = true;
	bool TYPE_ONLY = false;
	bool SEQ_FLAG_ONLY = false;

	//bool check_config = false;
	//bool accumulate = false;

	enum { ARGV_NONE, ARGV_IN, ARGV_OUT } state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__ 
				 << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << endl << APPNAME << " " << PACKAGE_VERSION << " built on " 
				 << __DATE__ 
				 << "\nUse to correct IASI CCSDS packet flag settings.\n\n"
				 << "usage:\n" << APPNAME
				 << " [--in|-i filename(s)]"
				 << " [--out|-o filename]"
				 << " [--type-only]"
				 << " [--sequence-flag-only]"
				 << " [-z]"
				 << " [--help|-h]"
				 << " [--version|-v]\n\n";
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_OUT;
		}
		else if (!strcmp(argv[x], "--type-only"))
		{
			TYPE_ONLY = true;
		}
		else if (!strcmp(argv[x], "--sequence-flag-only"))
		{
			SEQ_FLAG_ONLY = true;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else if (!strcmp(argv[x], "--quiet"))
		{
			verbose = false;
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else
		{
			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				in_file.push_back(argv[x]);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file can be specified\n";
					return -1;
				}
				else
				{
					out_file = argv[x];
					state = ARGV_NONE;
				}
				break;	  

			}
		}
	}
	

	// sort out streams
//	   if (out_file.empty())
//	   {
//	_info = &onull;
//	_warning = &onull;
//	verbose = false;
//	   }


	if (!verbose)
	{
		spinner->restream(&onull);
	}


	// set UTC time zone
	TimeConv::set_time_zone();


	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source, 0);
	CCSDSIASIFlagsFix iasi_fix(ccsds_reader, TYPE_ONLY, SEQ_FLAG_ONLY);

	if (!iasi_fix.is_valid())
	{
		error << "Exit...\n";
		return -1;
	}

	FileConsumer* writer;
	if (out_file.empty())
	{
		writer = new WriteToStdout(&iasi_fix);
	}
	else
	{
		writer = new SingleFileWriter(&iasi_fix, out_file);
	}

	if (verbose && !gerror)
	{
		first_source->output_text_summary();
		ccsds_reader.output_text_summary();
		iasi_fix.output_text_summary();
		writer->output_text_summary();

		if (!iasi_fix.is_valid())
		{
			error << "\n*** WARNING: The output produced might be corrupt "
				  << "or incomplete as errors were detected during the "
				  << "processing.\n\n";
			return -1;
		}
	}

	delete first_source;
	delete writer;

	return 0;
}
