////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_seq_fix.cpp
//
//	Provides: ccsds_seq_fix executable
//
//	Purpose: Use to modify/offset the sequence counter in the CCSDS primary
//			 header and the OBT time filed of the CCSDS ancillary data packet.
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "ccsdstimeandseqfix.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"
#include "textconfigdisplayer.hpp"
#include "multifilewriter.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

int gerror = 0; // global error
const string APPNAME = "ccsds_seq_fix";
const string CONFIG_DIR_VAR = "METOPIZER_HOME";
//const string DEFAULT_CONFIG_DIR = ".metopizer";
const string DEFAULT_CONFIG_FILE = "ccsds_seq_fix.config";

int main(int argc, char* argv[])
{
	ofstream onull("/dev/null");
	_debug = &onull;
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;

	deque<string> in_file;
	string out_file;
	string config_file;
	bool verbose = true;
	bool check_config = false;
	bool accumulate = false;
	bool single = false;

	enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_CONFIG} state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__ 
				 << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << endl << APPNAME << " " << PACKAGE_VERSION << " built on " 
				 << __DATE__ 
				 << "\nUse to modify/offset the sequence counter in the CCSDS "
				 << "primary header and the OBT time filed of the CCSDS "
				 << "ancillary data packet.\n\n"
				 << "usage:\n" << APPNAME
				 << " [--in|-i filename(s)]"
				 << " [--out|-o filename]"
				 << " [--config|-c filename]"
				 << " [--check-config]"
				 << " [--accumulate]"
				 << " [--single]"
				 << " [-z]"
				 << " [--help|-h]"
				 << " [--version|-v]\n\n";
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_OUT;
		}
		else if (!strcmp(argv[x], "--quiet"))
		{
			verbose = false;
		}
		else if (!strcmp(argv[x], "--config") || !strcmp(argv[x], "-c"))
		{
			state = ARGV_CONFIG;
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else if (!strcmp(argv[x], "--check-config"))
		{
			check_config = true;
		}
		else if (!strcmp(argv[x], "--accumulate"))
		{
			accumulate = true;
		}
		else if (!strcmp(argv[x], "--single"))
		{
			single = true;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else
		{

			if (!strncmp(argv[x],"--",2))
			{
				error << "unrecognised parameter - '" << argv[x] << "'\n";
				return -1;
			}

			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				in_file.push_back(argv[x]);
				break;


			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file can be specified\n";
					return -1;
				}
				else
				{
					out_file = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_CONFIG:
				if (!config_file.empty())
				{
					error << "Only one --config file can be specified\n";
					return -1;
				}
				else
				{
					config_file = argv[x];
					state=	ARGV_NONE;
				}
				break;		  
			}

		}

	}


	if (!verbose || isatty(1)!=1)
	{
		spinner->restream(&onull);
	}



	// if no config file specified check for an implicit one
	if (config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_file = string(config_dir_var) + "/" + DEFAULT_CONFIG_FILE;
		}
		else //if (getenv("HOME"))
		{
			// use the default filename
			//config_file = string(getenv("HOME")) + "/" + DEFAULT_CONFIG_DIR + 
			//	 "/" + DEFAULT_CONFIG_FILE;
			config_file = string(DEFAULT_METOPIZER_HOME) + "/" + DEFAULT_CONFIG_FILE;
		}
	}

	// set UTC time zone
	TimeConv::set_time_zone();

	if (check_config)
	{

		info << "Using config file '" << config_file << "'\n";

		FileReader fr("");
		CCSDSReader r(fr);

		CCSDSTimeAndSeqFix pd(r, config_file, accumulate, single);
		TextConfigDisplayer d(pd);

		return 0;
	}


	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source, 0);
	CCSDSTimeAndSeqFix time_filter(ccsds_reader, config_file, accumulate, single);

	if (!time_filter.is_valid())
	{
		error << "Exit...\n";
		return -1;
	}

	FileConsumer* writer;
	if (out_file.empty())
	{
		writer = new WriteToStdout(&time_filter);
	}
	else
	{
		writer = new SingleFileWriter(&time_filter, out_file);
	}

	if (verbose && !gerror)
	{
		first_source->output_text_summary();
		ccsds_reader.output_text_summary();
		time_filter.output_text_summary();
		writer->output_text_summary();

		if (!time_filter.is_valid())
		{
			error << "\n*** WARNING: The output produced might be corrupt "
				  << "or incomplete as errors were detected during the "
				  << "processing.\n\n";
			return -1;
		}
	}

	delete first_source;
	delete writer;

	return 0;
}
