////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_strip_filter.cpp
//
//	Provides: ccsds_strip_filter executable
//
//	Purpose: Removes CCSDS packets that do not satisfy the UTC start and end
//			 times specified in the ccsds_strip_filter.config file.
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "ccsdsstripfilter.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"
#include "textconfigdisplayer.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "ccsds_strip_filter";
const string CONFIG_DIR_VAR = "METOPIZER_HOME";
//const string DEFAULT_CONFIG_DIR = ".metopizer";
const string DEFAULT_CONFIG_FILE = "ccsds_strip_filter.config";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_file;
	string out_file;
	string config_file;

	StringList filter;

	bool verbose = true;
	bool check_config = false;


	enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_CONFIG } state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__ 
				 << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << endl << APPNAME << " " << PACKAGE_VERSION << " built on " 
				 << __DATE__ 
				 << "\nRemoves CCSDS packets that do not satisfy the UTC "
				 << "start and end times specified in the "
				 << "ccsds_strip_filter.config file.\n\n"
				 << "usage:\n" << APPNAME
				 << " [--in|-i filename(s)]"
				 << " [--out|-o filename]"
				 << " [--config|-c filename]"
				 << " [--check-config]"
				 << " [-z]"
				 << " [--help|-h]"
				 << " [--version|-v]\n\n";
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_OUT;
		}
		else if (!strcmp(argv[x], "--config") || !strcmp(argv[x], "-c"))
		{
			state = ARGV_CONFIG;
		}
		else if (!strcmp(argv[x], "--check-config"))
		{
			check_config = true;
		}

		else if (!strcmp(argv[x], "--quiet"))
		{
			verbose = false;
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else
		{
			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				in_file.push_back(argv[x]);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file can be specified\n";
					return -1;
				}
				else
				{
					out_file = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_CONFIG:
				if (!config_file.empty())
				{
					error << "Only one config file can be specified\n";
					return -1;
				}
				else
				{
					config_file = argv[x];
					state=	ARGV_NONE;
				}
				break;		  

			}
		}
	}
	

	// sort out streams
//	   if (out_file.empty())
//	   {
//	_info = &onull;
//	_warning = &onull;
//	verbose = false;
//	   }


//	   if (out_file.empty())
//	   {
//	out_file = "strip_output";
//	   }


	// if no config file specified check for an implicit one
	if (config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_file = string(config_dir_var) + "/" + DEFAULT_CONFIG_FILE;
		}
		else //if (getenv("HOME"))
		{
			// use the default filename
			//config_file = string(getenv("HOME")) + "/" + DEFAULT_CONFIG_DIR + 
			//	"/" + DEFAULT_CONFIG_FILE;
			config_file = string(DEFAULT_METOPIZER_HOME) + "/" + DEFAULT_CONFIG_FILE;
		}
	}


	if (!verbose)
	{
		spinner->restream(&onull);
	}


	// set UTC time zone
	TimeConv::set_time_zone();


	if (check_config)
	{

		info << "Using config file '" << config_file << "'\n";

		FileReader fr("");
		CCSDSReader r(fr);

		CCSDSStripFilter pd(r, config_file);
		TextConfigDisplayer d(pd);

		return 0;
	}


	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source, 0);
	CCSDSStripFilter strip_filter(ccsds_reader, config_file);

	FileConsumer* writer;
	if (out_file.empty())
	{
		writer = new WriteToStdout(&strip_filter);
	}
	else
	{
		writer = new SingleFileWriter(&strip_filter, out_file);
	}

	if (verbose && !gerror)
	{
		first_source->output_text_summary();
		ccsds_reader.output_text_summary();
		strip_filter.output_text_summary();
		writer->output_text_summary();
	}

	delete first_source;
	delete writer;

	return 0;
}

