////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_time_fix.cpp
//
//	Provides: ccsds_time_fix executable
//
//	Purpose: Use to set CCSDS packet UTC and OBT times. 
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "ccsdstimefix.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"
#include "textconfigdisplayer.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "ccsds_time_fix";
const string CONFIG_DIR_VAR = "METOPIZER_HOME";
//const string DEFAULT_CONFIG_DIR = ".metopizer";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_file;
	string out_file;
	string config_file;
	string utc_start;
	string obt_start;

	StringList filter;

	bool verbose = true;
	bool check_config = false;
	bool accumulate = false;
	bool maintain_time_gaps = false;

	enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_CONFIG,
		   ARGV_UTC_START, ARGV_OBT_START } state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__ 
				 << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << endl << APPNAME << " " << PACKAGE_VERSION << " built on " 
				 << __DATE__ 
				 << "\nSet CCSDS packet UTC times. Use the config file to "
				 << "modify the UTC increment rate.\n\n"
				 << "usage:\n" << APPNAME
				 << " [--in|-i filename(s)]"
				 << " [--out|-o filename]"
				 << " [--config|-c filename]"
				 << " [--check-config|-cc]"
				 << " [--accumulate|-a]"
				 << " [--maintain-time-gaps|-mtg]"
				 << " [--utc-start|-utc <YYYYMMDDhhmmssZ or YYYYMMDDhhmmssuuuZ>]"
				 << " [--obt-start|-obt <coarse>.<fine>]"
				 << " [-z]"
				 << " [--help|-h]"
				 << " [--version|-v]\n\n"
				 << "ie. " << APPNAME << " -i input-name -o output-name "
				 << "--utc-start 20020808120000Z --obt-start 20000.50\n\n";
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_OUT;
		}
		else if (!strcmp(argv[x], "--config") || !strcmp(argv[x], "-c"))
		{
			state = ARGV_CONFIG;
		}
		else if (!strcmp(argv[x], "--check-config") ||
				 !strcmp(argv[x], "-cc") )
		{
			check_config = true;
		}
		else if (!strcmp(argv[x], "--start-utc") ||
				 !strcmp(argv[x], "--utc-start") ||
				 !strcmp(argv[x], "-utc"))
		{
			state = ARGV_UTC_START;
		}
		else if (!strcmp(argv[x], "--start-obt") ||
				 !strcmp(argv[x], "--obt-start") ||
				 !strcmp(argv[x], "-obt"))
		{
			state = ARGV_OBT_START;
		}
		else if (!strcmp(argv[x], "--maintain-time-gaps") ||
				 !strcmp(argv[x], "--maintain-time-gap") ||
				 !strcmp(argv[x], "-mtg"))
		{
			maintain_time_gaps = true;
		}
		else if (!strcmp(argv[x], "--quiet"))
		{
			verbose = false;
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else if (!strcmp(argv[x], "--accumulate") ||
				 !strcmp(argv[x], "-a") )
		{
			accumulate = true;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else
		{
			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				in_file.push_back(argv[x]);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file can be specified\n";
					return -1;
				}
				else
				{
					out_file = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_CONFIG:
				if (!config_file.empty())
				{
					error << "Only one config file can be specified\n";
					return -1;
				}
				else
				{
					config_file = argv[x];
					state = ARGV_NONE;
				}
				break;		  

			case ARGV_UTC_START:
				if (!utc_start.empty())
				{
					error << "Only one UTC start time can be specified\n";
					return -1;
				}
				else
				{
					utc_start = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_OBT_START:
				if (!obt_start.empty())
				{
					error << "Only one OBT start time can be specified\n";
					return -1;
				}
				else
				{
					obt_start = argv[x];
					state = ARGV_NONE;
				}
				break;
		
			}
		}
	}
	

	// sort out streams
//	   if (out_file.empty())
//	   {
//	_info = &onull;
//	_warning = &onull;
//	verbose = false;
//	   }

	
	if (!verbose)
	{
		spinner->restream(&onull);
	}

	
	// set UTC time zone
	TimeConv::set_time_zone();


	if (check_config)
	{

		info << "Using config file '" << config_file << "'\n";

		FileReader fr("");
		CCSDSReader r(fr);

		CCSDSTimeFix pd(r, config_file, accumulate, utc_start, obt_start, maintain_time_gaps);
		TextConfigDisplayer d(pd);

		return 0;
	}


	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source, 0);
	CCSDSTimeFix utc_fix(ccsds_reader, config_file, accumulate,
						 utc_start, obt_start, maintain_time_gaps);


	FileConsumer* writer;
	if (out_file.empty())
	{
		writer = new WriteToStdout(&utc_fix);
	}
	else
	{
		writer = new SingleFileWriter(&utc_fix, out_file);
	}

	if (verbose && !gerror)
	{
		first_source->output_text_summary();
		ccsds_reader.output_text_summary();
		utc_fix.output_text_summary();
		writer->output_text_summary();
	}

	delete first_source;
	delete writer;

	return 0;
}
