////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsds_to_l0.cpp
//
//	Provides: ccsds_to_l0 executable
//
//	Purpose: Convert ISP files to L0 products
//
//	Copyright (c) 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "ccsdstol0.hpp"
#include "epsproductextractor.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"
#include "eps.hpp"
#include "subsat.hpp"
#include "osvs.hpp"
#include "mphr.hpp"

#define DEFAULT_METOPIZER_HOME "/usr/local/share/metopizer"
#define PACKAGE_VERSION ""

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "ccsds_to_l0";

const string CONFIG_DIR_VAR = "METOPIZER_HOME";
const string DEFAULT_CONFIG_FILE_MPHR = "l0.config";
const string DEFAULT_CONFIG_FILE_APID = "ccsds.config";


const string helpstr = string() +
	"PURPOSE:\n"
	"Create EPS L0 product from source packets.\n\n" +
    "USAGE:\n" + APPNAME + " [--in, -i <filename(s)>] [--out, -o <filename>]  [--outdir, -d <directory>] " +
	"[--utc-obt, -utc <UTC time>] [--obt-utc, -obt <OBT time>] [--clock-drift, -cd <number>] " +
	"[--rollovers, -r <number>] [--viadr <VIADR filename>] [--config-mphr, -mphr <filename>] " +
	"[--config-apid, -apid <filename>] [--show-viadr, -sviadr] [--show-config-mphr, -smphr] " +
    "[--show-config-apid, -sapid] [--use-mmam, -u <directory>] [--generate-mmam, -g <directory>] " +
    "[--generate-tle, --gtle, -t <directory>]  [--validate-mmam, --val <mmam_schema>] [--segment-size, -s <seconds>] " +
    "[--help, -h] [--add-suffix <suffix>] [--version, -v]\n\n" +
	"WHERE:\n" +
	"--in <filename(s)> : Allows input files to be specified. Multiple input files should be " +
	"delimited by spaces. If the --in option is omitted, stdin is used by default.\n" +
	"--out <filename> : Specify text to form the initial part of the autogenerated output file name.\n" +
    "--outdir <directory> : Specify directory to store the generated output file(s).\n" +
	"--utc-obt <UTC time> : Set the UTC-OBT (or UTC0) parameter for the OBT->UTC " +
	"equation to either an absolute time or time relative to the start of the input " +
	"data. Absolute times are given as either YYYYMMDDhhmmssZ or " +
	"YYYYMMDDhhmmssuuuZ where YYYY=year, MM=month, DD=day, hh=hour, mm=minute, " +
	"ss=second, uuu=milliseconds, and the final Z to show the format is EPS " +
	"generalised time. Relative times are given as -<number><units> where " +
	"<units> can be o=orbits, h=hours, m=minutes, s=seconds, or ms=milliseconds. " +
	"The default value is -31o.\n" +
	"--obt-utc <OBT time> : Set the OBT-UTC (or OBT0) parameter for the OBT->UTC " +
	"equation. The value is specified as C,F (C=coarse time, F=fine " +
	"time). The value defaults to 0,0.\n" +
	"--clock-drift <number> : The value for clock drift (or A_STEP) for the " +
	" OBT->UTC equation. The default value is 3906249305.\n" +
	"--rollovers <number> : See below.\n" +
	"--viadr <filename> : Specify a filename containing a viadr-l0-obtutc record " +
	"to take the parameters for the OBT->UTC correlation from. The file can be a " +
	"complete L0 product or just a single record.\n" +
	"--config-mphr <filename> : Define configuration file to be used to fill EPS-L0 specific MPHR fields.\n" +
	"--config-apid <filename> : Configuration file mapping CCSDS packet APIDs to instrument groups.\n" +
	"--show-viadr : Displayes the default values used or the values read if an input viadr-l0-obtutc record is specified.\n" +
	"--show-config-mphr : Displayes the default values used or the values read if a configuration file is specified.\n" +
	"--show-config-apid : Displayes the default values used or the values read if a configuration file is specified.\n" +
    "--use-mmam : Use available and cached MMAMS for OBT and MPHR contruction (otherwise behave as always and ignore options below). \n" +
    "--validate-mmam : Validate the MMAMs used during processing against a provided schema.\n" +
    "--generate-mmam : Generate MMAM files from the MMAM available in HK administrative packets (APID-6).\n" +
    "--generate-tle : Generate TLE files from the TLE xml elements inside the MMAMs.\n" +
    "--segment-size : Size in seconds that each L0 product will have.\n" +
    "--add-suffix : Adds a string as a suffix for the generated L0 pruduct filenames.\n" +
	"-z : Use simple animations while processing.\n" +
    "--quiet : Do not use animations while processing.\n" +
	"--help : Show usage information.\n" +
	"--version : Show program version.\n\n" +
	"DESCRIPTION:\n"
	"A L0 product is built using all input CCSDS packets in the order they are stored in the input files. The input files are processed as given on the command line.\n\n" +
	"The source packets are never modified and the only processing performed is to create MPHR, viadr-l0-utc2obt and IPR records and to form the L0 MDR header information including GRH RECORD_START_TIME and RECORD_STOP_TIME fields for each packet. The RECORD_STOP_TIME field is set to the same value as RECORD_START_TIME.\n\n" +
	"The OBT->UTC equation used to calculate the MDR time values is:\n\n" +
	"GRH time = ((OBT - OBT0)*CLOCK_DRIFT) + UTC0.\n\n" +
	"If no OBT->UTC parameters are given then a realistic set of defaults is be generated, with UTC0 set 31 orbits before the UTC time of the first input packet, and OBT0 based on the OBT time of the first packet.\n" +
	"Alternatively any of the 3 values (UTC0, OBT0, CLOCK_DRIFT) can be supplied by the user.\n\n" +
	"If the rollovers command line argument is used then the algorithm will assume that number of OBT clock rollovers have occurred between the UTC0 time and the current time, and this will be shown in the generated MDR record headers. These will be increased by approx. 49 days per increment of the rollovers parameter above zero.\n\n" +
	"To help fill the MPHR record, ccsds_to_l0 can try to read in OSVS values and subsatellite positions from the Eugene home directory, which is found by reading the EUGENE_HOME environment variable if defined, otherwise by running eugene --show-dir-home. If not found then default values will be placed into the product.\n\n" +
	"The MPHR field PRODUCT_NAME will be set correctly in the output product regardless of the actual name of the file. This can be renamed using the eps_renamer command from Eugene.\n";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_file;
	string out_file;
    string out_dir;
    string mmam_out_dir;
    string tle_out_dir;
    string mmam_in_dir;
    string mmam_schema;
    string out_suffix;
	string config_file;
	string config_apid;
	string viadr_file;
	string utc_obt;
	string obt_utc;
	string clock_drift;
	int rollovers = 0;

	bool verbose = true;
	bool SHOW_VIADR = false;
	bool SHOW_CONF_MPHR = false;
	bool SHOW_CONF_APID = false;

    bool generate_mmam = false;
    bool generate_tle = false;
    bool validate_mmam = false;
    bool use_mmam_data = false;
    unsigned int segment_size = 0;


    enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_OUTDIR, ARGV_CONFIG_MPHR, ARGV_CONFIG_APID,
		   ARGV_VIADR, ARGV_UTC_OBT, ARGV_OBT_UTC, ARGV_CLOCK_DRIFT,
           ARGV_ROLLOVERS, ARGV_MMAM_INDIR, ARGV_MMAM_OUTDIR, ARGV_TLE_OUTDIR, ARGV_MMAM_SCHEMA, ARGV_SEGMENT_SIZE,
           ARGV_SUFFIX} state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__
				 << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << "SUMMARY:\n" << APPNAME << " version " << PACKAGE_VERSION
				 << " built on " << __DATE__ << endl << endl << helpstr;
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_OUT;
		}
        else if (!strcmp(argv[x], "--outdir") || !strcmp(argv[x], "-d"))
        {
            state = ARGV_OUTDIR;
        }
		else if (!strcmp(argv[x], "--config-mphr") || !strcmp(argv[x], "-mphr"))
		{
			state = ARGV_CONFIG_MPHR;
		}
		else if (!strcmp(argv[x], "--config-apid") || !strcmp(argv[x], "-apid"))
		{
			state = ARGV_CONFIG_APID;
		}
		else if (!strcmp(argv[x], "--viadr"))
		{
			state = ARGV_VIADR;
		}
		else if (!strcmp(argv[x], "--show-viadr") || !strcmp(argv[x], "-sviadr"))
		{
			SHOW_VIADR = true;
		}
		else if (!strcmp(argv[x], "--show-config-mphr") || !strcmp(argv[x], "-smphr"))
		{
			SHOW_CONF_MPHR = true;
		}
		else if (!strcmp(argv[x], "--show-config-apid") || !strcmp(argv[x], "-sapid"))
		{
			SHOW_CONF_APID = true;
		}
        else if (!strcmp(argv[x], "--generate-mmam") || !strcmp(argv[x], "-g"))
        {
            generate_mmam = true;
            state = ARGV_MMAM_OUTDIR;
        }
        else if (!strcmp(argv[x], "--generate-tle") || !strcmp(argv[x], "--gtle") || !strcmp(argv[x], "-t"))
        {
            generate_tle = true;
            state = ARGV_TLE_OUTDIR;
        }
        else if (!strcmp(argv[x], "--validate-mmam") || !strcmp(argv[x], "--val"))
        {
            validate_mmam = true;
            state = ARGV_MMAM_SCHEMA;
        }
        else if (!strcmp(argv[x], "--use-mmam") || !strcmp(argv[x], "-u"))
        {
            use_mmam_data = true;
            state = ARGV_MMAM_INDIR;
        }
        else if (!strcmp(argv[x], "--segment-size") || !strcmp(argv[x], "-s"))
        {
            state = ARGV_SEGMENT_SIZE;
        }
        else if (!strcmp(argv[x], "--add-suffix"))
        {
            state = ARGV_SUFFIX;
        }

		else if (!strcmp(argv[x], "--utc-obt") ||
				 !strcmp(argv[x], "--utc-obt-time") ||
				 !strcmp(argv[x], "--utcobt") ||
				 !strcmp(argv[x], "--utcobttime") ||
				 !strcmp(argv[x], "-utc"))
		{
			state = ARGV_UTC_OBT;
		}
		else if (!strcmp(argv[x], "--obt-utc") ||
				 !strcmp(argv[x], "--obt-utc-time") ||
				 !strcmp(argv[x], "--obtutc") ||
				 !strcmp(argv[x], "--obtutctime") ||
				 !strcmp(argv[x], "-obt"))
		{
			state = ARGV_OBT_UTC;
		}
		else if (!strcmp(argv[x], "--clock-drift") ||
				 !strcmp(argv[x], "--drift") ||
				 !strcmp(argv[x], "-cd"))
		{
			state = ARGV_CLOCK_DRIFT;
		}
		else if (!strcmp(argv[x], "--rollovers") ||
				 !strcmp(argv[x], "-r"))
		{
			state = ARGV_ROLLOVERS;
		}
		else if (!strcmp(argv[x], "--quiet"))
		{
			verbose = false;
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else
		{
			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!strncmp(argv[x],"--",2))
				{
					error << "unrecognised parameter - '" << argv[x] << "'\n";
					return -1;
				}
				in_file.push_back(argv[x]);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file name can be specified\n";
					return -1;
				}
				else
				{
					out_file = argv[x];
					state = ARGV_NONE;
				}
				break;

            case ARGV_OUTDIR:
                if (!out_dir.empty())
                {
                    error << "Only one output directory name can be specified\n";
                    return -1;
                }
                else
                {
                    out_dir = argv[x];

                    //add trailing directory backslash if not already there
                    if (out_dir.at(out_dir.length()-1) != '/')
                    {
                        out_dir += "/";
                    }

                    state = ARGV_NONE;
                }
                break;

			case ARGV_CONFIG_MPHR:
				if (!config_file.empty())
				{
					error << "Only one MPHR config file can be specified\n";
					return -1;
				}
				else
				{
					config_file = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_CONFIG_APID:
				if (!config_apid.empty())
				{
					error << "Only one APID config file can be specified\n";
					return -1;
				}
				else
				{
					config_apid = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_UTC_OBT:
				if (!utc_obt.empty())
				{
					error << "Only one UTC_OBT time can be specified\n";
					return -1;
				}
				else
				{
					utc_obt = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_OBT_UTC:
				if (!obt_utc.empty())
				{
					error << "Only one OBT_UTC time can be specified\n";
					return -1;
				}
				else
				{
					obt_utc = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_CLOCK_DRIFT:
				if (!clock_drift.empty())
				{
					error << "Only one clock drift can be specified\n";
					return -1;
				}
				else
				{
					clock_drift = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_VIADR:
				if (!viadr_file.empty())
				{
					error << "Only one VIADR file can be specified\n";
					return -1;
				}
				else
				{
					viadr_file = argv[x];
					state = ARGV_NONE;
				}
				break;

			case ARGV_ROLLOVERS:
				rollovers = atoi(argv[x]);
				state = ARGV_NONE;
				break;

            case ARGV_MMAM_INDIR:
                mmam_in_dir = argv[x];

                //add trailing directory backslash if not already there
                if (mmam_in_dir.at(mmam_in_dir.length()-1) != '/')
                {
                    mmam_in_dir += "/";
                }

                state = ARGV_NONE;
                break;

            case ARGV_MMAM_OUTDIR:
                mmam_out_dir = argv[x];

                //add trailing directory backslash if not already there
                if (mmam_out_dir.at(mmam_out_dir.length()-1) != '/')
                {
                    mmam_out_dir += "/";
                }

                state = ARGV_NONE;
                break;

            case ARGV_TLE_OUTDIR:
                tle_out_dir = argv[x];

                //add trailing directory backslash if not already there
                if (tle_out_dir.at(tle_out_dir.length()-1) != '/')
                {
                    tle_out_dir += "/";
                }

                state = ARGV_NONE;
                break;

            case ARGV_MMAM_SCHEMA:
                mmam_schema = argv[x];
                state = ARGV_NONE;
                break;

            case ARGV_SEGMENT_SIZE:
                segment_size = atoi(argv[x]);
                state = ARGV_NONE;
                break;

            case ARGV_SUFFIX:
                out_suffix = argv[x];
                state = ARGV_NONE;
                break;

			}
		}
	}

    //check that the options selected for EARS make sense
    if (use_mmam_data)
    {
        if (!out_file.empty())
        {
            error << "-o option is not compatible with -u option. Please use '-d' to specify output directory.\n";
            return -1;
        }
    }

	// if no config file specified check for an implicit one
	if (config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_file = string(config_dir_var) + "/" +  DEFAULT_CONFIG_FILE_MPHR;
		}
		else
		{
			// use the default filename
			config_file = string(DEFAULT_METOPIZER_HOME) + "/" + DEFAULT_CONFIG_FILE_MPHR;
		}
	}


	// if no config file specified check for an implicit one
	if (config_apid.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_apid = string(config_dir_var) + "/" +  DEFAULT_CONFIG_FILE_APID;
		}
		else
		{
			// use the default filename
			config_apid = string(DEFAULT_METOPIZER_HOME) + "/" + DEFAULT_CONFIG_FILE_APID;
		}
	}


	string eugene_home;
	if (getenv("EUGENE_HOME"))
	{
		eugene_home = getenv("EUGENE_HOME");
//	cout << "Using " << eugene_home << " as base directory for "
//		 << "OSVS file and subsat directory\n";
	}
	else
	{
		FILE* fp = popen("eugene --show-dir-home 2> /dev/null", "r");
		if (fp)
		{
//		cout << "EUGENE_HOME variable not set, calling "
//		 << "'eugene --show-dir-home' instead...\n";
			const int BUFSIZE = 256;
			char buff[BUFSIZE];
			if (fgets(buff, BUFSIZE, fp))
			{
                // remove newline
				buff[strlen(buff)-1] = '\0';
//		cout << "Using '" << buff << "' as base directory for OSVS "
//			 << "file and subsat directory.\n";
				eugene_home = buff;
			}
			pclose(fp);
		}
		else
		{
			cerr << "Cannot run executable\n";
		}
	}

	string subsat_dirname = eugene_home + "/subsat";
	string osvs_filename = eugene_home + "/osvs/osvs.xml";

	if (!verbose)
	{
		spinner->restream(&onull);
	}

	// set UTC time zone
	TimeConv::set_time_zone();

	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}


	CCSDSReader ccsds_reader(*first_source, 0);
    CCSDSToL0 converter(ccsds_reader, out_file, out_dir, config_file, config_apid,
						subsat_dirname, osvs_filename, viadr_file, utc_obt,
                        obt_utc, clock_drift, rollovers, generate_mmam,
                        generate_tle, validate_mmam, use_mmam_data, mmam_in_dir,
                        mmam_out_dir, tle_out_dir, mmam_schema, segment_size, out_suffix);


	bool SHOW_FLAG = false;
	if (SHOW_VIADR)
	{
		converter.output_text_viadr();
		SHOW_FLAG = true;
	}
	if (SHOW_CONF_MPHR)
	{
		converter.output_text_mphr_config();
		SHOW_FLAG = true;
	}
	if (SHOW_CONF_APID)
	{
		converter.output_text_ccsds_config();
		SHOW_FLAG = true;
	}
	if (SHOW_FLAG)
	{
		return 0;
	}

	while(converter.create_epsl0());
	converter.complete_epsl0();

	first_source->output_text_summary();
	ccsds_reader.output_text_summary();
	converter.output_text_summary();

	delete first_source;

	return 0;
}
