////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsapidfilter.hpp
//
//	Provides: CCSDSAPIDFilter filter
//
//	Purpose: Pass through all packets in a CCSDS stream that match the filter
//			 string
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsapidfilter_defined
#define ccsdsapidfilter_defined

#include "ccsdsproducer.hpp"
#include "textsummaryproducer.hpp"
#include "fileproducer.hpp"
#include "fileconsumer.hpp"

class CCSDSAPIDFilter :
	public CCSDSProducer,
	public FileProducer
{
public:

	typedef multimap<string, APID> APIDS;

	//
	// constructor (display apids only)
	//

	CCSDSAPIDFilter(const StringList& in_filters)
		: source(NULL)
		, pass_count(0)
		, block_count(0)
		, valid(true)
		{
			ingest_filters(in_filters);
		}

	//
	// constructor (main)
	//

	CCSDSAPIDFilter( CCSDSProducer& _source , const StringList& in_filters)
		: source(&_source)
		, pass_count(0)
		, block_count(0)
		, valid(true)
		{
			ingest_filters(in_filters);
		}

	CCSDSAPIDFilter( CCSDSProducer& _source , const string& filter)
		: source(&_source)
		, pass_count(0)
		, block_count(0)
		, valid(true)
		{
			StringList in_filters;
			in_filters.push_back( string(filter) );
			ingest_filters(in_filters);
		}


	//
	// ingest_filters
	//

	void ingest_filters(const StringList& in_filters)
		{
			APIDS apids = get_apids();
//		string filter = _filter;

			// process each element of 'filters'
			for(StringList::const_iterator iter=in_filters.begin();
				iter!=in_filters.end();
				iter++)
			{
				string filter = *iter;

				// see if 'filter' is numerical
				if (filter.find_first_not_of("0123456789") == string::npos)
				{
					filters.push_back( atoi(filter.c_str()) );

//			DEBUG("ccsddsapidfilter constructor with numerical "
//			<< "parm '" << atoi(filter.c_str()) << endl);
				}
				else
				{
					// make 'filter' upper case
					//for_each(filter.begin(), filter.end(), toupper);
					//transform(filter.begin(), filter.end(), toupper);
					for(unsigned int i=0; i<filter.length(); i++)
					{
						filter.at(i) = toupper(filter[i]);
					}

					pair<APIDS::iterator, APIDS::iterator> p =
						apids.equal_range(filter);

					int count = 0;
					for(; p.first != p.second; p.first++)
					{
						filters.push_back( (*(p.first)).second );
						count++;
//			DEBUG("ccsdsapidfilter pushing back apid "
//				  << (*(p.first)).second << endl);
					}
					if (count==0)
					{
						error << "CCSDSApidFilter : unknown APID '"
							  << filter << "'\n";
						valid = false;
					}
				}
			}
		}

	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout )
		{
			if (filters.empty())
			{
				return;
			}

			os << "CCSDSAPIDFilter report:\n";

			if (pass_count>0)
			{
				os << "	   " << pass_count << " packets passed on\n";
			}

			if (block_count>0)
			{
				os << "	   " << block_count << " packets blocked\n";
			}

			os << "	   Filtering for APID :";

			if (filters.empty())
			{
				os << " <all>\n";
			}
			else
			{
				for(Filters::iterator iter=filters.begin(); iter!=filters.end();
					iter++)
				{
					os << " " << (*iter);
				}
			}
			os << endl << endl;
		}

	//
	// display
	//

	static void display(void)
		{
			info << "CCSDSAPIDFilter registered APID names are:\n";

			APIDS apids = get_apids();
			// build list of keys
			set<string> keys;
			for(APIDS::iterator iter=apids.begin(); iter!=apids.end(); iter++)
			{
				keys.insert( (*iter).first );
			}

			// display keys against value listings
			for(set<string>::iterator kiter=keys.begin(); kiter!=keys.end();
				kiter++)
			{
				info << "	 " << (*kiter) << " : ";
				pair<APIDS::iterator, APIDS::iterator> p =
					apids.equal_range(*kiter);

				for( ; p.first != p.second ; p.first++ )
				{
					info << ((*(p.first)).second) << " ";
				}

				info << endl;
			}
		}

	//
	// get_apids
	//

	static APIDS get_apids(void)
		{
			APIDS apids;

			apids.insert(APIDS::value_type(string("AVHRR-HR-3A"), 103));
			apids.insert(APIDS::value_type(string("AVHRR-HR-3B"), 104));
			apids.insert(APIDS::value_type(string("AVHRR-HR"), 103));
			apids.insert(APIDS::value_type(string("AVHRR-HR"), 104));

			apids.insert(APIDS::value_type(string("AVHRR-LR-1"), 64));
			apids.insert(APIDS::value_type(string("AVHRR-LR-2"), 65));
			apids.insert(APIDS::value_type(string("AVHRR-LR-3A"), 66));
			apids.insert(APIDS::value_type(string("AVHRR-LR-3B"), 67));
			apids.insert(APIDS::value_type(string("AVHRR-LR-4"), 68));
			apids.insert(APIDS::value_type(string("AVHRR-LR-5"), 69));
			apids.insert(APIDS::value_type(string("AVHRR-LR-CAL"), 70));
			apids.insert(APIDS::value_type(string("AVHRR-LR"), 64));
			apids.insert(APIDS::value_type(string("AVHRR-LR"), 65));
			apids.insert(APIDS::value_type(string("AVHRR-LR"), 66));
			apids.insert(APIDS::value_type(string("AVHRR-LR"), 67));
			apids.insert(APIDS::value_type(string("AVHRR-LR"), 68));
			apids.insert(APIDS::value_type(string("AVHRR-LR"), 69));
			apids.insert(APIDS::value_type(string("AVHRR-LR"), 70));

			apids.insert(APIDS::value_type(string("AVHRR"), 64));
			apids.insert(APIDS::value_type(string("AVHRR"), 65));
			apids.insert(APIDS::value_type(string("AVHRR"), 66));
			apids.insert(APIDS::value_type(string("AVHRR"), 67));
			apids.insert(APIDS::value_type(string("AVHRR"), 68));
			apids.insert(APIDS::value_type(string("AVHRR"), 69));
			apids.insert(APIDS::value_type(string("AVHRR"), 70));
			apids.insert(APIDS::value_type(string("AVHRR"), 103));
			apids.insert(APIDS::value_type(string("AVHRR"), 104));

			apids.insert(APIDS::value_type(string("AMSU-A1"), 39));
			apids.insert(APIDS::value_type(string("AMSU-A2"), 40));
			apids.insert(APIDS::value_type(string("AMSU-A"), 39));
			apids.insert(APIDS::value_type(string("AMSU-A"), 40));

			apids.insert(APIDS::value_type(string("HIRS"), 38));
			apids.insert(APIDS::value_type(string("SEM"), 37));
			apids.insert(APIDS::value_type(string("A-DCS"), 35));
			apids.insert(APIDS::value_type(string("MHS"), 34));

			apids.insert(APIDS::value_type(string("IASI-1"), 130));
			apids.insert(APIDS::value_type(string("IASI-2"), 135));
			apids.insert(APIDS::value_type(string("IASI-3"), 140));
			apids.insert(APIDS::value_type(string("IASI-4"), 145));
			apids.insert(APIDS::value_type(string("IASI-IMAGE"), 150));
			apids.insert(APIDS::value_type(string("IASI-SPECT-VERIF"), 160));
			apids.insert(APIDS::value_type(string("IASI-AUXILIARY"), 180));
			apids.insert(APIDS::value_type(string("IASI"), 130));
			apids.insert(APIDS::value_type(string("IASI"), 135));
			apids.insert(APIDS::value_type(string("IASI"), 140));
			apids.insert(APIDS::value_type(string("IASI"), 145));
			apids.insert(APIDS::value_type(string("IASI"), 150));
			apids.insert(APIDS::value_type(string("IASI"), 160));
			apids.insert(APIDS::value_type(string("IASI"), 180));

			for(int i=192; i<=255; i++)
			{
				apids.insert(APIDS::value_type(string("ASCAT"), i));
			}

			for(int i=384; i<=447; i++)
			{
				apids.insert(APIDS::value_type(string("GOME-2"), i));
			}

			for(int i=448; i<=511; i++)
			{
				apids.insert(APIDS::value_type(string("GRAS-MEASURE"), i));
			}

			apids.insert(APIDS::value_type(string("ADMIN"), 6));

			apids.insert(APIDS::value_type(string("GOME-NAV-REAL_TIME"), 2));
			apids.insert(APIDS::value_type(string("GOME-NAV-REDUCED"), 3));
			apids.insert(APIDS::value_type(string("GRAS-NAV"), 2));
			apids.insert(APIDS::value_type(string("GRAS-NAV"), 3));

			apids.insert(APIDS::value_type(string("SATELLITE"), 1));
			apids.insert(APIDS::value_type(string("SL-PACKET"), 1));

//		apids.insert(APIDS::value_type(string("HKTM"), 1));
//		apids.insert(APIDS::value_type(string("HKTM"), 2));
//		apids.insert(APIDS::value_type(string("HKTM"), 3));
//		apids.insert(APIDS::value_type(string("HKTM"), 6));

			return apids;
		}

	//
	// get_next_ccsds
	//

	virtual CCSDS* get_next_ccsds(void)
		{
			CCSDS* result = source->get_next_ccsds();
//		DEBUG("ccsdsapidfilter::get_next_ccsds() read "
			// << ((unsigned int)result) << " from producer\n");

			if (!result)
			{
//		DEBUG("ccsdsapidfilter::get_next_ccsds returning null\n");
				return NULL;
			}

			// apid is found in bits 5-15 of the packet

			APID apid = result->get_apid();

//		DEBUG("ccsdsapidfilter::get_next_ccsds Read input packet, "
//		  << "apid is " << apid << endl);

			if (filters.empty() ||
				find(filters.begin(), filters.end(), apid) != filters.end())
			{
				// looks good
				pass_count++;
//		DEBUG("ccsdsapidfilter::get_next_ccsds returning valid "
//		<< "packet\n");
				return result;
			}
			else
			{
				// not on the filters list
//		DEBUG("ccsdsapidfilter::get_next_ccsds APDI does not "
//		<< "match filter\n");
				block_count++;
				return get_next_ccsds();
			}
		}

	//
	// produce_files
	//

	virtual bool produce_files(FileConsumer& dest)
		{
			CCSDS* ccsds;
			while((ccsds=get_next_ccsds()))
			{

				stringstream s;
				s << "." << ccsds->get_apid() << ".ccsds";
				ostream_type* os = dest.get_named_output_stream(s.str());
				os->write((char*)ccsds->get_data(),ccsds->get_length());
			}
			return true;
		}

	//
	// get_next_file
	//

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}

	//
	// apid_to_string
	//

	static string apid_to_string(APID apid)
		{
			APIDS apids = get_apids();
			APIDS::iterator iter=apids.end();

			do
			{
				iter--;
				if ( (*iter).second == apid)
				{
					return (*iter).first;
				}
			} while(iter != apids.begin());

			return string("<unknown>");
		}


	//
	// is_valid
	//

	bool is_valid(void) const
		{
			return valid;
		}

	EPSProducer* as_eps_product(void)
		{
			return source->as_eps_product();
		}

	//
	// check_apid
	//
	bool check_apid(APID apid)
		{

			if (filters.empty() ||
				find(filters.begin(), filters.end(), apid) != filters.end())
			{
				// looks good
				return true;
			}
			else
			{
				return false;
			}
		}

	void reset_stream(void)
		{
			source->reset_stream();
		}

	void delete_source(void)
		{
			source->delete_source();
			delete source;
		}

private:

	CCSDSProducer* source;

	typedef deque<APID> Filters;
	Filters filters;

	int pass_count;
	int block_count;

	bool valid;

};

#endif
