////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsgeneratorbase.hpp
//
//	Provides: CCSDSGeneratorBase ABC
//
//	Purpose: Base class for filters that generate CCSDS packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef sourcepacketgenerator
#define sourcepacketgenerator

#include "ccsds.hpp"
#include "noaa.hpp"

// this is an abstract base class for filters that generate CCSDS
// packets

template <class PacketType> class CCSDSGeneratorBase
{
public:

	//
	// ccsds_initialise
	//

	// resets the current packet (data member) to a blank state
	void ccsds_initialise()
		{
			assert(sizeof(PacketType) == PacketType::SIZE);

			memset(&ccsds_packet, 0, sizeof(ccsds_packet));
			ccsds_packet.primary_header.set_version_number(0);
			ccsds_packet.primary_header.set_type(0);
			ccsds_packet.primary_header.set_secondary_header_flag(1);

			ccsds_packet.primary_header.set_apid(PacketType::APID);

			ccsds_packet.primary_header.set_sequence_flags(3);
			ccsds_packet.primary_header.set_sequence_count(
				ccsds_sequence_count++);
			ccsds_packet.primary_header.set_packet_length(
				PacketType::SIZE - 7);

			const bool DEEP_COPY = false;
			ccsds_gen_packet.reset(
				new CCSDS(reinterpret_cast<byte*>(&ccsds_packet), DEEP_COPY ));
		}

	//
	// ccsds_set_time
	//

	// extract utc time from input struct input source packet
	// modified by current time offsets
	// placed results into utc and obt sections of packet

	void ccsds_set_time(const NOAA_L1b_GEN_Data& gen)
		{
			const int MILLISECONDS_PER_DAY = 1000*60*60*24;

			DEBUG("ccsds_set_time input year " << gen.scan_line_year
				  << " day " << gen.scan_line_day_of_year
				  << " milliseconds " << gen.scan_line_millisecond_of_day
				  << endl);

			int days = TimeConv::abs_year_day_to_utc_day(
				gen.scan_line_year,
				gen.scan_line_day_of_year);

			int milliseconds = gen.scan_line_millisecond_of_day;

			int microseconds = 0;

			DEBUG("decode days " << days << " milliseconds "
				  << milliseconds << endl);

			// set OBT now, before UTC offsets are applied
			int obt_coarse = TimeConv::utc_days_milliseconds_to_seconds(
				days, milliseconds);
			int obt_fine = static_cast<int>(
				((milliseconds%1000)*(1<<16))/1000.0);

			DEBUG("base coarse " << obt_coarse << " fine " << obt_fine
				  << " fixedfinetime " << ccsds_obt_fixed_fine_time << endl);

			if (ccsds_obt_fixed_fine_time)
			{
				obt_fine = ccsds_obt_fine_offset;
				DEBUG("<setting obt_fine to " << obt_fine << ">\n");
			}
			else
			{
				obt_fine += ccsds_obt_fine_offset;
				DEBUG("<setting2 obt_fine to " << obt_fine << ">\n");
			}

			obt_coarse += ccsds_obt_coarse_offset;

			while(obt_fine > (1<<16))
			{
				obt_fine -= (1<<16);
				DEBUG("<setting3 obt_fine to " << obt_fine << ">\n");
				obt_coarse++;
			}

			while(obt_fine < 0)
			{
				obt_fine += (1<<16);
				DEBUG("<setting4 obt_fine to " << obt_fine << ">\n");
				obt_coarse--;
			}

			DEBUG("obt_coarse modified is " << obt_coarse << " obt_fine is "
				  << obt_fine << endl);

			ccsds_packet.ancillary_data.obt_coarse = obt_coarse & ((1<<24)-1);
			ccsds_packet.ancillary_data.obt_fine = obt_fine;

			if (ccsds_utc_microseconds_offset >= 0)
			{
				microseconds += ccsds_utc_microseconds_offset;
				if (microseconds >= 1000)
				{
					milliseconds += static_cast<int>
						(microseconds/1000);
					microseconds = microseconds % 1000;
				}
			}
			else
			{
				microseconds = -(ccsds_utc_microseconds_offset%1000);
				milliseconds -= static_cast<int>
					((ccsds_utc_microseconds_offset/1000)+1);
			}

			milliseconds += ccsds_utc_milliseconds_offset;
			if (milliseconds > MILLISECONDS_PER_DAY)
			{
				days += static_cast<int>(milliseconds/MILLISECONDS_PER_DAY+1);
				milliseconds %= MILLISECONDS_PER_DAY;
			}
			if (milliseconds < 0)
			{
				days -= static_cast<int>((
											 milliseconds/MILLISECONDS_PER_DAY)+1);
				milliseconds = MILLISECONDS_PER_DAY-
					(abs(milliseconds)%MILLISECONDS_PER_DAY);
			}

			days += ccsds_utc_days_offset;

			ccsds_packet.secondary_header.set_utc_days(days);
			ccsds_packet.secondary_header.set_utc_milliseconds(milliseconds);
			ccsds_packet.secondary_header.set_utc_microseconds(microseconds);
		}

	//
	// ccsds_set_utc_offsets
	//

	// set offsets to be applied to utc time stamps in source packets

	void ccsds_set_utc_offsets(int days, int milliseconds, int microseconds)
		{
			ccsds_utc_days_offset = days;
			ccsds_utc_milliseconds_offset = milliseconds;
			ccsds_utc_microseconds_offset = microseconds;
		}

	//
	// ccsds_modify_utc_base_time
	//

	// set the zero time for source packet utc time stamps

	void ccsds_modify_utc_base_time(NOAA_L1b_GEN_Data& gen)
		{
			int days = TimeConv::abs_year_day_to_utc_day(
				gen.scan_line_year,
				gen.scan_line_day_of_year);

			int milliseconds = gen.scan_line_millisecond_of_day;

			ccsds_utc_days_offset -= days;
			ccsds_utc_milliseconds_offset -= milliseconds;
		}

	//
	// ccsds_set_obt_zero_time
	//

	// set the utc time to be considered 0/0 in the utc -> obt conversion

	void ccsds_set_obt_zero_time(NOAA_L1b_GEN_Data& gen)
		{
			int days = TimeConv::abs_year_day_to_utc_day(
				gen.scan_line_year,
				gen.scan_line_day_of_year);

			int milliseconds = gen.scan_line_millisecond_of_day;

			ccsds_obt_coarse_offset = -TimeConv::utc_days_milliseconds_to_seconds(
				days, milliseconds);

			ccsds_obt_fine_offset = -static_cast<int>(
				((milliseconds%1000)*(1<<16))/1000.0);
		}

	//
	// ccsds_set_obt_offsets
	//

	void ccsds_apply_obt_offsets(int coarse, int fine, bool fixed_fine)
		{
			ccsds_obt_fixed_fine_time = fixed_fine;
			ccsds_obt_coarse_offset += coarse;

			if (fixed_fine)
			{
				ccsds_obt_fine_offset = fine;
			}
			else
			{
				ccsds_obt_fine_offset += fine;
			}

			DEBUG("ccsds_apply_obt_offsets coarse becomes "
				  << ccsds_obt_coarse_offset << " fine becomes "
				  << ccsds_obt_fine_offset << " fixed fine is "
				  << ccsds_obt_fixed_fine_time << endl);
		}

	//
	// ccsds_set_sequence_count
	//

	// reset ccsds sequence count to parameter

	void ccsds_set_sequence_count(unsigned int sc)
		{
			ccsds_sequence_count = sc;
			ccsds_packet.primary_header.set_sequence_count(sc);
		}

	//
	// ccsds_finalise
	//

	CCSDS* ccsds_finalise(void)
		{
			// insert checksum
			ccsds_packet.error_control = ccsds_gen_packet->calculate_error_control();

			return ccsds_gen_packet.get();
		}

	void ccsds_set_obt_time(unsigned int obt_coarse, unsigned int obt_fine)
		{
			ccsds_packet.ancillary_data.obt_coarse = obt_coarse & ((1<<24)-1);
			ccsds_packet.ancillary_data.obt_fine = obt_fine;
		}

protected:

	PacketType ccsds_packet;
	unsigned int ccsds_sequence_count;
	auto_ptr<CCSDS> ccsds_gen_packet;

	int ccsds_utc_days_offset;
	int ccsds_utc_milliseconds_offset;
	int ccsds_utc_microseconds_offset;

	int ccsds_obt_coarse_offset;
	bool ccsds_obt_fixed_fine_time;
	int ccsds_obt_fine_offset;

};

#endif
