////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsiasiflagsfix.hpp
//
//	Provides: CCSDSIASIFlagsFix filter
//
//	Purpose:  Use to correct IASI CCSDS packet flag settings.
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsiasiflagsfix_defined
#define ccsdsiasiflagsfix_defined

#include "interface.hpp"
#include "ccsds.hpp"
#include "ccsdsproducer.hpp"
#include "ccsdsapidfilter.hpp"

class CCSDSIASIFlagsFix :
	virtual public Interface,
	public CCSDSProducer,
	public FileProducer
{
public:

	//
	// constructor
	//

	CCSDSIASIFlagsFix(CCSDSProducer& _source, bool _TYPE_ONLY = false, bool _SEQ_FLAG_ONLY = false)
		: source(&_source)
		, apid(0)
		, packet_count(0)
		, valid(true)
		, TYPE_ONLY(_TYPE_ONLY)
		, SEQ_FLAG_ONLY(_SEQ_FLAG_ONLY)
		, sequence_counter(0)
		, days(0)
		, milliseconds(0)
		, microseconds(0)
		{

			sequence_count_first = 0;
			utc_days_first = 0;
			utc_milliseconds_first = 0;
			utc_microseconds_first = 0;
			obt_coarse_first = 0;
			obt_fine_first = 0;

			sequence_count_last = 0;
			utc_days_last = 0;
			utc_milliseconds_last = 0;
			utc_microseconds_last = 0;
			obt_coarse_last = 0;
			obt_fine_last = 0;

		}


	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout)
		{
			os << "CCSDSIASIFlagsFix report:\n"
			   << "	   " << packet_count << " total number of packets read\n"
			   << "	   " << "First Packet\n"
			   << "		   " << "Sequence Count		  : " << sequence_count_first << endl
			   << "		   " << "Time				  : " << TimeConv::utc_days_milliseconds_to_string(utc_days_first,
																									   utc_milliseconds_first)
			   << "." << ntofs(utc_milliseconds_first%1000, 3, '0')
			   << ntofs(utc_microseconds_first, 3, '0')	 << endl
			   << "		   " << "UTC-Days			  : " << utc_days_first	 << endl
			   << "		   " << "UTC-Milliseconds	  : " << utc_milliseconds_first	 << endl
			   << "		   " << "UTC-Microseconds	  : " << utc_microseconds_first	 << endl
			   << "		   " << "OBT-Coarse (sec)	  : " << obt_coarse_first  << endl
			   << "		   " << "OBT-Fine (sec/65536) : " << obt_fine_first	 << endl
			   << "	   " << "Last Packet\n"
			   << "		   " << "Sequence Count		  : " << sequence_count_last  << endl
			   << "		   " << "Time				  : " << TimeConv::utc_days_milliseconds_to_string(utc_days_last,
																									   utc_milliseconds_last)
			   << "." << ntofs(utc_milliseconds_last%1000, 3, '0')
			   << ntofs(utc_microseconds_last, 3, '0') << endl
			   << "		   " << "UTC-Days			  : " << utc_days_last	<< endl
			   << "		   " << "UTC-Milliseconds	  : " << utc_milliseconds_last	<< endl
			   << "		   " << "UTC-Microseconds	  : " << utc_microseconds_last	<< endl
			   << "		   " << "OBT-Coarse (sec)	  : " << obt_coarse_last  << endl
			   << "		   " << "OBT-Fine (sec/65536) : " << obt_fine_last	<< endl << endl;
		}



	//
	// extract_iasi_verification_data
	//

	void extract_iasi_verification_data(CCSDS* packet, deque<byte>& iasi_data)
		{
			for(int i=0; i<(packet->get_payload_length() - IASI_Verif_AncillaryData::SIZE + OBTTime::SIZE); i++)
			{
				byte a = packet->get_payload(IASI_Verif_AncillaryData::SIZE - OBTTime::SIZE)[i];
				iasi_data.push_back(a);
			}
		}



	//
	// get_next_ccsds
	//

	virtual CCSDS* get_next_ccsds(void)
		{
			CCSDS* result = source->get_next_ccsds();

			if (!result)
			{
				return NULL;
			}

			apid = result->get_apid();

			if ( strncmp((CCSDSAPIDFilter::apid_to_string(apid)).c_str(),"IASI",4) != 0 )
			{
				error << "Error: APID mismatch - This code can only process IASI packets ("
					  << "Intrument APID Found:" << (CCSDSAPIDFilter::apid_to_string(apid)).c_str()
					  << " Packet Count:" << packet_count+1 << ")\n";
				valid = false;
				return NULL;
			}


			if (packet_count == 0)
			{
				sequence_count_first = result->get_sequence_count();
				utc_days_first = result->get_utc_days();
				utc_milliseconds_first = result->get_utc_milliseconds();
				utc_microseconds_first = result->get_utc_microseconds();
				obt_coarse_first = result->get_obt_coarse();
				obt_fine_first = result->get_obt_fine();
			}


			sequence_count_last = result->get_sequence_count();
			utc_days_last = result->get_utc_days();
			utc_milliseconds_last = result->get_utc_milliseconds();
			utc_microseconds_last = result->get_utc_microseconds();
			obt_coarse_last = result->get_obt_coarse();
			obt_fine_last = result->get_obt_fine();


			if (SEQ_FLAG_ONLY == false)
			{
				//Packet Primary Header, Type
				if (result->get_type() == 0)
				{
					result->set_type(1);
				}
				else
				{
					result->set_type(0);
				}
			}

			if (TYPE_ONLY == false)
			{
				// As requested, set sequence_flags to 3 - 10/11/2004
				result->set_sequence_flags(3);
			}

			if (TYPE_ONLY == true || SEQ_FLAG_ONLY == true)
			{
				// Update Packet Error Control field after updating the flags
				result->set_error_control(result->calculate_error_control());

				packet_count++;
				return result;
			}


			uint16_be tmp;

			// To handle IASI Spectrum Packets for pixels 1 to 4
			if (apid == 130 || apid == 135 || apid == 140 || apid == 145)
			{
				CCSDS_IASI_Spec_PX1* iasi = (CCSDS_IASI_Spec_PX1*)result->get_data();

				//Operational data word 3
				tmp = 0;

				//CD
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>15)&1) == 0)
					tmp = tmp | 0<<15;
				else
					tmp = tmp | 1<<15;
				//CSQ
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>14)&1) == 0)
					tmp = tmp | 1<<14;
				else
					tmp = tmp | 0<<14;
				//SQ1
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>13)&1) == 0)
					tmp = tmp | 1<<13;
				else
					tmp = tmp | 0<<13;
				//SQ2
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>12)&1) == 0)
					tmp = tmp | 1<<12;
				else
					tmp = tmp | 0<<12;
				//IEQ
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>11)&1) == 0)
					tmp = tmp | 1<<11;
				else
					tmp = tmp | 0<<11;
				//SN_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>7)&1) == 0)
					tmp = tmp | 1<<7;
				else
					tmp = tmp | 0<<7;
				//CD_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>6)&1) == 0)
					tmp = tmp | 1<<6;
				else
					tmp = tmp | 0<<6;
				//CSQ_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>5)&1) == 0)
					tmp = tmp | 1<<5;
				else
					tmp = tmp | 0<<5;
				//SP_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>4)&1) == 0)
					tmp = tmp | 1<<4;
				else
					tmp = tmp | 0<<4;
				//SQ1_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>3)&1) == 0)
					tmp = tmp | 1<<3;
				else
					tmp = tmp | 0<<3;
				//SQ2_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>2)&1) == 0)
					tmp = tmp | 1<<2;
				else
					tmp = tmp | 0<<2;
				//NS_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>1)&1) == 0)
					tmp = tmp | 1<<1;
				else
					tmp = tmp | 0<<1;
				//IEQ_NV
				if ( static_cast<uint>(iasi->ancillary_data.word[2]&1) == 0)
					tmp = tmp | 1;
				else
					tmp = tmp | 0;

				result->set_data(tmp, (2*2));


				//Status Area Word 1
				tmp = 0;

				//DPS LNR-Rx DVL flag
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>15)&1) == 0)
					tmp = tmp | 1<<15;
				else
					tmp = tmp | 0<<15;
				//DPS LNR-Rx VLN flag
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>14)&1) == 0)
					tmp = tmp | 1<<14;
				else
					tmp = tmp | 0<<14;
				//BBofFlagSpectNonQual
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>13)&1) == 0)
					tmp = tmp | 1<<13;
				else
					tmp = tmp | 0<<13;
				//BDcoFlagMasErrorPath_B1
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>12)&1) == 0)
					tmp = tmp | 1<<12;
				else
					tmp = tmp | 0<<12;
				//BDcoFlagMasErrorPath_B2
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>11)&1) == 0)
					tmp = tmp | 1<<11;
				else
					tmp = tmp | 0<<11;
				//BDcoFlagMasErrorPath_B3
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>10)&1) == 0)
					tmp = tmp | 1<<10;
				else
					tmp = tmp | 0<<10;
				//BDcoFlagMasOverflow_B1
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>9)&1) == 0)
					tmp = tmp | 1<<9;
				else
					tmp = tmp | 0<<9;
				//BDcoFlagMasOverflow_B2
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>8)&1) == 0)
					tmp = tmp | 1<<8;
				else
					tmp = tmp | 0<<8;
				//BDcoFlagMasOverflow_B3
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>7)&1) == 0)
					tmp = tmp | 1<<7;
				else
					tmp = tmp | 0<<7;
				//BDcoFlagMasEcret_B1
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>6)&1) == 0)
					tmp = tmp | 1<<6;
				else
					tmp = tmp | 0<<6;
				//BDcoFlagMasEcret_B2
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>5)&1) == 0)
					tmp = tmp | 1<<5;
				else
					tmp = tmp | 0<<5;
				//BDcoFlagMasEcret_B3
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>4)&1) == 0)
					tmp = tmp | 1<<4;
				else
					tmp = tmp | 0<<4;
				//BDcoFlagMasErrorNbWords
				if ( static_cast<uint>((iasi->ancillary_data.word[139]>>3)&1) == 0)
					tmp = tmp | 1<<3;
				else
					tmp = tmp | 0<<3;

				result->set_data(tmp, (139*2));



				//Status Area Word 2
				tmp = 0;

				//BSpkFlagSpik_B1
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>15)&1) == 0)
					tmp = tmp | 1<<15;
				else
					tmp = tmp | 0<<15;
				//BSpkFlagSpik_B2
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>14)&1) == 0)
					tmp = tmp | 1<<14;
				else
					tmp = tmp | 0<<14;
				//BSpkFlagSpik_B3
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>13)&1) == 0)
					tmp = tmp | 1<<13;
				else
					tmp = tmp | 0<<13;
				//BzpdFlagNzpdNonQualEW
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>12)&1) == 0)
					tmp = tmp | 1<<12;
				else
					tmp = tmp | 0<<12;
				//BIsiFlagErrorFft_B1
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>11)&1) == 0)
					tmp = tmp | 1<<11;
				else
					tmp = tmp | 0<<11;
				//BIsiFlagErrorFft_B2
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>10)&1) == 0)
					tmp = tmp | 1<<10;
				else
					tmp = tmp | 0<<10;
				//BIsiFlagErrorFft_B3
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>9)&1) == 0)
					tmp = tmp | 1<<9;
				else
					tmp = tmp | 0<<9;
				//BArcFlagCalSpectNonQual_B1
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>8)&1) == 0)
					tmp = tmp | 1<<8;
				else
					tmp = tmp | 0<<8;
				//BArcFlagCalSpectNonQual_B2
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>7)&1) == 0)
					tmp = tmp | 1<<7;
				else
					tmp = tmp | 0<<7;
				//BArcFlagCalSpectNonQual_B3
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>6)&1) == 0)
					tmp = tmp | 1<<6;
				else
					tmp = tmp | 0<<6;
				//BCodFlagFlood
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>5)&1) == 0)
					tmp = tmp | 1<<5;
				else
					tmp = tmp | 0<<5;
				//BDcoFlagErrorInterf_B1
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>4)&1) == 0)
					tmp = tmp | 1<<4;
				else
					tmp = tmp | 0<<4;
				//BDcoFlagErrorInterf_B2
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>3)&1) == 0)
					tmp = tmp | 1<<3;
				else
					tmp = tmp | 0<<3;
				//BDcoFlagErrorInterf_B3
				if ( static_cast<uint>((iasi->ancillary_data.word[140]>>2)&1) == 0)
					tmp = tmp | 1<<2;
				else
					tmp = tmp | 0<<2;

				result->set_data(tmp, (140*2));

			}

			// To handle IASI Image Packets
			if (apid == 150)
			{
				CCSDS_IASI_Image* iasi = (CCSDS_IASI_Image*)result->get_data();

				//Operational data word 3
				tmp = 0;

				//CD
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>15)&1) == 0)
					tmp = tmp | 0<<15;
				else
					tmp = tmp | 1<<15;
				//CSQ
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>14)&1) == 0)
					tmp = tmp | 1<<14;
				else
					tmp = tmp | 0<<14;
				//SQ1
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>13)&1) == 0)
					tmp = tmp | 1<<13;
				else
					tmp = tmp | 0<<13;
				//SQ2
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>12)&1) == 0)
					tmp = tmp | 1<<12;
				else
					tmp = tmp | 0<<12;
				//IEQ
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>11)&1) == 0)
					tmp = tmp | 1<<11;
				else
					tmp = tmp | 0<<11;
				//SN_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>7)&1) == 0)
					tmp = tmp | 1<<7;
				else
					tmp = tmp | 0<<7;
				//CD_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>6)&1) == 0)
					tmp = tmp | 1<<6;
				else
					tmp = tmp | 0<<6;
				//CSQ_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>5)&1) == 0)
					tmp = tmp | 1<<5;
				else
					tmp = tmp | 0<<5;
				//SP_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>4)&1) == 0)
					tmp = tmp | 1<<4;
				else
					tmp = tmp | 0<<4;
				//SQ1_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>3)&1) == 0)
					tmp = tmp | 1<<3;
				else
					tmp = tmp | 0<<3;
				//SQ2_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>2)&1) == 0)
					tmp = tmp | 1<<2;
				else
					tmp = tmp | 0<<2;
				//NS_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>1)&1) == 0)
					tmp = tmp | 1<<1;
				else
					tmp = tmp | 0<<1;
				//IEQ_NV
				if ( static_cast<uint>(iasi->ancillary_data.word[2]&1) == 0)
					tmp = tmp | 1;
				else
					tmp = tmp | 0;

				result->set_data(tmp, (2*2));


				//Image status
				tmp = 0;

				//IIS_SIZE
				tmp = tmp | (static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255))<<8;

				//IIS ADC Overflow Flag
				if ( static_cast<uint>((iasi->ancillary_data.word[10]>>7)&1) == 0)
					tmp = tmp | 1<<7;
				else
					tmp = tmp | 0<<7;
				//No. of IIS Samples Flag
				if ( static_cast<uint>((iasi->ancillary_data.word[10]>>6)&1) == 0)
					tmp = tmp | 1<<6;
				else
					tmp = tmp | 0<<6;

				result->set_data(tmp, (10*2));

			}


			// To handle IASI Verification Packets
			if (apid == 160)
			{
				CCSDS_IASI_Verif* iasi = (CCSDS_IASI_Verif*)result->get_data();

				//Operational data word 3
				tmp = 0;

				//CD
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>15)&1) == 0)
					tmp = tmp | 0<<15;
				else
					tmp = tmp | 1<<15;
				//CSQ
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>14)&1) == 0)
					tmp = tmp | 1<<14;
				else
					tmp = tmp | 0<<14;
				//SQ1
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>13)&1) == 0)
					tmp = tmp | 1<<13;
				else
					tmp = tmp | 0<<13;
				//SQ2
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>12)&1) == 0)
					tmp = tmp | 1<<12;
				else
					tmp = tmp | 0<<12;
				//IEQ
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>11)&1) == 0)
					tmp = tmp | 1<<11;
				else
					tmp = tmp | 0<<11;
				//SN_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>7)&1) == 0)
					tmp = tmp | 1<<7;
				else
					tmp = tmp | 0<<7;
				//CD_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>6)&1) == 0)
					tmp = tmp | 1<<6;
				else
					tmp = tmp | 0<<6;
				//CSQ_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>5)&1) == 0)
					tmp = tmp | 1<<5;
				else
					tmp = tmp | 0<<5;
				//SP_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>4)&1) == 0)
					tmp = tmp | 1<<4;
				else
					tmp = tmp | 0<<4;
				//SQ1_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>3)&1) == 0)
					tmp = tmp | 1<<3;
				else
					tmp = tmp | 0<<3;
				//SQ2_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>2)&1) == 0)
					tmp = tmp | 1<<2;
				else
					tmp = tmp | 0<<2;
				//NS_NV
				if ( static_cast<uint>((iasi->ancillary_data.word[2]>>1)&1) == 0)
					tmp = tmp | 1<<1;
				else
					tmp = tmp | 0<<1;
				//IEQ_NV
				if ( static_cast<uint>(iasi->ancillary_data.word[2]&1) == 0)
					tmp = tmp | 1;
				else
					tmp = tmp | 0;

				result->set_data(tmp, (2*2));


				//Verification Status
				tmp = 0;

				//Packet Identifier
				tmp = tmp | (static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255))<<8;

				//BZpdFlagNzpdNonQual
				if ( static_cast<uint>((iasi->ancillary_data.word[10]>>7)&1) == 0)
					tmp = tmp | 1<<7;
				else
					tmp = tmp | 0<<7;
				//BdcoFlagMasErrorNbWords
				if ( static_cast<uint>((iasi->ancillary_data.word[10]>>6)&1) == 0)
					tmp = tmp | 1<<6;
				else
					tmp = tmp | 0<<6;

				result->set_data(tmp, (10*2));


				// Packet Identifier
				int packetID = static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255);

				//cout << "packetID = "	 << packetID << endl;
				if (packetID == 1)
				{
					// Extract Payload Data
					deque<byte> payload_data;
					extract_iasi_verification_data(result, payload_data);

					int payload_data_size = (iasi->ancillary_data.word[14] + (iasi->ancillary_data.word[14] % 2)) / 2;
					uint16_be *data = new uint16_be[payload_data_size];
					copy_n(payload_data.begin(), (payload_data_size*2), (byte*)(data));

					for (int i=100; i<payload_data_size; i++)
					{
						tmp = data[i];

						//Spike Detection Flag
						if ( static_cast<uint>((data[i]>>15)&1) == 0 )
							tmp = tmp | 1<<15;
						else
							tmp = tmp | 0<<15;
						//ADC Overflow Flag
						if ( static_cast<uint>((data[i]>>14)&1) == 0 )
							tmp = tmp | 1<<14;
						else
							tmp = tmp | 0<<14;
						//Amplification Path & Interferogram Sample Value
						tmp = tmp | (static_cast<uint>(data[i]&16383));

						result->set_data(tmp, ((30+i)*2));

					}

					delete[] data;
				}

				if (packetID == 2)
				{
					// Extract Payload Data
					deque<byte> payload_data;
					extract_iasi_verification_data(result, payload_data);

					int payload_data_size = (iasi->ancillary_data.word[14] - (iasi->ancillary_data.word[14] % 2)) / 2;
					uint16_be *data = new uint16_be[payload_data_size];
					copy_n(payload_data.begin(), (payload_data_size*2), (byte*)(data));


					for (int i=0; i<payload_data_size; i++)
					{
						tmp = data[i];

						//Spike Detection Flag
						if ( static_cast<uint>((data[i]>>15)&1) == 0 )
							tmp = tmp | 1<<15;
						else
							tmp = tmp | 0<<15;
						//ADC Overflow Flag
						if ( static_cast<uint>((data[i]>>14)&1) == 0 )
							tmp = tmp | 1<<14;
						else
							tmp = tmp | 0<<14;
						//Amplification Path & Interferogram Sample Value
						tmp = tmp | (static_cast<uint>(data[i]&16383));

						result->set_data(tmp, ((30+i)*2));
					}

					delete[] data;

				}


			}


			// To handle IASI Auxiliary Packets
			if (apid == 180)
			{
				CCSDS_IASI_Anc* iasi = (CCSDS_IASI_Anc*)result->get_data();


				//OTM data, OTM_NV
				tmp = 0;

				//BBT Validity Flag
				if ( static_cast<uint>((iasi->ancillary_data.word[17]>>15)&1) == 0)
					tmp = tmp | 1<<15;
				else
					tmp = tmp | 0<<15;
				//FPT Validity Flag
				if ( static_cast<uint>((iasi->ancillary_data.word[17]>>14)&1) == 0)
					tmp = tmp | 1<<14;
				else
					tmp = tmp | 1<<14;
				//HAUT Validity Flag
				if ( static_cast<uint>((iasi->ancillary_data.word[17]>>13)&1) == 0)
					tmp = tmp | 1<<13;
				else
					tmp = tmp | 1<<13;
				//IFPT Validity Flag
				if ( static_cast<uint>((iasi->ancillary_data.word[17]>>12)&1) == 0)
					tmp = tmp | 1<<12;
				else
					tmp = tmp | 1<<12;
				//OBPT Validity Flag
				if ( static_cast<uint>((iasi->ancillary_data.word[17]>>11)&1) == 0)
					tmp = tmp | 1<<11;
				else
					tmp = tmp | 1<<11;
				//CCAT Validity Flag
				if ( static_cast<uint>((iasi->ancillary_data.word[17]>>10)&1) == 0)
					tmp = tmp | 1<<10;
				else
					tmp = tmp | 1<<10;

				result->set_data(tmp, (17*2));



				//Application Data
				int pixCtr;
				int snCtr;
				int wordCtr = 0;

				for (pixCtr=1; pixCtr<=4; pixCtr++)
				{
					DEBUG("Main Data Area for Pixel " << pixCtr << endl);

					for (snCtr=32; snCtr<=36; snCtr++)
					{
						if	(snCtr == 34)
						{
							snCtr++;
						}

						DEBUG("Step Number (SN) = " << snCtr << endl);


						//Operational data word 3
						tmp = 0;

						//CD
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>15)&1) == 0)
							tmp = tmp | 0<<15;
						else
							tmp = tmp | 1<<15;
						//CSQ
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>14)&1) == 0)
							tmp = tmp | 1<<14;
						else
							tmp = tmp | 0<<14;
						//SQ1
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>13)&1) == 0)
							tmp = tmp | 1<<13;
						else
							tmp = tmp | 0<<13;
						//SQ2
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>12)&1) == 0)
							tmp = tmp | 1<<12;
						else
							tmp = tmp | 0<<12;
						//IEQ
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>11)&1) == 0)
							tmp = tmp | 1<<11;
						else
							tmp = tmp | 0<<11;
						//SN_NV
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>7)&1) == 0)
							tmp = tmp | 1<<7;
						else
							tmp = tmp | 0<<7;
						//CD_NV
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>6)&1) == 0)
							tmp = tmp | 1<<6;
						else
							tmp = tmp | 0<<6;
						//CSQ_NV
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>5)&1) == 0)
							tmp = tmp | 1<<5;
						else
							tmp = tmp | 0<<5;
						//SP_NV
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>4)&1) == 0)
							tmp = tmp | 1<<4;
						else
							tmp = tmp | 0<<4;
						//SQ1_NV
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>3)&1) == 0)
							tmp = tmp | 1<<3;
						else
							tmp = tmp | 0<<3;
						//SQ2_NV
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>2)&1) == 0)
							tmp = tmp | 1<<2;
						else
							tmp = tmp | 0<<2;
						//NS_NV
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>1)&1) == 0)
							tmp = tmp | 1<<1;
						else
							tmp = tmp | 0<<1;
						//IEQ_NV
						if ( static_cast<uint>(iasi->payload.word[wordCtr+2]&1) == 0)
							tmp = tmp | 1;
						else
							tmp = tmp | 0;

						result->set_data(tmp, ((62+wordCtr)*2));

						wordCtr = wordCtr + 15;
					}
				}


				wordCtr = 240;
				for (pixCtr=1; pixCtr<=4; pixCtr++)
				{
					DEBUG("Main Status Area for Pixel " << pixCtr << endl);

					for (snCtr=32; snCtr<=36; snCtr++)
					{
						if	(snCtr == 34)
						{
							snCtr++;
						}

						DEBUG("Step Number (SN) = " << snCtr << endl);

						//Status Area Word 1
						tmp = 0;

						//DPS LNR-Rx DVL flag
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>15)&1) == 0)
							tmp = tmp | 1<<15;
						else
							tmp = tmp | 0<<15;
						//DPS LNR-Rx VLN flag
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>14)&1) == 0)
							tmp = tmp | 1<<14;
						else
							tmp = tmp | 0<<14;
						//BBofFlagSpectNonQual
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>13)&1) == 0)
							tmp = tmp | 1<<13;
						else
							tmp = tmp | 0<<13;
						//BDcoFlagMasErrorPath_B1
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>12)&1) == 0)
							tmp = tmp | 1<<12;
						else
							tmp = tmp | 0<<12;
						//BDcoFlagMasErrorPath_B2
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>11)&1) == 0)
							tmp = tmp | 1<<11;
						else
							tmp = tmp | 0<<11;
						//BDcoFlagMasErrorPath_B3
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>10)&1) == 0)
							tmp = tmp | 1<<10;
						else
							tmp = tmp | 0<<10;
						//BDcoFlagMasOverflow_B1
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>9)&1) == 0)
							tmp = tmp | 1<<9;
						else
							tmp = tmp | 0<<9;
						//BDcoFlagMasOverflow_B2
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>8)&1) == 0)
							tmp = tmp | 1<<8;
						else
							tmp = tmp | 0<<8;
						//BDcoFlagMasOverflow_B3
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>7)&1) == 0)
							tmp = tmp | 1<<7;
						else
							tmp = tmp | 0<<7;
						//BDcoFlagMasEcret_B1
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>6)&1) == 0)
							tmp = tmp | 1<<6;
						else
							tmp = tmp | 0<<6;
						//BDcoFlagMasEcret_B2
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>5)&1) == 0)
							tmp = tmp | 1<<5;
						else
							tmp = tmp | 0<<5;
						//BDcoFlagMasEcret_B3
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>4)&1) == 0)
							tmp = tmp | 1<<4;
						else
							tmp = tmp | 0<<4;
						//BDcoFlagMasErrorNbWords
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>3)&1) == 0)
							tmp = tmp | 1<<3;
						else
							tmp = tmp | 0<<3;
						//BdcoFlagErrorInterf_B1
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>2)&1) == 0)
							tmp = tmp | 1<<2;
						else
							tmp = tmp | 0<<2;
						//BdcoFlagErrorInterf_B2
						if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>1)&1) == 0)
							tmp = tmp | 1<<1;
						else
							tmp = tmp | 0<<1;
						//BdcoFlagErrorInterf_B3
						if ( static_cast<uint>(iasi->payload.word[wordCtr+1]&1) == 0)
							tmp = tmp | 1;
						else
							tmp = tmp | 0;

						result->set_data(tmp, ((61+wordCtr)*2)); //61


						//Status Area Word 2
						tmp = 0;

						//BnlcFlagIntegrity_B1
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>15)&1) == 0)
							tmp = tmp | 0<<15;
						else
							tmp = tmp | 1<<15;
						//BnlcFlagIntegrity_B2
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>14)&1) == 0)
							tmp = tmp | 0<<14;
						else
							tmp = tmp | 1<<14;
						//BnlcFlagIntegrity_B3
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>13)&1) == 0)
							tmp = tmp | 0<<13;
						else
							tmp = tmp | 1<<13;
						//BSpkFlagSpik_B1
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>12)&1) == 0)
							tmp = tmp | 1<<12;
						else
							tmp = tmp | 0<<12;
						//BSpkFlagSpik_B2
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>11)&1) == 0)
							tmp = tmp | 1<<11;
						else
							tmp = tmp | 0<<11;
						//BSpkFlagSpik_B3
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>10)&1) == 0)
							tmp = tmp | 1<<10;
						else
							tmp = tmp | 0<<10;
						//BzpdFlagNzpdNonQual
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>9)&1) == 0)
							tmp = tmp | 1<<9;
						else
							tmp = tmp | 0<<9;
						//BIrsFlagSrdNonIntegrity
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>8)&1) == 0)
							tmp = tmp | 1<<8;
						else
							tmp = tmp | 0<<8;
						//BIsiFlagErrorFft_B1
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>7)&1) == 0)
							tmp = tmp | 1<<7;
						else
							tmp = tmp | 0<<7;
						//BIsiFlagErrorFft_B2
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>6)&1) == 0)
							tmp = tmp | 1<<6;
						else
							tmp = tmp | 0<<6;
						//BIsiFlagErrorFft_B3
						if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>5)&1) == 0)
							tmp = tmp | 1<<5;
						else
							tmp = tmp | 0<<5;

						result->set_data(tmp, ((62+wordCtr)*2)); //62


						wordCtr = wordCtr + 4;
					}


					DEBUG("Extra Status Area\n");


					//Extra Status Area Word 1
					tmp = 0;

					//BBofFlagSrdInit_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr]>>15)&1) == 0)
						tmp = tmp | 1<<15;
					else
						tmp = tmp | 0<<15;
					//BBofFlagSrdInit_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr]>>14)&1) == 0)
						tmp = tmp | 1<<14;
					else
						tmp = tmp | 0<<14;
					//BBofFlagSrdNonUpdate_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr]>>13)&1) == 0)
						tmp = tmp | 1<<13;
					else
						tmp = tmp | 0<<13;
					//BBofFlagSrdNonUpdate_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr]>>12)&1) == 0)
						tmp = tmp | 1<<12;
					else
						tmp = tmp | 0<<12;
					//BBofFlagCoefCalInit_B1_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr]>>11)&1) == 0)
						tmp = tmp | 1<<11;
					else
						tmp = tmp | 0<<11;
					//BBofFlagCoefCalInit_B2_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr]>>10)&1) == 0)
						tmp = tmp | 1<<10;
					else
						tmp = tmp | 0<<10;
					//BBofFlagCoefCalInit_B3_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr]>>9)&1) == 0)
						tmp = tmp | 1<<9;
					else
						tmp = tmp | 0<<9;


					result->set_data(tmp, ((60+wordCtr)*2));



					//Extra Status Area Word 2
					tmp = 0;

					//BBofFlagCoefCalInit_B1_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>15)&1) == 0)
						tmp = tmp | 1<<15;
					else
						tmp = tmp | 0<<15;
					//BBofFlagCoefCalInit_B2_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>14)&1) == 0)
						tmp = tmp | 1<<14;
					else
						tmp = tmp | 0<<14;
					//BBofFlagCoefCalInit_B3_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>13)&1) == 0)
						tmp = tmp | 1<<13;
					else
						tmp = tmp | 0<<13;
					//BBofFlagCoefCalNonUpdate_B1_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>12)&1) == 0)
						tmp = tmp | 1<<12;
					else
						tmp = tmp | 0<<12;
					//BBofFlagCoefCalNonUpdate_B2_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>11)&1) == 0)
						tmp = tmp | 1<<11;
					else
						tmp = tmp | 0<<11;
					//BBofFlagCoefCalNonUpdate_B3_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>10)&1) == 0)
						tmp = tmp | 1<<10;
					else
						tmp = tmp | 0<<10;
					//BBofFlagCoefCalNonUpdate_B1_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>9)&1) == 0)
						tmp = tmp | 1<<9;
					else
						tmp = tmp | 0<<9;
					//BBofFlagCoefCalNonUpdate_B2_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>8)&1) == 0)
						tmp = tmp | 1<<8;
					else
						tmp = tmp | 0<<8;
					//BBofFlagCoefCalNonUpdate_B3_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>7)&1) == 0)
						tmp = tmp | 1<<7;
					else
						tmp = tmp | 0<<7;
					//BRciFlagNonIntegritySlope_B1_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>6)&1) == 0)
						tmp = tmp | 1<<6;
					else
						tmp = tmp | 0<<6;
					//BRciFlagNonIntegritySlope_B2_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>5)&1) == 0)
						tmp = tmp | 1<<5;
					else
						tmp = tmp | 0<<5;
					//BRciFlagNonIntegritySlope_B3_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>4)&1) == 0)
						tmp = tmp | 1<<4;
					else
						tmp = tmp | 0<<4;
					//BRciFlagNonIntegritySlope_B1_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>3)&1) == 0)
						tmp = tmp | 1<<3;
					else
						tmp = tmp | 0<<3;
					//BRciFlagNonIntegritySlope_B2_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>2)&1) == 0)
						tmp = tmp | 1<<2;
					else
						tmp = tmp | 0<<2;
					//BRciFlagNonIntegritySlope_B3_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+1]>>1)&1) == 0)
						tmp = tmp | 1<<1;
					else
						tmp = tmp | 0<<1;


					result->set_data(tmp, ((61+wordCtr)*2));


					//Extra Status Area Word 3
					tmp = 0;

					//BRciFlagNonIntegrityOffset_B1_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>15)&1) == 0)
						tmp = tmp | 1<<15;
					else
						tmp = tmp | 0<<15;
					//BRciFlagNonIntegrityOffset_B2_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>14)&1) == 0)
						tmp = tmp | 1<<14;
					else
						tmp = tmp | 0<<14;
					//BRciFlagNonIntegrityOffset_B3_CD0
					if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>13)&1) == 0)
						tmp = tmp | 1<<13;
					else
						tmp = tmp | 0<<13;
					//BRciFlagNonIntegrityOffset_B1_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>12)&1) == 0)
						tmp = tmp | 1<<12;
					else
						tmp = tmp | 0<<12;
					//BRciFlagNonIntegrityOffset_B2_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>11)&1) == 0)
						tmp = tmp | 1<<11;
					else
						tmp = tmp | 0<<11;
					//BRciFlagNonIntegrityOffset_B3_CD1
					if ( static_cast<uint>((iasi->payload.word[wordCtr+2]>>10)&1) == 0)
						tmp = tmp | 1<<10;
					else
						tmp = tmp | 0<<10;

					result->set_data(tmp, ((62+wordCtr)*2));


					wordCtr = wordCtr + 4;

				}

			}


			// Update Packet Error Control field after updating the flags
			result->set_error_control(result->calculate_error_control());

			packet_count++;
			return result;

		}




	//
	// produce_files
	//

	virtual bool produce_files(FileConsumer& dest)
		{
			CCSDS* ccsds;
			while((ccsds=get_next_ccsds()))
			{

				stringstream s;
				s << "." << ccsds->get_apid() << ".ccsds";
				ostream_type* os = dest.get_named_output_stream(s.str());
				os->write((char*)ccsds->get_data(),ccsds->get_length());
			}
			return true;
		}



	//
	// get_next_file
	//

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}



	//
	// is_valid
	//

	bool is_valid(void) const
		{
			return valid;
		}


	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:

	CCSDSProducer* source;
	APID apid;

	int packet_count;
	bool valid;
	bool TYPE_ONLY;
	bool SEQ_FLAG_ONLY;

	unsigned int sequence_counter;
	unsigned int days;
	unsigned int milliseconds;
	unsigned int microseconds;

	unsigned int sequence_count_first;
	unsigned int utc_days_first;
	unsigned int utc_milliseconds_first;
	unsigned int utc_microseconds_first;
	unsigned int obt_coarse_first;
	unsigned int obt_fine_first;

	unsigned int sequence_count_last;
	unsigned int utc_days_last;
	unsigned int utc_milliseconds_last;
	unsigned int utc_microseconds_last;
	unsigned int obt_coarse_last;
	unsigned int obt_fine_last;

};

#endif

