////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsimageavhrr.hpp
//
//	Provides: CCSDSImageAVHRR filter
//
//	Purpose: Extract images from AVHRR packets in a stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsimageavhrr_defined
#define ccsdsimageavhrr_defined

#include "interface.hpp"
#include "ccsdsproducer.hpp"
#include "imageproducer.hpp"
#include "avhrrimagebase.hpp"

class CCSDSImageAvhrr :
	virtual public Interface,
	public AVHRRImageBase,
	public ImageProducer
{
public:

	enum { SCAN_LINE_WIDTH = 2048 };

	//
	// constructor
	//

	CCSDSImageAvhrr(CCSDSProducer& _source)
		: source(_source)
		{
		}

	//
	// prepare_images
	//

	virtual void prepare_images(void)
		{
//		Spinner::enable(); // ccsds_displayer.cpp switches it off
			spinner.reset(new Spinner("Reading in image matrix"));

			// read in the image data here
			while(CCSDS* ccsds = source.get_next_ccsds())
			{
				// got next packet
				if (ccsds->get_apid()==APID_AVHRR_3A ||
					ccsds->get_apid()==APID_AVHRR_3B)
				{
					// prepare to read earth scan

					// skip space_view*10 and ramp_calibration
					// 10 samples * 5 channels * 10bps = 550 bits
					// = 68 bytes 6 bits

					byte* payload_byte = ccsds->get_payload() + 68;
					int payload_bit = 7-6;

					// prepare new scan line
					avhrr_scan_lines.push_back(AVHRRScanLine());
					AVHRRScanLine& line = avhrr_scan_lines.back();
					line.resize(SCAN_LINE_WIDTH);
//			cout << "size if " << line.size() << endl;

					// read line
					for(int x=0; x<SCAN_LINE_WIDTH; x++)
					{
						AVHRRSample sample;
						memset(&sample, 0, sizeof(sample));
						for(int channel = 0; channel<5; channel++)
						{
							for(int bit = 0; bit<10; bit++)
							{
								sample.channels[channel] <<= 1;
								if ( ((*payload_byte)&(1<<payload_bit)) > 0 )
								{
									sample.channels[channel] += 1;
								}
								if (payload_bit > 0)
								{
									payload_bit--;
								}
								else
								{
									payload_bit = 7;
									payload_byte++;
								}
							}
						}
						line[x] = sample;
					}
				}
				spinner->tick();
			}
//		info << "done. " << avhrr_scan_lines.size() << " scan lines found\n";
			spinner->end();
		}

	//
	// get_image_count
	//

	virtual int get_image_count(void)
		{
			return AVHRRImageBase::get_image_count();
		}

	//
	// write_image
	//

	virtual void write_image(int number, Image* image)
		{
			AVHRRImageBase::write_image(number, image);
		}

	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout)
		{
			os << "CCSDSImageAVHRR report:\n"
			   << "	   Image width " << SCAN_LINE_WIDTH << endl
			   << "	   Image height " << avhrr_scan_lines.size() << endl
			   << endl;
		}

private:

	CCSDSProducer& source;
};

#endif

