////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsimageiasi.hpp
//
//	Provides: CCSDSImageIASI filter
//
//	Purpose: Extract images from IASI packets in a stream
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsimageiasi_defined
#define ccsdsimageiasi_defined

#include "interface.hpp"
#include "ccsdsproducer.hpp"
#include "imageproducer.hpp"
#include "textsummaryproducer.hpp"

class CCSDSImageIASI :
	virtual public Interface,
	public ImageProducer
{
public:

	typedef unsigned short Sample;
	enum { IIS_CELL_WIDTH = 64 };
	enum { IIS_CELL_HEIGHT = 64 };
	enum { IIS_LINE_WIDTH = IIS_CELL_WIDTH*30 };

	//
	// constructor
	//

	CCSDSImageIASI(CCSDSProducer& _source, bool _hflip, bool _vflip, bool _invert, bool _iis_trim)
		: source(_source),
		  hflip(_hflip),
		  vflip(_vflip),
		  invert(_invert),
		  iis_trim(_iis_trim),
		  image_packets(0)
		{
			if (iis_trim)
			{
				iis_line_offset = 8;
				iis_line_height = 48;
			}
			else
			{
				iis_line_offset = 0;
				iis_line_height = IIS_CELL_HEIGHT;
			}
		}

	//
	// prepare_images
	//

	virtual void prepare_images(void)
		{
			min_sample = numeric_limits<Sample>::max();
			max_sample = numeric_limits<Sample>::min();

			CCSDSPacketDisplayer displayer(source);

			spinner.reset(new Spinner("Reading in image matrix"));

			initial_line_number = -1;
			int previous_scanline_number = -1;

			// read in the image data here
			while(CCSDS* ccsds = source.get_next_ccsds())
			{
				// got next packet
				if (ccsds->get_apid()==CCSDS_IASI_Image::APID)
				{
					CCSDS_IASI_Image* iasi = (CCSDS_IASI_Image*)ccsds->get_data();
					//int num_samples = iasi->ancillary_data.word[0];
					int scan_position = static_cast<uint>(iasi->ancillary_data.word[1]&255);
					int line_number = iasi->ancillary_data.word[7];
					//cout << "scan position " << scan_position << " line number " << line_number << endl;

					if (scan_position <= 30)
					{

						if (initial_line_number==-1)
						{
							initial_line_number = line_number;
							//cout << "set initial line number to " << initial_line_number << endl;
						}

						if (line_number < initial_line_number)
						{
							initial_line_number = line_number - previous_scanline_number - 1;
							//cout << "reset initial line number to " << initial_line_number << endl;
						}
						int scanline_number = line_number - initial_line_number;

						//cout << "scanline number " << scanline_number << " scanlines.size " << scanlines.size() << endl;
						if (scanlines.size() < (scanline_number+1))
						{
							//cout << "resizing scanlines from " << scanlines.size() << " to " << (scanline_number+1) << endl;
							scanlines.resize(scanline_number+1);
						}

						// Extract Payload Data
						deque<byte> payload_data;
						displayer.extract_iasi_image_data(ccsds, payload_data);

						// should be able to use bits_per_image_sample = IIS_SIZE = static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255),
						// but the test data is of poor quality and does not allow this, hence use size of data instead.
						unsigned int bits_per_image_sample = 0;
						if ((ccsds->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 4100)
						{
							bits_per_image_sample = 8;
						}
						if ((ccsds->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 5152)
						{
							bits_per_image_sample = 10;
						}
						if ((ccsds->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 6150)
						{
							bits_per_image_sample = 12;
						}

						if ( bits_per_image_sample != static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255) )
						{
							cerr << "\n*** WARNING: IIS_SIZE and the calculated bits per image sample ("
								 << bits_per_image_sample
								 << ") do not match.\n\n";
						}

						Sample raw_samples[4096];
						int raw_sample_pos = 0;

						int payload_data_size = ((4100*bits_per_image_sample)/16)*2; //*8, *10, *12

						if (bits_per_image_sample == 8)
						{
							uint16_be data[2050];
							copy_n(payload_data.begin(), payload_data_size, (byte*)(&data));

							int ctr = 2;
							for (int i=0; i<1024; i++)
							{
								raw_samples[raw_sample_pos++] = static_cast<uint>((data[ctr]>>8)&255);
								raw_samples[raw_sample_pos++] = static_cast<uint>(data[ctr]&255);
								raw_samples[raw_sample_pos++] = static_cast<uint>((data[ctr+1]>>8)&255);
								raw_samples[raw_sample_pos++] = static_cast<uint>(data[ctr+1]&255);
								ctr = ctr + 2;
							}
						}
						else if (bits_per_image_sample == 10)
						{
							uint16_be data[2563];
							copy_n(payload_data.begin(), payload_data_size, (byte*)(&data));
							int offset = 16;
							int ctr2 = 0;

							int ctr = 2;
							for (int i=0; i<1024; i++)
							{
								raw_samples[raw_sample_pos++] = displayer.extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2);
								raw_samples[raw_sample_pos++] = displayer.extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2);
								raw_samples[raw_sample_pos++] = displayer.extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2);
								raw_samples[raw_sample_pos++] = displayer.extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2);
								ctr = ctr + 2;
							}
						}
						else if (bits_per_image_sample == 12)
						{
							uint16_be data[3075];
							copy_n(payload_data.begin(), payload_data_size, (byte*)(&data));
							int offset = 16;
							int ctr2 = 3;

							for (int i=0; i<1024; i++)
							{
								raw_samples[raw_sample_pos++] = displayer.extract_iasi_image(data,
																							 &offset,
																							 bits_per_image_sample,
																							 &ctr2);
								raw_samples[raw_sample_pos++] = displayer.extract_iasi_image(data,
																							 &offset,
																							 bits_per_image_sample,
																							 &ctr2);
								raw_samples[raw_sample_pos++] = displayer.extract_iasi_image(data,
																							 &offset,
																							 bits_per_image_sample,
																							 &ctr2);
								raw_samples[raw_sample_pos++] = displayer.extract_iasi_image(data,
																							 &offset,
																							 bits_per_image_sample,
																							 &ctr2);
							}
						}

						//cout << "total raw samples " << raw_sample_pos << endl;
						raw_sample_pos = 0;

						//cout << "accessing scanline " << scanline_number << endl;

						//cout << "line number " << line_number << " initial line number " << initial_line_number
						//<< " scanline " << scanline_number << endl;

						Scanline& scanline = scanlines[scanline_number];
						for(int j=0; j<IIS_CELL_HEIGHT; j++)
						{
							for(int i=0; i<IIS_CELL_WIDTH; i++)
							{
								Sample colour = raw_samples[raw_sample_pos++];

								scanline.samples[(scan_position-1)*IIS_CELL_WIDTH+j][i] = colour;
								if (colour < min_sample)
								{
									min_sample = colour;
								}
								if (colour > max_sample)
								{
									max_sample = colour;
								}
							}
						}
						previous_scanline_number = scanline_number;


					}

				}
				spinner->tick();
			}
			spinner->end();
		}

	//
	// get_image_count
	//

	virtual int get_image_count(void)
		{
			return 1;
		}

	//
	// write_image
	//

	virtual void write_image(int number, Image* image)
		{
			image->set_filename_suffix("");
			image->set_attributes( Image::IMAGE_MONO,
								   IIS_LINE_WIDTH,
								   scanlines.size()*iis_line_height);
			image->begin_output();
			for(int scanline_number=0; scanline_number<scanlines.size(); scanline_number++)
			{
				for(int y=iis_line_offset; y<(iis_line_height+iis_line_offset); y++)
				{
					byte row[IIS_LINE_WIDTH];
					for(int x=0; x<IIS_LINE_WIDTH; x++)
					{
						int j=hflip ? (IIS_LINE_WIDTH-x-1) : x;
						int i=vflip ? (IIS_CELL_HEIGHT-y-1) : y;
						Scanline& scanline = scanlines[ vflip ? scanlines.size()-scanline_number-1 : scanline_number ];
						Sample sample = scanline.samples[j][i];
						int colour = static_cast<byte>((sample-min_sample)*255.0/(max_sample-min_sample));
						if (invert)
						{
							colour = 255-colour;
						}
						row[x] = colour;
					}
					image->write_line(row);
				}
			}
			image->end_output();
		}

	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout)
		{
			os << "CCSDSImageIASI report:\n"
			   << "	   Initial line number " << initial_line_number << endl
			   << "	   Image packets " << image_packets << endl
			   << "	   Image width " << IIS_LINE_WIDTH << endl
			   << "	   Image height " << (scanlines.size()*iis_line_height) << endl
			   << endl;
		}

private:

	CCSDSProducer& source;
	bool hflip;
	bool vflip;
	bool invert;
	bool iis_trim;
	int iis_line_offset;
	int iis_line_height;

	int image_packets;

	struct Scanline
	{
		Sample samples[IIS_LINE_WIDTH][IIS_CELL_HEIGHT];
	};

	typedef vector<Scanline> Scanlines;
	Scanlines scanlines;

	int initial_line_number;
	Sample min_sample;
	Sample max_sample;
};

#endif
