////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsreader.cpp
//
//	Provides: CCSDSReader filter
//
//	Purpose: Extract CCSDS packets from a plain binary stream
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsdsreader.hpp"
#include "eps.hpp"

#ifdef PYMETOPIZER
#include "pymetopizer.hpp"
#endif

// allowable error in ms for which we still match an MDR to the obt2utc VIADR
const int VIADR_MATCH_THRESHOLD = 1;

CCSDSReader::CCSDSReader(FileProducer& _source, int _prepacket_header, bool _show_viadr)
	: source(&_source),
	  prepacket_header(_prepacket_header),
	  byte_count(0),
	  packet_count(0),
	  is(NULL),
	  hold(NULL),
	  eps(false),
	  show_viadr(_show_viadr)
{
	DEBUG("ccsdsreader constructor\n");

	grh_mphr = make_matching_rec_head();
	grh_mphr.record_class = 1;
	grh_mphr.instrument_group = 0;
	grh_mphr.record_subclass = 0;
	grh_mphr.record_subclass_version = 2;
	grh_mphr.record_size = 3307;

	grh_ipr = make_matching_rec_head();
	grh_ipr.record_class = 3;
	grh_ipr.instrument_group = 0;
	grh_ipr.record_subclass_version = 1;
	grh_ipr.record_size = 27;

	grh_viadr_l0_obt2utc = make_matching_rec_head();
	grh_viadr_l0_obt2utc.record_class = 7;
	grh_viadr_l0_obt2utc.instrument_group = 0;
	grh_viadr_l0_obt2utc.record_subclass = 0;
	grh_viadr_l0_obt2utc.record_subclass_version = 2;
	grh_viadr_l0_obt2utc.record_size = 38;

	grh_isp = make_matching_rec_head();
	grh_isp.record_class = 8;
	grh_isp.instrument_group = 0;
	grh_isp.record_subclass_version = 1;

	grh_dummy = make_matching_rec_head();
	grh_dummy.record_class = 8;
	grh_dummy.instrument_group = 13;
}

CCSDS* CCSDSReader::get_next_ccsds(void)
{
#ifndef PYMETOPIZER
	if (!spinner.get())
	{
		spinner.reset(new Spinner());
	}
	spinner->tick();
#endif

	// check if hold was set during a check operation
	if (hold)
	{
		CCSDS* ahold = hold;
		hold = NULL;
		return ahold;
	}

	if (!is)
	{
		DEBUG("CCSDSReader::get_next_ccsds getting next file\n");
		is = source->get_next_file();
		if (!is)
		{
			DEBUG("CCSDSReader::get_next_ccsds end of input" << " stream\n");
#ifndef PYMETOPIZER
			spinner->end();
#endif
			return NULL;
		}
	}

	if (prepacket_header > 0)
	{
		is->seekg(prepacket_header, ios::cur);
	}

	const int HEAD_SIZE = 20;
	byte header_buffer[HEAD_SIZE];
	is->read((char*)header_buffer, HEAD_SIZE);
	int count = is->gcount();
	DEBUG("CCSDSReader::get_next_ccsds read " << count
		  << " header bytes\n");
	byte_count += count;

	if (count != HEAD_SIZE)
	{
		is = NULL;
		return get_next_ccsds();
	}

	// does it look like an EPS record?

	REC_HEAD* grh = reinterpret_cast<REC_HEAD*>(header_buffer);
	DEBUG("RECORD_CLASS is " << (int)grh->record_class << endl);
	// if its an MPHR, IPR or viadr-l0-obtutc then skip it

	if (rec_head_equal(*grh, grh_mphr))
	{
		DEBUG("Skipping MPHR\n");
		is->seekg(20);
		ingest_mphr_values(is);
		string level = mphr_values["PROCESSING_LEVEL"];
		if (level!="00")
		{
#ifdef PYMETOPIZER
			DEBUG("This is not a L0 product\n");
#else
			cerr << "This is not a L0 product (PROCESSING_LEVEL field is not '00')\n";
#endif
			return NULL;
		}
		return get_next_ccsds();
	}
	else if (rec_head_equal(*grh, grh_ipr))
	{
		DEBUG("Skipping IPR\n");
		is->seekg(grh->record_size - HEAD_SIZE, ios::cur);
		return get_next_ccsds();
	}
	else if (rec_head_equal(*grh, grh_viadr_l0_obt2utc))
	{
		// if its a L0 VIADR read it
		DEBUG("Reading viadr_l0_obt2utc\n");
		const int VIADR_SIZE = 18;

		is->read(reinterpret_cast<char*>(&viadr_l0_obt2utc) + 20, VIADR_SIZE);

		// fill relevant VIADR struct
		viadr_l0_obt2utc.record_header.record_start_time = grh->record_start_time;
		viadr_l0_obt2utc.record_header.record_stop_time = grh->record_stop_time;
		viadr_l0_obt2utc_arr.push_back(viadr_l0_obt2utc);

		if (show_viadr)
		{
			cout << "\nVIADR L0 #" << viadr_l0_obt2utc_arr.size() << endl
				 << "	GRH.Start_Time					  = "
				 << viadr_l0_obt2utc.record_header.record_start_time << endl
				 << "	   .Stop_Time					  = "
				 << viadr_l0_obt2utc.record_header.record_stop_time	 << endl
				 << "	UTC_0.Day		 = "
				 << viadr_l0_obt2utc.utc_obt_day					 << endl
				 << "		 .MilliSec	 = "
				 << viadr_l0_obt2utc.utc_obt_millisec				 << endl
				 << "		 .MicroSec	 = "
				 << viadr_l0_obt2utc.utc_obt_microsec				 << endl
				 << "	CCU_OBT_0.Coarse = "
				 << (viadr_l0_obt2utc.ccu_obt_0>>8)					 << endl
				 << "			 .Fine	 = "
				 << ((viadr_l0_obt2utc.ccu_obt_0&0xff)<<8)			 << endl
				 << "	Clock_Step		 = "
				 << viadr_l0_obt2utc.clock_step						 << endl << endl;
		}

		if (viadr_l0_obt2utc_arr.size()>=2)
		{
			// give warning if this and previous VIADR overlap
			if (viadr_l0_obt2utc.record_header.record_start_time <
				viadr_l0_obt2utc_arr[viadr_l0_obt2utc_arr.size() - 2].record_header.record_stop_time)
			{
				cerr << "WARNING: VIADR #" << (viadr_l0_obt2utc_arr.size() - 1)
					 << " and #" << viadr_l0_obt2utc_arr.size()
					 << " overlap" << endl << endl;
			}
		}

		//DEBUG("Skipping viadr_l0_obt2utc\n");
		//is->seekg(grh->record_size-HEAD_SIZE, ios::cur);
		return get_next_ccsds();
	}
	else if (rec_head_equal(*grh, grh_dummy))
	{
		DEBUG("Skipping dummy MDR\n");
		is->seekg(grh->record_size - HEAD_SIZE, ios::cur);
		return get_next_ccsds();
	}
	else if (rec_head_equal(*grh, grh_isp))
	{
		// if its a L0 MDR read it
		grh_start = grh->record_start_time;
		grh_stop = grh->record_stop_time;

		if (!eps)
		{
#ifdef PYMETOPIZER
			DEBUG( "Extracting packets from EPS L0 product" << endl << endl );
#else
			cout << "Extracting packets from EPS L0 product\n\n";
#endif
			eps = true;
		}
		DEBUG("Reading an L0 MDR\n");
		const int MDR_HEAD = 20 + 6;
		byte* buffer = new byte[grh->record_size - MDR_HEAD];
		is->seekg(MDR_HEAD - HEAD_SIZE, ios::cur);
		DEBUG("Reading from L0 MDR size " << grh->record_size << endl);
		is->read((char*)buffer, grh->record_size - MDR_HEAD);
		byte_count += MDR_HEAD + is->gcount();
		const bool TAKE_OWNERSHIP = true;
		CCSDS* temp = new CCSDS(buffer, TAKE_OWNERSHIP);
		delete[] buffer;
		packet.reset(temp);
		packet_count++;
		return packet.get();
	}
//	  else
//	  {
//	error << "Found record " << (*grh) << " - this is not a L0 product\n";
//	return NULL;
//	  }

	// otherwise assume its an ISP

	// we just want to call get_length() on it.
	int length = CCSDS(header_buffer, false).get_length();

	if (length < HEAD_SIZE)
	{
		error << "Found ISP with length of " << length << ", the file "
			  << "cannot be read\n";
        return NULL;
	}

	DEBUG("CCSDSReader::get_next_packet read length as "
		  << length << endl);

	byte* buffer = new byte[length];
	memcpy(buffer, header_buffer, HEAD_SIZE);

	is->read((char*)buffer + HEAD_SIZE, length - HEAD_SIZE);
	count = is->gcount();
	DEBUG("CCSDSReader::get_next_ccsds read " << count << " bytes\n");
	byte_count += count;

	if (count == (length - HEAD_SIZE))
	{
		const bool TAKE_OWNERSHIP = true;
		CCSDS* temp = new CCSDS(buffer, TAKE_OWNERSHIP);
		delete[] buffer;
		packet.reset(temp);
		packet_count++;
		return packet.get();
	}
	else
	{
		delete[] buffer;
		error << "CCSDSReader::get_next_ccsds - Error in input "
			  << "stream (data)."
			  << "packet_count is " << packet_count
			  << " byte_count is " << byte_count << endl;
#ifndef PYMETOPIZER
		spinner->end();
#endif
		return NULL;
	}
}

// Validate given file in constructor is valid
CCSDSProducer::GROUPID CCSDSReader::get_data_type(void)
{
	if (hold == NULL)
	{
		hold = get_next_ccsds();
	}
	if (hold)
	{
		// Check given apid against known instrument groups
		APID apid = hold->get_apid();
		switch ( apid )
		{
		case APID_MHS:
			return CCSDSProducer::MHS;
		case APID_AMSUA_1:
		case APID_AMSUA_2: // intentional drop through
			return CCSDSProducer::AMSUA;
		case APID_AVHRR_3A:
		case APID_AVHRR_3B: // intentional drop through
			return CCSDSProducer::AVHRR;
		case APID_HIRS:
			return CCSDSProducer::HIRS;
		case APID_GOME:
			return CCSDSProducer::GOME;
		case APID_SEM:
			return CCSDSProducer::SEM;
		case APID_SAT:
		case APID_REDUCED:
		case APID_NAV:
		case APID_ADMIN:
			return CCSDSProducer::HKTM;
		default:
			if (apid >= APID_ASCAT_FIRST && apid <= APID_ASCAT_LAST)
			{
				return CCSDSProducer::ASCAT;
			}
			else if (apid >= APID_IASI_PX1 && apid <= APID_IASI_AP)
			{
				return CCSDSProducer::IASI;
			}
			else if (apid >= APID_GRAS_MEASUREMENT_1 && apid <= APID_GRAS_MEASUREMENT_2)
			{
				return CCSDSProducer::GRAS;
			}
			else
			{
				DEBUG("APID NOT supported yet: " << apid << endl);
				return CCSDSProducer::UNSUPPORTED;
			}
		}
	}
	else if (!source->is_valid())
	{
		return CCSDSProducer::FILE_ERROR;
	}
	else
	{
		return CCSDSProducer::UNRECOGNISED;
	}
}

// reset the CCSDS stream and return to first file
void CCSDSReader::reset_stream(void)
{
	packet.reset();
	hold = NULL;
	is = NULL;
	viadr_l0_obt2utc_arr.clear();

	source->reset_files();
}

void CCSDSReader::delete_source(void)
{
	delete source;
}

void CCSDSReader::output_text_summary( textoutstream_type& os )
{
	os << "CCSDSReader report:\n"
//		 << "	 " << byte_count << " bytes read\n"
	   << "	   " << packet_count << " packets extracted\n";
	if (prepacket_header > 0)
	{
		os << "	   " << prepacket_header << " bytes prepacket header\n";
	}
	os << endl;
}

bool CCSDSReader::produce_files(FileConsumer& dest)
{
	while( CCSDS* ccsds = get_next_ccsds() )
	{
		packet_count++;
		stringstream s;
		s << ccsds->get_apid() << "."
		  << ccsds->get_sequence_count() << ".ccsds";
		ostream_type* os = dest.get_named_output_stream(s.str());

		byte_count -= os->tellp();
		os->write((char*)ccsds->get_data(), ccsds->get_length());
		byte_count += os->tellp();
	}
	return true;
}

istream_type* CCSDSReader::get_next_file(void)
{
	return NULL;
}

REC_HEAD CCSDSReader::make_matching_rec_head(void)
{
	// return a REC_HEAD that matches anything according to
	// rec_head_equal()

	REC_HEAD result;
	result.record_class = 255;
	result.instrument_group = 255;
	result.record_subclass = 255;
	result.record_subclass_version = 255;
	result.record_size = numeric_limits<uint32_be>::max();
	result.record_start_time.day = 65535;
	result.record_start_time.millisec_day = numeric_limits<uint32_be>::max();
	result.record_stop_time.day = 65535;
	result.record_stop_time.millisec_day = numeric_limits<uint32_be>::max();
	return result;
}

bool CCSDSReader::rec_head_equal(const REC_HEAD& a, const REC_HEAD& b)
{
	if (a.record_class != 255 && b.record_class != 255 &&
		a.record_class != b.record_class)
	{
		return false;
	}

	if (a.instrument_group != 255 && b.instrument_group != 255 &&
		a.instrument_group != b.instrument_group)
	{
		return false;
	}

	if (a.record_subclass != 255 && b.record_subclass != 255 &&
		a.record_subclass != b.record_subclass)
	{
		return false;
	}

	if (a.record_subclass_version != 255 && b.record_subclass_version != 255 &&
		a.record_subclass_version != b.record_subclass_version)
	{
		return false;
	}

	if (a.record_size != numeric_limits<uint32_be>::max() &&
		b.record_class != numeric_limits<uint32_be>::max() &&
		a.record_class != b.record_class)
	{
		return false;
	}

	if (a.record_start_time.day != numeric_limits<uint16_be>::max() &&
		a.record_start_time.millisec_day != numeric_limits<uint32_be>::max() &&
		b.record_start_time.day != numeric_limits<uint16_be>::max() &&
		b.record_start_time.millisec_day != numeric_limits<uint32_be>::max() &&
		a.record_start_time != b.record_start_time)
	{
		return false;
	}

	if (a.record_stop_time.day != numeric_limits<uint16_be>::max() &&
		a.record_stop_time.millisec_day != numeric_limits<uint32_be>::max() &&
		b.record_stop_time.day != numeric_limits<uint16_be>::max() &&
		b.record_stop_time.millisec_day != numeric_limits<uint32_be>::max() &&
		a.record_stop_time != b.record_stop_time)
	{
		return false;
	}

	return true;
}

EPSProducer* CCSDSReader::as_eps_product(void)
{
	if (eps)
	{
		return this;
	}
	else
	{
		return NULL;
	}
}

short_cds_time CCSDSReader::get_grh_start(void) const
{
	return grh_start;
}

short_cds_time CCSDSReader::get_grh_stop(void) const
{
	return grh_stop;
}

const VIADR_OBT2UTC* CCSDSReader::get_viadr_l0_obt2utc(const short_cds_time& grh_start_time,
													   int *viadr_matching) const
{
	// return viadr_l0_obt2utc[] from the VIADR which is valid wrt grh_time
	for(vector<VIADR_OBT2UTC>::const_iterator iter=viadr_l0_obt2utc_arr.begin();
		iter!=viadr_l0_obt2utc_arr.end();
		iter++)
	{
		//cout << "Scanning VIADRs " << endl
		//	   << "	 VIADR# " << (iter - viadr_l0_obt2utc_arr.begin())
		//	   << " has Start Validity Time " << (*iter).record_header.record_start_time << endl
		//	   << "\t\t End Validity Time "	  << (*iter).record_header.record_stop_time	 << endl << endl;

		if (((grh_start_time+VIADR_MATCH_THRESHOLD) >= (*iter).record_header.record_start_time)
			&& ( grh_start_time <= (((*iter).record_header.record_stop_time)+VIADR_MATCH_THRESHOLD)))
		{
			*viadr_matching = (iter - viadr_l0_obt2utc_arr.begin());
			return &(*iter);
		}
	}
	return NULL;
}

string CCSDSReader::get_mphr_field(const string& field) const
{
	StringLookup::const_iterator iter = mphr_values.find(field);
	if (iter == mphr_values.end())
	{
#ifdef PYMETOPIZER
		PyException(PyExc_ValueError, "Field '" << field << "' not found");
#endif
		return "";
	}
	else
	{
		return (*iter).second;
	}
}

void CCSDSReader::ingest_mphr_values(istream_type* is)
{
	const int SIZE = 3307 - 20;
	char buffer[SIZE];
	is->read(buffer, SIZE);

	int pos = 0;
	while(pos < SIZE)
	{
		char* name_end = strchr(buffer+pos, ' ');
		*name_end = 0;
		string name = string(buffer + pos);
		char* value_start = buffer + pos + 32;
		while(*value_start == ' ')
		{
			value_start++;
		}
		char* value_end = strchr(buffer + pos + 32, '\n');
		*value_end = 0;
		string value = string(value_start);
		//cout << "name '" << name << "' value '" << value << "'\n";
		pos += 32 + strlen(buffer + pos + 32) + 1;
		mphr_values[name] = value;
	}
}

string CCSDSReader::get_scid(void)
{
	StringLookup::iterator mphr_iter = mphr_values.find("SPACECRAFT_ID");
	if (mphr_iter != mphr_values.end())
	{
		return (*mphr_iter).second;
	}
	else
	{
		return "";
	}
}
