////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsstatscalculator
//
//	Provides: CCSDSStatsCalculator filter
//
//	Purpose: Generate some statistical information on a packet stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsstatscalculator_defined
#define ccsdsstatscalculator_defined

#include "ccsds.hpp"
#include "hirsbase.hpp"

typedef unsigned int StatId;

const StatId MIN_STAT = 1;

const StatId STAT_AVHRR_CH1 = 1;
const StatId STAT_AVHRR_CH2 = 2;
const StatId STAT_AVHRR_CH3 = 3;
const StatId STAT_AVHRR_CH4 = 4;
const StatId STAT_AVHRR_CH5 = 5;

const StatId STAT_HIRS_BEGIN = 6;
const StatId STAT_HIRS_END = 25;

const StatId STAT_AMSUA1_MEASURE_BEGIN = 26;
const StatId STAT_AMSUA1_MEASURE_END = 38;

const StatId STAT_AMSUA1_UNIQUE_BEGIN = 39;
const StatId STAT_AMSUA1_UNIQUE_END = 51;

const StatId STAT_AMSUA2_MEASURE_BEGIN = 52;
const StatId STAT_AMSUA2_MEASURE_END = 53;

const StatId STAT_AMSUA2_UNIQUE_BEGIN = 54;
const StatId STAT_AMSUA2_UNIQUE_END = 55;

const StatId STAT_MHS_CH1 = 56;
const StatId STAT_MHS_CH2 = 57;
const StatId STAT_MHS_CH3 = 58;
const StatId STAT_MHS_CH4 = 59;
const StatId STAT_MHS_CH5 = 60;

const StatId MAX_STAT = 61;

class CCSDSStatsCalculator : public HIRSBase
{
public:

	enum PassType { FIRSTPASS, SECONDPASS };
	typedef NOAA_L1b_AVHRR_Data::Sample AVHRR_Sample;

	~CCSDSStatsCalculator(void)
		{
		}

	//
	// constructor
	//

	CCSDSStatsCalculator(void)
		: std_done(false)
		{
			stats.resize(MAX_STAT);

			set_stat_name(STAT_AVHRR_CH1, "AVHRR Channel 1");
			set_stat_name(STAT_AVHRR_CH2, "AVHRR Channel 2");
			set_stat_name(STAT_AVHRR_CH3, "AVHRR Channel 3");
			set_stat_name(STAT_AVHRR_CH4, "AVHRR Channel 4");
			set_stat_name(STAT_AVHRR_CH5, "AVHRR Channel 5");

			for(int i=STAT_HIRS_BEGIN; i<=STAT_HIRS_END; i++)
			{
				set_stat_name(i, string("HIRS Channel ") +
							  ntofs(i-STAT_HIRS_BEGIN+1, 2, '0') +
							  " (elements 1 to 56)");
			}

			for(int i=STAT_AMSUA2_MEASURE_BEGIN;
				i<=STAT_AMSUA2_MEASURE_END;
				i++)
			{
				set_stat_name(i, string("AMSUA2 (Measurement) Channel ") +
							  ntofs(i-STAT_AMSUA2_MEASURE_BEGIN+1, 2, '0'));
			}

			for(int i=STAT_AMSUA2_UNIQUE_BEGIN;
				i<=STAT_AMSUA2_UNIQUE_END;
				i++)
			{
				set_stat_name(i, string("AMSUA2 (Unique) Channel ") +
							  ntofs(i-STAT_AMSUA2_UNIQUE_BEGIN+1, 2, '0'));
			}

			for(int i=STAT_AMSUA1_MEASURE_BEGIN;
				i<=STAT_AMSUA1_MEASURE_END;
				i++)
			{
				set_stat_name(i, string("AMSUA1 (Measurement) Channel ") +
							  ntofs(i-STAT_AMSUA1_MEASURE_BEGIN+3, 2, '0'));
			}

			for(int i=STAT_AMSUA1_UNIQUE_BEGIN;
				i<=STAT_AMSUA1_UNIQUE_END;
				i++)
			{
				set_stat_name(i, string("AMSUA1 (Unique) Channel ") +
							  ntofs(i-STAT_AMSUA1_UNIQUE_BEGIN+3, 2, '0'));
			}

			set_stat_name(STAT_MHS_CH1, "MHS Channel 1");
			set_stat_name(STAT_MHS_CH2, "MHS Channel 2");
			set_stat_name(STAT_MHS_CH3, "MHS Channel 3");
			set_stat_name(STAT_MHS_CH4, "MHS Channel 4");
			set_stat_name(STAT_MHS_CH5, "MHS Channel 5");
		}

	//
	// first_pass
	//

	void first_pass(CCSDSProducer& source)
		{
			spinner.reset(new Spinner("First pass"));

			pass(source, FIRSTPASS);

			// calculate averages
//		for(StatList::iterator iter = stats.begin() + 1;
//		iter != stats.end();
//		iter++)
			for(int i=MIN_STAT; i<MAX_STAT; i++)
			{
				stats[i].avg_value = static_cast<double>(stats[i].acc_value)/
					static_cast<double>(stats[i].value_count);
			}

			spinner->end();
		}

	//
	// second_pass
	//

	void second_pass(CCSDSProducer& source)
		{
			spinner.reset(new Spinner("Second pass"));

			pass(source, SECONDPASS);

			// fix standard deviation
//		for(StatList::iterator iter = stats.begin() + 1;
//		iter != stats.end();
//		iter++)
			for(int i=MIN_STAT; i<MAX_STAT; i++)
			{
				stats[i].std_value = sqrt(stats[i].std_total /
										  stats[i].value_count);
			}

			std_done = true;

			spinner->end();
		}

	//
	// output_stats
	//

	void output_text_summary( ostream& os = cout )
		{

//		for(StatList::iterator iter = stats.begin()+1;
//		iter != stats.end();
//		iter++)
			for(int i=MIN_STAT; i<MAX_STAT; i++)
			{

				//		cout << "TEST value " << (i) << " namesize "
				// << stats[i].name.length() << endl;

				if (stats[i].value_count > 0)
				{
					os << stats[i].name
					   << " min " << stats[i].min_value
					   << " max " << stats[i].max_value
					   << " avg "
					   << dto2dp(stats[i].avg_value);

					if (std_done)
					{
						os << " std " << dto2dp(stats[i].std_value);
					}

					os << endl;
				}
			}
		}

private:

	//
	// pass
	//

	void pass(CCSDSProducer& source, PassType pt)
		{
			while(CCSDS* ccsds = source.get_next_ccsds())
			{
				if (spinner.get())
				{
					spinner->tick();
				}

				if (ccsds->get_apid() == APID_AVHRR_3A
					|| ccsds->get_apid() == APID_AVHRR_3B)
				{
					packet_avhrr(ccsds, pt);
				}
				else if (ccsds->get_apid() == CCSDS_HIRS::APID)
				{
					packet_hirs(ccsds, pt);
				}
				else if (ccsds->get_apid() == CCSDS_AMSUA1::APID
						 || ccsds->get_apid() == CCSDS_AMSUA2::APID)
				{
					packet_amsua(ccsds, pt);
				}
				else if (ccsds->get_apid() == CCSDS_MHS::APID)
				{
					packet_mhs(ccsds, pt);
				}
			}
		}

	//
	// packet_avhrr
	//

	void packet_avhrr(CCSDS* ccsds, PassType pt)
		{
			CCSDS_AVHRR* avhrr = (CCSDS_AVHRR*)ccsds->get_data();
			byte* payload_byte = avhrr->payload.data;
			int payload_bit = 7;

			// skip 11 samples
			for(int skip=0 ; skip<11; skip++)
			{
				read_avhrr_sample(payload_byte, payload_bit);
			}

			// ingest earth view
			for(int i=0; i<2048; i++)
			{
				AVHRR_Sample sample = read_avhrr_sample(
					payload_byte, payload_bit);

				ingest_value(STAT_AVHRR_CH1, sample.channels[0], pt);
				ingest_value(STAT_AVHRR_CH2, sample.channels[1], pt);
				ingest_value(STAT_AVHRR_CH3, sample.channels[2], pt);
				ingest_value(STAT_AVHRR_CH4, sample.channels[3], pt);
				ingest_value(STAT_AVHRR_CH5, sample.channels[4], pt);
			}
		}

	//
	// packet_hirs
	//

	void packet_hirs(CCSDS* ccsds, PassType pt)
		{
			CCSDS_HIRS* hirs = (CCSDS_HIRS*)ccsds->get_data();

			for(int i=0; i<56; i++)
			{
				HIRS_Element e = get_hirs_element(hirs, i);
				for(int j=0; j<20; j++)
				{
					ingest_value(STAT_HIRS_BEGIN+j, e.channels[j], pt);
				}
			}
		}

	//
	// packet_amsua
	//

	void packet_amsua(CCSDS* ccsds, PassType pt)
		{
//		CCSDS_AMSUA1* amsua1 = (CCSDS_AMSUA1*)ccsds->get_data();

			deque<byte> sci_data;
			extract_amsu_data(ccsds,sci_data);

			switch(sci_data.size())
			{
			case 1240:
			{
				AMSUA1MeasurementScienceData data;
				copy_n(sci_data.begin(), 1240, (byte*)(&data));
				packet_amsu_a1_measure(data, pt);
				break;
			}
			case 1120:
			{
				AMSUA1UniqueScienceData data;
				copy_n(sci_data.begin(), 1120, (byte*)(&data));
				packet_amsu_a1_unique(data, pt);
				break;
			}
			case 312:
			{
				AMSUA2MeasurementScienceData data;
				copy_n(sci_data.begin(), 312, (byte*)(&data));
				packet_amsu_a2_measure(data, pt);
				break;
			}
			case 288:
			{
				AMSUA2UniqueScienceData data;
				copy_n(sci_data.begin(), 288, (byte*)(&data));
				packet_amsu_a2_unique(data, pt);
				break;
			}
			}
		}

	//
	// packet_amsu_a1_measure
	//

	void packet_amsu_a1_measure(AMSUA1MeasurementScienceData& data, PassType pt)
		{
			for(int i=0; i<30; i++)
			{
				for(int j=0; j<=(STAT_AMSUA1_MEASURE_END-
								 STAT_AMSUA1_MEASURE_BEGIN); j++)
				{
					ingest_value(j+STAT_AMSUA1_MEASURE_BEGIN,
								 data.scene[i].counts[j], pt);
				}
			}
		}

	//
	// packet_amsu_a1_unique
	//

	void packet_amsu_a1_unique(AMSUA1UniqueScienceData& data, PassType pt)
		{
			for(int i=0; i<30; i++)
			{
				for(int j=0; j<=(STAT_AMSUA1_UNIQUE_END-
								 STAT_AMSUA1_UNIQUE_BEGIN); j++)
				{
					ingest_value(j+STAT_AMSUA1_UNIQUE_BEGIN,
								 data.scene[i].counts[j], pt);
				}
			}
		}

	//
	// packet_amsu_a2_measure
	//

	void packet_amsu_a2_measure(AMSUA2MeasurementScienceData& data, PassType pt)
		{
			for(int i=0; i<30; i++)
			{
				for(int j=0; j<=(STAT_AMSUA2_MEASURE_END-
								 STAT_AMSUA2_MEASURE_BEGIN); j++)
				{
					ingest_value(j+STAT_AMSUA2_MEASURE_BEGIN,
								 data.scene[i].counts[j], pt);
				}
			}
		}

	//
	// packet_amsu_a2_unique
	//

	void packet_amsu_a2_unique(AMSUA2UniqueScienceData& data, PassType pt)
		{
			for(int i=0; i<30; i++)
			{
				for(int j=0; j<=(STAT_AMSUA2_UNIQUE_END-
								 STAT_AMSUA2_UNIQUE_BEGIN); j++)
				{
					ingest_value(j+STAT_AMSUA2_UNIQUE_BEGIN,
								 data.scene[i].counts[j], pt);
				}
			}
		}

	void packet_mhs(CCSDS* ccsds, PassType pt)
		{
			MHSPayload& mhs = ((CCSDS_MHS*)ccsds->get_data())->payload;

			for(int i=0; i<90; i++)
			{
				ingest_value(STAT_MHS_CH1, mhs.earth_view[i].channels[0],pt);
				ingest_value(STAT_MHS_CH2, mhs.earth_view[i].channels[1],pt);
				ingest_value(STAT_MHS_CH3, mhs.earth_view[i].channels[2],pt);
				ingest_value(STAT_MHS_CH4, mhs.earth_view[i].channels[3],pt);
				ingest_value(STAT_MHS_CH5, mhs.earth_view[i].channels[4],pt);
			}
		}

	//
	// set_stat_name
	//

	void set_stat_name(StatId id, const string& name)
		{
//		cout << "SETSTATNAME " << id << " namesize " << name.length()
			// << " name '" << name << "'\n";
//		stats.resize(id+1);
			stats[id].name = name;
			stats[id].min_value = MAX_INT;
			stats[id].max_value = MIN_INT;
			stats[id].acc_value = 0;
			stats[id].value_count = 0;
			stats[id].avg_value = 0.0;
			stats[id].std_value = 0.0;
			stats[id].std_total = 0.0;
		}

	//
	// ingest_value
	//

	void ingest_value(StatId id, int new_value, PassType pt)
		{
			if (pt==FIRSTPASS)
			{
				if (new_value < stats[id].min_value)
				{
					stats[id].min_value = new_value;
				}
				if (new_value > stats[id].max_value)
				{
					stats[id].max_value = new_value;
				}
				stats[id].acc_value += new_value;
				stats[id].value_count++;
			}
			else
			{
				stats[id].std_total += pow(static_cast<double>(new_value)-
										   stats[id].avg_value, 2.0);
			}
		}

	//
	// read_avhrr_bit
	//

	int read_avhrr_bit(byte*& payload_byte, int& payload_bit)
		{
			int result;
			if ( ((*payload_byte)&(1<<payload_bit)) > 0)
			{
				result = 1;
			}
			else
			{
				result = 0;
			}

			if (payload_bit > 0)
			{
				payload_bit--;
			}
			else
			{
				payload_bit = 7;
				payload_byte++;
			}
			return result;
		}

	//
	// read_avhrr_sample
	//

	AVHRR_Sample read_avhrr_sample(byte*& payload_byte, int& payload_bit)
		{
			AVHRR_Sample sample;
			memset(&sample, 0, sizeof(sample));

			for(int channel = 0; channel<5; channel++)
			{
				for(int bit = 0; bit<10; bit++)
				{
					sample.channels[channel] <<= 1;
					sample.channels[channel] +=
						read_avhrr_bit(payload_byte, payload_bit);
				}
			}

			return sample;
		}

	//
	// extract_amsu_data
	//

	void extract_amsu_data(CCSDS* packet, deque<byte>& sci_data)
		{
			for(int i=0; i<packet->get_payload_length(); i+=2)
			{
				byte a = packet->get_payload()[i];
				byte b = packet->get_payload()[i+1];

				if (a!=0 || b!=1)
				{
					// not filler
					sci_data.push_back(a);
					sci_data.push_back(b);
				}
			}
		}

private:

	struct Stat
	{
		string name;
		int min_value;
		int max_value;
		double acc_value;
		int value_count;
		double avg_value;
		double std_total;
		double std_value;
	};

	bool std_done;

	typedef vector<Stat> StatList;
	StatList stats;
};

#endif
