////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdsstreamdisplayer.cpp
//
//	Provides: CCSDSStreamDisplayer filter
//
//	Purpose: Provide some statistical information on a stream of CCSDS packets
//
//	Copyright (c) 2005 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsdsstreamdisplayer.hpp"
#include "ccsdsapidfilter.hpp"

const int SEQ_COUNT_OVERFLOW = 1<<14;
const int MAX_GAP_WITHIN_SEQUENCE =	 60000; // ms

CCSDSStreamDisplayer::CCSDSStreamDisplayer(
	CCSDSProducer& _source,
	bool _shift_gras_obt)
	: source(_source),
	  total_packets(0),
	  last_packet(NULL),
	  shift_gras_obt(_shift_gras_obt)
{
	DEBUG("ccsdsstreamdisplayer constructor\n");

	CCSDS* packet;
	while( (packet=source.get_next_ccsds()) != NULL)
	{
//		DEBUG("got a packet\n");
		last_packet = packet;

		total_packets++;

		// check if its a new APID
		string apid = CCSDSAPIDFilter::apid_to_string(
			packet->get_apid());

		int count = packet->get_sequence_count();
		UTCTime utc_time;
		utc_time.days = 0;
		utc_time.milliseconds = 0;
		utc_time.microseconds = 0;
		if (packet->has_secondary_header())
		{
			utc_time = *(packet->get_secondary_header());
		}

		OBTTime obt_time = *packet->get_ancillary_data();

		short_cds_time grh_time;
		if (EPSProducer* eps = source.as_eps_product())
		{
			grh_time = eps->get_grh_start();
		}

		if (channels.find(apid) == channels.end())
		{
			// create a fresh channel for this APID
			Channel& channel = channels[apid];

			channel.total_packets = 1;

			Channel::Sequence new_sequence;
			new_sequence.initial_seq_count = count;
			new_sequence.no_of_packets = 0; // this is correct
			new_sequence.start_utc = utc_time;
			new_sequence.end_utc = utc_time;
			new_sequence.has_utc_time = packet->has_secondary_header();
			new_sequence.is_shifted_gras = shift_gras_obt && (packet->get_apid()>=448 && packet->get_apid()<=511);
			new_sequence.rollovers = 0;
			new_sequence.start_obt = obt_time;
			new_sequence.end_obt = obt_time;
			new_sequence.start_grh = grh_time;
			new_sequence.end_grh = grh_time;
			channel.counts.push_back(new_sequence);

			channel.apids.insert(packet->get_apid());
		}
		else
		{
			Channel& channel = channels[apid];
			channel.total_packets++;

			// (ascat specific)
			if (channel.apids.find(packet->get_apid())==channel.apids.end())
			{
				channel.apids.insert(packet->get_apid());
			}

			Channel::Counts::iterator iter=channel.counts.begin();
			for(;iter!=channel.counts.end();
				iter++)
			{
				// see if the new packet fits onto the end of an existing
				// sequence

				int expecting = (
					((*iter).initial_seq_count+
					 (*iter).no_of_packets)%SEQ_COUNT_OVERFLOW)+1;

				if (expecting==count ||
					(expecting==SEQ_COUNT_OVERFLOW && count==0))
				{
					// add entry to the UTC Gaps table
					long long utc_diff = utc_time - (*iter).end_utc;
					Gaps::iterator utc_gap = (*iter).utc_gaps.find(utc_diff);
					if (utc_gap==(*iter).utc_gaps.end())
					{
						(*iter).utc_gaps[utc_diff] = 1;
					}
					else
					{
						(*iter).utc_gaps[utc_diff]++;
					}

					// add entry to the OBT gaps table
					long long obt_diff;
					if (shift_gras_obt && packet->get_apid()>=448 && packet->get_apid()<=511)
					{
						obt_diff = obt_time.subtract_shifted((*iter).end_obt);
					}
					else
					{
						obt_diff = obt_time - (*iter).end_obt;
					}
					// try to catch OBT rollovers
					if (obt_diff < -OBT_DIFF_ROLLOVER_THRESHOLD)
					{
						obt_diff += static_cast<long long>(MAX_OBT_COARSE)*static_cast<long long>(MAX_OBT_FINE);
					}
					Gaps::iterator obt_gap = (*iter).obt_gaps.find(obt_diff);
					if (obt_gap==(*iter).obt_gaps.end())
					{
						(*iter).obt_gaps[obt_diff] = 1;
					}
					else
					{
						(*iter).obt_gaps[obt_diff]++;
					}

					if (source.as_eps_product())
					{
						long long grh_diff = grh_time - (*iter).end_grh;
						Gaps::iterator grh_gap = (*iter).grh_gaps.find(grh_diff);
						if (grh_gap==(*iter).grh_gaps.end())
						{
							(*iter).grh_gaps[grh_diff] = 1;
						}
						else
						{
							(*iter).grh_gaps[grh_diff]++;
						}
					}

					if (expecting==SEQ_COUNT_OVERFLOW && count==0)
					{
						(*iter).rollovers++;
					}

					(*iter).no_of_packets++;
					(*iter).end_utc = utc_time;
					(*iter).end_obt = obt_time;
					(*iter).end_grh = grh_time;
					break;
				}
			}
			if (iter == channel.counts.end())
			{
				Channel::Sequence new_sequence;
				new_sequence.initial_seq_count = count;
				new_sequence.no_of_packets = 0;
				new_sequence.start_utc = utc_time;
				new_sequence.end_utc = utc_time;
				new_sequence.rollovers = 0;
				new_sequence.start_obt = obt_time;
				new_sequence.end_obt = obt_time;
				new_sequence.start_grh = grh_time;
				new_sequence.end_grh = grh_time;
				new_sequence.is_shifted_gras =	shift_gras_obt && (packet->get_apid()>=448 && packet->get_apid()<=511);
				channel.counts.push_back(new_sequence);

				channel.apids.insert(packet->get_apid());

			}

			// see if we are continuing an existing sequence
		}
	}
}

//
// get_total_packets
//

int CCSDSStreamDisplayer::get_total_packets(void) const
{
	return total_packets;
}

//
// output_text_record
//

void CCSDSStreamDisplayer::output_text_summary(ostream& os)
{
	spinner->end();

	os << "Analysis of CCSDS packet stream (" << total_packets
	   << " packet";
	if (total_packets != 1)
	{
		os << "s";
	}
	os << " total)\n";

	if (total_packets == 0)
	{
		os << "	   <no inputs>\n";
	}
	else
	{
		for(Channels::iterator iter=channels.begin();
			iter!=channels.end();
			iter++)
		{
			Channel& channel = (*iter).second;

			os << "	   " << channel.total_packets
			   << " packets with APID" << ((channel.apids.size()==1)?" ":"s ");
			bool first = true;
			for(Channel::APIDS::iterator iter2=
					channel.apids.begin();
				iter2!=channel.apids.end();
				iter2++)
			{
				if (first)
				{
					first = false;
				}
				else
				{
					os << ",";
				}
				os << (*iter2);
			}
			os << " (" << ((*iter).first) << ")\n";
			for(Channel::Counts::iterator iter2 =
					channel.counts.begin();
				iter2 != channel.counts.end();
				iter2++)
			{
				int start = (*iter2).initial_seq_count;
				int count = (*iter2).no_of_packets;
				UTCTime start_utc =
					(*iter2).start_utc;

				if (count>0)
				{
					UTCTime end_utc =
						(*iter2).end_utc;

					os << "		   Sequence counts " << start << "-"
					   << ((start+count)%SEQ_COUNT_OVERFLOW);
					if ((*iter2).rollovers > 0)
					{
						os << " with " << (*iter2).rollovers
						   << " rollover";
						if ((*iter2).rollovers != 1)
						{
							os << "s";
						}
					}
					os << " (" << (count+1) << ")\n";

					if ((*iter2).has_utc_time)
					{
						os << "			   UTC initial time: " << start_utc
						   << endl
						   << "					 final time: " << end_utc
						   << endl
						   << "						   gaps: ";
						for(Gaps::iterator gaps=(*iter2).utc_gaps.begin();
							gaps!=(*iter2).utc_gaps.end();
							gaps++)
						{
							os << (*gaps).first << "ms (" << (*gaps).second << ") ";
						}
						os << endl;
					}
					else
					{
						os << "			   <no UTC time present>\n";
					}

					if ((*iter2).is_shifted_gras)
					{
						os << "			   OBT initial time: " << (*iter2).start_obt.shifted_to_string()
						   << endl
						   << "					 final time: " << (*iter2).end_obt.shifted_to_string()
						   << endl
						   << "				 gaps in obt-16: ";
						for(Gaps::iterator gaps=(*iter2).obt_gaps.begin();
							gaps!=(*iter2).obt_gaps.end();
							gaps++)
						{
							os << (((*gaps).first)>>24) << "/"
							   << ((((*gaps).first)>>8)&0xffff) << "/"
							   << (((*gaps).first)&0xff)
							   << " (" << (*gaps).second << ") ";
						}
					}
					else
					{
						os << "			   OBT initial time: " << (*iter2).start_obt
						   << endl
						   << "					 final time: " << (*iter2).end_obt
						   << endl
						   << "				 gaps in obt-16: ";
						for(Gaps::iterator gaps=(*iter2).obt_gaps.begin();
							gaps!=(*iter2).obt_gaps.end();
							gaps++)
						{
							os << ((*gaps).first>>16) << "/" << (((*gaps).first)&0xffff) << " (" << (*gaps).second << ") ";
						}
					}
					os << endl;

					if (source.as_eps_product())
					{
						os << "			   GRH initial time " << (*iter2).start_grh
						   << endl
						   << "				 GRH gaps: ";
						for(Gaps::iterator gaps=(*iter2).grh_gaps.begin();
							gaps!=(*iter2).grh_gaps.end();
							gaps++)
						{
							os << (*gaps).first << "ms (" << (*gaps).second << ") ";
						}
						os << endl;
					}
				}
				else
				{
					os << "		   Sequence count " << start << endl
					   << "			   Time " << start_utc << endl;
				}
			}
			os << endl;
		}
		os << endl;
	}
}

