////////////////////////////////////////////////////////////////////////////////
//
// File: ccsdsstripfilter.hpp
//
// Provides: CCSDSStripFilter filter
//
// Purpose: Removes CCSDS packets that do not satisfy the UTC start and end
//			times specified in the ccsds_strip_filter.config file.
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdsstripfilter_defined
#define ccsdsstripfilter_defined

#include "interface.hpp"
#include "ccsds.hpp"
#include "ccsdsproducer.hpp"
#include "config.hpp"
#include "textconfigproducer.hpp"
#include "ccsdsapidfilter.hpp"

#define Config Config::CCSDSStripFilter

class CCSDSStripFilter :
	public CCSDSProducer,
	public FileProducer,
	public TextConfigProducer
{
public:

	//
	// constructor
	//

	CCSDSStripFilter(CCSDSProducer& _source,
					 string _config_file)
		: source(&_source),
		  config_file(_config_file),
		  apid(0),
		  apid_old(0),
		  packet_count(0),
		  pass_count(0),
		  block_count_range(0),
		  block_count_overlap(0),
		  sequence_counter(0),
		  days(0),
		  milliseconds(0),
		  microseconds(0),
		  apid_ctr(0) ,
		  time_gap(0.0),
		  micro_sec(0.0),
		  micro_sec_old(0.0),
		  micro_sec_start(0.0),
		  micro_sec_end(0.0)
		{
			if (!config_file.empty())
			{
				Config::file.read(config_file);
			}

			unsigned int MAX_DAY = 65535;
			unsigned int MAX_MILI_SEC = 86399999;
			unsigned int MAX_MICRO_SEC = 999;

			sequence_count_first = 0;
			utc_days_first = 0;
			utc_milliseconds_first = 0;
			utc_microseconds_first = 0;
			obt_coarse_first = 0;
			obt_fine_first = 0;

			sequence_count_last = 0;
			utc_days_last = 0;
			utc_milliseconds_last = 0;
			utc_microseconds_last = 0;
			obt_coarse_last = 0;
			obt_fine_last = 0;


			if ((Config::utc_days_start < 0) || (Config::utc_days_start > MAX_DAY))
			{
				error << "\nThe value of the configurable field utc_days_start is out of range.\n";
			}

			if((Config::utc_milliseconds_start < 0) || (Config::utc_milliseconds_start > MAX_MILI_SEC))
			{
				error << "\nThe value of the configurable field utc_milliseconds_start is out of range.\n";
			}

			if ((Config::utc_microseconds_start < 0) || (Config::utc_microseconds_start > MAX_MICRO_SEC))
			{
				error << "\nThe value of the configurable field utc_microseconds_start is out of range.\n";
			}

			if ((Config::utc_days_end < 0) || (Config::utc_days_end > MAX_DAY))
			{
				error << "\nThe value of the configurable field utc_days_end is out of range.\n";
			}

			if((Config::utc_milliseconds_end < 0) || (Config::utc_milliseconds_end > MAX_MILI_SEC))
			{
				error << "\nThe value of the configurable field utc_milliseconds_end is out of range.\n";
			}

			if ((Config::utc_microseconds_end < 0) || (Config::utc_microseconds_end > MAX_MICRO_SEC))
			{
				error << "\nThe value of the configurable field utc_microseconds_end is out of range.\n";
			}


			if (Config::time_gap_limit < 0)
			{
				error << "\nThe value of the configurable field time_gap_limit is out of range.\n";
			}

			time_gap = Config::time_gap_limit;

			micro_sec_start = (	  (Config::utc_days_start*24*60*60*1e6)
								  + (Config::utc_milliseconds_start*1e3)
								  + (Config::utc_microseconds_start) );

			micro_sec_end = (	(Config::utc_days_end*24*60*60*1e6)
								+ (Config::utc_milliseconds_end*1e3)
								+ (Config::utc_microseconds_end) );

		}


	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout )
		{
			os << "CCSDSStripFilter report:\n"
			   << "	   " << packet_count << " total number of packets read\n"
			   << "	   " << pass_count << " packets passed on\n"
			   << "	   " << block_count_range << " out of range packets blocked\n"
			   << "	   " << block_count_overlap << " overlapped packets blocked\n"
			   << "	   " << "First Packet\n"
			   << "		   " << "Sequence Count		  : " << sequence_count_first << endl
			   << "		   " << "Time				  : " << TimeConv::utc_days_milliseconds_to_string(utc_days_first,
																									   utc_milliseconds_first)
			   << "." << ntofs(utc_milliseconds_first%1000, 3, '0')
			   << ntofs(utc_microseconds_first, 3, '0')	 << endl
			   << "		   " << "UTC-Days			  : " << utc_days_first	 << endl
			   << "		   " << "UTC-Milliseconds	  : " << utc_milliseconds_first	 << endl
			   << "		   " << "UTC-Microseconds	  : " << utc_microseconds_first	 << endl
			   << "		   " << "OBT-Coarse (sec)	  : " << obt_coarse_first  << endl
			   << "		   " << "OBT-Fine (sec/65536) : " << obt_fine_first	 << endl
			   << "	   " << "Last Packet\n"
			   << "		   " << "Sequence Count		  : " << sequence_count_last  << endl
			   << "		   " << "Time				  : " << TimeConv::utc_days_milliseconds_to_string(utc_days_last,
																									   utc_milliseconds_last)
			   << "." << ntofs(utc_milliseconds_last%1000, 3, '0')
			   << ntofs(utc_microseconds_last, 3, '0') << endl
			   << "		   " << "UTC-Days			  : " << utc_days_last	<< endl
			   << "		   " << "UTC-Milliseconds	  : " << utc_milliseconds_last	<< endl
			   << "		   " << "UTC-Microseconds	  : " << utc_microseconds_last	<< endl
			   << "		   " << "OBT-Coarse (sec)	  : " << obt_coarse_last  << endl
			   << "		   " << "OBT-Fine (sec/65536) : " << obt_fine_last	<< endl << endl;
		}



	//
	// get_next_ccsds
	//

	virtual CCSDS* get_next_ccsds(void)
		{
			CCSDS* result = source->get_next_ccsds();

			if (!result)
			{
				return NULL;
			}

			if (packet_count == 0)
			{
				apid = result->get_apid();
				apid_old = apid;
				sequence_counter = result->get_sequence_count();
				days = result->get_utc_days();
				milliseconds = result->get_utc_milliseconds();
				microseconds = result->get_utc_microseconds();
			}
			else
			{

				int flag = strncmp( (CCSDSAPIDFilter::apid_to_string(apid)).c_str(),
									(CCSDSAPIDFilter::apid_to_string(result->get_apid())).c_str(),
									3 );

				if (result->get_apid() == 2 || result->get_apid() == 3)
				{
					if ( strncmp((CCSDSAPIDFilter::apid_to_string(apid)).c_str(),"GRAS",4) == 0
						 || strncmp((CCSDSAPIDFilter::apid_to_string(apid)).c_str(),"GOME",4) == 0 )
					{
						flag = 0;
					}
				}

				if (apid == 2 || apid == 3)
				{
					if ( strncmp((CCSDSAPIDFilter::apid_to_string(result->get_apid())).c_str(),"GRAS",4) == 0
						 || strncmp((CCSDSAPIDFilter::apid_to_string(result->get_apid())).c_str(),"GOME",4) == 0 )
					{
						flag = 0;
					}
				}

				if (flag != 0) //if (apid != result->get_apid())
				{
					error << "Error: APID mismatch ("
						  << "Intrument APID Expected:" << (CCSDSAPIDFilter::apid_to_string(apid)).c_str()
						  << " Intrument APID Found:" << (CCSDSAPIDFilter::apid_to_string(result->get_apid())).c_str()
						  << " Packet Count:" << packet_count+1 << ")\n";
					return NULL;
				}

				sequence_counter = result->get_sequence_count();
				days = result->get_utc_days();
				milliseconds = result->get_utc_milliseconds();
				microseconds = result->get_utc_microseconds();
			}


			int current_apid_pos = 0;
			if (packet_count == 0)
			{
				array_tmp[0][0] = apid;
				array_tmp[0][1] = 0;
			}
			else
			{
				APID apid_tmp = result->get_apid();
				int total_apids = apid_ctr;
				bool new_apid_flag = true;
				for (int p=0; p<=total_apids; p++)
				{
					if (array_tmp[p][0] == apid_tmp)
					{
						current_apid_pos = p;
						//array_tmp[p][1] = micro_sec;
						new_apid_flag = false;
					}
				}
				if (new_apid_flag)
				{
					apid_ctr++;
					current_apid_pos = apid_ctr;
					array_tmp[apid_ctr][0] = result->get_apid();
					array_tmp[apid_ctr][1] = 0;
				}

			}

			packet_count++;
			micro_sec_old = micro_sec;
			micro_sec = (days*24*60*60*1e6) + (milliseconds*1e3) + (microseconds);

			if( (micro_sec < micro_sec_start) || (micro_sec > micro_sec_end) )
			{
				// out of range
				block_count_range++;
				if (micro_sec_old != 0)
				{
					micro_sec = micro_sec_old;
				}

//		if (micro_sec > micro_sec_end)
//		{
//			return NULL;
//		}

//		cout << "end micro_sec " << micro_sec << " micro_sec_end " << micro_sec_end << endl;

				if (micro_sec > micro_sec_end)
				{
					cout << "end\n";
					return NULL;
				}

				return get_next_ccsds();
			}
			else
			{

				if (result->get_apid() == 160)
				{
					if (array_tmp[current_apid_pos][1] == micro_sec)
					{
						array_tmp[current_apid_pos][1] = micro_sec - 1;
					}
				}

				if (micro_sec > array_tmp[current_apid_pos][1])
				{
					if ( ( (micro_sec - micro_sec_old) < (time_gap*1e6) ) || (packet_count == 1) )
					{
						// looks good

						if (sequence_count_first == 0)
						{
							sequence_count_first = sequence_counter;
							utc_days_first = days;
							utc_milliseconds_first = milliseconds;
							utc_microseconds_first = microseconds;
							obt_coarse_first = result->get_obt_coarse();
							obt_fine_first = result->get_obt_fine();
						}
						else
						{
							sequence_count_last = sequence_counter;
							utc_days_last = days;
							utc_milliseconds_last = milliseconds;
							utc_microseconds_last = microseconds;
							obt_coarse_last = result->get_obt_coarse();
							obt_fine_last = result->get_obt_fine();
						}

						array_tmp[current_apid_pos][1] = micro_sec;
						pass_count++;
						return result;
					}
					else
					{
						error << "Error: Time difference between packet " << packet_count-1 << " and packet "
							  << packet_count << " exceed the acceptable time gap limit of " << time_gap
							  << " seconds."<< endl;
						return NULL;
					}
				}
				else
				{
					// overlap
					block_count_overlap++;
					micro_sec = micro_sec_old;
					return get_next_ccsds();
				}
			}
		}




	//
	// produce_files
	//

	virtual bool produce_files(FileConsumer& dest)
		{
			CCSDS* ccsds;
			while((ccsds=get_next_ccsds()))
			{

				stringstream s;
				s << "." << ccsds->get_apid() << ".ccsds";
				ostream_type* os = dest.get_named_output_stream(s.str());
				os->write((char*)ccsds->get_data(),ccsds->get_length());
			}
			return true;
		}



	//
	// get_next_file
	//

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}



	//
	// output_text_config
	//

	virtual void output_text_config( textoutstream_type& os )
		{
			os << "\n\nCCSDS_Strip_Filter Configurable Fields\n--------------------------------------\n\n"
			   << "utc_days_start		  =	 " << Config::utc_days_start << endl
			   << "utc_milliseconds_start =	 " << Config::utc_milliseconds_start << endl
			   << "utc_microseconds_start =	 " << Config::utc_microseconds_start << endl
			   << "utc_days_end			  =	 " << Config::utc_days_end << endl
			   << "utc_milliseconds_end	  =	 " << Config::utc_milliseconds_end << endl
			   << "utc_microseconds_end	  =	 " << Config::utc_microseconds_end << endl
			   << "time_gap_limit		  =	 " << Config::time_gap_limit << endl << endl;
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:

	CCSDSProducer* source;
	string config_file;

	APID apid;
	APID apid_old;

	int packet_count;
	int pass_count;
	int block_count_range;
	int block_count_overlap;

	unsigned int sequence_counter;
	unsigned int days;
	unsigned int milliseconds;
	unsigned int microseconds;

	int apid_ctr;
	double time_gap;
	double micro_sec;
	double micro_sec_old;
	double micro_sec_start;
	double micro_sec_end;

	unsigned int sequence_count_first;
	unsigned int utc_days_first;
	unsigned int utc_milliseconds_first;
	unsigned int utc_microseconds_first;
	unsigned int obt_coarse_first;
	unsigned int obt_fine_first;

	unsigned int sequence_count_last;
	unsigned int utc_days_last;
	unsigned int utc_milliseconds_last;
	unsigned int utc_microseconds_last;
	unsigned int obt_coarse_last;
	unsigned int obt_fine_last;

	double array_tmp[100][2];
};

#undef Config

#endif
