////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdstimeandseqfix.hpp
//
//	Provides: CCSDSTimeAndSeqFix filter
//
//	Purpose: Modify/offset the sequence counter in the CCSDS primary header
//		 and the OBT time filed of the CCSDS ancillary data packet.
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdstimeandseqfix_defined
#define ccsdstimeandseqfix_defined

#include "interface.hpp"
#include "ccsds.hpp"
#include "ccsdsproducer.hpp"
#include "config.hpp"
#include "textconfigproducer.hpp"
#include "ccsdsapidfilter.hpp"

#define Config Config::CCSDSTimeAndSeqFix

class CCSDSTimeAndSeqFix :
	virtual public Interface,
	public CCSDSProducer,
	public FileProducer,
	public TextConfigProducer
{
public:


	//
	// constructor
	//

	CCSDSTimeAndSeqFix( CCSDSProducer& _source, string _config_file, bool _accumulate, bool _single)
		: source(&_source)
		, config_file(_config_file)
		, packet_count(0)
		, valid(true)
		, accumulate(_accumulate)
		, single(_single)
		{
			if (!config_file.empty())
			{
				Config::file.read(config_file);
			}
			apid_first = 0;
			sequence_count_first = 0;
			utc_days_first = 0;
			utc_milliseconds_first = 0;
			utc_microseconds_first = 0;
			obt_coarse_first = 0;
			obt_fine_first = 0;

			apid_last = 0;
			sequence_count_last = 0;
			utc_days_last = 0;
			utc_milliseconds_last = 0;
			utc_microseconds_last = 0;
			obt_coarse_last = 0;
			obt_fine_last = 0;


			counter = Config::counter_initial;
			if (counter < 0)
			{
				error << "\nERROR: The value of the configurable field COUNTER_INITIAL must not be less than 0.\n";
				valid = false;
				//return NULL;
			}
			if (counter > 16383)
			{
				error << "\nERROR: The value of the configurable field COUNTER_INITIAL must not exceed 16383 (2^14 - 1).\n";
				valid = false;
				//return NULL;
			}

			obt_coarse = Config::obt_coarse_initial;
			if (obt_coarse < 0)
			{
				error << "\nERROR: The value of the configurable field OBT_COARSE_INITIAL must not be less than 0.\n";
				valid = false;
				//return NULL;
			}
			if (obt_coarse > 16777215)
			{
				error << "\nERROR: The value of the configurable field OBT_COARSE_INITIAL must not exceed 16777215 (2^24 - 1).\n";
				valid = false;
				//return NULL;
			}

			obt_fine = Config::obt_fine_initial;
			if (obt_fine < 0)
			{
				error << "\nERROR: The value of the configurable field OBT_FINE_INITIAL must not be less than 0.\n";
				valid = false;
				//return NULL;
			}
			if (obt_fine > 65535)
			{
				error << "\nERROR: The value of the configurable field OBT_FINE_INITIAL must not exceed 65535 (2^16 - 1).\n";
				valid = false;
				//return NULL;
			}

		}


	//
	// get_next_ccsds
	//

	virtual CCSDS* get_next_ccsds(void)
		{
			CCSDS* packet = source->get_next_ccsds();

			if (!packet)
			{
				return NULL;
			}

			if (packet_count == 0)
			{
				apid = packet->get_apid();
				sequence_counter = packet->get_sequence_count();
				days = packet->get_utc_days();
				milliseconds = packet->get_utc_milliseconds();
				microseconds = packet->get_utc_microseconds();

				apid_first = apid;
				sequence_count_first = counter;
				utc_days_first = days;
				utc_milliseconds_first = milliseconds;
				utc_microseconds_first = microseconds;
				obt_coarse_first = obt_coarse;
				obt_fine_first = obt_fine;

				sc_ctr = 0;
				sc_array[sc_ctr][0] = apid;
				sc_array[sc_ctr][1] = sequence_counter;
				sc_array[sc_ctr][2] = counter;
				sc_array[sc_ctr][3] = counter;
				sc_array[sc_ctr][4] = 1;
				sc_array[99][1] = sequence_counter;
				sc_array[99][3] = counter;
			}
			else
			{
				int flag = strncmp( (CCSDSAPIDFilter::apid_to_string(apid)).c_str(),
									(CCSDSAPIDFilter::apid_to_string(packet->get_apid())).c_str(),
									3 );

				if (packet->get_apid() == 2 || packet->get_apid() == 3)
				{
					if ( strncmp((CCSDSAPIDFilter::apid_to_string(apid)).c_str(),"GRAS",4) == 0
						 || strncmp((CCSDSAPIDFilter::apid_to_string(apid)).c_str(),"GOME",4) == 0 )
					{
						flag = 0;
					}
				}

				if (apid == 2 || apid == 3)
				{
					if ( strncmp((CCSDSAPIDFilter::apid_to_string(packet->get_apid())).c_str(),"GRAS",4) == 0
						 || strncmp((CCSDSAPIDFilter::apid_to_string(packet->get_apid())).c_str(),"GOME",4) == 0 )
					{
						flag = 0;
					}
				}

				if (flag != 0) //if (apid != result->get_apid())
				{
					error << "Error: APID mismatch ("
						  << "Intrument APID Expected:" << (CCSDSAPIDFilter::apid_to_string(apid)).c_str()
						  << " Intrument APID Found:" << (CCSDSAPIDFilter::apid_to_string(packet->get_apid())).c_str()
						  << " Packet Count:" << packet_count+1 << ")\n";
					valid = false;
					return NULL;
				}
				else
				{
					int apid_pass = -1;
					APID apid_tmp = packet->get_apid();
					for (int p=0; p<=sc_ctr; p++)
					{
						if (sc_array[p][0] == apid_tmp)
						{
//			if ( sc_array[p][1] > packet->get_sequence_count()
//				 &&	 packet->get_sequence_count() != 0 )
//			{
//				error << "\nWARNING: Sequence Counter is found to decrease in value at packet_count = "
//				  << packet_count+1 << endl;
//				//valid = false;
//				//return NULL;
//				diff_sequence_count = 1;
//			}
//			else
//			{
							apid_pass = p;
							if ( strncmp((CCSDSAPIDFilter::apid_to_string(packet->get_apid())).c_str(),"ASCAT",5) == 0
								 || accumulate )
							{
								diff_sequence_count = packet->get_sequence_count() - sc_array[99][1];
								if (diff_sequence_count == -16383)
								{
									diff_sequence_count = 1;
								}
								if (single || diff_sequence_count==0)
								{
									diff_sequence_count = 1;
								}
								if (diff_sequence_count < 0)
								{
									error << "\nWARNING: Sequence Counter is found to decrease in value at packet_count = "
										  << packet_count+1 << ": Sequence Counter incremented by 1." << endl;
									//valid = false;
									//return NULL;
									diff_sequence_count = 1;
								}
								sc_array[p][1] = packet->get_sequence_count();
								sc_array[99][1] = sc_array[p][1];
								sc_array[p][3] = sc_array[99][3] + diff_sequence_count;
								if (sc_array[p][3] > 16383)
								{
									sc_array[p][3] = sc_array[p][3] - 16384;
								}

								sc_array[99][3] = sc_array[p][3];
								sc_array[p][4] = sc_array[p][4] + 1;
								counter = sc_array[p][3];
							}
							else
							{
								diff_sequence_count = packet->get_sequence_count() - sc_array[p][1];
								if (diff_sequence_count == -16383)
								{
									diff_sequence_count = 1;
								}
								if (single || diff_sequence_count==0)
								{
									diff_sequence_count = 1;
								}
								if (diff_sequence_count < 0)
								{
									error << "\nWARNING: Sequence Counter is found to decrease in value at packet_count = "
										  << packet_count+1 << ": Sequence Counter incremented by 1." << endl;
									//valid = false;
									//return NULL;
									diff_sequence_count = 1;
								}
								sc_array[p][1] = packet->get_sequence_count();
								sc_array[p][3] = sc_array[p][3] + diff_sequence_count;
								if (sc_array[p][3] > 16383)
								{
									sc_array[p][3] = sc_array[p][3] - 16384;
								}
								sc_array[p][4] = sc_array[p][4] + 1;
								counter = sc_array[p][3];
							}
						}
					}

					if (apid_pass == -1)
					{
						sc_ctr++;
						unsigned int tmp_counter = 0;
						string test_str = "counter_initial_" + ntos(sc_ctr);
						string test = Config::file.get_value(test_str);
						if (test != "")
						{
							tmp_counter = atoi(test.c_str());
							if (tmp_counter < 0)
							{
								error << "\nERROR: The value of the configurable field " << test_str
									  <<  " must not be less than 0.\n";
								valid = false;
								return NULL;
							}
							if (tmp_counter > 16383)
							{
								error << "\nERROR: The value of the configurable field " << test_str
									  << " must not exceed 16383 (2^14 - 1).\n";
								valid = false;
								return NULL;
							}
						}
						else
						{
							tmp_counter = Config::counter_initial;
						}

						sc_array[sc_ctr][0] = packet->get_apid();
						sc_array[sc_ctr][1] = packet->get_sequence_count();
						sc_array[sc_ctr][2] = tmp_counter;
						sc_array[sc_ctr][3] = tmp_counter;
						sc_array[sc_ctr][4] = 1;
						diff_sequence_count = 0;
						counter = sc_array[sc_ctr][3];


						if ( strncmp((CCSDSAPIDFilter::apid_to_string(packet->get_apid())).c_str(),"ASCAT",5) == 0
							 || accumulate)
						{
							diff_sequence_count = sc_array[sc_ctr][1] - sc_array[99][1];
							if (diff_sequence_count == -16383)
							{
								diff_sequence_count = 1;
							}
							if (single || diff_sequence_count==0)
							{
								diff_sequence_count = 1;
							}
							if (diff_sequence_count < 0)
							{
								error << "\nWARNING: Sequence Counter is found to decrease in value at packet_count = "
									  << packet_count+1 << ": Sequence Counter incremented by 1." << endl;
								//valid = false;
								//return NULL;
								diff_sequence_count = 1;
							}

							sc_array[sc_ctr][2] = sc_array[99][3] + diff_sequence_count;
							if (sc_array[sc_ctr][2] > 16383)
							{
								sc_array[sc_ctr][2] = sc_array[sc_ctr][2] - 16384;
							}
							sc_array[sc_ctr][3] = sc_array[sc_ctr][2];
							sc_array[sc_ctr][4] = 1;
							sc_array[99][1] = sc_array[sc_ctr][1];
							sc_array[99][3] = sc_array[sc_ctr][3];
							counter = sc_array[sc_ctr][3];
						}
					}

				}

//		diff_sequence_count = packet->get_sequence_count() - sequence_counter;
//		if (diff_sequence_count < 0)
//		{
//		error << "\nERROR: Sequence Counter is found to decrease in value at packet_count = "
//			<< packet_count+1 << endl;
//		valid = false;
//		return NULL;
//		}
//		sequence_counter = packet->get_sequence_count();

				diff_day = packet->get_utc_days() - days;
				if (diff_day < 0)
				{
					error << "\nERROR: UTC Days is found to decrease in value at packet_count = "
						  << packet_count+1 << endl;
					valid = false;
					return NULL;
				}
				days = packet->get_utc_days();

				diff_micro_sec = ( ((double)diff_day*24.0*60.0*60.0)*1.0e6
								   + ((double)packet->get_utc_milliseconds()*1000.0)
								   + packet->get_utc_microseconds() )
					- (((double)milliseconds*1000.0) + microseconds);

				if (diff_micro_sec < 0)
				{
					if ( strncmp((CCSDSAPIDFilter::apid_to_string(packet->get_apid())).c_str(),"IASI",4) != 0)
					{
						error << "\nERROR: UTC Time is found to decrease in value at packet_count = "
							  << packet_count+1 << endl;
						valid = false;
						return NULL;
					}
				}

				DEBUG("CCSDSTimeAndSeqFix: diff_micro_sec	 = " << diff_micro_sec << endl);

				milliseconds = packet->get_utc_milliseconds();
				microseconds = packet->get_utc_microseconds();

				//counter = counter +  diff_sequence_count;
				if (counter > 16383)
				{
					counter = counter - 16384;
				}

				obt_coarse = obt_coarse + (int)(diff_micro_sec/1000000);

				int int_sec = (int)(diff_micro_sec/1000000);
				double fine_tmp =  ((diff_micro_sec/1000000.0 - (double)int_sec) * 65536.0) + 0.5;

				fine_tmp = (double)obt_fine + fine_tmp;

				if (fine_tmp < 0)
				{
					fine_tmp = 65536 + fine_tmp;

					obt_coarse = obt_coarse - 1;
				}

				obt_fine = (unsigned int)fine_tmp;

				//obt_fine = obt_fine + (unsigned int)fine_tmp;

				if (obt_fine > 65535)
				{
					obt_coarse = obt_coarse + 1;
					obt_fine = obt_fine - 65536;
				}
				if (obt_coarse > 16777215)
				{
					obt_coarse = obt_coarse - 16777216;
				}


				// MHS ccsds.34.ccsds testing
//		if ((packet_count+3)%3==0)
//		{
//		obt_fine = obt_fine - 85;
//		}
//		if ((packet_count+2)%3==0)
//		{
//		obt_fine = obt_fine - 86;
//		}
//		if ((packet_count+1)%3==0)
//		{
//		obt_fine = obt_fine + 85 + 86;
//		}

			}

			packet->set_sequence_count(counter);
			packet->set_obt_coarse(obt_coarse);
			packet->set_obt_fine(obt_fine);

			if ( strncmp((CCSDSAPIDFilter::apid_to_string(packet->get_apid())).c_str(),"GRAS",4) != 0 )
			{
				packet->set_error_control(packet->calculate_error_control());
			}

			DEBUG( "		  Application ID			: "
				   << packet->get_apid()
				   << " (" << (CCSDSAPIDFilter::apid_to_string(
								   packet->get_apid())) << ")\n"
				   << "	   Packet Sequence Control\n"
				   << "		   Sequence flags			   : "
				   << packet->get_sequence_flags() << endl
				   << "		   Sequence count			   : "
				   << packet->get_sequence_count() << endl
				   << "		   UTC Days				   : "
				   << (packet->get_utc_days()) << endl
				   << "		   UTC Milliseconds		   : "
				   << (packet->get_utc_milliseconds()) << endl
				   << "		   UTC Microseconds		   : "
				   << (packet->get_utc_microseconds()) << endl
				   << "		   Time : "
				   << (*packet->get_secondary_header()) << endl
				   << "			   Ancillary Data\n"
				   << "				   OBT coarse time : "
				   << (packet->get_obt_coarse()) << endl
				   << "				   OBT fine time   : "
				   << (packet->get_obt_fine()) << endl);

			apid_last = packet->get_apid();
			sequence_count_last = counter;
			utc_days_last = days;
			utc_milliseconds_last = milliseconds;
			utc_microseconds_last = microseconds;
			obt_coarse_last = obt_coarse;
			obt_fine_last = obt_fine;

//	cout << "packet_count=" << packet_count
//		 << "  obt_coarse= " <<	 obt_coarse
//		 << "  obt_fine= " << obt_fine << endl;

			packet_count++;
			return packet;

		}


	//
	// produce_files
	//

	virtual bool produce_files(FileConsumer& dest)
		{
			CCSDS* ccsds;
			while((ccsds=get_next_ccsds()))
			{
				stringstream s;
				s << "." << ccsds->get_apid() << ".ccsds";
				ostream_type* os = dest.get_named_output_stream(s.str());
				os->write((char*)ccsds->get_data(),ccsds->get_length());
			}
			return true;
		}



	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout )
		{
			os << "CCSDSTimeAndSeqFix report:\n"
			   << "	   " << "First Packet\n"
			   << "		   " << "APID				  : " << apid_first << endl
			   << "		   " << "Sequence Count		  : " << sequence_count_first << endl
			   << "		   " << "Time				  : " << TimeConv::utc_days_milliseconds_to_string(utc_days_first,
																									   utc_milliseconds_first)
			   << "." << ntofs(utc_milliseconds_first%1000, 3, '0')
			   << ntofs(utc_microseconds_first, 3, '0')	 << endl
			   << "		   " << "UTC-Days			  : " << utc_days_first	 << endl
			   << "		   " << "UTC-Milliseconds	  : " << utc_milliseconds_first	 << endl
			   << "		   " << "UTC-Microseconds	  : " << utc_microseconds_first	 << endl
			   << "		   " << "OBT-Coarse (sec)	  : " << obt_coarse_first  << endl
			   << "		   " << "OBT-Fine (sec/65536) : " << obt_fine_first	 << endl
			   << "	   " << "Last Packet\n"
			   << "		   " << "APID				  : " << apid_last << endl
			   << "		   " << "Sequence Count		  : " << sequence_count_last  << endl
			   << "		   " << "Time				  : " << TimeConv::utc_days_milliseconds_to_string(utc_days_last,
																									   utc_milliseconds_last)
			   << "." << ntofs(utc_milliseconds_last%1000, 3, '0')
			   << ntofs(utc_microseconds_last, 3, '0') << endl
			   << "		   " << "UTC-Days			  : " << utc_days_last	<< endl
			   << "		   " << "UTC-Milliseconds	  : " << utc_milliseconds_last	<< endl
			   << "		   " << "UTC-Microseconds	  : " << utc_microseconds_last	<< endl
			   << "		   " << "OBT-Coarse (sec)	  : " << obt_coarse_last  << endl
			   << "		   " << "OBT-Fine (sec/65536) : " << obt_fine_last	<< endl
			   << "	   " << "Contents Summary\n";

			for (int p=0; p<=sc_ctr; p++)
			{
				os << "		   APID : " << sc_array[p][0] << endl
				   << "			   Total Number of Packets : " << sc_array[p][4] << endl
				   << "			   First Sequence Count	   : " << sc_array[p][2] << endl
				   << "			   Last Sequence Count	   : " << sc_array[p][3] << endl;
			}
			os << endl;

		}



	//
	// get_next_file
	//

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}


	//
	// is_valid
	//

	bool is_valid(void) const
		{
			return valid;
		}



	//
	// output_text_config
	//

	virtual void output_text_config( textoutstream_type& os )
		{
			os << "\n\nccsds_time_and_seq_fix Configurable Fields\n-------------------------------------\n\n"
			   << "OBT_COARSE_INITIAL = " << Config::obt_coarse_initial << endl
			   << "OBT_FINE_INITIAL	  = " << Config::obt_fine_initial << endl
			   << "COUNTER_INITIAL	  = " << Config::counter_initial << endl;

			int i = 1;
			bool flag = true;
			do
			{
				string test_str = "counter_initial_" + ntos(i);
				string test = Config::file.get_value(test_str);
				if (test != "")
				{
					os << "COUNTER_INITIAL_" << i << "	= " << test << endl;
				}
				else
				{
					flag = false;
				}
				i++;

			} while(flag);
			os << endl;
		}


	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:

	CCSDSProducer* source;
	string config_file;

	APID apid;

	int packet_count;

	bool valid;
	bool accumulate;
	bool single;

	unsigned int counter;
	unsigned int obt_coarse;
	unsigned int obt_fine;

	unsigned int sequence_counter;
	unsigned int days;
	unsigned int milliseconds;
	unsigned int microseconds;

	int diff_sequence_count;
	int diff_day;
	double diff_micro_sec;

	unsigned int apid_first;
	unsigned int sequence_count_first;
	unsigned int utc_days_first;
	unsigned int utc_milliseconds_first;
	unsigned int utc_microseconds_first;
	unsigned int obt_coarse_first;
	unsigned int obt_fine_first;

	unsigned int apid_last;
	unsigned int sequence_count_last;
	unsigned int utc_days_last;
	unsigned int utc_milliseconds_last;
	unsigned int utc_microseconds_last;
	unsigned int obt_coarse_last;
	unsigned int obt_fine_last;

	unsigned int sc_array[100][5];
	unsigned int sc_ctr;

};


#undef Config

#endif
