////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdstimefix.hpp
//
//	Provides: CCSDSTimeFix filter
//
//	Purpose:  Use to set CCSDS packet UTC times.
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef ccsdstimefix_defined
#define ccsdstimefix_defined

#include "interface.hpp"
#include "ccsds.hpp"
#include "ccsdsproducer.hpp"
#include "config.hpp"
#include "textconfigproducer.hpp"
#include "ccsdsapidfilter.hpp"

#define Config Config::CCSDSUTCFix

class CCSDSTimeFix :
	virtual public Interface,
	public CCSDSProducer,
	public FileProducer,
	public TextConfigProducer
{
public:

	//
	// constructor
	//

	CCSDSTimeFix(CCSDSProducer& _source,
				 string config_file,
				 bool _accumulate,
				 string _utc_start,
				 string _obt_start,
				 bool _maintain_time_gaps)
		: source(&_source),
		  apid(0),
		  packet_count(0),
		  accumulate(_accumulate),
		  utc_start(_utc_start),
		  obt_start(_obt_start),
		  maintain_time_gaps(_maintain_time_gaps),
		  days(-1),
		  milliseconds(-1),
		  microseconds(-1),
		  sequence_count_first(0),
		  sequence_count_last(0),
		  in_utc_days_first(0),
		  in_utc_milliseconds_first(0),
		in_utc_microseconds_first(0),
		in_obt_coarse_first(0),
		in_obt_fine_first(0),
		in_utc_days_last(0),
		in_utc_milliseconds_last(0),
		in_utc_microseconds_last(0),
		in_obt_coarse_last(0),
		in_obt_fine_last(0),
		out_utc_days_first(0),
		out_utc_milliseconds_first(0),
		out_utc_microseconds_first(0),
		out_obt_coarse_first(0),
		out_obt_fine_first(0),
		out_utc_days_last(0),
		out_utc_milliseconds_last(0),
		out_utc_microseconds_last(0),
		out_obt_coarse_last(0),
		out_obt_fine_last(0),
		mod_days(0),
		mod_milliseconds(0),
		mod_microseconds(0),
		mod_coarse(0),
		mod_fine(0),
		diff_day(0),
		diff_millisec(0),
		diff_microsec(0)
		{
			if (!config_file.empty())
			{
				Config::file.read(config_file);
			}

			if ((Config::utc_days_start < 0) ||
				(Config::utc_days_start >= MAX_UTC_DAY))
			{
				error << "\nThe value of the configurable field utc_days_start "
					  << "is out of range.\n";
			}

			if((Config::utc_milliseconds_start < 0) ||
			   (Config::utc_milliseconds_start >= MAX_UTC_MILLISECOND))
			{
				error << "\nThe value of the configurable field "
					  << "utc_milliseconds_start is out of range.\n";
			}

			if ((Config::utc_microseconds_start < 0) ||
				(Config::utc_microseconds_start >= MAX_UTC_MICROSECOND))
			{
				error << "\nThe value of the configurable field "
					  << "utc_microseconds_start is out of range.\n";
			}

			if ((Config::utc_days_increment < -1) ||
				(Config::utc_days_increment >= MAX_UTC_DAY))
			{
				error << "\nThe value of the configurable field utc_days_end "
					  << "is out of range.\n";
			}

			if((Config::utc_milliseconds_increment < -1) ||
			   (Config::utc_milliseconds_increment >= MAX_UTC_MILLISECOND))
			{
				error << "\nThe value of the configurable field "
					  << "utc_milliseconds_end is out of range.\n";
			}

			if ((Config::utc_microseconds_increment < -1) ||
				(Config::utc_microseconds_increment >= MAX_UTC_MICROSECOND))
			{
				error << "\nThe value of the configurable field "
					  << "utc_microseconds_end is out of range.\n";
			}

			days = Config::utc_days_start;
			milliseconds = Config::utc_milliseconds_start;
			microseconds = Config::utc_microseconds_start;

			if (Config::utc_days_increment==-1 &&
				Config::utc_milliseconds_increment==-1 &&
				Config::utc_microseconds_increment==-1&&
				utc_start.empty() &&
				obt_start.empty())
			{
				error << "Warning: No operations specified in either config "
					  << "file or command line parameters\n";
			}
		}


	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout )
		{
			os << "CCSDSTimeFix report:\n"
			   << "	   Applying modification of " << mod_days << " days, "
			   << mod_milliseconds << "ms, " << mod_microseconds << "us\n\n"
			   << "	   " << packet_count << " packets read\n"
			   << "		   Sequence Counts	: " << sequence_count_first
			   << "-" << sequence_count_last << endl << endl
			   << "	   Input file\n"
			   << "		   Time				: "
			   << TimeConv::utc_days_milliseconds_to_string(
				   in_utc_days_first, in_utc_milliseconds_first)
			   << " -> "
			   << TimeConv::utc_days_milliseconds_to_string(
				   in_utc_days_last, in_utc_milliseconds_last) << endl
			   << "		   UTC-Days			: "
			   << ntofs(in_utc_days_first,5,' ',true) << "				  -> "
			   << in_utc_days_last << endl
			   << "		   UTC-Milliseconds : "
			   << ntofs(in_utc_milliseconds_first,9,' ',true)
			   << "			   -> "
			   << in_utc_milliseconds_last << endl
			   << "		   UTC-Microseconds : "
			   << ntofs(in_utc_microseconds_first,4,' ',true)
			   << "					-> "
			   << in_utc_microseconds_last << endl
			   << "		   OBT-Coarse		: "
			   << ntofs(in_obt_coarse_first,9,' ',true)
			   << "			   -> " << in_obt_coarse_last << endl
			   << "		   OBT-Fine			: "
			   << ntofs(in_obt_fine_first,6,' ',true) << "				 -> "
			   << in_obt_fine_last << endl << endl
			   << "	   Output file\n"
			   << "		   Time				: "
			   << TimeConv::utc_days_milliseconds_to_string(
				   out_utc_days_first, out_utc_milliseconds_first)
			   << " -> "
			   << TimeConv::utc_days_milliseconds_to_string(
				   out_utc_days_last, out_utc_milliseconds_last) << endl
			   << "		   UTC-Days			: "
			   << ntofs(out_utc_days_first,5,' ',true) << "				   -> "
			   << out_utc_days_last << endl
			   << "		   UTC-Milliseconds : "
			   << ntofs(out_utc_milliseconds_first,9,' ',true)
			   << "			   -> "
			   << out_utc_milliseconds_last << endl
			   << "		   UTC-Microseconds : "
			   << ntofs(out_utc_microseconds_first,4,' ',true)
			   << "					-> "
			   << out_utc_microseconds_last << endl
			   << "		   OBT-Coarse		: "
			   << ntofs(out_obt_coarse_first,9,' ',true)
			   << "			   -> " << out_obt_coarse_last << endl
			   << "		   OBT-Fine			: "
			   << ntofs(out_obt_fine_first,6,' ',true) << "				  -> "
			   << out_obt_fine_last << endl
			   << endl;
		}


	//
	// get_next_ccsds
	//

	virtual CCSDS* get_next_ccsds(void)
		{
			CCSDS* result = source->get_next_ccsds();
			if (!result)
			{
				return NULL;
			}

			if (packet_count==0)
			{
				sequence_count_first = result->get_sequence_count();
				in_utc_days_first = result->get_utc_days();
				in_utc_milliseconds_first = result->get_utc_milliseconds();
				in_utc_microseconds_first = result->get_utc_microseconds();
				in_obt_coarse_first = result->get_obt_coarse();
				in_obt_fine_first = result->get_obt_fine();
			}
			else
			{
				diff_day = result->get_utc_days() - in_utc_days_last;
				diff_millisec = result->get_utc_milliseconds()- in_utc_milliseconds_last;
				diff_microsec = result->get_utc_microseconds() - in_utc_microseconds_last;
			}

			sequence_count_last = result->get_sequence_count();
			in_utc_days_last = result->get_utc_days();
			in_utc_milliseconds_last = result->get_utc_milliseconds();
			in_utc_microseconds_last = result->get_utc_microseconds();
			in_obt_coarse_last = result->get_obt_coarse();
			in_obt_fine_last = result->get_obt_fine();

			// modes:
			// - Original. We have an inital UTC time and a UTC offset time.
			//	 The offset is applied to each packet (or only ones with the
			//	 same APID as the first packet if --accumulate is set).
			// - Alternative. Here the Config::*_increment values are set to -1
			//	 and we apply a fixed offset (calculated during inital packet
			//	 processing, based on utc_time and obt_time) to every packet.

			if (Config::utc_days_increment!=-1 ||
				Config::utc_milliseconds_increment!=-1 ||
				Config::utc_microseconds_increment!=-1)
			{
				result = set_incrementing_time(result);
			}
			else
			{
				result = apply_fixed_offset(result);
			}

			if (result)
			{
				if (packet_count==0)
				{
					out_utc_days_first = result->get_utc_days();
					out_utc_milliseconds_first = result->get_utc_milliseconds();
					out_utc_microseconds_first = result->get_utc_microseconds();
					out_obt_coarse_first = result->get_obt_coarse();
					out_obt_fine_first = result->get_obt_fine();
				}

				out_utc_days_last = result->get_utc_days();
				out_utc_milliseconds_last = result->get_utc_milliseconds();
				out_utc_microseconds_last = result->get_utc_microseconds();
				out_obt_coarse_last = result->get_obt_coarse();
				out_obt_fine_last = result->get_obt_fine();

				result->set_error_control(result->calculate_error_control());
				packet_count++;
			}
			return result;
		}

	CCSDS* set_incrementing_time(CCSDS* result)
		{

			if (packet_count==0)
			{
				apid = result->get_apid();
			}


			if ( apid==result->get_apid() || accumulate )
			{
				int flag = strncmp(
					(CCSDSAPIDFilter::apid_to_string(apid)).c_str(),
					(CCSDSAPIDFilter::apid_to_string(
						result->get_apid())).c_str(), 3);

				if (result->get_apid() == 2 || result->get_apid() == 3)
				{
					if ( strncmp((CCSDSAPIDFilter::apid_to_string(apid)).
								 c_str(),"GRAS",4) == 0
						 || strncmp((CCSDSAPIDFilter::apid_to_string(apid)).
									c_str(),"GOME",4) == 0 )
					{
						flag = 0;
					}
				}

				if (apid == 2 || apid == 3)
				{
					if ( strncmp((CCSDSAPIDFilter::apid_to_string(
									  result->get_apid())).c_str(),
								 "GRAS",4)==0
						 || strncmp(
							 (CCSDSAPIDFilter::apid_to_string(
								 result->get_apid())).c_str(),
							 "GOME",4) == 0 )
					{
						flag = 0;
					}
				}

				if (flag != 0)
				{
					error << "Error: APID mismatch ("
						  << "Intrument APID Expected:"
						  << (CCSDSAPIDFilter::apid_to_string(apid)).c_str()
						  << " Intrument APID Found:"
						  << (CCSDSAPIDFilter::apid_to_string(
								  result->get_apid())).c_str()
						  << " Packet Count:" << packet_count+1 << ")\n";
					return NULL;
				}


				int tmp_microsec = 0;
				if (maintain_time_gaps)
				{
					tmp_microsec = diff_microsec;
				}
				else
				{
					tmp_microsec = Config::utc_microseconds_increment;
				}
				microseconds = microseconds + tmp_microsec;
				if (microseconds >= MAX_UTC_MICROSECOND)
				{
					unsigned int millisec_tmp = microseconds/1000;
					microseconds =	microseconds - (millisec_tmp*1000);
					milliseconds = milliseconds + millisec_tmp;
					if (microseconds >= MAX_UTC_MICROSECOND)
					{
						error << "\nUTC-Microseconds has exceeded its "
							  << "maximum limit of " << MAX_UTC_MICROSECOND-1
							  << " at packet count: " << packet_count+1
							  << ")\n";
						return NULL;
					}
				}

				int tmp_millisec = 0;
				if (maintain_time_gaps)
				{
					tmp_millisec = diff_millisec;
				}
				else
				{
					tmp_millisec = Config::utc_milliseconds_increment;
				}
				milliseconds = milliseconds + tmp_millisec;
				if (milliseconds > MAX_UTC_MILLISECOND)
				{
					unsigned int day_tmp = milliseconds/(1000*60*60*24);
					milliseconds =
						milliseconds - (day_tmp*1000*60*60*24);
					days = days + day_tmp;
					if (milliseconds >= MAX_UTC_MILLISECOND)
					{
						error << "\nUTC-Milliseconds has exceeded its "
							  << "maximum limit of " << MAX_UTC_MILLISECOND-1
							  << " at packet count: " << packet_count+1
							  << ")\n";
						return NULL;
					}
				}

				int tmp_days = 0;
				if (maintain_time_gaps)
				{
					tmp_days = diff_day;
				}
				else
				{
					tmp_days = Config::utc_days_increment;
				}
				days = days + tmp_days;
				if (days >= MAX_UTC_DAY)
				{
					error << "\nUTC-Days has exceeded its maximum "
						  << "limit of " << MAX_UTC_DAY-1
						  << " at packet count: " << packet_count+1
						  << ")\n";
					return NULL;
				}

			}

			result->set_utc_days(days);
			result->set_utc_milliseconds(milliseconds);
			result->set_utc_microseconds(microseconds);
			return result;
		}

	CCSDS* apply_fixed_offset(CCSDS* result)
		{
			int loc_days = result->get_utc_days();
			int loc_milliseconds = result->get_utc_milliseconds();
			int loc_microseconds = result->get_utc_microseconds();

			int coarse = result->get_obt_coarse();
			int fine = result->get_obt_fine();

			if (packet_count==0)
			{
				if (!utc_start.empty())
				{
					// calculate the offset that will bring the first packet
					// to the target initial UTC time

					int target_day;
					int target_millisecond;
					int target_microsecond = 0;

					if (!TimeConv::scan_gen_time(utc_start,
												 target_day,
												 target_millisecond))
					{
						error << "Cannot intepret inital UTC value of '"
							  << utc_start << "'\n";
						return NULL;
					}

					mod_days = target_day - loc_days;
					mod_milliseconds = target_millisecond - loc_milliseconds;
					mod_microseconds = target_microsecond - loc_microseconds;

//			cout << "mod_days " << mod_days << " mod_milliseconds "
//			 << mod_milliseconds << " mod_microseconds "
//			 << mod_microseconds << endl;
				}

				if (!obt_start.empty())
				{
					// calculate OBT time offsets that brings the first packet
					// to the required initial output time

					string::size_type pos = obt_start.find(".");
					if (pos==string::npos)
					{
						pos = obt_start.find(",");
						if (pos==string::npos)
						{
							error << "OBT time not in required format of "
								  << "<coarse>.<fine> ie. --obt-start 2000.5\n";
							return NULL;
						}
					}

					int target_coarse = atoi(obt_start.substr(0,pos).c_str());
					int target_fine = atoi(obt_start.substr(pos+1).c_str());

//			cout << "target_coarse " << target_coarse << " target_fine "
//			 << target_fine << endl;

					mod_coarse = target_coarse - coarse;
					mod_fine = target_fine - fine;
				}
			}

			// modify the packet UTC time

			loc_microseconds += mod_microseconds;
			if (loc_microseconds >= MAX_UTC_MICROSECOND)
			{
				loc_milliseconds += (loc_microseconds/MAX_UTC_MICROSECOND);
				loc_microseconds = loc_microseconds % MAX_UTC_MICROSECOND;
			}
			else if (loc_microseconds < 0)
			{
				loc_milliseconds += (loc_microseconds/(MAX_UTC_MICROSECOND)) - 1;
				loc_microseconds = MAX_UTC_MICROSECOND-
					((-loc_microseconds)%MAX_UTC_MICROSECOND);
			}

			loc_milliseconds += mod_milliseconds;
			if (loc_milliseconds >= MAX_UTC_MILLISECOND)
			{
				loc_days += (loc_milliseconds/MAX_UTC_MILLISECOND);
				loc_milliseconds = loc_milliseconds % MAX_UTC_MILLISECOND;
			}
			else if (loc_milliseconds < 0)
			{
				loc_days += (loc_milliseconds/MAX_UTC_MILLISECOND) - 1;
				loc_milliseconds = MAX_UTC_MILLISECOND-
					((-loc_milliseconds)%MAX_UTC_MILLISECOND);
			}

			loc_days += mod_days;

			// modify the packet OBT time

			fine += mod_fine;
			if (fine >= MAX_OBT_FINE)
			{
				coarse += (fine/MAX_OBT_FINE);
				fine = fine % MAX_OBT_FINE;
			}
			else if (fine < 0)
			{
				coarse += (fine/MAX_OBT_FINE) - 1;
				fine = MAX_OBT_FINE - ((-fine)%MAX_OBT_FINE);
			}
			coarse += mod_coarse;

			// write back values

			result->set_utc_days(loc_days);
			result->set_utc_microseconds(loc_microseconds);
			result->set_utc_milliseconds(loc_milliseconds);
			result->set_obt_coarse(coarse);
			result->set_obt_fine(fine);

			return result;
		}


	//
	// produce_files
	//

	virtual bool produce_files(FileConsumer& dest)
		{
			CCSDS* ccsds;
			while((ccsds=get_next_ccsds()))
			{
				stringstream s;
				s << "." << ccsds->get_apid() << ".ccsds";
				ostream_type* os = dest.get_named_output_stream(s.str());
				os->write((char*)ccsds->get_data(),ccsds->get_length());
			}
			return true;
		}



	//
	// get_next_file
	//

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}



	//
	// output_text_config
	//

	virtual void output_text_config( textoutstream_type& os )
		{
			os << "\n\nCCSDS_UTC_Fix Configurable Fields\n"
			   << "--------------------------------------\n\n"
			   << "utc_days_start		  =	 " << Config::utc_days_start << endl
			   << "utc_milliseconds_start =	 "
			   << Config::utc_milliseconds_start << endl
			   << "utc_microseconds_start =	 "
			   << Config::utc_microseconds_start << endl
			   << "utc_days_increment			=  "
			   << Config::utc_days_increment << endl
			   << "utc_milliseconds_ increment	=  "
			   << Config::utc_milliseconds_increment << endl
			   << "utc_microseconds_increment	=  "
			   << Config::utc_microseconds_increment << endl << endl;
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:

	CCSDSProducer* source;
	APID apid;
	int packet_count;
	bool accumulate;
	string utc_start;
	string obt_start;
	bool maintain_time_gaps;

	int days;
	int milliseconds;
	int microseconds;

	unsigned int sequence_count_first;
	unsigned int sequence_count_last;

	unsigned int in_utc_days_first;
	unsigned int in_utc_milliseconds_first;
	unsigned int in_utc_microseconds_first;
	unsigned int in_obt_coarse_first;
	unsigned int in_obt_fine_first;

	unsigned int in_utc_days_last;
	unsigned int in_utc_milliseconds_last;
	unsigned int in_utc_microseconds_last;
	unsigned int in_obt_coarse_last;
	unsigned int in_obt_fine_last;

	unsigned int out_utc_days_first;
	unsigned int out_utc_milliseconds_first;
	unsigned int out_utc_microseconds_first;
	unsigned int out_obt_coarse_first;
	unsigned int out_obt_fine_first;

	unsigned int out_utc_days_last;
	unsigned int out_utc_milliseconds_last;
	unsigned int out_utc_microseconds_last;
	unsigned int out_obt_coarse_last;
	unsigned int out_obt_fine_last;

	int diff_day;
	int diff_millisec;
	int diff_microsec;

	int mod_days;
	int mod_milliseconds;
	int mod_microseconds;

	int mod_coarse;
	int mod_fine;

};

#undef Config

#endif
