////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdstol0.cpp
//
//	Provides: CCSDSToL0filter
//
//	Purpose:
//
//	Copyright (c) 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <vector>
#include <algorithm>
#include <fstream>

#include "ccsdstol0.hpp"
#include "mphr.hpp"
#include "mmam.hpp"
#include "epsl0.hpp"
#include "epsl0segmenter.hpp"
#include "ccsdsproducer.hpp"

#include "dirent.h"

using namespace std;

string getProductSCID(const string& config_filename)
{
    fstream myFile(config_filename.c_str(),ios::in);
    string line;
    string mySCID;

    while (!myFile.eof())
    {
        getline(myFile, line);
        if (line.find("spacecraft_id") != string::npos)
        {
            mySCID = line.substr(line.find("=")+1);
        }
    }
    myFile.close();
    return mySCID;
}




//--------------------------------------------------------------------------
// CCSDSToL0::CCSDSToL0()
//--------------------------------------------------------------------------
CCSDSToL0::CCSDSToL0(CCSDSProducer& _source,
                     const string& _output_filename,
                     const string& _output_dirname,
                     const string& _config_filename,
                     const string& _config_apid,
                     const string& _subsat_dirname,
                     const string& _osvs_filename,
                     const string& _viadr_filename,
                     const string& _utc_obt_time,
                     const string& _obt_utc_time,
                     const string& _clock_drift,
                     int _rollovers,
                     bool _generateMMAM,
                     bool _generateTLE,
                     bool _validateMMAM,
                     bool _useMMAMdata,
                     const string& _mmam_indir,
                     const string& _mmam_outdir,
                     const string &_tle_outdir,
                     const string& _mmam_schema,
                     unsigned int _segmentSize, const string &_out_suffix)
    : source(_source),
      output_filename(_output_filename),
      output_dirname(_output_dirname),
      config_filename(_config_filename),
      config_apid(_config_apid),
      subsat_dirname(_subsat_dirname),
      osvs_filename(_osvs_filename),
      viadr_filename(_viadr_filename),
      utc_obt_time(_utc_obt_time),
      obt_utc_time(_obt_utc_time),
      clock_drift(_clock_drift),
      rollovers(_rollovers),
      validMMAMloaded(false),
      generateMMAM(_generateMMAM),
      generateTLE(_generateTLE),
      validateMMAM(_validateMMAM),
      m_useMMAMdata(_useMMAMdata),
      mmam_indir(_mmam_indir),
      mmam_outdir(_mmam_outdir),
      tle_outdir(_mmam_outdir),
      mmam_schema(_mmam_schema),
      segmentSize(_segmentSize),
      out_suffix(_out_suffix),
      mmam(0),
      firstTime(true),
      processAborted(false)
{
    //string ccsds_config_file = "/home/aqadir/work/metopizer-autoconf/ccsds.config";
    bool readFlag = ccsdsConfig.read(config_apid);
    if (readFlag)
    {
        debug << "CCSDS-APID config file was successfully read.\n";
        //ccsdsConfig.ccsds_config_summary(info);
        //string test = ccsdsConfig.apid_to_instrument(6);
        //cout << "test = " << test << endl;
    }

    //instatiate segmenter to configurably divide products
    epsl0_segmenter = new EPSL0Segmenter(this, _segmentSize);
}

//--------------------------------------------------------------------------
// CCSDSToL0::~CCSDSToL0()
//--------------------------------------------------------------------------
CCSDSToL0::~CCSDSToL0()
{
    if (mmam) delete mmam;
    delete epsl0_segmenter;
}

//--------------------------------------------------------------------------
// CCSDSToL0::loadCachedMMAM()
//--------------------------------------------------------------------------
bool CCSDSToL0::loadCachedMMAM(const short_cds_time& aTime)
{
    DIR *dir;
    struct dirent *entry;
    string myMMAMfileName;
    vector<string> myMMAMfiles;

    //if the user specified to use mmam but forgot to add the directory name assume current one
    if (mmam_indir.empty())
    {
        char cwd[1024];
        if (getcwd(cwd, sizeof(cwd)) != NULL)
        {
            mmam_indir = (string) cwd + "/";
            cout << "Input MMAM directory not specified. Using current directory: " << mmam_indir << endl;
        }
    }



    string mySCID = getProductSCID(config_filename);

    if ((dir = opendir (mmam_indir.c_str())) != NULL)
    {
        //find MMAM file to open it
        while ((entry = readdir (dir)) != NULL)
        {
            string myEntryName = entry->d_name;
            if (myEntryName.find("MMAM_") == 0 && myEntryName.find(mySCID) != string::npos)
            {
                myMMAMfiles.push_back(mmam_indir + myEntryName);
            }
        }
        closedir (dir);
    }

    //select the appropriate MMAM file
    if (!myMMAMfiles.empty())
    {
        myMMAMfileName = selectValidMMAM(myMMAMfiles, aTime);
    }

    //load the selected MMAM
    if (!myMMAMfileName.empty())
    {
        mmam = new MMAMhandler();
        mmam->parseFromFile(myMMAMfileName);

        //validate schema
        if (validateMMAM)
        {
            if (mmam->schemaValidation(mmam_schema))
            {
                cout << myMMAMfileName << " was succesfully validated against the schema." << endl;
            }
            else
            {
                cout << myMMAMfileName << " did not succesfully pass schema validation." << endl;

                exit(-1);
                return false;
            }
        }

        //keep all the data retrieved from the mmam file for later use
        saveMMAMdata();

        cout << "MMAM file " << myMMAMfileName << " loaded from disk." << endl;
        return true;
    }
    else
    {
        cout << "No suitable initial MMAM has been found in '" <<  mmam_indir;
        cout << "'. The conversion process will start without using an MMAM, " << endl;
        cout << "but as soon as a valid MMAM is found within the input product(s) it will be used." << endl;

        return false;
    }
}


//--------------------------------------------------------------------------
// CCSDSToL0::selectValidMMAM()
//--------------------------------------------------------------------------
string CCSDSToL0::selectValidMMAM(vector<string>& aMMAMfiles,const short_cds_time& aTimestamp)
{
    //sort the names and choose the last element (should have the highest numbered MMAM file)
    sort(aMMAMfiles.begin(), aMMAMfiles.end());

    for (int i=0; i< aMMAMfiles.size(); i++)
    {

        //split path and filename
        size_t found = aMMAMfiles[i].find_last_of("/");
        string myPath = aMMAMfiles[i].substr(0,found);
        string myFileName = aMMAMfiles[i].substr(found+1);

        short_cds_time myValidFrom(myFileName.substr(16,15));
        short_cds_time myValidTo(myFileName.substr(32,15));

        if (aTimestamp <= myValidTo && aTimestamp >= myValidFrom)
        {
            return aMMAMfiles[i];
        }
    }


    return "";
}

//--------------------------------------------------------------------------
// CCSDSToL0::saveMMAMdata()
//--------------------------------------------------------------------------
void CCSDSToL0::saveMMAMdata()
{
    if (useMMAMdata())
    {
		//retrieve parsed state vectors from the MMAM
		osvs_anx = mmam->getStateVectorsAtANX();
		osvs = mmam->getStateVectors();

        //retrieve OBT to UTC coefficients from the MMAM
        utc_obt_time = mmam->getUTC0();
        obt_utc_time = mmam->getOBT0();
        clock_drift = mmam->getClockStep();
        rollover_time = mmam->getWrapAroundTime();

        //we should propagate the new coefficients to the EPSL0 object so that it is used asap
        epsl0_segmenter->propagateOBTcoefficients();
    }
}


//--------------------------------------------------------------------------
// CCSDSToL0::checkTimeValidity()
//--------------------------------------------------------------------------
bool CCSDSToL0::checkTimeValidity(const short_cds_time& aInitialTimestamp,const short_cds_time& aEndTimestamp )
{
    if (useMMAMdata())
    {
        if (mmam)
        {
            bool combinedValidity = (mmam->checkTimeValidity(aInitialTimestamp) &&
                                    mmam->checkTimeValidity(aEndTimestamp));
            return  combinedValidity;
        }
    }

    return true;
}


//--------------------------------------------------------------------------
// CCSDSToL0::isMMAMdataAvailable()
//--------------------------------------------------------------------------
bool CCSDSToL0::isMMAMdataAvailable()
{
    if (mmam)
    {
        return true;
    }
    else
    {
        return false;
    }
}

//--------------------------------------------------------------------------
// CCSDSToL0::getStateVectorAtANX()
//--------------------------------------------------------------------------
OSVS::OSV CCSDSToL0::getStateVectorAtANX(const short_cds_time& time) const
{
	return osvs_anx.get_osv(time);
}

//--------------------------------------------------------------------------
// CCSDSToL0::getStateVector()
//--------------------------------------------------------------------------
OSVS::OSV CCSDSToL0::getStateVector(const short_cds_time& time) const
{
	return osvs.get_interpolated_osv(time);
}

//--------------------------------------------------------------------------
// CCSDSToL0::getUTC0()
//--------------------------------------------------------------------------
string CCSDSToL0::getUTC0()
{
    return utc_obt_time;
}
//--------------------------------------------------------------------------
// CCSDSToL0::getOBT0()
//--------------------------------------------------------------------------
string CCSDSToL0::getOBT0()
{

    if (obt_utc_time.empty())
    {
        return obt_utc_time;
    }
    else
	{
        //have to convert to a <coarse>.<fine> format. ie. 2000.5
        unsigned int  myOBT0 = atoll(obt_utc_time.c_str());

        stringstream ss;
        ss << myOBT0/256;
        string strCoarse = ss.str();

        stringstream ss2;
        ss2 << myOBT0%256;
        string strFine = ss2.str();


        string tmp = strCoarse + "." + strFine ;
        return tmp;
    }
}

//--------------------------------------------------------------------------
// CCSDSToL0::getClockDrift()
//--------------------------------------------------------------------------
string CCSDSToL0::getClockDrift()
{
    return clock_drift;
}

//--------------------------------------------------------------------------
// CCSDSToL0::getRolloverTime()
//--------------------------------------------------------------------------
string CCSDSToL0::getRolloverTime()
{
	return rollover_time;
}

//--------------------------------------------------------------------------
// CCSDSToL0::getRollovers()
//--------------------------------------------------------------------------
int CCSDSToL0::getRollovers()
{
    return rollovers;
}

//--------------------------------------------------------------------------
// CCSDSToL0::create_epsl0()
//--------------------------------------------------------------------------
bool CCSDSToL0::create_epsl0(void)
{
    CCSDS* ccsds = source.get_next_ccsds();
    if (!ccsds)
    {
        return false;
    }

    //if this is the first iteration, then load MMAM file from disk
    if (firstTime)
    {
        firstTime = false;

        if (useMMAMdata())
        {
            //we need to get ahold of valid cached MMAM data, so first retrieve the timestamp of the first packet...
            short_cds_time myInitial_utc;
            myInitial_utc.day = ccsds->get_utc_days();
            myInitial_utc.millisec_day = ccsds->get_utc_milliseconds();

            //...and then look for an adequate MMAM for that time
            loadCachedMMAM(myInitial_utc);
        }
    }

    //retrieve Administrative message (MMAM) from source packets
    if (ccsds->get_apid() == 6)
    {
        handleMMAMisp(ccsds);
    }

    //Group related apids in instruments for L0 products generation
    //The mapping is provided in a configuration file provided at input
    string myInstrumentName = ccsdsConfig.apid_to_instrument(ccsds->get_apid());

    //if the apid could not be mapped to an instrument, then ignore current packet
    if (myInstrumentName.empty())
    {
        return true;
    }

    //if the user wants to segment the output we must divide the ISPs in time segmented groups
    epsl0_segmenter->addISP(ccsds);

    return true;
}



//--------------------------------------------------------------------------
// CCSDSToL0::handleMMAMisp()
//--------------------------------------------------------------------------
void CCSDSToL0::handleMMAMisp(CCSDS* ccsds)
{
    //store the packets so that they can be later used for the rest of the processing
    MMAMhandler* myMmam = new MMAMhandler();
    if (myMmam->parseFromBuf((char*)ccsds->get_payload(),ccsds->get_payload_length()))
    {
        //as we have found a newer MMAM we delete the old cached one and retain this one
        delete mmam;
        mmam = myMmam;

        //if the user has selected to generate MMAM file as an option, do it.
        if (generateMMAM)
        {
            myMmam->generateMMAMfileFromBuf(mmam_outdir);
        }

        //if the user has selected to generate TLE file as an option, do it.
        if (generateTLE)
        {
            myMmam->generateTLEfile(tle_outdir);
        }

        //check whether the user want to validate the schema of the MMAM before using it
        if (validateMMAM)
        {
            //validate schema
            if (mmam->schemaValidation(mmam_schema))
            {
                //keep all the data retrieved from the mmam file for later use
                saveMMAMdata();
            }
            else
            {
                cerr << "Received MMAM did not succesfully pass schema validation." << endl;
            }
        }
        else
        {
            //keep all the data retrieved from the mmam file for later use
            saveMMAMdata();
        }
    }
}


//--------------------------------------------------------------------------
// CCSDSToL0::complete_epsl0()
//--------------------------------------------------------------------------
bool CCSDSToL0::complete_epsl0(void)
{
    epsl0_segmenter->completeL0();
    return true;
}


//--------------------------------------------------------------------------
// CCSDSToL0::useMMAMdata()
//--------------------------------------------------------------------------
bool CCSDSToL0::useMMAMdata()
{
    return m_useMMAMdata;
}


//--------------------------------------------------------------------------
// CCSDSToL0::setL0fileAborted()
//--------------------------------------------------------------------------
void CCSDSToL0::setL0fileAborted(bool aState)
{
    processAborted = aState;
}

//--------------------------------------------------------------------------
// CCSDSToL0::getL0fileAborted()
//--------------------------------------------------------------------------
bool CCSDSToL0::getL0fileAborted()
{
    return processAborted;
}
//--------------------------------------------------------------------------
// CCSDSToL0::output_text_summary()
//--------------------------------------------------------------------------
void CCSDSToL0::output_text_summary(ostream& os)
{
    os << "CCSDSToL0 report:\n";

    if (!processAborted)
    {
        epsl0_segmenter->output_text_summary(os);
    }
    else
    {
        os << "	   L0 file(s) were not produced. " << endl;
    }

    os << endl;
}

//--------------------------------------------------------------------------
// CCSDSToL0::output_text_ccsds_config()
//--------------------------------------------------------------------------
void CCSDSToL0::output_text_ccsds_config(void)
{
    ccsdsConfig.ccsds_config_summary(info);
}

//--------------------------------------------------------------------------
// CCSDSToL0::output_text_mphr_config()
//--------------------------------------------------------------------------
void CCSDSToL0::output_text_mphr_config(void)
{
    display_mphr_config(config_filename);
}

//--------------------------------------------------------------------------
// CCSDSToL0::output_text_viadr()
//--------------------------------------------------------------------------
void CCSDSToL0::output_text_viadr(void)
{

    short_cds_time utc_obt_time;

    //cout << "Scanning " << viadr_filename << " for EPS records... ";

    ifstream_type is(viadr_filename.c_str());
    if (!is)
    {
        error << "Cannot open viadr file '" << viadr_filename << "'\n\n";
        return;
    }

    while(!is.eof() && is.gcount()>0)
    {
        REC_HEAD grh;
        is.read(reinterpret_cast<char*>(&grh), REC_HEAD::SIZE);

        //cout << "GRH size " << grh.record_size << "... ";

        if (grh.record_class==7 &&
                grh.instrument_group==0 &&
                grh.record_subclass==0 &&
                grh.record_subclass_version==2 &&
                grh.record_size==38)
        {
            cout << "\nThe input VIADR file: " << viadr_filename << " contains the following data ... \n";

            VIADR_OBT2UTC record;
            is.read(reinterpret_cast<char*>(&record)+REC_HEAD::SIZE,
                    VIADR_OBT2UTC::SIZE-REC_HEAD::SIZE);
            cout << "utc_obt_time.day		   = " << record.utc_obt_day << endl
                 << "utc_obt_time.millisec_day = " << record.utc_obt_millisec << endl
                 << "obt_utc_time.ccu_obt_0		= " << record.ccu_obt_0 << endl
                 << "clock_drift			   = " << record.clock_step << endl
                 << endl;
            //cout << "\nFound viadr-l0-obt2utc record\n";
            return;
        }

        is.seekg(grh.record_size-REC_HEAD::SIZE, ios::cur);
    }

    cout << endl;
    error << "VIADR file does not look like a L0 product\n";
}





