////////////////////////////////////////////////////////////////////////////////
//
//	File: common.cpp
//
//	Provides: misc
//
//	Purpose: Standard libraries and common code
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

string tolower(const string& s)
{
	string t = s;
	for(string::iterator i = t.begin(); i!=t.end(); i++)
	{
		*i = tolower(*i);
	}
	return t;
}

string nts(int chars, char* str)
{
	static char* buffer = NULL;
	static int buffer_len;

	if (!buffer)
	{
		buffer = new char[chars+1];
	}
	else if (buffer && buffer_len < (chars+1))
	{
		delete[] buffer;
		buffer = new char[chars+1];
	}
	strncpy(buffer, str, chars);
	buffer[chars] = '\0';
	return string(buffer);
}

string ntos(int num)
{
	static char buffer[20];
	sprintf(buffer,"%d", num);
	return string(buffer);
}

string ntos(unsigned int num)
{
	static char buffer[20];
	sprintf(buffer,"%d", num);
	return string(buffer);
}

string ntobs(unsigned int num, int bits)
{
	string result;
	for(; bits >= 0; bits--)
	{
		result = string((num & 1) == 1 ? "1" : "0") + result;
		num >>= 1;
	}
	return result;
}

string ntohs(uint8 num)
{
	string result;
	int a = num&15;
	int b = (num>>4)&15;
	if (b<10)
	{
		result += ('0' + b);
	}
	else
	{
		result += ('A' + b - 10);
	}
	if (a<10)
	{
		result += ('0' + a);
	}
	else
	{
		result += ('A' + a - 10);
	}
	return result;
}

string ntofs(int num, unsigned int width, char fillchar, bool rpad)
{
	string res;
	stringstream s;
	if (!rpad)
	{
		s << setiosflags(ios::fixed) << setw(width) << setfill(fillchar)
		  << num;
		res = s.str();
	}
	else
	{
		s << num;
		res = s.str();
		while(res.length() < width)
		{
			res += fillchar;
		}
	}
	if (res.length() > width)
	{
		res = res.substr(width);
	}
	return res;
}

string dto2dp(double num)
{
	stringstream s;
	s << setiosflags(ios::fixed) << setprecision(2) << num;
	return s.str();
}

//
// TimeConv
//

void TimeConv::set_time_zone(void)
{
	putenv((char*)"TZ=UTC");
	tzset();
}

string TimeConv::time_t_to_string(time_t t, string format)
{
	char buffer[200];
#ifdef _HAVE_STRFTIME
	struct tm my_tm;
	gmtime_r(&t, &my_tm);
	strftime(buffer, 200, format.empty()?TIME_FORMAT.c_str():format.c_str(),
			 &my_tm);
#else
	cftime(buffer, reinterpret_cast<char*>(format.empty()?const_cast<char*>(TIME_FORMAT.c_str()):const_cast<char*>(format.c_str())), &t);
#endif
	return string(buffer);
}

int TimeConv::abs_year_day_to_utc_day(int years, int days)
{
	const int DAYS_PER_YEAR = 365;
	const int LEAP_YEAR_FREQ = 4;
	const int NO_LEAP_YEAR_FREQ = 100;
	const int OVERRIDE_NO_LEAP_YEAR_FREQ = 400;
	const int UTC_BASE_YEAR = 2000;

	return (
		(years-UTC_BASE_YEAR)*DAYS_PER_YEAR +
		days +
		static_cast<int>((years-UTC_BASE_YEAR-1)/LEAP_YEAR_FREQ) -
		static_cast<int>((years-UTC_BASE_YEAR-1)/NO_LEAP_YEAR_FREQ) +
		static_cast<int>((years-UTC_BASE_YEAR-1)/OVERRIDE_NO_LEAP_YEAR_FREQ
			));
}

bool TimeConv::scan_gen_time(const string& gen_time,
							 int& days,
							 int& milliseconds)
{
	int scan_year;
	int scan_month;
	int scan_day;
	int scan_hour;
	int scan_minute;
	int scan_second;
	int scan_millisecond = 0;

	if (gen_time.length() == 15)
	{
		sscanf(gen_time.c_str(), "%4d%2d%2d%2d%2d%2d", &scan_year, &scan_month,
			   &scan_day, &scan_hour, &scan_minute, &scan_second);
	}
	else if (gen_time.length()==18)
	{
		sscanf(gen_time.c_str(), "%4d%2d%2d%2d%2d%2d%3d", &scan_year,
			   &scan_month, &scan_day, &scan_hour, &scan_minute, &scan_second,
			   &scan_millisecond);
	}
	else if (gen_time.empty())
	{
		days = 0;
		milliseconds = 0;
	}
	else
	{
		return false;
	}
	struct tm my_tm;
	bzero(&my_tm, sizeof(my_tm));
	my_tm.tm_sec = scan_second;
	my_tm.tm_min = scan_minute;
	my_tm.tm_hour = scan_hour;
	my_tm.tm_mday = scan_day;
	my_tm.tm_mon = scan_month - 1;
	my_tm.tm_year = scan_year - BASE_YEAR;
	my_tm.tm_wday = 0;
	my_tm.tm_yday = 0;
	my_tm.tm_isdst = 0;
	time_t julian = mktime(&my_tm) - UTC_TO_JULIAN_SECONDS;
	days = julian / SECONDS_PER_DAY;
	milliseconds = (julian%SECONDS_PER_DAY)*1000 + scan_millisecond;

	return true;
}

double TimeConv::utc_days_milliseconds_to_matlab(unsigned int days, unsigned int milliseconds)
{
	double result = (
		static_cast<double>(PGC_DAYS_TO_UTC_DAYS) +
		static_cast<double>(days) +
		(static_cast<double>(milliseconds)/
		 static_cast<double>(SECONDS_PER_DAY*1000)));
	return result;
}

time_t TimeConv::utc_days_milliseconds_to_time_t(unsigned int days,
												 unsigned int milliseconds)
{
	return days*SECONDS_PER_DAY + milliseconds/1000 +
		UTC_TO_JULIAN_SECONDS;
}

struct tm TimeConv::utc_days_milliseconds_to_components(unsigned int days,
														unsigned int milliseconds)
{
	time_t t = utc_days_milliseconds_to_time_t(days, milliseconds);
	struct tm my_tm;
	gmtime_r(&t, &my_tm);
	return my_tm;
}

unsigned int TimeConv::utc_days_milliseconds_to_seconds(unsigned int days,
														unsigned int milliseconds)
{
	const unsigned int SECONDS_PER_DAY = 60*60*24;
	const unsigned int MILLISECONDS_PER_SECOND = 1000;
//	cout << "(days_milliseconds_to_seconds days " << days
	// << " milliseconds " << milliseconds << " result "
	// << ((days*SECONDS_PER_DAY)+milliseconds) << ")\n";
	return ((days*SECONDS_PER_DAY)+(milliseconds/MILLISECONDS_PER_SECOND));
}

string TimeConv::utc_days_milliseconds_to_string(unsigned int days,
												 unsigned int milliseconds,
												 string format)
{
	const int SECONDS_PER_DAY = 60*60*24;
	const int DAYS_PER_YEAR = 365;
	const int UTC_TO_JULIAN_YEARS = 2000-1970;
	const int LEAP_YEARS_1970_2000 = 7;
	const int UTC_TO_JULIAN_SECONDS = SECONDS_PER_DAY *
		(DAYS_PER_YEAR*UTC_TO_JULIAN_YEARS + LEAP_YEARS_1970_2000);

	time_t t = days*SECONDS_PER_DAY + milliseconds/1000 +
		UTC_TO_JULIAN_SECONDS;

	return time_t_to_string(t, format);
}

string TimeConv::abs_year_day_millisecond_to_string(int years,
													int days,
													int milliseconds)
{

	int utc_days = abs_year_day_to_utc_day(years, days);
	return utc_days_milliseconds_to_string(utc_days, milliseconds);

}

long long TimeConv::duration_to_int(const string& duration)
{
	string::size_type pos =
		duration.find_first_not_of("+-0123456789");
	string unit = pos==string::npos?"":duration.substr(pos);
	string number_str = duration.substr(0,pos);
//	  cout << "unit '" << unit << "' number_str '" << number_str << "'\n";
	if (unit.empty() || number_str.empty() ||
		number_str.find_first_not_of("+-0123456789")!=
		string::npos)
	{
		cerr << "Syntax of time duration must be "
			 << "[+|-]<number><ms|s|h|o|d> ie. +30s or -102m\n";
		return 0;
	}
	long long number = strtoll(number_str.c_str(),NULL,0);
	if (unit=="d")
	{
		number *= 1000*60*60*24;
	}
	else if (unit=="o")
	{
		number *= 1000*60*102;
	}
	else if (unit=="h")
	{
		number *= 1000*60*60;
	}
	else if (unit=="m")
	{
		number *= 1000*60;
	}
	else if (unit=="s")
	{
		number *= 1000;
	}
	else if (unit != "ms")
	{
		cerr << "unknown unit '" << unit << "' for relative "
			 << "time - use ms, s or m\n";
		return -1;
	}

//	  cout << "duration_to_int returning " << number << endl;

	return number;
}

//
//
// Little endian handlers
//
//

#ifndef WORDS_BIGENDIAN

//
// int16_be
//

int16_be::int16_be(short v)
{
	memcpy(data, &v, 2);
	swap(data[0], data[1]);
}

int16_be::operator short (void) const
{
	short result;
	byte* ptr = reinterpret_cast<byte*>(&result);
	ptr[0] = data[1];
	ptr[1] = data[0];
	return result;
}

//
// uint16_be
//

uint16_be::uint16_be(void)
{
//			  assert(sizeof(uint16_be)==2);
}

uint16_be::uint16_be(unsigned short v)
{
	set(v);
}

uint16_be& uint16_be::operator = (unsigned short v)
{
	set(v);
	return (*this);
}

void uint16_be::set(unsigned short v)
{
	a=(v>>8)&255;
	b=v&255;
}

uint16_be::operator unsigned short (void) const
{
	return get();
}

unsigned short uint16_be::get(void) const
{
	return ((a<<8)+b);
}

uint16_be& uint16_be::operator <<= (int count)
{
	set(get()<<count);
	return *this;
}

uint16_be& uint16_be::operator += (int count)
{
	set(get()+count);
	return *this;
}

uint16_be& uint16_be::operator -= (int count)
{
	set(get()-count);
	return *this;
}

uint16_be& uint16_be::operator ^= (const uint16_be& rhs)
{
	set(get()^rhs.get());
	return *this;
}

uint16_be& uint16_be::operator |= (const uint16_be& rhs)
{
	set(get()|rhs.get());
	return *this;
}

uint16_be uint16_be::operator -- (int)
{
	uint16_be r = get();
	set(get()-1);
	return r;
}

uint16_be& uint16_be::operator %= (int count)
{
	set(get()%count);
	return *this;
}

//
// int32_be
//

int32_be::int32_be(void)
{
//			  assert(sizeof(uint32_be)==4);
}

int32_be::int32_be(signed int v)
{
	set(v);
}

void int32_be::set(signed int v)
{
	a=(v>>24)&255;
	b=(v>>16)&255;
	c=(v>>8)&255;
	d=v&255;
}

int32_be::operator signed int (void) const
{
	return get();
}

signed int int32_be::get(void) const
{
	return ((a<<24)+(b<<16)+(c<<8)+d);
}

int32_be& int32_be::operator += (int count)
{
	set(get()+count);
	return *this;
}

int32_be& int32_be::operator -= (int count)
{
	set(get()-count);
	return *this;
}

int32_be int32_be::operator ++ (int)
{
	int32_be r = get();
	set(get()+1);
	return r;
}

int32_be& int32_be::operator %= (int count)
{
	set(get()%count);
	return *this;
}

//
// uint32_be
//

uint32_be::uint32_be(void)
{
//			  assert(sizeof(uint32_be)==4);
}

uint32_be::uint32_be(unsigned int v)
{
	set(v);
}

void uint32_be::set(unsigned int v)
{
	a=(v>>24)&255;
	b=(v>>16)&255;
	c=(v>>8)&255;
	d=v&255;
}

uint32_be::operator unsigned int (void) const
{
	return get();
}

unsigned int uint32_be::get(void) const
{
	return ((a<<24)+(b<<16)+(c<<8)+d);
}

uint32_be& uint32_be::operator += (int count)
{
	set(get()+count);
	return *this;
}

uint32_be& uint32_be::operator -= (int count)
{
	set(get()-count);
	return *this;
}

uint32_be uint32_be::operator ++ (int)
{
	uint32_be r = get();
	set(get()+1);
	return r;
}

uint32_be& uint32_be::operator %= (int count)
{
	set(get()%count);
	return *this;
}

//
// uint64_be
//

uint64_be::uint64_be(void)
{
//			  assert(sizeof(uint64_be)==8);
}

uint64_be::uint64_be(unsigned long long v)
{
	set(v);
}

void uint64_be::set(unsigned long long v)
{
	a=(v>>56)&255;
	b=(v>>48)&255;
	c=(v>>40)&255;
	d=(v>>32)&255;
	e=(v>>24)&255;
	f=(v>>16)&255;
	g=(v>>8)&255;
	h=v&255;
}

uint64_be::operator unsigned long long (void) const
{
	return get();
}

unsigned long long uint64_be::get(void) const
{
	return((static_cast<unsigned long long>(a)<<56)+(static_cast<unsigned long long>(b)<<48)+
		   (static_cast<unsigned long long>(c)<<40)+(static_cast<unsigned long long>(d)<<32)+
		   (static_cast<unsigned long long>(e)<<24)+(static_cast<unsigned long long>(f)<<16)+
		   (static_cast<unsigned long long>(g)<<8)+static_cast<unsigned long long>(h));
}

uint64_be& uint64_be::operator += (long long count)
{
	set(get()+count);
	return *this;
}

uint64_be& uint64_be::operator -= (long long count)
{
	set(get()-count);
	return *this;
}

uint64_be uint64_be::operator ++ (int)
{
	unsigned long long r = get();
	set(get()+1);
	return r;
}

uint64_be& uint64_be::operator %= (long long count)
{
	set(get()%count);
	return *this;
}

uint64_be& uint64_be::operator |= (unsigned long long count)
{
	set(get()|count);
	return *this;
}

float64_be::operator double (void) const
{
	double result;
	byte* ptr = reinterpret_cast<byte*>(&result);
	for(int i=0; i<8; i++)
	{
		ptr[7-i] = data[i];
	}
	return result;
}

#endif


//
// uint24_be
//

uint24_be::uint24_be(void)
{
//			  assert(sizeof(uint24_be)==3);
}

uint24_be::uint24_be(unsigned long v)
{
	set(v);
}

uint24_be& uint24_be::operator = (unsigned long v)
{
	set(v);
	return (*this);
}

void uint24_be::set(unsigned long v)
{
	a=(v>>16)&255;
	b=(v>>8)&255;
	c=v&255;
}

uint24_be::operator unsigned long (void) const
{
	return get();
}

unsigned long uint24_be::get(void) const
{
	return ((a<<16)+(b<<8)+c);
}


//
// int24_be
//

int24_be::int24_be(void)
{
//			  assert(sizeof(int24_be)==3);
}

int24_be::int24_be(signed long v)
{
	set(v);
}

int24_be& int24_be::operator = (signed long v)
{
	set(v);
	return (*this);
}

void int24_be::set(signed long v)
{
	a=(v>>16)&255;
	b=(v>>8)&255;
	c=v&255;
}

int24_be::operator signed long (void) const
{
	return get();
}

signed long int24_be::get(void) const
{
	return ((a<<16)+(b<<8)+c);
}



//
//
// Large file version of ifstream
//
//

#ifdef _USE_LARGE_FILES
ifstream64::ifstream64(const char* filename)
{
	file = fopen(filename, "r");
	owner = true;
}

ifstream64::ifstream64(FILE* f)
{
	file = f;
	owner = false;
}

ifstream64::~ifstream64(void)
{
	if (file && owner)
	{
		fclose(file);
	}
}

void ifstream64::read(char* buffer, streampos_type length)
{
	last_read = fread(buffer, 1, length, file);
}

bool ifstream64::good(void) const
{
	return ((file!=NULL) && (feof(file)==0));
}

ifstream64::operator bool (void)
{
	return good();
}

bool ifstream64::eof(void) const
{
	return (!good());
}

void ifstream64::seekg(streampos_type pos, ios::seekdir d)
{
	int whence;
	switch(d)
	{
	case ios::beg:
		whence = SEEK_SET;
		break;

	case ios::cur:
		whence = SEEK_CUR;
		break;

	case ios::end:
		whence = SEEK_END;
		break;

	default:
		break;
	}
	fseek(file, pos, whence);
}

streampos_type ifstream64::tellg(void) const
{
	return ftello(file);
}

streampos_type ifstream64::gcount(void) const
{
	return last_read;
}

ofstream64::ofstream64(const char* filename)
	: file(fopen(filename, "w")),
	  owner(true),
	  fail_bit(false),
	  total_written(0)
{
	if (!file)
	{
		stringstream str;
		str << "Cannot write to output file '" << filename << "'";
		perror(str.str().c_str());
		exit(-1);
	}
}

ofstream64::ofstream64(FILE* f)
	: file(f),
	  owner(false),
	  fail_bit(false),
	  total_written(0)
{
}

ofstream64::~ofstream64(void)
{
	if (owner)
	{
		fclose(file);
	}
}

void ofstream64::write(const char* buffer, unsigned int length)
{
	if (!fail_bit)
	{
		streampos_type written = fwrite(buffer, 1, length, file);
		if (written != length)
		{
			fail_bit = true;
			perror("Write error occured");
			exit(-1);
		}
		else
		{
			fail_bit = false;
			total_written += length;
		}
	}
}

streampos_type ofstream64::tellp(void) const
{
	return ftell(file);
}

void ofstream64::seekg(streampos_type pos, ios::seekdir d)
{
	int whence;
	switch(d)
	{
	case ios::beg:
		whence = SEEK_SET;
		break;

	case ios::cur:
		whence = SEEK_CUR;
		break;

	case ios::end:
		whence = SEEK_END;
		break;

	default:
		break;
	}
	fseek(file, pos, whence);
}

bool ofstream64::fail(void) const
{
	return fail_bit;
}

streampos_type ofstream64::get_total_written(void) const
{
	return total_written;
}

ostream& operator << (ostream& os, const StringList& sl)
{
	for(StringList::const_iterator iter=sl.begin();
		iter!=sl.end();
		iter++)
	{
		if (iter!=sl.begin())
		{
			os << ",";
		}
		os << (*iter);
	}
	return os;
}

DelimitedList::DelimitedList(char delimiter, string in)
{
	string::size_type pos;
	while((pos=in.find(delimiter)) != string::npos)
	{
		push_back(in.substr(0,pos));
		in.erase(0,pos+1);
	}
	push_back(in);
}
#endif

StringList make_glob(const string& spec)
{
	glob_t g;
	glob(spec.c_str(), 0, NULL, &g);
	char** ptr = g.gl_pathv;
	StringList result;
	if (ptr)
	{
		while(*ptr)
		{
			result.push_back(string(*ptr));
			ptr++;
		}
	}
	globfree(&g);
	return result;
}
