////////////////////////////////////////////////////////////////////////////////
//
//	File: config.hpp
//
//	Provides: Misc configuration code
//
//	Purpose: Parse and store configuration file settings
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef config_defined
#define config_defined

const string COMMENT_CHAR = "#";

namespace Config
{
	class ConfigFile
	{
	public:
		ConfigFile(void)
			: loaded(false)
			{
			}

		bool read(const string& filename)
			{
				DEBUG("config reading " << filename << endl);
				ifstream ifs(filename.c_str());
				if (!ifs)
				{
					if (filename.empty())
					{
						error << "Warning: No configuration file specified - using "
							  << "defaults\n";
					}
					else
					{
						error << "Warning: Configuration file '" << filename
							  << "' could not be opened - using defaults\n";
					}
					return false;
				}
				else
				{
					int line_number = 0;
					while(ifs)
					{
						line_number++;

						char line[256];
//			ifs >> line;
						ifs.getline(line, 256);
						string s = line;

//			cout << "LINE: '" << s << "'\n";

						// remove comments
						string::size_type c = s.find(COMMENT_CHAR);
						if (c != string::npos)
						{
							s.erase(c);
						}

//			cout << "nocomment '" << s << "'\n";

						// remove whitespace
						while( (c=s.find(" ")) != string::npos)
						{
							s.erase(c,1);
						}

//			cout << "nowhitespace '" << s << "'\n";


						// remove ascii carriage return
						while( (c=s.find(char(13))) != string::npos)
						{
							s.erase(c,1);
						}


						// only examine non empty lines
						if (!s.empty())
						{
							c = s.find("=");
							if (c != string::npos)
							{
								string keyword = s.substr(0,c);
								string value = s.substr(c+1);

//				cout << "keyword '" << keyword << "'\n";
//				cout << "value '" << value << "'\n";

								// lower case keywords
//				my_transform(keyword.begin(), keyword.end(),
								// tolower);
								keyword = tolower(keyword);
//				cout << "inserting keyword " << keyword << endl;


								pairs[keyword] = value;
							}
							else
							{
								error << "\nLine number " << line_number << " in file "
									  << filename << " is not understood. Check line for "
									  << "unrecognised characters.\n";

//				error << "Unrecognised line ( in config file "
//					  << filename
//					  << " '" << line << "' not understood"
//					  << endl;
							}
						}

					}

//			cout << "configfile " << ((unsigned int)this)
//			<< " loaded true\n";
					loaded = true;

				}
				return true;
			}

		string get_value(const string& keyword)
			{
//		cout << "looking up value '" << tolower(keyword)
				// << "' returning ";
				Pairs::iterator iter = pairs.find(tolower(keyword));
				if (iter != pairs.end())
				{
//			cout << "'" << ((*iter).second) << "'" << endl;
					return ((*iter).second);
				}
				else
				{
//			cout << "''\n";
					return string("");
				}
			}

		bool is_loaded(void) const
			{
				return loaded;
			}

	private:
		typedef map<string,string> Pairs;
		Pairs pairs;
		bool loaded;
	};

	template <typename T> struct Decoder
	{
	};

	template <> struct Decoder<unsigned int>
	{
		Decoder(const string& s)
			{
				int test = atoi(s.c_str());
				if (test>=0)
				{
					value = test;
					valid = true;
				}
				else
				{
					valid = false;
				}
			}
		unsigned int value;
		bool valid;
	};

	template <> struct Decoder<int>
	{
		Decoder(const string& s)
			{
				value = atoi(s.c_str());
				valid = true;
			}
		int value;
		bool valid;
	};

	template <> struct Decoder<double>
	{
		Decoder(const string& s)
			{
				value = atof(s.c_str());
				valid = true;
			}
		double value;
		bool valid;
	};

	template <> struct Decoder<string>
	{
		Decoder(const string& s)
			{
//		cout << "decoder<string> decoding '" << s << "'\n";
				value = s;
				valid = true;
			}
		string value;
		bool valid;
	};

	template <> struct Decoder<bool>
	{
		Decoder(const string& s)
			{
				if (!s.empty())
				{
					if (s=="1" || tolower(s[0])=='y' || tolower(s[0])=='t'
						|| strncasecmp(s.c_str(),"on",2)==0)
					{
						value = true;
						valid = true;
//			cout << "	 found a true\n";
					}
					else if (s=="0" || tolower(s[0])=='n' || tolower(s[0])=='f'
							 || strncasecmp(s.c_str(),"off",3)==0)
					{
						value = false;
						valid = true;
//			cout << "	 found a false\n";
					}
					else
					{
						valid = false;
//			cout << "	 found an invalid value\n";
					}
				}
				else
				{
					valid = false;
//			cout << "	 did not find entry\n";
				}
			}
		bool value;
		bool valid;
	};

	template <typename ParamType> class Parameter
	{
	public:
		Parameter(
			ConfigFile& _config_file,
			const string& _keyword,
			const ParamType& _default_value)
			: config_file(_config_file)
			, keyword(_keyword)
			, default_value(_default_value)
			, error_shown(false)
			{
			}

		operator ParamType (void) const
			{
				return get();
			}

		ParamType get(void) const
			{
//		cout << "looking for config '" << keyword << "'\n";

				if (!value.get())
				{
					if (config_file.is_loaded())
					{
						string svalue = config_file.get_value(keyword);
						if (svalue.empty())
						{

//				cout << "not found\n";

							if (!error_shown)
							{
								error << "Parameter '" << keyword
									  << "' not found in "
									  << "configuration file (default="
									  << default_value
									  << ")\n";
								error_shown = true;
							}
							value.reset(new ParamType(default_value));
						}
						else
						{
							Decoder<ParamType> decoder(svalue);
							if (decoder.valid)
							{
								value.reset(new ParamType(decoder.value));
							}
							else
							{
								if (!error_shown)
								{
									error << "Unrecognised value '" << svalue
										  << "for configuration parameter '"
										  << keyword << "'\n";
									error_shown = true;
								}
								value.reset(new ParamType(default_value));
							}
						}
					}
					else
					{
						value.reset(new ParamType(default_value));
					}
				}
				return ((*value.get()));
			}

	private:
		ConfigFile& config_file;
		string keyword;
		ParamType default_value;
		mutable bool error_shown;
		mutable auto_ptr<ParamType> value;
	};

	namespace AVHRR
	{
		ConfigFile file;

//	Parameter<bool> scan_line_number_as_sequence_count(file,
//		"Scan-Line-Number-As-Sequence-Count", false);
//	Parameter<unsigned int> initial_sequence_count(file,
//		"Initial-Sequence-Count", 0);

		Parameter<unsigned int> initial_sequence_count_103(
			file, "Initial-Sequence-Count-103", 0);
		Parameter<unsigned int> initial_sequence_count_104(
			file, "Initial-Sequence-Count-104", 0);

		Parameter<bool> utc_ignore_input_time(
			file, "UTC-Ignore-Input-Time", false);
		Parameter<int> utc_days_offset(file, "UTC-Days-Offset", 0);
		Parameter<int> utc_milliseconds_offset(
			file, "UTC-Milliseconds-Offset", 0);
		Parameter<int> utc_microseconds_offset(
			file, "UTC-Microseconds-Offset", 0);
		Parameter<int> obt_coarse_offset(file, "OBT-Coarse-Offset", 0);
		Parameter<int> obt_fine_offset(file, "OBT-Fine-Offset", 0);
		Parameter<bool> obt_fixed_fine_time(
			file, "OBT-Fixed-Fine-Time", false);

	}

	namespace AMSUA1
	{
		ConfigFile file;

		Parameter<int> reflector_pos_error_bit(file,
											   "Reflector-Pos-Error-Bit", 1);
		Parameter<int> serial_number(file, "Serial-Number", 29);
		Parameter<int> unit_id(file,"Unit-ID", 0);
		Parameter<string> filler_pattern(
			file,"Filler-Pattern", "XXXXXXXX........");
		Parameter<int> filler_random_speckle(
			file,"Filler-Random-Speckle", 100);

		Parameter<bool> scan_line_number_as_sequence_count(file,
														   "Scan-Line-Number-As-Sequence-Count", false);
		Parameter<unsigned int> initial_sequence_count(file,
													   "Initial-Sequence-Count", 0);
		Parameter<bool> utc_ignore_input_time(
			file, "UTC-Ignore-Input-Time", false);
		Parameter<int> utc_days_offset(file, "UTC-Days-Offset", 0);
		Parameter<int> utc_milliseconds_offset(
			file, "UTC-Milliseconds-Offset", 0);
		Parameter<int> utc_microseconds_offset(
			file, "UTC-Microseconds-Offset", 0);
		Parameter<int> obt_coarse_offset(file, "OBT-Coarse-Offset", 0);
		Parameter<int> obt_fine_offset(file, "OBT-Fine-Offset", 0);
		Parameter<bool> obt_fixed_fine_time(file, "OBT-Fixed-Fine-Time", false);
	}

	namespace AMSUA2
	{
		ConfigFile file;

		Parameter<int> reflector_pos_error_bit(file,
											   "Reflector-Pos-Error-Bit", 1);
		Parameter<int> serial_number(file, "Serial-Number", 30);
		Parameter<int> unit_id(file,"Unit-ID", 0);
		Parameter<string> filler_pattern(file, "Filler-Pattern", ".....;;Xxx");
		Parameter<int> filler_random_speckle(file, "Filler-Random-Speckle", 0);

		Parameter<bool> scan_line_number_as_sequence_count(file,
														   "Scan-Line-Number-As-Sequence-Count", false);
		Parameter<unsigned int> initial_sequence_count(file,
													   "Initial-Sequence-Count", 0);
		Parameter<bool> utc_ignore_input_time(
			file, "UTC-Ignore-Input-Time", false);
		Parameter<int> utc_days_offset(file, "UTC-Days-Offset", 0);
		Parameter<int> utc_milliseconds_offset(
			file, "UTC-Milliseconds-Offset", 0);
		Parameter<int> utc_microseconds_offset(
			file, "UTC-Microseconds-Offset", 0);
		Parameter<int> obt_coarse_offset(file, "OBT-Coarse-Offset", 0);
		Parameter<int> obt_fine_offset(file, "OBT-Fine-Offset", 0);
		Parameter<bool> obt_fixed_fine_time(file, "OBT-Fixed-Fine-Time", false);
	}

	namespace MHS
	{
		ConfigFile file;

		Parameter<int> scan_mode(file, "Scan-Mode", 3);
		Parameter<int> pie_id(file, "PIE-ID", 0);
		Parameter<int> subcommutation_initial_value(
			file, "Sub-Commutation-Initial-Value", 1);
		Parameter<int> subcommutation_mod_value(
			file, "Sub-Commutation-Mod-Value", 1);
		Parameter<int> subcommutation_min_value(
			file, "Sub-Commutation-Min-Value", 1);
		Parameter<int> subcommutation_max_value(
			file, "Sub-Commutation-Max-Value", 1);

		Parameter<int> tc_clean(file, "TC-Clean", 0);
		Parameter<int> tc_conforms(file, "TC-Conforms", 0);
		Parameter<int> tc_recognised(file, "TC-Recognised", 0);
		Parameter<int> tc_legal(file, "TC-Legal", 0);
		Parameter<int> fdm_motor_current_trip_status(
			file, "FDM-Motor-Current-Trip-Status", 0);
		Parameter<int> tc_application_id(file, "TC-Application-ID", 0);
		Parameter<int> tc_application_process_id(
			file,"TC-Application-Process-ID", 0);
		Parameter<int> tc_packet_sequence_count(
			file, "TC-Packet-Sequence-Count", 0);
		Parameter<int> tc_packet_sequence(file, "TC-Packet-Sequence", 0);
		Parameter<int> tc_received_count(file, "TC-Received-Count", 0);
		Parameter<int> monitor_current_fault(file, "Monitor-Current-Fault", 0);
		Parameter<int> thermistor_monitor_fault(
			file, "Thermistor-Monitor-Fault", 0);
		Parameter<int> switch_fault(file, "Switch-Fault", 0);
		Parameter<int> processor_fault(file, "Processor-Fault", 0);
		Parameter<int> rdm_motor_current_trip_status(
			file, "RDM-Motor-Current-Trip-Status", 0);
		Parameter<int> dc_offset_error(file, "DC-Offset-Error", 0);
		Parameter<int> scan_control_error(file, "Scan-Control-Error", 0);
		Parameter<int> ref_ck_error(file, "REF-CK-Error", 0);

		Parameter<int> receiver_channel_h1(
			file, "Receiver-Channel-H1-On-Off", 1);

		Parameter<int> receiver_channel_h1_local_oscillator(
			file, "Receiver-Channel-H1-Local-Oscillator-A-B-Selected", 0);

		Parameter<int> receiver_channel_h2(
			file, "Receiver-Channel-H2-On-Off", 1);

		Parameter<int> receiver_channel_h2_local_oscillator(
			file, "Receiver-Channel-H2-Local-Oscillator-A-B-Selected", 0);

		Parameter<int> receiver_channel_h3_h4_frontend(
			file, "Receiver-Channels-H3-H4-Frontend-On-Off", 1);

		Parameter<int> receiver_channel_h3_h4_local_oscillator(
			file, "Receiver-Channels-H3-H4-Local-Oscillator-A-B-Selected", 0);

		Parameter<int> receiver_channel_h3_backend(
			file, "Receiver-Channel-H3-Backend-On-Off", 1);

		Parameter<int> receiver_channel_h4_backend(
			file, "Receiver-Channel-H4-Backend-On-Off", 1);

		Parameter<int> receiver_channel_h5(
			file, "Receiver-Channel-H5-On-Off", 1);

		Parameter<int> receiver_channel_h5_local_oscillator(
			file, "Receiver-Channel-H5-Local-Oscillator-A-B-Selected", 0);

		Parameter<int> rx_cv(
			file, "RX-CV-On-Off", 1);

		Parameter<int> receiver_operational(
			file, "Receiver-Operational", 1);

		Parameter<int> scan_mechanism_operational_heaters(
			file, "Scan-Mechanism-Operational-Heaters-On-Off", 1);

		Parameter<int> auxiliary_operational_heaters(
			file, "Auxiliary-Operational-Heaters-On-Off", 1);

		Parameter<int> signal_processing_electronics(
			file, "Signal-Processing-Electronics-On-Off", 1);

		Parameter<int> prom_failure(
			file, "PROM-Failure", 0);

		Parameter<int> rdm_zero_position_sensors(
			file, "RDM-Zero-Position-Sensors-A-B-Selected", 0);

		Parameter<int> fdm_zero_position_sensors(
			file, "FDM-Zero-Position-Sensors-A-B-Selected", 0);

		Parameter<int> rdm_motor_sensors(
			file, "RDM-Motor-Sensors-A-B-Selected", 0);

		Parameter<int> fdm_motor_sensors(
			file, "FDM-Motor-Sensors-A-B-Selected", 0);

		Parameter<int> rdm_motor_supply(
			file, "RDM-Motor-Supply-On-Off", 1);

		Parameter<int> fdm_motor_supply(
			file, "FDM-Motor-Supply-On-Off", 1);

		Parameter<int> rdm_motor_current_trip_enabled(
			file, "RDM-Motor-Current-Trip-Enabled", 0);

		Parameter<int> fdm_motor_current_trip_enabled(
			file, "FDM-Motor-Current-Trip-Enabled", 0);

		Parameter<double> t1(file, "Temperature-T1", 25.0);
		Parameter<double> t2(file, "Temperature-T2", 25.0);
		Parameter<double> t3(file, "Temperature-T3", 25.0);
		Parameter<double> t4(file, "Temperature-T4", 25.0);
		Parameter<double> t5(file, "Temperature-T5", 25.0);
		Parameter<double> t6(file, "Temperature-T6", 25.0);
		Parameter<double> t7(file, "Temperature-T7", 25.0);
		Parameter<double> t8(file, "Temperature-T8", 25.0);
		Parameter<double> t9(file, "Temperature-T9", 25.0);
		Parameter<double> t10(file, "Temperature-T10", 25.0);
		Parameter<double> t11(file, "Temperature-T11", 25.0);
		Parameter<double> t12(file, "Temperature-T12", 25.0);
		Parameter<double> t13(file, "Temperature-T13", 20.0);
		Parameter<double> t14(file, "Temperature-T14", 20.0);
		Parameter<double> t15(file, "Temperature-T15", 20.0);
		Parameter<double> t16(file, "Temperature-T16", 20.0);
		Parameter<double> t17(file, "Temperature-T17", 10.0);
		Parameter<double> t18(file, "Temperature-T18", 10.0);
		Parameter<double> t19(file, "Temperature-T19", 10.0);
		Parameter<double> t20(file, "Temperature-T20", 30.0);
		Parameter<double> t21(file, "Temperature-T21", 31.0);
		Parameter<double> t22(file, "Temperature-T22", 30.0);
		Parameter<double> t23(file, "Temperature-T23", 30.0);
		Parameter<double> t24(file, "Temperature-T24", 26.0);

		Parameter<double> v5_secondary_current_k(
			file, "5V-Secondary-Current-k", 0.01644);

		Parameter<double> v8_secondary_current_k1(
			file, "8V-Secondary-Current-k1", 0.02105);

		Parameter<double> v8_secondary_current_k2(
			file, "8V-Secondary-Current-k2", -0.1067);

		Parameter<double> v15_receiver_current_k1(
			file, "15V-Receiver-Current-k1", 0.007869);

		Parameter<double> v15_receiver_current_k2(
			file, "15V-Receiver-Current-k2", -0.07271);

		Parameter<double> nv15_receiver_current_k1(
			file, "N15V-Receiver-Current-k1", 0.001852);

		Parameter<double> nv15_receiver_current_k2(
			file, "N15V-Receiver-Current-k1", -0.026218);

		Parameter<double> rdm_motor_current_k(
			file, "RDM-Motor-Current-k", 0.01307);

		Parameter<double> fdm_motor_current_k(
			file, "FDM-Motor-Current-k", 0.01307);




		Parameter<double> v5_secondary_current(
			file, "5V-Secondary-Current", 900);

		Parameter<double> v8_secondary_current(
			file, "8V-Secondary-Current", 1444);

		Parameter<double> v15_receiver_current(
			file, "15V-Receiver-Current", 831);

		Parameter<double> nv15_receiver_current(
			file, "N15V-Receiver-Current", 169);

		Parameter<double> rdm_motor_current(
			file, "RDM-Motor-Current", 668);

		Parameter<double> fdm_motor_current(
			file, "FDM-Motor-Current", 668);



		Parameter<int> dc_offset_valid(file, "DC-Offset-Valid", 1);
		Parameter<int> scan_control_valid(file, "Scan-Control-Valid", 1);
		Parameter<int> profile(file, "Profile", 0);

		Parameter<int> channel_h1_dc_offset_word(
			file, "Channel-H1-DC-Offset-Word", 0);

		Parameter<int> channel_h2_dc_offset_word(
			file, "Channel-H2-DC-Offset-Word", 0);

		Parameter<int> channel_h3_dc_offset_word(
			file, "Channel-H3-DC-Offset-Word", 0);

		Parameter<int> channel_h4_dc_offset_word(
			file, "Channel-H4-DC-Offset-Word", 0);

		Parameter<int> channel_h5_dc_offset_word(
			file, "Channel-H5-DC-Offset-Word", 0);

		Parameter<int> h1_valid(file, "H1-Valid", 1);
		Parameter<int> h2_valid(file, "H2-Valid", 1);
		Parameter<int> h3_valid(file, "H3-Valid", 1);
		Parameter<int> h4_valid(file, "H4-Valid", 1);
		Parameter<int> h5_valid(file, "H5-Valid", 1);

		Parameter<int> spe_mux_code(file, "SPE-Mux-Code", 3);
		Parameter<int> h1_gain(file, "H1-Gain", 0);
		Parameter<int> h2_gain(file, "H2-Gain", 0);
		Parameter<int> h3_gain(file, "H3-Gain", 0);
		Parameter<int> h4_gain(file, "H4-Gain", 0);
		Parameter<int> h5_gain(file, "H5-Gain", 0);

		Parameter<int> prt_cal_1(file, "PRT-CAL-1", 4122);
		Parameter<int> prt_cal_2(file, "PRT-CAL-2", 3333);
		Parameter<int> prt_cal_3(file, "PRT-CAL-3", 2822);


		Parameter<bool> scan_line_number_as_sequence_count(file,
														   "Scan-Line-Number-As-Sequence-Count", false);
		Parameter<unsigned int> initial_sequence_count(
			file, "Initial-Sequence-Count", 0);
		Parameter<bool> utc_ignore_input_time(
			file, "UTC-Ignore-Input-Time", false);
		Parameter<int> utc_days_offset(file, "UTC-Days-Offset", 0);
		Parameter<int> utc_milliseconds_offset(
			file, "UTC-Milliseconds-Offset", 0);
		Parameter<int> utc_microseconds_offset(
			file, "UTC-Microseconds-Offset", 0);
		Parameter<int> obt_coarse_offset(file, "OBT-Coarse-Offset", 0);
		Parameter<int> obt_fine_offset(file, "OBT-Fine-Offset", 0);
		Parameter<bool> obt_fixed_fine_time(file, "OBT-Fixed-Fine-Time", false);

		Parameter<int> shaft_position_offset(
			file, "Shaft-Position-Offset", 18638);


		Parameter<string> current_constants_name(
			file, "Current-Constants-Name", "<unknown>");
		Parameter<bool> copy_obt_times(file, "Copy-OBT-Times", true);
	}

	namespace HIRS
	{
		ConfigFile file;

		Parameter<bool> scan_line_number_as_sequence_count(file,
														   "Scan-Line-Number-As-Sequence-Count", false);
		Parameter<unsigned int> initial_sequence_count(file,
													   "Initial-Sequence-Count", 0);
		Parameter<bool> utc_ignore_input_time(
			file, "UTC-Ignore-Input-Time", false);
		Parameter<int> utc_days_offset(file, "UTC-Days-Offset", 0);
		Parameter<int> utc_milliseconds_offset(
			file, "UTC-Milliseconds-Offset", 0);
		Parameter<int> utc_microseconds_offset(
			file, "UTC-Microseconds-Offset", 0);
		Parameter<int> obt_coarse_offset(file, "OBT-Coarse-Offset", 0);
		Parameter<int> obt_fine_offset(file, "OBT-Fine-Offset", 0);
		Parameter<bool> obt_fixed_fine_time(file, "OBT-Fixed-Fine-Time", false);
	}

	namespace CCSDS_Displayer
	{
		ConfigFile file;

		Parameter<string> current_constants_name(
			file, "Current-Constants-Name", "<unknown>");
		Parameter<double> v5_secondary_current_k(
			file, "5V-Secondary-Current-k", 0.01644);
		Parameter<double> v8_secondary_current_k1(
			file, "8V-Secondary-Current-k1", 0.02105);
		Parameter<double> v8_secondary_current_k2(
			file, "8V-Secondary-Current-k2", -0.1067);
		Parameter<double> v15_receiver_current_k1(
			file, "15V-Receiver-Current-k1", 0.007869);
		Parameter<double> v15_receiver_current_k2(
			file, "15V-Receiver-Current-k2", -0.07271);
		Parameter<double> nv15_receiver_current_k1(
			file, "N15V-Receiver-Current-k1", 0.001852);
		Parameter<double> nv15_receiver_current_k2(
			file, "N15V-Receiver-Current-k1", -0.026218);
		Parameter<double> rdm_motor_current_k(
			file, "RDM-Motor-Current-k", 0.01307);
		Parameter<double> fdm_motor_current_k(
			file, "FDM-Motor-Current-k", 0.01307);
	}

	namespace tVCDU
	{
		ConfigFile file;

		Parameter<int> utc_offset_milliseconds(file, "UTCTimeOffset",
											   110*60*1000);
		Parameter<int> utc_time_difference(file, "UTCTimeDifference",
										   80);
		Parameter<int> reed_solomon_status(file, "ReedSolomonStatus", 128);
		Parameter<bool> strip_filler_packets(file, "StripVCDUFillerPackets",
											 false);
	}


	namespace CCSDSTimeAndSeqFix
	{
		ConfigFile file;
		Parameter<unsigned int> counter_initial(file, "COUNTER_INITIAL", 0);
		Parameter<unsigned int> obt_coarse_initial(
			file, "OBT_COARSE_INITIAL", 0);
		Parameter<unsigned int> obt_fine_initial(file, "OBT_FINE_INITIAL", 0);
	}

	namespace CCSDSStripFilter
	{
		ConfigFile file;
		Parameter<unsigned int> utc_days_start(file, "utc_days_start", 0);
		Parameter<unsigned int> utc_milliseconds_start(
			file, "utc_milliseconds_start", 0);
		Parameter<unsigned int> utc_microseconds_start(
			file, "utc_microseconds_start", 0);
		Parameter<unsigned int> utc_days_end(
			file, "utc_days_end", 0);
		Parameter<unsigned int> utc_milliseconds_end(
			file, "utc_milliseconds_end", 0);
		Parameter<unsigned int> utc_microseconds_end(
			file, "utc_microseconds_end", 0);
		Parameter<double> time_gap_limit(file, "time_gap_limit", 600.0);
	}

	namespace CCSDSUTCFix
	{
		ConfigFile file;
		Parameter<unsigned int> utc_days_start(
			file, "utc_days_start", 0);
		Parameter<unsigned int> utc_milliseconds_start(
			file, "utc_milliseconds_start", 0);
		Parameter<unsigned int> utc_microseconds_start(
			file, "utc_microseconds_start", 0);
		Parameter<int> utc_days_increment(
			file, "utc_days_increment", -1);
		Parameter<int> utc_milliseconds_increment(
			file, "utc_milliseconds_increment", -1);
		Parameter<int> utc_microseconds_increment(
			file, "utc_microseconds_increment", -1);
	}

	namespace L0
	{
		ConfigFile file;
		Parameter<string> product_type(file, "product_type", "xxx");
		Parameter<string> spacecraft_id(file, "spacecraft_id", "M01");
		Parameter<int> instrument_model(file, "instrument_model", 1);
		Parameter<string> processing_centre(file, "processing_centre", "CGS1");
		Parameter<string> processing_mode(file, "processing_mode", "N");
		Parameter<string> disposition_mode(file, "disposition_mode", "O");
		Parameter<int> processor_major_version(
			file, "processor_major_version", 1);
		Parameter<int> processor_minor_version(
			file, "processor_minor_version", 0);
		Parameter<string> recieving_ground_station(
			file, "receiving_ground_station", "SVL");
	}
}

#endif
