////////////////////////////////////////////////////////////////////////////////
//
//	File: cvcdutovcdu.cpp
//
//	Provides: CVCDUToVCDU filter
//
//	Purpose: Convert a stream of CVCDU packets to VCDU packets
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "cvcdutovcdu.hpp"
#include "rs.hpp"

CVCDUToVCDU::CVCDUToVCDU(CVCDUProducer* _source,
						 bool _strip_filler,
						 bool _strip_encrypt,
						 bool _rs_checks)
	: source(_source),
	  packet_count(0),
	  filler_count(0),
	  encrypt_count(0),
	  initial_filler(0),
	  final_filler(0),
	  strip_filler(_strip_filler),
	  strip_encrypt(_strip_encrypt),
	  rs_checks(_rs_checks),
	  rs_good_packets(0),
	  rs_corrected_packets(0),
	  rs_bad_packets(0),
	  rs_total_errors(0),
	  version_warnings_count(0)
{
}

void CVCDUToVCDU::output_text_summary( ostream& os )
{
	os << "CVCDUToVCDU report:\n";

	for(SCIDCount::iterator iter=scid_count.begin();
		iter!=scid_count.end();
		iter++)
	{
		os << "	   " << ((*iter).second) << " data packets had SCID "
		   << "of " << ((*iter).first);

		switch((*iter).first)
		{
		case VCID_METOP_1 : os << " (METOP-1)"; break;
		case VCID_METOP_2 : os << " (METOP-2)"; break;
		case VCID_METOP_3 : os << " (METOP-3)"; break;
		case VCID_METOP_SIM : os << " (METOP-SIM)"; break;
		default : os << " (unknown)"; break;
		}

		os << endl;
	}
	if (rs_checks)
	{
		os << "	   " << rs_good_packets<< " packets had correct "
		   << "Reed-Solomon check symbols\n"
		   << "	   " << rs_corrected_packets << " packets had RS "
		   << "correctable errors (" << rs_total_errors << " total"
		   << " corrected bytes)\n"
		   << "	   " << rs_bad_packets << " packets were dropped due "
		   << "to RS errors\n";
	}
	else
	{
		os << "	   Reed-Solomon checks not performed\n";
	}

	if (strip_filler)
	{
		os << "	   " << filler_count
		   << " fill VCDUs were skipped (inital " << initial_filler
		   << ", final " << final_filler << ")\n";
	}
	else
	{
		os << "	   " << filler_count << " Fill VCDU packets (initial " << initial_filler
		   << ", final " << final_filler << ") were passed through\n";
	}

	if (strip_encrypt)
	{
		os << "	   " << encrypt_count
		   << " encrypted VCDUs were removed\n";
	}
	else
	{
		os << "	   " << encrypt_count
		   << " encrypted VCDUs were passed through\n";
	}

	os << endl;
}

VCDU* CVCDUToVCDU::get_next_vcdu(void)
{
	while(CVCDU* cvcdu = source->get_next_cvcdu())
	{
		packet_count++;
		VCDU* vcdu = cvcdu_to_vcdu(cvcdu);
		if (vcdu)
		{
			return vcdu;
		}
	}
	return NULL;
}

VCDU* CVCDUToVCDU::cvcdu_to_vcdu(CVCDU* cvcdu)
{
	byte buffers[4][255];

	int packet_res = 0;
	if (rs_checks)
	{
		for(int i=0; i<4; i++)
		{
			for( int j=0; j<255; j++)
			{
				buffers[i][j] = reinterpret_cast<byte*>(cvcdu)[i+j*4];
			}
		}

		for(int i=0; i<4; i++)
		{
			int res = rs_decode(buffers[i], NULL, 0, 0);
			// int res = decode_rs_ccsds(buffers[i], NULL, 0, 0);
			if (res==-1)
			{
				rs_bad_packets++;
				return NULL;
			}
			packet_res += res;
		}
	}

	if (packet_res==0)
	{
		rs_good_packets++;
	}
	else
	{
		rs_corrected_packets++;
		rs_total_errors += packet_res;
	}

//		cout << "RS decoding: " << packet_res << " errors\n";

	VCDU* vcdu = &cvcdu->payload;

	// increment the SCID counter
//		if (vcdu->get_vcid() != VCID_FILL)
//		{
	scid_count[vcdu->get_scid()] =
		scid_count[vcdu->get_scid()]+1;
//		}

	// if its a filler packet strip it out now
	if (vcdu->get_vcid() == VCID_FILL)
	{
		filler_count++;
		if (filler_count==packet_count)
		{
			initial_filler++;
		}
		final_filler++;
		if (strip_filler)
		{
			return NULL;
		}
	}
	else
	{
		final_filler = 0;
	}

	if (vcdu->get_version_number() != VCDU_CURRECT_VERSION)
	{
		version_warnings_count++;
		if (version_warnings_count < 10)
		{
			info << "CVCDUToVCDU::get_next_vcdu Input packet found "
				 << " with incorrect VCDU version of "
				 << vcdu->get_version_number()
				 << " (expecting " << VCDU_CURRECT_VERSION
				 << ") in packet"
				 << "number " << packet_count << endl;
		}
		else if (version_warnings_count == 10)
		{
			info << "CVCDUToVCDU::get_next_vcdu Further VCDU version "
				 << " warnings will not be shown\n";
		}

		// allow the packet through anyway (it may still be ok)
	}

	// if its encrypted then strip it out
	if (strip_encrypt && vcdu->get_encrypt_flag() == VCDU_ENCRYPTED)
	{
		encrypt_count++;
		return get_next_vcdu();
	}

	return vcdu;
}

MPDU* CVCDUToVCDU::get_next_mpdu(void)
{
	VCDU* vcdu = get_next_vcdu();
	if (!vcdu)
	{
		return NULL;
	}
	return &(vcdu->mpdu);
}
