////////////////////////////////////////////////////////////////////////////////
//
//	File: eps.cpp
//
//	Provides: misc structs
//
//	Purpose: Code for representing generic EPS level lb products
//
//	Copyright (c) 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "eps.hpp"


short_cds_time make_short_cds_time(int day, int millisec_day)
{
	short_cds_time result;
	result.day = day;
	result.millisec_day = millisec_day;
	return result;
}

short_cds_time make_short_cds_time(const string& spec)
{
	short_cds_time result;

	// 20020808121315Z

	int scan_year = 0;
	int scan_month = 0;
	int scan_day = 0;
	int scan_hour = 0;
	int scan_minute = 0;
	int scan_second = 0;
	int scan_millisecond = 0;


    string temp = spec;

    //if we have a pretty print format, convert it
    if (spec.length() == 23)
    {
        //2014-01-09T17:59:57.000 ==> 20140109175957000Z
        char chars[] = "-T:.";

        for (unsigned int i = 0; i < strlen(chars); ++i)
        {
           temp.erase (std::remove(temp.begin(), temp.end(), chars[i]), temp.end());
        }
        temp = temp + "Z";
    }


    if (temp.length() == 15)
	{
        sscanf(temp.c_str(), "%4d%2d%2d%2d%2d%2d", &scan_year, &scan_month,
			   &scan_day, &scan_hour, &scan_minute, &scan_second);
	}
    else if (temp.length()==18)
	{
        sscanf(temp.c_str(), "%4d%2d%2d%2d%2d%2d%3d", &scan_year, &scan_month,
			   &scan_day, &scan_hour, &scan_minute, &scan_second,
			   &scan_millisecond);
	}
	else
	{
        cerr << "Cannot interpret generalised time '" << temp << "'\n";
		return make_short_cds_time();
	}
	if (scan_month<1 || scan_month>12)
	{
        cerr << "Month value in " << temp << " is out of range (must be between 1 and 12)\n";
		return	make_short_cds_time();
	}
	if (scan_day<1 || scan_day>31)
	{
        cerr << "Day value in " << temp << " is out of range (must be between 1 and 31)\n";
		return make_short_cds_time();
	}
	if (scan_hour<0 || scan_hour>23)
	{
        cerr << "Hour value in " << temp << " is out of range (must be between 0 and 23)\n";
		return make_short_cds_time();
	}
	if (scan_minute<0 || scan_minute>59)
	{
        cerr << "minute value in " << temp << " is out of range (must be between 0 and 59)\n";
		return make_short_cds_time();
	}
	if (scan_second<0 || scan_second>59)
	{
        cerr << "Second value in " << temp << " is out of range (must be between 0 and 59)\n";
		return make_short_cds_time();
	}
	struct tm my_tm;
	bzero(&my_tm, sizeof(my_tm));
	my_tm.tm_sec = scan_second;
	my_tm.tm_min = scan_minute;
	my_tm.tm_hour = scan_hour;
	my_tm.tm_mday = scan_day;
	my_tm.tm_mon = scan_month - 1;
	my_tm.tm_year = scan_year - BASE_YEAR;
	my_tm.tm_wday = 0;
	my_tm.tm_yday = 0;
	my_tm.tm_isdst = 0;
	time_t julian = mktime(&my_tm) - UTC_TO_JULIAN_SECONDS;
	result.day = julian / SECONDS_PER_DAY;
	result.millisec_day = (julian%SECONDS_PER_DAY)*1000 + scan_millisecond;

	return result;
}


short_cds_time::short_cds_time(void)
	: day(numeric_limits<uinteger2>::max()),
	  millisec_day(numeric_limits<uinteger4>::max())
{
}

short_cds_time::short_cds_time(const string& t)
{
	// 20020808121315Z

	int scan_year;
	int scan_month;
	int scan_day;
	int scan_hour;
	int scan_minute;
	int scan_second;
	int scan_millisecond = 0;

	if (t.length() == 15)
	{
		sscanf(t.c_str(), "%4d%2d%2d%2d%2d%2d", &scan_year, &scan_month,
			   &scan_day, &scan_hour, &scan_minute, &scan_second);
	}
	else if (t.length()==18)
	{
		sscanf(t.c_str(), "%4d%2d%2d%2d%2d%2d%3d", &scan_year, &scan_month,
			   &scan_day, &scan_hour, &scan_minute, &scan_second,
			   &scan_millisecond);
	}
	else if (t.empty())
	{
		day = 0;
		millisec_day = 0;
	}
	else
	{
		cerr << "Cannot interpret generalised time '" << t << "'\n";
		(*this) = short_cds_time();
		return;
	}
	struct tm my_tm;
	bzero(&my_tm, sizeof(my_tm));
	my_tm.tm_sec = scan_second;
	my_tm.tm_min = scan_minute;
	my_tm.tm_hour = scan_hour;
	my_tm.tm_mday = scan_day;
	my_tm.tm_mon = scan_month - 1;
	my_tm.tm_year = scan_year - BASE_YEAR;
	my_tm.tm_wday = 0;
	my_tm.tm_yday = 0;
	my_tm.tm_isdst = 0;
	time_t julian = mktime(&my_tm) - UTC_TO_JULIAN_SECONDS;
	day = julian / SECONDS_PER_DAY;
	millisec_day = (julian%SECONDS_PER_DAY)*1000 + scan_millisecond;
}

int short_cds_time::get_days()
{
	return day;
}

int short_cds_time::get_milliseconds()
{
	return millisec_day;
}

bool short_cds_time::operator == (const short_cds_time& o) const
{
	return (day==o.day && millisec_day==o.millisec_day);
}

bool short_cds_time::operator != (const short_cds_time& o) const
{
	return (day!=o.day || millisec_day!=o.millisec_day);
}

bool short_cds_time::operator < (const short_cds_time& o) const
{
	if (day!=o.day)
	{
		return (day<o.day);
	}
	else
	{
		return (millisec_day<o.millisec_day);
	}
}

bool short_cds_time::operator > (const short_cds_time& o) const
{
	if (day!=o.day)
	{
		return (day>o.day);
	}
	else
	{
		return (millisec_day>o.millisec_day);
	}
}

bool short_cds_time::operator <= (const short_cds_time& o) const
{
	if (day!=o.day)
	{
		return (day<=o.day);
	}
	else
	{
		return (millisec_day<=o.millisec_day);
	}
}

bool short_cds_time::operator >= (const short_cds_time& o) const
{
	if (day!=o.day)
	{
		return (day>=o.day);
	}
	else
	{
		return (millisec_day>=o.millisec_day);
	}
}

long long short_cds_time::operator - (const short_cds_time& o) const
{
	long long result = static_cast<long long>(day - o.day)*1000*60*60*24;
	result += millisec_day;
	result -= o.millisec_day;
	return result;
}


short_cds_time short_cds_time::operator + (long long mod_milli) const
{
	// ? maybe doesn't work with negative mods

//	  cout << "short_cds_time::- this " << (*this)
//	  << " mod_milli " << mod_milli << "...\n";

	long long new_milli = millisec_day;
	new_milli += mod_milli;
	short_cds_time result;

	if (new_milli > MILLISECONDS_PER_DAY)
	{
		result.day = day + (new_milli/MILLISECONDS_PER_DAY);
		result.millisec_day = new_milli % MILLISECONDS_PER_DAY;
	}
	else if (new_milli < 0)
	{
		result.day = day + (new_milli/MILLISECONDS_PER_DAY) - 1;
		result.millisec_day =
			MILLISECONDS_PER_DAY-((-new_milli)%MILLISECONDS_PER_DAY);
	}
	else
	{
		result.day = day;
		result.millisec_day = new_milli;
	}

//	  cout << "returning " << result << endl;
	return result;
}


short_cds_time short_cds_time::operator - (long long mod_milli) const
{
	short_cds_time result(*this);
	while(mod_milli > result.millisec_day)
	{
		result.millisec_day += 864000;
		result.day--;
	}
	result.millisec_day -= mod_milli;
	while(result.millisec_day < MILLISECONDS_PER_DAY)
	{
		result.millisec_day += MILLISECONDS_PER_DAY;
		result.day--;
	}
	return result;
}

string short_cds_time::to_gen_time(void) const
{
	char TIME_FORMAT[] = "%Y%m%d%H%M%S";

	time_t t = day*SECONDS_PER_DAY + millisec_day/1000 +
		UTC_TO_JULIAN_SECONDS;

	char buffer[200];
#ifdef _HAVE_STRFTIME
	struct tm my_tm;
	gmtime_r(&t, &my_tm);
	strftime(buffer, 200, TIME_FORMAT, &my_tm);
#else
	cftime(buffer, reinterpret_cast<char*>(TIME_FORMAT), &t);
#endif

	string result = string(buffer) + "Z";
	return result;
}

string short_cds_time::to_long_gen_time(void) const
{
	char TIME_FORMAT[] = "%Y%m%d%H%M%S";

	time_t t = day*SECONDS_PER_DAY + millisec_day/1000 +
		UTC_TO_JULIAN_SECONDS;

	char buffer[200];
#ifdef _HAVE_STRFTIME
	struct tm my_tm;
	gmtime_r(&t, &my_tm);
	strftime(buffer, 200, TIME_FORMAT, &my_tm);
#else
	cftime(buffer, reinterpret_cast<char*>(TIME_FORMAT), &t);
#endif

	string result = string(buffer) + ntofs(millisec_day%1000,3) + "Z";
	return result;
}


string short_cds_time::get_raw(void) const
{
	return string(lexical_cast<string>(day) + "," +
				  lexical_cast<string>(millisec_day));
}

time_t short_cds_time::get_epoch(void) const
{
	time_t t = day*SECONDS_PER_DAY + millisec_day/1000 + UTC_TO_JULIAN_SECONDS;
	return t;
}

string short_cds_time::get_cooked(void) const
{
	char TIME_FORMAT[] = "%Y/%m/%d %H:%M:%S";

	time_t t = day*SECONDS_PER_DAY + millisec_day/1000 + UTC_TO_JULIAN_SECONDS;

	char buffer[200];
#ifdef _HAVE_STRFTIME
	struct tm my_tm;
	gmtime_r(&t, &my_tm);
	strftime(buffer, 200, TIME_FORMAT, &my_tm);
#else
	cftime(buffer, TIME_FORMAT, &t);
#endif

	return string(string(buffer)+"."+ntofs(millisec_day%1000,3));
}

short_cds_time short_cds_time::demillisecond(void) const
{
	short_cds_time result(*this);
	result.millisec_day -= (result.millisec_day%1000);
	return result;
}

string short_cds_time::pretty_print(void) const
{
	const char TIME_FORMAT[] = "%d-%b-%Y,%H:%M:%S";
//	 char TIME_FORMAT[] = "%Y/%m/%d %H:%M:%S";

	time_t t = day*SECONDS_PER_DAY + millisec_day/1000 + UTC_TO_JULIAN_SECONDS;

	char buffer[200];
#ifdef _HAVE_STRFTIME
	struct tm my_tm;
	gmtime_r(&t, &my_tm);
	strftime(buffer, 200, TIME_FORMAT, &my_tm);
#else
	cftime(buffer, const_cast<char*>(TIME_FORMAT), &t);
#endif

	return string(string(buffer)+"."+ntofs(millisec_day%1000,3));
}

ostream& operator << (ostream& os, const short_cds_time& cds)
{
	os << cds.day << "/" << cds.millisec_day << " (" << cds.pretty_print()
	   << ")";
	return os;
}

// ostream& operator << (ostream& os, const short_cds_time& s)
// {
//	   os << s.get_cooked() << " (" << s.get_raw() << ")";
//	   return os;
// }

int write_short_cds_time(byte* buffer, const short_cds_time& cds)
{
//	  os.write(reinterpret_cast<const char*>(&cds.day), sizeof(cds.day));
//	  os.write(reinterpret_cast<const char*>(&cds.millisec_day),
//		 sizeof(cds.millisec_day));

	memcpy(buffer, &cds.day, sizeof(cds.day));
	memcpy(buffer+sizeof(cds.day), &cds.millisec_day,
		   sizeof(&cds.millisec_day));

	return 6;
}

int write_rec_head(byte* buffer, const REC_HEAD& rh)
{
//	  os.write(reinterpret_cast<const char*>(&rh.record_class),
//		 sizeof(rh.record_class));
//	  os.write(reinterpret_cast<const char*>(&rh.instrument_group),
//		 sizeof(rh.instrument_group));
//	  os.write(reinterpret_cast<const char*>(&rh.record_subclass),
//		 sizeof(rh.record_subclass));
//	  os.write(reinterpret_cast<const char*>(&rh.record_subclass_version),
//		 sizeof(rh.record_subclass_version));
//	  write_short_cds_time(os, rh.record_start_time);
//	  write_short_cds_time(os, rh.record_stop_time);

	memcpy(buffer, &rh.record_class, sizeof(rh.record_class));
	buffer += sizeof(rh.record_class);
	memcpy(buffer, &rh.instrument_group, sizeof(rh.instrument_group));
	buffer += sizeof(rh.instrument_group);
	memcpy(buffer, &rh.record_subclass, sizeof(rh.record_subclass));
	buffer += sizeof(rh.record_subclass);
	memcpy(buffer, &rh.record_subclass_version, sizeof(rh.record_subclass));
	buffer += sizeof(rh.record_subclass_version);
	memcpy(buffer, &rh.record_size, sizeof(rh.record_size));
	buffer += sizeof(rh.record_size);
	buffer += write_short_cds_time(buffer, rh.record_start_time);
	buffer += write_short_cds_time(buffer, rh.record_stop_time);

	return REC_HEAD::SIZE;
}

ostream& operator << (ostream& os, const REC_HEAD& grh)
{
	os << "class:" << static_cast<int>(grh.record_class)
	   << " instrument:" << static_cast<int>(grh.instrument_group)
	   << " subclass:" << static_cast<int>(grh.record_subclass)
	   << " version:" << static_cast<int>(grh.record_subclass_version)
	   << " size:" << grh.record_size
	   << " start:" << grh.record_start_time
	   << " stop:" << grh.record_stop_time;
	return os;
}
