////////////////////////////////////////////////////////////////////////////////
//
//	File: eps.hpp
//
//	Provides: misc structs
//
//	Purpose: Code for representing generic EPS level lb products
//
//	Copyright (c) 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef eps_defined
#define eps_defined

#include "common.hpp"

typedef	int16_be integer2; //2
typedef	uint16_be uinteger2; //2
typedef	int32_be integer4; //4
typedef	uint32_be uinteger4; //4

typedef	byte bitst8; //1

struct bitst16
{
	byte bitst16[2];
}; //2

struct bitst24
{
	byte bitst24[3];
}; //3

struct bitst32
{
	byte bitst32[4];
}; //4

struct general_time
{
	byte general_time[15];
}; //15

struct long_general_time
{
	char long_general_time[18];
}; //18

struct long_cds_time
{
	uinteger2 day;
	uinteger4 millisec_day;
	uinteger2 millisec_millisec;
}; //8

struct short_cds_time
{
	/// Constructor
	short_cds_time(void);

	/// Constructor
	short_cds_time(const string& time);

	/// days since 1 Jan 2000
	uinteger2 day;
	int get_days();

	/// Milliseconds of day
	uinteger4 millisec_day;
	int get_milliseconds();

	string to_gen_time(void) const;
	string to_long_gen_time(void) const;

	/// Comparison
	bool operator == (const short_cds_time& o) const;
	bool operator != (const short_cds_time& o) const;
	bool operator < (const short_cds_time& o) const;
	bool operator > (const short_cds_time& o) const;
	bool operator >= (const short_cds_time& o) const;
	bool operator <= (const short_cds_time& o) const;

	/// Subtract milliseconds from self
	long long operator - (const short_cds_time& o) const;

	/// Add milliseconds to self
	short_cds_time operator + (long long mod_milli) const;

	/// Subtract milliseconds from self
	short_cds_time operator - (long long mod_milli) const;

	/// Return underlying days, milliseconds
	string get_raw(void) const;

	/// Return time nicely formatted
	string get_cooked(void) const;

	/// Return time_t
	time_t get_epoch(void) const;

	/// Return self rounded down to nearest second
	short_cds_time demillisecond(void) const;

	// Print out value
	string pretty_print(void) const;

}; //6

ostream& operator << (ostream& o, const short_cds_time& s);

short_cds_time make_short_cds_time(const string& s);
short_cds_time make_short_cds_time(int day = 0, int millisec_day = 0);
int write_short_cds_time(byte* buffer, const short_cds_time& cds);

struct POINTER
{
	enum { SIZE = 7 };

	byte target_record_class;
	byte target_instrument_group;
	byte target_record_subclass;
	uinteger4 target_record_offset;
};

struct REC_HEAD
{
	enum { SIZE = 20 };

	enum RecordClass { RESERVED=0, MPHR, SPHR, IPR,
					   GEADR, GIADR, VEADR, VIADR, MDR };

	enum InstrumentGroup { GENERIC=0, AMSUA, ASCAT, ATOVS, AVHRR, GOME,
						   GRAS, HIRS, IASI, MHS, SEM, ADCS, SBUV, DUMMY };

	uint8 record_class;
	uint8 instrument_group;
	uint8 record_subclass;
	uint8 record_subclass_version;
	uint32_be record_size;
	short_cds_time record_start_time;
	short_cds_time record_stop_time;
};

int write_rec_head(byte* buffer, const REC_HEAD& rh);
ostream& operator << (textoutstream_type& os, const REC_HEAD& grh);

struct IPR
{
	enum { SIZE = 27 };

	//20
	REC_HEAD record_header;

	//7
	POINTER ptr;
};


//void output_text_ipr(textoutstream_type& os, IPR info, int count)
//{
//	  os << "--- IPR" << count << " Begin ---" << endl;
//	  output_text_rec_head( os, &(info.record_header) );
//	  os << "Record Class				= "
//		 << static_cast<int>(info.ptr.target_record_class) << endl
//		 << "Instrument Group			= "
//		 << static_cast<int>(info.ptr.target_instrument_group) << endl
//		 << "Record Subclass			= "
//		 << static_cast<int>(info.ptr.target_record_subclass) << endl
//		 << "Record Offset				= "
//		 << info.ptr.target_record_offset << endl;
//	  os << "---  IPR" << count << " End  ---" << endl;
//}


struct GEADR
{
	enum { SIZE = 120 };

	//20
	REC_HEAD record_header;

	//100
	char aux_data_pointer[100];

};


//void output_text_geadr(textoutstream_type& os, GEADR info, int count)
//{
//	  os << "--- GEADR" << count << " Begin ---" << endl;
//	  output_text_rec_head( os, &(info.record_header) );
//	  os << "Aux Data Pointer = " << info.aux_data_pointer << endl;
//	  os << "---  GEADR" << count << " End	---" << endl;
//}


struct VEADR
{
	enum { SIZE = 120 };

	//20
	REC_HEAD record_header;

	//100
	char aux_data_pointer[100];

};


struct VIADR_OBT2UTC
{
	enum { SIZE = 38 };
	enum { SUBCLASS = 0 };
	enum { SUBCLASS_VERSION = 2 };

	REC_HEAD record_header;
	uint16_be utc_obt_day;
	uint32_be utc_obt_millisec;
	uint16_be utc_obt_microsec;
	uint16_be zero;
	uint32_be ccu_obt_0;
	uint32_be clock_step;

	enum { DURATION = 12*60*60*1000 };
};


struct MDR_L0
{
	enum { SIZE = 26 };
	REC_HEAD record_header;
	uint8 degraded_inst_mdr;
	uint8 degraded_proc_mdr;
	uint32_be size_inst_data;
};

struct DMDR
{
	enum { SIZE = 21 };
	REC_HEAD record_header;
	uint8 reason;
};


//void output_text_veadr(textoutstream_type& os, VEADR info, int count)
//{
//	  os << "--- VEADR" << count << " Begin ---" << endl;
//	  output_text_rec_head( os, &(info.record_header) );
//	  os << "Aux Data Pointer = " << info.aux_data_pointer << endl;
//	  os << "---  VEADR" << count << " End	---" << endl;
//}


#endif
