////////////////////////////////////////////////////////////////////////////////
//
//	File: epsl0.cpp
//
//	Provides: Class definition
//
//	Purpose: Code for representing EPS-L0 products
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////
#include "common.hpp"
#include "ccsdsapidfilter.hpp"
#include "epsl0.hpp"
#include "subsat.hpp"
static const char* TMP_EXT = ".tmp";
static const int ROLLOVER_PERIOD = 16777216;
static const double THEORICAL_CLOCK_PERIOD_PS = 3906250000; //theoretical clock period in pico seconds

//--------------------------------------------------------------------------
// EPSL0::EPSL0
//--------------------------------------------------------------------------
EPSL0::EPSL0(CCSDSToL0 *_converter, CCSDS* _ccsds, string _instrument_name, string _output_file,
             string _output_dir, string _config_filename, string _subsat_dirname,
             string _osvs_filename, string _viadr_filename,
             string clock_drift, string utc_obt_time,
             string obt_utc_time, int _rollovers)
    : converter(_converter),
      ccsds(_ccsds),
      instrument_name(_instrument_name),
	  output_file(_output_file),
      output_dir(_output_dir),
	  config_filename(_config_filename),
	  subsat_dirname(_subsat_dirname),
	  osvs_filename(_osvs_filename),
	  viadr_filename(_viadr_filename),
	  initial_rollovers(_rollovers),
	rollovers(_rollovers),
	viadr_count(1),
	mdr_count(0),
	ipr_count(0),
	file_size(0),
	input_file_rollovers(0),
    previous_obt_single(0)
{
	DEBUG("EPLO:constructor CCSDS APID = " << ccsds->get_apid() << endl);

	last_obt_time.obt_coarse = 0;
	last_obt_time.obt_fine = 0;

	// this is the first packet read
	first_instrument_id = get_instrument_id();

	int tmp_apid = ccsds->get_apid();

	//account for ASCAT
	if (tmp_apid>=192 && tmp_apid<=255)
	{
		tmp_apid = 192;
	}

	// store data to enable sequence counter and time checks later
	apid_info_map[tmp_apid] = new INFO(ccsds);

	if (viadr_filename.empty())
	{
        if (converter->useMMAMdata())
        {
            //OBT UTC parameters come from MMAM files
            updateOBT_UTC_Correlation_Coefficients();			
        }
        else
        {
            //OBT UTC parameters come from the command line
            set_clock_drift(clock_drift);
            set_utc_obt_time(utc_obt_time);
            set_obt_utc_time(obt_utc_time);
        }
	}
	else
	{
		read_viadr_file(viadr_filename);
		if (!clock_drift.empty())
		{
			set_clock_drift(clock_drift);
		}
		if (!utc_obt_time.empty())
		{
			set_utc_obt_time(utc_obt_time);
		}
		if (!obt_utc_time.empty())
		{
			set_obt_utc_time(obt_utc_time);
		}
	}

	DEBUG("earsccsdstol0 constructor settings:\n"
		  << "	  utc_obt " << utc_obt_time << endl
		  << "	  obt_utc 3" << obt_utc_time << endl
		  << "	  clock_drift " << clock_drift << endl);


    //save firstL0productTime for the segmenter usage
    //TODO we can probably do this without introducing one more variable. CHECK!
    firstL0productTime.days = ccsds->get_utc_days();
    firstL0productTime.milliseconds = ccsds->get_utc_milliseconds();
    firstL0productTime.microseconds = ccsds->get_utc_microseconds();

    //save OBT of first packet of this L0 product
    initial_obt = get_obt_time();

	// define output file name
	stringstream s;
    s << instrument_name  <<  "_" << obt_to_utc(get_obt_time()).to_gen_time() << ".l0";
    output_filename = s.str();
	if (!output_file.empty())
	{
        output_filename = output_file + "." +  output_filename;
	}

    //add output directory if it was provided
    if (!output_dir.empty())
    {
        output_filename = output_dir + output_filename;
    }

    //for EARS we add the additional tmp extension while we create the file to avoid other agents
    //retrieving the file too soon
    if (converter->useMMAMdata())
    {
        output_filename += TMP_EXT;
    }

	// open file to write to
	output_stream = new ofstream_type(output_filename.c_str());

	// reserve space for MPHR and IPRs
	off_t skip_length = MPHR_body::SIZE +
		get_sphr_size() +
		get_ipr_count()*IPR::SIZE;
	output_stream->seekg(skip_length);

	file_size += skip_length;

	// create level 0 VIADR and write it to file
	write_viadr_l0();

	// create level 0 MDR and write it to file
    write_mdr_l0();
}

//--------------------------------------------------------------------------
// EPSL0::~EPSL0
//--------------------------------------------------------------------------
EPSL0::~EPSL0(void)
{
    //delete[] data;
}

//--------------------------------------------------------------------------
// EPSL0::get_ipr_count()
//--------------------------------------------------------------------------
int EPSL0::get_ipr_count(void)
{
	return 2;
}


//--------------------------------------------------------------------------
// EPSL0::get_sphr_size()
//--------------------------------------------------------------------------
int EPSL0::get_sphr_size(void)
{
	return 0;
}

//--------------------------------------------------------------------------
// EPSL0::get_output_filename()
//--------------------------------------------------------------------------
string EPSL0::get_output_filename(void)
{
    if (converter->useMMAMdata())
    {
        return product_name;
    }
    else
    {
        return output_filename;
    }

}

//--------------------------------------------------------------------------
// EPSL0::get_file_size()
//--------------------------------------------------------------------------
void EPSL0::finalizeFile(void)
{

    //this functionality will only be used for EARS, basically changes the .tmp files
    //for their real names
    if (converter->useMMAMdata())
    {
        //if temporal file exists, then change its name to the final name
        struct stat buffer;
        if ((stat (output_filename.c_str(), &buffer) == 0))
        {
            //rename the file (remove .tmp extension when we have finished)
            size_t extPos = output_filename.find(TMP_EXT);
            if (extPos != string::npos)
            {
                //add destination directory (should it have been provided)
                if (!output_dir.empty())
                {
                    product_name = output_dir + product_name + converter->getOutputSuffix();
                }

                int result = rename(output_filename.c_str(),product_name.c_str());
                if (result != 0)
                {
                    cerr << "Error renaming file " << output_filename.c_str() << endl;
                    perror(" ");
                }
            }
        }
    }
}

//--------------------------------------------------------------------------
// EPSL0::get_file_size()
//--------------------------------------------------------------------------
size_t EPSL0::get_file_size(void)
{
	return file_size;
}

//--------------------------------------------------------------------------
// EPSL0::set_clock_drift()
//--------------------------------------------------------------------------
void EPSL0::set_clock_drift(const string& _clock_drift)
{
	if (_clock_drift.empty())
	{
		// use a default clock drift time
		clock_drift = NOMINAL_CLOCK_DRIFT;
	}
	else
	{
		// use specified clock drift time
		clock_drift = strtoul(_clock_drift.c_str(), NULL, 0);
	}
}

//--------------------------------------------------------------------------
// EPSL0::set_utc_obt_time()
//--------------------------------------------------------------------------
void EPSL0::set_utc_obt_time(const string& _utc_obt_time)
{
	if (!_utc_obt_time.empty() && (*_utc_obt_time.rbegin())=='Z')
	{
		// user has given a specific time for the time correlation
		// base time

		utc_obt_time = make_short_cds_time(_utc_obt_time);
	}
	else
	{        
        short_cds_time initial_utc;
        initial_utc.day = ccsds->get_utc_days();
        initial_utc.millisec_day = ccsds->get_utc_milliseconds();

		long long utc_obt_offset;
		if (_utc_obt_time.empty())
		{
			// set the OBTUTC base time to the initial packet time
			// less 31 orbits

			utc_obt_offset = -1000*60*102*31;
		}
		else
		{
			// user has given an offset from the initial UTC time to
			// the OBTUTC base time

			utc_obt_offset = TimeConv::duration_to_int(_utc_obt_time);
		}

		DEBUG("applying utc_obt_time offset of " << utc_obt_offset
			  << "ms\n");

		DEBUG("making default utc_obt based on initial time of "
			  << initial_utc << endl);

		utc_obt_time = initial_utc + utc_obt_offset;

		DEBUG("mod time " << utc_obt_time << endl);
	}

	// Judd modification:
	//	OBT rolls over at 194 days, not 49... so I have no idea where the
	//	warning below is coming from. It's annoying, and likely wrong, so
	//	I'm removing it.
	//if (((initial_utc-utc_obt_time)/1000) > (60*60*24*49))
    //	if(0)
    //	{
    //		cout << "initial_utc " << initial_utc
    //			 << " utc_obt_time " << utc_obt_time
    //			 << " diff " << ((initial_utc-utc_obt_time)/1000)
    //			 << " num " << (60*60*24*49) << endl;
    //		cout << "WARNING: utc_obt_time is more than 49 days before the "
    //			 << "time of the first packet. The OBT rollover may not be "
    //			 << "handled correctly.\n";
    //	}
}

//--------------------------------------------------------------------------
// EPSL0::set_obt_utc_time()
//--------------------------------------------------------------------------
void EPSL0::set_obt_utc_time(const string& _obt_utc_time)
{
	// user has given a specific value for OBT UTC time.
	// utc_obt_offset is now set to offset from first OBT time
	// to specified OBT UTC time.

	DEBUG("Setting obt_utc\n");
	if (_obt_utc_time.empty())
	{
		// user has not given an OBT time for the OBTUTC base time
		// so we take the OBT time of the first packet then subtract
		// the difference between the OBTUTC base time and the UTC
		// time of the first packet

		short_cds_time initial_utc;
        initial_utc.day = ccsds->get_utc_days();
        initial_utc.millisec_day = ccsds->get_utc_milliseconds();

        OBTTime initial_obt = get_obt_time();

        double sdiff = static_cast<double>(initial_utc-utc_obt_time)/1000; // store diff in seconds
        double diff = (sdiff/clock_drift)*NOMINAL_CLOCK_DRIFT;

		// transform initial_obt into a single value
		double initial_obt_single = initial_obt.obt_coarse +
			initial_obt.obt_fine/(1<<16);

		initial_obt_single -= diff;

		// unwrap initial_obt_single to a value 0-2^23
		int cycle_correction;
		if (initial_obt_single > 0)
		{
			cycle_correction =
				static_cast<int>(initial_obt_single / (1<<24));
		}
		else
		{
			cycle_correction =
				static_cast<int>(-initial_obt_single / (1<<24)) + 1;
			rollovers = 1;
		}

		initial_obt_single += cycle_correction * (1<<24);

		obt_utc_time.obt_coarse =
			static_cast<int>(initial_obt_single) % (1<<24);
		obt_utc_time.obt_fine =
			(static_cast<int>(initial_obt_single)*(1<<16)) % (1<<16);
	}
	else
    {
		// user has specified an OBT time for the OBTUTC base time

		string::size_type pos = _obt_utc_time.find(".");
		if (pos==string::npos)
		{
			pos = _obt_utc_time.find(",");
			if (pos==string::npos)
			{
				error << "OBT time not in required format of "
					  << "<coarse>.<fine> ie. --obt-start 2000.5\n";

				obt_utc_time.obt_coarse = 0;
				obt_utc_time.obt_fine = 0;

				return;
			}
		}

		obt_utc_time.obt_coarse =
			atoi(_obt_utc_time.substr(0,pos).c_str());
		obt_utc_time.obt_fine =
			atoi(_obt_utc_time.substr(pos+1).c_str());
	}

    //cout << "Initial OBT rollovers is " << rollovers << endl;
}

//--------------------------------------------------------------------------
// EPSL0::read_viadr_file()
//--------------------------------------------------------------------------
void EPSL0::read_viadr_file(const string& viadr_file)
{
	cout << "Scanning " << viadr_file << " for EPS records... ";

	ifstream_type is(viadr_file.c_str());
	if (!is)
	{
		error << "Cannot open viadr file '" << viadr_file << "'\n";
		return;
	}

	while(!is.eof() && is.gcount()>0)
	{
		REC_HEAD grh;
		is.read(reinterpret_cast<char*>(&grh), REC_HEAD::SIZE);

//	cout << "GRH size " << grh.record_size << "... ";

//		cout << " pos " << is.tellg() << " ";
//		cout << " gcount " << is.gcount() << " ";

		if (grh.record_class==7 &&
			grh.instrument_group==0 &&
			grh.record_subclass==0 &&
			grh.record_subclass_version==2 &&
			grh.record_size==38)
		{
			VIADR_OBT2UTC record;
			is.read(reinterpret_cast<char*>(&record)+REC_HEAD::SIZE,
					VIADR_OBT2UTC::SIZE-REC_HEAD::SIZE);
			utc_obt_time.day = record.utc_obt_day;
			utc_obt_time.millisec_day = record.utc_obt_millisec;
            obt_utc_time.obt_coarse = record.ccu_obt_0 / 256;
            obt_utc_time.obt_fine = record.ccu_obt_0 % 256;
			clock_drift = record.clock_step;
			cout << "\nFound viadr-l0-obt2utc record utc0 " << utc_obt_time << " obt0 " << obt_utc_time.obt_coarse
				 << "." << obt_utc_time.obt_fine << " clock step " << clock_drift << endl;
			return;
		}

		is.seekg(grh.record_size-REC_HEAD::SIZE, ios::cur);
	}

	cout << endl;
	error << "VIADR file does not look like a L0 product\n";
}

//--------------------------------------------------------------------------
// EPSL0::epsl0_update()
//--------------------------------------------------------------------------
void EPSL0::epsl0_update(CCSDS* _ccsds, string instrument_name)
{
	ccsds = _ccsds;

	DEBUG("EPSL0:update APID = " << ccsds->get_apid() << endl);

	int tmp_apid = ccsds->get_apid();
	UTCTime tmp_time;
	tmp_time.days = ccsds->get_utc_days();
	tmp_time.milliseconds = ccsds->get_utc_milliseconds();
	tmp_time.microseconds = ccsds->get_utc_microseconds();

	//account for ASCAT
	if (tmp_apid>=192 && tmp_apid<=255)
	{
		tmp_apid = 192;
	}

	APIDMAP::iterator iter=apid_info_map.find(tmp_apid);
	if (iter!=apid_info_map.end())
	{
		//code to handle counter rollovers
		int expected_seq_ctr = (*iter).second->seq_ctr_end + 1;
		if (expected_seq_ctr > 16383)
		{
			expected_seq_ctr = expected_seq_ctr - 16384;
		}

        //if an unexpected sequence counter is detected, report it
		if ( expected_seq_ctr != ccsds->get_sequence_count() )
		{
			cout << "Warning: Sequence counter error: "
				 << "Expected (" << expected_seq_ctr
				 << "), Detected(" << ccsds->get_sequence_count()
				 <<"), APID(" << ccsds->get_apid()
				 <<"), UTC(" << static_cast<unsigned int>(ccsds->get_utc_days()) << ", "
				 << static_cast<unsigned int>(ccsds->get_utc_milliseconds()) << ", "
				 << static_cast<unsigned int>(ccsds->get_utc_microseconds()) << ")\n";
		}
		else if ( (*iter).second->time_end > tmp_time )
		{

            //This messages will typically appear in IASI packets APID(160), that are not timestamped
            if (ccsds->get_apid() != 160)
            {
                cout << "Warning: Time overlap detected: "
                     <<"APID(" << ccsds->get_apid()
                     << "), Sequence Counter(" << ccsds->get_sequence_count()
                     <<"), UTC-last(" << static_cast<unsigned int>((*iter).second->time_end.get_utc_days()) << ", "
                     << static_cast<unsigned int>((*iter).second->time_end.get_utc_milliseconds()) << ", "
                     << static_cast<unsigned int>((*iter).second->time_end.get_utc_microseconds())
                     <<"), UTC-current(" << static_cast<unsigned int>(ccsds->get_utc_days()) << ", "
                     << static_cast<unsigned int>(ccsds->get_utc_milliseconds()) << ", "
                     << static_cast<unsigned int>(ccsds->get_utc_microseconds()) << ")\n";
            }

			if ( expected_seq_ctr == ccsds->get_sequence_count() )
			{
				//update                
				(*iter).second->update_seq_count_only(ccsds);
			}
		}

		//update
		(*iter).second->update(ccsds);

		// create level 0 MDR and write it to file
		write_mdr_l0();
	}
	else
	{
		//cout << "*** Create NEW *** " << ccsds->get_apid() << endl;
		//create new
		apid_info_map[ccsds->get_apid()] = new INFO(ccsds);

		// create level 0 MDR and write it to file
		write_mdr_l0();
	}

}

//--------------------------------------------------------------------------
// EPSL0::epsl0_complete()
//--------------------------------------------------------------------------
void EPSL0::epsl0_complete(void)
{

    //to make sure we use the latest MMAM information we update VIADR
    update_viadr_l0();

	//off_t filesize = os->tellp();
	output_stream->seekg(0);
	write_mphr();
	while(write_iprs());

	// reset the write pointer so the SingleFileWriter
	// report is accurate
	//os->seekg(filesize);

	spinner->end();

    //close and clean things properly
    finalizeFile();
}


//--------------------------------------------------------------------------
// EPSL0::get_obt_time()
//--------------------------------------------------------------------------

// Judd modification (addition):
// This function does the appropriate processing to get the real OBT time
//	from the current CCSDS packet.
OBTTime EPSL0::get_obt_time(void)
{
	// Judd modification:
	// OBT time is completely different for GRAS, but it is used for the
	//	regular CCSDS time stamp that is in UTC format, so we need to get
	//	values for the OBT time.
	unsigned int apid = ccsds->get_apid();
	if (apid==APID_GRAS_MEASUREMENT_1 || apid==APID_GRAS_MEASUREMENT_2)
	{	// GRAS, so just use it's calc the fine and coarse fields:
		OBTTime obt_time_raw = *(ccsds->get_ancillary_data());
		uint64_be coarse_raw = (uint64_be)obt_time_raw.get_obt_coarse();
		uint64_be fine_raw	  = (uint64_be)obt_time_raw.get_obt_fine();
		uint64_be veryfine = (coarse_raw << 8) ^ (fine_raw >> 8);
		uint32_be coarse = (uint32_be)(veryfine / 65536LL);
		uint16_be fine = (uint16_be)(veryfine % 65536LL);

		OBTTime obt_time = make_obt_time( coarse, fine );

		//std::cout << "GRAS OBT Time: " << obt_time << std::endl;

		return obt_time;
	}
	else if (apid==APID_MHS)
	{	// MHS, which is just different in that the first byte is junk:
		OBTTime obt_time_raw = *(ccsds->get_ancillary_data());

		// Wipe out that first byte:
		uint32_be coarse = obt_time_raw.get_obt_coarse();
		coarse = coarse & 0x00FFFFFF;

		OBTTime obt_time = make_obt_time( coarse, obt_time_raw.get_obt_fine() );
		return obt_time;
	}
	else
	{
		// Not GRAS, this will work (except for MHS):
		OBTTime obt_time = *(ccsds->get_ancillary_data());
		return obt_time;
	}
}

//--------------------------------------------------------------------------
// EPSL0::write_viadr_l0()
//--------------------------------------------------------------------------
void EPSL0::write_viadr_l0(void)
{
	// save the rollovers time since make_mdr will call the
	// same function & we dont want to touch rollovers
	// twice for the same packet
	int rollovers_keep = rollovers;
	// Judd modification:
	// OBT time is completely different for GRAS, but it is used for the
	//	regular CCSDS time stamp that is in UTC format, so we need to get
	//	values for the OBT time.
	//OBTTime obt_time = *(ccsds->get_ancillary_data());
	//start_time = obt_to_utc(obt_time);
    OBTTime obt_time = initial_obt;

	if (((obt_time-obt_utc_time)/65536) > 8000000)
	{
		cout << "Decrementing rollovers since first OBT time (" << obt_time
			 << ") wraps over from obt_utc value (" << obt_utc_time << ")\n";
		initial_rollovers--;
		rollovers_keep--;
		rollovers--;
	}

    initial_grh = obt_to_utc(obt_time);
    initial_obt = obt_time;
	rollovers = rollovers_keep;

	viadr_start = initial_grh;
	viadr_stop = viadr_start + VIADR_OBT2UTC::DURATION;

	VIADR_OBT2UTC* viadr = new VIADR_OBT2UTC();
	viadr->record_header.record_class = REC_HEAD::VIADR;
	viadr->record_header.instrument_group = REC_HEAD::GENERIC;
	viadr->record_header.record_subclass = VIADR_OBT2UTC::SUBCLASS;
	viadr->record_header.record_subclass_version =
		VIADR_OBT2UTC::SUBCLASS_VERSION;
	viadr->record_header.record_size = VIADR_OBT2UTC::SIZE;
	viadr->record_header.record_start_time = viadr_start;
	viadr->record_header.record_stop_time = viadr_stop;

	viadr->utc_obt_day = utc_obt_time.day;
	viadr->utc_obt_millisec = utc_obt_time.millisec_day;
	viadr->utc_obt_microsec = 0;

	viadr->zero = 0;
	// cout << "writing obt 0 coarse " << obt_utc_time.obt_coarse <<	 " obt 0 fine " << obt_utc_time.obt_fine << endl;
	viadr->ccu_obt_0 = (obt_utc_time.obt_coarse * 256) + (obt_utc_time.obt_fine);
	// cout << "ccu_obt_0 " << viadr->ccu_obt_0 << endl;
	viadr->clock_step = clock_drift;

	//record = reinterpret_cast<byte*>(viadr);
	int length = sizeof(VIADR_OBT2UTC);
	file_size += length;

	output_stream->write(reinterpret_cast<char*>(viadr), length);
	spinner->tick();
	//delete[] record;
}

//--------------------------------------------------------------------------
// EPSL0::update_viadr_l0()
//--------------------------------------------------------------------------
void EPSL0::update_viadr_l0()
{
    //if we use MMAMdata we want to update the VIADR information from the latest
    //MMAM that we could get (in HKTM-APID 6)
    if (converter->useMMAMdata())
    {
        //save the current stream position
        streampos_type myCurrentPos = output_stream->tellp();

        // reserve space for MPHR and IPRs
        off_t skip_length = MPHR_body::SIZE + get_sphr_size() + get_ipr_count()*IPR::SIZE;
        output_stream->seekg(skip_length,ios::beg);

        //OBT UTC parameters come from MMAM files
        updateOBT_UTC_Correlation_Coefficients();

        //remove size of the last written VIADR (not to compute it twice)
        file_size -= sizeof(VIADR_OBT2UTC);

        // create level 0 VIADR and write it to file
        write_viadr_l0();

        //return the stream to its original position
        output_stream->seekg(myCurrentPos, ios::beg);
    }
}

//--------------------------------------------------------------------------
// EPSL0::updateOBT_UTC_Correlation_Coefficients()
//--------------------------------------------------------------------------
void EPSL0::updateOBT_UTC_Correlation_Coefficients()
{
    if (converter->isMMAMdataAvailable())
    {
        set_clock_drift(converter->getClockDrift());
        set_utc_obt_time(converter->getUTC0());
        set_obt_utc_time(converter->getOBT0());
    }
}

//--------------------------------------------------------------------------
// EPSL0::write_mdr_l0()
//--------------------------------------------------------------------------
void EPSL0::write_mdr_l0(void)
{

	// Judd modification:
	// Use new method to get the correct OBT time:
	//OBTTime obt_time = *(ccsds->get_ancillary_data());
	OBTTime obt_time = get_obt_time();

	if (obt_time.obt_coarse==0 && obt_time.obt_fine==0 and ccsds->get_apid()==APID_IASI_VPx)
	{
		//cout << "apid " << ccsds->get_apid() << " obt_time " << obt_time << endl;

		// its a 'special' IASI Spectral Verification packet with OBT time of 0/0
		// so we try to do something sensible with it
		// (this is the same algorithm the PGF uses)
		obt_time = last_obt_time;
	}
	else
	{
		last_obt_time = obt_time;
	}

	short_cds_time grh_time = obt_to_utc(obt_time);
//	cout << obt_time << "   " << grh_time << endl;


	unsigned int length = ccsds->get_length() + MDR_L0::SIZE;
	byte* record = new byte[length];
	MDR_L0* l0 = reinterpret_cast<MDR_L0*>(record);
	l0->record_header.record_class = REC_HEAD::MDR;
	l0->record_header.instrument_group = REC_HEAD::GENERIC;
	if (ccsds->get_apid() > 6)
	{
		l0->record_header.record_subclass = 0;
	}
	else
	{
		l0->record_header.record_subclass = 4;
	}
	l0->record_header.record_subclass_version = 1;
	l0->record_header.record_size = length;
	l0->record_header.record_start_time = grh_time;
	l0->record_header.record_stop_time = grh_time;
	l0->degraded_inst_mdr = 0;
	l0->degraded_proc_mdr = 0;
	l0->size_inst_data = ccsds->get_length();
	memcpy(record+MDR_L0::SIZE, ccsds->get_data(),
		   ccsds->get_length());
	file_size += length;
	mdr_count++;
	final_grh = grh_time;
	final_obt = obt_time;

	output_stream->write(reinterpret_cast<char*>(record), length);
	// cout << "writing packet\n";
	spinner->tick();
	delete[] record;

}


//--------------------------------------------------------------------------
// EPSL0::write_mphr()
//--------------------------------------------------------------------------
void EPSL0::write_mphr(void)
{
	REC_HEAD rec_head;
	MPHR_body mphr;
	byte* record = reinterpret_cast<byte*>(&mphr);
	unsigned int length = MPHR_body::SIZE;

    //check whether we use MMAM data, or the provided state vectors file
	OSVS::OSV myOSV_start;
	OSVS::OSV myOSV_end;
    Subsat::Pos start_pos;
    Subsat::Pos end_pos;

    if (converter->useMMAMdata())
    {
        if (converter->isMMAMdataAvailable())
        {
            //check for the validity of the MMAM data
            if (!converter->checkTimeValidity(initial_grh, final_grh))
            {
                cerr << "State vector information not valid for the product's time range." << endl;
            }

			//Orbit state vector
			myOSV_start = converter->getStateVectorAtANX(initial_grh);
			myOSV_end = converter->getStateVectorAtANX(final_grh);

			//Subsatellite info
			OSVS::OSV osv_start = converter->getStateVector(initial_grh);
			start_pos = compute_geolocation(osv_start.x, osv_start.y, osv_start.z);
			OSVS::OSV osv_end = converter->getStateVector(final_grh);
			end_pos = compute_geolocation(osv_end.x, osv_end.y,	osv_end.z);
        }
        else
        {
            cerr << "There is no MMAM data available to create L0 product. L0 product file will not be created." << endl;
            //delete tmp product file
            remove(output_filename.c_str());

            converter->setL0fileAborted(true);
            return;
        }
    }
    else
    {
        //Orbit state vector
        OSVS osvs(osvs_filename);
		myOSV_start = osvs.get_osv(initial_grh);
		myOSV_end = osvs.get_osv(final_grh);

		//Subsatellite info
		Subsat subsat(subsat_dirname);
		start_pos = subsat.get_pos(initial_grh);
		end_pos = subsat.get_pos(final_grh);
    }

    product_name = fill_mphr(rec_head,
			  mphr,
			  config_filename,
			  first_instrument_id,
			  initial_grh,
			  final_grh,
			  0,
			  get_ipr_count(),
			  0,
			  0,
			  0,
			  viadr_count,
			  mdr_count,
			  file_size,
              start_pos,
              end_pos,
			  myOSV_start,
			  myOSV_end);

	record = new byte[sizeof(mphr)+REC_HEAD::SIZE];
	write_rec_head(record,rec_head);
	memcpy(record+REC_HEAD::SIZE,&mphr,sizeof(mphr));
//	output_text_rec_head(cout,&rec_head);
	//file_size += length;
	//return true;

	output_stream->write(reinterpret_cast<char*>(record), length);
	spinner->tick();
	delete[] record;
}


//--------------------------------------------------------------------------
// EPSL0::compute_geolocation()
//--------------------------------------------------------------------------
Subsat::Pos EPSL0::compute_geolocation(double x, double y, double z)
{
	const double PI =3.141592653589793238463;
	Subsat::Pos myPos;
	myPos.lon = atan2(y, x) * (180/PI);
	// by convention longitude between 0 and 360 deg.
	if (myPos.lon < 0) myPos.lon += 360.0;
	myPos.lat = atan2(z, sqrt(x*x + y*y)) * (180/PI);
	return myPos;
}

//--------------------------------------------------------------------------
// EPSL0::write_iprs()
//--------------------------------------------------------------------------
bool EPSL0::write_iprs(void)
{
	if (ipr_count==2)
	{
		return false;
	}

	IPR* ipr = new IPR;
	ipr->record_header.record_class = REC_HEAD::IPR;
	ipr->record_header.instrument_group = REC_HEAD::GENERIC;
	ipr->record_header.record_subclass_version = 1;
	ipr->record_header.record_size = IPR::SIZE;

	if (ipr_count==0)
	{
		// ptr to VIADR
		ipr->record_header.record_subclass = REC_HEAD::VIADR;
		ipr->record_header.record_start_time = initial_grh;
		ipr->record_header.record_stop_time = final_grh;
		ipr->ptr.target_record_class = REC_HEAD::VIADR;
		ipr->ptr.target_instrument_group = 0;
		ipr->ptr.target_record_subclass = VIADR_OBT2UTC::SUBCLASS;
		ipr->ptr.target_record_offset = MPHR_body::SIZE +
			IPR::SIZE*2;
	}
	else if (ipr_count==1)
	{
		// ptr to MDRs
		ipr->record_header.record_subclass = REC_HEAD::MDR;
		ipr->record_header.record_start_time = initial_grh;
		ipr->record_header.record_stop_time = final_grh;
		ipr->ptr.target_record_class = REC_HEAD::MDR;
		ipr->ptr.target_instrument_group = 0;
		ipr->ptr.target_record_subclass = VIADR_OBT2UTC::SUBCLASS;
		ipr->ptr.target_record_offset =
			MPHR_body::SIZE + IPR::SIZE*2 + VIADR_OBT2UTC::SIZE;
	}

	ipr_count++;
	byte* record = reinterpret_cast<byte*>(ipr);
	unsigned int length = IPR::SIZE;
	//file_size += length;
	output_stream->write(reinterpret_cast<char*>(record), length);
	spinner->tick();
	delete[] record;
	return true;
}

//--------------------------------------------------------------------------
// EPSL0::obt_to_utc()
//--------------------------------------------------------------------------
short_cds_time EPSL0::obt_to_utc(const OBTTime& obt_time)
{
    // convert obt_time to seconds
    double obt_single =	static_cast<double>(obt_time.obt_coarse) +
                        static_cast<double>(obt_time.obt_fine) / (1<<16);

    // convert ccu-obt-0 to seconds
    double obt_utc_single =	static_cast<double>(obt_utc_time.obt_coarse) +
                            static_cast<double>(obt_utc_time.obt_fine) / (1<<16);

	// test if an OBT rollover has occurred in the middle of the product
	if (static_cast<int>(previous_obt_single)>(MAX_OBT_COARSE-100000) &&
		obt_single<1000000 &&
		obt_single < previous_obt_single &&
		input_file_rollovers==0)
	{
		cout << "OBT rollover found in the input\n" << endl;
		rollovers++;
		input_file_rollovers++;
	}

    // save current obt time to check for rollover in the next iteration
	previous_obt_single = obt_single;

    // check Multi-Mission Administrative Message User Guide (section 5.3 OBT-UTC Correlation)
    // for an in-depth explanation of the conversion (this formula works regardless of rollovers)
    double myDif = fmod((obt_single - obt_utc_single + 1.5 * ROLLOVER_PERIOD), ROLLOVER_PERIOD) - 0.5 * ROLLOVER_PERIOD;

    // correct the ellapsed time with the clock_drift
    double ellapsed_time = ((clock_drift/THEORICAL_CLOCK_PERIOD_PS) * myDif);

    // add the ellapsed time (converted to ms) to the UTC0 time
    short_cds_time utc_time =  utc_obt_time + ellapsed_time * 1000;

    return utc_time;
}

//--------------------------------------------------------------------------
// EPSL0::get_instrument_id()
//--------------------------------------------------------------------------
string EPSL0::get_instrument_id(void)
{
	string instrument_id = CCSDSAPIDFilter::apid_to_string(ccsds->get_apid());

	// Judd modification to fix the issue where the GRAS-NAV stuff comes in
	//	before the first ADMIN or SL-PACKET APIDs for the HKTM files:
	if( instrument_id == "GRAS-NAV" )
	{
		instrument_id = "HKTM";
	}

	if (instrument_id.length() > 4)
	{
		instrument_id = instrument_id.substr(0,4);
	}
	while(instrument_id.length() < 4)
	{
		instrument_id += "x";
	}

	if (instrument_id=="AMSU")
	{
		instrument_id = "AMSA";
	}
	else if (instrument_id=="A-DC")
	{
		instrument_id = "ADCS";
	}
	else if (instrument_id=="SATE" || instrument_id=="SL-P" ||
			 instrument_id=="ADMx")
	{
		instrument_id = "HKTM";
	}

	return instrument_id;
}

//--------------------------------------------------------------------------
// EPSL0::output_obtutc_summary()
//--------------------------------------------------------------------------
void EPSL0::output_obtutc_summary(ostream& os, const string& indent)
{
	os << indent << "UTC0 " << utc_obt_time << endl
	   << indent << "OBT0 " << obt_utc_time << endl
	   << indent << "Clock drift " << clock_drift << endl
	   << endl;
}

//--------------------------------------------------------------------------
// EPSL0::output_text_summary()
//--------------------------------------------------------------------------
void EPSL0::output_text_summary(ostream& os, const string& indent)
{
	os << indent << "MDR count " << mdr_count << endl
	   << indent << "Rollovers initial " << initial_rollovers << " final " << rollovers << endl
	   << indent << "Initial OBT " << initial_obt << " GRH " << initial_grh << endl
	   << indent << "Final	 OBT " << final_obt << " GRH " << final_grh << endl
	   << endl;
}

