////////////////////////////////////////////////////////////////////////////////
//
//	File: epsl0.hpp
//
//	Provides: Class definition
//
//	Purpose: Code for representing EPS-L0 products
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////


#ifndef epsl0_defined
#define epsl0_defined


#include "ccsdsproducer.hpp"
#include "mphr.hpp"
#include "ccsdstol0.hpp"

const unsigned int NOMINAL_CLOCK_DRIFT = 3906249305UL;

class EPSL0
{
public:

    EPSL0(CCSDSToL0 *_converter, CCSDS* _ccsds, string _instrument_name,
          string _output_file, string _output_dir, string _config_filename,
          string _subsat_dirname, string _osvs_filename,
          string _viadr_filename, string clock_drift,
          string utc_obt_time, string obt_utc_time, int _rollovers);

	~EPSL0(void);

	void epsl0_update(CCSDS* input_ccsds, string instrument_name);
	void epsl0_complete(void);
	string get_output_filename(void);
	size_t get_file_size(void);
    void finalizeFile(void);

    void updateOBT_UTC_Correlation_Coefficients();
    void update_viadr_l0();

	void output_obtutc_summary(ostream& = cout, const string& indent = "");
	void output_text_summary(ostream& = cout, const string& indent = "");

private:

	void write_mphr(void);
	bool write_iprs(void);
	void write_viadr_l0(void);
	void write_mdr_l0(void);

	int get_ipr_count(void);
	int get_sphr_size(void);

	void set_clock_drift(const string& _clock_drift);
	void set_utc_obt_time(const string& _utc_obt_time);
	void set_obt_utc_time(const string& _obt_utc_time);
	void read_viadr_file(const string& viadr_file);

	short_cds_time obt_to_utc(const OBTTime& obt_time);
	string get_instrument_id(void);
	OBTTime get_obt_time(void);

	Subsat::Pos compute_geolocation(double x, double y, double z);

	//MDR_L0 mdr;

	string first_instrument_id;
	string instrument_id;
	string output_filename;
	ostream_type* output_stream;

	struct INFO
	{
		unsigned int seq_ctr_start;
		unsigned int seq_ctr_end;
		UTCTime time_start;
		UTCTime time_end;

		INFO(CCSDS* ccsds)
			{
				seq_ctr_start = ccsds->get_sequence_count();
				seq_ctr_end = seq_ctr_start;
				time_start.days = ccsds->get_utc_days();
				time_start.milliseconds = ccsds->get_utc_milliseconds();
				time_start.microseconds = ccsds->get_utc_microseconds();
				time_end = time_start;
			}

		void update(CCSDS* ccsds)
			{
				seq_ctr_end = ccsds->get_sequence_count();
				time_end.days = ccsds->get_utc_days();
				time_end.milliseconds = ccsds->get_utc_milliseconds();
				time_end.microseconds = ccsds->get_utc_microseconds();
			}

		void update_seq_count_only(CCSDS* ccsds)
			{
				seq_ctr_end = ccsds->get_sequence_count();
			}

	};

	typedef map<int, INFO*> APIDMAP;
	APIDMAP apid_info_map;

	unsigned int mdr_count;
	unsigned int viadr_count;
	unsigned int ipr_count;
	size_t file_size;

	string output_file;
    string output_dir;
	string config_filename;
	string subsat_dirname;
	string osvs_filename;
	string viadr_filename;

	short_cds_time initial_grh;
	short_cds_time final_grh;

	OBTTime initial_obt;
	OBTTime final_obt;

	short_cds_time viadr_start;
	short_cds_time viadr_stop;

	CCSDS* ccsds;

	short_cds_time utc_obt_time;
	OBTTime obt_utc_time;
	unsigned int clock_drift;
	int rollovers; // total rollovers count, including initial rollovers
	int initial_rollovers;
	// specified by the user
	int input_file_rollovers; // rollovers in input file

	/// used to detect rollovers in the middle of the inputs
	double previous_obt_single;

	// we record the last good OBT time value for use when converting IASI
	// data so when we encounter a spectral verification packet w/ OBT
	// of 0/0 we can use the last good time instead
	OBTTime last_obt_time;

    //save pointer to mmam
    CCSDSToL0* converter;

    string instrument_name;

    //UTC time of the first isp of this L0 product
    UTCTime firstL0productTime;

    string product_name;

    CCSDS* firstccsds;

public:

    UTCTime getFirstTimestamp() { return firstL0productTime; }

};


#endif
