#include "epsl0segmenter.hpp"



//--------------------------------------------------------------------------
// EPSL0Segmenter::EPSL0Segmenter
//--------------------------------------------------------------------------
EPSL0Segmenter::EPSL0Segmenter(CCSDSToL0 *aConverter, unsigned int aSegmentSize)
    : m_Converter(aConverter),
      m_segmentSize(aSegmentSize)
{
}

//--------------------------------------------------------------------------
// EPSL0Segmenter::~EPSL0Segmenter
//--------------------------------------------------------------------------
EPSL0Segmenter::~EPSL0Segmenter()
{
}

//--------------------------------------------------------------------------
// EPSL0Segmenter::addISP()
//--------------------------------------------------------------------------
void EPSL0Segmenter::addISP(CCSDS *ccsds)
{
    //Group related apids in instruments for L0 products generation
    //The mapping is provided in a configuration file provided at input
    string myInstrumentName = m_Converter->getCCSDSconfig().apid_to_instrument(ccsds->get_apid());
    string output_filename =  m_Converter->getOutputFileName();
    string output_dirname =  m_Converter->getOutputDirName();
    string config_filename =  m_Converter->getConfigFileName();
    string subsat_dirname =  m_Converter->getSubSatDirName();
    string osvs_filename =  m_Converter->getOSVsFileName() ;
    string viadr_filename =  m_Converter->getVIADRFileName();


    string clock_drift =  m_Converter->getClockDrift();
    string utc_obt_time =  m_Converter->getUTC0();
    string obt_utc_time =  m_Converter->getOBT0();
    int rollovers = m_Converter->getRollovers() ;


//    cout << "clock_drift: " << clock_drift << endl;
//    cout << "utc_obt_time: " << utc_obt_time << endl;
//    cout << "obt_utc_time: " << obt_utc_time << endl;
//    cout << "rollovers: " << rollovers << endl;

    //first we locate position ourselves in the right instrument
    EPSL0_SEGMENTED_MAP::iterator iter=m_segmentedEpsl0_map.find(myInstrumentName);
    if (iter == m_segmentedEpsl0_map.end())
    {
        //we create a new entry in the map (new instrument found)
        vector<EPSL0*> mySegments;

        //create new EPSL0 product for the new instrument in the map
        mySegments.push_back(new EPSL0(m_Converter, ccsds, myInstrumentName, output_filename, output_dirname, config_filename,
                                           subsat_dirname, osvs_filename, viadr_filename,
                                           clock_drift, utc_obt_time, obt_utc_time, rollovers));

        //add new instrument segment vector to the map
        m_segmentedEpsl0_map[myInstrumentName] = mySegments;
    }
    else
    {
        //add ISP to an already existing instrument segmented map


        //we have to check whether we have to create a new entry in the vector or update an existing one
        //NOTE: for the time being we use the time of the ISPs but we can convert using the methods in EPSL0.cpp if necessary
        short_cds_time myCurrentISPtimeStamp;
        myCurrentISPtimeStamp.day = ccsds->get_utc_days();
        myCurrentISPtimeStamp.millisec_day = ccsds->get_utc_milliseconds();

        //get a reference to our current segment's EPSLO object
        vector<EPSL0*>& mySegments = ((*iter).second);
        EPSL0* myCurrentSegment = mySegments.back();

        //get the first timestamp of that EPSL0 object (we will use it to compare the current ISP timestamp
        //with the timestamp of the start of the current segment
        short_cds_time mySegmentFirstTimeStamp;
        mySegmentFirstTimeStamp.day = myCurrentSegment->getFirstTimestamp().get_utc_days();
        mySegmentFirstTimeStamp.millisec_day = myCurrentSegment->getFirstTimestamp().get_utc_milliseconds();

        //time difference between the current ISP and the first ISP of the current segment
        int myTimeDif = myCurrentISPtimeStamp.get_milliseconds() - mySegmentFirstTimeStamp.get_milliseconds();

        //for IASI some ISPs are NOT timestamped
        if (myCurrentISPtimeStamp.day == 0 && myCurrentISPtimeStamp.day == 0)
        {
            myCurrentSegment->epsl0_update(ccsds, myInstrumentName);
            return;
        }

        //see whether the timestamp for the new ISP is beyond the segment size or not
        if (m_segmentSize > 0 &&
            myTimeDif > 0 && // --> sometimes (found in IASI) some ISPs are not in order
            myTimeDif  > m_segmentSize*1000)
        {

//            cout << "MY current ISP timestamp:" << myCurrentISPtimeStamp.to_gen_time();
//            cout << "   My Start of segment timestamp:  " << mySegmentFirstTimeStamp.to_gen_time();
//            cout << "   My Time Diff:  " << myTimeDif << endl;

            //create a new segment in the current instrument as the ISP is beyond the current segment length
            mySegments.push_back(new EPSL0(m_Converter, ccsds, myInstrumentName, output_filename, output_dirname, config_filename,
                                           subsat_dirname, osvs_filename, viadr_filename,
                                           clock_drift, utc_obt_time, obt_utc_time, rollovers));
        }
        else
        {
            //update current segment
             myCurrentSegment->epsl0_update(ccsds, myInstrumentName);
        }
    }
}

//--------------------------------------------------------------------------
// EPSL0Segmenter::completeL0()
//--------------------------------------------------------------------------
void EPSL0Segmenter::completeL0()
{
    //once all ISPs have been added into EPSL0 products, complete them
    //by adding the MPHR, IPRS, etc.

    //sweep all instruments
    for (EPSL0_SEGMENTED_MAP::iterator iter=m_segmentedEpsl0_map.begin();
         iter!=m_segmentedEpsl0_map.end(); iter++)
    {
        vector<EPSL0*> mySegment = (*iter).second;

        //'complete' each segment of the instrument
        for (int i = 0; i < mySegment.size(); i++)
        {
            mySegment[i]->epsl0_complete();
        }
    }
}


//--------------------------------------------------------------------------
// EPSL0Segmenter::propagateOBTcoefficients()
//--------------------------------------------------------------------------
void EPSL0Segmenter::propagateOBTcoefficients()
{
    //the new coefficients from the MMAM are propagated to all instruments...
    for (EPSL0_SEGMENTED_MAP::iterator iter=m_segmentedEpsl0_map.begin();
         iter!=m_segmentedEpsl0_map.end(); iter++)
    {
        vector<EPSL0*> mySegments = (*iter).second;

        //.. and all segments within the instrument
        for (int i = 0; i < mySegments.size(); i++)
        {
            EPSL0* mySegment = mySegments[i];
            mySegment->updateOBT_UTC_Correlation_Coefficients();
        }
    }
}

//--------------------------------------------------------------------------
// EPSL0Segmenter::output_text_summary()
//--------------------------------------------------------------------------

void EPSL0Segmenter::output_text_summary(ostream& os)
{
    for (EPSL0_SEGMENTED_MAP::iterator iter=m_segmentedEpsl0_map.begin();
         iter!=m_segmentedEpsl0_map.end(); iter++)
    {
        vector<EPSL0*> mySegment = (*iter).second;

        for (int i = 0; i < mySegment.size(); i++)
        {
            if (i==0)
            {
                mySegment[i]->output_obtutc_summary(os, "	   ");
            }

            os << "	   Output file " << mySegment[i]->get_output_filename()
               << " (" << mySegment[i]->get_file_size()
               << " bytes)" << endl;
            mySegment[i]->output_text_summary(os,"		");
        }
    }
}
