////////////////////////////////////////////////////////////////////////////////
//
//	File: hirs_to_ccsds.cpp
//
//	Provides: hirs_to_ccsds executable
//
//	Purpose: Extract a stream of MetOp source packets from a NOAA L1b HIRS
//			 product
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "noaaproductreader.hpp"
#include "readfromstdin.hpp"
#include "multifilewriter.hpp"
#include "writetostdout.hpp"
#include "singlefilewriter.hpp"
#include "hirsreader.hpp"
#include "ccsdsextractor.hpp"
#include "imagewriter.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "hirs_to_ccsds";
const string CONFIG_DIR_VAR = "METOPIZER_HOME";
const string DEFAULT_CONFIG_FILE = "hirs.config";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	string in_file;
	string ccsds_file;
	string image_file;
	Image::Format image_format = Image::IMAGE_TIFF;
	string config_file;

	bool verbose = true;

	enum { ARGV_NONE, ARGV_IN, ARGV_CCSDS, ARGV_CONFIG, ARGV_IMAGE }
	state = ARGV_IN;

	for(int x=1; x<argc; x++)
	{
		if (!strcmp(argv[x], "--version") || !strcmp(argv[x], "-v"))
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__
				 << endl;
			return 0;
		}
		else if (!strcmp(argv[x], "--help") || !strcmp(argv[x], "-h"))
		{
			cout << endl << APPNAME << " " << PACKAGE_VERSION << " built on "
				 << __DATE__
				 << "\nExtract a stream of MetOp source packets from a NOAA "
				 << "L1b HIRS product.\n\n"
				 << "usage:\n" << APPNAME
				 << " [--in|-i <filename>]"
				 << " [--out|-o <filename>]"
				 << " [--config|-c <filename>]"
				 << " [--image <filename base>]"
#ifndef _NO_JPEG
				 << " [--jpeg]"
#endif
				 << " [-z]"
				 << " [--help|-h]"
				 << " [--version|-v]\n\n";
			return 0;
		}
		else if (!strcmp(argv[x], "--in") || !strcmp(argv[x], "-i"))
		{
			state = ARGV_IN;
		}
		else if (!strcmp(argv[x], "--out") || !strcmp(argv[x], "-o"))
		{
			state = ARGV_CCSDS;
		}
		else if (!strcmp(argv[x], "--config") || !strcmp(argv[x], "-c"))
		{
			state = ARGV_CONFIG;
		}
		else if (!strcmp(argv[x], "--image"))
		{
			state = ARGV_IMAGE;
		}
#ifndef _NO_JPEG
		else if (!strcmp(argv[x], "--jpeg"))
		{
			image_format = Image::IMAGE_JPEG;
		}
#endif
		else if (!strcmp(argv[x], "--quiet"))
		{
			verbose = false;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else if (!strcmp(argv[x], "--debug"))
		{
			_debug = &cout;
		}
		else
		{
			if (!strncmp(argv[x],"--",2))
			{
				error << "unrecognised parameter - '" << argv[x] << "'\n";
				return -1;
			}

			switch(state)
			{
			case ARGV_NONE:
				cerr << "Parameter not recognised '" << argv[x] << endl;
				return -1;

			case ARGV_IN:
				if (!in_file.empty())
				{
					error << "only one input file can be given\n";
					return -1;
				}
				else
				{
					in_file = argv[x];
				}
				break;

			case ARGV_CCSDS:
				if (!ccsds_file.empty())
				{
					cerr << "Only one output filename can be specified.\n";
					return -1;
				}
				else
				{
					ccsds_file = argv[x];
				}
				break;

			case ARGV_IMAGE:
				if (!image_file.empty())
				{
					cerr << "Only one image filename can be specified.\n";
					return -1;
				}
				else
				{
					image_file = argv[x];
				}
				break;

			case ARGV_CONFIG:
				if (!config_file.empty())
				{
					cerr << "Only one config file can be specified.\n";
					return -1;
				}
				else
				{
					config_file = argv[x];
				}
				break;
			}
		}
	}

	// if no config file specified check for an implicit one
	if (config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_file = string(config_dir_var) + "/" + DEFAULT_CONFIG_FILE;
		}
		else //if (getenv("HOME"))
		{
			// use the default filename
			config_file = string("DEFAULT_METOPIZER_HOME") + "/" + DEFAULT_CONFIG_FILE;
		}
	}

	// fix streams
	if (image_file.empty() && ccsds_file.empty())
	{
		_info = &cerr;
		_warning = &cerr;
		verbose = false;
		Spinner::restream(&onull);
	}

	FileProducer* file_source;
	if (in_file.empty())
	{
		file_source = new ReadFromStdin();
	}
	else
	{
		file_source = new FileReader(in_file);
	}

	NOAAProductReader noaa_reader(*file_source);
	HIRSReader hirs_reader(noaa_reader, config_file);

	if (!image_file.empty())
	{
		ImageWriter writer(hirs_reader, image_file, image_format);

		if (verbose && !gerror)
		{
			file_source->output_text_summary();
			noaa_reader.output_text_summary();
			hirs_reader.output_text_summary();
			writer.output_text_summary();
		}
	}
	else
	{
		CCSDSExtractor extractor(hirs_reader);
		if (ccsds_file.empty())
		{
			WriteToStdout x(&extractor);
		}
		else
		{
			SingleFileWriter writer(&extractor, ccsds_file);

			if (verbose && !gerror)
			{
				file_source->output_text_summary();
				noaa_reader.output_text_summary();
				hirs_reader.output_text_summary();
				extractor.output_text_summary();
				writer.output_text_summary();
			}
		}
	}

	delete file_source;

	return 0;
}
