////////////////////////////////////////////////////////////////////////////////
//
//	File: hirsbase.hpp
//
//	Provides: HIRSBase ABC
//
//	Purpose: Base class for filters that handle HIRS data
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef hirsbase_defined
#define hirsbase_defined

#include "image.hpp"
#include "noaa.hpp"

class HIRSBase
{
public:

	typedef NOAA_L1b_HIRS_Data::Element HIRS_Element;

	void patch_data_valid(HIRS_Element* element,
						  int element_number,
						  int line_number)
		{
			// if element_number is 63 do nothing
			// element bitfield 15 is data valid, 14 is odd parity
			// if valid and parity are both 1 do nothing
			// if valid is 0 and parity is 1 do nothing
			// if valid and parity are 0 the valid may be wrong
			//

			bool data_valid = (element->bitflags&&(1<<15))!=0;
			bool parity = (element->bitflags&&(1<<14))!=0;
			bool mirror_lock = (element->bitflags&&(1<<3))!=0;
//		int line_counter = element->gen.scan_position_number;

//		cout << "lock " << mirror_lock << "	   ";

			if (!data_valid &&
				!parity &&
				!mirror_lock &&
				(element_number > 3 || line_number > 0))
			{
				// set data valid to 1
				element->bitflags |= (1<<15);
			}

//		element->bitflags = -1;
		}

	//
	// output_text_element
	//

	void output_text_element(
		textoutstream_type& os,
		HIRS_Element* element, int element_no)
		{
			os << "Element " << element_no << " ";
			if (element_no<56)
			{
				os << "(Radiometric Data)\n";
			}
			else if (element_no==56)
			{
				os << "(Positive Electronics Calibration)\n";
			}
			else if (element_no==57)
			{
				os << "(Negative Electronics Calibration)\n";
			}
			else if (element_no==58)
			{
				os << "(Internal Warm Target Temperature Sensor 1-4)\n";
			}
			else if (element_no==59)
			{
				os << "(Internal Cold Target Temperature Sensor 1,3,5 and Ground)\n";
			}
			else if (element_no==60)
			{
				os << "(Filter Wheel Housing Temperature Sensor 1-4)\n";
			}
			else if (element_no==61)
			{
				os << "(Patch temperature, Radiator Temperature, Filter wheel heater current, D/A converter voltage)\n";
			}
			else if (element_no==62)
			{
				os << "(Misc Temperature and Current)\n";
			}
			else if (element_no==63)
			{
				os << "(Line Counter, Status Words and "
				   << "Data Verification Codes) ('u' prefix indicates unsigned value)\n";
			}

			os << "Scan Encoder Position		  : "
			   << static_cast<int>(element->header[0]) << endl
			   << "Electronic Cal Level Indicator : "
			   << static_cast<int>(element->header[1]>>3) << endl
			   << "Channel 1 Period Monitor		  : "
			   << (((element->header[2]&63)<<1)+(element->header[3]>>7)) << endl
			   << "Element Number				  : "
			   << ((element->header[3]&126)>>1) << endl
			   << "Filter Sync Designator		  : "
			   << (element->header[3]&1) << endl
			   << "Channel Data :\n";

			int filter_wheel_ordering[] = {1, 17, 2, 3, 13, 4, 18, 11, 19, 7, 8, 20, 10, 14, 6, 5, 15, 12, 16, 9};

			for(int channel=0; channel<20; channel++)
			{
				int item = element->channels[channel];
				if (element_no < 56)
				{
					int sign = ((item&4096)==0)?-1:1;
					item = (item&4095) * sign;
					os << "H" << ntofs(filter_wheel_ordering[channel],2,' ') << ":"
					   << ntofs(item, 5, ' ', true) << " ";
				}
				else
				{
					if ((element_no<63) || (element_no==63 && channel>=3))
					{
						int sign = ((item&4096)==0)?-1:1;
						item = (item&4095) * sign;
						os << ntofs(channel+1,2,' ') << ":" << ntofs(item, 5, ' ', true) << " ";
					}
					else
					{
						os << ntofs(channel+1,2,' ') << ":u" << ntofs(item, 4, ' ', true) << " ";
					}
				}

				if (channel%5==4)
				{
					os << endl;
				}
			}

			os << "\nData Valid : "
			   << ((((element->bitflags)&32768)==0)?"0":"1") << endl
			   << "Odd Bit Parity : "
			   << ((((element->bitflags)&16384)==0)?"0":"1") << endl
			   << endl;
		}

	//
	// get_image_count
	//

	int get_image_count(void)
		{
			return 20;
		}

	//
	// write_image
	//

	void write_image(int number, Image* image)
		{
			spinner.reset(new Spinner(string("Preparing image ")+
									  ntos(number+1)));

			DEBUG("min is " << hirs_min[number] << " max is "
				  << hirs_max[number] << endl);

			image->set_filename_suffix(string(".") + ntos(number+1));
			image->set_attributes(Image::IMAGE_MONO,
								  56, hirs_scan_lines.size());
			image->begin_output();

			DEBUG("\n\n\nstart of image " << number << "\n\n\n");

			byte row[56];

			for(HIRSScanLines::iterator yiter=hirs_scan_lines.begin();
				yiter!=hirs_scan_lines.end();
				yiter++)
			{
				for(int x=0; x<56; x++)
				{
					double pixel = static_cast<double>(
						(*yiter).pixels[x].channels[number]-hirs_min[number]) /
						static_cast<double>(hirs_max[number]-hirs_min[number]) *
						255.0;
					row[x] = static_cast<byte>(pixel);
				}

				image->write_line(row);
				spinner->tick();
			}

			image->end_output();
			spinner->end();
		}

	//
	// get_hirs_element
	//

	HIRS_Element get_hirs_element(CCSDS_HIRS* packet, int element)
		{
			HIRSPayload& payload = packet->payload;

			// ... convert the CCSDS format element to a L1b format element
			HIRS_Element result;
			bzero(&result,sizeof(result));

			byte* payload_byte = (byte*)(payload.elements+element);
			int payload_bit = 7;

			// ... each containing 22 words followed by 2 single bits
			for(int j=0; j<22; j++)
			{
				unsigned short word = 0;

				// .. using 13 bits per word
				for(int k=0; k<13; k++)
				{
					word <<= 1;
					if ( (((*payload_byte)&(1<<payload_bit))) != 0)
					{
						word |= 1;
					}

					if (payload_bit > 0)
					{
						payload_bit--;
					}
					else
					{
						payload_bit = 7;
						payload_byte++;
					}
				}

				//cout << "word " << j << " value " << word << endl;

				if (j==0)
				{
					result.header[0] = word>>5;
					result.header[1] = (word&31) << 3;
				}
				else if (j==1)
				{
					result.header[2] = word>>8;
					result.header[3] = word & 255;
				}
				else
				{
					result.channels[j-2] = word;
					// its regular channel data

					//	if (element<56)
					//	{
					//		short item;
					//		if ((word&4096) != 0)
					//		{
					//			item = word&4095;
					//		}
					//		else
					//		{
					//			item = -(word&4095);
					//		}
					//		item += 4096;
					//		unsigned short uitem = *reinterpret_cast<unsigned short*>(&item);
					//		result.channels[j-2] = uitem;
					//	}
					//	else if (element!=63 || j>4)
					//	{
					//		// last 8 elements are copied stright over
					//		short item;
					//		if ((word&4096) != 0)
					//		{
					//			item = word&4095;
					//		}
					//		else
					//		{
					//			item = -(word&4095);
					//		}
					//		unsigned short uitem = *reinterpret_cast<unsigned short*>(&item);
					//		result.channels[j-2] = uitem;
					//	}
					//	else
					//	{
					//		// first 3 channels of element 63 are copied over
					//		result.channels[j-2] = word;
					//	}
				}
			}

			// now read the final two bits

			result.bitflags = 0;
			if (((*payload_byte) & 2) == 2)
			{
				result.bitflags |= (1<<15);
			}

			if (((*payload_byte) & 1) == 1)
			{
				result.bitflags |= (1<<14);
			}

			return result;
		}


	struct HIRSPixel
	{
		int channels[20];
	};

	struct HIRSScanLine
	{
		HIRSPixel pixels[56];
	};

	typedef deque<HIRSScanLine> HIRSScanLines;

protected:

	HIRSScanLines hirs_scan_lines;

	int hirs_min[20];
	int hirs_max[20];

};

#endif
