////////////////////////////////////////////////////////////////////////////////
//
//	File: hirsreader.hpp
//
//	Provides: HIRSReader filter
//
//	Purpose: Convert NOAA HIRS L1b products to source packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef hirsreader_defined
#define hirsreader_defined

#include "noaa.hpp"
#include "ccsdsgeneratorbase.hpp"
#include "productproducer.hpp"
#include "config.hpp"
#include "ccsdsproducer.hpp"
#include "hirsbase.hpp"
#include "imageproducer.hpp"

#ifdef Config
#undef Config
#endif
#define Config Config::HIRS

class HIRSReader :
	public HIRSBase,
	public CCSDSGeneratorBase<CCSDS_HIRS>,
	public CCSDSProducer,
	public ImageProducer
{
public:

	//
	// constructor
	//

	HIRSReader(ProductProducer& _source, const string& config_file = "")
		: source(_source)
		, scan_line_count(0)
		, packets_generated(0)
		{
			if (!config_file.empty())
			{
				Config::file.read(config_file);
			}

			assert(NOAA_L1b_HIRS_Header::SIZE==sizeof(NOAA_L1b_HIRS_Header));
			assert(NOAA_L1b_HIRS_Data::SIZE==sizeof(NOAA_L1b_HIRS_Data));
			header = (NOAA_L1b_HIRS_Header*)source.get_product_header();

			if (header && header->gen.data_type_code !=
				NOAA_L1b_GEN_Header::DTC_HIRS)
			{
				error << "This program can only read HIRS products\n";
				return;
			}

			data = reinterpret_cast<NOAA_L1b_HIRS_Data*>(source.get_current_record());
			if (!data)
			{
				return;
			}

			// patch data valid bits
			for(int element=0; element<64; element++)
			{
				patch_data_valid(&(data->elements[element]), element, data->gen.scan_position_number);
			}

			start_scan_line = data->gen.scan_line_number;
			start_year = data->gen.scan_line_year;
			start_day = data->gen.scan_line_day_of_year;
			start_millisecond = data->gen.scan_line_millisecond_of_day;

			source.skip_records(1);
			scan_line_count++;
			next_data = reinterpret_cast<NOAA_L1b_HIRS_Data*>(source.get_current_record());
			if (!next_data)
			{
				return;
			}

			// patch data valid bits
			for(int element=0; element<64; element++)
			{
				patch_data_valid(&(next_data->elements[element]), element, next_data->gen.scan_position_number);
			}

			// prepare for source packet output

			ccsds_set_sequence_count(
				Config::initial_sequence_count);

			if (Config::utc_ignore_input_time)
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);

				ccsds_modify_utc_base_time(data->gen);
			}
			else
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);
			}

			ccsds_set_obt_zero_time(
				data->gen);

			ccsds_apply_obt_offsets(
				Config::obt_coarse_offset,
				Config::obt_fine_offset,
				Config::obt_fixed_fine_time);


		}

	//
	// output_text_summary
	//

	void output_text_summary( ostream& os = cout )
		{
			string start_time = TimeConv::abs_year_day_millisecond_to_string(
				start_year,
				start_day,
				start_millisecond);

//		data = (NOAA_L1b_HIRS_Data*)source.get_data_record(
//		source.get_record_count()-1);
			string end_time = TimeConv::abs_year_day_millisecond_to_string(
				data->gen.scan_line_year,
				data->gen.scan_line_day_of_year,
				data->gen.scan_line_millisecond_of_day);

			os << "HIRSReader report:\n"
			   << "	   " << scan_line_count << " scan lines processed ("
			   << start_scan_line << "-" << data->gen.scan_line_number << ")\n"
			   << "	   Product start time " << start_time << endl
			   << "	   Product end time " << end_time << endl
			   << "	   Elements per scan line 64\n"
			   << "	   Bits per sample 13\n"
			   << endl;
		}

	//
	// output_text_config
	//

	void output_text_config( ostream& os = cout )
		{
			os << "HIRSReader configuration:\n"
			   << "	   Scan line number as sequence count "
			   << (Config::scan_line_number_as_sequence_count?"on":"off")
			   << endl
			   << "	   Initial sequence count "
			   << Config::initial_sequence_count << endl
			   << "	   UTC days offset " << Config::utc_days_offset << endl
			   << "	   UTC milliseconds offset " <<
				Config::utc_milliseconds_offset << endl
			   << "	   UTC microseconds offset " <<
				Config::utc_microseconds_offset << endl
			   << "	   Ignore UTC time " <<
				(Config::utc_ignore_input_time?"on":"off") << endl
			   << "	   OBT coarse offset " << Config::obt_coarse_offset << endl
			   << "	   OBT fine offset " << Config::obt_fine_offset << endl
			   << "	   OBT fixed fine time " << Config::obt_fixed_fine_time
			   << endl << endl;
		}

	//
	// output_text_header
	//

	void output_text_header( ostream& os = cout )
		{
			source.output_text_archive_header(os);
			source.output_text_gen_header(os);

			os << "Instrument status				: "
			   << ntobs(header->instrument_status[0],8) << endl
			   << "									  "
			   << ntobs(header->instrument_status[1],8) << endl
			   << "									  "
			   << ntobs(header->instrument_status[2],8) << endl
			   << "									  "
			   << ntobs(header->instrument_status[3],8) << endl
			   << "Record number of status change	: "
			   << header->record_number_of_status_change << endl
			   << "Second instrument status			: "
			   << ntobs(header->second_instrument_status[0],8) << endl
			   << "									  "
			   << ntobs(header->second_instrument_status[1],8) << endl
			   << "									  "
			   << ntobs(header->second_instrument_status[2],8) << endl
			   << "									  "
			   << ntobs(header->second_instrument_status[3],8) << endl
			   << "Count of data records			: "
			   << header->count_of_data_records << endl
			   << "Count of good scan lines			: "
			   << header->count_of_good_scan_lines << endl
			   << "Count of missing lines			: "
			   << header->count_of_missing_scan_lines << endl
			   << "Count of data gaps				: "
			   << header->count_of_data_gaps << endl
			   << "Frames without frame sync errors : "
			   << header->frames_without_sync_errors << endl
			   << "PACS detected TIP parity errors	: "
			   << header->pacs_detected_tip_parity_errors << endl
			   << endl;
		}

	//
	// output_text_record
	//

	void output_text_record(int record = -1, bool show_header = false, ostream& os = cout )
		{
			// we already have record 0 in data, record 1 in next_data
			if (record > 1)
			{
				source.skip_records(record-1);
				next_data = reinterpret_cast<NOAA_L1b_HIRS_Data*>(source.get_current_record());
				if (!next_data)
				{
					os << "Cannot access record " << record << endl;
					return;
				}
			}

			// Introduction

			NOAA_L1b_HIRS_Data* display_data;
			if (record==0)
			{
				display_data = data;
			}
			else
			{
				display_data = next_data;
			}

			if (show_header)
			{
				hirs_output_text_gen_record(os, record, &display_data->gen);
			}

			os << endl;

			for(int i=0; i<64; i++)
			{
				output_text_element(os, display_data->elements+i, i);
			}
		}

	//
	// write_bit
	//

	void write_bit(byte*& ptr, int& bit, int value)
		{
			if (value>0)
			{
				(*ptr) |= (1<<bit);
			}

			if (bit>0)
			{
				bit--;
			}
			else
			{
				bit = 7;
				ptr++;
			}
		}

	//
	// write_word
	//

	void write_word(byte*& ptr, int& bit, unsigned int word)
		{

//		cout << "hirsreader::write_word word is " << word << endl;

			for(int i=0; i<13; i++)
			{
				write_bit(ptr, bit, word&(1<<12));
				word <<= 1;
			}
		}

	//
	// get_next_ccsds
	//

	CCSDS* get_next_ccsds(void)
		{
//		cout << "hirsreader::get_next_ccsds called\n";

			if (!source.get_current_record())
			{
				if (spinner.get())
				{
					spinner->end();
				}
				return NULL;
			}

			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			}
			spinner->tick();

			ccsds_initialise();
			ccsds_set_time(data->gen);
			// copy scan line number into sequence number if configured
			if (Config::scan_line_number_as_sequence_count)
			{
				ccsds_set_sequence_count(data->gen.scan_line_number+
										 Config::initial_sequence_count);
			}

			// translate element data

			// iterate through all elements, applying the same conversion
			// to each
			for(int element=0; element<64; element++)
			{
				byte* payload_byte = (byte*)(ccsds_packet.payload.elements+element);
				int payload_bit = 7;

				// add the header words

				NOAA_L1b_HIRS_Data* data_for_header;
				int element_for_header;

				if (element<63)
				{
					data_for_header = data;
					element_for_header = element+1;
				}
				else
				{
					data_for_header = next_data;
					element_for_header = 0;
				}

				write_word(payload_byte, payload_bit,
						   (data_for_header->elements[element_for_header].header[0]<<5)+
						   (data_for_header->elements[element_for_header].header[1]>>3));

				write_word(payload_byte, payload_bit,
						   (data_for_header->elements[element_for_header].header[2]<<8)+
						   (data_for_header->elements[element_for_header].header[3]));

				// add the main sensors data
				for(int channel=0; channel<20; channel++)
				{
					unsigned short item = data->elements[element].channels[channel];
					if (element < 56)
					{
						if (item > 4096)
						{
							item &= 4095;
							item |= 4096;
						}
						else
						{
							item ^= 4095;
							item++;
						}
					}
					else
					{

						if (element==63 && channel<=2)
						{
							// this is the line counter which is unsigned
//				cout << "linecounter " << item << endl;
						}
						else
						{
							if (item > 4096)
							{
								item &= 4095;
								item ^= 4095;
								item++;
							}
							else
							{
								item |= 4096;
//				item  =0;
							}
						}
					}

					write_word(payload_byte, payload_bit, item);
				}

				// write data valid bit
				write_bit(payload_byte, payload_bit,
						  (data_for_header->elements[element_for_header].
						   bitflags&(1<<15))==0?0:1);

				// write odd parity bit

//		cout << "writing to ccsds packet odd parity of "
//			 << ((data->elements[i].bitflags&(1<<14))==0?0:1) << endl;

				write_bit(payload_byte, payload_bit,
						  (data_for_header->elements[element_for_header].
						   bitflags&(1<<14))==0?0:1);
			}

			data = next_data;
			source.skip_records(1);
			NOAA_L1b_HIRS_Data* next =
				reinterpret_cast<NOAA_L1b_HIRS_Data*>(source.get_current_record());
			if (next != NULL)
			{
				next_data = next;

				// patch the data valid bits
				for(int element=0; element<64; element++)
				{
					patch_data_valid(&(next_data->elements[element]), element, next_data->gen.scan_position_number);
				}
			}

			scan_line_count++;
			packets_generated++;
			return ccsds_finalise();
		}

	//
	// prepare_images
	//


	void prepare_images(void)
		{
			// clear the min/max ranges (used for normalisation)
			for(int i=0; i<20; i++)
			{
				hirs_min[i] = 100000;
				hirs_max[i] = -100000;
			}

			int temp = 0;

			spinner.reset(new Spinner("Reading in image data"));

			while(source.get_current_record())
			{
				data = (NOAA_L1b_HIRS_Data*)source.get_current_record();

//		cout << "temp " << temp << " elelement 63 channel 1 period "
//		<< "monitor is "
//			 << (((data->elements[63].header[2]&63)<<1)+
				// (data->elements[63].
//		header[3]>>7)) << endl;
				temp++;

				HIRSScanLine scan_line;
				for(int position=0; position<56; position++)
				{
					for(int channel=0; channel<20; channel++)
					{
						unsigned int t = data->elements[position].
							channels[channel];
						int sample = (t&8191);
						if ((t&8192) == 0)
						{
							sample = -sample;
						}

						if (sample > hirs_max[channel] && sample < 8191)
						{
							hirs_max[channel] = sample;
						}

						if (sample < hirs_min[channel] && sample > -8191)
						{
							hirs_min[channel] = sample;
						}

						scan_line.pixels[position].channels[channel] = sample;
					}
				}

				spinner->tick();

				hirs_scan_lines.push_back(scan_line);
				source.skip_records(1);
			}

			spinner->end();
		}

	//
	// get_image_count
	//

	int get_image_count(void)
		{
			return HIRSBase::get_image_count();
		}

	//
	// write_image
	//

	void write_image(int number, Image* image)
		{
			return HIRSBase::write_image(number, image);
		}

	void hirs_output_text_gen_record(textoutstream_type& os, int record_no, NOAA_L1b_GEN_Data* d)
		{
			string t = TimeConv::abs_year_day_millisecond_to_string(
				d->scan_line_year,
				d->scan_line_day_of_year,
				d->scan_line_millisecond_of_day) +
				"." +
				ntofs(d->scan_line_millisecond_of_day%1000,3,'0') +
				"000";

			os << "Record number " << record_no << endl;

			os << "\nIntroduction\n------------\n\n"
			   << "Scan line number		 : " << d->scan_line_number << endl
			   << "Scan line year		 : " << d->scan_line_year << endl
			   << "Scan line day		 : " << d->scan_line_day_of_year
			   << endl
			   << "Scan line millisecond : "
			   << d->scan_line_millisecond_of_day << endl
			   << "Scan line time		 : " << t << endl << endl
			   << "Clock drift delta	 : "
			   << d->satelite_clock_drift_delta << endl
			   << "Clock drift applied	 : "
			   << ((d->scan_line_bit_field&(1<<14))!=0)
			   << endl << endl;
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

private:

	ProductProducer& source;
	NOAA_L1b_HIRS_Header* header;
	NOAA_L1b_HIRS_Data* data;
	NOAA_L1b_HIRS_Data* next_data;

	int start_scan_line;
	int start_year;
	int start_day;
	int start_millisecond;
	int scan_line_count;
	int packets_generated;
};

#undef Config

#endif
