////////////////////////////////////////////////////////////////////////////////
//
//	File: iasi_config.hpp
//
//	Provides: Misc configuration code specific to IASI CCSDS Spectrum packets
//
//	Purpose: Parse and store IASI Spectrum configuration file settings
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef iasi_config_defined
#define iasi_config_defined

#include "textconfigproducer.hpp"

const string IASI_COMMENT_CHAR = "#";


class IASIConfig :
	public TextConfigProducer
{
public:
	IASIConfig(void)
		: loaded(false),
		  ICtcNbSeg(0),
		  IDefCtcNbNsSeg(0),
		  ICtcTabNbBit_ctr(0)
		{
		}


	bool read(const string& filename)
		{
			DEBUG("iasi config reading " << filename << endl);
			ifstream ifs(filename.c_str());
			if (!ifs)
			{
				cout << "IASI Configuration file '" << filename << "' could "
					 << "not be opened\n\n";
				return false;
			}
			else
			{
				while(ifs)
				{
					char line[256];
					ifs.getline(line, 256);
					string s = line;

					//cout << "LINE: '" << s << "'\n";

					// remove comments
					string::size_type c = s.find(IASI_COMMENT_CHAR);
					if (c != string::npos)
					{
						s.erase(c);
					}

					//cout << "nocomment '" << s << "'\n";


					string::size_type kc;
					string::size_type vc;

					// only examine non empty lines
					if (!s.empty())
					{
						c = s.find(":");
						if (c != string::npos)
						{
							string keyword = s.substr(0,c);
							string value = s.substr(c+1);
							string value_array[15];

							// remove whitespace
							while( (kc=keyword.find(" ")) != string::npos)
							{
								keyword.erase(kc,1);
							}

							//cout << "keyword		= '" << keyword << "'\n";
							//cout << "value before = '" << value << "'\n";

							int ctr = 0;
							while( (vc=value.find(" ")) != string::npos)
							{
								//cout << "vc = " << vc << endl;
								value.erase(vc,1);
								if (vc > 0)
								{
									value_array[ctr] = value.substr(0, vc);

									for (int k=0; k<vc; k++)
									{
										value.erase(0,1);
									}
									ctr++;
								}
							}

							value_array[ctr] = value;

							//cout << "value after	= '" << value << "'\n";
							//for (int i=0; i<=ctr; i++)
							//{
							//	  cout << "value_array[" <<	 i << "] = " << value_array[i] << endl;
							//}

							keyword = tolower(keyword);
							if (keyword.find("ictcnbseg") !=  string::npos)
							{
								//cout << "keyword 1	= '" << keyword << "'\n";
								ICtcNbSeg = atoi(value_array[1].c_str());
							}

							else if (keyword.find("ictctabnbbit") !=  string::npos)
							{
								//cout << "keyword 2	= '" << keyword << "'\n";
								ICtcTabNbBit[ICtcTabNbBit_ctr][0] = atoi(value_array[2].c_str()); // No. Bits
								ICtcTabNbBit[ICtcTabNbBit_ctr][1] = atoi(value_array[3].c_str()); // Offset
								ICtcTabNbBit[ICtcTabNbBit_ctr][2] = atoi(value_array[4].c_str()); // Scale
								ICtcTabNbBit_ctr++;
							}

							else if (keyword.find("idefctcnbnsseg") !=	string::npos)
							{
								//cout << "keyword 3	= '" << keyword << "'\n";
								IDefCtcNbNsSeg = atoi(value.c_str());
							}

						}
						else
						{
							error << "Unrecognised line in config file "
								  << filename
								  << " '" << line << "' not understood\n";
						}
					}
				}

				loaded = true;
			}

			return true;

		}


	bool is_loaded(void) const
		{
			return loaded;
		}


	int get_ICtcNbSeg()
		{
			return ICtcNbSeg;
		}


	int get_IDefCtcNbNsSeg()
		{
			return IDefCtcNbNsSeg;
		}


	int* get_ICtcTabNbBit()
		{
			return &ICtcTabNbBit[0][0];
		}


	int get_ICtcTabNbBit_ctr()
		{
			return ICtcTabNbBit_ctr;
		}


	virtual void output_text_config( textoutstream_type& os )
		{
			os << "IASI Spectrum Packet Configuration:\n"
			   << "ICtcNbSeg		= " << ICtcNbSeg << endl
			   << "IDefCtcNbNsSeg	= " << IDefCtcNbNsSeg << endl
			   << "ICtcTabNbBit		= " << endl
			   << "\tCounter\tBits\tOffset\tScale\n";

			for (int i=0; i<ICtcTabNbBit_ctr; i++)
			{
				os << "\t" << i+1 << "\t";
				os << ICtcTabNbBit[i][0] << "\t";
				os << ICtcTabNbBit[i][1] << "\t";
				os << ICtcTabNbBit[i][2] << endl;
			}

		}


private:
	bool loaded;
	int ICtcNbSeg;
	int IDefCtcNbNsSeg;
	int ICtcTabNbBit_ctr;
	int ICtcTabNbBit[550][3];

};


#endif
