////////////////////////////////////////////////////////////////////////////////
//
//	File: l0_to_reduced_ccsds.cpp
//
//	Provides: l0_to_reduced_ccsds executable
//
//	Purpose: Convert CCSDS packets from a raw ISP file or L0 product into
//			 Reduced CCSDS packets with the SBT time removed.
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "ccsds.hpp"
#include "filereader.hpp"
#include "ccsdsreader.hpp"
#include "readfromstdin.hpp"
#include "ccsdsapidfilter.hpp"
#include "singlefilewriter.hpp"
#include "reducedccsdswriter.hpp"
#include "writetostdout.hpp"
#include "args.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "l0_to_reduced_ccsds";

const string MAINTAINER = "elson@eumetsat.de";
const string COPYRIGHT = "Copyright (c) 2005 Eumetsat.";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	deque<string> in_file;
	string output_directory;

	Args args;
	args.set_doc_head("Process either a stream of CCSDS packets ot a L0 product and convert it to Reduced CCSDS packets suitable for MHS analysis tools.");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright(COPYRIGHT);
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_example("l0_to_reduced_ccsds MHS_00_20060101120000Z -o SD_MHS", "Extract a tree of Reduced MHS packets from an MHS L0 product");

	args.add_group("File selection:");
	args.add(&in_file, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input file(s)", "If omitted packets are read from standard input");

	args.add_group("Output settings:");
	args.add(&output_directory, "output", 'o', 0, "Set output directory");

	args.add_group("Misc:");
	args.add(0, "quiet", 'q', 0, "Omit additional messages");
	args.add(0, "", 'z', 0, "Use simple busy animations");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case Args::USER_OPTION:
			_debug = &cout;
			break;

		case 'z':
			Spinner::use_simple_spinner();
			break;
		}
	}

	// set UTC time zone
	TimeConv::set_time_zone();

	// sort out streams
	if (output_directory.empty())
	{
		cerr << "Output directory not set\n";
		return -1;
	}

	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	CCSDSReader ccsds_reader(*first_source,0);
	ReducedCCSDSWriter writer(ccsds_reader, output_directory);

	if (!gerror)
	{
		first_source->output_text_summary();
		ccsds_reader.output_text_summary();
		writer.output_text_summary();
	}

	delete first_source;

	return 0;
}

