////////////////////////////////////////////////////////////////////////////////
//
//	File: mhs_to_ccsds.cpp
//
//	Provides: mhs_to_ccsds executable
//
//	Purpose: Convert a NOAA L1b v3 MHS product into either a file of Metop MHS
//			 CCSDS packets or a directory of Reduced MHS packets
//
//	Copyright (c) 2005 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "noaaproductreader.hpp"
#include "readfromstdin.hpp"
#include "multifilewriter.hpp"
#include "writetostdout.hpp"
#include "singlefilewriter.hpp"
#include "mhsreader.hpp"
#include "ccsdsextractor.hpp"
#include "imagewriter.hpp"
#include "reducedccsdswriter.hpp"
#include "args.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string CONFIG_DIR_VAR = "METOPIZER_HOME";
//const string DEFAULT_CONFIG_DIR = ".metopizer";

const string APPNAME = "mhs_to_ccsds";
const string MAINTAINER = "elson@eumetsat.de";
const string DEFAULT_CONFIG_FILE = "mhs.config";

#include "noaa.hpp"
#include "noaa_v3.hpp"

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	string in_file;
	string out_file;
	string config_file;
	string image_file;
	string reduced_mhs;
	bool verbose = true;
	bool no_type_check = false;
#ifndef _NO_JPEG
	bool jpeg = false;
#endif

	deque<string> in_files;

	Args args;
	args.set_doc_head("Converts a NOAA L1b v3 MHS product to MHS CCSDS packets.");
	args.set_doc_tail("");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2005 Eumetsat.");
	args.set_license(Args::GPL);
//	  args.set_usage("input-file output-file");
	args.set_version(VERSION);

	args.add_related_prog("amsub_to_mhs_ccsds");
	args.add_related_prog("noaa_l1b_displayer");
	args.add_related_prog("ccsds_displayer");

	args.add_example("mhs_to_ccsds P1xxx.mhs mhs.ccsds", "Convert a NOAA MHS product into a stream of CCSDS source packets:");
#ifndef _NO_JPEG
	args.add_example("mhs_to_ccsds P1xxx.mhs --image mhs-pic --jpeg", "Create a set of 5 Jpeg images from the channels of a NOAA MHS product:");
#endif
	args.add_env_var("METOPIZER_HOME", "Location to search for mhs.config file");

	args.add_group("File selection:");
	args.add(&in_file, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input file");
	args.add(&out_file, "out", 'o', Args::FILENAME, "Select output file");
	args.add(&config_file, "config", 'c', Args::FILENAME, "Use a config file");
	args.add(&image_file, "image", 0, Args::FILENAME, "Output 5 channel images in TIFF format", "Normal processing is suppressed. The value given is the filename prefix to use");

	args.add_group("Settings:");
#ifndef _NO_JPEG
	args.add(&jpeg, "jpeg", 'j', 0, "Use Jpeg output instead of TIFF", "Only applies if the --image option is selected");
#endif
	args.add(&no_type_check, "mhs", 0, 0, "Force the input file to be read as MHS", "When this flag is used the Data Type Code in the header block is ignored and the file will be treated as a NOAA L1b MHS product with no checks");
	args.add(&reduced_mhs, "reduced-mhs", 0, Args::FILENAME, "Output as Reduced MHS packets", "In this format each packet is written to a seperate file, with one directory per hour. File and directory names follow a fixed format for ingestion using MHS analysis software");

	enum {action_debug = Args::USER_OPTION};

	args.add(0, "quiet", 'q', 0, "Suppress most output messages");
	args.add(0, "debug", action_debug, Args::HIDDEN, "");

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case 'q':
			verbose = false;
			break;

		case action_debug:
			_debug = &cout;
			break;
		}
	}

//	  cout << "in_file: " <<  in_file << endl
//	 << "out_file: " << out_file << endl
//	 << "config_file: " << config_file << endl
//	 << "image_file: " << image_file << endl
//	 << "verbose: " << verbose << endl
//	 << "no_type_check: " << no_type_check << endl
//	 << "reduced_mhs: " << reduced_mhs << endl
//	 << "jpeg: " << jpeg << endl;
//	  cout << "in_files: ";
//	  for(deque<string>::iterator iter=in_files.begin();
//	iter!=in_files.end();
//	iter++)
//	  {
//	cout << (*iter) << " ";
//	  }
//	  cout << endl;

	// if no config file specified check for an implicit one
	if (config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_file = string(config_dir_var) + "/" + DEFAULT_CONFIG_FILE;
		}
		else //if (getenv("HOME"))
		{
			// use the default filename
			//config_file = string(getenv("HOME")) + "/" + DEFAULT_CONFIG_DIR +
			//	"/" + DEFAULT_CONFIG_FILE;
			config_file = string("DEFAULT_METOPIZER_HOME") + "/" + DEFAULT_CONFIG_FILE;
		}
	}

	// fix streams if we are outputting data to stdout
	if (out_file.empty() && reduced_mhs.empty())
	{
		_info = &onull;
		_warning = &cerr;
		verbose = false;
		Spinner::restream(&onull);
	}

	if (!verbose)
	{
		_info = &onull;
	}

	FileProducer* file_source;
	if (in_file.empty())
	{
		file_source = new ReadFromStdin();
	}
	else
	{
		file_source = new FileReader(in_file);
	}

	NOAAProductReader noaa_reader(*file_source);

	if (image_file.empty())
	{
		// source packet output

		MHSReader mhs_reader(noaa_reader, config_file, no_type_check);

		if (reduced_mhs.empty())
		{
			// normal packets

			CCSDSExtractor extractor(mhs_reader);
			if (out_file.empty())
			{
				WriteToStdout writer(&extractor);
			}
			else
			{
				SingleFileWriter writer(&extractor, out_file);

				if (verbose && !gerror)
				{
					file_source->output_text_summary();
					noaa_reader.output_text_summary();
					mhs_reader.output_text_summary();
					extractor.output_text_summary();
					writer.output_text_summary();
				}
			}
		}
		else
		{
			// reduced MHS packets

			ReducedCCSDSWriter writer(mhs_reader, reduced_mhs);
			if (verbose && !gerror)
			{
				file_source->output_text_summary();
				noaa_reader.output_text_summary();
				mhs_reader.output_text_summary();
				writer.output_text_summary();
			}
		}
	}
	else
	{
		// image output

		MHSReader mhs_reader(noaa_reader, config_file, no_type_check);
#ifdef _NO_JPEG
		ImageWriter writer(mhs_reader, image_file, Image::IMAGE_TIFF);
#else
		ImageWriter writer(mhs_reader, image_file, jpeg?Image::IMAGE_JPEG:Image::IMAGE_TIFF);
#endif

		if (verbose && !gerror)
		{
			file_source->output_text_summary();
			noaa_reader.output_text_summary();
			writer.output_text_summary();
		}
	}

	delete file_source;

	return 0;
}
