////////////////////////////////////////////////////////////////////////////////
//
//	File: mhsreader.hpp
//
//	Provides: MHSReader filter
//
//	Purpose: Display MHS products or generate images
//
//	Copyright (c) 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef mhs_defined
#define mhs_defined

#include "noaa_v3.hpp"
#include "imageproducer.hpp"
#include "image.hpp"
#include "ccsdsproducer.hpp"
#include "mhstempconv.hpp"
#include "config.hpp"
#include "ccsdsgeneratorbase.hpp"

#ifdef Config
#undef Config
#endif
#define Config Config::MHS

class MHSReader :
	public ImageProducer,
	public CCSDSGeneratorBase<CCSDS_MHS>,
	public CCSDSProducer
{
public:

	typedef NOAA_L1b_MHS_Data_v3::Sample Sample;

	//
	// constructor
	//

	MHSReader(ProductProducer& _source,
			  string config_file,
			  bool no_type_check)
		: source(_source),
		  scan_line_count(0)
		{
			Config::file.read(config_file);

//		cout << sizeof(NOAA_L1b_GEN_Header_v3) << endl;
//		cout << sizeof(NOAA_L1b_MHS_Header_v3) << endl;

			assert(NOAA_L1b_MHS_Header_v3::SIZE==
				   sizeof(NOAA_L1b_MHS_Header_v3));

			assert(NOAA_L1b_MHS_Data_v3::SIZE==
				   sizeof(NOAA_L1b_MHS_Data_v3));

			header = reinterpret_cast<NOAA_L1b_MHS_Header_v3*>(
				source.get_product_header());

			if (!no_type_check && header && header->gen.data_type_code !=
				NOAA_L1b_GEN_Header_v3::DTC_MHS)
			{
				error << "This program can only read MHS products (DTC code "
					  << static_cast<int>(NOAA_L1b_GEN_Header_v3::DTC_MHS)
					  << ") but the input product has code "
					  << static_cast<int>(header->gen.data_type_code)
					  << ".\nUse the --mhs option to force the input to be "
					  << "treated as an MHS product.\n";
				exit(-1);
			}

			data = reinterpret_cast<NOAA_L1b_MHS_Data_v3*>(
				source.get_current_record());
			if (!data)
			{
				return;
			}

			start_scan_line = data->gen.scan_line_number;
			start_year = data->gen.scan_line_year;
			start_day = data->gen.scan_line_day_of_year;
			start_millisecond = data->gen.scan_line_millisecond_of_day;

			// prepare for source packet output

			ccsds_set_sequence_count(
				Config::initial_sequence_count);

			if (Config::utc_ignore_input_time)
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);

				ccsds_modify_utc_base_time(data->gen);
			}
			else
			{
				ccsds_set_utc_offsets(
					Config::utc_days_offset,
					Config::utc_milliseconds_offset,
					Config::utc_microseconds_offset);
			}

			ccsds_set_obt_zero_time(
				data->gen);

			ccsds_apply_obt_offsets(
				Config::obt_coarse_offset,
				Config::obt_fine_offset,
				Config::obt_fixed_fine_time);

		}

	//
	// output_text_header
	//

	void output_text_header( ostream& os = cout )
		{
			// archive header
			source.output_text_archive_header(os);

			// generic header
			source.output_text_gen_header(os);

			os << endl;
		}

	//
	// output_text_record
	//

	void output_text_record(int record_no = -1, bool show_header = false, ostream& os = cout)
		{
			source.skip_records(record_no);
			data = reinterpret_cast<NOAA_L1b_MHS_Data_v3*>(
				source.get_current_record());
			if (!data)
			{
				os << "Cannot access record " << record_no << endl;
				return;
			}

			// Introduction

			if (show_header)
			{
				source.output_text_gen_record(record_no, os);
			}

			os << "OBT times\n---------\n\n"
			   << "Coarse : " << data->coarse_time << endl
			   << "Fine	  : " << data->fine_time << endl;

			os << "\nMode\n----\n\n"
			   << "Mode code		   : ";
			switch ((data->mode>>4)&15)
			{
			case 0 : os << "0 (Power on)\n"; break;
			case 1 : os << "1 (Warm up)\n"; break;
			case 2 : os << "2 (Standby)\n"; break;
			case 3 : os << "3 (Scan)\n"; break;
			case 4 : os << "4 (Fixed View)\n"; break;
			case 5 : os << "5 (Self test)\n"; break;
			case 6 : os << "6 (Safeing)\n"; break;
			case 7 : os << "7 (Fault)\n"; break;
			case 15 : os << "15 (Memory data packet ID)\n"; break;
			default:
				os << ((data->mode>>4)&15) << "(unused)\n"; break;
			}

			os << "PIE-ID			   : " << ((data->mode>>3)&1) << endl
			   << "Subcommutation code : " << (data->mode&7) << endl
			   << endl;

			// telecommand ack and fault code

			os << "Telecommand Acknowledgement and Fault Codes\n"
			   << "-------------------------------------------\n\n"
			   << "TC Clean False/True				   : "
			   << ((data->telecommand[0]>>7)&1) << endl
			   << "TC Conforms False/True			   : "
			   << ((data->telecommand[0]>>6)&1) << endl
			   << "TC Recognised False/True			   : "
			   << ((data->telecommand[0]>>5)&1) << endl
			   << "TC Legal False/True				   : "
			   << ((data->telecommand[0]>>4)&1) << endl
			   << "FDM Motor Current Trip False/True   : "
			   << ((data->telecommand[0]>>3)&1) << endl
			   << "TC Application ID				   : "
			   << (((data->telecommand[0]&7)<<8)+(data->telecommand[1]&255))
			   << endl
			   << "TC Packet Sequence				   : "
			   << (((data->telecommand[2]&255)<<8)+
				   ((data->telecommand[3]>>2)&63))
			   << endl
			   << "TC Received Count				   : "
			   << (data->telecommand[3]&3) << endl
			   << "Monitor Current Fault False/True	   : "
			   << ((data->telecommand[4]>>7)&1) << endl
			   << "Thermistor Monitor Fault False/True : "
			   << ((data->telecommand[4]>>6)&1) << endl
			   << "Switch Fault False/True			   : "
			   << ((data->telecommand[4]>>5)&1) << endl
			   << "Processor Fault False/True		   : "
			   << ((data->telecommand[4]>>4)&1) << endl
			   << "RDM Motor Current Trip False/True   : "
			   << ((data->telecommand[4]>>3)&1) << endl
			   << "DC Offset Error False/True		   : "
			   << ((data->telecommand[4]>>2)&1) << endl
			   << "Scan Control Error False/True	   : "
			   << ((data->telecommand[4]>>1)&1) << endl
			   << "REF CK Error False/True			   : "
			   << (data->telecommand[4]&1) << endl
			   << endl;

			// Switch Status

			os << "Switch Status\n--------------\n\n"
			   << "Receiver Channel H1 Off/On					: "
			   << ((data->switch_status[0]>>0)&1) << endl
			   << "Receiver Channel H1 Local Oscillator A/B		: "
			   << ((data->switch_status[0]>>1)&1) << endl
			   << "Receiver Channel H2 Off/On					: "
			   << ((data->switch_status[0]>>2)&1) << endl
			   << "Receiver Channel H2 Local Oscillator A/B		: "
			   << ((data->switch_status[0]>>3)&1) << endl
			   << "Receiver Channels H3/H4 Frontend Off/On		: "
			   << ((data->switch_status[0]>>4)&1) << endl
			   << "Receiver Channels H3/H4 Local Oscillator A/B : "
			   << ((data->switch_status[0]>>5)&1) << endl
			   << "Receiver Channel H3 Backend Off/On			: "
			   << ((data->switch_status[0]>>6)&1) << endl
			   << "Receiver Channel H4 Backend Off/On			: "
			   << ((data->switch_status[0]>>7)&1) << endl
			   << "Receiver Channel H5 Off/On					: "
			   << ((data->switch_status[1]>>0)&1) << endl
			   << "Receiver Channel H5 Local Oscillator A/B		: "
			   << ((data->switch_status[1]>>1)&1) << endl
			   << "Rx CV Off/On									: "
			   << ((data->switch_status[1]>>2)&1) << endl
			   << "Receiver Operational Heaters Off/On			: "
			   << ((data->switch_status[1]>>3)&1) << endl
			   << "Scan Mechanism Operational Heaters Off/On	: "
			   << ((data->switch_status[1]>>4)&1) << endl
			   << "Auxiliary Operational Heaters Off/On			: "
			   << ((data->switch_status[1]>>5)&1) << endl
			   << "Signal Processing Electronics Off/On			: "
			   << ((data->switch_status[1]>>6)&1) << endl
			   << "PROM Failure False/True						: "
			   << ((data->switch_status[1]>>7)&1) << endl
			   << "RDM Zero Position Sensors A/B				: "
			   << ((data->switch_status[2]>>0)&1) << endl
			   << "FDM Zero Position Sensors A/B				: "
			   << ((data->switch_status[2]>>1)&1) << endl
			   << "RDM Motor Sensors A/B						: "
			   << ((data->switch_status[2]>>2)&1) << endl
			   << "FDM Motor Sensors A/B						: "
			   << ((data->switch_status[2]>>3)&1) << endl
			   << "RDM Motor Supply Off/On						: "
			   << ((data->switch_status[2]>>4)&1) << endl
			   << "FDM Motor Supply Off/On						: "
			   << ((data->switch_status[2]>>5)&1) << endl
			   << "RDM Motor Current Trip Enabled/Disabled		: "
			   << ((data->switch_status[2]>>6)&1) << endl
			   << "FDM Motor Current Trip Enabled/Disabled		: "
			   << ((data->switch_status[2]>>7)&1) << endl
			   << endl;

			// temperature data

			os << "Temperature Data\n----------------\n\n"
			   << "Sensor	Counts	 Temperature\n";
			for(int i=0; i<24; i++)
			{
				os << "	 " << ntofs(i+1,2,' ') << "		"
				   << ntofs(data->temperature[i]) << "	  ";
				double temp = mhs_temp_conv(data->temperature[i]);
				if (temp < 100.0)
				{
					os << dto2dp(temp);
				}
				else
				{
					os << ">" << dto2dp(temp);
				}
				os << "C";

				if (i==12 || i==13)
				{
					os << "	   (see note 1)";
				}
				os << endl;
			}
			os << endl;

			// current consumption data

			os << "Raw Current Consumption Data\n"
			   << "----------------------------\n\n"
			   << "+5v Secondary Current : "
			   << ntofs(data->current[0], 3, ' ') << "	("
			   << static_cast<int>(Config::v5_secondary_current_k *
								   data->current[0] * 1000)
			   << "mA)\n"
			   << "+8v Secondary Current : "
			   << ntofs(data->current[1], 3, ' ') << "	("
			   << static_cast<int>((Config::v8_secondary_current_k1 *
									data->current[1] * 1000)
								   + Config::v8_secondary_current_k2) << "mA)\n"
			   << "+15v Receiver Current : "
			   << ntofs(data->current[2], 3, ' ') << "	("
			   << static_cast<int>((Config::v15_receiver_current_k1 *
									data->current[2] * 1000) +
								   Config::v15_receiver_current_k2) << "mA)\n"
			   << "-15v Receiver Current : "
			   << ntofs(data->current[3], 3, ' ') << "	("
			   << static_cast<int>((Config::nv15_receiver_current_k1 *
									data->current[3] * 1000) +
								   Config::nv15_receiver_current_k2) << "mA)\n"
			   << "RDM Motor Current	 : "
			   << ntofs(data->current[4], 3, ' ') << "	("
			   << static_cast<int>(Config::rdm_motor_current_k *
								   data->current[4] * 1000) << "mA)\n"
			   << "FDM Motor Current	 : "
			   << ntofs(data->current[5], 3, ' ') << "	("
			   << static_cast<int>(Config::fdm_motor_current_k *
								   data->current[5] * 1000) << "mA)\n"
			   << endl;

			// status word

			os << "Status Word\n-----------\n\n"
			   << "DC Offset Valid False/True	 : "
			   << ((data->status_word>>7)&1) << endl
			   << "Scan Control Valid False/True : "
			   << ((data->status_word>>6)&1) << endl
			   << "Profile						 : "
			   << ((data->status_word>>4)&3) << endl
			   << endl;

			// signal processing status

			os << "Signal Processing Status\n------------------------\n\n"
			   << "Channel H1 DC Offset : "
			   << static_cast<int>(data->offset_words[0]) << endl
			   << "Channel H2 DC Offset : "
			   << static_cast<int>(data->offset_words[1]) << endl
			   << "Channel H3 DC Offset : "
			   << static_cast<int>(data->offset_words[2]) << endl
			   << "Channel H4 DC Offset : "
			   << static_cast<int>(data->offset_words[3]) << endl
			   << "Channel H5 DC Offset : "
			   << static_cast<int>(data->offset_words[4]) << endl
			   << "H1 Valid False/True	: "
			   << ((data->valid_flags>>7)&1) << endl
			   << "H2 Valid False/True	: "
			   << ((data->valid_flags>>6)&1) << endl
			   << "H3 Valid False/True	: "
			   << ((data->valid_flags>>5)&1) << endl
			   << "H4 Valid False/True	: "
			   << ((data->valid_flags>>4)&1) << endl
			   << "H5 Valid False/True	: "
			   << ((data->valid_flags>>3)&1) << endl
			   << "SPE Mux Code			: "
			   << (data->valid_flags&7) << endl
			   << "H1 Gain				: "
			   << ((data->channel_gain[0]>>5)&7) << endl
			   << "H2 Gain				: "
			   << ((data->channel_gain[0]>>2)&7) << endl
			   << "H3 Gain				: "
			   << ((data->channel_gain[1]>>5)&7) << endl
			   << "H4 Gain				: "
			   << ((data->channel_gain[1]>>2)&7) << endl
			   << "H5 Gain				: "
			   << ((data->channel_gain[2]>>5)&7) << endl
			   << endl;

			os << "Sensor Data\n-----------\n\n";
			display_samples(os, data->sensor_data, 90);
			os << endl;

			os << "Space View\n----------\n\n";
			display_samples(os, data->space_view, 4);
			os << endl;

			os << "Target View\n-----------\n\n";
			display_samples(os, data->target_view,4);
			os << endl;

			// OBCT sensors

			os << "OBCT Sensors\n------------\n\n"
			   << "Target Temperature PRT#1 : "
			   << ntofs(data->prt[0]&4095, 5, ' ') << "	 ("
			   << dto2dp(obct_to_temp(data->prt[0]&4095)) << "C)\n"
			   << "Target Temperature PRT#2 : "
			   << ntofs(data->prt[1]&4095, 5, ' ') << "	 ("
			   << dto2dp(obct_to_temp(data->prt[1]&4095)) << "C)\n"
			   << "Target Temperature PRT#3 : "
			   << ntofs(data->prt[2]&4095, 5, ' ') << "	 ("
			   << dto2dp(obct_to_temp(data->prt[2]&4095)) << "C)\n"
			   << "Target Temperature PRT#4 : "
			   << ntofs(data->prt[3]&4095, 5, ' ') << "	 ("
			   << dto2dp(obct_to_temp(data->prt[3]&4095)) << "C)\n"
			   << "Target Temperature PRT#5 : "
			   << ntofs(data->prt[4]&4095, 5, ' ') << "	 ("
			   << dto2dp(obct_to_temp(data->prt[4]&4095)) << "C)\n"
			   << "Calibration Channel 1	: "
			   << ntofs(data->prt_cal[0]&4095, 5, ' ') << "	 ("
			   << dto2dp(obct_to_res(data->prt_cal[0]&4095)) << " ohms)\n"
			   << "Calibration Channel 2	: "
			   << ntofs(data->prt_cal[1]&4095, 5, ' ') << "	 ("
			   << dto2dp(obct_to_res(data->prt_cal[1]&4095)) << " ohms)\n"
			   << "Calibration Channel 3	: "
			   << ntofs(data->prt_cal[2]&4095, 5, ' ') << "	 ("
			   << dto2dp(obct_to_res(data->prt_cal[2]&4095)) << " ohms)\n";

			for(int i=0; i<5; i++)
			{
				os << "Target Temperature PRT#" << (i+1)
				   << " read from product : "
				   << ntofs(data->prt_temp[i], 5, ' ') << "K  ("
				   << (data->prt_temp[i]+273.15) << "C)\n";
			}
			os << endl;

			// notes
			os << "Note 1: Temperature sensors 13 and 14 use different "
			   << "thermistor types\nwhen PIE A is in use, on the "
			   << "Engineering Model only. See document\n\"MHS "
			   << "Housekeeping Telemetry Conversion to Engineering "
			   << "Units\"\nfor details\n\n";

			os << "Note 2: The \"" << Config::current_constants_name.get()
			   << "\" coefficients were used to convert this set of "
			   << "values.\nChannel 5:1 RDM Motor k="
			   << Config::rdm_motor_current_k << "\nChannel 5:2 FDM Motor "
			   << "k=" << Config::fdm_motor_current_k << "\nChannel 5:3 "
			   << "+5v k=" << Config::v5_secondary_current_k << "\nChannel"
			   << " 5:4 +8v k1=" << Config::v8_secondary_current_k1
			   << "	   k2=" << Config::v8_secondary_current_k2 << endl
			   << "Channel 5:5 +15v k1=" << Config::v15_receiver_current_k1
			   << "	   k2=" << Config::v15_receiver_current_k2 << endl
			   << "Channel 5:6 -15v k1=" << Config::nv15_receiver_current_k1
			   << "	   k2=" << Config::nv15_receiver_current_k2 << endl
			   << endl;
		}

	//
	// obct_to_temp
	//

	double obct_to_temp(int input)
		{
			return ((-21.84 + (input/88.86)) / 0.391);
		}

	//
	// obct_to_res
	//

	double obct_to_res(int input)
		{
			return ( 78.16 + (input/88.86) );
		}

	//
	// display_samples
	//

	void display_samples(textoutstream_type& os, Sample* samples, int count)
		{
			os << "Sample no.	Shaft pos Ch#16 Ch#17 Ch#18 Ch#19 Ch#20\n";
			for(int i=0; i<count; i++)
			{
				os << "		" << ntofs(i+1,2,' ') << "		"
				   << ntofs(samples[i].position) << "	  ";
				for(int j=0; j<5; j++)
				{
					os << ntofs(samples[i].H[j]) << " ";
				}
				os << endl;
			}
		}

	//
	// prepare_images
	//

	void prepare_images(void)
		{

			spinner.reset(new Spinner("Preparing image data"));

			uint16_be lo[5];
			uint16_be hi[5];
			for(int i=0; i<5; i++)
			{
				lo[i] = 65535;
				hi[i] = 0;
			}

			while(source.get_current_record())
			{
				data = reinterpret_cast<NOAA_L1b_MHS_Data_v3*>(
					source.get_current_record());
				ScanLine scan_line;

				for(int j=0; j<90; j++)
				{
					for(int k=0; k<5; k++)
					{
						uint16_be s = data->sensor_data[j].H[k];
						if (s>hi[k])
						{
							hi[k] = s;
						}
						if (s!=0 && s<lo[k])
						{
							lo[k] = s;
						}
						scan_line.samples[j].H[k] = s;
					}
				}

				source.skip_records(1);
				scan_lines.push_back(scan_line);
				spinner->tick();
			}

			spinner->end();

			// sort out normalisation factors
			for(int i=0; i<5; i++)
			{
				offset[i] = static_cast<double>(lo[i]);
				gain[i] = 255.0/(static_cast<double>(hi[i])-
								 static_cast<double>(lo[i]));
				info << "channel " << i << " lo " << lo[i] << " hi " << hi[i]
					 << " offset " << offset[i] << " gain " << gain[i] << endl;
			}
		}

	//
	// get_image_count
	//

	int get_image_count(void)
		{
			return 5;
		}

	//
	// write_image
	//

	void write_image(int number, Image* image)
		{
			spinner.reset(new Spinner("Assembling image"));

			image->set_filename_suffix(string(".")+ntos(number+1));
			image->set_attributes(Image::IMAGE_MONO, 90, scan_lines.size());
			image->begin_output();

			byte row[90];
			for(ScanLines::iterator iter=scan_lines.begin();
				iter!=scan_lines.end();
				iter++)
			{
				for(int x=0; x<90; x++)
				{
					row[x] = static_cast<byte>(
						(static_cast<double>((*iter).samples[x].H[number])-
						 offset[number])*gain[number]);
				}
				image->write_line(row);

				spinner->tick();

			}

			image->end_output();
			spinner->end();

		}

	//
	// get_next_ccsds
	//

	CCSDS* get_next_ccsds(void)
		{
			if (!source.get_current_record())
			{
				if (spinner.get())
				{
					spinner->end();
				}
				return NULL;
			}

			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			}
			spinner->tick();

			end_scan_line = data->gen.scan_line_number;
			end_year = data->gen.scan_line_year;
			end_day = data->gen.scan_line_day_of_year;
			end_millisecond = data->gen.scan_line_millisecond_of_day;

			// copy scan line number into sequence number if configured
			if (Config::scan_line_number_as_sequence_count)
			{
				ccsds_set_sequence_count(data->gen.scan_line_number+
										 Config::initial_sequence_count);
			}

			// prepare source packet
			ccsds_initialise();

			// set utc time
			ccsds_set_time(data->gen);

			// set OBT time if configured to transfer it
			// (otherwise ccsds_set_time() calculates one)

			if (Config::copy_obt_times)
			{
				ccsds_set_obt_time(data->coarse_time, data->fine_time);
			}

			// fill the mode field

			ccsds_packet.payload.mode = data->mode;

			// fill the telecommand acknowledgement field

			for(int i=0; i<5; i++)
			{
				ccsds_packet.payload.telecommand[i] = data->telecommand[i];
			}

			// fill switch status section

			for(int i=0; i<3; i++)
			{
				ccsds_packet.payload.switch_status[i] =
					data->switch_status[i];
			}

			// fill temperature measurements

			for(int i=0; i<24; i++)
			{
				ccsds_packet.payload.temperature[i] = data->temperature[i];
			}

			// current values

			for(int i=0; i<6; i++)
			{
				ccsds_packet.payload.current[i] = data->current[i];
			}

			// fill status word

			ccsds_packet.payload.status_word = data->status_word;

			// fill signal processing status

			for(int i=0; i<5; i++)
			{
				ccsds_packet.payload.signal_status[i] =
					data->offset_words[i];
			}

			ccsds_packet.payload.signal_status[5] = data->valid_flags;

			for(int i=0; i<3; i++)
			{
				ccsds_packet.payload.signal_status[i+6] = data->channel_gain[i];
			}

			// fill sample data
			// earth view

			for(int i=0; i<90; i++)
			{
				ccsds_packet.payload.earth_view[i].shaft_pos =
					data->sensor_data[i].position;
				for(int j=0; j<5; j++)
				{
					ccsds_packet.payload.earth_view[i].channels[j] =
						data->sensor_data[i].H[j];
				}
			}

			// space view

			for(int i=0; i<4; i++)
			{
				ccsds_packet.payload.space_view[i].shaft_pos =
					data->space_view[i].position;
				for(int j=0; j<5; j++)
				{
					ccsds_packet.payload.space_view[i].channels[j] =
						data->space_view[i].H[j];
				}
			}

			// obct view

			for(int i=0; i<4; i++)
			{
				ccsds_packet.payload.obct_view[i].shaft_pos =
					data->target_view[i].position;
				for(int j=0; j<5; j++)
				{
					ccsds_packet.payload.obct_view[i].channels[j] =
						data->target_view[i].H[j];
				}
			}

			// fill obct values

			for(int i=0; i<5; i++)
			{
				ccsds_packet.payload.prt[i] = data->prt[i];
			}

			for(int i=0; i<3; i++)
			{
				ccsds_packet.payload.prt_cal[i] = data->prt_cal[i];
			}

			// prepare next line

			source.skip_records(1);
			data = reinterpret_cast<NOAA_L1b_MHS_Data_v3*>(
				source.get_current_record());

			scan_line_count++;
			packets_generated++;

			// compute PEC value and finish

			return ccsds_finalise();
		}

	EPSProducer* as_eps_product(void)
		{
			return NULL;
		}

	//
	// output_text_summary
	//

	void output_text_summary( ostream& os = cout )
		{
			string start_time = TimeConv::abs_year_day_millisecond_to_string(
				start_year, start_day, start_millisecond);

			string end_time = TimeConv::abs_year_day_millisecond_to_string(
				end_year, end_day, end_millisecond);

			os << "MHSReader report\n"
			   << "	   " << scan_line_count << " scan lines processed ("
			   << start_scan_line << "-" << end_scan_line << ")\n"
			   << "	   Product start time " << start_time << endl
			   << "	   Product end time " << end_time << endl
			   << endl;
		}

protected:

	ProductProducer& source;
	NOAA_L1b_MHS_Header_v3* header;
	NOAA_L1b_MHS_Data_v3* data;

	struct ScanLine
	{
		struct Sample
		{
			uint16_be H[5];
		} samples[90];
	};
	typedef deque<ScanLine> ScanLines;
	ScanLines scan_lines;

	double offset[5];
	double gain[5];

	int start_scan_line;
	int start_year;
	int start_day;
	int start_millisecond;

	int end_scan_line;
	int end_year;
	int end_day;
	int end_millisecond;

	int scan_line_count;
	int packets_generated;
};

#endif
