#include "mmam.hpp"
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <cstring>
#include <algorithm>

#include <libxml/valid.h>
#include <libxml/xmlschemas.h>

#include <dlfcn.h>

using namespace std;

void* _bz2_library = NULL;

typedef int (*BZ2_BufferDecompress)(char*         dest,
									unsigned int* destLen,
									char*         source,
									unsigned int  sourceLen,
									int           small,
									int           verbosity );
BZ2_BufferDecompress decompress_buffer;


//--------------------------------------------------------------------------
// get_bz2_library
//--------------------------------------------------------------------------
void* get_bz2_library()
{

	/**
	 * We load the BZ2_bzBuffToBuffDecompress function dinamically to avoid
	 * adding the library libbz2 as a dependency to the whole metopizer suite.
	 *
	 * This will fail at runtime in all systems that do not have libbz2 installed.
	 */
	
	if (!_bz2_library)
	{
		_bz2_library = dlopen("libbz2.so", RTLD_LAZY);
		if (!_bz2_library)
		{
			fprintf (stderr, "%s\n", dlerror());
			cerr << "Cannot open bz2 library (" << dlerror() << ")\n";
			exit(-1);
		}

		decompress_buffer = reinterpret_cast<BZ2_BufferDecompress>(dlsym(_bz2_library, "BZ2_bzBuffToBuffDecompress"));
		if (!decompress_buffer)
		{
			cerr << "Cannot find function BZ2_bzBuffToBuffDecompress\n";
			exit(-1);
		}
	}
	return _bz2_library;
}


//--------------------------------------------------------------------------
// bz2_decode
//--------------------------------------------------------------------------
int bz2_decode(char*   dest,
				unsigned int* destLen,
				char*         source,
				unsigned int  sourceLen,
				int           small,
				int           verbosity )
{
	get_bz2_library();
	int result = decompress_buffer(dest, destLen, source, sourceLen, small, verbosity);
	return result;
}


//--------------------------------------------------------------------------
// timeStampConvert
//--------------------------------------------------------------------------
string timeStampConvert(const string& aTimestamp)
{
    //we dont want to change the original value
    string temp = aTimestamp;

    //convert timestamp format
    if (temp.length() == 23)
    {
        //2014-01-09T17:59:57.000 ==> 20140109175957000Z
        char chars[] = "-T:.";

        for (unsigned int i = 0; i < strlen(chars); ++i)
        {
           temp.erase (std::remove(temp.begin(), temp.end(), chars[i]), temp.end());
        }
        temp = temp + "Z";
    }

    return temp;
}


/**
 * @brief BUFSIZE
 *
 *  In principle the size of the (compressed) MMAM message is 8832 bytes.
 *  Once decompressed the size of the (example) MMAM is just 47338 bytes.
 *  131061 is the minimum buffer size to decompress it. Why?
 *  I am not sure why 'BZ2_bzBuffToBuffDecompress' needs a bigger buffer.
 *
 *  Since memory is likely not be an issue, to be on the safe side we can
 *  accomodate a bigger buffer in case a bigger MMAM arrives...
 *
 */

//BZ2 compresion stuff
static unsigned int BZ_BUFSIZE = 131061*2;
static const char* BZFILE_START_SYMBOL = "BZh";
static unsigned int BZFILE_START = 27;

//SATELLITE NAMES
static const char* METOP_A = "Metop-A";
static const char* METOP_B = "Metop-B";
static const char* METOP_C = "Metop-C";
static const char* M01 = "M01";
static const char* M02 = "M02";
static const char* M03 = "M03";

//MMAM xml tags
static const char* MMAM_ROOT_ELEM = "multi-mission-administrative-message";
static const char* MMAM_MSG_ELEM = "message";
static const char* MMAM_NAVIGATION_ELEM = "navigation";
static const char* MMAM_PROCESSING_ELEM = "processing";
static const char* MMAM_SATELLITE_ELEM =  "satellite";

//MMAM state vector tags
static const char* MMAM_STATEVECTORS_ELEM = "statevectors-at-anx";
static const char* MMAM_ORBIT_EPHEMERIS_ELEM = "orbit-ephemeris";
static const char* MMAM_VALIDFROM_ATTR = "valid-from";
static const char* MMAM_VALIDUNTIL_ATTR = "valid-until";
static const char* MMAM_STATEVECTOR_AT_ANX_ELEM = "statevector-at-anx";
static const char* MMAM_STATEVECTOR_ELEM = "statevector";
static const char* MMAM_SEMI_MAJOR_AXIS_ELEM = "semi-major-axis";
static const char* MMAM_ECCENTRICITY_ELEM = "eccentricity";
static const char* MMAM_INCLINATION_ELEM = "inclination";
static const char* MMAM_PERIGEE_ELEM = "perigee";
static const char* MMAM_RIGHT_ASCENSION_ELEM = "right-ascension";
static const char* MMAM_MEAN_ANOMALY_ELEM = "mean-anomaly";
static const char* MMAM_X_POS_ELEM = "x-pos";
static const char* MMAM_Y_POS_ELEM = "y-pos";
static const char* MMAM_Z_POS_ELEM = "z-pos";
static const char* MMAM_X_VEL_ELEM = "x-vel";
static const char* MMAM_Y_VEL_ELEM = "y-vel";
static const char* MMAM_Z_VEL_ELEM = "z-vel";

////OBT UTC correlation
static const char* MMAM_OBT_UTC_CORRELATION_ELEM = "obt-utc-correlation";
static const char* MMAM_UTC0_ELEM = "utc-0";
static const char* MMAM_CCU_OBT0_ELEM = "ccu-obt-0";
static const char* MMAM_CLOCK_STEP_ELEM = "clock-step";
static const char* MMAM_ESTIMATED_WRAPAROUND_TIME_ELEM = "estimated-obt-utc-wrap-around-time";
static const char* MMAM_EPOCH_ATTR = "epoch";
static const char* MMAM_ORBIT_ATTR = "orbit";

//TLE state vector tags
static const char* MMAM_TLE_ELEM = "two-line-elements";
static const char* MMAM_TLE_LINE1_ELEM = "line-1";
static const char* MMAM_TLE_LINE2_ELEM = "line-2";
static const char* MMAM_ISSUE_NUMBER = "issue-number";

//output files prefix- and postfixes
static const char* MMAM_FILE_PREFIX = "MMAM";
static const char* MMAM_FILE_EXTENSION = "xml";
static const char* TLE_FILE_PREFIX = "TLE";
static const char* TLE_FILE_EXTENSION = "tle";

//MMAM validation schema to validate against
static const char* MMAM_SCHEMA_FILE = "MMAM.xsd";

//--------------------------------------------------------------------------
// fileExists
//--------------------------------------------------------------------------
bool fileExists(const string& aFilename)
{
    DIR *dir;
    struct dirent *entry;
    if ((dir = opendir (".")) != NULL)
    {
        //find MMAM file
        while ((entry = readdir (dir)) != NULL)
        {
            string myEntryName = entry->d_name;
            if (myEntryName.find(aFilename) == 0)
            {
                closedir (dir);
                return true;
            }
        }
        closedir (dir);
    }

    return false;
}

//--------------------------------------------------------------------------
// MMAMhandler::MMAM()
//--------------------------------------------------------------------------
MMAMhandler::MMAMhandler(bool aGenerateTLEfile)
    :m_generateTLE(aGenerateTLEfile),m_MMAMbuf(0),m_xmlDoc(0)
{
}


//--------------------------------------------------------------------------
// MMAMhandler::~MMAM()
//--------------------------------------------------------------------------
MMAMhandler::~MMAMhandler()
{
    if (m_xmlDoc)
    {
        xmlFreeDoc(m_xmlDoc);
        m_xmlDoc = 0;
    }
    if (m_MMAMbuf)
    {
        delete m_MMAMbuf;
        m_MMAMbuf = 0;
    }
}

//--------------------------------------------------------------------------
// MMAMhandler::generateMMAMfileFromBuf()
//--------------------------------------------------------------------------
void MMAMhandler::generateMMAMfileFromBuf(const string& outdir)
{
    if (m_MMAMbuf)
    {
        //this is the name of the MMAM file we will create
        string sIssueNumber = m_issueNumber;
        while (sIssueNumber.length() < 6)
        	 sIssueNumber = "0" + sIssueNumber;
        string myFileName = outdir + (string)MMAM_FILE_PREFIX + "_" + m_spacecraftID + "_" +
                sIssueNumber + "_" + m_mmamValidityFrom.to_gen_time()  + "_" +
                m_mmamValidityUntil.to_gen_time() + "." + MMAM_FILE_EXTENSION;

        //check if the same file already exists to avoid creating it again
        if (!fileExists(myFileName))
        {
            //write the converted buffer into a file that can later be parse using an XML library
            ofstream myFile;

            //I need data to come up with a suitable name for the files
            myFile.open(myFileName.c_str());

            if (myFile.is_open())
            {
                //write the decompress buffer coming from the admin ISPs
                myFile.write(m_MMAMbuf, m_MMAMbufSize);
                myFile.close();
            }
            else
            {
                cerr << "Error: Could not open the MMAM file for writing" << endl;
            }
        }
    }
    else
    {
        cout << "Cannot generate MMAM because the buffer is empty" << endl;
    }
}

//--------------------------------------------------------------------------
// MMAMhandler::generateTLEfile()
//--------------------------------------------------------------------------
void MMAMhandler::generateTLEfile(const string& outdir)
{
    //this is the name of the TLE file we will create
    string sIssueNumber = m_issueNumber;
    while (sIssueNumber.length() < 6)
    	 sIssueNumber = "0" + sIssueNumber;
    string myFileName = outdir + (string)TLE_FILE_PREFIX + "_" +  m_spacecraftID + "_"  +
            sIssueNumber + "_" + m_mmamValidityFrom.to_gen_time()  + "_" +
            m_mmamValidityUntil.to_gen_time() + "." + TLE_FILE_EXTENSION;

    //check if the same file already exists to avoid creating it again
    if (!fileExists(myFileName))
    {
        //write the converted buffer into a file that can later be parse using an XML library
        ofstream myFile;

        //I need data to come up with a suitable name for the files
        myFile.open(myFileName.c_str());

        if (myFile.is_open())
        {
            //write the decompress buffer coming from the admin ISPs
            myFile << m_TLElines;
            myFile.close();
        }
        else
        {
            cerr << "Error: Could not open the TLE file for writing" << endl;
        }
    }
}

//--------------------------------------------------------------------------
// MMAMhandler::tagNameMatchesText()
//--------------------------------------------------------------------------
bool MMAMhandler::tagNameMatchesText(const xmlChar* aTagName, const char* aText)
{
    if ((!xmlStrcmp(aTagName, (const xmlChar *)aText)))
    {
        return true;
    }
    return false;
}


//--------------------------------------------------------------------------
// MMAMhandler::getNodeAttribute()
//--------------------------------------------------------------------------
string MMAMhandler::getNodeAttribute(xmlNodePtr aNode, const string& aName)
{
    xmlChar* myAtrib = xmlGetProp(aNode, (const xmlChar *)aName.c_str());
    string myAttrStr((const char*)myAtrib);
    xmlFree(myAtrib);
    return myAttrStr;
}

//--------------------------------------------------------------------------
// MMAMhandler::getNodeValue()
//--------------------------------------------------------------------------
string MMAMhandler::getNodeValue(xmlNodePtr aNode)
{
    xmlChar* myValue = xmlNodeListGetString(m_xmlDoc, aNode->xmlChildrenNode, 1);
    string myValueStr((const char*)myValue);
    xmlFree(myValue);
    return myValueStr;
}

//--------------------------------------------------------------------------
// MMAMhandler::parseMessage()
//--------------------------------------------------------------------------
void MMAMhandler::parseMessage (xmlNodePtr aNode)
{
    //add Sat name to TLE buffer

    if (m_spacecraftID.empty())
    {
        string mySatName = getNodeAttribute(aNode, MMAM_SATELLITE_ELEM);
        if (mySatName == METOP_A) m_spacecraftID = M02;
        else if (mySatName == METOP_B) m_spacecraftID = M01;
        else if (mySatName == METOP_C) m_spacecraftID = M03;
    }

    m_TLElines += getNodeAttribute(aNode, MMAM_SATELLITE_ELEM) + "\n";

    aNode = aNode->xmlChildrenNode;
    while (aNode != 0)
    {
        if (tagNameMatchesText(aNode->name, MMAM_NAVIGATION_ELEM))
        {
            //parse Two line elements
            parseTLEs (aNode);

			//retrieve State vector At ANX
			parseStateVectorAtANX(aNode);

			//retrieve State vectors
			parseStateVector(aNode);
        }

        if (tagNameMatchesText(aNode->name, MMAM_PROCESSING_ELEM))
        {
            //parse Two line elements
            parseObtUtcCorrelation(aNode);
        }
        aNode = aNode->next;
    }
}

//--------------------------------------------------------------------------
// MMAMhandler::parseTLEs()
//--------------------------------------------------------------------------
void MMAMhandler::parseTLEs (xmlNodePtr cur)
{
    xmlChar *key;
    cur = cur->xmlChildrenNode;
    while (cur != NULL)
    {
        if (tagNameMatchesText(cur->name, MMAM_TLE_ELEM))
        {

            xmlNodePtr children = cur->xmlChildrenNode;
            while (children != NULL)
            {
                if (tagNameMatchesText(children->name, MMAM_TLE_LINE1_ELEM))
                {
                    m_TLElines += getNodeValue(children);
                    m_TLElines += "\n";
                }

                if (tagNameMatchesText(children->name, MMAM_TLE_LINE2_ELEM))
                {
                    m_TLElines += getNodeValue(children);
                    m_TLElines += "\n";
                }
                children = children->next;
            }
        }

        cur = cur->next;
    }
}


//--------------------------------------------------------------------------
// MMAMhandler::parseStateVectorAtANX()
//--------------------------------------------------------------------------
void MMAMhandler::parseStateVectorAtANX(xmlNodePtr cur)
{
    xmlChar *key;
    cur = cur->xmlChildrenNode;
    while (cur != NULL)
    {
        if (tagNameMatchesText(cur->name, MMAM_STATEVECTORS_ELEM))
        {
            xmlNodePtr children2 = cur->xmlChildrenNode;

            //get MMAM state vectors validity
            m_mmamValidityFrom = make_short_cds_time(getNodeAttribute(cur, MMAM_VALIDFROM_ATTR));
            m_mmamValidityUntil = make_short_cds_time(getNodeAttribute(cur, MMAM_VALIDUNTIL_ATTR));

            while (children2 != NULL)
            {
				if (tagNameMatchesText(children2->name, MMAM_STATEVECTOR_AT_ANX_ELEM))
                {
                    OSVS::OSV myOSV;

                    xmlNodePtr children = children2->xmlChildrenNode;
                    while (children != NULL)
                    {
                        if (tagNameMatchesText(children->name, MMAM_SEMI_MAJOR_AXIS_ELEM))
                        {
                            //MMAM semi_major_axis in Km, we must store them in mm (milimetres)
                            myOSV.semi_major_axis = 1000000 * strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_ECCENTRICITY_ELEM))
                        {
                            myOSV.eccentricity = strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_INCLINATION_ELEM))
                        {
                            myOSV.inclination = strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_PERIGEE_ELEM))
                        {
                            myOSV.perigee_argument = strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_RIGHT_ASCENSION_ELEM))
                        {
                            myOSV.right_ascension = strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_MEAN_ANOMALY_ELEM))
                        {
                            myOSV.mean_anomaly = strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_X_POS_ELEM))
                        {
                            //MMAM positions in Km, we must store them in m (metres)
                            myOSV.x = 1000 * strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_Y_POS_ELEM))
                        {
                            //MMAM positions in Km, we must store them in m (metres)
                            myOSV.y = 1000 * strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_Z_POS_ELEM))
                        {
                            //MMAM positions in Km, we must store them in m (metres)
                            myOSV.z = 1000 * strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_X_VEL_ELEM))
                        {
                            //MMAM velocities in Km/s, we must store them in m/s
                            myOSV.vx = 1000 * strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_Y_VEL_ELEM))
                        {
                            //MMAM velocities in Km/s, we must store them in m/s
                            myOSV.vy = 1000 * strtod(getNodeValue(children).c_str(),0);
                        }
                        else if (tagNameMatchesText(children->name, MMAM_Z_VEL_ELEM))
                        {
                            //MMAM velocities in Km/s, we must store them in m/s
                            myOSV.vz = 1000 * strtod(getNodeValue(children).c_str(),0);
                        }

                        children = children->next;
                    }

                    //get orbit number
                    string myOrbitNumber = getNodeAttribute(children2,MMAM_ORBIT_ATTR);
                    myOSV.orbit_number = atoi(myOrbitNumber.c_str());

                    //save state vector in map
                    string myEpoch = getNodeAttribute(children2,MMAM_EPOCH_ATTR);
                    short_cds_time myCds = make_short_cds_time(myEpoch);
                    myOSV.state_vector_time = myCds;
					m_osvs_anx.add_osv(myCds, myOSV);
                }

                children2 = children2->next;
            }
        }

        cur = cur->next;
    }
}


//--------------------------------------------------------------------------
// MMAMhandler::parseStateVector()
//--------------------------------------------------------------------------
void MMAMhandler::parseStateVector(xmlNodePtr cur)
{
	xmlChar *key;
	cur = cur->xmlChildrenNode;
	while (cur != NULL)
	{
		if (tagNameMatchesText(cur->name, MMAM_ORBIT_EPHEMERIS_ELEM))
		{
			xmlNodePtr children2 = cur->xmlChildrenNode;

			//get MMAM state vectors validity
			m_mmamValidityFrom = make_short_cds_time(getNodeAttribute(cur, MMAM_VALIDFROM_ATTR));
			m_mmamValidityUntil = make_short_cds_time(getNodeAttribute(cur, MMAM_VALIDUNTIL_ATTR));

			while (children2 != NULL)
			{
				if (tagNameMatchesText(children2->name, MMAM_STATEVECTOR_ELEM))
				{
					OSVS::OSV myOSV;

					xmlNodePtr children = children2->xmlChildrenNode;
					while (children != NULL)
					{
						if (tagNameMatchesText(children->name, MMAM_X_POS_ELEM))
						{
							//MMAM positions in Km, we must store them in m (metres)
							myOSV.x = 1000 * strtod(getNodeValue(children).c_str(),0);
						}
						else if (tagNameMatchesText(children->name, MMAM_Y_POS_ELEM))
						{
							//MMAM positions in Km, we must store them in m (metres)
							myOSV.y = 1000 * strtod(getNodeValue(children).c_str(),0);
						}
						else if (tagNameMatchesText(children->name, MMAM_Z_POS_ELEM))
						{
							//MMAM positions in Km, we must store them in m (metres)
							myOSV.z = 1000 * strtod(getNodeValue(children).c_str(),0);
						}
						else if (tagNameMatchesText(children->name, MMAM_X_VEL_ELEM))
						{
							//MMAM velocities in Km/s, we must store them in m/s
							myOSV.vx = 1000 * strtod(getNodeValue(children).c_str(),0);
						}
						else if (tagNameMatchesText(children->name, MMAM_Y_VEL_ELEM))
						{
							//MMAM velocities in Km/s, we must store them in m/s
							myOSV.vy = 1000 * strtod(getNodeValue(children).c_str(),0);
						}
						else if (tagNameMatchesText(children->name, MMAM_Z_VEL_ELEM))
						{
							//MMAM velocities in Km/s, we must store them in m/s
							myOSV.vz = 1000 * strtod(getNodeValue(children).c_str(),0);
						}

						children = children->next;
					}

					//save state vector in map
					string myEpoch = getNodeAttribute(children2,MMAM_EPOCH_ATTR);
					short_cds_time myCds = make_short_cds_time(myEpoch);
					myOSV.state_vector_time = myCds;
					m_osvs.add_osv(myCds, myOSV);
				}

				children2 = children2->next;
			}
		}

		cur = cur->next;
	}
}

//--------------------------------------------------------------------------
// MMAMhandler::parseObtUtcCorrelation()
//--------------------------------------------------------------------------
void MMAMhandler::parseObtUtcCorrelation(xmlNodePtr cur)
{
    xmlChar *key;
    cur = cur->xmlChildrenNode;
    while (cur != NULL)
    {
        if (tagNameMatchesText(cur->name, MMAM_OBT_UTC_CORRELATION_ELEM))
        {
            m_UTC0 = getNodeAttribute(cur, MMAM_UTC0_ELEM);
            m_CCU_OBT0 = getNodeAttribute(cur, MMAM_CCU_OBT0_ELEM);
            m_ClockStep = getNodeAttribute(cur, MMAM_CLOCK_STEP_ELEM);
            m_WrapAroundTime = getNodeAttribute(cur, MMAM_ESTIMATED_WRAPAROUND_TIME_ELEM);
        }

        cur = cur->next;
    }
}

//--------------------------------------------------------------------------
// MMAMhandler::parseFromBuf()
//--------------------------------------------------------------------------
bool MMAMhandler::parseFromBuf(char* aMmamBuffer, unsigned int aSourceLen)
{
    //start position of the MMAM in the ISP (according to TM format)
    int myMMAMstartPos = BZFILE_START;

//    //look for the start of the compressed MMAM file in the ISP
//    for (int i=0; i< aSourceLen; i++)
//    {
//        if (strncmp(aMmamBuffer+i, BZFILE_START_SYMBOL, 3) == 0)
//        {
//            myMMAMstartPos = i;
//        }
//    }

    //now that we know where the MMAM compressed file is we decompress it
    m_MMAMbuf = new char[BZ_BUFSIZE];
    m_MMAMbufSize = BZ_BUFSIZE;
    int result = bz2_decode(m_MMAMbuf, &m_MMAMbufSize, aMmamBuffer + myMMAMstartPos,
                            aSourceLen - myMMAMstartPos, 0, 0);

    if (result != 0)  //error handling
    {
        cout << "Could not decompress bz2 buffer. Error code: " << result << endl;
        return false;
    }
    else
    {
        m_xmlDoc = xmlParseMemory(m_MMAMbuf, m_MMAMbufSize);
        return parseMMAM();
    }
}

//--------------------------------------------------------------------------
// MMAMhandler::parseFromFile()
//--------------------------------------------------------------------------
bool MMAMhandler::parseFromFile(const string& m_aFileName)
{
    m_xmlDoc = xmlParseFile(m_aFileName.c_str());
    return parseMMAM();
}

//--------------------------------------------------------------------------
// MMAMhandler::parseMMAM()
//--------------------------------------------------------------------------
bool MMAMhandler::parseMMAM()
{
    if (m_xmlDoc == 0)
    {
        cerr << "The MMAM document was not parsed successfully" << endl;
        return false;
    }

    xmlNodePtr cur = xmlDocGetRootElement(m_xmlDoc);

    if (cur == NULL)
    {
        cerr << "This MMAM document is empty" << endl;
        return false;
    }

    if (!tagNameMatchesText(cur->name, MMAM_ROOT_ELEM))
    {
        cerr << "This document does not seem to be a valid MMAM message" << endl;
        return false;
    }

    //we will get the "issue-number" attribute to name the produced MMAM file
    m_issueNumber = getNodeAttribute(cur, MMAM_ISSUE_NUMBER);

    cur = cur->xmlChildrenNode;
    while (cur != NULL)
    {
        if (tagNameMatchesText(cur->name, MMAM_MSG_ELEM))
        {
            parseMessage(cur);
        }
        cur = cur->next;
    }

    return true;
}

//--------------------------------------------------------------------------
// MMAMhandler::getUTC0()
//--------------------------------------------------------------------------
string MMAMhandler::getUTC0()
{
    return timeStampConvert(m_UTC0);
}
//--------------------------------------------------------------------------
// MMAMhandler::getOBT0()
//--------------------------------------------------------------------------
string MMAMhandler::getOBT0()
{
    return m_CCU_OBT0;
}

//--------------------------------------------------------------------------
// MMAMhandler::getClockStep()
//--------------------------------------------------------------------------
string MMAMhandler::getClockStep()
{
    return m_ClockStep;
}

//--------------------------------------------------------------------------
// MMAMhandler::getWrapAroundTime()
//--------------------------------------------------------------------------
string MMAMhandler::getWrapAroundTime()
{
    return timeStampConvert(m_WrapAroundTime);
}

//--------------------------------------------------------------------------
// MMAMhandler::getStateVectorsAtANX()
//--------------------------------------------------------------------------
OSVS MMAMhandler::getStateVectorsAtANX()
{
	return m_osvs_anx;
}

//--------------------------------------------------------------------------
// MMAMhandler::getStateVectors()
//--------------------------------------------------------------------------
OSVS MMAMhandler::getStateVectors()
{
    return m_osvs;
}

//--------------------------------------------------------------------------
// MMAMhandler::checkTimeValidity()
//--------------------------------------------------------------------------
bool MMAMhandler::checkTimeValidity(const short_cds_time& aTimestamp)
{
    if (aTimestamp <= m_mmamValidityUntil &&
        aTimestamp >= m_mmamValidityFrom)
    {
        return true;
    }
    else
    {
        return false;
    }
}

//--------------------------------------------------------------------------
// MMAMhandler::schemaValidation()
//--------------------------------------------------------------------------
bool MMAMhandler::schemaValidation(const string& aSchema)
{
    if (m_xmlDoc)
    {
        xmlSchemaParserCtxtPtr mParserCtxt = xmlSchemaNewParserCtxt(aSchema.c_str());
        if (mParserCtxt == NULL)
        {
            cerr << "Could not create XSD schema parsing context." << endl;
            return false;
        }

        xmlSchemaPtr mySchema = xmlSchemaParse(mParserCtxt);
        if (mySchema == NULL)
        {
            cerr << "Could not parse XSD schema." << endl;
            return false;
        }

        xmlSchemaValidCtxtPtr myValidCtxt = xmlSchemaNewValidCtxt(mySchema);
        if (!myValidCtxt)
        {
            cerr << "Could not create XSD schema validation context." << endl;
            return false;
        }

        int result = xmlSchemaValidateDoc(myValidCtxt, m_xmlDoc);
        if (result != 0)
        {
            return false;
        }

        return true;
    }
    else
    {
        //there is no parsed MMAM yet, so return false
        cerr << "No MMAM file has been parsed yet, validation not possible." << endl;
        return false;
    }


}


