////////////////////////////////////////////////////////////////////////////////
//
//	File: mphr.cpp
//
//	Provides: fill_mphr()
//
//	Purpose: hr functions
//
//	Copyright (c) 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "eps.hpp"
#include "config.hpp"
#include "mphr.hpp"
#include <ctime>

#define Config Config::L0

const int FLAG_LEN = 1;
const int ASC2_LEN = 2;
const int ASC3_LEN = 3;
const int BYTE_LEN = 4;
const int ASC5_LEN = 5;
const int INT2_LEN = 6;
const int ASC8_LEN = 8;
const int INT4_LEN = 11;
const int GT_LEN = 15;
const int LGT_LEN = 18;
const int TXT_LEN = 67;

string fill_mphr(REC_HEAD& rec_head,
			   MPHR_body& mphr,
			   const string& config_file,
			   const string& instrument_id,
			   const short_cds_time& sensing_start,
			   const short_cds_time& sensing_end,
			   int total_sphr,
			   int total_ipr,
			   int total_geadr,
			   int total_giadr,
			   int total_veadr,
			   int total_viadr,
			   int total_mdr,
			   int actual_product_size,
               const Subsat::Pos& start_pos,
               const Subsat::Pos& end_pos,
			   const OSVS::OSV& osv_start,
			   const OSVS::OSV& osv_end)
{
	Config::file.read(config_file);
	
	// Initialise structure by filling it with blank spaces
	memset((void*)&rec_head,'\0', sizeof(rec_head));
	memset((void*)&mphr,' ', sizeof(mphr));

	// Read values into local fields

	string product_type = Config::product_type;
	string processing_level = "00";
	string spacecraft_id = Config::spacecraft_id;
	int instrument_model = Config::instrument_model;
	string processing_centre = Config::processing_centre;
	string processing_mode = Config::processing_mode;
	string disposition_mode = Config::disposition_mode;
	int processor_major_version = Config::processor_major_version;
	int processor_minor_version = Config::processor_minor_version;
    int format_major_version = 10;
	int format_minor_version = 0;
	string receiving_ground_station = Config::recieving_ground_station;
 
    //Get the current time as processing time
    time_t current_time;
    struct tm * time_info;
    char timeString[15];
    time(&current_time);
    time_info = localtime(&current_time);

    //The format we want is: 20020808121315Z
    strftime(timeString, 15, "%Y%m%d%H%M%S", time_info);
    string myProcessingTime(timeString);
    myProcessingTime+="Z";

    short_cds_time processing_time_start(myProcessingTime);
    short_cds_time processing_time_end(myProcessingTime);

	// receive time
	short_cds_time receive_time_start = sensing_start;
    short_cds_time receive_time_end = sensing_end;

	short_cds_time state_vector_time = osv_start.state_vector_time;
	long long semi_major_axis = static_cast<long long>(
		osv_start.semi_major_axis*MPHR::SEMI_MAJOR_AXIS_SF);
	int eccentricity =
		static_cast<int>(osv_start.eccentricity*MPHR::ECCENTRICITY_SF);
	int inclination =
		static_cast<int>(osv_start.inclination*MPHR::INCLINATION_SF);

	int perigee_argument =
		static_cast<int>(osv_start.perigee_argument*MPHR::PERIGEE_ARGUMENT_SF);
	int right_ascension = 
		static_cast<int>(osv_start.right_ascension*MPHR::RIGHT_ASCENSION_SF);
	int mean_anomaly = 
		static_cast<int>(osv_start.mean_anomaly*MPHR::MEAN_ANOMOLY_SF);
	long long x_position = 
		static_cast<long long>(osv_start.x*MPHR::POSITION_SF);

	long long y_position = 
		static_cast<long long>(osv_start.y*MPHR::POSITION_SF);
	long long z_position =
		static_cast<long long>(osv_start.z*MPHR::POSITION_SF);
	long long x_velocity = 
		static_cast<long long>(osv_start.vx*MPHR::VELOCITY_SF);
	long long y_velocity = 
		static_cast<long long>(osv_start.vy*MPHR::VELOCITY_SF);
	long long z_velocity = 
		static_cast<long long>(osv_start.vz*MPHR::VELOCITY_SF);
	int orbit_start = osv_start.orbit_number;
	int orbit_end = osv_end.orbit_number;
	
	const double PI =3.141592653589793238463;
	tm Djul_tm;
	time_t time = sensing_start.get_epoch();
	gmtime_r(&time, &Djul_tm);
	int Djul = Djul_tm.tm_yday;
	double ratio = 1 - 0.01673 * cos(0.9856 * (Djul-4) * PI/180.0);
	int earth_sun_distance_ratio = static_cast<int>(ratio * MPHR::EARTH_SUN_SF);

	int location_tolerance_radial = 0;
	int location_tolerance_crosstrack = 0;
	int location_tolerance_alongtrack = 0;


	int subsat_latitude_start =
		static_cast<int>(start_pos.lat * MPHR::SUBSAT_SF);
	int subsat_longitude_start =
		static_cast<int>(start_pos.lon * MPHR::SUBSAT_SF);
		
	int subsat_latitude_end =
		static_cast<int>(end_pos.lat * MPHR::SUBSAT_SF);
	int subsat_longitude_end =
		static_cast<int>(end_pos.lon * MPHR::SUBSAT_SF);

	int total_records = 1+total_sphr+total_geadr+total_giadr+
		total_viadr+total_veadr+total_mdr+total_ipr;

	// Determining the duration of the product
	int milliseconds_of_data_missing = 0;
	int duration = sensing_end-sensing_start;

	int count_degraded_inst_mdr = 0;
	int count_degraded_proc_mdr = 0;
	int count_degraded_inst_mdr_blocks = 0;
	int count_degraded_proc_mdr_blocks = 0;

    /**
        For the product stop time we must round up the time, so for
        instance if the products end at  01:05:59.363 we will write
        010600 (instead of 010559)
    */

    short_cds_time product_end_time = sensing_end;
    if (product_end_time.get_milliseconds() % 1000 > 0)
    {
        long long remainder = product_end_time.get_milliseconds() % 1000;
        long long millisecsToFullSecond = 1000 - remainder;
        product_end_time = product_end_time + millisecsToFullSecond;
    }

	string product_name =
		instrument_id + "_" +
		product_type + "_" +
		processing_level + "_" +
		spacecraft_id + "_" +
		sensing_start.to_gen_time() + "_" +
        product_end_time.to_gen_time() + "_" +
		processing_mode + "_" +
		disposition_mode + "_" +
		processing_time_start.to_gen_time();

	//
	// Fill MPHR Binary GRH Fields
	//
  
	rec_head.record_class = REC_HEAD::MPHR;
	rec_head.instrument_group = REC_HEAD::GENERIC;
	rec_head.record_subclass = 0;
	rec_head.record_subclass_version = 2;
	rec_head.record_size = MPHR_body::SIZE;

	rec_head.record_start_time = sensing_start;
	rec_head.record_stop_time = sensing_end;

	//
	// Fill MPHR Ascii Fields
	//

	char buff[200];
	char conv_string[200];
	string tmpStr;
    strncpy(mphr.product_name_label, "PRODUCT_NAME                  = ", 32);
	memset(buff,' ', sizeof(buff));
	//sprintf(conv_string, "%%%ds\n", TXT_LEN); // triggers heap overflow
	sprintf(conv_string, "%%%ds", TXT_LEN);     // in sprintf below
	sprintf(mphr.product_name, conv_string, product_name.c_str());
	mphr.product_name[TXT_LEN] = '\n';          // insert newline here
	
	strncpy(mphr.parent_product_name_1_label,	
            "PARENT_PRODUCT_NAME_1         = ", 32);
	memset(mphr.parent_product_name_1, 'x', 67);
	mphr.parent_product_name_1[TXT_LEN] = '\n';

	strncpy(mphr.parent_product_name_2_label,
            "PARENT_PRODUCT_NAME_2         = ", 32);
	memset(mphr.parent_product_name_2, 'x', 67);
	mphr.parent_product_name_2[TXT_LEN] = '\n';

	strncpy(mphr.parent_product_name_3_label,
            "PARENT_PRODUCT_NAME_3         = ", 32);
	memset(mphr.parent_product_name_3, 'x', 67);
	mphr.parent_product_name_3[TXT_LEN] = '\n';

	strncpy(mphr.parent_product_name_4_label,
            "PARENT_PRODUCT_NAME_4         = ", 32);
	memset(mphr.parent_product_name_4, 'x', 67);
	mphr.parent_product_name_4[TXT_LEN] = '\n';

	strncpy(mphr.instrument_id_label,
            "INSTRUMENT_ID                 = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", BYTE_LEN, "\n");
	sprintf(buff, conv_string, instrument_id.c_str());
	memcpy(mphr.instrument_id, buff, sizeof(mphr.instrument_id));

	strncpy(mphr.instrument_model_label,
            "INSTRUMENT_MODEL              = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%% %dd%s", ASC3_LEN, "\n");
	sprintf(buff, conv_string, instrument_model);
	memcpy(mphr.instrument_model, buff, sizeof(mphr.instrument_model));

	strncpy(mphr.product_type_label,
            "PRODUCT_TYPE                  = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", ASC3_LEN, "\n");
	sprintf(buff, conv_string, product_type.c_str());
	memcpy(mphr.product_type, buff, sizeof(mphr.product_type));

	strncpy(mphr.processing_level_label,
            "PROCESSING_LEVEL              = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", ASC2_LEN, "\n");
	sprintf(buff, conv_string, processing_level.c_str());
	memcpy(mphr.processing_level, buff, sizeof(mphr.processing_level));

	strncpy(mphr.spacecraft_id_label,
            "SPACECRAFT_ID                 = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", ASC3_LEN, "\n");
	sprintf(buff, conv_string, spacecraft_id.c_str());
	memcpy(mphr.spacecraft_id, buff, sizeof(mphr.spacecraft_id));

	strncpy(mphr.sensing_start_label,
			"SENSING_START                 = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	sprintf(buff, conv_string, sensing_start.to_gen_time().c_str());
	memcpy(mphr.sensing_start, buff, sizeof(mphr.sensing_start));

	strncpy(mphr.sensing_end_label,
			"SENSING_END                   = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	sprintf(buff, conv_string, product_end_time.to_gen_time().c_str());
	memcpy(mphr.sensing_end, buff, sizeof(mphr.sensing_end));

	strncpy(mphr.sensing_start_theoretical_label,
			"SENSING_START_THEORETICAL     = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	sprintf(buff, conv_string, sensing_start.to_gen_time().c_str());
	memcpy(mphr.sensing_start_theoretical, buff,
		   sizeof(mphr.sensing_start_theoretical));

	strncpy(mphr.sensing_end_theoretical_label,
			"SENSING_END_THEORETICAL       = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	sprintf(buff, conv_string, sensing_end.to_gen_time().c_str());
	memcpy(mphr.sensing_end_theoretical, buff,
		   sizeof(mphr.sensing_end_theoretical));

	strncpy(mphr.processing_centre_label,
			"PROCESSING_CENTRE             = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", BYTE_LEN, "\n");
	tmpStr.clear();
	tmpStr = processing_centre;
	sprintf(buff, conv_string, tmpStr.c_str());
	memcpy(mphr.processing_centre, buff, sizeof(mphr.processing_centre));

	strncpy(mphr.processor_major_version_label,
			"PROCESSOR_MAJOR_VERSION       = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%% %dd%s", ASC5_LEN, "\n");
	sprintf(buff, conv_string, processor_major_version);
	memcpy(mphr.processor_major_version, buff,
		   sizeof(mphr.processor_major_version));

	strncpy(mphr.processor_minor_version_label,
			"PROCESSOR_MINOR_VERSION       = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s %dd%s", "%", ASC5_LEN, "\n");
	sprintf(buff, conv_string, processor_minor_version);
	memcpy(mphr.processor_minor_version, buff,
		   sizeof(mphr.processor_minor_version));

	strncpy(mphr.format_major_version_label,
			"FORMAT_MAJOR_VERSION          = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%% %dd%s", ASC5_LEN, "\n");
	sprintf(buff, conv_string, format_major_version);
	memcpy(mphr.format_major_version, buff,
		   sizeof(mphr.format_major_version));

	strncpy(mphr.format_minor_version_label,
			"FORMAT_MINOR_VERSION          = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%% %dd%s", ASC5_LEN, "\n");
	sprintf(buff, conv_string, format_minor_version);
	memcpy(mphr.format_minor_version, buff,
		   sizeof(mphr.format_minor_version));

	strncpy(mphr.processing_time_start_label,
			"PROCESSING_TIME_START         = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	sprintf(buff, conv_string, processing_time_start.to_gen_time().c_str());
	memcpy(mphr.processing_time_start, buff,
		   sizeof(mphr.processing_time_start));

	strncpy(mphr.processing_time_end_label,
			"PROCESSING_TIME_END           = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	sprintf(buff, conv_string, processing_time_end.to_gen_time().c_str());
	memcpy(mphr.processing_time_end, buff,
		   sizeof(mphr.processing_time_end));

	strncpy(mphr.processing_mode_label,
			"PROCESSING_MODE               = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", FLAG_LEN, "\n");
	sprintf(buff, conv_string, processing_mode.c_str());
	memcpy(mphr.processing_mode, buff, sizeof(mphr.processing_mode));

	strncpy(mphr.disposition_mode_label,
			"DISPOSITION_MODE              = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", FLAG_LEN, "\n");
	sprintf(buff, conv_string, disposition_mode.c_str());
	memcpy(mphr.disposition_mode, buff, sizeof(mphr.disposition_mode));

	strncpy(mphr.receiving_ground_station_label,
			"RECEIVING_GROUND_STATION      = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", ASC3_LEN, "\n");
	tmpStr.clear();
	tmpStr = receiving_ground_station;
	sprintf(buff, conv_string, tmpStr.c_str());
	memcpy(mphr.receiving_ground_station, buff,
		   sizeof(mphr.receiving_ground_station));

	strncpy(mphr.receive_time_start_label,
			"RECEIVE_TIME_START            = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	sprintf(buff, conv_string, receive_time_start.to_gen_time().c_str());
	memcpy(mphr.receive_time_start, buff,
		   sizeof(mphr.receive_time_start));

	strncpy(mphr.receive_time_end_label,
			"RECEIVE_TIME_END              = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	sprintf(buff, conv_string, receive_time_end.to_gen_time().c_str());
	memcpy(mphr.receive_time_end, buff, sizeof(mphr.receive_time_end));

	strncpy(mphr.orbit_start_label,
			"ORBIT_START                   = ", 32);
	memset(buff,' ', sizeof(buff));
    sprintf(buff, "%5d\n", orbit_start);
	memcpy(mphr.orbit_start, buff, sizeof(mphr.orbit_start));

	strncpy(mphr.orbit_end_label,
			"ORBIT_END                     = ", 32);
	memset(buff,' ', sizeof(buff));
    sprintf(buff, "%5d\n", orbit_end);
	memcpy(mphr.orbit_end, buff, sizeof(mphr.orbit_end));

	strncpy(mphr.actual_product_size_label,
			"ACTUAL_PRODUCT_SIZE           = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, actual_product_size);
	memcpy(mphr.actual_product_size, buff,
		   sizeof(mphr.actual_product_size));

	strncpy(mphr.state_vector_time_label,
			"STATE_VECTOR_TIME             = ", 32);
	//sprintf(mphr.state_vector_time, "%s\n"        // also triggers
	sprintf(mphr.state_vector_time, "%18s",         // heap overflow
	    state_vector_time.to_long_gen_time().c_str());
	mphr.state_vector_time[18] = '\n';              // insert newline here  

	strncpy(mphr.semi_major_axis_label,
            "SEMI_MAJOR_AXIS               = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%%%dlld\n", INT4_LEN);
	sprintf(buff, conv_string, semi_major_axis);
	memcpy(mphr.semi_major_axis, buff, sizeof(mphr.semi_major_axis));

	strncpy(mphr.eccentricity_label,
			"ECCENTRICITY                  = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, eccentricity);
	memcpy(mphr.eccentricity, buff, sizeof(mphr.eccentricity));

	strncpy(mphr.inclination_label,
			"INCLINATION                   = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, inclination);
	memcpy(mphr.inclination, buff, sizeof(mphr.inclination));

	strncpy(mphr.perigee_argument_label,
			"PERIGEE_ARGUMENT              = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, perigee_argument);
	memcpy(mphr.perigee_argument,
		   buff,
		   sizeof(mphr.perigee_argument));

	strncpy(mphr.right_ascension_label,
			"RIGHT_ASCENSION               = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, right_ascension );
	memcpy(mphr.right_ascension, buff, sizeof(mphr.right_ascension));

	strncpy(mphr.mean_anomaly_label,
			"MEAN_ANOMALY                  = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, mean_anomaly );
	memcpy(mphr.mean_anomaly, buff, sizeof(mphr.mean_anomaly));

	strncpy(mphr.x_position_label,
			"X_POSITION                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dlld%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, x_position);
	memcpy(mphr.x_position, buff, sizeof(mphr.x_position));

	strncpy(mphr.y_position_label,
			"Y_POSITION                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dlld%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, y_position);
	memcpy(mphr.y_position, buff, sizeof(mphr.y_position));

	strncpy(mphr.z_position_label,
			"Z_POSITION                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dlld%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, z_position );
	memcpy(mphr.z_position, buff, sizeof(mphr.z_position));

	strncpy(mphr.x_velocity_label,
			"X_VELOCITY                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, x_velocity );
	memcpy(mphr.x_velocity, buff, sizeof(mphr.x_velocity));

	strncpy(mphr.y_velocity_label,
			"Y_VELOCITY                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, y_velocity);
	memcpy(mphr.y_velocity, buff, sizeof(mphr.y_velocity));

	strncpy(mphr.z_velocity_label,
			"Z_VELOCITY                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, z_velocity);
	memcpy(mphr.z_velocity, buff, sizeof(mphr.z_velocity));

	strncpy(mphr.earth_sun_distance_ratio_label,
			"EARTH_SUN_DISTANCE_RATIO      = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, earth_sun_distance_ratio);
	memcpy(mphr.earth_sun_distance_ratio, buff,
		   sizeof(mphr.earth_sun_distance_ratio));

	strncpy(mphr.location_tolerance_radial_label,
			"LOCATION_TOLERANCE_RADIAL     = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, location_tolerance_radial);
	memcpy(mphr.location_tolerance_radial, buff,
		   sizeof(mphr.location_tolerance_radial));

	strncpy(mphr.location_tolerance_crosstrack_label,
			"LOCATION_TOLERANCE_CROSSTRACK = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, location_tolerance_crosstrack);
	memcpy(mphr.location_tolerance_crosstrack, buff,
		   sizeof(mphr.location_tolerance_crosstrack));

	strncpy(mphr.location_tolerance_alongtrack_label,
			"LOCATION_TOLERANCE_ALONGTRACK = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, location_tolerance_alongtrack);
	memcpy(mphr.location_tolerance_alongtrack, buff,
		   sizeof(mphr.location_tolerance_alongtrack));
	
	strncpy(mphr.yaw_error_label,
			"YAW_ERROR                     = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, 0);
	memcpy(mphr.yaw_error, buff, sizeof(mphr.yaw_error));
	
	strncpy(mphr.roll_error_label,
			"ROLL_ERROR                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, 0);
	memcpy(mphr.roll_error, buff, sizeof(mphr.roll_error));
		
	strncpy(mphr.pitch_error_label,
			"PITCH_ERROR                   = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, 0);
	memcpy(mphr.pitch_error, buff, sizeof(mphr.pitch_error));
		
	strncpy(mphr.subsat_latitude_start_label,
			"SUBSAT_LATITUDE_START         = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, subsat_latitude_start);
	memcpy(mphr.subsat_latitude_start, buff,
		   sizeof(mphr.subsat_latitude_start));
		
	strncpy(mphr.subsat_longitude_start_label,
			"SUBSAT_LONGITUDE_START        = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, subsat_longitude_start);
	memcpy(mphr.subsat_longitude_start, buff,
		   sizeof(mphr.subsat_longitude_start));
		
	strncpy(mphr.subsat_latitude_end_label,
			"SUBSAT_LATITUDE_END           = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, subsat_latitude_end);
	memcpy(mphr.subsat_latitude_end, buff,
		   sizeof(mphr.subsat_latitude_end));
		
	strncpy(mphr.subsat_longitude_end_label,
			"SUBSAT_LONGITUDE_END          = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT4_LEN, "\n");
	sprintf(buff, conv_string, subsat_longitude_end);
	memcpy(mphr.subsat_longitude_end, buff,
		   sizeof(mphr.subsat_longitude_end));
		
	strncpy(mphr.leap_second_label,
			"LEAP_SECOND                   = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", ASC2_LEN, "\n");	
	sprintf(buff, conv_string, 0);
	memcpy(mphr.leap_second, buff, sizeof(mphr.leap_second));

	strncpy(mphr.leap_second_utc_label,
			"LEAP_SECOND_UTC               = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%ds%s", "%", GT_LEN, "\n");
	tmpStr.clear();
	tmpStr = make_short_cds_time().to_gen_time();
	sprintf(buff, conv_string, tmpStr.c_str());
	memcpy(mphr.leap_second_utc, buff, sizeof(mphr.leap_second_utc));

	strncpy(mphr.total_records_label,
			"TOTAL_RECORDS                 = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, total_records);
	memcpy(mphr.total_records, buff, sizeof(mphr.total_records));

	strncpy(mphr.total_mphr_label,
			"TOTAL_MPHR                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, 1);
	memcpy(mphr.total_mphr, buff, sizeof(mphr.total_mphr));

	strncpy(mphr.total_sphr_label,
			"TOTAL_SPHR                    = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, total_sphr);
	memcpy(mphr.total_sphr, buff, sizeof(mphr.total_sphr));

	strncpy(mphr.total_ipr_label,
			"TOTAL_IPR                     = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, total_ipr);
	memcpy(mphr.total_ipr, buff, sizeof(mphr.total_ipr));

	strncpy(mphr.total_geadr_label,
			"TOTAL_GEADR                   = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, total_geadr);
	memcpy(mphr.total_geadr, buff, sizeof(mphr.total_geadr));

	strncpy(mphr.total_giadr_label,
			"TOTAL_GIADR                   = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, total_geadr);
	memcpy(mphr.total_giadr, buff, sizeof(mphr.total_giadr));

	strncpy(mphr.total_veadr_label,
			"TOTAL_VEADR                   = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, total_veadr);
	memcpy(mphr.total_veadr, buff, sizeof(mphr.total_veadr));

	strncpy(mphr.total_viadr_label,
			"TOTAL_VIADR                   = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, total_viadr);
	memcpy(mphr.total_viadr, buff, sizeof(mphr.total_viadr));

	strncpy(mphr.total_mdr_label,
			"TOTAL_MDR                     = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, total_mdr);
	memcpy(mphr.total_mdr, buff, sizeof(mphr.total_mdr));

	strncpy(mphr.count_degraded_inst_mdr_label,
			"COUNT_DEGRADED_INST_MDR       = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, count_degraded_inst_mdr);
	memcpy(mphr.count_degraded_inst_mdr, buff,
		   sizeof(mphr.count_degraded_inst_mdr));

	strncpy(mphr.count_degraded_proc_mdr_label,
			"COUNT_DEGRADED_PROC_MDR       = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, count_degraded_proc_mdr);
	memcpy(mphr.count_degraded_proc_mdr, buff,
		   sizeof(mphr.count_degraded_proc_mdr));

	strncpy(mphr.count_degraded_inst_mdr_blocks_label,
			"COUNT_DEGRADED_INST_MDR_BLOCKS= ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, count_degraded_inst_mdr_blocks);
	memcpy(mphr.count_degraded_inst_mdr_blocks, buff,
		   sizeof(mphr.count_degraded_inst_mdr_blocks));

	strncpy(mphr.count_degraded_proc_mdr_blocks_label,
			"COUNT_DEGRADED_PROC_MDR_BLOCKS= ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", INT2_LEN, "\n");
	sprintf(buff, conv_string, count_degraded_proc_mdr_blocks);
	memcpy(mphr.count_degraded_proc_mdr_blocks, buff,
		   sizeof(mphr.count_degraded_proc_mdr_blocks));

	strncpy(mphr.duration_of_product_label,
			"DURATION_OF_PRODUCT           = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", ASC8_LEN, "\n");
	sprintf(buff, conv_string, duration);
	memcpy(mphr.duration_of_product, buff,
		   sizeof(mphr.duration_of_product));

	strncpy(mphr.milliseconds_of_data_present_label,
			"MILLISECONDS_OF_DATA_PRESENT  = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", ASC8_LEN, "\n");
	sprintf(buff, conv_string, duration);
	memcpy(mphr.milliseconds_of_data_present, buff,
		   sizeof(mphr.milliseconds_of_data_present));

	strncpy(mphr.milliseconds_of_data_missing_label,
			"MILLISECONDS_OF_DATA_MISSING  = ", 32);
	memset(buff,' ', sizeof(buff));
	sprintf(conv_string, "%s%dd%s", "%", ASC8_LEN, "\n"); 
	sprintf(buff, conv_string, milliseconds_of_data_missing);
	memcpy(mphr.milliseconds_of_data_missing, buff,
		   sizeof(mphr.milliseconds_of_data_missing));

	strncpy(mphr.subsetted_product_label,
			"SUBSETTED_PRODUCT             = ", 32);
	strncpy(mphr.subsetted_product, "F\n", FLAG_LEN+1);

    return product_name;
}


void display_mphr_config(const string& config_file)
{
	Config::file.read(config_file);
	cout << "EPS-L0 MPHR Configuration File\n"
		 << "------------------------------\n"
		 << "product_type			  : " << (Config::product_type).get() << endl
		 << "spacecraft_id			  : " << (Config::spacecraft_id).get() << endl
		 << "instrument_model		  : " << (Config::instrument_model).get() << endl
		 << "processing_centre		  : " << (Config::processing_centre).get() << endl
		 << "processing_mode		  : " << (Config::processing_mode).get() << endl
		 << "disposition_mode		  : " << (Config::disposition_mode).get() << endl
		 << "processor_major_version  : " << (Config::processor_major_version).get() << endl
		 << "processor_minor_version  : " << (Config::processor_minor_version).get() << endl
		 << "receiving_ground_station : " << (Config::recieving_ground_station).get() << endl
		 << endl;
}

