////////////////////////////////////////////////////////////////////////////////
//
//	File: noaa.hpp
//
//	Provides: Structs for representing NOAA L1b product files
//
//	Purpose:
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef noaa_defined
#define noaa_defined

//
// Archive Header
//

struct NOAA_Archive_Header
{
	enum { SIZE = 512 };

	struct // size 30
	{
		char cost_number[6];
		char saa_number[8];
		char year[4];
		char day[3];
		char processing_site;
		char processing_software[8];
	} order_id;

	struct // 89
	{
		char dataset_name[42];
		char space[2];
		char select_flag; // T=all, S=subset
		char beginning_latitude[3];
		char ending_latitude[3];
		char beginning_longitude[4];
		char ending_longitude[4];
		char start_hour[2];
		char start_minute[2];
		char number_of_minutes[3];
		char appended_data_flag;
		char channel_select_flags[20]; // Y/N
		char sensor_data_word_size[2];
	} query;

	struct // 48
	{
		char space[27];
		char direction_flag; // Ascend/Descend/Both
		char first_latitude[3];
		char last_latitude[3];
		char first_longitude[4];
		char last_longitude[4];
		char data_format[20];
		char record_size[6];
		char number_of_records[6];
	} data;

	char filler[319];

};

//
// Generic
//

struct NOAA_L1b_GEN_Header
{
	enum { SIZE = 116 };
	enum DataTypeCode {DTC_LAC=1, DTC_GAC, DTC_HRPT, DTC_TIP, DTC_HIRS,
					   DTC_MSU, DTC_SSU, DTC_DCS, DTC_SEM, DTC_AMSUA,
					   DTC_AMSUB, DTC_12, DTC_FRAC };

	// 0
	char creation_site[3];
	char space;

	// 4
	uint16_be level1b_format_version_number;
	uint16_be level1b_format_version_year;
	uint16_be level1b_format_version_day;
	uint16_be logical_record_length;
	byte reserved_2[2];
	uint16_be count_of_header_records;

	// 16
	int16_be zero_1[3];
	char dataset_name[42];
	char processing_block_identification[8];

	// 72
	uint16_be scid;
	uint16_be instrument_id; // 2=NOAA16 4=NOAA15
	uint16_be data_type_code; // 1=LAC, 2=GAC, 3=HRPT, 4=TIP, 5=HIRS, 6=MSU, 7=SSU,
	// 8=DCS, 9=SEM, 10=AMSU-A, 11=AMSU-B
	uint16_be tip_source_code;

	// 80
	uint32_be start_day; // days since 00:00 01/01/1950
	uint16_be start_year;
	uint16_be start_day_of_year;
	uint32_be start_millisecond;

	// 92
	uint32_be end_day; // days since 00:00 01/01/1950
	uint16_be end_year;
	uint16_be end_day_of_year;
	uint32_be end_millisecond;

	// 104
	uint16_be year_of_last_cpids_update;
	uint16_be day_of_year_of_last_cpids_update;
	int16_be zero_2[4];

	// 116
};

struct NOAA_L1b_GEN_Data
{
	enum { SIZE = 16 };

	uint16_be scan_line_number;
	uint16_be scan_line_year;
	uint16_be scan_line_day_of_year;
	uint16_be satelite_clock_drift_delta;
	uint32_be scan_line_millisecond_of_day;
	uint16_be scan_line_bit_field;
	uint8 major_frame_count;
	uint8 scan_position_number;
};

//
// AVHRR LAC
//

struct NOAA_L1b_AVHRR_Header
{
	enum { SIZE = 15872 }; // nb this isn't really used

	// offset 0
	NOAA_L1b_GEN_Header gen;

	// offset 116
	uint32_be instrument_status;
	int16_be zero_fill;
	uint16_be record_number_of_status_change;
	uint32_be second_instrument_status;
	uint16_be count_of_data_records;
	uint16_be count_of_good_scan_lines;
	uint16_be count_of_missing_scan_lines;
	uint16_be count_of_data_gaps;

	// offset 136
	byte skip[15736];
};

struct NOAA_L1b_AVHRR_Data
{
	enum { SIZE = 15872 }; // nb this isn't really used

	struct Sample
	{
		uint16_be channels[5];
	};

	NOAA_L1b_GEN_Data gen; // offset 0
	int8 gen_zero_fill[8];

	byte qual_ind[24]; // 24

	byte cal_coeff[252]; // 48

	struct
	{
		int16_be computed_roll_angle;
		int16_be computed_pitch_angle;
		int16_be computed_yaw_angle;
		int16_be applied_roll_angle;
		int16_be applied_pitch_angle;
		int16_be applied_yaw_angle;
		uint32_be status;
		int32_be euler_time;
		int16_be euler_roll;
		int16_be euler_pitch;
		int16_be euler_yaw;
		uint16_be altitude;
		struct
		{
			int16_be solar_zenith_angle;
			int16_be sat_zenith_angle;
			int16_be relative_azimuth_angle;
		} angluar_relationships[51];
		uint16_be zero_fill_1[3];
		struct
		{
			int32_be latitude;
			int32_be longitude;
		} earth_locations[51];
		uint32_be zero_fill_2[2];
	} nav; // 300

	byte zero_fill_1[12];

	uint16_be id[2]; // offset 1068


	byte skip_2[8]; // offset 1072

	struct Telemetry
	{
		Sample ramp_calibration;
		uint16_be prt[3];
		uint16_be patch_temperature;
		uint16_be undefined;
	} telemetry; // offset 1080

	struct BackScan
	{
		uint16_be channel_3;
		uint16_be channel_4;
		uint16_be channel_5;
	} back_scan[10]; // offset 1100

	Sample space_data[10]; // offset 1160
	uint16_be sync_delta; // offset 1260
	int16_be zero_fill_2;
	uint32_be sensor_data[3414]; // offset 1264
	int32_be zero_fill_3[2]; // offset 14920
	byte skip[944]; // offset 14928
	// offset 15870
};

//
// AMSU-A
//

struct NOAA_L1b_AMSUA_Header
{
	enum { SIZE = 2560 };

	NOAA_L1b_GEN_Header gen;

	// offset 116
	int16_be zero[2];

	// offset 120
	uint32_be instrument_status_a2;

	// offset 124
	byte skip_1[8];

	// offset 132
	uint32_be instrument_status_a1;

	// offset 136
	byte skip_2[8];

	// offset 144
	uint16_be count_of_data_records;
	uint16_be count_of_good_scan_lines;
	uint16_be count_of_missing_scan_lines;
	uint16_be count_of_data_gaps;
	uint16_be frames_without_frame_sync_errors;
	uint16_be pacs_detected_tip_parity_errors;

	// offset 156
	byte skip_3[2404];

};

struct NOAA_L1b_AMSUA_Data
{
	enum { SIZE = 2560 };

	NOAA_L1b_GEN_Data gen;
	int8 gen_zero_fill[8];

	// offset 24+8
	byte skip_1[872];

	// offset 896+8
	struct
	{
		struct
		{
			uint8 sync_sequence[3];
			uint8 serial_number;
			uint8 housekeeping[4];
		} digital_a;

		// offset 904+8
		struct
		{
			uint16_be first_reading[2];
			uint16_be second_reading[2];
			uint16_be counts[13];
		} scene[30];

		// offset 1924+8
		struct
		{
			uint16_be first_reading[2];
			uint16_be second_reading[2];
			uint16_be counts_1[13];
			uint16_be counts_2[13];
		} cold_calibration;

		// offset 1984+8
		uint16_be temperature_sensors[46];

		// offset 2076+8
		struct
		{
			uint16_be first_reading[2];
			uint16_be second_reading[2];
			uint16_be counts_1[13];
			uint16_be counts_2[13];
		} warm_calibration;

		// offset 2136+8
		int32_be zero_fill_1;

		// offset 2140+8
		struct
		{
			uint16_be invalid_word_bit_flags;
			uint16_be housekeeping;
			int32_be zero_fill_2;
		} digital_b;

		// offset 2148+8
		struct
		{
			uint32_be invalid_word_bit_flags;
			uint8 words[28];
			int32_be zero_fill;
		} analog_housekeeping;

	} a1;

	struct
	{
		// offset 2184+8
		struct
		{
			uint8 sync_sequence[3];
			uint8 serial_number;
			uint8 housekeeping[4];
		} digital_a;

		// offset 2192+8
		struct
		{
			uint16_be first_reading;
			uint16_be second_reading;
			uint16_be counts[2];
		} scene[30];

		// offset 2432+8
		struct
		{
			uint16_be first_reading;
			uint16_be second_reading;
			uint16_be counts_1[2];
			uint16_be counts_2[2];
		} cold_calibration;

		// offset 2444+8
		uint16_be temperature_sensors[20];

		// offset 2484+8
		struct
		{
			uint16_be first_reading;
			uint16_be second_reading;
			uint16_be counts_1[2];
			uint16_be counts_2[2];
		} warm_calibration;

		// offset 2496+8
		int32_be zero_fill;

		// offset 2500+8
		struct
		{
			uint16_be invalid_word_bit_flags;
			uint16_be telemetry;
			int32_be zero_fill;
		} digital_b;

		// offset 2508+8
		struct
		{
			uint32_be invalid_word_bit_flags;
			uint8 words[16];
		} analog_housekeeping;

	} a2;

	// offset 2528+8
	int32_be zero_fill[8];

	// offset 2560+8
};



//
// AMSU-B1
//

struct NOAA_L1b_AMSUB_Header
{
	enum { SIZE = 3072 };

	NOAA_L1b_GEN_Header gen;

	// offset 116
	int32_be zero_fill_1;

	// offset 120
	uint8 instrument_status[4];
	int16_be zero_fill;
	uint16_be record_number_of_status_change;
	uint32_be second_instrument_status;
	uint16_be count_of_data_records;
	uint16_be count_of_good_scan_lines;
	uint16_be count_of_missing_scan_lines;
	uint16_be count_of_data_gaps;
	uint16_be count_of_frames_without_frame_sync;
	uint16_be pacs_detected_tip_parity_errors;

	// offset 144
	byte skip_1[416];

	// offset 560
	int16_be prt_coeffs[7][4];

	// offset 616
	byte skip[2456];

	// offset 3072
};

struct NOAA_L1b_AMSUB_Data
{
	enum { SIZE = 3072 };

	struct Sample
	{
		uint16_be shaft_pos;
		uint16_be channels[5];
	};

	NOAA_L1b_GEN_Data gen;
	int8 gen_zero_fill[8];

	// offset 24
	uint32_be quality_indicator;
	uint32_be scanl_line_quality;
	uint16_be calibration_quality;

	// offset 34
	byte skip[1446];

	// offset 1480
	Sample sensor_data[90];

	// offset 2560
	int32_be zero_fill_1[2];

	// offset 2568;
	Sample space_view[4];

	// offset 2616
	Sample target_view[4];

	// offset 2664
	int32_be zero_fill_2[2];

	struct
	{
		// offset 2672
		uint32_be invalid_word_bits;

		// offset 2676;
		int32_be zero_fill_1;

		// offset 2680
		uint16_be data_words[26];

		// offset 2732
		int32_be zero_fill_2[3];

	} digital_a;

	struct
	{
		// offset 2744
		uint32_be invalid_word_bits;

		// offset 2748
		uint16_be words[28];
	} analog;

	// offset 2804
	int32_be zero_fill_3[67];

	// offset 3072
};

//
// HIRS
//

struct NOAA_L1b_HIRS_Header
{
	enum { SIZE = 4608 };

	NOAA_L1b_GEN_Header gen;

	// offset 116
	uint8 instrument_status[4];

	// offset 120
	int16_be zero_fill_1;

	// offset 122
	uint16_be record_number_of_status_change;

	// offset 124
	uint8 second_instrument_status[4];

	// offset 128
	uint16_be count_of_data_records;
	uint16_be count_of_good_scan_lines;
	uint16_be count_of_missing_scan_lines;
	uint16_be count_of_data_gaps;
	uint16_be frames_without_sync_errors;
	uint16_be pacs_detected_tip_parity_errors;

	// offset 140
	byte skip[4468];
};

struct NOAA_L1b_HIRS_Data
{
	enum { SIZE = 4608 };

	struct Element
	{
		uint8 header[4];
		uint16_be channels[20];
		uint16_be bitflags;
		uint16_be zero_fill;
	};

	NOAA_L1b_GEN_Data gen;
	int8 gen_zero_fill[8];

	// offset 24
	uint32_be zero_fill_1;

	// offset 28
	uint8 data_quality[4];

	// offset 32
	uint8 scan_line_quality[4];

	// offset 36
	byte skip[1420];

	// offset 1456
	Element elements[64];

	// offset 4528
	byte zero_fill_2[12];

	// offset 4540
	struct
	{
		uint16_be invalid_bits;
		uint16_be bits;
	} digital_b;

	// offset 4544;
	struct
	{
		uint32_be invalid_bytes;
		uint8 bytes[16];
	} analog;

	// offset 4564;
	int32_be zero_fill_3[11];

	// offset 4608
};

#endif
