////////////////////////////////////////////////////////////////////////////////
//
//	File: noaa_l1b_displayer.cpp
//
//	Provides: noaa_l1b_displayer executable
//
//	Purpose: Show selected fields from a NOAA L1b product as text
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "noaa.hpp"
#include "noaa_v3.hpp"
#include "noaaproductreader.hpp"
#include "avhrrreader.hpp"
#include "amsuareader.hpp"
#include "amsubreader.hpp"
#include "mhsreader.hpp"
#include "hirsreader.hpp"
#include "args.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

const string CONFIG_DIR_VAR = "METOPIZER_HOME";
//const string DEFAULT_CONFIG_DIR = ".metopizer";
const string DEFAULT_CONFIG_FILE = "mhs.config";

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "noaa_l1b_displayer";
const string MAINTAINER = "elson@eumetsat.de";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	bool hide_header = false;
	int bps=-1;
	string channels;
	bool force_mhs = false;
	string config_file;

	Args args;
	args.set_doc_head("Display a block from a NOAA L1b product as text.");
	args.set_doc_tail("The block number can be 0 for the header block, or 1 or greater for data blocks.\nSome fields that are not used for Metopizer processing are not shown.\n");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2005 Eumetsat.");
	args.set_license(Args::GPL);
	args.set_version(VERSION);
	args.set_usage("<product> <block number>");

	args.add_related_prog("ccsds_displayer");

	args.add_example("noaa_l1b_displayer nxrHIRS.l1b 50", "Display the 50th scan line of a HIRS	 L1b product");

	args.add_env_var("METOPIZER_HOME", "Location to search for mhs.config file");

	enum { option_debug = Args::USER_OPTION };

	args.add(&hide_header, "hide-header", 0, 0, "Do not display generic header part");
	args.add(&bps, "bps", 0, 0, "Set Bits Per Sample for AVHRR products", "If the Archive Header is present this value can be read. Otherwise it is assumed to be 10");
	args.add(&channels, "channels", 0, 0, "Select which AVHRR channels are present", "This is a string such as 'YYNYY' to select channels 1,2,4,5 only");
	args.add(&force_mhs, "force-mhs", 0, 0, "Force input to be treated as an MHS product regardless of the value in the Data Type Code field");
	args.add(0, "debug", option_debug, Args::HIDDEN);
	args.add(&config_file, "config", 'c', Args::FILENAME, "Use a config file. MHS display only - $HOME/.metopizer/mhs.config is the default.");

	args.set_args(argc, argv);

	string in_file;
	int record_no = -1;

	int arg;
	while((arg=args.get_next())!=Args::DONE)
	{
		switch(arg)
		{
		case Args::PARAMETER:
			if (in_file.empty())
			{
				in_file = args.get_string();
			}
			else if (record_no==-1)
			{
				record_no = args.get_longlong();
			}
			else
			{
				args.throw_error("Only specify 2 parameters");
			}
			break;

		case option_debug:
			_debug = &cout;
			break;
		}
	}

	if (in_file.empty())
	{
		args.throw_error("input file not specified");
	}

	if (record_no == -1)
	{
		args.throw_error("scan line not specified");
	}

	// if no config file specified check for an implicit one
	if (config_file.empty())
	{
		// if theres an environment variable set look there first
		const char* config_dir_var = getenv(CONFIG_DIR_VAR.c_str());
		if (config_dir_var)
		{
			config_file = string(config_dir_var) + "/" + DEFAULT_CONFIG_FILE;
		}
		else //if (getenv("HOME"))
		{
			// use the default filename
			//config_file = string(getenv("HOME")) + "/" + DEFAULT_CONFIG_DIR +
			//	"/" + DEFAULT_CONFIG_FILE;
			config_file = string("DEFAULT_METOPIZER_HOME") + "/" + DEFAULT_CONFIG_FILE;
		}
	}

	FileReader source(in_file);
	NOAAProductReader noaa_reader(source, bps, channels);

	if (noaa_reader.get_product_header())
	{
		int type = noaa_reader.get_product_header()->data_type_code;

		if (force_mhs)
		{
			type = NOAA_L1b_GEN_Header_v3::DTC_MHS;
		}

		switch(type)
		{
		case NOAA_L1b_GEN_Header_v3::DTC_GAC:
		case NOAA_L1b_GEN_Header_v3::DTC_LAC:
		case NOAA_L1b_GEN_Header_v3::DTC_HRPT:
		case NOAA_L1b_GEN_Header_v3::DTC_FRAC:
		{
			AVHRRReader avhrr_reader(noaa_reader, "", bps, channels);
			if (record_no==0)
			{
				avhrr_reader.output_text_header();
			}
			else
			{
				avhrr_reader.output_text_record(record_no-1, !hide_header);
			}
			break;
		}

		case NOAA_L1b_GEN_Header_v3::DTC_AMSUA:
		{
			AMSUAReader amsua_reader(noaa_reader, true, true);
			if (record_no==0)
			{
				amsua_reader.output_text_header();
			}
			else
			{
				amsua_reader.output_text_record(record_no-1, !hide_header);
			}
			break;
		}

		case NOAA_L1b_GEN_Header_v3::DTC_AMSUB:
		{
			AMSUBReader amsub_reader(noaa_reader);
			if (record_no==0)
			{
				amsub_reader.output_text_header();
			}
			else
			{
				amsub_reader.output_text_record(record_no-1, !hide_header);
			}
			break;
		}

		case NOAA_L1b_GEN_Header_v3::DTC_HIRS:
		{
			HIRSReader hirs_reader(noaa_reader);
			if (record_no==0)
			{
				hirs_reader.output_text_header();
			}
			else
			{
				hirs_reader.output_text_record(record_no-1, !hide_header);
			}
			break;
		}

		case NOAA_L1b_GEN_Header_v3::DTC_MHS:
		{
			MHSReader mhs_reader(noaa_reader, config_file, force_mhs);

			if (record_no==0)
			{
				mhs_reader.output_text_header();
			}
			else
			{
				mhs_reader.output_text_record(record_no-1, !hide_header);
			}
			break;
		}

		default:
		{
			error << "Input product type not supported ("
				  << noaa_reader.get_product_header()->data_type_code << ")\n";
			return -1;
		}
		}
	}
	else
	{
		error << "Cannot open input product header\n";
		return -1;
	}

	return 0;
}
