////////////////////////////////////////////////////////////////////////////////
//
//	File: noaaproductreader.cpp
//
//	Provides: NOAAProductReader filter
//
//	Purpose: Interpret a binary file as a NOAA L1b product
//
//	Copyright (c) 2002, 2003, 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "noaaproductreader.hpp"

NOAAProductReader::NOAAProductReader( FileProducer& source,
									  int user_bps,
									  string user_channels)
	: header(NULL)
	, buffer_1(NULL)
	, buffer_2(NULL)
	, user_specified_bps(false)
	, user_specified_channels(false)
{
	assert(sizeof(NOAA_Archive_Header) == NOAA_Archive_Header::SIZE);

	is = source.get_next_file();
	if (!is)
	{
		return;
	}

	byte buffer[sizeof(NOAA_Archive_Header)];
	is->read((char*)buffer, sizeof(NOAA_Archive_Header));

	if (is->gcount() < sizeof(NOAA_Archive_Header))
	{
		// input file is too small to have an archive header
		error << "Input file is too small to be a product\n";
		return;
	}
	else
	{
		// test if it has an archive header
		// this isnt completely reliable but will probably do
		bool has_header = true;
		for(int i=0; i<sizeof(NOAA_Archive_Header); i++)
		{
			if (buffer[i] < 32 || buffer[i] > 128)
			{
				debug << "noaaproductreader no archive header found\n";
				// debug << "byte " << i << " is "
				// << ((unsigned int)buffer[i]) << endl;
				has_header = false;
				break;
			}
		}

		if (has_header)
		{
			// an archive header was found.
			debug << "noaaproductreader archive header detected\n";

			// take a copy
			archive_header.reset(new NOAA_Archive_Header);
			memcpy(archive_header.get(), buffer,
				   sizeof(NOAA_Archive_Header));

			// read its block size

			// user should not override this when the archive header
			// is present

//			if (user_block_size != -1)
//			{
//			warning << "User specified block size ignored. "
//				<< "Block size read from archive header\n";
//			}

			char buffer[7];
			strncpy(buffer, archive_header->data.record_size, 6);
			buffer[6] = '\0';
			block_size = atoi(buffer);

			debug << "noaaproductreader block_size buffer '" << buffer
				  << "' size " << block_size << endl;

			// read in the product header
			header = new byte[block_size];
			is->read((char*)header, block_size);
			if (is->gcount() < block_size)
			{
				error << "NOAAProductReader:: error reading product "
					  << "header\n";
				return;
			}
		}
	}

	if (!archive_header.get())
	{
		// no archive header found.
		// we read the basic data type and presume the block size
		// from that

		// (nb. this routine relies on the fact all header blocks are
		// > 512 bytes long.
		// We avoid using seekg() since that breaks when the input is a
		// stream

		// The user can override this
//		if (user_block_size == -1)
//		{
		NOAA_L1b_GEN_Header* h = (NOAA_L1b_GEN_Header*)buffer;

		debug << "noaaprductreader:: read data type code "
			  << h->data_type_code << endl;

		switch(h->data_type_code)
		{
		case NOAA_L1b_GEN_Header::DTC_LAC:
		case NOAA_L1b_GEN_Header::DTC_HRPT:
		case NOAA_L1b_GEN_Header::DTC_FRAC:

			// its AVHRR high res. See if the user has specified
			// an exact type

			if (user_bps != -1 || !user_channels.empty())
			{
				if (user_bps == -1)
				{
					user_bps = 10;
				}
				else
				{
					user_specified_bps = true;
				}

				if (user_channels.empty() || user_channels=="")
				{
					user_channels = "YYYYY";
				}
				else
				{
					user_specified_channels = true;
				}

				if (user_bps == 10)
				{
					block_size = 15872;
				}
				else
				{
					int channel_count = 0;

					if (user_channels.size() != 5)
					{
						error << "User specified channels string "
							  << "must be five chars long\n";
						return;
					}

					for(int i=0; i<5; i++)
					{
						if (tolower(user_channels[i])=='y')
						{
							channel_count++;
						}
					}

					if (user_bps == 8)
					{
						block_size = 2048 + channel_count*2048;
					}
					else if (user_bps == 16)
					{
						block_size = 2048 + channel_count*4096;
					}
					else
					{
						error << "Illegal bps, use 8, 10 or 16\n";
						return;
					}
				}
			}
			else
			{
				// possible block sizes are:
				// 12288 - 8bps 5 channel
				// 15872 - 10bps 5 channel
				// 22528 - 16bps 5 channel

				// We scan the file and test for each block
				// size seperately by assuming each in turn is true
				// and reading in a header and 2 records for
				// that size, then see if the scan line number
				// is successive for any of our guesses.

				streampos initial_pos = is->tellg();

				bool matches[3];
				matches[0] = false;
				matches[1] = false;
				matches[2] = false;
				int match_count = 0;

				int block_sizes[3];
				block_sizes[0] = 3584;
				block_sizes[1] = 4608;
				block_sizes[2] = 5632;

				char* buffer = new char[block_sizes[2]*2];

				for(int i=0; i<3; i++)
				{
					is->seekg(block_sizes[i]);
					is->read(buffer, block_sizes[i]*2);
					int scan_1 = (reinterpret_cast<NOAA_L1b_GEN_Data*>
								  (buffer))->scan_line_number;
					int scan_2 = (reinterpret_cast<NOAA_L1b_GEN_Data*>
								  (buffer+block_sizes[i]))->scan_line_number;
					cout << "scan1 " << scan_1 << " scan2 " << scan_2
						 << endl;
					if ((scan_1+1) == scan_2)
					{
						matches[i] = true;
						match_count++;
						block_size = block_sizes[i];
					}
				}

				is->seekg(initial_pos);
				delete[] buffer;

				if (match_count != 1)
				{
					error << "Cannot determine type of AVHRR product."
						  << " Try using the --bps and --channels"
						  << " options\n";
					return;
				}
			}

			break;

		case NOAA_L1b_GEN_Header::DTC_GAC:

			// its AVHRR low res. See if the user has specified
			// an exact type

			if (user_bps != -1 || !user_channels.empty())
			{
				if (user_bps == -1)
				{
					user_bps = 10;
				}
				else
				{
					user_specified_bps = true;
				}

				if (user_channels.empty() || user_channels=="")
				{
					user_channels = "YYYYY";
				}
				else
				{
					user_specified_channels = true;
				}

				if (user_bps == 10)
				{
					block_size = 4608;
				}
				else
				{
					int channel_count = 0;

					if (user_channels.size() != 5)
					{
						error << "User specified channels string "
							  << "must be five chars long\n";
						return;
					}

					for(int i=0; i<5; i++)
					{
						if (tolower(user_channels[i])=='y')
						{
							channel_count++;
						}
					}

					if (user_bps == 8)
					{
						// note: not 409, the alignment
						// zero fill also changes inside
						block_size = 1544 + channel_count*408;
					}
					else if (user_bps == 16)
					{
//				block_size = 1544 + channel_count*816;
						switch(channel_count)
						{
						case 1:
							block_size = 2360;
							break;

						case 2:
							block_size = 3176;
							break;

						case 3:
							block_size = 3992;
							break;

						case 4:
							block_size = 4816;
							break;

						case 5:
							block_size = 5632;
							break;
						}
					}
					else
					{
						error << "Illegal bps, use 8, 10 or 16\n";
						return;
					}
				}
			}
			else
			{
				// Possible block sizes are
				// 3584 - 8 bps 5 channel
				// 4608 - 10 bps 5 channel
				// 5632 - 16 bps 5 channel

				// We scan the file and test for each block
				// size seperately by assuming each in turn is true
				// and reading in a header and 2 records for
				// that size, then see if the scan line number
				// is successive for any of our guesses.

				streampos initial_pos = is->tellg();

				bool matches[3];
				matches[0] = false;
				matches[1] = false;
				matches[2] = false;
				int match_count = 0;

				int block_sizes[3];
				block_sizes[0] = 3584;
				block_sizes[1] = 4608;
				block_sizes[2] = 5632;

				char* buffer = new char[block_sizes[2]*2];

				for(int i=0; i<3; i++)
				{
					is->seekg(block_sizes[i]);
					is->read(buffer, block_sizes[i]*2);
					int scan_1 = (reinterpret_cast<NOAA_L1b_GEN_Data*>
								  (buffer))->scan_line_number;
					int scan_2 = (reinterpret_cast<NOAA_L1b_GEN_Data*>
								  (buffer+block_sizes[i]))->scan_line_number;
					cout << "scan1 " << scan_1 << " scan2 " << scan_2
						 << endl;
					if ((scan_1+1) == scan_2)
					{
						matches[i] = true;
						match_count++;
						block_size = block_sizes[i];
					}
				}

				is->seekg(initial_pos);
				delete[] buffer;

				if (match_count != 1)
				{
					error << "Cannot determine type of AVHRR product."
						  << " Try using the --bps and --channels"
						  << " options\n";
					return;
				}
			}

			break;

		case NOAA_L1b_GEN_Header::DTC_AMSUA:
			block_size = NOAA_L1b_AMSUA_Header::SIZE;
			break;

		case NOAA_L1b_GEN_Header::DTC_AMSUB:
			block_size = NOAA_L1b_AMSUB_Header::SIZE;
			break;

		case NOAA_L1b_GEN_Header::DTC_HIRS:
			block_size = NOAA_L1b_HIRS_Header::SIZE;
			break;

		case NOAA_L1b_GEN_Header_v3::DTC_MHS:
			block_size = NOAA_L1b_MHS_Header_v3::SIZE;
			break;

		default:
			error << "NOAAProductReader:: unsupported data type\n"
				  << "(header generic data_type_code "
				  << h->data_type_code << ")\n";
			return;
		}
//		}
//		else
//		{
//		block_size = user_block_size;
//		user_specified_block_size = true;
//		}

		debug << "noaaproductreader gen block size "
			  << block_size << endl;

		// get the first (header) block
		header = new byte[block_size];

		// copy over the archive header buffer
		memcpy(header, buffer, sizeof(NOAA_Archive_Header));

		// read in the remainder of the block from disk
		is->read((char*)header+sizeof(NOAA_Archive_Header),
				 block_size-sizeof(NOAA_Archive_Header));

		if (is->gcount() < (block_size-sizeof(NOAA_Archive_Header)))
		{
			error << "NOAAProductReader:: error in product header\n";
			return;
		}
	}

	if (block_size==0)
	{
		error << "noaaproductreader block_size is zero\n";
		return;
	}

	// read in first data block (buffer_2 is a scratch area
	// used to test new records before overwriting the real
	// buffer_1)
	// This is required because clients can maintain pointers
	// to the current data curent which should remain valid after
	// eof is reached.
	buffer_1 = new byte[block_size];
	buffer_2 = new byte[block_size];
	is->read((char*)buffer_1, block_size);
	if (is->gcount() < block_size)
	{
		error << "Could not read first data block\n";
	}

}

NOAAProductReader::~NOAAProductReader(void)
{
	delete[] header;
	delete[] buffer_1;
	delete[] buffer_2;
}

int NOAAProductReader::get_record_size(void) const
{
	return block_size;
}

void NOAAProductReader::output_text_summary( textoutstream_type& os )
{
	os << "NOAAProductReader report:\n"
	   << "	   Archive Header "
	   << (archive_header.get()==NULL?"no":"yes")
	   << endl
	   << "	   Block Size " << block_size;
//		if (user_specified_block_size)
//		{
//		os << " (user specified)";
//		}

	if (user_specified_bps)
	{
		os << "		   (user specified bps)\n";
	}

	if (user_specified_channels)
	{
		os << "		   (user specified channels)\n";
	}

	os << endl;
//		   << "	   " << file_count << " files read\n\n";
	os << endl;
}

void NOAAProductReader::output_text_archive_header( textoutstream_type& os)
{
	if (!archive_header.get())
	{
		os << "No Archive Header\n\n";
		return;
	}

	os << "\nArchive Header\n--------------\n\n"
	   << "Order Creation Year : "
	   << nts(4,archive_header->order_id.year) << endl
	   << "				   Day : "
	   << nts(3,archive_header->order_id.day) << endl
	   << "Requested latitude  : "
	   << nts(3,archive_header->query.beginning_latitude)
	   << " to " << nts(3,archive_header->query.ending_latitude)
	   << endl
	   << "Requested longitude : "
	   << nts(4,archive_header->query.beginning_longitude)
	   << " to " << nts(4,archive_header->query.ending_longitude)
	   << endl
	   << "Start hour		   : "
	   << nts(2,archive_header->query.start_hour) << endl
	   << "Start minute		   : "
	   << nts(2,archive_header->query.start_minute) << endl
	   << "Number of minutes   : "
	   << nts(3,archive_header->query.number_of_minutes) << endl
	   << "Channels selected   : "
	   << nts(20,archive_header->query.channel_select_flags) << endl
	   << "Data word size	   : "
	   << nts(2,archive_header->query.sensor_data_word_size) << endl
	   << "Actual latitude	   : "
	   << nts(3,archive_header->data.first_latitude)
	   << " to " << nts(3,archive_header->data.last_latitude) << endl
	   << "Actual longitude	   : "
	   << nts(4,archive_header->data.first_longitude)
	   << " to " << nts(4,archive_header->data.last_longitude) << endl
	   << "Data format		   : "
	   << nts(20,archive_header->data.data_format) << endl
	   << "Size of records	   : "
	   << nts(6,archive_header->data.record_size) << endl
	   << "Number of records   : "
	   << nts(6,archive_header->data.number_of_records) << endl
	   << endl;
}

void NOAAProductReader::output_text_gen_header(textoutstream_type& os)
{
	NOAA_L1b_GEN_Header* h = (NOAA_L1b_GEN_Header*)header;

	os << "\nHeader\n------\n\n"
	   << "Creation site id			 : "
	   << nts(3,h->creation_site) << endl
	   << "L1b format version number : "
	   << h->level1b_format_version_number << endl
	   << "L1b format year			 : "
	   << h->level1b_format_version_year << endl
	   << "L1b format day			 : "
	   << h->level1b_format_version_day << endl
	   << "Logical record length	 : "
	   << h->logical_record_length << endl
	   << "Count of header records	 : "
	   << h->count_of_header_records << endl
	   << "Data set name			 : "
	   << nts(42, h->dataset_name) << endl
	   << "Spacecraft Id			 : "
	   << h->scid;

	switch(h->scid)
	{
	case 2 :
		os << " (NOAA-16)\n";
		break;
	case 4:
		os << " (NOAA-15)\n";
		break;
	case 7:
		os << " (NOAA-N)\n";
		break;
	case 8:
		os << " (NOAA-N')\n";
		break;
	case 11:
		os << " (MetOp-1)\n";
		break;
	case 12:
		os << " (MetOp-2)\n";
		break;
	case 13:
		os << " (MetOp-3)\n";
		break;
	default:
		os << " (unknown)\n";
	}

	os << "Instrument Id			 : " << h->instrument_id << endl
	   << "Data type code			 : " << h->data_type_code;
	switch(h->data_type_code)
	{
	case 1:
		os << " (AVHRR LAC)\n";
		break;
	case 2:
		os << " (AVHRR GAC)\n";
		break;
	case 3:
		os << " (HRPT)\n";
		break;
	case 4:
		os << " (TIP)\n";
		break;
	case 5:
		os << " (HIRS)\n";
		break;
	case 6:
		os << " (MSU)\n";
		break;
	case 7:
		os << " (SSU)\n";
		break;
	case 8:
		os << " (DCS)\n";
		break;
	case 9:
		os << " (SEM)\n";
		break;
	case 10:
		os << " (AMSU-A)\n";
		break;
	case 11:
		os << " (AMSU-B)\n";
		break;
	case 12:
		os << " (MHS)\n";
		break;
	case 13:
		os << " (FRAC)\n";
		break;
	default:
		os << " (unknown)\n";
	}
	os << "Start year				 : " << h->start_year << endl
	   << "Start day				 : " << h->start_day_of_year
	   << endl
	   << "Start millisecond		 : " << h->start_millisecond
	   << endl
	   << "Start time				 : "
	   << TimeConv::abs_year_day_millisecond_to_string(
		   h->start_year,
		   h->start_day_of_year,
		   h->start_millisecond)
	   << "." << ntofs(h->start_millisecond%1000,3,'0') << "000)\n"
	   << "End year					 : " << h->end_year << endl
	   << "End day					 : " << h->end_day_of_year << endl
	   << "End millisecond			 : " << h->end_millisecond << endl
	   << "End time					 : "
	   << TimeConv::abs_year_day_millisecond_to_string(
		   h->end_year,
		   h->end_day_of_year,
		   h->end_millisecond)
	   << "." << ntofs(h->end_millisecond%1000,3,'0') << "000)\n";
}

void NOAAProductReader::output_text_gen_record(int record_no, ostream& os)
{
	NOAA_L1b_GEN_Data* d = get_current_record();

	string t = TimeConv::abs_year_day_millisecond_to_string(
		d->scan_line_year,
		d->scan_line_day_of_year,
		d->scan_line_millisecond_of_day) +
		"." +
		ntofs(d->scan_line_millisecond_of_day%1000,3,'0') +
		"000";

	os << "Record number " << record_no << endl;

	os << "\nIntroduction\n------------\n\n"
	   << "Scan line number		 : " << d->scan_line_number << endl
	   << "Scan line year		 : " << d->scan_line_year << endl
	   << "Scan line day		 : " << d->scan_line_day_of_year
	   << endl
	   << "Scan line millisecond : "
	   << d->scan_line_millisecond_of_day << endl
	   << "Scan line time		 : " << t << endl << endl
	   << "Clock drift delta	 : "
	   << d->satelite_clock_drift_delta << endl
	   << "Clock drift applied	 : "
	   << ((d->scan_line_bit_field&(1<<14))!=0)
	   << endl << endl;
}

NOAA_Archive_Header* NOAAProductReader::get_archive_header(void)
{
	return archive_header.get();
}

NOAA_L1b_GEN_Header* NOAAProductReader::get_product_header(void)
{
	return reinterpret_cast<NOAA_L1b_GEN_Header*>(header);
}

NOAA_L1b_GEN_Data* NOAAProductReader::get_current_record(void) const
{
	if (!buffer_2)
	{
		// eof
		return NULL;
	}
	else
	{
		return ((NOAA_L1b_GEN_Data*)buffer_1);
	}
}

void NOAAProductReader::skip_records(unsigned int offset)
{
	if (offset == 0)
	{
		return;
	}

	if (offset > 1)
	{
		is->seekg((offset-1)*block_size, ios::cur);
		if (is->tellg() == streampos(-1))
		{
			debug << "noaaproductreader must manually skip because "
				  << "input is a stream\n";
			for(int i=0;i<(offset-1);i++)
			{
				// read into buffer_2 (scratch)
				is->read((char*)buffer_2, block_size);
			}
		}
		else
		{
			debug << "noaaproductreader using seekg as input "
				  << "is a file\n";
		}
	}

	// read into scratch buffer
	is->read((char*)buffer_2, block_size);
	if (is->gcount() < block_size)
	{
		debug << "noaaproductread has hit eof\n";
		delete[] buffer_2;
		buffer_2 = NULL;
	}
	else
	{
		// the read was a success, swap buffers
		swap(buffer_1, buffer_2);
	}
}
