////////////////////////////////////////////////////////////////////////////////
//
//	File: osvs.cpp
//
//	Purpose: Parse an OSVS xml file
//
//	Copyright (c) 2004 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "osvs.hpp"

#ifdef HAVE_LIBXML_PARSER_H
#include <libxml/tree.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#endif

//void error_func(void* user, xmlErrorPtr error)
void error_func(void* user, const char* msg, ...)
{
//	  cout << "xml error: " << msg << endl;
}

//
//
// OSV
//
//

OSVS::OSV::OSV(void)
	: semi_major_axis(0),
	  eccentricity(0),
	  inclination(0),
	  perigee_argument(0),
	  right_ascension(0),
	  mean_anomaly(0),
	  x(0),
	  y(0),
	  z(0),
	  vx(0),
	  vy(0),
	  vz(0),
	  orbit_number(0)
{
}

bool OSVS::OSV::operator != (const OSVS::OSV& o) const
{
	return (semi_major_axis!=o.semi_major_axis ||
			eccentricity!=o.eccentricity || inclination!=o.inclination ||
			perigee_argument!=o.perigee_argument ||
			right_ascension!=o.right_ascension ||
			mean_anomaly!=o.mean_anomaly || x!=o.x || y!=o.y || z!=o.z ||
			vx!=o.vx || vy!=o.vy || vz!=o.vz || orbit_number!=o.orbit_number);
}

//
//
// OSVS
//
//

OSVS::OSVS(void)
{
}

OSVS::OSVS(const string& _filename)
	: filename(_filename),
	  warned(false)
{
#ifdef HAVE_LIBXML_PARSER_H

	LIBXML_TEST_VERSION;
	xmlKeepBlanksDefault(0);

	xmlSetGenericErrorFunc(NULL,error_func);

	xmlDocPtr doc = xmlParseFile(filename.c_str());
	if (!doc)
	{
        //cerr << "Warning: could not open OSV file '" << filename << '\n';
		warned = true;
		return;
	}

	xmlNodePtr osvs_node = xmlDocGetRootElement(doc);
	if (!osvs_node)
	{
		cerr << "Invalid OSV file " << filename << endl;
		warned = true;
		return;
	}

	if (xmlStrcmp(osvs_node->name, (xmlChar*)"osvs"))
	{
		cerr << "Top level node of OSV file is "
			 << osvs_node->name << ", not 'osvs'\n";
		return;
	}

	xmlNodePtr osv_node = osvs_node->xmlChildrenNode;
	while(osv_node)
	{
		if (strcmp((char*)osv_node->name, "osv"))
		{
			cerr << "Found bad element '" << osv_node->name
				 << "' inside top level node\n";
			return;
		}

		OSV osv;

		xmlNodePtr param_node = osv_node->xmlChildrenNode;
		if (!param_node)
		{
			cerr << "Found an empty OSV element\n";
			return;
		}
		while(param_node)
		{
			if (!param_node->xmlChildrenNode)
			{
				cerr << "No content found for element " << param_node->name
					 << endl;
				return;
			}

			char* content = (char*)(param_node->xmlChildrenNode->content);
			if (!content)
			{
				cerr << "No content found for element " << param_node->name
					 << endl;
				return;
			}

			if (!strcmp((char*)param_node->name, "semi-major-axis"))
			{
				osv.semi_major_axis = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "eccentricity"))
			{
				osv.eccentricity = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "inclination"))
			{
				osv.inclination = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "perigee-argument"))
			{
				osv.perigee_argument = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "right-ascension"))
			{
				osv.right_ascension = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "mean-anomaly"))
			{
				osv.mean_anomaly = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "x"))
			{
				osv.x = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "y"))
			{
				osv.y = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "z"))
			{
				osv.z = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "vx"))
			{
				osv.vx = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "vy"))
			{
				osv.vy = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "vz"))
			{
				osv.vz = atof(content);
			}
			else if (!strcmp((char*)param_node->name, "orbit"))
			{
				osv.orbit_number = atoi(content);
			}

			param_node = param_node->next;
		}

		xmlChar* time_str = xmlGetProp(osv_node, (const xmlChar*)"epoch");
		if (!time_str)
		{
			cerr << "Found an OSV element with no 'epoch' attribute\n";
			return;
		}

		string s((char*)time_str);
		short_cds_time cds = make_short_cds_time(s);
		if (s.length() != 15)
		{
			cout << "Warning: found default/unreadable time in " << filename
				 << endl;
		}
		free(time_str);

//	cout << "inserting osv time " << cds << " content " << osv << endl;
		osv.state_vector_time = cds;
		osvs[cds] = osv;

		osv_node = osv_node->next;
	}

	xmlCleanupParser();

//	  if (!quiet)
//	  {
//	cout << "Read " << osvs.size() << " OSVs from " << filename << endl;
//	  }

#endif
}

OSVS::OSV OSVS::get_osv(const short_cds_time& start_cds) const
{
	short_cds_time earliest_cds = start_cds - 120*60*1000;

//	  if (!quiet)
//	  {
//	cout << "Searching OSV list for time " << start_cds << endl;
//	cout << "min time " << earliest_cds << endl;
//	  }

	for(map<short_cds_time,OSV>::const_reverse_iterator iter=osvs.rbegin();
		iter!=osvs.rend();
		iter++)
	{
		if (start_cds > (*iter).first && (*iter).first > earliest_cds)
		{
//		if (!quiet)
//		{
//		cout << "	 found orbit with start time " << (*iter).first
//			 << endl;
//		}           
			return (*iter).second;
		}
	}

	if (!warned)
	{
#ifndef HAVE_LIBXML_PARSER_H
//	cout << "Not looking up OSV values as libxml2 was not used at compile "
//		 << "time\n";
#else
//	cout << "Could not find OSV for time " << start_cds << " in file "
//		 << filename << " (further warnings will be suppressed)\n";
#endif
		warned = true;
	}

	return OSV();
}

OSVS::OSV OSVS::get_interpolated_osv(const short_cds_time& start_cds) const
{
	time_t target_time = start_cds.get_epoch();

	//use an 8-point Lagrange interpolation
	vector<time_t> t;
	vector<OSV> k;

	int indx = 0;
	for(map<short_cds_time,OSV>::const_iterator iter=osvs.begin();
		iter!=osvs.end();
		iter++)
	{
		time_t current_time = (*iter).first.get_epoch();
		t.push_back(current_time);
		k.push_back((*iter).second);

		if (target_time > current_time)
		{
			indx++;
		}
	}

	/*
	   The nominal interpolation will take the previous 4 and next 4 samples.
	   If we have less samples we take as many as possible on one side and the
	   remaining on the other side.
	*/
	int offset;
	int interpolation_samples = 8;
	if (indx-4 >= 0 && indx+4 < t.size()) //we take 4 samples from each side
	{
		offset = indx - 4;
	}
	else if (indx-4 < 0 && indx+4 < t.size()) //we take as many samples from the left as possible (<4)
	{
		offset = 0;
	}
	else if (indx-4 >= 0 && indx+4 >= t.size()) //we take as many samples from the right as possible (<4)
	{
		offset = t.size() - interpolation_samples;
	}

	//perform Lagrange interpolation
	float mult;
	float sum_x=0;
	float sum_y=0;
	float sum_z=0;
	float sum_vx=0;
	float sum_vy=0;
	float sum_vz=0;

	for(unsigned i=0; i< interpolation_samples; i++)
	{
		mult=1;
		for(unsigned j=0; j< interpolation_samples; j++)
		{
			if(j!=i)
			{
				double coeff = double(target_time-t[offset+j]) / double(t[offset+i]-t[offset+j]);
				mult*= coeff;
			}
		}

		//perform the calculation for each variable
		sum_x += (mult *(k[offset+i].x));
		sum_y += (mult *(k[offset+i].y));
		sum_z += (mult *(k[offset+i].z));
		sum_vx += (mult *(k[offset+i].vx));
		sum_vy += (mult *(k[offset+i].vy));
		sum_vz += (mult *(k[offset+i].vz));
	}

	OSVS::OSV myResult;
	myResult.x = sum_x;
	myResult.y = sum_y;
	myResult.z = sum_z;
	myResult.vx = sum_vx;
	myResult.vy = sum_vy;
	myResult.vz= sum_vz;
	return myResult;
}

void OSVS::add_osv(short_cds_time cds, OSV o)
{
    osvs[cds] = o;
}

ostream& operator << (ostream& os, const OSVS::OSV& osv)
{
	os << "OSV:\n"
	   << "	   semi-major-axis: " << osv.semi_major_axis << endl
	   << "	   eccentricity: " << osv.eccentricity << endl
	   << "	   inclination: " << osv.inclination << endl
	   << "	   perigee-argument: " << osv.perigee_argument << endl
	   << "	   right-ascension: " << osv.right_ascension << endl
	   << "	   mean-anomaly: " << osv.mean_anomaly << endl
	   << "	   x: " << osv.x << endl
	   << "	   y: " << osv.y << endl
	   << "	   z: " << osv.z << endl
	   << "	   vx: " << osv.vx << endl
	   << "	   vy: " << osv.vy << endl
	   << "	   vz: " << osv.vz << endl
	   << "	   orbit number: " << osv.orbit_number << endl;
	return os;
}

