////////////////////////////////////////////////////////////////////////////////
//
//	File: productextractor.hpp
//
//	Provides: ProductExtractor extractor
//
//	Purpose: Convert product objects into binary files for storage
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef productextractor_defined
#define productextractor_defined

#include "fileproducer.hpp"
#include "productproducer.hpp"

class ProductExtractor :
	public FileProducer,
	public TextSummaryProducer
{
public:

	ProductExtractor(ProductProducer& _source)
		: source(_source)
		, triggered(false)
		, data_block_count(0)
		, has_archive_header(false)
		, has_product_header(false)
		, block_size(0)
		{
		}

	bool produce_files( FileConsumer& dest )
		{
			if (!spinner.get())
			{
				spinner.reset(new Spinner());
			}

			ostream_type* os =dest.get_output_stream();
			streampos pos;

			// write the archive header
			NOAA_Archive_Header* archive_header = source.get_archive_header();
			if (archive_header)
			{
				pos = os->tellp();
				os->write((char*)archive_header, sizeof(NOAA_Archive_Header));
				if (os->tellp()-pos != streampos(sizeof(NOAA_Archive_Header)))
				{
					error << "Cannot write archive header\n";
					return true;
				}
				has_archive_header = true;

				spinner->tick();
			}

			// write product header
			block_size = source.get_record_size();
			NOAA_L1b_GEN_Header* header = source.get_product_header();
			if (header)
			{
				pos = os->tellp();
				os->write((char*)header, block_size);
				if (os->tellp()-pos != streampos(block_size))
				{
					error << "Cannot write product header\n";
					return true;
				}

				has_product_header = true;

				spinner->tick();
			}
			else
			{
				error << "Input product header not found\n";
			}

			// write data blocks
			while(NOAA_L1b_GEN_Data* data = source.get_current_record())
			{
				pos = os->tellp();
				os->write((char*)data, block_size);
				if (os->tellp()-pos != streampos(block_size))
				{
					error << "Cannot write data block " << data_block_count
						  << endl;
					return true;
				}

				source.skip_records(1);
				data_block_count++;

				spinner->tick();
			}

			spinner->end();
			return true;
		}

	istream_type* get_next_file(void)
		{
			return NULL;
		}

	void output_text_summary( ostream& os = cout )
		{
			os << "ProductExtractor report:\n";
			if (has_archive_header)
			{
				os << "	   Wrote archive header ("
				   << sizeof(NOAA_Archive_Header) << " bytes)\n";
			}

			if (has_product_header)
			{
				os << "	   Wrote product header (" << block_size
				   << " bytes)\n";
			}

			os << "	   Wrote " << data_block_count << " data blocks ("
			   << (data_block_count*block_size) << " bytes)\n"
			   << endl;
		}

private:

	ProductProducer& source;
	bool triggered;
	int data_block_count;
	bool has_archive_header;
	bool has_product_header;
	int block_size;

};

#endif




