////////////////////////////////////////////////////////////////////////////////
//
//	File: pyamsuareader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose: Code for representing source packets as Python Objects
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "pymetopizer.hpp"
#include "pyamsuareader.hpp"

#include "ccsds.hpp"

const int CHANNEL_COUNT = 15;
const int NUM_SAMPLES = 30;

#include <intobject.h>

////////////////////////////////////////////////////////////////////////////////
//
PyAMSUAReader::PyAMSUAReader(CCSDSProducer* source)
	: PyReader(source)
{
}

//	describe available data
PyObject* PyAMSUAReader::get_channels(void) const
{
	// "(Samples,(Channel 1, Channel 2, Channel 3,...))"

	PyObject* samples = PyList_New(CHANNEL_COUNT);
	for(int i=0; i<CHANNEL_COUNT; i++)
	{
		PyList_SetItem(samples, i, ::make_pyobject(string("Channel ")+ntos(i+1)));
	}
	PyObject* band = PyTuple_New(2);
	PyTuple_SetItem(band, 0, ::make_pyobject("Channels"));
	PyTuple_SetItem(band, 1, samples);
	PyObject* bands = PyList_New(1);
	PyList_SetItem(bands, 0, band);

	return bands;
}

// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
PyObject* PyAMSUAReader::get_samples(int channel, int band, int modulus, int count, int skip, int modulus2)
{
	if (modulus2 == -1)
	{
		modulus2 = modulus;
	}

	set_count_filter(count, skip, modulus);

	// reset back to start of file cache
	reset_files();

	if ( channel < 0 || channel >= CHANNEL_COUNT )
	{
		PyErr_SetString(PyMetopizerError, "Given Channel is out of range" );
		return NULL;
	}

	if ( modulus <= 0 )
	{
		modulus = 1;
	}

	PyObject* sample_list = PyList_New(0);
	while ( CCSDS* packet = get_next_ccsds() )
	{
		if ( ( (packet->get_apid() == APID_AMSUA_1) && (channel >=2 && channel < 15) ) ||
			 ( (packet->get_apid() == APID_AMSUA_2) && (channel >=0 && channel < 2 ) ) )
		{
			PyObject* scan_list = SceneData(packet).get_samples(channel, modulus );
			if ( scan_list )
			{
				PyList_Append( sample_list, scan_list );
				Py_DECREF(scan_list);
			}
			else
			{
				// need to raise fatal exception here
				PyException(PyMetopizerError, "Bad data in packet\n";);
				Py_DECREF(sample_list);
				return NULL;
			}

			store_times(packet);
		}
	}

	return sample_list;
}

// instrument name for this reader
string PyAMSUAReader::get_instrument_name(void) const
{
	return string("AMSU-A");
}

PyAMSUAReader::SceneData::SceneData(CCSDS* packet)
	: type(BAD)
{
	// First assemble the packet data
	sci_data.resize( packet->get_payload_length() );
	int sci_data_index = 0;

	// detect and disguard filler data
	for(int i=0; i<packet->get_payload_length(); i+=2)
	{
		byte a = packet->get_payload()[i];
		byte b = packet->get_payload()[i+1];

		if (a!=0 || b!=1)
		{
			// not filler data
			sci_data[sci_data_index++] = a;
			sci_data[sci_data_index++] = b;
		}
	}

	sci_data.resize(sci_data_index);

	//cout << " ---> Packet info: "	   << endl
	//	   << "		 APID: "		   << packet->get_apid()		   << endl
	//	   << "		 Payload Length: " << packet->get_payload_length() << endl
	//	   << "		 Data Index: "	   << sci_data_index			   << endl
	//	   << "		 Size: "		   << sci_data.size()			   << endl;

	unsigned int apid = packet->get_apid();

	// Then read bytes into required struct to access each field
	if ( apid == APID_AMSUA_1 && sci_data.size() == AMSUA1MeasurementScienceData::SIZE )
	{
		data.a1m = reinterpret_cast<AMSUA1MeasurementScienceData*>(&(*(sci_data.begin())));
		type = A1M;
	}
	else if ( apid == APID_AMSUA_1 && sci_data.size() == AMSUA1UniqueScienceData::SIZE )
	{
		data.a1u = reinterpret_cast<AMSUA1UniqueScienceData*>(&(*(sci_data.begin())));
		type = A1U;
	}
	else if ( apid == APID_AMSUA_2 && sci_data.size() == AMSUA2MeasurementScienceData::SIZE )
	{
		data.a2m = reinterpret_cast<AMSUA2MeasurementScienceData*>(&(*(sci_data.begin())));
		type = A2M;
	}
	else if ( apid == APID_AMSUA_2 && sci_data.size() == AMSUA2UniqueScienceData::SIZE )
	{
		data.a2u = reinterpret_cast<AMSUA2UniqueScienceData*>(&(*(sci_data.begin())));
		type = A2U;
	}
	else
	{
		//	cout << "Invalid Packet Size (corrupt packet)" << endl;
		//	PyErr_SetString(PyMetopizerError, "Packet data corrupt or invalid format, unable to get sample data" );
		type = BAD;
	}
}

PyObject* PyAMSUAReader::SceneData::get_samples( int channel, int modulus )
{
	// convert to a zero based index for extracting channel values
	if ( channel > 1 && channel < 15 )
	{
		channel = channel - 2;
	}

	int num_samples = max(1,NUM_SAMPLES/modulus);

	PyObject* scan_list = PyList_New( num_samples );

	if ( type != BAD )
	{
		for(int i=0; i<NUM_SAMPLES;i++)
		{
			uint16_be item;
			switch( type )
			{
			case A1M: item = data.a1m->scene[i*modulus].counts[channel]; break;
			case A1U: item = data.a1u->scene[i*modulus].counts[channel]; break;
			case A2M: item = data.a2m->scene[i*modulus].counts[channel]; break;
			case A2U: item = data.a2u->scene[i*modulus].counts[channel]; break;
			case BAD: break;
			}
			PyList_SetItem( scan_list, i, PyInt_FromLong(item) );
		}
	}
	else
	{
		for(int i=0; i<NUM_SAMPLES;i++)
		{
			Py_INCREF(Py_None);
			PyList_SetItem( scan_list, i, Py_None);
		}
	}
	return scan_list;
}
