////////////////////////////////////////////////////////////////////////////////
//
//	File: pyascatreader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pyascatreader.hpp"
#include "ccsds.hpp"

#ifdef PY3K
#include <unicodeobject.h>
#define PyString_FromString PyUnicode_FromString
#endif

const int CHANNEL_COUNT = 6;
const int NUM_SAMPLES = 256;

PyASCATReader::PyASCATReader(CCSDSProducer* source)
	: PyReader(source)
{
}

//	return a list of strings
PyObject* PyASCATReader::get_channels(void) const
{
	// "(Samples,(Channel 1, Channel 2, Channel 3,...))"

	PyObject* samples = PyList_New(CHANNEL_COUNT);
	PyList_SetItem(samples, 0, PyString_FromString("Left Fore"));
	PyList_SetItem(samples, 1, PyString_FromString("Left Mid"));
	PyList_SetItem(samples, 2, PyString_FromString("Left Aft"));
	PyList_SetItem(samples, 3, PyString_FromString("Right Fore"));
	PyList_SetItem(samples, 4, PyString_FromString("Right Mid"));
	PyList_SetItem(samples, 5, PyString_FromString("Right Aft"));
	PyObject* band = PyTuple_New(2);
	PyTuple_SetItem(band, 0, ::make_pyobject("Antenna"));
	PyTuple_SetItem(band, 1, samples);
	PyObject* bands = PyList_New(1);
	PyList_SetItem(bands, 0, band);

	return bands;
}

// return an array of 256-element long arrays. optional 2nd parameter gives modulus [only every 5th sample included])
PyObject* PyASCATReader::get_samples(int channel, int band, int modulus, int count, int skip, int modulus2)
{
	if (modulus2 == -1)
	{
		modulus2 = modulus;
	}

	set_count_filter(count, skip, 1);

	// reset back to start of file cache
	reset_files();

	if ( channel < 0 || channel >= CHANNEL_COUNT )
	{
		PyErr_SetString(PyMetopizerError, "Given Channel is out of range" );
		return NULL;
	}

	if ( modulus <= 0 )
	{
		modulus = 1;
	}

	PyObject* sample_list = PyList_New(0);

	int num_samples = max(1,NUM_SAMPLES/modulus2);
	int mod_count = 0;

	while ( CCSDS* packet = get_next_ccsds() )
	{
		if ( packet->get_apid() == static_cast<unsigned int>(channel + 208) ||
			 packet->get_apid() == static_cast<unsigned int>(channel + 216) )
		{
			if ((mod_count%modulus) == 0)
			{
				ASCAT_NOM_Payload& ascat = reinterpret_cast<CCSDS_ASCAT_NOM*>(packet->get_data())->payload;
				//cout << " --> Processing ASCAT APID = " << packet->get_apid() << endl << endl;  // TBR
				// only channels 0..5 are available with channel 0 == APID_208 || APID_214
				PyObject* scan_list = PyList_New( num_samples );
				for(int i=0; i<num_samples; i++)
				{
					// get value directly from payload data & apply modulus
					unsigned int ascat_payload_sample = ascat.data[i*modulus2];
					double item = convert_ASCAT_payload(ascat_payload_sample);

					PyList_SetItem( scan_list, i, PyFloat_FromDouble(item) );
				}
				PyList_Append( sample_list, scan_list );
				Py_DECREF(scan_list);

				store_times(packet);
			}
			mod_count++;
		}
	}

	return sample_list;
}

// instrument name for this reader
string PyASCATReader::get_instrument_name(void) const
{
	return string("ASCAT");
}

// read-in the S-FLOAT type Payload data
double PyASCATReader::convert_ASCAT_payload(unsigned int ascat_payload_sample)
{
	bool s = (ascat_payload_sample>>15)&0x1;
	unsigned int e = (ascat_payload_sample>>7)&0xff;
	unsigned int f = (ascat_payload_sample)&0x7f;

	//int sign = (-1)^s;
	double sign = 1.0;
	if ( s == 1.0 )
	{
		sign = -1.0;
	}
	double fraction = static_cast<double>(f)/128.0;
	double v;

	switch ( e )
	{
	case 255 :
		if ( f == 0 ) // ISD case 2
		{
			//cout << " --x-> Case 2 " << endl;
#ifdef NO_INFINITY
			v = 0.0;   // = (-1)^s x oo
#else
			v = sign*INFINITY;	 // = (-1)^s x oo
#endif
		}
		else		  // ISD case 1
		{
			//cout << " --x-> Case 1 " << endl;
#ifdef NO_NAN
			v = 0.0;
#else
			v = nanf("");		// = NaN
#endif
		}

	case 0 :
		if ( f == 0 ) // ISD case 5
		{
			//cout << " --x-> Case 5 " << endl;
			v = 0.0;
		}
		else		  // ISD case 4
		{
			//cout << " --x-> Case 4 " << endl;
			v = sign*pow(2.0,-126.0)*fraction;	  // = ((-1)^s)*(2^(-126))*(0f)
		}

	default:		  // ISD case 3
	{
		//cout << " --x-> Case 3 " << endl;
		v = sign*pow(2.0,static_cast<double>(e-127))*(fraction+1.0);   // = ((-1)^s)*(2^(e-127))*(1f)
	}
	}

	//cout << " --> ASCAT payload sample values: "			   << endl
	//	   << "		payload_sample = " << ascat_payload_sample << endl
	//	   << "		s = "			   << s					   << endl
	//	   << "		e = "			   << e					   << endl
	//	   << "		f = "			   << f					   << endl
	//	   << "		sign = "		   << sign				   << endl
	//	   << "		fraction = "	   << fraction			   << endl
	//	   << "		v = "			   << v					   << endl << endl;

	return v;
}

