////////////////////////////////////////////////////////////////////////////////
//
//	File: pyavhrrreader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose: Code for representing source packets as Python Objects
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pyavhrrreader.hpp"

#include "ccsds.hpp"
#include <intobject.h>

const int CHANNEL_COUNT = 5;
const int NUM_SAMPLES = 2048;

// Specialisation for AVHRR Instrument with APID of ??
PyAVHRRReader::PyAVHRRReader(CCSDSProducer* source)
	: PyReader(source)
{
}

//	return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
PyObject* PyAVHRRReader::get_channels(void) const
{
	// "(Samples,(Channel 1, Channel 2, Channel 3,...))"

	PyObject* samples = PyList_New(CHANNEL_COUNT);
	for(int i=0; i<CHANNEL_COUNT; i++)
	{
		PyList_SetItem(samples, i, ::make_pyobject(string("Channel ")+ntos(i+1)));
	}
	PyObject* band = PyTuple_New(2);
	PyTuple_SetItem(band, 0, ::make_pyobject("Channels"));
	PyTuple_SetItem(band, 1, samples);
	PyObject* bands = PyList_New(1);
	PyList_SetItem(bands, 0, band);

	return bands;
}

// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
PyObject* PyAVHRRReader::get_samples(int channel, int band, int modulus, int count, int skip, int modulus2)
{
//	  cout << "avhrr get_samples channel " << channel << " band " << band << " modulus " << modulus << " count " << count << " skip " << skip << " modulus2 " << modulus2 << endl;

	if (modulus2 == -1)
	{
		modulus2 = modulus;
	}

	set_count_filter(count, skip, modulus);

	// reset back to start of file cache
	reset_files();

	if ( channel < 0 || channel >= CHANNEL_COUNT )
	{
		PyErr_SetString(PyMetopizerError, "Given Channel is out of range" );
		return NULL;
	}

	if ( modulus <= 0 )
	{
		modulus = 1;
	}

	PyObject* sample_list = PyList_New(0);
	int num_samples = max(1,NUM_SAMPLES/modulus2);

	// Constants used to offset byte stream & skip space view and Calibration data
	const int start_byte = 68;
	const int start_bit = 1;

	while ( CCSDS* packet = get_next_ccsds() )
	{
		if (packet->get_apid()!=103 and packet->get_apid()!=104)
		{
			continue;
		}
		byte* payload_byte = (reinterpret_cast<CCSDS_AVHRR*>(packet->get_data())->payload.data)+start_byte;
		int payload_bit = start_bit;

//	// Shift byte stream accordingly
//	int shift_total = CHANNEL_COUNT * 11; // skipping 10 space views + calibration
//	for ( int y=0; y< shift_total; y++ )
//	{
//		for(int bit = 0; bit<10; bit++)
//		{
//		if (payload_bit > 0)
//		{
//			payload_bit--;
//		}
//		else
//		{
//			payload_bit = 7;
//			payload_byte++;
//		}
//		}
//	}
//	cout << "1st Bits: " << payload_bit << " Bytes: " << payload_byte-(reinterpret_cast<CCSDS_AVHRR*>(packet->get_data())->payload.data);

		// read the samples data from the raw packets data
		AVHRR_Sample samples[NUM_SAMPLES];
		for(int i=0; i<NUM_SAMPLES; i++)
		{
			samples[i] = read_avhrr_sample(payload_byte, payload_bit);
		}

		// Now put all sub sampled values into a PyList
		PyObject* scan_list = PyList_New( num_samples );
		for(int i=0; i < num_samples; i++)
		{
			uint16_be item = samples[i*modulus2].channels[channel];
			PyList_SetItem( scan_list, i, PyInt_FromLong(item) );
		}
		PyList_Append( sample_list, scan_list );
		Py_DECREF(scan_list);

		store_times(packet);
	}

	return sample_list;
}

// instrument name for this reader
string PyAVHRRReader::get_instrument_name(void) const
{
	return string("AVHRR");
}

AVHRR_Sample PyAVHRRReader::read_avhrr_sample(byte*& payload_byte, int& payload_bit)
{
	AVHRR_Sample sample;
	memset(&sample, 0, sizeof(sample));

	for(int channel = 0; channel<CHANNEL_COUNT; channel++)
	{
		for(int bit = 0; bit<10; bit++)
		{
			sample.channels[channel] <<= 1;
			sample.channels[channel] +=
				read_avhrr_bit(payload_byte, payload_bit);
		}
	}

	return sample;
}

int PyAVHRRReader::read_avhrr_bit(byte*& payload_byte, int& payload_bit)
{
	int result;
	if ( ((*payload_byte)&(1<<payload_bit)) > 0)
	{
		result = 1;
	}
	else
	{
		result = 0;
	}

	if (payload_bit > 0)
	{
		payload_bit--;
	}
	else
	{
		payload_bit = 7;
		payload_byte++;
	}

	return result;
}
