////////////////////////////////////////////////////////////////////////////////
//
//	File: pymhsreader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 Eumetsat
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pygomereader.hpp"
#include "ccsds.hpp"
#include "ccsdsproducer.hpp"

#include <intobject.h>

const int BAND_COUNT = 6;
const int MAX_CHANNEL_COUNT = 1024;
const int WIDTH = 32;

PyGOMEReader::PyGOMEReader( CCSDSProducer* source )
	: PyReader(source)//,
//	  hold(NULL)
{
	//cout << "struct size " << sizeof(CCSDS_GOME) << " exp " << CCSDS_GOME::SIZE << endl;

	// record which FPA channel (0-3) contains each band (PMD is ignored)
	band_channels[0] = 0;
	band_channels[1] = 0;
	band_channels[2] = 1;
	band_channels[3] = 1;
	band_channels[4] = 2;
	band_channels[5] = 3;

	while(CCSDS* ccsds = source->get_next_ccsds())
	{
		// we have to read up to 3 packets from the start of the file to identify the
		// band start/stop points.
		// The `buffer` queue was a plan to cache these packets for replaying later.
		// But it is not implemented.
		//buffer.push(new CCSDS(reinterpret_cast<byte*>(ccsds->get_data()), true));
		CCSDS_GOME* gome = reinterpret_cast<CCSDS_GOME*>(ccsds->get_data());
		int hk_mod = gome->hk[6];
		//cout << "hk_mod " << hk_mod << endl;
		if ((hk_mod&3) == 3)
		{
			int sep_ch1 = gome->hk[268];
			int sep_ch2 = gome->hk[269];

			//cout << "sep_ch1 " << sep_ch1 << " sep_ch2 " << sep_ch2 << endl;

			// record the position within the channel where each band begins ...
			band_starts[0] = 0;
			band_starts[1] = sep_ch1 + 1;
			band_starts[2] = 0;
			band_starts[3] = sep_ch2 + 1;
			band_starts[4] = 0;
			band_starts[5] = 0;

			// ... and where it ends
			band_ends[0] = sep_ch1;
			band_ends[1] = 1023;
			band_ends[2] = sep_ch2;
			band_ends[3] = 1023;
			band_ends[4] = 1023;
			band_ends[5] = 1023;

			break;
		}
	}
}

//	return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
PyObject* PyGOMEReader::get_channels(void) const
{
	PyObject* bands = PyList_New(6);

	const char* band_names[] = {"1a", "1b", "2a", "2b", "3", "4"};

	for(int i=0; i<6; i++)
	{
		PyObject* band = PyTuple_New(2);
		PyTuple_SetItem(band, 0, ::make_pyobject(band_names[i]));
		int j = band_ends[i]-band_starts[i]+1;
		//cout << "band " << i << " name " << band_names[i] << " start " << band_starts[i] << " end " << band_ends[i] << " count " << j << endl;
		PyObject* channels = PyList_New(j);
		for(int k=0; k<j; k++)
		{
			PyList_SetItem(channels, k, ::make_pyobject(string("Channel ")+ntos(k)));
		}
		PyTuple_SetItem(band, 1, channels);
		PyList_SetItem(bands, i, band);
	}

	return bands;
}

// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
PyObject* PyGOMEReader::get_samples(int channel, int band, int modulus, int count, int skip, int modulus2)
{
	// reset back to start of file cache
	reset_files();

	if (band<0 || band>=BAND_COUNT)
	{
		PyException(PyMetopizerError, "Band " << band << " is out of range min 0 max " << (BAND_COUNT-1));
		return NULL;
	}

	int max_channel = (band_ends[band]-band_starts[band]);

	if ( channel < 0 || channel > max_channel)
	{
		PyException(PyMetopizerError, "Channel " << channel << " is out of range min 0 max " << max_channel);
		return NULL;
	}

	PyObject* lines = PyList_New(0);

	UTCTime utc_time;
	OBTTime obt_time;
	short_cds_time grh_time;

	bool time_set = false;
	bool primed = false;

	typedef vector<int> Readings;
	Readings readings;

	int minpos = 65536;
	int maxpos = 0;

	while ( CCSDS* packet = get_next_ccsds()) // get_gome_ccsds())
	{
		CCSDS_GOME* gome = reinterpret_cast<CCSDS_GOME*>(packet->get_data());

		int subset_count = gome->hk[6];
		//cout << "subset count " << subset_count << endl;

		//cout << "int 1 1A " << static_cast<uint>((anc.word[17]>>10)&3)
		//	   << " 1B " << static_cast<uint>((anc.word[17]>>8)&3)
		//	   << " 2A " << static_cast<uint>((anc.word[17]>>6)&3)
		//	   << " 2B " << static_cast<uint>((anc.word[17]>>4)&3)
		//	   << " 3 " << static_cast<uint>((anc.word[17]>>2)&3)
		//	   << " 4 " << static_cast<uint>(anc.word[17]&3)

		//	   << " int 2 1A " << static_cast<uint>((anc.word[25]>>10)&3)
		//	   << " 1B " << static_cast<uint>((anc.word[25]>>8)&3)
		//	   << " 2A " << static_cast<uint>((anc.word[25]>>6)&3)
		//	   << " 2B " << static_cast<uint>((anc.word[25]>>4)&3)
		//	   << " 3 " << static_cast<uint>((anc.word[25]>>2)&3)
		//	   << " 4 " << static_cast<uint>(anc.word[25]&3);

		if (subset_count == 0)
		{
			// primed is used to ensure we only read complete scanlines, ignoring
			// partial lines at the start of a file
			primed = true;
			if (!readings.empty())
			{
				PyObject* line = PyList_New(WIDTH);

				if (minpos<16385)
				{
					for(int o=0; o<WIDTH; o++)
					{
						int i = (readings.size()*o)/WIDTH;
						PyList_SetItem(line, o, PyInt_FromLong(readings[i]));
					}
				}
				else
				{
					for(int o=0; o<WIDTH; o++)
					{
						PyList_SetItem(line, o, PyInt_FromLong(-1));
					}
				}

				PyList_Append(lines, line);
				Py_DECREF(line);

				utc_times.push_back(utc_time);
				obt_times.push_back(obt_time);
				if (eps)
				{
					grh_times.push_back(grh_time);
				}
				time_set = false;

				readings.resize(0);
				minpos = 16385;
				maxpos = 0;
			}
		}

		if (primed)
		{
			bool readout_1 = (gome->hk[17] >> (10-band*2)) & 3;
			bool readout_2 = (gome->hk[18] >> (5-band)) & 1;

			if ( (readout_1||readout_2) && !time_set)
			{
				if (packet->has_secondary_header())
				{
					utc_time = *(packet->get_secondary_header());
				}
				obt_time = *packet->get_ancillary_data();
				if (eps)
				{
					grh_time = eps->get_grh_start();
				}
				time_set = true;
			}

			if (readout_1)
			{
				int pos = gome->hk[96];
				int val = gome->fpa[0][band_channels[band]].data[band_starts[band]+channel];
				readings.push_back(val);
				minpos = min(minpos, pos);
				maxpos = max(maxpos, pos);
			}

			if (readout_2)
			{
				int pos = gome->hk[98];
				int val = gome->fpa[1][band_channels[band]].data[band_starts[band]+channel];
				readings.push_back(val);
				minpos = min(minpos, pos);
				maxpos = max(maxpos, pos);
			}
		}
	}

	return lines;
}

string PyGOMEReader::get_instrument_name(void) const
{
	return string("GOME");
}

// CCSDS* PyGOMEReader::get_gome_ccsds(void)
// {
//	// if we buffered up some packets initially return them first
//	if (!buffer.empty())
//	{
//		// if (hold)
//		// {
//			// delete hold;
//		// }
//		// hold = buffer.front();
//		buffer.pop();
//		return hold;
//	}
//	return source->get_next_ccsds();
// }
