////////////////////////////////////////////////////////////////////////////////
//
//	File: pygrasreader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pygrasreader.hpp"
#include "ccsds.hpp"

// Specialisation for GRAS Instrument
PyGRASReader::PyGRASReader( CCSDSProducer* source)
	: PyReader(source)
{
}

// get number of samples produced for a single scan line
int PyGRASReader::get_samples_count(void)
{
	return -1;
}

//	return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
PyObject* PyGRASReader::get_channels(void) const
{
	return Py_BuildValue("[(s,[s])]", "Subpackets", "Tracking state (C,TS,SAT,CHN,IMT,UTC)");
}

// return an array of tracking state tuples
PyObject* PyGRASReader::get_samples(int channel, int bands, int modulus, int count, int skip, int modulus2)
{
	// reset back to start of file cache
	reset_files();

	unsigned long long imt_ref = 0;	 // note, could get used uninitialised
	UTCTime utc_ref;
	bool time_set = false;

	PyObject* lines = PyList_New(0);

	//int cc=0;
	//int packet_count = 0;

	while ( CCSDS* packet = get_next_ccsds() )
	{

		unsigned int apid = packet->get_apid();

		if (apid < APID_GRAS_MEASUREMENT_1 || apid > APID_GRAS_MEASUREMENT_2)
		{
			continue;
		}

		//packet_count++;

		byte* data = packet->get_data();
		store_times(packet);
		//cout << "onboard UTC time " << (*packet->get_secondary_header()) << endl;

		UTCTime* utc_header = packet->get_secondary_header();

		//int packet_len = data[5] | data[4]<<8;
		int anc_len = data[21] | data[20]<<8;

		//printf("\n### Ancill data len = %d ###\n", anc_len);

		if (!time_set)
		{
			int aix = 24;
			while (aix < anc_len)
			{
				int packet_type = data[aix+1] & 0xF;
				int sub_packet_size = data[aix]<<4 | data[aix+1]>>4;

				//printf("packet_type = %d sub_packet_size = %d\n", packet_type, sub_packet_size);

				if ((packet_type == 1) && (sub_packet_size == 48))
				{
					uint64_be* imt = reinterpret_cast<uint64_be *>(data + aix + 16);
					uint16_be* days = reinterpret_cast<uint16_be *>(data + aix + 8);
					uint32_be* msec = reinterpret_cast<uint32_be *>(data + aix + 10);
					uint16_be d = data[aix+14]<<8 | data[aix+15];
					int usec = static_cast<int>(d & 0x3F);
					//int		us64 = static_cast<int>(d >> 10);

					utc_ref = make_utc_time(*days, *msec, usec);
					imt_ref = *imt;
					time_set = true;
					//cout << "\n*** Calibrating time: imt_ref=" << imt_ref << " utc_ref="
					//<< utc_ref << endl;
					break;

				}
				aix += sub_packet_size;
			}
		}


		int aix = 24;
		while (aix < (anc_len+14))
		{
			int packet_type = data[aix+1] & 0xF;
			int sub_packet_size = data[aix]<<4 | data[aix+1]>>4;

			//cout << "aix " << aix << " type " << packet_type << " size " << sub_packet_size << " ctr " << (aix-24)/2+5 << endl;

			if ((packet_type == 4) && (sub_packet_size == 12))
			{
				uint16_be d = data[aix+2]<<8 | data[aix+3];
				uint64_be* imt = reinterpret_cast<uint64_be*>(data + aix + 4);
				int C = static_cast<int>(d>>14 & 1);
				int TS = static_cast<int>(d>>10 & 0xF);
				int SAT = static_cast<int>(d>>4 & 0x3F);
				int CHN = static_cast<int>(d & 0xF);

				PyObject* time;
				if (time_set)		// we have seen the ENDP, get the IMT-based UTC
				{
					UTCTime utc_fine = utc_ref;
					long long imt_diff;
					if (*imt > imt_ref)
					{
						imt_diff = *imt - imt_ref;
					}
					else
					{
						imt_diff = - (imt_ref - *imt);
					}
					utc_fine.mod_micro(imt_diff * 344 / 10000);
					time = make_pyobject(utc_fine);
				}
				else				// no IMT/UTC reference yet, use UTC from packet header
				{
					//UTCTime utc_fine = utc_ref;
					//printf("\nNo time correction\n");
					time = make_pyobject(*utc_header);
				}

				//if (SAT==14 && CHN==9)
				//{
				//cout << cc++ << "--- Tracking State IMT=" << (*imt) << " UTCheader=" << utc_ref
				//<< " TS " << TS << endl;
				//}

				PyObject* tstate = Py_BuildValue("iiiiKO",
												 C, TS, SAT, CHN, static_cast<unsigned long long>(*imt), time);
				PyList_Append(lines, tstate);
				Py_DECREF(tstate);
				Py_DECREF(time);
			}
			aix += sub_packet_size;
		}
	}

	//cout << "Read " << packet_count << " packets\n";
	return lines;
}

// instrument name for this reader
string PyGRASReader::get_instrument_name(void) const
{
	return string("GRAS");
}
