////////////////////////////////////////////////////////////////////////////////
//
//	File: pyhirsreader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pyhirsreader.hpp"
#include "ccsds.hpp"

#include <intobject.h>

////////////////////////////////////////////////////////////////////////////////
//

PyHIRSReader::PyHIRSReader(CCSDSProducer* source)
	: PyReader(source)
{
}

///	 return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
PyObject* PyHIRSReader::get_channels(void) const
{
	// "(Samples,(Channel 1, Channel 2, Channel 3,...))"

	PyObject* samples = PyList_New(CHANNEL_COUNT);

	PyList_SetItem(samples, 0, ::make_pyobject("Channel 1"));
	PyList_SetItem(samples, 1, ::make_pyobject("Channel 17"));
	PyList_SetItem(samples, 2, ::make_pyobject("Channel 2"));
	PyList_SetItem(samples, 3, ::make_pyobject("Channel 3"));
	PyList_SetItem(samples, 4, ::make_pyobject("Channel 13"));
	PyList_SetItem(samples, 5, ::make_pyobject("Channel 4"));
	PyList_SetItem(samples, 6, ::make_pyobject("Channel 18"));
	PyList_SetItem(samples, 7, ::make_pyobject("Channel 11"));
	PyList_SetItem(samples, 8, ::make_pyobject("Channel 19"));
	PyList_SetItem(samples, 9, ::make_pyobject("Channel 7"));
	PyList_SetItem(samples, 10, ::make_pyobject("Channel 8"));
	PyList_SetItem(samples, 11, ::make_pyobject("Channel 20"));
	PyList_SetItem(samples, 12, ::make_pyobject("Channel 10"));
	PyList_SetItem(samples, 13, ::make_pyobject("Channel 14"));
	PyList_SetItem(samples, 14, ::make_pyobject("Channel 6"));
	PyList_SetItem(samples, 15, ::make_pyobject("Channel 5"));
	PyList_SetItem(samples, 16, ::make_pyobject("Channel 15"));
	PyList_SetItem(samples, 17, ::make_pyobject("Channel 12"));
	PyList_SetItem(samples, 18, ::make_pyobject("Channel 16"));
	PyList_SetItem(samples, 19, ::make_pyobject("Channel 19"));
	PyObject* band = PyTuple_New(2);
	PyTuple_SetItem(band, 0, ::make_pyobject("Channels"));
	PyTuple_SetItem(band, 1, samples);
	PyObject* bands = PyList_New(1);
	PyList_SetItem(bands, 0, band);

	return bands;
}

/// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
PyObject* PyHIRSReader::get_samples(int channel, int band, int modulus, int count, int skip, int modulus2)
{
	if (modulus2 == -1)
	{
		modulus2 = modulus;
	}

	set_count_filter(count, skip, modulus);

	// reset back to start of file cache
	reset_files();

	if ( channel < 0 || channel >= CHANNEL_COUNT )
	{
		PyErr_SetString(PyMetopizerError, "Given Channel is out of range" );
		return NULL;
	}
	else if ( modulus <= 0 )
	{
		modulus = 1;
	}

	PyObject* sample_list = PyList_New(0);
	int num_samples = max(1,NUM_SAMPLES/modulus2);

	while ( CCSDS* packet = get_next_ccsds() )
	{
		CCSDS_HIRS* hirs = reinterpret_cast<CCSDS_HIRS*>(packet->get_data());

		// first populate elements data and grap all channels for each sample
		for(int i=0; i<NUM_SAMPLES; i++)
		{
			elements[i] = get_hirs_element(hirs, i);
		}

		// Now put all subsampled values into a PyList
		PyObject* scan_list = PyList_New( num_samples );
		for(int i=0; i < num_samples; i++)
		{
			int value = elements[i*modulus].channels[channel];

			PyList_SetItem( scan_list, i, PyInt_FromLong(value) );
		}

		PyList_Append( sample_list, scan_list );
		Py_DECREF(scan_list);

		store_times(packet);
	}

	return sample_list;
}

/// instrument name for this reader
string PyHIRSReader::get_instrument_name(void) const
{
	return string("HIRS");
}

PyHIRSReader::ChannelData PyHIRSReader::get_hirs_element(CCSDS_HIRS* packet, int element)
{
	HIRSPayload& payload = packet->payload;

	ChannelData result;
	bzero(&result,sizeof(result));

	// skip elements and 4 13 bit words header section
	byte* payload_byte = ((byte*)(&(payload.elements[element])))+3;
	int payload_bit = 5;

	// ... each containing 22 words followed by 2 single bits
	for(int j=0; j<20; j++)
	{
		unsigned short word = 0;

		// .. using 13 bits per word
		for(int k=0; k<13; k++)
		{
			word <<= 1;
			if ( (((*payload_byte)&(1<<payload_bit))) != 0)
			{
				word |= 1;
			}

			if (payload_bit > 0)
			{
				payload_bit--;
			}
			else
			{
				payload_bit = 7;
				payload_byte++;
			}
		}

		if ((word&4096) != 0)
		{
			result.channels[j] = word&4095;
		}
		else
		{
			result.channels[j] = -(word&4095);
		}
	}
	return result;
}
