////////////////////////////////////////////////////////////////////////////////
//
//	File: pyiasireader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pyiasireader.hpp"
#include "ccsds.hpp"
#include "ccsdsproducer.hpp"

#include <intobject.h>

// BRD file search order:
// metopizer home directory (set at compile time, overridden by $METOPIZER_HOME)
// $METOPIZER_IASI_BRD_DIR (multiple dirs separated with colons)
// /tcc1/fbh/tcdras/store/gs1/IASI_BRD_xx_M01
// /tcc1/fbh/tcdras/store/gs1/IASI_BRD_xx_M02
// /QCF_DATA/DATA/ADF/M01/IASI/xx
// /QCF_DATA/DATA/ADF/M02/IASI/xx

// All these directories are scanned for IASI_BRD_xx_* and recorded.
// The record is a tuple of (SCID, BRD number) against filename
// Directories are scanned in the order above and new entries replace existing entries
// Files with higher filenames replace files with lower filenames within the same directory

// environment variable to read runtime home directory from
const string METOPIZER_HOME = "METOPIZER_HOME";

// glob for searching for BRD files
const string BRD_PATTERN = "IASI_BRD_xx_*";

// Hard coded default locations
const char* DEFAULT_BRD_DIRS[] =  {
	"/tcc1/fbf/tcdras/store/gs1/IASI_BRD_xx_M01",
	"/tcc1/fbf/tcdras/store/gs1/IASI_BRD_xx_M02",
	"/QCF_DATA/DATA/ADF/M01/IASI/xx",
	"/QCF_DATA/DATA/ADF/M02/IASI/xx"};

// alternative environment variable name to read directory(s) to read BRD files from
const string METOPIZER_IASI_BRD_DIR = "METOPIZER_IASI_BRD_DIR";

//
const int SOUNDER_CELLS = 30;

//
const int NO_OF_SPECTRA = 16*523;

PyIASIReader::BRDs PyIASIReader::brds;

PyIASIReader::PyIASIReader(CCSDSProducer* source)
	: PyReader(source)
{
}

//	return a list of strings
PyObject* PyIASIReader::get_channels(void) const
{
	// "(Samples,(Channel 1, Channel 2, Channel 3,...))"

	PyObject* bands = PyList_New(2);

	PyObject* samples = PyList_New(1);
	PyList_SetItem(samples, 0, ::make_pyobject("Data"));
	PyObject* band = PyTuple_New(2);
	PyTuple_SetItem(band, 0, ::make_pyobject("Imager"));
	PyTuple_SetItem(band, 1, samples);
	PyList_SetItem(bands, 0, band);

	int channels_count = NO_OF_SPECTRA;
	samples = PyList_New(channels_count);
	for(int i=0; i<channels_count; i++)
	{
		PyList_SetItem(samples, i, ::make_pyobject(string("Channel "+ntos(i))));
	}
	band = PyTuple_New(2);
	PyTuple_SetItem(band, 0, ::make_pyobject("Sounder"));
	PyTuple_SetItem(band, 1, samples);
	PyList_SetItem(bands, 1, band);

	return bands;
}

// return an array of 256-element long arrays. optional 2nd parameter gives modulus [only every 5th sample included])
PyObject* PyIASIReader::get_samples(int channel, int band, int modulus, int count, int skip, int modulus2)
{
	if (modulus2==-1)
	{
		modulus2 = modulus;
	}

	reset_files();

	switch(band)
	{
	case 0:
		return get_samples_imager(modulus, count, skip, modulus2);
	case 1:
		return get_samples_sounder(channel, modulus, count, skip, modulus2);
	default:
		PyException(PyMetopizerError, "Unsupported band '" << band << "'. Defined bands are 0 (Imager) and 1 (Sounder)");
		return NULL;
	}
}

PyObject* PyIASIReader::get_samples_imager(int modulus, int count, int skip, int modulus2)
{
	int crosstrack_mod = min(modulus2, 32);
	crosstrack_mod = max(crosstrack_mod, 1);
	int alongtrack_mod = min(modulus, 32);
	alongtrack_mod = max(alongtrack_mod, 1);

	//cout << "	 crosstrack = " << crosstrack_mod << "	alongtrack = " << alongtrack_mod << endl;

	Samples* line = new Samples;

	int cell_width = IIS_CELL_WIDTH / crosstrack_mod;
	int cell_height = IIS_CELL_HEIGHT / alongtrack_mod;
	int line_width = cell_width * 30;

	PyObject* sample_list = PyList_New(0);

	short_cds_time base_grh_time;
	UTCTime base_utc_time;
	OBTTime base_obt_time;

	memset(line, 0xff, sizeof(Samples));
	//bool line_empty = true;

	CCSDS* ccsds = NULL;
	int scan_position = 0;

	bool use_line = false;
	int prev_line = -1;

	// read in the image data here
	while ((ccsds = get_next_ccsds()))
	{
		// got next packet
		if (ccsds->get_apid() != CCSDS_IASI_Image::APID)
		{
			continue;
		}

		CCSDS_IASI_Image* iasi = (CCSDS_IASI_Image*)ccsds->get_data();

		byte* data = &iasi->payload;

		scan_position = static_cast<uint>(iasi->ancillary_data.word[1]&255);
		if (scan_position > 30)
		{
			continue;
		}

		//int num_samples = iasi->ancillary_data.word[0];
		int line_number = iasi->ancillary_data.word[7];
		//cout << "scan position " << scan_position << endl;

		if (line_number!=prev_line && use_line)
		{
			//if (!line_empty)
			//{
			push_samples(base_utc_time, base_obt_time, base_grh_time,
						 sample_list, line, line_width, cell_height, modulus, modulus2);
			//}
			memset(line, 0xff, sizeof(Samples));
			//line_empty = false;
		}

		if (line_number!=prev_line && prev_line!=-1)
		{
			use_line = true;
		}

		if (line_number!=prev_line)
		{
			base_utc_time = *ccsds->get_secondary_header();
			base_obt_time = *ccsds->get_ancillary_data();
			if (eps)
			{
				base_grh_time = eps->get_grh_start();
			}
		}

		if (scan_position==1)
		{
			use_line = true;
		}

		prev_line = line_number;

		// should be able to use bits_per_image_sample = IIS_SIZE = static_cast<uint>(
		// (iasi->ancillary_data.word[10]>>8)&255),
		// but the test data is of poor quality and does not allow this, hence use size of data instead.

		unsigned int bits_per_image_sample = 0;

		if ((ccsds->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 4100)
		{
			bits_per_image_sample = 8;
			for (int j=0; j<cell_height; j++)
			{
				for (int i=0; i<cell_width; i++)
				{
					int sample_number = i * (IIS_CELL_WIDTH * crosstrack_mod) + j * alongtrack_mod;
					int offset = sample_number + 4;
					Sample colour = data[offset];
					line->samples[(scan_position-1)*cell_width+i][j] = colour;
				}
			}
		}
		else if ((ccsds->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 5152)
		{
			bits_per_image_sample = 10;
			for (int j=0; j<cell_height; j++)
			{
				for (int i=0; i<cell_width; i++)
				{
					int sample_number = i * (IIS_CELL_WIDTH * crosstrack_mod) + j * alongtrack_mod;
					int offset = 5 * 8 + sample_number * 10;
					Sample colour = (data[offset/8] << 8) | data[offset/8+1];
					colour >>= (6 - (offset % 8));
					colour &= 0x000003FF;
					line->samples[(scan_position-1)*cell_width+i][j] = colour;
				}
			}
		}
		else if ((ccsds->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 6150)
		{
			bits_per_image_sample = 12;
			for (int j=0; j<cell_height; j++)
			{
				for (int i=0; i<cell_width; i++)
				{
					int sample_number = i * (IIS_CELL_WIDTH * crosstrack_mod) + j * alongtrack_mod;
					int offset = 6 * 8 + sample_number * 12;
					Sample colour = (data[offset/8] << 8) | data[offset/8+1];
					colour >>= (4 - (offset % 8));
					colour &= 0x00000FFF;
					line->samples[(scan_position-1)*cell_width+i][j] = colour;
				}
			}
		}
//		else
//		{
//			cout << "payload size is " << (ccsds->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) << endl;
//		}

		if ( bits_per_image_sample != static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255) )
		{
			PyException(PyMetopizerError, "WARNING: IIS_SIZE and the calculated bits per image sample ("
						<< bits_per_image_sample + ") do not match");
			return NULL;
		}
	}

	// spit out the last line
	//cout << "pushing last scanline\n";
	if (scan_position==30)
	{
		push_samples(base_utc_time,base_obt_time,base_grh_time,
					 sample_list, line, line_width, cell_height, modulus, modulus2);
	}

	delete line;
	return sample_list;
}

void PyIASIReader::push_samples(const UTCTime& base_utc_time,
								const OBTTime& base_obt_time,
								const short_cds_time& base_grh_time,
								PyObject* result,
								Samples* line,
								int line_width,
								int cell_height,
								int modulus,
								int modulus2)
{
	int offset_ms = static_cast<int>( (8000.0/64.0) * modulus);
	int offset_fine = static_cast<int>( (8000.0*65536.0/64.0) * modulus);

	//cout << "cell height " << cell_height << " offset_ms " << offset_ms << endl;

	for (int y=0; y<cell_height; y++)
	{
		PyObject* line_list = PyList_New(line_width);
		for (int x=0; x<line_width; x++)
		{
			Sample sample = line->samples[x][y];
			if (sample == -1)
			{
				Py_INCREF(Py_None);
				PyList_SetItem(line_list, x, Py_None);
			}
			else
			{
				PyList_SetItem(line_list, x, PyInt_FromLong(sample));
			}
		}
		PyList_Append(result, line_list);
		Py_DECREF(line_list);

		UTCTime utc_time = base_utc_time + y*offset_ms;
		utc_times.push_back(utc_time);

		OBTTime obt_time = base_obt_time + y*offset_fine;
		obt_times.push_back(obt_time);

		if (eps)
		{
			short_cds_time grh_time = base_grh_time + y*offset_ms;
			grh_times.push_back(grh_time);
		}
	}
}

void PyIASIReader::scan_brd_dirs(void)
{
	bool pydebug_brd = pydebug == "brd";

	// Default metopizer home (set at compile time)
	// $METOPIZER_HOME
	// $METOPIZER_IASI_BRD_DIR (multiple dirs separated with colons)
	// /tcc1/fbh/tcdras/store/gs1/IASI_BRD_xx_M01
	// /tcc1/fbh/tcdras/store/gs1/IASI_BRD_xx_M02
	// /QCF_DATA/DATA/ADF/M01/IASI/xx
	// /QCF_DATA/DATA/ADF/M02/IASI/xx

	// home dir (default or $METOPIZER_HOME)
	string homedir;
	const char* env = getenv(METOPIZER_HOME.c_str());
	if (env)
	{
		homedir = env;
	}
	else
	{
		homedir = DEFAULT_METOPIZER_HOME;
	}
	scan_brd_dir(homedir);

	// $METOPIZER_IASI_BRD_DIR
	const char* brd_dirs = getenv(METOPIZER_IASI_BRD_DIR.c_str());
	if (brd_dirs)
	{
		if (pydebug_brd)
		{
			cout << "Splitting METOPIZER_IASI_BRD_DIR of " << brd_dirs << endl;
		}
		stringstream ss(brd_dirs);
		string brd_dir;
		while(getline(ss, brd_dir, ':'))
		{
			scan_brd_dir(brd_dir);
		}
	}

	// built in supplemental dirs
	for(int i=0; i<sizeof(DEFAULT_BRD_DIRS)/sizeof(DEFAULT_BRD_DIRS[0]); i++)
	{
		scan_brd_dir(DEFAULT_BRD_DIRS[i]);
	}
}

void PyIASIReader::scan_brd_dir(const string& brd_dir)
{
	bool pydebug_brd = pydebug == "brd";
	if (pydebug_brd)
	{
		cout << "BRD scan dir " << brd_dir << endl;
	}

	StringList filenames = make_glob(brd_dir + "/" + BRD_PATTERN);
	for(StringList::iterator filename=filenames.begin(); filename!=filenames.end(); filename++)
	{
		//cout << "Loading " << *filename << endl;
		BRD* brd = new BRD;
		FILE* f = fopen((*filename).c_str(), "rb");
		if (!f)
		{
			cerr << "Cannot open BRD file '" << *filename << "'\n";
		}
		else
		{
			if (fread(brd, sizeof(BRD), 1, f)!=1)
			{
				fclose(f);
				cerr << "Cannot read " << sizeof(BRD) << " bytes from BRD file '" << *filename << "'\n";
			}
			else
			{
				fclose(f);
				int ptsi = brd->IDefPTSI;
				int metop = brd->IDefMETOPNumber; // 1 or 2
				if (pydebug_brd)
				{
					cout << "Inserting BRD SAT " << metop << " PTSI " << ptsi << " " << *filename << endl;
				}

				pair<int, int> index = pair<int, int>(metop, ptsi);
				if (brds.find(index) != brds.end())
				{
					delete brds[index];
				}
				brds[index] = brd;

/*I
  cout << "BRD info:\n"
  << "size " << sizeof(BRD) << " bytes\n"
  << "IDefPTSI (PTSI) " << brd->IDefPTSI << endl
  << "IDefParamDate " << brd->IDefParamDate << endl
  << "IDefMETOPNumber " << brd->IDefMETOPNumber << endl
  << "IDefCtcNbNsSeg (samples_per_segment) " << brd->IDefCtcNbNsSeg << endl
  << "ICtcNbSeg[0] (no_of_segments) " << brd->ICtcNbSeg[0] << endl
  << "ICtcNbSeg ";
  for(int i=0; i<4; i++)
  {
  cout << brd->ICtcNbSeg[i] << "	";
  }
  cout << endl
  << "IDefNSegNsFirst " << brd->IDefNSegNsFirst << endl
  << "IDefCtcNbNsSeg " << brd->IDefCtcNbNsSeg << endl
  << "ICtcTabOffsetC ";
  for(int i=0; i<4; i++)
  {
  cout << brd->ICtcTabOffsetC[i] << "	 ";
  }
  cout << endl
  << "ICtcTabOffsetM ";
  for(int i=0; i<4; i++)
  {
  cout << brd->ICtcTabOffsetM[i] << "	 ";
  }
  cout << endl
  << "ICtcTabScaleC ";
  for(int i=0; i<4; i++)
  {
  cout << brd->ICtcTabScaleC[i] << "	";
  }
  cout << endl
  << "ICtcTabScaleM ";
  for(int i=0; i<4; i++)
  {
  cout << brd->ICtcTabScaleM[i] << "	";
  }
  cout << endl
  << "IDefNsfirst " << brd->IDefNsfirst[0] << "	   " << brd->IDefNsfirst[1] << "	" << brd->IDefNsfirst[2] << endl
  << "IDefNslast " << brd->IDefNslast[0] << "	 " << brd->IDefNslast[1] << "	 " << brd->IDefNslast[2] << endl
  << "BitsPerSample	   Offset	 Scale\n";
  for(int i=0; i<55; i++)
  {
  cout << i << "	[" << brd->ICtcTabNbBit[0][i] << "	  " << brd->ICtcTabNbBit[1][i] << "	   "
  << brd->ICtcTabNbBit[2][i] << "	 " << brd->ICtcTabNbBit[3][i] << "]	   ["
  << brd->ICtcTabOffset[0][i] << "	  " << brd->ICtcTabOffset[1][i] << "	"
  << brd->ICtcTabOffset[2][i] << "	  " << brd->ICtcTabOffset[3][i] << "]	 ["
  << brd->ICtcTabScale[0][i] << "	 " << brd->ICtcTabScale[1][i] << "	  "
  << brd->ICtcTabScale[2][i] << "	 " << brd->ICtcTabScale[3][i] << "]\n";
  }
*/
			}
		}
	}
}

PyObject* PyIASIReader::get_samples_sounder(int channel, int modulus, int count, int skip, int modulus2)
{
	bool pydebug_brd = pydebug == "brd";

	PyObject* result = PyList_New(0);
	PyObject* two_lines[2];
	for(int i=0; i<2; i++)
	{
		two_lines[i] = PyList_New(SOUNDER_CELLS*2);
		for(int j=0; j<SOUNDER_CELLS*2; j++)
		{
			PyList_SetItem(two_lines[i], j, ::make_pyobject(0.0)); // Py_None);
		}
	}
	bool use_line = false;
	int prev_line = -1;
	CCSDS* ccsds = NULL;
	bool pydebug_packets = pydebug=="packets";
	int pos = -1;
	unsigned int apid = 0;

	while ((ccsds = get_next_ccsds()))
	{
		apid = ccsds->get_apid();
		if (apid != APID_IASI_PX1 && apid != APID_IASI_PX2 && apid != APID_IASI_PX3 &&
			apid != APID_IASI_PX4)
		{
			continue;
		}

		if (pydebug_packets)
		{
			cout << "APID " << apid << " seq " << ccsds->get_sequence_count() << endl;
		}

		const int px = 0; // px=0 *always* Other values of px (i.e. the other 3 sets of coeffs in the BRD file)
		// refer to non-nominal instrument modes

		CCSDS_IASI_Spec_PX1* iasi = reinterpret_cast<CCSDS_IASI_Spec_PX1*>(ccsds->get_data());

		int ptsi = iasi->ancillary_data.word[6];
		string scid = source->get_scid();
		pair<int, int> index;
		if (scid == "M01")
		{
			index = pair<int, int>(1, ptsi);
		}
		else if (scid == "M02")
		{
			index = pair<int, int>(2, ptsi);
		}
		else if (scid == "M03")
		{
			index = pair<int, int>(3, ptsi);
		}
		// cout << "Seeking scid " << scid << " ptsi " << ptsi << endl;
		BRDs::iterator brd_iter = brds.find(index);
		if (brd_iter == brds.end())
		{
			scan_brd_dirs();
			brd_iter = brds.find(index);
			if (brd_iter==brds.end())
			{
				PyException(PyMetopizerError,
							"Cannot find BRD file for SCID " << scid << " PTSI " << ptsi << endl);
				// << ". Search path is:\n - $METOPIZER_HOME/share/metopizer\n - "
				// "$METOPIZER_IASI_BRD_DIR\n - (from ./configure options) "
				// "$prefix/share/metopizer/IASI\n - " << DEFAULT_QCF_BRD_DIR << endl);
				return NULL;
			}
		}

		if (pydebug_brd)
		{
			cout << "Using coeffs for SCID " << scid << " PTSI " << ptsi << endl;
			pydebug_brd = false;
		}
		BRD& brd = *((*brd_iter).second);

		pos = iasi->ancillary_data.word[1]&255;
		int line = iasi->ancillary_data.word[7];
		//int payload_len = ccsds->get_payload_length() - 300 - 2;
		//cout << "APID " << ccsds->get_apid() << " payload len " << payload_len << " PTSI " << ptsi
		//<< " scan position " << pos << " line number " << line << endl;

		if (line != prev_line && use_line)
		{
			//cout << "line " << line << " prev_line " << prev_line << " pushing " << endl;
			for(int i=0; i<2; i++)
			{
				if (skip>0)
				{
					Py_DECREF(two_lines[i]);
					skip--;
				}
				else
				{
					PyList_Append(result, two_lines[i]);
					Py_DECREF(two_lines[i]);
					if (count>0)
					{
						if (--count==0)
						{
//							Py_DECREF(two_lines[0]);
//							if (i==0)
//							{
//								Py_DECREF(two_lines[1]);
//							}
							return result;
						}
					}
				}

				two_lines[i] = PyList_New(SOUNDER_CELLS*2);
				for(int j=0; j<SOUNDER_CELLS*2; j++)
				{
					PyList_SetItem(two_lines[i], j, ::make_pyobject(0.0)); // Py_None);
				}

				UTCTime utc_time = *ccsds->get_secondary_header() + i*4000;
				utc_times.push_back(utc_time);

				OBTTime obt_time = *ccsds->get_ancillary_data() + i*65536*4;
				obt_times.push_back(obt_time);

				if (eps)
				{
					short_cds_time grh_time = eps->get_grh_start() + i*4000;
					//cout << "grh time " << eps->get_grh_start() << " i " << i << " mod time " << grh_time << endl;
					grh_times.push_back(grh_time);
					//cout << "	   Pushline time " << grh_time << endl;
				}
			}
		}

		if (line!=prev_line && prev_line!=-1)
		{
			//cout << "Setting use_line line " << line << " prev_line " << prev_line << endl;
			use_line = true;
		}

		prev_line = line;

		if (apid==APID_IASI_PX1 && pos==1)
		{
			//cout << "Seting use_line apid " << apid << " pos " << pos << endl;
			use_line = true;
		}

		int bit_pos = 0;
		uint16_be* payload = reinterpret_cast<uint16_be*>(&(iasi->payload));
		int samples_per_segment = brd.IDefCtcNbNsSeg;
		int no_of_segments = brd.ICtcNbSeg[px];
		int cc = 0;
		for(int segment=0; segment<no_of_segments; segment++)
		{
			int sample_length = brd.ICtcTabNbBit[px][segment];
			//cout << "segment " << segment << " word length " << sample_length << endl;
			for(int sample=0; sample<samples_per_segment; sample++)
			{
				if (cc==channel && pos<=30)
				{
					//cout << "Reading channel " << channel << " at " << bit_pos << " (byte " << bit_pos/8
					//	 << " bit " << bit_pos%8 << ") sample_length " << sample_length << endl;

					unsigned int raw_result = 0;
					for(int i=0; i<sample_length; i++)
					{
						unsigned int bit = ((*(payload+bit_pos/16)) >> ((bit_pos%16))) & 1;
						raw_result ^= (bit<<i);
						bit_pos++;
					}
					double offset = brd.ICtcTabOffsetM[px] * static_cast<double>(brd.ICtcTabOffset[px][segment]) + brd.ICtcTabOffsetC[px];
					double scale = brd.ICtcTabScaleM[px] * static_cast<double>(brd.ICtcTabScale[px][segment]) + brd.ICtcTabScaleC[px];
					double result = (static_cast<double>(raw_result)-1.0+offset) / scale;
					if (pydebug_packets)
					{
						cout << "px " << px << " spectra " << cc << " raw_result " << raw_result
							 << " sample length " << sample_length << " offset " << offset
							 << " scale " << scale << " result " << result
							 << " initial bit pos " << (bit_pos-sample_length) << endl;
					}

					if (cc==channel)
					{
						switch(apid)
						{
						case APID_IASI_PX1:
							PyList_SetItem(two_lines[0], (pos-1)*2+1, ::make_pyobject(result));
							break;
						case APID_IASI_PX2:
							PyList_SetItem(two_lines[1], (pos-1)*2+1, ::make_pyobject(result));
							break;
						case APID_IASI_PX3:
							PyList_SetItem(two_lines[1], (pos-1)*2, ::make_pyobject(result));
							break;
						case APID_IASI_PX4:
							PyList_SetItem(two_lines[0], (pos-1)*2, ::make_pyobject(result));
							break;
						default:
							break;
						}
					}

					if (!pydebug_packets)
					{
						segment = no_of_segments;
						break;
					}
				}
				else
				{
					bit_pos += sample_length;
				}
				cc++;
			}
		}

		//cout << "Finished reading, bit_pos " << bit_pos << " (" << bit_pos/8 << " bytes)\n";
	}

	if (apid == APID_IASI_PX4 && pos == 30)
	{
		for(int i=0; i<2; i++)
		{
			PyList_Append(result, two_lines[i]);
		}
	}

	for(int i=0; i<2; i++)
	{
		Py_DECREF(two_lines[i]);
	}

	return result;
}

// instrument name for this reader
string PyIASIReader::get_instrument_name(void) const
{
	return string("IASI");
}
