////////////////////////////////////////////////////////////////////////////////
//
//	File: pymetopizer.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"

#include "pyreader.hpp"
#include "pymhsreader.hpp"
#include "pyhirsreader.hpp"
#include "pyamsuareader.hpp"
#include "pyavhrrreader.hpp"
#include "pyascatreader.hpp"
#include "pyiasireader.hpp"
#include "pygomereader.hpp"
#include "pysemreader.hpp"
#include "pygrasreader.hpp"
#include "pyhktmreader.hpp"

#include "filereader.hpp"
#include "ccsdsproducer.hpp"
#include "ccsdsreader.hpp"
#include "ccsdsapidfilter.hpp"

#include <intobject.h>

#ifdef PY3K
#include <unicodeobject.h>
#define PyString_FromString PyUnicode_FromString
#define PyString_Check PyUnicode_Check
//#define PyString_AsString PyUnicode_AsASCIIString
const char* PyString_AsString(PyObject* object)
{
	PyObject* bytes = PyUnicode_AsASCIIString(object);
	return PyBytes_AsString(bytes);
}
#endif

// this is a Python header
#include "datetime.h"
const int GMTIME_BASE_YEAR = 1900;

// This must come after all Metopizer structure definitions that rely on
// having no compiler padding, but before declaring any structures
// defined in Python header files
#pragma pack()

//int obj_count = 0;

//
// Global variable section
//
textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

PyObject* PyMetopizerError = NULL;
PyObject* PyUnsupportedFileError = NULL;
PyObject* PyUnrecognisedFileError = NULL;

PY_TIME_TYPE py_time_type = PY_TIME_DATETIME;
string pydebug;
ofstream outstream("/dev/null");

//
// The definition of a Python PyMetopizer.PyProduct object
//

struct PyProduct
{
	PyObject_HEAD;
	PyReader* reader;
	string filename;
};

//
// PyMetopizer.PyProduct functions
//

extern "C" PyObject* pymetopizer_set_time_type(PyObject* self, PyObject* args)
{
	char* datatype = NULL;

	//PyObject_Print(args, stdout, 0); cout.flush();

	if (!PyArg_ParseTuple(args, "s", &datatype))
	{
		PyErr_SetString(PyMetopizerError, "String parameter required for time type.");
		return NULL;
	}

	if (!strcasecmp(datatype, "gentime"))
	{
		py_time_type = PY_TIME_GENTIME;
	}
	else if (!strcasecmp(datatype, "datetime"))
	{
		py_time_type = PY_TIME_DATETIME;
	}
	else if (!strcasecmp(datatype, "matlab"))
	{
		py_time_type = PY_TIME_MATLAB;
	}
	else
	{
		PyErr_SetString(PyMetopizerError, "String [gentime|datetime|matlab] required for time type");
		return NULL;
	}

	Py_INCREF(Py_None);
	return Py_None;
}

extern "C" PyObject* pymetopizer_set_debug(PyObject* self, PyObject* args)
{
	char* new_pydebug;
	if (!PyArg_ParseTuple(args, "s", &new_pydebug))
	{
		PyException(PyMetopizerError, "parameter must be a single string");
		return NULL;
	}
	pydebug = new_pydebug;
	cout << "Metopizer set debug string to '" << pydebug << "'\n";
	Py_INCREF(Py_None);
	return Py_None;
}

extern "C" int PyProduct_init(PyProduct* self, PyObject *args, PyObject *keywords)
{
	const char* filename = NULL;
	static const char *keyword_list[] = {"filename", NULL};
	if (!PyArg_ParseTupleAndKeywords(args, keywords, "s", const_cast<char**>(keyword_list), &filename))
	{
		return -1;
	}

	// create memory space for the filename string
	FileProducer* source = new FileReader(filename);
	CCSDSProducer* reader = new CCSDSReader(*source);

	// Attempt to get a valid group id for this reader by reading first packet
	CCSDSProducer::GROUPID datatype = reader->get_data_type();

	if (datatype==CCSDSProducer::UNRECOGNISED)
	{
		string instname = string(filename, 4);
		if (instname=="MHSx")
		{
			datatype = CCSDSProducer::MHS;
		}
		else if (instname=="AMSU")
		{
			datatype = CCSDSProducer::AMSUA;
		}
		else if (instname=="AVHR")
		{
			datatype = CCSDSProducer::AVHRR;
		}
		else if (instname=="HIRS")
		{
			datatype = CCSDSProducer::HIRS;
		}
		else if (instname=="ASCA")
		{
			datatype = CCSDSProducer::ASCAT;
		}
		else if (instname=="IASI")
		{
			datatype = CCSDSProducer::IASI;
		}
		else if (instname=="GOME")
		{
			datatype = CCSDSProducer::GOME;
		}
		else if (instname=="GRAS")
		{
			datatype = CCSDSProducer::GRAS;
		}
		else if (instname=="HKTM")
		{
			datatype = CCSDSProducer::HKTM;
		}
		else if (instname=="SEMx")
		{
			datatype = CCSDSProducer::SEM;
		}
	}

	switch (datatype)
	{
	case CCSDSProducer::MHS:
		self->reader = new PyMHSReader(reader);
		break;
	case CCSDSProducer::AMSUA:
		self->reader = new PyAMSUAReader(reader);
		break;
	case CCSDSProducer::AVHRR:
		self->reader = new PyAVHRRReader(reader);
		break;
	case CCSDSProducer::HIRS:
		self->reader = new PyHIRSReader(reader);
		break;
	case CCSDSProducer::ASCAT:
		self->reader = new PyASCATReader(reader);
		break;
	case CCSDSProducer::IASI:
		self->reader = new PyIASIReader(reader);
		break;
	case CCSDSProducer::GOME:
		self->reader = new PyGOMEReader(reader);
		break;
	case CCSDSProducer::GRAS:
		self->reader = new PyGRASReader(reader);
		break;
	case CCSDSProducer::SEM:
		self->reader = new PySEMReader(reader);
		break;
	case CCSDSProducer::HKTM:
		self->reader = new PyHKTMReader(reader);
		break;
	case CCSDSProducer::ATOVS:
	case CCSDSProducer::ADCS:
	case CCSDSProducer::SBUV:
	case CCSDSProducer::DUMMY:
	case CCSDSProducer::GENERIC:
	case CCSDSProducer::UNSUPPORTED:
		PyException(PyUnsupportedFileError, "Unsupported file '" << filename << "'");
		reader->delete_source();
		delete reader;
		return -1;
	case CCSDSProducer::FILE_ERROR:
		PyException(PyExc_IOError, "Unable to open file '" << filename << "'");
		reader->delete_source();
		delete reader;
		return -1;
	case CCSDSProducer::UNRECOGNISED:
	default:
		PyException(PyUnrecognisedFileError, "Unrecognised file '" << filename << "'");
		reader->delete_source();
		delete reader;
		return -1;
	}

	if ( self->reader == NULL )
	{
		return -1;
	}

	new (&self->filename) string(filename==NULL?"":filename);

//	  obj_count++;
//	  cout << "metopizer create, " << obj_count << " objects present\n";

	return 0;
}

extern "C" void PyProduct_dealloc(PyProduct* self)
{
	if (self->reader)
	{
		delete self->reader;
		self->filename.~string();
		self->reader = NULL;
	}
#ifdef PY3K
	self->ob_base.ob_type->tp_free((PyObject*)self);
#else
	self->ob_type->tp_free((PyObject*)self);
#endif

//	  obj_count--;
//	  cout << "metopizer delete, " << obj_count << " objects present\n";
}

extern "C" PyObject* PyProduct_name(PyProduct* self)
{
	return Py_BuildValue("s", "PyProduct");
}

extern "C" PyObject* PyProduct_str(PyProduct* self)
{
	string pname = string("PyProduct<") + self->filename
		+ "," + self->reader->get_instrument_name();
	return Py_BuildValue( "s", pname.c_str() );
}

//	return a list of strings ["Channel1", "Channel2", "Channel3", "Channel4", "Channel5"]
extern "C" PyObject* PyProduct_get_channels(PyProduct* self)
{
	return self->reader->get_channels();
}

// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
extern "C" PyObject* PyProduct_get_samples(PyProduct* self, PyObject* args, PyObject *keywords)
{
	PyObject* channel = NULL;
	PyObject* band = NULL;
	int modulus = 1;
	int count = -1;
	int skip = 0;
	int modulus2 = -1;
	static const char *keyword_list[] = {"channel", "band", "modulus", "count", "skip", "modulus2", NULL};

	PyObject* result = NULL;

	if (!PyArg_ParseTupleAndKeywords(args, keywords, "O|Oiiii", const_cast<char**>(keyword_list), &channel, &band, &modulus, &count, &skip, &modulus2))
	{
		//cout << "args ";
		//PyObject_Print(args, stdout, 0); cout.flush();
		//cout << endl;
		//cout << "keywords ";
		//PyObject_Print(keywords, stdout, 0); cout.flush();
		//cout << endl;

		PyErr_SetString(PyMetopizerError, "Requires at least 'channel' parameter. 'band', 'modulus', \
'count', 'skip' and 'modulus2' are optional");
		return NULL;
	}

	//cout << "channel ";
	//PyObject_Print(channel, stdout, 0);

	//cout << " band ";
	//PyObject_Print(band, stdout, 0);
	//cout << endl;

	int channel_val = 0;
	PyObject* channel_iter = NULL;
	if (PyList_Check(channel) || PyTuple_Check(channel))
	{
		//cout << "channel iter\n";
		channel_iter = PyObject_GetIter(channel);
	}
	else
	{
		if (PyInt_Check(channel))
		{
			//cout << "channel int\n";
			channel_val = PyInt_AsLong(channel);
			//Py_DECREF(channel);
		}
		else
		{
			PyException(PyMetopizerError, "channel parameter must be an integer or a list of integers");
			return NULL;
		}
	}

	int band_val = 0;
	PyObject* band_iter = NULL;
	if (band)
	{
		if (PyList_Check(band) || PyTuple_Check(band))
		{
			//cout << "band iter\n";
			band_iter = PyObject_GetIter(band);
		}
		else
		{
			if (PyInt_Check(band))
			{
				//cout << "band int\n";
				band_val = PyInt_AsLong(band);
				//Py_DECREF(band);
			}
			else
			{
				PyException(PyMetopizerError, "band parameter must be an integer or a list of integers");
				if (channel_iter)
				{
					Py_DECREF(channel_iter);
				}
				return NULL;
			}
		}
	}

	if (channel_iter || band_iter)
	{
		result = PyList_New(0);
		do
		{
			if (channel_iter)
			{
				PyObject* channel_item = PyIter_Next(channel_iter);
				if (channel_item)
				{
					if (!PyInt_Check(channel_item))
					{
						PyException(PyMetopizerError, "All elements of channels list must be integers");
						Py_DECREF(channel_iter);
						if (band_iter)
						{
							Py_DECREF(band_iter);
						}
						Py_DECREF(channel_item);
						return NULL;
					}
					channel_val = PyInt_AsLong(channel_item);
					Py_DECREF(channel_item);
				}
				else
				{
					Py_DECREF(channel_iter);
					//cout << "channel refcount " << channel->ob_refcnt << endl;
					//Py_DECREF(channel); // ??
					channel_iter = NULL;
				}
			}

			if (band_iter)
			{
				PyObject* band_item = PyIter_Next(band_iter);
				if (band_item)
				{
					if (!PyInt_Check(band_item))
					{
						PyException(PyMetopizerError, "All elements of bands list must be integers");
						Py_DECREF(band_iter);
						if (channel_iter)
						{
							Py_DECREF(channel_iter);
						}
						Py_DECREF(band_item);
						return NULL;
					}
					band_val = PyInt_AsLong(band_item);
					Py_DECREF(band_item);
				}
				else
				{
					Py_DECREF(band_iter);
					//Py_DECREF(band); // ??
					band_iter = NULL;
				}
			}

			if (band_iter || channel_iter)
			{
				// cout << "channel_val " << channel_val << " band_val " << band_val << endl;
				PyObject* c = self->reader->get_samples(channel_val, band_val, modulus, count, skip, modulus2);
				if (!c)
				{
					return NULL;
				}
				PyList_Append(result, c);
				Py_DECREF(c);
				//Py_INCREF(Py_None);
				//PyList_Append(result, Py_None);
			}
		} while(channel_iter || band_iter);

//		cout << "returning result list with refcount " << result->ob_refcnt << endl;
//		//PyListObject* l = (PyListObject*)result;
//		for(int i=0; i<PyList_Size(result); i++)
//		{
//			PyObject* memb = PyList_GetItem(result,i);
//			cout << "	 member " << i << " refcnt " << memb->ob_refcnt << endl;
//		}
		return result;
	}

	return self->reader->get_samples(channel_val,band_val,modulus,count,skip,modulus2);
}

// getters for Time fields within all packets
extern "C" PyObject* PyProduct_get_obt_times(PyProduct* self)
{
	return self->reader->get_obt_times();
}

extern "C" PyObject* PyProduct_get_utc_times(PyProduct* self)
{
	return self->reader->get_utc_times();
}

extern "C" PyObject* PyProduct_get_grh_times(PyProduct* self)
{
	return self->reader->get_grh_times();
}

extern "C" PyObject* PyProduct_get_instrument_name(PyProduct* self)
{
	return Py_BuildValue("s", self->reader->get_instrument_name().c_str() );
}

extern "C" PyObject* PyProduct_get_mphr_fields(PyProduct* self, PyObject* args)
{
	if (PyString_Check(args))
	{
		string field(PyString_AsString(args));
		string value = self->reader->get_mphr_field(field);
		if (value.empty())
		{
			return NULL;
		}
		return make_pyobject(value);
	}
	else if (PyTuple_Check(args))
	{
		StringList result;
		for(int i=0; i<PyTuple_Size(args); i++)
		{
			PyObject* item = PyTuple_GetItem(args, i);
			//PyObject_Print(item, stdout, 0); cout.flush();
			if (!PyString_Check(item))
			{
				PyException(PyExc_TypeError, "Tuple contains non-string value");
				return NULL;
			}
			string field(PyString_AsString(item));
			if (field.empty())
			{
				return NULL;
			}
			result.push_back(self->reader->get_mphr_field(field));
		}
		return make_pyobject(result);
	}
	else
	{
		PyException(PyExc_TypeError, "Parameter must be a string or tuple of strings");
		return NULL;
	}
}

extern "C" PyObject* PyProduct_get_gaps(PyProduct* self, PyObject* args)
{
	return make_pyobject("hello");
}

//
// PyMetopizer.PyProduct type definition
//

static PyMethodDef PyProductMethods[] = {
	{"name", (PyCFunction)PyProduct_name, METH_NOARGS,
	 "Return the name of the object"},
	{"get_channels",(PyCFunction)PyProduct_get_channels, METH_NOARGS,
	 "get all channel names"},
	{"get_samples",(PyCFunction)PyProduct_get_samples, METH_KEYWORDS,
	 "get all samples of given channel and apply optional modulus"},
	{"get_obt_times",(PyCFunction)PyProduct_get_obt_times, METH_NOARGS,
	 "get all OBT Times as doubles"},
	{"get_utc_times",(PyCFunction)PyProduct_get_utc_times, METH_NOARGS,
	 "get all UTC Times as strings"},
	{"get_grh_times",(PyCFunction)PyProduct_get_grh_times, METH_NOARGS,
	 "get all GRH Times as strings"},
	{"get_instrument_name",(PyCFunction)PyProduct_get_instrument_name, METH_NOARGS,
	 "get instrument name used by this PyProduct"},
	{"get_mphr_fields", (PyCFunction)PyProduct_get_mphr_fields, METH_VARARGS,
	 "look up MPHR field(s)"},
	{"get_gaps", (PyCFunction)PyProduct_get_gaps, METH_VARARGS,
	 "return information on gaps in L0 product, based on CCSDS sequence counts"},
	{NULL}
};

static PyTypeObject PyProductType = {
	PyObject_HEAD_INIT(NULL)
#ifndef PY3K
	0,							   /*ob_size*/
#endif
	"metopizer.Product",		 /*tp_name*/
	sizeof(PyProduct),			   /*tp_basicsize*/
	0,							   /*tp_itemsize*/
	(destructor)PyProduct_dealloc, /*tp_dealloc*/
	0,							   /*tp_print*/
	0,							   /*tp_getattr*/
	0,							   /*tp_setattr*/
	0,							   /*tp_compare*/
	(reprfunc)PyProduct_str,	   /*tp_repr*/
	0,							   /*tp_as_number*/
	0,							   /*tp_as_sequence*/
	0,							   /*tp_as_mapping*/
	0,							   /*tp_hash */
	0,							   /*tp_call*/
	0,							   /*tp_str*/
	0,							   /*tp_getattro*/
	0,							   /*tp_setattro*/
	0,							   /*tp_as_buffer*/
	Py_TPFLAGS_DEFAULT,			   /*tp_flags*/
	"Metopizer Product",		 /*tp_doc */
	0,							   /*tp_traverse*/
	0,							   /*tp_clear*/
	0,							   /*tp_richcompare*/
	0,							   /*tp_weaklistoffset*/
	0,							   /*tp_iter*/
	0,							   /*tp_iternext*/
	PyProductMethods,			   /*tp_methods*/
	0,							   /*tp_members*/
	0,							   /*tp_getset*/
	0,							   /*tp_base*/
	0,							   /*tp_dict*/
	0,							   /*tp_descr_get*/
	0,							   /*tp_descr_set*/
	0,							   /*tp_dictoffset*/
	(initproc)PyProduct_init,		 /*tp_init*/
	PyType_GenericAlloc,		   /*tp_alloc*/
	PyType_GenericNew,			   /*tp_new*/
	0							   /*tp_free*/
};

static PyMethodDef metopizer_methods[] = {
	{"set_time_type", pymetopizer_set_time_type, METH_VARARGS,
	 "Set time format to return for individual products"},
	{"set_debug", pymetopizer_set_debug, METH_VARARGS,
	 "Set debug string. Do not use."},

	{NULL, NULL, 0, NULL}
};

#ifdef PY3K
PyDoc_STRVAR(metopizer_module_documentation,
			 "The functions in this module allow compression and decompression using the\n"
			 "zlib library, which is based on GNU zip.\n"
			 "\n"
			 "adler32(string[, start]) -- Compute an Adler-32 checksum.\n"
			 "compress(string[, level]) -- Compress string, with compression level in 1-9.\n"
			 "compressobj([level]) -- Return a compressor object.\n"
			 "crc32(string[, start]) -- Compute a CRC-32 checksum.\n"
			 "decompress(string,[wbits],[bufsize]) -- Decompresses a compressed string.\n"
			 "decompressobj([wbits]) -- Return a decompressor object.\n"
			 "\n"
			 "'wbits' is window buffer size.\n"
			 "Compressor objects support compress() and flush() methods; decompressor\n"
			 "objects support decompress() and flush().");


static struct PyModuleDef metopizermodule = {
	PyModuleDef_HEAD_INIT,
	"metopizer",
	metopizer_module_documentation,
	-1,
	metopizer_methods,
	NULL,
	NULL,
	NULL,
	NULL
};
#endif

//
// Entry point from Python interpreter
//


#ifdef PY3K
extern "C" PyMODINIT_FUNC PyInit_metopizer(void)
#else
	extern "C" PyMODINIT_FUNC initmetopizer(void)
#endif
{
	putenv((char*)"TZ=UTC");
	tzset();

	// initialise the local global variables
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;
	_debug = &outstream;

#ifdef PY3K
	PyObject* m = PyModule_Create(&metopizermodule);
#else
	// PyObject* m = Py_InitModule("metopizer", metopizer_methods);
	PyObject* m = Py_InitModule4("metopizer", metopizer_methods, NULL, NULL, PYTHON_API_VERSION);
#endif

	// Error handler object
	PyMetopizerError = PyErr_NewException(const_cast<char*>("metopizer.error"), NULL, NULL);
	PyUnsupportedFileError = PyErr_NewException(const_cast<char*>("metopizer.UnsupportedFileError"), NULL, NULL);
	PyUnrecognisedFileError = PyErr_NewException(const_cast<char*>("metopizer.UnrecognisedFileError"), NULL, NULL);

	Py_INCREF(PyMetopizerError);
	Py_INCREF(PyUnsupportedFileError);
	Py_INCREF(PyUnrecognisedFileError);

	PyModule_AddObject(m, "error", PyMetopizerError);
	PyModule_AddObject(m, "UnsupportedFileError", PyUnsupportedFileError);
	PyModule_AddObject(m, "UnrecognisedFileError", PyUnrecognisedFileError);

	PyObject* md = PyModule_GetDict(m);
	// version string, major, minor, trivial, svn info, build date

	PyObject* version = PyTuple_New(6);
	istringstream str(VERSION);
	PyTuple_SetItem(version, 0, PyString_FromString(str.str().c_str()));
	StringList tokens;
	while(!str.eof())
	{
		string token;
		getline(str, token, '.');
		tokens.push_back(token);
	}
	switch(tokens.size())
	{
	case 2:
		PyTuple_SetItem(version, 1, ::make_pyobject(tokens[0]));
		PyTuple_SetItem(version, 2, ::make_pyobject(tokens[1]));
		PyTuple_SetItem(version, 3, ::make_pyobject(0));
		break;
	case 3:
		PyTuple_SetItem(version, 1, ::make_pyobject(tokens[0]));
		PyTuple_SetItem(version, 2, ::make_pyobject(tokens[1]));
		PyTuple_SetItem(version, 3, ::make_pyobject(tokens[2]));
		break;
	}
	PyTuple_SetItem(version, 4, ::make_pyobject(__DATE__));
	PyTuple_SetItem(version, 5, ::make_pyobject("exported"));
	PyDict_SetItem(md, ::make_pyobject("version"), version);

	if (PyType_Ready(&PyProductType) < 0)
	{
#ifdef PY3K
		return m;
#else
		return;
#endif
	}

	Py_INCREF(&PyProductType);

	PyModule_AddObject(m, "Product", (PyObject*)&PyProductType);
#ifdef PY3K
	return m;
#else
	return;
#endif
}

PyObject* make_pyobject(const StringList& l)
{
	PyObject* result = PyTuple_New(l.size());
	for(StringList::const_iterator iter=l.begin();
		iter!=l.end();
		iter++)
	{
		PyTuple_SetItem(result, iter-l.begin(), make_pyobject(*iter));
	}
	return result;
}

PyObject* make_pyobject(const string& s)
{
	return PyString_FromString(s.c_str());
}

PyObject* make_pyobject(int i)
{
	return PyInt_FromLong(i);
}

PyObject* make_pyobject(double d)
{
	return PyFloat_FromDouble(d);
}

PyObject* make_pyobject(const UTCTime& utc_time)
{
	switch (py_time_type)
	{
	case PY_TIME_MATLAB:
	{
		double time_val =
			TimeConv::utc_days_milliseconds_to_matlab(utc_time.days, utc_time.milliseconds);
		return PyFloat_FromDouble(time_val);
	}
	break;
	case PY_TIME_DATETIME:
	{
		struct tm my_tm =
			TimeConv::utc_days_milliseconds_to_components(utc_time.days, utc_time.milliseconds);
		if (!PyDateTimeAPI)
		{
			PyDateTime_IMPORT;
		}
		return PyDateTime_FromDateAndTime(my_tm.tm_year+GMTIME_BASE_YEAR, my_tm.tm_mon+1, my_tm.tm_mday,
										  my_tm.tm_hour, my_tm.tm_min, my_tm.tm_sec,
										  (utc_time.milliseconds%1000)*1000);
	}
	break;
	case PY_TIME_GENTIME:
	default:
	{
		string time_val =
			make_short_cds_time(utc_time.days,	utc_time.milliseconds).to_long_gen_time();
		return PyString_FromString(time_val.c_str());
	}
	break;
	}
	assert(0);
	return NULL;
}
