////////////////////////////////////////////////////////////////////////////////
//
//	File: pymhsreader.cpp
//
//	Provides: python EPS Product reader class
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

#include "pymetopizer.hpp"
#include "pymhsreader.hpp"
#include "ccsds.hpp"

#include <intobject.h>

const int CHANNEL_COUNT = 5;
const int NUM_SAMPLES = 90;

// Specialisation for MHS Instrument with APID of 34
PyMHSReader::PyMHSReader( CCSDSProducer* source)
	: PyReader(source)
{
}

//	return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
PyObject* PyMHSReader::get_channels(void) const
{
	// "(Samples,(Channel 1, Channel 2, Channel 3,...))"

	PyObject* samples = PyList_New(CHANNEL_COUNT);
	for(int i=0; i<CHANNEL_COUNT; i++)
	{
		PyList_SetItem(samples, i, ::make_pyobject(string("Channel ")+ntos(i+1)));
	}
	PyObject* band = PyTuple_New(2);
	PyTuple_SetItem(band, 0, ::make_pyobject("Channels"));
	PyTuple_SetItem(band, 1, samples);
	PyObject* bands = PyList_New(1);
	PyList_SetItem(bands, 0, band);

	return bands;
}

// get number of samples produced for a single scan line
int PyMHSReader::get_samples_count(void)
{
	return NUM_SAMPLES;
}

// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
PyObject* PyMHSReader::get_samples(int channel, int bands, int modulus, int count, int skip, int modulus2)
{
	if ( modulus <= 0 )
	{
		modulus = 1;
	}

	if (modulus2 == -1)
	{
		modulus2 = modulus;
	}

	set_count_filter(count, skip, modulus);

	// reset back to start of file cache
	reset_files();

	if ( channel < 0 || channel >= CHANNEL_COUNT )
	{
		PyErr_SetString(PyMetopizerError, "Given Channel is out of range" );
		return NULL;
	}

	PyObject* sample_list = PyList_New(0);

	int num_samples = max(1,NUM_SAMPLES/modulus2);
	while ( CCSDS* packet = get_next_ccsds() )
	{
		MHSPayload& mhs = reinterpret_cast<CCSDS_MHS*>(packet->get_data())->payload;

		PyObject* scan_list = PyList_New( num_samples );
		for(int i=0; i<num_samples;i++)
		{
			// get value directly from payload data & apply modulus
			uint16_be item = mhs.earth_view[i*modulus2].channels[channel];
			PyList_SetItem( scan_list, i, PyInt_FromLong(item) );
		}
		PyList_Append( sample_list, scan_list );
		Py_DECREF(scan_list);

		store_times(packet);
	}

	return sample_list;
}

// instrument name for this reader
string PyMHSReader::get_instrument_name(void) const
{
	return string("MHS");
}
