////////////////////////////////////////////////////////////////////////////////
//
//	File: pyreader.cpp
//
//	Provides: python interface
//
//	Purpose:
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"

// this is a Python header
#include "datetime.h"
const int GMTIME_BASE_YEAR = 1900;

#include "pymetopizer.hpp"
#include "pyreader.hpp"
#include "ccsdsproducer.hpp"

#ifdef PY3K
#include <unicodeobject.h>
#define PyString_FromString PyUnicode_FromString
#endif

// Python Specific EPS Product reader, generic base class to handle
// CCSDS packet data extraction
PyReader::PyReader(CCSDSProducer* _source, const string filterbyapid )
	: utc_times(),
	  obt_times(),
	  grh_times(),
	  source(_source),
	  skip_count(0),
	  read_count(0),
	  modulus_count(0),
	  skip_filter(0),
	  count_filter(0),
	  modulus_filter(0),
	  first_reset(true)
{
	eps = source->as_eps_product();
}

PyReader::~PyReader()
{
	source->delete_source();
	delete source;
}

// Get the next available CCSDS packet
CCSDS* PyReader::get_next_ccsds(void)
{
	// get the next available packet, possibly APID filtered already
	CCSDS* ccsds = source->get_next_ccsds();

	if (!ccsds)
	{
		return NULL;
	}

	// Count filtering
	if (count_filter > 0)
	{
		if (read_count > 0)
		{
			read_count--;
		}
		else
		{
			return NULL;
		}
	}

	// Skip filtering
	if (skip_filter > 0)
	{
		while(skip_count > 0)
		{
			ccsds = source->get_next_ccsds();
			if (ccsds == NULL)
			{
				return NULL;
			}
			skip_count--;
		}
	}

	// Modulus AND Count Filtering
	if (modulus_filter > 1)
	{
		modulus_count--;
		while (modulus_count>0)
		{
			if (!ccsds || ((count_filter > 0) && (read_count == 0)))
			{
				return NULL;
			}
			else
			{
				modulus_count--;
				// also check if count filtering is not being used
				if (modulus_count > 0 && read_count > 0)
				{
					read_count--;
				}
				ccsds = source->get_next_ccsds();
			}
		}
		modulus_count = modulus_filter;
	}

	return ccsds;
}

// reset files pointer back to first file and first byte
void PyReader::reset_files(void)
{
	if (!first_reset)
	{
		// make sure the file streams is at the beginning
		source->reset_stream();

		skip_count = skip_filter;
		read_count = count_filter;
		modulus_count = 0;

		utc_times.clear();
		obt_times.clear();
		grh_times.clear();
	}
	else
	{
		first_reset = false;
	}
}

// scan only the first packet
void PyReader::set_count_filter(int count, int skip, int modulus )
{
	if (count_filter>=0)
	{
		count_filter = count;
		read_count = count_filter;
	}

	if (skip>=0)
	{
		skip_filter = skip;
		skip_count = skip_filter;
	}

	if (modulus>=0)
	{
		modulus_filter = modulus;
		modulus_count = 0;
	}
}

void PyReader::store_times( CCSDS* ccsds )
{
	if (ccsds->has_secondary_header())
	{
		UTCTime utc_time = *(ccsds->get_secondary_header());
		utc_times.push_back(utc_time);
	}

	OBTTime obt_time = *ccsds->get_ancillary_data();
	obt_times.push_back(obt_time);

	if (eps)
	{
		short_cds_time grh_time = eps->get_grh_start();
		grh_times.push_back(grh_time);
	}
}


// getters for Time fields within all packets
PyObject* PyReader::get_obt_times(void)
{
	if (obt_times.empty())
	{
		//PyErr_SetString(PyMetopizerError, "Call get_samples() before reading any time values" );
		//return NULL;
		return PyList_New(0);
	}

	PyObject* obt_list = PyList_New(obt_times.size());

	for (unsigned int i=0;i<obt_times.size();i++)
	{
		// need to show as double time value
		double timeval = (
			static_cast<double>(obt_times[i].obt_coarse) +
			( static_cast<double>(obt_times[i].obt_fine) /
			  static_cast<double>(OBTTime::FINE_TICKS_PER_COARSE_TICK) ) );

		PyList_SetItem( obt_list,i, PyFloat_FromDouble(timeval) );
	}

	return obt_list;
}

PyObject* PyReader::get_utc_times(void)
{
	if ( utc_times.empty() )
	{
		return PyList_New(0);
		//PyErr_SetString(PyMetopizerError,
		//		"No UTC times available. Call get_samples() first if they are present in the input file");
		//return NULL;
	}

	PyObject* utc_list = PyList_New( utc_times.size() );

	for (unsigned int i=0;i<utc_times.size();i++)
	{
		PyList_SetItem( utc_list, i, make_pyobject(utc_times[i]));
	}

	return utc_list;
}

PyObject* PyReader::get_grh_times(void)
{
	if ( grh_times.empty() )
	{
		//PyErr_SetString(PyMetopizerError, "No GRH times available as this is not a L0 product" );
		//return NULL;
		return PyList_New(0);
	}

	PyObject* grh_list = PyList_New( grh_times.size() );

	for (unsigned int i=0;i<grh_times.size();i++)
	{
		switch (py_time_type)
		{
		case PY_TIME_MATLAB:
		{
			double time_val = TimeConv::utc_days_milliseconds_to_matlab(
				grh_times[i].get_days(),
				grh_times[i].get_milliseconds() );
			PyList_SetItem( grh_list,i, PyFloat_FromDouble(time_val) );
		}
		break;
		case PY_TIME_DATETIME:
		{
			struct tm my_tm = TimeConv::utc_days_milliseconds_to_components(
				grh_times[i].get_days(),
				grh_times[i].get_milliseconds());
			if (!PyDateTimeAPI)
			{
				PyDateTime_IMPORT;
			}
			PyList_SetItem(grh_list,
						   i,
						   PyDateTime_FromDateAndTime(
							   my_tm.tm_year + GMTIME_BASE_YEAR,
							   my_tm.tm_mon + 1,
							   my_tm.tm_mday,
							   my_tm.tm_hour,
							   my_tm.tm_min,
							   my_tm.tm_sec,
							   (grh_times[i].get_milliseconds() % 1000) * 1000));
		}
		break;
		case PY_TIME_GENTIME:
		default:
		{
			string time_val = grh_times[i].to_long_gen_time();
			PyList_SetItem(grh_list, i, PyString_FromString(time_val.c_str()));
		}
		break;
		}
	}

	return grh_list;
}

string PyReader::get_mphr_field(const string& field) const
{
	if (eps)
	{
		return eps->get_mphr_field(field);
	}
	else
	{
		PyException(PyExc_TypeError, "Input file is not an EPS product");
		return "";
	}
}
