////////////////////////////////////////////////////////////////////////////////
//
//	File: pyreader.hpp
//
//	Provides: python EPS Product reader class
//
//	Purpose: Code for representing source packets as Python Objects
//
//	Copyright (c) 2006 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef pyreader_defined
#define pyreader_defined

#include "ccsds.hpp"
#include "epsproducer.hpp"

// Forward declare methods
class CCSDSProducer;

////////////////////////////////////////////////////////////////////////////////
// Python Specific EPS Product reader, generic base class to handle
/// CCSDS packet data extraction
class PyReader : public Interface
{
public:
	/// Default constructor stores source and filter string if any
	PyReader(CCSDSProducer* _source,const string filterbyapid = "");
	virtual ~PyReader(void);

	///	 return a list of strings ["Channel1". "Channel2", "Channel3", "Channel4", "Channel5"]
	virtual PyObject* get_channels(void) const=0;

	/// return a array of 90-element long arrays. optionalal 2nd parameter gives modulus [only every 5th sample included])
	virtual PyObject* get_samples(int channel, int band, int modulus, int count, int skip, int modulus2) =0;

	/// instrument name for this reader
	virtual string get_instrument_name(void) const=0;

	/// getters for Time fields within all accepted packets
	PyObject* get_obt_times(void); // return array of doubles
	PyObject* get_utc_times(void); // return array of strings
	PyObject* get_grh_times(void); // return array of strings

	string get_mphr_field(const string& field) const;

protected:

	/// scan only the first packet
	void set_count_filter(int count, int skip, int modulus);

	/// Default implementation of get next available CCSDS
	virtual CCSDS* get_next_ccsds(void);

	/// reset files pointer back to first file and first byte
	virtual void reset_files(void);

	/// Insert new times data into three time arrays
	void store_times( CCSDS* ccsds );

	typedef vector<UTCTime> UTCTimes;
	typedef vector<OBTTime> OBTTimes;
	typedef vector<short_cds_time> GRHTimes;

	/// Cached storage for all three time types
	/// Times are only stored here if they correspond to returned scanlines
	/// from the get_samples function. There are several conditions that can cause
	/// these arrays to be smaller than the number of MDRs in a L0 product:
	///	 - Some instruments combine multiple packets into a single scanline
	///	 - L0 product may include packets that do not contain image-able data
	///	 - The client may have requested subsampling.
	UTCTimes utc_times;
	OBTTimes obt_times;
	GRHTimes grh_times;

	/// packet extractor stored as CCSDS source
	CCSDSProducer* source;
	EPSProducer* eps;

private:

	/// mainatain internal counters to control get_next_ccsds() function
	int skip_count;
	int read_count;
	int modulus_count;

	/// values remain fixed after call to count_filter
	int skip_filter;
	int count_filter;
	int modulus_filter;

	/// Flag used to determine when stream should be reset
	bool first_reset;
};

#endif
