////////////////////////////////////////////////////////////////////////////////
//
//	File: reducedccsdswriter.hpp
//
//	Provides: ReducedCCSDSWriter
//
//	Purpose: Accept a stream of standard CCSDS, convert them to Reduced CCSDS
//			 packets by removing the UTC time, then writing them to disk in a
//			 heirarchy used by MHS test tools.
//
//	Copyright (c) 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "interface.hpp"

class ReducedCCSDSWriter : public Interface
{
public:

	ReducedCCSDSWriter(CCSDSProducer& source, const string& _base_dirname)
		: base_dirname(_base_dirname),
		  packet_count(0)
		{
			if (mkdir(base_dirname.c_str(),
					  S_IRUSR|S_IWUSR|S_IXUSR|
					  S_IRGRP|S_IWGRP|S_IXGRP|
					  S_IROTH|S_IWOTH|S_IXOTH)==-1 && errno!=EEXIST)
			{
				error << "Cannot create directory " << base_dirname << endl;
				return;
			}

			int counter = 0;
			int last_minute = -1;
			string last_dirname;

			while(CCSDS* ccsds = source.get_next_ccsds())
			{
				int milli = ccsds->get_utc_milliseconds();
				string thisdir = TimeConv::utc_days_milliseconds_to_string(
					ccsds->get_utc_days(), milli, "%y%m%d%H.dd");

				string dirname = base_dirname + "/" + thisdir;
				if (dirname != last_dirname)
				{
					dirs.push_back(thisdir);
					last_dirname = dirname;
				}

				if (mkdir(dirname.c_str(),
						  S_IRUSR|S_IWUSR|S_IXUSR|
						  S_IRGRP|S_IWGRP|S_IXGRP|
						  S_IROTH|S_IWOTH|S_IXOTH)==-1 && errno!=EEXIST)
				{
					error << "Cannot create directory " << dirname << endl;
					return;
				}

				int minute = (milli/60000)%60;
				//int hour = static_cast<int>(milli/3600000);

				if (minute!=last_minute)
				{
					counter = 0;
				}
				else
				{
					counter++;
				}
				last_minute = minute;

				string filename = ntofs(minute,2) + ntofs(counter,2) + ".sd";

				byte* in_buff = ccsds->get_data();
				int in_len = ccsds->get_length();
				int out_len = in_len - 8;

				byte* out_buff = new byte[out_len];
				memcpy(out_buff, in_buff, 6);
				memcpy(out_buff+6, in_buff+14, out_len-6);
				bzero(out_buff+out_len-2, 2);

				reinterpret_cast<CCSDSPrimaryHeader*>(
					out_buff)->set_packet_length(1293);

				string fullname = dirname + "/" + filename;

				ofstream64 of(fullname.c_str());
				of.write(reinterpret_cast<char*>(out_buff), out_len);

				delete[] out_buff;

				packet_count++;
			}
		}

	virtual void output_text_summary( ostream& os = cout )
		{
			os << "ReducedCCSDSWriter report:\n"
			   << "	   Base directory: " << base_dirname << "/\n"
			   << "	   Subdirectories: ";
			for(StringList::iterator iter=dirs.begin();
				iter!=dirs.end();
				iter++)
			{
				os << (*iter) << "/ ";
			}
			os << "\n	 Packet count: " << packet_count << endl;
		}

private:

	string base_dirname;
	int packet_count;
	StringList dirs;
};
