////////////////////////////////////////////////////////////////////////////////
//
//	File: sband_tm_displayer.cpp
//
//	Provides: sband_tm_displayer executable
//
//	Purpose: Display information on single S-Band telemetry packets or
//			 streams of packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "sband.hpp"
#include "filereader.hpp"
#include "sbandreader.hpp"
#include "readfromstdin.hpp"
#include "sbandtmdisplayer.hpp"
#include "textsummarydisplayer.hpp"
#include "textrecorddisplayer.hpp"
#include "args.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

int gerror = 0; // global error
const string APPNAME = "sband_tm_displayer";
const string MAINTAINER = "elson@eumetsat.de";
const string COPYRIGHT = "Copyright (c) 2005 Eumetsat.";

int main(int argc, char* argv[])
{
	ofstream onull("/dev/null");
	_debug = &onull;
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;

	deque<string> in_file;
	bool verbose = true;

	int count = -1;
	int skip = 0;

	Args args;
	args.set_doc_head("Decode and display the contents of single/multiple s-band telemetry packets.");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright(COPYRIGHT);
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("ccsds_displayer");

	args.add_example("Example 1", "sband_tm_displayer -i /data/sband.test -k 20 -c 5");
	args.add_example("Example 2", "sband_tm_displayer --in /data/sband.test --skip 20 --count 5");

	args.add_group("File selection:");
	args.add(&in_file, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME,
			 "Select input file(s)", "If omitted packets are read from standard input");

	args.add_group("Packet selection:");
	args.add(&count, "count", 'c', 0, "Display only <n> packets");
	args.add(&skip, "skip", 'k', 0, "Skip <n> packets before processing");

	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case 'q':
			verbose = false;
			break;

		case Args::USER_OPTION:
			_debug = &cout;
			break;

		case 'z':
			Spinner::use_simple_spinner();
			break;
		}
	}


	if (!verbose || isatty(1)!=1)
	{
		spinner->restream(&onull);
	}


	// set UTC time zone
	TimeConv::set_time_zone();

	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}

	SBandReader reader(*first_source);
	SBandTMDisplayer disp(reader, skip, count);
	TextRecordDisplayer trd(disp, -1, true);


	delete first_source;
	return 0;
}
