////////////////////////////////////////////////////////////////////////////////
//
//	File: sbandreader.cpp
//
//	Provides: SBandReader filter
//
//	Purpose: Extract S-Band telemetry packets from a plain binary stream
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "sbandreader.hpp"


//
// Constructor
//

SBandReader::SBandReader( FileProducer& _source)
	: source(_source)
	, buffer_pos(0)
	, buffer_size(0)
	, in_stream(NULL)
	, good_packets(0)
	, skipped_bytes(0)
	, gap_size(0) // track any holes in the input stream
	, file_pos(0)

{
	DEBUG("s-band constructor\n");
}




//
// output_text_summary
//

void SBandReader::output_text_summary( textoutstream_type& os )
{
	skipped_bytes += (buffer_size-buffer_pos);

	os << "S-Band Reader Report:\n"
	   << "	   " << good_packets << " s-band packets found\n"
	   << "	   " << skipped_bytes << " bytes were not used\n";

	if (gaps.empty())
	{
		os << "	   No gaps found between s-band packets\n";
	}
	else
	{
		os << "	   Gaps found between s-band packets: ";
		for(Gaps::iterator iter=gaps.begin(); iter!=gaps.end(); iter++)
		{
			if (iter!=gaps.begin())
			{
				os << ", ";
			}
			os << (*iter).first << " bytes (" << (*iter).second
			   << " packets)";
		}
		os << endl;
	}

	if (gap_size > 0)
	{
		os << "	   " << gap_size << " byte gap at end of "
		   << "stream\n";
	}
	os << endl;
}



//
// get_next_sband
//

SBand* SBandReader::get_next_sband(void)
{

	DEBUG("sbandreader::get_next_sband start\n");

	SBand* result = NULL;
	// try alternately scanning the buffer for S-Band packets or
	// reading data from disk into the buffer until we either
	// hit the end of the input stream, or find a S-Band packet.
	do
	{
		result = scan_buffer();
		if (!result)
		{
			if (!fill_buffer())
			{
				// if fill_buffer returns false there is no more input
				// data
				return NULL;
			}
		}
	} while(!result);

	good_packets++;
	return result;
}



//
// scan_buffer
//

// look for s-band packet in the buffer
SBand* SBandReader::scan_buffer(void)
{
	int spincc = 0;

	if (!spinner.get())
	{
		spinner.reset(new Spinner());
	}

	const uint32_be MAGIC_NUMBER = 0x1ACFFC1D;

	DEBUG("buffer_pos is " << buffer_pos << " buffer_size is "
		  << buffer_size << endl);

	for(; (int)buffer_pos<=
			(int)((int)buffer_size-(int)sizeof(SBand));
		buffer_pos++)
	{
		if (spincc++%100==0)
		{
			spinner->tick();
		}


		SBand* candidate = reinterpret_cast<SBand*>((buffer+buffer_pos));
//		if (candidate->sync_marker == MAGIC_NUMBER)
		if (!memcmp(&(candidate->sync_marker[0]), &MAGIC_NUMBER,
					sizeof(MAGIC_NUMBER)))
		{
			// we've found a match
			// raise a warning if another sync marker is found inside
			// the packet (this might happen occasionally by
			// coincidence)

			// (disabled for performance)
			for(
				int i=sizeof(uint32_be);
				i<(sizeof(SBand)-sizeof(uint32_be));
				i++)
			{
				if ( reinterpret_cast<SBand*>(buffer+buffer_pos+i)->get_sync_marker()==MAGIC_NUMBER )
				{
					warning << "WARNING SBandReader::scan_buffer found "
							<< "unexpected sync marker inside packet "
							<< good_packets
							<< " at offset " << i << endl;
				}
			}

			buffer_pos += sizeof(SBand);

			if (gap_size > 0)
			{
				DEBUG("SBandReader::scan_buffer found gap of "
					  << gap_size << " unreadable bytes\n");

				Gaps::iterator iter=gaps.find(gap_size);
				if (iter==gaps.end())
				{
					gaps[gap_size] = 1;
				}
				else
				{
					(*iter).second++;
				}

				gap_size = 0;
			}

			file_pos += sizeof(SBand);
			return candidate;
		}
		else
		{
			skipped_bytes++;
			gap_size++;

			if (gap_size == WARNING_GAP_SIZE)
			{
				spinner->pause();
				warning << "Gap found between s-band packets in input "
						<< "stream at position " << file_pos
						<< "\nSize " << gap_size << " bytes and"
						<< " still scanning\n";
				spinner->unpause();

			}
			else if (gap_size % WARNING_GAP_SIZE == 0)
			{
				spinner->pause();
				warning << "Gap of " << gap_size << " bytes and still"
						<< " scanning (gap started at pos "
						<< (file_pos-gap_size) << " bytes)\n";
				spinner->unpause();
			}
		}

		file_pos++;

	}
	return NULL;
}



//
// fill_buffer
//

// grab more input data and place into the buffer
bool SBandReader::fill_buffer(void)
{
	DEBUG("sbandreader::fill_buffer start\n");

	if (!in_stream)
	{
		DEBUG("getting next file\n");
		in_stream = source.get_next_file();
		if (!in_stream)
		{
			DEBUG("end of stream\n");
			// return fail if no data was read
			return false;
		}
	}

	DEBUG("sbandreader::fill_buffer initial buffer_pos " << buffer_pos
		  << " buffer_size " << buffer_size << endl);

	// firstly we shift any remaining data to the front of the buffer
	int bytes_remaining = buffer_size - buffer_pos;
	memmove(buffer, buffer+buffer_pos, bytes_remaining);
	buffer_pos = 0;//bytes_remaining;
	buffer_size = bytes_remaining;

	DEBUG("sbandreader::fillbuffer postmove buffer_pos " << buffer_pos
		  << " buffer_size " << buffer_size << endl);

	// now fill the top, using as many input files as needed
	while(buffer_size < BUFFER_SIZE && in_stream)
	{
		DEBUG("sbandreader::fillbuffer reading "
			  << (BUFFER_SIZE-buffer_size)
			  << " bytes into position " << (buffer_size) << endl);

		in_stream->read((char*)buffer+buffer_size,
						BUFFER_SIZE-buffer_size);
		int count = in_stream->gcount();

		DEBUG("sbandreader::fillbuffer got " << count << endl);

		if (count <= 0)
		{
			// end of file, look for another one

			DEBUG("sbandreader::fillbuffer reading new file...\n");

			in_stream = source.get_next_file();
			if (!in_stream)
			{
				DEBUG("	   ...failed\n");
			}
			DEBUG("	   ...ok\n");
		}
		else
		{
			buffer_size += count;
		}
	}

	buffer_pos = 0;
	return true;
}





