////////////////////////////////////////////////////////////////////////////////
//
//	File: spinner.cpp
//
//	Provides: Spinner class
//
//	Purpose: Display an ascii animation during long calculations
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "spinner.hpp"

#ifdef _SIMPLE_SPINNER
bool Spinner::simple_spinner = true;
#else
bool Spinner::simple_spinner = false;
#endif

auto_ptr<Spinner> spinner; 
textoutstream_type* Spinner::outstr=&cerr;
int Spinner::type=-1;

Spinner::Spinner(const string& _message)
	: ended(false)
	, activated(false)
	, message(_message)
{
	activate_time = current_time() + 1000;
	activate_time += INITIAL_DELAY;

	earliest_end_time = current_time() + MINIMUM_SPIN_TIME;
}

void Spinner::tick(void)
{
	unsigned int time_now = current_time();

	// not ready to start up yet
	if (!activated && time_now < activate_time)
	{
		return;
	}

	if (simple_spinner)
	{
		if (!activated && time_now >= activate_time)
		{
			(*outstr) << "Processing...\n";
			activated = true;
			next_spin_time = time_now + STAR_DELAY;
		}
		if (time_now >= next_spin_time)
		{
			(*outstr) << "*";
			next_spin_time = time_now + STAR_DELAY;
		}
		return;
	}

	// initialisation message
	if (!activated && time_now >= activate_time)
	{
		next_spin_time = time_now + SPIN_DELAY;
		next_dot_time = time_now + DOT_DELAY;

		if (!message.empty())
		{
			(*outstr) << message << "\n.  ";
		}
		else
		{
			(*outstr) << "Processing\n.	 ";
		}
		activate_time = 0;

		if (type == -1)
		{
			type = time_now % 9;
		}

		switch(type)
		{
		case 0:
			symbols = "|/-\\";
			break;
			
		case 1:
			symbols = "|\\-/";
			break;
			
		case 2:
			symbols = ".o0o";
			break;
			
		case 3:
			symbols = "..o0O0o";
			break;
			
		case 4:
			symbols = "((||))||";
			break;
			
		case 5:
			symbols = "^^>>vv<<";
			break;
			
		case 6:
			symbols = "__-^-";	
			break;
		
		case 7:
			symbols = "_-^\"	  ";
			break;
			
		case 8:
			symbols = "|)}]>-  ";
			break;
		}
		
		pos = 0; 
		activated = true;
	} // end of initialisation

	if (time_now >= next_spin_time)
	{
		char out[6];
		int i=0;
		out[i++] = '\b';
		out[i++] = '\b';

		if (time_now >= next_dot_time)
		{
			out[i++] = '.';
			out[i++] = symbols[pos];
			out[i++] = ' ';
			next_dot_time = time_now + DOT_DELAY;
		}
		else
		{
			out[i++] = symbols[pos];
			out[i++] = ' ';
			pos = (pos+1)%symbols.size();
			next_spin_time = time_now+SPIN_DELAY;
		}

		out[i++] = '\0';
		(*outstr).write(out,i);
	}
}

void Spinner::end(void)
{
	while(current_time() < earliest_end_time)
	{
		tick();
	}

	if (activated && !ended)
	{
		if (simple_spinner)
		{
			(*outstr) << endl << endl;
		}
		else
		{
			(*outstr) << "\b\b	\n";
		}
		ended = true;
	}
}

void Spinner::pause(void)
{
	if (activated)
	{
		(*outstr) << "\b\b	\n";
	}
}

void Spinner::unpause(void)
{
	if (activated)
	{
		(*outstr) << "	";
	}
}

unsigned int Spinner::current_time(void)
{
	struct timeb t;
	ftime(&t);
	return (t.millitm + t.time*1000);
}

void Spinner::restream( textoutstream_type* new_outstr)
{
	outstr = new_outstr;
}

void Spinner::use_simple_spinner(bool use)
{
	simple_spinner = use;
}
