////////////////////////////////////////////////////////////////////////////////
//
//	File: subsat.hpp
//
//	Purpose: Handle database of subsatellite positions
//
//	Copyright (c) 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "subsat.hpp"

//
//
// Subsat::Pos
//
//

Subsat::Pos::Pos(void)
	: lat(1.0),
	  lon(1.0)
{
}

bool Subsat::Pos::operator != (const Subsat::Pos& o) const
{
	return (lat!=o.lat || lon!=o.lon);
}

ostream& operator << (ostream& os, const Subsat::Pos& pos)
{
	os << pos.lat << ", " << pos.lon;
	return os;
}

//
//
// Subsat
//
//

Subsat::Subsat(void)
{
}

Subsat::Subsat(const string& dirname)
	: warned(false)
{
	DIR* dir = opendir(dirname.c_str());
	if (!dir)
	{
        //cerr << "Warning: could not open subsat directory '" << dirname << "'\n";
		warned = true;
		return;
	}
	struct dirent* entry;
	while((entry=readdir(dir)))
	{
		if (strlen(entry->d_name)>10 &&
			strncmp(entry->d_name, "subsat", 6)==0 &&
			strncmp(entry->d_name+strlen(entry->d_name)-4,".txt",4)==0)
		{
			scan_file(dirname + "/" + entry->d_name);
		}
	}
	closedir(dir);
}

void Subsat::scan_file(const string& filename)
{
//	  cout << "scan_file " << filename << endl;
	//int old_poscount = pos.size();

	FILE* file = fopen(filename.c_str(),"r");
	if (!file)
	{
		cerr << "Cannot open subsatellite file " << filename << endl;
		return;
	}

	bool reading = false;
	char line[1024];
	while(fgets(line, 1024, file) != NULL)
	{
		if (strlen(line)>4)
		{
			if (!reading)
			{
				if (strncmp(line,"----",4)==0)
				{
					reading = true;
				}
			}
			else
			{
				int day;
				char month_str[4];
				int year;
				int hour;
				int min;
				int sec;
				Pos new_pos;
				sscanf(line,"%d %3s %d %2d:%2d:%2d.00%*[ ]%lf%*[ ]%lf",
					   &day,month_str,&year,&hour,&min,&sec,&new_pos.lat,
					   &new_pos.lon);
				month_str[3] = '\0';
				int month;
				if (!strcmp(month_str,"Jan"))
				{
					month = 1;
				}
				else if (!strcmp(month_str,"Feb"))
				{
					month = 2;
				}
				else if (!strcmp(month_str,"Mar"))
				{
					month = 3;
				}
				else if (!strcmp(month_str,"Apr"))
				{
					month = 4;
				}
				else if (!strcmp(month_str,"May"))
				{
					month = 5;
				}
				else if (!strcmp(month_str,"Jun"))
				{
					month = 6;
				}
				else if (!strcmp(month_str,"Jul"))
				{
					month = 7;
				}
				else if (!strcmp(month_str,"Aug"))
				{
					month = 8;
				}
				else if (!strcmp(month_str,"Sep"))
				{
					month = 9;
				}
				else if (!strcmp(month_str,"Oct"))
				{
					month = 10;
				}
				else if (!strcmp(month_str,"Nov"))
				{
					month = 11;
				}
				else if (!strcmp(month_str,"Dec"))
				{
					month = 12;
				}
//		cout << "LINE '" << line << "' day " << day << " month "
//		<< month_str << " year " << year << " hour " << hour <<
//		" min " << min << " sec " << sec << " lat " << lat << " lon "
//		<< lon << endl;

				struct tm my_tm;
				bzero(&my_tm, sizeof(my_tm));
				my_tm.tm_sec = sec;
				my_tm.tm_min = min;
				my_tm.tm_hour = hour;
				my_tm.tm_mday = day;
				my_tm.tm_mon = month - 1;
				my_tm.tm_year = year - BASE_YEAR;
				my_tm.tm_wday = 0;
				my_tm.tm_yday = 0;
				my_tm.tm_isdst = 0;

				time_t julian_t = mktime(&my_tm) - UTC_TO_JULIAN_SECONDS;

				short_cds_time cds;
				cds.day = julian_t / SECONDS_PER_DAY;
				cds.millisec_day = ((julian_t)%SECONDS_PER_DAY)*1000;

				pos[cds] = new_pos;

			}
		}
	}
	fclose(file);

//	  if (!quiet)
//	  {
//	cout << "Read " << pos.size()-old_poscount
//		 << " subsatellite positions from " << filename << endl;
//	  }
}

Subsat::Pos Subsat::get_pos(short_cds_time t) const
{
//	  if (!quiet)
//	  {
//	cout << "Looking up " << t << " from " << pos.size()
//		 << " subsat records\n";
//	  }

	t.millisec_day -= t.millisec_day % 1000;

	map<short_cds_time,Pos>::const_iterator iter = pos.find(t);
	if (iter==pos.end())
	{
		if (!warned)
		{
//		cout << "Warning: could not find subsatellite position for time "
//		 << t << " (further warnings will be supressed)\n";
			warned = true;
		}
		return Pos();
	}
	else
	{
		return ((*iter).second);
	}
}
