////////////////////////////////////////////////////////////////////////////////
//
//	File: tiff.hpp
//
//	Provides: TIFF class
//
//	Purpose: Generate TIFF format files
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef tiff_defined
#define tiff_defined

#define SHIFT(x) x<<16
//#define SHIFT(x) x

class TIFF : public Image
{
public:

	//
	// constructor
	//

	TIFF(void)
		: tiff_image(NULL),
		  ypos(0)
		{
			memset(&tiff_header, 0, sizeof(tiff_header));

			tiff_header.ifh.byte_order = 0x4d4d; // big-endian
			tiff_header.ifh.magic_number = 42;
			tiff_header.ifh.ifd_offset = sizeof(IFH);
			tiff_header.no_of_ifds = 13;
			tiff_header.next_ifd = 0;

			tiff_header.new_subfile.tag = TIFF_NewSubfile;
			tiff_header.new_subfile.type = TIFF_LONG;
			tiff_header.new_subfile.count = 1;
			tiff_header.new_subfile.offset = 0;

			tiff_header.image_width.tag = TIFF_ImageWidth;
			tiff_header.image_width.type = TIFF_LONG;
			tiff_header.image_width.count = 1;

			tiff_header.image_height.tag = TIFF_ImageHeight;
			tiff_header.image_height.type = TIFF_LONG;
			tiff_header.image_height.count = 1;

			tiff_header.bits_per_sample.tag = TIFF_BitsPerSample;
			tiff_header.bits_per_sample.type = TIFF_SHORT;
			tiff_header.bits_per_sample.count = 3;
			tiff_header.bits_per_sample.offset = (byte*)(&(tiff_header.bpp1)) -
				(byte*)(&tiff_header);
			tiff_header.bpp1 = 8;
			tiff_header.bpp2 = 8;
			tiff_header.bpp3 = 8;

			tiff_header.compression.tag = TIFF_Compression;
			tiff_header.compression.type = TIFF_SHORT;
			tiff_header.compression.count = 1;
			tiff_header.compression.offset = SHIFT(1);

			tiff_header.photometric_interpretation.tag =
				TIFF_PhotometricInterpretation;
			tiff_header.photometric_interpretation.type = TIFF_SHORT;
			tiff_header.photometric_interpretation.count = 1;
			tiff_header.photometric_interpretation.offset = SHIFT(2);

			tiff_header.strip_offsets.tag = TIFF_StripOffsets;
			tiff_header.strip_offsets.type = TIFF_LONG;
			tiff_header.strip_offsets.count = 1;
			tiff_header.strip_offsets.offset = sizeof(tiff_header);;

			tiff_header.samples_per_pixel.tag = TIFF_SamplesPerPixel;
			tiff_header.samples_per_pixel.type = TIFF_SHORT;
			tiff_header.samples_per_pixel.count = 1;
			tiff_header.samples_per_pixel.offset = SHIFT(3);

			tiff_header.rows_per_strip.tag = TIFF_RowsPerStrip;
			tiff_header.rows_per_strip.type = TIFF_LONG;
			tiff_header.rows_per_strip.count = 1;

			tiff_header.strip_byte_counts.tag = TIFF_StripByteCounts;
			tiff_header.strip_byte_counts.type = TIFF_LONG;
			tiff_header.strip_byte_counts.count = 1;

			tiff_header.x_resolution.tag = TIFF_XResolution;
			tiff_header.x_resolution.type = TIFF_RATIONAL;
			tiff_header.x_resolution.count = 1;
			tiff_header.x_resolution.offset = (byte*)(&(tiff_header.x1)) -
				(byte*)(&tiff_header);
			tiff_header.x1 = 1;
			tiff_header.x2 = 1;

			tiff_header.y_resolution.tag = TIFF_YResolution;
			tiff_header.y_resolution.type = TIFF_RATIONAL;
			tiff_header.y_resolution.count = 1;
			tiff_header.y_resolution.offset = (byte*)(&(tiff_header.y1)) -
				(byte*)(&tiff_header);
			tiff_header.y1 = 1;
			tiff_header.y2 = 1;;

			tiff_header.resolution_unit.tag = TIFF_ResolutionUnit;
			tiff_header.resolution_unit.type = TIFF_SHORT;
			tiff_header.resolution_unit.count = 1;
			tiff_header.resolution_unit.offset = SHIFT(2); // inches
		}

	//
	// destructor
	//

	virtual ~TIFF(void)
		{
			delete[] tiff_image;
		}

	//
	// get_filetype
	//

	virtual string get_filename_extension(void) const
		{
			return ".tiff";
		}

	//
	// set_size
	//

	virtual void begin_output(void)
		{
			tiff_header.image_width.offset = width;
			tiff_header.image_height.offset = height;
			tiff_header.rows_per_strip.offset = height+1;
			tiff_header.strip_byte_counts.offset = width*height*3;

			tiff_image = new byte[width*height*3];
			memset(tiff_image, 0, width*height*3);
		}

	//
	// set_pixel
	//

	virtual void write_line(byte* line)
		{
			byte* base = tiff_image + ypos*width*3;
			if (colour_type == IMAGE_COLOUR)
			{
				memcpy(base, line, width*3);
			}
			else
			{
				for(int x=0; x<width; x++)
				{
					*(base+x*3) = line[x];
					*(base+x*3+1) = line[x];
					*(base+x*3+2) = line[x];
				}
			}
			ypos++;
		}

	//
	// write
	//

	virtual void end_output(void)
		{
			FILE* file = fopen(get_full_filename().c_str(), "w");
			cout << "writing image to " << get_full_filename() << endl;
			if (fwrite(&tiff_header, sizeof(tiff_header), 1, file) < 1)
			{
				cout << "cannot write header\n";
			}
			if (fwrite(tiff_image, width*height*3, 1, file) < 1)
			{
				cout << "TIFF:: cannot write body\n";
			}
			fclose(file);
		}

	enum TIFF_TYPES {TIFF_BYTE=1, TIFF_ASCII, TIFF_SHORT, TIFF_LONG,
					 TIFF_RATIONAL};

	enum { TIFF_NewSubfile = 254 };
	enum { TIFF_Compression = 259 };
	enum { TIFF_ImageWidth = 256 };
	enum { TIFF_ImageHeight = 257 };
	enum { TIFF_BitsPerSample = 258 };
	enum { TIFF_PhotometricInterpretation = 262 };
	enum { TIFF_StripOffsets = 273 };
	enum { TIFF_SamplesPerPixel = 277 };
	enum { TIFF_RowsPerStrip = 278 };
	enum { TIFF_StripByteCounts = 279 };
	enum { TIFF_XResolution = 282 };
	enum { TIFF_YResolution = 283 };
	enum { TIFF_ResolutionUnit = 296 };

	struct IFH
	{
		uint16_be byte_order;
		uint16_be magic_number;
		uint32_be ifd_offset;
	};

	struct IFDEntry
	{
		uint16_be tag;
		uint16_be type;
		uint32_be count;
		uint32_be offset;
	};

private:

	struct TIFF_Header
	{
		IFH ifh;
		uint16_be no_of_ifds;
		IFDEntry new_subfile;
		IFDEntry image_width;
		IFDEntry image_height;
		IFDEntry bits_per_sample;
		IFDEntry compression;
		IFDEntry photometric_interpretation;
		IFDEntry strip_offsets;
		IFDEntry samples_per_pixel;
		IFDEntry rows_per_strip;
		IFDEntry strip_byte_counts;
		IFDEntry x_resolution;
		IFDEntry y_resolution;
		IFDEntry resolution_unit;
		uint32_be next_ifd;
		uint32_be x1, x2, y1, y2;
		uint16_be bpp1, bpp2, bpp3;

	} tiff_header;

	byte* tiff_image;
	int ypos;
};

#endif

