////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcdu_corrupt_filter.cpp
//
//	Provides: tvcdu_corrupt_filter executable
//
//	Purpose: To enable the corruption of t-VCDU packet common fields.
//
//	Copyright (c) 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "eps.hpp"
#include "filereader.hpp"
#include "readfromstdin.hpp"
#include "tvcdureader.hpp"
#include "singlefilewriter.hpp"
#include "writetostdout.hpp"
#include "tvcducorruptfilter.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "tvcdu_corrupt_filter";

bool decode_input (INFO info, string argv, string name);

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_file;
	string out_file;

	bool verbose = true;

	deque<int> selected_vcid;
	int skip_val = -1;
	int count_val = -1;
	int sequence_counter_start = -1;
	int sequence_counter_end = -1;

	INFO version_number_info;
	INFO scid_info;
	INFO vcid_info;
	INFO counter_info;
	INFO replay_flag_info;
	INFO spare_info;
	INFO encrypt_flag_info;
	INFO encrypt_key_info;
	INFO mpdu_spare_info;
	INFO mpdu_header_info;
	INFO utc_day_info;
	INFO utc_millisecond_info;
	INFO utc_microsecond_info;
	INFO reed_solomon_status_info;
	INFO byte_info;

	int byte_offset_start = -1;
	int byte_offset_end = -1;
	string byte_increment = "both";

	short_cds_time start_time;
	int start_time_relative = -1;
	short_cds_time end_time;
	int end_time_relative = -1;

	int input_ctr = 0;

	enum { ARGV_NONE, ARGV_IN, ARGV_OUT, ARGV_SKIP, ARGV_CNT,
		   ARGV_FOR_VCID, ARGV_FOR_SEQ_CNT, ARGV_FOR_UTC,
		   ARGV_SET_VERSION, ARGV_SET_SCID, ARGV_SET_VCID,
		   ARGV_SET_COUNTER, ARGV_SET_REPLAY_FLG, ARGV_SET_SPARE,
		   ARGV_SET_ENCRYPT_FLG, ARGV_SET_ENCRYPT_KEY,
		   ARGV_SET_UTC_DAY, ARGV_SET_UTC_MIL_SEC,
		   ARGV_SET_UTC_MIC_SEC, ARGV_SET_RSS,
		   ARGV_SET_MPDU_SPARE, ARGV_SET_MPDU_HEADER,
		   ARGV_BYT_VAL, ARGV_BYT_OFFST, ARGV_BYT_INC } state = ARGV_IN;

// --in	 -i
// --out  -o
// --skip  -s
// --count	-c
// --for-virtual-channel-id	 --for-vcid
// --for-vcdu-counter  --for-vcnt
// --for-utc-time --for-time
// --set-version-number	 --set-vn
// --set-space-craft-id	 --set-scid
// --set-virtual-channel-id	 --set-vcid
// --set-vcdu-counter --set-vcnt
// --set-replay-flag  --set-rf
// --set-spare	--set-sp
// --set-encrypt-flag  --set-ef
// --set-encrypt-key  --set-ek
// --set-utc-day --set-day
// --set-utc-millisecond  --set-mil
// --set-utc-microsecond  --set--mic
// --set-reed-solomon-status  --set-rss
// --set-mpdu-spare --set-msp
// --set-mpdu-header --set-mh
// --set-byte  --set-by
// --for-byte-offset  --for-by
// --set-byte-increment	 --set-inc
// --show-vcids -a
// --show-tvcdu-table -t
// --show-mpdu-table -m
// --show-rss-table -r
// --help  -h
// --version  -v

	string help_msg = string("\nThe tvcdu_corrupt_filter enables the modification and corruption of tVCDU packets.\n\n")
		+ "Usage:\n\n" + APPNAME
		+ " [-i|--in <filename(s)>]"
		+ " [-o|--out <filename>]"
		+ " [-s|--skip <N>]"
		+ " [-c|--count <N>]"
		+ " [For Options]"
		+ " [Set Options]"
		+ " [Byte Options]"
		+ " [-a|--show-vcids]"
		+ " [-t|--show-tvcdu-table]"
		+ " [-m|--show-mpdu-table]"
		+ " [-r|--show-rss-table]"
		+ "[-z]"
		+ " [-h|--help]"
		+ " [-v|--version]\n\n"
		+ "Note: To avoid unnecessary typing, all the options above have short hand names"
		+ " by which they can be refereed to (-i, -o, -s, -c, -l, -a, -t, -m, -r, -h and -v).\n\n"
		+ "where:\n\n"
		+ "--in <filename(s)> : Allows multiple input files to be specified. Note that"
		+ " multiple input file names should be delimited by spaces. If the '--in'"
		+ " option is omitted, stdin is used by default.\n\n"
		+ "--out <filename> : Allows the name of an output file to be specified. If"
		+ " omitted stdout is used.\n\n"
		+ "--skip <N> : Allows N packets to be skipped before applying any corruptions.\n\n"
		+ "--count <N> : Allows corruptions to be applied to only N packets starting at"
		+ " the first packet or after skipping a number of packets if the --skip option is used.\n\n"
		+ "For Options : The '--for-' options discussed below allow the corruption"
		+ " constraints to be defined and thus allow single or a group of packets"
		+ " to be isolated and corrupted while leaving any packets that do not meet"
		+ " the constraints untouched.\n\n"
		+ "Set Options :  The '--set-' options discussed below allow the exact nature"
		+ " of the corruptions to be defined.\n\n"
		+ "Byte Options: The byte options discussed below are a special case that"
		+ " allow corruptions to be applied to any part of a packet.\n\n"
		+ "--show-vcids : Shows a list of recognised Virtual Channel IDs (VCIDs)."
		+ " The recognised VCID numbers can be used with the --for-vcid option to"
		+ " only apply corruptions to a select number of packets that match the"
		+ " specified VCID constraints.\n\n"
		+ " --show-tvcdu-table : Displays a table showing the structure of tVCDU"
		+ " packets (Note that the screen must be wide enough for the table to be"
		+ " displayed properly).\n\n"
		+ "--show-mpdu-table : Displays a table showing the structure of M-PDU packets.\n\n"
		+ "--show-rss-table : Displays a table showing the structure of the Reed"
		+ " Solomon Status bitfield (Note that the screen must be wide enough for"
		+ " the table to be displayed properly).\n\n"
		+ "-z : Use simple animations while processing.\n\n"
		+ "--help : Shows usage information.\n\n"
		+ "--version : Shows version information.\n\n";

	string help_msg_for = string("\nFor Options\n-----------\n\n")
		+ "--for-virtual-channel-id (--for-vcid) <vcid(s)> : Allows single"
		+ " or multiple numeric VCIDs to be specified. Only packets with"
		+ " VCIDs matching the ones specified will be corrupted. To obtain"
		+ " a full list of acceptable VCID numbers use the --show-vcids option.\n\n"
		+ "--for-vcdu-counter (--for-vcnt) <start_ctr> <end_ctr> : Only packets"
		+ " with VCDU counters greater or equal to start_ctr and less than or equal"
		+ " to end_ctr will be corrupted.\n\n"
		+ "--for-utc-time (--for-time) <start_time> <end_time> : Only packets"
		+ " with UTC-times greater or equal to start_time and less than or equal"
		+ " to end_time will be corrupted. Start and end times can be specified"
		+ " in two ways: a relative time and an absolute time. The absolute time"
		+ " must be in the format YYYYMMDDhhmmssZ, where the letters respectively"
		+ " represent Years, Months, Days, hours, minutes and seconds (e.g."
		+ " 20041223151139Z). The relative time can be defined as an offset of"
		+ " +N(ms|s|m), where N represents a number and	 must be followed by units"
		+ " of millisecond(ms), seconds(s) or minutes(m) (e.g. +2m). The relative"
		+ " start time is always relative to the UTC-time of the first packet."
		+ " The relative end time is a little more complicated. If an absolute"
		+ " start time is followed by a relative end time, then the relative end"
		+ " time will be relative to the absolute start time. If a relative start"
		+ " time is followed by a relative end time, then the relative end time"
		+ " will be relative to the UTC time of the first packet offset by the"
		+ " relative start time. It might sound a little complicated in words,"
		+ " but mathematically its quite simple:\n"
		+ "	  if start_time = A	   then	 start_time = A\n"
		+ "	  if end_time	= +5m  then	 end_time	= A + 5m\n"
		+ "	  if start_time = +2m  then	 start_time = T1 + 2m\n"
		+ "	  if end_time	= +5m  then	 end_time	= T1 + 2m + 5m\n"
		+ "Where A represents a specified absolute time, T1 represents the"
		+ " UTC-time of the first tVCDU packet read and +2m and +5m represent"
		+ " 2 and 5 minute relative time offsets.\n\n";

	string help_msg_set = string("\nSet Options\n-----------\n\n")
		+ " --set-version-number  (--set-vn) <random|N1|N1 N2>\n"
		+ "--set-space-craft-id	 (--set-scid) <random|N1|N1 N2>\n"
		+ "--set-virtual-channel-id	 (--set-vcid) <random|N1|N1 N2>\n"
		+ "--set-vcdu-counter (--set-vcnt) <random|N1|N1 N2>\n"
		+ "--set-replay-flag  (--set-rf) <random|N1|N1 N2>\n"
		+ "--set-spare	(--set-sp) <random|N1|N1 N2>\n"
		+ "--set-encrypt-flag  (--set-ef) <random|N1|N1 N2>\n"
		+ "--set-encrypt-key  (--set-ek) <random|N1|N1 N2>\n"
		+ "--set-mpdu-spare (--set-msp) <random|N1|N1 N2>\n"
		+ "--set-mpdu-header (--set-mh) <random|N1|N1 N2>\n"
		+ "--set-utc-day (--set-day) <random|N1|N1 N2>\n"
		+ "--set-utc-millisecond  (--set-mil) <random|N1|N1 N2>\n"
		+ "--set-utc-microsecond  (--set-mic) <random|N1|N1 N2>\n"
		+ "--set-reed-solomon-status  (--set-rss) <random|N1|N1 N2>\n\n"
		+ "The functions of the above set options are obvious and do not need"
		+ " to be individually explained. Each set option allows one of the"
		+ " tVCDU fields to be modified or corrupted. Note that three types"
		+ " of corruptions can be introduced:\n\n"
		+ "1. The first type of corruption allows the fields of consecutive"
		+ " tVCDU packets to be set to an appropriate* random number.\n\n"
		+ "2. The second type of corruption allows the fields of consecutive"
		+ " tVCDU packets to be set to an appropriate* single number (N1).\n\n"
		+ "3. The third type of corruption allows the fields of consecutive"
		+ " tVCDU packets to be filled with an appropriate* number series or"
		+ " patter starting with the number N1 and incremented by the number N2."
		+ " For example if N1=0 and N2=2, then consecutive packets will be filled"
		+ " with the numbers 0, 2, 4, 6, ...   If the field has a maximum value"
		+ " that is reached, the pattern or series will simply wrap around"
		+ " and continue. For example if the maximum acceptable value is 7,"
		+ " then the pattern will be: 0, 2, 4, 6, 0,  2, 4, 6, 0, ...  Note"
		+ " that N2 can also be assigned a negative value and thus produce"
		+ " increasing and decreasing number patterns.\n\n"
		+ "The term appropriate* has been used in the above paragraphs to"
		+ " imply that the corruptions applied must be greater or equal to"
		+ " the minimum value the field can accept and of course less than"
		+ " or equal to the maximum value the field can accept. The code"
		+ " attempts to handle all this automatically. If however, the user"
		+ " does input values that are out of range, they will be prompted"
		+ " with error messages and advised to revise their inputs.\n\n";


	string help_msg_byte = string("\nByte Options\n------------\n\n")
		+ "Unlike the set options, described above, that only act to modify"
		+ " specific fields, the byte options allow corruptions to be applied"
		+ " to any part of a tVCDU packet.\n\n"
		+ "--for-byte-offset <N1|N1 N2> :  Corruptions can be introduced"
		+ " to a single byte at byte offset N1 or to a range of bytes"
		+ " starting at byte offset N1 and ending at byte offset N2."
		+ " Note that if the corruption is to be applied to the first byte,"
		+ " the byte offset must be set to zero. Hence, the byte offset"
		+ " must be greater or equal to zero and of course less than the"
		+ " packet length.\n\n"
		+ "--set-byte <random|N1|N1 N2> : Similar to the set options,"
		+ " the --set-byte option allows a single byte per packet or a"
		+ " range of bytes per packet to be set to random values, to be"
		+ " set to a single value N1, or to be filled with a number"
		+ " pattern starting with a value N1 and incremented by a value N2."
		+ " Note that N1 and N2 can not be set to anything greater than 255."
		+ " While N1 can not be less than zero, the increment N2 can be"
		+ " assigned a negative value.\n\n"
		+ "--set-byte-increment <column|row> : This option only becomes"
		+ " relevant if one is attempting to assign a number pattern to a"
		+ " range of bytes per packet. For example a user might want to"
		+ " corrupt byte offsets 101 to 105 (--for-byte-offset 101 105)"
		+ " with a number pattern: 1, 3, 5, 7, 9, ... (--set-byte 1 2)."
		+ " If the --set-byte-increment option is left undefined, the"
		+ " resulting corruption per packet will be as follows:\n\n"
		+ "	  Byte Offset  101	102	 103  104  105\n"
		+ "	  Packet 1	 1	  3	   5	7	 9\n"
		+ "	  Packet 2	 3	  5	   7	9	 11\n"
		+ "	  Packet 3	 5	  7	   9	11	 13\n"
		+ "	  Packet 4	 7	  9	   11	13	 15\n"
		+ "	  Packet 5	 9	  11   13	15	 17\n"
		+ "	  Packet 6	 11	  13   15	17	 19\n\n"
		+ "Note that the pattern has been increment over both column"
		+ " (Byte Offset) and row (Packet). Hence, if the"
		+ " --set-byte-increment option is left undefined, the default"
		+ " option is to apply the pattern over both columns and rows."
		+ " If however, the --set-byte-increment is set to column, the"
		+ " pattern will only be incremented over the different byte"
		+ " offsets and will result into the following corruption:\n\n"
		+ "	  Byte Offset	 101  102  103	104	 105\n"
		+ "	  Packet 1	 1	  3	   5	7	 9\n"
		+ "	  Packet 2	 1	  3	   5	7	 9\n"
		+ "	  Packet 3	 1	  3	   5	7	 9\n"
		+ "	  Packet 4	 1	  3	   5	7	 9\n"
		+ "	  Packet 5	 1	  3	   5	7	 9\n"
		+ "	  Packet 6	 1	  3	   5	7	 9\n\n"
		+ "Similarly, if the --set-byte-increment option is set to row,"
		+ " the pattern will only be incremented over the different packets"
		+ " and will result into the following corruption:\n\n"
		+ "	  Byte Offset	 101  102  103	104	 105\n"
		+ "	  Packet 1	 1	  1	   1	1	 1\n"
		+ "	  Packet 2	 3	  3	   3	3	 3\n"
		+ "	  Packet 3	 5	  5	   5	5	 5\n"
		+ "	  Packet 4	 7	  7	   7	7	 7\n"
		+ "	  Packet 5	 9	  9	   9	9	 9\n"
		+ "	  Packet 6	   11	11	 11	  11   11\n\n";

	string help_msg_example = string("\nExamples\n--------\n\n")
		+ "tvcdu_corrupt_filter -i in.tvcdu -o out.tvcdu"
		+ " --for-vcdu-counter 3 15 --set-version-number 2\n\n"
		+ "tvcdu_corrupt_filter -i in.tvcdu -o out.tvcdu --for-vcid 9 10"
		+ " --for-vcdu-counter 3 15 --set-version-number random\n\n"
		+ "tvcdu_corrupt_filter -i in.tvcdu -o out.tvcdu --for-vcid 9 10"
		+ " --for-vcdu-counter 3 15 --set-version-number 1 2\n\n"
		+ "tvcdu_corrupt_filter -i in.tvcdu -o out.tvcdu --for-vcid 9 10"
		+ " --for-vcdu-counter 3 15 --for-utc-time 20050211095233Z +5m"
		+ " --set-version-number 1 2 --set-scid 12 --set-rss random\n\n";

	string tvcdu_table = string("\nThe tVCDU packet structure is defined as follow:\n\n")
		+ "--------------------------------------------------------------------------------------------\n"
		+ "|Version|Space |Virtual|VCDU		|Replay|Spare |Encrypt|Encrypt|Payload	|UTC	  |Reed	  |\n"
		+ "|Number |Craft |Channel|Counter	|Flag  |	  |Flag	  |Key	  |			|Time	  |Solomon|\n"
		+ "|	   |ID	  |ID	  |			|	   |	  |		  |		  |			|Stamp	  |Status |\n"
		+ "--------------------------------------------------------------------------------------------\n"
		+ "|2 Bits |8 Bits|6 Bits |24 Bits	|1 Bit |7 Bits|8 Bits |8 Bits |884 Bytes|8 Bytes  |8 Bits |\n"
		+ "|	   |	  |		  |			|	   |	  |		  |		  |			|		  |		  |\n"
		+ "--------------------------------------------------------------------------------------------\n"
		+ "|Packet |11, 12|See	  |Incremted|0 for |Zero  |Insert Zone	  |M-PDU	|Days	  |See	  |\n"
		+ "|Format |13, 14|		  |uniquely |real  |fill  |Not Used		  |Packet	|(2 bytes)|		  |\n"
		+ "|1 or 2 |For	  | show- |for each |time  |	  |Zero	 fill	  |See		|Millisecs| show- |\n"
		+ "|	   |METOP | vcids |VCID		|data  |	  |				  | show-	|(4 bytes)| rss-  |\n"
		+ "|	   |1, 2, |		  |			|	   |	  |				  | mpdu-	|Microsecs| table |\n"
		+ "|	   |3 or  |option |			|	   |	  |				  | table	|(2 bytes)|		  |\n"
		+ "|	   |sim.  |		  |			|	   |	  |				  |option	|		  |option |\n"
		+ "--------------------------------------------------------------------------------------------\n\n";

	string mpdu_table = string("\nThe M-PDU packet structure is defined as follow:\n\n")
		+ "------------------------------------------------------------------------\n"
		+ "|Spare				|First Header Pointer  |Packet Zone				  |\n"
		+ "------------------------------------------------------------------------\n"
		+ "|5 bits				|11 bits			   |882 bytes				  |\n"
		+ "------------------------------------------------------------------------\n"
		+ "|00000 - if a CCSDS	|Index into the packet |Contains CCSDS packets or |\n"
		+ "|header is present	|zone where the first  |fragments of packets.	  |\n"
		+ "|in the packet zone	|CCSDS header begins.  |						  |\n"
		+ "|11111 - if no header|					   |						  |\n"
		+ "|is in packet zone.	|					   |						  |\n"
		+ "------------------------------------------------------------------------\n\n";

	string vcid_table = string("\nList of recognised VCIDs\n------------------------\n\n")
		+ "3:	  AMSUA-A1, AMSUA-A2, HIRS/4, SEM\n"
		+ "5:	  AVHRR Low Rate\n"
		+ "9:	  AVHRR High Rate\n"
		+ "10:	  IASI\n"
		+ "12:	  MHS\n"
		+ "15:	  ASCAT\n"
		+ "24:	  GOME-2\n"
		+ "27:	  A-DCS\n"
		+ "29:	  GRAS\n"
		+ "34:	  GRAS-NAV, Satellite Packet, Admin Packet\n"
		+ "63:	  Fill VCDU\n\n";

	string rss_table = string("\nThe Reed Solomon Status bitfield is defined ")
		+ "as follows (where 0 represents the least significant bit):\n\n"
		+ "---------------------------------------------\n"
		+ "|Bit |Name				 |Description	   |\n"
		+ "---------------------------------------------\n"
		+ "|7-6 |Status of SCLF		 |00 - Search Mode |\n"
		+ "|	|state machine		 |01 - Check Mode  |\n"
		+ "|	|					 |10 - Lock Mode   |\n"
		+ "---------------------------------------------\n"
		+ "|5	|Best Match			 |0 - No, 1 - Yes  |\n"
		+ "---------------------------------------------\n"
		+ "|4	|Flush Frame		 |0 - No, 1 - Yes  |\n"
		+ "---------------------------------------------\n"
		+ "|3	|Slip Frame			 |0 - No, 1 - Yes  |\n"
		+ "---------------------------------------------\n"
		+ "|2	|Unroutable Farme	 |0 - No, 1 - Yes  |\n"
		+ "---------------------------------------------\n"
		+ "|1	|Uncorrect Frame	 |0 - No, 1 - Yes  |\n"
		+ "---------------------------------------------\n"
		+ "|0	|R-5 (255,233) found |0 - No, 1 - Yes  |\n"
		+ "|	|errors in frame	 |0 - No, 1 - Yes  |\n"
		+ "---------------------------------------------\n\n";


	for(int x=1; x<argc; x++)
	{
		string arg(argv[x]);

		if (arg == "--version" || arg == "-v")
		{
			cout << APPNAME << " " << PACKAGE_VERSION << " built on " << __DATE__
				 << endl;
			return 0;
		}
		else if (arg == "--help" || arg == "-h")
		{
			cout << help_msg;
			cout << help_msg_for;
			cout << help_msg_set;
			cout << help_msg_byte;
			cout << help_msg_example;
			return 0;
		}
		else if (arg == "--help-for" || arg == "-hf" || arg == "-h-f")
		{
			cout << help_msg_for;
			return 0;
		}
		else if (arg == "--help-set" || arg == "-hs" || arg == "-h-s")
		{
			cout << help_msg_set;
			return 0;
		}
		else if (arg == "--help-byte" || arg == "-hb" || arg == "-h-b")
		{
			cout << help_msg_byte;
			return 0;
		}
		else if (arg == "--help-example" || arg == "-hx" || arg == "-h-x")
		{
			cout << help_msg_example;
			return 0;
		}
		else if (arg == "--show-vcids" || arg == "--show-vcid"
				 || arg == "-a")
		{
			cout << vcid_table;
			return 0;
		}
		else if (arg == "--show-tvcdu-table" || arg == "--show-tvcdu"
				 || arg == "-t")
		{
			cout << tvcdu_table;
			return 0;
		}
		else if (arg == "--show-mpdu-table" || arg == "--show-mpdu"
				 || arg == "-m")
		{
			cout << mpdu_table;
			return 0;
		}
		else if (arg == "--show-rss-table" || arg == "--show-rss"
				 || arg == "-r")
		{
			cout << rss_table;
			return 0;
		}
		else if (arg == "--in" || arg == "-i")
		{
			state = ARGV_IN;
		}
		else if (arg == "--out" || arg == "-o")
		{
			state = ARGV_OUT;
		}
		else if (arg == "--quiet")
		{
			verbose = false;
		}
		else if (arg == "--debug")
		{
			_debug = &cout;
		}
		else if (arg == "--skip" || arg == "-s")
		{
			state = ARGV_SKIP;
		}
		else if (arg == "--count" || arg == "-c")
		{
			state = ARGV_CNT;
		}
		else if (arg == "--for-vcid" || arg == "--filter-vcid" ||
				 arg == "--for-virtual-channel-id")
		{
			state = ARGV_FOR_VCID;
		}
		else if (arg == "--for-vcdu-counter" || arg == "--filter-vcdu-counter"
				 || arg == "--for-counter" || arg == "--filter-counter" ||
				 arg == "--for-cnt" || arg == "--for-vcnt")
		{
			input_ctr = 0;
			state = ARGV_FOR_SEQ_CNT;
		}
		else if (arg == "--for-utc-time" || arg == "--filter-utc-time" ||
				 arg == "--for-time")
		{
			input_ctr = 0;
			state = ARGV_FOR_UTC;
		}
		else if (arg == "--set-version-number"	|| arg == "--set-vn"
				 || arg == "--set-version" || arg == "--set-v")
		{
			state = ARGV_SET_VERSION;
		}
		else if (arg == "--set-scid" || arg == "--set-space-craft-id")
		{
			state = ARGV_SET_SCID;
		}
		else if (arg == "--set-vcid" || arg == "--set-virtual-channel-id")
		{
			state = ARGV_SET_VCID;
		}
		else if (arg == "--set-counter" || arg == "--set-vcdu-counter"
				 || arg == "--set-vcnt" || arg == "--set-cnt")
		{
			state = ARGV_SET_COUNTER;
		}
		else if (arg == "--set-replay-flag" || arg == "--set-replay"
				 || arg == "--set-rf")
		{
			state = ARGV_SET_REPLAY_FLG;
		}
		else if (arg == "--set-spare" || arg == "--set-sp")
		{
			state = ARGV_SET_SPARE;
		}
		else if (arg == "--set-encrypt-flag" || arg == "--set-ef")
		{
			state = ARGV_SET_ENCRYPT_FLG;
		}
		else if (arg == "--set-encrypt-key" || arg == "--set-ek")
		{
			state = ARGV_SET_ENCRYPT_KEY;
		}
		else if (arg == "--set-utc-day" || arg == "--set-utc-d" || arg == "--set-day")
		{
			state = ARGV_SET_UTC_DAY;
		}
		else if (arg == "--set-utc-millisecond" || arg == "--set-utc-milliseconds" ||
				 arg == "--set-utc-millisec" || arg == "--set-utc-mil" ||
				 arg == "--set-utc-milli" || arg == "--set-mil" || arg == "--set-milli")
		{
			state = ARGV_SET_UTC_MIL_SEC;
		}
		else if (arg == "--set-utc-microsecond" || arg == "--set-utc-microseconds" ||
				 arg == "--set-utc-microsec" || arg == "--set-utc-mic" ||
				 arg == "--set-utc-micro" || arg == "--set-micro" || arg == "--set-mic")
		{
			state = ARGV_SET_UTC_MIC_SEC;
		}
		else if (arg == "--set-reed-solomon-status" || arg == "--set-rss"
				 || arg == "--set-reed-solomon")
		{
			state = ARGV_SET_RSS;
		}
		else if (arg == "--set-mpdu-spare" || arg == "--set-msp")
		{
			state = ARGV_SET_MPDU_SPARE;
		}
		else if (arg == "--set-mpdu-header" || arg == "--set-mh"
				 || arg == "--set-mpdu-head")
		{
			state = ARGV_SET_MPDU_HEADER;
		}
		else if (arg == "--set-byte" || arg == "--set-by")
		{
			state = ARGV_BYT_VAL;
		}
		else if (arg == "--for-offset" || arg == "--filter-offset" ||
				 arg == "--for-byte-offset" || arg == "--filter-byte-offset" ||
				 arg == "--for-byte" || arg == "--filter-byte" || arg == "--for-by" )

		{
			state = ARGV_BYT_OFFST;
		}
		else if (arg == "--set-byte-increment" || arg == "--set-byte-inc" ||
				 arg == "--set-increment" || arg == "--set-inc")
		{
			state = ARGV_BYT_INC;
		}
		else if (!strcmp(argv[x], "-z"))
		{
			Spinner::use_simple_spinner();
		}
		else
		{
			switch(state)
			{

			case ARGV_NONE:
				cerr << "Parameter not recognised '" << arg << endl;
				return -1;

			case ARGV_IN:
				in_file.push_back(arg);
				break;

			case ARGV_OUT:
				if (!out_file.empty())
				{
					error << "Only one output file can be specified\n";
					return -1;
				}
				else
				{
					out_file = arg;
					state = ARGV_NONE;
				}
				break;

			case ARGV_SKIP:
				if (skip_val != -1)
				{
					error << "The option --skip only accepts one numeric input\n";
					return -1;
				}
				else
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n";
						return -1;
					}
					skip_val = lexical_cast<int>(arg);
					state = ARGV_NONE;
				}
				break;

			case ARGV_CNT:
				if (count_val != -1)
				{
					error << "The option --count only accepts one numeric input\n";
					return -1;
				}
				else
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n";
						return -1;
					}
					count_val = lexical_cast<int>(arg);
					state = ARGV_NONE;
				}
				break;

			case ARGV_FOR_VCID:
				if (arg.find_first_not_of("0123456789")!=string::npos)
				{
					error << "Unacceptable input '" << arg << "'\n"
						  << "The option --for-vcid accepts single/multiple "
						  << "numeric VCID values separated by spaces.\n";
					return -1;
				}
				selected_vcid.push_back(lexical_cast<int>(arg));
				break;

			case ARGV_FOR_SEQ_CNT:
				if (arg.find_first_not_of("0123456789")!=string::npos)
				{
					error << "Unacceptable input '" << arg << "'\n"
						  << "The option --for-vcdu-counter only accepts two numeric inputs "
						  << "to mark the start and end of the corruptions to be applied.\n";
					return -1;
				}
				if (sequence_counter_start == -1)
				{
					sequence_counter_start = lexical_cast<int>(arg);
					sequence_counter_end = sequence_counter_start;
					input_ctr++;
				}
				else if (sequence_counter_end == -1 ||
						 sequence_counter_end == sequence_counter_start &&
						 input_ctr < 2)
				{
					sequence_counter_end = lexical_cast<int>(arg);
					input_ctr++;
				}
				else
				{
					error << "The option --for-vcdu-counter only accepts two numeric inputs "
						  << "to mark the start and end of the corruptions to be applied.\n";
					return -1;
				}
				break;

			case ARGV_FOR_UTC:
				if (arg.find_first_not_of("0123456789zZ+mMsS")!=string::npos &&
					arg.length()<16)
				{
					error << "Unacceptable input '" << arg << "'\n"
						  << "Format of time not recognised. Use absolute "
						  << "time YYYYMMDDhhmmssZ or relative "
						  << "+<number><ms|s|m>\n";
					return -1;
				}
				if (arg[0]=='+')
				{
					// relative time specified
					int number = TimeConv::duration_to_int(arg.substr(1));
					if (number == -1)
					{
						return -1;
					}
					if (start_time_relative == -1 && input_ctr < 2)
					{
						//cout << "Using relative start time of +" << number << "ms\n";
						start_time_relative = number;
						input_ctr++;
					}
					else if (end_time_relative == -1 && input_ctr < 2)
					{
						//cout << "Using relative end time of +" << number << "ms\n";
						end_time_relative = number;
						input_ctr++;
					}
					else
					{
						error << "The option --for-utc-time only accepts two inputs "
							  << "to mark the start and end of the corruptions to be applied.\n";
						return -1;
					}
				}
				else
				{
					short_cds_time cds = short_cds_time(arg);
					if (cds == short_cds_time())
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "Format of time not recognised. Use absolute "
							  << "time YYYYMMDDhhmmssZ or relative "
							  << "+<number><ms|s|m>\n";
						return -1;
					}
					else if (start_time == short_cds_time()	 && input_ctr < 2)
					{
						start_time = cds;
						input_ctr++;
					}
					else if (end_time == short_cds_time() && input_ctr < 2)
					{
						end_time = cds;
						input_ctr++;
					}
					else
					{
						error << "The option --for-utc-time only accepts two inputs "
							  << "to mark the start and end of the corruptions to be applied.\n";
						return -1;
					}
				}
				break;

			case ARGV_SET_VERSION:
				if (!decode_input(&version_number_info, arg, "version-number"))
				{
					return -1;
				}
				break;

			case ARGV_SET_SCID:
				if (!decode_input(&scid_info, arg, "scid"))
				{
					return -1;
				}
				break;

			case ARGV_SET_VCID:
				if (!decode_input(&vcid_info, arg, "vcid"))
				{
					return -1;
				}
				break;

			case ARGV_SET_COUNTER:
				if (!decode_input(&counter_info, arg, "counter"))
				{
					return -1;
				}
				break;

			case ARGV_SET_REPLAY_FLG:
				if (!decode_input(&replay_flag_info, arg, "replay-flag"))
				{
					return -1;
				}
				break;

			case ARGV_SET_SPARE:
				if (!decode_input(&spare_info, arg, "spare"))
				{
					return -1;
				}
				break;

			case ARGV_SET_ENCRYPT_FLG:
				if (!decode_input(&encrypt_flag_info, arg, "encrypt-flag"))
				{
					return -1;
				}
				break;

			case ARGV_SET_ENCRYPT_KEY:
				if (!decode_input(&encrypt_key_info, arg, "encrypt-key"))
				{
					return -1;
				}
				break;

			case ARGV_SET_UTC_DAY:
				if (!decode_input(&utc_day_info, arg, "utc-day"))
				{
					return -1;
				}
				break;

			case ARGV_SET_UTC_MIL_SEC:
				if (!decode_input(&utc_millisecond_info, arg, "utc-millisecond"))
				{
					return -1;
				}
				break;

			case ARGV_SET_UTC_MIC_SEC:
				if (!decode_input(&utc_microsecond_info, arg, "utc-microsecond"))
				{
					return -1;
				}
				break;

			case ARGV_SET_RSS:
				if (!decode_input(&reed_solomon_status_info, arg, "reed-solomon-status"))
				{
					return -1;
				}
				break;

			case ARGV_SET_MPDU_SPARE:
				if (!decode_input(&mpdu_spare_info, arg, "mpdu-spare"))
				{
					return -1;
				}
				break;

			case ARGV_SET_MPDU_HEADER:
				if (!decode_input(&mpdu_header_info, arg, "mpdu-header"))
				{
					return -1;
				}
				break;

			case ARGV_BYT_VAL:
				if (arg == "random" && byte_info.key == INFO::OFF)
				{
					byte_info.key = INFO::RANDOM;
					state = ARGV_NONE;
				}
				else
				{
					if (byte_info.start == -1 && byte_info.key != INFO::RANDOM)
					{
						if (arg.find_first_not_of("0123456789")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-byte only accepts the inputs "
								  << "random/single number/two numbers separated by a space. "
								  << "For example:\n--set-byte random, "
								  << "\n--set-byte 3, "
								  << "\n--set-byte 2 1\n";
							return -1;
						}
						byte_info.start = lexical_cast<int>(arg);
						if (byte_info.start > 255)
						{
							error << "A byte can not be set to anything greater than 255\n";
							return -1;
						}
						byte_info.key = INFO::SINGLE;
					}
					else if (byte_info.increment == 0 && byte_info.start != -1
							 && byte_info.key != INFO::RANDOM)
					{
						if (arg.find_first_not_of("-0123456789")!=string::npos)
						{
							error << "Unacceptable input '" << arg << "'\n"
								  << "The option --set-byte only accepts the inputs "
								  << "random/single number/two numbers separated by a space. "
								  << "For example:\n--set-byte random, "
								  << "\n--set-byte 3, "
								  << "\n--set-byte 2 1\n";
							return -1;
						}
						byte_info.increment = lexical_cast<int>(arg);
						if (byte_info.increment > 255)
						{
							error << "A byte can not be set to anything greater than 255\n";
							return -1;
						}
						byte_info.key = INFO::INCREMENT;
					}
					else
					{
						error << "The option --set-byte only accepts the inputs "
							  << "random/single number/two numbers separated by a space. "
							  << "For example:\n--set-byte random, "
							  << "\n--set-byte 3, "
							  << "\n--set-byte 2 1\n";
						return -1;
					}
				}
				break;

			case ARGV_BYT_OFFST:
				if (byte_offset_start == -1)
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --for-byte-offset only accepts the inputs "
							  << "single number/two numbers separated by a space. "
							  << "For example:\n--for-byte-offset 28, "
							  << "\n--for-byte-offset 28 48\n";
						return -1;
					}
					byte_offset_start = lexical_cast<int>(arg);
				}
				else if (byte_offset_end == -1 && byte_offset_start != -1)
				{
					if (arg.find_first_not_of("0123456789")!=string::npos)
					{
						error << "Unacceptable input '" << arg << "'\n"
							  << "The option --for-byte-offset only accepts the inputs "
							  << "single number/two numbers separated by a space. "
							  << "For example:\n--for-byte-offset 28, "
							  << "\n--for-byte-offset 28 48\n";
						return -1;
					}
					byte_offset_end = lexical_cast<int>(arg);
				}
				else
				{
					error << "The option --for-byte-offset only accepts the inputs "
						  << "single number/two numbers separated by a space. "
						  << "For example:\n--for-byte-offset 28, "
						  << "\n--for-byte-offset 28 48\n";
					return -1;
				}
				break;


			case ARGV_BYT_INC:
				if (arg == "column" || arg == "col" && byte_increment == "both")
				{
					byte_increment = "column";
					state = ARGV_NONE;
				}
				else if (arg == "row" && byte_increment == "both")
				{
					byte_increment = "row";
					state = ARGV_NONE;
				}
				else
				{
					error << "Unacceptable input '" << arg << "'\n"
						  << "The option --set-byte-increment only accepts the inputs "
						  << "column or row. For example:\n--set-byte-increment column, "
						  << "\n--set-byte-increment row\n";
					return -1;
				}
				break;

			}
		}
	}



	if (!verbose)
	{
		spinner->restream(&onull);
	}


	if (byte_info.key == INFO::OFF && byte_offset_start == -1 && byte_info.start == -1)
	{
		// No byte corruption has been defined and thus leave as is
	}
	else
	{
		if (byte_info.key == INFO::RANDOM && byte_offset_start == -1)
		{
			error << "NOTE: The options --for-byte-offset and --set-byte must be used together\n";
			return -1;
		}
		else if (byte_offset_start == -1 || byte_info.start == -1
				 && byte_info.key != INFO::OFF && byte_info.key != INFO::RANDOM)
		{
			error << "NOTE: The options --for-byte-offset and --set-byte must be used together\n";
			return -1;
		}
	}


	FileProducer* first_source;
	if (in_file.empty())
	{
		first_source = new ReadFromStdin();
	}
	else
	{
		first_source = new FileReader(in_file);
	}


	tVCDUReader tvcdu_reader(*first_source);
	tVCDUCorruptFilter corrupt(tvcdu_reader, skip_val, count_val, selected_vcid,
							   start_time, end_time, start_time_relative,
							   end_time_relative, sequence_counter_start,
							   sequence_counter_end, version_number_info, scid_info,
							   vcid_info, counter_info, replay_flag_info,
							   spare_info, encrypt_flag_info, encrypt_key_info,
							   mpdu_spare_info, mpdu_header_info, utc_day_info,
							   utc_millisecond_info, utc_microsecond_info,
							   reed_solomon_status_info, byte_info,
							   byte_offset_start, byte_offset_end,
							   byte_increment);

	if (!corrupt.is_valid())
	{
		error << "Exit...\n";
		return -1;
	}

	FileConsumer* writer;
	if (out_file.empty())
	{
		writer = new WriteToStdout(&corrupt);
	}
	else
	{
		writer = new SingleFileWriter(&corrupt, out_file);
	}

	if (verbose && !gerror)
	{
		if (!corrupt.is_valid())
		{
			error << "\n*** WARNING: The output produced might be"
				  << " corrupt or incomplete as errors were detected"
				  << " during the processing.\n\n";
			return -1;
		}
		else
		{
			first_source->output_text_summary();
			tvcdu_reader.output_text_summary();
			corrupt.output_text_summary();
			writer->output_text_summary();
		}
	}

	delete first_source;
	delete writer;

	return 0;
}

bool decode_input (INFO info, string argv, string name)
{

	string error_msg = string ("Unacceptable input '") + argv +	 "'\n";

	error_msg = error_msg
		+ "The option --set-" + name + " only accepts the inputs "
		+ "random/single number/two numbers separated by a space. "
		+ "For example:\n--set-" + name + " random\n"
		+ "--set-" + name + " 3\n"
		+ "--set-" + name + " 2 1\n"
		+ "--set-" + name + " 2 -1\n\n";

	if (argv.length() > 9)
	{
		error << "Input too large: "
			  << argv << endl;
		return false;
	}

	if (argv == "random")
	{
		info.key = INFO::RANDOM;
	}
	else
	{
		if (info.start == -1 && info.key != INFO::RANDOM)
		{
			if (argv.find_first_not_of("0123456789")!=string::npos)
			{
				error << error_msg;
				return false;
			}
			info.start = lexical_cast<int>(argv);
			info.key = INFO::SINGLE;
		}
		else if (info.increment == 0 || info.key == INFO::SINGLE)
		{
			if (argv.find_first_not_of("-0123456789")!=string::npos
				|| strncmp(argv.c_str(),"--",2) == 0)
			{
				error <<  error_msg;
				return false;
			}
			info.increment = lexical_cast<int>(argv);
			info.key = INFO::INCREMENT;
		}
		else
		{
			error << "Unrecognised parameter '" << argv << "'. Run '"
				  << APPNAME << " --help' for a list of options\n";
			return false;
		}
	}

	return true;
}
