////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcdu_to_cadur.cpp
//
//	Provides: tvcdu_to_cadu executable
//
//	Purpose: Convert tVCDU packets to CADU packets
//
//	Copyright (c) 2010 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "args.hpp"
#include "filereader.hpp"
#include "readfromstdin.hpp"
#include "tvcdureader.hpp"
#include "tvcdutovcdu.hpp"
#include "vcduscidsetter.hpp"
#include "vcdutocadu.hpp"
#include "caduwriter.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

int gerror = 0; // global error
const string APPNAME = "tvcdu_to_cadu";
const string MAINTAINER = "mike.elson@eumetsat.int";

int main(int argc, char* argv[])
{
	ofstream onull("/dev/null");
	_debug = &onull;
	_info = &cout;
	_warning = &cerr;
	_error = &cerr;

	deque<string> in_files;
	string out_file;
	int new_scid=-1;

	Args args;
	args.set_doc_head("Convert tVCDU packets to CADU packets, with optional modification of SCID values.");
	args.set_doc_tail("SCID values:\n\t11 - METOP 1\n\t12 - METOP 2\n\t13 - METOP 3\n\t14 - METOP SIM");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2010 Eumetsat.");
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("cadu_displayer");
	args.add_related_prog("tvcdu_displayer");

	args.add_example("tvcdu_to_cadu input.tvcdu --set-scid 5 --out output.cadu", "");

	args.add(&in_files, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME,
			 "Select input file(s)", "If omitted packets are read from standard input.");
	args.add(&out_file, "out", 'o', Args::FILENAME,
			 "Select output filename", "If ommited packets are written to standard output");
	args.add(&new_scid, "set-scid", 's', 0,
			 "New SCID value (omit this option to keep the existing SCID values)");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case Args::USER_OPTION:
			_debug = &cout;
			break;
		}
	}

	if (out_file.empty())
	{
		cerr << "No output name specified\n";
		return -1;
	}

	spinner->restream(&onull);

	// set UTC time zone
	TimeConv::set_time_zone();

	FileProducer* file_source;
	if (in_files.empty())
	{
		file_source = new ReadFromStdin();
	}
	else
	{
		file_source = new FileReader(in_files);
	}

	tVCDUReader tvcdu_reader(*file_source);

	const bool strip_filler = false;
	const bool strip_encrypted_packets = false;

	tVCDUToVCDU vcdu_source(tvcdu_reader,
							strip_filler,
							strip_encrypted_packets);

	VCDUToCADU* cadu_source;
	VCDUSCIDSetter* setter = NULL;

	if (new_scid==-1)
	{
		cadu_source = new VCDUToCADU(&vcdu_source);
	}
	else
	{
		setter = new VCDUSCIDSetter(vcdu_source, new_scid);
		cadu_source = new VCDUToCADU(setter);
	}

	const bool pad = false;
	CADUWriter writer(*cadu_source, out_file, pad);

	file_source->output_text_summary();
	tvcdu_reader.output_text_summary();
	vcdu_source.output_text_summary();
	if (setter)
	{
		setter->output_text_summary();
		delete setter;
	}
	writer.output_text_summary();

	delete file_source;
	delete cadu_source;
	return 0;
}
