////////////////////////////////////////////////////////////////////////////////
//
//	File: cadu_to_ccsds.cpp
//
//	Provides: cadu_to_ccsds executable
//
//	Purpose: Convert a stream of CADU packets to CCSDS packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "filereader.hpp"
#include "tvcdureader.hpp"
#include "tvcdutovcdu.hpp"
#include "ccsdsdemultiplexer.hpp"
#include "vcdutoccsds.hpp"
#include "readfromstdin.hpp"
#include "multifilewriter.hpp"
#include "writetostdout.hpp"
#include "singlefilewriter.hpp"
#include "mpdudisplayer.hpp"
#include "textrecorddisplayer.hpp"
#include "ccsdsextractor.hpp"
#include "writetonull.hpp"
#include "args.hpp"

textoutstream_type* _debug;
textoutstream_type* _info;
textoutstream_type* _warning;
textoutstream_type* _error;

ofstream onull("/dev/null");
int gerror = 0; // global error
const string APPNAME = "tvcdu_to_ccsds";
const string MAINTAINER = "elson@eumetsat.de";

int main(int argc, char* argv[])
{
	// set up streams
	_debug = &onull;
	_info = &cout;
	_warning = &cout;
	_error = &cerr;

	// set UTC time zone
	TimeConv::set_time_zone();

	deque<string> in_files;
	string out_file;

	bool verbose = true;
	bool split_by_inst = false;

	Args args;
	args.set_doc_head("Extract CCSDS packets from tVCDU packets.");
	args.set_doc_tail("");
	args.set_package(PACKAGE);
	args.set_app(APPNAME);
	args.set_maintainer(MAINTAINER);
	args.set_copyright("Copyright (c) 2005 Eumetsat.");
	args.set_license(Args::GPL);
	args.set_version(VERSION);

	args.add_related_prog("cadu_to_ccsds");
	args.add_related_prog("tvcdu_displayer");

	args.add_example("tvcdu_to_ccsds file.tvcdu -o isps", "Convert 'file.tvdu' into a set of CCSDS packet files, with names beginning with 'isps'.");
	args.add_example("tvcdu_to_ccsds file.tvcdu", "Show a report of the CCSDS packets within file.tvcdu, but do not write anything.");

	args.add(&in_files, "in", 'i', Args::DEFAULT_OPTION|Args::FILENAME, "Select input file(s)", "If omitted packets are read from standard input.");
	args.add(&out_file, "out", 'o', 0, "Select output file name prefix. If ommited and standard output is a terminal, normal processing is carried out and a report displayed by no files written. If ommited and standard output is not a terminal, CCSDS packets are streamed to it.");
	args.add(&split_by_inst, "inst", 0, 0, "Split output files by instrument instead of by APID");

	args.add(0, "quiet", 'q', 0, "Omit additional messages");
	args.add(0, "", 'z', 0, "Use simple busy animations");
	args.add(0, "debug", Args::USER_OPTION, Args::HIDDEN);

	args.set_args(argc, argv);

	int arg;
	while((arg=args.get_next()) != Args::DONE)
	{
		switch(arg)
		{
		case 'q':
			verbose = false;
			break;

		case Args::USER_OPTION:
			_debug = &cout;
			break;

		case 'z':
			Spinner::use_simple_spinner();
			break;
		}
	}

	// fix streams if writing to stdout
	if ((out_file=="-" || in_files.empty()) && isatty(1)!=1)
	{
		verbose = false;
	}

	FileProducer* source;
	if (in_files.empty())
	{
		source = new ReadFromStdin();
	}
	else
	{
		source = new FileReader(in_files);
	}

	tVCDUReader tvcdu_reader(*source);
	tVCDUToVCDU tvcdu_to_vcdu(tvcdu_reader);
	VCDUToCCSDS vcdu_to_ccsds(tvcdu_to_vcdu);

	if (out_file.empty())
	{
		// no output files given and not streaming to anyone so
		// we just display the interpreted inputs
		CCSDSDemultiplexer demultiplexer(vcdu_to_ccsds, split_by_inst);
		WriteToNull x(demultiplexer);

		source->output_text_summary();
		tvcdu_reader.output_text_summary();
		tvcdu_to_vcdu.output_text_summary();
		vcdu_to_ccsds.output_text_summary();
		demultiplexer.output_text_summary();
		x.output_text_summary();
	}
	else
	{
		CCSDSDemultiplexer demultiplexer(vcdu_to_ccsds, split_by_inst);

		if (out_file=="-")
		{
			WriteToStdout x(&demultiplexer);
		}
		else
		{
			MultiFileWriter x(demultiplexer, out_file);
			if (verbose && !gerror)
			{
				source->output_text_summary();
				tvcdu_reader.output_text_summary();
				tvcdu_to_vcdu.output_text_summary();
				vcdu_to_ccsds.output_text_summary();
				demultiplexer.output_text_summary();
				x.output_text_summary();
			}
		}
	}

	delete source;

	return 0;
}
