////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcducorruptFilter.hpp
//
//	Provides: tVCDUCorruptFilter filter
//
//	Purpose:  Enables the corruption of tVCDU packet common fields
//
//	Copyright (c) 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef tvcducorruptfilter_defined
#define tvcducorruptfilter_defined

#include "interface.hpp"
#include "tvcdu.hpp"
#include "tvcduproducer.hpp"

#define MAX_DAY 65535
#define MAX_MILI_SEC 86399999
#define MAX_MICRO_SEC 999

struct INFO
{
	enum {OFF, SINGLE, INCREMENT, RANDOM} key;
	int start;
	int increment;
	int temp;
	int ctr;

	INFO(void)
		{
			key = OFF;
			start = -1;
			increment = 0;
			temp = -1;
			ctr = 0;
		}

	INFO(const INFO& in)
		{
			key = in.key;
			start = in.start;
			increment = in.increment;
			temp = in.temp;
			ctr = in.ctr;
		}

};


class tVCDUCorruptFilter :
	virtual public Interface,
	public tVCDUProducer,
	public FileProducer
{
public:

	//
	// constructor
	//
	tVCDUCorruptFilter(tVCDUProducer& _source, int _skip, int _count,
					   deque<int>& _selected_vcid, short_cds_time start_time,
					   short_cds_time end_time, int _start_time_relative,
					   int _end_time_relative, int _sequence_counter_start,
					   int _sequence_counter_end, INFO _version_number_info,
					   INFO _scid_info, INFO _vcid_info, INFO _counter_info,
					   INFO _replay_flag_info, INFO _spare_info, INFO _encrypt_flag_info,
					   INFO _encrypt_key_info, INFO _mpdu_spare_info, INFO _mpdu_header_info,
					   INFO _utc_day_info, INFO _utc_millisec_info, INFO _utc_microsec_info,
					   INFO _reed_solomon_status_info, INFO _byte_info,
					   int _byte_offset_start, int _byte_offset_end, string _byte_increment)
	: source(&_source),
		valid(true),
		skip_val(_skip),
		count_val(_count),
		selected_vcid(_selected_vcid),
		start_time_relative(_start_time_relative),
		end_time_relative(_end_time_relative),
		sequence_counter_start(_sequence_counter_start),
		sequence_counter_end(_sequence_counter_end),
		version_number_info(_version_number_info),
		scid_info(_scid_info),
		vcid_info(_vcid_info),
		counter_info(_counter_info),
		replay_flag_info(_replay_flag_info),
		spare_info(_spare_info),
		encrypt_flag_info(_encrypt_flag_info),
		encrypt_key_info(_encrypt_key_info),
		mpdu_spare_info(_mpdu_spare_info),
		mpdu_header_info(_mpdu_header_info),
		utc_day_info(_utc_day_info),
		utc_millisec_info(_utc_millisec_info),
		utc_microsec_info(_utc_microsec_info),
		reed_solomon_status_info(_reed_solomon_status_info),
		byte_info(_byte_info),
		byte_offset_start(_byte_offset_start),
		byte_offset_end(_byte_offset_end),
		byte_increment(_byte_increment)
		{

			// seed the pseudo random number generator with the current time
			srand(static_cast<unsigned>(time(0)));


			packet_count = 0;
			copied_count = 0;
			corrupt_count = 0;

			vcid_match_ctr = 0;
			counter_match_ctr = 0;
			time_match_ctr = 0;
			corruption_ctr = 0;
			affected_ctr = 0;


			if (sequence_counter_start == -1)
			{
				sequence_counter_start = 0;
			}

			if (sequence_counter_end == -1)
			{
				sequence_counter_end = 16777215;
			}


			micro_sec_start = -1.0;
			micro_sec_end = -1.0;

			//short_cds_time start_time, short_cds_time end_time,
			//int start_time_relative, int end_time_relative,
			if (start_time == short_cds_time() && start_time_relative == -1)
			{
				micro_sec_start = 0;
			}
			else
			{
				if (start_time == short_cds_time())
				{
					//use start_time_relative combined with first packet time
					//see get_next_ccsds()
				}
				else
				{
					micro_sec_start = (	  (start_time.get_days()*24.0*60.0*60.0*1.0e6)
										  + (start_time.get_milliseconds()*1.0e3) );
					//+ start_time.get_microseconds() );
				}
			}


			//short_cds_time start_time, short_cds_time end_time,
			//int start_time_relative, int end_time_relative,
			if (end_time == short_cds_time() && end_time_relative == -1)
			{
				micro_sec_end = ( (65535*24.0*60.0*60.0*1.0e6) + (999*1.0e3) + 999 );
			}
			else
			{
				if (end_time == short_cds_time())
				{
					//use end_time_relative combined with first packet time and start_time_relative
					//see get_next_ccsds()
				}
				else
				{
					micro_sec_end = (	(end_time.get_days()*24.0*60.0*60.0*1.0e6)
										+ (end_time.get_milliseconds()*1.0e3) );
					//+ end_time.get_microseconds() );
				}
			}


			for (deque<int>::iterator iter=selected_vcid.begin();
				 iter!=selected_vcid.end(); iter++)
			{
				error_check(*iter);
			}

			error_check(version_number_info, 0, 3, "version-number");
			error_check(scid_info, 0, 255, "space-craft-id");
			error_check(vcid_info, 0, 63, "virtual-channel-id");
			error_check(counter_info, 0, 16777215, "vcdu-counter");
			error_check(replay_flag_info, 0, 1, "replay-flag");
			error_check(spare_info, 0, 127, "spare");
			error_check(encrypt_flag_info, 0, 255, "encrypt-flag");
			error_check(encrypt_key_info, 0, 255, "encrypt-key");
			error_check(mpdu_spare_info, 0, 31, "mpdu-spare");
			error_check(mpdu_header_info, 0, 2047, "mpdu-header");
			error_check(utc_day_info, 0, MAX_DAY, "utc-day");
			error_check(utc_millisec_info, 0, MAX_MILI_SEC, "utc-millisecond");
			error_check(utc_microsec_info, 0, MAX_MICRO_SEC, "utc-microsecond");
			error_check(reed_solomon_status_info, 0, 255, "reed-solomon-status");
			error_check(byte_info, 0, 255, "byte");


		}


	//
	// error_check - VCID
	//
	void error_check(int test_vcid)
		{
			deque <int> acceptable_vcids;
			acceptable_vcids.push_back(3);	//AMSUA-A1, AMSUA-A2, HIRS/4, SEM
			acceptable_vcids.push_back(5);	//AVHRR Low Rate
			acceptable_vcids.push_back(9);	//AVHRR High Rate
			acceptable_vcids.push_back(10); //IASI
			acceptable_vcids.push_back(12); //MHS
			acceptable_vcids.push_back(15); //ASCAT
			acceptable_vcids.push_back(24); //GOME-2
			acceptable_vcids.push_back(27); //A-DCS
			acceptable_vcids.push_back(29); //GRAS
			acceptable_vcids.push_back(34); //GRAS-NAV, Satellite Packet, Admin Packet
			acceptable_vcids.push_back(63); //Fill VCDU

			if (find(acceptable_vcids.begin(),acceptable_vcids.end(),test_vcid) != acceptable_vcids.end())
			{
				// looks good
			}
			else
			{
				error << "Selected VCID " << test_vcid << " is not recognised. "
					  << "To view a list of acceptable VCIDs, use the --show-vcids option.\n";
				valid = false;
			}
		}


	//
	// error_check - INFO structures
	//
	void error_check(INFO test, int min, int max, string field_name)
		{
			if (test.key == INFO::SINGLE || test.key == INFO::INCREMENT)
			{
				if (test.start < min || test.start > max)
				{
					valid = false;
					error << "\nThe value of the input field set-"
						  << field_name << " is out of range: "
						  << test.start << " ("
						  << min << "-" << max << ").\n";
				}
			}

			if (test.key == INFO::INCREMENT)
			{
				if ( (test.increment > max) || (test.increment < (max * -1)) )
				{
					valid = false;
					error << "\nThe value of the input field set-"
						  << field_name << " is out of range: "
						  << test.increment << "\nNote that the increment"
						  << " or step value must be set to a number"
						  << " with a magnitude less than or equal to"
						  << " the maximum value the field of interest"
						  << " can accept (which in this case is "
						  << max << ").\n";
				}
			}

		}



	//
	// check_vcid
	//
	bool check_vcid(int _vcid)
		{

			if (selected_vcid.empty() ||
				find(selected_vcid.begin(), selected_vcid.end(), _vcid) != selected_vcid.end())
			{
				// looks good
				return true;
			}
			else
			{
				return false;
			}
		}




	//
	// print_info
	//

	string print_info (INFO info)
		{
			if (info.key == INFO::OFF)
			{
				return "no";
			}
			else if (info.key == INFO::RANDOM)
			{
				return "random";
			}
			else if (info.key == INFO::SINGLE)
			{
				return lexical_cast<string>(info.start);
			}
			else if (info.key == INFO::INCREMENT)
			{
				return (lexical_cast<string>(info.start) + ", "
						+ lexical_cast<string>(info.increment));
			}
			else
			{
				return "<unknown>";
			}
		}




	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout)
		{

			string vcid_str;
			if (selected_vcid.empty())
			{
				vcid_str = string("ALL");
			}
			else
			{
				for (deque<int>::iterator iter=selected_vcid.begin();
					 iter!=selected_vcid.end(); iter++)
				{
					if (iter == selected_vcid.begin())
					{
						vcid_str = lexical_cast<string>(*iter);
					}
					else
					{
						vcid_str = vcid_str + "," + lexical_cast<string>(*iter);
					}
				}
			}

			double days =  (micro_sec_start / (24.0*60.0*60.0*1.0e6));
			unsigned int utc_days_first = (unsigned int)days;
			double milli_sec = (days - utc_days_first) * (24.0*60.0*60.0*1.0e3);
			unsigned int utc_milliseconds_first = (unsigned int)milli_sec;
			//unsigned int utc_microseconds_first = (unsigned int)((milli_sec - utc_milliseconds_first) * 1e3);

			days =	(micro_sec_end / (24.0*60.0*60.0*1.0e6));
			unsigned int utc_days_second = (unsigned int)days;
			milli_sec = (days - utc_days_second) * (24.0*60.0*60.0*1.0e3);
			unsigned int utc_milliseconds_second = (unsigned int)milli_sec;
			//unsigned int utc_microseconds_second = (unsigned int)((milli_sec - utc_milliseconds_second) * 1e3);

			string firstTimeStr;
			string secondTimeStr;
			if (utc_days_first > 13880)
			{
				firstTimeStr = "Time has exceeded 1st January 2038";
			}
			else
			{
				firstTimeStr = TimeConv::utc_days_milliseconds_to_string(utc_days_first, utc_milliseconds_first);
			}

			if (utc_days_second > 13880)
			{
				secondTimeStr = "Time has exceeded 1st January 2038";
			}
			else
			{
				secondTimeStr = TimeConv::utc_days_milliseconds_to_string(utc_days_second, utc_milliseconds_second);
			}

			if (count_val == -1)
			{
				count_val = packet_count - skip_val;
			}
			if (count_val > packet_count)
			{
				count_val = packet_count;
			}
			if (skip_val >= packet_count)
			{
				skip_val = packet_count;
				count_val = 0 ;
			}
			if ((skip_val + count_val) > packet_count)
			{
				count_val = packet_count - skip_val;
			}

			string tmpOffset;
			if (byte_offset_start != -1 && byte_offset_end != -1)
			{
				tmpOffset = (lexical_cast<string>(byte_offset_start)
							 + ", " + lexical_cast<string>(byte_offset_end));
			}
			else if (byte_offset_start != -1)
			{
				tmpOffset = lexical_cast<string>(byte_offset_start);
			}
			else
			{
				tmpOffset = "none";
			}


			os << "tVCDU_Corrupt_Filter Report:\n"
			   << "	   " << "Corruption Statistics\n"
			   << "		   " << packet_count << " packets read in total\n"
			   << "		   " << copied_count << " packets copied\n"
			   << "		   " << corrupt_count << " packets corrupted\n"
			   << "		   " << corruption_ctr << " corruptions applied to each packet\n"
			   << "		   " << skip_val << " packets skipped\n"
			   << "		   " << count_val << " packets counted\n"
			   << "		   " << vcid_match_ctr << " packets matching selected VCID\n"
			   << "		   " << counter_match_ctr << " packets matching selected VCID and Sequence Counter constraints\n"
			   << "		   " << time_match_ctr << " packets matching selected VCID, Sequence Counter and UTC-time constraints\n"
			   << "	   " << "Corruption Constraints\n"
			   << "		   " << "Virtual Channel ID		 : " << vcid_str << endl
			   << "		   " << "VCDU Counter Start		 : " << sequence_counter_start << endl
			   << "		   " << "VCDU Counter End		 : " << sequence_counter_end << endl
			   << "		   " << "Time Start				 : " << firstTimeStr << endl
			   << "		   " << "UTC-Days Start			 : " << utc_days_first	<< endl
			   << "		   " << "UTC-Milliseconds Start	 : " << utc_milliseconds_first	<< endl
			   << "		   " << "Time End				 : " << secondTimeStr << endl
			   << "		   " << "UTC-Days End			 : " << utc_days_second	 << endl
			   << "		   " << "UTC-Milliseconds End	 : " << utc_milliseconds_second	 << endl
			   << "		   " << "Selected Byte Offset	 : " << tmpOffset << endl
			   << "		   " << "Byte Increment Mode	 : " << byte_increment << endl
			   << "	   " << "Corruptions Applied\n"
			   << "		   " << "set-version-number		 : " << print_info(&version_number_info) << endl
			   << "		   " << "set-space-craft-id		 : " << print_info(&scid_info) << endl
			   << "		   " << "set-virtual-channel-id	 : " << print_info(&vcid_info) << endl
			   << "		   " << "set-vcdu-counter		 : " << print_info(&counter_info) << endl
			   << "		   " << "set-replay-flag		 : " << print_info(&replay_flag_info) << endl
			   << "		   " << "set-spare				 : " << print_info(&spare_info) << endl
			   << "		   " << "set-encrypt-flag		 : " << print_info(&encrypt_flag_info) << endl
			   << "		   " << "set-encrypt-key		 : " << print_info(&encrypt_key_info) << endl
			   << "		   " << "set-mpdu-spare			 : " << print_info(&mpdu_spare_info) << endl
			   << "		   " << "set-mpdu-header		 : " << print_info(&mpdu_header_info) << endl
			   << "		   " << "set-utc-day			 : " << print_info(&utc_day_info) << endl
			   << "		   " << "set-utc-millisecond	 : " << print_info(&utc_millisec_info) << endl
			   << "		   " << "set-utc-microsecond	 : " << print_info(&utc_microsec_info) << endl
			   << "		   " << "set-reed-solomon-status : " << print_info(&reed_solomon_status_info) << endl
			   << "		   " << "set-byte				 : " << print_info(&byte_info) << endl
			   << "	   " << "Packets Affected\n";

			if (affected_ctr == 0)
			{
				os << "		   " << "None" << endl;
			}
			else
			{
				for (int i=0; i<affected_ctr; i++)
				{
					os << "		   " << "VCID					 : " << affected_vcids[i] << endl
					   << "		   " << "Time Start				 : " << affected_times_start[i] << endl
					   << "		   " << "Time End				 : " << affected_times_end[i] << endl;
				}
			}
			os << endl;


		}



	//
	// random_range
	//

	int random_range(int lowest_number, int highest_number)
		{
			if (lowest_number > highest_number){
				swap(lowest_number, highest_number);
			}

			double range = highest_number - lowest_number + 1;
			return lowest_number + int(range * rand()/(RAND_MAX + 1.0));
		}



	//
	// increment
	//

	int increment(INFO input, string param_name, int max_val)
		{
			int result = 0;
			int step = input.increment;
			int ctr = input.ctr;

			if ( (step <= max_val) && (step >= (-1.0 * max_val)) )
			{
				if (ctr == 0)
				{
					result = input.start;
				}
				else
				{
					result = input.temp + step;
				}

				if (result > max_val)
				{
					result = result - (max_val + 1);
				}

				if (result < 0)
				{
					result = result + (max_val + 1);
				}

				return result;
			}
			else
			{
				if (ctr == 0)
				{
					error << "\n*** WARNING: The value of the input field " << param_name << " is out of "
						  << "range and is thus being ignored. Note that the increment or step value "
						  << "must be set to a value with a magnitude less than or equal to the maximum "
						  << "value the field of interest can accept (which in this case is " << max_val << ").\n";
				}
				valid = false;
				return -1;
			}

		}


	//
	// get_next_tvcdu
	//

	virtual tVCDU* get_next_tvcdu(void)
		{
			tVCDU* result = source->get_next_tvcdu();

			// Check to ensure result is meaningful
			if (!result)
			{
				return NULL;
			}

			utc_time = result->get_utc();

			micro_sec = (utc_time.days*24.0*60.0*60.0*1.0e6)
				+ (utc_time.milliseconds*1.0e3)
				+ (utc_time.microseconds);


			if (packet_count == 0)
			{
				if (start_time_relative != -1 && micro_sec_start != -1.0)
				{
					micro_sec_end = micro_sec_start + (start_time_relative * 1.0e3);
				}
				else if (start_time_relative != -1 && micro_sec_start == -1.0)
				{
					micro_sec_start =  micro_sec + (start_time_relative * 1.0e3);
				}
				if (end_time_relative != -1)
				{
					micro_sec_end =	 micro_sec_start + (end_time_relative * 1.0e3);
				}
			}

			if (skip_val == -1)
			{
				skip_val = 0;
			}
			if (packet_count < skip_val)
			{
				// return unmodified CCSDS packet
				packet_count++;
				copied_count++;
				return result;
			}
			if ((count_val != -1) && (packet_count >= (skip_val + count_val)))
			{
				// return unmodified CCSDS packet
				packet_count++;
				copied_count++;
				return result;
			}


			vcid = result->get_vcid();

			// Only proceed if VCID conditions are met
			if (check_vcid(vcid))
			{
				vcid_match_ctr++;

				sequence_counter = result->get_counter();

				// Only proceed if the sequence counter conditions are met
				if ( (sequence_counter >= sequence_counter_start)
					 && (sequence_counter <= sequence_counter_end) )
				{
					counter_match_ctr++;


					// Only proceed if the UTC time conditions are met
					if ( (micro_sec >= micro_sec_start) && (micro_sec <= micro_sec_end) )
					{

						corruption_ctr = 0;
						time_match_ctr++;


						// Corrupting Version Number (0-3)
						if (version_number_info.key == INFO::SINGLE)
						{
							result->set_version_number(version_number_info.start);
							corruption_ctr++;
						}
						else if (version_number_info.key == INFO::RANDOM)
						{
							result->set_version_number(random_range(0, 3));
							corruption_ctr++;
						}
						else if (version_number_info.key == INFO::INCREMENT)
						{
							int tmp = increment (version_number_info,
												 "set-version-number", 3);

							version_number_info.ctr++;
							if (tmp != -1)
							{
								version_number_info.temp = tmp;
								result->set_version_number(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting SCID (0-255)
						if (scid_info.key == INFO::SINGLE)
						{
							result->set_scid(scid_info.start);
							corruption_ctr++;
						}
						else if (scid_info.key == INFO::RANDOM)
						{
							result->set_scid(random_range(0, 255));
							corruption_ctr++;
						}
						else if (scid_info.key == INFO::INCREMENT)
						{
							int tmp = increment (scid_info,
												 "set-space-craft-id", 255);

							scid_info.ctr++;
							if (tmp != -1)
							{
								scid_info.temp = tmp;
								result->set_scid(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting VCID (0-63)
						if (vcid_info.key == INFO::SINGLE)
						{
							result->set_vcid(vcid_info.start);
							corruption_ctr++;
						}
						else if (vcid_info.key == INFO::RANDOM)
						{
							result->set_vcid(random_range(0, 63));
							corruption_ctr++;
						}
						else if (vcid_info.key == INFO::INCREMENT)
						{
							int tmp = increment (vcid_info,
												 "set-virtual-channel-id", 63);

							vcid_info.ctr++;
							if (tmp != -1)
							{
								vcid_info.temp = tmp;
								result->set_vcid(tmp);
								corruption_ctr++;
							}
						}

						// Corrupting VCDU Counter (0-16777215)
						if (counter_info.key == INFO::SINGLE)
						{
							result->set_counter(counter_info.start);
							corruption_ctr++;
						}
						else if (counter_info.key == INFO::RANDOM)
						{
							result->set_counter(random_range(0, 16777215));
							corruption_ctr++;
						}
						else if (counter_info.key == INFO::INCREMENT)
						{
							int tmp = increment (counter_info,
												 "set-vcdu-counter",
												 16777215);
							counter_info.ctr++;
							if (tmp != -1)
							{
								counter_info.temp = tmp;
								result->set_counter(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting Replay Flag (0-1)
						if (replay_flag_info.key == INFO::SINGLE)
						{
							result->set_replay_flag(replay_flag_info.start);
							corruption_ctr++;
						}
						else if (replay_flag_info.key == INFO::RANDOM)
						{
							result->set_replay_flag(random_range(0, 1));
							corruption_ctr++;
						}
						else if (replay_flag_info.key == INFO::INCREMENT)
						{
							int tmp = increment (replay_flag_info,
												 "set-replay-flag", 1);

							replay_flag_info.ctr++;
							if (tmp != -1)
							{
								replay_flag_info.temp = tmp;
								result->set_replay_flag(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting Spare (0-127)
						if (spare_info.key == INFO::SINGLE)
						{
							result->set_spare(spare_info.start);
							corruption_ctr++;
						}
						else if (spare_info.key == INFO::RANDOM)
						{
							result->set_spare(random_range(0, 127));
							corruption_ctr++;
						}
						else if (spare_info.key == INFO::INCREMENT)
						{
							int tmp = increment (spare_info,
												 "set-spare", 127);

							spare_info.ctr++;
							if (tmp != -1)
							{
								spare_info.temp = tmp;
								result->set_spare(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting Encrypt Flag (0-255)
						if (encrypt_flag_info.key == INFO::SINGLE)
						{
							result->set_encrypt_flag(encrypt_flag_info.start);
							corruption_ctr++;
						}
						else if (encrypt_flag_info.key == INFO::RANDOM)
						{
							result->set_encrypt_flag(random_range(0, 255));
							corruption_ctr++;
						}
						else if (encrypt_flag_info.key == INFO::INCREMENT)
						{
							int tmp = increment (encrypt_flag_info,
												 "set-encrypt-flag", 255);

							encrypt_flag_info.ctr++;
							if (tmp != -1)
							{
								encrypt_flag_info.temp = tmp;
								result->set_encrypt_flag(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting Encrypt Key (0-255)
						if (encrypt_key_info.key == INFO::SINGLE)
						{
							result->set_encrypt_key(encrypt_key_info.start);
							corruption_ctr++;
						}
						else if (encrypt_key_info.key == INFO::RANDOM)
						{
							result->set_encrypt_key(random_range(0, 255));
							corruption_ctr++;
						}
						else if (encrypt_key_info.key == INFO::INCREMENT)
						{
							int tmp = increment (encrypt_key_info,
												 "set-encrypt-key", 255);

							encrypt_key_info.ctr++;
							if (tmp != -1)
							{
								encrypt_key_info.temp = tmp;
								result->set_encrypt_key(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting MPDU-Spare (0-31)
						if (mpdu_spare_info.key == INFO::SINGLE)
						{
							result->set_mpdu_spare(mpdu_spare_info.start);
							corruption_ctr++;
						}
						else if (mpdu_spare_info.key == INFO::RANDOM)
						{
							result->set_mpdu_spare(random_range(0, 31));
							corruption_ctr++;
						}
						else if (mpdu_spare_info.key == INFO::INCREMENT)
						{
							int tmp = increment (mpdu_spare_info,
												 "set-mpdu-spare", 31);

							mpdu_spare_info.ctr++;
							if (tmp != -1)
							{
								mpdu_spare_info.temp = tmp;
								result->set_mpdu_spare(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting MPDU-header (0-2047)
						if (mpdu_header_info.key == INFO::SINGLE)
						{
							result->set_mpdu_header(mpdu_header_info.start);
							corruption_ctr++;
						}
						else if (mpdu_header_info.key == INFO::RANDOM)
						{
							result->set_mpdu_header(random_range(0, 2047));
							corruption_ctr++;
						}
						else if (mpdu_header_info.key == INFO::INCREMENT)
						{
							int tmp = increment (mpdu_header_info,
												 "set-mpdu-header", 2047);

							mpdu_header_info.ctr++;
							if (tmp != -1)
							{
								mpdu_header_info.temp = tmp;
								result->set_mpdu_header(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting UTC-Days (0-65535)
						if (utc_day_info.key == INFO::SINGLE)
						{
							result->set_utc_day(utc_day_info.start);
							corruption_ctr++;
						}
						else if (utc_day_info.key == INFO::RANDOM)
						{
							result->set_utc_day(random_range(0, 65535));
							corruption_ctr++;
						}
						else if (utc_day_info.key == INFO::INCREMENT)
						{
							int tmp = increment (utc_day_info,
												 "set-utc-day", 65535);

							utc_day_info.ctr++;
							if (tmp != -1)
							{
								utc_day_info.temp = tmp;
								result->set_utc_day(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting UTC-Milliseconds (0-86399999)
						if (utc_millisec_info.key == INFO::SINGLE)
						{
							result->set_utc_millisec(utc_millisec_info.start);
							corruption_ctr++;
						}
						else if (utc_millisec_info.key == INFO::RANDOM)
						{
							result->set_utc_millisec(random_range(0, 86399999));
							corruption_ctr++;
						}
						else if (utc_millisec_info.key == INFO::INCREMENT)
						{
							int tmp = increment (utc_millisec_info,
												 "set-utc-millisec", 86399999);

							utc_millisec_info.ctr++;
							if (tmp != -1)
							{
								utc_millisec_info.temp = tmp;
								result->set_utc_millisec(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting UTC-Microseconds (0-999)
						if (utc_microsec_info.key == INFO::SINGLE)
						{
							result->set_utc_microsec(utc_microsec_info.start);
							corruption_ctr++;
						}
						else if (utc_microsec_info.key == INFO::RANDOM)
						{
							result->set_utc_microsec(random_range(0, 999));
							corruption_ctr++;
						}
						else if (utc_microsec_info.key == INFO::INCREMENT)
						{
							int tmp = increment (utc_microsec_info,
												 "set-utc-microsec", 999);

							utc_microsec_info.ctr++;
							if (tmp != -1)
							{
								utc_microsec_info.temp = tmp;
								result->set_utc_microsec(tmp);
								corruption_ctr++;
							}
						}


						// Corrupting Reed Solomon Status (0-255)
						if (reed_solomon_status_info.key == INFO::SINGLE)
						{
							result->set_reed_solomon_status(reed_solomon_status_info.start);
							corruption_ctr++;
						}
						else if (reed_solomon_status_info.key == INFO::RANDOM)
						{
							result->set_reed_solomon_status(random_range(0, 255));
							corruption_ctr++;
						}
						else if (reed_solomon_status_info.key == INFO::INCREMENT)
						{
							int tmp = increment (reed_solomon_status_info,
												 "set-reed-solomon-status", 255);

							reed_solomon_status_info.ctr++;
							if (tmp != -1)
							{
								reed_solomon_status_info.temp = tmp;
								result->set_reed_solomon_status(tmp);
								corruption_ctr++;
							}
						}



						// Corrupting a range of bytes at a given offset
						if (byte_offset_start >= tVCDU::SIZE)
						{
							error << "\nSpecified byte offset is equal to or greater "
								  << "than the total t-VCDU packet length ("
								  << tVCDU::SIZE << ").\n";
							valid = false;
							return NULL;
						}
						if (byte_offset_end >= tVCDU::SIZE)
						{
							byte_offset_end = tVCDU::SIZE - 1;
						}
						if (byte_offset_start > byte_offset_end && byte_offset_end != -1)
						{
							int tmp_offset = byte_offset_end;
							byte_offset_end = byte_offset_start;
							byte_offset_start = tmp_offset;
						}

						if (byte_offset_start != -1 && byte_offset_end == -1)
						{
							uint8 tmp_byte_val;
							if (byte_info.key == INFO::RANDOM)
							{
								tmp_byte_val = (uint8)random_range(0, 255);
								result->set_byte(tmp_byte_val, byte_offset_start);
								corruption_ctr++;
							}
							else
							{
								if (byte_info.start != -1 && byte_info.increment == 0 &&
									byte_info.key == INFO::SINGLE)
								{
									tmp_byte_val = (uint8)byte_info.start;
									result->set_byte(tmp_byte_val, byte_offset_start);
									corruption_ctr++;
								}
								else if (byte_info.start != -1 && byte_info.increment != 0 &&
										 byte_info.key == INFO::INCREMENT)
								{
									int tmp = increment (byte_info, "set-byte", 255);
									byte_info.ctr++;
									if (tmp != -1)
									{
										byte_info.temp = tmp;
										tmp_byte_val = (uint8)tmp;
										result->set_byte(tmp_byte_val, byte_offset_start);
										corruption_ctr++;
									}
								}
							}
						}
						else if (byte_offset_start != -1 && byte_offset_end != -1)
						{
							uint8 tmp_byte_val;
							if (byte_info.key == INFO::RANDOM)
							{
								for (int i=byte_offset_start; i<=byte_offset_end; i++)
								{
									tmp_byte_val = (uint8)random_range(0, 255);
									result->set_byte(tmp_byte_val, i);
								}
								corruption_ctr++;
							}
							else
							{
								if (byte_info.start != -1 && byte_info.increment == 0 &&
									byte_info.key == INFO::SINGLE)
								{
									tmp_byte_val = (uint8)byte_info.start;
									for (int i=byte_offset_start; i<=byte_offset_end; i++)
									{
										result->set_byte(tmp_byte_val, i);
									}
									corruption_ctr++;
								}
								else if (byte_info.start != -1 && byte_info.increment != 0 &&
										 byte_info.key == INFO::INCREMENT)
								{
									int tmp;
									if (byte_increment == "row" || byte_increment == "both")
									{
										tmp = increment (byte_info, "set-byte", 255);
										byte_info.ctr++;
									}
									else
									{
										tmp = byte_info.start;
									}

									if (tmp != -1)
									{
										byte_info.temp = tmp;
										tmp_byte_val = (uint8)tmp;
										int ctr = 0;
										for (int i=byte_offset_start; i<=byte_offset_end; i++)
										{
											if (byte_increment == "column" || byte_increment == "both")
											{
												tmp_byte_val = byte_info.temp + (byte_info.increment * ctr);
												ctr++;
											}
											result->set_byte(tmp_byte_val, i);
										}
										corruption_ctr++;
									}
								}
							}
						}



						// return modified packet
						packet_count++;
						if (corruption_ctr > 0)
						{
							corrupt_count++;

							if (corrupt_count == 1)
							{
								affected_vcids[affected_ctr] = vcid;
								affected_times_start[affected_ctr] =
									TimeConv::utc_days_milliseconds_to_string(utc_time.days,
																			  utc_time.milliseconds);
								affected_times_end[affected_ctr] =
									TimeConv::utc_days_milliseconds_to_string(utc_time.days,
																			  utc_time.milliseconds);
								affected_ctr++;
							}
							else
							{
								bool new_vcid = true;
								for (int i=0; i<affected_ctr; i++)
								{
									if (affected_vcids[i] == vcid)
									{
										affected_times_end[i] =
											TimeConv::utc_days_milliseconds_to_string(utc_time.days,
																					  utc_time.milliseconds);
										new_vcid = false;
									}
								}
								if (new_vcid)
								{
									affected_vcids[affected_ctr] = vcid;
									affected_times_start[affected_ctr] =
										TimeConv::utc_days_milliseconds_to_string(utc_time.days,
																				  utc_time.milliseconds);
									affected_times_end[affected_ctr] =
										TimeConv::utc_days_milliseconds_to_string(utc_time.days,
																				  utc_time.milliseconds);
									affected_ctr++;
								}
							}

						}
						else
						{
							copied_count++;
						}
						return result;


					} //UTC Time Condition
					else
					{
						// return unmodified
						packet_count++;
						copied_count++;
						return result;
					}

				} // Sequence Counter Condition
				else
				{
					// return unmodified
					packet_count++;
					copied_count++;
					return result;
				}

			} // VCID Condition
			else
			{
				// return unmodified
				packet_count++;
				copied_count++;
				return result;
			}

//	X	set_version_number(int vn)
//	X	set_scid(int scid)
//	X	set_vcid(int vcid)
//	X	set_counter(int counter)
//	X	set_replay_flag(int flag)
//	X	set_spare(int spare)
//	X	set_encrypt_flag(int flag)
//	X	set_encrypt_key(int key)
//	X	set_mpdu_spare()
//	X	set_mpdu_header()
//	X	set_utc(UTC& t)
//	X	set_reed_solomon_status(int rss)
//	   set_byte

			return result;

		}




	//
	// produce_files
	//

	virtual bool produce_files(FileConsumer& dest)
		{
			tVCDU* tvcdu;
			while((tvcdu=get_next_tvcdu()))
			{
				ostream_type* os = dest.get_named_output_stream("output.tvcdu");
				os->write((char*)tvcdu->get_data(),tVCDU::SIZE);
			}
			return true;
		}



	//
	// get_next_file
	//

	virtual istream_type* get_next_file(void)
		{
			return NULL;
		}



	//
	// is_valid
	//

	bool is_valid(void) const
		{
			return valid;
		}


private:


	tVCDUProducer* source;
	bool valid;

	int packet_count;
	int copied_count;
	int corrupt_count;

	int vcid_match_ctr;
	int counter_match_ctr;
	int time_match_ctr;
	int corruption_ctr;
	int affected_ctr;

	int affected_vcids[512];
	string affected_times_start[512];
	string affected_times_end[512];

	int vcid;
	int sequence_counter;
	UTCTime utc_time;
	double micro_sec;

	int skip_val;
	int count_val;
	deque<int> selected_vcid;
	int sequence_counter_start;
	int sequence_counter_end;
	double micro_sec_start;
	double micro_sec_end;
	int start_time_relative;
	int end_time_relative;


	INFO version_number_info;
	INFO scid_info;
	INFO vcid_info;
	INFO counter_info;
	INFO replay_flag_info;
	INFO spare_info;
	INFO encrypt_flag_info;
	INFO encrypt_key_info;
	INFO mpdu_spare_info;
	INFO mpdu_header_info;
	INFO utc_day_info;
	INFO utc_millisec_info;
	INFO utc_microsec_info;
	INFO reed_solomon_status_info;
	INFO byte_info;

	int byte_offset_start;
	int byte_offset_end;
	string byte_increment;

};


#endif

