////////////////////////////////////////////////////////////////////////////////
//
//	File: tvcdutovcdu.hpp
//
//	Provides: tVCDUToVCDU filter
//
//	Purpose: Convert a stream of t-VCDU packets to VCDU packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef tvcdutovcdu_defined
#define tvcdutovcdu_defined

#include "interface.hpp"
#include "tvcduproducer.hpp"
#include "vcduproducer.hpp"
#include "mpduproducer.hpp"

class tVCDUToVCDU :
	virtual public Interface,
	public VCDUProducer,
	public MPDUProducer
{
public:

	//
	// constructor
	//

	tVCDUToVCDU(tVCDUProducer& _source,
				bool _strip_filler = true,
				bool _strip_encrypted_packets = false)
		: source(_source),
		  packet_count(0),
		  filler_count(0),
		  encrypt_count(0),
		  strip_filler(_strip_filler),
		  strip_encrypted_packets(_strip_encrypted_packets),
		  version_warnings_count(0)
		{
			earliest_time.days = 65000; //numeric_limits<uint16_be>::max(); ??
			earliest_time.milliseconds = 0;
			earliest_time.microseconds = 0;
			latest_time.days = 0;
			latest_time.milliseconds = 0;
			latest_time.microseconds = 0;
		}

	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout)
		{
			os << "tVCDUToVCDU report:\n"
			   << "	   " << packet_count << " packets were read\n"
			   << "	   Earliest tVCDU timestamp was " << earliest_time << endl
			   << "	   Latest tVCDU timestamp was " << latest_time << endl;

			if (filler_count > 0)
			{
				if (strip_filler)
				{
					os << "	   " << filler_count
					   << " fill VCDUs were skipped\n";
				}
				else
				{
					os << "	   " << filler_count << " filler packets were passed through\n";
				}
			}

			if (strip_encrypted_packets)
			{
				os << "	   " << encrypt_count << " encrypted VCDUs were removed\n";
			}
			else
			{
				os << "	   " << encrypt_count << " encrypted VCDUs were let through\n";
			}

			for(SCIDCount::iterator iter=scid_count.begin();
				iter!=scid_count.end();
				iter++)
			{
				os << "	   " << ((*iter).second) << " data packets had SCID "
				   << "of " << ((*iter).first);

				switch((*iter).first)
				{
				case VCID_METOP_1 : os << " (METOP-1)"; break;
				case VCID_METOP_2 : os << " (METOP-2)"; break;
				case VCID_METOP_3 : os << " (METOP-3)"; break;
				case VCID_METOP_SIM : os << " (METOP-SIM)"; break;
				default : os << " (unknown)"; break;
				}

				os << endl;
			}
			os << endl;
		}

	//
	// get_next_vcdu
	//

	virtual VCDU* get_next_vcdu(void)
		{
			tVCDU* tvcdu = source.get_next_tvcdu();

			VCDU* vcdu = reinterpret_cast<VCDU*>(tvcdu);
			if (!vcdu)
			{
				return NULL;
			}

			if (tvcdu->utc < earliest_time)
			{
				earliest_time = tvcdu->utc;
			}

			if (tvcdu->utc > latest_time)
			{
				latest_time = tvcdu->utc;
			}

			// if its a filler packet strip it out now
			if (strip_filler)
			{
				while(vcdu->get_vcid() == VCID_FILL)
				{
					filler_count++;

					VCDU* vcdu =
						reinterpret_cast<VCDU*>(source.get_next_tvcdu());

					if (!vcdu)
					{
						return NULL;
					}
				}
			}
			else
			{
				if (vcdu->get_vcid() == VCID_FILL)
				{
					filler_count++;
				}
			}

			if (vcdu->get_version_number() != VCDU_CURRECT_VERSION)
			{
				version_warnings_count++;
				if (version_warnings_count < 10)
				{
					info << "CVCDUToVCDU::get_next_vcdu Input packet found "
						 << " with incorrect VCDU version of "
						 << vcdu->get_version_number()
						 << " (expecting " << VCDU_CURRECT_VERSION
						 << ") in packet"
						 << "number " << packet_count << endl;
				}
				else if (version_warnings_count == 10)
				{
					info << "CVCDUToVCDU::get_next_vcdu Further VCDU version "
						 << " warnings will not be shown\n";
				}

				// allow the packet through anyway (it may still be ok)
			}

			// if its encrypted then strip it out
			if (vcdu->get_encrypt_flag() == VCDU_ENCRYPTED)
			{
				encrypt_count++;
				if (strip_encrypted_packets)
				{
					return get_next_vcdu();
				}
			}

			// increment the SCID counter
			if (vcdu->get_vcid() != VCID_FILL)
			{
				scid_count[vcdu->get_scid()] =
					scid_count[vcdu->get_scid()]+1;
			}

			packet_count++;
			return vcdu;
		}

	virtual MPDU* get_next_mpdu(void)
		{
			VCDU* vcdu = get_next_vcdu();
			if (!vcdu)
			{
				return NULL;
			}
			return (&(vcdu->mpdu));
		}

private:

	tVCDUProducer& source;

	int packet_count;
	int filler_count;
	int encrypt_count;

	bool strip_filler;
	bool strip_encrypted_packets;

	typedef map<unsigned int, int> SCIDCount;
	SCIDCount scid_count;

	UTCTime earliest_time;
	UTCTime latest_time;

	int version_warnings_count;
};

#endif
