////////////////////////////////////////////////////////////////////////////////
//
//	File: vcdutoccsds.cpp
//
//	Provides: VCDUToCCSDS filter
//
//	Purpose: Convert a stream of VCDU packets to CCSDs packets
//
//	Copyright (c) 2002, 2003, 2004, 2005 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "vcdutoccsds.hpp"

#include "cvcduproducer.hpp"
#include "ccsdsproducer.hpp"

const int MAX_VCCOUNT_ERRORS = 500;

//
//
// VCDUToCCSDS
//
//

//
// constructor
//

VCDUToCCSDS::VCDUToCCSDS(VCDUProducer& _source)
	: source(_source),
	  total_vcdus(0),
	  good_ccsds(0),
	  current_vc(MAX_VCID),
	  mpdu_spare_noheader(0),
	  mpdu_pointer_noheader(0),
	  unmapped_bytes(0),
	  initial_unmapped(0),
	  hold(NULL)
{
	assert(VCDU::PAYLOAD_SIZE==sizeof(MPDU));
}

//
// destructor
//

VCDUToCCSDS::~VCDUToCCSDS(void)
{
	delete[] hold;
}

//
// output_text_summary
//

void VCDUToCCSDS::output_text_summary(ostream& os)
{
	os << "VCDUToCCSDS report:\n"
	   << "	   " << total_vcdus << " VCDUs were recieved\n"
	   << "	   Virtual Channel traffic:\n";
	for(ChannelCounters::iterator iter = vccounts.begin();
		iter!=vccounts.end();
		iter++)
	{
		os << "		   Channel " << ((*iter).first)
		   << " had " << ((*iter).second) << " packets\n";
	}

	os << "	   " << unmapped_bytes << " unused bytes inside M-PDUs ("
	   << initial_unmapped << " initial unused bytes)\n"
	   << "	   " << vccount_errors.size() << " VCDUs had "
	   << "non-consecutive counter values\n"
	   << "	   " << good_ccsds << " CCSDS packets produced\n";
	if (!vccount_errors.empty())
	{
		os << "	   Error Listing (Packet No:VCID:Desc): ";
		for(VCErrors::iterator iter=vccount_errors.begin();
			iter!=vccount_errors.end();
			iter++)
		{
			os << (*iter) << " ";
		}
		os << "\n";
	}

	if (mpdu_spare_noheader != mpdu_pointer_noheader)
	{
		os << "	   WARNING: There was a mismatch between the number of "
		   << "M-PDU packets indicating no CCSDS header present in the "
		   << "SPARE field (" << mpdu_spare_noheader << ") and in the "
		   << "FIRST_HEADER_POINTER field (" << mpdu_pointer_noheader
		   << ")\n\n";
	}

	os << endl;
}

EPSProducer* VCDUToCCSDS::as_eps_product(void)
{
	return NULL;
}

//
// get_next_ccsds
//

CCSDS* VCDUToCCSDS::get_next_ccsds(void)
{
	if (!spinner.get())
	{
		spinner.reset(new Spinner());
	}
	spinner->tick();

	// first flush the VBs
	for(VirtualBuffers::iterator iter=vbs.begin();
		iter!=vbs.end();
		iter++)
	{
		CCSDS* ccsds = (*iter).second.get_ccsds();
		if (ccsds)
		{
			DEBUG("Returning FLUSHED packet - APID:"
				  << ccsds->get_apid()
				  << " Length: "  << ccsds->get_length()
				  << " Flags: "	  << ccsds->get_sequence_flags()
				  << " Counter: " << total_vcdus
				  << "\n");
			result.reset(ccsds);
			good_ccsds++;
			return result.get();
		}
	}

	// check if a previous packet zone was held for later extraction
	if (hold)
	{
		// hold_vc is VCID that needs to be held while reading consecutive packets
		vbs[hold_vc].insert(hold, hold_size);
		delete[] hold;
		hold = NULL;
		CCSDS* ccsds = vbs[hold_vc].get_ccsds();
		if (ccsds)
		{
			DEBUG("Returning HELD packet\n");
			good_ccsds++;
			result.reset(ccsds);
			return result.get();
		}
	}

	// check if we have another VCDU to unpack from the source data
	while(VCDU* vcdu = source.get_next_vcdu())
	{
		// keep count of total VCDUs
		total_vcdus++;

		VCID vcid = vcdu->get_vcid();
		if (vcid==VCID_FILL)
		{
			continue;
		}

		DEBUG("VCID version " << vcdu->get_version_number()
			  << " scid " << vcdu->get_scid()
			  << " vcid " << vcdu->get_vcid()
			  << " counter " << vcdu->get_vccount()
			  << " replay " << vcdu->get_replay_flag()
			  << " spare " << vcdu->get_spare()
			  << " encrypt flag " << vcdu->get_encrypt_flag()
			  << " key " << vcdu->get_encrypt_key()
			  << " FHP " << vcdu->mpdu.get_first_header_pointer()
			  << endl);

		// create virtual buffer for given VCID
		VirtualBuffer& vb = vbs[vcid];

		// check first header pointer of the enclosed MPDU of the CVCDU
		int fhp = vcdu->mpdu.get_first_header_pointer();

		// increment counter for current VCID only
		int last_count = vccounts[vcid];
		vccounts[vcid] = last_count + 1;

		// check the VCDU counter values
		bool allow_hold = check_counter(vb, vcdu, last_count);

		// means M-PDU packet zone contains CCSDS Header and Data
		// i.e. we have a large CCSDS packet
		if (fhp != 2047)
		{
			// FHP cannot be larger than allowed payload size as this will
			// go beyond the payload data pointer
			if (fhp >= MPDU::PAYLOAD_SIZE)
			{
				cerr << "Found bad M-PDU packet with FHP of " << fhp
					 << " (max " << MPDU::PAYLOAD_SIZE << ")\n";
				continue;
			}

			// read remaining packet data for previous CCSDS packet into
			// current CCSDS, however disgard this data if an out of sequence VCDU was detected
			//
			// NOTE: The insert function may encounter a problem with regards to
			//		 recovery whereby non-consecutive packets will interfere with the
			//		 payload section of the MPDU therefore disguard all data before fhp
			CCSDS* ccsds = NULL;
			if (allow_hold)
			{
				vb.insert(
					reinterpret_cast<byte*>(&vcdu->mpdu.payload),
					fhp);
				ccsds = vb.get_ccsds();
			}
			else
			{
				// dropped all data that resides just before the FHP
				unmapped_bytes += fhp;
			}

			// is this a duplicate VCDU, if not then process as normal, otherwise drop
			if ( vb.get_bad() != VCDU_DUPS )
			{
				hold_size = MPDU::PAYLOAD_SIZE-fhp;
				delete[] hold;
				hold = new byte[hold_size];
				memcpy(hold,
					   reinterpret_cast<byte*>(&vcdu->mpdu.payload)+fhp,
					   hold_size);
				hold_vc = vcdu->get_vcid();
				DEBUG("Put " << hold_size << " bytes into hold for VCID "
					  << hold_vc << endl);
			}
			else
			{
				unmapped_bytes += MPDU::PAYLOAD_SIZE-fhp;
			}

			// found full CCSDS within this Virtual Channel
			if (ccsds)
			{
				good_ccsds++;
				result.reset(ccsds);
				return result.get();
			}
			// otherwise check if we have an almost complete CCSDS
			else if ( vb.get_bad() != VCDU_DUPS )
			{
				if (vb.is_warm())
				{
					unmapped_bytes += vb.reinit();
				}
				else
				{
					initial_unmapped += vb.reinit();
				}
				if ( hold != NULL ) {
					vb.insert(hold, hold_size);
					delete[] hold;
					hold = NULL;
				}
				ccsds = vb.get_ccsds();
				if (ccsds)
				{
					good_ccsds++;
					result.reset(ccsds);
					return result.get();
				}
			}
		}
		// Packet Zone contains Data only
		else if (allow_hold)
		{
			vb.insert(
				reinterpret_cast<byte*>(&vcdu->mpdu.payload),
				MPDU::PAYLOAD_SIZE);
			CCSDS* ccsds = vb.get_ccsds();
			//DEBUG("extracted pkt " << (unsigned long long)ccsds << endl);
			if (ccsds)
			{
				good_ccsds++;
				result.reset(ccsds);
				return result.get();
			}
		}
		else 
		{
			// entire packet must be dropped as the data is incomplete
			unmapped_bytes += MPDU::PAYLOAD_SIZE;
		}
	}
	spinner->end();
	return NULL;
}

//
// check_counter
//

bool VCDUToCCSDS::check_counter(VirtualBuffer& vb, VCDU* vcdu, int last_count)
{
	// Current counter for this VCDU
	int counter = vcdu->get_vccount();

	// current VCID value of VDCU
	VCID vcid = vcdu->get_vcid();

	// flag used to detect non-consecutive VCDUs
	bool allow_hold = true;

	// check if vcount is valid according to known criteria
	if ( !vb.check_vccount(counter) )
	{
		// default bad state is unknown
		VCDU_STATE bad_flag = VCDU_UNKNOWN;

		// duplicate packet was found
		if (counter == last_count)
		{
			bad_flag = VCDU_DUPS;
		}
		else
		{
			bad_flag = VCDU_BREAK;

			// clear previous buffer of previous VCID since its incomplete
			unmapped_bytes += vb.clear();
		}

		// check allowable maximum count and only add if its not reached
		if (vccount_errors.size() < MAX_VCCOUNT_ERRORS)
		{
			string reason = string("(")
				+ lexical_cast<string>(total_vcdus) + ":"
				+ lexical_cast<string>(vcid)
				+ ":" + get_bad_text(bad_flag) + ") ";
			// add formatted text string to errors list
			vccount_errors.push_back(reason);
		}
		allow_hold = false; // disguard all data that was before fhp

		// store state of bad flag in current Virtual Channel
		vb.set_bad(bad_flag);

		DEBUG("Found non-consecutive VCID: "
			  << vcid
			  << " At: "
			  << last_count
			  << " Possible reason: "
			  << get_bad_text(bad_flag) << endl);
	}
	else
	{
		vb.clear_bad();
	}

	return allow_hold;
}


///
/// get_bad_text
///

string VCDUToCCSDS::get_bad_text(VCDU_STATE bad_state) const
{
	// handle all possible bad flag states
	switch( bad_state )
	{
	case VCDU_UNKNOWN:
		return "UNKNOWN";
	case VCDU_OK:
		return "OK";
	case VCDU_BREAK:
		return "BREAK";
	case VCDU_DUPS:
		return "DUPS";
	default:
		return "UNKNOWN";
	}
}

//
//
// VCDUToCCSDS::VirtualBuffer
//
//

//
// Constructor
//

VCDUToCCSDS::VirtualBuffer::VirtualBuffer(void)
	: ptr(0),
	  ptr_target(0),
	  last_vccount(-1),
	  warm(false)
{
}

//
// check_vccount
//

bool VCDUToCCSDS::VirtualBuffer::check_vccount(int vccount)
{
	bool ok = false;
	// is vcount next in sequence
	if (vccount == last_vccount+1)
	{
		ok = true;
	}
	// vcount is zero but last cached count is MAXIMUM VC Count
	// meaning that we have a wrap around case for SAME VCID
	else if (vccount==0 && last_vccount==(MAX_VCCOUNT-1))
	{
		ok = true;
	}
	// Last VCOUNT is -1
	else if (last_vccount==-1)
	{
		ok = true;
	}
	last_vccount = vccount;
	return ok;
}


//
// reinit
//

int VCDUToCCSDS::VirtualBuffer::reinit(void)
{
	DEBUG("reinit\n");
	int lost = ptr;
	ptr = 0;
	ptr_target = 0;

	if (!warm)
	{
		warm = true;
	}
	return lost;
}


//
// clear
//

int VCDUToCCSDS::VirtualBuffer::clear(void)
{
	DEBUG("clear\n");
	int lost = ptr;
	ptr = 0;
	ptr_target = 0;

	last_vccount = -1;

	warm = false;
	return lost;
}


//
// insert
//

void VCDUToCCSDS::VirtualBuffer::insert(byte* new_buffer, int len)
{
	memcpy(buffer+ptr, new_buffer, len);
	ptr += len;

	DEBUG("inserted " << len << " ptr is now " << ptr << endl);
}

//
// get_ccsds
//

CCSDS* VCDUToCCSDS::VirtualBuffer::get_ccsds(void)
{
	if (!warm)
	{
		return NULL;
	}

	if (ptr_target == 0)
	{
		DEBUG("looking for CCSDS header\n");

		if (ptr >= 6)
		{
			// extract first 6 bytes of the buffer to construct header
			CCSDSPrimaryHeader* head =
				reinterpret_cast<CCSDSPrimaryHeader*>(buffer);

			// Total CCSDS packet length including packet header and data
			ptr_target = head->get_packet_length() + 7;

			DEBUG("Header found version "
				  << head->get_version_number() << " type "
				  << head->get_type()  << " 2nd hdr "
				  << head->get_secondary_header_flag() << " apid "
				  << head->get_apid() << " seq flags "
				  << head->get_sequence_flags() << " seq count "
				  << head->get_sequence_count() << " len "
				  << head->get_packet_length() << " target "
				  << ptr_target << endl);
		}
	}

	// if packet length and data equates to total combined MPDU payload then
	// we have a full CCSDS and so extract the complete CCSDS and return CCSDS object
	if (ptr_target!=0 && ptr>=ptr_target)
	{
		DEBUG("Target of " << ptr_target
			  << " is reached with buffer content of " << ptr
			  << " ...\n");

		// extract complete CCSDS from buffer, but make COPY of source data
		CCSDS* result = new CCSDS(buffer, true);

		// move remaining data to start of buffer
		memmove(buffer, buffer+ptr_target, ptr-ptr_target);
		ptr -= ptr_target; // and decrement buffer length by remainder
		ptr_target = 0;

		DEBUG("	   ... returning packet APID "
			  << result->get_apid() << " real length "
			  << result->get_length() << " buffer now has "
			  << ptr << " bytes\n");

		return result;
	}
	return NULL;
}
