////////////////////////////////////////////////////////////////////////////////
//
//	File: vcdutotvcdu.hpp
//
//	Provides: VCDUTotVCDU filter
//
//	Purpose: Convert a stream of CVCDU packets to VCDU packets
//
//	Copyright (c) 2002, 2003, 2004 EUMETSAT
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef vcdutotvcdu_defined
#define vcdutotvcdu_defined

#include "interface.hpp"
#include "textsummaryproducer.hpp"
#include "tvcduproducer.hpp"
#include "vcdutoccsds.hpp"
#include "config.hpp"

#define Config Config::tVCDU

class VCDUTotVCDU :
	virtual public Interface,
	public tVCDUProducer,
	public VCDUProducer
{
public:

	//
	// constructor
	//

	VCDUTotVCDU( VCDUProducer& _source,
				 bool _ignore_times)
		: source(_source),
		  packet_count(0),
		  ignore_times(_ignore_times)
		{
		}

	//
	// output_text_summary
	//

	virtual void output_text_summary( ostream& os = cout)
		{
			os << "VCDUTotVCDU report:\n"
			   << "	   " << packet_count << " packets processed\n"
			   << "	   Initial CCSDS time "
			   << tVCDU::time_to_string(initial_time) << endl
			   << "	   Initial tVCDU time "
			   << tVCDU::time_to_string(initial_offsetted_time) << endl
			   << "	   Final tVCDU time "
			   << tVCDU::time_to_string(current_time) << endl
			   << endl;
		}

	virtual VCDU* get_next_vcdu(void)
		{
			VCDU* vcdu = source.get_next_vcdu();
			if (!vcdu)
			{
				return NULL;
			}

			VCDU* store = new VCDU;
			memcpy(store, vcdu, sizeof(VCDU));
			packet_queue.push(store);

			return vcdu;
		}

	//
	// get_next_vcdu
	//

	virtual tVCDU* get_next_tvcdu(void)
		{
			// first time initialisation
			if (!packet.get())
			{
				packet.reset(new tVCDU);

				// check configuration

				if (Config::reed_solomon_status < 0 ||
					Config::reed_solomon_status > 255)
				{
					error << "Configuration parameter ReedSolomonStatus "
						  << "must be between 0 and 255 (found "
						  << Config::reed_solomon_status << ")\n";
					return NULL;
				}

				if (Config::utc_time_difference < 0)
				{
					error << "Configuration parameter UTCTimeDifference "
						  << "cannot be negative (found "
						  << Config::utc_time_difference << ")\n";
					return NULL;
				}

				if (Config::utc_offset_milliseconds < 0)
				{
					error << "Configuration parameter UTCOffsetMilliseconds "
						  << "cannot be negative (found "
						  << Config::utc_offset_milliseconds << ")\n";
					return NULL;
				}

				// find initial time value
				initial_time.microseconds = 0;
				initial_time.milliseconds = 0;
				initial_time.days = 0;

				if (!ignore_times)
				{
					VCDUToCCSDS* filter = new VCDUToCCSDS(*this);
					CCSDS* ccsds = filter->get_next_ccsds();
					if (ccsds)
					{
						initial_time.microseconds =
							ccsds->get_utc_microseconds();
						initial_time.milliseconds =
							ccsds->get_utc_milliseconds();
						initial_time.days =
							ccsds->get_utc_days();
					}
					delete filter;
				}

				// this is to catch bad input values
				initial_time.microseconds %= 1000;
				initial_time.milliseconds %= MILLISECONDS_PER_DAY;

				// offset initial time value
				initial_offsetted_time = initial_time;
				initial_offsetted_time.milliseconds +=
					Config::utc_offset_milliseconds;
				initial_offsetted_time.days +=
					static_cast<int>(initial_offsetted_time.milliseconds /
									 MILLISECONDS_PER_DAY);
				initial_offsetted_time.milliseconds %=
					MILLISECONDS_PER_DAY;

				// set current time
				current_time = initial_offsetted_time;
			}

			VCDU* vcdu;
			bool delete_vcdu;

			if (!packet_queue.empty())
			{
				vcdu = packet_queue.front();
				packet_queue.pop();
				delete_vcdu = true;
			}
			else
			{
				vcdu = source.get_next_vcdu();
				if (!vcdu)
				{
					return NULL;
				}
				delete_vcdu = false;
			}

			// copy over initial 6 bytes to output packet
			memcpy(packet->header,
				   vcdu,
				   6);

			// zero fill the insert zone
			memset(packet->insert_zone, 0, 2);

			// copy over the MPDU paylod
			packet->mpdu = vcdu->mpdu;

			// set the reed solomon status
			packet->reed_solomon_status =
				Config::reed_solomon_status;

			// set the time
			packet->utc = current_time;

			// offset the current time
			current_time.microseconds += Config::utc_time_difference;

			// roll over to milliseconds
			while(current_time.microseconds >= 1000)
			{
				current_time.microseconds -= 1000;
				current_time.milliseconds++;
			}

			// roll over to days
			while(current_time.milliseconds >= MILLISECONDS_PER_DAY)
			{
				current_time.milliseconds -= MILLISECONDS_PER_DAY;
				current_time.days--;
			}

			packet_count++;

			if (delete_vcdu)
			{
				delete vcdu;
			}

			return packet.get();
		}

private:

	VCDUProducer& source;

	int packet_count;
	UTCTime initial_time;
	UTCTime initial_offsetted_time;
	UTCTime current_time;

	bool ignore_times;

	auto_ptr<tVCDU> packet;
	queue<VCDU*> packet_queue;
};

#undef Config

#endif
