////////////////////////////////////////////////////////////////////////////////
//
//	File: ccsdspacketdisplayer.cpp
//
//	Provides: CCSDSPacketDisplayer filter
//
//	Purpose: Show contents of CCSDS packets as text
//
//	Copyright (c) 2002, 2003, 2004, 2007 Eumetsat
//
//	This file is part of the Metopizer.
//
////////////////////////////////////////////////////////////////////////////////

#include "common.hpp"
#include "config.hpp"
#include "mhstempconv.hpp"
#include "ccsdspacketdisplayer.hpp"

#define Config Config::CCSDS_Displayer

const long long OBT8_TO_PICOSECONDS = (1<<8) * static_cast<long long>(pow(10.0, 9.0));
const long long OBT8_TO_NANOSECONDS = (1<<8) * static_cast<long long>(pow(10.0, 6.0));
const unsigned int SECONDS_PER_ROLLOVER = 1<<24;

//
// constructor
//

CCSDSPacketDisplayer::CCSDSPacketDisplayer(
	CCSDSProducer& _source,
	const string& _config_file,
	const string& _iasi_config_file)
	: source(_source),
	  config_file(_config_file),
	  iasi_config_file(_iasi_config_file),
	  //	  no_body(_no_body),
	  //	  show_gaps(_show_gaps),
	  rollover(0),
	  anti_rollover(0),
	  packet_counter(0)
	  //	  show_seq_diffs(_show_seq_diffs),
	  //	  shift_gras_obt(_shift_gras_obt)
{
	if (!config_file.empty())
	{
		Config::file.read(config_file);
	}
}


//
// output_text_record
//

void CCSDSPacketDisplayer::output_text_record(ostream& os,
											  bool hex,
											  bool header_only,
											  bool show_gaps,
											  bool show_seq_gaps,
											  bool shift_gras_obt)
{
	int apid = 0;

	DEBUG("ccsdspacketdisplayer constructor\n");

	CCSDS* packet;
	while((packet=source.get_next_ccsds()))
	{

		os << "\n--------------------  Packet # " << packet_counter << "  ---------------------\n\n";

		packet_counter ++;

		output_header(packet, os, show_gaps, shift_gras_obt, show_seq_gaps);

		if (header_only)
		{
			cout << endl<< endl;
			continue;
		}

		if (hex)
		{
			if (APID_AMSUA_1==packet->get_apid() ||
				APID_AMSUA_2==packet->get_apid())
			{
				output_amsu_hex_dump(packet, os);
			}
			else
			{
				output_raw_hex_dump(packet, os);
			}
			continue;
		}

		apid = packet->get_apid();

		if ( (apid >= 192) && (apid <= 255) )
		{
			apid = CCSDS_ASCAT_NOM::APID;
		}

		if ( (apid==135) || (apid==140) || (apid==145) )
		{
			apid = CCSDS_IASI_Spec_PX1::APID;
		}

		switch(apid)
		{

		case CCSDS_SATELLITE::APID:

			output_sband_record(packet, os);
			break;

		case APID_AVHRR_3A:
		case APID_AVHRR_3B: // intentional drop through

			output_avhrr_record(packet, os);
			break;

		case APID_AMSUA_1:
		case APID_AMSUA_2: // intentional drop through
		{
			deque<byte> sci_data;
			extract_amsu_data(packet,sci_data);

			os << "\nDecoding " << sci_data.size()
			   << " bytes of AMSU application data\n";

			switch(sci_data.size())
			{
			case 1240:
			{
				AMSUA1MeasurementScienceData data;
				copy_n(sci_data.begin(), 1240, (byte*)(&data));
				output_amsu_a1_measure(os, data);
				break;
			}
			case 1120:
			{
				AMSUA1UniqueScienceData data;
				copy_n(sci_data.begin(), 1120, (byte*)(&data));
				output_amsu_a1_unique(os, data);
				break;
			}
			case 312:
			{
				AMSUA2MeasurementScienceData data;
				copy_n(sci_data.begin(), 312, (byte*)(&data));
				output_amsu_a2_measure(os, data);
				break;
			}
			case 288:
			{
				AMSUA2UniqueScienceData data;
				copy_n(sci_data.begin(), 288, (byte*)(&data));
				output_amsu_a2_unique(os, data);
				break;
			}

			default:
				error << "Cannot decode AMSU packet of "
					  << sci_data.size() << " science data bytes\n";
			}
			break;
		}

		case CCSDS_MHS::APID:
			output_mhs_record(os, packet);
			break;

		case CCSDS_HIRS::APID:
			output_hirs_record(os, packet);
			break;

		case CCSDS_IASI_Spec_PX1::APID:
			output_iasi_spec_record(os, packet);
			break;

		case CCSDS_IASI_Image::APID:
			output_iasi_image_record(os, packet);
			break;

		case CCSDS_IASI_Verif::APID:
			output_iasi_verif_record(os, packet);
			break;

		case CCSDS_IASI_Anc::APID:
			output_iasi_anc_record(os, packet);
			break;

		case CCSDS_GOME::APID:
			output_gome_record(os, packet);
			break;

		case APID_GRAS_MEASUREMENT_1:
		case APID_GRAS_MEASUREMENT_2:	 // intentional drop through
			output_gras_measurement_record(os, packet);
			break;

		case APID_GRAS_NAVIGATION_1:
			output_gras_navigation_nominal_record(os, packet);
			break;

		case APID_GRAS_NAVIGATION_2:
			output_gras_navigation_reduced_record(os, packet);
			break;

		case CCSDS_ASCAT_NOM::APID:
		{
			int payload_length = packet->get_payload_length();
			os << "\nDecoding " << payload_length
			   << " bytes of ASCAT ancillary and application"
			   << " data\n";

			switch(payload_length)
			{
			case 638:
			{
				output_ascat_nom_record(os, packet);
				break;
			}
			default:
			{
				output_ascat_raw_record(os, packet);
				break;
			}
			}
			break;

		}

		default:
			info << "Cannot decode packet contents (APID="
				 << packet->get_apid() << ")\n";
			break;
		}
	}
}


//
// output_header
//

void CCSDSPacketDisplayer::output_header(CCSDS* packet, ostream& os, bool show_gaps, bool shift_gras_obt, bool show_seq_diffs)
{
	EPSProducer* eps = source.as_eps_product();

	if (eps)
	{
		short_cds_time grh_start_time = eps->get_grh_start();
		short_cds_time grh_stop_time  = eps->get_grh_stop();
		os << "GRH Start Time " << grh_start_time << endl
		   << "		Stop Time " << grh_stop_time  << endl
		   << endl;

		// convert GRH start_time to UTC time
		UTCTime grh_start_time_utc;
		grh_start_time_utc.days			= grh_start_time.day;
		grh_start_time_utc.milliseconds = grh_start_time.millisec_day;
		grh_start_time_utc.microseconds = 0;
		//cout << "	  UTCTime converted GRH Start Time	  = " << grh_start_time_utc << endl << endl;

		//
		// VIADR-based OBTUTC time conversion - for comparison with GRH Start Time
		// ----------------------------------

		// The logic below resembles the OBT_to_UTC conversion function (from epsl0.hpp) using the following formula:

		//	 VIADR-based Packet_Time_UTC = VIADR_Clock_Step * ( ( Packet_Time_OBT / 256 ) - VIADR_OBT0 ) + VIADR_UTC0
		// i.e.
		//	 UTC_Time_VIADR = viadr_l0_obt2utc.clock_step * (( CCSDS_SBTTime / 256 ) - ( viadr_l0_obt2utc.obt_utc_fine ))
		//					+ ( viadr_l0_obt2utc.utc_obt_millisec * 10^3 ) + ( viadr_l0_obt2utc.utc_obt_microsec * 10^6 )

		// get VIADR OBT-UTC time parameters from the VIADR corresponding with grh_start_time
		int viadr_matching = 0;
		const VIADR_OBT2UTC* viadr_l0_obt2utc = eps->get_viadr_l0_obt2utc(grh_start_time, &viadr_matching);

		if ( ! viadr_l0_obt2utc )
		{
			cout << "WARNING: No VIADR found matching this packet time !" << endl << endl;
		}
		else
		{
			// convert VIADR UTC0 to UTC time
			UTCTime viadr_utc0;
			viadr_utc0.days			= viadr_l0_obt2utc->utc_obt_day;
			viadr_utc0.milliseconds = viadr_l0_obt2utc->utc_obt_millisec;
			viadr_utc0.microseconds = viadr_l0_obt2utc->utc_obt_microsec;

			//cout << "VIADR #" << viadr_matching << " details: "  << endl
			//	   << "	  VIADR_L0_OBT2UTC.UTC_0			  = " << viadr_utc0								 << endl
			//	   << "					  .UTC_0.Day		  = " << viadr_l0_obt2utc->utc_obt_day			 << endl
			//	   << "					  .UTC_0.MilliSec	  = " << viadr_l0_obt2utc->utc_obt_millisec		 << endl
			//	   << "					  .UTC_0.MicroSec	  = " << viadr_l0_obt2utc->utc_obt_microsec		 << endl
			//	   << "	  VIADR_L0_OBT2UTC.CCU_OBT_0		  = " << viadr_l0_obt2utc->ccu_obt_0			 << endl
			//	   << "					  .CCU_OBT_0.Coarse	  = " << (viadr_l0_obt2utc->ccu_obt_0>>8)		 << endl
			//	   << "					  .CCU_OBT_0.Fine	  = " << ((viadr_l0_obt2utc->ccu_obt_0&0xff)<<8) << endl
			//	   << "	  VIADR_L0_OBT2UTC.Clock_Step		  = " << viadr_l0_obt2utc->clock_step			 << endl << endl;

			// assign Clock_Step_count (10^-12 s/ 16-bit counts)
			long long clock_step_cnt = static_cast<long long>(viadr_l0_obt2utc->clock_step) ;
			//cout << "	  clock_step_cnt = " << clock_step_cnt << endl;

			// read in the CCSDS SBT (=OBT) Time
			OBTTime& obt_time = *packet->get_ancillary_data();
			//cout << "	  Packet OBTTime					  = " << obt_time << endl << endl;

			// correct IASI packets (sometimes VPx timestamp of previous packet must be used)
			if ( last_obt_time.get() && ( packet->get_apid() == APID_IASI_VPx ) )
			{
				obt_time = *(last_iasi_obt_time.get());
				//cout << "	  APID_IASI_VPx packet detected" << endl;
			}
			if ( packet->get_apid() != APID_IASI_VPx )
			{
				if (!last_iasi_obt_time.get())
				{
					last_iasi_obt_time.reset(new OBTTime(obt_time));
				}
				else
				{
					*last_iasi_obt_time = obt_time;
				}
			}

			// track rollover in OBT time

			// stop checking any further if the VIADR is too old (> 1/2 year)
			//			 (because the number of rollovers may be larger than 1)
			//if ( viadr_l0_obt2utc < ( obt time - 2^24 counts ) )	<== IMPLEMENT THIS LATER

			// SECONDS_PER_ROLLOVER = MAX_OBT_COARSE + 1 = (1<<24)

			// track rollover in OBT time with respect to VIADR OBT_0
			//		 current threshold = 6000000 sec = 70 days
			if ( (viadr_l0_obt2utc->ccu_obt_0>>8) > (SECONDS_PER_ROLLOVER - 6000000) &&
				 obt_time.obt_coarse < 1000000 )
			{
				//cout << "	  OBT rollover found wrt VIADR OBT_0 time " << endl << endl;
				rollover = true;
			}

			if ( last_obt_time.get() )
			{
				// track rollover in OBT time with respect to last packet
				if ( last_obt_time->obt_coarse > (SECONDS_PER_ROLLOVER - 100000) &&
					 obt_time.obt_coarse < 1000000 )
				{
					//cout << "	  OBT rollover found wrt OBT time of previous packet " << endl << endl;
					rollover = true;
				}
				else if ( *(last_obt_time.get()) > obt_time )
				{
					os << "WARNING: OBT Time of the packet goes back in time" << endl << endl;
				}
			}

			OBTTime obt_time_corr = obt_time;

			// correct GRAS packets (bug: bitshift 2^8)
			if ( packet->get_apid() == APID_GRAS_MEASUREMENT_1 || packet->get_apid() == APID_GRAS_MEASUREMENT_2 )
			{
				//cout << "	  GRAS_MEASURE packet detected" << endl << endl;
				obt_time_corr.obt_fine = ((obt_time_corr.obt_fine>>8)&0xff) | ((obt_time_corr.obt_coarse&0xff)<<8);
				obt_time_corr.obt_coarse = obt_time_corr.obt_coarse>>8;
			}

			// correct packet OBT time for rollover
			if ( rollover )
			{
				obt_time_corr.obt_coarse += SECONDS_PER_ROLLOVER;
			}
			//cout << "	  obt_time_corr = " << obt_time_corr << endl;

			// track anti-rollover in OBT time with respect to VIADR OBT_0
			//		 current threshold = 8000000 sec = 90 days (~= 2^23 s)
			if ( static_cast<int>(obt_time_corr.obt_coarse) - static_cast<int>(viadr_l0_obt2utc->ccu_obt_0>>8) > 8000000 )
			{
				if (viadr_l0_obt2utc->ccu_obt_0!=0)
				{
					// possibly hack. This fixes correlation computation w/ some HRPT data
					//cout << "	  OBT anti-rollover found wrt VIADR OBT_0 time " << endl << endl;
					anti_rollover = true;
				}
			}

			// convert VIADR OBT_0 time to OBT time as used here
			OBTTime ccu_obt_0 = make_obt_time(viadr_l0_obt2utc->ccu_obt_0>>8, (viadr_l0_obt2utc->ccu_obt_0&0xff)<<8);
			//cout << "	  ccu_obt_0 = " << ccu_obt_0 << endl;

			// calculate diff ( packet OBT time - VIADR OBT_0 time ) (in counts) using OBTTime::operator-
			int64 diff_obt_cnt = obt_time_corr - ccu_obt_0;	  // <== this term is allowed negative to cope with "future offsets" as well
			//cout << "	  diff_obt_cnt = " << diff_obt_cnt << endl;

			// correct packet OBT time for anti_rollover
			if ( anti_rollover )
			{
				diff_obt_cnt -= static_cast<int64>(SECONDS_PER_ROLLOVER)<<16;
			}

			// calculate OBT term (in us)
			long double obt_term = static_cast<long double>(clock_step_cnt) * static_cast<long double>(diff_obt_cnt) / OBT8_TO_NANOSECONDS;
			//cout << "	  obt_term = " << obt_term << " (" << static_cast<int64_be>(obt_term) << ")" << endl;

			// obtain VIADR-based UTC Time by adding OBT_Term (in ms) to UTC_0 using the UTCTime::operator+
			UTCTime utc_time_viadr = viadr_utc0;
			utc_time_viadr.mod_micro(static_cast<int64>(obt_term));

			os << "Computed VIADR-based UTC Time : "
				//	 << endl << "		 UTC Days				 : " << utc_time_viadr.days
				//	 << endl << "		 UTC Milliseconds		 : " << utc_time_viadr.milliseconds
				//	 << endl << "		 UTC Microseconds		 : " << utc_time_viadr.microseconds
				//	 << endl << "		 Time : "
			   << utc_time_viadr << endl << endl;

			// calculate Time Offset (in ms) using the UTCTime::operator-
			long long time_offset = grh_start_time_utc.diff_micro(utc_time_viadr);
			//cout << "	   time_offset = " << time_offset << endl << endl;

			// issue a WARNING if Time Offset > 1 ms
			if ( fabs(static_cast<double>(time_offset)) > 1000 )
			{
				os.setf(ios::fixed);
				os.setf(ios::showpoint);
				os << "WARNING: VIADR-based UTC Time (using VIADR# " << viadr_matching
				   << ") does not match the GRH start time; "
				   << "offset: " << setprecision(3) << (time_offset/1000.0) << " ms" << endl << endl;
				os.unsetf(ios::fixed);
				os.unsetf(ios::showpoint);
			}
		}
	}

	OBTTime& obt_time = *packet->get_ancillary_data();
	if (!last_obt_time.get())
	{
		last_obt_time.reset(new OBTTime(obt_time));

		if (show_gaps)
		{
			if (shift_gras_obt)
			{
				os << "OBT difference 0 / 0 / 0\n";
			}
			else
			{
				os << "OBT difference 0 / 0\n";
			}
		}
	}
	else
	{
		if (show_gaps)
		{
			if (shift_gras_obt && packet->get_apid()>=448 && packet->get_apid()<=511)
			{
				long long obt_diff = obt_time.subtract_shifted(*last_obt_time);
				os << "OBT difference " << (obt_diff>>24) << " / " << ((obt_diff>>8)&0xff) << " / " << (obt_diff&0xff) << endl;
			}
			else
			{
				long long obt_diff = obt_time-*last_obt_time;
				if (obt_diff < -OBT_DIFF_ROLLOVER_THRESHOLD)
				{
					obt_diff += static_cast<long long>(MAX_OBT_COARSE)*static_cast<long long>(MAX_OBT_FINE);
				}
				os << "OBT difference " << (obt_diff/65536) << " / " << (obt_diff%65536) << endl;
			}
		}
		*last_obt_time = obt_time;
	}

	if (show_gaps)
	{
		if (packet->has_secondary_header())
		{
			UTCTime& utc_time = *packet->get_secondary_header();
			if (!last_utc_time.get())
			{
				last_utc_time.reset(new UTCTime(utc_time));
				os << "UTC difference 0\n";
			}
			else
			{
				os << "UTC difference " << utc_time-*last_utc_time << " ms\n";
				*last_utc_time = utc_time;
			}
		}

		if (eps)
		{
			short_cds_time grh_time = eps->get_grh_start();
			if (!last_grh_time.get())
			{
				last_grh_time.reset(new short_cds_time(grh_time));
				os << "GRH difference 0\n";
			}
			else
			{
				os << "GRH difference " << grh_time-*last_grh_time << " ms\n";
				*last_grh_time = grh_time;
			}
		}

		os << endl;
	}

	if (show_seq_diffs)
	{
		if (!last_seq_count.get())
		{
			os << "Sequence count diff 0\n";
			last_seq_count.reset(new int(packet->get_sequence_count()));
		}
		else
		{
			os << "Sequence count diff "
			   << (static_cast<int>(packet->get_sequence_count())-*last_seq_count)
			   << endl;
			*last_seq_count = packet->get_sequence_count();
		}
		os << endl;
	}

	os << "Packet Primary Header\n"
	   << "	   Packet Identification\n"
	   << "		   Version Number			   : "
	   << packet->get_version_number() << endl
	   << "		   Type						   : "
	   << packet->get_type() << endl
	   << "		   Secondary Header			   : "
	   << packet->get_secondary_header_flag() << endl
	   << "			 Application ID			   : "
	   << packet->get_apid()
	   << " (" << (CCSDSAPIDFilter::apid_to_string(
					   packet->get_apid())) << ")\n"
	   << "	   Packet Sequence Control\n"
	   << "		   Sequence flags			   : "
	   << packet->get_sequence_flags() << endl
	   << "		   Sequence count			   : "
	   << packet->get_sequence_count() << endl
	   << "	   Packet length				   : "
	   << packet->get_packet_length()
	   << " (actual packet size " << packet->get_length() << ")\n\n"
	   << "Packet Data Field\n";
	// reduced MHS packets have no UTC time
	if (packet->has_secondary_header())
	{
		os << "	   Secondary Header\n"
		   << "		   UTC Days				   : "
		   << (packet->get_utc_days()) << endl
		   << "		   UTC Milliseconds		   : "
		   << (packet->get_utc_milliseconds()) << endl
		   << "		   UTC Microseconds		   : "
		   << (packet->get_utc_microseconds()) << endl
		   << "		   Time : "
		   << (*packet->get_secondary_header()) << endl;
	}

	unsigned int real_pec = packet->get_error_control();
	unsigned int calc_pec = packet->calculate_error_control();

	os << "	   Source Data\n"
	   << "		   User Data\n"
	   << "			   Ancillary Data\n";

	if (shift_gras_obt && packet->get_apid()>=448 && packet->get_apid()<=511)
	{
		os << "				   OBT coarse time : " << packet->get_shifted_obt_coarse() << endl
		   << "				   OBT fine time   : " << packet->get_shifted_obt_fine() << endl
		   << "				   OBT very fine time : " << packet->get_shifted_obt_veryfine() << endl;
	}
	else
	{
		os << "				   OBT coarse time : " << packet->get_obt_coarse() << endl
		   << "				   OBT fine time   : " << packet->get_obt_fine() << endl;
	}

	os << "			   Application Data\n"
	   << "				   <see below>\n"
	   << "		   Error Control : " << real_pec << " (";
	if (real_pec!=calc_pec)
	{
		os << "WARNING ";
	}
	os << "calculated value " << calc_pec << ")\n\n";
}


//
// output_raw_hex_dump
//

void CCSDSPacketDisplayer::output_raw_hex_dump(CCSDS* packet, textoutstream_type& os)
{
	os << "\nApplication Data Hex Dump:\n";

	for(int i=0; i<packet->get_payload_length(); i++)
	{
		if (i%16==0)
		{
			printf("%5d:  ",i);
		}
		unsigned int b = static_cast<unsigned int>(
			packet->get_payload()[i]);
		char buffer[20];
		sprintf(buffer, "%02x ", b);
		os << buffer;
		if (i%16==15)
		{
			os << endl;
		}
	}
	os << endl;
}


//
// output_sband_record
//

void CCSDSPacketDisplayer::output_sband_record(CCSDS* packet, textoutstream_type& os)
{
	CCSDS_SATELLITE* sband_frame = (CCSDS_SATELLITE*)packet->get_data();
	sband_frame->create_sband_output();
}


//
// output_amsu_hex_dump
//

void CCSDSPacketDisplayer::output_amsu_hex_dump(CCSDS* packet, textoutstream_type& os)
{
	os << "\nAMSU-A Raw Hex Dump:\n";

	for(int i=0; i<packet->get_payload_length(); i+=2)
	{
		if (i%16==0)
		{
			char buffer[20];
			sprintf(buffer, "%4d:  ",i);
			os << buffer;
		}
		unsigned int b = static_cast<unsigned int>
			(packet->get_payload()[i]);
		unsigned int c = static_cast<unsigned int>
			(packet->get_payload()[i+1]);
		if (b==0 && c==1)
		{
			// its filler
			os << ".. .. ";
		}
		else
		{
			char buffer[20];
			sprintf(buffer, "%02x %02x ", b, c);
			os << buffer;
		}
		if (i%16==14)
		{
			os << endl;
		}
	}

	os << "\n\n\nAMSU-A Science Data (filler removed) :\n";
	int word_count = 0;
	for(int i=0; i<packet->get_payload_length(); i+=2)
	{
		unsigned int b = static_cast<unsigned int>
			(packet->get_payload()[i]);
		unsigned int c = static_cast<unsigned int>
			(packet->get_payload()[i+1]);

		if (!(b==0 && c==1))
		{
			char buffer[20];
			if (word_count%8 == 0)
			{
				sprintf(buffer, "%4d:  ",word_count*2);
				os << buffer;
			}
			sprintf(buffer, "%02x %02x ", b, c);
			os << buffer;

			word_count++;

			if (word_count%8 == 0)
			{
				os << endl;
			}
		}
	}
	os << endl;
}


//
// read_avhrr_bit
//

int CCSDSPacketDisplayer::read_avhrr_bit(byte*& payload_byte, int& payload_bit)
{
	int result;
	if ( ((*payload_byte)&(1<<payload_bit)) > 0)
	{
		result = 1;
	}
	else
	{
		result = 0;
	}

	if (payload_bit > 0)
	{
		payload_bit--;
	}
	else
	{
		payload_bit = 7;
		payload_byte++;
	}
	return result;
}


//
// read_avhrr_sample
//

CCSDSPacketDisplayer::AVHRR_Sample CCSDSPacketDisplayer::read_avhrr_sample(byte*& payload_byte, int& payload_bit)
{
	AVHRR_Sample sample;
	memset(&sample, 0, sizeof(sample));

	for(int channel = 0; channel<5; channel++)
	{
		for(int bit = 0; bit<10; bit++)
		{
			sample.channels[channel] <<= 1;
			sample.channels[channel] +=
				read_avhrr_bit(payload_byte, payload_bit);
		}
	}

	return sample;
}


//
// output_avhrr_record
//

void CCSDSPacketDisplayer::output_avhrr_record(CCSDS* packet, textoutstream_type& os)
{
	CCSDS_AVHRR* avhrr = (CCSDS_AVHRR*)packet->get_data();
	byte* payload_byte = avhrr->payload.data;
	int payload_bit = 7;
//	NOAA_L1b_LAC_Data::Sample sample;

	AVHRR_Sample space_view[10];
	for(int i=0; i<10; i++)
	{
		space_view[i] = read_avhrr_sample(payload_byte, payload_bit);
	}

	AVHRR_Sample ramp_calibration = read_avhrr_sample(payload_byte, payload_bit);

	AVHRR_Sample earth_view[2048];
	for(int i=0; i<2048; i++)
	{
		earth_view[i] = read_avhrr_sample(payload_byte, payload_bit);

	}

	AVHRR_Sample target_temp = read_avhrr_sample(payload_byte, payload_bit);

	AVHRR_Sample patch_temperature = read_avhrr_sample(payload_byte, payload_bit);

	AVHRR_Sample back_scan[10];
	for(int i=0; i<10; i++)
	{
		back_scan[i] = read_avhrr_sample(payload_byte, payload_bit);
	}

	// Telemetry

	os << "\nTelemetry\n---------\n\n"
	   << "Ramp calibration ch#1 : "
	   << ramp_calibration.channels[0] << endl
	   << "					ch#2 : "
	   << ramp_calibration.channels[1] << endl
	   << "					ch#3 : "
	   << ramp_calibration.channels[2] << endl
	   << "					ch#4 : "
	   << ramp_calibration.channels[3] << endl
	   << "					ch#5 : "
	   << ramp_calibration.channels[4] << endl << endl
	   << "				  PRT #1 : " << target_temp.channels[2] << endl
	   << "					  #2 : " << target_temp.channels[3] << endl
	   << "					  #3 : " << target_temp.channels[4]
	   << endl << endl
	   << "	   Patch Temperature : " << patch_temperature.channels[2]
	   << endl
	   << endl;

	// Back Scan

	os << "Back Scan\n----------\n\n"
	   << "Sample ch#3 ch#4 ch#5\n";

	for(int i=0; i<10; i++)
	{
		os << "	   " << setiosflags(ios::fixed) << setw(1) << i << "  "
		   << setiosflags(ios::fixed) << setw(4) << setfill('0')
		   << back_scan[i].channels[2] << " "
		   << setiosflags(ios::fixed) << setw(4) << setfill('0')
		   << back_scan[i].channels[3] << " "
		   << setiosflags(ios::fixed) << setw(4) << setfill('0')
		   << back_scan[i].channels[4] << endl;
	}
	os << endl;

	// Space View

	os << "Space View\n----------\n\n"
	   << "Sample ch#1 ch#2 ch#3 ch#4 ch#5\n";

	for(int i=0; i<10; i++)
	{
		os << "		" << setiosflags(ios::fixed) << setw(1) << i;
		for(int j=0; j<5; j++)
		{
			os << " " << setiosflags(ios::fixed) << setw(4)
			   << setfill('0') << space_view[i].channels[j];
		}
		os << endl;
	}
	os << endl;

	// Earth View

	os << "Earth view\n----------\n\n"
	   << "Sample ch#1 ch#2 ch#3 ch#4 ch#5	   "
	   << "Sample ch#1 ch#2 ch#3 ch#4 ch#5\n";

	// set output stream to force 4 char outputs, right aligned,
	// zero padded
	os << setiosflags(ios::fixed) << setw(4);

	// write earth views
	for(int i=0; i<2048; i+=2)
	{
		os << setiosflags(ios::fixed) << setfill(' ') << setw(4)
		   << i << "  ";

		for(int j=0; j<5; j++)
		{
			os << " " << setiosflags(ios::fixed) << setfill('0')
			   << setw(4) << earth_view[i].channels[j];
		}

		os << "		" << setiosflags(ios::fixed) << setfill(' ')
		   << setw(4) << (i+1) << "	 ";

		for(int j=0; j<5; j++)
		{
			os << " " << setiosflags(ios::fixed) << setfill('0')
			   << setw(4) << earth_view[i+1].channels[j];
		}

		os << endl;

		// restore stream settings
		os << resetiosflags(ios::fixed);
	}
}


//
// extract_amsu_data
//

void CCSDSPacketDisplayer::extract_amsu_data(CCSDS* packet, deque<byte>& sci_data)
{
	for(int i=0; i<packet->get_payload_length(); i+=2)
	{
		byte a = packet->get_payload()[i];
		byte b = packet->get_payload()[i+1];

		if (a!=0 || b!=1)
		{
			// not filler
			sci_data.push_back(a);
			sci_data.push_back(b);
		}
	}
}


//
// output_amsu_a1_measure
//

void CCSDSPacketDisplayer::output_amsu_a1_measure(ostream& os,
												  AMSUA1MeasurementScienceData& data)
{
	os << "\nAMSU-A1 Measurement Data\n------------------------\n";

	// AMSU-A1 Digital Telemetry

	os << "\nAMSU-A1 Digital A Telemetry\n"
	   <<	"---------------------------\n";

	AMSUABase::output_digital_telemetry(os,
										data.sync_sequence,
										data.serial_number,
										data.digital_housekeeping);

	// AMSU-A1 Scene Data

	os << "AMSU-A1 Scene Data\n------------------\n";
	os << "Position 1st reading 2nd reading Counts\n"
	   << "			1	  2		1	  2		3	  4		5	  6		7\n"
	   << "			  8		9	  10	11	  12	13	  14	15\n\n";

//		  12345 12345 12345 12345 12345 12345 12345 12345 12345

	for(int i=0; i<30; i++)
	{
		os << "	  " << setiosflags(ios::fixed) << setw(2)
		   << setfill(' ') << (i+1) << "	"
		   << setiosflags(ios::fixed) << setw(5) << setfill('0')
		   << data.scene[i].first_reading[0] << " "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].first_reading[1] << " "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].second_reading[0] << " "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].second_reading[1] << " ";
		for(int j=0; j<13; j++)
		{
			os << setiosflags(ios::fixed) << setw(5)
			   << data.scene[i].counts[j] << " ";
			if (j==4)
			{
				os << "\n			";
			}
		}
		os << endl;
	}
	os << endl;

	// AMSU-A1 Cold Calibration

	os << "AMSU-A1 Cold Calibration\n------------------------\n"
	   << "Reflector 1 first reading  : "
	   << data.cold_calibration.first_reading[0] << endl
	   << "Reflector 2 first reading  : "
	   << data.cold_calibration.first_reading[1] << endl
	   << "Reflector 1 second reading : "
	   << data.cold_calibration.second_reading[0] << endl
	   << "Reflector 2 second reading : "
	   << data.cold_calibration.second_reading[1] << endl
	   << "Counts	1	  2\n";
	for(int i=0; i<13; i++)
	{
		os << "	   " << ntofs(i+3,2,' ') << " : "
		   << ntofs(data.cold_calibration.counts_1[i]) << " "
		   << ntofs(data.cold_calibration.counts_2[i]) << endl;
	}
	os << endl;

	// AMSU-A1 Temperature sensors
	os << "AMSU-A1 Temperature sensors\n---------------------------\n";
	for(int i=0; i<46; i++)
	{
		os << ntofs(i+1,2,' ') << ": "
		   << ntofs(data.temperature_sensors[i]) << "  ";
		if (i%6==5)
		{
			os << endl;
		}
	}
	os << endl << endl;

	// AMSU-A1 Warm Calibration

	os << "AMSU-A1 Warm Calibration\n------------------------\n"
	   << "Reflector 1 first reading  : "
	   << data.warm_calibration.first_reading[0] << endl
	   << "Reflector 2 first reading  : "
	   << data.warm_calibration.first_reading[1] << endl
	   << "Reflector 1 second reading : "
	   << data.warm_calibration.second_reading[0] << endl
	   << "Reflector 2 second reading : "
	   << data.warm_calibration.second_reading[1] << endl
	   << "Counts	1	  2\n";
	for(int i=0; i<13; i++)
	{
		os << "	   " << ntofs(i+3,2,' ') << " : "
		   << ntofs(data.warm_calibration.counts_1[i]) << " "
		   << ntofs(data.warm_calibration.counts_2[i]) << endl;
	}
	os << endl;
}


//
// output_amsu_a1_unique
//

void CCSDSPacketDisplayer::output_amsu_a1_unique(textoutstream_type& os,
												 AMSUA1UniqueScienceData& data)
{
	os << "\nAMSU-A1 Unique Data\n------------------------\n";

	// AMSU-A1 Digital Telemetry

	os << "\nAMSU-A1 Digital A Telemetry\n---------------------------\n";

	AMSUABase::output_digital_telemetry(os,
										data.sync_sequence,
										data.serial_number,
										data.digital_housekeeping);

	// AMSU-A1 Scene Data

	os << "AMSU-A1 Scene Data\n------------------\n";
	os << "Position 1st reading 2nd reading Counts\n"
	   << "			1	  2		1	  2		3	  4		5	  6		7\n"
	   << "			  8		9	  10	11	  12	13	  14	15\n\n";

//		   12345 12345 12345 12345 12345 12345 12345 12345 12345

	for(int i=0; i<30; i++)
	{
		os << "	  " << setiosflags(ios::fixed) << setw(2)
		   << setfill(' ') << (i+1) << "	"
		   << setiosflags(ios::fixed) << setw(5) << setfill('0')
		   << data.scene[i].first_reading[0] << " "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].first_reading[1] << " "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].second_reading[0] << " "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].second_reading[1] << " ";
		for(int j=0; j<13; j++)
		{
			os << setiosflags(ios::fixed) << setw(5)
			   << data.scene[i].counts[j] << " ";
			if (j==4)
			{
				os << "\n			";
			}
		}
		os << endl;
	}
	os << endl;

	// AMSU-A1 Temperature sensors
	os << "AMSU-A1 Temperature sensors\n---------------------------\n";
	for(int i=0; i<46; i++)
	{
		os << ntofs(i+1,2,' ') << ": "
		   << ntofs(data.temperature_sensors[i]) << "  ";
		if (i%6==5)
		{
			os << endl;
		}
	}
	os << endl;
}


//
// output_amsu_a2_measure
//

void CCSDSPacketDisplayer::output_amsu_a2_measure(ostream& os,
												  AMSUA2MeasurementScienceData& data)
{
	os << "\nAMSU-A2 Measurement Data\n------------------------\n";

	// AMSU-A2 Digital Telemetry

	os << "\nAMSU-A2 Digital A Telemetry\n---------------------------\n";

	AMSUABase::output_digital_telemetry(os,
										data.sync_sequence,
										data.serial_number,
										data.digital_housekeeping);


	// AMSU-A2 Scene Data

	os << "AMSU-A2 Scene Data\n------------------\n"
	   << "Position	 First	  Second   Counts\n"
	   << "			 reading  reading  Ch#1		Ch#2\n\n";

	for(int i=0; i<30; i++)
	{
		os << "	  " << setiosflags(ios::fixed) << setw(2)
		   << setfill(' ') << (i+1) << "	 "
		   << setiosflags(ios::fixed) << setw(5) << setfill('0')
		   << data.scene[i].first_reading << "	  "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].second_reading << "	   "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].counts[0] << "	  "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].counts[1] << endl;
	}
	os << endl;

	// AMSU-A2 Cold Calibration

	os << "AMSU-A2 Cold Calibration\n------------------------\n"
	   << "Reflector first reading	: "
	   << data.cold_calibration.first_reading << endl
	   << "Reflector second reading : "
	   << data.cold_calibration.second_reading << endl
	   << "			 Channels\n"
	   << "			 1	   2\n"
	   << "Counts 1: " << ntofs(data.cold_calibration.counts_1[0])
	   << " " << ntofs(data.cold_calibration.counts_2[0]) << endl
	   << "		  2: " << ntofs(data.cold_calibration.counts_1[1])
	   << " " << ntofs(data.cold_calibration.counts_2[1]) << endl
	   << endl;

	// AMSU-A2 Temperature sensors
	os << "AMSU-A2 Temperature sensors\n---------------------------\n";
	for(int i=0; i<20; i++)
	{
		os << ntofs(i+1,2,' ') << ": "
		   << ntofs(data.temperature_sensors[i]) << "  ";
		if (i%6==5)
		{
			os << endl;
		}
	}
	os << endl << endl;

	// AMSU-A2 Warm Calibration

	os << "AMSU-A2 Warm Calibration\n------------------------\n"
	   << "Reflector first reading	: "
	   << data.warm_calibration.first_reading << endl
	   << "Reflector second reading : "
	   << data.warm_calibration.second_reading << endl
	   << "			 Channels\n"
	   << "			 1	   2\n"
	   << "Counts 1: " << ntofs(data.warm_calibration.counts_1[0])
	   << " " << ntofs(data.warm_calibration.counts_2[0]) << endl
	   << "		  2: " << ntofs(data.warm_calibration.counts_1[1])
	   << " " << ntofs(data.warm_calibration.counts_2[1]) << endl
	   << endl;
}


//
// output_amsu_a2_measure
//

void CCSDSPacketDisplayer::output_amsu_a2_unique(ostream& os,
												 AMSUA2UniqueScienceData& data)
{
	os << "\nAMSU-A2 Unique Data\n------------------------\n";

	// AMSU-A2 Digital Telemetry

	os << "\nAMSU-A2 Digital A Telemetry\n---------------------------\n";

	AMSUABase::output_digital_telemetry(os,
										data.sync_sequence,
										data.serial_number,
										data.digital_housekeeping);

	// AMSU-A2 Scene Data

	os << "AMSU-A2 Scene Data\n------------------\n"
	   << "Position	 First	  Second   Counts\n"
	   << "			 reading  reading  Ch#1		Ch#2\n\n";

	for(int i=0; i<30; i++)
	{
		os << "	  " << setiosflags(ios::fixed) << setw(2)
		   << setfill(' ') << (i+1) << "	 "
		   << setiosflags(ios::fixed) << setw(5) << setfill('0')
		   << data.scene[i].first_reading << "	  "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].second_reading << "	   "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].counts[0] << "	  "
		   << setiosflags(ios::fixed) << setw(5)
		   << data.scene[i].counts[1] << endl;
	}
	os << endl;

	// AMSU-A2 Temperature sensors
	os << "AMSU-A2 Temperature sensors\n---------------------------\n";
	for(int i=0; i<20; i++)
	{
		os << ntofs(i+1,2,' ') << ": "
		   << ntofs(data.temperature_sensors[i]) << "  ";
		if (i%6==5)
		{
			os << endl;
		}
	}
	os << endl;
}


//
// display_mhs_samples
//

void CCSDSPacketDisplayer::display_mhs_samples(ostream& os,
											   MHS_Sample* samples,
											   int count)
{
	os << "Sample no.	Shaft pos Ch#16 Ch#17 Ch#18 Ch#19 Ch#20\n";
	for(int i=0; i<count; i++)
	{
		os << "		" << ntofs(i+1,2,' ') << "		"
		   << ntofs(samples[i].shaft_pos) << "	   ";
		for(int j=0; j<5; j++)
		{
			os << ntofs(samples[i].channels[j]) << " ";
		}
		os << endl;
	}
}


//
// output_mhs_record
//

void CCSDSPacketDisplayer::output_mhs_record(ostream& os,
											 CCSDS* packet)
{
	byte* app = packet->get_data();

	if (packet->get_apid()==APID_MHS &&
		packet->get_length()==REDUCED_MHS_LENGTH)
	{
		os << "\nDisplaying reduced MHS packet\n";
		app -= 8;
	}

//		MHSPayload& mhs = ((CCSDS_MHS*)packet->get_data())->payload;

	MHSPayload& mhs = ((CCSDS_MHS*)app)->payload;

	// mode

	os << "\nMode\n----\n\n"
	   << "Mode code		   : ";
	switch ((mhs.mode>>4)&15)
	{
	case 0 : os << "0 (Power on)\n"; break;
	case 1 : os << "1 (Warm up)\n"; break;
	case 2 : os << "2 (Standby)\n"; break;
	case 3 : os << "3 (Scan)\n"; break;
	case 4 : os << "4 (Fixed View)\n"; break;
	case 5 : os << "5 (Self test)\n"; break;
	case 6 : os << "6 (Safeing)\n"; break;
	case 7 : os << "7 (Fault)\n"; break;
	case 15 : os << "15 (Memory data packet ID)\n"; break;
	default:
		os << ((mhs.mode>>4)&15) << "(unused)\n"; break;
	}

	os << "PIE-ID			   : " << ((mhs.mode>>3)&1) << endl
	   << "Subcommutation code : " << (mhs.mode&7) << endl
	   << endl;

	// telecommand ack and fault code

	os << "Telecommand Acknowledgement and Fault Codes\n"
	   << "-------------------------------------------\n\n"
	   << "TC Clean False/True				   : "
	   << ((mhs.telecommand[0]>>7)&1) << endl
	   << "TC Conforms False/True			   : "
	   << ((mhs.telecommand[0]>>6)&1) << endl
	   << "TC Recognised False/True			   : "
	   << ((mhs.telecommand[0]>>5)&1) << endl
	   << "TC Legal False/True				   : "
	   << ((mhs.telecommand[0]>>4)&1) << endl
	   << "FDM Motor Current Trip False/True   : "
	   << ((mhs.telecommand[0]>>3)&1) << endl
	   << "TC Application ID				   : "
	   << ((mhs.telecommand[0]&7)<<8) + (mhs.telecommand[1]&255)
	   << endl
	   << "TC Packet Sequence				   : "
	   << ((mhs.telecommand[2]&255)<<8) + ((mhs.telecommand[3]>>2)&63)
	   << endl
	   << "TC Received Count				   : "
	   << (mhs.telecommand[3]&3) << endl
	   << "Monitor Current Fault False/True	   : "
	   << ((mhs.telecommand[4]>>7)&1) << endl
	   << "Thermistor Monitor Fault False/True : "
	   << ((mhs.telecommand[4]>>6)&1) << endl
	   << "Switch Fault False/True			   : "
	   << ((mhs.telecommand[4]>>5)&1) << endl
	   << "Processor Fault False/True		   : "
	   << ((mhs.telecommand[4]>>4)&1) << endl
	   << "RDM Motor Current Trip False/True   : "
	   << ((mhs.telecommand[4]>>3)&1) << endl
	   << "DC Offset Error False/True		   : "
	   << ((mhs.telecommand[4]>>2)&1) << endl
	   << "Scan Control Error False/True	   : "
	   << ((mhs.telecommand[4]>>1)&1) << endl
	   << "REF CK Error False/True			   : "
	   << (mhs.telecommand[4]&1) << endl
	   << endl;

	// Switch Status

	os << "Switch Status\n--------------\n\n"
	   << "Receiver Channel H1 Off/On					: "
	   << ((mhs.switch_status[0]>>0)&1) << endl
	   << "Receiver Channel H1 Local Oscillator A/B		: "
	   << ((mhs.switch_status[0]>>1)&1) << endl
	   << "Receiver Channel H2 Off/On					: "
	   << ((mhs.switch_status[0]>>2)&1) << endl
	   << "Receiver Channel H2 Local Oscillator A/B		: "
	   << ((mhs.switch_status[0]>>3)&1) << endl
	   << "Receiver Channels H3/H4 Frontend Off/On		: "
	   << ((mhs.switch_status[0]>>4)&1) << endl
	   << "Receiver Channels H3/H4 Local Oscillator A/B : "
	   << ((mhs.switch_status[0]>>5)&1) << endl
	   << "Receiver Channel H3 Backend Off/On			: "
	   << ((mhs.switch_status[0]>>6)&1) << endl
	   << "Receiver Channel H4 Backend Off/On			: "
	   << ((mhs.switch_status[0]>>7)&1) << endl
	   << "Receiver Channel H5 Off/On					: "
	   << ((mhs.switch_status[1]>>0)&1) << endl
	   << "Receiver Channel H5 Local Oscillator A/B		: "
	   << ((mhs.switch_status[1]>>1)&1) << endl
	   << "Rx CV Off/On									: "
	   << ((mhs.switch_status[1]>>2)&1) << endl
	   << "Receiver Operational Heaters Off/On			: "
	   << ((mhs.switch_status[1]>>3)&1) << endl
	   << "Scan Mechanism Operational Heaters Off/On	: "
	   << ((mhs.switch_status[1]>>4)&1) << endl
	   << "Auxiliary Operational Heaters Off/On			: "
	   << ((mhs.switch_status[1]>>5)&1) << endl
	   << "Signal Processing Electronics Off/On			: "
	   << ((mhs.switch_status[1]>>6)&1) << endl
	   << "PROM Failure False/True						: "
	   << ((mhs.switch_status[1]>>7)&1) << endl
	   << "RDM Zero Position Sensors A/B				: "
	   << ((mhs.switch_status[2]>>0)&1) << endl
	   << "FDM Zero Position Sensors A/B				: "
	   << ((mhs.switch_status[2]>>1)&1) << endl
	   << "RDM Motor Sensors A/B						: "
	   << ((mhs.switch_status[2]>>2)&1) << endl
	   << "FDM Motor Sensors A/B						: "
	   << ((mhs.switch_status[2]>>3)&1) << endl
	   << "RDM Motor Supply Off/On						: "
	   << ((mhs.switch_status[2]>>4)&1) << endl
	   << "FDM Motor Supply Off/On						: "
	   << ((mhs.switch_status[2]>>5)&1) << endl
	   << "RDM Motor Current Trip Enabled/Disabled		: "
	   << ((mhs.switch_status[2]>>6)&1) << endl
	   << "FDM Motor Current Trip Enabled/Disabled		: "
	   << ((mhs.switch_status[2]>>7)&1) << endl
	   << endl;

	// temperature data

	os << "Temperature Data\n----------------\n\n"
	   << "Sensor	Counts	 Temperature\n";
	for(int i=0; i<24; i++)
	{
		os << "	 " << ntofs(i+1,2,' ') << "		"
		   << ntofs(mhs.temperature[i]) << "	";
		double temp = mhs_temp_conv(mhs.temperature[i]);
		if (temp < 100.0)
		{
			os << dto2dp(temp);
		}
		else
		{
			os << ">" << dto2dp(temp);
		}
		os << "C";

		if (i==12 || i==13)
		{
			os << "	   (see note 1)";
		}

		os << endl;
	}

	os << endl << endl;

	// current consumption data

	os << "Raw Current Consumption Data\n"
	   << "----------------------------\n\n"
	   << "+5v Secondary Current : "
	   << ntofs(mhs.current[0], 3, ' ') << "  ("
	   << static_cast<int>(Config::v5_secondary_current_k * mhs.current[0] * 1000)
	   << "mA)\n"
	   << "+8v Secondary Current : "
	   << ntofs(mhs.current[1], 3, ' ') << "  ("
	   << static_cast<int>((Config::v8_secondary_current_k1 * mhs.current[1] * 1000)
						   + Config::v8_secondary_current_k2) << "mA)\n"
	   << "+15v Receiver Current : "
	   << ntofs(mhs.current[2], 3, ' ') << "  ("
	   << static_cast<int>((Config::v15_receiver_current_k1 * mhs.current[2] * 1000) +
						   Config::v15_receiver_current_k2) << "mA)\n"
	   << "-15v Receiver Current : "
	   << ntofs(mhs.current[3], 3, ' ') << "  ("
	   << static_cast<int>((Config::nv15_receiver_current_k1 * mhs.current[3] * 1000) +
						   Config::nv15_receiver_current_k2) << "mA)\n"
	   << "RDM Motor Current	 : "
	   << ntofs(mhs.current[4], 3, ' ') << "  ("
	   << static_cast<int>(Config::rdm_motor_current_k * mhs.current[4] * 1000) << "mA)\n"
	   << "FDM Motor Current	 : "
	   << ntofs(mhs.current[5], 3, ' ') << "  ("
	   << static_cast<int>(Config::fdm_motor_current_k * mhs.current[5] * 1000) << "mA)\n"
	   << endl;

	// status word

	os << "Status Word\n-----------\n\n"
	   << "DC Offset Valid False/True	 : "
	   << ((mhs.status_word>>7)&1) << endl
	   << "Scan Control Valid False/True : "
	   << ((mhs.status_word>>6)&1) << endl
	   << "Profile						 : "
	   << ((mhs.status_word>>4)&3) << endl
	   << endl;

	// signal processing status

	os << "Signal Processing Status\n------------------------\n\n"
	   << "Channel H1 DC Offset : "
	   << static_cast<int>(mhs.signal_status[0]) << endl
	   << "Channel H2 DC Offset : "
	   << static_cast<int>(mhs.signal_status[1]) << endl
	   << "Channel H3 DC Offset : "
	   << static_cast<int>(mhs.signal_status[2]) << endl
	   << "Channel H4 DC Offset : "
	   << static_cast<int>(mhs.signal_status[3]) << endl
	   << "Channel H5 DC Offset : "
	   << static_cast<int>(mhs.signal_status[4]) << endl
	   << "H1 Valid False/True	: "
	   << ((mhs.signal_status[5]>>7)&1) << endl
	   << "H2 Valid False/True	: "
	   << ((mhs.signal_status[5]>>6)&1) << endl
	   << "H3 Valid False/True	: "
	   << ((mhs.signal_status[5]>>5)&1) << endl
	   << "H4 Valid False/True	: "
	   << ((mhs.signal_status[5]>>4)&1) << endl
	   << "H5 Valid False/True	: "
	   << ((mhs.signal_status[5]>>3)&1) << endl
	   << "SPE Mux Code			: "
	   << (mhs.signal_status[5]&7) << endl
	   << "H1 Gain				: "
	   << ((mhs.signal_status[6]>>5)&7) << endl
	   << "H2 Gain				: "
	   << ((mhs.signal_status[6]>>2)&7) << endl
	   << "H3 Gain				: "
	   << ((mhs.signal_status[7]>>5)&7) << endl
	   << "H4 Gain				: "
	   << ((mhs.signal_status[7]>>2)&7) << endl
	   << "H5 Gain				: "
	   << ((mhs.signal_status[8]>>5)&7) << endl
	   << endl;

	// pixel data

	os << "Sensor Data\n-----------\n\n";
	display_mhs_samples(os, mhs.earth_view, 90);
	os << endl;

	// space view

	os << "Space View\n----------\n\n";
	display_mhs_samples(os, mhs.space_view, 4);
	os << endl;

	// calibration target view

	os << "Target View\n-----------\n\n";
	display_mhs_samples(os, mhs.obct_view,4);
	os << endl;

	// OBCT sensors

	os << "OBCT Sensors\n------------\n\n"
	   << "Target Temperature PRT#1 : "
	   << ntofs(mhs.prt[0]&4095, 5, ' ') << "  ("
	   << dto2dp(obct_to_temp(mhs.prt[0]&4095)) << "C)\n"
	   << "Target Temperature PRT#2 : "
	   << ntofs(mhs.prt[1]&4095, 5, ' ') << "  ("
	   << dto2dp(obct_to_temp(mhs.prt[1]&4095)) << "C)\n"
	   << "Target Temperature PRT#3 : "
	   << ntofs(mhs.prt[2]&4095, 5, ' ') << "  ("
	   << dto2dp(obct_to_temp(mhs.prt[2]&4095)) << "C)\n"
	   << "Target Temperature PRT#4 : "
	   << ntofs(mhs.prt[3]&4095, 5, ' ') << "  ("
	   << dto2dp(obct_to_temp(mhs.prt[3]&4095)) << "C)\n"
	   << "Target Temperature PRT#5 : "
	   << ntofs(mhs.prt[4]&4095, 5, ' ') << "  ("
	   << dto2dp(obct_to_temp(mhs.prt[4]&4095)) << "C)\n"
	   << "Calibration Channel 1	: "
	   << ntofs(mhs.prt_cal[0]&4095, 5, ' ') << "  ("
	   << dto2dp(obct_to_res(mhs.prt_cal[0]&4095)) << " ohms)\n"
	   << "Calibration Channel 2	: "
	   << ntofs(mhs.prt_cal[1]&4095, 5, ' ') << "  ("
	   << dto2dp(obct_to_res(mhs.prt_cal[1]&4095)) << " ohms)\n"
	   << "Calibration Channel 3	: "
	   << ntofs(mhs.prt_cal[2]&4095, 5, ' ') << "  ("
	   << dto2dp(obct_to_res(mhs.prt_cal[2]&4095)) << " ohms)\n"
	   << endl;

	// notes
	os << "Note 1: Temperature sensors 13 and 14 use different "
	   << "thermistor types\nwhen PIE A is in use, on the "
	   << "Engineering Model only. See document\n\"MHS "
	   << "Housekeeping Telemetry Conversion to Engineering "
	   << "Units\"\nfor details\n\n";

	os << "Note 2: The \"" << Config::current_constants_name.get()
	   << "\" coefficients were used to convert this set of "
	   << "values.\nChannel 5:1 RDM Motor k="
	   << Config::rdm_motor_current_k << "\nChannel 5:2 FDM Motor "
	   << "k=" << Config::fdm_motor_current_k << "\nChannel 5:3 "
	   << "+5v k=" << Config::v5_secondary_current_k << "\nChannel"
	   << " 5:4 +8v k1=" << Config::v8_secondary_current_k1
	   << "	   k2=" << Config::v8_secondary_current_k2 << endl
	   << "Channel 5:5 +15v k1=" << Config::v15_receiver_current_k1
	   << "	   k2=" << Config::v15_receiver_current_k2 << endl
	   << "Channel 5:6 -15v k1=" << Config::nv15_receiver_current_k1
	   << "	   k2=" << Config::nv15_receiver_current_k2 << endl
	   << endl;
}


//
// output_hirs_record
//

void CCSDSPacketDisplayer::output_hirs_record(ostream& os,
											  CCSDS* packet)
{
	CCSDS_HIRS* hirs = (CCSDS_HIRS*)packet->get_data();

	os << endl;

	// HIRS payloads only contain an array of elements.

	for(int i=0; i<64; i++)
	{
		HIRS_Element e = get_hirs_element(hirs, i);
		output_text_element(os, &e, i);
	}
}


//
// extract_iasi_spectrum_data
//

void CCSDSPacketDisplayer::extract_iasi_spectrum_data(CCSDS* packet, deque<byte>& iasi_data)
{
	for(int i=0; i<(packet->get_payload_length() - IASI_Spec_AncillaryData::SIZE + OBTTime::SIZE); i++)
	{
		byte a = packet->get_payload(IASI_Spec_AncillaryData::SIZE - OBTTime::SIZE)[i];
		iasi_data.push_back(a);
	}
}


//
// extract_iasi_spectrum
//

uint16_be CCSDSPacketDisplayer::extract_iasi_spectrum(uint16_be* data, int noBits, int* bitsCtr, int* wordCtr)
{
	uint16_be value;
	// bitsCtr ranges from 0 to 15 - reset to 1 everytime it goes over 15
	// wordCtr starts at 0 and increments by 1 everytime bits goes over 15
	//if (noBits == 7)
	//{
	//	noBits = 7;
	//}

	if ( (*bitsCtr == 0) && ((*bitsCtr + noBits) < 16) )
	{
		value = static_cast<uint16_be>((data[*wordCtr]) & ((1<<noBits) - 1));
		*bitsCtr = *bitsCtr + noBits;
	}
	else if ( (*bitsCtr == 0) && ((*bitsCtr + noBits) >= 16) )
	{
		value = static_cast<uint16_be>((data[*wordCtr]) & ((1<<16) - 1));
		value = ((static_cast<uint16_be>((data[*wordCtr+1]) & ((1<<(noBits-16)) - 1)))<<16) | value;
		*bitsCtr = *bitsCtr + noBits - 16;
		*wordCtr = *wordCtr + 1;
	}
	else if ( (*bitsCtr > 0) && ((*bitsCtr + noBits) < 16) )
	{
		value = static_cast<uint16_be>((data[*wordCtr]>>*bitsCtr) & ((1<<noBits) - 1));
		*bitsCtr = *bitsCtr + noBits;
	}
	else if ( (*bitsCtr > 0) && ((*bitsCtr + noBits) >= 16) )
	{
		value = static_cast<uint16_be>((data[*wordCtr]>>*bitsCtr) & ((1<<(16-*bitsCtr)) - 1));
		value = ((static_cast<uint16_be>((data[*wordCtr+1]) & ((1<<(noBits-16+*bitsCtr)) - 1)))<<(16-*bitsCtr)) | value;
		*bitsCtr = *bitsCtr + noBits - 16;
		*wordCtr = *wordCtr + 1;
	}

	return value;
}


//
// extract_iasi_verification_data
//

void CCSDSPacketDisplayer::extract_iasi_verification_data(CCSDS* packet, deque<byte>& iasi_data)
{

	//cout << "\n***Payload Length = " << packet->get_payload_length() << endl
	//	 << "\n***Offset = "<< (IASI_Verif_AncillaryData::SIZE - OBTTime::SIZE) << endl;

	for(int i=0; i<(packet->get_payload_length() - IASI_Verif_AncillaryData::SIZE + OBTTime::SIZE); i++)
	{
		byte a = packet->get_payload(IASI_Verif_AncillaryData::SIZE - OBTTime::SIZE)[i];
		iasi_data.push_back(a);
	}
}


//
// extract_iasi_image_data
//

void CCSDSPacketDisplayer::extract_iasi_image_data(CCSDS* packet, deque<byte>& iasi_data)
{
	for(int i=0; i<(packet->get_payload_length() - IASI_Image_AncillaryData::SIZE + OBTTime::SIZE); i++)
	{
		byte a = packet->get_payload(IASI_Image_AncillaryData::SIZE - OBTTime::SIZE)[i];
		iasi_data.push_back(a);
	}
}


//
// extract_iasi_image
//

uint16_be CCSDSPacketDisplayer::extract_iasi_image(uint16_be* data, int* offsetPtr, int bpis, int* ctr)
{
	int offset = *offsetPtr;
	uint16_be value;

	offset = offset - bpis;


	if (offset >= 0)
	{
		value = static_cast<uint16_be>(((data[*ctr])>>offset)&((int)(pow(2.0,bpis)-1)));
	}
	else
	{
		int tmp = offset;
		offset = 16 + offset;
		int bit1 = bpis + tmp;
		int bit2 = bpis - bit1;

		value = static_cast<uint16_be>((data[*ctr])&((int)(pow(2.0,bit1)-1)));

		value = (value<<bit2) | (static_cast<uint16_be>(((data[*ctr+1])>>offset)&((int)(pow(2.0,bit2)-1))));
		*ctr = *ctr + 1;
	}

	*offsetPtr = offset;
	return value;
}


//
// extract_iasi_image_ctr
//

uint16_be CCSDSPacketDisplayer::extract_iasi_image_ctr(uint16_be* data, int* offsetPtr, int bpis, int* ctr)
{
	int offset = *offsetPtr;
	uint16_be value;

	offset = offset - bpis;

	if (offset >= 0)
	{
		value = static_cast<uint16_be>((data[*ctr]>>(offset+(bpis-8)))&255);
	}
	else
	{
		int tmp = offset;
		offset = 16 + offset;
		int bit1 = bpis + tmp;
		int bit2 = 8 - bit1;

		value = static_cast<uint16_be>((data[*ctr])&((int)(pow(2.0,bit1)-1)));
		value = (value<<bit2) | (static_cast<uint16_be>((data[*ctr+1]>>(offset+(bpis-8)))&((int)(pow(2.0,bit2)-1))));
		*ctr = *ctr + 1;
	}


	// if ((16-offset) >= bpis)
	// {
	//	   value = static_cast<uint>((data[*ctr]>>offset)&255);
	//	   offset = offset + bpis;
	//	   if (offset >= 16)
	//	   {
	//	offset = offset - 16;
	//	if (offset == 0) *ctr = *ctr + 1;
	//	   }
	// }
	// else
	// {
	//	   value = static_cast<uint>((data[*ctr]>>offset)&((int)(pow(2.0,(16-offset))-1)));
	//	   offset = offset + bpis;
	//	   if (offset >= 16) offset = offset - 16;
	//	   value = value | (static_cast<uint>(data[*ctr+1]&((int)(pow(2.0,(offset-(bpis-8)))-1))))<<(bpis-offset);
	//	   *ctr = *ctr + 1;
	// }

	*offsetPtr = offset;
	return value;
}


//
// output_iasi_spec_record
//

void CCSDSPacketDisplayer::output_iasi_spec_record(ostream& os,
												   CCSDS* packet)
{
	int i, j, ctr;

	CCSDS_IASI_Spec_PX1* iasi = (CCSDS_IASI_Spec_PX1*)packet->get_data();

	os << "\nAncillary Data\n--------------\n\n"
	   << "Operational data word 1\n"
	   << "\tNumber of Samples (NS) : " << iasi->ancillary_data.word[0] << endl << endl
	   << "Operational data word 2\n"
	   << "\tStep Number (SN)		: " << static_cast<uint>((iasi->ancillary_data.word[1]>>8)&255) << endl
	   << "\tScan Position (SP)		: " << static_cast<uint>(iasi->ancillary_data.word[1]&255) << endl << endl
	   << "Operational data word 3\n"
	   << "\tCD						: " << static_cast<uint>((iasi->ancillary_data.word[2]>>15)&1) << endl
	   << "\tCSQ					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>14)&1) << endl
	   << "\tSQ1					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>13)&1) << endl
	   << "\tSQ2					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>12)&1) << endl
	   << "\tIEQ					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>11)&1) << endl
	   << "\tSN_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>7)&1) << endl
	   << "\tCD_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>6)&1) << endl
	   << "\tCSQ_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>5)&1) << endl
	   << "\tSP_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>4)&1) << endl
	   << "\tSQ1_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>3)&1) << endl
	   << "\tSQ2_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>2)&1) << endl
	   << "\tNS_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>1)&1) << endl
	   << "\tIEQ_NV					: " << static_cast<uint>(iasi->ancillary_data.word[2]&1) << endl << endl
	   << "IASI Configuration word 1\n"
	   << "\tIASI Chain				: " << static_cast<uint>((iasi->ancillary_data.word[3]>>12)&1) << endl
	   << "\tMAS HAU1 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>7)&1) << endl
	   << "\tMAS HAU2 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>6)&1) << endl
	   << "\tMAS HAU3 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>5)&1) << endl
	   << "\tMAS HAU4 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>4)&1) << endl
	   << "\tLaser Identifier		: " << static_cast<uint>(iasi->ancillary_data.word[3]&1) << endl << endl
	   << "IASI Configuration word 2\n"
	   << "\tRC						: " << static_cast<uint>((iasi->ancillary_data.word[4]>>15)&1) << endl
	   << "\tLNR					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>14)&1) << endl
	   << "\tASE					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>13)&1) << endl
	   << "\tLBR					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>12)&1) << endl
	   << "\tPIXEL 1				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>9)&7) << endl
	   << "\tPIXEL 2				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>6)&7) << endl
	   << "\tPIXEL 3				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>3)&7) << endl
	   << "\tPIXEL 4				: " << static_cast<uint>(iasi->ancillary_data.word[4]&7) << endl << endl
	   << "Parameter Table Set Identifier\n"
	   << "\tPTSI (MSW)				: " << iasi->ancillary_data.word[5] << endl
	   << "\tPTSI (LSW)				: " << iasi->ancillary_data.word[6] << endl << endl
	   << "Line Number\n"
	   << "\tLN						: " << iasi->ancillary_data.word[7] << endl << endl
	   << "Instrument Mode\n"
	   << "\tIM						: " << ntobs(iasi->ancillary_data.word[8], 15) << endl << endl
	   << "System Quality Index for Spectra\n"
	   << "\tSQIS					: " << ntobs(iasi->ancillary_data.word[9], 15) << endl << endl
	   << "Pixel Identifier\n"
	   << "\tPI						: " << ntobs(iasi->ancillary_data.word[10], 15) << endl << endl
	   << "Number of MAS frames counted by DPS\n"
	   << "\tBDcoNbReceivedWords	: " << iasi->ancillary_data.word[11] << endl << endl
	   << "Mean value of interferogram after nonlinearity correction\n "
	   << "\tBNlcAnaMV for Band B1	: " << uint16tofloat(iasi->ancillary_data.word[12],
														 iasi->ancillary_data.word[13]) << endl
	   << "\tBNlcAnaMV for Band B2	: " << uint16tofloat(iasi->ancillary_data.word[14],
														 iasi->ancillary_data.word[15]) << endl
	   << "\tBNlcAnaMV for Band B3	: " << uint16tofloat(iasi->ancillary_data.word[16],
														 iasi->ancillary_data.word[17]) << endl << endl
	   << "NZPD value for the NZPD selected band\n"
	   << "\tBZpdNzpd				: " << iasi->ancillary_data.word[18] << endl << endl
	   << "NZPD Quality Index\n"
	   << "\tBzpdNzpdQualIndexEW	: " << uint16tofloat(iasi->ancillary_data.word[19],
														 iasi->ancillary_data.word[20]) << endl << endl;

	os << "Mean value of imaginary part of the calibarted spectrum for bands B1-B3\n";
	for(i=0; i<8; i++)
	{
		ctr = 21;
		os << "\tBArcImagMean_B1[" << i+1 <<"]	   : "	<< uint16tofloat(iasi->ancillary_data.word[ctr],
																		 iasi->ancillary_data.word[ctr+1]) << endl;
		ctr = ctr + 2;
	}
	os << endl;


	for(i=0; i<10; i++)
	{
		ctr = 37;
		if(i < 9)
		{
			os << "\tBArcImagMean_B2[" << i+1 <<"]	   : "	<< uint16tofloat(iasi->ancillary_data.word[ctr],
																			 iasi->ancillary_data.word[ctr+1]) << endl;
		}
		else
		{
			os << "\tBArcImagMean_B2[" << i+1 <<"]	  : "  << uint16tofloat(iasi->ancillary_data.word[ctr],
																			iasi->ancillary_data.word[ctr+1]) << endl;
		}
		ctr = ctr + 2;
	}
	os << endl;


	for(i=0; i<10; i++)
	{
		ctr = 57;
		if (i < 9)
		{
			os << "\tBArcImagMean_B3[" << i+1 <<"]	   : "	<< uint16tofloat(iasi->ancillary_data.word[ctr],
																			 iasi->ancillary_data.word[ctr+1]) << endl;
		}
		else
		{
			os << "\tBArcImagMean_B3[" << i+1 <<"]	  : "  << uint16tofloat(iasi->ancillary_data.word[ctr],
																			iasi->ancillary_data.word[ctr+1]) << endl;
		}
		ctr = ctr + 2;
	}
	os << endl;


	os << "Mean value of square of imaginary part of the calibarted spectrum for bands B1-B3\n";
	for(i=0; i<8; i++)
	{
		ctr = 77;
		os << "\tBArcImagRMS_B1[" << i+1 <<"]	   : "	<< uint16tofloat(iasi->ancillary_data.word[ctr],
																		 iasi->ancillary_data.word[ctr+1]) << endl;
		ctr = ctr + 2;
	}
	os << endl;


	for(i=0; i<10; i++)
	{
		ctr = 93;
		if (i < 9)
		{
			os << "\tBArcImagRMS_B2[" << i+1 <<"]	   : "	<< uint16tofloat(iasi->ancillary_data.word[ctr],
																			 iasi->ancillary_data.word[ctr+1]) << endl;
		}
		else
		{
			os << "\tBArcImagRMS_B2[" << i+1 <<"]	  : "  << uint16tofloat(iasi->ancillary_data.word[ctr],
																			iasi->ancillary_data.word[ctr+1]) << endl;
		}
		ctr = ctr + 2;
	}
	os << endl;


	for(i=0; i<10; i++)
	{
		ctr = 113;
		if (i < 9)
		{
			os << "\tBArcImagRMS_B3[" << i+1 <<"]	   : "	<< uint16tofloat(iasi->ancillary_data.word[ctr],
																			 iasi->ancillary_data.word[ctr+1]) << endl;
		}
		else
		{
			os << "\tBArcImagRMS_B3[" << i+1 <<"]	  : "  << uint16tofloat(iasi->ancillary_data.word[ctr],
																			iasi->ancillary_data.word[ctr+1]) << endl;
		}
		ctr = ctr + 2;
	}
	os << endl;

	os << "Mean value of mean value of square of imaginary part of the calibarted spectrum for bands B1-B3\n"
	   << "\tBArcImagMeanRMS_B1		: " << uint16tofloat(iasi->ancillary_data.word[133],
														 iasi->ancillary_data.word[134]) << endl
	   << "\tBArcImagMeanRMS_B2		: " << uint16tofloat(iasi->ancillary_data.word[135],
														 iasi->ancillary_data.word[136]) << endl
	   << "\tBArcImagMeanRMS_B3		: " << uint16tofloat(iasi->ancillary_data.word[137],
														 iasi->ancillary_data.word[138]) << endl << endl
	   << "Status_area_word_1\n"
	   << "\tDPS LNR-Rx DVL flag	: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>15)&1) << endl
	   << "\tDPS LNR-Rx VLN flag	: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>14)&1) << endl
	   << "\tBBofFlagSpectNonQual	: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>13)&1) << endl
	   << "\tBDcoFlagMasErrorPath_B1: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>12)&1) << endl
	   << "\tBDcoFlagMasErrorPath_B2: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>11)&1) << endl
	   << "\tBDcoFlagMasErrorPath_B3: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>10)&1) << endl
	   << "\tBDcoFlagMasOverflow_B1 : " << static_cast<uint>(((iasi->ancillary_data.word[139])>>9)&1) << endl
	   << "\tBDcoFlagMasOverflow_B2 : " << static_cast<uint>(((iasi->ancillary_data.word[139])>>8)&1) << endl
	   << "\tBDcoFlagMasOverflow_B3 : " << static_cast<uint>(((iasi->ancillary_data.word[139])>>7)&1) << endl
	   << "\tBDcoFlagMasEcret_B1	: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>6)&1) << endl
	   << "\tBDcoFlagMasEcret_B2	: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>5)&1) << endl
	   << "\tBDcoFlagMasEcret_B3	: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>4)&1) << endl
	   << "\tBDcoFlagMasErrorNbWords: " << static_cast<uint>(((iasi->ancillary_data.word[139])>>3)&1) << endl << endl
	   << "Status_area_word_2\n"
	   << "\tBSpkFlagSpik_B1			: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>15)&1) << endl
	   << "\tBSpkFlagSpik_B2			: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>14)&1) << endl
	   << "\tBSpkFlagSpik_B3			: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>13)&1) << endl
	   << "\tBzpdFlagNzpdNonQualEW		: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>12)&1) << endl
	   << "\tBIsiFlagErrorFft_B1		: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>11)&1) << endl
	   << "\tBIsiFlagErrorFft_B2		: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>10)&1) << endl
	   << "\tBIsiFlagErrorFft_B3		: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>9)&1) << endl
	   << "\tBArcFlagCalSpectNonQual_B1 : " << static_cast<uint>(((iasi->ancillary_data.word[140])>>8)&1) << endl
	   << "\tBArcFlagCalSpectNonQual_B2 : " << static_cast<uint>(((iasi->ancillary_data.word[140])>>7)&1) << endl
	   << "\tBArcFlagCalSpectNonQual_B3 : " << static_cast<uint>(((iasi->ancillary_data.word[140])>>6)&1) << endl
	   << "\tBCodFlagFlood				: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>5)&1) << endl
	   << "\tBDcoFlagErrorInterf_B1		: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>4)&1) << endl
	   << "\tBDcoFlagErrorInterf_B2		: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>3)&1) << endl
	   << "\tBDcoFlagErrorInterf_B3		: " << static_cast<uint>(((iasi->ancillary_data.word[140])>>2)&1) << endl << endl
	   << "Maximum number of bits allowed for spectrum coding\n"
	   << "\tIDefCodNbBitMax			: " <<	static_cast<uint>((iasi->ancillary_data.word[144]<<16)
																  +iasi->ancillary_data.word[145]) << endl;

	os << "\nSpectrum Application Data\n-------------------------\n\n";

	// Extract Payload Data
	deque<byte> payload_data;
	extract_iasi_spectrum_data(packet, payload_data);

	int payload_data_size = packet->get_payload_length() - IASI_Spec_AncillaryData::SIZE + 6;
	DEBUG(" payload_data_size = " <<  payload_data_size << endl);

	uint16_be *data = new uint16_be[payload_data_size];
	copy_n(payload_data.begin(), payload_data_size, (byte*)(data));

	IASIConfig spec_config;
	bool readFlag = spec_config.read(iasi_config_file);

	if (readFlag)
	{
		int IDefCtcNbNsSeg = spec_config.get_IDefCtcNbNsSeg();
		int ICtcTabNbBit_ctr = spec_config.get_ICtcTabNbBit_ctr();
		int *ptrICtcTabNbBit = spec_config.get_ICtcTabNbBit();

//	os << "is_loaded		= " << spec_config.is_loaded() << endl
//	   << "ICtcNbSeg		= " << spec_config.get_ICtcNbSeg() << endl
//	   << "IDefCtcNbNsSeg	= " << spec_config.get_IDefCtcNbNsSeg() << endl
//	   << "ICtcTabNbBit_ctr = " << spec_config.get_ICtcTabNbBit_ctr() << endl
//	   << "ICtcTabNbBit		= " << endl;

//	int *ptrICtcTabNbBit = spec_config.get_ICtcTabNbBit();
//	ctr = 0;
//	for (i=0; i<spec_config.get_ICtcTabNbBit_ctr(); i++)
//	{
//		os << i+1 << "\t";
//		os << ptrICtcTabNbBit[ctr] << "\t";
//		os << ptrICtcTabNbBit[ctr+1] << "\t";
//		os << ptrICtcTabNbBit[ctr+2] << endl;
//		ctr = ctr + 3;
//	}

		os	<< "Counter\tValue\n";
		//os << "Counter\tRaw result\tGain\tOffset\tValue\n";

		ctr = 0;
		int bitsCtr = 0;
		int wordCtr = 0;
		int tempData = 0;
		for (i=0; i<ICtcTabNbBit_ctr; i++)
		{
			for (j=0; j<IDefCtcNbNsSeg; j++)
			{
				// Decode ptrICtcTabNbBit[ctr] bits of data
				os << ((i*IDefCtcNbNsSeg)+j+1) << "\t";
				tempData = extract_iasi_spectrum(data, ptrICtcTabNbBit[ctr], &bitsCtr, &wordCtr);

				// Apply scale and offset to convert decoded value into something more meaningful
				// offset = ptrICtcTabNbBit[ctr+1] and scale = ptrICtcTabNbBit[ctr+2]
				//os << tempData << "\t" << ptrICtcTabNbBit[ctr+1] << "\t" << ptrICtcTabNbBit[ctr+2] << "\t";
				os << ((double)(tempData -1 + ptrICtcTabNbBit[ctr+1]))/((double)ptrICtcTabNbBit[ctr+2]) << endl;
			}
			ctr = ctr + 3;
		}

//		Used for testing purposes only
//
//	for (j=0; j<100; j++)
//	{
//		os << ((i*IDefCtcNbNsSeg)+j+1) << "\t";
//		os << extract_iasi_spectrum(data, 8, &bitsCtr, &wordCtr) << endl;
//	}

	}

	delete[] data;

}


//
// output_iasi_image_record
//

void CCSDSPacketDisplayer::output_iasi_image_record(ostream& os,
													CCSDS* packet)
{
	int i, ctr;

	CCSDS_IASI_Image* iasi = (CCSDS_IASI_Image*)packet->get_data();

	os << "\nAncillary Data\n--------------\n\n"
	   << "Operational data word 1\n"
	   << "\tNumber of Samples (NS) : " << iasi->ancillary_data.word[0] << endl << endl
	   << "Operational data word 2\n"
	   << "\tStep Number (SN)		: " << static_cast<uint>((iasi->ancillary_data.word[1]>>8)&255) << endl
	   << "\tScan Position (SP)		: " << static_cast<uint>(iasi->ancillary_data.word[1]&255) << endl << endl
	   << "Operational data word 3\n"
	   << "\tCD						: " << static_cast<uint>((iasi->ancillary_data.word[2]>>15)&1) << endl
	   << "\tCSQ					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>14)&1) << endl
	   << "\tSQ1					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>13)&1) << endl
	   << "\tSQ2					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>12)&1) << endl
	   << "\tIEQ					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>11)&1) << endl
	   << "\tSN_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>7)&1) << endl
	   << "\tCD_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>6)&1) << endl
	   << "\tCSQ_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>5)&1) << endl
	   << "\tSP_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>4)&1) << endl
	   << "\tSQ1_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>3)&1) << endl
	   << "\tSQ2_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>2)&1) << endl
	   << "\tNS_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>1)&1) << endl
	   << "\tIEQ_NV					: " << static_cast<uint>(iasi->ancillary_data.word[2]&1) << endl << endl
	   << "IASI Configuration word 1\n"
	   << "\tIASI Chain				: " << static_cast<uint>((iasi->ancillary_data.word[3]>>12)&1) << endl
	   << "\tMAS HAU1 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>7)&1) << endl
	   << "\tMAS HAU2 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>6)&1) << endl
	   << "\tMAS HAU3 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>5)&1) << endl
	   << "\tMAS HAU4 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>4)&1) << endl
	   << "\tLaser Identifier		: " << static_cast<uint>(iasi->ancillary_data.word[3]&1) << endl << endl
	   << "IASI Configuration word 2\n"
	   << "\tRC						: " << static_cast<uint>((iasi->ancillary_data.word[4]>>15)&1) << endl
	   << "\tLNR					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>14)&1) << endl
	   << "\tASE					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>13)&1) << endl
	   << "\tLBR					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>12)&1) << endl
	   << "\tPIXEL 1				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>9)&7) << endl
	   << "\tPIXEL 2				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>6)&7) << endl
	   << "\tPIXEL 3				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>3)&7) << endl
	   << "\tPIXEL 4				: " << static_cast<uint>(iasi->ancillary_data.word[4]&7) << endl << endl
	   << "Parameter Table Set Identifier\n"
	   << "\tPTSI (MSW)				: " << iasi->ancillary_data.word[5] << endl
	   << "\tPTSI (LSW)				: " << iasi->ancillary_data.word[6] << endl << endl
	   << "Line Number\n"
	   << "\tLN						: " << iasi->ancillary_data.word[7] << endl << endl
	   << "Instrument Mode\n"
	   << "\tIM						: " << ntobs(iasi->ancillary_data.word[8], 15) << endl << endl
	   << "System Quality Index for Image\n"
	   << "\tSQII					: " << ntobs(iasi->ancillary_data.word[9], 15) << endl << endl
	   << "Image status\n"
	   << "\tIIS_SIZE				: " << static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255) << endl
	   << "\tIIS ADC Overflow Flag	: " << static_cast<uint>((iasi->ancillary_data.word[10]>>7)&1) << endl
	   << "\tNo. of IIS Samples Flag: " << static_cast<uint>((iasi->ancillary_data.word[10]>>6)&1) << endl << endl
	   << "Number of IIS Samples\n"
	   << "\tIIS_COUNT				: " << iasi->ancillary_data.word[11] << endl << endl;


	// Extract Payload Data
	deque<byte> payload_data;
	extract_iasi_image_data(packet, payload_data);

	// should be able to use bits_per_image_sample = IIS_SIZE = static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255),
	// but the test data is of poor quality and does not allow this, hence use size of data instead.
	unsigned int bits_per_image_sample = 0;
	if ((packet->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 4100)
	{
		bits_per_image_sample = 8;
		DEBUG("\n*** 8 ***\n");
	}
	if ((packet->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 5152)
	{
		bits_per_image_sample = 10;
		DEBUG("\n*** 10 ***\n");
	}
	if ((packet->get_payload_length() + 6 - IASI_Image_AncillaryData::SIZE) == 6150)
	{
		bits_per_image_sample = 12;
		DEBUG("\n*** 12 ***\n");
	}


	if ( bits_per_image_sample != static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255) )
	{
		os << "\n*** WARNING: IIS_SIZE and the calculated bits per image sample ("
		   << bits_per_image_sample
		   << ") do not match.\n\n";
		//return;
	}

	if (bits_per_image_sample > 0)
	{
		int payload_data_size = ((4100*bits_per_image_sample)/16)*2; //*8, *10, *12
		//uint16_be *data = new uint16_be[payload_data_size/2];
		//uint16_be data[payload_data_size/2];
		//copy_n(payload_data.begin(), payload_data_size, (byte*)(data));

		if (bits_per_image_sample == 8)
		{
			uint16_be data[2050];
			copy_n(payload_data.begin(), payload_data_size, (byte*)(&data));

			os << "\nImage Application Data\n----------------------\n\n"
			   << "Equalization counter part 1	: " << static_cast<uint>((data[0]>>8)&255) << endl;
			os << "Equalization counter part 2	: " << static_cast<uint>(data[0]&255) << endl;
			os << "Equalization counter part 3	: " << static_cast<uint>((data[1]>>8)&255) << endl;
			os << "Equalization counter part 4	: " << static_cast<uint>(data[1]&255)<< endl
			   << "\nImage Sample\n------------\n\n";

			ctr = 2;
			for (i=0; i<1024; i++)
			{
				os << "\t" << ((ctr-2)*2)+1 << "\t";
				os << static_cast<uint>((data[ctr]>>8)&255) << "\t";
				os << static_cast<uint>(data[ctr]&255) << "\t";
				os << static_cast<uint>((data[ctr+1]>>8)&255) << "\t";
				os << static_cast<uint>(data[ctr+1]&255) << endl;
				ctr = ctr + 2;
			}
			os << endl << endl;
		}


		// uint16_be extract_iasi_image(uint16_be* data, int* offsetPtr, int bpis, int* ctr)
		if (bits_per_image_sample == 10)
		{
			uint16_be data[2563];
			copy_n(payload_data.begin(), payload_data_size, (byte*)(&data));
			int offset = 16;
			int ctr2 = 0;

			os << "\nImage Application Data\n----------------------\n\n"
			   << "Equalization counter part 1	: "
			   << extract_iasi_image_ctr(data, &offset, bits_per_image_sample, &ctr2) << endl;
			os << "Equalization counter part 2	: "
			   << extract_iasi_image_ctr(data, &offset, bits_per_image_sample, &ctr2) << endl;
			os << "Equalization counter part 3	: "
			   << extract_iasi_image_ctr(data, &offset, bits_per_image_sample, &ctr2) << endl;
			os << "Equalization counter part 4	: "
			   << extract_iasi_image_ctr(data, &offset, bits_per_image_sample, &ctr2) << endl
			   << "\nImage Sample\n------------\n\n";

			ctr = 2;
			for (i=0; i<1024; i++)
			{
				os << "\t" << ((ctr-2)*2)+1 << "\t";
				os << extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2) << "\t";
				os << extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2) << "\t";
				os << extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2) << "\t";
				os << extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2) << endl;
				ctr = ctr + 2;
			}
			os << endl << endl;
		}


		// uint16_be extract_iasi_image(uint16_be* data, int* offsetPtr, int bpis, int* ctr)
		if (bits_per_image_sample == 12)
		{
			uint16_be data[3075];
			copy_n(payload_data.begin(), payload_data_size, (byte*)(&data));
			int offset = 16;
			int ctr2 = 0;

			os << "\nImage Application Data\n----------------------\n\n"
			   << "Equalization counter part 1	: "
			   << extract_iasi_image_ctr(data, &offset, bits_per_image_sample, &ctr2) << endl;
			os << "Equalization counter part 2	: "
			   << extract_iasi_image_ctr(data, &offset, bits_per_image_sample, &ctr2) << endl;
			os << "Equalization counter part 3	: "
			   << extract_iasi_image_ctr(data, &offset, bits_per_image_sample, &ctr2) << endl;
			os << "Equalization counter part 4	: "
			   << extract_iasi_image_ctr(data, &offset, bits_per_image_sample, &ctr2) << endl
			   << "\nImage Sample\n------------\n"
			   << "\tCount\tData\n";

			ctr = 2;
			for (i=0; i<1024; i++)
			{
				os << "\t" << ((ctr-2)*2)+1 << "\t";
				os << extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2) << "\t";
				os << extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2) << "\t";
				os << extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2) << "\t";
				os << extract_iasi_image(data, &offset, bits_per_image_sample, &ctr2) << endl;

				ctr = ctr + 2;
			}
			os << endl << endl;
		}

		//delete data;
	}
	else
	{
		os << "\nImage Application Data\n----------------------\n\n"
		   << "*** Data Not Found ***" << endl << endl;

	}
}


//
// output_iasi_verif_record
//

void CCSDSPacketDisplayer::output_iasi_verif_record(ostream& os,
													CCSDS* packet)
{
	int i, ctr;

	CCSDS_IASI_Verif* iasi = (CCSDS_IASI_Verif*)packet->get_data();

	os << "\nAncillary Data\n--------------\n\n"
	   << "Operational data word 1\n"
	   << "\tNumber of Samples (NS) : " << iasi->ancillary_data.word[0] << endl << endl
	   << "Operational data word 2\n"
	   << "\tStep Number (SN)		: " << static_cast<uint>((iasi->ancillary_data.word[1]>>8)&255) << endl
	   << "\tScan Position (SP)		: " << static_cast<uint>(iasi->ancillary_data.word[1]&255) << endl << endl
	   << "Operational data word 3\n"
	   << "\tCD						: " << static_cast<uint>((iasi->ancillary_data.word[2]>>15)&1) << endl
	   << "\tCSQ					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>14)&1) << endl
	   << "\tSQ1					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>13)&1) << endl
	   << "\tSQ2					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>12)&1) << endl
	   << "\tIEQ					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>11)&1) << endl
	   << "\tSN_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>7)&1) << endl
	   << "\tCD_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>6)&1) << endl
	   << "\tCSQ_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>5)&1) << endl
	   << "\tSP_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>4)&1) << endl
	   << "\tSQ1_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>3)&1) << endl
	   << "\tSQ2_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>2)&1) << endl
	   << "\tNS_NV					: " << static_cast<uint>((iasi->ancillary_data.word[2]>>1)&1) << endl
	   << "\tIEQ_NV					: " << static_cast<uint>(iasi->ancillary_data.word[2]&1) << endl << endl
	   << "IASI Configuration word 1\n"
	   << "\tIASI Chain				: " << static_cast<uint>((iasi->ancillary_data.word[3]>>12)&1) << endl
	   << "\tMAS HAU1 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>7)&1) << endl
	   << "\tMAS HAU2 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>6)&1) << endl
	   << "\tMAS HAU3 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>5)&1) << endl
	   << "\tMAS HAU4 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[3]>>4)&1) << endl
	   << "\tLaser Identifier		: " << static_cast<uint>(iasi->ancillary_data.word[3]&1) << endl << endl
	   << "IASI Configuration word 2\n"
	   << "\tRC						: " << static_cast<uint>((iasi->ancillary_data.word[4]>>15)&1) << endl
	   << "\tLNR					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>14)&1) << endl
	   << "\tASE					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>13)&1) << endl
	   << "\tLBR					: " << static_cast<uint>((iasi->ancillary_data.word[4]>>12)&1) << endl
	   << "\tPIXEL 1				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>9)&7) << endl
	   << "\tPIXEL 2				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>6)&7) << endl
	   << "\tPIXEL 3				: " << static_cast<uint>((iasi->ancillary_data.word[4]>>3)&7) << endl
	   << "\tPIXEL 4				: " << static_cast<uint>(iasi->ancillary_data.word[4]&7) << endl << endl
	   << "Parameter Table Set Identifier\n"
	   << "\tPTSI (MSW)				: " << iasi->ancillary_data.word[5] << endl
	   << "\tPTSI (LSW)				: " << iasi->ancillary_data.word[6] << endl << endl
	   << "Line Number\n"
	   << "\tLN						: " << iasi->ancillary_data.word[7] << endl << endl
	   << "Instrument Mode\n"
	   << "\tIM						: " << ntobs(iasi->ancillary_data.word[8], 15) << endl << endl
	   << "Verification Data Selection (VDS)\n"
	   << "\tPixel Identifier (P)	: " << static_cast<uint>((iasi->ancillary_data.word[9]>>12)&15) << endl
	   << "\tBand Identifier (B)	: " << static_cast<uint>((iasi->ancillary_data.word[9]>>8)&15) << endl
	   << "\tStep Identifier (S)	: " << static_cast<uint>((iasi->ancillary_data.word[9])&255) << endl << endl
	   << "Verification Status\n"
	   << "\tPacket Identifier		: " << static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255) << endl
	   << "\tBZpdFlagNzpdNonQual	: " << static_cast<uint>((iasi->ancillary_data.word[10]>>7)&1) << endl
	   << "\tBdcoFlagMasErrorNbWords: " << static_cast<uint>((iasi->ancillary_data.word[10]>>6)&1) << endl << endl
	   << "NZPD value for the NZPD selected band realted to the selected VDS\n"
	   << "\tBZpdNzpd				: " << iasi->ancillary_data.word[11] << endl << endl
	   << "NZPD Quality Index\n"
	   << "\tBzpdNzpdQualIndex		: " << uint16tofloat(iasi->ancillary_data.word[12],
														 iasi->ancillary_data.word[13]) << endl << endl
	   << "Number of MAS frames counted by DPS\n"
	   << "\tBdcoNbReceivedWords	: " << iasi->ancillary_data.word[14] << endl << endl
	   << "ZPD first sample\n"
	   << "\tIZsbNsfirstSrd			: " << iasi->ancillary_data.word[15] << endl << endl
	   << "ZPD last sample\n"
	   << "\tIZsbNslastSrd			: " << iasi->ancillary_data.word[16] << endl << endl
	   <<"First spectral sample\n"
	   << "\tIUsbNsfirst			: " << iasi->ancillary_data.word[17] << endl << endl
	   <<"Last spectral sample\n"
	   << "\tIUsbNslast				: " << iasi->ancillary_data.word[18] << endl << endl
	   << "First B1 and B2 overlapping spectral sample\n"
	   << "\tIOsbNsFirstMb1b2		: " << iasi->ancillary_data.word[19] << endl << endl
	   << "Last B1 and B2 overlapping spectral sample\n"
	   << "\tIOsbNsLastMb1b2		: " << iasi->ancillary_data.word[20] << endl << endl
	   << "First B2 and B3 overlapping spectral sample\n"
	   << "\tIOsbNsFirstMb2b3		: " << iasi->ancillary_data.word[21] << endl << endl
	   << "Last B2 and B3 overlapping spectral sample\n"
	   << "\tIOsbNsLastMb2b3		: " << iasi->ancillary_data.word[22] << endl << endl;


	// Extract Payload Data
	deque<byte> payload_data;
	extract_iasi_verification_data(packet, payload_data);

	// Packet Identifier
	int packetID = static_cast<uint>((iasi->ancillary_data.word[10]>>8)&255);


	if (packetID == 1)
	{
		os << "\nVerification Application Data - Type A\n--------------------------------------\n";

		int payload_data_size = (iasi->ancillary_data.word[14] + (iasi->ancillary_data.word[14] % 2)) / 2;
		uint16_be *data = new uint16_be[payload_data_size];
		copy_n(payload_data.begin(), (payload_data_size*2), (byte*)(data));

		DEBUG("payload_data_size = " << payload_data_size << endl << endl);

		os << "MAS Header Frames" << endl
		   << "\tBDcoRawOffset1			: " << data[0] << endl
		   << "\tBDcoRawOffset2			: " << data[1] << endl
		   << "\tBDcoRawOffset3			: " << data[2] << endl
		   << "\tAuxiliary Values Set 1 : " << data[3] << "\t" <<data[4] << endl;
		ctr = 5;
		for (i=0; i<10; i++)
		{
			os << "\t						  " << data[ctr] << "\t" <<data[ctr+1] << endl;
			ctr = ctr + 2;
		}

		os << "\tBDcoRawMVcoarse		: " << data[25] << endl
		   << "\tAuxiliary Values Set 2 : " << data[26] << "\t" <<data[27] << endl;
		ctr = 28;
		for (i=0; i<17; i++)
		{
			os << "\t						  " << data[ctr] << "\t" <<data[ctr+1] << endl;
			ctr = ctr + 2;
		}

		os << "\tBDcoRawMVfine			: " << data[62] << endl
		   << "\tAuxiliary Values Set 3 : " << data[63] << "\t" <<data[64] << endl;
		ctr = 65;
		for (i=0; i<13; i++)
		{
			os << "\t						  " << data[ctr] << "\t" <<data[ctr+1] << endl;
			ctr = ctr + 2;
		}

		os << "\tBDcoRawMV3				: " << data[92] << endl
		   << "\tBDcoRawMV2				: " << data[93] << endl
		   << "\tBDcoRawMV1				: " << data[94] << endl
		   << "\tAuxiliary Values Set 4 : " << data[95] << "\t" <<data[96] << endl
		   << "\t						  " << data[97] << "\t" <<data[98] << endl
		   << "\t						  " << data[99] << endl << endl;

		os << "MAS Interferogram Frames" << endl;
		for (i=100; i<payload_data_size; i++)
		{
			os << "\tSample Count = " << i-99 << " (" << data[i] << ")" << endl
			   << "\t\tSpike Detection Flag		  : " << static_cast<uint>((data[i]>>15)&1) << endl
			   << "\t\tADC Overflow Flag		  : " << static_cast<uint>((data[i]>>14)&1) << endl
			   << "\t\tAmplification Path		  : " << static_cast<uint>((data[i]>>12)&3) << endl
			   << "\t\tInterferogram Sample Value : " << static_cast<uint>(data[i]&4095) << endl;
		}

		delete[] data;

	}

	else if (packetID == 2)
	{

		os << "\nVerification Application Data - Type B\n--------------------------------------\n";

		int payload_data_size = (iasi->ancillary_data.word[14] - (iasi->ancillary_data.word[14] % 2)) / 2;
		uint16_be *data = new uint16_be[payload_data_size];
		copy_n(payload_data.begin(), (payload_data_size*2), (byte*)(data));

		DEBUG("payload_data_size = " << payload_data_size << endl);

		os << "MAS Interferogram Frames" << endl;
		for (i=0; i<payload_data_size; i++)
		{
			os << "\tSample Count = " << i+1 << " (" << data[i] << ")" << endl
			   << "\t\tSpike Detection Flag		  : " << static_cast<uint>((data[i]>>15)&1) << endl
			   << "\t\tADC Overflow Flag		  : " << static_cast<uint>((data[i]>>14)&1) << endl
			   << "\t\tAmplification Path		  : " << static_cast<uint>((data[i]>>12)&3) << endl
			   << "\t\tInterferogram Sample Value : " << static_cast<uint>(data[i]&4095) << endl;
		}

		delete[] data;

	}

	else if (packetID == 3)
	{
		os << "\nVerification Application Data - Type C\n--------------------------------------\n";

		int IZsbNsDiffSrd = iasi->ancillary_data.word[16] - iasi->ancillary_data.word[15] + 1; //IZsbNsfirstSrd - IZsbNslastSrd
		int IUsbNsDiff = iasi->ancillary_data.word[18] - iasi->ancillary_data.word[17] + 1; //IUsbNslast - IUsbNsfirst

		int payload_data_size = (IZsbNsDiffSrd * 4) + (IUsbNsDiff * 4);

		int payload_data_size_tmp = packet->get_payload_length() - IASI_Verif_AncillaryData::SIZE + OBTTime::SIZE;

		if ((payload_data_size*4) != payload_data_size_tmp)
		{
			os << "\n***WARNING: Calculated (" << (payload_data_size*4) << ") and actual ("
			   << payload_data_size_tmp << ") payload data sizes do not match (IZsbNsDiffSrd="
			   << IZsbNsDiffSrd << ", IUsbNsDiff=" << IUsbNsDiff << "). ";

			payload_data_size = payload_data_size_tmp/4;
			IUsbNsDiff = (payload_data_size/2 - (2*IZsbNsDiffSrd)) / 2;

			os << "It is assumed that the actual payload data size is correct and thus "
			   << "the following updated parameters will be used for decoding the rest "
			   << "of this packet (IZsbNsDiffSrd=" << IZsbNsDiffSrd
			   << ", IUsbNsDiff=" << IUsbNsDiff << "). \n\n";
		}

		uint16_be *data = new uint16_be[payload_data_size*2];
		copy_n(payload_data.begin(), (payload_data_size*4), (byte*)(data));

		DEBUG("payload_data_size = " << payload_data_size << endl
			  << "IZsbNsDiffSrd		= " << IZsbNsDiffSrd << endl
			  << "IUsbNsDiff		= " << IUsbNsDiff << endl);

		os << "BFrsSrdCS" << endl
		   << "\tCounter\t\tReal\t\t\tImaginary" << endl;
		ctr = 0;
		for (i=0; i<IZsbNsDiffSrd; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1])
			   << "\t\t" << uint16tofloat(data[ctr+2], data[ctr+3]) << endl;
			ctr = ctr + 4;
		}

		os << "BFrsSrdBB" << endl
		   << "\tCounter\t\tReal\t\t\tImaginary" << endl;
		for (i=0; i<IZsbNsDiffSrd; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1])
			   << "\t\t" << uint16tofloat(data[ctr+2], data[ctr+3]) << endl;
			ctr = ctr + 4;
		}

		os << "BFrcOffset" << endl
		   << "\tCounter\t\tReal\t\t\tImaginary" << endl;
		for (i=0; i<IUsbNsDiff; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1])
			   << "\t\t" << uint16tofloat(data[ctr+2], data[ctr+3]) << endl;
			ctr = ctr + 4;
		}

		os << "BFrcSlope" << endl
		   << "\tCounter\t\tReal\t\t\tImaginary" << endl;
		for (i=0; i<IUsbNsDiff; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1])
			   << "\t\t" << uint16tofloat(data[ctr+2], data[ctr+3]) << endl;
			ctr = ctr + 4;
		}

		delete[] data;

	}

	else if (packetID == 4)
	{
		os << "\nVerification Application Data - Type D\n--------------------------------------\n";

		int IUsbNsDiff = iasi->ancillary_data.word[18] - iasi->ancillary_data.word[17] + 1; //IUsbNslast - IUsbNsfirst

		int payload_data_size = IUsbNsDiff * 4;

		int payload_data_size_tmp = packet->get_payload_length() - IASI_Verif_AncillaryData::SIZE + OBTTime::SIZE;

		if ((payload_data_size*4) != payload_data_size_tmp)
		{
			os << "\n***WARNING: Calculated (" << (payload_data_size*4) << ") and actual ("
			   << payload_data_size_tmp << ") payload data sizes do not match (IUsbNsDiff=" << IUsbNsDiff << "). ";

			payload_data_size = payload_data_size_tmp/4;
			IUsbNsDiff = payload_data_size / 4;

			os << "It is assumed that the actual payload data size is correct and thus "
			   << "the following updated parameters will be used for decoding the rest "
			   << "of this packet (IUsbNsDiff=" << IUsbNsDiff << "). \n\n";
		}

		uint16_be *data = new uint16_be[payload_data_size*2];
		copy_n(payload_data.begin(), (payload_data_size*4), (byte*)(data));

		DEBUG("payload_data_size = " << payload_data_size << endl
			  << "IUsbNsDiff		= " << IUsbNsDiff << endl);

		os << "BcrcOffset" << endl
		   << "\tCounter\t\tReal\t\t\tImaginary" << endl;
		ctr = 0;
		for (i=0; i<IUsbNsDiff; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1])
			   << "\t\t" << uint16tofloat(data[ctr+2], data[ctr+3]) << endl;
			ctr = ctr + 4;
		}

		os << "BcrcSlope" << endl
		   << "\tCounter\t\tReal\t\t\tImaginary" << endl;
		for (i=0; i<IUsbNsDiff; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1])
			   << "\t\t" << uint16tofloat(data[ctr+2], data[ctr+3]) << endl;
			ctr = ctr + 4;
		}

		delete[] data;

	}

	else if (packetID == 5)
	{
		os << "\nVerification Application Data - Type E\n--------------------------------------\n";

		//IOsbNsLastMb1b2 - IOsbNsFirstMb1b2
		int IOsbNsDiffMb1b2 = iasi->ancillary_data.word[20] - iasi->ancillary_data.word[19] + 1;

		//IOsbNsLastMb2b3 - IOsbNsFirstMb2b3
		int IOsbNsDiffMb2b3 = iasi->ancillary_data.word[22] - iasi->ancillary_data.word[21] + 1;

		int payload_data_size = (IOsbNsDiffMb1b2 * 2) + (IOsbNsDiffMb2b3 * 2);

		uint16_be *data = new uint16_be[payload_data_size*2];
		copy_n(payload_data.begin(), (payload_data_size*4), (byte*)(data));

		DEBUG("payload_data_size = " << payload_data_size << endl
			  << "IOsbNsDiffMb1b2	= " << IOsbNsDiffMb1b2 << endl
			  << "IOsbNsDiffMb2b3	= " << IOsbNsDiffMb2b3 << endl);

		os << "BArcSpect_b=1" << endl
		   << "\tCounter\t\tValue" << endl;
		ctr = 0;
		for (i=0; i<IOsbNsDiffMb1b2; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1]) << endl;
			ctr = ctr + 2;
		}

		os << "BArcSpect_b=1/2" << endl
		   << "\tCounter\t\tValue"	<< endl;
		for (i=0; i<IOsbNsDiffMb1b2; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1]) << endl;
			ctr = ctr + 2;
		}

		os << "BArcSpect_b=2/3" << endl
		   << "\tCounter\t\tValue" << endl;
		for (i=0; i<IOsbNsDiffMb2b3; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1]) << endl;
			ctr = ctr + 2;
		}

		os << "BArcSpect_b=3" << endl
		   << "\tCounter\t\tValue" << endl;
		for (i=0; i<IOsbNsDiffMb2b3; i++)
		{
			os << setiosflags(ios::fixed) << setfill(' ')
			   << "\t" << i+1
			   << "\t\t" << setw(10) << left << uint16tofloat(data[ctr], data[ctr+1]) << endl;
			ctr = ctr + 2;
		}

		delete[] data;

	}

	else
	{
		os << "\nPacket Identifier not recognised: " << packetID << endl << endl;
	}

}


//
// output_iasi_anc_record
//

void CCSDSPacketDisplayer::output_iasi_anc_record(ostream& os,
												  CCSDS* packet)
{
	CCSDS_IASI_Anc* iasi = (CCSDS_IASI_Anc*)packet->get_data();

	os << "\nAncillary Data\n--------------\n\n"
	   << "IASI Configuration word 1\n"
	   << "\tIASI Chain				: " << static_cast<uint>((iasi->ancillary_data.word[0]>>12)&1) << endl
	   << "\tMAS HAU1 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[0]>>7)&1) << endl
	   << "\tMAS HAU2 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[0]>>6)&1) << endl
	   << "\tMAS HAU3 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[0]>>5)&1) << endl
	   << "\tMAS HAU4 Power Status	: " << static_cast<uint>((iasi->ancillary_data.word[0]>>4)&1) << endl
	   << "\tLaser Identifier		: " << static_cast<uint>(iasi->ancillary_data.word[0]&1) << endl << endl
	   << "IASI Configuration word 2\n"
	   << "\tRC						: " << static_cast<uint>((iasi->ancillary_data.word[1]>>15)&1) << endl
	   << "\tLNR					: " << static_cast<uint>((iasi->ancillary_data.word[1]>>14)&1) << endl
	   << "\tASE					: " << static_cast<uint>((iasi->ancillary_data.word[1]>>13)&1) << endl
	   << "\tLBR					: " << static_cast<uint>((iasi->ancillary_data.word[1]>>12)&1) << endl
	   << "\tPIXEL 1				: " << static_cast<uint>((iasi->ancillary_data.word[1]>>9)&7) << endl
	   << "\tPIXEL 2				: " << static_cast<uint>((iasi->ancillary_data.word[1]>>6)&7) << endl
	   << "\tPIXEL 3				: " << static_cast<uint>((iasi->ancillary_data.word[1]>>3)&7) << endl
	   << "\tPIXEL 4				: " << static_cast<uint>(iasi->ancillary_data.word[1]&7) << endl << endl
	   << "Parameter Table Set Identifier\n"
	   << "\tPTSI (MSW)				: " << iasi->ancillary_data.word[2] << endl
	   << "\tPTSI (LSW)				: " << iasi->ancillary_data.word[3] << endl << endl
	   << "OTM data\n"
	   << "\tBBT					: " << static_cast<uint>((iasi->ancillary_data.word[4]<<16)
															 +(iasi->ancillary_data.word[5])) << endl
	   << "\tInstrument Mode		: " << ntobs(iasi->ancillary_data.word[6], 15) << endl
	   << "\tLine Number			: " << iasi->ancillary_data.word[7] << endl
	   << "\tSQIS					: " << ntobs(iasi->ancillary_data.word[8], 15) << endl
	   << "\tSQII					: " << ntobs(iasi->ancillary_data.word[9], 15) << endl
	   << "\tRTS					: " << static_cast<signed int>(iasi->ancillary_data.word[10]) << endl
	   << "\tRTL					: " << iasi->ancillary_data.word[11] << endl
	   << "\tIFPT					: " << iasi->ancillary_data.word[12] << endl
	   << "\tFPT					: " << iasi->ancillary_data.word[13] << endl
	   << "\tHAUT					: " << iasi->ancillary_data.word[14] << endl
	   << "\tOPBT					: " << iasi->ancillary_data.word[15] << endl
	   << "\tOTM_NV					: \n"
	   << "\t  BBT Validity Flag	: " << static_cast<uint>((iasi->ancillary_data.word[17]>>15)&1) << endl
	   << "\t  FPT Validity Flag	: " << static_cast<uint>((iasi->ancillary_data.word[17]>>14)&1) << endl
	   << "\t  HAUT Validity Flag	: " << static_cast<uint>((iasi->ancillary_data.word[17]>>13)&1) << endl
	   << "\t  IFPT Validity Flag	: " << static_cast<uint>((iasi->ancillary_data.word[17]>>12)&1) << endl
	   << "\t  OBPT Validity Flag	: " << static_cast<uint>((iasi->ancillary_data.word[17]>>11)&1) << endl
	   << "\t  CCAT Validity Flag	: " << static_cast<uint>((iasi->ancillary_data.word[17]>>10)&1) << endl
	   << "\tSPTSI (MSW)			: " << iasi->ancillary_data.word[18] << endl
	   << "\tSPTSI (LSW)			: " << iasi->ancillary_data.word[19] << endl << endl
	   << "DPS Status\n"
	   << "\tDPS Power Status		: \n"
	   << "\t\tDPC1 ON/OFF status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>15)&1) << endl
	   << "\t\tDPC2 ON/OFF status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>14)&1) << endl
	   << "\t\tDPC3 ON/OFF status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>13)&1) << endl
	   << "\t\tDPC4 ON/OFF status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>12)&1) << endl
	   << "\t\tDPC5 ON/OFF status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>11)&1) << endl
	   << "\t\tFMU I/F N status		: " << static_cast<uint>((iasi->ancillary_data.word[22]>>10)&1) << endl
	   << "\t\tFMU I/F R status		: " << static_cast<uint>((iasi->ancillary_data.word[22]>>9)&1) << endl
	   << "\t\tLNR Input status		: " << static_cast<uint>((iasi->ancillary_data.word[22]>>8)&1) << endl
	   << "\tDPS INIT Status		: \n"
	   << "\t\tPX1A/PX1B Init status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>7)&1) << endl
	   << "\t\tPX2A/PX2B Init status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>6)&1) << endl
	   << "\t\tPX3A/PX3B Init status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>5)&1) << endl
	   << "\t\tPX4A/PX4B Init status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>4)&1) << endl
	   << "\t\tDPS DMC SW Init status	: " << static_cast<uint>((iasi->ancillary_data.word[22]>>2)&1) << endl
	   << "\t\tDPS EEPROM to RAM status : " << static_cast<uint>((iasi->ancillary_data.word[22]>>1)&1) << endl
	   << "\t\tDPS DMC RAM check status : " << static_cast<uint>((iasi->ancillary_data.word[22])&1) << endl << endl
	   << "DPS Mode\n"
	   << "\tP1 Mode				: " << static_cast<uint>((iasi->ancillary_data.word[23]>>13)&7) << endl
	   << "\tP2 Mode				: " << static_cast<uint>((iasi->ancillary_data.word[23]>>10)&7) << endl
	   << "\tP3 Mode				: " << static_cast<uint>((iasi->ancillary_data.word[23]>>7)&7) << endl
	   << "\tP4 Mode				: " << static_cast<uint>((iasi->ancillary_data.word[23]>>4)&7) << endl
	   << "\tOP Mode				: " << static_cast<uint>((iasi->ancillary_data.word[23])&15) << endl << endl
	   << "Verification Data Selection (VDS)\n"
	   << "\tPixel Identifier (P)	: " << static_cast<uint>((iasi->ancillary_data.word[24]>>12)&15) << endl
	   << "\tBand Identifier (B)	: " << static_cast<uint>((iasi->ancillary_data.word[24]>>8)&15) << endl
	   << "\tStep Identifier (S)	: " << static_cast<uint>((iasi->ancillary_data.word[24])&255) << endl << endl
	   << "NO_OD_INFO				: " << ntobs(iasi->ancillary_data.word[25], 15) << endl
	   << "							: " << ntobs(iasi->ancillary_data.word[26], 15) << endl << endl
	   << "NO_IIS_INFO				: " << ntobs(iasi->ancillary_data.word[27], 15) << endl
	   << "							: " << ntobs(iasi->ancillary_data.word[28], 15) << endl << endl
	   << "NO_MAS_INFO				: " << ntobs(iasi->ancillary_data.word[29], 15) << endl
	   << "							: " << ntobs(iasi->ancillary_data.word[30], 15) << endl << endl
	   << "ELT1\n"
	   << "\tCD						: " << static_cast<uint>((iasi->ancillary_data.word[31]>>15)&1) << endl
	   << "\tError					: " << static_cast<uint>((iasi->ancillary_data.word[31]>>6)&511) << endl
	   << "\tStep Number			: " << static_cast<uint>((iasi->ancillary_data.word[31])&63) << endl
	   << "\tLine Number			: " << iasi->ancillary_data.word[32] << endl
	   << "\tPixel					: " << static_cast<uint>((iasi->ancillary_data.word[33]>>13)&7) << endl
	   << "\tBand					: " << static_cast<uint>((iasi->ancillary_data.word[33]>>11)&3) << endl
	   << "\tSeverity				: " << static_cast<uint>((iasi->ancillary_data.word[33]>>8)&7) << endl << endl
	   << "ELT2\n"
	   << "\tCD						: " << static_cast<uint>((iasi->ancillary_data.word[34]>>15)&1) << endl
	   << "\tError					: " << static_cast<uint>((iasi->ancillary_data.word[34]>>6)&511) << endl
	   << "\tStep Number			: " << static_cast<uint>((iasi->ancillary_data.word[34])&63) << endl
	   << "\tLine Number			: " << iasi->ancillary_data.word[35] << endl
	   << "\tPixel					: " << static_cast<uint>((iasi->ancillary_data.word[36]>>13)&7) << endl
	   << "\tBand					: " << static_cast<uint>((iasi->ancillary_data.word[36]>>11)&3) << endl
	   << "\tSeverity				: " << static_cast<uint>((iasi->ancillary_data.word[36]>>8)&7) << endl << endl
	   << "ELT3\n"
	   << "\tCD						: " << static_cast<uint>((iasi->ancillary_data.word[37]>>15)&1) << endl
	   << "\tError					: " << static_cast<uint>((iasi->ancillary_data.word[37]>>6)&511) << endl
	   << "\tStep Number			: " << static_cast<uint>((iasi->ancillary_data.word[37])&63) << endl
	   << "\tLine Number			: " << iasi->ancillary_data.word[38] << endl
	   << "\tPixel					: " << static_cast<uint>((iasi->ancillary_data.word[39]>>13)&7) << endl
	   << "\tBand					: " << static_cast<uint>((iasi->ancillary_data.word[39]>>11)&3) << endl
	   << "\tSeverity				: " << static_cast<uint>((iasi->ancillary_data.word[39]>>8)&7) << endl << endl
	   << "ELT4\n"
	   << "\tCD						: " << static_cast<uint>((iasi->ancillary_data.word[40]>>15)&1) << endl
	   << "\tError					: " << static_cast<uint>((iasi->ancillary_data.word[40]>>6)&511) << endl
	   << "\tStep Number			: " << static_cast<uint>((iasi->ancillary_data.word[40])&63) << endl
	   << "\tLine Number			: " << iasi->ancillary_data.word[41] << endl
	   << "\tPixel					: " << static_cast<uint>((iasi->ancillary_data.word[42]>>13)&7) << endl
	   << "\tBand					: " << static_cast<uint>((iasi->ancillary_data.word[42]>>11)&3) << endl
	   << "\tSeverity				: " << static_cast<uint>((iasi->ancillary_data.word[42]>>8)&7) << endl << endl;


	int pixCtr;
	int snCtr;
	int wordCtr = 0;

	os << "\nApplication Data\n----------------\n\n";

	for (pixCtr=1; pixCtr<=4; pixCtr++)
	{
		os << "Main Data Area for Pixel " << pixCtr << endl;

		for (snCtr=32; snCtr<=36; snCtr++)
		{
			if	(snCtr == 34)
			{
				snCtr++;
			}

			os << "Step Number (SN) = " << snCtr << endl
			   << "\tOperational data word 1\n"
			   << "\t\tNumber of Samples (NS) : " << iasi->payload.word[wordCtr] << endl << endl
			   << "\tOperational data word 2\n"
			   << "\t\tStep Number (SN)		  : " << static_cast<uint>((iasi->payload.word[wordCtr+1]>>8)&255) << endl
			   << "\t\tScan Position (SP)	  : " << static_cast<uint>(iasi->payload.word[wordCtr+1]&255) << endl << endl
			   << "\tOperational data word 3\n"
			   << "\t\tCD					  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>15)&1) << endl
			   << "\t\tCSQ					  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>14)&1) << endl
			   << "\t\tSQ1					  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>13)&1) << endl
			   << "\t\tSQ2					  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>12)&1) << endl
			   << "\t\tIEQ					  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>11)&1) << endl
			   << "\t\tSN_NV				  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>7)&1) << endl
			   << "\t\tCD_NV				  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>6)&1) << endl
			   << "\t\tCSQ_NV				  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>5)&1) << endl
			   << "\t\tSP_NV				  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>4)&1) << endl
			   << "\t\tSQ1_NV				  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>3)&1) << endl
			   << "\t\tSQ2_NV				  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>2)&1) << endl
			   << "\t\tNS_NV				  : " << static_cast<uint>((iasi->payload.word[wordCtr+2]>>1)&1) << endl
			   << "\t\tIEQ_NV				  : " << static_cast<uint>(iasi->payload.word[wordCtr+2]&1) << endl << endl
			   << "\tNumber of MAS frames counted by DPS\n"
			   << "\t\tBDcoNbReceivedWords	  : " << iasi->payload.word[wordCtr+3] << endl << endl
			   << "\tMean value of interferogram after nonlinearity correction\n "
			   << "\t\tBNlcAnaMV for Band B1  : " << uint16tofloat(iasi->payload.word[wordCtr+4],
																   iasi->payload.word[wordCtr+5]) << endl
			   << "\t\tBNlcAnaMV for Band B2  : " << uint16tofloat(iasi->payload.word[wordCtr+6],
																   iasi->payload.word[wordCtr+7]) << endl
			   << "\t\tBNlcAnaMV for Band B3  : " << uint16tofloat(iasi->payload.word[wordCtr+8],
																   iasi->payload.word[wordCtr+9]) << endl << endl
			   << "\tNZPD value for the NZPD selected band\n"
			   << "\t\tBZpdNzpd				  : " << iasi->payload.word[wordCtr+10] << endl << endl
			   << "\tNZPD Quality Index\n"
			   << "\t\tBzpdNzpdQualIndex	  : " << uint16tofloat(iasi->payload.word[wordCtr+11],
																   iasi->payload.word[wordCtr+12]) << endl << endl;
			wordCtr = wordCtr + 15;
		}
	}


	wordCtr = 240;
	for (pixCtr=1; pixCtr<=4; pixCtr++)
	{
		os << "Main Status Area for Pixel " << pixCtr << endl;

		for (snCtr=32; snCtr<=36; snCtr++)
		{
			if	(snCtr == 34)
			{
				snCtr++;
			}

			os << "Step Number (SN) = " << snCtr << endl
			   << "\tStatus_area_word_1\n"
			   << "\t\tDPS LNR-Rx DVL flag	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>15)&1) << endl
			   << "\t\tDPS LNR-Rx VLN flag	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>14)&1) << endl
			   << "\t\tBBofFlagSpectNonQual	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>13)&1) << endl
			   << "\t\tBDcoFlagMasErrorPath_B1: " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>12)&1) << endl
			   << "\t\tBDcoFlagMasErrorPath_B2: " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>11)&1) << endl
			   << "\t\tBDcoFlagMasErrorPath_B3: " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>10)&1) << endl
			   << "\t\tBDcoFlagMasOverflow_B1 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>9)&1) << endl
			   << "\t\tBDcoFlagMasOverflow_B2 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>8)&1) << endl
			   << "\t\tBDcoFlagMasOverflow_B3 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>7)&1) << endl
			   << "\t\tBDcoFlagMasEcret_B1	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>6)&1) << endl
			   << "\t\tBDcoFlagMasEcret_B2	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>5)&1) << endl
			   << "\t\tBDcoFlagMasEcret_B3	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>4)&1) << endl
			   << "\t\tBDcoFlagMasErrorNbWords: " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>3)&1) << endl
			   << "\t\tBdcoFlagErrorInterf_B1 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>2)&1) << endl
			   << "\t\tBdcoFlagErrorInterf_B2 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>1)&1) << endl
			   << "\t\tBdcoFlagErrorInterf_B3 : " << static_cast<uint>((iasi->payload.word[wordCtr+1])&1) << endl << endl
			   << "\tStatus_area_word_2\n"
			   << "\t\tBnlcFlagIntegrity_B1	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>15)&1) << endl
			   << "\t\tBnlcFlagIntegrity_B2	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>14)&1) << endl
			   << "\t\tBnlcFlagIntegrity_B3	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>13)&1) << endl
			   << "\t\tBSpkFlagSpik_B1		  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>12)&1) << endl
			   << "\t\tBSpkFlagSpik_B2		  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>11)&1) << endl
			   << "\t\tBSpkFlagSpik_B3		  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>10)&1) << endl
			   << "\t\tBzpdFlagNzpdNonQual	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>9)&1) << endl
			   << "\t\tBIrsFlagSrdNonIntegrity: " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>8)&1) << endl
			   << "\t\tBIsiFlagErrorFft_B1	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>7)&1) << endl
			   << "\t\tBIsiFlagErrorFft_B2	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>6)&1) << endl
			   << "\t\tBIsiFlagErrorFft_B3	  : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>5)&1) << endl << endl;

			wordCtr = wordCtr + 4;
		}

		os << "Extra Status Area\n"
		   << "\tWord 1\n"
		   << "\t\tBBofFlagSrdInit_CD0				 : " << static_cast<uint>(((iasi->payload.word[wordCtr])>>15)&1) << endl
		   << "\t\tBBofFlagSrdInit_CD1				 : " << static_cast<uint>(((iasi->payload.word[wordCtr])>>14)&1) << endl
		   << "\t\tBBofFlagSrdNonUpdate_CD0			 : " << static_cast<uint>(((iasi->payload.word[wordCtr])>>13)&1) << endl
		   << "\t\tBBofFlagSrdNonUpdate_CD1			 : " << static_cast<uint>(((iasi->payload.word[wordCtr])>>12)&1) << endl
		   << "\t\tBBofFlagCoefCalInit_B1_CD0		 : " << static_cast<uint>(((iasi->payload.word[wordCtr])>>11)&1) << endl
		   << "\t\tBBofFlagCoefCalInit_B2_CD0		 : " << static_cast<uint>(((iasi->payload.word[wordCtr])>>10)&1) << endl
		   << "\t\tBBofFlagCoefCalInit_B3_CD0		 : " << static_cast<uint>(((iasi->payload.word[wordCtr])>>9)&1) << endl << endl
		   << "\tWord 2\n"
		   << "\t\tBBofFlagCoefCalInit_B1_CD1		 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>15)&1) << endl
		   << "\t\tBBofFlagCoefCalInit_B2_CD1		 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>14)&1) << endl
		   << "\t\tBBofFlagCoefCalInit_B3_CD1		 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>13)&1) << endl
		   << "\t\tBBofFlagCoefCalNonUpdate_B1_CD0	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>12)&1) << endl
		   << "\t\tBBofFlagCoefCalNonUpdate_B2_CD0	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>11)&1) << endl
		   << "\t\tBBofFlagCoefCalNonUpdate_B3_CD0	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>10)&1) << endl
		   << "\t\tBBofFlagCoefCalNonUpdate_B1_CD1	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>9)&1) << endl
		   << "\t\tBBofFlagCoefCalNonUpdate_B2_CD1	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>8)&1) << endl
		   << "\t\tBBofFlagCoefCalNonUpdate_B3_CD1	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>7)&1) << endl
		   << "\t\tBRciFlagNonIntegritySlope_B1_CD0	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>6)&1) << endl
		   << "\t\tBRciFlagNonIntegritySlope_B2_CD0	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>5)&1) << endl
		   << "\t\tBRciFlagNonIntegritySlope_B3_CD0	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>4)&1) << endl
		   << "\t\tBRciFlagNonIntegritySlope_B1_CD1	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>3)&1) << endl
		   << "\t\tBRciFlagNonIntegritySlope_B2_CD1	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>2)&1) << endl
		   << "\t\tBRciFlagNonIntegritySlope_B3_CD1	 : " << static_cast<uint>(((iasi->payload.word[wordCtr+1])>>1)&1) << endl << endl
		   << "\tWord 3\n"
		   << "\t\tBRciFlagNonIntegrityOffset_B1_CD0 : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>15)&1) << endl
		   << "\t\tBRciFlagNonIntegrityOffset_B2_CD0 : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>14)&1) << endl
		   << "\t\tBRciFlagNonIntegrityOffset_B3_CD0 : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>13)&1) << endl
		   << "\t\tBRciFlagNonIntegrityOffset_B1_CD1 : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>12)&1) << endl
		   << "\t\tBRciFlagNonIntegrityOffset_B2_CD1 : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>11)&1) << endl
		   << "\t\tBRciFlagNonIntegrityOffset_B3_CD1 : " << static_cast<uint>(((iasi->payload.word[wordCtr+2])>>10)&1) << endl << endl
		   << "\tWord 4 (Spare - set to zero)\n\n";

		wordCtr = wordCtr + 4;

	}

}


//
// output_gome_record
//

void CCSDSPacketDisplayer::output_gome_record(ostream& os,
											  CCSDS* packet)
{
	int i, j, k, ctr;
	char tmpChar;

	CCSDS_GOME* gome = (CCSDS_GOME*)packet->get_data();

	// Print out Auxiliary Data common to all four GOME packets

	os << "\nAncillary Data\n--------------\n\n"
	   << "Common Auxiliary Data\n---------------------\n"
	   << "Length of Ancillary Data				  : " << gome->hk[0] << endl
	   << "Fixed Field 1						  : " << ntohs((gome->hk[1]>>8)&255)
	   << ntohs(gome->hk[1]&255) << endl
	   << "Fixed Field 2						  : " << ntohs((gome->hk[2]>>8)&255)
	   << ntohs(gome->hk[2]&255) << endl
	   << "Fixed Field 3						  : " << ntohs((gome->hk[3]>>8)&255)
	   << ntohs(gome->hk[3]&255) << endl
	   << "Fixed Field 4						  : " << ntohs((gome->hk[4]>>8)&255)
	   << ntohs(gome->hk[4]&255) << endl
	   << "GOME Model							  : " << ntohs((gome->hk[5]>>8)&255)
	   << ntohs(gome->hk[5]&255) << endl
	   << "Data Subset Counter					  : " << gome->hk[6] << endl
	   << "ICU SBT at 1st 93.75 ms				  : " << gome->hk[7] << endl
	   << "ICU SBT at 2nd 93.75 ms				  : " << gome->hk[8] << endl
	   << "ICU SBT at 3rd 93.75 ms				  : " << gome->hk[9] << endl
	   << "ICU SBT at 4th 93.75 ms				  : " << gome->hk[10] << endl
	   << "Actual PMD transfer Mode				  : " << gome->hk[11] << endl
	   << "Actual PMD s/p Integration Time Status : " << endl
	   << "\tPMD-s IT Status   : " << static_cast<uint>(gome->hk[12]&3) << endl
	   << "\tPMD-p IT Status   : " << static_cast<uint>((gome->hk[12]>>8)&3) << endl
	   << "Actual PMD-s Integration Time Value	  : " << gome->hk[13] << endl
	   << "Actual PMD-p Integration Time Value	  : " << gome->hk[14] << endl
	   << "Actual PMD Readout Mode				  : " << gome->hk[15] << endl
	   << "Actual FPA co-adding status			  : " << endl
	   << "\t|		 Spare		 |			Readout 1		   |		  Readout 2			 |" << endl
	   << "\t|					 | 1A | 1B | 2A | 2B | 3  | 4  | 1A | 1B | 2A | 2B | 3	| 4	 |" << endl
	   << "\t  " << stretch_8((gome->hk[16]>>8)&255) << stretch_8(gome->hk[16]&255) << endl
	   << "\nIntegration time status readout 1		   : " << endl
	   << "\tFPA 1A IT Status	: " << static_cast<uint>((gome->hk[17]>>10)&3) << endl
	   << "\tFPA 1B IT Status	: " << static_cast<uint>((gome->hk[17]>>8)&3) << endl
	   << "\tFPA 2A IT Status	: " << static_cast<uint>((gome->hk[17]>>6)&3) << endl
	   << "\tFPA 2B IT Status	: " << static_cast<uint>((gome->hk[17]>>4)&3) << endl
	   << "\tFPA 3 IT Status	: " << static_cast<uint>((gome->hk[17]>>2)&3) << endl
	   << "\tFPA 4 IT Status	: " << static_cast<uint>(gome->hk[17]&3) << endl
	   << "Integration time-phase status readout 1	 : " << endl
	   << "\tBand 1A IT Status	: " << static_cast<uint>((gome->hk[18]>>5)&1) << endl
	   << "\tBand 1B IT Status	: " << static_cast<uint>((gome->hk[18]>>4)&1) << endl
	   << "\tBand 2A IT Status	: " << static_cast<uint>((gome->hk[18]>>3)&1) << endl
	   << "\tBand 2B IT Status	: " << static_cast<uint>((gome->hk[18]>>2)&1) << endl
	   << "\tBand 3 IT Status	: " << static_cast<uint>((gome->hk[18]>>1)&1) << endl
	   << "\tBand 4 IT Status	: " << static_cast<uint>(gome->hk[18]&1) << endl
	   << "Integration time value readout 1 band IA	 : " << gome->hk[19] << endl
	   << "Integration time value readout 1 band IB	 : " << gome->hk[20] << endl
	   << "Integration time value readout 1 band IIA : " << gome->hk[21] << endl
	   << "Integration time value readout 1 band IIA : " << gome->hk[22] << endl
	   << "Integration time value readout 1 band III : " << gome->hk[23] << endl
	   << "Integration time value readout 1 band IV	 : " << gome->hk[24] << endl
	   << "Integration time status readout 2		 : " << endl
	   << "\tFPA 1A IT Status	: " << static_cast<uint>((gome->hk[25]>>10)&3) << endl
	   << "\tFPA 1B IT Status	: " << static_cast<uint>((gome->hk[25]>>8)&3) << endl
	   << "\tFPA 2A IT Status	: " << static_cast<uint>((gome->hk[25]>>6)&3) << endl
	   << "\tFPA 2B IT Status	: " << static_cast<uint>((gome->hk[25]>>4)&3) << endl
	   << "\tFPA 3 IT Status	: " << static_cast<uint>((gome->hk[25]>>2)&3) << endl
	   << "\tFPA 4 IT Status	: " << static_cast<uint>(gome->hk[25]&3) << endl
	   << "Integration time-phase status readout 2	 : " << endl
	   << "\tBand 1A IT Status	: " << static_cast<uint>((gome->hk[26]>>5)&1) << endl
	   << "\tBand 1B IT Status	: " << static_cast<uint>((gome->hk[26]>>4)&1) << endl
	   << "\tBand 2A IT Status	: " << static_cast<uint>((gome->hk[26]>>3)&1) << endl
	   << "\tBand 2B IT Status	: " << static_cast<uint>((gome->hk[26]>>2)&1) << endl
	   << "\tBand 3 IT Status	: " << static_cast<uint>((gome->hk[26]>>1)&1) << endl
	   << "\tBand 4 IT Status	: " << static_cast<uint>(gome->hk[26]&1) << endl
	   << "Integration time value readout 2 band IA	 : " << gome->hk[27] << endl
	   << "Integration time value readout 2 band IB	 : " << gome->hk[28] << endl
	   << "Integration time value readout 2 band IIA : " << gome->hk[29] << endl
	   << "Integration time value readout 2 band IIA : " << gome->hk[30] << endl
	   << "Integration time value readout 2 band III : " << gome->hk[31] << endl
	   << "Integration time value readout 2 band IV	 : " << gome->hk[32] << endl
	   << "FPA 1 Latch-up counter					 : " << gome->hk[33] << endl
	   << "FPA 2 Latch-up counter					 : " << gome->hk[34] << endl
	   << "FPA 3 Latch-up counter					 : " << gome->hk[35] << endl
	   << "FPA 4 Latch-up counter					 : " << gome->hk[36] << endl
	   << "PMD-p Latch-up counter					 : " << gome->hk[37] << endl
	   << "PMD-s Latch-up counter					 : " << gome->hk[38] << endl
	   << "HDM Latch-up counter						 : " << gome->hk[39] << endl
	   << "Radiometric (QTH) Lamp Voltage			 : " << gome->hk[40] << endl
	   << "Radiometric (QTH) Lamp Current			 : " << gome->hk[41] << endl
	   << "Spectral Lamp Voltage					 : " << gome->hk[42] << endl
	   << "Spectral Lamp Current					 : " << gome->hk[43] << endl
	   << "FPA1 array Temperature-1					 : " << setiosflags(ios::left) << setiosflags(ios::fixed)
	   << setw(5) << setfill(' ')
	   << gome->hk[44] << "\t"
	   << gome->hk[45] << "\t"
	   << gome->hk[46] << "\t"
	   << gome->hk[47] << endl
	   << "FPA2 array Temperature-1					 : " << setw(5) << setfill(' ')
	   << gome->hk[48] << "\t"
	   << gome->hk[49] << "\t"
	   << gome->hk[50] << "\t"
	   << gome->hk[51] << endl
	   << "FPA3 array Temperature-1					 : " << setw(5) << setfill(' ')
	   << gome->hk[52] << "\t"
	   << gome->hk[53] << "\t"
	   << gome->hk[54] << "\t"
	   << gome->hk[55] << endl
	   << "FPA4 array Temperature-1					 : " << setw(5) << setfill(' ')
	   << gome->hk[56] << "\t"
	   << gome->hk[57] << "\t"
	   << gome->hk[58] << "\t"
	   << gome->hk[59] << endl
	   << "FPA1 array Temperature-2					 : " << setw(5) << setfill(' ')
	   << gome->hk[60] << "\t"
	   << gome->hk[61] << "\t"
	   << gome->hk[62] << "\t"
	   << gome->hk[63] << endl
	   << "FPA2 array Temperature-2					 : " << setw(5) << setfill(' ')
	   << gome->hk[64] << "\t"
	   << gome->hk[65] << "\t"
	   << gome->hk[66] << "\t"
	   << gome->hk[67] << endl
	   << "FPA3 array Temperature-2					 : " << setw(5) << setfill(' ')
	   << gome->hk[68] << "\t"
	   << gome->hk[69] << "\t"
	   << gome->hk[70] << "\t"
	   << gome->hk[71] << endl
	   << "FPA4 array Temperature-2					 : " << setw(5) << setfill(' ')
	   << gome->hk[72] << "\t"
	   << gome->hk[73] << "\t"
	   << gome->hk[74] << "\t"
	   << gome->hk[75] << endl
	   << "FPA1 Peltier Loop Output					 : " << setw(5) << setfill(' ')
	   << gome->hk[76] << "\t"
	   << gome->hk[77] << "\t"
	   << gome->hk[78] << "\t"
	   << gome->hk[79] << endl
	   << "FPA2 Peltier Loop Output					 : " << setw(5) << setfill(' ')
	   << gome->hk[80] << "\t"
	   << gome->hk[81] << "\t"
	   << gome->hk[82] << "\t"
	   << gome->hk[83] << endl
	   << "FPA3 Peltier Loop Output					 : " << setw(5) << setfill(' ')
	   << gome->hk[84] << "\t"
	   << gome->hk[85] << "\t"
	   << gome->hk[86] << "\t"
	   << gome->hk[87] << endl
	   << "FPA4 Peltier Loop Output					 : " << setw(5) << setfill(' ')
	   << gome->hk[88] << "\t"
	   << gome->hk[89] << "\t"
	   << gome->hk[90] << "\t"
	   << gome->hk[91] << endl
	   << "FPA1 Peltier Loop Error					 : " << gome->hk[92] << endl
	   << "FPA2 Peltier Loop Error					 : " << gome->hk[93] << endl
	   << "FPA3 Peltier Loop Error					 : " << gome->hk[94] << endl
	   << "FPA4 Peltier Loop Error					 : " << gome->hk[95] << endl
	   << "Scan Mirror Position 1 to 4				 : " << setw(5) << setfill(' ')
	   << gome->hk[96] << "\t"
	   << gome->hk[97] << "\t"
	   << gome->hk[98] << "\t"
	   << gome->hk[99] << endl
	   << "Scan Mirror Torque 1 to 4				 : " << setw(5) << setfill(' ')
	   << gome->hk[100] << "\t"
	   << gome->hk[101] << "\t"
	   << gome->hk[102] << "\t"
	   << gome->hk[103] << endl
	   << "Scan Mirror Torque Shift 1 to 4			 : " << setw(5) << setfill(' ')
	   << gome->hk[104] << "\t"
	   << gome->hk[105] << "\t"
	   << gome->hk[106] << "\t"
	   << gome->hk[107] << endl;


	ctr = 108;
	for (i=0; i<4; i++)
	{
		os << "FPA" << i+1 << " Samples								 : ";
		for (j=0; j<3; j++)
		{
			os << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(5) << setfill(' ')
			   << gome->hk[ctr] << "\t"
			   << gome->hk[ctr+1] << "\t"
			   << gome->hk[ctr+2] << "\t"
			   << gome->hk[ctr+3] << endl;
			ctr = ctr + 4;
			if (j<2)
			{
				os << "											   ";
			}
		}
	}

	os << "PMD-p Sample								 : " << setiosflags(ios::left) << setiosflags(ios::fixed)
	   << setw(5) << setfill(' ')
	   << gome->hk[156] << "\t"
	   << gome->hk[157] << "\t"
	   << gome->hk[158] << "\t"
	   << gome->hk[159] << endl
	   << "PMD-s Sample								 : " << setw(5) << setfill(' ')
	   << gome->hk[160] << "\t"
	   << gome->hk[161] << "\t"
	   << gome->hk[162] << "\t"
	   << gome->hk[163] << endl
	   << "Processor ID								 : " << gome->hk[164] << endl
	   << "Dump Length								 : " << gome->hk[165] << endl
	   << "Dump Start Address MSW					 : " << gome->hk[166] << endl
	   << "Dump Start Address LSW					 : " << gome->hk[167] << endl
	   << "SU prc dump start address				 : " << setw(5) << setfill(' ')
	   << gome->hk[170] << "\t"
	   << gome->hk[171] << "\t"
	   << gome->hk[172] << "\t"
	   << gome->hk[173] << endl
	   << "SU prc number of dumped words			 : " << gome->hk[174] << endl
	   << "SU prc memory dump						 : " << setw(5) << setfill(' ')
	   << gome->hk[175] << "\t"
	   << gome->hk[176] << "\t"
	   << gome->hk[177] << "\t"
	   << gome->hk[178] << endl
	   << "											   " << gome->hk[179] << "\t"
	   << gome->hk[180] << "\t"
	   << gome->hk[181] << "\t"
	   << gome->hk[182] << endl
	   << "											   " << gome->hk[183] << "\t"
	   << gome->hk[184] << "\t"
	   << gome->hk[185] << "\t"
	   << gome->hk[186] << endl
	   << "											   " << gome->hk[187] << "\t"
	   << gome->hk[188] << "\t"
	   << gome->hk[189] << "\t"
	   << gome->hk[190] << endl
	   << "UTCo										 : " << setw(5) << setfill(' ')
	   << gome->hk[191] << "\t"
	   << gome->hk[192] << "\t"
	   << gome->hk[193] << "\t"
	   << gome->hk[194] << endl
	   << "OBTo										 : " << setw(5) << setfill(' ')
	   << gome->hk[195] << "\t"
	   << gome->hk[196] << "\t"
	   << gome->hk[197] << endl
	   << "Ts										 : " << setw(5) << setfill(' ')
	   << gome->hk[198] << "\t"
	   << gome->hk[199] << endl
	   << endl;


	// End of Auxiliary Data common to all four GOME packets

	int subsetCtr = static_cast<int>(gome->hk[6]);

	if(subsetCtr == 0 || subsetCtr == 4 || subsetCtr == 8 || subsetCtr == 12)
	{

		// Print out Auxiliary Data spacific to the first GOME packet

		os << "\nAuxiliary Data spacific to the first GOME packet\n------------------------------------------------\n"
		   << "Instrument S/S Mode							   : " << setiosflags(ios::left) << setiosflags(ios::fixed)
		   << setw(5) << setfill(' ')
		   << gome->hk[210] << "\t"
		   << gome->hk[211] << "\t"
		   << gome->hk[212] << endl
		   << "CDHU ICU										   : " << setw(5) << setfill(' ')
		   << gome->hk[213] << "\t"
		   << gome->hk[214] << "\t"
		   << gome->hk[215] << endl
		   << "													 " << gome->hk[216] << "\t"
		   << gome->hk[217] << "\t"
		   << gome->hk[218] << endl
		   << "													 " << gome->hk[219] << "\t"
		   << gome->hk[220] << "\t"
		   << gome->hk[221] << endl
		   << "													 " << gome->hk[222] << "\t"
		   << gome->hk[223] << endl
		   << "Instrument Operational Mode					   : " << gome->hk[224] << endl
		   << "2nd prc words								   : " << setw(5) << setfill(' ')
		   << gome->hk[225] << "\t"
		   << gome->hk[226] << endl
		   << "Timeline sequence report						   : " << endl
		   << "\tProcess		 : " << static_cast<uint>((gome->hk[231]>>15)&1) << endl
		   << "\tStatus			 : " << static_cast<uint>((gome->hk[231]>>14)&1) << endl
		   << "\tGTL Identifier	 : " << static_cast<uint>((gome->hk[231]>>10)&15) << endl
		   << "\tLoop Number	 : " << static_cast<uint>((gome->hk[231]>>6)&15) << endl
		   << "\tLine Number	 : " << static_cast<uint>(gome->hk[231]&63) << endl
		   << "GTT sequence report							   : " << endl
		   << "\tProcess		 : " << static_cast<uint>((gome->hk[232]>>15)&1) << endl
		   << "\tStatus			 : " << static_cast<uint>((gome->hk[232]>>14)&1) << endl
		   << "\tGTT Identifier	 : " << static_cast<uint>((gome->hk[232]>>10)&15) << endl
		   << "\tLine Number	 : " << static_cast<uint>(gome->hk[232]&63) << endl
		   << "Last routine command word 0					   : " << gome->hk[233] << endl
		   << "Last routine command word 1					   : " << gome->hk[234] << endl
		   << "Last routine command word 2					   : " << gome->hk[235] << endl
		   << "Last routine command word 3					   : " << gome->hk[236] << endl
		   << "Last routine command word 4					   : " << gome->hk[237] << endl
		   << "Last routine command word 5					   : " << gome->hk[238] << endl
		   << "Last routine command expansion time ICU MSW	   : " << gome->hk[239] << endl
		   << "Last routine command expansion time ICU LSW	   : " << gome->hk[240] << endl
		   << "Last non-routine command word 0				   : " << gome->hk[241] << endl
		   << "Last non-routine command word 1				   : " << gome->hk[242] << endl
		   << "Last non-routine command word 2				   : " << gome->hk[243] << endl
		   << "Last non-routine command word 3				   : " << gome->hk[244] << endl
		   << "Last non-routine command word 4				   : " << gome->hk[245] << endl
		   << "Last non-routine command word 5				   : " << gome->hk[246] << endl
		   << "Last non-routine command expansion time ICU MSW : " << gome->hk[247] << endl
		   << "Last non-routine command expansion time ICU LSW : " << gome->hk[248] << endl
		   << "Next timeline command word 0					   : " << gome->hk[249] << endl
		   << "Next timeline command word 1					   : " << gome->hk[250] << endl
		   << "Next timeline command word 2					   : " << gome->hk[251] << endl
		   << "Next timeline command word 3					   : " << gome->hk[252] << endl
		   << "Next timeline command word 4					   : " << gome->hk[253] << endl
		   << "Next timeline command word 5					   : " << gome->hk[254] << endl
		   << "Time of next timeline command MSW			   : " << gome->hk[255] << endl
		   << "Time of next timeline command LSW			   : " << gome->hk[256] << endl
		   << "Monitoring flags								   : " << setw(5) << setfill(' ')
		   << gome->hk[265] << "\t"
		   << gome->hk[266] << "\t"
		   << gome->hk[267] << "\t"
		   << gome->hk[268] << endl
		   << "Switch-down flags							   : " << setw(5) << setfill(' ')
		   << gome->hk[269] << "\t"
		   << gome->hk[270] << "\t"
		   << gome->hk[271] << "\t"
		   << gome->hk[272] << endl
		   << "Min/Max reporting enable/disable flag		   : " << setw(5) << setfill(' ')
		   << gome->hk[277] << "\t"
		   << gome->hk[278] << "\t"
		   << gome->hk[279] << "\t"
		   << gome->hk[280] << endl
		   << "CDHU temp 1 (DC/DC1 - Power Supply)			   : " << gome->hk[327] << endl
		   << "CDHU temp 2 (S&HS)							   : " << gome->hk[328] << endl
		   << "CDHU temp 3 (HDM board)						   : " << gome->hk[329] << endl
		   << "CDHU temp 4 (SDM board)						   : " << gome->hk[330] << endl
		   << "CDHU temp 5 (FTHM)							   : " << gome->hk[331] << endl
		   << "GPDU EQ DC/DC temp							   : " << gome->hk[332] << endl
		   << "Radiator Temperature							   : " << gome->hk[333] << endl
		   << "Scan Motor  Temperature						   : " << gome->hk[334] << endl
		   << "Scan Unit Electronics  Temperature			   : " << gome->hk[335] << endl
		   << "Radiometric Lamp	 Temperature				   : " << gome->hk[336] << endl
		   << "Radiometric Unit DC/DC  Temperature			   : " << gome->hk[337] << endl
		   << "Spectral Lamp Temperature					   : " << gome->hk[338] << endl
		   << "Spectral DC/DC Temperature					   : " << gome->hk[339] << endl
		   << "Sun Diffuser Temperature						   : " << gome->hk[340] << endl
		   << "Shutter Motor Temperature					   : " << gome->hk[341] << endl
		   << "OptBench under CU Temperature				   : " << gome->hk[342] << endl
		   << "Center of OptBench Temperature				   : " << gome->hk[343] << endl
		   << "+Z side OptBench Temperature					   : " << gome->hk[344] << endl
		   << "-X side OptBench Temperature					   : " << gome->hk[345] << endl
		   << "-Z side OptBench Temperature					   : " << gome->hk[346] << endl
		   << "Predisperser Prism Temperature				   : " << gome->hk[347] << endl
		   << "Analog board offset value (0.1%)				   : " << gome->hk[348] << endl
		   << "Analog board gain  value (90%)				   : " << gome->hk[349] << endl
		   << "FPA1 module temperature 1					   : " << gome->hk[350] << endl
		   << "FPA2 module temperature 1					   : " << gome->hk[351] << endl
		   << "FPA3 module temperature 1					   : " << gome->hk[352] << endl
		   << "FPA4 module temperature 1					   : " << gome->hk[353] << endl
		   << "FPA1 module temperature 2					   : " << gome->hk[354] << endl
		   << "FPA2 module temperature 2					   : " << gome->hk[355] << endl
		   << "FPA3 module temperature 2					   : " << gome->hk[356] << endl
		   << "FPA4 module temperature 2					   : " << gome->hk[357] << endl
		   << "PMD-p array temperature						   : " << gome->hk[358] << endl
		   << "PMD-s array temperature						   : " << gome->hk[359] << endl
		   << "PMD-p module temperature						   : " << gome->hk[360] << endl
		   << "PMD-s module temperature						   : " << gome->hk[361] << endl
		   << "Scan mirror requested mode (from ICU software)  : " << gome->hk[366] << endl
		   << "Scan mirror requested bias (from ICU software)  : " << gome->hk[367] << endl
		   << "Scan mirror real mode (from SU TLM OPMODE)	   : " << gome->hk[368] << endl
		   << "Scan mirror real bias (from SU TLM BIAS/POS)	   : " << gome->hk[369] << endl
		   << "SU Status Word								   : " << gome->hk[370] << endl
		   << "SU Biases									   : " << setw(5) << setfill(' ')
		   << gome->hk[371] << "\t"
		   << gome->hk[372] << "\t"
		   << gome->hk[373] << "\t"
		   << gome->hk[374] << endl
		   << "SU CDHU protocol								   : " << gome->hk[375] << endl
		   << "PMD-p band 1 to 15 start						   : " << setw(5) << setfill(' ')
		   << gome->hk[376] << "\t"
		   << gome->hk[377] << "\t"
		   << gome->hk[378] << "\t"
		   << gome->hk[379] << "\t"
		   << gome->hk[380] << endl
		   << "													 " << gome->hk[381] << "\t"
		   << gome->hk[382] << "\t"
		   << gome->hk[383] << "\t"
		   << gome->hk[384] << "\t"
		   << gome->hk[385] << endl
		   << "													 " << gome->hk[386] << "\t"
		   << gome->hk[387] << "\t"
		   << gome->hk[388] << "\t"
		   << gome->hk[389] << "\t"
		   << gome->hk[390] << endl
		   << "PMD-p nb pixel in band 1 to 15				   : " << setw(5) << setfill(' ')
		   << gome->hk[391] << "\t"
		   << gome->hk[392] << "\t"
		   << gome->hk[393] << "\t"
		   << gome->hk[394] << "\t"
		   << gome->hk[395] << endl
		   << "													 " << gome->hk[396] << "\t"
		   << gome->hk[397] << "\t"
		   << gome->hk[398] << "\t"
		   << gome->hk[399] << "\t"
		   << gome->hk[400] << endl
		   << "													 " << gome->hk[401] << "\t"
		   << gome->hk[402] << "\t"
		   << gome->hk[403] << "\t"
		   << gome->hk[404] << "\t"
		   << gome->hk[405] << endl
		   << "PMD-s band 1 to 15 start						   : " << setw(5) << setfill(' ')
		   << gome->hk[406] << "\t"
		   << gome->hk[407] << "\t"
		   << gome->hk[408] << "\t"
		   << gome->hk[409] << "\t"
		   << gome->hk[410] << endl
		   << "													 " << gome->hk[411] << "\t"
		   << gome->hk[412] << "\t"
		   << gome->hk[413] << "\t"
		   << gome->hk[414] << "\t"
		   << gome->hk[415] << endl
		   << "													 " << gome->hk[416] << "\t"
		   << gome->hk[417] << "\t"
		   << gome->hk[418] << "\t"
		   << gome->hk[419] << "\t"
		   << gome->hk[420] << endl
		   << "PMD-s nb pixel in band 1 to 15				   : " << setw(5) << setfill(' ')
		   << gome->hk[421] << "\t"
		   << gome->hk[422] << "\t"
		   << gome->hk[423] << "\t"
		   << gome->hk[424] << "\t"
		   << gome->hk[425] << endl
		   << "													 " << gome->hk[426] << "\t"
		   << gome->hk[427] << "\t"
		   << gome->hk[428] << "\t"
		   << gome->hk[429] << "\t"
		   << gome->hk[430] << endl
		   << "													 " << gome->hk[431] << "\t"
		   << gome->hk[432] << "\t"
		   << gome->hk[433] << "\t"
		   << gome->hk[434] << "\t"
		   << gome->hk[435] << endl
		   << "PMD-p start of short wavelength rdout		   : " << gome->hk[436] << endl
		   << "PMD-s start of short wavelength rdout		   : " << gome->hk[438] << endl
		   << "Xlate bits									   : " << endl
		   << "\tFPA0 Xlate bit	   : " << static_cast<uint>((gome->hk[439]>>15)&1) << endl
		   << "\tFPA1 Xlate bit	   : " << static_cast<uint>((gome->hk[439]>>14)&1) << endl
		   << "\tFPA2 Xlate bit	   : " << static_cast<uint>((gome->hk[439]>>13)&1) << endl
		   << "\tFPA3 Xlate bit	   : " << static_cast<uint>((gome->hk[439]>>12)&1) << endl
		   << "\tPMD-p Xlate bit   : " << static_cast<uint>((gome->hk[439]>>11)&1) << endl
		   << "\tPMD-s Xlate bit   : " << static_cast<uint>((gome->hk[439]>>10)&1) << endl
		   << "Programmed Integration Time Band IA			   : " << gome->hk[440] << endl
		   << "Programmed Integration Time Band IIA			   : " << gome->hk[441] << endl
		   << "Programmed Integration Time Band IB			   : " << gome->hk[442] << endl
		   << "Programmed Integration Time Band IIB			   : " << gome->hk[443] << endl
		   << "Programmed Integration Time Band III			   : " << gome->hk[444] << endl
		   << "Programmed Integration Time Band IV			   : " << gome->hk[445] << endl
		   << "ICU EEPROM memory checksum					   : " << gome->hk[446] << endl
		   << "Science processor EEPROM memory checksum		   : " << gome->hk[447] << endl
		   << endl;


		// End of Auxiliary Data spacific to the first GOME packet

	}


	if(subsetCtr == 1 || subsetCtr == 5 || subsetCtr == 9 || subsetCtr == 13)
	{

		// Print out Auxiliary Data spacific to the second GOME packet

		os << "\nAuxiliary Data spacific to the second GOME packet\n-------------------------------------------------\n"
		   << "Previous Timeline  : \n"
		   << "Line \tWord1\tWord2\tWord3\tWord4\tWord5\tWord6\tWord7\n";

		ctr = 210;
		for (i=0; i<33; i++)
		{
			os << setiosflags(ios::left) << setiosflags(ios::fixed) << setw(5) << setfill(' ') << (i+1);
			for (j=0; j<7; j++)
			{
				os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
				   << setw(5) << setfill(' ') << gome->hk[ctr];
				ctr++;
			}
			os << endl;
		}
		os << endl;

		// End of Auxiliary Data spacific to the second GOME packet

	}


	if(subsetCtr == 2 || subsetCtr == 6 || subsetCtr == 10 || subsetCtr == 14)
	{

		// Print out Auxiliary Data spacific to the third GOME packet

		os << "\nAuxiliary Data spacific to the third GOME packet\n------------------------------------------------\n"
		   << "Present Timeline	 : \n"
		   << "Line \tWord1\tWord2\tWord3\tWord4\tWord5\tWord6\tWord7\n";

		ctr = 210;
		for (i=0; i<33; i++)
		{
			os << setiosflags(ios::left) << setiosflags(ios::fixed) << setw(5) << setfill(' ') << (i+1);
			for (j=0; j<7; j++)
			{
				os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
				   << setw(5) << setfill(' ') <<  gome->hk[ctr];
				ctr++;
			}
			os << endl;
		}
		os << endl;


		// End of Auxiliary Data spacific to the third GOME packet

	}


	if(subsetCtr == 3 || subsetCtr == 7 || subsetCtr == 11 || subsetCtr == 15)
	{

		// Print out Auxiliary Data spacific to the fourth GOME packet

		os << "\nAuxiliary Data spacific to the fourth GOME packet\n-------------------------------------------------\n\n"
		   << "GTT Contents\n------------\n"
		   << "Line \tTimeline ID\tStart Time\n";

		ctr = 210;
		for (i=0; i<28; i++)
		{
			os << setiosflags(ios::left) << setiosflags(ios::fixed) << setw(5) << setfill(' ') << (i+1);

			os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(10) << setfill(' ') <<  static_cast<uint>((gome->hk[ctr+1])&255);

			os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(10) << setfill(' ') <<  static_cast<unsigned long>( ((gome->hk[ctr]&65535)<<8)
																		   | ((gome->hk[ctr+1]>>8)&255) );
			ctr = ctr + 2;

//		for (j=0; j<2; j++)
//		{
//		os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
//		   << setw(5) << setfill(' ') <<  gome->hk[ctr];
//		ct++;
//		}
			os << endl;
		}
		os << endl;


		os << "DSM Table\n---------\n"
		   << "Shutter Emergency Enable/Disable			 : " << gome->hk[266] << endl
		   << "Shutter Movement Enable/Disable			 : " << gome->hk[267] << endl
		   << "Channel 1 Band Separation				 : " << gome->hk[268] << endl
		   << "Channel 2 Band Separation				 : " << gome->hk[269] << endl
		   << "Cooler 1 Cooling Enable/Disable			 : " << gome->hk[270] << endl
		   << "Cooler 1 Annealing Enable/Disable		 : " << gome->hk[271] << endl
		   << "Cooler 1 set cold target temperature		 : " << gome->hk[272] << endl
		   << "Cooler 1 set hot target temperature		 : " << gome->hk[273] << endl
		   << "Cooler 2 Cooling Enable/Disable			 : " << gome->hk[274] << endl
		   << "Cooler 2 Annealing Enable/Disable		 : " << gome->hk[275] << endl
		   << "Cooler 2 set cold target temperature		 : " << gome->hk[276] << endl
		   << "Cooler 2 set hot target temperature		 : " << gome->hk[277] << endl
		   << "Cooler 3 Cooling Enable/Disable			 : " << gome->hk[278] << endl
		   << "Cooler 3 Annealing Enable/Disable		 : " << gome->hk[279] << endl
		   << "Cooler 3 set cold target temperature		 : " << gome->hk[280] << endl
		   << "Cooler 3 set hot target temperature		 : " << gome->hk[281] << endl
		   << "Cooler 4 Cooling Enable/Disable			 : " << gome->hk[282] << endl
		   << "Cooler 4 Annealing Enable/Disable		 : " << gome->hk[283] << endl
		   << "Cooler 4 set cold target temperature		 : " << gome->hk[284] << endl
		   << "Cooler 4 set hot target temperature		 : " << gome->hk[285] << endl
		   << "End duration flag 93.75 ms / 6 s			 : " << gome->hk[286] << endl
		   << "TEC always on							 : " << gome->hk[287] << endl
		   << "Lamp Interlock Enable/Disable			 : " << gome->hk[288] << endl
		   << "FPA 1 readout sequence					 : " << gome->hk[289] << endl
		   << "FPA 2 readout sequence					 : " << gome->hk[290] << endl
		   << "FPA 3 readout sequence					 : " << gome->hk[291] << endl
		   << "FPA 4 readout sequence					 : " << gome->hk[292] << endl
		   << "PMD-s readout sequence					 : " << gome->hk[293] << endl
		   << "PMD-p readout sequence					 : " << gome->hk[294] << endl
		   << "Default Integration Time Band IA (FPA 1)	 : " << gome->hk[295] << endl
		   << "Default Integration Time Band IB (FPA 1)	 : " << gome->hk[296] << endl
		   << "Default Integration Time Band IIA (FPA 2) : " << gome->hk[297] << endl
		   << "Default Integration Time Band IIB (FPA 2) : " << gome->hk[298] << endl
		   << "Default Integration Time Band III (FPA 3) : " << gome->hk[299] << endl
		   << "Default Integration Time Band IV (FPA 4)	 : " << gome->hk[300] << endl
		   << "Direct Command Enable/Disable			 : " << gome->hk[301] << endl
		   << "PMD readout mode							 : " << gome->hk[302] << endl
		   << "FPA shortest integration time mode		 : " << gome->hk[303] << endl
		   << "PMD Coolers Enable/Disable				 : " << gome->hk[304] << endl
		   << "PMD Line									 : " << gome->hk[305] << endl
		   << "PMD IT									 : " << gome->hk[306] << endl
		   << "Standby/Refuse on Switch Fault			 : " << gome->hk[307] << endl
		   << "PMD reset cycle of flyback				 : " << gome->hk[308] << endl
		   << "Max history entries						 : " << gome->hk[309] << endl
		   << "FPA/PMD switch on						 : " << gome->hk[310] << endl
		   << "Scan Mirror Nadir Bias					 : " << gome->hk[311] << endl
		   << "Scan Mirror North Pole Bias				 : " << gome->hk[312] << endl
		   << "Scan Mirror South Pole Bias				 : " << gome->hk[313] << endl
		   << "Scan Mirror Programmable Bias			 : " << gome->hk[314] << endl
		   << "Set qth current							 : " << gome->hk[315] << endl
		   << "FPA 1 Sensors Enable						 : " << gome->hk[316] << endl
		   << "FPA 2 Sensors Enable						 : " << gome->hk[317] << endl
		   << "FPA 3 Sensors Enable						 : " << gome->hk[318] << endl
		   << "FPA 4 Sensors Enable						 : " << gome->hk[319] << endl
		   << "Patch Enable Flag						 : " << gome->hk[320] << endl
		   << "PMD transfer mode						 : " << gome->hk[323] << endl
		   << "PMD-s channel B start					 : " << gome->hk[324] << endl
		   << "PMD-s channel B length					 : " << gome->hk[325] << endl
		   << "PMD-p channel B start					 : " << gome->hk[326] << endl
		   << "PMD-p channel B length					 : " << gome->hk[327] << endl
		   << endl;


		// End of Auxiliary Data spacific to the fourth GOME packet

	}


	os << "\nApplication Data\n----------------\n\n"
	   << "PMD Data\n--------\n";


	os << "PMD short wavelength indicator (fixed value)	 : " << gome->pmd[0] << endl
	   << "PMD short wavelength pixels (6 seconds)		 : \n\tCount\tValues\n";

	ctr = 1;
	for (i=0; i<14; i++)
	{
		os << setiosflags(ios::right) << setiosflags(ios::fixed)
		   << setw(5) << setfill(' ') << "\t" << ctr << "\t"
		   << gome->pmd[ctr] << "\t"
		   << gome->pmd[ctr+1] << "\t"
		   << gome->pmd[ctr+2] << "\t"
		   << gome->pmd[ctr+3] << "\t"
		   << gome->pmd[ctr+4] << "\t"
		   << endl;
		ctr = ctr + 5;
	}

	os	<< "PMD readout indicator						  : \n"
		<< "\tPattern				  : " << ntobs(static_cast<uint>((gome->pmd[71]>>12)&15), 3) << endl
		<< "\tFrame ID				  : " << ntobs(static_cast<uint>((gome->pmd[71]>>8)&15), 3) << endl
		<< "\tData Subset Counter	  : " << ntobs(static_cast<uint>((gome->pmd[71]>>4)&15), 3) << endl
		<< "\tPacket Sequence Control : " << ntobs(static_cast<uint>(gome->pmd[71]&15), 3) << endl;


	int pmd_transfer_mode = static_cast<int>(gome->hk[11]);
	int pmd_readout_mode = static_cast<int>(gome->hk[15]);


	if ((pmd_transfer_mode == 1) && (pmd_readout_mode != 2))
	{
		os << "\nPMD Mode 1 (co-added)\n---------------------\n";

		ctr = 72;
		for (i=0; i<16; i++)
		{
			for (j=0; j<2; j++)
			{
				if (j==0) tmpChar = 'p';
				if (j==1) tmpChar = 's';

				os << "PMD-" << tmpChar << " readout " << i << " bands 1-4 status word 1	: \n"
				   << "\tShift for band 1  : " << static_cast<uint>((gome->pmd[ctr]>>12)&15) << endl
				   << "\tShift for band 2  : " << static_cast<uint>((gome->pmd[ctr]>>8)&15) << endl
				   << "\tShift for band 3  : " << static_cast<uint>((gome->pmd[ctr]>>4)&15) << endl
				   << "\tShift for band 4  : " << static_cast<uint>(gome->pmd[ctr]&15) << endl
				   << "PMD-" << tmpChar << " readout " << i << " bands 5-8 status word 2	: \n"
				   << "\tShift for band 5  : " << static_cast<uint>((gome->pmd[ctr+1]>>12)&15) << endl
				   << "\tShift for band 6  : " << static_cast<uint>((gome->pmd[ctr+1]>>8)&15) << endl
				   << "\tShift for band 7  : " << static_cast<uint>((gome->pmd[ctr+1]>>4)&15) << endl
				   << "\tShift for band 8  : " << static_cast<uint>(gome->pmd[ctr+1]&15) << endl
				   << "PMD-" << tmpChar << " readout " << i << " bands 9-12 status word 3	: \n"
				   << "\tShift for band 9  : " << static_cast<uint>((gome->pmd[ctr+2]>>12)&15) << endl
				   << "\tShift for band 10 : " << static_cast<uint>((gome->pmd[ctr+2]>>8)&15) << endl
				   << "\tShift for band 11 : " << static_cast<uint>((gome->pmd[ctr+2]>>4)&15) << endl
				   << "\tShift for band 12 : " << static_cast<uint>(gome->pmd[ctr+2]&15) << endl
				   << "PMD-" << tmpChar << " readout " << i << " bands 13-15 status word 4	: \n"
				   << "\tShift for band 13 : " << static_cast<uint>((gome->pmd[ctr+3]>>12)&15) << endl
				   << "\tShift for band 14 : " << static_cast<uint>((gome->pmd[ctr+3]>>8)&15) << endl
				   << "\tShift for band 15 : " << static_cast<uint>((gome->pmd[ctr+3]>>4)&15) << endl
				   << "PMD-" << tmpChar << " readout " << i << " bands 1-15					: \n"
				   << "\tCount\tValues\n"
				   << setiosflags(ios::right) << setiosflags(ios::fixed)
				   << setw(5) << setfill(' ')
				   << "\t1\t"
				   << gome->pmd[ctr+4] << "\t"
				   << gome->pmd[ctr+5] << "\t"
				   << gome->pmd[ctr+6] << "\t"
				   << gome->pmd[ctr+7] << "\t"
				   << gome->pmd[ctr+8] << endl
				   << "\t6\t"
				   << gome->pmd[ctr+9] << "\t"
				   << gome->pmd[ctr+10] << "\t"
				   << gome->pmd[ctr+11] << "\t"
				   << gome->pmd[ctr+12] << "\t"
				   << gome->pmd[ctr+13] << endl
				   << "\t11\t"
				   << gome->pmd[ctr+14] << "\t"
				   << gome->pmd[ctr+15] << "\t"
				   << gome->pmd[ctr+16] << "\t"
				   << gome->pmd[ctr+17] << "\t"
				   << gome->pmd[ctr+18] << endl;

				ctr = ctr + 19;
			}
		}

	}

	else if ((pmd_transfer_mode == 2) && (pmd_readout_mode != 2))
	{
		os << "\nPMD Mode 2\n----------\n"
		   << "Raw Data	 : \n"
		   << "Counter\tPMD-p\tPMD-s\n";

		for (i=0; i<76; i++)
		{
			os << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(5) << setfill(' ') << (i+1);

			os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(5) << setfill(' ') << gome->pmd[i+72];

			os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(5) << setfill(' ') << gome->pmd[i+148] << endl;
		}

		os << endl << "Co-added Data  : \n";

		ctr = 224;
		for (i=0; i<12; i++)
		{
			for (j=0; j<2; j++)
			{
				if (j==0) tmpChar = 'p';
				if (j==1) tmpChar = 's';

				os << "PMD-" << tmpChar << " readout " << i << " bands 1-4 status word 1	: \n"
				   << "\tShift for band 1  : " << static_cast<uint>((gome->pmd[ctr]>>12)&15) << endl
				   << "\tShift for band 2  : " << static_cast<uint>((gome->pmd[ctr]>>8)&15) << endl
				   << "\tShift for band 3  : " << static_cast<uint>((gome->pmd[ctr]>>4)&15) << endl
				   << "\tShift for band 4  : " << static_cast<uint>(gome->pmd[ctr]&15) << endl
				   << "PMD-" << tmpChar << " readout " << i << " bands 5-8 status word 2	: \n"
				   << "\tShift for band 5  : " << static_cast<uint>((gome->pmd[ctr+1]>>12)&15) << endl
				   << "\tShift for band 6  : " << static_cast<uint>((gome->pmd[ctr+1]>>8)&15) << endl
				   << "\tShift for band 7  : " << static_cast<uint>((gome->pmd[ctr+1]>>4)&15) << endl
				   << "\tShift for band 8  : " << static_cast<uint>(gome->pmd[ctr+1]&15) << endl
				   << "PMD-" << tmpChar << " readout " << i << " bands 9-12 status word 3	: \n"
				   << "\tShift for band 9  : " << static_cast<uint>((gome->pmd[ctr+2]>>12)&15) << endl
				   << "\tShift for band 10 : " << static_cast<uint>((gome->pmd[ctr+2]>>8)&15) << endl
				   << "\tShift for band 11 : " << static_cast<uint>((gome->pmd[ctr+2]>>4)&15) << endl
				   << "\tShift for band 12 : " << static_cast<uint>(gome->pmd[ctr+2]&15) << endl
				   << "PMD-" << tmpChar << " readout " << i << " bands 13-15 status word 4	: \n"
				   << "\tShift for band 13 : " << static_cast<uint>((gome->pmd[ctr+3]>>12)&15) << endl
				   << "\tShift for band 14 : " << static_cast<uint>((gome->pmd[ctr+3]>>8)&15) << endl
				   << "\tShift for band 15 : " << static_cast<uint>((gome->pmd[ctr+3]>>4)&15) << endl
				   << "PMD-" << tmpChar << " readout " << i << " bands 1-15					: \n"
				   << "\tCount\tValues\n"
				   << setiosflags(ios::right) << setiosflags(ios::fixed)
				   << setw(5) << setfill(' ')
				   << "\t1\t"
				   << gome->pmd[ctr+4] << "\t"
				   << gome->pmd[ctr+5] << "\t"
				   << gome->pmd[ctr+6] << "\t"
				   << gome->pmd[ctr+7] << "\t"
				   << gome->pmd[ctr+8] << endl
				   << "\t6\t"
				   << gome->pmd[ctr+9] << "\t"
				   << gome->pmd[ctr+10] << "\t"
				   << gome->pmd[ctr+11] << "\t"
				   << gome->pmd[ctr+12] << "\t"
				   << gome->pmd[ctr+13] << endl
				   << "\t11\t"
				   << gome->pmd[ctr+14] << "\t"
				   << gome->pmd[ctr+15] << "\t"
				   << gome->pmd[ctr+16] << "\t"
				   << gome->pmd[ctr+17] << "\t"
				   << gome->pmd[ctr+18] << endl;

				ctr = ctr + 19;
			}
		}

	}

	else if ((pmd_transfer_mode == 3) || (pmd_readout_mode == 2))
	{

		os << "\nPMD Mode 3\n----------\n"
		   << "Counter\tPMD-p\tPMD-s\n";

		for (i=0; i<256; i++)
		{
			os << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(5) << setfill(' ') << (i+1);

			os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(5) << setfill(' ') << gome->pmd[i+72];

			os << "\t" << setiosflags(ios::left) << setiosflags(ios::fixed)
			   << setw(5) << setfill(' ') << gome->pmd[i+376] << endl;
		}
	}

	else
	{
		// Error don't know how to decode PMD Data
		os << "\n***WARNING: PMD Mode not identified (PMD Transfer Mode=" << pmd_transfer_mode
		   << "PMD Readout Mode=" << pmd_readout_mode << ")";
	}

	os << endl << endl;


	os << "FPA Data\n--------\n";
	ctr = 680;
	for (i=1; i<=2; i++)
	{
		for (j=1; j<=4; j++)
		{
			os << "FPA " << j << " Readout " << i << " Indicator : \n"
			   << "\tPattern				 : " << ntobs(static_cast<uint>((gome->pmd[ctr]>>12)&15), 3) << endl
			   << "\tFrame ID				 : " << ntobs(static_cast<uint>((gome->pmd[ctr]>>8)&15), 3) << endl
			   << "\tData Subset Counter	 : " << ntobs(static_cast<uint>((gome->pmd[ctr]>>4)&15), 3) << endl
			   << "\tPacket Sequence Control : " << ntobs(static_cast<uint>(gome->pmd[ctr]&15), 3) << endl;

			os << "FPA " << j << " Readout " << i << " Data		 : \n\tCount\tValues\n";
			ctr++;
			for (k=0; k<256; k++)
			{
				os << "\t" << (k*4) + 1	 << "\t"
				   << gome->pmd[ctr] << "\t"
				   << gome->pmd[ctr+1] << "\t"
				   << gome->pmd[ctr+2] << "\t"
				   << gome->pmd[ctr+3] << endl;
				ctr = ctr + 4;
			}
		}
	}
	os << endl;

}


//
// extract_ascat_data
//

void CCSDSPacketDisplayer::extract_ascat_data(CCSDS* packet, deque<byte>& sci_data)
{
	for(int i=0; i<packet->get_payload_length(); i++)
	{
		byte a = packet->get_payload(ASCAT_RAW_AncillaryData::SIZE - OBTTime::SIZE)[i];
		sci_data.push_back(a);
	}
}


//
// output_ascat_nom_record
//

void CCSDSPacketDisplayer::output_ascat_nom_record(ostream& os,
												   CCSDS* packet)
{
	int i, ctr;

	// deque<byte> payload_data;
	// extract_ascat_data(packet, payload_data);
	// uint16_be data[256];
	// copy_n(payload_data.begin(), 512, (byte*)(&data));

	CCSDS_ASCAT_NOM* ascat = (CCSDS_ASCAT_NOM*)packet->get_data();

	os << "\nAncillary Data\n--------------\n\n"
	   << "PRI Count at Time Tag	: " << ascat->ancillary_data.pri_count_time_tag << endl << endl
	   << "							| Antenna |	 Data	|Cal.|	   Data		|\n"
	   << "							|  Type	  |	 Type	| ID |	  Source	|\n"
	   << "Tag Field				: "
	   << stretch_8(ascat->ancillary_data.tag_field) << endl << endl
	   << "							|Val.|Rate|			 Zero Fill			|\n"
	   << "							|Flag|Lim.|								|\n"
	   << "Ground Processor Flags	: "
	   << stretch_8(ascat->ancillary_data.ground_processor_flags) << endl << endl
	   << "PRI Count				: " << ascat->ancillary_data.pri_count << endl << endl
	   << "							  DPU SW Version \t ICU SW Version\n"
	   << "Software Configuration	: \t" << ntohs(ascat->ancillary_data.soft_config>>8)
	   << "\t\t\t" << ntohs(ascat->ancillary_data.soft_config) << endl << endl
	   << "							  |Drive| DPU |Rev|\n"
	   << "Parameter Configuration	: "
	   << ntobs(ascat->ancillary_data.param_config, 15) << endl << endl
	   << "							  |ID|Spare|Config|\n"
	   << "Instrument Configuration : " << ntobs(ascat->ancillary_data.inst_config, 15) << endl << endl;

	for(i=0; i<6; i++)
	{
		os << "SFE Temperature " << i+1 << "		: " << ascat->ancillary_data.SFE_Temp[i] << endl;
	}
	os << endl;

	for(i=0; i<12; i++)
	{
		if (i < 9)
		{
			os << "Antenna	Temperature " << i+1 << "	: " << ascat->ancillary_data.Antenna_Temp[i] << endl;
		}
		else
		{
			os << "Antenna	Temperature " << i+1 << "  : " << ascat->ancillary_data.Antenna_Temp[i] << endl;
		}
	}


	os << "\nReceiver Gain			  : " << ascat->ancillary_data.receiver_gain << endl << endl
	   << "Out of Range Count		: " << ascat->ancillary_data.out_of_range_count << endl << endl;


	for(i=0; i<4; i++)
	{
		os << "Integrated Transmitted Power " << i+1 << " : " << ascat->ancillary_data.int_trans_power[i] << endl;
	}
	os << endl;

	for(i=0; i<4; i++)
	{
		os <<  "Integrated Reflected Power " << i+1 << "   : " << ascat->ancillary_data.int_refl_power[i] << endl;
	}
	os << endl;

	for(i=0; i<4; i++)
	{
		os <<  "Integrated Calibration Power " << i+1 << " : " << ascat->ancillary_data.int_calib_power[i] << endl;
	}
	os << endl;

	os << "\nCalibration Power : " << endl
	   << "Antenna \tFrequency k1 \tFrequency k2 \tFrequency k3\n"
	   << "		   \tReal	 Imag \tReal	Imag \tReal	   Imag\n";
	ctr = 0;
	for(i=0; i<4; i++)
	{
		os << "	  " << i+1 << "	   \t"
		   <<  ascat->ancillary_data.calib_power[ctr] << "\t"
		   <<  ascat->ancillary_data.calib_power[ctr+1] << "\t"
		   <<  ascat->ancillary_data.calib_power[ctr+2] << "\t"
		   <<  ascat->ancillary_data.calib_power[ctr+3] << "\t"
		   <<  ascat->ancillary_data.calib_power[ctr+4] << "\t"
		   <<  ascat->ancillary_data.calib_power[ctr+5] << endl;
		ctr = ctr + 6;
	}


	os << "\n\nApplication Data\n----------------\n\n";

	os << "Nominal Source Packet Application Data : " << endl
	   << "Count \tData" << endl;
	ctr = 0;

	// for (int i=0; i<256/4; i++)
	// {
	//		os << ctr+1 << "\t"
	//		   << data[ctr] << "\t"
	//		   << data[ctr+1] << "\t"
	//		   << data[ctr+2] << "\t"
	//		   << data[ctr+3] << endl;
	//		ctr = ctr + 4;
	// }

	for (int i=0; i<256/4; i++)
	{
		os << ctr+1 << "\t"
		   << uint16tosfloat(ascat->payload.data[ctr]) << "\t"
		   << uint16tosfloat(ascat->payload.data[ctr+1]) << "\t"
		   << uint16tosfloat(ascat->payload.data[ctr+2]) << "\t"
		   << uint16tosfloat(ascat->payload.data[ctr+3]) << endl;
		ctr = ctr + 4;
	}
	os << endl;

}


//
// output_ascat_raw_record
//

void CCSDSPacketDisplayer::output_ascat_raw_record(ostream& os,
												   CCSDS* packet)
{
	int i;
	CCSDS_ASCAT_RAW* ascat = (CCSDS_ASCAT_RAW*)packet->get_data();

	deque<byte> payload_data;
	extract_ascat_data(packet, payload_data);

	int payload_data_size = ( ascat->ancillary_data.noise_samples
							  + ascat->ancillary_data.calib_samples
							  + ascat->ancillary_data.echo_samples ) * 2;

	// int payload_data_size = payload_data.size() + 6 - ASCAT_RAW_AncillaryData::SIZE;
	uint16_be *data = new uint16_be[payload_data_size/2];
	copy_n(payload_data.begin(), payload_data_size, (byte*)(data));

	uint16_be *echo_samples = new uint16_be[ascat->ancillary_data.echo_samples];
	for(i=0; i<ascat->ancillary_data.echo_samples; i++)
	{
		echo_samples[i] = data[i];
	}

	uint16_be *calib_samples = new uint16_be[ascat->ancillary_data.calib_samples];
	for(i=ascat->ancillary_data.echo_samples;
		i<(ascat->ancillary_data.calib_samples+ascat->ancillary_data.echo_samples);
		i++)
	{
		calib_samples[i] = data[i];
	}

	uint16_be *noise_samples = new uint16_be[ascat->ancillary_data.noise_samples];
	for(i=(ascat->ancillary_data.calib_samples+ascat->ancillary_data.echo_samples);
		i<(ascat->ancillary_data.noise_samples+ascat->ancillary_data.calib_samples+ascat->ancillary_data.echo_samples);
		i++)
	{
		noise_samples[i] = data[i];
	}

	delete data;

	os << "\nAncillary Data\n--------------\n";

	os << "PRI Count at Time Tag	= " << ascat->ancillary_data.pri_count_time_tag << endl
	   << "Tag Field				= " << ntobs(ascat->ancillary_data.tag_field, 7) << endl
	   << "Ground Processor Flags	= " << ntobs(ascat->ancillary_data.ground_processor_flags, 7) << endl
	   << "PRI Count				= " << ascat->ancillary_data.pri_count << endl
	   << "Software Configuration	= " << ntohs(ascat->ancillary_data.soft_config>>8)
	   << ntohs(ascat->ancillary_data.soft_config) << endl
	   << "Parameter Configuration	= " << ntobs(ascat->ancillary_data.param_config, 15) << endl
	   << "Instrument Configuration = " << ntobs(ascat->ancillary_data.inst_config, 15) << endl;

	for(i=0; i<6; i++)
	{
		os << "SFE Temperature " << i+1 << "		= " << ascat->ancillary_data.SFE_Temp[i] << endl;
	}

	for(i=0; i<12; i++)
	{
		if (i < 9)
		{
			os << "Antenna	Temperature " << i+1 << "	= " << ascat->ancillary_data.Antenna_Temp[i] << endl;
		}
		else
		{
			os << "Antenna	Temperature " << i+1 << "  = " << ascat->ancillary_data.Antenna_Temp[i] << endl;
		}
	}

	os << "Receiver Gain			= " << ascat->ancillary_data.receiver_gain << endl
	   << "Out of Range Count		= " << ascat->ancillary_data.out_of_range_count << endl;

	os << "Integrated Transmitted Power = " << ascat->ancillary_data.int_trans_power << endl
	   << "Integrated Reflected Power	= " << ascat->ancillary_data.int_refl_power << endl
	   << "Integrated Calibration Power = " << ascat->ancillary_data.int_calib_power << endl
	   << "Number of Echo Samples		= " << ascat->ancillary_data.echo_samples << endl
	   << "Number of Calibration Samples= " << ascat->ancillary_data.calib_samples << endl
	   << "Number of Noise Samples		= " << ascat->ancillary_data.noise_samples << endl;

	os << "\nApplication Data\n----------------\n";

	for (i=0; i<ascat->ancillary_data.echo_samples; i++)
	{
		os << "Echo Sample [" << i+1 << "]	= " << echo_samples[i] << endl;
	}

	for (i=0; i<ascat->ancillary_data.calib_samples; i++)
	{
		os << "Calib Sample [" << i+1 << "] = " << calib_samples[i] << endl;
	}

	for (i=0; i<ascat->ancillary_data.noise_samples; i++)
	{
		os << "Noise Sample [" << i+1 << "] = " << noise_samples[i] << endl;
	}

	delete echo_samples;
	delete calib_samples;
	delete noise_samples;
}


//
// extract_gras_data
//

void CCSDSPacketDisplayer::extract_gras_data(CCSDS* packet, deque<byte>& tmp_data, int length)
{
	for(int i=0; i<length; i++)
	{
		byte a = packet->get_gras_data()[i];
		tmp_data.push_back(a);
	}
}


//
// output_gras_navigation_nominal_record  ( AP_ID = 2 )
//

void CCSDSPacketDisplayer::output_gras_navigation_nominal_record(ostream& os,
																 CCSDS* packet)
{
	CCSDS_GRAS_NAVIGATION_Nominal* gras = (CCSDS_GRAS_NAVIGATION_Nominal*)packet->get_data();

	os << "\t\tBasic Block\n"
	   << "\t\t\tNSM (Bin)			  : " << (((gras->basic_block.OBT_very_fine_NSM)>>2)&7)	 << endl << endl

	   << "\t\t\tOBT	coarse time	  : " << gras->basic_block.OBT_coarse					 << endl
	   << "\t\t\tOBT	  fine time	  : " << gras->basic_block.OBT_fine						 << endl
	   << "\t\t\tOBT very fine time	  : " << (((gras->basic_block.OBT_very_fine_NSM)>>5)&7)	 << endl << endl

	   << "\t\t\tUTC Days			  : " << gras->basic_block.UTC.days			 << endl
	   << "\t\t\tUTC Milliseconds	  : " << gras->basic_block.UTC.milliseconds	 << endl
	   << "\t\t\tUTC Microseconds	  : " << gras->basic_block.UTC.microseconds	 << endl
	   << "\t\t\tTime : "				  << gras->basic_block.UTC				 << endl << endl

	   << "\t\t\tPOS_X (cm)			  : " << gras->basic_block.POS_X			 << endl
	   << "\t\t\tPOS_Y (cm)			  : " << gras->basic_block.POS_Y			 << endl
	   << "\t\t\tPOS_Z (cm)			  : " << gras->basic_block.POS_Z			 << endl << endl;

	for (int i=0; i<7; i++)
	{
		unsigned int DUTC = gras->incremental_block[i].DUTC;
		unsigned int DUTC_Seconds = DUTC / 1000000;
		unsigned int DUTC_Microseconds = DUTC - (DUTC_Seconds * 1000000);

		os << "\t\tIncremental Block #" << i+1 << "\n"
		   << "\t\t\tNSM (Bin)			  : " << (((gras->incremental_block[i].DOBT_very_fine_NSM)>>2)&7)  << endl << endl

		   << "\t\t\tDOBT	 coarse time  : " << static_cast<uint>(gras->incremental_block[i].DOBT_coarse) << endl
		   << "\t\t\tDOBT	   fine time  : " << gras->incremental_block[i].DOBT_fine					   << endl
		   << "\t\t\tDOBT very fine time  : " << (((gras->incremental_block[i].DOBT_very_fine_NSM)>>5)&7)  << endl << endl

		   << "\t\t\tDUTC Seconds		  : " << DUTC_Seconds					<< endl
		   << "\t\t\tDUTC Microseconds	  : " << DUTC_Microseconds				<< endl << endl

		   << "\t\t\tDX (cm)			  : " << gras->incremental_block[i].DX	<< endl
		   << "\t\t\tDY (cm)			  : " << gras->incremental_block[i].DY	<< endl
		   << "\t\t\tDZ (cm)			  : " << gras->incremental_block[i].DZ	<< endl << endl;
	}
}


//
// output_gras_navigation_reduced_record  ( AP_ID = 3 )
//

void CCSDSPacketDisplayer::output_gras_navigation_reduced_record(ostream& os,
																 CCSDS* packet)
{
	CCSDS_GRAS_NAVIGATION_Reduced* gras = (CCSDS_GRAS_NAVIGATION_Reduced*)packet->get_data();

	os << "\t\tBasic Block\n"
	   << "\t\t\tNSM (Bin)			  : " << (((gras->basic_block.OBT_very_fine_NSM)>>2)&7)	 << endl << endl

	   << "\t\t\tOBT	coarse time	  : " << gras->basic_block.OBT_coarse					 << endl
	   << "\t\t\tOBT	  fine time	  : " << gras->basic_block.OBT_fine						 << endl
	   << "\t\t\tOBT very fine time	  : " << (((gras->basic_block.OBT_very_fine_NSM)>>5)&7)	 << endl << endl

	   << "\t\t\tUTC Days			  : " << gras->basic_block.UTC.days			 << endl
	   << "\t\t\tUTC Milliseconds	  : " << gras->basic_block.UTC.milliseconds	 << endl
	   << "\t\t\tUTC Microseconds	  : " << gras->basic_block.UTC.microseconds	 << endl
	   << "\t\t\tTime : "				  << gras->basic_block.UTC				 << endl << endl

	   << "\t\t\tPOS_X (cm)			  : " << gras->basic_block.POS_X			 << endl
	   << "\t\t\tPOS_Y (cm)			  : " << gras->basic_block.POS_Y			 << endl
	   << "\t\t\tPOS_Z (cm)			  : " << gras->basic_block.POS_Z			 << endl << endl;
}


//
// output_gras_measurement_record  ( AP_ID = 448, 480 )
//

void CCSDSPacketDisplayer::output_gras_measurement_record(ostream& os,
														  CCSDS* packet)
{
	int i, ctr;
	CCSDS_GRAS_MEASUREMENT* gras = (CCSDS_GRAS_MEASUREMENT*)packet->get_data();

	os << "\nAncillary Data\n--------------\n\n"
	   << "LEN	: " << gras->ancillary_data.len << endl << endl
	   << "INFO : \n"
	   << "\tDiscarding Data Flag	: " << static_cast<uint>((gras->ancillary_data.inf>>9)&1) << endl
	   << "\tExperimental Data Flag : " << static_cast<uint>((gras->ancillary_data.inf>>8)&1) << endl
	   << "\tInstrument Mode (Hex)	: " << ntohs(static_cast<uint>(gras->ancillary_data.inf&255)) << endl << endl;


	int ancil_size = gras->ancillary_data.len;
	int payload_size = packet->get_length() -  ancil_size - 14;
	deque<byte> tmp_data;
	extract_gras_data(packet, tmp_data, (ancil_size + payload_size));
	uint16_be *data = new uint16_be[((ancil_size/2) + (payload_size/2))];
	copy_n(tmp_data.begin(), (ancil_size + payload_size), (byte*)(data));

	ctr = 5;
	int packet_type = 0;
	int sub_packet_size = 0;
	while (ctr < (ancil_size/2))
	{
		packet_type = static_cast<uint>(data[ctr]&15);
		sub_packet_size = static_cast<uint>((data[ctr]>>4)&4095);
		//cout << "ctr " << ctr << " packet_type " << packet_type << " sub_packet_size " << sub_packet_size << endl;

		if ((packet_type == 1) && (sub_packet_size == 48))
		{
			os	<< "Extended Navigation Packet : \n"
				<< "\tLEN		   : " << sub_packet_size << endl
				<< "\tTYP		   : " << packet_type << endl
				<< "\tOBT		   : " <<
				((static_cast<unsigned long long>(data[ctr+1])<<27) |
				 (static_cast<unsigned long long>(data[ctr+2])<<11) |
				 (static_cast<unsigned long long>((data[ctr+3])>>5)&2047)) << endl
				<< "\t	coarse	   : " << ((static_cast<unsigned int>(data[ctr+1])<<8) | (static_cast<unsigned int>(data[ctr+2])>>8)) << endl
				<< "\t	fine	   : " << (((data[ctr+2]&0xff)<<8) | data[ctr+3]>>8) << endl
				<< "\t	very fine  : " << ((data[ctr+3]>>5)&7) << endl;

			int days = data[ctr+4];
			int milliseconds = ( (data[ctr+5]<<16) | data[ctr+6] );
			int microseconds = data[ctr+7];
			os	<< "\tNSM (Bin)	   : " << ntobs(static_cast<uint>((data[ctr+3]>>2)&7), 2) << endl
				<< "\tUTC		   : " << TimeConv::utc_days_milliseconds_to_string(days,milliseconds) << "." << ntofs(microseconds,3) << endl
				<< "\t	days  : " << days << endl
				<< "\t	msec  : " << milliseconds << endl
				<< "\t	usec  : " << microseconds << endl
				<< "\tIMT		   : " << ( (static_cast<long long>(data[ctr+8])<<48)
											| (static_cast<long long>(data[ctr+9])<<32)
											| (static_cast<long long>(data[ctr+10])<<16)
											| static_cast<long long>(data[ctr+11]) ) << endl
				<< "\tPOS_X (cm)   : " << static_cast<int>( ((static_cast<uint32_be>(data[ctr+12]))<<16)
															| data[ctr+13] ) << endl
				<< "\tPOS_Y (cm)   : " << static_cast<int>( ((static_cast<uint32_be>(data[ctr+14]))<<16)
															| data[ctr+15] ) << endl
				<< "\tPOS_Z (cm)   : " << static_cast<int>( ((static_cast<uint32_be>(data[ctr+16]))<<16)
															| data[ctr+17] ) << endl
				<< "\tVEL_X (cm/s) : " << static_cast<int>( ((static_cast<uint32_be>(data[ctr+18]))<<16)
															| data[ctr+19] ) << endl
				<< "\tVEL_Y (cm/s) : " << static_cast<int>( ((static_cast<uint32_be>(data[ctr+20]))<<16)
															| data[ctr+21] ) << endl
				<< "\tVEL_Z (cm/s) : " << static_cast<int>( ((static_cast<uint32_be>(data[ctr+22]))<<16)
															| data[ctr+23] ) << endl
				<< endl;
			ctr = ctr + 24;
		}

		else if ((packet_type == 2) && (sub_packet_size == 44))
		{
			os	<< "Gain Setting and Histogram Packet : \n"
				<< "\tLEN	   : " << sub_packet_size << endl
				<< "\tTYP	   : " << packet_type << endl
				<< "\tALS	   : " << static_cast<int>((data[ctr+1]>>1)&1) << endl
				<< "\tCHN	   : " << static_cast<int>(data[ctr+1]&1) << endl
				<< "\tHIMT	   : " << ( (static_cast<long long>(data[ctr+2])<<48)
										| (static_cast<long long>(data[ctr+3])<<32)
										| (static_cast<long long>(data[ctr+4])<<16)
										| static_cast<long long>(data[ctr+5]) ) << endl
				<< "\tHIST	+3 : " << data[ctr+6] << endl
				<< "\tHIST	+2 : " << data[ctr+7] << endl
				<< "\tHIST	+1 : " << data[ctr+8] << endl
				<< "\tHIST	 0 : " << data[ctr+9] << endl
				<< "\tHIST	-1 : " << data[ctr+10] << endl
				<< "\tHIST	-2 : " << data[ctr+11] << endl
				<< "\tHIST	-3 : " << data[ctr+12] << endl
				<< "\tGCIMT	   : " << ( (static_cast<long long>(data[ctr+14])<<48)
										| (static_cast<long long>(data[ctr+15])<<32)
										| (static_cast<long long>(data[ctr+16])<<16)
										| static_cast<long long>(data[ctr+17]) ) << endl
				<< "\tANA	   : " << ntobs(static_cast<int>(data[ctr+18]&15), 3) << endl
				<< "\tDIG	   : " << endl
				<< "\t	CCT_0 : " << static_cast<int>((data[ctr+19]>>8)&255) << endl
				<< "\t	CCT_1 : " << static_cast<int>(data[ctr+19]&255) << endl
				<< "\t	CCT_2 : " << static_cast<int>((data[ctr+20]>>8)&255) << endl
				<< "\t	CCT_3 : " << static_cast<int>(data[ctr+20]&255) << endl
				<< "\t	CCT_4 : " << static_cast<int>((data[ctr+21]>>8)&255) << endl
				<< "\t	CCT_5 : " << static_cast<int>(data[ctr+21]&255) << endl
				<< endl;
			ctr = ctr + 22;
		}

		else if ((packet_type == 3) && (sub_packet_size == 44))
		{
			os	<< "Temperature and Voltage Packet : \n"
				<< "\tLEN			 : " << sub_packet_size << endl
				<< "\tTYP			 : " << packet_type << endl
				<< "\tOBT			 : " << static_cast<uint32_be>( (data[ctr+1]<<16) | data[ctr+2] ) << endl
				<< "\t	coarse		 : " << (static_cast<unsigned int>((data[ctr+1]<<8))|(data[ctr+2]>>8)) << endl
				<< "\t	  fine		 : " << static_cast<unsigned int>(data[ctr+2]&0xff) << endl
				<< "\tGEU_STS (Bin)	 : " << ntobs(static_cast<int>(data[ctr+3]&255), 7) << endl
				<< "\tUSO_TIM		 : " << static_cast<uint32_be>( (data[ctr+4]<<16) | data[ctr+5] ) << endl
				<< "\tZAT			 : " << static_cast<int>(data[ctr+6]&4095) << endl
				<< "\tVAT			 : " << static_cast<int>(data[ctr+7]&4095) << endl
				<< "\tAVAT			 : " << static_cast<int>(data[ctr+8]&4095) << endl
				<< "\tRZT			 : " << static_cast<int>(data[ctr+9]&4095) << endl
				<< "\tRVT			 : " << static_cast<int>(data[ctr+10]&4095) << endl
				<< "\tRAVT			 : " << static_cast<int>(data[ctr+11]&4095) << endl
				<< "\tGEUT			 : " << static_cast<int>(data[ctr+12]&4095) << endl
				<< "\tISACT			 : " << static_cast<int>(data[ctr+13]&4095) << endl
				<< "\tUSOIT			 : " << static_cast<int>(data[ctr+14]&4095) << endl
				<< "\tUSOET			 : " << static_cast<int>(data[ctr+15]&4095) << endl
				<< "\tDBUPV			 : " << static_cast<int>(data[ctr+16]&4095) << endl
				<< "\tTSV			 : " << static_cast<int>(data[ctr+17]&4095) << endl
				<< "\tFGT			 : " << static_cast<int>(data[ctr+18]&4095) << endl
				<< "\tUSOG			 : " << static_cast<int>(data[ctr+19]&4095) << endl
				<< "\tD5V			 : " << static_cast<int>(data[ctr+20]&4095) << endl
				<< endl;
			ctr = ctr + 22;
		}

		else if ((packet_type == 4) && (sub_packet_size == 12))
		{
			os	<< "Tracking State Packet : \n"
				<< "\tLEN : " << sub_packet_size << endl
				<< "\tTYP : " << packet_type << endl
				<< "\tC	  : " << static_cast<int>((data[ctr+1]>>14)&1) << endl
				<< "\tTS  : " << static_cast<int>((data[ctr+1]>>10)&15) << endl
				<< "\tSAT : " << static_cast<int>((data[ctr+1]>>4)&63) << endl
				<< "\tCHN : " << static_cast<int>(data[ctr+1]&15) << endl
				<< "\tIMT : " << ( (static_cast<long long>(data[ctr+2])<<48)
								   | (static_cast<long long>(data[ctr+3])<<32)
								   | (static_cast<long long>(data[ctr+4])<<16)
								   | static_cast<long long>(data[ctr+5]) ) << endl
				<< endl;
			ctr = ctr + 6;
		}

		else if ((packet_type == 5) && (sub_packet_size == 88))
		{
			os	<< "GPS Data Packet : \n"
				<< "\tLEN		 : " << sub_packet_size << endl
				<< "\tTYP		 : " << packet_type << endl
				<< "\tOBT		 : " << static_cast<uint32_be>( (data[ctr+1]<<16) | data[ctr+2] ) << endl
				<< "\t	coarse	 : " << (static_cast<unsigned int>((data[ctr+1]<<8))|(data[ctr+2]>>8)) << endl
				<< "\t	  fine	 : " << static_cast<unsigned int>(data[ctr+2]&0xff) << endl
				<< "\tSAT		 : " << static_cast<uint>((data[ctr+3]>>10)&63) << endl
				<< "\tA			 : " << static_cast<uint>((data[ctr+3]>>9)&1) << endl
				<< "\tAS		 : " << static_cast<uint>((data[ctr+3]>>8)&1) << endl
				<< "\tC2 (Bin)	 : " << ntobs(static_cast<uint>((data[ctr+3]>>6)&3), 1) << endl
				<< "\tSVA		 : " << static_cast<uint>((data[ctr+3]>>2)&15) << endl
				<< "\tD2		 : " << static_cast<uint>((data[ctr+3]>>1)&1) << endl
				<< "\tCF		 : " << static_cast<uint>(data[ctr+3]&1) << endl
				<< "\tWN		 : " << static_cast<uint>((data[ctr+4]>>6)&1023) << endl
				<< "\tNH		 : " << static_cast<uint>((data[ctr+4]>>5)&1) << endl
				<< "\tSH		 : " << static_cast<uint>(data[ctr+4]&31) << endl
				<< "\tIODC		 : " << static_cast<uint>((data[ctr+5]>>6)&1023) << endl
				<< "\ta_f0		 : " << static_cast<signed int>( (static_cast<uint>((data[ctr+5]&63)<<6)
																  | static_cast<uint>(data[ctr+6]) ) ) << endl
				<< "\ta_f1		 : " << static_cast<signed int>(data[ctr+7]) << endl
				<< "\ta_f2		 : " << static_cast<signed int>((data[ctr+8]>>8)&255) << endl
				<< "\tT_gd		 : " << static_cast<signed int>(data[ctr+8]&255) << endl
				<< "\tt_oc		 : " << static_cast<unsigned int>(data[ctr+9]) << endl
				<< "\tC_rs		 : " << static_cast<signed int>(data[ctr+10]) << endl
				<< "\tDelta_n	 : " << static_cast<signed int>(data[ctr+11]) << endl
				<< "\tM_0		 : " << static_cast<signed int>((data[ctr+12]<<16) | data[ctr+13]) << endl
				<< "\tC_ue		 : " << static_cast<signed int>(data[ctr+14]) << endl
				<< "\te			 : " << static_cast<signed int>((data[ctr+15]<<16) |  data[ctr+16]) << endl
				<< "\tC_us		 : " << static_cast<signed int>(data[ctr+17]) << endl
				<< "\tsqrt_A	 : " << static_cast<unsigned int>((data[ctr+18]<<16) | data[19]) << endl
				<< "\tt_oe		 : " << static_cast<unsigned int>(data[ctr+20]) << endl
				<< "\tC_ic		 : " << static_cast<signed int>(data[ctr+21]) << endl
				<< "\tOhmega_0	 : " << static_cast<signed int>((data[ctr+22]<<16) | data[23]) << endl
				<< "\tC_is		 : " << static_cast<signed int>(data[ctr+24]) << endl
				<< "\ti_0		 : " << static_cast<signed int>((data[ctr+25]<<16) | data[26]) << endl
				<< "\tC_re		 : " << static_cast<signed int>(data[ctr+27]) << endl
				<< "\tohmega	 : " << static_cast<signed int>((data[ctr+28]<<16) | data[29]) << endl
				<< "\tOhmega_DOT : " << static_cast<signed int>(data[ctr+30]<<16 | data[ctr+31]>>8) << endl
				<< "\tIODE		 : " << static_cast<unsigned int>(data[ctr+31]&255) << endl
				<< "\tI_DOT		 : " << static_cast<signed int>((data[ctr+32]>>5)&2047) << endl
				<< "\tASAT		 : " << static_cast<signed int>((data[ctr+33]>>10)&63) << endl
				<< "\tA_1		 : " << static_cast<signed int>( ((data[ctr+33]&255)<<8) | data[ctr+34]) << endl
				<< "\tA_0		 : " << static_cast<signed int>((data[ctr+35]<<16) | data[36]) << endl
				<< "\tDelta_t_LS : " << static_cast<signed int>((data[ctr+37]>>8)&255) << endl
				<< "\tt_ot		 : " << static_cast<unsigned int>(data[ctr+37]&255) << endl
				<< "\tWN_t		 : " << static_cast<unsigned int>((data[ctr+38]>>8)&255) << endl
				<< "\tWN_LSF	 : " << static_cast<unsigned int>(data[ctr+38]&255) << endl
				<< "\tDN		 : " << static_cast<unsigned int>((data[ctr+39]>>8)&255) << endl
				<< "\tDelta_t_LSF: " << static_cast<signed int>(data[ctr+39]&255) << endl
				<< "\tAlpha_0	 : " << static_cast<signed int>((data[ctr+40]>>8)&255) << endl
				<< "\tAlpha_1	 : " << static_cast<signed int>(data[ctr+40]&255) << endl
				<< "\tAlpha_2	 : " << static_cast<signed int>((data[ctr+41]>>8)&255) << endl
				<< "\tAlpha_3	 : " << static_cast<signed int>(data[ctr+41]&255) << endl
				<< "\tBeta_0	 : " << static_cast<signed int>((data[ctr+42]>>8)&255) << endl
				<< "\tBeta_1	 : " << static_cast<signed int>(data[ctr+43]&255) << endl
				<< "\tBeta_2	 : " << static_cast<signed int>((data[ctr+43]>>8)&255) << endl
				<< "\tBeta_3	 : " << static_cast<signed int>(data[ctr+43]&255) << endl
				<< endl;
			ctr = ctr + 44;
		}

		else if (packet_type == 6)
		{
			int chn = static_cast<uint>(data[ctr+1]&3);
			if (chn == 0)
			{
				os	<< "Noise Data Packet Zenith Chain : \n"
					<< "\tLEN		 : " << sub_packet_size << endl
					<< "\tTYP		 : " << packet_type << endl
					<< "\tCHN		 : " << ntobs(chn, 1) << endl
					<< "\tIMT		 : " << ( (static_cast<long long>(data[ctr+2])<<48)
											  | (static_cast<long long>(data[ctr+3])<<32)
											  | (static_cast<long long>(data[ctr+4])<<16)
											  | static_cast<long long>(data[ctr+5]) ) << endl
					<< "\tDT		 : " << static_cast<uint32_be>( (data[ctr+6]<<16) | data[ctr+7] ) << endl;

				i = 8;
				while (i < (((sub_packet_size-16)/2)+8))
				{
					os << "\t(M = " << (i/8) << ")" << endl
					   << "\t\tGZA_L1_I_M : " << static_cast<signed int>(data[ctr+i]) << endl
					   << "\t\tGZA_L1_I_S : " << static_cast<signed int>(data[ctr+i+1]) << endl
					   << "\t\tGZA_L2_I_M : " << static_cast<signed int>(data[ctr+i+2]) << endl
					   << "\t\tGZA_L2_I_S : " << static_cast<signed int>(data[ctr+i+3]) << endl
					   << "\t\tGZA_L1_Q_M : " << static_cast<signed int>(data[ctr+i+4]) << endl
					   << "\t\tGZA_L1_Q_S : " << static_cast<signed int>(data[ctr+i+5]) << endl
					   << "\t\tGZA_L2_Q_M : " << static_cast<signed int>(data[ctr+i+6]) << endl
					   << "\t\tGZA_L2_Q_S : " << static_cast<signed int>(data[ctr+i+7]) << endl;
					i = i + 8;
				}

				os	<< endl;
				ctr = ctr + (sub_packet_size/2);
			}
			else if (chn == 3)
			{
				os	<< "Noise Data Packet Occultation Chains : \n"
					<< "\tLEN		  : " << sub_packet_size << endl
					<< "\tTYP		  : " << packet_type << endl
					<< "\tCHN		  : " << ntobs(chn, 1) << endl
					<< "\tIMT		  : " << ( (static_cast<long long>(data[ctr+2])<<48)
											   | (static_cast<long long>(data[ctr+3])<<32)
											   | (static_cast<long long>(data[ctr+4])<<16)
											   | static_cast<long long>(data[ctr+5]) ) << endl
					<< "\tDT		  : " << static_cast<uint32_be>( (data[ctr+6]<<16) | data[ctr+7] ) << endl;

				i = 8;
				while (i < (((sub_packet_size-16)/2)+8))
				{
					os << "\t(M = " << ((i+8)/16) << ")" << endl
					   << "\t\tGAVA_L1_I_M : " << static_cast<signed int>(data[ctr+i]) << endl
					   << "\t\tGAVA_L1_I_S : " << static_cast<signed int>(data[ctr+i+1]) << endl
					   << "\t\tGAVA_L2_I_M : " << static_cast<signed int>(data[ctr+i+2]) << endl
					   << "\t\tGAVA_L2_I_S : " << static_cast<signed int>(data[ctr+i+3]) << endl
					   << "\t\tGVA_L1_I_M  : " << static_cast<signed int>(data[ctr+i+4]) << endl
					   << "\t\tGVA_L1_I_S  : " << static_cast<signed int>(data[ctr+i+5]) << endl
					   << "\t\tGVA_L2_I_M  : " << static_cast<signed int>(data[ctr+i+6]) << endl
					   << "\t\tGVA_L2_I_S  : " << static_cast<signed int>(data[ctr+i+7]) << endl
					   << "\t\tGAVA_L1_Q_M : " << static_cast<signed int>(data[ctr+i+8]) << endl
					   << "\t\tGAVA_L1_Q_S : " << static_cast<signed int>(data[ctr+i+9]) << endl
					   << "\t\tGAVA_L2_Q_M : " << static_cast<signed int>(data[ctr+i+10]) << endl
					   << "\t\tGAVA_L2_Q_S : " << static_cast<signed int>(data[ctr+i+11]) << endl
					   << "\t\tGVA_L1_Q_M  : " << static_cast<signed int>(data[ctr+i+12]) << endl
					   << "\t\tGVA_L1_Q_S  : " << static_cast<signed int>(data[ctr+i+13]) << endl
					   << "\t\tGVA_L2_Q_M  : " << static_cast<signed int>(data[ctr+i+14]) << endl
					   << "\t\tGVA_L2_Q_S  : " << static_cast<signed int>(data[ctr+i+15]) << endl;
					i = i + 16;
				}

				os	<< endl;
				ctr = ctr + (sub_packet_size/2);
			}
			else
			{
				os	<< "*** Noise Data Packet Not Recognised *** : \n"
					<< "\tLEN : " << sub_packet_size << endl
					<< "\tTYP : " << packet_type << endl
					<< "\tCHN : " << ntobs(chn, 1) << endl
					<< endl;
				ctr = ctr + (sub_packet_size/2);
			}
		}

		else
		{
			os	<< "*** WARNING Data Packet Not Recognised *** : "
				<< "LEN : " << sub_packet_size
				<< "TYP : " << packet_type << endl
				<< endl;
			ctr = ctr + (sub_packet_size/2);
		}

	} // End of while loop to output ancillary data

	if (ctr != (ancil_size/2))
	{
		os << "WARNING unused data in packet ctr " << ctr
		   << " ancil_size/2 " << (ancil_size/2) << endl;
	}

	os << "\nPayload Data\n------------\n\n";

	// ***Note that ctr is not set to zero and is
	// used as the main counter in the next while loop.
	packet_type = 0;
	sub_packet_size = 0;
	while (ctr < ((ancil_size/2) + (payload_size/2)) )
	{
		packet_type = static_cast<uint>(data[ctr]&15);
		sub_packet_size = static_cast<uint>((data[ctr]>>4)&4095);

		if (packet_type == 9)
		{
			os << "Occultation Raw Sampling : \n"
			   << "\tLEN	   : " << sub_packet_size << endl
			   << "\tTYP	   : " << packet_type << endl
			   << "\tSAT	   : " << static_cast<uint>((data[ctr+1]>>4)&63) << endl
			   << "\tCHN	   : " << static_cast<uint>(data[ctr+1]&15) << endl
			   << "\tIMT	   : " << ( (static_cast<long long>(data[ctr+2])<<48)
										| (static_cast<long long>(data[ctr+3])<<32)
										| (static_cast<long long>(data[ctr+4])<<16)
										| static_cast<long long>(data[ctr+5]) ) << endl
			   << "\tP_1	   : " << ( (static_cast<long long>(data[ctr+6])<<48)
										| (static_cast<long long>(data[ctr+7])<<32)
										| (static_cast<long long>(data[ctr+8])<<16)
										| static_cast<long long>(data[ctr+9]) ) << endl
			   << "\tF1_1	   : " << static_cast<signed int>((data[ctr+10]<<16) | data[ctr+11]) << endl
			   << "\tTINT1	   : " << static_cast<unsigned int>((data[ctr+12]<<16) | data[ctr+13]) << endl
			   << "\tF2_1	   : " << static_cast<signed int>((data[ctr+14]<<16) | data[ctr+15]) << endl
			   << "\tTINT2	   : " << static_cast<unsigned int>((data[ctr+16]<<16) | data[ctr+17]) << endl
			   << "\tIQ_CA_EXP : " << static_cast<uint>(data[ctr+18]&255) << endl;

			i = 20;
			while (i < (((sub_packet_size-40)/2)+20))
			{
				os << "(M =	 " << i-19 << ")\n"
				   << "\tI_CA : " << static_cast<signed int>((data[ctr+i]>>8)&255) << endl
				   << "\tQ_CA : " << static_cast<signed int>(data[ctr+i]&255) << endl;
				i++;
			}

			os << endl;
			ctr = ctr + (sub_packet_size/2);
		}

		else if (packet_type == 10)
		{
			os	<< "Occultation & Zenith Carrier Phase with Single Carrier Frequency Tracking : \n"
				<< "\tLEN		: " << sub_packet_size << endl
				<< "\tTYP		: " << packet_type << endl
				<< "\tSAT		: " << static_cast<uint>((data[ctr+1]>>4)&63) << endl
				<< "\tCHN		: " << static_cast<uint>(data[ctr+1]&15) << endl
				<< "\tIMT		: " << ( (static_cast<long long>(data[ctr+2])<<48)
										 | (static_cast<long long>(data[ctr+3])<<32)
										 | (static_cast<long long>(data[ctr+4])<<16)
										 | static_cast<long long>(data[ctr+5]) ) << endl
				<< "\tP_1		: " << ( (static_cast<long long>(data[ctr+6])<<48)
										 | (static_cast<long long>(data[ctr+7])<<32)
										 | (static_cast<long long>(data[ctr+8])<<16)
										 | static_cast<long long>(data[ctr+9]) ) << endl
				<< "\tF1		: " << static_cast<signed int>((data[ctr+10]<<16) | data[ctr+11]) << endl
				<< "\tTINT		: " << static_cast<unsigned int>((data[ctr+12]<<16) | data[ctr+13]) << endl
				<< "\tIQ_CA_EXP : " << static_cast<uint>(data[ctr+14]&255) << endl;

			i = 16;
			while (i < (((sub_packet_size-32)/2)+16))
			{
				os << "(M =	 " << (i/4)-3 << ")\n"
				   << "\tDP_1 : " << static_cast<signed int>((data[ctr+i]<<16) | data[ctr+i+1]) << endl
				   << "\tI_CA : " << static_cast<signed int>(data[ctr+i+2]) << endl
				   << "\tQ_CA : " << static_cast<signed int>(data[ctr+i+3]) << endl;
				i = i + 4;
			}

			os << endl;
			ctr = ctr + (sub_packet_size/2);
		}

		else if (packet_type == 11)
		{
			os	<< "Occultation & Zenith Carrier Phase with Dual Carrier Frequency Tracking : \n"
				<< "\tLEN		: " << sub_packet_size << endl
				<< "\tTYP		: " << packet_type << endl
				<< "\tSAT		: " << static_cast<uint>((data[ctr+1]>>4)&63) << endl
				<< "\tCHN		: " << static_cast<uint>(data[ctr+1]&15) << endl
				<< "\tIMT		: " << ( (static_cast<long long>(data[ctr+2])<<48)
										 | (static_cast<long long>(data[ctr+3])<<32)
										 | (static_cast<long long>(data[ctr+4])<<16)
										 | static_cast<long long>(data[ctr+5]) ) << endl
				<< "\tP_1		: " << ( (static_cast<long long>(data[ctr+6])<<48)
										 | (static_cast<long long>(data[ctr+7])<<32)
										 | (static_cast<long long>(data[ctr+8])<<16)
										 | static_cast<long long>(data[ctr+9]) ) << endl
				<< "\tF1		: " << static_cast<signed int>((data[ctr+10]<<16) | data[ctr+11]) << endl
				<< "\tP_2		: " << ( (static_cast<long long>(data[ctr+12])<<48)
										 | (static_cast<long long>(data[ctr+13])<<32)
										 | (static_cast<long long>(data[ctr+14])<<16)
										 | static_cast<long long>(data[ctr+15]) ) << endl
				<< "\tF2		: " << static_cast<signed int>((data[ctr+16]<<16) | data[ctr+17]) << endl
				<< "\tTINT		: " << static_cast<unsigned int>((data[ctr+18]<<16) | data[ctr+19]) << endl
				<< "\tIQ_CA_EXP : " << static_cast<uint>(data[ctr+20]&255) << endl
				<< "\tIQ_P1_EXP : " << static_cast<uint>((data[ctr+21]>>8)&255) << endl
				<< "\tIQ_P2_EXP : " << static_cast<uint>(data[ctr+21]&255) << endl;

			i = 22;
			while (i < (((sub_packet_size-44)/2)+22))
			{
				os << "(M =	 " << (i/10)-1 << ")\n"
				   << "\tDP_1 : " << static_cast<signed int>((data[ctr+i]<<16) | data[ctr+i+1]) << endl
				   << "\tDP_2 : " << static_cast<signed int>((data[ctr+i+2]<<16) | data[ctr+i+3]) << endl
				   << "\tI_CA : " << static_cast<signed int>(data[ctr+i+4]) << endl
				   << "\tQ_CA : " << static_cast<signed int>(data[ctr+i+5]) << endl
				   << "\tI_P1 : " << static_cast<signed int>(data[ctr+i+6]) << endl
				   << "\tQ_P1 : " << static_cast<signed int>(data[ctr+i+7]) << endl
				   << "\tI_P2 : " << static_cast<signed int>(data[ctr+i+8]) << endl
				   << "\tQ_P2 : " << static_cast<signed int>(data[ctr+i+9]) << endl;
				i = i + 10;
			}

			os << endl;
			ctr = ctr + (sub_packet_size/2);
		}

		else if (packet_type == 12)
		{
			os	<< "Occultation & Zenith C/A-code Phase : \n"
				<< "\tLEN : " << sub_packet_size << endl
				<< "\tTYP : " << packet_type << endl
				<< "\tSAT : " << static_cast<uint>((data[ctr+1]>>4)&63) << endl
				<< "\tCHN : " << static_cast<uint>(data[ctr+1]&15) << endl
				<< "\tIMT : " << ( (static_cast<long long>(data[ctr+2])<<48)
								   | (static_cast<long long>(data[ctr+3])<<32)
								   | (static_cast<long long>(data[ctr+4])<<16)
								   | static_cast<long long>(data[ctr+5]) ) << endl
				<< "\tCEC : " << static_cast<uint>((data[ctr+6]>>10)&31) << endl
				<< "\tCC  : " << static_cast<uint>(data[ctr+6]&1023) << endl
				<< "\tCP  : " << data[ctr+7] << endl
				<< endl;
			ctr = ctr + (sub_packet_size/2);
		}

		else if (packet_type == 13)
		{
			os	<< "Occultation & Zenith C/A-code Phase and L1 & L2 P-code Phase : \n"
				<< "\tLEN	: " << sub_packet_size << endl
				<< "\tTYP	: " << packet_type << endl
				<< "\tSAT	: " << static_cast<uint>((data[ctr+1]>>4)&63) << endl
				<< "\tCHN	: " << static_cast<uint>(data[ctr+1]&15) << endl
				<< "\tIMT	: " << ( (static_cast<long long>(data[ctr+2])<<48)
									 | (static_cast<long long>(data[ctr+3])<<32)
									 | (static_cast<long long>(data[ctr+4])<<16)
									 | static_cast<long long>(data[ctr+5]) ) << endl
				<< "\tCEC	: " << static_cast<uint>((data[ctr+6]>>10)&31) << endl
				<< "\tCC	: " << static_cast<uint>(data[ctr+6]&1023) << endl
				<< "\tCP	: " << data[ctr+7] << endl
				<< "\tL12DC : " << static_cast<signed int>((data[ctr+8]>>8)&31) << endl
				<< "\tPCC	: " << ( (static_cast<uint>(data[ctr+8]&255)<<16)
									 | static_cast<uint>(data[ctr+9]) ) << endl
				<< "\tP1PF	: " << data[ctr+10] << endl
				<< "\tP2PF	: " << data[ctr+11] << endl
				<< endl;
			ctr = ctr + (sub_packet_size/2);
		}

		else if (packet_type == 15)
		{
			os	<< "Navigation Data Frame Dump : \n"
				<< "\tLEN	  : " << sub_packet_size << endl
				<< "\tTYP	  : " << packet_type << endl
				<< "\tNO_BITS : " << static_cast<uint>((data[ctr+1]>>10)&63) << endl
				<< "\tSAT	  : " << static_cast<uint>((data[ctr+1]>>4)&63) << endl
				<< "\tCHN	  : " << static_cast<uint>(data[ctr+1]&15) << endl
				<< "\tIMT	  : " << ( (static_cast<long long>(data[ctr+2])<<48)
									   | (static_cast<long long>(data[ctr+3])<<32)
									   | (static_cast<long long>(data[ctr+4])<<16)
									   | static_cast<long long>(data[ctr+5]) ) << endl
				<< "\tNDF_BITS: " << ntobs(data[ctr+6], 15) << endl
				<< "\t			" << ntobs(data[ctr+7], 15) << endl
				<< "\t			" << ntobs(data[ctr+8], 15) << endl
				<< "\t			" << ntobs(data[ctr+9], 15) << endl
				<< endl;
			ctr = ctr + (sub_packet_size/2);
		}

		else
		{
			os	<< "*** WARNING Data Packet Not Recognised *** : "
				<< "LEN : " << sub_packet_size
				<< "TYP : " << packet_type << endl
				<< endl;
			ctr = ctr + (sub_packet_size/2);
		}

	} // End of while loop to output payload data
	os << endl;

	if (ctr != (ancil_size/2+payload_size/2))
	{
		os << "WARNING unused data in packet ctr " << ctr
		   << " ancil_size/2+payload_size/2 " << (ancil_size/2+payload_size/2)
		   << endl;
	}
}


//
// obct_to_temp
//

double CCSDSPacketDisplayer::obct_to_temp(int input)
{
	return ((-21.84 + (input/88.86)) / 0.391);
}


//
// obct_to_res
//

double CCSDSPacketDisplayer::obct_to_res(int input)
{
	return ( 78.16 + (input/88.86) );
}
